/****************************************************************************
 ** @license
 ** This file is part of yFiles for HTML 2.3.0.3.
 **
 ** yWorks proprietary/confidential. Use is subject to license terms.
 **
 ** Copyright (c) 2020 by yWorks GmbH, Vor dem Kreuzberg 28,
 ** 72070 Tuebingen, Germany. All rights reserved.
 **
 ***************************************************************************/
export interface HashMap<TKey, TValue> extends YObject, IMap<TKey, TValue>, ICollection<MapEntry<TKey, TValue>>, IEnumerable<MapEntry<TKey, TValue>> {
/**
@inheritdoc*/
add(item: MapEntry<TKey, TValue>): void;
/**
@inheritdoc*/
clear(): void;
/**
@inheritdoc*/
copyTo(array: MapEntry<TKey, TValue>[], arrayIndex: number): void;
/**
@inheritdoc*/
delete(key: TKey | null): boolean;
/**
@inheritdoc*/
getEnumerator(): IEnumerator<MapEntry<TKey, TValue>>;
/**
@inheritdoc*/
has(key: TKey | null): boolean;
/**
@inheritdoc*/
remove(item: MapEntry<TKey, TValue>): boolean;
/**
@inheritdoc*/
readonly isReadOnly: boolean;
/**
@inheritdoc*/
readonly keys: ICollection<TKey>;
/**
@inheritdoc*/
readonly values: ICollection<TValue>;
} /**
The default implementation for {@link IMap}.
Type parameter <code>TKey</code>: The type of the key elements of this dictionary.
Type parameter <code>TValue</code>: The type of the value elements of this dictionary.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HashMap Online Documentation}*/
export class HashMap<TKey, TValue> {
/**
Creates a new instance.
@param [options] The parameters to pass.
@param options.entries An array of items which are added to this map. Each item will be added via the {@link HashMap#add} method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HashMap%23HashMap() Online Documentation}*/
constructor(options?: {
entries?: Array<any>;
});
/**
Whether this dictionary contains the given <code>value</code>.
@param value The value to search for.
@returns <code>true</code> if this dictionary contains the given value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HashMap%23containsValue(TValue) Online Documentation}*/
containsValue(value: TValue | null): boolean;
/**
Gets the value of a given key.
Setting a value for an already existing key overwrites the previous value. Otherwise, a new key with the given value is
added.
Getting the value of a non-existing key returns <code>null</code>.
@param key The key.
@returns The value of the given key.
@see HashMap#set
@see {@link https://docs.yworks.com/yfileshtml/#/api/HashMap%23get(TKey) Online Documentation}*/
get(key: TKey | null): TValue | null;
/**
Sets the value of a given key.
Setting a value for an already existing key overwrites the previous value. Otherwise, a new key with the given value is
added.
Getting the value of a non-existing key returns <code>null</code>.
@param key The key.
@param value The value of the given key.
@see HashMap#get
@see {@link https://docs.yworks.com/yfileshtml/#/api/HashMap%23set(TKey%2CTValue) Online Documentation}*/
set(key: TKey | null, value: TValue | null): void;
static $class: Class;
private $_Map: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TKey, TValue>(o: any): o is HashMap<TKey, TValue>;
}
export interface EventArgs extends YObject {
}
/**
Subclasses of {@link EventArgs} are passed to event handlers to provide the information they need to handle an
event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EventArgs Online Documentation}*/
export class EventArgs {
/**
An empty {@link EventArgs} object that can be used when no information needs to be passed to the event handler.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EventArgs%23EMPTY Online Documentation}*/
static EMPTY: EventArgs;
static $class: Class;
private $_EventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EventArgs;
}
export interface ICollection<T> extends IEnumerable<T> {
}
/**
* Defines methods to manipulate a collection of objects with the same type.
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters when annotated as such:
*
* It is possible to specify an Array or to use iterables or generators to define the {@link ICollection}:
*
* <pre>
* // Array
* object.property = [item1, item2]
*
* // Iterables
* object.property = new Set([item1, item2])
* object.property = new Map([[key1, item1], [key2, item2]]).entries()
*
* // Generators
* function* generator(i) {
*   yield i;
*   yield i + 10;
* }
* object.property = generator(10)
* </pre>
*
* If the element adheres to the <a href="https://developer.mozilla.org/de/docs/Web/JavaScript/Reference/Iteration_protocols">'iterable' protocol</a> it will also be automatically converted.
*
* Type parameter <code>T</code>: The type of the elements in the collection.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ICollection Online Documentation}
*/
export abstract class ICollection<T> {
/**
Adds the given <code>item</code> to the collection.
@param item
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICollection%23add(T) Online Documentation}*/
abstract add(item: T): void;
/**
Removes all items from this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICollection%23clear() Online Documentation}*/
abstract clear(): void;
/**
Copies all elements of this collection into the given array.
@param array The array to copy the elements to.
@param arrayIndex The index in the given array where the first element should be copied to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICollection%23copyTo(Array%3CT%3E%2Cnumber) Online Documentation}*/
abstract copyTo(array: T[], arrayIndex: number): void;
/**
Creates a {@link ICollection} instance from the given object by performing automatic type conversion.
This will perform the automatic type conversion on the given argument and return a proper {@link ICollection} instance.
Type parameter <code>T</code>.
@param arrayLike The object to convert to a {@link ICollection}
@returns A proper instance, initialized to the values found in <code>arrayLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICollection%23from(ICollection%3CT%3E) Online Documentation}*/
static from<T>(arrayLike: ICollection<T> | ICollectionConvertible<T>): ICollection<T>;
/**
Whether the given <code>item</code> is contained in this collection.
@param item The item to search for.
@returns <code>true</code> if the given <code>item</code> is contained in this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICollection%23includes(T) Online Documentation}*/
abstract includes(item: T): boolean;
/**
Removes the given <code>item</code> from this collection.
@param item The item to remove.
@returns Whether the item was removed from the collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICollection%23remove(T) Online Documentation}*/
abstract remove(item: T): boolean;
/**
The number of elements in this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICollection%23size Online Documentation}*/
abstract readonly size: number;
/**
Whether this collection is read-only.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICollection%23isReadOnly Online Documentation}*/
abstract readonly isReadOnly: boolean;
static $class: Class;
private $_ICollection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ICollection<T>;
}
/** Automatic conversion to {@link ICollection}. */
type ICollectionConvertible<T> = T[] | ArrayLike<T> | Iterable<T>;
export interface IObservableCollection<T> extends ICollection<T> {
}
/**
The interface for a collection that will notify registered event handlers of changes to its contents.
Type parameter <code>T</code>: The type of the items in the collection.
@see ObservableCollection
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObservableCollection Online Documentation}*/
export abstract class IObservableCollection<T> {
/**
Adds the given listener for the <code>ItemAdded</code> event that occurs when an item has been added to this collection.
@param listener The listener to add.
@see IObservableCollection#removeItemAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObservableCollection%23ItemAdded Online Documentation}*/
abstract addItemAddedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Removes the given listener for the <code>ItemAdded</code> event that occurs when an item has been added to this collection.
@param listener The listener to remove.
@see IObservableCollection#addItemAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObservableCollection%23ItemAdded Online Documentation}*/
abstract removeItemAddedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Adds the given listener for the <code>ItemRemoved</code> event that occurs when an item has been removed from this collection.
@param listener The listener to add.
@see IObservableCollection#removeItemRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObservableCollection%23ItemRemoved Online Documentation}*/
abstract addItemRemovedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Removes the given listener for the <code>ItemRemoved</code> event that occurs when an item has been removed from this collection.
@param listener The listener to remove.
@see IObservableCollection#addItemRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObservableCollection%23ItemRemoved Online Documentation}*/
abstract removeItemRemovedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Adds the given listener for the <code>ItemChanged</code> event that occurs when an item in this collection has changed
significantly.
It is up to the implementation whether and when to trigger this event.
@param listener The listener to add.
@see IObservableCollection#removeItemChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObservableCollection%23ItemChanged Online Documentation}*/
abstract addItemChangedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Removes the given listener for the <code>ItemChanged</code> event that occurs when an item in this collection has changed
significantly.
It is up to the implementation whether and when to trigger this event.
@param listener The listener to remove.
@see IObservableCollection#addItemChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObservableCollection%23ItemChanged Online Documentation}*/
abstract removeItemChangedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
static $class: Class;
private $_IObservableCollection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is IObservableCollection<T>;
}
export interface IListEnumerable<T> extends IEnumerable<T> {
}
/**
* An extension to the {@link IEnumerable} interface that allows for indexed access and retrieval of the element count.
* This interface is like a rudimentary read-only {@link IList}
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters when annotated as such:
*
* It is possible to specify an Array or to use iterables or generators to define the {@link IListEnumerable}:
*
* <pre>
* // Array
* [item1, item2]
*
* // Iterables
* new Set([item1, item2])
* new Map([[key1, item1], [key2, item2]]).entries()
*
* // Generators
* function* generator(i) {
*   yield i;
*   yield i + 10;
* }
* generator(10)
* </pre>
*
* If the element adheres to the <a href="https://developer.mozilla.org/de/docs/Web/JavaScript/Reference/Iteration_protocols">'iterable' protocol</a> it will also be automatically converted.
*
* Type parameter <code>T</code>: The type of the contained elements.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IListEnumerable Online Documentation}
*/
export abstract class IListEnumerable<T> {
/**
Creates a IListEnumerable instance from the given object by performing automatic type conversion.
Type parameter <code>T</code>: The type of the contained elements.
@param listLike The object to convert to a IListEnumerable.
@returns Either <code>listLike</code> if it is already a IListEnumerable or a new instance, initialized to the values found in <code>listLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IListEnumerable%23from(IListEnumerable%3CT%3E) Online Documentation}*/
static from<T>(listLike: IListEnumerable<T> | IListEnumerableConvertible<T>): IListEnumerable<T>;
/**
Gets the <code>i</code>-th element in the collection.
@param i the zero-based index of the item in this collection
@returns the item for the given index
@throws {Exception} <b>(ArgumentError)</b> <code>i</code> is not a valid index in the {@link IListEnumerable}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IListEnumerable%23get(number) Online Documentation}*/
abstract get(i: number): T;
/**
Gets the number of elements in this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IListEnumerable%23size Online Documentation}*/
abstract readonly size: number;
/**
An empty, immutable instance of {@link IListEnumerable}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IListEnumerable%23EMPTY Online Documentation}*/
static EMPTY: IListEnumerable<any>;
static $class: Class;
private $_IListEnumerable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is IListEnumerable<T>;
}
/** Automatic conversion to {@link IListEnumerable}. */
type IListEnumerableConvertible<T> = T[] | ArrayLike<T> | Iterable<T>;
export interface ListEnumerable<T> extends YObject, IListEnumerable<T> {
/**
@inheritdoc*/
get(i: number): T;
/**
@inheritdoc*/
getEnumerator(): IEnumerator<T>;
} /**
* Wraps a simple {@link IEnumerable} into a {@link IListEnumerable}.
* This implementation is optimized for {@link IList} and {@link ICollection}, as well as {@link Array<any>} {@link IEnumerable}s.
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* It is possible to specify an {@link Array<any>} to define the {@link ListEnumerable}:
*
* <pre>
* // Array
* object.property = [item1, item2]
*
* // Iterables
* object.property = new Set([item1, item2])
* object.property = new Map([[key1, item1], [key2, item2]]).entries()
*
* // Generators
* function* generator(i) {
*   yield i;
*   yield i + 10;
* }
* object.property = generator(10)
* </pre>
*
* Type parameter <code>T</code>: The type of the enumerable.
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ListEnumerable Online Documentation}
*/
export class ListEnumerable<T> {
/**
Creates a new instance using the given backing enumerable.
@param backingEnumerable The enumerable to back this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ListEnumerable%23ListEnumerable(IEnumerable%3CT%3E) Online Documentation}*/
constructor(backingEnumerable: IEnumerable<T> | IEnumerableConvertible<T>);
/**
Creates a new instance with a backing {@link List} as the backing store.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ListEnumerable%23ListEnumerable() Online Documentation}*/
constructor();
/**
Gets the backing enumerable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ListEnumerable%23backingEnumerable Online Documentation}*/
readonly backingEnumerable: IEnumerable<T>;
static $class: Class;
private $_ListEnumerable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ListEnumerable<T>;
}
export interface ItemEventArgs<T> extends EventArgs {
}
/**
Event argument class used by {@link IObservableCollection} and similar that holds a specific item that is related to the event.
Type parameter <code>T</code>: The type of the item contained in the event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemEventArgs Online Documentation}*/
export class ItemEventArgs<T> {
/**
Creates a new event argument for the given item.
@param item The item which is the subject of the event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemEventArgs Online Documentation}*/
constructor(item: T);
/**
Gets the item that is the subject of the event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemEventArgs%23item Online Documentation}*/
readonly item: T;
static $class: Class;
private $_ItemEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ItemEventArgs<T>;
}
export interface IMap<TKey, TValue> extends IMapper<TKey, TValue>, ICollection<MapEntry<TKey, TValue>> {
}
/**
A collection of value objects which are mapped to key objects and can be queried using their keys.
The keys must have a well defined <code>hashCode</code> method that must not change between registering a value for the key and
retrieving a value using the key. Thus it is advisable to use stable, i.e. immutable, objects as the key.
Type parameter <code>TKey</code>: The type of the keys.
Type parameter <code>TValue</code>: The type of the values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMap Online Documentation}*/
export abstract class IMap<TKey, TValue> {
/**
Removes the key/value pair which is represented by the given <code>key</code>.
@param key The key of the key/value pair to remove.
@returns <code>true</code> if a key/value pair with the given key was removed from this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMap%23delete(TKey) Online Documentation}*/
abstract delete(key: TKey): boolean;
/**
Whether this dictionary's key collection contains the given key.
@param key The key to search for.
@returns <code>true</code> if this dictionary contains the given <code>key</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMap%23has(TKey) Online Documentation}*/
abstract has(key: TKey): boolean;
/**
A {@link ICollection} of the keys of this dictionary.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMap%23keys Online Documentation}*/
abstract readonly keys: ICollection<TKey>;
/**
A {@link ICollection} of the values of this dictionary.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMap%23values Online Documentation}*/
abstract readonly values: ICollection<TValue>;
static $class: Class;
private $_IMap: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TKey, TValue>(o: any): o is IMap<TKey, TValue>;
}
export interface IEnumerable<T> extends Iterable<T> {
}
/**
* Defines ways to iterate over the items contained in this type by providing a method to get an {@link IEnumerator}.
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects, and
* setters.
*
* It is possible to specify an Array or to use iterables or generators to define the {@link IEnumerable}:
*
* <pre>
* // Array
* object.property = [item1, item2]
*
* // Iterables
* object.property = new Set([item1, item2])
* object.property = new Map([[key1, item1], [key2, item2]]).entries()
*
* // Generators
* function* generator(i) {
*   yield i;
*   yield i + 10;
* }
* object.property = generator(10)
* </pre>
*
* All enumerables automatically implement the EcmaScript "iterable protocol" if the platform supports the
* <code>Symbol.iterator</code>.
*
* <br><b>Basic for-of usage</b>
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4])
*
* for (const item of enumerable) {
*   console.log(item)
* }
* // 1 2 3 4
* </pre>
*
* They also mimic many of the functions of the <code>Array.prototype</code>, but typically evaluate lazily.
*
* <br><b>Basic for-of usage</b>
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4])
*
* enumerable.forEach(item => {
*   console.log(item)
* }) // 1 2 3 4
*
* // accessing the index
* enumerable.forEach((item, index) => console.log(`Item #${index} is ${item}`))
*
* // passing "this" into the function and the original enumerable
* enumerable.forEach(function(item, index, owner) {
*   this.log(`Item #${index} of ${owner.size} is ${item}`)
* }, console)
* </pre>
*
* Enumerables are very flexible with their various helper methods:
*
* <br><b>Advanced enumerable usage</b>
* <pre>
* const widestNodes = graphComponent.graph.nodes
*   // group nodes by their width
*   .groupBy(
*     n => n.layout.width,
*     (width, items) => ({ width, items })
*   )
*   // and order the groups by their size
*   .orderByDescending(entry => entry.items.size)
*   // consider the largest two groups only
*   .take(2)
*   // and take all their items
*   .flatMap(entry => entry.items)
*
* // check which nodes can be reached from the two largest groups
* new Reachability({ startNodes: widestNodes })
*   .run(graphComponent.graph)
*   // and select all those nodes
*   .reachableNodes.forEach(n => graphComponent.selection.setSelected(n, true))
* </pre>
*
* Type parameter <code>T</code>: The type of the enumerable objects.
* @see IEnumerable#toArray
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable Online Documentation}
*/
export abstract class IEnumerable<T> {
/**
* Creates a wrapped enumerable that has one or several elements appended to it.
* In contrast to {@link IEnumerable#concat}, this method will not treat the <code>items</code> any special if they are array-like or enumerable-like.
*
* <pre>
* const list = List.fromArray([1, 2, 3, 4])
*
* const appendedEnumerable = list.append(5, 6)
* console.log(appendedEnumerable.toArray()) // yields [1,2,3,4,5,6]
*
* // show live updates
* list.add(4.5)
*
* console.log(appendedEnumerable.toArray()) // yields [1,2,3,4,4.5,5,6]
* </pre>
*
* @param items The item(s) to prepend to the enumeration
* @returns A new live view over the original enumerable that has each of the <code>items</code> appended to it.
* @see IEnumerable#prepend
* @see IEnumerable#concat
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23append(T) Online Documentation}
*/
append(...items: T[]): IEnumerable<T>;
/**
* Concatenates this enumerable with the given other <code>elements</code> in a new dynamic {@link IEnumerable}.
* The returned enumerable is dynamic, in other words, if elements are added or removed from one of the original
* enumerables, the returned enumerable changes, too. If you want to store the current state of the concatenation, create a
* {@link IEnumerable#toList list} or {@link IEnumerable#toArray array} of it.
*
* <pre>
* const enumerable1 = IEnumerable.from([1, 2])
* const enumerable2 = IEnumerable.from([3, 4])
*
* console.log(enumerable1.concat(enumerable2).toArray()) // yields [1,2,3,4]
*
* // single values can be used, too
* console.log(enumerable1.concat(3, 4).toArray()) // yields [1,2,3,4]
*
* // arrays are flattened
* console.log(enumerable1.concat([5, 6], [7, 8]).toArray()) // now contains [1,2,5,6,7,8]
*
* // works with arrays and array-like collections, too
* console.log(enumerable1.concat([3, 4]).toArray()) // yields [1,2,3,4]
* </pre>
*
* <br><b>The returned enumerable is dynamic</b>
* <pre>
* const list1 = List.fromArray([1, 2])
* const list2 = List.fromArray([3, 4])
*
* const result = list1.concat(list2)
*
* console.log(result.toArray()) // yields [1,2,3,4]
*
* list1.add(2.5)
*
* // result changed also
* console.log(result.toArray()) // yields [1,2,2.5,3,4]
*
* list2.add(5)
*
* // result changed also
* console.log(result.toArray()) // yields [1,2,2.5,3,4,5]
* </pre>
*
* @param elements Rest arguments to be appended. In case the elements are enumerable-like, too, they will be will be enumerated after the
* others have been enumerated. But this can also be simple objects of type <code>T</code> in which case they will be appended to the
* enumerable just like with {@link IEnumerable#append}
* @returns A dynamic composite enumerable.
* @see IEnumerable#append
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23concat(Object) Online Documentation}
*/
concat(...elements: any[]): IEnumerable<T>;
/**
* Creates a wrapped enumerable that yields each element only once.
* To determine equality between elements, the logic of a {@link HashMap} is used, i.e. the {@link YObject#equals} and {@link YObject#hashCode}
* methods are used for those elements that derive from {@link Object}.
*
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 3, 4])
*
* const result = enumerable.distinct()
*
* console.log(result.toArray()) // yields [1,2,3,4]
* </pre>
*
* @param [keySelector=null] The optional function that determines the key to determine distinctness of the elements. If omitted, the element itself
* is used.
* @param [thisArg=null] The optional object to use for <code>this</code>in the <code>keySelector</code> <code>function</code>.
* @returns A new live view over the original enumerable that yields each element with a given key at most once.
* @see HashMap
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23distinct(function(T)%3AObject%2CObject) Online Documentation}
*/
distinct(keySelector?: ((arg: T) => any | null) | null, thisArg?: any | null): IEnumerable<T>;
/**
* Creates a wrapped enumerable that yields each element only once.
* To determine equality between elements, the logic of a {@link HashMap} is used, i.e. the {@link YObject#equals} and {@link YObject#hashCode}
* methods are used for those elements that derive from {@link Object}.
*
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 3, 4])
*
* const result = enumerable.distinct()
*
* console.log(result.toArray()) // yields [1,2,3,4]
* </pre>
*
* @param options The parameters to pass.
* @param [options.keySelector=null] The optional function that determines the key to determine distinctness of the elements. If omitted, the element itself
* is used.
* @param [options.thisArg=null] The optional object to use for <code>this</code>in the <code>keySelector</code> <code>function</code>.
* @returns A new live view over the original enumerable that yields each element with a given key at most once.
* @see HashMap
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23distinct(function(T)%3AObject%2CObject) Online Documentation}
*/
distinct(options: {
keySelector?: ((arg: T) => any | null) | null;
thisArg?: any | null;
}): IEnumerable<T>;
/**
* Returns the element at the given <code>index</code> in this enumerable.
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4])
*
* const element = enumerable.elementAt(0)
*
* console.log(element) // yields 1
* </pre>
*
* @param index The index of the element to return.
* @returns The element at the given <code>index</code>.
* @throws {Exception} <b>(ArgumentError)</b> If the <code>index</code> is not within the enumerable's range.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23elementAt(number) Online Documentation}
*/
elementAt(index: number): T;
/**
* Determines whether every element of the enumerable matches the given <code>predicate</code>.
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4])
*
* console.log(enumerable.every(item => item < 5)) // true
*
* console.log(enumerable.every(item => item < 4)) // false
* </pre>
*
* @param predicate A predicate function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the element matches a
* condition.
* @param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns <code>true</code> if all elements of the enumerable match the given <code>predicate</code>, <code>false</code> otherwise.
* @throws {Exception} <b>(ArgumentError)</b> <code>predicate</code> is <code>null</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23every(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
every(predicate: (arg1: T, arg2: number, arg3: IEnumerable<T> | null) => boolean, thisArg?: any | null): boolean;
/**
* Returns a dynamic {@link IEnumerable} of the elements of this enumerable which match the given <code>predicate</code>.
* The returned enumerable is dynamic, in other words, if elements are added or removed from one of the original
* enumerables, the returned enumerable changes, too. If you want to store the current state of the filtered enumerable,
* create a {@link IEnumerable#toList list} or {@link IEnumerable#toArray array} of it.
*
* <pre>
* const enumerable = IEnumerable.from([10, 2, 30, 4, 5, 6])
*
* const result = enumerable.filter(item => item < 5)
*
* console.log(result.toArray()) // yields [2,4]
* </pre>
*
* <br><b>The returned enumerable is dynamic</b>
* <pre>
* const list = List.fromArray([10, 2, 30, 4, 5, 6])
*
* const result = list.filter(item => item < 5)
*
* list.add(1)
*
* // result changed also
* console.log(result.toArray()) // yields [2,4,1]
* </pre>
*
* @param predicate A predicate function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the given element should be
* included in the resulting enumerable.
* @param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns A dynamic enumerable of the elements which match the given <code>predicate</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23filter(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
filter(predicate: (arg1: T, arg2: number, arg3: IEnumerable<T> | null) => boolean, thisArg?: any | null): IEnumerable<T>;
/**
* Returns the first element of this enumerable, the first element of the enumerable that matches a given <code>predicate</code> if
* specified, or <code>null</code> if there is no such element.
* <pre>
* const enumerable = IEnumerable.from([10, 2, 3, 9, 5, 6])
*
* console.log(enumerable.find(item => item < 5)) // yields 2
*
* // no matches with the given predicate
* console.log(enumerable.find(item => item < 2)) // yields null
* </pre>
*
* @param [predicate=null] A predicate function which returns <code>true</code> if the given element should be returned.
* @param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns The first element of this enumerable, its first element that matches a given predicate, or <code>null</code> if there is no such
* element.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23find(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
find(predicate?: ((arg1: T, arg2: number, arg3: IEnumerable<T> | null) => boolean) | null, thisArg?: any | null): T;
/**
* Returns the first element of this enumerable, the first element of the enumerable that matches a given <code>predicate</code> if
* specified, or <code>null</code> if there is no such element.
* <pre>
* const enumerable = IEnumerable.from([10, 2, 3, 9, 5, 6])
*
* console.log(enumerable.find(item => item < 5)) // yields 2
*
* // no matches with the given predicate
* console.log(enumerable.find(item => item < 2)) // yields null
* </pre>
*
* @param options The parameters to pass.
* @param [options.predicate=null] A predicate function which returns <code>true</code> if the given element should be returned.
* @param [options.thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns The first element of this enumerable, its first element that matches a given predicate, or <code>null</code> if there is no such
* element.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23find(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
find(options: {
predicate?: ((arg1: T, arg2: number, arg3: IEnumerable<T>) => boolean) | null;
thisArg?: any | null;
}): T;
/**
Searches the enumerable for the first item for which <code>predicate</code> returns <code>true</code> and returns its index.
@param predicate A predicate function which returns <code>true</code> if an item matches the search conditions.
@param [thisArg=null]
@returns The index of the first item for which <code>predicate</code> returns <code>true</code> or -1 if there is no such item in the enumerable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23findIndex(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}*/
findIndex(predicate: (arg1: T, arg2: number, arg3: IEnumerable<T> | null) => boolean, thisArg?: any | null): number;
/**
* Returns the first element of this enumerable which matches a given <code>predicate</code>.
* <pre>
* const enumerable = IEnumerable.from([10, 2, 3, 9, 5, 6])
*
* console.log(enumerable.first(item => item < 5)) // yields 2
*
* // no matches with the given predicate
* console.log(enumerable.first(item => item < 2)) // throws exception
* </pre>
*
* @param [predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the given element should be returned.
* @param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns The first matching element of this enumerable.
* @throws {Exception} <b>(InvalidOperationError)</b> If this enumerable does not contain a matching element.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23first(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
first(predicate?: ((arg1: T, arg2: number, arg3: IEnumerable<T> | null) => boolean) | null, thisArg?: any | null): T;
/**
* Returns the first element of this enumerable which matches a given <code>predicate</code>.
* <pre>
* const enumerable = IEnumerable.from([10, 2, 3, 9, 5, 6])
*
* console.log(enumerable.first(item => item < 5)) // yields 2
*
* // no matches with the given predicate
* console.log(enumerable.first(item => item < 2)) // throws exception
* </pre>
*
* @param options The parameters to pass.
* @param [options.predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the given element should be returned.
* @param [options.thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns The first matching element of this enumerable.
* @throws {Exception} <b>(InvalidOperationError)</b> If this enumerable does not contain a matching element.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23first(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
first(options: {
predicate?: ((arg1: T, arg2: number, arg3: IEnumerable<T>) => boolean) | null;
thisArg?: any | null;
}): T;
/**
* Returns the first element of this enumerable, the first element of this enumerable that matches a given <code>predicate</code> if
* specified, or <code>null</code> if there is no such element.
* <pre>
* const enumerable = IEnumerable.from([10, 2, 3, 9, 5, 6])
*
* console.log(enumerable.firstOrDefault(item => item < 5)) // yields 2
*
* // no matches with the given predicate
* console.log(enumerable.firstOrDefault(item => item < 2)) // yields null
* </pre>
*
* @param [predicate=null] A predicate function which returns <code>true</code> if the given element should be returned.
* @param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns The first element of this enumerable, its first element that matches a given predicate, or <code>null</code> if there is no such
* element.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23firstOrDefault(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
firstOrDefault(predicate?: ((arg1: T, arg2: number, arg3: IEnumerable<T> | null) => boolean) | null, thisArg?: any | null): T;
/**
* Returns the first element of this enumerable, the first element of this enumerable that matches a given <code>predicate</code> if
* specified, or <code>null</code> if there is no such element.
* <pre>
* const enumerable = IEnumerable.from([10, 2, 3, 9, 5, 6])
*
* console.log(enumerable.firstOrDefault(item => item < 5)) // yields 2
*
* // no matches with the given predicate
* console.log(enumerable.firstOrDefault(item => item < 2)) // yields null
* </pre>
*
* @param options The parameters to pass.
* @param [options.predicate=null] A predicate function which returns <code>true</code> if the given element should be returned.
* @param [options.thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns The first element of this enumerable, its first element that matches a given predicate, or <code>null</code> if there is no such
* element.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23firstOrDefault(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
firstOrDefault(options: {
predicate?: ((arg1: T, arg2: number, arg3: IEnumerable<T>) => boolean) | null;
thisArg?: any | null;
}): T;
/**
* Returns a flattened dynamic {@link IEnumerable} of this enumerable using the given <code>selector</code> function which returns an enumerable for each
* element of this enumerable.
* The returned enumerable is dynamic, in other words, if elements are added or removed from one of the original
* enumerables, the returned enumerable changes, too. If you want to store its current state, create a {@link IEnumerable#toList list} or {@link IEnumerable#toArray array} of
* it.
*
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3])
*
* const result = enumerable.flatMap(item => [item, item * 2])
*
* console.log(result.toArray()) // yields [1,2,2,4,3,6]
* </pre>
*
* <br><b>The returned enumerable is dynamic</b>
* <pre>
* const list = List.fromArray([[1], [2], [3]])
*
* const result = list.flatMap(item => IEnumerable.from([item * 2]))
*
* list.add([4])
*
* // result changed also
* console.log(result.toArray()) // yields [2,4,6,8]
* </pre>
*
* Type parameter <code>TResult</code>.
* @param selector A function with the signature <code>function(element):IEnumerable</code> which returns an enumerable for each element of this
* enumerable.
* @param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns A flattened dynamic enumerable of the enumerables provided for each element of this enumerable.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23flatMap(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3AIEnumerable%3CTResult%3E%2CObject) Online Documentation}
*/
flatMap<TResult>(selector: (arg1: T, arg2: number, arg3: IEnumerable<T> | null) => IEnumerable<TResult> | null, thisArg?: any | null): IEnumerable<TResult>;
/**
* Iterates this enumerable and invokes the given function for each element with the element, its index, and this
* enumerable as arguments.
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4])
*
* enumerable.forEach(item => {
*   console.log(item)
* }) // 1 2 3 4
*
* // accessing the index
* enumerable.forEach((item, index) => console.log(`Item #${index} is ${item}`))
*
* // passing "this" into the function and the original enumerable
* enumerable.forEach(function(item, index, owner) {
*   this.log(`Item #${index} of ${owner.size} is ${item}`)
* }, console)
* </pre>
*
* @param action The function to call for each element.
* @param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23forEach(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Avoid%2CObject) Online Documentation}
*/
forEach(action: (arg1: T, arg2: number, arg3: IEnumerable<T> | null) => void, thisArg?: any | null): void;
/**
* Creates an enumerable from a given array or enumerable like object.
* <br><b>Creating enumerable views</b>
* <pre>
* const array = [1, 2, 3, 4, 5, 6, 7]
*
* const enumerable = IEnumerable.from(array)
*
* const result = enumerable.skip(3)
*
* console.log(result.size) // yields 4
*
* // arrays are converted to live enumerables
* array.push(8)
*
* console.log(result.size) // yields 5
*
* function* generator(i) {
*   yield i
*   yield i + 10
* }
*
* console.log(IEnumerable.from(generator(5)).last) // yields 15
* </pre>
*
* Type parameter <code>T</code>: The type of the contents of the enumerable.
* @param arrayLike The object that is either array-like, an enumerable, or a generator.
* @returns A new instance that may be a view of the given argument.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23from(IEnumerable%3CT%3E) Online Documentation}
*/
static from<T>(arrayLike: IEnumerable<T> | IEnumerableConvertible<T>): IEnumerable<T>;
/**
* Gets an {@link IEnumerator} which can be used to iterate over the items in this instance
* This is the low-level API that needs to be implemented by all enumerables. Clients will only have to use this API,
* rarely, as iteration is more conveniently handled by the {@link IEnumerable#forEach} method or using the iterable protocol.
*
* <br><b>Low-level enumerator usage</b>
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4])
*
* const enumerator = enumerable.getEnumerator()
* while (enumerator.moveNext()) {
*   console.log(enumerator.current)
* }
* // 1 2 3 4
*
* // enumerators can be reused
* enumerator.reset()
* while (enumerator.moveNext()) {
*   console.log(enumerator.current)
* }
* // 1 2 3 4
* </pre>
*
* @returns The {@link IEnumerator} which can be used to iterate over the items in this instance
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23getEnumerator() Online Documentation}
*/
abstract getEnumerator(): IEnumerator<T>;
/**
* Groups the elements in the enumerable according to the equality of a key.
* For each element in the original enumerable the <code>keySelector</code> will be used to retrieve a key. All elements with the same
* key will be provided to the optional <code>resultCreator</code> function that receives both the key, and an {@link IEnumerable} of the elements that
* match that key to create a resulting object that will be yielded by this enumerable. Note that the resulting enumerable
* does not necessarily preserve the order of the (first) appearance of their elements in the original sequence. The
* elements of each group passed to the <code>resultCreator</code> are in the order of their original appearance.
*
* <pre>
* const enumerable = IEnumerable.from([
*   { type: 'A', amount: 1 },
*   { type: 'A', amount: 2 },
*   { type: 'B', amount: 4 }
* ])
*
* // without result creator
* const groups1 = enumerable.groupBy(element => element.type)
*
* for (const g of groups1) {
*   console.log(`Type is ${g[0]} and has ${g[1].size} members.`)
* }
* // Type is A and has 2 members.
* // Type is B and has 1 members.
*
* // with result creator
* const groups2 = enumerable.groupBy(
*   element => element.type,
*   (type, elements) => ({ type, amount: elements.sum(item => item.amount) })
* )
*
* console.log(groups2.toArray()) // yields [{"type":"A","amount":4},{"type":"B","amount":4}]
* </pre>
*
* Type parameter <code>TResult</code>: The type of the created elements.
* Type parameter <code>TKey</code>: The type of the key elements.
* @param keySelector A function with the signature <code>function(element, index):TKey</code> which selects from each element the key.
* @param [resultCreator=null] An optional function with the signature <code>function(element, IEnumerable>TSource&lt;):TResult</code> which transforms each group of
* elements and their common key into a resulting object. If omitted, the default implementation will create arrays of
* length 2 where the first element is the key and the second element is an IEnumerable>TSource&lt; of the elements with the
* corresponding key.
* @param [thisArg=null] The optional value to use as <code>this</code> when executing the <code>keySelector</code> and <code>resultCreator</code> .
* @returns An enumeration of groups of the original enumerable.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23groupBy(function(T)%3ATKey%2Cfunction(TKey%2CIEnumerable%3CT%3E)%3ATResult%2CObject) Online Documentation}
*/
groupBy<TKey, TResult>(keySelector: (arg: T) => TKey, resultCreator?: ((arg1: TKey, arg2: IEnumerable<T> | null) => TResult) | null, thisArg?: any | null): IEnumerable<TResult>;
/**
* Groups the elements in the enumerable according to the equality of a key.
* For each element in the original enumerable the <code>keySelector</code> will be used to retrieve a key. All elements with the same
* key will be provided to the optional <code>resultCreator</code> function that receives both the key, and an {@link IEnumerable} of the elements that
* match that key to create a resulting object that will be yielded by this enumerable. Note that the resulting enumerable
* does not necessarily preserve the order of the (first) appearance of their elements in the original sequence. The
* elements of each group passed to the <code>resultCreator</code> are in the order of their original appearance.
*
* <pre>
* const enumerable = IEnumerable.from([
*   { type: 'A', amount: 1 },
*   { type: 'A', amount: 2 },
*   { type: 'B', amount: 4 }
* ])
*
* // without result creator
* const groups1 = enumerable.groupBy(element => element.type)
*
* for (const g of groups1) {
*   console.log(`Type is ${g[0]} and has ${g[1].size} members.`)
* }
* // Type is A and has 2 members.
* // Type is B and has 1 members.
*
* // with result creator
* const groups2 = enumerable.groupBy(
*   element => element.type,
*   (type, elements) => ({ type, amount: elements.sum(item => item.amount) })
* )
*
* console.log(groups2.toArray()) // yields [{"type":"A","amount":4},{"type":"B","amount":4}]
* </pre>
*
* Type parameter <code>TResult</code>: The type of the created elements.
* Type parameter <code>TKey</code>: The type of the key elements.
* @param options The parameters to pass.
* @param options.keySelector A function with the signature <code>function(element, index):TKey</code> which selects from each element the key.
* @param [options.resultCreator=null] An optional function with the signature <code>function(element, IEnumerable>TSource&lt;):TResult</code> which transforms each group of
* elements and their common key into a resulting object. If omitted, the default implementation will create arrays of
* length 2 where the first element is the key and the second element is an IEnumerable>TSource&lt; of the elements with the
* corresponding key.
* @param [options.thisArg=null] The optional value to use as <code>this</code> when executing the <code>keySelector</code> and <code>resultCreator</code> .
* @returns An enumeration of groups of the original enumerable.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23groupBy(function(T)%3ATKey%2Cfunction(TKey%2CIEnumerable%3CT%3E)%3ATResult%2CObject) Online Documentation}
*/
groupBy<TKey, TResult>(options: {
keySelector: (arg: T) => TKey;
resultCreator?: ((arg1: TKey, arg2: IEnumerable<T>) => TResult) | null;
thisArg?: any | null;
}): IEnumerable<TResult>;
/**
* Determines whether the given value is part of this enumerable.
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4])
*
* console.log(enumerable.includes(2)) // true
*
* console.log(enumerable.includes(5)) // false
* </pre>
*
* @param value The value to search for.
* @returns <code>true</code> if this enumerable contains the given value, <code>false</code> otherwise.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23includes(T) Online Documentation}
*/
includes(value: T): boolean;
/**
The index of the given item in the enumerable.
@param item The item to search for.
@param [fromIndex=0]
@returns The index of the given item in the enumerable. <code>-1</code> if the item is not in the enumerable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23indexOf(T%2Cnumber) Online Documentation}*/
indexOf(item: T | null, fromIndex?: number): number;
/**
* Returns the last element of this enumerable.
* <pre>
* const enumerable1 = IEnumerable.from([1, 2, 3, 4])
* console.log(enumerable1.last()) // yields 4
*
* // if the enumerable is empty then an exception will be thrown
* const enumerable2 = IEnumerable.from([])
* console.log(enumerable2.last()) // throws exception
* </pre>
*
* @returns The last element of this enumerable.
* @throws {Exception} <b>(InvalidOperationError)</b> If the enumerable is empty.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23last() Online Documentation}
*/
last(): T;
/**
* Returns the last element of this enumerable or <code>null</code> if this enumerable is empty.
* <pre>
* const enumerable1 = IEnumerable.from([1, 2, 3, 4])
* console.log(enumerable1.lastOrDefault()) // yields 4
*
* // if the enumerable is empty then null will be returned
* const enumerable2 = IEnumerable.from([])
* console.log(enumerable2.lastOrDefault()) // yields null
* </pre>
*
* @returns The last element of the enumerable or <code>null</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23lastOrDefault() Online Documentation}
*/
lastOrDefault(): T;
/**
* Returns a dynamic {@link IEnumerable} of this enumerable using the given <code>selector</code> function which returns a new object for each element of
* this enumerable.
* The returned enumerable is dynamic, in other words, if elements are added or removed from one of the original
* enumerables, the returned enumerable changes, too. If you want to store its current state, create a {@link IEnumerable#toList list} or {@link IEnumerable#toArray array} of
* it.
*
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3])
*
* const result = enumerable.map(item => item * 2)
*
* console.log(result.toArray()) // yields [2, 4, 6]
*
* // with index
* const result2 = enumerable.map((item, index) => item * index)
*
* console.log(result2.toArray()) // yields [0, 2, 6]
*
* // with index and enumerable
* const result3 = enumerable.map(
*   (item, index, enumerable) => `Item no ${index} of ${enumerable.size} is ${item}`
* )
*
* console.log(result3.toArray())
* // ["Item no 0 of 3 is 1","Item no 1 of 3 is 2","Item no 2 of 3 is 3"]
* </pre>
*
* <br><b>The returned enumerable is dynamic</b>
* <pre>
* const list = List.fromArray([1, 2, 3])
*
* const result = list.map(item => item * 2)
*
* list.add(4)
*
* // result changed also
* console.log(result.toArray()) // yields [2,4,6,8]
* </pre>
*
* Type parameter <code>TResult</code>: The type of the mapped elements.
* @param selector A function with the signature <code>function(element, index):TResult</code> which converts each element into a new element of the
* type <code>TResult</code>.
* @param [thisArg=null] The optional value to use as <code>this</code> when executing the <code>selector</code>.
* @returns A dynamic enumerable of the results of the provided function.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23map(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3ATResult%2CObject) Online Documentation}
*/
map<TResult>(selector: (arg1: T, arg2: number, arg3: IEnumerable<T> | null) => TResult, thisArg?: any | null): IEnumerable<TResult>;
/**
* Creates an enumerable that consists of the sequence of increasing values starting with <code>from</code> and a length of <code>count</code>.
* For each element the value returned will increase by 1.
*
* <pre>
* const enumerable = IEnumerable.ofRange(10, 4)
*
* console.log(enumerable.toArray()) // yields [10,11,12,13]
* </pre>
*
* @param from the first value to yield
* @param count the number of values to yield
* @returns A new enumerable that can yield <code>count</code> numbers starting with <code>from</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23ofRange(number%2Cnumber) Online Documentation}
*/
static ofRange(from: number, count: number): IEnumerable<number>;
/**
* Creates an enumerable that consists of <code>count</code> <code>item</code>s.
* <pre>
* const enumerable = IEnumerable.ofRepeat(10, 4)
*
* console.log(enumerable.toArray()) // yields [10,10,10,10]
* </pre>
*
* Type parameter <code>T</code>: The type of the element.
* @param item The element to yield.
* @param count The number of elements the new enumerable yields.
* @returns A new enumerable that can yield <code>count</code> <code>item</code>s
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23ofRepeat(T%2Cnumber) Online Documentation}
*/
static ofRepeat<T>(item: T, count: number): IEnumerable<T>;
/**
Converts this untyped enumerable into an enumerable with the given type.
The source enumerable may contain elements which are not of the given type. Those elements will not be contained in the
returned enumerable.
Type parameter <code>TResult</code>.
@returns A typed enumerable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23ofType() Online Documentation}*/
ofType<TResult>(resultType: Class): IEnumerable<TResult>;
/**
* Yields an ordered enumerable of the elements given a key selection function and optional comparison function and creates
* a ordered view of the enumerable.
* The sorting will happen each time the enumerable is {@link IEnumerable#getEnumerator enumerated}.
*
* <pre>
* const enumerable = IEnumerable.from([
*   { type: 'A.B', amount: 3 },
*   { type: 'A.C', amount: 2 },
*   { type: 'B.A', amount: 5 }
* ])
*
* // order by the elements length, without a comparer function
* const sortedEnumerable1 = enumerable.orderBy(entry => entry.amount)
*
* console.log(sortedEnumerable1.toArray())
* // [{"type":"A.C","amount":2},{"type":"A.B","amount":3},{"type":"B.A","amount":5}]
*
* // order by the elements length, with a comparer function
* const sortedEnumerable2 = enumerable.orderBy(
*   entry => entry.type,
*   (item1, item2) => (item1.charCodeAt(2) > item2.charCodeAt(2) ? 1 : -1)
* )
*
* console.log(sortedEnumerable2.toArray())
* // [{"type":"B.A","amount":5},{"type":"A.B","amount":3},{"type":"A.C","amount":2}]
* </pre>
*
* Type parameter <code>TKey</code>.
* @param [keySelector=null] The function that determines the key for sorting the elements. If omitted, the element itself is used.
* @param [comparer=null] A function with the signature <code>function(key1, key2):Number</code> which is used for the sort operation to compare the key
* elements in the enumerable. May be omitted.
* @param [thisArg=null]
* @returns An enumeration of groups of the original enumerable.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23orderBy(function(T)%3ATKey%2Cfunction(TKey%2CTKey)%3Anumber%2CObject) Online Documentation}
*/
orderBy<TKey>(keySelector?: ((arg: T) => TKey) | null, comparer?: ((arg1: TKey, arg2: TKey) => number) | null, thisArg?: any | null): IEnumerable<T>;
/**
* Yields an ordered enumerable of the elements given a key selection function and optional comparison function and creates
* a ordered view of the enumerable.
* The sorting will happen each time the enumerable is {@link IEnumerable#getEnumerator enumerated}.
*
* <pre>
* const enumerable = IEnumerable.from([
*   { type: 'A.B', amount: 3 },
*   { type: 'A.C', amount: 2 },
*   { type: 'B.A', amount: 5 }
* ])
*
* // order by the elements length, without a comparer function
* const sortedEnumerable1 = enumerable.orderBy(entry => entry.amount)
*
* console.log(sortedEnumerable1.toArray())
* // [{"type":"A.C","amount":2},{"type":"A.B","amount":3},{"type":"B.A","amount":5}]
*
* // order by the elements length, with a comparer function
* const sortedEnumerable2 = enumerable.orderBy(
*   entry => entry.type,
*   (item1, item2) => (item1.charCodeAt(2) > item2.charCodeAt(2) ? 1 : -1)
* )
*
* console.log(sortedEnumerable2.toArray())
* // [{"type":"B.A","amount":5},{"type":"A.B","amount":3},{"type":"A.C","amount":2}]
* </pre>
*
* Type parameter <code>TKey</code>.
* @param options The parameters to pass.
* @param [options.keySelector=null] The function that determines the key for sorting the elements. If omitted, the element itself is used.
* @param [options.comparer=null] A function with the signature <code>function(key1, key2):Number</code> which is used for the sort operation to compare the key
* elements in the enumerable. May be omitted.
* @param [options.thisArg=null]
* @returns An enumeration of groups of the original enumerable.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23orderBy(function(T)%3ATKey%2Cfunction(TKey%2CTKey)%3Anumber%2CObject) Online Documentation}
*/
orderBy<TKey>(options: {
keySelector?: ((arg: T) => TKey) | null;
comparer?: ((arg1: TKey, arg2: TKey) => number) | null;
thisArg?: any | null;
}): IEnumerable<T>;
/**
* Yields an ordered enumerable of the elements given a key selection function and optional comparison function and creates
* a ordered view of the enumerable where the elements are yielded in descending order.
* The sorting will happen each time the enumerable is {@link IEnumerable#getEnumerator enumerated}.
*
* <pre>
* const enumerable = IEnumerable.from([
*   { type: 'A', amount: 3 },
*   { type: 'B', amount: 2 },
*   { type: 'C', amount: 2 },
*   { type: 'D', amount: 5 }
* ])
*
* // order by the elements length, without a comparer function
* const sortedEnumerable1 = enumerable.orderByDescending(entry => entry.amount)
*
* console.log(sortedEnumerable1.toArray())
* // [{"type":"D","amount":5},{"type":"A","amount":3},{"type":"B","amount":2},{"type":"C","amount":2}]
* </pre>
*
* Type parameter <code>TKey</code>.
* @param [keySelector=null] The function that determines the key for sorting the elements. If omitted, the element itself is used.
* @param [comparer=null] A function with the signature <code>function(key1, key2):Number</code> which is used for the sort operation to compare the key
* elements in the enumerable. May be omitted.
* @param [thisArg=null]
* @returns An enumeration of groups of the original enumerable.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23orderByDescending(function(T)%3ATKey%2Cfunction(TKey%2CTKey)%3Anumber%2CObject) Online Documentation}
*/
orderByDescending<TKey>(keySelector?: ((arg: T) => TKey) | null, comparer?: ((arg1: TKey, arg2: TKey) => number) | null, thisArg?: any | null): IEnumerable<T>;
/**
* Yields an ordered enumerable of the elements given a key selection function and optional comparison function and creates
* a ordered view of the enumerable where the elements are yielded in descending order.
* The sorting will happen each time the enumerable is {@link IEnumerable#getEnumerator enumerated}.
*
* <pre>
* const enumerable = IEnumerable.from([
*   { type: 'A', amount: 3 },
*   { type: 'B', amount: 2 },
*   { type: 'C', amount: 2 },
*   { type: 'D', amount: 5 }
* ])
*
* // order by the elements length, without a comparer function
* const sortedEnumerable1 = enumerable.orderByDescending(entry => entry.amount)
*
* console.log(sortedEnumerable1.toArray())
* // [{"type":"D","amount":5},{"type":"A","amount":3},{"type":"B","amount":2},{"type":"C","amount":2}]
* </pre>
*
* Type parameter <code>TKey</code>.
* @param options The parameters to pass.
* @param [options.keySelector=null] The function that determines the key for sorting the elements. If omitted, the element itself is used.
* @param [options.comparer=null] A function with the signature <code>function(key1, key2):Number</code> which is used for the sort operation to compare the key
* elements in the enumerable. May be omitted.
* @param [options.thisArg=null]
* @returns An enumeration of groups of the original enumerable.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23orderByDescending(function(T)%3ATKey%2Cfunction(TKey%2CTKey)%3Anumber%2CObject) Online Documentation}
*/
orderByDescending<TKey>(options: {
keySelector?: ((arg: T) => TKey) | null;
comparer?: ((arg1: TKey, arg2: TKey) => number) | null;
thisArg?: any | null;
}): IEnumerable<T>;
/**
* Creates a wrapped enumerable that has on or several elements prepended to it.
* <pre>
* const enumerable1 = IEnumerable.from([1, 2, 3, 4])
* console.log(enumerable1.prepend(0).toArray()) // yields [0,1,2,3,4]
*
* // more than one value can be used too
* const enumerable2 = IEnumerable.from([3, 4, 5, 6])
* console.log(enumerable2.prepend(1, 2).toArray()) // yields [1,2,3,4,5,6]
* </pre>
*
* @param items The item(s) to prepend to the enumeration
* @returns A new live view over the original enumerable that has <code>items</code> prepended to it.
* @see IEnumerable#append
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23prepend(T) Online Documentation}
*/
prepend(...items: T[]): IEnumerable<T>;
/**
* Applies the <code>accumulator</code> function to this enumerable's elements.
* <br><b>The returned enumerable is dynamic</b>
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4, 5])
*
* const result = enumerable.reduce(
*   (accumulator, currentValue) => Math.max(accumulator, currentValue),
*   0
* )
*
* console.log(result) // expected output: 5
* </pre>
*
* Type parameter <code>TAccumulate</code>: The type of the accumulated value.
* @param accumulator A function with the signature <code>function(accumulate,element,index,source):TAccumulate</code> which "adds" (accumulates) a value
* depending on the element and index to the initial value and returns the result.
* @param initialValue The initial value for the accumulator. Omit/use the {@link IEnumerable#reduce} overload if you don't want to specify an initial value.
* @returns The result of the accumulation.
* @see IEnumerable#reduceRight
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23reduce(function(TAccumulate%2CT%2Cnumber%2CIEnumerable%3CT%3E)%3ATAccumulate%2CTAccumulate) Online Documentation}
*/
reduce<TAccumulate>(accumulator: (arg1: TAccumulate, arg2: T, arg3: number, arg4: IEnumerable<T> | null) => TAccumulate, initialValue: TAccumulate | null): TAccumulate;
/**
* Applies the <code>accumulator</code> function to this enumerable's elements.
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4, 5])
*
* const result = enumerable.reduce((accumulator, currentValue) => accumulator + currentValue)
*
* console.log(result) // expected output: 15
* </pre>
*
* @param accumulator A function with the signature <code>function(accumulate,element,index,source):TSource</code> which "adds" (accumulates) a value
* depending on the element and index to the initial value and returns the result.
* @returns The result of the accumulation.
* @see IEnumerable#reduce
* @see IEnumerable#reduceRight
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23reduce(function(T%2CT%2Cnumber%2CIEnumerable%3CT%3E)%3AT) Online Documentation}
*/
reduce(accumulator: (arg1: T, arg2: T, arg3: number, arg4: IEnumerable<T> | null) => T): T;
/**
* Applies the <code>accumulator</code> function to this enumerable's elements in reverse order.
* <br><b>The returned enumerable is dynamic</b>
* <pre>
* const enumerable = IEnumerable.from(['a', 'b', 'c', 'd'])
*
* const result = enumerable.reduceRight(
*   (accumulator, currentValue) => currentValue + accumulator,
*   '.'
* )
*
* console.log(result) // yields 'abcd.'
* </pre>
*
* Type parameter <code>TAccumulate</code>: The type of the accumulated value.
* @param accumulator A function with the signature <code>function(accumulate,element,index,source):TAccumulate</code> which "adds" (accumulates) a value
* depending on the element and index to the seed value and returns the result.
* @param initialValue The initial value for the accumulator. Omit/use the {@link IEnumerable#reduceRight} overload if you don't want to specify an initial
* value.
* @returns The final value of the accumulation.
* @see IEnumerable#reduceRight
* @see IEnumerable#reduce
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23reduceRight(function(TAccumulate%2CT%2Cnumber%2CIEnumerable%3CT%3E)%3ATAccumulate%2CTAccumulate) Online Documentation}
*/
reduceRight<TAccumulate>(accumulator: (arg1: TAccumulate, arg2: T, arg3: number, arg4: IEnumerable<T> | null) => TAccumulate, initialValue: TAccumulate | null): TAccumulate;
/**
* Applies the <code>accumulator</code> function to this enumerable's elements in reverse order.
* <pre>
* const enumerable = IEnumerable.from([
*   [0, 1],
*   [2, 3],
*   [4, 5]
* ])
*
* const result = enumerable.reduceRight((accumulator, currentValue) =>
*   accumulator.concat(currentValue)
* )
*
* console.log(result) // yields [4,5,2,3,0,1]
* </pre>
*
* @param accumulator A function with the signature <code>function(accumulate,element,index,source):TSource</code> which "adds" (accumulates) a value
* depending on the element and index to the seed value and returns the result.
* @returns The result of the accumulation.
* @see IEnumerable#reduceRight
* @see IEnumerable#reduce
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23reduceRight(function(T%2CT%2Cnumber%2CIEnumerable%3CT%3E)%3AT) Online Documentation}
*/
reduceRight(accumulator: (arg1: T, arg2: T, arg3: number, arg4: IEnumerable<T> | null) => T): T;
/**
* Creates a wrapped view of an enumerable which skips the given amount of elements.
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4, 5, 6, 7])
*
* const result = enumerable.skip(3)
*
* console.log(result.toArray()) // yields [4,5,6,7]
* </pre>
*
* @param amount The number of elements to skip at the beginning.
* @returns A new live view over the original enumerable that skips <code>amount</code> elements.
* @see IEnumerable#skipWhile
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23skip(number) Online Documentation}
*/
skip(amount: number): IEnumerable<T>;
/**
* Creates a wrapped view of an enumerable which skips a number of elements at the beginning while the given predicate
* yields <code>true</code>.
* While the <code>predicate</code> yields <code>true</code> for the elements of the original enumerable, the elements are skipped. The new
* enumeration will start with the first element for which the predicate yields
* <code>false</code>. All subsequent elements will be yielded.
*
* <pre>
* const enumerable = IEnumerable.from(['a', 'b', 'c', 'd', 'a', 'b', 'c', 'd'])
*
* const result = enumerable.skipWhile(element => element.charCodeAt(0) < 'c'.charCodeAt(0))
*
* console.log(result.toArray()) // yields ["c", "d", "a", "b", "c", "d"]
* </pre>
*
* @param predicate The predicate function that determines what elements should be skipped. While the predicate yields <code>true</code> for the provided
* item, the element will not be yielded.
* @param [thisArg=null] The optional object to use for <code>this</code>in the predicate <code>function</code>.
* @returns A new live view over the original enumerable that discards elements at the beginnging of the original enumeration while
* the predicate yields <code>true</code>.
* @see IEnumerable#take
* @see IEnumerable#takeWhile
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23skipWhile(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
skipWhile(predicate: (arg1: T, arg2: number, arg3: IEnumerable<T> | null) => boolean, thisArg?: any | null): IEnumerable<T>;
/**
* Determines whether this enumerable contains any elements matching the given <code>predicate</code>.
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4, 5, 6, 7])
*
* const result = enumerable.some(item => item % 2 === 0)
*
* console.log(result) // true
* </pre>
*
* @param [predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the element matches a condition.
* @param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns Whether this enumerable contains any elements matching the given <code>predicate</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23some(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
some(predicate?: ((arg1: T, arg2: number, arg3: IEnumerable<T> | null) => boolean) | null, thisArg?: any | null): boolean;
/**
* Determines whether this enumerable contains any elements matching the given <code>predicate</code>.
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4, 5, 6, 7])
*
* const result = enumerable.some(item => item % 2 === 0)
*
* console.log(result) // true
* </pre>
*
* @param options The parameters to pass.
* @param [options.predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the element matches a condition.
* @param [options.thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns Whether this enumerable contains any elements matching the given <code>predicate</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23some(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
some(options: {
predicate?: ((arg1: T, arg2: number, arg3: IEnumerable<T>) => boolean) | null;
thisArg?: any | null;
}): boolean;
/**
* Calculates the sum of the elements of this enumerable.
* <pre>
* const enumerable = IEnumerable.from([
*   { type: 'A', amount: 3 },
*   { type: 'B', amount: 2 },
*   { type: 'C', amount: 2 },
*   { type: 'D', amount: 5 }
* ])
*
* const result = enumerable.sum(element => element.amount)
*
* console.log(result) // yields 12
* </pre>
*
* @param selector A function with the signature <code>function(element):Number</code> which returns a numeric value for the given element.
* @param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
* @returns The sum of the elements of the enumerable.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23sum(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Anumber%2CObject) Online Documentation}
*/
sum(selector: (arg1: T, arg2: number, arg3: IEnumerable<T> | null) => number, thisArg?: any | null): number;
/**
* Creates a dynamic view of this enumerable with the given number of elements taken from the start or end of this
* enumerable.
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4, 5, 6, 7])
*
* // positive parameter means taking elements from the beginning of the enumerable
* console.log(enumerable.take(2).toArray()) // yields [1,2]
*
* // negative parameter means taking elements from the end of the enumerable
* console.log(enumerable.take(-2).toArray()) // yields [6,7]
* </pre>
*
* @param count The number of elements in the created enumerable. If the number is positive, the elements are taken from the beginning,
* otherwise, the elements are taken from the end.
* @returns A dynamic iew of this enumerable with the given number of elements.
* @see IEnumerable#takeWhile
* @see IEnumerable#skip
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23take(number) Online Documentation}
*/
take(count: number): IEnumerable<T>;
/**
* Returns a dynamic {@link IEnumerable} that contains the elements from this enumerable as long as the given <code>predicate</code> is <code>true</code>.
* The returned enumerable is dynamic, in other words, if elements are added or removed from one of the original
* enumerables, the returned enumerable changes, too. If you want to store its current state, create a {@link IEnumerable#toList list} or {@link IEnumerable#toArray array} of
* it.
*
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 30, 4, 5])
*
* const result = enumerable.takeWhile(item => item < 5)
*
* console.log(result.toArray()) // yields [1,2,3]
* </pre>
*
* @param predicate A function with the signature <code>function(element):boolean</code> which returns <code>true</code> as long as the elements should be added to
* the returned enumerable.
* @param [thisArg=null] The optional object to use for <code>this</code>in the <code>function</code>.
* @returns A dynamic enumerable that is a subset of the original enumerable.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23takeWhile(function(T%2Cnumber%2CIEnumerable%3CT%3E)%3Aboolean%2CObject) Online Documentation}
*/
takeWhile(predicate: (arg1: T, arg2: number, arg3: IEnumerable<T> | null) => boolean, thisArg?: any | null): IEnumerable<T>;
/**
Creates an array with the elements of this enumerable.
@returns An array with the enumerable's elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23toArray() Online Documentation}*/
toArray(): T[];
/**
Creates a {@link List} with the values of this enumerable.
@returns A {@link List} of the enumerable's elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23toList() Online Documentation}*/
toList(): List<T>;
/**
* Returns an dynamic enumerable which contains the elements of this enumerable in reverse order.
* The returned enumerable is dynamic, in other words, if elements are added or removed from one of the original
* enumerables, the returned enumerable changes, too. If you want to store its current state, create a {@link IEnumerable#toList list} or {@link IEnumerable#toArray array} of
* it.
*
* <pre>
* const enumerable = IEnumerable.from([1, 2, 3, 4, 5])
*
* const result = enumerable.toReversed()
*
* console.log(result.toArray()) // yields [5,4,3,2,1]
* </pre>
*
* <br><b>The returned enumerable is dynamic</b>
* <pre>
* const list = List.fromArray([1, 2, 3, 4, 5])
*
* const result = list.toReversed()
*
* list.add(0)
*
* // result changed also
* console.log(result.toArray()) // yields [0,5,4,3,2,1]
* </pre>
*
* @returns A dynamic enumerable which contains the elements of the original enumerable in reverse order.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23toReversed() Online Documentation}
*/
toReversed(): IEnumerable<T>;
/**
Returns the number of elements of this enumerable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerable%23size Online Documentation}*/
readonly size: number;
static $class: Class;
private $_IEnumerable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is IEnumerable<T>;
}
/** Automatic conversion to {@link IEnumerable}. */
type IEnumerableConvertible<T> = T[] | ArrayLike<T> | Iterable<T>;
export interface IEnumerator<T> {
}
/**
* Provides methods to iterate over a collection or list of elements of the same type.
* <pre>
* var e = someList.getEnumerator();
* // always move before you access
* while(e.moveNext()) {
*   // get the current element
*   var current = e.current;
* }
* </pre>
*
* Type parameter <code>T</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerator Online Documentation}
*/
export abstract class IEnumerator<T> {
/**
Moves this enumerator to the next element.
A newly created enumerator's {@link IEnumerator#current current position} is before the first element. Thus, this method must be called before first
access to the {@link IEnumerator#current} property. The same applies to the state after calling {@link IEnumerator#reset}.
@returns <code>true</code> if the enumerator was moved to a valid position
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerator%23moveNext() Online Documentation}*/
abstract moveNext(): boolean;
/**
Resets the enumerator to its starting state.
In other words: sets the current position before the first element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerator%23reset() Online Documentation}*/
abstract reset(): void;
/**
Gets the element at the enumerator's current position.
This value is undefined if the enumerator is in initial state (after creation or
{@link IEnumerator#reset}) or has been moved past the end of the represented collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnumerator%23current Online Documentation}*/
abstract readonly current: T;
static $class: Class;
private $_IEnumerator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is IEnumerator<T>;
}
export interface IList<T> extends ICollection<T> {
}
/**
* A read-write collection of objects of the same type.
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters when annotated as such:
*
* It is possible to specify an Array or to use iterables or generators to define the {@link IList}:
*
* <pre>
* // Array
* object.property = [item1, item2]
*
* // Iterables
* object.property = new Set([item1, item2])
* object.property = new Map([[key1, item1], [key2, item2]]).entries()
*
* // Generators
* function* generator(i) {
*   yield i;
*   yield i + 10;
* }
* object.property = generator(10)
* </pre>
*
* If the element adheres to the <a href="https://developer.mozilla.org/de/docs/Web/JavaScript/Reference/Iteration_protocols">'iterable' protocol</a> it will also be automatically converted.
*
* Type parameter <code>T</code>: The type of objects in the list.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IList Online Documentation}
*/
export abstract class IList<T> {
/**
Creates a {@link IList} instance from the given object by performing automatic type conversion.
This will perform the automatic type conversion on the given argument and return a proper {@link IList} instance.
Type parameter <code>T</code>.
@param arrayLike The object to convert to a {@link IList}
@returns A proper instance, initialized to the values found in <code>arrayLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IList%23from(IList%3CT%3E) Online Documentation}*/
static from<T>(arrayLike: IList<T> | IListConvertible<T>): IList<T>;
/**
Gets the item at the given index.
@param index The index of the item to access.
@returns The item at the given index.
@see IList#set
@see {@link https://docs.yworks.com/yfileshtml/#/api/IList%23get(number) Online Documentation}*/
abstract get(index: number): T;
/**
Inserts the given <code>item</code> at the given <code>index</code>.
@param index The index to insert the item at.
@param item The item to insert.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IList%23insert(number%2CT) Online Documentation}*/
abstract insert(index: number, item: T): void;
/**
Removes and returns the last value in this list.
This function mimics the JavaScript <code>Array.pop()</code> function and returns <code>undefined</code> if the list is empty.
@returns The removed value. <code>undefined</code> if the list was empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IList%23pop() Online Documentation}*/
pop(): T;
/**
Adds one or more elements to this list and returns the new length.
This function mimics the JavaScript <code>Array.push()</code> function.
@param values The values to append to the list.
@returns The new length of the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IList%23push(T) Online Documentation}*/
push(...values: T[]): number;
/**
Removes the item at the given index from the list.
@param index The index of the item to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IList%23removeAt(number) Online Documentation}*/
abstract removeAt(index: number): void;
/**
Sets the item at the given index.
@param index The index of the item to access.
@param value The item at the given index.
@see IList#get
@see {@link https://docs.yworks.com/yfileshtml/#/api/IList%23set(number%2CT) Online Documentation}*/
abstract set(index: number, value: T): void;
/**
Removes and returns the first value in this list.
This function mimics the JavaScript <code>Array.shift()</code> function and returns <code>undefined</code> if the list is empty.
@returns The removed value. <code>undefined</code> if the list was empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IList%23shift() Online Documentation}*/
shift(): T;
/**
Adds one or more elements to the beginning of this list and returns the new length.
This function mimics the JavaScript <code>Array.unshift()</code> function.
@param values The values to prepend to this list. The first argument will become the new first element in the list
@returns The new length of the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IList%23unshift(T) Online Documentation}*/
unshift(...values: T[]): number;
static $class: Class;
private $_IList: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is IList<T>;
}
/** Automatic conversion to {@link IList}. */
type IListConvertible<T> = T[] | ArrayLike<T> | Iterable<T>;
export interface MapEntry<TKey, TValue> extends YObject {
}
/**
The key/value pair of a {@link IMap}.
Type parameter <code>TKey</code>: The type of the key element.
Type parameter <code>TValue</code>: The type of the value element.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapEntry Online Documentation}*/
export class MapEntry<TKey, TValue> {
/**
Creates a new instance with the given <code>key</code> and <code>value</code>.
@param key The key for this pair.
@param value The value for this pair.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapEntry%23MapEntry(TKey%2CTValue) Online Documentation}*/
constructor(key: TKey, value: TValue);
/**
Returns whether this instance and the given <code>obj</code> are equal.
Two instances of this class are considered equal if they have the same values for each property.
@param obj The other instance to test for equality.
@returns <code>true</code> if both objects are equal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapEntry%23equals(Object) Online Documentation}*/
equals(obj: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapEntry%23hashCode() Online Documentation}*/
hashCode(): number;
/**
The key for this key/value pair.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapEntry%23key Online Documentation}*/
readonly key: TKey;
/**
The value for this key/value pair.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapEntry%23value Online Documentation}*/
readonly value: TValue;
static $class: Class;
private $_MapEntry: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TKey, TValue>(o: any): o is MapEntry<TKey, TValue>;
}
export interface List<T> extends YObject, IList<T> {
/**
@inheritdoc*/
add(item: T): void;
/**
@inheritdoc*/
clear(): void;
/**
@inheritdoc*/
copyTo(array: T[], arrayIndex: number): void;
/**
@inheritdoc*/
get(index: number): T;
/**
@inheritdoc*/
getEnumerator(): IEnumerator<T>;
/**
@inheritdoc*/
insert(index: number, item: T): void;
/**
@inheritdoc*/
remove(item: T): boolean;
/**
@inheritdoc*/
removeAt(index: number): void;
/**
@inheritdoc*/
set(index: number, value: T): void;
/**
@inheritdoc*/
readonly isReadOnly: boolean;
} /**
Default implementation of {@link IList}.
This implementation provides fast random indexed access. However, removing or adding elements from the beginning or in
the middle of the list will slower.
Type parameter <code>T</code>: The type of the list's elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List Online Documentation}*/
export class List<T> {
/**
Creates a new instance and fills it with the elements of the given <code>enumerable</code>.
@param enumerable The enumerable to fill the list with.
@param [mapFunction=null] The optional function to call for each element to map the generate the elements in the newly created list. The first
argument is the element, the second one the index and the third one is the <code>enumerable</code> itself (after possible
conversion).
@param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23List(IEnumerable%3CT%3E%2Cfunction(T%2Cnumber%2CIEnumerable%3CT%3E)%3AT%2CObject) Online Documentation}*/
constructor(enumerable: IEnumerable<T> | IEnumerableConvertible<T>, mapFunction?: ((arg1: T, arg2: number, arg3: IEnumerable<T> | null) => T) | null, thisArg?: any | null);
/**
Creates a new instance and fills it with the elements of the given <code>enumerable</code>.
@param options The parameters to pass.
@param options.enumerable The enumerable to fill the list with.
@param [options.mapFunction=null] The optional function to call for each element to map the generate the elements in the newly created list. The first
argument is the element, the second one the index and the third one is the <code>enumerable</code> itself (after possible
conversion).
@param [options.thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23List(IEnumerable%3CT%3E%2Cfunction(T%2Cnumber%2CIEnumerable%3CT%3E)%3AT%2CObject) Online Documentation}*/
constructor(options: {
enumerable: IEnumerable<T> | IEnumerableConvertible<T>;
mapFunction?: ((arg1: T, arg2: number, arg3: IEnumerable<T>) => T) | null;
thisArg?: any | null;
});
/**
Creates a new instance.
@param [options] The parameters to pass.
@param options.items An array of items which are added to this list. Each item will be added via the {@link List#add} method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23List() Online Documentation}*/
constructor(options?: {
items?: Array<any>;
});
/**
Adds the elements of the given collection to this list.
@param enumerable The collection to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23addRange(IEnumerable%3CT%3E) Online Documentation}*/
addRange(enumerable: IEnumerable<T> | IEnumerableConvertible<T>): void;
/**
Searches the (sorted) list for the given <code>item</code>.
This search is done as a binary search, therefore the list must be sorted. The given <code>comparer</code> is used to compare the
elements to each other.
@param item The item to search for.
@param comparer The comparer to use for comparing the elements.
@returns The index of the item in the list. -1 if the item is not found.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23binarySearch(T%2CIComparer%3CT%3E) Online Documentation}*/
binarySearch(item: T, comparer: IComparer<T>): number;
/**
Shallow copies part of this list to another location in the same list and returns it.
This function mimics the JavaScript <code>Array.copyWithin()</code> function, but returns a {@link List} instead of an array.
@param target Index at which to copy the sequence to. If negative <code>target</code> will be counted from the end.
@param [begin=0] Index at which to start copying from, <code>0</code> if omitted. If negative <code>begin</code> will be counted from the end.
@param [end=0x7FFFFFFF] The index of the element after the last element at which to end copying elements from. Defaults to the length of this
list. If negative <code>end</code> will be counted from the end.
@returns A new, shallow, independent copy of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23copyWithin(number%2Cnumber%2Cnumber) Online Documentation}*/
copyWithin(target: number, begin?: number, end?: number): List<T>;
/**
Shallow copies part of this list to another location in the same list and returns it.
This function mimics the JavaScript <code>Array.copyWithin()</code> function, but returns a {@link List} instead of an array.
@param options The parameters to pass.
@param options.target Index at which to copy the sequence to. If negative <code>target</code> will be counted from the end.
@param [options.begin=0] Index at which to start copying from, <code>0</code> if omitted. If negative <code>begin</code> will be counted from the end.
@param [options.end=0x7FFFFFFF] The index of the element after the last element at which to end copying elements from. Defaults to the length of this
list. If negative <code>end</code> will be counted from the end.
@returns A new, shallow, independent copy of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23copyWithin(number%2Cnumber%2Cnumber) Online Documentation}*/
copyWithin(options: {
target: number;
begin?: number;
end?: number;
}): List<T>;
/**
Fills the elements in this list from the <code>start</code> to the <code>end</code> index with <code>value</code>.
This function mimics the JavaScript <code>Array.fill()</code> function and returns <code>this</code>.
@param value The value to fill the elements with
@param [start=0] The index of the first cell in the list to fill with the value. <code>0</code> if not provided and <code>size+start</code> if negative.
@param [end=0x7FFFFFFF] The index after the last cell in the list to fill with the value. <code>size-1</code> if not provided and <code>size+end</code> if negative.
@returns The list itself for method chaining.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23fill(T%2Cnumber%2Cnumber) Online Documentation}*/
fill(value: T, start?: number, end?: number): List<T>;
/**
Fills the elements in this list from the <code>start</code> to the <code>end</code> index with <code>value</code>.
This function mimics the JavaScript <code>Array.fill()</code> function and returns <code>this</code>.
@param options The parameters to pass.
@param options.value The value to fill the elements with
@param [options.start=0] The index of the first cell in the list to fill with the value. <code>0</code> if not provided and <code>size+start</code> if negative.
@param [options.end=0x7FFFFFFF] The index after the last cell in the list to fill with the value. <code>size-1</code> if not provided and <code>size+end</code> if negative.
@returns The list itself for method chaining.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23fill(T%2Cnumber%2Cnumber) Online Documentation}*/
fill(options: {
value: T | null;
start?: number;
end?: number;
}): List<T>;
/**
Creates a new list similar in behavior to <code>Array.from()</code>.
Type parameter <code>TSource</code>.
Type parameter <code>T</code>: The type of the list's elements.
@param enumerable The enumerable to fill the list with.
@param [mapFunction=null] The optional function to call for each element to map the generate the elements in the newly created list
@param [thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
@returns A new {@link List} initialized with the values of the given enumerable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23from(IEnumerable%3CTSource%3E%2Cfunction(TSource%2Cnumber%2CObject)%3AT%2CObject) Online Documentation}*/
static from<TSource, T>(enumerable: IEnumerable<TSource> | IEnumerableConvertible<TSource>, mapFunction?: ((arg1: TSource, arg2: number, arg3: any | null) => T) | null, thisArg?: any | null): List<T>;
/**
Creates a new list similar in behavior to <code>Array.from()</code>.
Type parameter <code>TSource</code>.
Type parameter <code>T</code>: The type of the list's elements.
@param options The parameters to pass.
@param options.enumerable The enumerable to fill the list with.
@param [options.mapFunction=null] The optional function to call for each element to map the generate the elements in the newly created list
@param [options.thisArg=null] The optional object to use for <code>this</code> in the <code>function</code>.
@returns A new {@link List} initialized with the values of the given enumerable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23from(IEnumerable%3CTSource%3E%2Cfunction(TSource%2Cnumber%2CObject)%3AT%2CObject) Online Documentation}*/
static from<TSource, T>(options: {
enumerable: IEnumerable<TSource> | IEnumerableConvertible<TSource>;
mapFunction?: ((arg1: TSource, arg2: number, arg3: any) => T) | null;
thisArg?: any | null;
}): List<T>;
/**
Creates a new list with the given array used as the live backing store.
Modifying the list or the array will modify both instances.
Type parameter <code>T</code>: The type of the list's elements.
@param array The array to create a list view over from.
@returns A list backed by the given array.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23fromArray(Array%3CT%3E) Online Documentation}*/
static fromArray<T>(array: T[]): List<T>;
/**
Removes all elements for which <code>match</code> returns <code>true</code>.
@param match A predicate function with the signature <code>function(o:Object):boolean</code> which returns <code>true</code> if o matches the conditions of the
element to search for.
@returns The number of elements which were removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23removeAll(function(T)%3Aboolean) Online Documentation}*/
removeAll(match: (obj: T) => boolean): number;
/**
Removes a given <code>count</code> of elements starting from the given <code>index</code>.
@param index The index of the first element to remove.
@param count The number of elements to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23removeRange(number%2Cnumber) Online Documentation}*/
removeRange(index: number, count: number): void;
/**
Reverses the order of the elements in this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23reverse() Online Documentation}*/
reverse(): void;
/**
Returns a shallow copy of a portion of this list in the form of a new and independent list.
This function mimics the JavaScript <code>Array.slice()</code> function, but returns a {@link List} instead of an array.
@param [begin=0] The start index for the copy, <code>0</code> if omitted
@param [end=0x7FFFFFFF] The index of the element after the last element that will be copied. If omitted, the
@returns A new, shallow, independent copy of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23slice(number%2Cnumber) Online Documentation}*/
slice(begin?: number, end?: number): List<T>;
/**
Returns a shallow copy of a portion of this list in the form of a new and independent list.
This function mimics the JavaScript <code>Array.slice()</code> function, but returns a {@link List} instead of an array.
@param options The parameters to pass.
@param [options.begin=0] The start index for the copy, <code>0</code> if omitted
@param [options.end=0x7FFFFFFF] The index of the element after the last element that will be copied. If omitted, the
@returns A new, shallow, independent copy of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23slice(number%2Cnumber) Online Documentation}*/
slice(options: {
begin?: number;
end?: number;
}): List<T>;
/**
Sorts all elements in the list using the given <code>comparison</code> function.
@param comparison The optional comparison function to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23sort(function(T%2CT)%3Anumber) Online Documentation}*/
sort(comparison: (arg1: T, arg2: T) => number): void;
/**
Sorts all elements in the list using the given <code>comparer</code>.
@param comparer The comparer to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23sort(IComparer%3CT%3E) Online Documentation}*/
sort(comparer: IComparer<T>): void;
/**
Sorts all elements in list using the default comparer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23sort() Online Documentation}*/
sort(): void;
/**
Alters the contents of this list like JavaScript's <code>Array.splice()</code> method.
This function mimics the JavaScript <code>Array.splice()</code> function, but returns a {@link List} instead of an array.
@param start The index into the list at which elements should be removed and inserted.
@param [deleteCount=-1] The number of items to remove starting at <code>start</code>. If omitted all elements after <code>start</code> will be removed.
@param items The items to insert. If omitted, no items will be added.
@returns The elements that have been removed in the form of a {@link List}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23splice(number%2Cnumber%2CT) Online Documentation}*/
splice(start: number, deleteCount?: number, ...items: T[]): List<T>;
/**
Alters the contents of this list like JavaScript's <code>Array.splice()</code> method.
This function mimics the JavaScript <code>Array.splice()</code> function, but returns a {@link List} instead of an array.
@param options The parameters to pass.
@param options.start The index into the list at which elements should be removed and inserted.
@param [options.deleteCount=-1] The number of items to remove starting at <code>start</code>. If omitted all elements after <code>start</code> will be removed.
@param options.items The items to insert. If omitted, no items will be added.
@returns The elements that have been removed in the form of a {@link List}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23splice(number%2Cnumber%2CT) Online Documentation}*/
splice(options: {
start: number;
deleteCount?: number;
items?: T[];
}): List<T>;
/**
Copies the elements of this list into an array.
@returns An array containing the elements of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/List%23toArray() Online Documentation}*/
toArray(): T[];
static $class: Class;
private $_List: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is List<T>;
}
export interface PropertyChangedEventArgs extends EventArgs {
}
/**
Event arguments which contain information for a {@link IPropertyObservable#addPropertyChangedListener PropertyChanged}
event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyChangedEventArgs Online Documentation}*/
export class PropertyChangedEventArgs {
/**
Creates a new instance.
@param propertyName The name of the changed property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyChangedEventArgs%23PropertyChangedEventArgs(string) Online Documentation}*/
constructor(propertyName: string);
/**
The name of the changed property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyChangedEventArgs%23propertyName Online Documentation}*/
readonly propertyName: string;
static $class: Class;
private $_PropertyChangedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PropertyChangedEventArgs;
}
export interface IPropertyObservable {
}
/**
Defines an event which is dispatched when a property changes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPropertyObservable Online Documentation}*/
export abstract class IPropertyObservable {
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property changes.
@param listener The listener to add.
@see IPropertyObservable#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPropertyObservable%23PropertyChanged Online Documentation}*/
abstract addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property changes.
@param listener The listener to remove.
@see IPropertyObservable#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPropertyObservable%23PropertyChanged Online Documentation}*/
abstract removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
static $class: Class;
private $_IPropertyObservable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPropertyObservable;
}
export interface ICloneable {
}
/**
Implemented by objects which can create a clone of themselves.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICloneable Online Documentation}*/
export abstract class ICloneable {
/**
Create a clone of this object.
@returns A clone of this object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICloneable%23clone() Online Documentation}*/
abstract clone(): this;
static $class: Class;
private $_ICloneable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ICloneable;
}
export interface IComparer<T> {
}
/**
Provides a method to compare two objects of the same type.
Type parameter <code>T</code>: The type of the objects to compare.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IComparer Online Documentation}*/
export abstract class IComparer<T> {
/**
Compares two objects of type T.
@param x The first object.
@param y The second object.
@returns
<ul>
<li>-1: <code>x</code> is less than <code>y</code>
<li>0: <code>x</code> is equal to <code>y</code>
<li>1: <code>x</code> is greater than <code>y</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/IComparer%23compare(T%2CT) Online Documentation}*/
abstract compare(x: T, y: T): number;
/**
Creates an implementation of the interface {@link IComparer} by using the given function as implementation for its {@link IComparer#compare} method.
@param compare A function for {@link IComparer}'s single abstract method {@link IComparer#compare}.
@returns An instance of the {@link IComparer} interface based on the given function.*/
static create<T>(compare: (x: T, y: T) => number): IComparer<T>;
static $class: Class;
private $_IComparer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is IComparer<T>;
}
export interface Workarounds extends YObject {
}
/**
* This object is a container for all workarounds introduced for different environments.
* When ES modules are used, it can be imported like this:
*
* <pre>
* import { Workarounds } from 'yfiles';
*
* Workarounds.cr320635 = true;
* </pre>
*
* In an environment without ES2015, workarounds can alternatively be activated like before:
*
* <pre>
* yfiles.workaroundCR320635 = true;
* </pre>
*
* or using this container type:
*
* <pre>
* yfiles.lang.Workarounds.cr320635 = true;
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds Online Documentation}
*/
export class Workarounds {
/**
* Safari Issue: two finger pinch zooming fires bogus mouse events when releasing both fingers at the same time.
* Symptoms: {@link MoveViewportInputMode} gets stuck and panning is no longer possible.
*
* Workaround: ignore these bogus mouse events shortly after the last touchend event. The time interval in which these
* events shall be ignored can be set via this workaround flag. Setting it to <code>&lt;= 0</code> will disable the workaround. The default
* value is <code>100</code>.
*
* <pre>
* Workarounds.wk203237 = 100;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundWK203237 = 100;
* </pre>
*
* Vendor Link: <a href="https://bugs.webkit.org/show_bug.cgi?id=203237">https://bugs.webkit.org/show_bug.cgi?id=203237</a>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23wk203237 Online Documentation}
*/
static wk203237: number;
/**
* Chrome Issue: Accessing <code>SVGElement.transform.baseVal</code> results in severe memory leak.
* Symptoms: Memory leak when using SVG styles in large graphs.
*
* Workaround: Avoid setting <code>SVGElement.transform.baseVal</code>.
*
* <pre>
* Workarounds.cr320635 = true;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundCR320635 = true;
* </pre>
*
* Vendor Link: <a href="https://code.google.com/p/chromium/issues/detail?id=320635">https://code.google.com/p/chromium/issues/detail?id=320635</a>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23cr320635 Online Documentation}
*/
static cr320635: boolean;
/**
* Chrome Issue: Don't fire an artificial <code>mousedown</code> event when context menu key is pressed.
* Symptoms: Unexpected behaviors when using mouse button in conjunction with the context menu key.
*
* Workaround: Prevent artificial <code>mousedown</code> events in Chrome by calling <code>preventDefault()</code>.
*
* <pre>
* Workarounds.cr433873 = true;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundCR433873 = true;
* </pre>
*
* Vendor Link: <a href="https://code.google.com/p/chromium/issues/detail?id=433873">https://code.google.com/p/chromium/issues/detail?id=433873</a>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23cr433873 Online Documentation}
*/
static cr433873: boolean;
/**
* Chrome Issue: setInterval and setTimeout stop during input if timer callbacks are expensive.
* Symptoms: Unresponsive GraphComponent during mouse drag, screen updates at ~1 FPS while moving the mouse.
*
* Workaround: Make sure that no <code>setTimeout(fn,0)</code> calls are "lost" by flushing Dispatcher actions on mouse move.
*
* This workaround is enabled by default and can be disabled by setting a value &lt; 0. A value > 0 sets the maximum tolerated
* delay between last dispatcher invoke call and manual flush, e.g. 26 (flush uses requestAnimationFrame).
*
* <pre>
* Workarounds.cr570845 = true;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundCR570845 = true;
* </pre>
*
* Vendor Link: <a href="https://code.google.com/p/chromium/issues/detail?id=570845">https://code.google.com/p/chromium/issues/detail?id=570845</a>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23cr570845 Online Documentation}
*/
static cr570845: number;
/**
* Internet Explorer Issue: IE does not implement <code>xml:space="preserve"</code>.
* Symptoms: White space characters are incorrectly rendered in SVG labels.
*
* Workaround: Replaces each non-newline whitespace in label text with '<code>\u00A0</code>'.
*
* <pre>
* Workarounds.ie964525 = true;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundIE964525 = true;
* </pre>
*
* Vendor Link: <a href="https://connect.microsoft.com/IE/Feedback/Details/964525">https://connect.microsoft.com/IE/Feedback/Details/964525</a>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23ie964525 Online Documentation}
*/
static ie964525: boolean;
/**
* Edge Issue: Edge's <code>DOMParser.parseFromString</code> method converts known SVG attributes to all upper case.
* Symptoms: Template bindings not working.
*
* Workaround: Change attribute names with only upper case characters in string template styles to lower case attribute
* names.
*
* <pre>
* Workarounds.edge2057021 = true;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundEDGE2057021 = true;
* </pre>
*
* Vendor Link: <a href="https://connect.microsoft.com/IE/Feedback/Details/2057021">https://connect.microsoft.com/IE/Feedback/Details/2057021</a>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23edge2057021 Online Documentation}
*/
static edge2057021: boolean;
/**
* Internet Explorer Issue: SVG <code>&lt;use></code> elements disappear on Windows 10.
* Symptoms: Resize handles are missing sometimes or other <code>&lt;use></code> elements are not there anymore.
*
* Workaround: Fix broken <code>href</code> attributes in DOM elements after they have been reinserted, reappended, or replaced.
*
* Seems to only happen in Internet Explorer 11 on Windows 10.
*
* <pre>
* Workarounds.ie2337112 = true;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundIE2337112 = true;
* </pre>
*
* Vendor Link: <a href="https://connect.microsoft.com/IE/feedback/details/2337112/svg-use-elements-disappear-on-windows-10">https://connect.microsoft.com/IE/feedback/details/2337112/svg-use-elements-disappear-on-windows-10</a>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23ie2337112 Online Documentation}
*/
static ie2337112: boolean;
/**
Refers to the same workaround as {@link Workarounds#ie2337112}.
This field is kept for backwards compatibility.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23ie233711 Online Documentation}*/
static ie233711: boolean;
/**
* Sets the CSS <code>pointer-events</code> property to <code>none</code> for <code>&lt;use></code> elements in visual templates such as handles.
* Note that in contrast to most other workarounds, this workaround is enabled unless explicitly disabled by setting
* <code>false</code>.
*
* <pre>
* Workarounds.ie205775 = true;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundIE205775 = true;
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23ie205775 Online Documentation}
*/
static ie205775: boolean;
/**
* Chrome Issue: Chrome 57 does not apply the SVG transform matrix to the GraphComponent's <code>&lt;svg></code> element.
* Symptoms: The graph rendering seems to be out of sync with the actual graph item's locations.
*
* Workaround: Re-apply the SVG transform matrix once the GraphComponent's <code>&lt;svg></code> element is the DOM.
*
* <pre>
* Workarounds.cr701075 = true;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundCR701075 = true;
* </pre>
*
* Vendor Link: <a href="https://bugs.chromium.org/p/chromium/issues/detail?id=701075">https://bugs.chromium.org/p/chromium/issues/detail?id=701075</a>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23cr701075 Online Documentation}
*/
static cr701075: boolean;
/**
* GWT Issue: Array.isArray causes JavaScript error in Internet Explorer with GWT.
* Symptoms: JavaScript error or unexpected code behavior that can be traced down to <code>Array.isArray</code>.
*
* <pre>
* Workarounds.ie7766782 = true;
* </pre>
*
* or
*
* <pre>
* yfiles.workaroundIE7766782 = true;
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23ie7766782 Online Documentation}
*/
static ie7766782: boolean;
/**
The maximum duration of the leave animation of a tooltip in milliseconds.
After this duration, a tooltip is eventually removed from the DOM after the leave animation was triggered.
By default, the tooltip element is automatically removed from the DOM after its CSS transition/animation ended. However,
the "animation end" event can fail under rare circumstances such that the element must be removed explicitly.
The leave transition should not take longer than this duration.
The default value is <code>5000</code> (5 seconds).
@see {@link https://docs.yworks.com/yfileshtml/#/api/Workarounds%23toolTipLeaveAnimationMaximumDuration Online Documentation}*/
static toolTipLeaveAnimationMaximumDuration: number;
static $class: Class;
private $_Workarounds: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Workarounds;
}
export interface XmlName extends YObject {
}
/**
Represents an XML name consisting of a local (unqualified) name for an element or attribute and the URI of the namespace
for that element or attribute.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlName Online Documentation}*/
export class XmlName {
/**
Initializes a new instance of the {@link XmlName} class.
@param localName The local name of the element.
@param ns The namespace it belongs to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlName Online Documentation}*/
constructor(localName: string, ns: string | null);
/**
Performs a conversion from {@link string} to {@link XmlName}.
The name is made up of at most two parts: the namespace and the local name. When given an input like <code>"http://www.w3.org/2000/xmlns/}local"</code>
it will be split at the "}" character and the first part will be used as the namespace URI, the second part as the local
name. If no "}" character is used in the input then it will be assumed that the input consists of the local name only.
@param expandedName Name of the expanded.
@returns The result of the conversion.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlName%23op_Implicit(string) Online Documentation}*/
static convertFrom(expandedName: string): XmlName | null;
/**
Determines whether another object is equal to this one.
@param obj the other object to compare for equality
@returns <code>true</code> if <code>obj</code> is considered equal to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlName%23equals(Object) Online Documentation}*/
equals(obj: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlName%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlName%23toString() Online Documentation}*/
toString(): string;
/**
Gets or sets the local name.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlName%23localName Online Documentation}*/
localName: string;
/**
Gets the namespace.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlName%23namespace Online Documentation}*/
namespace: string | null;
static $class: Class;
private $_XmlName: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is XmlName;
}
export interface TimeSpan extends YObject, IComparable {
/**
@inheritdoc*/
compareTo(obj: any | null): number;
} /**
* A structure which represents a time span.
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* There are some string formats that are converted to {@link TimeSpan}. For example:
*
* <pre>
* '1s'
* '0.5m'
* '1h'
* '1h 20m'
* '1:20'
* '1:20:30.5'
* </pre>
*
* It is also possible to use a {@link number} which will be interpreted in milliseconds.
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan Online Documentation}
*/
export class TimeSpan {
/**
Creates a new time span with a given length.
@param days The days part of the time span.
@param hours The hour part of the time span.
@param minutes The minute part of the time span.
@param seconds The seconds part of the time span.
@param millis The milliseconds part of the time span.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23TimeSpan(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(days: number, hours: number, minutes: number, seconds: number, millis: number);
/**
Creates a new time span which is <code>ticks</code> milliseconds long.
@param ticks The length of the time span in milliseconds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23TimeSpan(number) Online Documentation}*/
constructor(ticks: number);
/**
Creates a {@link TimeSpan} instance from the given TimeSpan-like object by performing automatic type conversion.
@param timeSpanLike The object to convert to a {@link TimeSpan}.
@returns Either <code>timeSpanLike</code> if it is already a {@link TimeSpan} or a new instance, initialized to the values found in
<code>timeSpanLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23from(TimeSpan) Online Documentation}*/
static from(timeSpanLike: TimeSpan | TimeSpanConvertible): TimeSpan;
/**
Creates a new time span with the given length in milliseconds.
@param milliseconds The length of the time span in milliseconds.
@returns A newly created time span.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23fromMilliseconds(number) Online Documentation}*/
static fromMilliseconds(milliseconds: number): TimeSpan;
/**
Creates a new time span with the given length in minutes.
@param minutes The length of the time span in minutes.
@returns A newly created time span.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23fromMinutes(number) Online Documentation}*/
static fromMinutes(minutes: number): TimeSpan;
/**
Creates a new time span with the given length in seconds.
@param seconds The length of the time span in seconds.
@returns A newly created time span.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23fromSeconds(number) Online Documentation}*/
static fromSeconds(seconds: number): TimeSpan;
/**
Empty time span.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23ZERO Online Documentation}*/
static ZERO: TimeSpan;
/**
The total milliseconds (ticks) of this time span.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23totalMilliseconds Online Documentation}*/
readonly totalMilliseconds: number;
/**
The total seconds of this time span.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23totalSeconds Online Documentation}*/
readonly totalSeconds: number;
/**
The total seconds of this time span.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TimeSpan%23totalMinutes Online Documentation}*/
readonly totalMinutes: number;
static $class: Class;
private $_TimeSpan: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TimeSpan;
}
/** Automatic conversion to {@link TimeSpan}. */
type TimeSpanConvertible = string;
export interface IMapper<K, V> {
}
/**
An interface for simple typed dictionary-like implementations.
{@link IMapper}s are most prominently used when working with {@link LayoutData supplemental configuration for layout algorithms} and custom data in
{@link GraphMLIOHandler GraphML}. {@link IGraph} also provides the {@link IGraph#mapperRegistry} to add additional information to a graph.
Type parameter <code>K</code>: The type of element this interface accepts as keys for the mapping.
Type parameter <code>V</code>: The type of element this interface accepts as values for the mapping.
Using {@link IMapper} is described in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_graph-binding_data Associating Data with Graph Elements}.
@see Mapper
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapper Online Documentation}*/
export abstract class IMapper<K, V> {
/**
Creates an {@link IMapper} instance from the given mapper-like object by performing automatic type conversion.
<b>Warning:</b> This method is not available unless the module <b>view-component</b> is loaded.
Type parameter <code>K</code>: The type of element this interface accepts as keys for the mapping.
Type parameter <code>V</code>: The type of element this interface accepts as values for the mapping.
@param mapperLike The object to convert to a {@link IMapper}
@returns An {@link IMapper} instance, initialized with the values found in <code>mapperLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapper%23from(IMapper%3CK%2CV%3E) Online Documentation}*/
static from<K, V>(mapperLike: IMapper<K, V> | IMapperConvertible<K, V>): IMapper<K, V>;
/**
Creates a constant mapper that will always return the <code>constant</code>.
Type parameter <code>K</code>: The type of the key to use.
Type parameter <code>V</code>: The type of the value to use.
@param constant The constant to return in {@link IMapper#get}
@returns A new instance that will always return the same constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapper%23fromConstant(V) Online Documentation}*/
static fromConstant<K, V>(constant: V): IMapper<K, V>;
/**
Create an implementation of {@link IMapper} that delegates {@link IMapper#get} getter calls to the provided handler.
The implementation returned silently ignores calls to the {@link IMapper#get} setter.
Type parameter <code>K</code>: The type of the key to use.
Type parameter <code>V</code>: The type of the value to use.
@param getter The delegate to delegate {@link IMapper#get} getter calls to.
@returns An implementation that delegates to <code>getter</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapper%23fromDelegate(function(K)%3AV) Online Documentation}*/
static fromDelegate<K, V>(getter: (key: K) => V): IMapper<K, V>;
/**
Gets the value for a given key in the mapping.
Setting a value for an already existing key overwrites the previous value.
It depends on the specific implementation of the mapper whether a mapping can be removed. If an implementation provides
a way to remove a mapping, that should be preferred over setting a <code>null</code> value. For example, see the {@link HashMap#delete} and {@link Mapper#delete}
methods of {@link HashMap} and {@link Mapper}, respectively.
@param key The key of the mapping.
@returns The value.
@see IMapper#set
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapper%23get(K) Online Documentation}*/
abstract get(key: K | null): V | null;
/**
Sets the value for a given key in the mapping.
Setting a value for an already existing key overwrites the previous value.
It depends on the specific implementation of the mapper whether a mapping can be removed. If an implementation provides
a way to remove a mapping, that should be preferred over setting a <code>null</code> value. For example, see the {@link HashMap#delete} and {@link Mapper#delete}
methods of {@link HashMap} and {@link Mapper}, respectively.
@param key The key of the mapping.
@param value The value.
@see IMapper#get
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapper%23set(K%2CV) Online Documentation}*/
abstract set(key: K | null, value: V | null): void;
static $class: Class;
private $_IMapper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<K, V>(o: any): o is IMapper<K, V>;
}
/** Automatic conversion to {@link IMapper}. */
type IMapperConvertible<K, V> = Map<K, V> | IMap<K, V>;
export interface Animator extends YObject {
}
/**
Runs {@link IAnimation animations} in a {@link CanvasComponent}.
For running {@link IAnimation} instances the method {@link Animator#animate} can be used. For simpler cases of
custom animations there are also overloads that take an animation callback.
@see IAnimation
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator Online Documentation}*/
export class Animator {
/**
Initializes a new instance of the {@link Animator} class for the given {@link CanvasComponent}.
Using this constructor this instance will query an instance of {@link WaitInputMode} from the {@link CanvasComponent#lookup canvas's context} to
automatically set the {@link WaitInputMode#waiting} property during animations if {@link Animator#allowUserInteraction} is set to <code>false</code>. The animation is not started until {@link Animator#animate}
or {@link Animator#animate} is called.
@param canvas The canvas control.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator Online Documentation}*/
constructor(canvas: CanvasComponent);
/**
Initializes a new instance of the {@link Animator} class for the given {@link CanvasComponent}.
Using this constructor this instance will query an instance of {@link WaitInputMode} from the {@link CanvasComponent#lookup canvas's context} to
automatically set the {@link WaitInputMode#waiting} property during animations if {@link Animator#allowUserInteraction} is set to <code>false</code>. The animation is not started until {@link Animator#animate}
or {@link Animator#animate} is called.
@param options The parameters to pass.
@param options.canvas The canvas control.
@param [options.allowUserInteraction] Whether user interaction should be allowed during the animation. This option sets the {@link Animator#allowUserInteraction} property on the created object.
@param [options.autoInvalidation] A value indicating whether to automatically {@link Animator#invalidateComponent invalidate the control} this instance has been created for. This option sets the {@link Animator#autoInvalidation} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator Online Documentation}*/
constructor(options: {
canvas: CanvasComponent;
allowUserInteraction?: boolean;
autoInvalidation?: boolean;
});
/**
Starts animating the given animation for the specified duration.
@param callback The callback to use for the animation.
@param duration The duration in milliseconds that the animation should last.
@returns A Promise that will resolve once the animation is finished.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator%23animate(function(number)%3Avoid%2CTimeSpan) Online Documentation}*/
animate(callback: (time: number) => void, duration: TimeSpan | TimeSpanConvertible): Promise<IAnimation>;
/**
Animates the given animation instance and triggers the callback upon completion.
@param animation The animation to perform.
@returns A Promise that will resolve once the animation is finished.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator%23animate(IAnimation) Online Documentation}*/
animate(animation: IAnimation): Promise<IAnimation>;
/**
Invalidates the control if {@link Animator#autoInvalidation} is set to <code>true</code>.
This implementation calls {@link CanvasComponent#updateVisual }.
@param canvas The control.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator%23invalidateComponent(CanvasComponent) Online Documentation}*/
invalidateComponent(canvas: CanvasComponent): void;
/**
Stops all running animations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator%23stop() Online Documentation}*/
stop(): void;
/**
Gets or sets whether user interaction should be allowed during the animation.
If <code>false</code>, the {@link WaitInputMode} is queried from the {@link CanvasComponent} and {@link WaitInputMode#waiting} is enabled during the
animation. The default is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator%23allowUserInteraction Online Documentation}*/
allowUserInteraction: boolean;
/**
Gets the control the animations are run on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator%23canvasComponent Online Documentation}*/
readonly canvasComponent: CanvasComponent;
/**
Gets or sets a value indicating whether the animator should pause or resume.
Adding an animation in a suspended animator does not start the animation unless Pause is set to true.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator%23paused Online Documentation}*/
paused: boolean;
/**
Gets or sets a value indicating whether to automatically {@link Animator#invalidateComponent invalidate the control} this instance has been created for.
The default is true.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Animator%23autoInvalidation Online Documentation}*/
autoInvalidation: boolean;
static $class: Class;
private $_Animator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Animator;
}
export interface IAnimation {
}
/**
An interface for animations that can be run by {@link Animator}.
Instances of classes implementing this interface change their state according to a relative animation time. The
animation time is a double between 0 for the start of the animation and 1 for the end of the animation.
<ul>
<li>An animated object is first created using the constructor of a subclass.
<li>Immediately before the animation a client calls {@link IAnimation#initialize}.
<li>To change the state of the animated object a client does a series of calls to
{@link IAnimation#animate}, usually with an increasing relative time parameter.
<li>If the animation is done, the client calls {@link IAnimation#cleanUp} once.
<li>While {@link IAnimation#animate} uses a relative time scheme, there might be an absolute time value for the
preferred duration of the whole sequence of animation steps. Since an IAnimation instance has no control on the number
of {@link IAnimation#animate} calls or the intervals between them, the preferred duration is only a hint on how long
the animation should take. Clients can ask for that hint using {@link IAnimation#preferredDuration} and try to find a suitable sequence of {@link IAnimation#animate}
calls in order to fulfill the preference.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation Online Documentation}*/
export abstract class IAnimation {
/**
Does the animation according to the relative animation time.
The animation starts with the time 0 and ends with time 1.
@param time the animation time [0,1]
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23animate(number) Online Documentation}*/
abstract animate(time: number): void;
/**
Cleans up after an animation has finished.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23cleanUp() Online Documentation}*/
abstract cleanUp(): void;
/**
Creates an eased animation for the given base animation.
The animation speed linearly increases up to a <code>easeIn</code> and linearly decreases from
<code>easeOut</code>. Between those two points, the animation speed is constant.
The duration of the ease in and ease out is determined by a mapping [0,1] → [0,1]. The mapping has to be increasing to
preserve the order of the frames in the decorated animation. In general, we consider continuously differentiable
functions <i>f</i> where integral of <i>f</i>′ equals 1 a suitable choice for the mapping.
@param [easeIn=0.5] The ratio for the ease-in time [0,1] where 0 is the start of the animation and 1 the end. <code>0.5</code> is the default value.
@param [easeOut=0.5] The ratio for the ease-out time [0,1] where 0 is the start of the animation and 1 the end. <code>0.5</code> is the default value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createEasedAnimation(number%2Cnumber) Online Documentation}*/
createEasedAnimation(easeIn?: number, easeOut?: number): IAnimation;
/**
Creates an eased animation for the given base animation.
The animation speed linearly increases up to a <code>easeIn</code> and linearly decreases from
<code>easeOut</code>. Between those two points, the animation speed is constant.
The duration of the ease in and ease out is determined by a mapping [0,1] → [0,1]. The mapping has to be increasing to
preserve the order of the frames in the decorated animation. In general, we consider continuously differentiable
functions <i>f</i> where integral of <i>f</i>′ equals 1 a suitable choice for the mapping.
@param options The parameters to pass.
@param [options.easeIn=0.5] The ratio for the ease-in time [0,1] where 0 is the start of the animation and 1 the end. <code>0.5</code> is the default value.
@param [options.easeOut=0.5] The ratio for the ease-out time [0,1] where 0 is the start of the animation and 1 the end. <code>0.5</code> is the default value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createEasedAnimation(number%2Cnumber) Online Documentation}*/
createEasedAnimation(options: {
easeIn?: number;
easeOut?: number;
}): IAnimation;
/**
Creates a new {@link IAnimation} that animates the given <code>edge</code>'s bends from its current shape linearly to the shape
given by the <code>endBends</code> and final port locations.
The bends of the edge layout are animated from their current location to their target locations. It is assumed that the
edge style is a polyline style and the control points are the bends accordingly. The animation works for other styles
too, but the effect might not be as expected. The number of bends in <code>endBends</code> can be different to the current number of
bends. This will result in the animation automatically removing bends at the end of the animation or inserting new bends
accordingly at the beginning of the animation.
The source and target points are also animated to new locations. However, the ports will not be moved by this animation.
Instead a separate {@link IAnimation#createPortAnimation animation for the ports} is needed to actually move
{@link IPort}s at the ends of the edge if desired.
@param graph The graph the animated edge belongs to.
@param edge The edge layout to animate.
@param endBends The bend positions after the animation.
@param endSourceLocation The absolute position of the source port after the animation.
@param endTargetLocation The absolute position of the target port after the animation.
@param preferredDuration The preferred duration of the animation in milliseconds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createEdgeSegmentAnimation(IGraph%2CIEdge%2CArray%3CIPoint%3E%2CPoint%2CPoint%2CTimeSpan) Online Documentation}*/
static createEdgeSegmentAnimation(graph: IGraph, edge: IEdge, endBends: IPoint[], endSourceLocation: Point | PointConvertible, endTargetLocation: Point | PointConvertible, preferredDuration: TimeSpan | TimeSpanConvertible): IAnimation;
/**
Creates a new {@link IAnimation} that animates the given layout of all types of graph items.
@param graph The graph for which the layout should be animated.
@param targetNodeLayouts The node layouts after the animation.
@param targetBendLocations The bend locations after the animation, for each edge the points are interpreted as the position of the bends along the
edge in the given order.
@param targetLayoutParameters The label model parameters for each label after the animation.
@param targetPortLocations The {@link IPortLocationModelParameter}s for each {@link IPort} in the graph that will be morphed.
@param preferredDuration The preferred duration of the animation in milliseconds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createGraphAnimation(IGraph%2CIMapper%3CINode%2CIRectangle%3E%2CIMapper%3CIEdge%2CArray%3CIPoint%3E%3E%2CIMapper%3CIPort%2CIPortLocationModelParameter%3E%2CIMapper%3CILabel%2CILabelModelParameter%3E%2CTimeSpan) Online Documentation}*/
static createGraphAnimation(graph: IGraph, targetNodeLayouts: IMapper<INode, IRectangle> | null | IMapperConvertible<INode, IRectangle>, targetBendLocations: IMapper<IEdge, IPoint[]> | null | IMapperConvertible<IEdge, IPoint[]>, targetPortLocations: IMapper<IPort, IPortLocationModelParameter> | null | IMapperConvertible<IPort, IPortLocationModelParameter>, targetLayoutParameters: IMapper<ILabel, ILabelModelParameter> | null | IMapperConvertible<ILabel, ILabelModelParameter>, preferredDuration: TimeSpan | TimeSpanConvertible): IAnimation;
/**
Creates a new {@link IAnimation} that animates the given <code>label</code> from its current layout linearly to the layout given
by the <code>targetLayoutParameter</code>.
This animation will as a side effect assign the <code>targetLayoutParameter</code> to the label at the {@link IAnimation#cleanUp end of the animation}.
@param graph The graph that contains the labels.
@param label The label to animate the parameter of.
@param targetLayoutParameter The target parameter of the label.
@param preferredDuration The preferred duration of the animation.
@returns A new instance of {@link IAnimation}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createLabelAnimation(IGraph%2CILabel%2CILabelModelParameter%2CTimeSpan) Online Documentation}*/
static createLabelAnimation(graph: IGraph, label: ILabel, targetLayoutParameter: ILabelModelParameter, preferredDuration: TimeSpan | TimeSpanConvertible): IAnimation;
/**
* Creates a new {@link IAnimation} instance that animates the current graph to the layout given by a {@link CopiedLayoutGraph}
* instance.
* The returned instance is suitable for morphing an {@link IGraph} to a layout computed by one of the layout
* algorithms.
*
* <b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
*
* While it is usually recommended to use {@link LayoutExecutor} to handle layout calculation and subsequent
* animation, the {@link IAnimation} produced by this method can be used manually, as well:
*
* <br><b>Using manually created layout animations</b>
* <pre>
* const adapter = new LayoutGraphAdapter(graph)
* const layoutGraph = adapter.createCopiedLayoutGraph()
* const layout = new OrthogonalLayout()
* layout.applyLayout(layoutGraph)
*
* const morphAnimation = IAnimation.createLayoutAnimation(
*   graph,
*   layoutGraph,
*   TimeSpan.fromSeconds(1)
* )
* </pre>
*
* @param graph the graph for which the layout should be animated
* @param layoutGraph the {@link CopiedLayoutGraph} that contains all target layout information
* @param preferredDuration the preferred duration of the animation
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createLayoutAnimation(IGraph%2CCopiedLayoutGraph%2CTimeSpan) Online Documentation}
*/
static createLayoutAnimation(graph: IGraph, layoutGraph: CopiedLayoutGraph, preferredDuration: TimeSpan | TimeSpanConvertible): IAnimation;
/**
Creates a new {@link IAnimation} that animates the given <code>node</code> from its current layout to the new given
<code>targetLayout</code>.
@param graph The graph the animated node belongs to.
@param node The node whose {@link INode#layout} to animate.
@param targetLayout The expected node layout after the animation.
@param preferredDuration The preferred duration of the animation.
@returns A new instance of {@link IAnimation}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createNodeAnimation(IGraph%2CINode%2CIRectangle%2CTimeSpan) Online Documentation}*/
static createNodeAnimation(graph: IGraph, node: INode, targetLayout: IRectangle | IRectangleConvertible, preferredDuration: TimeSpan | TimeSpanConvertible): IAnimation;
/**
Creates an {@link IAnimation} according to the composite design pattern that animates multiple animations in
parallel.
The animations can be synchronized to the longest running animation.
@param animations The animations to run in parallel.
@param [synchronized=true] Whether all child animations should be adjusted to have the same (maximum) {@link IAnimation#preferredDuration}. By default this is enabled.
@returns A new instance that is the composite of the given animations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createParallelAnimation(IEnumerable%3CIAnimation%3E%2Cboolean) Online Documentation}*/
static createParallelAnimation(animations: IEnumerable<IAnimation> | IEnumerableConvertible<IAnimation>, synchronized?: boolean): IAnimation;
/**
Creates a new instance of an {@link IAnimation} that animates the given {@link IMutablePoint} along a path.
@param path The path to animate the point along.
@param animationPoint The mutable point instance that will be manipulated by the animation.
@param preferredDuration The preferred duration of the animation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createPathAnimation(GeneralPath%2CIMutablePoint%2CTimeSpan) Online Documentation}*/
static createPathAnimation(path: GeneralPath, animationPoint: IMutablePoint, preferredDuration: TimeSpan | TimeSpanConvertible): IAnimation;
/**
Creates a new {@link IAnimation} that animates the given <code>port</code> from its current location to the new location given by
the <code>targetLocationParameter</code>.
This animation will as a side effect assign the <code>targetLocationParameter</code> to the port at the {@link IAnimation#cleanUp end of the animation}.
@param graph The graph that contains the port.
@param port The port to animate.
@param targetLocationParameter The parameter to linearly interpolate to.
@param preferredDuration The {@link IAnimation#preferredDuration preferred duration} of this animation.
@returns A new instance of {@link IAnimation}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createPortAnimation(IGraph%2CIPort%2CIPortLocationModelParameter%2CTimeSpan) Online Documentation}*/
static createPortAnimation(graph: IGraph, port: IPort, targetLocationParameter: IPortLocationModelParameter, preferredDuration: TimeSpan | TimeSpanConvertible): IAnimation;
/**
Creates an {@link IAnimation} that animates multiple animations in sequence.
Each animation in <code>animations</code> runs in sequence one after another. The duration of the sequential animation is the sum of
its child animation durations.
@param animations The animations to run in sequence.
@returns A new sequential animation from the given animations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createSequentialAnimation(IEnumerable%3CIAnimation%3E) Online Documentation}*/
static createSequentialAnimation(animations: IEnumerable<IAnimation> | IEnumerableConvertible<IAnimation>): IAnimation;
/**
Creates a new {@link IAnimation} for the given table that animates the table and its {@link IStripe stripes}.
Note that only the sizes of
<b>leaf stripes</b>, i.e. those without child stripes, are considered. The sizes of a stripe with children is implicitly determined by its
contents.
For more control of the animation, implement a custom {@link TableAnimation} and override method
{@link TableAnimation#createStripeAnimation}.
<b>Warning:</b> This method is not available unless the module <b>view-table</b> is loaded.
@param table The table to animate.
@param columnLayout The sizes of the <b>leaf</b> columns, in natural order.
@param rowLayout The sizes of the <b>leaf</b> rows, in natural order.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23createTableAnimation(ITable%2CArray%3Cnumber%3E%2CArray%3Cnumber%3E) Online Documentation}*/
static createTableAnimation(table: ITable, columnLayout: number[], rowLayout: number[]): IAnimation;
/**
Initializes the animation.
Call this method once before subsequent calls to {@link IAnimation#animate}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23initialize() Online Documentation}*/
abstract initialize(): void;
/**
Gets the preferred duration of the animation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IAnimation%23preferredDuration Online Documentation}*/
abstract readonly preferredDuration: TimeSpan;
static $class: Class;
private $_IAnimation: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IAnimation;
}
export interface ViewportAnimation extends YObject, IAnimation {
/**
@inheritdoc*/
animate(time: number): void;
} /**
Animates changing the viewport bounds and the zoom factor for a {@link CanvasComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation Online Documentation}*/
export class ViewportAnimation {
/**
Creates a new instance of {@link ViewportAnimation}.
@param canvas The canvas control.
@param targetBounds The target world bounds for the animation.
@param [preferredDuration=null] The preferred duration. 1 second if omitted.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation Online Documentation}*/
constructor(canvas: CanvasComponent, targetBounds: Rect | RectConvertible, preferredDuration?: TimeSpan | null | TimeSpanConvertible);
/**
Creates a new instance of {@link ViewportAnimation}.
@param options The parameters to pass.
@param options.canvas The canvas control.
@param options.targetBounds The target world bounds for the animation.
@param [options.preferredDuration=null] The preferred duration. 1 second if omitted.
@param [options.considerViewportLimiter] Whether to respect the {@link CanvasComponent#viewportLimiter} of the {@link CanvasComponent} whose {@link CanvasComponent#viewport} is animated. This option sets the {@link ViewportAnimation#considerViewportLimiter} property on the created object.
@param [options.maximumTargetZoom] The maximum zoom level to use after the animation. This option sets the {@link ViewportAnimation#maximumTargetZoom} property on the created object.
@param [options.targetViewMargins] The margins in view coordinate dimensions to use at the target zoom level. This option sets the {@link ViewportAnimation#targetViewMargins} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation Online Documentation}*/
constructor(options: {
canvas: CanvasComponent;
targetBounds: Rect | RectConvertible;
preferredDuration?: TimeSpan | null | TimeSpanConvertible;
considerViewportLimiter?: boolean;
maximumTargetZoom?: number;
targetViewMargins?: Insets | InsetsConvertible;
});
/**
Effectively applies the center point value.
@param focus
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23applyCenterPoint(Point) Online Documentation}*/
applyCenterPoint(focus: Point): void;
/**
Effectively applies the view point value.
@param focus
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23applyViewPoint(Point) Online Documentation}*/
applyViewPoint(focus: Point): void;
/**
Effectively applies the zoom value.
@param value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23applyZoom(number) Online Documentation}*/
applyZoom(value: number): void;
/**
Cancels this viewport animation so that subsequent calls to {@link ViewportAnimation#animate} or {@link ViewportAnimation#cleanUp}
won't affect the viewport anymore.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23cancel() Online Documentation}*/
cancel(): void;
/**
Cleans up after the animation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23cleanUp() Online Documentation}*/
cleanUp(): void;
/**
Initializes this animation.
This has to be called once before any calls to {@link ViewportAnimation#animate}. An instance of {@link ViewportAnimation}
can be reused after an animation by setting new {@link ViewportAnimation#targetBounds} or a new {@link ViewportAnimation#preferredDuration} and calling Initialize() again.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23initialize() Online Documentation}*/
initialize(): void;
/**
Gets or sets the target world bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23targetBounds Online Documentation}*/
targetBounds: Rect;
/**
Gets or sets whether to respect the {@link CanvasComponent#viewportLimiter} of the {@link CanvasComponent} whose {@link CanvasComponent#viewport} is animated.
Default is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23considerViewportLimiter Online Documentation}*/
considerViewportLimiter: boolean;
/**
Gets or sets the preferred duration of the animation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23preferredDuration Online Documentation}*/
preferredDuration: TimeSpan;
/**
Gets or sets the maximum zoom level to use after the animation.
The default is {@link Number#POSITIVE_INFINITY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23maximumTargetZoom Online Documentation}*/
maximumTargetZoom: number;
/**
Gets or sets the scroll bar visibility that will be used during the animation.
@default ScrollBarVisibility.NEVER
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23scrollBarVisibility Online Documentation}*/
scrollBarVisibility: ScrollBarVisibility;
/**
Gets or sets the margins in view coordinate dimensions to use at the target zoom level.
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportAnimation%23targetViewMargins Online Documentation}*/
targetViewMargins: Insets;
static $class: Class;
private $_ViewportAnimation: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ViewportAnimation;
}
/**
Enumeration used by the {@link CanvasComponent#horizontalScrollBarPolicy} and {@link CanvasComponent#verticalScrollBarPolicy} properties.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ScrollBarVisibility Online Documentation}*/
export enum ScrollBarVisibility {
/**
Never display the scroll bar, even if not all of the content is visible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ScrollBarVisibility%23NEVER Online Documentation}*/
NEVER,
/**
Display the scroll bar when needed.
This is the default behavior in most applications.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ScrollBarVisibility%23AS_NEEDED Online Documentation}*/
AS_NEEDED,
/**
Always display the scroll bar.
If it is not needed, i.e. if all content is already displaying, the scroll bar will remain visible but cannot be moved.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ScrollBarVisibility%23ALWAYS Online Documentation}*/
ALWAYS
}
/** Automatic conversion to {@link ScrollBarVisibility}. */
type ScrollBarVisibilityStringValues = "never" | "as-needed" | "always";
/**
Enumeration for the mouse buttons.
The enumeration can be used to create a bitwise combination of the types.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseButtons Online Documentation}*/
export enum MouseButtons {
/**
No mouse button.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseButtons%23NONE Online Documentation}*/
NONE,
/**
The left mouse button.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseButtons%23LEFT Online Documentation}*/
LEFT,
/**
The right mouse button.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseButtons%23RIGHT Online Documentation}*/
RIGHT,
/**
The middle mouse button.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseButtons%23MIDDLE Online Documentation}*/
MIDDLE,
/**
Extra mouse button 1.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseButtons%23X1 Online Documentation}*/
X1,
/**
Extra mouse button 2.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseButtons%23X2 Online Documentation}*/
X2
}
type MouseButtonsStringValues = "none" | "left" | "right" | "middle" | "x1" | "x2";
/** Automatic conversion to {@link MouseButtons}. */
type MouseButtonsConvertible = MouseButtonsStringValues | MouseButtonsStringValues[];
/**
Specifies constants that define the effect of turning the mouse wheel in a {@link CanvasComponent}.
If {@link MouseWheelBehaviors#ZOOM} and {@link MouseWheelBehaviors#SCROLL} are both set, scrolling will be the
default behavior. The control will zoom with the Control key is pressed. {@link MouseWheelBehaviors#ZOOM} and {@link MouseWheelBehaviors#SCROLL}
can be combined with {@link MouseWheelBehaviors#ONLY_WHEN_FOCUSED} to activate this behavior only when the control
has focus.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseWheelBehaviors Online Documentation}*/
export enum MouseWheelBehaviors {
/**
The mouse wheel has no effect.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseWheelBehaviors%23NONE Online Documentation}*/
NONE,
/**
The mouse wheel changes the zoom factor.
This constant can be combined with {@link MouseWheelBehaviors#SCROLL} to allow both at the same time with different
modifiers. In that case scrolling will be the default behavior. The control will zoom with the Control key is pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseWheelBehaviors%23ZOOM Online Documentation}*/
ZOOM,
/**
The mouse wheel scrolls the view.
Scrolling defaults to scrolling vertically. Pressing Shift while scrolling will scroll horizontally instead.
This constant can be combined with {@link MouseWheelBehaviors#ZOOM} to allow both at the same time with different
modifiers. In that case scrolling will be the default behavior. The control will zoom with the Control key is pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseWheelBehaviors%23SCROLL Online Documentation}*/
SCROLL,
/**
Can be combined with {@link MouseWheelBehaviors#ZOOM} or {@link MouseWheelBehaviors#SCROLL} to perform the
action only when the control has focus.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseWheelBehaviors%23ONLY_WHEN_FOCUSED Online Documentation}*/
ONLY_WHEN_FOCUSED
}
type MouseWheelBehaviorsStringValues = "none" | "zoom" | "scroll" | "only-when-focused";
/** Automatic conversion to {@link MouseWheelBehaviors}. */
type MouseWheelBehaviorsConvertible = MouseWheelBehaviorsStringValues | MouseWheelBehaviorsStringValues[];
export interface PrepareRenderContextEventArgs extends EventArgs {
}
/**
The event arguments for the event handler used by {@link CanvasComponent#raisePrepareRenderContextEvent}
This event argument can be used to modify the {@link PrepareRenderContextEventArgs#context} during {@link CanvasComponent#exportContent} and
{@link CanvasComponent#invalidate}
@see CanvasComponent#addPrepareRenderContextListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/PrepareRenderContextEventArgs Online Documentation}*/
export class PrepareRenderContextEventArgs {
/**
Initializes a new instance of the {@link PrepareRenderContextEventArgs} class.
@param context The initial context which can be modified by the event handlers using the {@link PrepareRenderContextEventArgs#context} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PrepareRenderContextEventArgs Online Documentation}*/
constructor(context: IRenderContext);
/**
Gets or sets the context.
@throws {Exception} <b>(ArgumentError)</b> If the argument is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PrepareRenderContextEventArgs%23context Online Documentation}*/
context: IRenderContext;
static $class: Class;
private $_PrepareRenderContextEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PrepareRenderContextEventArgs;
}
/**
Available size change detection modes for the property {@link CanvasComponent#sizeChangedDetection}.
Since there is no size changed event in HTML, {@link CanvasComponent} comes with its own methods to detect whether
the size of its {@link CanvasComponent#div} element has changed. This is needed to update its {@link CanvasComponent#size} accordingly.
@see CanvasComponent#sizeChangedDetection
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SizeChangedDetectionMode Online Documentation}*/
export enum SizeChangedDetectionMode {
/**
No size change detection.
Consequently, no {@link CanvasComponent#addSizeChangedListener SizeChanged} events are fired.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SizeChangedDetectionMode%23NONE Online Documentation}*/
NONE,
/**
Size change detection based on invisible sensor elements.
This is the preferred mode since it provides
{@link CanvasComponent#addSizeChangedListener SizeChanged}events in real time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SizeChangedDetectionMode%23SENSOR Online Documentation}*/
SENSOR,
/**
Timer based size change detection.
A timer periodically checks the current size with the previous one. Typically, this method is used as fall back in case
the {@link SizeChangedDetectionMode#SENSOR} mode is not available in a browser.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SizeChangedDetectionMode%23TIMER Online Documentation}*/
TIMER,
/**
This is the default mode which uses the preferred {@link SizeChangedDetectionMode#SENSOR} mode if possible,
otherwise the {@link SizeChangedDetectionMode#TIMER} mode is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SizeChangedDetectionMode%23AUTO Online Documentation}*/
AUTO
}
/** Automatic conversion to {@link SizeChangedDetectionMode}. */
type SizeChangedDetectionModeStringValues = "none" | "sensor" | "timer" | "auto";
export interface SizeChangedEventArgs extends EventArgs {
}
/**
This event occurs when the size of a control is changed.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SizeChangedEventArgs Online Documentation}*/
export class SizeChangedEventArgs {
/**
Gets or sets the element whose size was changed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SizeChangedEventArgs%23source Online Documentation}*/
source: HTMLElement;
/**
Gets or sets the old size of the control.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SizeChangedEventArgs%23oldSize Online Documentation}*/
oldSize: Size;
static $class: Class;
private $_SizeChangedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SizeChangedEventArgs;
}
export interface CanvasComponent extends YObject, ILookup {
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
The {@link CanvasComponent} can be used to efficiently display all kinds of data.
The control supports high performance zooming and panning of visual content and provides a high level view of the visual
tree. The control uses double precision world-coordinates to render items that are composed using a visual tree. In its
most simple form the scene graph is a tree-like structure of elements that will be rendered in depth-first-search
order.
The scene graph tree is made up of {@link ICanvasObject} elements. Multiple elements are grouped in {@link ICanvasObjectGroup}
elements. An {@link ICanvasObject} consists of a {@link ICanvasObject#userObject user object} and an {@link ICanvasObjectDescriptor} that is used
by the rendering engine to query interfaces that perform the actual creation of the
{@link Visual}s, hit testing, and bounds calculation.
A {@link CanvasComponent#projection} that is applied to the whole scene graph can be set to simulate different parallel projections of the
{@link CanvasComponent}'s content (e.g. isometric).
This component handles the following commands by default:
<ul>
<li>{@link ICommand#INCREASE_ZOOM}
<li>{@link ICommand#DECREASE_ZOOM}
<li>{@link ICommand#ZOOM}
<li>{@link ICommand#FIT_CONTENT}
<li>{@link ICommand#SCROLL_PAGE_DOWN}
<li>{@link ICommand#SCROLL_PAGE_UP}
<li>{@link ICommand#SCROLL_PAGE_LEFT}
<li>{@link ICommand#SCROLL_PAGE_RIGHT}
</ul>
The handlers for {@link ICommand#INCREASE_ZOOM} and {@link ICommand#DECREASE_ZOOM} use the specified command
parameter as the factor by which the {@link CanvasComponent#zoom} value is multiplied or divided, respectively. The parameter is not required. If it
is not present a default value is used. The handler for the {@link ICommand#ZOOM} command needs a parameter. The
parameter can either be a number which is interpreted as the new zoom level, or it can be a suitable object. Supported
objects are {@link Rect}, {@link Point} and {@link ILookup}. If a rectangle is specified as the
parameter for the {@link ICommand#ZOOM} command, the handler adjusts the {@link CanvasComponent#viewport} such that the rectangle tightly fits into
it. If a point is specified, the handler sets the {@link CanvasComponent#center} to that point. If the parameter implements {@link ILookup}, then
the handler tries to get a {@link IBoundsProvider} from the lookup and uses the bounds obtained from the bounds
provider as the rectangle to zoom to.
{@link CanvasComponent} is discussed in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/view_graphcontrol The Component for Displaying Graphs}.
@see IVisualCreator
@see IHitTestable
@see IVisibilityTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent Online Documentation}*/
export class CanvasComponent {
/**
Creates a new instance of {@link CanvasComponent} using the <code>div</code> element that is specified by the <code>selector</code>.
The canvas will be empty with no associated {@link CanvasComponent#inputMode}.
All child nodes of the <code>div</code> element are passed to the {@link CanvasComponent#overlayPanel}.
@param selector The CSS selector or id for an existing <code>div</code> element. All child nodes of this element are added to the {@link CanvasComponent#overlayPanel}
@throws {Exception} <b>(ArgumentError)</b> if the &lt;div> element specified by the <code>selector</code> is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23CanvasComponent(string) Online Documentation}*/
constructor(selector: string);
/**
Creates a new instance of {@link CanvasComponent} for the provided <code>div</code> element.
The canvas will be empty with no associated {@link CanvasComponent#inputMode}.
All child nodes of the <code>div</code> element are added to the {@link CanvasComponent#overlayPanel}.
@param div The existing <code>div</code> element to use for rendering the canvas into. All its child nodes are added to the {@link CanvasComponent#overlayPanel}
@throws {Exception} <b>(ArgumentError)</b> if the <code>div</code> element is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23CanvasComponent(HTMLDivElement) Online Documentation}*/
constructor(div: HTMLDivElement);
/**
Creates a new instance of {@link CanvasComponent} with default properties.
The canvas will be empty with no associated InputModes. This constructor creates a new {@link CanvasComponent#div} element that needs to be added
to the DOM manually.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23CanvasComponent() Online Documentation}*/
constructor();
/**
Cleans up the div element by removing any connection from the element to the {@link CanvasComponent} instance, as
well as any associated element that was created during the lifetime of the {@link CanvasComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23cleanUp() Online Documentation}*/
cleanUp(): void;
/**
Helper method that ensures that the view port limit as returned by the {@link CanvasComponent#viewportLimiter} are obeyed.
This method will {@link CanvasComponent#zoomTo} the limited bounds if necessary.
@see CanvasComponent#viewportLimiter
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23coerceViewportLimits() Online Documentation}*/
coerceViewportLimits(): void;
/**
Compares two {@link ICanvasObject} instances that are live in this canvas.
The comparison will yield values greater than zero if and only if the second object is rendered after the first one.
@param canvasObject1 the first object to compare
@param canvasObject2 the second object to compare
@returns 0 if canvasObject1 == canvasObject2, >0 if canvasObject1 is painted before canvasObject2, &lt;0 if canvasObject1 is painted
after canvasObject2
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23compareRenderOrder(ICanvasObject%2CICanvasObject) Online Documentation}*/
compareRenderOrder(canvasObject1: ICanvasObject, canvasObject2: ICanvasObject): number;
/**
Replaces the <code>cursor</code> to account for {@link CanvasComponent#projection}.
@param cursor The cursor to adjust, can be <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23compensateCursorForProjection(Cursor) Online Documentation}*/
compensateCursorForProjection(cursor: Cursor | null | CursorConvertible): Cursor | null;
/**
Factory method for the BackgroundGroup property.
This method will be called upon first access to the {@link CanvasComponent#backgroundGroup} property.
@returns a new instance of ICanvasObjectGroup
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23createBackgroundGroup() Online Documentation}*/
createBackgroundGroup(): ICanvasObjectGroup;
/**
Factory method for the ContentGroup property.
This method will be called upon first access to the {@link CanvasComponent#contentGroup} property.
@returns a new instance of ICanvasObjectGroup
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23createContentGroup() Online Documentation}*/
createContentGroup(): ICanvasObjectGroup;
/**
Factory method for the {@link CanvasComponent#focusGroup} property.
This method will be called upon first access to the {@link CanvasComponent#focusGroup} property.
@returns a new instance of ICanvasObjectGroup
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23createFocusGroup() Online Documentation}*/
createFocusGroup(): ICanvasObjectGroup;
/**
Factory method for the HighlightGroup property.
This method will be called upon first access to the {@link CanvasComponent#highlightGroup} property.
@returns a new instance of ICanvasObjectGroup
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23createHighlightGroup() Online Documentation}*/
createHighlightGroup(): ICanvasObjectGroup;
/**
Factory method for the {@link CanvasComponent#inputModeContext} property.
This method will be called upon first access to the {@link CanvasComponent#inputModeContext} property.
@returns a new instance of {@link IInputModeContext} that has this instance set as its {@link ICanvasContext#canvasComponent} and uses this instance's {@link CanvasComponent#inputModeContextLookup}
method to satisfy requests.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23createInputModeContext() Online Documentation}*/
createInputModeContext(): IInputModeContext;
/**
Factory method for the InputModeGroup property.
This method will be called upon first access to the {@link CanvasComponent#inputModeGroup} property.
@returns a new instance of ICanvasObjectGroup
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23createInputModeGroup() Online Documentation}*/
createInputModeGroup(): ICanvasObjectGroup;
/**
Creates an appropriate {@link IRenderContext render context} that can be used to {@link IVisualCreator#createVisual create visuals} using {@link IVisualCreator}
implementations.
This method is a convenience method to obtain a context for special visual creator implementations and normally needs
not be used by application developers.
@returns A new context instance that is bound to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23createRenderContext() Online Documentation}*/
createRenderContext(): IRenderContext;
/**
Factory method for the SelectionGroup property.
This method will be called upon first access to the {@link CanvasComponent#selectionGroup} property.
@returns a new instance of ICanvasObjectGroup
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23createSelectionGroup() Online Documentation}*/
createSelectionGroup(): ICanvasObjectGroup;
/**
Ensures that the provided bounds in world coordinates are visible in this control by adjusting the viewport
correspondingly.
@param bounds The bounds to make visible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ensureVisible(Rect) Online Documentation}*/
ensureVisible(bounds: Rect): void;
/**
Exports the graphical content for this {@link CanvasComponent}.
This method will create the visual content using the world coordinate system.
Note that for creating standalone SVG exports, the {@link SvgExport} should be used instead.
@param context The context to use.
@returns A SVG element that represents the contents of this control without the viewport transform or the {@link CanvasComponent#projection}.
@see SvgExport#createRenderContext
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23exportContent(IRenderContext) Online Documentation}*/
exportContent(context: IRenderContext): Element;
/**
Fires the size changed event.
@param oldSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23fireSizeChanged(Size) Online Documentation}*/
fireSizeChanged(oldSize: Size): void;
/**
Adjusts the view port to fully encompass the {@link CanvasComponent#contentRect}.
This is usually used to make sure a large graph fits into the viewport. The {@link CanvasComponent#limitFitContentZoom} property controls the behavior for small
graphs. If {@link CanvasComponent#limitFitContentZoom} is <code>true</code>, the maximum zoom level is 1; setting the property to <code>false</code> will zoom in far enough so that even
small graphs fit the viewport (limited to {@link CanvasComponent#maximumZoom}, though).
The {@link CanvasComponent#viewportLimiter} is taken into account.
If a {@link CanvasComponent#projection} is set, the bounding rectangle of the {@link CanvasComponent#contentRect} is used and and {@link CanvasComponent#viewportLimiter} is ignored.
@see CanvasComponent#updateContentRect
@see CanvasComponent#zoom
@see CanvasComponent#zoomTo
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23fitContent() Online Documentation}*/
fitContent(): void;
/**
Focuses the {@link CanvasComponent#div} element that is backing this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23focus() Online Documentation}*/
focus(): void;
/**
Calculates the bounds for a given canvas object in the scene graph.
This method queries the descriptor for the {@link IBoundsProvider} for the user object and returns the result.
@param canvasObject the canvas object to query the bounds from
@returns the non-<code>null</code> bounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23getBounds(ICanvasObject) Online Documentation}*/
getBounds(canvasObject: ICanvasObject): Rect;
/**
Returns the top most canvas object instance that is hit at the given coordinate set.
This will return the canvas object that is painted last at the given position.
@param location the coordinates of the query in the world coordinate system
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23getCanvasObject(Point) Online Documentation}*/
getCanvasObject(location: Point): ICanvasObject | null;
/**
Returns a list of all canvas objects in hit order at the given world coordinate location.
The order of the elements in the list is the natural hit test order, i.e. elements painted on top of others will be
reported first in the list.
@param location the coordinates of the query in the world coordinate system
@returns an enumerable of canvas object that are hit in reverse painting order
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23getCanvasObjects(Point) Online Documentation}*/
getCanvasObjects(location: Point): IEnumerable<ICanvasObject>;
/**
Enumerates over all possible {@link ICanvasObject} instances in the tree below the given group.
@param [root=null] The group at which the enumeration should yield all the children recursively. If
omitted/<code>null</code>, this will iterate over all elements in the tree.
@returns An enumerable for all {@link ICanvasObject}s in the tree below the given group.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23getCanvasObjects(ICanvasObjectGroup) Online Documentation}*/
getCanvasObjects(root?: ICanvasObjectGroup | null): IEnumerable<ICanvasObject>;
/**
Gets the {@link CanvasComponent} instance that is associated with the given DOM element, or null.
@param element
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23getComponent(Element) Online Documentation}*/
static getComponent(element: Element): CanvasComponent | null;
/**
Gets the {@link Visual} that is currently visualizing the given {@link ICanvasObject}.
Note that depending on the current state of the viewport and the control no such instance may be available and instead
this method will yield <code>null</code>. However if a visual is currently being displayed for the given canvas object, it will be
returned. This method should be used with care. Manipulation of the given instance should normally be done through the
corresponding {@link IVisualCreator} that created the visual in the first place. This method rather serves as a
utility method for UI testing frameworks and similar use cases.
@param canvasObject The canvas object.
@returns The {@link Visual} that is currently used by the <code>canvasObject</code>. This may be <code>null</code> if the given <code>canvasObject</code> is
currently not visible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23getVisual(ICanvasObject) Online Documentation}*/
getVisual(canvasObject: ICanvasObject): Visual | null;
/**
Retrieves the {@link IVisualCreator} for a given {@link ICanvasObject}.
@param canvasObject the canvas object to query the visual creator implementation from
@returns an instance of the visual creator interface
@see ICanvasObjectDescriptor#getVisualCreator
@see VoidVisualCreator#INSTANCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23getVisualCreator(ICanvasObject) Online Documentation}*/
getVisualCreator(canvasObject: ICanvasObject): IVisualCreator;
/**
Assures that the {@link CanvasComponent#contentRect content rectangle} encompasses the given rectangle.
Note that this will not change the view port, it will only change the scroll bars if necessary.
@param rectangle the rectangle that should be included in the content rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23growContentRect(Rect) Online Documentation}*/
growContentRect(rectangle: Rect): void;
/**
Enumerates all hit elements in the canvas below the given group that are accepted by a given filter using a specific {@link ICanvasContext}
as the argument to the {@link IHitTestable#isHit} method.
Hit testing is performed using the {@link ICanvasObjectDescriptor#getHitTestable} instance returned for each visible
{@link ICanvasObject} in the current scene graph. The enumeration is performed lazily.
@param context The context instance to pass to {@link IHitTestable#isHit}.
@param location the coordinates to perform the hit test at
@param [root=null] the root of the scene graph to use
@param [filter=null] The predicate that decides whether a given canvas object should be considered for testing at all or <code>null</code>.
@returns a live enumeration of the elements that are hit
@see IHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23hitElementsAt(IInputModeContext%2CPoint%2CICanvasObjectGroup%2Cfunction(ICanvasObject)%3Aboolean) Online Documentation}*/
hitElementsAt(context: IInputModeContext, location: Point | PointConvertible, root?: ICanvasObjectGroup | null, filter?: ((obj: ICanvasObject) => boolean) | null): IEnumerable<ICanvasObject>;
/**
Enumerates all hit elements in the canvas below the given group that are accepted by a given filter using a specific {@link ICanvasContext}
as the argument to the {@link IHitTestable#isHit} method.
Hit testing is performed using the {@link ICanvasObjectDescriptor#getHitTestable} instance returned for each visible
{@link ICanvasObject} in the current scene graph. The enumeration is performed lazily.
@param options The parameters to pass.
@param options.context The context instance to pass to {@link IHitTestable#isHit}.
@param options.location the coordinates to perform the hit test at
@param [options.root=null] the root of the scene graph to use
@param [options.filter=null] The predicate that decides whether a given canvas object should be considered for testing at all or <code>null</code>.
@returns a live enumeration of the elements that are hit
@see IHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23hitElementsAt(IInputModeContext%2CPoint%2CICanvasObjectGroup%2Cfunction(ICanvasObject)%3Aboolean) Online Documentation}*/
hitElementsAt(options: {
context: IInputModeContext;
location: Point | PointConvertible;
root?: ICanvasObjectGroup | null;
filter?: ((obj: ICanvasObject) => boolean) | null;
}): IEnumerable<ICanvasObject>;
/**
Enumerates all hit elements in the canvas below the given group that are accepted by a given filter.
Hit testing is performed using the {@link ICanvasObjectDescriptor#getHitTestable} instance returned for each visible
{@link ICanvasObject} in the current scene graph. The enumeration is performed lazily.
@param location the coordinates to perform the hit test at
@param [root=null] the root of the scene graph to use
@param [filter=null] The predicate that decides whether a given canvas object should be considered for testing at all or <code>null</code>.
@returns a live enumeration of the elements that are hit
@see IHitTestable
@see CanvasComponent#hitTestRadius
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23hitElementsAt(Point%2CICanvasObjectGroup%2Cfunction(ICanvasObject)%3Aboolean) Online Documentation}*/
hitElementsAt(location: Point | PointConvertible, root?: ICanvasObjectGroup | null, filter?: ((obj: ICanvasObject) => boolean) | null): IEnumerable<ICanvasObject>;
/**
Enumerates all hit elements in the canvas below the given group that are accepted by a given filter.
Hit testing is performed using the {@link ICanvasObjectDescriptor#getHitTestable} instance returned for each visible
{@link ICanvasObject} in the current scene graph. The enumeration is performed lazily.
@param options The parameters to pass.
@param options.location the coordinates to perform the hit test at
@param [options.root=null] the root of the scene graph to use
@param [options.filter=null] The predicate that decides whether a given canvas object should be considered for testing at all or <code>null</code>.
@returns a live enumeration of the elements that are hit
@see IHitTestable
@see CanvasComponent#hitTestRadius
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23hitElementsAt(Point%2CICanvasObjectGroup%2Cfunction(ICanvasObject)%3Aboolean) Online Documentation}*/
hitElementsAt(options: {
location: Point | PointConvertible;
root?: ICanvasObjectGroup | null;
filter?: ((obj: ICanvasObject) => boolean) | null;
}): IEnumerable<ICanvasObject>;
/**
Used by the default implementation of {@link CanvasComponent#inputModeContext} to resolve {@link ILookup#lookup} calls.
@param type The Type to query
@returns The result of the query.
@see IInputModeContext
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23inputModeContextLookup(Class) Online Documentation}*/
inputModeContextLookup(type: Class): any | null;
/**
Converts intermediate coordinates to view coordinates expressed in the control's coordinate system.
Essentially, this applies the set {@link CanvasComponent#projection}. If no projection is set, this method has no effect.
@param intermediatePoint the coordinates in the view coordinate system
@returns The coordinates in pixels relative to the controls upper left corner in the view coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23intermediateToViewCoordinates(Point) Online Documentation}*/
intermediateToViewCoordinates(intermediatePoint: Point | PointConvertible): Point;
/**
Converts intermediate coordinates to world coordinates.
If no {@link CanvasComponent#projection} is set, this method produces the same result as {@link CanvasComponent#toWorldCoordinates}.
@param intermediatePoint Coordinates expressed in the intermediate coordinate system.
@returns The coordinates expressed in the world coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23intermediateToWorldCoordinates(Point) Online Documentation}*/
intermediateToWorldCoordinates(intermediatePoint: Point | PointConvertible): Point;
/**
Invalidates this instance and marks it as in need for {@link CanvasComponent#updateVisual an update}.
Calling this method is fast and will only mark the component as dirty for a future, asynchronously executed call to
{@link CanvasComponent#updateVisual}.
Alternatively, {@link CanvasComponent#updateVisualAsync} can be used which returns a promise that is resolved once
the component has been {@link CanvasComponent#updateVisual updated}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23invalidate() Online Documentation}*/
invalidate(): void;
/**
Calculates the hit tests a given canvas object in the scene graph.
This method queries the descriptor for the {@link IHitTestable} for the user object and returns the result of the
hit test query. If there is no <code>IHitTestable</code> returned by the descriptor this method returns <code>false</code>.
@param canvasObject the canvas object to query the bounds from
@param location the coordinates of the query in the world coordinate system
@returns whether the canvas object is hit at the given coordinates
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23isHit(ICanvasObject%2CPoint) Online Documentation}*/
isHit(canvasObject: ICanvasObject, location: Point): boolean;
/**
Calls <code>evt.preventDefault()</code> for each <code>contextMenu</code> event that occurs on this instance.
By overriding this method, the browser's default behavior for the <code>contextMenu</code> event can be enabled.
@param evt The event.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23maybePreventContextMenuDefault(Event) Online Documentation}*/
maybePreventContextMenuDefault(evt: Event): void;
/**
Calls <code>evt.preventDefault()</code> if {@link CanvasComponent#captureAllPointerInput} is enabled.
This method is queried for each mouse, touch or pointer event that occurs on this instance. By overriding this method,
the browser's default behavior can effectively be controlled for every single event.
@param evt The event.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23maybePreventPointerDefault(Event) Online Documentation}*/
maybePreventPointerDefault(evt: Event): void;
/**
This method will be called when the mouse wheel was turned and if the {@link CanvasComponent#mouseWheelBehavior} property is set to
{@link MouseWheelBehaviors#SCROLL}.
This method will scroll the view in vertical direction. If the Shift key modifier has been pressed this method will
scroll in horizontal direction.
@param evt the event describing the action
@see CanvasComponent#mouseWheelBehavior
@see CanvasComponent#mouseWheelScrollFactor
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23mouseWheelScroll(MouseEventArgs) Online Documentation}*/
mouseWheelScroll(evt: MouseEventArgs): void;
/**
This method will be called when the mouse wheel was turned and if the {@link CanvasComponent#mouseWheelBehavior} property is set to
{@link MouseWheelBehaviors#ZOOM}.
This method will adjust the current zoom level. If the Control key modifier has not been pressed this method will keep
the world coordinates at the current mouse position, i.e. the zoom will not necessarily be into the center of the
canvas.
@param evt the event describing the action
@see CanvasComponent#mouseWheelBehavior
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23mouseWheelZoom(MouseEventArgs) Online Documentation}*/
mouseWheelZoom(evt: MouseEventArgs): void;
/**
Raises the {@link CanvasComponent#addPrepareRenderContextListener PrepareRenderContext}.
@param evt The event arguments to raise.
@see CanvasComponent#addPrepareRenderContextListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23raisePrepareRenderContextEvent(PrepareRenderContextEventArgs) Online Documentation}*/
raisePrepareRenderContextEvent(evt: PrepareRenderContextEventArgs): void;
/**
Raises the {@link CanvasComponent#addUpdatedVisualListener UpdatedVisual}.
@see CanvasComponent#addUpdatedVisualListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23raiseUpdatedVisualEvent() Online Documentation}*/
raiseUpdatedVisualEvent(): void;
/**
Raises the {@link CanvasComponent#addUpdatingVisualListener UpdatingVisual}.
@see CanvasComponent#addUpdatingVisualListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23raiseUpdatingVisualEvent() Online Documentation}*/
raiseUpdatingVisualEvent(): void;
/**
Adds a callback to the invocation chain and triggers the chain's execution.
Therefore, the given callback will be executed in sequence with previously added actions, e.g. the
{@link CanvasComponent}'s invalidation.
@param callback The function that should be executed.
@param args The arguments with which the function should be called.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23schedule(Function%2CObject) Online Documentation}*/
schedule(callback: Function, ...args: any[]): void;
/**
Sets the content rectangle.
The content rectangle is the space in world coordinates that the user should at least be able to scroll to. I.e. if the
area currently visible in the control does not encompass the content rect, the scrollbars will be made visible, unless
they are disabled.
@param x
@param y
@param w
@param h
@see CanvasComponent#contentRect
@see CanvasComponent#horizontalScrollBarPolicy
@see CanvasComponent#verticalScrollBarPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23setContentRect(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
setContentRect(x: number, y: number, w: number, h: number): void;
/**
Stops the mouse capture and returns to normal event capturing.
When {@link CanvasComponent#mouseCapture} is set to <code>true</code> and a mouse down event is received, the control starts capturing all mouse events by registering
handlers on the document element. Mouse capturing is normally stopped when the mouse button is released. Mouse capture
can be manually stopped by calling this function.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23stopMouseCapture() Online Documentation}*/
stopMouseCapture(): void;
/**
Converts view coordinates to the <a href="https://developer.mozilla.org/en-US/docs/Web/CSS/CSSOM_View/Coordinate_systems">html page coordinates</a>.
@param viewLocation The view coordinates to convert.
@returns The coordinates in the html document's coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23toPageFromView(Point) Online Documentation}*/
toPageFromView(viewLocation: Point | PointConvertible): Point;
/**
Converts world coordinates to view coordinates expressed in the control's coordinate system.
@param worldPoint Coordinates in the world coordinate system.
@returns The coordinates in pixels relative to the control's upper left corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23toViewCoordinates(Point) Online Documentation}*/
toViewCoordinates(worldPoint: Point | PointConvertible): Point;
/**
Converts <a href="https://developer.mozilla.org/en-US/docs/Web/CSS/CSSOM_View/Coordinate_systems">html page coordinates</a> to view coordinates.
@param pageLocation The html page coordinates to convert.
@returns The view coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23toViewFromPage(Point) Online Documentation}*/
toViewFromPage(pageLocation: Point | PointConvertible): Point;
/**
Converts coordinates expressed in the component's coordinate system to world coordinates.
@param viewPoint The coordinates in pixels relative to the component's upper left corner.
@returns The coordinates expressed in the world coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23toWorldCoordinates(Point) Online Documentation}*/
toWorldCoordinates(viewPoint: Point | PointConvertible): Point;
/**
Converts <a href="https://developer.mozilla.org/en-US/docs/Web/CSS/CSSOM_View/Coordinate_systems">html page coordinates</a> to world coordinates.
@param pageLocation The html page coordinates to convert.
@returns The world coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23toWorldFromPage(Point) Online Documentation}*/
toWorldFromPage(pageLocation: Point | PointConvertible): Point;
/**
Updates the {@link CanvasComponent#contentRect} to encompass the bounds by all elements in the current scene graph plus the given margins.
This method will traverse all {@link ICanvasObject#visible visible} elements in the scene graph and query their {@link ICanvasObjectDescriptor#getBoundsProvider bounds} The resulting bounds will be
set to the content rectangle plus the provided <code>margins</code>.
@param [margins=null]
@param [group=null]
@see IBoundsProvider
@see CanvasComponent#contentRect
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23updateContentRect(Insets%2CICanvasObjectGroup) Online Documentation}*/
updateContentRect(margins?: Insets | null | InsetsConvertible, group?: ICanvasObjectGroup | null): void;
/**
Updates the {@link CanvasComponent#contentRect} to encompass the bounds by all elements in the current scene graph plus the given margins.
This method will traverse all {@link ICanvasObject#visible visible} elements in the scene graph and query their {@link ICanvasObjectDescriptor#getBoundsProvider bounds} The resulting bounds will be
set to the content rectangle plus the provided <code>margins</code>.
@param options The parameters to pass.
@param [options.margins=null]
@param [options.group=null]
@see IBoundsProvider
@see CanvasComponent#contentRect
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23updateContentRect(Insets%2CICanvasObjectGroup) Online Documentation}*/
updateContentRect(options: {
margins?: Insets | null | InsetsConvertible;
group?: ICanvasObjectGroup | null;
}): void;
/**
Updates the visual tree that displays the contents of this control.
Calling this method will synchronously update the visual tree and the SVG DOM and Canvas rendering.
This method will determine the {@link ICanvasObject#dirty} canvas objects using the {@link ICanvasObjectDescriptor#isDirty} method and will {@link IVisualCreator#createVisual create}
or {@link IVisualCreator#updateVisual update} the visuals that make up the visual tree.
Note that most of the time this method does not need to be called by client code. Instead calling {@link CanvasComponent#invalidate}
will ultimately trigger the execution of this method. However invalidation calls will be coalesced and the actual
execution of the update will be delayed until the next event dispatch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23updateVisual() Online Documentation}*/
updateVisual(): void;
/**
Invalidates this instance and marks it as in need for {@link CanvasComponent#updateVisual an update}.
This method is similar in functionality to {@link CanvasComponent#invalidate}, but it will yield a Promise that
resolves once {@link CanvasComponent#updateVisual} was called asynchronously.
@returns A Promise that will resolve once the component has been {@link CanvasComponent#updateVisual updated}
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23updateVisualAsync() Online Documentation}*/
updateVisualAsync(): Promise<any>;
/**
Converts view coordinates to intermediate coordinates, effectively canceling any {@link CanvasComponent#projection} if set.
If no {@link CanvasComponent#projection} is set, this method has no effect
@param viewPoint the coordinates in the view coordinate system
@returns The coordinates in pixels relative to the controls upper left corner in the intermediate coordinate system
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23viewToIntermediateCoordinates(Point) Online Documentation}*/
viewToIntermediateCoordinates(viewPoint: Point | PointConvertible): Point;
/**
Converts world coordinates to the component's coordinate system before the {@link CanvasComponent#projection} is applied.
If no {@link CanvasComponent#projection} is set, this method produces the same result as {@link CanvasComponent#toViewCoordinates}.
@param worldPoint the coordinates in the world coordinate system
@returns Coordinates expressed in pixels relative to the controls upper left corner in the intermediate coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23worldToIntermediateCoordinates(Point) Online Documentation}*/
worldToIntermediateCoordinates(worldPoint: Point | PointConvertible): Point;
/**
* Sets the zoom level and viewport {@link CanvasComponent#center} to the given values.
* This method effectively sets both the {@link CanvasComponent#center} and {@link CanvasComponent#zoom} properties.
*
* <br><b>Change the viewport to a new center point with a new zoom level</b>
* <pre>
* graphComponent.zoomTo(new Point(42, 42), 2)
* </pre>
*
* @param center The new center of the viewport in world coordinates.
* @param zoom The new zoom level.
* @see CanvasComponent#zoomTo
* @see CanvasComponent#zoomToAnimated
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23zoomTo(Point%2Cnumber) Online Documentation}
*/
zoomTo(center: Point, zoom: number): void;
/**
* Sets the zoom level and viewport center so that the given rectangle in world coordinates fits the viewport.
* <br><b>Change the viewport so that a given rectangle fits exactly</b>
* <pre>
* graphComponent.zoomTo(new Rect(50, 30, 120, 75))
* </pre>
*
* @param bounds The coordinates of the rectangle to zoom to.
* @see CanvasComponent#zoomTo
* @see CanvasComponent#zoomToAnimated
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23zoomTo(Rect) Online Documentation}
*/
zoomTo(bounds: Rect): void;
/**
* Transitions to a new zoom level and viewport center in an animated fashion.
* <br><b>Animate the viewport to a new center point with a new zoom level</b>
* <pre>
* await graphComponent.zoomToAnimated(new Point(42, 42), 2)
* // Subsequent code runs after the animation completes
* </pre>
*
* @param center The new center of the viewport in world coordinates.
* @param zoom The new zoom level.
* @returns A Promise that will resolve once the animation is finished.
* @see CanvasComponent#zoomToAnimated
* @see CanvasComponent#zoomTo
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23zoomToAnimated(Point%2Cnumber) Online Documentation}
*/
zoomToAnimated(center: Point | PointConvertible, zoom: number): Promise<any>;
/**
* Transitions the viewport in an animated fashion to a new zoom level and center so that the given rectangle in world
* coordinates fits the viewport.
* <br><b>Animate the viewport so that a given rectangle fits exactly</b>
* <pre>
* await graphComponent.zoomToAnimated(new Rect(50, 30, 120, 75))
* // Subsequent code runs after the animation completes
* </pre>
*
* @param bounds The coordinates of the rectangle to zoom to.
* @returns A Promise that will resolve once the animation is finished.
* @see CanvasComponent#zoomTo
* @see CanvasComponent#zoomToAnimated
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23zoomToAnimated(Rect) Online Documentation}
*/
zoomToAnimated(bounds: Rect | RectConvertible): Promise<any>;
/**
Gets the {@link CanvasComponent#dropTarget} associated with this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23dropTarget Online Documentation}*/
readonly dropTarget: DropTarget;
/**
Gets the <code>div</code> element that represents this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23div Online Documentation}*/
readonly div: HTMLDivElement;
/**
Gets or sets a value indicating whether all keyboard input is captured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23captureAllKeyboardInput Online Documentation}*/
captureAllKeyboardInput: boolean;
/**
Gets or sets a value indicating whether all pointer input is captured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23captureAllPointerInput Online Documentation}*/
captureAllPointerInput: boolean;
/**
Gets or sets a value that specifies whether this control receives mouse input after the mouse is dragged out of its
bounding area.
If enabled, this control receives mouse events when the mouse is moved out of its bounding area while a mouse button is
pressed, until the button is release (the mouse up event is the last event this control listens to).
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23mouseCapture Online Documentation}*/
mouseCapture: boolean;
/**
Gets a value indicating whether this instance currently has mouse capture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23hasMouseCapture Online Documentation}*/
readonly hasMouseCapture: boolean;
/**
Gets a value indicating whether this {@link CanvasComponent} is currently focused.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23focused Online Documentation}*/
readonly focused: boolean;
/**
Gets a value indicating whether the UI is right to left.
The value is fetched and cached on first access and the cache gets invalidated when the size of the component has
changed. This is to avoid costly queries into the CSS engine.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23rightToLeft Online Documentation}*/
readonly rightToLeft: boolean;
/**
Gets or sets a value indicating whether to prevent scrolling if the element gets focused.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23preventFocusScrolling Online Documentation}*/
preventFocusScrolling: boolean;
/**
Gets or sets the scaling factor for HTML5 canvas and WebGL rendering.
Default value is 1.0. Adjust this value to achieve crisp rendering on high DPI devices.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23devicePixelRatio Online Documentation}*/
devicePixelRatio: number;
/**
Gets the control's {@link CanvasComponent#svgDefsManager}
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23svgDefsManager Online Documentation}*/
readonly svgDefsManager: SvgDefsManager;
/**
Gets or sets whether to add a 'glass pane' overlay to the {@link CanvasComponent} that acts as the source for all
mouse and touch events.
The glass pane can be used as a workaround for bogus mouse and touch event handling: If the touch down or mouse down
event occurs on an element that is removed from the DOM while the pointer is down, the up event may not bubble up to the
{@link CanvasComponent} anymore, which leaves {@link IInputMode input modes} in a bad state.
<b>Warning:</b>
Be aware that all direct touch and mouse interaction with the SVG DOM is disabled if the glass pane is enabled, so this
should be used with care.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23useGlassPane Online Documentation}*/
useGlassPane: boolean;
/**
Gets the aria-live region of the {@link CanvasComponent}.
Elements appended to this region are read by screen readers.
The live region is initialized with <code>aria-live="polite"</code> and <code>aria-atomic="false"</code> and added to the DOM upon first access.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ariaLiveRegion Online Documentation}*/
readonly ariaLiveRegion: HTMLDivElement;
/**
Gets or sets the {@link ViewportLimiter} instance that can be used to limit the explorable region.
By default there are no limits set on the explorable region.
When a {@link CanvasComponent#projection} is set, the {@link CanvasComponent#viewportLimiter} is ignored.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23viewportLimiter Online Documentation}*/
viewportLimiter: ViewportLimiter;
/**
* Gets or sets the action to perform when turning the mouse wheel.
* Valid values for this property are as follows:
*
* <ul>
* <li><b>{@link MouseWheelBehaviors#NONE}</b> – No action is being taken when turning the mouse wheel.</li>
* <li><b>{@link MouseWheelBehaviors#ZOOM}</b> – The zoom level of the viewport is adjusted when turning the mouse wheel. This is the default.</li>
* <li><b>{@link MouseWheelBehaviors#SCROLL}</b> – The viewport is scrolled vertically when turning the mouse wheel, and horizontally when turning the mouse wheel while
* pressing the Shift key.</li>
* <li><b>{@link MouseWheelBehaviors#SCROLL}, {@link MouseWheelBehaviors#ZOOM}</b> – Both {@link MouseWheelBehaviors#SCROLL} and {@link MouseWheelBehaviors#ZOOM} can be combined to allow both
* scrolling and zooming. The default action without any modifiers pressed is scrolling vertically. Pressing Shift while
* turning the mouse wheel will scroll horizontally. Pressing Control while turning the mouse wheel will zoom the
* viewport.</li>
* <li>Any of the above options, except for {@link MouseWheelBehaviors#NONE} can be combined with {@link MouseWheelBehaviors#ONLY_WHEN_FOCUSED}
* to allow the interaction only when the {@link CanvasComponent} is focused.</li>
* </ul>
* <br><b>Changing what turning the mouse wheel does</b>
* <pre>
* // The mouse wheel should be used for scrolling.
* graphComponent.mouseWheelBehavior = MouseWheelBehaviors.SCROLL
* // The mouse wheel can be used either for scrolling or zooming, depending on the modifiers
* // but only when the control is focused.
* graphComponent.mouseWheelBehavior =
*   MouseWheelBehaviors.SCROLL | MouseWheelBehaviors.ZOOM | MouseWheelBehaviors.ONLY_WHEN_FOCUSED
* </pre>
*
* @default MouseWheelBehaviors.ZOOM
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23mouseWheelBehavior Online Documentation}
*/
mouseWheelBehavior: MouseWheelBehaviors;
/**
* Gets or sets an event recognizer that determines whether zooming with the mouse wheel should zoom to the center of the
* view instead of the mouse location.
* The default is {@link KeyEventRecognizers#SHIFT_IS_DOWN}.
*
* <br><b>Changing the recognizer to use Shift to zoom to the viewport center</b>
* <pre>
* graphComponent.centerZoomEventRecognizer = KeyEventRecognizers.SHIFT_IS_DOWN
* </pre>
*
* <br><b>Always zoom to the mouse location</b>
* <pre>
* graphComponent.centerZoomEventRecognizer = EventRecognizers.NEVER
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23centerZoomEventRecognizer Online Documentation}
*/
centerZoomEventRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the visibility policy for the vertical scrollbar.
Scrollbars don't need to be displayed in order to move the viewport. This can be achieved programmatically or using
special IInputMode instances. The default is {@link ScrollBarVisibility#AS_NEEDED}.
@see MoveViewportInputMode
@see CanvasComponent#horizontalScrollBarPolicy
@default ScrollBarVisibility.AS_NEEDED
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23verticalScrollBarPolicy Online Documentation}*/
verticalScrollBarPolicy: ScrollBarVisibility;
/**
Gets or sets the visibility policy for the horizontal scrollbar.
Scrollbars don't need to be displayed in order to move the viewport. This can be achieved programmatically or using
special IInputMode instances. The default is {@link ScrollBarVisibility#AS_NEEDED}.
@see MoveViewportInputMode
@see CanvasComponent#verticalScrollBarPolicy
@default ScrollBarVisibility.AS_NEEDED
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23horizontalScrollBarPolicy Online Documentation}*/
horizontalScrollBarPolicy: ScrollBarVisibility;
/**
Gets or sets a value indicating whether {@link ICommand}s for scrolling like for example
{@link ICommand#SCROLL_PAGE_DOWN}, the scrolling should be performed in animated fashion.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23animateScrollCommands Online Documentation}*/
animateScrollCommands: boolean;
/**
Gets and sets the Rectangle in world coordinates that holds the contents.
This influences the display of the scroll bars. If the content rectangle is not currently visible in the viewport,
scroll bars will be displayed, unless they are turned off completely. Note that in general the content rectangle is not
updated automatically but declared as a fixed rectangle. Rather the application programmer needs to set this value or
call respective automatic update methods.
@default '0,0,100,100'
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23contentRect Online Documentation}*/
contentRect: Rect;
/**
Adds the given listener for the <code>ContentRectChanged</code> event that occurs when the {@link CanvasComponent#contentRect content rectangle} has been changed.
@param listener The listener to add.
@see CanvasComponent#removeContentRectChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ContentRectChanged Online Documentation}*/
addContentRectChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ContentRectChanged</code> event that occurs when the {@link CanvasComponent#contentRect content rectangle} has been changed.
@param listener The listener to remove.
@see CanvasComponent#addContentRectChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ContentRectChanged Online Documentation}*/
removeContentRectChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Gets and sets the insets in view coordinates that should be used by the {@link CanvasComponent#fitContent}
operation.
This influences the amount of visible whitespace in the view coordinate system around the {@link CanvasComponent#contentRect} after a {@link CanvasComponent#fitContent}
operation. The default value is <code>(10,10,10,10)</code>.
Note limits that are enforced by the {@link CanvasComponent#viewportLimiter} have a higher priority than these insets.
@default '10'
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23fitContentViewMargins Online Documentation}*/
fitContentViewMargins: Insets;
/**
Adds the given listener for the <code>FitContentViewMarginsChanged</code> event that occurs when the {@link CanvasComponent#fitContentViewMargins margins for the fit content operation}
have been changed.
@param listener The listener to add.
@see CanvasComponent#removeFitContentViewMarginsChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23FitContentViewMarginsChanged Online Documentation}*/
addFitContentViewMarginsChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>FitContentViewMarginsChanged</code> event that occurs when the {@link CanvasComponent#fitContentViewMargins margins for the fit content operation}
have been changed.
@param listener The listener to remove.
@see CanvasComponent#addFitContentViewMarginsChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23FitContentViewMarginsChanged Online Documentation}*/
removeFitContentViewMarginsChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>MouseLeave</code> event that occurs when the mouse has exited the canvas.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
If the mouse leaves this canvas control with the mouse button pressed, this event is deferred until the mouse button is
released. Thus, this event will not interfere with the typical mouse button event cycle.
@param listener The listener to add.
@see CanvasComponent#removeMouseLeaveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseLeave Online Documentation}*/
addMouseLeaveListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Removes the given listener for the <code>MouseLeave</code> event that occurs when the mouse has exited the canvas.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
If the mouse leaves this canvas control with the mouse button pressed, this event is deferred until the mouse button is
released. Thus, this event will not interfere with the typical mouse button event cycle.
@param listener The listener to remove.
@see CanvasComponent#addMouseLeaveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseLeave Online Documentation}*/
removeMouseLeaveListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Adds the given listener for the <code>MouseEnter</code> event that occurs when the mouse has entered the canvas.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
If the mouse enters this canvas control with a mouse button pressed, this event is fired instantly but the current
button state reported by the {@link MouseEventArgs} does not include this button unless the browser supports the
property <code>MouseEvent.buttons</code>. See the Known Issues for more details.
@param listener The listener to add.
@see CanvasComponent#removeMouseEnterListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseEnter Online Documentation}*/
addMouseEnterListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Removes the given listener for the <code>MouseEnter</code> event that occurs when the mouse has entered the canvas.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
If the mouse enters this canvas control with a mouse button pressed, this event is fired instantly but the current
button state reported by the {@link MouseEventArgs} does not include this button unless the browser supports the
property <code>MouseEvent.buttons</code>. See the Known Issues for more details.
@param listener The listener to remove.
@see CanvasComponent#addMouseEnterListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseEnter Online Documentation}*/
removeMouseEnterListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Adds the given listener for the <code>MouseMove</code> event that occurs when the mouse has been moved in world coordinates.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
Move elements are delivered if no mouse button is pressed. This event will be fired, too, if the mouse does not move but
the world coordinates to which the current mouse position maps change. E.g. this will happen if the zoom level or the
view point is changed.
@param listener The listener to add.
@see CanvasComponent#removeMouseMoveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseMove Online Documentation}*/
addMouseMoveListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Removes the given listener for the <code>MouseMove</code> event that occurs when the mouse has been moved in world coordinates.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
Move elements are delivered if no mouse button is pressed. This event will be fired, too, if the mouse does not move but
the world coordinates to which the current mouse position maps change. E.g. this will happen if the zoom level or the
view point is changed.
@param listener The listener to remove.
@see CanvasComponent#addMouseMoveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseMove Online Documentation}*/
removeMouseMoveListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Adds the given listener for the <code>MouseDown</code> event that occurs when a mouse button has been pressed, i.e., on <code>mousedown</code>.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeMouseDownListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseDown Online Documentation}*/
addMouseDownListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Removes the given listener for the <code>MouseDown</code> event that occurs when a mouse button has been pressed, i.e., on
<code>mousedown</code>.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addMouseDownListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseDown Online Documentation}*/
removeMouseDownListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Adds the given listener for the <code>MouseDrag</code> event that occurs when the mouse is being moved while at least one of the
mouse buttons is pressed.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeMouseDragListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseDrag Online Documentation}*/
addMouseDragListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Removes the given listener for the <code>MouseDrag</code> event that occurs when the mouse is being moved while at least one of the
mouse buttons is pressed.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addMouseDragListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseDrag Online Documentation}*/
removeMouseDragListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Adds the given listener for the <code>MouseUp</code> event that occurs when the mouse button has been released, i.e., on <code>mouseup</code>.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeMouseUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseUp Online Documentation}*/
addMouseUpListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Removes the given listener for the <code>MouseUp</code> event that occurs when the mouse button has been released, i.e., on <code>mouseup</code>.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addMouseUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseUp Online Documentation}*/
removeMouseUpListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Adds the given listener for the <code>MouseLostCapture</code> event that occurs when the mouse capture has been lost.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeMouseLostCaptureListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseLostCapture Online Documentation}*/
addMouseLostCaptureListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Removes the given listener for the <code>MouseLostCapture</code> event that occurs when the mouse capture has been lost.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addMouseLostCaptureListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseLostCapture Online Documentation}*/
removeMouseLostCaptureListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Adds the given listener for the <code>MouseWheel</code> event that occurs when the mouse wheel has turned.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeMouseWheelListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseWheel Online Documentation}*/
addMouseWheelListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Removes the given listener for the <code>MouseWheel</code> event that occurs when the mouse wheel has turned.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addMouseWheelListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseWheel Online Documentation}*/
removeMouseWheelListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Adds the given listener for the <code>MouseClick</code> event that occurs when the user clicked the mouse.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
This happens if press and release happens at the same position.
@param listener The listener to add.
@see CanvasComponent#removeMouseClickListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseClick Online Documentation}*/
addMouseClickListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Removes the given listener for the <code>MouseClick</code> event that occurs when the user clicked the mouse.
This event delivers {@link MouseEventArgs} in world coordinates using double precision floating points.
This happens if press and release happens at the same position.
@param listener The listener to remove.
@see CanvasComponent#addMouseClickListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23MouseClick Online Documentation}*/
removeMouseClickListener(listener: (sender: any, evt: MouseEventArgs) => void): void;
/**
Adds the given listener for the <code>KeyDown</code> event that occurs when keys are being pressed, i.e., on <code>keydown</code>.
This event delivers {@link KeyEventArgs}.
@param listener The listener to add.
@see CanvasComponent#removeKeyDownListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23KeyDown Online Documentation}*/
addKeyDownListener(listener: (sender: any, evt: KeyEventArgs) => void): void;
/**
Removes the given listener for the <code>KeyDown</code> event that occurs when keys are being pressed, i.e., on <code>keydown</code>.
This event delivers {@link KeyEventArgs}.
@param listener The listener to remove.
@see CanvasComponent#addKeyDownListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23KeyDown Online Documentation}*/
removeKeyDownListener(listener: (sender: any, evt: KeyEventArgs) => void): void;
/**
Adds the given listener for the <code>KeyPress</code> event that occurs when keys are being typed, i.e., on <code>keypress</code>.
This event delivers {@link KeyEventArgs}.
@param listener The listener to add.
@see CanvasComponent#removeKeyPressListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23KeyPress Online Documentation}*/
addKeyPressListener(listener: (sender: any, evt: KeyEventArgs) => void): void;
/**
Removes the given listener for the <code>KeyPress</code> event that occurs when keys are being typed, i.e., on <code>keypress</code>.
This event delivers {@link KeyEventArgs}.
@param listener The listener to remove.
@see CanvasComponent#addKeyPressListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23KeyPress Online Documentation}*/
removeKeyPressListener(listener: (sender: any, evt: KeyEventArgs) => void): void;
/**
Adds the given listener for the <code>KeyUp</code> event that occurs when keys are being released, i.e., on <code>keyup</code>.
This event delivers {@link KeyEventArgs}.
@param listener The listener to add.
@see CanvasComponent#removeKeyUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23KeyUp Online Documentation}*/
addKeyUpListener(listener: (sender: any, evt: KeyEventArgs) => void): void;
/**
Removes the given listener for the <code>KeyUp</code> event that occurs when keys are being released, i.e., on <code>keyup</code>.
This event delivers {@link KeyEventArgs}.
@param listener The listener to remove.
@see CanvasComponent#addKeyUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23KeyUp Online Documentation}*/
removeKeyUpListener(listener: (sender: any, evt: KeyEventArgs) => void): void;
/**
Adds the given listener for the <code>ViewportChanged</code> event that occurs when the {@link CanvasComponent#viewport} property has been changed.
@param listener The listener to add.
@see CanvasComponent#removeViewportChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ViewportChanged Online Documentation}*/
addViewportChangedListener(listener: (sender: any, evt: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>ViewportChanged</code> event that occurs when the {@link CanvasComponent#viewport} property has been changed.
@param listener The listener to remove.
@see CanvasComponent#addViewportChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ViewportChanged Online Documentation}*/
removeViewportChangedListener(listener: (sender: any, evt: PropertyChangedEventArgs) => void): void;
/**
Adds the given listener for the <code>TouchDown</code> event that occurs when a finger has been put on the touch screen.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeTouchDownListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchDown Online Documentation}*/
addTouchDownListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Removes the given listener for the <code>TouchDown</code> event that occurs when a finger has been put on the touch screen.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addTouchDownListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchDown Online Documentation}*/
removeTouchDownListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Adds the given listener for the <code>TouchUp</code> event that occurs when a finger has been removed from the touch screen.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeTouchUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchUp Online Documentation}*/
addTouchUpListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Removes the given listener for the <code>TouchUp</code> event that occurs when a finger has been removed from the touch screen.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addTouchUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchUp Online Documentation}*/
removeTouchUpListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Adds the given listener for the <code>TouchMove</code> event that occurs when a finger has been moved on the touch screen.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeTouchMoveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchMove Online Documentation}*/
addTouchMoveListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Removes the given listener for the <code>TouchMove</code> event that occurs when a finger has been moved on the touch screen.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addTouchMoveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchMove Online Documentation}*/
removeTouchMoveListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Adds the given listener for the <code>TouchEnter</code> event that occurs when a finger on the touch screen has entered the canvas.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeTouchEnterListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchEnter Online Documentation}*/
addTouchEnterListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Removes the given listener for the <code>TouchEnter</code> event that occurs when a finger on the touch screen has entered the
canvas.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addTouchEnterListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchEnter Online Documentation}*/
removeTouchEnterListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Adds the given listener for the <code>TouchLeave</code> event that occurs when a finger on the touch screen has exited the canvas.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeTouchLeaveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchLeave Online Documentation}*/
addTouchLeaveListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Removes the given listener for the <code>TouchLeave</code> event that occurs when a finger on the touch screen has exited the
canvas.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addTouchLeaveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchLeave Online Documentation}*/
removeTouchLeaveListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Adds the given listener for the <code>TouchLostCapture</code> event that occurs when the touch capture has been lost.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to add.
@see CanvasComponent#removeTouchLostCaptureListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchLostCapture Online Documentation}*/
addTouchLostCaptureListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Removes the given listener for the <code>TouchLostCapture</code> event that occurs when the touch capture has been lost.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
@param listener The listener to remove.
@see CanvasComponent#addTouchLostCaptureListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchLostCapture Online Documentation}*/
removeTouchLostCaptureListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Adds the given listener for the <code>TouchClick</code> event that occurs when the user performed a tap gesture with a finger on the
touch screen.
An event that delivers {@link TouchEventArgs} in world coordinates using double precision floating points when the
user performed a tap gesture with a finger on the touch screen.
This happens if down and up happens at the same position.
@param listener The listener to add.
@see CanvasComponent#removeTouchClickListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchClick Online Documentation}*/
addTouchClickListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Removes the given listener for the <code>TouchClick</code> event that occurs when the user performed a tap gesture with a finger on
the touch screen.
An event that delivers {@link TouchEventArgs} in world coordinates using double precision floating points when the
user performed a tap gesture with a finger on the touch screen.
This happens if down and up happens at the same position.
@param listener The listener to remove.
@see CanvasComponent#addTouchClickListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchClick Online Documentation}*/
removeTouchClickListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Adds the given listener for the <code>TouchLongPress</code> event that occurs when the user performed a long press gesture with a
finger on the touch screen.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
This happens if the finger is held in the same position for the duration specified in {@link CanvasComponent#longPressTime} after a
{@link CanvasComponent#addTouchDownListener TouchDown}.
@param listener The listener to add.
@see CanvasComponent#removeTouchLongPressListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchLongPress Online Documentation}*/
addTouchLongPressListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Removes the given listener for the <code>TouchLongPress</code> event that occurs when the user performed a long press gesture with a
finger on the touch screen.
This event delivers {@link TouchEventArgs} in world coordinates using double precision floating points.
This happens if the finger is held in the same position for the duration specified in {@link CanvasComponent#longPressTime} after a
{@link CanvasComponent#addTouchDownListener TouchDown}.
@param listener The listener to remove.
@see CanvasComponent#addTouchLongPressListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23TouchLongPress Online Documentation}*/
removeTouchLongPressListener(listener: (sender: any, evt: TouchEventArgs) => void): void;
/**
Gets a list of the active {@link TouchDevice}s at the time of invocation.
Note that this is not a live view, but rather a snapshot.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23currentTouchDevices Online Documentation}*/
readonly currentTouchDevices: IListEnumerable<TouchDevice>;
/**
Gets a value indicating whether at least one finger is on the touch screen.
@returns <code>true</code> if there is at least one {@link TouchDevice touch device} that is down on the screen.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23isTouchDeviceDown Online Documentation}*/
readonly isTouchDeviceDown: boolean;
/**
Gets the last {@link TouchEventArgs touch event} triggered by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23lastTouchEvent Online Documentation}*/
lastTouchEvent: TouchEventArgs;
/**
Gets the last {@link MouseEventArgs mouse event} triggered by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23lastMouseEvent Online Documentation}*/
lastMouseEvent: MouseEventArgs;
/**
Gets the last {@link MouseEventArgs mouse event} or {@link TouchEventArgs touch event} triggered by this instance.
@see CanvasComponent#addMouseEnterListener
@see CanvasComponent#addMouseMoveListener
@see CanvasComponent#addMouseLeaveListener
@see CanvasComponent#addMouseDownListener
@see CanvasComponent#addMouseUpListener
@see CanvasComponent#addMouseDragListener
@see CanvasComponent#addMouseWheelListener
@see CanvasComponent#addMouseClickListener
@see CanvasComponent#addTouchEnterListener
@see CanvasComponent#addTouchMoveListener
@see CanvasComponent#addTouchLeaveListener
@see CanvasComponent#addTouchDownListener
@see CanvasComponent#addTouchUpListener
@see CanvasComponent#addTouchClickListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23lastInputEvent Online Documentation}*/
lastInputEvent: EventArgs;
/**
Gets the last location provided by a pointing device (for instance mouse or touch).
Unlike {@link CanvasComponent#lastMouseEvent} and {@link CanvasComponent#lastTouchEvent} this will always contain the latest location, regardless of what input method generated it.
@see CanvasComponent#lastMouseEvent
@see CanvasComponent#lastTouchEvent
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23lastEventLocation Online Documentation}*/
readonly lastEventLocation: Point;
/**
Adds the given listener for the <code>InputModeChanged</code> event that occurs when the {@link CanvasComponent#inputMode} property is changed.
@param listener The listener to add.
@see CanvasComponent#inputMode
@see CanvasComponent#removeInputModeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23InputModeChanged Online Documentation}*/
addInputModeChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>InputModeChanged</code> event that occurs when the {@link CanvasComponent#inputMode} property is changed.
@param listener The listener to remove.
@see CanvasComponent#inputMode
@see CanvasComponent#addInputModeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23InputModeChanged Online Documentation}*/
removeInputModeChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
* Gets or sets the single {@link IInputMode} instance that shall be installed for this canvas.
* <br><b>Installing a GraphEditorInputMode</b>
* <pre>
* graphComponent.inputMode = new GraphEditorInputMode()
* </pre>
*
* <br><b>Installing a GraphViewerInputMode</b>
* <pre>
* graphComponent.inputMode = new GraphViewerInputMode()
* </pre>
*
* @default null
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23inputMode Online Documentation}
*/
inputMode: IInputMode | null;
/**
Gets or sets the radius of the area around the mouse in view coordinates in which an {@link IHitTestable} may lie
to be considered a valid hit.
This value converted into world coordinates can be queried from within the {@link IHitTestable} implementation from
the {@link ICanvasContext#hitTestRadius} property, which automatically takes the zoom level into account. Note that that property automatically switched
between this value and {@link CanvasComponent#hitTestRadiusTouch}, based on the last input event.
The default value is <code>3</code>.
@default 3.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23hitTestRadius Online Documentation}*/
hitTestRadius: number;
/**
Gets or sets the radius of the area around the touch point in view coordinates in which an {@link IHitTestable} may
lie to be considered a valid hit.
This value converted into world coordinates can be queried from within the {@link IHitTestable} implementation from
the {@link ICanvasContext#hitTestRadius} property, which automatically takes the zoom level into account. Note that that property automatically switched
between this value and {@link CanvasComponent#hitTestRadius}, based on the last input event.
The default value is <code>10</code>.
@default 10.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23hitTestRadiusTouch Online Documentation}*/
hitTestRadiusTouch: number;
/**
Gets or sets a value indicating whether the maximum zoom level for {@link CanvasComponent#fitContent} and {@link ICommand#FIT_CONTENT}
is limited to 1.
If this property is <code>true</code>, {@link CanvasComponent#fitContent} and {@link ICommand#FIT_CONTENT} will limit the
maximum zoom level to 1. This can lead to very small graphs not filling the viewport. If this property is set to <code>false</code>,
zooming will only be limited by {@link CanvasComponent#maximumZoom}.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23limitFitContentZoom Online Documentation}*/
limitFitContentZoom: boolean;
/**
* Gets or sets the zoom factor for this {@link CanvasComponent}.
* A zoom level of <code>1.0</code> will make each unit in world-coordinate space appear exactly one unit in screen coordinates wide.
* The default is <code>1.0</code>. When setting this property to change the zoom level, the center point will remain the same.
*
* The valid range is bounded by {@link CanvasComponent#minimumZoom} and {@link CanvasComponent#maximumZoom}
*
* <br><b>Resetting the zoom level to 100 %</b>
* <pre>
* console.log(`Resetting the zoom level from ${graphComponent.zoom} to 100 %.`)
* graphComponent.zoom = 1
* </pre>
*
* @see CanvasComponent#addZoomChangedListener
* @see CanvasComponent#minimumZoom
* @see CanvasComponent#maximumZoom
* @see CanvasComponent#zoomTo
* @see CanvasComponent#zoomToAnimated
* @default 1.0
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23zoom Online Documentation}
*/
zoom: number;
/**
Adds the given listener for the <code>ZoomChanged</code> event that occurs when the value of the {@link CanvasComponent#zoom} property has been changed.
@param listener The listener to add.
@see CanvasComponent#removeZoomChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ZoomChanged Online Documentation}*/
addZoomChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ZoomChanged</code> event that occurs when the value of the {@link CanvasComponent#zoom} property has been changed.
@param listener The listener to remove.
@see CanvasComponent#addZoomChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ZoomChanged Online Documentation}*/
removeZoomChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
* Gets or sets the minimum zoom factor for this {@link CanvasComponent}.
* This property sets a lower bound for the allowed values of the {@link CanvasComponent#zoom} property. If this property is written, the canvas will
* zoom to the new value while keeping the center of view at the same world coordinates. The default value is <code>0.0001</code>, the
* enforced minimum <code>0.0000001</code>.
*
* <br><b>Ensuring that the zoom level can not go below 100 %</b>
* <pre>
* graphComponent.minimumZoom = 1
* </pre>
*
* @see CanvasComponent#zoom
* @see CanvasComponent#maximumZoom
* @default 0.0001
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23minimumZoom Online Documentation}
*/
minimumZoom: number;
/**
* Gets or sets the maximum zoom factor for this {@link CanvasComponent}.
* This property sets an upper bound for the allowed values of the {@link CanvasComponent#zoom} property. The default is <code>100000.0</code> and the enforced
* maximum value is <code>1000000.0</code>.
*
* <br><b>Ensuring that the zoom level can not go above 400 %</b>
* <pre>
* graphComponent.maximumZoom = 4
* </pre>
*
* @default 100000.0
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23maximumZoom Online Documentation}
*/
maximumZoom: number;
/**
* Gets or sets the current view point.
* The view point is the point in world coordinates that is mapped to the top left corner point in the current viewport.
* Setting this point to another value will redispatch the last mouse event as the mouse will appear to have been moved or
* dragged in the world coordinate system.
*
* <br><b>Setting a new value for the ViewPoint property</b>
* <pre>
* console.log(
*   `Current view point: ${graphComponent.viewPoint} , and center: ${graphComponent.center}`
* )
* graphComponent.viewPoint = new Point(150, 50)
* console.log(`New view point is: ${graphComponent.viewPoint}`)
* console.log(`The center has also changed: ${graphComponent.center}`)
* </pre>
*
* @default '0,0'
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23viewPoint Online Documentation}
*/
viewPoint: Point;
/**
Gets the size of the usable area in which the graph will be displayed.
This value excludes the scroll bars.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23innerSize Online Documentation}*/
readonly innerSize: Size;
/**
Returns the size of the HTML element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23size Online Documentation}*/
readonly size: Size;
/**
Gets the smallest rectangle in world coordinates that encompasses the visible viewing region.
@see CanvasComponent#viewPoint
@see CanvasComponent#zoom
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23viewport Online Documentation}*/
readonly viewport: Rect;
/**
Gets the {@link LookupChain} that can be used do decorate the {@link ILookup#lookup} call in the {@link CanvasComponent#inputModeContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23inputModeContextLookupChain Online Documentation}*/
readonly inputModeContextLookupChain: LookupChain;
/**
Gets an implementation of {@link ICanvasContext} that describes the state of this {@link CanvasComponent}.
@see IHitTestable
@see IBoundsProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23canvasContext Online Documentation}*/
readonly canvasContext: ICanvasContext;
/**
* Gets or sets the world coordinate at the center of the control.
* Setting this property moves the viewport, similarly to {@link CanvasComponent#viewPoint}. It will not change the zoom level.
*
* <br><b>Setting a new center point for the viewport</b>
* <pre>
* console.log(
*   `Current center: ${graphComponent.center} , and view point: ${graphComponent.viewPoint}`
* )
* graphComponent.center = new Point(150, 50)
* console.log(`New center is: ${graphComponent.center}`)
* console.log(`The view point has also changed: ${graphComponent.viewPoint}`)
* </pre>
*
* @see CanvasComponent#viewPoint
* @see CanvasComponent#zoomTo
* @see CanvasComponent#zoomToAnimated
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23center Online Documentation}
*/
center: Point;
/**
Gets or sets a property that causes the control to automatically pan the view when the mouse gets dragged outside of the
bounds of the control.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23autoDrag Online Documentation}*/
autoDrag: boolean;
/**
Gets or sets the policy for caching {@link Visual}s which are temporarily removed from the visual tree.
Visuals are temporarily removed when they are moved outside the visual part of the canvas (the {@link CanvasComponent#viewport}) and re-added when they
enter the {@link CanvasComponent#viewport} again. Caching prevents the need to re-build the visuals from scratch.
Visuals are only cached if this property is set to another value than {@link VisualCachingPolicy#NEVER} and if a
dispose visual callback is {@link IRenderContext#setDisposeCallback registered} during {@link IVisualCreator#createVisual creation}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23visualCaching Online Documentation}*/
visualCaching: VisualCachingPolicy;
/**
Adds the given listener for the <code>UpdatedVisual</code> event that occurs after the {@link CanvasComponent#updateVisual visual tree has been updated}.
@param listener The listener to add.
@see CanvasComponent#removeUpdatedVisualListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23UpdatedVisual Online Documentation}*/
addUpdatedVisualListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>UpdatedVisual</code> event that occurs after the {@link CanvasComponent#updateVisual visual tree has been updated}.
@param listener The listener to remove.
@see CanvasComponent#addUpdatedVisualListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23UpdatedVisual Online Documentation}*/
removeUpdatedVisualListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>UpdatingVisual</code> event that occurs before the {@link CanvasComponent#updateVisual visual tree is updated}.
@param listener The listener to add.
@see CanvasComponent#removeUpdatingVisualListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23UpdatingVisual Online Documentation}*/
addUpdatingVisualListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>UpdatingVisual</code> event that occurs before the {@link CanvasComponent#updateVisual visual tree is updated}.
@param listener The listener to remove.
@see CanvasComponent#addUpdatingVisualListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23UpdatingVisual Online Documentation}*/
removeUpdatingVisualListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Gets the root of the scene graph.
This group cannot be {@link ICanvasObject#remove removed}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23rootGroup Online Documentation}*/
readonly rootGroup: ICanvasObjectGroup;
/**
Gets or sets the ContentGroup property.
The content group is the {@link ICanvasObjectGroup} that should be used by the application code to put actual
content in.
If the field has not yet been initialized upon first access, the factory method {@link CanvasComponent#createContentGroup}
will be called. Upon change the {@link CanvasComponent#addContentGroupChangedListener ContentGroupChanged}
event will be fired.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23contentGroup Online Documentation}*/
contentGroup: ICanvasObjectGroup;
/**
Adds the given listener for the <code>ContentGroupChanged</code> event that occurs when the {@link CanvasComponent#contentGroup} property has been changed.
@param listener The listener to add.
@see CanvasComponent#removeContentGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ContentGroupChanged Online Documentation}*/
addContentGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>ContentGroupChanged</code> event that occurs when the {@link CanvasComponent#contentGroup} property has been changed.
@param listener The listener to remove.
@see CanvasComponent#addContentGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23ContentGroupChanged Online Documentation}*/
removeContentGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the HighlightGroup property.
If the field has not yet been initialized upon first access, the factory method {@link CanvasComponent#createHighlightGroup}
will be called. Upon change the {@link CanvasComponent#addHighlightGroupChangedListener HighlightGroupChanged}
event will be fired.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23highlightGroup Online Documentation}*/
highlightGroup: ICanvasObjectGroup;
/**
Gets or sets the HighlightGroup property.
If the field has not yet been initialized upon first access, the factory method {@link CanvasComponent#createHighlightGroup}
will be called. Upon change the {@link CanvasComponent#addHighlightGroupChangedListener HighlightGroupChanged}
event will be fired.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23focusGroup Online Documentation}*/
focusGroup: ICanvasObjectGroup;
/**
Adds the given listener for the <code>FocusGroupChanged</code> event that occurs when the {@link CanvasComponent#focusGroup} property has been changed.
@param listener The listener to add.
@see CanvasComponent#removeFocusGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23FocusGroupChanged Online Documentation}*/
addFocusGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>FocusGroupChanged</code> event that occurs when the {@link CanvasComponent#focusGroup} property has been changed.
@param listener The listener to remove.
@see CanvasComponent#addFocusGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23FocusGroupChanged Online Documentation}*/
removeFocusGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Adds the given listener for the <code>HighlightGroupChanged</code> event that occurs when the {@link CanvasComponent#highlightGroup} property has been changed.
@param listener The listener to add.
@see CanvasComponent#removeHighlightGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23HighlightGroupChanged Online Documentation}*/
addHighlightGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>HighlightGroupChanged</code> event that occurs when the {@link CanvasComponent#highlightGroup} property has been changed.
@param listener The listener to remove.
@see CanvasComponent#addHighlightGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23HighlightGroupChanged Online Documentation}*/
removeHighlightGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the SelectionGroup property.
The selection group is the {@link ICanvasObjectGroup} that should be used by the application code to put the
selection indicators in.
If the field has not yet been initialized upon first access, the factory method {@link CanvasComponent#createSelectionGroup}
will be called. Upon change the {@link CanvasComponent#addSelectionGroupChangedListener SelectionGroupChanged}
event will be fired.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23selectionGroup Online Documentation}*/
selectionGroup: ICanvasObjectGroup;
/**
Adds the given listener for the <code>SelectionGroupChanged</code> event that occurs when the {@link CanvasComponent#selectionGroup} property has been changed.
@param listener The listener to add.
@see CanvasComponent#removeSelectionGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23SelectionGroupChanged Online Documentation}*/
addSelectionGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>SelectionGroupChanged</code> event that occurs when the {@link CanvasComponent#selectionGroup} property has been changed.
@param listener The listener to remove.
@see CanvasComponent#addSelectionGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23SelectionGroupChanged Online Documentation}*/
removeSelectionGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the InputModeGroup property.
This is the canvas object group where the {@link IInputMode}s should add their temporary content to. This group by
default is in front of the {@link CanvasComponent#contentGroup}.
If the field has not yet been initialized upon first access, the factory method {@link CanvasComponent#createInputModeGroup}
will be called. Upon change the {@link CanvasComponent#addInputModeGroupChangedListener InputModeGroupChanged}
event will be fired.
@see CanvasComponent#contentGroup
@see CanvasComponent#backgroundGroup
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23inputModeGroup Online Documentation}*/
inputModeGroup: ICanvasObjectGroup;
/**
Adds the given listener for the <code>InputModeGroupChanged</code> event that occurs when the {@link CanvasComponent#inputModeGroup} property has been changed.
@param listener The listener to add.
@see CanvasComponent#removeInputModeGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23InputModeGroupChanged Online Documentation}*/
addInputModeGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>InputModeGroupChanged</code> event that occurs when the {@link CanvasComponent#inputModeGroup} property has been changed.
@param listener The listener to remove.
@see CanvasComponent#addInputModeGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23InputModeGroupChanged Online Documentation}*/
removeInputModeGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
* Gets or sets the BackgroundGroup property.
* The background group is the {@link ICanvasObjectGroup} that should be used by the application code to put background
* elements in.
*
* If the field has not yet been initialized upon first access, the factory method {@link CanvasComponent#createBackgroundGroup}
* will be called. Upon change the {@link CanvasComponent#addBackgroundGroupChangedListener BackgroundGroupChanged}
* event will be fired.
*
* The {@link CanvasComponent#backgroundGroup} can be used for displaying visuals in the background of the main content. For example, a background image:
*
* <br><b>Adding a background image to a CanvasControl</b>
* <pre>
* /**
*  * Creates the image for the background visual.
*  *‍/
* class ImageVisualCreator extends BaseClass(IVisualCreator) {
*   createVisual(context) {
*     const image = window.document.createElementNS('http://www.w3.org/2000/svg', 'image')
*     image.setAttribute('width', '640')
*     image.setAttribute('height', '480')
*     image.setAttribute('x', '-150')
*     image.setAttribute('y', '-160')
*     image.setAttributeNS('http://www.w3.org/1999/xlink', 'href', 'ylogo.svg')
*     return new SvgVisual(image)
*   }
*
*   updateVisual(context, oldVisual) {
*     return oldVisual
*   }
* }
*
* graphComponent.backgroundGroup.addChild(
*   new ImageVisualCreator(),
*   ICanvasObjectDescriptor.ALWAYS_DIRTY_INSTANCE
* )
* </pre>
*
* This image will scroll and zoom with the content, though. If a static image that doesn't move with the content is
* needed, add it as CSS background to the <code>div</code> element of this component.
*
* @see CanvasComponent#contentGroup
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23backgroundGroup Online Documentation}
*/
backgroundGroup: ICanvasObjectGroup;
/**
Adds the given listener for the <code>BackgroundGroupChanged</code> event that occurs when the {@link CanvasComponent#backgroundGroup} property has been changed.
@param listener The listener to add.
@see CanvasComponent#removeBackgroundGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23BackgroundGroupChanged Online Documentation}*/
addBackgroundGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>BackgroundGroupChanged</code> event that occurs when the {@link CanvasComponent#backgroundGroup} property has been changed.
@param listener The listener to remove.
@see CanvasComponent#addBackgroundGroupChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23BackgroundGroupChanged Online Documentation}*/
removeBackgroundGroupChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the shape-rendering type for this visual and its children.
Shape rendering can improve performance but affects the display quality of this visual.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23shapeRendering Online Documentation}*/
shapeRendering: ShapeRenderingType;
/**
Gets or sets the image-rendering type for this visual and its children.
Image rendering can improve performance but affects the display quality of images.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23imageRendering Online Documentation}*/
imageRendering: ImageRenderingType;
/**
Returns an HTML element that can be used to show arbitrary HTML content like overlays, fly-outs, or pop-ups on top of
this {@link CanvasComponent}.
An HTML element that was added to the overlay panel is drawn above the content of this {@link CanvasComponent} but
below its scrollbars. In contrast to labels or other model items, this element keeps its size when the canvas is zoomed.
It should be placed with the CSS property <code>position: absolute</code> to prevent multiple overlay elements to interfere with each
other.
If a <code>div</code> element is passed to the constructor of this
{@link CanvasComponent}, all its child nodes are passed to this overlay panel. Apart from that, content may be added
to and removed from this panel at any time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23overlayPanel Online Documentation}*/
readonly overlayPanel: HTMLElement;
/**
Gets or sets the value of the drag time.
This value indicates the amount of time that may pass before a mouse movement is considered a drag when the mouse stays
within its {@link CanvasComponent#dragSize} area (or a touch point within its {@link CanvasComponent#dragSizeTouch} area). The higher the value the later a mouse or touch movement will be
recognized as a drag. This influences the click-sensitivity, since a mouse button release is only considered a click if
there was no drag since the last mouse button press. The default value is 300 milliseconds.
@see CanvasComponent#dragSize
@see CanvasComponent#dragSizeTouch
@see CanvasComponent#addMouseDragListener
@see CanvasComponent#addMouseClickListener
@see CanvasComponent#addTouchMoveListener
@see CanvasComponent#addTouchClickListener
@default '00:00:00.3000000'
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23dragTime Online Documentation}*/
dragTime: TimeSpan;
/**
Gets or sets the value of the double click time.
This value indicates the amount of time that may pass within which two subsequent mouse clicks are considered multi
clicks. The higher the value the easier it will be to double-click. The default value is 500 milliseconds.
@see CanvasComponent#doubleClickSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23doubleClickTime Online Documentation}*/
doubleClickTime: TimeSpan;
/**
Gets or sets the value of the double tap time.
This value indicates the amount of time that may pass within which two subsequent touch taps are considered multi taps.
The higher the value the easier it will be to double-tap. The default value is 500 milliseconds.
@see CanvasComponent#doubleTapSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23doubleTapTime Online Documentation}*/
doubleTapTime: TimeSpan;
/**
Gets or sets the value of the long press time.
This value indicates the amount of time that may pass before a touch down is considered a long press when the pointer
stays within its {@link CanvasComponent#dragSizeTouch} area. The higher the value the later a touch pointer movement will be recognized as a long press. The
default value is 250 milliseconds.
@see CanvasComponent#dragSizeTouch
@see CanvasComponent#addTouchDownListener
@see CanvasComponent#addTouchMoveListener
@see CanvasComponent#addTouchLongPressListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23longPressTime Online Documentation}*/
longPressTime: TimeSpan;
/**
Gets or sets the area in view coordinates the mouse may stay in before a movement is considered a drag.
If the mouse is moved within this area longer than {@link CanvasComponent#dragTime} milliseconds, the movement will be considered a drag, nevertheless.
The larger the area the later a mouse movement will be recognized as a drag. This influences the click-sensitivity,
since a mouse button release is only considered a click if there was no drag since the last mouse button press. The
default value is <code>(5, 5)</code>.
@see CanvasComponent#dragSizeTouch
@see CanvasComponent#addMouseDragListener
@see CanvasComponent#addMouseClickListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23dragSize Online Documentation}*/
dragSize: Size;
/**
Gets or sets the area in view coordinates the touch point may stay in before a movement is considered a drag.
If the finger is moved within this area longer than {@link CanvasComponent#dragTime} milliseconds, the movement will be considered a drag, nevertheless.
The larger the area the later a finger movement will be recognized as a drag. This influences the sensitivity to taps,
since lifting the finger is only considered a tap if there was no drag since the touch point appeared. The default value
is <code>(20, 20)</code>.
@see CanvasComponent#dragSize
@see CanvasComponent#addTouchMoveListener
@see CanvasComponent#addTouchClickListener
@default '20,20'
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23dragSizeTouch Online Documentation}*/
dragSizeTouch: Size;
/**
Gets or sets the area in view coordinates the mouse needs to stay in before multiple clicks are considered multiple
single clicks instead of multi-clicks.
If the mouse stays within this within than {@link CanvasComponent#doubleClickTime}, multiple clicks will be considered double or multi-clicks. The default
value is <code>(10, 10)</code>.
@see CanvasComponent#doubleClickTime
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23doubleClickSize Online Documentation}*/
doubleClickSize: Size;
/**
Gets or sets the area in view coordinates the touch pointer needs to stay in before multiple taps are considered
multiple single taps instead of multi-taps.
If the touch pointer stays within this within than {@link CanvasComponent#doubleTapTime}, multiple taps will be considered double or multi-taps. The default
value is <code>(20, 20)</code>.
@see CanvasComponent#doubleTapTime
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23doubleTapSize Online Documentation}*/
doubleTapSize: Size;
/**
Gets or sets the factor by which the zoom level changes when the mouse wheel is turned.
The default value is 1.2.
@default 1.2
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23mouseWheelZoomFactor Online Documentation}*/
mouseWheelZoomFactor: number;
/**
Gets or sets a factor that controls how fast the viewport scrolls when the mouse wheel is turned.
Scrolling can work either a few lines at a time (per notch of the scroll wheel) or one page at a time. This is a system
setting. This property controls how many pixels represent a line if scrolling is done by line. In case scrolling is done
by page, this property has no effect.
The default value of 5 pixels. To speed up scrolling, set a larger value, to slow down scrolling, set a smaller value.
@see CanvasComponent#mouseWheelZoomFactor
@see CanvasComponent#mouseWheelBehavior
@see MouseEventArgs#scrollAmount
@see MouseEventArgs#deltaMode
@default 5
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23mouseWheelScrollFactor Online Documentation}*/
mouseWheelScrollFactor: number;
/**
Gets or sets the auto drag insets.
If the mouse is being dragged within the insets of the control specified using this property, the control will
automatically scroll.
@see CanvasComponent#autoDrag
@default '10'
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23autoDragInsets Online Documentation}*/
autoDragInsets: Insets;
/**
Adds the given listener for the <code>PrepareRenderContext</code> event that occurs before the visual tree is painted to prepare the
{@link IRenderContext}.
@param listener The listener to add.
@see CanvasComponent#removePrepareRenderContextListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23PrepareRenderContext Online Documentation}*/
addPrepareRenderContextListener(listener: (sender: any, evt: PrepareRenderContextEventArgs) => void): void;
/**
Removes the given listener for the <code>PrepareRenderContext</code> event that occurs before the visual tree is painted to prepare
the {@link IRenderContext}.
@param listener The listener to remove.
@see CanvasComponent#addPrepareRenderContextListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23PrepareRenderContext Online Documentation}*/
removePrepareRenderContextListener(listener: (sender: any, evt: PrepareRenderContextEventArgs) => void): void;
/**
Gets or sets the InputModeContext property.
This context object is passed to {@link IInputMode} instances during {@link IInputMode#install installation} and removal.
If the field has not yet been initialized upon first access, the factory method {@link CanvasComponent#createInputModeContext}
will be called.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23inputModeContext Online Documentation}*/
inputModeContext: IInputModeContext;
/**
Adds the given listener for the <code>InputModeContextChanged</code> event that occurs when the {@link CanvasComponent#inputModeContext} property has been changed.
@param listener The listener to add.
@see CanvasComponent#removeInputModeContextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23InputModeContextChanged Online Documentation}*/
addInputModeContextChangedListener(listener: (sender: any, evt: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>InputModeContextChanged</code> event that occurs when the {@link CanvasComponent#inputModeContext} property has been changed.
@param listener The listener to remove.
@see CanvasComponent#addInputModeContextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23InputModeContextChanged Online Documentation}*/
removeInputModeContextChangedListener(listener: (sender: any, evt: PropertyChangedEventArgs) => void): void;
/**
* Gets or sets a value indicating whether navigation related command bindings are enabled.
* By default this feature is enabled.
*
* This property enables/disables the following commands:
*
* <ul>
* <li>{@link ICommand#INCREASE_ZOOM}</li>
* <li>{@link ICommand#ZOOM}</li>
* <li>{@link ICommand#DECREASE_ZOOM}</li>
* <li>{@link ICommand#SCROLL_PAGE_DOWN}</li>
* <li>{@link ICommand#SCROLL_PAGE_UP}</li>
* <li>{@link ICommand#SCROLL_PAGE_LEFT}</li>
* <li>{@link ICommand#SCROLL_PAGE_RIGHT}</li>
* </ul>
* <br><b>Disabling the navigation commands in code</b>
* <pre>
* graphComponent.navigationCommandsEnabled = false
* </pre>
*
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23navigationCommandsEnabled Online Documentation}
*/
navigationCommandsEnabled: boolean;
/**
Gets the resources for this instance.
Resources are named instances of objects that can be retrieved by other parts of the infrastructure that deal with this
instance, either directly or indirectly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23resources Online Documentation}*/
readonly resources: IMap<string, any>;
/**
Gets or sets a value indicating whether input coordinates (mouse and touch) should be quantized depending on the zoom
level to give nicer values.
By default, the coordinates of input events are translated directly to world coordinates, which, with many zoom levels
can end up with world coordinates like 326.76821937283548; this excessive precision is an artifact of the fact that
every viewport pixel covers a fractional amount of world coordinates.
This option quantizes the coordinates depending on the zoom level so that above value may end up, e.g. as 326.75
instead. This is done in a way to minimize visual deviation from the exact value (much less than a pixel), so visually
almost nothing changes. Zooming in will increase the precision as much as necessary, but not so far to be excessive. The
benefit is that coordinates are easier to recognize in a debugger, graph elements end up with locations that are easier
to read, GraphML files become smaller because less space is wasted on unnecessary floating point precision, and
numerical instability in certain cases (e.g. some snapping scenarios) is eliminated.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23quantizeInputCoordinates Online Documentation}*/
quantizeInputCoordinates: boolean;
/**
Gets or sets the interval of the timer that runs to detect size changes as a {@link TimeSpan}.
@see CanvasComponent#addSizeChangedListener
@see CanvasComponent#sizeChangedDetection
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23sizeChangedTimerInterval Online Documentation}*/
static sizeChangedTimerInterval: TimeSpan;
/**
Adds the given listener for the <code>SizeChanged</code> event that occurs when the size of this Control is changed.
@param listener The listener to add.
@see CanvasComponent#sizeChangedDetection
@see SizeChangedDetectionMode
@see CanvasComponent#removeSizeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23SizeChanged Online Documentation}*/
addSizeChangedListener(listener: (sender: any, evt: SizeChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>SizeChanged</code> event that occurs when the size of this Control is changed.
@param listener The listener to remove.
@see CanvasComponent#sizeChangedDetection
@see SizeChangedDetectionMode
@see CanvasComponent#addSizeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23SizeChanged Online Documentation}*/
removeSizeChangedListener(listener: (sender: any, evt: SizeChangedEventArgs) => void): void;
/**
Gets or sets how size changes of this {@link CanvasComponent} will be detected.
Since there is no size changed event in HTML, {@link CanvasComponent} comes with its own methods to detect whether
the size of its {@link CanvasComponent#div} element has changed. This is needed to update its {@link CanvasComponent#size} accordingly.
The preferred and default mode is {@link SizeChangedDetectionMode#SENSOR}. If that is not available, {@link SizeChangedDetectionMode#TIMER}
is used as fall back instead.
If the {@link CanvasComponent#div} of this control is removed from the DOM and the current mode is
{@link SizeChangedDetectionMode#TIMER}, the timer will be stopped to allow garbage collection. Thus, if the element
is later re-added to the DOM, you must explicitly set the timer mode again.
{@link SizeChangedDetectionMode#SENSOR} will automatically detect reinsertions into the DOM except in Internet
Explorer 9. Thus, if the element is re-added to the DOM, you must explicitly set the timer mode again in this browser.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23sizeChangedDetection Online Documentation}*/
sizeChangedDetection: SizeChangedDetectionMode;
/**
Gets or sets a {@link Matrix} that is applied to the viewport.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CanvasComponent%23projection Online Documentation}*/
projection: Matrix;
static $class: Class;
private $_CanvasComponent: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CanvasComponent;
}
export interface SvgExport extends YObject {
}
/**
This class sets up an {@link IRenderContext} object for exporting the contents of a {@link CanvasComponent} (see
{@link SvgExport#createRenderContext}).
Also it provides ready-to-use methods to export the canvas contents to a standalone SVG.
<ul>
<li>It is possible to export only a part of the contents, see {@link SvgExport#worldBounds}.
<li>The size of the target graphics is controlled by the {@link SvgExport#scale}. For a given target width or height, the appropriate scale can be
calculated using {@link SvgExport#calculateScaleForWidth} and {@link SvgExport#calculateScaleForHeight}.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport Online Documentation}*/
export class SvgExport {
/**
* Creates a new {@link SvgExport} instance for the given world bounds and scale.
* <br><b>Exports the contents of a GraphComponent to an SVG element</b>
* <pre>
* const exporter = new SvgExport(graphComponent.contentRect)
* // configure the export instance
* exporter.encodeImagesBase64 = true
* exporter.inlineSvgImages = true
* // create the <svg> element
* const element = exporter.exportSvg(graphComponent)
* </pre>
*
* @param worldBounds the bounds of the content to export, see {@link SvgExport#worldBounds}
* @param [scale=1.0] the scale, see {@link SvgExport#scale}
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport Online Documentation}
*/
constructor(worldBounds: Rect | RectConvertible, scale?: number);
/**
Calculates the value to set the {@link SvgExport#scale} to in order to achieve the given target height (without {@link SvgExport#margins}).
This method returns 1.0 if the {@link SvgExport#worldBounds} height is zero or negative.
@param height the height of the target image
@param [projection=null] An optional projection {@link Matrix} (cf.
{@link CanvasComponent#projection}) for which the scale should be calculated. The resulting scale ensures that the projected image fits into the desired
height.
@returns the scale value to use
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23calculateScaleForHeight(number%2CMatrix) Online Documentation}*/
calculateScaleForHeight(height: number, projection?: Matrix | null): number;
/**
Calculates the value to set the {@link SvgExport#scale} to in order to achieve the given target width (without {@link SvgExport#margins}).
This method returns 1.0 if the {@link SvgExport#worldBounds} width is zero or negative.
@param width the width of the target image
@param [projection=null] An optional projection {@link Matrix} (cf.
{@link CanvasComponent#projection}) for which the scale should be calculated. The resulting scale ensures that the projected image fits into the desired
width.
@returns the scale value to use
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23calculateScaleForWidth(number%2CMatrix) Online Documentation}*/
calculateScaleForWidth(width: number, projection?: Matrix | null): number;
/**
Returns a suitable clip to use for the visual for this export.
When exporting a {@link CanvasComponent} with a {@link CanvasComponent#projection} the clip is still just the {@link SvgExport#worldBounds} rectangle.
@returns The clip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23createClip() Online Documentation}*/
createClip(): Rect;
/**
Returns an initialized {@link IRenderContext} and changes the given transform and the clip bounds such that a part
of the given {@link CanvasComponent} instance can be exported to an image.
@param canvas The {@link CanvasComponent} instance to export.
@returns An {@link IRenderContext} to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23createRenderContext(CanvasComponent) Online Documentation}*/
createRenderContext(canvas: CanvasComponent): IRenderContext;
/**
Returns a suitable transform to use for the visual to paint in the world coordinate system for this export.
@returns The transform to use for the visual to paint in the world coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23createWorldToIntermediateTransform() Online Documentation}*/
createWorldToIntermediateTransform(): Matrix;
/**
Encodes the given SVG document as Data URI.
This method escapes non-Latin1 characters, does a base64 encoding, and returns the result as data URI.
@param svgXml A string containing the SVG document.
@returns A data URI of the SVG document.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23encodeSvgDataUrl(string) Online Documentation}*/
static encodeSvgDataUrl(svgXml: string): string;
/**
Exports the contents of the {@link CanvasComponent} to a single, standalone SVG element.
The asynchronous version of this method,
{@link SvgExport#exportSvgAsync}, is capable of encoding and inlining images that are embedded in the SVG.
@param canvas The canvas to be exported.
@returns An SVG element with the canvas contents.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23exportSvg(CanvasComponent) Online Documentation}*/
exportSvg(canvas: CanvasComponent): Element;
/**
Exports the contents of the {@link CanvasComponent} to a single, standalone SVG element.
In addition, images are inlined into the SVG using base64 encoding if {@link SvgExport#encodeImagesBase64} is set to <code>true</code>. Image inlining can be controlled
for each image individually by overwriting the predicate method {@link SvgExport#shouldEncodeImageBase64}.
SVG image elements are replaced with a copy of the referenced SVG element if {@link SvgExport#inlineSvgImages} is set to <code>true</code>. Image encoding can be
controlled for each image individually by overwriting the predicate method {@link SvgExport#shouldInlineSvgImage}.
@param canvas The canvas to be exported.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23exportSvgAsync(CanvasComponent) Online Documentation}*/
exportSvgAsync(canvas: CanvasComponent): Promise<Element>;
/**
Serializes the given SVG element to a string.
This method adds SVG namespace and xlink namespace declarations and replaces <code>href</code> with <code>xlink:href</code>.
@param svg The SVG element to export.
@returns A string representation of the SVG.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23exportSvgString(Element) Online Documentation}*/
static exportSvgString(svg: Element): string;
/**
Fixes the <code>viewBox</code> attribute on SVG images that have been inlined for export.
If an SVG <code>image</code> tag should be inlined, but doesn't have a <code>viewBox</code> attribute, the resulting element might be rendered
with wrong dimensions. This specific implementation sets the <code>viewBox</code> attribute to the natural dimensions of the original
image.
@param importedSvg The root element of the SVG file that's referenced by the original image element.
@param originalImage The original image element that has been inlined.
@returns A Promise that will be fulfilled once the <code>viewBox</code> attribute has been set.
@see SvgExport#inlineSvgImages
@see SvgExport#exportSvgAsync
@see SvgExport#shouldInlineSvgImage
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23fixViewBoxAttribute(Element%2CSVGImageElement) Online Documentation}*/
fixViewBoxAttribute(importedSvg: Element, originalImage: SVGImageElement): Promise<any>;
/**
Prepares the imported SVG element before it is placed in the DOM as a replacement for the original image.
This method can be used to modify the element that is placed in the DOM. This specific implementation copies the values
of the x, y, width, height and preserveAspectRatio attributes from the original image element to the imported SVG
element. In addition, the SVG is wrapped in a <code>g</code> element to support the <code>transform</code> of the original image. If the original
image element has an id attribute, it is copied to the wrapping <code>g</code> element.
@param importedSvg The root element of the SVG file that's referenced by the original image element.
@param originalImage The original image element that will be replaced.
@returns The element that's actually placed in the DOM as a replacement of the <code>originalImage</code>.
@see SvgExport#inlineSvgImages
@see SvgExport#exportSvgAsync
@see SvgExport#shouldInlineSvgImage
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23prepareInlinedImage(Element%2CSVGImageElement) Online Documentation}*/
prepareInlinedImage(importedSvg: Element, originalImage: SVGImageElement): Element;
/**
Specifies whether or not a given external image should be encoded and inlined as a Base64 data-url in the resulting SVG
document.
This implementation returns the value of {@link SvgExport#encodeImagesBase64} for all values of <code>image</code>.
@param image The image element that refers to the image to be encoded.
@returns Whether the given image should be encoded.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23shouldEncodeImageBase64(SVGImageElement) Online Documentation}*/
shouldEncodeImageBase64(image: SVGImageElement): boolean;
/**
Specifies whether or not an SVG image should be inlined.
This implementation returns the value of {@link SvgExport#inlineSvgImages} for all values of <code>image</code>.
@param image The image to be inlined.
@returns Whether the given image should be inlined.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23shouldInlineSvgImage(SVGImageElement) Online Documentation}*/
shouldInlineSvgImage(image: SVGImageElement): boolean;
/**
Gets or sets the bounds of the content to export in world coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23worldBounds Online Documentation}*/
worldBounds: Rect;
/**
Gets or sets the scale for the export.
A scale of <code>1</code> preserves the original size, a scale of <code>0.5</code> results in a target image with half the original size and so
on.
This value has to be strictly greater than <code>0</code> and finite. Its default value is <code>1.0</code>
@default 1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23scale Online Documentation}*/
scale: number;
/**
Gets or sets the {@link ICanvasContext#zoom} property to use during the creation of the visualization.
In contrast to the {@link SvgExport#scale} property, which works on the output graphics, this property determines what zoom value is to be
assumed on the canvas when creating the visual. This can affect the rendering of zoom-dependent visuals, especially
level-of-detail rendering.
This value has to be strictly greater than <code>0</code>. Its default value is <code>1.0</code>
@default 1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23zoom Online Documentation}*/
zoom: number;
/**
Gets or sets the margins for the exported image.
The margins are added to the content. This means that an image with non-zero margins is larger than the {@link SvgExport#worldBounds} even if the {@link SvgExport#scale} is
<code>1.0</code>. The margins are not scaled. They are interpreted to be in units (pixels for bitmaps) for the resulting image. The
default is an empty margin instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23margins Online Documentation}*/
margins: Insets;
/**
Gets the resulting width of the target image with respect to {@link SvgExport#worldBounds}, {@link SvgExport#scale} and {@link SvgExport#margins}.
<b>Warning:</b> This property only returns the correct image width when exporting a {@link CanvasComponent} without a {@link CanvasComponent#projection}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23viewWidth Online Documentation}*/
readonly viewWidth: number;
/**
Gets the resulting height of the target image with respect to {@link SvgExport#worldBounds}, {@link SvgExport#scale} and {@link SvgExport#margins}.
<b>Warning:</b> This property only returns the correct image height when exporting a {@link CanvasComponent} without a {@link CanvasComponent#projection}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23viewHeight Online Documentation}*/
readonly viewHeight: number;
/**
Determines whether children of the {@link CanvasComponent}'s defs section that aren't known to the {@link SvgDefsManager}
should be copied to the exported SVG.
The default value is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23copyDefsElements Online Documentation}*/
copyDefsElements: boolean;
/**
Determines whether all external images should be inlined and encoded as Base64 data-urls in
{@link SvgExport#exportSvgAsync}.
Override {@link SvgExport#shouldEncodeImageBase64} to control the encoding and embedding for each image element
separately.
The default value is <code>false</code>.
@see SvgExport#exportSvgAsync
@see SvgExport#shouldEncodeImageBase64
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23encodeImagesBase64 Online Documentation}*/
encodeImagesBase64: boolean;
/**
Determines whether all SVG images should be inlined in the SVG element in {@link SvgExport#exportSvgAsync}.
Override {@link SvgExport#shouldInlineSvgImage} to control the inlining for each image separately.
The default value is <code>false</code>.
@see SvgExport#exportSvgAsync
@see SvgExport#shouldInlineSvgImage
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23inlineSvgImages Online Documentation}*/
inlineSvgImages: boolean;
/**
Determines whether the IDs of an inline SVG image should be scoped to ensure that they are unique for the given image.
This setting is enabled by default but affects only IDs of inlined images that do not contain an (optional) <a href="https://www.w3.org/TR/SVG/linking.html" target="_blank">base IRI</a>.
@see SvgExport#inlineSvgImages
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23ensureUniqueIds Online Documentation}*/
ensureUniqueIds: boolean;
/**
Determines whether the export should fail when images could not be inlined or encoded to Base64 or if it should continue
and replace those elements with a fallback image.
The default setting is <code>false</code>, i.e. a fallback image is used.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23strictMode Online Documentation}*/
strictMode: boolean;
/**
Gets or sets the CSS style sheet for the exported SVG.
If set to a non-empty string, this string will be added to the created SVG enclosed by a <code>&lt;style></code> element. If set to an
empty string, no <code>&lt;style></code> element will be added to the created SVG. If set to <code>null</code>, the {@link SvgExport} will
automatically extract all style sheets available to the document through the CSSStyleSheet API and filters all rules
that don't match any elements in the created SVG.
Note that setting this property to <code>null</code> might affect the runtime performance of the export significantly. Copying and
filtering the style rules has a runtime complexity of roughly the number of elements in the SVG times the number of CSS
rules.
Note also, that all style sheets that cannot be accessed because of CORS policies are ignored.
The default value is <code>""</code>.
@default ''
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgExport%23cssStyleSheet Online Documentation}*/
cssStyleSheet: string | null;
static $class: Class;
private $_SvgExport: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SvgExport;
}
export interface SvgDefsManager extends YObject {
}
/**
Helper class for {@link CanvasComponent} that manages the <code>defs</code> section of the SVG document.
This class is used internally by {@link CanvasComponent} and operates on instances of {@link ISvgDefsCreator}.
@see CanvasComponent#svgDefsManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgDefsManager Online Documentation}*/
export class SvgDefsManager {
/**
Initializes a new instance of the {@link SvgDefsManager} class.
@param defsElement The defs element of the svg document that will be managed by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgDefsManager Online Documentation}*/
constructor(defsElement: SVGDefsElement);
/**
Triggers the clean up for svg defs on the provided control.
@param canvas
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgDefsManager%23cleanUpDefs(CanvasComponent) Online Documentation}*/
cleanUpDefs(canvas: CanvasComponent): void;
/**
Generates an ID that is globally unique among all {@link SvgDefsManager} instances.
This ID should be used for the "id" attribute of DOM nodes.
@returns A globally unique ID.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgDefsManager%23generateUniqueDefsId() Online Documentation}*/
generateUniqueDefsId(): string;
/**
Gets or sets the interval for the timer to trigger the cleanup event.
By default this is 10 seconds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgDefsManager%23cleanUpTimerInterval Online Documentation}*/
cleanUpTimerInterval: TimeSpan;
/**
Gets or sets a soft limit for the runtime of the {@link SvgDefsManager#cleanUpDefs defs cleanup} that the method tries to respect.
By default this is 50 milliseconds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgDefsManager%23cleanUpDuration Online Documentation}*/
cleanUpDuration: TimeSpan;
/**
Gets or sets the prefix to use for the {@link SvgDefsManager#generateUniqueDefsId unique IDs}.
By default this is <code>ygc&lt;number></code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgDefsManager%23customPrefix Online Documentation}*/
customPrefix: string;
/**
Yields the <code>defs</code> element this instance is managing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgDefsManager%23defs Online Documentation}*/
readonly defs: SVGDefsElement;
static $class: Class;
private $_SvgDefsManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SvgDefsManager;
}
export interface GraphComponent extends CanvasComponent {
}
/**
A component for displaying and editing an {@link IGraph}.
This is a specialized {@link CanvasComponent} that can hold, display, and edit an {@link IGraph} instance. It
provides access to the {@link GraphComponent#graph} and {@link GraphComponent#selection} instances.
This class has bindings for the following {@link ICommand}s.
<ul>
<li>{@link ICommand#FIT_GRAPH_BOUNDS} that invokes {@link GraphComponent#fitGraphBounds}
<li>{@link ICommand#SET_CURRENT_ITEM} for setting the {@link GraphComponent#currentItem}
<li>{@link ICommand#ZOOM_TO_CURRENT_ITEM} for adjusting the {@link CanvasComponent#viewport} to the {@link GraphComponent#currentItem}
</ul>
The {@link ICommand#ZOOM_TO_CURRENT_ITEM} is enabled, if {@link CanvasComponent#navigationCommandsEnabled} is <code>true</code>.
The components that can be used to display and edit a model are discussed in the section
{@link https://docs.yworks.com/yfileshtml/#/dguide/view_graphcontrol The Component for Displaying Graphs}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent Online Documentation}*/
export class GraphComponent {
/**
Creates a new instance of {@link GraphComponent} using the <code>div</code> element that is specified by the <code>selector</code>.
If the {@link GraphComponent#graph} and {@link GraphSelection} properties are not populated externally they will be initialized with default
values upon first access. Thus this instance can be used right away without any further initialization.
All child nodes of the <code>div</code> element are added to the OverlayPanel.
@param selector The CSS selector or id for an existing <code>div</code> element. All child nodes of this element are added to the OverlayPanel.
@throws {Exception} <b>(ArgumentError)</b> if the &lt;div> element specified by the <code>selector</code> is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23GraphComponent(string) Online Documentation}*/
constructor(selector: string);
/**
Creates a new instance of {@link GraphComponent} using the <code>div</code> element that is specified by the <code>selector</code>.
If the {@link GraphComponent#graph} and {@link GraphSelection} properties are not populated externally they will be initialized with default
values upon first access. Thus this instance can be used right away without any further initialization.
All child nodes of the <code>div</code> element are added to the OverlayPanel.
@param options The parameters to pass.
@param options.selector The CSS selector or id for an existing <code>div</code> element. All child nodes of this element are added to the OverlayPanel.
@param [options.currentItem] The currently focused item. This option sets the {@link GraphComponent#currentItem} property on the created object.
@param [options.selectionIndicatorManager] The {@link GraphComponent#selectionIndicatorManager} property. This option sets the {@link GraphComponent#selectionIndicatorManager} property on the created object.
@param [options.highlightIndicatorManager] The {@link GraphComponent#highlightIndicatorManager} property. This option sets the {@link GraphComponent#highlightIndicatorManager} property on the created object.
@param [options.focusIndicatorManager] The {@link GraphComponent#focusIndicatorManager} property. This option sets the {@link GraphComponent#focusIndicatorManager} property on the created object.
@param [options.graphModelManager] The {@link GraphComponent#graphModelManager} property. This option sets the {@link GraphComponent#graphModelManager} property on the created object.
@param [options.graph] The graph that is displayed in this control. This option sets the {@link GraphComponent#graph} property on the created object.
@param [options.selection] The selection model that is used for this control. This option sets the {@link GraphComponent#selection} property on the created object.
@param [options.clipboard] The {@link GraphClipboard} associated with this control. This option sets the {@link GraphComponent#clipboard} property on the created object.
@param [options.captureAllKeyboardInput] A value indicating whether all keyboard input is captured. This option sets the {@link CanvasComponent#captureAllKeyboardInput} property on the created object.
@param [options.captureAllPointerInput] A value indicating whether all pointer input is captured. This option sets the {@link CanvasComponent#captureAllPointerInput} property on the created object.
@param [options.mouseCapture] A value that specifies whether this control receives mouse input after the mouse is dragged out of its bounding area. This option sets the {@link CanvasComponent#mouseCapture} property on the created object.
@param [options.preventFocusScrolling] A value indicating whether to prevent scrolling if the element gets focused. This option sets the {@link CanvasComponent#preventFocusScrolling} property on the created object.
@param [options.devicePixelRatio] The scaling factor for HTML5 canvas and WebGL rendering. This option sets the {@link CanvasComponent#devicePixelRatio} property on the created object.
@param [options.useGlassPane] Whether to add a 'glass pane' overlay to the {@link CanvasComponent} that acts as the source for all mouse and touch events. This option sets the {@link CanvasComponent#useGlassPane} property on the created object.
@param [options.viewportLimiter] The {@link ViewportLimiter} instance that can be used to limit the explorable region. This option sets the {@link CanvasComponent#viewportLimiter} property on the created object.
@param [options.mouseWheelBehavior] The action to perform when turning the mouse wheel. This option sets the {@link CanvasComponent#mouseWheelBehavior} property on the created object.
@param [options.centerZoomEventRecognizer] An event recognizer that determines whether zooming with the mouse wheel should zoom to the center of the view instead of the mouse location. This option sets the {@link CanvasComponent#centerZoomEventRecognizer} property on the created object.
@param [options.verticalScrollBarPolicy] The visibility policy for the vertical scrollbar. This option sets the {@link CanvasComponent#verticalScrollBarPolicy} property on the created object.
@param [options.horizontalScrollBarPolicy] The visibility policy for the horizontal scrollbar. This option sets the {@link CanvasComponent#horizontalScrollBarPolicy} property on the created object.
@param [options.animateScrollCommands] A value indicating whether {@link ICommand}s for scrolling like for example {@link ICommand#SCROLL_PAGE_DOWN}, the scrolling should be performed in animated fashion. This option sets the {@link CanvasComponent#animateScrollCommands} property on the created object.
@param [options.contentRect] The Rectangle in world coordinates that holds the contents. This option sets the {@link CanvasComponent#contentRect} property on the created object.
@param [options.fitContentViewMargins] The insets in view coordinates that should be used by the {@link CanvasComponent#fitContent} operation. This option sets the {@link CanvasComponent#fitContentViewMargins} property on the created object.
@param [options.inputMode] The single {@link IInputMode} instance that shall be installed for this canvas. This option sets the {@link CanvasComponent#inputMode} property on the created object.
@param [options.hitTestRadius] The radius of the area around the mouse in view coordinates in which an {@link IHitTestable} may lie to be considered a valid hit. This option sets the {@link CanvasComponent#hitTestRadius} property on the created object.
@param [options.hitTestRadiusTouch] The radius of the area around the touch point in view coordinates in which an {@link IHitTestable} may lie to be considered a valid hit. This option sets the {@link CanvasComponent#hitTestRadiusTouch} property on the created object.
@param [options.limitFitContentZoom] A value indicating whether the maximum zoom level for {@link CanvasComponent#fitContent} and {@link ICommand#FIT_CONTENT} is limited to 1. This option sets the {@link CanvasComponent#limitFitContentZoom} property on the created object.
@param [options.zoom] The zoom factor for this {@link CanvasComponent}. This option sets the {@link CanvasComponent#zoom} property on the created object.
@param [options.minimumZoom] The minimum zoom factor for this {@link CanvasComponent}. This option sets the {@link CanvasComponent#minimumZoom} property on the created object.
@param [options.maximumZoom] The maximum zoom factor for this {@link CanvasComponent}. This option sets the {@link CanvasComponent#maximumZoom} property on the created object.
@param [options.viewPoint] The current view point. This option sets the {@link CanvasComponent#viewPoint} property on the created object.
@param [options.center] The world coordinate at the center of the control. This option sets the {@link CanvasComponent#center} property on the created object.
@param [options.autoDrag] A property that causes the control to automatically pan the view when the mouse gets dragged outside of the bounds of the control. This option sets the {@link CanvasComponent#autoDrag} property on the created object.
@param [options.visualCaching] The policy for caching {@link Visual}s which are temporarily removed from the visual tree. This option sets the {@link CanvasComponent#visualCaching} property on the created object.
@param [options.contentGroup] The ContentGroup property. This option sets the {@link CanvasComponent#contentGroup} property on the created object.
@param [options.highlightGroup] The HighlightGroup property. This option sets the {@link CanvasComponent#highlightGroup} property on the created object.
@param [options.focusGroup] The HighlightGroup property. This option sets the {@link CanvasComponent#focusGroup} property on the created object.
@param [options.selectionGroup] The SelectionGroup property. This option sets the {@link CanvasComponent#selectionGroup} property on the created object.
@param [options.inputModeGroup] The InputModeGroup property. This option sets the {@link CanvasComponent#inputModeGroup} property on the created object.
@param [options.backgroundGroup] The BackgroundGroup property. This option sets the {@link CanvasComponent#backgroundGroup} property on the created object.
@param [options.shapeRendering] The shape-rendering type for this visual and its children. This option sets the {@link CanvasComponent#shapeRendering} property on the created object.
@param [options.imageRendering] The image-rendering type for this visual and its children. This option sets the {@link CanvasComponent#imageRendering} property on the created object.
@param [options.dragTime] The value of the drag time. This option sets the {@link CanvasComponent#dragTime} property on the created object.
@param [options.doubleClickTime] The value of the double click time. This option sets the {@link CanvasComponent#doubleClickTime} property on the created object.
@param [options.doubleTapTime] The value of the double tap time. This option sets the {@link CanvasComponent#doubleTapTime} property on the created object.
@param [options.longPressTime] The value of the long press time. This option sets the {@link CanvasComponent#longPressTime} property on the created object.
@param [options.dragSize] The area in view coordinates the mouse may stay in before a movement is considered a drag. This option sets the {@link CanvasComponent#dragSize} property on the created object.
@param [options.dragSizeTouch] The area in view coordinates the touch point may stay in before a movement is considered a drag. This option sets the {@link CanvasComponent#dragSizeTouch} property on the created object.
@param [options.doubleClickSize] The area in view coordinates the mouse needs to stay in before multiple clicks are considered multiple single clicks instead of multi-clicks. This option sets the {@link CanvasComponent#doubleClickSize} property on the created object.
@param [options.doubleTapSize] The area in view coordinates the touch pointer needs to stay in before multiple taps are considered multiple single taps instead of multi-taps. This option sets the {@link CanvasComponent#doubleTapSize} property on the created object.
@param [options.mouseWheelZoomFactor] The factor by which the zoom level changes when the mouse wheel is turned. This option sets the {@link CanvasComponent#mouseWheelZoomFactor} property on the created object.
@param [options.mouseWheelScrollFactor] A factor that controls how fast the viewport scrolls when the mouse wheel is turned. This option sets the {@link CanvasComponent#mouseWheelScrollFactor} property on the created object.
@param [options.autoDragInsets] The auto drag insets. This option sets the {@link CanvasComponent#autoDragInsets} property on the created object.
@param [options.inputModeContext] The InputModeContext property. This option sets the {@link CanvasComponent#inputModeContext} property on the created object.
@param [options.navigationCommandsEnabled] A value indicating whether navigation related command bindings are enabled. This option sets the {@link CanvasComponent#navigationCommandsEnabled} property on the created object.
@param [options.quantizeInputCoordinates] A value indicating whether input coordinates (mouse and touch) should be quantized depending on the zoom level to give nicer values. This option sets the {@link CanvasComponent#quantizeInputCoordinates} property on the created object.
@param [options.sizeChangedDetection] How size changes of this {@link CanvasComponent} will be detected. This option sets the {@link CanvasComponent#sizeChangedDetection} property on the created object.
@param [options.projection] A {@link Matrix} that is applied to the viewport. This option sets the {@link CanvasComponent#projection} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the &lt;div> element specified by the <code>selector</code> is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23GraphComponent(string) Online Documentation}*/
constructor(options: {
selector: string;
currentItem?: IModelItem | null;
selectionIndicatorManager?: SelectionIndicatorManager<IModelItem>;
highlightIndicatorManager?: HighlightIndicatorManager<IModelItem>;
focusIndicatorManager?: FocusIndicatorManager<IModelItem>;
graphModelManager?: GraphModelManager;
graph?: IGraph;
selection?: IGraphSelection;
clipboard?: GraphClipboard;
captureAllKeyboardInput?: boolean;
captureAllPointerInput?: boolean;
mouseCapture?: boolean;
preventFocusScrolling?: boolean;
devicePixelRatio?: number;
useGlassPane?: boolean;
viewportLimiter?: ViewportLimiter;
mouseWheelBehavior?: MouseWheelBehaviors | MouseWheelBehaviorsConvertible;
centerZoomEventRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
verticalScrollBarPolicy?: ScrollBarVisibility | ScrollBarVisibilityStringValues;
horizontalScrollBarPolicy?: ScrollBarVisibility | ScrollBarVisibilityStringValues;
animateScrollCommands?: boolean;
contentRect?: Rect | RectConvertible;
fitContentViewMargins?: Insets | InsetsConvertible;
inputMode?: IInputMode | null;
hitTestRadius?: number;
hitTestRadiusTouch?: number;
limitFitContentZoom?: boolean;
zoom?: number;
minimumZoom?: number;
maximumZoom?: number;
viewPoint?: Point | PointConvertible;
center?: Point | PointConvertible;
autoDrag?: boolean;
visualCaching?: VisualCachingPolicy | VisualCachingPolicyStringValues;
contentGroup?: ICanvasObjectGroup;
highlightGroup?: ICanvasObjectGroup;
focusGroup?: ICanvasObjectGroup;
selectionGroup?: ICanvasObjectGroup;
inputModeGroup?: ICanvasObjectGroup;
backgroundGroup?: ICanvasObjectGroup;
shapeRendering?: ShapeRenderingType | ShapeRenderingTypeStringValues;
imageRendering?: ImageRenderingType | ImageRenderingTypeStringValues;
dragTime?: TimeSpan | TimeSpanConvertible;
doubleClickTime?: TimeSpan | TimeSpanConvertible;
doubleTapTime?: TimeSpan | TimeSpanConvertible;
longPressTime?: TimeSpan | TimeSpanConvertible;
dragSize?: Size | SizeConvertible;
dragSizeTouch?: Size | SizeConvertible;
doubleClickSize?: Size | SizeConvertible;
doubleTapSize?: Size | SizeConvertible;
mouseWheelZoomFactor?: number;
mouseWheelScrollFactor?: number;
autoDragInsets?: Insets | InsetsConvertible;
inputModeContext?: IInputModeContext;
navigationCommandsEnabled?: boolean;
quantizeInputCoordinates?: boolean;
sizeChangedDetection?: SizeChangedDetectionMode | SizeChangedDetectionModeStringValues;
projection?: Matrix;
});
/**
Creates a new instance of the {@link GraphComponent} in the given <code>div</code> element.
If the {@link GraphComponent#graph} and {@link GraphSelection} properties are not populated externally they will be initialized with default
values upon first access. Thus this instance can be used right away without any further initialization.
All child nodes of the <code>div</code> element are added to the OverlayPanel.
@param div The existing <code>div</code> element to use for rendering the canvas into. All its child nodes are added to the OverlayPanel
@throws {Exception} <b>(ArgumentError)</b> if the <code>div</code> element is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23GraphComponent(HTMLDivElement) Online Documentation}*/
constructor(div: HTMLDivElement);
/**
Creates a new instance of the {@link GraphComponent} in the given <code>div</code> element.
If the {@link GraphComponent#graph} and {@link GraphSelection} properties are not populated externally they will be initialized with default
values upon first access. Thus this instance can be used right away without any further initialization.
All child nodes of the <code>div</code> element are added to the OverlayPanel.
@param options The parameters to pass.
@param options.div The existing <code>div</code> element to use for rendering the canvas into. All its child nodes are added to the OverlayPanel
@param [options.currentItem] The currently focused item. This option sets the {@link GraphComponent#currentItem} property on the created object.
@param [options.selectionIndicatorManager] The {@link GraphComponent#selectionIndicatorManager} property. This option sets the {@link GraphComponent#selectionIndicatorManager} property on the created object.
@param [options.highlightIndicatorManager] The {@link GraphComponent#highlightIndicatorManager} property. This option sets the {@link GraphComponent#highlightIndicatorManager} property on the created object.
@param [options.focusIndicatorManager] The {@link GraphComponent#focusIndicatorManager} property. This option sets the {@link GraphComponent#focusIndicatorManager} property on the created object.
@param [options.graphModelManager] The {@link GraphComponent#graphModelManager} property. This option sets the {@link GraphComponent#graphModelManager} property on the created object.
@param [options.graph] The graph that is displayed in this control. This option sets the {@link GraphComponent#graph} property on the created object.
@param [options.selection] The selection model that is used for this control. This option sets the {@link GraphComponent#selection} property on the created object.
@param [options.clipboard] The {@link GraphClipboard} associated with this control. This option sets the {@link GraphComponent#clipboard} property on the created object.
@param [options.captureAllKeyboardInput] A value indicating whether all keyboard input is captured. This option sets the {@link CanvasComponent#captureAllKeyboardInput} property on the created object.
@param [options.captureAllPointerInput] A value indicating whether all pointer input is captured. This option sets the {@link CanvasComponent#captureAllPointerInput} property on the created object.
@param [options.mouseCapture] A value that specifies whether this control receives mouse input after the mouse is dragged out of its bounding area. This option sets the {@link CanvasComponent#mouseCapture} property on the created object.
@param [options.preventFocusScrolling] A value indicating whether to prevent scrolling if the element gets focused. This option sets the {@link CanvasComponent#preventFocusScrolling} property on the created object.
@param [options.devicePixelRatio] The scaling factor for HTML5 canvas and WebGL rendering. This option sets the {@link CanvasComponent#devicePixelRatio} property on the created object.
@param [options.useGlassPane] Whether to add a 'glass pane' overlay to the {@link CanvasComponent} that acts as the source for all mouse and touch events. This option sets the {@link CanvasComponent#useGlassPane} property on the created object.
@param [options.viewportLimiter] The {@link ViewportLimiter} instance that can be used to limit the explorable region. This option sets the {@link CanvasComponent#viewportLimiter} property on the created object.
@param [options.mouseWheelBehavior] The action to perform when turning the mouse wheel. This option sets the {@link CanvasComponent#mouseWheelBehavior} property on the created object.
@param [options.centerZoomEventRecognizer] An event recognizer that determines whether zooming with the mouse wheel should zoom to the center of the view instead of the mouse location. This option sets the {@link CanvasComponent#centerZoomEventRecognizer} property on the created object.
@param [options.verticalScrollBarPolicy] The visibility policy for the vertical scrollbar. This option sets the {@link CanvasComponent#verticalScrollBarPolicy} property on the created object.
@param [options.horizontalScrollBarPolicy] The visibility policy for the horizontal scrollbar. This option sets the {@link CanvasComponent#horizontalScrollBarPolicy} property on the created object.
@param [options.animateScrollCommands] A value indicating whether {@link ICommand}s for scrolling like for example {@link ICommand#SCROLL_PAGE_DOWN}, the scrolling should be performed in animated fashion. This option sets the {@link CanvasComponent#animateScrollCommands} property on the created object.
@param [options.contentRect] The Rectangle in world coordinates that holds the contents. This option sets the {@link CanvasComponent#contentRect} property on the created object.
@param [options.fitContentViewMargins] The insets in view coordinates that should be used by the {@link CanvasComponent#fitContent} operation. This option sets the {@link CanvasComponent#fitContentViewMargins} property on the created object.
@param [options.inputMode] The single {@link IInputMode} instance that shall be installed for this canvas. This option sets the {@link CanvasComponent#inputMode} property on the created object.
@param [options.hitTestRadius] The radius of the area around the mouse in view coordinates in which an {@link IHitTestable} may lie to be considered a valid hit. This option sets the {@link CanvasComponent#hitTestRadius} property on the created object.
@param [options.hitTestRadiusTouch] The radius of the area around the touch point in view coordinates in which an {@link IHitTestable} may lie to be considered a valid hit. This option sets the {@link CanvasComponent#hitTestRadiusTouch} property on the created object.
@param [options.limitFitContentZoom] A value indicating whether the maximum zoom level for {@link CanvasComponent#fitContent} and {@link ICommand#FIT_CONTENT} is limited to 1. This option sets the {@link CanvasComponent#limitFitContentZoom} property on the created object.
@param [options.zoom] The zoom factor for this {@link CanvasComponent}. This option sets the {@link CanvasComponent#zoom} property on the created object.
@param [options.minimumZoom] The minimum zoom factor for this {@link CanvasComponent}. This option sets the {@link CanvasComponent#minimumZoom} property on the created object.
@param [options.maximumZoom] The maximum zoom factor for this {@link CanvasComponent}. This option sets the {@link CanvasComponent#maximumZoom} property on the created object.
@param [options.viewPoint] The current view point. This option sets the {@link CanvasComponent#viewPoint} property on the created object.
@param [options.center] The world coordinate at the center of the control. This option sets the {@link CanvasComponent#center} property on the created object.
@param [options.autoDrag] A property that causes the control to automatically pan the view when the mouse gets dragged outside of the bounds of the control. This option sets the {@link CanvasComponent#autoDrag} property on the created object.
@param [options.visualCaching] The policy for caching {@link Visual}s which are temporarily removed from the visual tree. This option sets the {@link CanvasComponent#visualCaching} property on the created object.
@param [options.contentGroup] The ContentGroup property. This option sets the {@link CanvasComponent#contentGroup} property on the created object.
@param [options.highlightGroup] The HighlightGroup property. This option sets the {@link CanvasComponent#highlightGroup} property on the created object.
@param [options.focusGroup] The HighlightGroup property. This option sets the {@link CanvasComponent#focusGroup} property on the created object.
@param [options.selectionGroup] The SelectionGroup property. This option sets the {@link CanvasComponent#selectionGroup} property on the created object.
@param [options.inputModeGroup] The InputModeGroup property. This option sets the {@link CanvasComponent#inputModeGroup} property on the created object.
@param [options.backgroundGroup] The BackgroundGroup property. This option sets the {@link CanvasComponent#backgroundGroup} property on the created object.
@param [options.shapeRendering] The shape-rendering type for this visual and its children. This option sets the {@link CanvasComponent#shapeRendering} property on the created object.
@param [options.imageRendering] The image-rendering type for this visual and its children. This option sets the {@link CanvasComponent#imageRendering} property on the created object.
@param [options.dragTime] The value of the drag time. This option sets the {@link CanvasComponent#dragTime} property on the created object.
@param [options.doubleClickTime] The value of the double click time. This option sets the {@link CanvasComponent#doubleClickTime} property on the created object.
@param [options.doubleTapTime] The value of the double tap time. This option sets the {@link CanvasComponent#doubleTapTime} property on the created object.
@param [options.longPressTime] The value of the long press time. This option sets the {@link CanvasComponent#longPressTime} property on the created object.
@param [options.dragSize] The area in view coordinates the mouse may stay in before a movement is considered a drag. This option sets the {@link CanvasComponent#dragSize} property on the created object.
@param [options.dragSizeTouch] The area in view coordinates the touch point may stay in before a movement is considered a drag. This option sets the {@link CanvasComponent#dragSizeTouch} property on the created object.
@param [options.doubleClickSize] The area in view coordinates the mouse needs to stay in before multiple clicks are considered multiple single clicks instead of multi-clicks. This option sets the {@link CanvasComponent#doubleClickSize} property on the created object.
@param [options.doubleTapSize] The area in view coordinates the touch pointer needs to stay in before multiple taps are considered multiple single taps instead of multi-taps. This option sets the {@link CanvasComponent#doubleTapSize} property on the created object.
@param [options.mouseWheelZoomFactor] The factor by which the zoom level changes when the mouse wheel is turned. This option sets the {@link CanvasComponent#mouseWheelZoomFactor} property on the created object.
@param [options.mouseWheelScrollFactor] A factor that controls how fast the viewport scrolls when the mouse wheel is turned. This option sets the {@link CanvasComponent#mouseWheelScrollFactor} property on the created object.
@param [options.autoDragInsets] The auto drag insets. This option sets the {@link CanvasComponent#autoDragInsets} property on the created object.
@param [options.inputModeContext] The InputModeContext property. This option sets the {@link CanvasComponent#inputModeContext} property on the created object.
@param [options.navigationCommandsEnabled] A value indicating whether navigation related command bindings are enabled. This option sets the {@link CanvasComponent#navigationCommandsEnabled} property on the created object.
@param [options.quantizeInputCoordinates] A value indicating whether input coordinates (mouse and touch) should be quantized depending on the zoom level to give nicer values. This option sets the {@link CanvasComponent#quantizeInputCoordinates} property on the created object.
@param [options.sizeChangedDetection] How size changes of this {@link CanvasComponent} will be detected. This option sets the {@link CanvasComponent#sizeChangedDetection} property on the created object.
@param [options.projection] A {@link Matrix} that is applied to the viewport. This option sets the {@link CanvasComponent#projection} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the <code>div</code> element is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23GraphComponent(HTMLDivElement) Online Documentation}*/
constructor(options: {
div: HTMLDivElement;
currentItem?: IModelItem | null;
selectionIndicatorManager?: SelectionIndicatorManager<IModelItem>;
highlightIndicatorManager?: HighlightIndicatorManager<IModelItem>;
focusIndicatorManager?: FocusIndicatorManager<IModelItem>;
graphModelManager?: GraphModelManager;
graph?: IGraph;
selection?: IGraphSelection;
clipboard?: GraphClipboard;
captureAllKeyboardInput?: boolean;
captureAllPointerInput?: boolean;
mouseCapture?: boolean;
preventFocusScrolling?: boolean;
devicePixelRatio?: number;
useGlassPane?: boolean;
viewportLimiter?: ViewportLimiter;
mouseWheelBehavior?: MouseWheelBehaviors | MouseWheelBehaviorsConvertible;
centerZoomEventRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
verticalScrollBarPolicy?: ScrollBarVisibility | ScrollBarVisibilityStringValues;
horizontalScrollBarPolicy?: ScrollBarVisibility | ScrollBarVisibilityStringValues;
animateScrollCommands?: boolean;
contentRect?: Rect | RectConvertible;
fitContentViewMargins?: Insets | InsetsConvertible;
inputMode?: IInputMode | null;
hitTestRadius?: number;
hitTestRadiusTouch?: number;
limitFitContentZoom?: boolean;
zoom?: number;
minimumZoom?: number;
maximumZoom?: number;
viewPoint?: Point | PointConvertible;
center?: Point | PointConvertible;
autoDrag?: boolean;
visualCaching?: VisualCachingPolicy | VisualCachingPolicyStringValues;
contentGroup?: ICanvasObjectGroup;
highlightGroup?: ICanvasObjectGroup;
focusGroup?: ICanvasObjectGroup;
selectionGroup?: ICanvasObjectGroup;
inputModeGroup?: ICanvasObjectGroup;
backgroundGroup?: ICanvasObjectGroup;
shapeRendering?: ShapeRenderingType | ShapeRenderingTypeStringValues;
imageRendering?: ImageRenderingType | ImageRenderingTypeStringValues;
dragTime?: TimeSpan | TimeSpanConvertible;
doubleClickTime?: TimeSpan | TimeSpanConvertible;
doubleTapTime?: TimeSpan | TimeSpanConvertible;
longPressTime?: TimeSpan | TimeSpanConvertible;
dragSize?: Size | SizeConvertible;
dragSizeTouch?: Size | SizeConvertible;
doubleClickSize?: Size | SizeConvertible;
doubleTapSize?: Size | SizeConvertible;
mouseWheelZoomFactor?: number;
mouseWheelScrollFactor?: number;
autoDragInsets?: Insets | InsetsConvertible;
inputModeContext?: IInputModeContext;
navigationCommandsEnabled?: boolean;
quantizeInputCoordinates?: boolean;
sizeChangedDetection?: SizeChangedDetectionMode | SizeChangedDetectionModeStringValues;
projection?: Matrix;
});
/**
Creates a new instance of the {@link GraphComponent}.
If the {@link GraphComponent#graph} and {@link GraphSelection} properties are not populated externally they will be initialized with default
values upon first access. Thus this instance can be used right away without any further initialization. This constructor
creates a new <code>div</code> element that needs to be manually added to the DOM.
@param [options] The parameters to pass.
@param [options.currentItem] The currently focused item. This option sets the {@link GraphComponent#currentItem} property on the created object.
@param [options.selectionIndicatorManager] The {@link GraphComponent#selectionIndicatorManager} property. This option sets the {@link GraphComponent#selectionIndicatorManager} property on the created object.
@param [options.highlightIndicatorManager] The {@link GraphComponent#highlightIndicatorManager} property. This option sets the {@link GraphComponent#highlightIndicatorManager} property on the created object.
@param [options.focusIndicatorManager] The {@link GraphComponent#focusIndicatorManager} property. This option sets the {@link GraphComponent#focusIndicatorManager} property on the created object.
@param [options.graphModelManager] The {@link GraphComponent#graphModelManager} property. This option sets the {@link GraphComponent#graphModelManager} property on the created object.
@param [options.graph] The graph that is displayed in this control. This option sets the {@link GraphComponent#graph} property on the created object.
@param [options.selection] The selection model that is used for this control. This option sets the {@link GraphComponent#selection} property on the created object.
@param [options.clipboard] The {@link GraphClipboard} associated with this control. This option sets the {@link GraphComponent#clipboard} property on the created object.
@param [options.captureAllKeyboardInput] A value indicating whether all keyboard input is captured. This option sets the {@link CanvasComponent#captureAllKeyboardInput} property on the created object.
@param [options.captureAllPointerInput] A value indicating whether all pointer input is captured. This option sets the {@link CanvasComponent#captureAllPointerInput} property on the created object.
@param [options.mouseCapture] A value that specifies whether this control receives mouse input after the mouse is dragged out of its bounding area. This option sets the {@link CanvasComponent#mouseCapture} property on the created object.
@param [options.preventFocusScrolling] A value indicating whether to prevent scrolling if the element gets focused. This option sets the {@link CanvasComponent#preventFocusScrolling} property on the created object.
@param [options.devicePixelRatio] The scaling factor for HTML5 canvas and WebGL rendering. This option sets the {@link CanvasComponent#devicePixelRatio} property on the created object.
@param [options.useGlassPane] Whether to add a 'glass pane' overlay to the {@link CanvasComponent} that acts as the source for all mouse and touch events. This option sets the {@link CanvasComponent#useGlassPane} property on the created object.
@param [options.viewportLimiter] The {@link ViewportLimiter} instance that can be used to limit the explorable region. This option sets the {@link CanvasComponent#viewportLimiter} property on the created object.
@param [options.mouseWheelBehavior] The action to perform when turning the mouse wheel. This option sets the {@link CanvasComponent#mouseWheelBehavior} property on the created object.
@param [options.centerZoomEventRecognizer] An event recognizer that determines whether zooming with the mouse wheel should zoom to the center of the view instead of the mouse location. This option sets the {@link CanvasComponent#centerZoomEventRecognizer} property on the created object.
@param [options.verticalScrollBarPolicy] The visibility policy for the vertical scrollbar. This option sets the {@link CanvasComponent#verticalScrollBarPolicy} property on the created object.
@param [options.horizontalScrollBarPolicy] The visibility policy for the horizontal scrollbar. This option sets the {@link CanvasComponent#horizontalScrollBarPolicy} property on the created object.
@param [options.animateScrollCommands] A value indicating whether {@link ICommand}s for scrolling like for example {@link ICommand#SCROLL_PAGE_DOWN}, the scrolling should be performed in animated fashion. This option sets the {@link CanvasComponent#animateScrollCommands} property on the created object.
@param [options.contentRect] The Rectangle in world coordinates that holds the contents. This option sets the {@link CanvasComponent#contentRect} property on the created object.
@param [options.fitContentViewMargins] The insets in view coordinates that should be used by the {@link CanvasComponent#fitContent} operation. This option sets the {@link CanvasComponent#fitContentViewMargins} property on the created object.
@param [options.inputMode] The single {@link IInputMode} instance that shall be installed for this canvas. This option sets the {@link CanvasComponent#inputMode} property on the created object.
@param [options.hitTestRadius] The radius of the area around the mouse in view coordinates in which an {@link IHitTestable} may lie to be considered a valid hit. This option sets the {@link CanvasComponent#hitTestRadius} property on the created object.
@param [options.hitTestRadiusTouch] The radius of the area around the touch point in view coordinates in which an {@link IHitTestable} may lie to be considered a valid hit. This option sets the {@link CanvasComponent#hitTestRadiusTouch} property on the created object.
@param [options.limitFitContentZoom] A value indicating whether the maximum zoom level for {@link CanvasComponent#fitContent} and {@link ICommand#FIT_CONTENT} is limited to 1. This option sets the {@link CanvasComponent#limitFitContentZoom} property on the created object.
@param [options.zoom] The zoom factor for this {@link CanvasComponent}. This option sets the {@link CanvasComponent#zoom} property on the created object.
@param [options.minimumZoom] The minimum zoom factor for this {@link CanvasComponent}. This option sets the {@link CanvasComponent#minimumZoom} property on the created object.
@param [options.maximumZoom] The maximum zoom factor for this {@link CanvasComponent}. This option sets the {@link CanvasComponent#maximumZoom} property on the created object.
@param [options.viewPoint] The current view point. This option sets the {@link CanvasComponent#viewPoint} property on the created object.
@param [options.center] The world coordinate at the center of the control. This option sets the {@link CanvasComponent#center} property on the created object.
@param [options.autoDrag] A property that causes the control to automatically pan the view when the mouse gets dragged outside of the bounds of the control. This option sets the {@link CanvasComponent#autoDrag} property on the created object.
@param [options.visualCaching] The policy for caching {@link Visual}s which are temporarily removed from the visual tree. This option sets the {@link CanvasComponent#visualCaching} property on the created object.
@param [options.contentGroup] The ContentGroup property. This option sets the {@link CanvasComponent#contentGroup} property on the created object.
@param [options.highlightGroup] The HighlightGroup property. This option sets the {@link CanvasComponent#highlightGroup} property on the created object.
@param [options.focusGroup] The HighlightGroup property. This option sets the {@link CanvasComponent#focusGroup} property on the created object.
@param [options.selectionGroup] The SelectionGroup property. This option sets the {@link CanvasComponent#selectionGroup} property on the created object.
@param [options.inputModeGroup] The InputModeGroup property. This option sets the {@link CanvasComponent#inputModeGroup} property on the created object.
@param [options.backgroundGroup] The BackgroundGroup property. This option sets the {@link CanvasComponent#backgroundGroup} property on the created object.
@param [options.shapeRendering] The shape-rendering type for this visual and its children. This option sets the {@link CanvasComponent#shapeRendering} property on the created object.
@param [options.imageRendering] The image-rendering type for this visual and its children. This option sets the {@link CanvasComponent#imageRendering} property on the created object.
@param [options.dragTime] The value of the drag time. This option sets the {@link CanvasComponent#dragTime} property on the created object.
@param [options.doubleClickTime] The value of the double click time. This option sets the {@link CanvasComponent#doubleClickTime} property on the created object.
@param [options.doubleTapTime] The value of the double tap time. This option sets the {@link CanvasComponent#doubleTapTime} property on the created object.
@param [options.longPressTime] The value of the long press time. This option sets the {@link CanvasComponent#longPressTime} property on the created object.
@param [options.dragSize] The area in view coordinates the mouse may stay in before a movement is considered a drag. This option sets the {@link CanvasComponent#dragSize} property on the created object.
@param [options.dragSizeTouch] The area in view coordinates the touch point may stay in before a movement is considered a drag. This option sets the {@link CanvasComponent#dragSizeTouch} property on the created object.
@param [options.doubleClickSize] The area in view coordinates the mouse needs to stay in before multiple clicks are considered multiple single clicks instead of multi-clicks. This option sets the {@link CanvasComponent#doubleClickSize} property on the created object.
@param [options.doubleTapSize] The area in view coordinates the touch pointer needs to stay in before multiple taps are considered multiple single taps instead of multi-taps. This option sets the {@link CanvasComponent#doubleTapSize} property on the created object.
@param [options.mouseWheelZoomFactor] The factor by which the zoom level changes when the mouse wheel is turned. This option sets the {@link CanvasComponent#mouseWheelZoomFactor} property on the created object.
@param [options.mouseWheelScrollFactor] A factor that controls how fast the viewport scrolls when the mouse wheel is turned. This option sets the {@link CanvasComponent#mouseWheelScrollFactor} property on the created object.
@param [options.autoDragInsets] The auto drag insets. This option sets the {@link CanvasComponent#autoDragInsets} property on the created object.
@param [options.inputModeContext] The InputModeContext property. This option sets the {@link CanvasComponent#inputModeContext} property on the created object.
@param [options.navigationCommandsEnabled] A value indicating whether navigation related command bindings are enabled. This option sets the {@link CanvasComponent#navigationCommandsEnabled} property on the created object.
@param [options.quantizeInputCoordinates] A value indicating whether input coordinates (mouse and touch) should be quantized depending on the zoom level to give nicer values. This option sets the {@link CanvasComponent#quantizeInputCoordinates} property on the created object.
@param [options.sizeChangedDetection] How size changes of this {@link CanvasComponent} will be detected. This option sets the {@link CanvasComponent#sizeChangedDetection} property on the created object.
@param [options.projection] A {@link Matrix} that is applied to the viewport. This option sets the {@link CanvasComponent#projection} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23GraphComponent() Online Documentation}*/
constructor(options?: {
currentItem?: IModelItem | null;
selectionIndicatorManager?: SelectionIndicatorManager<IModelItem>;
highlightIndicatorManager?: HighlightIndicatorManager<IModelItem>;
focusIndicatorManager?: FocusIndicatorManager<IModelItem>;
graphModelManager?: GraphModelManager;
graph?: IGraph;
selection?: IGraphSelection;
clipboard?: GraphClipboard;
captureAllKeyboardInput?: boolean;
captureAllPointerInput?: boolean;
mouseCapture?: boolean;
preventFocusScrolling?: boolean;
devicePixelRatio?: number;
useGlassPane?: boolean;
viewportLimiter?: ViewportLimiter;
mouseWheelBehavior?: MouseWheelBehaviors | MouseWheelBehaviorsConvertible;
centerZoomEventRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
verticalScrollBarPolicy?: ScrollBarVisibility | ScrollBarVisibilityStringValues;
horizontalScrollBarPolicy?: ScrollBarVisibility | ScrollBarVisibilityStringValues;
animateScrollCommands?: boolean;
contentRect?: Rect | RectConvertible;
fitContentViewMargins?: Insets | InsetsConvertible;
inputMode?: IInputMode | null;
hitTestRadius?: number;
hitTestRadiusTouch?: number;
limitFitContentZoom?: boolean;
zoom?: number;
minimumZoom?: number;
maximumZoom?: number;
viewPoint?: Point | PointConvertible;
center?: Point | PointConvertible;
autoDrag?: boolean;
visualCaching?: VisualCachingPolicy | VisualCachingPolicyStringValues;
contentGroup?: ICanvasObjectGroup;
highlightGroup?: ICanvasObjectGroup;
focusGroup?: ICanvasObjectGroup;
selectionGroup?: ICanvasObjectGroup;
inputModeGroup?: ICanvasObjectGroup;
backgroundGroup?: ICanvasObjectGroup;
shapeRendering?: ShapeRenderingType | ShapeRenderingTypeStringValues;
imageRendering?: ImageRenderingType | ImageRenderingTypeStringValues;
dragTime?: TimeSpan | TimeSpanConvertible;
doubleClickTime?: TimeSpan | TimeSpanConvertible;
doubleTapTime?: TimeSpan | TimeSpanConvertible;
longPressTime?: TimeSpan | TimeSpanConvertible;
dragSize?: Size | SizeConvertible;
dragSizeTouch?: Size | SizeConvertible;
doubleClickSize?: Size | SizeConvertible;
doubleTapSize?: Size | SizeConvertible;
mouseWheelZoomFactor?: number;
mouseWheelScrollFactor?: number;
autoDragInsets?: Insets | InsetsConvertible;
inputModeContext?: IInputModeContext;
navigationCommandsEnabled?: boolean;
quantizeInputCoordinates?: boolean;
sizeChangedDetection?: SizeChangedDetectionMode | SizeChangedDetectionModeStringValues;
projection?: Matrix;
});
/**
Factory method for the {@link GraphComponent#focusIndicatorManager} property.
This method will be called upon first access to the {@link GraphComponent#focusIndicatorManager} property.
@returns a new instance of {@link FocusIndicatorManager}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23createFocusIndicatorManager() Online Documentation}*/
createFocusIndicatorManager(): FocusIndicatorManager<IModelItem>;
/**
Factory method for the {@link GraphComponent#graph} property.
This method will be called upon first access to the {@link GraphComponent#graph} property.
@returns a new instance of {@link DefaultGraph}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23createGraph() Online Documentation}*/
createGraph(): IGraph;
/**
Creates the default instance to use if no custom instance has been set.
@returns A default {@link GraphClipboard} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23createGraphClipboard() Online Documentation}*/
createGraphClipboard(): GraphClipboard;
/**
Factory method for the {@link GraphComponent#graphModelManager} property.
This method will be called upon first access to the {@link GraphComponent#graphModelManager} property.
@returns a new instance of {@link GraphModelManager}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23createGraphModelManager() Online Documentation}*/
createGraphModelManager(): GraphModelManager;
/**
Factory method for the {@link GraphComponent#highlightIndicatorManager} property.
This method will be called upon first access to the {@link GraphComponent#highlightIndicatorManager} property.
@returns a new instance of {@link GraphComponent#highlightIndicatorManager}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23createHighlightIndicatorManager() Online Documentation}*/
createHighlightIndicatorManager(): HighlightIndicatorManager<IModelItem>;
/**
Factory method for the {@link GraphComponent#selection} property.
This method will be called upon first access to the {@link GraphComponent#selection} property.
@returns a new instance of {@link GraphSelection}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23createSelection() Online Documentation}*/
createSelection(): IGraphSelection;
/**
Factory method for the {@link GraphComponent#selectionIndicatorManager} property.
This method will be called upon first access to the {@link GraphComponent#selectionIndicatorManager} property.
@returns a new instance of {@link SelectionIndicatorManager}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23createSelectionIndicatorManager() Online Documentation}*/
createSelectionIndicatorManager(): SelectionIndicatorManager<IModelItem>;
/**
{@link CanvasComponent#updateContentRect Updates the content rectangle} for the {@link CanvasComponent#contentGroup} considering the provided insets and
{@link CanvasComponent#fitContent adjusts the viewport to encompass the contents}.
@param [insets=null]
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23fitGraphBounds(Insets) Online Documentation}*/
fitGraphBounds(insets?: Insets | null | InsetsConvertible): void;
/**
Runs a layout on the {@link GraphComponent#graph graph} of the {@link GraphComponent} and animates the transition.
The implementation simply delegates to a properly configured instance of {@link LayoutExecutor}. In addition to
the default configuration, it enables {@link LayoutExecutor#animateViewport}, {@link LayoutExecutor#easedAnimation}, and {@link LayoutExecutor#updateContentRect}. For more fine-grained control of the animation, use {@link LayoutExecutor}
directly.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@param layout The layout to run.
@param [morphDuration=null] The duration of the animation of the layout.
@param [layoutData=null] The layout data to apply.
@param [animateViewport=true] A value indicating whether the viewport should be animated to the new bounds of the graph.
@param [easedAnimation=true] A value indicating whether to use eased animation.
@param [updateContentRect=true] A value indicating whether the content rectangle property of the {@link CanvasComponent}
@param [allowUserInteraction=false] A value indicating whether user interaction should be allowed during the animation.
@param [automaticEdgeGrouping=true] A value indicating whether edge groups are automatically created for edges that are connected to the same port.
@param [configureTableLayout=true] A value indicating whether to automatically perform calls to {@link TableLayoutConfigurator#prepare} and {@link TableLayoutConfigurator#restore}
in order to layout table nodes.
@param [considerViewportLimiter=false] A value that controls whether strong port constraints are automatically created.
@param [fixPorts=false] A value that controls whether strong port constraints are automatically created.
@param [hideEdgesAtEdges=false] A value that controls whether edges at other edges will be hidden from the layout graph or included.
@param [portAdjustmentPolicy=PortAdjustmentPolicy.NEVER] The policy that specifies how port locations should be adjusted after a layout has been calculated.
@param [portLabelPolicies=null] Sets how {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm.
@param [targetBoundsInsets=null] the insets (in world coordinates) that will be added to the content rectangle when calculating the target viewport.
@returns A Promise that will resolve once the layout animation is finished.
@see IGraph#applyLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23morphLayout(ILayoutAlgorithm%2CTimeSpan%2CLayoutData%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2CPortAdjustmentPolicy%2CItemMapping%3CILabel%2CPortLabelPolicy%3E%2CInsets) Online Documentation}*/
morphLayout(layout: ILayoutAlgorithm, morphDuration?: TimeSpan | null | TimeSpanConvertible, layoutData?: LayoutData | null, animateViewport?: boolean, easedAnimation?: boolean, updateContentRect?: boolean, allowUserInteraction?: boolean, automaticEdgeGrouping?: boolean, configureTableLayout?: boolean, considerViewportLimiter?: boolean, fixPorts?: boolean, hideEdgesAtEdges?: boolean, portAdjustmentPolicy?: PortAdjustmentPolicy | PortAdjustmentPolicyStringValues, portLabelPolicies?: ItemMapping<ILabel, PortLabelPolicy> | null | ItemMappingConvertible<ILabel, PortLabelPolicy>, targetBoundsInsets?: Insets | null | InsetsConvertible): Promise<any>;
/**
Runs a layout on the {@link GraphComponent#graph graph} of the {@link GraphComponent} and animates the transition.
The implementation simply delegates to a properly configured instance of {@link LayoutExecutor}. In addition to
the default configuration, it enables {@link LayoutExecutor#animateViewport}, {@link LayoutExecutor#easedAnimation}, and {@link LayoutExecutor#updateContentRect}. For more fine-grained control of the animation, use {@link LayoutExecutor}
directly.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@param options The parameters to pass.
@param options.canvas The graph control.
@param options.layout The layout to run.
@param [options.morphDuration=null] The duration of the animation of the layout.
@param [options.layoutData=null] The layout data to apply.
@param [options.animateViewport=true] A value indicating whether the viewport should be animated to the new bounds of the graph.
@param [options.easedAnimation=true] A value indicating whether to use eased animation.
@param [options.updateContentRect=true] A value indicating whether the content rectangle property of the {@link CanvasComponent}
@param [options.allowUserInteraction=false] A value indicating whether user interaction should be allowed during the animation.
@param [options.automaticEdgeGrouping=true] A value indicating whether edge groups are automatically created for edges that are connected to the same port.
@param [options.configureTableLayout=true] A value indicating whether to automatically perform calls to {@link TableLayoutConfigurator#prepare} and {@link TableLayoutConfigurator#restore}
in order to layout table nodes.
@param [options.considerViewportLimiter=false] A value that controls whether strong port constraints are automatically created.
@param [options.fixPorts=false] A value that controls whether strong port constraints are automatically created.
@param [options.hideEdgesAtEdges=false] A value that controls whether edges at other edges will be hidden from the layout graph or included.
@param [options.portAdjustmentPolicy=PortAdjustmentPolicy.NEVER] The policy that specifies how port locations should be adjusted after a layout has been calculated.
@param [options.portLabelPolicies=null] Sets how {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm.
@param [options.targetBoundsInsets=null] the insets (in world coordinates) that will be added to the content rectangle when calculating the target viewport.
@returns A Promise that will resolve once the layout animation is finished.
@see IGraph#applyLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23morphLayout(ILayoutAlgorithm%2CTimeSpan%2CLayoutData%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2CPortAdjustmentPolicy%2CItemMapping%3CILabel%2CPortLabelPolicy%3E%2CInsets) Online Documentation}*/
morphLayout(options: {
layout: ILayoutAlgorithm;
morphDuration?: TimeSpan | null | TimeSpanConvertible;
layoutData?: LayoutData | null;
animateViewport?: boolean;
easedAnimation?: boolean;
updateContentRect?: boolean;
allowUserInteraction?: boolean;
automaticEdgeGrouping?: boolean;
configureTableLayout?: boolean;
considerViewportLimiter?: boolean;
fixPorts?: boolean;
hideEdgesAtEdges?: boolean;
portAdjustmentPolicy?: PortAdjustmentPolicy | PortAdjustmentPolicyStringValues;
portLabelPolicies?: ItemMapping<ILabel, PortLabelPolicy> | null | ItemMappingConvertible<ILabel, PortLabelPolicy>;
targetBoundsInsets?: Insets | null | InsetsConvertible;
}): Promise<any>;
/**
Gets or sets the currently focused item.
The focused item is shown with a faint dotted border by default.
This is a concept related to
{@link GraphComponent#selection}. The main difference is that there can be at most one focused item, while multiple items can be selected at once.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23currentItem Online Documentation}*/
currentItem: IModelItem | null;
/**
Adds the given listener for the <code>CurrentItemChanged</code> event that occurs when the {@link GraphComponent#currentItem} property has changed its value.
@param listener The listener to add.
@see GraphComponent#removeCurrentItemChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23CurrentItemChanged Online Documentation}*/
addCurrentItemChangedListener(listener: (sender: any, evt: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>CurrentItemChanged</code> event that occurs when the {@link GraphComponent#currentItem} property has changed its value.
@param listener The listener to remove.
@see GraphComponent#addCurrentItemChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23CurrentItemChanged Online Documentation}*/
removeCurrentItemChangedListener(listener: (sender: any, evt: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the {@link GraphComponent#selectionIndicatorManager} property.
If the field has not yet been initialized upon first access, the factory method {@link GraphComponent#createSelectionIndicatorManager}
will be called.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23selectionIndicatorManager Online Documentation}*/
selectionIndicatorManager: SelectionIndicatorManager<IModelItem>;
/**
* Gets or sets the {@link GraphComponent#highlightIndicatorManager} property.
* If the field has not yet been initialized upon first access, the factory method {@link GraphComponent#createHighlightIndicatorManager}
* will be called.
*
* The following example shows how to highlight items:
*
* <pre>
* // Add a highlight
* graphComponent.highlightIndicatorManager.addHighlight(node)
*
* // Determine whether an item is highlighted
* graphComponent.highlightIndicatorManager.selectionModel.isSelected(node)
*
* // Remove it again
* graphComponent.highlightIndicatorManager.removeHighlight(node)
*
* // Clear all highlights
* graphComponent.highlightIndicatorManager.clearHighlights()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23highlightIndicatorManager Online Documentation}
*/
highlightIndicatorManager: HighlightIndicatorManager<IModelItem>;
/**
Gets or sets the {@link GraphComponent#focusIndicatorManager} property.
If the field has not yet been initialized upon first access, the factory method {@link GraphComponent#createFocusIndicatorManager}
will be called.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23focusIndicatorManager Online Documentation}*/
focusIndicatorManager: FocusIndicatorManager<IModelItem>;
/**
Gets or sets the {@link GraphComponent#graphModelManager} property.
If the field has not yet been initialized upon first access, the factory method {@link GraphComponent#createGraphModelManager}
will be called.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23graphModelManager Online Documentation}*/
graphModelManager: GraphModelManager;
/**
Gets or sets the graph that is displayed in this control.
Some {@link IInputMode}s (notably {@link GraphViewerInputMode} and
{@link GraphEditorInputMode}) may retain references to items in the old graph until certain interactions, so it
sometimes can be a good idea to re-create and replace the input mode when changing to graph so the old graph reference
is eligible for garbage collection.
When changing the graph to one that uses the same {@link IModelItem} instances as the old one (e.g. when using
{@link FilteredGraphWrapper}), the current selection is retained.
If the property has not been explicitly initialized upon first access, the factory method {@link GraphComponent#createGraph}
will be called.
@see GraphComponent#createGraph
@see GraphComponent#addGraphChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23graph Online Documentation}*/
graph: IGraph;
/**
Adds the given listener for the <code>GraphChanged</code> event that occurs when the {@link GraphComponent#graph} property has been changed.
@param listener The listener to add.
@see GraphComponent#removeGraphChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23GraphChanged Online Documentation}*/
addGraphChangedListener(listener: (sender: any, evt: ItemEventArgs<IGraph>) => void): void;
/**
Removes the given listener for the <code>GraphChanged</code> event that occurs when the {@link GraphComponent#graph} property has been changed.
@param listener The listener to remove.
@see GraphComponent#addGraphChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23GraphChanged Online Documentation}*/
removeGraphChangedListener(listener: (sender: any, evt: ItemEventArgs<IGraph>) => void): void;
/**
* Gets or sets the selection model that is used for this control.
* The selection model instance needs to be adjusted, normally if the {@link GraphComponent#graph} instance is changed. If the field has not yet been
* initialized upon first access, the factory method {@link GraphComponent#createSelection} will be called.
*
* A concept related to selection is the
* {@link GraphComponent#currentItem}, which represent the currently focused item. This is also shown in a selection-like fashion.
*
* The selection can be either queried for selected items or manipulated to select or deselect items programmatically. The
* following example shows a few things that can be done:
*
* <pre>
* // Determine whether a node is selected
* const selected = graphComponent.selection.isSelected(node)
*
* // Deselect an edge
* graphComponent.selection.setSelected(edge, false)
*
* // Iterate over selected nodes
* for (const n of graphComponent.selection.selectedNodes) {
*   // ...
* }
* </pre>
*
* @see GraphComponent#currentItem
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23selection Online Documentation}
*/
selection: IGraphSelection;
/**
Gets or sets the {@link GraphClipboard} associated with this control.
If no instance has been configured with this control, a default implementation will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphComponent%23clipboard Online Documentation}*/
clipboard: GraphClipboard;
static $class: Class;
private $_GraphComponent: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphComponent;
}
export interface GraphOverviewComponent extends CanvasComponent {
}
/**
A specialized {@link CanvasComponent} that shows an overview of an {@link IGraph} contained in another {@link GraphOverviewComponent#graphComponent}.
This component uses a {@link GraphOverviewComponent#graphVisualCreator} to display the graph in a preview mode and an {@link GraphOverviewComponent#overviewInputMode} to allow for easily navigating in the {@link GraphOverviewComponent#graphComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent Online Documentation}*/
export class GraphOverviewComponent {
/**
Creates a new instance of {@link GraphOverviewComponent} using the <code>div</code> element that is specified by the <code>selector</code> and
that is optionally bound to the provided {@link GraphOverviewComponent#graphComponent}.
If no <code>graphComponent</code> is provided, in order for this instance to be useful, either the {@link GraphOverviewComponent#graphComponent} or the {@link GraphOverviewComponent#graph} property need to be set.
@param [graphComponent=null] The {@link GraphOverviewComponent#graphComponent} to bind this instance to. If omitted, the property needs to be set to initialize the component.
@param selector The CSS selector or id for an existing <code>div</code> element
@throws {Exception} <b>(ArgumentError)</b> if the &lt;div> element specified by the <code>selector</code> is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23GraphOverviewComponent(string%2CGraphComponent) Online Documentation}*/
constructor(selector: string, graphComponent?: GraphComponent | null);
/**
Creates a new instance of {@link GraphOverviewComponent} using the <code>div</code> element that is specified by the <code>selector</code> and
that is optionally bound to the provided {@link GraphOverviewComponent#graphComponent}.
If no <code>graphComponent</code> is provided, in order for this instance to be useful, either the {@link GraphOverviewComponent#graphComponent} or the {@link GraphOverviewComponent#graph} property need to be set.
@param options The parameters to pass.
@param [options.graphComponent=null] The {@link GraphOverviewComponent#graphComponent} to bind this instance to. If omitted, the property needs to be set to initialize the component.
@param options.selector The CSS selector or id for an existing <code>div</code> element
@param [options.renderMode] A value indicating whether the graph is rendered using SVG rendering. This option sets the {@link GraphOverviewComponent#renderMode} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the &lt;div> element specified by the <code>selector</code> is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23GraphOverviewComponent(string%2CGraphComponent) Online Documentation}*/
constructor(options: {
selector: string;
graphComponent?: GraphComponent | null;
renderMode?: RenderModes | RenderModesStringValues;
});
/**
Creates a new instance of {@link GraphOverviewComponent} using the <code>div</code> element that is optionally bound to the
provided {@link GraphOverviewComponent#graphComponent}.
If no <code>graphComponent</code> is provided, in order for this instance to be useful, either the {@link GraphOverviewComponent#graphComponent} or the {@link GraphOverviewComponent#graph} property need to be set.
@param div The div element to use for holding the canvas.
@param [graphComponent=null] The optional graph control to {@link GraphOverviewComponent#graphComponent bind} this instance to. If omitted, the property needs to be set to initialize the
component.
@throws {Exception} <b>(ArgumentError)</b> if the <code>div</code> element is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23GraphOverviewComponent(HTMLDivElement%2CGraphComponent) Online Documentation}*/
constructor(div: HTMLDivElement, graphComponent?: GraphComponent | null);
/**
Creates a new instance of {@link GraphOverviewComponent} using the <code>div</code> element that is optionally bound to the
provided {@link GraphOverviewComponent#graphComponent}.
If no <code>graphComponent</code> is provided, in order for this instance to be useful, either the {@link GraphOverviewComponent#graphComponent} or the {@link GraphOverviewComponent#graph} property need to be set.
@param options The parameters to pass.
@param options.div The div element to use for holding the canvas.
@param [options.graphComponent=null] The optional graph control to {@link GraphOverviewComponent#graphComponent bind} this instance to. If omitted, the property needs to be set to initialize the
component.
@param [options.renderMode] A value indicating whether the graph is rendered using SVG rendering. This option sets the {@link GraphOverviewComponent#renderMode} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the <code>div</code> element is already used by another component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23GraphOverviewComponent(HTMLDivElement%2CGraphComponent) Online Documentation}*/
constructor(options: {
div: HTMLDivElement;
graphComponent?: GraphComponent | null;
renderMode?: RenderModes | RenderModesStringValues;
});
/**
Creates a new instance that is optionally bound to the provided {@link GraphOverviewComponent#graphComponent}.
This constructor creates a new {@link CanvasComponent#div} element that needs to be added to the DOM manually. If no <code>graphComponent</code> is provided, in
order for this instance to be useful, either the {@link GraphOverviewComponent#graphComponent} or the {@link GraphOverviewComponent#graph} property need to be set.
@param [graphComponent=null] The {@link GraphOverviewComponent#graphComponent} to bind this instance to. If omitted, the property needs to be set to initialize the component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23GraphOverviewComponent(GraphComponent) Online Documentation}*/
constructor(graphComponent?: GraphComponent | null);
/**
Creates a new instance that is optionally bound to the provided {@link GraphOverviewComponent#graphComponent}.
This constructor creates a new {@link CanvasComponent#div} element that needs to be added to the DOM manually. If no <code>graphComponent</code> is provided, in
order for this instance to be useful, either the {@link GraphOverviewComponent#graphComponent} or the {@link GraphOverviewComponent#graph} property need to be set.
@param options The parameters to pass.
@param [options.graphComponent=null] The {@link GraphOverviewComponent#graphComponent} to bind this instance to. If omitted, the property needs to be set to initialize the component.
@param [options.renderMode] A value indicating whether the graph is rendered using SVG rendering. This option sets the {@link GraphOverviewComponent#renderMode} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23GraphOverviewComponent(GraphComponent) Online Documentation}*/
constructor(options: {
graphComponent?: GraphComponent | null;
renderMode?: RenderModes | RenderModesStringValues;
});
/**
Prevents cursor modification from Projection.
@param cursor
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23compensateCursorForProjection(Cursor) Online Documentation}*/
compensateCursorForProjection(cursor: Cursor | null | CursorConvertible): Cursor | null;
/**
Factory method that creates the {@link IVisualCreator} that renders the preview of the graph in a HTML canvas
element.
@param graph The graph to draw.
@returns An {@link IVisualCreator} that creates HTML canvas rendering.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23createCanvasVisualCreator(IGraph) Online Documentation}*/
createCanvasVisualCreator(graph: IGraph): IVisualCreator;
/**
Factory method that creates the {@link IVisualCreator} that renders the preview of the graph in SVG.
@param graph The graph to draw.
@returns An {@link IVisualCreator} that creates SVG.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23createSvgVisualCreator(IGraph) Online Documentation}*/
createSvgVisualCreator(graph: IGraph): IVisualCreator;
/**
Factory method that creates the {@link IVisualCreator} that renders the preview of the graph in a WebGL canvas
element.
@param graph The graph to draw.
@returns An {@link IVisualCreator} that creates WebGL canvas rendering.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23createWebGLVisualCreator(IGraph) Online Documentation}*/
createWebGLVisualCreator(graph: IGraph): IVisualCreator;
/**
Gets or sets a value indicating whether the graph is rendered using SVG rendering.
@default RenderModes.CANVAS
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23renderMode Online Documentation}*/
renderMode: RenderModes;
/**
Gets or sets the {@link GraphOverviewSvgVisualCreator} that is used to render the overview.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23graphVisualCreator Online Documentation}*/
graphVisualCreator: IVisualCreator | null;
/**
Gets or sets the {@link OverviewInputMode} that is used to navigate the client control.
Setting this property will automatically update the {@link OverviewInputMode#canvasComponent} property of the new {@link OverviewInputMode}.
This property is merely an alias for the {@link CanvasComponent#inputMode} property. Setting either this property or the {@link CanvasComponent#inputMode} property has the same effect if
the input mode is an {@link OverviewInputMode}. On the other hand, if an input mode is set that is not an
{@link OverviewInputMode}, then the value of this property is <code>null</code>.
<b>Warning:</b> If the value of this property is <code>null</code>, the {@link GraphOverviewSvgVisualCreator} is not notified about changes of
the {@link GraphOverviewComponent#graphComponent}s {@link GraphComponent#graph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23overviewInputMode Online Documentation}*/
overviewInputMode: OverviewInputMode | null;
/**
Gets or sets the graph that is rendered in the overview.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23graph Online Documentation}*/
graph: IGraph | null;
/**
Gets or sets the client component, that will be used to retrieve the graph from and that will be used for navigating.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewComponent%23graphComponent Online Documentation}*/
graphComponent: GraphComponent | null;
static $class: Class;
private $_GraphOverviewComponent: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphOverviewComponent;
}
export interface GridInfo extends YObject {
}
/**
This class describes a grid by providing the grid's origin as well as the horizontal and vertical spacing between the
grid points.
It is used by the {@link IGridConstraintProvider} as well as by the <code>GridSnapResultVisualCreator&lt;T></code>
@see IGridConstraintProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridInfo Online Documentation}*/
export class GridInfo {
/**
Creates a new instance using the given horizontal and vertical grid spacing as well as the given origin.
@param horizontalSpacing The horizontal spacing between the grid points.
@param verticalSpacing The vertical spacing between the grid points.
@param origin The canonical origin of the grid.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridInfo%23GridInfo(number%2Cnumber%2CPoint) Online Documentation}*/
constructor(horizontalSpacing: number, verticalSpacing: number, origin: Point | PointConvertible);
/**
Creates a new instance using "25" as the grid spacing.
@param [spacing=25] The horizontal and vertical spacing between the grid points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridInfo%23GridInfo(number) Online Documentation}*/
constructor(spacing?: number);
/**
Gets or sets the horizontal spacing between grid points in the world coordinate system.
@default 25
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridInfo%23horizontalSpacing Online Documentation}*/
horizontalSpacing: number;
/**
Gets or sets the vertical spacing between grid points in the world coordinate system.
@default 25
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridInfo%23verticalSpacing Online Documentation}*/
verticalSpacing: number;
/**
Gets or sets the canonical origin of the grid.
@default '0,0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridInfo%23origin Online Documentation}*/
origin: Point;
static $class: Class;
private $_GridInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GridInfo;
}
/**
The values of this enum describe to which grid elements an object shall snap.
It can be used for the GraphSnapContext's GridSnapType property to determine what {@link SnapResult}s shall be
created for the set {@link IGridConstraintProvider}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridSnapTypes Online Documentation}*/
export enum GridSnapTypes {
/**
There is no grid snapping at all.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridSnapTypes%23NONE Online Documentation}*/
NONE,
/**
Elements shall be snapped to horizontal grid lines.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridSnapTypes%23HORIZONTAL_LINES Online Documentation}*/
HORIZONTAL_LINES,
/**
Elements shall be snapped to vertical grid lines.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridSnapTypes%23VERTICAL_LINES Online Documentation}*/
VERTICAL_LINES,
/**
Elements shall be snapped to horizontal and vertical grid lines.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridSnapTypes%23LINES Online Documentation}*/
LINES,
/**
Elements shall be snapped to grid points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridSnapTypes%23GRID_POINTS Online Documentation}*/
GRID_POINTS,
/**
Elements shall be snapped to horizontal and vertical grid lines as well as to grid points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridSnapTypes%23ALL Online Documentation}*/
ALL
}
type GridSnapTypesStringValues = "none" | "horizontal-lines" | "vertical-lines" | "lines" | "grid-points" | "all";
/** Automatic conversion to {@link GridSnapTypes}. */
type GridSnapTypesConvertible = GridSnapTypesStringValues | GridSnapTypesStringValues[];
export interface IHitTester<T> {
}
/**
An interface that can enumerate hits of a given type for a certain position in world coordinates.
Type parameter <code>T</code>: the type of hits returned by the enumerator
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHitTester Online Documentation}*/
export abstract class IHitTester<T> {
/**
Creates an implementation of the interface {@link IHitTester} by using the given function as implementation for its {@link IHitTester#enumerateHits}
method.
@param enumerateHits A function for {@link IHitTester}'s single abstract method {@link IHitTester#enumerateHits}.
@returns An instance of the {@link IHitTester} interface based on the given function.*/
static create<T>(enumerateHits: (context: IInputModeContext, location: Point) => IEnumerable<T>): IHitTester<T>;
/**
Yields an enumerable that enumerates the hits for a given world coordinate.
@param context The context in which to perform the hit testing.
@param location the coordinates in the world coordinate system
@returns an enumerable that yields hits for the given coordinates
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHitTester%23enumerateHits(IInputModeContext%2CPoint) Online Documentation}*/
abstract enumerateHits(context: IInputModeContext, location: Point): IEnumerable<T>;
static $class: Class;
private $_IHitTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is IHitTester<T>;
}
export interface INodeHitTester extends IHitTester<INode> {
}
/**
This is a flagging interface for instances of {@link IHitTester} that enumerate over a collection of {@link INode INodes}.
An implementation of this interface can be queried using the lookup of a {@link GraphComponent}
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeHitTester Online Documentation}*/
export abstract class INodeHitTester {
/**
Creates an implementation of the interface {@link INodeHitTester} by using the given function as implementation for
its {@link INodeHitTester#enumerateHits} method.
@param enumerateHits A function for {@link INodeHitTester}'s single abstract method {@link INodeHitTester#enumerateHits}.
@returns An instance of the {@link INodeHitTester} interface based on the given function.*/
static create(enumerateHits: (context: IInputModeContext, location: Point) => IEnumerable<INode>): INodeHitTester;
static $class: Class;
private $_INodeHitTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeHitTester;
}
export interface ILabelHitTester extends IHitTester<ILabel> {
}
/**
This is a flagging interface for instances of {@link IHitTester} that enumerate over a collection of {@link ILabel ILabels}.
An implementation of this interface can be queried using the lookup of a {@link GraphComponent}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelHitTester Online Documentation}*/
export abstract class ILabelHitTester {
/**
Creates an implementation of the interface {@link ILabelHitTester} by using the given function as implementation
for its {@link ILabelHitTester#enumerateHits} method.
@param enumerateHits A function for {@link ILabelHitTester}'s single abstract method {@link ILabelHitTester#enumerateHits}.
@returns An instance of the {@link ILabelHitTester} interface based on the given function.*/
static create(enumerateHits: (context: IInputModeContext, location: Point) => IEnumerable<ILabel>): ILabelHitTester;
static $class: Class;
private $_ILabelHitTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelHitTester;
}
export interface ILabelOwnerHitTester extends IHitTester<ILabelOwner> {
}
/**
This is a flagging interface for instances of {@link IHitTester} that enumerate over a collection of {@link ILabelOwner}.
An implementation of this interface can be queried using the lookup of a {@link GraphComponent}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelOwnerHitTester Online Documentation}*/
export abstract class ILabelOwnerHitTester {
/**
Creates an implementation of the interface {@link ILabelOwnerHitTester} by using the given function as
implementation for its {@link ILabelOwnerHitTester#enumerateHits} method.
@param enumerateHits A function for {@link ILabelOwnerHitTester}'s single abstract method
{@link ILabelOwnerHitTester#enumerateHits}.
@returns An instance of the {@link ILabelOwnerHitTester} interface based on the given function.*/
static create(enumerateHits: (context: IInputModeContext, location: Point) => IEnumerable<ILabelOwner>): ILabelOwnerHitTester;
static $class: Class;
private $_ILabelOwnerHitTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelOwnerHitTester;
}
export interface IPortHitTester extends IHitTester<IPort> {
}
/**
This is a flagging interface for instances of {@link IHitTester} that enumerate over a collection of {@link IPort IPorts}.
An implementation of this interface can be queried using the lookup of a {@link GraphComponent}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortHitTester Online Documentation}*/
export abstract class IPortHitTester {
/**
Creates an implementation of the interface {@link IPortHitTester} by using the given function as implementation for
its {@link IPortHitTester#enumerateHits} method.
@param enumerateHits A function for {@link IPortHitTester}'s single abstract method {@link IPortHitTester#enumerateHits}.
@returns An instance of the {@link IPortHitTester} interface based on the given function.*/
static create(enumerateHits: (context: IInputModeContext, location: Point) => IEnumerable<IPort>): IPortHitTester;
static $class: Class;
private $_IPortHitTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortHitTester;
}
export interface IEdgeHitTester extends IHitTester<IEdge> {
}
/**
This is a flagging interface for instances of {@link IHitTester} that enumerate over a collection of {@link IEdge IEdges}.
An implementation of this interface can be queried using the lookup of a {@link GraphComponent}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeHitTester Online Documentation}*/
export abstract class IEdgeHitTester {
/**
Creates an implementation of the interface {@link IEdgeHitTester} by using the given function as implementation for
its {@link IEdgeHitTester#enumerateHits} method.
@param enumerateHits A function for {@link IEdgeHitTester}'s single abstract method {@link IEdgeHitTester#enumerateHits}.
@returns An instance of the {@link IEdgeHitTester} interface based on the given function.*/
static create(enumerateHits: (context: IInputModeContext, location: Point) => IEnumerable<IEdge>): IEdgeHitTester;
static $class: Class;
private $_IEdgeHitTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeHitTester;
}
export interface IBendHitTester extends IHitTester<IBend> {
}
/**
This is a flagging interface for instances of {@link IHitTester} that enumerate over a collection of {@link IBend IBends}.
An implementation of this interface can be queried using the lookup of a {@link GraphComponent}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendHitTester Online Documentation}*/
export abstract class IBendHitTester {
/**
Creates an implementation of the interface {@link IBendHitTester} by using the given function as implementation for
its {@link IBendHitTester#enumerateHits} method.
@param enumerateHits A function for {@link IBendHitTester}'s single abstract method {@link IBendHitTester#enumerateHits}.
@returns An instance of the {@link IBendHitTester} interface based on the given function.*/
static create(enumerateHits: (context: IInputModeContext, location: Point) => IEnumerable<IBend>): IBendHitTester;
static $class: Class;
private $_IBendHitTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IBendHitTester;
}
export interface ICanvasContext extends ILookup {
}
/**
This interface describes the context for many operations performed on items in a {@link ICanvasContext#canvasComponent}.
This interface extends the {@link ILookup} interface to provide {@link ILookup#lookup dynamic lookup capability}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasContext Online Documentation}*/
export abstract class ICanvasContext {
/**
Provides a default immutable {@link ICanvasContext} with the given values.
If the values for zoom or hitTestRadius are not specified, the values will be obtained from the given canvas.
@param canvas the canvas
@param [zoom=0] the zoom level
@param [hitTestRadius=-1] the hit test radius
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasContext%23createCanvasContext(CanvasComponent%2Cnumber%2Cnumber) Online Documentation}*/
static createCanvasContext(canvas: CanvasComponent | null, zoom?: number, hitTestRadius?: number): ICanvasContext;
/**
Provides a default immutable {@link ICanvasContext} with the given values.
If the values for zoom or hitTestRadius are not specified, the values will be obtained from the given canvas.
@param options The parameters to pass.
@param options.canvas the canvas
@param [options.zoom=0] the zoom level
@param [options.hitTestRadius=-1] the hit test radius
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasContext%23createCanvasContext(CanvasComponent%2Cnumber%2Cnumber) Online Documentation}*/
static createCanvasContext(options: {
canvas: CanvasComponent | null;
zoom?: number;
hitTestRadius?: number;
}): ICanvasContext;
/**
Adds an element to the defs section of the document, if it has not been added yet.
@param defsSupport The instance that is used to create and update the element and to query if the element is still referenced
@returns The id of the element with which it can be referenced using an url reference.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasContext%23getDefsId(ISvgDefsCreator) Online Documentation}*/
abstract getDefsId(defsSupport: ISvgDefsCreator): string;
/**
Gets the current zoom level that is the ratio of world coordinates to screen pixels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasContext%23zoom Online Documentation}*/
abstract readonly zoom: number;
/**
Gets the radius for hit tests and marquee selection tests in world coordinates.
In the default implementations this property automatically distinguishes whether to refer to {@link CanvasComponent#hitTestRadius} (for mouse), or {@link CanvasComponent#hitTestRadiusTouch} (for
touch).
This value already takes the zoom level into account.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasContext%23hitTestRadius Online Documentation}*/
abstract readonly hitTestRadius: number;
/**
Gets the Canvas that is associated with this context.
This value may be null in those cases where no control can be associated with the current context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasContext%23canvasComponent Online Documentation}*/
abstract readonly canvasComponent: CanvasComponent | null;
/**
Provides a default immutable context with a hit radius of 3 and a zoom level of 1.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasContext%23DEFAULT Online Documentation}*/
static DEFAULT: ICanvasContext;
static $class: Class;
private $_ICanvasContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ICanvasContext;
}
export interface ICanvasObject {
}
/**
This is the interface implemented by the basic building blocks of a scene graph in a {@link CanvasComponent}.
The interface is not meant to be implemented by application programmers. Instead, instances implementing this interface
can be retrieved from the factory methods in the canvas control.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject Online Documentation}*/
export abstract class ICanvasObject {
/**
Moves this instance exactly behind the reference item in the referenced item's group rendering list, so that it will be
rendered directly above the given reference.
This method returns the current instance so that calls can be chained, e.g. <code>object.Above(other).Raise()</code>.
Observe that this instance will move into the referenced item's group if needed.
@param reference
@returns <code>this</code> so that calls can be chained.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23above(ICanvasObject) Online Documentation}*/
abstract above(reference: ICanvasObject): ICanvasObject;
/**
Moves this instance exactly before the reference item in the referenced item's group rendering list, so that it will be
rendered directly behind the given reference.
This method returns the current instance so that calls can be chained, e.g. <code>object.Below(other).Lower()</code>.
Observe that this instance will move into the referenced item's group if needed.
@param reference
@returns <code>this</code> so that calls can be chained.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23below(ICanvasObject) Online Documentation}*/
abstract below(reference: ICanvasObject): ICanvasObject;
/**
Moves this instance one step closer to the beginning of the group rendering list, so that it will be rendered behind its
current predecessor.
This method returns the current instance so that calls can be conveniently chained, e.g. <code>object.Lower().Lower()</code>
@returns <code>this</code> so that calls can be chained.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23lower() Online Documentation}*/
abstract lower(): ICanvasObject;
/**
Moves this instance one step closer to the end of the group rendering list, so that it will be rendered on top of its
current successor.
This method returns the current instance so that calls can be conveniently chained, e.g. <code>object.Raise().Raise().Raise()</code>
@returns <code>this</code> so that calls can be chained.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23raise() Online Documentation}*/
abstract raise(): ICanvasObject;
/**
Removes this item from the scene graph it currently belongs to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23remove() Online Documentation}*/
abstract remove(): void;
/**
Moves this instance to the beginning of the group rendering list, so that it will be rendered behind all other items in
the same group.
This method returns the current instance so that calls can be conveniently chained, e.g.
<code>object.ToBack().Raise().Raise()</code>
@returns <code>this</code> so that calls can be chained.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23toBack() Online Documentation}*/
abstract toBack(): ICanvasObject;
/**
Moves this instance to the end of the group rendering list, so that it will be rendered in front of all other items in
the same group.
This method returns the current instance so that calls can be conveniently chained, e.g. <code>object.ToFront().Lower()</code>
@returns <code>this</code> so that calls can be chained.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23toFront() Online Documentation}*/
abstract toFront(): ICanvasObject;
/**
Gets or sets the visibility state of this instance.
If an ICanvasObject is not visible, it will not be considered for hit tests.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23visible Online Documentation}*/
abstract visible: boolean;
/**
Gets or sets a flag that indicates whether {@link ICanvasObject} needs to be updated during the next call to
{@link CanvasComponent#invalidate}.
Note that it is up to the implementation of the {@link ICanvasObjectDescriptor} to determine the actual dirty state
of this instance. {@link ICanvasObjectDescriptor#isDirty} can use any strategy to indicate dirtiness. The {@link CanvasComponent}
will reset this flag during {@link CanvasComponent#invalidate} to <code>false</code> if the visual is created or updated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23dirty Online Documentation}*/
abstract dirty: boolean;
/**
Gets or sets the parent group of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23group Online Documentation}*/
abstract group: ICanvasObjectGroup | null;
/**
Gets or sets the user object associated with this instance.
The user object will be used by the {@link ICanvasObject#descriptor} during rendering, hit testing, and bounds calculation
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23userObject Online Documentation}*/
abstract userObject: any | null;
/**
Gets or sets the descriptor instance that interprets the {@link ICanvasObject#userObject} during rendering, hit testing, and bounds calculation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23descriptor Online Documentation}*/
abstract descriptor: ICanvasObjectDescriptor;
/**
Gets the next item in the list of rendered items that is owned by this element's {@link ICanvasObject#group}.
The returned item will be rendered after this item is rendered. If this item is the last one in the list, this method
returns <code>null</code>.
@returns <code>null</code> if this item is last in the rendering list of its group or the next item otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23nextSibling Online Documentation}*/
abstract readonly nextSibling: ICanvasObject | null;
/**
Gets the previous item in the list of rendered items that is owned by this element's {@link ICanvasObject#group}.
The returned item will be rendered directly before this item is rendered. If this item is the first one in the list,
this method returns <code>null</code>.
@returns <code>null</code> if this item is first in the rendering list of its group or the previous item otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObject%23previousSibling Online Documentation}*/
abstract readonly previousSibling: ICanvasObject | null;
static $class: Class;
private $_ICanvasObject: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ICanvasObject;
}
export interface ICanvasObjectDescriptor {
}
/**
Central interface used by the {@link CanvasComponent} to render elements from the scene graph.
Implementations will be queried using the user object obtained from {@link ICanvasObject#userObject} for the corresponding implementations of the
various aspects of the user object. Implementations of this interface need to be provided by the application programmer
to {@link ICanvasObjectGroup}'s {@link ICanvasObjectGroup#addChild} method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor Online Documentation}*/
export abstract class ICanvasObjectDescriptor {
/**
Returns an implementation of {@link IBoundsProvider} that can determine the visible bounds of the rendering of the
user object.
This method may always return the same instance. By contract, clients will not cache instances returned, but will always
call this method before the methods on the instance will be called. This contract enables the use of the fly-weight
design pattern. Implementations of this class can always return the same instance and reconfigure this instance upon
each call to this method.
@param forUserObject the user object to query the bounds for
@returns An implementation of {@link IBoundsProvider} to determine the bounds of the visualization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23getBoundsProvider(Object) Online Documentation}*/
abstract getBoundsProvider(forUserObject: any | null): IBoundsProvider;
/**
Returns an implementation of {@link IHitTestable} that can determine whether the rendering of the user object has
been hit at a given coordinate.
This method may always return the same instance. By contract, clients will not cache instances returned, but will always
call this method before the methods on the instance will be called. This contract enables the use of the fly-weight
design pattern. Implementations of this class can always return the same instance and reconfigure this instance upon
each call to this method.
@param forUserObject the user object to do the hit testing for
@returns An implementation of {@link IHitTestable} to determine visibility.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23getHitTestable(Object) Online Documentation}*/
abstract getHitTestable(forUserObject: any | null): IHitTestable;
/**
Returns an implementation of {@link IVisibilityTestable} that can determine if the rendering of the user object
would be visible in a given context.
This method may always return the same instance. By contract clients will not cache instances returned but will always
call this method before the methods on the instance will be called. This contract enables the use of the fly-weight
design pattern. Implementations of this class can always return the same instance and reconfigure this instance upon
each call to this method.
@param forUserObject the user object to query visibility for
@returns An implementation of {@link IVisibilityTestable} to determine visibility.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23getVisibilityTestable(Object) Online Documentation}*/
abstract getVisibilityTestable(forUserObject: any | null): IVisibilityTestable;
/**
Returns an implementation of {@link IVisualCreator} that will create the {@link Visual} tree for the user
object.
This method may always return the same instance. By contract, clients will not cache instances returned, but will always
call this method before the methods on the instance will be called. This contract enables the use of the fly-weight
design pattern. Implementations of this class can always return the same instance and reconfigure this instance upon
each call to this method.
@param forUserObject the user object to create a Visual for
@returns An implementation of {@link IVisualCreator} to create or update the visualization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23getVisualCreator(Object) Online Documentation}*/
abstract getVisualCreator(forUserObject: any | null): IVisualCreator;
/**
Determines whether the given canvas object is deemed dirty and needs updating.
@param context The context that will be used for the update.
@param canvasObject The object to check.
@returns Whether an update is needed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23isDirty(ICanvasContext%2CICanvasObject) Online Documentation}*/
abstract isDirty(context: ICanvasContext, canvasObject: ICanvasObject): boolean;
/**
An implementation that returns an {@link ICanvasObjectDescriptor} that does nothing.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23VOID Online Documentation}*/
static VOID: ICanvasObjectDescriptor;
/**
An implementation that treats {@link ICanvasObject#userObject} as an instance of {@link IVisualCreator} and considers the {@link ICanvasObject#dirty} flag.
This descriptor tries to cast the {@link ICanvasObject#userObject} to {@link IVisualCreator} and use this instance to handle the visual
representation of the {@link ICanvasObject}.
If the {@link ICanvasObject#userObject} does not implement {@link IVisualCreator} nothing will be displayed.
This descriptor will respect the {@link ICanvasObject#dirty} flag.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23DYNAMIC_DIRTY_INSTANCE Online Documentation}*/
static DYNAMIC_DIRTY_INSTANCE: ICanvasObjectDescriptor;
/**
A descriptor that treats {@link ICanvasObject#userObject} as an instance of {@link IVisualCreator} and ignores the {@link ICanvasObject#dirty} flag.
This descriptor tries to cast the {@link ICanvasObject#userObject} to {@link IVisualCreator} and use this instance to handle the visual
representation of the {@link ICanvasObject}.
If the {@link ICanvasObject#userObject} does not implement {@link IVisualCreator} nothing will be displayed.
This descriptor will always {@link IVisualCreator#updateVisual update the visual} and thus ignore the {@link ICanvasObject#dirty} flag.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23ALWAYS_DIRTY_INSTANCE Online Documentation}*/
static ALWAYS_DIRTY_INSTANCE: ICanvasObjectDescriptor;
/**
A descriptor that treats {@link ICanvasObject#userObject} as an instance of {@link ILookup} to obtain suitable interface implementations and
considers the {@link ICanvasObject#dirty} flag.
This descriptor tries to cast the {@link ICanvasObject#userObject} to {@link ILookup} and obtain the appropriate interface implementations for its
methods from it.
If the {@link ICanvasObject#userObject} does not implement {@link ILookup} nothing will be displayed.
This descriptor will respect the {@link ICanvasObject#dirty} flag.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23DYNAMIC_DIRTY_LOOKUP Online Documentation}*/
static DYNAMIC_DIRTY_LOOKUP: ICanvasObjectDescriptor;
/**
A descriptor that treats {@link ICanvasObject#userObject} as an instance of {@link ILookup} to obtain suitable interface implementations and
ignores the {@link ICanvasObject#dirty} flag.
This descriptor tries to cast the {@link ICanvasObject#userObject} to {@link ILookup} and obtain the appropriate interface implementations for its
methods from it.
If the {@link ICanvasObject#userObject} does not implement {@link ILookup} nothing will be displayed.
This descriptor will always {@link IVisualCreator#updateVisual update the Visual} and thus ignore the {@link ICanvasObject#dirty} flag.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23ALWAYS_DIRTY_LOOKUP Online Documentation}*/
static ALWAYS_DIRTY_LOOKUP: ICanvasObjectDescriptor;
/**
A descriptor that treats {@link ICanvasObject#userObject} as {@link Visual} to use directly.
If the {@link ICanvasObject#userObject} is not a {@link Visual} nothing will be displayed.
This descriptor ignores the {@link ICanvasObject#dirty} flag.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectDescriptor%23VISUAL Online Documentation}*/
static VISUAL: ICanvasObjectDescriptor;
static $class: Class;
private $_ICanvasObjectDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ICanvasObjectDescriptor;
}
export interface ICanvasObjectGroup extends ICanvasObject, IListEnumerable<ICanvasObject> {
}
/**
This is the interface implemented by the composite building blocks of a scene graph in a {@link CanvasComponent}.
It is meant to group multiple {@link ICanvasObject} instances as a logic unit. The interface is not meant to be
implemented by application programmers. Instances implementing this interface can be retrieved from the factory methods
in the canvas control.
This interface extends the {@link IListEnumerable} interface to allow for easy iteration and determination of the size of the group. Note that
the default implementation is based on a linked list, thus index-based iteration is not recommended.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectGroup Online Documentation}*/
export abstract class ICanvasObjectGroup {
/**
Adds a child element to the scene graph as a child of this group.
The descriptor will be queried for the various rendering related implementations for the given <code>userObject</code> at rendering
time.
@param userObject The user object to associate with this child element. This object will be passed to the <code>descriptor</code>'s methods.
@param [descriptor=null] An implementation of the {@link ICanvasObjectDescriptor} interface that will be passed the <code>userObject</code> to provide the
various implementations that are used during rendering. If no descriptor is passed, {@link ICanvasObjectDescriptor#ALWAYS_DIRTY_INSTANCE}
is used by the default implementation.
@returns A handle that can be used to control the rendering order and to later {@link ICanvasObject#remove remove} the element from the scene graph, again.
@see ICanvasObjectDescriptor
@see ICanvasObjectDescriptor#ALWAYS_DIRTY_INSTANCE
@see ICanvasObjectDescriptor#DYNAMIC_DIRTY_INSTANCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectGroup%23addChild(Object%2CICanvasObjectDescriptor) Online Documentation}*/
abstract addChild(userObject: any | null, descriptor?: ICanvasObjectDescriptor | null): ICanvasObject;
/**
Adds a new {@link ICanvasObjectGroup} to this group in the current scene graph.
This can be used to build groups of {@link ICanvasObject} instances that can be moved within the scene graph or
whose {@link ICanvasObject#visible visibility} can be controlled easily. The group will have a <code>null</code> {@link ICanvasObject#userObject}.
@returns A handle to the newly created group that can be used to control the rendering order and to later {@link ICanvasObject#remove remove} the group from
the scene graph, again.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectGroup%23addGroup() Online Documentation}*/
abstract addGroup(): ICanvasObjectGroup;
/**
* Gets the first item in the rendering list of this group or <code>null</code>.
* This is the item that will be rendered first, i.e. behind all of the other items in the rendering list. In order to
* iterate over all immediate children, use the following idiom:
*
* <pre>
* for (
*   let canvasObject = canvasObjectGroup.firstChild;
*   canvasObject !== null;
*   canvasObject = canvasObject.nextSibling
* ) {
*   // do something with canvasObject
* }
* </pre>
*
* @see IListEnumerable
* @see IEnumerable#getEnumerator
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectGroup%23firstChild Online Documentation}
*/
abstract readonly firstChild: ICanvasObject | null;
/**
* Gets the last item in the rendering list of this group or <code>null</code>.
* This is the item that will be on top of all other items in the list. In order to iterate over all immediate children in
* reverse order, use the following idiom:
*
* <pre>
* for (
*   let canvasObject = canvasObjectGroup.lastChild;
*   canvasObject !== null;
*   canvasObject = canvasObject.previousSibling
* ) {
*   // do something with canvasObject
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectGroup%23lastChild Online Documentation}
*/
abstract readonly lastChild: ICanvasObject | null;
static $class: Class;
private $_ICanvasObjectGroup: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ICanvasObjectGroup;
}
/**
Enumeration that controls whether properties should be written as elements or attributes.
@see GraphMLAttribute#writeAsAttribute
@see XamlSerializer
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlAttributeWritePolicy Online Documentation}*/
export enum XamlAttributeWritePolicy {
/**
Specifies that the write mode of a member should be determined automatically according to the current property value and
the presence of {@link ValueSerializer}s or implementations of {@link IMarkupExtensionConverter}s.
Usually, an implementation would write the member as attribute if the content can somehow be (directly or indirectly)
converted to a valid XML attribute content (i.e. a simple string)
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlAttributeWritePolicy%23AUTO Online Documentation}*/
AUTO,
/**
Never write as a XML attribute, even if a possible conversion exists.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlAttributeWritePolicy%23NEVER Online Documentation}*/
NEVER,
/**
Always write as XML attribute.
This can be used for special cases where the default implementation does not detect a valid conversion, but some part of
the (de)serialization is handled manually.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlAttributeWritePolicy%23ALWAYS Online Documentation}*/
ALWAYS
}
/** Automatic conversion to {@link XamlAttributeWritePolicy}. */
type XamlAttributeWritePolicyStringValues = "auto" | "never" | "always";
/**
Enumeration that controls whether resources should be shared or not.
@see GraphMLAttribute#shareable
@see XamlSerializer
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSharingPolicy Online Documentation}*/
export enum GraphMLSharingPolicy {
/**
Specifies that the share mode of a member should be determined automatically according to the current property value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSharingPolicy%23AUTO Online Documentation}*/
AUTO,
/**
Never share the object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSharingPolicy%23NEVER Online Documentation}*/
NEVER,
/**
Always share the object.
This will also always place the object in the resources section, even if it only occurs once in the document.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSharingPolicy%23ALWAYS Online Documentation}*/
ALWAYS
}
/** Automatic conversion to {@link GraphMLSharingPolicy}. */
type GraphMLSharingPolicyStringValues = "auto" | "never" | "always";
/**
Specifies the visibility a member has in GraphML.
@see GraphMLAttribute#visibility
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLMemberVisibility Online Documentation}*/
export enum GraphMLMemberVisibility {
/**
The member is treated according to the normal XAML serialization rules.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLMemberVisibility%23VISIBLE Online Documentation}*/
VISIBLE,
/**
Only the content of a member value is read or written, without an enclosing explicit container object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLMemberVisibility%23CONTENT Online Documentation}*/
CONTENT,
/**
The member is ignored by GraphML.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLMemberVisibility%23HIDDEN Online Documentation}*/
HIDDEN
}
/** Automatic conversion to {@link GraphMLMemberVisibility}. */
type GraphMLMemberVisibilityStringValues = "visible" | "content" | "hidden";
/**
Enumeration that controls how undefined value is treated in (de-)serialization.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndefinedHandling Online Documentation}*/
export enum UndefinedHandling {
/**
Explicitly read and write undefined values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndefinedHandling%23EXPLICIT Online Documentation}*/
EXPLICIT,
/**
Throw an exception when undefined values are encountered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndefinedHandling%23THROW Online Documentation}*/
THROW,
/**
Treat undefined values like null values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndefinedHandling%23AS_NULL Online Documentation}*/
AS_NULL,
/**
Treat undefined values as default values.
Default values are never written.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndefinedHandling%23AS_DEFAULT Online Documentation}*/
AS_DEFAULT
}
/** Automatic conversion to {@link UndefinedHandling}. */
type UndefinedHandlingStringValues = "explicit" | "throw" | "as-null" | "as-default";
/**
Custom attribute to control specific aspects of GraphML deserialization.
The specialized {@link XamlSerializer} will evaluate this attribute to override its built in serialization rules.
The main use cases for this attribute are to override object reference sharing, configure consistent member naming and
handle legacy serialization.
@see XamlSerializer
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute Online Documentation}*/
export function GraphMLAttribute(): GraphMLAttribute;
interface GraphMLAttribute extends Attribute {
/**
* Creates a new instance.
* @param [options] The parameters to pass.
* @param [options.name] Gets or sets the name of the attributed member.
* This will be used as the XML element or XML attribute name.
*
* This property is ignored if the attribute targets a type declaration.
*
* @param [options.xmlNamespace] Gets or sets the XML namespace URI for the member where this attribute is used.
* This will be used as the XML namespace URI for the XML element or XML attribute.
*
* This property is ignored if the attribute targets a type declaration.
*
* @param [options.writeEnclosingTag] Gets or sets whether an enclosing tag for the property itself is written.
* If <code>true</code>, the resulting XML will look like
*
* <pre>
* <ParentObjectName.Name>
*    <PropertyTypeName>
*       content
*    </PropertyTypeName>
* </ParentObjectName.Name>
* </pre>
*
* Default value is <code>true</code>.
*
* This property is ignored if the attribute targets a type declaration.
*
* @param [options.writeAsAttribute] Gets or sets whether the member should be written as an XML element or as an XML attribute.
* Default value is {@link XamlAttributeWritePolicy#AUTO}.
*
* This property is ignored if the attribute targets a type declaration.
*
* @param [options.shareable] Gets or sets whether the object reference identity of the property values should be kept intact.
* Default value is {@link GraphMLSharingPolicy#AUTO}.
*
* This property is ignored if the attribute targets a type declaration.
*
* @param [options.visibility] Gets or sets how a member should be visible to GraphML.
* Default value is {@link GraphMLMemberVisibility#VISIBLE}.
*
* This property is ignored if the attribute targets a type declaration.
*
* @param [options.contentProperty] Specifies the property of a class that is used as the content property in XAML.
* The value of the content property can appear in XAML as a direct child of the class element, without the need to wrap it
* into a <code>&lt;Class.Property></code> element.
*
* This property is ignored if the attribute targets a member declaration.
*
* @param [options.valueSerializer] Gets or sets the type of the {@link GraphMLAttribute#valueSerializer} implementation.
* @param [options.markupExtensionConverter] Gets or sets a custom {@link IMarkupExtensionConverter} for attribute serialization.
* The specialized {@link XamlSerializer} will evaluate this attribute to override its built in mechanism to lookup
* an {@link IMarkupExtensionConverter} for an object. The specified type must implement
* {@link IMarkupExtensionConverter}.
*
* @param [options.singletonContainers] Types that should be searched for static field or property members of the annotated type.
* When a type is annotated with this attribute, all types in the {@link GraphMLAttribute#singletonContainers} array are searched for public static fields or
* properties that have the annotated type. The GraphML writer will write these members as static extensions instead of the
* usual serialization. This property is ignored if the attribute targets a member declaration.
*
* Using the following annotation
*
* <pre>
* const ContainedType = BaseClass('ContainedType', {
*   $meta() {
*     return [
*       GraphMLAttribute().init({ singletonContainers: [PoolHeaderLabelModelExtension.$class] })
*     ]
*   },
*
*   constructor: function() {}
* })
*
* const ContainingType = BaseClass('ContainingType', {
*   $static: {
*     INSTANCE: {
*       $meta() {
*         return [TypeAttribute(ContainedType.$class)]
*       },
*       value: new ContainedType()
*     }
*   }
* })
* </pre>
*
* has the effect that all references to field <code>ContainedTypeSingleton</code> will be written as static extensions, e.g.
*
* <pre>
* {x:Static myNSDeclarationPrefix:ContainingType.ContainedTypeSingleton}
* </pre>
*
* @param [options.defaultValue] Gets or sets the default value of the property this attribute is bound to.
* This property is ignored if the attribute targets a type declaration.
*
*/
init(options?: {
name?: string;
xmlNamespace?: string;
writeEnclosingTag?: boolean;
writeAsAttribute?: XamlAttributeWritePolicy | XamlAttributeWritePolicyStringValues;
shareable?: GraphMLSharingPolicy | GraphMLSharingPolicyStringValues;
visibility?: GraphMLMemberVisibility | GraphMLMemberVisibilityStringValues;
contentProperty?: string;
valueSerializer?: Class;
markupExtensionConverter?: Class;
singletonContainers?: Class[];
defaultValue?: any;
}): GraphMLAttribute;
/**
Gets or sets the name of the attributed member.
This will be used as the XML element or XML attribute name.
This property is ignored if the attribute targets a type declaration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23name Online Documentation}*/
name: string | null;
/**
Gets or sets the XML namespace URI for the member where this attribute is used.
This will be used as the XML namespace URI for the XML element or XML attribute.
This property is ignored if the attribute targets a type declaration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23xmlNamespace Online Documentation}*/
xmlNamespace: string | null;
/**
* Gets or sets whether an enclosing tag for the property itself is written.
* If <code>true</code>, the resulting XML will look like
*
* <pre>
* <ParentObjectName.Name>
*    <PropertyTypeName>
*       content
*    </PropertyTypeName>
* </ParentObjectName.Name>
* </pre>
*
* Default value is <code>true</code>.
*
* This property is ignored if the attribute targets a type declaration.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23writeEnclosingTag Online Documentation}
*/
writeEnclosingTag: boolean;
/**
Gets or sets whether the member should be written as an XML element or as an XML attribute.
Default value is {@link XamlAttributeWritePolicy#AUTO}.
This property is ignored if the attribute targets a type declaration.
@see XamlAttributeWritePolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23writeAsAttribute Online Documentation}*/
writeAsAttribute: XamlAttributeWritePolicy;
/**
Gets or sets whether the object reference identity of the property values should be kept intact.
Default value is {@link GraphMLSharingPolicy#AUTO}.
This property is ignored if the attribute targets a type declaration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23shareable Online Documentation}*/
shareable: GraphMLSharingPolicy;
/**
Gets or sets how a member should be visible to GraphML.
Default value is {@link GraphMLMemberVisibility#VISIBLE}.
This property is ignored if the attribute targets a type declaration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23visibility Online Documentation}*/
visibility: GraphMLMemberVisibility;
/**
Specifies the property of a class that is used as the content property in XAML.
The value of the content property can appear in XAML as a direct child of the class element, without the need to wrap it
into a <code>&lt;Class.Property></code> element.
This property is ignored if the attribute targets a member declaration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23contentProperty Online Documentation}*/
contentProperty: string | null;
/**
Gets or sets the type of the {@link GraphMLAttribute#valueSerializer} implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23valueSerializer Online Documentation}*/
valueSerializer: Class | null;
/**
Gets or sets a custom {@link IMarkupExtensionConverter} for attribute serialization.
The specialized {@link XamlSerializer} will evaluate this attribute to override its built in mechanism to lookup
an {@link IMarkupExtensionConverter} for an object. The specified type must implement
{@link IMarkupExtensionConverter}.
@see XamlSerializer
@see IMarkupExtensionConverter
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23markupExtensionConverter Online Documentation}*/
markupExtensionConverter: Class | null;
/**
* Types that should be searched for static field or property members of the annotated type.
* When a type is annotated with this attribute, all types in the {@link GraphMLAttribute#singletonContainers} array are searched for public static fields or
* properties that have the annotated type. The GraphML writer will write these members as static extensions instead of the
* usual serialization. This property is ignored if the attribute targets a member declaration.
*
* Using the following annotation
*
* <pre>
* const ContainedType = BaseClass('ContainedType', {
*   $meta() {
*     return [
*       GraphMLAttribute().init({ singletonContainers: [PoolHeaderLabelModelExtension.$class] })
*     ]
*   },
*
*   constructor: function() {}
* })
*
* const ContainingType = BaseClass('ContainingType', {
*   $static: {
*     INSTANCE: {
*       $meta() {
*         return [TypeAttribute(ContainedType.$class)]
*       },
*       value: new ContainedType()
*     }
*   }
* })
* </pre>
*
* has the effect that all references to field <code>ContainedTypeSingleton</code> will be written as static extensions, e.g.
*
* <pre>
* {x:Static myNSDeclarationPrefix:ContainingType.ContainedTypeSingleton}
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23singletonContainers Online Documentation}
*/
singletonContainers: Class[] | null;
/**
Gets or sets the default value of the property this attribute is bound to.
This property is ignored if the attribute targets a type declaration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLAttribute%23defaultValue Online Documentation}*/
defaultValue: any | null;
}
export module GraphMLAttribute {
const $class: Class;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
function isInstance(o: any): o is GraphMLAttribute;
}
export interface InputModeBase extends YObject, IInputMode {
/**
@inheritdoc*/
priority: number;
} /**
Abstract base class implementation of an {@link IInputMode} that can be used concurrently with other {@link IInputMode}
implementations if its {@link InputModeBase#exclusive} property is <code>true</code>.
This class provides methods which facilitate implementing a custom {@link IInputMode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase Online Documentation}*/
export abstract class InputModeBase {
/**
Initializes a new instance of the {@link InputModeBase} class.
@param exclusive Iff this mode should be {@link InputModeBase#exclusive}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23InputModeBase(boolean) Online Documentation}*/
constructor(exclusive: boolean);
/**
Initializes a new instance of the {@link InputModeBase} class that is {@link InputModeBase#exclusive} by default.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23InputModeBase() Online Documentation}*/
constructor();
/**
Cancels the editing of this mode by first disabling and afterwards reenabling the installed {@link InputModeBase#controller}.
This implementation disables and reenables the {@link InputModeBase#controller} and fires the {@link InputModeBase#onCanceled} event afterwards.
Classes that need to perform clean-up should override the {@link InputModeBase#onConcurrencyControllerDeactivated}
method and invoke <b>base.OnConcurrencyControllerDeactivated</b> as the last statement.
@see InputModeBase#onCanceled
@see InputModeBase#addCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23cancel() Online Documentation}*/
cancel(): void;
/**
Determines whether this instance can request the mutex.
This will return true, iff a {@link InputModeBase#controller} has been injected and the helper allows requesting the mutex.
@returns Whether a call to {@link InputModeBase#requestMutex} would currently succeed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23canRequestMutex() Online Documentation}*/
canRequestMutex(): boolean;
/**
Creates an implementation of {@link IInputModeContext} that is specific to this mode.
This is needed for events that are triggered by this InputMode to signalize where the event is coming from. A common
usage for this is for example in {@link IHitTestable#isHit} calls.
The result can be given to {@link IHitTestable#isHit} or {@link CanvasComponent#hitElementsAt} to let
implementations of {@link IHitTestable} get access to the current {@link IInputModeContext}. This can be done
by looking up {@link IInputModeContext} from the {@link ICanvasContext}'s {@link ILookup#lookup} method or
simply downcasting.
@returns A readily configured {@link IInputModeContext}.
@see CanvasComponent#hitElementsAt
@see IHitTestable#isHit
@see IInputModeContext
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23createChildInputModeContext() Online Documentation}*/
createChildInputModeContext(): IInputModeContext;
/**
Helper method that yields a suitably configured {@link InputModeEventArgs} for this input mode.
@param context An input mode context that is available in the {@link InputModeEventArgs}.
Can be <code>null</code> in which case a new context for this instance is created automatically.
@returns An input mode event argument that is configured for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23createInputModeEventArgs(IInputModeContext) Online Documentation}*/
createInputModeEventArgs(context: IInputModeContext | null): InputModeEventArgs;
/**
Determines whether this instance owns the input mutex.
If no {@link InputModeBase#controller} had been associated with this instance upon the call to {@link InputModeBase#requestMutex} this method will
return <code>false</code>.
@returns Whether this instance has the input mutex.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23hasMutex() Online Documentation}*/
hasMutex(): boolean;
/**
Performs one-time initialization of this instance.
This method should not be invoked by subclasses. This will be done automatically upon first {@link InputModeBase#install installation} of this mode.
This code will be executed only once per instance. The {@link InputModeBase#inputModeContext} property will be <code>null</code> when this code is executed. This method
should not be used to install this mode into a specific canvas. Subclasses should always call <code>base.Initialize()</code> first.
@see InputModeBase#install
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23initialize() Online Documentation}*/
initialize(): void;
/**
Installs this mode into a {@link CanvasComponent} using the provided {@link IInputModeContext}.
Subclasses should override this method and call<code>base.Install(context)</code>, first. One-time initialization should be performed
in the {@link InputModeBase#initialize} method. This implementation will call the {@link InputModeBase#initialize}
method the first time this mode gets installed. The initialization will be surrounded by calls to {@link InputModeBase#onInitializing}
and {@link InputModeBase#onInitialized} to trigger the corresponding events.
@param context the context to install this mode into
@param controller The {@link InputModeBase#controller} for this mode.
@see InputModeBase#inputModeContext
@see CanvasComponent#inputMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Event dispatch method that will dispatch the {@link InputModeBase#addCanceledListener Canceled}
event.
This method is called by subclasses when the editing process has been {@link InputModeBase#cancel}ed.
@param evt The event arguments.
@see InputModeBase#initialize
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23onCanceled(InputModeEventArgs) Online Documentation}*/
onCanceled(evt: InputModeEventArgs): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Event dispatch method that will dispatch the {@link InputModeBase#addInitializedListener Initialized}
event.
This method will be called by {@link InputModeBase#install} for the first installation directly after {@link InputModeBase#initialize}
has returned.
@param evt The event arguments.
@see InputModeBase#initialize
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23onInitialized(InputModeEventArgs) Online Documentation}*/
onInitialized(evt: InputModeEventArgs): void;
/**
Event dispatch method that will dispatch the {@link InputModeBase#addInitializingListener Initializing}
event.
This method will be called by {@link InputModeBase#install} for the first installation directly before {@link InputModeBase#initialize}
is invoked.
@param evt The event arguments.
@see InputModeBase#initialize
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23onInitializing(InputModeEventArgs) Online Documentation}*/
onInitializing(evt: InputModeEventArgs): void;
/**
Called when this instance obtains the mutex.
If no {@link InputModeBase#controller} had been associated with this instance upon the call to {@link InputModeBase#requestMutex} this method will
not be called.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23onMutexObtained() Online Documentation}*/
onMutexObtained(): void;
/**
Called when this instance released the mutex.
If no {@link InputModeBase#controller} had been associated with this instance upon the call to {@link InputModeBase#requestMutex} this method will
not be called.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23onMutexReleased() Online Documentation}*/
onMutexReleased(): void;
/**
Event dispatch method that will dispatch the {@link InputModeBase#addStoppedListener Stopped}
event.
This method will be called by {@link InputModeBase#tryStop} or can be called by subclasses during a {@link InputModeBase#tryStop}
operation.
@param evt The event arguments.
@see InputModeBase#tryStop
@see InputModeBase#addStoppedListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23onStopped(InputModeEventArgs) Online Documentation}*/
onStopped(evt: InputModeEventArgs): void;
/**
Releases previously captured keyboard input.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23releaseKeyboard() Online Documentation}*/
releaseKeyboard(): void;
/**
Releases the mutex that is currently owned by this instance.
This will trigger the {@link InputModeBase#onMutexReleased} method if this instance previously owned the mutex.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23releaseMutex() Online Documentation}*/
releaseMutex(): void;
/**
Releases previously captured mouse and touch input.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23releasePointer() Online Documentation}*/
releasePointer(): void;
/**
Requests capturing of all keyboard input that happens in the {@link CanvasComponent}.
This sets the CaptureAllKeyboardInput property to
<code>true</code>, which will prevent visuals inside the canvas from stealing key strokes.
@see InputModeBase#releaseKeyboard
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23requestKeyboard() Online Documentation}*/
requestKeyboard(): void;
/**
Requests the mutex from the current {@link InputModeBase#controller}.
This implementation will call the {@link InputModeBase#onMutexObtained} method if the mutex has been successfully
obtained. If the mutex cannot be obtained this method will trigger an exception.
@throws {Exception} <b>(Error)</b> If the mutex could not be obtained.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23requestMutex() Online Documentation}*/
requestMutex(): void;
/**
Requests capturing of all mouse and touch input that happens in the {@link CanvasComponent}.
This sets the {@link CanvasComponent#captureAllPointerInput} property to <code>true</code>, which will prevent visuals inside the canvas from stealing mouse input.
@see InputModeBase#releasePointer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23requestPointer() Online Documentation}*/
requestPointer(): void;
/**
* Tries to stop the editing.
* This implementation returns if this instance does not currently {@link InputModeBase#hasMutex have the input mutex}.
*
* This implementation calls {@link InputModeBase#onStopped}. Subclasses that want to prevent a stop or need to do
* special clean up should do this here if the base class implementation of {@link InputModeBase#tryStop} returns
* <code>true</code>. A typical implementation should follow this idiom if it wants to stop:
*
* <pre>
* if (mode.tryStop()) {
*   // do cleanup
*   return true
* } else {
*   return false
* }
* </pre>
*
* and if it doesn't want to stop simply return <code>false</code> immediately.
*
* Calling this method raises the {@link InputModeBase#addStoppedListener Stopped} event.
*
* @returns <code>true</code> iff this instance does not {@link InputModeBase#hasMutex own the mutex}.
* @see InputModeBase#onStopped
* @see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23tryStop() Online Documentation}
*/
tryStop(): boolean;
/**
Uninstalls this mode from the canvas.
Subclasses should always call <code>base.Uninstall(context)</code> as the last statement.
@param context The context to remove this mode from. This is the same instance that has been passed to
{@link InputModeBase#install}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Adds the given listener for the <code>Initializing</code> event that occurs before the call to {@link InputModeBase#initialize}
Clients can use this event to perform configuration of the instance shortly before the initialization begins.
@param listener The listener to add.
@see InputModeBase#install
@see InputModeBase#removeInitializingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23Initializing Online Documentation}*/
addInitializingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Initializing</code> event that occurs before the call to
{@link InputModeBase#initialize}
Clients can use this event to perform configuration of the instance shortly before the initialization begins.
@param listener The listener to remove.
@see InputModeBase#install
@see InputModeBase#addInitializingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23Initializing Online Documentation}*/
removeInitializingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Initialized</code> event that occurs after the call to {@link InputModeBase#initialize}
Clients can use this event to perform post configuration steps of the instance shortly after the initialization has been
performed.
@param listener The listener to add.
@see InputModeBase#install
@see InputModeBase#removeInitializedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23Initialized Online Documentation}*/
addInitializedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Initialized</code> event that occurs after the call to
{@link InputModeBase#initialize}
Clients can use this event to perform post configuration steps of the instance shortly after the initialization has been
performed.
@param listener The listener to remove.
@see InputModeBase#install
@see InputModeBase#addInitializedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23Initialized Online Documentation}*/
removeInitializedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Canceled</code> event that occurs after the call to {@link InputModeBase#cancel}
Clients can use this event to perform post cleanup steps of the instance shortly after the mode has been canceled.
@param listener The listener to add.
@see InputModeBase#onCanceled
@see InputModeBase#cancel
@see InputModeBase#tryStop
@see InputModeBase#addStoppedListener
@see InputModeBase#removeCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23Canceled Online Documentation}*/
addCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Canceled</code> event that occurs after the call to {@link InputModeBase#cancel}
Clients can use this event to perform post cleanup steps of the instance shortly after the mode has been canceled.
@param listener The listener to remove.
@see InputModeBase#onCanceled
@see InputModeBase#cancel
@see InputModeBase#tryStop
@see InputModeBase#addStoppedListener
@see InputModeBase#addCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23Canceled Online Documentation}*/
removeCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Stopped</code> event that occurs after the call to {@link InputModeBase#tryStop}
Clients can use this event to perform post cleanup steps of the instance shortly after the mode has been stopped.
@param listener The listener to add.
@see InputModeBase#onStopped
@see InputModeBase#cancel
@see InputModeBase#tryStop
@see InputModeBase#addCanceledListener
@see InputModeBase#removeStoppedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23Stopped Online Documentation}*/
addStoppedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Stopped</code> event that occurs after the call to {@link InputModeBase#tryStop}
Clients can use this event to perform post cleanup steps of the instance shortly after the mode has been stopped.
@param listener The listener to remove.
@see InputModeBase#onStopped
@see InputModeBase#cancel
@see InputModeBase#tryStop
@see InputModeBase#addCanceledListener
@see InputModeBase#addStoppedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23Stopped Online Documentation}*/
removeStoppedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Gets whether this mode is currently installed in a {@link CanvasComponent}.
@see InputModeBase#install
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23installed Online Documentation}*/
readonly installed: boolean;
/**
Gets the context instance this mode is currently installed in or <code>null</code> if this instance is not installed.
Note that this instance should not be passed down to subordinate modes or instances. Instead a corresponding context
that has this instance set as the {@link IInputModeContext#parentInputMode} should be used. One such instance can be obtained through helper method
{@link InputModeBase#createChildInputModeContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23inputModeContext Online Documentation}*/
readonly inputModeContext: IInputModeContext | null;
/**
Gets or sets the enabled state of this input mode.
Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
{@link InputModeBase#controller} so a disabled instance should never try to acquire the input mutex.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23enabled Online Documentation}*/
enabled: boolean;
/**
Gets if this mode is active.
This mode is active if a {@link InputModeBase#controller} is installed and is {@link ConcurrencyController#active}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23active Online Documentation}*/
readonly active: boolean;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link InputModeBase#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23exclusive Online Documentation}*/
exclusive: boolean;
/**
Gets the installed {@link InputModeBase#controller}.
@see InputModeBase#requestMutex
@see InputModeBase#releaseMutex
@see InputModeBase#hasMutex
@see InputModeBase#canRequestMutex
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeBase%23controller Online Documentation}*/
controller: ConcurrencyController | null;
static $class: Class;
private $_InputModeBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is InputModeBase;
}
export interface ClickEventArgs extends InputModeEventArgs {
}
/**
Event arguments used by {@link ClickInputMode#addClickedListener Clicked}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickEventArgs Online Documentation}*/
export class ClickEventArgs {
/**
Initializes a new instance of the {@link ClickEventArgs} class.
@param context The context in which the click occurred.
@param location The location.
@param modifiers State of the modifiers.
@param mouseButtons The mouse button(s) that have changed.
@param clickCount The number of clicks associated with this event.
@param [originalEvent=null] The original event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickEventArgs Online Documentation}*/
constructor(context: IInputModeContext, location: Point | PointConvertible, modifiers: ModifierKeys | ModifierKeysConvertible, mouseButtons: MouseButtons | MouseButtonsConvertible, clickCount: number, originalEvent?: Event | null);
/**
Gets the location of the click.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickEventArgs%23location Online Documentation}*/
readonly location: Point;
/**
Gets the state of the modifiers at the time of the click.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickEventArgs%23modifiers Online Documentation}*/
readonly modifiers: ModifierKeys;
/**
* Gets or sets a value indicating whether this {@link ClickEventArgs} have been handled.
* Client code that is triggered by the event and does not want the event to become handled by other handles should set
* this property to <code>true</code> to stop propagation of the event.
*
* Note that this property needs to be set before the function synchronously returns. This means that setting the value in
* an <code>async</code> function after the first <code>await</code> or in an asynchronous callback, it cannot not be seen by the emitter of the
* event and the sender will consider the event as not being handled.
*
* <pre>
* // wrong:
* mode.addItemClickedListener(async (sender, args) => {
*   await something()
*   args.handled = true  // will be set *after* all other event listeners have been invoked, only
* })
*
* // correct:
* mode.addItemClickedListener(async (sender, args) => {
*   args.handled = true // will be set immediately and 'mode' will see the change
*   await something()
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClickEventArgs%23handled Online Documentation}
*/
handled: boolean;
/**
Gets the {@link ClickEventArgs#mouseButtons} that have changed for this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickEventArgs%23mouseButtons Online Documentation}*/
readonly mouseButtons: MouseButtons;
/**
Gets the number of clicks associated with this event.
This can be used to distinguish between single and multiple clicks. Note that for compatibility reasons, a negative
value means that no click count has been provided at event creation time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickEventArgs%23clickCount Online Documentation}*/
readonly clickCount: number;
/**
Gets the last input event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickEventArgs%23originalEvent Online Documentation}*/
originalEvent: Event | null;
static $class: Class;
private $_ClickEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClickEventArgs;
}
/**
Policy to determine how single and double click events are raised in response to mouse clicks.
@see ClickInputMode#doubleClickPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleClickPolicy Online Documentation}*/
export enum DoubleClickPolicy {
/**
In case of a double click, only the double click event is raised, but no single click events.
Single clicks not belonging to a double click will be reported with a short delay to ensure that they do not become
double clicks.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleClickPolicy%23DOUBLE_CLICK_ONLY Online Documentation}*/
DOUBLE_CLICK_ONLY,
/**
In case of a double click, one single click event is raised, followed by a double click event.
The second click of the double click is not reported as a single click. Single clicks not belonging to a double click
will be reported instantly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleClickPolicy%23INITIAL_SINGLE_AND_DOUBLE_CLICK Online Documentation}*/
INITIAL_SINGLE_AND_DOUBLE_CLICK,
/**
In case of a double click, two single click events are raised, followed by a double click event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleClickPolicy%23BOTH_SINGLE_CLICKS_AND_DOUBLE_CLICK Online Documentation}*/
BOTH_SINGLE_CLICKS_AND_DOUBLE_CLICK
}
/** Automatic conversion to {@link DoubleClickPolicy}. */
type DoubleClickPolicyStringValues = "double-click-only" | "initial-single-and-double-click" | "both-single-clicks-and-double-click";
export interface ClickInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An input mode that recognizes simple mouse clicks.
* Clients register to {@link ClickInputMode#addClickedListener Clicked} to get notified of mouse
* clicks. The {@link ClickInputMode#activeButtons} property can be used to switch between the detection of right and left mouse clicks.
*
* This mode can be instructed to {@link ClickInputMode#swallowFocusClick swallow clicks} if they lead happen shortly after the control gained focus. This is useful
* to prevent accidental clicks from being processed if the user wanted to put the focus into the control, only.
*
* This mode is {@link ClickInputMode#exclusive} by default.
*
* Typically the <code>ClickInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#clickInputMode} property.
*
* <br><b>Getting the ClickInputMode from its parent input mode</b>
* <pre>
* const clickInputMode = mode.clickInputMode
* </pre>
*
* @see ClickInputMode#doubleClickPolicy
* @see ClickInputMode#swallowFocusClick
* @see ClickInputMode#activeButtons
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode Online Documentation}
*/
export class ClickInputMode {
/**
Creates a new instance of this mode that detects left mouse clicks.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode Online Documentation}*/
constructor();
/**
Cancels the editing of this mode.
This implementation {@link ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
{@link ClickInputMode#onCanceled}.
@see ClickInputMode#onCanceled
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Creates the event recognizer that is used to recognize the clicks for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23createClickRecognizer() Online Documentation}*/
createClickRecognizer(): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Creates the event recognizer that is used to recognize the press event for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23createPressRecognizer() Online Documentation}*/
createPressRecognizer(): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Creates the event recognizer that is used to recognize the release event for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23createReleaseRecognizer() Online Documentation}*/
createReleaseRecognizer(): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link ClickInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Determines whether the given event is a press event that occurred at an invalid location.
@param eventSource
@param evt
@see ClickInputMode#validClickHitTestable
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23isInvalidPress(Object%2CEventArgs) Online Documentation}*/
isInvalidPress(eventSource: any, evt: EventArgs): boolean;
/**
Determines whether the given event is a valid click event.
@param src
@param evt
@see ClickInputMode#validClickHitTestable
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23isValidClick(Object%2CEventArgs) Online Documentation}*/
isValidClick(src: any, evt: EventArgs): boolean;
/**
Determines whether the given event is a valid press event.
@param eventSource
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23isValidPress(Object%2CEventArgs) Online Documentation}*/
isValidPress(eventSource: any, evt: EventArgs): boolean;
/**
Determines whether the given event is a valid release event.
@param src
@param evt
@see ClickInputMode#validClickHitTestable
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23isValidRelease(Object%2CEventArgs) Online Documentation}*/
isValidRelease(src: any, evt: EventArgs): boolean;
/**
Called after {@link ClickInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called once this mode has recognized a click gesture.
This method will trigger the {@link ClickInputMode#addClickedListener Clicked} event.
@param evt The arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onClicked(ClickEventArgs) Online Documentation}*/
onClicked(evt: ClickEventArgs): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Called once this mode has recognized a double-click gesture.
This method will trigger the {@link ClickInputMode#addDoubleClickedListener DoubleClicked} event.
@param evt The arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onDoubleClicked(ClickEventArgs) Online Documentation}*/
onDoubleClicked(evt: ClickEventArgs): void;
/**
Called once this mode has recognized a left click gesture.
This method will trigger the {@link ClickInputMode#addLeftClickedListener LeftClicked} event.
@param evt The arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onLeftClicked(ClickEventArgs) Online Documentation}*/
onLeftClicked(evt: ClickEventArgs): void;
/**
Called once this mode has recognized a left double-click gesture.
This method will trigger the {@link ClickInputMode#addLeftDoubleClickedListener LeftDoubleClicked}
event.
@param evt The arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onLeftDoubleClicked(ClickEventArgs) Online Documentation}*/
onLeftDoubleClicked(evt: ClickEventArgs): void;
/**
Called once this mode has recognized a right click gesture.
This method will trigger the {@link ClickInputMode#addRightClickedListener RightClicked} event.
@param evt The arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onRightClicked(ClickEventArgs) Online Documentation}*/
onRightClicked(evt: ClickEventArgs): void;
/**
Called once this mode has recognized a double-click gesture.
This method will trigger the {@link ClickInputMode#addRightDoubleClickedListener RightDoubleClicked}
event.
@param evt The arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onRightDoubleClicked(ClickEventArgs) Online Documentation}*/
onRightDoubleClicked(evt: ClickEventArgs): void;
/**
Called after {@link ClickInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Prevents a double-click event from being issued if the next click would do so and sends only a single click instead.
The intention is for "breaking" a double click if an input mode using {@link ClickInputMode} performs actions that
would lead to surprising behavior if the next click would lead to a double-click. Examples of this are
{@link GraphEditorInputMode}'s selection cycling where several clicks in short succession may occur, as well as
creating a node by clicking on the canvas and selecting it immediately afterwards with another click.
This method's effect is very short-lived. It really only prevents a double-click event for the very next click that this
input mode handles. The internal flag set by this method is re-set on every click received. This also means that if you
call this method and the next click is just a regular single-click the flag is cleared nonetheless.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23preventNextDoubleClick() Online Documentation}*/
preventNextDoubleClick(): void;
/**
Returns and resets the flag set by {@link ClickInputMode#preventNextDoubleClick}.
To ensure that the flag really acts just once, this method resets the flag and returns its value prior to the reset.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23queryAndResetPreventNextDoubleClick() Online Documentation}*/
queryAndResetPreventNextDoubleClick(): boolean;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link ClickInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link ClickInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the ClickInputMode on its parent input mode</b>
* <pre>
* mode.clickInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link ClickInputMode#controller}.
@see IInputMode#install
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Adds the given listener for the <code>Clicked</code> event that occurs once a click has been detected.
@param listener The listener to add.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#removeClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23Clicked Online Documentation}*/
addClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Removes the given listener for the <code>Clicked</code> event that occurs once a click has been detected.
@param listener The listener to remove.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#addClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23Clicked Online Documentation}*/
removeClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Adds the given listener for the <code>LeftClicked</code> event that occurs once a left click has been detected.
@param listener The listener to add.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#removeLeftClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23LeftClicked Online Documentation}*/
addLeftClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Removes the given listener for the <code>LeftClicked</code> event that occurs once a left click has been detected.
@param listener The listener to remove.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#addLeftClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23LeftClicked Online Documentation}*/
removeLeftClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Adds the given listener for the <code>RightClicked</code> event that occurs once a right click has been detected.
@param listener The listener to add.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#removeRightClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23RightClicked Online Documentation}*/
addRightClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Removes the given listener for the <code>RightClicked</code> event that occurs once a right click has been detected.
@param listener The listener to remove.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#addRightClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23RightClicked Online Documentation}*/
removeRightClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Adds the given listener for the <code>DoubleClicked</code> event that occurs once a double-click has been detected.
@param listener The listener to add.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#removeDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23DoubleClicked Online Documentation}*/
addDoubleClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Removes the given listener for the <code>DoubleClicked</code> event that occurs once a double-click has been detected.
@param listener The listener to remove.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#addDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23DoubleClicked Online Documentation}*/
removeDoubleClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Adds the given listener for the <code>LeftDoubleClicked</code> event that occurs once a left double-click has been detected.
@param listener The listener to add.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#removeLeftDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23LeftDoubleClicked Online Documentation}*/
addLeftDoubleClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Removes the given listener for the <code>LeftDoubleClicked</code> event that occurs once a left double-click has been detected.
@param listener The listener to remove.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#addLeftDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23LeftDoubleClicked Online Documentation}*/
removeLeftDoubleClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Adds the given listener for the <code>RightDoubleClicked</code> event that occurs once a right double-click has been detected.
@param listener The listener to add.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#removeRightDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23RightDoubleClicked Online Documentation}*/
addRightDoubleClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Removes the given listener for the <code>RightDoubleClicked</code> event that occurs once a right double-click has been detected.
@param listener The listener to remove.
@see ClickEventArgs
@see ClickInputMode#doubleClickPolicy
@see ClickInputMode#addRightDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23RightDoubleClicked Online Documentation}*/
removeRightDoubleClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Gets or sets the click handling mode that determines the triggering behavior of {@link ClickInputMode#addClickedListener Clicked}
and {@link ClickInputMode#addDoubleClickedListener DoubleClicked}.
@see ClickInputMode#doubleClickPolicy
@default DoubleClickPolicy.BOTH_SINGLE_CLICKS_AND_DOUBLE_CLICK
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23doubleClickPolicy Online Documentation}*/
doubleClickPolicy: DoubleClickPolicy;
/**
Gets or sets a hit test that determines where this mode should recognize clicks.
This implementation is tested during the {@link ClickInputMode#isValidPress} to determine whether it is valid to
click here. The default implementation is {@link IHitTestable#ALWAYS}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23validClickHitTestable Online Documentation}*/
validClickHitTestable: IHitTestable;
/**
Gets or sets whether to {@link ConcurrencyController#requestMutex request the mutex} on a click.
If set to <code>true</code> this will discard other concurrent input modes on a click. The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23requestMutexOnClick Online Documentation}*/
requestMutexOnClick: boolean;
/**
Gets or sets the cursor to use when hovering over a valid hit region.
The default is <code>null</code> and the current cursor is used instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23validClickHitCursor Online Documentation}*/
validClickHitCursor: Cursor | null;
/**
Gets or sets for which button(s) mouse clicks should be processed.
Default value is {@link MouseButtons#LEFT}
@see ClickInputMode#createClickRecognizer
@default MouseButtons.LEFT
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23activeButtons Online Documentation}*/
activeButtons: MouseButtons;
/**
Gets or sets a property that determines whether clicks should be swallowed if they happen within a short amount of time
after the {@link CanvasComponent} got focus.
The default is <code>false</code>. If set to <code>true</code> clicks are discarded within 100 ms after the focus entered.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23swallowFocusClick Online Documentation}*/
swallowFocusClick: boolean;
/**
Gets the location of the last click.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23clickLocation Online Documentation}*/
readonly clickLocation: Point;
/**
Gets the state of the modifier keys at the time {@link ClickInputMode#addClickedListener Clicked}
is invoked.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClickInputMode%23modifiers Online Documentation}*/
readonly modifiers: ModifierKeys;
static $class: Class;
private $_ClickInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClickInputMode;
}
export interface CollectSnapResultsEventArgs extends InputModeEventArgs {
}
/**
Event arguments for the {@link SnapContext#addCollectSnapResultsListener CollectSnapResults} event.
Event handlers can use the {@link CollectSnapResultsEventArgs#addSnapResult} method to add snapping results for the
current move operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectSnapResultsEventArgs Online Documentation}*/
export class CollectSnapResultsEventArgs {
/**
Creates a new instance of the event arguments using the provided values for initialization.
@param context The context for which the event is queried.
@param originalLocation The original location of the mouse at the beginning of the gesture.
@param newLocation The new (current) location of the mouse.
@param snapDistance The zoom invariant snap distance.
@param gridSnapDistance The zoom invariant grid snap distance.
@param snapResults The collection to store the snap results in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectSnapResultsEventArgs Online Documentation}*/
constructor(context: IInputModeContext, originalLocation: Point | PointConvertible, newLocation: Point | PointConvertible, snapDistance: number, gridSnapDistance: number, snapResults: ICollection<SnapResult> | ICollectionConvertible<SnapResult>);
/**
Adds a {@link SnapResult} to the collection of snap results.
<code>null</code> values will be discarded.
@param snapResult The result to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectSnapResultsEventArgs%23addSnapResult(SnapResult) Online Documentation}*/
addSnapResult(snapResult: SnapResult | null): void;
/**
Gets the original location of the mouse at the beginning of the gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectSnapResultsEventArgs%23originalLocation Online Documentation}*/
readonly originalLocation: Point;
/**
Gets the new (current) location of the mouse that is going to be manipulated by the snapping process.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectSnapResultsEventArgs%23newLocation Online Documentation}*/
readonly newLocation: Point;
/**
Gets the vector by which the mouse has been moved since initialization of the snapping process.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectSnapResultsEventArgs%23delta Online Documentation}*/
readonly delta: Point;
/**
Gets the zoom invariant snap distance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectSnapResultsEventArgs%23snapDistance Online Documentation}*/
readonly snapDistance: number;
/**
Gets the zoom invariant grid snap distance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectSnapResultsEventArgs%23gridSnapDistance Online Documentation}*/
readonly gridSnapDistance: number;
static $class: Class;
private $_CollectSnapResultsEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CollectSnapResultsEventArgs;
}
export interface ConcurrencyController extends YObject {
}
/**
A controller class that manages a single {@link IInputMode} implementation.
The controller will be passed to its associated {@link IInputMode} in the input mode's {@link IInputMode#install}
method. The associated mode can {@link ConcurrencyController#requestMutex request} an input mutex to {@link ConcurrencyController#active deactivate} other modes added to the same {@link MultiplexingInputMode}
and using {@link ConcurrencyController#exclusive} controllers. {@link ConcurrencyController#releaseMutex Releasing the mutex} will reactivate the controller of other modes.
@see IInputMode
@see MultiplexingInputMode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController Online Documentation}*/
export class ConcurrencyController {
/**
Creates an instance that can be used to {@link IInputMode#install} the given <code>mode</code>.
The returned {@link ConcurrencyController} is not synchronized with any other one so the <code>mode</code> will effectively run
in a non-{@link ConcurrencyController#exclusive exclusive} mode.
@param mode The mode, the {@link ConcurrencyController} shall be created for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23ConcurrencyController(IInputMode) Online Documentation}*/
constructor(mode: IInputMode);
/**
Determines if the mutex can be successfully requested currently.
This will return <code>true</code>, iff this instance is {@link ConcurrencyController#active} and doesn't hold the mutex.
@returns Whether a call to {@link ConcurrencyController#requestMutex} would currently succeed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23canRequestMutex() Online Documentation}*/
canRequestMutex(): boolean;
/**
Determines whether this instance owns the input mutex.
The instance owns the input mutex if it has successfully {@link ConcurrencyController#requestMutex requested} the mutex in the past and not yet {@link ConcurrencyController#releaseMutex released} it.
@returns Whether this instance has the input mutex.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23hasMutex() Online Documentation}*/
hasMutex(): boolean;
/**
Triggers a {@link ConcurrencyController#addPriorityChangedListener PriorityChanged} event with
{@link EventArgs#EMPTY}.
This method should be called by the associated mode if its {@link IInputMode#priority} has changed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23onPriorityChanged() Online Documentation}*/
onPriorityChanged(): void;
/**
Releases the input mutex.
Modes should call this method when they have finished editing the contents exclusively. This will reactivate those modes
whose controller that had been {@link ConcurrencyController#active deactivated} at the time the mutex has been requested.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23releaseMutex() Online Documentation}*/
releaseMutex(): void;
/**
Can be called to request an input mutex for the input mode.
Modes should call this method to request exclusive editing for a canvas control. {@link ConcurrencyController#canRequestMutex}
should be called before requesting the mutex. If the mutex cannot be obtained, this method will trigger an exception.
@throws {Exception} <b>(Error)</b> If the mutex could not be obtained.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23requestMutex() Online Documentation}*/
requestMutex(): void;
/**
Adds the given listener for the <code>MutexObtained</code> event that occurs when the mutex has been successfully obtained.
This event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to add.
@see ConcurrencyController#removeMutexObtainedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23MutexObtained Online Documentation}*/
addMutexObtainedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>MutexObtained</code> event that occurs when the mutex has been successfully obtained.
This event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to remove.
@see ConcurrencyController#addMutexObtainedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23MutexObtained Online Documentation}*/
removeMutexObtainedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>MutexReleased</code> event that occurs when the mutex has been released.
This event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to add.
@see ConcurrencyController#removeMutexReleasedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23MutexReleased Online Documentation}*/
addMutexReleasedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>MutexReleased</code> event that occurs when the mutex has been released.
This event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to remove.
@see ConcurrencyController#addMutexReleasedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23MutexReleased Online Documentation}*/
removeMutexReleasedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Gets the current owner of the input mutex or <code>null</code> if no mode currently owns the mutex.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23mutexOwner Online Documentation}*/
readonly mutexOwner: IInputMode | null;
/**
Gets or sets if this controller is enabled.
The default state is disabled. A disabled controller is always {@link ConcurrencyController#active inactive}.
When the controller is {@link IInputMode#install installed} and the input mode is ready for editing, it has to set this property to <code>true</code> if it
wants to use the input mutex.
@see ConcurrencyController#active
@see ConcurrencyController#addActiveChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23enabled Online Documentation}*/
enabled: boolean;
/**
Gets if this controller is active.
An {@link IInputMode} should only handle events and gestures if this property on its associated {@link ConcurrencyController}
is set to <code>true</code>.
If this controller is set as {@link ConcurrencyController#exclusive non-exclusive} by its input mode it is active iff it is {@link ConcurrencyController#enabled}. Otherwise it has to be {@link ConcurrencyController#enabled} and no
other input mode may have the {@link ConcurrencyController#requestMutex input mutex}.
@see ConcurrencyController#addActiveChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23active Online Documentation}*/
active: boolean;
/**
Adds the given listener for the <code>ActiveChanged</code> event that occurs the {@link ConcurrencyController#active} property changes.
This event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to add.
@see ConcurrencyController#removeActiveChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23ActiveChanged Online Documentation}*/
addActiveChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ActiveChanged</code> event that occurs the {@link ConcurrencyController#active} property changes.
This event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to remove.
@see ConcurrencyController#addActiveChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23ActiveChanged Online Documentation}*/
removeActiveChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Gets or sets a value indicating whether this controller will be the only one running when it has the mutex.
If this controller is marked as exclusive and has the mutex, all controllers of other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be {@link ConcurrencyController#active deactivated}. Otherwise it will always run concurrently with all other controllers.
Changing this property will temporarily {@link ConcurrencyController#enabled disable} the controller.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23exclusive Online Documentation}*/
exclusive: boolean;
/**
Gets or sets the cursor the associated mode of this controller would like to have displayed in the
{@link CanvasComponent}.
To indicate that the mode does not have any preferences the property should be set to <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23preferredCursor Online Documentation}*/
preferredCursor: Cursor | null;
/**
Adds the given listener for the <code>PreferredCursorChanged</code> event that occurs when the {@link ConcurrencyController#preferredCursor} property changes.
{@link MultiplexingInputMode} listens to property changes of the controller of its sub modes. properties. This
event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to add.
@see ConcurrencyController#removePreferredCursorChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23PreferredCursorChanged Online Documentation}*/
addPreferredCursorChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>PreferredCursorChanged</code> event that occurs when the {@link ConcurrencyController#preferredCursor} property changes.
{@link MultiplexingInputMode} listens to property changes of the controller of its sub modes. properties. This
event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to remove.
@see ConcurrencyController#addPreferredCursorChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23PreferredCursorChanged Online Documentation}*/
removePreferredCursorChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>PriorityChanged</code> event that occurs when {@link ConcurrencyController#onPriorityChanged}
is called by the associated mode.
{@link MultiplexingInputMode} listens to priority changes of the controller of its sub modes.
@param listener The listener to add.
@see ConcurrencyController#removePriorityChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23PriorityChanged Online Documentation}*/
addPriorityChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>PriorityChanged</code> event that occurs when {@link ConcurrencyController#onPriorityChanged}
is called by the associated mode.
{@link MultiplexingInputMode} listens to priority changes of the controller of its sub modes.
@param listener The listener to remove.
@see ConcurrencyController#addPriorityChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConcurrencyController%23PriorityChanged Online Documentation}*/
removePriorityChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
static $class: Class;
private $_ConcurrencyController: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ConcurrencyController;
}
export interface PopulateMenuEventArgs extends InputModeEventArgs {
}
/**
Event arguments for the {@link ContextMenuInputMode#addPopulateMenuListener PopulateMenu}
event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PopulateMenuEventArgs Online Documentation}*/
export class PopulateMenuEventArgs {
/**
Initializes a new instance of the {@link PopulateMenuEventArgs} class.
@param context The context.
@param queryLocation The query location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PopulateMenuEventArgs Online Documentation}*/
constructor(context: IInputModeContext, queryLocation: Point | PointConvertible);
/**
Gets the queried location in world coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PopulateMenuEventArgs%23queryLocation Online Documentation}*/
queryLocation: Point;
/**
* Gets or sets a value indicating whether this {@link PopulateMenuEventArgs} has been handled.
* If this event is marked as handled, the input mode will use the property {@link PopulateMenuEventArgs#showMenu} to decide whether to show the context menu.
*
* This property is automatically set to <code>true</code> if a value is assigned to the property {@link PopulateMenuEventArgs#showMenu}.
*
* Note that this property needs to be set before the function synchronously returns. This means that setting the value in
* an <code>async</code> function after the first <code>await</code> or in an asynchronous callback, it cannot not be seen by the emitter of the
* event and the sender will consider the event as not being handled.
*
* <pre>
* // wrong:
* mode.addPopulateMenuListener(async (sender, args) => {
*   await something()
*   args.handled = true  // will be set *after* all other event listeners have been invoked, only
* })
*
* // correct:
* mode.addPopulateMenuListener(async (sender, args) => {
*   args.handled = true // will be set immediately and 'mode' will see the change
*   await something()
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PopulateMenuEventArgs%23handled Online Documentation}
*/
handled: boolean;
/**
Gets or sets a value indicating whether to actually display the context menu.
This will set the {@link PopulateMenuEventArgs#handled} property to <code>true</code>.
The default value of this property is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PopulateMenuEventArgs%23showMenu Online Documentation}*/
showMenu: boolean;
static $class: Class;
private $_PopulateMenuEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PopulateMenuEventArgs;
}
export interface ContextMenuInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An implementation of the {@link IInputMode} interface that handles the display of a custom context menu when the
* user right clicks on the {@link CanvasComponent}.
* This mode can be used with any context menu implementation since it does not impose any specific requirements.
* Typically, it is used in the following way.
*
* First, this mode must be informed that the user wants to show the context menu with the method
* {@link ContextMenuInputMode#shouldOpenMenu}, for example because a <code>contextmenu</code> event was fired.
*
* Then, in response, this mode checks whether its current state allows opening a context menu, and fires the {@link ContextMenuInputMode#addPopulateMenuListener PopulateMenu}
* and <code>GraphInputMode.populateItemContextMenu</code>.
*
* If the context menu was closed because of user interaction with it, for example the user selected an menu entry, this
* mode must be informed by calling the method {@link ContextMenuInputMode#menuClosed}.
*
* This mode detects automatically if the menu must be closed because of other user interaction, for example the user
* clicked somewhere else, and fires the event {@link ContextMenuInputMode#addCloseMenuListener CloseMenu}
* to request closing of the context menu.
*
* Typically the <code>ContextMenuInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#contextMenuInputMode} property.
*
* <br><b>Getting the ContextMenuInputMode from its parent input mode</b>
* <pre>
* const contextMenuInputMode = mode.contextMenuInputMode
* </pre>
*
* It is recommended to configure the context menu handling on the parent {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* instead of the <code>ContextMenuInputMode</code>. This involves registering for the {@link GraphInputMode#addPopulateItemContextMenuListener PopulateItemContextMenu}
* event instead of the {@link ContextMenuInputMode#addPopulateMenuListener PopulateMenu} event:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.contextMenuItems = GraphItemTypes.NODE
* mode.addPopulateItemContextMenuListener((sender, args) => {
*   // Get the node which is handled or null if the item is not a node
*   const node = args.item
*   // Create the context menu items
*   if (node !== null) {
*     // Create a menu item to delete the node
*
*     // Show the menu
*     args.showMenu = true
*     // Mark the event as handled
*     args.handled = true
*   }
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode Online Documentation}
*/
export class ContextMenuInputMode {
/**
Initializes a new instance of the {@link ContextMenuInputMode} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode Online Documentation}*/
constructor();
/**
Cancels the display of the context menu and requests that the context menu is closed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Creates an {@link IInputModeContext} for use with the {@link ContextMenuInputMode#addPopulateMenuListener PopulateMenu}
call in the upcoming query.
@returns An instance of {@link IInputModeContext}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23createPopulateMenuContext() Online Documentation}*/
createPopulateMenuContext(): IInputModeContext;
/**
Installs this mode in the canvas.
@param context The context to install this mode into.
@param controller The {@link ContextMenuInputMode#controller} for this mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
This method must be called by custom code to indicate that the menu has been closed.
Since this input mode cannot know when the user has closed a context menu, this callback should be used to allow the
instance to adjust the internal state.
@see ContextMenuInputMode#addCloseMenuListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23menuClosed() Online Documentation}*/
menuClosed(): void;
/**
Called after {@link ContextMenuInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called when the menu is closed and triggers the {@link ContextMenuInputMode#addCloseMenuListener CloseMenu}
event.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23onCloseMenu() Online Documentation}*/
onCloseMenu(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
This will populate the context menu for the given world coordinate.
This implementation will trigger the {@link ContextMenuInputMode#addPopulateMenuListener PopulateMenu}
event and will use the {@link PopulateMenuEventArgs#showMenu} value as the return value. If the event will not be handled, this method will return <code>true</code> if the
menu already contains elements.
@param location The location in the world coordinate system for which the context menu has been invoked.
@returns Whether to show the context menu.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23onMenuOpening(Point) Online Documentation}*/
onMenuOpening(location: Point): boolean;
/**
Raises the {@link ContextMenuInputMode#addPopulateMenuListener PopulateMenu} event.
@param evt The {@link PopulateMenuEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23onPopulateMenu(PopulateMenuEventArgs) Online Documentation}*/
onPopulateMenu(evt: PopulateMenuEventArgs): void;
/**
Called after {@link ContextMenuInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
This method must be called by custom code to inform this input mode that a context menu is about to be opened by a user
gesture.
Since this input mode does not know when the user wants to open a context menu, this method must be called to possibly
cancel the request and, otherwise, update the internal state of this mode.
@param [queryLocation=null] The optional location for which the context menu content should be queried. This value will be passed to {@link ContextMenuInputMode#onPopulateMenu}
and will ultimately be available in {@link PopulateMenuEventArgs#queryLocation}. If not specified, the last known mouse location will be used.
@see ContextMenuInputMode#menuClosed
@see ContextMenuInputMode#addCloseMenuListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23shouldOpenMenu(Point) Online Documentation}*/
shouldOpenMenu(queryLocation?: Point | null): boolean;
/**
Stops the display of the context menu and requests that the context menu is closed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23tryStop() Online Documentation}*/
tryStop(): boolean;
/**
Removes the menu from the context and replaces it with the old instance.
@param context The context to uninstall this mode from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link ContextMenuInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link ContextMenuInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the ContextMenuInputMode on its parent input mode</b>
* <pre>
* mode.contextMenuInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link ConcurrencyController}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
/**
* Adds the given listener for the <code>PopulateMenu</code> event that occurs when the context menu is about to be shown.
* Handlers of the event can populate their context menu, set the {@link PopulateMenuEventArgs#showMenu} and set or respect the {@link PopulateMenuEventArgs#handled} property. Note that all handlers
* will be called, even if one of them sets the {@link PopulateMenuEventArgs#handled} property to
* <code>true</code>. Every handler should query this property and decide carefully what to do.
*
* It is recommended to configure the context menu handling on the parent {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* instead of the <code>ContextMenuInputMode</code>. This involves registering for the {@link GraphInputMode#addPopulateItemContextMenuListener PopulateItemContextMenu}
* event instead of this event:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.contextMenuItems = GraphItemTypes.NODE
* mode.addPopulateItemContextMenuListener((sender, args) => {
*   // Get the node which is handled or null if the item is not a node
*   const node = args.item
*   // Create the context menu items
*   if (node !== null) {
*     // Create a menu item to delete the node
*
*     // Show the menu
*     args.showMenu = true
*     // Mark the event as handled
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see ContextMenuInputMode#removePopulateMenuListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23PopulateMenu Online Documentation}
*/
addPopulateMenuListener(listener: (sender: any, evt: PopulateMenuEventArgs) => void): void;
/**
* Removes the given listener for the <code>PopulateMenu</code> event that occurs when the context menu is about to be shown.
* Handlers of the event can populate their context menu, set the {@link PopulateMenuEventArgs#showMenu} and set or respect the {@link PopulateMenuEventArgs#handled} property. Note that all handlers
* will be called, even if one of them sets the {@link PopulateMenuEventArgs#handled} property to
* <code>true</code>. Every handler should query this property and decide carefully what to do.
*
* It is recommended to configure the context menu handling on the parent {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* instead of the <code>ContextMenuInputMode</code>. This involves registering for the {@link GraphInputMode#addPopulateItemContextMenuListener PopulateItemContextMenu}
* event instead of this event:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.contextMenuItems = GraphItemTypes.NODE
* mode.addPopulateItemContextMenuListener((sender, args) => {
*   // Get the node which is handled or null if the item is not a node
*   const node = args.item
*   // Create the context menu items
*   if (node !== null) {
*     // Create a menu item to delete the node
*
*     // Show the menu
*     args.showMenu = true
*     // Mark the event as handled
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see ContextMenuInputMode#addPopulateMenuListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23PopulateMenu Online Documentation}
*/
removePopulateMenuListener(listener: (sender: any, evt: PopulateMenuEventArgs) => void): void;
/**
Gets or sets an {@link IHitTestable} that determines whether it is valid to open a context menu at the queried
position.
If the instance yields <code>false</code> {@link ContextMenuInputMode#onMenuOpening} will not be queried and the context menu
will not be shown. By default there is a hit testable instance that always yields <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23validMenuLocationHitTestable Online Documentation}*/
validMenuLocationHitTestable: IHitTestable | null;
/**
Gets or sets a value that determines whether clicks are swallowed if they happen within a short amount of time after a
context menu was closed.
This is especially useful to cancel the menu by clicking somewhere outside the menu without creating a node. The default
is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23swallowCloseClick Online Documentation}*/
swallowCloseClick: boolean;
/**
Adds the given listener for the <code>CloseMenu</code> event that occurs when this instance requests closing an open context menu.
Custom context menus must be closed in response to this event.
@param listener The listener to add.
@see ContextMenuInputMode#menuClosed
@see ContextMenuInputMode#removeCloseMenuListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23CloseMenu Online Documentation}*/
addCloseMenuListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>CloseMenu</code> event that occurs when this instance requests closing an open context
menu.
Custom context menus must be closed in response to this event.
@param listener The listener to remove.
@see ContextMenuInputMode#menuClosed
@see ContextMenuInputMode#addCloseMenuListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextMenuInputMode%23CloseMenu Online Documentation}*/
removeCloseMenuListener(listener: (sender: any, evt: EventArgs) => void): void;
static $class: Class;
private $_ContextMenuInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ContextMenuInputMode;
}
export interface DropInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
An input mode which handles drag and drop related events on a {@link CanvasComponent}.
An instance of {@link DropInputMode} is initialized for an expected data type in the constructor. There are four
events raised by an instance of this class.
<ul>
<li>{@link DropInputMode#addDragEnteredListener DragEntered} is raised, if a drag operation enters the
{@link CanvasComponent}.
<li>{@link DropInputMode#addDragOverListener DragOver} is raised, if a drag operation moves over the
{@link CanvasComponent} after having entered it.
<li>{@link DropInputMode#addDragDroppedListener DragDropped} is raised, if a drag is dropped onto the
{@link CanvasComponent}.
<li>{@link DropInputMode#addDragLeftListener DragLeft} is raised, if a drag is operation leaves the
{@link CanvasComponent}.
</ul>
Clients can query the current {@link DropInputMode#mousePosition} which is updated while a drag is in progress. Clients get the data from the drag
operation by calling {@link DropInputMode#dropData} from an event handler registered for any of these events.
This mode is {@link DropInputMode#exclusive} by default.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode Online Documentation}*/
export class DropInputMode {
/**
Initializes a new instance of the {@link DropInputMode} class for the expected data type.
This instance fires events only if a drag operation carries data of the expected type.
@param expectedType A list of types that are used in {@link DragDropItem#getData}.
@see DropInputMode#addDragEnteredListener
@see DropInputMode#addDragDroppedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DropInputMode(Array%3Cstring%3E) Online Documentation}*/
constructor(expectedType: string[]);
/**
Initializes a new instance of the {@link DropInputMode} class for the expected data type.
This instance fires events only if a drag operation carries data of the expected type.
@param expectedType The type that is used in
{@link DragDropItem#getData}. This may also be a string that contains a comma-separated list of types.
@see DropInputMode#addDragEnteredListener
@see DropInputMode#addDragDroppedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DropInputMode(string) Online Documentation}*/
constructor(expectedType: string);
/**
Called to adjust the {@link DragEventArgs#dropEffect} accordingly.
@param evt The event args of the drag event to adjust the effects for.
@returns <code>true</code> if the drag event is handled by this input mode.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23adjustEffect(DragEventArgs) Online Documentation}*/
adjustEffect(evt: DragEventArgs): boolean;
/**
Cleanly cancels this mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Helper method that yields a suitably configured {@link InputModeEventArgs} for this input mode.
@returns An input mode event argument that is configured for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23createInputModeEventArgs() Online Documentation}*/
createInputModeEventArgs(): InputModeEventArgs;
/**
Installs this mode into the given canvas.
Clients should not call this method as this is handled by the framework.
@param context the context to install this mode into
@param controller The {@link DropInputMode#controller} for this mode.
@see CanvasComponent
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called after {@link DropInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Called once a drag has been dropped on the canvas.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23onDragDropped(InputModeEventArgs) Online Documentation}*/
onDragDropped(evt: InputModeEventArgs): void;
/**
Called once a drag has entered the canvas.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23onDragEntered(InputModeEventArgs) Online Documentation}*/
onDragEntered(evt: InputModeEventArgs): void;
/**
Called whenever a drag is over the canvas.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23onDraggedOver(InputModeEventArgs) Online Documentation}*/
onDraggedOver(evt: InputModeEventArgs): void;
/**
Called once a drag has left the canvas.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23onDragLeft(InputModeEventArgs) Online Documentation}*/
onDragLeft(evt: InputModeEventArgs): void;
/**
Called after {@link DropInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
This method updates the {@link DropInputMode#mousePosition} according to the coordinates passed in.
It is called prior to the {@link DropInputMode#onDragEntered}, {@link DropInputMode#onDraggedOver}, {@link DropInputMode#onDragDropped}
and {@link DropInputMode#onDragLeft} methods.
@param location The current location of the mouse in world coordinates.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23setDragLocation(Point) Online Documentation}*/
setDragLocation(location: Point): void;
/**
Overridden to only return <code>true</code> if this instance does not currently {@link ConcurrencyController#hasMutex have the input mutex}.
@returns <code>true</code> iff this instance does not {@link ConcurrencyController#hasMutex own the mutex}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23tryStop() Online Documentation}*/
tryStop(): boolean;
/**
Uninstalls this mode from the canvas.
Clients should not call this method as this is handled by the framework.
@param context the context
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link DropInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link DropInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the NodeDropInputMode on its parent input mode</b>
* <pre>
* mode.nodeDropInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link DropInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
/**
Adds the given listener for the <code>DragEntered</code> event that occurs when a drag operation enters the {@link CanvasComponent}
into which this instance is installed.
No event is raised
<ul>
<li>if this instance is not {@link DropInputMode#enabled},
<li>if another concurrent input mode is active or
<li>the data from the drag operation does not conform to the expected type.
</ul>
To get additional data for the event, clients should use the sender which is set to the sending
{@link DropInputMode}. The {@link EventArgs} provide no useful data.
@param listener The listener to add.
@see DropInputMode#removeDragEnteredListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DragEntered Online Documentation}*/
addDragEnteredListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragEntered</code> event that occurs when a drag operation enters the {@link CanvasComponent}
into which this instance is installed.
No event is raised
<ul>
<li>if this instance is not {@link DropInputMode#enabled},
<li>if another concurrent input mode is active or
<li>the data from the drag operation does not conform to the expected type.
</ul>
To get additional data for the event, clients should use the sender which is set to the sending
{@link DropInputMode}. The {@link EventArgs} provide no useful data.
@param listener The listener to remove.
@see DropInputMode#addDragEnteredListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DragEntered Online Documentation}*/
removeDragEnteredListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragOver</code> event that occurs when a drag operation drags over the {@link CanvasComponent}
into which this instance is installed.
No event is raised
<ul>
<li>if this instance is not {@link DropInputMode#enabled},
<li>if another concurrent input mode is active or
<li>the data from the drag operation does not conform to the expected type.
</ul>
To get additional data for the event, clients should use the sender which is set to the sending
{@link DropInputMode}. The {@link EventArgs} provide no useful data.
@param listener The listener to add.
@see DropInputMode#removeDragOverListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DragOver Online Documentation}*/
addDragOverListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragOver</code> event that occurs when a drag operation drags over the {@link CanvasComponent}
into which this instance is installed.
No event is raised
<ul>
<li>if this instance is not {@link DropInputMode#enabled},
<li>if another concurrent input mode is active or
<li>the data from the drag operation does not conform to the expected type.
</ul>
To get additional data for the event, clients should use the sender which is set to the sending
{@link DropInputMode}. The {@link EventArgs} provide no useful data.
@param listener The listener to remove.
@see DropInputMode#addDragOverListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DragOver Online Documentation}*/
removeDragOverListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragLeft</code> event that occurs when a drag operation leaves the {@link CanvasComponent}
into which this instance is installed.
No event is raised
<ul>
<li>if this instance is not {@link DropInputMode#enabled},
<li>if another concurrent input mode is active or
<li>the data from the drag operation does not conform to the expected type.
</ul>
To get additional data for the event, clients should use the sender which is set to the sending
{@link DropInputMode}. The {@link EventArgs} provide no useful data.
@param listener The listener to add.
@see DropInputMode#removeDragLeftListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DragLeft Online Documentation}*/
addDragLeftListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragLeft</code> event that occurs when a drag operation leaves the {@link CanvasComponent}
into which this instance is installed.
No event is raised
<ul>
<li>if this instance is not {@link DropInputMode#enabled},
<li>if another concurrent input mode is active or
<li>the data from the drag operation does not conform to the expected type.
</ul>
To get additional data for the event, clients should use the sender which is set to the sending
{@link DropInputMode}. The {@link EventArgs} provide no useful data.
@param listener The listener to remove.
@see DropInputMode#addDragLeftListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DragLeft Online Documentation}*/
removeDragLeftListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragDropped</code> event that occurs when a drag operation is dropped onto the {@link CanvasComponent}
into which this instance is installed.
No event is raised
<ul>
<li>if this instance is not {@link DropInputMode#enabled},
<li>if another concurrent input mode is active or
<li>the data from the drag operation does not conform to the expected type.
</ul>
To get additional data for the event, clients should use the sender which is set to the sending
{@link DropInputMode}. The {@link EventArgs} provide no useful data.
@param listener The listener to add.
@see DropInputMode#removeDragDroppedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DragDropped Online Documentation}*/
addDragDroppedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragDropped</code> event that occurs when a drag operation is dropped onto the {@link CanvasComponent}
into which this instance is installed.
No event is raised
<ul>
<li>if this instance is not {@link DropInputMode#enabled},
<li>if another concurrent input mode is active or
<li>the data from the drag operation does not conform to the expected type.
</ul>
To get additional data for the event, clients should use the sender which is set to the sending
{@link DropInputMode}. The {@link EventArgs} provide no useful data.
@param listener The listener to remove.
@see DropInputMode#addDragDroppedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23DragDropped Online Documentation}*/
removeDragDroppedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Gets or sets the {@link DragDropEffects} that will be used for drags on the canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23dragDropEffect Online Documentation}*/
dragDropEffect: DragDropEffects;
/**
Gets or sets the {@link IHitTestable} that determines whether the given location is a valid drop location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23validDropHitTestable Online Documentation}*/
validDropHitTestable: IHitTestable;
/**
Gets the last processed drag event argument.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23lastDragEventArgs Online Documentation}*/
readonly lastDragEventArgs: DragEventArgs | null;
/**
Gets the current mouse position during drag operations.
The position is returned in world coordinates according to the {@link CanvasComponent} into which this input mode is
installed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23mousePosition Online Documentation}*/
readonly mousePosition: IPoint;
/**
Gets the mouse position after dropping an item.
The position is returned in world coordinates according to the {@link CanvasComponent} into which this input mode is
installed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23dropLocation Online Documentation}*/
readonly dropLocation: Point;
/**
Gets the data carried by a drag operation.
This is only valid, if a drag entered the {@link CanvasComponent} first.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23dropData Online Documentation}*/
readonly dropData: any | null;
/**
Gets or sets a value indicating whether to capture mouse input during drag operations.
The default is <code>true</code> and thus prevents mouse events from being processed by visuals in the canvas.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23captureMouseInputDuringDrag Online Documentation}*/
captureMouseInputDuringDrag: boolean;
/**
Gets a value indicating whether a drag operation is currently in progress.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropInputMode%23isDragging Online Documentation}*/
isDragging: boolean;
static $class: Class;
private $_DropInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DropInputMode;
}
export interface FocusGuardInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
cancel(): void;
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
An {@link IInputMode} that can be added to a {@link MultiplexingInputMode} to suppress other modes from doing
unwanted things if the control has just become focused.
This mode requests the input mutex once the canvas gains focus. It returns the mutex after the mode has regained focus
and the mouse has been clicked or 100 milliseconds have passed. This mode should be registered with the {@link MultiplexingInputMode}
using the {@link MultiplexingInputMode#add} method and a large priority number (e.g.
<code>1000</code>). So that other modes don't receive the input mutex once this one has released the mutex.
This mode is {@link ConcurrencyController#exclusive} by default.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode Online Documentation}*/
export class FocusGuardInputMode {
/**
Creates a new instance of this mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode Online Documentation}*/
constructor();
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link FocusGuardInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called after {@link FocusGuardInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Called after {@link FocusGuardInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets the enabled state of this input mode.
Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
{@link FocusGuardInputMode#controller} so a disabled instance should never try to acquire the input mutex.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode%23enabled Online Documentation}*/
enabled: boolean;
/**
Gets the installed {@link FocusGuardInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
/**
Gets or sets the {@link Cursor} that will be shown during active guards.
The default is {@link Cursor#DEFAULT}
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusGuardInputMode%23guardCursor Online Documentation}*/
guardCursor: Cursor | null;
static $class: Class;
private $_FocusGuardInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FocusGuardInputMode;
}
export interface HandleInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
cancel(): void;
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} implementation that can handle a collection of {@link IHandle}s.
* This mode will render a visual representation of {@link IHandle}s in the {@link CanvasComponent} and deal with
* mouse gestures that drag the visual representations accordingly.
*
* This mode is {@link HandleInputMode#exclusive} by default.
*
* Typically the <code>HandleInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} and can be retrieved
* from the {@link HandleInputMode} property.
*
* <br><b>Getting the HandleInputMode from its parent input mode</b>
* <pre>
* const handleInputMode = mode.handleInputMode
* </pre>
*
* If one needs to execute some code after some items have been moved by the {@link HandleInputMode} he can register a
* handler to its {@link HandleInputMode#addDragFinishedListener DragFinished} event. The moved items
* can be retrieved from the {@link HandleInputMode#affectedItems} property:
*
* <pre>
* mode.handleInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.handleInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @see IHandle
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode Online Documentation}
*/
export class HandleInputMode {
/**
Creates a new mode with no handles initially.
@see HandleInputMode#handles
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode Online Documentation}*/
constructor();
/**
Adds a handle to the collection of handles managed by this instance.
@param handle The handle to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23addHandle(IHandle) Online Documentation}*/
addHandle(handle: IHandle): void;
/**
Factory method that creates and {@link HandleInputMode#addHandle adds} a handle to this mode by wrapping a given {@link IMutablePoint} instance.
The <code>cursor</code> and <code>type</code> are used to initialize the {@link IHandle} instance.
@param location The point to use as a handle.
@param [cursor=null] The cursor to use.
@param [type=HandleTypes.DEFAULT] The type of the handle to create.
@returns The handle created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23addHandle(IMutablePoint%2CCursor%2CHandleTypes) Online Documentation}*/
addHandle(location: IMutablePoint, cursor?: Cursor | null | CursorConvertible, type?: HandleTypes | HandleTypesConvertible): IHandle;
/**
Factory method that creates and {@link HandleInputMode#addHandle adds} a handle to this mode by wrapping a given {@link IMutablePoint} instance.
The <code>cursor</code> and <code>type</code> are used to initialize the {@link IHandle} instance.
@param options The parameters to pass.
@param options.location The point to use as a handle.
@param [options.cursor=null] The cursor to use.
@param [options.type=HandleTypes.DEFAULT] The type of the handle to create.
@returns The handle created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23addHandle(IMutablePoint%2CCursor%2CHandleTypes) Online Documentation}*/
addHandle(options: {
location: IMutablePoint;
cursor?: Cursor | null | CursorConvertible;
type?: HandleTypes | HandleTypesConvertible;
}): IHandle;
/**
Invoked when this mode gets "armed".
This happens when the mouse pointer hovers over a handle. This changes the mouse cursor to the <code>handle</code>'s {@link IHandle#cursor}.
@param handle
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23arm(IHandle) Online Documentation}*/
arm(handle: IHandle | null): void;
/**
Method that can be used to initialize the dragging of a given handle by hand.
This will simulate the user having initialized a drag of the given handle.
@param handle The handle to be dragged.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23beginDragging(IHandle) Online Documentation}*/
beginDragging(handle: IHandle): void;
/**
Creates an {@link IInputModeContext} for use with the {@link IHandle} interface for the upcoming drag
operation.
@returns An instance of {@link IInputModeContext}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23createHandleInputModeContext() Online Documentation}*/
createHandleInputModeContext(): IInputModeContext;
/**
Helper method that yields a suitably configured {@link InputModeEventArgs} for this input mode.
@returns An input mode event argument that is configured for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23createInputModeEventArgs() Online Documentation}*/
createInputModeEventArgs(): InputModeEventArgs;
/**
Creates the {@link Visual} for the given handle, if {@link HandleInputMode#renderMode} is set to {@link RenderModes#SVG}.
@param context The context that describes where the visual will be used.
@param handle The handle to create a visual representation for.
@returns The {@link Visual}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23createVisual(IRenderContext%2CIHandle) Online Documentation}*/
createVisual(context: IRenderContext, handle: IHandle | null): SvgVisual;
/**
Invoked when this mode gets "disarmed".
This happens when the mouse stops hovering over a handle. This resets the mouse cursor
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23disarm() Online Documentation}*/
disarm(): void;
/**
Finds the closest hit handle for the given world coordinate pair.
The default implementation compares the Manhattan distance of the handles to find the closest handle.
@param location The coordinates in the world coordinate system.
@returns The closest handle that has been hit or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23getClosestHitHandle(Point) Online Documentation}*/
getClosestHitHandle(location: Point): IHandle | null;
/**
Finds the closest hit handle for the given world coordinate pair.
@param location The coordinates in the world coordinate system.
@returns The closest handle that has been hit or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23getClosestHitHandleTouch(Point) Online Documentation}*/
getClosestHitHandleTouch(location: Point): IHandle | null;
/**
Determines whether a given handle has been visually hit by the mouse at the given world coordinates.
This implementation uses the Manhattan distance as a metric to determine if the handle was hit.
@param handle The handle to check
@param location The view coordinates to check.
@param distance The distance of the handle to the location. In the default implementation, this is a tuple representing the x- and
y-distance of the handle to the location.
@returns Whether the handle has been hit.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23handleIsHit(IHandle%2CPoint%2CPoint) Online Documentation}*/
handleIsHit(handle: IHandle, location: Point, distance: Point): boolean;
/**
Determines whether a given handle has been visually hit by the touch device at the given world coordinates.
@param handle The handle to check
@param location The world coordinates to check.
@param distance The distance of the handle to the touch location. In the default implementation, this is a tuple representing the x- and
y-distance of the handle to the touch location.
@returns Whether the handle has been hit.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23handleIsHitTouch(IHandle%2CPoint%2CPoint) Online Documentation}*/
handleIsHitTouch(handle: IHandle, location: Point, distance: Point): boolean;
/**
Installs the visual representation of the handles into the canvas into the {@link CanvasComponent#inputModeGroup}.
@param context
@param controller
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
{@link CanvasComponent#invalidate Invalidates} the canvas this mode is currently installed in.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23invalidate() Online Documentation}*/
invalidate(): void;
/**
Event recognizer used to determine whether the mouse is currently hovering over a handle.
@param sender
@param evt
@returns Whether the mouse is hovering over handle.
@see HandleInputMode#getClosestHitHandle
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23isHovering(Object%2CEventArgs) Online Documentation}*/
isHovering(sender: any, evt: EventArgs): boolean;
/**
Called after {@link HandleInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Triggers the {@link HandleInputMode#addDragCanceledListener DragCanceled} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onDragCanceled(InputModeEventArgs) Online Documentation}*/
onDragCanceled(evt: InputModeEventArgs): void;
/**
Triggers the {@link HandleInputMode#addDragCancelingListener DragCanceling} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onDragCanceling(InputModeEventArgs) Online Documentation}*/
onDragCanceling(evt: InputModeEventArgs): void;
/**
Triggered once the drag has been finalized.
This method triggers the {@link HandleInputMode#addDragFinishedListener DragFinished} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onDragFinished(InputModeEventArgs) Online Documentation}*/
onDragFinished(evt: InputModeEventArgs): void;
/**
Triggered before the drag will be finalized.
This method triggers the {@link HandleInputMode#addDragFinishingListener DragFinishing} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onDragFinishing(InputModeEventArgs) Online Documentation}*/
onDragFinishing(evt: InputModeEventArgs): void;
/**
Triggered at the end of each drag.
This method triggers the {@link HandleInputMode#addDraggedListener Dragged} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onDragged(InputModeEventArgs) Online Documentation}*/
onDragged(evt: InputModeEventArgs): void;
/**
Triggered at the start of each drag.
This method triggers the {@link HandleInputMode#addDraggingListener Dragging} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onDragging(InputModeEventArgs) Online Documentation}*/
onDragging(evt: InputModeEventArgs): void;
/**
Triggers the {@link HandleInputMode#addDragStartedListener DragStarted} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onDragStarted(InputModeEventArgs) Online Documentation}*/
onDragStarted(evt: InputModeEventArgs): void;
/**
Triggers the {@link HandleInputMode#addDragStartingListener DragStarting} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onDragStarting(InputModeEventArgs) Online Documentation}*/
onDragStarting(evt: InputModeEventArgs): void;
/**
Called after {@link HandleInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Removes a handle from the collection of handles managed by this instance.
@param handle The handle to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23removeHandle(IHandle) Online Documentation}*/
removeHandle(handle: IHandle): void;
/**
Removes the {@link ICanvasObject} that displays the handles from the canvas.
@param context
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link HandleInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link HandleInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* Disabling this mode will hide the handles and prevent the user from interacting with them.
*
* <br><b>Disabling the HandleInputMode on its parent input mode</b>
* <pre>
* mode.handleInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link HandleInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Gets or sets a value indicating whether handles are drawn in view coordinates or intermediate coordinates.
This only makes a difference if the {@link CanvasComponent#projection} is different from the {@link Matrix#IDENTITY} transform.
The default value is to draw handles in the intermediate coordinate system.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23useViewCoordinates Online Documentation}*/
useViewCoordinates: boolean;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#DEFAULT}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-default-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_DEFAULT_KEY Online Documentation}*/
static HANDLE_DRAWING_DEFAULT_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#MOVE}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-move-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_MOVE_KEY Online Documentation}*/
static HANDLE_DRAWING_MOVE_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#RESIZE}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-resize-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_RESIZE_KEY Online Documentation}*/
static HANDLE_DRAWING_RESIZE_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#ROTATE}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-rotate-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_ROTATE_KEY Online Documentation}*/
static HANDLE_DRAWING_ROTATE_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#SHEAR}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-shear-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_SHEAR_KEY Online Documentation}*/
static HANDLE_DRAWING_SHEAR_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#INVISIBLE}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-invisible-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_INVISIBLE_KEY Online Documentation}*/
static HANDLE_DRAWING_INVISIBLE_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#INVISIBLE} |
{@link HandleTypes#VARIANT2}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-invisible-variant2-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_INVISIBLE_VARIANT2_KEY Online Documentation}*/
static HANDLE_DRAWING_INVISIBLE_VARIANT2_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#INVISIBLE} |
{@link HandleTypes#VARIANT3}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-invisible-variant3-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_INVISIBLE_VARIANT3_KEY Online Documentation}*/
static HANDLE_DRAWING_INVISIBLE_VARIANT3_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#INVISIBLE} |
{@link HandleTypes#VARIANT4}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-invisible-variant4-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_INVISIBLE_VARIANT4_KEY Online Documentation}*/
static HANDLE_DRAWING_INVISIBLE_VARIANT4_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#DEFAULT} |
{@link HandleTypes#VARIANT2}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-default-variant2-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_DEFAULT_VARIANT2_KEY Online Documentation}*/
static HANDLE_DRAWING_DEFAULT_VARIANT2_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#MOVE} |
{@link HandleTypes#VARIANT2}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-move-variant2-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_MOVE_VARIANT2_KEY Online Documentation}*/
static HANDLE_DRAWING_MOVE_VARIANT2_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#RESIZE} |
{@link HandleTypes#VARIANT2}
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-resize-variant2-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_RESIZE_VARIANT2_KEY Online Documentation}*/
static HANDLE_DRAWING_RESIZE_VARIANT2_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#ROTATE} |
{@link HandleTypes#VARIANT2}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-rotate-variant2-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_ROTATE_VARIANT2_KEY Online Documentation}*/
static HANDLE_DRAWING_ROTATE_VARIANT2_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#SHEAR} |
{@link HandleTypes#VARIANT2}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-shear-variant2-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_SHEAR_VARIANT2_KEY Online Documentation}*/
static HANDLE_DRAWING_SHEAR_VARIANT2_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#DEFAULT} |
{@link HandleTypes#VARIANT3}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-default-variant3-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_DEFAULT_VARIANT3_KEY Online Documentation}*/
static HANDLE_DRAWING_DEFAULT_VARIANT3_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#MOVE} |
{@link HandleTypes#VARIANT3}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-move-variant3-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_MOVE_VARIANT3_KEY Online Documentation}*/
static HANDLE_DRAWING_MOVE_VARIANT3_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#RESIZE} |
{@link HandleTypes#VARIANT3}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-resize-variant3-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_RESIZE_VARIANT3_KEY Online Documentation}*/
static HANDLE_DRAWING_RESIZE_VARIANT3_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#ROTATE} |
{@link HandleTypes#VARIANT3}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-rotate-variant3-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_ROTATE_VARIANT3_KEY Online Documentation}*/
static HANDLE_DRAWING_ROTATE_VARIANT3_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#SHEAR} |
{@link HandleTypes#VARIANT3}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-shear-variant3-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_SHEAR_VARIANT3_KEY Online Documentation}*/
static HANDLE_DRAWING_SHEAR_VARIANT3_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#DEFAULT} |
{@link HandleTypes#VARIANT4}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-default-variant4-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_DEFAULT_VARIANT4_KEY Online Documentation}*/
static HANDLE_DRAWING_DEFAULT_VARIANT4_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#MOVE} |
{@link HandleTypes#VARIANT4}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-move-variant4-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_MOVE_VARIANT4_KEY Online Documentation}*/
static HANDLE_DRAWING_MOVE_VARIANT4_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#RESIZE} |
{@link HandleTypes#VARIANT4}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-resize-variant4-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_RESIZE_VARIANT4_KEY Online Documentation}*/
static HANDLE_DRAWING_RESIZE_VARIANT4_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#ROTATE} |
{@link HandleTypes#VARIANT4}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-drawing-variant4-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_ROTATE_VARIANT4_KEY Online Documentation}*/
static HANDLE_DRAWING_ROTATE_VARIANT4_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#SHEAR} |
{@link HandleTypes#VARIANT4}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-shear-variant4-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_SHEAR_VARIANT4_KEY Online Documentation}*/
static HANDLE_DRAWING_SHEAR_VARIANT4_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#WARP}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-warp-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_WARP_KEY Online Documentation}*/
static HANDLE_DRAWING_WARP_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#WARP} |
{@link HandleTypes#VARIANT2}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-warp-variant2-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_WARP_VARIANT2_KEY Online Documentation}*/
static HANDLE_DRAWING_WARP_VARIANT2_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#WARP} |
{@link HandleTypes#VARIANT3}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-warp-variant3-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_WARP_VARIANT3_KEY Online Documentation}*/
static HANDLE_DRAWING_WARP_VARIANT3_KEY: string;
/**
A {@link string} that can be used by instances of the {@link HandleInputMode} to look up a {@link IVisualTemplate} that will
be used for the rendering of {@link IHandle}s of {@link HandleTypes type} {@link HandleTypes#WARP} |
{@link HandleTypes#VARIANT4}.
The drawing returned should be centered around <code>(0,0)</code>.
The element in the SVG defs has the <code>yfiles-handle-warp-variant4-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23HANDLE_DRAWING_WARP_VARIANT4_KEY Online Documentation}*/
static HANDLE_DRAWING_WARP_VARIANT4_KEY: string;
/**
Gets or sets the event recognizer that determines whether to temporarily disable snapping.
@see HandleInputMode#enableSnappingRecognizer
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23disableSnappingRecognizer Online Documentation}*/
disableSnappingRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to reenable temporarily disabled snapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23enableSnappingRecognizer Online Documentation}*/
enableSnappingRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to start moving the handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23pressedRecognizer Online Documentation}*/
pressedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to move the handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23draggedRecognizer Online Documentation}*/
draggedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to cancel moving the handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23cancelRecognizer Online Documentation}*/
cancelRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish moving the handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23releasedRecognizer Online Documentation}*/
releasedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to start moving the handle via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23pressedRecognizerTouch Online Documentation}*/
pressedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to move the handle via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23draggedRecognizerTouch Online Documentation}*/
draggedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish moving the handle via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23releasedRecognizerTouch Online Documentation}*/
releasedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the mode used to render the handles.
@default RenderModes.SVG
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23renderMode Online Documentation}*/
renderMode: RenderModes;
/**
Gets or sets the collection of {@link IHandle}s this mode manages.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23handles Online Documentation}*/
handles: IObservableCollection<IHandle>;
/**
Gets or sets the {@link HandleInputMode#snapContext} which manages snapping model items to certain coordinates (for instance, other items).
If set to <code>null</code> (the default) this input mode tries to obtain the {@link HandleInputMode#snapContext} from the {@link IInputModeContext}. To explicitly
disable snapping, a {@link HandleInputMode#snapContext} implementation that does nothing has to be set to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23snapContext Online Documentation}*/
snapContext: SnapContext | null;
/**
Gets the handle that is currently being moved or <code>null</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23currentHandle Online Documentation}*/
readonly currentHandle: IHandle | null;
/**
* Gets an immutable snapshot of the {@link IModelItem}s affected by the currently modified {@link HandleInputMode#currentHandle handle} gesture.
* When the gesture is {@link HandleInputMode#addDragStartingListener starting} and the {@link HandleInputMode#currentHandle} is {@link IDragHandler#initializeDrag initialized}, the implementation can {@link IModelItemCollector#add register} the affected item(s) through the {@link IModelItemCollector}
* instance that is bound to the {@link HandleInputMode#createHandleInputModeContext context} available via its {@link ILookup#lookup}.
*
* Client code can register with the {@link HandleInputMode#addDragStartedListener DragStarted}
* event, as well as the {@link HandleInputMode#addDragFinishedListener DragFinished} event to get
* notified of the elements that may be or have been affected respectively by this input mode.
*
* If one needs to execute some code after some items have been moved by the {@link HandleInputMode} he can register a
* handler to its {@link HandleInputMode#addDragFinishedListener DragFinished} event. The moved items
* can be retrieved from the {@link HandleInputMode#affectedItems} property:
*
* <pre>
* mode.handleInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.handleInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23affectedItems Online Documentation}
*/
readonly affectedItems: IEnumerable<IModelItem>;
/**
Gets a value indicating whether a drag operation is currently in progress.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23isDragging Online Documentation}*/
isDragging: boolean;
/**
Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to add.
@see HandleInputMode#removeDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragFinishing Online Documentation}*/
addDragFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to remove.
@see HandleInputMode#addDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragFinishing Online Documentation}*/
removeDragFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Adds the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
* If one needs to execute some code after some items have been moved by the {@link HandleInputMode} he can register a
* handler to its {@link HandleInputMode#addDragFinishedListener DragFinished} event. The moved items
* can be retrieved from the {@link HandleInputMode#affectedItems} property:
*
* <pre>
* mode.handleInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.handleInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see HandleInputMode#removeDragFinishedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragFinished Online Documentation}
*/
addDragFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Removes the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
* If one needs to execute some code after some items have been moved by the {@link HandleInputMode} he can register a
* handler to its {@link HandleInputMode#addDragFinishedListener DragFinished} event. The moved items
* can be retrieved from the {@link HandleInputMode#affectedItems} property:
*
* <pre>
* mode.handleInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.handleInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see HandleInputMode#addDragFinishedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragFinished Online Documentation}
*/
removeDragFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to add.
@see HandleInputMode#removeDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragStarting Online Documentation}*/
addDragStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to remove.
@see HandleInputMode#addDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragStarting Online Documentation}*/
removeDragStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to add.
@see HandleInputMode#removeDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragStarted Online Documentation}*/
addDragStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to remove.
@see HandleInputMode#addDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragStarted Online Documentation}*/
removeDragStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to add.
@see HandleInputMode#removeDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23Dragging Online Documentation}*/
addDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to remove.
@see HandleInputMode#addDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23Dragging Online Documentation}*/
removeDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to add.
@see HandleInputMode#removeDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23Dragged Online Documentation}*/
addDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to remove.
@see HandleInputMode#addDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23Dragged Online Documentation}*/
removeDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to add.
@see HandleInputMode#removeDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragCanceled Online Documentation}*/
addDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to remove.
@see HandleInputMode#addDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragCanceled Online Documentation}*/
removeDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to add.
@see HandleInputMode#removeDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragCanceling Online Documentation}*/
addDragCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to remove.
@see HandleInputMode#addDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleInputMode%23DragCanceling Online Documentation}*/
removeDragCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
static $class: Class;
private $_HandleInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HandleInputMode;
}
export interface IDragHandler {
}
/**
Interface for an object that can be used to drag something displayed in a {@link CanvasComponent}
Items can be dragged with the mouse and an instance of this class will modify their position accordingly. A drag will
trigger the invocation of {@link IDragHandler#initializeDrag}, zero or more {@link IDragHandler#handleMove}
calls, and will be finalized by either {@link IDragHandler#dragFinished} or {@link IDragHandler#cancelDrag}.
@see IPositionHandler
@see IHandle
@see IInputModeContext
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDragHandler Online Documentation}*/
export abstract class IDragHandler {
/**
Called by clients to indicate that the dragging has been canceled by the user.
This method may be called after the initial {@link IDragHandler#initializeDrag} and zero or more invocations of
{@link IDragHandler#handleMove}. Implementations should reset the position of the items they modify to their
initial state. Alternatively to this method the {@link IDragHandler#dragFinished} method might be called.
@param context The context to retrieve information about the drag from.
@param originalLocation The value of the coordinate of the {@link IDragHandler#location} property at the time of {@link IDragHandler#initializeDrag}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDragHandler%23cancelDrag(IInputModeContext%2CPoint) Online Documentation}*/
abstract cancelDrag(context: IInputModeContext, originalLocation: Point): void;
/**
Creates an implementation of the interface {@link IDragHandler} from the given definition.
The given object must contain exactly the abstract members of the {@link IDragHandler} interface. No member may be
omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IDragHandler} interface.
@returns An instance of the {@link IDragHandler} interface based on the given definition.*/
static create(interfaceDefinition: {
initializeDrag(context: IInputModeContext): void;
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
location: IPoint;
}): IDragHandler;
/**
Called by clients to indicate that the repositioning has just been finished.
This method may be called after the initial {@link IDragHandler#initializeDrag} and zero or more invocations of
{@link IDragHandler#handleMove}. Alternatively to this method the {@link IDragHandler#cancelDrag} method might
be called.
@param context The context to retrieve information about the drag from.
@param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
implementation the {@link IDragHandler#location} may or may not be modified to reflect the new value. This is the same value as delivered in the last
invocation of {@link IDragHandler#handleMove}
@param originalLocation The value of the {@link IDragHandler#location} property at the time of {@link IDragHandler#initializeDrag}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDragHandler%23dragFinished(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
abstract dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
Called by clients to indicate that the element has been dragged and its position should be updated.
This method may be called more than once after an initial {@link IDragHandler#initializeDrag} and will the final
call will be followed by either one {@link IDragHandler#dragFinished} or one {@link IDragHandler#cancelDrag}
call.
@param context The context to retrieve information about the drag from.
@param originalLocation The value of the {@link IDragHandler#location} property at the time of {@link IDragHandler#initializeDrag}.
@param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
implementation the {@link IDragHandler#location} may or may not be modified to reflect the new value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDragHandler%23handleMove(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
abstract handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
Called by clients to indicate that the element is going to be dragged.
This call will be followed by one or more calls to {@link IDragHandler#handleMove}, and a final {@link IDragHandler#dragFinished}
or {@link IDragHandler#cancelDrag}.
@param context The context to retrieve information about the drag from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDragHandler%23initializeDrag(IInputModeContext) Online Documentation}*/
abstract initializeDrag(context: IInputModeContext): void;
/**
Gets a view of the location of the item.
The point describes the current world coordinate of the element that can be modified by this handler.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDragHandler%23location Online Documentation}*/
abstract readonly location: IPoint;
static $class: Class;
private $_IDragHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IDragHandler;
}
/**
The policy constants used by {@link IGridConstraintProvider}
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPolicy Online Documentation}*/
export enum SnapPolicy {
/**
Don't perform snapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPolicy%23NO_SNAPPING Online Documentation}*/
NO_SNAPPING,
/**
Snap to the nearest grid coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPolicy%23TO_NEAREST Online Documentation}*/
TO_NEAREST,
/**
Snap to the nearest grid coordinate whose value is not less than the current coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPolicy%23TO_GREATER Online Documentation}*/
TO_GREATER,
/**
Snap to the nearest grid coordinate whose value is not greater than the current coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPolicy%23TO_SMALLER Online Documentation}*/
TO_SMALLER
}
/** Automatic conversion to {@link SnapPolicy}. */
type SnapPolicyStringValues = "no-snapping" | "to-nearest" | "to-greater" | "to-smaller";
export interface IGridConstraintProvider<T> {
}
/**
* Interface for classes that can snap coordinates of items of a specific type to a grid.
* Enabling grid snapping
*
* <pre>
* const snapContext = new GraphSnapContext()
*
* const gridInfo = new GridInfo(50)
* snapContext.nodeGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.bendGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.portGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.gridSnapType = GridSnapTypes.GRID_POINTS
*
* graphEditorInputMode.snapContext = snapContext
* </pre>
*
* Note that the grid visualization has to be enabled separately
*
* <pre>
* graphComponent.backgroundGroup.addChild(new GridVisualCreator(gridInfo))
* </pre>
*
* Type parameter <code>T</code>: The type of the items this class can snap
* @see GridConstraintProvider
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGridConstraintProvider Online Documentation}
*/
export abstract class IGridConstraintProvider<T> {
/**
Creates an implementation of the interface {@link IGridConstraintProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link IGridConstraintProvider} interface. No member may be omitted and no additional
members can be added. While additional custom members can later be added directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IGridConstraintProvider} interface.
@returns An instance of the {@link IGridConstraintProvider} interface based on the given definition.*/
static create<T>(interfaceDefinition: {
snapToGrid(context: IInputModeContext, item: T, location: IMutablePoint, xSnapPolicy: SnapPolicy | SnapPolicyStringValues, ySnapPolicy: SnapPolicy | SnapPolicyStringValues): boolean;
horizontalGridWidth: number;
verticalGridWidth: number;
gridOrigin: Point;
}): IGridConstraintProvider<T>;
/**
Performs the actual snapping to grid coordinates.
@param context The context in which the snapping should occur.
@param item The item of which the coordinates should be snapped.
@param location The coordinates to snap.
@param xSnapPolicy The policy for the x coordinate.
@param ySnapPolicy The policy for the y coordinate.
@returns Whether the point has been snapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGridConstraintProvider%23snapToGrid(IInputModeContext%2CT%2CIMutablePoint%2CSnapPolicy%2CSnapPolicy) Online Documentation}*/
abstract snapToGrid(context: IInputModeContext, item: T, location: IMutablePoint, xSnapPolicy: SnapPolicy | SnapPolicyStringValues, ySnapPolicy: SnapPolicy | SnapPolicyStringValues): boolean;
/**
Gets the canonical grid width in horizontal direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGridConstraintProvider%23horizontalGridWidth Online Documentation}*/
abstract readonly horizontalGridWidth: number;
/**
Gets the canonical grid width in vertical direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGridConstraintProvider%23verticalGridWidth Online Documentation}*/
abstract readonly verticalGridWidth: number;
/**
Gets the canonical origin of the grid.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGridConstraintProvider%23gridOrigin Online Documentation}*/
abstract readonly gridOrigin: Point;
static $class: Class;
private $_IGridConstraintProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is IGridConstraintProvider<T>;
}
export interface INodeGridConstraintProvider extends IGridConstraintProvider<INode> {
}
/**
This is a flagging interface for instances of {@link IGridConstraintProvider} that can snap coordinates of {@link INode INodes} to a grid.
An implementation of this interface can be queried using the lookup of a {@link IInputModeContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeGridConstraintProvider Online Documentation}*/
export abstract class INodeGridConstraintProvider {
/**
Creates an implementation of the interface {@link INodeGridConstraintProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link INodeGridConstraintProvider} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link INodeGridConstraintProvider} interface.
@returns An instance of the {@link INodeGridConstraintProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
snapToGrid(context: IInputModeContext, item: INode | null, location: IMutablePoint, xSnapPolicy: SnapPolicy | SnapPolicyStringValues, ySnapPolicy: SnapPolicy | SnapPolicyStringValues): boolean;
horizontalGridWidth: number;
verticalGridWidth: number;
gridOrigin: Point;
}): INodeGridConstraintProvider;
static $class: Class;
private $_INodeGridConstraintProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeGridConstraintProvider;
}
export interface ILabelGridConstraintProvider extends IGridConstraintProvider<ILabel> {
}
/**
This is a flagging interface for instances of {@link IGridConstraintProvider} that can snap coordinates of {@link ILabel ILabels} to a grid.
An implementation of this interface can be queried using the lookup of a {@link IInputModeContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelGridConstraintProvider Online Documentation}*/
export abstract class ILabelGridConstraintProvider {
/**
Creates an implementation of the interface {@link ILabelGridConstraintProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link ILabelGridConstraintProvider} interface.
No member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link ILabelGridConstraintProvider} interface.
@returns An instance of the {@link ILabelGridConstraintProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
snapToGrid(context: IInputModeContext, item: ILabel | null, location: IMutablePoint, xSnapPolicy: SnapPolicy | SnapPolicyStringValues, ySnapPolicy: SnapPolicy | SnapPolicyStringValues): boolean;
horizontalGridWidth: number;
verticalGridWidth: number;
gridOrigin: Point;
}): ILabelGridConstraintProvider;
static $class: Class;
private $_ILabelGridConstraintProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelGridConstraintProvider;
}
export interface ILabelOwnerGridConstraintProvider extends IGridConstraintProvider<ILabelOwner> {
}
/**
This is a flagging interface for instances of {@link IGridConstraintProvider} that can snap coordinates of {@link ILabelOwner ILabelOwners} to a grid.
An implementation of this interface can be queried using the lookup of a {@link IInputModeContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelOwnerGridConstraintProvider Online Documentation}*/
export abstract class ILabelOwnerGridConstraintProvider {
/**
Creates an implementation of the interface {@link ILabelOwnerGridConstraintProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link ILabelOwnerGridConstraintProvider}
interface. No member may be omitted and no additional members can be added. While additional custom members can later be
added directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link ILabelOwnerGridConstraintProvider} interface.
@returns An instance of the {@link ILabelOwnerGridConstraintProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
snapToGrid(context: IInputModeContext, item: ILabelOwner | null, location: IMutablePoint, xSnapPolicy: SnapPolicy | SnapPolicyStringValues, ySnapPolicy: SnapPolicy | SnapPolicyStringValues): boolean;
horizontalGridWidth: number;
verticalGridWidth: number;
gridOrigin: Point;
}): ILabelOwnerGridConstraintProvider;
static $class: Class;
private $_ILabelOwnerGridConstraintProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelOwnerGridConstraintProvider;
}
export interface IPortGridConstraintProvider extends IGridConstraintProvider<IPort> {
}
/**
This is a flagging interface for instances of {@link IGridConstraintProvider} that can snap coordinates of {@link IPort IPorts} to a grid.
An implementation of this interface can be queried using the lookup of a {@link IInputModeContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortGridConstraintProvider Online Documentation}*/
export abstract class IPortGridConstraintProvider {
/**
Creates an implementation of the interface {@link IPortGridConstraintProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link IPortGridConstraintProvider} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IPortGridConstraintProvider} interface.
@returns An instance of the {@link IPortGridConstraintProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
snapToGrid(context: IInputModeContext, item: IPort | null, location: IMutablePoint, xSnapPolicy: SnapPolicy | SnapPolicyStringValues, ySnapPolicy: SnapPolicy | SnapPolicyStringValues): boolean;
horizontalGridWidth: number;
verticalGridWidth: number;
gridOrigin: Point;
}): IPortGridConstraintProvider;
static $class: Class;
private $_IPortGridConstraintProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortGridConstraintProvider;
}
export interface IBendGridConstraintProvider extends IGridConstraintProvider<IBend> {
}
/**
This is a flagging interface for instances of {@link IGridConstraintProvider} that can snap coordinates of {@link IBend IBends} to a grid.
An implementation of this interface can be queried using the lookup of a {@link IInputModeContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendGridConstraintProvider Online Documentation}*/
export abstract class IBendGridConstraintProvider {
/**
Creates an implementation of the interface {@link IBendGridConstraintProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link IBendGridConstraintProvider} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IBendGridConstraintProvider} interface.
@returns An instance of the {@link IBendGridConstraintProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
snapToGrid(context: IInputModeContext, item: IBend | null, location: IMutablePoint, xSnapPolicy: SnapPolicy | SnapPolicyStringValues, ySnapPolicy: SnapPolicy | SnapPolicyStringValues): boolean;
horizontalGridWidth: number;
verticalGridWidth: number;
gridOrigin: Point;
}): IBendGridConstraintProvider;
static $class: Class;
private $_IBendGridConstraintProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IBendGridConstraintProvider;
}
export interface GridConstraintProvider<T> extends YObject, IGridConstraintProvider<T> {
/**
@inheritdoc*/
snapToGrid(context: IInputModeContext, item: T, location: IMutablePoint, xSnapPolicy: SnapPolicy | SnapPolicyStringValues, ySnapPolicy: SnapPolicy | SnapPolicyStringValues): boolean;
} /**
* A simple integer based implementation of {@link IGridConstraintProvider}.
* Enabling grid snapping
*
* <pre>
* const snapContext = new GraphSnapContext()
*
* const gridInfo = new GridInfo(50)
* snapContext.nodeGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.bendGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.portGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.gridSnapType = GridSnapTypes.GRID_POINTS
*
* graphEditorInputMode.snapContext = snapContext
* </pre>
*
* Note that the grid visualization has to be enabled separately
*
* <pre>
* graphComponent.backgroundGroup.addChild(new GridVisualCreator(gridInfo))
* </pre>
*
* Type parameter <code>T</code>: The type of the items.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GridConstraintProvider Online Documentation}
*/
export class GridConstraintProvider<T> {
/**
Creates a new instance using the given grid info.
@param gridInfo The grid info object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridConstraintProvider%23GridConstraintProvider(GridInfo) Online Documentation}*/
constructor(gridInfo: GridInfo);
/**
Creates a new instance using the given grid specification.
@param gridWidth Width of the grid.
@param gridHeight Height of the grid.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridConstraintProvider%23GridConstraintProvider(number%2Cnumber) Online Documentation}*/
constructor(gridWidth: number, gridHeight: number);
/**
Creates a new instance using the given grid distances.
@param gridSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridConstraintProvider%23GridConstraintProvider(number) Online Documentation}*/
constructor(gridSize: number);
/**
Gets or sets the grid info that describes the geometry of the grid.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridConstraintProvider%23gridInfo Online Documentation}*/
gridInfo: GridInfo;
/**
Gets the grid height.
This implementation delegates to the {@link GridConstraintProvider#gridInfo}
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridConstraintProvider%23horizontalGridWidth Online Documentation}*/
readonly horizontalGridWidth: number;
/**
Gets the grid width.
This implementation delegates to the {@link GridConstraintProvider#gridInfo}
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridConstraintProvider%23verticalGridWidth Online Documentation}*/
readonly verticalGridWidth: number;
/**
Gets the grid origin.
This implementation delegates to the {@link GridConstraintProvider#gridInfo}
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridConstraintProvider%23gridOrigin Online Documentation}*/
readonly gridOrigin: Point;
static $class: Class;
private $_GridConstraintProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is GridConstraintProvider<T>;
}
/**
Describes the type of a {@link IHandle}.
This type can be used by the rendering engine to render different types of handles differently.
@see IHandle#type
@see HandleInputMode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes Online Documentation}*/
export enum HandleTypes {
/**
A handle which serves a general purpose but should not be displayed.
This is the basic type of a handle and can be queried using the {@link HandleTypes#TYPE_MASK}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23INVISIBLE Online Documentation}*/
INVISIBLE,
/**
A default handle which serves a general purpose.
This is the basic type of a handle and can be queried using the {@link HandleTypes#TYPE_MASK}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23DEFAULT Online Documentation}*/
DEFAULT,
/**
A handle which can be used to resize something.
This is the basic type of a handle and can be queried using the {@link HandleTypes#TYPE_MASK}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23RESIZE Online Documentation}*/
RESIZE,
/**
A handle which can be used to rotate something.
This is the basic type of a handle and can be queried using the {@link HandleTypes#TYPE_MASK}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23ROTATE Online Documentation}*/
ROTATE,
/**
A handle which can be used to move something.
This is the basic type of a handle and can be queried using the {@link HandleTypes#TYPE_MASK}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23MOVE Online Documentation}*/
MOVE,
/**
A handle which can be used to shear an object.
This is the basic type of a handle and can be queried using the {@link HandleTypes#TYPE_MASK}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23SHEAR Online Documentation}*/
SHEAR,
/**
A handle which can be used to warp an object.
This is the basic type of a handle and can be queried using the {@link HandleTypes#TYPE_MASK}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23WARP Online Documentation}*/
WARP,
/**
A bitmask that can be used to query the basic type of a handle type.
@see HandleTypes#DEFAULT
@see HandleTypes#MOVE
@see HandleTypes#RESIZE
@see HandleTypes#ROTATE
@see HandleTypes#SHEAR
@see HandleTypes#WARP
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23TYPE_MASK Online Documentation}*/
TYPE_MASK,
/**
A default variant modifier that decorates a basic type.
@see HandleTypes#VARIANT_MASK
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23VARIANT1 Online Documentation}*/
VARIANT1,
/**
The second variant modifier that decorates a basic type.
@see HandleTypes#VARIANT_MASK
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23VARIANT2 Online Documentation}*/
VARIANT2,
/**
The third variant modifier that decorates a basic type.
@see HandleTypes#VARIANT_MASK
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23VARIANT3 Online Documentation}*/
VARIANT3,
/**
The forth variant modifier that decorates a basic type.
@see HandleTypes#VARIANT_MASK
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23VARIANT4 Online Documentation}*/
VARIANT4,
/**
A bitmask that can be used to query the variant of a basic handle type.
@see HandleTypes#VARIANT1
@see HandleTypes#VARIANT2
@see HandleTypes#VARIANT3
@see HandleTypes#VARIANT4
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23VARIANT_MASK Online Documentation}*/
VARIANT_MASK,
/**
A modifier that can be used to tell the rendering engine to render the visual representation of this handle with an
offset to the north.
@see HandleTypes#OFFSET_MASK
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23OFFSET_NORTH Online Documentation}*/
OFFSET_NORTH,
/**
A modifier that can be used to tell the rendering engine to render the visual representation of this handle with an
offset to the east.
@see HandleTypes#OFFSET_MASK
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23OFFSET_EAST Online Documentation}*/
OFFSET_EAST,
/**
A modifier that can be used to tell the rendering engine to render the visual representation of this handle with an
offset to the south.
@see HandleTypes#OFFSET_MASK
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23OFFSET_SOUTH Online Documentation}*/
OFFSET_SOUTH,
/**
A modifier that can be used to tell the rendering engine to render the visual representation of this handle with an
offset to the west.
@see HandleTypes#OFFSET_MASK
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23OFFSET_WEST Online Documentation}*/
OFFSET_WEST,
/**
A bitmask that can be used to query the offsets of a basic handle type.
@see HandleTypes#OFFSET_NORTH
@see HandleTypes#OFFSET_EAST
@see HandleTypes#OFFSET_SOUTH
@see HandleTypes#OFFSET_WEST
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleTypes%23OFFSET_MASK Online Documentation}*/
OFFSET_MASK
}
type HandleTypesStringValues = "invisible" | "default" | "resize" | "rotate" | "move" | "shear" | "warp" | "type-mask" | "variant1" | "variant2" | "variant3" | "variant4" | "variant-mask" | "offset-north" | "offset-east" | "offset-south" | "offset-west" | "offset-mask";
/** Automatic conversion to {@link HandleTypes}. */
type HandleTypesConvertible = HandleTypesStringValues | HandleTypesStringValues[];
/**
The possible positions of a rectangle {@link IHandle}s.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions Online Documentation}*/
export enum HandlePositions {
/**
Indicates no position at all.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23NONE Online Documentation}*/
NONE,
/**
Indicates the position at north west.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23NORTH_WEST Online Documentation}*/
NORTH_WEST,
/**
Indicates the position at north.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23NORTH Online Documentation}*/
NORTH,
/**
Indicates the position at north east.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23NORTH_EAST Online Documentation}*/
NORTH_EAST,
/**
Indicates the position at west.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23WEST Online Documentation}*/
WEST,
/**
Indicates the position at the center.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23CENTER Online Documentation}*/
CENTER,
/**
Indicates the position at east.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23EAST Online Documentation}*/
EAST,
/**
Indicates the position at south west.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23SOUTH_WEST Online Documentation}*/
SOUTH_WEST,
/**
Indicates the position at south.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23SOUTH Online Documentation}*/
SOUTH,
/**
Indicates the position at south east.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23SOUTH_EAST Online Documentation}*/
SOUTH_EAST,
/**
A combination of all possible handle positions at the four corners of a rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23CORNERS Online Documentation}*/
CORNERS,
/**
A combination of the two horizontal handles of a rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23HORIZONTAL Online Documentation}*/
HORIZONTAL,
/**
A combination of the two vertical handles of a rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23VERTICAL Online Documentation}*/
VERTICAL,
/**
A combination of all possible handle positions at the border of a rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23BORDER Online Documentation}*/
BORDER,
/**
A combination of all possible handle positions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandlePositions%23ALL Online Documentation}*/
ALL
}
type HandlePositionsStringValues = "none" | "north-west" | "north" | "north-east" | "west" | "center" | "east" | "south-west" | "south" | "south-east" | "corners" | "horizontal" | "vertical" | "border" | "all";
/** Automatic conversion to {@link HandlePositions}. */
type HandlePositionsConvertible = HandlePositionsStringValues | HandlePositionsStringValues[];
export interface IHandle extends IDragHandler {
}
/**
Interface for a handle that can be displayed in a {@link CanvasComponent} as a UI element for the user to modify the
contents of the control.
Handles can be dragged with the mouse and will thereby modify their context. Typically client code will use the {@link IDragHandler#location} to
render a visual handle that can be dragged using the mouse. A drag will trigger the invocation of
{@link IDragHandler#initializeDrag}, zero or more {@link IDragHandler#handleMove} calls, and will be finalized
by either {@link IDragHandler#dragFinished} or {@link IDragHandler#cancelDrag}.
@see HandleInputMode
@see IDragHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHandle Online Documentation}*/
export abstract class IHandle {
/**
Creates an implementation of the interface {@link IHandle} from the given definition.
The given object must contain exactly the abstract members of the {@link IHandle} interface. No member may be
omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IHandle} interface.
@returns An instance of the {@link IHandle} interface based on the given definition.*/
static create(interfaceDefinition: {
initializeDrag(context: IInputModeContext): void;
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
type: HandleTypes;
cursor: Cursor;
location: IPoint;
}): IHandle;
/**
Gets the type of the handle that can be used by the rendering engine to render types differently.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHandle%23type Online Documentation}*/
abstract readonly type: HandleTypes;
/**
Gets the cursor to display when the mouse hovers over or drags this handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHandle%23cursor Online Documentation}*/
abstract readonly cursor: Cursor;
static $class: Class;
private $_IHandle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IHandle;
}
export interface IHandleProvider {
}
/**
Interface for an object that provides a number of {@link IHandle} implementations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHandleProvider Online Documentation}*/
export abstract class IHandleProvider {
/**
Creates an implementation of the interface {@link IHandleProvider} by using the given function as implementation
for its {@link IHandleProvider#getHandles} method.
@param getHandles A function for {@link IHandleProvider}'s single abstract method {@link IHandleProvider#getHandles}.
@returns An instance of the {@link IHandleProvider} interface based on the given function.*/
static create(getHandles: (context: IInputModeContext) => IEnumerable<IHandle>): IHandleProvider;
/**
Returns a collection of zero or more {@link IHandle} implementations that are associated with this instance.
@param context
@returns A collection of handles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHandleProvider%23getHandles(IInputModeContext) Online Documentation}*/
abstract getHandles(context: IInputModeContext): IEnumerable<IHandle>;
static $class: Class;
private $_IHandleProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IHandleProvider;
}
export interface IReshapeHandleProvider {
}
/**
A specialized version of the {@link IHandleProvider} interface that can be used to query {@link IHandle}
implementation for resizing an object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReshapeHandleProvider Online Documentation}*/
export abstract class IReshapeHandleProvider {
/**
Creates an implementation of the interface {@link IReshapeHandleProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link IReshapeHandleProvider} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IReshapeHandleProvider} interface.
@returns An instance of the {@link IReshapeHandleProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
getAvailableHandles(context: IInputModeContext): HandlePositions;
getHandle(context: IInputModeContext, position: HandlePositions | HandlePositionsConvertible): IHandle;
}): IReshapeHandleProvider;
/**
Returns a bitwise combination of all of the {@link HandlePositions} this interface can provide an implementation
for.
Client code will use this method to query the available positions and may then call for each of the positions the {@link IReshapeHandleProvider#getHandle}
method to retrieve an implementation.
@param context The context for which the handles are queried.
@returns A bitwise combination of all positions the {@link IReshapeHandleProvider#getHandle} method can be queried for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReshapeHandleProvider%23getAvailableHandles(IInputModeContext) Online Documentation}*/
abstract getAvailableHandles(context: IInputModeContext): HandlePositions;
/**
Returns an implementation of an {@link IHandle} for the given position, that can be used to reshape an object.
This method may be called for each possible single position contained in the set as returned by
{@link IReshapeHandleProvider#getAvailableHandles}.
@param context The context for which the handles are queried.
@param position The single position a handle implementation should be returned for.
@returns An implementation of {@link IHandle} for the given position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReshapeHandleProvider%23getHandle(IInputModeContext%2CHandlePositions) Online Documentation}*/
abstract getHandle(context: IInputModeContext, position: HandlePositions | HandlePositionsConvertible): IHandle;
static $class: Class;
private $_IReshapeHandleProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IReshapeHandleProvider;
}
export interface IHighlightIndicatorInstaller extends ICanvasObjectInstaller {
}
/**
An interface for objects that can install a visual representation of a highlight decoration of an item in the model
displayed in a canvas.
This interface is a tagging sub interface of the {@link ICanvasObjectInstaller}. This interface is used for {@link ILookup#lookup}
operations.
@see HighlightIndicatorManager
@see ISelectionIndicatorInstaller
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHighlightIndicatorInstaller Online Documentation}*/
export abstract class IHighlightIndicatorInstaller {
/**
Creates an implementation of the interface {@link IHighlightIndicatorInstaller} by using the given function as
implementation for its {@link IHighlightIndicatorInstaller#addCanvasObject} method.
@param addCanvasObject A function for {@link IHighlightIndicatorInstaller}'s single abstract method
{@link IHighlightIndicatorInstaller#addCanvasObject}.
@returns An instance of the {@link IHighlightIndicatorInstaller} interface based on the given function.*/
static create(addCanvasObject: (context: ICanvasContext, group: ICanvasObjectGroup, item: any | null) => ICanvasObject): IHighlightIndicatorInstaller;
static $class: Class;
private $_IHighlightIndicatorInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IHighlightIndicatorInstaller;
}
export interface IFocusIndicatorInstaller extends ICanvasObjectInstaller {
}
/**
An interface for objects that can install a visual focus indicator decoration of an item in the model displayed in a
canvas.
This interface is a tagging sub interface of the {@link ICanvasObjectInstaller}. This interface is used for {@link ILookup#lookup}
operations.
@see FocusIndicatorManager
@see ISelectionIndicatorInstaller
@see IHighlightIndicatorInstaller
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFocusIndicatorInstaller Online Documentation}*/
export abstract class IFocusIndicatorInstaller {
/**
Creates an implementation of the interface {@link IFocusIndicatorInstaller} by using the given function as
implementation for its {@link IFocusIndicatorInstaller#addCanvasObject} method.
@param addCanvasObject A function for {@link IFocusIndicatorInstaller}'s single abstract method
{@link IFocusIndicatorInstaller#addCanvasObject}.
@returns An instance of the {@link IFocusIndicatorInstaller} interface based on the given function.*/
static create(addCanvasObject: (context: ICanvasContext, group: ICanvasObjectGroup, item: any | null) => ICanvasObject): IFocusIndicatorInstaller;
static $class: Class;
private $_IFocusIndicatorInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IFocusIndicatorInstaller;
}
export interface IInputMode {
}
/**
Interface implemented by input modes that can be registered with a {@link CanvasComponent} instance.
Input modes capture user gestures (or any other kind of "input") and manipulate the contents of the scene graph or the
model behind the scene graph.
Implementations may request an input mutex from the {@link ConcurrencyController} that will be given to them in
their {@link IInputMode#install} method. This allows them to exclusively modify the contents of the view. If
another {@link IInputMode} successfully acquires the mutex, the {@link ConcurrencyController} will be
{@link ConcurrencyController#active deactivated}.
@see InputModeBase
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputMode Online Documentation}*/
export abstract class IInputMode {
/**
* Called by the client to unconditionally cancel all editing.
* This will be called prior to the uninstalling of this instance.
*
* In order to stop an active input mode manually, client code should use the following idiom:
*
* <pre>
* if (!mode.tryStop()) {
*   mode.cancel()
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IInputMode%23cancel() Online Documentation}
*/
abstract cancel(): void;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
When this instance gets {@link IInputMode#uninstall uninstalled} from the context the same context instance will be passed to it.
Implementations may hold a reference to the <code>context</code> instance and use it while they are being installed.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link ConcurrencyController} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
abstract install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called by the client in order to stop a current editing progress.
This should stop the current edit, if one is in progress and possibly commit all of the changes. If stopping is not
possible, this method can return <code>false</code>
@returns <code>true</code> if and only if the editing has been stopped or there was no edit in progress
@see IInputMode#cancel
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputMode%23tryStop() Online Documentation}*/
abstract tryStop(): boolean;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputMode%23uninstall(IInputModeContext) Online Documentation}*/
abstract uninstall(context: IInputModeContext): void;
/**
Gets the context instance this mode is currently installed in or <code>null</code> if this instance is not installed.
Note that this instance should not be passed down to subordinate modes or instances. Instead a corresponding context
that has this instance set as the {@link IInputModeContext#parentInputMode} should be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputMode%23inputModeContext Online Documentation}*/
abstract readonly inputModeContext: IInputModeContext | null;
/**
Gets the priority of this input mode.
The priority will influence the order in which the modes will be {@link IInputMode#install installed} into the canvas control. The lower the
priority value, the earlier it will be installed. If two modes are installed using the same priority value, the first
one will be installed earlier.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputMode%23priority Online Documentation}*/
abstract readonly priority: number;
static $class: Class;
private $_IInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IInputMode;
}
export interface IInputModeContext extends ICanvasContext {
}
/**
The interface for the context object that is passed to {@link IInputMode} instances during {@link IInputMode#install}
and {@link IInputMode#uninstall} and {@link IDragHandler} implementations like {@link IPositionHandler}
and {@link IHandle} to provide additional context for the implementations.
Implementations of this interface can carry additional information through their {@link ILookup#lookup} method.
@see ILookup
@see IInputMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext Online Documentation}*/
export abstract class IInputModeContext {
/**
Factory method that creates an {@link IInputModeContext} that uses the provided parent mode, parent context, and
lookup decoration.
@param parent The input mode this context is created for.
@param parentContext A wrapped {@link IInputModeContext}. This implementation delegates to to this context.
@param lookupCallback A callback which is used for this context's {@link ILookup#lookup} method. The context's {@link ILookup#lookup}
will query <code>lookupCallback</code> first. If it returns <code>null</code>, the <code>parentContext</code>'s {@link ILookup#lookup} will be queried,
then.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23createInputModeContext(IInputMode%2CIInputModeContext%2Cfunction(Object%2CClass)%3AObject) Online Documentation}*/
static createInputModeContext(parent: IInputMode, parentContext: IInputModeContext, lookupCallback: (subject: any | null, type: Class | null) => any | null): IInputModeContext;
/**
Factory method that creates an {@link IInputModeContext} that uses the provided parent mode, parent context, and
lookup decoration.
@param parent The input mode this context is created for.
@param parentContext A wrapped {@link IInputModeContext}. This implementation delegates to to this context.
@param lookup A {@link ILookup} implementation which is used for this context's {@link ILookup#lookup} method. The context's
{@link ILookup#lookup} will query <code>lookup</code> first. If it returns <code>null</code>, the <code>parentContext</code>'s {@link ILookup#lookup}
will be queried, then.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23createInputModeContext(IInputMode%2CIInputModeContext%2CILookup) Online Documentation}*/
static createInputModeContext(parent: IInputMode, parentContext: IInputModeContext, lookup: ILookup): IInputModeContext;
/**
Factory method that creates an {@link IInputModeContext} that uses the provided parent mode and lookup decoration.
@param parent The input mode this context is created for.
@param lookupCallback Callback which is used by this context's {@link ILookup#lookup} method. The callback is used exclusively, i.e. the
{@link ILookup#lookup} method returns null if the callback returns <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23createInputModeContext(IInputMode%2Cfunction(Object%2CClass)%3AObject) Online Documentation}*/
static createInputModeContext(parent: IInputMode, lookupCallback: (subject: any | null, type: Class | null) => any | null): IInputModeContext;
/**
Factory method that creates an {@link IInputModeContext} that uses the provided values.
@param canvas The {@link CanvasComponent} to use, may not be <code>null</code>.
@param [parent=null] The mode to set as the parent. This may be <code>null</code> for the canvas' context.
@param [lookup=null] The lookup to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23createInputModeContext(CanvasComponent%2CIInputMode%2CILookup) Online Documentation}*/
static createInputModeContext(canvas: CanvasComponent, parent?: IInputMode | null, lookup?: ILookup | null): IInputModeContext;
/**
Factory method that creates an {@link IInputModeContext} that uses the provided values.
@param options The parameters to pass.
@param options.canvas The {@link CanvasComponent} to use, may not be <code>null</code>.
@param [options.parent=null] The mode to set as the parent. This may be <code>null</code> for the canvas' context.
@param [options.lookup=null] The lookup to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23createInputModeContext(CanvasComponent%2CIInputMode%2CILookup) Online Documentation}*/
static createInputModeContext(options: {
canvas: CanvasComponent;
parent?: IInputMode | null;
lookup?: ILookup | null;
}): IInputModeContext;
/**
Factory method that creates an {@link IInputModeContext} that uses the provided parent mode and lookup decoration.
@param parent The input mode this context is created for.
@param [lookup=null] The {@link ILookup} implementation which is used for this context's {@link ILookup#lookup} method. If one is
provided it is used exclusively, i.e. the {@link ILookup#lookup} method returns null if <code>lookup</code> returns <code>null</code>. If
none is provided the <code>parent</code>'s {@link IInputMode#inputModeContext}'s {@link ILookup#lookup} is queried.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23createInputModeContext(IInputMode%2CILookup) Online Documentation}*/
static createInputModeContext(parent: IInputMode, lookup?: ILookup | null): IInputModeContext;
/**
Tries to call {@link IGraph#invalidateDisplays} on the {@link IInputModeContext#graph graph} obtains from the {@link IInputModeContext}.
If the graph instance cannot be found, {@link CanvasComponent#invalidate} will be called instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23invalidateDisplays() Online Documentation}*/
invalidateDisplays(): void;
/**
Gets the {@link IInputMode} which issued the context object.
This can be <code>null</code> if this context is used outside the context of an IInputMode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23parentInputMode Online Documentation}*/
abstract readonly parentInputMode: IInputMode | null;
/**
Tries to retrieve an {@link IGraph} instance from an {@link IInputModeContext}.
This will use the {@link ILookup} and if this yields nothing, it will use the {@link ILookup} of the {@link ICanvasContext#canvasComponent} or
finally test if it is a {@link GraphComponent} to obtain the graph from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23graph Online Documentation}*/
readonly graph: IGraph | null;
/**
An empty context where both the input mode and the canvas is <code>null</code>.
The lookup always yields null, also.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputModeContext%23EMPTY Online Documentation}*/
static EMPTY: IInputModeContext;
static $class: Class;
private $_IInputModeContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IInputModeContext;
}
export interface IModelItemCollector {
}
/**
An interface that allows input modes to collect {@link IModelItem}s for various purposes.
This is mainly used for {@link HandleInputMode} and {@link MoveInputMode} to allow access to the items that
were affected during the respective operation.
@see HandleInputMode#affectedItems
@see MoveInputMode#affectedItems
@see {@link https://docs.yworks.com/yfileshtml/#/api/IModelItemCollector Online Documentation}*/
export abstract class IModelItemCollector {
/**
Adds a model item to this collector instance.
@param item The model item to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IModelItemCollector%23add(IModelItem) Online Documentation}*/
abstract add(item: IModelItem): void;
static $class: Class;
private $_IModelItemCollector: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IModelItemCollector;
}
export interface IPositionHandler extends IDragHandler {
}
/**
Interface for an object that can handle the position of an item displayed in a {@link CanvasComponent}
The point as indicated by {@link IDragHandler#location} describes the current world coordinate position of the element. It is up to the
implementation how this position is interpreted. The values returned by that instance will be used for the
"originalLocation" parameter in the {@link IDragHandler#handleMove}, {@link IDragHandler#cancelDrag} and {@link IDragHandler#dragFinished}
methods. Items can be dragged with the mouse and an instance of this class will modify their position accordingly.
Typically client code will use the {@link IDragHandler#location}'s coordinates to determine the current position of the elements. A drag will trigger
the invocation of {@link IDragHandler#initializeDrag}, zero or more {@link IDragHandler#handleMove} calls, and
finally by either {@link IDragHandler#dragFinished} or {@link IDragHandler#cancelDrag}.
@see IDragHandler
@see IHandle
@see MoveInputMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPositionHandler Online Documentation}*/
export abstract class IPositionHandler {
/**
Creates a generic composite implementation for the {@link IPositionHandler} interface.
@param handlers The handlers to create the composite from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPositionHandler%23combine(IPositionHandler) Online Documentation}*/
static combine(...handlers: IPositionHandler[]): IPositionHandler;
/**
Creates a generic composite implementation for the {@link IPositionHandler} interface.
@param handlers The handlers to create the composite from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPositionHandler%23combine(IEnumerable%3CIPositionHandler%3E) Online Documentation}*/
static combine(handlers: IEnumerable<IPositionHandler> | IEnumerableConvertible<IPositionHandler>): IPositionHandler;
/**
Creates an implementation of the interface {@link IPositionHandler} from the given definition.
The given object must contain exactly the abstract members of the {@link IPositionHandler} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IPositionHandler} interface.
@returns An instance of the {@link IPositionHandler} interface based on the given definition.*/
static create(interfaceDefinition: {
initializeDrag(context: IInputModeContext): void;
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
location: IPoint;
}): IPositionHandler;
static $class: Class;
private $_IPositionHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPositionHandler;
}
export interface IReshapeHandler {
}
/**
Interface for an object that can be interactively reshaped in a {@link CanvasComponent}
Items can be reshaped interactively and an instance of this class can be used to handle that process. A reshape
operation will be initialized by a call to {@link IReshapeHandler#initializeReshape}, then zero or more {@link IReshapeHandler#handleReshape}
calls follow, and the operation will be finalized by a call to either {@link IReshapeHandler#reshapeFinished} or
{@link IReshapeHandler#cancelReshape}. Note that implementations of this class will only be used during interactive
reshape operations.
@see IDragHandler
@see IPositionHandler
@see IHandle
@see IInputModeContext
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReshapeHandler Online Documentation}*/
export abstract class IReshapeHandler {
/**
Called by clients to indicate that the reshaping has been canceled by the user.
This method may be called after the initial {@link IReshapeHandler#initializeReshape} and zero or more invocations
of {@link IReshapeHandler#handleReshape}. Implementations should reset the bounds of the items they modify to their
initial state. Alternatively to this method the {@link IReshapeHandler#reshapeFinished} method might be called.
@param context The context to retrieve information about the reshaping from.
@param originalBounds The value of the coordinate of the {@link IReshapeHandler#bounds} property at the time of {@link IReshapeHandler#initializeReshape}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReshapeHandler%23cancelReshape(IInputModeContext%2CRect) Online Documentation}*/
abstract cancelReshape(context: IInputModeContext, originalBounds: Rect): void;
/**
Creates an implementation of the interface {@link IReshapeHandler} from the given definition.
The given object must contain exactly the abstract members of the {@link IReshapeHandler} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IReshapeHandler} interface.
@returns An instance of the {@link IReshapeHandler} interface based on the given definition.*/
static create(interfaceDefinition: {
initializeReshape(context: IInputModeContext): void;
handleReshape(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
cancelReshape(context: IInputModeContext, originalBounds: Rect): void;
reshapeFinished(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
bounds: IRectangle;
}): IReshapeHandler;
/**
Called by clients to indicate that the element has been dragged and its position should be updated.
This method may be called more than once after an initial {@link IReshapeHandler#initializeReshape} and will the
final call will be followed by either one {@link IReshapeHandler#reshapeFinished} or one {@link IReshapeHandler#cancelReshape}
call.
@param context The context to retrieve information about the reshaping from.
@param originalBounds The value of the {@link IReshapeHandler#bounds} property at the time of {@link IReshapeHandler#initializeReshape}.
@param newBounds The coordinates of the bounds in the world coordinate system that the client wants the shape to be at. Depending on the
implementation the {@link IReshapeHandler#bounds} may or may not be modified to reflect the new value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReshapeHandler%23handleReshape(IInputModeContext%2CRect%2CRect) Online Documentation}*/
abstract handleReshape(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
/**
Called by clients to indicate that the element is going to be reshaped.
This call will be followed by one or more calls to {@link IReshapeHandler#handleReshape}, and a final {@link IReshapeHandler#reshapeFinished}
or {@link IReshapeHandler#cancelReshape}.
@param context The context to retrieve information about the drag from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReshapeHandler%23initializeReshape(IInputModeContext) Online Documentation}*/
abstract initializeReshape(context: IInputModeContext): void;
/**
Called by clients to indicate that the reshaping has just been finished.
This method may be called after the initial {@link IReshapeHandler#initializeReshape} and zero or more invocations
of {@link IReshapeHandler#handleReshape}. Alternatively to this method the {@link IReshapeHandler#cancelReshape}
method might be called.
@param context The context to retrieve information about the drag from.
@param newBounds The coordinates of the bounds in the world coordinate system that the client wants the shape to be at. Depending on the
implementation the {@link IReshapeHandler#bounds} may or may not be modified to reflect the new value. This is the same value as delivered in the last
invocation of {@link IReshapeHandler#handleReshape}
@param originalBounds The value of the coordinate of the {@link IReshapeHandler#bounds} property at the time of {@link IReshapeHandler#initializeReshape}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReshapeHandler%23reshapeFinished(IInputModeContext%2CRect%2CRect) Online Documentation}*/
abstract reshapeFinished(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
/**
Gets a view of the bounds of the item.
The rectangle describes the current world coordinate of the element that can be modified by this handler.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReshapeHandler%23bounds Online Documentation}*/
abstract readonly bounds: IRectangle;
static $class: Class;
private $_IReshapeHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IReshapeHandler;
}
export interface InputModeEventArgs extends EventArgs {
}
/**
Event argument base class that is used by events that are triggered by {@link IInputMode} implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeEventArgs Online Documentation}*/
export class InputModeEventArgs {
/**
Initializes a new instance of the {@link InputModeEventArgs} class.
@param context The context to initialize the {@link InputModeEventArgs#context} property with.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeEventArgs Online Documentation}*/
constructor(context: IInputModeContext);
/**
Gets the context for the current event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputModeEventArgs%23context Online Documentation}*/
readonly context: IInputModeContext;
static $class: Class;
private $_InputModeEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is InputModeEventArgs;
}
export interface ItemTappedEventArgs<T> extends TapEventArgs {
}
/**
Provides data for the {@link GraphInputMode#addItemTappedListener ItemTapped} and {@link GraphInputMode#addItemDoubleTappedListener ItemDoubleTapped}
events.
Type parameter <code>T</code>: The type of the item this event carries.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemTappedEventArgs Online Documentation}*/
export class ItemTappedEventArgs<T> {
/**
Initializes a new instance of the {@link ItemTappedEventArgs} class.
@param item The item which is the subject of the event.
@param context The context in which the tap occurred.
@param location The location.
@param [tapCount=1] The number of taps associated with this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemTappedEventArgs Online Documentation}*/
constructor(item: T, context: IInputModeContext, location: Point | PointConvertible, tapCount?: number);
/**
Gets the item that is the subject of the event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemTappedEventArgs%23item Online Documentation}*/
item: T;
static $class: Class;
private $_ItemTappedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ItemTappedEventArgs<T>;
}
export interface KeyboardInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
cancel(): void;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} that recognizes simple key events and invokes a registered handler.
* This mode is {@link KeyboardInputMode#exclusive} by default.
*
* Typically the <code>KeyboardInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#keyboardInputMode} property.
*
* <br><b>Getting the KeyboardInputMode from its parent input mode</b>
* <pre>
* const keyboardInputMode = mode.keyboardInputMode
* </pre>
*
* <br><b>Adding a handler for a keyboard event</b>
* <pre>
* const keyboardInputMode = graphEditorInputMode.keyboardInputMode
* const myCommand = ICommand.createCommand('MyCommand')
*
* // The command creates a new node in the center of the viewport
* keyboardInputMode.addCommandBinding(
*   myCommand,
*   (command, parameter, target) =>
*     graphEditorInputMode.createNode(graphEditorInputMode.graphComponent.viewport.center),
*   (command, parameter, target) => true
* )
*
* // Execute the command if the Insert key is pressed
* keyboardInputMode.addKeyBinding(Key.INSERT, ModifierKeys.NONE, myCommand)
* </pre>
*
* <br><b>Adding a handler triggered by a keyboard event recognizer</b>
* <pre>
* const keyboardInputMode = graphEditorInputMode.keyboardInputMode
*
* // Execute the FIT_GRAPH_BOUNDS command if the Enter key is pressed
* keyboardInputMode.addRecognizerBinding(
*   (src, args) => args.originalEvent && args.originalEvent.key === 'Enter',
*   ICommand.FIT_GRAPH_BOUNDS
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode Online Documentation}
*/
export class KeyboardInputMode {
constructor();
/**
Adds a command and associated execution handlers to this instance.
The command will not be allowed to execute if this mode is {@link KeyboardInputMode#enabled disabled} or {@link ConcurrencyController#active inactive}.
@param command The command to register handlers with.
@param [execute=null] The handler for the execution.
@param [canExecute=null] The handler that determines executability or always enabled if omitted.
@returns A token for the newly created command binding that can used to later {@link KeyboardInputModeBinding#remove remove} this binding from this instance again.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23addCommandBinding(ICommand%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean) Online Documentation}*/
addCommandBinding(command: ICommand, execute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null, canExecute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null): KeyboardInputModeBinding;
/**
Adds a command and associated execution handlers to this instance.
The command will not be allowed to execute if this mode is {@link KeyboardInputMode#enabled disabled} or {@link ConcurrencyController#active inactive}.
@param options The parameters to pass.
@param options.command The command to register handlers with.
@param [options.execute=null] The handler for the execution.
@param [options.canExecute=null] The handler that determines executability or always enabled if omitted.
@returns A token for the newly created command binding that can used to later {@link KeyboardInputModeBinding#remove remove} this binding from this instance again.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23addCommandBinding(ICommand%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean) Online Documentation}*/
addCommandBinding(options: {
command: ICommand;
execute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null;
canExecute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null;
}): KeyboardInputModeBinding;
/**
Adds an event handler for a specific key press gesture to this mode.
Using this registration method, the command will not be allowed to execute if this mode is {@link KeyboardInputMode#enabled disabled} or {@link ConcurrencyController#active inactive}.
@param key The key constant that should be recognized.
@param modifiers The modifiers that should be recognized when the key is pressed.
@param command The command to execute.
@param [commandParameter=null] The command parameter to use for the {@link ICommand#execute execution}; <code>null</code> if omitted.
@returns A token for the newly created command binding that can used to later {@link KeyboardInputModeBinding#remove remove} this binding from this instance again.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23addKeyBinding(Key%2CModifierKeys%2CICommand%2CObject) Online Documentation}*/
addKeyBinding(key: Key | KeyStringValues, modifiers: ModifierKeys | ModifierKeysConvertible, command: ICommand, commandParameter?: any | null): KeyboardInputModeBinding;
/**
Adds an event handler for a specific key press gesture to this mode.
@param key The key constant that should be recognized.
@param [modifiers=ModifierKeys.NONE] The modifiers that should be recognized when the key is pressed. Defaults to {@link ModifierKeys#NONE} if omitted.
@param [execute=null] The handler for the execution.
@param [canExecute=null] The handler that determines executability or always enabled if omitted.
@returns A token for the newly created command binding that can used to later {@link KeyboardInputModeBinding#remove remove} this binding from this instance again.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23addKeyBinding(Key%2CModifierKeys%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean) Online Documentation}*/
addKeyBinding(key: Key | KeyStringValues, modifiers?: ModifierKeys | ModifierKeysConvertible, execute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null, canExecute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null): KeyboardInputModeBinding;
/**
Adds an event handler for a specific key press gesture to this mode.
@param options The parameters to pass.
@param options.key The key constant that should be recognized.
@param [options.modifiers=ModifierKeys.NONE] The modifiers that should be recognized when the key is pressed. Defaults to {@link ModifierKeys#NONE} if omitted.
@param [options.execute=null] The handler for the execution.
@param [options.canExecute=null] The handler that determines executability or always enabled if omitted.
@returns A token for the newly created command binding that can used to later {@link KeyboardInputModeBinding#remove remove} this binding from this instance again.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23addKeyBinding(Key%2CModifierKeys%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean) Online Documentation}*/
addKeyBinding(options: {
key: Key | KeyStringValues;
modifiers?: ModifierKeys | ModifierKeysConvertible;
execute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null;
canExecute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null;
}): KeyboardInputModeBinding;
/**
Adds a given handler to this instance that will be triggered if the event recognizer recognizes a {@link KeyEventArgs key event} that has
been triggered by the {@link CanvasComponent}.
@param recognizer An event recognizer that will be fed with all key events.
@param command The command to invoke if the recognizer matches a key event.
@param [commandParameter=null] The optional parameter to provide to the command.
@returns The newly constructed binding on which {@link KeyboardInputModeBinding#remove} can be called to remove the
resulting binding from this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23addRecognizerBinding(function(Object%2CEventArgs)%3Aboolean%2CICommand%2CObject) Online Documentation}*/
addRecognizerBinding(recognizer: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean), command: ICommand, commandParameter?: any | null): KeyboardInputModeBinding;
/**
Adds a given handler to this instance that will be triggered if the event recognizer recognizes a {@link KeyEventArgs key event} that has
been triggered by the {@link CanvasComponent}.
@param recognizer An event recognizer that will be fed with all key events.
@param [execute=null] The handler for the execution.
@param [canExecute=null] The handler that determines executability or always enabled if omitted.
@returns The newly constructed binding on which {@link KeyboardInputModeBinding#remove} can be called to remove the
resulting binding from this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23addRecognizerBinding(function(Object%2CEventArgs)%3Aboolean%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean) Online Documentation}*/
addRecognizerBinding(recognizer: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean), execute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null, canExecute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null): KeyboardInputModeBinding;
/**
Adds a given handler to this instance that will be triggered if the event recognizer recognizes a {@link KeyEventArgs key event} that has
been triggered by the {@link CanvasComponent}.
@param options The parameters to pass.
@param options.recognizer An event recognizer that will be fed with all key events.
@param [options.execute=null] The handler for the execution.
@param [options.canExecute=null] The handler that determines executability or always enabled if omitted.
@returns The newly constructed binding on which {@link KeyboardInputModeBinding#remove} can be called to remove the
resulting binding from this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23addRecognizerBinding(function(Object%2CEventArgs)%3Aboolean%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean%2Cfunction(ICommand%2CObject%2CObject)%3Aboolean) Online Documentation}*/
addRecognizerBinding(options: {
recognizer: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
execute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null;
canExecute?: ((command: ICommand, parameter: any | null, source: any) => boolean) | null;
}): KeyboardInputModeBinding;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link KeyboardInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called after {@link KeyboardInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Enables the registered commands and can be overridden in subclasses to perform additional actions after the mode has
been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Disables the registered commands and can be overridden in subclasses to perform additional actions after the mode has
been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Called after {@link KeyboardInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Removes all previously registered commands from this instance.
This effectively disables the command for the instance managed by this mode.
@param command The command to remove from all registered bindings.
@see KeyboardInputMode#addCommandBinding
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23removeCommand(ICommand) Online Documentation}*/
removeCommand(command: ICommand): void;
/**
Overridden to only return <code>true</code> if this instance does not currently {@link ConcurrencyController#hasMutex have the input mutex}.
@returns <code>true</code> iff this instance does not {@link ConcurrencyController#hasMutex own the mutex}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23tryStop() Online Documentation}*/
tryStop(): boolean;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link KeyboardInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link KeyboardInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the KeyboardInputMode on its parent input mode</b>
* <pre>
* mode.keyboardInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link KeyboardInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
static $class: Class;
private $_KeyboardInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is KeyboardInputMode;
}
export interface KeyboardInputModeBinding extends YObject {
}
/**
A token returned by {@link KeyboardInputMode}'s factory methods that can be used to {@link KeyboardInputModeBinding#remove}
the binding again.
Instances of this class can only be obtained from {@link KeyboardInputMode#addCommandBinding}, AddKeyBinding, and
AddRecognizerBinding. Removing the instances is done via the instance method {@link KeyboardInputModeBinding#remove}
of this type.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputModeBinding Online Documentation}*/
export class KeyboardInputModeBinding {
/**
Removes the binding represented by this instance from the {@link KeyboardInputMode} it has been created for.
Once this method has been called, the instance can be
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputModeBinding%23remove() Online Documentation}*/
remove(): void;
/**
Gets the {@link ICommand} that is associated with this binding.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyboardInputModeBinding%23command Online Documentation}*/
readonly command: ICommand;
static $class: Class;
private $_KeyboardInputModeBinding: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is KeyboardInputModeBinding;
}
export interface LassoSelectionInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} implementation that can handle lasso selections performed by the user with a mouse or via
* touch.
* This implementation will request the mutex as soon as it recognizes a mouse or touch drag.
*
* By default the lasso is a single freehand gesture. However, both a polyline path and mixed freehand/polyline paths are
* possible with different configuration.
*
* This mode is {@link LassoSelectionInputMode#exclusive} by default.
*
* The {@link LassoSelectionInputMode} can easily be configured for different gestures, e.g. for a polyline path
* instead of the default freehand configuration:
*
* <pre>
* const lassoSelectionInputMode = graphEditorInputMode.lassoSelectionInputMode
* lassoSelectionInputMode.dragFreeRecognizer = MouseEventRecognizers.DRAG
* lassoSelectionInputMode.startSegmentRecognizer = EventRecognizers.ALWAYS
* lassoSelectionInputMode.endSegmentRecognizer = EventRecognizers.createOrRecognizer(
*   MouseEventRecognizers.LEFT_IS_DOWN,
*   MouseEventRecognizers.LEFT_UP
* )
* lassoSelectionInputMode.dragSegmentRecognizer = MouseEventRecognizers.MOVE_OR_DRAG
* lassoSelectionInputMode.finishRecognizer = MouseEventRecognizers.LEFT_DOUBLE_CLICK
* lassoSelectionInputMode.finishRadius = 10
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode Online Documentation}
*/
export class LassoSelectionInputMode {
/**
Creates a new instance of the lasso selection mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode Online Documentation}*/
constructor();
/**
Cancels any pending lasso selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Helper method that yields a suitably configured {@link InputModeEventArgs} for this input mode.
@returns An input mode event argument that is configured for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23createInputModeEventArgs() Online Documentation}*/
createInputModeEventArgs(): LassoSelectionEventArgs;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link LassoSelectionInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Used by the {@link LassoSelectionInputMode#finishRecognizer} and {@link LassoSelectionInputMode#finishRecognizerTouch} to check the current cursor position.
This method checks if the distance from <code>location</code> to the {@link LassoSelectionInputMode#startLocation} is less then {@link LassoSelectionInputMode#finishRadius} in view coordinates.
@param location The location in world coordinates that should be checked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23isValidEndLocation(Point) Online Documentation}*/
isValidEndLocation(location: Point): boolean;
/**
Called after {@link LassoSelectionInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Triggers the {@link LassoSelectionInputMode#addDragCanceledListener DragCanceled} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onDragCanceled(LassoSelectionEventArgs) Online Documentation}*/
onDragCanceled(evt: LassoSelectionEventArgs): void;
/**
Triggers the {@link LassoSelectionInputMode#addDragCancelingListener DragCanceling} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onDragCanceling(LassoSelectionEventArgs) Online Documentation}*/
onDragCanceling(evt: LassoSelectionEventArgs): void;
/**
Called once the drag has been finalized.
This method triggers the {@link LassoSelectionInputMode#addDragFinishedListener DragFinished}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onDragFinished(LassoSelectionEventArgs) Online Documentation}*/
onDragFinished(evt: LassoSelectionEventArgs): void;
/**
Called before the drag will be finalized.
This method triggers the {@link LassoSelectionInputMode#addDragFinishingListener DragFinishing}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onDragFinishing(LassoSelectionEventArgs) Online Documentation}*/
onDragFinishing(evt: LassoSelectionEventArgs): void;
/**
Called at the end of each drag.
This method triggers the {@link LassoSelectionInputMode#addDraggedListener Dragged}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onDragged(LassoSelectionEventArgs) Online Documentation}*/
onDragged(evt: LassoSelectionEventArgs): void;
/**
Called at the start of each drag.
This method triggers the {@link LassoSelectionInputMode#addDraggingListener Dragging}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onDragging(LassoSelectionEventArgs) Online Documentation}*/
onDragging(evt: LassoSelectionEventArgs): void;
/**
Triggers the {@link LassoSelectionInputMode#addDragStartedListener DragStarted} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onDragStarted(LassoSelectionEventArgs) Online Documentation}*/
onDragStarted(evt: LassoSelectionEventArgs): void;
/**
Triggers the {@link LassoSelectionInputMode#addDragStartingListener DragStarting} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onDragStarting(LassoSelectionEventArgs) Online Documentation}*/
onDragStarting(evt: LassoSelectionEventArgs): void;
/**
Called after {@link LassoSelectionInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
A {@link string} that can be used to store a {@link LassoSelectionInputMode#stroke} for the visualization of the lasso path.
Alternatively, the default path template can be styled with the <code>yfiles-lasso-template</code> or <code>yfiles-lasso-path</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23LASSO_STROKE_KEY Online Documentation}*/
static LASSO_STROKE_KEY: string;
/**
A {@link string} that can be used to store a {@link Fill} for the visualization of the fill of the lasso path.
Alternatively, the default path template can be styled with the <code>yfiles-lasso-template</code> or <code>yfiles-lasso-path</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23LASSO_FILL_KEY Online Documentation}*/
static LASSO_FILL_KEY: string;
/**
A {@link string} that can be used to store a {@link IVisualTemplate} that can be used to create the visual to indicate the finish
region around the {@link LassoSelectionInputMode#startLocation}.
The region indicates where the lasso selection can be finished.
The default template has the <code>yfiles-lasso-template</code> and <code>yfiles-lasso-finish-region</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23FINISH_REGION_TEMPLATE_KEY Online Documentation}*/
static FINISH_REGION_TEMPLATE_KEY: string;
/**
A {@link string} that can be used to store a {@link IVisualTemplate} that can be used to create the visual to indicate the
highlighted finish region around the {@link LassoSelectionInputMode#startLocation}.
The highlighted region indicates that the lasso selection can be finished at the current location.
The default template has the <code>yfiles-lasso-template</code> and <code>yfiles-lasso-finish-region-highlight</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23FINISH_REGION_HIGHLIGHT_TEMPLATE_KEY Online Documentation}*/
static FINISH_REGION_HIGHLIGHT_TEMPLATE_KEY: string;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link LassoSelectionInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
Gets or sets the enabled state of this input mode.
Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
{@link LassoSelectionInputMode#controller} so a disabled instance should never try to acquire the input mutex.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23enabled Online Documentation}*/
enabled: boolean;
/**
Gets the installed {@link LassoSelectionInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Gets the current or last selection path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23selectionPath Online Documentation}*/
readonly selectionPath: GeneralPath;
/**
Gets or sets the stroke that is used for the visualization of the lasso path.
Alternatively, the default path template can be styled with the <code>yfiles-lasso-template</code> or <code>yfiles-lasso-path</code> CSS classes.
@see LassoSelectionInputMode#LASSO_STROKE_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23stroke Online Documentation}*/
stroke: Stroke | null;
/**
Gets or sets the fill that is used for the visualization of the lasso path.
Alternatively, the default path template can be styled with the <code>yfiles-lasso-template</code> or <code>yfiles-lasso-path</code> CSS classes.
@see LassoSelectionInputMode#LASSO_FILL_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23fill Online Documentation}*/
fill: Fill | null;
/**
Gets or sets the template that is used for the visualization of the region around the {@link LassoSelectionInputMode#startLocation} where the lasso may be finished.
Alternatively, the default template can be styled with the <code>yfiles-lasso-template</code> or <code>yfiles-lasso-finish-region</code> CSS
classes.
@see LassoSelectionInputMode#FINISH_REGION_TEMPLATE_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23finishRegionTemplate Online Documentation}*/
finishRegionTemplate: IVisualTemplate | null;
/**
Gets or sets the template that is used for the visualization of the highlighted region around the {@link LassoSelectionInputMode#startLocation} where the lasso may
be finished.
Alternatively, the default template can be styled with the <code>yfiles-lasso-template</code> or <code>yfiles-lasso-finish-region-highlight</code>
CSS classes.
@see LassoSelectionInputMode#FINISH_REGION_HIGHLIGHT_TEMPLATE_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23finishRegionHighlightTemplate Online Documentation}*/
finishRegionHighlightTemplate: IVisualTemplate | null;
/**
Gets the location where the {@link LassoSelectionInputMode#prepareRecognizer} was triggered and the lasso selection started.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23startLocation Online Documentation}*/
startLocation: Point;
/**
Gets or sets the radius in view coordinates around the {@link LassoSelectionInputMode#startLocation} where the lasso selection can be finished.
A value of <code>0</code> (the default) disables ending the gesture near the {@link LassoSelectionInputMode#startLocation} and also disable showing the indicator.
Default is <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23finishRadius Online Documentation}*/
finishRadius: number;
/**
Gets or sets the cursor to use during the dragging of the lasso.
The default is Pen
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23lassoCursor Online Documentation}*/
lassoCursor: Cursor | null;
/**
Gets or sets the cursor to use when hovering over a valid end location.
The default is <code>Cursors.Pen</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23validEndCursor Online Documentation}*/
validEndCursor: Cursor | null;
/**
Gets or sets the event recognizer that determines whether to cancel lasso selection.
The default recognizer reacts on {@link KeyEventRecognizers#ESCAPE_DOWN} or
{@link MouseEventRecognizers#LOST_CAPTURE_DURING_DRAG}, or {@link MouseEventRecognizers#RIGHT_UP}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23cancelRecognizer Online Documentation}*/
cancelRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to start drawing the lasso.
The default value is {@link MouseEventRecognizers#LEFT_DOWN}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23prepareRecognizer Online Documentation}*/
prepareRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to drag the lasso free hand.
The default value is {@link MouseEventRecognizers#LEFT_DRAG}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23dragFreeRecognizer Online Documentation}*/
dragFreeRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to drag a straight-line segment for the lasso.
The default value is {@link MouseEventRecognizers#MOVE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23dragSegmentRecognizer Online Documentation}*/
dragSegmentRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to switch from freehand to segment mode.
The default value is
{@link EventRecognizers#NEVER}, which locks this mode in freehand-only mode. However, this can be set differently
to allow for a mixed freehand/segment mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23startSegmentRecognizer Online Documentation}*/
startSegmentRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to end a segment and switch from segment to freehand mode.
The default value is {@link MouseEventRecognizers#LEFT_DOWN}. However, since {@link LassoSelectionInputMode#startSegmentRecognizer} is disabled, this has no effect in
the default configuration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23endSegmentRecognizer Online Documentation}*/
endSegmentRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish the gesture away from the {@link LassoSelectionInputMode#startLocation}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23finishRecognizer Online Documentation}*/
finishRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish the gesture when near the {@link LassoSelectionInputMode#startLocation}.
This recognizer is always combined with a check whether the input coordinates are within {@link LassoSelectionInputMode#finishRadius} around {@link LassoSelectionInputMode#startLocation}.
@see LassoSelectionInputMode#finishRadius
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23finishRegionRecognizer Online Documentation}*/
finishRegionRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to start drawing the lasso via touch.
The default value is {@link TouchEventRecognizers#TOUCH_LONG_PRESS_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23prepareRecognizerTouch Online Documentation}*/
prepareRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to drag the lasso free hand via touch.
The default value is {@link TouchEventRecognizers#TOUCH_MOVE_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23dragFreeRecognizerTouch Online Documentation}*/
dragFreeRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to drag a straight-line segment for the lasso via touch.
The default value is {@link EventRecognizers#NEVER} which means that the straight-line segment will only be
finished when switching back to free hand creation or when finishing the gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23dragSegmentRecognizerTouch Online Documentation}*/
dragSegmentRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets a recognizer determining when to switch from freehand to segment mode for touch input.
The default value is
{@link EventRecognizers#NEVER}, which locks this mode in freehand-only mode. However, this can be set differently
to allow for a mixed freehand/segment mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23startSegmentRecognizerTouch Online Documentation}*/
startSegmentRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to end a segment and switch from segment to freehand mode via
touch.
The default value is {@link TouchEventRecognizers#TOUCH_DOWN_PRIMARY}. However, since {@link LassoSelectionInputMode#startSegmentRecognizerTouch} is disabled, this has no
effect in the default configuration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23endSegmentRecognizerTouch Online Documentation}*/
endSegmentRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish the gesture away from the {@link LassoSelectionInputMode#startLocation} via touch.
The default value is {@link TouchEventRecognizers#TOUCH_UP_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23finishRecognizerTouch Online Documentation}*/
finishRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish the gesture when near the {@link LassoSelectionInputMode#startLocation} via touch.
This recognizer is always combined with a check whether the input coordinates are within {@link LassoSelectionInputMode#finishRadius} around {@link LassoSelectionInputMode#startLocation}.
The default value is {@link TouchEventRecognizers#TOUCH_UP_PRIMARY}.
Due to the default value for {@link LassoSelectionInputMode#finishRecognizerTouch} and {@link LassoSelectionInputMode#finishRadius} this is not normally used, except the mentioned properties are changed.
@see LassoSelectionInputMode#finishRadius
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23finishRegionRecognizerTouch Online Documentation}*/
finishRegionRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to add.
@see LassoSelectionInputMode#removeDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23Dragging Online Documentation}*/
addDraggingListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to remove.
@see LassoSelectionInputMode#addDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23Dragging Online Documentation}*/
removeDraggingListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to add.
@see LassoSelectionInputMode#removeDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23Dragged Online Documentation}*/
addDraggedListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to remove.
@see LassoSelectionInputMode#addDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23Dragged Online Documentation}*/
removeDraggedListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to add.
@see LassoSelectionInputMode#removeDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragCanceled Online Documentation}*/
addDragCanceledListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to remove.
@see LassoSelectionInputMode#addDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragCanceled Online Documentation}*/
removeDragCanceledListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to add.
@see LassoSelectionInputMode#removeDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragCanceling Online Documentation}*/
addDragCancelingListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to remove.
@see LassoSelectionInputMode#addDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragCanceling Online Documentation}*/
removeDragCancelingListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to add.
@see LassoSelectionInputMode#removeDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragFinishing Online Documentation}*/
addDragFinishingListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to remove.
@see LassoSelectionInputMode#addDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragFinishing Online Documentation}*/
removeDragFinishingListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragFinished</code> event that occurs when the drag has been finished.
@param listener The listener to add.
@see LassoSelectionInputMode#removeDragFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragFinished Online Documentation}*/
addDragFinishedListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinished</code> event that occurs when the drag has been finished.
@param listener The listener to remove.
@see LassoSelectionInputMode#addDragFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragFinished Online Documentation}*/
removeDragFinishedListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to add.
@see LassoSelectionInputMode#removeDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragStarting Online Documentation}*/
addDragStartingListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to remove.
@see LassoSelectionInputMode#addDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragStarting Online Documentation}*/
removeDragStartingListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to add.
@see LassoSelectionInputMode#removeDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragStarted Online Documentation}*/
addDragStartedListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to remove.
@see LassoSelectionInputMode#addDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionInputMode%23DragStarted Online Documentation}*/
removeDragStartedListener(listener: (sender: any, evt: LassoSelectionEventArgs) => void): void;
static $class: Class;
private $_LassoSelectionInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LassoSelectionInputMode;
}
export interface LassoSelectionEventArgs extends InputModeEventArgs {
}
/**
Event arguments used by {@link LassoSelectionInputMode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionEventArgs Online Documentation}*/
export class LassoSelectionEventArgs {
/**
Gets the current {@link LassoSelectionInputMode#selectionPath lasso selection path} of the {@link LassoSelectionInputMode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoSelectionEventArgs%23selectionPath Online Documentation}*/
readonly selectionPath: GeneralPath;
static $class: Class;
private $_LassoSelectionEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LassoSelectionEventArgs;
}
export interface MarqueeSelectionInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} implementation that can handle marquee selections performed by the user with a mouse.
* This implementation will request the mutex as soon as it recognizes a mouse drag.
*
* This mode is {@link MarqueeSelectionInputMode#exclusive} by default.
*
* Typically the <code>MarqueeSelectionInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#marqueeSelectionInputMode} property.
*
* <br><b>Getting the MarqueeSelectionInputMode from its parent input mode</b>
* <pre>
* const marqueeSelectionInputMode = mode.marqueeSelectionInputMode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode Online Documentation}
*/
export class MarqueeSelectionInputMode {
/**
Creates a new instance of the marquee selection mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode Online Documentation}*/
constructor();
/**
Calculates the location and size of the marquee rectangle.
The default implementation returns a rectangle that is exactly large enough to contain the two specified points.
<h2>Note</h2>
When {@link MarqueeSelectionInputMode#useViewCoordinates} is <code>true</code> and a {@link CanvasComponent#projection} is set, <code>startDragLocation</code> and <code>currentDragLocation</code> are given in <b>view</b> coordinates instead of world
coordinates.
@param startDragLocation the location in world coordinates where the drag has been started
@param currentDragLocation the current location of the drag in world coordinates
@returns the marquee rectangle to be determined from the given locations
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23calculateMarqueeRectangle(Point%2CPoint) Online Documentation}*/
calculateMarqueeRectangle(startDragLocation: Point, currentDragLocation: Point): Rect;
/**
Cancels any pending marquee selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Helper method that yields a suitably configured {@link InputModeEventArgs} for this input mode.
@returns An input mode event argument that is configured for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23createInputModeEventArgs() Online Documentation}*/
createInputModeEventArgs(): MarqueeSelectionEventArgs;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link MarqueeSelectionInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called after {@link MarqueeSelectionInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Triggers the {@link MarqueeSelectionInputMode#addDragCanceledListener DragCanceled}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onDragCanceled(MarqueeSelectionEventArgs) Online Documentation}*/
onDragCanceled(evt: MarqueeSelectionEventArgs): void;
/**
Triggers the {@link MarqueeSelectionInputMode#addDragCancelingListener DragCanceling}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onDragCanceling(MarqueeSelectionEventArgs) Online Documentation}*/
onDragCanceling(evt: MarqueeSelectionEventArgs): void;
/**
Called once the drag has been finalized.
This method triggers the {@link MarqueeSelectionInputMode#addDragFinishedListener DragFinished}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onDragFinished(MarqueeSelectionEventArgs) Online Documentation}*/
onDragFinished(evt: MarqueeSelectionEventArgs): void;
/**
Called before the drag will be finalized.
This method triggers the {@link MarqueeSelectionInputMode#addDragFinishingListener DragFinishing}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onDragFinishing(MarqueeSelectionEventArgs) Online Documentation}*/
onDragFinishing(evt: MarqueeSelectionEventArgs): void;
/**
Called at the end of each drag.
This method triggers the {@link MarqueeSelectionInputMode#addDraggedListener Dragged}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onDragged(MarqueeSelectionEventArgs) Online Documentation}*/
onDragged(evt: MarqueeSelectionEventArgs): void;
/**
Called at the start of each drag.
This method triggers the {@link MarqueeSelectionInputMode#addDraggingListener Dragging}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onDragging(MarqueeSelectionEventArgs) Online Documentation}*/
onDragging(evt: MarqueeSelectionEventArgs): void;
/**
Triggers the {@link MarqueeSelectionInputMode#addDragStartedListener DragStarted}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onDragStarted(MarqueeSelectionEventArgs) Online Documentation}*/
onDragStarted(evt: MarqueeSelectionEventArgs): void;
/**
Triggers the {@link MarqueeSelectionInputMode#addDragStartingListener DragStarting}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onDragStarting(MarqueeSelectionEventArgs) Online Documentation}*/
onDragStarting(evt: MarqueeSelectionEventArgs): void;
/**
Called after {@link MarqueeSelectionInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link MarqueeSelectionInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link MarqueeSelectionInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the MarqueeSelectionInputMode on its parent input mode</b>
* <pre>
* mode.marqueeSelectionInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link MarqueeSelectionInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Gets the current or last marquee selection rectangle.
@see MarqueeSelectionInputMode#selectionPath
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23selectionRectangle Online Documentation}*/
readonly selectionRectangle: Rect;
/**
Gets the current or last marquee selection in world coordinates as a path.
This path can be used if a {@link CanvasComponent#projection} is set and the marquee is no longer a rectangle in world coordinates. If no {@link CanvasComponent#projection} is set, the
resulting shape is identical to the {@link MarqueeSelectionInputMode#selectionRectangle}
@see MarqueeSelectionInputMode#selectionRectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23selectionPath Online Documentation}*/
readonly selectionPath: GeneralPath;
/**
Gets or sets a value indicating whether the marquee rectangle is drawn in view coordinates (which include
{@link CanvasComponent#projection}), or intermediate coordinates.
View coordinates ensure that the user always sees a rectangle for the marquee. However, the resulting shape of the
marquee in world coordinates may not be a rectangle.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23useViewCoordinates Online Documentation}*/
useViewCoordinates: boolean;
/**
Gets or sets the template that is used for the visualization of the marquee rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23template Online Documentation}*/
template: IVisualTemplate | null;
/**
Gets or sets the event recognizer that determines whether to cancel marquee selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23cancelRecognizer Online Documentation}*/
cancelRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
A {@link string} that can be used to store a {@link IVisualTemplate} that can be used to create the visual that will be used to
draw the marquee box.
The default template has the <code>yfiles-marquee-rect-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23MARQUEE_RECTANGLE_TEMPLATE_KEY Online Documentation}*/
static MARQUEE_RECTANGLE_TEMPLATE_KEY: string;
/**
Gets or sets the cursor to use during the dragging of the marquee.
The default is <code>Cursors.Cross</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23marqueeCursor Online Documentation}*/
marqueeCursor: Cursor | null;
/**
Gets or sets the event recognizer that determines whether to start creating the marquee.
By default this recognizer detects left mouse button presses. If this mode is a child mode of the {@link GraphViewerInputMode}
and created by method {@link GraphViewerInputMode#createMarqueeSelectionInputMode} the recognizer is changed to
left mouse button press with Shift held down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23pressedRecognizer Online Documentation}*/
pressedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to drag the marquee.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23draggedRecognizer Online Documentation}*/
draggedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish creating the marquee.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23releasedRecognizer Online Documentation}*/
releasedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to start creating the marquee via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23pressedRecognizerTouch Online Documentation}*/
pressedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to drag the marquee via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23draggedRecognizerTouch Online Documentation}*/
draggedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish creating the marquee via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23releasedRecognizerTouch Online Documentation}*/
releasedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to add.
@see MarqueeSelectionInputMode#removeDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23Dragging Online Documentation}*/
addDraggingListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to remove.
@see MarqueeSelectionInputMode#addDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23Dragging Online Documentation}*/
removeDraggingListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to add.
@see MarqueeSelectionInputMode#removeDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23Dragged Online Documentation}*/
addDraggedListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to remove.
@see MarqueeSelectionInputMode#addDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23Dragged Online Documentation}*/
removeDraggedListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to add.
@see MarqueeSelectionInputMode#removeDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragCanceled Online Documentation}*/
addDragCanceledListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to remove.
@see MarqueeSelectionInputMode#addDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragCanceled Online Documentation}*/
removeDragCanceledListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to add.
@see MarqueeSelectionInputMode#removeDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragCanceling Online Documentation}*/
addDragCancelingListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to remove.
@see MarqueeSelectionInputMode#addDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragCanceling Online Documentation}*/
removeDragCancelingListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to add.
@see MarqueeSelectionInputMode#removeDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragFinishing Online Documentation}*/
addDragFinishingListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to remove.
@see MarqueeSelectionInputMode#addDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragFinishing Online Documentation}*/
removeDragFinishingListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragFinished</code> event that occurs the drag has been finished.
@param listener The listener to add.
@see MarqueeSelectionInputMode#removeDragFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragFinished Online Documentation}*/
addDragFinishedListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinished</code> event that occurs the drag has been finished.
@param listener The listener to remove.
@see MarqueeSelectionInputMode#addDragFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragFinished Online Documentation}*/
removeDragFinishedListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to add.
@see MarqueeSelectionInputMode#removeDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragStarting Online Documentation}*/
addDragStartingListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to remove.
@see MarqueeSelectionInputMode#addDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragStarting Online Documentation}*/
removeDragStartingListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to add.
@see MarqueeSelectionInputMode#removeDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragStarted Online Documentation}*/
addDragStartedListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to remove.
@see MarqueeSelectionInputMode#addDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionInputMode%23DragStarted Online Documentation}*/
removeDragStartedListener(listener: (sender: any, evt: MarqueeSelectionEventArgs) => void): void;
static $class: Class;
private $_MarqueeSelectionInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MarqueeSelectionInputMode;
}
export interface MarqueeSelectionEventArgs extends InputModeEventArgs {
}
/**
Event arguments used by {@link MarqueeSelectionInputMode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionEventArgs Online Documentation}*/
export class MarqueeSelectionEventArgs {
/**
Gets the current {@link MarqueeSelectionInputMode#selectionRectangle selection rectangle} of the {@link MarqueeSelectionInputMode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionEventArgs%23rectangle Online Documentation}*/
readonly rectangle: Rect;
/**
Gets the current {@link MarqueeSelectionInputMode#selectionPath selection as a path} of the {@link MarqueeSelectionInputMode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionEventArgs%23path Online Documentation}*/
readonly path: GeneralPath;
/**
Gets a value determining whether {@link MarqueeSelectionEventArgs#path} takes precedence over {@link MarqueeSelectionEventArgs#rectangle}.
This is the case when {@link MarqueeSelectionInputMode#useViewCoordinates} is <code>true</code> and a {@link CanvasComponent#projection} is set. In that case, the {@link MarqueeSelectionEventArgs#rectangle} does not represent a useful value, since a rectangle in
view coordinates is represented by a different shape in world coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarqueeSelectionEventArgs%23usePath Online Documentation}*/
readonly usePath: boolean;
static $class: Class;
private $_MarqueeSelectionEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MarqueeSelectionEventArgs;
}
export interface ToolTipQueryEventArgs extends InputModeEventArgs {
}
/**
Event argument that can be used to query the text to display when the mouse hovers over the given point in world
coordinate space.
@see MouseHoverInputMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTipQueryEventArgs Online Documentation}*/
export class ToolTipQueryEventArgs {
/**
Initializes a new instance of the {@link ToolTipQueryEventArgs} class.
@param context The context of the query.
@param queryLocation The query location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTipQueryEventArgs Online Documentation}*/
constructor(context: IInputModeContext, queryLocation: Point | PointConvertible);
/**
Gets the query location in world coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTipQueryEventArgs%23queryLocation Online Documentation}*/
readonly queryLocation: Point;
/**
Gets or sets the tooltip content to use.
The content can be either a string or any HTML element.
Setting this property will set the {@link ToolTipQueryEventArgs#handled} property to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTipQueryEventArgs%23toolTip Online Documentation}*/
toolTip: any | null;
/**
* Gets or sets a value indicating whether this {@link ToolTipQueryEventArgs} has been handled.
* If this property is set to <code>true</code>, the content of the {@link ToolTipQueryEventArgs#toolTip} property will be displayed as tooltip. Otherwise, the input mode
* will continue determining the tooltip contents, typically by querying for the content with another graph item at the
* current {@link ToolTipQueryEventArgs#queryLocation}.
*
* Handlers which handle this {@link ToolTipQueryEventArgs} should always test first, whether this property is set to
* <code>true</code> and, if so, exit without doing anything.
*
* Note that this property needs to be set before the function synchronously returns. This means that setting the value in
* an <code>async</code> function after the first <code>await</code> or in an asynchronous callback, it cannot not be seen by the emitter of the
* event and the sender will consider the event as not being handled.
*
* <pre>
* // wrong:
* mode.addQueryToolTipListener(async (sender, args) => {
*   await something()
*   args.handled = true  // will be set *after* all other event listeners have been invoked, only
* })
*
* // correct:
* mode.addQueryToolTipListener(async (sender, args) => {
*   args.handled = true // will be set immediately and 'mode' will see the change
*   await something()
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTipQueryEventArgs%23handled Online Documentation}
*/
handled: boolean;
static $class: Class;
private $_ToolTipQueryEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ToolTipQueryEventArgs;
}
export interface MouseHoverInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
cancel(): void;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} that detects when the mouse hovers over the {@link CanvasComponent}.
* This implementation will show a {@link MouseHoverInputMode#toolTip ToolTip}. In order to make use of this instance one has to register with the {@link MouseHoverInputMode#addQueryToolTipListener QueryToolTip}
* event. Setting a custom {@link MouseHoverInputMode#validHoverLocationHitTestable} will restrict the area where a tooltip can be shown.
*
* This mode is {@link MouseHoverInputMode#exclusive} by default.
*
* Typically the <code>MouseHoverInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#mouseHoverInputMode} property.
*
* <br><b>Getting the MouseHoverInputMode from its parent input mode</b>
* <pre>
* const mouseHoverInputMode = mode.mouseHoverInputMode
* </pre>
*
* It is recommended to configure the tooltip handling on the parent {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* instead of the <code>MouseHoverInputMode</code>. This involves registering for the {@link GraphInputMode#addQueryItemToolTipListener QueryItemToolTip}
* event instead of the {@link MouseHoverInputMode#addQueryToolTipListener QueryToolTip} event:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.toolTipItems = GraphItemTypes.NODE
* // register a listener
* mode.addQueryItemToolTipListener((src, args) => {
*   if (args.handled) {
*     // A tooltip has already been assigned -> nothing to do.
*     return
*   }
*   // We can safely cast here because we set ToolTipItems to only Node.
*   /** @type {INode} *‍/
*   const hitNode = args.item
*   if (hitNode.labels.size > 0) {
*     // Show the text of the first label as tooltip.
*     args.toolTip = hitNode.labels.get(0).text
*
*     // Indicate that the tooltip content has been set.
*     args.handled = true
*   }
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode Online Documentation}
*/
export class MouseHoverInputMode {
/**
Creates a new instance using the provided optional text provider.
@param [textProvider=null]
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode Online Documentation}*/
constructor(textProvider?: ((sender: any | null, evt: ToolTipQueryEventArgs | null) => void) | null);
/**
Adjust the calculated tooltip position.
This default implementation of this method moves the tooltip into the visible area if it extends beyond the document
bounds.
@param originalPosition The calculated tooltip position relative to the document root
@returns The adjusted tooltip location
@see MouseHoverInputMode#getToolTipLocation
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23adjustTooltipPosition(Point) Online Documentation}*/
adjustTooltipPosition(originalPosition: Point): Point;
/**
Creates an {@link IInputModeContext} for use with the {@link MouseHoverInputMode#addQueryToolTipListener QueryToolTip}
event for the upcoming text query operation.
@returns An instance of {@link IInputModeContext}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23createHoverInputModeContext() Online Documentation}*/
createHoverInputModeContext(): IInputModeContext;
/**
Factory method that creates the {@link MouseHoverInputMode#toolTip}.
@returns A simple {@link MouseHoverInputMode#toolTip}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23createToolTip() Online Documentation}*/
createToolTip(): ToolTip;
/**
Callback method that provides a text for the given location or <code>null</code>.
@param location The location to find a text for.
@returns A string or <code>null</code> to indicate that no tooltip should be shown.
@see MouseHoverInputMode#addQueryToolTipListener
@see MouseHoverInputMode#createHoverInputModeContext
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23getToolTipContent(Point) Online Documentation}*/
getToolTipContent(location: Point): any | null;
/**
Finds the position in canvas view coordinates to display the tooltip at for the given world coordinates.
This method adds the {@link MouseHoverInputMode#toolTipLocationOffset} after converting the given location to view coordinates.
@param location The position in world coordinates.
@returns The position in view coordinates.
@see MouseHoverInputMode#toolTipLocationOffset
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23getToolTipLocation(Point) Online Documentation}*/
getToolTipLocation(location: Point): Point;
/**
Immediately hides the tooltip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23hide() Online Documentation}*/
hide(): void;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link MouseHoverInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called after {@link MouseHoverInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Called when the tooltip should be hidden.
@param tooltip The tooltip that will be hidden.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23onHide(ToolTip) Online Documentation}*/
onHide(tooltip: ToolTip): void;
/**
Raises the {@link MouseHoverInputMode#addQueryToolTipListener QueryToolTip} event.
@param evt The {@link ToolTipQueryEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23onQueryToolTip(ToolTipQueryEventArgs) Online Documentation}*/
onQueryToolTip(evt: ToolTipQueryEventArgs): void;
/**
Triggered when the mouse hovers over the given coordinates.
If <code>content</code> is <code>null</code>, this method will call {@link MouseHoverInputMode#getToolTipContent} to query the content. If
there is any content it will display the {@link MouseHoverInputMode#toolTip} at the location returned by
{@link MouseHoverInputMode#getToolTipLocation}.
@param location
@param [content=null]
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23onShow(Point%2CObject) Online Documentation}*/
onShow(location: Point, content?: any | null): void;
/**
Called after {@link MouseHoverInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Immediately shows the tooltip at the given location.
@param location The location in the world coordinate system.
@param [content=null] The content of the tooltip. If <code>null</code>, {@link MouseHoverInputMode#getToolTipContent} will be called to get the
content.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23show(Point%2CObject) Online Documentation}*/
show(location: Point, content?: any | null): void;
/**
Overridden to only return <code>true</code> if this instance does not currently {@link ConcurrencyController#hasMutex have the input mutex}.
@returns <code>true</code> iff this instance does not {@link ConcurrencyController#hasMutex own the mutex}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23tryStop() Online Documentation}*/
tryStop(): boolean;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link MouseHoverInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
Gets or sets the enabled state of this input mode.
Clients can use this property to disable or re-enable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
{@link MouseHoverInputMode#controller} so a disabled instance should never try to acquire the input mutex.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23enabled Online Documentation}*/
enabled: boolean;
/**
Gets the installed {@link MouseHoverInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
/**
Gets or sets an {@link IHitTestable} that determines where the mouse may hover and a tooltip can be {@link MouseHoverInputMode#onShow shown}.
The default value of this property is {@link IHitTestable#ALWAYS}
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23validHoverLocationHitTestable Online Documentation}*/
validHoverLocationHitTestable: IHitTestable | null;
/**
Gets the {@link MouseHoverInputMode#toolTip} instance to use during display.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23toolTip Online Documentation}*/
readonly toolTip: ToolTip | null;
/**
Gets or sets whether the tooltip is currently showing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23showing Online Documentation}*/
showing: boolean;
/**
* Gets or sets the tooltip location offset in view coordinates.
* The value of this property is used by {@link MouseHoverInputMode#getToolTipLocation} in order to move the tooltip
* away from the mouse pointer. The default value is (0,0).
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.mouseHoverInputMode.toolTipLocationOffset = new Point(20, 20)
* </pre>
*
* @see MouseHoverInputMode#getToolTipLocation
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23toolTipLocationOffset Online Documentation}
*/
toolTipLocationOffset: Point;
/**
Gets or sets the duration to show the tooltip.
The default value is 3 seconds.
@default '00:00:03.0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23duration Online Documentation}*/
duration: TimeSpan;
/**
Gets or sets the duration the mouse has to hover in one place to show a tooltip.
The default is one second.
@default '00:00:01.0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23delay Online Documentation}*/
delay: TimeSpan;
/**
Gets or sets the amount the mouse pointer has to move in order to hide the tooltip.
The default value is 10, 10.
@default '10, 10'
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23mouseHoverSize Online Documentation}*/
mouseHoverSize: Size;
/**
If <code>true</code> the tooltip is closed when the a click is performed over the tooltip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23closeOnClick Online Documentation}*/
closeOnClick: boolean;
/**
* Adds the given listener for the <code>QueryToolTip</code> event that occurs when this mode queries the tooltip for a certain query
* location.
* Handlers should set the {@link ToolTipQueryEventArgs#toolTip} property or set/respect the {@link ToolTipQueryEventArgs#handled} property accordingly.
*
* It is recommended to configure the tooltip handling on the parent {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* instead of the <code>MouseHoverInputMode</code>. This involves registering for the {@link GraphInputMode#addQueryItemToolTipListener QueryItemToolTip}
* event instead of this event:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.toolTipItems = GraphItemTypes.NODE
* // register a listener
* mode.addQueryItemToolTipListener((src, args) => {
*   if (args.handled) {
*     // A tooltip has already been assigned -> nothing to do.
*     return
*   }
*   // We can safely cast here because we set ToolTipItems to only Node.
*   /** @type {INode} *‍/
*   const hitNode = args.item
*   if (hitNode.labels.size > 0) {
*     // Show the text of the first label as tooltip.
*     args.toolTip = hitNode.labels.get(0).text
*
*     // Indicate that the tooltip content has been set.
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see MouseHoverInputMode#removeQueryToolTipListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23QueryToolTip Online Documentation}
*/
addQueryToolTipListener(listener: (sender: any, evt: ToolTipQueryEventArgs) => void): void;
/**
* Removes the given listener for the <code>QueryToolTip</code> event that occurs when this mode queries the tooltip for a certain query
* location.
* Handlers should set the {@link ToolTipQueryEventArgs#toolTip} property or set/respect the {@link ToolTipQueryEventArgs#handled} property accordingly.
*
* It is recommended to configure the tooltip handling on the parent {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* instead of the <code>MouseHoverInputMode</code>. This involves registering for the {@link GraphInputMode#addQueryItemToolTipListener QueryItemToolTip}
* event instead of this event:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.toolTipItems = GraphItemTypes.NODE
* // register a listener
* mode.addQueryItemToolTipListener((src, args) => {
*   if (args.handled) {
*     // A tooltip has already been assigned -> nothing to do.
*     return
*   }
*   // We can safely cast here because we set ToolTipItems to only Node.
*   /** @type {INode} *‍/
*   const hitNode = args.item
*   if (hitNode.labels.size > 0) {
*     // Show the text of the first label as tooltip.
*     args.toolTip = hitNode.labels.get(0).text
*
*     // Indicate that the tooltip content has been set.
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see MouseHoverInputMode#addQueryToolTipListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MouseHoverInputMode%23QueryToolTip Online Documentation}
*/
removeQueryToolTipListener(listener: (sender: any, evt: ToolTipQueryEventArgs) => void): void;
static $class: Class;
private $_MouseHoverInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MouseHoverInputMode;
}
export interface MoveInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
/**
@inheritdoc*/
priority: number;
} /**
* Generic {@link IInputMode} implementation that can be used to move something in the canvas using the mouse.
* This implementation uses the {@link MoveInputMode#hitTestable} to determine where the user can begin dragging the selection. It will then use the {@link MoveInputMode#positionHandler} to
* delegate the actual work of moving the elements to.
*
* This mode is {@link MoveInputMode#exclusive} by default.
*
* Typically the <code>MoveInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} and can be retrieved from
* the {@link GraphEditorInputMode#moveInputMode} property.
*
* <br><b>Getting the MoveInputMode from its parent input mode</b>
* <pre>
* const moveInputMode = mode.moveInputMode
* </pre>
*
* If one needs to execute some code after some items have been moved by the {@link MoveInputMode} he can register a
* handler to its {@link MoveInputMode#addDragFinishedListener DragFinished} event. The moved items can
* be retrieved from the {@link MoveInputMode#affectedItems} property:
*
* <pre>
* mode.moveInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.moveInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode Online Documentation}
*/
export class MoveInputMode {
/**
Initializes a new instance of the {@link MoveInputMode} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode Online Documentation}*/
constructor();
/**
Called when this mode is "armed".
This happens when the mouse {@link MoveInputMode#hoverRecognizer hovers} over the item(s) to move. This implementation sets the mouse cursor to {@link MoveInputMode#moveCursor}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23arm() Online Documentation}*/
arm(): void;
/**
Cancels the editing of this mode.
This implementation {@link ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
{@link MoveInputMode#onCanceled}.
@see MoveInputMode#onCanceled
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Creates an {@link IInputModeContext} for use with the {@link IPositionHandler} interface for the upcoming drag
operation.
@returns An instance of {@link IInputModeContext}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23createPositionHandlerInputModeContext() Online Documentation}*/
createPositionHandlerInputModeContext(): IInputModeContext;
/**
Called when this mode is "disarmed".
This happens when the mouse stops hovering over the item(s). This implementation resets the mouse cursor again.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23disarm() Online Documentation}*/
disarm(): void;
/**
Simulates the start of a drag and puts the state machine into the dragging state.
Subsequent moves of input devices will behave as if the dragging has been successfully initiated at <code>location</code>.
@param location The location where the drag had been initialized.
@returns A task that represents the asynchronous drag operation. The result of the task contains the affected items or <code>null</code> if
the drag was canceled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23doStartDrag(Point) Online Documentation}*/
doStartDrag(location: Point | PointConvertible): Promise<IEnumerable<IModelItem>>;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link MoveInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Determines whether the given location is a valid start location for the move gesture.
This implementation delegates to the {@link MoveInputMode#hitTestable}. During this callback it is possible to reset or reconfigure the {@link MoveInputMode#positionHandler}.
@param location The location in the world coordinate system.
@returns <code>true</code> if at the given location it is valid to start the move gesture; otherwise, <code>false</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23isValidStartLocation(Point) Online Documentation}*/
isValidStartLocation(location: Point): boolean;
/**
Called after {@link MoveInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Triggers the {@link MoveInputMode#addDragCanceledListener DragCanceled} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onDragCanceled(InputModeEventArgs) Online Documentation}*/
onDragCanceled(evt: InputModeEventArgs): void;
/**
Triggers the {@link MoveInputMode#addDragCancelingListener DragCanceling} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onDragCanceling(InputModeEventArgs) Online Documentation}*/
onDragCanceling(evt: InputModeEventArgs): void;
/**
Called once the drag has been finalized.
This method triggers the {@link MoveInputMode#addDragFinishedListener DragFinished} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onDragFinished(InputModeEventArgs) Online Documentation}*/
onDragFinished(evt: InputModeEventArgs): void;
/**
Called before the drag will be finalized.
This method triggers the {@link MoveInputMode#addDragFinishingListener DragFinishing} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onDragFinishing(InputModeEventArgs) Online Documentation}*/
onDragFinishing(evt: InputModeEventArgs): void;
/**
Called at the end of each drag.
This method triggers the {@link MoveInputMode#addDraggedListener Dragged} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onDragged(InputModeEventArgs) Online Documentation}*/
onDragged(evt: InputModeEventArgs): void;
/**
Called at the start of each drag.
This method triggers the {@link MoveInputMode#addDraggingListener Dragging} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onDragging(InputModeEventArgs) Online Documentation}*/
onDragging(evt: InputModeEventArgs): void;
/**
Triggers the {@link MoveInputMode#addDragStartedListener DragStarted} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onDragStarted(InputModeEventArgs) Online Documentation}*/
onDragStarted(evt: InputModeEventArgs): void;
/**
Triggers the {@link MoveInputMode#addDragStartingListener DragStarting} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onDragStarting(InputModeEventArgs) Online Documentation}*/
onDragStarting(evt: InputModeEventArgs): void;
/**
Raises the {@link MoveInputMode#addQueryPositionHandlerListener QueryPositionHandler} event.
@param evt The {@link QueryPositionHandlerEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onQueryPositionHandler(QueryPositionHandlerEventArgs) Online Documentation}*/
onQueryPositionHandler(evt: QueryPositionHandlerEventArgs): void;
/**
Called after {@link MoveInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link MoveInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the MoveInputMode on its parent input mode</b>
* <pre>
* mode.moveInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link MoveInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link MoveInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
Gets or sets the event recognizer that determines whether to temporarily disable snapping.
@see MoveInputMode#enableSnappingRecognizer
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23disableSnappingRecognizer Online Documentation}*/
disableSnappingRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to reenable temporarily disabled snapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23enableSnappingRecognizer Online Documentation}*/
enableSnappingRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to start moving the selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23pressedRecognizer Online Documentation}*/
pressedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to the user is hovering over an item where pressing can
initialize a move operation.
The default implementation uses the {@link MoveInputMode#isValidStartLocation} method to determine whether the mode
can be initialized, here.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23hoverRecognizer Online Documentation}*/
hoverRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to move the selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23draggedRecognizer Online Documentation}*/
draggedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to cancel the move.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23cancelRecognizer Online Documentation}*/
cancelRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish moving the selection.
This instance determines when the user has finished the move.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23releasedRecognizer Online Documentation}*/
releasedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to start moving the selection via touch.
This recognizer instance determines when the user begins the move selection operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23pressedRecognizerTouch Online Documentation}*/
pressedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to move the selection via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23draggedRecognizerTouch Online Documentation}*/
draggedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish moving the selection via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23releasedRecognizerTouch Online Documentation}*/
releasedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the {@link IHitTestable} that will be used by this mode to determine where the user may start
dragging.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23hitTestable Online Documentation}*/
hitTestable: IHitTestable;
/**
Gets or sets the {@link IPositionHandler} that will be used as fallback to handle that actual movement of the
elements during the drag.
Note that the {@link IPositionHandler} retrieved by the {@link MoveInputMode#addQueryPositionHandlerListener QueryPositionHandler}
event has priority before this property. Only if the event is not {@link QueryPositionHandlerEventArgs#handled} this property is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23positionHandler Online Documentation}*/
positionHandler: IPositionHandler | null;
/**
Gets or sets the cursor to use during the move.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23moveCursor Online Documentation}*/
moveCursor: Cursor | null;
/**
Gets or sets the {@link MoveInputMode#snapContext} which manages snapping model items to certain coordinates (for instance, other items).
If set to <code>null</code> (the default) this input mode tries to obtain the {@link MoveInputMode#snapContext} from the {@link IInputModeContext}. To explicitly
disable snapping, a {@link MoveInputMode#snapContext} implementation that does nothing has to be set to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23snapContext Online Documentation}*/
snapContext: SnapContext | null;
/**
Adds the given listener for the <code>QueryPositionHandler</code> event that occurs when a drag is recognized for the
{@link MoveInputMode}.
If the event args are {@link QueryPositionHandlerEventArgs#handled}, the {@link IPositionHandler} that is carried with the event args is used to actually move
the items. Otherwise the pre-set {@link MoveInputMode#positionHandler} property is used.
@param listener The listener to add.
@see MoveInputMode#removeQueryPositionHandlerListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23QueryPositionHandler Online Documentation}*/
addQueryPositionHandlerListener(listener: (sender: any, evt: QueryPositionHandlerEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryPositionHandler</code> event that occurs when a drag is recognized for the
{@link MoveInputMode}.
If the event args are {@link QueryPositionHandlerEventArgs#handled}, the {@link IPositionHandler} that is carried with the event args is used to actually move
the items. Otherwise the pre-set {@link MoveInputMode#positionHandler} property is used.
@param listener The listener to remove.
@see MoveInputMode#addQueryPositionHandlerListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23QueryPositionHandler Online Documentation}*/
removeQueryPositionHandlerListener(listener: (sender: any, evt: QueryPositionHandlerEventArgs) => void): void;
/**
Gets a value indicating whether a drag operation is currently in progress.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23isDragging Online Documentation}*/
readonly isDragging: boolean;
/**
Gets the initial position where the dragging was initiated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23initialLocation Online Documentation}*/
readonly initialLocation: Point;
/**
* Gets an immutable snapshot of the {@link IModelItem}s affected by the currently {@link MoveInputMode#isDragging active} gesture.
* When the gesture is {@link MoveInputMode#addDragStartingListener starting} and the {@link MoveInputMode#positionHandler} is {@link IDragHandler#initializeDrag initialized}, the implementation can {@link IModelItemCollector#add register} the affected item(s) through the {@link IModelItemCollector}
* instance that is bound to the {@link MoveInputMode#createPositionHandlerInputModeContext context} available via its {@link ILookup#lookup}.
*
* Client code can register with the {@link MoveInputMode#addDragStartedListener DragStarted} event, as
* well as the {@link MoveInputMode#addDragFinishedListener DragFinished} event to get notified of the
* elements that may be or have been affected respectively by this input mode.
*
* If one needs to execute some code after some items have been moved by the {@link MoveInputMode} he can register a
* handler to its {@link MoveInputMode#addDragFinishedListener DragFinished} event. The moved items can
* be retrieved from the {@link MoveInputMode#affectedItems} property:
*
* <pre>
* mode.moveInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.moveInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23affectedItems Online Documentation}
*/
readonly affectedItems: IEnumerable<IModelItem>;
/**
Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to add.
@see MoveInputMode#removeDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragFinishing Online Documentation}*/
addDragFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to remove.
@see MoveInputMode#addDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragFinishing Online Documentation}*/
removeDragFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Adds the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
* If one needs to execute some code after some items have been moved by the {@link MoveInputMode} he can register a
* handler to its {@link MoveInputMode#addDragFinishedListener DragFinished} event. The moved items can
* be retrieved from the {@link MoveInputMode#affectedItems} property:
*
* <pre>
* mode.moveInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.moveInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see MoveInputMode#removeDragFinishedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragFinished Online Documentation}
*/
addDragFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Removes the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
* If one needs to execute some code after some items have been moved by the {@link MoveInputMode} he can register a
* handler to its {@link MoveInputMode#addDragFinishedListener DragFinished} event. The moved items can
* be retrieved from the {@link MoveInputMode#affectedItems} property:
*
* <pre>
* mode.moveInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.moveInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see MoveInputMode#addDragFinishedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragFinished Online Documentation}
*/
removeDragFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to add.
@see MoveInputMode#removeDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragStarting Online Documentation}*/
addDragStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to remove.
@see MoveInputMode#addDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragStarting Online Documentation}*/
removeDragStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to add.
@see MoveInputMode#removeDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragStarted Online Documentation}*/
addDragStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to remove.
@see MoveInputMode#addDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragStarted Online Documentation}*/
removeDragStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to add.
@see MoveInputMode#removeDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23Dragging Online Documentation}*/
addDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to remove.
@see MoveInputMode#addDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23Dragging Online Documentation}*/
removeDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to add.
@see MoveInputMode#removeDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23Dragged Online Documentation}*/
addDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to remove.
@see MoveInputMode#addDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23Dragged Online Documentation}*/
removeDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to add.
@see MoveInputMode#removeDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragCanceled Online Documentation}*/
addDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to remove.
@see MoveInputMode#addDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragCanceled Online Documentation}*/
removeDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to add.
@see MoveInputMode#removeDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragCanceling Online Documentation}*/
addDragCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to remove.
@see MoveInputMode#addDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveInputMode%23DragCanceling Online Documentation}*/
removeDragCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
static $class: Class;
private $_MoveInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MoveInputMode;
}
export interface QueryPositionHandlerEventArgs extends InputModeEventArgs {
}
/**
Event argument that is used to query the {@link IPositionHandler} that is used to move the items while dragging via
{@link MoveInputMode}.
@see MoveInputMode
@see MoveInputMode#addQueryPositionHandlerListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryPositionHandlerEventArgs Online Documentation}*/
export class QueryPositionHandlerEventArgs {
/**
Instantiates a new {@link QueryPositionHandlerEventArgs}
@param context The {@link IInputModeContext} of the operation.
@param queryLocation The initial location of the movement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryPositionHandlerEventArgs Online Documentation}*/
constructor(context: IInputModeContext, queryLocation: Point | PointConvertible);
/**
Gets or sets the {@link IPositionHandler} to use for the movement of the items that are affected by the
{@link MoveInputMode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryPositionHandlerEventArgs%23positionHandler Online Documentation}*/
positionHandler: IPositionHandler | null;
/**
Gets the location of the movement (where the drag has started).
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryPositionHandlerEventArgs%23queryLocation Online Documentation}*/
queryLocation: Point;
/**
Gets or sets a value indicating whether this {@link QueryPositionHandlerEventArgs} has been handled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryPositionHandlerEventArgs%23handled Online Documentation}*/
handled: boolean;
static $class: Class;
private $_QueryPositionHandlerEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is QueryPositionHandlerEventArgs;
}
/**
Flaggable enumeration used by {@link MoveViewportInputMode#inertia} to determine when to use inertia moving.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/InertiaPolicies Online Documentation}*/
export enum InertiaPolicies {
/**
Never move the viewport using inertia.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InertiaPolicies%23NEVER Online Documentation}*/
NEVER,
/**
Move the viewport using inertia when the mouse has been used to drag the viewport.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InertiaPolicies%23MOUSE Online Documentation}*/
MOUSE,
/**
Move the viewport using inertia when touch has been used to drag the viewport.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InertiaPolicies%23TOUCH Online Documentation}*/
TOUCH,
/**
Always move the viewport using inertia no matter what input gesture has been used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InertiaPolicies%23ALWAYS Online Documentation}*/
ALWAYS
}
type InertiaPoliciesStringValues = "never" | "mouse" | "touch" | "always";
/** Automatic conversion to {@link InertiaPolicies}. */
type InertiaPoliciesConvertible = InertiaPoliciesStringValues | InertiaPoliciesStringValues[];
/**
Flaggable enumeration to determine to which axis touch panning should snap to.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPanningBehaviors Online Documentation}*/
export enum SnapPanningBehaviors {
/**
Don't snap to x- and y-axis when panning
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPanningBehaviors%23NONE Online Documentation}*/
NONE,
/**
Snap to x-axis when panning
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPanningBehaviors%23HORIZONTAL Online Documentation}*/
HORIZONTAL,
/**
Snap to y-axis when panning
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPanningBehaviors%23VERTICAL Online Documentation}*/
VERTICAL,
/**
Snap to x- and y-axis when panning
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapPanningBehaviors%23BOTH Online Documentation}*/
BOTH
}
type SnapPanningBehaviorsStringValues = "none" | "horizontal" | "vertical" | "both";
/** Automatic conversion to {@link SnapPanningBehaviors}. */
type SnapPanningBehaviorsConvertible = SnapPanningBehaviorsStringValues | SnapPanningBehaviorsStringValues[];
export interface MoveViewportInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
uninstall(context: IInputModeContext): void;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} implementation that can be used to grab and drag the viewport of the control it is installed
* in.
* This implementation will request the mutex as soon as it recognizes a drag with the mouse or touch. Zooming is done with
* two fingers using the pinch zoom gesture. If the primary pointer is released during pinch, the secondary pointer is used
* for viewpoint dragging. The pinch zoom then can be restarted by touching another pointer that is recognized as the
* primary pointer.
*
* This mode is {@link MoveViewportInputMode#exclusive} by default.
*
* Typically the <code>MoveViewportInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#moveViewportInputMode} property.
*
* <br><b>Getting the MoveViewportInputMode from its parent input mode</b>
* <pre>
* const moveViewportInputMode = mode.moveViewportInputMode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode Online Documentation}
*/
export class MoveViewportInputMode {
/**
Creates a new instance of the move viewport mode
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode Online Documentation}*/
constructor();
/**
Called to prepare moving the viewport when the mouse is pressed.
This will set the mouse cursor to {@link MoveViewportInputMode#dragCursor}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23arm() Online Documentation}*/
arm(): void;
/**
Cancels navigation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Called after moving the viewport is finished or canceled.
This will reset the mouse cursor.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23disarm() Online Documentation}*/
disarm(): void;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link MoveViewportInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called after {@link MoveViewportInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Triggers the {@link MoveViewportInputMode#addDragCanceledListener DragCanceled} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onDragCanceled(InputModeEventArgs) Online Documentation}*/
onDragCanceled(evt: InputModeEventArgs): void;
/**
Triggers the {@link MoveViewportInputMode#addDragCancelingListener DragCanceling} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onDragCanceling(InputModeEventArgs) Online Documentation}*/
onDragCanceling(evt: InputModeEventArgs): void;
/**
Called once the drag has been finalized.
This method triggers the {@link MoveViewportInputMode#addDragFinishedListener DragFinished}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onDragFinished(InputModeEventArgs) Online Documentation}*/
onDragFinished(evt: InputModeEventArgs): void;
/**
Called before the drag will be finalized.
This method triggers the {@link MoveViewportInputMode#addDragFinishingListener DragFinishing}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onDragFinishing(InputModeEventArgs) Online Documentation}*/
onDragFinishing(evt: InputModeEventArgs): void;
/**
Called at the end of each drag.
This method triggers the {@link MoveViewportInputMode#addDraggedListener Dragged}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onDragged(InputModeEventArgs) Online Documentation}*/
onDragged(evt: InputModeEventArgs): void;
/**
Called at the start of each drag.
This method triggers the {@link MoveViewportInputMode#addDraggingListener Dragging}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onDragging(InputModeEventArgs) Online Documentation}*/
onDragging(evt: InputModeEventArgs): void;
/**
Triggers the {@link MoveViewportInputMode#addDragStartedListener DragStarted} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onDragStarted(InputModeEventArgs) Online Documentation}*/
onDragStarted(evt: InputModeEventArgs): void;
/**
Triggers the {@link MoveViewportInputMode#addDragStartingListener DragStarting} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onDragStarting(InputModeEventArgs) Online Documentation}*/
onDragStarting(evt: InputModeEventArgs): void;
/**
Called after {@link MoveViewportInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link MoveViewportInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link MoveViewportInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the MoveViewportInputMode on its parent input mode</b>
* <pre>
* mode.moveViewportInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link MoveViewportInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Gets or sets the event recognizer that determines whether to start the 'move viewport' gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23primaryDownRecognizer Online Documentation}*/
primaryDownRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to move the viewport or zoom the canvas if a secondary device
is down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23primaryMoveRecognizer Online Documentation}*/
primaryMoveRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish the 'move viewport' or 'pinch' gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23primaryUpRecognizer Online Documentation}*/
primaryUpRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to start the 'pinch' gesture
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23secondaryDownRecognizer Online Documentation}*/
secondaryDownRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to zoom the canvas using the 'pinch' gesture or move the
viewport if no primary device is down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23secondaryMoveRecognizer Online Documentation}*/
secondaryMoveRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish the pinch zoom gesture or finish the move if no
primary device is down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23secondaryUpRecognizer Online Documentation}*/
secondaryUpRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the duration of the inertia movement.
The default value is <code>1.5</code> seconds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23inertiaDuration Online Documentation}*/
inertiaDuration: TimeSpan;
/**
Gets or sets the mode that determines when to use inertia when dragging the viewport.
@default InertiaPolicies.TOUCH
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23inertia Online Documentation}*/
inertia: InertiaPolicies;
/**
Gets or sets a value indicating whether to allow the pinch zoom gesture.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23allowPinchZoom Online Documentation}*/
allowPinchZoom: boolean;
/**
Gets or sets a value indicating whether touch panning should snap to the x- or y-axis.
This only applies to panning with touch gestures.
@see MoveViewportInputMode#snapPanningThreshold
@default SnapPanningBehaviors.NONE
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23snapPanning Online Documentation}*/
snapPanning: SnapPanningBehaviors;
/**
Gets or sets a value that determines how far the pointer has to move away from the snapped axis in order to start with
free panning.
This only applies to panning with touch gestures.
@see MoveViewportInputMode#snapPanning
@default 100
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23snapPanningThreshold Online Documentation}*/
snapPanningThreshold: number;
/**
Gets or sets a value that determines how far the two pointers have to move apart while panning with two fingers in order
to start a zoom gesture.
Moving the fingers away from each other right from the start will instantly result in a pinch zoom.
@default 50
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23pinchZoomThreshold Online Documentation}*/
pinchZoomThreshold: number;
/**
Gets or sets the distance threshold for returning from pinch zoom to two-finger panning with locked zoom level.
@default 5
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23pinchLockZoomThreshold Online Documentation}*/
pinchLockZoomThreshold: number;
/**
Gets or sets the cursor to use during the dragging.
The default is {@link Cursor#POINTER}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23dragCursor Online Documentation}*/
dragCursor: Cursor | null;
/**
Gets or sets the event recognizer that determines whether to start the navigation gesture.
By default this recognizer detects left mouse button presses. If this input mode is a child mode of {@link GraphEditorInputMode}
method {@link GraphEditorInputMode#createMoveViewportInputMode} sets a different recognizer which detects left
clicks with Ctrl held down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23pressedRecognizer Online Documentation}*/
pressedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to drag the viewport.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23draggedRecognizer Online Documentation}*/
draggedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish dragging the viewport.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23releasedRecognizer Online Documentation}*/
releasedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to add.
@see MoveViewportInputMode#removeDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23Dragging Online Documentation}*/
addDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to remove.
@see MoveViewportInputMode#addDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23Dragging Online Documentation}*/
removeDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to add.
@see MoveViewportInputMode#removeDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23Dragged Online Documentation}*/
addDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to remove.
@see MoveViewportInputMode#addDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23Dragged Online Documentation}*/
removeDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to add.
@see MoveViewportInputMode#removeDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragFinishing Online Documentation}*/
addDragFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to remove.
@see MoveViewportInputMode#addDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragFinishing Online Documentation}*/
removeDragFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
@param listener The listener to add.
@see MoveViewportInputMode#removeDragFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragFinished Online Documentation}*/
addDragFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
@param listener The listener to remove.
@see MoveViewportInputMode#addDragFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragFinished Online Documentation}*/
removeDragFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to add.
@see MoveViewportInputMode#removeDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragStarting Online Documentation}*/
addDragStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to remove.
@see MoveViewportInputMode#addDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragStarting Online Documentation}*/
removeDragStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to add.
@see MoveViewportInputMode#removeDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragStarted Online Documentation}*/
addDragStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to remove.
@see MoveViewportInputMode#addDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragStarted Online Documentation}*/
removeDragStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to add.
@see MoveViewportInputMode#removeDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragCanceled Online Documentation}*/
addDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to remove.
@see MoveViewportInputMode#addDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragCanceled Online Documentation}*/
removeDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to add.
@see MoveViewportInputMode#removeDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragCanceling Online Documentation}*/
addDragCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to remove.
@see MoveViewportInputMode#addDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveViewportInputMode%23DragCanceling Online Documentation}*/
removeDragCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
static $class: Class;
private $_MoveViewportInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MoveViewportInputMode;
}
export interface MultiplexingInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
@inheritdoc*/
uninstall(context: IInputModeContext): void;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* A composite {@link IInputMode} implementation that additionally can deal with {@link ConcurrencyController#exclusive} instances.
* Instances of this class can {@link MultiplexingInputMode#install} and {@link MultiplexingInputMode#uninstall}
* multiple {@link IInputMode}s. Child input modes can be added using the {@link MultiplexingInputMode#add}
* method. By providing {@link IInputMode#priority priorities} to the different modes, their {@link IInputMode#install installation} order can be influenced. Input modes with
* lower priorities are handled earlier.
*
* Child input modes can run exclusively while they hold the mutex. Other exclusive input modes are temporarily {@link ConcurrencyController#active deactivated}
* until the mutex is released. This can be controlled by setting the {@link ConcurrencyController#exclusive} property on the controller. Input modes that disable
* the {@link ConcurrencyController#exclusive} property will always run concurrently with all other input modes.
*
* This class itself implements the {@link IInputMode} interface so that hierarchies of instances of this class can
* easily be built. This class will request the input mutex if one of its child modes requests the mutex. Also if the
* instance itself gets {@link MultiplexingInputMode#enabled disabled}, it will stop or cancel the current {@link MultiplexingInputMode#mutexOwner owner of the mutex}.
*
* <pre>
* const multiplexingInputMode = new MultiplexingInputMode()
* const waitInputMode = new WaitInputMode()
* waitInputMode.priority = 0
* multiplexingInputMode.add(waitInputMode)
* const moveInputMode = new MoveViewportInputMode()
* moveInputMode.priority = 5
* multiplexingInputMode.add(moveInputMode)
* graphComponent.inputMode = multiplexingInputMode
* </pre>
*
* @see MultiplexingInputMode#mutexOwner
* @see MultiplexingInputMode#add
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode Online Documentation}
*/
export class MultiplexingInputMode {
/**
Creates an instance with no initial modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode Online Documentation}*/
constructor();
/**
* Adds the given <code>mode</code>.
* The input modes will be ordered according to their {@link IInputMode#priority}: Input modes with lower priority will be {@link IInputMode#install installed} earlier.
*
* Input modes will run exclusively if the {@link ConcurrencyController#exclusive} property of their {@link IInputMode#install installed controller} is set to <code>true</code>. Otherwise they can not
* and will not be deactivated if another {@link IInputMode} acquires the mutex.
*
* <pre>
* const multiplexingInputMode = new MultiplexingInputMode()
* const waitInputMode = new WaitInputMode()
* waitInputMode.priority = 0
* multiplexingInputMode.add(waitInputMode)
* const moveInputMode = new MoveViewportInputMode()
* moveInputMode.priority = 5
* multiplexingInputMode.add(moveInputMode)
* graphComponent.inputMode = multiplexingInputMode
* </pre>
*
* <pre>
* const mode = new GraphEditorInputMode()
* const customInputMode = new CustomInputMode()
* customInputMode.priority = 10
* mode.add(customInputMode)
* </pre>
*
* @param mode The input mode to add to this mode.
* @throws {Exception} <b>(ArgumentError)</b> If the same <code>mode</code> has is already added to this instance.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23add(IInputMode) Online Documentation}
*/
add(mode: IInputMode): void;
/**
Adjusts the cursor of the {@link CanvasComponent} according to the current input mutex owner or the first mode in
the list whose {@link ConcurrencyController} returns a non-null {@link ConcurrencyController#preferredCursor}.
This method will set {@link MultiplexingInputMode#defaultCursor} as the current cursor if no other {@link ConcurrencyController#preferredCursor} has been specified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23adjustCursor() Online Documentation}*/
adjustCursor(): void;
/**
Cancels all modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Called by the {@link MultiplexingInputMode#createChildInputModeContext child context's} {@link ILookup#lookup} method.
@param type The type argument passed to {@link ILookup#lookup}.
@returns The result of the lookup query, or <code>null</code>.
@see MultiplexingInputMode#createChildInputModeContext
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23childInputModeContextLookup(Class) Online Documentation}*/
childInputModeContextLookup(type: Class): any | null;
/**
Yields an {@link IInputModeContext} for the child modes of this mode.
This method is called during {@link MultiplexingInputMode#install installation} to create a new context for the child modes and can be used by client code to
obtain a suitable context object. The {@link MultiplexingInputMode#inputModeContext} property is already set, when this method is called. {@link ILookup#lookup}
calls for the created context will be resolved by this instance's {@link MultiplexingInputMode#childInputModeContextLookup}
method.
@returns A new instance that delegates to the {@link MultiplexingInputMode#inputModeContext parent's context.}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23createChildInputModeContext() Online Documentation}*/
createChildInputModeContext(): IInputModeContext;
/**
Helper method that yields a suitably configured {@link InputModeEventArgs} for this input mode.
@param context An input mode context that is available in the {@link InputModeEventArgs}.
Can be <code>null</code> in which case a new context for this instance is created automatically.
@returns An input mode event argument that is configured for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23createInputModeEventArgs(IInputModeContext) Online Documentation}*/
createInputModeEventArgs(context: IInputModeContext | null): InputModeEventArgs;
/**
Returns a list of all modes managed by this instance in sorted order.
@returns A list of the modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23getSortedModes() Online Documentation}*/
getSortedModes(): IList<IInputMode>;
/**
Performs one-time initialization of this instance.
This method should not be invoked by subclasses. This will be done automatically upon first {@link MultiplexingInputMode#install installation} of this mode.
This code will be executed only once per instance. The {@link MultiplexingInputMode#inputModeContext} property will be <code>null</code> when this code is executed. This method
should not be used to install this mode into a specific canvas. Subclasses should always call <code>base.Initialize()</code> first.
@see MultiplexingInputMode#install
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23initialize() Online Documentation}*/
initialize(): void;
/**
{@link CanvasComponent#invalidate Invalidates} the canvas this mode is currently installed in.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23invalidate() Online Documentation}*/
invalidate(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
This implementation reenables previously disabled concurrent child modes.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
This implementation sets the {@link ConcurrencyController#preferredCursor} property to <code>null</code> and {@link ConcurrencyController#releaseMutex releases} the mutex if the mutex is currently owned by this instance.
Also, all concurrent child modes will be disabled.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Removes the given mode from this compound mode.
@param mode The mode to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23remove(IInputMode) Online Documentation}*/
remove(mode: IInputMode): void;
/**
Called when the {@link IInputMode#priority} of an {@link MultiplexingInputMode#add installed} sub mode has changed.
This implementation {@link MultiplexingInputMode#uninstall uninstalls} all sub modes and then re-{@link MultiplexingInputMode#install installs} them according to their new priorities.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23subModePriorityChanged() Online Documentation}*/
subModePriorityChanged(): void;
/**
Tries to stop all modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23tryStop() Online Documentation}*/
tryStop(): boolean;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link MultiplexingInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
Gets or sets the enabled state of this input mode.
Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
{@link MultiplexingInputMode#controller} so a disabled instance should never try to acquire the input mutex.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23enabled Online Documentation}*/
enabled: boolean;
/**
Gets the installed {@link MultiplexingInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
/**
Gets the {@link IInputMode} that currently owns the mutex.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23mutexOwner Online Documentation}*/
readonly mutexOwner: IInputMode | null;
/**
Gets or sets the cursor to use whenever no child mode prefers a different cursor.
The default is <code>null</code>
@see MultiplexingInputMode#adjustCursor
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiplexingInputMode%23defaultCursor Online Documentation}*/
defaultCursor: Cursor | null;
static $class: Class;
private $_MultiplexingInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MultiplexingInputMode;
}
export interface OverviewInputMode extends MultiplexingInputMode {
}
/**
An {@link IInputMode} that can be used for an overview {@link CanvasComponent}.
This mode will add navigation capabilities to the control it is installed in.
@see OverviewInputMode#canvasComponent
@see OverviewInputMode#handleInputMode
@see OverviewInputMode#moveInputMode
@see OverviewInputMode#keyboardInputMode
@see OverviewInputMode#clickInputMode
@see OverviewInputMode#tapInputMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode Online Documentation}*/
export class OverviewInputMode {
/**
Creates a new instance.
@param [options] The parameters to pass.
@param [options.autoInvalidate] Whether the the canvas this mode is installed in should automatically be invalidated if the client canvas gets invalidated. This option sets the {@link OverviewInputMode#autoInvalidate} property on the created object.
@param [options.canvasComponent] The canvas this canvas should use to navigate. This option sets the {@link OverviewInputMode#canvasComponent} property on the created object.
@param [options.template] The template that is used for the visualization of the marquee rectangle. This option sets the {@link OverviewInputMode#template} property on the created object.
@param [options.mouseWheelBehavior] The behavior of the mouse wheel. This option sets the {@link OverviewInputMode#mouseWheelBehavior} property on the created object.
@param [options.margins] The insets in view coordinates that should be used by the {@link OverviewInputMode#updateVisibleArea} operation. This option sets the {@link OverviewInputMode#margins} property on the created object.
@param [options.exclusive] A value indicating whether this mode will be the only one running when it has the mutex. This option sets the {@link MultiplexingInputMode#exclusive} property on the created object.
@param [options.enabled] The enabled state of this input mode. This option sets the {@link MultiplexingInputMode#enabled} property on the created object.
@param [options.priority] Sets the priority property.
@param [options.defaultCursor] The cursor to use whenever no child mode prefers a different cursor. This option sets the {@link MultiplexingInputMode#defaultCursor} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode Online Documentation}*/
constructor(options?: {
autoInvalidate?: boolean;
canvasComponent?: CanvasComponent | null;
template?: IVisualTemplate | null;
mouseWheelBehavior?: MouseWheelBehaviors | MouseWheelBehaviorsConvertible;
margins?: Insets | InsetsConvertible;
exclusive?: boolean;
enabled?: boolean;
priority?: number;
defaultCursor?: Cursor | null | CursorConvertible;
});
/**
Factory method for the {@link OverviewInputMode#clickInputMode} property.
This method will be called upon first access to the {@link OverviewInputMode#clickInputMode} property.
@returns a new instance of ClickInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23createClickInputMode() Online Documentation}*/
createClickInputMode(): ClickInputMode;
/**
Factory method for the {@link OverviewInputMode#handleInputMode} property.
This method will be called upon first access to the {@link OverviewInputMode#handleInputMode} property.
@returns a new instance of {@link HandleInputMode}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23createHandleInputMode() Online Documentation}*/
createHandleInputMode(): HandleInputMode;
/**
Factory method for the {@link OverviewInputMode#keyboardInputMode} property.
This method will be called upon first access to the {@link OverviewInputMode#keyboardInputMode} property.
@returns a new instance of {@link KeyboardInputMode}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23createKeyboardInputMode() Online Documentation}*/
createKeyboardInputMode(): KeyboardInputMode;
/**
Factory method for the {@link OverviewInputMode#moveInputMode} property.
This method will be called upon first access to the {@link OverviewInputMode#moveInputMode} property.
@returns a new instance of MoveInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23createMoveInputMode() Online Documentation}*/
createMoveInputMode(): MoveInputMode;
/**
Factory method for the {@link OverviewInputMode#tapInputMode} property.
This method will be called upon first access to the {@link OverviewInputMode#tapInputMode} property.
@returns a new instance of TapInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23createTapInputMode() Online Documentation}*/
createTapInputMode(): TapInputMode;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link ConcurrencyController} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called by {@link OverviewInputMode#createKeyboardInputMode} to determine which of the built-in
{@link ICommand}s to install.
This implementation unconditionally returns <code>true</code>, subclasses may override this method to adjust the behavior.
@param command The command to install.
@returns Whether to install this command.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23shouldInstallCommand(ICommand) Online Documentation}*/
shouldInstallCommand(command: ICommand): boolean;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Updates the {@link CanvasComponent#zoom zoom} and {@link CanvasComponent#viewPoint viewpoint} of the overview control.
This method is called when the {@link CanvasComponent#contentRect graph bounds} of the {@link OverviewInputMode#canvasComponent } have changed. The {@link OverviewInputMode#margins margins} should be respected.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23updateVisibleArea() Online Documentation}*/
updateVisibleArea(): void;
/**
Gets or sets whether the the canvas this mode is installed in should automatically be invalidated if the client canvas
gets invalidated.
Automatic invalidation will be deferred to avoid too frequent updates of the overview.
To manually update the overview when this property is <code>false</code>, call the {@link CanvasComponent#invalidate} method on
the {@link GraphOverviewComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23autoInvalidate Online Documentation}*/
autoInvalidate: boolean;
/**
A resource key that should yield a {@link IVisualTemplate} that will be used by this mode render the viewport.
When explicitly setting a {@link OverviewInputMode#template}, this key will not be used. This is only a fallback if {@link OverviewInputMode#template} is <code>null</code>.
The default template has the <code>yfiles-overview-viewport-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23VIEWPORT_TEMPLATE_KEY Online Documentation}*/
static VIEWPORT_TEMPLATE_KEY: string;
/**
Gets the {@link HandleInputMode}.
This mode is responsible for handling the single handle that allows to resize the viewport rectangle.
If the field has not yet been initialized upon first access, the factory method {@link OverviewInputMode#createHandleInputMode}
will be called.
By default this input mode has a priority of <code>10</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23handleInputMode Online Documentation}*/
readonly handleInputMode: HandleInputMode;
/**
Gets the {@link MoveInputMode}.
This mode is responsible for moving the viewport rectangle.
If the field has not yet been initialized upon first access, the factory method {@link OverviewInputMode#createMoveInputMode}
will be called.
By default this input mode has a priority of <code>20</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23moveInputMode Online Documentation}*/
readonly moveInputMode: MoveInputMode;
/**
Gets the {@link OverviewInputMode#clickInputMode}.
If the field has not yet been initialized upon first access, the factory method {@link OverviewInputMode#createClickInputMode}
will be called.
By default this input mode has a priority of <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23clickInputMode Online Documentation}*/
readonly clickInputMode: ClickInputMode;
/**
Gets the {@link OverviewInputMode#tapInputMode}.
If the field has not yet been initialized upon first access, the factory method {@link OverviewInputMode#createTapInputMode}
will be called.
By default this input mode has a priority of <code>30</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23tapInputMode Online Documentation}*/
readonly tapInputMode: TapInputMode;
/**
Gets the {@link KeyboardInputMode}.
This mode handles all keyboard interaction gestures with the overview control.
If the field has not yet been initialized upon first access, the factory method {@link OverviewInputMode#createKeyboardInputMode}
will be called.
By default this input mode has a priority of <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23keyboardInputMode Online Documentation}*/
readonly keyboardInputMode: KeyboardInputMode;
/**
Gets the list of commands that are available in this instance.
Removing commands from this collection also removes the command bindings registered by this instance.
Add supported commands to make them available in this instance.
Supported commands are
<ul>
<li>{@link ICommand#MOVE_LEFT}
<li>{@link ICommand#MOVE_RIGHT}
<li>{@link ICommand#MOVE_UP}
<li>{@link ICommand#MOVE_DOWN}
<li>{@link ICommand#ZOOM}
<li>{@link ICommand#INCREASE_ZOOM}
<li>{@link ICommand#DECREASE_ZOOM}
<li>{@link ICommand#SCROLL_PAGE_DOWN}
<li>{@link ICommand#SCROLL_PAGE_UP}
<li>{@link ICommand#SCROLL_PAGE_LEFT}
<li>{@link ICommand#SCROLL_PAGE_RIGHT}
<li>{@link ICommand#FIT_CONTENT}
<li>{@link ICommand#UPDATE_CONTENT_RECT}
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23availableCommands Online Documentation}*/
readonly availableCommands: ICollection<ICommand>;
/**
Gets or sets the canvas this canvas should use to navigate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23canvasComponent Online Documentation}*/
canvasComponent: CanvasComponent | null;
/**
Gets or sets the template that is used for the visualization of the marquee rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23template Online Documentation}*/
template: IVisualTemplate | null;
/**
Gets or sets the behavior of the mouse wheel.
The behavior can be switched between {@link MouseWheelBehaviors#ZOOM}, {@link MouseWheelBehaviors#SCROLL}, and
{@link MouseWheelBehaviors#NONE}. Additionally, this property can be configured to perform the action
{@link MouseWheelBehaviors#ONLY_WHEN_FOCUSED only when the control is focused}.
The configured behavior affects the {@link OverviewInputMode#canvasComponent}' viewport.
@default MouseWheelBehaviors.ZOOM
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23mouseWheelBehavior Online Documentation}*/
mouseWheelBehavior: MouseWheelBehaviors;
/**
Gets and sets the insets in view coordinates that should be used by the {@link OverviewInputMode#updateVisibleArea}
operation.
This influences the amount of visible whitespace in the view coordinate system around the graph after the {@link CanvasComponent#contentRect content rect}
of the {@link OverviewInputMode#canvasComponent} has changed. The default value is <code>(5,5,5,5)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OverviewInputMode%23margins Online Documentation}*/
margins: Insets;
static $class: Class;
private $_OverviewInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OverviewInputMode;
}
export interface PopulateItemContextMenuEventArgs<TModelItem> extends PopulateMenuEventArgs {
}
/**
A specialized subclass of {@link PopulateMenuEventArgs} that carries the {@link PopulateItemContextMenuEventArgs#item} for which the context menu should be
populated.
Type parameter <code>TModelItem</code>: The type of the {@link PopulateItemContextMenuEventArgs#item model item}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PopulateItemContextMenuEventArgs Online Documentation}*/
export class PopulateItemContextMenuEventArgs<TModelItem> {
/**
Initializes a new instance of the {@link PopulateItemContextMenuEventArgs} class.
@param context The context to populate the menu for.
@param queryLocation The query location.
@param item The item for which the context menu should be populated, may be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PopulateItemContextMenuEventArgs Online Documentation}*/
constructor(context: IInputModeContext, queryLocation: Point | PointConvertible, item: TModelItem | null);
/**
Gets the item for which the tool tip is queried.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PopulateItemContextMenuEventArgs%23item Online Documentation}*/
readonly item: TModelItem | null;
static $class: Class;
private $_PopulateItemContextMenuEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TModelItem>(o: any): o is PopulateItemContextMenuEventArgs<TModelItem>;
}
export interface QueryItemToolTipEventArgs<TModelItem> extends ToolTipQueryEventArgs {
}
/**
A specialized subclass of {@link ToolTipQueryEventArgs} that carries the {@link QueryItemToolTipEventArgs#item} for which the tool tip is queried.
Type parameter <code>TModelItem</code>: The type of the {@link QueryItemToolTipEventArgs#item model item}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryItemToolTipEventArgs Online Documentation}*/
export class QueryItemToolTipEventArgs<TModelItem> {
/**
Initializes a new instance of the {@link QueryItemToolTipEventArgs} class.
@param context The context to yield the tool tip for.
@param queryLocation The query location.
@param item The item for which the tool tip is queried..
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryItemToolTipEventArgs Online Documentation}*/
constructor(context: IInputModeContext, queryLocation: Point | PointConvertible, item: TModelItem | null);
/**
Gets the item for which the tool tip is queried.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryItemToolTipEventArgs%23item Online Documentation}*/
readonly item: TModelItem | null;
static $class: Class;
private $_QueryItemToolTipEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TModelItem>(o: any): o is QueryItemToolTipEventArgs<TModelItem>;
}
export interface RectangleReshapeHandleProvider extends ReshapeHandleProviderBase {
}
/**
A simple implementation of an {@link IReshapeHandleProvider} that returns handles that modify a
{@link IMutableRectangle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleReshapeHandleProvider Online Documentation}*/
export class RectangleReshapeHandleProvider {
/**
Creates a default instance that provide handles for each of the positions given for the rectangle.
@param rectangle The rectangle to create handles for.
@param [handlePositions=HandlePositions.BORDER] A bitwise combination of all handle positions this instance should provide a handle for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleReshapeHandleProvider Online Documentation}*/
constructor(rectangle: IMutableRectangle, handlePositions?: HandlePositions | HandlePositionsConvertible);
/**
Provides a {@link IHandle} that uses the rectangle instance to perform the actual reshaping.
This method may be called for each possible single position contained in the set as returned by
{@link IReshapeHandleProvider#getAvailableHandles}.
@param context The context for which the handles are queried.
@param position The position to provide an instance for.
@returns A handle implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleReshapeHandleProvider%23getHandle(IInputModeContext%2CHandlePositions) Online Documentation}*/
getHandle(context: IInputModeContext, position: HandlePositions | HandlePositionsConvertible): IHandle;
/**
Gets the rectangle to read the current state from.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleReshapeHandleProvider%23rectangle Online Documentation}*/
rectangle: IMutableRectangle;
static $class: Class;
private $_RectangleReshapeHandleProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RectangleReshapeHandleProvider;
}
export interface ReshapeHandleProviderBase extends YObject, IReshapeHandleProvider {
}
/**
Abstract base class implementation of the {@link IReshapeHandleProvider} interface that has properties to restrict
the size and area of a reshapeable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandleProviderBase Online Documentation}*/
export abstract class ReshapeHandleProviderBase {
constructor();
/**
Returns the {@link ReshapeHandleProviderBase#handlePositions} property.
@param context
@returns A bitwise combination of all handle positions this instance provides a handle for if queried in
{@link RectangleReshapeHandleProvider#getHandle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandleProviderBase%23getAvailableHandles(IInputModeContext) Online Documentation}*/
getAvailableHandles(context: IInputModeContext): HandlePositions;
/**
Provides a {@link IHandle} that uses the rectangle and reshapeable instance bound to this instance to perform the
actual reshaping.
This method may be called for each possible single position contained in the set as returned by
{@link ReshapeHandleProviderBase#getAvailableHandles}.
@param context The context for which the handles are queried.
@param position The position to provide an instance for.
@returns A handle implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandleProviderBase%23getHandle(IInputModeContext%2CHandlePositions) Online Documentation}*/
abstract getHandle(context: IInputModeContext, position: HandlePositions | HandlePositionsConvertible): IHandle;
/**
Gets or sets the minimum size allowed for the reshapeable.
The default value is {@link Size#ZERO}.
@see RectangleHandle#minimumSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandleProviderBase%23minimumSize Online Documentation}*/
minimumSize: ISize;
/**
Gets or sets the minimum rectangular area that needs to be contained in the reshapeable.
The default value is {@link Rect#EMPTY}.
@see RectangleHandle#minimumEnclosedArea
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandleProviderBase%23minimumEnclosedArea Online Documentation}*/
minimumEnclosedArea: IRectangle;
/**
Gets or sets the maximal bounds this reshapeable may fill.
The value is stored by reference. The default is
{@link Rect#INFINITE}, which does not restrict the area at all.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandleProviderBase%23maximumBoundingArea Online Documentation}*/
maximumBoundingArea: IRectangle;
/**
Gets or sets the maximum size allowed for the reshapeable.
The default value is {@link Size#INFINITE}.
@see RectangleHandle#maximumSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandleProviderBase%23maximumSize Online Documentation}*/
maximumSize: ISize;
/**
Gets or sets the bitwise combination of all handle positions this instance provides a handle for.
@see ReshapeHandleProviderBase#getAvailableHandles
@see RectangleReshapeHandleProvider#getHandle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandleProviderBase%23handlePositions Online Documentation}*/
handlePositions: HandlePositions;
static $class: Class;
private $_ReshapeHandleProviderBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReshapeHandleProviderBase;
}
/**
The policy constant used by {@link NodeReshapeHandleProvider} and {@link ReshapeHandlerHandle}.
The policy values determine how the mouse location is interpreted when the reshape gesture shall keep the aspect ratio.
@see NodeReshapeHandleProvider#ratioReshapeRecognizer
@see ReshapeHandlerHandle#ratioReshapeRecognizer
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapePolicy Online Documentation}*/
export enum ReshapePolicy {
/**
The aspect ratio is ignored.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapePolicy%23NONE Online Documentation}*/
NONE,
/**
Takes the bigger of the two bounds that would result when taking the {@link ReshapePolicy#HORIZONTAL} and {@link ReshapePolicy#VERTICAL}
policies.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapePolicy%23MAXIMUM Online Documentation}*/
MAXIMUM,
/**
Takes the smaller of the two bounds that would result when taking the {@link ReshapePolicy#HORIZONTAL} and {@link ReshapePolicy#VERTICAL}
policies.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapePolicy%23MINIMUM Online Documentation}*/
MINIMUM,
/**
Considers only the horizontal part of the mouse movement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapePolicy%23HORIZONTAL Online Documentation}*/
HORIZONTAL,
/**
Considers only the vertical part of the mouse movement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapePolicy%23VERTICAL Online Documentation}*/
VERTICAL,
/**
Projects the mouse location onto the resize direction line.
The line in resize direction runs through the moved handle and its opposite handle, so e.g. when moving the handle at
{@link HandlePositions#NORTH_EAST}, the line between the {@link HandlePositions#NORTH_EAST} and {@link HandlePositions#SOUTH_WEST}
is used for the projection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapePolicy%23PROJECTION Online Documentation}*/
PROJECTION
}
/** Automatic conversion to {@link ReshapePolicy}. */
type ReshapePolicyStringValues = "none" | "maximum" | "minimum" | "horizontal" | "vertical" | "projection";
export interface SelectionEventArgs<T extends IModelItem> extends InputModeEventArgs {
}
/**
Event arguments that are used when modifying a {@link ISelectionModel}
Type parameter <code>T</code>: The type of the items in the {@link ISelectionModel}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionEventArgs Online Documentation}*/
export class SelectionEventArgs<T extends IModelItem> {
/**
Creates a new instance.
@param context The context the event with these event args is dispatched.
@param selection The selection that is modified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionEventArgs Online Documentation}*/
constructor(context: IInputModeContext, selection: ISelectionModel<T>);
/**
Gets the selection these event arguments refer to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionEventArgs%23selection Online Documentation}*/
selection: ISelectionModel<T>;
static $class: Class;
private $_SelectionEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T extends IModelItem>(o: any): o is SelectionEventArgs<T>;
}
export interface SnapContext extends YObject {
}
/**
Manages interactive snapping of elements to other elements during drag operations like movements.
This class should be used by {@link IInputMode} implementations and similar code as follows:
<ul>
<li>To initialize the operation {@link SnapContext#initializeDrag} needs to be called.
<li>Method {@link SnapContext#dragInitialized} needs to be called after the mode has performed all of its own
initialization.
<li>Then a series of zero or more invocations {@link SnapContext#handleMove} needs to be called for each incremental
movement of the user's input device.
<li>After {@link SnapContext#handleMove} has been called, the mode should perform the move and report back to the
instance by calling the {@link SnapContext#dragged} method.
<li>If the operation is canceled by the user {@link SnapContext#cancelDrag} should be called.
<li>If the operation is finalized the {@link SnapContext#dragFinished} method should be called.
</ul>
@see HandleInputMode#snapContext
@see MoveInputMode#snapContext
@see SnapContext#createSnapResultsModelManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext Online Documentation}*/
export class SnapContext {
/**
Initializes a new instance of the {@link SnapContext} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext Online Documentation}*/
constructor();
/**
Called when a drag has been canceled.
Clears all temporary lists and sets {@link SnapContext#isInitialized} back to false, as well as performs {@link SnapContext#cleanUp} and finally
triggers the {@link SnapContext#addCleanedUpListener CleanedUp} event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23cancelDrag() Online Documentation}*/
cancelDrag(): void;
/**
Performs clean up procedures.
This is called in response to {@link SnapContext#cancelDrag} and {@link SnapContext#dragFinished} as well as
initially during {@link SnapContext#initializeDrag}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23cleanUp() Online Documentation}*/
cleanUp(): void;
/**
Helper method used by {@link SnapContext#createSnapResultsModelManager} to create the {@link ICanvasObjectDescriptor}
for the {@link SnapResult} type.
@returns An instance that can install {@link SnapResult}s in a {@link CanvasComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23createSnapResultCanvasObjectDescriptor() Online Documentation}*/
createSnapResultCanvasObjectDescriptor(): ICanvasObjectDescriptor;
/**
Helper method that creates the a {@link CollectionModelManager} that can be used to present the current {@link SnapResult}s in the provided
{@link CanvasComponent}.
@param canvas The control to initialize the {@link CollectionModelManager} with.
@returns The model manager that manages the visual representation of the {@link SnapResult}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23createSnapResultsModelManager(CanvasComponent) Online Documentation}*/
createSnapResultsModelManager(canvas: CanvasComponent): CollectionModelManager<SnapResult>;
/**
Called when a drag has been successfully finished.
Clears all temporary lists and sets {@link SnapContext#isInitialized} back to false. Also the {@link SnapContext#cleanUp} method is invoked and the
{@link SnapContext#addCleanedUpListener CleanedUp} event is triggered.
@param newLocation The current mouse location.
@param snappingDisabled If set to <code>true</code> snapping is temporarily disabled. This will remove the snap line visualizations and return a {@link SnapState}
with the unchanged coordinates and the {@link SnapTypes#NOT_SNAPPED} {@link SnapState#snapType}.
@returns A {@link SnapState} that specifies the adjusted coordinates and how they have been adjusted. The unchanged
coordinates if snapping has been temporarily disabled by setting <code>snappingDisabled</code> to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23dragFinished(Point%2Cboolean) Online Documentation}*/
dragFinished(newLocation: Point, snappingDisabled: boolean): SnapState;
/**
This method needs to be called by the client {@link IInputMode} that {@link SnapContext#initializeDrag initialized} this instance after it has
performed the actual move operation with the adjusted coordinates after a call to {@link SnapContext#handleMove}.
Most of the client code implementations will just call this method with the exact values returned by
{@link SnapContext#handleMove}. This method does not need to be called if snapping has been temporarily disabled by
calling {@link SnapContext#handleMove} with its <code>snappingDisabled</code> parameter set to <code>true</code>.
@param mouseLocation The actual mouse location.
@param finalSnapState The final snap state that contains the location that has been used by the client code for the move and how it has been
snapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23dragged(Point%2CSnapState) Online Documentation}*/
dragged(mouseLocation: Point, finalSnapState: SnapState): void;
/**
Collects a list of snap lines.
Snap lines represent possible horizontal or vertical lines to which a moved item can "snap".
This method is called at the start of {@link MoveInputMode}'s or
{@link HandleInputMode}'s gesture after all items to be moved or reshaped have been added.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23dragInitialized() Online Documentation}*/
dragInitialized(): void;
/**
Handles a move.
This method returns the adjusted (mouse) coordinates. It also manages the collection of {@link SnapResult}s. If
snapping should be temporarily (for a mouse move) disabled, <code>snappingDisabled</code> has to be set to <code>true</code>.
@param newLocation The current mouse location.
@param snappingDisabled If set to <code>true</code> snapping is temporarily disabled. This will remove the snap line visualizations and return a {@link SnapState}
with the unchanged coordinates and the {@link SnapState#snapType} {@link SnapTypes#NOT_SNAPPED}.
@returns The adjusted coordinates and how they have been adjusted. The unchanged coordinates if snapping has been temporarily
disabled by setting <code>snappingDisabled</code> to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23handleMove(Point%2Cboolean) Online Documentation}*/
handleMove(newLocation: Point, snappingDisabled: boolean): SnapState;
/**
Called by {@link IInputMode} implementations when an interactive drag is started.
Updates {@link SnapContext#isInitializing}, {@link SnapContext#originalLocation}, and {@link SnapContext#currentInputModeContext} and then triggers the {@link SnapContext#addInitializingListener Initializing} event.
@param context The context in which the interactive drag is started.
@param originalLocation The original location of the mouse.
@throws {Exception} <b>(InvalidOperationError)</b> If this context is already {@link SnapContext#isInitialized initialized}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23initializeDrag(IInputModeContext%2CPoint) Online Documentation}*/
initializeDrag(context: IInputModeContext, originalLocation: Point): void;
/**
Raises the {@link SnapContext#cleanUp} event.
@param evt The {@link InputModeEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23onCleanUp(InputModeEventArgs) Online Documentation}*/
onCleanUp(evt: InputModeEventArgs): void;
/**
Raises the {@link SnapContext#addCollectSnapResultsListener CollectSnapResults} event.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23onCollectSnapResults(CollectSnapResultsEventArgs) Online Documentation}*/
onCollectSnapResults(evt: CollectSnapResultsEventArgs): void;
/**
Raises the {@link SnapContext#addInitializedListener Initialized} event.
@param evt The {@link InputModeEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23onInitialized(InputModeEventArgs) Online Documentation}*/
onInitialized(evt: InputModeEventArgs): void;
/**
Raises the {@link SnapContext#addInitializingListener Initializing} event.
@param evt The {@link InputModeEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23onInitializing(InputModeEventArgs) Online Documentation}*/
onInitializing(evt: InputModeEventArgs): void;
/**
Chooses the current {@link SnapResult}s from a temporary list.
Chooses from a list of possible results the most important ones. Also maintains the collection of snap results.
@param snapResults A temporary list of suggested {@link SnapResult}s.
@param originalLocation The point in world coordinates where the mouse move started.
@param newLocation The current mouse location.
@returns The adjusted coordinates and how they have been adjusted.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23processSnapResults(List%3CSnapResult%3E%2CPoint%2CPoint) Online Documentation}*/
processSnapResults(snapResults: List<SnapResult>, originalLocation: Point, newLocation: Point): SnapState;
/**
Helper method that wraps the given <code>context</code> so that a {@link ILookup#lookup} query on the wrapped context for the {@link SnapContext}
type yields this instance.
@param context The context to wrap and delegate all calls to.
@returns A modified instance that yields this instance if it is queried for the {@link SnapContext} type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23wrapContext(IInputModeContext) Online Documentation}*/
wrapContext(context: IInputModeContext): IInputModeContext;
/**
Adds the given listener for the <code>CollectSnapResults</code> event that occurs while {@link IDragHandler#handleMove moving} items and handles.
The moving model items must register for this event, preferably in the {@link IDragHandler#initializeDrag} method
of the {@link IDragHandler}.
The event handler has to add {@link SnapResult}s for the moving object to the list. The
{@link IDragHandler}s of the moved items have to take care themselves whether to add a handler to the list
depending on the their settings.
@param listener The listener to add.
@see SnapContext#removeCollectSnapResultsListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23CollectSnapResults Online Documentation}*/
addCollectSnapResultsListener(listener: (sender: any, evt: CollectSnapResultsEventArgs) => void): void;
/**
Removes the given listener for the <code>CollectSnapResults</code> event that occurs while {@link IDragHandler#handleMove moving} items and handles.
The moving model items must register for this event, preferably in the {@link IDragHandler#initializeDrag} method
of the {@link IDragHandler}.
The event handler has to add {@link SnapResult}s for the moving object to the list. The
{@link IDragHandler}s of the moved items have to take care themselves whether to add a handler to the list
depending on the their settings.
@param listener The listener to remove.
@see SnapContext#addCollectSnapResultsListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23CollectSnapResults Online Documentation}*/
removeCollectSnapResultsListener(listener: (sender: any, evt: CollectSnapResultsEventArgs) => void): void;
/**
Adds the given listener for the <code>CleanedUp</code> event that occurs when this instance has been cleaned up.
@param listener The listener to add.
@see SnapContext#cleanUp
@see SnapContext#removeCleanedUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23CleanedUp Online Documentation}*/
addCleanedUpListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>CleanedUp</code> event that occurs when this instance has been cleaned up.
@param listener The listener to remove.
@see SnapContext#cleanUp
@see SnapContext#addCleanedUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23CleanedUp Online Documentation}*/
removeCleanedUpListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Initialized</code> event that occurs when this instance has been initialized.
@param listener The listener to add.
@see SnapContext#initializeDrag
@see SnapContext#removeInitializedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23Initialized Online Documentation}*/
addInitializedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Initialized</code> event that occurs when this instance has been initialized.
@param listener The listener to remove.
@see SnapContext#initializeDrag
@see SnapContext#addInitializedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23Initialized Online Documentation}*/
removeInitializedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Initializing</code> event that occurs when this instance is about to be initialized.
@param listener The listener to add.
@see SnapContext#initializeDrag
@see SnapContext#removeInitializingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23Initializing Online Documentation}*/
addInitializingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Initializing</code> event that occurs when this instance is about to be initialized.
@param listener The listener to remove.
@see SnapContext#initializeDrag
@see SnapContext#addInitializingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23Initializing Online Documentation}*/
removeInitializingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Gets or sets a value indicating whether to visualize the snap results.
@see SnapContext#createSnapResultsModelManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23visualizeSnapResults Online Documentation}*/
visualizeSnapResults: boolean;
/**
Gets or sets a value indicating whether this instance is currently initializing.
@see SnapContext#addInitializedListener
@see SnapContext#addInitializingListener
@see SnapContext#isInitialized
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23isInitializing Online Documentation}*/
isInitializing: boolean;
/**
Gets the current input mode context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23currentInputModeContext Online Documentation}*/
readonly currentInputModeContext: IInputModeContext | null;
/**
Gets the collection of {@link SnapResult}s.
This collection is automatically updated during moving the mouse.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23snapResults Online Documentation}*/
readonly snapResults: IObservableCollection<SnapResult>;
/**
Gets whether the context is initialized.
The context is initialized between the calls to {@link SnapContext#dragInitialized} and {@link SnapContext#dragFinished}
or {@link SnapContext#cancelDrag}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23isInitialized Online Documentation}*/
isInitialized: boolean;
/**
Gets or sets the global snap distance, which is the maximum distance between the current mouse coordinates and the
coordinates to which the mouse will snap.
The distance is interpreted in view coordinates.
The default is 5.0d
@see CollectSnapResultsEventArgs#snapDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23snapDistance Online Documentation}*/
snapDistance: number;
/**
* Gets or sets the type of the grid snapping that should be performed.
* Determines which grid elements an object should snap to.
*
* Enabling grid snapping
*
* <pre>
* const snapContext = new GraphSnapContext()
*
* const gridInfo = new GridInfo(50)
* snapContext.nodeGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.bendGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.portGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.gridSnapType = GridSnapTypes.GRID_POINTS
*
* graphEditorInputMode.snapContext = snapContext
* </pre>
*
* Note that the grid visualization has to be enabled separately
*
* <pre>
* graphComponent.backgroundGroup.addChild(new GridVisualCreator(gridInfo))
* </pre>
*
* @see GraphSnapContext#nodeGridConstraintProvider
* @see GraphSnapContext#bendGridConstraintProvider
* @see GraphSnapContext#portGridConstraintProvider
* @default GridSnapTypes.NONE
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23gridSnapType Online Documentation}
*/
gridSnapType: GridSnapTypes;
/**
Gets or sets the GridSnapDistance.
The maximum distance between the current mouse coordinates and the coordinates to which the mouse will snap. The
distance is interpreted in view coordinates.
@default 10.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23gridSnapDistance Online Documentation}*/
gridSnapDistance: number;
/**
Gets the original location of the mouse at the time the gesture was initialized.
@see SnapContext#initializeDrag
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23originalLocation Online Documentation}*/
originalLocation: Point;
/**
Gets or sets a value indicating whether this {@link SnapContext} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapContext%23enabled Online Documentation}*/
enabled: boolean;
static $class: Class;
private $_SnapContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SnapContext;
}
export interface SnapResult extends YObject, IComparable {
}
/**
This is the abstract base class used by the implementers of {@link SnapContext} to model the result of the mouse
being snapped to a certain location.
It carries a {@link SnapResult#weight} and can be used to obtain a {@link IVisualCreator} that will be included in the view if the result is
actually {@link SnapResult#isSnapped snapped} after all other results with higher weight have been {@link SnapResult#snap snapped}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult Online Documentation}*/
export abstract class SnapResult {
/**
Factory method that creates a {@link SnapResult} to which a {@link ILabel} will snap, based on the given
<code>snapLine</code>.
<b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
@param layoutParameter The {@link ILabelModelParameter} which represents the position to snap to.
@param weight The weight of this result. The higher the weight, the more important it is.
@param delta The value to add to the mouse coordinates to snap to this result.
@param tag The tag associated with this result. If more than one result uses the same tag (not <code>null</code>), only the one with the highest
{@link SnapResult#weight} will be rendered.
@param snapLine The snap line this class would snap to or <code>null</code> if it doesn't snap to a <code>SnapLine</code>.
@param snappedCoordinates The coordinates at the moved item at which the drawn snap line should end.
@param movedObject The moved object for which this result is created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23createLabelSnapResult(ILabelModelParameter%2Cnumber%2CPoint%2CObject%2CSnapLine%2CPoint%2CObject) Online Documentation}*/
static createLabelSnapResult(layoutParameter: ILabelModelParameter, weight: number, delta: Point | PointConvertible, tag: any | null, snapLine: SnapLine | null, snappedCoordinates: Point | PointConvertible, movedObject: any | null): SnapResult;
/**
Factory method that creates a {@link SnapResult} that snaps to a given location.
<b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
@param location The location to snap to.
@param weight The weight to assign to the result.
@param tag The tag associated with this result. If more than one result uses the same tag (not <code>null</code>), only the one with the highest
{@link SnapResult#weight} will be rendered.
@param visualCreator The visual creator to use.
@param snapType The type of the snapping operation to apply.
@returns A result that snaps to the provided location using the provided mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23createPointSnapResult(Point%2Cnumber%2CObject%2CIVisualCreator%2CSnapTypes) Online Documentation}*/
static createPointSnapResult(location: Point | PointConvertible, weight: number, tag: any | null, visualCreator: IVisualCreator, snapType: SnapTypes | SnapTypesConvertible): SnapResult;
/**
Factory method that creates a {@link SnapResult} which snaps to a given size through a resize operation.
<b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
@param weight The weight of this result. The higher the weight, the more important it is.
@param delta The value to add to the mouse coordinates to snap to this result.
@param tag The tag associated with this result. If more than one result uses the same tag (not <code>null</code>), only the one with the highest
{@link SnapResult#weight} will be rendered.
@param rectangles The rectangles that have the same width or height as the node will. The rectangle are used for visualization and should
include all objects for which a visualization should be displayed. Typically, these are all objects which have the same
size and the object which is currently reshaped.
@param size The target size to which the resized object will snap.
@param horizontal Indicates whether width or height should snap. <code>true</code> if the width should snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23createResizeSnapResult(number%2CPoint%2CObject%2CIEnumerable%3CIRectangle%3E%2Cnumber%2Cboolean) Online Documentation}*/
static createResizeSnapResult(weight: number, delta: Point | PointConvertible, tag: any | null, rectangles: IEnumerable<IRectangle> | IEnumerableConvertible<IRectangle>, size: number, horizontal: boolean): SnapResult;
/**
Factory method that creates a {@link SnapResult} which represents a {@link SnapLine} to which the current
moved item will potentially snap.
<b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
@param weight The weight of this result. The higher the weight, the more important it is.
@param delta The value to add to the mouse coordinates to snap to this result.
@param tag The tag associated with this result. If more than one result uses the same tag (not <code>null</code>), only the one with the highest
{@link SnapResult#weight} will be rendered.
@param snapLine The snap line this class would snap to or <code>null</code> if it doesn't snap to a <code>SnapLine</code>.
@param snappedLocation The coordinates at the moved item at which the drawn snap line should end.
@param movedObject The moved object for which this result is created.
@param [preferredSnapTypes=SnapTypes.SNAPPED_X_Y] Whether the cursor should preferably snap in horizontal or vertical direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23createSnapLineSnapResult(number%2CPoint%2CObject%2CSnapLine%2CPoint%2CObject%2CSnapTypes) Online Documentation}*/
static createSnapLineSnapResult(weight: number, delta: Point | PointConvertible, tag: any | null, snapLine: SnapLine | null, snappedLocation: Point | PointConvertible, movedObject: any | null, preferredSnapTypes?: SnapTypes | SnapTypesConvertible): SnapResult;
/**
Returns a {@link IVisualCreator} instance that can be used to create a {@link Visual} for this result.
@returns A visual creator which creates a {@link Visual} which depicts this snap result.
@see VoidVisualCreator#INSTANCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23getVisualCreator() Online Documentation}*/
abstract getVisualCreator(): IVisualCreator;
/**
Checks whether this instance is still snapped given the final mouse location.
This implementation simply returns <code>false</code>.
@param unsnappedLocation The unsnapped location.
@param finalSnapState The final snap state that has been used by the client.
@returns Whether the location are still snapped for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23isSnapped(Point%2CSnapState) Online Documentation}*/
isSnapped(unsnappedLocation: Point, finalSnapState: SnapState): boolean;
/**
Core method that performs the actual snapping.
This implementation does nothing.
@param unsnappedLocation The location prior to the snapping.
@param currentSnapState The currently snapped location and the state of the snapping. This instance can be modified by subclasses.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23snap(Point%2CSnapState) Online Documentation}*/
snap(unsnappedLocation: Point, currentSnapState: SnapState): void;
/**
Gets the weight of this result.
Higher weights represent more important results.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23weight Online Documentation}*/
weight: number;
/**
Gets the tag associated with this result.
If more than one result uses the same tag (not <code>null</code>), only the one with the highest {@link SnapResult#weight} will be rendered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23tag Online Documentation}*/
tag: any | null;
/**
Gets a {@link SnapResult} representing that there is no snapping of the x or y coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapResult%23NULL_RESULT Online Documentation}*/
static readonly NULL_RESULT: SnapResult;
static $class: Class;
private $_SnapResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SnapResult;
}
/**
A flaggable enumeration of the different types of snaps that is used by {@link SnapState} in {@link SnapResult}
implementations.
@see SnapResult#snap
@see SnapContext
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapTypes Online Documentation}*/
export enum SnapTypes {
/**
Indicates that the location has been snapped to its x value.
Constant for use in {@link SnapResult#snap} and {@link SnapResult#isSnapped}. The methods can take a bitwise
combination of this field and {@link SnapTypes#SNAPPED_Y}, {@link SnapTypes#SNAPPED_X_Y} and
{@link SnapTypes#NOT_SNAPPED}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapTypes%23SNAPPED_X Online Documentation}*/
SNAPPED_X,
/**
Indicates that the location has been snapped to its y value.
Constant for use in {@link SnapResult#snap} and {@link SnapResult#isSnapped}. The methods can take a bitwise
combination of this field and {@link SnapTypes#SNAPPED_X}, {@link SnapTypes#SNAPPED_X_Y} and
{@link SnapTypes#NOT_SNAPPED}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapTypes%23SNAPPED_Y Online Documentation}*/
SNAPPED_Y,
/**
Indicates that the location has been snapped to its x and y values.
Constant for use in {@link SnapResult#snap} and {@link SnapResult#isSnapped}. This is a bitwise combination of
{@link SnapTypes#SNAPPED_X} and {@link SnapTypes#SNAPPED_Y}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapTypes%23SNAPPED_X_Y Online Documentation}*/
SNAPPED_X_Y,
/**
Indicates that the location has not been snapped at all.
Constant for use in {@link SnapResult#snap} and {@link SnapResult#isSnapped}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapTypes%23NOT_SNAPPED Online Documentation}*/
NOT_SNAPPED
}
type SnapTypesStringValues = "snapped-x" | "snapped-y" | "snapped-x-y" | "not-snapped";
/** Automatic conversion to {@link SnapTypes}. */
type SnapTypesConvertible = SnapTypesStringValues | SnapTypesStringValues[];
export interface SnapState extends YObject {
}
/**
Describes the state of a snapping operation.
This class is used by {@link SnapContext} to {@link SnapContext#processSnapResults process} {@link SnapResult#snap snap results}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapState Online Documentation}*/
export class SnapState {
/**
Initializes a new instance of the {@link SnapState} class.
@param location The initial location.
@param [snapType=SnapTypes.NOT_SNAPPED] The current type of the snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapState Online Documentation}*/
constructor(location: Point | PointConvertible, snapType?: SnapTypes | SnapTypesConvertible);
/**
Sets the {@link SnapState#location} to the given point and adjusts the {@link SnapState#snapType} accordingly.
@param newLocation The new location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapState%23snapTo(Point) Online Documentation}*/
snapTo(newLocation: Point): void;
/**
Modifies the X property of the {@link SnapState#location} and adjusts the {@link SnapState#snapType} accordingly.
@param newX The new X.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapState%23snapX(number) Online Documentation}*/
snapX(newX: number): void;
/**
Modifies the Y property of the {@link SnapState#location} and adjusts the {@link SnapState#snapType} accordingly.
@param newY The new Y.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapState%23snapY(number) Online Documentation}*/
snapY(newY: number): void;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapState%23toString() Online Documentation}*/
toString(): string;
/**
Gets the current location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapState%23location Online Documentation}*/
readonly location: Point;
/**
Gets or sets the type of the snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapState%23snapType Online Documentation}*/
snapType: SnapTypes;
static $class: Class;
private $_SnapState: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SnapState;
}
export interface TapEventArgs extends InputModeEventArgs {
}
/**
Event arguments used by {@link TapInputMode#addTappedListener Tapped}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapEventArgs Online Documentation}*/
export class TapEventArgs {
/**
Initializes a new instance of the {@link TapEventArgs} class.
@param context The context in which the tap occurred.
@param location The location.
@param [tapCount=1] The number of taps associated with this event.
@param [originalEvent=null] The original event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapEventArgs Online Documentation}*/
constructor(context: IInputModeContext, location: Point | PointConvertible, tapCount?: number, originalEvent?: Event | null);
/**
Initializes a new instance of the {@link TapEventArgs} class.
@param options The parameters to pass.
@param options.context The context in which the tap occurred.
@param options.location The location.
@param [options.tapCount=1] The number of taps associated with this event.
@param [options.originalEvent=null] The original event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapEventArgs Online Documentation}*/
constructor(options: {
context: IInputModeContext;
location: Point | PointConvertible;
tapCount?: number;
originalEvent?: Event | null;
});
/**
Gets the location of the tap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapEventArgs%23location Online Documentation}*/
readonly location: Point;
/**
* Gets or sets a value indicating whether this {@link TapEventArgs} have been handled.
* Client code that is triggered by the event and does not want the event to become handled by other handles should set
* this property to <code>true</code> to stop propagation of the event.
*
* Note that this property needs to be set before the function synchronously returns. This means that setting the value in
* an <code>async</code> function after the first <code>await</code> or in an asynchronous callback, it cannot not be seen by the emitter of the
* event and the sender will consider the event as not being handled.
*
* <pre>
* // wrong:
* mode.addTappedListener(async (sender, args) => {
*   await something()
*   args.handled = true  // will be set *after* all other event listeners have been invoked, only
* })
*
* // correct:
* mode.addTappedListener(async (sender, args) => {
*   args.handled = true // will be set immediately and 'mode' will see the change
*   await something()
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TapEventArgs%23handled Online Documentation}
*/
handled: boolean;
/**
Gets the number of taps associated with this event.
This can be used to distinguish between single and multiple taps.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapEventArgs%23tapCount Online Documentation}*/
tapCount: number;
/**
Gets the last input event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapEventArgs%23originalEvent Online Documentation}*/
originalEvent: Event | null;
static $class: Class;
private $_TapEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TapEventArgs;
}
/**
Enumeration used by {@link TapHandlingPolicy}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapHandlingPolicy Online Documentation}*/
export enum TapHandlingPolicy {
/**
Using this mode, in case of a double tap, only the double tap is reported, but neither of the two single taps.
Single taps will be reported with a short delay to ensure that they do not become double taps.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapHandlingPolicy%23DOUBLE_TAP_ONLY Online Documentation}*/
DOUBLE_TAP_ONLY,
/**
In this mode, in case of a double tap, only one single tap is reported and one double tap.
The second tap for the double tap is not reported as a single tap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapHandlingPolicy%23INITIAL_SINGLE_AND_DOUBLE_TAP Online Documentation}*/
INITIAL_SINGLE_AND_DOUBLE_TAP,
/**
In this mode, every tap (both single tap and either of the taps of a multi-tap) is reported as a single tap and every
double tap is reported as a double tap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapHandlingPolicy%23BOTH_SINGLE_TAPS_AND_DOUBLE_TAP Online Documentation}*/
BOTH_SINGLE_TAPS_AND_DOUBLE_TAP
}
/** Automatic conversion to {@link TapHandlingPolicy}. */
type TapHandlingPolicyStringValues = "double-tap-only" | "initial-single-and-double-tap" | "both-single-taps-and-double-tap";
export interface TapInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An input mode that recognizes simple touch taps.
* Clients register to {@link TapInputMode#addTappedListener Tapped} to get notified of touch taps.
* This mode can be instructed to {@link TapInputMode#swallowFocusTap swallow taps} if they lead happen shortly after the control gained focus. This is useful
* to prevent accidental taps from being processed if the user wanted to put the focus into the control, only.
*
* This mode is {@link TapInputMode#exclusive} by default.
*
* Typically the <code>TapInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#tapInputMode} property.
*
* <br><b>Getting the TapInputMode from its parent input mode</b>
* <pre>
* const tapInputMode = mode.tapInputMode
* </pre>
*
* @see TapInputMode#tapHandlingPolicy
* @see TapInputMode#swallowFocusTap
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode Online Documentation}
*/
export class TapInputMode {
/**
Creates a new instance of this mode that detects touch taps.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode Online Documentation}*/
constructor();
/**
Cancels the editing of this mode.
This implementation {@link ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
{@link TapInputMode#onCanceled}.
@see TapInputMode#onCanceled
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Creates the event recognizer that is used to recognize the press event for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23createPressRecognizer() Online Documentation}*/
createPressRecognizer(): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Creates the event recognizer that is used to recognize the release event for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23createReleaseRecognizer() Online Documentation}*/
createReleaseRecognizer(): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Creates the event recognizer that is used to recognize the taps for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23createTapRecognizer() Online Documentation}*/
createTapRecognizer(): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link ConcurrencyController} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Determines whether the given event is a press event that occurred at an invalid location.
@param eventSource
@param evt
@see TapInputMode#validTapHitTestable
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23isInvalidPress(Object%2CEventArgs) Online Documentation}*/
isInvalidPress(eventSource: any, evt: EventArgs): boolean;
/**
Determines whether the given event is a valid press event.
@param eventSource
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23isValidPress(Object%2CEventArgs) Online Documentation}*/
isValidPress(eventSource: any, evt: EventArgs): boolean;
/**
Determines whether the given event is a valid release event.
@param eventSource
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23isValidRelease(Object%2CEventArgs) Online Documentation}*/
isValidRelease(eventSource: any, evt: EventArgs): boolean;
/**
Determines whether the given event is a valid tap event.
@param eventSource
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23isValidTap(Object%2CEventArgs) Online Documentation}*/
isValidTap(eventSource: any, evt: EventArgs): boolean;
/**
Called after {@link TapInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Called once this mode has recognized a double-tap gesture.
This method will trigger the {@link TapInputMode#addDoubleTappedListener DoubleTapped} event.
@param evt The arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23onDoubleTapped(TapEventArgs) Online Documentation}*/
onDoubleTapped(evt: TapEventArgs): void;
/**
Called after {@link TapInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Called once this mode has recognized a tap gesture.
This method will trigger the {@link TapInputMode#addTappedListener Tapped} event.
@param evt The arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23onTapped(TapEventArgs) Online Documentation}*/
onTapped(evt: TapEventArgs): void;
/**
Prevents a double-tap event from being issued if the next tap would do so and sends only a single tap instead.
The intention is for "breaking" a double tap if an input mode using {@link TapInputMode} performs actions that
would lead to surprising behavior if the next tap would lead to a double-tap. Examples of this are
{@link GraphEditorInputMode}'s selection cycling where several taps in short succession may occur, as well as
creating a node by tapping on the canvas and selecting it immediately afterwards with another tap.
This method's effect is very short-lived. It really only prevents a double-tap event for the very next tap that this
input mode handles. The internal flag set by this method is re-set on every tap received. This also means that if you
call this method and the next tap is just a regular single-tap the flag is cleared nonetheless.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23preventNextDoubleTap() Online Documentation}*/
preventNextDoubleTap(): void;
/**
Returns and resets the flag set by {@link TapInputMode#preventNextDoubleTap}.
To ensure that the flag really acts just once, this method resets the flag and returns its value prior to the reset.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23queryAndResetPreventNextDoubleTap() Online Documentation}*/
queryAndResetPreventNextDoubleTap(): boolean;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link TapInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link TapInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the TapInputMode on its parent input mode</b>
* <pre>
* mode.tapInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Returns the installed {@link ConcurrencyController}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Adds the given listener for the <code>Tapped</code> event that occurs once a tap has been detected.
@param listener The listener to add.
@see TapEventArgs
@see TapInputMode#tapHandlingPolicy
@see TapInputMode#removeTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23Tapped Online Documentation}*/
addTappedListener(listener: (sender: any, evt: TapEventArgs) => void): void;
/**
Removes the given listener for the <code>Tapped</code> event that occurs once a tap has been detected.
@param listener The listener to remove.
@see TapEventArgs
@see TapInputMode#tapHandlingPolicy
@see TapInputMode#addTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23Tapped Online Documentation}*/
removeTappedListener(listener: (sender: any, evt: TapEventArgs) => void): void;
/**
Adds the given listener for the <code>DoubleTapped</code> event that occurs once a double-tap has been detected.
@param listener The listener to add.
@see TapEventArgs
@see TapInputMode#tapHandlingPolicy
@see TapInputMode#removeDoubleTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23DoubleTapped Online Documentation}*/
addDoubleTappedListener(listener: (sender: any, evt: TapEventArgs) => void): void;
/**
Removes the given listener for the <code>DoubleTapped</code> event that occurs once a double-tap has been detected.
@param listener The listener to remove.
@see TapEventArgs
@see TapInputMode#tapHandlingPolicy
@see TapInputMode#addDoubleTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23DoubleTapped Online Documentation}*/
removeDoubleTappedListener(listener: (sender: any, evt: TapEventArgs) => void): void;
/**
Gets or sets the tap handling policy that determines the triggering behavior of {@link TapInputMode#addTappedListener Tapped}
and {@link TapInputMode#addDoubleTappedListener DoubleTapped}.
@see TapInputMode#tapHandlingPolicy
@default TapHandlingPolicy.BOTH_SINGLE_TAPS_AND_DOUBLE_TAP
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23tapHandlingPolicy Online Documentation}*/
tapHandlingPolicy: TapHandlingPolicy;
/**
Gets or sets a hit test that determines where this mode should recognize taps.
This implementation is tested during the {@link TapInputMode#isValidPress} to determine whether it is valid to tap
here. The default implementation is {@link IHitTestable#ALWAYS}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23validTapHitTestable Online Documentation}*/
validTapHitTestable: IHitTestable;
/**
Whether to {@link ConcurrencyController#requestMutex request the mutex} on a tap.
If set to <code>true</code> this will discard other concurrent input modes on a tap. The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23requestMutexOnTap Online Documentation}*/
requestMutexOnTap: boolean;
/**
Gets or sets a property that determines whether taps should be swallowed if they happen within a short amount of time
after the {@link CanvasComponent} got focus.
The default is <code>false</code>. If set to <code>true</code> taps are discarded within 100 ms after the focus entered.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23swallowFocusTap Online Documentation}*/
swallowFocusTap: boolean;
/**
Gets the location of the last tap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TapInputMode%23tapLocation Online Documentation}*/
readonly tapLocation: Point;
static $class: Class;
private $_TapInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TapInputMode;
}
export interface TextEditorInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* A simple {@link IInputMode} that displays a <code>textarea</code> in the {@link CanvasComponent} to let the user edit a
* text.
* Hitting escape or enter will cancel or stop editing.
*
* This mode is {@link TextEditorInputMode#exclusive} by default.
*
* Typically the <code>TextEditorInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} and can be retrieved
* from the {@link GraphEditorInputMode#textEditorInputMode} property.
*
* <br><b>Getting the TextEditorInputMode from its parent input mode</b>
* <pre>
* const textEditorInputMode = mode.textEditorInputMode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode Online Documentation}
*/
export class TextEditorInputMode {
/**
Creates a new instance using a default text box or the provided editor.
@param [editorContainer=null] The editor to use as the text editor. If none is provided a default text editor will be used.
@see TextEditorInputMode#createEditorContainer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode Online Documentation}*/
constructor(editorContainer?: HTMLDivElement | null);
/**
Adjusts the position of the text box in the {@link CanvasComponent}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23adjustPosition() Online Documentation}*/
adjustPosition(): void;
/**
Adjusts the size of the text box in the {@link TextEditorInputMode#editorContainer}.
@param maxSize The suggested maximum size to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23adjustSize(Size) Online Documentation}*/
adjustSize(maxSize: Size): void;
/**
Calculates the maximum size for this instance.
The result is passed to {@link TextEditorInputMode#adjustSize}.
This implementation makes sure the textbox does not exceed the bounds of the canvas.
@returns The maximum size for the textbox.
@see TextEditorInputMode#adjustSize
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23calculateMaxTextBoxSize() Online Documentation}*/
calculateMaxTextBoxSize(): Size;
/**
Cancels editing of the text box.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Resets the contents of the text box.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23clear() Online Documentation}*/
clear(): void;
/**
Creates the default editor and it's container div element.
This is called when no editor was provided to the constructor and populates {@link TextEditorInputMode#editorContainer} lazily on demand.
If custom code overrides this factory method to provide a custom editor, the following methods might need adjustment,
too:
<ul>
<li>{@link TextEditorInputMode#editorText} - alternatively use the {@link TextEditorInputMode#addEditorTextChangedListener EditorTextChanged}
event.
<li>{@link TextEditorInputMode#adjustSize}
<li>{@link TextEditorInputMode#adjustPosition}
</ul>
@returns The div that will be placed in the {@link CanvasComponent} for editing the text.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23createEditorContainer() Online Documentation}*/
createEditorContainer(): HTMLDivElement;
/**
Starts interactive text editing.
@returns A Promise that will fulfill with the edited text or fulfill with <code>null</code> if editing is cancelled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23edit() Online Documentation}*/
edit(): Promise<string>;
/**
Adjusts the {@link CanvasComponent#viewport} of the {@link CanvasComponent} such that the text box is in the visible area.
Called once when the editing starts.
@see TextEditorInputMode#textBoxPlacementPolicy
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23ensureVisible() Online Documentation}*/
ensureVisible(): void;
/**
Returns the bounds of the text area in world coordinates.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23getTextBoxBounds() Online Documentation}*/
getTextBoxBounds(): Rect;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link TextEditorInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Installs the {@link TextEditorInputMode#editorContainer} in the canvas and puts focus into it.
The default implementation also selects all text in default text box and sets the caret to the end of the text.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23installTextBox() Online Documentation}*/
installTextBox(): void;
/**
Called when editing starts.
Requests the input mutex, and installs the text box.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onBeginEditing() Online Documentation}*/
onBeginEditing(): void;
/**
Called after {@link TextEditorInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called when editing gets canceled.
Removes the text box and releases the input mutex.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onCancelEditing() Online Documentation}*/
onCancelEditing(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Raises the {@link TextEditorInputMode#addEditingCanceledListener EditingCanceled} event.
@param evt The event argument
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onEditingCanceled(TextEventArgs) Online Documentation}*/
onEditingCanceled(evt: TextEventArgs): void;
/**
Raises the {@link TextEditorInputMode#addEditingStartedListener EditingStarted} event.
@param evt The event argument.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onEditingStarted(TextEventArgs) Online Documentation}*/
onEditingStarted(evt: TextEventArgs): void;
/**
Called when editing is stopped.
Triggers the {@link TextEditorInputMode#addTextEditedListener TextEdited} event, removes the
box and releases the input mutex.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onStopEditing() Online Documentation}*/
onStopEditing(): void;
/**
Called after {@link TextEditorInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Raises the {@link TextEditorInputMode#addTextEditedListener TextEdited} event.
@param evt The event arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23onTextEdited(TextEventArgs) Online Documentation}*/
onTextEdited(evt: TextEventArgs): void;
/**
Stops editing and returns <code>true</code>.
@returns <code>base.Stop()</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23tryStop() Online Documentation}*/
tryStop(): boolean;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Removes the text box.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23uninstallTextBox() Online Documentation}*/
uninstallTextBox(): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link TextEditorInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link TextEditorInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the TextEditorInputMode on its parent input mode</b>
* <pre>
* mode.textEditorInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link TextEditorInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
/**
Adds the given listener for the <code>TextEdited</code> event that occurs once the text has been edited.
@param listener The listener to add.
@see TextEditorInputMode#removeTextEditedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23TextEdited Online Documentation}*/
addTextEditedListener(listener: (sender: any, evt: TextEventArgs) => void): void;
/**
Removes the given listener for the <code>TextEdited</code> event that occurs once the text has been edited.
@param listener The listener to remove.
@see TextEditorInputMode#addTextEditedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23TextEdited Online Documentation}*/
removeTextEditedListener(listener: (sender: any, evt: TextEventArgs) => void): void;
/**
Adds the given listener for the <code>EditingCanceled</code> event that occurs if the editing has not been finished.
@param listener The listener to add.
@see TextEditorInputMode#removeEditingCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23EditingCanceled Online Documentation}*/
addEditingCanceledListener(listener: (sender: any, evt: TextEventArgs) => void): void;
/**
Removes the given listener for the <code>EditingCanceled</code> event that occurs if the editing has not been finished.
@param listener The listener to remove.
@see TextEditorInputMode#addEditingCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23EditingCanceled Online Documentation}*/
removeEditingCanceledListener(listener: (sender: any, evt: TextEventArgs) => void): void;
/**
Adds the given listener for the <code>EditingStarted</code> event that occurs when text editing is started.
@param listener The listener to add.
@see TextEditorInputMode#removeEditingStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23EditingStarted Online Documentation}*/
addEditingStartedListener(listener: (sender: any, evt: TextEventArgs) => void): void;
/**
Removes the given listener for the <code>EditingStarted</code> event that occurs when text editing is started.
@param listener The listener to remove.
@see TextEditorInputMode#addEditingStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23EditingStarted Online Documentation}*/
removeEditingStartedListener(listener: (sender: any, evt: TextEventArgs) => void): void;
/**
Gets or sets a property that determines whether the text edited should be committed if the focus gets lost or the
editing should be canceled.
The default behavior is to cancel the editing process.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23autoCommitOnFocusLost Online Documentation}*/
autoCommitOnFocusLost: boolean;
/**
Gets or sets a property that determines whether this mode should try to {@link CanvasComponent#focus} the {@link GraphComponent}
once the editor is closed via a keyboard gesture.
The default is <code>true</code>
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23autoFocusCanvasOnEditorClosed Online Documentation}*/
autoFocusCanvasOnEditorClosed: boolean;
/**
Gets or sets whether this mode should automatically fire up the editor once it gets installed.
If this mode is permanently installed, set this flag to <code>false</code> and set the {@link TextEditorInputMode#editing} flag to <code>true</code> in order to start editing.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23autoStartEditing Online Documentation}*/
autoStartEditing: boolean;
/**
Gets or sets the event recognizer to detect the key gesture for new line breaks.
When the provided {@link KeyEventArgs} are recognized, a new line break is inserted at the current caret position.
@see TextEditorInputMode#cancelRecognizer
@see TextEditorInputMode#stopRecognizer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23lineBreakRecognizer Online Documentation}*/
lineBreakRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer to detect when editing the text shall be canceled.
When the provided {@link KeyEventArgs} are recognized, {@link TextEditorInputMode#cancel} is called.
@see TextEditorInputMode#lineBreakRecognizer
@see TextEditorInputMode#stopRecognizer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23cancelRecognizer Online Documentation}*/
cancelRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer to detect when editing the text shall be stopped.
When the provided {@link KeyEventArgs} are recognized, {@link TextEditorInputMode#tryStop} is called.
@see TextEditorInputMode#lineBreakRecognizer
@see TextEditorInputMode#cancelRecognizer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23stopRecognizer Online Documentation}*/
stopRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the current editing state of the editor.
Setting a new value will trigger the {@link TextEditorInputMode#onBeginEditing} and {@link TextEditorInputMode#onCancelEditing}
methods respectively.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23editing Online Documentation}*/
editing: boolean;
/**
* Gets or sets the location of the text box in world coordinates.
* If this mode is used for label editing as a sub mode of {@link GraphEditorInputMode} the value of this property is
* overridden by a default value. It can be set, though, using the {@link GraphEditorInputMode#textEditorInputModeConfigurator}.
*
* <pre>
* mode.textEditorInputModeConfigurator = (context, inputMode, label) => {
*   // anchor the text editor centered above the label
*   inputMode.anchor = new Point(0.5, 1)
*   // always place the editor horizontal
*   inputMode.upVector = new Point(0, -1)
*   // place the editor relative to the label's center
*   inputMode.location = label.layout.orientedRectangleCenter
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23location Online Documentation}
*/
location: IPoint;
/**
* Gets or sets the point that describes the "up"-vector that will be used to determine the orientation of the text box in
* the world coordinate system.
* The default is <code>0, -1</code>.
*
* If this mode is used for label editing as a sub mode of {@link GraphEditorInputMode} the value of this property is
* overridden by a default value. It can be set, though, using the {@link GraphEditorInputMode#textEditorInputModeConfigurator}.
*
* <pre>
* mode.textEditorInputModeConfigurator = (context, inputMode, label) => {
*   // anchor the text editor centered above the label
*   inputMode.anchor = new Point(0.5, 1)
*   // always place the editor horizontal
*   inputMode.upVector = new Point(0, -1)
*   // place the editor relative to the label's center
*   inputMode.location = label.layout.orientedRectangleCenter
* }
* </pre>
*
* @see TextEditorInputMode#rotateTextBox
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23upVector Online Documentation}
*/
upVector: IPoint;
/**
* Gets or sets the anchor of the text box.
* The anchor is the point inside the coordinate system of the text box that shall coincide with the {@link TextEditorInputMode#location} in the world
* coordinate system. The X and Y values are expressed as relative width/height ratios.
*
* If this mode is used for label editing as a sub mode of {@link GraphEditorInputMode} the value of this property is
* overridden by a default value. It can be set, though, using the {@link GraphEditorInputMode#textEditorInputModeConfigurator}.
*
* <pre>
* mode.textEditorInputModeConfigurator = (context, inputMode, label) => {
*   // anchor the text editor centered above the label
*   inputMode.anchor = new Point(0.5, 1)
*   // always place the editor horizontal
*   inputMode.upVector = new Point(0, -1)
*   // place the editor relative to the label's center
*   inputMode.location = label.layout.orientedRectangleCenter
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23anchor Online Documentation}
*/
anchor: IPoint;
/**
Gets the <code>div</code>-element that contains the text editor.
@see TextEditorInputMode#createEditorContainer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23editorContainer Online Documentation}*/
readonly editorContainer: HTMLDivElement;
/**
Gets or sets the text that is displayed and edited.
If customizations change the way the text is edited by providing a custom editor, this property can be overridden to
return the current text contents and set the contents accordingly. Alternatively the {@link TextEditorInputMode#addEditorTextChangedListener EditorTextChanged}
event can be used for this task.
@returns The textual contents of the editor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23editorText Online Documentation}*/
editorText: string;
/**
Adds the given listener for the <code>EditorTextChanged</code> event that occurs when the {@link TextEditorInputMode#editorText editor's text} should be changed.
This event can be used by custom code that uses its own editor component. Listen to this event to update the text in the
editor. Write changes in text back to the {@link TextEditorInputMode#editorText} property.
@param listener The listener to add.
@see TextEditorInputMode#removeEditorTextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23EditorTextChanged Online Documentation}*/
addEditorTextChangedListener(listener: (sender: any, evt: TextEventArgs) => void): void;
/**
Removes the given listener for the <code>EditorTextChanged</code> event that occurs when the {@link TextEditorInputMode#editorText editor's text} should be changed.
This event can be used by custom code that uses its own editor component. Listen to this event to update the text in the
editor. Write changes in text back to the {@link TextEditorInputMode#editorText} property.
@param listener The listener to remove.
@see TextEditorInputMode#addEditorTextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23EditorTextChanged Online Documentation}*/
removeEditorTextChangedListener(listener: (sender: any, evt: TextEventArgs) => void): void;
/**
Gets or sets a value indicating whether to automatically flip the orientation of the text box if the {@link TextEditorInputMode#upVector} actually points
downwards.
The default is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23autoFlipTextBox Online Documentation}*/
autoFlipTextBox: boolean;
/**
Gets or sets a value indiciating whether the text box should be rotated with the {@link TextEditorInputMode#upVector}.
If the value is <code>false</code> (the default), the text box will always be horizontal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23rotateTextBox Online Documentation}*/
rotateTextBox: boolean;
/**
Gets or sets the {@link TextEditorInputMode#textBoxPlacementPolicy} to use when the {@link TextEditorInputMode#editorContainer} isn't completely visible in the current {@link CanvasComponent#viewport}.
Default is TextBoxPlacementPolicy.ScrollCanvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23textBoxPlacementPolicy Online Documentation}*/
textBoxPlacementPolicy: TextBoxPlacementPolicy;
/**
Gets or sets the padding to use around the {@link TextEditorInputMode#editorContainer} when it comes near to the viewport borders and needs to be adjusted.
This is a value in view coordinates. Default is 20 on each side.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEditorInputMode%23textBoxPadding Online Documentation}*/
textBoxPadding: Insets;
static $class: Class;
private $_TextEditorInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TextEditorInputMode;
}
/**
Describe what policy to use when the {@link TextEditorInputMode#editorContainer} would be located outside of the current {@link CanvasComponent#viewport}
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextBoxPlacementPolicy Online Documentation}*/
export enum TextBoxPlacementPolicy {
/**
Do nothing
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextBoxPlacementPolicy%23NONE Online Documentation}*/
NONE,
/**
Move the {@link CanvasComponent} so that the {@link TextEditorInputMode#editorContainer} is visible
If the Canvas cannot be sufficiently scrolled due to limits imposed by the
{@link ViewportLimiter}, the text box is moved instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextBoxPlacementPolicy%23SCROLL_CANVAS Online Documentation}*/
SCROLL_CANVAS,
/**
Move the {@link TextEditorInputMode#editorContainer} so that it is visible in the current {@link CanvasComponent#viewport}
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextBoxPlacementPolicy%23MOVE_TEXT_BOX Online Documentation}*/
MOVE_TEXT_BOX
}
/** Automatic conversion to {@link TextBoxPlacementPolicy}. */
type TextBoxPlacementPolicyStringValues = "none" | "scroll-canvas" | "move-text-box";
export interface TextEventArgs extends InputModeEventArgs {
}
/**
Event arguments used by {@link TextEditorInputMode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEventArgs Online Documentation}*/
export class TextEventArgs {
/**
Creates a new instance.
@param context The context in which this text event happened.
@param text The text that was or will be edited.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEventArgs Online Documentation}*/
constructor(context: IInputModeContext, text: string);
/**
Gets the text that was or will be edited.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextEventArgs%23text Online Documentation}*/
text: string;
static $class: Class;
private $_TextEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TextEventArgs;
}
export interface WaitInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
cancel(): void;
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} implementation that can be used to block user interaction.
* Setting the {@link WaitInputMode#waiting} property will try to cancel ongoing edits of the {@link ConcurrencyController#mutexOwner mutex owner} and set the preferred cursor to {@link WaitInputMode#waitCursor}. This mode
* will try to add itself to the {@link CanvasComponent#lookup Lookup of the CanvasControl}, so that other clients can make use of its functionality.
*
* This mode uses an {@link ConcurrencyController#exclusive} controller by default.
*
* Typically the <code>WaitInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#waitInputMode} property.
*
* <br><b>Getting the WaitInputMode from its parent input mode</b>
* <pre>
* const waitInputMode = mode.waitInputMode
* </pre>
*
* <br><b>Blocking user interaction during an asynchronous process</b>
* <pre>
* graphEditorInputMode.waiting = true
* longRunningAsyncProcess().then(() => {
*   graphEditorInputMode.waiting = false
* })
* </pre>
*
* @see WaitInputMode#waiting
* @see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode Online Documentation}
*/
export class WaitInputMode {
/**
Creates a new instance of this mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode Online Documentation}*/
constructor();
/**
Returns if this mode can request the mutex when {@link WaitInputMode#waiting} is set to <code>true</code>.
@returns if this mode can request the mutex when {@link WaitInputMode#waiting} is set to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23canStartWaiting() Online Documentation}*/
canStartWaiting(): boolean;
/**
Called when waiting is finished.
This will reset the {@link ConcurrencyController#preferredCursor} and {@link ConcurrencyController#releaseMutex the mutex is released.}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23endWaiting() Online Documentation}*/
endWaiting(): void;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link WaitInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called after {@link WaitInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Called after {@link WaitInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Raises the {@link WaitInputMode#addWaitingEndedListener WaitingEnded} event.
@param evt The {@link InputModeEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23onWaitingEnded(InputModeEventArgs) Online Documentation}*/
onWaitingEnded(evt: InputModeEventArgs): void;
/**
Raises the {@link WaitInputMode#addWaitingStartedListener WaitingStarted} event.
@param evt The {@link InputModeEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23onWaitingStarted(InputModeEventArgs) Online Documentation}*/
onWaitingStarted(evt: InputModeEventArgs): void;
/**
This will initiate the waiting process by trying to
{@link ConcurrencyController#requestMutex}request the input mutex.
This will set the {@link ConcurrencyController#preferredCursor} to {@link WaitInputMode#waitCursor}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23startWaiting() Online Documentation}*/
startWaiting(): void;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link WaitInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the WaitInputMode on its parent input mode</b>
* <pre>
* mode.waitInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link WaitInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
/**
* Gets or sets the Waiting property.
* Setting this property to <code>true</code> will {@link WaitInputMode#startWaiting start the waiting process.} Setting it to <code>false</code> will {@link WaitInputMode#endWaiting end the waiting}.
*
* <br><b>Blocking user interaction during an asynchronous process</b>
* <pre>
* const waitInputMode = graphEditorInputMode.waitInputMode
* waitInputMode.waiting = true
* longRunningAsyncProcess().then(() => {
*   waitInputMode.waiting = false
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23waiting Online Documentation}
*/
waiting: boolean;
/**
Adds the given listener for the <code>WaitingStarted</code> event that occurs when the waiting started.
@param listener The listener to add.
@see WaitInputMode#removeWaitingStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23WaitingStarted Online Documentation}*/
addWaitingStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>WaitingStarted</code> event that occurs when the waiting started.
@param listener The listener to remove.
@see WaitInputMode#addWaitingStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23WaitingStarted Online Documentation}*/
removeWaitingStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>WaitingEnded</code> event that occurs when the waiting ended.
@param listener The listener to add.
@see WaitInputMode#removeWaitingEndedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23WaitingEnded Online Documentation}*/
addWaitingEndedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>WaitingEnded</code> event that occurs when the waiting ended.
@param listener The listener to remove.
@see WaitInputMode#addWaitingEndedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23WaitingEnded Online Documentation}*/
removeWaitingEndedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Gets or sets the WaitCursor property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WaitInputMode%23waitCursor Online Documentation}*/
waitCursor: Cursor | null;
static $class: Class;
private $_WaitInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WaitInputMode;
}
/**
Specifies constants that define the the style how bridges are drawn by {@link BridgeManager}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingStyle Online Documentation}*/
export enum BridgeCrossingStyle {
/**
A simple gap in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingStyle%23GAP Online Documentation}*/
GAP,
/**
A rectangular bridge will be inserted in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingStyle%23RECTANGLE Online Documentation}*/
RECTANGLE,
/**
A rectangular bridge with a fixed aspect ratio will be inserted in the path.
This bridge will be scaled for wider bridges so that its aspect ratio remains constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingStyle%23RECTANGLE_SCALED Online Documentation}*/
RECTANGLE_SCALED,
/**
A triangular bridge will be inserted in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingStyle%23TWO_SIDES Online Documentation}*/
TWO_SIDES,
/**
A triangular bridge with a fixed aspect ratio will be inserted in the path.
This bridge will be scaled for wider bridges so that its aspect ratio remains constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingStyle%23TWO_SIDES_SCALED Online Documentation}*/
TWO_SIDES_SCALED,
/**
A bridge, consisting of two quarter circle arcs connected by a straight line, will be inserted in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingStyle%23ARC Online Documentation}*/
ARC,
/**
A semi-circle bridge will be inserted in the path.
This bridge will be scaled for wider bridges so that its aspect ratio remains constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingStyle%23ARC_SCALED Online Documentation}*/
ARC_SCALED,
/**
Inserts a custom bridge into the path.
Custom bridges require an appropriate implementation of the {@link IBridgeCreator} interface and {@link IBridgeCreator#createCustomBridge}
method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingStyle%23CUSTOM Online Documentation}*/
CUSTOM
}
/** Automatic conversion to {@link BridgeCrossingStyle}. */
type BridgeCrossingStyleStringValues = "gap" | "rectangle" | "rectangle-scaled" | "two-sides" | "two-sides-scaled" | "arc" | "arc-scaled" | "custom";
/**
Specifies constants that define the direction in which bridges are drawn.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeOrientationStyle Online Documentation}*/
export enum BridgeOrientationStyle {
/**
The bridge is drawn to the right of the flow of the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeOrientationStyle%23FLOW_RIGHT Online Documentation}*/
FLOW_RIGHT,
/**
The bridge is drawn to the left of the flow of the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeOrientationStyle%23FLOW_LEFT Online Documentation}*/
FLOW_LEFT,
/**
The bridge is drawn in the direction of the positive axes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeOrientationStyle%23POSITIVE Online Documentation}*/
POSITIVE,
/**
The bridge is drawn in the direction of the negative axes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeOrientationStyle%23NEGATIVE Online Documentation}*/
NEGATIVE,
/**
The bridge points to the right.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeOrientationStyle%23LEFT Online Documentation}*/
LEFT,
/**
The bridge points to the left.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeOrientationStyle%23RIGHT Online Documentation}*/
RIGHT,
/**
The bridge points upwards.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeOrientationStyle%23UP Online Documentation}*/
UP,
/**
The bridge points downwards.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeOrientationStyle%23DOWN Online Documentation}*/
DOWN
}
/** Automatic conversion to {@link BridgeOrientationStyle}. */
type BridgeOrientationStyleStringValues = "flow-right" | "flow-left" | "positive" | "negative" | "left" | "right" | "up" | "down";
/**
Enumeration used by {@link BridgeManager} that determines how crossings between obstacles and paths are determined.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingPolicy Online Documentation}*/
export enum BridgeCrossingPolicy {
/**
Makes horizontal segments bridge over vertical obstacles.
Horizontal segments do not cross each other, just as vertical ones will not cross each other. Horizontal segments are
segments with a slope between 1 and -1. All others are vertical segments. For non-parallel intersecting pairs of
segments there does not need to be a crossing unless only one of them is vertical.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingPolicy%23HORIZONTAL_BRIDGES_VERTICAL Online Documentation}*/
HORIZONTAL_BRIDGES_VERTICAL,
/**
Makes vertical segments bridge over horizontal obstacles.
Horizontal segments do not cross each other, just as vertical ones will not cross each other. Horizontal segments are
segments with a slope between 1 and -1. All others are vertical segments. For non-parallel intersecting pairs of
segments there does not need to be a crossing unless only one of them is horizontal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingPolicy%23VERTICAL_BRIDGES_HORIZONTAL Online Documentation}*/
VERTICAL_BRIDGES_HORIZONTAL,
/**
Makes more horizontal edges bridge over edges which are less horizontal.
The smaller the absolute slope of an edge, the more horizontal it is. For each non-parallel intersecting pair of
segments, there will be a crossing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingPolicy%23MORE_HORIZONTAL_BRIDGES_LESS_HORIZONTAL Online Documentation}*/
MORE_HORIZONTAL_BRIDGES_LESS_HORIZONTAL,
/**
Makes more vertical edges bridge over edges which are less vertical.
The larger the absolute slope of an edge, the more vertical it is. For each non-parallel intersecting pair of segments,
there will be a crossing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeCrossingPolicy%23MORE_VERTICAL_BRIDGES_LESS_VERTICAL Online Documentation}*/
MORE_VERTICAL_BRIDGES_LESS_VERTICAL
}
/** Automatic conversion to {@link BridgeCrossingPolicy}. */
type BridgeCrossingPolicyStringValues = "horizontal-bridges-vertical" | "vertical-bridges-horizontal" | "more-horizontal-bridges-less-horizontal" | "more-vertical-bridges-less-vertical";
export interface BridgeManager extends YObject {
}
/**
* Helper class that calculates visual hints in a path where said path crosses an obstacle.
* Most notably this feature is used to render crossings of {@link IEdge}s with other edges.
*
* To set up bridges in a {@link GraphComponent} at the least the following is necessary:
*
* <br><b>Creating a BridgeManager for a GraphControl</b>
* <pre>
* // Create a BridgeManager and bind it to the GraphComponent
* const bridgeManager = new BridgeManager()
* bridgeManager.canvasComponent = graphComponent
* // Add an obstacle provider that queries graph items (notably edges)
* // for obstacles which can then be bridged
* bridgeManager.addObstacleProvider(new GraphObstacleProvider())
* </pre>
*
* Furthermore, an edge style that supports bridges is necessary; {@link PolylineEdgeStyle} is a good choice. Various
* other properties can be set to change the defaults, e.g. {@link BridgeCrossingStyle how bridges appear}, or {@link BridgeOrientationStyle which way bridges point}.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager Online Documentation}
*/
export class BridgeManager {
/**
Initializes a new instance of the {@link BridgeManager} class.
Initially the {@link BridgeManager#defaultBridgeCreator} will be set to an instance that will delegate to the set of <code>Default</code> properties in this class and the {@link BridgeManager#insertDefaultCustomBridge}
method respectively.
@param [options] The parameters to pass.
@param [options.defaultBridgeCreator] The default implementation of the {@link IBridgeCreator} that will be used if {@link BridgeManager#addBridges} is invoked with a <code>null</code> callback argument. This option sets the {@link BridgeManager#defaultBridgeCreator} property on the created object.
@param [options.considerCurves] A value that determines whether curves ({@link GeneralPath#cubicTo cubic} and {@link GeneralPath#quadTo quadratic}) should be considered as obstacles. This option sets the {@link BridgeManager#considerCurves} property on the created object.
@param [options.zoomThreshold] A threshold value that determines below which zoom level, there should be no more bridge calculation. This option sets the {@link BridgeManager#zoomThreshold} property on the created object.
@param [options.bridgeCrossingPolicy] The {@link BridgeCrossingPolicy} mode that is used by this instance. This option sets the {@link BridgeManager#bridgeCrossingPolicy} property on the created object.
@param [options.canvasComponent] The {@link BridgeManager#canvasComponent} that should be managed by this instance. This option sets the {@link BridgeManager#canvasComponent} property on the created object.
@param [options.defaultBridgeCrossingStyle] The default {@link BridgeCrossingStyle} that will be used by the default implementation of the {@link IBridgeCreator} that is initially assigned to the {@link BridgeManager#defaultBridgeCreator} property. This option sets the {@link BridgeManager#defaultBridgeCrossingStyle} property on the created object.
@param [options.defaultBridgeWidth] The default width of a bridge that will be used by the default implementation of the {@link IBridgeCreator} that is initially assigned to the {@link BridgeManager#defaultBridgeCreator} property. This option sets the {@link BridgeManager#defaultBridgeWidth} property on the created object.
@param [options.clipMargin] The margin by which the {@link IRenderContext#clip} should be enlarged to accommodate for clipped away lines being taken into account for bridge calculation. This option sets the {@link BridgeManager#clipMargin} property on the created object.
@param [options.defaultBridgeHeight] The default height of a bridge that will be used by the default implementation of the {@link IBridgeCreator} that is initially assigned to the {@link BridgeManager#defaultBridgeCreator} property. This option sets the {@link BridgeManager#defaultBridgeHeight} property on the created object.
@param [options.defaultBridgeOrientationStyle] The default {@link BridgeOrientationStyle} for bridges that will be used by the default implementation of the {@link IBridgeCreator} that is initially assigned to the {@link BridgeManager#defaultBridgeCreator} property. This option sets the {@link BridgeManager#defaultBridgeOrientationStyle} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager Online Documentation}*/
constructor(options?: {
defaultBridgeCreator?: IBridgeCreator;
considerCurves?: boolean;
zoomThreshold?: number;
bridgeCrossingPolicy?: BridgeCrossingPolicy | BridgeCrossingPolicyStringValues;
canvasComponent?: CanvasComponent | null;
defaultBridgeCrossingStyle?: BridgeCrossingStyle | BridgeCrossingStyleStringValues;
defaultBridgeWidth?: number;
clipMargin?: number;
defaultBridgeHeight?: number;
defaultBridgeOrientationStyle?: BridgeOrientationStyle | BridgeOrientationStyleStringValues;
});
/**
Core method that will add bridges to a {@link GeneralPath} using the {@link IBridgeCreator} callback to
determine the type of the bridges.
This method may return <code>path</code> itself if no crossings have been determined. Otherwise it will return a new path instance
that has the bridges added at the corresponding places.
@param context The context to use.
@param path The path to calculate the bridged variant for.
@param [callback=null] The callback or <code>null</code> that determines the appearance of the bridges.
@returns The path that might have been augmented by the bridges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23addBridges(IRenderContext%2CGeneralPath%2CIBridgeCreator) Online Documentation}*/
addBridges(context: IRenderContext, path: GeneralPath, callback?: IBridgeCreator | null): GeneralPath;
/**
Adds another {@link IObstacleProvider} to query for obstacles.
The {@link IObstacleProvider#getObstacles} method will be invoked during construction of the obstacles.
@param provider The provider instance to use for later queries.
@see BridgeManager#registerObstacles
@see BridgeManager#removeObstacleProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23addObstacleProvider(IObstacleProvider) Online Documentation}*/
addObstacleProvider(provider: IObstacleProvider): void;
/**
Disposes of this instance by resetting the {@link BridgeManager#canvasComponent} property to <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23dispose() Online Documentation}*/
dispose(): void;
/**
Gets a hash code that describes the current state of the obstacles.
This method can be used to get a concise, digest snapshot of the current obstacle locations. Storing that result and
comparing it to another snapshot later allows to determine quickly whether obstacles have changed since the last call.
This is helpful, e.g. in implementations of {@link IEdgeStyle} that support bridges to not recreate the edge path
(including bridges) in {@link IVisualCreator#updateVisual} if no obstacles changed.
@param context The context to inspect.
@returns A hash of the state of the obstacles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23getObstacleHash(IRenderContext) Online Documentation}*/
getObstacleHash(context: IRenderContext): number;
/**
Implementation that will be called by the default value of the {@link BridgeManager#defaultBridgeCreator} to satisfy requests to
{@link IBridgeCreator#createCustomBridge}.
This implementation will insert a gap style bridge by first creating a {@link GeneralPath#lineTo line} to <code>start</code> and then {@link GeneralPath#moveTo moving} to <code>end</code>.
@param context The context for the call.
@param path The path to append the next segment to.
@param start The coordinates of the starting point of the bridge.
@param end The coordinates of the ending point of the bridge.
@param gapLength The distance between the starting point and the end point.
@see IBridgeCreator#createCustomBridge
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23insertDefaultCustomBridge(IRenderContext%2CGeneralPath%2CPoint%2CPoint%2Cnumber) Online Documentation}*/
insertDefaultCustomBridge(context: IRenderContext, path: GeneralPath, start: Point, end: Point, gapLength: number): void;
/**
Installs this manager for the specified {@link BridgeManager#canvasComponent}.
@param canvas The {@link BridgeManager#canvasComponent}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23install(CanvasComponent) Online Documentation}*/
install(canvas: CanvasComponent): void;
/**
Dynamically registers an obstacle path with the context during the rendering.
Normally all obstacles will be registered initially by method
{@link BridgeManager#registerObstacles}. This method can be called during the rendering phase to register additional
obstacles at a later point in time.
@param context The context to register the obstacles with.
@param path The path that describes the obstacles.
@see BridgeManager#registerObstacleLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23registerObstacle(IRenderContext%2CGeneralPath) Online Documentation}*/
registerObstacle(context: IRenderContext, path: GeneralPath): void;
/**
Dynamically registers a single obstacle cubic curve with the context during the rendering.
Normally all obstacles will be registered initially by method
{@link BridgeManager#registerObstacles}. This method can be called during the rendering phase to register additional
obstacles at a later point in time.
@param context The context to register the obstacles with.
@param p1 The coordinates of the starting point of the line.
@param cp1 The coordinates of the first control point of the curve.
@param cp2 The coordinates of the second control point of the curve.
@param p2 The coordinates of the ending point of the line.
@see BridgeManager#registerObstacleLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23registerObstacleCubicCurve(IRenderContext%2CPoint%2CPoint%2CPoint%2CPoint) Online Documentation}*/
registerObstacleCubicCurve(context: IRenderContext, p1: Point, cp1: Point, cp2: Point, p2: Point): void;
/**
Dynamically registers a single obstacle line with the context during the rendering.
Normally all obstacles will be registered initially by method
{@link BridgeManager#registerObstacles}. This method can be called during the rendering phase to register additional
obstacles at a later point in time.
@param context The context to register the obstacles with.
@param p1 The coordinates of the first point of the line.
@param p2 The coordinates of the second point of the line.
@see BridgeManager#registerObstacleLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23registerObstacleLine(IRenderContext%2CPoint%2CPoint) Online Documentation}*/
registerObstacleLine(context: IRenderContext, p1: Point, p2: Point): void;
/**
Dynamically registers a single obstacle quadratic curve with the context during the rendering.
Normally all obstacles will be registered initially by method
{@link BridgeManager#registerObstacles}. This method can be called during the rendering phase to register additional
obstacles at a later point in time.
@param context The context to register the obstacles with.
@param p1 The coordinates of the starting point of the line.
@param cp The coordinates of the control point of the curve.
@param p2 The coordinates of the ending point of the line.
@see BridgeManager#registerObstacleLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23registerObstacleQuadCurve(IRenderContext%2CPoint%2CPoint%2CPoint) Online Documentation}*/
registerObstacleQuadCurve(context: IRenderContext, p1: Point, cp: Point, p2: Point): void;
/**
Called to initialize the context with the obstacles.
@param context The context that will be used to store the obstacles with.
@see BridgeManager#registerObstacle
@see BridgeManager#registerObstacleLine
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23registerObstacles(IRenderContext) Online Documentation}*/
registerObstacles(context: IRenderContext): void;
/**
Removes a previously {@link BridgeManager#addObstacleProvider added} {@link IObstacleProvider} from the list of registered providers.
@param provider The provider instance to remove from this instance.
@see BridgeManager#addObstacleProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23removeObstacleProvider(IObstacleProvider) Online Documentation}*/
removeObstacleProvider(provider: IObstacleProvider): void;
/**
Reverts the {@link BridgeManager#install} method.
@param canvas The {@link BridgeManager#canvasComponent}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23uninstall(CanvasComponent) Online Documentation}*/
uninstall(canvas: CanvasComponent): void;
/**
Gets or sets the default implementation of the {@link IBridgeCreator} that will be used if {@link BridgeManager#addBridges}
is invoked with a <code>null</code> callback argument.
@see BridgeManager#defaultBridgeCrossingStyle
@see BridgeManager#defaultBridgeWidth
@see BridgeManager#defaultBridgeHeight
@see BridgeManager#insertDefaultCustomBridge
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23defaultBridgeCreator Online Documentation}*/
defaultBridgeCreator: IBridgeCreator;
/**
Gets or sets a value that determines whether curves ({@link GeneralPath#cubicTo cubic} and {@link GeneralPath#quadTo quadratic}) should be considered as obstacles.
This can be a costly operation so the default is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23considerCurves Online Documentation}*/
considerCurves: boolean;
/**
Gets or sets a threshold value that determines below which zoom level, there should be no more bridge calculation.
Since the calculation of the bridges is a costly operation if there are many items visible and bridges are not clearly
visible at low zoom levels, it is convenient to disable the calculation of bridges for these zoom levels to speed up the
rendering process. The default value is <code>0.3d</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23zoomThreshold Online Documentation}*/
zoomThreshold: number;
/**
Gets or sets the {@link BridgeCrossingPolicy} mode that is used by this instance.
By default the {@link BridgeCrossingPolicy#HORIZONTAL_BRIDGES_VERTICAL} mode is set.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23bridgeCrossingPolicy Online Documentation}*/
bridgeCrossingPolicy: BridgeCrossingPolicy;
/**
Gets or sets the {@link BridgeManager#canvasComponent} that should be managed by this instance.
This method will {@link BridgeManager#install} a {@link CanvasComponent#addPrepareRenderContextListener preparation stage} for the {@link IRenderContext} in the
provided {@link BridgeManager#canvasComponent} via the {@link ILookup#lookup} mechanism.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23canvasComponent Online Documentation}*/
canvasComponent: CanvasComponent | null;
/**
Gets or sets the default {@link BridgeCrossingStyle} that will be used by the default implementation of the {@link IBridgeCreator}
that is initially assigned to the {@link BridgeManager#defaultBridgeCreator} property.
The default value is {@link BridgeCrossingStyle#ARC}.
@see IBridgeCreator#getCrossingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23defaultBridgeCrossingStyle Online Documentation}*/
defaultBridgeCrossingStyle: BridgeCrossingStyle;
/**
Gets or sets the default width of a bridge that will be used by the default implementation of the {@link IBridgeCreator}
that is initially assigned to the {@link BridgeManager#defaultBridgeCreator} property.
The default value is <code>10.0d</code>. Setting this value will adjust the {@link BridgeManager#clipMargin}.
@see IBridgeCreator#getBridgeWidth
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23defaultBridgeWidth Online Documentation}*/
defaultBridgeWidth: number;
/**
Gets or sets the margin by which the {@link IRenderContext#clip} should be enlarged to accommodate for clipped away lines being taken into account
for bridge calculation.
This value should be set to at least half the value of {@link IBridgeCreator#getBridgeWidth the width of the bridges} used in this context. This is because
obstacles might otherwise be clipped from the view and not taken into account for the calculation of the bridges. By
default this value will be 2 more than half of the {@link BridgeManager#defaultBridgeWidth}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23clipMargin Online Documentation}*/
clipMargin: number;
/**
Gets or sets the default height of a bridge that will be used by the default implementation of the {@link IBridgeCreator}
that is initially assigned to the {@link BridgeManager#defaultBridgeCreator} property.
The default value is <code>5.0d</code>.
@see IBridgeCreator#getBridgeHeight
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23defaultBridgeHeight Online Documentation}*/
defaultBridgeHeight: number;
/**
Gets or sets the default {@link BridgeOrientationStyle} for bridges that will be used by the default implementation
of the {@link IBridgeCreator} that is initially assigned to the {@link BridgeManager#defaultBridgeCreator} property.
The default is {@link BridgeOrientationStyle#UP}.
@see IBridgeCreator#getOrientationStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/BridgeManager%23defaultBridgeOrientationStyle Online Documentation}*/
defaultBridgeOrientationStyle: BridgeOrientationStyle;
static $class: Class;
private $_BridgeManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BridgeManager;
}
export interface IBridgeCreator {
}
/**
Allows to add bridge visualizations to a {@link GeneralPath}.
An implementation of this interface is used by {@link BridgeManager} to {@link BridgeManager#addBridges insert bridges} into an edge path.
A custom implementation of this interface is necessary if a {@link BridgeCrossingStyle#CUSTOM custom bridge drawing style} is used.
@see BridgeManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBridgeCreator Online Documentation}*/
export abstract class IBridgeCreator {
/**
Creates an implementation of the interface {@link IBridgeCreator} from the given definition.
The given object must contain exactly the abstract members of the {@link IBridgeCreator} interface. No member may be
omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IBridgeCreator} interface.
@returns An instance of the {@link IBridgeCreator} interface based on the given definition.*/
static create(interfaceDefinition: {
getCrossingStyle(context: IRenderContext): BridgeCrossingStyle;
getOrientationStyle(context: IRenderContext): BridgeOrientationStyle;
getBridgeWidth(context: IRenderContext): number;
getBridgeHeight(context: IRenderContext): number;
createCustomBridge(context: IRenderContext, path: GeneralPath, start: Point, end: Point, gapLength: number): void;
}): IBridgeCreator;
/**
Called by the {@link BridgeManager} if the {@link IBridgeCreator#getCrossingStyle} method yields {@link BridgeCrossingStyle#CUSTOM}
to actually insert a bridge into the given {@link GeneralPath}.
When this method is called the {@link GeneralPath}'s {@link GeneralPath#lastCoordinate last} coordinates are not yet at <code>start</code>. So most
implementation should first {@link GeneralPath#moveTo} that location. Also at the end of the call,
implementation should make sure that the GeneralPath's end is at <code>end</code>. The <code>gapLength</code> is provided for convenience so that
the distance between <code>startPoint</code> and <code>endPoint</code> does not need to be calculated if it is needed for the drawing.
@param context The context for the call.
@param path The path to append the bridge segment to.
@param start The coordinates of the starting point of the bridge.
@param end The coordinates of the ending point of the bridge.
@param gapLength The distance between the starting point and the end point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBridgeCreator%23createCustomBridge(IRenderContext%2CGeneralPath%2CPoint%2CPoint%2Cnumber) Online Documentation}*/
abstract createCustomBridge(context: IRenderContext, path: GeneralPath, start: Point, end: Point, gapLength: number): void;
/**
Gets the height of the bridge for the given {@link IRenderContext}.
The height is the default height that will be used for drawing the non-{@link BridgeCrossingStyle#CUSTOM custom} {@link BridgeCrossingStyle}s.
@param context The context where the bridge will be created for.
@returns The non-negative preferred basic unscaled height of the bridge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBridgeCreator%23getBridgeHeight(IRenderContext) Online Documentation}*/
abstract getBridgeHeight(context: IRenderContext): number;
/**
Gets the width of the bridge for the given {@link IRenderContext}.
The width is the preferred length of the segment that will be removed from the original segment if it crosses an
obstacle. That bridge will be made larger if multiple obstacle bridges would otherwise overlap. The bridge will be
automatically made smaller if it happens to be near the end of a segment and would otherwise extend beyond the end of
the segment.
@param context The context where the bridge will be created for.
@returns The positive preferred width of the bridge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBridgeCreator%23getBridgeWidth(IRenderContext) Online Documentation}*/
abstract getBridgeWidth(context: IRenderContext): number;
/**
Gets the {@link BridgeCrossingStyle} to use in the given {@link IRenderContext}.
@param context The context in which the crossing will be used.
@returns The style to use, or {@link BridgeCrossingStyle#CUSTOM} in order to let {@link BridgeManager} call
{@link IBridgeCreator#createCustomBridge}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBridgeCreator%23getCrossingStyle(IRenderContext) Online Documentation}*/
abstract getCrossingStyle(context: IRenderContext): BridgeCrossingStyle;
/**
Gets the {@link BridgeOrientationStyle} to use in the given {@link IRenderContext}.
@param context The context in which the style will be used..
@returns The style to use. Any value can be returned and will be ignored if the {@link IBridgeCreator#getCrossingStyle crossing style} is set to
{@link BridgeCrossingStyle#CUSTOM}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBridgeCreator%23getOrientationStyle(IRenderContext) Online Documentation}*/
abstract getOrientationStyle(context: IRenderContext): BridgeOrientationStyle;
static $class: Class;
private $_IBridgeCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IBridgeCreator;
}
export interface IObstacleProvider {
}
/**
Implementations of this interface can provide a path that is used to determine bridges by {@link BridgeManager}.
Typically the {@link ILookup} implementation of styles will return an implementation of this interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObstacleProvider Online Documentation}*/
export abstract class IObstacleProvider {
/**
Creates an implementation of the interface {@link IObstacleProvider} by using the given function as implementation
for its {@link IObstacleProvider#getObstacles} method.
@param getObstacles A function for {@link IObstacleProvider}'s single abstract method {@link IObstacleProvider#getObstacles}.
@returns An instance of the {@link IObstacleProvider} interface based on the given function.*/
static create(getObstacles: (context: IRenderContext) => GeneralPath): IObstacleProvider;
/**
Returns a path representing obstacles that would need bridges.
For performance reasons the implementation should only return a non-<code>null</code> {@link GeneralPath} if it might be {@link IVisibilityTestable#isVisible visible}
in the provided {@link IRenderContext#clip}.
@param context The context for which the obstacles are queried.
@returns A path that describes the obstacles or <code>null</code> if there are no obstacles for the given context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObstacleProvider%23getObstacles(IRenderContext) Online Documentation}*/
abstract getObstacles(context: IRenderContext): GeneralPath | null;
static $class: Class;
private $_IObstacleProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IObstacleProvider;
}
export interface CollectionModelManager<T> extends ItemModelManager<T> {
}
/**
An {@link ItemModelManager} that automatically installs and removes visual representations of items in a {@link IObservableCollection} into or from the canvas.
Type parameter <code>T</code>: The type of items in the observable collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectionModelManager Online Documentation}*/
export class CollectionModelManager<T> {
/**
Creates a new instance using the given <code>model</code>.
@param itemType The type of items in the observable collection.
@param model The model to manage.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectionModelManager%23CollectionModelManager(IObservableCollection%3CT%3E) Online Documentation}*/
constructor(itemType: Class, model: IObservableCollection<T> | null);
/**
Creates a new instance.
@param itemType The type of items in the observable collection.
@see CollectionModelManager#model
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectionModelManager%23CollectionModelManager() Online Documentation}*/
constructor(itemType: Class);
/**
Called when this mode gets {@link CollectionModelManager#enabled disabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectionModelManager%23onDisabled() Online Documentation}*/
onDisabled(): void;
/**
Called when this mode gets {@link CollectionModelManager#enabled enabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectionModelManager%23onEnabled() Online Documentation}*/
onEnabled(): void;
/**
Gets or sets a value indicating whether this {@link CollectionModelManager} is enabled.
If the manager is disabled, it will not install the {@link ICanvasObject}s into the associated
{@link CanvasComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectionModelManager%23enabled Online Documentation}*/
enabled: boolean;
/**
Gets or sets the model of the items to display in the canvas.
Setting the model will automatically remove the items installed for a previously installed model. Also the items
contained in the new model will immediately be queried and installed into the canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectionModelManager%23model Online Documentation}*/
model: IObservableCollection<T> | null;
static $class: Class;
private $_CollectionModelManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is CollectionModelManager<T>;
}
export interface ItemModelManager<T> extends YObject {
}
/**
Installs and removes visual representations of items into or from the canvas.
Type parameter <code>T</code>: The type of managed items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager Online Documentation}*/
export class ItemModelManager<T> {
/**
Creates a new instance of {@link ItemModelManager}.
@param itemType The type of managed items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager Online Documentation}*/
constructor(itemType: Class);
/**
Registers and {@link ItemModelManager#install installs} the given <code>item</code> into the canvas.
@param item The item to install.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23addItem(T) Online Documentation}*/
addItem(item: T): void;
/**
{@link ItemModelManager#addItem Adds} all items to the canvas.
This is useful to initialize this manager with an existing collection of items.
@param items The items to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23addItems(IEnumerable%3CT%3E) Online Documentation}*/
addItems(items: IEnumerable<T> | IEnumerableConvertible<T>): void;
/**
Retrieves the canvas object that has been registered for the given item or <code>null</code>.
@param item The item to retrieve the canvas object for.
@returns A canvas object instance or <code>null</code>.
@see ItemModelManager#install
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23getCanvasObject(T) Online Documentation}*/
getCanvasObject(item: T): ICanvasObject | null;
/**
Gets the {@link ICanvasObjectGroup} used to {@link ItemModelManager#install} the <code>item</code> in the canvas.
The default implementation simply returns the value of {@link ItemModelManager#canvasObjectGroup}.
@param item The item to get the group for.
@returns The {@link ICanvasObjectGroup} used to {@link ItemModelManager#install} an item in the canvas.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23getCanvasObjectGroup(T) Online Documentation}*/
getCanvasObjectGroup(item: T): ICanvasObjectGroup | null;
/**
Returns the {@link ICanvasObjectDescriptor} used to {@link ItemModelManager#install} the <code>item</code> in the canvas.
The default implementation simply returns the value of {@link ItemModelManager#descriptor}.
@param item The item to get the descriptor for.
@returns The {@link ICanvasObjectDescriptor} used to {@link ItemModelManager#install} an item in the canvas.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23getDescriptor(T) Online Documentation}*/
getDescriptor(item: T): ICanvasObjectDescriptor | null;
/**
Tries to find the item managed by this instance that is associated with the given canvas object.
@param canvasObject The canvas object to query the corresponding model item for.
@returns The item or <code>null</code> if no such item could be found.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23getItem(ICanvasObject) Online Documentation}*/
getItem(canvasObject: ICanvasObject): T | null;
/**
Installs a member of the collection into the canvas using the {@link ItemModelManager#descriptor}.
@param item The member to install.
@returns The canvas object that has been returned by the installer or <code>null</code> if the item was not installed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23install(T) Online Documentation}*/
install(item: T): ICanvasObject | null;
/**
* Event handler for item creation events that adds the item to this manager.
* This method is intended to be used to delegate item creation events from observable collections:
*
* <br><b>Using the ItemAddedHandler method as an event handler on an observable collection</b>
* <pre>
* manager.addItemAddedListener(model.itemAddedHandler)
* </pre>
*
* This method calls {@link ItemModelManager#addItem} with {@link ItemEventArgs#item}.
*
* @param source The instance that raised the event.
* @param evt The event arguments.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23itemAddedHandler(Object%2CItemEventArgs%3CT%3E) Online Documentation}
*/
itemAddedHandler(source: any | null, evt: ItemEventArgs<T>): void;
/**
* Event handler for item change events that updates the visual representation for that item.
* This method is intended to be used to delegate item change events from observable collections:
*
* <br><b>Using the ItemChangedHandler method as an event handler on an observable collection</b>
* <pre>
* manager.addItemChangedListener(model.itemChangedHandler)
* </pre>
*
* This is necessary if the group the canvas object should be added to might change after the item change. This might
* happen, e.g. after changing an edge's port in a grouped graph.
*
* @param source The event source.
* @param evt The item event args which indicate the change.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23itemChangedHandler(Object%2CItemEventArgs%3CT%3E) Online Documentation}
*/
itemChangedHandler(source: any, evt: ItemEventArgs<T>): void;
/**
* Event handler for item removal events that removes the item from this manager.
* This method is intended to be used to delegate item removal events from observable collections:
*
* <br><b>Using the ItemRemovedHandler method as an event handler on an observable collection</b>
* <pre>
* manager.addItemRemovedListener(model.itemRemovedHandler)
* </pre>
*
* This method calls {@link ItemModelManager#removeItem} with {@link ItemEventArgs#item}.
*
* @param source The instance that raised the event.
* @param evt The event arguments.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23itemRemovedHandler(Object%2CItemEventArgs%3CT%3E) Online Documentation}
*/
itemRemovedHandler(source: any, evt: ItemEventArgs<T>): void;
/**
Deregisters and uninstalls the given <code>item</code> from the canvas.
@param item The item to uninstall.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23removeItem(T) Online Documentation}*/
removeItem(item: T): void;
/**
{@link ItemModelManager#removeItem Removes} all items from the canvas.
This method can be used to clean up this manager before disposing it.
@param items The items to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23removeItems(IEnumerable%3CT%3E) Online Documentation}*/
removeItems(items: IEnumerable<T> | IEnumerableConvertible<T>): void;
/**
This method may be called to update the visual representation of the given item in the canvas.
If no visual representation is registered, yet, {@link ItemModelManager#install} will be called.
@param item The item to update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23update(T) Online Documentation}*/
update(item: T): void;
/**
Gets or sets an {@link IComparer} that compares the items so that they are put into the correct order in the canvas tree.
If this instance is set to <code>null</code> this feature is effectively turned off.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23comparer Online Documentation}*/
comparer: IComparer<T> | null;
/**
Gets or sets the canvas object descriptor used by {@link ItemModelManager#install} when an item is {@link ICanvasObjectGroup#addChild added} to an canvas object group.
Setting the descriptor to <code>null</code> will prevent this manager from installing new items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23descriptor Online Documentation}*/
descriptor: ICanvasObjectDescriptor | null;
/**
Gets or sets the canvas object group used by {@link ItemModelManager#install}.
Setting this property to <code>null</code> will prevent this manager from installing new items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemModelManager%23canvasObjectGroup Online Documentation}*/
canvasObjectGroup: ICanvasObjectGroup | null;
static $class: Class;
private $_ItemModelManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ItemModelManager<T>;
}
export interface ObservableCollection<T> extends YObject, IObservableCollection<T> {
}
/**
A simple default implementation of an {@link IObservableCollection} that is backed by an ordinary {@link ICollection}
Type parameter <code>T</code>: The type of the elements contained in this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection Online Documentation}*/
export class ObservableCollection<T> {
/**
Creates a new instance using a provided collection as the backing store.
@param [items=null] The collection to use as the backing store. If no collection is provided a {@link List} will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection Online Documentation}*/
constructor(items?: ICollection<T> | null | ICollectionConvertible<T>);
/**
Adds the item the the collection and triggers the {@link ObservableCollection#addItemAddedListener ItemAdded} event.
This implementation calls the {@link ObservableCollection#onItemAdded} method.
@param item The item to add to the collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23add(T) Online Documentation}*/
add(item: T): void;
/**
Clears this collection, triggering an {@link ObservableCollection#addItemRemovedListener ItemRemoved} event for each element removed.
@see ObservableCollection#onItemRemoved
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23clear() Online Documentation}*/
clear(): void;
/**
Copies the items to the given array.
@param array
@param arrayIndex
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23copyTo(Array%3CT%3E%2Cnumber) Online Documentation}*/
copyTo(array: T[], arrayIndex: number): void;
/**
Returns an enumerator over the elements in this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23getEnumerator() Online Documentation}*/
getEnumerator(): IEnumerator<T>;
/**
Checks for containment of the item in this collection.
@param item The item to check.
@returns Whether the item is contained in this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23includes(T) Online Documentation}*/
includes(item: T): boolean;
/**
Triggers the {@link ObservableCollection#addItemAddedListener ItemAdded} event.
@param item The item that has just been added to the collection.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23onItemAdded(T) Online Documentation}*/
onItemAdded(item: T): void;
/**
Triggers the {@link ObservableCollection#addItemChangedListener ItemChanged} event.
@param item The item that has changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23onItemChanged(T) Online Documentation}*/
onItemChanged(item: T): void;
/**
Triggers the {@link ObservableCollection#addItemRemovedListener ItemRemoved} event.
@param item The item that has just been removed from the collection.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23onItemRemoved(T) Online Documentation}*/
onItemRemoved(item: T): void;
/**
Method that may be called by client code to trigger an {@link ObservableCollection#addItemChangedListener ItemChanged} event.
@param item The item to trigger the change event for.
@see ObservableCollection#onItemChanged
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23publishItemChanged(T) Online Documentation}*/
publishItemChanged(item: T): void;
/**
Tries to remove the item from the collection and triggers the {@link ObservableCollection#addItemRemovedListener ItemRemoved} event.
@param item The item to remove.
@returns Whether the item has been removed.
@see ObservableCollection#onItemRemoved
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23remove(T) Online Documentation}*/
remove(item: T): boolean;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23toString() Online Documentation}*/
toString(): string;
/**
Gets the number of elements in this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23size Online Documentation}*/
readonly size: number;
/**
Gets whether this collection is read only.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23isReadOnly Online Documentation}*/
readonly isReadOnly: boolean;
/**
Gets the collection used internally as the backing store.
Normally code should not use this method to gain access to the collection in order to modify it.
@returns The backing store of this observable collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23items Online Documentation}*/
readonly items: ICollection<T>;
/**
Adds the given listener for the <code>ItemAdded</code> event that occurs when an item has been added to this collection.
@param listener The listener to add.
@see ObservableCollection#removeItemAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23ItemAdded Online Documentation}*/
addItemAddedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Removes the given listener for the <code>ItemAdded</code> event that occurs when an item has been added to this collection.
@param listener The listener to remove.
@see ObservableCollection#addItemAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23ItemAdded Online Documentation}*/
removeItemAddedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Adds the given listener for the <code>ItemRemoved</code> event that occurs when an item has been removed from this collection.
@param listener The listener to add.
@see ObservableCollection#removeItemRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23ItemRemoved Online Documentation}*/
addItemRemovedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Removes the given listener for the <code>ItemRemoved</code> event that occurs when an item has been removed from this collection.
@param listener The listener to remove.
@see ObservableCollection#addItemRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23ItemRemoved Online Documentation}*/
removeItemRemovedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Adds the given listener for the <code>ItemChanged</code> event that occurs when an item has been changed inside this collection.
@param listener The listener to add.
@see ObservableCollection#removeItemChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23ItemChanged Online Documentation}*/
addItemChangedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Removes the given listener for the <code>ItemChanged</code> event that occurs when an item has been changed inside this collection.
@param listener The listener to remove.
@see ObservableCollection#addItemChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObservableCollection%23ItemChanged Online Documentation}*/
removeItemChangedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
static $class: Class;
private $_ObservableCollection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ObservableCollection<T>;
}
export interface DefaultSelectionModel<T> extends YObject, ISelectionModel<T> {
/**
@inheritdoc*/
clear(): void;
/**
@inheritdoc*/
getEnumerator(): IEnumerator<T>;
/**
@inheritdoc*/
addItemSelectionChangedListener(listener: (sender: any, evt: ItemSelectionChangedEventArgs<T>) => void): void;
/**
@inheritdoc*/
removeItemSelectionChangedListener(listener: (sender: any, evt: ItemSelectionChangedEventArgs<T>) => void): void;
} /**
Default implementation of an {@link ISelectionModel} that is based on an observable collection to hold the selected elements.
This selection model has a domain that's controlled with a {@link DefaultSelectionModel#isInDomain predicate} which encompasses the items that can be selected.
If that domain changes it is important to delegate item removal events from the domain model to this selection model to
ensure that items that have been removed are deselected as well. An {@link DefaultSelectionModel#itemRemovedFromDomainHandler event handler} exists for that purpose.
Type parameter <code>T</code>: The type of the items to be selected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSelectionModel Online Documentation}*/
export class DefaultSelectionModel<T> {
/**
Creates a default selection model implementation that uses a {@link ObservableCollection} to hold the selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSelectionModel Online Documentation}*/
constructor();
/**
Determines whether an item is selected by checking for the containment in the {@link DefaultSelectionModel#selection}.
@param item The item to check
@returns Whether the item is selected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSelectionModel%23isSelected(T) Online Documentation}*/
isSelected(item: T): boolean;
/**
* Event handler for domain item removal events that removes the item from this model.
* This method is intended to be used to delegate item removal events from the respective domain collection:
*
* <br><b>Using the ItemAddedHandler method as an event handler on an observable collection</b>
* <pre>
* model.addItemRemovedListener(defaultSelectionModel.itemRemovedFromDomainHandler)
* </pre>
*
* This method calls {@link DefaultSelectionModel#removeDomainItem} with {@link ItemEventArgs#item}.
*
* @param source The instance that raised the event.
* @param evt The event arguments.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSelectionModel%23itemRemovedFromDomainHandler(Object%2CItemEventArgs%3CT%3E) Online Documentation}
*/
itemRemovedFromDomainHandler(source: any, evt: ItemEventArgs<T>): void;
/**
Called when an item gets selected or deselected.
Triggers the {@link DefaultSelectionModel#addItemSelectionChangedListener ItemSelectionChanged} event.
@param o The item.
@param selected Whether or not the item is selected after the change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSelectionModel%23onItemSelectionChanged(T%2Cboolean) Online Documentation}*/
onItemSelectionChanged(o: T, selected: boolean): void;
/**
Removes the given <code>item</code> from the selection in response to the item being removed from the domain.
@param item The item to remove from the considered items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSelectionModel%23removeDomainItem(T) Online Documentation}*/
removeDomainItem(item: T): void;
/**
Sets the selection state of the item.
@param item
@param selected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSelectionModel%23setSelected(T%2Cboolean) Online Documentation}*/
setSelected(item: T, selected: boolean): void;
/**
Gets or sets the predicate which determines whether the queried item should be considered for this selection model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSelectionModel%23isInDomain Online Documentation}*/
isInDomain: ((obj: T) => boolean) | null;
/**
Gets an observable collection view over the selected items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSelectionModel%23selection Online Documentation}*/
readonly selection: IObservableCollection<T>;
static $class: Class;
private $_DefaultSelectionModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is DefaultSelectionModel<T>;
}
export interface Mapper<K, V> extends YObject, IMapper<K, V> {
/**
@inheritdoc*/
get(key: K | null): V | null;
/**
@inheritdoc*/
set(key: K | null, value: V | null): void;
} /**
An {@link IMapper} implementation backed by a {@link IMap}.
A {@link Mapper#defaultValue} can be set that is returned for keys that don't have an explicit value set.
It is legal to use <code>null</code> as key, which can be assigned a value just like other keys.
Type parameter <code>K</code>.
Type parameter <code>V</code>.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Mapper Online Documentation}*/
export class Mapper<K, V> {
/**
Creates an instance using the provided {@link HashMap} as the backing store.
The value for the null item is stored separately.
@param dictionary
@see {@link https://docs.yworks.com/yfileshtml/#/api/Mapper%23Mapper(IMap%3CK%2CV%3E) Online Documentation}*/
constructor(dictionary: IMap<K, V>);
/**
Creates an instance using the provided {@link HashMap} as the backing store.
The value for the null item is stored separately.
@param options The parameters to pass.
@param options.dictionary
@param [options.defaultValue] The default value to return if no specific value has been set for a given key. This option sets the {@link Mapper#defaultValue} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Mapper%23Mapper(IMap%3CK%2CV%3E) Online Documentation}*/
constructor(options: {
dictionary: IMap<K, V>;
defaultValue?: V | null;
});
/**
Creates an instance using a {@link HashMap} as the backing store.
The value for the null item is stored separately.
@param [options] The parameters to pass.
@param [options.defaultValue] The default value to return if no specific value has been set for a given key. This option sets the {@link Mapper#defaultValue} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Mapper%23Mapper() Online Documentation}*/
constructor(options?: {
defaultValue?: V | null;
});
/**
Calls {@link HashMap#clear} on the backing implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Mapper%23clear() Online Documentation}*/
clear(): void;
/**
Removes the mapping of the given key.
@param key The key of the mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Mapper%23delete(K) Online Documentation}*/
delete(key: K | null): void;
/**
Gets an enumerable over all entries in this (dictionary based) mapper.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Mapper%23entries Online Documentation}*/
readonly entries: IEnumerable<MapEntry<K, V>>;
/**
Gets or sets the default value to return if no specific value has been set for a given key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Mapper%23defaultValue Online Documentation}*/
defaultValue: V | null;
static $class: Class;
private $_Mapper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<K, V>(o: any): o is Mapper<K, V>;
}
/**
The enumeration that describes the possible different policies for displaying the focus of items managed by the {@link FocusIndicatorManager}
@see FocusIndicatorManager
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShowFocusPolicy Online Documentation}*/
export enum ShowFocusPolicy {
/**
Indicates that irrespectively of the keyboard focus, the focus should always be rendered for the {@link FocusIndicatorManager#focusedItem}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShowFocusPolicy%23ALWAYS Online Documentation}*/
ALWAYS,
/**
Indicates that depending on the value of {@link FocusIndicatorManager#focused} the focus should only be rendered if this property is set to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShowFocusPolicy%23ONLY_WHEN_FOCUSED Online Documentation}*/
ONLY_WHEN_FOCUSED
}
/** Automatic conversion to {@link ShowFocusPolicy}. */
type ShowFocusPolicyStringValues = "always" | "only-when-focused";
export interface FocusIndicatorManager<T extends ILookup> extends ModelManager<T>, IPropertyObservable {
/**
@inheritdoc*/
onDisabled(): void;
/**
@inheritdoc*/
onEnabled(): void;
} /**
A {@link ModelManager} implementation that manages the visual decoration of the focused element in a canvas.
For the item that should be focused this class uses the {@link ILookup} mechanism of the item to lookup an {@link IFocusIndicatorInstaller}
implementation which will do the actual installation of the decoration in the canvas.
Type parameter <code>T</code>: The type of the items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager Online Documentation}*/
export class FocusIndicatorManager<T extends ILookup> {
/**
Creates an instance that for all focused items in the model dynamically installs a decoration.
@param canvas The canvas to add the focus marks to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager Online Documentation}*/
constructor(canvas: CanvasComponent);
/**
Overridden to do nothing.
@param item
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23add(T) Online Documentation}*/
add(item: T | null): ICanvasObject | null;
/**
Called by {@link ModelManager#install} to retrieve the installer for a given item.
This implementation always returns the {@link CanvasComponent#focusGroup} of the {@link ModelManager#canvasComponent} of this instance.
@param item The item to find a canvas object group for.
@returns An {@link ICanvasObjectGroup} or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23getCanvasObjectGroup(T) Online Documentation}*/
getCanvasObjectGroup(item: T | null): ICanvasObjectGroup | null;
/**
Called by {@link ModelManager#install} to retrieve the installer for a given item.
This implementation uses the {@link ILookup#lookup} of the item to retrieve an {@link IFocusIndicatorInstaller}
instance.
@param item The item to find an installer for.
@returns An installer or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23getInstaller(T) Online Documentation}*/
getInstaller(item: T): ICanvasObjectInstaller | null;
/**
Called when the {@link FocusIndicatorManager#focused} property has changed.
@see FocusIndicatorManager#focused
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23onFocusedChanged() Online Documentation}*/
onFocusedChanged(): void;
/**
Called when the focused item has changed.
This method raises the {@link FocusIndicatorManager#addPropertyChangedListener PropertyChanged} event. Subclasses overriding this method should make sure to invoke
the super class implementation.
@param evt a description of the event
@see FocusIndicatorManager#focusedItem
@see FocusIndicatorManager#addPropertyChangedListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23onPropertyChanged(PropertyChangedEventArgs) Online Documentation}*/
onPropertyChanged(evt: PropertyChangedEventArgs): void;
/**
Overridden to do nothing.
@param item
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23remove(T) Online Documentation}*/
remove(item: T | null): void;
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when the {@link FocusIndicatorManager#focusedItem} property has been changed.
@param listener The listener to add.
@see FocusIndicatorManager#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23PropertyChanged Online Documentation}*/
addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when the {@link FocusIndicatorManager#focusedItem} property has been changed.
@param listener The listener to remove.
@see FocusIndicatorManager#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23PropertyChanged Online Documentation}*/
removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the focused item, updates the canvas and fires the corresponding property change event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23focusedItem Online Documentation}*/
focusedItem: T | null;
/**
Gets or sets the show focus policy.
The default is {@link ShowFocusPolicy#ONLY_WHEN_FOCUSED}.
@default ShowFocusPolicy.ONLY_WHEN_FOCUSED
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23showFocusPolicy Online Documentation}*/
showFocusPolicy: ShowFocusPolicy;
/**
Gets or sets a value indicating whether the {@link CanvasComponent container} this manager is installed in currently is focused.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FocusIndicatorManager%23focused Online Documentation}*/
focused: boolean;
static $class: Class;
private $_FocusIndicatorManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T extends ILookup>(o: any): o is FocusIndicatorManager<T>;
}
export interface HighlightIndicatorManager<T extends ILookup> extends ModelManager<T> {
/**
@inheritdoc*/
onDisabled(): void;
/**
@inheritdoc*/
onEnabled(): void;
} /**
A {@link ModelManager} implementation that manages the visual decorations of highlighted elements in a canvas.
For each item that should be highlighted this class uses the {@link ILookup} mechanism of the items to lookup an {@link IHighlightIndicatorInstaller}
implementation which will do the actual installation of the decoration in the canvas.
Type parameter <code>T</code>: The type of the items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HighlightIndicatorManager Online Documentation}*/
export class HighlightIndicatorManager<T extends ILookup> {
/**
Creates an instance that for all items in the model dynamically installs a highlight decoration.
@param canvas The canvas to add the selection marks to.
@param [selectionModel=null] The model that determines which items are highlighted.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HighlightIndicatorManager Online Documentation}*/
constructor(canvas: CanvasComponent, selectionModel?: ISelectionModel<T> | null);
/**
Adds another highlight.
@param item The item to highlight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HighlightIndicatorManager%23addHighlight(T) Online Documentation}*/
addHighlight(item: T): void;
/**
Removes all highlights.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HighlightIndicatorManager%23clearHighlights() Online Documentation}*/
clearHighlights(): void;
/**
Called by {@link ModelManager#install} to retrieve the installer for a given item.
This implementation always returns the {@link CanvasComponent#highlightGroup} of the {@link ModelManager#canvasComponent} of this instance.
@param item The item to find a canvas object group for.
@returns An {@link ICanvasObjectGroup} or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HighlightIndicatorManager%23getCanvasObjectGroup(T) Online Documentation}*/
getCanvasObjectGroup(item: T): ICanvasObjectGroup | null;
/**
Called by {@link ModelManager#install} to retrieve the installer for a given item.
This implementation uses the {@link ILookup#lookup} of the item to retrieve an {@link IHighlightIndicatorInstaller}
instance.
@param item The item to find an installer for.
@returns An installer or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HighlightIndicatorManager%23getInstaller(T) Online Documentation}*/
getInstaller(item: T): ICanvasObjectInstaller | null;
/**
Removes an item from the current highlight selection.
@param item The item whose highlight decorator will be removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HighlightIndicatorManager%23removeHighlight(T) Online Documentation}*/
removeHighlight(item: T): void;
/**
Gets or sets the item collection that determines which items are highlighted.
May be <code>null</code>, in which case no elements are considered highlighted.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HighlightIndicatorManager%23selectionModel Online Documentation}*/
selectionModel: ISelectionModel<T> | null;
static $class: Class;
private $_HighlightIndicatorManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T extends ILookup>(o: any): o is HighlightIndicatorManager<T>;
}
export interface ICanvasObjectInstaller {
}
/**
Installs visual representations of items in a {@link CanvasComponent}.
This interface is most prominently used by {@link ModelManager} and its descendants to manage visuals in the canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectInstaller Online Documentation}*/
export abstract class ICanvasObjectInstaller {
/**
This the main method of the interface that performs the installation of an item's visual representation in the {@link CanvasComponent canvas} by
adding {@link ICanvasObject}s.
@param context The context that contains the information required to install the visual representation.
@param group The canvas object group to add the newly generated {@link ICanvasObject} to.
@param item The item to install.
@returns The newly generated {@link ICanvasObject} for the item's visual representation. <code>null</code> if nothing was installed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICanvasObjectInstaller%23addCanvasObject(ICanvasContext%2CICanvasObjectGroup%2CObject) Online Documentation}*/
abstract addCanvasObject(context: ICanvasContext, group: ICanvasObjectGroup, item: any | null): ICanvasObject | null;
/**
Creates an implementation of the interface {@link ICanvasObjectInstaller} by using the given function as
implementation for its {@link ICanvasObjectInstaller#addCanvasObject} method.
@param addCanvasObject A function for {@link ICanvasObjectInstaller}'s single abstract method
{@link ICanvasObjectInstaller#addCanvasObject}.
@returns An instance of the {@link ICanvasObjectInstaller} interface based on the given function.*/
static create(addCanvasObject: (context: ICanvasContext, group: ICanvasObjectGroup, item: any | null) => ICanvasObject): ICanvasObjectInstaller;
static $class: Class;
private $_ICanvasObjectInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ICanvasObjectInstaller;
}
export interface IModelItem extends ILookup, ITagOwner {
}
/**
Base interface for items that can be used in a {@link CanvasComponent}.
Most prominently this is the interface all graph elements used in {@link IGraph} implement.
It adds no functionality over the {@link ILookup} and {@link ITagOwner} interfaces it inherits.
To add custom visuals to a {@link CanvasComponent} it is not necessary to implement this interface. Instead, any
object can be {@link ICanvasObjectGroup#addChild added directly to the canvas}, provided there is a suitable {@link ICanvasObjectDescriptor}
implementation for rendering it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IModelItem Online Documentation}*/
export abstract class IModelItem {
static $class: Class;
private $_IModelItem: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IModelItem;
}
export interface ISelectionIndicatorInstaller extends ICanvasObjectInstaller {
}
/**
An interface for objects that can install a visual representation of a selection decoration of an item in the model
displayed in a canvas.
This interface is a tagging sub interface of the {@link ICanvasObjectInstaller}. This interface is used for {@link ILookup#lookup}
operations.
@see SelectionIndicatorManager
@see IHighlightIndicatorInstaller
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISelectionIndicatorInstaller Online Documentation}*/
export abstract class ISelectionIndicatorInstaller {
/**
Creates an implementation of the interface {@link ISelectionIndicatorInstaller} by using the given function as
implementation for its {@link ISelectionIndicatorInstaller#addCanvasObject} method.
@param addCanvasObject A function for {@link ISelectionIndicatorInstaller}'s single abstract method
{@link ISelectionIndicatorInstaller#addCanvasObject}.
@returns An instance of the {@link ISelectionIndicatorInstaller} interface based on the given function.*/
static create(addCanvasObject: (context: ICanvasContext, group: ICanvasObjectGroup, item: any | null) => ICanvasObject): ISelectionIndicatorInstaller;
static $class: Class;
private $_ISelectionIndicatorInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ISelectionIndicatorInstaller;
}
export interface ISelectionModel<T> extends IEnumerable<T> {
}
/**
An interface that models the selection state of a set of items.
This interface extends the {@link IEnumerable} interface for convenient enumeration of the selected items. Use the {@link IEnumerable#getEnumerator} to get
all selected items.
Type parameter <code>T</code>: The type of items contained in the selection's domain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISelectionModel Online Documentation}*/
export abstract class ISelectionModel<T> {
/**
Clears the selection.
Sets the selection state of all elements to unselected.
@see ISelectionModel#addItemSelectionChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISelectionModel%23clear() Online Documentation}*/
abstract clear(): void;
/**
Determines whether an item is currently selected.
@param item The item to check.
@returns Whether it is currently selected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISelectionModel%23isSelected(T) Online Documentation}*/
abstract isSelected(item: T): boolean;
/**
Sets the selection state of an item.
If the state changes, this will trigger the {@link ISelectionModel#addItemSelectionChangedListener ItemSelectionChanged} event.
@param item The item to set the selection state for.
@param selected Whether to select the object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISelectionModel%23setSelected(T%2Cboolean) Online Documentation}*/
abstract setSelected(item: T, selected: boolean): void;
/**
Gets the number of selected items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISelectionModel%23size Online Documentation}*/
abstract readonly size: number;
/**
Adds the given listener for the <code>ItemSelectionChanged</code> event that occurs when an item changed its selection state from
selected to unselected or vice versa.
@param listener The listener to add.
@see ISelectionModel#setSelected
@see ISelectionModel#removeItemSelectionChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISelectionModel%23ItemSelectionChanged Online Documentation}*/
abstract addItemSelectionChangedListener(listener: (sender: any, evt: ItemSelectionChangedEventArgs<T>) => void): void;
/**
Removes the given listener for the <code>ItemSelectionChanged</code> event that occurs when an item changed its selection state from
selected to unselected or vice versa.
@param listener The listener to remove.
@see ISelectionModel#setSelected
@see ISelectionModel#addItemSelectionChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISelectionModel%23ItemSelectionChanged Online Documentation}*/
abstract removeItemSelectionChangedListener(listener: (sender: any, evt: ItemSelectionChangedEventArgs<T>) => void): void;
static $class: Class;
private $_ISelectionModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ISelectionModel<T>;
}
export interface ItemSelectionChangedEventArgs<T> extends ItemEventArgs<T> {
}
/**
Provides data for the {@link ISelectionModel#addItemSelectionChangedListener ItemSelectionChanged} event.
Type parameter <code>T</code>.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemSelectionChangedEventArgs Online Documentation}*/
export class ItemSelectionChangedEventArgs<T> {
/**
Initializes a new event argument instance for the given item and its selection state.
@param item The item whose {@link ISelectionModel#isSelected selection state} has changed.
@param selected The value indicating whether the item is selected after the change.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemSelectionChangedEventArgs Online Documentation}*/
constructor(item: T, selected: boolean);
/**
Gets a value indicating whether the item is selected after the change.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemSelectionChangedEventArgs%23itemSelected Online Documentation}*/
itemSelected: boolean;
static $class: Class;
private $_ItemSelectionChangedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ItemSelectionChangedEventArgs<T>;
}
export interface ModelManager<T> extends YObject {
}
/**
Manages the visual representations of items in a {@link CanvasComponent}.
Type parameter <code>T</code>: The type of items this instance deals with.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager Online Documentation}*/
export abstract class ModelManager<T> {
/**
Creates a new instance that will manage the items for the given canvas.
@param canvas The canvas to manage.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager Online Documentation}*/
constructor(canvas: CanvasComponent);
/**
Installs an item and adds it to this managers set of managed items.
If {@link ModelManager#enabled} is set to false the item will not be installed.
@param item The item to add.
@returns The newly created canvas object or <code>null</code> if nothing was installed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23add(T) Online Documentation}*/
add(item: T): ICanvasObject | null;
/**
Called by {@link ModelManager#install} to retrieve the canvas object group for a given item.
Returning <code>null</code> will cause {@link ModelManager#install} not to install anything.
@param item The item to find a canvas object group for.
@returns An {@link ICanvasObjectGroup} or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23getCanvasObjectGroup(T) Online Documentation}*/
abstract getCanvasObjectGroup(item: T): ICanvasObjectGroup | null;
/**
Called by {@link ModelManager#install} to retrieve the installer for a given item.
Implementations may use the {@link ILookup#lookup} of the item.
Returning <code>null</code> will cause {@link ModelManager#install} not to install anything.
@param item The item to find an installer for.
@returns An installer or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23getInstaller(T) Online Documentation}*/
abstract getInstaller(item: T): ICanvasObjectInstaller | null;
/**
Installs a member of the collection into the canvas using the {@link ICanvasObjectDescriptor} which is returned by
{@link ModelManager#getInstaller}.
@param item The member to install.
@returns The canvas object that has been returned by the installer.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23install(T) Online Documentation}*/
install(item: T): ICanvasObject | null;
/**
Called when this mode gets {@link ModelManager#enabled disabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23onDisabled() Online Documentation}*/
abstract onDisabled(): void;
/**
Called when this mode gets {@link ModelManager#enabled enabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23onEnabled() Online Documentation}*/
abstract onEnabled(): void;
/**
Uninstalls the provided item and removes it from this managers set of managed items.
@param item The item to remove.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23remove(T) Online Documentation}*/
remove(item: T): void;
/**
Simple implementation that removes the given {@link ICanvasObject}.
@param canvasObject The canvas objects that have been associated with the descriptor.
@see ICanvasObject#remove
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23uninstall(ICanvasObject) Online Documentation}*/
uninstall(canvasObject: ICanvasObject): void;
/**
Gets the canvas instance this instance is managing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23canvasComponent Online Documentation}*/
readonly canvasComponent: CanvasComponent;
/**
Gets or sets whether this {@link ModelManager} is enabled.
If the manager is disabled, it will not install the {@link ICanvasObject}s into the associated
{@link CanvasComponent}.
The default is <code>true</code>, i.e. this manager is enabled by default.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModelManager%23enabled Online Documentation}*/
enabled: boolean;
static $class: Class;
private $_ModelManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ModelManager<T>;
}
export interface SelectionIndicatorManager<T extends ILookup> extends ModelManager<T> {
/**
@inheritdoc*/
onDisabled(): void;
/**
@inheritdoc*/
onEnabled(): void;
} /**
A {@link ModelManager} implementation that manages the visual decorations of selected elements in a canvas.
For each item in the {@link SelectionIndicatorManager#model} that is marked as selected by the {@link SelectionIndicatorManager#selectionModel} this class uses the {@link ILookup} mechanism of the items
to lookup an {@link ISelectionIndicatorInstaller} implementation which will do the actual installation of the
decoration in the canvas.
Type parameter <code>T</code>: The type of the items in the collection and selection model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionIndicatorManager Online Documentation}*/
export class SelectionIndicatorManager<T extends ILookup> {
/**
Creates an instance that for all selected items in the model dynamically installs a selection render decoration.
@param canvas The canvas to add the selection marks to.
@param [model=null] The model that holds the domain.
@param [selectionModel=null] The model that determines which items are selected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionIndicatorManager Online Documentation}*/
constructor(canvas: CanvasComponent, model?: IObservableCollection<T> | null, selectionModel?: ISelectionModel<T> | null);
/**
Creates an instance that for all selected items in the model dynamically installs a selection render decoration.
@param options The parameters to pass.
@param options.canvas The canvas to add the selection marks to.
@param [options.model=null] The model that holds the domain.
@param [options.selectionModel=null] The model that determines which items are selected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionIndicatorManager Online Documentation}*/
constructor(options: {
canvas: CanvasComponent;
model?: IObservableCollection<T> | null;
selectionModel?: ISelectionModel<T> | null;
});
/**
Adds an item to the current selection.
@param item The item whose selection decorator will added.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionIndicatorManager%23addSelection(T) Online Documentation}*/
addSelection(item: T): ICanvasObject | null;
/**
Called by {@link ModelManager#install} to retrieve the installer for a given item.
This implementation always returns the {@link CanvasComponent#selectionGroup} of the {@link ModelManager#canvasComponent} of this instance.
@param item The item to find a canvas object group for.
@returns An {@link ICanvasObjectGroup} or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionIndicatorManager%23getCanvasObjectGroup(T) Online Documentation}*/
getCanvasObjectGroup(item: T): ICanvasObjectGroup | null;
/**
Called by {@link ModelManager#install} to retrieve the installer for a given item.
This implementation uses the {@link ILookup#lookup} of the item to retrieve an {@link ISelectionIndicatorInstaller}
instance.
@param item The item to find an installer for.
@returns An installer or <code>null</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionIndicatorManager%23getInstaller(T) Online Documentation}*/
getInstaller(item: T): ICanvasObjectInstaller | null;
/**
Removes an item from the current selection.
@param item The item whose selection decorator will be removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionIndicatorManager%23removeSelection(T) Online Documentation}*/
removeSelection(item: T): void;
/**
Gets or sets the selection model that determines which items are selected.
May be <code>null</code>, in which case no elements are considered selected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionIndicatorManager%23selectionModel Online Documentation}*/
selectionModel: ISelectionModel<T> | null;
/**
Gets or sets the model that describes the domain for the selection.
Only item contained in this model can be rendered as selected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelectionIndicatorManager%23model Online Documentation}*/
model: IObservableCollection<T> | null;
static $class: Class;
private $_SelectionIndicatorManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T extends ILookup>(o: any): o is SelectionIndicatorManager<T>;
}
export interface SvgVisualGroup extends SvgVisual {
}
/**
A container that can hold a number of {@link SvgVisual}s.
Use this class in a {@link IVisualCreator} implementation to group several {@link SvgVisual}s.
The class cannot be used to group {@link HtmlCanvasVisual}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisualGroup Online Documentation}*/
export class SvgVisualGroup {
/**
Initializes a new instance of the {@link SvgVisualGroup} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisualGroup Online Documentation}*/
constructor();
/**
Adds another {@link SvgVisual} to the children collection of this instance.
@param child The child to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisualGroup%23add(SvgVisual) Online Documentation}*/
add(child: SvgVisual | null): void;
/**
Removes a previously added {@link SvgVisual} from the children collection of this instance.
@param child The child to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisualGroup%23remove(SvgVisual) Online Documentation}*/
remove(child: SvgVisual | null): void;
/**
Gets or sets a transform that is applied to the children of the container.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisualGroup%23transform Online Documentation}*/
transform: Matrix;
/**
Gets the list of children with read and write access.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisualGroup%23children Online Documentation}*/
readonly children: IList<SvgVisual>;
static $class: Class;
private $_SvgVisualGroup: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SvgVisualGroup;
}
export interface ConstrainedPositionHandler extends ConstrainedDragHandler<IPositionHandler>, IPositionHandler {
}
/**
An implementation of the {@link IPositionHandler} interface that can be used to constrain the position of an item.
@see ConstrainedDragHandler
@see ConstrainedDragHandler#constrainNewLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedPositionHandler Online Documentation}*/
export abstract class ConstrainedPositionHandler {
/**
Initializes a new instance of the {@link ConstrainedPositionHandler} class that delegates to the <code>wrappedHandler</code>.
@param wrappedHandler The handler to wrap.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedPositionHandler Online Documentation}*/
constructor(wrappedHandler: IPositionHandler);
static $class: Class;
private $_ConstrainedPositionHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ConstrainedPositionHandler;
}
export interface ConstrainedHandle extends ConstrainedDragHandler<IHandle>, IHandle {
}
/**
An implementation of the {@link IHandle} interface that can be used to constrain the handle of an item.
@see ConstrainedDragHandler
@see ConstrainedDragHandler#constrainNewLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedHandle Online Documentation}*/
export abstract class ConstrainedHandle {
/**
Initializes a new instance of the {@link ConstrainedHandle} class that delegates to the <code>wrappedHandle</code>.
@param wrappedHandle The handle to wrap.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedHandle Online Documentation}*/
constructor(wrappedHandle: IHandle);
/**
Gets the {@link ConstrainedDragHandler#wrappedHandler}'s {@link IHandle#type} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedHandle%23type Online Documentation}*/
readonly type: HandleTypes;
/**
Gets the {@link ConstrainedDragHandler#wrappedHandler}'s {@link IHandle#cursor} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedHandle%23cursor Online Documentation}*/
readonly cursor: Cursor;
static $class: Class;
private $_ConstrainedHandle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ConstrainedHandle;
}
export interface ConstrainedReshapeHandler extends YObject, IReshapeHandler {
}
/**
An implementation of the {@link IReshapeHandler} interface that can be used to constrain the reshaping of an
existing instance conveniently.
@see ConstrainedHandle
@see ConstrainedPositionHandler
@see ConstrainedReshapeHandler#constrainNewBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler Online Documentation}*/
export abstract class ConstrainedReshapeHandler {
/**
Initializes a new instance of the {@link ConstrainedReshapeHandler} class that delegates to the <code>wrappedHandler</code>.
@param wrappedHandler The handler to wrap.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler Online Documentation}*/
constructor(wrappedHandler: IReshapeHandler);
/**
Cancels the move operation and calls {@link ConstrainedReshapeHandler#onCanceled}
@param context The context to retrieve information about the drag from.
@param originalBounds The value of the coordinate of the {@link ConstrainedReshapeHandler#bounds} property at the time of {@link ConstrainedReshapeHandler#initializeReshape}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23cancelReshape(IInputModeContext%2CRect) Online Documentation}*/
cancelReshape(context: IInputModeContext, originalBounds: Rect): void;
/**
Applies the constraints for the new bounds.
This method should return a {@link Rect} which represents the <code>newBounds</code> after applying the constraints.
@param context The context in which the reshape will be performed.
@param originalBounds The value of the {@link ConstrainedReshapeHandler#bounds} property at the time of {@link ConstrainedReshapeHandler#initializeReshape}.
@param newBounds The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
implementation the {@link ConstrainedReshapeHandler#bounds} may or may not be modified to reflect the new value.
@returns The constrained value of <code>newBounds</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23constrainNewBounds(IInputModeContext%2CRect%2CRect) Online Documentation}*/
abstract constrainNewBounds(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): Rect;
/**
Handles the reshape operation and subsequently calls {@link ConstrainedReshapeHandler#onReshaped}
This method will use the {@link ConstrainedReshapeHandler#constrainNewBounds} callback to let subclasses constrain
the reshaping.
@param context The context in which the reshape will be performed.
@param originalBounds The value of the {@link ConstrainedReshapeHandler#bounds} property at the time of {@link ConstrainedReshapeHandler#initializeReshape}.
@param newBounds The coordinates in the world coordinate system that the client wants to reshape the item to. Depending on the
implementation the {@link ConstrainedReshapeHandler#bounds} may or may not be modified to reflect the new value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23handleReshape(IInputModeContext%2CRect%2CRect) Online Documentation}*/
handleReshape(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
/**
Initializes the reshape operation and subsequently calls {@link ConstrainedReshapeHandler#onInitialized}
@param context The context in which the drag will be performed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23initializeReshape(IInputModeContext) Online Documentation}*/
initializeReshape(context: IInputModeContext): void;
/**
Called when the {@link ConstrainedReshapeHandler#cancelReshape} method has been called.
@param context The input mode context.
@param originalBounds The original bounds.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23onCanceled(IInputModeContext%2CRect) Online Documentation}*/
onCanceled(context: IInputModeContext, originalBounds: Rect): void;
/**
Called when the {@link ConstrainedReshapeHandler#reshapeFinished} method has been called.
@param context The input mode context.
@param originalBounds The original bounds.
@param newBounds The new bounds.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23onFinished(IInputModeContext%2CRect%2CRect) Online Documentation}*/
onFinished(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
/**
Called when the {@link ConstrainedReshapeHandler#initializeReshape} method has been called.
@param context The input mode context.
@param originalBounds The original bounds.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23onInitialized(IInputModeContext%2CRect) Online Documentation}*/
onInitialized(context: IInputModeContext, originalBounds: Rect): void;
/**
Called when the {@link ConstrainedReshapeHandler#handleReshape} operation has been performed.
@param context The input mode context.
@param originalBounds The original Bounds.
@param newBounds The new Bounds.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23onReshaped(IInputModeContext%2CRect%2CRect) Online Documentation}*/
onReshaped(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
/**
Handles the finish operation and invokes {@link ConstrainedReshapeHandler#onFinished}
@param context The context to retrieve information about the drag from.
@param originalBounds The value of the {@link ConstrainedReshapeHandler#bounds} property at the time of {@link ConstrainedReshapeHandler#initializeReshape}.
@param newBounds The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
implementation the {@link ConstrainedReshapeHandler#bounds} may or may not be modified to reflect the new value. This is the same value as delivered in the last
invocation of {@link ConstrainedReshapeHandler#handleReshape}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23reshapeFinished(IInputModeContext%2CRect%2CRect) Online Documentation}*/
reshapeFinished(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
/**
Gets the wrapped handler to which calls will be ultimately delegated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23wrappedHandler Online Documentation}*/
readonly wrappedHandler: IReshapeHandler;
/**
Gets the {@link ConstrainedReshapeHandler#wrappedHandler}'s {@link IReshapeHandler#bounds} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedReshapeHandler%23bounds Online Documentation}*/
readonly bounds: IRectangle;
static $class: Class;
private $_ConstrainedReshapeHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ConstrainedReshapeHandler;
}
export interface ConstrainedDragHandler<TWrapped extends IDragHandler> extends YObject, IDragHandler {
}
/**
An implementation of the {@link IDragHandler} interface that can be used to constrain the movement of an existing
instance.
Type parameter <code>TWrapped</code>: The exact type of the wrapped implementation.
@see ConstrainedHandle
@see ConstrainedPositionHandler
@see ConstrainedDragHandler#constrainNewLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler Online Documentation}*/
export abstract class ConstrainedDragHandler<TWrapped extends IDragHandler> {
/**
Initializes a new instance of the {@link ConstrainedDragHandler} class that delegates to the <code>wrappedHandler</code>.
@param wrappedHandler The handler to wrap.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler Online Documentation}*/
constructor(wrappedHandler: TWrapped);
/**
Cancels the move operation and calls {@link ConstrainedDragHandler#onCanceled}
@param context The context to retrieve information about the drag from.
@param originalLocation The value of the coordinate of the {@link ConstrainedDragHandler#location} property at the time of {@link ConstrainedDragHandler#initializeDrag}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23cancelDrag(IInputModeContext%2CPoint) Online Documentation}*/
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
/**
Applies the constraints for the new location.
@param context The context in which the drag will be performed.
@param originalLocation The value of the {@link ConstrainedDragHandler#location} property at the time of {@link ConstrainedDragHandler#initializeDrag}.
@param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
implementation the {@link ConstrainedDragHandler#location} may or may not be modified to reflect the new value.
@returns The constrained value of <code>newLocation</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23constrainNewLocation(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
abstract constrainNewLocation(context: IInputModeContext, originalLocation: Point, newLocation: Point): Point;
/**
Handles the finish operation and invokes {@link ConstrainedDragHandler#onFinished}
@param context The context to retrieve information about the drag from.
@param originalLocation The value of the {@link ConstrainedDragHandler#location} property at the time of {@link ConstrainedDragHandler#initializeDrag}.
@param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
implementation the {@link ConstrainedDragHandler#location} may or may not be modified to reflect the new value. This is the same value as delivered in the last
invocation of {@link ConstrainedDragHandler#handleMove}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23dragFinished(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
Handles the move operation and subsequently calls {@link ConstrainedDragHandler#onMoved}
This method will use the {@link ConstrainedDragHandler#constrainNewLocation} callback to let subclasses constrain the movement.
@param context The context in which the drag will be performed.
@param originalLocation The value of the {@link ConstrainedDragHandler#location} property at the time of {@link ConstrainedDragHandler#initializeDrag}.
@param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
implementation the {@link ConstrainedDragHandler#location} may or may not be modified to reflect the new value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23handleMove(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
Initializes the drag operation and subsequently calls {@link ConstrainedDragHandler#onInitialized}
@param context The context in which the drag will be performed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23initializeDrag(IInputModeContext) Online Documentation}*/
initializeDrag(context: IInputModeContext): void;
/**
Called when the {@link ConstrainedDragHandler#cancelDrag} method has been called.
@param context The input mode context.
@param originalLocation The original location.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23onCanceled(IInputModeContext%2CPoint) Online Documentation}*/
onCanceled(context: IInputModeContext, originalLocation: Point): void;
/**
Called when the {@link ConstrainedDragHandler#dragFinished} method has been called.
@param context The input mode context.
@param originalLocation The original location.
@param newLocation The new location.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23onFinished(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
onFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
Called when the {@link ConstrainedDragHandler#initializeDrag} method has been called.
@param context The input mode context.
@param originalLocation The original location.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23onInitialized(IInputModeContext%2CPoint) Online Documentation}*/
onInitialized(context: IInputModeContext, originalLocation: Point): void;
/**
Called when the {@link ConstrainedDragHandler#handleMove} operation has been performed.
@param context The input mode context.
@param originalLocation The original location.
@param newLocation The new location.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23onMoved(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
onMoved(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
Gets the wrapped handler to which calls will be ultimately delegated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23wrappedHandler Online Documentation}*/
readonly wrappedHandler: TWrapped;
/**
Gets the {@link ConstrainedDragHandler#wrappedHandler}'s {@link IDragHandler#location} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstrainedDragHandler%23location Online Documentation}*/
readonly location: IPoint;
static $class: Class;
private $_ConstrainedDragHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TWrapped extends IDragHandler>(o: any): o is ConstrainedDragHandler<TWrapped>;
}
/**
An enumeration of all possible types that make up a {@link GeneralPath}
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathType Online Documentation}*/
export enum PathType {
/**
The type that is used to move the {@link Stroke} to the next position.
This type uses one coordinate pair.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathType%23MOVE_TO Online Documentation}*/
MOVE_TO,
/**
The type that is used to add a line from the current position to the next position.
This type uses one coordinate pair.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathType%23LINE_TO Online Documentation}*/
LINE_TO,
/**
The type that is used to add a quadratic Bézier curve from the current position to the next position using one control
point.
This type uses two coordinate pairs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathType%23QUAD_TO Online Documentation}*/
QUAD_TO,
/**
The type that is used to add a cubic Bézier curve from the current position to the next position using two intermediate
control points.
This type uses three coordinate pairs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathType%23CUBIC_TO Online Documentation}*/
CUBIC_TO,
/**
The type that is used to add a line from the current position to the last {@link PathType#MOVE_TO} position.
This type uses no coordinate pair.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathType%23CLOSE Online Documentation}*/
CLOSE
}
/** Automatic conversion to {@link PathType}. */
type PathTypeStringValues = "move-to" | "line-to" | "quad-to" | "cubic-to" | "close";
export interface GeneralPathCursor extends YObject {
}
/**
A class that is used by {@link GeneralPath} to iterate over the elements the path is made up of.
@see GeneralPath#createCursor
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor Online Documentation}*/
export class GeneralPathCursor {
/**
Places the coordinates for the current type into the given array.
The array needs to have a length of at least 6, since {@link PathType#CUBIC_TO} needs six coordinates. If the
path has been flattened a length of two suffices.
@param coordinates
@returns The current type of the path element.
@see GeneralPathCursor#getCurrentEndPoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23getCurrent(Array%3Cnumber%3E) Online Documentation}*/
getCurrent(coordinates: number[]): PathType;
/**
Places the current end coordinates of the last path element into the array.
@param coordinates An array with length at least 2.
@returns The type of the current element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23getCurrentEndPoint(Array%3Cnumber%3E) Online Documentation}*/
getCurrentEndPoint(coordinates: number[]): PathType;
/**
Advances the cursor to the next position if possible.
@returns Whether the cursor has been moved successfully. <code>false</code> if the cursor has been moved beyond the end of the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23moveNext() Online Documentation}*/
moveNext(): boolean;
/**
Moves the cursor to the previous position in the path.
@returns Whether the move was successful.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23movePrevious() Online Documentation}*/
movePrevious(): boolean;
/**
Reset the cursor to point to the beginning of the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23reset() Online Documentation}*/
reset(): void;
/**
Resets the cursor to point to the last element in the path.
@returns Whether the cursor has been moved successfully.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23toLast() Online Documentation}*/
toLast(): boolean;
/**
Gets the current position of the cursor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23index Online Documentation}*/
readonly index: number;
/**
Gets the type of the path at the position of the cursor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23pathType Online Documentation}*/
readonly pathType: PathType;
/**
Gets the current x coordinate of the last path element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23currentEndPointX Online Documentation}*/
readonly currentEndPointX: number;
/**
Gets the current end point of the last path element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23currentEndPoint Online Documentation}*/
readonly currentEndPoint: Point;
/**
Gets the current y coordinate of the last path element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathCursor%23currentEndPointY Online Documentation}*/
readonly currentEndPointY: number;
static $class: Class;
private $_GeneralPathCursor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GeneralPathCursor;
}
export interface GeneralPath extends YObject, ICloneable {
}
/**
Models a path that consists of lines and Bézier curves.
This class uses double precision coordinates to model a virtual pen that can be moved across the world coordinate system
and which can be used to draw lines and Bézier curves. This class provides many geometric utility methods, such as
calculating the bounds, hit tests, containment tests, curve flattening, path transformation, path modification, and path
iteration.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath Online Documentation}*/
export class GeneralPath {
/**
Creates a new instance with given initial capacity.
@param [capacity=16] The number of elements for which storage will be allocated initially.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23GeneralPath(number) Online Documentation}*/
constructor(capacity?: number);
/**
Appends parts of the contents of the given path to this path.
If <code>connect</code> is <code>true</code>, an initial {@link PathType#MOVE_TO} in <code>other</code> will be transformed into a
{@link PathType#LINE_TO}.
@param other The path to append to this path.
@param connect Whether to connect the path to the end of this path.
@param beginIndex The first element to be copied from <code>other</code>, inclusively.
@param endIndex The last element to be copied from <code>other</code>, exclusively.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23append(GeneralPath%2Cnumber%2Cnumber%2Cboolean) Online Documentation}*/
append(other: GeneralPath, beginIndex: number, endIndex: number, connect: boolean): void;
/**
Appends the contents of the given path to this path.
If <code>connect</code> is <code>true</code>, an initial {@link PathType#MOVE_TO} in <code>other</code> will be transformed into a
{@link PathType#LINE_TO}.
@param other The path to append to this path.
@param connect Whether to connect the path to the end of this path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23append(GeneralPath%2Cboolean) Online Documentation}*/
append(other: GeneralPath, connect: boolean): void;
/**
Appends an ellipse using the given bounding box to this instance.
@param bounds The bounds of the ellipse.
@param connect Whether to initially {@link GeneralPath#lineTo} the ellipses lowest point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23appendEllipse(IRectangle%2Cboolean) Online Documentation}*/
appendEllipse(bounds: IRectangle, connect: boolean): void;
/**
Appends an {@link IOrientedRectangle} to this path instance.
@param rectangle The {@link IOrientedRectangle} to get the coordinates from.
@param connect Whether to use a {@link GeneralPath#lineTo} to the anchor corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23appendOrientedRectangle(IOrientedRectangle%2Cboolean) Online Documentation}*/
appendOrientedRectangle(rectangle: IOrientedRectangle, connect: boolean): void;
/**
Appends a rectangle to this path instance.
@param rectangle The rectangle to get the coordinates from.
@param connect Whether to use a {@link GeneralPath#lineTo} to the upper left corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23appendRectangle(IRectangle%2Cboolean) Online Documentation}*/
appendRectangle(rectangle: IRectangle, connect: boolean): void;
/**
Checks whether the point lies inside the area that is described by this path instance.
Note that this will only work as expected for closed paths. The epsilon is used for internally flattening Bézier curves.
Note that for points residing exactly on the path this may return false due to rounding issues, consider also checking
for {@link GeneralPath#pathContains} in this case.
@param point The coordinates of the point to test.
@param [eps=0.5] The value to use for flattening Bézier curves during the test. <code>0.5</code> is the default value.
@returns Whether the point lies within the area described by this path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23areaContains(IPoint%2Cnumber) Online Documentation}*/
areaContains(point: IPoint, eps?: number): boolean;
/**
Determines whether the area formed by this instance intersects with the area formed by the given path instance.
@param other The other instance to check against.
@param [eps=0.5] The epsilon value that is used for the accuracy with which the implementation should test Bézier curves within the path
for intersections. The default value is <code>0.5</code>.
@returns Whether the filled area of this path intersects with the stroke or filled area of the given path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23areaIntersects(GeneralPath%2Cnumber) Online Documentation}*/
areaIntersects(other: GeneralPath, eps?: number): boolean;
/**
Resets this path to be an empty path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23clear() Online Documentation}*/
clear(): void;
/**
Creates a clone of this instance, copying the path information to the new instance.
@returns An exact clone of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23clone() Online Documentation}*/
clone(): this;
/**
Appends a {@link PathType#CLOSE} operation to the path elements, creating a line to the last {@link PathType#MOVE_TO}
position.
This draws a line from the current end point to the last {@link GeneralPath#moveTo} position, closing the
current sub path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23close() Online Documentation}*/
close(): void;
/**
Compresses the internal data structures so that the amount of memory used by this instance is minimal with respect to
the information stored in it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23compress() Online Documentation}*/
compress(): void;
/**
Creates a cursor for iterating over the elements of this path.
The cursor is fail-fast, i.e. if the path's structure is modified after the construction of the cursor any cursor
operation will fail.
@returns A cursor to iterate over this path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23createCursor() Online Documentation}*/
createCursor(): GeneralPathCursor;
/**
Creates a new path based on this one where corners are smoothed with a quadratic Bézier arc starting at <code>smoothingLength</code>
away from the corner.
@param smoothingLength Distance from the corner where the arc begins.
@param [smoothingPolicy=SmoothingPolicy.ASYMMETRIC] A {@link SmoothingPolicy} value specifying how corners should be smoothed.
@param [straightEnds=true] A value indicating whether the first and last segment of a path figure should end with a straight line segment, or
whether the adjacent corner can be smoothed up to the very end of the path (if <code>smoothingLength</code> is large enough).
@returns A new path, with its corners smoothed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23createSmoothedPath(number%2CSmoothingPolicy%2Cboolean) Online Documentation}*/
createSmoothedPath(smoothingLength: number, smoothingPolicy?: SmoothingPolicy | SmoothingPolicyStringValues, straightEnds?: boolean): GeneralPath;
/**
Creates a new path based on this one where corners are smoothed with a quadratic Bézier arc starting at <code>smoothingLength</code>
away from the corner.
@param options The parameters to pass.
@param options.smoothingLength Distance from the corner where the arc begins.
@param [options.smoothingPolicy=SmoothingPolicy.ASYMMETRIC] A {@link SmoothingPolicy} value specifying how corners should be smoothed.
@param [options.straightEnds=true] A value indicating whether the first and last segment of a path figure should end with a straight line segment, or
whether the adjacent corner can be smoothed up to the very end of the path (if <code>smoothingLength</code> is large enough).
@returns A new path, with its corners smoothed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23createSmoothedPath(number%2CSmoothingPolicy%2Cboolean) Online Documentation}*/
createSmoothedPath(options: {
smoothingLength: number;
smoothingPolicy?: SmoothingPolicy | SmoothingPolicyStringValues;
straightEnds?: boolean;
}): GeneralPath;
/**
Creates an SVG <code>path</code> element using the provided parameters.
@param [transform=null] The matrix to apply to the path data.
@returns A {@link SVGPathElement path} that displays this instance using the provided attributes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23createSvgPath(Matrix) Online Documentation}*/
createSvgPath(transform?: Matrix | null): SVGPathElement;
/**
Creates the data for a SVG <code>path</code> element that corresponds to this instance with the provided transform.
@param [transform=null] The transform to apply to the created data.
@returns The path data for an SVG <code>path</code> element that corresponds to this instance with the provided transform.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23createSvgPathData(Matrix) Online Documentation}*/
createSvgPathData(transform?: Matrix | null): string;
/**
Creates a new {@link GeneralPath} that is a copy of this path with all geometry transformed using the supplied
matrix.
@param transform The matrix to multiply the geometry with.
@returns A new path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23createTransformedPath(Matrix) Online Documentation}*/
createTransformedPath(transform: Matrix): GeneralPath;
/**
Crops the path after the position indicated by the cursor.
This will remove all path operations that follow the position indicated by the cursor.
@param cursor The cursor that depicts a position in this path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23cropAfter(GeneralPathCursor) Online Documentation}*/
cropAfter(cursor: GeneralPathCursor): void;
/**
Crops the path before the position indicated by the cursor.
This will remove all path operations that precede the position indicated by the cursor.
@param cursor The cursor that depicts a position in this path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23cropBefore(GeneralPathCursor) Online Documentation}*/
cropBefore(cursor: GeneralPathCursor): void;
/**
Appends a {@link PathType#CUBIC_TO cubic Bézier curve} to the path elements.
This draws a cubic Bézier curve from the current end point to the given position using the two given control points.
@param c1 The coordinates of the first Bézier control point.
@param c2 The coordinates of the second Bézier control point.
@param end The coordinates of the curve's end point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23cubicTo(IPoint%2CIPoint%2CIPoint) Online Documentation}*/
cubicTo(c1: IPoint, c2: IPoint, end: IPoint): void;
/**
Appends a {@link PathType#CUBIC_TO cubic Bézier curve} to the path elements.
This draws a cubic Bézier curve from the current end point to the given position using the two given control points.
@param c1x The x coordinate of the first Bézier control point.
@param c1y The y coordinate of the first Bézier control point.
@param c2x The x coordinate of the second Bézier control point.
@param c2y The y coordinate of the second Bézier control point.
@param x The x coordinate of the curve's end point.
@param y The y coordinate of the curve's end point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23cubicTo(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
cubicTo(c1x: number, c1y: number, c2x: number, c2y: number, x: number, y: number): void;
/**
Finds an intersection point between a line and this {@link GeneralPath}.
The value returned will be the smallest positive value smaller than 1 such that the point depicted by <code>start + result * (end - start)</code>
is an intersection point between the line and the path.
@param start The coordinates of the first point of the line.
@param end The coordinates of the second point of the line.
@param [eps=0.5] The value to use for interpolating Bézier curves.
@returns The factor to calculate the intersection point or <code>double.PositiveInfinity</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23findLineIntersection(Point%2CPoint%2Cnumber) Online Documentation}*/
findLineIntersection(start: Point, end: Point, eps?: number): number;
/**
Finds the first intersection of a ray with this {@link GeneralPath} using a given epsilon for flattening Bézier
curves contained in the path.
The ray is treated like an infinite line starting at the given anchor point and continuing in the direction of rayX,
rayY. The ray vector does not need to be normalized. The value returned will be the smallest positive value such that
the point depicted by <code>(anchorX + result * rayX, anchorY + result * rayY)</code> is an intersection point between the ray and
the path.
@param anchor The coordinates of the anchor point of the ray.
@param direction The coordinates of the direction vector of the ray.
@param [eps=0.5] The epsilon value that is used for the accuracy with which the implementation should test Bézier curves within the path
for intersections.
@returns The factor to calculate the intersection point or <code>double.PositiveInfinity</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23findRayIntersection(Point%2CPoint%2Cnumber) Online Documentation}*/
findRayIntersection(anchor: Point, direction: Point, eps?: number): number;
/**
Creates a new path from this instance flattening all Bézier curves using the given epsilon.
This will create a path that consists of {@link PathType#MOVE_TO}, {@link PathType#LINE_TO}, and {@link PathType#CLOSE}
operations only.
@param eps The maximum value the flattened path may diverge from the original path for the Bézier curve line interpolations.
@returns A new path that does not contain Bézier curves.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23flatten(number) Online Documentation}*/
flatten(eps: number): GeneralPath;
/**
Gets the bounds of a flattened version the path.
@param eps The epsilon to use for the flattening operation.
@returns The bounds of the flattened path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23getBounds(number) Online Documentation}*/
getBounds(eps: number): Rect;
/**
Calculates the bounds of this path and returns it.
This will not yield tight bounds if the path contains Bézier curves. The bounds are calculated using the union of all
points and control points contained in this path.
@returns A rectangle that describes the geometric bounds of this path.
@see GeneralPath#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23getBounds() Online Documentation}*/
getBounds(): Rect;
/**
Determines new control points for splitting a cubic Bézier segment.
@param cp0 The start point of the segment
@param cp1 The first control point of the segment
@param cp2 The second control point of the segment
@param cp3 The control points of the cubic segment
@param splitBefore A value indicating whether the new control points for the segment before or after the cut should be returned
@param t The parameter for De Casteljau's algorithm to determine the splitting point in the interval [0..1].
@returns An array of length 4 with the new start, end, and control points for the segment either preceding or following the
desired split point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23getCubicSplitPoints(Point%2CPoint%2CPoint%2CPoint%2Cboolean%2Cnumber) Online Documentation}*/
static getCubicSplitPoints(cp0: Point, cp1: Point, cp2: Point, cp3: Point, splitBefore: boolean, t: number): Point[];
/**
Returns the geometric length of this path.
Note that Bézier curves are still treated as line segments.
@returns The length of this path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23getLength() Online Documentation}*/
getLength(): number;
/**
Gets a point on this path instance at the given ratio.
Note that this implementation still treats Bézier curves as linear segments.
@param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of this path.
@returns The coordinates of the point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23getPoint(number) Online Documentation}*/
getPoint(ratio: number): Point;
/**
Gets the projection of <code>p</code> onto the segment <code>segmentIndex</code>
This method does not directly return the projection point, but the parameter ratio for the segment, or <code>null</code> if no
projection on the segment could be found within the interval [0,1]. To determine the actual projection point, you can
use the ratio as input for {@link GeneralPath#getTangentForSegment}
Note that this implementation still treats quadratic as linear segments.
@param p The point which should be projected onto the segment
@param segmentIndex The segment index. Not that only linear, cubic or close segments are considered.
@returns A ratio describing the projection onto the segement, or <code>null</code> if no such projection exists in the interval [0,1].
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23getProjection(Point%2Cnumber) Online Documentation}*/
getProjection(p: Point, segmentIndex: number): number | null;
/**
Gets the number of segments in this path, that is the number of {@link PathType#LINE_TO} and {@link PathType#CLOSE}
operations.
@returns The number of segments.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23getSegmentCount() Online Documentation}*/
getSegmentCount(): number;
/**
Gets the tangent on this path instance at the given ratio.
Note that this implementation still treats Bézier curves as linear segments.
@param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of this path.
@returns The tangent, if any exists; <code>null</code> otherwise. The tangent vector may not be normalized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23getTangent(number) Online Documentation}*/
getTangent(ratio: number): Tangent | null;
/**
Gets the tangent on this path instance at the segment and segment ratio.
Note that this implementation still treats quadratic curves as linear segments.
@param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of the segment at segmentIndex
for this path.
@param segmentIndex The segment index to determine a point at.
@returns The tangent, if any exists; <code>null</code> otherwise. The tangent vector may not be normalized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23getTangentForSegment(number%2Cnumber) Online Documentation}*/
getTangentForSegment(segmentIndex: number, ratio: number): Tangent | null;
/**
Determines whether a given {@link GeneralPath} instance can be considered equivalent to this one, that is, both
represent the same path with the same sub-path types and coordinates.
@param path The other {@link GeneralPath} instance to compare.
@returns <code>true</code> if <code>path</code> and this instance have the same sub-path types and coordinates, <code>false</code> if not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23hasSameValue(GeneralPath) Online Documentation}*/
hasSameValue(path: GeneralPath | null): boolean;
/**
Determines whether this path's stroked segments intersects with a given path.
If one path is contained within an enclosed area of the other but the stroked segments do not intersect, this method
will return false.
@param other The other path to check against this path.
@param [eps=0.5] The epsilon value that is used for the accuracy with which the implementation should test Bézier curves within the path
for intersections.
@returns True if this path's stroked segments intersects the other path's stroked segments.
@see GeneralPath#areaIntersects
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23intersects(GeneralPath%2Cnumber) Online Documentation}*/
intersects(other: GeneralPath, eps?: number): boolean;
/**
Checks whether this path intersects the given rectangle.
This will yield false if the rectangle is fully contained within the area of the path.
@param rectangle The rectangle to test.
@param eps The epsilon to use for fuzzy testing.
@returns Whether the path described by this instance intersects the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23intersects(Rect%2Cnumber) Online Documentation}*/
intersects(rectangle: Rect, eps: number): boolean;
/**
Appends a {@link PathType#LINE_TO} operation to the path elements.
This draws a line from the current end point to the given position.
@param point The coordinates of the line's end point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23lineTo(IPoint) Online Documentation}*/
lineTo(point: IPoint): void;
/**
Appends a {@link PathType#LINE_TO} operation to the path elements.
This draws a line from the current end point to the given position.
@param x The x coordinate of the line's end point.
@param y The y coordinate of the line's end point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23lineTo(number%2Cnumber) Online Documentation}*/
lineTo(x: number, y: number): void;
/**
Performs a quick test to decide whether this path might intersect the clipping rectangle, provided the path would be
rendered using stroke of the given width.
This method will return false if the clip is entirely inside the area of this path.
@param clip The clip to check for intersection.
@param width The width of the stroke.
@returns <code>false</code> if it is guaranteed that this instance would not intersect the given clip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23mayIntersectClip(Rect%2Cnumber) Online Documentation}*/
mayIntersectClip(clip: Rect, width: number): boolean;
/**
Appends a {@link PathType#MOVE_TO} operation to the path elements.
This moves the pen to a new position without drawing a line.
@param point The next coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23moveTo(IPoint) Online Documentation}*/
moveTo(point: IPoint): void;
/**
Appends a {@link PathType#MOVE_TO} operation to the path elements.
This moves the pen to a new position without drawing a line.
@param x The next x coordinate.
@param y The next y coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23moveTo(number%2Cnumber) Online Documentation}*/
moveTo(x: number, y: number): void;
/**
Tests whether the line of the path is hit at the given point using an epsilon for fuzzy hit testing.
@param point The p coordinates of the point to test.
@param [eps=0.5] The allowed distance from the point to the path that is considered a hit.<code>0.5</code> is the default value.
@returns Whether the point hits the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23pathContains(IPoint%2Cnumber) Online Documentation}*/
pathContains(point: IPoint, eps?: number): boolean;
/**
Adds the contents of the given path to the beginning of this path.
If <code>connect</code> is <code>true</code>, the initial {@link PathType#MOVE_TO} in <code>this</code> will be transformed into a
{@link PathType#LINE_TO}.
@param other The path to append to this path.
@param connect Whether to connect the path to the end of this path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23prepend(GeneralPath%2Cboolean) Online Documentation}*/
prepend(other: GeneralPath, connect: boolean): void;
/**
Appends a {@link PathType#QUAD_TO quadratic Bézier curve} to the path elements.
This draws a quadratic Bézier curve from the current end point to the given position using the given control point.
@param center The coordinates of the Bézier control point.
@param point The coordinates of the curve's end point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23quadTo(IPoint%2CIPoint) Online Documentation}*/
quadTo(center: IPoint, point: IPoint): void;
/**
Appends a {@link PathType#QUAD_TO quadratic Bézier curve} to the path elements.
This draws a quadratic Bézier curve from the current end point to the given position using the given control point.
@param cx The x coordinate of the Bézier control point.
@param cy The y coordinate of the Bézier control point.
@param x The x coordinate of the curve's end point.
@param y The y coordinate of the curve's end point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23quadTo(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
quadTo(cx: number, cy: number, x: number, y: number): void;
/**
Transforms this {@link GeneralPath} in place using the given transform.
@param transform The matrix to multiply the geometry with.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23transform(Matrix) Online Documentation}*/
transform(transform: Matrix): void;
/**
Gets the last x coordinate of the last path element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23lastX Online Documentation}*/
readonly lastX: number;
/**
Gets the last coordinates of the last path element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23lastCoordinate Online Documentation}*/
readonly lastCoordinate: Point;
/**
Gets the last y coordinate of the last path element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23lastY Online Documentation}*/
readonly lastY: number;
/**
Gets a value indicating whether this path contains elements other than an initial {@link PathType#MOVE_TO}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23isEmpty Online Documentation}*/
readonly isEmpty: boolean;
/**
Gets a value indicating whether the path contains visible parts.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23isVisible Online Documentation}*/
readonly isVisible: boolean;
/**
Gets the number of {@link PathType} operations stored in this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPath%23size Online Documentation}*/
readonly size: number;
static $class: Class;
private $_GeneralPath: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GeneralPath;
}
/**
Policy determining how corners in paths are smoothed in {@link GeneralPath#createSmoothedPath}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmoothingPolicy Online Documentation}*/
export enum SmoothingPolicy {
/**
For each corner the smoothing length can be different on both sides, for example when the adjacent segments have
different lengths.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmoothingPolicy%23ASYMMETRIC Online Documentation}*/
ASYMMETRIC,
/**
Limits the smoothing length to the same value on both sides of a corner to avoid lopsided curves around a mixture of
short and long segments.
This improves smoothing results especially with large smoothing lengths on paths that have very short and very long
segments adjacent to each other.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmoothingPolicy%23SYMMETRIC Online Documentation}*/
SYMMETRIC
}
/** Automatic conversion to {@link SmoothingPolicy}. */
type SmoothingPolicyStringValues = "asymmetric" | "symmetric";
export interface GridVisualCreator extends YObject, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
* Renders a grid, using various different {@link GridVisualCreator#gridStyle styles} and {@link GridVisualCreator#renderMode render technologies}.
* The {@link RenderModes#SVG} {@link GridVisualCreator#renderMode} provides the following CSS classes to conveniently change the color of the grid
* visual:
*
* <ul>
* <li><code>yfiles-grid</code> for any {@link GridStyle}</li>
* <li><code>yfiles-grid-dots</code> for {@link GridStyle#DOTS}</li>
* <li><code>yfiles-grid-crosses</code> for {@link GridStyle#CROSSES}</li>
* <li><code>yfiles-grid-lines</code> for {@link GridStyle#LINES}</li>
* </ul>
* Enabling grid snapping
*
* <pre>
* const snapContext = new GraphSnapContext()
*
* const gridInfo = new GridInfo(50)
* snapContext.nodeGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.bendGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.portGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.gridSnapType = GridSnapTypes.GRID_POINTS
*
* graphEditorInputMode.snapContext = snapContext
* </pre>
*
* Note that the grid visualization has to be enabled separately
*
* <pre>
* graphComponent.backgroundGroup.addChild(new GridVisualCreator(gridInfo))
* </pre>
*
* @see GridInfo
* @see GridVisualCreator#gridStyle
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator Online Documentation}
*/
export class GridVisualCreator {
/**
Initializes a new instance of the {@link GridVisualCreator} class.
@param gridInfo The grid info.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator%23GridVisualCreator(GridInfo) Online Documentation}*/
constructor(gridInfo: GridInfo);
/**
Initializes a new instance of the {@link GridVisualCreator} class.
This initializes the {@link GridVisualCreator#gridInfo} field using the given values.
@param horizontalSpacing The x spacing.
@param verticalSpacing The y spacing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator%23GridVisualCreator(number%2Cnumber) Online Documentation}*/
constructor(horizontalSpacing: number, verticalSpacing: number);
/**
Initializes a new instance of the {@link GridVisualCreator} class.
The spacing is initialized to 25 units.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator%23GridVisualCreator() Online Documentation}*/
constructor();
/**
Gets or sets the mode used to render the grid.
@default RenderModes.SVG
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator%23renderMode Online Documentation}*/
renderMode: RenderModes;
/**
Gets or sets the grid style that this implementation uses.
@default GridStyle.DOTS
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator%23gridStyle Online Documentation}*/
gridStyle: GridStyle;
/**
Gets or sets the grid info that describes the geometry of the grid.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator%23gridInfo Online Documentation}*/
gridInfo: GridInfo;
/**
Gets or sets a value indicating whether this {@link GridVisualCreator} is visible.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator%23visible Online Documentation}*/
visible: boolean;
/**
Gets or sets the {@link GridVisualCreator#stroke} to use for painting the grid.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator%23stroke Online Documentation}*/
stroke: Stroke;
/**
Gets or sets the visibility threshold.
This indicates the minimum distance in the view coordinate system between two grid points. If the distance becomes
smaller, the grid will not be rendered. The default value is <code>20.0d</code>.
@default 20.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridVisualCreator%23visibilityThreshold Online Documentation}*/
visibilityThreshold: number;
static $class: Class;
private $_GridVisualCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GridVisualCreator;
}
/**
The different possible styles supported by {@link GridVisualCreator}'s {@link GridVisualCreator#gridStyle} property.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridStyle Online Documentation}*/
export enum GridStyle {
/**
For each grid point, there is a small dot rendered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridStyle%23DOTS Online Documentation}*/
DOTS,
/**
For each grid point, there is a small cross rendered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridStyle%23CROSSES Online Documentation}*/
CROSSES,
/**
Draws horizontal and vertical lines through the grid points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridStyle%23LINES Online Documentation}*/
LINES,
/**
Only horizontal lines are rendered.
This is mostly useful for {@link GridSnapTypes#HORIZONTAL_LINES} so snapping aligns with the grid visualization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridStyle%23HORIZONTAL_LINES Online Documentation}*/
HORIZONTAL_LINES,
/**
Only vertical lines are rendered.
This is mostly useful for {@link GridSnapTypes#VERTICAL_LINES} so snapping aligns with the grid visualization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridStyle%23VERTICAL_LINES Online Documentation}*/
VERTICAL_LINES
}
/** Automatic conversion to {@link GridStyle}. */
type GridStyleStringValues = "dots" | "crosses" | "lines" | "horizontal-lines" | "vertical-lines";
export interface IBoundsProvider {
}
/**
Interface that is implemented for elements that can provide rendering bounds in the world coordinate system.
This information can be used by the rendering engine to increase rendering performance significantly.
@see IVisualCreator
@see CanvasComponent
@see ICanvasObjectDescriptor
@see ICanvasObject
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBoundsProvider Online Documentation}*/
export abstract class IBoundsProvider {
/**
Creates an implementation of the interface {@link IBoundsProvider} by using the given function as implementation for
its {@link IBoundsProvider#getBounds} method.
@param getBounds A function for {@link IBoundsProvider}'s single abstract method {@link IBoundsProvider#getBounds}.
@returns An instance of the {@link IBoundsProvider} interface based on the given function.*/
static create(getBounds: (context: ICanvasContext) => Rect): IBoundsProvider;
/**
Creates an instance that returns the given model as the bounds.
@param bounds the rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBoundsProvider%23fromRectangle(IRectangle) Online Documentation}*/
static fromRectangle(bounds: IRectangle | IRectangleConvertible): IBoundsProvider;
/**
Returns a tight rectangular area where the whole rendering would fit into.
If calculating the bounds is too expensive or the rendering is not bound to a certain area, this method may return
{@link Rect#INFINITE}. If nothing is rendered, this method should return an empty rectangle, where either or
both the width and height is non-positive or {@link Rect#EMPTY}.
@param context the context to calculate the bounds for
@returns the bounds or {@link Rect#EMPTY} to indicate an unbound area
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBoundsProvider%23getBounds(ICanvasContext) Online Documentation}*/
abstract getBounds(context: ICanvasContext): Rect;
/**
An implementation that returns an {@link IBoundsProvider} that yields an "Empty" bounds.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBoundsProvider%23EMPTY Online Documentation}*/
static EMPTY: IBoundsProvider;
/**
An implementation that returns an {@link IBoundsProvider} that yields {@link Rect#INFINITE "infinite"} bounds.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBoundsProvider%23UNBOUNDED Online Documentation}*/
static UNBOUNDED: IBoundsProvider;
static $class: Class;
private $_IBoundsProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IBoundsProvider;
}
export interface IVisibilityTestable {
}
/**
Interface that is implemented for elements that can determine whether they might be visible in a given clipping
rectangle.
@see IVisualCreator
@see CanvasComponent
@see ICanvasObjectDescriptor
@see ICanvasObject
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisibilityTestable Online Documentation}*/
export abstract class IVisibilityTestable {
/**
Creates an implementation of the interface {@link IVisibilityTestable} by using the given function as implementation
for its {@link IVisibilityTestable#isVisible} method.
@param isVisible A function for {@link IVisibilityTestable}'s single abstract method {@link IVisibilityTestable#isVisible}.
@returns An instance of the {@link IVisibilityTestable} interface based on the given function.*/
static create(isVisible: (context: ICanvasContext, rectangle: Rect) => boolean): IVisibilityTestable;
/**
Determines whether an element might intersect the visible region for a given context.
Conservative implementations can always return <code>true</code>.
@param context The context to determine the visibility for.
@param rectangle The visible region clip.
@returns <code>false</code> if and only if it is safe not to paint the element because it would not affect the given clipping region.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisibilityTestable%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
abstract isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
An implementation of a {@link IVisibilityTestable} that always yields <code>true</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisibilityTestable%23ALWAYS Online Documentation}*/
static ALWAYS: IVisibilityTestable;
/**
An implementation of a {@link IVisibilityTestable} that always yields <code>false</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisibilityTestable%23NEVER Online Documentation}*/
static NEVER: IVisibilityTestable;
static $class: Class;
private $_IVisibilityTestable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IVisibilityTestable;
}
export interface IHitTestable {
}
/**
Interface that is implemented for elements that can be hit or picked in a coordinate system.
@see CanvasComponent
@see ICanvasObjectDescriptor
@see ICanvasObject
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHitTestable Online Documentation}*/
export abstract class IHitTestable {
/**
Wrap a handler into an interface.
@param handler
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHitTestable%23create(function(IInputModeContext%2CPoint)%3Aboolean) Online Documentation}*/
static create(handler: (context: IInputModeContext, location: Point) => boolean): IHitTestable;
/**
Determines if something has been hit at the given coordinates in the world coordinate system.
Implementations should inspect the {@link ICanvasContext#hitTestRadius} and take the value into account when performing hit tests. This allows the user to
hit elements even if the zoom level is very small and allows for accurate hit tests at greater zoom levels.
@param context the context the hit test is performed in
@param location the coordinates in world coordinate system
@returns whether something has been hit
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHitTestable%23isHit(IInputModeContext%2CPoint) Online Documentation}*/
abstract isHit(context: IInputModeContext, location: Point): boolean;
/**
An {@link IHitTestable} that always returns false.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHitTestable%23NEVER Online Documentation}*/
static NEVER: IHitTestable;
/**
An {@link IHitTestable} that always returns true.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHitTestable%23ALWAYS Online Documentation}*/
static ALWAYS: IHitTestable;
static $class: Class;
private $_IHitTestable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IHitTestable;
}
export interface ILassoTestable {
}
/**
Interface that is used to test whether a rendered item is deemed to be inside a lasso selection path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILassoTestable Online Documentation}*/
export abstract class ILassoTestable {
/**
Creates an implementation of the interface {@link ILassoTestable} by using the given function as implementation for
its {@link ILassoTestable#isInPath} method.
@param isInPath A function for {@link ILassoTestable}'s single abstract method {@link ILassoTestable#isInPath}.
@returns An instance of the {@link ILassoTestable} interface based on the given function.*/
static create(isInPath: (context: IInputModeContext, lassoPath: GeneralPath) => boolean): ILassoTestable;
/**
Returns <code>true</code> if the corresponding item is considered to intersect the given path.
This method may return <code>false</code> if the item cannot be selected using a lasso selection or optionally if the item is only
partially contained within the path.
Implementations should respect the {@link ICanvasContext#hitTestRadius} if lasso selections should behave differently on different zoom levels.
@param context the current input mode context
@param lassoPath the path describing the lasso selection bounds
@returns <code>true</code> if the item is considered to be captured by the lasso
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILassoTestable%23isInPath(IInputModeContext%2CGeneralPath) Online Documentation}*/
abstract isInPath(context: IInputModeContext, lassoPath: GeneralPath): boolean;
static $class: Class;
private $_ILassoTestable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILassoTestable;
}
export interface LassoTestables extends YObject {
}
/**
Provides simple {@link ILassoTestable} implementations.
@see LassoTestables#ALWAYS
@see LassoTestables#NEVER
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoTestables Online Documentation}*/
export class LassoTestables {
/**
Creates an implementation of the {@link ILassoTestable} interface that tests if the <code>orientedRectangle</code> intersects
the area of the lasso path.
@param orientedRectangle The oriented rectangle to check.
@returns An implementation of {@link ILassoTestable} for the oriented rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoTestables%23fromOrientedRectangle(IOrientedRectangle) Online Documentation}*/
static fromOrientedRectangle(orientedRectangle: IOrientedRectangle): ILassoTestable;
/**
Creates an implementation of the {@link ILassoTestable} interface that tests if the <code>point</code> is {@link GeneralPath#areaContains contained} in the lasso
path.
@param point The point to check.
@returns An implementation of {@link ILassoTestable} for the point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoTestables%23fromPoint(IPoint) Online Documentation}*/
static fromPoint(point: IPoint | IPointConvertible): ILassoTestable;
/**
Creates an implementation of the {@link ILassoTestable} interface that tests if the <code>rectangle</code> intersects the area
of the lasso path.
@param rectangle The rectangle to check.
@returns An implementation of {@link ILassoTestable} for the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoTestables%23fromRectangle(IRectangle) Online Documentation}*/
static fromRectangle(rectangle: IRectangle | IRectangleConvertible): ILassoTestable;
/**
An implementation that always yields <code>true</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoTestables%23ALWAYS Online Documentation}*/
static ALWAYS: ILassoTestable;
/**
An implementation that always yields <code>false</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LassoTestables%23NEVER Online Documentation}*/
static NEVER: ILassoTestable;
static $class: Class;
private $_LassoTestables: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LassoTestables;
}
export interface IMarqueeTestable {
}
/**
Interface that is used to test whether a rendered item is deemed to be inside a rectangular marquee selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMarqueeTestable Online Documentation}*/
export abstract class IMarqueeTestable {
/**
Creates an implementation of the interface {@link IMarqueeTestable} by using the given function as implementation
for its {@link IMarqueeTestable#isInBox} method.
@param isInBox A function for {@link IMarqueeTestable}'s single abstract method {@link IMarqueeTestable#isInBox}.
@returns An instance of the {@link IMarqueeTestable} interface based on the given function.*/
static create(isInBox: (context: IInputModeContext, rectangle: Rect) => boolean): IMarqueeTestable;
/**
Returns <code>true</code> if the corresponding item is considered to intersect the given rectangular box.
This method may return <code>false</code> if the item cannot be selected using a selection marquee or optionally if the item is only
partially contained within the box.
Implementations should respect the {@link ICanvasContext#hitTestRadius} if marquee selections should behave differently on different zoom levels.
@param context the current input mode context
@param rectangle the box describing the marquee's bounds
@returns <code>true</code> if the item is considered to be captured by the marquee
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMarqueeTestable%23isInBox(IInputModeContext%2CRect) Online Documentation}*/
abstract isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
An implementation that always yields <code>true</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMarqueeTestable%23ALWAYS Online Documentation}*/
static ALWAYS: IMarqueeTestable;
/**
An implementation that always yields <code>false</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMarqueeTestable%23NEVER Online Documentation}*/
static NEVER: IMarqueeTestable;
static $class: Class;
private $_IMarqueeTestable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IMarqueeTestable;
}
/**
Determines the policy whether and how {@link Visual}s are cached when they are removed temporarily from the
{@link CanvasComponent}'s visual tree.
{@link Visual}s are temporarily removed when they are moved outside the visual part of the canvas (the {@link CanvasComponent#viewport}) and
re-added when they enter the {@link CanvasComponent#viewport} again. Caching prevents the need to re-build the visuals from scratch.
In order to be cached a dispose visual callback has to be {@link IRenderContext#setDisposeCallback registered} during {@link IVisualCreator#createVisual creation}.
@see CanvasComponent#visualCaching
@see IRenderContext#setDisposeCallback
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VisualCachingPolicy Online Documentation}*/
export enum VisualCachingPolicy {
/**
Disables caching of {@link Visual}s completely.
@see {@link https://docs.yworks.com/yfileshtml/#/api/VisualCachingPolicy%23NEVER Online Documentation}*/
NEVER,
/**
{@link Visual}s will always be cached.
The cached reference will be kept until the corresponding {@link ICanvasObject} is removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/VisualCachingPolicy%23ALWAYS Online Documentation}*/
ALWAYS
}
/** Automatic conversion to {@link VisualCachingPolicy}. */
type VisualCachingPolicyStringValues = "never" | "always";
export interface IRenderContext extends ICanvasContext {
}
/**
An interface that describes the current context for rendering routines.
@see IVisualCreator
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext Online Documentation}*/
export abstract class IRenderContext {
/**
Notifies the context that the <code>visual</code> has been removed from the visual tree.
This has to be done in a {@link IVisualCreator}'s {@link IVisualCreator#updateVisual} method if a
{@link IVisualCreator}'s {@link IVisualCreator#createVisual} is called or a previously added visual will be
removed or replaced. Calling this method will call a previously {@link IRenderContext#setDisposeCallback registered} dispose visual callback that the <code>visual</code> has
been removed and needs to free its resources.
If the <code>visual</code> has not been registered previously this method will do nothing.
@param visual The visual which has been removed.
@see IRenderContext#setDisposeCallback
@see IVisualCreator
@see IVisualCreator#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23childVisualRemoved(Visual) Online Documentation}*/
childVisualRemoved(visual: Visual | null): void;
/**
Converts the given set of intermediate coordinates to a coordinate pair that can be used to render within the {@link IRenderContext#viewTransform}.
If {@link IRenderContext#projection} is {@link Matrix#IDENTITY}, this method has no effect
@param intermediatePoint The coordinates in the intermediate coordinate system.
@returns The coordinates in the view coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23intermediateToViewCoordinates(Point) Online Documentation}*/
abstract intermediateToViewCoordinates(intermediatePoint: Point): Point;
/**
Registers a visual if children have registered themselves for disposal.
This is necessary for visual creators which delegate to another visual creator to create one or more child visuals, like
style decorators.
Implementors have to add a call to this method at the end of their {@link IVisualCreator#createVisual} and {@link IVisualCreator#updateVisual}
methods.
The registered <code>callback</code> has to call {@link IRenderContext#childVisualRemoved} on all child visuals of the <code>visual</code>.
@param visual The visual to be registered. This is the visual which is returned by the {@link IVisualCreator#createVisual} or {@link IVisualCreator#updateVisual}
method which calls this method.
@param callback A dispose visual callback which calls {@link IRenderContext#childVisualRemoved} on all child visuals of the
registered <code>visual</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23registerForChildrenIfNecessary(Visual%2Cfunction(IRenderContext%2CVisual%2Cboolean)%3AVisual) Online Documentation}*/
registerForChildrenIfNecessary(visual: Visual | null, callback: (context: IRenderContext, removedVisual: Visual, dispose: boolean) => Visual | null): void;
/**
Registers a <code>callback</code> which will be called when the <code>visual</code> will be removed from the visual tree.
This applies to a {@link IVisualCreator}'s {@link IVisualCreator#createVisual} method if the created <code>visual</code>
allocates resources which should be freed after the visual is no longer used. In this case the visual has to be
registered together with a dispose visual callback which can free the allocated resources. The callback will be called
by the framework when the visual is removed from the visual tree.
The callback can also be used to support caching for temporarily removed visuals. See dispose visual callback.
@param visual The visual which needs to be disposed after removal.
@param callback A callback which will dispose the <code>visual</code>.
@see IVisualCreator
@see IVisualCreator#createVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23setDisposeCallback(Visual%2Cfunction(IRenderContext%2CVisual%2Cboolean)%3AVisual) Online Documentation}*/
setDisposeCallback(visual: Visual, callback: (context: IRenderContext, removedVisual: Visual, dispose: boolean) => Visual | null): void;
/**
Converts the given set of world coordinates to a coordinate pair that can be used to render within the {@link IRenderContext#viewTransform}.
@param worldPoint The coordinates in the world coordinate system.
@returns The coordinates in the view coordinate system.
@see IRenderContext#viewTransform
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23toViewCoordinates(Point) Online Documentation}*/
abstract toViewCoordinates(worldPoint: Point): Point;
/**
Converts the given set of world coordinates to a coordinate pair that can be used to render within the {@link IRenderContext#intermediateTransform}.
If {@link IRenderContext#projection} is {@link Matrix#IDENTITY}, this method returns the same value as {@link IRenderContext#toViewCoordinates}.
@param worldPoint The coordinates in the world coordinate system.
@returns The coordinates in the intermediate coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23worldToIntermediateCoordinates(Point) Online Documentation}*/
abstract worldToIntermediateCoordinates(worldPoint: Point): Point;
/**
Gets the clipping rectangle in world coordinates.
Rendering outside of this clip may be skipped altogether.
Usually this is the viewport rectangle of the
{@link ICanvasContext#canvasComponent}, but may also be a sub-region of it. For image export or printing, this is the rectangle that's exported or printed on
a page.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23clip Online Documentation}*/
abstract readonly clip: Rect;
/**
Gets a transform that can be added to the transform list of a visual to be displayed in the intermediate coordinate
system (which still has the same axes directions as the world coordinate system, but doesn't include the {@link CanvasComponent#projection}).
In contrast to the world coordinate system where the logical coordinates of the elements that are managed by a {@link CanvasComponent}
(for example, graph elements like node and edges), the intermediate coordinate system is used to display visualizations
that are independent to scale (zoom) or translation transformations (like handles or selection indicators).
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23intermediateTransform Online Documentation}*/
abstract readonly intermediateTransform: Matrix;
/**
Gets a transform that can be added to the transform list of a visual to be displayed in the view coordinate system.
The view coordinate system includes the {@link CanvasComponent#projection} and may change directions of axes when using, e.g. an isometric transform. To
display elements that are only invariant to the zoom factor, the intermediate coordinate system may be a better choice.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23viewTransform Online Documentation}*/
abstract readonly viewTransform: Matrix;
/**
Gets a transform that is applied to the contents of the {@link IRenderContext#clip} rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23projection Online Documentation}*/
abstract readonly projection: Matrix;
/**
Provides access to the <code>defs</code> element in the SVG DOM that is managed by this instance.
@see IRenderContext#svgDefsManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23defsElement Online Documentation}*/
abstract readonly defsElement: Element;
/**
Provides access to the {@link SvgDefsManager} that manages this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23svgDefsManager Online Documentation}*/
abstract readonly svgDefsManager: SvgDefsManager;
/**
Provides access to the {@link WebGLSupport} that provides convenience methods for WebGL rendering.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRenderContext%23webGLSupport Online Documentation}*/
abstract readonly webGLSupport: WebGLSupport;
static $class: Class;
private $_IRenderContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IRenderContext;
}
export interface IVisualCreator {
}
/**
Factory interface which is used by implementations that know how to create {@link Visual}s for rendering into a
given {@link IRenderContext}.
@see CanvasComponent
@see ICanvasObjectDescriptor
@see ICanvasObject
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisualCreator Online Documentation}*/
export abstract class IVisualCreator {
/**
Creates an implementation of the interface {@link IVisualCreator} from the given definition.
The given object must contain exactly the abstract members of the {@link IVisualCreator} interface. No member may be
omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IVisualCreator} interface.
@returns An instance of the {@link IVisualCreator} interface based on the given definition.*/
static create(interfaceDefinition: {
createVisual(context: IRenderContext): Visual;
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual;
}): IVisualCreator;
/**
This method is called by the framework to create a {@link Visual} that will be included into the
{@link IRenderContext}.
{@link CanvasComponent} uses this interface through the {@link ICanvasObjectDescriptor} to populate the visual
canvas object tree.
@param context The context that describes where the visual will be used.
@returns The visual to include in the canvas object visual tree. This may be <code>null</code>.
@see IVisualCreator#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisualCreator%23createVisual(IRenderContext) Online Documentation}*/
abstract createVisual(context: IRenderContext): Visual | null;
/**
This method updates or replaces a previously created {@link Visual} for inclusion in the
{@link IRenderContext}.
The {@link CanvasComponent} uses this method to give implementations a chance to update an existing Visual that has
previously been created by the same instance during a call to {@link IVisualCreator#createVisual}. Implementation
may update the <code>oldVisual</code> and return that same reference, or create a new visual and return the new instance or <code>null</code>.
@param context The context that describes where the visual will be used in.
@param oldVisual The visual instance that had been returned the last time the {@link IVisualCreator#createVisual} method was called
on this instance.
@returns <code>oldVisual</code>, if this instance modified the visual, or a new visual that should replace the existing one in the canvas
object visual tree.
@see IVisualCreator#createVisual
@see ICanvasObjectDescriptor
@see CanvasComponent
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisualCreator%23updateVisual(IRenderContext%2CVisual) Online Documentation}*/
abstract updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
static $class: Class;
private $_IVisualCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IVisualCreator;
}
export interface VoidVisualCreator extends YObject, IVisualCreator {
}
/**
A void/null implementation of the {@link IVisualCreator} interface that does nothing.
Use {@link VoidVisualCreator#INSTANCE} for methods that require non-<code>null</code> {@link IVisualCreator} implementations
if you want to have no visualization.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidVisualCreator Online Documentation}*/
export class VoidVisualCreator {
/**
Returns <code>null</code>.
@param context
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidVisualCreator%23createVisual(IRenderContext) Online Documentation}*/
createVisual(context: IRenderContext): Visual | null;
/**
Returns <code>null</code>.
@param context
@param oldVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidVisualCreator%23updateVisual(IRenderContext%2CVisual) Online Documentation}*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
/**
An immutable and shareable instance of the {@link VoidVisualCreator} class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidVisualCreator%23INSTANCE Online Documentation}*/
static INSTANCE: IVisualCreator;
static $class: Class;
private $_VoidVisualCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidVisualCreator;
}
export interface OrientedRectangleIndicatorInstaller extends YObject, ISelectionIndicatorInstaller, IHighlightIndicatorInstaller, IFocusIndicatorInstaller {
/**
@inheritdoc*/
addCanvasObject(context: ICanvasContext, group: ICanvasObjectGroup, item: any | null): ICanvasObject | null;
} /**
An abstract base class for the {@link ISelectionIndicatorInstaller}, {@link IFocusIndicatorInstaller} and {@link IHighlightIndicatorInstaller}
implementations that draws an oriented rectangle using a {@link OrientedRectangleIndicatorInstaller#template}.
In the {@link ICanvasObjectInstaller#addCanvasObject} method, this implementation will add either a fixed {@link IOrientedRectangle}
instance that is passed in the constructor as a parameter or the return value of the method {@link OrientedRectangleIndicatorInstaller#getRectangle}
if the former does not exist (i.e. the parameterless constructor was used to instantiate this class).
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangleIndicatorInstaller Online Documentation}*/
export class OrientedRectangleIndicatorInstaller {
/**
Initializes a new instance of the {@link OrientedRectangleIndicatorInstaller} class with fixed bounds and a {@link string}.
The given {@link IOrientedRectangle} is used for all items as object that is added to the {@link ICanvasObjectGroup}
in the {@link ICanvasObjectInstaller#addCanvasObject} method.
If the {@link OrientedRectangleIndicatorInstaller#template} property is not set, the <code>templateKey</code> is used to lookup a {@link IVisualTemplate} in the resources of the
{@link CanvasComponent}. If there is none, a default template is used.
@param fixedBounds
@param [templateKey=null]
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangleIndicatorInstaller%23OrientedRectangleIndicatorInstaller(IOrientedRectangle%2Cstring) Online Documentation}*/
constructor(fixedBounds: IOrientedRectangle | null, templateKey?: string | null);
/**
Initializes a new instance of the {@link OrientedRectangleIndicatorInstaller} class without fixed bounds.
The object that is created through this constructor relies on the value returned by the {@link OrientedRectangleIndicatorInstaller#getRectangle}
method to {@link ICanvasObjectGroup#addChild add} to the {@link ICanvasObjectGroup} in the {@link ICanvasObjectInstaller#addCanvasObject} method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangleIndicatorInstaller%23OrientedRectangleIndicatorInstaller() Online Documentation}*/
constructor();
/**
Factory method that returns an {@link IOrientedRectangle} for a given user object.
This method is called for the object to {@link ICanvasObjectGroup#addChild add} to the
{@link ICanvasObjectGroup}, if there are no fixed bounds set on this instance (i.e. the parameterless constructor
was used to instantiate this class).
This implementation returns null for every given object.
@param item
@returns An {@link IOrientedRectangle} for the given item that is {@link ICanvasObjectGroup#addChild added} to the {@link ICanvasObjectGroup}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangleIndicatorInstaller%23getRectangle(Object) Online Documentation}*/
getRectangle(item: any | null): IOrientedRectangle | null;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} for drawing the selection indicator.
The default template has the <code>yfiles-selection-template</code> and <code>yfiles-label-selection-template</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangleIndicatorInstaller%23SELECTION_TEMPLATE_KEY Online Documentation}*/
static SELECTION_TEMPLATE_KEY: string;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} for drawing the focus indicator.
The default template has the <code>yfiles-focus-template</code> and <code>yfiles-label-focus-template</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangleIndicatorInstaller%23FOCUS_TEMPLATE_KEY Online Documentation}*/
static FOCUS_TEMPLATE_KEY: string;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} for drawing the highlight indicator.
The default template has the <code>yfiles-highlight-template</code> and <code>yfiles-label-highlight-template</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangleIndicatorInstaller%23HIGHLIGHT_TEMPLATE_KEY Online Documentation}*/
static HIGHLIGHT_TEMPLATE_KEY: string;
/**
Gets or sets the {@link IVisualTemplate} to use for this indicator.
If this property is not set, the {@link string} passed in the constructor is used to lookup a {@link IVisualTemplate} in the
resources of the {@link CanvasComponent}. If there is none, a default template is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangleIndicatorInstaller%23template Online Documentation}*/
template: IVisualTemplate | null;
static $class: Class;
private $_OrientedRectangleIndicatorInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrientedRectangleIndicatorInstaller;
}
export interface PointSelectionIndicatorInstaller extends YObject, ISelectionIndicatorInstaller {
/**
@inheritdoc*/
addCanvasObject(context: ICanvasContext, group: ICanvasObjectGroup, item: any | null): ICanvasObject | null;
} /**
A simple {@link ISelectionIndicatorInstaller} implementation that draws an ellipse centered at a given point in
world coordinate space.
The template has the <code>yfiles-selection-template</code> and <code>yfiles-port-selection-template</code> CSS classes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointSelectionIndicatorInstaller Online Documentation}*/
export class PointSelectionIndicatorInstaller {
/**
Creates a selection indicator that indicates a point.
@param point The point to use for the center of the painting.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointSelectionIndicatorInstaller Online Documentation}*/
constructor(point: IPoint | IPointConvertible);
/**
Factory method that retrieves the center for a given user object.
This implementation simply returns the value provided to the constructor.
@param userObject
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointSelectionIndicatorInstaller%23getCenterPoint(Object) Online Documentation}*/
getCenterPoint(userObject: any | null): IPoint | null;
/**
Gets or sets the {@link Stroke} to use for drawing the point in the view coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointSelectionIndicatorInstaller%23stroke Online Documentation}*/
stroke: Stroke | null;
/**
Gets or sets the {@link Fill} to use for filling the point's ellipse in the view coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointSelectionIndicatorInstaller%23fill Online Documentation}*/
fill: Fill | null;
/**
Gets or sets a value indicating whether the visualization is drawn in view coordinates, instead of intermediate
coordinates.
This setting has no visible effect, unless a {@link CanvasComponent#projection} is set.
The default value is <code>true</code>, to draw the visualization in the view coordinate system.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointSelectionIndicatorInstaller%23useViewCoordinates Online Documentation}*/
useViewCoordinates: boolean;
static $class: Class;
private $_PointSelectionIndicatorInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PointSelectionIndicatorInstaller;
}
export interface RectangleHandle extends YObject, IMutablePoint, IHandle {
}
/**
A more sophisticated {@link IHandle} implementation that modifies a rectangle.
Implementations of this class work on a mutable rectangle instance for handling drags of the handle. A separate point
implementation is used for specifying the position of the handle. As a convenience this class also implements the {@link IMutablePoint}
interface which modifies the handle's position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle Online Documentation}*/
export class RectangleHandle {
/**
Subclass constructor that creates a handle for the given position using the location instance as the {@link IDragHandler#location} for the handle.
The instance modifies the instances using the reshapeable interface and reads the current state using the rect. The
handle will automatically trim changes of the rectangle to the specified minimum and maximum size.
@param position The position of the handle.
@param rectangle The rectangle to change.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle Online Documentation}*/
constructor(position: HandlePositions | HandlePositionsConvertible, rectangle: IMutableRectangle);
/**
Delegates the original values to {@link RectangleHandle#set}
@param context
@param originalLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23cancelDrag(IInputModeContext%2CPoint) Online Documentation}*/
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
/**
This implementation does nothing.
@param context
@param originalLocation
@param newLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23dragFinished(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
Delegates to {@link RectangleHandle#set}.
@param context
@param originalLocation
@param newLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23handleMove(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
This implementation does nothing.
@param context
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23initializeDrag(IInputModeContext) Online Documentation}*/
initializeDrag(context: IInputModeContext): void;
/**
Delegates the work to the reshapeable.
@param delta
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23moveBy(Point) Online Documentation}*/
moveBy(delta: Point): void;
/**
Delegates to {@link RectangleHandle#moveBy}.
@param x
@param y
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23set(number%2Cnumber) Online Documentation}*/
set(x: number, y: number): void;
/**
Gets the position of the handle.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23position Online Documentation}*/
position: HandlePositions;
/**
Gets the rectangle instance to modify.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23rectangle Online Documentation}*/
rectangle: IMutableRectangle;
/**
Gets or sets the minimum size allowed for the reshapeable.
The value is stored by reference. The default is {@link Size#ZERO}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23minimumSize Online Documentation}*/
minimumSize: ISize;
/**
Gets or sets the minimum enclosed area that needs to be encompassed by this reshapeable.
The value is stored by reference. The default is {@link Size#ZERO}, which does not restrict the area at all.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23minimumEnclosedArea Online Documentation}*/
minimumEnclosedArea: IRectangle;
/**
Gets or sets the maximal bounds this reshapeable may fill.
The value is stored by reference. The default is
{@link Rect#INFINITE}, which does not restrict the area at all.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23maximumBoundingArea Online Documentation}*/
maximumBoundingArea: IRectangle;
/**
Gets or sets the maximum size allowed for the reshapeable.
The value is stored by reference. The default is {@link Size#INFINITE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23maximumSize Online Documentation}*/
maximumSize: ISize;
/**
Gets a view of the center of the handle.
The point describes the current world coordinate of the element that can be modified by this handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23location Online Documentation}*/
readonly location: IPoint;
/**
Gets or sets the type of the handle that can be used by the rendering engine to render types differently.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23type Online Documentation}*/
type: HandleTypes;
/**
Gets or sets the cursor to display when the mouse hovers over or drags this handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23cursor Online Documentation}*/
cursor: Cursor;
/**
Gets or sets the x coordinate.
The setter delegates to {@link RectangleHandle#moveBy}
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23x Online Documentation}*/
x: number;
/**
Gets or sets the y coordinate.
The setter delegates to {@link RectangleHandle#moveBy}
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleHandle%23y Online Documentation}*/
y: number;
static $class: Class;
private $_RectangleHandle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RectangleHandle;
}
export interface RectangleIndicatorInstaller extends YObject, ISelectionIndicatorInstaller, IHighlightIndicatorInstaller, IFocusIndicatorInstaller {
/**
@inheritdoc*/
addCanvasObject(context: ICanvasContext, group: ICanvasObjectGroup, item: any | null): ICanvasObject | null;
} /**
A basis implementation of {@link ISelectionIndicatorInstaller}, {@link IFocusIndicatorInstaller} and {@link IHighlightIndicatorInstaller}
that draws a rectangle using a {@link RectangleIndicatorInstaller#template}.
In the {@link ICanvasObjectInstaller#addCanvasObject} method, this implementation will add either a fixed {@link IRectangle}
instance that is passed in the constructor as a parameter or the return value of the method {@link RectangleIndicatorInstaller#getRectangle}
if the former does not exist (i.e. the parameterless constructor was used to instantiate this class).
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleIndicatorInstaller Online Documentation}*/
export class RectangleIndicatorInstaller {
/**
Initializes a new instance of the {@link RectangleIndicatorInstaller} class with fixed bounds and a {@link string}.
The given {@link IRectangle} is used for all items as object that is added to the {@link ICanvasObjectGroup}
in the {@link ICanvasObjectInstaller#addCanvasObject} method.
If the {@link RectangleIndicatorInstaller#template} property is not set, the <code>resourceKey</code> is used to lookup a {@link IVisualTemplate} in the resources of the
{@link CanvasComponent}. If there is none, a default template is used.
@param fixedBounds
@param [resourceKey=null]
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleIndicatorInstaller%23RectangleIndicatorInstaller(IRectangle%2Cstring) Online Documentation}*/
constructor(fixedBounds: IRectangle | null | IRectangleConvertible, resourceKey?: string | null);
/**
Initializes a new instance of the {@link RectangleIndicatorInstaller} class without fixed bounds.
The object that is created through this constructor relies on the value returned by the {@link RectangleIndicatorInstaller#getRectangle}
method to {@link ICanvasObjectGroup#addChild add} to the {@link ICanvasObjectGroup} in the {@link ICanvasObjectInstaller#addCanvasObject} method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleIndicatorInstaller%23RectangleIndicatorInstaller() Online Documentation}*/
constructor();
/**
Factory method that returns an {@link IRectangle} for a given user object.
This method is called for the object to {@link ICanvasObjectGroup#addChild add} to the
{@link ICanvasObjectGroup}, if there are no fixed bounds set on this instance (i.e. the parameterless constructor
was used to instantiate this class).
This implementation returns null for every given object.
@param userObject
@returns An {@link IRectangle} for the given item that is {@link ICanvasObjectGroup#addChild added} to the {@link ICanvasObjectGroup}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleIndicatorInstaller%23getRectangle(Object) Online Documentation}*/
getRectangle(userObject: any | null): IRectangle | null;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} for drawing the selection indicator.
The default template has the <code>yfiles-selection-template</code> and <code>yfiles-node-selection-template</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleIndicatorInstaller%23SELECTION_TEMPLATE_KEY Online Documentation}*/
static SELECTION_TEMPLATE_KEY: string;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} for drawing the focus indicator.
The default template has the <code>yfiles-focus-template</code> and <code>yfiles-node-focus-template</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleIndicatorInstaller%23FOCUS_TEMPLATE_KEY Online Documentation}*/
static FOCUS_TEMPLATE_KEY: string;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} for drawing the highlight indicator.
The default template has the <code>yfiles-highlight-template</code> and <code>yfiles-node-highlight-template</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleIndicatorInstaller%23HIGHLIGHT_TEMPLATE_KEY Online Documentation}*/
static HIGHLIGHT_TEMPLATE_KEY: string;
/**
Gets or sets whether the installed rectangle will be drawn in the View coordinate system as opposed to the Intermediate
coordinate system.
Drawing in the View coordinate system forces the {@link Visual} to be rectangular under any {@link CanvasComponent#projection}.
If no {@link CanvasComponent#projection} is set, this setting has no effect.
Default is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleIndicatorInstaller%23useViewCoordinates Online Documentation}*/
useViewCoordinates: boolean;
/**
Gets or sets the {@link IVisualTemplate} to use for this indicator.
If this property is not set, the {@link string} passed in the constructor is used to lookup a {@link IVisualTemplate} in the
resources of the {@link CanvasComponent}. If there is none, a default template is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RectangleIndicatorInstaller%23template Online Documentation}*/
template: IVisualTemplate | null;
static $class: Class;
private $_RectangleIndicatorInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RectangleIndicatorInstaller;
}
/**
Describes the technologies used to render visual elements.
This type can be used to configure how certain visual elements are rendered.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RenderModes Online Documentation}*/
export enum RenderModes {
/**
Renders in SVG.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RenderModes%23SVG Online Documentation}*/
SVG,
/**
Renders in HTML5 canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RenderModes%23CANVAS Online Documentation}*/
CANVAS,
/**
Renders in WebGL.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RenderModes%23WEB_GL Online Documentation}*/
WEB_GL
}
/** Automatic conversion to {@link RenderModes}. */
type RenderModesStringValues = "svg" | "canvas" | "web-gl";
export interface ReshapeHandlerHandle extends YObject, IHandle {
/**
@inheritdoc*/
initializeDrag(context: IInputModeContext): void;
} /**
A more sophisticated {@link IHandle} implementation that delegates to a {@link IReshapeHandler}.
This class works on an {@link IReshapeHandler}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle Online Documentation}*/
export class ReshapeHandlerHandle {
/**
Subclass constructor that creates a handle for the given position using the location instance as the {@link IDragHandler#location} for the handle.
The instance modifies the instances using the reshapeable interface and reads the current state using the rect. The
handle will automatically trim changes of the rectangle to the specified minimum and maximum size.
@param position The position of the handle.
@param reshapeHandler The reshapeable to write the changes to.
@param [minimumSize=null] The minimum size allowed for the rectangle.
@param [maximumSize=null] The maximum size allowed for the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle Online Documentation}*/
constructor(position: HandlePositions | HandlePositionsConvertible, reshapeHandler: IReshapeHandler, minimumSize?: ISize | null | ISizeConvertible, maximumSize?: ISize | null | ISizeConvertible);
/**
Subclass constructor that creates a handle for the given position using the location instance as the {@link IDragHandler#location} for the handle.
The instance modifies the instances using the reshapeable interface and reads the current state using the rect. The
handle will automatically trim changes of the rectangle to the specified minimum and maximum size.
@param options The parameters to pass.
@param options.position The position of the handle.
@param options.reshapeHandler The reshapeable to write the changes to.
@param [options.minimumSize=null] The minimum size allowed for the rectangle.
@param [options.maximumSize=null] The maximum size allowed for the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle Online Documentation}*/
constructor(options: {
position: HandlePositions | HandlePositionsConvertible;
reshapeHandler: IReshapeHandler;
minimumSize?: ISize | null | ISizeConvertible;
maximumSize?: ISize | null | ISizeConvertible;
});
/**
Delegates the cancel request to the {@link IReshapeHandler}.
@param context
@param originalLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23cancelDrag(IInputModeContext%2CPoint) Online Documentation}*/
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
/**
Delegates the finish request to the {@link IReshapeHandler} using the last bounds calculated by
{@link ReshapeHandlerHandle#handleMove}.
@param context
@param originalLocation
@param newLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23dragFinished(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
Translates the move request into new bounds and delegates applying the new bounds to the {@link IReshapeHandler#handleReshape}
method.
When translating the move request into new bounds, the {@link HandlePositions}, {@link ReshapeHandlerHandle#centerReshapeRecognizer}, {@link ReshapeHandlerHandle#ratioReshapeRecognizer}, {@link ReshapeHandlerHandle#reshapePolicy}, {@link ReshapeHandlerHandle#minimumSize}, {@link ReshapeHandlerHandle#maximumSize}, {@link ReshapeHandlerHandle#minimumEnclosedArea} and {@link ReshapeHandlerHandle#maximumBoundingArea} are considered.
@param context
@param originalLocation
@param newLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23handleMove(IInputModeContext%2CPoint%2CPoint) Online Documentation}*/
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
Gets the {@link IReshapeHandler} instance to write the changes to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23reshapeHandler Online Documentation}*/
readonly reshapeHandler: IReshapeHandler;
/**
Gets the position of the handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23position Online Documentation}*/
readonly position: HandlePositions;
/**
Gets or sets the minimum size allowed for the reshapeable.
The value is stored by reference. The default is {@link Size#ZERO}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23minimumSize Online Documentation}*/
minimumSize: ISize;
/**
Gets or sets the minimum enclosed area that needs to be encompassed by this reshapeable.
The value is stored by reference. The default is {@link Rect#EMPTY}, which does not restrict the area at all.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23minimumEnclosedArea Online Documentation}*/
minimumEnclosedArea: IRectangle;
/**
Gets or sets the maximal bounds this reshapeable may fill.
The value is stored by reference. The default is
{@link Rect#INFINITE}, which does not restrict the area at all.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23maximumBoundingArea Online Documentation}*/
maximumBoundingArea: IRectangle;
/**
Gets or sets the maximum size allowed for the reshapeable.
The value is stored by reference. The default is {@link Size#INFINITE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23maximumSize Online Documentation}*/
maximumSize: ISize;
/**
* Gets or sets the event recognizer that determines if the width/height ratio of the reshapeable is kept during the
* reshape gesture.
* The default value is {@link KeyEventRecognizers#SHIFT_IS_DOWN}.
*
* <br><b>Always keep the aspect ratio of the reshapeable</b>
* <pre>
* // always keep the aspect ratio
* handle.ratioReshapeRecognizer = EventRecognizers.ALWAYS
* </pre>
*
* @see ReshapeHandlerHandle#centerReshapeRecognizer
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23ratioReshapeRecognizer Online Documentation}
*/
ratioReshapeRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
* Gets or sets the event recognizer that determines if the center of the reshapeable is kept during the reshape gesture.
* The default value is {@link KeyEventRecognizers#ALT_IS_DOWN}.
*
* <br><b>Always keeping the center of the reshapeable</b>
* <pre>
* // always keep the center of the node
* handle.centerReshapeRecognizer = EventRecognizers.ALWAYS
* </pre>
*
* @see ReshapeHandlerHandle#ratioReshapeRecognizer
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23centerReshapeRecognizer Online Documentation}
*/
centerReshapeRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
* Gets or sets the policy that determines how the mouse location is interpreted when the reshape gesture shall keep the
* aspect ratio.
* The {@link ReshapeHandlerHandle#ratioReshapeRecognizer} is used to determine whether the aspect ratio shall be kept at all.
*
* The default value is {@link ReshapePolicy#PROJECTION}.
*
* <br><b>Extend the reshapeable bounds to the mouse location</b>
* <pre>
* // when keeping the aspect ratio extend the node bounds to the mouse location
* handle.reshapePolicy = ReshapePolicy.MAXIMUM
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23reshapePolicy Online Documentation}
*/
reshapePolicy: ReshapePolicy;
/**
Gets a view of the center of the handle.
The point describes the current world coordinate of the element that can be modified by this handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23location Online Documentation}*/
readonly location: IPoint;
/**
Gets or sets the type of the handle that can be used by the rendering engine to render types differently.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23type Online Documentation}*/
type: HandleTypes;
/**
Gets or sets the cursor to display when the mouse hovers over or drags this handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeHandlerHandle%23cursor Online Documentation}*/
cursor: Cursor;
static $class: Class;
private $_ReshapeHandlerHandle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReshapeHandlerHandle;
}
export interface ViewportLimiter extends YObject {
}
/**
Used by {@link CanvasComponent} to limit the interactive movement of the viewport.
The {@link CanvasComponent#viewportLimiter} instance should be consulted whenever the user tries to change the viewport. Using method {@link ViewportLimiter#limitViewport}
the desired viewport can be validated by the implementation. Note that setting the {@link CanvasComponent#zoom} property or {@link CanvasComponent#viewPoint} property will not be
influenced by implementations of this class. It is up to the caller to perform verification.
@see ViewportLimiter#bounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimiter Online Documentation}*/
export class ViewportLimiter {
/**
Initializes a new instance of the {@link ViewportLimiter} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimiter Online Documentation}*/
constructor();
/**
Gets the to bounds that should be honored for the upcoming call.
This method is used as callback by {@link ViewportLimiter#limitViewport} for each request. The default
implementation just yields {@link ViewportLimiter#bounds}.
@param canvas The canvas control.
@param suggestedViewport The suggested viewport.
@returns The bounds to honor or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimiter%23getCurrentBounds(CanvasComponent%2CRect) Online Documentation}*/
getCurrentBounds(canvas: CanvasComponent, suggestedViewport: Rect): Rect | null;
/**
Inspects the desired or suggested new viewport for the given control and returns a valid viewport to use.
@param canvas The canvas control on which the viewport should be applied.
@param suggestedViewport The suggested viewport.
@returns The viewport that should be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimiter%23limitViewport(CanvasComponent%2CRect) Online Documentation}*/
limitViewport(canvas: CanvasComponent, suggestedViewport: Rect): Rect;
/**
Gets or sets a value indicating whether both dimensions of {@link ViewportLimiter#bounds} need to be honored.
If set to <code>true</code> the viewport is limited in a way that the {@link CanvasComponent#viewport} must fit in both width and height of the
{@link ViewportLimiter#bounds}. If the aspect ratio of viewport and bounds are different parts of the viewport will always be cropped. In video
terminology this setting is often referred to as "crop" or "cropping". In other words, this option will ensure that the
viewport does never contain parts outside the limited area.
If set to <code>false</code> the viewport is only limited in the smaller dimension. If the aspect ratio of viewport and bounds are
different parts of the viewport will be outside the limits. In video terminology this setting is often referred to as
"letterbox" or "letterboxing". In other words, this option will ensure that the viewport is always filled in at least
one dimension with the limited area.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimiter%23honorBothDimensions Online Documentation}*/
honorBothDimensions: boolean;
/**
Gets or sets the maximal allowed navigable bounds for the viewport.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimiter%23bounds Online Documentation}*/
bounds: Rect | null;
/**
Gets or sets the policy how the viewport is limited if the viewport is <b>not</b> within the limits.
The default is {@link ViewportLimitingPolicy#TOWARDS_LIMIT}.
@default ViewportLimitingPolicy.TOWARDS_LIMIT
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimiter%23limitingPolicy Online Documentation}*/
limitingPolicy: ViewportLimitingPolicy;
static $class: Class;
private $_ViewportLimiter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ViewportLimiter;
}
/**
Defines how {@link ViewportLimiter} should {@link ViewportLimiter#limitViewport limit} the suggested {@link CanvasComponent#viewport}.
This policy defines how viewport limits should be treated if the viewport is <b>not</b> within the limits.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimitingPolicy Online Documentation}*/
export enum ViewportLimitingPolicy {
/**
Strictly applies the bounds.
In case the viewport happens to be outside the limits this might result in a noticeable "jump" upon the first pan or
zoom gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimitingPolicy%23STRICT Online Documentation}*/
STRICT,
/**
If the current {@link CanvasComponent#viewport} lies outside the {@link ViewportLimiter#bounds} the {@link ViewportLimiter} allows for changes towards the {@link ViewportLimiter#bounds} but forbids changes
away from them.
In case the viewport happens to be outside the limits all following viewport changes will be forced to move the viewport
in the direction of the limits.
While the viewport will not comply to the limits right away this policy avoids the visible "jump" which can happen with
the {@link ViewportLimitingPolicy#STRICT} policy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ViewportLimitingPolicy%23TOWARDS_LIMIT Online Documentation}*/
TOWARDS_LIMIT
}
/** Automatic conversion to {@link ViewportLimitingPolicy}. */
type ViewportLimitingPolicyStringValues = "strict" | "towards-limit";
export interface WebGLSupport extends YObject {
}
/**
Class containing helper methods for WebGL rendering.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLSupport Online Documentation}*/
export class WebGLSupport {
/**
* Sets the active program as defined by the vertex and fragment shader sources and provides view and world transforms that
* can be used in the shader definitions.
* The following transforms are available for use in the shader sources:
*
* <ul>
* <li><b><code>viewTransform</code></b> – A uniform of type <code>mat3</code> that defines the 2D
* {@link IRenderContext#viewTransform view transform}. Apply this transform to world coordinates to render in the WebGL coordinate system.</li>
* <li><b><code>worldTransform</code></b> – A uniform of type <code>mat3</code> that defines the 2D world transform. Apply this transform to view coordinates to render in the
* WebGL coordinate system.</li>
* <li><b><code>projection</code></b> – A uniform of type <code>mat3</code> that defines the 2D projection transform. Apply this transform to intermediate coordinates to
* render in the WebGL coordinate system.</li>
* <li><b><code>viewTransform_3d</code></b> – A uniform of type <code>mat4</code> that defines the 3D
* {@link IRenderContext#viewTransform view transform}. Apply this transform to world coordinates to render in the WebGL coordinate system.</li>
* <li><b><code>worldTransform_3d</code></b> – A uniform of type <code>mat4</code> that defines the 3D world transform. Apply this transform to view coordinates to render in the
* WebGL coordinate system.</li>
* <li><b><code>projection_3d</code></b> – A uniform of type <code>mat4</code> that defines the 3D projection transform. Apply this transform to intermediate coordinates to
* render in the WebGL coordinate system.</li>
* </ul>
* Note that the 3d transforms clips z-values smaller that <code>0</code> or greater than <code>1000</code>.
*
* Example vertex shader using 2D coordinates:
*
* <pre>
* attribute vec2 position;
*
* void main() {
*   gl_position = vec4((viewTransform * vec3(position, 1)).xy, 0, 1);
* }
* </pre>
*
* Example vertex shader using 3D coordinates:
*
* <pre>
* attribute vec3 position;
*
* void main() {
*   gl_Position = viewTransform_3d * vec4(position, 1);
* }
* </pre>
*
* @param vertexShaderSource
* @param fragmentShaderSource
* @see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLSupport%23useProgram(string%2Cstring) Online Documentation}
*/
useProgram(vertexShaderSource: string, fragmentShaderSource: string): WebGLProgram;
static $class: Class;
private $_WebGLSupport: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WebGLSupport;
}
export interface Size extends YObject, ISize {
}
/**
* Represents the size of a rectangular object with a given {@link Size#width} and {@link Size#height}.
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* It is possible to specify an {@link Object} or plain {@link Object} to define the {@link Size}:
*
* <pre>
* [20, 20] // width, height
* { width: 20, height: 20 }
* </pre>
*
* The following example shows a few ways to use {@link Size}:
*
* <pre>
* // Create a new size
* const size = new Size(10, 15)
*
* console.log(
*   `The size has width ${size.width} and height ${size.height} representing an area of ${size.area}`
* )
*
* // Create another size
* const size2 = new Size(5, 17)
*
* // Enlarge it by multiplying with a factor
* const larger = size2.multiply(1.5)
*
* // Make the first size only half as large
* const half = size.multiply(0.5)
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Size Online Documentation}
*/
export class Size {
/**
Creates a new size using the provided width and height.
@param width The new width.
@param height The new height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size Online Documentation}*/
constructor(width: number, height: number);
/**
Performs an implicit conversion from {@link MutableSize} to {@link Size}.
@param size The size.
@returns The result of the conversion.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23op_Implicit(MutableSize) Online Documentation}*/
static convertFrom(size: MutableSize): Size;
/**
Performs an explicit conversion from {@link Size} to {@link MutableSize}.
@param size The size.
@returns The result of the conversion.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23op_Explicit(Size) Online Documentation}*/
static convertToMutableSize(size: Size | SizeConvertible): MutableSize | null;
/**
Creates a {@link Size} instance from the given size-like object by performing automatic type conversion.
@param sizeLike The object to convert to a {@link Size}.
@returns Either <code>sizeLike</code> if it is already a {@link Size} or a new instance, initialized to the values found in <code>sizeLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23from(Size) Online Documentation}*/
static from(sizeLike: Size | SizeConvertible): Size;
/**
Returns a size whose {@link Size#width} and {@link Size#height} are the respective maximum of the two provided {@link Size} instances.
@param size1 The first size.
@param size2 The second size.
@returns A size whose {@link Size#width} and {@link Size#height} is the {@link Math#max} of the respective properties of the two parameters.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23max(Size%2CSize) Online Documentation}*/
static max(size1: Size, size2: Size): Size;
/**
Returns a size whose {@link Size#width} and {@link Size#height} are the respective minimum of the two provided {@link Size} instances.
@param size1 The first size.
@param size2 The second size.
@returns A size whose {@link Size#width} and {@link Size#height} is the {@link Math#min} of the respective properties of the two parameters.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23min(Size%2CSize) Online Documentation}*/
static min(size1: Size, size2: Size): Size;
/**
Multiplies the {@link Size#width} and {@link Size#height} by the given factor and returns the result.
@param factor The factor to multiply the width and height by.
@returns A new size that has the width and height multiplied by the factor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23multiply(number) Online Documentation}*/
multiply(factor: number): Size;
/**
Converts this instance to an {@link MutableSize} instance
@returns The {@link MutableSize} instance, whose attributes have been initialized with the values of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23toMutableSize() Online Documentation}*/
toMutableSize(): MutableSize;
/**
Returns a human readable string that contains information about the values of this instance.
@returns A {@link string} containing the width and height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23toString() Online Documentation}*/
toString(): string;
/**
Creates a {@link YDimension} from a given {@link Size}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link YDimension}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23toYDimension() Online Documentation}*/
toYDimension(): YDimension;
/**
The "empty" size that has {@link Size#width} and {@link Size#height} set to <code>-1.0d</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23EMPTY Online Documentation}*/
static EMPTY: Size;
/**
The "zero" size that has {@link Size#width} and {@link Size#height} set to <code>0.0d</code>.
This size is not treated as {@link Size#isEmpty}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23ZERO Online Documentation}*/
static ZERO: Size;
/**
The "infinite" size that has {@link Size#width} and {@link Size#height} set to {@link Number#POSITIVE_INFINITY}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23INFINITE Online Documentation}*/
static INFINITE: Size;
/**
Gets the width.
Negative values for the width indicate an {@link Size#isEmpty empty} size.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23width Online Documentation}*/
readonly width: number;
/**
Gets the height.
Negative values for the height indicate an {@link Size#isEmpty empty} size.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23height Online Documentation}*/
readonly height: number;
/**
Gets the area of this instance which is the product of {@link Size#width} and {@link Size#height}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23area Online Documentation}*/
readonly area: number;
/**
Gets a value indicating whether this instance is deemed empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23isEmpty Online Documentation}*/
readonly isEmpty: boolean;
/**
Gets a value indicating whether this instance is finite.
This means that the instance is not {@link Size#isEmpty empty} and neither {@link Size#width} nor {@link Size#height} are infinite.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Size%23isFinite Online Documentation}*/
readonly isFinite: boolean;
static $class: Class;
private $_Size: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Size;
}
/** Automatic conversion to {@link Size}. */
type SizeConvertible = [number, number] | {
width: number;
height: number;
};
export interface Point extends YObject, IPoint {
}
/**
* Represents an immutable point in two-dimensional Cartesian coordinates with {@link number double precision}.
* In addition to being used to model points {@link Point} also includes a number of useful properties and methods
* that allow it to be treated as a vector.
*
* Furthermore, {@link Point} provides various methods for analyzing and manipulating geometry in general, such
* as:
*
* <ul>
* <li><b>Distance measurements</b> – {@link IPoint#distanceTo}, {@link Point#distanceToSegment}</li>
* <li><b>Projections</b> – {@link Point#getProjectionOnSegment}, {@link Point#getProjectionOnLine},
* {@link Point#getProjectionOnRay}</li>
* <li><b>Hit-tests</b> – {@link Point#hits}, {@link Point#hitsLineSegment}, {@link Point#hitsPolyline}</li>
* </ul>
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* It is possible to specify an {@link Object} or plain {@link Object} to define the {@link Point}:
*
* <pre>
* [5, 5] // x, y
* { x: 5, y: 5 }
* </pre>
*
* The following example shows a few ways to use {@link Point}:
*
* <pre>
* // Create a new point
* const p1 = new Point(-5, 2)
* // Create another point somewhere else
* const p2 = new Point(23, 42)
*
* console.log(`Point 1 is at (${p1.X},${p1.Y}), point 2 is at (${p2.X},${p2.y})`)
*
* // Subtract one point from the other to get a vector between them
* // which is also represented as a Point
* const v = p2.subtract(p1)
*
* // Point offers a few handy properties when interpreting it as a vector
* console.log(`VectorLength: ${v.vectorLength}`)
* // VectorLength in this case is equivalent to p1.distanceTo(p2):
* console.log(`Distance: ${p1.distanceTo(p2)}`)
*
* console.log(`IsHorizontalVector: ${v.isHorizontalVector}`)
* console.log(`IsVerticalVector: ${v.isVerticalVector}`)
* console.log(`Normalized: ${v.normalized}`)
*
* const halfLength = v.multiply(0.2).vectorLength
* const doubleLength = v.multiply(2).vectorLength
*
* console.log(`Original vector: ${v.vectorLength}`)
* console.log(`Half-length vector: ${halfLength}`)
* console.log(`Double-length vector: ${doubleLength}`)
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point Online Documentation}
*/
export class Point {
/**
Initializes a new instance using the given values for the {@link Point#x} and {@link Point#y} properties.
@param x The x coordinate.
@param y The y coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point Online Documentation}*/
constructor(x: number, y: number);
/**
* Adds the given vector and this instance and returns the result as new instance.
* This operation is applied componentwise to {@link Point#x} and {@link Point#y}.
*
* <br><b>Using the add and subtract methods</b>
* <pre>
* const p1 = new Point(2, 5)
* const p2 = new Point(17, -6)
*
* // Translate p1 by the vector p2
* const p3 = p1.add(p2)
* console.log(`p1 + p2 = ${p3}`) // (19, -1)
*
* // Subtract p1 from p3 to get p2 again
* const p4 = p3.subtract(p1)
* console.log(p2.equals(p4)) // true
*
* // Reverse the direction of p2
* const reversed = p2.multiply(-1)
* console.log(`p2 reversed: ${reversed}`) // (-17, 6)
* </pre>
*
* @param otherPoint The second point.
* @returns The result of the vector addition.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23add(Point) Online Documentation}
*/
add(otherPoint: Point): Point;
/**
* Performs an explicit conversion from {@link MutablePoint} to {@link Point}.
* The following example shows how to convert a {@link Point} to a {@link MutablePoint} and back:
*
* <pre>
* const point = new Point(10, 15)
*
* console.log(`Initial point: ${point}`) // (10, 15)
*
* const mPoint = point.toMutablePoint()
* // The following also works, due to an implicit conversion:
* const mPoint2 = point
*
* mPoint.x = 42
* const point2 = mPoint.toPoint()
* // The following also works, due an explicit conversion:
* const point3 = mPoint
*
* console.log(`Changed point: ${point2}`) // (42, 15)
* </pre>
*
* @param p The point to convert.
* @returns The result of the conversion.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23op_Explicit(MutablePoint) Online Documentation}
*/
static convertFrom(p: MutablePoint): Point;
/**
* Performs an implicit conversion from {@link Point} to {@link MutablePoint}.
* The following example shows how to convert a {@link Point} to a {@link MutablePoint} and back:
*
* <pre>
* const point = new Point(10, 15)
*
* console.log(`Initial point: ${point}`) // (10, 15)
*
* const mPoint = point.toMutablePoint()
* // The following also works, due to an implicit conversion:
* const mPoint2 = point
*
* mPoint.x = 42
* const point2 = mPoint.toPoint()
* // The following also works, due an explicit conversion:
* const point3 = mPoint
*
* console.log(`Changed point: ${point2}`) // (42, 15)
* </pre>
*
* @param point The point to convert.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23op_Implicit(Point) Online Documentation}
*/
static convertToMutablePoint(point: Point | PointConvertible): MutablePoint | null;
/**
Determines the distance between this point and a line segment.
@param start The coordinates of the first point of the line.
@param end The coordinates of the second point of the line.
@returns The distance between this point and the closest point on the line segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23distanceToSegment(Point%2CPoint) Online Documentation}*/
distanceToSegment(start: Point, end: Point): number;
/**
* Indicates whether this instance and a specified object are equal.
* <br><b>Determining whether two points are equal</b>
* <pre>
* const p1 = new Point(2, 5)
* const p2 = new Point(23, 52)
* const p3 = new Point(2, 5)
*
* // You can call the equals method to determine equality of two points
* console.log(p1.equals(p2)) // false
* console.log(p1.equals(p3)) // true
*
* // In case you want to determine whether two Point instances are approximately equal
* // you can use equalsEps
*
* const almostP1 = new Point(2.000001, 4.99999)
* console.log(p1.equalsEps(almostP1, 1e-4)) // true
* console.log(p1.equalsEps(almostP1, 1e-8)) // false
* </pre>
*
* @param other Another object to compare to.
* @returns true if <code>other</code> and this instance are the same type and represent the same value; otherwise, false.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23equals(Object) Online Documentation}
*/
equals(other: any | null): boolean;
/**
* Determines whether the two given points have the same coordinates with respect to a certain given <code>eps</code>.
* <br><b>Determining whether two points are equal</b>
* <pre>
* const p1 = new Point(2, 5)
* const p2 = new Point(23, 52)
* const p3 = new Point(2, 5)
*
* // You can call the equals method to determine equality of two points
* console.log(p1.equals(p2)) // false
* console.log(p1.equals(p3)) // true
*
* // In case you want to determine whether two Point instances are approximately equal
* // you can use equalsEps
*
* const almostP1 = new Point(2.000001, 4.99999)
* console.log(p1.equalsEps(almostP1, 1e-4)) // true
* console.log(p1.equalsEps(almostP1, 1e-8)) // false
* </pre>
*
* @param other The other point to check for equality against this point.
* @param eps The epsilon value.
* @returns Whether both coordinates are equal with respect for the given epsilon.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23equalsEps(Point%2Cnumber) Online Documentation}
*/
equalsEps(other: Point, eps: number): boolean;
/**
Creates a {@link Point} instance from the given point-like object by performing automatic type conversion.
@param pointLike The object to convert to a {@link Point}.
@returns Either <code>pointLike</code> if it is already a {@link Point} or a new instance, initialized to the values found in
<code>pointLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23from(Point) Online Documentation}*/
static from(pointLike: Point | PointConvertible): Point;
/**
Create a constrained copy of this instance that lies within the given non-empty <code>rectangle</code>.
If the point lies outside the <code>rectangle</code> its projection on the closest border will be returned.
If the given rectangle is {@link Rect#EMPTY}, the return value is unchanged.
@param rectangle The rectangle to constrain this instance by.
@returns A constrained copy of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23getConstrained(Rect) Online Documentation}*/
getConstrained(rectangle: Rect): Point;
/**
Calculates the projection of this point onto a line.
@param anchor An anchor for the infinite line.
@param direction A direction vector for the infinite line.
@returns The point on the line that is closest to this point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23getProjectionOnLine(Point%2CPoint) Online Documentation}*/
getProjectionOnLine(anchor: Point, direction: Point): Point;
/**
Calculates the projection of this point onto a ray.
If the perpendicular projection onto the line is outside of the ray ("behind" the <code>rayStart</code>) the <code>rayStart</code> is returned
instead.
@param rayStart The start of the segment.
@param direction The direction of the ray.
@returns The point on the ray that is closest to this point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23getProjectionOnRay(Point%2CPoint) Online Documentation}*/
getProjectionOnRay(rayStart: Point, direction: Point): Point;
/**
Calculates the projection of this point onto a segment.
If the perpendicular projection onto the line is outside of the segment the nearest segment endpoint is returned.
@param start The start of the segment.
@param end The end of the segment.
@returns The point on the segment that is closest to this point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23getProjectionOnSegment(Point%2CPoint) Online Documentation}*/
getProjectionOnSegment(start: Point, end: Point): Point;
/**
Returns the hash code for this instance that is calculated using the {@link Point#x} and {@link Point#y} values.
@returns A 32-bit signed integer that is the hash code for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Determines if the point lies close to this point given an epsilon.
@param other The coordinates of the other point.
@param hitTestRadius The hit test epsilon.
@returns Whether the distance between the two points is smaller than <code>hitTestRadius</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23hits(Point%2Cnumber) Online Documentation}*/
hits(other: Point, hitTestRadius: number): boolean;
/**
Determines whether this point hits the line segment with respect to a given <code>radius</code>.
@param start The starting point of the line segment to test.
@param end The ending point of the line segment to test.
@param radius The hit test radius.
@returns Whether this point hits the given line segment within the radius.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23hitsLineSegment(Point%2CPoint%2Cnumber) Online Documentation}*/
hitsLineSegment(start: Point, end: Point, radius: number): boolean;
/**
Determines whether a polygonal line is hit by this point given an epsilon.
@param points The list of points that is treated as a polygon
@param radius A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
or equal to that value, it will be considered a hit.
@returns Whether the point hits the polygon.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23hitsPolyline(IEnumerable%3CIPoint%3E%2Cnumber) Online Documentation}*/
hitsPolyline(points: IEnumerable<IPoint> | IEnumerableConvertible<IPoint>, radius: number): boolean;
/**
Calculates a linear interpolation between two points.
The points are simply joined by a straight line. The parameter <code>t</code> defines where to estimate the value on the interpolated
line. It is <code>0</code> at <code>point1</code> and <code>1</code> at <code>point2</code>. For interpolated values between the two points <code>t</code> ranges between <code>0</code> and <code>1</code>. Values
of <code>t</code> outside this range result in extrapolation.
@param point1 The point where <code>t</code> is <code>0</code>.
@param point2 The point where <code>t</code> is <code>1</code>.
@param t The parameter that defines where to estimate the value on the interpolated line.
@returns The interpolated value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23interpolate(Point%2CPoint%2Cnumber) Online Documentation}*/
static interpolate(point1: Point, point2: Point, t: number): Point;
/**
* Applies the given matrix to this instance by calling {@link Matrix#transform} and returns the result.
* <br><b>Using the multiply methods</b>
* <pre>
* const v = new Point(2, 5)
*
* // Make the vector three times as long
* const longer = v.multiply(3)
* console.log(longer) // (6, 15)
*
* // Make the vector half as long
* const shorter = v.multiply(0.5)
* console.log(shorter) // (1, 2.5)
*
* // Translate and scale the vector via a matrix
* const matrix = new Matrix()
* matrix.scale(2, 1.5)
* matrix.translate(new Point(5, -3))
*
* const p = new Point(7, 2)
* const transformed = p.multiply(matrix)
* // ... or the transform method on Matrix
* const transformed1 = matrix.transform(p)
* console.log(transformed) // (24, -1.5)
* </pre>
*
* @param matrix The matrix to use for the transformation.
* @returns The result of the transformation.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23multiply(Matrix) Online Documentation}
*/
multiply(matrix: Matrix): Point;
/**
* Multiplies the given factor with this instance using scalar multiplication and returns the result as new instance.
* This factor is applied componentwise to {@link Point#x} and {@link Point#y}.
*
* <br><b>Using the multiply methods</b>
* <pre>
* const v = new Point(2, 5)
*
* // Make the vector three times as long
* const longer = v.multiply(3)
* console.log(longer) // (6, 15)
*
* // Make the vector half as long
* const shorter = v.multiply(0.5)
* console.log(shorter) // (1, 2.5)
*
* // Translate and scale the vector via a matrix
* const matrix = new Matrix()
* matrix.scale(2, 1.5)
* matrix.translate(new Point(5, -3))
*
* const p = new Point(7, 2)
* const transformed = p.multiply(matrix)
* // ... or the transform method on Matrix
* const transformed1 = matrix.transform(p)
* console.log(transformed) // (24, -1.5)
* </pre>
*
* @param factor The factor to scale the components by.
* @returns The result of the scalar multiplication.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23multiply(number) Online Documentation}
*/
multiply(factor: number): Point;
/**
Calculates the scalar product of this and the given vector.
@param other The other vector.
@returns The scalar product (<code>X*other.X + Y*other.Y</code>)
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23scalarProduct(Point) Online Documentation}*/
scalarProduct(other: Point): number;
/**
Calculates the scalar product of the two given points.
@param point1 The first point.
@param point2 The second point.
@returns The scalar product of the two points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23scalarProduct(Point%2CPoint) Online Documentation}*/
static scalarProduct(point1: Point, point2: Point): number;
/**
* Subtracts the given vector from this instance and returns the result as new instance.
* This operation is applied componentwise to {@link Point#x} and {@link Point#y}.
*
* <br><b>Using the + and - operators</b>
* <pre>
* const p1 = new Point(2, 5)
* const p2 = new Point(17, -6)
*
* // Translate p1 by the vector p2
* const p3 = p1.add(p2)
* console.log(`p1 + p2 = ${p3}`) // (19, -1)
*
* // Subtract p1 from p3 to get p2 again
* const p4 = p3.subtract(p1)
* console.log(p2.equals(p4)) // true
*
* // Reverse the direction of p2
* const reversed = p2.multiply(-1)
* console.log(`p2 reversed: ${reversed}`) // (-17, 6)
* </pre>
*
* @param otherPoint The second point.
* @returns The result of the vector subtraction.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23subtract(Point) Online Documentation}
*/
subtract(otherPoint: Point): Point;
/**
* Creates a {@link MutablePoint} that has the same coordinates as this instance.
* The following example shows how to convert a {@link Point} to a {@link MutablePoint} and back:
*
* <pre>
* const point = new Point(10, 15)
*
* console.log(`Initial point: ${point}`) // (10, 15)
*
* const mPoint = point.toMutablePoint()
* // The following also works, due to an implicit conversion:
* const mPoint2 = point
*
* mPoint.x = 42
* const point2 = mPoint.toPoint()
* // The following also works, due an explicit conversion:
* const point3 = mPoint
*
* console.log(`Changed point: ${point2}`) // (42, 15)
* </pre>
*
* @returns A {@link MutablePoint} with the same coordinates.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23toMutablePoint() Online Documentation}
*/
toMutablePoint(): MutablePoint;
/**
Creates a {@link Point2D} from a given {@link Point}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Point2D}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23toPoint2D() Online Documentation}*/
toPoint2D(): Point2D;
/**
Returns a human readable string of this point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23toString() Online Documentation}*/
toString(): string;
/**
Creates a {@link YPoint} from a given {@link Point}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link YPoint}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23toYPoint() Online Documentation}*/
toYPoint(): YPoint;
/**
Creates a {@link YVector} from a given {@link Point}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link YVector}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23toYVector() Online Documentation}*/
toYVector(): YVector;
/**
Gets a {@link Point} instance that has both {@link Point#x} and {@link Point#y} set to <code>0</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23ORIGIN Online Documentation}*/
static ORIGIN: Point;
/**
Gets the X coordinate of the point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23x Online Documentation}*/
readonly x: number;
/**
Gets the Y coordinate of the point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23y Online Documentation}*/
readonly y: number;
/**
Gets the length of the vector that has {@link Point#x} and {@link Point#y} as its components.
If the vector length should be compared to another length, e.g. to determine whether the point is close to something
else, it's faster to compare {@link Point#squaredVectorLength} with the squared other length, as this avoids a costly square root operation.
@see Point#squaredVectorLength
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23vectorLength Online Documentation}*/
readonly vectorLength: number;
/**
Gets the squared length of the vector that has {@link Point#x} and {@link Point#y} as its components.
@see Point#vectorLength
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23squaredVectorLength Online Documentation}*/
readonly squaredVectorLength: number;
/**
Gets the normalized version of this vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23normalized Online Documentation}*/
readonly normalized: Point;
/**
Gets whether this instance is a vertically oriented vector.
Vertical in this case does not mean exactly vertical, but rather that the vertical component is larger than the
horizontal one, i.e. the vector is more vertical than horizontal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23isVerticalVector Online Documentation}*/
readonly isVerticalVector: boolean;
/**
Gets whether this instance is a horizontally oriented vector.
Horizontal in this case does not mean exactly horizontal, but rather that the horizontal component is larger than the
vertical one, i.e. the vector is more horizontal than vertical.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point%23isHorizontalVector Online Documentation}*/
readonly isHorizontalVector: boolean;
static $class: Class;
private $_Point: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Point;
}
/** Automatic conversion to {@link Point}. */
type PointConvertible = [number, number] | {
x: number;
y: number;
};
export interface Insets extends YObject {
}
/**
* Represents insets (or outsets) for a rectangular area in two-dimensional Cartesian coordinates.
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* It is possible to specify a single {@link number}, an {@link Object} or a plain {@link Object} to define {@link Insets}:
*
* <pre>
* 5 // horizontal and vertical
* [5, 10] // vertical, horizontal
* [5, 10, 15] // top, horizontal, bottom
* [5, 10, 15, 20] // top, right, bottom, left
* { top: 5, right: 10, bottom: 15, left: 20}
* </pre>
*
* The following example shows a few ways to use {@link Insets}:
*
* <pre>
* // Create new insets
* const insets = new Insets(10, 10, 5, 2)
* // Create other insets that are uniform on every side
* const insets2 = new Insets(12)
*
* console.log(`Top:    ${insets.top}`)
* console.log(`Bottom: ${insets.bottom}`)
* console.log(`Left:   ${insets.left}`)
* console.log(`Right:  ${insets.right}`)
*
* console.log(`Total vertical:   ${insets.verticalInsets}`)
* console.log(`Total horizontal: ${insets.horizontalInsets}`)
*
* // Enlarge the first insets
* const larger = insets.getEnlarged(new Insets(2))
* console.log(larger) // left: 12, top: 12, right: 7, bottom: 4
*
* // Reduce them again
* const smaller = larger.getReduced(new Insets(1, 5, 2, 3))
* console.log(smaller) // left: 11, top: 7, right: 5, bottom: 1
*
* // Scale by a uniform factor
* const scaled = Insets.multiply(insets, 1.5)
* console.log(scaled) // left: 15, top: 15, right: 7.5, bottom: 3
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Insets Online Documentation}
*/
export class Insets {
/**
Initializes a new instance.
@param left The left inset.
@param top The top inset.
@param right The right inset.
@param bottom The bottom inset.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23Insets(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(left: number, top: number, right: number, bottom: number);
/**
Initializes a new instance using the provided inset for all four sides.
@param inset The inset to use for all sides.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23Insets(number) Online Documentation}*/
constructor(inset: number);
/**
Calculates the union of this insets and the given inset which is done by performing {@link Math#max} on all four inset values.
@param insets The insets to max with these insets.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23createUnion(Insets) Online Documentation}*/
createUnion(insets: Insets | InsetsConvertible): Insets;
/**
Devides each side of the insets by the given factor.
@param insets The insets to scales down.
@param factor The factor to scales down by.
@returns New insets scaled down by the factor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23divide(Insets%2Cnumber) Online Documentation}*/
static divide(insets: Insets | InsetsConvertible, factor: number): Insets;
/**
Indicates whether this instance and a specified object are equal.
@param other Another object to compare to.
@returns true if <code>other</code> and this instance are the same type and represent the same value; otherwise, false.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Creates a {@link Insets} instance from the given insets-like object by performing automatic type conversion.
@param insetsLike The object to convert to a {@link Insets}.
@returns Either <code>insetsLike</code> if it is already a {@link Insets} or a new instance, initialized to the values found in
<code>insetsLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23from(Insets) Online Documentation}*/
static from(insetsLike: Insets | InsetsConvertible): Insets;
/**
Creates an enlarged instance by adding the insets of the specified insets to this instance and returning the result.
@param insets The insets to add to this insets.
@see Insets#getReduced
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23getEnlarged(Insets) Online Documentation}*/
getEnlarged(insets: Insets | InsetsConvertible): Insets;
/**
Creates a reduced instance by subtracting the insets of the specified insets to this instance and returning the result.
@param insets The insets to subtract from this struct.
@see Insets#getEnlarged
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23getReduced(Insets) Online Documentation}*/
getReduced(insets: Insets | InsetsConvertible): Insets;
/**
Returns the hash code for this instance.
@returns A 32-bit signed integer that is the hash code for this instance. The hash code is computed using the {@link Insets#left}, {@link Insets#right}, {@link Insets#top}, and {@link Insets#bottom}
property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Multiplies each side of the insets by the given factor and returns the result.
@param insets The insets.
@param factor The factor to scales up by.
@returns New insets scaled up by the factor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23multiply(Insets%2Cnumber) Online Documentation}*/
static multiply(insets: Insets | InsetsConvertible, factor: number): Insets;
/**
Creates a {@link YInsets} from a given {@link Insets}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Insets}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23toInsets() Online Documentation}*/
toInsets(): YInsets;
/**
Returns a {@link string} that represents this instance.
@returns A {@link string} that represents this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23toString() Online Documentation}*/
toString(): string;
/**
The "empty" insets that has all properties set to <code>0.0d</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23EMPTY Online Documentation}*/
static EMPTY: Insets;
/**
Gets the top inset.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23top Online Documentation}*/
readonly top: number;
/**
Gets the left inset.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23left Online Documentation}*/
readonly left: number;
/**
Gets the bottom inset.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23bottom Online Documentation}*/
readonly bottom: number;
/**
Gets the right inset.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23right Online Documentation}*/
readonly right: number;
/**
Gets the vertical insets, which is the sum of {@link Insets#top} and {@link Insets#bottom}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23verticalInsets Online Documentation}*/
readonly verticalInsets: number;
/**
Gets the horizontal insets, which is the sum of {@link Insets#left} and {@link Insets#right}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23horizontalInsets Online Documentation}*/
readonly horizontalInsets: number;
/**
Gets a value indicating whether this instance is empty, that is all insets are <code>0.0d</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Insets%23isEmpty Online Documentation}*/
readonly isEmpty: boolean;
static $class: Class;
private $_Insets: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Insets;
}
/** Automatic conversion to {@link Insets}. */
type InsetsConvertible = number | [number, number] | [number, number, number] | [number, number, number, number] | {
top: number;
right: number;
bottom: number;
left: number;
};
export interface Rect extends YObject, IRectangle {
}
/**
* Represents an axis-aligned rectangle in two-dimensional Cartesian coordinates with {@link number double precision}.
* A {@link Rect} with negative {@link Rect#width} and {@link Rect#height} is considered {@link Rect#isEmpty empty} and differs in interpretation from a rectangle with zero
* {@link Rect#width} and {@link Rect#height} (which is equivalent to a point).
*
* {@link Rect} offers various utility methods for analyzing containment and intersections between rectangles and
* other geometric primitives: {@link Rect#containsWithEps}, {@link Rect#intersects},
* {@link Rect#intersectsLine}, {@link Rect#intersectsPolyline},
* {@link Rect#findLineIntersection}.
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* It is possible to specify an {@link Object} or plain {@link Object} to define the {@link Rect}:
*
* <pre>
* [5, 5, 20, 20] // x, y, width, height
* { x: 5, y: 5, width: 20, height: 20 }
* </pre>
*
* The following example shows a few ways to use {@link Rect}:
*
* <pre>
* // Create a new rectangle
* const rect1 = new Rect(-5, 2, 40, 20)
* // Create another rectangle from two corner points
* const rect2 = new Rect(new Point(17, 23), new Point(65, 30))
* // Create yet another rectangle from a center point and a size
* const rect3 = new Rect(new Point(0, 0), new Size(20, 30))
*
* // Print some general data about the first rectangle
* console.log(`Top left:     ${rect1.topLeft}`)
* console.log(`Top right:    ${rect1.topRight}`)
* console.log(`Bottom left:  ${rect1.bottomLeft}`)
* console.log(`Bottom right: ${rect1.bottomRight}`)
* console.log(`Center:       ${rect1.center}`)
* console.log(`Size:         ${rect1.width} x ${rect1.height}`)
* console.log(`Area:         ${rect1.area}`)
*
* // Rect instances integrate with the other geometry types in various ways:
*
* // Enlarge a rectangle to include a given point
* const p = new Point(47, 11)
* const union = rect1.add(p)
*
* // Check whether a given point is inside a rectangle
* console.log(union.contains(p)) // true
*
* // Check whether two rectangles intersect
* console.log(rect1.intersects(rect2)) // true
*
* // Enlarge or shrink a rectangle
* const larger = rect2.getEnlarged(new Insets(2, 5, 2, 10))
* console.log(`Original rectangle: ${rect2}`)
* console.log(`Enlarged rectangle: ${larger}`)
*
* const smaller = rect3.getReduced(5)
* console.log(`Original rectangle: ${rect3}`)
* console.log(`Reduced rectangle: ${smaller}`)
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Rect Online Documentation}
*/
export class Rect {
/**
Initializes a new instance of the {@link Rect} class with the initial values obtained from the given rectangle.
@param rectangle The rectangle to retrieve the initial values from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23Rect(IRectangle) Online Documentation}*/
constructor(rectangle: IRectangle);
/**
Initializes a new instance.
@param x The {@link Rect#x} coordinate of the top left corner.
@param y The {@link Rect#y} coordinate of the top left corner.
@param width The {@link Rect#width} of the rectangle.
@param height The {@link Rect#height} of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23Rect(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(x: number, y: number, width: number, height: number);
/**
Initializes a new instance using the {@link Rect#topLeft} corner and the {@link Rect#size}.
@param location The top left corner.
@param size The size to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23Rect(Point%2CSize) Online Documentation}*/
constructor(location: Point, size: Size);
/**
Initializes a new instance using two point to define the bounds
This will always result in non-{@link Rect#isEmpty} rectangles as the coordinates of the points are sorted so that the smaller gets assigned
to {@link Rect#x} and {@link Rect#y} respectively and the greater ones define the {@link Rect#width} and {@link Rect#height}.
@param p1 The first point to determine the bounds.
@param p2 The second point to determine the bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23Rect(Point%2CPoint) Online Documentation}*/
constructor(p1: Point, p2: Point);
/**
Returns the union of this rectangle and the given point.
@param point The point to include in the rectangle's bounds.
@returns The enlarged rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23add(Point) Online Documentation}*/
add(point: Point): Rect;
/**
Returns the union of the given rectangles.
{@link Rect#isEmpty empty} rectangles will not be considered.
@param firstRectangle The first rectangle to use for the union.
@param secondRectangle The second rectangle to use for the union.
@returns A rectangle that encompasses the area of the two given rectangles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23add(Rect%2CRect) Online Documentation}*/
static add(firstRectangle: Rect, secondRectangle: Rect): Rect;
/**
Determines whether this rectangle contains the specified point with respect to a given epsilon.
@param point The point to test.
@param [eps=0] The positive epsilon distance that the point may lie outside the rectangle and still be considered contained.
@returns <code>true</code> if this rectangle contains the specified point; <code>false</code> otherwise. An {@link Rect#isEmpty empty} instance never contains any point. A
point is considered to be contained in the rectangle when the coordinates are not smaller than {@link Rect#minX} and {@link Rect#minY} minus <code>eps</code> nor
greater than {@link Rect#maxX} and {@link Rect#maxY} plus <code>eps</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23containsWithEps(Point%2Cnumber) Online Documentation}*/
containsWithEps(point: Point, eps?: number): boolean;
/**
Performs an explicit conversion from {@link MutableRectangle} to {@link Rect}.
@param rectangle The rectangle to convert.
@returns The result of the conversion.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23op_Explicit(MutableRectangle) Online Documentation}*/
static convertFrom(rectangle: MutableRectangle): Rect;
/**
Performs an explicit conversion from {@link Rect} to {@link MutableRectangle}.
@param rectangle The rectangle to convert.
@returns The result of the conversion.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23op_Explicit(Rect) Online Documentation}*/
static convertToMutableRectangle(rectangle: Rect | RectConvertible): MutableRectangle | null;
/**
Indicates whether this instance and a specified object are equal.
@param other Another object to compare to.
@returns true if <code>other</code> and this instance are the same type and represent the same value; otherwise, false.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Finds the intersection between a rectangle and a line.
@param inner The coordinates of a point lying inside the rectangle.
@param outer The coordinates of a point lying outside the rectangle.
@returns The intersection point if the inner point lies inside the rectangle, the outer point lies outside the rectangle and thus
an intersection point has been found, or <code>null</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23findLineIntersection(Point%2CPoint) Online Documentation}*/
findLineIntersection(inner: Point, outer: Point): Point | null;
/**
Creates a {@link Rect} instance from the given rectangle-like object by performing automatic type conversion.
@param rectLike The object to convert to a {@link Rect}.
@returns Either <code>rectLike</code> if it is already a {@link Rect} or a new instance, initialized to the values found in <code>rectLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23from(Rect) Online Documentation}*/
static from(rectLike: Rect | RectConvertible): Rect;
/**
Creates a new instance given the center of the rectangle and its size.
@param center The center to use.
@param size The size to assign.
@returns An instance whose center is set to <code>center</code> and size is <code>size</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23fromCenter(IPoint%2CISize) Online Documentation}*/
static fromCenter(center: IPoint | IPointConvertible, size: ISize | ISizeConvertible): Rect;
/**
Creates an enlarged instance that is the same as this one but enlarged by the specified insets.
If this instance {@link Rect#isEmpty}, the same will be returned.
@param insets The insets to use to add to the instance.
@see Rect#getReduced
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23getEnlarged(Insets) Online Documentation}*/
getEnlarged(insets: Insets | InsetsConvertible): Rect;
/**
Creates a reduced instance that is the same as this one but reduced by the specified insets.
@param insets The insets to use to subtract from the instance.
@see Rect#getEnlarged
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23getReduced(Insets) Online Documentation}*/
getReduced(insets: Insets | InsetsConvertible): Rect;
/**
Applies the given transformation matrix to the four corner points and returns the new bounding box that includes all of
them.
@param transform The transform matrix to apply.
@returns A new {@link Rect} that represents the bounding box around the four transformed corner points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23getTransformed(Matrix) Online Documentation}*/
getTransformed(transform: Matrix): Rect;
/**
Returns a translated instance of this which has modified {@link Rect#x} and {@link Rect#y} by the given amount.
@param delta The delta to add to {@link Rect#x} and {@link Rect#y}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23getTranslated(Point) Online Documentation}*/
getTranslated(delta: Point): Rect;
/**
Returns the hash code for this instance.
@returns A 32-bit signed integer that is the hash code for this instance. The hash code is computed using the {@link Rect#x}, {@link Rect#y}, {@link Rect#width}, and {@link Rect#height}
property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Determines whether this rectangle intersects an {@link IOrientedRectangle}, given an epsilon.
@param rectangle The {@link IOrientedRectangle} to test.
@param [eps=0] A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
or equal to that value, it will be considered a hit.
@returns Whether they have a non-empty intersection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23intersects(IOrientedRectangle%2Cnumber) Online Documentation}*/
intersects(rectangle: IOrientedRectangle, eps?: number): boolean;
/**
Determines whether the bounds of this instance intersect with the bounds of the specified rectangle.
@param rectangle The rectangle to check.
@returns Whether both instances are non-empty and have an intersection with positive {@link Rect#area}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23intersects(Rect) Online Documentation}*/
intersects(rectangle: Rect): boolean;
/**
Determines whether this rectangle intersects a line.
@param start The first end point of the line.
@param end The second end point of the line.
@returns Whether the line intersects the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23intersectsLine(Point%2CPoint) Online Documentation}*/
intersectsLine(start: Point, end: Point): boolean;
/**
Determines whether a rectangle intersects a polygonal line.
@param points The list of points that is interpreted as a number of line segments.
@returns <code>true</code> if the rectangle intersects at least one segment of the line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23intersectsPolyline(IEnumerable%3CIPoint%3E) Online Documentation}*/
intersectsPolyline(points: IEnumerable<IPoint> | IEnumerableConvertible<IPoint>): boolean;
/**
Creates a {@link MutableRectangle} using the values from this instance.
@returns An instance that has been initialized from the values of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23toMutableRectangle() Online Documentation}*/
toMutableRectangle(): MutableRectangle;
/**
Creates a {@link Rectangle2D Rectangle2D} from a given {@link Rect}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Rectangle2D Rectangle2D}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23toRectangle2D() Online Documentation}*/
toRectangle2D(): Rectangle2D;
/**
Creates a human readable from of this instance.
@returns A string describing the properties of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23toString() Online Documentation}*/
toString(): string;
/**
Creates a {@link YRectangle} from a given {@link Rect}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link YRectangle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23toYRectangle() Online Documentation}*/
toYRectangle(): YRectangle;
/**
An {@link Rect#isEmpty empty} rectangle.
{@link Rect#width} and {@link Rect#height} are set to <code>-1.0d</code>, {@link Rect#x} and {@link Rect#y} are set to <code>0.0d</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23EMPTY Online Documentation}*/
static EMPTY: Rect;
/**
An infinite rectangle.
{@link Rect#width} and {@link Rect#height} are set to {@link Number#POSITIVE_INFINITY}, {@link Rect#x} and {@link Rect#y} are set to {@link Number#NEGATIVE_INFINITY}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23INFINITE Online Documentation}*/
static INFINITE: Rect;
/**
Gets the X coordinate of the upper left corner of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23x Online Documentation}*/
readonly x: number;
/**
Gets the Y coordinate of the upper left corner of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23y Online Documentation}*/
readonly y: number;
/**
Gets the width of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23width Online Documentation}*/
readonly width: number;
/**
Gets the height of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23height Online Documentation}*/
readonly height: number;
/**
Gets the center X coordinate of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23centerX Online Documentation}*/
readonly centerX: number;
/**
Gets the center Y coordinate of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23centerY Online Documentation}*/
readonly centerY: number;
/**
Gets the center of the rectangle using {@link Rect#centerX} and {@link Rect#centerY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23center Online Documentation}*/
readonly center: Point;
/**
Gets the coordinates of the top left corner of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23topLeft Online Documentation}*/
readonly topLeft: Point;
/**
Gets the coordinates of the top right corner of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23topRight Online Documentation}*/
readonly topRight: Point;
/**
Gets the coordinates of the bottom left corner of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23bottomLeft Online Documentation}*/
readonly bottomLeft: Point;
/**
Gets the coordinates of the bottom right corner of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23bottomRight Online Documentation}*/
readonly bottomRight: Point;
/**
Gets the size of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23size Online Documentation}*/
readonly size: Size;
/**
Gets a value indicating whether this instance is considered empty.
Yields <code>true</code> if this at least one of {@link Rect#width} or {@link Rect#height} is negative; <code>false</code> otherwise. Note that a width and height of both <code>0.0d</code> is not
considered empty for this implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23isEmpty Online Documentation}*/
readonly isEmpty: boolean;
/**
Gets a value indicating whether this instance is finite.
This means that the instance is not {@link Rect#isEmpty empty} and neither {@link Rect#width} nor {@link Rect#height} are infinite.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23isFinite Online Documentation}*/
readonly isFinite: boolean;
/**
Gets the coordinates of the right border for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23maxX Online Documentation}*/
readonly maxX: number;
/**
Gets the coordinates of the lower border for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23maxY Online Documentation}*/
readonly maxY: number;
/**
Gets the X coordinates of the left side of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23minX Online Documentation}*/
readonly minX: number;
/**
Gets the Y coordinates of the top side of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23minY Online Documentation}*/
readonly minY: number;
/**
Gets the area of this instance which is the product of {@link Rect#width} and {@link Rect#height}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rect%23area Online Documentation}*/
readonly area: number;
static $class: Class;
private $_Rect: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Rect;
}
/** Automatic conversion to {@link Rect}. */
type RectConvertible = [number, number, number, number] | {
x: number;
y: number;
width: number;
height: number;
};
export interface Tangent extends YObject {
}
/**
An immutable class that represents a tangent in a 2-d Cartesian coordinate space with double values.
The tangent consists of its point of tangency ({@link Tangent#point}) and a direction ({@link Tangent#vector}).
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Tangent Online Documentation}*/
export class Tangent {
/**
Initializes a new instance of the {@link Tangent} class using the given values for the {@link Tangent#point} and {@link Tangent#vector} properties.
@param point The point of tangency.
@param vector The directional vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Tangent Online Documentation}*/
constructor(point: Point | PointConvertible, vector: Point | PointConvertible);
/**
Gets the point of tangency.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Tangent%23point Online Documentation}*/
readonly point: Point;
/**
Gets the directional vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Tangent%23vector Online Documentation}*/
readonly vector: Point;
static $class: Class;
private $_Tangent: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Tangent;
}
export interface GeomUtilities extends YObject {
}
/**
Provides methods for geometric operations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeomUtilities Online Documentation}*/
export class GeomUtilities {
/**
Checks whether an ellipse contains the given point.
@param bounds The coordinates of the bounds of the ellipse's enclosing rectangle.
@param point The coordinates of the point to test.
@param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
or equal to that value, it will be considered a hit.
@returns Whether the point lies within the ellipse
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeomUtilities%23ellipseContains(Rect%2CPoint%2Cnumber) Online Documentation}*/
static ellipseContains(bounds: Rect, point: Point, eps: number): boolean;
/**
Returns the ellipse/line intersection point for the given point pair.
This will always return the intersection point that lies in the direction from inner to outer.
@param bounds The coordinates of the bounds of the ellipse's enclosing rectangle.
@param inner The coordinates of a point lying inside the ellipse.
@param outer The coordinates of a point lying outside the ellipse.
@returns The intersection point iff the inner point lies inside the ellipse and an intersection point has been found, otherwise
<code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeomUtilities%23findEllipseLineIntersection(Rect%2CPoint%2CPoint) Online Documentation}*/
static findEllipseLineIntersection(bounds: Rect, inner: Point, outer: Point): Point | null;
/**
* Finds the intersection between a line segment and an infinite ray.
* The ray is described using an anchor point and a ray direction. The direction vector does not need to be normalized. In
* order to obtain the intersection point do the following:
*
* <pre>
* const factor = GeomUtilities.findRayIntersection(l1, l2, anchor, ray)
* if (factor < Number.POSITIVE_INFINITY) {
*   const intersection = anchor.add(ray.multiply(factor))
* }
* </pre>
*
* @param start The coordinates of the first end point of the line segment.
* @param end The coordinates of the second end point of the line segment.
* @param anchor The coordinates of the starting point of the ray.
* @param rayDirection The direction vector of the ray.
* @returns The distance factor or {@link Number#POSITIVE_INFINITY} if the ray does not intersect the line.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GeomUtilities%23findRayIntersection(Point%2CPoint%2CPoint%2CPoint) Online Documentation}
*/
static findRayIntersection(start: Point, end: Point, anchor: Point, rayDirection: Point): number;
static $class: Class;
private $_GeomUtilities: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GeomUtilities;
}
export interface IMutableOrientedRectangle extends IOrientedRectangle, IMutableSize {
}
/**
Interface for mutable oriented rectangles in 2D coordinate space with double precision coordinates.
@see ISize
@see IOrientedRectangle
@see OrientedRectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableOrientedRectangle Online Documentation}*/
export abstract class IMutableOrientedRectangle {
/**
Sets the anchor vector of the oriented rectangle to the given value.
@param location The coordinates of the new anchor location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableOrientedRectangle%23setAnchor(Point) Online Documentation}*/
setAnchor(location: Point): void;
/**
Sets the center of the oriented rectangle to the given value.
@param center The coordinates of the new center.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableOrientedRectangle%23setCenter(Point) Online Documentation}*/
setCenter(center: Point): void;
/**
Sets the orientation of this oriented rectangle by modifying the up vector components.
It is up to the caller to assure that the values describe a vector of length 1.
@param upx The x component of the normalized up vector.
@param upy The y component of the normalized up vector.
@see IOrientedRectangle#upX
@see IOrientedRectangle#upY
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableOrientedRectangle%23setUpVector(number%2Cnumber) Online Documentation}*/
abstract setUpVector(upx: number, upy: number): void;
/**
Gets or sets the x coordinate for the anchor of the rectangle.
Implementations may adjust the coordinates internally, e.g. to automatically snap the coordinates to grid points. So
depending on context the value that can be read might not necessarily be the exact same value that has just been
written.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableOrientedRectangle%23anchorX Online Documentation}*/
abstract anchorX: number;
/**
Gets or sets the y coordinate for the anchor of the rectangle.
Implementations may adjust the coordinates internally, e.g. to automatically snap the coordinates to grid points. So
depending on context the value that can be read might not necessarily be the exact same value that has just been
written.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableOrientedRectangle%23anchorY Online Documentation}*/
abstract anchorY: number;
static $class: Class;
private $_IMutableOrientedRectangle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IMutableOrientedRectangle;
}
export interface IMutablePoint extends IPoint {
}
/**
Interface for a mutable point in 2D coordinate space with double precision coordinates.
This interface provides read and write access to the coordinates. It adds write access to the read only interface
{@link IPoint}.
@see IPoint
@see MutablePoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutablePoint Online Documentation}*/
export abstract class IMutablePoint {
/**
Sets the coordinates of the point to the given values.
@param location The new location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutablePoint%23relocate(IPoint) Online Documentation}*/
relocate(location: IPoint): void;
/**
Sets the coordinates of the point to the given values.
@param x The new x coordinate
@param y The new y coordinate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutablePoint%23relocate(number%2Cnumber) Online Documentation}*/
relocate(x: number, y: number): void;
/**
Gets or sets the x coordinate for this point.
Implementations may adjust the coordinates internally, e.g. to automatically snap the coordinates to grid points. So
depending on context the value that can be read might not necessarily be the exact same value that has just been
written.
@see IPoint#x
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutablePoint%23x Online Documentation}*/
abstract x: number;
/**
Gets or sets the y coordinate for this point.
Implementations may adjust the coordinates internally, e.g. to automatically snap the coordinates to grid points. So
depending on context the value that can be read might not necessarily be the exact same value that has just been
written.
@see IPoint#y
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutablePoint%23y Online Documentation}*/
abstract y: number;
static $class: Class;
private $_IMutablePoint: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IMutablePoint;
}
export interface IMutableRectangle extends IMutableSize, IMutablePoint, IRectangle {
}
/**
Interface for mutable rectangles aligned to the axes in 2D coordinate space with double precision coordinates.
This interface provides read and write access to the features of the rectangle. This interface does not declare any
additional methods. It combines the {@link IMutablePoint} interface that describes the upper left corner of the
rectangle with the {@link IMutableSize} interface that describes the width and height of the rectangle.
@see ISize
@see IPoint
@see MutableRectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableRectangle Online Documentation}*/
export abstract class IMutableRectangle {
/**
Adds a point to a rectangle, possibly enlarging the rectangle.
If the rectangle is initially empty, i.e. its width or height is negative, the bounds of the rectangle will be set to
(p.x, p.y, 0, 0)
@param point The coordinate to include in the bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableRectangle%23add(IPoint) Online Documentation}*/
add(point: IPoint): void;
/**
Adds a rectangle to another one.
The result is placed into the first rectangle, which is returned. If either of the two rectangles is empty, i.e. its
width or height is negative, the result will be the other rectangle. If either of the two rectangles is infinite, i.e.
its width or height is positive infinity, the result is an infinite rectangle.
@param rectangle2 The rectangle to be added.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableRectangle%23add(IRectangle) Online Documentation}*/
add(rectangle2: IRectangle): void;
/**
Applies a new position and size to a given mutable rectangle.
@param position The new location.
@param size The new size.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableRectangle%23reshape(IPoint%2CISize) Online Documentation}*/
reshape(position: IPoint, size: ISize): void;
/**
Applies a new position and size to a given mutable rectangle.
@param newRectangle The bounds to set to the <code>rectangle</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableRectangle%23reshape(IRectangle) Online Documentation}*/
reshape(newRectangle: IRectangle): void;
/**
Applies a new position and size to a given mutable rectangle.
@param x The new x coordinate of the upper left corner of the rectangle.
@param y The new y coordinate of the upper left corner of the rectangle.
@param width The new width of the rectangle.
@param height The new height of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableRectangle%23reshape(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
reshape(x: number, y: number, width: number, height: number): void;
/**
Sets the center of the rectangle to the provided value.
@param center The new center coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableRectangle%23setCenter(Point) Online Documentation}*/
setCenter(center: Point): void;
/**
Creates the union of two rectangles, placing the result in the this parameter.
Either of the two parameters rectangles may be the same as the first parameter. The result is placed into the this
parameter. This method treats rectangles with negative width or height as empty. If one of the rectangles is an infinite
rectangle, an infinite rectangle is returned.
@param rectangle1 The first rectangle to create the union of.
@param rectangle2 The second rectangle to create the union of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableRectangle%23setToUnion(IRectangle%2CIRectangle) Online Documentation}*/
setToUnion(rectangle1: IRectangle, rectangle2: IRectangle): void;
static $class: Class;
private $_IMutableRectangle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IMutableRectangle;
}
export interface IMutableSize extends ISize {
}
/**
Interface for a mutable size in 2D coordinate space with double precision width and height.
This interface provides read and write access to the size. It extends the read only interface {@link ISize}.
@see ISize
@see MutableSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableSize Online Documentation}*/
export abstract class IMutableSize {
/**
Sets the coordinates of the size to the given values.
@param newSize The new location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableSize%23resize(ISize) Online Documentation}*/
resize(newSize: ISize): void;
/**
Sets the coordinates of the size to the given values.
@param newWidth The new x coordinate
@param newHeight The new y coordinate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableSize%23resize(number%2Cnumber) Online Documentation}*/
resize(newWidth: number, newHeight: number): void;
/**
Gets or sets the width of this instance.
Implementations may adjust the values internally, e.g. to automatically snap the values to grid sizes. So depending on
context the value that can be read might not necessarily be the exact same value that has just been written.
@see ISize#width
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableSize%23width Online Documentation}*/
abstract width: number;
/**
Gets or sets the height of this instance.
Implementations may adjust the values internally, e.g. to automatically snap the values to grid sizes. So depending on
context the value that can be read might not necessarily be the exact same value that has just been written.
@see ISize#height
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMutableSize%23height Online Documentation}*/
abstract height: number;
static $class: Class;
private $_IMutableSize: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IMutableSize;
}
export interface IOrientedRectangle extends ISize {
}
/**
Interface for oriented rectangles in 2D coordinate space with double precision coordinates.
The rectangle described by this interface has its lower left corner at the anchor as described by the {@link IOrientedRectangle#anchorX} and {@link IOrientedRectangle#anchorY} properties
and is oriented in space so that its upper left corner lies in direction of the ({@link IOrientedRectangle#upX},{@link IOrientedRectangle#upY}) vector. The size of the instance
does not describe the bounds of the instance but the lengths of the sides of the rectangle. An {@link IRectangle}
would thus be an oriented rectangle, whose anchor is the lower left corner of the rectangle, whose size is the same as
that of the rectangle and whose up vector is <code>(0,-1)</code>. This interface provides read access to the features of the oriented
rectangle only, however this does not mean that an instance that implements <code>IOrientedRectangle</code> will always return the
same values for its properties. Often times the instance provides a dynamic read access to the current state of a
rectangle. It depends on the context whether it is allowed to or even necessary to copy the state of the properties or
whether the reference to the instance should always be used to query the values.
@see ISize
@see IRectangle
@see OrientedRectangle
@see MutableRectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle Online Documentation}*/
export abstract class IOrientedRectangle {
/**
Determines whether the given oriented rectangle contains the provided point, using an epsilon value.
@param point The coordinates of the point to test.
@param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
or equal to that value, it will be considered a hit.
@returns <code>true</code> iff the point lies inside the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23containsWithEps(Point%2Cnumber) Online Documentation}*/
containsWithEps(point: Point, eps: number): boolean;
/**
Creates a transformation matrix that can be used to transform points that are in the local coordinate system of the
oriented rectangle if the top-left corner is the origin.
@returns A matrix that can be used to transform from oriented rectangle coordinates to world coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23createTransform() Online Documentation}*/
createTransform(): Matrix;
/**
Determines whether the oriented rectangle contains the provided point, using an epsilon value.
@param location The coordinates of the point to test.
@param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
or equal to that value, it will be considered a hit.
@returns <code>true</code> iff the point lies inside the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23hits(Point%2Cnumber) Online Documentation}*/
hits(location: Point, eps: number): boolean;
/**
Creates a {@link YOrientedRectangle} from a given {@link IOrientedRectangle}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link YOrientedRectangle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23toOrientedRectangle() Online Documentation}*/
toOrientedRectangle(): YOrientedRectangle;
/**
Gets the x coordinate of the anchor of the oriented rectangle.
The anchor is the lower left corner of the oriented rectangle if the up vector is <code>(0,-1)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23anchorX Online Documentation}*/
abstract readonly anchorX: number;
/**
Gets the y coordinate of the anchor of the oriented rectangle.
The anchor is the lower left corner of the oriented rectangle if the up vector is <code>(0,-1)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23anchorY Online Documentation}*/
abstract readonly anchorY: number;
/**
Gets the x value of the up vector.
The up vector points from the lower left corner to the upper left corner and is always normalized, i.e. it's length is
always 1.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23upX Online Documentation}*/
abstract readonly upX: number;
/**
Gets the y value of the up vector.
The up vector points from the lower left corner to the upper left corner and is always normalized, i.e. it's length is
always 1.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23upY Online Documentation}*/
abstract readonly upY: number;
/**
An immutable empty {@link IOrientedRectangle} with width and height of <code>-1</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23EMPTY Online Documentation}*/
static EMPTY: IOrientedRectangle;
/**
Gets the up vector of the oriented rectangle as a {@link Point} struct.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23upVector Online Documentation}*/
readonly upVector: Point;
/**
Gets the anchor location of the oriented rectangle as a {@link Point} struct.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23anchorLocation Online Documentation}*/
readonly anchorLocation: Point;
/**
Gets the current center of the oriented rectangle as a {@link Point} struct.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23orientedRectangleCenter Online Documentation}*/
readonly orientedRectangleCenter: Point;
/**
Determines the bounds of an oriented rectangle.
Bounding rectangles are parallel to the coordinate axes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrientedRectangle%23bounds Online Documentation}*/
readonly bounds: Rect;
static $class: Class;
private $_IOrientedRectangle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IOrientedRectangle;
}
export interface IPoint {
}
/**
* Interface for a point in 2D coordinate space with double precision coordinates.
* This interface provides read access to the coordinates only, however this does not mean that an instance that implements
* <code>IPoint</code> will always return the same values for the coordinate properties. Often times the instance provides a dynamic
* read access to the current state of a point. It depends on the context whether it is allowed to or even necessary to
* copy the state of the properties or whether the reference to the instance should always be used to query the values.
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* It is possible to specify an {@link Object} or plain {@link Object} to define the {@link IPoint}:
*
* <pre>
* [5, 5] // x, y
* { x: 5, y: 5 }
* </pre>
*
* @see IMutablePoint
* @see MutablePoint
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPoint Online Documentation}
*/
export abstract class IPoint {
/**
Calculates the Euclidean distance between two points.
@param point2 The second point.
@returns The distance between the two points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPoint%23distanceTo(IPoint) Online Documentation}*/
distanceTo(point2: IPoint): number;
/**
Creates a {@link Point} instance from the given point-like object by performing automatic type conversion.
@param pointLike The object to convert to a {@link Point}.
@returns Either <code>pointLike</code> if it is already a {@link Point} or a new instance, initialized to the values found in
<code>pointLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPoint%23from(Point) Online Documentation}*/
static from(pointLike: Point | PointConvertible): Point;
/**
Copies the current values of the coordinates of the point to a {@link Point} struct.
This method is useful to obtain a copy of the state and for making use of the various utility methods that are provided
by {@link Point}.
@returns The current values of the coordinates of the point.
@see Point#toMutablePoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPoint%23toPoint() Online Documentation}*/
toPoint(): Point;
/**
Gets the current x coordinate of this point.
Depending on context the values returned may change over time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPoint%23x Online Documentation}*/
abstract x: number;
/**
Gets the current y coordinate of this point.
Depending on context the values returned may change over time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPoint%23y Online Documentation}*/
abstract y: number;
static $class: Class;
private $_IPoint: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPoint;
}
/** Automatic conversion to {@link IPoint}. */
type IPointConvertible = [number, number] | {
x: number;
y: number;
};
export interface IRectangle extends ISize, IPoint {
}
/**
* Interface for rectangles aligned to the axes in 2D coordinate space with double precision coordinates.
* This interface provides read access to the features of the rectangle only, however this does not mean that an instance
* that implements <code>IRectangle</code> will always return the same values for its properties. Often times the instance provides a
* dynamic read access to the current state of a rectangle. It depends on the context whether it is allowed to or even
* necessary to copy the state of the properties or whether the reference to the instance should always be used to query
* the values. This interface does not declare any additional methods. It combines the {@link IPoint} interface
* that describes the upper left corner of the rectangle with the {@link ISize} interface that describes the width
* and height of the rectangle.
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* It is possible to specify an {@link Object} or plain {@link Object} to define the {@link Rect}:
*
* <pre>
* [5, 5, 20, 20] // x, y, width, height
* { x: 5, y: 5, width: 20, height: 20 }
* </pre>
*
* @see IMutableRectangle
* @see IMutablePoint
* @see ISize
* @see IPoint
* @see MutableRectangle
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle Online Documentation}
*/
export abstract class IRectangle {
/**
Determines whether the given rectangle contains the provided point.
@param point The point to test.
@returns <code>true</code> iff the point lies inside the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23contains(IPoint) Online Documentation}*/
contains(point: IPoint): boolean;
/**
Creates a {@link Rect} instance from the given rectangle-like object by performing automatic type conversion.
@param rectLike The object to convert to a {@link Rect}.
@returns Either <code>rectLike</code> if it is already a {@link Rect} or a new instance, initialized to the values found in <code>rectLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23from(Rect) Online Documentation}*/
static from(rectLike: Rect | RectConvertible): Rect;
/**
Copies the current values of the rectangle to {@link Rect} struct.
This method can be used to obtain a copy of the current state of the rectangle and for using the utility methods that
are available for the {@link Rect} type.
@returns A {@link Rect} that holds the values of the <code>rectangle</code> at the time of the invocation.
@see IMutableRectangle#reshape
@see Rect#toMutableRectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23toRect() Online Documentation}*/
toRect(): Rect;
/**
Gets the coordinates of the top left corner of the rectangle as a {@link Point}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23topLeft Online Documentation}*/
readonly topLeft: Point;
/**
Gets the coordinates of the bottom left corner of the rectangle as a {@link Point}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23bottomLeft Online Documentation}*/
readonly bottomLeft: Point;
/**
Gets the coordinates of the center of the rectangle as a {@link Point}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23center Online Documentation}*/
readonly center: Point;
/**
Gets the maximum X coordinate of the rectangle.
This is the x coordinate of the right side of the rectangle, or the left side if the rectangle is {@link IRectangle#isEmpty}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23maxX Online Documentation}*/
readonly maxX: number;
/**
Determines whether the specified rectangle is empty.
{@link IRectangle} instances are considered empty if their {@link ISize#width} or {@link ISize#height} is less than <code>0.0d</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23isEmpty Online Documentation}*/
readonly isEmpty: boolean;
/**
Gets the maximum Y coordinate of the rectangle.
This is the y coordinate of the bottom side of the rectangle, or the top side if the rectangle is {@link IRectangle#isEmpty}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23maxY Online Documentation}*/
readonly maxY: number;
/**
Gets the coordinates of the top right corner of the rectangle as a {@link Point}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23topRight Online Documentation}*/
readonly topRight: Point;
/**
Gets the coordinates of the bottom right corner of the rectangle as a {@link Point}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRectangle%23bottomRight Online Documentation}*/
readonly bottomRight: Point;
static $class: Class;
private $_IRectangle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IRectangle;
}
/** Automatic conversion to {@link IRectangle}. */
type IRectangleConvertible = [number, number, number, number] | {
x: number;
y: number;
width: number;
height: number;
};
export interface ISize {
}
/**
* Interface for an object that has a size in 2D coordinate space with double precision floating point size values.
* This interface provides read access to the size only, however this does not mean that an instance that implements <code>ISize</code>
* will always return the same values for the size properties. Often times the instance provides a dynamic read access to
* the current state of an instance. It depends on the context whether it is allowed to or even necessary to copy the state
* of the properties or whether the reference to the instance should always be used to query the values.
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* It is possible to specify an {@link Object} or plain {@link Object} to define the {@link ISize}:
*
* <pre>
* [20, 20] // width, height
* { width: 20, height: 20 }
* </pre>
*
* @see IMutableSize
* @see MutableSize
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ISize Online Documentation}
*/
export abstract class ISize {
/**
Creates a {@link Size} instance from the given size-like object by performing automatic type conversion.
@param sizeLike The object to convert to a {@link Size}.
@returns Either <code>sizeLike</code> if it is already a {@link Size} or a new instance, initialized to the values found in <code>sizeLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISize%23from(Size) Online Documentation}*/
static from(sizeLike: Size | SizeConvertible): Size;
/**
Converts the {@link ISize} to a {@link Size} struct.
@returns A {@link Size} struct that has been initialized with the current values of <code>size</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISize%23toSize() Online Documentation}*/
toSize(): Size;
/**
Gets the current width of this instance.
Depending on context the values returned may change over time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISize%23width Online Documentation}*/
abstract width: number;
/**
Gets the current height of this instance.
Depending on context the values returned may change over time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISize%23height Online Documentation}*/
abstract height: number;
static $class: Class;
private $_ISize: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ISize;
}
/** Automatic conversion to {@link ISize}. */
type ISizeConvertible = [number, number] | {
width: number;
height: number;
};
/**
Enumeration constant for use in {@link Matrix} operations.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MatrixOrder Online Documentation}*/
export enum MatrixOrder {
/**
Constant that defines a prepend operation for matrix multiplications.
Prepending a matrix <b>T</b> to another matrix <b>M</b> results in the operation <b>M</b>′&nbsp;=&nbsp;<b>M</b>&nbsp;<b>T</b>. Essentially, this means that <b>T</b> is applied before
<b>M</b> when applying <b>M</b>′ to a vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MatrixOrder%23PREPEND Online Documentation}*/
PREPEND,
/**
Constant that defines an append operation for matrix multiplications.
Appending a matrix <b>T</b> to another matrix <b>M</b> results in the operation <b>M</b>′&nbsp;=&nbsp;<b>T</b>&nbsp;<b>M</b>. Essentially, this means that <b>T</b> is applied after <b>M</b>
when applying <b>M</b>′ to a vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MatrixOrder%23APPEND Online Documentation}*/
APPEND
}
/** Automatic conversion to {@link MatrixOrder}. */
type MatrixOrderStringValues = "prepend" | "append";
export interface Matrix extends YObject, ICloneable {
}
/**
An implementation of a matrix that represents an affine transformation and works with {@link Point}.
The matrix is interpreted row-major. The rows are defined as follows: [ m11 m12 dx ], [ m21 m22 dy ], ([ 0 0 1 ]
implicitly).
When transforming a vector <b>v</b> using this matrix, the multiplication is done in this order: <b>v</b>'&nbsp;=&nbsp;<b>M</b>&nbsp;<b>v</b>.
{@link MatrixOrder#PREPEND Prepending} a matrix <b>T</b> to this instance results in the operation <b>M</b>′&nbsp;=&nbsp;<b>M</b>&nbsp;<b>T</b>. In concept, this means that <b>T</b> is applied before <b>M</b>
when applying <b>M</b>' to a vector. {@link MatrixOrder#APPEND Appending} <b>T</b> to <b>M</b> results in <b>M</b>′&nbsp;=&nbsp;<b>T</b>&nbsp;<b>M</b>.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix Online Documentation}*/
export class Matrix {
/**
Create a matrix using the provided matrix entries.
@param m11
@param m12
@param m21
@param m22
@param dx
@param dy
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23Matrix(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(m11: number, m12: number, m21: number, m22: number, dx: number, dy: number);
/**
Create an identity matrix.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23Matrix() Online Documentation}*/
constructor();
/**
Applies this matrix to the given SVG element.
@param element The element to transform.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23applyTo(SVGElement) Online Documentation}*/
applyTo(element: SVGElement): void;
/**
Applies this matrix to the given HTML canvas rendering context.
@param ctx The context to transform.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23applyTo(CanvasRenderingContext2D) Online Documentation}*/
applyTo(ctx: CanvasRenderingContext2D): void;
/**
Creates a matrix rotation instance around the origin.
@param theta The rotation angle in radians.
@returns A new matrix.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23createRotateInstance(number) Online Documentation}*/
static createRotateInstance(theta: number): Matrix;
/**
Determines whether <code>this</code> has the same values like <code>other</code>.
@param other The object to compare <code>this</code> to. Must be of the same type.
@returns <code>true</code> if <code>this</code> has the same values like the other object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23hasSameValue(Matrix) Online Documentation}*/
hasSameValue(other: Matrix | null): boolean;
/**
Inverts this instance in place.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23invert() Online Documentation}*/
invert(): void;
/**
Multiplies this matrix instance by the given instance using the given order.
@param matrix The matrix to multiply with this one.
@param [order=MatrixOrder.PREPEND] The order of the multiplication. {@link MatrixOrder#PREPEND} if not specified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23multiply(Matrix%2CMatrixOrder) Online Documentation}*/
multiply(matrix: Matrix, order?: MatrixOrder | MatrixOrderStringValues): void;
/**
Resets this instance to the identity.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23reset() Online Documentation}*/
reset(): void;
/**
Prepends or appends a rotation operation to this matrix around the specified rotation center.
@param theta The rotation angle in radians
@param center The coordinate of the center of the rotation.
@param [order=MatrixOrder.PREPEND] Whether to append or prepend the rotation matrix.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23rotate(number%2CPoint%2CMatrixOrder) Online Documentation}*/
rotate(theta: number, center: Point, order?: MatrixOrder | MatrixOrderStringValues): void;
/**
Prepends or appends a rotation operation to this matrix around the origin.
@param theta The rotation angle in radians
@param [order=MatrixOrder.PREPEND] Whether to append or prepend the rotation matrix.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23rotate(number%2CMatrixOrder) Online Documentation}*/
rotate(theta: number, order?: MatrixOrder | MatrixOrderStringValues): void;
/**
Appends or prepends a scale operation to this instance.
@param x
@param y
@param [order=MatrixOrder.PREPEND]
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23scale(number%2Cnumber%2CMatrixOrder) Online Documentation}*/
scale(x: number, y: number, order?: MatrixOrder | MatrixOrderStringValues): void;
/**
Sets the values of the given <code>matrix</code> to this instance.
@param matrix The matrix to get the values from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23set(Matrix) Online Documentation}*/
set(matrix: Matrix): void;
/**
Sets all elements of this instance.
@param m0
@param m1
@param m2
@param m3
@param dx
@param dy
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23set(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
set(m0: number, m1: number, m2: number, m3: number, dx: number, dy: number): void;
/**
Converts the {@link Matrix} object to an SVG transform string.
The returned string can be used in a transform attribute.
@returns The transform string
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23toSvgTransform() Online Documentation}*/
toSvgTransform(): string;
/**
Transforms the given point in place.
@param point The point to transform and return.
@returns <code>point</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23transform(IMutablePoint) Online Documentation}*/
transform(point: IMutablePoint): IMutablePoint;
/**
Transforms the given coordinate.
@param point The coordinate to transform.
@returns The transformed coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23transform(Point) Online Documentation}*/
transform(point: Point): Point;
/**
Appends or Prepends a translation to this instance.
@param delta
@param [order=MatrixOrder.PREPEND]
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23translate(Point%2CMatrixOrder) Online Documentation}*/
translate(delta: Point, order?: MatrixOrder | MatrixOrderStringValues): void;
/**
Gets a new double[] of the elements describing the matrix.
The order is m11, m12, m21, m22, dx, dy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23elements Online Documentation}*/
readonly elements: number[];
/**
Gets an unmodifiable identity matrix.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23IDENTITY Online Documentation}*/
static readonly IDENTITY: Matrix;
/**
Gets an unmodifiable projection matrix that makes the graph look isometric.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Matrix%23ISOMETRIC Online Documentation}*/
static readonly ISOMETRIC: Matrix;
static $class: Class;
private $_Matrix: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Matrix;
}
export interface MutablePoint extends YObject, IMutablePoint, ICloneable {
}
/**
A simple default implementation of a mutable point in 2D coordinate space with double precision coordinates.
This implementation stores the values of the coordinates in double precision floating point members. As a convenience it
implements the {@link ICloneable} interface.
@see IPoint
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint Online Documentation}*/
export class MutablePoint {
/**
Creates an instance using the given coordinate pair.
@param point The coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23MutablePoint(IPoint) Online Documentation}*/
constructor(point: IPoint | IPointConvertible);
/**
Creates an instance using the given coordinate pair.
@param x The x coordinate
@param y The y coordinate
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23MutablePoint(number%2Cnumber) Online Documentation}*/
constructor(x: number, y: number);
/**
Creates the point with initially <code>0.0d</code> values for {@link MutablePoint#x} and {@link MutablePoint#y}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23MutablePoint() Online Documentation}*/
constructor();
/**
Determines whether another object is equal to this one.
@param other the other object to compare for equality
@returns <code>true</code> if <code>other</code> is considered equal to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Moves this instance by adding the provided offsets to the coordinates of this point.
@param delta The offset to add to this point's x coordinate
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23moveBy(Point) Online Documentation}*/
moveBy(delta: Point): void;
/**
Normalizes this point as if it was a vector.
After this the "length" of this point is <code>1.0d</code> if it is treated as a vector.
@see Point#normalized
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23normalize() Online Documentation}*/
normalize(): void;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23toString() Online Documentation}*/
toString(): string;
/**
Gets or sets the x coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23x Online Documentation}*/
x: number;
/**
Gets or sets the y coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutablePoint%23y Online Documentation}*/
y: number;
static $class: Class;
private $_MutablePoint: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MutablePoint;
}
export interface MutableRectangle extends YObject, IMutableRectangle, IOrientedRectangle, ICloneable {
}
/**
A simple default implementation of a mutable rectangle in 2D coordinate space with double precision values stored in an
instance of {@link IMutablePoint} and {@link IMutableSize}.
This implementation delegates the storage to implementations of {@link IMutablePoint} for the upper left corner
of the rectangle and {@link IMutableSize} for the size of the rectangle. members. As a convenience it implements
the {@link IOrientedRectangle}, and the {@link ICloneable} interfaces.
@see IRectangle
@see IMutableRectangle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle Online Documentation}*/
export class MutableRectangle {
/**
Creates a new instance using the provided values to initialize the position and size.
An instance of {@link MutablePoint} will be used to store the position and an instance of {@link MutableRectangle#size} will be used to store
the size.
@param location The coordinates of the upper left corner of the rectangle.
@param size The size of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23MutableRectangle(Point%2CSize) Online Documentation}*/
constructor(location: Point, size: Size);
/**
Creates a new instance using the provided implementation for the storage of the position and size of the rectangle.
The instances provided will be referenced by this instance. This instance will be used as a live view over the provided
instances. Changes to them will indirectly change the state of this instance and changes applied through this instance
will be written to the referenced implementations.
@param location The provider for the dynamic position of this instance.
@param size The provider for the dynamic size of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23MutableRectangle(IMutablePoint%2CIMutableSize) Online Documentation}*/
constructor(location: IMutablePoint, size: IMutableSize);
/**
Creates a new instance initialized to the values of the provided argument.
@param rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23MutableRectangle(IRectangle) Online Documentation}*/
constructor(rectangle: IRectangle);
/**
Creates a new instance using the provided values to initialize the position and size.
An instance of {@link MutablePoint} will be used to store the position and an instance of {@link MutableRectangle#size} will be used to store
the size.
@param x The x coordinate of the upper left corner of the rectangle.
@param y The y coordinate of the upper left corner of the rectangle.
@param width The width of the rectangle.
@param height The height of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23MutableRectangle(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(x: number, y: number, width: number, height: number);
/**
Creates a new empty instance located at the origin with width and height set to <code>-1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23MutableRectangle() Online Documentation}*/
constructor();
/**
Determines whether another object is equal to this one.
@param other the other object to compare for equality
@returns <code>true</code> if <code>other</code> is considered equal to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Moves this rectangle by applying the offset to the {@link MutableRectangle#location}
@param delta The offset to move the rectangle's position by.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23moveBy(Point) Online Documentation}*/
moveBy(delta: Point): void;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23toString() Online Documentation}*/
toString(): string;
/**
Gets whether this instance has negative width or height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23isEmpty Online Documentation}*/
readonly isEmpty: boolean;
/**
Gets the position of this rectangle.
This will return a live view of the position of this rectangle. However setting a position instance will only copy the
values of the point provided to the current position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23location Online Documentation}*/
location: IPoint;
/**
Gets the size of this rectangle.
This will return a live view of the size of this rectangle. However setting a size instance will only copy the values of
the size provided to the current size.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23size Online Documentation}*/
size: ISize;
/**
Gets or sets the width of this instance from the {@link MutableRectangle#size} instance.
Note that depending on the implementation used for holding the size the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23width Online Documentation}*/
width: number;
/**
Gets or sets the height of this instance from the {@link MutableRectangle#size} instance.
Note that that depending on the implementation used for holding the size the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23height Online Documentation}*/
height: number;
/**
Gets or sets the x coordinate of the upper left corner of this instance from the {@link MutableRectangle#location} instance.
Note that depending on the implementation used for holding the position the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23x Online Documentation}*/
x: number;
/**
Gets or sets the y coordinate of the upper left corner of this instance from the {@link MutableRectangle#location} instance.
Note that depending on the implementation used for holding the position the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23y Online Documentation}*/
y: number;
/**
Gets or sets the x coordinate of the lower right corner of this instance using the {@link MutableRectangle#location} and {@link MutableRectangle#size} instances.
Setting this value will update the width accordingly. Note that depending on the implementation used for holding the {@link MutableRectangle#size}
the results may vary. However the default implementation will do just that: read and write the values without modifying
them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23x2 Online Documentation}*/
x2: number;
/**
Gets or sets the y coordinate of the lower right corner of this instance using the {@link MutableRectangle#location} and {@link MutableRectangle#size} instances.
Setting this value will update the height accordingly. Note that depending on the implementation used for holding the {@link MutableRectangle#size}
the results may vary. However the default implementation will do just that: read and write the values without modifying
them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23y2 Online Documentation}*/
y2: number;
/**
Gets the x coordinate of the {@link MutableRectangle#location}.
This implementation behaves like an {@link IOrientedRectangle} that is anchored at the lower left corner of this
rectangle and whose up vector is (0, -1).
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23anchorX Online Documentation}*/
anchorX: number;
/**
Gets the y coordinate of the {@link MutableRectangle#location} plus the {@link ISize#height}.
This implementation behaves like an {@link IOrientedRectangle} that is anchored at the lower left corner of this
rectangle and whose up vector is (0, -1).
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23anchorY Online Documentation}*/
anchorY: number;
/**
Gets the x value of the up vector which is always <code>0</code>
This implementation behaves like an {@link IOrientedRectangle} that is anchored at the lower left corner of this
rectangle and whose up vector is (0, -1).
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23upX Online Documentation}*/
readonly upX: number;
/**
Gets the y value of the up vector which is always <code>-1</code>
This implementation behaves like an {@link IOrientedRectangle} that is anchored at the lower left corner of this
rectangle and whose up vector is (0, -1).
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableRectangle%23upY Online Documentation}*/
readonly upY: number;
static $class: Class;
private $_MutableRectangle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MutableRectangle;
}
export interface MutableSize extends YObject, IMutableSize, ICloneable {
}
/**
A simple default implementation of a mutable size in 2D coordinate space with double precision values.
This implementation stores the values of the width and height in double precision floating point members.
@see ISize
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableSize Online Documentation}*/
export class MutableSize {
/**
Creates an instance using the values provided by the size instance.
This will not create a dynamic instance. The values will be copied from <code>size</code> immediately and no reference is held to
that instance thereafter.
@param size A size to retrieve the initial values from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableSize%23MutableSize(ISize) Online Documentation}*/
constructor(size: ISize | ISizeConvertible);
/**
Creates an instance using the given width and height.
@param width The width.
@param height The height
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableSize%23MutableSize(number%2Cnumber) Online Documentation}*/
constructor(width: number, height: number);
/**
Creates the initially empty size, that is both {@link MutableSize#width} and {@link MutableSize#height} are <code>0.0d</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableSize%23MutableSize() Online Documentation}*/
constructor();
/**
Determines whether another object is equal to this one.
@param other the other object to compare for equality
@returns <code>true</code> if <code>other</code> is considered equal to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableSize%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableSize%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Gets or sets the width.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableSize%23width Online Documentation}*/
width: number;
/**
Gets or sets the height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MutableSize%23height Online Documentation}*/
height: number;
static $class: Class;
private $_MutableSize: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MutableSize;
}
export interface OrientedRectangle extends YObject, IMutablePoint, IMutableOrientedRectangle, ICloneable {
}
/**
A simple default implementation of a mutable oriented rectangle in 2D coordinate space with double precision values
stored in an instance of {@link IMutablePoint} and {@link IMutableSize} as well as two doubles for the
up vector components.
This implementation delegates the storage to implementations of {@link IMutablePoint} for the anchor of the
oriented rectangle. and {@link IMutableSize} for the size of the oriented rectangle. members. As a convenience
it implements the {@link IMutablePoint} and the {@link ICloneable} interfaces.
@see IOrientedRectangle
@see IMutableOrientedRectangle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle Online Documentation}*/
export class OrientedRectangle {
/**
Creates a new instance using the provided values to initialize the anchor and size.
The up vector will be initialized to
<code>(0, -1)</code>. The instances provided will be referenced by this instance. This instance will be used as a live view over the
provided instances. Changes to them will indirectly change the state of this instance and changes applied through this
instance will be written to the referenced implementations.
@param anchor The provider for the dynamic anchor of this instance.
@param size The provider for the dynamic size of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23OrientedRectangle(IMutablePoint%2CIMutableSize) Online Documentation}*/
constructor(anchor: IMutablePoint, size: IMutableSize);
/**
Creates a new instance using the provided values to initialize.
This will basically create a copy of the given rectangle
@param rectangle The rectangle to initialize the anchor and size and up vector from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23OrientedRectangle(IOrientedRectangle) Online Documentation}*/
constructor(rectangle: IOrientedRectangle);
/**
Creates a new instance using the provided rectangle to initialize the anchor and size.
The anchor will be set to the lower left corner of the provided rectangle. Thus this instance will have the exact same
bounds as the rectangle initially. The up vector will be initialized to <code>(0, -1)</code>.
@param rectangle The rectangle to initialize the anchor and size from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23OrientedRectangle(IRectangle) Online Documentation}*/
constructor(rectangle: IRectangle);
/**
Creates a new instance using the provided values to initialize the anchor and size.
It is up to the caller to ensure that the up vector is normalized.
@param anchorX The x coordinate of the anchor of the oriented rectangle.
@param anchorY The y coordinate of the anchor of the oriented rectangle.
@param width The width of the rectangle.
@param height The height of the rectangle.
@param [upX=0] The x component of the up vector.
@param [upY=-1] The y component of the up vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23OrientedRectangle(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(anchorX: number, anchorY: number, width: number, height: number, upX?: number, upY?: number);
/**
Creates a new instance using the provided values to initialize the anchor and size.
It is up to the caller to ensure that the up vector is normalized.
@param options The parameters to pass.
@param options.anchorX The x coordinate of the anchor of the oriented rectangle.
@param options.anchorY The y coordinate of the anchor of the oriented rectangle.
@param options.width The width of the rectangle.
@param options.height The height of the rectangle.
@param [options.upX=0] The x component of the up vector.
@param [options.upY=-1] The y component of the up vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23OrientedRectangle(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(options: {
anchorX: number;
anchorY: number;
width: number;
height: number;
upX?: number;
upY?: number;
});
/**
Initializes a new instance of the {@link OrientedRectangle} class located at <code>0.0d,0.0d</code> with empty width and
height (<code>-1.0d</code>).
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23OrientedRectangle() Online Documentation}*/
constructor();
/**
A casting operator that creates a new instance of {@link OrientedRectangle} using the current state of the
rectangle that is cast.
The anchor will be set to the lower left corner of the provided rectangle. Thus this instance will have the exact same
bounds as the rectangle initially. The up vector will be initialized to
<code>(0, -1)</code>. Unlike true casting this will create a new instance that will be initialized to the current state of the object
being cast, but which will not stay in sync with the state of the object.
@param rect the rectangle to get the current state from
@returns A new instance of {@link OrientedRectangle} that holds no reference to the provided rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23op_Explicit(MutableRectangle) Online Documentation}*/
static convertFrom(rect: MutableRectangle): OrientedRectangle | null;
/**
Creates an immutable {@link IOrientedRectangle} with the given values.
@param anchorX
@param anchorY
@param width
@param height
@param upX
@param upY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23createImmutable(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static createImmutable(anchorX: number, anchorY: number, width: number, height: number, upX: number, upY: number): IOrientedRectangle;
/**
Implements the equals method by comparing the given object's values.
Note that only instances of this type and the ones returned by the factory methods of this type can be equal to each
other.
@param other The other instance to compare the values against.
@returns Whether the other instance is of this type or the immutable types created by this class and whether their values are the
same as for this type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Yields a hash code for this type based on the values of {@link OrientedRectangle#upX}, {@link OrientedRectangle#upY}, {@link OrientedRectangle#width}, {@link OrientedRectangle#anchorX}, {@link OrientedRectangle#anchorY}, and {@link OrientedRectangle#height}.
@returns A hashcode for the values of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Moves this rectangle by applying the offset to the {@link OrientedRectangle#anchor}
@param delta The offset to move the rectangle's anchor by.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23moveBy(Point) Online Documentation}*/
moveBy(delta: Point): void;
/**
Applies the values provided by the given instance to the values of this instance.
@param rectangle The instance to retrieve the values from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23reshape(IOrientedRectangle) Online Documentation}*/
reshape(rectangle: IOrientedRectangle): void;
/**
Sets the components of the up vector to the new values.
It is up to the caller to assure that the values describe a vector of length 1.
@param upx The x component of the normalized up vector.
@param upy The y component of the normalized up vector.
@see OrientedRectangle#angle
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23setUpVector(number%2Cnumber) Online Documentation}*/
setUpVector(upx: number, upy: number): void;
/**
Creates a human readable from of this instance.
@returns A string describing the properties of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23toString() Online Documentation}*/
toString(): string;
/**
Gets whether this instance has negative width or height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23isEmpty Online Documentation}*/
readonly isEmpty: boolean;
/**
Gets or sets the anchor of this oriented rectangle.
This will return a live view of the anchor of this rectangle. However setting an anchor instance will only copy the
values of the point provided to the current anchor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23anchor Online Documentation}*/
anchor: IPoint;
/**
Gets or sets the size of this rectangle.
This will return a live view of the size of this rectangle. However setting a size instance will only copy the values of
the size provided to the current size.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23size Online Documentation}*/
size: ISize;
/**
Gets or sets the width of this instance from the {@link OrientedRectangle#size} instance.
Note that depending on the implementation used for holding the size the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23width Online Documentation}*/
width: number;
/**
Gets or sets the height of this instance from the {@link OrientedRectangle#size} instance.
Note that that depending on the implementation used for holding the size the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23height Online Documentation}*/
height: number;
/**
Gets or sets the x coordinate of the anchor of this instance from the {@link OrientedRectangle#anchor} instance.
Note that depending on the implementation used for holding the anchor the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23x Online Documentation}*/
x: number;
/**
Gets or sets the y coordinate of the anchor of this instance from the {@link OrientedRectangle#anchor} instance.
Note that depending on the implementation used for holding the anchor the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23y Online Documentation}*/
y: number;
/**
Gets or sets the x coordinate of the anchor of this instance from the {@link OrientedRectangle#anchor} instance.
Note that depending on the implementation used for holding the anchor the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23anchorX Online Documentation}*/
anchorX: number;
/**
Gets or sets the y coordinate of the anchor of this instance from the {@link OrientedRectangle#anchor} instance.
Note that depending on the implementation used for holding the anchor the results may vary. However the default
implementation will do just that: read and write the values without modifying them in any way.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23anchorY Online Documentation}*/
anchorY: number;
/**
Gets or sets the x component of the up vector to the new value.
It is up to the caller to assure that the values describe a vector of length 1.
@see OrientedRectangle#angle
@see OrientedRectangle#setUpVector
@see OrientedRectangle#upY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23upX Online Documentation}*/
upX: number;
/**
Gets or sets the y component of the up vector to the new value.
It is up to the caller to assure that the values describe a vector of length 1.
@see OrientedRectangle#angle
@see OrientedRectangle#setUpVector
@see OrientedRectangle#upX
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23upY Online Documentation}*/
upY: number;
/**
Gets or sets the orientation of this rectangle.
The angle is specified in radians. An angle of <code>0</code> means the up vector points up in direction <code>(0,-1)</code>. The angle value is
not stored and will be recalculated from {@link OrientedRectangle#upX} and {@link OrientedRectangle#upY} every time this is invoked, so this is a costly method. Setting the angle
changes the {@link OrientedRectangle#upX} and {@link OrientedRectangle#upY} values accordingly.
The angle is between 0 an PI if the up vector points to quadrant 2 or 3 and between -PI and 0 if the up vector points to
quadrant 1 or 4.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientedRectangle%23angle Online Documentation}*/
angle: number;
static $class: Class;
private $_OrientedRectangle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrientedRectangle;
}
export interface CollectGraphSnapLinesEventArgs extends InputModeEventArgs {
}
/**
Event arguments for the {@link GraphSnapContext#addCollectSnapLinesListener CollectSnapLines} event.
Instances of this class are also passed to
{@link ISnapLineProvider#addSnapLines}. Implementations of that interface can use the following methods to add snap
lines for specific purposes:
<ul>
<li><b>{@link CollectGraphSnapLinesEventArgs#addFixedNodeSnapLine}</b> – Adds an orthogonal snap line for nodes.
<li><b>{@link CollectGraphSnapLinesEventArgs#addFixedSegmentSnapLine}</b> – Adds an orthogonal snap line for edge segments and bends.
<li><b>{@link CollectGraphSnapLinesEventArgs#addPortSnapLine}</b> – Adds an orthogonal snap line for ports.
<li><b>{@link CollectGraphSnapLinesEventArgs#addAdditionalSnapLine}</b> – Adds an additional orthogonal snap line to which all element will snap.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectGraphSnapLinesEventArgs Online Documentation}*/
export class CollectGraphSnapLinesEventArgs {
/**
Initializes a new instance of the {@link CollectGraphSnapLinesEventArgs} class.
@param context The context where the event is being used in.
@param fixedNodeSnapLines The fixed node snap lines collection to add to.
@param fixedSegmentSnapLines The fixed segment snap lines collection to add to.
@param fixedPortSnapLines The fixed port snap lines collection to add to.
@param additionalSnapLines The additional snap lines collection to add to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectGraphSnapLinesEventArgs Online Documentation}*/
constructor(context: IInputModeContext, fixedNodeSnapLines: ICollection<OrthogonalSnapLine> | ICollectionConvertible<OrthogonalSnapLine>, fixedSegmentSnapLines: ICollection<OrthogonalSnapLine> | ICollectionConvertible<OrthogonalSnapLine>, fixedPortSnapLines: ICollection<OrthogonalSnapLine> | ICollectionConvertible<OrthogonalSnapLine>, additionalSnapLines: ICollection<OrthogonalSnapLine> | ICollectionConvertible<OrthogonalSnapLine>);
/**
Adds a snap line all elements will snap to.
@param snapLine The snap line to add to the list of lines that all elements snap to by default.
@see GraphSnapContext#additionalSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectGraphSnapLinesEventArgs%23addAdditionalSnapLine(OrthogonalSnapLine) Online Documentation}*/
addAdditionalSnapLine(snapLine: OrthogonalSnapLine): void;
/**
Adds a snap line for nodes.
@param snapLine The snap line to add to the list of lines that nodes snap to by default.
@see GraphSnapContext#fixedNodeSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectGraphSnapLinesEventArgs%23addFixedNodeSnapLine(OrthogonalSnapLine) Online Documentation}*/
addFixedNodeSnapLine(snapLine: OrthogonalSnapLine): void;
/**
Adds a snap line for edge segments and bends.
@param snapLine The snap line to add to the list of lines that segments snap to by default.
@see GraphSnapContext#fixedSegmentSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectGraphSnapLinesEventArgs%23addFixedSegmentSnapLine(OrthogonalSnapLine) Online Documentation}*/
addFixedSegmentSnapLine(snapLine: OrthogonalSnapLine): void;
/**
Adds a snap line for ports.
@param snapLine The snap line to add to the list of lines that ports snap to by default.
@see GraphSnapContext#fixedPortSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectGraphSnapLinesEventArgs%23addPortSnapLine(OrthogonalSnapLine) Online Documentation}*/
addPortSnapLine(snapLine: OrthogonalSnapLine): void;
static $class: Class;
private $_CollectGraphSnapLinesEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CollectGraphSnapLinesEventArgs;
}
/**
The enumeration of the various keys that are relevant to drag and drop operations.
This enumeration is used by {@link DragEventArgs}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropKeyStates Online Documentation}*/
export enum DragDropKeyStates {
/**
The left mouse button is down or a touch gesture is in progress.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropKeyStates%23POINTER_DOWN Online Documentation}*/
POINTER_DOWN,
/**
No button and no key is pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropKeyStates%23NONE Online Documentation}*/
NONE,
/**
The shift key is down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropKeyStates%23SHIFT_KEY Online Documentation}*/
SHIFT_KEY,
/**
The control key is down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropKeyStates%23CONTROL_KEY Online Documentation}*/
CONTROL_KEY,
/**
The alt key is down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropKeyStates%23ALT_KEY Online Documentation}*/
ALT_KEY
}
type DragDropKeyStatesStringValues = "pointer-down" | "none" | "shift-key" | "control-key" | "alt-key";
/** Automatic conversion to {@link DragDropKeyStates}. */
type DragDropKeyStatesConvertible = DragDropKeyStatesStringValues | DragDropKeyStatesStringValues[];
export interface GraphInputMode extends MultiplexingInputMode {
/**
@inheritdoc*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
@inheritdoc*/
uninstall(context: IInputModeContext): void;
} /**
Abstract base class for {@link GraphViewerInputMode} and {@link GraphEditorInputMode}.
This class is not supposed to and in fact cannot be extended from user code. It only exists to provide common
functionality to the two aforementioned input modes. If customizations are wanted or needed both {@link GraphViewerInputMode}
and {@link GraphEditorInputMode} can be extended.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode Online Documentation}*/
export abstract class GraphInputMode {
/**
Clears the selection on the current {@link GraphInputMode#graphSelection}.
This implementation may be overridden for customizations. All it does is call {@link ISelectionModel#clear}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23clearSelection() Online Documentation}*/
clearSelection(): void;
/**
Actually performs the click on the given item.
This will raise the {@link GraphInputMode#addItemLeftClickedListener ItemLeftClicked}, {@link GraphInputMode#addItemRightClickedListener ItemRightClicked}
(depending on the button), and {@link GraphInputMode#addItemClickedListener ItemClicked} events. If
either event is unhandled by all its event handlers the item will be {@link GraphInputMode#setSelected selected} and {@link GraphInputMode#setCurrentItem focused}. Before selection
{@link GraphInputMode#clearSelection the current selection will be cleared}, unless the {@link GraphInputMode#multiSelectionRecognizer} is triggered.
@param item The item that has been clicked.
@param evt The original event arguments for the click. Setting its {@link ClickEventArgs#handled} property to <code>true</code> will indicate that the click was handled. By
default this happens when the clicked item is either selected or focused.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23click(IModelItem%2CClickEventArgs) Online Documentation}*/
click(item: IModelItem, evt: ClickEventArgs): void;
/**
Clears the selection on click if {@link CanvasComponent#lastInputEvent the click} is not recognized by {@link GraphInputMode#multiSelectionRecognizer}.
This method is only called if no item has been hit and at least one item is currently selected.
This will use the {@link GraphInputMode#clearSelection} method to deselect all items.
@param context The context where the click appeared
@returns Whether the selection has been cleared by this method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23clickClearSelection(IInputModeContext) Online Documentation}*/
clickClearSelection(context: IInputModeContext): boolean;
/**
* Copies the currently selected elements to the clipboard.
* <pre>
* mode.copy()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23copy() Online Documentation}
*/
copy(): void;
/**
Factory method for the {@link GraphInputMode#clickInputMode} property.
This method will be called upon first access to the {@link GraphInputMode#clickInputMode} property.
This implementation {@link ClickInputMode#activeButtons activates} both {@link MouseButtons#LEFT} and {@link MouseButtons#RIGHT}
@returns a new instance of ClickInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createClickInputMode() Online Documentation}*/
createClickInputMode(): ClickInputMode;
/**
Factory method for the {@link GraphInputMode#contextMenuInputMode} property.
This method will be called upon first access to the {@link GraphInputMode#contextMenuInputMode} property.
@returns a new instance of ContextMenuInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createContextMenuInputMode() Online Documentation}*/
createContextMenuInputMode(): ContextMenuInputMode;
/**
Factory method for the {@link GraphInputMode#itemHoverInputMode} property.
This method will be called upon first access to the {@link GraphInputMode#itemHoverInputMode} property.
@returns a new instance of {@link GraphInputMode#itemHoverInputMode} with the {@link ItemHoverInputMode#hoverItems} property set to {@link GraphItemTypes#NONE}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createItemHoverInputMode() Online Documentation}*/
createItemHoverInputMode(): ItemHoverInputMode;
/**
Factory method that creates the {@link GraphInputMode#keyboardInputMode} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createKeyboardInputMode() Online Documentation}*/
createKeyboardInputMode(): KeyboardInputMode;
/**
Factory method for the {@link GraphInputMode#lassoSelectionInputMode} property.
This method will be called upon first access to the {@link GraphInputMode#lassoSelectionInputMode} property.
The created input mode is {@link LassoSelectionInputMode#enabled disabled} per default.
@returns a new instance of LassoSelectionInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createLassoSelectionInputMode() Online Documentation}*/
createLassoSelectionInputMode(): LassoSelectionInputMode;
/**
Factory method for the {@link GraphInputMode#marqueeSelectionInputMode} property.
This method will be called upon first access to the {@link GraphInputMode#marqueeSelectionInputMode} property.
@returns a new instance of MarqueeSelectionInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createMarqueeSelectionInputMode() Online Documentation}*/
createMarqueeSelectionInputMode(): MarqueeSelectionInputMode;
/**
Factory method for the {@link MouseHoverInputMode} property.
This method will be called upon first access to the {@link MouseHoverInputMode} property.
@returns a new instance of {@link MouseHoverInputMode}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createMouseHoverInputMode() Online Documentation}*/
createMouseHoverInputMode(): MouseHoverInputMode;
/**
Factory method for the {@link GraphInputMode#moveViewportInputMode} property.
This method will be called upon first access to the {@link GraphInputMode#moveViewportInputMode} property.
@returns a new instance of {@link GraphInputMode#moveViewportInputMode}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createMoveViewportInputMode() Online Documentation}*/
createMoveViewportInputMode(): MoveViewportInputMode;
/**
Factory method for the {@link GraphInputMode#navigationInputMode} property.
This method will be called upon first access to the {@link GraphInputMode#navigationInputMode} property.
@returns a new instance of NavigationInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createNavigationInputMode() Online Documentation}*/
createNavigationInputMode(): NavigationInputMode;
/**
Helper method that yields a suitably configured {@link SelectionEventArgs} using the {@link GraphInputMode#graphSelection} for this input mode.
@param context An input mode context that is available in the {@link SelectionEventArgs}.
Can be <code>null</code> in which case a new context for this instance is created automatically.
@returns A selection event argument that is configured for this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createSelectionEventArgs(IInputModeContext) Online Documentation}*/
createSelectionEventArgs(context: IInputModeContext | null): SelectionEventArgs<IModelItem>;
/**
Factory method for the {@link GraphInputMode#tapInputMode} property.
This method will be called upon first access to the {@link GraphInputMode#tapInputMode} property.
@returns a new instance of TapInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createTapInputMode() Online Documentation}*/
createTapInputMode(): TapInputMode;
/**
Factory method for the {@link GraphInputMode#waitInputMode} property.
This method will be called upon first access to the {@link GraphInputMode#waitInputMode} property.
@returns a new instance of {@link GraphInputMode#waitInputMode}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23createWaitInputMode() Online Documentation}*/
createWaitInputMode(): WaitInputMode;
/**
Actually performs a double-click on the given item.
This will raise the {@link GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked} and {@link GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked}
(depending on the button), and {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
events.
@param item The item that has been double-clicked.
@param evt The original event arguments for the click. Setting its {@link ClickEventArgs#handled} property to <code>true</code> will indicate that the double-click was
handled and events on other items in the same location are no longer raised.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23doubleClick(IModelItem%2CClickEventArgs) Online Documentation}*/
doubleClick(item: IModelItem, evt: ClickEventArgs): void;
/**
Actually performs a double-tap on the given item.
This will raise the {@link GraphInputMode#addItemDoubleTappedListener ItemDoubleTapped} and {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
event.
@param item The item that has been double-tapped.
@param evt The original event arguments for the tap. Setting its {@link TapEventArgs#handled} property to <code>true</code> will indicate that the double-tap was handled
and events on other items in the same location are no longer raised.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23doubleTap(IModelItem%2CTapEventArgs) Online Documentation}*/
doubleTap(item: IModelItem, evt: TapEventArgs): void;
/**
* Used as a callback to find the items hit underneath a certain point.
* This implementation uses the {@link GraphEditorInputMode#hitTester} to determine the hit items.
*
* <pre>
* /**
*  *  @param {IInputModeContext} context
*  *  @param {Point} location
*  *‍/
* function HandleItemAtLocation(context, location) {
*   /** @type {GraphEditorInputMode} *‍/
*   const graphEditorInputMode = context.lookup(GraphEditorInputMode.$class)
*
*   const modelItem = graphEditorInputMode
*     .findItems(location, [GraphItemTypes.NODE | GraphItemTypes.EDGE])
*     .firstOrDefault()
*
*   if (INode.$class.isInstance(modelItem)) {
*     // ... handle node
*   } else if (IEdge.$class.isInstance(modelItem)) {
*     // ... handle edge
*   }
* </pre>
*
* @param context The context to use for to the {@link IHitTestable#isHit} callback.
* @param location The location to test.
* @param tests An array of {@link GraphItemTypes} values that encode for which model items the hit test should be performed for
* prioritizing. Arrays that contain strings describing the name of {@link GraphItemTypes} are converted to an array
* of {@link GraphItemTypes}. For example:
* <pre>
* ['node', 'edge']
* ['NODE', 'EDGE']
* </pre>
*
* @param [filter=null] The predicate that can be used to filter the results. May be <code>null</code>.
* @returns An enumerable over the items that have been found for the location.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23findItems(IInputModeContext%2CPoint%2CArray%3CGraphItemTypes%3E%2Cfunction(IModelItem)%3Aboolean) Online Documentation}
*/
findItems(context: IInputModeContext, location: Point, tests: GraphItemTypes[], filter?: ((obj: IModelItem) => boolean) | null): IEnumerable<IModelItem>;
/**
* Used as a callback to find the items hit underneath a certain point.
* This implementation uses the {@link GraphEditorInputMode#hitTester} to determine the hit items.
*
* <pre>
* /**
*  *  @param {IInputModeContext} context
*  *  @param {Point} location
*  *‍/
* function HandleItemAtLocation(context, location) {
*   /** @type {GraphEditorInputMode} *‍/
*   const graphEditorInputMode = context.lookup(GraphEditorInputMode.$class)
*
*   const modelItem = graphEditorInputMode
*     .findItems(location, [GraphItemTypes.NODE | GraphItemTypes.EDGE])
*     .firstOrDefault()
*
*   if (INode.$class.isInstance(modelItem)) {
*     // ... handle node
*   } else if (IEdge.$class.isInstance(modelItem)) {
*     // ... handle edge
*   }
* </pre>
*
* @param location The location to test.
* @param tests An array of {@link GraphItemTypes} values that encode for which model items the hit test should be performed for
* prioritizing. Arrays that contain strings describing the name of {@link GraphItemTypes} are converted to an array
* of {@link GraphItemTypes}. For example:
* <pre>
* ['node', 'edge']
* ['NODE', 'EDGE']
* </pre>
*
* @param [filter=null] The predicate that can be used to filter the results. May be <code>null</code>.
* @returns An enumerable over the items that have been found for the location.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23findItems(Point%2CArray%3CGraphItemTypes%3E%2Cfunction(IModelItem)%3Aboolean) Online Documentation}
*/
findItems(location: Point, tests: GraphItemTypes[], filter?: ((obj: IModelItem) => boolean) | null): IEnumerable<IModelItem>;
/**
Called by {@link GraphInputMode#click} to query the <code>item</code> for an {@link IClickListener} in its {@link ILookup#lookup}
and handle it appropriately.
This method will query the {@link IClickListener} and check whether it was {@link IClickListener#getHitTestable hit} by the click, and if so, will invoke
{@link IClickListener#onClicked} and return <code>true</code>.
@param context The context for the click.
@param item The item that has been clicked.
@param location The click location.
@returns Whether the action has been invoked and handling should be stopped.
@see IClickListener
@see GraphInputMode#click
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23handleClickListener(IInputModeContext%2CIModelItem%2CPoint) Online Documentation}*/
handleClickListener(context: IInputModeContext, item: IModelItem, location: Point): boolean;
/**
Performs lasso-selection with the given path.
It delegates directly to the {@link GraphInputMode#onLassoSelect} overload internally.
@param lassoBorder The lasso selection path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23lassoSelect(GeneralPath) Online Documentation}*/
lassoSelect(lassoBorder: GeneralPath): void;
/**
Performs marquee selection with the given rectangle.
It delegates directly to the {@link GraphInputMode#onMarqueeSelect} overload internally.
@param marqueeRectangle The selection rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23marqueeSelect(Rect) Online Documentation}*/
marqueeSelect(marqueeRectangle: Rect): void;
/**
Raises the {@link GraphInputMode#addCanvasClickedListener CanvasClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link ClickEventArgs} instance that contains the information about the click.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onCanvasClicked(ClickEventArgs) Online Documentation}*/
onCanvasClicked(evt: ClickEventArgs): void;
/**
Raises the {@link GraphInputMode#addCanvasTappedListener CanvasTapped} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link TapEventArgs#handled} property to <code>true</code>.
@param evt The {@link TapEventArgs} instance that contains the information about the tap.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onCanvasTapped(TapEventArgs) Online Documentation}*/
onCanvasTapped(evt: TapEventArgs): void;
/**
Called when the {@link GraphInputMode#clickInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onClickInputModeChanged(ClickInputMode%2CClickInputMode) Online Documentation}*/
onClickInputModeChanged(oldMode: ClickInputMode | null, newMode: ClickInputMode): void;
/**
Called when the {@link ContextMenuInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onContextMenuInputModeChanged(ContextMenuInputMode%2CContextMenuInputMode) Online Documentation}*/
onContextMenuInputModeChanged(oldMode: ContextMenuInputMode | null, newMode: ContextMenuInputMode): void;
/**
Raises the {@link GraphInputMode#addItemClickedListener ItemClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link ItemClickedEventArgs} instance that contains the item that has been clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onItemClicked(ItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemClicked(evt: ItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link ItemClickedEventArgs} instance that contains the item that has been double clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onItemDoubleClicked(ItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemDoubleClicked(evt: ItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link GraphInputMode#addItemDoubleTappedListener ItemDoubleTapped} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link TapEventArgs#handled} property to <code>true</code>.
@param evt The {@link ItemTappedEventArgs} instance that contains the item that has been double tapped.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onItemDoubleTapped(ItemTappedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemDoubleTapped(evt: ItemTappedEventArgs<IModelItem>): void;
/**
Called when the {@link GraphInputMode#itemHoverInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onItemHoverInputModeChanged(ItemHoverInputMode%2CItemHoverInputMode) Online Documentation}*/
onItemHoverInputModeChanged(oldMode: ItemHoverInputMode | null, newMode: ItemHoverInputMode): void;
/**
Raises the {@link GraphInputMode#addItemLeftClickedListener ItemLeftClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link ItemClickedEventArgs} instance that contains the item that has been clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onItemLeftClicked(ItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemLeftClicked(evt: ItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link ItemClickedEventArgs} instance that contains the item that has been double clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onItemLeftDoubleClicked(ItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemLeftDoubleClicked(evt: ItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link GraphInputMode#addItemRightClickedListener ItemRightClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link ItemClickedEventArgs} instance that contains the item that has been clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onItemRightClicked(ItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemRightClicked(evt: ItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link ItemClickedEventArgs} instance that contains the item that has been double clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onItemRightDoubleClicked(ItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemRightDoubleClicked(evt: ItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link GraphInputMode#addItemTappedListener ItemTapped} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link TapEventArgs#handled} property to <code>true</code>.
@param evt The {@link ItemTappedEventArgs} instance that contains the item that has been tapped.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onItemTapped(ItemTappedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemTapped(evt: ItemTappedEventArgs<IModelItem>): void;
/**
Called when the {@link GraphInputMode#keyboardInputMode} changed.
@param oldMode The old {@link GraphInputMode#keyboardInputMode}.
@param newMode The new {@link GraphInputMode#keyboardInputMode}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onKeyboardInputModeChanged(KeyboardInputMode%2CKeyboardInputMode) Online Documentation}*/
onKeyboardInputModeChanged(oldMode: KeyboardInputMode | null, newMode: KeyboardInputMode): void;
/**
Performs lasso-selection with the given path.
This method is called internally when either the {@link GraphInputMode#lassoSelect} method is called or the user
performs a lasso-selection gesture via {@link GraphInputMode#lassoSelectionInputMode}.
@param context The input mode context.
@param lassoBorder The lasso selection path.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onLassoSelect(IInputModeContext%2CGeneralPath) Online Documentation}*/
onLassoSelect(context: IInputModeContext, lassoBorder: GeneralPath): void;
/**
Called when the {@link GraphInputMode#lassoSelectionInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onLassoSelectionInputModeChanged(LassoSelectionInputMode%2CLassoSelectionInputMode) Online Documentation}*/
onLassoSelectionInputModeChanged(oldMode: LassoSelectionInputMode | null, newMode: LassoSelectionInputMode): void;
/**
Selects the given elements in the editor inside the provided lasso-selection path.
This implementation iterates over the items and queries them for an implementation of {@link ILassoTestable} using
their {@link ILookup#lookup}. This instance will be used to determine whether the item should be selected.
This method is, in the default implementation, only called with individual item types in <code>items</code>.
@param context The input mode context.
@param lassoBorder The lasso path in world coordinates.
@param items The items to iterate over.
@param predicate The predicate that determines whether the provided item should be tested at all. May be <code>null</code>.
@see GraphInputMode#setSelected
@see GraphInputMode#onLassoSelect
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onLassoSelectItems(IInputModeContext%2CGeneralPath%2CIEnumerable%3CIModelItem%3E%2Cfunction(IModelItem)%3Aboolean) Online Documentation}*/
onLassoSelectItems(context: IInputModeContext, lassoBorder: GeneralPath, items: IEnumerable<IModelItem>, predicate: ((obj: IModelItem) => boolean) | null): void;
/**
Performs marquee selection with the given rectangle.
This method is called internally when either the {@link GraphInputMode#marqueeSelect} method is called or the user
performs a marquee-selection gesture via {@link GraphInputMode#marqueeSelectionInputMode}.
@param context The input mode context.
@param marqueeRectangle The selection rectangle.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onMarqueeSelect(IInputModeContext%2CRect) Online Documentation}*/
onMarqueeSelect(context: IInputModeContext, marqueeRectangle: Rect): void;
/**
Called when the {@link GraphInputMode#marqueeSelectionInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onMarqueeSelectionInputModeChanged(MarqueeSelectionInputMode%2CMarqueeSelectionInputMode) Online Documentation}*/
onMarqueeSelectionInputModeChanged(oldMode: MarqueeSelectionInputMode | null, newMode: MarqueeSelectionInputMode): void;
/**
Selects the given elements in the editor inside the provided marquee-selection rectangle.
This implementation iterates over the items and queries them for an implementation of {@link IMarqueeTestable}
using their {@link ILookup#lookup}. This instance will be used to determine whether the item should be selected.
This method is, in the default implementation, only called with individual item types in <code>items</code>.
@param context The input mode context.
@param marqueeRectangle The rectangle in world coordinates.
@param items The items to iterate over.
@param [predicate=null] The predicate that determines whether the provided item should be tested at all. May be <code>null</code>.
@see GraphInputMode#setSelected
@see GraphInputMode#onMarqueeSelect
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onMarqueeSelectItems(IInputModeContext%2CRect%2CIEnumerable%3CIModelItem%3E%2Cfunction(IModelItem)%3Aboolean) Online Documentation}*/
onMarqueeSelectItems(context: IInputModeContext, marqueeRectangle: Rect, items: IEnumerable<IModelItem>, predicate?: ((obj: IModelItem) => boolean) | null): void;
/**
Called when the {@link MouseHoverInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onMouseHoverInputModeChanged(MouseHoverInputMode%2CMouseHoverInputMode) Online Documentation}*/
onMouseHoverInputModeChanged(oldMode: MouseHoverInputMode | null, newMode: MouseHoverInputMode): void;
/**
Called when the {@link GraphInputMode#moveViewportInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onMoveViewportInputModeChanged(MoveViewportInputMode%2CMoveViewportInputMode) Online Documentation}*/
onMoveViewportInputModeChanged(oldMode: MoveViewportInputMode | null, newMode: MoveViewportInputMode): void;
/**
Raises the {@link GraphInputMode#addMultiSelectionFinishedListener MultiSelectionFinished} event.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onMultiSelectionFinished(SelectionEventArgs%3CIModelItem%3E) Online Documentation}*/
onMultiSelectionFinished(evt: SelectionEventArgs<IModelItem>): void;
/**
Raises the {@link GraphInputMode#addMultiSelectionStartedListener MultiSelectionStarted} event.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onMultiSelectionStarted(SelectionEventArgs%3CIModelItem%3E) Online Documentation}*/
onMultiSelectionStarted(evt: SelectionEventArgs<IModelItem>): void;
/**
Called when the {@link GraphInputMode#navigationInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onNavigationInputModeChanged(NavigationInputMode%2CNavigationInputMode) Online Documentation}*/
onNavigationInputModeChanged(oldMode: NavigationInputMode | null, newMode: NavigationInputMode): void;
/**
Raises the {@link GraphInputMode#addPopulateItemContextMenuListener PopulateItemContextMenu} event.
@param evt The {@link PopulateItemContextMenuEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onPopulateItemContextMenu(PopulateItemContextMenuEventArgs%3CIModelItem%3E) Online Documentation}*/
onPopulateItemContextMenu(evt: PopulateItemContextMenuEventArgs<IModelItem>): void;
/**
Raises the {@link GraphInputMode#addQueryItemToolTipListener QueryItemToolTip} event.
@param evt The {@link QueryItemToolTipEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onQueryItemToolTip(QueryItemToolTipEventArgs%3CIModelItem%3E) Online Documentation}*/
onQueryItemToolTip(evt: QueryItemToolTipEventArgs<IModelItem>): void;
/**
Called when the {@link GraphInputMode#tapInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onTapInputModeChanged(TapInputMode%2CTapInputMode) Online Documentation}*/
onTapInputModeChanged(oldMode: TapInputMode | null, newMode: TapInputMode): void;
/**
Called when the {@link GraphInputMode#waitInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23onWaitInputModeChanged(WaitInputMode%2CWaitInputMode) Online Documentation}*/
onWaitInputModeChanged(oldMode: WaitInputMode | null, newMode: WaitInputMode): void;
/**
Clears the current selection and selects all nodes and bends in this graph.
By default, this method will be called in response to a Ctrl+A key press recognized by {@link GraphInputMode#keyboardInputMode}.
@see GraphInputMode#clearSelection
@see GraphInputMode#setSelected
@see ICommand#SELECT_ALL
@see GraphInputMode#selectableItems
@see GraphInputMode#ignoreVoidStyles
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23selectAll() Online Documentation}*/
selectAll(): void;
/**
Called by this instance to set the current item on the {@link NavigationInputMode#setCurrentItem NavigationInputMode}
@param item The item to set as the new "current" item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23setCurrentItem(IModelItem) Online Documentation}*/
setCurrentItem(item: IModelItem | null): void;
/**
Uses the {@link GraphInputMode#graphSelection} to select the given item.
@param item The item to set the selection state on.
@param selected The new selection state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23setSelected(IModelItem%2Cboolean) Online Documentation}*/
setSelected(item: IModelItem, selected: boolean): void;
/**
Called to determine whether the given <code>item</code> can be {@link GraphInputMode#click clicked}.
This implementation uses the {@link GraphInputMode#clickableItems} value to determine whether the item should be clicked. Note that this method will not be
called for items that don't match {@link GraphInputMode#clickableItems} for performance reasons.
@param item The item to check.
@returns Whether the item should be {@link GraphInputMode#click clicked} in response to a detected mouse click.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23shouldClick(IModelItem) Online Documentation}*/
shouldClick(item: IModelItem): boolean;
/**
Called to determine whether the given <code>item</code> should be selected when {@link GraphInputMode#click clicked with the primary mouse button}.
This implementation uses the {@link GraphInputMode#clickSelectableItems} value to determine whether the item should be selected. Note that this method will not be
called for items that don't match {@link GraphInputMode#clickSelectableItems} and {@link GraphInputMode#selectableItems} for performance reasons.
@param item The item to check.
@returns Whether the item should be selected in response to a detected mouse click with the primary button.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23shouldClickSelect(IModelItem) Online Documentation}*/
shouldClickSelect(item: IModelItem): boolean;
/**
Called to determine whether the given <code>item</code> can be {@link GraphInputMode#doubleClick double clicked}.
This implementation uses the {@link GraphInputMode#shouldClick} method to determine whether the item should be
double clicked. Note that this method will not be called for items that don't match {@link GraphInputMode#clickableItems} for performance reasons.
@param item The item to check.
@returns Whether the item should be {@link GraphInputMode#doubleClick double clicked} in response to a detected mouse click.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23shouldDoubleClick(IModelItem) Online Documentation}*/
shouldDoubleClick(item: IModelItem): boolean;
/**
Determines whether the provided {@link IModelItem} should be {@link GraphInputMode#setCurrentItem set to the current item}.
This implementation uses the {@link GraphInputMode#focusableItems} property to determine whether the item can be set to the current item. It also honors the
{@link GraphInputMode#ignoreVoidStyles} property.
@param item The item to check.
@returns Whether to set the item as the current item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23shouldFocus(IModelItem) Online Documentation}*/
shouldFocus(item: IModelItem): boolean;
/**
CUsed to determine which of the built-in {@link ICommand}s to install.
This implementation unconditionally returns <code>true</code>, subclasses may override this method to adjust the behavior.
@param command The command to install.
@returns Whether to install this command.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23shouldInstallCommand(ICommand) Online Documentation}*/
shouldInstallCommand(command: ICommand): boolean;
/**
Used by {@link GraphInputMode#marqueeSelect} and {@link GraphInputMode#lassoSelect} that takes {@link GraphInputMode#marqueeSelectableItems} and {@link GraphInputMode#shouldSelectItem}
into account.
@param item The item to check.
@returns Whether to possibly marquee or lasso select the item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23shouldMarqueeSelect(IModelItem) Online Documentation}*/
shouldMarqueeSelect(item: IModelItem): boolean;
/**
Determines whether for the given item a context menu should be queried.
This implementation uses the {@link GraphInputMode#contextMenuItems} property to determine whether a context menu should be queried for the given item.
@param item The item to check.
@returns Whether to raise a {@link GraphInputMode#addPopulateItemContextMenuListener PopulateItemContextMenu} event for
the given item.
@see GraphInputMode#contextMenuInputMode
@see ContextMenuInputMode
@see GraphInputMode#addPopulateItemContextMenuListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23shouldPopulateContextMenu(IModelItem) Online Documentation}*/
shouldPopulateContextMenu(item: IModelItem): boolean;
/**
Determines whether for the given item a tool tip should be queried.
This implementation uses the {@link GraphInputMode#toolTipItems} property to determine whether tool tip should be queried for the given item.
@param item The item to check.
@returns Whether to raise a {@link GraphInputMode#addQueryItemToolTipListener QueryItemToolTip} event for the
given item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23shouldQueryToolTip(IModelItem) Online Documentation}*/
shouldQueryToolTip(item: IModelItem): boolean;
/**
Called as a broad classification whether an item can generally be selected.
This implementation uses and honors the {@link GraphInputMode#selectableItems}, {@link GraphInputMode#ignoreVoidStyles} and {@link GraphInputMode#selectablePredicate} properties. This method is by default also called by {@link GraphInputMode#shouldClickSelect}
and {@link GraphInputMode#shouldMarqueeSelect}, so turning selection off here also turns it off in other places.
@param item The item to check.
@returns <code>true</code> if the item is selectable in general, <code>false</code> if not.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23shouldSelectItem(IModelItem) Online Documentation}*/
shouldSelectItem(item: IModelItem): boolean;
/**
Actually performs the tap on the given item.
This will raise the {@link GraphInputMode#addItemClickedListener ItemClicked} event. If the event
is unhandled by all its event handlers the item will be {@link GraphInputMode#setSelected selected} and {@link GraphInputMode#setCurrentItem focused}. Before selection
{@link GraphInputMode#clearSelection the current selection will be cleared}.
@param item The item that has been tapped.
@param evt The original event arguments for the tap. Setting its {@link TapEventArgs#handled} property to <code>true</code> will indicate that the tap was handled. By
default this happens when the tapped item is either selected or focused.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23tap(IModelItem%2CTapEventArgs) Online Documentation}*/
tap(item: IModelItem, evt: TapEventArgs): void;
/**
Gets the graph instance from the {@link MultiplexingInputMode#inputModeContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23graph Online Documentation}*/
readonly graph: IGraph | null;
/**
Gets the graph selection from the {@link MultiplexingInputMode#inputModeContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23graphSelection Online Documentation}*/
readonly graphSelection: IGraphSelection | null;
/**
* Gets or sets the order of the types of items that should be used to determine what item has been clicked or tapped.
* Change this field to adjust which items will be {@link GraphInputMode#setSelected selected} if there are multiple items at a given location. The default
* order is
*
* <ul>
* <li>{@link GraphItemTypes#BEND}</li>
* <li>{@link GraphItemTypes#EDGE_LABEL}</li>
* <li>{@link GraphItemTypes#EDGE}</li>
* <li>{@link GraphItemTypes#NODE}</li>
* <li>{@link GraphItemTypes#NODE_LABEL}</li>
* <li>{@link GraphItemTypes#PORT_LABEL}</li>
* <li>{@link GraphItemTypes#PORT}</li>
* </ul>
* Arrays that contain strings describing the name of {@link GraphItemTypes} are converted to an array of
* {@link GraphItemTypes}. For example:
*
* <pre>
* ['node', 'edge']
* ['NODE', 'EDGE']
* </pre>
*
* The following example defines the order 1. bends, 2. nodes and edges (with equal priority), 3. labels.
*
* <pre>
* mode.clickHitTestOrder = [
*   GraphItemTypes.BEND,
*   GraphItemTypes.NODE | GraphItemTypes.EDGE,
*   GraphItemTypes.LABEL
* ]
* </pre>
*
* @see GraphInputMode#clickSelectableItems
* @see GraphInputMode#selectableItems
* @see GraphInputMode#findItems
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23clickHitTestOrder Online Documentation}
*/
clickHitTestOrder: GraphItemTypes[];
/**
* Gets or sets the order of the types of items that should be used to determine what item has been double-clicked or
* double-tapped.
* Change this field to adjust which items will be considered when {@link GraphInputMode#addItemDoubleClickedListener double-clicked} if there are multiple items at a given
* location. The default order is
*
* <ul>
* <li>{@link GraphItemTypes#EDGE_LABEL}</li>
* <li>{@link GraphItemTypes#NODE_LABEL}</li>
* <li>{@link GraphItemTypes#PORT_LABEL}</li>
* <li>{@link GraphItemTypes#BEND}</li>
* <li>{@link GraphItemTypes#EDGE}</li>
* <li>{@link GraphItemTypes#NODE}</li>
* <li>{@link GraphItemTypes#PORT}</li>
* </ul>
* Arrays that contain strings describing the name of {@link GraphItemTypes} are converted to an array of
* {@link GraphItemTypes}. For example:
*
* <pre>
* ['node', 'edge']
* ['NODE', 'EDGE']
* </pre>
*
* The following example defines the order 1. bends, 2. nodes and edges (with equal priority), 3. labels.
*
* <pre>
* mode.doubleClickHitTestOrder = [
*   GraphItemTypes.BEND,
*   GraphItemTypes.NODE | GraphItemTypes.EDGE,
*   GraphItemTypes.LABEL
* ]
* </pre>
*
* @see GraphInputMode#clickSelectableItems
* @see GraphInputMode#selectableItems
* @see GraphInputMode#findItems
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23doubleClickHitTestOrder Online Documentation}
*/
doubleClickHitTestOrder: GraphItemTypes[];
/**
* Gets or sets the event recognizer that determines if a click should select the item that is currently visible under the
* mouse cursor, without giving more important items higher priority.
* If the recognizer determines that the user wanted to use the detail clicking behavior, the {@link GraphEditorInputMode#hitTester} will be queried for the
* given item and the first item that is returned will be considered a hit. Otherwise all hit items are examined and are
* prioritized by item type. E.g. by default clicking on a node will select the node, even if there is a node label which
* has been clicked at the same point. If detail selection is recognized, the label will be selected.
*
* The default is {@link KeyEventRecognizers#SHIFT_IS_DOWN}.
*
* <pre>
* mode.detailSelectionRecognizer = KeyEventRecognizers.SHIFT_IS_DOWN
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23detailSelectionRecognizer Online Documentation}
*/
detailSelectionRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines if a click should cycle through all the items that are currently under
the mouse cursor.
If the recognizer determines that the user wanted to use the cyclic click selection behavior, the {@link GraphEditorInputMode#hitTester} will be queried for
the given item and previously reported items will be ignored. E.g. if multiple nodes are overlapping each other,
clicking on them will report the first item for the first click, the second one for the second click and so on. If all
elements have been reported previously, the first one is going to be reported again. If the order of the elements at the
clicked location changes, the cyclic selection is restarted anew.
The default is {@link KeyEventRecognizers#ALT_IS_DOWN}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23cyclicSelectionRecognizer Online Documentation}*/
cyclicSelectionRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines if a click is deemed a multi selection gesture.
If the {@link GraphInputMode#multiSelectionRecognizer} returns <code>true</code> for the current gesture the element which is selected with that gesture will be added to the
current selection. Otherwise, the selection will be cleared before adding the newly selected element.
The default is an or-combination of {@link KeyEventRecognizers#CTRL_IS_DOWN} and
{@link KeyEventRecognizers#META_IS_DOWN}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23multiSelectionRecognizer Online Documentation}*/
multiSelectionRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
* Gets or sets which types of items should be reported through the {@link GraphInputMode#addItemClickedListener item click events}.
* This also affects taps.
*
* The default is {@link GraphItemTypes#ALL}.
*
* <pre>
* mode.clickableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE
* </pre>
*
* @see GraphInputMode#shouldClick
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23clickableItems Online Documentation}
*/
clickableItems: GraphItemTypes;
/**
* Gets or sets the items that can be given focus via the {@link GraphInputMode#setCurrentItem} method.
* The focusable items. The default is {@link GraphItemTypes#NODE}
*
* <pre>
* mode.focusableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE
* </pre>
*
* @default GraphItemTypes.NODE
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23focusableItems Online Documentation}
*/
focusableItems: GraphItemTypes;
/**
* Gets or sets which types of items should be selectable at all.
* The default is {@link GraphItemTypes#ALL}.
*
* <br><b>Determine which items should be selectable by different selection gestures</b>
* <pre>
* // Nodes, edges, and bends are selectable
* mode.selectableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE | GraphItemTypes.BEND
* // Only nodes and bends are selectable by marquee selection
* mode.marqueeSelectableItems = GraphItemTypes.NODE | GraphItemTypes.BEND
* // Only nodes and edges are selectable by click selection
* mode.clickSelectableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE
* </pre>
*
* @see GraphInputMode#setSelected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23selectableItems Online Documentation}
*/
selectableItems: GraphItemTypes;
/**
* Gets or sets a predicate that is queried to determine whether a given {@link IModelItem} is selectable.
* This allows to further restrict which items are selectable, compared to {@link GraphInputMode#selectableItems}. If this property is set to <code>null</code> the effect is
* the same as a predicate that always returns <code>true</code>.
*
* In the following example items are only selectable if their tag is the string "I am selectable":
*
* <pre>
* mode.selectablePredicate = item => item.tag === 'I am selectable'
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23selectablePredicate Online Documentation}
*/
selectablePredicate: ((obj: IModelItem) => boolean) | null;
/**
* Gets or sets which types of items should be selectable by mouse clicks.
* The default is {@link GraphItemTypes#ALL}.
*
* <br><b>Determine which items should be selectable by different selection gestures</b>
* <pre>
* // Nodes, edges, and bends are selectable
* mode.selectableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE | GraphItemTypes.BEND
* // Only nodes and bends are selectable by marquee selection
* mode.marqueeSelectableItems = GraphItemTypes.NODE | GraphItemTypes.BEND
* // Only nodes and edges are selectable by click selection
* mode.clickSelectableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE
* </pre>
*
* @see GraphInputMode#shouldClickSelect
* @see GraphInputMode#setSelected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23clickSelectableItems Online Documentation}
*/
clickSelectableItems: GraphItemTypes;
/**
* Gets or sets which types of items should be selected during {@link GraphInputMode#marqueeSelect marquee selections} or {@link GraphInputMode#lassoSelect lasso selection}.
* The default is a combination of {@link GraphItemTypes#NODE}, {@link GraphItemTypes#EDGE}, and
* {@link GraphItemTypes#BEND}.
*
* <br><b>Determine which items should be selectable by different selection gestures</b>
* <pre>
* // Nodes, edges, and bends are selectable
* mode.selectableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE | GraphItemTypes.BEND
* // Only nodes and bends are selectable by marquee selection
* mode.marqueeSelectableItems = GraphItemTypes.NODE | GraphItemTypes.BEND
* // Only nodes and edges are selectable by click selection
* mode.clickSelectableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE
* </pre>
*
* @see GraphInputMode#shouldMarqueeSelect
* @see GraphInputMode#setSelected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23marqueeSelectableItems Online Documentation}
*/
marqueeSelectableItems: GraphItemTypes;
/**
Gets or sets a value indicating whether void styles ({@link VoidNodeStyle}, {@link VoidEdgeStyle},
{@link VoidLabelStyle}, and {@link VoidPortStyle}) should be ignored when selecting or focusing items.
Void styles are styles which render their styled elements invisible. Typically, one does not want to interact with these
elements, either. The value of this property is considered by {@link GraphInputMode#shouldSelectItem} and
{@link GraphInputMode#shouldFocus}. Note that even if this property is set to <code>false</code> the invisible elements are not
selectable by mouse clicks or marquee selection since their {@link INodeStyleRenderer#getHitTestable hit testable} always returns <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ignoreVoidStyles Online Documentation}*/
ignoreVoidStyles: boolean;
/**
* Gets or sets the {@link ContextMenuInputMode} which handles context menus on a {@link CanvasComponent}.
* If no value has been set explicitly, the factory method {@link GraphInputMode#createContextMenuInputMode} will be
* called upon first access. Upon change the {@link GraphInputMode#onContextMenuInputModeChanged} method will be
* called.
*
* By default this input mode has a priority of <code>60</code>.
*
* <br><b>Disabling the ContextMenuInputMode on its parent input mode</b>
* <pre>
* mode.contextMenuInputMode.enabled = false
* </pre>
*
* It is recommended to configure the context menu handling on the parent {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* instead of the <code>ContextMenuInputMode</code>. This involves registering for the {@link GraphInputMode#addPopulateItemContextMenuListener PopulateItemContextMenu}
* event instead of the {@link ContextMenuInputMode#addPopulateMenuListener PopulateMenu} event:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.contextMenuItems = GraphItemTypes.NODE
* mode.addPopulateItemContextMenuListener((sender, args) => {
*   // Get the node which is handled or null if the item is not a node
*   const node = args.item
*   // Create the context menu items
*   if (node !== null) {
*     // Create a menu item to delete the node
*
*     // Show the menu
*     args.showMenu = true
*     // Mark the event as handled
*     args.handled = true
*   }
* })
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23contextMenuInputMode Online Documentation}
*/
contextMenuInputMode: ContextMenuInputMode;
/**
* Gets or sets the types of items a context menu should be queried for.
* The items for which a context menu should be {@link GraphInputMode#addPopulateItemContextMenuListener queried}. The default value is
* {@link GraphItemTypes#NODE}|{@link GraphItemTypes#EDGE}.
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.contextMenuItems = GraphItemTypes.NODE
* mode.addPopulateItemContextMenuListener((sender, args) => {
*   // Get the node which is handled or null if the item is not a node
*   const node = args.item
*   // Create the context menu items
*   if (node !== null) {
*     // Create a menu item to delete the node
*
*     // Show the menu
*     args.showMenu = true
*     // Mark the event as handled
*     args.handled = true
*   }
* })
* </pre>
*
* @default GraphItemTypes.NODE | GraphItemTypes.EDGE
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23contextMenuItems Online Documentation}
*/
contextMenuItems: GraphItemTypes;
/**
* Adds the given listener for the <code>PopulateItemContextMenu</code> event that occurs when the {@link ContextMenuInputMode context menu} over an item is about to
* be opened to determine the contents of the Menu.
* This method will only be called for items that match the {@link GraphInputMode#contextMenuItems} type.
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.contextMenuItems = GraphItemTypes.NODE
* mode.addPopulateItemContextMenuListener((sender, args) => {
*   // Get the node which is handled or null if the item is not a node
*   const node = args.item
*   // Create the context menu items
*   if (node !== null) {
*     // Create a menu item to delete the node
*
*     // Show the menu
*     args.showMenu = true
*     // Mark the event as handled
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#removePopulateItemContextMenuListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23PopulateItemContextMenu Online Documentation}
*/
addPopulateItemContextMenuListener(listener: (sender: any, evt: PopulateItemContextMenuEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>PopulateItemContextMenu</code> event that occurs when the {@link ContextMenuInputMode context menu} over an item is about
* to be opened to determine the contents of the Menu.
* This method will only be called for items that match the {@link GraphInputMode#contextMenuItems} type.
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.contextMenuItems = GraphItemTypes.NODE
* mode.addPopulateItemContextMenuListener((sender, args) => {
*   // Get the node which is handled or null if the item is not a node
*   const node = args.item
*   // Create the context menu items
*   if (node !== null) {
*     // Create a menu item to delete the node
*
*     // Show the menu
*     args.showMenu = true
*     // Mark the event as handled
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#addPopulateItemContextMenuListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23PopulateItemContextMenu Online Documentation}
*/
removePopulateItemContextMenuListener(listener: (sender: any, evt: PopulateItemContextMenuEventArgs<IModelItem>) => void): void;
/**
* Gets or sets the {@link MouseHoverInputMode} which is responsible for displaying Tooltips on a
* {@link CanvasComponent}.
* If no value has been set explicitly, the factory method {@link GraphInputMode#createMouseHoverInputMode} will be
* called upon first access. Upon change the {@link GraphInputMode#onMouseHoverInputModeChanged} method will be
* called.
*
* By default this input mode has a priority of <code>100</code>.
*
* <br><b>Disabling the MouseHoverInputMode on its parent input mode</b>
* <pre>
* mode.mouseHoverInputMode.enabled = false
* </pre>
*
* It is recommended to configure the tooltip handling on the parent {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* instead of the <code>MouseHoverInputMode</code>. This involves registering for the {@link GraphInputMode#addQueryItemToolTipListener QueryItemToolTip}
* event instead of the {@link MouseHoverInputMode#addQueryToolTipListener QueryToolTip} event:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.toolTipItems = GraphItemTypes.NODE
* // register a listener
* mode.addQueryItemToolTipListener((src, args) => {
*   if (args.handled) {
*     // A tooltip has already been assigned -> nothing to do.
*     return
*   }
*   // We can safely cast here because we set ToolTipItems to only Node.
*   /** @type {INode} *‍/
*   const hitNode = args.item
*   if (hitNode.labels.size > 0) {
*     // Show the text of the first label as tooltip.
*     args.toolTip = hitNode.labels.get(0).text
*
*     // Indicate that the tooltip content has been set.
*     args.handled = true
*   }
* })
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23mouseHoverInputMode Online Documentation}
*/
mouseHoverInputMode: MouseHoverInputMode;
/**
* Gets or sets the types of the items that should be queried a tool tip for.
* The items for which a tool tip text should be {@link GraphInputMode#addQueryItemToolTipListener queried}. The default value is
* {@link GraphItemTypes#NODE}|{@link GraphItemTypes#EDGE}|{@link GraphItemTypes#LABEL}|{@link GraphItemTypes#PORT}|.
*
* <br><b>Configuring the MouseHoverInputMode to display tooltips for nodes</b>
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.toolTipItems = GraphItemTypes.NODE
* // register a listener
* mode.addQueryItemToolTipListener((src, args) => {
*   if (args.handled) {
*     // A tooltip has already been assigned -> nothing to do.
*     return
*   }
*   // We can safely cast here because we set ToolTipItems to only Node.
*   /** @type {INode} *‍/
*   const hitNode = args.item
*   if (hitNode.labels.size > 0) {
*     // Show the text of the first label as tooltip.
*     args.toolTip = hitNode.labels.get(0).text
*
*     // Indicate that the tooltip content has been set.
*     args.handled = true
*   }
* })
* </pre>
*
* @default GraphItemTypes.NODE | GraphItemTypes.EDGE | GraphItemTypes.LABEL | GraphItemTypes.PORT
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23toolTipItems Online Documentation}
*/
toolTipItems: GraphItemTypes;
/**
* Adds the given listener for the <code>QueryItemToolTip</code> event that occurs when the mouse is {@link MouseHoverInputMode hovering} over an item to determine
* the tool tip to display.
* This method will only be called for items that match the {@link GraphInputMode#toolTipItems} type.
*
* <br><b>Configuring the MouseHoverInputMode to display tooltips for nodes</b>
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.toolTipItems = GraphItemTypes.NODE
* // register a listener
* mode.addQueryItemToolTipListener((src, args) => {
*   if (args.handled) {
*     // A tooltip has already been assigned -> nothing to do.
*     return
*   }
*   // We can safely cast here because we set ToolTipItems to only Node.
*   /** @type {INode} *‍/
*   const hitNode = args.item
*   if (hitNode.labels.size > 0) {
*     // Show the text of the first label as tooltip.
*     args.toolTip = hitNode.labels.get(0).text
*
*     // Indicate that the tooltip content has been set.
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#removeQueryItemToolTipListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23QueryItemToolTip Online Documentation}
*/
addQueryItemToolTipListener(listener: (sender: any, evt: QueryItemToolTipEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>QueryItemToolTip</code> event that occurs when the mouse is {@link MouseHoverInputMode hovering} over an item to
* determine the tool tip to display.
* This method will only be called for items that match the {@link GraphInputMode#toolTipItems} type.
*
* <br><b>Configuring the MouseHoverInputMode to display tooltips for nodes</b>
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.toolTipItems = GraphItemTypes.NODE
* // register a listener
* mode.addQueryItemToolTipListener((src, args) => {
*   if (args.handled) {
*     // A tooltip has already been assigned -> nothing to do.
*     return
*   }
*   // We can safely cast here because we set ToolTipItems to only Node.
*   /** @type {INode} *‍/
*   const hitNode = args.item
*   if (hitNode.labels.size > 0) {
*     // Show the text of the first label as tooltip.
*     args.toolTip = hitNode.labels.get(0).text
*
*     // Indicate that the tooltip content has been set.
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#addQueryItemToolTipListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23QueryItemToolTip Online Documentation}
*/
removeQueryItemToolTipListener(listener: (sender: any, evt: QueryItemToolTipEventArgs<IModelItem>) => void): void;
/**
* Gets or sets the {@link ClickInputMode} that is used by this instance to determine clicks.
* Whenever the mode detects a {@link ClickInputMode#addClickedListener click}, this mode will use the {@link IHitTester} from the {@link MultiplexingInputMode#inputModeContext} to determine the items being hit. If they match the
* {@link GraphInputMode#clickableItems} type, the {@link GraphInputMode#addItemClickedListener ItemClicked} event will be triggered. If
* they {@link GraphInputMode#shouldFocus}, the item will be {@link GraphInputMode#setCurrentItem set as the current item}.
*
* If no value has been set explicitly, the factory method {@link GraphInputMode#createClickInputMode} will be called
* upon first access. Upon change the {@link GraphInputMode#onClickInputModeChanged} method will be called.
*
* By default this input mode has a priority of <code>10</code>.
*
* <br><b>Disabling the ClickInputMode on its parent input mode</b>
* <pre>
* mode.clickInputMode.enabled = false
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see GraphInputMode#onItemClicked
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23clickInputMode Online Documentation}
*/
clickInputMode: ClickInputMode;
/**
* Gets or sets the {@link TapInputMode} which detects single and multiple taps on a {@link CanvasComponent}.
* If no value has been set explicitly, the factory method {@link GraphInputMode#createTapInputMode} will be called
* upon first access. Upon change the {@link GraphInputMode#onTapInputModeChanged} method will be called.
*
* By default this input mode has a priority of <code>20</code>.
*
* <br><b>Disabling the TapInputMode on its parent input mode</b>
* <pre>
* mode.tapInputMode.enabled = false
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23tapInputMode Online Documentation}
*/
tapInputMode: TapInputMode;
/**
* Adds the given listener for the <code>ItemClicked</code> event that occurs when an item has been clicked or tapped.
* If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
*
* This event will also be raised for {@link GraphInputMode#tap taps}.
*
* Button-specific and input type specific events
* ({@link GraphInputMode#addItemLeftClickedListener ItemLeftClicked}, {@link GraphInputMode#addItemRightClickedListener ItemRightClicked}
* and {@link GraphInputMode#addItemTappedListener ItemTapped}) will be raised before this event. If
* one of those is marked {@link ClickEventArgs#handled} this event will not be raised anymore.
*
* <pre>
* mode.addItemClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemLeftClickedListener
* @see GraphInputMode#addItemRightClickedListener
* @see GraphInputMode#addItemTappedListener
* @see GraphInputMode#removeItemClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemClicked Online Documentation}
*/
addItemClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>ItemClicked</code> event that occurs when an item has been clicked or tapped.
* If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
*
* This event will also be raised for {@link GraphInputMode#tap taps}.
*
* Button-specific and input type specific events
* ({@link GraphInputMode#addItemLeftClickedListener ItemLeftClicked}, {@link GraphInputMode#addItemRightClickedListener ItemRightClicked}
* and {@link GraphInputMode#addItemTappedListener ItemTapped}) will be raised before this event. If
* one of those is marked {@link ClickEventArgs#handled} this event will not be raised anymore.
*
* <pre>
* mode.addItemClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemLeftClickedListener
* @see GraphInputMode#addItemRightClickedListener
* @see GraphInputMode#addItemTappedListener
* @see GraphInputMode#addItemClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemClicked Online Documentation}
*/
removeItemClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Adds the given listener for the <code>ItemLeftClicked</code> event that occurs when an item has been left clicked.
* This event will be raised before the {@link GraphInputMode#addItemClickedListener ItemClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
* case.
*
* <pre>
* mode.addItemLeftClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemClickedListener
* @see GraphInputMode#addItemRightClickedListener
* @see GraphInputMode#addItemTappedListener
* @see GraphInputMode#addItemLeftDoubleClickedListener
* @see GraphInputMode#removeItemLeftClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemLeftClicked Online Documentation}
*/
addItemLeftClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>ItemLeftClicked</code> event that occurs when an item has been left clicked.
* This event will be raised before the {@link GraphInputMode#addItemClickedListener ItemClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
* case.
*
* <pre>
* mode.addItemLeftClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemClickedListener
* @see GraphInputMode#addItemRightClickedListener
* @see GraphInputMode#addItemTappedListener
* @see GraphInputMode#addItemLeftDoubleClickedListener
* @see GraphInputMode#addItemLeftClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemLeftClicked Online Documentation}
*/
removeItemLeftClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Adds the given listener for the <code>ItemRightClicked</code> event that occurs when an item has been right clicked.
* This event will be raised before the {@link GraphInputMode#addItemClickedListener ItemClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
* case.
*
* <pre>
* mode.addItemRightClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemClickedListener
* @see GraphInputMode#addItemLeftClickedListener
* @see GraphInputMode#addItemTappedListener
* @see GraphInputMode#addItemRightDoubleClickedListener
* @see GraphInputMode#removeItemRightClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemRightClicked Online Documentation}
*/
addItemRightClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>ItemRightClicked</code> event that occurs when an item has been right clicked.
* This event will be raised before the {@link GraphInputMode#addItemClickedListener ItemClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
* case.
*
* <pre>
* mode.addItemRightClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemClickedListener
* @see GraphInputMode#addItemLeftClickedListener
* @see GraphInputMode#addItemTappedListener
* @see GraphInputMode#addItemRightDoubleClickedListener
* @see GraphInputMode#addItemRightClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemRightClicked Online Documentation}
*/
removeItemRightClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Adds the given listener for the <code>ItemTapped</code> event that occurs when an item has been tapped.
* This event will be raised before the {@link GraphInputMode#addItemClickedListener ItemClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
* case.
*
* <pre>
* mode.addItemTappedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemClickedListener
* @see GraphInputMode#addItemLeftClickedListener
* @see GraphInputMode#addItemRightClickedListener
* @see GraphInputMode#addItemDoubleTappedListener
* @see GraphInputMode#removeItemTappedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemTapped Online Documentation}
*/
addItemTappedListener(listener: (sender: any, evt: ItemTappedEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>ItemTapped</code> event that occurs when an item has been tapped.
* This event will be raised before the {@link GraphInputMode#addItemClickedListener ItemClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
* case.
*
* <pre>
* mode.addItemTappedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemClickedListener
* @see GraphInputMode#addItemLeftClickedListener
* @see GraphInputMode#addItemRightClickedListener
* @see GraphInputMode#addItemDoubleTappedListener
* @see GraphInputMode#addItemTappedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemTapped Online Documentation}
*/
removeItemTappedListener(listener: (sender: any, evt: ItemTappedEventArgs<IModelItem>) => void): void;
/**
* Adds the given listener for the <code>ItemDoubleClicked</code> event that occurs when an item has been double clicked.
* If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
*
* Depending on the value of the {@link ClickInputMode#doubleClickPolicy} property of {@link GraphInputMode#clickInputMode} this event may be preceded by no, one, or two {@link GraphInputMode#addItemClickedListener ItemClicked}
* events (as well as the corresponding button-specific event).
*
* Button-specific and input-type specific events
* ({@link GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked}, {@link GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked}
* and {@link GraphInputMode#addItemDoubleTappedListener ItemDoubleTapped}) will be raised before this
* event. If one of those is marked {@link ClickEventArgs#handled} this event will not be raised anymore.
*
* <pre>
* mode.addItemDoubleClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#addItemClickedListener
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemLeftDoubleClickedListener
* @see GraphInputMode#addItemRightDoubleClickedListener
* @see GraphInputMode#addItemDoubleTappedListener
* @see GraphInputMode#removeItemDoubleClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemDoubleClicked Online Documentation}
*/
addItemDoubleClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>ItemDoubleClicked</code> event that occurs when an item has been double clicked.
* If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
*
* Depending on the value of the {@link ClickInputMode#doubleClickPolicy} property of {@link GraphInputMode#clickInputMode} this event may be preceded by no, one, or two {@link GraphInputMode#addItemClickedListener ItemClicked}
* events (as well as the corresponding button-specific event).
*
* Button-specific and input-type specific events
* ({@link GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked}, {@link GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked}
* and {@link GraphInputMode#addItemDoubleTappedListener ItemDoubleTapped}) will be raised before this
* event. If one of those is marked {@link ClickEventArgs#handled} this event will not be raised anymore.
*
* <pre>
* mode.addItemDoubleClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#addItemClickedListener
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemLeftDoubleClickedListener
* @see GraphInputMode#addItemRightDoubleClickedListener
* @see GraphInputMode#addItemDoubleTappedListener
* @see GraphInputMode#addItemDoubleClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemDoubleClicked Online Documentation}
*/
removeItemDoubleClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Adds the given listener for the <code>ItemLeftDoubleClicked</code> event that occurs when an item has been left double clicked.
* This event will be raised before the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
* in that case.
*
* <pre>
* mode.addItemLeftDoubleClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemRightDoubleClickedListener
* @see GraphInputMode#addItemDoubleTappedListener
* @see GraphInputMode#removeItemLeftDoubleClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemLeftDoubleClicked Online Documentation}
*/
addItemLeftDoubleClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>ItemLeftDoubleClicked</code> event that occurs when an item has been left double clicked.
* This event will be raised before the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
* in that case.
*
* <pre>
* mode.addItemLeftDoubleClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemRightDoubleClickedListener
* @see GraphInputMode#addItemDoubleTappedListener
* @see GraphInputMode#addItemLeftDoubleClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemLeftDoubleClicked Online Documentation}
*/
removeItemLeftDoubleClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Adds the given listener for the <code>ItemRightDoubleClicked</code> event that occurs when an item has been right double clicked.
* This event will be raised before the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
* in that case.
*
* <pre>
* mode.addItemRightDoubleClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemLeftDoubleClickedListener
* @see GraphInputMode#addItemDoubleTappedListener
* @see GraphInputMode#removeItemRightDoubleClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemRightDoubleClicked Online Documentation}
*/
addItemRightDoubleClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>ItemRightDoubleClicked</code> event that occurs when an item has been right double clicked.
* This event will be raised before the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
* in that case.
*
* <pre>
* mode.addItemRightDoubleClickedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemLeftDoubleClickedListener
* @see GraphInputMode#addItemDoubleTappedListener
* @see GraphInputMode#addItemRightDoubleClickedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemRightDoubleClicked Online Documentation}
*/
removeItemRightDoubleClickedListener(listener: (sender: any, evt: ItemClickedEventArgs<IModelItem>) => void): void;
/**
* Adds the given listener for the <code>ItemDoubleTapped</code> event that occurs when an item has been double tapped.
* This event will be raised before the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
* in that case.
*
* <pre>
* mode.addItemDoubleTappedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemLeftDoubleClickedListener
* @see GraphInputMode#addItemRightDoubleClickedListener
* @see GraphInputMode#removeItemDoubleTappedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemDoubleTapped Online Documentation}
*/
addItemDoubleTappedListener(listener: (sender: any, evt: ItemTappedEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>ItemDoubleTapped</code> event that occurs when an item has been double tapped.
* This event will be raised before the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
* event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
* that the {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
* in that case.
*
* <pre>
* mode.addItemDoubleTappedListener((sender, args) => {
*   const node = args.item
*   if (node !== null) {
*     doSomethingWithNode(node)
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphInputMode#clickableItems
* @see GraphInputMode#addItemDoubleClickedListener
* @see GraphInputMode#addItemLeftDoubleClickedListener
* @see GraphInputMode#addItemRightDoubleClickedListener
* @see GraphInputMode#addItemDoubleTappedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ItemDoubleTapped Online Documentation}
*/
removeItemDoubleTappedListener(listener: (sender: any, evt: ItemTappedEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>CanvasClicked</code> event that occurs when the empty canvas area has been clicked or tapped.
If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
This event will also be raised for {@link GraphInputMode#tap taps}. In this case the {@link GraphInputMode#addCanvasTappedListener CanvasTapped}
event will be raised before this event. If it is marked {@link TapEventArgs#handled} this event will not be raised anymore.
@param listener The listener to add.
@see GraphInputMode#addItemClickedListener
@see GraphInputMode#addCanvasTappedListener
@see GraphInputMode#removeCanvasClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23CanvasClicked Online Documentation}*/
addCanvasClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Removes the given listener for the <code>CanvasClicked</code> event that occurs when the empty canvas area has been clicked or
tapped.
If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
This event will also be raised for {@link GraphInputMode#tap taps}. In this case the {@link GraphInputMode#addCanvasTappedListener CanvasTapped}
event will be raised before this event. If it is marked {@link TapEventArgs#handled} this event will not be raised anymore.
@param listener The listener to remove.
@see GraphInputMode#addItemClickedListener
@see GraphInputMode#addCanvasTappedListener
@see GraphInputMode#addCanvasClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23CanvasClicked Online Documentation}*/
removeCanvasClickedListener(listener: (sender: any, evt: ClickEventArgs) => void): void;
/**
Adds the given listener for the <code>CanvasTapped</code> event that occurs when the empty canvas area has been tapped.
This event will be raised before the {@link GraphInputMode#addCanvasClickedListener CanvasClicked}
event. If one of the event handlers sets the {@link TapEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link GraphInputMode#addCanvasClickedListener CanvasClicked} event will not be raised in
that case.
@param listener The listener to add.
@see GraphInputMode#addItemTappedListener
@see GraphInputMode#addCanvasClickedListener
@see GraphInputMode#removeCanvasTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23CanvasTapped Online Documentation}*/
addCanvasTappedListener(listener: (sender: any, evt: TapEventArgs) => void): void;
/**
Removes the given listener for the <code>CanvasTapped</code> event that occurs when the empty canvas area has been tapped.
This event will be raised before the {@link GraphInputMode#addCanvasClickedListener CanvasClicked}
event. If one of the event handlers sets the {@link TapEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link GraphInputMode#addCanvasClickedListener CanvasClicked} event will not be raised in
that case.
@param listener The listener to remove.
@see GraphInputMode#addItemTappedListener
@see GraphInputMode#addCanvasClickedListener
@see GraphInputMode#addCanvasTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23CanvasTapped Online Documentation}*/
removeCanvasTappedListener(listener: (sender: any, evt: TapEventArgs) => void): void;
/**
* Gets or sets the {@link MarqueeSelectionInputMode} which handles the selection of multiple elements by drawing a
* rectangle around them.
* If no value has been set explicitly, the factory method {@link GraphInputMode#createMarqueeSelectionInputMode} will
* be called upon first access. Upon change the {@link GraphInputMode#onMarqueeSelectionInputModeChanged} method will
* be called.
*
* The {@link GraphInputMode#marqueeSelectionInputMode} respects the {@link GraphInputMode#selectableItems} and the {@link GraphInputMode#marqueeSelectableItems} as well as method {@link GraphInputMode#shouldMarqueeSelect}.
*
* By default this input mode has a priority of <code>50</code>.
*
* <br><b>Disabling the MarqueeSelectionInputMode on its parent input mode</b>
* <pre>
* mode.marqueeSelectionInputMode.enabled = false
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23marqueeSelectionInputMode Online Documentation}
*/
marqueeSelectionInputMode: MarqueeSelectionInputMode;
/**
Gets or sets the {@link LassoSelectionInputMode} which handles the selection of multiple elements by drawing a
polygon around them.
If no value has been set explicitly, the factory method {@link GraphInputMode#createLassoSelectionInputMode} will
be called upon first access. Upon change the {@link GraphInputMode#onLassoSelectionInputModeChanged} method will be
called.
The {@link GraphInputMode#lassoSelectionInputMode} respects the {@link GraphInputMode#selectableItems} and the {@link GraphInputMode#marqueeSelectableItems} as well as method {@link GraphInputMode#shouldMarqueeSelect}.
By default this input mode has a priority of <code>49</code> and is disabled.
@throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
{@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23lassoSelectionInputMode Online Documentation}*/
lassoSelectionInputMode: LassoSelectionInputMode;
/**
* Gets or sets the {@link NavigationInputMode} which is responsible for navigating and traversing the elements in the
* {@link IGraph}.
* If no value has been set explicitly, the factory method {@link GraphInputMode#createNavigationInputMode} will be
* called upon first access. Upon change the {@link GraphInputMode#onNavigationInputModeChanged} method will be
* called.
*
* By default this input mode has a priority of <code>55</code>.
*
* <br><b>Disabling the NavigationInputMode on its parent input mode</b>
* <pre>
* mode.navigationInputMode.enabled = false
* </pre>
*
* The <code>NavigationInputMode</code> supports e.g. opening and closing group nodes. These actions can be configured in details:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable expanding and collapsing group nodes
* mode.navigationInputMode.allowCollapseGroup = true
* mode.navigationInputMode.allowExpandGroup = true
* // don't fit the content after expand/collapse
* mode.navigationInputMode.fitContentAfterGroupActions = false
* // expand/collapse group nodes in a way that their
* // top right corner stays at its current coordinates
* mode.navigationInputMode.autoGroupNodeAlignmentPolicy = NodeAlignmentPolicy.TOP_RIGHT
* </pre>
*
* It also reports when groups are opened or collapsed interactively by raising the following events:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.navigationInputMode.addGroupExpandingListener((sender, args) => {
*   const node = args.item
*   // do something with the node before it will be expanded
* })
* mode.navigationInputMode.addGroupExpandedListener((sender, args) => {
*   const node = args.item
*   // do something with the node after it has been expanded
* })
* mode.navigationInputMode.addGroupCollapsingListener((sender, args) => {
*   /* same for collapse operations *‍/
* })
* mode.navigationInputMode.addGroupCollapsedListener((sender, args) => {
*   /* same for collapse operations *‍/
* })
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23navigationInputMode Online Documentation}
*/
navigationInputMode: NavigationInputMode;
/**
Adds the given listener for the <code>MultiSelectionStarted</code> event that occurs when a single or multi select operation has been
started.
The event is <b>not</b> triggered for the individual selection operations that constitute a multi selection operation.
@param listener The listener to add.
@see GraphInputMode#removeMultiSelectionStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23MultiSelectionStarted Online Documentation}*/
addMultiSelectionStartedListener(listener: (sender: any, evt: SelectionEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>MultiSelectionStarted</code> event that occurs when a single or multi select operation has
been started.
The event is <b>not</b> triggered for the individual selection operations that constitute a multi selection operation.
@param listener The listener to remove.
@see GraphInputMode#addMultiSelectionStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23MultiSelectionStarted Online Documentation}*/
removeMultiSelectionStartedListener(listener: (sender: any, evt: SelectionEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>MultiSelectionFinished</code> event that occurs when a single or multi select operation has
been finished.
The event is <b>not</b> triggered for the individual selection operations that constitute a multi selection operation.
@param listener The listener to add.
@see GraphInputMode#removeMultiSelectionFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23MultiSelectionFinished Online Documentation}*/
addMultiSelectionFinishedListener(listener: (sender: any, evt: SelectionEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>MultiSelectionFinished</code> event that occurs when a single or multi select operation has
been finished.
The event is <b>not</b> triggered for the individual selection operations that constitute a multi selection operation.
@param listener The listener to remove.
@see GraphInputMode#addMultiSelectionFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23MultiSelectionFinished Online Documentation}*/
removeMultiSelectionFinishedListener(listener: (sender: any, evt: SelectionEventArgs<IModelItem>) => void): void;
/**
* Gets or sets the {@link WaitInputMode} that is provided by this instance for those who need to make use of it.
* If no value has been set explicitly, the factory method {@link GraphInputMode#createWaitInputMode} will be called
* upon first access. Upon change the {@link GraphInputMode#onWaitInputModeChanged} method will be called.
*
* By default this input mode has a priority of <code>-1</code>.
*
* <br><b>Disabling the WaitInputMode on its parent input mode</b>
* <pre>
* mode.waitInputMode.enabled = false
* </pre>
*
* <br><b>Blocking user interaction during an asynchronous process</b>
* <pre>
* graphEditorInputMode.waiting = true
* longRunningAsyncProcess().then(() => {
*   graphEditorInputMode.waiting = false
* })
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see WaitInputMode#waiting
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23waitInputMode Online Documentation}
*/
waitInputMode: WaitInputMode;
/**
* Gets or sets the {@link ItemHoverInputMode} that is provided by this instance to detect when the cursor hovers over
* a graph item.
* Note that initially the {@link ItemHoverInputMode#hoverItems} property is set to
* {@link GraphItemTypes#NONE}, which effectively disables the functionality of the mode initially. In order to get
* the mode to fire events, the property should be set to a corresponding value.
*
* If the backing field has not yet been initialized upon first access, the factory method {@link GraphInputMode#createItemHoverInputMode}
* will be called. Upon change the {@link GraphInputMode#onItemHoverInputModeChanged} method will be called.
*
* By default this input mode has a priority of <code>55</code>.
*
* <br><b>Disabling the ItemHoverInputMode on its parent input mode</b>
* <pre>
* mode.itemHoverInputMode.enabled = false
* </pre>
*
* The {@link GraphInputMode#itemHoverInputMode} raises the {@link ItemHoverInputMode#addHoveredItemChangedListener HoveredItemChanged} event each
* time cursor hovers over a different item or the canvas in which case the hovered item is
* <code>null</code>. One can also configure the item types which are reported:
*
* <br><b>Configuring the ItemHoverInputMode to handle nodes and edges</b>
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable the ItemHoverInputMode and let it handle edges and nodes
* mode.itemHoverInputMode.enabled = true
* mode.itemHoverInputMode.hoverItems = GraphItemTypes.EDGE | GraphItemTypes.NODE
* // ignore items of other types which might be in front of them
* mode.itemHoverInputMode.discardInvalidItems = false
* // handle changes on the hovered items
* mode.itemHoverInputMode.addHoveredItemChangedListener((sender, args) => {
*   const oldItem = args.oldItem
*   // e.g. remove the highlight from oldItem here
*   const newItem = args.item
*   // e.g. add a highlight to newItem here
* })
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23itemHoverInputMode Online Documentation}
*/
itemHoverInputMode: ItemHoverInputMode;
/**
* Gets or sets the {@link MoveViewportInputMode} associated with this instance.
* If no value has been set explicitly, the factory method {@link GraphInputMode#createMoveViewportInputMode} will be
* called upon first access. Upon change the {@link GraphInputMode#onMoveViewportInputModeChanged} method will be
* called.
*
* By default this input mode has a priority of <code>39</code>.
*
* <br><b>Disabling the MoveViewportInputMode on its parent input mode</b>
* <pre>
* mode.moveViewportInputMode.enabled = false
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23moveViewportInputMode Online Documentation}
*/
moveViewportInputMode: MoveViewportInputMode;
/**
* Gets or sets the {@link KeyboardInputMode} associated with this instance.
* By default this input mode has a priority of <code>0</code>.
*
* <br><b>Disabling the KeyboardInputMode on its parent input mode</b>
* <pre>
* mode.keyboardInputMode.enabled = false
* </pre>
*
* <br><b>Adding a handler for a keyboard event</b>
* <pre>
* const keyboardInputMode = graphEditorInputMode.keyboardInputMode
* const myCommand = ICommand.createCommand('MyCommand')
*
* // The command creates a new node in the center of the viewport
* keyboardInputMode.addCommandBinding(
*   myCommand,
*   (command, parameter, target) =>
*     graphEditorInputMode.createNode(graphEditorInputMode.graphComponent.viewport.center),
*   (command, parameter, target) => true
* )
*
* // Execute the command if the Insert key is pressed
* keyboardInputMode.addKeyBinding(Key.INSERT, ModifierKeys.NONE, myCommand)
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23keyboardInputMode Online Documentation}
*/
keyboardInputMode: KeyboardInputMode;
/**
Gets a collection of commands that this input mode will handle.
This collection can be modified by removing commands to prevent the input mode from handling them. Previously removed
commands can also be re-added again to enable them again. However, the input mode will never handle commands that
weren't in the initial collection. Adding those will do nothing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23availableCommands Online Documentation}*/
readonly availableCommands: ICollection<ICommand>;
/**
* Gets or sets a property that determines whether clipboard operations with the usual shortcuts are enabled on the
* canvas.
* Default value is <code>true</code>
*
* <br><b>Disabling clipboard operations</b>
* <pre>
* mode.allowClipboardOperations = false
* </pre>
*
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23allowClipboardOperations Online Documentation}
*/
allowClipboardOperations: boolean;
/**
Adds the given listener for the <code>ElementsCopied</code> event that occurs when the {@link GraphInputMode#copy} operation
finished successfully.
@param listener The listener to add.
@see GraphInputMode#removeElementsCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ElementsCopied Online Documentation}*/
addElementsCopiedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ElementsCopied</code> event that occurs when the {@link GraphInputMode#copy} operation
finished successfully.
@param listener The listener to remove.
@see GraphInputMode#addElementsCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphInputMode%23ElementsCopied Online Documentation}*/
removeElementsCopiedListener(listener: (sender: any, evt: EventArgs) => void): void;
static $class: Class;
private $_GraphInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphInputMode;
}
export interface GraphSnapContext extends SnapContext {
}
/**
* Concrete {@link SnapContext} implementation which handles snapping movement for {@link IGraph} items.
* Snapping is enabled for {@link IGraph} items by setting an instance of this on the {@link GraphEditorInputMode}
* as {@link GraphEditorInputMode#snapContext}.
*
* This class provides a set of properties that determine the way snapping operations are being performed in the context of
* {@link IGraph}s.
*
* {@link IPositionHandler} implementations and the like can {@link ILookup#lookup retrieve} instances of this class from the
* {@link IInputModeContext}.
*
* <br><b>Enabling snapping by setting the snap context</b>
* <pre>
* mode.snapContext = new GraphSnapContext()
* </pre>
*
* <br><b>Customizing the snap context</b>
* <pre>
* mode.snapContext = new GraphSnapContext({
*   nodeToNodeDistance: 30,
*   collectNodePairCenterSnapLines: true,
*   collectNodePairSegmentSnapLines: false
* })
* </pre>
*
* <br><b>Retrieving a GraphSnapContext instance from the input mode context</b>
* <pre>
* /** @type {GraphSnapContext} *‍/
* const snapContext = inputModeContext.lookup(GraphSnapContext.$class)
* if (snapContext && snapContext.isInitializing) {
*   // ...
* }
* </pre>
*
* @see GraphEditorInputMode
* @see GraphEditorInputMode#snapContext
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext Online Documentation}
*/
export class GraphSnapContext {
/**
Creates a new instance with default settings.
@param [options] The parameters to pass.
@param [options.collectEdgeSnapLines] Whether this instance should collect {@link GraphSnapContext#fixedSegmentSnapLines fixed snap lines} for edge segments. This option sets the {@link GraphSnapContext#collectEdgeSnapLines} property on the created object.
@param [options.collectNodeSizes] Whether this instance should collect the sizes of fixed nodes. This option sets the {@link GraphSnapContext#collectNodeSizes} property on the created object.
@param [options.collectNodeSnapLines] Whether this instance should collect {@link GraphSnapContext#fixedNodeSnapLines fixed snap lines} for nodes. This option sets the {@link GraphSnapContext#collectNodeSnapLines} property on the created object.
@param [options.collectPortSnapLines] Whether this instance should collect {@link GraphSnapContext#fixedPortSnapLines fixed snap lines} for ports. This option sets the {@link GraphSnapContext#collectPortSnapLines} property on the created object.
@param [options.snapBendAdjacentSegments] Whether this context will automatically snap the movements of bends so that the two adjacent segments become horizontally or vertically oriented. This option sets the {@link GraphSnapContext#snapBendAdjacentSegments} property on the created object.
@param [options.snapPortAdjacentSegments] Whether this context will automatically snap the movements of nodes so that the first and last segments of edges become orthogonal. This option sets the {@link GraphSnapContext#snapPortAdjacentSegments} property on the created object.
@param [options.snapSegmentsToSnapLines] Whether this context will automatically snap orthogonal edge segments to snap lines. This option sets the {@link GraphSnapContext#snapSegmentsToSnapLines} property on the created object.
@param [options.snapBendsToSnapLines] Whether this context will automatically snap the bends to snap lines. This option sets the {@link GraphSnapContext#snapBendsToSnapLines} property on the created object.
@param [options.snapNodesToSnapLines] Whether {@link INodeSnapResultProvider} implementations should snap nodes to snap lines. This option sets the {@link GraphSnapContext#snapNodesToSnapLines} property on the created object.
@param [options.nodeToNodeDistance] The preferred distance between node borders. This option sets the {@link GraphSnapContext#nodeToNodeDistance} property on the created object.
@param [options.nodeToEdgeDistance] The preferred distance between node borders and edge segments. This option sets the {@link GraphSnapContext#nodeToEdgeDistance} property on the created object.
@param [options.edgeToEdgeDistance] The preferred distance between edges. This option sets the {@link GraphSnapContext#edgeToEdgeDistance} property on the created object.
@param [options.snapOrthogonalMovement] Whether this context will automatically snap the movement in such a way that the overall move gesture is constrained to the horizontal or vertical axis. This option sets the {@link GraphSnapContext#snapOrthogonalMovement} property on the created object.
@param [options.collectNodePairCenterSnapLines] Whether this instance will try to align the centers of adjacent nodes so that their centers are equally far away from each other. This option sets the {@link GraphSnapContext#collectNodePairCenterSnapLines} property on the created object.
@param [options.collectNodePairSnapLines] A value indicating whether this instance will collect pairs of adjacent nodes and add snapping logic so that moved nodes will snap so that the distances to the adjacent nodes are equal. This option sets the {@link GraphSnapContext#collectNodePairSnapLines} property on the created object.
@param [options.collectNodePairSegmentSnapLines] A value indicating whether this instance will collect pairs of adjacent nodes and add snapping logic so that moved edge segments will snap so that the distances to the adjacent nodes are equal. This option sets the {@link GraphSnapContext#collectNodePairSegmentSnapLines} property on the created object.
@param [options.nodeGridConstraintProvider] The grid constraint provider for the {@link INode}s. This option sets the {@link GraphSnapContext#nodeGridConstraintProvider} property on the created object.
@param [options.bendGridConstraintProvider] The grid constraint provider for the {@link IBend}s. This option sets the {@link GraphSnapContext#bendGridConstraintProvider} property on the created object.
@param [options.portGridConstraintProvider] The grid constraint provider for the {@link IPort}s. This option sets the {@link GraphSnapContext#portGridConstraintProvider} property on the created object.
@param [options.snapLineExtension] The amount by which snap lines that are induced by existing edge segments and node borders are being extended. This option sets the {@link GraphSnapContext#snapLineExtension} property on the created object.
@param [options.cropSnapLines] A value indicating whether to crop the snap lines at obstacles. This option sets the {@link GraphSnapContext#cropSnapLines} property on the created object.
@param [options.visualizeSnapResults] A value indicating whether to visualize the snap results. This option sets the {@link SnapContext#visualizeSnapResults} property on the created object.
@param [options.snapDistance] The global snap distance, which is the maximum distance between the current mouse coordinates and the coordinates to which the mouse will snap. This option sets the {@link SnapContext#snapDistance} property on the created object.
@param [options.gridSnapType] The type of the grid snapping that should be performed. This option sets the {@link SnapContext#gridSnapType} property on the created object.
@param [options.gridSnapDistance] The GridSnapDistance. This option sets the {@link SnapContext#gridSnapDistance} property on the created object.
@param [options.enabled] A value indicating whether this {@link SnapContext} is enabled. This option sets the {@link SnapContext#enabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext Online Documentation}*/
constructor(options?: {
collectEdgeSnapLines?: boolean;
collectNodeSizes?: boolean;
collectNodeSnapLines?: boolean;
collectPortSnapLines?: boolean;
snapBendAdjacentSegments?: boolean;
snapPortAdjacentSegments?: boolean;
snapSegmentsToSnapLines?: boolean;
snapBendsToSnapLines?: boolean;
snapNodesToSnapLines?: boolean;
nodeToNodeDistance?: number;
nodeToEdgeDistance?: number;
edgeToEdgeDistance?: number;
snapOrthogonalMovement?: boolean;
collectNodePairCenterSnapLines?: boolean;
collectNodePairSnapLines?: boolean;
collectNodePairSegmentSnapLines?: boolean;
nodeGridConstraintProvider?: IGridConstraintProvider<INode> | null;
bendGridConstraintProvider?: IGridConstraintProvider<IBend> | null;
portGridConstraintProvider?: IGridConstraintProvider<IPort> | null;
snapLineExtension?: number;
cropSnapLines?: boolean;
visualizeSnapResults?: boolean;
snapDistance?: number;
gridSnapType?: GridSnapTypes | GridSnapTypesConvertible;
gridSnapDistance?: number;
enabled?: boolean;
});
/**
Adds {@link MovementInfo} representing an {@link IBend} that will be considered for the next drag gesture.
The default {@link IHandle} of an {@link IBend} calls this method in its {@link IDragHandler#initializeDrag}
method so snap results are calculated when this bend is repositioned. Custom {@link IHandle} implementations can
add infos created by {@link GraphSnapContext#getMovementInfo}.
@param info The {@link MovementInfo} representing the bend to be moved. Can be created with
{@link GraphSnapContext#getMovementInfo}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23addBendToBeMoved(MovementInfo) Online Documentation}*/
addBendToBeMoved(info: MovementInfo): void;
/**
Adds {@link MovementInfo} representing the end of an {@link IEdge} that will be considered for the next drag
gesture.
The {@link IHandle} returned by the default {@link IEdgePortHandleProvider} of an {@link IEdge} calls
this method in its {@link IDragHandler#initializeDrag} method so snap results are calculated when reparenting this
edge end. Custom {@link IHandle} implementations can call {@link GraphSnapContext#getMovementInfos} and add
the first or last {@link MovementInfo} of the returned list.
@param info The {@link MovementInfo} representing the edge end to be reparented. Can be created with
{@link GraphSnapContext#getMovementInfos}.
@param sourceEnd <code>true</code> if the <code>info</code> represents the source end of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23addEdgeEndToBeMoved(MovementInfo%2Cboolean) Online Documentation}*/
addEdgeEndToBeMoved(info: MovementInfo, sourceEnd: boolean): void;
/**
Adds an item which will be reshaped.
@param item The item to be reshaped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23addItemToBeReshaped(IModelItem) Online Documentation}*/
addItemToBeReshaped(item: IModelItem): void;
/**
Adds {@link MovementInfo} representing an {@link INode} that will be considered for the next drag gesture.
The default {@link IPositionHandler} of an {@link INode} calls this method in its {@link IDragHandler#initializeDrag}
method so snap results are calculated when this node is repositioned. Custom {@link IPositionHandler}
implementations can add infos created by {@link GraphSnapContext#getMovementInfo}.
@param info The {@link MovementInfo} representing the node to be moved. Can be created with
{@link GraphSnapContext#getMovementInfo}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23addNodeToBeMoved(MovementInfo) Online Documentation}*/
addNodeToBeMoved(info: MovementInfo): void;
/**
Adds an orthogonally moved segment to this context for which this instance will automatically collect
{@link SnapResult}s and provide a default visualization.
This method can be called during initialization to collect orthogonal segments that should be tested against {@link GraphSnapContext#fixedSegmentSnapLines} and {@link GraphSnapContext#additionalSnapLines}.
@param start The starting point of the manipulated segment.
@param end The end point of the manipulated segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23addOrthogonalSegment(MovementInfo%2CMovementInfo) Online Documentation}*/
addOrthogonalSegment(start: MovementInfo, end: MovementInfo): void;
/**
Adds {@link MovementInfo} representing an {@link IPort} that will be considered for the next drag gesture.
The default {@link IHandle} of an {@link IPort} calls this method in its {@link IDragHandler#initializeDrag}
method so snap results are calculated for this port. Custom {@link IHandle} implementations can add infos created
by {@link GraphSnapContext#getMovementInfo}.
@param info The {@link MovementInfo} representing the port to be moved. Can be created with
{@link GraphSnapContext#getMovementInfo}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23addPortToBeMoved(MovementInfo) Online Documentation}*/
addPortToBeMoved(info: MovementInfo): void;
/**
Adds an entry for the same-height-snapping.
During {@link SnapContext#isInitializing initialization} this method can be used to register entries that can later be retrieved using the {@link GraphSnapContext#findSameHeightEntries}
method. The rectangle will be used to obtain the height value and will also be used for the visualization of the snap
result.
@param rectangle The rectangle to obtain the height value from. This rectangle will be used for the snap result visualization.
@see GraphSnapContext#addSameWidthEntry
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23addSameHeightEntry(Rect) Online Documentation}*/
addSameHeightEntry(rectangle: Rect): void;
/**
Adds an entry for the same-width-snapping.
During {@link SnapContext#isInitializing initialization} this method can be used to register entries that can later be retrieved using the {@link GraphSnapContext#findSameWidthEntries}
method. The rectangle will be used to obtain the width value and will also be used for the visualization of the snap
result.
@param rectangle The rectangle to obtain the width value from. This rectangle will be used for the snap result visualization.
@see GraphSnapContext#addSameHeightEntry
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23addSameWidthEntry(Rect) Online Documentation}*/
addSameWidthEntry(rectangle: Rect): void;
/**
Disposes of all previously temporarily collected state and clears the {@link SnapContext#snapResults} collection.
Subclasses should make sure to call the base implementation as their last operation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23cleanUp() Online Documentation}*/
cleanUp(): void;
/**
Collects the {@link OrthogonalSnapLine}s for the nodes which are not moving.
Delegates to the {@link ISnapLineProvider} implementation in the lookup of the {@link INode}.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectFixedNodeSnapLines(CollectGraphSnapLinesEventArgs) Online Documentation}*/
collectFixedNodeSnapLines(evt: CollectGraphSnapLinesEventArgs): void;
/**
Collects the {@link OrthogonalSnapLine}s for the ports which are not moving.
Delegates to the {@link ISnapLineProvider} implementation in the lookup of the {@link IPort}.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectFixedPortSnapLines(CollectGraphSnapLinesEventArgs) Online Documentation}*/
collectFixedPortSnapLines(evt: CollectGraphSnapLinesEventArgs): void;
/**
Collects snap lines for edge segments.
Delegates to the {@link ISnapLineProvider} implementation in the lookup of the {@link IEdge}. This
implementation should then make us of {@link GraphSnapContext#getMovementInfos} to determine what segments are
fixed.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectFixedSegmentSnapLines(CollectGraphSnapLinesEventArgs) Online Documentation}*/
collectFixedSegmentSnapLines(evt: CollectGraphSnapLinesEventArgs): void;
/**
Collects all node widths and heights in the collections {@link GraphSnapContext#findSameWidthEntries} and
{@link GraphSnapContext#findSameHeightEntries}.
This method will only be called if {@link GraphSnapContext#collectNodeSizes} is enabled and delegates to {@link GraphSnapContext#addSameHeightEntry} and
{@link GraphSnapContext#addSameWidthEntry}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectSameSizeEntries() Online Documentation}*/
collectSameSizeEntries(): void;
/**
Finds pairs of nodes that are visually adjacent to create {@link GraphSnapContext#collectNodePairCenterSnapLines center} and {@link GraphSnapContext#collectNodePairSnapLines same distance} snap lines.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectVisibleNodePairs() Online Documentation}*/
collectVisibleNodePairs(): void;
/**
Collects a list of snap lines.
This method has been overridden to collect the snap lines for all fixed graph elements and {@link GraphSnapContext#cropSnapLines crop} them after collection.
@see GraphSnapContext#onCollectSnapLines
@see GraphSnapContext#cropSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23dragInitialized() Online Documentation}*/
dragInitialized(): void;
/**
Finds the entries ({@link Rect}s) that have been registered using the {@link GraphSnapContext#addSameHeightEntry}
previously that best match the given constraints.
@param size The size to find the closest set of identical sizes.
@param maxSize The maximum allowed size.
@param minSize The minimum allowed size.
@returns The set of rectangles that have been added previously using {@link GraphSnapContext#addSameHeightEntry}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23findSameHeightEntries(number%2Cnumber%2Cnumber) Online Documentation}*/
findSameHeightEntries(size: number, maxSize: number, minSize: number): IListEnumerable<Rect>;
/**
Finds the entries ({@link Rect}s) that have been registered using the {@link GraphSnapContext#addSameWidthEntry}
previously that best match the given constraints.
@param size The size to find the closest set of identical sizes.
@param maxSize The maximum allowed size.
@param minSize The minimum allowed size.
@returns The set of rectangles that have been added previously using {@link GraphSnapContext#addSameWidthEntry}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23findSameWidthEntries(number%2Cnumber%2Cnumber) Online Documentation}*/
findSameWidthEntries(size: number, maxSize: number, minSize: number): IListEnumerable<Rect>;
/**
Gets a subset of {@link GraphSnapContext#additionalSnapLines} that matches the given <code>orientation</code>, <code>snapType</code> and intersects with the provided <code>area</code>.
@param orientation The orientation of the lines to return.
@param snapType The snap type to be included.
@param area The area to test for intersection.
@returns An enumerable of the snap lines that match the provided criteria.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23getAdditionalSnapLines(SnapLineOrientation%2CSnapLineSnapTypes%2CRect) Online Documentation}*/
getAdditionalSnapLines(orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, area: Rect): IEnumerable<OrthogonalSnapLine>;
/**
Gets a subset of {@link GraphSnapContext#fixedNodeSnapLines} that matches the given <code>orientation</code>, <code>snapType</code> and intersects with the provided <code>area</code>.
@param orientation The orientation of the lines to return.
@param snapType The snap type to be included.
@param area The area to test for intersection.
@returns An enumerable of the snap lines that match the provided criteria.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23getFixedNodeSnapLines(SnapLineOrientation%2CSnapLineSnapTypes%2CRect) Online Documentation}*/
getFixedNodeSnapLines(orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, area: Rect): IEnumerable<OrthogonalSnapLine>;
/**
Gets a subset of {@link GraphSnapContext#fixedPortSnapLines} that matches the given <code>orientation</code>, <code>snapType</code> and intersects with the provided <code>area</code>.
@param orientation The orientation of the lines to return.
@param snapType The snap type to be included.
@param area The area to test for intersection.
@returns An enumerable of the snap lines that match the provided criteria.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23getFixedPortSnapLines(SnapLineOrientation%2CSnapLineSnapTypes%2CRect) Online Documentation}*/
getFixedPortSnapLines(orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, area: Rect): IEnumerable<OrthogonalSnapLine>;
/**
Gets a subset of {@link GraphSnapContext#fixedSegmentSnapLines} that matches the given <code>orientation</code>, <code>snapType</code> and intersects with the provided <code>area</code>.
@param orientation The orientation of the lines to return.
@param snapType The snap type to be included.
@param area The area to test for intersection.
@returns An enumerable of the snap lines that match the provided criteria.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23getFixedSegmentSnapLines(SnapLineOrientation%2CSnapLineSnapTypes%2CRect) Online Documentation}*/
getFixedSegmentSnapLines(orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, area: Rect): IEnumerable<OrthogonalSnapLine>;
/**
Gets the movement info for the given port.
This is either the info passed to {@link GraphSnapContext#addNodeToBeMoved} or another, dynamically created
instance, if no specific one had been registered.
@param port The port to obtain the info for.
@returns An instance of {@link MovementInfo} that describes the movement of the port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23getMovementInfo(IPort) Online Documentation}*/
getMovementInfo(port: IPort): MovementInfo;
/**
Gets the movement info for the given node.
This is either the info passed to {@link GraphSnapContext#addNodeToBeMoved} or another, dynamically created
instance, if no specific one had been registered.
@param node The node to obtain the info for.
@returns An instance of {@link MovementInfo} that describes the movement of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23getMovementInfo(INode) Online Documentation}*/
getMovementInfo(node: INode): MovementInfo;
/**
Gets the movement info for the given bend.
This is either the info passed to {@link GraphSnapContext#addBendToBeMoved} or a dynamically created instance if no
specific one had been registered.
@param bend The bend to obtain the info for.
@returns An instance of {@link MovementInfo} that describes the movement of the bend.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23getMovementInfo(IBend) Online Documentation}*/
getMovementInfo(bend: IBend): MovementInfo;
/**
Gets the {@link MovementInfo movement information} for the provided edge.
The result describes the shape of the edge and the reshaping behavior of the geometry. The first element in the
enumerable is the source end of the edge, followed by bend like elements and the target side. Note that it cannot in
general be assumed {@link MovementInfo#movedItem} of the first and last element is either a {@link IPort} or {@link IEdge}, also items in
between may not necessarily be bound to {@link IBend}s and therefore their {@link MovementInfo#movedItem} can be of a different type than
{@link IBend}.
@param edge The edge to obtain the movement information of.
@returns An {@link IListEnumerable} of {@link MovementInfo}s items that describe the geometry of the edge being modified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23getMovementInfos(IEdge) Online Documentation}*/
getMovementInfos(edge: IEdge): IListEnumerable<MovementInfo>;
/**
Called by {@link IInputMode} implementations when an interactive drag is started.
Initializes the list of nodes to be moved. Called by the wrapping position handler on initialization of a drag action.
@param context The context in which the interactive drag is started.
@param originalLocation The original location of the mouse.
@throws {Exception} <b>(InvalidOperationError)</b> If this context is already {@link SnapContext#isInitialized initialized}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23initializeDrag(IInputModeContext%2CPoint) Online Documentation}*/
initializeDrag(context: IInputModeContext, originalLocation: Point): void;
/**
Raises the {@link GraphSnapContext#addCollectSnapLinesListener CollectSnapLines} event.
@param evt The {@link CollectGraphSnapLinesEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23onCollectSnapLines(CollectGraphSnapLinesEventArgs) Online Documentation}*/
onCollectSnapLines(evt: CollectGraphSnapLinesEventArgs): void;
/**
Gets the nodes that stay fixed for the current edit.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23fixedNodes Online Documentation}*/
readonly fixedNodes: ICollection<INode>;
/**
Adds the given listener for the <code>CollectSnapLines</code> event that occurs every time this instance has been {@link SnapContext#dragInitialized initialized} to
collect {@link OrthogonalSnapLine}s.
Event handlers should add snap lines to the caller using the methods provided by
{@link CollectGraphSnapLinesEventArgs}.
@param listener The listener to add.
@see GraphSnapContext#removeCollectSnapLinesListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23CollectSnapLines Online Documentation}*/
addCollectSnapLinesListener(listener: (sender: any, evt: CollectGraphSnapLinesEventArgs) => void): void;
/**
Removes the given listener for the <code>CollectSnapLines</code> event that occurs every time this instance has been {@link SnapContext#dragInitialized initialized} to
collect {@link OrthogonalSnapLine}s.
Event handlers should add snap lines to the caller using the methods provided by
{@link CollectGraphSnapLinesEventArgs}.
@param listener The listener to remove.
@see GraphSnapContext#addCollectSnapLinesListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23CollectSnapLines Online Documentation}*/
removeCollectSnapLinesListener(listener: (sender: any, evt: CollectGraphSnapLinesEventArgs) => void): void;
/**
Gets or sets whether this instance should collect {@link GraphSnapContext#fixedSegmentSnapLines fixed snap lines} for edge segments.
The default is <code>true</code>.
@see ISnapLineProvider
@see GraphSnapContext#collectFixedSegmentSnapLines
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectEdgeSnapLines Online Documentation}*/
collectEdgeSnapLines: boolean;
/**
Gets or sets whether this instance should collect the sizes of fixed nodes.
The default is <code>true</code>.
@see GraphSnapContext#collectSameSizeEntries
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectNodeSizes Online Documentation}*/
collectNodeSizes: boolean;
/**
Gets or sets whether this instance should collect {@link GraphSnapContext#fixedNodeSnapLines fixed snap lines} for nodes.
The default is <code>true</code>.
@see ISnapLineProvider
@see GraphSnapContext#collectFixedNodeSnapLines
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectNodeSnapLines Online Documentation}*/
collectNodeSnapLines: boolean;
/**
Gets or sets whether this instance should collect {@link GraphSnapContext#fixedPortSnapLines fixed snap lines} for ports.
The default is <code>true</code>.
@see ISnapLineProvider
@see GraphSnapContext#collectFixedPortSnapLines
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectPortSnapLines Online Documentation}*/
collectPortSnapLines: boolean;
/**
Gets or sets whether this context will automatically snap the movements of bends so that the two adjacent segments
become horizontally or vertically oriented.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23snapBendAdjacentSegments Online Documentation}*/
snapBendAdjacentSegments: boolean;
/**
Gets or sets whether this context will automatically snap the movements of nodes so that the first and last segments of
edges become orthogonal.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23snapPortAdjacentSegments Online Documentation}*/
snapPortAdjacentSegments: boolean;
/**
Gets or sets whether this context will automatically snap orthogonal edge segments to snap lines.
The default is true.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23snapSegmentsToSnapLines Online Documentation}*/
snapSegmentsToSnapLines: boolean;
/**
Gets or sets whether this context will automatically snap the bends to snap lines.
The default is false.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23snapBendsToSnapLines Online Documentation}*/
snapBendsToSnapLines: boolean;
/**
Gets or sets whether {@link INodeSnapResultProvider} implementations should snap nodes to snap lines.
The default is true.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23snapNodesToSnapLines Online Documentation}*/
snapNodesToSnapLines: boolean;
/**
Gets or sets the preferred distance between node borders.
This will influence the creation of the node snap lines. For each fixed node there will be snap lines on the borders of
the node itself and around the node at the specified distance.
The default is 0.0d which disables distant snap lines around the nodes.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23nodeToNodeDistance Online Documentation}*/
nodeToNodeDistance: number;
/**
Gets or sets the preferred distance between node borders and edge segments.
This will influence the creation of the node snap lines. For each fixed node there will be snap lines on the borders of
the node itself and around the node at the specified distance to which orthogonal edge segments can snap.
The default is -1.0d which disables distant snap lines around the nodes for edge segments.
@default -1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23nodeToEdgeDistance Online Documentation}*/
nodeToEdgeDistance: number;
/**
Gets or sets the preferred distance between edges.
This will influence the creation of the segment snap lines. For each orthogonally oriented fixed segment there will be
snap lines on the segment itself and to both sides of the segment at the specified distance.
The default is 0.0d which disables parallel snap lines.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23edgeToEdgeDistance Online Documentation}*/
edgeToEdgeDistance: number;
/**
Gets or sets whether this context will automatically snap the movement in such a way that the overall move gesture is
constrained to the horizontal or vertical axis.
The default is false.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23snapOrthogonalMovement Online Documentation}*/
snapOrthogonalMovement: boolean;
/**
Gets or sets whether this instance will try to align the centers of adjacent nodes so that their centers are equally far
away from each other.
Note that this will only make a difference for differently sized nodes.
By default this feature is turned off, so that the instance will try to distribute nodes evenly, so that there is the
same spacing between their borders, not between their centers.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectNodePairCenterSnapLines Online Documentation}*/
collectNodePairCenterSnapLines: boolean;
/**
Gets or sets a value indicating whether this instance will collect pairs of adjacent nodes and add snapping logic so
that moved nodes will snap so that the distances to the adjacent nodes are equal.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectNodePairSnapLines Online Documentation}*/
collectNodePairSnapLines: boolean;
/**
Gets or sets a value indicating whether this instance will collect pairs of adjacent nodes and add snapping logic so
that moved edge segments will snap so that the distances to the adjacent nodes are equal.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23collectNodePairSegmentSnapLines Online Documentation}*/
collectNodePairSegmentSnapLines: boolean;
/**
* Gets or sets the grid constraint provider for the {@link INode}s.
* Enabling grid snapping
*
* <pre>
* const snapContext = new GraphSnapContext()
*
* const gridInfo = new GridInfo(50)
* snapContext.nodeGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.bendGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.portGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.gridSnapType = GridSnapTypes.GRID_POINTS
*
* graphEditorInputMode.snapContext = snapContext
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23nodeGridConstraintProvider Online Documentation}
*/
nodeGridConstraintProvider: IGridConstraintProvider<INode> | null;
/**
* Gets or sets the grid constraint provider for the {@link IBend}s.
* Enabling grid snapping
*
* <pre>
* const snapContext = new GraphSnapContext()
*
* const gridInfo = new GridInfo(50)
* snapContext.nodeGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.bendGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.portGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.gridSnapType = GridSnapTypes.GRID_POINTS
*
* graphEditorInputMode.snapContext = snapContext
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23bendGridConstraintProvider Online Documentation}
*/
bendGridConstraintProvider: IGridConstraintProvider<IBend> | null;
/**
* Gets or sets the grid constraint provider for the {@link IPort}s.
* Enabling grid snapping
*
* <pre>
* const snapContext = new GraphSnapContext()
*
* const gridInfo = new GridInfo(50)
* snapContext.nodeGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.bendGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.portGridConstraintProvider = new GridConstraintProvider(gridInfo)
* snapContext.gridSnapType = GridSnapTypes.GRID_POINTS
*
* graphEditorInputMode.snapContext = snapContext
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23portGridConstraintProvider Online Documentation}
*/
portGridConstraintProvider: IGridConstraintProvider<IPort> | null;
/**
Gets or sets the amount by which snap lines that are induced by existing edge segments and node borders are being
extended.
The default is <code>40.0d</code>, this value will be used to extend the ends of the snap lines.
@default 40.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23snapLineExtension Online Documentation}*/
snapLineExtension: number;
/**
Gets or sets a value indicating whether to crop the snap lines at obstacles.
The default is <code>true</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23cropSnapLines Online Documentation}*/
cropSnapLines: boolean;
/**
Gets the collection of {@link OrthogonalSnapLine}s for fixed nodes.
This collection is only available if {@link SnapContext#isInitialized} is <code>true</code>.
@see GraphSnapContext#getFixedNodeSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23fixedNodeSnapLines Online Documentation}*/
readonly fixedNodeSnapLines: IEnumerable<OrthogonalSnapLine>;
/**
Gets the collection of {@link OrthogonalSnapLine}s for fixed segments.
This collection is only available if {@link SnapContext#isInitialized} is <code>true</code>.
@see GraphSnapContext#getFixedSegmentSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23fixedSegmentSnapLines Online Documentation}*/
readonly fixedSegmentSnapLines: IEnumerable<OrthogonalSnapLine>;
/**
Gets the collection of {@link OrthogonalSnapLine}s to which ports should be snapping.
This collection is only available if {@link SnapContext#isInitialized} is <code>true</code>.
@see GraphSnapContext#getFixedPortSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23fixedPortSnapLines Online Documentation}*/
readonly fixedPortSnapLines: IEnumerable<OrthogonalSnapLine>;
/**
Gets the collection of additional {@link OrthogonalSnapLine}s.
This collection is only available if {@link SnapContext#isInitialized} is <code>true</code>.
@see GraphSnapContext#getAdditionalSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSnapContext%23additionalSnapLines Online Documentation}*/
readonly additionalSnapLines: IEnumerable<OrthogonalSnapLine>;
static $class: Class;
private $_GraphSnapContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphSnapContext;
}
/**
Describes the type of the movement of a point during an edit in the context of {@link GraphSnapContext}.
@see GraphSnapContext
@see MovementInfo
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes Online Documentation}*/
export enum MoveTypes {
/**
The element does not move at all during the edit.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes%23FIXED Online Documentation}*/
FIXED,
/**
The element moves linearly with the mouse in horizontal direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes%23LINEAR_MOVE_X Online Documentation}*/
LINEAR_MOVE_X,
/**
The element moves linearly with the mouse in vertical direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes%23LINEAR_MOVE_Y Online Documentation}*/
LINEAR_MOVE_Y,
/**
The element moves linearly with the mouse in both directions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes%23LINEAR_MOVE Online Documentation}*/
LINEAR_MOVE,
/**
The element moves in horizontal direction but not linearly/predictably with the mouse.
A typical example is a port whose position is adjusted during a resize of its owning node.
Elements which do not move linearly with the mouse cannot snap to a defined position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes%23ARBITRARY_MOVE_X Online Documentation}*/
ARBITRARY_MOVE_X,
/**
The element moves in vertical direction but not linearly/predictably with the mouse.
A typical example is a port whose position is adjusted during a resize of its owning node.
Elements which do not move linearly with the mouse cannot snap to a defined position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes%23ARBITRARY_MOVE_Y Online Documentation}*/
ARBITRARY_MOVE_Y,
/**
The element moves in both directions but not linearly/predictably with the mouse.
A typical example is a port whose position is adjusted during a resize of its owning node.
Elements which do not move linearly with the mouse cannot snap to a defined position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes%23ARBITRARY_MOVE Online Documentation}*/
ARBITRARY_MOVE,
/**
The bitwise combination of {@link MoveTypes#LINEAR_MOVE_X} and {@link MoveTypes#ARBITRARY_MOVE_X}
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes%23MOVEMENT_X Online Documentation}*/
MOVEMENT_X,
/**
The bitwise combination of {@link MoveTypes#LINEAR_MOVE_Y} and {@link MoveTypes#ARBITRARY_MOVE_Y}
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveTypes%23MOVEMENT_Y Online Documentation}*/
MOVEMENT_Y
}
type MoveTypesStringValues = "fixed" | "linear-move-x" | "linear-move-y" | "linear-move" | "arbitrary-move-x" | "arbitrary-move-y" | "arbitrary-move" | "movement-x" | "movement-y";
/** Automatic conversion to {@link MoveTypes}. */
type MoveTypesConvertible = MoveTypesStringValues | MoveTypesStringValues[];
export interface MovementInfo extends YObject {
}
/**
Used in the context of {@link GraphSnapContext} to describe the movement of entities during the edit process.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo Online Documentation}*/
export abstract class MovementInfo {
/**
Initializes a new instance of the {@link MovementInfo} class.
@param originalLocation The original location.
@param moveType The type of the move.
@param movedItem The item that is being moved.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo Online Documentation}*/
constructor(originalLocation: Point, moveType: MoveTypes | MoveTypesConvertible, movedItem: IModelItem);
/**
Gets the new x coordinate of the element if the mouse has been moved by <code>dx</code> during the gesture.
This takes the {@link MovementInfo#moveType} and {@link MovementInfo#originalLocation} into account.
@param dx The delta x coordinate.
@returns The new x coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23getX(number) Online Documentation}*/
getX(dx: number): number;
/**
Gets the new y coordinate of the element if the mouse has been moved by <code>dy</code> during the gesture.
This takes the {@link MovementInfo#moveType} and {@link MovementInfo#originalLocation} into account.
@param dy The delta y coordinate.
@returns The new y coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23getY(number) Online Documentation}*/
getY(dy: number): number;
/**
Determines whether this instances moves synchronized with the provided info.
@param info The info to check.
@returns Whether this info and the given info move horizontally synchronized.
@see MovementInfo#horizontalAnchor
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23movesHorizontallySynchronizedWith(MovementInfo) Online Documentation}*/
movesHorizontallySynchronizedWith(info: MovementInfo): boolean;
/**
Determines whether this instances moves synchronized with the provided info.
@param info The info to check.
@returns Whether this info and the given info move vertically synchronized.
@see MovementInfo#verticalAnchor
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23movesVerticallySynchronizedWith(MovementInfo) Online Documentation}*/
movesVerticallySynchronizedWith(info: MovementInfo): boolean;
/**
Gets the item that is being moved.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23movedItem Online Documentation}*/
readonly movedItem: IModelItem;
/**
Gets or sets the horizontal movement info to which this info is anchored horizontally.
This info can be declared to be implicitly anchored to another info. Method {@link MovementInfo#movesHorizontallySynchronizedWith}
will try to match against the anchors to determine if two infos are moving synchronized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23horizontalAnchor Online Documentation}*/
horizontalAnchor: MovementInfo | null;
/**
Gets or sets the vertical movement info to which this info is anchored vertically.
This info can be declared to be implicitly anchored to another info. Method {@link MovementInfo#movesVerticallySynchronizedWith}
will try to match against the anchors to determine if two infos are moving synchronized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23verticalAnchor Online Documentation}*/
verticalAnchor: MovementInfo | null;
/**
Gets the original location of the element at the time the editing started.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23originalLocation Online Documentation}*/
readonly originalLocation: Point;
/**
Gets the current location as a live point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23currentLocation Online Documentation}*/
abstract readonly currentLocation: IPoint;
/**
Gets or sets the type of the movement for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23moveType Online Documentation}*/
moveType: MoveTypes;
/**
Gets a value indicating whether the y coordinate of the item is fixed during the move operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23isVerticallyFixed Online Documentation}*/
readonly isVerticallyFixed: boolean;
/**
Gets a value indicating whether the x coordinate of the item is fixed during the move operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MovementInfo%23isHorizontallyFixed Online Documentation}*/
readonly isHorizontallyFixed: boolean;
static $class: Class;
private $_MovementInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MovementInfo;
}
export interface GraphViewerInputMode extends GraphInputMode {
}
/**
* An {@link IInputMode} implementation for use in a {@link GraphComponent} to assign to the {@link CanvasComponent#inputMode} property.
* This mode can be used in a viewer-centric application where there is no need to modify the {@link IGraph} but
* browsing of the graph should be convenient. This mode allows for {@link GraphInputMode#addItemClickedListener clicking on items using the mouse},
* {@link GraphInputMode#addQueryItemToolTipListener displaying tool tips for items}, {@link GraphInputMode#addPopulateItemContextMenuListener showing a context menu}, {@link NavigationInputMode allowing for easy navigation and traversal},
* {@link MoveViewportInputMode moving the view port}, and {@link GraphInputMode#marqueeSelectableItems optionally doing marquee selection}. Also {@link ItemHoverInputMode} can be used to get
* notified of the elements that the mouse is hovering over.
*
* Due to the viewer-centric nature of this input mode, collapsing and expanding groups is disabled per default. To enable
* this feature, set {@link NavigationInputMode#allowCollapseGroup} and {@link NavigationInputMode#allowExpandGroup} to <code>true</code>.
*
* This input mode manages a set of specialized input modes, each handling a specific part of the interaction with the
* graph. The following list details those along with their default
* {@link IInputMode#priority priorities}. Lower priorities come earlier when handing events.
*
* <ul>
* <li><b>{@link WaitInputMode} (−1)</b> – Disables this input mode when layouts or animations run.</li>
* <li><b>{@link KeyboardInputMode} (0)</b> – Handles commands and arbitrary keyboard shortcuts.</li>
* <li><b>{@link ClickInputMode} (10)</b> – Handles mouse clicks and double-clicks.</li>
* <li><b>{@link TapInputMode} (20)</b> – Handles taps and double-tap.</li>
* <li><b>{@link MarqueeSelectionInputMode} (30)</b> – Allows dragging a rectangle to select items within it.</li>
* <li><b>{@link MoveViewportInputMode} (39)</b> – Allows panning the viewport.</li>
* <li><b>{@link ItemHoverInputMode} (55)</b> – Provides events to indicate that the mouse pointer hovers over an item.</li>
* <li><b>{@link NavigationInputMode} (55)</b> – Provides general navigation facilities, such as navigating from one item to another with the arrow keys.</li>
* <li><b>{@link ContextMenuInputMode} (60)</b> – Handles querying the context menu contents based on a location as well as preparing the menu for display.</li>
* <li><b>{@link MouseHoverInputMode} (100)</b> – Handles tool tips.</li>
* </ul>
* The <code>GraphViewerInputMode</code> can be installed by setting it as {@link CanvasComponent#inputMode} on the {@link GraphComponent} it handles.
*
* <br><b>Installing a configured GraphViewerInputMode</b>
* <pre>
* graphComponent.inputMode = new GraphViewerInputMode({
*   toolTipItems: GraphItemTypes.LABEL_OWNER,
*   clickableItems: GraphItemTypes.NODE,
*   focusableItems: GraphItemTypes.NODE,
*   selectableItems: GraphItemTypes.NONE,
*   marqueeSelectableItems: GraphItemTypes.NONE
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphViewerInputMode Online Documentation}
*/
export class GraphViewerInputMode {
/**
Initializes a new instance of the {@link GraphViewerInputMode} class.
@param [options] The parameters to pass.
@param [options.clickHitTestOrder] The order of the types of items that should be used to determine what item has been clicked or tapped. This option sets the {@link GraphInputMode#clickHitTestOrder} property on the created object.
@param [options.doubleClickHitTestOrder] The order of the types of items that should be used to determine what item has been double-clicked or double-tapped. This option sets the {@link GraphInputMode#doubleClickHitTestOrder} property on the created object.
@param [options.detailSelectionRecognizer] The event recognizer that determines if a click should select the item that is currently visible under the mouse cursor, without giving more important items higher priority. This option sets the {@link GraphInputMode#detailSelectionRecognizer} property on the created object.
@param [options.cyclicSelectionRecognizer] The event recognizer that determines if a click should cycle through all the items that are currently under the mouse cursor. This option sets the {@link GraphInputMode#cyclicSelectionRecognizer} property on the created object.
@param [options.multiSelectionRecognizer] The event recognizer that determines if a click is deemed a multi selection gesture. This option sets the {@link GraphInputMode#multiSelectionRecognizer} property on the created object.
@param [options.clickableItems] Which types of items should be reported through the {@link GraphInputMode#addItemClickedListener item click events}. This option sets the {@link GraphInputMode#clickableItems} property on the created object.
@param [options.focusableItems] The items that can be given focus via the {@link GraphInputMode#setCurrentItem} method. This option sets the {@link GraphInputMode#focusableItems} property on the created object.
@param [options.selectableItems] Which types of items should be selectable at all. This option sets the {@link GraphInputMode#selectableItems} property on the created object.
@param [options.selectablePredicate] A predicate that is queried to determine whether a given {@link IModelItem} is selectable. This option sets the {@link GraphInputMode#selectablePredicate} property on the created object.
@param [options.clickSelectableItems] Which types of items should be selectable by mouse clicks. This option sets the {@link GraphInputMode#clickSelectableItems} property on the created object.
@param [options.marqueeSelectableItems] Which types of items should be selected during {@link GraphInputMode#marqueeSelect marquee selections} or {@link GraphInputMode#lassoSelect lasso selection}. This option sets the {@link GraphInputMode#marqueeSelectableItems} property on the created object.
@param [options.ignoreVoidStyles] A value indicating whether void styles ({@link VoidNodeStyle}, {@link VoidEdgeStyle}, {@link VoidLabelStyle}, and {@link VoidPortStyle}) should be ignored when selecting or focusing items. This option sets the {@link GraphInputMode#ignoreVoidStyles} property on the created object.
@param [options.contextMenuInputMode] The {@link ContextMenuInputMode} which handles context menus on a {@link CanvasComponent}. This option sets the {@link GraphInputMode#contextMenuInputMode} property on the created object.
@param [options.contextMenuItems] The types of items a context menu should be queried for. This option sets the {@link GraphInputMode#contextMenuItems} property on the created object.
@param [options.mouseHoverInputMode] The {@link MouseHoverInputMode} which is responsible for displaying Tooltips on a {@link CanvasComponent}. This option sets the {@link GraphInputMode#mouseHoverInputMode} property on the created object.
@param [options.toolTipItems] The types of the items that should be queried a tool tip for. This option sets the {@link GraphInputMode#toolTipItems} property on the created object.
@param [options.clickInputMode] The {@link ClickInputMode} that is used by this instance to determine clicks. This option sets the {@link GraphInputMode#clickInputMode} property on the created object.
@param [options.tapInputMode] The {@link TapInputMode} which detects single and multiple taps on a {@link CanvasComponent}. This option sets the {@link GraphInputMode#tapInputMode} property on the created object.
@param [options.marqueeSelectionInputMode] The {@link MarqueeSelectionInputMode} which handles the selection of multiple elements by drawing a rectangle around them. This option sets the {@link GraphInputMode#marqueeSelectionInputMode} property on the created object.
@param [options.lassoSelectionInputMode] The {@link LassoSelectionInputMode} which handles the selection of multiple elements by drawing a polygon around them. This option sets the {@link GraphInputMode#lassoSelectionInputMode} property on the created object.
@param [options.navigationInputMode] The {@link NavigationInputMode} which is responsible for navigating and traversing the elements in the {@link IGraph}. This option sets the {@link GraphInputMode#navigationInputMode} property on the created object.
@param [options.waitInputMode] The {@link WaitInputMode} that is provided by this instance for those who need to make use of it. This option sets the {@link GraphInputMode#waitInputMode} property on the created object.
@param [options.itemHoverInputMode] The {@link ItemHoverInputMode} that is provided by this instance to detect when the cursor hovers over a graph item. This option sets the {@link GraphInputMode#itemHoverInputMode} property on the created object.
@param [options.moveViewportInputMode] The {@link MoveViewportInputMode} associated with this instance. This option sets the {@link GraphInputMode#moveViewportInputMode} property on the created object.
@param [options.keyboardInputMode] The {@link KeyboardInputMode} associated with this instance. This option sets the {@link GraphInputMode#keyboardInputMode} property on the created object.
@param [options.allowClipboardOperations] A property that determines whether clipboard operations with the usual shortcuts are enabled on the canvas. This option sets the {@link GraphInputMode#allowClipboardOperations} property on the created object.
@param [options.exclusive] A value indicating whether this mode will be the only one running when it has the mutex. This option sets the {@link MultiplexingInputMode#exclusive} property on the created object.
@param [options.enabled] The enabled state of this input mode. This option sets the {@link MultiplexingInputMode#enabled} property on the created object.
@param [options.priority] Sets the priority property.
@param [options.defaultCursor] The cursor to use whenever no child mode prefers a different cursor. This option sets the {@link MultiplexingInputMode#defaultCursor} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphViewerInputMode Online Documentation}*/
constructor(options?: {
clickHitTestOrder?: GraphItemTypes[];
doubleClickHitTestOrder?: GraphItemTypes[];
detailSelectionRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
cyclicSelectionRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
multiSelectionRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
clickableItems?: GraphItemTypes | GraphItemTypesConvertible;
focusableItems?: GraphItemTypes | GraphItemTypesConvertible;
selectableItems?: GraphItemTypes | GraphItemTypesConvertible;
selectablePredicate?: ((obj: IModelItem) => boolean) | null;
clickSelectableItems?: GraphItemTypes | GraphItemTypesConvertible;
marqueeSelectableItems?: GraphItemTypes | GraphItemTypesConvertible;
ignoreVoidStyles?: boolean;
contextMenuInputMode?: ContextMenuInputMode;
contextMenuItems?: GraphItemTypes | GraphItemTypesConvertible;
mouseHoverInputMode?: MouseHoverInputMode;
toolTipItems?: GraphItemTypes | GraphItemTypesConvertible;
clickInputMode?: ClickInputMode;
tapInputMode?: TapInputMode;
marqueeSelectionInputMode?: MarqueeSelectionInputMode;
lassoSelectionInputMode?: LassoSelectionInputMode;
navigationInputMode?: NavigationInputMode;
waitInputMode?: WaitInputMode;
itemHoverInputMode?: ItemHoverInputMode;
moveViewportInputMode?: MoveViewportInputMode;
keyboardInputMode?: KeyboardInputMode;
allowClipboardOperations?: boolean;
exclusive?: boolean;
enabled?: boolean;
priority?: number;
defaultCursor?: Cursor | null | CursorConvertible;
});
static $class: Class;
private $_GraphViewerInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphViewerInputMode;
}
export interface IClickListener {
}
/**
A simple interface that can be used to react to clicks on regions of {@link INodeStyle}s etc.
{@link GraphEditorInputMode} will query the {@link IModelItem}'s Lookup method for this interface if it has
detected {@link GraphInputMode#click}s. If the item yields an instance of this interface and the click was inside
the {@link IClickListener#getHitTestable}, then {@link IClickListener#onClicked} is invoked.
@see GraphEditorInputMode
@see CollapsibleNodeStyleDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClickListener Online Documentation}*/
export abstract class IClickListener {
/**
Creates an implementation of the interface {@link IClickListener} from the given definition.
The given object must contain exactly the abstract members of the {@link IClickListener} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IClickListener} interface.
@returns An instance of the {@link IClickListener} interface based on the given definition.*/
static create(interfaceDefinition: {
getHitTestable(): IHitTestable;
onClicked(context: IInputModeContext, location: Point): void;
}): IClickListener;
/**
Gets an {@link IHitTestable} that can be used to check if {@link IClickListener#onClicked} should be invoked.
@returns A hit testable that can determine whether {@link IClickListener#onClicked} should be invoked by a given click.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClickListener%23getHitTestable() Online Documentation}*/
abstract getHitTestable(): IHitTestable;
/**
Called by the framework to indicate that the {@link IModelItem} has been clicked by the user at the specified
location.
This method is called by {@link GraphInputMode#click} for the
{@link IModelItem}s that yielded an instance of this interface.
@param context The context to use for the click operation.
@param location The location of the click
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClickListener%23onClicked(IInputModeContext%2CPoint) Online Documentation}*/
abstract onClicked(context: IInputModeContext, location: Point): void;
static $class: Class;
private $_IClickListener: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IClickListener;
}
export interface IEdgePortHandleProvider {
}
/**
An interface for classes that can yield {@link IHandle} implementations for the {@link IEdge#sourcePort source} and {@link IEdge#targetPort target} ends of an
{@link IEdge}.
This interface will be queried by the default implementation of the {@link IHandleProvider} that is in the lookup
of an {@link IEdge}. It is recommended to use this interface instead of the more generic {@link IHandleProvider}
interface, because other generic code can then use this interface to gain access to a specific {@link IHandle} more
easily.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgePortHandleProvider Online Documentation}*/
export abstract class IEdgePortHandleProvider {
/**
Creates an implementation of the interface {@link IEdgePortHandleProvider} by using the given function as
implementation for its {@link IEdgePortHandleProvider#getHandle} method.
@param getHandle A function for {@link IEdgePortHandleProvider}'s single abstract method
{@link IEdgePortHandleProvider#getHandle}.
@returns An instance of the {@link IEdgePortHandleProvider} interface based on the given function.*/
static create(getHandle: (context: IInputModeContext, edge: IEdge, sourceHandle: boolean) => IHandle): IEdgePortHandleProvider;
/**
Gets an {@link IHandle} implementation for one end of the provided <code>edge</code>.
@param context The context in which the handle will be used.
@param edge The edge for which an handle is needed.
@param sourceHandle if set to <code>true</code> the handle for the source side/port should be returned.
@returns The handle to use for the provided side or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgePortHandleProvider%23getHandle(IInputModeContext%2CIEdge%2Cboolean) Online Documentation}*/
abstract getHandle(context: IInputModeContext, edge: IEdge, sourceHandle: boolean): IHandle | null;
static $class: Class;
private $_IEdgePortHandleProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgePortHandleProvider;
}
export interface IEditLabelHelper {
}
/**
Helper interface to customize interactive label editing.
Implementations of this interface can be put into the {@link ILookup} of {@link ILabelOwner}s and
{@link ILabel}s so that it is possible to tweak the behavior on a case by case basis.
Conceptually, in most cases, implementations of this interface behaves like an additional event handler for {@link GraphEditorInputMode#addLabelAddingListener LabelAdding}
and {@link GraphEditorInputMode#addLabelEditingListener LabelEditing}. If no event handler
handled those events, an {@link IEditLabelHelper} will be queried from the {@link ILookup} and the unhandled
event arguments passed to the respective methods.
@see EditLabelHelper
@see GraphEditorInputMode#createLabel
@see GraphEditorInputMode#editLabel
@see GraphEditorInputMode#addLabel
@see GraphEditorInputMode#addLabelAddingListener
@see GraphEditorInputMode#addLabelEditingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEditLabelHelper Online Documentation}*/
export abstract class IEditLabelHelper {
/**
Creates an implementation of the interface {@link IEditLabelHelper} from the given definition.
The given object must contain exactly the abstract members of the {@link IEditLabelHelper} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IEditLabelHelper} interface.
@returns An instance of the {@link IEditLabelHelper} interface based on the given definition.*/
static create(interfaceDefinition: {
onLabelAdding(evt: LabelEditingEventArgs): void;
onLabelEditing(evt: LabelEditingEventArgs): void;
}): IEditLabelHelper;
/**
Helper method that will be called when trying to add a label to an {@link ILabelOwner}.
Implementations of this method can set the {@link ILabel#style}, {@link ILabel#layoutParameter}, {@link ILabel#preferredSize}, and {@link ITagOwner#tag} for newly-added labels by setting the appropriate properties on
<code>evt</code>.
Setting the {@link LabelEditingEventArgs#cancel} property on <code>evt</code> to <code>true</code> will disallow label creation.
@param evt The original event arguments.
@see IEditLabelHelper#onLabelEditing
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEditLabelHelper%23onLabelAdding(LabelEditingEventArgs) Online Documentation}*/
abstract onLabelAdding(evt: LabelEditingEventArgs): void;
/**
Helper method that will be called when trying to edit a label.
This method can be called for editing labels for an {@link ILabelOwner}, in which case the {@link LabelEditingEventArgs#label} property on <code>evt</code> will be
<code>null</code>, but the {@link LabelEditingEventArgs#owner} property will be non-<code>null</code>. It can also be called for editing a specific label, in which case the {@link LabelEditingEventArgs#label}
property on <code>evt</code> will be non-<code>null</code>, and the {@link LabelEditingEventArgs#owner} property will be set to the label's owner.
Implementations of this method can set a specific label to be edited, by setting the {@link LabelEditingEventArgs#label} property of the <code>evt</code> to an existing
label instance. This label does not need to belong to the same owner that is queried for. Setting the {@link LabelEditingEventArgs#label} property to <code>null</code>
will not edit an existing label, but instead add a new one on the {@link ILabelOwner} set in the {@link LabelEditingEventArgs#owner} property.
When a new label is added, the properties {@link LabelEditingEventArgs#style}, {@link LabelEditingEventArgs#layoutParameter}, {@link LabelEditingEventArgs#preferredSize}, and {@link LabelEditingEventArgs#tag} are used for the newly-created label, just as in
{@link IEditLabelHelper#onLabelAdding}.
Setting the {@link LabelEditingEventArgs#cancel} property on <code>evt</code> to <code>true</code> will disallow label editing and creation.
@param evt The original event arguments.
@see IEditLabelHelper#onLabelAdding
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEditLabelHelper%23onLabelEditing(LabelEditingEventArgs) Online Documentation}*/
abstract onLabelEditing(evt: LabelEditingEventArgs): void;
static $class: Class;
private $_IEditLabelHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEditLabelHelper;
}
export interface ILabelSnapContextHelper {
}
/**
Provides the snap lines and the snap results to the {@link LabelSnapContext} during dragging of labels.
@see LabelSnapContext
@see LabelDecorator#labelSnapContextHelperDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelSnapContextHelper Online Documentation}*/
export abstract class ILabelSnapContextHelper {
/**
Called during {@link IDragHandler#initializeDrag initialization} of a <code>label</code> dragging to add {@link SnapLine snap lines} to which the label can potentially snap to the
<code>snapContext</code>.
@param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
the context explicitly.
@param inputModeContext The context of the input mode that handles the dragging.
@param label The label that is dragged.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelSnapContextHelper%23addSnapLines(LabelSnapContext%2CIInputModeContext%2CILabel) Online Documentation}*/
abstract addSnapLines(snapContext: LabelSnapContext, inputModeContext: IInputModeContext, label: ILabel): void;
/**
Called while the given <code>label</code> is {@link IDragHandler#handleMove dragged} to add {@link SnapResult snap results} for the {@link SnapLine snap lines} provided by the <code>context</code>.
@param context The snap context which manages the snap lines and the settings.
@param evt The event argument to obtain the necessary information from and {@link CollectSnapResultsEventArgs#addSnapResult add results to}.
@param suggestedLayout The {@link ILabel#layout layout} of the label that would be used without snapping.
@param label The label that is dragged.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelSnapContextHelper%23collectSnapResults(LabelSnapContext%2CCollectSnapResultsEventArgs%2CIOrientedRectangle%2CILabel) Online Documentation}*/
abstract collectSnapResults(context: LabelSnapContext, evt: CollectSnapResultsEventArgs, suggestedLayout: IOrientedRectangle, label: ILabel): void;
/**
Creates an implementation of the interface {@link ILabelSnapContextHelper} from the given definition.
The given object must contain exactly the abstract members of the {@link ILabelSnapContextHelper} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link ILabelSnapContextHelper} interface.
@returns An instance of the {@link ILabelSnapContextHelper} interface based on the given definition.*/
static create(interfaceDefinition: {
addSnapLines(snapContext: LabelSnapContext, inputModeContext: IInputModeContext, label: ILabel): void;
collectSnapResults(context: LabelSnapContext, evt: CollectSnapResultsEventArgs, suggestedLayout: IOrientedRectangle, label: ILabel): void;
}): ILabelSnapContextHelper;
static $class: Class;
private $_ILabelSnapContextHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelSnapContextHelper;
}
export interface INodeSizeConstraintProvider {
}
/**
An interface used by implementations that provide various size constraints for {@link INode}s.
@see NodeSizeConstraintProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeSizeConstraintProvider Online Documentation}*/
export abstract class INodeSizeConstraintProvider {
/**
Creates an implementation of the interface {@link INodeSizeConstraintProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link INodeSizeConstraintProvider} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link INodeSizeConstraintProvider} interface.
@returns An instance of the {@link INodeSizeConstraintProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
getMinimumSize(node: INode): Size;
getMaximumSize(node: INode): Size;
getMinimumEnclosedArea(node: INode): Rect;
}): INodeSizeConstraintProvider;
/**
Returns the maximum size allowed for the given <code>node</code>.
@param node The node to return the maximum size for.
@returns The maximum size or {@link Size#INFINITE} if there is no constraint on the size.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeSizeConstraintProvider%23getMaximumSize(INode) Online Documentation}*/
abstract getMaximumSize(node: INode): Size;
/**
Returns the minimum area that needs to be enclosed by the given <code>node</code>.
@param node The node to return the area for.
@returns The area to enclose or {@link Rect#EMPTY} if there is no constraint on the size due to an enclosed area.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeSizeConstraintProvider%23getMinimumEnclosedArea(INode) Online Documentation}*/
abstract getMinimumEnclosedArea(node: INode): Rect;
/**
Returns the minimum size allowed for the given <code>node</code>.
@param node The node to return the minimum size for.
@returns The minimum size or {@link Size#EMPTY} if there is no constraint on the size.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeSizeConstraintProvider%23getMinimumSize(INode) Online Documentation}*/
abstract getMinimumSize(node: INode): Size;
static $class: Class;
private $_INodeSizeConstraintProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeSizeConstraintProvider;
}
export interface NodeSizeConstraintProvider extends YObject, INodeSizeConstraintProvider {
/**
@inheritdoc*/
getMaximumSize(node: INode): Size;
/**
@inheritdoc*/
getMinimumEnclosedArea(node: INode): Rect;
/**
@inheritdoc*/
getMinimumSize(node: INode): Size;
} /**
A simple default implementation of {@link INodeSizeConstraintProvider}
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSizeConstraintProvider Online Documentation}*/
export class NodeSizeConstraintProvider {
/**
Creates a new instance using the provided size instances as the initial values.
Not specifying the <code>minimumEnclosedArea</code> will set it to {@link Rect#EMPTY}.
@param minimumSize The {@link NodeSizeConstraintProvider#minimumSize}.
@param maximumSize The {@link NodeSizeConstraintProvider#maximumSize}.
@param [minimumEnclosedArea=null] The {@link NodeSizeConstraintProvider#minimumEnclosedArea}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSizeConstraintProvider Online Documentation}*/
constructor(minimumSize: ISize | ISizeConvertible, maximumSize: ISize | ISizeConvertible, minimumEnclosedArea?: IRectangle | null | IRectangleConvertible);
/**
Gets or sets the instance to return by {@link NodeSizeConstraintProvider#getMaximumSize}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSizeConstraintProvider%23maximumSize Online Documentation}*/
maximumSize: ISize;
/**
Gets or sets the instance to return by {@link NodeSizeConstraintProvider#getMinimumEnclosedArea}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSizeConstraintProvider%23minimumEnclosedArea Online Documentation}*/
minimumEnclosedArea: IRectangle;
/**
Gets or sets the instance to return by {@link NodeSizeConstraintProvider#getMinimumSize}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSizeConstraintProvider%23minimumSize Online Documentation}*/
minimumSize: ISize;
static $class: Class;
private $_NodeSizeConstraintProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeSizeConstraintProvider;
}
export interface IOrthogonalEdgeHelper {
}
/**
Interface used by {@link OrthogonalEdgeEditingContext} and the like that can be decorated to the {@link ILookup}
of {@link IEdge}s.
This interface is for editing edges in such a way that their path stays orthogonal, i.e. all of the segments are
oriented either horizontally or vertically.
@see OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrthogonalEdgeHelper Online Documentation}*/
export abstract class IOrthogonalEdgeHelper {
/**
Invoked after the provided edge has been edited orthogonally.
This callback can be used to remove unused bends and finalize the orthogonal edge editing gesture.
@param context The input mode context which edited the edge.
@param graph The graph to use for modifying the edge instance.
@param edge The edge to clean up the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrthogonalEdgeHelper%23cleanUpEdge(IInputModeContext%2CIGraph%2CIEdge) Online Documentation}*/
abstract cleanUpEdge(context: IInputModeContext, graph: IGraph, edge: IEdge): void;
/**
Creates an implementation of the interface {@link IOrthogonalEdgeHelper} from the given definition.
The given object must contain exactly the abstract members of the {@link IOrthogonalEdgeHelper} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IOrthogonalEdgeHelper} interface.
@returns An instance of the {@link IOrthogonalEdgeHelper} interface based on the given definition.*/
static create(interfaceDefinition: {
getSegmentOrientation(context: IInputModeContext, edge: IEdge, segmentIndex: number): SegmentOrientation;
shouldMoveEndImplicitly(context: IInputModeContext, edge: IEdge, sourceEnd: boolean): boolean;
shouldEditOrthogonally(context: IInputModeContext, edge: IEdge): boolean;
cleanUpEdge(context: IInputModeContext, graph: IGraph, edge: IEdge): void;
}): IOrthogonalEdgeHelper;
/**
Gets the declared orientation of the given segment at the provided edge.
The orientation cannot always be inferred from the current geometry: If a segment has a zero length it is unclear what
orientation it should have, also a segment could be accidentally orthogonally oriented, while in fact it is considered
{@link SegmentOrientation#NON_ORTHOGONAL}.
@param context The input mode context in which the orientation is needed.
@param edge The edge to inspect.
@param segmentIndex The index of the segment.
@returns The declared orientation of the segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrthogonalEdgeHelper%23getSegmentOrientation(IInputModeContext%2CIEdge%2Cnumber) Online Documentation}*/
abstract getSegmentOrientation(context: IInputModeContext, edge: IEdge, segmentIndex: number): SegmentOrientation;
/**
Determines whether the provided edge should be edited orthogonally in the specified input mode context.
If this method returns <code>false</code>, the other methods will not be queried at all.
@param context The input mode context in which the edge is about to be edited.
@param edge The edge to inspect.
@returns <code>true</code> if the edge should be orthogonally edited in the specified input mode context; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrthogonalEdgeHelper%23shouldEditOrthogonally(IInputModeContext%2CIEdge) Online Documentation}*/
abstract shouldEditOrthogonally(context: IInputModeContext, edge: IEdge): boolean;
/**
Determines whether this end of the provided edge can be moved in the input mode context.
This information is required to determine whether an edge's first or last segment needs to be split or the adjacent edge
end can be moved along with the other end of the segment.
@param context The input mode context in which the segment is edited.
@param edge The edge to inspect.
@param sourceEnd if set to <code>true</code> the source end of the edge is queried, otherwise the target end.
@returns <code>true</code> if the end of the edge can be moved for the specified input mode context; <code>false</code> otherwise, in which case the
corresponding segment needs to be split to keep the segment orientation orthogonal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOrthogonalEdgeHelper%23shouldMoveEndImplicitly(IInputModeContext%2CIEdge%2Cboolean) Online Documentation}*/
abstract shouldMoveEndImplicitly(context: IInputModeContext, edge: IEdge, sourceEnd: boolean): boolean;
static $class: Class;
private $_IOrthogonalEdgeHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IOrthogonalEdgeHelper;
}
export interface IReparentNodeHandler {
}
/**
Interface used for implementations that recognize, approve and disapprove node reparenting gestures, as well as actually
performs the reparenting.
Implementations of this interface are queried from the {@link IInputModeContext} by code that wants to reparent a
node. Specifically this is used by default to reparent nodes during the {@link MoveInputMode dragging of nodes}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReparentNodeHandler Online Documentation}*/
export abstract class IReparentNodeHandler {
/**
Creates an implementation of the interface {@link IReparentNodeHandler} from the given definition.
The given object must contain exactly the abstract members of the {@link IReparentNodeHandler} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IReparentNodeHandler} interface.
@returns An instance of the {@link IReparentNodeHandler} interface based on the given definition.*/
static create(interfaceDefinition: {
isReparentGesture(context: IInputModeContext, node: INode): boolean;
shouldReparent(context: IInputModeContext, node: INode): boolean;
isValidParent(context: IInputModeContext, node: INode, newParent: INode | null): boolean;
reparent(context: IInputModeContext, node: INode, newParent: INode | null): void;
}): IReparentNodeHandler;
/**
Determines whether the current gesture that can be determined through the context is a reparent gesture.
@param context The context that provides information about the user input.
@param node The node that will possibly be reparented.
@returns Whether this is a reparenting gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReparentNodeHandler%23isReparentGesture(IInputModeContext%2CINode) Online Documentation}*/
abstract isReparentGesture(context: IInputModeContext, node: INode): boolean;
/**
Determines whether the provided node may be reparented to a <code>newParent</code>.
@param context The context that provides information about the user input.
@param node The node that will be reparented.
@param newParent The potential new parent or <code>null</code>. Can be a group or a non-group node.
@returns Whether <code>newParent</code> is a valid new parent for <code>node</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReparentNodeHandler%23isValidParent(IInputModeContext%2CINode%2CINode) Online Documentation}*/
abstract isValidParent(context: IInputModeContext, node: INode, newParent: INode | null): boolean;
/**
Performs the actual reparenting after the reparent gesture has been finalized.
Implementations should {@link IGraph#setParent set the parent} of <code>node</code> to <code>newParent</code>.
@param context The context that provides information about the user input.
@param node The node that will be reparented.
@param newParent The potential new parent or <code>null</code>. Can be a group or a non-group node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReparentNodeHandler%23reparent(IInputModeContext%2CINode%2CINode) Online Documentation}*/
abstract reparent(context: IInputModeContext, node: INode, newParent: INode | null): void;
/**
Determines whether the user may detach the given node from its current parent in order to reparent it.
@param context The context that provides information about the user input.
@param node The node that is about to be detached from its current parent.
@returns Whether the node may be detached and reparented.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReparentNodeHandler%23shouldReparent(IInputModeContext%2CINode) Online Documentation}*/
abstract shouldReparent(context: IInputModeContext, node: INode): boolean;
static $class: Class;
private $_IReparentNodeHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IReparentNodeHandler;
}
export interface ISnapLineProvider {
}
/**
Queried by the {@link GraphSnapContext} to add {@link OrthogonalSnapLine}s to its collections during the
{@link SnapContext#isInitializing initialization phase}.
Implementations of this interface are queried by {@link GraphSnapContext} for each item that is not modified during
the edit using the {@link ILookup#lookup} of the respective item.
@see NodeDecorator#snapLineProviderDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISnapLineProvider Online Documentation}*/
export abstract class ISnapLineProvider {
/**
Called by the {@link GraphSnapContext} when a {@link GraphSnapContext#initializeDrag drag} is about to start.
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param evt The argument to use for adding snap lines.
@param item The item to add snap lines for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISnapLineProvider%23addSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CIModelItem) Online Documentation}*/
abstract addSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, item: IModelItem): void;
/**
Creates an implementation of the interface {@link ISnapLineProvider} by using the given function as implementation
for its {@link ISnapLineProvider#addSnapLines} method.
@param addSnapLines A function for {@link ISnapLineProvider}'s single abstract method {@link ISnapLineProvider#addSnapLines}.
@returns An instance of the {@link ISnapLineProvider} interface based on the given function.*/
static create(addSnapLines: (context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, item: IModelItem) => void): ISnapLineProvider;
static $class: Class;
private $_ISnapLineProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ISnapLineProvider;
}
export interface ItemClickedEventArgs<T> extends ClickEventArgs {
}
/**
Provides data for the {@link GraphInputMode#addItemClickedListener ItemClicked},
{@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked},
{@link GraphInputMode#addItemLeftClickedListener ItemLeftClicked},
{@link GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked}, {@link GraphInputMode#addItemRightClickedListener ItemRightClicked}
and {@link GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked} events.
Type parameter <code>T</code>: The type of the item this event carries.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemClickedEventArgs Online Documentation}*/
export class ItemClickedEventArgs<T> {
/**
Initializes a new instance of the {@link ItemClickedEventArgs} class.
@param item The item which is the subject of the event.
@param context The context in which the click or tap occurred.
@param location The location.
@param [modifiers=ModifierKeys.NONE] State of the modifiers.
@param [mouseButtons=MouseButtons.LEFT] The mouse button(s) that have changed.
@param [clickCount=1] The number of clicks.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemClickedEventArgs Online Documentation}*/
constructor(item: T, context: IInputModeContext, location: Point | PointConvertible, modifiers?: ModifierKeys | ModifierKeysConvertible, mouseButtons?: MouseButtons | MouseButtonsConvertible, clickCount?: number);
/**
Initializes a new instance of the {@link ItemClickedEventArgs} class.
@param options The parameters to pass.
@param options.item The item which is the subject of the event.
@param options.context The context in which the click or tap occurred.
@param options.location The location.
@param [options.modifiers=ModifierKeys.NONE] State of the modifiers.
@param [options.mouseButtons=MouseButtons.LEFT] The mouse button(s) that have changed.
@param [options.clickCount=1] The number of clicks.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemClickedEventArgs Online Documentation}*/
constructor(options: {
item: T | null;
context: IInputModeContext;
location: Point | PointConvertible;
modifiers?: ModifierKeys | ModifierKeysConvertible;
mouseButtons?: MouseButtons | MouseButtonsConvertible;
clickCount?: number;
});
/**
Gets the item that is the subject of the event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemClickedEventArgs%23item Online Documentation}*/
item: T;
static $class: Class;
private $_ItemClickedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ItemClickedEventArgs<T>;
}
export interface ItemHoverInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
cancel(): void;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An input mode for use in a {@link GraphComponent} that fires events when the mouse enters or leaves the
* visualization of a {@link IModelItem graph item}.
* This mode can be used to determine when the mouse is being moved from one item to the next.
*
* This mode is {@link ItemHoverInputMode#exclusive} by default.
*
* Typically the <code>ItemHoverInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#itemHoverInputMode} property.
*
* <br><b>Getting the ItemHoverInputMode from its parent input mode</b>
* <pre>
* const itemHoverInputMode = mode.itemHoverInputMode
* </pre>
*
* The <code>ItemHoverInputMode</code> raises the {@link ItemHoverInputMode#addHoveredItemChangedListener HoveredItemChanged}
* event each time cursor hovers over a different item or the canvas in which case the hovered item is
* <code>null</code>. One can also configure the item types which are reported:
*
* <br><b>Configuring the ItemHoverInputMode to handle nodes and edges</b>
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable the ItemHoverInputMode and let it handle edges and nodes
* mode.itemHoverInputMode.enabled = true
* mode.itemHoverInputMode.hoverItems = GraphItemTypes.EDGE | GraphItemTypes.NODE
* // ignore items of other types which might be in front of them
* mode.itemHoverInputMode.discardInvalidItems = false
* // handle changes on the hovered items
* mode.itemHoverInputMode.addHoveredItemChangedListener((sender, args) => {
*   const oldItem = args.oldItem
*   // e.g. remove the highlight from oldItem here
*   const newItem = args.item
*   // e.g. add a highlight to newItem here
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode Online Documentation}
*/
export class ItemHoverInputMode {
/**
Initializes a new instance of the {@link ItemHoverInputMode} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode Online Documentation}*/
constructor();
/**
Gets the items that have been hit at the given location.
@param location The location in world coordinates to query.
@returns An enumerable over all items that have been hit at the given location.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23getHitItemsAt(Point) Online Documentation}*/
getHitItemsAt(location: Point): IEnumerable<IModelItem>;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link ItemHoverInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Determines whether the given item is a valid item to be considered for hovering.
This implementation checks whether the item is covered by the {@link ItemHoverInputMode#hoverItems} set, only.
@param item The item to check.
@returns <code>true</code> if it is valid to report a hover over the specified item; <code>false</code> otherwise.
@see ItemHoverInputMode#discardInvalidItems
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23isValidHoverItem(IModelItem) Online Documentation}*/
isValidHoverItem(item: IModelItem): boolean;
/**
Called after {@link ItemHoverInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Raises the {@link ItemHoverInputMode#addHoveredItemChangedListener HoveredItemChanged} event.
@param evt The {@link HoveredItemChangedEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23onHoveredItemChanged(HoveredItemChangedEventArgs) Online Documentation}*/
onHoveredItemChanged(evt: HoveredItemChangedEventArgs): void;
/**
Called after {@link ItemHoverInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Overridden to only return <code>true</code> if this instance does not currently {@link ConcurrencyController#hasMutex have the input mutex}.
@returns <code>true</code> iff this instance does not {@link ConcurrencyController#hasMutex own the mutex}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23tryStop() Online Documentation}*/
tryStop(): boolean;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Forces a reevaluation of the item that the mouse is currently hovering over or at a specific location.
This method may be called by code that is aware of the fact that the visualization has changed, but the mouse may not
have been moved. By default this implementation will only re-query the items at the mouse location when the mouse has
moved. This method can be called to force a reevaluation in other cases.
@param [location=null] The query location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23updateHover(Point) Online Documentation}*/
updateHover(location?: Point | null): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link ItemHoverInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link ItemHoverInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the ItemHoverInputMode on its parent input mode</b>
* <pre>
* mode.itemHoverInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link ItemHoverInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
/**
* Adds the given listener for the <code>HoveredItemChanged</code> event that occurs when the item that is being hovered over with the
* mouse changes.
* This event is also fired when the mouse pointer leaves an item.
*
* The {@link ItemHoverInputMode} raises the {@link ItemHoverInputMode#addHoveredItemChangedListener HoveredItemChanged}
* event each time cursor hovers over a different item or the canvas in which case the hovered item is
* <code>null</code>. One can also configure the item types which are reported:
*
* <br><b>Configuring the ItemHoverInputMode to handle nodes and edges</b>
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable the ItemHoverInputMode and let it handle edges and nodes
* mode.itemHoverInputMode.enabled = true
* mode.itemHoverInputMode.hoverItems = GraphItemTypes.EDGE | GraphItemTypes.NODE
* // ignore items of other types which might be in front of them
* mode.itemHoverInputMode.discardInvalidItems = false
* // handle changes on the hovered items
* mode.itemHoverInputMode.addHoveredItemChangedListener((sender, args) => {
*   const oldItem = args.oldItem
*   // e.g. remove the highlight from oldItem here
*   const newItem = args.item
*   // e.g. add a highlight to newItem here
* })
* </pre>
*
* @param listener The listener to add.
* @see ItemHoverInputMode#removeHoveredItemChangedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23HoveredItemChanged Online Documentation}
*/
addHoveredItemChangedListener(listener: (sender: any, evt: HoveredItemChangedEventArgs) => void): void;
/**
* Removes the given listener for the <code>HoveredItemChanged</code> event that occurs when the item that is being hovered over with
* the mouse changes.
* This event is also fired when the mouse pointer leaves an item.
*
* The {@link ItemHoverInputMode} raises the {@link ItemHoverInputMode#addHoveredItemChangedListener HoveredItemChanged}
* event each time cursor hovers over a different item or the canvas in which case the hovered item is
* <code>null</code>. One can also configure the item types which are reported:
*
* <br><b>Configuring the ItemHoverInputMode to handle nodes and edges</b>
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable the ItemHoverInputMode and let it handle edges and nodes
* mode.itemHoverInputMode.enabled = true
* mode.itemHoverInputMode.hoverItems = GraphItemTypes.EDGE | GraphItemTypes.NODE
* // ignore items of other types which might be in front of them
* mode.itemHoverInputMode.discardInvalidItems = false
* // handle changes on the hovered items
* mode.itemHoverInputMode.addHoveredItemChangedListener((sender, args) => {
*   const oldItem = args.oldItem
*   // e.g. remove the highlight from oldItem here
*   const newItem = args.item
*   // e.g. add a highlight to newItem here
* })
* </pre>
*
* @param listener The listener to remove.
* @see ItemHoverInputMode#addHoveredItemChangedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23HoveredItemChanged Online Documentation}
*/
removeHoveredItemChangedListener(listener: (sender: any, evt: HoveredItemChangedEventArgs) => void): void;
/**
Gets the current item the mouse is hovering over.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23currentHoverItem Online Documentation}*/
readonly currentHoverItem: IModelItem | null;
/**
* Gets or sets which graph items are considered by this input mode.
* The default is {@link GraphItemTypes#ALL}.
*
* Note that the default is set to a different value if this mode is a child mode of the {@link GraphEditorInputMode}
* or {@link GraphViewerInputMode}. The different default is set in {@link GraphInputMode#createItemHoverInputMode}
* and documented there.
*
* <br><b>Configuring the ItemHoverInputMode to handle nodes and edges</b>
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable the ItemHoverInputMode and let it handle edges and nodes
* mode.itemHoverInputMode.enabled = true
* mode.itemHoverInputMode.hoverItems = GraphItemTypes.EDGE | GraphItemTypes.NODE
* // ignore items of other types which might be in front of them
* mode.itemHoverInputMode.discardInvalidItems = false
* // handle changes on the hovered items
* mode.itemHoverInputMode.addHoveredItemChangedListener((sender, args) => {
*   const oldItem = args.oldItem
*   // e.g. remove the highlight from oldItem here
*   const newItem = args.item
*   // e.g. add a highlight to newItem here
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23hoverItems Online Documentation}
*/
hoverItems: GraphItemTypes;
/**
Gets or sets the cursor to use when the mouse is hovering over a {@link ItemHoverInputMode#isValidHoverItem} valid
hover item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23hoverCursor Online Documentation}*/
hoverCursor: Cursor | null;
/**
Gets or sets a value indicating whether items that have been hit at the current location that are not {@link ItemHoverInputMode#isValidHoverItem valid items} should
be ignored (and an underlying valid item be considered instead) or reported as <code>null</code>.
Set this to <code>true</code> if invalid items should be reported as <code>null</code>. Set this to <code>false</code> to let hit test enumeration continue to
find the next valid item.
The default is <code>true</code>.
@see ItemHoverInputMode#isValidHoverItem
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemHoverInputMode%23discardInvalidItems Online Documentation}*/
discardInvalidItems: boolean;
static $class: Class;
private $_ItemHoverInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ItemHoverInputMode;
}
export interface HoveredItemChangedEventArgs extends ItemEventArgs<IModelItem> {
}
/**
Event argument class containing information about which {@link IModelItem} the mouse is currently hovering over.
This event is used by the {@link ItemHoverInputMode#addHoveredItemChangedListener HoveredItemChanged}
event. The {@link ItemEventArgs#item} property yields the item that is currently being hovered over or <code>null</code> if the mouse just left an item without
hovering over an other one. The {@link HoveredItemChangedEventArgs#oldItem} property provides the previously hovered item or <code>null</code> if the mouse wasn't previously
hovering over a valid item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HoveredItemChangedEventArgs Online Documentation}*/
export class HoveredItemChangedEventArgs {
/**
Initializes a new instance of the {@link HoveredItemChangedEventArgs} class.
@param item The model item, possibly <code>null</code>.
@param oldItem The old item, possibly <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HoveredItemChangedEventArgs Online Documentation}*/
constructor(item: IModelItem | null, oldItem: IModelItem | null);
/**
Gets or sets the item that was previously hovered, possibly <code>null</code>.
@see ItemEventArgs#item
@see {@link https://docs.yworks.com/yfileshtml/#/api/HoveredItemChangedEventArgs%23oldItem Online Documentation}*/
oldItem: IModelItem | null;
static $class: Class;
private $_HoveredItemChangedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HoveredItemChangedEventArgs;
}
export interface LabelEditingEventArgs extends InputModeEventArgs {
}
/**
Event arguments for the {@link GraphEditorInputMode#addLabelAddingListener LabelAdding} and {@link GraphEditorInputMode#addLabelEditingListener LabelEditing}
events, as well as the {@link IEditLabelHelper} interface.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs Online Documentation}*/
export class LabelEditingEventArgs {
/**
Initializes a new instance of the {@link LabelEditingEventArgs} class with the given context, owner, and label.
@param context The input mode context to use.
@param owner The {@link ILabelOwner} whose label should be edited.
@param label The {@link ILabel} that should be edited.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs Online Documentation}*/
constructor(context: IInputModeContext, owner: ILabelOwner | null, label: ILabel | null);
/**
Gets or sets the label to edit.
This can be set to an exiting label to edit a specific one, or set to <code>null</code> to add a new label instead (e.g. for
{@link ILabelOwner}s that have no label yet).
If this property is explicitly set to <code>null</code>, the properties {@link LabelEditingEventArgs#style}, {@link LabelEditingEventArgs#preferredSize}, {@link LabelEditingEventArgs#layoutParameter}, and {@link LabelEditingEventArgs#tag} are used for the newly-added label, just like with
{@link LabelEditingEventArgs}.
Setting this property implicitly also sets {@link LabelEditingEventArgs#handled} to <code>true</code>.
Note that this property may be initialized to <code>null</code> by calling code. However, in order to trigger automatically adding a
new label, you need to set the property value explicitly to <code>null</code> even in that case.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs%23label Online Documentation}*/
label: ILabel | null;
/**
Gets or sets a value indicating whether the action (adding or editing a label) should be canceled.
Setting this property implicitly also sets {@link LabelEditingEventArgs#handled} to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs%23cancel Online Documentation}*/
cancel: boolean;
/**
* Gets or sets a value indicating whether this event has been handled.
* A handled instance of {@link LabelEditingEventArgs} will not be passed to further event handlers or an {@link IEditLabelHelper}
* instance.
*
* Setting any of the other properties implicitly also sets this to <code>true</code>.
*
* Note that this property needs to be set before the function synchronously returns. This means that setting the value in
* an <code>async</code> function after the first <code>await</code> or in an asynchronous callback, it cannot not be seen by the emitter of the
* event and the sender will consider the event as not being handled.
*
* <pre>
* // wrong:
* mode.addLabelAddingListener(async (sender, args) => {
*   await something()
*   args.handled = true  // will be set *after* all other event listeners have been invoked, only
* })
*
* // correct:
* mode.addLabelAddingListener(async (sender, args) => {
*   args.handled = true // will be set immediately and 'mode' will see the change
*   await something()
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs%23handled Online Documentation}
*/
handled: boolean;
/**
Gets or sets the owner of the label.
This property can be <code>null</code> if there is no owner known, e.g. if the event is raised for
non-{@link ILabelOwner}s. Otherwise it is initially set to the item for which the event is raised.
Setting this property implicitly also sets {@link LabelEditingEventArgs#handled} to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs%23owner Online Documentation}*/
owner: ILabelOwner | null;
/**
Gets or sets the {@link ILabel#style} of the label to add.
Setting this property implicitly also sets {@link LabelEditingEventArgs#handled} to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs%23style Online Documentation}*/
style: ILabelStyle | null;
/**
Gets or sets the {@link ILabel#preferredSize} of the label to add.
Setting this property implicitly also sets {@link LabelEditingEventArgs#handled} to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs%23preferredSize Online Documentation}*/
preferredSize: Size | null;
/**
Gets or sets the {@link ILabel#layoutParameter} of the label to add.
Setting this property implicitly also sets {@link LabelEditingEventArgs#handled} to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs%23layoutParameter Online Documentation}*/
layoutParameter: ILabelModelParameter | null;
/**
Gets or sets the {@link ITagOwner#tag} of the label to add.
Setting this property implicitly also sets {@link LabelEditingEventArgs#handled} to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs%23tag Online Documentation}*/
tag: any | null;
/**
Gets or sets the configurator for the {@link TextEditorInputMode} for editing the provided label instance.
Setting this property implicitly also sets {@link LabelEditingEventArgs#handled} to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEditingEventArgs%23textEditorInputModeConfigurator Online Documentation}*/
textEditorInputModeConfigurator: ((context: IInputModeContext, mode: TextEditorInputMode, label: ILabel) => void) | null;
static $class: Class;
private $_LabelEditingEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelEditingEventArgs;
}
export interface LabelSnapContext extends SnapContext {
}
/**
* Manages interactive snapping of {@link ILabel}s to their owner during drag operations like movements.
* Snapping is enabled for {@link IGraph} items by setting an instance of this on the {@link GraphEditorInputMode}
* as {@link GraphEditorInputMode#labelSnapContext}.
*
* This class provides a set of properties to customize the snapping behavior.
*
* {@link IPositionHandler} implementations and similar classes can {@link ILookup#lookup retrieve} instances of this class from the
* {@link IInputModeContext}.
*
* Snapping is enabled by installing the snapping context on the {@link GraphEditorInputMode}
*
* <br><b>Enabling snapping by setting the snap context</b>
* <pre>
* mode.snapContext = new GraphSnapContext()
* mode.labelSnapContext = new LabelSnapContext()
* </pre>
*
* <br><b>Customizing the snap context</b>
* <pre>
* mode.snapContext = new LabelSnapContext({
*   collectNodeShapeSnapLines: true,
*   snapDistance: 3
* })
* </pre>
*
* <br><b>Obtaining the LabelSnapContext from the IInputModeContext</b>
* <pre>
* /** @type {LabelSnapContext} *‍/
* const snapContext = inputModeContext.lookup(LabelSnapContext.$class)
* if (snapContext !== null && snapContext.isInitializing) {
*   // do something
* }
* </pre>
*
* @see MoveLabelInputMode
* @see GraphEditorInputMode#snapContext
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext Online Documentation}
*/
export class LabelSnapContext {
/**
Initializes a new instance of the {@link LabelSnapContext} class.
@param [options] The parameters to pass.
@param [options.collectInitialLocationSnapLines] Whether this instance should collect snap lines for the initial position of a label. This option sets the {@link LabelSnapContext#collectInitialLocationSnapLines} property on the created object.
@param [options.collectNodeShapeSnapLines] Whether this instance should collect six snap lines, two through the center and four through the border sides of the label owner. This option sets the {@link LabelSnapContext#collectNodeShapeSnapLines} property on the created object.
@param [options.collectOwnNodeDistanceSnapLines] Whether this instance should collect four snap lines in parallel to the owner's borders at the initial label distance. This option sets the {@link LabelSnapContext#collectOwnNodeDistanceSnapLines} property on the created object.
@param [options.collectSameOwnerNodeDistanceSnapLines] Whether this instance should collect snap lines in parallel to the border of the label owner at the distance of other node labels of that owner. This option sets the {@link LabelSnapContext#collectSameOwnerNodeDistanceSnapLines} property on the created object.
@param [options.collectAllNodeDistanceSnapLines] Whether this instance should collect snap lines in parallel to the border of the label owner at the distance of other node labels in the graph. This option sets the {@link LabelSnapContext#collectAllNodeDistanceSnapLines} property on the created object.
@param [options.collectEdgePathSnapLines] Whether this instance should collect snap lines on the edge path. This option sets the {@link LabelSnapContext#collectEdgePathSnapLines} property on the created object.
@param [options.collectOwnEdgeDistanceSnapLines] Whether this instance should collect two snap lines in parallel to the edge path of the label owner at the initial distance of the edge label. This option sets the {@link LabelSnapContext#collectOwnEdgeDistanceSnapLines} property on the created object.
@param [options.collectSameOwnerEdgeDistanceSnapLines] Whether this instance should collect snap lines in parallel to the edge path of the label owner at the distances of all edge labels of that owner. This option sets the {@link LabelSnapContext#collectSameOwnerEdgeDistanceSnapLines} property on the created object.
@param [options.collectAllEdgeDistanceSnapLines] Whether this instance should collect snap lines in parallel to the edge path of the label owner at the distance of other edge labels in the graph. This option sets the {@link LabelSnapContext#collectAllEdgeDistanceSnapLines} property on the created object.
@param [options.snapNodeLabels] Whether this context will automatically snap the node labels to snap lines. This option sets the {@link LabelSnapContext#snapNodeLabels} property on the created object.
@param [options.snapEdgeLabels] Whether this context will automatically snap the edge labels to snap lines. This option sets the {@link LabelSnapContext#snapEdgeLabels} property on the created object.
@param [options.snapPortLabels] Whether this context will automatically snap the port labels to snap lines. This option sets the {@link LabelSnapContext#snapPortLabels} property on the created object.
@param [options.snapLineExtension] The amount by which snap lines that are induced by existing edge segments are being extended. This option sets the {@link LabelSnapContext#snapLineExtension} property on the created object.
@param [options.collectPortSnapLines] Whether this instance should collect snap lines to the center of the owner port. This option sets the {@link LabelSnapContext#collectPortSnapLines} property on the created object.
@param [options.visualizeSnapResults] A value indicating whether to visualize the snap results. This option sets the {@link SnapContext#visualizeSnapResults} property on the created object.
@param [options.snapDistance] The global snap distance, which is the maximum distance between the current mouse coordinates and the coordinates to which the mouse will snap. This option sets the {@link SnapContext#snapDistance} property on the created object.
@param [options.gridSnapType] The type of the grid snapping that should be performed. This option sets the {@link SnapContext#gridSnapType} property on the created object.
@param [options.gridSnapDistance] The GridSnapDistance. This option sets the {@link SnapContext#gridSnapDistance} property on the created object.
@param [options.enabled] A value indicating whether this {@link SnapContext} is enabled. This option sets the {@link SnapContext#enabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext Online Documentation}*/
constructor(options?: {
collectInitialLocationSnapLines?: boolean;
collectNodeShapeSnapLines?: boolean;
collectOwnNodeDistanceSnapLines?: boolean;
collectSameOwnerNodeDistanceSnapLines?: boolean;
collectAllNodeDistanceSnapLines?: boolean;
collectEdgePathSnapLines?: boolean;
collectOwnEdgeDistanceSnapLines?: boolean;
collectSameOwnerEdgeDistanceSnapLines?: boolean;
collectAllEdgeDistanceSnapLines?: boolean;
snapNodeLabels?: boolean;
snapEdgeLabels?: boolean;
snapPortLabels?: boolean;
snapLineExtension?: number;
collectPortSnapLines?: boolean;
visualizeSnapResults?: boolean;
snapDistance?: number;
gridSnapType?: GridSnapTypes | GridSnapTypesConvertible;
gridSnapDistance?: number;
enabled?: boolean;
});
/**
Adds the given <code>snapLine</code> to the {@link LabelSnapContext#snapLines snap line} collection of this context.
@param snapLine The snap line to add.
@see GraphSnapContext#additionalSnapLines
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23addSnapLine(SnapLine) Online Documentation}*/
addSnapLine(snapLine: SnapLine): void;
/**
Raises the {@link LabelSnapContext#addCollectSnapLinesListener CollectSnapLines} event.
@param evt The {@link CollectLabelSnapLineEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23onCollectSnapLines(CollectLabelSnapLineEventArgs) Online Documentation}*/
onCollectSnapLines(evt: CollectLabelSnapLineEventArgs): void;
/**
Gets or sets whether this instance should collect snap lines for the initial position of a label.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectInitialLocationSnapLines Online Documentation}*/
collectInitialLocationSnapLines: boolean;
/**
Gets or sets whether this instance should collect six snap lines, two through the center and four through the border
sides of the label owner.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectNodeShapeSnapLines Online Documentation}*/
collectNodeShapeSnapLines: boolean;
/**
Gets or sets whether this instance should collect four snap lines in parallel to the owner's borders at the initial
label distance.
Note that the label snaps to these snap lines with the side that is closer to the node's border. That way, snapping to
such a snap line preserves the perceived distance from the node border for both labels inside and outside the node
bounds.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectOwnNodeDistanceSnapLines Online Documentation}*/
collectOwnNodeDistanceSnapLines: boolean;
/**
Gets or sets whether this instance should collect snap lines in parallel to the border of the label owner at the
distance of other node labels of that owner.
Note that the label snaps to these snap lines with the side that is closer to the node's border. That way, snapping to
such a snap line preserves the perceived distance from the node border for both labels inside and outside the node
bounds.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectSameOwnerNodeDistanceSnapLines Online Documentation}*/
collectSameOwnerNodeDistanceSnapLines: boolean;
/**
Gets or sets whether this instance should collect snap lines in parallel to the border of the label owner at the
distance of other node labels in the graph.
Note that the label snaps to these snap lines with the side that is closer to the node's border. That way, snapping to
such a snap line preserves the perceived distance from the node border for both labels inside and outside the node
bounds.
The default is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectAllNodeDistanceSnapLines Online Documentation}*/
collectAllNodeDistanceSnapLines: boolean;
/**
Gets or sets whether this instance should collect snap lines on the edge path.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectEdgePathSnapLines Online Documentation}*/
collectEdgePathSnapLines: boolean;
/**
Gets or sets whether this instance should collect two snap lines in parallel to the edge path of the label owner at the
initial distance of the edge label.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectOwnEdgeDistanceSnapLines Online Documentation}*/
collectOwnEdgeDistanceSnapLines: boolean;
/**
Gets or sets whether this instance should collect snap lines in parallel to the edge path of the label owner at the
distances of all edge labels of that owner.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectSameOwnerEdgeDistanceSnapLines Online Documentation}*/
collectSameOwnerEdgeDistanceSnapLines: boolean;
/**
Gets or sets whether this instance should collect snap lines in parallel to the edge path of the label owner at the
distance of other edge labels in the graph.
The default is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectAllEdgeDistanceSnapLines Online Documentation}*/
collectAllEdgeDistanceSnapLines: boolean;
/**
Gets or sets whether this context will automatically snap the node labels to snap lines.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23snapNodeLabels Online Documentation}*/
snapNodeLabels: boolean;
/**
Gets or sets whether this context will automatically snap the edge labels to snap lines.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23snapEdgeLabels Online Documentation}*/
snapEdgeLabels: boolean;
/**
Gets or sets whether this context will automatically snap the port labels to snap lines.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23snapPortLabels Online Documentation}*/
snapPortLabels: boolean;
/**
Gets or sets the amount by which snap lines that are induced by existing edge segments are being extended.
The default is <code>40.0d</code>, this value will be used to lengthen the ends of the snap lines.
@default 40.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23snapLineExtension Online Documentation}*/
snapLineExtension: number;
/**
Gets or sets whether this instance should collect snap lines to the center of the owner port.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23collectPortSnapLines Online Documentation}*/
collectPortSnapLines: boolean;
/**
Gets the collection of the {@link SnapLine snap lines} that have been {@link LabelSnapContext#addSnapLine added} to this context.
This collection is only available if {@link SnapContext#isInitialized} is <code>true</code>.
@see LabelSnapContext#addSnapLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23snapLines Online Documentation}*/
readonly snapLines: IEnumerable<SnapLine>;
/**
Adds the given listener for the <code>CollectSnapLines</code> event that occurs every time this instance has been {@link SnapContext#dragInitialized initialized} to
collect {@link SnapLine}s.
Event handlers should add snap lines to the caller using the methods provided by
{@link CollectLabelSnapLineEventArgs}.
@param listener The listener to add.
@see LabelSnapContext#removeCollectSnapLinesListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23CollectSnapLines Online Documentation}*/
addCollectSnapLinesListener(listener: (sender: any, evt: CollectLabelSnapLineEventArgs) => void): void;
/**
Removes the given listener for the <code>CollectSnapLines</code> event that occurs every time this instance has been {@link SnapContext#dragInitialized initialized} to
collect {@link SnapLine}s.
Event handlers should add snap lines to the caller using the methods provided by
{@link CollectLabelSnapLineEventArgs}.
@param listener The listener to remove.
@see LabelSnapContext#addCollectSnapLinesListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContext%23CollectSnapLines Online Documentation}*/
removeCollectSnapLinesListener(listener: (sender: any, evt: CollectLabelSnapLineEventArgs) => void): void;
static $class: Class;
private $_LabelSnapContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelSnapContext;
}
export interface CollectLabelSnapLineEventArgs extends InputModeEventArgs {
}
/**
The event arguments used by {@link LabelSnapContext} to collect custom snap lines for each drag.
@see LabelSnapContext#addCollectSnapLinesListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectLabelSnapLineEventArgs Online Documentation}*/
export class CollectLabelSnapLineEventArgs {
/**
Creates a new instance of this class.
@param context The context this event is being used in.
@param snapLines The snap lines collection to add to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectLabelSnapLineEventArgs Online Documentation}*/
constructor(context: IInputModeContext, snapLines: ICollection<SnapLine> | ICollectionConvertible<SnapLine>);
/**
Adds the given snap line to the snap line collection of this class.
@param snapLine the snap line to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollectLabelSnapLineEventArgs%23addSnapLine(OrthogonalSnapLine) Online Documentation}*/
addSnapLine(snapLine: OrthogonalSnapLine): void;
static $class: Class;
private $_CollectLabelSnapLineEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CollectLabelSnapLineEventArgs;
}
export interface NavigationInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
cancel(): void;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} that can {@link NavigationInputMode#moveTo navigate} an {@link IGraph} displayed in a {@link NavigationInputMode#graphComponent}.
* This mode is {@link NavigationInputMode#exclusive} by default.
*
* Typically the <code>NavigationInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} or {@link GraphViewerInputMode}
* and can be retrieved from the {@link GraphInputMode#navigationInputMode} property.
*
* <pre>
* const navigationInputMode = mode.navigationInputMode
* </pre>
*
* The <code>NavigationInputMode</code> supports e.g. opening and closing group nodes. These actions can be configured in details:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable expanding and collapsing group nodes
* mode.navigationInputMode.allowCollapseGroup = true
* mode.navigationInputMode.allowExpandGroup = true
* // don't fit the content after expand/collapse
* mode.navigationInputMode.fitContentAfterGroupActions = false
* // expand/collapse group nodes in a way that their
* // top right corner stays at its current coordinates
* mode.navigationInputMode.autoGroupNodeAlignmentPolicy = NodeAlignmentPolicy.TOP_RIGHT
* </pre>
*
* It also reports when groups are opened or collapsed interactively by raising the following events:
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
* mode.navigationInputMode.addGroupExpandingListener((sender, args) => {
*   const node = args.item
*   // do something with the node before it will be expanded
* })
* mode.navigationInputMode.addGroupExpandedListener((sender, args) => {
*   const node = args.item
*   // do something with the node after it has been expanded
* })
* mode.navigationInputMode.addGroupCollapsingListener((sender, args) => {
*   /* same for collapse operations *‍/
* })
* mode.navigationInputMode.addGroupCollapsedListener((sender, args) => {
*   /* same for collapse operations *‍/
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode Online Documentation}
*/
export class NavigationInputMode {
/**
Initializes a new instance of the {@link NavigationInputMode} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode Online Documentation}*/
constructor();
/**
Adjusts the {@link CanvasComponent#contentRect} to encompass all elements.
This implementation will try to delegate to {@link GraphEditorInputMode#adjustContentRect} if it can find the {@link GraphEditorInputMode}
in the {@link NavigationInputMode#inputModeContext}, otherwise {@link CanvasComponent#updateContentRect} is called.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23adjustContentRect() Online Documentation}*/
adjustContentRect(): void;
/**
Adjust the group node location according to the value of {@link NavigationInputMode#autoGroupNodeAlignmentPolicy}.
@param collapse <code>true</code> iff the node has just been collapsed.
@param groupNode The node that has changed its state.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23adjustGroupNodeLocation(boolean%2CINode) Online Documentation}*/
adjustGroupNodeLocation(collapse: boolean, groupNode: INode): void;
/**
Clears the selection in the {@link NavigationInputMode#graphComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23clearSelection() Online Documentation}*/
clearSelection(): void;
/**
{@link IFoldingView#collapse Collapses} the given group node to hide the contents of the group node from this {@link IFoldingView}.
@param groupNode The group node to collapse.
@see FoldingManager
@see NavigationInputMode#allowCollapseGroup
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23collapseGroup(INode) Online Documentation}*/
collapseGroup(groupNode: INode): void;
/**
Performs {@link NavigationInputMode#collapseGroup} for all {@link IGraphSelection#selectedNodes}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23collapseSelection() Online Documentation}*/
collapseSelection(): void;
/**
This method allows for entering a group node so that the currently displayed {@link IFoldingView}'s {@link IFoldingView#localRoot} will be reset
to the provided node.
It is possible to both use a local group node that is part of the currently displayed {@link NavigationInputMode#graph} as the <code>node</code> argument, as well as
an item that belongs to the {@link FoldingManager#masterGraph} to allow for switching to group nodes which are currently not being displayed in this
view.
@param node The node that needs to be either part of the current graph, or part of the {@link FoldingManager#masterGraph}.
@see NavigationInputMode#shouldEnterGroup
@see NavigationInputMode#allowEnterGroup
@see FoldingManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23enterGroup(INode) Online Documentation}*/
enterGroup(node: INode | null): void;
/**
Performs {@link NavigationInputMode#enterGroup} for the first valid {@link IGraphSelection#selectedNodes}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23enterSelectedGroup() Online Documentation}*/
enterSelectedGroup(): void;
/**
Exits the current {@link IFoldingView#localRoot} of the currently displayed {@link IFoldingView view} and shows the contents of the parent container.
This method will also {@link GraphInputMode#clearSelection clear the selection} and {@link GraphInputMode#setSelected select} the exited group node.
@see FoldingManager
@see NavigationInputMode#allowExitGroup
@see NavigationInputMode#shouldExitGroup
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23exitGroup() Online Documentation}*/
exitGroup(): void;
/**
{@link IFoldingView#expand Expands} the given group node to show the contents of the collapsed group node in this {@link IFoldingView}.
@param groupNode The group node to expand
@see FoldingManager
@see NavigationInputMode#allowExpandGroup
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23expandGroup(INode) Online Documentation}*/
expandGroup(groupNode: INode): void;
/**
Performs {@link NavigationInputMode#expandGroup} for all {@link IGraphSelection#selectedNodes}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23expandSelection() Online Documentation}*/
expandSelection(): void;
/**
Moves the focus into the given direction, extending the selection to the new element.
@param direction The direction to move the focus and extend the selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23extendSelectionTo(MoveFocusDirection) Online Documentation}*/
extendSelectionTo(direction: MoveFocusDirection | MoveFocusDirectionStringValues): void;
/**
Determines the nearest item in the given <code>direction</code>.
The default implementation uses the euclidean distance from <code>startLocation</code> which is additionally weighted to prefer items
in the geometric <code>direction</code>. This method is called by the default implementation of {@link NavigationInputMode#findNextItem}
when {@link MoveFocusDirection} corresponds to an actual direction in the geometric sense. The method will be
called at most twice by the default implementation of {@link NavigationInputMode#findNextItem}. If it returns <code>null</code>
the first time with a smaller set of possible candidates, the default implementation will call it again with more
possible candidates.
Note that the default implementation ignores any {@link CanvasComponent#projection} and treats directions to be in world coordinates unless {@link NavigationInputMode#useViewCoordinates} is set.
@param context The current input mode context.
@param startLocation The reference location for the gesture.
@param direction The direction for the move action.
@param itemsToCheck The items that should be considered.
@returns The nearest item in <code>direction</code> or <code>null</code> if no suitable item can be determined.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23findNearestItem(IInputModeContext%2CPoint%2CMoveFocusDirection%2CIEnumerable%3CIModelItem%3E) Online Documentation}*/
findNearestItem(context: IInputModeContext, startLocation: Point, direction: MoveFocusDirection | MoveFocusDirectionStringValues, itemsToCheck: IEnumerable<IModelItem>): IModelItem | null;
/**
Determines the next item when a navigation command is executed.
@param context The current input mode context.
@param direction The direction of the command.
@returns The next item in <code>direction</code>, or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23findNextItem(IInputModeContext%2CMoveFocusDirection) Online Documentation}*/
findNextItem(context: IInputModeContext, direction: MoveFocusDirection | MoveFocusDirectionStringValues): IModelItem | null;
/**
Adjusts the viewport to fully encompass the {@link CanvasComponent#contentRect}.
This implementation will try to delegate to {@link ICommand#FIT_CONTENT} if it {@link ICommand#canExecute can be executed}, otherwise {@link CanvasComponent#fitContent}
is called.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23fitContent() Online Documentation}*/
fitContent(): void;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link NavigationInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Moves the focus into the given direction, not changing the current selection.
@param direction The direction to move the focus.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23moveFocusTo(MoveFocusDirection) Online Documentation}*/
moveFocusTo(direction: MoveFocusDirection | MoveFocusDirectionStringValues): void;
/**
Moves the focus into the given direction, setting the selection to the new element.
@param direction The direction to move the focus and selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23moveTo(MoveFocusDirection) Online Documentation}*/
moveTo(direction: MoveFocusDirection | MoveFocusDirectionStringValues): void;
/**
Called after {@link NavigationInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Raises the {@link NavigationInputMode#addGroupCollapsedListener GroupCollapsed} event.
@param evt The {@link ItemEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onGroupCollapsed(ItemEventArgs%3CINode%3E) Online Documentation}*/
onGroupCollapsed(evt: ItemEventArgs<INode>): void;
/**
Raises the {@link NavigationInputMode#addGroupCollapsingListener GroupCollapsing} event.
@param evt The {@link ItemEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onGroupCollapsing(ItemEventArgs%3CINode%3E) Online Documentation}*/
onGroupCollapsing(evt: ItemEventArgs<INode>): void;
/**
Raises the {@link NavigationInputMode#addGroupEnteredListener GroupEntered} event.
@param evt The {@link ItemEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onGroupEntered(ItemEventArgs%3CINode%3E) Online Documentation}*/
onGroupEntered(evt: ItemEventArgs<INode>): void;
/**
Raises the {@link NavigationInputMode#addGroupEnteringListener GroupEntering} event.
@param evt The {@link ItemEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onGroupEntering(ItemEventArgs%3CINode%3E) Online Documentation}*/
onGroupEntering(evt: ItemEventArgs<INode>): void;
/**
Raises the {@link NavigationInputMode#addGroupExitedListener GroupExited} event.
@param evt The {@link ItemEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onGroupExited(ItemEventArgs%3CINode%3E) Online Documentation}*/
onGroupExited(evt: ItemEventArgs<INode>): void;
/**
Raises the {@link NavigationInputMode#addGroupExitingListener GroupExiting} event.
@param evt The {@link ItemEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onGroupExiting(ItemEventArgs%3CINode%3E) Online Documentation}*/
onGroupExiting(evt: ItemEventArgs<INode>): void;
/**
Raises the {@link NavigationInputMode#addGroupExpandedListener GroupExpanded} event.
@param evt The {@link ItemEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onGroupExpanded(ItemEventArgs%3CINode%3E) Online Documentation}*/
onGroupExpanded(evt: ItemEventArgs<INode>): void;
/**
Raises the {@link NavigationInputMode#addGroupExpandingListener GroupExpanding} event.
@param evt The {@link ItemEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onGroupExpanding(ItemEventArgs%3CINode%3E) Online Documentation}*/
onGroupExpanding(evt: ItemEventArgs<INode>): void;
/**
Actually sets the current item property.
@param graphComponent The {@link NavigationInputMode#graphComponent} to set the current item of.
@param item The item to set.
@returns Whether the operation was actually performed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onSetCurrentItem(GraphComponent%2CIModelItem) Online Documentation}*/
onSetCurrentItem(graphComponent: GraphComponent, item: IModelItem | null): boolean;
/**
Called after {@link NavigationInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Selects the {@link GraphComponent#currentItem} current item.
This method delegates to {@link NavigationInputMode#selectItem}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23selectCurrentItem() Online Documentation}*/
selectCurrentItem(): void;
/**
Selects the given item.
@param graphComponent The {@link NavigationInputMode#graphComponent}.
@param item The item to select.
@param extendSelection if set to <code>true</code> the current selection is extended otherwise it is cleared beforehand.
@see NavigationInputMode#clearSelection
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23selectItem(GraphComponent%2CIModelItem%2Cboolean) Online Documentation}*/
selectItem(graphComponent: GraphComponent, item: IModelItem, extendSelection: boolean): void;
/**
Sets the "current" item to the given one.
The current item will not be set if {@link NavigationInputMode#graphComponent} is <code>null</code>.
@param item The item to set as the current item.
@see NavigationInputMode#onSetCurrentItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23setCurrentItem(IModelItem) Online Documentation}*/
setCurrentItem(item: IModelItem | null): void;
/**
Sets the {@link NavigationInputMode#graphComponent} to use for the various actions.
This will register or unregister the {@link ICommand}s for the control. This method is called in response to {@link NavigationInputMode#install}
and {@link NavigationInputMode#uninstall}.
@param graphComponent The control to use or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23setGraphComponent(GraphComponent) Online Documentation}*/
setGraphComponent(graphComponent: GraphComponent | null): void;
/**
Predicate method that decides whether it is allowed to {@link NavigationInputMode#collapseGroup collapse} the given group node.
This implementation returns the value of {@link NavigationInputMode#allowCollapseGroup}.
@param groupNode
@returns Whether to collapse the given group or not.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23shouldCollapseGroup(INode) Online Documentation}*/
shouldCollapseGroup(groupNode: INode): boolean;
/**
Predicate method that decides whether it is allowed to {@link NavigationInputMode#enterGroup enter} the specific group node.
This implementation yields the value of {@link NavigationInputMode#allowEnterGroup}.
@param node The group node to enter.
@returns Whether to enter the group or not.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23shouldEnterGroup(INode) Online Documentation}*/
shouldEnterGroup(node: INode | null): boolean;
/**
Predicate method that decides whether it is allowed to {@link NavigationInputMode#exitGroup exit} the current group node.
This implementation yields the value of {@link NavigationInputMode#allowExitGroup}.
@returns Whether to exit the current group or not.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23shouldExitGroup() Online Documentation}*/
shouldExitGroup(): boolean;
/**
Predicate method that decides whether it is allowed to {@link NavigationInputMode#expandGroup expand} the given group node.
This implementation yields the value of {@link NavigationInputMode#allowExpandGroup}.
@param node
@returns Whether to expand the given group or not.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23shouldExpandGroup(INode) Online Documentation}*/
shouldExpandGroup(node: INode): boolean;
/**
Determines whether a given model item should be navigated to.
This implementation uses the {@link NavigationInputMode#navigableItems} property to determine whether the <code>item</code> can be navigated to.
@param item The model item.
@returns Whether the item should be considered for navigational commands.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23shouldNavigateTo(IModelItem) Online Documentation}*/
shouldNavigateTo(item: IModelItem): boolean;
/**
Determines whether the input mode should try to request the mutex when a command is executed.
This resets the state of any other input mode, unless it currently possesses the mutex.
The value of this method is queried for {@link ICommand#COLLAPSE_GROUP}, {@link ICommand#EXPAND_GROUP},
{@link ICommand#TOGGLE_EXPANSION_STATE}, {@link ICommand#ENTER_GROUP} and {@link ICommand#EXIT_GROUP}.
@returns whether the input mode should request the mutex.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23shouldRequestMutex() Online Documentation}*/
shouldRequestMutex(): boolean;
/**
Determines whether a given model item should be selected.
This implementation uses the {@link NavigationInputMode#selectableItems} property to determine whether the <code>item</code> should be selected.
@param item The model item.
@returns Whether the item should be selected by the navigational commands.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23shouldSelect(IModelItem) Online Documentation}*/
shouldSelect(item: IModelItem): boolean;
/**
Predicate method that decides whether it is allowed to {@link NavigationInputMode#toggleExpansionState toggle} the collapsed state of the given group node.
This implementation uses {@link NavigationInputMode#shouldExpandGroup} and {@link NavigationInputMode#shouldCollapseGroup}
respectively.
@param node
@returns Whether to expand the given group or not.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23shouldToggleExpansionState(INode) Online Documentation}*/
shouldToggleExpansionState(node: INode): boolean;
/**
Toggles the {@link IFoldingView#isExpanded expanded/collapsed} state for the given group node in the {@link IFoldingView}.
@param groupNode The group node to toggle the state for.
@see FoldingManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23toggleExpansionState(INode) Online Documentation}*/
toggleExpansionState(groupNode: INode): void;
/**
Overridden to only return <code>true</code> if this instance does not currently {@link ConcurrencyController#hasMutex have the input mutex}.
@returns <code>true</code> iff this instance does not {@link ConcurrencyController#hasMutex own the mutex}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23tryStop() Online Documentation}*/
tryStop(): boolean;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link NavigationInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link NavigationInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the NavigationInputMode on its parent input mode</b>
* <pre>
* mode.navigationInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link NavigationInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23controller Online Documentation}*/
controller: ConcurrencyController | null;
/**
Gets or sets the item types that can be navigated to by this mode.
@see NavigationInputMode#shouldNavigateTo
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23navigableItems Online Documentation}*/
navigableItems: GraphItemTypes;
/**
Gets the types of the items that should be selectable by this instance.
The selectable items. The default value is {@link GraphItemTypes#NODE}.
@default GraphItemTypes.NODE
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23selectableItems Online Documentation}*/
selectableItems: GraphItemTypes;
/**
* Gets or sets the the location that should be kept fixed if toggling a group node state.
* Default value is {@link NodeAlignmentPolicy#NONE}
*
* This property can be used to synchronize the fixed location with the button location of a
* {@link CollapsibleNodeStyleDecorator}:
*
* <pre>
* navigationMode.autoGroupNodeAlignmentPolicy = NodeAlignmentPolicy.TOP_RIGHT
* graph.groupNodeDefaults.style = new CollapsibleNodeStyleDecorator({
*   buttonPlacement: InteriorLabelModel.NORTH_EAST
* })
* </pre>
*
* @see NavigationInputMode#expandGroup
* @see NavigationInputMode#collapseGroup
* @default NodeAlignmentPolicy.NONE
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23autoGroupNodeAlignmentPolicy Online Documentation}
*/
autoGroupNodeAlignmentPolicy: NodeAlignmentPolicy;
/**
Gets or sets the {@link NavigationInputMode#graphComponent} this mode acts on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23graphComponent Online Documentation}*/
graphComponent: GraphComponent | null;
/**
Gets the list of commands that are available in this instance.
By default, all supported commands are available
Removing commands from this collection also removes the command bindings registered by this instance.
Add supported commands to make them available in this instance.
Supported commands are
<ul>
<li>{@link ICommand#MOVE_LEFT}
<li>{@link ICommand#MOVE_RIGHT}
<li>{@link ICommand#MOVE_UP}
<li>{@link ICommand#MOVE_DOWN}
<li>{@link ICommand#MOVE_TO_PAGE_UP}
<li>{@link ICommand#MOVE_TO_PAGE_DOWN}
<li>{@link ICommand#MOVE_FOCUS_BACK}
<li>{@link ICommand#MOVE_FOCUS_FORWARD}
<li>{@link ICommand#MOVE_FOCUS_UP}
<li>{@link ICommand#MOVE_FOCUS_DOWN}
<li>{@link ICommand#MOVE_FOCUS_PAGE_UP}
<li>{@link ICommand#MOVE_FOCUS_PAGE_DOWN}
<li>{@link ICommand#EXTEND_SELECTION_LEFT}
<li>{@link ICommand#EXTEND_SELECTION_RIGHT}
<li>{@link ICommand#EXTEND_SELECTION_UP}
<li>{@link ICommand#EXTEND_SELECTION_DOWN}
<li>{@link ICommand#SELECT_TO_PAGE_UP}
<li>{@link ICommand#SELECT_TO_PAGE_DOWN}
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23availableCommands Online Documentation}*/
readonly availableCommands: ICollection<ICommand>;
/**
Gets or sets a value indicating whether to use the {@link GraphComponent#currentItem} as a fallback for the commands if no item is provided in the
parameter and the current selection is empty.
This applies to the following commands:
<ul>
<li>{@link ICommand#ENTER_GROUP}
<li>{@link ICommand#EXPAND_GROUP}
<li>{@link ICommand#COLLAPSE_GROUP}
<li>{@link ICommand#TOGGLE_EXPANSION_STATE}
</ul>
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23useCurrentItemForCommands Online Documentation}*/
useCurrentItemForCommands: boolean;
/**
Gets or sets the {@link IGraphSelection} this mode operates on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23graphSelection Online Documentation}*/
readonly graphSelection: IGraphSelection | null;
/**
Gets or sets the graph this mode operates on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23graph Online Documentation}*/
readonly graph: IGraph | null;
/**
* Gets or sets a value that determines whether it is allowed to {@link NavigationInputMode#enterGroup enter group nodes} via the
* {@link ICommand#ENTER_GROUP}.
* The default value is <code>true</code>.
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable expanding and collapsing group nodes
* mode.navigationInputMode.allowCollapseGroup = true
* mode.navigationInputMode.allowExpandGroup = true
* // don't fit the content after expand/collapse
* mode.navigationInputMode.fitContentAfterGroupActions = false
* // expand/collapse group nodes in a way that their
* // top right corner stays at its current coordinates
* mode.navigationInputMode.autoGroupNodeAlignmentPolicy = NodeAlignmentPolicy.TOP_RIGHT
* </pre>
*
* @see NavigationInputMode#shouldEnterGroup
* @see FoldingManager
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23allowEnterGroup Online Documentation}
*/
allowEnterGroup: boolean;
/**
* Gets or sets a value that determines whether it is allowed to collapse group nodes via the
* {@link ICommand#COLLAPSE_GROUP}.
* The default value is <code>true</code>.
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable expanding and collapsing group nodes
* mode.navigationInputMode.allowCollapseGroup = true
* mode.navigationInputMode.allowExpandGroup = true
* // don't fit the content after expand/collapse
* mode.navigationInputMode.fitContentAfterGroupActions = false
* // expand/collapse group nodes in a way that their
* // top right corner stays at its current coordinates
* mode.navigationInputMode.autoGroupNodeAlignmentPolicy = NodeAlignmentPolicy.TOP_RIGHT
* </pre>
*
* @see NavigationInputMode#collapseGroup
* @see NavigationInputMode#shouldCollapseGroup
* @see FoldingManager
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23allowCollapseGroup Online Documentation}
*/
allowCollapseGroup: boolean;
/**
* Gets or sets a value that determines whether it is allowed to {@link NavigationInputMode#expandGroup expand group nodes} via the
* {@link ICommand#EXPAND_GROUP}.
* The default value is <code>true</code>.
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable expanding and collapsing group nodes
* mode.navigationInputMode.allowCollapseGroup = true
* mode.navigationInputMode.allowExpandGroup = true
* // don't fit the content after expand/collapse
* mode.navigationInputMode.fitContentAfterGroupActions = false
* // expand/collapse group nodes in a way that their
* // top right corner stays at its current coordinates
* mode.navigationInputMode.autoGroupNodeAlignmentPolicy = NodeAlignmentPolicy.TOP_RIGHT
* </pre>
*
* @see NavigationInputMode#shouldExpandGroup
* @see FoldingManager
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23allowExpandGroup Online Documentation}
*/
allowExpandGroup: boolean;
/**
* Gets or sets a value that determines whether it is allowed to {@link NavigationInputMode#exitGroup exit the current group node} via the
* {@link ICommand#EXIT_GROUP}.
* The default value is <code>true</code>.
*
* <pre>
* // mode is either an instance of GraphEditorInputMode or GraphViewerInputMode
*
* // enable expanding and collapsing group nodes
* mode.navigationInputMode.allowCollapseGroup = true
* mode.navigationInputMode.allowExpandGroup = true
* // don't fit the content after expand/collapse
* mode.navigationInputMode.fitContentAfterGroupActions = false
* // expand/collapse group nodes in a way that their
* // top right corner stays at its current coordinates
* mode.navigationInputMode.autoGroupNodeAlignmentPolicy = NodeAlignmentPolicy.TOP_RIGHT
* </pre>
*
* @see NavigationInputMode#shouldExitGroup
* @see FoldingManager
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23allowExitGroup Online Documentation}
*/
allowExitGroup: boolean;
/**
Gets or sets a value that determines whether {@link CanvasComponent#fitContent} or the {@link ICommand#FIT_CONTENT}
should be triggered after a group navigation action.
Note that {@link NavigationInputMode#collapseGroup} and {@link NavigationInputMode#expandGroup} only fit the
content when {@link NavigationInputMode#autoGroupNodeAlignmentPolicy} is {@link NodeAlignmentPolicy#NONE}.
The default value is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23fitContentAfterGroupActions Online Documentation}*/
fitContentAfterGroupActions: boolean;
/**
Gets or sets a value determining whether a node that has been focused or selected with a keyboard gesture should be made
visible by adjusting the viewport.
The default value is <code>true</code>.
@see CanvasComponent#ensureVisible
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23scrollToSelection Online Documentation}*/
scrollToSelection: boolean;
/**
Adds the given listener for the <code>GroupCollapsing</code> event that occurs before a group will be {@link NavigationInputMode#collapseGroup Collapsed}.
@param listener The listener to add.
@see NavigationInputMode#removeGroupCollapsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupCollapsing Online Documentation}*/
addGroupCollapsingListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupCollapsing</code> event that occurs before a group will be {@link NavigationInputMode#collapseGroup Collapsed}.
@param listener The listener to remove.
@see NavigationInputMode#addGroupCollapsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupCollapsing Online Documentation}*/
removeGroupCollapsingListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>GroupCollapsed</code> event that occurs whenever a group has been {@link NavigationInputMode#collapseGroup Collapsed}.
@param listener The listener to add.
@see NavigationInputMode#removeGroupCollapsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupCollapsed Online Documentation}*/
addGroupCollapsedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupCollapsed</code> event that occurs whenever a group has been {@link NavigationInputMode#collapseGroup Collapsed}.
@param listener The listener to remove.
@see NavigationInputMode#addGroupCollapsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupCollapsed Online Documentation}*/
removeGroupCollapsedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>GroupExpanding</code> event that occurs before a group will be {@link NavigationInputMode#expandGroup Expanded}.
@param listener The listener to add.
@see NavigationInputMode#removeGroupExpandingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupExpanding Online Documentation}*/
addGroupExpandingListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupExpanding</code> event that occurs before a group will be {@link NavigationInputMode#expandGroup Expanded}.
@param listener The listener to remove.
@see NavigationInputMode#addGroupExpandingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupExpanding Online Documentation}*/
removeGroupExpandingListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>GroupExpanded</code> event that occurs whenever a group has been {@link NavigationInputMode#expandGroup Expanded}.
@param listener The listener to add.
@see NavigationInputMode#removeGroupExpandedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupExpanded Online Documentation}*/
addGroupExpandedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupExpanded</code> event that occurs whenever a group has been {@link NavigationInputMode#expandGroup Expanded}.
@param listener The listener to remove.
@see NavigationInputMode#addGroupExpandedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupExpanded Online Documentation}*/
removeGroupExpandedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>GroupEntering</code> event that occurs before a group will be {@link NavigationInputMode#enterGroup Entered}.
@param listener The listener to add.
@see NavigationInputMode#removeGroupEnteringListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupEntering Online Documentation}*/
addGroupEnteringListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupEntering</code> event that occurs before a group will be {@link NavigationInputMode#enterGroup Entered}.
@param listener The listener to remove.
@see NavigationInputMode#addGroupEnteringListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupEntering Online Documentation}*/
removeGroupEnteringListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>GroupEntered</code> event that occurs whenever a group has been {@link NavigationInputMode#enterGroup Entered}.
@param listener The listener to add.
@see NavigationInputMode#removeGroupEnteredListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupEntered Online Documentation}*/
addGroupEnteredListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupEntered</code> event that occurs whenever a group has been {@link NavigationInputMode#enterGroup Entered}.
@param listener The listener to remove.
@see NavigationInputMode#addGroupEnteredListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupEntered Online Documentation}*/
removeGroupEnteredListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>GroupExiting</code> event that occurs before a group will be {@link NavigationInputMode#exitGroup Exited}.
@param listener The listener to add.
@see NavigationInputMode#removeGroupExitingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupExiting Online Documentation}*/
addGroupExitingListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupExiting</code> event that occurs before a group will be {@link NavigationInputMode#exitGroup Exited}.
@param listener The listener to remove.
@see NavigationInputMode#addGroupExitingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupExiting Online Documentation}*/
removeGroupExitingListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>GroupExited</code> event that occurs whenever a group has been {@link NavigationInputMode#exitGroup Exited}.
@param listener The listener to add.
@see NavigationInputMode#removeGroupExitedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupExited Online Documentation}*/
addGroupExitedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupExited</code> event that occurs whenever a group has been {@link NavigationInputMode#exitGroup Exited}.
@param listener The listener to remove.
@see NavigationInputMode#addGroupExitedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23GroupExited Online Documentation}*/
removeGroupExitedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Gets or sets a value indicating whether navigating the graph with the arrow keys interprets the direction in view
coordinates.
If set to <code>true</code>, the direction is interpreted in view coordinates.
Default is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/NavigationInputMode%23useViewCoordinates Online Documentation}*/
useViewCoordinates: boolean;
static $class: Class;
private $_NavigationInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NavigationInputMode;
}
/**
Enumeration used by {@link NavigationInputMode} to determine the direction of the search for items to navigate to.
@see NavigationInputMode
@see NavigationInputMode#moveTo
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveFocusDirection Online Documentation}*/
export enum MoveFocusDirection {
/**
No direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveFocusDirection%23NONE Online Documentation}*/
NONE,
/**
In the direction of the negative x Axis.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveFocusDirection%23LEFT Online Documentation}*/
LEFT,
/**
In the direction of the positive x Axis.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveFocusDirection%23RIGHT Online Documentation}*/
RIGHT,
/**
In the direction of the negative y Axis.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveFocusDirection%23UP Online Documentation}*/
UP,
/**
In the direction of the positive y Axis.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveFocusDirection%23DOWN Online Documentation}*/
DOWN,
/**
In the direction of the {@link IGraph#setParent parent} in the grouped graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveFocusDirection%23PARENT Online Documentation}*/
PARENT,
/**
In the direction of the {@link IGraph#getChildren children} in the grouped graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveFocusDirection%23CHILD Online Documentation}*/
CHILD
}
/** Automatic conversion to {@link MoveFocusDirection}. */
type MoveFocusDirectionStringValues = "none" | "left" | "right" | "up" | "down" | "parent" | "child";
/**
Enumeration used by {@link NavigationInputMode} to determine which point (if any) of a group or folder node should
be kept fixed when toggling the group node state.
@see NavigationInputMode
@see NavigationInputMode#expandGroup
@see NavigationInputMode#collapseGroup
@see NavigationInputMode#toggleExpansionState
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy Online Documentation}*/
export enum NodeAlignmentPolicy {
/**
Don't fix any specific point, only use the layout from the target state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23NONE Online Documentation}*/
NONE,
/**
Fix the center point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23CENTER Online Documentation}*/
CENTER,
/**
Fix the upper left corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23TOP_LEFT Online Documentation}*/
TOP_LEFT,
/**
Fix the lower left corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23BOTTOM_LEFT Online Documentation}*/
BOTTOM_LEFT,
/**
Fix the upper right corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23TOP_RIGHT Online Documentation}*/
TOP_RIGHT,
/**
Fix the lower right corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23BOTTOM_RIGHT Online Documentation}*/
BOTTOM_RIGHT,
/**
Fix the uppermost center.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23TOP_CENTER Online Documentation}*/
TOP_CENTER,
/**
Fix the lowermost center.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23BOTTOM_CENTER Online Documentation}*/
BOTTOM_CENTER,
/**
Fix the left center.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23CENTER_LEFT Online Documentation}*/
CENTER_LEFT,
/**
Fix the right center.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignmentPolicy%23CENTER_RIGHT Online Documentation}*/
CENTER_RIGHT
}
/** Automatic conversion to {@link NodeAlignmentPolicy}. */
type NodeAlignmentPolicyStringValues = "none" | "center" | "top-left" | "bottom-left" | "top-right" | "bottom-right" | "top-center" | "bottom-center" | "center-left" | "center-right";
/**
Enumeration defining different policies to enable or disable some aspects of creation or editing of orthogonal edges.
@see CreateEdgeInputMode#orthogonalEdgeCreation
@see GraphEditorInputMode#orthogonalBendRemoval
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingPolicy Online Documentation}*/
export enum OrthogonalEdgeEditingPolicy {
/**
Enables orthogonal edge creation or editing for a specific sub-aspect automatically if an {@link OrthogonalEdgeEditingContext#enabled} {@link OrthogonalEdgeEditingContext}
instance is set in the {@link GraphEditorInputMode#orthogonalEdgeEditingContext parent input mode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingPolicy%23AUTO Online Documentation}*/
AUTO,
/**
Enables orthogonal edge creation or editing for a specific sub-aspect of edge creation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingPolicy%23ALWAYS Online Documentation}*/
ALWAYS,
/**
Disables orthogonal edge creation or editing for a specific sub-aspect of edge creation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingPolicy%23NEVER Online Documentation}*/
NEVER
}
/** Automatic conversion to {@link OrthogonalEdgeEditingPolicy}. */
type OrthogonalEdgeEditingPolicyStringValues = "auto" | "always" | "never";
/**
Enumeration for possible orientations of a segment in a (partially) orthogonal edge routing.
This class is mainly used by {@link IOrthogonalEdgeHelper} and {@link OrthogonalEdgeEditingContext}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentOrientation Online Documentation}*/
export enum SegmentOrientation {
/**
The segment has a horizontal orientation, which means that the end points should be moved vertically in synch so that
the orientation is not destroyed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentOrientation%23HORIZONTAL Online Documentation}*/
HORIZONTAL,
/**
The segment has a vertical orientation, which means that the end points should be moved horizontally in synch so that
the orientation is not destroyed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentOrientation%23VERTICAL Online Documentation}*/
VERTICAL,
/**
The segment does not need to be treated especially, both endpoints can be moved independently from each other.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentOrientation%23NON_ORTHOGONAL Online Documentation}*/
NON_ORTHOGONAL
}
/** Automatic conversion to {@link SegmentOrientation}. */
type SegmentOrientationStringValues = "horizontal" | "vertical" | "non-orthogonal";
export interface SnapLine extends YObject {
}
/**
A snap line is a line segment in the world coordinate system to which other items (lines or points) snap during
interactive movements.
{@link GraphSnapContext} and {@link LabelSnapContext} make use of this class to interactively snap moving
elements to snap lines.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine Online Documentation}*/
export class SnapLine {
/**
Creates a new snap line using the provided attributes.
@param snapType The type of the line that describes how other items will snap to this line. This is one of
{@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM}, {@link SnapLineSnapTypes#LEFT},
{@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@param visualizationType The type of the visual representation of this snap line.
@param coordinates The coordinates of the center point of the snap line.
@param from The first location that delimits this snap line.
@param to The second location that delimits this snap line.
@param tag A tag that is associated with this snap line - see {@link SnapLine#tag} for a typical use of this value.
@param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine Online Documentation}*/
constructor(snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, visualizationType: SnapLineVisualizationType | SnapLineVisualizationTypeStringValues, coordinates: Point | PointConvertible, from: Point | PointConvertible, to: Point | PointConvertible, tag: any | null, weight: number);
/**
Resource key which determines the {@link Stroke} used for the snap line visualization.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine%23SNAP_LINE_STROKE_KEY Online Documentation}*/
static SNAP_LINE_STROKE_KEY: string;
/**
Gets the type of the line that describes how other items will snap to this line.
his is one of {@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM},
{@link SnapLineSnapTypes#LEFT}, {@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine%23snapType Online Documentation}*/
readonly snapType: SnapLineSnapTypes;
/**
Gets the coordinates of the center point of the snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine%23coordinates Online Documentation}*/
readonly coordinates: Point;
/**
Gets the tag that is associated with this snap line.
The tag is most commonly used for the {@link SnapResult#tag snap result tag}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine%23tag Online Documentation}*/
readonly tag: any | null;
/**
Gets the weight/importance of this snap line.
Greater values indicate greater importance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine%23weight Online Documentation}*/
readonly weight: number;
/**
Gets or sets the first location that delimits this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine%23from Online Documentation}*/
from: Point;
/**
Gets or sets the second location that delimits this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine%23to Online Documentation}*/
to: Point;
/**
Gets the visualization type which determines the visual appearance of the snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLine%23visualizationType Online Documentation}*/
readonly visualizationType: SnapLineVisualizationType;
static $class: Class;
private $_SnapLine: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SnapLine;
}
export interface OrthogonalSnapLine extends SnapLine {
}
/**
An orthogonal snap line, in other words, one that is parallel to the x or y axis.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSnapLine Online Documentation}*/
export class OrthogonalSnapLine {
/**
Creates a new snap line using the provided attributes.
@param orientation The orientation of the snap line. This is one of {@link SnapLineOrientation#HORIZONTAL} or
{@link SnapLineOrientation#VERTICAL}.
@param snapType The type of the line that describes how other items will snap to this line. This is one of
{@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM}, {@link SnapLineSnapTypes#LEFT},
{@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@param visualizationType The type of the visual representation of this snap line.
@param coordinates The coordinates of the center point of the snap line.
@param from The smaller value of the coordinate that delimits this snap line.
@param to The greater value of the coordinate that delimits this snap line.
@param tag A tag that is associated with this snap line - see {@link SnapLine#tag} for a typical use of this value.
@param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSnapLine%23OrthogonalSnapLine(SnapLineOrientation%2CSnapLineSnapTypes%2CSnapLineVisualizationType%2CPoint%2Cnumber%2Cnumber%2CObject%2Cnumber) Online Documentation}*/
constructor(orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, visualizationType: SnapLineVisualizationType | SnapLineVisualizationTypeStringValues, coordinates: Point | PointConvertible, from: number, to: number, tag: any | null, weight: number);
/**
Creates a new snap line using the provided attributes.
@param orientation The orientation of the snap line. This is one of {@link SnapLineOrientation#HORIZONTAL} or
{@link SnapLineOrientation#VERTICAL}.
@param snapType The type of the line that describes how other items will snap to this line. This is one of
{@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM}, {@link SnapLineSnapTypes#LEFT},
{@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@param visualizationType The type of the visual representation of this snap line.
@param coordinates The coordinates of the center point of the snap line.
@param tag A tag that is associated with this snap line - see {@link SnapLine#tag} for a typical use of this value.
@param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSnapLine%23OrthogonalSnapLine(SnapLineOrientation%2CSnapLineSnapTypes%2CSnapLineVisualizationType%2CPoint%2CObject%2Cnumber) Online Documentation}*/
constructor(orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, visualizationType: SnapLineVisualizationType | SnapLineVisualizationTypeStringValues, coordinates: Point | PointConvertible, tag: any | null, weight: number);
/**
Gets the orientation of the snap line.
This is one of {@link SnapLineOrientation#HORIZONTAL} or {@link SnapLineOrientation#VERTICAL}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSnapLine%23orientation Online Documentation}*/
readonly orientation: SnapLineOrientation;
static $class: Class;
private $_OrthogonalSnapLine: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalSnapLine;
}
export interface NodeBasedSnapLine extends OrthogonalSnapLine {
}
/**
A specialized {@link OrthogonalSnapLine} that stores a {@link NodeBasedSnapLine#node} as additional information that can be used for the
visualization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeBasedSnapLine Online Documentation}*/
export class NodeBasedSnapLine {
/**
Initializes a new instance of the {@link NodeBasedSnapLine} class.
@param node The node to assign to {@link NodeBasedSnapLine#node}.
@param orientation The orientation of the snap line. This is one of {@link SnapLineOrientation#HORIZONTAL} or
{@link SnapLineOrientation#VERTICAL}.
@param snapType The type of the line that describes how other items will snap to this line. This is one of
{@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM}, {@link SnapLineSnapTypes#LEFT},
{@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@param visualizationType The type of the visual representation of this snap line.
@param coordinates The coordinates of the center point of the snap line.
@param tag A tag that is associated with this snap line - see {@link SnapLine#tag} for a typical use of this value.
@param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
will be used.
@param from The smaller value of the coordinate that delimits this snap line.
@param to The greater value of the coordinate that delimits this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeBasedSnapLine%23NodeBasedSnapLine(INode%2CSnapLineOrientation%2CSnapLineSnapTypes%2CSnapLineVisualizationType%2CPoint%2Cnumber%2Cnumber%2CObject%2Cnumber) Online Documentation}*/
constructor(node: INode, orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, visualizationType: SnapLineVisualizationType | SnapLineVisualizationTypeStringValues, coordinates: Point | PointConvertible, from: number, to: number, tag: any | null, weight: number);
/**
Initializes a new instance of the {@link NodeBasedSnapLine} class.
@param node The node to assign to {@link NodeBasedSnapLine#node}.
@param orientation The orientation of the snap line. This is one of {@link SnapLineOrientation#HORIZONTAL} or
{@link SnapLineOrientation#VERTICAL}.
@param snapType The type of the line that describes how other items will snap to this line. This is one of
{@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM}, {@link SnapLineSnapTypes#LEFT},
{@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@param visualizationType The type of the visual representation of this snap line.
@param coordinates The coordinates of the center point of the snap line.
@param tag A tag that is associated with this snap line - see {@link SnapLine#tag} for a typical use of this value.
@param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeBasedSnapLine%23NodeBasedSnapLine(INode%2CSnapLineOrientation%2CSnapLineSnapTypes%2CSnapLineVisualizationType%2CPoint%2CObject%2Cnumber) Online Documentation}*/
constructor(node: INode, orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, visualizationType: SnapLineVisualizationType | SnapLineVisualizationTypeStringValues, coordinates: Point | PointConvertible, tag: any | null, weight: number);
/**
Gets the node that is associated with this snap line.
Usually this is the instance that induced this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeBasedSnapLine%23node Online Documentation}*/
readonly node: INode;
static $class: Class;
private $_NodeBasedSnapLine: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeBasedSnapLine;
}
export interface NodePairBasedSnapLine extends OrthogonalSnapLine {
}
/**
A specialized {@link OrthogonalSnapLine} that stores two {@link INode nodes} as additional information that can be used for the
visualization.
@see NodePairBasedSnapLine#firstNode
@see NodePairBasedSnapLine#secondNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePairBasedSnapLine Online Documentation}*/
export class NodePairBasedSnapLine {
/**
Initializes a new instance of the {@link NodePairBasedSnapLine} class.
@param firstNode The first node to assign to {@link NodePairBasedSnapLine#firstNode}.
@param secondNode The second node to assign to {@link NodePairBasedSnapLine#secondNode}.
@param orientation The orientation of the snap line. This is one of {@link SnapLineOrientation#HORIZONTAL} or
{@link SnapLineOrientation#VERTICAL}.
@param snapType The type of the line that describes how other items will snap to this line. This is one of
{@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM}, {@link SnapLineSnapTypes#LEFT},
{@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@param visualizationType The type of the visual representation of this snap line.
@param coordinates The coordinates of the center point of the snap line.
@param tag A tag that is associated with this snap line - see {@link SnapLine#tag} for a typical use of this value.
@param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
will be used.
@param from The smaller value of the coordinate that delimits this snap line.
@param to The greater value of the coordinate that delimits this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePairBasedSnapLine%23NodePairBasedSnapLine(INode%2CINode%2CSnapLineOrientation%2CSnapLineSnapTypes%2CSnapLineVisualizationType%2CPoint%2Cnumber%2Cnumber%2CObject%2Cnumber) Online Documentation}*/
constructor(firstNode: INode, secondNode: INode, orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, visualizationType: SnapLineVisualizationType | SnapLineVisualizationTypeStringValues, coordinates: Point | PointConvertible, from: number, to: number, tag: any | null, weight: number);
/**
Initializes a new instance of the {@link NodePairBasedSnapLine} class.
@param firstNode The first node to assign to {@link NodePairBasedSnapLine#firstNode}.
@param secondNode The second node to assign to {@link NodePairBasedSnapLine#secondNode}.
@param orientation The orientation of the snap line. This is one of {@link SnapLineOrientation#HORIZONTAL} or
{@link SnapLineOrientation#VERTICAL}.
@param snapType The type of the line that describes how other items will snap to this line. This is one of
{@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM}, {@link SnapLineSnapTypes#LEFT},
{@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@param visualizationType The type of the visual representation of this snap line.
@param coordinates The coordinates of the center point of the snap line.
@param tag A tag that is associated with this snap line - see {@link SnapLine#tag} for a typical use of this value.
@param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePairBasedSnapLine%23NodePairBasedSnapLine(INode%2CINode%2CSnapLineOrientation%2CSnapLineSnapTypes%2CSnapLineVisualizationType%2CPoint%2CObject%2Cnumber) Online Documentation}*/
constructor(firstNode: INode, secondNode: INode, orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, visualizationType: SnapLineVisualizationType | SnapLineVisualizationTypeStringValues, coordinates: Point | PointConvertible, tag: any | null, weight: number);
/**
Gets the first node that is associated with this snap line.
Usually this is one of the instances that induced this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePairBasedSnapLine%23firstNode Online Documentation}*/
readonly firstNode: INode;
/**
Gets the second node that is associated with this snap line.
Usually this is one of the instances that induced this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePairBasedSnapLine%23secondNode Online Documentation}*/
readonly secondNode: INode;
static $class: Class;
private $_NodePairBasedSnapLine: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodePairBasedSnapLine;
}
export interface PointBasedSnapLine extends OrthogonalSnapLine {
}
/**
A specialized {@link OrthogonalSnapLine} that stores a {@link PointBasedSnapLine#point} instance as additional information that can be used for the
visualization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointBasedSnapLine Online Documentation}*/
export class PointBasedSnapLine {
/**
Initializes a new instance of the {@link PointBasedSnapLine} class.
@param point The point to assign to {@link PointBasedSnapLine#point}.
@param orientation The orientation of the snap line. This is one of {@link SnapLineOrientation#HORIZONTAL} or
{@link SnapLineOrientation#VERTICAL}.
@param snapType The type of the line that describes how other items will snap to this line. This is one of
{@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM}, {@link SnapLineSnapTypes#LEFT},
{@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@param visualizationType The type of the visual representation of this snap line.
@param coordinates The coordinates of the center point of the snap line.
@param tag A tag that is associated with this snap line - see {@link SnapLine#tag} for a typical use of this value.
@param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
will be used.
@param from The smaller value of the coordinate that delimits this snap line.
@param to The greater value of the coordinate that delimits this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointBasedSnapLine%23PointBasedSnapLine(IPoint%2CSnapLineOrientation%2CSnapLineSnapTypes%2CSnapLineVisualizationType%2CPoint%2Cnumber%2Cnumber%2CObject%2Cnumber) Online Documentation}*/
constructor(point: IPoint | IPointConvertible, orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, visualizationType: SnapLineVisualizationType | SnapLineVisualizationTypeStringValues, coordinates: Point | PointConvertible, from: number, to: number, tag: any | null, weight: number);
/**
Initializes a new instance of the {@link PointBasedSnapLine} class.
@param point The point to assign to {@link PointBasedSnapLine#point}.
@param orientation The orientation of the snap line. This is one of {@link SnapLineOrientation#HORIZONTAL} or
{@link SnapLineOrientation#VERTICAL}.
@param snapType The type of the line that describes how other items will snap to this line. This is one of
{@link SnapLineSnapTypes#TOP}, {@link SnapLineSnapTypes#BOTTOM}, {@link SnapLineSnapTypes#LEFT},
{@link SnapLineSnapTypes#RIGHT}, or {@link SnapLineSnapTypes#CENTER}.
@param visualizationType The type of the visual representation of this snap line.
@param coordinates The coordinates of the center point of the snap line.
@param tag A tag that is associated with this snap line - see {@link SnapLine#tag} for a typical use of this value.
@param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointBasedSnapLine%23PointBasedSnapLine(IPoint%2CSnapLineOrientation%2CSnapLineSnapTypes%2CSnapLineVisualizationType%2CPoint%2CObject%2Cnumber) Online Documentation}*/
constructor(point: IPoint | IPointConvertible, orientation: SnapLineOrientation | SnapLineOrientationStringValues, snapType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, visualizationType: SnapLineVisualizationType | SnapLineVisualizationTypeStringValues, coordinates: Point | PointConvertible, tag: any | null, weight: number);
/**
Gets the point that is associated with this snap line.
Usually this is the instance that induced this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PointBasedSnapLine%23point Online Documentation}*/
readonly point: IPoint;
static $class: Class;
private $_PointBasedSnapLine: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PointBasedSnapLine;
}
export interface EdgeSegmentSnapLine extends SnapLine {
}
/**
A specialized {@link SnapLine} that is parallel to an edge segment and stores that edge.
In addition to the line itself, this class has three special nape locations the represent the source, center and end
location of the related edge segment. All snap lines of this type have the {@link SnapLine#snapType} {@link SnapLineSnapTypes#CENTER} and
the {@link SnapLine#visualizationType} {@link SnapLineVisualizationType#EDGE_SEGMENT}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine Online Documentation}*/
export class EdgeSegmentSnapLine {
/**
Initializes a new instance of the {@link PointBasedSnapLine} class.
@param edge The edge to assign to {@link EdgeSegmentSnapLine#edge}.
@param from The first location that delimits this snap line.
@param to The second location that delimits this snap line.
@param segmentSource The location that represents the source of the related segment on this line.
@param segmentTarget The location that represents the target of the related segment on this line.
@param distance The distance of this snap line from its related edge segment.
@param segmentIndex The segment index of the related edge segment.
@param tag A tag that is associated with this snap line.
@param weight The weight (importance) of this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23EdgeSegmentSnapLine(IEdge%2CPoint%2CPoint%2CPoint%2CPoint%2Cnumber%2Cnumber%2CObject%2Cnumber) Online Documentation}*/
constructor(edge: IEdge, from: Point | PointConvertible, to: Point | PointConvertible, segmentSource: Point | PointConvertible, segmentTarget: Point | PointConvertible, distance: number, segmentIndex: number, tag: any | null, weight: number);
/**
Initializes a new instance of the {@link PointBasedSnapLine} class.
@param edge The edge to assign to {@link EdgeSegmentSnapLine#edge}.
@param segmentSource The location that represents the source of the related segment on this line. In addition, this location is used as {@link SnapLine#from}
location.
@param segmentTarget The location that represents the target of the related segment on this line. In addition, this location is used as {@link SnapLine#to}
location.
@param distance The distance of this snap line from its related edge segment.
@param segmentIndex The segment index of the related edge segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23EdgeSegmentSnapLine(IEdge%2CPoint%2CPoint%2Cnumber%2Cnumber) Online Documentation}*/
constructor(edge: IEdge, segmentSource: Point | PointConvertible, segmentTarget: Point | PointConvertible, distance: number, segmentIndex: number);
/**
Gets the edge that is associated with this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23edge Online Documentation}*/
edge: IEdge;
/**
Gets the direction vector of this snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23direction Online Documentation}*/
direction: Point;
/**
Gets the distance of this snap line from its related edge segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23distance Online Documentation}*/
distance: number;
/**
Gets the segment index of the related edge segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23segmentIndex Online Documentation}*/
segmentIndex: number;
/**
Gets the location that represents the source of the related segment on this line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23segmentSource Online Documentation}*/
segmentSource: Point;
/**
Gets the location that represents the target of the related segment on this line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23segmentTarget Online Documentation}*/
segmentTarget: Point;
/**
Gets or sets the location of the source marker of this snap line.
If this is <code>null</code>, the location of the {@link EdgeSegmentSnapLine#segmentSource} is marked instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23sourceMarker Online Documentation}*/
sourceMarker: Point | null;
/**
Gets or sets the location of the target marker of this snap line.
If this is <code>null</code>, the location of the {@link EdgeSegmentSnapLine#segmentTarget} is marked instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentSnapLine%23targetMarker Online Documentation}*/
targetMarker: Point | null;
static $class: Class;
private $_EdgeSegmentSnapLine: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeSegmentSnapLine;
}
/**
Enumeration of snap line visualization types.
Determines the visual appearance of the snap lines.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType Online Documentation}*/
export enum SnapLineVisualizationType {
/**
Determines a snap line having a fixed visualization independent of the {@link SnapResult}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23FIXED_LINE Online Documentation}*/
FIXED_LINE,
/**
Determines a decorated line between the {@link SnapLine#coordinates} and the snapped location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23VARIABLE_LINE Online Documentation}*/
VARIABLE_LINE,
/**
Determines a blank line between the {@link SnapLine#coordinates} and the snapped location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23BLANK_VARIABLE_LINE Online Documentation}*/
BLANK_VARIABLE_LINE,
/**
Determines a decorated line between the {@link SnapLine#coordinates} and the snapped location that is extended on both sides.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23EXTENDED_VARIABLE_LINE Online Documentation}*/
EXTENDED_VARIABLE_LINE,
/**
Determines a snap line which snaps to a distance between different graph items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23FIXED_DISTANCE Online Documentation}*/
FIXED_DISTANCE,
/**
Determines a snap line which lets an item snap in the center between two bounds of other items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23CENTER_BETWEEN_BOUNDS Online Documentation}*/
CENTER_BETWEEN_BOUNDS,
/**
Determines a snap line which lets an item snap in the center between two centers of other items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23CENTER_BETWEEN_CENTERS Online Documentation}*/
CENTER_BETWEEN_CENTERS,
/**
Determines a snap line which lets an item snap if it has an equal distance to two other items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23EQUAL_DISTANCE_BETWEEN_BOUNDS Online Documentation}*/
EQUAL_DISTANCE_BETWEEN_BOUNDS,
/**
Determines a snap line which lets an item snap if it's center has an equal distance to the centers' of two other items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23EQUAL_DISTANCE_BETWEEN_CENTERS Online Documentation}*/
EQUAL_DISTANCE_BETWEEN_CENTERS,
/**
Determines the snapping of a resized item if it's new width is equal to another item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23EQUAL_WIDTH Online Documentation}*/
EQUAL_WIDTH,
/**
Determines the snapping of a resized item if it's new height is equal to another item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23EQUAL_HEIGHT Online Documentation}*/
EQUAL_HEIGHT,
/**
Determines a snap line for an edge segment with three marks for special start, end and center positions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23EDGE_SEGMENT Online Documentation}*/
EDGE_SEGMENT,
/**
Determines a snap line for snapping on a grid line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23GRID_FIXED_LINE Online Documentation}*/
GRID_FIXED_LINE,
/**
Determines a snap line that doesn't have a visualization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineVisualizationType%23NONE Online Documentation}*/
NONE
}
/** Automatic conversion to {@link SnapLineVisualizationType}. */
type SnapLineVisualizationTypeStringValues = "fixed-line" | "variable-line" | "blank-variable-line" | "extended-variable-line" | "fixed-distance" | "center-between-bounds" | "center-between-centers" | "equal-distance-between-bounds" | "equal-distance-between-centers" | "equal-width" | "equal-height" | "edge-segment" | "grid-fixed-line" | "none";
/**
The orientation used by {@link OrthogonalSnapLine#orientation orthogonal snap lines}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineOrientation Online Documentation}*/
export enum SnapLineOrientation {
/**
Constant that describes a horizontal {@link SnapLineOrientation orientation} of a snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineOrientation%23HORIZONTAL Online Documentation}*/
HORIZONTAL,
/**
Constant that describes a vertical {@link SnapLineOrientation orientation} of a snap line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineOrientation%23VERTICAL Online Documentation}*/
VERTICAL
}
/** Automatic conversion to {@link SnapLineOrientation}. */
type SnapLineOrientationStringValues = "horizontal" | "vertical";
/**
The type of a {@link SnapLine#snapType snap line}.
A snap line can be used for multiple types of snaps.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes Online Documentation}*/
export enum SnapLineSnapTypes {
/**
Describes snap lines to which nothing will snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes%23NONE Online Documentation}*/
NONE,
/**
Describes snap lines to which the top sides of rectangular elements will snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes%23TOP Online Documentation}*/
TOP,
/**
Describes snap lines to which the bottom sides of rectangular elements will snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes%23BOTTOM Online Documentation}*/
BOTTOM,
/**
Describes snap lines to which the left sides of rectangular elements will snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes%23LEFT Online Documentation}*/
LEFT,
/**
Describes snap lines to which the right sides of rectangular elements will snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes%23RIGHT Online Documentation}*/
RIGHT,
/**
Describes snap lines to which the center of elements will snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes%23CENTER Online Documentation}*/
CENTER,
/**
Describes vertical snap lines
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes%23VERTICAL Online Documentation}*/
VERTICAL,
/**
Describes horizontal snap lines
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes%23HORIZONTAL Online Documentation}*/
HORIZONTAL,
/**
Describes snap lines to which all elements will snap at all sides
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapLineSnapTypes%23ALL Online Documentation}*/
ALL
}
type SnapLineSnapTypesStringValues = "none" | "top" | "bottom" | "left" | "right" | "center" | "vertical" | "horizontal" | "all";
/** Automatic conversion to {@link SnapLineSnapTypes}. */
type SnapLineSnapTypesConvertible = SnapLineSnapTypesStringValues | SnapLineSnapTypesStringValues[];
/**
Enumeration for the allowed data types of a GraphML attribute.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyType Online Documentation}*/
export enum KeyType {
/**
Identifier for attribute of type <code>int</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyType%23INT Online Documentation}*/
INT,
/**
Identifier for attribute of type <code>long</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyType%23LONG Online Documentation}*/
LONG,
/**
Identifier for attribute of type <code>float</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyType%23FLOAT Online Documentation}*/
FLOAT,
/**
Identifier for attribute of type <code>double</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyType%23DOUBLE Online Documentation}*/
DOUBLE,
/**
Identifier for attribute of type <code>string</code> that denotes simple string content as opposed to
{@link KeyType#COMPLEX}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyType%23STRING Online Documentation}*/
STRING,
/**
Identifier for attribute of type <code>boolean</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyType%23BOOLEAN Online Documentation}*/
BOOLEAN,
/**
Identifier for arbitrary complex xml content
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyType%23COMPLEX Online Documentation}*/
COMPLEX
}
/** Automatic conversion to {@link KeyType}. */
type KeyTypeStringValues = "int" | "long" | "float" | "double" | "string" | "boolean" | "complex";
/**
Enumeration for the possible scopes of a GraphML attribute.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyScope Online Documentation}*/
export enum KeyScope {
/**
Unscoped for key definition
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyScope%23ALL Online Documentation}*/
ALL,
/**
Node scope for key definition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyScope%23NODE Online Documentation}*/
NODE,
/**
Edge scope for key definition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyScope%23EDGE Online Documentation}*/
EDGE,
/**
Graph scope for key definition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyScope%23GRAPH Online Documentation}*/
GRAPH,
/**
Port scope for key definition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyScope%23PORT Online Documentation}*/
PORT,
/**
Graphml scope for key definition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyScope%23GRAPHML Online Documentation}*/
GRAPHML
}
/** Automatic conversion to {@link KeyScope}. */
type KeyScopeStringValues = "all" | "node" | "edge" | "graph" | "port" | "graphml";
export interface IParseContext extends ILookup {
}
/**
Interface that defines the context of a GraphML parsing context.
All state data that is needed for the parsing of a concrete input should go here, allowing all handlers and
deserializers to be implemented stateless.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseContext Online Documentation}*/
export abstract class IParseContext {
/**
Deserialize the object representation in <code>targetNode</code>.
This method calls {@link IParseContext#deserializeCore} with <code>context</code> as first argument and should be used in
virtually all cases.
Type parameter <code>T</code>: The desired target type.
@param targetType The desired target type.
@param targetNode The XML node that should be deserialized.
@returns an instance of <code>T</code> or <code>null</code>.
@throws {Exception} <b>(DeserializationNotSupportedError)</b>
@see IParseContext#deserializeCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseContext%23deserialize(Class%2CNode) Online Documentation}*/
deserialize<T>(targetType: Class, targetNode: Node): T;
/**
Deserialize the object representation in <code>targetNode</code>
Client code should usually use the method {@link IParseContext#deserialize} instead to ensure the the correct <code>context</code>
instance is used.
Type parameter <code>T</code>: The expected return type.
@param targetType The expected return type.
@param context The current parse context.
@param targetNode The XML node that should be deserialized.
@returns The deserialized object from <code>targetNode</code> or an {@link Exception DeserializationNotSupportedError} if <code>targetNode</code> could not be deserialized.
@see IParseContext#deserialize
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseContext%23deserializeCore(Class%2CIParseContext%2CNode) Online Documentation}*/
abstract deserializeCore<T>(targetType: Class, context: IParseContext, targetNode: Node): T;
/**
Returns the most current (the last element) within the container hierarchy as it is returned by {@link IParseContext#objectStack} or <code>default(T)</code> if the
most current element is not of the desired target type.
Type parameter <code>T</code>: The desired target type.
@param itemType The desired target type.
@returns the most current element within the container hierarchy or <code>default(T)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseContext%23getCurrent(Class) Online Documentation}*/
abstract getCurrent<T>(itemType: Class): T | null;
/**
Get a property value that specifies information about how to handle specific cases.
@param key The identifier for the property
@returns The property value, or <code>null</code> if no such property exists.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseContext%23getDeserializationProperty(string) Online Documentation}*/
abstract getDeserializationProperty(key: string): any | null;
/**
Gets the current nesting of created graphs and graph elements.
The list contains the user objects which correspond to the GraphML elements which are ancestors of the current node in
the DOM tree.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseContext%23objectStack Online Documentation}*/
abstract readonly objectStack: IListEnumerable<any>;
/**
Gets an implementation of {@link IParseEvents} that allows to subscribe to various events in the parse process.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseContext%23parseEvents Online Documentation}*/
abstract readonly parseEvents: IParseEvents;
/**
Gets the currently active graph object
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseContext%23graph Online Documentation}*/
abstract readonly graph: IGraph;
static $class: Class;
private $_IParseContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IParseContext;
}
export interface IParseEvents {
}
/**
The interface of the event trigger class that is used by the {@link IParseContext#parseEvents} property.
This interface defines the available events that client code can register to during the parsing of {@link GraphMLParser GraphML}.
@see ParseEventArgs
@see GraphMLParser
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents Online Documentation}*/
export abstract class IParseEvents {
/**
Adds the given listener for the <code>DocumentParsing</code> event that occurs when the document is about to be parsed.
This event is triggered when the document element has been encountered.
@param listener The listener to add.
@see IParseEvents#removeDocumentParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23DocumentParsing Online Documentation}*/
abstract addDocumentParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>DocumentParsing</code> event that occurs when the document is about to be parsed.
This event is triggered when the document element has been encountered.
@param listener The listener to remove.
@see IParseEvents#addDocumentParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23DocumentParsing Online Documentation}*/
abstract removeDocumentParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>DocumentParsed</code> event that occurs when the document has been parsed.
This event is triggered when the document has been fully parsed.
@param listener The listener to add.
@see IParseEvents#removeDocumentParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23DocumentParsed Online Documentation}*/
abstract addDocumentParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>DocumentParsed</code> event that occurs when the document has been parsed.
This event is triggered when the document has been fully parsed.
@param listener The listener to remove.
@see IParseEvents#addDocumentParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23DocumentParsed Online Documentation}*/
abstract removeDocumentParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>NodeParsing</code> event that occurs when a node element is about to be parsed.
This event is triggered when a GraphML 'node' element has been encountered.
@param listener The listener to add.
@see IParseEvents#removeNodeParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23NodeParsing Online Documentation}*/
abstract addNodeParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>NodeParsing</code> event that occurs when a node element is about to be parsed.
This event is triggered when a GraphML 'node' element has been encountered.
@param listener The listener to remove.
@see IParseEvents#addNodeParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23NodeParsing Online Documentation}*/
abstract removeNodeParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>NodeParsed</code> event that occurs after a node element has been fully parsed.
This event is triggered when a GraphML 'node' closing tag has been handled.
@param listener The listener to add.
@see IParseEvents#removeNodeParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23NodeParsed Online Documentation}*/
abstract addNodeParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>NodeParsed</code> event that occurs after a node element has been fully parsed.
This event is triggered when a GraphML 'node' closing tag has been handled.
@param listener The listener to remove.
@see IParseEvents#addNodeParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23NodeParsed Online Documentation}*/
abstract removeNodeParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>EdgeParsing</code> event that occurs when an edge element is about to be parsed.
This event is triggered when a GraphML 'edge' element has been encountered.
@param listener The listener to add.
@see IParseEvents#removeEdgeParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23EdgeParsing Online Documentation}*/
abstract addEdgeParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>EdgeParsing</code> event that occurs when an edge element is about to be parsed.
This event is triggered when a GraphML 'edge' element has been encountered.
@param listener The listener to remove.
@see IParseEvents#addEdgeParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23EdgeParsing Online Documentation}*/
abstract removeEdgeParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>EdgeParsed</code> event that occurs after a edge element has been fully parsed.
This event is triggered when a GraphML 'edge' closing tag has been handled.
@param listener The listener to add.
@see IParseEvents#removeEdgeParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23EdgeParsed Online Documentation}*/
abstract addEdgeParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>EdgeParsed</code> event that occurs after a edge element has been fully parsed.
This event is triggered when a GraphML 'edge' closing tag has been handled.
@param listener The listener to remove.
@see IParseEvents#addEdgeParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23EdgeParsed Online Documentation}*/
abstract removeEdgeParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>PortParsing</code> event that occurs when a port element is about to be parsed.
This event is triggered when a GraphML 'port' element has been encountered.
@param listener The listener to add.
@see IParseEvents#removePortParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23PortParsing Online Documentation}*/
abstract addPortParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>PortParsing</code> event that occurs when a port element is about to be parsed.
This event is triggered when a GraphML 'port' element has been encountered.
@param listener The listener to remove.
@see IParseEvents#addPortParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23PortParsing Online Documentation}*/
abstract removePortParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>PortParsed</code> event that occurs after a port element has been fully parsed.
This event is triggered when a GraphML 'port' closing tag has been handled.
@param listener The listener to add.
@see IParseEvents#removePortParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23PortParsed Online Documentation}*/
abstract addPortParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>PortParsed</code> event that occurs after a port element has been fully parsed.
This event is triggered when a GraphML 'port' closing tag has been handled.
@param listener The listener to remove.
@see IParseEvents#addPortParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23PortParsed Online Documentation}*/
abstract removePortParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>GraphParsing</code> event that occurs when a graph element is about to be parsed.
This event is triggered when a GraphML 'graph' element has been encountered.
@param listener The listener to add.
@see IParseEvents#removeGraphParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23GraphParsing Online Documentation}*/
abstract addGraphParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>GraphParsing</code> event that occurs when a graph element is about to be parsed.
This event is triggered when a GraphML 'graph' element has been encountered.
@param listener The listener to remove.
@see IParseEvents#addGraphParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23GraphParsing Online Documentation}*/
abstract removeGraphParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>GraphParsed</code> event that occurs after a graph element has been fully parsed.
This event is triggered when a GraphML 'graph' closing tag has been handled.
@param listener The listener to add.
@see IParseEvents#removeGraphParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23GraphParsed Online Documentation}*/
abstract addGraphParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>GraphParsed</code> event that occurs after a graph element has been fully parsed.
This event is triggered when a GraphML 'graph' closing tag has been handled.
@param listener The listener to remove.
@see IParseEvents#addGraphParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23GraphParsed Online Documentation}*/
abstract removeGraphParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>GraphMLParsing</code> event that occurs when the graphml element is about to be parsed.
This event is triggered when a GraphML 'graphml' root-element has been encountered.
@param listener The listener to add.
@see IParseEvents#removeGraphMLParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23GraphMLParsing Online Documentation}*/
abstract addGraphMLParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>GraphMLParsing</code> event that occurs when the graphml element is about to be parsed.
This event is triggered when a GraphML 'graphml' root-element has been encountered.
@param listener The listener to remove.
@see IParseEvents#addGraphMLParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23GraphMLParsing Online Documentation}*/
abstract removeGraphMLParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>GraphMLParsed</code> event that occurs after the graphml element has been fully parsed.
This event is triggered when a GraphML 'graphml' closing tag has been handled.
@param listener The listener to add.
@see IParseEvents#removeGraphMLParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23GraphMLParsed Online Documentation}*/
abstract addGraphMLParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>GraphMLParsed</code> event that occurs after the graphml element has been fully parsed.
This event is triggered when a GraphML 'graphml' closing tag has been handled.
@param listener The listener to remove.
@see IParseEvents#addGraphMLParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23GraphMLParsed Online Documentation}*/
abstract removeGraphMLParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>DataParsing</code> event that occurs when a data element is about to be parsed.
This event is triggered when a GraphML 'data' element has been encountered.
@param listener The listener to add.
@see IParseEvents#removeDataParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23DataParsing Online Documentation}*/
abstract addDataParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>DataParsing</code> event that occurs when a data element is about to be parsed.
This event is triggered when a GraphML 'data' element has been encountered.
@param listener The listener to remove.
@see IParseEvents#addDataParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23DataParsing Online Documentation}*/
abstract removeDataParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>DataParsed</code> event that occurs after a data element has been fully parsed.
This event is triggered when a GraphML 'data' element has been handled.
@param listener The listener to add.
@see IParseEvents#removeDataParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23DataParsed Online Documentation}*/
abstract addDataParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>DataParsed</code> event that occurs after a data element has been fully parsed.
This event is triggered when a GraphML 'data' element has been handled.
@param listener The listener to remove.
@see IParseEvents#addDataParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23DataParsed Online Documentation}*/
abstract removeDataParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>KeyParsing</code> event that occurs when a key element is about to be parsed.
This event is triggered when a GraphML 'key' element has been encountered.
@param listener The listener to add.
@see IParseEvents#removeKeyParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23KeyParsing Online Documentation}*/
abstract addKeyParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>KeyParsing</code> event that occurs when a key element is about to be parsed.
This event is triggered when a GraphML 'key' element has been encountered.
@param listener The listener to remove.
@see IParseEvents#addKeyParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23KeyParsing Online Documentation}*/
abstract removeKeyParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>KeyParsed</code> event that occurs after a key element has been fully parsed.
This event is triggered when a GraphML 'key' element has been handled.
@param listener The listener to add.
@see IParseEvents#removeKeyParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23KeyParsed Online Documentation}*/
abstract addKeyParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>KeyParsed</code> event that occurs after a key element has been fully parsed.
This event is triggered when a GraphML 'key' element has been handled.
@param listener The listener to remove.
@see IParseEvents#addKeyParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IParseEvents%23KeyParsed Online Documentation}*/
abstract removeKeyParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
static $class: Class;
private $_IParseEvents: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IParseEvents;
}
export interface ParseEventArgs extends EventArgs {
}
/**
This is the event argument that is used by the {@link IParseEvents} interface that can be queried from the {@link IParseContext#parseEvents}
property.
This class holds a reference to the {@link ParseEventArgs#context parse context} and the {@link ParseEventArgs#element} that is currently being parsed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParseEventArgs Online Documentation}*/
export class ParseEventArgs {
/**
Initializes a new instance of the {@link ParseEventArgs} class.
@param context The context to assign to {@link ParseEventArgs#context}.
@param element The XML element to assign to {@link ParseEventArgs#element}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParseEventArgs Online Documentation}*/
constructor(context: IParseContext, element: Element | null);
/**
Gets the context that is being used for parsing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParseEventArgs%23context Online Documentation}*/
context: IParseContext;
/**
Gets the XML element that is currently parsed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParseEventArgs%23element Online Documentation}*/
element: Element | null;
static $class: Class;
private $_ParseEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ParseEventArgs;
}
export interface GraphMLXmlAttribute extends YObject {
}
/**
A data holder for use in the {@link IOutputHandler} interface that encapsulates an XML attribute.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLXmlAttribute Online Documentation}*/
export class GraphMLXmlAttribute {
/**
Initializes a new instance of the {@link GraphMLXmlAttribute} class.
@param name The attribute name.
@param value The value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLXmlAttribute Online Documentation}*/
constructor(name: XmlName, value: string);
/**
Gets the fully qualified name of the attribute.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLXmlAttribute%23name Online Documentation}*/
name: XmlName;
/**
Gets the value of the attribute.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLXmlAttribute%23value Online Documentation}*/
value: string;
static $class: Class;
private $_GraphMLXmlAttribute: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphMLXmlAttribute;
}
export interface IOutputHandler {
}
/**
The interface used by the {@link GraphMLIOHandler} for writing data to the GraphML.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOutputHandler Online Documentation}*/
export abstract class IOutputHandler {
/**
Determines whether in the current context, the value is the default value and therefore no data element needs to be
written.
@param context The context.
@returns <code>true</code> if for the current context the default value applies and therefore no data element needs to be written.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOutputHandler%23isDefaultValue(IWriteContext) Online Documentation}*/
abstract isDefaultValue(context: IWriteContext): boolean;
/**
Writes the contents of the key definition.
At the time this method is called, the surrounding 'key' element has already been written. However no 'default' element
is written by the framework.
@param context The context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOutputHandler%23writeKeyDefinitionContent(IWriteContext) Online Documentation}*/
abstract writeKeyDefinitionContent(context: IWriteContext): void;
/**
Actually writes the value for the current context.
At the time this method is called, the surrounding 'data' element has already been written.
@param context The context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOutputHandler%23writeValue(IWriteContext) Online Documentation}*/
abstract writeValue(context: IWriteContext): void;
/**
Gets the write precedence that is associated with this instance.
Registered output handlers are written either before or after the (optional) child content of the governing GraphML
element.
@see WritePrecedence
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOutputHandler%23precedence Online Documentation}*/
abstract readonly precedence: WritePrecedence;
/**
Gets the XML attributes that should be added to the key definition in the GraphML key element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOutputHandler%23keyDefinitionAttributes Online Documentation}*/
abstract readonly keyDefinitionAttributes: IEnumerable<GraphMLXmlAttribute>;
/**
Gets the XML attributes that should be added to the data element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IOutputHandler%23dataTagAttributes Online Documentation}*/
abstract readonly dataTagAttributes: IEnumerable<GraphMLXmlAttribute>;
static $class: Class;
private $_IOutputHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IOutputHandler;
}
/**
Enumeration values to control the order in which {@link IOutputHandler} instances are used.
These enumeration values control whether the output handler is evaluated before or after any (optional) structural child
content of the owner element.
@see IOutputHandler#precedence
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/WritePrecedence Online Documentation}*/
export enum WritePrecedence {
/**
The handler writes its content before the child nodes of the governing GraphML element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WritePrecedence%23BEFORE_CHILDREN Online Documentation}*/
BEFORE_CHILDREN,
/**
Default precedence, the order in relation to child content is unspecified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WritePrecedence%23DEFAULT Online Documentation}*/
DEFAULT,
/**
The handler writes its content after the child nodes of the governing GraphML element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WritePrecedence%23AFTER_CHILDREN Online Documentation}*/
AFTER_CHILDREN
}
/** Automatic conversion to {@link WritePrecedence}. */
type WritePrecedenceStringValues = "before-children" | "default" | "after-children";
export interface IWriteContext extends ILookup {
}
/**
Interface that provides the context of a GraphML write process.
All state data that is needed for the writing of a concrete graph instance should go here, allowing all handlers and
deserializers to be implemented stateless.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext Online Documentation}*/
export abstract class IWriteContext {
/**
Returns the most current (the last element) within the container hierarchy as it is returned by {@link IWriteContext#objectStack} or <code>default(T)</code> if the
most current element is not of the desired target type.
Type parameter <code>T</code>: The desired target type.
@param itemType The desired target type.
@returns the most current element within the container hierarchy or <code>default(T)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23getCurrent(Class) Online Documentation}*/
abstract getCurrent<T>(itemType: Class): T | null;
/**
Get a property value that specifies information about how to handle specific cases
@param key The identifier for the property
@returns The property value, or <code>null</code> if no such property exists
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23getSerializationProperty(string) Online Documentation}*/
abstract getSerializationProperty(key: string): any | null;
/**
Serializes the specified item using the provided type information.
<code>item</code> should be assignable to type <code>t</code>. This method simply delegates to
{@link IWriteContext#serializeCore}, using the correct type parameter.
<b>Warning:</b> This method is not available unless the module <b>view-graphml</b> is loaded.
@param item The item to serialize.
@param t The type the item is known to be of at deserialization time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23serialize(Object%2CClass) Online Documentation}*/
serialize(item: any | null, t: Class): void;
/**
Serialize the <code>item</code>.
This method calls {@link IWriteContext#serializeCore} with <code>context</code> as first argument and should be used in
virtually all cases.
<b>Warning:</b> This method is not available unless the module <b>view-graphml</b> is loaded.
Type parameter <code>T</code>: The type of the object that is serialized.
@param targetType The type of the object that is serialized.
@param item The object that should be serialized.
@throws {Exception} <b>(SerializationNotSupportedError)</b>
@see IWriteContext#serializeCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23serialize(Class%2CT) Online Documentation}*/
serialize<T>(targetType: Class, item: T | null): void;
/**
Serialize <code>item</code> to an XML representation.
Client code should usually use the method {@link IWriteContext#serialize} instead to ensure the the correct <code>context</code>
instance is used.
Type parameter <code>T</code>: The type of the object that is serialized.
@param targetType The type of the object that is serialized.
@param context The current write context.
@param item The object that should be serialized.
@throws {Exception} <b>(SerializationNotSupportedError)</b>
@see IWriteContext#serialize
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23serializeCore(Class%2CIWriteContext%2CT) Online Documentation}*/
abstract serializeCore<T>(targetType: Class, context: IWriteContext, item: T | null): void;
/**
Serialize an replacement object <code>replacement</code> instead of the original <code>originalItem</code>.
Calling this method allows to remember the original object instance, even if the object is really written through a {@link MarkupExtension}
or similar replacement mechanisms.
<b>Warning:</b> This method is not available unless the module <b>view-graphml</b> is loaded.
Type parameter <code>T</code>: The type of the object that is serialized.
@param originalItem The original object that should be serialized.
@param replacement The replacement object that will actually be serialized instead.
@throws {Exception} <b>(SerializationNotSupportedError)</b>
@see IWriteContext#serializeCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23serializeReplacement(Object%2CT) Online Documentation}*/
serializeReplacement<T>(targetType: Class, originalItem: any | null, replacement: T | null): void;
/**
Gets the current nesting of graphs and graph elements.
The list contains the user objects which correspond to the GraphML elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23objectStack Online Documentation}*/
abstract readonly objectStack: IListEnumerable<any>;
/**
Gets an implementation of {@link IWriteEvents} that allows to subscribe to various events in the write process.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23writeEvents Online Documentation}*/
abstract readonly writeEvents: IWriteEvents;
/**
Gets the current {@link IXmlWriter} implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23writer Online Documentation}*/
abstract readonly writer: IXmlWriter;
/**
Gets the currently active graph object
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteContext%23graph Online Documentation}*/
abstract readonly graph: IGraph;
static $class: Class;
private $_IWriteContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IWriteContext;
}
export interface IWriteEvents {
}
/**
The interface of the event trigger class that is used by the {@link IWriteContext#writeEvents} property.
This interface defines the available events that client code can register to during the writing of {@link GraphMLWriter GraphML}.
@see WriteEventArgs
@see GraphMLWriter
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents Online Documentation}*/
export abstract class IWriteEvents {
/**
Adds the given listener for the <code>DocumentWriting</code> event that occurs just after the {@link IXmlWriter#writeStartDocument}
has been called.
@param listener The listener to add.
@see IWriteEvents#removeDocumentWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23DocumentWriting Online Documentation}*/
abstract addDocumentWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>DocumentWriting</code> event that occurs just after the {@link IXmlWriter#writeStartDocument}
has been called.
@param listener The listener to remove.
@see IWriteEvents#addDocumentWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23DocumentWriting Online Documentation}*/
abstract removeDocumentWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>DocumentWritten</code> event that occurs just before the {@link IXmlWriter#writeEndDocument}
will be called.
@param listener The listener to add.
@see IWriteEvents#removeDocumentWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23DocumentWritten Online Documentation}*/
abstract addDocumentWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>DocumentWritten</code> event that occurs just before the {@link IXmlWriter#writeEndDocument}
will be called.
@param listener The listener to remove.
@see IWriteEvents#addDocumentWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23DocumentWritten Online Documentation}*/
abstract removeDocumentWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>NodeWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'node' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to add.
@see IWriteEvents#removeNodeWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23NodeWriting Online Documentation}*/
abstract addNodeWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>NodeWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'node' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to remove.
@see IWriteEvents#addNodeWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23NodeWriting Online Documentation}*/
abstract removeNodeWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>NodeWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'node' element will be called.
This can be used to append more custom elements to the node element.
@param listener The listener to add.
@see IWriteEvents#removeNodeWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23NodeWritten Online Documentation}*/
abstract addNodeWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>NodeWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'node' element will be called.
This can be used to append more custom elements to the node element.
@param listener The listener to remove.
@see IWriteEvents#addNodeWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23NodeWritten Online Documentation}*/
abstract removeNodeWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>EdgeWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'edge' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to add.
@see IWriteEvents#removeEdgeWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23EdgeWriting Online Documentation}*/
abstract addEdgeWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>EdgeWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'edge' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to remove.
@see IWriteEvents#addEdgeWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23EdgeWriting Online Documentation}*/
abstract removeEdgeWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>EdgeWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'edge' element will be called.
This can be used to append more custom elements to the edge element.
@param listener The listener to add.
@see IWriteEvents#removeEdgeWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23EdgeWritten Online Documentation}*/
abstract addEdgeWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>EdgeWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'edge' element will be called.
This can be used to append more custom elements to the edge element.
@param listener The listener to remove.
@see IWriteEvents#addEdgeWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23EdgeWritten Online Documentation}*/
abstract removeEdgeWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>PortWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'port' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to add.
@see IWriteEvents#removePortWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23PortWriting Online Documentation}*/
abstract addPortWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>PortWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'port' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to remove.
@see IWriteEvents#addPortWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23PortWriting Online Documentation}*/
abstract removePortWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>PortWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'port' element will be called.
This can be used to append more custom elements to the port element.
@param listener The listener to add.
@see IWriteEvents#removePortWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23PortWritten Online Documentation}*/
abstract addPortWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>PortWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'port' element will be called.
This can be used to append more custom elements to the port element.
@param listener The listener to remove.
@see IWriteEvents#addPortWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23PortWritten Online Documentation}*/
abstract removePortWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>GraphWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'graph' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to add.
@see IWriteEvents#removeGraphWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23GraphWriting Online Documentation}*/
abstract addGraphWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>GraphWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'graph' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to remove.
@see IWriteEvents#addGraphWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23GraphWriting Online Documentation}*/
abstract removeGraphWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>GraphWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'graph' element will be called.
This can be used to append more custom elements to the graph element.
@param listener The listener to add.
@see IWriteEvents#removeGraphWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23GraphWritten Online Documentation}*/
abstract addGraphWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>GraphWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'graph' element will be called.
This can be used to append more custom elements to the graph element.
@param listener The listener to remove.
@see IWriteEvents#addGraphWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23GraphWritten Online Documentation}*/
abstract removeGraphWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>GraphMLWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'graphml' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to add.
@see IWriteEvents#removeGraphMLWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23GraphMLWriting Online Documentation}*/
abstract addGraphMLWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>GraphMLWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'graphml' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to remove.
@see IWriteEvents#addGraphMLWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23GraphMLWriting Online Documentation}*/
abstract removeGraphMLWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>GraphMLWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'graphml' element will be called.
This can be used to append more custom elements to the graphml element.
@param listener The listener to add.
@see IWriteEvents#removeGraphMLWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23GraphMLWritten Online Documentation}*/
abstract addGraphMLWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>GraphMLWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'graphml' element will be called.
This can be used to append more custom elements to the graphml element.
@param listener The listener to remove.
@see IWriteEvents#addGraphMLWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23GraphMLWritten Online Documentation}*/
abstract removeGraphMLWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>DataWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'data' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to add.
@see IWriteEvents#removeDataWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23DataWriting Online Documentation}*/
abstract addDataWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>DataWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'data' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to remove.
@see IWriteEvents#addDataWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23DataWriting Online Documentation}*/
abstract removeDataWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>DataWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'data' element will be called.
This can be used to append more custom elements to the data element.
@param listener The listener to add.
@see IWriteEvents#removeDataWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23DataWritten Online Documentation}*/
abstract addDataWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>DataWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'data' element will be called.
This can be used to append more custom elements to the data element.
@param listener The listener to remove.
@see IWriteEvents#addDataWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23DataWritten Online Documentation}*/
abstract removeDataWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>KeyWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'key' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to add.
@see IWriteEvents#removeKeyWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23KeyWriting Online Documentation}*/
abstract addKeyWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>KeyWriting</code> event that occurs right after the {@link IXmlWriter#writeStartElement}
method for writing a GraphMl 'key' element has been called.
At that state, callees can use the {@link IXmlWriter#writeAttributeNS} methods to add custom attributes to the
XML element or write a first custom inner element using {@link IXmlWriter#writeStartElement}
@param listener The listener to remove.
@see IWriteEvents#addKeyWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23KeyWriting Online Documentation}*/
abstract removeKeyWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>KeyWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'key' element will be called.
This can be used to append more custom elements to the key element.
@param listener The listener to add.
@see IWriteEvents#removeKeyWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23KeyWritten Online Documentation}*/
abstract addKeyWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>KeyWritten</code> event that occurs just before the {@link IXmlWriter#writeEndElement}
for the GraphML 'key' element will be called.
This can be used to append more custom elements to the key element.
@param listener The listener to remove.
@see IWriteEvents#addKeyWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IWriteEvents%23KeyWritten Online Documentation}*/
abstract removeKeyWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
static $class: Class;
private $_IWriteEvents: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IWriteEvents;
}
export interface IXmlWriter {
}
/**
Abstraction of XML output classes that provide a consistent way to write XML elements to some output
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter Online Documentation}*/
export abstract class IXmlWriter {
/**
Closes the document.
This is typically used to actually flush the document to an output stream
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23flushDocument() Online Documentation}*/
abstract flushDocument(): void;
/**
Gets a text representation of all content that has been written so far.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23getXmlString() Online Documentation}*/
abstract getXmlString(): string;
/**
Write an XML attribute with the given parameters.
The attribute is not bound to a XML namespace.
@param localName The name of the attribute.
@param value The value of the attribute.
@returns The writer instance <code>writer</code> for writer chaining.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeAttribute(string%2Cstring) Online Documentation}*/
writeAttribute(localName: string, value: string): IXmlWriter;
/**
Writes a XML attribute node.
The attribute is attached to the closest open XML element that has been started with {@link IXmlWriter#writeStartElement}
or {@link IXmlWriter#writeStartElement}. Note that namespace declarations should not be written directly as
attributes. Instead the {@link IXmlWriter#namespaceManager} should be used.
@param prefix The namespace prefix of the attribute
@param localName The local name of the attribute
@param ns The namespace URI of this attribute
@param value The value of this attribute
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeAttributeNS(string%2Cstring%2Cstring%2Cstring) Online Documentation}*/
abstract writeAttributeNS(prefix: string | null, localName: string, ns: string | null, value: string): IXmlWriter;
/**
Writes a CDATA section.
@param content The content of the CDATA section
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeCData(string) Online Documentation}*/
abstract writeCData(content: string): IXmlWriter;
/**
Writes a XML comment node
@param comment The content of the comment
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeComment(string) Online Documentation}*/
abstract writeComment(comment: string): IXmlWriter;
/**
Writes a document fragment
@param fragment
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeDocumentFragment(Document) Online Documentation}*/
abstract writeDocumentFragment(fragment: Document): IXmlWriter;
/**
Closes the output.
Attempts to write after this method has been called will have undefined results.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeEndDocument() Online Documentation}*/
abstract writeEndDocument(): void;
/**
Closes a XML element previously opened with {@link IXmlWriter#writeStartElement} or
{@link IXmlWriter#writeStartElement}.
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeEndElement() Online Documentation}*/
abstract writeEndElement(): IXmlWriter;
/**
Write an XML attribute from the given <code>attribute</code> struct.
@param attribute Encapsulates the current attribute values.
@returns The writer instance <code>writer</code> for writer chaining.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeGraphMLXmlAttribute(GraphMLXmlAttribute) Online Documentation}*/
writeGraphMLXmlAttribute(attribute: GraphMLXmlAttribute): IXmlWriter;
/**
Writes a xml processing instruction.
@param target The target of the PI
@param data The data of the PI
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeProcessingInstruction(string%2Cstring) Online Documentation}*/
abstract writeProcessingInstruction(target: string, data: string): IXmlWriter;
/**
Begins the output process
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeStartDocument() Online Documentation}*/
abstract writeStartDocument(): IXmlWriter;
/**
Begins a new XML element with given parameters
All subsequent output is attached to this element until {@link IXmlWriter#writeEndElement} has been called or a
new element has been started.
@param prefix The namespace prefix to use for this element
@param localName The local name of this element
@param ns The namespace of this element
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeStartElement(string%2Cstring%2Cstring) Online Documentation}*/
abstract writeStartElement(prefix: string | null, localName: string, ns: string | null): IXmlWriter;
/**
Begin a new XML element with given parameters
All subsequent output is attached to this element until {@link IXmlWriter#writeEndElement} has been called or a
new element has been started.
This acts like {@link IXmlWriter#writeStartElement}, but uses the default prefix for the namespace
@param localName The local name of this element
@param ns The namespace of this element
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeStartElement(string%2Cstring) Online Documentation}*/
abstract writeStartElement(localName: string, ns: string | null): IXmlWriter;
/**
Writes a text node.
The string value of <code>s</code> will be correctly escaped
@param s The string that gets written as XML text
@returns The writer instance for further chaining
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23writeString(string) Online Documentation}*/
abstract writeString(s: string): IXmlWriter;
/**
Gets the namespace manager associated with this writer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlWriter%23namespaceManager Online Documentation}*/
abstract readonly namespaceManager: IXmlNamespaceManager;
static $class: Class;
private $_IXmlWriter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IXmlWriter;
}
export interface IXmlNamespaceManager {
}
/**
Facilitates namespace handling together with {@link IXmlWriter} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlNamespaceManager Online Documentation}*/
export abstract class IXmlNamespaceManager {
/**
Returns a unique namespace prefix, or creates and registers one.
@param ns The namespace URI for which a prefix is searched.
@returns a unique namespace prefix for <code>ns</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlNamespaceManager%23getOrCreateUniqueMapping(string) Online Documentation}*/
abstract getOrCreateUniqueMapping(ns: string): string;
/**
Pops a current declaration scope.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlNamespaceManager%23popScope() Online Documentation}*/
abstract popScope(): void;
/**
Pushes a current declaration scope.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlNamespaceManager%23pushScope() Online Documentation}*/
abstract pushScope(): void;
/**
Register a local namespace/prefix mapping
@param nsUri The namespace URI
@param prefix The preferred namespace prefix or <code>null</code> if any prefix may be used.
@returns The prefix that finally has been used for the namespace URI.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlNamespaceManager%23registerLocalMapping(string%2Cstring) Online Documentation}*/
abstract registerLocalMapping(nsUri: string, prefix: string | null): string;
/**
Seals the current declaration scope.
No further namespace mappings may be added to a sealed scope.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlNamespaceManager%23sealScope() Online Documentation}*/
abstract sealScope(): void;
/**
Gets a list of all namespace declarations that should be declared at the document's root element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXmlNamespaceManager%23globalNamespaces Online Documentation}*/
abstract readonly globalNamespaces: IMap<string, string>;
static $class: Class;
private $_IXmlNamespaceManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IXmlNamespaceManager;
}
export interface SerializationProperties extends YObject {
}
/**
Constant class for common serialization properties.
@see IWriteContext#getSerializationProperty
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties Online Documentation}*/
export class SerializationProperties {
/**
Property key to control user tag serialization and deserialization
This property has a value of type {@link GraphItemTypes}.
Setting this property as a {@link GraphMLIOHandler#serializationPropertyOverrides serialization property} disables tag serialization for all of the provided
{@link GraphItemTypes}. Setting this property as a {@link GraphMLIOHandler#deserializationPropertyOverrides deserialization property} disables tag deserialization only for
nodes, edges and/or ports. By default, this property is set to {@link GraphItemTypes#NONE}
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23DISABLE_USER_TAGS Online Documentation}*/
static DISABLE_USER_TAGS: string;
/**
Property key to control graph element style serialization and deserialization
This property has a value of type {@link GraphItemTypes}.
Setting this property as a {@link GraphMLIOHandler#serializationPropertyOverrides serialization property} disables style serialization for all of the provided
{@link GraphItemTypes}. Setting this property as a {@link GraphMLIOHandler#deserializationPropertyOverrides deserialization property} disables style deserialization only for
nodes, edges and/or ports. By default, this property is set to {@link GraphItemTypes#NONE}
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23DISABLE_STYLES Online Documentation}*/
static DISABLE_STYLES: string;
/**
Property key to control graph elements serialization and deserialization
This property has a value of type {@link GraphItemTypes}.
Setting this property disables serialization and deserialization for the provided {@link GraphItemTypes}. By
default, this property is set to {@link GraphItemTypes#NONE}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23DISABLE_ITEMS Online Documentation}*/
static DISABLE_ITEMS: string;
/**
Property key to control graph element geometry serialization and deserialization
This property has a value of type {@link GraphItemTypes}.
Setting this property as a {@link GraphMLIOHandler#serializationPropertyOverrides serialization property} disables geometry serialization for all of the provided
{@link GraphItemTypes}. Setting this property as a {@link GraphMLIOHandler#deserializationPropertyOverrides deserialization property} disables geometry deserialization only
for nodes, edges and/or ports. By default, this property is set to {@link GraphItemTypes#NONE}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23DISABLE_GEOMETRY Online Documentation}*/
static DISABLE_GEOMETRY: string;
/**
Property key to control user tag serialization and deserialization of {@link IStripe}s.
This property has a value of type {@link StripeTypes}.
Setting this property disables tag serialization for the provided {@link StripeTypes}. By default, this property is
set to {@link StripeTypes#NONE}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23DISABLE_STRIPE_USER_TAGS Online Documentation}*/
static DISABLE_STRIPE_USER_TAGS: string;
/**
Property key to control style serialization and deserialization of {@link IStripe}s.
This property has a value of type {@link StripeTypes}.
Setting this property disables style serialization for the provided {@link StripeTypes}. By default, this property
is set to {@link StripeTypes#NONE}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23DISABLE_STRIPE_STYLES Online Documentation}*/
static DISABLE_STRIPE_STYLES: string;
/**
Property key to control label serialization of {@link IStripe}s.
This property has a value of type {@link StripeTypes}.
Setting this property disables label serialization for the provided {@link StripeTypes}. By default, this property
is set to {@link StripeTypes#NONE}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23DISABLE_STRIPE_LABELS Online Documentation}*/
static DISABLE_STRIPE_LABELS: string;
/**
Property key to omit serialization and deserialization of the {@link IGraph}'s default settings.
This property has a boolean value.
If this property is set to <code>true</code> in {@link GraphMLIOHandler#serializationPropertyOverrides} or the write context, the current graph defaults are not written.
If this property is set to <code>true</code> in {@link GraphMLIOHandler#deserializationPropertyOverrides} or the parse context, any graph defaults present in the input are not parsed and
applied to the current graph.
Default for this property is <code>true</code> for writing and <code>false</code> for parsing.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23DISABLE_GRAPH_SETTINGS Online Documentation}*/
static DISABLE_GRAPH_SETTINGS: string;
/**
Property key for alllowing to parse member names (properties and enum values) case-insensitively during XAML
deserialization.
This will convert property and member names to lower-case and remove underscores to be able to compare things like <code>SOME_CONSTANT_NAME</code>
and <code>SomeConstantName</code> as well as <code>somePropertyName</code> and <code>SomePropertyName</code>, respectively.
This property has a boolean value.
Default for this property is <code>false</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23IGNORE_PROPERTY_CASE Online Documentation}*/
static IGNORE_PROPERTY_CASE: string;
/**
Property key to control how undefined values are serialized.
This property has a value of type {@link UndefinedHandling}.
The default for this property is {@link UndefinedHandling#EXPLICIT }.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23UNDEFINED_HANDLING Online Documentation}*/
static UNDEFINED_HANDLING: string;
/**
Property key whether the XML output should be properly indented.
This property has a boolean values.
Default for this property is <code>true</code>
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23INDENT_OUTPUT Online Documentation}*/
static INDENT_OUTPUT: string;
/**
The current {@link KeyScope} in which an {@link IOutputHandler},
{@link IInputHandler}, or custom serialization code is called.
This property has a value of type {@link KeyScope}
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23CURRENT_KEY_SCOPE Online Documentation}*/
static CURRENT_KEY_SCOPE: string;
/**
The edge that is currently being represented by the dummy edge that is written instead.
This property has a value of type {@link IEdge}.
If this value is set, then the edge currently being serialized represents the given edge in reality.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23REPRESENTED_EDGE Online Documentation}*/
static REPRESENTED_EDGE: string;
/**
Property key to specify whether to write the default node style in the &lt;default> element of the key definition.
This property has a boolean value.
Default value is <code>false</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23WRITE_NODE_STYLE_DEFAULT Online Documentation}*/
static WRITE_NODE_STYLE_DEFAULT: string;
/**
Property key to specify whether to write the default edge style in the &lt;default> element of the key definition.
This property has a boolean value.
Default value is <code>false</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23WRITE_EDGE_STYLE_DEFAULT Online Documentation}*/
static WRITE_EDGE_STYLE_DEFAULT: string;
/**
Property key to specify whether to write the default port style in the &lt;default> element of the key definition.
This property has a boolean value.
Default value is <code>false</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23WRITE_PORT_STYLE_DEFAULT Online Documentation}*/
static WRITE_PORT_STYLE_DEFAULT: string;
/**
Property key to specify whether the stripe defaults of a table should be written.
This property has a boolean value.
If set to <code>true</code> the {@link ITable#columnDefaults} and {@link ITable#rowDefaults} will be written. Default value is <code>true</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23WRITE_STRIPE_DEFAULTS Online Documentation}*/
static WRITE_STRIPE_DEFAULTS: string;
/**
Property key to provide a predicate that determines if the preferred size of a label should be written.
If this property is not set, the label size is always written.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23WRITE_LABEL_SIZE_PREDICATE Online Documentation}*/
static WRITE_LABEL_SIZE_PREDICATE: string;
/**
Property key to specify whether the preferred size of labels should be parsed.
This property has a boolean value.
If this property is set to <code>true</code>, the preferred size provided by GraphML is used, otherwise, the preferred size is
determined automatically. This property is mainly meant for interoperability usecases where the text size calculations
can be quite different for different platforms. Default value is <code>true</code>
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23PARSE_LABEL_SIZE Online Documentation}*/
static PARSE_LABEL_SIZE: string;
/**
The current {@link string} base URI which is used for the GraphML file being parsed or written.
This property has a string value.
If set, this can be used to resolve or create relative paths.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23BASE_URI Online Documentation}*/
static BASE_URI: string;
/**
Property key to specify whether to rewrite relative URIs relative to the current document location.
This property has a boolean value.
If set, relative URIs for images and other resources are rewritten relative to
{@link SerializationProperties#BASE_URI}. If this property is <code>false</code>, or if {@link SerializationProperties#BASE_URI}
is not set (either explicitly or implicitly) or the resource URI is not relative, the resource URI will not be
rewritten.
Default value is <code>true</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23REWRITE_RELATIVE_RESOURCE_URIS Online Documentation}*/
static REWRITE_RELATIVE_RESOURCE_URIS: string;
/**
Property key to specify whether to ignore errors during XAML deserialization will be ignored and instead return <code>null</code>.
This property has a boolean value.
Default value is <code>false</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23IGNORE_XAML_DESERIALIZATION_ERRORS Online Documentation}*/
static IGNORE_XAML_DESERIALIZATION_ERRORS: string;
/**
Property key to control caching for external references.
This property has a boolean value.
If this property is set to <code>false</code> in the write context, the writer will not cache remember <b>external</b> reference ids provided via
{@link GraphMLIOHandler#addQueryReferenceIdListener QueryReferenceId}, i.e. the event will be
raised whenever an instance of the external object is encountered during the write process. This allows to provide
different values for the same object, depending on the current context. Default for this property is <code>true</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SerializationProperties%23CACHE_EXTERNAL_REFERENCES Online Documentation}*/
static CACHE_EXTERNAL_REFERENCES: string;
static $class: Class;
private $_SerializationProperties: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SerializationProperties;
}
export interface WriteEventArgs extends EventArgs {
}
/**
This is the event argument that is used by the {@link IWriteEvents} interface that can be queried from the {@link IWriteContext#writeEvents}
property.
This class holds a reference to the {@link WriteEventArgs#context write context} and the {@link WriteEventArgs#item} that is currently being written.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WriteEventArgs Online Documentation}*/
export class WriteEventArgs {
/**
Initializes a new instance of the {@link WriteEventArgs} class.
@param context The context to assign to {@link WriteEventArgs#context}.
@param item The item to assign to {@link WriteEventArgs#item}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WriteEventArgs Online Documentation}*/
constructor(context: IWriteContext, item: any | null);
/**
Gets the context that is being used for writing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WriteEventArgs%23context Online Documentation}*/
context: IWriteContext;
/**
Gets the item that is currently being written or <code>null</code> if nothing is applicable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WriteEventArgs%23item Online Documentation}*/
item: any | null;
static $class: Class;
private $_WriteEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WriteEventArgs;
}
export interface IMarkupExtensionConverter {
}
/**
An interface for converters that can convert an object to a corresponding {@link MarkupExtension}.
This interface is only used for GraphML serialization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMarkupExtensionConverter Online Documentation}*/
export abstract class IMarkupExtensionConverter {
/**
Returns if the converter can convert the <code>value</code> to a {@link MarkupExtension} in the current write context.
@param context The current write context.
@param value The object to convert.
@returns Whether the converter can convert the passed value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMarkupExtensionConverter%23canConvert(IWriteContext%2CObject) Online Documentation}*/
abstract canConvert(context: IWriteContext, value: any): boolean;
/**
Converts the <code>value</code> to a {@link MarkupExtension} using the current write context.
@param context The current write context.
@param value The object to convert.
@returns The {@link MarkupExtension} the value was converted to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMarkupExtensionConverter%23convert(IWriteContext%2CObject) Online Documentation}*/
abstract convert(context: IWriteContext, value: any): MarkupExtension;
static $class: Class;
private $_IMarkupExtensionConverter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IMarkupExtensionConverter;
}
export interface IXamlNameMapper {
}
/**
Facilitates determining type information from XML elements and writing of markup extensions as attributes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXamlNameMapper Online Documentation}*/
export abstract class IXamlNameMapper {
/**
Returns a {@link Class} that matches the given <code>fqTagName</code> of an XML element.
<code>fqTagName</code> must follow the usual XAML naming rules.
@param context The current parse context.
@param fqTagName The fully qualified XML element name that must describe a valid type in this CLR namespace.
@returns The first matching type, if any.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXamlNameMapper%23getClassForName(IParseContext%2CXmlName) Online Documentation}*/
abstract getClassForName(context: IParseContext, fqTagName: XmlName): Class | null;
/**
Returns the XML tag name that belongs to type <code>type</code>
@param context The current write context.
@param type The type for which an XML namespace mapping is searched.
@returns the XML tag name that belongs to type <code>type</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXamlNameMapper%23getNameForClass(IWriteContext%2CClass) Online Documentation}*/
abstract getNameForClass(context: IWriteContext, type: Class): XmlName;
static $class: Class;
private $_IXamlNameMapper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IXamlNameMapper;
}
export interface Property extends YObject {
}
/**
Wraps a property together with default values and meta data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property Online Documentation}*/
export class Property {
/**
Gets the value the wrapped property has on the given instance.
@param ownerInstance The instance to get the property value from.
@returns The value the wrapped property has on the given instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23getValue(Object) Online Documentation}*/
getValue(ownerInstance: any | null): any | null;
/**
Sets the value to the wrapped property on the given instance.
@param ownerInstance The instance to set the property value on.
@param value The value to set.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23setValue(Object%2CObject) Online Documentation}*/
setValue(ownerInstance: any | null, value: any | null): void;
/**
Gets whether or not the property has a <b>public</b> getter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23canRead Online Documentation}*/
readonly canRead: boolean;
/**
Gets whether or not the property has a <b>public</b> setter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23canWrite Online Documentation}*/
readonly canWrite: boolean;
/**
Gets whether or not this is a static property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23isStatic Online Documentation}*/
readonly isStatic: boolean;
/**
Gets the serialization visibility.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23serializationVisibility Online Documentation}*/
readonly serializationVisibility: GraphMLMemberVisibility;
/**
Gets the {@link Property#valueSerializer}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23valueSerializer Online Documentation}*/
readonly valueSerializer: ValueSerializer | null;
/**
Gets the MarkupExtensionConverter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23markupExtensionConverter Online Documentation}*/
readonly markupExtensionConverter: IMarkupExtensionConverter | null;
/**
Gets the {@link Property#graphMLAttribute}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23graphMLAttribute Online Documentation}*/
readonly graphMLAttribute: GraphMLAttribute | null;
/**
Gets a value indicating whether this instance is the one defined in the {@link GraphMLAttribute}'s {@link GraphMLAttribute#contentProperty}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23isContentProperty Online Documentation}*/
readonly isContentProperty: boolean;
/**
Gets the type of the property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23type Online Documentation}*/
readonly type: Class;
/**
Gets the type of the owner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23ownerType Online Documentation}*/
readonly ownerType: Class;
/**
Gets a value indicating whether this instance has a default value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23hasDefaultValue Online Documentation}*/
readonly hasDefaultValue: boolean;
/**
Gets the name of the property, as it will appear in GraphML.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Property%23name Online Documentation}*/
readonly name: string;
static $class: Class;
private $_Property: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Property;
}
export interface BendAnchoredPortLocationModel extends YObject, IPortLocationModel {
/**
@inheritdoc*/
createParameter(owner: IPortOwner, location: Point): IPortLocationModelParameter;
/**
@inheritdoc*/
getContext(port: IPort, locationParameter: IPortLocationModelParameter): ILookup;
/**
@inheritdoc*/
getLocation(port: IPort, locationParameter: IPortLocationModelParameter): Point;
} /**
An implementation of the {@link IPortLocationModel} that can create {@link IPortLocationModelParameter}s that
describe the location of ports that are attached to the bends of {@link IEdge}s.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendAnchoredPortLocationModel Online Documentation}*/
export class BendAnchoredPortLocationModel {
/**
Creates a parameter that indicates the location of the bend with index <code>bendIndex</code> starting from the source side.
@param bendIndex The index of the bend counting from the source side.
@returns A parameter that anchors the port at a bend.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendAnchoredPortLocationModel%23createFromSource(number) Online Documentation}*/
createFromSource(bendIndex: number): IPortLocationModelParameter;
/**
Creates a parameter that indicates the location of the bend with index <code>bendIndex</code> starting from the target side.
@param bendIndex The index of the bend counting from the target side.
@returns A parameter that anchors the port at a bend.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendAnchoredPortLocationModel%23createFromTarget(number) Online Documentation}*/
createFromTarget(bendIndex: number): IPortLocationModelParameter;
/**
This implementation has nothing in its lookup and will always yield <code>null</code>
@param type
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendAnchoredPortLocationModel%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
A static immutable global instance of this class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendAnchoredPortLocationModel%23INSTANCE Online Documentation}*/
static INSTANCE: BendAnchoredPortLocationModel;
/**
A parameter that is bound to the first {@link IBend} of the edge.
@see BendAnchoredPortLocationModel#createFromSource
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendAnchoredPortLocationModel%23FIRST_BEND Online Documentation}*/
static FIRST_BEND: IPortLocationModelParameter;
/**
A parameter that is bound to the last {@link IBend} of the edge.
@see BendAnchoredPortLocationModel#createFromTarget
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendAnchoredPortLocationModel%23LAST_BEND Online Documentation}*/
static LAST_BEND: IPortLocationModelParameter;
static $class: Class;
private $_BendAnchoredPortLocationModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BendAnchoredPortLocationModel;
}
export interface BezierEdgePathLabelModel extends YObject, ILabelModel, ILabelModelParameterFinder {
/**
@inheritdoc*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
An edge label model that allows placement of labels along the edge path for bezier edges.
This model is intended to work with a {@link BezierEdgeStyle} and assumes an edge path consisting of cubic
segments.
It places the label at a given ratio of the whole path length and distance from the path, which are parameter specific.
Additionally, the label can be automatically rotated to be parallel to the tangent vector at the given ratio
({@link BezierEdgePathLabelModel#autoRotation}) and can have an additional rotation angle (independently of the automatic rotation ({@link BezierEdgePathLabelModel#angle})
If the parameter is specified as an absolute parameter, a placement outside of the path is assumed by taking the
connection between source and target as base line and applying the ratio to this line segment. In this case, if auto
rotation is enabled, the label will be rotated to match the path slope at the beginning resp. end of the edge path in
order to ensure a smooth movement.
Labels are freely movable and movement can optionally be snapped so that the center of the label lies on the edge path,
if the label intersects the edge path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgePathLabelModel Online Documentation}*/
export class BezierEdgePathLabelModel {
/**
Creates a default instance of the label model with no angle, auto ratio enabled and snapping disabled.
@param [options] The parameters to pass.
@param [options.autoRotation] Whether or not edge labels are automatically rotated according to the tangent of the corresponding reference edge segment. This option sets the {@link BezierEdgePathLabelModel#autoRotation} property on the created object.
@param [options.angle] The rotation angle of all labels with this model in radians. This option sets the {@link BezierEdgePathLabelModel#angle} property on the created object.
@param [options.autoSnapping] Whether the model should snap automatically the layout center to the edge path. This option sets the {@link BezierEdgePathLabelModel#autoSnapping} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgePathLabelModel Online Documentation}*/
constructor(options?: {
autoRotation?: boolean;
angle?: number;
autoSnapping?: boolean;
});
/**
Creates a parameter at middle of the edge path directly on the path.
@returns A parameter at middle of the edge path directly on the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgePathLabelModel%23createDefaultParameter() Online Documentation}*/
createDefaultParameter(): ILabelModelParameter;
/**
Creates a parameter at the provided edge path ratio.
@param ratio The cubic ratio on the segment
@param [distance=0.0] The distance from the segment. This may be interpreted differently, depending on whether {@link BezierEdgePathLabelModel#autoSnapping} is enabled or not.
@param [absolute=false] Whether the ratio should be interpreted as the ratio between the source and target ports of the edge. Default is false.
@returns A new parameter with the given specification.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgePathLabelModel%23createParameter(number%2Cnumber%2Cboolean) Online Documentation}*/
createParameter(ratio: number, distance?: number, absolute?: boolean): ILabelModelParameter;
/**
Creates a parameter at the provided edge path ratio.
@param options The parameters to pass.
@param options.ratio The cubic ratio on the segment
@param [options.distance=0.0] The distance from the segment. This may be interpreted differently, depending on whether {@link BezierEdgePathLabelModel#autoSnapping} is enabled or not.
@param [options.absolute=false] Whether the ratio should be interpreted as the ratio between the source and target ports of the edge. Default is false.
@returns A new parameter with the given specification.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgePathLabelModel%23createParameter(number%2Cnumber%2Cboolean) Online Documentation}*/
createParameter(options: {
ratio: number;
distance?: number;
absolute?: boolean;
}): ILabelModelParameter;
/**
Returns an empty context.
@param label The label to use in the context.
@param layoutParameter The parameter to use for the label in the context.
@returns An empty context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgePathLabelModel%23getContext(ILabel%2CILabelModelParameter) Online Documentation}*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
Gets or sets whether or not edge labels are automatically rotated according to the tangent of the corresponding
reference edge segment.
By default, this feature is enabled.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgePathLabelModel%23autoRotation Online Documentation}*/
autoRotation: boolean;
/**
Gets or sets the rotation angle of all labels with this model in radians.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgePathLabelModel%23angle Online Documentation}*/
angle: number;
/**
Gets or sets whether the model should snap automatically the layout center to the edge path.
If set to <code>true</code>: - The distance of the parameter will be adjusted so that the whole box has a minimum distance to the
edge path - Interactive movements will snap to the edge path as soon as the label layout would come too near to the edge
path. Default value is <code>false</code>
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgePathLabelModel%23autoSnapping Online Documentation}*/
autoSnapping: boolean;
static $class: Class;
private $_BezierEdgePathLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BezierEdgePathLabelModel;
}
export interface BezierEdgeSegmentLabelModel extends YObject, ILabelModel, ILabelModelParameterFinder {
/**
@inheritdoc*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
An edge label model that allows placement of labels relative to an edge segment for bezier edges.
This model is intended to work with a {@link BezierEdgeStyle} and assumes an edge path consisting of cubic
segments.
It places the label at a fixed segment with given ratio (a cubic curve parameter value) and distance from the path,
which are parameter specific. Additionally, the label can be automatically rotated to be parallel to the tangent vector
at the given ratio ({@link BezierEdgeSegmentLabelModel#autoRotation}) and can have an additional rotation angle (independently of the automatic rotation ({@link BezierEdgeSegmentLabelModel#angle})
If the segment index is less than 0, a placement outside of the path is assumed by taking the connection between source
and target as base line and applying the ratio to this line segment. In this case, if auto rotation is enabled, the
label will be rotated to match the path slope at the beginning resp. end of the edge path in order to ensure a smooth
movement.
Labels are freely movable and movement can optionally be snapped so that the center of the label lies on the edge path,
if the label intersects the edge path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel Online Documentation}*/
export class BezierEdgeSegmentLabelModel {
/**
Creates a default instance of the label model with no angle, auto ratio enabled and snapping disabled.
@param [options] The parameters to pass.
@param [options.autoRotation] Whether or not edge labels are automatically rotated according to the tangent of the corresponding reference edge segment. This option sets the {@link BezierEdgeSegmentLabelModel#autoRotation} property on the created object.
@param [options.angle] The rotation angle of all labels with this model in radians. This option sets the {@link BezierEdgeSegmentLabelModel#angle} property on the created object.
@param [options.autoSnapping] Whether the model should snap automatically the layout center to the edge path. This option sets the {@link BezierEdgeSegmentLabelModel#autoSnapping} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel Online Documentation}*/
constructor(options?: {
autoRotation?: boolean;
angle?: number;
autoSnapping?: boolean;
});
/**
Creates a parameter at middle of the first segment directly on the path.
@returns A parameter at middle of the first segment directly on the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel%23createDefaultParameter() Online Documentation}*/
createDefaultParameter(): ILabelModelParameter;
/**
Creates a parameter at the given segment measured from the source side of the edge path, segment ratio and distance from
the segment.
@param segmentIndex The segment index. If this value is negative, placement outside of the edge path is assumed.
@param [segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the source side of the
segment, a ratio of <code>1.0</code> at the target side. Ratios are interpreted as parameter values of a cubic spline, i.e. not as
percentages of the segment length. If the index is <code>-1</code>, ratios are interpreted as absolute values instead. The default is
<code>0.5</code>.
@param [distance=0.0] The distance from the segment. This may be interpreted differently, depending on whether {@link BezierEdgeSegmentLabelModel#autoSnapping} is enabled or not.
@returns A new parameter with the given specification.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel%23createParameterFromSource(number%2Cnumber%2Cnumber) Online Documentation}*/
createParameterFromSource(segmentIndex: number, segmentRatio?: number, distance?: number): ILabelModelParameter;
/**
Creates a parameter at the given segment measured from the source side of the edge path, segment ratio and distance from
the segment.
@param options The parameters to pass.
@param options.segmentIndex The segment index. If this value is negative, placement outside of the edge path is assumed.
@param [options.segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the source side of the
segment, a ratio of <code>1.0</code> at the target side. Ratios are interpreted as parameter values of a cubic spline, i.e. not as
percentages of the segment length. If the index is <code>-1</code>, ratios are interpreted as absolute values instead. The default is
<code>0.5</code>.
@param [options.distance=0.0] The distance from the segment. This may be interpreted differently, depending on whether {@link BezierEdgeSegmentLabelModel#autoSnapping} is enabled or not.
@returns A new parameter with the given specification.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel%23createParameterFromSource(number%2Cnumber%2Cnumber) Online Documentation}*/
createParameterFromSource(options: {
segmentIndex: number;
segmentRatio?: number;
distance?: number;
}): ILabelModelParameter;
/**
Creates a parameter at the given segment measured from the target side of the edge path, segment ratio and distance from
the segment.
@param segmentIndex The segment index. If this value is negative, placement outside of the edge path is assumed.
@param [segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the target side of the
segment, a ratio of <code>1.0</code> at the source side. Ratios are interpreted as parameter values of a cubic spline, i.e. not as
percentages of the segment length. If the index is <code>-1</code>, ratios are interpreted as absolute values instead. The default is
<code>0.5.</code>
@param [distance=0.0] The distance from the segment. This may be interpreted differently, depending on whether {@link BezierEdgeSegmentLabelModel#autoSnapping} is enabled or not.
@returns A new parameter with the given specification.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel%23createParameterFromTarget(number%2Cnumber%2Cnumber) Online Documentation}*/
createParameterFromTarget(segmentIndex: number, segmentRatio?: number, distance?: number): ILabelModelParameter;
/**
Creates a parameter at the given segment measured from the target side of the edge path, segment ratio and distance from
the segment.
@param options The parameters to pass.
@param options.segmentIndex The segment index. If this value is negative, placement outside of the edge path is assumed.
@param [options.segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the target side of the
segment, a ratio of <code>1.0</code> at the source side. Ratios are interpreted as parameter values of a cubic spline, i.e. not as
percentages of the segment length. If the index is <code>-1</code>, ratios are interpreted as absolute values instead. The default is
<code>0.5.</code>
@param [options.distance=0.0] The distance from the segment. This may be interpreted differently, depending on whether {@link BezierEdgeSegmentLabelModel#autoSnapping} is enabled or not.
@returns A new parameter with the given specification.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel%23createParameterFromTarget(number%2Cnumber%2Cnumber) Online Documentation}*/
createParameterFromTarget(options: {
segmentIndex: number;
segmentRatio?: number;
distance?: number;
}): ILabelModelParameter;
/**
Returns an empty context.
@param label The label to use in the context.
@param layoutParameter The parameter to use for the label in the context.
@returns An empty context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel%23getContext(ILabel%2CILabelModelParameter) Online Documentation}*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
Gets or sets whether or not edge labels are automatically rotated according to the tangent of the corresponding
reference edge segment.
By default, this feature is enabled.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel%23autoRotation Online Documentation}*/
autoRotation: boolean;
/**
Gets or sets the rotation angle of all labels with this model in radians.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel%23angle Online Documentation}*/
angle: number;
/**
Gets or sets whether the model should snap automatically the layout center to the edge path.
If set to <code>true</code>: - The distance of the parameter will be adjusted so that the whole box has a minimum distance to the
edge path - Interactive movements will snap to the edge path as soon as the label layout would come too near to the edge
path. Default value is <code>false</code>
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeSegmentLabelModel%23autoSnapping Online Documentation}*/
autoSnapping: boolean;
static $class: Class;
private $_BezierEdgeSegmentLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BezierEdgeSegmentLabelModel;
}
export interface CompositeLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider, ILabelModelParameterFinder {
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
* An implementation of {@link ILabelModel} that can be used to create custom label models out of existing {@link ILabelModel}
* instances.
* This class holds a number of models and can provide all of their parameters in
* {@link CompositeLabelModel#getParameters}. A custom model can easily be created that way:
*
* <br><b>Creating a CompositeLabelModel</b>
* <pre>
* const compositeLabelModel = new CompositeLabelModel()
* compositeLabelModel.labelModels.add(new ExteriorLabelModel())
* compositeLabelModel.labelModels.add(new InteriorStretchLabelModel())
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLabelModel Online Documentation}
*/
export class CompositeLabelModel {
/**
Creates a new instance of this label model with an empty {@link CompositeLabelModel#labelModels} collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLabelModel Online Documentation}*/
constructor();
/**
Creates a default parameter that can be used for this model.
If {@link CompositeLabelModel#labelModels} is not empty, the default parameter for the first model in that list is returned. Otherwise, the default parameter
for {@link FreeLabelModel#createDefaultParameter} is returned.
@returns a parameter for this model instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLabelModel%23createDefaultParameter() Online Documentation}*/
createDefaultParameter(): ILabelModelParameter;
/**
Tries to find a parameter that best matches the given layout for the provided label instance.
This implementation tests all parameters from all models in {@link CompositeLabelModel#labelModels}.
@param label The label to find a parameter for.
@param model The model instance to use. This should be the instance this instance has been obtained from.
@param layout The anticipated layout for the label.
@returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLabelModel%23findBestParameter(ILabel%2CILabelModel%2CIOrientedRectangle) Online Documentation}*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
Returns an enumerator over a set of possible {@link ILabelModelParameter} instances that can be used for the given
label and model.
This implementation returns all parameters from all models in {@link CompositeLabelModel#labelModels}
@param label The label instance to use.
@param model The model to provide parameters for.
@returns A possibly empty enumerator over a set of label model parameters.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLabelModel%23getParameters(ILabel%2CILabelModel) Online Documentation}*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
Gets a list of {@link ILabelModel} implementations that are wrapped by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLabelModel%23labelModels Online Documentation}*/
readonly labelModels: IList<ILabelModel>;
static $class: Class;
private $_CompositeLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CompositeLabelModel;
}
export interface Arrow extends YObject, IArrow, ICloneable {
}
/**
* A default implementation of the {@link IArrow} interface that can render {@link ArrowType}s.
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* The strings applying to this scheme are converted to {@link Arrow}s:
*
* <pre>
* '[color] [scale] type'
*
* color = CSS color strings
* scale = xx-small|x-small|small|medium|large|x-large|xx-large
* type = normal|none|default|simple|short|diamond|cross|circle|ball|triangle
* </pre>
*
* Valid color strings are the same as for {@link Color}. Square brackets mark optional declarations.
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow Online Documentation}
*/
export class Arrow {
/**
Creates a new arrow with the given color to use for {@link Arrow#stroke} and brush.
The {@link Arrow#stroke} is created with thickness 1.
@param color The color to use for {@link Arrow#stroke} and brush.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23Arrow(Color) Online Documentation}*/
constructor(color: Color | ColorConvertible);
/**
Creates a new arrow with the given color to use for {@link Arrow#stroke} and brush.
The {@link Arrow#stroke} is created with thickness 1.
@param options The parameters to pass.
@param options.color The color to use for {@link Arrow#stroke} and brush.
@param [options.cropLength] The cropping length associated with this instance. This option sets the {@link Arrow#cropLength} property on the created object.
@param [options.stroke] The {@link Arrow#stroke} of the outline of this arrow. This option sets the {@link Arrow#stroke} property on the created object.
@param [options.scale] The scale factor of this arrow. This option sets the {@link Arrow#scale} property on the created object.
@param [options.type] The type of this arrow. This option sets the {@link Arrow#type} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23Arrow(Color) Online Documentation}*/
constructor(options: {
color: Color | ColorConvertible;
cropLength?: number;
stroke?: Stroke | null | StrokeConvertible;
scale?: number;
type?: ArrowType | ArrowTypeStringValues;
});
/**
Creates a new arrow with default values for {@link Arrow#stroke} and brush.
@param [options] The parameters to pass.
@param [options.fill] The {@link Arrow#fill} of the fill of this arrow. This option sets the {@link Arrow#fill} property on the created object.
@param [options.cropLength] The cropping length associated with this instance. This option sets the {@link Arrow#cropLength} property on the created object.
@param [options.stroke] The {@link Arrow#stroke} of the outline of this arrow. This option sets the {@link Arrow#stroke} property on the created object.
@param [options.scale] The scale factor of this arrow. This option sets the {@link Arrow#scale} property on the created object.
@param [options.type] The type of this arrow. This option sets the {@link Arrow#type} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23Arrow() Online Documentation}*/
constructor(options?: {
fill?: Fill | null | FillConvertible;
cropLength?: number;
stroke?: Stroke | null | StrokeConvertible;
scale?: number;
type?: ArrowType | ArrowTypeStringValues;
});
/**
Returns an immutable {@link IArrow} instance that uses the current properties.
@returns An immutable instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23asFrozen() Online Documentation}*/
asFrozen(): IArrow;
/**
Determines whether another object is equal to this one.
@param other the other object to compare for equality
@returns <code>true</code> if <code>other</code> is considered equal to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23toString() Online Documentation}*/
toString(): string;
/**
Gets or sets the type of this arrow.
@default ArrowType.DEFAULT
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23type Online Documentation}*/
type: ArrowType;
/**
Gets or sets the scale factor of this arrow.
The arrow bounds are scaled with this factor.
@default 1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23scale Online Documentation}*/
scale: number;
/**
Gets or sets the {@link Arrow#stroke} of the outline of this arrow.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23stroke Online Documentation}*/
stroke: Stroke | null;
/**
Gets or sets the {@link Arrow#fill} of the fill of this arrow.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23fill Online Documentation}*/
fill: Fill | null;
/**
Gets the cropping length associated with this instance.
This value is used by {@link IEdgeStyle} implementations to let the edge appear to end shortly before its actual
target.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/Arrow%23cropLength Online Documentation}*/
cropLength: number;
static $class: Class;
private $_Arrow: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Arrow;
}
/** Automatic conversion to {@link Arrow}. */
type ArrowConvertible = string;
export interface SimpleBend extends YObject, IBend {
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
tag: any | null;
} /**
A simple mutable implementation of the {@link IBend} interface that can be used stand-alone.
Note that this class is not the one used as the default implementation of a {@link DefaultGraph}'s
{@link IBend}s and casting them to {@link SimpleBend} will fail. Instances of this class can be used without
an accompanying graph instance. All of the properties are mutable. An example of a possible use case would be the
rendering of a {@link SimpleEdge} with bends into a graphics context: Simply create two {@link SimpleNode}s, a
{@link SimpleEdge} to connect the two nodes and add instances of this class to the edge's {@link SimpleEdge#bends} model. Then render the
edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleBend Online Documentation}*/
export class SimpleBend {
/**
Initializes a new instance of the {@link SimpleBend} class using the given owner and location.
@param [owner=null] The owner of the bend.
@param [location=null] The bend's location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleBend Online Documentation}*/
constructor(owner?: IEdge | null, location?: IPoint | IPointConvertible);
/**
Initializes a new instance of the {@link SimpleBend} class using the given owner and location.
@param options The parameters to pass.
@param [options.owner=null] The owner of the bend.
@param [options.location=null] The bend's location.
@param [options.tag] Sets the tag property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleBend Online Documentation}*/
constructor(options: {
owner?: IEdge | null;
location?: IPoint | IPointConvertible;
tag?: any | null;
});
/**
Gets or sets the edge this bend instance belongs to.
This implies that <code>Owner.Bends</code> contains this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleBend%23owner Online Documentation}*/
owner: IEdge | null;
/**
Gets or sets the location of this bend.
@see SimpleBend#x
@see SimpleBend#y
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleBend%23location Online Documentation}*/
location: IPoint;
/**
Gets the current x coordinate of this bend.
This delegates to the {@link SimpleBend#location} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleBend%23x Online Documentation}*/
readonly x: number;
/**
Gets the current y coordinate of this bend.
This delegates to the {@link SimpleBend#location} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleBend%23y Online Documentation}*/
readonly y: number;
/**
Gets or sets the {@link ILookup} implementation used for calls to {@link SimpleBend#lookup} on this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleBend%23lookupImplementation Online Documentation}*/
lookupImplementation: ILookup | null;
static $class: Class;
private $_SimpleBend: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SimpleBend;
}
export interface SimpleEdge extends YObject, IEdge {
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
tag: any | null;
} /**
A simple mutable implementation of the {@link IEdge} interface that can be used stand-alone.
Note that this class is not the one used as the default implementation of a {@link DefaultGraph}'s
{@link IEdge}s and casting them to {@link SimpleEdge} will fail. Instances of this class can be used without
an accompanying graph instance. All of the properties are mutable. An example of a possible use case would be the
rendering of an edge into a graphics context: Simply create a
{@link SimpleEdge}, associate a style with it and use the style's renderer instance to obtain a rendering for the
edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge Online Documentation}*/
export class SimpleEdge {
/**
Creates an edge with a {@link DefaultGraph#DEFAULT_EDGE_LOOKUP default lookup}, and the given source and target ports.
While technically the source and target port can be <code>null</code> here, this will cause all kinds of issues, e.g. with {@link IEdgeStyle styles} or
{@link ILabelModel label models} that require the ports to be non-<code>null</code> to work properly.
@param [sourcePort=null] The source port for the edge.
@param [targetPort=null] The target port for the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge Online Documentation}*/
constructor(sourcePort?: IPort | null, targetPort?: IPort | null);
/**
Creates an edge with a {@link DefaultGraph#DEFAULT_EDGE_LOOKUP default lookup}, and the given source and target ports.
While technically the source and target port can be <code>null</code> here, this will cause all kinds of issues, e.g. with {@link IEdgeStyle styles} or
{@link ILabelModel label models} that require the ports to be non-<code>null</code> to work properly.
@param options The parameters to pass.
@param [options.sourcePort=null] The source port for the edge.
@param [options.targetPort=null] The target port for the edge.
@param [options.ports] The collection of {@link IPort ports} that are owned by this instance. This option sets the {@link SimpleEdge#ports} property on the created object.
@param [options.labels] The collection of {@link ILabel labels} that are owned by this instance. This option sets the {@link SimpleEdge#labels} property on the created object.
@param [options.bends] The collection of {@link IBend bends} that describe the geometry of this edge. This option sets the {@link SimpleEdge#bends} property on the created object.
@param [options.style] The style that is responsible for the visual representation of this edge in a {@link CanvasComponent}. This option sets the {@link SimpleEdge#style} property on the created object.
@param [options.tag] Sets the tag property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge Online Documentation}*/
constructor(options: {
sourcePort?: IPort | null;
targetPort?: IPort | null;
ports?: IListEnumerable<IPort> | IListEnumerableConvertible<IPort>;
labels?: IListEnumerable<ILabel> | IListEnumerableConvertible<ILabel>;
bends?: IListEnumerable<IBend> | IListEnumerableConvertible<IBend>;
style?: IEdgeStyle;
tag?: any | null;
});
/**
Gets or sets the style that is responsible for the visual representation of this edge in a {@link CanvasComponent}.
Note that the style instance associated with an edge instance may be shared between multiple edge instances and that the
modification of this style will result in a change of the appearance of all edges that are associated with the same
style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge%23style Online Documentation}*/
style: IEdgeStyle;
/**
Gets or sets the collection of {@link IBend bends} that describe the geometry of this edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge%23bends Online Documentation}*/
bends: IListEnumerable<IBend>;
/**
Gets or sets the source port instance this edge is connected to.
Although the notion of source and target port is used for directed edges, it is still up to the client to decide whether
the edge should be treated as such.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge%23sourcePort Online Documentation}*/
sourcePort: IPort | null;
/**
Gets or sets the target port instance this edge is connected to.
Although the notion of source and target port is used for directed edges, it is still up to the client to decide whether
the edge should be treated as such.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge%23targetPort Online Documentation}*/
targetPort: IPort | null;
/**
Gets or sets the collection of {@link IPort ports} that are owned by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge%23ports Online Documentation}*/
ports: IListEnumerable<IPort>;
/**
Gets or sets the collection of {@link ILabel labels} that are owned by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge%23labels Online Documentation}*/
labels: IListEnumerable<ILabel>;
/**
Gets or sets the {@link ILookup} implementation used for calls to {@link SimpleEdge#lookup} on this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleEdge%23lookupImplementation Online Documentation}*/
lookupImplementation: ILookup | null;
static $class: Class;
private $_SimpleEdge: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SimpleEdge;
}
export interface DefaultEdgePathCropper extends YObject, IEdgePathCropper {
}
/**
The default implementation of the {@link IEdgePathCropper}.
@see IEdgePathCropper#INSTANCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper Online Documentation}*/
export class DefaultEdgePathCropper {
/**
Creates a new instance of {@link DefaultEdgePathCropper}.
@param [options] The parameters to pass.
@param [options.cropAtPort] The value that determines if the edge path is cropped at the port or at the node bounds. This option sets the {@link DefaultEdgePathCropper#cropAtPort} property on the created object.
@param [options.extraCropLength] An extra length the edge is cropped. This option sets the {@link DefaultEdgePathCropper#extraCropLength} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper Online Documentation}*/
constructor(options?: {
cropAtPort?: boolean;
extraCropLength?: number;
});
/**
Calculates the total length the edge path is cropped.
This method can be overridden to change the length that is cropped from an edge. The default implementation returns the
sum of {@link DefaultEdgePathCropper#extraCropLength}, {@link IArrow#length arrow length} and {@link IArrow#cropLength arrow crop length}
@param arrow The arrow at this edge end.
@param atSource <code>true</code> if the crop length should be calculated at the edge source. <code>false</code> otherwise.
@returns The total length the edge path is cropped.
@see DefaultEdgePathCropper#cropEdgePath
@see DefaultEdgePathCropper#cropEdgePathAtArrow
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23calculateTotalCropLength(IArrow%2Cboolean) Online Documentation}*/
calculateTotalCropLength(arrow: IArrow | null, atSource: boolean): number;
/**
Crops the provided <code>path</code> at one end of an edge.
If {@link DefaultEdgePathCropper#cropAtPort} is <code>true</code> this implementation delegates to {@link DefaultEdgePathCropper#cropEdgePathAtPortGeometry}. Otherwise
the edge path is cropped at the {@link DefaultEdgePathCropper#getNodeGeometry node geometry}. If this would result in an empty path, the cropping is delegated to {@link DefaultEdgePathCropper#handleEmptyPath}
instead.
@param edge The edge whose path is to be cropped.
@param atSource Whether to crop the source or target side of the path.
@param arrow The arrow that is used at the end of the edge.
@param path The path to crop.
@returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23cropEdgePath(IEdge%2Cboolean%2CIArrow%2CGeneralPath) Online Documentation}*/
cropEdgePath(edge: IEdge, atSource: boolean, arrow: IArrow | null, path: GeneralPath): GeneralPath;
/**
Crops an edge's path at the source or target side with respect to the given arrow.
The <code>path</code> is cropped by the length {@link DefaultEdgePathCropper#calculateTotalCropLength} returns for <code>arrow</code> and
<code>atSource</code>.
@param atSource if set to <code>true</code> the source side is cropped.
@param arrow The arrow to consider for the cropping.
@param path The edge's path to crop.
@returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23cropEdgePathAtArrow(boolean%2CIArrow%2CGeneralPath) Online Documentation}*/
cropEdgePathAtArrow(atSource: boolean, arrow: IArrow | null, path: GeneralPath): GeneralPath;
/**
Crops an edge's path at the source or target side at the {@link DefaultEdgePathCropper#getPortGeometry port geometry} with respect to the given arrow.
@param edge The edge whose path is to be cropped.
@param atSource Whether to crop the source or target side of the path.
@param arrow The arrow that is used at the end of the edge.
@param path The path to crop.
@returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23cropEdgePathAtPortGeometry(IEdge%2Cboolean%2CIArrow%2CGeneralPath) Online Documentation}*/
cropEdgePathAtPortGeometry(edge: IEdge, atSource: boolean, arrow: IArrow | null, path: GeneralPath): GeneralPath;
/**
Finds the intersection between a node and the edge.
@param node
@param nodeShapeGeometry
@param edge
@param inner
@param outer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23getIntersection(INode%2CIShapeGeometry%2CIEdge%2CPoint%2CPoint) Online Documentation}*/
getIntersection(node: INode, nodeShapeGeometry: IShapeGeometry, edge: IEdge, inner: Point, outer: Point): Point | null;
/**
Returns the {@link IShapeGeometry geometry} that is used to crop the edge at the node outline.
The default implementation queries the {@link INodeStyleRenderer#getShapeGeometry node style renderer} for an implementation of {@link IShapeGeometry}. This
method can be overridden to return an arbitrary {@link IShapeGeometry geometry} for a node.
@param node The node the edge should be cropped at.
@returns The {@link IShapeGeometry geometry} of the node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23getNodeGeometry(INode) Online Documentation}*/
getNodeGeometry(node: INode): IShapeGeometry | null;
/**
Returns the {@link IShapeGeometry geometry} that is used to crop the edge at the port outline if {@link DefaultEdgePathCropper#cropAtPort} is set to <code>true</code>.
The default implementation returns <code>null</code>. This method can be overridden to return an arbitrary {@link IShapeGeometry geometry} for a port.
@param port The port the edge should be cropped at.
@returns The {@link IShapeGeometry geometry} of the port.
@see DefaultEdgePathCropper#cropAtPort
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23getPortGeometry(IPort) Online Documentation}*/
getPortGeometry(port: IPort): IShapeGeometry | null;
/**
Handles the edge path cropping if {@link DefaultEdgePathCropper#cropEdgePath} would result in a {@link GeneralPath#clear cleared} path.
This method is called by {@link DefaultEdgePathCropper#cropEdgePath} if {@link DefaultEdgePathCropper#cropAtPort} is <code>false</code> and cropping the edge path at
the {@link DefaultEdgePathCropper#getNodeGeometry node geometry} would result in an empty path.
The default implementation is to {@link GeneralPath#clear} the edge path.
@param edge The edge whose path is to be cropped.
@param atSource Whether to crop the source or target side of the path.
@param arrow The arrow that is used at the end of the edge.
@param path The path to crop.
@returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23handleEmptyPath(IEdge%2Cboolean%2CIArrow%2CGeneralPath) Online Documentation}*/
handleEmptyPath(edge: IEdge, atSource: boolean, arrow: IArrow | null, path: GeneralPath): GeneralPath;
/**
Checks whether a given point is inside a node's shape geometry with respect to the edge that is being calculated.
@param location
@param node
@param nodeShapeGeometry
@param edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23isInside(Point%2CINode%2CIShapeGeometry%2CIEdge) Online Documentation}*/
isInside(location: Point, node: INode, nodeShapeGeometry: IShapeGeometry, edge: IEdge): boolean;
/**
Gets or sets an extra length the edge is cropped.
The default value is 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23extraCropLength Online Documentation}*/
extraCropLength: number;
/**
Gets the value that determines if the edge path is cropped at the port or at the node bounds.
The default is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultEdgePathCropper%23cropAtPort Online Documentation}*/
cropAtPort: boolean;
static $class: Class;
private $_DefaultEdgePathCropper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultEdgePathCropper;
}
export interface DefaultGraph extends YObject, IGraph {
/**
@inheritdoc*/
addBend(edge: IEdge, location: Point | PointConvertible, index?: number): IBend;
/**
@inheritdoc*/
addLabel(owner: ILabelOwner, text: string, layoutParameter?: ILabelModelParameter | null, style?: ILabelStyle | null, preferredSize?: Size | null | SizeConvertible, tag?: any | null): ILabel;
/**
@inheritdoc*/
addLabel(options: {
owner: ILabelOwner;
text: string;
layoutParameter?: ILabelModelParameter | null;
style?: ILabelStyle | null;
preferredSize?: Size | null | SizeConvertible;
tag?: any | null;
}): ILabel;
/**
@inheritdoc*/
createEdge(sourcePort: IPort, targetPort: IPort, style?: IEdgeStyle | null, tag?: any | null): IEdge;
/**
@inheritdoc*/
createEdge(options: {
sourcePort: IPort;
targetPort: IPort;
style?: IEdgeStyle | null;
tag?: any | null;
}): IEdge;
/**
Creates and returns an edge that connects to the given node instances using the given style instance.
The nodes must be part of this graph at the time of the invocation, and the implementation will choose the {@link IPort}
instances to which the edge will be connected. The edge will be a part of this graph after the method returns. This will
trigger the corresponding events.
@param source The source node the created edge will connect to. This implementation queries the {@link DefaultGraph#getNewSourcePort}
method to determine which port to use.
@param target The target node the created edge will connect to. This implementation queries the {@link DefaultGraph#getNewSourcePort}
method to determine which port to use.
@param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
@param [tag=null] the initial {@link ITagOwner#tag} to assign.
@returns the newly created edge instance
@throws {Exception} <b>(ArgumentError)</b> If <code>source</code> or <code>target</code> is not in this graph.
@see DefaultGraph#addEdgeCreatedListener
@see DefaultGraph#getNewSourcePort
@see DefaultGraph#getNewTargetPort
@see DefaultGraph#createEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createEdge(INode%2CINode%2CIEdgeStyle%2CObject) Online Documentation}*/
createEdge(source: INode, target: INode, style?: IEdgeStyle | null, tag?: any | null): IEdge;
/**
Creates and returns an edge that connects to the given node instances using the given style instance.
The nodes must be part of this graph at the time of the invocation, and the implementation will choose the {@link IPort}
instances to which the edge will be connected. The edge will be a part of this graph after the method returns. This will
trigger the corresponding events.
@param options The parameters to pass.
@param options.source The source node the created edge will connect to. This implementation queries the {@link DefaultGraph#getNewSourcePort}
method to determine which port to use.
@param options.target The target node the created edge will connect to. This implementation queries the {@link DefaultGraph#getNewSourcePort}
method to determine which port to use.
@param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
@param [options.tag=null] the initial {@link ITagOwner#tag} to assign.
@returns the newly created edge instance
@throws {Exception} <b>(ArgumentError)</b> If <code>source</code> or <code>target</code> is not in this graph.
@see DefaultGraph#addEdgeCreatedListener
@see DefaultGraph#getNewSourcePort
@see DefaultGraph#getNewTargetPort
@see DefaultGraph#createEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createEdge(INode%2CINode%2CIEdgeStyle%2CObject) Online Documentation}*/
createEdge(options: {
source: INode;
target: INode;
style?: IEdgeStyle | null;
tag?: any | null;
}): IEdge;
/**
@inheritdoc*/
createGroupNode(parent?: INode | null, layout?: Rect | null | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
@inheritdoc*/
createGroupNode(options: {
parent?: INode | null;
layout?: Rect | null | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
}): INode;
/**
@inheritdoc*/
createNode(layout: Rect | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
@inheritdoc*/
createNode(options: {
layout: Rect | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
}): INode;
/**
@inheritdoc*/
createNode(parent?: INode | null, layout?: Rect | null | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
@inheritdoc*/
createNode(options: {
parent?: INode | null;
layout?: Rect | null | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
}): INode;
/**
@inheritdoc*/
edgesAt(owner: IPortOwner, type?: AdjacencyTypes | AdjacencyTypesConvertible): IListEnumerable<IEdge>;
/**
@inheritdoc*/
edgesAt(port: IPort, type?: AdjacencyTypes | AdjacencyTypesConvertible): IListEnumerable<IEdge>;
/**
@inheritdoc*/
getChildren(node: INode | null): IListEnumerable<INode>;
/**
@inheritdoc*/
getParent(node: INode): INode | null;
/**
@inheritdoc*/
isGroupNode(node: INode | null): boolean;
/**
@inheritdoc*/
remove(item: IModelItem): void;
/**
@inheritdoc*/
setIsGroupNode(node: INode | null, isGroupNode: boolean): void;
/**
@inheritdoc*/
setLabelLayoutParameter(label: ILabel, layoutParameter: ILabelModelParameter): void;
/**
@inheritdoc*/
setLabelPreferredSize(label: ILabel, size: Size): void;
/**
@inheritdoc*/
setParent(node: INode, parent: INode | null): void;
/**
@inheritdoc*/
setPortLocationParameter(port: IPort, locationParameter: IPortLocationModelParameter): void;
/**
@inheritdoc*/
setStyle(port: IPort, style: IPortStyle): void;
/**
@inheritdoc*/
setStyle(edge: IEdge, style: IEdgeStyle): void;
/**
Assigns the given style instance by reference to the label.
Style instances can be shared.
This implementation will {@link IGraph#adjustLabelPreferredSize adjust the preferred size} for the label if {@link ILabelDefaults#autoAdjustPreferredSize} is enabled.
@param label The label that will be assigned the new style
@param style The style instance that will be assigned to the label.
@throws {Exception} <b>(ArgumentError)</b> If <code>label</code> is not in this graph.
@throws {Exception} <b>(ArgumentError)</b> If <code>style</code> is <code>null</code>.
@see ILabel#style
@see DefaultGraph#onChangingLabelStyle
@see DefaultGraph#addLabelStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23setStyle(ILabel%2CILabelStyle) Online Documentation}*/
setStyle(label: ILabel, style: ILabelStyle): void;
/**
Assigns the given style instance by reference to the node.
Style instances can be shared.
@param node The node that will be assigned the new style
@param style The style instance that will be assigned to the node.
@throws {Exception} <b>(ArgumentError)</b> If <code>node</code> is not in this graph.
@throws {Exception} <b>(ArgumentError)</b> If <code>style</code> is <code>null</code>.
@see INode#style
@see DefaultGraph#onChangingNodeStyle
@see DefaultGraph#addNodeStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23setStyle(INode%2CINodeStyle) Online Documentation}*/
setStyle(node: INode, style: INodeStyle): void;
/**
@inheritdoc*/
edgeDefaults: IEdgeDefaults;
/**
@inheritdoc*/
nodeDefaults: INodeDefaults;
/**
@inheritdoc*/
readonly nodes: IListEnumerable<INode>;
/**
@inheritdoc*/
readonly labels: IListEnumerable<ILabel>;
/**
@inheritdoc*/
readonly ports: IListEnumerable<IPort>;
/**
@inheritdoc*/
readonly edges: IListEnumerable<IEdge>;
/**
@inheritdoc*/
groupNodeDefaults: INodeDefaults;
/**
@inheritdoc*/
tag: any | null;
/**
@inheritdoc*/
addGraphTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IGraph, any>) => void): void;
/**
@inheritdoc*/
removeGraphTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IGraph, any>) => void): void;
/**
@inheritdoc*/
addNodeCreatedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
@inheritdoc*/
removeNodeCreatedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
@inheritdoc*/
addNodeRemovedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
removeNodeRemovedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
addNodeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, INodeStyle>) => void): void;
/**
@inheritdoc*/
removeNodeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, INodeStyle>) => void): void;
/**
@inheritdoc*/
addNodeLayoutChangedListener(listener: (source: any, node: INode, oldLayout: Rect) => void): void;
/**
@inheritdoc*/
removeNodeLayoutChangedListener(listener: (source: any, node: INode, oldLayout: Rect) => void): void;
/**
@inheritdoc*/
addNodeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, any>) => void): void;
/**
@inheritdoc*/
removeNodeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, any>) => void): void;
/**
@inheritdoc*/
addEdgeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IEdge>) => void): void;
/**
@inheritdoc*/
removeEdgeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IEdge>) => void): void;
/**
@inheritdoc*/
addEdgeRemovedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
@inheritdoc*/
removeEdgeRemovedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
@inheritdoc*/
addEdgeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, IEdgeStyle>) => void): void;
/**
@inheritdoc*/
removeEdgeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, IEdgeStyle>) => void): void;
/**
@inheritdoc*/
addEdgePortsChangedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
@inheritdoc*/
removeEdgePortsChangedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
@inheritdoc*/
addEdgeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, any>) => void): void;
/**
@inheritdoc*/
removeEdgeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, any>) => void): void;
/**
@inheritdoc*/
addBendAddedListener(listener: (sender: any, evt: ItemEventArgs<IBend>) => void): void;
/**
@inheritdoc*/
removeBendAddedListener(listener: (sender: any, evt: ItemEventArgs<IBend>) => void): void;
/**
@inheritdoc*/
addBendRemovedListener(listener: (sender: any, evt: BendEventArgs) => void): void;
/**
@inheritdoc*/
removeBendRemovedListener(listener: (sender: any, evt: BendEventArgs) => void): void;
/**
@inheritdoc*/
addBendLocationChangedListener(listener: (source: any, bend: IBend, oldLocation: Point) => void): void;
/**
@inheritdoc*/
removeBendLocationChangedListener(listener: (source: any, bend: IBend, oldLocation: Point) => void): void;
/**
@inheritdoc*/
addBendTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IBend, any>) => void): void;
/**
@inheritdoc*/
removeBendTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IBend, any>) => void): void;
/**
@inheritdoc*/
addLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
@inheritdoc*/
removeLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
@inheritdoc*/
addLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
@inheritdoc*/
removeLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
@inheritdoc*/
addLabelStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelStyle>) => void): void;
/**
@inheritdoc*/
removeLabelStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelStyle>) => void): void;
/**
@inheritdoc*/
addLabelPreferredSizeChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, Size>) => void): void;
/**
@inheritdoc*/
removeLabelPreferredSizeChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, Size>) => void): void;
/**
@inheritdoc*/
addLabelTextChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, string>) => void): void;
/**
@inheritdoc*/
removeLabelTextChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, string>) => void): void;
/**
@inheritdoc*/
addLabelLayoutParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelModelParameter>) => void): void;
/**
@inheritdoc*/
removeLabelLayoutParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelModelParameter>) => void): void;
/**
@inheritdoc*/
addLabelTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, any>) => void): void;
/**
@inheritdoc*/
removeLabelTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, any>) => void): void;
/**
@inheritdoc*/
addPortAddedListener(listener: (sender: any, evt: ItemEventArgs<IPort>) => void): void;
/**
@inheritdoc*/
removePortAddedListener(listener: (sender: any, evt: ItemEventArgs<IPort>) => void): void;
/**
@inheritdoc*/
addPortRemovedListener(listener: (sender: any, evt: PortEventArgs) => void): void;
/**
@inheritdoc*/
removePortRemovedListener(listener: (sender: any, evt: PortEventArgs) => void): void;
/**
@inheritdoc*/
addPortStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortStyle>) => void): void;
/**
@inheritdoc*/
removePortStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortStyle>) => void): void;
/**
@inheritdoc*/
addPortLocationParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortLocationModelParameter>) => void): void;
/**
@inheritdoc*/
removePortLocationParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortLocationModelParameter>) => void): void;
/**
@inheritdoc*/
addPortTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, any>) => void): void;
/**
@inheritdoc*/
removePortTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, any>) => void): void;
/**
@inheritdoc*/
addDisplaysInvalidatedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
@inheritdoc*/
removeDisplaysInvalidatedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
@inheritdoc*/
addParentChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
removeParentChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
addIsGroupNodeChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
removeIsGroupNodeChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
} /**
Central implementation of the {@link IGraph} interface.
This implementation deals with the proper firing of events and handling of default values.
This implementation has inherent support for undoability. If the {@link DefaultGraph#undoEngineEnabled} property is set to <code>true</code>, changes to the structure of
the graph will automatically be recorded using the {@link UndoEngine} instance that can be queried from this
instance's {@link DefaultGraph#lookup} method. Clients that wish to enqueue custom undo events or that need to make
non-structural changes to the graph can use the {@link UndoEngine} to do so. The current instance of {@link UndoEngine}
can be queried using the {@link ILookup#lookup} method of this instance. Also there are several protected methods <code>Create...UndoUnit</code>
that will be used by this instance to create the actual undo units for structural changes in the graph's structure. If
clients need to customize these units they can create a subclass of {@link DefaultGraph} and override these
methods.
Finally this implementation supports grouped graphs.
<code>DefaultGraph</code> is {@link IGraph}'s default implementation as described in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph_model-graph The Graph Model - IGraph}.
@see DefaultGraph#createUndoEngine
@see DefaultGraph#lookup
@see DefaultGraph
@see FoldingManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph Online Documentation}*/
export class DefaultGraph {
/**
Creates a new instance of a graph that provides an {@link ILookupDecorator} for all of its entities.
@param [options] The parameters to pass.
@param options.nodes An array of nodes to create in this graph. Each item will be passed to the {@link DefaultGraph#createNode} method.
@param [options.edgeDefaults] Sets the edgeDefaults property.
@param [options.groupNodeDefaults] Sets the groupNodeDefaults property.
@param [options.mapperRegistry] The mapper registry that is associated with this graph instance. This option sets the {@link DefaultGraph#mapperRegistry} property on the created object.
@param [options.nodeDefaults] Sets the nodeDefaults property.
@param [options.undoEngineEnabled] Whether or not the {@link UndoEngine} used for this instance should be enabled. This option sets the {@link DefaultGraph#undoEngineEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph Online Documentation}*/
constructor(options?: {
edgeDefaults?: IEdgeDefaults;
groupNodeDefaults?: INodeDefaults;
mapperRegistry?: IMapperRegistry;
nodeDefaults?: INodeDefaults;
undoEngineEnabled?: boolean;
nodes?: Array<any>;
});
/**
Adds the chain element to the lookup for this instance.
@param lookup The lookup to decorate the current instance with.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23addLookup(IContextLookupChainLink) Online Documentation}*/
addLookup(lookup: IContextLookupChainLink): void;
/**
Add a port to the given port owner using the coordinates as the new initial position of the port anchor.
@param owner the owner to add the port instance to.
@param [locationParameter=null] the parameter to use for the port to determine its location.
@param [style=null] the style to assign initially to the port, e.g. {@link VoidPortStyle#INSTANCE}.
@param [tag=null] The tag to associate with the port, may be <code>null</code>.
@returns the newly created port
@throws {Exception} <b>(NotSupportedError)</b> If this instance cannot add a port to <code>owner</code>.
@throws {Exception} <b>(ArgumentError)</b> If <code>owner</code> is not in this graph.
@see DefaultGraph#addPortAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23addPort(IPortOwner%2CIPortLocationModelParameter%2CIPortStyle%2CObject) Online Documentation}*/
addPort(owner: IPortOwner, locationParameter?: IPortLocationModelParameter | null, style?: IPortStyle | null, tag?: any | null): IPort;
/**
Add a port to the given port owner using the coordinates as the new initial position of the port anchor.
@param options The parameters to pass.
@param options.owner the owner to add the port instance to.
@param [options.locationParameter=null] the parameter to use for the port to determine its location.
@param [options.style=null] the style to assign initially to the port, e.g. {@link VoidPortStyle#INSTANCE}.
@param [options.tag=null] The tag to associate with the port, may be <code>null</code>.
@returns the newly created port
@throws {Exception} <b>(NotSupportedError)</b> If this instance cannot add a port to <code>owner</code>.
@throws {Exception} <b>(ArgumentError)</b> If <code>owner</code> is not in this graph.
@see DefaultGraph#addPortAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23addPort(IPortOwner%2CIPortLocationModelParameter%2CIPortStyle%2CObject) Online Documentation}*/
addPort(options: {
owner: IPortOwner;
locationParameter?: IPortLocationModelParameter | null;
style?: IPortStyle | null;
tag?: any | null;
}): IPort;
/**
Determines whether this graph contains the specified item.
@param item The item.
@returns <code>true</code> if this graph contains the specified item; otherwise, <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23contains(IModelItem) Online Documentation}*/
contains(item: IModelItem | null): boolean;
/**
Creates an {@link UndoEngine} instance that automatically triggers {@link DefaultGraph#invalidateDisplays}
upon each invocation of {@link UndoEngine#undo} and {@link UndoEngine#redo}
@returns An {@link UndoEngine} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoEngine() Online Documentation}*/
createUndoEngine(): UndoEngine;
/**
Creates the bend addition {@link IUndoUnit undo unit} for the given bend.
@param bend The bend that has been added.
@param index The index at which the bend has been added.
@returns The unit that can be used to undo and redo the creation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForBendAddition(IBend%2Cnumber) Online Documentation}*/
createUndoUnitForBendAddition(bend: IBend, index: number): IUndoUnit;
/**
Creates the bend removal {@link IUndoUnit undo unit} for the given bend.
@param bend The bend that will be removed.
@returns The unit that can be used to undo and redo the removal.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForBendRemoval(IBend) Online Documentation}*/
createUndoUnitForBendRemoval(bend: IBend): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the tag of the given bend.
@param bend The bend whose tag has been changed.
@param oldTag The old tag used before the change.
@returns The unit that can be used to undo and redo the tag change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForBendTagChange(IBend%2CObject) Online Documentation}*/
createUndoUnitForBendTagChange(bend: IBend, oldTag: any | null): IUndoUnit;
/**
Creates the edge creation {@link IUndoUnit undo unit} for the given edge.
@param edge The edge that has been created.
@returns The unit that can be used to undo and redo the creation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForEdgeCreation(IEdge) Online Documentation}*/
createUndoUnitForEdgeCreation(edge: IEdge): IUndoUnit;
/**
Creates the edge reconnection {@link IUndoUnit undo unit} for the given edge.
@param edge The edge that has been reconnected to other ports.
@param oldSource The old source port the edge connected to before the change.
@param oldTarget The old target port the edge connected to before the change.
@returns The unit that can be used to undo and redo the port change operation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForEdgeReconnection(IEdge%2CIPort%2CIPort) Online Documentation}*/
createUndoUnitForEdgeReconnection(edge: IEdge, oldSource: IPort, oldTarget: IPort): IUndoUnit;
/**
Creates the edge removal {@link IUndoUnit undo unit} for the given edge.
@param edge The edge that will be removed.
@returns The unit that can be used to undo and redo the removal.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForEdgeRemoval(IEdge) Online Documentation}*/
createUndoUnitForEdgeRemoval(edge: IEdge): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the style of the given edge.
@param edge The edge whose style has been changed.
@param oldStyle The old style used before the change.
@returns The unit that can be used to undo and redo the style change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForEdgeStyleChange(IEdge%2CIEdgeStyle) Online Documentation}*/
createUndoUnitForEdgeStyleChange(edge: IEdge, oldStyle: IEdgeStyle): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the tag of the given edge.
@param edge The edge whose tag has been changed.
@param oldTag The old tag used before the change.
@returns The unit that can be used to undo and redo the tag change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForEdgeTagChange(IEdge%2CObject) Online Documentation}*/
createUndoUnitForEdgeTagChange(edge: IEdge, oldTag: any | null): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the tag of the given graph.
@param tagOwner The graph whose tag has been changed.
@param oldTag The old tag used before the change.
@returns The unit that can be used to undo and redo the tag change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForGraphTagChange(ITagOwner%2CObject) Online Documentation}*/
createUndoUnitForGraphTagChange(tagOwner: ITagOwner, oldTag: any | null): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing whether the given node is a group node.
@param node The node whose group node status has been changed.
@param isGroupNode The new group node status.
@returns The unit that can be used to undo and redo the group node status change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForIsGroupNodeChange(INode%2Cboolean) Online Documentation}*/
createUndoUnitForIsGroupNodeChange(node: INode, isGroupNode: boolean): IUndoUnit;
/**
Creates the label addition {@link IUndoUnit undo unit} for the given label.
@param label The label that has been added.
@returns The unit that can be used to undo and redo the creation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForLabelAddition(ILabel) Online Documentation}*/
createUndoUnitForLabelAddition(label: ILabel): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the layout parameter of the given label.
@param label The label whose layout parameter has been changed.
@param oldParam The old layout parameter used before the change.
@returns The unit that can be used to undo and redo the layout parameter change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForLabelModelParameterChange(ILabel%2CILabelModelParameter) Online Documentation}*/
createUndoUnitForLabelModelParameterChange(label: ILabel, oldParam: ILabelModelParameter): IUndoUnit;
/**
Creates the label removal {@link IUndoUnit undo unit} for the given label.
@param label The label that will be removed.
@returns The unit that can be used to undo and redo the removal.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForLabelRemoval(ILabel) Online Documentation}*/
createUndoUnitForLabelRemoval(label: ILabel): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the style of the given label.
@param label The label whose style has been changed.
@param oldStyle The old style used before the change.
@returns The unit that can be used to undo and redo the style change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForLabelStyleChange(ILabel%2CILabelStyle) Online Documentation}*/
createUndoUnitForLabelStyleChange(label: ILabel, oldStyle: ILabelStyle): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the tag of the given label.
@param label The label whose tag has been changed.
@param oldTag The old tag used before the change.
@returns The unit that can be used to undo and redo the tag change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForLabelTagChange(ILabel%2CObject) Online Documentation}*/
createUndoUnitForLabelTagChange(label: ILabel, oldTag: any | null): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the text of the given label.
@param label The label whose text has been changed.
@param oldText The old text used before the change.
@returns The unit that can be used to undo and redo the text change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForLabelTextChange(ILabel%2Cstring) Online Documentation}*/
createUndoUnitForLabelTextChange(label: ILabel, oldText: string): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the location parameter of the given port.
@param port The port whose location parameter has been changed.
@param oldParameter The old location parameter used before the change.
@returns The unit that can be used to undo and redo the location parameter change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForLocationParameterChange(IPort%2CIPortLocationModelParameter) Online Documentation}*/
createUndoUnitForLocationParameterChange(port: IPort, oldParameter: IPortLocationModelParameter): IUndoUnit;
/**
Creates the node creation {@link IUndoUnit undo unit} for the given node.
@param node The node that has been created.
@param parent
@param isGroupNode
@returns The unit that can be used to undo and redo the creation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForNodeCreation(INode%2CINode%2Cboolean) Online Documentation}*/
createUndoUnitForNodeCreation(node: INode, parent: INode | null, isGroupNode: boolean): IUndoUnit;
/**
Creates the node removal {@link IUndoUnit undo unit} for the given node.
@param node The node that will be removed.
@returns The unit that can be used to undo and redo the removal.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForNodeRemoval(INode) Online Documentation}*/
createUndoUnitForNodeRemoval(node: INode): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the style of the given node.
@param node The node whose style has been changed.
@param oldStyle The old style used before the change.
@returns The unit that can be used to undo and redo the style change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForNodeStyleChange(INode%2CINodeStyle) Online Documentation}*/
createUndoUnitForNodeStyleChange(node: INode, oldStyle: INodeStyle): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the tag of the given node.
@param node The node whose tag has been changed.
@param oldTag The old tag used before the change.
@returns The unit that can be used to undo and redo the tag change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForNodeTagChange(INode%2CObject) Online Documentation}*/
createUndoUnitForNodeTagChange(node: INode, oldTag: any | null): IUndoUnit;
/**
Creates the parent change {@link IUndoUnit undo unit} for the given node.
@param node The node whose parent has been changed.
@param oldParent
@param newParent The node which is the new parent.
@returns The unit that can be used to undo and redo the parent change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForParentChange(INode%2CINode%2CINode) Online Documentation}*/
createUndoUnitForParentChange(node: INode, oldParent: INode | null, newParent: INode | null): IUndoUnit;
/**
Creates the port addition {@link IUndoUnit undo unit} for the given port.
@param port The port that has been added.
@returns The unit that can be used to undo and redo the creation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForPortAddition(IPort) Online Documentation}*/
createUndoUnitForPortAddition(port: IPort): IUndoUnit;
/**
Creates the port removal {@link IUndoUnit undo unit} for the given port.
@param port The port that will be removed.
@returns The unit that can be used to undo and redo the removal.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForPortRemoval(IPort) Online Documentation}*/
createUndoUnitForPortRemoval(port: IPort): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the style of the given port.
@param port The port whose style has been changed.
@param oldStyle The old style used before the change.
@returns The unit that can be used to undo and redo the style change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForPortStyleChange(IPort%2CIPortStyle) Online Documentation}*/
createUndoUnitForPortStyleChange(port: IPort, oldStyle: IPortStyle): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the tag of the given port.
@param port The port whose tag has been changed.
@param oldTag The old tag used before the change.
@returns The unit that can be used to undo and redo the tag change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForPortTagChange(IPort%2CObject) Online Documentation}*/
createUndoUnitForPortTagChange(port: IPort, oldTag: any | null): IUndoUnit;
/**
Creates the {@link IUndoUnit undo unit} for changing the preferred size of the given label.
@param label The label whose preferred size has been changed.
@param oldSize The old preferred size used before the change.
@returns The unit that can be used to undo and redo the preferred size change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23createUndoUnitForPreferredSizeChange(ILabel%2CSize) Online Documentation}*/
createUndoUnitForPreferredSizeChange(label: ILabel, oldSize: Size): IUndoUnit;
/**
Returns the lookup implementation that is used for {@link DefaultGraph#lookup} calls.
@returns The replacing lookup instance or <code>null</code> if the internal lookup mechanism is used.
@see DefaultGraph#setLookupImplementation
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23getLookup() Online Documentation}*/
getLookup(): ILookup | null;
/**
Determines a {@link IPort} instance to use for the creation of a new edge that starts at the given
{@link IPortOwner}.
This implementation will {@link DefaultGraph#addPort add a new center anchored port} to the node.
@param source The source node to find a port for.
@returns The port to use for a newly created edge.
@see DefaultGraph#createEdge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23getNewSourcePort(IPortOwner) Online Documentation}*/
getNewSourcePort(source: IPortOwner): IPort;
/**
Determines a {@link IPort} instance to use for the creation of a new edge that ends at the given
{@link IPortOwner}.
This implementation will {@link DefaultGraph#addPort add a new center anchored port} to the node.
@param targetOwner The target node to find a port for.
@returns The port to use for a newly created edge.
@see DefaultGraph#createEdge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23getNewTargetPort(IPortOwner) Online Documentation}*/
getNewTargetPort(targetOwner: IPortOwner): IPort;
/**
The last element in the lookup chain is implemented by this method.
@param type
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23innerLookup(Class) Online Documentation}*/
innerLookup(type: Class): any | null;
/**
Triggers the {@link DefaultGraph#addDisplaysInvalidatedListener DisplaysInvalidated} event.
Clients can call this method to force an invalidation of the displays that show this graph instance. This implementation
delegates to {@link DefaultGraph#onInvalidateDisplays}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23invalidateDisplays() Online Documentation}*/
invalidateDisplays(): void;
/**
Returns an instance that provides another aspect of this instance of the given type or <code>null</code>.
Typically, this method will be called in order to obtain a different view or aspect of the current instance. This is
quite similar to casting or using a super type or interface of this instance, but is not limited to inheritance or
compile time constraints. This method is not required to return non-<code>null</code> implementations for the types, nor does it have
to return the same instance any time. Also it depends on the type and context whether the instance returned stays up to
date or needs to be re-obtained for subsequent use. This implementation uses the {@link DefaultGraph#setLookupImplementation set lookup implementation} if it has
been set or an internal {@link LookupChain lookup chain}, that can be customized using the {@link DefaultGraph#addLookup} and {@link DefaultGraph#removeLookup}
methods.
@param type the type for which an instance shall be returned
@returns An instance that is assignable to type or <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Called before a bend is added to this graph's structure.
@param edge The edge the bend will be added to.
@param bend The bend that will be added to the edge.
@param index The index where the bend will be added to the edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onAddingBend(IEdge%2CIBend%2Cnumber) Online Documentation}*/
onAddingBend(edge: IEdge, bend: IBend, index: number): void;
/**
Called just before an edge label is added to an edge.
@param edge
@param label
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onAddingEdgeLabel(IEdge%2CILabel) Online Documentation}*/
onAddingEdgeLabel(edge: IEdge, label: ILabel): void;
/**
Called just before a node label is added to a node.
@param node
@param label
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onAddingNodeLabel(INode%2CILabel) Online Documentation}*/
onAddingNodeLabel(node: INode, label: ILabel): void;
/**
Called when a port is going to be added to a node.
@param owner The node.
@param port The port to be added.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onAddingPort(IPortOwner%2CIPort) Online Documentation}*/
onAddingPort(owner: IPortOwner, port: IPort): void;
/**
Called just before a port label is added to a port.
@param port
@param label
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onAddingPortLabel(IPort%2CILabel) Online Documentation}*/
onAddingPortLabel(port: IPort, label: ILabel): void;
/**
Called after a bend has been added to this graph's structure.
When overriding this method it is mandatory to call the base implementation.
@param bend The bend that has been added to the edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onBendAdded(IBend) Online Documentation}*/
onBendAdded(bend: IBend): void;
/**
Called after a bend location has changed.
This will trigger the {@link DefaultGraph#addBendLocationChangedListener BendLocationChanged} event.
@param bend The bend whose location has changed.
@param oldLocation The bend location before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onBendLocationChanged(IBend%2CPoint) Online Documentation}*/
onBendLocationChanged(bend: IBend, oldLocation: Point): void;
/**
Called just after a bend has been removed from its edge.
When overriding this method it is mandatory to call the base implementation.
@param owner The old edge.
@param bend The bend.
@param index The former index of the bend in the {@link IEdge#bends} list.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onBendRemoved(IEdge%2CIBend%2Cnumber) Online Documentation}*/
onBendRemoved(owner: IEdge, bend: IBend, index: number): void;
/**
Called after a bend tag has changed.
This will trigger the {@link DefaultGraph#addBendTagChangedListener BendTagChanged} event.
@param bend The bend whose tag has changed.
@param oldTag The bend tag before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onBendTagChanged(IBend%2CObject) Online Documentation}*/
onBendTagChanged(bend: IBend, oldTag: any | null): void;
/**
Called before the ports of an edge are being changed.
@param edge An edge whose the ports are to be changed.
@param newSource The new source port
@param newTarget The new target port
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingEdgePorts(IEdge%2CIPort%2CIPort) Online Documentation}*/
onChangingEdgePorts(edge: IEdge, newSource: IPort, newTarget: IPort): void;
/**
Called before an edge style is being changed.
@param edge An edge whose the style is going to be changed.
@param newStyle The new edge style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingEdgeStyle(IEdge%2CIEdgeStyle) Online Documentation}*/
onChangingEdgeStyle(edge: IEdge, newStyle: IEdgeStyle): void;
/**
Called before the graph's tag is being changed.
@param newTag The new port tag
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingGraphTag(Object) Online Documentation}*/
onChangingGraphTag(newTag: any | null): void;
/**
Called before the {@link ILabel#layoutParameter} of a label is being changed.
@param label A label whose {@link ILabel#layoutParameter} is to be changed.
@param newLayoutParameter The new label model parameter
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingLabelLayoutParameter(ILabel%2CILabelModelParameter) Online Documentation}*/
onChangingLabelLayoutParameter(label: ILabel, newLayoutParameter: ILabelModelParameter): void;
/**
Called before the preferred size of a label is being changed.
@param label A label whose preferred size is going to be changed.
@param newSize The new preferred size
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingLabelPreferredSize(ILabel%2CSize) Online Documentation}*/
onChangingLabelPreferredSize(label: ILabel, newSize: Size): void;
/**
Called before a label style is being changed.
@param label A label whose the style is going to be changed.
@param newStyle The new label style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingLabelStyle(ILabel%2CILabelStyle) Online Documentation}*/
onChangingLabelStyle(label: ILabel, newStyle: ILabelStyle): void;
/**
Called before the text of a label is being changed.
@param label A label whose text is going to be changed.
@param newText The new label text
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingLabelText(ILabel%2Cstring) Online Documentation}*/
onChangingLabelText(label: ILabel, newText: string): void;
/**
Called before a node style is being changed.
@param node A node whose the style is going to be changed.
@param newStyle The new node style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingNodeStyle(INode%2CINodeStyle) Online Documentation}*/
onChangingNodeStyle(node: INode, newStyle: INodeStyle): void;
/**
Called before the {@link IPort#locationParameter} of a port is being changed.
@param port A port whose {@link IPort#locationParameter} is to be changed.
@param newLocationParameter The new location model parameter
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingPortLocationModelParameter(IPort%2CIPortLocationModelParameter) Online Documentation}*/
onChangingPortLocationModelParameter(port: IPort, newLocationParameter: IPortLocationModelParameter): void;
/**
Called before a port style is being changed.
@param port A port whose the style is going to be changed.
@param newStyle The new port style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onChangingPortStyle(IPort%2CIPortStyle) Online Documentation}*/
onChangingPortStyle(port: IPort, newStyle: IPortStyle): void;
/**
Called before the edge is added to this graph's structure.
@param edge An edge that is not yet contained in this graph.
@param sourcePort The source port this edge will connect to.
@param targetPort The target port this edge will connect to.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onCreatingEdge(IEdge%2CIPort%2CIPort) Online Documentation}*/
onCreatingEdge(edge: IEdge, sourcePort: IPort, targetPort: IPort): void;
/**
Called before the node is added to this graph's structure.
@param node A node that is not yet contained in this graph.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onCreatingNode(INode) Online Documentation}*/
onCreatingNode(node: INode): void;
/**
Triggers the {@link DefaultGraph#addEdgeCreatedListener EdgeCreated} event.
@param edge The edge that has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onEdgeCreated(IEdge) Online Documentation}*/
onEdgeCreated(edge: IEdge): void;
/**
Called after a label has been added to an edge.
This method triggers the {@link DefaultGraph#addLabelAddedListener LabelAdded} event.
When overriding this method it is mandatory to call the base implementation.
@param label The label that has just been added.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onEdgeLabelAdded(ILabel) Online Documentation}*/
onEdgeLabelAdded(label: ILabel): void;
/**
Called after an edge label has been removed from its edge.
When overriding this method it is mandatory to call the base implementation.
@param label The label that has just been removed.
@param owner The previous owner of the label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onEdgeLabelRemoved(ILabelOwner%2CILabel) Online Documentation}*/
onEdgeLabelRemoved(owner: ILabelOwner, label: ILabel): void;
/**
Called after the ports of an edge has changed.
This will trigger the {@link DefaultGraph#addEdgePortsChangedListener EdgePortsChanged} event.
@param edge The edge whose ports have changed.
@param oldSource The source port that the edge had been connected to before the change.
@param oldTarget The target port that the edge had been connected to before the change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onEdgePortsChanged(IEdge%2CIPort%2CIPort) Online Documentation}*/
onEdgePortsChanged(edge: IEdge, oldSource: IPort, oldTarget: IPort): void;
/**
Triggers the {@link DefaultGraph#addEdgeRemovedListener EdgeRemoved} event.
@param edge The edge that got removed
@param oldSource The source port that the edge had been connected to.
@param oldTarget The target port that the edge had been connected to.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onEdgeRemoved(IEdge%2CIPort%2CIPort) Online Documentation}*/
onEdgeRemoved(edge: IEdge, oldSource: IPort, oldTarget: IPort): void;
/**
Called after the style of an edge has changed.
This will trigger the {@link DefaultGraph#addEdgeStyleChangedListener EdgeStyleChanged} event.
@param edge The edge whose style that has changed.
@param oldStyle The edge style before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onEdgeStyleChanged(IEdge%2CIEdgeStyle) Online Documentation}*/
onEdgeStyleChanged(edge: IEdge, oldStyle: IEdgeStyle): void;
/**
Called after the tag of an edge has changed.
This will trigger the {@link DefaultGraph#addEdgeTagChangedListener EdgeTagChanged} event.
@param edge The edge whose tag has changed.
@param oldTag The edge tag before the change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onEdgeTagChanged(IEdge%2CObject) Online Documentation}*/
onEdgeTagChanged(edge: IEdge, oldTag: any | null): void;
/**
Triggers the {@link DefaultGraph#addGraphTagChangedListener GraphTagChanged} event.
@param oldTag The graph tag before the change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onGraphTagChanged(Object) Online Documentation}*/
onGraphTagChanged(oldTag: any | null): void;
/**
Triggers the {@link DefaultGraph#addDisplaysInvalidatedListener DisplaysInvalidated} event.
@param evt The event arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onInvalidateDisplays(EventArgs) Online Documentation}*/
onInvalidateDisplays(evt: EventArgs): void;
/**
Called after a label model parameter has changed.
This will trigger the {@link DefaultGraph#addLabelLayoutParameterChangedListener LabelLayoutParameterChanged} event.
@param label The label whose model parameter has changed.
@param oldLayoutParamater The label model parameter before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onLabelLayoutParameterChanged(ILabel%2CILabelModelParameter) Online Documentation}*/
onLabelLayoutParameterChanged(label: ILabel, oldLayoutParamater: ILabelModelParameter): void;
/**
Called after the preferred size of a label has changed.
This will trigger the {@link DefaultGraph#addLabelPreferredSizeChangedListener LabelPreferredSizeChanged} event.
@param label The label whose preferred size has changed.
@param oldSize The preferred size of the label before the change
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onLabelPreferredSizeChanged(ILabel%2CSize) Online Documentation}*/
onLabelPreferredSizeChanged(label: ILabel, oldSize: Size): void;
/**
Called after a label style has changed.
This will trigger the {@link DefaultGraph#addLabelStyleChangedListener LabelStyleChanged} event.
@param label The label whose style has changed.
@param oldStyle The label style before the change.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onLabelStyleChanged(ILabel%2CILabelStyle) Online Documentation}*/
onLabelStyleChanged(label: ILabel, oldStyle: ILabelStyle): void;
/**
Called after a label tag has changed.
This will trigger the {@link DefaultGraph#addLabelTagChangedListener LabelTagChanged} event.
@param label The label whose tag has changed.
@param oldTag The label tag before the change
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onLabelTagChanged(ILabel%2CObject) Online Documentation}*/
onLabelTagChanged(label: ILabel, oldTag: any | null): void;
/**
Called after a label text has changed.
This will trigger the {@link DefaultGraph#addLabelTextChangedListener LabelTextChanged} event.
@param label The label whose text has changed.
@param oldText The label text before the change.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onLabelTextChanged(ILabel%2Cstring) Online Documentation}*/
onLabelTextChanged(label: ILabel, oldText: string): void;
/**
Triggers the {@link DefaultGraph#addNodeCreatedListener NodeCreated} event.
@param node The node that has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onNodeCreated(INode) Online Documentation}*/
onNodeCreated(node: INode): void;
/**
Called after a label has been added to a node.
This method triggers the {@link DefaultGraph#addLabelAddedListener LabelAdded} event.
When overriding this method it is mandatory to call the base implementation.
@param label The label that has just been added.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onNodeLabelAdded(ILabel) Online Documentation}*/
onNodeLabelAdded(label: ILabel): void;
/**
Called after a label has been removed from its node.
When overriding this method it is mandatory to call the base implementation.
@param label The label that has just been removed.
@param owner The old owner of the label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onNodeLabelRemoved(ILabelOwner%2CILabel) Online Documentation}*/
onNodeLabelRemoved(owner: ILabelOwner, label: ILabel): void;
/**
Called after a node layout has changed.
This will trigger the {@link DefaultGraph#addNodeLayoutChangedListener NodeLayoutChanged} event.
@param node The node whose layout has changed.
@param oldLayout The node layout before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onNodeLayoutChanged(INode%2CRect) Online Documentation}*/
onNodeLayoutChanged(node: INode, oldLayout: Rect): void;
/**
Triggers the {@link DefaultGraph#addNodeRemovedListener NodeRemoved} event.
@param node The node that got removed
@param oldParent
@param oldIsGroupNode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onNodeRemoved(INode%2CINode%2Cboolean) Online Documentation}*/
onNodeRemoved(node: INode, oldParent: INode | null, oldIsGroupNode: boolean): void;
/**
Called after a node style has changed.
This will trigger the {@link DefaultGraph#addNodeStyleChangedListener NodeStyleChanged} event.
@param node The node whose style has changed.
@param oldStyle The node style before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onNodeStyleChanged(INode%2CINodeStyle) Online Documentation}*/
onNodeStyleChanged(node: INode, oldStyle: INodeStyle): void;
/**
Called after a node tag has changed.
This will trigger the {@link DefaultGraph#addNodeTagChangedListener NodeTagChanged} event.
@param node The node whose tag has changed.
@param oldTag The node tag before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onNodeTagChanged(INode%2CObject) Online Documentation}*/
onNodeTagChanged(node: INode, oldTag: any | null): void;
/**
Called when a port has been added to a node.
When overriding this method it is mandatory to call the base implementation.
@param port The port that has just been added to its owner.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onPortAdded(IPort) Online Documentation}*/
onPortAdded(port: IPort): void;
/**
Called after a label has been added to a port.
This method triggers the {@link DefaultGraph#addLabelAddedListener LabelAdded} event.
When overriding this method it is mandatory to call the base implementation.
@param label The label that has just been added.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onPortLabelAdded(ILabel) Online Documentation}*/
onPortLabelAdded(label: ILabel): void;
/**
Called after a port label has been removed from its port.
When overriding this method it is mandatory to call the base implementation.
@param label The label that has just been removed.
@param owner The old owner of the label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onPortLabelRemoved(ILabelOwner%2CILabel) Online Documentation}*/
onPortLabelRemoved(owner: ILabelOwner, label: ILabel): void;
/**
Called after a port location model parameter has changed.
This will trigger the {@link DefaultGraph#addPortLocationParameterChangedListener PortLocationParameterChanged}
event.
@param port The port whose location model parameter has changed.
@param oldLocationParameter The port location model parameter before the change
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onPortLocationParameterChanged(IPort%2CIPortLocationModelParameter) Online Documentation}*/
onPortLocationParameterChanged(port: IPort, oldLocationParameter: IPortLocationModelParameter): void;
/**
Called just after a port has been removed from its owner.
When overriding this method it is mandatory to call the base implementation.
@param port The port that has been removed.
@param oldOwner The previous owner of the port.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onPortRemoved(IPortOwner%2CIPort) Online Documentation}*/
onPortRemoved(oldOwner: IPortOwner, port: IPort): void;
/**
Called after a port style has changed.
This will trigger the {@link DefaultGraph#addPortStyleChangedListener PortStyleChanged} event.
@param port The port whose style has changed.
@param oldStyle The port style before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onPortStyleChanged(IPort%2CIPortStyle) Online Documentation}*/
onPortStyleChanged(port: IPort, oldStyle: IPortStyle): void;
/**
Called after a port tag has changed.
This will trigger the {@link DefaultGraph#addPortTagChangedListener PortTagChanged} event.
@param port The port whose tag has changed.
@param oldTag The port tag before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onPortTagChanged(IPort%2CObject) Online Documentation}*/
onPortTagChanged(port: IPort, oldTag: any | null): void;
/**
Called just before a bend is removed.
@param bend The bend that will be removed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onRemovingBend(IBend) Online Documentation}*/
onRemovingBend(bend: IBend): void;
/**
Called before the node will be removed.
@param edge That edge that is going to be removed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onRemovingEdge(IEdge) Online Documentation}*/
onRemovingEdge(edge: IEdge): void;
/**
Called just before an edge label is removed from its edge.
@param label
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onRemovingEdgeLabel(ILabel) Online Documentation}*/
onRemovingEdgeLabel(label: ILabel): void;
/**
Called before the node will be removed.
@param node That node that is going to be removed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onRemovingNode(INode) Online Documentation}*/
onRemovingNode(node: INode): void;
/**
Called just before a node label is removed from its node.
@param label
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onRemovingNodeLabel(ILabel) Online Documentation}*/
onRemovingNodeLabel(label: ILabel): void;
/**
Called just before a port will be removed.
@param port The port that is about to be removed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onRemovingPort(IPort) Online Documentation}*/
onRemovingPort(port: IPort): void;
/**
Called just before a port label is removed from its port.
@param label
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23onRemovingPortLabel(ILabel) Online Documentation}*/
onRemovingPortLabel(label: ILabel): void;
/**
Removes a previously added lookup chain element from the lookup of <code>this</code>.
@param lookup The element to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23removeLookup(IContextLookupChainLink) Online Documentation}*/
removeLookup(lookup: IContextLookupChainLink): void;
/**
Modifies the location of the given bend.
This implementation will also trigger an {@link DefaultGraph#invalidateDisplays} call.
@param bend the bend whose location is to be modified
@param location the new coordinates of the bend
@throws {Exception} <b>(ArgumentError)</b> If <code>bend</code> is not in this graph.
@see DefaultGraph#addBend
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23setBendLocation(IBend%2CPoint) Online Documentation}*/
setBendLocation(bend: IBend, location: Point): void;
/**
Sets the ports of the given edge to the new values.
This will trigger an {@link DefaultGraph#addEdgePortsChangedListener EdgePortsChanged} event if source or
target ports differ from the current ones. Both ports and the edge must belong to the current graph instance. If {@link IPortDefaults#autoCleanUp} is
enabled, this method will remove unoccupied ports after they have been changed.
@param edge The edge to change the ports.
@param sourcePort The new source port instance.
@param targetPort The new target port instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23setEdgePorts(IEdge%2CIPort%2CIPort) Online Documentation}*/
setEdgePorts(edge: IEdge, sourcePort: IPort, targetPort: IPort): void;
/**
Sets the label text of the given label.
This implementation will {@link IGraph#adjustLabelPreferredSize adjust the preferred size} for the label if {@link ILabelDefaults#autoAdjustPreferredSize} is enabled.
@param label the label to modify
@param text the new text of the label
@throws {Exception} <b>(ArgumentError)</b> If <code>label</code> is not in this graph.
@throws {Exception} <b>(ArgumentError)</b> If <code>text</code> is <code>null</code>.
@see ILabel#text
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23setLabelText(ILabel%2Cstring) Online Documentation}*/
setLabelText(label: ILabel, text: string): void;
/**
Sets the lookup implementation that will be used for {@link DefaultGraph#lookup} calls.
A value of <code>null</code> will revert to the default behavior.
@param lookup The lookup instance to delegate lookup calls to.
@see DefaultGraph#addLookup
@see DefaultGraph#getLookup
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23setLookupImplementation(ILookup) Online Documentation}*/
setLookupImplementation(lookup: ILookup | null): void;
/**
Sets the layout of the given node to the new values.
This will trigger a call to {@link DefaultGraph#invalidateDisplays}.
@param node a live node that belongs to this graph
@param layout the new absolute layout in world coordinates of the node
@throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph, or <code>layout</code> contains one or more NaN values.
@see INode#layout
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23setNodeLayout(INode%2CRect) Online Documentation}*/
setNodeLayout(node: INode, layout: Rect): void;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23toString() Online Documentation}*/
toString(): string;
/**
Gets an {@link IContextLookup} that provides the default implementations returned by an
{@link INode}'s lookup.
The provided default implementations can for example be used as fallback when the node lookup is wrapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23DEFAULT_NODE_LOOKUP Online Documentation}*/
static DEFAULT_NODE_LOOKUP: IContextLookup;
/**
Gets an {@link IContextLookup} that provides the default implementations returned by an
{@link IEdge}'s lookup.
The provided default implementations can for example be used as fallback when the edge lookup is wrapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23DEFAULT_EDGE_LOOKUP Online Documentation}*/
static DEFAULT_EDGE_LOOKUP: IContextLookup;
/**
Gets an {@link IContextLookup} that provides the default implementations returned by an
{@link ILabel}'s lookup.
The provided default implementations can for example be used as fallback when the label lookup is wrapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23DEFAULT_LABEL_LOOKUP Online Documentation}*/
static DEFAULT_LABEL_LOOKUP: IContextLookup;
/**
Gets an {@link IContextLookup} that provides the default implementations returned by an
{@link IPort}'s lookup.
The provided default implementations can for example be used as fallback when the port lookup is wrapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23DEFAULT_PORT_LOOKUP Online Documentation}*/
static DEFAULT_PORT_LOOKUP: IContextLookup;
/**
Gets an {@link IContextLookup} that provides the default implementations returned by an
{@link IBend}'s lookup.
The provided default implementations can for example be used as fallback when the bend lookup is wrapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23DEFAULT_BEND_LOOKUP Online Documentation}*/
static DEFAULT_BEND_LOOKUP: IContextLookup;
/**
Gets the mapper registry that is associated with this graph instance.
The registry can be used to store data mappings for the items in this instance. This field is populated upon first
access. Also this property provides write access.
@see IMapperRegistry
@see IModelItem
@see ITagOwner#tag
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23mapperRegistry Online Documentation}*/
mapperRegistry: IMapperRegistry;
/**
Gets or sets whether or not the {@link UndoEngine} used for this instance should be enabled.
The default is <code>false</code>. In order to enable undoability for this instance, set this property to <code>true</code> and {@link DefaultGraph#lookup}
the {@link UndoEngine} type. Disabling this property clears and removes the current {@link UndoEngine} (and
all enqueued IUndoUnits).
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultGraph%23undoEngineEnabled Online Documentation}*/
undoEngineEnabled: boolean;
static $class: Class;
private $_DefaultGraph: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultGraph;
}
export interface SimpleLabel extends YObject, ILabel {
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
tag: any | null;
} /**
A simple mutable implementation of the {@link ILabel} interface that can be used stand-alone.
Note that this class is not the one used as the default implementation of a {@link DefaultGraph}'s
{@link ILabel}s and casting them to {@link SimpleLabel} will fail. Instances of this class can be used without
an accompanying graph instance. All of the properties are mutable. An example of a possible use case would be the
rendering of a label into a graphics context: Simply create a
{@link SimpleLabel}, associate a style with it and use the style's renderer instance to obtain a rendering for the
label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel Online Documentation}*/
export class SimpleLabel {
/**
Creates an instance of the {@link SimpleLabel} class with default {@link DefaultGraph#DEFAULT_LABEL_LOOKUP lookup}, an instance of {@link VoidLabelStyle}
and the given {@link ILabelModelParameter parameter}, text, and owner.
@param [layoutParameter=null] The parameter that describes the label's layout.
@param [text=''] The label's text.
@param [owner=null] The label's owner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel Online Documentation}*/
constructor(owner?: ILabelOwner | null, text?: string, layoutParameter?: ILabelModelParameter);
/**
Creates an instance of the {@link SimpleLabel} class with default {@link DefaultGraph#DEFAULT_LABEL_LOOKUP lookup}, an instance of {@link VoidLabelStyle}
and the given {@link ILabelModelParameter parameter}, text, and owner.
@param options The parameters to pass.
@param [options.layoutParameter=null] The parameter that describes the label's layout.
@param [options.text=''] The label's text.
@param [options.owner=null] The label's owner.
@param [options.style] The style that is responsible for the visual representation of this node in a {@link CanvasComponent}. This option sets the {@link SimpleLabel#style} property on the created object.
@param [options.tag] Sets the tag property.
@param [options.preferredSize] The preferred size of the label with respect to its current contents and the implementation of the visualization. This option sets the {@link SimpleLabel#preferredSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel Online Documentation}*/
constructor(options: {
owner?: ILabelOwner | null;
text?: string;
layoutParameter?: ILabelModelParameter;
style?: ILabelStyle;
tag?: any | null;
preferredSize?: Size | SizeConvertible;
});
/**
Gets the {@link ILabelStyleRenderer#getPreferredSize} from the labels style and sets the {@link SimpleLabel#preferredSize} to that value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel%23adoptPreferredSizeFromStyle() Online Documentation}*/
adoptPreferredSizeFromStyle(): void;
/**
Gets or sets the label model's parameter that determines the positioning of this label.
Note that the label model parameter instance associated with a label instance may be shared between multiple label
instances and that the modification of this instance or its {@link ILabelModelParameter#model} will result in a change of the positioning of all labels
that are associated with the same parameter or model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel%23layoutParameter Online Documentation}*/
layoutParameter: ILabelModelParameter;
/**
Gets or sets the text string associated with this label.
It is up to the visualization engine to interpret this property for the visualization of the label. Normally, it will
render the text into the {@link ILabel#layout layout} of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel%23text Online Documentation}*/
text: string;
/**
Gets or sets the style that is responsible for the visual representation of this node in a {@link CanvasComponent}.
Note that the style instance associated with a label instance may be shared between multiple label instances and that
the modification of this style will result in a change of the appearance of all labels that are associated with the same
style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel%23style Online Documentation}*/
style: ILabelStyle;
/**
Gets or sets the owner of this label.
Typically this will yield an instance of {@link INode} or {@link IEdge}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel%23owner Online Documentation}*/
owner: ILabelOwner | null;
/**
Gets or sets the preferred size of the label with respect to its current contents and the implementation of the
visualization.
Often times the {@link ILabel#layout layout's} size will be the same as the preferred size, but it's up to the implementation of the {@link ILabelModel}
to interpret it differently.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel%23preferredSize Online Documentation}*/
preferredSize: Size;
/**
Gets or sets the {@link ILookup} implementation used for calls to {@link SimpleLabel#lookup} on this
instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleLabel%23lookupImplementation Online Documentation}*/
lookupImplementation: ILookup | null;
static $class: Class;
private $_SimpleLabel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SimpleLabel;
}
export interface SimpleNode extends YObject, INode {
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
tag: any | null;
} /**
A simple mutable implementation of the {@link INode} interface that can be used stand-alone.
Note that this class is not the one used as the default implementation of a {@link DefaultGraph}'s
{@link INode}s and casting them to {@link SimpleNode} will fail.
Instances of this class can be used without an accompanying graph instance. All of the properties are mutable. An
example of a possible use case would be the rendering of a node into a graphics context: Simply create a
{@link SimpleNode}, associate a style with it and use the style's renderer instance to obtain a rendering for the
node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNode Online Documentation}*/
export class SimpleNode {
/**
Creates a default node with default {@link DefaultGraph#DEFAULT_NODE_LOOKUP lookup}, an instance of {@link VoidNodeStyle}, empty {@link SimpleNode#layout} and no labels or ports.
The instances assigned to {@link SimpleNode#layout}, {@link SimpleNode#ports} and {@link SimpleNode#labels} are immutable by default. To modify the values of those properties, first assign an
instance of a mutable class.
@param [options] The parameters to pass.
@param [options.ports] The collection of {@link IPort ports} that are owned by this instance. This option sets the {@link SimpleNode#ports} property on the created object.
@param [options.labels] The collection of {@link ILabel labels} that are owned by this instance. This option sets the {@link SimpleNode#labels} property on the created object.
@param [options.layout] The layout of the node. This option sets the {@link SimpleNode#layout} property on the created object.
@param [options.style] The style that is responsible for the visual representation of this node in a {@link CanvasComponent}. This option sets the {@link SimpleNode#style} property on the created object.
@param [options.tag] Sets the tag property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNode Online Documentation}*/
constructor(options?: {
ports?: IListEnumerable<IPort> | IListEnumerableConvertible<IPort>;
labels?: IListEnumerable<ILabel> | IListEnumerableConvertible<ILabel>;
layout?: IRectangle | IRectangleConvertible;
style?: INodeStyle;
tag?: any | null;
});
/**
Gets or sets the layout of the node.
The layout of a node is a rectangle in the world coordinate system that describes the bounding box of the representation
of a node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNode%23layout Online Documentation}*/
layout: IRectangle;
/**
Gets or sets the style that is responsible for the visual representation of this node in a {@link CanvasComponent}.
Note that the style instance associated with a node instance may be shared between multiple node instances and that the
modification of this style will result in a change of the appearance of all nodes that are associated with the same
style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNode%23style Online Documentation}*/
style: INodeStyle;
/**
Gets or sets the collection of {@link IPort ports} that are owned by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNode%23ports Online Documentation}*/
ports: IListEnumerable<IPort>;
/**
Gets or sets the collection of {@link ILabel labels} that are owned by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNode%23labels Online Documentation}*/
labels: IListEnumerable<ILabel>;
/**
Gets or sets the {@link ILookup} implementation used for calls to {@link SimpleNode#lookup} on this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNode%23lookupImplementation Online Documentation}*/
lookupImplementation: ILookup | null;
static $class: Class;
private $_SimpleNode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SimpleNode;
}
export interface SimplePort extends YObject, IPort {
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
tag: any | null;
} /**
A simple mutable implementation of the {@link IPort} interface that can be used stand-alone.
Note that this class is not the one used as the default implementation of a {@link DefaultGraph}'s
{@link IPort}s and casting them to {@link SimplePort} will fail. Instances of this class can be used without
an accompanying graph instance. All of the properties are mutable. An example of a possible use case would be the
rendering of a port into a graphics context: Simply create a
{@link SimplePort}, associate a style with it and use the style's renderer instance to obtain a rendering for the
port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplePort Online Documentation}*/
export class SimplePort {
/**
Creates a port with a {@link DefaultGraph#DEFAULT_PORT_LOOKUP default lookup} using the given location and owner.
While technically the owner can be <code>null</code> here, this can cause issues, e.g. with {@link IPortLocationModel port location models} that position the
port relative to its owner.
@param [owner=null] The port's owner.
@param [locationParameter=null] The port's location model parameter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplePort Online Documentation}*/
constructor(owner?: IPortOwner | null, locationParameter?: IPortLocationModelParameter);
/**
Creates a port with a {@link DefaultGraph#DEFAULT_PORT_LOOKUP default lookup} using the given location and owner.
While technically the owner can be <code>null</code> here, this can cause issues, e.g. with {@link IPortLocationModel port location models} that position the
port relative to its owner.
@param options The parameters to pass.
@param [options.owner=null] The port's owner.
@param [options.locationParameter=null] The port's location model parameter.
@param [options.labels] The collection of {@link ILabel labels} that are owned by this instance. This option sets the {@link SimplePort#labels} property on the created object.
@param [options.style] The style that is responsible for the visual representation of this port in a {@link CanvasComponent}. This option sets the {@link SimplePort#style} property on the created object.
@param [options.tag] Sets the tag property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplePort Online Documentation}*/
constructor(options: {
owner?: IPortOwner | null;
locationParameter?: IPortLocationModelParameter;
labels?: IListEnumerable<ILabel> | IListEnumerableConvertible<ILabel>;
style?: IPortStyle;
tag?: any | null;
});
/**
Sets the location of this port.
Changes the {@link IMutablePoint} if one is found in this port's {@link ILookup}. Creates an appropriate
{@link SimplePort#locationParameter}, otherwise.
@param location
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplePort%23setLocation(Point) Online Documentation}*/
setLocation(location: Point): void;
/**
Gets or sets the style that is responsible for the visual representation of this port in a {@link CanvasComponent}.
Note that the style instance associated with a port instance may be shared between multiple port instances and that the
modification of this style will result in a change of the appearance of all ports that are associated with the same
style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplePort%23style Online Documentation}*/
style: IPortStyle;
/**
Gets or sets the {@link IPortLocationModelParameter} that is used to determine the location of this port.
The port location can be calculated using the {@link IPortLocationModelParameter#model}'s {@link IPortLocationModel#getLocation} method or methods {@link IPort#location} and {@link IPort#dynamicLocation}.
Note that parameters may be shared across port instances.
@see IGraph#setPortLocationParameter
@see IPort#location
@see IPort#dynamicLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplePort%23locationParameter Online Documentation}*/
locationParameter: IPortLocationModelParameter;
/**
Gets or sets the owner of this port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplePort%23owner Online Documentation}*/
owner: IPortOwner | null;
/**
Gets or sets the collection of {@link ILabel labels} that are owned by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplePort%23labels Online Documentation}*/
labels: IListEnumerable<ILabel>;
/**
Gets or sets the {@link ILookup} implementation used for calls to {@link SimplePort#lookup} on this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplePort%23lookupImplementation Online Documentation}*/
lookupImplementation: ILookup | null;
static $class: Class;
private $_SimplePort: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SimplePort;
}
export interface DefaultPortCandidate extends YObject, IPortCandidate {
}
/**
A simple default implementation of an {@link IPortCandidate}.
Subclasses need to override {@link DefaultPortCandidate#createInstance} if they don't {@link DefaultPortCandidate#setInstance set} a port instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate Online Documentation}*/
export class DefaultPortCandidate {
/**
Creates a candidate for the given <code>owner</code> that uses the graph's defaults for the port's style and a default parameter for
the given <code>model</code>.
The {@link DefaultPortCandidate#locationParameter} is set to the parameter that best matches the {@link Point#ORIGIN origin} for the given <code>model</code>.
@param owner The owner.
@param model The model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23DefaultPortCandidate(IPortOwner%2CIPortLocationModel) Online Documentation}*/
constructor(owner: IPortOwner, model: IPortLocationModel);
/**
Creates a candidate for the given <code>owner</code> that uses the graph's defaults for the port's style and the given
<code>locationParameter</code>.
If no <code>locationParameter</code> is given, the used candidate has a fixed location at {@link Point#ORIGIN} that does not
depend on the <code>owner</code>.
@param owner The owner.
@param [locationParameter=null] The location.
@param [validity=PortCandidateValidity.VALID] The validity of the port candidate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23DefaultPortCandidate(IPortOwner%2CIPortLocationModelParameter%2CPortCandidateValidity) Online Documentation}*/
constructor(owner: IPortOwner, locationParameter?: IPortLocationModelParameter, validity?: PortCandidateValidity | PortCandidateValidityStringValues);
/**
Creates a candidate for the given <code>owner</code> that uses the graph's defaults for the port's style and the given
<code>locationParameter</code>.
If no <code>locationParameter</code> is given, the used candidate has a fixed location at {@link Point#ORIGIN} that does not
depend on the <code>owner</code>.
@param options The parameters to pass.
@param options.owner The owner.
@param [options.locationParameter=null] The location.
@param [options.validity=PortCandidateValidity.VALID] The validity of the port candidate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23DefaultPortCandidate(IPortOwner%2CIPortLocationModelParameter%2CPortCandidateValidity) Online Documentation}*/
constructor(options: {
owner: IPortOwner;
locationParameter?: IPortLocationModelParameter;
validity?: PortCandidateValidity | PortCandidateValidityStringValues;
});
/**
Creates a candidate using the given port as the template.
@param port The port to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23DefaultPortCandidate(IPort) Online Documentation}*/
constructor(port: IPort);
/**
Actually creates the {@link IPort} instance.
This method is called in response to a call to {@link DefaultPortCandidate#createPort} on this instance and the
instances created by {@link DefaultPortCandidate#getPortCandidateAt}.
@param context The input mode context.
@param graph The graph to add the port to.
@param locationParameter The parameter to use.
@param style The style to use.
@param tag The tag to use.
@returns The newly created port.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23createInstance(IInputModeContext%2CIGraph%2CIPortLocationModelParameter%2CIPortStyle%2CObject) Online Documentation}*/
createInstance(context: IInputModeContext, graph: IGraph, locationParameter: IPortLocationModelParameter, style: IPortStyle, tag: any | null): IPort;
/**
This implementation will throw a {@link Exception NotSupportedError} if the validity is {@link PortCandidateValidity#DYNAMIC}.
This implementation will delegate to the {@link DefaultPortCandidate#createInstance} method.
@param context
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23createPort(IInputModeContext) Online Documentation}*/
createPort(context: IInputModeContext): IPort;
/**
Returns a candidate that is derived from this instance that best fits the provided <code>location</code>.
The returned implementation will finally call {@link DefaultPortCandidate#createInstance} using the newly
calculated {@link DefaultPortCandidate#locationParameter}.
@param context The input mode context for which a concrete candidate is queried.
@param location The location for which a candidate should be returned.
@returns A {@link IPortCandidate} implementation whose {@link DefaultPortCandidate#validity} is guaranteed to be non-{@link PortCandidateValidity#DYNAMIC}.
@see DefaultPortCandidate#validity
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23getPortCandidateAt(IInputModeContext%2CPoint) Online Documentation}*/
getPortCandidateAt(context: IInputModeContext, location: Point): IPortCandidate;
/**
Sets the instance that will be returned by {@link DefaultPortCandidate#port} and {@link DefaultPortCandidate#createPort}.
@param port A port that already exists in the graph or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23setInstance(IPort) Online Documentation}*/
setInstance(port: IPort | null): void;
/**
Gets or sets whether this candidate is a valid one depending on the context it has been obtained from.
@default PortCandidateValidity.VALID
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23validity Online Documentation}*/
validity: PortCandidateValidity;
/**
Gets or set the tag for this candidate.
The interpretation of this value is implementation dependent and may be used for visual feedback or interactive
behavior.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23candidateTag Online Documentation}*/
candidateTag: any | null;
/**
Gets or sets the port tag to create the port with.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23portTag Online Documentation}*/
portTag: any | null;
/**
Gets or sets the model parameter that will be used for the {@link IPort#locationParameter} if this candidate is chosen.
The parameter which may not be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23locationParameter Online Documentation}*/
locationParameter: IPortLocationModelParameter;
/**
Gets or sets the model that will be used by this instance to determine the new {@link DefaultPortCandidate#locationParameter} if the {@link DefaultPortCandidate#getPortCandidateAt}
is called and the {@link DefaultPortCandidate#validity} is set to {@link PortCandidateValidity#DYNAMIC}.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23model Online Documentation}*/
model: IPortLocationModel | null;
/**
Gets or sets the style to use for the newly created port.
In order to change the style of the port candidate itself, you can either use the provided CSS classes, see {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_view_css_styling#customizing_view_css_port_canidates Styling of Port Candidates}
or use the {@link DefaultPortCandidateDescriptor} and its candidate drawing keys to change the appearance of port
candidates.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23style Online Documentation}*/
style: IPortStyle | null;
/**
Gets or sets the owner of the port to be created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23owner Online Documentation}*/
owner: IPortOwner;
/**
Gets the instance provided to the constructor or to the {@link DefaultPortCandidate#setInstance setter}.
@see DefaultPortCandidate#createPort
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidate%23port Online Documentation}*/
readonly port: IPort | null;
static $class: Class;
private $_DefaultPortCandidate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultPortCandidate;
}
export interface DefaultPortCandidateDescriptor extends YObject, ICanvasObjectDescriptor, IVisualCreator, IBoundsProvider, IHitTestable, IVisibilityTestable, IMarqueeTestable, ILassoTestable {
/**
@inheritdoc*/
getBoundsProvider(forUserObject: any | null): IBoundsProvider;
/**
@inheritdoc*/
getHitTestable(forUserObject: any | null): IHitTestable;
/**
@inheritdoc*/
getVisibilityTestable(forUserObject: any | null): IVisibilityTestable;
/**
@inheritdoc*/
getVisualCreator(forUserObject: any | null): IVisualCreator;
/**
@inheritdoc*/
isDirty(context: ICanvasContext, canvasObject: ICanvasObject): boolean;
} /**
A simple {@link ICanvasObjectDescriptor} that can be used to draw {@link IPortCandidate} instances in a
{@link CanvasComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor Online Documentation}*/
export class DefaultPortCandidateDescriptor {
/**
Instantiates a new descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor Online Documentation}*/
constructor();
/**
Sets the {@link IVisualTemplate} to use for the given {@link string} by this instance.
This method can be used to reconfigure the visualization for the various keys that are declared by this class.
@param resourceKey The key to reconfigure. This is one of
{@link DefaultPortCandidateDescriptor#CANDIDATE_DRAWING_VALID_NON_FOCUSED_KEY},
{@link DefaultPortCandidateDescriptor#CANDIDATE_DRAWING_INVALID_NON_FOCUSED_KEY},
{@link DefaultPortCandidateDescriptor#CANDIDATE_DRAWING_INVALID_FOCUSED_KEY},
{@link DefaultPortCandidateDescriptor#CANDIDATE_DRAWING_VALID_FOCUSED_KEY}.
@param template The template to use instead of the default.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor%23setTemplate(string%2CIVisualTemplate) Online Documentation}*/
setTemplate(resourceKey: string, template: IVisualTemplate | null): void;
/**
A {@link string} that can be used by instances of the {@link DefaultPortCandidateDescriptor} to look up a {@link IVisualTemplate}
that will be used for rendering a valid, non-focused {@link IPortCandidate}.
The drawing returned should be centered around <code>(0,0)</code>.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_indicators#customizing_indicators-defaults Customizing the Predefined Indicators} on how to register a custom {@link IVisualTemplate} for this {@link string}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor%23CANDIDATE_DRAWING_VALID_NON_FOCUSED_KEY Online Documentation}*/
static CANDIDATE_DRAWING_VALID_NON_FOCUSED_KEY: string;
/**
A {@link string} that can be used by instances of the {@link DefaultPortCandidateDescriptor} to look up a {@link IVisualTemplate}
that will be used for rendering a valid, focused {@link IPortCandidate}.
The drawing returned should be centered around <code>(0,0)</code>.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_indicators#customizing_indicators-defaults Customizing the Predefined Indicators} on how to register a custom {@link IVisualTemplate} for this {@link string}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor%23CANDIDATE_DRAWING_VALID_FOCUSED_KEY Online Documentation}*/
static CANDIDATE_DRAWING_VALID_FOCUSED_KEY: string;
/**
A {@link string} that can be used by instances of the {@link DefaultPortCandidateDescriptor} to look up a {@link IVisualTemplate}
that will be used for rendering an invalid, non-focused {@link IPortCandidate}.
The drawing returned should be centered around <code>(0,0)</code>.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_indicators#customizing_indicators-defaults Customizing the Predefined Indicators} on how to register a custom {@link IVisualTemplate} for this {@link string}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor%23CANDIDATE_DRAWING_INVALID_NON_FOCUSED_KEY Online Documentation}*/
static CANDIDATE_DRAWING_INVALID_NON_FOCUSED_KEY: string;
/**
A {@link string} that can be used by instances of the {@link DefaultPortCandidateDescriptor} to look up a {@link IVisualTemplate}
that will be used for rendering a invalid, focused {@link IPortCandidate}.
The drawing returned should be centered around <code>(0,0)</code>.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_indicators#customizing_indicators-defaults Customizing the Predefined Indicators} on how to register a custom {@link IVisualTemplate} for this {@link string}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor%23CANDIDATE_DRAWING_INVALID_FOCUSED_KEY Online Documentation}*/
static CANDIDATE_DRAWING_INVALID_FOCUSED_KEY: string;
/**
Gets or sets a property that determines whether this descriptor uses the "current" candidates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor%23currentCandidate Online Documentation}*/
currentCandidate: boolean;
/**
Gets or sets a value indicating whether port candidates are drawn in view coordinates, instead of intermediate
coordinates.
This setting has no visible effect, unless a {@link CanvasComponent#projection} is set.
The default value is <code>false</code>, to draw port candidates in the intermediate coordinate system.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor%23useViewCoordinates Online Documentation}*/
useViewCoordinates: boolean;
/**
Gets or sets the logical size of the candidate.
This size is used for hit testing, visibility tests, etc. The default is <code>3</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortCandidateDescriptor%23size Online Documentation}*/
size: number;
static $class: Class;
private $_DefaultPortCandidateDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultPortCandidateDescriptor;
}
export interface DescriptorWrapperLabelModel extends YObject, ILabelModel, ILabelCandidateDescriptorProvider, ILabelModelParameterProvider {
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getDescriptor(label: ILabel, layoutParameter: ILabelModelParameter): ILabelCandidateDescriptor | null;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A label model that wraps an existing label model and decorates it with a {@link ILabelCandidateDescriptor descriptor}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DescriptorWrapperLabelModel Online Documentation}*/
export class DescriptorWrapperLabelModel {
/**
Initializes a new instance of the {@link DescriptorWrapperLabelModel} class.
@param [innerModel=null] The wrapped label model or {@link FreeLabelModel} if not specified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DescriptorWrapperLabelModel Online Documentation}*/
constructor(innerModel?: ILabelModel);
/**
Initializes a new instance of the {@link DescriptorWrapperLabelModel} class.
@param options The parameters to pass.
@param [options.innerModel=null] The wrapped label model or {@link FreeLabelModel} if not specified.
@param [options.descriptor] The descriptor. This option sets the {@link DescriptorWrapperLabelModel#descriptor} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DescriptorWrapperLabelModel Online Documentation}*/
constructor(options: {
innerModel?: ILabelModel;
descriptor?: ILabelCandidateDescriptor | null;
});
/**
Creates a default parameter that can be used for this model.
@returns The default parameter from {@link DescriptorWrapperLabelModel#innerModel}
@see {@link https://docs.yworks.com/yfileshtml/#/api/DescriptorWrapperLabelModel%23createDefaultParameter() Online Documentation}*/
createDefaultParameter(): ILabelModelParameter;
/**
Gets or sets the descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DescriptorWrapperLabelModel%23descriptor Online Documentation}*/
descriptor: ILabelCandidateDescriptor | null;
/**
Gets or sets the inner label model
@see {@link https://docs.yworks.com/yfileshtml/#/api/DescriptorWrapperLabelModel%23innerModel Online Documentation}*/
innerModel: ILabelModel;
static $class: Class;
private $_DescriptorWrapperLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DescriptorWrapperLabelModel;
}
export interface FoldingEdgeStateId extends YObject {
}
/**
A data holder that can be used to describe a {@link IFoldingView#isInFoldingState folding} edge that connects to nodes inside a collapsed group or to other
folder nodes in a {@link IFoldingView}.
Instances of this type are passed to the various methods in {@link FoldingManager} to identify different possible
states of an edge inside {@link IFoldingView}s. A folding edge is identified through the master edge, which is an
edge in the {@link FoldingManager#masterGraph} and a source and target node. Also whether those source and target nodes are currently {@link IFoldingView#isInFoldingState folder} nodes needs
to be taken into account.
The source and target nodes need to be the nodes in the {@link FoldingManager#masterGraph} which represent the source and target nodes of the view edge,
i.e. either the true {@link IPort#owner owners} of the adjacent edge's {@link IEdge#sourcePort} and {@link IEdge#targetPort} respectively, or {@link IGraph#getParent ancestor} group nodes in the {@link FoldingManager#masterGraph}. In the latter
case these folding edges are used to describe the state of the folding edges that are present in the views of the
corresponding group nodes are in {@link IFoldingView#collapse collapsed} state, so that the true source and target nodes are not currently visible in
the view. As such the {@link FoldingEdgeStateId#currentMasterSource} and {@link FoldingEdgeStateId#currentMasterTarget} nodes represent the {@link IFoldingView#getMasterItem masters} of the nearest ancestors in the {@link FoldingManager#masterGraph} that are visible in the {@link IFoldingView folding view}
graph.
@see FoldingManager#hasFoldingEdgeState
@see FoldingManager#getFoldingEdgeState
@see IFoldingView#isInFoldingState
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeStateId Online Documentation}*/
export class FoldingEdgeStateId {
/**
Initializes an id for a folding edge in a {@link IFoldingView view} that represent a master edge and connects a specific pair of nodes.
All of the arguments need to be entities of the {@link FoldingManager#masterGraph master graph}.
@param masterEdge An edge in the {@link FoldingManager#masterGraph} that is represented by the folding edge.
@param currentMasterSource The source node of the edge or one of its ancestors to which the folding edge connects in the view, if the actual parent
is currently {@link IFoldingView#collapse collapsed in an ancestor node}.
@param sourceCollapsed <code>true</code> if the <code>currentMasterSource</code> is the true source of the <code>masterEdge</code>, but is in its {@link IFoldingView#collapse collapsed/dummy} state.
@param targetCollapsed <code>true</code> if the <code>currentMasterTarget</code> is the true target of the <code>masterEdge</code>, but is in its {@link IFoldingView#collapse collapsed/dummy} state.
@param currentMasterTarget The target node of the edge or one of its ancestors to which the folding edge connects in the view, if the actual parent
is currently {@link IFoldingView#collapse collapsed in an ancestor node}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeStateId%23FoldingEdgeStateId(IEdge%2CINode%2Cboolean%2CINode%2Cboolean) Online Documentation}*/
constructor(masterEdge: IEdge, currentMasterSource: INode, sourceCollapsed: boolean, currentMasterTarget: INode, targetCollapsed: boolean);
/**
Initializes an id for a folding edge that is currently part of the given view.
@param foldingView The view the folding edge is part of.
@param foldingEdge The instance that is part of the {@link IFoldingView#graph folding view graph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeStateId%23FoldingEdgeStateId(IFoldingView%2CIEdge) Online Documentation}*/
constructor(foldingView: IFoldingView, foldingEdge: IEdge);
/**
Gets the edge instance in the {@link FoldingManager#masterGraph} that is represented by the folding edge in a view.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeStateId%23masterEdge Online Documentation}*/
readonly masterEdge: IEdge;
/**
Gets the source node of the edge or an ancestor (both in the {@link FoldingManager#masterGraph}).
This may be an ancestor of the true source node if this ancestor is {@link IFoldingView#collapse collapsed}. More specific it is the source node of
the {@link FoldingEdgeStateId#masterEdge} if the source node is represented in the view and otherwise the first ancestor of the source node which is
represented in the view.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeStateId%23currentMasterSource Online Documentation}*/
readonly currentMasterSource: INode;
/**
Gets the target node of the edge or an ancestor (both in the {@link FoldingManager#masterGraph}).
This may be an ancestor of the true target node if this ancestor is {@link IFoldingView#collapse collapsed}. More precisely it is the target node of
the {@link FoldingEdgeStateId#masterEdge} if the target node is represented in the view and otherwise the first ancestor of the target node which is
represented in the view.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeStateId%23currentMasterTarget Online Documentation}*/
readonly currentMasterTarget: INode;
/**
Gets whether the {@link FoldingEdgeStateId#currentMasterSource} node {@link IFoldingView#isInFoldingState is in folder state}.
Since an edge will be converted to a folding edge if the adjacent node itself will be {@link IFoldingView#collapse collapsed} to a folder node, this
property determines which state the source node is in for this folding edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeStateId%23sourceIsCollapsed Online Documentation}*/
readonly sourceIsCollapsed: boolean;
/**
Gets whether the {@link FoldingEdgeStateId#currentMasterTarget} node {@link IFoldingView#isInFoldingState is in folder state}.
Since an edge will be converted to a folding edge if the adjacent node itself will be {@link IFoldingView#collapse collapsed} to a folder node, this
property determines which state the source node is in for this folding edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeStateId%23targetIsCollapsed Online Documentation}*/
readonly targetIsCollapsed: boolean;
static $class: Class;
private $_FoldingEdgeStateId: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FoldingEdgeStateId;
}
export interface EdgePathLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider, ILabelModelParameterFinder {
/**
@inheritdoc*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
An edge label model that allows placement of labels along the edge path at a set of continuous positions at specified
{@link EdgePathLabelModel#sideOfEdge sides of the edge path}.
The set of positions can be influenced by specifying the {@link EdgePathLabelModel#sideOfEdge} value that controls on which side of the edge labels positions
should be considered. Furthermore, it's possible to specify distance values that control the distance between label and
edge and between label and nodes.
If a label model parameter is created {@link EdgeSides#ON_EDGE}, the label's center is usually placed on the edge.
It can be shifted by specifying a {@link EdgePathLabelModel#offset} value but the {@link EdgePathLabelModel#distance} value is ignored. For all other values of {@link EdgeSides} the
total distance between the label bounds and the edge path is the sum of {@link EdgePathLabelModel#offset} and {@link EdgePathLabelModel#distance}.
The label placements use a {@link EdgePathLabelModel#createRatioParameter ratio} value as fraction of the full edge path length. This is the main difference to the {@link EdgePathLabelModel}
which uses a specified segment index from the {@link EdgeSegmentLabelModel#createParameterFromSource source} or {@link EdgeSegmentLabelModel#createParameterFromTarget target} side and a ratio on this segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel Online Documentation}*/
export class EdgePathLabelModel {
/**
Initializes a new instance of the {@link EdgePathLabelModel} class.
@param [distance=0] the distance to the edge.
@param [offset=0] the offset to the default placement.
@param [angle=0] the angle of the label's rotation in radians.
@param [autoRotation=true] if set to <code>true</code> auto rotation is enabled.
@param [sideOfEdge=EdgeSides.ON_EDGE  |  EdgeSides.LEFT_OF_EDGE  |  EdgeSides.RIGHT_OF_EDGE] the side of the edge labels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel Online Documentation}*/
constructor(distance?: number, offset?: number, angle?: number, autoRotation?: boolean, sideOfEdge?: EdgeSides | EdgeSidesConvertible);
/**
Initializes a new instance of the {@link EdgePathLabelModel} class.
@param options The parameters to pass.
@param [options.distance=0] the distance to the edge.
@param [options.offset=0] the offset to the default placement.
@param [options.angle=0] the angle of the label's rotation in radians.
@param [options.autoRotation=true] if set to <code>true</code> auto rotation is enabled.
@param [options.sideOfEdge=EdgeSides.ON_EDGE  |  EdgeSides.LEFT_OF_EDGE  |  EdgeSides.RIGHT_OF_EDGE] the side of the edge labels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel Online Documentation}*/
constructor(options: {
distance?: number;
offset?: number;
angle?: number;
autoRotation?: boolean;
sideOfEdge?: EdgeSides | EdgeSidesConvertible;
});
/**
A model parameter that encodes the default position of this model's allowed edge label positions.
Returns a model parameter that encodes the default position of this model's allowed edge label positions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel%23createDefaultParameter() Online Documentation}*/
createDefaultParameter(): ILabelModelParameter;
/**
Creates a parameter at the provided edge path ratio.
@param [edgePathRatio=0.5] The ratio at which to place the label at the edge path. A ratio of 0.0 will place the label at the source side of the
edge path, a ratio of 1.0 at the target side. The default is 0.5.
@param [sideOfEdge=EdgeSides.ON_EDGE] The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed. The
default is {@link EdgeSides#ON_EDGE}.
@returns A label parameter at the provided edge path ratio for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel%23createRatioParameter(number%2CEdgeSides) Online Documentation}*/
createRatioParameter(edgePathRatio?: number, sideOfEdge?: EdgeSides | EdgeSidesConvertible): ILabelModelParameter;
/**
Creates a parameter at the provided edge path ratio.
@param options The parameters to pass.
@param [options.edgePathRatio=0.5] The ratio at which to place the label at the edge path. A ratio of 0.0 will place the label at the source side of the
edge path, a ratio of 1.0 at the target side. The default is 0.5.
@param [options.sideOfEdge=EdgeSides.ON_EDGE] The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed. The
default is {@link EdgeSides#ON_EDGE}.
@returns A label parameter at the provided edge path ratio for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel%23createRatioParameter(number%2CEdgeSides) Online Documentation}*/
createRatioParameter(options: {
edgePathRatio?: number;
sideOfEdge?: EdgeSides | EdgeSidesConvertible;
}): ILabelModelParameter;
/**
Returns an empty context.
@param label The label to use in the context.
@param layoutParameter The parameter to use for the label in the context.
@returns An empty context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel%23getContext(ILabel%2CILabelModelParameter) Online Documentation}*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
Gets or sets the side placement specifiers for edge labels.
The label model parameter {@link ILabelModelParameterFinder finder} and {@link ILabelModelParameterProvider provider} returned by {@link EdgePathLabelModel#lookup} consider the specified
sides.
@see EdgeSides
@default 'OnEdge, LeftOfEdge, RightOfEdge'
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel%23sideOfEdge Online Documentation}*/
sideOfEdge: EdgeSides;
/**
Gets or sets the distance between the label box and the edge path.
The interpretation of the values depends on the {@link EdgePathLabelModel#sideOfEdge} of the individual {@link ILabelModelParameter}.
Note that for {@link EdgeSides#ON_EDGE} this property is ignored. For all other values of {@link EdgeSides}
the {@link EdgePathLabelModel#offset} property is added to this distance.
@see EdgePathLabelModel#sideOfEdge
@see EdgeSides
@see EdgePathLabelModel#offset
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel%23distance Online Documentation}*/
distance: number;
/**
Gets or sets the offset of the label box and the edge path relative to the default placement.
For {@link EdgeSides#ON_EDGE} this is the offset between the center of the label box and the edge path. For all
other {@link EdgeSides} values this offset is added to the {@link EdgePathLabelModel#distance} property which results in the total distance between
the label box and the edge path.
@see EdgePathLabelModel#sideOfEdge
@see EdgeSides
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel%23offset Online Documentation}*/
offset: number;
/**
Gets or sets whether edge labels are automatically rotated according to the angle of the corresponding reference edge
segment.
By default, this feature is enabled.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel%23autoRotation Online Documentation}*/
autoRotation: boolean;
/**
Gets or sets the rotation angle of all labels with this model in radians.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgePathLabelModel%23angle Online Documentation}*/
angle: number;
static $class: Class;
private $_EdgePathLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgePathLabelModel;
}
export interface EdgeSegmentLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider, ILabelModelParameterFinder {
/**
@inheritdoc*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
An edge label model that allows placement of labels relative to an edge segment at a set of continuous positions at
specified {@link EdgeSegmentLabelModel#sideOfEdge sides of the edge path}.
The set of positions can be influenced by specifying the {@link EdgeSegmentLabelModel#sideOfEdge} value that controls on which side of the edge labels positions
should be considered. Furthermore, it's possible to specify distance values that control the distance between label and
edge and between label and nodes.
If a label model parameter is created {@link EdgeSides#ON_EDGE}, the label's center is usually placed on the edge.
It can be shifted by specifying a {@link EdgeSegmentLabelModel#offset} value but the {@link EdgeSegmentLabelModel#distance} value is ignored. For all other values of {@link EdgeSides} the
total distance between the label bounds and the edge path is the sum of {@link EdgeSegmentLabelModel#offset} and {@link EdgeSegmentLabelModel#distance}.
The label placements use a specified segment index from the {@link EdgeSegmentLabelModel#createParameterFromSource source} or {@link EdgeSegmentLabelModel#createParameterFromTarget target} side and a ratio on this segment. This is
the main difference to the {@link EdgePathLabelModel} which only uses a ratio value of the full edge path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel Online Documentation}*/
export class EdgeSegmentLabelModel {
/**
Initializes a new instance of the {@link EdgeSegmentLabelModel} class.
@param [distance=0] the distance to the edge.
@param [offset=0] the offset to the default placement.
@param [angle=0] the angle of the label's rotation in radians.
@param [autoRotation=true] if set to <code>true</code> auto rotation is enabled.
@param [sideOfEdge=EdgeSides.ON_EDGE  |  EdgeSides.LEFT_OF_EDGE  |  EdgeSides.RIGHT_OF_EDGE] the side of the edge labels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel Online Documentation}*/
constructor(distance?: number, offset?: number, angle?: number, autoRotation?: boolean, sideOfEdge?: EdgeSides | EdgeSidesConvertible);
/**
Initializes a new instance of the {@link EdgeSegmentLabelModel} class.
@param options The parameters to pass.
@param [options.distance=0] the distance to the edge.
@param [options.offset=0] the offset to the default placement.
@param [options.angle=0] the angle of the label's rotation in radians.
@param [options.autoRotation=true] if set to <code>true</code> auto rotation is enabled.
@param [options.sideOfEdge=EdgeSides.ON_EDGE  |  EdgeSides.LEFT_OF_EDGE  |  EdgeSides.RIGHT_OF_EDGE] the side of the edge labels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel Online Documentation}*/
constructor(options: {
distance?: number;
offset?: number;
angle?: number;
autoRotation?: boolean;
sideOfEdge?: EdgeSides | EdgeSidesConvertible;
});
/**
A model parameter that encodes the default position of this model's allowed edge label positions.
Returns a model parameter that encodes the default position of this model's allowed edge label positions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23createDefaultParameter() Online Documentation}*/
createDefaultParameter(): ILabelModelParameter;
/**
Creates a parameter where the label is attached to the edge segment that contains the edge's midpoint.
@param [segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of 0.0 will place the label at the source side of the
segment, a ratio of 1.0 at the target side. Ratios &lt; 0.0 or > 1.0 will be interpreted as absolute values in world
coordinates. The default is 0.5.
@param [sideOfEdge=EdgeSides.ON_EDGE] The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed. The
default is {@link EdgeSides#ON_EDGE}.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23createParameterFromCenter(number%2CEdgeSides) Online Documentation}*/
createParameterFromCenter(segmentRatio?: number, sideOfEdge?: EdgeSides | EdgeSidesConvertible): ILabelModelParameter;
/**
Creates a parameter where the label is attached to the edge segment that contains the edge's midpoint.
@param options The parameters to pass.
@param [options.segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of 0.0 will place the label at the source side of the
segment, a ratio of 1.0 at the target side. Ratios &lt; 0.0 or > 1.0 will be interpreted as absolute values in world
coordinates. The default is 0.5.
@param [options.sideOfEdge=EdgeSides.ON_EDGE] The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed. The
default is {@link EdgeSides#ON_EDGE}.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23createParameterFromCenter(number%2CEdgeSides) Online Documentation}*/
createParameterFromCenter(options: {
segmentRatio?: number;
sideOfEdge?: EdgeSides | EdgeSidesConvertible;
}): ILabelModelParameter;
/**
Creates a parameter that measures the provided segment index from the source side of the edge path.
@param segmentIndex The zero-based index of the segment beginning from the source side.
@param [segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of 0.0 will place the label at the source side of the
segment, a ratio of 1.0 at the target side. Ratios &lt; 0.0 or > 1.0 will be interpreted as absolute values in world
coordinates. The default is 0.5.
@param [sideOfEdge=EdgeSides.ON_EDGE] The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed. The
default is {@link EdgeSides#ON_EDGE}.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23createParameterFromSource(number%2Cnumber%2CEdgeSides) Online Documentation}*/
createParameterFromSource(segmentIndex: number, segmentRatio?: number, sideOfEdge?: EdgeSides | EdgeSidesConvertible): ILabelModelParameter;
/**
Creates a parameter that measures the provided segment index from the source side of the edge path.
@param options The parameters to pass.
@param options.segmentIndex The zero-based index of the segment beginning from the source side.
@param [options.segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of 0.0 will place the label at the source side of the
segment, a ratio of 1.0 at the target side. Ratios &lt; 0.0 or > 1.0 will be interpreted as absolute values in world
coordinates. The default is 0.5.
@param [options.sideOfEdge=EdgeSides.ON_EDGE] The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed. The
default is {@link EdgeSides#ON_EDGE}.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23createParameterFromSource(number%2Cnumber%2CEdgeSides) Online Documentation}*/
createParameterFromSource(options: {
segmentIndex: number;
segmentRatio?: number;
sideOfEdge?: EdgeSides | EdgeSidesConvertible;
}): ILabelModelParameter;
/**
Creates a parameter that measures the provided segment index from the target side of the edge path.
@param segmentIndex The zero-based index of the segment beginning from the target side.
@param [segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of 0.0 will place the label at the target side of the
segment, a ratio of 1.0 at the source side. Ratios &lt; 0.0 or > 1.0 will be interpreted as absolute values in world
coordinates. The default is 0.5.
@param [sideOfEdge=EdgeSides.ON_EDGE] The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed. The
default is {@link EdgeSides#ON_EDGE}.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23createParameterFromTarget(number%2Cnumber%2CEdgeSides) Online Documentation}*/
createParameterFromTarget(segmentIndex: number, segmentRatio?: number, sideOfEdge?: EdgeSides | EdgeSidesConvertible): ILabelModelParameter;
/**
Creates a parameter that measures the provided segment index from the target side of the edge path.
@param options The parameters to pass.
@param options.segmentIndex The zero-based index of the segment beginning from the target side.
@param [options.segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of 0.0 will place the label at the target side of the
segment, a ratio of 1.0 at the source side. Ratios &lt; 0.0 or > 1.0 will be interpreted as absolute values in world
coordinates. The default is 0.5.
@param [options.sideOfEdge=EdgeSides.ON_EDGE] The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed. The
default is {@link EdgeSides#ON_EDGE}.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23createParameterFromTarget(number%2Cnumber%2CEdgeSides) Online Documentation}*/
createParameterFromTarget(options: {
segmentIndex: number;
segmentRatio?: number;
sideOfEdge?: EdgeSides | EdgeSidesConvertible;
}): ILabelModelParameter;
/**
Returns an empty context.
@param label The label to use in the context.
@param layoutParameter The parameter to use for the label in the context.
@returns An empty context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23getContext(ILabel%2CILabelModelParameter) Online Documentation}*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
Gets or sets the side placement specifiers for edge labels.
The label model parameter {@link ILabelModelParameterFinder finder} and {@link ILabelModelParameterProvider provider} returned by {@link EdgeSegmentLabelModel#lookup} consider the
specified sides.
@see EdgeSides
@default 'OnEdge, LeftOfEdge, RightOfEdge'
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23sideOfEdge Online Documentation}*/
sideOfEdge: EdgeSides;
/**
Gets or sets the distance between the label box and the edge path.
The interpretation of the values depends on the {@link EdgeSegmentLabelModel#sideOfEdge} of the individual {@link ILabelModelParameter}.
Note that for {@link EdgeSides#ON_EDGE} this property is ignored. For all other values of {@link EdgeSides}
the {@link EdgeSegmentLabelModel#offset} property is added to this distance.
@see EdgeSegmentLabelModel#sideOfEdge
@see EdgeSides
@see EdgeSegmentLabelModel#offset
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23distance Online Documentation}*/
distance: number;
/**
Gets or sets the offset of the label box and the edge path relative to the default placement.
For {@link EdgeSides#ON_EDGE} this is the offset between the center of the label box and the edge path. For all
other {@link EdgeSides} values this offset is added to the {@link EdgeSegmentLabelModel#distance} property which results in the total distance between
the label box and the edge path.
@see EdgeSegmentLabelModel#sideOfEdge
@see EdgeSides
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23offset Online Documentation}*/
offset: number;
/**
Gets or sets whether edge labels are automatically rotated according to the angle of the corresponding reference edge
segment.
By default, this feature is enabled.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23autoRotation Online Documentation}*/
autoRotation: boolean;
/**
Gets or sets the rotation angle of all labels with this model in radians.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentLabelModel%23angle Online Documentation}*/
angle: number;
static $class: Class;
private $_EdgeSegmentLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeSegmentLabelModel;
}
/**
The side placement specifier for edge labels used by {@link EdgeSegmentLabelModel}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSides Online Documentation}*/
export enum EdgeSides {
/**
The label should be placed on the path of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSides%23ON_EDGE Online Documentation}*/
ON_EDGE,
/**
The label should be placed on the left hand side of the edge path if looking from the source node into the direction of
the target node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSides%23LEFT_OF_EDGE Online Documentation}*/
LEFT_OF_EDGE,
/**
The label should be placed on the right hand side of the edge path if looking from the source node into the direction of
the target node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSides%23RIGHT_OF_EDGE Online Documentation}*/
RIGHT_OF_EDGE,
/**
The label should be placed above the edge (in geometric sense).
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSides%23ABOVE_EDGE Online Documentation}*/
ABOVE_EDGE,
/**
The label should be placed below the edge (in geometric sense).
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSides%23BELOW_EDGE Online Documentation}*/
BELOW_EDGE
}
type EdgeSidesStringValues = "on-edge" | "left-of-edge" | "right-of-edge" | "above-edge" | "below-edge";
/** Automatic conversion to {@link EdgeSides}. */
type EdgeSidesConvertible = EdgeSidesStringValues | EdgeSidesStringValues[];
/**
Enumeration that is used by {@link EdgeSegmentLabelModel} to indicate the position of the label along the edge.
This enumeration is mainly used by the XAML serialization mechanisms.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceAlongEdge Online Documentation}*/
export enum PlaceAlongEdge {
/**
The segment index is counted from the edge source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceAlongEdge%23AT_SOURCE Online Documentation}*/
AT_SOURCE,
/**
The segment index is counted from the edge target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceAlongEdge%23AT_TARGET Online Documentation}*/
AT_TARGET,
/**
The label is at the center of the edge and the segment index is ignored.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceAlongEdge%23AT_CENTER Online Documentation}*/
AT_CENTER
}
/** Automatic conversion to {@link PlaceAlongEdge}. */
type PlaceAlongEdgeStringValues = "at-source" | "at-target" | "at-center";
/**
Enumeration of the possible positions of the {@link ExteriorLabelModel}.
This field can be used as an argument for the {@link ExteriorLabelModel#createParameter} factory method.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModelPosition Online Documentation}*/
export enum ExteriorLabelModelPosition {
/**
Encodes a position outside the node at the top
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModelPosition%23NORTH Online Documentation}*/
NORTH,
/**
Encodes a position outside the node at the right side
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModelPosition%23EAST Online Documentation}*/
EAST,
/**
Encodes a position outside the node at the bottom
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModelPosition%23SOUTH Online Documentation}*/
SOUTH,
/**
Encodes a position outside the node at the left side
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModelPosition%23WEST Online Documentation}*/
WEST,
/**
Encodes a position outside the node at the upper right corner
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModelPosition%23NORTH_EAST Online Documentation}*/
NORTH_EAST,
/**
Encodes a position outside the node at the lower right corner
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModelPosition%23SOUTH_EAST Online Documentation}*/
SOUTH_EAST,
/**
Encodes a position outside the node at the upper left corner
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModelPosition%23NORTH_WEST Online Documentation}*/
NORTH_WEST,
/**
Encodes a position outside the node at the lower left corner
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModelPosition%23SOUTH_WEST Online Documentation}*/
SOUTH_WEST
}
/** Automatic conversion to {@link ExteriorLabelModelPosition}. */
type ExteriorLabelModelPositionStringValues = "north" | "east" | "south" | "west" | "north-east" | "south-east" | "north-west" | "south-west";
export interface ExteriorLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A label model for node labels that lie outside of the node's {@link INode#layout} bounds.
This label model supports eight positions outside of the layout's bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel Online Documentation}*/
export class ExteriorLabelModel {
/**
Creates a new instance of this model
@param [options] The parameters to pass.
@param [options.insets] The insets to use by this model. This option sets the {@link ExteriorLabelModel#insets} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel Online Documentation}*/
constructor(options?: {
insets?: Insets | InsetsConvertible;
});
/**
Creates a new parameter that is associated with this instance for the provided position.
@param position The position constant that describes the position of the label.
@returns A new parameter instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23createParameter(ExteriorLabelModelPosition) Online Documentation}*/
createParameter(position: ExteriorLabelModelPosition | ExteriorLabelModelPositionStringValues): ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link ExteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23NORTH Online Documentation}*/
static NORTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link ExteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23EAST Online Documentation}*/
static EAST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link ExteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23SOUTH Online Documentation}*/
static SOUTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link ExteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23SOUTH_EAST Online Documentation}*/
static SOUTH_EAST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link ExteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23SOUTH_WEST Online Documentation}*/
static SOUTH_WEST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link ExteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23WEST Online Documentation}*/
static WEST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link ExteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23NORTH_EAST Online Documentation}*/
static NORTH_EAST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link ExteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23NORTH_WEST Online Documentation}*/
static NORTH_WEST: ILabelModelParameter;
/**
Gets or sets the insets to use by this model.
The insets are in fact "outsets", i.e. they are treated as margins. Positive values will increase the distance of the
labels to the node.
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorLabelModel%23insets Online Documentation}*/
insets: Insets;
static $class: Class;
private $_ExteriorLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ExteriorLabelModel;
}
export interface FilteredGraphWrapper extends GraphWrapperBase {
/**
@inheritdoc*/
contains(item: IModelItem | null): boolean;
/**
@inheritdoc*/
createGroupNode(parent?: INode | null, layout?: Rect | null | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
@inheritdoc*/
createGroupNode(options: {
parent?: INode | null;
layout?: Rect | null | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
}): INode;
/**
@inheritdoc*/
getChildren(node: INode | null): IListEnumerable<INode>;
/**
@inheritdoc*/
getParent(node: INode): INode | null;
/**
@inheritdoc*/
isGroupNode(node: INode | null): boolean;
/**
@inheritdoc*/
setIsGroupNode(node: INode | null, isGroupNode: boolean): void;
/**
@inheritdoc*/
setParent(node: INode, parent: INode | null): void;
/**
@inheritdoc*/
readonly nodes: IListEnumerable<INode>;
/**
@inheritdoc*/
readonly edges: IListEnumerable<IEdge>;
/**
@inheritdoc*/
readonly labels: IListEnumerable<ILabel>;
/**
@inheritdoc*/
readonly ports: IListEnumerable<IPort>;
} /**
An {@link IGraph} that contains a live subgraph of another {@link IGraph}.
{@link FilteredGraphWrapper} is a decorator for another {@link IGraph} instance and can be used to retain only
a subset of nodes and edges of the original graph. The properties {@link FilteredGraphWrapper#nodePredicate} and {@link FilteredGraphWrapper#edgePredicate} control which nodes and edges are retained in
the filtered graph. Edges adjacent to filtered nodes are also filtered automatically (as their source or target would
not be in the graph anyway).
Note that {@link FilteredGraphWrapper} has to listen to events raised by the wrapped graph to keep itself updated,
so {@link FilteredGraphWrapper#dispose} should be called if this instance is not used any more. Otherwise the
wrapped graph may be kept alive for longer than necessary.
{@link IPortOwner#ports Ports} at {@link IEdge}s (i.e. edge-to-edge connections) are not fully supported.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FilteredGraphWrapper Online Documentation}*/
export class FilteredGraphWrapper {
/**
Creates a new graph instance that wraps the original graph and uses the predicates to determine which nodes and edges
should be contained in the graph.
Note that this instance will register listeners with the <code>graph</code>, so {@link FilteredGraphWrapper#dispose} should be
called if this instance is not used any more.
@param graph The graph to wrap
@param nodePredicate The predicate that determines which nodes to keep.
@param [edgePredicate=null] The optional predicate that determines which edges to keep. If omitted, all edges are kept, unless connected to filtered
nodes.
@throws {Exception} <b>(ArgumentError)</b> if <code>graph</code>, <code>nodePredicate</code>, or <code>edgePredicate</code> is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FilteredGraphWrapper Online Documentation}*/
constructor(graph: IGraph, nodePredicate: (obj: INode) => boolean, edgePredicate?: ((obj: IEdge) => boolean) | null);
/**
Should be called by the client if the {@link FilteredGraphWrapper#edgePredicate} changed for the given edge in the graph.
@param edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/FilteredGraphWrapper%23edgePredicateChanged(IEdge) Online Documentation}*/
edgePredicateChanged(edge: IEdge): void;
/**
Should be called by the client if the {@link FilteredGraphWrapper#edgePredicate} changed for the edge in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FilteredGraphWrapper%23edgePredicateChanged() Online Documentation}*/
edgePredicateChanged(): void;
/**
Should be called by clients if the {@link FilteredGraphWrapper#nodePredicate} changed for the given node.
@param node The node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FilteredGraphWrapper%23nodePredicateChanged(INode) Online Documentation}*/
nodePredicateChanged(node: INode): void;
/**
Should be called by the client if the {@link FilteredGraphWrapper#nodePredicate} changed for the nodes in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FilteredGraphWrapper%23nodePredicateChanged() Online Documentation}*/
nodePredicateChanged(): void;
/**
Gets the predicate function for the nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FilteredGraphWrapper%23nodePredicate Online Documentation}*/
readonly nodePredicate: (obj: INode) => boolean;
/**
Gets the predicate function for the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FilteredGraphWrapper%23edgePredicate Online Documentation}*/
readonly edgePredicate: (obj: IEdge) => boolean;
static $class: Class;
private $_FilteredGraphWrapper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FilteredGraphWrapper;
}
export interface FreeEdgeLabelModel extends YObject, ILabelModelParameterFinder, ILabelModel {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
An edge label model that can be used to describe any position and orientation of a label.
The position of the label is stored relative to the positions of the ports of the edge the label belongs to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelModel Online Documentation}*/
export class FreeEdgeLabelModel {
/**
Creates a new instance of a {@link FreeEdgeLabelModel} with the default values.
@param [options] The parameters to pass.
@param [options.edgeRelativeAngle] A property that indicates whether the angle used by the {@link FreeEdgeLabelModel#createEdgeAnchored parameter} should be interpreted relative to the edge's orientation or as an absolute angle in the coordinate system. This option sets the {@link FreeEdgeLabelModel#edgeRelativeAngle} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelModel Online Documentation}*/
constructor(options?: {
edgeRelativeAngle?: boolean;
});
/**
Creates a parameter for the given location.
@param [ratio=0.5] The ratio at which the label will be anchored between the location of the source port and the location of the target
port. The default is 0.5.
@param [distance=0] The distance orthogonally to the connection between the source and target point. The default is 0.
@param [angle=0] The angle by which the label should be rotated. The default is 0.
@returns A parameter that describes the position.
@see FreeEdgeLabelModel#edgeRelativeAngle
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelModel%23createEdgeAnchored(number%2Cnumber%2Cnumber) Online Documentation}*/
createEdgeAnchored(ratio?: number, distance?: number, angle?: number): ILabelModelParameter;
/**
Creates a parameter for the given location.
@param options The parameters to pass.
@param [options.ratio=0.5] The ratio at which the label will be anchored between the location of the source port and the location of the target
port. The default is 0.5.
@param [options.distance=0] The distance orthogonally to the connection between the source and target point. The default is 0.
@param [options.angle=0] The angle by which the label should be rotated. The default is 0.
@returns A parameter that describes the position.
@see FreeEdgeLabelModel#edgeRelativeAngle
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelModel%23createEdgeAnchored(number%2Cnumber%2Cnumber) Online Documentation}*/
createEdgeAnchored(options: {
ratio?: number;
distance?: number;
angle?: number;
}): ILabelModelParameter;
/**
Tries to find a parameter that best matches the given layout for the provided label instance.
This implementation will match the provided layout perfectly.
@param label The label to find a parameter for.
@param model The model instance to use. This should be this instance.
@param layout The anticipated layout for the label.
@returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelModel%23findBestParameter(ILabel%2CILabelModel%2CIOrientedRectangle) Online Documentation}*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
Gets or sets a property that indicates whether the angle used by the {@link FreeEdgeLabelModel#createEdgeAnchored parameter} should be interpreted relative to the
edge's orientation or as an absolute angle in the coordinate system.
By default the angle is interpreted to be relative to the world coordinates axes.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelModel%23edgeRelativeAngle Online Documentation}*/
edgeRelativeAngle: boolean;
/**
A shared unmodifiable public instance that can be used to obtain parameters.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelModel%23INSTANCE Online Documentation}*/
static INSTANCE: FreeEdgeLabelModel;
static $class: Class;
private $_FreeEdgeLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreeEdgeLabelModel;
}
export interface FreeLabelModel extends YObject, ILabelModelParameterFinder, ILabelModel {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A label model that can be used to describe any position and orientation of a label.
The position of the label is stored absolutely and is not influenced by the layout or position of the owner of the
label. In fact the owner is not even queried during a call to {@link FreeLabelModel#getGeometry}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeLabelModel Online Documentation}*/
export class FreeLabelModel {
/**
Creates a new label model parameter that places the label at the given absolute position using the provided angle.
@param anchorLocation The location for the parameter.
@param [angle=0] The angle of rotation. The default is 0.
@returns A new parameter that is associated with this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeLabelModel%23createAbsolute(Point%2Cnumber) Online Documentation}*/
createAbsolute(anchorLocation: Point | PointConvertible, angle?: number): ILabelModelParameter;
/**
Creates a new label at the given dynamic location using the provided angle.
@param location The location to place the parameter at. That instance is stored by reference.
@param [angle=0] The angle of rotation. The default is 0.
@returns A new parameter that is associated with this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeLabelModel%23createAnchored(IPoint%2Cnumber) Online Documentation}*/
createAnchored(location: IPoint | IPointConvertible, angle?: number): ILabelModelParameter;
/**
Creates a new label model parameter that dynamically reads the label layout from the provided instance.
@param layout The oriented rectangle to dynamically retrieve the values from.
@returns A new parameter that is associated with this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeLabelModel%23createDynamic(IOrientedRectangle) Online Documentation}*/
createDynamic(layout: IOrientedRectangle): ILabelModelParameter;
/**
A shared public instance that can be used to obtain parameters.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeLabelModel%23INSTANCE Online Documentation}*/
static INSTANCE: FreeLabelModel;
static $class: Class;
private $_FreeLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreeLabelModel;
}
export interface FreeNodeLabelModel extends YObject, ILabelModel, ILabelModelParameterFinder {
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
} /**
This node label model can be used to specify any position and orientation of a label.
The position of the label is stored relative to the layout of the node the label belongs to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelModel Online Documentation}*/
export class FreeNodeLabelModel {
/**
Creates a parameter instance that anchors the <code>labelLayout</code> relative to the <code>nodeLayout</code>.
This method ultimately delegates to {@link FreeNodeLabelModel#createParameter} so its main purpose is to calculate
the according node and label layout ratio and offset parameters.
These parameters are calculated in the following way: Each coordinate axis is considered independently. If the reference
point coordinate is within the node layout interval, its relative location specifies the layout ratio for this
coordinate. Otherwise, the absolute distance of the reference point and the node layout interval specifies the layout
offset and the layout ratio is either either <code>0.0</code> or
<code>1.0</code>. The parameters for the label ratio and label offset are calculated analogously. If no reference point is provided,
the reference point is determined automatically in the following way: For each coordinate, if the node layout and label
layout do not overlap, the reference is the label corner that is closest to the node. If they overlap and if in addition
the start or end coordinate of the label matches the start, center or end coordinate of the node, that coordinate is the
reference. Otherwise, the center of the label specifies the reference.
@param nodeLayout The node layout.
@param labelLayout The label layout.
@param [referenceLocation=null] A reference location that influences the ratio and offset calculation for the {@link FreeNodeLabelModel#createParameter}
call.
@returns The canonical parameter for the specified node and label layout.
@throws {Exception} <b>(ArgumentError)</b> <code>nodeLayout</code> or <code>referenceLocation</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelModel%23createCanonicalParameter(Rect%2CIOrientedRectangle%2CPoint) Online Documentation}*/
createCanonicalParameter(nodeLayout: Rect | RectConvertible, labelLayout: IOrientedRectangle, referenceLocation?: Point | null | PointConvertible): ILabelModelParameter;
/**
Creates a parameter that is anchored at the node center and has no rotation.
@returns A parameter that is anchored at the node center and has no rotation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelModel%23createDefaultParameter() Online Documentation}*/
createDefaultParameter(): ILabelModelParameter;
/**
Creates a parameter instance that anchors the label to the node at a given position.
The parameter will use the ratio and offset values to determine a point on both the label and the node. It will place
the label in such a way that both points coincide.
@param layoutRatio The ratio that describes the point on the node's layout relative to its upper-left corner. A value of (0,0) describes
the upper-left corner, while (1,1) is the lower-right corner.
@param layoutOffset The absolute offset to apply to the point on the node after the ratio has been determined.
@param labelRatio The ratio that describes the point on the label's layout relative to its upper-left corner. A value of (0,0) describes
the upper-left corner, while (1,1) is the lower-right corner.
@param [labelOffset=null] The absolute offset to apply to the point on the label after the ratio has been determined. This defaults to (0,0),
representing no further offset.
@param [angle=0] The angle by which the label should be rotated around the point described using the ratio and offset, measured in
radians. The default is 0.
@returns A parameter that exactly fits the described location.
@throws {Exception} <b>(ArgumentError)</b> <code>layoutRatio</code>, <code>layoutOffset</code>, <code>labelRatio</code> or <code>labelOffset</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelModel%23createParameter(Point%2CPoint%2CPoint%2CPoint%2Cnumber) Online Documentation}*/
createParameter(layoutRatio: Point | PointConvertible, layoutOffset: Point | PointConvertible, labelRatio: Point | PointConvertible, labelOffset?: Point | null | PointConvertible, angle?: number): ILabelModelParameter;
/**
Creates a parameter instance that anchors the label to the node at a given position.
The parameter will use the ratio and offset values to determine a point on both the label and the node. It will place
the label in such a way that both points coincide.
@param options The parameters to pass.
@param options.layoutRatio The ratio that describes the point on the node's layout relative to its upper-left corner. A value of (0,0) describes
the upper-left corner, while (1,1) is the lower-right corner.
@param options.layoutOffset The absolute offset to apply to the point on the node after the ratio has been determined.
@param options.labelRatio The ratio that describes the point on the label's layout relative to its upper-left corner. A value of (0,0) describes
the upper-left corner, while (1,1) is the lower-right corner.
@param [options.labelOffset=null] The absolute offset to apply to the point on the label after the ratio has been determined. This defaults to (0,0),
representing no further offset.
@param [options.angle=0] The angle by which the label should be rotated around the point described using the ratio and offset, measured in
radians. The default is 0.
@returns A parameter that exactly fits the described location.
@throws {Exception} <b>(ArgumentError)</b> <code>layoutRatio</code>, <code>layoutOffset</code>, <code>labelRatio</code> or <code>labelOffset</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelModel%23createParameter(Point%2CPoint%2CPoint%2CPoint%2Cnumber) Online Documentation}*/
createParameter(options: {
layoutRatio: Point | PointConvertible;
layoutOffset: Point | PointConvertible;
labelRatio: Point | PointConvertible;
labelOffset?: Point | null | PointConvertible;
angle?: number;
}): ILabelModelParameter;
/**
Tries to find a parameter that best matches the given layout for the provided label instance.
This implementation will match the provided layout perfectly.
@param label The label to find a parameter for.
@param model The model instance to use. This should be this instance.
@param layout The anticipated layout for the label.
@returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelModel%23findBestParameter(ILabel%2CILabelModel%2CIOrientedRectangle) Online Documentation}*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
Returns an empty context.
@param label The label to use in the context.
@param layoutParameter The parameter to use for the label in the context.
@returns An empty context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelModel%23getContext(ILabel%2CILabelModelParameter) Online Documentation}*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
Returns this instance if an instance of {@link ILabelModelParameterFinder} is requested and <code>null</code> otherwise.
@param type The type for which an instance is requested.
@returns An instance that is assignable to the given type or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelModel%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
A shared public instance that can be used to obtain parameters.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelModel%23INSTANCE Online Documentation}*/
static INSTANCE: FreeNodeLabelModel;
static $class: Class;
private $_FreeNodeLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreeNodeLabelModel;
}
export interface FreeNodePortLocationModel extends YObject, IPortLocationModel {
/**
@inheritdoc*/
createParameter(owner: IPortOwner, location: Point): IPortLocationModelParameter;
/**
@inheritdoc*/
getContext(port: IPort, locationParameter: IPortLocationModelParameter): ILookup;
/**
@inheritdoc*/
getLocation(port: IPort, locationParameter: IPortLocationModelParameter): Point;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A straightforward implementation of an {@link IPortLocationModel} for {@link IPort}s at {@link INode}s
that determine the location dynamically using the {@link INode#layout}.
Internally the location is stored as the ratios by which the {@link ISize#width} and {@link ISize#height} of the {@link INode#layout} need to be scaled and additional offsets to
obtain the offset to the top-left corner of the node layout.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel Online Documentation}*/
export class FreeNodePortLocationModel {
/**
Creates a parameter that uses the given ratios and offsets.
The <code>ratios</code> will be scaled by the node layout's size and added to the node's top left corner and the <code>offset</code> to determine
the port location.
@param ratios The ratios of the node layout's size.
@param [offset=null] The offset that is added after the <code>ratios</code> are applied.
@returns A new parameter that matches the specification.
@throws {Exception} <b>(ArgumentError)</b> <code>ratios</code> or <code>offset</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23createParameterForRatios(Point%2CPoint) Online Documentation}*/
createParameterForRatios(ratios: Point | PointConvertible, offset?: Point | null | PointConvertible): IPortLocationModelParameter;
/**
A read only immutable instance of the {@link FreeNodePortLocationModel}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23INSTANCE Online Documentation}*/
static INSTANCE: FreeNodePortLocationModel;
/**
A read only immutable singleton instance of a parameter that locates the port at the center of the node layout.
This is the same as {@link FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0.5,0.5)</code> as ratios and <code>(0,0)</code> as offsets.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23NODE_CENTER_ANCHORED Online Documentation}*/
static NODE_CENTER_ANCHORED: IPortLocationModelParameter;
/**
A read only immutable singleton instance of a parameter that locates the port at the center of the left border of the
node layout.
This is the same as {@link FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0,0.5)</code> as ratios and <code>(0,0)</code> as offsets.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23NODE_LEFT_ANCHORED Online Documentation}*/
static NODE_LEFT_ANCHORED: IPortLocationModelParameter;
/**
A read only immutable singleton instance of a parameter that locates the port at the center of the right border of the
node layout.
This is the same as {@link FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(1,0.5)</code> as ratios and <code>(0,0)</code> as offsets.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23NODE_RIGHT_ANCHORED Online Documentation}*/
static NODE_RIGHT_ANCHORED: IPortLocationModelParameter;
/**
A read only immutable singleton instance of a parameter that locates the port at the center of the top border of the
node layout.
This is the same as {@link FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0.5,0)</code> as ratios and <code>(0,0)</code> as offsets.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23NODE_TOP_ANCHORED Online Documentation}*/
static NODE_TOP_ANCHORED: IPortLocationModelParameter;
/**
A read only immutable singleton instance of a parameter that locates the port at the center of the bottom border of the
node layout.
This is the same as {@link FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0.5,1)</code> as ratios and <code>(0,0)</code> as offsets.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23NODE_BOTTOM_ANCHORED Online Documentation}*/
static NODE_BOTTOM_ANCHORED: IPortLocationModelParameter;
/**
A read only immutable singleton instance of a parameter that locates the port at the top left corner of the node
layout.
This is the same as {@link FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0,0)</code> as ratios and <code>(0,0)</code> as offsets.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23NODE_TOP_LEFT_ANCHORED Online Documentation}*/
static NODE_TOP_LEFT_ANCHORED: IPortLocationModelParameter;
/**
A read only immutable singleton instance of a parameter that locates the port at the top right corner of the node
layout.
This is the same as {@link FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(1,0)</code> as ratios and <code>(0,0)</code> as offsets.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23NODE_TOP_RIGHT_ANCHORED Online Documentation}*/
static NODE_TOP_RIGHT_ANCHORED: IPortLocationModelParameter;
/**
A read only immutable singleton instance of a parameter that locates the port at the bottom right corner of the node
layout.
This is the same as {@link FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(1,1)</code> as ratios and <code>(0,0)</code> as offsets.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23NODE_BOTTOM_RIGHT_ANCHORED Online Documentation}*/
static NODE_BOTTOM_RIGHT_ANCHORED: IPortLocationModelParameter;
/**
A read only immutable singleton instance of a parameter that locates the port at the bottom left corner of the node
layout.
This is the same as {@link FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0,1)</code> as ratios and <code>(0,0)</code> as offsets.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePortLocationModel%23NODE_BOTTOM_LEFT_ANCHORED Online Documentation}*/
static NODE_BOTTOM_LEFT_ANCHORED: IPortLocationModelParameter;
static $class: Class;
private $_FreeNodePortLocationModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreeNodePortLocationModel;
}
export interface FreePortLabelModel extends YObject, ILabelModel, ILabelModelParameterFinder {
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
} /**
This port label model can be used to specify any position and orientation of a label.
The position of the label is stored relative to the position of the port the label belongs to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreePortLabelModel Online Documentation}*/
export class FreePortLabelModel {
/**
Creates a parameter that is anchored at the center of the port and has no rotation.
@returns A parameter that is anchored at the center of the port and has no rotation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreePortLabelModel%23createDefaultParameter() Online Documentation}*/
createDefaultParameter(): ILabelModelParameter;
/**
Creates a parameter instance that anchors the label to the port at a given position.
The parameter will use the ratio and offset values to determine a point on both the label and the port. It will place
the label in such a way that both points coincide.
@param locationOffset The offset to apply to the port's location.
@param labelRatio The ratio that describes the point on the label's layout relative to its size.
@param [labelOffset=null] The offset to apply to the point on the label after the ratio has been determined. This defaults to (0,0), representing
no further offset.
@param [angle=0] The angle by which the label should be rotated around the point described using the ratio and offset, measured in
radians. The default is 0.
@returns A parameter that exactly fits the described location.
@throws {Exception} <b>(ArgumentError)</b> <code>locationOffset</code>, <code>labelRatio</code> or <code>labelOffset</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreePortLabelModel%23createParameter(Point%2CPoint%2CPoint%2Cnumber) Online Documentation}*/
createParameter(locationOffset: Point | PointConvertible, labelRatio: Point | PointConvertible, labelOffset?: Point | null | PointConvertible, angle?: number): ILabelModelParameter;
/**
Creates a parameter instance that anchors the label to the port at a given position.
The parameter will use the ratio and offset values to determine a point on both the label and the port. It will place
the label in such a way that both points coincide.
@param options The parameters to pass.
@param options.locationOffset The offset to apply to the port's location.
@param options.labelRatio The ratio that describes the point on the label's layout relative to its size.
@param [options.labelOffset=null] The offset to apply to the point on the label after the ratio has been determined. This defaults to (0,0), representing
no further offset.
@param [options.angle=0] The angle by which the label should be rotated around the point described using the ratio and offset, measured in
radians. The default is 0.
@returns A parameter that exactly fits the described location.
@throws {Exception} <b>(ArgumentError)</b> <code>locationOffset</code>, <code>labelRatio</code> or <code>labelOffset</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreePortLabelModel%23createParameter(Point%2CPoint%2CPoint%2Cnumber) Online Documentation}*/
createParameter(options: {
locationOffset: Point | PointConvertible;
labelRatio: Point | PointConvertible;
labelOffset?: Point | null | PointConvertible;
angle?: number;
}): ILabelModelParameter;
/**
Tries to find a parameter that best matches the given layout for the provided label instance.
This implementation will match the provided layout perfectly.
@param label The label to find a parameter for.
@param model The model instance to use. This should be this instance.
@param layout The anticipated layout for the label.
@returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreePortLabelModel%23findBestParameter(ILabel%2CILabelModel%2CIOrientedRectangle) Online Documentation}*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
Returns an empty context.
@param label The label to use in the context.
@param layoutParameter The parameter to use for the label in the context.
@returns An empty context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreePortLabelModel%23getContext(ILabel%2CILabelModelParameter) Online Documentation}*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
Returns this instance if an instance of {@link ILabelModelParameterFinder} is requested and <code>null</code> otherwise.
@param type The type for which an instance is requested.
@returns An instance that is assignable to the given type or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreePortLabelModel%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
A shared public instance that can be used to obtain parameters.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreePortLabelModel%23INSTANCE Online Documentation}*/
static INSTANCE: FreePortLabelModel;
static $class: Class;
private $_FreePortLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreePortLabelModel;
}
export interface GenericLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider, ILabelCandidateDescriptorProvider, IMarkupExtensionConverter {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
* An implementation of {@link ILabelModel} that can be used to create custom label models out of existing {@link ILabelModelParameter}
* instances.
* This class holds a number of parameters and serves as a container for these parameters. Custom models can easily be
* created that way:
*
* <br><b>Creating a GenericLabelModel instance with a few parameters from different models</b>
* <pre>
* const genericLabelModel = new GenericLabelModel(InteriorLabelModel.CENTER)
* genericLabelModel.addParameter(ExteriorLabelModel.NORTH)
* genericLabelModel.addParameter(ExteriorLabelModel.SOUTH)
* genericLabelModel.addParameter(InteriorStretchLabelModel.EAST)
* genericLabelModel.addParameter(InteriorStretchLabelModel.WEST)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GenericLabelModel Online Documentation}
*/
export class GenericLabelModel {
/**
Create a new generic label model using the provided parameter as the {@link GenericLabelModel#createDefaultParameter default parameter.}
The parameter is stored by reference.
@param defaultLayoutParameter The parameter to use as the default.
@param [descriptor=null] The descriptor to associate with the parameter.
@throws {Exception} <b>(ArgumentError)</b> <code>defaultParameter</code> is null.
@see GenericLabelModel#getDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericLabelModel Online Documentation}*/
constructor(defaultLayoutParameter: ILabelModelParameter, descriptor?: ILabelCandidateDescriptor | null);
/**
Adds another parameter to this models collection of parameters.
Note that the parameter will be wrapped by this class and the resulting parameter which will use this model instance as
its {@link ILabelModelParameter#model} will be returned. The parameter is stored by reference and subsequent modification of the parameter or its
associated model will be reflected by this instance. Also the methods allows for associating a {@link ILabelCandidateDescriptor}
with each parameter that can later be retrieved via the {@link GenericLabelModel#getDescriptor} method.
@param layoutParameter The parameter to add to this instance.
@param [descriptor=null] The descriptor to associate with the parameter.
@returns A new parameter that wraps the provided parameter and uses this instance as its model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericLabelModel%23addParameter(ILabelModelParameter%2CILabelCandidateDescriptor) Online Documentation}*/
addParameter(layoutParameter: ILabelModelParameter, descriptor?: ILabelCandidateDescriptor | null): ILabelModelParameter;
/**
Yields the {@link ILabelCandidateDescriptor} that has been associated with the instance during
{@link GenericLabelModel#addParameter}.
@param label The label instance to use.
@param layoutParameter The parameter to get the descriptor for.
@returns The descriptor instance that had been added to this instance during {@link GenericLabelModel#addParameter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericLabelModel%23getDescriptor(ILabel%2CILabelModelParameter) Online Documentation}*/
getDescriptor(label: ILabel, layoutParameter: ILabelModelParameter): ILabelCandidateDescriptor | null;
static $class: Class;
private $_GenericLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GenericLabelModel;
}
export interface GenericPortLocationModel extends YObject, IPortLocationModel, IEnumerable<IPortLocationModelParameter>, IMarkupExtensionConverter {
/**
@inheritdoc*/
getContext(port: IPort, locationParameter: IPortLocationModelParameter): ILookup;
/**
@inheritdoc*/
getLocation(port: IPort, locationParameter: IPortLocationModelParameter): Point;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
* An implementation of {@link IPortLocationModel} that can be used to create custom port location models out of
* existing {@link IPortLocationModelParameter} instances.
* This class holds a number of parameters and serves as a container for these parameters.
*
* Custom models can easily be created in the following way:
*
* <br><b>Creating a custom GenericPortLocationModel</b>
* <pre>
* const genericModel = new GenericPortLocationModel()
*
* genericModel.add(FreeNodePortLocationModel.NODE_CENTER_ANCHORED)
* genericModel.add(FreeNodePortLocationModel.NODE_BOTTOM_ANCHORED)
* genericModel.add(FreeNodePortLocationModel.NODE_TOP_ANCHORED)
* genericModel.add(FreeNodePortLocationModel.NODE_RIGHT_ANCHORED)
* genericModel.add(FreeNodePortLocationModel.NODE_LEFT_ANCHORED)
* graph.nodeDefaults.ports.locationParameter = genericModel.locationParameters.first()
* </pre>
*
* Note that in order to work, this model needs to have at least one {@link GenericPortLocationModel#addParameter parameter added}.
*
* @see GenericPortLocationModel#addParameter
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPortLocationModel Online Documentation}
*/
export class GenericPortLocationModel {
/**
Initializes a new instance of the {@link GenericPortLocationModel} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPortLocationModel Online Documentation}*/
constructor();
/**
This method simply delegates to {@link GenericPortLocationModel#addParameter}.
It is there to allow for convenient initialization in code.
This method is used by the compiler if the collection initializer syntax is used.
@param locationParameter The parameter to add to this instance.
@see GenericPortLocationModel#addParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPortLocationModel%23add(IPortLocationModelParameter) Online Documentation}*/
add(locationParameter: IPortLocationModelParameter): void;
/**
Adds the given parameter to the {@link GenericPortLocationModel#locationParameters} of this instance.
Note that this will create a new parameter instance that is then actually bound to this instance and returned. The
instance is stored by reference so if this is a mutable instance this instance will be modified, implicitly, too.
@param locationParameter The parameter to add to this instance.
@returns The parameter to use with this model that wraps the given parameter.
@see GenericPortLocationModel#add
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPortLocationModel%23addParameter(IPortLocationModelParameter) Online Documentation}*/
addParameter(locationParameter: IPortLocationModelParameter): IPortLocationModelParameter;
/**
Factory method that creates a parameter for the given port that tries to match the provided location in absolute world
coordinates.
This implementation iterates over all {@link GenericPortLocationModel#locationParameters} to return the one that matches the given location the best.
@param owner The port owner that will own the port for which the parameter shall be created.
@param location The location in the world coordinate system that should be matched as best as possible.
@returns A new instance that can be used to describe the location of an {@link IPort} at the given <code>owner</code>.
@throws {Exception} <b>(ArgumentError)</b> <code>location</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPortLocationModel%23createParameter(IPortOwner%2CPoint) Online Documentation}*/
createParameter(owner: IPortOwner, location: Point): IPortLocationModelParameter;
/**
Gets all parameters that are associated with this instance.
This enumerates the instances that wraps the parameters that have been added to this instance via
{@link GenericPortLocationModel#addParameter}
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPortLocationModel%23locationParameters Online Documentation}*/
readonly locationParameters: IEnumerable<IPortLocationModelParameter>;
static $class: Class;
private $_GenericPortLocationModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GenericPortLocationModel;
}
export interface IClipboardHelper {
}
/**
* This interface can be used to customize the clipboard behavior for individual {@link IModelItem}s.
* Implementations of this interface can {@link ILookupDecorator#addLookup decorate} the {@link ILookup#lookup} method of {@link IModelItem}s to
* change their behavior during {@link GraphClipboard clipboard} actions.
*
* The easiest way to decorate certain model items with a custom {@link IClipboardHelper} implementation is via the
* various decorators available on {@link GraphDecorator}:
*
* <br><b>Using GraphDecorator to attach a custom IClipboardHelper to nodes</b>
* <pre>
* const nodeDecorator = graph.decorator.nodeDecorator
* // Set a custom IClipboardHelper for all nodes in the graph
* nodeDecorator.clipboardHelperDecorator.setImplementation(new MyClipboardHelper())
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardHelper Online Documentation}
*/
export abstract class IClipboardHelper {
/**
This method is called during the copy operation of the given item to retrieve additional state that can be later used
during a {@link IClipboardHelper#paste} operation.
If the source graph is a managed folding view, the <code>item</code> is always a master item.
@param context The context in which this interface is used, can be <code>null</code>
@param item The item to copy to the clipboard.
@returns A custom user object that will later be provided to {@link IClipboardHelper#paste}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardHelper%23copy(IGraphClipboardContext%2CIModelItem) Online Documentation}*/
abstract copy(context: IGraphClipboardContext | null, item: IModelItem): any | null;
/**
Creates an implementation of the interface {@link IClipboardHelper} from the given definition.
The given object must contain exactly the abstract members of the {@link IClipboardHelper} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IClipboardHelper} interface.
@returns An instance of the {@link IClipboardHelper} interface based on the given definition.*/
static create(interfaceDefinition: {
shouldCopy(context: IGraphClipboardContext | null, item: IModelItem): boolean;
shouldCut(context: IGraphClipboardContext | null, item: IModelItem): boolean;
shouldPaste(context: IGraphClipboardContext | null, item: IModelItem, userData: any | null): boolean;
copy(context: IGraphClipboardContext | null, item: IModelItem): any;
cut(context: IGraphClipboardContext | null, item: IModelItem): any;
paste(context: IGraphClipboardContext | null, item: IModelItem, userData: any | null): void;
}): IClipboardHelper;
/**
This method is called during the cut operation of the given item to retrieve additional state that can be later used
during a {@link IClipboardHelper#paste} operation.
If the source graph is a managed folding view, the <code>item</code> is always a master item.
@param context The context in which this interface is used, can be <code>null</code>
@param item The item to cut to the clipboard.
@returns A custom user object that will later be provided to {@link IClipboardHelper#paste}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardHelper%23cut(IGraphClipboardContext%2CIModelItem) Online Documentation}*/
abstract cut(context: IGraphClipboardContext | null, item: IModelItem): any | null;
/**
This method is called after the item has been pasted from the clipboard.
The <code>item</code> is always a master item since the clipboard graph is not a managed folding view.
@param context The context in which this interface is used, can be <code>null</code>
@param item The item that has been created during the paste operation.
@param userData The data that had been queried during {@link IClipboardHelper#cut} or {@link IClipboardHelper#copy}
respectively.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardHelper%23paste(IGraphClipboardContext%2CIModelItem%2CObject) Online Documentation}*/
abstract paste(context: IGraphClipboardContext | null, item: IModelItem, userData: any | null): void;
/**
Determines whether the given item can or should be copied to the clipboard.
If the source graph is a managed folding view, the <code>item</code> is always a view item. Furthermore, if collapsed nodes are to be
copied, this method will be queried with the collapsed group node as <code>item</code> for each "hidden" node that is currently not
visible.
@param context The context in which this interface is used, can be <code>null</code>
@param item The item in question.
@returns Whether this item should be copied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardHelper%23shouldCopy(IGraphClipboardContext%2CIModelItem) Online Documentation}*/
abstract shouldCopy(context: IGraphClipboardContext | null, item: IModelItem): boolean;
/**
Determines whether the given item can or should be cut to the clipboard.
If the source graph is a managed folding view, the <code>item</code> is always a view item. Furthermore, if collapsed nodes are to be
cut, this method will be queried with the collapsed group node as <code>item</code> for each "hidden" node that is currently not
visible.
@param context The context in which this interface is used, can be <code>null</code>
@param item The item in question.
@returns Whether this item should be cut.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardHelper%23shouldCut(IGraphClipboardContext%2CIModelItem) Online Documentation}*/
abstract shouldCut(context: IGraphClipboardContext | null, item: IModelItem): boolean;
/**
Determines whether the given item can or should be pasted from the clipboard to the target graph.
The <code>item</code> is always a master item since the clipboard graph is not a managed folding view.
@param context The context in which this interface is used, can be <code>null</code>
@param item The item in the clipboard graph.
@param userData The state memento that had been created during {@link IClipboardHelper#cut} or {@link IClipboardHelper#copy}.
@returns Whether this item should be pasted.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardHelper%23shouldPaste(IGraphClipboardContext%2CIModelItem%2CObject) Online Documentation}*/
abstract shouldPaste(context: IGraphClipboardContext | null, item: IModelItem, userData: any | null): boolean;
static $class: Class;
private $_IClipboardHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IClipboardHelper;
}
export interface IGraphClipboardContext extends ILookup {
}
/**
The interface for the context that is passed by the {@link GraphClipboard} implementation to the {@link IClipboardHelper}
methods and {@link IClipboardIdProvider}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphClipboardContext Online Documentation}*/
export abstract class IGraphClipboardContext {
/**
Gets the clipboard that created this context and is used during the current operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphClipboardContext%23clipboard Online Documentation}*/
abstract readonly clipboard: GraphClipboard;
/**
Gets the source graph for the current operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphClipboardContext%23sourceGraph Online Documentation}*/
abstract readonly sourceGraph: IGraph;
/**
Gets the target graph for the current operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphClipboardContext%23targetGraph Online Documentation}*/
abstract readonly targetGraph: IGraph;
static $class: Class;
private $_IGraphClipboardContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGraphClipboardContext;
}
export interface IClipboardIdProvider {
}
/**
Used by the {@link GraphClipboard} to assign IDs for elements in the clipboard.
IDs are created when items are copied (or cut) into the clipboard and queried when the items are pasted from the
clipboard.
Providing a custom ID provider allows for identifying conceptually identical items across different graph instances.
The default implementation uses the {@link IModelItem} object reference as the ID.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardIdProvider Online Documentation}*/
export abstract class IClipboardIdProvider {
/**
Creates an implementation of the interface {@link IClipboardIdProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link IClipboardIdProvider} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IClipboardIdProvider} interface.
@returns An instance of the {@link IClipboardIdProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
getId(context: IGraphClipboardContext, originalItem: IModelItem): any;
getItem(context: IGraphClipboardContext, id: any): IModelItem;
}): IClipboardIdProvider;
/**
Gets the ID for the given <code>originalItem</code>.
@param context The clipboard context
@param originalItem The original item which is copied. The item is part of the original graph from which the item is copied.
@returns An ID for the given item. The object should be suitable for use in a {@link HashMap}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardIdProvider%23getId(IGraphClipboardContext%2CIModelItem) Online Documentation}*/
abstract getId(context: IGraphClipboardContext, originalItem: IModelItem): any;
/**
* Gets the item which is represented by the given <code>id</code>.
* <br><b>Retrieving the original item which has been copied into the clipboard</b>
* <pre>
* /**
*  * @param {ILabel} clipboardLabel - The label to be copied.
*  *‍/
* getTargetLabeledItem(clipboardLabel) {
*   // clipboardLabel lives in the clipboard graph
*
*   // get the original label, i.e. the label which has been copied from the original graph
*   const originalLabel = this.idProvider.getItem(
*     this.clipboardContext,
*     this.getId(clipboardLabel)
*   )
*
*   // return the original label's owner as the pasted label's owner
*   return originalLabel.owner
* }
* </pre>
*
* @param context The clipboard context.
* @param id The ID to get the item for.
* @returns The item which is represented by the given ID or <code>null</code> if the item cannot be located or is invalid. The item needs to be
* part of the target graph for paste operations.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IClipboardIdProvider%23getItem(IGraphClipboardContext%2CObject) Online Documentation}
*/
abstract getItem(context: IGraphClipboardContext, id: any): IModelItem | null;
static $class: Class;
private $_IClipboardIdProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IClipboardIdProvider;
}
/**
Defines the rules for pasting nodes into group nodes.
The enumeration can be used to create a bitwise combination of the types.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentNodeDetectionModes Online Documentation}*/
export enum ParentNodeDetectionModes {
/**
Nodes should always be pasted into the root group.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentNodeDetectionModes%23ROOT Online Documentation}*/
ROOT,
/**
If exactly one group node is selected the nodes should be pasted into that group.
As long as {@link ParentNodeDetectionModes#ALLOW_SELF} is not set, too, a node will not be pasted into its original
node. Note that it can be pasted into a copy of its original, though.
If this setting is combined with {@link ParentNodeDetectionModes#ROOT} or {@link ParentNodeDetectionModes#PREVIOUS_PARENT}
it will override that setting if a group node is selected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentNodeDetectionModes%23SELECTION Online Documentation}*/
SELECTION,
/**
A node should be pasted into its original parent.
If the parent has been removed from the graph in between the copy and paste operation the node will be placed into the
parent's parent.
If the previous parent has been moved too far away the node will be placed into the root unless {@link ParentNodeDetectionModes#FORCE}
is set, too.
If this setting is combined with {@link ParentNodeDetectionModes#SELECTION} or {@link ParentNodeDetectionModes#AT_LOCATION}
it will be overridden by the other setting.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentNodeDetectionModes%23PREVIOUS_PARENT Online Documentation}*/
PREVIOUS_PARENT,
/**
A node should be pasted into the group node at the location where the node is pasted to or the root if there is no group
node.
As long as {@link ParentNodeDetectionModes#ALLOW_SELF} is not set, too, the node will not be pasted into its
original node. Note that it can be pasted into a copy of its original, though.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentNodeDetectionModes%23AT_LOCATION Online Documentation}*/
AT_LOCATION,
/**
When used in combination with {@link ParentNodeDetectionModes#SELECTION} or {@link ParentNodeDetectionModes#AT_LOCATION}
this setting will allow pasting nodes into their original.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentNodeDetectionModes%23ALLOW_SELF Online Documentation}*/
ALLOW_SELF,
/**
If combined with {@link ParentNodeDetectionModes#PREVIOUS_PARENT} forces the node to be pasted into its original
parent, even if the parent has moved far away and pasting the node would increase the parents size considerably.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentNodeDetectionModes%23FORCE Online Documentation}*/
FORCE,
/**
A bitmask that can be used to query the actual mode ({@link ParentNodeDetectionModes#ROOT},
{@link ParentNodeDetectionModes#SELECTION}, {@link ParentNodeDetectionModes#AT_LOCATION}, or
{@link ParentNodeDetectionModes#PREVIOUS_PARENT}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentNodeDetectionModes%23MODE_MASK Online Documentation}*/
MODE_MASK,
/**
A bitmask that can be used to query modifications of the actual mode such as {@link ParentNodeDetectionModes#ALLOW_SELF}
or {@link ParentNodeDetectionModes#FORCE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentNodeDetectionModes%23MODIFIER_MASK Online Documentation}*/
MODIFIER_MASK
}
type ParentNodeDetectionModesStringValues = "root" | "selection" | "previous-parent" | "at-location" | "allow-self" | "force" | "mode-mask" | "modifier-mask";
/** Automatic conversion to {@link ParentNodeDetectionModes}. */
type ParentNodeDetectionModesConvertible = ParentNodeDetectionModesStringValues | ParentNodeDetectionModesStringValues[];
export interface GraphClipboard extends YObject {
}
/**
Clipboard implementation for {@link IGraph} instances.
This implementation copies a given graph to an internally held
{@link GraphClipboard#clipboardGraph clipboard graph}. The clipboard graph can then be pasted to a target graph instance.
The clipboard can be configured to copy and paste {@link GraphClipboard#independentCopyItems items that depend on other items} without copying the items it depends
on, e.g. labels, ports, edges, and bends.
Also depending on the selection in the target graph, items can be pasted at multiple locations at once, e.g. a single
label can be copied to the clipboard independently and pasted onto several selected target items.
@see IClipboardHelper
@see GraphClipboard#cut
@see GraphClipboard#onCopy
@see GraphClipboard#clipboardGraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard Online Documentation}*/
export class GraphClipboard {
/**
Creates a new initially empty clipboard.
@param [options] The parameters to pass.
@param [options.pasteDelta] A vector that will be used to move nodes and bends by after they have been pasted to the target graph. This option sets the {@link GraphClipboard#pasteDelta} property on the created object.
@param [options.toClipboardCopier] The {@link GraphCopier} instance that will be used to copy the graph to the clipboard. This option sets the {@link GraphClipboard#toClipboardCopier} property on the created object.
@param [options.fromClipboardCopier] The {@link GraphCopier} instance that will be used to copy the graph from the clipboard to the target graph. This option sets the {@link GraphClipboard#fromClipboardCopier} property on the created object.
@param [options.duplicateCopier] The {@link GraphCopier} instance that will be used to duplicate elements inside the graph. This option sets the {@link GraphClipboard#duplicateCopier} property on the created object.
@param [options.idProvider] The {@link IClipboardIdProvider} used by this instance to provide IDs for copied items. This option sets the {@link GraphClipboard#idProvider} property on the created object.
@param [options.clipboardGraph] The graph that contains the clipboard's contents. This option sets the {@link GraphClipboard#clipboardGraph} property on the created object.
@param [options.parentNodeDetection] Whether or not nodes should be automatically put into a group node and if so how this group will be determined. This option sets the {@link GraphClipboard#parentNodeDetection} property on the created object.
@param [options.dependentCopyItems] The type of items that depend on other elements but should automatically be copied implicitly to the clipboard if all elements that it depends on are explicitly selected for copying. This option sets the {@link GraphClipboard#dependentCopyItems} property on the created object.
@param [options.independentCopyItems] The type of items that can be copied and pasted independently of their owning element. This option sets the {@link GraphClipboard#independentCopyItems} property on the created object.
@param [options.copyItems] The type of items that can be copied and pasted at all using the clipboard. This option sets the {@link GraphClipboard#copyItems} property on the created object.
@param [options.ignoreEdgesWithOneNode] Whether to paste edges whose source or target node was selected when copying to the clipboard while the other node was not. This option sets the {@link GraphClipboard#ignoreEdgesWithOneNode} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard Online Documentation}*/
constructor(options?: {
pasteDelta?: Point | PointConvertible;
toClipboardCopier?: GraphCopier;
fromClipboardCopier?: GraphCopier;
duplicateCopier?: GraphCopier;
idProvider?: IClipboardIdProvider;
clipboardGraph?: IGraph;
parentNodeDetection?: ParentNodeDetectionModes | ParentNodeDetectionModesConvertible;
dependentCopyItems?: GraphItemTypes | GraphItemTypesConvertible;
independentCopyItems?: GraphItemTypes | GraphItemTypesConvertible;
copyItems?: GraphItemTypes | GraphItemTypesConvertible;
ignoreEdgesWithOneNode?: boolean;
});
/**
Clears the clipboard's contents.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23clear() Online Documentation}*/
clear(): void;
/**
Copies the elements indicated by the filter to the clipboard graph.
For each element copied to the clipboard graph the {@link GraphClipboard#onElementCopied} delegate will be called.
The actual work of this method is delegated to the {@link GraphClipboard#toClipboardCopier}'s Copy method.
@param sourceGraph The graph to copy the contents from.
@param filter The filter to apply to the source graph's elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23copy(IGraph%2Cfunction(IModelItem)%3Aboolean) Online Documentation}*/
copy(sourceGraph: IGraph, filter: ((obj: IModelItem) => boolean) | null): void;
/**
Creates the graph used for the clipboard.
@returns A new readily configured {@link DefaultGraph} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createClipboardGraph() Online Documentation}*/
createClipboardGraph(): IGraph;
/**
Creates a predicate that is based upon the outcome of the {@link IClipboardHelper#shouldCopy} method that can be
found in the {@link ILookup#lookup} of the provided {@link IModelItem}.
@returns A predicate that uses {@link IClipboardHelper#shouldCopy}.
@see GraphClipboard#onCopy
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createClipBoardHelperCopyFilter() Online Documentation}*/
createClipBoardHelperCopyFilter(): (obj: IModelItem) => boolean;
/**
Creates a predicate that is based upon the outcome of the {@link IClipboardHelper#shouldCut} method that can be
found in the {@link ILookup#lookup} of the provided {@link IModelItem}.
@returns A predicate that uses {@link IClipboardHelper#shouldCut}.
@see GraphClipboard#cut
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createClipBoardHelperCutFilter() Online Documentation}*/
createClipBoardHelperCutFilter(): (obj: IModelItem) => boolean;
/**
Creates a predicate that is based upon the outcome of the {@link IClipboardHelper#shouldPaste} method that can be
found in the {@link ILookup#lookup} of the provided {@link IModelItem}.
@returns A predicate that uses {@link IClipboardHelper#shouldPaste}.
@see GraphClipboard#cut
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createClipBoardHelperPasteFilter() Online Documentation}*/
createClipBoardHelperPasteFilter(): (obj: IModelItem) => boolean;
/**
Factory method that creates the context for the upcoming calls to the {@link IClipboardHelper} methods.
@param sourceGraph The source graph of the operation.
@param targetGraph The target graph of the operation.
@returns A context that has the properties configured according to the parameters.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createContext(IGraph%2CIGraph) Online Documentation}*/
createContext(sourceGraph: IGraph, targetGraph: IGraph): IGraphClipboardContext;
/**
Creates the default {@link IClipboardIdProvider} used by this instance to provide IDs for copied items.
The default implementation simply uses a reference to the original item as ID.
Custom ID can be used to map between elements in different graphs.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createDefaultClipboardIdProvider() Online Documentation}*/
createDefaultClipboardIdProvider(): IClipboardIdProvider;
/**
Creates a predicate for use in the {@link GraphClipboard#onCopy} method.
This filter uses two predicates to decide whether a graph item can be copied. The base set of items specified by this
filter is induced by the {@link GraphClipboard#createSelectionFilter}. Optionally, this set can be restricted
further by an {@link IClipboardHelper#shouldCopy} predicate. If such a predicate returns <code>false</code> for the item itself
or any item this item depends on, then the item will not be copied. Items without such a predicate are handled solely
according to their selection state.
@param selection The selection model to base the filter on.
@param graph The graph to use for determining adjacency.
@returns A predicate that filters subgraph induced by the selection of nodes and restricted by an optional {@link IClipboardHelper#shouldCopy}
predicate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createDefaultCopyFilter(IGraphSelection%2CIGraph) Online Documentation}*/
createDefaultCopyFilter(selection: IGraphSelection, graph: IGraph): (obj: IModelItem) => boolean;
/**
Creates a predicate for use in the {@link GraphClipboard#cut} method.
This filter uses two predicates to decide whether a graph item can be cut. The base set of items specified by this
filter is induced by the {@link GraphClipboard#createSelectionFilter}. Optionally, this set can be restricted
further by an {@link IClipboardHelper#shouldCut} predicate. If such a predicate returns <code>false</code> for the item itself
or any item this item depends on, then the item will not be cut. Items without such a predicate are handled solely
according to their selection state.
@param selection The selection model to base the filter on.
@param graph The graph to use for determining adjacency.
@returns A predicate that filters subgraph induced by the selection of nodes and restricted by an optional {@link IClipboardHelper#shouldCut}
predicate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createDefaultCutFilter(IGraphSelection%2CIGraph) Online Documentation}*/
createDefaultCutFilter(selection: IGraphSelection, graph: IGraph): (obj: IModelItem) => boolean;
/**
Creates a predicate for use in the {@link GraphClipboard#duplicate} method.
By default this method uses the same filter implementation as {@link GraphClipboard#createDefaultCopyFilter}: This
filter uses two predicates to decide whether a graph item will be duplicated. The base set of items specified by this
filter is induced by the {@link GraphClipboard#createSelectionFilter}. Optionally, this set can be restricted
further by an {@link IClipboardHelper#shouldCopy} predicate. If such a predicate returns <code>false</code> for the item itself
or any item this item depends on, then the item will not be copied. Items without such a predicate are handled solely
according to their selection state.
@param selection The selection model to base the filter on.
@param graph The graph to use for determining adjacency.
@returns A predicate that filters subgraph induced by the selection of nodes and restricted by an optional {@link IClipboardHelper#shouldCopy}
predicate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createDefaultDuplicateFilter(IGraphSelection%2CIGraph) Online Documentation}*/
createDefaultDuplicateFilter(selection: IGraphSelection, graph: IGraph): (obj: IModelItem) => boolean;
/**
Factory method that creates the {@link GraphClipboard#duplicateCopier}.
@returns A new {@link GraphCopier} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createDuplicateCopier() Online Documentation}*/
createDuplicateCopier(): GraphCopier;
/**
Factory method that creates the {@link GraphClipboard#fromClipboardCopier}.
@returns A new {@link GraphCopier} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createFromClipboardCopier() Online Documentation}*/
createFromClipboardCopier(): GraphCopier;
/**
Creates a predicate for use in the {@link GraphClipboard#cut} and {@link GraphClipboard#onCopy} methods.
This predicate is based on the element induced by the set of selected nodes, e.g. each edge that has both of its
adjacent port owners selected will be included as well as each label and port whose owner is included. Also ports where
no adjacent edge is included (but edges exist), will not be included, unless the {@link IPortDefaults#autoCleanUp} is set to <code>false</code>.
@param selection The selection model to base the filter on.
@param graph The graph to use for determining adjacency.
@returns A predicate that filters subgraph induced by the selection of nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createSelectionFilter(IGraphSelection%2CIGraph) Online Documentation}*/
createSelectionFilter(selection: IGraphSelection, graph: IGraph): (obj: IModelItem) => boolean;
/**
Factory method that creates the {@link GraphClipboard#toClipboardCopier}.
@returns A new {@link GraphCopier} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createToClipboardCopier() Online Documentation}*/
createToClipboardCopier(): GraphCopier;
/**
Creates an induced predicate from <code>coreFilter</code> for use in the {@link GraphClipboard#cut} and {@link GraphClipboard#onCopy}
methods that takes dependent items into account.
This method can create a predicate for only a core set of items which is then extended or restricted depending on the
predicate values of graph item dependencies.
The <code>restrictive</code> parameter determines whether the result restricts or extends the set of elements handled by <code>coreFilter</code>:
<ul>
<li>If <code>restrictive</code> is <code>false</code>, an item is allowed if either <code>coreFilter</code> returns <code>true</code> for itself or if all items it depends on
(e.g. label/port owner, source/target port etc.) are allowed by <code>coreFilter</code>.
<li>If <code>restrictive</code> is <code>true</code>, an item is forbidden if either <code>coreFilter</code> returns <code>false</code> for itself or if any item it depends on
(e.g. label/port owner, source/target port etc.) is forbidden by <code>coreFilter</code>.
</ul>
@param coreFilter The core predicate for the base set of elements.
@param graph The graph to use for determining adjacency.
@param restrictive Whether to create a restriction or extension for the <code>coreFilter</code> predicate
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23createWrappedFilter(function(IModelItem)%3Aboolean%2CIGraph%2Cboolean) Online Documentation}*/
static createWrappedFilter(coreFilter: (obj: IModelItem) => boolean, graph: IGraph, restrictive: boolean): (obj: IModelItem) => boolean;
/**
Cuts the items indicated by <code>filter</code> to the clipboard and removes them from the graph.
This method delegates the core work to {@link GraphClipboard#onCopy} which will use the {@link GraphClipboard#toClipboardCopier} to copy the cut elements
to the clipboard.
@param sourceGraph The graph to cut the items from.
@param filter The filter to use.
@see GraphClipboard#createSelectionFilter
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23cut(IGraph%2Cfunction(IModelItem)%3Aboolean) Online Documentation}*/
cut(sourceGraph: IGraph, filter: ((obj: IModelItem) => boolean) | null): void;
/**
Duplicates the elements indicated by the filter.
The {@link GraphClipboard#clipboardGraph} is not altered by this operation.
@param context The current {@link IInputModeContext}.
@param sourceGraph The graph whose elements should be duplicated.
@param filter A predicate which returns <code>true</code> for elements which should be duplicated.
@param [elementDuplicated=null] An element copied callback which gets invoked for each element which has been duplicated. The two parameters passed into
the callback are guaranteed to be part of the <code>sourceGraph</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23duplicate(IInputModeContext%2CIGraph%2Cfunction(IModelItem)%3Aboolean%2Cfunction(IModelItem%2CIModelItem)%3Avoid) Online Documentation}*/
duplicate(context: IInputModeContext | null, sourceGraph: IGraph, filter: ((obj: IModelItem) => boolean) | null, elementDuplicated?: ((original: IModelItem, copy: IModelItem) => void) | null): void;
/**
Retrieves the {@link IClipboardHelper} instance associated with the given item.
This implementation uses the item's {@link ILookup#lookup} method to retrieve the helper instance.
@param item The item to find the clipboard helper for.
@returns An instance or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getClipboardHelper(IModelItem) Online Documentation}*/
getClipboardHelper(item: IModelItem): IClipboardHelper | null;
/**
Gets a folding state of an edge for a given {@link FoldingEdgeStateId}.
This method might be overridden to map folding states for edges whose source or target node's hierarchy might have
changed during a paste operation.
@param edge The edge to get the folding state for.
@param id The original {@link FoldingEdgeStateId} to get the folding state for. Passing <code>null</code> will return the folding state
which was visible when the edge was copied into the clipboard.
@returns The folding state which is mapped to the given <code>id</code>. May be <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getFoldingEdgeState(IEdge%2CFoldingEdgeStateId) Online Documentation}*/
getFoldingEdgeState(edge: IEdge, id: FoldingEdgeStateId | null): IEdge | null;
/**
* Returns the {@link IClipboardIdProvider#getId ID} which is linked with the given clipboard item.
* Meant for use in methods {@link GraphClipboard#getTargetNode}, {@link GraphClipboard#getTargetPort},
* {@link GraphClipboard#getTargetLabeledItem}, {@link GraphClipboard#getTargetPortOwner}, or
* {@link GraphClipboard#getTargetEdge}. Can be used together with the {@link GraphClipboard#idProvider} to retrieve the item in the original graph
* from which the <code>clipboardItem</code> has been copied.
*
* <br><b>Retrieving the original item which has been copied into the clipboard</b>
* <pre>
* /**
*  * @param {ILabel} clipboardLabel - The label to be copied.
*  *‍/
* getTargetLabeledItem(clipboardLabel) {
*   // clipboardLabel lives in the clipboard graph
*
*   // get the original label, i.e. the label which has been copied from the original graph
*   const originalLabel = this.idProvider.getItem(
*     this.clipboardContext,
*     this.getId(clipboardLabel)
*   )
*
*   // return the original label's owner as the pasted label's owner
*   return originalLabel.owner
* }
* </pre>
*
* @param clipboardItem An {@link IModelItem} which lives in the clipboard graph.
* @returns The {@link IModelItem} which lives in the original graph. The ID which represents the item from which <code>clipboardItem</code>
* has originally been copied into the clipboard. {@link GraphClipboard#idProvider}'s {@link IClipboardIdProvider#getItem} can be used to retrieve the
* original item. Note that both, the returned ID as well as the item returned by {@link IClipboardIdProvider#getItem}
* might be <code>null</code>.
* @protected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getId(IModelItem) Online Documentation}
*/
getId(clipboardItem: IModelItem): any | null;
/**
Gets a {@link ILabelModelParameter} for the given <code>label</code> to be placed at the given <code>newOwner</code>.
Implementations might delegate to {@link GraphCopier#copyLabelLayoutParameter} to copy the existing parameter or
use the {@link IGraph#nodeDefaults} or {@link IGraph#edgeDefaults} to create a new one. Note that the type of the owner might have changed during the copy operation.
@param copier The {@link GraphCopier} whose {@link GraphCopier#copyLabelLayoutParameter} method can be used to copy the
existing parameter.
@param graph The {@link IGraph} whose {@link IGraph#nodeDefaults} or {@link IGraph#edgeDefaults} can be used to create a new parameter.
@param label The label to be copied.
@param newOwner The new owner for the label.
@returns An {@link ILabelModelParameter} for the copied label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getLabelLayoutParameter(GraphCopier%2CIGraph%2CILabel%2CILabelOwner) Online Documentation}*/
getLabelLayoutParameter(copier: GraphCopier, graph: IGraph, label: ILabel, newOwner: ILabelOwner): ILabelModelParameter;
/**
Retrieves the memento associated with an item in the clipboard graph that has been stored with the item during {@link GraphClipboard#onElementCopied}
and {@link GraphClipboard#onElementCut}.
@param clipboardItem The item in the clipboard graph.
@returns The object returned by the {@link IClipboardHelper} methods or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getMemento(IModelItem) Online Documentation}*/
getMemento(clipboardItem: IModelItem): any | null;
/**
Gets a {@link IPortLocationModelParameter} for the given <code>originalPort</code> to be placed at the given <code>newOwner</code>.
Implementations might delegate to {@link GraphCopier#copyPortLocationParameter} to copy the existing parameter or
use the {@link IGraph#nodeDefaults} or {@link IGraph#edgeDefaults} to create a new one. Note that the type of the owner might have changed during the copy operation.
@param copier The {@link GraphCopier} whose {@link GraphCopier#copyPortLocationParameter} method can be used to copy the
existing parameter.
@param graph The {@link IGraph} whose {@link IGraph#nodeDefaults} or {@link IGraph#edgeDefaults} can be used to create a new parameter.
@param originalPort The port to be copied.
@param newOwner The new owner for the port.
@returns An {@link IPortLocationModelParameter} for the copied port.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getPortLocationParameter(GraphCopier%2CIGraph%2CIPort%2CIPortOwner) Online Documentation}*/
getPortLocationParameter(copier: GraphCopier, graph: IGraph, originalPort: IPort, newOwner: IPortOwner): IPortLocationModelParameter;
/**
Provides an {@link IEdge} where the given <code>clipboardBend</code> can be added to.
Called from within
Paste(yWorks.Graph.IGraph,System.Predicate{yWorks.Graph.IModelItem},yWorks.Graph.ElementCopiedCallback) for
{@link IBend}s whose edge is not copied to provide a new owner. Might return <code>null</code> if no target for the given <code>clipboardBend</code>
is found. In this case the bend will not be copied.
@param clipboardBend The bend to be copied from the {@link GraphClipboard#clipboardGraph}.
@returns An {@link IEdge} for the bend to be added to or <code>null</code> if the bend should not be copied.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getTargetEdge(IBend) Online Documentation}*/
getTargetEdge(clipboardBend: IBend): IEdge | null;
/**
* Provides an {@link ILabelOwner} where the given <code>clipboardLabel</code> can be added to.
* Called from within
* Paste(yWorks.Graph.IGraph,System.Predicate{yWorks.Graph.IModelItem},yWorks.Graph.ElementCopiedCallback) for
* {@link ILabel}s whose owners are not copied to provide a new owner. Might return <code>null</code> if no target for the given <code>clipboardLabel</code>
* is found. In this case the label will not be copied.
*
* <br><b>Retrieving the original item which has been copied into the clipboard</b>
* <pre>
* /**
*  * @param {ILabel} clipboardLabel - The label to be copied.
*  *‍/
* getTargetLabeledItem(clipboardLabel) {
*   // clipboardLabel lives in the clipboard graph
*
*   // get the original label, i.e. the label which has been copied from the original graph
*   const originalLabel = this.idProvider.getItem(
*     this.clipboardContext,
*     this.getId(clipboardLabel)
*   )
*
*   // return the original label's owner as the pasted label's owner
*   return originalLabel.owner
* }
* </pre>
*
* @param clipboardLabel The label to be copied from the {@link GraphClipboard#clipboardGraph}.
* @returns An {@link ILabelOwner} for the label to be added to or <code>null</code> if the label should not be copied.
* @protected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getTargetLabeledItem(ILabel) Online Documentation}
*/
getTargetLabeledItem(clipboardLabel: ILabel): ILabelOwner | null;
/**
Provides a parent group node for the given <code>clipboardNode</code>.
Called from within
Paste(yWorks.Graph.IGraph,System.Predicate{yWorks.Graph.IModelItem},yWorks.Graph.ElementCopiedCallback) for
{@link INode}s to place them inside a group node. Might return <code>null</code> if the given node should be placed at the root
or if the graph is not grouped.
@param clipboardNode The node to be copied from the {@link GraphClipboard#clipboardGraph}.
@returns The new parent group of the node or <code>null</code> if the node should be placed at the root of the hierarchy or if the graph is
not grouped.
@see GraphClipboard#parentNodeDetection
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getTargetNode(INode) Online Documentation}*/
getTargetNode(clipboardNode: INode): INode | null;
/**
Provides an {@link IPort} where the given <code>clipboardEdge</code> can be added to.
Called from within
Paste(yWorks.Graph.IGraph,System.Predicate{yWorks.Graph.IModelItem},yWorks.Graph.ElementCopiedCallback) for
{@link IEdge}s whose source or target ports are not copied to provide a new port. Might return <code>null</code> if no target
for the given <code>clipboardEdge</code> is found. In this case the edge will not be copied.
@param clipboardEdge The edge to be copied from the {@link GraphClipboard#clipboardGraph}.
@param atSource Whether to look for a source port.
@returns An {@link IPort} for the edge to be added to or <code>null</code> if the edge should not be copied.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getTargetPort(IEdge%2Cboolean) Online Documentation}*/
getTargetPort(clipboardEdge: IEdge, atSource: boolean): IPort | null;
/**
Provides an {@link IPortOwner} where the given <code>clipboardPort</code> can be added to.
Called from within
Paste(yWorks.Graph.IGraph,System.Predicate{yWorks.Graph.IModelItem},yWorks.Graph.ElementCopiedCallback) for
{@link IPort}s whose owners are not copied to provide a new owner. Might return <code>null</code> if no target for the given <code>clipboardPort</code>
is found. In this case the port will not be copied.
@param clipboardPort The port to be copied from the {@link GraphClipboard#clipboardGraph}.
@returns An {@link IPortOwner} for the port to be added to or <code>null</code> if the port should not be copied.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23getTargetPortOwner(IPort) Online Documentation}*/
getTargetPortOwner(clipboardPort: IPort): IPortOwner | null;
/**
Whether the given <code>item</code> is a "dummy" item which should not be pasted.
A dummy item is used to keep {@link IModelItem}s which cannot exist without an owner in the {@link GraphClipboard#clipboardGraph} (e.g.
{@link ILabel}s).
@param item The item to test.
@returns <code>true</code> if the item is a "dummy" item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23isDummy(IModelItem) Online Documentation}*/
isDummy(item: IModelItem | null): boolean;
/**
Whether the given <code>item</code> exists in a folding state of its <code>owner</code>.
@param owner The owner of the given item.
@param item An {@link ILabel}, {@link IPort}, or {@link IBend}.
@returns <code>true</code> if the given item exists in a folding state.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23isFoldingStateElement(IModelItem%2CIModelItem) Online Documentation}*/
static isFoldingStateElement(owner: IModelItem, item: IModelItem | null): boolean;
/**
Calls Copy on the current {@link GraphClipboard#toClipboardCopier} instance or {@link GraphClipboard#fromClipboardCopier} instance, depending on whether the <code>targetGraph</code> is the {@link GraphClipboard#clipboardGraph} instance.
@param sourceGraph The source graph to pass to Copy
@param filter The predicate to pass on.
@param targetGraph The target graph to pass on.
@param targetRootNode The node to use as the root node in the grouped <code>targetGraph</code>.
@param elementCopiedCallback The delegate to pass on. For each element that is copied during the operation to the clipboard, this callback will be
passed the original and the copy of the item. In case the <code>sourceGraph</code> is a {@link FoldingManager#views folding view graph} both elements are passed
as {@link FoldingManager#masterGraph master} elements or {@link FoldingManager#getFolderNodeState folding state elements} if a master element does not exist.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23onCopy(IGraph%2Cfunction(IModelItem)%3Aboolean%2CIGraph%2CINode%2Cfunction(IModelItem%2CIModelItem)%3Avoid) Online Documentation}*/
onCopy(sourceGraph: IGraph, filter: (obj: IModelItem) => boolean, targetGraph: IGraph, targetRootNode: INode | null, elementCopiedCallback: ((original: IModelItem, copy: IModelItem) => void) | null): void;
/**
A callback that will be invoked for each item that has been copied into the clipboard graph in response to a {@link GraphClipboard#onCopy}
operation.
This implementation will {@link ILookup#lookup} a {@link IClipboardHelper} for each item in the original graph
and remember the memento retrieved by the {@link IClipboardHelper#copy} method for later use during the {@link GraphClipboard#onElementPasted}
operation.
@param original The original element from the source graph instance. If the source graph is a managed folding view, this is always a
master item.
@param copy The copy from the clipboard graph.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23onElementCopied(IModelItem%2CIModelItem) Online Documentation}*/
onElementCopied(original: IModelItem, copy: IModelItem): void;
/**
A callback function that will be invoked for each item that has been copied into the clipboard graph in response to a {@link GraphClipboard#cut}
operation.
This implementation will {@link ILookup#lookup} a {@link IClipboardHelper} for each item in the original graph
and remember the memento retrieved by the {@link IClipboardHelper#cut} method for later use during the {@link GraphClipboard#onElementPasted}
operation.
@param original The original element from the source graph instance. If the source graph is a managed folding view, this is always a
master item.
@param copy The copy from the clipboard graph.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23onElementCut(IModelItem%2CIModelItem) Online Documentation}*/
onElementCut(original: IModelItem, copy: IModelItem): void;
/**
A callback function that will be invoked for each item that has been pasted into the target graph in response to a
Paste(IGraph) operation.
This method will use a previously stored {@link IClipboardHelper} to let it {@link IClipboardHelper#paste finish} its work.
@param original The original element from the clipboard graph instance. If the target graph is a managed folding view, this is always a
master item.
@param copy The copy from the target graph.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23onElementPasted(IModelItem%2CIModelItem) Online Documentation}*/
onElementPasted(original: IModelItem, copy: IModelItem): void;
/**
Calls Copy on the current {@link GraphClipboard#fromClipboardCopier} instance, depending on whether the <code>targetGraph</code> is the {@link GraphClipboard#clipboardGraph} instance.
@param sourceGraph The source graph to pass to Copy
@param filter The predicate to pass on.
@param targetGraph The target graph to pass on.
@param targetRootNode The node to use as the root node in the grouped <code>targetGraph</code>.
@param elementCopiedCallback The delegate to pass on. For each element that is copied during the operation to the clipboard, this callback will be
passed the original and the copy of the item. In case the <code>targetGraph</code> is a {@link FoldingManager#views folding view graph} the original element is
passed as {@link FoldingManager#masterGraph master} elements or {@link FoldingManager#getFolderNodeState folding state elements} if a master element does not exist. The copied element is always
part of the <code>targetGraph</code>.
@param targetFilter This predicate is used to determine which {@link INode}s and {@link IEdge}s can be used as targets for paste
operations, e.g. if a labels or ports are copied without their owners. If <code>null</code> all model items can be used as targets.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23onPaste(IGraph%2Cfunction(IModelItem)%3Aboolean%2CIGraph%2CINode%2Cfunction(IModelItem%2CIModelItem)%3Avoid%2Cfunction(IModelItem)%3Aboolean) Online Documentation}*/
onPaste(sourceGraph: IGraph, filter: ((obj: IModelItem) => boolean) | null, targetGraph: IGraph, targetRootNode: INode | null, elementCopiedCallback: ((original: IModelItem, copy: IModelItem) => void) | null, targetFilter: ((obj: IModelItem) => boolean) | null): void;
/**
Pastes the contents from the {@link GraphClipboard#clipboardGraph} to the <code>targetGraph</code> after applying a <code>filter</code>.
For each element pasted into the graph the <code>elementPasted</code> delegate will be invoked and the {@link GraphClipboard#onElementPasted}
method will be called. The actual work of this method is delegated to the {@link GraphClipboard#fromClipboardCopier}'s Copy method. Providing a valid <code>targetFilter</code>
allows for pasting {@link ILabel}s, {@link IPort}s, and {@link IEdge}s without valid owners.
@param targetGraph The graph to paste the contents in.
@param [filter=null] The filter to apply to the clipboard graph.
@param [elementPasted=null] The callback to use for each element that has been pasted. For each element that is copied during the operation to the
clipboard, this callback will be passed the original and the copy of the item. In case the <code>targetGraph</code> is a {@link FoldingManager#views folding view graph}
the original element is passed as {@link FoldingManager#masterGraph master} elements or {@link FoldingManager#getFolderNodeState folding state elements} if a master element does not exist. The
copied element is always part of the <code>targetGraph</code>.
@param [targetFilter=null] A filter which returns <code>true</code> for model items which can be used as target for clipboard items which need an owner (e.g.
{@link ILabel}s which were copied without their owner). If <code>null</code> all model items can be used as targets.
@param [context=null] A valid input mode context. Passing a valid input mode context is required for keeping orthogonal edges at group nodes
whose size might be adjusted after the paste operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23paste(IGraph%2Cfunction(IModelItem)%3Aboolean%2Cfunction(IModelItem%2CIModelItem)%3Avoid%2Cfunction(IModelItem)%3Aboolean%2CIInputModeContext) Online Documentation}*/
paste(targetGraph: IGraph, filter?: ((obj: IModelItem) => boolean) | null, elementPasted?: ((original: IModelItem, copy: IModelItem) => void) | null, targetFilter?: ((obj: IModelItem) => boolean) | null, context?: IInputModeContext | null): void;
/**
Pastes the contents from the {@link GraphClipboard#clipboardGraph} to the <code>targetGraph</code> after applying a <code>filter</code>.
For each element pasted into the graph the <code>elementPasted</code> delegate will be invoked and the {@link GraphClipboard#onElementPasted}
method will be called. The actual work of this method is delegated to the {@link GraphClipboard#fromClipboardCopier}'s Copy method. Providing a valid <code>targetFilter</code>
allows for pasting {@link ILabel}s, {@link IPort}s, and {@link IEdge}s without valid owners.
@param options The parameters to pass.
@param options.targetGraph The graph to paste the contents in.
@param [options.filter=null] The filter to apply to the clipboard graph.
@param [options.elementPasted=null] The callback to use for each element that has been pasted. For each element that is copied during the operation to the
clipboard, this callback will be passed the original and the copy of the item. In case the <code>targetGraph</code> is a {@link FoldingManager#views folding view graph}
the original element is passed as {@link FoldingManager#masterGraph master} elements or {@link FoldingManager#getFolderNodeState folding state elements} if a master element does not exist. The
copied element is always part of the <code>targetGraph</code>.
@param [options.targetFilter=null] A filter which returns <code>true</code> for model items which can be used as target for clipboard items which need an owner (e.g.
{@link ILabel}s which were copied without their owner). If <code>null</code> all model items can be used as targets.
@param [options.context=null] A valid input mode context. Passing a valid input mode context is required for keeping orthogonal edges at group nodes
whose size might be adjusted after the paste operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23paste(IGraph%2Cfunction(IModelItem)%3Aboolean%2Cfunction(IModelItem%2CIModelItem)%3Avoid%2Cfunction(IModelItem)%3Aboolean%2CIInputModeContext) Online Documentation}*/
paste(options: {
targetGraph: IGraph;
filter?: ((obj: IModelItem) => boolean) | null;
elementPasted?: ((original: IModelItem, copy: IModelItem) => void) | null;
targetFilter?: ((obj: IModelItem) => boolean) | null;
context?: IInputModeContext | null;
}): void;
/**
Pastes the {@link GraphClipboard#clipboardGraph}'s contents into the <code>targetGraph</code>, selecting all pasted elements.
The actual work of this method is delegated to the {@link GraphClipboard#fromClipboardCopier}'s Copy method.
@param targetGraph The graph to paste into.
@param selection The selection model to set the selection on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23pasteAndSelect(IGraph%2CIGraphSelection) Online Documentation}*/
pasteAndSelect(targetGraph: IGraph, selection: IGraphSelection): void;
/**
Removes elements from the graph based on a predicate.
@param sourceGraph The graph to remove elements from.
@param predicate The predicate that decides which elements to remove.
@see GraphClipboard#cut
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23removeElements(IGraph%2Cfunction(IModelItem)%3Aboolean) Online Documentation}*/
removeElements(sourceGraph: IGraph, predicate: (obj: IModelItem) => boolean): void;
/**
Gets or sets a vector that will be used to move nodes and bends by after they have been pasted to the target graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23pasteDelta Online Documentation}*/
pasteDelta: Point;
/**
Gets the clipboard context for the current clipboard operation.
The context is only valid during {@link GraphClipboard#cut}, {@link GraphClipboard#copy},
{@link GraphClipboard#pasteAndSelect}, and {@link GraphClipboard#duplicate} and is <code>null</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23clipboardContext Online Documentation}*/
readonly clipboardContext: IGraphClipboardContext | null;
/**
Gets whether the clipboard graph is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23empty Online Documentation}*/
readonly empty: boolean;
/**
Gets or sets the {@link GraphCopier} instance that will be used to copy the graph to the clipboard.
This instance is lazily created using {@link GraphClipboard#createToClipboardCopier}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23toClipboardCopier Online Documentation}*/
toClipboardCopier: GraphCopier;
/**
Gets or sets the {@link GraphCopier} instance that will be used to copy the graph from the clipboard to the target
graph.
This instance is lazily created using {@link GraphClipboard#createFromClipboardCopier}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23fromClipboardCopier Online Documentation}*/
fromClipboardCopier: GraphCopier;
/**
Gets or sets the {@link GraphCopier} instance that will be used to duplicate elements inside the graph.
This instance is lazily created using {@link GraphClipboard#createDuplicateCopier}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23duplicateCopier Online Documentation}*/
duplicateCopier: GraphCopier;
/**
Gets or sets the {@link IClipboardIdProvider} used by this instance to provide IDs for copied items.
The default implementation simply uses a reference to the original item as ID.
Custom ID can be used to map between elements in different graphs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23idProvider Online Documentation}*/
idProvider: IClipboardIdProvider;
/**
Gets or sets the graph that contains the clipboard's contents.
@throws {Exception} <b>(ArgumentError)</b> If <code>value</code> is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23clipboardGraph Online Documentation}*/
clipboardGraph: IGraph;
/**
Gets or sets whether or not nodes should be automatically put into a group node and if so how this group will be
determined.
This property is used by {@link GraphClipboard#getTargetNode} to decide what node to yield as the target group node
for the newly pasted node.
The default is {@link ParentNodeDetectionModes#SELECTION}|{@link ParentNodeDetectionModes#PREVIOUS_PARENT}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23parentNodeDetection Online Documentation}*/
parentNodeDetection: ParentNodeDetectionModes;
/**
Gets or sets the type of items that depend on other elements but should automatically be copied implicitly to the
clipboard if all elements that it depends on are explicitly selected for copying.
{@link ILabel Labels} depend on their {@link ILabelOwner owner}, like {@link IPort ports} depend on their {@link IPortOwner owner}. Also {@link IBend bends} of course depend on their {@link IEdge owning edge}.
If the corresponding element type is set in this property, items that depend on other items are implicitly included in
the copy operation, if all of the items that they depend on are included recursively, even if they are not included
explicitly themselves.
The default is {@link GraphItemTypes#ALL} minus
{@link GraphItemTypes#NODE}, which means that selecting a number of nodes will automatically select all elements
that directly and indirectly depend on the existence of this set of nodes will be copied to the clipboard, too.
Technically nodes do not depend on anything. Setting this property to a value that includes {@link GraphItemTypes#NODE}
will include all child nodes for every selected group node.
@default GraphItemTypes.ALL & ~GraphItemTypes.NODE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23dependentCopyItems Online Documentation}*/
dependentCopyItems: GraphItemTypes;
/**
Gets or sets the type of items that can be copied and pasted independently of their owning element.
{@link ILabel Labels} depend on their {@link ILabelOwner owner}, like {@link IPort ports} depend on their {@link IPortOwner owner}. Also {@link IBend bends} of course depend on their {@link IEdge owning edge}.
If the corresponding element type is set in this property, items can be copied alone to the clipboard and be pasted onto
different owning elements.
This enables copying of a label to one or more owners, without having to copy the owner of the label. Also ports can be
copied to other {@link IPortOwner}s and bends to other edges.
The default is {@link GraphItemTypes#ALL} which means that all elements can be copied to the clipboard even if the
items that they depend on are not copied to the clipboard.
Technically owning elements are still part of the clipboard as
{@link GraphClipboard#isDummy dummy elements}, but these elements will not be pasted to the target during paste or duplicate operations.
@default GraphItemTypes.ALL
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23independentCopyItems Online Documentation}*/
independentCopyItems: GraphItemTypes;
/**
Gets or sets the type of items that can be copied and pasted at all using the clipboard.
If the corresponding element type is set in this property, items can be effectively copied.
If the corresponding element type is not set in this property, items will not be pasted, even if they have been
explicitly selected. They may copied to the clipboard and become part of the {@link GraphClipboard#clipboardGraph}, nevertheless though, if they are required
as {@link GraphClipboard#isDummy dummy elements}. But these elements will not be pasted to the target during paste or duplicate operations.
The default is {@link GraphItemTypes#ALL}.
@default GraphItemTypes.ALL
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23copyItems Online Documentation}*/
copyItems: GraphItemTypes;
/**
Gets or sets whether to paste edges whose source or target node was selected when copying to the clipboard while the
other node was not.
The default is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23ignoreEdgesWithOneNode Online Documentation}*/
ignoreEdgesWithOneNode: boolean;
/**
Adds the given listener for the <code>ElementsCut</code> event that occurs when the {@link GraphClipboard#cut} operation
finished successfully.
@param listener The listener to add.
@see GraphClipboard#removeElementsCutListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23ElementsCut Online Documentation}*/
addElementsCutListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ElementsCut</code> event that occurs when the {@link GraphClipboard#cut} operation
finished successfully.
@param listener The listener to remove.
@see GraphClipboard#addElementsCutListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23ElementsCut Online Documentation}*/
removeElementsCutListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>ElementsCopied</code> event that occurs when the {@link GraphClipboard#copy} operation
finished successfully.
@param listener The listener to add.
@see GraphClipboard#removeElementsCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23ElementsCopied Online Documentation}*/
addElementsCopiedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ElementsCopied</code> event that occurs when the {@link GraphClipboard#copy} operation
finished successfully.
@param listener The listener to remove.
@see GraphClipboard#addElementsCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23ElementsCopied Online Documentation}*/
removeElementsCopiedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>ElementsPasted</code> event that occurs when the {@link GraphClipboard#paste} operation
finished successfully.
@param listener The listener to add.
@see GraphClipboard#removeElementsPastedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23ElementsPasted Online Documentation}*/
addElementsPastedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ElementsPasted</code> event that occurs when the {@link GraphClipboard#paste} operation
finished successfully.
@param listener The listener to remove.
@see GraphClipboard#addElementsPastedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23ElementsPasted Online Documentation}*/
removeElementsPastedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>ElementsDuplicated</code> event that occurs when the {@link GraphClipboard#duplicate}
operation finished successfully.
@param listener The listener to add.
@see GraphClipboard#removeElementsDuplicatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23ElementsDuplicated Online Documentation}*/
addElementsDuplicatedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ElementsDuplicated</code> event that occurs when the {@link GraphClipboard#duplicate}
operation finished successfully.
@param listener The listener to remove.
@see GraphClipboard#addElementsDuplicatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphClipboard%23ElementsDuplicated Online Documentation}*/
removeElementsDuplicatedListener(listener: (sender: any, evt: EventArgs) => void): void;
static $class: Class;
private $_GraphClipboard: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphClipboard;
}
/**
Enumeration to indicate which items should be {@link ICloneable#clone cloned} during copy operations.
The enumeration can be used to create a bitwise combination of the types.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CloneTypes Online Documentation}*/
export enum CloneTypes {
/**
Indicates the {@link INodeStyle} constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CloneTypes%23NODE_STYLE Online Documentation}*/
NODE_STYLE,
/**
Indicates the {@link IEdgeStyle} constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CloneTypes%23EDGE_STYLE Online Documentation}*/
EDGE_STYLE,
/**
Indicates the {@link IPortStyle} constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CloneTypes%23PORT_STYLE Online Documentation}*/
PORT_STYLE,
/**
Indicates the {@link ILabelStyle} constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CloneTypes%23LABEL_STYLE Online Documentation}*/
LABEL_STYLE,
/**
Indicates the {@link ILabelModelParameter} constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CloneTypes%23LABEL_MODEL_PARAMETER Online Documentation}*/
LABEL_MODEL_PARAMETER,
/**
Indicates the {@link IPortLocationModelParameter} constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CloneTypes%23PORT_LOCATION_MODEL_PARAMETER Online Documentation}*/
PORT_LOCATION_MODEL_PARAMETER,
/**
Indicates the {@link ITagOwner#tag} constant.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CloneTypes%23TAGS Online Documentation}*/
TAGS,
/**
A combination of all flags in this enumeration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CloneTypes%23ALL Online Documentation}*/
ALL
}
type CloneTypesStringValues = "node-style" | "edge-style" | "port-style" | "label-style" | "label-model-parameter" | "port-location-model-parameter" | "tags" | "all";
/** Automatic conversion to {@link CloneTypes}. */
type CloneTypesConvertible = CloneTypesStringValues | CloneTypesStringValues[];
export interface GraphCopier extends YObject {
}
/**
Copies parts of an {@link IGraph} to another instance.
Instances of this class are used in {@link GraphClipboard}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier Online Documentation}*/
export class GraphCopier {
/**
Creates a new instance of an UtilGraphCopier with the default settings.
@param [options] The parameters to pass.
@param [options.clone] The types of items that should be {@link ICloneable#clone cloned} for the copy operation. This option sets the {@link GraphCopier#clone} property on the created object.
@param [options.referentialIdentityTypes] The types of objects where the referential identity should be kept across clipboard operations. This option sets the {@link GraphCopier#referentialIdentityTypes} property on the created object.
@param [options.copyGrouping] A property that determines whether grouping information should be copied to target graphs. This option sets the {@link GraphCopier#copyGrouping} property on the created object.
@param [options.copyFoldingStates] A property that determines whether {@link FoldingManager additional folding states}. This option sets the {@link GraphCopier#copyFoldingStates} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier Online Documentation}*/
constructor(options?: {
clone?: CloneTypes | CloneTypesConvertible;
referentialIdentityTypes?: CloneTypes | CloneTypesConvertible;
copyGrouping?: boolean;
copyFoldingStates?: boolean;
});
/**
Adds a bend to the targetEdge in the targetGraph as a copy of the source bend.
@param sourceGraph The source graph.
@param targetGraph The target graph.
@param targetEdge The target edge to add the bend to.
@param sourceBend The source bend to copy the values from.
@param offset The offset to apply to the copied bend.
@param index The index to add the bend at.
@returns The newly created bend.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23addBend(IGraph%2CIGraph%2CIEdge%2CIBend%2CPoint%2Cnumber) Online Documentation}*/
addBend(sourceGraph: IGraph, targetGraph: IGraph, targetEdge: IEdge, sourceBend: IBend, offset: Point, index: number): IBend | null;
/**
Adds a label to the target item in the targetGraph as a copy of the source label.
This implementation uses {@link GraphCopier#copyLabelLayoutParameter}, {@link GraphCopier#copyLabelStyle}, and
{@link GraphCopier#copyLabelTag} as callbacks.
@param sourceGraph The source graph.
@param targetGraph The target graph.
@param targetOwner The item owner in the target graph to add the label to.
@param sourceLabel The source label in the source graph.
@returns The newly created label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23addLabel(IGraph%2CIGraph%2CILabelOwner%2CILabel) Online Documentation}*/
addLabel(sourceGraph: IGraph, targetGraph: IGraph, targetOwner: ILabelOwner, sourceLabel: ILabel): ILabel | null;
/**
Adds a port to the targetPortOwner in the targetGraph as a copy of the source port.
This implementation does not need to copy the adjacent {@link IEdge}s of the <code>sourcePort</code>. This implementation uses
{@link GraphCopier#copyPortLocationParameter}, {@link GraphCopier#copyPortStyle}, and {@link GraphCopier#copyPortTag}
as callbacks.
@param sourceGraph The source graph.
@param targetGraph The target graph.
@param targetOwner The port owner in the target graph to add the port to.
@param sourcePort The source port in the source graph.
@returns The newly created port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23addPort(IGraph%2CIGraph%2CIPortOwner%2CIPort) Online Documentation}*/
addPort(sourceGraph: IGraph, targetGraph: IGraph, targetOwner: IPortOwner, sourcePort: IPort): IPort | null;
/**
This method places the given copy into an internal cache.
The cache can be queried using {@link GraphCopier#getCopy}. The cache is held until {@link GraphCopier#clearCopyCache}
is called.
Type parameter <code>T</code>: The type of the item to cache.
@param original The original item.
@param copy The copy of the item.
@see GraphCopier#getOrCreateCopy
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23cacheCopy(T%2CT) Online Documentation}*/
cacheCopy<T>(original: T | null, copy: T | null): void;
/**
Clears the cache for the {@link GraphCopier#cacheCopy cached copies} of the instance copied using this class.
This method is called before and after a copy operation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23clearCopyCache() Online Documentation}*/
clearCopyCache(): void;
/**
Copies a subgraph induced by a predicate from the <code>sourceGraph</code> to the <code>targetGraph</code>.
For each element that has been copied, the <code>elementCopiedCallback</code> will be invoked.
<b>Warning:</b> If elements from a {@link IFoldingView folding-enabled} graph should be copied both <code>sourceGraph</code> and <code>targetGraph</code> must be a
{@link FoldingManager#masterGraph}. Furthermore, folding must be enabled on both graphs.
@param sourceGraph The graph to copy the elements from.
@param filter The filter that decides which elements to copy.
@param targetGraph The graph to copy the elements to.
@param targetRootNode The group node that should act as the root node for the pasted elements. May be <code>null</code> if there is no such node.
@param [offset=null] The offset to apply to the copied items. Only nodes and bends will be offset explicitly, while other items will move
implicitly.
@param [elementCopiedCallback=null] A delegate to invoke for each element being copied.
@throws {Exception} <b>(ArgumentError)</b> if <code>targetRootNode</code> is not in <code>targetGraph</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copy(IGraph%2Cfunction(IModelItem)%3Aboolean%2CIGraph%2CINode%2CPoint%2Cfunction(IModelItem%2CIModelItem)%3Avoid) Online Documentation}*/
copy(sourceGraph: IGraph, filter: (obj: IModelItem) => boolean, targetGraph: IGraph, targetRootNode: INode | null, offset?: Point | null, elementCopiedCallback?: ((original: IModelItem, copy: IModelItem) => void) | null): void;
/**
Copies a subgraph induced by a predicate from the <code>sourceGraph</code> to the <code>targetGraph</code>.
For each element that has been copied, the <code>elementCopiedCallback</code> will be invoked.
<b>Warning:</b> If elements from a {@link IFoldingView folding-enabled} graph should be copied both <code>sourceGraph</code> and <code>targetGraph</code> must be a
{@link FoldingManager#masterGraph}. Furthermore, folding must be enabled on both graphs.
@param options The parameters to pass.
@param options.sourceGraph The graph to copy the elements from.
@param options.filter The filter that decides which elements to copy.
@param options.targetGraph The graph to copy the elements to.
@param options.targetRootNode The group node that should act as the root node for the pasted elements. May be <code>null</code> if there is no such node.
@param [options.offset=null] The offset to apply to the copied items. Only nodes and bends will be offset explicitly, while other items will move
implicitly.
@param [options.elementCopiedCallback=null] A delegate to invoke for each element being copied.
@throws {Exception} <b>(ArgumentError)</b> if <code>targetRootNode</code> is not in <code>targetGraph</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copy(IGraph%2Cfunction(IModelItem)%3Aboolean%2CIGraph%2CINode%2CPoint%2Cfunction(IModelItem%2CIModelItem)%3Avoid) Online Documentation}*/
copy(options: {
sourceGraph: IGraph;
filter: (obj: IModelItem) => boolean;
targetGraph: IGraph;
targetRootNode: INode | null;
offset?: Point | null;
elementCopiedCallback?: ((original: IModelItem, copy: IModelItem) => void) | null;
}): void;
/**
Copies a subgraph described by a predicate from the <code>sourceGraph</code> to the <code>targetGraph</code>.
For each element that has been copied, the <code>elementCopiedCallback</code> will be invoked. This method actually delegates to
Copy(IGraph,Predicate{IModelItem},IGraph,INode,PointD,ElementCopiedCallback) and provides the root as the "targetGraph"
parameter if there is a hierarchy associated with <code>targetGraph</code>.
<b>Warning:</b> If elements from a {@link IFoldingView folding-enabled} graph should be copied both <code>sourceGraph</code> and <code>targetGraph</code> must be a
{@link FoldingManager#masterGraph}. Furthermore, folding must be enabled on both graphs.
@param sourceGraph The graph to copy the elements from.
@param filter The filter that decides which elements to copy.
@param targetGraph The graph to copy the elements to.
@param [offset=null] The offset to apply to the copied items. Only nodes and bends will be offset explicitly, while other items will move
implicitly.
@param [elementCopiedCallback=null] A delegate to invoke for each element being copied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copy(IGraph%2Cfunction(IModelItem)%3Aboolean%2CIGraph%2CPoint%2Cfunction(IModelItem%2CIModelItem)%3Avoid) Online Documentation}*/
copy(sourceGraph: IGraph, filter: (obj: IModelItem) => boolean, targetGraph: IGraph, offset?: Point | null, elementCopiedCallback?: ((original: IModelItem, copy: IModelItem) => void) | null): void;
/**
Copies a subgraph described by a predicate from the <code>sourceGraph</code> to the <code>targetGraph</code>.
For each element that has been copied, the <code>elementCopiedCallback</code> will be invoked. This method actually delegates to
Copy(IGraph,Predicate{IModelItem},IGraph,INode,PointD,ElementCopiedCallback) and provides the root as the "targetGraph"
parameter if there is a hierarchy associated with <code>targetGraph</code>.
<b>Warning:</b> If elements from a {@link IFoldingView folding-enabled} graph should be copied both <code>sourceGraph</code> and <code>targetGraph</code> must be a
{@link FoldingManager#masterGraph}. Furthermore, folding must be enabled on both graphs.
@param options The parameters to pass.
@param options.sourceGraph The graph to copy the elements from.
@param options.filter The filter that decides which elements to copy.
@param options.targetGraph The graph to copy the elements to.
@param [options.offset=null] The offset to apply to the copied items. Only nodes and bends will be offset explicitly, while other items will move
implicitly.
@param [options.elementCopiedCallback=null] A delegate to invoke for each element being copied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copy(IGraph%2Cfunction(IModelItem)%3Aboolean%2CIGraph%2CPoint%2Cfunction(IModelItem%2CIModelItem)%3Avoid) Online Documentation}*/
copy(options: {
sourceGraph: IGraph;
filter: (obj: IModelItem) => boolean;
targetGraph: IGraph;
offset?: Point | null;
elementCopiedCallback?: ((original: IModelItem, copy: IModelItem) => void) | null;
}): void;
/**
Copies <code>sourceGraph</code> to <code>targetGraph</code>.
This method actually delegates to Copy(IGraph,Predicate{IModelItem},IGraph,INode,PointD,ElementCopiedCallback) and
provides the root as the "targetGraph" parameter if there is a hierarchy associated with <code>targetGraph</code>.
<b>Warning:</b> If elements from a {@link IFoldingView folding-enabled} graph should be copied both <code>sourceGraph</code> and <code>targetGraph</code> must be a
{@link FoldingManager#masterGraph}. Furthermore, folding must be enabled on both graphs.
@param sourceGraph The graph to copy.
@param targetGraph The graph to copy to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copy(IGraph%2CIGraph) Online Documentation}*/
copy(sourceGraph: IGraph, targetGraph: IGraph): void;
/**
Creates a copy of the <code>node</code> in the <code>targetGraph</code>.
This implementation does not need to copy {@link IPort}s or {@link ILabel}s. This implementation uses {@link GraphCopier#copyNodeStyle}
and {@link GraphCopier#copyNodeTag} as callbacks.
<b>Warning:</b> This method is only called if {@link GraphCopier#copyGrouping} is set to <code>true</code>. Otherwise, {@link GraphCopier#copyNode} is called instead.
@param sourceGraph The source graph.
@param targetGraph The target graph.
@param targetParent The parent of the group node in the target graph.
@param node The source node to create a copy of in the <code>targetGraph</code>.
@param offset The offset to apply to the copied node.
@returns The newly created node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyChildNode(IGraph%2CIGraph%2CINode%2CINode%2CPoint) Online Documentation}*/
copyChildNode(sourceGraph: IGraph, targetGraph: IGraph, node: INode, targetParent: INode | null, offset: Point): INode | null;
/**
Adds an edge to the targetGraph as a copy of the source edge.
This implementation does not need to copy the {@link IBend}s of the <code>sourceEdge</code>, nor should it copy
{@link IPort}s or {@link ILabel}s. This implementation uses {@link GraphCopier#copyEdgeStyle} and {@link GraphCopier#copyEdgeTag}
as callbacks.
@param sourceGraph The source graph.
@param targetGraph The target graph.
@param sourceEdge The edge in the source graph.
@param targetSourcePort The new source port in the target graph.
@param targetTargetPort The new target port in the target graph.
@returns The newly created edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyEdge(IGraph%2CIGraph%2CIEdge%2CIPort%2CIPort) Online Documentation}*/
copyEdge(sourceGraph: IGraph, targetGraph: IGraph, sourceEdge: IEdge, targetSourcePort: IPort, targetTargetPort: IPort): IEdge | null;
/**
Copies an {@link IEdgeStyle} instance.
This implementation uses the {@link ICloneable#clone} method to create a copy.
@param graph The graph that contains the edge.
@param edge The edge to copy the style from.
@returns The style to apply for the copy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyEdgeStyle(IGraph%2CIEdge) Online Documentation}*/
copyEdgeStyle(graph: IGraph, edge: IEdge): IEdgeStyle;
/**
Copies the {@link ITagOwner#tag} of an {@link IEdge}.
This implementation simply delegates to {@link GraphCopier#copyTag}
@param edge The old edge.
@param tag The tag to copy.
@returns The copy of the tag.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyEdgeTag(IEdge%2CObject) Online Documentation}*/
copyEdgeTag(edge: IEdge, tag: any | null): any | null;
/**
Creates a copy of the <code>sourceGroupNode</code> in the <code>targetGraph</code>.
This implementation does not need to copy {@link IPort}s or {@link ILabel}s. This implementation uses {@link GraphCopier#copyNodeStyle}
and {@link GraphCopier#copyNodeTag} as callbacks.
<b>Warning:</b> This method is only called if {@link GraphCopier#copyGrouping} is set to <code>true</code>. Otherwise, {@link GraphCopier#copyNode} is called instead.
@param sourceGraph The source graph.
@param targetGraph The target graph.
@param sourceGroupNode The source node to create a copy of in the <code>targetGraph</code>.
@param targetParent The parent of the group node in the target graph.
@param offset The offset to apply to the copied node.
@returns The newly created node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyGroupNode(IGraph%2CIGraph%2CINode%2CINode%2CPoint) Online Documentation}*/
copyGroupNode(sourceGraph: IGraph, targetGraph: IGraph, sourceGroupNode: INode, targetParent: INode | null, offset: Point): INode | null;
/**
Copies a {@link ILabelModelParameter} for a given label.
This implementation uses the {@link ICloneable#clone} method to create a copy.
@param graph The graph that contains the label.
@param label The label to copy the label model parameter from.
@returns The copy of the parameter to apply for the copy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyLabelLayoutParameter(IGraph%2CILabel) Online Documentation}*/
copyLabelLayoutParameter(graph: IGraph, label: ILabel): ILabelModelParameter;
/**
Copies an {@link ILabelStyle} instance.
This implementation uses the {@link ICloneable#clone} method to create a copy.
@param graph The graph that contains the label.
@param label The label to copy the style from.
@returns The style to apply for the copy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyLabelStyle(IGraph%2CILabel) Online Documentation}*/
copyLabelStyle(graph: IGraph, label: ILabel): ILabelStyle;
/**
Copies the {@link ITagOwner#tag} of a {@link ILabel}.
This implementation simply delegates to {@link GraphCopier#copyTag}
@param label The old label.
@param tag The tag to copy.
@returns The copy of the tag.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyLabelTag(ILabel%2CObject) Online Documentation}*/
copyLabelTag(label: ILabel, tag: any | null): any | null;
/**
Creates a copy of the <code>node</code> in the <code>targetGraph</code>.
This implementation does not need to copy {@link IPort}s or {@link ILabel}s. This implementation uses {@link GraphCopier#copyNodeStyle}
and {@link GraphCopier#copyNodeTag} as callbacks.
<b>Warning:</b> This method is only called if {@link GraphCopier#copyGrouping} is set to <code>false</code>. Otherwise, {@link GraphCopier#copyChildNode} and {@link GraphCopier#copyGroupNode}
are called instead.
@param sourceGraph The source graph.
@param targetGraph The target graph.
@param node The source node to create a copy of in the <code>targetGraph</code>.
@param offset The offset to apply to the copied node.
@returns The newly created node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyNode(IGraph%2CIGraph%2CINode%2CPoint) Online Documentation}*/
copyNode(sourceGraph: IGraph, targetGraph: IGraph, node: INode, offset: Point): INode | null;
/**
Copies an {@link INodeStyle} instance.
This implementation uses the {@link ICloneable#clone} method to create a copy.
@param graph The graph that contains the node.
@param node The node to copy the style from.
@returns The style to apply for the copy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyNodeStyle(IGraph%2CINode) Online Documentation}*/
copyNodeStyle(graph: IGraph, node: INode): INodeStyle;
/**
Copies the {@link ITagOwner#tag} of a {@link INode}.
This implementation simply delegates to {@link GraphCopier#copyTag}
@param node The old node.
@param tag The tag to copy.
@returns The copy of the tag.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyNodeTag(INode%2CObject) Online Documentation}*/
copyNodeTag(node: INode, tag: any | null): any | null;
/**
Copies a {@link IPortLocationModelParameter} for a given port.
This implementation uses the {@link ICloneable#clone} method to create a copy.
@param graph The graph that contains the port.
@param port The port to copy the model parameter from.
@returns The copy of the parameter to apply for the copy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyPortLocationParameter(IGraph%2CIPort) Online Documentation}*/
copyPortLocationParameter(graph: IGraph, port: IPort): IPortLocationModelParameter;
/**
Copies an {@link IPortStyle} instance.
This implementation uses the {@link ICloneable#clone} method to create a copy.
@param graph The graph that contains the port.
@param port The port to copy the style from.
@returns The style to apply for the copy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyPortStyle(IGraph%2CIPort) Online Documentation}*/
copyPortStyle(graph: IGraph, port: IPort): IPortStyle;
/**
Copies the {@link ITagOwner#tag} of a {@link IPort}.
This implementation simply delegates to {@link GraphCopier#copyTag}
@param port The old port.
@param tag The tag to copy.
@returns The copy of the tag.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyPortTag(IPort%2CObject) Online Documentation}*/
copyPortTag(port: IPort, tag: any | null): any | null;
/**
Callback that will copy a {@link ITagOwner#tag} for the target graph.
This implementation will try to {@link ICloneable#clone} the <code>tag</code> if the {@link GraphCopier#clone} property has the {@link CloneTypes#TAGS}
flag set. If this is not the case or the tag is not {@link ICloneable}, the instance will be returned.
@param owner The old owner of the tag.
@param tag The tag to copy.
@returns A copy of the tag.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyTag(IModelItem%2CObject) Online Documentation}*/
copyTag(owner: IModelItem, tag: any | null): any | null;
/**
Tries to obtain a previously {@link GraphCopier#cacheCopy cached} copy for the given original.
Values will be stored in the cache until the next call to {@link GraphCopier#clearCopyCache}. In order to indicate
that no copy is available, this implementation yields {@link GraphCopier#NO_COPY}. In contrast, <code>null</code> is a valid
return value that is the copied value of the original.
Type parameter <code>T</code>: The type of the item to copy.
@param itemType The type of the item to copy.
@param original The original item.
@returns The copy of <code>original</code> or {@link GraphCopier#NO_COPY} if no copy has been found.
@see GraphCopier#cacheCopy
@see GraphCopier#getOrCreateCopy
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23getCopy(Class%2CT) Online Documentation}*/
getCopy<T>(itemType: Class, original: T | null): T | null;
/**
Either yields a previously {@link GraphCopier#cacheCopy cached} copy for the given original or uses the <code>copyDelegate</code> to create the copy of the
original.
If there is a cache miss, the delegate will be used to create the copy and the copy will be placed into the {@link GraphCopier#cacheCopy cache}.
Type parameter <code>T</code>: The type of the item to copy.
@param tType The type of the item to copy.
@param original The original item.
@param copyDelegate The copy delegate to create the copy.
@returns A copy of the original, either cached, or newly created and then cached.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23getOrCreateCopy(Class%2CT%2Cfunction(T)%3AT) Online Documentation}*/
getOrCreateCopy<T>(tType: Class, original: T | null, copyDelegate: (arg: T) => T | null): T | null;
/**
Called after a bend has been copied.
Raises the {@link GraphCopier#addBendCopiedListener BendCopied} event.
@param original The original of the copied bend.
@param copy The copied bend.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23onBendCopied(IBend%2CIBend) Online Documentation}*/
onBendCopied(original: IBend, copy: IBend): void;
/**
Called after an edge has been copied.
Raises the {@link GraphCopier#addEdgeCopiedListener EdgeCopied} event.
@param original The original of the copied edge.
@param copy The copied edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23onEdgeCopied(IEdge%2CIEdge) Online Documentation}*/
onEdgeCopied(original: IEdge, copy: IEdge): void;
/**
Called after the entire graph has been copied.
Raises the {@link GraphCopier#addGraphCopiedListener GraphCopied} event.
@param sourceGraph The source graph from which the elements have been copied.
@param targetGraph The graph to which the elements have been copied.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23onGraphCopied(IGraph%2CIGraph) Online Documentation}*/
onGraphCopied(sourceGraph: IGraph, targetGraph: IGraph): void;
/**
Called after a label has been copied.
Raises the {@link GraphCopier#addLabelCopiedListener LabelCopied} event.
@param original The original of the copied label.
@param copy The copied label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23onLabelCopied(ILabel%2CILabel) Online Documentation}*/
onLabelCopied(original: ILabel, copy: ILabel): void;
/**
Called after a node has been copied.
Raises the {@link GraphCopier#addNodeCopiedListener NodeCopied} event.
@param original The original of the copied node.
@param copy The copied node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23onNodeCopied(INode%2CINode) Online Documentation}*/
onNodeCopied(original: INode, copy: INode): void;
/**
Emits the {@link GraphCopier#addObjectCopiedListener ObjectCopied} event.
@param evt The argument to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23onObjectCopied(ItemCopiedEventArgs%3CObject%3E) Online Documentation}*/
onObjectCopied(evt: ItemCopiedEventArgs<any>): void;
/**
Called after a port has been copied.
Raises the {@link GraphCopier#addPortCopiedListener PortCopied} event.
@param original The original of the copied port.
@param copy The copied port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23onPortCopied(IPort%2CIPort) Online Documentation}*/
onPortCopied(original: IPort, copy: IPort): void;
/**
Gets or sets the types of items that should be {@link ICloneable#clone cloned} for the copy operation.
By default everything but tags are cloned.
<b>Warning:</b> Only items which implement {@link ICloneable} can be cloned.
@default CloneTypes.ALL & ~CloneTypes.TAGS
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23clone Online Documentation}*/
clone: CloneTypes;
/**
Gets or sets the types of objects where the referential identity should be kept across clipboard operations.
For each of the types that are set via this property, this implementation will use a caching copy mechanism as follows:
If there has already been an instance copied via this clipboard ({@link GraphCopier#getCopy}), that instance will
instead be returned, otherwise, this implementation will clone the original instance (depending on the setting of {@link GraphCopier#clone}) and
{@link GraphCopier#cacheCopy cache the copy} until after the operation the {@link GraphCopier#clearCopyCache cache is cleared}. This feature is implemented directly by the various {@link GraphCopier#copyNodeStyle CopyXXX}
methods.
Let's say nodes <code>n1</code> and <code>n2</code> share the same style instance <code>s1</code>. Then its copies share the same style instance <code>s2</code> which is,
however, another reference than the original <code>s1</code>.
By default this is enabled for all types.
<b>Warning:</b> Only types which implement {@link ICloneable} can be cloned.
@see GraphCopier#getOrCreateCopy
@default CloneTypes.ALL
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23referentialIdentityTypes Online Documentation}*/
referentialIdentityTypes: CloneTypes;
/**
The sentinel return value for use in {@link GraphCopier#getCopy}.
@returns This value is used as a place-holder to indicate that no copy has been stored previously.
@protected
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23NO_COPY Online Documentation}*/
static NO_COPY: any;
/**
Gets or sets a property that determines whether grouping information should be copied to target graphs.
With this property set to <code>false</code> the {@link GraphCopier} can be used to flatten a grouped graph, i.e. remove the
grouping information.
The default is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyGrouping Online Documentation}*/
copyGrouping: boolean;
/**
Gets or sets a property that determines whether {@link FoldingManager additional folding states}.
Should be copied to target graphs.
If the source and target graphs have an instance of {@link FoldingManager} in their lookup, the additional {@link FoldingManager#getFolderNodeState folding state}
of the nodes and edges will be copied. The default is <code>true</code>.
<b>Warning:</b> If elements from a {@link IFoldingView folding-enabled} graph should be copied both source graph and target graph must be a
{@link FoldingManager#masterGraph}. Furthermore, folding must be enabled on both graphs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23copyFoldingStates Online Documentation}*/
copyFoldingStates: boolean;
/**
Adds the given listener for the <code>NodeCopied</code> event that occurs after a node has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
@param listener The listener to add.
@see GraphCopier#removeNodeCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23NodeCopied Online Documentation}*/
addNodeCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>NodeCopied</code> event that occurs after a node has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
@param listener The listener to remove.
@see GraphCopier#addNodeCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23NodeCopied Online Documentation}*/
removeNodeCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>BendCopied</code> event that occurs after a bend has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
For {@link FoldingManager#getFoldingEdgeState folding edge} bends, their view state representation is passed. These elements are neither part of the {@link FoldingManager#masterGraph} nor the
{@link FoldingManager#views view graph}.
@param listener The listener to add.
@see GraphCopier#removeBendCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23BendCopied Online Documentation}*/
addBendCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<IBend>) => void): void;
/**
Removes the given listener for the <code>BendCopied</code> event that occurs after a bend has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
For {@link FoldingManager#getFoldingEdgeState folding edge} bends, their view state representation is passed. These elements are neither part of the {@link FoldingManager#masterGraph} nor the
{@link FoldingManager#views view graph}.
@param listener The listener to remove.
@see GraphCopier#addBendCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23BendCopied Online Documentation}*/
removeBendCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<IBend>) => void): void;
/**
Adds the given listener for the <code>ObjectCopied</code> event that occurs after a style has been copied.
This event is triggered for all minor objects the graph items are made up of, e.g. the {@link GraphCopier#copyNodeStyle styles}, {@link GraphCopier#copyTag tags}, {@link GraphCopier#copyLabelLayoutParameter labels}, and {@link GraphCopier#copyPortLocationParameter port}
parameters.
This event will <b>not</b> be triggered for the graph items themselves and will not be triggered if the object reference is
reused, i.e. it will only be triggered if the old and new references actually differ.
@param listener The listener to add.
@see GraphCopier#removeObjectCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23ObjectCopied Online Documentation}*/
addObjectCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<any>) => void): void;
/**
Removes the given listener for the <code>ObjectCopied</code> event that occurs after a style has been copied.
This event is triggered for all minor objects the graph items are made up of, e.g. the {@link GraphCopier#copyNodeStyle styles}, {@link GraphCopier#copyTag tags}, {@link GraphCopier#copyLabelLayoutParameter labels}, and {@link GraphCopier#copyPortLocationParameter port}
parameters.
This event will <b>not</b> be triggered for the graph items themselves and will not be triggered if the object reference is
reused, i.e. it will only be triggered if the old and new references actually differ.
@param listener The listener to remove.
@see GraphCopier#addObjectCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23ObjectCopied Online Documentation}*/
removeObjectCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<any>) => void): void;
/**
Adds the given listener for the <code>EdgeCopied</code> event that occurs after an edge has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
@param listener The listener to add.
@see GraphCopier#removeEdgeCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23EdgeCopied Online Documentation}*/
addEdgeCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<IEdge>) => void): void;
/**
Removes the given listener for the <code>EdgeCopied</code> event that occurs after an edge has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
@param listener The listener to remove.
@see GraphCopier#addEdgeCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23EdgeCopied Online Documentation}*/
removeEdgeCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<IEdge>) => void): void;
/**
Adds the given listener for the <code>PortCopied</code> event that occurs after a port has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
@param listener The listener to add.
@see GraphCopier#removePortCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23PortCopied Online Documentation}*/
addPortCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<IPort>) => void): void;
/**
Removes the given listener for the <code>PortCopied</code> event that occurs after a port has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
@param listener The listener to remove.
@see GraphCopier#addPortCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23PortCopied Online Documentation}*/
removePortCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<IPort>) => void): void;
/**
Adds the given listener for the <code>LabelCopied</code> event that occurs after a label has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
For {@link FoldingManager#getFoldingEdgeState folding edge} or dummy node labels, their view state representation is passed. These elements are neither part of the
{@link FoldingManager#masterGraph} nor the {@link FoldingManager#views view graph}.
@param listener The listener to add.
@see GraphCopier#removeLabelCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23LabelCopied Online Documentation}*/
addLabelCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<ILabel>) => void): void;
/**
Removes the given listener for the <code>LabelCopied</code> event that occurs after a label has been copied.
This event is dispatched after the element copied callback has been executed.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original original} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy copied} item is a master item, i.e. it belongs to the {@link FoldingManager#masterGraph}.
For {@link FoldingManager#getFoldingEdgeState folding edge} or dummy node labels, their view state representation is passed. These elements are neither part of the
{@link FoldingManager#masterGraph} nor the {@link FoldingManager#views view graph}.
@param listener The listener to remove.
@see GraphCopier#addLabelCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23LabelCopied Online Documentation}*/
removeLabelCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<ILabel>) => void): void;
/**
Adds the given listener for the <code>GraphCopied</code> event that occurs after the entire graph has been copied.
This event is dispatched at the end of the Copy(IGraph,Predicate{IModelItem},IGraph,INode,PointD,ElementCopiedCallback)
method.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original source graph} item is the {@link FoldingManager#masterGraph master graph} of the folding view.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy target graph} is the {@link FoldingManager#masterGraph master graph} of the folding view.
@param listener The listener to add.
@see GraphCopier#removeGraphCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23GraphCopied Online Documentation}*/
addGraphCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<IGraph>) => void): void;
/**
Removes the given listener for the <code>GraphCopied</code> event that occurs after the entire graph has been copied.
This event is dispatched at the end of the Copy(IGraph,Predicate{IModelItem},IGraph,INode,PointD,ElementCopiedCallback)
method.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
graph the {@link ItemCopiedEventArgs#original source graph} item is the {@link FoldingManager#masterGraph master graph} of the folding view.
If this GraphCopier instance is used as {@link GraphClipboard}'s {@link GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
graph the {@link ItemCopiedEventArgs#copy target graph} is the {@link FoldingManager#masterGraph master graph} of the folding view.
@param listener The listener to remove.
@see GraphCopier#addGraphCopiedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCopier%23GraphCopied Online Documentation}*/
removeGraphCopiedListener(listener: (sender: any, evt: ItemCopiedEventArgs<IGraph>) => void): void;
static $class: Class;
private $_GraphCopier: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphCopier;
}
export interface ItemCopiedEventArgs<T> extends EventArgs {
}
/**
Event arguments for the events in {@link GraphCopier}.
This event is used for the {@link GraphCopier#addNodeCopiedListener NodeCopied},
{@link GraphCopier#addEdgeCopiedListener EdgeCopied},
{@link GraphCopier#addPortCopiedListener PortCopied},
{@link GraphCopier#addGraphCopiedListener GraphCopied},
{@link GraphCopier#addLabelCopiedListener LabelCopied}, and {@link GraphCopier#addObjectCopiedListener ObjectCopied}
events. This event holds references to the {@link ItemCopiedEventArgs#original original} of the copied item and the {@link ItemCopiedEventArgs#copy copied item} itself.
Type parameter <code>T</code>: The logical base type of the copied item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCopiedEventArgs Online Documentation}*/
export class ItemCopiedEventArgs<T> {
/**
Creates a new argument for the given item and its original.
@param original The {@link ItemCopiedEventArgs#original original} of the copied item.
@param copy The {@link ItemCopiedEventArgs#copy copied item}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCopiedEventArgs Online Documentation}*/
constructor(original: T | null, copy: T | null);
/**
Gets the original of the {@link ItemCopiedEventArgs#copy copied item}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCopiedEventArgs%23original Online Documentation}*/
readonly original: T | null;
/**
Gets the copied item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCopiedEventArgs%23copy Online Documentation}*/
readonly copy: T | null;
static $class: Class;
private $_ItemCopiedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ItemCopiedEventArgs<T>;
}
export interface GraphDecorator extends YObject {
}
/**
Facilitates decorating {@link IModelItem}s from an {@link IGraph} instance.
This class servers as a factory for lookup decorators for the various items that make up an {@link IGraph}. It
obtains the {@link ILookupDecorator} from the {@link IGraph}'s {@link ILookup#lookup} and passes that to
the various specialized decorator classes {@link NodeDecorator}, {@link EdgeDecorator},
{@link LabelDecorator}, {@link PortDecorator}, and {@link BendDecorator}.
@see GraphDecorator#nodeDecorator
@see NodeDecorator
@see GraphDecorator#edgeDecorator
@see EdgeDecorator
@see GraphDecorator#labelDecorator
@see LabelDecorator
@see GraphDecorator#portDecorator
@see PortDecorator
@see GraphDecorator#bendDecorator
@see BendDecorator
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphDecorator Online Documentation}*/
export class GraphDecorator {
/**
Initializes a new instance of the {@link GraphDecorator} class.
@param graph The graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphDecorator Online Documentation}*/
constructor(graph: IGraph);
/**
* Gets a {@link GraphDecorator#nodeDecorator} instance that can be used to decorate the {@link INode} instances in the graph that this decorator has been
* created for.
* The node can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the node's {@link ILookup#lookup} to return
* the same instance for each node.
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   new ConstantSizeConstraintProvider()
* )
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the node's {@link ILookup#lookup} only to return the same instance
* for each node for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   node => node.tag !== null,
*   new ConstantSizeConstraintProvider()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given node or
* for nodes for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setFactory(
*   node => new CustomSizeConstraintProvider(node)
* )
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setFactory(
*   node => node.tag !== null,
*   node => new CustomSizeConstraintProvider(node)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all nodes and once only for nodes for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementationWrapper(
*   (node, originalProvider) => new SizeConstraintProviderWrapper(node, originalProvider)
* )
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementationWrapper(
*   node => node.tag !== null,
*   (node, originalProvider) => new SizeConstraintProviderWrapper(node, originalProvider)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all nodes and
* for nodes for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.hideImplementation()
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.hideImplementation(
*   node => node.tag !== null
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphDecorator%23nodeDecorator Online Documentation}
*/
readonly nodeDecorator: NodeDecorator;
/**
* Gets an {@link GraphDecorator#edgeDecorator} instance that can be used to decorate the {@link IEdge} instances in the graph that this decorator has
* been created for.
* The edge can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the edge's {@link ILookup#lookup} to return
* the same instance for each edge.
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.setImplementation(new ConstantSelection())
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the edge's {@link ILookup#lookup} only to return the same instance
* for each edge for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.setImplementation(
*   edge => edge.tag !== null,
*   new ConstantSelection()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given edge or
* for edges for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.setFactory(edge => new CustomSelection(edge))
* graph.decorator.edgeDecorator.selectionDecorator.setFactory(
*   edge => edge.tag !== null,
*   edge => new CustomSelection(edge)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all edges and once only for edges for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.setImplementationWrapper(
*   (edge, original) => new SelectionWrapper(edge, original)
* )
* graph.decorator.edgeDecorator.selectionDecorator.setImplementationWrapper(
*   edge => edge.tag !== null,
*   (edge, original) => new SelectionWrapper(edge, original)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all edges and
* for edges for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.hideImplementation()
* graph.decorator.edgeDecorator.selectionDecorator.hideImplementation(edge => edge.tag !== null)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphDecorator%23edgeDecorator Online Documentation}
*/
readonly edgeDecorator: EdgeDecorator;
/**
* Gets a {@link GraphDecorator#labelDecorator} instance that can be used to decorate the {@link ILabel} instances in the graph that this decorator has
* been created for.
* The label can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the label's {@link ILookup#lookup} to
* return the same instance for each label.
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.setImplementation(
*   new ConstantEditLabelHelper()
* )
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the label's {@link ILookup#lookup} only to return the same instance
* for each label for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.setImplementation(
*   label => label.tag !== null,
*   new ConstantEditLabelHelper()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given label
* or for labels for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.setFactory(
*   label => new CustomEditLabelHelper(label)
* )
* graph.decorator.labelDecorator.editLabelHelperDecorator.setFactory(
*   label => label.tag !== null,
*   label => new CustomEditLabelHelper(label)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all labels and once only for labels for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.setImplementationWrapper(
*   (label, original) => new EditLabelHelperWrapper(label, original)
* )
* graph.decorator.labelDecorator.editLabelHelperDecorator.setImplementationWrapper(
*   label => label.tag !== null,
*   (label, original) => new EditLabelHelperWrapper(label, original)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all labels and
* for labels for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.hideImplementation()
* graph.decorator.labelDecorator.editLabelHelperDecorator.hideImplementation(
*   label => label.tag !== null
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphDecorator%23labelDecorator Online Documentation}
*/
readonly labelDecorator: LabelDecorator;
/**
* Gets a {@link GraphDecorator#portDecorator} instance that can be used to decorate the {@link IPort} instances in the graph that this decorator has been
* created for.
* The port can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the port's {@link ILookup#lookup} to return
* the same instance for each port.
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.setImplementation(
*   new ConstantHandleProvider()
* )
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the port's {@link ILookup#lookup} only to return the same instance
* for each port for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.setImplementation(
*   port => port.tag !== null,
*   new ConstantHandleProvider()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given port or
* for ports for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.setFactory(
*   port => new CustomHandleProvider(port)
* )
* graph.decorator.portDecorator.handleProviderDecorator.setFactory(
*   port => port.tag !== null,
*   port => new CustomHandleProvider(port)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all ports and once only for ports for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.setImplementationWrapper(
*   (port, original) => new HandleProviderWrapper(port, original)
* )
* graph.decorator.portDecorator.handleProviderDecorator.setImplementationWrapper(
*   port => port.tag !== null,
*   (port, original) => new HandleProviderWrapper(port, original)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all ports and
* for ports for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.hideImplementation()
* graph.decorator.portDecorator.handleProviderDecorator.hideImplementation(
*   port => port.tag !== null
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphDecorator%23portDecorator Online Documentation}
*/
readonly portDecorator: PortDecorator;
/**
* Gets a {@link GraphDecorator#bendDecorator} instance that can be used to decorate the {@link IBend} instances in the graph that this decorator has been
* created for.
* The bend can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the bend's {@link ILookup#lookup} to return
* the same instance for each bend.
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.setImplementation(new ConstantHandle())
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the bend's {@link ILookup#lookup} only to return the same instance
* for each bend for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.setImplementation(
*   bend => bend.tag !== null,
*   new ConstantHandle()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given bend or
* for bends for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.setFactory(bend => new CustomHandle(bend))
* graph.decorator.bendDecorator.handleDecorator.setFactory(
*   bend => bend.tag !== null,
*   bend => new CustomHandle(bend)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all bends and once only for bends for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.setImplementationWrapper(
*   (bend, original) => new HandleWrapper(bend, original)
* )
* graph.decorator.bendDecorator.handleDecorator.setImplementationWrapper(
*   bend => bend.tag !== null,
*   (bend, original) => new HandleWrapper(bend, original)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all bends and
* for bends for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.hideImplementation()
* graph.decorator.bendDecorator.handleDecorator.hideImplementation(bend => bend.tag !== null)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphDecorator%23bendDecorator Online Documentation}
*/
readonly bendDecorator: BendDecorator;
static $class: Class;
private $_GraphDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphDecorator;
}
export interface LabelDecorator extends YObject {
}
/**
* Helps in {@link ILookupDecorator#addLookup decorating} the {@link ILookup#lookup} method of {@link ILabel} instances by providing access to
* decorators for the most common types.
* Predefined {@link LookupDecorator} are made available through properties like e.g.
* {@link LabelDecorator#highlightDecorator}. Note that the list of possible decorations that are available using this class is not exhaustive in any way. This is
* merely a way to provide access to those interfaces that are more commonly being decorated by the developer.
*
* Typically, a configured instance of this class is available through {@link GraphDecorator#labelDecorator} on the {@link GraphDecorator} which is
* returned by {@link IGraph#decorator} on an {@link IGraph}.
*
* This decorator can be used to conveniently {@link ILookupDecorator decorate the lookup} of {@link ILabel}s in an {@link IGraph}. It
* serves as a factory for predefined {@link LookupDecorator} instances that are specifically useful for the {@link ILabel} type.
*
* The label can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the label's {@link ILookup#lookup} to
* return the same instance for each label.
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.setImplementation(
*   new ConstantEditLabelHelper()
* )
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the label's {@link ILookup#lookup} only to return the same instance
* for each label for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.setImplementation(
*   label => label.tag !== null,
*   new ConstantEditLabelHelper()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given label
* or for labels for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.setFactory(
*   label => new CustomEditLabelHelper(label)
* )
* graph.decorator.labelDecorator.editLabelHelperDecorator.setFactory(
*   label => label.tag !== null,
*   label => new CustomEditLabelHelper(label)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all labels and once only for labels for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.setImplementationWrapper(
*   (label, original) => new EditLabelHelperWrapper(label, original)
* )
* graph.decorator.labelDecorator.editLabelHelperDecorator.setImplementationWrapper(
*   label => label.tag !== null,
*   (label, original) => new EditLabelHelperWrapper(label, original)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all labels and
* for labels for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.labelDecorator.editLabelHelperDecorator.hideImplementation()
* graph.decorator.labelDecorator.editLabelHelperDecorator.hideImplementation(
*   label => label.tag !== null
* )
* </pre>
*
* Each of these methods returns a {@link IContextLookupChainLink} which can be used to {@link LabelDecorator#remove}
* the modified behavior again:
*
* <pre>
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.labelDecorator.editLabelHelperDecorator.setFactory(
*     label => new CustomEditLabelHelper(label)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.labelDecorator.remove(this.chainLink)
* }
* </pre>
*
* @see GraphDecorator
* @see LookupDecorator
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator Online Documentation}
*/
export class LabelDecorator {
/**
Initializes a new instance of the {@link LabelDecorator} class.
@param decorator The decorator to use, which can be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator Online Documentation}*/
constructor(decorator: ILookupDecorator | null);
/**
Creates a {@link LookupDecorator} for {@link ILabel}s that can be used to decorate <code>TInterface</code> types of the labels.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(type: Class, decorateNulls?: boolean, nullIsFallback?: boolean): LookupDecorator<ILabel, TInterface>;
/**
Creates a {@link LookupDecorator} for {@link ILabel}s that can be used to decorate <code>TInterface</code> types of the labels.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param options The parameters to pass.
@param options.type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(options: {
type: Class;
decorateNulls?: boolean;
nullIsFallback?: boolean;
}): LookupDecorator<ILabel, TInterface>;
/**
* Removes an {@link IContextLookupChainLink} again that has been added by one of the various decorators for labels.
* <pre>
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.labelDecorator.editLabelHelperDecorator.setFactory(
*     label => new CustomEditLabelHelper(label)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.labelDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param lookup The lookup chain link to remove.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23remove(IContextLookupChainLink) Online Documentation}
*/
remove(lookup: IContextLookupChainLink): void;
/**
Gets a {@link LookupDecorator} that decorates the {@link IPositionHandler} type for the
{@link ILabel}s this instance has been created for.
Custom {@link IPositionHandler} implementations can be used to restrict or enhance the way the user moves
{@link ILabel}s interactively in the editor. This interface is mainly used by the {@link MoveInputMode}. This
is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23positionHandlerDecorator Online Documentation}*/
readonly positionHandlerDecorator: LookupDecorator<ILabel, IPositionHandler>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IClipboardHelper} type for
{@link ILabel}s this instance has been created for.
Custom {@link IClipboardHelper} implementations can be used to customize the way the clipboard operations are
performed by {@link GraphClipboard}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23clipboardHelperDecorator Online Documentation}*/
readonly clipboardHelperDecorator: LookupDecorator<ILabel, IClipboardHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMarqueeTestable} type for
{@link ILabel}s this instance has been created for.
Custom {@link IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
performed on {@link ILabel}s in the editor. This interface is mainly used by the {@link GraphEditorInputMode}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23marqueeTestableDecorator Online Documentation}*/
readonly marqueeTestableDecorator: LookupDecorator<ILabel, IMarqueeTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILassoTestable} type for
{@link ILabel}s this instance has been created for.
Custom {@link ILassoTestable} implementations can be used to change the way tests for lasso inclusions are
performed on {@link ILabel}s in the editor. This interface is mainly used by the {@link GraphEditorInputMode}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23lassoTestableDecorator Online Documentation}*/
readonly lassoTestableDecorator: LookupDecorator<ILabel, ILassoTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMementoSupport} type for
{@link ILabel}s this instance has been created for.
Custom {@link IMementoSupport} implementations can be used to make the {@link UndoEngine} aware of changes to
data that is associated with the {@link ILabel}s. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23mementoSupportDecorator Online Documentation}*/
readonly mementoSupportDecorator: LookupDecorator<ILabel, IMementoSupport>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ISelectionIndicatorInstaller} type for
{@link ILabel}s this instance has been created for.
Custom {@link ISelectionIndicatorInstaller} implementations can be used to change the way the selection of an {@link ILabel}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link SelectionIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23selectionDecorator Online Documentation}*/
readonly selectionDecorator: LookupDecorator<ILabel, ISelectionIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHighlightIndicatorInstaller} type for
{@link ILabel}s this instance has been created for.
Custom {@link IHighlightIndicatorInstaller} implementations can be used to change the way the highlighting of an {@link ILabel}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link HighlightIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23highlightDecorator Online Documentation}*/
readonly highlightDecorator: LookupDecorator<ILabel, IHighlightIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IFocusIndicatorInstaller} type for
{@link ILabel}s this instance has been created for.
Custom {@link IFocusIndicatorInstaller} implementations can be used change the way the focus of an {@link ILabel}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link FocusIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23focusIndicatorDecorator Online Documentation}*/
readonly focusIndicatorDecorator: LookupDecorator<ILabel, IFocusIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHandleProvider} type for
{@link ILabel}s this instance has been created for.
Custom {@link IHandleProvider} implementations provide interactive draggable handles for the user to change the
geometry or other aspects of {@link ILabel}s in the {@link GraphComponent}. This interface is mainly used by
the {@link HandleInputMode} in the {@link GraphEditorInputMode}. This is a convenient alternative to the
direct usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s
{@link ILookup#lookup}.
When the decorated {@link IHandleProvider} provides a set of handles that may change over time, a call to {@link GraphEditorInputMode#requeryHandles}
may be necessary to refresh the displayed handles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23handleProviderDecorator Online Documentation}*/
readonly handleProviderDecorator: LookupDecorator<ILabel, IHandleProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IObstacleProvider} type for
{@link ILabel}s this instance has been created for.
Custom {@link IObstacleProvider} implementations provide information about obstacles that will be considered by {@link GraphObstacleProvider}
which itself serves as a provider for geometric obstacles to {@link BridgeManager} that manages the rendering of
bridges in edge paths. This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is
provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see GraphObstacleProvider
@see BridgeManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23obstacleProviderDecorator Online Documentation}*/
readonly obstacleProviderDecorator: LookupDecorator<ILabel, IObstacleProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILabelModelParameterFinder} type for
{@link ILabel}s this instance has been created for.
Custom {@link ILabelModelParameterFinder} implementations can be used to change the way {@link ILabel}s can be
dragged interactively by the user. It helps in determining the right {@link ILabelModelParameter} for a given
location that has been chosen by the user. This interface is mainly used by the {@link LabelPositionHandler} in the
{@link GraphComponent}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23labelModelParameterFinderDecorator Online Documentation}*/
readonly labelModelParameterFinderDecorator: LookupDecorator<ILabel, ILabelModelParameterFinder>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IEditLabelHelper} type for
{@link ILabel}s this instance has been created for.
Custom {@link IEditLabelHelper} implementations can be used to change or customize interactive label editing. This
interface is mainly used by the {@link GraphEditorInputMode} which uses the callbacks from the implementation to
determine whether the label can be edited, as well as to customize the {@link TextEditorInputMode} appearance for
the upcoming edit. This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is
provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23editLabelHelperDecorator Online Documentation}*/
readonly editLabelHelperDecorator: LookupDecorator<ILabel, IEditLabelHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILabelModelParameterProvider} type for
{@link ILabel}s this instance has been created for.
Custom {@link ILabelModelParameterProvider} implementations can be used to change the set of possible
{@link ILabelModelParameter}s that the user can assign to a label interactively by dragging it in a
{@link GraphComponent}. This interface is mainly used by the {@link LabelPositionHandler} and {@link MoveLabelInputMode}
in the {@link GraphComponent}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23labelModelParameterProviderDecorator Online Documentation}*/
readonly labelModelParameterProviderDecorator: LookupDecorator<ILabel, ILabelModelParameterProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILabelSnapContextHelper} type for
{@link ILabel}s this instance has been created for.
Custom {@link ILabelSnapContextHelper} implementations can be used to provide custom {@link SnapLine snap lines} and {@link SnapResult snap results}
based on the {@link ILabel}s this instance has been created for. This interface is mainly used by the label's {@link IPositionHandler}
to collect snap results during a drag gesture for the label. This is a convenient alternative to the direct usage of the
{@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDecorator%23labelSnapContextHelperDecorator Online Documentation}*/
readonly labelSnapContextHelperDecorator: LookupDecorator<ILabel, ILabelSnapContextHelper>;
static $class: Class;
private $_LabelDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelDecorator;
}
export interface EdgeDecorator extends YObject {
}
/**
* Helps in {@link ILookupDecorator#addLookup decorating} the {@link ILookup#lookup} method of {@link IEdge} instances by providing access to
* decorators for the most common types.
* Predefined {@link LookupDecorator} are made available through properties like e.g.
* {@link EdgeDecorator#highlightDecorator}. Note that the list of possible decorations that are available using this class is not exhaustive in any way. This is
* merely a way to provide access to those interfaces that are more commonly being decorated by the developer.
*
* Typically, a configured instance of this class is available through {@link GraphDecorator#edgeDecorator} on the {@link GraphDecorator} which is
* returned by {@link IGraph#decorator} on an {@link IGraph}.
*
* This decorator can be used to conveniently {@link ILookupDecorator decorate the lookup} of {@link IEdge}s in an {@link IGraph}. It
* serves as a factory for predefined {@link LookupDecorator} instances that are specifically useful for the {@link IEdge} type.
*
* The edge can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the edge's {@link ILookup#lookup} to return
* the same instance for each edge.
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.setImplementation(new ConstantSelection())
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the edge's {@link ILookup#lookup} only to return the same instance
* for each edge for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.setImplementation(
*   edge => edge.tag !== null,
*   new ConstantSelection()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given edge or
* for edges for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.setFactory(edge => new CustomSelection(edge))
* graph.decorator.edgeDecorator.selectionDecorator.setFactory(
*   edge => edge.tag !== null,
*   edge => new CustomSelection(edge)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all edges and once only for edges for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.setImplementationWrapper(
*   (edge, original) => new SelectionWrapper(edge, original)
* )
* graph.decorator.edgeDecorator.selectionDecorator.setImplementationWrapper(
*   edge => edge.tag !== null,
*   (edge, original) => new SelectionWrapper(edge, original)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all edges and
* for edges for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.edgeDecorator.selectionDecorator.hideImplementation()
* graph.decorator.edgeDecorator.selectionDecorator.hideImplementation(edge => edge.tag !== null)
* </pre>
*
* Each of these methods returns a {@link IContextLookupChainLink} which can be used to {@link EdgeDecorator#remove}
* the modified behavior again:
*
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.edgeDecorator.selectionDecorator.setFactory(
*     edge => new CustomSelection(edge)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.edgeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @see GraphDecorator
* @see LookupDecorator
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator Online Documentation}
*/
export class EdgeDecorator {
/**
Initializes a new instance of the {@link EdgeDecorator} class.
@param decorator The decorator to use, which can be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator Online Documentation}*/
constructor(decorator: ILookupDecorator | null);
/**
Creates a {@link LookupDecorator} for {@link IEdge}s that can be used to decorate <code>TInterface</code> types of the edges.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(type: Class, decorateNulls?: boolean, nullIsFallback?: boolean): LookupDecorator<IEdge, TInterface>;
/**
Creates a {@link LookupDecorator} for {@link IEdge}s that can be used to decorate <code>TInterface</code> types of the edges.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param options The parameters to pass.
@param options.type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(options: {
type: Class;
decorateNulls?: boolean;
nullIsFallback?: boolean;
}): LookupDecorator<IEdge, TInterface>;
/**
* Removes an {@link IContextLookupChainLink} again that has been added by one of the various decorators for edges.
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.edgeDecorator.selectionDecorator.setFactory(
*     edge => new CustomSelection(edge)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.edgeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param lookup The lookup chain link to remove.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23remove(IContextLookupChainLink) Online Documentation}
*/
remove(lookup: IContextLookupChainLink): void;
/**
Gets a {@link LookupDecorator} that decorates the {@link IEdgeReconnectionPortCandidateProvider} type for
{@link IEdge}s this instance has been created for.
Custom {@link IEdgeReconnectionPortCandidateProvider} implementations can be used to change the set of possible
{@link IPortCandidate}s that are available for an existing edge in the {@link IGraph} if the user
interactively tries to {@link IGraph#setEdgePorts change the ports} of an {@link IEdge} in a {@link GraphComponent}. This interface is
mainly used by the {@link PortRelocationHandle}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23edgeReconnectionPortCandidateProviderDecorator Online Documentation}*/
readonly edgeReconnectionPortCandidateProviderDecorator: LookupDecorator<IEdge, IEdgeReconnectionPortCandidateProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IPortCandidateProvider} type for
{@link IEdge}s this instance has been created for.
Custom {@link IPortCandidateProvider} implementations can be used to present the user with a set of
{@link IPortCandidate}s that newly created edges can use as their {@link IEdge#sourcePort source} and {@link IEdge#targetPort target} {@link IPort}s. This
interface is mainly used by the {@link CreateEdgeInputMode} mode of {@link GraphEditorInputMode} if {@link CreateEdgeInputMode#allowEdgeToEdgeConnections} is
enabled. This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by
the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23portCandidateProviderDecorator Online Documentation}*/
readonly portCandidateProviderDecorator: LookupDecorator<IEdge, IPortCandidateProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IEdgePortHandleProvider} type for
{@link IEdge}s this instance has been created for.
Custom {@link IEdgePortHandleProvider} implementations can be used to provide interactive draggable handles for the
source and target end points of an edge. This interface is mainly used by the {@link HandleInputMode} in the {@link GraphEditorInputMode}
and by the implementations that work with {@link OrthogonalEdgeEditingContext}. Implementations may decide, e.g.
whether to yield {@link IHandle} implementations that either move the edge to another port(-{@link IPortCandidate candidate}) using {@link IGraph#setEdgePorts}
or whether to move the port the edge is currently connected to itself, instead. This is a convenient alternative to the
direct usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s
{@link ILookup#lookup}.
When the decorated {@link IEdgePortHandleProvider} provides a set of handles that may change over time, a call to {@link GraphEditorInputMode#requeryHandles}
may be necessary to refresh the displayed handles.
@see PortRelocationHandle
@see IHandle
@see IHandleProvider
@see OrthogonalEdgeEditingContext
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23edgePortHandleProviderDecorator Online Documentation}*/
readonly edgePortHandleProviderDecorator: LookupDecorator<IEdge, IEdgePortHandleProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IOrthogonalEdgeHelper} type for
{@link IEdge}s this instance has been created for.
Custom {@link IOrthogonalEdgeHelper} implementations can be used to change the orthogonal edge editing behavior in
the context of {@link OrthogonalEdgeEditingContext}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23orthogonalEdgeHelperDecorator Online Documentation}*/
readonly orthogonalEdgeHelperDecorator: LookupDecorator<IEdge, IOrthogonalEdgeHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IEdgeSnapResultProvider} type for
{@link IEdge}s this instance has been created for.
Custom {@link IEdgeSnapResultProvider} implementations can be used to change the snapping behavior of edges while
they are being moved in the context of a {@link GraphSnapContext}. This is a convenient alternative to the direct
usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23edgeSnapResultProviderDecorator Online Documentation}*/
readonly edgeSnapResultProviderDecorator: LookupDecorator<IEdge, IEdgeSnapResultProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IEditLabelHelper} type for
{@link IEdge}s this instance has been created for.
Custom {@link IEditLabelHelper} implementations can be used to change or customize interactive label editing and
label creation. This interface is mainly used by the {@link GraphEditorInputMode} which uses the callbacks from the
implementation to determine the label to add or edit, as well as to customize the {@link TextEditorInputMode}
appearance for the upcoming edit. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23editLabelHelperDecorator Online Documentation}*/
readonly editLabelHelperDecorator: LookupDecorator<IEdge, IEditLabelHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IBendCreator} type for {@link IEdge}s this instance has been created for.
Custom {@link IBendCreator} implementations can be used to influence the interactive creation of new
{@link IBend}s. This interface is mainly used by the {@link CreateBendInputMode} in
{@link GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23bendCreatorDecorator Online Documentation}*/
readonly bendCreatorDecorator: LookupDecorator<IEdge, IBendCreator>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IBendSelectionTester} type for
{@link IEdge}s this instance has been created for.
Custom {@link IBendSelectionTester} implementations can be used to influence hit test behavior for bends on edges.
This interface is mainly used by the {@link GraphEditorInputMode} and {@link CreateBendInputMode} to determine
whether an existing bend has been hit. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23bendSelectionTesterDecorator Online Documentation}*/
readonly bendSelectionTesterDecorator: LookupDecorator<IEdge, IBendSelectionTester>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ISnapLineProvider} type for
{@link IEdge}s this instance has been created for.
Custom {@link ISnapLineProvider} implementations can be used to provide custom {@link OrthogonalSnapLine}s
based on the {@link IEdge}s this instance has been created for. This could e.g. be a horizontal {@link OrthogonalSnapLine}
vertically centered between two horizontal segments of one of these edges. This interface is mainly used by the {@link GraphSnapContext}
to collect all available {@link OrthogonalSnapLine}s of orthogonal edge segments. This is a convenient alternative
to the direct usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s
{@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23snapLineProviderDecorator Online Documentation}*/
readonly snapLineProviderDecorator: LookupDecorator<IEdge, ISnapLineProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IPositionHandler} type for
{@link IEdge}s this instance has been created for.
Custom {@link IPositionHandler} implementations can be used to restrict or enhance the way the user moves
{@link IEdge}s interactively in the editor. This interface is mainly used by the {@link MoveInputMode}. This
is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23positionHandlerDecorator Online Documentation}*/
readonly positionHandlerDecorator: LookupDecorator<IEdge, IPositionHandler>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IClipboardHelper} type for
{@link IEdge}s this instance has been created for.
Custom {@link IClipboardHelper} implementations can be used to customize the way the clipboard operations are
performed by {@link GraphClipboard}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23clipboardHelperDecorator Online Documentation}*/
readonly clipboardHelperDecorator: LookupDecorator<IEdge, IClipboardHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMarqueeTestable} type for
{@link IEdge}s this instance has been created for.
Custom {@link IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
performed on {@link IEdge}s in the editor. This interface is mainly used by the {@link GraphEditorInputMode}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23marqueeTestableDecorator Online Documentation}*/
readonly marqueeTestableDecorator: LookupDecorator<IEdge, IMarqueeTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILassoTestable} type for {@link IEdge}s this instance has been created for.
Custom {@link ILassoTestable} implementations can be used to change the way tests for lasso inclusions are
performed on the {@link IEdge}s in the editor. This interface is mainly used by the
{@link GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23lassoTestableDecorator Online Documentation}*/
readonly lassoTestableDecorator: LookupDecorator<IEdge, ILassoTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMementoSupport} type for
{@link IEdge}s this instance has been created for.
Custom {@link IMementoSupport} implementations can be used to make the {@link UndoEngine} aware of changes to
data that is associated with the {@link IEdge}s. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23mementoSupportDecorator Online Documentation}*/
readonly mementoSupportDecorator: LookupDecorator<IEdge, IMementoSupport>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ISelectionIndicatorInstaller} type for
{@link IEdge}s this instance has been created for.
Custom {@link ISelectionIndicatorInstaller} implementations can be used to change the way the selection of an {@link IEdge}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link SelectionIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23selectionDecorator Online Documentation}*/
readonly selectionDecorator: LookupDecorator<IEdge, ISelectionIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHighlightIndicatorInstaller} type for
{@link IEdge}s this instance has been created for.
Custom {@link IHighlightIndicatorInstaller} implementations can be used to change the way the highlighting of an {@link IEdge}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link HighlightIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23highlightDecorator Online Documentation}*/
readonly highlightDecorator: LookupDecorator<IEdge, IHighlightIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IFocusIndicatorInstaller} type for
{@link IEdge}s this instance has been created for.
Custom {@link IFocusIndicatorInstaller} implementations can be used to change the way the focus of an {@link IEdge}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link FocusIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23focusIndicatorDecorator Online Documentation}*/
readonly focusIndicatorDecorator: LookupDecorator<IEdge, IFocusIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHandleProvider} type for
{@link IEdge}s this instance has been created for.
Custom {@link IHandleProvider} implementations provide interactive draggable handles for the user to change the
geometry or other aspects of {@link IEdge}s in the {@link GraphComponent}. This interface is mainly used by the
{@link HandleInputMode} in the {@link GraphEditorInputMode}. This is a convenient alternative to the direct
usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
When the decorated {@link IHandleProvider} provides a set of handles that may change over time, a call to {@link GraphEditorInputMode#requeryHandles}
may be necessary to refresh the displayed handles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23handleProviderDecorator Online Documentation}*/
readonly handleProviderDecorator: LookupDecorator<IEdge, IHandleProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IObstacleProvider} type for
{@link IEdge}s this instance has been created for.
Custom {@link IObstacleProvider} implementations provide information about obstacles that will be considered by {@link GraphObstacleProvider}
which itself serves as a provider for geometric obstacles to {@link BridgeManager} that manages the rendering of
bridges in edge paths. This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is
provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see GraphObstacleProvider
@see BridgeManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorator%23obstacleProviderDecorator Online Documentation}*/
readonly obstacleProviderDecorator: LookupDecorator<IEdge, IObstacleProvider>;
static $class: Class;
private $_EdgeDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeDecorator;
}
export interface BendDecorator extends YObject {
}
/**
* Helps in {@link ILookupDecorator#addLookup decorating} the {@link ILookup#lookup} method of {@link IBend} instances by providing access to
* decorators for the most common types.
* Predefined {@link LookupDecorator} are made available through properties like e.g.
* {@link BendDecorator#highlightDecorator}. Note that the list of possible decorations that are available using this class is not exhaustive in any way. This is
* merely a way to provide access to those interfaces that are more commonly being decorated by the developer.
*
* Typically, a configured instance of this class is available through {@link GraphDecorator#bendDecorator} on the {@link GraphDecorator} which is
* returned by {@link IGraph#decorator} on an {@link IGraph}.
*
* This decorator can be used to conveniently {@link ILookupDecorator decorate the lookup} of {@link IBend}s in an {@link IGraph}. It
* serves as a factory for predefined {@link LookupDecorator} instances that are specifically useful for the {@link IBend} type.
*
* The bend can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the bend's {@link ILookup#lookup} to return
* the same instance for each bend.
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.setImplementation(new ConstantHandle())
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the bend's {@link ILookup#lookup} only to return the same instance
* for each bend for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.setImplementation(
*   bend => bend.tag !== null,
*   new ConstantHandle()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given bend or
* for bends for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.setFactory(bend => new CustomHandle(bend))
* graph.decorator.bendDecorator.handleDecorator.setFactory(
*   bend => bend.tag !== null,
*   bend => new CustomHandle(bend)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all bends and once only for bends for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.setImplementationWrapper(
*   (bend, original) => new HandleWrapper(bend, original)
* )
* graph.decorator.bendDecorator.handleDecorator.setImplementationWrapper(
*   bend => bend.tag !== null,
*   (bend, original) => new HandleWrapper(bend, original)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all bends and
* for bends for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.bendDecorator.handleDecorator.hideImplementation()
* graph.decorator.bendDecorator.handleDecorator.hideImplementation(bend => bend.tag !== null)
* </pre>
*
* Each of these methods returns a {@link IContextLookupChainLink} which can be used to {@link BendDecorator#remove}
* the modified behavior again:
*
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.bendDecorator.handleDecorator.setFactory(
*     bend => new CustomHandle(bend)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.bendDecorator.remove(this.chainLink)
* }
* </pre>
*
* @see GraphDecorator
* @see LookupDecorator
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator Online Documentation}
*/
export class BendDecorator {
/**
Initializes a new instance of the {@link BendDecorator} class.
@param decorator The decorator to use, which can be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator Online Documentation}*/
constructor(decorator: ILookupDecorator | null);
/**
Creates a {@link LookupDecorator} for {@link IBend}s that can be used to decorate <code>TInterface</code> types of the bends.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(type: Class, decorateNulls?: boolean, nullIsFallback?: boolean): LookupDecorator<IBend, TInterface>;
/**
Creates a {@link LookupDecorator} for {@link IBend}s that can be used to decorate <code>TInterface</code> types of the bends.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param options The parameters to pass.
@param options.type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(options: {
type: Class;
decorateNulls?: boolean;
nullIsFallback?: boolean;
}): LookupDecorator<IBend, TInterface>;
/**
* Removes an {@link IContextLookupChainLink} again that has been added by one of the various decorators for
* {@link IBend}s.
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.bendDecorator.handleDecorator.setFactory(
*     bend => new CustomHandle(bend)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.bendDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param lookup The lookup chain link to remove.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23remove(IContextLookupChainLink) Online Documentation}
*/
remove(lookup: IContextLookupChainLink): void;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHandle} type for {@link IBend}s this instance has been created for.
Custom {@link IHandle} implementations can be used to change the way the user can drag a bend interactively to
change its position in a {@link GraphComponent}. This interface is mainly used by the {@link HandleInputMode}
in {@link GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
When the decorated {@link IHandle} is sometimes present and sometimes not, a call to {@link GraphEditorInputMode#requeryHandles}
may be necessary to refresh the displayed handles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23handleDecorator Online Documentation}*/
readonly handleDecorator: LookupDecorator<IBend, IHandle>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IBendSnapResultProvider} type for
{@link IBend}s this instance has been created for.
Custom {@link IBendSnapResultProvider} implementations can be used to provide custom {@link SnapResult}s based
on the {@link IBend}s this instance has been created for. This could e.g. be a {@link SnapResult} describing
that the bend wants to snap vertically to the top border of a node. This interface is mainly used by the default {@link IHandle}
for {@link IBend}s to collect {@link SnapResult}s during a drag gesture for the bend. This is a convenient
alternative to the direct usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s
{@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23bendSnapResultProviderDecorator Online Documentation}*/
readonly bendSnapResultProviderDecorator: LookupDecorator<IBend, IBendSnapResultProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IPositionHandler} type for
{@link IBend}s this instance has been created for.
Custom {@link IPositionHandler} implementations can be used to restrict or enhance the way the user moves
{@link IBend}s interactively in the editor. This interface is mainly used by the {@link MoveInputMode}. This
is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23positionHandlerDecorator Online Documentation}*/
readonly positionHandlerDecorator: LookupDecorator<IBend, IPositionHandler>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IClipboardHelper} type for
{@link IBend}s this instance has been created for.
Custom {@link IClipboardHelper} implementations can be used to customize the way the clipboard operations are
performed by {@link GraphClipboard}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23clipboardHelperDecorator Online Documentation}*/
readonly clipboardHelperDecorator: LookupDecorator<IBend, IClipboardHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMarqueeTestable} type for
{@link IBend}s this instance has been created for.
Custom {@link IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
performed on {@link IBend}s in the editor. This interface is mainly used by the {@link GraphEditorInputMode}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23marqueeTestableDecorator Online Documentation}*/
readonly marqueeTestableDecorator: LookupDecorator<IBend, IMarqueeTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILassoTestable} type for {@link IBend}s this instance has been created for.
Custom {@link ILassoTestable} implementations can be used to change the way tests for lasso inclusions are
performed on {@link IBend}s in the editor. This interface is mainly used by the {@link GraphEditorInputMode}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23lassoTestableDecorator Online Documentation}*/
readonly lassoTestableDecorator: LookupDecorator<IBend, ILassoTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMementoSupport} type for
{@link IBend}s this instance has been created for.
Custom {@link IMementoSupport} implementations can be used to make the {@link UndoEngine} aware of changes to
data that is associated with the {@link IBend}s. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23mementoSupportDecorator Online Documentation}*/
readonly mementoSupportDecorator: LookupDecorator<IBend, IMementoSupport>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ISelectionIndicatorInstaller} type for
{@link IBend}s this instance has been created for.
Custom {@link ISelectionIndicatorInstaller} implementations can be used to change the way the selection of an {@link IBend}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link SelectionIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23selectionDecorator Online Documentation}*/
readonly selectionDecorator: LookupDecorator<IBend, ISelectionIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHighlightIndicatorInstaller} type for
{@link IBend}s this instance has been created for.
Custom {@link IHighlightIndicatorInstaller} implementations can be used to change the way the highlighting of an {@link IBend}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link HighlightIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23highlightDecorator Online Documentation}*/
readonly highlightDecorator: LookupDecorator<IBend, IHighlightIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IFocusIndicatorInstaller} type for
{@link IBend}s this instance has been created for.
Custom {@link IFocusIndicatorInstaller} implementations can be used to change the way the focus of an {@link IBend}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link FocusIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23focusIndicatorDecorator Online Documentation}*/
readonly focusIndicatorDecorator: LookupDecorator<IBend, IFocusIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHandleProvider} type for
{@link IBend}s this instance has been created for.
Custom {@link IHandleProvider} implementations provide interactive draggable handles for the user to change the
geometry or other aspects of {@link IBend}s in the {@link GraphComponent}. This interface is mainly used by the
{@link HandleInputMode} in the {@link GraphEditorInputMode}. This is a convenient alternative to the direct
usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
When the decorated {@link IHandleProvider} provides a set of handles that may change over time, a call to {@link GraphEditorInputMode#requeryHandles}
may be necessary to refresh the displayed handles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23handleProviderDecorator Online Documentation}*/
readonly handleProviderDecorator: LookupDecorator<IBend, IHandleProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IObstacleProvider} type for
{@link IBend}s this instance has been created for.
Custom {@link IObstacleProvider} implementations provide information about obstacles that will be considered by {@link GraphObstacleProvider}
which itself serves as a provider for geometric obstacles to {@link BridgeManager} that manages the rendering of
bridges in edge paths. This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is
provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see GraphObstacleProvider
@see BridgeManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendDecorator%23obstacleProviderDecorator Online Documentation}*/
readonly obstacleProviderDecorator: LookupDecorator<IBend, IObstacleProvider>;
static $class: Class;
private $_BendDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BendDecorator;
}
export interface PortDecorator extends YObject {
}
/**
* Helps in {@link ILookupDecorator#addLookup decorating} the {@link ILookup#lookup} method of {@link IPort} instances by providing access to
* decorators for the most common types.
* Predefined {@link LookupDecorator} are made available through properties like e.g.
* {@link PortDecorator#highlightDecorator}. Note that the list of possible decorations that are available using this class is not exhaustive in any way. This is
* merely a way to provide access to those interfaces that are more commonly being decorated by the developer.
*
* Typically, a configured instance of this class is available through {@link GraphDecorator#portDecorator} on the {@link GraphDecorator} which is
* returned by {@link IGraph#decorator} on an {@link IGraph}.
*
* This decorator can be used to conveniently {@link ILookupDecorator decorate the lookup} of {@link IPort}s in an {@link IGraph}. It
* serves as a factory for predefined {@link LookupDecorator} instances that are specifically useful for the {@link IPort} type.
*
* The port can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the port's {@link ILookup#lookup} to return
* the same instance for each port.
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.setImplementation(
*   new ConstantHandleProvider()
* )
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the port's {@link ILookup#lookup} only to return the same instance
* for each port for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.setImplementation(
*   port => port.tag !== null,
*   new ConstantHandleProvider()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given port or
* for ports for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.setFactory(
*   port => new CustomHandleProvider(port)
* )
* graph.decorator.portDecorator.handleProviderDecorator.setFactory(
*   port => port.tag !== null,
*   port => new CustomHandleProvider(port)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all ports and once only for ports for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.setImplementationWrapper(
*   (port, original) => new HandleProviderWrapper(port, original)
* )
* graph.decorator.portDecorator.handleProviderDecorator.setImplementationWrapper(
*   port => port.tag !== null,
*   (port, original) => new HandleProviderWrapper(port, original)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all ports and
* for ports for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.portDecorator.handleProviderDecorator.hideImplementation()
* graph.decorator.portDecorator.handleProviderDecorator.hideImplementation(
*   port => port.tag !== null
* )
* </pre>
*
* Each of these methods returns a {@link IContextLookupChainLink} which can be used to {@link PortDecorator#remove}
* the modified behavior again:
*
* <pre>
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.portDecorator.handleProviderDecorator.setFactory(
*     port => new CustomHandleProvider(port)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.portDecorator.remove(this.chainLink)
* }
* </pre>
*
* @see GraphDecorator
* @see LookupDecorator
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator Online Documentation}
*/
export class PortDecorator {
/**
Initializes a new instance of the {@link PortDecorator} class.
@param decorator The decorator to use, which can be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator Online Documentation}*/
constructor(decorator: ILookupDecorator | null);
/**
Creates a {@link LookupDecorator} for {@link IPort}s that can be used to decorate <code>TInterface</code> types of the ports.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(type: Class, decorateNulls?: boolean, nullIsFallback?: boolean): LookupDecorator<IPort, TInterface>;
/**
Creates a {@link LookupDecorator} for {@link IPort}s that can be used to decorate <code>TInterface</code> types of the ports.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param options The parameters to pass.
@param options.type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(options: {
type: Class;
decorateNulls?: boolean;
nullIsFallback?: boolean;
}): LookupDecorator<IPort, TInterface>;
/**
* Removes an {@link IContextLookupChainLink} again that has been added by one of the various decorators for ports.
* <pre>
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.portDecorator.handleProviderDecorator.setFactory(
*     port => new CustomHandleProvider(port)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.portDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param lookup The lookup chain link to remove.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23remove(IContextLookupChainLink) Online Documentation}
*/
remove(lookup: IContextLookupChainLink): void;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHandle} type for {@link IPort}s this instance has been created for.
Custom {@link IHandle} implementations can be used to change the way the user can drag a {@link IPort}
interactively to change its position in a {@link GraphComponent}. This interface is mainly used by the {@link HandleInputMode}
in {@link GraphEditorInputMode} which obtains the {@link IHandle} from the {@link IPort} by querying the
{@link IHandleProvider} from it, which in turn will query the {@link IHandle} from itself. Alternatively,
selected nodes can put the {@link IHandle} of their ports into their {@link IHandleProvider} implementation.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
When the decorated {@link IHandle} is sometimes present, and sometimes not, a call to {@link GraphEditorInputMode#requeryHandles}
may be necessary to refresh the displayed handles.
<h2>Note</h2>
By default, port handles are provided by an {@link IEdgePortHandleProvider}. In that case changes to this property
don't have an effect. Instead, a custom {@link IEdgePortHandleProvider} implementation has to be set to the {@link GraphDecorator#edgeDecorator}'s {@link EdgeDecorator#edgePortHandleProviderDecorator}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23handleDecorator Online Documentation}*/
readonly handleDecorator: LookupDecorator<IPort, IHandle>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IEdgePathCropper} type for
{@link IPort}s this instance has been created for.
Custom {@link IEdgePathCropper} implementations can be used to change the way the path of {@link PathBasedEdgeStyleRenderer} implementations
calculate the {@link GeneralPath} of the visual representation of an {@link IEdge} in a
{@link GraphComponent}. This interface is mainly used by the {@link PathBasedEdgeStyleRenderer} to find the intersection of an edge with the bounds of
the adjacent node to properly crop the edge path. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23edgePathCropperDecorator Online Documentation}*/
readonly edgePathCropperDecorator: LookupDecorator<IPort, IEdgePathCropper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ISnapLineProvider} type for
{@link IPort}s this instance has been created for.
Custom {@link ISnapLineProvider} implementations can be used to provide custom {@link OrthogonalSnapLine}s
based on the {@link IPort}s this instance has been created for. This could e.g. be a {@link OrthogonalSnapLine}
with a horizontal or vertical offset to the port's location. This interface is mainly used by the {@link GraphSnapContext}
to collect all available {@link OrthogonalSnapLine}s. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23snapLineProviderDecorator Online Documentation}*/
readonly snapLineProviderDecorator: LookupDecorator<IPort, ISnapLineProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IPortSnapResultProvider} type for
{@link IPort}s this instance has been created for.
Custom {@link IPortSnapResultProvider} implementations can be used to provide custom {@link SnapResult}s based
on the {@link IPort}s this instance has been created for. This could e.g. be a {@link SnapResult} describing
that the port wants to snap to a grid point. This interface is mainly used by the {@link PortLocationModelParameterHandle}
to collect {@link SnapResult}s during a drag gesture for the port. This is a convenient alternative to the direct
usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23portSnapResultProviderDecorator Online Documentation}*/
readonly portSnapResultProviderDecorator: LookupDecorator<IPort, IPortSnapResultProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IPositionHandler} type for
{@link IPort}s this instance has been created for.
Custom {@link IPositionHandler} implementations can be used to restrict or enhance the way the user moves
{@link IPort}s interactively in the editor. This interface is mainly used by the {@link MoveInputMode}. This
is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23positionHandlerDecorator Online Documentation}*/
readonly positionHandlerDecorator: LookupDecorator<IPort, IPositionHandler>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IClipboardHelper} type for
{@link IPort}s this instance has been created for.
Custom {@link IClipboardHelper} implementations can be used to customize the way the clipboard operations are
performed by {@link GraphClipboard}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23clipboardHelperDecorator Online Documentation}*/
readonly clipboardHelperDecorator: LookupDecorator<IPort, IClipboardHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMarqueeTestable} type for
{@link IPort}s this instance has been created for.
Custom {@link IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
performed on {@link IPort}s in the editor. This interface is mainly used by the {@link GraphEditorInputMode}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23marqueeTestableDecorator Online Documentation}*/
readonly marqueeTestableDecorator: LookupDecorator<IPort, IMarqueeTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILassoTestable} type for {@link IPort}s this instance has been created for.
Custom {@link ILassoTestable} implementations can be used to change the way tests for lasso inclusions are
performed on {@link IPort}s in the editor. This interface is mainly used by the {@link GraphEditorInputMode}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23lassoTestableDecorator Online Documentation}*/
readonly lassoTestableDecorator: LookupDecorator<IPort, ILassoTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMementoSupport} type for
{@link IPort}s this instance has been created for.
Custom {@link IMementoSupport} implementations can be used to make the {@link UndoEngine} aware of changes to
data that is associated with the {@link IPort}s. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23mementoSupportDecorator Online Documentation}*/
readonly mementoSupportDecorator: LookupDecorator<IPort, IMementoSupport>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ISelectionIndicatorInstaller} type for
{@link IPort}s this instance has been created for.
Custom {@link ISelectionIndicatorInstaller} implementations can be used change the way the selection of an {@link IPort}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link SelectionIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23selectionDecorator Online Documentation}*/
readonly selectionDecorator: LookupDecorator<IPort, ISelectionIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHighlightIndicatorInstaller} type for
{@link IPort}s this instance has been created for.
Custom {@link IHighlightIndicatorInstaller} implementations can be used to change the way the highlighting of an {@link IPort}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link HighlightIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23highlightDecorator Online Documentation}*/
readonly highlightDecorator: LookupDecorator<IPort, IHighlightIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IFocusIndicatorInstaller} type for
{@link IPort}s this instance has been created for.
Custom {@link IFocusIndicatorInstaller} implementations can be used to change the way the focus of an {@link IPort}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link FocusIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23focusIndicatorDecorator Online Documentation}*/
readonly focusIndicatorDecorator: LookupDecorator<IPort, IFocusIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHandleProvider} type for
{@link IPort}s this instance has been created for.
Custom {@link IHandleProvider} implementations provide interactive draggable handles for the user to change the
geometry or other aspects of {@link IPort}s in the {@link GraphComponent}. This interface is mainly used by the
{@link HandleInputMode} in the {@link GraphEditorInputMode}. This is a convenient alternative to the direct
usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
When the decorated {@link IHandleProvider} provides a set of handles that may change over time, a call to {@link GraphEditorInputMode#requeryHandles}
may be necessary to refresh the displayed handles.
<h2>Note</h2>
By default, port handles are provided by an {@link IEdgePortHandleProvider}. In that case changes to this property
don't have an effect. Instead, a custom {@link IEdgePortHandleProvider} implementation has to be set to the {@link GraphDecorator#edgeDecorator}'s {@link EdgeDecorator#edgePortHandleProviderDecorator}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23handleProviderDecorator Online Documentation}*/
readonly handleProviderDecorator: LookupDecorator<IPort, IHandleProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IObstacleProvider} type for
{@link IPort}s this instance has been created for.
Custom {@link IObstacleProvider} implementations provide information about obstacles that will be considered by {@link GraphObstacleProvider}
which itself serves as a provider for geometric obstacles to {@link BridgeManager} that manages the rendering of
bridges in edge paths. This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is
provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see GraphObstacleProvider
@see BridgeManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDecorator%23obstacleProviderDecorator Online Documentation}*/
readonly obstacleProviderDecorator: LookupDecorator<IPort, IObstacleProvider>;
static $class: Class;
private $_PortDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortDecorator;
}
export interface NodeDecorator extends YObject {
}
/**
* Helps in {@link ILookupDecorator#addLookup decorating} the {@link ILookup#lookup} method of {@link INode} instances by providing access to
* decorators for the most common types.
* Predefined {@link LookupDecorator} are made available through properties like e.g.
* {@link NodeDecorator#highlightDecorator}. Note that the list of possible decorations that are available using this class is not exhaustive in any way. This is
* merely a way to provide access to those interfaces that are more commonly being decorated by the developer.
*
* Typically, a configured instance of this class is available through {@link GraphDecorator#nodeDecorator} on the {@link GraphDecorator} which is
* returned by {@link IGraph#decorator} on an {@link IGraph}.
*
* This decorator can be used to conveniently {@link ILookupDecorator decorate the lookup} of {@link INode}s in an {@link IGraph}. It
* serves as a factory for predefined {@link LookupDecorator} instances that are specifically useful for the {@link INode} type.
*
* The node can be decorated in various ways. {@link LookupDecorator#setImplementation} decorates the node's {@link ILookup#lookup} to return
* the same instance for each node.
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   new ConstantSizeConstraintProvider()
* )
* </pre>
*
* {@link LookupDecorator#setImplementation} with a predicate decorates the node's {@link ILookup#lookup} only to return the same instance
* for each node for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   node => node.tag !== null,
*   new ConstantSizeConstraintProvider()
* )
* </pre>
*
* {@link LookupDecorator#setFactory} calls the provided factory method to return a new instance of the queried type created for the given node or
* for nodes for which the predicate returns <code>true</code>, respectively:
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setFactory(
*   node => new CustomSizeConstraintProvider(node)
* )
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setFactory(
*   node => node.tag !== null,
*   node => new CustomSizeConstraintProvider(node)
* )
* </pre>
*
* {@link LookupDecorator#setImplementationWrapper} calls the provided factory method to return a new instance of the queried type which wraps the
* default implementation, once for all nodes and once only for nodes for which the predicate returns <code>true</code>.
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementationWrapper(
*   (node, originalProvider) => new SizeConstraintProviderWrapper(node, originalProvider)
* )
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementationWrapper(
*   node => node.tag !== null,
*   (node, originalProvider) => new SizeConstraintProviderWrapper(node, originalProvider)
* )
* </pre>
*
* Finally {@link LookupDecorator#hideImplementation} hides the default implementation by letting the lookup always return <code>null</code> for all nodes and
* for nodes for which the predicate returns <code>true</code>, respectively
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.hideImplementation()
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.hideImplementation(
*   node => node.tag !== null
* )
* </pre>
*
* Each of these methods returns a {@link IContextLookupChainLink} which can be used to {@link NodeDecorator#remove}
* the modified behavior again:
*
* <pre>
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setFactory(
*     node => new CustomSizeConstraintProvider(node)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @see GraphDecorator
* @see LookupDecorator
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator Online Documentation}
*/
export class NodeDecorator {
/**
Initializes a new instance of the {@link NodeDecorator} class.
@param decorator The decorator to use, which can be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator Online Documentation}*/
constructor(decorator: ILookupDecorator | null);
/**
Creates a {@link LookupDecorator} for {@link INode}s that can be used to decorate <code>TInterface</code> types of the nodes.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(type: Class, decorateNulls?: boolean, nullIsFallback?: boolean): LookupDecorator<INode, TInterface>;
/**
Creates a {@link LookupDecorator} for {@link INode}s that can be used to decorate <code>TInterface</code> types of the nodes.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param options The parameters to pass.
@param options.type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(options: {
type: Class;
decorateNulls?: boolean;
nullIsFallback?: boolean;
}): LookupDecorator<INode, TInterface>;
/**
* Removes an {@link IContextLookupChainLink} again that has been added by one of the various decorators for nodes.
* <pre>
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setFactory(
*     node => new CustomSizeConstraintProvider(node)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param lookup The lookup chain link to remove.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23remove(IContextLookupChainLink) Online Documentation}
*/
remove(lookup: IContextLookupChainLink): void;
/**
Gets a {@link LookupDecorator} that decorates the {@link IReshapeHandleProvider} type for the
{@link INode}s this instance has been created for.
Custom {@link IReshapeHandleProvider} implementations can be used to change the number and type of
{@link IHandle}s that can be used by the user to reshape the {@link INode}s in interactively in the editor.
This interface is mainly used by the {@link GraphEditorInputMode} which passes the {@link IHandle}s to the
{@link HandleInputMode}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
When the decorated {@link IReshapeHandleProvider} provides a set of handles that may change over time, a call to {@link GraphEditorInputMode#requeryHandles}
may be necessary to refresh the displayed handles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23reshapeHandleProviderDecorator Online Documentation}*/
readonly reshapeHandleProviderDecorator: LookupDecorator<INode, IReshapeHandleProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IReshapeHandler} type for the
{@link INode}s this instance has been created for.
Custom {@link IReshapeHandler} implementations can be used to alter how nodes are resized interactively. This
interface is mainly used by {@link IReshapeHandleProvider} and corresponding {@link IHandle} implementations
as the low-level interface to delegate the actual size changes to. This is a convenient alternative to the direct usage
of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23reshapeHandlerDecorator Online Documentation}*/
readonly reshapeHandlerDecorator: LookupDecorator<INode, IReshapeHandler>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IEditLabelHelper} type for the
{@link INode}s this instance has been created for.
Custom {@link IEditLabelHelper} implementations can be used to change or customize interactive label editing and
label creation. This interface is mainly used by the {@link GraphEditorInputMode} which uses the callbacks from the
implementation to determine the label to add or edit, as well as to customize the {@link TextEditorInputMode}
appearance for the upcoming edit. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23editLabelHelperDecorator Online Documentation}*/
readonly editLabelHelperDecorator: LookupDecorator<INode, IEditLabelHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IShapeGeometry} type for the
{@link INode}s this instance has been created for.
Custom {@link IShapeGeometry} implementations can be used to change the way different code parts interpret the {@link INode#layout} of
an {@link INode}. This interface is mainly used by the {@link DefaultEdgePathCropper}, and a custom {@link IPortCandidateProvider}
that can be obtained from {@link IPortCandidateProvider#fromShapeGeometry}. Note that decorating the {@link IShapeGeometry}
for an {@link INode} will not change the result of a call to {@link INodeStyleRenderer#getShapeGeometry} of
the {@link INode#style}. So standard hit tests will not be influenced by such a change. This is a convenient alternative to the direct
usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23shapeGeometryDecorator Online Documentation}*/
readonly shapeGeometryDecorator: LookupDecorator<INode, IShapeGeometry>;
/**
Gets a {@link LookupDecorator} that decorates the {@link INodeSizeConstraintProvider} type for the
{@link INode}s this instance has been created for.
Custom {@link INodeSizeConstraintProvider} implementations can be used to add constraints to the size of a node.
These constraints are used by the default {@link IReshapeHandleProvider} implementation for nodes to enforce size
constraints for interactive resize gestures. Also {@link GraphEditorInputMode} uses this interface to constrain the
size of a node whenever its labels are being edited interactively. This is a convenient alternative to the direct usage
of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23sizeConstraintProviderDecorator Online Documentation}*/
readonly sizeConstraintProviderDecorator: LookupDecorator<INode, INodeSizeConstraintProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IGroupBoundsCalculator} type for the
{@link INode}s this instance has been created for.
Custom {@link IGroupBoundsCalculator} implementations can be used to change the way the bounds of {@link IGraph#createGroupNode group nodes} are
being calculated if the geometry of child nodes is changed by the user. This is a convenient alternative to the direct
usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23groupBoundsCalculatorDecorator Online Documentation}*/
readonly groupBoundsCalculatorDecorator: LookupDecorator<INode, IGroupBoundsCalculator>;
/**
Gets a {@link LookupDecorator} that decorates the {@link INodeInsetsProvider} type for the
{@link INode}s this instance has been created for.
Custom {@link INodeInsetsProvider} implementations can be used to change the way the bounds of {@link IGraph#createGroupNode group nodes} are
being calculated if the geometry of child nodes is changed by the user. This interface will be used if there is no
custom {@link IGroupBoundsCalculator} associated with the nodes (see {@link NodeDecorator#groupBoundsCalculatorDecorator}). This is a convenient alternative to the
direct usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s
{@link ILookup#lookup}. Note that many {@link INodeStyle} implementations provide an <code>Insets</code> property whose
value will be reported by a {@link INodeInsetsProvider} implementation that is provided by the corresponding
{@link INodeStyleRenderer}. Using this property instead of decorating the lookup of the {@link INode} is thus
often simpler.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23insetsProviderDecorator Online Documentation}*/
readonly insetsProviderDecorator: LookupDecorator<INode, INodeInsetsProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IPortCandidateProvider} type for the
{@link INode}s this instance has been created for.
Custom {@link IPortCandidateProvider} implementations can be used to present the user with a set of
{@link IPortCandidate}s that newly created edges can use as their {@link IEdge#sourcePort source} and {@link IEdge#targetPort target} {@link IPort}s. This
interface is mainly used by the {@link CreateEdgeInputMode} mode of {@link GraphEditorInputMode}. Also the
default implementation of {@link IEdgeReconnectionPortCandidateProvider} will use this interface to determine the
candidates that are available if the user interactively moves the ports using {@link PortRelocationHandle} (see {@link EdgeDecorator#edgePortHandleProviderDecorator}).
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23portCandidateProviderDecorator Online Documentation}*/
readonly portCandidateProviderDecorator: LookupDecorator<INode, IPortCandidateProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ISnapLineProvider} type for the
{@link INode}s this instance has been created for.
Custom {@link ISnapLineProvider} implementations can be used to provide custom {@link OrthogonalSnapLine}s
based on the {@link INode}s this instance has been created for. This could e.g. be a horizontal {@link OrthogonalSnapLine}
at the vertical center of the node to which other nodes should snap with their top border. This interface is mainly used
by the {@link GraphSnapContext} to collect all available {@link OrthogonalSnapLine}s. This is a convenient
alternative to the direct usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s
{@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23snapLineProviderDecorator Online Documentation}*/
readonly snapLineProviderDecorator: LookupDecorator<INode, ISnapLineProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link INodeSnapResultProvider} type for the
{@link INode}s this instance has been created for.
Custom {@link INodeSnapResultProvider} implementations can be used to provide custom {@link SnapResult}s based
on the {@link INode}s this instance has been created for. This could e.g. be a {@link SnapResult} describing
that the top left corner of the node wants to snap to a grid point. This interface is mainly used by the default {@link IPositionHandler}
for {@link INode}s to collect {@link SnapResult}s during a drag gesture for the node. This is a convenient
alternative to the direct usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s
{@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23nodeSnapResultProviderDecorator Online Documentation}*/
readonly nodeSnapResultProviderDecorator: LookupDecorator<INode, INodeSnapResultProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link INodeReshapeSnapResultProvider} type for the
{@link INode}s this instance has been created for.
Custom {@link INodeReshapeSnapResultProvider} implementations can be used to provide custom
{@link SnapResult}s based on the {@link INode}s this instance has been created for. This could e.g. be a {@link SnapResult}
describing that the top left corner of the node wants to snap to a grid point. This interface is mainly used by the
default {@link IReshapeHandler} for {@link INode}s to collect {@link SnapResult}s during a resize gesture
for the node. This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided
by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23nodeReshapeSnapResultProviderDecorator Online Documentation}*/
readonly nodeReshapeSnapResultProviderDecorator: LookupDecorator<INode, INodeReshapeSnapResultProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IPositionHandler} type for the
{@link INode}s this instance has been created for.
Custom {@link IPositionHandler} implementations can be used to restrict or enhance the way the user moves
{@link INode}s interactively in the editor. This interface is mainly used by the {@link MoveInputMode}. This
is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23positionHandlerDecorator Online Documentation}*/
readonly positionHandlerDecorator: LookupDecorator<INode, IPositionHandler>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IClipboardHelper} type for
{@link INode}s this instance has been created for.
Custom {@link IClipboardHelper} implementations can be used to customize the way the clipboard operations are
performed by {@link GraphClipboard}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23clipboardHelperDecorator Online Documentation}*/
readonly clipboardHelperDecorator: LookupDecorator<INode, IClipboardHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMarqueeTestable} type for
{@link INode}s this instance has been created for.
Custom {@link IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
performed on {@link INode}s in the editor. This interface is mainly used by the {@link GraphEditorInputMode}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23marqueeTestableDecorator Online Documentation}*/
readonly marqueeTestableDecorator: LookupDecorator<INode, IMarqueeTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILassoTestable} type for {@link INode}s this instance has been created for.
Custom {@link ILassoTestable} implementations can be used to change the way tests for lasso inclusions are
performed on {@link INode}s in the editor. This interface is mainly used by the {@link GraphEditorInputMode}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23lassoTestableDecorator Online Documentation}*/
readonly lassoTestableDecorator: LookupDecorator<INode, ILassoTestable>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IMementoSupport} type for
{@link INode}s this instance has been created for.
Custom {@link IMementoSupport} implementations can be used to make the {@link UndoEngine} aware of changes to
data that is associated with the {@link INode}s. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23mementoSupportDecorator Online Documentation}*/
readonly mementoSupportDecorator: LookupDecorator<INode, IMementoSupport>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ISelectionIndicatorInstaller} type for
{@link INode}s this instance has been created for.
Custom {@link ISelectionIndicatorInstaller} implementations can be used change the way the selection of an {@link INode}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link SelectionIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23selectionDecorator Online Documentation}*/
readonly selectionDecorator: LookupDecorator<INode, ISelectionIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHighlightIndicatorInstaller} type for
{@link INode}s this instance has been created for.
Custom {@link IHighlightIndicatorInstaller} implementations can be used change the way the highlighting of an {@link INode}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link HighlightIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23highlightDecorator Online Documentation}*/
readonly highlightDecorator: LookupDecorator<INode, IHighlightIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IFocusIndicatorInstaller} type for
{@link INode}s this instance has been created for.
Custom {@link IFocusIndicatorInstaller} implementations can be used change the way the focus of an {@link INode}
is represented in a {@link GraphComponent}. This interface is mainly used by the {@link FocusIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link IGraph}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23focusIndicatorDecorator Online Documentation}*/
readonly focusIndicatorDecorator: LookupDecorator<INode, IFocusIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHandleProvider} type for
{@link INode}s this instance has been created for.
Custom {@link IHandleProvider} implementations provide interactive draggable handles for the user to change the
geometry or other aspects of {@link INode}s in the {@link GraphComponent}. This interface is mainly used by the
{@link HandleInputMode} in the {@link GraphEditorInputMode}. This is a convenient alternative to the direct
usage of the {@link ILookupDecorator} that is provided by the {@link IGraph}'s {@link ILookup#lookup}.
When the decorated {@link IHandleProvider} provides a set of handles that may change over time, a call to {@link GraphEditorInputMode#requeryHandles}
may be necessary to refresh the displayed handles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23handleProviderDecorator Online Documentation}*/
readonly handleProviderDecorator: LookupDecorator<INode, IHandleProvider>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IObstacleProvider} type for
{@link INode}s this instance has been created for.
Custom {@link IObstacleProvider} implementations provide information about obstacles that will be considered by {@link GraphObstacleProvider}
which itself serves as a provider for geometric obstacles to {@link BridgeManager} that manages the rendering of
bridges in edge paths. This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is
provided by the {@link IGraph}'s {@link ILookup#lookup}.
@see GraphObstacleProvider
@see BridgeManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDecorator%23obstacleProviderDecorator Online Documentation}*/
readonly obstacleProviderDecorator: LookupDecorator<INode, IObstacleProvider>;
static $class: Class;
private $_NodeDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeDecorator;
}
export interface LookupDecorator<TDecoratedType, TInterface> extends YObject {
}
/**
* Simplifies using an {@link ILookupDecorator}.
* This class provides methods that help in performing common tasks related to decorating the {@link ILookup#lookup}
* method for a certain <code>TDecoratedType</code>:
*
* <ul>
* <li>Set a single implementation</li>
* <li>Set a factory that creates item-specific implementations</li>
* <li>Wrap the default implementation</li>
* <li>Hide the default implementation</li>
* </ul>
* Each of these methods adds a new {@link IContextLookupChainLink} to the lookup chain which adds the wanted
* behavior. The link can be removed again by either calling the {@link ILookupDecorator#removeLookup} method on the {@link LookupDecorator#decorator}
* or by calling the {@link NodeDecorator#remove} method on the parent decorator, e.g. the
* {@link NodeDecorator}.
*
* Typically, one does not create instances of this class but uses the item specific implementations provided by
* {@link GraphDecorator}: {@link GraphDecorator#nodeDecorator}, {@link GraphDecorator#edgeDecorator}, {@link GraphDecorator#labelDecorator}, {@link GraphDecorator#portDecorator}, and {@link GraphDecorator#bendDecorator}.
*
* <br><b>Getting the decorator for a node's size constraint provider</b>
* <pre>
* const decorator = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator
* </pre>
*
* <br><b>Setting a single implementation</b>
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   new ConstantSizeConstraintProvider()
* )
* </pre>
*
* <br><b>Setting a factory that creates item-specific implementations</b>
* <pre>
* graph.decorator.nodeDecorator.selectionDecorator.setFactory(
*   node => new RedSelectionInstaller(node)
* )
* </pre>
*
* <br><b>Wrapping the default implementation</b>
* <pre>
* graph.decorator.nodeDecorator.positionHandlerDecorator.setImplementationWrapper(
*   (node, original) => new ConstrainedPositionHandlerWrapper(original, node)
* )
* </pre>
*
* <br><b>Hiding the default implementation</b>
* <pre>
* graph.decorator.nodeDecorator.positionHandlerDecorator.hideImplementation()
* </pre>
*
* Type parameter <code>TDecoratedType</code>: The type of the instances whose {@link ILookup#lookup} behavior should be modified.
* Type parameter <code>TInterface</code>: The interface or type that clients will use during the {@link ILookup#lookup} call. Note that this is normally an
* interface and not the implementation of the interface since client code will use the interface for the query.
* @see ILookup
* @see GraphDecorator
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator Online Documentation}
*/
export class LookupDecorator<TDecoratedType, TInterface> {
/**
Initializes a new instance of the {@link LookupDecorator} class.
@param decoratedType The type of the instances whose {@link ILookup#lookup} behavior should be modified.
@param interfaceType The interface or type that clients will use during the {@link ILookup#lookup} call. Note that this is normally an
interface and not the implementation of the interface since client code will use the interface for the query.
@param decorator The decorator instance that will be used to {@link ILookupDecorator#addLookup extend the Lookup mechanism}.
@param decorateNulls The initial value of the {@link LookupDecorator#decorateNulls} property.
@param nullIsFallback The initial value of the {@link LookupDecorator#nullIsFallback} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator Online Documentation}*/
constructor(decoratedType: Class, interfaceType: Class, decorator: ILookupDecorator | null, decorateNulls: boolean, nullIsFallback: boolean);
/**
Simple helper method that tries to add the provided chain link to the {@link LookupDecorator#decorator}.
@param lookup The link to add.
@returns The <code>lookup</code> that has been {@link ILookupDecorator#addLookup added} to the {@link LookupDecorator#decorator} as a result of this call, or <code>null</code> if the decoration was unsuccessful.
@see ILookupDecorator#addLookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23addChainLink(IContextLookupChainLink) Online Documentation}*/
addChainLink(lookup: IContextLookupChainLink | null): IContextLookupChainLink | null;
/**
* Sets the {@link ILookup#lookup} of all <code>TDecoratedType</code> items matching a <code>predicate</code> to return <code>null</code> if <code>TInterface</code> is queried during the
* {@link ILookup#lookup}.
* This can be used to temporarily or permanently disable a certain behavior for all items. To do this only for items
* matching a predicate, use {@link LookupDecorator#hideImplementation} instead.
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method.
*
* This method can be used to disable the default behavior by hiding the default implementation returned by the lookup. The
* following example prevents all nodes from being moved by hiding their position handler:
*
* <pre>
* graph.decorator.nodeDecorator.positionHandlerDecorator.hideImplementation()
* </pre>
*
* The following example prevents only nodes with a tag from being moved:
*
* <pre>
* graph.decorator.nodeDecorator.positionHandlerDecorator.hideImplementation(
*   node => node.tag !== null
* )
* </pre>
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method:
*
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.hideImplementation()
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param [predicate=null] The predicate that determines for which items the implementation shall be hidden.
* @returns An {@link IContextLookupChainLink} that has been {@link ILookupDecorator#addLookup added} to the {@link LookupDecorator#decorator} as a result of this call, or <code>null</code> if decoration was
* unsuccessful.
* @see ILookupDecorator#addLookup
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23hideImplementation(function(TDecoratedType)%3Aboolean) Online Documentation}
*/
hideImplementation(predicate?: ((obj: TDecoratedType) => boolean) | null): IContextLookupChainLink | null;
/**
* Sets the {@link ILookup#lookup} of items matching a <code>predicate</code> with a <code>factory</code> that creates a <code>TInterface</code> instance if <code>TInterface</code> is
* queried during the {@link ILookup#lookup}.
* This can be used to modify the lookup of all items matching a certain predicate where the returned instance either
* depends on the item or uses it in some way.
*
* To adjust the lookup of all items in this manner, use {@link LookupDecorator#setFactory} instead. To adjust the lookup of a single item, use
* {@link LookupDecorator#setImplementation}.
*
* Note that depending on the value of the {@link LookupDecorator#nullIsFallback} property a <code>null</code> return value from the <code>factory</code> will be either returned directly,
* or interpreted as a hint to return a fallback value which is obtained by querying the remainder of the lookup chain.
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method.
*
* The following example shows how to decorate the lookup to create different instances for different types of nodes when
* queried. The example creates a red selection decorator for nodes with tag "type 1", a blue one for nodes with tag "type
* 2", and the default for all other nodes.
*
* <pre>
* graph.decorator.nodeDecorator.selectionDecorator.setFactory(
*   node => node.tag === 'type 1',
*   node => new RedSelectionInstaller(node)
* )
* graph.decorator.nodeDecorator.selectionDecorator.setFactory(
*   node => node.tag === 'type 2',
*   node => new BlueSelectionInstaller(node)
* )
* </pre>
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method:
*
* <pre>
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setFactory(
*     node => new CustomSizeConstraintProvider(node)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param predicate The predicate that determines for which items the factory shall be called.
* @param factory The factory that is queried to create the result.
* @returns An {@link IContextLookupChainLink} that has been {@link ILookupDecorator#addLookup added} to the {@link LookupDecorator#decorator} as a result of this call, or <code>null</code> if decoration was
* unsuccessful.
* @see ILookupDecorator#addLookup
* @see LookupDecorator#nullIsFallback
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23setFactory(function(TDecoratedType)%3Aboolean%2Cfunction(TDecoratedType)%3ATInterface) Online Documentation}
*/
setFactory(predicate: (obj: TDecoratedType) => boolean, factory: (arg: TDecoratedType) => TInterface | null): IContextLookupChainLink | null;
/**
* Sets the {@link ILookup#lookup} of all <code>TDecoratedType</code> items, so that <code>factory</code> is used to create a <code>TInterface</code> instance if <code>TInterface</code> is queried during
* the {@link ILookup#lookup}.
* This can be used to modify the lookup of all items where the returned instance either depends on the item or uses it in
* some way.
*
* To only adjust the lookup of items matching a predicate in this manner, use {@link LookupDecorator#setFactory} instead. To adjust the lookup of
* a single item, use {@link LookupDecorator#setImplementation}.
*
* Note that depending on the value of the {@link LookupDecorator#nullIsFallback} property a <code>null</code> return value from the <code>factory</code> will be either returned directly,
* or interpreted as a hint to return a fallback value which is obtained by querying the remainder of the lookup chain.
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method.
*
* This method can be used for implementations which depend on the item they are queried from
*
* <pre>
* graph.decorator.nodeDecorator.selectionDecorator.setFactory(
*   node => new RedSelectionInstaller(node)
* )
* </pre>
*
* The following example shows how to decorate the lookup to create different instances for different types of nodes when
* queried. The example creates a red selection decorator for nodes with tag "type 1", a blue one for nodes with tag "type
* 2", and the default for all other nodes.
*
* <pre>
* graph.decorator.nodeDecorator.selectionDecorator.setFactory(node => {
*   switch (node.tag) {
*     case 'type 1':
*       return new RedSelectionInstaller(node)
*     case 'type 2':
*       return new BlueSelectionInstaller(node)
*     default:
*       return null
*   }
* })
* </pre>
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method:
*
* <pre>
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setFactory(
*     node => new CustomSizeConstraintProvider(node)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param factory The factory that is queried to create the result.
* @returns An {@link IContextLookupChainLink} that has been {@link ILookupDecorator#addLookup added} to the {@link LookupDecorator#decorator} as a result of this call, or <code>null</code> if decoration was
* unsuccessful.
* @see ILookupDecorator#addLookup
* @see LookupDecorator#nullIsFallback
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23setFactory(function(TDecoratedType)%3ATInterface) Online Documentation}
*/
setFactory(factory: (arg: TDecoratedType) => TInterface | null): IContextLookupChainLink | null;
/**
* Sets the {@link ILookup#lookup} of all items of type <code>TDecoratedType</code> which match a <code>predicate</code> to return the given <code>TInterface</code> <code>implementation</code>
* if <code>TInterface</code> is queried on an item's {@link ILookup#lookup}.
* This can be used to modify the lookup of all items matching a certain predicate in the same way.
*
* To adjust the lookup of only a single item, use {@link LookupDecorator#setImplementation} instead. To adjust the lookup of all items, use
* {@link LookupDecorator#setImplementation}.
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method.
*
* The following example shows how to decorate the lookup to return different values for different types of node (nodes
* with different tags in this example). Note that the decorations can be chained.
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   node => node.tag === 'type 1',
*   new NodeSizeConstraintProvider(new Size(10, 10), new Size(100, 100))
* )
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   node => node.tag === 'type 2',
*   new NodeSizeConstraintProvider(new Size(20, 20), new Size(200, 200))
* )
* </pre>
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method:
*
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*     new ConstantSizeConstraintProvider()
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param predicate A predicate that determines which items are affected.
* @param implementation The implementation to return if the <code>predicate</code> matches and <code>TInterface</code> is queried.
* @returns An {@link IContextLookupChainLink} that has been {@link ILookupDecorator#addLookup added} to the {@link LookupDecorator#decorator} as a result of this call, or <code>null</code> if decoration was
* unsuccessful.
* @see ILookupDecorator#addLookup
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23setImplementation(function(TDecoratedType)%3Aboolean%2CTInterface) Online Documentation}
*/
setImplementation(predicate: (obj: TDecoratedType) => boolean, implementation: TInterface | null): IContextLookupChainLink | null;
/**
* Sets the {@link ILookup#lookup} of the given <code>item</code> to return the given <code>TInterface</code> <code>implementation</code> if <code>TInterface</code> is queried on the <code>item</code>'s
* {@link ILookup#lookup}.
* This can be used to modify the lookup of a single item only.
*
* To adjust the lookup of all items matching a predicate, use {@link LookupDecorator#setImplementation} instead. To adjust the lookup of all
* items, use {@link LookupDecorator#setImplementation}.
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove}.
*
* The following example shows how to decorate the lookup to return different values for different nodes. Note that the
* decorations can be chained.
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   node1,
*   new NodeSizeConstraintProvider(new Size(10, 10), new Size(100, 100))
* )
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   node2,
*   new NodeSizeConstraintProvider(new Size(20, 20), new Size(200, 200))
* )
* </pre>
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method:
*
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*     new ConstantSizeConstraintProvider()
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param item The item for which the implementation shall be returned.
* @param implementation The implementation to return if <code>item</code> is queried for <code>TInterface</code>.
* @returns An {@link IContextLookupChainLink} that has been {@link ILookupDecorator#addLookup added} to the {@link LookupDecorator#decorator} as a result of this call, or <code>null</code> if decoration was
* unsuccessful.
* @see ILookupDecorator#addLookup
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23setImplementation(TDecoratedType%2CTInterface) Online Documentation}
*/
setImplementation(item: TDecoratedType, implementation: TInterface | null): IContextLookupChainLink | null;
/**
* Sets the {@link ILookup#lookup} of all items of type <code>TDecoratedType</code> to return the given <code>TInterface</code> <code>implementation</code> if <code>TInterface</code> is queried on an
* item's {@link ILookup#lookup}.
* This can be used to modify the lookup of all items in the same way.
*
* To adjust the lookup of only a single item, use {@link LookupDecorator#setImplementation} instead. To adjust the lookup of all items matching a
* certain predicate, use {@link LookupDecorator#setImplementation}.
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method.
*
* The following example shows how to decorate the lookup to return a constant value for all nodes:
*
* <pre>
* graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*   new ConstantSizeConstraintProvider()
* )
* </pre>
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method:
*
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementation(
*     new ConstantSizeConstraintProvider()
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param implementation The implementation to return unconditionally.
* @returns An {@link IContextLookupChainLink} that has been {@link ILookupDecorator#addLookup added} to the {@link LookupDecorator#decorator} as a result of this call, or <code>null</code> if decoration was
* unsuccessful.
* @see ILookupDecorator#addLookup
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23setImplementation(TInterface) Online Documentation}
*/
setImplementation(implementation: TInterface | null): IContextLookupChainLink | null;
/**
* Sets the {@link ILookup#lookup} of items matching a <code>predicate</code> with a <code>factory</code> callback that wraps the underlying
* implementation if <code>TInterface</code> is queried during the {@link ILookup#lookup}.
* This can be used to modify the lookup of all items matching a certain predicate where the factory implementation may
* delegate to the wrapped implementation in certain cases.
*
* To adjust the lookup for all items in this manner, use {@link LookupDecorator#setImplementationWrapper} instead. To adjust the lookup of a
* single item, use {@link LookupDecorator#setImplementation}.
*
* Note that depending on the value of the {@link LookupDecorator#decorateNulls} property, the factory will not be called to decorate <code>null</code> results.
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method.
*
* This method can be used for implementations which wrap the default implementation
*
* <pre>
* graph.decorator.nodeDecorator.positionHandlerDecorator.setImplementationWrapper(
*   node => node.tag !== null,
*   (node, original) => new ConstrainedPositionHandlerWrapper(original, node)
* )
* </pre>
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method:
*
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementationWrapper(
*     (node, provider) => new SizeConstraintProviderWrapper(node, provider)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param predicate The predicate that determines for which items the wrapping shall be performed.
* @param factory The factory that will be passed the underlying implementation and that is queried to create the result.
* @returns An {@link IContextLookupChainLink} that has been {@link ILookupDecorator#addLookup added} to the {@link LookupDecorator#decorator} as a result of this call, or <code>null</code> if decoration was
* unsuccessful.
* @see ILookupDecorator#addLookup
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23setImplementationWrapper(function(TDecoratedType)%3Aboolean%2Cfunction(TDecoratedType%2CTInterface)%3ATInterface) Online Documentation}
*/
setImplementationWrapper(predicate: (obj: TDecoratedType) => boolean, factory: (arg1: TDecoratedType | null, arg2: TInterface | null) => TInterface | null): IContextLookupChainLink | null;
/**
* Sets the {@link ILookup#lookup} of all items with a <code>factory</code> callback that wraps the underlying implementation if <code>TInterface</code>
* is queried during the {@link ILookup#lookup}.
* This can be used to modify the lookup of all items matching where the factory implementation may delegate to the wrapped
* implementation in certain cases.
*
* To adjust the lookup in this manner only for items matching a predicate, use {@link LookupDecorator#setImplementationWrapper} instead. To
* adjust the lookup of a single item, use {@link LookupDecorator#setImplementation}.
*
* Note that depending on the value of the {@link LookupDecorator#decorateNulls} property, the factory will not be called to decorate <code>null</code> results.
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method.
*
* This method can be used for implementations which wrap the default implementation
*
* <pre>
* graph.decorator.nodeDecorator.positionHandlerDecorator.setImplementationWrapper(
*   (node, original) => new ConstrainedPositionHandlerWrapper(original, node)
* )
* </pre>
*
* The effects of this method call can be undone by passing the return value to {@link ILookupDecorator#removeLookup}
* or to the containing decorator's (e.g. {@link NodeDecorator} {@link NodeDecorator#remove} method:
*
* <pre>
* constructor() {
*   this.chainLink = null
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* registerLookup(graph) {
*   this.chainLink = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator.setImplementationWrapper(
*     (node, provider) => new SizeConstraintProviderWrapper(node, provider)
*   )
* }
*
* /**
*  * @param {IGraph} graph
*  *‍/
* deregisterLookup(graph) {
*   graph.decorator.nodeDecorator.remove(this.chainLink)
* }
* </pre>
*
* @param factory The factory that will be passed the underlying implementation and that is queried to create the result.
* @returns An {@link IContextLookupChainLink} that has been {@link ILookupDecorator#addLookup added} to the {@link LookupDecorator#decorator} as a result of this call, or <code>null</code> if decoration was
* unsuccessful.
* @see ILookupDecorator#addLookup
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23setImplementationWrapper(function(TDecoratedType%2CTInterface)%3ATInterface) Online Documentation}
*/
setImplementationWrapper(factory: (arg1: TDecoratedType | null, arg2: TInterface | null) => TInterface | null): IContextLookupChainLink | null;
/**
Gets the {@link ILookupDecorator} instance that is used.
This can be <code>null</code>. In this case all methods of this class do nothing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23decorator Online Documentation}*/
readonly decorator: ILookupDecorator | null;
/**
* Gets or sets a value indicating whether wrapping implementations will wrap <code>null</code> values from the rest of the lookup chain
* or those values will simply be returned instead.
* If this is <code>false</code>, then the {@link LookupDecorator#setImplementationWrapper SetImplementationWrapper} methods will not wrap <code>null</code> values from the rest of the lookup chain
* and the result of the call to {@link ILookup#lookup} will simply be <code>null</code>. If this is <code>true</code>, the wrapper will get <code>null</code>
* as the implementation to wrap and should handle this accordingly.
*
* Note that changing this value does not affect wrappers which are already set. Changing the value will only affect
* wrappers which are set after the change.
*
* <pre>
* /**
*  * @param {INode} node1
*  * @param {INode} node2
*  * @param {IGraph} graph
*  *‍/
* function decorateNulls(node1, node2, graph) {
*   // let's assume for this example, that node1's lookup returns an implementation by default
*   // and node2's lookup returns null
*   let provider1 = node1.lookup(INodeSizeConstraintProvider.$class) // not null
*   let provider2 = node2.lookup(INodeSizeConstraintProvider.$class) // null
*
*   // set the decorator to wrap null values and wrap the default size constraint provider
*   const decorator = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator
*   decorator.decorateNulls = true
*   decorator.setImplementationWrapper(
*     (node, provider) => new SizeConstraintProviderWrapper(node, provider)
*   )
*
*   provider1 = node1.lookup(INodeSizeConstraintProvider.$class) // a SizeConstraintProviderWrapper which wraps the default
*   provider2 = node2.lookup(INodeSizeConstraintProvider.$class) // a SizeConstraintProviderWrapper which wraps null
* }
*
* /**
*  * @param {INode} node1
*  * @param {INode} node2
*  * @param {IGraph} graph
*  *‍/
* function dontDecorateNulls(node1, node2, graph) {
*   let provider1
*   let provider2
*   // let's assume for this example, that node1's lookup returns an implementation by default
*   // and node2's lookup returns null
*   provider1 = node1.lookup(INodeSizeConstraintProvider.$class) // not null
*   provider2 = node2.lookup(INodeSizeConstraintProvider.$class) // null
*
*   // set the decorator not to wrap null values and wrap the default size constraint provider
*   const decorator = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator
*   decorator.decorateNulls = false
*   decorator.setImplementationWrapper(
*     (node, provider) => new SizeConstraintProviderWrapper(node, provider)
*   )
*
*   provider1 = node1.lookup(INodeSizeConstraintProvider.$class) // a SizeConstraintProviderWrapper which wraps the default
*   provider2 = node2.lookup(INodeSizeConstraintProvider.$class) // null
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23decorateNulls Online Documentation}
*/
decorateNulls: boolean;
/**
* Gets or sets a value indicating whether a <code>null</code> result value of the methods {@link LookupDecorator#setImplementation} and {@link LookupDecorator#setFactory} should be
* interpreted as taking the fallback value, or actually return <code>null</code>.
* If this is <code>true</code>, then <code>null</code> values returned from the factory set with {@link LookupDecorator#setFactory} are not returned, but rather
* interpreted as a hint to query the rest of the lookup chain. If this is <code>false</code>, then <code>null</code> values are returned as is.
*
* Changing this value only affects decorations which are set after the change. Existing decorations remain unaffected.
*
* <pre>
* /**
*  * @param {INode} node
*  * @param {IGraph} graph
*  *‍/
* function nullIsFallback(node, graph) {
*   // set to return the fallback if the decoration returns null and set a factory which returns always null
*   const decorator = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator
*   decorator.nullIsFallback = true
*   decorator.setFactory(n => null)
*
*   const provider = node.lookup(INodeSizeConstraintProvider.$class) // the default INodeSizeConstraintProvider implementation
* }
*
* /**
*  * @param {INode} node
*  * @param {IGraph} graph
*  *‍/
* function nullIsNotFallback(node, graph) {
*   // set the decorator to always use the returned value, even if its null and set a factory which returns always null
*   const decorator = graph.decorator.nodeDecorator.sizeConstraintProviderDecorator
*   decorator.nullIsFallback = false
*   decorator.setFactory(n => null)
*
*   const provider = node.lookup(INodeSizeConstraintProvider.$class) // null
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LookupDecorator%23nullIsFallback Online Documentation}
*/
nullIsFallback: boolean;
static $class: Class;
private $_LookupDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDecoratedType, TInterface>(o: any): o is LookupDecorator<TDecoratedType, TInterface>;
}
export interface GroupingSupport extends YObject {
}
/**
Provides less frequently used methods for analyzing and managing grouped graphs.
An instance of this class which is configured for a given {@link IGraph} instance can be obtained via {@link IGraph#groupingSupport}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport Online Documentation}*/
export class GroupingSupport {
/**
Creates a new instance which operates on the given <code>graph</code>.
@param graph The graph to use for operations on this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport Online Documentation}*/
constructor(graph: IGraph);
/**
Calculates the minimum area to enclose by the given group node with respect to its {@link IGroupBoundsCalculator}.
@param groupNode The group node to calculate the minimum enclosed area of.
@returns The area to enclose for the given group node. This is {@link Rect#EMPTY} if the node is not a group node, or if
it hasn't any children.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23calculateMinimumEnclosedArea(INode) Online Documentation}*/
calculateMinimumEnclosedArea(groupNode: INode): Rect;
/**
Enlarges all group nodes in the graph in an interactive scenario, so that the {@link GroupingSupport#calculateMinimumEnclosedArea minimum enclosed area} is respected.
This method should be used in an interactive editing scenario. It records an {@link ICompoundEdit} to support
undoability and uses the {@link IReshapeHandler} for the resizing of the nodes. That way, e.g. orthogonally edited
edges will be properly reshaped.
@param context The context to use for the {@link IReshapeHandler}s.
@see GroupingSupport#enlargeGroupNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23enlargeAllGroupNodes(IInputModeContext) Online Documentation}*/
enlargeAllGroupNodes(context: IInputModeContext): void;
/**
Enlarges all group nodes in the graph so that the {@link GroupingSupport#calculateMinimumEnclosedArea minimum enclosed area} is respected.
@see GroupingSupport#enlargeGroupNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23enlargeAllGroupNodes() Online Documentation}*/
enlargeAllGroupNodes(): void;
/**
Enlarges the group nodes in an interactive scenario, using {@link IReshapeHandler} implementations of the group
nodes to perform the actual resizing.
This method should be used in an interactive editing scenario. It records an {@link ICompoundEdit} to support
undoability and uses the {@link IReshapeHandler} for the resizing of the nodes. That way, e.g. orthogonally edited
edges will be properly reshaped.
@param context The context to use for the {@link IReshapeHandler}s.
@param node The node to enlarge appropriately.
@param ancestors if set to <code>true</code> ancestors the ancestor group nodes will be adjusted, too, if necessary.
@see IReshapeHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23enlargeGroupNode(IInputModeContext%2CINode%2Cboolean) Online Documentation}*/
enlargeGroupNode(context: IInputModeContext, node: INode, ancestors: boolean): void;
/**
Enlarges the given group node to ensure that the {@link GroupingSupport#calculateMinimumEnclosedArea minimum enclosed} area is inside of its bounds.
This method can be used to resize a group node and all of its parent group nodes to fully contain its children, e.g.
after a child node has changed its bounds. Note that this method will only enlarge the bounds of the group nodes, it
will never reduce the size of a group node.
@param groupNode The group node to resize.
@param ancestors if set to <code>true</code> all ancestor group nodes will be resized, too, if necessary
@see GroupingSupport#enlargeGroupNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23enlargeGroupNode(INode%2Cboolean) Online Documentation}*/
enlargeGroupNode(groupNode: INode, ancestors: boolean): void;
/**
* Returns all descendants of the provided node that are part of the grouped graph.
* The enumeration will be top down, i.e. all of a node's ancestors will be enumerated before the respective node.
*
* <pre>
* // GetDescendants is available on the graph's grouping support
* const descendants = graph.groupingSupport.getDescendants(outerGroup)
* // nodeInOuterGroup, innerGroup, group2, node2_1, group1, node1_2, node1_1
* </pre>
*
* @param node The root node to get the descendants from.
* @returns An enumeration of the children of the node at the time of this invocation in reverse DFS prefix order.
* @see IGraph#getChildren
* @see GroupingSupport#getDescendantsBottomUp
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23getDescendants(INode) Online Documentation}
*/
getDescendants(node: INode | null): IEnumerable<INode>;
/**
* Returns all descendants of the provided node that are part of the grouped graph.
* The enumeration will be bottom up, i.e. all of a node's children will be enumerated before the respective node.
*
* <pre>
* const descendantsBottomUp = graph.groupingSupport.getDescendantsBottomUp(outerGroup)
* // node1_1, node1_2, group1, node2_1, group2, innerGroup, nodeInOuterGroup
* </pre>
*
* @param node The root node to get the descendants from.
* @returns An enumeration of the children of the node at the time of this invocation in DFS postfix order.
* @see IGraph#getChildren
* @see GroupingSupport#getDescendants
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23getDescendantsBottomUp(INode) Online Documentation}
*/
getDescendantsBottomUp(node: INode | null): IEnumerable<INode>;
/**
* Determines the nearest common ancestor of the provided nodes in the graph.
* <pre>
* // get the grouping support for the graph
* const groupingSupport = graph.groupingSupport
* // get the nearest common ancestor for node1_1 and node1_2
* groupingSupport.getNearestCommonAncestor(node1_1, node1_2) // group1
* // get the nearest common ancestor for node1_1 and node2_1
* groupingSupport.getNearestCommonAncestor(node1_1, node2_1) // innerGroup
* </pre>
*
* @param nodes The nodes to find the nearest common ancestor of.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23getNearestCommonAncestor(INode) Online Documentation}
*/
getNearestCommonAncestor(...nodes: INode[]): INode | null;
/**
* Determines the nearest common ancestor of the provided nodes in the graph.
* <pre>
* // get the grouping support for the graph
* const groupingSupport = graph.groupingSupport
* // get the nearest common ancestor for node1_1 and node1_2
* groupingSupport.getNearestCommonAncestor(node1_1, node1_2) // group1
* // get the nearest common ancestor for node1_1 and node2_1
* groupingSupport.getNearestCommonAncestor(node1_1, node2_1) // innerGroup
* </pre>
*
* @param nodes The nodes to find the nearest common ancestor of.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23getNearestCommonAncestor(IEnumerable%3CINode%3E) Online Documentation}
*/
getNearestCommonAncestor(nodes: IEnumerable<INode> | IEnumerableConvertible<INode>): INode | null;
/**
* Gets the path to root traversing all {@link IGraph#getParent parents} of the given item.
* <pre>
* const pathToRoot = graph.groupingSupport.getPathToRoot(node1_1)
* // node1_1, group1, innerGroup, outerGroup
* </pre>
*
* @param node The node to start.
* @returns A list that includes the node but not the root, unless they are identical and all parents in between them.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23getPathToRoot(INode) Online Documentation}
*/
getPathToRoot(node: INode | null): IList<INode>;
/**
Determines whether there are group nodes in the graph.
This methods considers all {@link IGraph#isGroupNode group} nodes, that is, nodes that could have children.
@returns <code>true</code> if there are {@link IGraph#isGroupNode group}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23hasGroupNodes() Online Documentation}*/
hasGroupNodes(): boolean;
/**
* Determines whether <code>ancestor</code> is an ancestor of <code>node</code> in the grouped graph.
* <pre>
* const graph = graphComponent.graph
* // create a grouped graph:
* //                       Root
* //                     outerGroup
* //           innerGroup          nodeInOuterGroup
* //      group1        group2
* // node1_1 node1_2    node2_1
* const outerGroup = graph.createGroupNode()
* const innerGroup = graph.createGroupNode(outerGroup)
* const nodeInOuterGroup = graph.createNode(outerGroup)
* const group1 = graph.createGroupNode(innerGroup)
* const group2 = graph.createGroupNode(innerGroup)
* const node1_1 = graph.createNode(group1)
* const node1_2 = graph.createNode(group1)
* const node2_1 = graph.createNode(group2)
*
* // get the graph's grouping support
* const groupingSupport = graph.groupingSupport
*
* // test whether a node is a descendant of a given group
* const node1_1IsDescendantOfGroup1 = groupingSupport.isDescendant(node1_1, group1) // true
* const node1_1IsDescendantOfOuterGroup = groupingSupport.isDescendant(node1_1, outerGroup) // true
* const node1_1IsDescendantOfGroup2 = groupingSupport.isDescendant(node1_1, group2) // false
* </pre>
*
* @param node The node to check.
* @param ancestor The ancestor to check.
* @returns Whether <code>ancestor</code> is an ancestor of <code>node</code>.
* @see GroupingSupport#getDescendants
* @see GroupingSupport#getDescendantsBottomUp
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingSupport%23isDescendant(INode%2CINode) Online Documentation}
*/
isDescendant(node: INode | null, ancestor: INode | null): boolean;
static $class: Class;
private $_GroupingSupport: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GroupingSupport;
}
export interface GraphModelManager extends YObject {
}
/**
Manages the visual appearance of an {@link IGraph} instance in a {@link CanvasComponent}.
This class monitors the structural changes of an {@link IGraph} implementation and installs the necessary {@link ICanvasObject}
instance in the canvas for each item in the graph. Note that {@link GraphComponent} has an instance of this type
built-in already and usually it is not necessary to create an own one.
This class will also {@link GraphModelManager#hierarchicNestingPolicy use the node hierarchy} to nest the nodes and edges accordingly and respect the {@link GraphModelManager#labelLayerPolicy} and {@link GraphModelManager#portLayerPolicy} settings.
@see GraphModelManager#nodeGroup
@see HierarchicNestingPolicy
@see LabelLayerPolicy
@see PortLayerPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager Online Documentation}*/
export class GraphModelManager {
/**
Creates a manager instance for the given control.
The contents of the graph will be added to given group that is must be part of the canvas.
In order to get anything useful out of this instance the {@link GraphModelManager#graph} property should be set to an instance after construction.
@param canvas the canvas to manage the visual appearance of a graph for.
@param contentGroup the group to add the graph's visual content to. Can be null, in which case {@link GraphModelManager#contentGroup} should be set afterwards or {@link GraphModelManager#createContentGroup}
will automatically create a group lazily.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23GraphModelManager(CanvasComponent%2CICanvasObjectGroup) Online Documentation}*/
constructor(canvas: CanvasComponent, contentGroup: ICanvasObjectGroup | null);
/**
Creates a manager instance for the given control.
The contents of the graph will be added to a newly created group that is added to the root of the scene graph.
In order to get anything useful out of this instance the {@link GraphModelManager#graph} property should be set to an instance after construction.
@param canvas the canvas to manage the visual appearance of a graph for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23GraphModelManager(CanvasComponent) Online Documentation}*/
constructor(canvas: CanvasComponent);
/**
Compares two items with respect to their visibility order.
@param item1 the first item to compare
@param item2 the second item to compare
@returns an integer value indicating the relative rendering order of the two items
@see GraphModelManager#comparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23compare(IModelItem%2CIModelItem) Online Documentation}*/
compare(item1: IModelItem, item2: IModelItem): number;
/**
Factory method that creates the {@link GraphModelManager#contentGroup}
The default implementation will just add another group to the {@link CanvasComponent}.
@returns The group to use for adding the content.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createContentGroup() Online Documentation}*/
createContentGroup(): ICanvasObjectGroup;
/**
Factory method for the {@link GraphModelManager#edgeGroup} property.
This method will be called upon first access to the property.
@returns a new group that has been added to the {@link GraphModelManager#contentGroup}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createEdgeGroup() Online Documentation}*/
createEdgeGroup(): ICanvasObjectGroup;
/**
Factory method for the {@link GraphModelManager#edgeLabelGroup} property.
This method will be called upon first access to the property.
@returns a new group that has been added to the {@link GraphModelManager#contentGroup}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createEdgeLabelGroup() Online Documentation}*/
createEdgeLabelGroup(): ICanvasObjectGroup;
/**
Factory method for creating the {@link ItemModelManager} that handles the visual representation of {@link ILabel edge labels}.
@param descriptor An {@link ICanvasObjectDescriptor} for {@link ILabel edge labels}.
@param callback A callback to get the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given edge label.
@returns a new instance of {@link ItemModelManager} for edge labels
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createEdgeLabelModelManager(ICanvasObjectDescriptor%2Cfunction(ILabel)%3AICanvasObjectGroup) Online Documentation}*/
createEdgeLabelModelManager(descriptor: ICanvasObjectDescriptor, callback: (arg: ILabel) => ICanvasObjectGroup | null): ItemModelManager<ILabel>;
/**
Factory method for creating the {@link ItemModelManager} that handles the visual representation of {@link IEdge edges}.
@param descriptor An {@link ICanvasObjectDescriptor} for {@link IEdge}s.
@param callback A callback to get the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given edge.
@returns a new instance of {@link ItemModelManager} for edges
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createEdgeModelManager(ICanvasObjectDescriptor%2Cfunction(IEdge)%3AICanvasObjectGroup) Online Documentation}*/
createEdgeModelManager(descriptor: ICanvasObjectDescriptor, callback: (arg: IEdge) => ICanvasObjectGroup | null): ItemModelManager<IEdge>;
/**
Factory method for the {@link GraphModelManager#groupNodeGroup} property.
This method will be called upon first access to the property.
@returns a new group that has been added to the {@link GraphModelManager#contentGroup}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createGroupNodeGroup() Online Documentation}*/
createGroupNodeGroup(): ICanvasObjectGroup;
/**
Gets a {@link IHitTester} that can be used to enumerate hits on the canvas at a given world coordinate position.
This enumerator will enumerate hits in the order so that the visually topmost element will be reported first.
Type parameter <code>T</code>.
@param itemType
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createHitTester(Class) Online Documentation}*/
createHitTester<T extends IModelItem>(itemType: Class): IHitTester<T>;
/**
Factory method for the {@link GraphModelManager#nodeGroup} property.
This method will be called upon first access to the property.
@returns a new group that has been added to the {@link GraphModelManager#contentGroup}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createNodeGroup() Online Documentation}*/
createNodeGroup(): ICanvasObjectGroup;
/**
Factory method for the {@link GraphModelManager#nodeLabelGroup} property.
This method will be called upon first access to the property.
@returns a new group that has been added to the {@link GraphModelManager#contentGroup}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createNodeLabelGroup() Online Documentation}*/
createNodeLabelGroup(): ICanvasObjectGroup;
/**
Factory method for creating the {@link ItemModelManager} that handles the visual representation of {@link ILabel node labels}.
@param descriptor An {@link ICanvasObjectDescriptor} for {@link ILabel node labels}.
@param callback A callback to get the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given node label.
@returns a new instance of {@link ItemModelManager} for node labels
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createNodeLabelModelManager(ICanvasObjectDescriptor%2Cfunction(ILabel)%3AICanvasObjectGroup) Online Documentation}*/
createNodeLabelModelManager(descriptor: ICanvasObjectDescriptor, callback: (arg: ILabel) => ICanvasObjectGroup | null): ItemModelManager<ILabel>;
/**
Factory method for creating the {@link ItemModelManager} that handles the visual representation of {@link INode nodes}.
@param descriptor An {@link ICanvasObjectDescriptor} for {@link INode}s.
@param callback A callback to get the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given node.
@returns a new instance of {@link ItemModelManager} for nodes
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createNodeModelManager(ICanvasObjectDescriptor%2Cfunction(INode)%3AICanvasObjectGroup) Online Documentation}*/
createNodeModelManager(descriptor: ICanvasObjectDescriptor, callback: (arg: INode) => ICanvasObjectGroup | null): ItemModelManager<INode>;
/**
Factory method for the {@link GraphModelManager#portGroup} property.
This method will be called upon first access to the property.
@returns a new group that has been added to the {@link GraphModelManager#contentGroup}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createPortGroup() Online Documentation}*/
createPortGroup(): ICanvasObjectGroup;
/**
Factory method for the {@link GraphModelManager#portLabelGroup} property.
This method will be called upon first access to the property.
@returns a new group that has been added to the {@link GraphModelManager#contentGroup}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createPortLabelGroup() Online Documentation}*/
createPortLabelGroup(): ICanvasObjectGroup;
/**
Factory method for creating the {@link ItemModelManager} that handles the visual representation of {@link ILabel port labels}.
@param descriptor An {@link ICanvasObjectDescriptor} for {@link ILabel port labels}.
@param callback A callback to get the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given port label.
@returns a new instance of {@link ItemModelManager} for port labels
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createPortLabelModelManager(ICanvasObjectDescriptor%2Cfunction(ILabel)%3AICanvasObjectGroup) Online Documentation}*/
createPortLabelModelManager(descriptor: ICanvasObjectDescriptor, callback: (arg: ILabel) => ICanvasObjectGroup | null): ItemModelManager<ILabel>;
/**
Factory method for creating the {@link ItemModelManager} that handles the visual representation of {@link IPort ports}.
@param descriptor An {@link ICanvasObjectDescriptor} for {@link IPort}s.
@param callback A callback to get the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given port.
@returns a new instance of {@link ItemModelManager} for ports
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23createPortModelManager(ICanvasObjectDescriptor%2Cfunction(IPort)%3AICanvasObjectGroup) Online Documentation}*/
createPortModelManager(descriptor: ICanvasObjectDescriptor, callback: (arg: IPort) => ICanvasObjectGroup | null): ItemModelManager<IPort>;
/**
Retrieves the {@link ICanvasObject} visualizing a given model item managed by this instance.
Note that for changing the z-order via {@link ICanvasObject#raise}, {@link ICanvasObject#lower},
{@link ICanvasObject#toFront}, {@link ICanvasObject#toBack}, {@link ICanvasObject#above} or {@link ICanvasObject#below}
or for switching the {@link ICanvasObject#visible visibility} the {@link GraphModelManager#getMainCanvasObject main canvas object} should be used instead.
@param item the item
@returns the canvas object for the given item or <code>null</code>
@see ItemModelManager#getCanvasObject
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23getCanvasObject(IModelItem) Online Documentation}*/
getCanvasObject(item: IModelItem): ICanvasObject | null;
/**
Retrieves the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given edge.
This implementation is called by the default {@link GraphModelManager#edgeDescriptor} instance.
@param edge The edge that will be installed in the canvas.
@returns The group to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23getEdgeCanvasObjectGroup(IEdge) Online Documentation}*/
getEdgeCanvasObjectGroup(edge: IEdge): ICanvasObjectGroup;
/**
Retrieves the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given label.
This implementation is called by the default {@link GraphModelManager#nodeLabelDescriptor} {@link GraphModelManager#edgeLabelDescriptor} and {@link GraphModelManager#portLabelDescriptor} instances.
@param label The label that will be installed in the canvas.
@returns The group to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23getLabelCanvasObjectGroup(ILabel) Online Documentation}*/
getLabelCanvasObjectGroup(label: ILabel): ICanvasObjectGroup;
/**
Retrieves the {@link ICanvasObject} for a given model item that can be used for z-order or visibility operations.
The z-order can be changed on the returned {@link ICanvasObject} via {@link ICanvasObject#raise},
{@link ICanvasObject#lower}, {@link ICanvasObject#toFront}, {@link ICanvasObject#toBack}, {@link ICanvasObject#above}
or {@link ICanvasObject#below}.
The visibility can be changed via the {@link ICanvasObject#visible} property.
Note that these z-order or visibility changes will also affect model items whose z-order directly depends on <code>item</code> like
labels if {@link LabelLayerPolicy#AT_OWNER} is chosen.
@param item the item
@returns The main canvas object for the given item or <code>null</code>
@see ItemModelManager#getCanvasObject
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23getMainCanvasObject(IModelItem) Online Documentation}*/
getMainCanvasObject(item: IModelItem): ICanvasObject | null;
/**
Retrieves the corresponding {@link IModelItem} for the given canvas object.
@param canvasObject the instance to use for the query
@returns The item that is associated with the object or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23getModelItem(ICanvasObject) Online Documentation}*/
getModelItem(canvasObject: ICanvasObject): IModelItem | null;
/**
Retrieves the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given node.
This implementation is called by the default {@link GraphModelManager#nodeDescriptor} instance.
@param node The node that will be installed in the canvas.
@returns The group to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23getNodeCanvasObjectGroup(INode) Online Documentation}*/
getNodeCanvasObjectGroup(node: INode): ICanvasObjectGroup;
/**
Retrieves the {@link ICanvasObjectGroup} containing the {@link GraphModelManager#getCanvasObject canvas object} of a given port.
This implementation is called by the default {@link GraphModelManager#portDescriptor} instance.
@param port The port that will be installed in the canvas.
@returns The group to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23getPortCanvasObjectGroup(IPort) Online Documentation}*/
getPortCanvasObjectGroup(port: IPort): ICanvasObjectGroup;
/**
Enumerates hits on the canvas at a given world coordinate position for a given context.
The instance returned will enumerate hits in the order so that the topmost visual element will be reported first. The
enumeration is performed lazily.
@param context The context to provide to {@link IHitTestable#isHit}
@param location the world coordinates to perform the hit test
@param root The group to search the subtree of.
@returns an enumerable that will enumerate the hits at the given coordinates
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23hitElementsAt(IInputModeContext%2CPoint%2CICanvasObjectGroup) Online Documentation}*/
hitElementsAt(context: IInputModeContext, location: Point | PointConvertible, root: ICanvasObjectGroup): IEnumerable<IModelItem>;
/**
Enumerates hits on the canvas at a given world coordinate position.
The instance returned will enumerate hits in the order so that the topmost visual element will be reported first. The
enumeration is performed lazily.
@param location the world coordinates to perform the hit test
@returns an enumerable that will enumerate the hits at the given coordinates
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23hitElementsAt(Point) Online Documentation}*/
hitElementsAt(location: Point | PointConvertible): IEnumerable<IModelItem>;
/**
Lowers the visual representation of the <code>item</code> below its current predecessor.
Lowers the {@link GraphModelManager#getMainCanvasObject main canvas object} of the <code>item</code> below its current predecessor in its {@link ICanvasObject#group containing group}.
@param item The model item to lower.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23lower(IModelItem) Online Documentation}*/
lower(item: IModelItem): void;
/**
Lowers the visual representations of all {@link IModelItem}s in <code>items</code> below their current predecessors.
Lowers the {@link GraphModelManager#getMainCanvasObject main canvas objects} of all {@link IModelItem}s in <code>items</code> below their current predecessors in their
{@link ICanvasObject#group containing group}.
@param items
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23lower(IEnumerable%3CIModelItem%3E) Online Documentation}*/
lower(items: IEnumerable<IModelItem> | IEnumerableConvertible<IModelItem>): void;
/**
Called when the {@link GraphModelManager#graph} property changes.
This method registers the collections of the graph instance with the {@link CollectionModelManager mechanism} that manages the mapping between the
elements and the
{@link ICanvasObject}s. Subclasses should call the base method if they want to use the default mechanisms.
@param oldGraph The old graph instance.
@param newGraph The new graph instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23onGraphChanged(IGraph%2CIGraph) Online Documentation}*/
onGraphChanged(oldGraph: IGraph | null, newGraph: IGraph | null): void;
/**
Raises the visual representation of the <code>item</code> above its current successor.
Raises the {@link GraphModelManager#getMainCanvasObject main canvas object} of the <code>item</code> above its current successor in the {@link ICanvasObject#group containing group}.
@param item The model item to raise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23raise(IModelItem) Online Documentation}*/
raise(item: IModelItem): void;
/**
Raises the visual representations of all {@link IModelItem}s in <code>items</code> above their current successors.
Raises the {@link GraphModelManager#getMainCanvasObject main canvas objects} of all {@link IModelItem}s in <code>items</code> above their current successors in their
{@link ICanvasObject#group containing group}.
@param items
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23raise(IEnumerable%3CIModelItem%3E) Online Documentation}*/
raise(items: IEnumerable<IModelItem> | IEnumerableConvertible<IModelItem>): void;
/**
Lowers the visual representation of the <code>item</code> to the back.
Lowers the {@link GraphModelManager#getMainCanvasObject main canvas object} of the <code>item</code> to the back in its {@link ICanvasObject#group containing group}.
@param item The model item to lower to the back.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23toBack(IModelItem) Online Documentation}*/
toBack(item: IModelItem): void;
/**
Lowers the visual representation of all {@link IModelItem}s in <code>items</code> to the back.
Lowers the {@link GraphModelManager#getMainCanvasObject main canvas objects} of all {@link IModelItem}s in <code>items</code> to the back in their {@link ICanvasObject#group containing group}.
@param items
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23toBack(IEnumerable%3CIModelItem%3E) Online Documentation}*/
toBack(items: IEnumerable<IModelItem> | IEnumerableConvertible<IModelItem>): void;
/**
Raises the visual representation of the <code>item</code> to the front.
Raises the {@link GraphModelManager#getMainCanvasObject main canvas object} of the <code>item</code> to the front in its {@link ICanvasObject#group containing group}.
@param item The model item to raise to the front.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23toFront(IModelItem) Online Documentation}*/
toFront(item: IModelItem): void;
/**
Raises the visual representations of all {@link IModelItem}s in <code>items</code> to the front.
Raises the {@link GraphModelManager#getMainCanvasObject main canvas objects} of all {@link IModelItem}s in <code>items</code> to the front in their {@link ICanvasObject#group containing group}.
@param items
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23toFront(IEnumerable%3CIModelItem%3E) Online Documentation}*/
toFront(items: IEnumerable<IModelItem> | IEnumerableConvertible<IModelItem>): void;
/**
Enumerates hits on the canvas at a given world coordinate position in a given context.
The instance returned will enumerate hits in the order so that the topmost visual element will be reported first. The
enumeration is performed lazily.
Type parameter <code>T</code>.
@param modelItemType
@param context The context to provide to {@link IHitTestable#isHit}
@param location the world coordinates to perform the hit test
@param root The group to search the subtree of.
@returns an enumerable that will enumerate the hits at the given coordinates
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23typedHitElementsAt(Class%2CIInputModeContext%2CPoint%2CICanvasObjectGroup) Online Documentation}*/
typedHitElementsAt<T extends IModelItem>(modelItemType: Class, context: IInputModeContext, location: Point | PointConvertible, root: ICanvasObjectGroup): IEnumerable<T>;
/**
Enumerates hits on the canvas at a given world coordinate position.
The instance returned will enumerate hits in the order so that the topmost visual element will be reported first. The
enumeration is performed lazily.
Type parameter <code>T</code>.
@param itemType
@param location the world coordinates to perform the hit test
@returns an enumerable that will enumerate the hits at the given coordinates
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23typedHitElementsAt(Class%2CPoint) Online Documentation}*/
typedHitElementsAt<T extends IModelItem>(itemType: Class, location: Point | PointConvertible): IEnumerable<T>;
/**
{@link CollectionModelManager#update Updates} the visual representation of the given item.
@param item The item to update the visual representation
@see CollectionModelManager#update
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23update(IModelItem) Online Documentation}*/
update(item: IModelItem | null): void;
/**
Provides access to the {@link ItemModelManager} which handles the {@link INode}s.
The manager is created by {@link GraphModelManager#createNodeModelManager} upon first access.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23nodeManager Online Documentation}*/
readonly nodeManager: ItemModelManager<INode>;
/**
Provides access to the {@link ItemModelManager} which handles the {@link IEdge}s.
The manager is created by {@link GraphModelManager#createEdgeModelManager} upon first access.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23edgeManager Online Documentation}*/
readonly edgeManager: ItemModelManager<IEdge>;
/**
Provides access to the {@link ItemModelManager} which handles the {@link IPort}s.
The manager is created by {@link GraphModelManager#createPortModelManager} upon first access.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23portManager Online Documentation}*/
readonly portManager: ItemModelManager<IPort>;
/**
Provides access to the {@link ItemModelManager} which handles the {@link ILabel}s at edges.
The manager is created by {@link GraphModelManager#createEdgeLabelModelManager} upon first access.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23edgeLabelManager Online Documentation}*/
readonly edgeLabelManager: ItemModelManager<ILabel>;
/**
Provides access to the {@link ItemModelManager} which handles the {@link ILabel}s at nodes.
The manager is created by {@link GraphModelManager#createNodeLabelModelManager} upon first access.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23nodeLabelManager Online Documentation}*/
readonly nodeLabelManager: ItemModelManager<ILabel>;
/**
Provides access to the {@link ItemModelManager} which handles the {@link ILabel}s at ports.
The manager is created by {@link GraphModelManager#createPortLabelModelManager} upon first access.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23portLabelManager Online Documentation}*/
readonly portLabelManager: ItemModelManager<ILabel>;
/**
Gets the {@link ICanvasObjectGroup} instance that will be used for installing node styles of {@link IGraph#isGroupNode group nodes}.
If the field has not yet been initialized upon first access, the factory method {@link GraphModelManager#createGroupNodeGroup}
will be called. Note that this group is only used when {@link GraphModelManager#hierarchicNestingPolicy} is {@link HierarchicNestingPolicy#GROUP_NODES}.
@see GraphModelManager#getNodeCanvasObjectGroup
@see GraphModelManager#nodeDescriptor
@see GraphModelManager#hierarchicNestingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23groupNodeGroup Online Documentation}*/
readonly groupNodeGroup: ICanvasObjectGroup;
/**
Gets the {@link ICanvasObjectGroup} instance that will be used for installing node styles.
If the field has not yet been initialized upon first access, the factory method {@link GraphModelManager#createNodeGroup}
will be called.
Note that the content of this group depends on the {@link GraphModelManager#hierarchicNestingPolicy}:
<ul>
<li><b>{@link HierarchicNestingPolicy#NONE}</b> – Contains the {@link GraphModelManager#getMainCanvasObject main canvas objects} of all nodes top-level.
<li><b>{@link HierarchicNestingPolicy#NODES}</b> – Contains the {@link GraphModelManager#getMainCanvasObject main canvas objects} of all nodes in a more complicated nesting of dynamically created
{@link ICanvasObjectGroup}s.
<li><b>{@link HierarchicNestingPolicy#NODES_AND_EDGES}</b> – Contains the {@link GraphModelManager#getMainCanvasObject main canvas objects} of all nodes and edges in a more complicated nesting of dynamically created
{@link ICanvasObjectGroup}s.
<li><b>{@link HierarchicNestingPolicy#GROUP_NODES}</b> – Contains the {@link GraphModelManager#getMainCanvasObject main canvas objects} of all leaf nodes. Group nodes having the same hierarchical depth are in one child {@link ICanvasObjectGroup}
of the {@link GraphModelManager#groupNodeGroup} instead.
</ul>
@see GraphModelManager#getNodeCanvasObjectGroup
@see GraphModelManager#getMainCanvasObject
@see GraphModelManager#nodeDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23nodeGroup Online Documentation}*/
readonly nodeGroup: ICanvasObjectGroup;
/**
Gets the {@link ICanvasObjectGroup} instance that will be used for installing edge styles.
If the field has not yet been initialized upon first access, the factory method {@link GraphModelManager#createEdgeGroup}
will be called.
Note that in the case of {@link GraphModelManager#hierarchicNestingPolicy hierarchically nested graphs} the the {@link GraphModelManager#getMainCanvasObject main canvas objects} of edges will only be put directly in
this {@link ICanvasObjectGroup}, if the property is not set to {@link HierarchicNestingPolicy#NODES_AND_EDGES}.
Otherwise they will actually be nested in the subtree of the {@link GraphModelManager#nodeGroup} according to the node nesting.
@see GraphModelManager#getEdgeCanvasObjectGroup
@see GraphModelManager#edgeDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23edgeGroup Online Documentation}*/
readonly edgeGroup: ICanvasObjectGroup;
/**
Gets the {@link ICanvasObjectGroup} instance that will be used for installing edge label styles.
If the field has not yet been initialized upon first access, the factory method {@link GraphModelManager#createEdgeLabelGroup}
will be called.
@see GraphModelManager#getLabelCanvasObjectGroup
@see GraphModelManager#edgeLabelDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23edgeLabelGroup Online Documentation}*/
readonly edgeLabelGroup: ICanvasObjectGroup;
/**
Gets the {@link ICanvasObjectGroup} instance that will be used for installing node label styles.
If the field has not yet been initialized upon first access, the factory method {@link GraphModelManager#createNodeLabelGroup}
will be called.
@see GraphModelManager#getLabelCanvasObjectGroup
@see GraphModelManager#nodeLabelDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23nodeLabelGroup Online Documentation}*/
readonly nodeLabelGroup: ICanvasObjectGroup;
/**
Gets the {@link ICanvasObjectGroup} instance that will be used for installing port styles.
If the field has not yet been initialized upon first access, the factory method {@link GraphModelManager#createPortGroup}
will be called.
@see GraphModelManager#getPortCanvasObjectGroup
@see GraphModelManager#portDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23portGroup Online Documentation}*/
readonly portGroup: ICanvasObjectGroup;
/**
Gets the {@link ICanvasObjectGroup} instance that will be used for installing port label styles.
If the field has not yet been initialized upon first access, the factory method {@link GraphModelManager#createPortLabelGroup}
will be called.
@see GraphModelManager#getLabelCanvasObjectGroup
@see GraphModelManager#portLabelDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23portLabelGroup Online Documentation}*/
readonly portLabelGroup: ICanvasObjectGroup;
/**
Gets or sets the {@link ICanvasObjectDescriptor} instance used for installing an {@link IEdge} into the
canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23edgeDescriptor Online Documentation}*/
edgeDescriptor: ICanvasObjectDescriptor;
/**
Gets or sets the {@link ICanvasObjectDescriptor} instance used for installing an {@link IPort} into the
canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23portDescriptor Online Documentation}*/
portDescriptor: ICanvasObjectDescriptor;
/**
Gets or sets the {@link ICanvasObjectDescriptor} instance used for installing an {@link ILabel} of an {@link IEdge}
into the canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23edgeLabelDescriptor Online Documentation}*/
edgeLabelDescriptor: ICanvasObjectDescriptor;
/**
Gets or sets the {@link ICanvasObjectDescriptor} instance used for installing an {@link ILabel} of an {@link INode}
into the canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23nodeLabelDescriptor Online Documentation}*/
nodeLabelDescriptor: ICanvasObjectDescriptor;
/**
Gets or sets the {@link ICanvasObjectDescriptor} instance used for installing an {@link ILabel} of an {@link IPort}
into the canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23portLabelDescriptor Online Documentation}*/
portLabelDescriptor: ICanvasObjectDescriptor;
/**
Gets or sets the {@link ICanvasObjectDescriptor} instance used for installing an {@link INode} into the
canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23nodeDescriptor Online Documentation}*/
nodeDescriptor: ICanvasObjectDescriptor;
/**
Gets or sets the content group this manager instance uses to add visual representation of the graph to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23contentGroup Online Documentation}*/
contentGroup: ICanvasObjectGroup;
/**
Gets an {@link IComparer} instance that can be used to compare two model items with respect to their visual display order.
An item is considered to be greater than another item if it is rendered on top of it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23comparer Online Documentation}*/
readonly comparer: IComparer<IModelItem>;
/**
Gets a {@link IHitTester} that can be used to enumerate hits on the canvas at a given world coordinate position.
This enumerator will enumerate hits in the order so that the visually topmost element will be reported first.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23hitTester Online Documentation}*/
readonly hitTester: IHitTester<IModelItem>;
/**
Gets or sets the graph this manager manages.
@see GraphModelManager#onGraphChanged
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23graph Online Documentation}*/
graph: IGraph | null;
/**
Gets or sets the policy whether and how this instance should nest the nodes and edges if the graph is grouped.
The default is {@link HierarchicNestingPolicy#NODES_AND_EDGES}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23hierarchicNestingPolicy Online Documentation}*/
hierarchicNestingPolicy: HierarchicNestingPolicy;
/**
Gets or sets the policy to determines at which visual layer node, edge and port labels shall be inserted.
The default value is {@link LabelLayerPolicy#SEPARATE_LAYER}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23labelLayerPolicy Online Documentation}*/
labelLayerPolicy: LabelLayerPolicy;
/**
Gets or sets the policy to determines at which visual layer node and edge ports shall be inserted.
The default value is {@link PortLayerPolicy#SEPARATE_LAYER}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23portLayerPolicy Online Documentation}*/
portLayerPolicy: PortLayerPolicy;
/**
Gets or sets if the {@link ICanvasObject#userObject} should be provided by its {@link GraphModelManager#getMainCanvasObject main canvas object}.
The {@link ICanvasObject#userObject} is already provided by the {@link GraphModelManager#getCanvasObject canvas object} visualizing the model item. As this canvas object may differ from the
{@link GraphModelManager#getMainCanvasObject main canvas object}, this property determines only if the {@link ICanvasObject#userObject} is provided by its {@link GraphModelManager#getMainCanvasObject main canvas object} if it differs from
{@link GraphModelManager#getCanvasObject}.
The default value is <code>false</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23provideUserObjectOnMainCanvasObject Online Documentation}*/
provideUserObjectOnMainCanvasObject: boolean;
/**
The immutable instance of the default {@link ICanvasObjectDescriptor} for {@link INode}s which simply defers to
the node's {@link INodeStyle}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23DEFAULT_NODE_DESCRIPTOR Online Documentation}*/
static DEFAULT_NODE_DESCRIPTOR: ICanvasObjectDescriptor;
/**
The immutable instance of the default {@link ICanvasObjectDescriptor} for {@link IEdge}s which simply defers to
the edge's {@link IEdgeStyle}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23DEFAULT_EDGE_DESCRIPTOR Online Documentation}*/
static DEFAULT_EDGE_DESCRIPTOR: ICanvasObjectDescriptor;
/**
The immutable instance of the default {@link ICanvasObjectDescriptor} for {@link IPort}s which simply defers to
the port's {@link IPortStyle}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23DEFAULT_PORT_DESCRIPTOR Online Documentation}*/
static DEFAULT_PORT_DESCRIPTOR: ICanvasObjectDescriptor;
/**
The immutable instance of the default {@link ICanvasObjectDescriptor} for {@link ILabel}s which simply defers
to the label's {@link ILabelStyle}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphModelManager%23DEFAULT_LABEL_DESCRIPTOR Online Documentation}*/
static DEFAULT_LABEL_DESCRIPTOR: ICanvasObjectDescriptor;
static $class: Class;
private $_GraphModelManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphModelManager;
}
export interface GraphObstacleProvider extends YObject, IObstacleProvider {
}
/**
An implementation of the {@link IObstacleProvider} interface that uses the {@link IGraph#edges edges} and {@link IGraph#nodes nodes} of an {@link IGraph}
as obstacles.
The {@link BridgeManager} then calculates bridges for edge paths that cross these obstacles.
This class can be used to add bridge support via the {@link BridgeManager} to a rendered {@link IGraph}.
@see BridgeManager#addObstacleProvider
@see BridgeManager
@see IObstacleProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphObstacleProvider Online Documentation}*/
export class GraphObstacleProvider {
/**
Helper method that retrieves the {@link IGraph} to use from the <code>context</code>
This implementation uses the {@link ILookup#lookup} of the {@link ICanvasContext#canvasComponent} to query the {@link IGraph} implementation.
@param context The context to retrieve the implementation from.
@returns The {@link IGraph} instance to query or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphObstacleProvider%23getGraph(IRenderContext) Online Documentation}*/
getGraph(context: IRenderContext): IGraph | null;
/**
Iterates over all {@link IGraph#edges} and {@link IGraph#nodes} to query an {@link IObstacleProvider} from the {@link IModelItem}'s
{@link ILookup#lookup}.
This method will depending on the {@link GraphObstacleProvider#queryEdges} and {@link GraphObstacleProvider#queryNodes} property query the items for an implementation of {@link IObstacleProvider}
and concatenate all resulting {@link GeneralPath} obstacles into one path that will then be returned.
@param context The context where the obstacles are queried for.
@returns A path that is the concatenated path of all obstacles for the given context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphObstacleProvider%23getObstacles(IRenderContext) Online Documentation}*/
getObstacles(context: IRenderContext): GeneralPath | null;
/**
Gets or sets a value indicating whether to query the {@link IGraph#edges} for an {@link IObstacleProvider} implementation.
The default is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphObstacleProvider%23queryEdges Online Documentation}*/
queryEdges: boolean;
/**
Gets or sets a value indicating whether to query the {@link IGraph#nodes} for an {@link IObstacleProvider} implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphObstacleProvider%23queryNodes Online Documentation}*/
queryNodes: boolean;
static $class: Class;
private $_GraphObstacleProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphObstacleProvider;
}
export interface GraphSelection extends YObject, IGraphSelection {
/**
@inheritdoc*/
clear(): void;
/**
@inheritdoc*/
isSelected(item: IModelItem): boolean;
/**
@inheritdoc*/
setSelected(item: IModelItem, selected: boolean): void;
/**
@inheritdoc*/
readonly selectedNodes: ISelectionModel<INode>;
/**
@inheritdoc*/
readonly selectedEdges: ISelectionModel<IEdge>;
/**
@inheritdoc*/
readonly selectedLabels: ISelectionModel<ILabel>;
/**
@inheritdoc*/
readonly selectedPorts: ISelectionModel<IPort>;
/**
@inheritdoc*/
readonly selectedBends: ISelectionModel<IBend>;
/**
@inheritdoc*/
addItemSelectionChangedListener(listener: (sender: any, evt: ItemSelectionChangedEventArgs<IModelItem>) => void): void;
/**
@inheritdoc*/
removeItemSelectionChangedListener(listener: (sender: any, evt: ItemSelectionChangedEventArgs<IModelItem>) => void): void;
} /**
Simple implementation of the {@link IGraphSelection} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection Online Documentation}*/
export class GraphSelection {
/**
Instantiates an instance using the the graph as the model.
@param graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23GraphSelection(IGraph) Online Documentation}*/
constructor(graph: IGraph | null);
/**
Creates a new instance with no initial {@link GraphSelection#graph} assigned.
In order to be always in sync with the structural changes in a graph, an instance should be associated with an {@link IGraph}
using the {@link GraphSelection#graph} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23GraphSelection() Online Documentation}*/
constructor();
/**
Factory method that creates the {@link DefaultSelectionModel} to use for the bends.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23createBendSelectionModel() Online Documentation}*/
createBendSelectionModel(): DefaultSelectionModel<IBend>;
/**
Factory method that creates the {@link DefaultSelectionModel} to use for the edges.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23createEdgeSelectionModel() Online Documentation}*/
createEdgeSelectionModel(): DefaultSelectionModel<IEdge>;
/**
Factory method that creates the {@link DefaultSelectionModel} to use for the labels.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23createLabelSelectionModel() Online Documentation}*/
createLabelSelectionModel(): DefaultSelectionModel<ILabel>;
/**
Factory method that creates the {@link DefaultSelectionModel} to use for the nodes.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23createNodeSelectionModel() Online Documentation}*/
createNodeSelectionModel(): DefaultSelectionModel<INode>;
/**
Factory method that creates the {@link DefaultSelectionModel} to use for the ports.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23createPortSelectionModel() Online Documentation}*/
createPortSelectionModel(): DefaultSelectionModel<IPort>;
/**
Returns an enumerator that iterates through the collection of all selected elements.
@returns An {@link IEnumerator} object that can be used to iterate through all selected elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23getEnumerator() Online Documentation}*/
getEnumerator(): IEnumerator<IModelItem>;
/**
Raises the {@link GraphSelection#addItemSelectionChangedListener ItemSelectionChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23onItemSelectionChanged(ItemSelectionChangedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemSelectionChanged(evt: ItemSelectionChangedEventArgs<IModelItem>): void;
/**
Gets or sets the {@link IGraph} this instance is using as the domain for the selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphSelection%23graph Online Documentation}*/
graph: IGraph | null;
static $class: Class;
private $_GraphSelection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphSelection;
}
export interface GraphWrapperBase extends YObject, IGraph {
/**
@inheritdoc*/
addBend(edge: IEdge, location: Point | PointConvertible, index?: number): IBend;
/**
@inheritdoc*/
addLabel(owner: ILabelOwner, text: string, layoutParameter?: ILabelModelParameter | null, style?: ILabelStyle | null, preferredSize?: Size | null | SizeConvertible, tag?: any | null): ILabel;
/**
@inheritdoc*/
addLabel(options: {
owner: ILabelOwner;
text: string;
layoutParameter?: ILabelModelParameter | null;
style?: ILabelStyle | null;
preferredSize?: Size | null | SizeConvertible;
tag?: any | null;
}): ILabel;
/**
@inheritdoc*/
addPort(owner: IPortOwner, locationParameter?: IPortLocationModelParameter | null, style?: IPortStyle | null, tag?: any | null): IPort;
/**
@inheritdoc*/
addPort(options: {
owner: IPortOwner;
locationParameter?: IPortLocationModelParameter | null;
style?: IPortStyle | null;
tag?: any | null;
}): IPort;
/**
@inheritdoc*/
contains(item: IModelItem | null): boolean;
/**
@inheritdoc*/
createEdge(source: INode, target: INode, style?: IEdgeStyle | null, tag?: any | null): IEdge;
/**
@inheritdoc*/
createEdge(options: {
source: INode;
target: INode;
style?: IEdgeStyle | null;
tag?: any | null;
}): IEdge;
/**
@inheritdoc*/
createEdge(sourcePort: IPort, targetPort: IPort, style?: IEdgeStyle | null, tag?: any | null): IEdge;
/**
@inheritdoc*/
createEdge(options: {
sourcePort: IPort;
targetPort: IPort;
style?: IEdgeStyle | null;
tag?: any | null;
}): IEdge;
/**
@inheritdoc*/
createGroupNode(parent?: INode | null, layout?: Rect | null | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
@inheritdoc*/
createGroupNode(options: {
parent?: INode | null;
layout?: Rect | null | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
}): INode;
/**
@inheritdoc*/
createNode(layout: Rect | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
@inheritdoc*/
createNode(options: {
layout: Rect | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
}): INode;
/**
@inheritdoc*/
createNode(parent?: INode | null, layout?: Rect | null | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
@inheritdoc*/
createNode(options: {
parent?: INode | null;
layout?: Rect | null | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
}): INode;
/**
@inheritdoc*/
edgesAt(owner: IPortOwner, type?: AdjacencyTypes | AdjacencyTypesConvertible): IListEnumerable<IEdge>;
/**
@inheritdoc*/
edgesAt(port: IPort, type?: AdjacencyTypes | AdjacencyTypesConvertible): IListEnumerable<IEdge>;
/**
@inheritdoc*/
getChildren(node: INode | null): IListEnumerable<INode>;
/**
@inheritdoc*/
getParent(node: INode): INode | null;
/**
@inheritdoc*/
invalidateDisplays(): void;
/**
@inheritdoc*/
isGroupNode(node: INode | null): boolean;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
remove(item: IModelItem): void;
/**
@inheritdoc*/
setBendLocation(bend: IBend, location: Point): void;
/**
@inheritdoc*/
setEdgePorts(edge: IEdge, sourcePort: IPort, targetPort: IPort): void;
/**
@inheritdoc*/
setIsGroupNode(node: INode | null, isGroupNode: boolean): void;
/**
@inheritdoc*/
setLabelLayoutParameter(label: ILabel, layoutParameter: ILabelModelParameter): void;
/**
@inheritdoc*/
setLabelPreferredSize(label: ILabel, preferredSize: Size): void;
/**
@inheritdoc*/
setLabelText(label: ILabel, text: string): void;
/**
@inheritdoc*/
setNodeLayout(node: INode, layout: Rect): void;
/**
@inheritdoc*/
setParent(node: INode, parent: INode | null): void;
/**
@inheritdoc*/
setPortLocationParameter(port: IPort, locationParameter: IPortLocationModelParameter): void;
/**
@inheritdoc*/
setStyle(port: IPort, style: IPortStyle): void;
/**
@inheritdoc*/
setStyle(edge: IEdge, style: IEdgeStyle): void;
/**
@inheritdoc*/
setStyle(label: ILabel, style: ILabelStyle): void;
/**
@inheritdoc*/
setStyle(node: INode, style: INodeStyle): void;
/**
@inheritdoc*/
readonly nodes: IListEnumerable<INode>;
/**
@inheritdoc*/
readonly edges: IListEnumerable<IEdge>;
/**
@inheritdoc*/
readonly labels: IListEnumerable<ILabel>;
/**
@inheritdoc*/
readonly ports: IListEnumerable<IPort>;
/**
@inheritdoc*/
edgeDefaults: IEdgeDefaults;
/**
@inheritdoc*/
nodeDefaults: INodeDefaults;
/**
@inheritdoc*/
readonly mapperRegistry: IMapperRegistry;
/**
@inheritdoc*/
groupNodeDefaults: INodeDefaults;
/**
@inheritdoc*/
tag: any | null;
/**
@inheritdoc*/
addGraphTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IGraph, any>) => void): void;
/**
@inheritdoc*/
removeGraphTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IGraph, any>) => void): void;
/**
@inheritdoc*/
addNodeCreatedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
@inheritdoc*/
removeNodeCreatedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
@inheritdoc*/
addNodeRemovedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
removeNodeRemovedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
addNodeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, INodeStyle>) => void): void;
/**
@inheritdoc*/
removeNodeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, INodeStyle>) => void): void;
/**
@inheritdoc*/
addNodeLayoutChangedListener(listener: (source: any, node: INode, oldLayout: Rect) => void): void;
/**
@inheritdoc*/
removeNodeLayoutChangedListener(listener: (source: any, node: INode, oldLayout: Rect) => void): void;
/**
@inheritdoc*/
addNodeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, any>) => void): void;
/**
@inheritdoc*/
removeNodeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, any>) => void): void;
/**
@inheritdoc*/
addEdgeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IEdge>) => void): void;
/**
@inheritdoc*/
removeEdgeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IEdge>) => void): void;
/**
@inheritdoc*/
addEdgeRemovedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
@inheritdoc*/
removeEdgeRemovedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
@inheritdoc*/
addEdgeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, IEdgeStyle>) => void): void;
/**
@inheritdoc*/
removeEdgeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, IEdgeStyle>) => void): void;
/**
@inheritdoc*/
addEdgePortsChangedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
@inheritdoc*/
removeEdgePortsChangedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
@inheritdoc*/
addEdgeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, any>) => void): void;
/**
@inheritdoc*/
removeEdgeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, any>) => void): void;
/**
@inheritdoc*/
addBendAddedListener(listener: (sender: any, evt: ItemEventArgs<IBend>) => void): void;
/**
@inheritdoc*/
removeBendAddedListener(listener: (sender: any, evt: ItemEventArgs<IBend>) => void): void;
/**
@inheritdoc*/
addBendRemovedListener(listener: (sender: any, evt: BendEventArgs) => void): void;
/**
@inheritdoc*/
removeBendRemovedListener(listener: (sender: any, evt: BendEventArgs) => void): void;
/**
@inheritdoc*/
addBendLocationChangedListener(listener: (source: any, bend: IBend, oldLocation: Point) => void): void;
/**
@inheritdoc*/
removeBendLocationChangedListener(listener: (source: any, bend: IBend, oldLocation: Point) => void): void;
/**
@inheritdoc*/
addBendTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IBend, any>) => void): void;
/**
@inheritdoc*/
removeBendTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IBend, any>) => void): void;
/**
@inheritdoc*/
addLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
@inheritdoc*/
removeLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
@inheritdoc*/
addLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
@inheritdoc*/
removeLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
@inheritdoc*/
addLabelStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelStyle>) => void): void;
/**
@inheritdoc*/
removeLabelStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelStyle>) => void): void;
/**
@inheritdoc*/
addLabelPreferredSizeChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, Size>) => void): void;
/**
@inheritdoc*/
removeLabelPreferredSizeChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, Size>) => void): void;
/**
@inheritdoc*/
addLabelTextChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, string>) => void): void;
/**
@inheritdoc*/
removeLabelTextChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, string>) => void): void;
/**
@inheritdoc*/
addLabelLayoutParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelModelParameter>) => void): void;
/**
@inheritdoc*/
removeLabelLayoutParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelModelParameter>) => void): void;
/**
@inheritdoc*/
addLabelTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, any>) => void): void;
/**
@inheritdoc*/
removeLabelTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, any>) => void): void;
/**
@inheritdoc*/
addPortAddedListener(listener: (sender: any, evt: ItemEventArgs<IPort>) => void): void;
/**
@inheritdoc*/
removePortAddedListener(listener: (sender: any, evt: ItemEventArgs<IPort>) => void): void;
/**
@inheritdoc*/
addPortRemovedListener(listener: (sender: any, evt: PortEventArgs) => void): void;
/**
@inheritdoc*/
removePortRemovedListener(listener: (sender: any, evt: PortEventArgs) => void): void;
/**
@inheritdoc*/
addPortStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortStyle>) => void): void;
/**
@inheritdoc*/
removePortStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortStyle>) => void): void;
/**
@inheritdoc*/
addPortLocationParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortLocationModelParameter>) => void): void;
/**
@inheritdoc*/
removePortLocationParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortLocationModelParameter>) => void): void;
/**
@inheritdoc*/
addPortTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, any>) => void): void;
/**
@inheritdoc*/
removePortTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, any>) => void): void;
/**
@inheritdoc*/
addDisplaysInvalidatedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
@inheritdoc*/
removeDisplaysInvalidatedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
@inheritdoc*/
addParentChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
removeParentChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
addIsGroupNodeChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
@inheritdoc*/
removeIsGroupNodeChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
} /**
An abstract {@link IGraph} implementation that delegates all of its work to an existing implementation.
Subclasses can override all of the methods to customize the behavior of this instance. Note that this instance will
register listeners with the wrapped graph instance, so {@link GraphWrapperBase#dispose} should be called if this
instance is not used any more.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase Online Documentation}*/
export abstract class GraphWrapperBase {
/**
Creates a new instance that wraps a given {@link IGraph}.
Note that this instance will {@link GraphWrapperBase#addEventHandlers register listeners} with the <code>graph</code>, so {@link GraphWrapperBase#dispose} should be
called if this instance is not used any more.
@param graph The graph to wrap.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase Online Documentation}*/
constructor(graph: IGraph | null);
/**
Adds event handlers that propagate graph events from <code>graph</code> to listeners on this instance.
@param graph The graph for which event propagators should be created
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23addEventHandlers(IGraph) Online Documentation}*/
addEventHandlers(graph: IGraph): void;
/**
Clears the internal state of this instance and released memory and bound listeners.
After this instance has been disposed of, it will not work as expected anymore. This implementation {@link GraphWrapperBase#removeEventHandlers removes all event handlers}
from the {@link GraphWrapperBase#wrappedGraph} and releases the reference.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23dispose() Online Documentation}*/
dispose(): void;
/**
Gets the lookup instance to use in {@link GraphWrapperBase#lookup}
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23getLookup() Online Documentation}*/
getLookup(): ILookup | null;
/**
Raises the {@link GraphWrapperBase#addBendAddedListener BendAdded} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onBendAdded(ItemEventArgs%3CIBend%3E) Online Documentation}*/
onBendAdded(evt: ItemEventArgs<IBend>): void;
/**
Raises the {@link GraphWrapperBase#addBendLocationChangedListener BendLocationChanged} event
@param bend The bend whose location changed
@param oldLocation The bend location before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onBendLocationChanged(IBend%2CPoint) Online Documentation}*/
onBendLocationChanged(bend: IBend, oldLocation: Point): void;
/**
Raises the {@link GraphWrapperBase#addBendRemovedListener BendRemoved} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onBendRemoved(BendEventArgs) Online Documentation}*/
onBendRemoved(evt: BendEventArgs): void;
/**
Raises the {@link GraphWrapperBase#addBendTagChangedListener BendTagChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onBendTagChanged(ItemChangedEventArgs%3CIBend%2CObject%3E) Online Documentation}*/
onBendTagChanged(evt: ItemChangedEventArgs<IBend, any>): void;
/**
Raises the {@link GraphWrapperBase#addEdgeCreatedListener EdgeCreated} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onEdgeCreated(ItemEventArgs%3CIEdge%3E) Online Documentation}*/
onEdgeCreated(evt: ItemEventArgs<IEdge>): void;
/**
Raises the {@link GraphWrapperBase#addEdgePortsChangedListener EdgePortsChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onEdgePortsChanged(EdgeEventArgs) Online Documentation}*/
onEdgePortsChanged(evt: EdgeEventArgs): void;
/**
Raises the {@link GraphWrapperBase#addEdgeRemovedListener EdgeRemoved} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onEdgeRemoved(EdgeEventArgs) Online Documentation}*/
onEdgeRemoved(evt: EdgeEventArgs): void;
/**
Raises the {@link GraphWrapperBase#addEdgeStyleChangedListener EdgeStyleChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onEdgeStyleChanged(ItemChangedEventArgs%3CIEdge%2CIEdgeStyle%3E) Online Documentation}*/
onEdgeStyleChanged(evt: ItemChangedEventArgs<IEdge, IEdgeStyle>): void;
/**
Raises the {@link GraphWrapperBase#addEdgeTagChangedListener EdgeTagChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onEdgeTagChanged(ItemChangedEventArgs%3CIEdge%2CObject%3E) Online Documentation}*/
onEdgeTagChanged(evt: ItemChangedEventArgs<IEdge, any>): void;
/**
Called when the {@link GraphWrapperBase#wrappedGraph} property value changes and after initialization of the field.
Delegates to {@link GraphWrapperBase#addEventHandlers} for <code>newGraph</code> and to {@link GraphWrapperBase#removeEventHandlers}
for <code>oldGraph</code>.
@param oldGraph the old value, which may be <code>null</code> the first time
@param newGraph the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onGraphChanged(IGraph%2CIGraph) Online Documentation}*/
onGraphChanged(oldGraph: IGraph | null, newGraph: IGraph | null): void;
/**
Raises the {@link GraphWrapperBase#addGraphTagChangedListener GraphTagChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onGraphTagChanged(ItemChangedEventArgs%3CIGraph%2CObject%3E) Online Documentation}*/
onGraphTagChanged(evt: ItemChangedEventArgs<IGraph, any>): void;
/**
Raises the {@link GraphWrapperBase#addDisplaysInvalidatedListener DisplaysInvalidated} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onInvalidateDisplays(EventArgs) Online Documentation}*/
onInvalidateDisplays(evt: EventArgs): void;
/**
Dispatches the {@link GraphWrapperBase#addIsGroupNodeChangedListener IsGroupNodeChanged} event.
@param evt The event args to dispatch.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onIsGroupNodeChanged(NodeEventArgs) Online Documentation}*/
onIsGroupNodeChanged(evt: NodeEventArgs): void;
/**
Raises the {@link GraphWrapperBase#addLabelAddedListener LabelAdded} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onLabelAdded(ItemEventArgs%3CILabel%3E) Online Documentation}*/
onLabelAdded(evt: ItemEventArgs<ILabel>): void;
/**
Raises the {@link GraphWrapperBase#addLabelLayoutParameterChangedListener LabelLayoutParameterChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onLabelLayoutParameterChanged(ItemChangedEventArgs%3CILabel%2CILabelModelParameter%3E) Online Documentation}*/
onLabelLayoutParameterChanged(evt: ItemChangedEventArgs<ILabel, ILabelModelParameter>): void;
/**
Raises the {@link GraphWrapperBase#addLabelPreferredSizeChangedListener LabelPreferredSizeChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onLabelPreferredSizeChanged(ItemChangedEventArgs%3CILabel%2CSize%3E) Online Documentation}*/
onLabelPreferredSizeChanged(evt: ItemChangedEventArgs<ILabel, Size>): void;
/**
Raises the {@link GraphWrapperBase#addLabelRemovedListener LabelRemoved} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onLabelRemoved(LabelEventArgs) Online Documentation}*/
onLabelRemoved(evt: LabelEventArgs): void;
/**
Raises the {@link GraphWrapperBase#addLabelStyleChangedListener LabelStyleChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onLabelStyleChanged(ItemChangedEventArgs%3CILabel%2CILabelStyle%3E) Online Documentation}*/
onLabelStyleChanged(evt: ItemChangedEventArgs<ILabel, ILabelStyle>): void;
/**
Raises the {@link GraphWrapperBase#addLabelTagChangedListener LabelTagChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onLabelTagChanged(ItemChangedEventArgs%3CILabel%2CObject%3E) Online Documentation}*/
onLabelTagChanged(evt: ItemChangedEventArgs<ILabel, any>): void;
/**
Raises the {@link GraphWrapperBase#addLabelTextChangedListener LabelTextChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onLabelTextChanged(ItemChangedEventArgs%3CILabel%2Cstring%3E) Online Documentation}*/
onLabelTextChanged(evt: ItemChangedEventArgs<ILabel, string>): void;
/**
Raises the {@link GraphWrapperBase#addNodeCreatedListener NodeCreated} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onNodeCreated(ItemEventArgs%3CINode%3E) Online Documentation}*/
onNodeCreated(evt: ItemEventArgs<INode>): void;
/**
Raises the {@link GraphWrapperBase#addNodeLayoutChangedListener NodeLayoutChanged} event
@param node The node whose layout changed
@param oldLayout The node layout before the change
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onNodeLayoutChanged(INode%2CRect) Online Documentation}*/
onNodeLayoutChanged(node: INode, oldLayout: Rect): void;
/**
Raises the {@link GraphWrapperBase#addNodeRemovedListener NodeRemoved} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onNodeRemoved(NodeEventArgs) Online Documentation}*/
onNodeRemoved(evt: NodeEventArgs): void;
/**
Raises the {@link GraphWrapperBase#addNodeStyleChangedListener NodeStyleChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onNodeStyleChanged(ItemChangedEventArgs%3CINode%2CINodeStyle%3E) Online Documentation}*/
onNodeStyleChanged(evt: ItemChangedEventArgs<INode, INodeStyle>): void;
/**
Raises the {@link GraphWrapperBase#addNodeTagChangedListener NodeTagChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onNodeTagChanged(ItemChangedEventArgs%3CINode%2CObject%3E) Online Documentation}*/
onNodeTagChanged(evt: ItemChangedEventArgs<INode, any>): void;
/**
Dispatches the {@link GraphWrapperBase#addParentChangedListener ParentChanged} event.
@param evt The event args to dispatch.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onParentChanged(NodeEventArgs) Online Documentation}*/
onParentChanged(evt: NodeEventArgs): void;
/**
Raises the {@link GraphWrapperBase#addPortAddedListener PortAdded} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onPortAdded(ItemEventArgs%3CIPort%3E) Online Documentation}*/
onPortAdded(evt: ItemEventArgs<IPort>): void;
/**
Raises the {@link GraphWrapperBase#addPortLocationParameterChangedListener PortLocationParameterChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onPortLocationParameterChanged(ItemChangedEventArgs%3CIPort%2CIPortLocationModelParameter%3E) Online Documentation}*/
onPortLocationParameterChanged(evt: ItemChangedEventArgs<IPort, IPortLocationModelParameter>): void;
/**
Raises the {@link GraphWrapperBase#addPortRemovedListener PortRemoved} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onPortRemoved(PortEventArgs) Online Documentation}*/
onPortRemoved(evt: PortEventArgs): void;
/**
Raises the {@link GraphWrapperBase#addPortStyleChangedListener PortStyleChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onPortStyleChanged(ItemChangedEventArgs%3CIPort%2CIPortStyle%3E) Online Documentation}*/
onPortStyleChanged(evt: ItemChangedEventArgs<IPort, IPortStyle>): void;
/**
Raises the {@link GraphWrapperBase#addPortTagChangedListener PortTagChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23onPortTagChanged(ItemChangedEventArgs%3CIPort%2CObject%3E) Online Documentation}*/
onPortTagChanged(evt: ItemChangedEventArgs<IPort, any>): void;
/**
Removes event handlers for <code>graph</code> that have been added with {@link GraphWrapperBase#addEventHandlers}.
@param graph The graph for which event propagators should be removed
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23removeEventHandlers(IGraph) Online Documentation}*/
removeEventHandlers(graph: IGraph): void;
/**
Sets the lookup instance to use in {@link GraphWrapperBase#lookup}
@param lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23setLookup(ILookup) Online Documentation}*/
setLookup(lookup: ILookup | null): void;
/**
Gets the graph that has been wrapped by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphWrapperBase%23wrappedGraph Online Documentation}*/
wrappedGraph: IGraph | null;
static $class: Class;
private $_GraphWrapperBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphWrapperBase;
}
export interface GroupingNodePositionHandler extends YObject, IPositionHandler {
/**
@inheritdoc*/
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
/**
@inheritdoc*/
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
initializeDrag(context: IInputModeContext): void;
} /**
The {@link IPositionHandler} implementation used in the context of {@link IGraph}s where there is a
{@link IGraph#getChildren active hierarchy}.
This implementation delegates through to a simpler implementation that only deals with the geometric relocation of the
node. In addition to that it finds the {@link IReparentNodeHandler} from the {@link IInputModeContext} to
detect {@link IReparentNodeHandler#isReparentGesture a reparenting gesture} by the user. If such a gesture is recognized, the node can be moved freely until a {@link IReparentNodeHandler#isValidParent valid new parent node}
is found. This implementation then temporarily sets that node as a new visual parent and highlight the node. When the
gesture is finished, the {@link IReparentNodeHandler#reparent} method is called to perform the actual reparenting.
In order to customize this default behavior, subclasses of this type can be {@link NodeDecorator#positionHandlerDecorator decorated} to the
{@link INode}s in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingNodePositionHandler Online Documentation}*/
export class GroupingNodePositionHandler {
/**
Initializes a new instance of the {@link GroupingNodePositionHandler} class.
This implementation delegates the handling of the node's {@link INode#layout} changes to the provided <code>wrappedHandler</code>. If the given <code>wrappedHandler</code>
is <code>null</code> an implementation of {@link IPositionHandler} is queried from the {@link DefaultGraph#DEFAULT_NODE_LOOKUP default node lookup} to be used as
delegate.
@param node The node to reposition.
@param [wrappedHandler=null] The {@link IPositionHandler} to wrap. If omitted, this will be the default {@link IPositionHandler} for
nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingNodePositionHandler Online Documentation}*/
constructor(node: INode, wrappedHandler?: IPositionHandler | null);
/**
Sets a temporary visual parent for the node during reparenting.
This is used to show the grouping hierarchy level a node would end in after finishing the reparenting gesture. Override
this method with an empty implementation to not change the z-order of the dragged node during the gesture.
@param context The current {@link IInputModeContext}.
@param node The node being reparented.
@param parent The temporary parent for the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingNodePositionHandler%23setCurrentParent(IInputModeContext%2CINode%2CINode) Online Documentation}*/
setCurrentParent(context: IInputModeContext, node: INode, parent: INode | null): void;
/**
Gets or sets a value determining whether the layout of the parent group node are adjusted to always keep the moved node
inside.
Default is <code>true</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingNodePositionHandler%23adjustParentNodeLayout Online Documentation}*/
adjustParentNodeLayout: boolean;
/**
Gets or sets a value determining whether {@link IGraph#getChildren children} of the moved node are implicitly moved with it.
Default is <code>true</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingNodePositionHandler%23moveChildNodes Online Documentation}*/
moveChildNodes: boolean;
/**
Gets or sets a value determining whether {@link IReparentNodeHandler#reparent reparenting} the moved node is enabled.
Default is <code>true</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingNodePositionHandler%23reparentingEnabled Online Documentation}*/
reparentingEnabled: boolean;
/**
Gets a view of the location of the item.
Delegates to the wrapped {@link IPositionHandler}'s {@link IDragHandler#location location}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingNodePositionHandler%23location Online Documentation}*/
readonly location: IPoint;
static $class: Class;
private $_GroupingNodePositionHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GroupingNodePositionHandler;
}
/**
Policy to determine whether and how the {@link GraphModelManager} should nest the {@link ICanvasObject canvas objects} of nodes and edges
if the graph is grouped.
@see GraphModelManager#hierarchicNestingPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicNestingPolicy Online Documentation}*/
export enum HierarchicNestingPolicy {
/**
The {@link ICanvasObject canvas objects} of nodes and edges are not nested.
The {@link ICanvasObject canvas objects} of nodes are top-level in {@link GraphModelManager#nodeGroup} while the {@link ICanvasObject canvas objects} of edges are top-level in {@link GraphModelManager#edgeGroup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicNestingPolicy%23NONE Online Documentation}*/
NONE,
/**
The {@link ICanvasObject canvas objects} of nodes are nested according to their location in the node hierarchy.
All nodes are visualized in a more complicated nesting of dynamically created {@link ICanvasObjectGroup}s whose root
group is the {@link GraphModelManager#nodeGroup}. All edges are top-level in {@link GraphModelManager#edgeGroup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicNestingPolicy%23NODES Online Documentation}*/
NODES,
/**
The {@link ICanvasObject canvas objects} of nodes and edges are nested according to their location in the node hierarchy.
All nodes and edges are visualized in a more complicated nesting of dynamically created {@link ICanvasObjectGroup}s
whose root group is the {@link GraphModelManager#nodeGroup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicNestingPolicy%23NODES_AND_EDGES Online Documentation}*/
NODES_AND_EDGES,
/**
Only the {@link ICanvasObject canvas objects} of {@link IGraph#isGroupNode group nodes} are nested according to their depth in the node hierarchy.
All {@link IGraph#isGroupNode leaf nodes} are top-level in the {@link GraphModelManager#nodeGroup}.
All edges are top-level in {@link GraphModelManager#edgeGroup}.
Group nodes are combined by their hierarchical depth (i.e. the number of their ancestor nodes). For all nodes having the
same hierarchical depth, an {@link ICanvasObjectGroup} is added to the {@link GraphModelManager#groupNodeGroup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicNestingPolicy%23GROUP_NODES Online Documentation}*/
GROUP_NODES
}
/** Automatic conversion to {@link HierarchicNestingPolicy}. */
type HierarchicNestingPolicyStringValues = "none" | "nodes" | "nodes-and-edges" | "group-nodes";
export interface IArrow {
}
/**
* The interface for the visual representation of an arrow that is normally used to decorate the visual representation of
* an {@link IEdge}.
* Instances of this class are used by {@link IEdgeStyle} implementations.
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* The strings applying to this scheme are converted to {@link Arrow}s:
*
* <pre>
* '[color] [scale] type'
*
* color = CSS color strings
* scale = xx-small|x-small|small|medium|large|x-large|xx-large
* type = normal|none|default|simple|short|diamond|cross|circle|ball|triangle
* </pre>
*
* Valid color strings are the same as for {@link Color}. Square brackets mark optional declarations.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow Online Documentation}
*/
export abstract class IArrow {
/**
Creates an {@link Arrow} instance from the given arrow-like object by performing automatic type conversion.
@param arrowLike The object to convert to an {@link Arrow}.
@returns Either <code>arrowLike</code> if it is already an {@link Arrow} or a new instance, initialized to the values found in
<code>arrowLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23from(Arrow) Online Documentation}*/
static from(arrowLike: Arrow | ArrowConvertible): Arrow | null;
/**
Gets an {@link IBoundsProvider} implementation that can yield this arrow's bounds if painted at the given location
using the given direction for the given edge.
@param edge the edge this arrow belongs to
@param atSource whether this will be the source arrow
@param anchor the anchor point for the tip of the arrow
@param directionVector the direction the arrow is pointing in
@returns an implementation of the {@link IBoundsProvider} interface that can subsequently be used to query the bounds.
Clients will always call this method before using the implementation and may not cache the instance returned. This
allows for applying the flyweight design pattern to implementations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23getBoundsProvider(IEdge%2Cboolean%2CPoint%2CPoint) Online Documentation}*/
abstract getBoundsProvider(edge: IEdge, atSource: boolean, anchor: Point, directionVector: Point): IBoundsProvider;
/**
Gets an {@link IVisualCreator} implementation that will paint this arrow at the given location using the given
direction for the given edge.
@param edge the edge this arrow belongs to
@param atSource whether this will be the source arrow
@param anchor the anchor point for the tip of the arrow
@param direction the direction the arrow is pointing in
@returns an implementation of the {@link IVisualCreator} interface that can subsequently be used to perform the actual
painting. Clients will always call this method before using the implementation and may not cache the instance returned.
This allows for applying the flyweight design pattern to implementations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23getVisualCreator(IEdge%2Cboolean%2CPoint%2CPoint) Online Documentation}*/
abstract getVisualCreator(edge: IEdge, atSource: boolean, anchor: Point, direction: Point): IVisualCreator;
/**
Gets the length of the arrow (the distance from the arrow's tip to the position where the visual representation of the
edge's path should begin).
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23length Online Documentation}*/
abstract readonly length: number;
/**
Gets the cropping length associated with this instance.
This value is used by {@link IEdgeStyle} implementations to let the edge appear to end shortly before its actual
target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23cropLength Online Documentation}*/
abstract readonly cropLength: number;
/**
A singleton {@link IArrow} instance that paints no arrow at all.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23NONE Online Documentation}*/
static NONE: IArrow;
/**
A singleton {@link IArrow} instance that paints a simple black arrow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23SIMPLE Online Documentation}*/
static readonly SIMPLE: IArrow;
/**
A singleton {@link IArrow} instance that paints a default shaped black arrow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23DEFAULT Online Documentation}*/
static readonly DEFAULT: IArrow;
/**
A singleton {@link IArrow} instance that paints a short black arrow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23SHORT Online Documentation}*/
static readonly SHORT: IArrow;
/**
A singleton {@link IArrow} instance that paints a simple diamond-shaped black arrow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23DIAMOND Online Documentation}*/
static readonly DIAMOND: IArrow;
/**
A singleton {@link IArrow} instance that paints a simple cross-shaped black arrow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23CROSS Online Documentation}*/
static readonly CROSS: IArrow;
/**
A singleton {@link IArrow} instance that paints a simple circle-shaped black arrow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23CIRCLE Online Documentation}*/
static readonly CIRCLE: IArrow;
/**
A singleton {@link IArrow} instance that paints a simple triangle-shaped black arrow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IArrow%23TRIANGLE Online Documentation}*/
static readonly TRIANGLE: IArrow;
static $class: Class;
private $_IArrow: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IArrow;
}
/** Automatic conversion to {@link IArrow}. */
type IArrowConvertible = string;
export interface IBend extends IModelItem {
}
/**
* The interface used in an {@link IGraph} implementation to control the layout of {@link IEdge edges}.
* This interface provides read-only access to the properties of a bend. In order to modify the state of an instance use
* the various methods provided by the {@link IGraph} this instance belongs to. Bends are stored in {@link IListEnumerable}s, that can be
* obtained from the {@link IEdge} implementation that owns this bend. Like all items in an <code>IGraph</code>, this item supports
* the {@link ILookup#lookup} method that can be used to query additional aspects of the item.
*
* <br><b>Working with a bend in the graph</b>
* <pre>
* const graph = graphComponent.graph
* const node1 = graph.createNodeAt(new Point(0, 0))
* const node2 = graph.createNodeAt(new Point(100, 0))
* const edge = graph.createEdge(node1, node2)
*
* // add a bend to the edge
* const bend = graph.addBend(edge, new Point(50, 20))
*
* // the newly created bend is part of the graph
* console.log(graph.contains(bend)) // true
*
* // the bend belongs to its owner
* console.log(bend.owner === edge) // true
* console.log(edge.bends.size) // 1
* console.log(edge.bends.get(0) === bend) // true
*
* // removing the bend removes it from the graph
* graph.remove(bend)
* console.log(graph.contains(bend)) // false
* // and from its owner
* console.log(bend.owner === null) // true
* console.log(edge.bends.size) // 0
* </pre>
*
* <br><b>Bends cannot live without an owner in the graph</b>
* <pre>
* console.log(graph.contains(bend)) // true
* graph.remove(bend.owner)
* console.log(graph.contains(bend)) // false
* </pre>
*
* The graph model with all relevant types and their relationships is presented in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph The Graph Model}.
*
* Using the look-up mechanism is explained in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_concepts_lookup Service Locator Pattern: Lookup}.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IBend Online Documentation}
*/
export abstract class IBend {
/**
* Gets the edge this bend instance belongs to.
* This implies that <code>Owner.Bends</code> contains this instance.
*
* Note that for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, the owner can't be changed after creation and the bend has to be deleted and recreated with a new
* owner.
*
* <br><b>Relationship between bend and edge</b>
* <pre>
* // add a bend to the owner
* const bend = graph.addBend(edge, new Point(x, y))
*
* // the bend's Owner property is set to the owner
* console.log(bend.owner === edge) // true
* // the bend is in its owner's Bends collection
* console.log(edge.bends.includes(bend)) // true
* </pre>
*
* @see IGraph#addBend
* @see IEdge#bends
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IBend%23owner Online Documentation}
*/
abstract readonly owner: IEdge | null;
/**
* Gets the location of the bend in the world coordinate system.
* Provides a live view of the location which will be updated when the location changes. To keep the current state of the
* location, callers need to create a copy of it, e.g. by calling {@link IPoint#toPoint} on it.
*
* To change the location for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setBendLocation}
*
* <br><b>Setting the location of a bend</b>
* <pre>
* graph.setBendLocation(bend, new Point(newX, newY))
* </pre>
*
* <br><b>Getting the location of a bend</b>
* <pre>
* const location = bend.location
* </pre>
*
* @see IPoint#toPoint
* @see IGraph#setBendLocation
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IBend%23location Online Documentation}
*/
abstract readonly location: IPoint;
/**
Gets the index of the bend at its {@link IBend#owner}.
@see IGraph#addBend
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBend%23index Online Documentation}*/
readonly index: number;
static $class: Class;
private $_IBend: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IBend;
}
export interface IBendCreator {
}
/**
Callback interface for instances that can create bends for a given edge and graph.
Implementations of this interface may be found in the {@link ILookup#lookup} of {@link IEdge} instances. This
interface is used by {@link GraphEditorInputMode} to create new bends for an edge as soon as the user performs the
bend creation gesture.
@see GraphEditorInputMode#onCreateBendInputModeBendCreated
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendCreator Online Documentation}*/
export abstract class IBendCreator {
/**
Creates an implementation of the interface {@link IBendCreator} by using the given function as implementation for
its {@link IBendCreator#createBend} method.
@param createBend A function for {@link IBendCreator}'s single abstract method {@link IBendCreator#createBend}.
@returns An instance of the {@link IBendCreator} interface based on the given function.*/
static create(createBend: (context: IInputModeContext, graph: IGraph, edge: IEdge, location: Point) => number): IBendCreator;
/**
Creates a bend at the given graph for the given edge at the position supplied.
@param context The context for which the bend should be created.
@param graph The graph, the edge belongs to.
@param edge The edge.
@param location The preferred coordinates of the bend.
@returns The index of the bend in the edges' {@link IEdge#bends} or <code>-1</code> if no bend has been created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendCreator%23createBend(IInputModeContext%2CIGraph%2CIEdge%2CPoint) Online Documentation}*/
abstract createBend(context: IInputModeContext, graph: IGraph, edge: IEdge, location: Point): number;
/**
A static instance that does not create any bends.
<b>Warning:</b> This field is not available unless the module <b>view-editor</b> is loaded.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendCreator%23NO_BENDS Online Documentation}*/
static NO_BENDS: IBendCreator;
static $class: Class;
private $_IBendCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IBendCreator;
}
export interface IBendSelectionTester {
}
/**
Interface that can be used to find bends at specific locations in the canvas.
This interface is usually provided through the {@link ILookup#lookup} of {@link IEdge} implementations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendSelectionTester Online Documentation}*/
export abstract class IBendSelectionTester {
/**
Creates an implementation of the interface {@link IBendSelectionTester} from the given definition.
The given object must contain exactly the abstract members of the {@link IBendSelectionTester} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IBendSelectionTester} interface.
@returns An instance of the {@link IBendSelectionTester} interface based on the given definition.*/
static create(interfaceDefinition: {
getHitBend(context: IInputModeContext, location: Point): IBend;
getBendsInBox(context: IInputModeContext, rectangle: Rect): IEnumerable<IBend>;
}): IBendSelectionTester;
/**
Returns the bends within the given rectangle.
@param context The input mode context to use for querying the position.
@param rectangle A rectangle in world coordinates.
@returns The bends inside of <code>rectangle</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendSelectionTester%23getBendsInBox(IInputModeContext%2CRect) Online Documentation}*/
abstract getBendsInBox(context: IInputModeContext, rectangle: Rect): IEnumerable<IBend>;
/**
Returns the bend at the given world coordinate position.
@param context The input mode context to use for querying the position.
@param location A location in world coordinates.
@returns The bend at the position or <code>null</code> if there is no bend.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendSelectionTester%23getHitBend(IInputModeContext%2CPoint) Online Documentation}*/
abstract getHitBend(context: IInputModeContext, location: Point): IBend | null;
static $class: Class;
private $_IBendSelectionTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IBendSelectionTester;
}
export interface IContainsBendTester {
}
/**
An interface that checks for containment of an {@link IBend} in 2D coordinate space.
This interface is used to determine which bends of edges that connect child nodes are moved when the parent node is
moved. Therefore, special care has to be taken if you want to provide this interface in the <b>node lookup</b> for other purposes, as this
can also affect implicit bend movement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContainsBendTester Online Documentation}*/
export abstract class IContainsBendTester {
/**
Determines whether the provided <code>bend</code> is deemed to lie within the shape this test is implemented for.
This method is used to determine which bends of edges that connect child nodes are moved when the parent node is moved.
If an implementation of {@link IContainsBendTester} is present in the node's {@link ILookup#lookup}, this
method is called with the bend and the bend will be moved together with the parent node iff <code>true</code> is returned. If no
implementation is provided, only bends that lie inside the bounds of the parent node are moved.
@param context The input mode context to use.
@param bend The bend to test
@returns <code>true</code> iff the bend is contained within the shape.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContainsBendTester%23contains(IInputModeContext%2CIBend) Online Documentation}*/
abstract contains(context: IInputModeContext, bend: IBend): boolean;
static $class: Class;
private $_IContainsBendTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IContainsBendTester;
}
export interface IEdge extends IPortOwner, ILabelOwner, IModelItem {
}
/**
* The interface used to model edges in an {@link IGraph} implementation.
* This interface provides read-only access to the properties of an edge. In order to modify the state of an instance use
* the various methods provided by the {@link IGraph} this instance belongs to. An edge connects to two ports and may
* consist of a number of bends. Also it can have zero or more labels. This interface combines the functionality of {@link IPortOwner}
* to get access to the ports that edges can optionally have, {@link ILabelOwner} to get access to the labels, and,
* like all items in an <code>IGraph</code>, edges support the {@link ILookup#lookup} method inherited from the {@link IModelItem}
* interface that can be used to query additional aspects of each instance.
*
* <br><b>Working with an edge in the graph</b>
* <pre>
* const graph = graphComponent.graph
* const node1 = graph.createNodeAt(new Point(0, 0))
* const node2 = graph.createNodeAt(new Point(100, 0))
*
* // create an edge between the nodes
* const edge = graph.createEdge(
*   node1,
*   node2,
*   new PolylineEdgeStyle({ targetArrow: IArrow.DEFAULT })
* )
*
* // the newly created edge is part of the graph
* console.log(graph.contains(edge)) // true
* console.log(graph.edges.size) // 1
* console.log(graph.edges.get(0) === edge) // true
*
* // the edge is linked via source and target port
* // with its source and target nodes
* console.log(edge.sourcePort.owner === node1) // true
* console.log(edge.targetPort.owner === node2) // true
*
* // removing the edge removes it from the graph
* graph.remove(edge)
* console.log(graph.contains(edge)) // false
* console.log(graph.edges.size) // 0
* </pre>
*
* <br><b>Edges cannot live without source or target in graph</b>
* <pre>
* console.log(graph.contains(edge)) // true
* graph.remove(edge.sourcePort.owner)
* console.log(graph.contains(edge)) // false
* </pre>
*
* The graph model with all relevant types and their relationships is presented in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph The Graph Model}.
*
* Using the look-up mechanism is explained in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_concepts_lookup Service Locator Pattern: Lookup}.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge Online Documentation}
*/
export abstract class IEdge {
/**
Gets a snapshot of the points describing the path of an edge.
The path points of an edge are the {@link IPort#location location} of the source port, followed by the {@link IBend#location locations} of the bends, followed by the
location of the target port.
@param edge The edge whose path points are returned.
@returns A list of {@link Point points} which describe the current path of the given <code>edge</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23getPathPoints(IEdge) Online Documentation}*/
static getPathPoints(edge: IEdge): IListEnumerable<IPoint>;
/**
Gets the opposite port of an {@link IEdge}.
@param port The port that the {@link IEdge} is connected to.
@returns The opposite port.
@throws {Exception} <b>(ArgumentError)</b> If <code>port</code> is neither the source or target of the edge.
@see IEdge#sourcePort
@see IEdge#targetPort
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23opposite(IPort) Online Documentation}*/
opposite(port: IPort): IPort;
/**
Gets the opposite port owner of an {@link IEdge}.
@param owner The owner of the port that the {@link IEdge} is connected to.
@returns The owner of the opposite port.
@throws {Exception} <b>(ArgumentError)</b> If <code>owner</code> is neither the source or target of the edge.
@see IEdge#sourcePort
@see IEdge#targetPort
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23opposite(IPortOwner) Online Documentation}*/
opposite(owner: IPortOwner): IPortOwner;
/**
* Gets a collection of {@link IBend bends} that describe the geometry of this edge.
* This gives access to a read-only live view of the bends, i.e. the collection can change over time, as well as the bends
* contained in it. If a snapshot of the current state is needed, one needs to copy the collection.
*
* To modify the bend collection for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#addBend} and {@link IGraph#remove}.
*
* <br><b>Adding a bend to an edge.</b>
* <pre>
* const bend = graph.addBend(edge, new Point(x, y))
* </pre>
*
* <br><b>Inserting a bend at position 1.</b>
* <pre>
* const anotherBend = graph.addBend(edge, new Point(x, y), 1)
* </pre>
*
* <br><b>Adding multiple bends at once</b>
* <pre>
* graph.addBends(edge, [new Point(10, 10), new Point(20, 20), new Point(30, 30)])
* </pre>
*
* <br><b>Iterating over all bends of an edge.</b>
* <pre>
* for (const bend of edge.bends) {
*   // ...
* }
* </pre>
*
* <br><b>Removing a bend.</b>
* <pre>
* graph.remove(bend)
* </pre>
*
* <br><b>Removing all bends from an edge.</b>
* <pre>
* graph.clearBends(edge)
* console.log(edge.bends.size) // 0
* </pre>
*
* <br><b>Relationship between a bend and its edge</b>
* <pre>
* // add a bend to the owner
* const bend = graph.addBend(edge, new Point(x, y))
*
* // the bend's Owner property is set to the owner
* console.log(bend.owner === edge) // true
* // the bend is in its owner's Bends collection
* console.log(edge.bends.includes(bend)) // true
* </pre>
*
* @see IGraph#addBend
* @see IGraph#remove
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23bends Online Documentation}
*/
abstract readonly bends: IListEnumerable<IBend>;
/**
* Gets the source port instance this edge is connected to.
* Although the notion of source and target port is used for directed edges, it is still up to the client to decide whether
* the edge should be treated as such.
*
* To change the ports for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setEdgePorts}
*
* <br><b>Getting the source port of an edge</b>
* <pre>
* const sourcePort = edge.sourcePort
* </pre>
*
* <br><b>Getting the source node of an edge</b>
* <pre>
* const sourceNode = edge.sourceNode
* // is the same as
* const sameSourceNode = edge.sourcePort.owner
* </pre>
*
* <br><b>Setting source and target port of an edge</b>
* <pre>
* graph.setEdgePorts(edge, newSourcePort, newTargetPort)
* </pre>
*
* @see IEdge#sourceNode
* @see IGraph#setEdgePorts
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23sourcePort Online Documentation}
*/
abstract readonly sourcePort: IPort | null;
/**
* Gets the target port instance this edge is connected to.
* Although the notion of source and target port is used for directed edges, it is still up to the client to decide whether
* the edge should be treated as such.
*
* To change the ports for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setEdgePorts}
*
* <br><b>Getting the target port of an edge</b>
* <pre>
* const targetPort = edge.targetPort
* </pre>
*
* <br><b>Getting the target node of an edge</b>
* <pre>
* const targetNode = edge.targetNode
* // is the same as
* const sameTargetNode = edge.targetPort.owner
* </pre>
*
* <br><b>Setting source and target port of an edge</b>
* <pre>
* graph.setEdgePorts(edge, newSourcePort, newTargetPort)
* </pre>
*
* @see IEdge#targetNode
* @see IGraph#setEdgePorts
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23targetPort Online Documentation}
*/
abstract readonly targetPort: IPort | null;
/**
* Gets the style that is responsible for the visual representation of this edge in a {@link CanvasComponent}.
* Note that the style instance associated with an edge instance may be shared between multiple edge instances and that the
* modification of this style will result in a change of the appearance of all edges that are associated with the same
* style instance.
*
* To change the style for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setStyle}
*
* <br><b>Setting the style of an edge</b>
* <pre>
* graph.setStyle(edge, new PolylineEdgeStyle())
* </pre>
*
* <br><b>Obtaining the style of an edge</b>
* <pre>
* const style = edge.style
* </pre>
*
* @see IGraph#setStyle
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23style Online Documentation}
*/
abstract readonly style: IEdgeStyle;
/**
Gets the source node for the given edge or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23sourceNode Online Documentation}*/
readonly sourceNode: INode | null;
/**
Gets the target node for the given edge or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23targetNode Online Documentation}*/
readonly targetNode: INode | null;
/**
Determines whether an {@link IEdge edge} is a self-loop, that is, it starts and ends at the same port owner.
@see IEdge#sourceNode
@see IEdge#targetNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdge%23isSelfloop Online Documentation}*/
readonly isSelfloop: boolean;
static $class: Class;
private $_IEdge: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdge;
}
export interface IEdgeDefaults {
}
/**
* Interface used by {@link IGraph} to declare and obtain the defaults for edges and their labels and ports.
* Note that changing these defaults does <i>not</i> change properties of already created model items. Rather, only items created
* after the change are affected.
*
* <br><b>Setting defaults for edges</b>
* <pre>
* // the defaults for edges can be set on the IEdgeDefaults instance
* // which can be retrieved from the graph's EdgeDefaults property
* graph.edgeDefaults.style = new PolylineEdgeStyle({
*   stroke: Stroke.BLACK,
*   targetArrow: Arrow.DEFAULT
* })
*
* // the defaults for edge labels can be set on the ILabelDefaults instance
* // found at the Labels property of the edge defaults
* graph.edgeDefaults.labels.layoutParameter = new EdgeSegmentLabelModel().createDefaultParameter()
* graph.edgeDefaults.labels.style = new DefaultLabelStyle()
* </pre>
*
* @see IGraph#edgeDefaults
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeDefaults Online Documentation}
*/
export abstract class IEdgeDefaults {
/**
Factory method that returns a style instance for use with newly created edges.
Most implementations will yield either, a {@link ICloneable#clone clone} of or the {@link IEdgeDefaults#style} property, if {@link IEdgeDefaults#shareStyleInstance} is enabled, but they might use more complicated
logic, too.
@returns The style to use, which for most implementations is either a {@link ICloneable#clone clone} of or the {@link IEdgeDefaults#style} property, if {@link IEdgeDefaults#shareStyleInstance} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeDefaults%23getStyleInstance() Online Documentation}*/
abstract getStyleInstance(): IEdgeStyle;
/**
* Gets or sets the defaults for ports at edges.
* Note that these are not the defaults for the {@link IEdge#sourcePort}s or {@link IEdge#targetPort}s. Instead, these are the defaults for the ports in the edge's {@link IPortOwner#ports}
* collection to which other edges can connect.
*
* <pre>
* // don't remove node labels after their
* // last adjacent edge has been removed
* graph.edgeDefaults.ports.autoCleanUp = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeDefaults%23ports Online Documentation}
*/
abstract ports: IPortDefaults;
/**
* Gets or sets the defaults for labels at edges.
* <pre>
* graph.edgeDefaults.labels.layoutParameter = new EdgeSegmentLabelModel().createDefaultParameter()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeDefaults%23labels Online Documentation}
*/
abstract labels: ILabelDefaults;
/**
* Gets or sets the style to use for edges.
* Depending on the setting of {@link IEdgeDefaults#shareStyleInstance}, the {@link IEdgeDefaults#getStyleInstance} method should return a {@link ICloneable#clone clone} of this
* instance or the very same instance.
*
* <pre>
* graph.edgeDefaults.style = new PolylineEdgeStyle({
*   stroke: Stroke.BLACK,
*   targetArrow: Arrow.DEFAULT
* })
* </pre>
*
* @see IEdgeDefaults#shareStyleInstance
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeDefaults%23style Online Documentation}
*/
abstract style: IEdgeStyle;
/**
* Gets or sets a value indicating whether the {@link IEdgeDefaults#style} instance should be shared referentially or {@link ICloneable#clone cloned} upon a call to
* {@link IEdgeDefaults#getStyleInstance}.
* <br><b>Shared vs. cloned styles</b>
* <pre>
* const n1 = graph.createNodeAt(new Point(0, 100))
* const n2 = graph.createNodeAt(new Point(100, 0))
* const n3 = graph.createNodeAt(new Point(100, 50))
* const n4 = graph.createNodeAt(new Point(100, 100))
* const n5 = graph.createNodeAt(new Point(100, 150))
*
* const defaultStyle = new PolylineEdgeStyle()
* graph.edgeDefaults.style = defaultStyle
*
* // share style instances
* graph.edgeDefaults.shareStyleInstance = true
* const e1 = graph.createEdge(n1, n2)
* const e2 = graph.createEdge(n1, n3)
* // e1.style === defaultStyle; e2.style === defaultStyle; e1.style === e2.style;
* defaultStyle.stroke = Stroke.RED // e1 and e2 both turn red, too
*
* // clone style instances
* graph.edgeDefaults.shareStyleInstance = false
* const e3 = graph.createEdge(n1, n4)
* const e4 = graph.createEdge(n1, n5)
* // e3.style !== defaultStyle; e4.style !== defaultStyle; e3.style !== e4.style;
* defaultStyle.stroke = Stroke.YELLOW // e3 and e4 don't change their color
* </pre>
*
* @see IEdgeDefaults#getStyleInstance
* @see IEdgeDefaults#style
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeDefaults%23shareStyleInstance Online Documentation}
*/
abstract shareStyleInstance: boolean;
static $class: Class;
private $_IEdgeDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeDefaults;
}
export interface VoidEdgeStyle extends YObject, IEdgeStyle {
}
/**
A void implementation of an edge style that does nothing and behaves like an invisible style.
This class implements the singleton pattern. Use the shared static {@link VoidEdgeStyle#INSTANCE} of this class
instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link IEdge#style} property of an
{@link IEdge}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyle Online Documentation}*/
export class VoidEdgeStyle {
/**
Returns <code>this</code>.
@returns <code>this</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyle%23clone() Online Documentation}*/
clone(): this;
/**
The {@link VoidEdgeStyle} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyle%23INSTANCE Online Documentation}*/
static INSTANCE: VoidEdgeStyle;
/**
Gets the {@link VoidEdgeStyleRenderer#INSTANCE VoidEdgeStyleRenderer instance}.
@see IEdgeStyle#renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyle%23renderer Online Documentation}*/
readonly renderer: IEdgeStyleRenderer;
static $class: Class;
private $_VoidEdgeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidEdgeStyle;
}
export interface VoidEdgeStyleRenderer extends YObject, IEdgeStyleRenderer {
}
/**
A void implementation of an edge style renderer that does nothing.
This class implements the singleton pattern. Use the shared static {@link VoidEdgeStyleRenderer#INSTANCE} of this
class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link IEdgeStyle#renderer} property of an
{@link IEdgeStyle}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyleRenderer Online Documentation}*/
export class VoidEdgeStyleRenderer {
/**
Yields the {@link IBoundsProvider#EMPTY} that will return empty bounds.
@param edge
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyleRenderer%23getBoundsProvider(IEdge%2CIEdgeStyle) Online Documentation}*/
getBoundsProvider(edge: IEdge, style: IEdgeStyle): IBoundsProvider;
/**
Yields a lookup that will only return {@link LassoTestables#NEVER} if an {@link ILassoTestable} is queried.
@param edge
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyleRenderer%23getContext(IEdge%2CIEdgeStyle) Online Documentation}*/
getContext(edge: IEdge, style: IEdgeStyle): ILookup;
/**
Yields the {@link IHitTestable#NEVER} that will always report misses.
@param edge
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyleRenderer%23getHitTestable(IEdge%2CIEdgeStyle) Online Documentation}*/
getHitTestable(edge: IEdge, style: IEdgeStyle): IHitTestable;
/**
Yields the {@link IMarqueeTestable#NEVER} that will always report misses.
@param edge
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyleRenderer%23getMarqueeTestable(IEdge%2CIEdgeStyle) Online Documentation}*/
getMarqueeTestable(edge: IEdge, style: IEdgeStyle): IMarqueeTestable;
/**
This method always returns the {@link VoidPathGeometry#INSTANCE VoidPathGeometry instance}.
@param edge The edge to provide an instance for
@param style The style to use for the rendering
@returns The {@link VoidPathGeometry#INSTANCE VoidPathGeometry instance}.
@see IEdgeStyleRenderer#getPathGeometry
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyleRenderer%23getPathGeometry(IEdge%2CIEdgeStyle) Online Documentation}*/
getPathGeometry(edge: IEdge, style: IEdgeStyle): IPathGeometry;
/**
Yields the {@link IVisibilityTestable#NEVER} that will always claim invisibility.
@param edge
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyleRenderer%23getVisibilityTestable(IEdge%2CIEdgeStyle) Online Documentation}*/
getVisibilityTestable(edge: IEdge, style: IEdgeStyle): IVisibilityTestable;
/**
Yields the {@link VoidVisualCreator#INSTANCE} that will do nothing.
@param edge
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyleRenderer%23getVisualCreator(IEdge%2CIEdgeStyle) Online Documentation}*/
getVisualCreator(edge: IEdge, style: IEdgeStyle): IVisualCreator;
/**
The {@link VoidEdgeStyleRenderer} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidEdgeStyleRenderer%23INSTANCE Online Documentation}*/
static INSTANCE: IEdgeStyleRenderer;
static $class: Class;
private $_VoidEdgeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidEdgeStyleRenderer;
}
export interface VoidPathGeometry extends YObject, IPathGeometry {
}
/**
A void implementation of the {@link IPathGeometry} interface that does nothing and behaves like an empty path.
This class implements the singleton pattern. Use the shared static {@link VoidPathGeometry#INSTANCE} of this class
instead of <code>null</code> were <code>null</code> is not allowed. The {@link VoidEdgeStyleRenderer} does so for example.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPathGeometry Online Documentation}*/
export class VoidPathGeometry {
/**
The {@link VoidPathGeometry} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPathGeometry%23INSTANCE Online Documentation}*/
static INSTANCE: IPathGeometry;
static $class: Class;
private $_VoidPathGeometry: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidPathGeometry;
}
export interface IEdgeStyle extends ICloneable {
}
/**
Style implementation for {@link IEdge} instances in an {@link IGraph}.
Implementations are responsible for rendering {@link IEdge} instances in a {@link CanvasComponent}. It is up to
the implementation to interpret the visual appearance of an edge. The framework uses the associated, possibly shared {@link IEdgeStyle#renderer} to
perform the actual rendering of this style for a given edge. This interface extends the {@link ICloneable}
interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
implementations may return themselves.
Working with styles (and their style renderers) is explained in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/view_visualization-graph-elements Visualization of Graph Elements}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyle Online Documentation}*/
export abstract class IEdgeStyle {
/**
* Gets the renderer implementation that can be queried for implementations that provide details about the visual
* appearance and visual behavior for a given edge and this style instance.
* The idiom for retrieving, e.g. an {@link IVisualCreator} implementation for a given style is:
*
* <pre>
* const creator = style.renderer.getVisualCreator(edge, style)
* const visual = creator.createVisual(context)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyle%23renderer Online Documentation}
*/
abstract readonly renderer: IEdgeStyleRenderer;
static $class: Class;
private $_IEdgeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeStyle;
}
export interface IEdgeStyleRenderer {
}
/**
Interface used by {@link IEdgeStyle} implementations that can be used to query the actual
{@link IVisualCreator}, {@link IVisibilityTestable}, {@link IBoundsProvider}, {@link IHitTestable}, and
{@link IMarqueeTestable} implementations for an edge and an associated style.
Implementations of this interface should be designed to be sharable between multiple style instances and should
therefore not contain style specific state.
@see IEdgeStyle
@see IEdge
@see INodeStyleRenderer
@see ILabelStyleRenderer
@see IPortStyleRenderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyleRenderer Online Documentation}*/
export abstract class IEdgeStyleRenderer {
/**
Gets an implementation of the {@link IBoundsProvider} interface that can handle the provided edge and its associated
style.
This method may return a flyweight implementation.
@param edge The edge to provide an instance for
@param style The style to use for the calculating the painting bounds
@returns An implementation that may be used to subsequently query the edge's painting bounds. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyleRenderer%23getBoundsProvider(IEdge%2CIEdgeStyle) Online Documentation}*/
abstract getBoundsProvider(edge: IEdge, style: IEdgeStyle): IBoundsProvider;
/**
Gets a temporary context instance that can be used to query additional information for the edge's style.
Implementations may return {@link ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
@param edge The edge to provide a context instance for.
@param style The style to use for the context.
@returns An non-<code>null</code> lookup implementation.
@see ILookup#EMPTY
@see ILookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyleRenderer%23getContext(IEdge%2CIEdgeStyle) Online Documentation}*/
abstract getContext(edge: IEdge, style: IEdgeStyle): ILookup;
/**
Gets an implementation of the {@link IHitTestable} interface that can handle the provided edge and its associated
style.
This method may return a flyweight implementation.
@param edge The edge to provide an instance for
@param style The style to use for the querying hit tests
@returns An implementation that may be used to subsequently perform hit tests. Clients should not cache this instance and must
always call this method immediately before using the value returned. This enables the use of the flyweight design
pattern for implementations. This method may return <code>null</code> to indicate that the edge cannot be hit tested.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyleRenderer%23getHitTestable(IEdge%2CIEdgeStyle) Online Documentation}*/
abstract getHitTestable(edge: IEdge, style: IEdgeStyle): IHitTestable;
/**
Gets an implementation of the {@link IMarqueeTestable} interface that can handle the provided edge and its
associated style.
This method may return a flyweight implementation.
@param edge The edge to provide an instance for
@param style The style to use for the querying marquee intersection test.
@returns An implementation that may be used to subsequently query the marquee intersections. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyleRenderer%23getMarqueeTestable(IEdge%2CIEdgeStyle) Online Documentation}*/
abstract getMarqueeTestable(edge: IEdge, style: IEdgeStyle): IMarqueeTestable;
/**
Gets an implementation of the {@link IPathGeometry} interface that can handle the provided edge and its associated
style.
This method may return a flyweight implementation.
@param edge The edge to provide an instance for
@param style The style to use for the rendering
@returns An implementation that may be used to subsequently query geometry information from. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyleRenderer%23getPathGeometry(IEdge%2CIEdgeStyle) Online Documentation}*/
abstract getPathGeometry(edge: IEdge, style: IEdgeStyle): IPathGeometry;
/**
Gets an implementation of the {@link IVisibilityTestable} interface that can handle the provided edge and its
associated style.
This method may return a flyweight implementation.
@param edge The edge to provide an instance for
@param style The style to use for the testing the visibility
@returns An implementation that may be used to subsequently query the edge's visibility. Clients should not cache this instance
and must always call this method immediately before using the value returned. This enables the use of the flyweight
design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyleRenderer%23getVisibilityTestable(IEdge%2CIEdgeStyle) Online Documentation}*/
abstract getVisibilityTestable(edge: IEdge, style: IEdgeStyle): IVisibilityTestable;
/**
Gets an implementation of the {@link IVisualCreator} interface that can handle the provided edge and its associated
style.
This method may return a flyweight implementation, but never <code>null</code>.
@param edge The edge to provide an instance for
@param style The style to use for the creation of the visual
@returns An implementation that may be used to subsequently create or update the visual for the edge. Clients should not cache
this instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link VoidVisualCreator#INSTANCE void} implementation
instead.
@see VoidVisualCreator#INSTANCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeStyleRenderer%23getVisualCreator(IEdge%2CIEdgeStyle) Online Documentation}*/
abstract getVisualCreator(edge: IEdge, style: IEdgeStyle): IVisualCreator;
static $class: Class;
private $_IEdgeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeStyleRenderer;
}
export interface IFoldingView extends ILookup, IPropertyObservable {
}
/**
* Interface that is used for manipulating a {@link FoldingManager#createFoldingView managed view} of an {@link IGraph} that supports folding operations like {@link IFoldingView#expand expanding}
* and {@link IFoldingView#collapse collapsing} of nodes.
* Folding is an extension to the grouping concept. It adds the functionality of {@link IFoldingView#collapse collapsing} (closing) and {@link IFoldingView#expand expanding}
* (opening) group nodes. Collapsing a group node means that its content is hidden and the group node itself becomes a
* normal (i.e. non-group or leaf) node.
*
* In yFiles folding works with two graph instances: a {@link FoldingManager#masterGraph master graph} which is is the complete unfolded graph and the so
* called {@link IFoldingView#graph view graph} which supports collapsing and expanding. This view graph is a modified and managed copy of the master
* graph. To make sure that the state of both graphs is correctly synchronized, yFiles provides the types {@link FoldingManager}
* and the {@link IFoldingView}.
*
* The {@link IFoldingView} stores the view graph and manages the mapping from elements of the master graph to
* elements in the view graph and vice versa. Typically nodes and edges are being copied from the {@link FoldingManager#masterGraph} and they share many of
* their properties by reference (e.g. {@link INode#style}). However if group nodes are being represented by {@link IFoldingView#collapse collapsed} group nodes in this
* view, they maintain a separate set of {@link ILabel labels}, {@link IPort ports}, {@link IBend bends}, and {@link INode#layout geometry}. This view-local state is shared between all
* views that have been {@link FoldingManager#createFoldingView created} by the same Manager and can be obtained and modified via the {@link FoldingManager}'s
* methods {@link FoldingManager#getFoldingEdgeState} for folding edges and {@link FoldingManager#getFolderNodeState}
* for folder nodes in the absence of a corresponding {@link IFoldingView} view instance.
*
* Developers should never implement this interface themselves. Rather, an implementation is provided by the framework and
* can be obtained from the {@link FoldingManager}'s method {@link FoldingManager#createFoldingView}. This
* implementation provides a folding-enabled {@link IGraph} implementation in its {@link IFoldingView#graph} property:
*
* <br><b>Setting up folding</b>
* <pre>
* // first create the manager
* const manager = new FoldingManager()
* // the manager serves as factory to create views
* const foldingView = manager.createFoldingView()
* // each view contains a folding-enabled graph: the view graph
* const viewGraph = foldingView.graph
* // the view graph is the graph which should actually be displayed
* graphComponent.graph = viewGraph
* </pre>
*
* For folding enabled graphs {@link IGraph#foldingView} always returns the {@link IFoldingView} implementation which manages the graph:
*
* <br><b>Obtaining an IFoldingView from an IGraph</b>
* <pre>
* const foldingView = viewGraph.foldingView
* </pre>
*
* This interface extends the {@link IPropertyObservable} interface. This can be used to easily monitor the {@link IFoldingView#invalid validity} of
* this view. Also changes to the {@link IFoldingView#localRoot} property will be published via this event.
*
* A brief description of the interface's API and some usage aspects is given in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/folding-working_with_folding Working with Folding}. Class
* <code>FoldingManager</code>'s central role for folding support in general is discussed in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/folding-technical_background Folding Revisited}.
*
* @see FoldingManager
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView Online Documentation}
*/
export abstract class IFoldingView {
/**
* This method will collapse the {@link IGraph#getChildren children} of a group node into the group node.
* If a {@link IGraph#isGroupNode group} node that belongs to this view's hierarchy is in {@link IFoldingView#expand expanded} state, calling this method will remove the {@link IGraph#getChildren children}
* from this {@link IFoldingView#graph view}. This method will do nothing if the node is already collapsed. Note that collapsing a group node will
* make it a {@link IGraph#isGroupNode non-group} node in the hierarchy.
*
* The collapsed group node has its own separate set of attributes which may be different from the expanded node. To be
* precise, the attributes of a collapsed node, i.e. its {@link INode#layout} and {@link INode#style} (as well as its labels and ports) can be obtained using the
* {@link FoldingManager#getFolderNodeState} method. In contrast, the expanded group node is an exact copy of its
* {@link IFoldingView#getMasterItem master node}.
*
* Because of these differences it may appear that the node changes its location during a collapse operation. This behavior
* ensures that collapsing a node does not modify the state of the model (only the state in the view is changed), but
* depending on the application this may confuse the user because he might loose his mental map of the diagram. In order to
* customize the behavior custom code can register with the {@link IFoldingView#addGroupCollapsedListener GroupCollapsed}
* event to perform the necessary modifications, like adjusting the view port or moving the node to the desired location.
*
* In addition to that this method can also be used for {@link FoldingManager#masterGraph master} nodes which are currently not represented in this view. In
* this case these nodes will be displayed as collapsed the next time they will be included in this view.
*
* Collapsing a group node interactively is handled by {@link NavigationInputMode} and by command
* {@link ICommand#COLLAPSE_GROUP}.
*
* <b>Warning:</b> When a group node is {@link IFoldingView#collapse collapsed}, all its child nodes are removed from the {@link IFoldingView#graph view graph} while the {@link FoldingManager#masterGraph master graph} doesn't
* change. When the group node is {@link IFoldingView#expand expanded} again, new child nodes are added to the view graph so any references to the
* 'old' child nodes are invalid.
*
* <pre>
* // get the folding view to manage expanding and collapsing
* const view = graph.foldingView
*
* // group is initially expanded
* view.isExpanded(group) // true
*
* // collapse group
* view.collapse(group)
* view.isExpanded(group) // false
* console.log(graph.isGroupNode(group)) // false
* console.log(graph.nodes.size) // 3 (node4, node5, group)
* console.log(graph.edges.size) // 2 (group -> node4, node5 -> group)
*
* // expand group again
* view.expand(group)
* view.isExpanded(group) // true
* console.log(graph.isGroupNode(group)) // true
* console.log(graph.nodes.size) // 6 (all nodes)
* console.log(graph.edges.size) // 5 (all edges)
* </pre>
*
* @param groupNode A group node that is part of the {@link IFoldingView#graph} or the {@link FoldingManager#masterGraph master graph}.
* @see IFoldingView#isExpanded
* @see IFoldingView#expand
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23collapse(INode) Online Documentation}
*/
abstract collapse(groupNode: INode): void;
/**
* Directly creates a collapsed node on this instance with the given parameters.
* This will implicitly create a new empty group node on the {@link FoldingManager#masterGraph} with the defaults for such a group node. The <code>parent</code> must be
* part of the {@link IFoldingView#graph Folded Graph View} or <code>null</code> to create a folder at the root of the current hierarchy. It will be converted into
* a group node if necessary. The returned {@link INode} is also part of the {@link IFoldingView#graph Folded Graph View}.
*
* <br><b>Creating group and folder nodes</b>
* <pre>
* const graph = view.graph
*
* // CreateGroupNode creates a group node
* const group = graph.createGroupNode()
* console.log(graph.isGroupNode(group)) // true
*
* // CreateFolderNode creates a normal node
* const folder = view.createFolderNode()
* console.log(graph.isGroupNode(folder)) // false
* // which can expand though
* view.expand(folder)
* console.log(graph.isGroupNode(folder)) // true
* </pre>
*
* {@link IGraph#createGroupNode} on the view graph creates a group node which is initially expanded. {@link IFoldingView#createFolderNode}
* creates a node which is an initially collapsed group node (i.e. a folder node)
*
* <pre>
* const graph = view.graph
*
* // CreateFolderNode creates a normal node on the view graph
* const folder = view.createFolderNode()
* console.log(graph.isGroupNode(folder)) // false
*
* // but its master element is a group on the master graph
* const master = view.getMasterItem(folder)
* const masterGraph = view.manager.masterGraph
* console.log(masterGraph.isGroupNode(master)) // true
* </pre>
*
* The folder node is a group node on the master graph, though
*
* @param [parent=null] The parent node in the {@link IFoldingView#graph}, or <code>null</code>
* @param [layout=null] The layout for the collapsed node.
* @param [style=null] The style for the collapsed node
* @param [tag=null] The tag of the node.
* @returns A collapsed (folder) node that lives in {@link IFoldingView#graph}
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23createFolderNode(INode%2CRect%2CINodeStyle%2CObject) Online Documentation}
*/
abstract createFolderNode(parent?: INode | null, layout?: Rect | null | RectConvertible, style?: INodeStyle, tag?: any | null): INode;
/**
* Directly creates a collapsed node on this instance with the given parameters.
* This will implicitly create a new empty group node on the {@link FoldingManager#masterGraph} with the defaults for such a group node. The <code>parent</code> must be
* part of the {@link IFoldingView#graph Folded Graph View} or <code>null</code> to create a folder at the root of the current hierarchy. It will be converted into
* a group node if necessary. The returned {@link INode} is also part of the {@link IFoldingView#graph Folded Graph View}.
*
* <br><b>Creating group and folder nodes</b>
* <pre>
* const graph = view.graph
*
* // CreateGroupNode creates a group node
* const group = graph.createGroupNode()
* console.log(graph.isGroupNode(group)) // true
*
* // CreateFolderNode creates a normal node
* const folder = view.createFolderNode()
* console.log(graph.isGroupNode(folder)) // false
* // which can expand though
* view.expand(folder)
* console.log(graph.isGroupNode(folder)) // true
* </pre>
*
* {@link IGraph#createGroupNode} on the view graph creates a group node which is initially expanded. {@link IFoldingView#createFolderNode}
* creates a node which is an initially collapsed group node (i.e. a folder node)
*
* <pre>
* const graph = view.graph
*
* // CreateFolderNode creates a normal node on the view graph
* const folder = view.createFolderNode()
* console.log(graph.isGroupNode(folder)) // false
*
* // but its master element is a group on the master graph
* const master = view.getMasterItem(folder)
* const masterGraph = view.manager.masterGraph
* console.log(masterGraph.isGroupNode(master)) // true
* </pre>
*
* The folder node is a group node on the master graph, though
*
* @param options The parameters to pass.
* @param [options.parent=null] The parent node in the {@link IFoldingView#graph}, or <code>null</code>
* @param [options.layout=null] The layout for the collapsed node.
* @param [options.style=null] The style for the collapsed node
* @param [options.tag=null] The tag of the node.
* @returns A collapsed (folder) node that lives in {@link IFoldingView#graph}
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23createFolderNode(INode%2CRect%2CINodeStyle%2CObject) Online Documentation}
*/
abstract createFolderNode(options: {
parent?: INode | null;
layout?: Rect | null | RectConvertible;
style?: INodeStyle;
tag?: any | null;
}): INode;
/**
Disposes of this view explicitly.
Normally it is not necessary to do so, because the view will be garbage collected automatically. However if the view
still displays contents, this might have unwanted negative side effects (group node sizes being calculated, etc.) A
disposed of instance is not synchronized with the {@link FoldingManager} anymore and should not be used anymore.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23dispose() Online Documentation}*/
abstract dispose(): void;
/**
* This method will expand a previously {@link IFoldingView#collapse collapsed} group node to make its contents appear in this view.
* If a node that belongs to this view is in {@link IFoldingView#collapse collapsed} state, calling this method will make the {@link IGraph#getChildren children} appear in this
* {@link IFoldingView#graph view}. This method will do nothing if the node is already expanded. Note that expanding a collapsed group node will make
* the node a
* {@link IGraph#isGroupNode group node}, whereas a collapsed group node is no group node in this hierarchy and thus cannot have visible children.
*
* The expanded group node has its own set of attributes which may be different from the collapsed node. To be precise, the
* expanded group node is an exact copy of its {@link IFoldingView#getMasterItem master node}, i.e. it has the same {@link INode#layout} and {@link INode#style} (as well as the same set of labels
* and ports). In contrast, a collapsed node has a separate set of attributes which can be obtained using the {@link FoldingManager#getFolderNodeState}
* method.
*
* Because of these differences it may appear that the node changes its location during an expand operation. This behavior
* ensures that expanding a node does not modify the state of the model by moving the node or its descendants (only the
* state in the view is changed), but depending on the application this may confuse the user because he might loose his
* mental map of the diagram. In order to customize the behavior custom code can register with the {@link IFoldingView#addGroupExpandedListener GroupExpanded}
* event to perform the necessary modifications, like adjusting the view port or moving the node and its descendants to the
* desired location.
*
* In addition to that this method can also be used for {@link FoldingManager#masterGraph master} nodes which are currently not represented in this view. In
* this case these nodes will be displayed as expanded the next time they will be included in this view.
*
* Expanding a group node interactively is handled by {@link NavigationInputMode} and by command
* {@link ICommand#EXPAND_GROUP}.
*
* <b>Warning:</b> When a group node is {@link IFoldingView#collapse collapsed}, all its child nodes are removed from the {@link IFoldingView#graph view graph} while the {@link FoldingManager#masterGraph master graph} doesn't
* change. When the group node is {@link IFoldingView#expand expanded} again, new child nodes are added to the view graph so any references to the
* 'old' child nodes are invalid.
*
* <pre>
* // get the folding view to manage expanding and collapsing
* const view = graph.foldingView
*
* // group is initially expanded
* view.isExpanded(group) // true
*
* // collapse group
* view.collapse(group)
* view.isExpanded(group) // false
* console.log(graph.isGroupNode(group)) // false
* console.log(graph.nodes.size) // 3 (node4, node5, group)
* console.log(graph.edges.size) // 2 (group -> node4, node5 -> group)
*
* // expand group again
* view.expand(group)
* view.isExpanded(group) // true
* console.log(graph.isGroupNode(group)) // true
* console.log(graph.nodes.size) // 6 (all nodes)
* console.log(graph.edges.size) // 5 (all edges)
* </pre>
*
* @param groupNode A group node that is part of the {@link IFoldingView#graph} or the {@link FoldingManager#masterGraph master graph}.
* @see IFoldingView#isExpanded
* @see IFoldingView#collapse
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23expand(INode) Online Documentation}
*/
abstract expand(groupNode: INode): void;
/**
Helper method that corresponds to the {@link IFoldingView#getMasterItem} method, but is used for folding edges,
only.
Since the {@link IFoldingView#graph} view that is associated with this instance maintains a copy of the entities in the Manager's {@link FoldingManager#masterGraph}, there is a
mapping between elements that belong to this view's {@link IFoldingView#graph} and the elements in the master graph.
Since folding edges can represent more than one master edge, this method can be used to query all of the master edges
that the provided folding edge represents. If the folding edge corresponds to a single master edge, the result will
enumerate that edge, only.
@param foldingEdge The folding edge for which the master edges should be returned.
@returns An enumerable that can enumerate all of the edges in the {@link FoldingManager#masterGraph} that are being represented by the folding edge in this {@link IFoldingView#graph view}.
@see IFoldingEdgeFactory#addToExistingFoldingEdge
@see IFoldingView#getViewItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23getMasterEdges(IEdge) Online Documentation}*/
abstract getMasterEdges(foldingEdge: IEdge): IEnumerable<IEdge>;
/**
Helper method that can be used to retrieve the original "master" items in the {@link FoldingManager#masterGraph} that is managed by the Manager associated
with this view.
Since the {@link IFoldingView#graph} view that is associated with this instance maintains a copy of the entities in the Manager's {@link FoldingManager#masterGraph}, there is a
mapping between elements that belong to this view's {@link IFoldingView#graph} and the elements in the master graph.
This method can be called for the elements contained in this {@link IGraph} and will return the corresponding item
in the master graph that the element corresponds to, if any. Note that for some items in the {@link IFoldingView#graph}, there is no master item
in the {@link FoldingManager#masterGraph}, e.g. the {@link IPort} instances to which {@link FoldingEdgeStateId folding edges} connect have no corresponding ports in the master
graph. The same holds true for the {@link IBend bends} and {@link ILabel labels} of folding edges and folder nodes.
For folding edges, this method will yield the main representing edge in the master, if the folding edge represents more
than one master edge. For nodes, the master node will be returned, no matter whether the node is currently {@link IFoldingView#collapse collapsed} or
not.
Type parameter <code>T</code>: The type of the argument, which is typically {@link INode}, {@link IEdge}, {@link ILabel},
{@link IBend}, or {@link IPort}.
@param item The item that is part of this {@link IFoldingView#graph} for which the original "master" item in the {@link FoldingManager#masterGraph} will be returned.
@returns An item in the {@link FoldingManager#masterGraph} that corresponds to the <code>item</code> or <code>null</code>, if there is no such item.
@see IFoldingView#getViewItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23getMasterItem(T) Online Documentation}*/
abstract getMasterItem<T extends IModelItem>(item: T): T | null;
/**
Helper method that can be used to retrieve the items in this {@link IFoldingView#graph}-view for all items that are part of the {@link FoldingManager#masterGraph} that is
associated with the Manager of this instance.
Since the {@link IFoldingView#graph} view that is associated with this instance maintains a copy of the entities in the Manager's {@link FoldingManager#masterGraph}, there is a
mapping between elements that belong to master graph and the elements in the {@link IFoldingView#graph} of this view.
This method can be called for the elements contained in this {@link FoldingManager#masterGraph master graph's} items and will return the corresponding item
in this view's {@link IFoldingView#graph} if the element is represented by an item in this view. Note that for some items in the {@link FoldingManager#masterGraph}, there may be no
item in the {@link IFoldingView#graph} instance, e.g. those elements which are part of a {@link IFoldingView#collapse collapsed} subtree in the {@link FoldingManager#masterGraph} hierarchy or those items that
do not belong to the subtree that is induced by the {@link IFoldingView#localRoot} of this view. Also, for nodes and edges that are being represented
by dummies in this view, labels, ports, and bends are not being represented directly by corresponding entities in this
graph. If more than one edge is represented by a folding edge in this view, this method will yield the same folding edge
instance for each of them.
Type parameter <code>T</code>: The type of the argument, which is typically {@link INode}, {@link IEdge}, {@link ILabel},
{@link IBend}, or {@link IPort}.
@param item An item that is part of the {@link FoldingManager#masterGraph} that is associated with the Manager of this instance.
@returns An item in the {@link IFoldingView#graph local graph view} that corresponds to the <code>item</code> or
<code>null</code>, if the item is not currently being represented in this view.
@see IFoldingView#getMasterItem
@see IFoldingView#getMasterEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23getViewItem(T) Online Documentation}*/
abstract getViewItem<T extends IModelItem>(item: T): T | null;
/**
* Determines whether the provided node is currently expanded or will be displayed in
* {@link IFoldingView#expand}ed state the next time it will be included in this view.
* Expanded group nodes will have all of their {@link IGraph#getChildren children} visible in this {@link IFoldingView#graph view}.
*
* The <code>groupNode</code> can be either a node of the {@link IFoldingView#graph view} or the {@link FoldingManager#masterGraph master graph}. If the node is from the master graph and is
* currently not represented in this view, this method will return the
* {@link IFoldingView#expand}ed state it will have the next time it will be included in this view.
*
* <pre>
* // get the folding view to manage expanding and collapsing
* const view = graph.foldingView
*
* // group is initially expanded
* view.isExpanded(group) // true
*
* // collapse group
* view.collapse(group)
* view.isExpanded(group) // false
* console.log(graph.isGroupNode(group)) // false
* console.log(graph.nodes.size) // 3 (node4, node5, group)
* console.log(graph.edges.size) // 2 (group -> node4, node5 -> group)
*
* // expand group again
* view.expand(group)
* view.isExpanded(group) // true
* console.log(graph.isGroupNode(group)) // true
* console.log(graph.nodes.size) // 6 (all nodes)
* console.log(graph.edges.size) // 5 (all edges)
* </pre>
*
* @param groupNode The group node for which the {@link IFoldingView#expand expanded} state will be returned.
* @returns <code>true</code>, if the group node can be {@link IFoldingView#collapse collapsed}.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23isExpanded(INode) Online Documentation}
*/
abstract isExpanded(groupNode: INode): boolean;
/**
Determines whether the specified item is a folding state in this view.
This method can be used to determine if an {@link INode}, {@link IEdge}, {@link IPort},
{@link ILabel}, or {@link IBend} has a {@link FoldingManager#getFolderNodeState view local state} and is not a direct copy of the {@link IFoldingView#getMasterItem master item}, if any.
Note that {@link IFoldingView#collapse collapsed} group nodes are in folding state, even though they do have a {@link IFoldingView#getMasterItem master} item in the master graph and
folding edges always have at least one
{@link IFoldingView#getMasterEdges master edge}, while all other folding items don't have a corresponding master item in the master graph.
@param item One of the items that are {@link IGraph#contains contained} in the {@link IFoldingView#graph}.
@returns Whether the item is in folding state and is not an exact copy of an item in the {@link FoldingManager#masterGraph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23isInFoldingState(IModelItem) Online Documentation}*/
abstract isInFoldingState(item: IModelItem): boolean;
/**
Adds the given listener for the <code>GroupCollapsed</code> event that occurs whenever a group has been
{@link IFoldingView#collapse}d.
@param listener The listener to add.
@see IFoldingView#removeGroupCollapsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23GroupCollapsed Online Documentation}*/
abstract addGroupCollapsedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupCollapsed</code> event that occurs whenever a group has been
{@link IFoldingView#collapse}d.
@param listener The listener to remove.
@see IFoldingView#addGroupCollapsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23GroupCollapsed Online Documentation}*/
abstract removeGroupCollapsedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>GroupExpanded</code> event that occurs whenever a group has been
{@link IFoldingView#expand}ed.
@param listener The listener to add.
@see IFoldingView#removeGroupExpandedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23GroupExpanded Online Documentation}*/
abstract addGroupExpandedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>GroupExpanded</code> event that occurs whenever a group has been
{@link IFoldingView#expand}ed.
@param listener The listener to remove.
@see IFoldingView#addGroupExpandedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23GroupExpanded Online Documentation}*/
abstract removeGroupExpandedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
* Gets the view graph managed by this view.
* The view graph is the {@link IGraph} implementation that provides the actual view data. The view graph instance
* holds the elements in the view only, which in general is a subset of the {@link FoldingManager#masterGraph}.
*
* The returned {@link IGraph} instance has this implementation of {@link IFoldingView} in its
* {@link ILookup}.
*
* <b>Warning:</b> Note that the elements are not the same references as their
* {@link IFoldingView#getMasterItem masters}, they are copies that may share the same attributes.
*
* This example shows how to create a folding view and set it's graph (the view graph) as the {@link GraphComponent}'s
* {@link GraphComponent#graph}
*
* <pre>
* // first create the manager
* const manager = new FoldingManager()
* // the manager serves as factory to create views
* const foldingView = manager.createFoldingView()
* // each view contains a folding-enabled graph: the view graph
* const viewGraph = foldingView.graph
* // the view graph is the graph which should actually be displayed
* graphComponent.graph = viewGraph
* </pre>
*
* The view graph provides the {@link IFoldingView} instance is is managed by in its {@link ILookup}:
*
* <pre>
* const graph = view.graph
* console.log(graph.foldingView === view) // true
* // is the same as
* console.log(graph.lookup(IFoldingView.$class) === view) // true
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23graph Online Documentation}
*/
abstract readonly graph: IGraph;
/**
Gets or sets whether this instance should automatically switch the {@link IFoldingView#localRoot} to an ancestor of the current root, if this view
becomes {@link IFoldingView#invalid}.
By default this property is set to <code>true</code> which assures that this view will never become
{@link IFoldingView#invalid}. It is the responsibility of the client code to check for the validity of this view before trying to modify it if this
feature is disabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23autoSwitchToAncestor Online Documentation}*/
abstract autoSwitchToAncestor: boolean;
/**
Gets whether this view is currently valid and can be edited.
A view is invalid if the {@link IFoldingView#localRoot} is not currently part of the {@link FoldingManager#masterGraph} hierarchy. This can happen if the root node is removed from the
{@link FoldingManager#masterGraph}. Note that the view can become valid again if the removal is undone.
@see IFoldingView#autoSwitchToAncestor
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23invalid Online Documentation}*/
abstract readonly invalid: boolean;
/**
Gets or sets whether changes to the {@link IFoldingView#localRoot} property and calls to {@link IFoldingView#collapse} and {@link IFoldingView#expand}
should be recorded in undo history.
By default this property is set to <code>false</code>, however depending on the use case it may make sense to enqueue
{@link IUndoUnit}s for this type of operations.
@see IFoldingView#autoSwitchToAncestor
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23enqueueNavigationalUndoUnits Online Documentation}*/
abstract enqueueNavigationalUndoUnits: boolean;
/**
* Gets or sets an entity of the {@link FoldingManager#masterGraph} hierarchy to serve as the virtual root for the view's local hierarchy.
* This property can be set to any {@link IGraph#isGroupNode group} node in the
* {@link FoldingManager#masterGraph}'s hierarchy whose contents will then be represented in this view. Note that the root of the view's local hierarchy is
* not set to the same instance.
*
* If set to <code>null</code> the root of the local hierarchy is the overall root of the master graph.
*
* <pre>
* // get the folding view to manage expanding and collapsing
* const view = graph.foldingView
*
* // show only the contents of groupNode
* // as if they were in a separate graph
* view.localRoot = group
*
* graph.contains(group) // false
* console.log(graph.nodes.size) // 3 (node1, node2, node3)
* console.log(graph.edges.size) // 3 (1 -> 2, 2 -> 3, 3 -> 1)
*
* // show the entire graph
* // i.e. the root of the master graph is the root of this view graph
* view.localRoot = null
*
* graph.contains(group) // true
* console.log(graph.nodes.size) // 6 (all nodes)
* console.log(graph.edges.size) // 5 (all edges)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23localRoot Online Documentation}
*/
abstract localRoot: INode | null;
/**
* Gets the manager that created this view and that contains a reference to the {@link FoldingManager#masterGraph}.
* <b>Warning:</b> This property is not available unless the module <b>view-folding</b> is loaded.
*
* <pre>
* // get the folding view from the view graph
* const view = viewGraph.foldingView
* // get the manager from the folding view
* const manager = view.manager
* // get the master graph from the manager
* const masterGraph = manager.masterGraph
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingView%23manager Online Documentation}
*/
readonly manager: FoldingManager;
static $class: Class;
private $_IFoldingView: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IFoldingView;
}
/**
Enumeration type for the different types of adjacency.
Directed edges in a graph can be either incoming, outgoing, or both.
@see IGraph#edgesAt
@see IGraph#edgesAt
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyTypes Online Documentation}*/
export enum AdjacencyTypes {
/**
Neither {@link AdjacencyTypes#INCOMING} nor {@link AdjacencyTypes#OUTGOING}
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyTypes%23NONE Online Documentation}*/
NONE,
/**
The constant for incoming edges.
An edge that is incoming at its owner has the owner as its target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyTypes%23INCOMING Online Documentation}*/
INCOMING,
/**
The constant for outgoing edges.
An edge that is outgoing from its owner has the owner as its source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyTypes%23OUTGOING Online Documentation}*/
OUTGOING,
/**
Both {@link AdjacencyTypes#INCOMING} and {@link AdjacencyTypes#OUTGOING}
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyTypes%23ALL Online Documentation}*/
ALL
}
type AdjacencyTypesStringValues = "none" | "incoming" | "outgoing" | "all";
/** Automatic conversion to {@link AdjacencyTypes}. */
type AdjacencyTypesConvertible = AdjacencyTypesStringValues | AdjacencyTypesStringValues[];
/**
An enumeration for use in various properties that can take a bitwise combination of item types that make up an
{@link IGraph}
The enumeration can be used to create a bitwise combination of the types.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes Online Documentation}*/
export enum GraphItemTypes {
/**
Constant for nodes.
@see INode
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23NODE Online Documentation}*/
NODE,
/**
Constant for edges.
@see IEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23EDGE Online Documentation}*/
EDGE,
/**
Constant for node labels.
@see ILabel
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23NODE_LABEL Online Documentation}*/
NODE_LABEL,
/**
Constant for edge labels.
@see ILabel
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23EDGE_LABEL Online Documentation}*/
EDGE_LABEL,
/**
Constant for ports.
@see IPort
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23PORT Online Documentation}*/
PORT,
/**
Constant for port labels.
@see ILabel
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23PORT_LABEL Online Documentation}*/
PORT_LABEL,
/**
Constant for all kinds of labels.
This is a bitwise combination of {@link GraphItemTypes#NODE_LABEL} {@link GraphItemTypes#EDGE_LABEL}, and
{@link GraphItemTypes#PORT_LABEL}.
@see ILabel
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23LABEL Online Documentation}*/
LABEL,
/**
Constant for all kinds of {@link ILabelOwner}.
This is a bitwise combination of {@link GraphItemTypes#NODE}, {@link GraphItemTypes#EDGE}, and
{@link GraphItemTypes#PORT}.
@see ILabelOwner
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23LABEL_OWNER Online Documentation}*/
LABEL_OWNER,
/**
Constant for edge bends.
@see IBend
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23BEND Online Documentation}*/
BEND,
/**
Constant for all item types.
This is a bitwise combination of all other types declared in this enum.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23ALL Online Documentation}*/
ALL,
/**
Constant for no item type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23NONE Online Documentation}*/
NONE
}
export namespace GraphItemTypes {
/**
Gets the type of the item.
@param item The item.
@returns The type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23getItemType(IModelItem) Online Documentation}*/
function getItemType(item: IModelItem | null): GraphItemTypes;
/**
Determines whether the <code>item</code> is of the specified <code>types</code>.
@param item The item.
@param types The type.
@returns <code>true</code> if the <code>item</code> is of the specified <code>types</code>; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23itemIsOfTypes(GraphItemTypes%2CIModelItem) Online Documentation}*/
function itemIsOfTypes(types: GraphItemTypes | GraphItemTypesConvertible, item: IModelItem | null): boolean;
/**
Determines whether the specified items collection contains any of the types described by <code>types</code>.
@param items The items.
@param types The types.
@returns <code>true</code> if the specified items contains any of the types; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23enumerableContainsTypes(GraphItemTypes%2CIEnumerable%3CIModelItem%3E) Online Documentation}*/
function enumerableContainsTypes(types: GraphItemTypes | GraphItemTypesConvertible, items: IEnumerable<IModelItem> | IEnumerableConvertible<IModelItem>): boolean;
/**
Determines whether the specified items collection contains none of the types described by <code>types</code>.
@param items The items.
@param types The types.
@returns <code>true</code> if the specified items contains none of the types; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23enumerableNotContainsTypes(GraphItemTypes%2CIEnumerable%3CIModelItem%3E) Online Documentation}*/
function enumerableNotContainsTypes(types: GraphItemTypes | GraphItemTypesConvertible, items: IEnumerable<IModelItem> | IEnumerableConvertible<IModelItem>): boolean;
/**
Determines the types of the items contained in the collection of items.
@param items The items.
@returns The types of the items.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphItemTypes%23getItemTypes(IEnumerable%3CIModelItem%3E) Online Documentation}*/
function getItemTypes(items: IEnumerable<IModelItem> | IEnumerableConvertible<IModelItem>): GraphItemTypes;
}
type GraphItemTypesStringValues = "node" | "edge" | "node-label" | "edge-label" | "port" | "port-label" | "label" | "label-owner" | "bend" | "all" | "none";
/** Automatic conversion to {@link GraphItemTypes}. */
type GraphItemTypesConvertible = GraphItemTypesStringValues | GraphItemTypesStringValues[];
export interface IGraph extends ILookup, ITagOwner {
}
/**
* Central interface that models a graph which can be displayed in a {@link CanvasComponent canvas} or {@link GraphComponent}.
* This interface can be used to query structural information, it also offers methods that change the structure of the
* graph and its attributes. Furthermore, a number of events will be triggered if the structure of the graph changes.
*
* The graph is made up of collections of {@link INode nodes} and {@link IEdge edges}. Each node and edge can be associated with a number of {@link ILabel labels} and
* possibly {@link IPort ports} to which edges connect. An edge connects to two ports and may consist of zero or more {@link IBend bends}. The graph is
* treated as a directed graph, i.e. edges have a {@link IEdge#sourcePort source port} and a
* {@link IEdge#targetPort target port}. It is up to the algorithms and the visualization to possibly treat them as undirected.
*
* This interface also provides support for hierarchically organized, i.e. grouped graphs. This means that nodes together
* with their connecting edges can be {@link IGraph#setParent put into} other nodes. The containing node is referred to as a "group node." To create
* and edit group nodes interactively use the {@link GraphEditorInputMode} as input mode and enable the
* {@link GraphEditorInputMode#allowGroupingOperations grouping operations}.
*
* The {@link IGraph#edgesAt} and {@link IGraph#edgesAt} methods can be used to query adjacency information from
* the graph's structure.
*
* Each element in the graph can be associated with a style that is used for the visualization of the element. Styles can
* be shared between multiple instances.
*
* To associate data with the elements in the graph, code can make use of the {@link ITagOwner#tag} property that all {@link IModelItem}s
* provide. In order to associate more than one data element with the graph items, compound data objects can be used.
* Alternatively the {@link IMapperRegistry} that can be obtained from the {@link IGraph#mapperRegistry} property can be used to associate
* arbitrary data sets with the items in this graph.
*
* This interface provides a number of events that can be used to get notified for changes in the graph structure. These
* events are raised whenever the corresponding state change occurs, e.g. also when loading the graph from a file. If you
* are only interested in changes that are triggered interactively, you should subscribe to the corresponding events on the
* {@link IInputMode} implementations that implement the user interaction. Especially, you may not modify the graph
* structure in handlers for these event, e.g. try to prevent or undo the change that has raised the event.
*
* New elements can be created either using defaults or with explicitly provided styles:
*
* <br><b>Creating graph elements</b>
* <pre>
* const graph = graphComponent.graph
*
* // set defaults for newly created elements
*
* // appearance of nodes
* graph.nodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.RECTANGLE,
*   fill: Fill.ORANGE
* })
* // placement and appearance of node labels
* graph.nodeDefaults.labels.layoutParameter = InteriorLabelModel.CENTER
* graph.nodeDefaults.labels.style = new DefaultLabelStyle({
*   backgroundStroke: Stroke.BLACK,
*   backgroundFill: Fill.WHITE
* })
*
* // create some nodes and edges
* const node1 = graph.createNode()
* const node2 = graph.createNode(new Rect(100, 0, 60, 40), new PanelNodeStyle(), 'some tag')
* const edge1 = graph.createEdge(
*   node1,
*   node2,
*   new PolylineEdgeStyle({ targetArrow: Arrow.DEFAULT })
* )
* </pre>
*
* Existing elements can be changed using methods provided by {@link IGraph}:
*
* <br><b>Changing the graph</b>
* <pre>
* let index = 0
* for (const node of graph.nodes) {
*   if (node.labels.size === 0) {
*     // if the node has no label, yet, add one
*     graph.addLabel(node, `New label for node ${index}`)
*   } else {
*     // otherwise change the first label's text
*     graph.setLabelText(node.labels.get(0), `Change text for node ${index}`)
*   }
*   // set another style for the node
*   graph.setStyle(
*     node,
*     new ShapeNodeStyle({
*       shape: ShapeNodeShape.RECTANGLE,
*       fill: Fill.GREEN
*     })
*   )
*   index++
* }
* </pre>
*
* The graph can be traversed along the edges with the help of a number of methods which provide adjacency information from
* the graph's structure.
*
* <br><b>Analyzing the graph structure</b>
* <pre>
* for (const node of graph.nodes) {
*   const nodeName = node.labels.get(0).text
*   // no predecessors: the node is a root
*   if (!graph.predecessors(node).some()) {
*     console.log(`Node ${nodeName} is a root.`)
*   }
*   // no successor: the node is a leaf
*   if (!graph.successors(node).some()) {
*     console.log(`Node ${nodeName} is a leaf.`)
*   }
*   // list all nodes which are linked to the current node
*   for (const edge of graph.edgesAt(node)) {
*     const otherNode = edge.opposite(node)
*     console.log(`Node ${nodeName} is linked to ${otherNode.labels.get(0).text}.`)
*   }
* }
* </pre>
*
* Finally, graph items can also be {@link IGraph#remove removed}.
*
* <br><b>Removing graph elements and clearing the graph</b>
* <pre>
* // IMPORTANT: graph.nodes will throw exception if it is changed during iteration
* // therefore we have to iterate over a copy of the node collection
* // while we remove the nodes
* const nodesToRemove = graph.nodes.toList()
* for (const node of nodesToRemove) {
*   graph.remove(node)
* }
*
* // clear the existing graph
* graph.clear()
* </pre>
*
* {@link IGraph} also provides support for hierarchically organized, i.e. grouped graphs. This means that nodes
* together with their connecting edges can be {@link IGraph#setParent put into} other nodes.
*
* <br><b>Grouping features overview</b>
* <pre>
* const graph = graphComponent.graph
*
* // set the defaults for a newly created group node
* graph.groupNodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.ROUND_RECTANGLE,
*   stroke: Stroke.AQUAMARINE,
*   fill: null
* })
* // create a group node
* const groupNode = graph.createGroupNode()
* // add existing nodes as children
*
* graph.groupNodes(groupNode, [node1, node2, node3])
*
* // whether the node is a group node
* console.log(graph.isGroupNode(groupNode)) // true
* console.log(graph.isGroupNode(node1)) // false
*
* // get the parent of a node
* console.log(graph.getParent(node1)) // groupNode
* console.log(graph.getParent(groupNode)) // null
*
* // get the children of a group node
* const children = graph.getChildren(groupNode) // node1, node2, node3
* console.log(children.size) // 3
* </pre>
*
* The graph model with all relevant types and their relationships is presented in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph The Graph Model}.
*
* More information on visual styles can be found in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/view_visualization-graph-elements Visualization of Graph Elements}.
*
* @see DefaultGraph
* @see INode
* @see IEdge
* @see IPort
* @see ILabel
* @see IBend
* @see IMapperRegistry
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph Online Documentation}
*/
export abstract class IGraph {
/**
* Adds a bend at the given index to the given edge using the coordinates provided.
* The added instance will be returned.
*
* To retrieve the current bends of an edge, use {@link IEdge#bends}
*
* Bends can be added either as last bend or at a given index
*
* <br><b>Adding a bend to an edge.</b>
* <pre>
* const bend = graph.addBend(edge, new Point(x, y))
* </pre>
*
* <br><b>Inserting a bend at position 1.</b>
* <pre>
* const anotherBend = graph.addBend(edge, new Point(x, y), 1)
* </pre>
*
* To add multiple points as bends at once another method, {@link IGraph#addBends}, can be used instead:
*
* <br><b>Adding multiple bends at once</b>
* <pre>
* graph.addBends(edge, [new Point(10, 10), new Point(20, 20), new Point(30, 30)])
* </pre>
*
* @param edge The edge to which the bend will be added.
* @param location the coordinates to use for the newly created bend. To change the location after the bend has been added, use
* {@link IGraph#setBendLocation}.
* @param [index=-1] The index for the newly added bend; a negative value (which is the default) indicates that the bend should be appended
* to the end of the list of bends.
* @returns a newly created live bend
* @throws {Exception} <b>(ArgumentError)</b> <code>edge</code> is not in this graph, or <code>location</code> contains one or more NaN values.
* @see IGraph#remove
* @see IGraph#setBendLocation
* @see IEdge#bends
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addBend(IEdge%2CPoint%2Cnumber) Online Documentation}
*/
abstract addBend(edge: IEdge, location: Point | PointConvertible, index?: number): IBend;
/**
Adds bends with the given locations to the end of the bend list of the given edge.
@param edge The edge to add the bends to.
@param locations The locations of the bends.
@throws {Exception} <b>(ArgumentError)</b> <code>edge</code> is not in this graph.
@throws {Exception} <b>(ArgumentError)</b> <code>locations</code> contains one or more NaN values.
@see IGraph#addBend
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addBends(IEdge%2CIEnumerable%3CPoint%3E) Online Documentation}*/
addBends(edge: IEdge, locations: IEnumerable<Point> | IEnumerableConvertible<Point>): void;
/**
* Add a label to the given node or edge using the text as the initial label text and label model parameter, style and
* tag.
* <h2>Note</h2>
*
* To add labels to {@link IStripe}s in {@link ITable}s, you have to use the {@link ITable}'s {@link ITable#addLabel}
* method instead. Even though stripes are {@link ILabelOwner}s, they don't actually belong to the graph.
*
* <pre>
* // add label with given text, default style, and default placement
* // and determine the preferred size automatically
*
* const label1 = graph.addLabel(node, 'Some Label')
*
* // add label with given text, placement, style, size, and tag (user object)
* const label2 = graph.addLabel(
*   node,
*   'Some Label',
*   InteriorLabelModel.CENTER,
*   new DefaultLabelStyle(),
*   new Size(10, 150),
*   userObject
* )
*
* // add label with given text and style but default placement
* // and determine the preferred size automatically
* const label3 = graph.addLabel({
*   owner: node,
*   text: 'Some Label',
*   layoutParameter: InteriorLabelModel.CENTER,
*   style: new DefaultLabelStyle()
* })
* </pre>
*
* In most cases it is recommended to let yFiles determine the preferred size automatically:
*
* <pre>
* // add label with given text, placement, style, and tag (user object)
* // however, determine the size automatically (recommended in  most cases)
*
* const label4 = graph.addLabel({
*   owner: node,
*   text: 'Some Label',
*   layoutParameter: InteriorLabelModel.CENTER,
*   style: new DefaultLabelStyle(),
*   tag: userObject
* })
* </pre>
*
* @param owner The node, edge or port to add the label to. Note that the owner cannot be changed after the label has been added.
* @param text The initial text of the label. To change the text after the label has been added, use {@link IGraph#setLabelText}.
* @param [layoutParameter=null] The label model parameter instance to use. If omitted the {@link ILabelDefaults default parameter} will be set. To change the parameter after
* the label has been added, use {@link IGraph#setLabelLayoutParameter}.
* @param [style=null] The style to use for the label. If omitted the {@link ILabelDefaults default style} will be set. To change the style after the label has been
* added, use {@link IGraph#setStyle}.
* @param [preferredSize=null] The initial values to use for the {@link ILabel#preferredSize}. If omitted size will be determined automatically. To change the preferred size after
* the label has been added, use {@link IGraph#setLabelPreferredSize}.
* @param [tag=null] The initial {@link ITagOwner#tag} to assign.
* @returns the newly created label
* @throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph or <code>preferredSize</code> contains one or more NaN values.
* @see IGraph#addLabelAddedListener
* @see IGraph#setLabelText
* @see ILabel#text
* @see IGraph#setLabelLayoutParameter
* @see ILabel#layoutParameter
* @see IGraph#setStyle
* @see ILabel#style
* @see IGraph#setLabelPreferredSize
* @see ILabel#preferredSize
* @see ILabel#owner
* @see VoidLabelStyle#INSTANCE
* @see INodeDefaults#labels
* @see IEdgeDefaults#labels
* @see IPortDefaults#labels
* @see ILabelOwner#labels
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addLabel(ILabelOwner%2Cstring%2CILabelModelParameter%2CILabelStyle%2CSize%2CObject) Online Documentation}
*/
abstract addLabel(owner: ILabelOwner, text: string, layoutParameter?: ILabelModelParameter | null, style?: ILabelStyle | null, preferredSize?: Size | null | SizeConvertible, tag?: any | null): ILabel;
/**
* Add a label to the given node or edge using the text as the initial label text and label model parameter, style and
* tag.
* <h2>Note</h2>
*
* To add labels to {@link IStripe}s in {@link ITable}s, you have to use the {@link ITable}'s {@link ITable#addLabel}
* method instead. Even though stripes are {@link ILabelOwner}s, they don't actually belong to the graph.
*
* <pre>
* // add label with given text, default style, and default placement
* // and determine the preferred size automatically
*
* const label1 = graph.addLabel(node, 'Some Label')
*
* // add label with given text, placement, style, size, and tag (user object)
* const label2 = graph.addLabel(
*   node,
*   'Some Label',
*   InteriorLabelModel.CENTER,
*   new DefaultLabelStyle(),
*   new Size(10, 150),
*   userObject
* )
*
* // add label with given text and style but default placement
* // and determine the preferred size automatically
* const label3 = graph.addLabel({
*   owner: node,
*   text: 'Some Label',
*   layoutParameter: InteriorLabelModel.CENTER,
*   style: new DefaultLabelStyle()
* })
* </pre>
*
* In most cases it is recommended to let yFiles determine the preferred size automatically:
*
* <pre>
* // add label with given text, placement, style, and tag (user object)
* // however, determine the size automatically (recommended in  most cases)
*
* const label4 = graph.addLabel({
*   owner: node,
*   text: 'Some Label',
*   layoutParameter: InteriorLabelModel.CENTER,
*   style: new DefaultLabelStyle(),
*   tag: userObject
* })
* </pre>
*
* @param options The parameters to pass.
* @param options.owner The node, edge or port to add the label to. Note that the owner cannot be changed after the label has been added.
* @param options.text The initial text of the label. To change the text after the label has been added, use {@link IGraph#setLabelText}.
* @param [options.layoutParameter=null] The label model parameter instance to use. If omitted the {@link ILabelDefaults default parameter} will be set. To change the parameter after
* the label has been added, use {@link IGraph#setLabelLayoutParameter}.
* @param [options.style=null] The style to use for the label. If omitted the {@link ILabelDefaults default style} will be set. To change the style after the label has been
* added, use {@link IGraph#setStyle}.
* @param [options.preferredSize=null] The initial values to use for the {@link ILabel#preferredSize}. If omitted size will be determined automatically. To change the preferred size after
* the label has been added, use {@link IGraph#setLabelPreferredSize}.
* @param [options.tag=null] The initial {@link ITagOwner#tag} to assign.
* @returns the newly created label
* @throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph or <code>preferredSize</code> contains one or more NaN values.
* @see IGraph#addLabelAddedListener
* @see IGraph#setLabelText
* @see ILabel#text
* @see IGraph#setLabelLayoutParameter
* @see ILabel#layoutParameter
* @see IGraph#setStyle
* @see ILabel#style
* @see IGraph#setLabelPreferredSize
* @see ILabel#preferredSize
* @see ILabel#owner
* @see VoidLabelStyle#INSTANCE
* @see INodeDefaults#labels
* @see IEdgeDefaults#labels
* @see IPortDefaults#labels
* @see ILabelOwner#labels
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addLabel(ILabelOwner%2Cstring%2CILabelModelParameter%2CILabelStyle%2CSize%2CObject) Online Documentation}
*/
abstract addLabel(options: {
owner: ILabelOwner;
text: string;
layoutParameter?: ILabelModelParameter | null;
style?: ILabelStyle | null;
preferredSize?: Size | null | SizeConvertible;
tag?: any | null;
}): ILabel;
/**
* Add a port to the given port owner using the location model parameter, style and tag.
* The <code>locationParameter</code> determines the location of the port.
*
* An implementation may throw an {@link Exception NotSupportedError} if the type of the <code>owner</code> instance does not support adding of ports.
*
* <br><b>Adding a port</b>
* <pre>
* // add a port at x,y with default style
* const port1 = graph.addPortAt(node, new Point(x, y))
*
* const portStyle = new NodeStylePortStyleAdapter()
*
* portStyle.nodeStyle = new ShapeNodeStyle({ shape: ShapeNodeShape.ELLIPSE })
* portStyle.renderSize = new Size(3, 3)
*
* // add a port at x,y with the given style and tag (user object)
* const port2 = graph.addPortAt(node, new Point(x, y), portStyle, userObject)
* </pre>
*
* The port's position is determined by the <code>locationParameter</code>. Method {@link IGraph#addPortAt} places the port at the
* given (absolute) location.
*
* <br><b>Adding a port at an (absolute) location</b>
* <pre>
* // add a port at x,y with default style
* const port1 = graph.addPortAt(node, new Point(x, y))
*
* const portStyle = new NodeStylePortStyleAdapter()
*
* portStyle.nodeStyle = new ShapeNodeStyle({ shape: ShapeNodeShape.ELLIPSE })
* portStyle.renderSize = new Size(3, 3)
*
* // add a port at x,y with the given style and tag (user object)
* const port2 = graph.addPortAt(node, new Point(x, y), portStyle, userObject)
* </pre>
*
* Method {@link IGraph#addRelativePort} adds the port at the given location relative to the given node's top left
* corner.
*
* <br><b>Adding a port at an relative location</b>
* <pre>
* // add a port at the center of the node with the given style and tag (user object)
* const port = graph.addRelativePort(
*   node,
*   new Point(node.layout.width / 2, node.layout.height / 2)
* )
* </pre>
*
* @param owner the owner to add the port instance to.
* @param [locationParameter=null] the parameter to use for the port to determine its location. If omitted the {@link IPortDefaults default parameter} will be set. To change the
* parameter after the port has been added, use {@link IGraph#setPortLocationParameter}.
* @param [style=null] the style to initially assign to the {@link IPort#style} property. If omitted the {@link IPortDefaults default style} will be set, e.g.
* {@link VoidPortStyle#INSTANCE}. To change the style after the port has been added, use
* {@link IGraph#setStyle}.
* @param [tag=null] the initial {@link ITagOwner#tag} to assign.
* @returns the newly created port
* @throws {Exception} <b>(NotSupportedError)</b> If this instance cannot add a port to <code>owner</code>.
* @throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
* @see IGraph#addPortAddedListener
* @see VoidPortStyle#INSTANCE
* @see IGraph#setPortLocationParameter
* @see IPort#locationParameter
* @see IGraph#setStyle
* @see IPort#style
* @see INodeDefaults#ports
* @see IEdgeDefaults#ports
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addPort(IPortOwner%2CIPortLocationModelParameter%2CIPortStyle%2CObject) Online Documentation}
*/
abstract addPort(owner: IPortOwner, locationParameter?: IPortLocationModelParameter | null, style?: IPortStyle | null, tag?: any | null): IPort;
/**
* Add a port to the given port owner using the location model parameter, style and tag.
* The <code>locationParameter</code> determines the location of the port.
*
* An implementation may throw an {@link Exception NotSupportedError} if the type of the <code>owner</code> instance does not support adding of ports.
*
* <br><b>Adding a port</b>
* <pre>
* // add a port at x,y with default style
* const port1 = graph.addPortAt(node, new Point(x, y))
*
* const portStyle = new NodeStylePortStyleAdapter()
*
* portStyle.nodeStyle = new ShapeNodeStyle({ shape: ShapeNodeShape.ELLIPSE })
* portStyle.renderSize = new Size(3, 3)
*
* // add a port at x,y with the given style and tag (user object)
* const port2 = graph.addPortAt(node, new Point(x, y), portStyle, userObject)
* </pre>
*
* The port's position is determined by the <code>locationParameter</code>. Method {@link IGraph#addPortAt} places the port at the
* given (absolute) location.
*
* <br><b>Adding a port at an (absolute) location</b>
* <pre>
* // add a port at x,y with default style
* const port1 = graph.addPortAt(node, new Point(x, y))
*
* const portStyle = new NodeStylePortStyleAdapter()
*
* portStyle.nodeStyle = new ShapeNodeStyle({ shape: ShapeNodeShape.ELLIPSE })
* portStyle.renderSize = new Size(3, 3)
*
* // add a port at x,y with the given style and tag (user object)
* const port2 = graph.addPortAt(node, new Point(x, y), portStyle, userObject)
* </pre>
*
* Method {@link IGraph#addRelativePort} adds the port at the given location relative to the given node's top left
* corner.
*
* <br><b>Adding a port at an relative location</b>
* <pre>
* // add a port at the center of the node with the given style and tag (user object)
* const port = graph.addRelativePort(
*   node,
*   new Point(node.layout.width / 2, node.layout.height / 2)
* )
* </pre>
*
* @param options The parameters to pass.
* @param options.owner the owner to add the port instance to.
* @param [options.locationParameter=null] the parameter to use for the port to determine its location. If omitted the {@link IPortDefaults default parameter} will be set. To change the
* parameter after the port has been added, use {@link IGraph#setPortLocationParameter}.
* @param [options.style=null] the style to initially assign to the {@link IPort#style} property. If omitted the {@link IPortDefaults default style} will be set, e.g.
* {@link VoidPortStyle#INSTANCE}. To change the style after the port has been added, use
* {@link IGraph#setStyle}.
* @param [options.tag=null] the initial {@link ITagOwner#tag} to assign.
* @returns the newly created port
* @throws {Exception} <b>(NotSupportedError)</b> If this instance cannot add a port to <code>owner</code>.
* @throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
* @see IGraph#addPortAddedListener
* @see VoidPortStyle#INSTANCE
* @see IGraph#setPortLocationParameter
* @see IPort#locationParameter
* @see IGraph#setStyle
* @see IPort#style
* @see INodeDefaults#ports
* @see IEdgeDefaults#ports
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addPort(IPortOwner%2CIPortLocationModelParameter%2CIPortStyle%2CObject) Online Documentation}
*/
abstract addPort(options: {
owner: IPortOwner;
locationParameter?: IPortLocationModelParameter | null;
style?: IPortStyle | null;
tag?: any | null;
}): IPort;
/**
Add a port to the given port owner using the absolute coordinates as the new initial position of the port anchor.
@param owner the owner to add the port instance to.
@param location the location to use for the port to determine its location. This is passed to the {@link IGraph#createDefaultPortLocationParameter}
method to determine the initial {@link IPortLocationModelParameter} to use.
@param [style=null] the style to initially assign to the {@link IPort#style} property, e.g. {@link VoidPortStyle#INSTANCE}.
@param [tag=null] the initial {@link ITagOwner#tag} to assign.
@returns the newly created port
@throws {Exception} <b>(NotSupportedError)</b> If this instance cannot add a port to <code>owner</code>.
@throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
@throws {Exception} <b>(ArgumentError)</b> <code>location</code> contains one or more NaN values.
@see IGraph#addPort
@see IGraph#setPortLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addPortAt(IPortOwner%2CPoint%2CIPortStyle%2CObject) Online Documentation}*/
addPortAt(owner: IPortOwner, location: Point | PointConvertible, style?: IPortStyle | null, tag?: any | null): IPort;
/**
Add a port to the given port owner using the absolute coordinates as the new initial position of the port anchor.
@param options The parameters to pass.
@param options.owner the owner to add the port instance to.
@param options.location the location to use for the port to determine its location. This is passed to the {@link IGraph#createDefaultPortLocationParameter}
method to determine the initial {@link IPortLocationModelParameter} to use.
@param [options.style=null] the style to initially assign to the {@link IPort#style} property, e.g. {@link VoidPortStyle#INSTANCE}.
@param [options.tag=null] the initial {@link ITagOwner#tag} to assign.
@returns the newly created port
@throws {Exception} <b>(NotSupportedError)</b> If this instance cannot add a port to <code>owner</code>.
@throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
@throws {Exception} <b>(ArgumentError)</b> <code>location</code> contains one or more NaN values.
@see IGraph#addPort
@see IGraph#setPortLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addPortAt(IPortOwner%2CPoint%2CIPortStyle%2CObject) Online Documentation}*/
addPortAt(options: {
owner: IPortOwner;
location: Point | PointConvertible;
style?: IPortStyle | null;
tag?: any | null;
}): IPort;
/**
Adds a new port to the graph at the node using a location that is relative to the center of the node.
The port style for the newly-created port is taken from the graph's defaults. The location parameter is determined by
delegating to {@link IGraph#addPortAt}.
@param node The owner of the port.
@param relativeLocation The offset of the port relative to the center of the {@link INode#layout}.
@returns The newly added port instance.
@throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph.
@throws {Exception} <b>(ArgumentError)</b> <code>relativeLocation</code> contains one or more NaN values.
@see IGraph#addPort
@see IGraph#addPortAt
@see IGraph#setRelativePortLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addRelativePort(INode%2CPoint) Online Documentation}*/
addRelativePort(node: INode, relativeLocation: Point | PointConvertible): IPort;
/**
Convenience method that uses the {@link UndoEngine} from the {@link IGraph}'s {@link ILookup} to
{@link UndoEngine#addUnit add a unit}.
@param undoName The name of the undo operation.
@param redoName The name of the redo operation.
@param undo The undo action.
@param redo The redo action.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23addUndoUnit(string%2Cstring%2Cfunction()%3Avoid%2Cfunction()%3Avoid) Online Documentation}*/
addUndoUnit(undoName: string, redoName: string, undo: () => void, redo: () => void): void;
/**
* Method to adjust the size of a group node.
* This will resize the group node bounds such that the node requires the least amount of space. If the node does not have
* any {@link IGraph#getChildren children} its bounds will be left unchanged. This will also respect any {@link INodeSizeConstraintProvider}s for
* {@link INode}s, that are available in the lookup of the <code>groupNode</code>.
*
* <br><b>How to create create a group node with children</b>
* <pre>
* // create a group node
* const group = graph.createGroupNode()
* // add some child nodes
* graph.createNode(group, new Rect(10, 10, 30, 30))
* graph.createNode(group, new Rect(50, 10, 30, 30))
* graph.createNode(group, new Rect(50, 50, 30, 30))
* // adjust the group node's layout to include all its children
* graph.adjustGroupNodeLayout(group)
* </pre>
*
* <code>AdjustGroupNodeLayout</code> only affects the node it is called for. To properly adjust all of its ancestors, too, one has to
* adjust all of them from the given node to the root:
*
* <pre>
* for (const nodeToAdjust of graph.groupingSupport.getPathToRoot(innermostGroup)) {
*   graph.adjustGroupNodeLayout(nodeToAdjust)
* }
* </pre>
*
* @param groupNode The group node to adjust the size of.
* @throws {Exception} <b>(ArgumentError)</b> <code>groupNode</code> is not in this graph.
* @see IGroupBoundsCalculator
* @see INodeSizeConstraintProvider
* @see GroupingSupport#calculateMinimumEnclosedArea
* @see GroupingSupport#enlargeGroupNode
* @see GroupingSupport#enlargeAllGroupNodes
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23adjustGroupNodeLayout(INode) Online Documentation}
*/
adjustGroupNodeLayout(groupNode: INode): void;
/**
Adjusts the {@link ILabel#preferredSize} property of a label to fit the suggested size of its {@link ILabelStyleRenderer}.
This implementation uses the {@link ILabel#style style's} renderer for the label to determine the preferred rendering size. This is useful
after the label's content or style have been changed.
@param label The label to adjust the size for.
@see ILabelDefaults#autoAdjustPreferredSize
@see IGraph#setLabelPreferredSize
@see ILabel#preferredSize
@see IGraph#calculateLabelPreferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23adjustLabelPreferredSize(ILabel) Online Documentation}*/
adjustLabelPreferredSize(label: ILabel): void;
/**
Runs an {@link ILayoutAlgorithm} synchronously on the given graph.
For more control over how to apply an layout use {@link LayoutExecutor}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@param layout The layout.
@param [layoutData=null] The layout data.
@param [updateContentRect=true] A value indicating whether the content rectangle property of the {@link CanvasComponent}
@param [automaticEdgeGrouping=true] A value indicating whether edge groups are automatically created for edges that are connected to the same port.
@param [configureTableLayout=true] A value indicating whether to automatically perform calls to {@link TableLayoutConfigurator#prepare} and {@link TableLayoutConfigurator#restore}
in order to layout table nodes.
@param [considerViewportLimiter=false] A value that controls whether strong port constraints are automatically created.
@param [fixPorts=false] A value that controls whether strong port constraints are automatically created.
@param [hideEdgesAtEdges=false] A value that controls whether edges at other edges will be hidden from the layout graph or included.
@param [portAdjustmentPolicy=PortAdjustmentPolicy.NEVER] The policy that specifies how port locations should be adjusted after a layout has been calculated.
@param [portLabelPolicies=null] Sets how {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm.
@param [targetBoundsInsets=null] the insets (in world coordinates) that will be added to the content rectangle when calculating the target viewport.
@see LayoutExecutor
@see GraphComponent#morphLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23applyLayout(ILayoutAlgorithm%2CLayoutData%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2CPortAdjustmentPolicy%2CItemMapping%3CILabel%2CPortLabelPolicy%3E%2CInsets) Online Documentation}*/
applyLayout(layout: ILayoutAlgorithm, layoutData?: LayoutData | null, updateContentRect?: boolean, automaticEdgeGrouping?: boolean, configureTableLayout?: boolean, considerViewportLimiter?: boolean, fixPorts?: boolean, hideEdgesAtEdges?: boolean, portAdjustmentPolicy?: PortAdjustmentPolicy | PortAdjustmentPolicyStringValues, portLabelPolicies?: ItemMapping<ILabel, PortLabelPolicy> | null | ItemMappingConvertible<ILabel, PortLabelPolicy>, targetBoundsInsets?: Insets | null | InsetsConvertible): void;
/**
Runs an {@link ILayoutAlgorithm} synchronously on the given graph.
For more control over how to apply an layout use {@link LayoutExecutor}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@param options The parameters to pass.
@param options.layout The layout.
@param [options.layoutData=null] The layout data.
@param [options.updateContentRect=true] A value indicating whether the content rectangle property of the {@link CanvasComponent}
@param [options.automaticEdgeGrouping=true] A value indicating whether edge groups are automatically created for edges that are connected to the same port.
@param [options.configureTableLayout=true] A value indicating whether to automatically perform calls to {@link TableLayoutConfigurator#prepare} and {@link TableLayoutConfigurator#restore}
in order to layout table nodes.
@param [options.considerViewportLimiter=false] A value that controls whether strong port constraints are automatically created.
@param [options.fixPorts=false] A value that controls whether strong port constraints are automatically created.
@param [options.hideEdgesAtEdges=false] A value that controls whether edges at other edges will be hidden from the layout graph or included.
@param [options.portAdjustmentPolicy=PortAdjustmentPolicy.NEVER] The policy that specifies how port locations should be adjusted after a layout has been calculated.
@param [options.portLabelPolicies=null] Sets how {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm.
@param [options.targetBoundsInsets=null] the insets (in world coordinates) that will be added to the content rectangle when calculating the target viewport.
@see LayoutExecutor
@see GraphComponent#morphLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23applyLayout(ILayoutAlgorithm%2CLayoutData%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2Cboolean%2CPortAdjustmentPolicy%2CItemMapping%3CILabel%2CPortLabelPolicy%3E%2CInsets) Online Documentation}*/
applyLayout(options: {
layout: ILayoutAlgorithm;
layoutData?: LayoutData | null;
updateContentRect?: boolean;
automaticEdgeGrouping?: boolean;
configureTableLayout?: boolean;
considerViewportLimiter?: boolean;
fixPorts?: boolean;
hideEdgesAtEdges?: boolean;
portAdjustmentPolicy?: PortAdjustmentPolicy;
portLabelPolicies?: ItemMapping<ILabel, PortLabelPolicy> | null | ItemMappingConvertible<ILabel, PortLabelPolicy>;
targetBoundsInsets?: Insets | null | InsetsConvertible;
}): void;
/**
* Starts an {@link ICompoundEdit} that uses the {@link IMementoSupport memento design pattern} to record changes to the items in the given <code>items</code>
* collection.
* This method uses the {@link IMementoSupport} returned by the <code>provider</code> to record the state of an item at the
* beginning of the edit and when {@link ICompoundEdit#commit} is called to create an {@link IUndoUnit} that can
* revert the item to the recorded state and back. If no <code>provider</code> is given, this method uses the {@link IMementoSupport}
* returned by the {@link ILookup#lookup} implementation of the items to record the state of an item at the beginning
* of the edit and when {@link ICompoundEdit#commit} is called to create an {@link IUndoUnit} that can revert the
* item to the recorded state and back.
*
* Calling this method will immediately enqueue an {@link IUndoUnit} into the undo queue. Subsequent additions to the
* queue will be added after the created instance, even if they are added to the queue before the {@link ICompoundEdit#commit}
* method has been called.
*
* The following is an example implementation of an item that is being managed using {@link IMementoSupport}:
*
* <br><b>A sample business object that implements ILookup to return IMementoSupport</b>
* <pre>
* class Employee extends BaseClass(ILookup) {
*   constructor(name, position, age) {
*     super()
*     this._name = name
*     this.position = position
*     this.age = age
*   }
*
*   get name() {
*     return this._name
*   }
*
*   lookup(type) {
*     if (type === IMementoSupport.$class) {
*       return new EmployeeMementoSupport()
*     }
*     return null
*   }
* }
* </pre>
*
* A collection of items from this type can then be watched using the following code snippet:
*
* <br><b>Using an ICompoundEdit</b>
* <pre>
* const edit = graph.beginEdit(undoName, redoName, listWithMyEmployeesToWatch)
*
* // changes to the employees are done here
*
* if (!success) {
*   // if we don't want the changes to be done after all, then we need to cancel the edit
*   edit.cancel()
* }
* </pre>
*
* Alternatively, when using a specific provider, consider the following examples. The following is an example
* implementation of an item that is being managed using {@link IMementoSupport}:
*
* <br><b>A sample business object</b>
* <pre>
* class Employee {
*   constructor(name, position, age) {
*     this._name = name
*     this.position = position
*     this.age = age
*   }
*
*   get name() {
*     return this._name
*   }
* }
* </pre>
*
* A collection of items from this type can then be watched using the following code snippet, using the <code>provider</code> to return
* an appropriate {@link IMementoSupport} implementation:
*
* <br><b>Using an ICompoundEdit</b>
* <pre>
* const edit = graph.beginEdit(
*   undoName,
*   redoName,
*   listWithMyEmployeesToWatch,
*   item => new EmployeeMementoSupport()
* )
*
* // changes to the employees are done here
*
* if (!success) {
*   // if we don't want the changes to be done after all, then we need to cancel the edit
*   edit.cancel()
* }
* </pre>
*
* Implementing the {@link IMementoSupport} interface is quite unrestrained, the type of the state returned by {@link IMementoSupport#getState}
* method can by anything as long as the {@link IMementoSupport#applyState} and {@link IMementoSupport#stateEquals}
* methods can deal with it:
*
* <br><b>Sample implementation of IMementoSupport</b>
* <pre>
* class EmployeeMementoSupport extends BaseClass(IMementoSupport) {
*   getState(subject) {
*     if (subject instanceof Employee) {
*       return new EmployeeState(subject.position, subject.age)
*     }
*     return null
*   }
*
*   applyState(subject, state) {
*     if (subject instanceof Employee && state instanceof EmployeeState) {
*       subject.position = state.position
*       subject.age = state.age
*     }
*   }
*
*   stateEquals(state1, state2) {
*     if (state1 instanceof EmployeeState && state2 instanceof EmployeeState) {
*       return state1.position === state2.position && state1.age === state2.age
*     }
*     return state1 === state2
*   }
* }
* class EmployeeState {
*   constructor(position, age) {
*     this._position = position
*     this._age = age
*   }
*
*   get position() {
*     return this._position
*   }
*
*   get age() {
*     return this._age
*   }
* }
* </pre>
*
* In summary, use this concept when you want to track the state of items during certain operations for undo/redo. This is
* efficient if it's easier to handle an item's state than the changes to the item themselves. If you want to focus on the
* changes or on certain events, you should use custom {@link IUndoUnit}
*
* Type parameter <code>T</code>: The type of the items that will be modified subsequently.
* @param undoName The {@link IUndoUnit#undoName} of the {@link IUndoUnit} that will be placed into the undo queue after {@link ICompoundEdit#commit} has
* been called.
* @param redoName The {@link IUndoUnit#redoName} of the {@link IUndoUnit} that will be placed into the undo queue after {@link ICompoundEdit#commit} has
* been called.
* @param items The items that will be changed after this call and before the call to {@link ICompoundEdit#commit}.
* @param [provider=null] The provider for the {@link IMementoSupport} of the <code>items</code>. if the provider returns <code>null</code> for a given item, changes
* to this item are not being recorded.
* @returns An implementation of the {@link ICompoundEdit} interface whose {@link ICompoundEdit#commit} or {@link ICompoundEdit#cancel}
* methods need to be called after the items have been modified.
* @see IMementoSupport
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23beginEdit(string%2Cstring%2CIEnumerable%3CT%3E%2Cfunction(T)%3AIMementoSupport) Online Documentation}
*/
beginEdit<T>(undoName: string, redoName: string, items: IEnumerable<T> | IEnumerableConvertible<T>, provider?: ((item: T) => IMementoSupport | null) | null): ICompoundEdit;
/**
Starts an {@link ICompoundEdit} that records graph changes and {@link IGraph#addUndoUnit custom undo units} in a single compound unit.
This method can be used to bracket several undo units. All edits added to the queue after this call and before a call to
{@link ICompoundEdit#cancel} or {@link ICompoundEdit#commit} will be placed into the queue as a single block.
Client code needs to make sure that either the {@link ICompoundEdit#cancel} or {@link ICompoundEdit#commit}
method is called on the returned instance.
@param undoName The undo name for the compound edit.
@param redoName The redo name for the compound edit.
@returns The handle to stop the recording by calling {@link ICompoundEdit#cancel} or {@link ICompoundEdit#commit} on
it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23beginEdit(string%2Cstring) Online Documentation}*/
beginEdit(undoName: string, redoName: string): ICompoundEdit;
/**
Calculates the preferred size of a label with the given properties.
@param owner The item that will own the label.
@param text The text.
@param [layoutParameter=null] The label model parameter.
@param [style=null] The label style.
@param [tag=null] The tag for the label.
@returns The size as calculated by the {@link ILabelStyleRenderer}.
@see IGraph#setLabelPreferredSize
@see ILabel#preferredSize
@see IGraph#adjustLabelPreferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23calculateLabelPreferredSize(ILabelOwner%2Cstring%2CILabelModelParameter%2CILabelStyle%2CObject) Online Documentation}*/
calculateLabelPreferredSize(owner: ILabelOwner, text: string, layoutParameter?: ILabelModelParameter | null, style?: ILabelStyle | null, tag?: any | null): Size;
/**
Calculates the preferred size of a label with the given properties.
@param options The parameters to pass.
@param options.owner The item that will own the label.
@param options.text The text.
@param [options.layoutParameter=null] The label model parameter.
@param [options.style=null] The label style.
@param [options.tag=null] The tag for the label.
@returns The size as calculated by the {@link ILabelStyleRenderer}.
@see IGraph#setLabelPreferredSize
@see ILabel#preferredSize
@see IGraph#adjustLabelPreferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23calculateLabelPreferredSize(ILabelOwner%2Cstring%2CILabelModelParameter%2CILabelStyle%2CObject) Online Documentation}*/
calculateLabelPreferredSize(options: {
owner: ILabelOwner;
text: string;
layoutParameter?: ILabelModelParameter | null;
style?: ILabelStyle | null;
tag?: any | null;
}): Size;
/**
Clears the graph, removing all items in proper order.
@see IGraph#remove
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23clear() Online Documentation}*/
clear(): void;
/**
Removes all bends from the given edge.
The edge must be part of this graph at the time of the invocation. This will trigger the corresponding events.
@param edge the edge whose bends will be removed
@throws {Exception} <b>(ArgumentError)</b> <code>edge</code> is not in this graph.
@see IGraph#remove
@see IGraph#addBendRemovedListener
@see IEdge#bends
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23clearBends(IEdge) Online Documentation}*/
clearBends(edge: IEdge): void;
/**
* Determines whether this graph contains the specified item.
* <pre>
* const node = graph.createNode()
*
* // an element which is created in a graph is contained in that graph
* console.log(graph.contains(node)) // true
*
* // an element which is created in a graph is not contained in another graph
* console.log(anotherGraph.contains(node)) // false
*
* graph.remove(node)
* // an element which is removed is not contained in the graph anymore
* console.log(graph.contains(node)) // false
* </pre>
*
* @param item The item.
* @returns <code>true</code> if this graph contains the specified item; otherwise, <code>false</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23contains(IModelItem) Online Documentation}
*/
abstract contains(item: IModelItem | null): boolean;
/**
Creates the label layout parameter for a given {@link ILabelOwner}.
This implementation uses the {@link IGraph#getLabelDefaults label defaults} for the <code>graph</code> to {@link ILabelDefaults#getLayoutParameterInstance obtain the parameter instance}.
@param owner The item that is the {@link ILabel#owner} of the label in question.
@returns The default label layout parameter to use for newly created labels at the item.
@see ILabelDefaults#getLayoutParameterInstance
@see IGraph#getLabelDefaults
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createDefaultLabelLayoutParameter(ILabelOwner) Online Documentation}*/
createDefaultLabelLayoutParameter(owner: ILabelOwner): ILabelModelParameter;
/**
Creates a location model parameter for a newly created {@link IPort} at the <code>owner</code> that matches the <code>location</code>.
If <code>location</code> is <code>null</code>, this method uses the {@link IGraph#getPortDefaults port defaults} for the <code>owner</code> to obtain the {@link IPortDefaults#getLocationParameterInstance location parameter}.
@param owner The owner of the port.
@param [location=null] The location in the world coordinate system where the port should be added.
@returns Either a location model parameter that matches the location, or the default parameter to use for the {@link IPortOwner}
as returned by {@link IPortDefaults#getLocationParameterInstance}.
@throws {Exception} <b>(ArgumentError)</b> <code>location</code> contains one or more NaN values.
@see IPortLocationModel#createParameter
@see IGraph#getPortDefaults
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createDefaultPortLocationParameter(IPortOwner%2CPoint) Online Documentation}*/
createDefaultPortLocationParameter(owner: IPortOwner, location?: Point | null | PointConvertible): IPortLocationModelParameter;
/**
* Creates and returns an edge that connects to the given node instances using the given style instance.
* The nodes must be part of this graph at the time of the invocation, and an implementation can choose the {@link IPort}
* instances to which the edge will be connected. The edge will be a part of this graph after the method returns. This will
* trigger the corresponding events.
*
* <br><b>Creating edges between source and target node</b>
* <pre>
* const node1 = graph.createNodeAt(new Point(0, 0))
* const node2 = graph.createNodeAt(new Point(100, 0))
*
* // Create an edge between node1 and node2 with default style
* const edge1 = graph.createEdge(node1, node2)
*
* // Create an edge between node1 and node2 with the given style and tag
* const edge2 = graph.createEdge(
*   node1,
*   node2,
*   new PolylineEdgeStyle({ targetArrow: Arrow.DEFAULT }),
*   tag
* )
* </pre>
*
* @param source The source node the created edge will connect to. It is up to the implementation to decide which port to use at the
* given node. An implementation may create a new port of the edge. To change the source port after the edge has been
* created, use {@link IGraph#setEdgePorts}.
* @param target The target node the created edge will connect to. It is up to the implementation to decide which port to use at the
* given node. An implementation may create a new port of the edge. To change the target port after the edge has been
* created, use {@link IGraph#setEdgePorts}.
* @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference. If omitted the {@link IGraph#edgeDefaults default style}
* will be set. To change the style after the edge has been created, use {@link IGraph#setStyle}.
* @param [tag=null] The initial value of the {@link ITagOwner#tag} that will be assigned to the new edge.
* @returns the newly created edge instance
* @see IGraph#addEdgeCreatedListener
* @see VoidEdgeStyle#INSTANCE
* @see IGraph#edgeDefaults
* @see IGraph#setEdgePorts
* @see IEdge#sourcePort
* @see IEdge#targetPort
* @see IGraph#setStyle
* @see IEdge#style
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createEdge(INode%2CINode%2CIEdgeStyle%2CObject) Online Documentation}
*/
abstract createEdge(source: INode, target: INode, style?: IEdgeStyle | null, tag?: any | null): IEdge;
/**
* Creates and returns an edge that connects to the given node instances using the given style instance.
* The nodes must be part of this graph at the time of the invocation, and an implementation can choose the {@link IPort}
* instances to which the edge will be connected. The edge will be a part of this graph after the method returns. This will
* trigger the corresponding events.
*
* <br><b>Creating edges between source and target node</b>
* <pre>
* const node1 = graph.createNodeAt(new Point(0, 0))
* const node2 = graph.createNodeAt(new Point(100, 0))
*
* // Create an edge between node1 and node2 with default style
* const edge1 = graph.createEdge(node1, node2)
*
* // Create an edge between node1 and node2 with the given style and tag
* const edge2 = graph.createEdge(
*   node1,
*   node2,
*   new PolylineEdgeStyle({ targetArrow: Arrow.DEFAULT }),
*   tag
* )
* </pre>
*
* @param options The parameters to pass.
* @param options.source The source node the created edge will connect to. It is up to the implementation to decide which port to use at the
* given node. An implementation may create a new port of the edge. To change the source port after the edge has been
* created, use {@link IGraph#setEdgePorts}.
* @param options.target The target node the created edge will connect to. It is up to the implementation to decide which port to use at the
* given node. An implementation may create a new port of the edge. To change the target port after the edge has been
* created, use {@link IGraph#setEdgePorts}.
* @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference. If omitted the {@link IGraph#edgeDefaults default style}
* will be set. To change the style after the edge has been created, use {@link IGraph#setStyle}.
* @param [options.tag=null] The initial value of the {@link ITagOwner#tag} that will be assigned to the new edge.
* @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link IGraph#addLabel}
* method.
* @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link IGraph#addPort} method.
* @param options.bends An array of bends to add to the newly created edge. Each item will be passed to the {@link IGraph#addBend} method.
* @returns the newly created edge instance
* @see IGraph#addEdgeCreatedListener
* @see VoidEdgeStyle#INSTANCE
* @see IGraph#edgeDefaults
* @see IGraph#setEdgePorts
* @see IEdge#sourcePort
* @see IEdge#targetPort
* @see IGraph#setStyle
* @see IEdge#style
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createEdge(INode%2CINode%2CIEdgeStyle%2CObject) Online Documentation}
*/
abstract createEdge(options: {
source: INode;
target: INode;
style?: IEdgeStyle | null;
tag?: any | null;
labels?: Array<any>;
ports?: Array<any>;
bends?: Array<any>;
}): IEdge;
/**
* Creates and returns an edge that connects to the given port instances.
* The ports must be part of this graph at the time of the invocation. The edge will be a part of this graph after the
* method returns. This will trigger the corresponding events.
*
* <br><b>Creating edges between ports</b>
* <pre>
* const node1 = graph.createNodeAt(new Point(0, 0))
* const port1 = graph.addPort(node1)
* const node2 = graph.createNodeAt(new Point(100, 0))
* const port2 = graph.addPort(node2)
*
* // Create an edge between port1 and port2 with default style
* const edge1 = graph.createEdge(port1, port2)
*
* // Create an edge between port1 and port2 with the given style and tag
* const edge2 = graph.createEdge(
*   port1,
*   port2,
*   new PolylineEdgeStyle({ targetArrow: Arrow.DEFAULT }),
*   tag
* )
* </pre>
*
* @param sourcePort The source port the created edge will connect to. To change the source port after the edge has been created, use
* {@link IGraph#setEdgePorts}.
* @param targetPort The target port the created edge will connect to. To change the target port after the edge has been created, use
* {@link IGraph#setEdgePorts}.
* @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference. If omitted the {@link IGraph#edgeDefaults default style}
* will be set. To change the style after the edge has been created, use {@link IGraph#setStyle}.
* @param [tag=null] The initial value of the {@link ITagOwner#tag} that will be assigned to the new edge.
* @returns the newly created edge instance
* @see IGraph#addEdgeCreatedListener
* @see VoidEdgeStyle#INSTANCE
* @see IGraph#edgeDefaults
* @see IGraph#setEdgePorts
* @see IEdge#sourcePort
* @see IEdge#targetPort
* @see IGraph#setStyle
* @see IEdge#style
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createEdge(IPort%2CIPort%2CIEdgeStyle%2CObject) Online Documentation}
*/
abstract createEdge(sourcePort: IPort, targetPort: IPort, style?: IEdgeStyle | null, tag?: any | null): IEdge;
/**
* Creates and returns an edge that connects to the given port instances.
* The ports must be part of this graph at the time of the invocation. The edge will be a part of this graph after the
* method returns. This will trigger the corresponding events.
*
* <br><b>Creating edges between ports</b>
* <pre>
* const node1 = graph.createNodeAt(new Point(0, 0))
* const port1 = graph.addPort(node1)
* const node2 = graph.createNodeAt(new Point(100, 0))
* const port2 = graph.addPort(node2)
*
* // Create an edge between port1 and port2 with default style
* const edge1 = graph.createEdge(port1, port2)
*
* // Create an edge between port1 and port2 with the given style and tag
* const edge2 = graph.createEdge(
*   port1,
*   port2,
*   new PolylineEdgeStyle({ targetArrow: Arrow.DEFAULT }),
*   tag
* )
* </pre>
*
* @param options The parameters to pass.
* @param options.sourcePort The source port the created edge will connect to. To change the source port after the edge has been created, use
* {@link IGraph#setEdgePorts}.
* @param options.targetPort The target port the created edge will connect to. To change the target port after the edge has been created, use
* {@link IGraph#setEdgePorts}.
* @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference. If omitted the {@link IGraph#edgeDefaults default style}
* will be set. To change the style after the edge has been created, use {@link IGraph#setStyle}.
* @param [options.tag=null] The initial value of the {@link ITagOwner#tag} that will be assigned to the new edge.
* @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link IGraph#addLabel}
* method.
* @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link IGraph#addPort} method.
* @param options.bends An array of bends to add to the newly created edge. Each item will be passed to the {@link IGraph#addBend} method.
* @returns the newly created edge instance
* @see IGraph#addEdgeCreatedListener
* @see VoidEdgeStyle#INSTANCE
* @see IGraph#edgeDefaults
* @see IGraph#setEdgePorts
* @see IEdge#sourcePort
* @see IEdge#targetPort
* @see IGraph#setStyle
* @see IEdge#style
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createEdge(IPort%2CIPort%2CIEdgeStyle%2CObject) Online Documentation}
*/
abstract createEdge(options: {
sourcePort: IPort;
targetPort: IPort;
style?: IEdgeStyle | null;
tag?: any | null;
labels?: Array<any>;
ports?: Array<any>;
bends?: Array<any>;
}): IEdge;
/**
* Creates a new group node using the provided style and layout as a child of <code>parent</code>.
* The group node will be a direct descendant of <code>parent</code>.
*
* To create group nodes interactively use the {@link GraphEditorInputMode} as input mode and enable the
* {@link GraphEditorInputMode#allowGroupingOperations grouping operations}.
*
* <pre>
* // create a group node top level (without parent) at 0,0 with default size and style
* const node1 = graph.createGroupNode()
*
* // create a group node as child of the given parent node at 0,0 with default size and style
* const node2 = graph.createGroupNode(parent)
*
* // create a group node as child of the given parent node with the given layout, style, and tag (user object)
* const node3 = graph.createGroupNode(
*   parent,
*   new Rect(x, y, width, height),
*   new ShapeNodeStyle(),
*   userObject
* )
*
* // create a group node as child of the given parent node at 0,0 with default size and style and the given tag
* const node4 = graph.createGroupNode({ parent, tag: userObject })
* </pre>
*
* The newly created node is a group node from the beginning. As such, it takes its defaults from {@link IGraph#groupNodeDefaults}:
*
* <pre>
* graph.nodeDefaults.style = new ShapeNodeStyle()
* graph.groupNodeDefaults.style = new PanelNodeStyle()
*
* const group = graph.createGroupNode()
* // the newly created node is a group node from the beginning
* console.log(graph.isGroupNode(group)) // true
* // the newly created node is created with the GroupNodeDefaults
* console.log(PanelNodeStyle.isInstance(group.style)) // true
* </pre>
*
* @param [parent=null] The node to use as the {@link IGraph#getParent parent} in the grouping hierarchy or <code>null</code> if the new node should become a top-level node. To
* change the parent after the group node has been created, use {@link IGraph#setParent}.
* @param [layout=null] The initial layout to use for the new node. If omitted the node will be placed with its top left corner at 0,0 and the
* {@link IGraph#groupNodeDefaults default size}. To change the layout after the group node has been created, use {@link IGraph#setNodeLayout}.
* @param [style=null] The style to use for the new node. If omitted the {@link IGraph#groupNodeDefaults default style} will be set. To change the style after the group node
* has been created, use {@link IGraph#setStyle}.
* @param [tag=null] The {@link ITagOwner#tag} to assign to the {@link INode}.
* @returns The newly created group node.
* @throws {Exception} <b>(ArgumentError)</b> The <code>layout</code> contains one or more NaN values.
* @see IGraph#createNode
* @see IGraph#setParent
* @see IGraph#getParent
* @see IGraph#setNodeLayout
* @see INode#layout
* @see IGraph#setStyle
* @see INode#style
* @see ITagOwner#tag
* @see IGraph#groupNodeDefaults
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createGroupNode(INode%2CRect%2CINodeStyle%2CObject) Online Documentation}
*/
abstract createGroupNode(parent?: INode | null, layout?: Rect | null | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
* Creates a new group node using the provided style and layout as a child of <code>parent</code>.
* The group node will be a direct descendant of <code>parent</code>.
*
* To create group nodes interactively use the {@link GraphEditorInputMode} as input mode and enable the
* {@link GraphEditorInputMode#allowGroupingOperations grouping operations}.
*
* <pre>
* // create a group node top level (without parent) at 0,0 with default size and style
* const node1 = graph.createGroupNode()
*
* // create a group node as child of the given parent node at 0,0 with default size and style
* const node2 = graph.createGroupNode(parent)
*
* // create a group node as child of the given parent node with the given layout, style, and tag (user object)
* const node3 = graph.createGroupNode(
*   parent,
*   new Rect(x, y, width, height),
*   new ShapeNodeStyle(),
*   userObject
* )
*
* // create a group node as child of the given parent node at 0,0 with default size and style and the given tag
* const node4 = graph.createGroupNode({ parent, tag: userObject })
* </pre>
*
* The newly created node is a group node from the beginning. As such, it takes its defaults from {@link IGraph#groupNodeDefaults}:
*
* <pre>
* graph.nodeDefaults.style = new ShapeNodeStyle()
* graph.groupNodeDefaults.style = new PanelNodeStyle()
*
* const group = graph.createGroupNode()
* // the newly created node is a group node from the beginning
* console.log(graph.isGroupNode(group)) // true
* // the newly created node is created with the GroupNodeDefaults
* console.log(PanelNodeStyle.isInstance(group.style)) // true
* </pre>
*
* @param options The parameters to pass.
* @param [options.parent=null] The node to use as the {@link IGraph#getParent parent} in the grouping hierarchy or <code>null</code> if the new node should become a top-level node. To
* change the parent after the group node has been created, use {@link IGraph#setParent}.
* @param [options.layout=null] The initial layout to use for the new node. If omitted the node will be placed with its top left corner at 0,0 and the
* {@link IGraph#groupNodeDefaults default size}. To change the layout after the group node has been created, use {@link IGraph#setNodeLayout}.
* @param [options.style=null] The style to use for the new node. If omitted the {@link IGraph#groupNodeDefaults default style} will be set. To change the style after the group node
* has been created, use {@link IGraph#setStyle}.
* @param [options.tag=null] The {@link ITagOwner#tag} to assign to the {@link INode}.
* @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link IGraph#addLabel}
* method.
* @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link IGraph#addPort} method.
* @param options.children An array of nodes to create as child nodes of this group node. Each item will be passed to the {@link IGraph#createNode}
* method.
* @returns The newly created group node.
* @throws {Exception} <b>(ArgumentError)</b> The <code>layout</code> contains one or more NaN values.
* @see IGraph#createNode
* @see IGraph#setParent
* @see IGraph#getParent
* @see IGraph#setNodeLayout
* @see INode#layout
* @see IGraph#setStyle
* @see INode#style
* @see ITagOwner#tag
* @see IGraph#groupNodeDefaults
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createGroupNode(INode%2CRect%2CINodeStyle%2CObject) Online Documentation}
*/
abstract createGroupNode(options: {
parent?: INode | null;
layout?: Rect | null | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
labels?: Array<any>;
ports?: Array<any>;
children?: Array<any>;
}): INode;
/**
* Creates and returns a node using the specified values for the initial geometry, style, and {@link ITagOwner#tag}.
* The node will be a part of this graph after the method returns. This will trigger the corresponding events.
*
* <br><b>Creating a node with different parameters</b>
* <pre>
* // Create a node with default style and size at 0,0
* const node1 = graph.createNode()
*
* // create a node with the given layout and the default style
* const node2 = graph.createNode(new Rect(x, y, width, height))
*
* // create a node with the given layout, style, and tag (user object)
* const node3 = graph.createNode(new Rect(x, y, width, height), new ShapeNodeStyle(), userObject)
*
* // Create a node with default size at 0,0 with the given style
* const node4 = graph.createNode({ style: new ShapeNodeStyle() })
*
* // create a node at the given location with default style and size
* const node5 = graph.createNodeAt(new Point(x, y))
*
* // create a node at the given location with default size and the given style and tag
* const node6 = graph.createNodeAt(new Point(x, y), new ShapeNodeStyle(), userObject)
* </pre>
*
* @param layout The layout to use initially. The values will be copied to the node's {@link INode#layout Layout} field. To change the layout after the node
* has been created, use {@link IGraph#setNodeLayout}.
* @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference. If omitted the {@link IGraph#nodeDefaults default style}
* will be set. To change the style after the node has been added, use {@link IGraph#setStyle}.
* @param [tag=null] The initial value of the {@link ITagOwner#tag} that will be assigned to the new node.
* @returns A newly created node instance
* @throws {Exception} <b>(ArgumentError)</b> <code>layout</code> contains one or more NaN values.
* @see IGraph#addNodeCreatedListener
* @see VoidNodeStyle#INSTANCE
* @see IGraph#nodeDefaults
* @see IGraph#setNodeLayout
* @see INode#layout
* @see IGraph#setStyle
* @see INode#style
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createNode(Rect%2CINodeStyle%2CObject) Online Documentation}
*/
abstract createNode(layout: Rect | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
* Creates and returns a node using the specified values for the initial geometry, style, and {@link ITagOwner#tag}.
* The node will be a part of this graph after the method returns. This will trigger the corresponding events.
*
* <br><b>Creating a node with different parameters</b>
* <pre>
* // Create a node with default style and size at 0,0
* const node1 = graph.createNode()
*
* // create a node with the given layout and the default style
* const node2 = graph.createNode(new Rect(x, y, width, height))
*
* // create a node with the given layout, style, and tag (user object)
* const node3 = graph.createNode(new Rect(x, y, width, height), new ShapeNodeStyle(), userObject)
*
* // Create a node with default size at 0,0 with the given style
* const node4 = graph.createNode({ style: new ShapeNodeStyle() })
*
* // create a node at the given location with default style and size
* const node5 = graph.createNodeAt(new Point(x, y))
*
* // create a node at the given location with default size and the given style and tag
* const node6 = graph.createNodeAt(new Point(x, y), new ShapeNodeStyle(), userObject)
* </pre>
*
* @param options The parameters to pass.
* @param options.layout The layout to use initially. The values will be copied to the node's {@link INode#layout Layout} field. To change the layout after the node
* has been created, use {@link IGraph#setNodeLayout}.
* @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference. If omitted the {@link IGraph#nodeDefaults default style}
* will be set. To change the style after the node has been added, use {@link IGraph#setStyle}.
* @param [options.tag=null] The initial value of the {@link ITagOwner#tag} that will be assigned to the new node.
* @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link IGraph#addLabel}
* method.
* @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link IGraph#addPort} method.
* @returns A newly created node instance
* @throws {Exception} <b>(ArgumentError)</b> <code>layout</code> contains one or more NaN values.
* @see IGraph#addNodeCreatedListener
* @see VoidNodeStyle#INSTANCE
* @see IGraph#nodeDefaults
* @see IGraph#setNodeLayout
* @see INode#layout
* @see IGraph#setStyle
* @see INode#style
* @see ITagOwner#tag
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createNode(Rect%2CINodeStyle%2CObject) Online Documentation}
*/
abstract createNode(options: {
layout: Rect | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
labels?: Array<any>;
ports?: Array<any>;
}): INode;
/**
* Creates a new ordinary node as a direct descendant of <code>parent</code> using the given layout and style.
* <br><b>Creating nodes in a grouping hierarchy</b>
* <pre>
* // create a node top level (without parent) at 0,0 with default size and style
* const node1 = graph.createNode()
*
* // create a node as child of the given parent node at 0,0 with default size and style
* const node2 = graph.createNode(parent)
*
* // create a node as child of the given parent node with the given layout, style, and tag (user object)
* const node3 = graph.createNode(
*   parent,
*   new Rect(x, y, width, height),
*   new ShapeNodeStyle(),
*   userObject
* )
*
* // create a node as child of the given parent node at 0,0 with default size and style and the given tag
* const node4 = graph.createNode({ parent, tag: userObject })
* </pre>
*
* @param [parent=null] The node to use as the parent in the grouping hierarchy or <code>null</code> if the new node should become a top-level node. To
* change the parent after group node has been created, use {@link IGraph#setParent}.
* @param [layout=null] The layout to use initially. The values will be copied to the node's {@link INode#layout Layout} field. If omitted the {@link IGraph#nodeDefaults default size} will be
* set and the node's top left corner will be placed at 0,0. To change the layout after the group node has been created,
* use {@link IGraph#setNodeLayout}.
* @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference. If omitted the {@link IGraph#nodeDefaults default style}
* will be set.
* @param [tag=null] The {@link ITagOwner#tag} to assign to the {@link INode}.
* @returns The newly created node.
* @throws {Exception} <b>(ArgumentError)</b> <code>parent</code> is not in this graph, or <code>layout</code> contains one or more NaN values.
* @see IGraph#setParent
* @see IGraph#getParent
* @see IGraph#setNodeLayout
* @see INode#layout
* @see IGraph#setStyle
* @see INode#style
* @see ITagOwner#tag
* @see IGraph#createNode
* @see IGraph#createGroupNode
* @see IGraph#nodeDefaults
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createNode(INode%2CRect%2CINodeStyle%2CObject) Online Documentation}
*/
abstract createNode(parent?: INode | null, layout?: Rect | null | RectConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
* Creates a new ordinary node as a direct descendant of <code>parent</code> using the given layout and style.
* <br><b>Creating nodes in a grouping hierarchy</b>
* <pre>
* // create a node top level (without parent) at 0,0 with default size and style
* const node1 = graph.createNode()
*
* // create a node as child of the given parent node at 0,0 with default size and style
* const node2 = graph.createNode(parent)
*
* // create a node as child of the given parent node with the given layout, style, and tag (user object)
* const node3 = graph.createNode(
*   parent,
*   new Rect(x, y, width, height),
*   new ShapeNodeStyle(),
*   userObject
* )
*
* // create a node as child of the given parent node at 0,0 with default size and style and the given tag
* const node4 = graph.createNode({ parent, tag: userObject })
* </pre>
*
* @param options The parameters to pass.
* @param [options.parent=null] The node to use as the parent in the grouping hierarchy or <code>null</code> if the new node should become a top-level node. To
* change the parent after group node has been created, use {@link IGraph#setParent}.
* @param [options.layout=null] The layout to use initially. The values will be copied to the node's {@link INode#layout Layout} field. If omitted the {@link IGraph#nodeDefaults default size} will be
* set and the node's top left corner will be placed at 0,0. To change the layout after the group node has been created,
* use {@link IGraph#setNodeLayout}.
* @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference. If omitted the {@link IGraph#nodeDefaults default style}
* will be set.
* @param [options.tag=null] The {@link ITagOwner#tag} to assign to the {@link INode}.
* @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link IGraph#addLabel}
* method.
* @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link IGraph#addPort} method.
* @returns The newly created node.
* @throws {Exception} <b>(ArgumentError)</b> <code>parent</code> is not in this graph, or <code>layout</code> contains one or more NaN values.
* @see IGraph#setParent
* @see IGraph#getParent
* @see IGraph#setNodeLayout
* @see INode#layout
* @see IGraph#setStyle
* @see INode#style
* @see ITagOwner#tag
* @see IGraph#createNode
* @see IGraph#createGroupNode
* @see IGraph#nodeDefaults
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createNode(INode%2CRect%2CINodeStyle%2CObject) Online Documentation}
*/
abstract createNode(options: {
parent?: INode | null;
layout?: Rect | null | RectConvertible;
style?: INodeStyle | null;
tag?: any | null;
labels?: Array<any>;
ports?: Array<any>;
}): INode;
/**
Creates and returns a node using the specified initial center location and style, as well as the tag.
The node will be a part of this graph after the method returns. This will trigger the corresponding events.
@param location the initial coordinates of the center of the node's {@link INode#layout Layout} property
@param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
@param [tag=null] The initial value of the {@link ITagOwner#tag} that will be assigned to the new node.
@returns A newly created node instance
@throws {Exception} <b>(ArgumentError)</b> <code>location</code> contains one or more NaN values.
@see IGraph#addNodeCreatedListener
@see IGraph#createNode
@see IGraph#setNodeCenter
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createNodeAt(Point%2CINodeStyle%2CObject) Online Documentation}*/
createNodeAt(location: Point | PointConvertible, style?: INodeStyle | null, tag?: any | null): INode;
/**
Creates and returns a node using the specified initial center location and style, as well as the tag.
The node will be a part of this graph after the method returns. This will trigger the corresponding events.
@param options The parameters to pass.
@param options.location the initial coordinates of the center of the node's {@link INode#layout Layout} property
@param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
@param [options.tag=null] The initial value of the {@link ITagOwner#tag} that will be assigned to the new node.
@param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link IGraph#addLabel}
method.
@param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link IGraph#addPort} method.
@returns A newly created node instance
@throws {Exception} <b>(ArgumentError)</b> <code>location</code> contains one or more NaN values.
@see IGraph#addNodeCreatedListener
@see IGraph#createNode
@see IGraph#setNodeCenter
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23createNodeAt(Point%2CINodeStyle%2CObject) Online Documentation}*/
createNodeAt(options: {
location: Point | PointConvertible;
style?: INodeStyle | null;
tag?: any | null;
labels?: Array<any>;
ports?: Array<any>;
}): INode;
/**
Calculates the number of edges at the given {@link IPort} for this graph.
Note that an edge that is both incoming and outgoing will be counted twice.
@param port The port owner to count the degree of.
@returns The number of edges that are incident to the port.
@throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph.
@see IGraph#edgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23degree(IPort) Online Documentation}*/
degree(port: IPort): number;
/**
Calculates the number of edges at the given {@link IPortOwner} for this graph.
Note that an edge that is both incoming and outgoing will be counted twice.
@param owner The port owner to count the degree of.
@returns The number of edges that are incident to the port owner.
@throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
@see IGraph#edgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23degree(IPortOwner) Online Documentation}*/
degree(owner: IPortOwner): number;
/**
* Returns an {@link IListEnumerable} for all edges that have the given port owner as their {@link IEdge#targetPort target port's} or {@link IEdge#sourcePort source port's} {@link IPort#owner} depending on the
* {@link AdjacencyTypes}
* Note that even though edges can be accessed via index, the underlying graph structure in the default {@link IGraph}
* implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
* your own list, if possible. This is not necessary for the first or last element or when iterating over the adjacent
* edges via a <code>foreach</code> loop.
*
* <br><b>Iterating over all edges at the center node</b>
* <pre>
* for (const edge of graph.edgesAt(center)) {
*   console.log(edge)
*   // edge1, edge4, edge5, edge2, edge3, edge6
* }
* </pre>
*
* Note that the result is built by iterating over the edges at each port, in this example these are first the edges at
* port 1 (1, 4, 5), then the edges at port 2 (2, 3, 6)
*
* <br><b>Iterating over the outgoing edges at the center node</b>
* <pre>
* for (const edge of graph.edgesAt(center, AdjacencyTypes.OUTGOING)) {
*   console.log(edge) // edge4, edge5, edge6
* }
* // is the same as
* for (const edge of graph.outEdgesAt(center)) {
*   console.log(edge) // edge4, edge5, edge6
* }
* </pre>
*
* @param [type=AdjacencyTypes.ALL] The type of adjacency to consider.
* @param owner the port owner to check
* @returns An enumeration of all adjacent edges of the given type.
* @throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
* @see IGraph#inEdgesAt
* @see IGraph#outEdgesAt
* @see AdjacencyTypes
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23edgesAt(IPortOwner%2CAdjacencyTypes) Online Documentation}
*/
abstract edgesAt(owner: IPortOwner, type?: AdjacencyTypes | AdjacencyTypesConvertible): IListEnumerable<IEdge>;
/**
* Returns an {@link IEnumerable} for all edges that are adjacent to the given port as a {@link IEdge#sourcePort} or {@link IEdge#targetPort}.
* Note that even though edges can be accessed via index, the underlying graph structure in the default {@link IGraph}
* implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
* your own list, if possible. This is not necessary for the first or last element or when iterating over the adjacent
* edges via a <code>foreach</code> loop.
*
* <br><b>Iterating over all edges at port 1</b>
* <pre>
* for (const edge of graph.edgesAt(port1)) {
*   console.log(edge)
*   // edge1, edge4, edge5
* }
* </pre>
*
* <br><b>Iterating over the outgoing edges at port 1</b>
* <pre>
* for (const edge of graph.edgesAt(port1, AdjacencyTypes.OUTGOING)) {
*   console.log(edge) // edge4, edge5
* }
* // is the same as
* for (const edge of graph.outEdgesAt(port1)) {
*   console.log(edge) // edge4, edge5
* }
* </pre>
*
* @param port the port to check
* @param [type=AdjacencyTypes.ALL] The type of adjacency to consider.
* @returns An enumeration of all adjacent edges of the given type.
* @throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph.
* @see IGraph#inEdgesAt
* @see IGraph#outEdgesAt
* @see AdjacencyTypes
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23edgesAt(IPort%2CAdjacencyTypes) Online Documentation}
*/
abstract edgesAt(port: IPort, type?: AdjacencyTypes | AdjacencyTypesConvertible): IListEnumerable<IEdge>;
/**
* Returns an enumerable over the children of the provided node.
* This method returns the direct children, i.e. all nodes that have <code>node</code> as their {@link IGraph#getParent parent}. To get all descendants method {@link GroupingSupport#getDescendants}
* can be used.
*
* To make a node a child of <code>node</code>, use {@link IGraph#setParent} or create the node directly as a child with
* {@link IGraph#createNode}
*
* <pre>
* // create a group node which contains three existing nodes
* const groupNode = graph.groupNodes([node1, node2, node3])
*
* // get the children of a group node
* const children = graph.getChildren(groupNode) // node1, node2, node3
* const childCount = children.size // 3
* </pre>
*
* @param node The node for which to return the children or <code>null</code> if the top-level nodes should be returned.
* @returns All nodes that have <code>node</code> as their {@link IGraph#getParent parent}.
* @throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph.
* @see GroupingSupport#getDescendants
* @see IGraph#setParent
* @see IGraph#getParent
* @see IGraph#createNode
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23getChildren(INode) Online Documentation}
*/
abstract getChildren(node: INode | null): IListEnumerable<INode>;
/**
Finds an edge that connects <code>sourcePort</code> and <code>targetPort</code> in the given graph.
@param sourcePort The {@link IEdge#sourcePort} of the edge to find.
@param targetPort The {@link IEdge#targetPort} of the edge to find.
@returns An edge that satisfies the constraints or <code>null</code>, if none was found.
@throws {Exception} <b>(ArgumentError)</b> <code>sourcePort</code> or <code>targetPort</code> are not in this graph.
@see IGraph#edgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23getEdge(IPort%2CIPort) Online Documentation}*/
getEdge(sourcePort: IPort, targetPort: IPort): IEdge | null;
/**
Finds an edge that connects <code>from</code> and <code>to</code> in the given graph.
@param from The {@link IEdge#sourcePort} owner of the edge to find.
@param to The {@link IEdge#targetPort} owner of the edge to find.
@returns An edge that satisfies the constraints or <code>null</code>, if none was found.
@throws {Exception} <b>(ArgumentError)</b> <code>from</code> or <code>to</code> are not in this graph.
@see IGraph#edgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23getEdge(IPortOwner%2CIPortOwner) Online Documentation}*/
getEdge(from: IPortOwner, to: IPortOwner): IEdge | null;
/**
Gets the {@link ILabelDefaults label defaults} for a given {@link ILabelOwner} in the context of the <code>graph</code>.
@param owner The item that the label defaults are returned for. If this is a group node, the {@link IGraph#groupNodeDefaults}'s {@link INodeDefaults#labels label defaults} will be returned,
otherwise the {@link IGraph#nodeDefaults} or {@link IGraph#edgeDefaults} {@link INodeDefaults#labels labels} will be returned.
@returns Appropriate {@link ILabelDefaults} for the provided <code>owner</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23getLabelDefaults(ILabelOwner) Online Documentation}*/
getLabelDefaults(owner: ILabelOwner): ILabelDefaults;
/**
* Returns the parent node of the node or <code>null</code> if <code>node</code> is a top-level node.
* <pre>
* // create a group node
* const groupNode = graph.createGroupNode()
*
* // set groupNode as parent of node1
* graph.setParent(node1, groupNode)
*
* // get the parent of a node
* const node1Parent = graph.getParent(node1) // groupNode
* const groupParent = graph.getParent(groupNode) // null
* </pre>
*
* @param node The node to retrieve the parent node for.
* @returns The parent node in this hierarchy or <code>null</code> if <code>node</code> is a top-level node.
* @throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph.
* @see IGraph#setParent
* @see IGraph#getChildren
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23getParent(INode) Online Documentation}
*/
abstract getParent(node: INode): INode | null;
/**
Gets the {@link IPortDefaults port defaults} for a given {@link IPortOwner} in the context of the <code>graph</code>.
@param owner The item that the label defaults are returned for. If this is a group node, the {@link IGraph#groupNodeDefaults}'s {@link INodeDefaults#ports port defaults} will be returned,
otherwise the {@link IGraph#nodeDefaults} or {@link IGraph#edgeDefaults} {@link INodeDefaults#ports ports} will be returned.
@returns Appropriate {@link IPortDefaults} for the provided <code>owner</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23getPortDefaults(IPortOwner) Online Documentation}*/
getPortDefaults(owner: IPortOwner): IPortDefaults;
/**
* Groups the nodes in <code>children</code> into the provided group node.
* The <code>parent</code> needs to be a group node at the time of the invocation. This operation is basically the same as calling {@link IGraph#setParent}
* for each node in <code>children</code> whose parent is not part of the set.
*
* <br><b>How to create add multiple children to a group node</b>
* <pre>
* // create some nodes
* const children = []
* for (let i = 0; i < 5; i++) {
*   children.push(graph.createNode())
* }
* // create a group node
* const parent = graph.createGroupNode()
* // add the nodes as children to the group
* graph.groupNodes(parent, children)
* // adjust the group node's layout to include all its children
* graph.adjustGroupNodeLayout(parent)
* </pre>
*
* @param parent The node to use as the parent in the grouping hierarchy.
* @param children The children to group into the group node.
* @throws {Exception} <b>(ArgumentError)</b> <code>parent</code> or one of <code>children</code> is not in this graph.
* @see IGraph#groupNodes
* @see IGraph#setParent
* @see IGraph#createGroupNode
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23groupNodes(INode%2CIEnumerable%3CINode%3E) Online Documentation}
*/
groupNodes(parent: INode, children: IEnumerable<INode> | IEnumerableConvertible<INode>): void;
/**
* Groups the nodes in <code>children</code> into a newly created group node.
* The group node will be created at the common ancestor level of all nodes in <code>children</code>.
*
* <br><b>How to create add multiple children to a group node</b>
* <pre>
* // create some nodes
* const children = []
* for (let i = 0; i < 5; i++) {
*   children.push(graph.createNode())
* }
* // create a group node and add the given nodes as children
* const group = graph.groupNodes(children)
* // adjust the group node's layout to include all its children
* graph.adjustGroupNodeLayout(group)
* </pre>
*
* @param children The children to group into the new group node.
* @returns The newly created group node.
* @throws {Exception} <b>(ArgumentError)</b> One of <code>children</code> is not in this graph.
* @see GroupingSupport#getNearestCommonAncestor
* @see IGraph#groupNodes
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23groupNodes(IEnumerable%3CINode%3E) Online Documentation}
*/
groupNodes(children: IEnumerable<INode> | IEnumerableConvertible<INode>): INode;
/**
* Groups the nodes in <code>children</code> into a newly created group node.
* The group node will be created at the common ancestor level of all nodes in <code>children</code>.
*
* <br><b>How to create add multiple children to a group node</b>
* <pre>
* // create some nodes
* const children = []
* for (let i = 0; i < 5; i++) {
*   children.push(graph.createNode())
* }
* // create a group node and add the given nodes as children
* const group = graph.groupNodes(children)
* // adjust the group node's layout to include all its children
* graph.adjustGroupNodeLayout(group)
* </pre>
*
* @param options The parameters to pass.
* @param options.children The children to group into the new group node.
* @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link IGraph#addLabel}
* method.
* @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link IGraph#addPort} method.
* @returns The newly created group node.
* @throws {Exception} <b>(ArgumentError)</b> One of <code>children</code> is not in this graph.
* @see GroupingSupport#getNearestCommonAncestor
* @see IGraph#groupNodes
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23groupNodes(IEnumerable%3CINode%3E) Online Documentation}
*/
groupNodes(options: {
children: IEnumerable<INode> | IEnumerableConvertible<INode>;
labels?: Array<any>;
ports?: Array<any>;
}): INode;
/**
Calculates the number of incoming edges at the given {@link IPort} for this graph.
@param port The port to count the incoming edges of.
@returns The number of edges that have the port as their target port.
@throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph.
@see IGraph#edgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23inDegree(IPort) Online Documentation}*/
inDegree(port: IPort): number;
/**
Calculates the number of incoming edges at the given {@link IPortOwner} for this graph.
@param owner The port owner to count the incoming edges of.
@returns The number of edges that have the port owner as their target port's owner.
@throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
@see IGraph#edgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23inDegree(IPortOwner) Online Documentation}*/
inDegree(owner: IPortOwner): number;
/**
Convenience method that yields the incoming edges at the given port.
This method delegates to {@link IGraph#edgesAt} using {@link AdjacencyTypes#INCOMING}.
Note that even though edges can be accessed via index, the underlying graph structure in the default {@link IGraph}
implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
your own list, if possible. This is not necessary for the first or last element or when iterating over the incoming
edges via a <code>foreach</code> loop.
@param port The port of the edges.
@returns An enumerable for the edges.
@throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph.
@see IGraph#edgesAt
@see IGraph#outEdgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23inEdgesAt(IPort) Online Documentation}*/
inEdgesAt(port: IPort): IListEnumerable<IEdge>;
/**
Convenience method that yields the incoming edges at the given owner.
This method delegates to {@link IGraph#edgesAt} using {@link AdjacencyTypes#INCOMING}.
Note that even though edges can be accessed via index, the underlying graph structure in the default {@link IGraph}
implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
your own list, if possible. This is not necessary for the first or last element or when iterating over the incoming
edges via a <code>foreach</code> loop.
@param owner The owner of the edges.
@returns An enumerable for the edges.
@throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
@see IGraph#edgesAt
@see IGraph#outEdgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23inEdgesAt(IPortOwner) Online Documentation}*/
inEdgesAt(owner: IPortOwner): IListEnumerable<IEdge>;
/**
Causes the {@link IGraph#addDisplaysInvalidatedListener DisplaysInvalidated} event to be triggered.
This method may be called by client code to invalidate all views of the graph that have registered with the {@link IGraph#addDisplaysInvalidatedListener DisplaysInvalidated}
event. Views that need to be informed if non-structural changes have been made to the graph should register with the
corresponding event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23invalidateDisplays() Online Documentation}*/
abstract invalidateDisplays(): void;
/**
* Returns whether the given node is a group node.
* Group nodes are nodes which can have {@link IGraph#getChildren children}. They may have children but do not necessarily have to.
*
* <br><b>How to set or query a node's group node state</b>
* <pre>
* // create a node in the graph
* const node1 = graph.createNode()
* // query whether it is a group node
* const node1IsGroupNode = graph.isGroupNode(node1) // false
*
* // create a group node in the graph
* const node2 = graph.createGroupNode()
* // query whether it is a group node
* const node2IsGroupNode = graph.isGroupNode(node2) // true
*
* // convert node1 into a group node
* graph.setIsGroupNode(node1, true)
* // convert node2 into a normal, i.e. non-group, node
* graph.setIsGroupNode(node2, false)
*
* // group is a group node with a child
* const group = graph.groupNodes([node1])
* // you cannot convert a group node with a child into a non-group node
* graph.setIsGroupNode(group, false) // throws exception!
* </pre>
*
* @param node The node to check.
* @returns Whether the node is considered a group node.
* @throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph.
* @see IGraph#getChildren
* @see IGraph#setIsGroupNode
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23isGroupNode(INode) Online Documentation}
*/
abstract isGroupNode(node: INode | null): boolean;
/**
Enumerates the neighbors of a given {@link INode}.
Neighbors are calculated by going through all {@link IPortOwner#ports} and inspecting the {@link IGraph#edgesAt edges at these ports}, collecting the
{@link IEdge#opposite}s.
@param node The node.
@returns An enumerable over all neighbors.
@throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph.
@see IGraph#edgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23neighbors(INode) Online Documentation}*/
neighbors(node: INode): IEnumerable<INode>;
/**
Calculates the number of outgoing edges at the given {@link IPort} for this graph.
@param port The port to count the outgoing edges of.
@returns The number of edges that have the port as their source port.
@throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph.
@see IGraph#edgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23outDegree(IPort) Online Documentation}*/
outDegree(port: IPort): number;
/**
Calculates the number of outgoing edges at the given {@link IPortOwner} for this graph.
@param owner The port owner to count the outgoing edges of.
@returns The number of edges that have the port owner as their source port's owner.
@throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
@see IGraph#edgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23outDegree(IPortOwner) Online Documentation}*/
outDegree(owner: IPortOwner): number;
/**
Convenience method that yields the outgoing edges at the given port.
This method delegates to {@link IGraph#edgesAt} using {@link AdjacencyTypes#OUTGOING}.
Note that even though edges can be accessed via index, the underlying graph structure in the default {@link IGraph}
implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
your own list, if possible. This is not necessary for the first or last element or when iterating over the outgoing
edges via a <code>foreach</code> loop.
@param port The owner of the edges.
@returns An enumerable for the edges.
@throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph.
@see IGraph#edgesAt
@see IGraph#inEdgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23outEdgesAt(IPort) Online Documentation}*/
outEdgesAt(port: IPort): IListEnumerable<IEdge>;
/**
Convenience method that yields the outgoing edges at the given owner.
This method delegates to {@link IGraph#edgesAt} using {@link AdjacencyTypes#OUTGOING}.
Note that even though edges can be accessed via index, the underlying graph structure in the default {@link IGraph}
implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
your own list, if possible. This is not necessary for the first or last element or when iterating over the outgoing
edges via a <code>foreach</code> loop.
@param owner The owner of the edges.
@returns An enumerable for the edges.
@throws {Exception} <b>(ArgumentError)</b> <code>owner</code> is not in this graph.
@see IGraph#edgesAt
@see IGraph#inEdgesAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23outEdgesAt(IPortOwner) Online Documentation}*/
outEdgesAt(owner: IPortOwner): IListEnumerable<IEdge>;
/**
Enumerates the predecessors of a given {@link INode}.
Predecessors are calculated by going through all {@link IPortOwner#ports} and inspecting the {@link IGraph#inEdgesAt incoming edges at these ports}, collecting the {@link IEdge#sourcePort} {@link IPort#owner}s.
@param node The node.
@returns An enumerable over all predecessors.
@throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23predecessors(INode) Online Documentation}*/
predecessors(node: INode): IEnumerable<INode>;
/**
* Removes the given {@link IModelItem item} from this graph.
* The item must be a part of this graph.
*
* If the item is a {@link INode node} the {@link IGraph#addNodeRemovedListener NodeRemoved} event will be triggered. This
* will remove all adjacent edges and their corresponding ports in proper order before the node will be removed. Also this
* will trigger the removal of all labels owned by this instance.
*
* If the item is an {@link IEdge edge} the {@link IGraph#addEdgeRemovedListener EdgeRemoved} event will be triggered. Also
* this will trigger the removal of all labels and bends owned by this instance. An implementation may decide to remove the
* corresponding ports from the node if no other edge connects to them after the given edge has been removed. The
* implementations provided by yFiles do so according to the value set to {@link IPortDefaults#autoCleanUp} in their {@link IPortDefaults}.
*
* If the item is a {@link IBend bend} the {@link IGraph#addBendRemovedListener BendRemoved} event will be triggered.
*
* If the item is a {@link IPort port} the {@link IGraph#addPortRemovedListener PortRemoved} event will be triggered. This
* will also remove all edges that are currently connected to the port and all labels and bends owned by this instance.
*
* If the item is a {@link ILabel label} the {@link IGraph#addLabelRemovedListener LabelRemoved} event will be triggered.
*
* <br><b>Removing an item from the graph</b>
* <pre>
* const node = graph.createNode()
* console.log(graph.contains(node)) // true
* graph.remove(node)
* console.log(graph.contains(node)) // false
* </pre>
*
* <br><b>Removing an item removes its dependent items, too</b>
* <pre>
* const node = graph.createNode()
* const label = graph.addLabel(node, 'Test')
* console.log(graph.contains(label)) // true
* graph.remove(node)
* console.log(graph.contains(label)) // false
* </pre>
*
* <br><b>Removing items from their collections</b>
* <pre>
* // IMPORTANT: graph.nodes will throw exception if it is changed during iteration
* // therefore, we have to iterate over a copy of the node collection
* // while we remove the nodes
* const nodesToRemove = graph.nodes.toList()
* for (const node of nodesToRemove) {
*   graph.remove(node)
* }
* </pre>
*
* @param item the item to be removed from this graph instance
* @throws {Exception} <b>(ArgumentError)</b> <code>item</code> is not in this graph.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23remove(IModelItem) Online Documentation}
*/
abstract remove(item: IModelItem): void;
/**
Reverses an edge by {@link IGraph#setEdgePorts setting source and target port} to {@link IEdge#targetPort} and {@link IEdge#sourcePort}.
This also reverses the bends by clearing them and reinserting them in reverse order if there is more than one bend.
@param edge The edge to reverse.
@throws {Exception} <b>(ArgumentError)</b> <code>edge</code> is not in this graph.
@see IGraph#setEdgePorts
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23reverse(IEdge) Online Documentation}*/
reverse(edge: IEdge): void;
/**
* Modifies the location of the given bend.
* To query a bend location, use {@link IBend#location}.
*
* <br><b>Setting the location of a bend</b>
* <pre>
* graph.setBendLocation(bend, new Point(newX, newY))
* </pre>
*
* <br><b>Getting the location of a bend</b>
* <pre>
* const location = bend.location
* </pre>
*
* @param bend the bend whose location is to be modified
* @param location the new location of the bend
* @throws {Exception} <b>(ArgumentError)</b> <code>bend</code> is not in this graph, or <code>location</code> contains one or more NaN values.
* @see IGraph#addBend
* @see IBend#location
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setBendLocation(IBend%2CPoint) Online Documentation}
*/
abstract setBendLocation(bend: IBend, location: Point): void;
/**
* Sets the source and target ports of the given edge to the new values.
* This will trigger an {@link IGraph#addEdgePortsChangedListener EdgePortsChanged} event if source or target ports
* differ from the current ones. Both ports and the edge must belong to the current graph instance.
*
* An implementation may decide to remove the corresponding ports if no other edge connects to them after the given edge
* has its source or target port changed. The implementations provided by yFiles do so according to the value set to {@link IPortDefaults#autoCleanUp} in
* their {@link IPortDefaults}.
*
* To query the current source and target ports, you can use the {@link IEdge#sourcePort} and {@link IEdge#targetPort} properties.
*
* <br><b>Setting source and target port of an edge</b>
* <pre>
* graph.setEdgePorts(edge, newSourcePort, newTargetPort)
* </pre>
*
* <br><b>Getting the target node of an edge</b>
* <pre>
* const sourcePort = edge.sourcePort
* </pre>
*
* <br><b>Getting the target port of an edge</b>
* <pre>
* const targetPort = edge.targetPort
* </pre>
*
* @param edge The edge to change the ports.
* @param sourcePort The new source port instance.
* @param targetPort The new target port instance.
* @throws {Exception} <b>(ArgumentError)</b> Either <code>edge</code>, <code>sourcePort</code>, or <code>targetPort</code> are not in this graph.
* @see IEdge#sourcePort
* @see IEdge#targetPort
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setEdgePorts(IEdge%2CIPort%2CIPort) Online Documentation}
*/
abstract setEdgePorts(edge: IEdge, sourcePort: IPort, targetPort: IPort): void;
/**
* Changes whether the given node is a group node or not.
* Group nodes are nodes which <i>can</i> have {@link IGraph#getChildren children}. They may not necessarily have to, however.
*
* Attempting to set a node to the non-group-node-status while it has children at the same time will result in an
* {@link Exception InvalidOperationError}.
*
* <br><b>How to set or query a node's group node status</b>
* <pre>
* // create a node in the graph
* const node1 = graph.createNode()
* // query whether it is a group node
* const node1IsGroupNode = graph.isGroupNode(node1) // false
*
* // create a group node in the graph
* const node2 = graph.createGroupNode()
* // query whether it is a group node
* const node2IsGroupNode = graph.isGroupNode(node2) // true
*
* // convert node1 into a group node
* graph.setIsGroupNode(node1, true)
* // convert node2 into a normal, i.e. non-group, node
* graph.setIsGroupNode(node2, false)
*
* // group is a group node with a child
* const group = graph.groupNodes([node1])
* // you cannot convert a group node with a child into a non-group node
* graph.setIsGroupNode(group, false) // throws exception!
* </pre>
*
* @param node The node to set the group node status for.
* @param isGroupNode Whether to make the node a group node.
* @throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph.
* @throws {Exception} <b>(InvalidOperationError)</b> <code>node</code> is <code>null</code> or currently has children and <code>isGroupNode</code> is <code>false</code>.
* @see IGraph#getChildren
* @see IGraph#isGroupNode
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setIsGroupNode(INode%2Cboolean) Online Documentation}
*/
abstract setIsGroupNode(node: INode | null, isGroupNode: boolean): void;
/**
* Sets the label model parameter for the given label.
* To query the label layout parameter, use {@link ILabel#layoutParameter}.
*
* <br><b>Setting the layout parameter of a label</b>
* <pre>
* graph.setLabelLayoutParameter(label, InteriorLabelModel.center)
* </pre>
*
* <br><b>Obtaining the layout parameter of a label</b>
* <pre>
* const parameter = label.layoutParameter
* </pre>
*
* @param label The label.
* @param layoutParameter The new parameter.
* @throws {Exception} <b>(ArgumentError)</b> <code>label</code> is not in this graph, or <code>layoutParameter</code> cannot be used for <code>label</code>.
* @see ILabel#layoutParameter
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setLabelLayoutParameter(ILabel%2CILabelModelParameter) Online Documentation}
*/
abstract setLabelLayoutParameter(label: ILabel, layoutParameter: ILabelModelParameter): void;
/**
* Sets the preferred size of the label.
* To query the label preferred size, use {@link ILabel#preferredSize}.
*
* <br><b>Setting the preferred size of a label</b>
* <pre>
* graph.setLabelPreferredSize(label, new Size(width, height))
* </pre>
*
* <br><b>Obtaining the preferred size of a label</b>
* <pre>
* const size = label.preferredSize
* </pre>
*
* @param label The label.
* @param preferredSize The new preferred size.
* @throws {Exception} <b>(ArgumentError)</b> <code>label</code> is not in this graph or <code>preferredSize</code> contains one or more NaN values.
* @see ILabel#preferredSize
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setLabelPreferredSize(ILabel%2CSize) Online Documentation}
*/
abstract setLabelPreferredSize(label: ILabel, preferredSize: Size): void;
/**
* Sets the label text of the given label.
* To query the label text, use {@link ILabel#text}.
*
* <br><b>Setting the text of a label</b>
* <pre>
* graph.setLabelText(label, 'Some Text')
* </pre>
*
* <br><b>Obtaining the text of a label</b>
* <pre>
* const text = label.text
* </pre>
*
* @param label the label to modify
* @param text the new text of the label
* @throws {Exception} <b>(ArgumentError)</b> <code>label</code> is not in this graph.
* @see ILabel#text
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setLabelText(ILabel%2Cstring) Online Documentation}
*/
abstract setLabelText(label: ILabel, text: string): void;
/**
Sets the center of a node to the given world coordinates.
This implementation delegates to {@link IGraph#setNodeLayout}
@param node The node to recenter.
@param center The new center coordinates of the node in the world coordinate system.
@throws {Exception} <b>(ArgumentError)</b> <code>center</code> contains one or more NaN values.
@see IGraph#setNodeLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setNodeCenter(INode%2CPoint) Online Documentation}*/
setNodeCenter(node: INode, center: Point): void;
/**
* Sets the layout of the given node to the new value.
* To query the node layout, use {@link INode#layout}. To place the node center at an absolute location, you can use the {@link IGraph#setNodeCenter}
* convenience method.
*
* <br><b>Setting the layout of a node</b>
* <pre>
* graph.setNodeLayout(node, new Rect(x, y, width, height))
* </pre>
*
* <br><b>Obtaining the layout of a node</b>
* <pre>
* const layout = node.layout
* </pre>
*
* @param node a live node that belongs to this graph
* @param layout The new layout of the node to assign to its {@link INode#layout}.
* @throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph, or <code>layout</code> contains one or more NaN values.
* @see INode#layout
* @see INode#layout
* @see IGraph#setNodeCenter
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setNodeLayout(INode%2CRect) Online Documentation}
*/
abstract setNodeLayout(node: INode, layout: Rect): void;
/**
* Sets the parent node for a given node.
* Use <code>null</code> as <code>parent</code> to make <code>node</code> a top-level node for this graph.
*
* This method does not move or enlarge the <code>parent</code> to enclose its new <code>node</code>. Developers have to take care to adjust the node
* layout after grouping, e.g. by calling {@link IGraph#adjustGroupNodeLayout}.
*
* If <code>parent</code> is not a {@link IGraph#isGroupNode group node} before the call it will be converted into one.
*
* To query the parent of a node, use {@link IGraph#getParent}.
*
* <pre>
* // create a group node
* const groupNode = graph.createGroupNode()
*
* // set groupNode as parent of node1
* graph.setParent(node1, groupNode)
*
* // get the parent of a node
* const node1Parent = graph.getParent(node1) // groupNode
* const groupParent = graph.getParent(groupNode) // null
* </pre>
*
* @param node The node to assign a new parent.
* @param parent The parent group node to assign to <code>node</code> or <code>null</code> to make <code>node</code> a top-level node.
* @throws {Exception} <b>(ArgumentError)</b> Either <code>node</code> or <code>parent</code> are not in this graph.
* @see IGraph#getParent
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setParent(INode%2CINode) Online Documentation}
*/
abstract setParent(node: INode, parent: INode | null): void;
/**
Convenience method that tries to set the absolute coordinates of the given port to the given values.
For full control over the placement of the ports, the {@link IGraph#setPortLocationParameter} method should be used
instead. This implementation will use the port's {@link IPort#locationParameter}'s {@link IPortLocationModelParameter#model} to obtain a new {@link IPortLocationModelParameter} via the {@link IPortLocationModel#createParameter}
method. This might result in the port using a different location, because the model might not support parameters that
result in the given location. This will also trigger an {@link IGraph#invalidateDisplays} call.
@param port The port to modify
@param location the new absolute coordinates of the port
@throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph.
@throws {Exception} <b>(ArgumentError)</b> <code>location</code> contains one or more NaN values.
@see IGraph#addPort
@see IGraph#setPortLocationParameter
@see IGraph#setRelativePortLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setPortLocation(IPort%2CPoint) Online Documentation}*/
setPortLocation(port: IPort, location: Point): void;
/**
* Sets a new {@link IPortLocationModelParameter} for the given port.
* To query the port location parameter, use {@link IPort#locationParameter}. To place the port at an absolute resp. relative location, you can use the {@link IGraph#setPortLocation}
* resp. {@link IGraph#setRelativePortLocation} convenience methods.
*
* <br><b>Setting the location parameter to a port</b>
* <pre>
* graph.setPortLocationParameter(port, FreeNodePortLocationModel.NODE_CENTER_ANCHORED)
* </pre>
*
* <br><b>Obtaining the location parameter from a port</b>
* <pre>
* const parameter = port.locationParameter
* </pre>
*
* @param port The port to modify
* @param locationParameter the new parameter that determines the coordinates of the port
* @throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph, or <code>locationParameter</code> cannot be used for <code>port</code>.
* @see IPort#locationParameter
* @see IGraph#setPortLocation
* @see IGraph#setRelativePortLocation
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setPortLocationParameter(IPort%2CIPortLocationModelParameter) Online Documentation}
*/
abstract setPortLocationParameter(port: IPort, locationParameter: IPortLocationModelParameter): void;
/**
Tries to set the location of the port relative to its {@link IPort#owner owner} if the owner is a {@link INode node}.
If the port is not owned by a node that is part of this graph, this method will throw an {@link Exception ArgumentError}. This method will
delegate to {@link IGraph#setPortLocation}.
@param port the port
@param relativeLocation the new coordinate offsets relative to the center of the node's {@link INode#layout}'s center.
@throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph or has no owner.
@throws {Exception} <b>(ArgumentError)</b> <code>relativeLocation</code> contains one or more NaN values.
@see INode#layout
@see IGraph#setRelativePortLocation
@see IGraph#setPortLocationParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setRelativePortLocation(IPort%2CPoint) Online Documentation}*/
setRelativePortLocation(port: IPort, relativeLocation: Point): void;
/**
* Assigns the given style instance by reference to the port.
* Style instances can be shared.
*
* To query the port style, use {@link IPort#style}.
*
* <br><b>Setting the style of a port</b>
* <pre>
* graph.setStyle(port, portStyle)
* </pre>
*
* <br><b>Obtaining the style of a port</b>
* <pre>
* const style = port.style
* </pre>
*
* @param port The port that will be assigned the new style
* @param style The style instance that will be assigned to the port.
* @throws {Exception} <b>(ArgumentError)</b> <code>port</code> is not in this graph.
* @see IPort#style
* @see IGraph#addPortStyleChangedListener
* @see VoidPortStyle#INSTANCE
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setStyle(IPort%2CIPortStyle) Online Documentation}
*/
abstract setStyle(port: IPort, style: IPortStyle): void;
/**
* Assigns the given style instance by reference to the edge.
* Style instances can be shared.
*
* To query the edge style, use {@link IEdge#style}.
*
* <br><b>Setting the style of an edge</b>
* <pre>
* graph.setStyle(edge, new PolylineEdgeStyle())
* </pre>
*
* <br><b>Obtaining the style of an edge</b>
* <pre>
* const style = edge.style
* </pre>
*
* @param edge The edge that will be assigned the new style
* @param style The style instance that will be assigned to the edge.
* @throws {Exception} <b>(ArgumentError)</b> <code>edge</code> is not in this graph.
* @see IEdge#style
* @see IGraph#addEdgeStyleChangedListener
* @see VoidEdgeStyle#INSTANCE
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setStyle(IEdge%2CIEdgeStyle) Online Documentation}
*/
abstract setStyle(edge: IEdge, style: IEdgeStyle): void;
/**
* Assigns the given style instance by reference to the label.
* Style instances can be shared.
*
* To query the label style, use {@link ILabel#style}.
*
* <br><b>Setting the style of a label</b>
* <pre>
* graph.setStyle(label, new DefaultLabelStyle())
* </pre>
*
* <br><b>Obtaining the style of a label</b>
* <pre>
* const style = label.style
* </pre>
*
* @param label The label that will be assigned the new style
* @param style The style instance that will be assigned to the label.
* @throws {Exception} <b>(ArgumentError)</b> <code>label</code> is not in this graph.
* @see ILabel#style
* @see IGraph#addLabelStyleChangedListener
* @see VoidLabelStyle#INSTANCE
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setStyle(ILabel%2CILabelStyle) Online Documentation}
*/
abstract setStyle(label: ILabel, style: ILabelStyle): void;
/**
* Assigns the given style instance by reference to the node.
* Style instances can be shared.
*
* To query the node style, use {@link INode#style}.
*
* <br><b>Setting the style of a node</b>
* <pre>
* graph.setStyle(node, new ShapeNodeStyle())
* </pre>
*
* <br><b>Obtaining the style of a node</b>
* <pre>
* const style = node.style
* </pre>
*
* @param node The node that will be assigned the new style
* @param style The style instance that will be assigned to the node.
* @throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph.
* @see INode#style
* @see IGraph#addNodeStyleChangedListener
* @see VoidNodeStyle#INSTANCE
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23setStyle(INode%2CINodeStyle) Online Documentation}
*/
abstract setStyle(node: INode, style: INodeStyle): void;
/**
Enumerates the successors of a given {@link INode}.
Successors are calculated by going through all {@link IPortOwner#ports} and inspecting the {@link IGraph#outEdgesAt outgoing edges at these ports}, collecting the {@link IEdge#targetPort} {@link IPort#owner}s.
@param node The node.
@returns An enumerable over all successors.
@throws {Exception} <b>(ArgumentError)</b> <code>node</code> is not in this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23successors(INode) Online Documentation}*/
successors(node: INode): IEnumerable<INode>;
/**
* Gets a view of the nodes contained in this graph.
* This is a live view of the nodes that always represents the current state of the graph. The same reference will be
* returned for each invocation.
*
* Note that even though nodes can be accessed via index, the underlying graph structure in the default {@link IGraph}
* implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the nodes in
* your own list, if possible. This is not necessary for the first or last element or when iterating over the nodes via a <code>foreach</code>
* loop.
*
* <br><b>Iterating over all nodes in the graph</b>
* <pre>
* for (const node of graph.nodes) {
*   // ...
* }
* </pre>
*
* Note that <code>Nodes</code> is an {@link IListEnumerable} which extends {@link IEnumerable}.
*
* <br><b>Iterating over the blue nodes in the graph</b>
* <pre>
* for (const node of graph.nodes.filter(
*   n => n.style instanceof ShapeNodeStyle && n.style.fill === Fill.BLUE
* )) {
*   // ...
* }
*
* graph.nodes
*   .filter(n => n.style instanceof ShapeNodeStyle && n.style.fill === Fill.BLUE)
*   .forEach(node => {
*     // ...
*   })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23nodes Online Documentation}
*/
abstract readonly nodes: IListEnumerable<INode>;
/**
* Gets a view of the edges contained in this graph.
* This is a live view of the edges that always represents the current state of the graph. The same reference will be
* returned for each invocation.
*
* Note that even though edges can be accessed via index, the underlying graph structure in the default {@link IGraph}
* implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
* your own list, if possible. This is not necessary for the first or last element or when iterating over the edges via a <code>foreach</code>
* loop.
*
* <br><b>Iterating over all edges in the graph</b>
* <pre>
* for (const edge of graph.edges) {
*   // ...
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23edges Online Documentation}
*/
abstract readonly edges: IListEnumerable<IEdge>;
/**
* Gets a view of the labels contained in this graph.
* This is a live view of the labels that always represents the current state of the graph. The same reference will be
* returned for each invocation.
*
* Note that even though labels can be accessed via index, the underlying graph structure in the default {@link IGraph}
* implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the labels in
* your own list, if possible. This is not necessary for the first or last element or when iterating over the labels via a
* <code>foreach</code> loop.
*
* <code>Labels</code> provides a view of all labels in the graph.
*
* <pre>
* for (const label of graph.labels) {
*   // ...
* }
* </pre>
*
* To retrieve only labels at nodes on can either filter <code>Labels</code> or use method {@link IGraph#nodeLabels}. The following examples are equivalent:
*
* <pre>
* const nodeLabels = graph.labels.filter(l => INode.isInstance(l.owner)).toList()
* const nodeLabels2 = graph.nodeLabels.toList()
* </pre>
*
* The same applies for labels at edges and ports:
*
* <pre>
* const edgeLabels = graph.labels.filter(l => IEdge.isInstance(l.owner)).toList()
* const edgeLabels2 = graph.edgeLabels.toList()
* </pre>
*
* <pre>
* const portLabels = graph.labels.filter(l => IPort.isInstance(l.owner)).toList()
* const portLabels2 = graph.portLabels.toList()
* </pre>
*
* The following example shows how to get a list of all nodes with labels:
*
* <pre>
* const labeledNodes = graph.nodeLabels.map(l => l.owner).toList()
* </pre>
*
* @see IGraph#nodeLabels
* @see IGraph#edgeLabels
* @see IGraph#portLabels
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23labels Online Documentation}
*/
abstract readonly labels: IListEnumerable<ILabel>;
/**
* Gets a view of the ports contained in this graph.
* This is a live view of the ports that always represents the current state of the graph. The same reference will be
* returned for each invocation.
*
* Note that even though ports can be accessed via index, the underlying graph structure in the default {@link IGraph}
* implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the ports in
* your own list, if possible. This is not necessary for the first or last element or when iterating over the ports via a <code>foreach</code>
* loop.
*
* <br><b>Iterating over all ports in the graph</b>
* <pre>
* for (const port of graph.ports) {
*   // ...
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23ports Online Documentation}
*/
abstract readonly ports: IListEnumerable<IPort>;
/**
* Gets or sets the defaults for normal nodes.
* The settings that are obtained from the instance influence newly created elements only. Setting different defaults
* afterwards does not influence existing elements.
*
* The {@link IGraph} implementations provided by yFiles for HTML always provide a pre-configured instance in this
* property. New defaults can be set by changing its properties.
*
* <br><b>Setting some defaults for newly created nodes</b>
* <pre>
* // the defaults for nodes can be set on the INodeDefaults instance
* // which can be retrieved from the graph's node defaults property
* graph.nodeDefaults.size = new Size(50, 30)
* graph.nodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.ELLIPSE,
*   fill: Fill.YELLOW
* })
*
* // the defaults for node labels can be set on the ILabelDefaults instance
* // found at the Labels property of the node defaults
* graph.nodeDefaults.labels.layoutParameter = InteriorLabelModel.CENTER
* graph.nodeDefaults.labels.style = new DefaultLabelStyle()
*
* // the defaults for node ports can be set on the IPortDefaults instance
* // found at the Ports property of the node defaults
* graph.nodeDefaults.ports.autoCleanUp = false
* graph.nodeDefaults.ports.locationParameter = FreeNodePortLocationModel.NODE_CENTER_ANCHORED
* </pre>
*
* It is possible to set a new {@link INodeDefaults} instance here, as well. That way, it is possible to share the
* same defaults for group nodes and normal nodes:
*
* <br><b>Sharing node defaults</b>
* <pre>
* // use the same defaults for nodes and group nodes
* graph.groupNodeDefaults = graph.nodeDefaults
* graph.nodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.ELLIPSE,
*   fill: Fill.YELLOW
* })
* </pre>
*
* or to share only the label defaults:
*
* <br><b>Sharing label defaults only</b>
* <pre>
* // use the different defaults for nodes and group nodes
* graph.nodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.ELLIPSE,
*   fill: Fill.YELLOW
* })
* graph.groupNodeDefaults.style = new PanelNodeStyle({ color: Color.LIGHT_BLUE })
* // but use the same label defaults
* graph.groupNodeDefaults.labels = graph.nodeDefaults.labels
* graph.nodeDefaults.labels.style = new DefaultLabelStyle({ backgroundFill: Fill.DARK_GRAY })
* graph.nodeDefaults.labels.layoutParameter = ExteriorLabelModel.SOUTH
* </pre>
*
* or to keep pre-configured defaults:
*
* <br><b>Providing different defaults</b>
* <pre>
* // region NodeLabelDefaults
*
* // place node labels on the node center by default
* graph.nodeDefaults.labels.layoutParameter = InteriorLabelModel.CENTER
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23nodeDefaults Online Documentation}
*/
abstract nodeDefaults: INodeDefaults;
/**
* Gets or sets the defaults for edges.
* The settings that are obtained from the instance influence newly created elements only. Setting different defaults
* afterwards does not influence existing elements.
*
* <br><b>Setting some defaults for newly created edges</b>
* <pre>
* // the defaults for edges can be set on the IEdgeDefaults instance
* // which can be retrieved from the graph's EdgeDefaults property
* graph.edgeDefaults.style = new PolylineEdgeStyle({
*   stroke: Stroke.BLACK,
*   targetArrow: Arrow.DEFAULT
* })
*
* // the defaults for edge labels can be set on the ILabelDefaults instance
* // found at the Labels property of the edge defaults
* graph.edgeDefaults.labels.layoutParameter = new EdgeSegmentLabelModel().createDefaultParameter()
* graph.edgeDefaults.labels.style = new DefaultLabelStyle()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23edgeDefaults Online Documentation}
*/
abstract edgeDefaults: IEdgeDefaults;
/**
Gets the mapper registry that is associated with this graph instance.
The registry can be used to store data mappings for the items in this instance.
@see IMapperRegistry
@see IModelItem
@see ITagOwner#tag
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23mapperRegistry Online Documentation}*/
abstract readonly mapperRegistry: IMapperRegistry;
/**
Adds the given listener for the <code>GraphTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of the graph has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see ITagOwner#tag
@see IGraph#removeGraphTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23GraphTagChanged Online Documentation}*/
abstract addGraphTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IGraph, any>) => void): void;
/**
Removes the given listener for the <code>GraphTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of the graph has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see ITagOwner#tag
@see IGraph#addGraphTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23GraphTagChanged Online Documentation}*/
abstract removeGraphTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IGraph, any>) => void): void;
/**
Adds the given listener for the <code>NodeCreated</code> event that occurs when a node has been created.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addNodeCreatedListener NodeCreated}
event if you are interested only in node creation events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the node creation
that has triggered this event.
@param listener The listener to add.
@see IGraph#createNode
@see IGraph#createNode
@see IGraph#createGroupNode
@see GraphEditorInputMode#addNodeCreatedListener
@see IGraph#removeNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeCreated Online Documentation}*/
abstract addNodeCreatedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Removes the given listener for the <code>NodeCreated</code> event that occurs when a node has been created.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addNodeCreatedListener NodeCreated}
event if you are interested only in node creation events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the node creation
that has triggered this event.
@param listener The listener to remove.
@see IGraph#createNode
@see IGraph#createNode
@see IGraph#createGroupNode
@see GraphEditorInputMode#addNodeCreatedListener
@see IGraph#addNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeCreated Online Documentation}*/
abstract removeNodeCreatedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
Adds the given listener for the <code>DisplaysInvalidated</code> event that occurs when the graph has changed visually and the
display should be updated to reflect the changes.
This event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to add.
@see IGraph#removeDisplaysInvalidatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23DisplaysInvalidated Online Documentation}*/
abstract addDisplaysInvalidatedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>DisplaysInvalidated</code> event that occurs when the graph has changed visually and the
display should be updated to reflect the changes.
This event is invoked with {@link EventArgs#EMPTY} per default.
@param listener The listener to remove.
@see IGraph#addDisplaysInvalidatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23DisplaysInvalidated Online Documentation}*/
abstract removeDisplaysInvalidatedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in node removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the node removal
that has triggered this event.
@param listener The listener to add.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see IGraph#removeNodeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeRemoved Online Documentation}*/
abstract addNodeRemovedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
Removes the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in node removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the node removal
that has triggered this event.
@param listener The listener to remove.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see IGraph#addNodeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeRemoved Online Documentation}*/
abstract removeNodeRemovedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
Adds the given listener for the <code>NodeStyleChanged</code> event that occurs when a node style has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setStyle
@see IGraph#removeNodeStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeStyleChanged Online Documentation}*/
abstract addNodeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, INodeStyle>) => void): void;
/**
Removes the given listener for the <code>NodeStyleChanged</code> event that occurs when a node style has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setStyle
@see IGraph#addNodeStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeStyleChanged Online Documentation}*/
abstract removeNodeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, INodeStyle>) => void): void;
/**
Adds the given listener for the <code>NodeLayoutChanged</code> event that occurs when a node layout has been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setNodeLayout
@see IGraph#removeNodeLayoutChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeLayoutChanged Online Documentation}*/
abstract addNodeLayoutChangedListener(listener: (source: any, node: INode, oldLayout: Rect) => void): void;
/**
Removes the given listener for the <code>NodeLayoutChanged</code> event that occurs when a node layout has been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setNodeLayout
@see IGraph#addNodeLayoutChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeLayoutChanged Online Documentation}*/
abstract removeNodeLayoutChangedListener(listener: (source: any, node: INode, oldLayout: Rect) => void): void;
/**
Adds the given listener for the <code>NodeTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of a node has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see ITagOwner#tag
@see IGraph#removeNodeTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeTagChanged Online Documentation}*/
abstract addNodeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of a node has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see ITagOwner#tag
@see IGraph#addNodeTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23NodeTagChanged Online Documentation}*/
abstract removeNodeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link CreateEdgeInputMode#addEdgeCreatedListener EdgeCreated}
event if you are interested only in edge creation events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the edge creation
that has triggered this event.
@param listener The listener to add.
@see IGraph#createEdge
@see CreateEdgeInputMode#addEdgeCreatedListener
@see IGraph#removeEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgeCreated Online Documentation}*/
abstract addEdgeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IEdge>) => void): void;
/**
Removes the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link CreateEdgeInputMode#addEdgeCreatedListener EdgeCreated}
event if you are interested only in edge creation events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the edge creation
that has triggered this event.
@param listener The listener to remove.
@see IGraph#createEdge
@see CreateEdgeInputMode#addEdgeCreatedListener
@see IGraph#addEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgeCreated Online Documentation}*/
abstract removeEdgeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IEdge>) => void): void;
/**
Adds the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
This event will be triggered, too, prior to a node removal.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in edge removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the edge removal
that has triggered this event.
@param listener The listener to add.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see IGraph#removeEdgeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgeRemoved Online Documentation}*/
abstract addEdgeRemovedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
Removes the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
This event will be triggered, too, prior to a node removal.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in edge removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the edge removal
that has triggered this event.
@param listener The listener to remove.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see IGraph#addEdgeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgeRemoved Online Documentation}*/
abstract removeEdgeRemovedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
Adds the given listener for the <code>EdgeStyleChanged</code> event that occurs when an edge style has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setStyle
@see IGraph#removeEdgeStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgeStyleChanged Online Documentation}*/
abstract addEdgeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, IEdgeStyle>) => void): void;
/**
Removes the given listener for the <code>EdgeStyleChanged</code> event that occurs when an edge style has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setStyle
@see IGraph#addEdgeStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgeStyleChanged Online Documentation}*/
abstract removeEdgeStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, IEdgeStyle>) => void): void;
/**
Adds the given listener for the <code>EdgePortsChanged</code> event that occurs when an edge had its {@link IEdge#sourcePort} or {@link IEdge#targetPort} {@link IGraph#setEdgePorts changed}.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setEdgePorts
@see IGraph#removeEdgePortsChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgePortsChanged Online Documentation}*/
abstract addEdgePortsChangedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
Removes the given listener for the <code>EdgePortsChanged</code> event that occurs when an edge had its {@link IEdge#sourcePort} or {@link IEdge#targetPort} {@link IGraph#setEdgePorts changed}.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setEdgePorts
@see IGraph#addEdgePortsChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgePortsChanged Online Documentation}*/
abstract removeEdgePortsChangedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
Adds the given listener for the <code>EdgeTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of an edge has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see ITagOwner#tag
@see IGraph#removeEdgeTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgeTagChanged Online Documentation}*/
abstract addEdgeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of an edge has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see ITagOwner#tag
@see IGraph#addEdgeTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23EdgeTagChanged Online Documentation}*/
abstract removeEdgeTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>BendAdded</code> event that occurs when a bend has been added to an edge in this graph.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link CreateBendInputMode#addBendCreatedListener BendCreated}
event if you are interested only in bend creation events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the bend creation
that has triggered this event.
@param listener The listener to add.
@see IGraph#addBend
@see CreateBendInputMode#addBendCreatedListener
@see IGraph#removeBendAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23BendAdded Online Documentation}*/
abstract addBendAddedListener(listener: (sender: any, evt: ItemEventArgs<IBend>) => void): void;
/**
Removes the given listener for the <code>BendAdded</code> event that occurs when a bend has been added to an edge in this graph.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link CreateBendInputMode#addBendCreatedListener BendCreated}
event if you are interested only in bend creation events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the bend creation
that has triggered this event.
@param listener The listener to remove.
@see IGraph#addBend
@see CreateBendInputMode#addBendCreatedListener
@see IGraph#addBendAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23BendAdded Online Documentation}*/
abstract removeBendAddedListener(listener: (sender: any, evt: ItemEventArgs<IBend>) => void): void;
/**
Adds the given listener for the <code>BendRemoved</code> event that occurs when a bend has been removed from an edge in this graph.
This event will be triggered, too, if an edge has been removed from the graph, for each of the bends that belonged to
the edge.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in bend removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the bend removal
that has triggered this event.
@param listener The listener to add.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see IGraph#removeBendRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23BendRemoved Online Documentation}*/
abstract addBendRemovedListener(listener: (sender: any, evt: BendEventArgs) => void): void;
/**
Removes the given listener for the <code>BendRemoved</code> event that occurs when a bend has been removed from an edge in this
graph.
This event will be triggered, too, if an edge has been removed from the graph, for each of the bends that belonged to
the edge.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in bend removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the bend removal
that has triggered this event.
@param listener The listener to remove.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see IGraph#addBendRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23BendRemoved Online Documentation}*/
abstract removeBendRemovedListener(listener: (sender: any, evt: BendEventArgs) => void): void;
/**
Adds the given listener for the <code>BendLocationChanged</code> event that occurs when the location of a bend has been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setBendLocation
@see IGraph#removeBendLocationChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23BendLocationChanged Online Documentation}*/
abstract addBendLocationChangedListener(listener: (source: any, bend: IBend, oldLocation: Point) => void): void;
/**
Removes the given listener for the <code>BendLocationChanged</code> event that occurs when the location of a bend has been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setBendLocation
@see IGraph#addBendLocationChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23BendLocationChanged Online Documentation}*/
abstract removeBendLocationChangedListener(listener: (source: any, bend: IBend, oldLocation: Point) => void): void;
/**
Adds the given listener for the <code>BendTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of a bend has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see ITagOwner#tag
@see IGraph#removeBendTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23BendTagChanged Online Documentation}*/
abstract addBendTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IBend, any>) => void): void;
/**
Removes the given listener for the <code>BendTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of a bend has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see ITagOwner#tag
@see IGraph#addBendTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23BendTagChanged Online Documentation}*/
abstract removeBendTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IBend, any>) => void): void;
/**
Adds the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to this graph instance.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addLabelAddedListener LabelAdded}
event if you are interested only in label creation events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the label creation
that has triggered this event.
@param listener The listener to add.
@see IGraph#addLabel
@see GraphEditorInputMode#addLabelAddedListener
@see IGraph#removeLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelAdded Online Documentation}*/
abstract addLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Removes the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to this graph instance.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addLabelAddedListener LabelAdded}
event if you are interested only in label creation events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the label creation
that has triggered this event.
@param listener The listener to remove.
@see IGraph#addLabel
@see GraphEditorInputMode#addLabelAddedListener
@see IGraph#addLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelAdded Online Documentation}*/
abstract removeLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Adds the given listener for the <code>LabelRemoved</code> event that occurs when a label has been removed from this graph instance.
This event will also be triggered, prior to the removal of the {@link ILabel#owner owner} of the label.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in label removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the label removal
that has triggered this event.
@param listener The listener to add.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see GraphEditorInputMode#addLabelTextChangedListener
@see IGraph#removeLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelRemoved Online Documentation}*/
abstract addLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelRemoved</code> event that occurs when a label has been removed from this graph
instance.
This event will also be triggered, prior to the removal of the {@link ILabel#owner owner} of the label.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in label removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the label removal
that has triggered this event.
@param listener The listener to remove.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see GraphEditorInputMode#addLabelTextChangedListener
@see IGraph#addLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelRemoved Online Documentation}*/
abstract removeLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Adds the given listener for the <code>LabelStyleChanged</code> event that occurs when a label style has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setStyle
@see IGraph#removeLabelStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelStyleChanged Online Documentation}*/
abstract addLabelStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelStyle>) => void): void;
/**
Removes the given listener for the <code>LabelStyleChanged</code> event that occurs when a label style has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setStyle
@see IGraph#addLabelStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelStyleChanged Online Documentation}*/
abstract removeLabelStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelStyle>) => void): void;
/**
Adds the given listener for the <code>LabelPreferredSizeChanged</code> event that occurs when the preferred size of a label has been
changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setLabelPreferredSize
@see IGraph#removeLabelPreferredSizeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelPreferredSizeChanged Online Documentation}*/
abstract addLabelPreferredSizeChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, Size>) => void): void;
/**
Removes the given listener for the <code>LabelPreferredSizeChanged</code> event that occurs when the preferred size of a label has
been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setLabelPreferredSize
@see IGraph#addLabelPreferredSizeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelPreferredSizeChanged Online Documentation}*/
abstract removeLabelPreferredSizeChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, Size>) => void): void;
/**
Adds the given listener for the <code>LabelTextChanged</code> event that occurs when the text of a label has been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setLabelText
@see IGraph#removeLabelTextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelTextChanged Online Documentation}*/
abstract addLabelTextChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, string>) => void): void;
/**
Removes the given listener for the <code>LabelTextChanged</code> event that occurs when the text of a label has been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setLabelText
@see IGraph#addLabelTextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelTextChanged Online Documentation}*/
abstract removeLabelTextChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, string>) => void): void;
/**
Adds the given listener for the <code>LabelLayoutParameterChanged</code> event that occurs when the model parameter of a label has
been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setLabelLayoutParameter
@see IGraph#removeLabelLayoutParameterChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelLayoutParameterChanged Online Documentation}*/
abstract addLabelLayoutParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelModelParameter>) => void): void;
/**
Removes the given listener for the <code>LabelLayoutParameterChanged</code> event that occurs when the model parameter of a label has
been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setLabelLayoutParameter
@see IGraph#addLabelLayoutParameterChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelLayoutParameterChanged Online Documentation}*/
abstract removeLabelLayoutParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, ILabelModelParameter>) => void): void;
/**
Adds the given listener for the <code>LabelTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of a label has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see ITagOwner#tag
@see IGraph#removeLabelTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelTagChanged Online Documentation}*/
abstract addLabelTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of a label has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see ITagOwner#tag
@see IGraph#addLabelTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23LabelTagChanged Online Documentation}*/
abstract removeLabelTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<ILabel, any>) => void): void;
/**
Adds the given listener for the <code>PortAdded</code> event that occurs when a port has been added to this graph instance.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the port creation
that has triggered this event.
@param listener The listener to add.
@see IGraph#addPort
@see IGraph#createEdge
@see IGraph#removePortAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortAdded Online Documentation}*/
abstract addPortAddedListener(listener: (sender: any, evt: ItemEventArgs<IPort>) => void): void;
/**
Removes the given listener for the <code>PortAdded</code> event that occurs when a port has been added to this graph instance.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the port creation
that has triggered this event.
@param listener The listener to remove.
@see IGraph#addPort
@see IGraph#createEdge
@see IGraph#addPortAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortAdded Online Documentation}*/
abstract removePortAddedListener(listener: (sender: any, evt: ItemEventArgs<IPort>) => void): void;
/**
Adds the given listener for the <code>PortRemoved</code> event that occurs when a port has been removed from its {@link IPort#owner owner}.
This event will also be triggered prior to the removal of the corresponding owner of the port.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in port removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the port removal
that has triggered this event.
@param listener The listener to add.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see IGraph#removePortRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortRemoved Online Documentation}*/
abstract addPortRemovedListener(listener: (sender: any, evt: PortEventArgs) => void): void;
/**
Removes the given listener for the <code>PortRemoved</code> event that occurs when a port has been removed from its {@link IPort#owner owner}.
This event will also be triggered prior to the removal of the corresponding owner of the port.
This event is intended to provide notification of low level changes in the graph structure. Please use the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
event if you are interested only in port removal events that result from user interaction.
<b>Warning:</b>
You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the port removal
that has triggered this event.
@param listener The listener to remove.
@see IGraph#remove
@see GraphEditorInputMode#addDeletedItemListener
@see IGraph#addPortRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortRemoved Online Documentation}*/
abstract removePortRemovedListener(listener: (sender: any, evt: PortEventArgs) => void): void;
/**
Adds the given listener for the <code>PortStyleChanged</code> event that occurs when a port style has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setStyle
@see IGraph#removePortStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortStyleChanged Online Documentation}*/
abstract addPortStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortStyle>) => void): void;
/**
Removes the given listener for the <code>PortStyleChanged</code> event that occurs when a port style has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setStyle
@see IGraph#addPortStyleChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortStyleChanged Online Documentation}*/
abstract removePortStyleChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortStyle>) => void): void;
/**
Adds the given listener for the <code>PortLocationParameterChanged</code> event that occurs when the location model parameter of a
port has been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#setPortLocationParameter
@see IGraph#removePortLocationParameterChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortLocationParameterChanged Online Documentation}*/
abstract addPortLocationParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortLocationModelParameter>) => void): void;
/**
Removes the given listener for the <code>PortLocationParameterChanged</code> event that occurs when the location model parameter of a
port has been changed.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#setPortLocationParameter
@see IGraph#addPortLocationParameterChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortLocationParameterChanged Online Documentation}*/
abstract removePortLocationParameterChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, IPortLocationModelParameter>) => void): void;
/**
Adds the given listener for the <code>PortTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of a port has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see ITagOwner#tag
@see IGraph#removePortTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortTagChanged Online Documentation}*/
abstract addPortTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, any>) => void): void;
/**
Removes the given listener for the <code>PortTagChanged</code> event that occurs when the {@link ITagOwner#tag tag} of a port has been replaced.
This event is intended to provide notification of low level changes in the graph structure.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see ITagOwner#tag
@see IGraph#addPortTagChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23PortTagChanged Online Documentation}*/
abstract removePortTagChangedListener(listener: (sender: any, evt: ItemChangedEventArgs<IPort, any>) => void): void;
/**
* Gets or sets the defaults for group nodes.
* The settings that are obtained from the instance influence newly created elements only. Setting different defaults
* afterwards does not influence existing elements.
*
* <br><b>Defaults for newly created group nodes</b>
* <pre>
* graph.groupNodeDefaults.style = new PanelNodeStyle()
* graph.groupNodeDefaults.labels.layoutParameter = InteriorLabelModel.NORTH
* </pre>
*
* It is possible to set a new {@link INodeDefaults} instance here, as well. That way, it is possible to share the
* same defaults for group nodes and normal nodes:
*
* <br><b>Sharing node defaults</b>
* <pre>
* // use the same defaults for nodes and group nodes
* graph.groupNodeDefaults = graph.nodeDefaults
* graph.nodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.ELLIPSE,
*   fill: Fill.YELLOW
* })
* </pre>
*
* or to share only the label defaults:
*
* <br><b>Sharing label defaults only</b>
* <pre>
* // use the different defaults for nodes and group nodes
* graph.nodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.ELLIPSE,
*   fill: Fill.YELLOW
* })
* graph.groupNodeDefaults.style = new PanelNodeStyle({ color: Color.LIGHT_BLUE })
* // but use the same label defaults
* graph.groupNodeDefaults.labels = graph.nodeDefaults.labels
* graph.nodeDefaults.labels.style = new DefaultLabelStyle({ backgroundFill: Fill.DARK_GRAY })
* graph.nodeDefaults.labels.layoutParameter = ExteriorLabelModel.SOUTH
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23groupNodeDefaults Online Documentation}
*/
abstract groupNodeDefaults: INodeDefaults;
/**
Adds the given listener for the <code>ParentChanged</code> event that occurs if a node has been reparented in the model.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#removeParentChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23ParentChanged Online Documentation}*/
abstract addParentChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
Removes the given listener for the <code>ParentChanged</code> event that occurs if a node has been reparented in the model.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#addParentChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23ParentChanged Online Documentation}*/
abstract removeParentChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
Adds the given listener for the <code>IsGroupNodeChanged</code> event that occurs if the group node status of a node has changed.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to add.
@see IGraph#removeIsGroupNodeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23IsGroupNodeChanged Online Documentation}*/
abstract addIsGroupNodeChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
Removes the given listener for the <code>IsGroupNodeChanged</code> event that occurs if the group node status of a node has changed.
<b>Warning:</b> You may not modify the graph in the event handler for this event.
@param listener The listener to remove.
@see IGraph#addIsGroupNodeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23IsGroupNodeChanged Online Documentation}*/
abstract removeIsGroupNodeChangedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
Gets a {@link GraphDecorator} instance for use with this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23decorator Online Documentation}*/
readonly decorator: GraphDecorator;
/**
Creates a {@link GroupingSupport} instance for the graph.
{@link GroupingSupport} provides less frequently used methods for grouped graphs, such as methods for analyzing and
walking the grouping hierarchy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23groupingSupport Online Documentation}*/
readonly groupingSupport: GroupingSupport;
/**
Gets the {@link IFoldingView folding view} instance associated with this {@link IGraph} or <code>null</code> if none is associated with it.
This convenience method uses the {@link ILookup#lookup} of the {@link IGraph} to obtain the
{@link IFoldingView}.
@see FoldingManager
@see FoldingManager#createFoldingView
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23foldingView Online Documentation}*/
readonly foldingView: IFoldingView | null;
/**
Gets the {@link UndoEngine} instance associated with this {@link IGraph} or <code>null</code> if none is associated with
it.
This convenience method uses the {@link ILookup#lookup} of the {@link IGraph} to obtain the
{@link UndoEngine}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23undoEngine Online Documentation}*/
readonly undoEngine: UndoEngine | null;
/**
Gets or sets whether or not the {@link UndoEngine} used for this instance should be enabled.
@see IGraph#undoEngine
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23undoEngineEnabled Online Documentation}*/
undoEngineEnabled: boolean;
/**
Gets a live view of all bends contained in this {@link IGraph}.
@see IEdge#bends
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23bends Online Documentation}*/
readonly bends: IEnumerable<IBend>;
/**
Gets a live view of all edge labels contained in this {@link IGraph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23edgeLabels Online Documentation}*/
readonly edgeLabels: IEnumerable<ILabel>;
/**
Gets a live view of all node labels contained in this {@link IGraph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23nodeLabels Online Documentation}*/
readonly nodeLabels: IEnumerable<ILabel>;
/**
Gets a live view of all port labels contained in this {@link IGraph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraph%23portLabels Online Documentation}*/
readonly portLabels: IEnumerable<ILabel>;
static $class: Class;
private $_IGraph: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGraph;
}
export interface NodeEventArgs extends ItemEventArgs<INode> {
}
/**
A specialized subclass of the {@link ItemEventArgs} class that is bound to the {@link INode} type and carries additional information
about the parent node and the group node status.
Since for some events ({@link IGraph#addNodeRemovedListener NodeRemoved},
{@link IGraph#addIsGroupNodeChangedListener IsGroupNodeChanged},
{@link IGraph#addParentChangedListener ParentChanged}) it is unclear which {@link NodeEventArgs#parent} or {@link NodeEventArgs#isGroupNode} the node had before the event
this event can be used to carry exactly that information.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeEventArgs Online Documentation}*/
export class NodeEventArgs {
/**
Creates a new instance.
@param item The node that has changed.
@param oldParent The parent the node had before this event.
@param oldIsGroupNode Whether the node was a group node before this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeEventArgs Online Documentation}*/
constructor(item: INode, oldParent: INode | null, oldIsGroupNode: boolean);
/**
Gets the parent of the {@link ItemEventArgs#item node} before this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeEventArgs%23parent Online Documentation}*/
readonly parent: INode | null;
/**
Gets whether the {@link ItemEventArgs#item node} was a group node before this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeEventArgs%23isGroupNode Online Documentation}*/
readonly isGroupNode: boolean;
static $class: Class;
private $_NodeEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeEventArgs;
}
export interface EdgeEventArgs extends ItemEventArgs<IEdge> {
}
/**
A specialized subclass of the {@link ItemEventArgs} class that is bound to the {@link IEdge} type and carries additional source and
target port information.
Since for some events ({@link IGraph#addEdgePortsChangedListener EdgePortsChanged} and
{@link IGraph#addEdgeRemovedListener EdgeRemoved}) it is unclear what {@link IEdge#sourcePort} and {@link IEdge#targetPort} the edge had been connected to
before the event, this event can be used to carry exactly that information.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeEventArgs Online Documentation}*/
export class EdgeEventArgs {
/**
Initializes a new instance of the {@link EdgeEventArgs} class.
@param edge The edge to assign to the {@link ItemEventArgs#item} property.
@param [sourcePort=null] The source port the edge was connected to before the event happened.
@param [targetPort=null] The target port the edge was connected to before the event happened.
@param [sourcePortOwner=null] The owner of the source port the edge was connected to before the event happened.
@param [targetPortOwner=null] The owner of the target port the edge was connected to before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeEventArgs Online Documentation}*/
constructor(edge: IEdge, sourcePort?: IPort | null, targetPort?: IPort | null, sourcePortOwner?: IPortOwner | null, targetPortOwner?: IPortOwner | null);
/**
Initializes a new instance of the {@link EdgeEventArgs} class.
@param options The parameters to pass.
@param options.edge The edge to assign to the {@link ItemEventArgs#item} property.
@param [options.sourcePort=null] The source port the edge was connected to before the event happened.
@param [options.targetPort=null] The target port the edge was connected to before the event happened.
@param [options.sourcePortOwner=null] The owner of the source port the edge was connected to before the event happened.
@param [options.targetPortOwner=null] The owner of the target port the edge was connected to before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeEventArgs Online Documentation}*/
constructor(options: {
edge: IEdge;
sourcePort?: IPort | null;
targetPort?: IPort | null;
sourcePortOwner?: IPortOwner | null;
targetPortOwner?: IPortOwner | null;
});
/**
Gets the source port the edge was connected to before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeEventArgs%23sourcePort Online Documentation}*/
readonly sourcePort: IPort;
/**
Gets the target port the edge was connected to before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeEventArgs%23targetPort Online Documentation}*/
readonly targetPort: IPort;
/**
Gets the owner of the source port the edge was connected to before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeEventArgs%23sourcePortOwner Online Documentation}*/
readonly sourcePortOwner: IPortOwner;
/**
Gets the owner of the target port the edge was connected to before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeEventArgs%23targetPortOwner Online Documentation}*/
readonly targetPortOwner: IPortOwner;
static $class: Class;
private $_EdgeEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeEventArgs;
}
export interface PortEventArgs extends ItemEventArgs<IPort> {
}
/**
A specialized subclass of the {@link ItemEventArgs} class that is bound to the {@link IPort} type and carries additional {@link IPortOwner}
information.
Since for some events ({@link IGraph#addPortRemovedListener PortRemoved}) it is unclear what {@link IPort#owner} the port had
been attached to before the event, this event can be used to carry exactly that information.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortEventArgs Online Documentation}*/
export class PortEventArgs {
/**
Initializes a new instance of the {@link PortEventArgs} class.
@param port The port to assign to the {@link ItemEventArgs#item} property.
@param owner The owner of the port the port was attached to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortEventArgs Online Documentation}*/
constructor(port: IPort, owner: IPortOwner);
/**
Gets the owner the port the was connected to before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortEventArgs%23owner Online Documentation}*/
readonly owner: IPortOwner;
static $class: Class;
private $_PortEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortEventArgs;
}
export interface LabelEventArgs extends ItemEventArgs<ILabel> {
}
/**
A specialized subclass of the {@link ItemEventArgs} class that is bound to the {@link ILabel} type and carries additional {@link ILabelOwner owner}
information.
Since for some events ({@link IGraph#addLabelRemovedListener LabelRemoved}) it is unclear what {@link ILabel#owner} the label
was owned by before the event, this event can be used to carry exactly that information.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEventArgs Online Documentation}*/
export class LabelEventArgs {
/**
Initializes a new instance of the {@link LabelEventArgs} class.
@param label The label to assign to the {@link ItemEventArgs#item} property.
@param owner The previous owner of the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEventArgs Online Documentation}*/
constructor(label: ILabel, owner: ILabelOwner);
/**
Gets the owner of the label that owned the label before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelEventArgs%23owner Online Documentation}*/
readonly owner: ILabelOwner;
static $class: Class;
private $_LabelEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelEventArgs;
}
export interface BendEventArgs extends ItemEventArgs<IBend> {
}
/**
A specialized subclass of the {@link ItemEventArgs} class that is bound to the {@link IBend} type and carries additional {@link IEdge owner} and index
information.
Since for some events ({@link IGraph#addBendRemovedListener BendRemoved}) it is unclear what {@link IBend#owner} the bend was
owned by before the event, this event can be used to carry exactly that information.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendEventArgs Online Documentation}*/
export class BendEventArgs {
/**
Initializes a new instance of the {@link BendEventArgs} class.
@param bend The bend to assign to the {@link ItemEventArgs#item} property.
@param owner The previous owner of the bend.
@param index The previous index of the bend in the {@link IEdge#bends} list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendEventArgs Online Documentation}*/
constructor(bend: IBend, owner: IEdge, index: number);
/**
Gets the owner of the bend that owned the bend before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendEventArgs%23owner Online Documentation}*/
readonly owner: IEdge;
/**
Gets the former index of the bend in the {@link IEdge#bends} list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendEventArgs%23index Online Documentation}*/
readonly index: number;
static $class: Class;
private $_BendEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BendEventArgs;
}
export interface ItemChangedEventArgs<TItem, TValue> extends ItemEventArgs<TItem> {
}
/**
Event argument class used by {@link IGraph} if a model item's property has been changed.
Type parameter <code>TItem</code>: The type of the item contained in the event.
Type parameter <code>TValue</code>: The type of the item's property that has been changed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemChangedEventArgs Online Documentation}*/
export class ItemChangedEventArgs<TItem, TValue> {
/**
Initializes a new instance of the {@link ItemChangedEventArgs} class.
@param item The item whose property has been changed.
@param oldValue The value of the property before the change.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemChangedEventArgs Online Documentation}*/
constructor(item: TItem, oldValue: TValue | null);
/**
Gets the value of the property before the change.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemChangedEventArgs%23oldValue Online Documentation}*/
oldValue: TValue | null;
static $class: Class;
private $_ItemChangedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TItem, TValue>(o: any): o is ItemChangedEventArgs<TItem, TValue>;
}
export interface IGraphSelection extends ISelectionModel<IModelItem> {
}
/**
Manages the selection state of items in an {@link IGraph} instance.
This interface provides access to the {@link ISelectionModel} instances that manage the selection of nodes, edges, labels, ports, and bends.
The generic methods that use {@link IModelItem} parameters delegate to the corresponding domain-specific selection
models.
@see ISelectionModel
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphSelection Online Documentation}*/
export abstract class IGraphSelection {
/**
Gets an {@link ISelectionModel} of the selected nodes.
This is the node part of the selection that this instance is a composite of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphSelection%23selectedNodes Online Documentation}*/
abstract readonly selectedNodes: ISelectionModel<INode>;
/**
Gets an {@link ISelectionModel} of the selected labels.
This is the label part of the selection that this instance is a composite of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphSelection%23selectedLabels Online Documentation}*/
abstract readonly selectedLabels: ISelectionModel<ILabel>;
/**
Gets an {@link ISelectionModel} of the selected edges.
This is the edge part of the selection that this instance is a composite of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphSelection%23selectedEdges Online Documentation}*/
abstract readonly selectedEdges: ISelectionModel<IEdge>;
/**
Gets an {@link ISelectionModel} of the selected ports.
This is the ports part of the selection that this instance is a composite of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphSelection%23selectedPorts Online Documentation}*/
abstract readonly selectedPorts: ISelectionModel<IPort>;
/**
Gets an {@link ISelectionModel} of the selected bends.
This is the bend part of the selection that this instance is a composite of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphSelection%23selectedBends Online Documentation}*/
abstract readonly selectedBends: ISelectionModel<IBend>;
static $class: Class;
private $_IGraphSelection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGraphSelection;
}
export interface ILabel extends IModelItem {
}
/**
* The interface used in an {@link IGraph} implementation for labels.
* This interface provides read-only access to the properties of a label. In order to modify the state of an instance use
* the various methods provided by the {@link IGraph} this instance belongs to. Labels are stored in
* {@link IListEnumerable IListEnumerable&lt;ILabel>}s, that can be obtained from {@link ILabelOwner} implementations' {@link ILabelOwner#labels} property. Typical owners
* of a label are {@link INode} and {@link IEdge} instances. Like all items in an <code>IGraph</code>, this item supports the
* {@link ILookup#lookup} method that can be used to query additional aspects of the item.
*
* <br><b>Working with a label in the graph</b>
* <pre>
* const graph = graphComponent.graph
* const node = graph.createNodeAt(new Point(0, 0))
*
* // add a label to the node
* const label = graph.addLabel(node, 'Some Text', ExteriorLabelModel.SOUTH)
*
* // the newly created label is part of the graph
* console.log(graph.contains(label)) // true
* console.log(graph.labels.size) // 1
*
* // the label belongs to its owner
* console.log(label.owner === node) // true
* console.log(node.labels.size) // 1
* console.log(node.labels.get(0) === label) // true
*
* // removing the label removes it from the graph
* graph.remove(label)
* console.log(graph.contains(label)) // false
* console.log(graph.labels.size) // 0
* // and from its owner
* console.log(label.owner === null) // true
* console.log(node.labels.size) // 0
* </pre>
*
* <br><b>Labels cannot live without an owner in the graph</b>
* <pre>
* console.log(graph.contains(label)) // true
* graph.remove(label.owner)
* console.log(graph.contains(label)) // false
* </pre>
*
* The graph model with all relevant types and their relationships is presented in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph The Graph Model}.
*
* Using the look-up mechanism is explained in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_concepts_lookup Service Locator Pattern: Lookup}.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabel Online Documentation}
*/
export abstract class ILabel {
/**
* Gets the style that is responsible for the visual representation of this node in a {@link CanvasComponent}.
* Note that the style instance associated with a label instance may be shared between multiple label instances and that
* the modification of this style will result in a change of the appearance of all labels that are associated with the same
* style instance.
*
* To change the style for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setStyle}
*
* <br><b>Setting the style of a label</b>
* <pre>
* graph.setStyle(label, new DefaultLabelStyle())
* </pre>
*
* <br><b>Obtaining the style of a label</b>
* <pre>
* const style = label.style
* </pre>
*
* @see IGraph#setStyle
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabel%23style Online Documentation}
*/
abstract readonly style: ILabelStyle;
/**
* Gets the preferred size of the label with respect to its current contents and the implementation of the visualization.
* Often the {@link ILabel#layout layout's} size will be the same as the preferred size, but it's up to the implementation of the {@link ILabelModel}
* to interpret it differently.
*
* To change the ports for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setEdgePorts}
*
* <br><b>Setting the preferred size of a label</b>
* <pre>
* graph.setLabelPreferredSize(label, new Size(width, height))
* </pre>
*
* <br><b>Obtaining the preferred size of a label</b>
* <pre>
* const size = label.preferredSize
* </pre>
*
* @see IGraph#setLabelPreferredSize
* @see IGraph#adjustLabelPreferredSize
* @see IGraph#calculateLabelPreferredSize
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabel%23preferredSize Online Documentation}
*/
abstract readonly preferredSize: Size;
/**
* Gets the owner of this label.
* Note that for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, the owner can't be changed after creation and the label has to be deleted and recreated with a new
* owner.
*
* <br><b>Relationship between label and label owner</b>
* <pre>
* // add a label to the owner
* const label = graph.addLabel(owner, 'Some Text')
*
* // the label's Owner property is set to the owner
* console.log(label.owner === owner) // true
* // the label is in its owner's Labels collection
* console.log(owner.labels.includes(label)) // true
* </pre>
*
* @see ILabelOwner#labels
* @see IGraph#addLabel
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabel%23owner Online Documentation}
*/
abstract readonly owner: ILabelOwner | null;
/**
* Gets the text string associated with this label.
* It is up to the visualization engine to interpret this property for the visualization of the label. Normally, it will
* render the text into the {@link ILabel#layout layout} of this instance.
*
* To change the text for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setLabelText}
*
* <br><b>Setting the text of a label</b>
* <pre>
* graph.setLabelText(label, 'Some Text')
* </pre>
*
* <br><b>Obtaining the text of a label</b>
* <pre>
* const text = label.text
* </pre>
*
* @see IGraph#setLabelText
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabel%23text Online Documentation}
*/
abstract readonly text: string;
/**
* Gets the label model's parameter that determines the positioning of this label.
* Note that the label model parameter instance associated with a label instance may be shared between multiple label
* instances and that the modification of this instance or its {@link ILabelModelParameter#model} will result in a change of the positioning of all labels
* that are associated with the same parameter or model instance.
*
* To change the layout parameter for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setLabelLayoutParameter}
*
* <br><b>Setting the layout parameter of a label</b>
* <pre>
* graph.setLabelLayoutParameter(label, InteriorLabelModel.center)
* </pre>
*
* <br><b>Obtaining the layout parameter of a label</b>
* <pre>
* const parameter = label.layoutParameter
* </pre>
*
* <br><b>Obtaining the layout of a label</b>
* <pre>
* const layout = label.layout
* </pre>
*
* @see IGraph#setLabelLayoutParameter
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabel%23layoutParameter Online Documentation}
*/
abstract readonly layoutParameter: ILabelModelParameter;
/**
Gets the index of the label at its {@link ILabel#owner}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabel%23index Online Documentation}*/
readonly index: number;
/**
Gets a snapshot of the current layout of the label.
Labels have a {@link ISize size} and are anchored at a given location in world coordinate space. The anchor is the point around which
the oriented rectangle is rotated in the world coordinate system. If the oriented rectangle has default orientation,
i.e. its up vector points north (<code>&lt;0,-1></code>), it coincides with its lower left corner. The positioning of the label is
determined using a pair of {@link ILabelModel} and {@link ILabelModelParameter}. In order to modify the layout
of a label instance, set another {@link ILabel#layoutParameter} for this label or modify its associated instance.
@see ILabel#layoutParameter
@see ILabelModel#getGeometry
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabel%23layout Online Documentation}*/
readonly layout: IOrientedRectangle;
static $class: Class;
private $_ILabel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabel;
}
export interface ILabelDefaults {
}
/**
* Interface used by {@link IGraph} and the like to declare and obtain the defaults for labels.
* Note that changing these defaults does <i>not</i> change properties of already created model items. Rather, only items created
* after the change are affected.
*
* <br><b>Setting defaults for labels</b>
* <pre>
* // Labels on nodes, edges, and ports have different ILabelDefaults instances.
* // These can be retrieved from their owner type's defaults
*
* // the defaults for node labels can be set on the ILabelDefaults instance
* // found at the Labels property of the node defaults
* graph.nodeDefaults.labels.layoutParameter = InteriorLabelModel.CENTER
* graph.nodeDefaults.labels.style = new DefaultLabelStyle()
*
* // the defaults for edge labels can be set on the ILabelDefaults instance
* // found at the Labels property of the edge defaults
* graph.edgeDefaults.labels.layoutParameter = InteriorLabelModel.CENTER
* graph.edgeDefaults.labels.style = new DefaultLabelStyle()
*
* // defaults for port labels can be set on the ILabelDefaults instance
* // found at the Labels property of their owner type's defaults
* graph.nodeDefaults.ports.labels.layoutParameter = new InsideOutsidePortLabelModel().createInsideParameter()
* graph.edgeDefaults.ports.labels.layoutParameter = new FreePortLabelModel().createDefaultParameter()
* </pre>
*
* @see INodeDefaults#labels
* @see IEdgeDefaults#labels
* @see IGraph#nodeDefaults
* @see IGraph#edgeDefaults
* @see IGraph#groupNodeDefaults
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelDefaults Online Documentation}
*/
export abstract class ILabelDefaults {
/**
Factory method that returns a label model parameter instance for use with newly created labels.
Most implementations will yield either, a {@link ICloneable#clone clone} of or the {@link ILabelDefaults#layoutParameter} property, if {@link ILabelDefaults#shareLayoutParameterInstance} is enabled, but they might use more complicated
logic, too.
@param owner The owner of the label that will be created.
@returns The parameter to use, which for most implementations is either a {@link ICloneable#clone clone} of or the {@link ILabelDefaults#layoutParameter} property, if {@link ILabelDefaults#shareLayoutParameterInstance} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelDefaults%23getLayoutParameterInstance(ILabelOwner) Online Documentation}*/
abstract getLayoutParameterInstance(owner: ILabelOwner): ILabelModelParameter;
/**
Factory method that returns a style instance for use with newly created labels.
Most implementations will yield either, a {@link ICloneable#clone clone} of or the {@link ILabelDefaults#style} property, if {@link ILabelDefaults#shareStyleInstance} is enabled, but they might use more complicated
logic, too.
@param owner The owner of the label that will be created.
@returns The style to use, which for most implementations is either a {@link ICloneable#clone clone} of or the {@link ILabelDefaults#style} property, if {@link ILabelDefaults#shareStyleInstance} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelDefaults%23getStyleInstance(ILabelOwner) Online Documentation}*/
abstract getStyleInstance(owner: ILabelOwner): ILabelStyle;
/**
* Gets or sets the style to use for labels.
* Depending on the setting of {@link ILabelDefaults#shareStyleInstance}, the {@link ILabelDefaults#getStyleInstance} method should return a {@link ICloneable#clone clone} of this
* instance or the very same instance.
*
* <pre>
* graph.nodeDefaults.labels.layoutParameter = InteriorLabelModel.CENTER
* graph.nodeDefaults.labels.style = new DefaultLabelStyle()
* </pre>
*
* @see ILabelDefaults#shareStyleInstance
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelDefaults%23style Online Documentation}
*/
abstract style: ILabelStyle;
/**
* Gets or sets a property that determines whether to automatically adjust the preferred size of a label.
* On a call to {@link IGraph#setLabelText} or
* {@link IGraph#setStyle}, the preferred size of the label will automatically be adjusted to the preferred size that
* is suggested by the label's style renderer, if this property is set to true.
*
* <pre>
* const node = graph.createNodeAt(new Point(0, 0))
* const label = graph.addLabel(node, 'Text')
* const preferredSize1 = label.preferredSize
* // enable auto adjust
* graph.nodeDefaults.labels.autoAdjustPreferredSize = true
* graph.setLabelText(label, 'Another Label Text')
* const preferredSize2 = label.preferredSize
* // preferred size has been adjusted to changed text
* console.log(preferredSize1.equals(preferredSize2)) // false
*
* // disable auto adjust
* graph.nodeDefaults.labels.autoAdjustPreferredSize = false
* graph.setLabelText(label, 'x')
* const preferredSize3 = label.preferredSize
* // preferred size has not been adjusted to changed text
* console.log(preferredSize2.equals(preferredSize3)) // true
*
* // adjusting the preferred size can be triggered manually, though
* graph.adjustLabelPreferredSize(label)
* </pre>
*
* @see ILabelStyleRenderer#getPreferredSize
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelDefaults%23autoAdjustPreferredSize Online Documentation}
*/
abstract autoAdjustPreferredSize: boolean;
/**
Gets or sets a value indicating whether the {@link ILabelDefaults#layoutParameter} instance should be shared referentially or {@link ICloneable#clone cloned} upon a call to
{@link ILabelDefaults#getLayoutParameterInstance}.
@see ILabelDefaults#getLayoutParameterInstance
@see ILabelDefaults#layoutParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelDefaults%23shareLayoutParameterInstance Online Documentation}*/
abstract shareLayoutParameterInstance: boolean;
/**
* Gets or sets a value indicating whether the {@link ILabelDefaults#style} instance should be shared referentially or {@link ICloneable#clone cloned} upon a call to
* {@link ILabelDefaults#getStyleInstance}.
* <br><b>Shared vs. cloned styles</b>
* <pre>
* const n1 = graph.createNodeAt(new Point(0, 0))
* const n2 = graph.createNodeAt(new Point(0, 50))
* const n3 = graph.createNodeAt(new Point(0, 100))
* const n4 = graph.createNodeAt(new Point(0, 150))
*
* const defaultStyle = new DefaultLabelStyle()
* graph.nodeDefaults.labels.style = defaultStyle
*
* // share style instances
* graph.nodeDefaults.labels.shareStyleInstance = true
* const l1 = graph.addLabel(n1, 'Text')
* const l2 = graph.addLabel(n2, 'Text')
* // l1.style === defaultStyle; l2.style === defaultStyle; l1.style === l2.style;
* defaultStyle.backgroundFill = Fill.RED // l1 and l2 both turn red, too
*
* // clone style instances
* graph.nodeDefaults.labels.shareStyleInstance = false
* const l3 = graph.addLabel(n3, 'Text')
* const l4 = graph.addLabel(n4, 'Text')
* // l3.style !== defaultStyle; l4.style !== defaultStyle; l3.style !== l4.style;
* defaultStyle.backgroundFill = Fill.YELLOW // l3 and l4 don't change their color
* </pre>
*
* @see ILabelDefaults#getStyleInstance
* @see ILabelDefaults#style
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelDefaults%23shareStyleInstance Online Documentation}
*/
abstract shareStyleInstance: boolean;
/**
* Gets or sets the label model parameter to use for labels.
* Depending on the setting of {@link ILabelDefaults#shareLayoutParameterInstance}, the {@link ILabelDefaults#getLayoutParameterInstance} method should return a {@link ICloneable#clone clone} of
* this instance or the very same instance.
*
* <pre>
* graph.nodeDefaults.labels.layoutParameter = InteriorLabelModel.CENTER
* graph.nodeDefaults.labels.style = new DefaultLabelStyle()
* </pre>
*
* @see ILabelDefaults#shareLayoutParameterInstance
* @see ILabelDefaults#getLayoutParameterInstance
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelDefaults%23layoutParameter Online Documentation}
*/
abstract layoutParameter: ILabelModelParameter;
static $class: Class;
private $_ILabelDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelDefaults;
}
export interface ILabelModel extends ILookup {
}
/**
Defines the geometry of {@link ILabel} instances with respect to specific {@link ILabelModelParameter}s.
In addition to defining the geometry of a label with respect to a specific
{@link ILabelModelParameter}, a label model is typically the factory of the label model parameters that are
compatible with it, too.
Typically, the label model defines a, possibly infinite, compilation of valid geometries for labels, and a specific
label model parameter specifies an exact geometry for a label.
By design, the same instance of a label model can be shared by multiple label model parameters. Similarly, the same
instance of a label model parameter can be shared by multiple labels.
This interface supports the {@link ILookup#lookup} method that can be used to query additional aspects of the
implementation.
@see ILabelModelParameter
@see ILabel
@see IGraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModel Online Documentation}*/
export abstract class ILabelModel {
/**
Creates a default parameter that can be used for this model.
@returns a parameter for this model instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModel%23createDefaultParameter() Online Documentation}*/
abstract createDefaultParameter(): ILabelModelParameter;
/**
Provides a {@link ILookup lookup context} for the given combination of label and parameter.
@param label The label to use in the context.
@param layoutParameter The parameter to use for the label in the context.
@returns An implementation of the {@link ILookup} interface that can be used to query additional aspects of the
label/parameter combination.
@see ILookup#EMPTY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModel%23getContext(ILabel%2CILabelModelParameter) Online Documentation}*/
abstract getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
Calculates the {@link IOrientedRectangle geometry} of the given label using the given model parameter.
@param label The label to calculate the geometry for.
@param layoutParameter A parameter that is compatible with this model. Typically, this is a parameter that has been created by this model, and
its property {@link ILabelModelParameter#model} returns this instance.
@returns An {@link IOrientedRectangle} that describes the geometry of the label. This is typically designed as a
flyweight, therefore clients should not cache the instance but store the values if they need a snapshot for later use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModel%23getGeometry(ILabel%2CILabelModelParameter) Online Documentation}*/
abstract getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
static $class: Class;
private $_ILabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelModel;
}
export interface ILabelCandidateDescriptor {
}
/**
Interface that provides more information about a potential label geometry (a candidate) to an automatic labeling
algorithm.
Typically, instances of this interface are created by an implementation of {@link ILabelCandidateDescriptorProvider}
for a pair of an {@link ILabel} and a {@link ILabelModelParameter} that describes a potential placement of
that label (a 'candidate').
@see ILabelCandidateDescriptorProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelCandidateDescriptor Online Documentation}*/
export abstract class ILabelCandidateDescriptor {
/**
Gets a value indicating whether this candidate is deemed an external candidate.
This is mainly for {@link INode} labels that are a visual part of the node's internals. It could be used for edge
labels, too, if the edge occupies significant amounts of visual space.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelCandidateDescriptor%23externalCandidate Online Documentation}*/
abstract readonly externalCandidate: boolean;
/**
Gets a penalty value that is used by labeling algorithms to judge the severity if this candidate overlaps with an edge
path.
This value must be between 0 and 1 (inclusive). A value of 0 means that an overlap is considered not important, while a
value of 1 means that an overlap is considered severe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelCandidateDescriptor%23edgeOverlapPenalty Online Documentation}*/
abstract readonly edgeOverlapPenalty: number;
/**
Gets a penalty value that is used by labeling algorithms to judge the severity if this label candidate overlaps with a
node's bounds.
This value must be between 0 and 1 (inclusive). A value of 0 means that an overlap is considered not important, while a
value of 1 means that an overlap is considered severe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelCandidateDescriptor%23nodeOverlapPenalty Online Documentation}*/
abstract readonly nodeOverlapPenalty: number;
/**
Gets a profit value that is used by labeling algorithms to judge how good this label candidate is.
This value must be between 0 and 1 (inclusive). Higher profit means better candidates. Hence, there is a higher
probability that the candidate is chosen by a labeling algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelCandidateDescriptor%23profit Online Documentation}*/
abstract readonly profit: number;
static $class: Class;
private $_ILabelCandidateDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelCandidateDescriptor;
}
export interface ILabelCandidateDescriptorProvider {
}
/**
An interface that can be used to retrieve {@link ILabelCandidateDescriptor} instances for a given pair of a {@link ILabel}
and a {@link ILabelModelParameter candidate parameter}.
This interface can be used by automatic labeling algorithms to help decide which labels candidate positions should be
used.
@see ILabelCandidateDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelCandidateDescriptorProvider Online Documentation}*/
export abstract class ILabelCandidateDescriptorProvider {
/**
Gets the descriptor for a given pair of a label and a {@link ILabelModelParameter}.
@param label The label to possibly assign the <code>layoutParameter</code>.
@param layoutParameter The parameter to provide a descriptor for if applied to the given <code>label</code>.
@returns A descriptor or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelCandidateDescriptorProvider%23getDescriptor(ILabel%2CILabelModelParameter) Online Documentation}*/
abstract getDescriptor(label: ILabel, layoutParameter: ILabelModelParameter): ILabelCandidateDescriptor | null;
static $class: Class;
private $_ILabelCandidateDescriptorProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelCandidateDescriptorProvider;
}
export interface ConstantLabelCandidateDescriptorProvider extends YObject, ILabelCandidateDescriptorProvider {
}
/**
An implementation of the {@link ILabelCandidateDescriptorProvider} interface that always yields a constant
{@link ILabelCandidateDescriptor}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstantLabelCandidateDescriptorProvider Online Documentation}*/
export class ConstantLabelCandidateDescriptorProvider {
/**
Initializes a new instance of the {@link ConstantLabelCandidateDescriptorProvider} class that always yields <code>descriptor</code>
in {@link ConstantLabelCandidateDescriptorProvider#getDescriptor}.
@param descriptor The descriptor to yield in {@link ConstantLabelCandidateDescriptorProvider#getDescriptor}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstantLabelCandidateDescriptorProvider Online Documentation}*/
constructor(descriptor: ILabelCandidateDescriptor | null);
/**
Always yields the parameter that has been passed to the constructor.
@param label
@param layoutParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstantLabelCandidateDescriptorProvider%23getDescriptor(ILabel%2CILabelModelParameter) Online Documentation}*/
getDescriptor(label: ILabel, layoutParameter: ILabelModelParameter): ILabelCandidateDescriptor | null;
/**
A sharable implementation of the {@link ILabelCandidateDescriptorProvider} interface that always yields
{@link LabelCandidateDescriptor#EXTERNAL_DESCRIPTOR}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstantLabelCandidateDescriptorProvider%23EXTERNAL_DESCRIPTOR_PROVIDER Online Documentation}*/
static EXTERNAL_DESCRIPTOR_PROVIDER: ILabelCandidateDescriptorProvider;
/**
A sharable implementation of the {@link ILabelCandidateDescriptorProvider} interface that always yields
{@link LabelCandidateDescriptor#INTERNAL_DESCRIPTOR}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstantLabelCandidateDescriptorProvider%23INTERNAL_DESCRIPTOR_PROVIDER Online Documentation}*/
static INTERNAL_DESCRIPTOR_PROVIDER: ILabelCandidateDescriptorProvider;
static $class: Class;
private $_ConstantLabelCandidateDescriptorProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ConstantLabelCandidateDescriptorProvider;
}
export interface LabelCandidateDescriptor extends YObject, ILabelCandidateDescriptor {
/**
@inheritdoc*/
externalCandidate: boolean;
} /**
A simple mutable default implementation of the {@link ILabelCandidateDescriptor} interface.
All properties are read/write in this implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidateDescriptor Online Documentation}*/
export class LabelCandidateDescriptor {
/**
Creates a new instance of a {@link LabelCandidateDescriptor} with the default values.
@param [options] The parameters to pass.
@param [options.edgeOverlapPenalty] Sets the edgeOverlapPenalty property.
@param [options.externalCandidate] Sets the externalCandidate property.
@param [options.nodeOverlapPenalty] Sets the nodeOverlapPenalty property.
@param [options.profit] Sets the profit property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidateDescriptor Online Documentation}*/
constructor(options?: {
edgeOverlapPenalty?: number;
externalCandidate?: boolean;
nodeOverlapPenalty?: number;
profit?: number;
});
/**
A sharable immutable implementation of the {@link ILabelCandidateDescriptor} interface that has penalties of <code>1.0d</code>
and a profit of <code>1.0d</code> and yields <code>true</code> for the {@link ILabelCandidateDescriptor#externalCandidate} property.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidateDescriptor%23EXTERNAL_DESCRIPTOR Online Documentation}*/
static EXTERNAL_DESCRIPTOR: ILabelCandidateDescriptor;
/**
A sharable immutable implementation of the {@link ILabelCandidateDescriptor} interface that has penalties of <code>1.0d</code>
and a profit of <code>1.0d</code> and yields <code>false</code> for the {@link ILabelCandidateDescriptor#externalCandidate} property.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidateDescriptor%23INTERNAL_DESCRIPTOR Online Documentation}*/
static INTERNAL_DESCRIPTOR: ILabelCandidateDescriptor;
/**
@default 1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidateDescriptor%23edgeOverlapPenalty Online Documentation}*/
edgeOverlapPenalty: number;
/**
@default 1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidateDescriptor%23nodeOverlapPenalty Online Documentation}*/
nodeOverlapPenalty: number;
/**
@default 1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidateDescriptor%23profit Online Documentation}*/
profit: number;
static $class: Class;
private $_LabelCandidateDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelCandidateDescriptor;
}
export interface ILabelModelParameter extends ICloneable {
}
/**
Defines a state that can be used to {@link ILabelModel#getGeometry determine the geometry of a label} with a {@link ILabelModel}.
Typically, implementations of this interface are created by factory methods of the corresponding {@link ILabelModelParameter#model}.
This interface extends the {@link ICloneable} interface. This allows clients to obtain a persistent copy of the
current state of this parameter. Immutable parameter implementations may return themselves.
@see ILabelModel
@see ILabel
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModelParameter Online Documentation}*/
export abstract class ILabelModelParameter {
/**
Determines whether this parameter can be used for a given label instance.
A parameter must not necessarily support all types of labels. This method returns whether it supports the given <code>label</code>.
For example, a parameter might only support labels that are owned by an {@link INode}.
@param label The label to test.
@returns <code>true</code> if this instance can be used for the given label, <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModelParameter%23supports(ILabel) Online Documentation}*/
abstract supports(label: ILabel): boolean;
/**
Gets the model that can be used to determine the geometry of a label given this parameter and a label instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModelParameter%23model Online Documentation}*/
abstract readonly model: ILabelModel;
static $class: Class;
private $_ILabelModelParameter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelModelParameter;
}
export interface ILabelModelParameterFinder {
}
/**
Implementations can be used to find the best {@link ILabelModelParameter} to approximate a given layout for a given
{@link ILabelModel}.
Implementations of this class may be retrieved from {@link ILabelModel} implementations using their {@link ILookup#lookup}
method. Note that {@link ILabelModel} implementations are not required to return an instance if queried for this
interface.
@see DefaultLabelModelParameterFinder
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModelParameterFinder Online Documentation}*/
export abstract class ILabelModelParameterFinder {
/**
Creates an implementation of the interface {@link ILabelModelParameterFinder} by using the given function as
implementation for its {@link ILabelModelParameterFinder#findBestParameter} method.
@param findBestParameter A function for {@link ILabelModelParameterFinder}'s single abstract method
{@link ILabelModelParameterFinder#findBestParameter}.
@returns An instance of the {@link ILabelModelParameterFinder} interface based on the given function.*/
static create(findBestParameter: (label: ILabel, model: ILabelModel, layout: IOrientedRectangle) => ILabelModelParameter): ILabelModelParameterFinder;
/**
Tries to find a parameter that best matches the given layout for the provided label instance.
This method may not necessarily find a parameter that matches the provided layout exactly. Implementations may choose to
simply return the model's {@link ILabelModel#createDefaultParameter default parameter} but may never return <code>null</code>.
@param label The label to find a parameter for.
@param model The model instance to use. This should be the instance this instance has been obtained from.
@param layout The anticipated layout for the label.
@returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModelParameterFinder%23findBestParameter(ILabel%2CILabelModel%2CIOrientedRectangle) Online Documentation}*/
abstract findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
static $class: Class;
private $_ILabelModelParameterFinder: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelModelParameterFinder;
}
export interface DefaultLabelModelParameterFinder extends YObject, ILabelModelParameterFinder {
}
/**
Implementation of the {@link ILabelModelParameterFinder} interface that uses the {@link ILabelModel}'s own {@link ILabelModelParameterProvider}
instance to find the closest match.
This implementation uses a metric to find the closest of the provided parameter candidates to the layout provided in
{@link DefaultLabelModelParameterFinder#selectBestParameter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelModelParameterFinder Online Documentation}*/
export class DefaultLabelModelParameterFinder {
/**
Calculates a metric for the given label/parameter pair and the target label layout.
This implementation gets the geometry from the parameter's model's {@link ILabelModel#getGeometry} method and
returns the result of a call to {@link DefaultLabelModelParameterFinder#distance}.
@param label The label to apply the parameter to.
@param layoutParameter The parameter candidate.
@param layout The targeted layout rectangle.
@returns A metric that indicates the distance between the candidate's layout and the target layout.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelModelParameterFinder%23calculateMetric(ILabel%2CILabelModelParameter%2CIOrientedRectangle) Online Documentation}*/
calculateMetric(label: ILabel, layoutParameter: ILabelModelParameter, layout: IOrientedRectangle): number;
/**
Returns the distance between two {@link IOrientedRectangle} instances.
The distance is calculated using the Euclidean distance between the two centers and the angular deviation between the
two up vectors.
@param rectangle1 The first geometry.
@param rectangle2 The geometry to compare the first one with.
@returns A value indicating the distance between the two instances.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelModelParameterFinder%23distance(IOrientedRectangle%2CIOrientedRectangle) Online Documentation}*/
distance(rectangle1: IOrientedRectangle, rectangle2: IOrientedRectangle): number;
/**
Tries to find a parameter that best matches the given layout for the provided label instance.
This method may not necessarily find a parameter that matches the provided layout exactly. It will first query the {@link DefaultLabelModelParameterFinder#getCandidateParameters}
parameter for an enumeration of candidates and will then find the best matching candidate using a metric calculated by
{@link DefaultLabelModelParameterFinder#calculateMetric}. If no parameter can be found this method returns the {@link ILabelModel#createDefaultParameter default parameter}
for the model.
@param label The label to find a parameter for.
@param model The model instance to use. This should be the instance this instance has been obtained from.
@param layout The anticipated layout for the label.
@returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelModelParameterFinder%23findBestParameter(ILabel%2CILabelModel%2CIOrientedRectangle) Online Documentation}*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
Retrieves an enumerator over all candidates for a given label and model.
@param label The label.
@param model The model of the label.
@returns An enumerator over the candidates. The enumerator may be empty.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelModelParameterFinder%23getCandidateParameters(ILabel%2CILabelModel) Online Documentation}*/
getCandidateParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
Tries to find a parameter that best matches the given layout for the provided label instance among the set of given
parameters.
Null may be returned if an empty enumerable is passed.
@param label The label to find a parameter for.
@param layout The anticipated layout for the label.
@param layoutParameters A set of parameters to choose from.
@returns A parameter that can be used for the label to approximate the provided layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelModelParameterFinder%23selectBestParameter(ILabel%2CIOrientedRectangle%2CIEnumerable%3CILabelModelParameter%3E) Online Documentation}*/
selectBestParameter(label: ILabel, layout: IOrientedRectangle, layoutParameters: IEnumerable<ILabelModelParameter> | IEnumerableConvertible<ILabelModelParameter>): ILabelModelParameter | null;
/**
A singleton instance of this class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelModelParameterFinder%23INSTANCE Online Documentation}*/
static INSTANCE: DefaultLabelModelParameterFinder;
static $class: Class;
private $_DefaultLabelModelParameterFinder: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultLabelModelParameterFinder;
}
export interface ILabelModelParameterProvider {
}
/**
Helper interface used by {@link ILabelModel} implementations to provide possible candidate
{@link ILabelModelParameter}s for a given label and model.
Implementations of this interface can be retrieved from the {@link ILabelModel}'s {@link ILookup#lookup}
method. Note that not all models necessarily need to or can provide implementations of this interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModelParameterProvider Online Documentation}*/
export abstract class ILabelModelParameterProvider {
/**
Creates an implementation of the interface {@link ILabelModelParameterProvider} by using the given function as
implementation for its {@link ILabelModelParameterProvider#getParameters} method.
@param getParameters A function for {@link ILabelModelParameterProvider}'s single abstract method
{@link ILabelModelParameterProvider#getParameters}.
@returns An instance of the {@link ILabelModelParameterProvider} interface based on the given function.*/
static create(getParameters: (label: ILabel, model: ILabelModel) => IEnumerable<ILabelModelParameter>): ILabelModelParameterProvider;
/**
Returns an enumerator over a set of possible {@link ILabelModelParameter} instances that can be used for the given
label and model.
@param label The label instance to use.
@param model The model to provide parameters for.
@returns A possibly empty enumerator over a set of label model parameters.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelModelParameterProvider%23getParameters(ILabel%2CILabelModel) Online Documentation}*/
abstract getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
static $class: Class;
private $_ILabelModelParameterProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelModelParameterProvider;
}
export interface ILabelOwner extends IModelItem {
}
/**
* Common interface for items in an {@link IGraph} implementation that may own {@link ILabel labels}.
* Like all items in an <code>IGraph</code>, this item supports the {@link ILookup#lookup} method that can be used to query
* additional aspects of the item.
*
* <br><b>Relationship between label and label owner</b>
* <pre>
* // add a label to the owner
* const label = graph.addLabel(owner, 'Some Text')
*
* // the label's Owner property is set to the owner
* console.log(label.owner === owner) // true
* // the label is in its owner's Labels collection
* console.log(owner.labels.includes(label)) // true
* </pre>
*
* @see INode
* @see IEdge
* @see IPort
* @see IGraph#addLabel
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelOwner Online Documentation}
*/
export abstract class ILabelOwner {
/**
* Gets a collection of {@link ILabel labels} that are owned by this instance.
* This gives access to a read-only live view of the labels, i.e. the collection can change over time, as well as the
* labels contained in it. If a snapshot of the current state is needed, one needs to copy the collection and its
* contents.
*
* To modify the label collection for instances of the default implementations that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#addLabel} and {@link IGraph#remove}.
*
* <br><b>Adding labels to a node.</b>
* <pre>
* // add label with given text, default style, and default placement
* // and determine the preferred size automatically
*
* const label1 = graph.addLabel(node, 'Some Label')
*
* // add label with given text, placement, style, size, and tag (user object)
* const label2 = graph.addLabel(
*   node,
*   'Some Label',
*   InteriorLabelModel.CENTER,
*   new DefaultLabelStyle(),
*   new Size(10, 150),
*   userObject
* )
*
* // add label with given text and style but default placement
* // and determine the preferred size automatically
* const label3 = graph.addLabel({
*   owner: node,
*   text: 'Some Label',
*   layoutParameter: InteriorLabelModel.CENTER,
*   style: new DefaultLabelStyle()
* })
* </pre>
*
* <br><b>Iterating over all labels of a node.</b>
* <pre>
* for (const label of node.labels) {
*   // ...
* }
* </pre>
*
* <br><b>Removing a label from a node.</b>
* <pre>
* graph.remove(label)
* </pre>
*
* @see IGraph#addLabel
* @see IGraph#remove
* @see ILabel#owner
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelOwner%23labels Online Documentation}
*/
abstract readonly labels: IListEnumerable<ILabel>;
static $class: Class;
private $_ILabelOwner: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelOwner;
}
export interface IMapperRegistry {
}
/**
A registry that can be used to store {@link IMapper} implementations by tag.
Implementations of this interface can be found in the {@link ILookup#lookup} of the default {@link IGraph}
implementations in the framework.
Using <code>IMapperRegistry</code> is described in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_graph-binding_data Associating Data with Graph Elements}.
@see MapperRegistry
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry Online Documentation}*/
export abstract class IMapperRegistry {
/**
Registers a mapper for the given tag.
If there already was a mapper for <code>tag</code>, it will be replaced.
Type parameter <code>K</code>: The key type for the mapper.
Type parameter <code>V</code>: The value type for the mapper.
@param valueType The value type for the mapper.
@param keyType The key type for the mapper.
@param tag The tag to use.
@param mapper The mapper to register.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23addMapper(Class%2CClass%2CObject%2CIMapper%3CK%2CV%3E) Online Documentation}*/
abstract addMapper<K, V>(keyType: Class, valueType: Class, tag: any, mapper: IMapper<K, V> | IMapperConvertible<K, V>): void;
/**
Creates a simple read-only {@link IMapper} implementation with key type {@link ILabel} and {@link IMapperRegistry#addMapper adds} it to the <code>registry</code> using the
provided <code>tag</code>.
The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
<code>tag</code>, it will be replaced.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@param constant The value to yield during {@link IMapper} reads.
@returns The newly created mapper instance.
@see IMapper#fromDelegate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createConstantMapper(ILabelLayoutDpKey%3CV%3E%2CV) Online Documentation}*/
createConstantMapper<V>(tag: ILabelLayoutDpKey<V>, constant: V): IMapper<ILabel, V>;
/**
Creates a simple read-only {@link IMapper} implementation with key type {@link IEdge} and {@link IMapperRegistry#addMapper adds} it to the <code>registry</code> using the
provided <code>tag</code>.
The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
<code>tag</code>, it will be replaced.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@param constant The value to yield during {@link IMapper} reads.
@returns The newly created mapper instance.
@see IMapper#fromDelegate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createConstantMapper(EdgeDpKey%3CV%3E%2CV) Online Documentation}*/
createConstantMapper<V>(tag: EdgeDpKey<V>, constant: V | null): IMapper<IEdge, V>;
/**
Creates a simple read-only {@link IMapper} implementation with key type {@link INode} and {@link IMapperRegistry#addMapper adds} it to the <code>registry</code> using the
provided <code>tag</code>.
The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
<code>tag</code>, it will be replaced.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@param constant The value to yield during {@link IMapper} reads.
@returns The newly created mapper instance.
@see IMapper#fromDelegate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createConstantMapper(NodeDpKey%3CV%3E%2CV) Online Documentation}*/
createConstantMapper<V>(tag: NodeDpKey<V>, constant: V | null): IMapper<INode, V>;
/**
Creates a simple read-only {@link IMapper} implementation and {@link IMapperRegistry#addMapper adds} it to the <code>registry</code> using the provided <code>tag</code>.
If there already was a mapper for <code>tag</code>, it will be replaced.
Type parameter <code>K</code>: The type of the keys of the mapper.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@param constant The value to yield during {@link IMapper} reads.
@returns The newly created mapper instance.
@see IMapper#fromConstant
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createConstantMapper(Object%2CV) Online Documentation}*/
createConstantMapper<K, V>(keyType: Class, valueType: Class, tag: any, constant: V | null): IMapper<K, V>;
/**
Creates a delegate-based read-only {@link IMapper} implementation with key type {@link ILabel} and {@link IMapperRegistry#addMapper adds} it to the <code>registry</code> using
the provided <code>tag</code>.
The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
<code>tag</code>, it will be replaced.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@param getter The getter delegate to which {@link IMapper} reads will be delegated.
@returns The newly created mapper instance.
@see IMapper#fromDelegate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createDelegateMapper(ILabelLayoutDpKey%3CV%3E%2Cfunction(ILabel)%3AV) Online Documentation}*/
createDelegateMapper<V>(tag: ILabelLayoutDpKey<V>, getter: (key: ILabel | null) => V): IMapper<ILabel, V>;
/**
Creates a delegate-based read-only {@link IMapper} implementation with key type {@link IEdge} and {@link IMapperRegistry#addMapper adds} it to the <code>registry</code> using
the provided <code>tag</code>.
The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
<code>tag</code>, it will be replaced.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@param getter The getter delegate to which {@link IMapper} reads will be delegated.
@returns The newly created mapper instance.
@see IMapper#fromDelegate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createDelegateMapper(EdgeDpKey%3CV%3E%2Cfunction(IEdge)%3AV) Online Documentation}*/
createDelegateMapper<V>(tag: EdgeDpKey<V>, getter: (key: IEdge | null) => V): IMapper<IEdge, V>;
/**
Creates a delegate-based read-only {@link IMapper} implementation with key type {@link INode} and {@link IMapperRegistry#addMapper adds} it to the <code>registry</code> using
the provided <code>tag</code>.
The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
<code>tag</code>, it will be replaced.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@param getter The getter delegate to which {@link IMapper} reads will be delegated.
@returns The newly created mapper instance.
@see IMapper#fromDelegate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createDelegateMapper(NodeDpKey%3CV%3E%2Cfunction(INode)%3AV) Online Documentation}*/
createDelegateMapper<V>(tag: NodeDpKey<V>, getter: (key: INode | null) => V): IMapper<INode, V>;
/**
Creates a delegate-based read-only {@link IMapper} implementation and {@link IMapperRegistry#addMapper adds} it to the <code>registry</code> using the provided <code>tag</code>.
If there already was a mapper for <code>tag</code>, it will be replaced.
Type parameter <code>K</code>: The type of the keys of the mapper.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@param getter The getter delegate to which {@link IMapper} reads will be delegated.
@returns The newly created mapper instance.
@see IMapper#fromDelegate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createDelegateMapper(Object%2Cfunction(K)%3AV) Online Documentation}*/
createDelegateMapper<K, V>(keyType: Class, valueType: Class, tag: any, getter: (key: K) => V): IMapper<K, V>;
/**
Creates and {@link IMapperRegistry#addMapper adds} a new {@link Mapper} with key type {@link IEdge} to the <code>registry</code> using the provided <code>tag</code>.
The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
<code>tag</code>, it will be replaced.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@returns The newly created mapper instance.
@see Mapper
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createMapper(EdgeDpKey%3CV%3E) Online Documentation}*/
createMapper<V>(tag: EdgeDpKey<V>): Mapper<IEdge, V>;
/**
Creates and {@link IMapperRegistry#addMapper adds} a new {@link Mapper} with key type {@link ILabel} to the <code>registry</code> using the provided <code>tag</code>.
The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
<code>tag</code>, it will be replaced.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@returns The newly created mapper instance.
@see Mapper
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createMapper(ILabelLayoutDpKey%3CV%3E) Online Documentation}*/
createMapper<V>(tag: ILabelLayoutDpKey<V>): Mapper<ILabel, V>;
/**
Creates and {@link IMapperRegistry#addMapper adds} a new {@link Mapper} with key type {@link INode} to the <code>registry</code> using the provided <code>tag</code>.
The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
<code>tag</code>, it will be replaced.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@returns The newly created mapper instance.
@see Mapper
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createMapper(NodeDpKey%3CV%3E) Online Documentation}*/
createMapper<V>(tag: NodeDpKey<V>): Mapper<INode, V>;
/**
Creates and {@link IMapperRegistry#addMapper adds} a new {@link Mapper} to the <code>registry</code> using the provided <code>tag</code>.
If there already was a mapper for <code>tag</code>, it will be replaced.
Type parameter <code>K</code>: The type of the keys of the mapper.
Type parameter <code>V</code>: The type of the values of the mapper.
@param tag The tag.
@returns The newly created mapper instance.
@see Mapper
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23createMapper(Object) Online Documentation}*/
createMapper<K, V>(keyType: Class, valueType: Class, tag: any): Mapper<K, V>;
/**
Returns a mapper that has previously been registered with this instance for the given tag.
Type parameter <code>K</code>: The key type for the mapper.
Type parameter <code>V</code>: The value type for the mapper.
@param tag The tag to use to look up the mapper.
@returns The previously registered instance or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23getMapper(Object) Online Documentation}*/
abstract getMapper<K, V>(tag: any): IMapper<K, V> | null;
/**
Gets the mapper {@link MapperMetadata meta data} for the mapper that is registered using the <code>tag</code>.
@param tag The tag.
@returns The meta data or <code>null</code> if there is no such mapper registered for the given tag.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23getMapperMetadata(Object) Online Documentation}*/
abstract getMapperMetadata(tag: any): MapperMetadata | null;
/**
Removes a previously registered mapper for the given tag.
@param tag The tag that has been used for registering the mapper.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23removeMapper(Object) Online Documentation}*/
abstract removeMapper(tag: any): void;
/**
Sets the mapper meta data for the mapper that has been {@link IMapperRegistry#addMapper registered} using the provided <code>tag</code>.
@param tag The tag.
@param metadata The meta data to store with the mapper.
@throws {Exception} <b>(ArgumentError)</b> If the {@link MapperMetadata#keyType} or {@link MapperMetadata#valueType} mismatch the mapper instance in the registry.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23setMapperMetadata(Object%2CMapperMetadata) Online Documentation}*/
abstract setMapperMetadata(tag: any, metadata: MapperMetadata): void;
/**
Gets an enumerable over all tags that have been used to register mapper implementations with this interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMapperRegistry%23registeredTags Online Documentation}*/
abstract readonly registeredTags: IEnumerable<any>;
static $class: Class;
private $_IMapperRegistry: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IMapperRegistry;
}
export interface INode extends IPortOwner, ILabelOwner {
}
/**
* The interface for node entities in an {@link IGraph}.
* This interface provides read-only access to the properties of a node. In order to modify the state of an instance use
* the various methods provided by the {@link IGraph} this instance belongs to. Nodes and {@link IEdge edges} are the main entities
* that make up an {@link IGraph}. Nodes have zero or more {@link IPortOwner#ports} to which zero or more edges can connect. {@link IGraph}
* provides the {@link IGraph#edgesAt} method for querying the edges that connect to nodes. Also it is possible to
* query the {@link IGraph#edgesAt adjacent edges for each port} that is owned by nodes. This interface combines the functionality of {@link IPortOwner}
* to get access to the ports, {@link ILabelOwner} to get access to the labels, and, like all items in an <code>IGraph</code>,
* nodes support the {@link ILookup#lookup} method inherited from the {@link IModelItem} interface can be used to
* query additional aspects of each instance.
*
* <br><b>Working with a node in the graph</b>
* <pre>
* const graph = graphComponent.graph
*
* // node creation
* const node = graph.createNode(
*   new Rect(0, 0, 60, 40),
*   new ShapeNodeStyle({ shape: ShapeNodeShape.RECTANGLE, stroke: Stroke.BLACK, fill: Fill.ORANGE })
* )
*
* // the newly created node is part of the graph
* console.log(graph.contains(node)) // true
* console.log(graph.nodes.size) // 1
* console.log(graph.nodes.get(0) === node) // true
*
* // changing node properties have to be done via the graph
* console.log(node.layout) // (0, 0, 60, 40)
* graph.setNodeLayout(node, new Rect(100, 0, 60, 40))
* console.log(node.layout) // (100, 0, 60, 40)
*
* // removing the node removes it from the graph
* graph.remove(node)
* console.log(graph.contains(node)) // false
* console.log(graph.nodes.size) // 0
* </pre>
*
* The graph model with all relevant types and their relationships is presented in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph The Graph Model}.
*
* Using the look-up mechanism is explained in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_concepts_lookup Service Locator Pattern: Lookup}.
*
* @see IGraph
* @see IEdge
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INode Online Documentation}
*/
export abstract class INode {
/**
* Gets a rectangle describing the position and size of the node.
* The layout of a node is defined as a rectangle in the world coordinate system that describes the bounding box of the
* representation of a node.
*
* To change the layout for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setNodeLayout}
*
* This method will yield a live view. To obtain a snapshot one has to copy the values of the instance, e.g. by calling {@link IRectangle#toRect}
* on it.
*
* <br><b>Setting the layout of a node</b>
* <pre>
* graph.setNodeLayout(node, new Rect(x, y, width, height))
* </pre>
*
* <br><b>Obtaining the layout of a node</b>
* <pre>
* const layout = node.layout
* </pre>
*
* @see IGraph#setNodeLayout
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INode%23layout Online Documentation}
*/
abstract readonly layout: IRectangle;
/**
* Gets the style that is responsible for the visual representation of this node in a {@link CanvasComponent}.
* Note that the style instance associated with a node instance may be shared between multiple node instances and that the
* modification of this style will result in a change of the appearance of all nodes that are associated with the same
* style instance.
*
* To change the style for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setStyle}
*
* <br><b>Setting the style of a node</b>
* <pre>
* graph.setStyle(node, new ShapeNodeStyle())
* </pre>
*
* <br><b>Obtaining the style of a node</b>
* <pre>
* const style = node.style
* </pre>
*
* @see IGraph#setStyle
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INode%23style Online Documentation}
*/
abstract readonly style: INodeStyle;
static $class: Class;
private $_INode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INode;
}
export interface INodeDefaults {
}
/**
* Interface used by {@link IGraph} and the like to declare and obtain the defaults for nodes and their labels or
* ports.
* Note that changing these defaults does <i>not</i> change properties of already created model items. Rather, only items created
* after the change are affected.
*
* <br><b>Setting defaults for nodes</b>
* <pre>
* // the defaults for nodes can be set on the INodeDefaults instance
* // which can be retrieved from the graph's node defaults property
* graph.nodeDefaults.size = new Size(50, 30)
* graph.nodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.ELLIPSE,
*   fill: Fill.YELLOW
* })
*
* // the defaults for node labels can be set on the ILabelDefaults instance
* // found at the Labels property of the node defaults
* graph.nodeDefaults.labels.layoutParameter = InteriorLabelModel.CENTER
* graph.nodeDefaults.labels.style = new DefaultLabelStyle()
*
* // the defaults for node ports can be set on the IPortDefaults instance
* // found at the Ports property of the node defaults
* graph.nodeDefaults.ports.autoCleanUp = false
* graph.nodeDefaults.ports.locationParameter = FreeNodePortLocationModel.NODE_CENTER_ANCHORED
* </pre>
*
* @see IGraph#nodeDefaults
* @see IGraph#groupNodeDefaults
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INodeDefaults Online Documentation}
*/
export abstract class INodeDefaults {
/**
Factory method that returns a style instance for use with newly created nodes.
Most implementations will yield either, a {@link ICloneable#clone clone} of or the {@link INodeDefaults#style} property, if {@link INodeDefaults#shareStyleInstance} is enabled, but they might use more complicated
logic, too.
@returns The style to use, which for most implementations is either a {@link ICloneable#clone clone} of or the {@link INodeDefaults#style} property, if {@link INodeDefaults#shareStyleInstance} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeDefaults%23getStyleInstance() Online Documentation}*/
abstract getStyleInstance(): INodeStyle;
/**
* Gets or sets the style to use for nodes.
* Depending on the setting of {@link INodeDefaults#shareStyleInstance}, the {@link INodeDefaults#getStyleInstance} method should return a {@link ICloneable#clone clone} of this
* instance or the very same instance.
*
* <pre>
* // set the size of newly created nodes to 50, 30
* graph.nodeDefaults.size = new Size(50, 30)
* // make them appear as yellow ellipse
* graph.nodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.ELLIPSE,
*   fill: Fill.YELLOW
* })
* </pre>
*
* @see INodeDefaults#shareStyleInstance
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INodeDefaults%23style Online Documentation}
*/
abstract style: INodeStyle;
/**
* Gets or sets a value indicating whether the {@link INodeDefaults#style} instance should be shared referentially or {@link ICloneable#clone cloned} upon a call to
* {@link INodeDefaults#getStyleInstance}.
* <br><b>Shared vs. cloned styles</b>
* <pre>
* const defaultStyle = new ShapeNodeStyle()
* graph.nodeDefaults.style = defaultStyle
*
* // share style instances
* graph.nodeDefaults.shareStyleInstance = true
* const n1 = graph.createNodeAt(new Point(0, 0))
* const n2 = graph.createNodeAt(new Point(0, 50))
* // n1.style === defaultStyle; n2.style == defaultStyle; n1.style === n2.style;
* defaultStyle.fill = Fill.RED // n1 and n2 both turn red, too
*
* // clone style instances
* graph.nodeDefaults.shareStyleInstance = false
* const n3 = graph.createNodeAt(new Point(0, 100))
* const n4 = graph.createNodeAt(new Point(0, 150))
* // n3.style !== defaultStyle; n4.style !== defaultStyle; n3.style !== n4.style;
* defaultStyle.fill = Fill.YELLOW // n3 and n4 don't change their color
* </pre>
*
* @see INodeDefaults#getStyleInstance
* @see INodeDefaults#style
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INodeDefaults%23shareStyleInstance Online Documentation}
*/
abstract shareStyleInstance: boolean;
/**
* Gets or sets the defaults for ports at nodes.
* <br><b>Defaults for node ports have to be set on the Ports property of the NodeDefaults</b>
* <pre>
* // don't remove node labels after their
* // last adjacent edge has been removed
* graph.nodeDefaults.ports.autoCleanUp = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INodeDefaults%23ports Online Documentation}
*/
abstract ports: IPortDefaults;
/**
* Gets or sets the defaults for labels at nodes.
* <br><b>Defaults for node labels have to be set on the Labels property of the NodeDefaults</b>
* <pre>
* // place node labels on the node center by default
* graph.nodeDefaults.labels.layoutParameter = InteriorLabelModel.CENTER
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INodeDefaults%23labels Online Documentation}
*/
abstract labels: ILabelDefaults;
/**
* Gets or sets the default node size.
* The values of this size will be used by {@link IGraph#createNodeAt}.
*
* <pre>
* // set the size of newly created nodes to 50, 30
* graph.nodeDefaults.size = new Size(50, 30)
* // make them appear as yellow ellipse
* graph.nodeDefaults.style = new ShapeNodeStyle({
*   shape: ShapeNodeShape.ELLIPSE,
*   fill: Fill.YELLOW
* })
* </pre>
*
* @throws {Exception} <b>(ArgumentError)</b> <code>value</code> contains one or more NaN values.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INodeDefaults%23size Online Documentation}
*/
abstract size: Size;
static $class: Class;
private $_INodeDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeDefaults;
}
export interface INodeSnapResultProvider {
}
/**
Interface that is used by the {@link GraphSnapContext} for {@link INode nodes} to collect a number of
{@link SnapResult}s during the move/edit operation.
This interface is queried from the {@link ILookup} of the
{@link INode}s that are being moved during the edit.
@see GraphSnapContext
@see NodeDecorator#nodeSnapResultProviderDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeSnapResultProvider Online Documentation}*/
export abstract class INodeSnapResultProvider {
/**
Called when a node is {@link IDragHandler#handleMove dragged} to add {@link SnapResult}s for
{@link OrthogonalSnapLine}s to which this node can potentially snap.
@param context The snap context which manages the snap lines and the settings.
@param evt The event argument to obtain the necessary information from and {@link CollectSnapResultsEventArgs#addSnapResult add results to}.
@param suggestedLayout The {@link INode#layout layout} of the node if the node would not snap.
@param node The node that is being moved.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeSnapResultProvider%23collectSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CRect%2CINode) Online Documentation}*/
abstract collectSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, suggestedLayout: Rect, node: INode): void;
/**
Creates an implementation of the interface {@link INodeSnapResultProvider} by using the given function as
implementation for its {@link INodeSnapResultProvider#collectSnapResults} method.
@param collectSnapResults A function for {@link INodeSnapResultProvider}'s single abstract method
{@link INodeSnapResultProvider#collectSnapResults}.
@returns An instance of the {@link INodeSnapResultProvider} interface based on the given function.*/
static create(collectSnapResults: (context: GraphSnapContext, evt: CollectSnapResultsEventArgs, suggestedLayout: Rect, node: INode) => void): INodeSnapResultProvider;
static $class: Class;
private $_INodeSnapResultProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeSnapResultProvider;
}
export interface IEdgeSnapResultProvider {
}
/**
Interface for classes that help during snapping of parts of edges.
An implementation of this class is obtained from the lookup of the {@link IEdge}s which are being edited during an
operation where {@link GraphSnapContext graph snapping} is involved.
@see EdgeDecorator#edgeSnapResultProviderDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeSnapResultProvider Online Documentation}*/
export abstract class IEdgeSnapResultProvider {
/**
Called to during the snapping operation whenever {@link SnapContext#addCollectSnapResultsListener snap results are collected}.
This method will be called for each collection of the results.
@param context The context that is using this instance.
@param evt The event argument to obtain the necessary information from and {@link CollectSnapResultsEventArgs#addSnapResult add results to}.
@param movementInfos The movement infos that describe the <code>edge</code>.
@param edge The edge for which the {@link SnapResult}s should be {@link CollectSnapResultsEventArgs#addSnapResult collected}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeSnapResultProvider%23collectSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CIListEnumerable%3CMovementInfo%3E%2CIEdge) Online Documentation}*/
abstract collectSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, movementInfos: IListEnumerable<MovementInfo>, edge: IEdge): void;
/**
Creates an implementation of the interface {@link IEdgeSnapResultProvider} by using the given function as
implementation for its {@link IEdgeSnapResultProvider#collectSnapResults} method.
@param collectSnapResults A function for {@link IEdgeSnapResultProvider}'s single abstract method
{@link IEdgeSnapResultProvider#collectSnapResults}.
@returns An instance of the {@link IEdgeSnapResultProvider} interface based on the given function.*/
static create(collectSnapResults: (context: GraphSnapContext, evt: CollectSnapResultsEventArgs, movementInfos: IListEnumerable<MovementInfo>, edge: IEdge) => void): IEdgeSnapResultProvider;
static $class: Class;
private $_IEdgeSnapResultProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeSnapResultProvider;
}
export interface IBendSnapResultProvider {
}
/**
Interface that is used by the {@link GraphSnapContext} for {@link IBend bends} to collect a number of
{@link SnapResult}s during the move/edit operation.
This interface is queried from the {@link ILookup} of the
{@link IBend}s that are being moved during the edit.
@see GraphSnapContext
@see BendDecorator#bendSnapResultProviderDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendSnapResultProvider Online Documentation}*/
export abstract class IBendSnapResultProvider {
/**
Called when a node is {@link IDragHandler#handleMove dragged} to add {@link SnapResult}s for
{@link OrthogonalSnapLine}s to which this bend can potentially snap.
@param context The snap context which manages the snap lines and the settings.
@param evt The event argument to obtain the necessary information from and {@link CollectSnapResultsEventArgs#addSnapResult add results to}.
@param suggestedLocation The {@link IBend#location location} of the bend if the bend would not snap.
@param bend The bend that is being moved.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IBendSnapResultProvider%23collectSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CPoint%2CIBend) Online Documentation}*/
abstract collectSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, suggestedLocation: Point, bend: IBend): void;
/**
Creates an implementation of the interface {@link IBendSnapResultProvider} by using the given function as
implementation for its {@link IBendSnapResultProvider#collectSnapResults} method.
@param collectSnapResults A function for {@link IBendSnapResultProvider}'s single abstract method
{@link IBendSnapResultProvider#collectSnapResults}.
@returns An instance of the {@link IBendSnapResultProvider} interface based on the given function.*/
static create(collectSnapResults: (context: GraphSnapContext, evt: CollectSnapResultsEventArgs, suggestedLocation: Point, bend: IBend) => void): IBendSnapResultProvider;
static $class: Class;
private $_IBendSnapResultProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IBendSnapResultProvider;
}
export interface IPortSnapResultProvider {
}
/**
Interface that is used by the {@link GraphSnapContext} for {@link IPort port} to collect a number of
{@link SnapResult}s during the move/edit operation.
This interface is queried from the {@link ILookup} of the
{@link IPort}s that are being moved during the edit.
@see GraphSnapContext
@see PortDecorator#portSnapResultProviderDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortSnapResultProvider Online Documentation}*/
export abstract class IPortSnapResultProvider {
/**
Called when a node is {@link IDragHandler#handleMove dragged} to add {@link SnapResult}s for
{@link OrthogonalSnapLine}s to which this bend can potentially snap.
@param context The snap context which manages the snap lines and the settings.
@param evt The event argument to obtain the necessary information from and {@link CollectSnapResultsEventArgs#addSnapResult add results to}.
@param suggestedLocation The {@link IBend#location location} of the bend if the bend would not snap.
@param port The port that is being moved
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortSnapResultProvider%23collectSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CPoint%2CIPort) Online Documentation}*/
abstract collectSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, suggestedLocation: Point, port: IPort): void;
/**
Creates an implementation of the interface {@link IPortSnapResultProvider} by using the given function as
implementation for its {@link IPortSnapResultProvider#collectSnapResults} method.
@param collectSnapResults A function for {@link IPortSnapResultProvider}'s single abstract method
{@link IPortSnapResultProvider#collectSnapResults}.
@returns An instance of the {@link IPortSnapResultProvider} interface based on the given function.*/
static create(collectSnapResults: (context: GraphSnapContext, evt: CollectSnapResultsEventArgs, suggestedLocation: Point, port: IPort) => void): IPortSnapResultProvider;
static $class: Class;
private $_IPortSnapResultProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortSnapResultProvider;
}
export interface INodeReshapeSnapResultProvider {
}
/**
Interface for an implementation which evaluates a list of given {@link SnapLine}s for a node which is resized and
adds a set of {@link SnapResult}s for sizes to which this node can potentially snap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeReshapeSnapResultProvider Online Documentation}*/
export abstract class INodeReshapeSnapResultProvider {
/**
Called when a node's handle is {@link IDragHandler#handleMove dragged} to add {@link SnapResult}s for
{@link OrthogonalSnapLine}s or sizes to which this node can potentially snap during resizing.
@param context The snap context which manages the snap lines and the settings.
@param evt The event argument to obtain the context from and add the results to.
@param node The node that is being reshaped.
@param reshapeContext Carries information about the reshape operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeReshapeSnapResultProvider%23collectSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2CReshapeRectangleContext) Online Documentation}*/
abstract collectSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, reshapeContext: ReshapeRectangleContext): void;
/**
Creates an implementation of the interface {@link INodeReshapeSnapResultProvider} by using the given function as
implementation for its {@link INodeReshapeSnapResultProvider#collectSnapResults} method.
@param collectSnapResults A function for {@link INodeReshapeSnapResultProvider}'s single abstract method
{@link INodeReshapeSnapResultProvider#collectSnapResults}.
@returns An instance of the {@link INodeReshapeSnapResultProvider} interface based on the given function.*/
static create(collectSnapResults: (context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, reshapeContext: ReshapeRectangleContext) => void): INodeReshapeSnapResultProvider;
static $class: Class;
private $_INodeReshapeSnapResultProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeReshapeSnapResultProvider;
}
export interface ReshapeRectangleContext extends YObject {
}
/**
Carries information about a reshape operation.
This class is mainly intended for use by {@link INodeReshapeSnapResultProvider#collectSnapResults}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext Online Documentation}*/
export class ReshapeRectangleContext {
/**
Initializes a new instance of the {@link ReshapeRectangleContext} class.
@param initialBounds The initial bounds of the node.
@param minimumSize The minimum size of the node.
@param maximumSize The maximum size of the node.
@param minimumEnclosedArea The minimum area the node's layout needs to encompass. Providing an {@link Rect#EMPTY empty rectangle} will disable this restriction.
@param maximumBoundingArea The maximum bounding area the node's layout may fill. Providing an {@link Rect#INFINITE infinite rectangle} will disable this restriction.
@param reshapePosition The position of the reshape handle.
@param topLeftChangeFactor The factor by which the top left corner of the node layout changes when the handle is dragged.
@param bottomRightChangeFactor The factor by which the bottom right corner of the node layout changes when the handle is dragged.
@param sizeChangeFactor The factor by which the size of the node layout changes when the handle is dragged.
@param reshapePolicy The policy used for aspect ratio resizing.
@param ratio The aspect ratio used for aspect ratio resizing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext Online Documentation}*/
constructor(initialBounds: Rect | RectConvertible, minimumSize: ISize | ISizeConvertible, maximumSize: ISize | ISizeConvertible, minimumEnclosedArea: IRectangle | IRectangleConvertible, maximumBoundingArea: IRectangle | IRectangleConvertible, reshapePosition: HandlePositions | HandlePositionsConvertible, topLeftChangeFactor: Point | PointConvertible, bottomRightChangeFactor: Point | PointConvertible, sizeChangeFactor: Size | SizeConvertible, reshapePolicy: ReshapePolicy | ReshapePolicyStringValues, ratio: number);
/**
Gets the initial bounds of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23initialBounds Online Documentation}*/
initialBounds: Rect;
/**
Gets the minimum size of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23minimumSize Online Documentation}*/
minimumSize: ISize;
/**
Gets or sets the maximum size of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23maximumSize Online Documentation}*/
maximumSize: ISize;
/**
Gets minimum area the node's layout needs to encompass
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23minimumEnclosedArea Online Documentation}*/
minimumEnclosedArea: IRectangle;
/**
Gets the maximal bounds the node's layout may fill.
The value is stored by reference. The default is
{@link Rect#INFINITE}, which does not restrict the area at all.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23maximumBoundingArea Online Documentation}*/
maximumBoundingArea: IRectangle;
/**
Gets the policy if and how the change in size of the reshapeable keeps the {@link ReshapeRectangleContext#ratio} during the reshape gesture.
@see ReshapeRectangleContext#ratio
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23reshapePolicy Online Documentation}*/
reshapePolicy: ReshapePolicy;
/**
Gets the ratio between the change of the width and the change of the height that is kept during the reshape gesture.
This ratio has to be considered if {@link ReshapeRectangleContext#reshapePolicy} is not {@link ReshapePolicy#NONE}.
@see ReshapeRectangleContext#reshapePolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23ratio Online Documentation}*/
ratio: number;
/**
Gets the position of the reshape handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23reshapePosition Online Documentation}*/
reshapePosition: HandlePositions;
/**
Gets the factor by which the top left corner of the node layout changes when the handle is dragged.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23topLeftChangeFactor Online Documentation}*/
topLeftChangeFactor: Point;
/**
Gets the factor by which the bottom right corner of the node layout changes when the handle is dragged.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23bottomRightChangeFactor Online Documentation}*/
bottomRightChangeFactor: Point;
/**
Gets the factor by which the size of the node layout changes when the handle is dragged.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReshapeRectangleContext%23sizeChangeFactor Online Documentation}*/
sizeChangeFactor: Size;
static $class: Class;
private $_ReshapeRectangleContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReshapeRectangleContext;
}
export interface IPort extends ILabelOwner {
}
/**
* The interface used in an {@link IGraph} implementation for {@link IEdge}s to connect to.
* This interface provides read-only access to the properties of a port. In order to modify the state of an instance use
* the various methods provided by the {@link IGraph} this instance belongs to. Ports are owned by {@link IPortOwner IPortOwners},
* normally an {@link INode}, but this can also be an {@link IEdge} in special graph implementations. To obtain
* the {@link IEdge} instances that are connected to a certain port instance, applications need to use the {@link IGraph#edgesAt}
* method provided by {@link IGraph}. Zero or more edges may be connected to a port, depending on the implementation
* of the graph. A port is also an {@link ILabelOwner}, hence it may own {@link ILabel labels}. Like all items in an <code>IGraph</code>, this
* item supports the {@link ILookup#lookup} method that can be used to query additional aspects of the item.
*
* <br><b>Working with a port in the graph</b>
* <pre>
* const graph = graphComponent.graph
* const node = graph.createNodeAt(new Point(0, 0))
*
* // add a port to the node: at the center with default style (invisible)
* const port = graph.addPort(node, FreeNodePortLocationModel.NODE_CENTER_ANCHORED)
*
* // the newly created port is part of the graph
* console.log(graph.contains(port)) // true
* console.log(graph.ports.size) // 1
*
* // the port belongs to its owner
* console.log(port.owner === node) // true
* console.log(node.ports.size) // 1
* console.log(node.ports.get(0) === port) // true
*
* // removing the port removes it from the graph
* graph.remove(port)
* console.log(graph.contains(port)) // false
* console.log(graph.ports.size) // 0
* // and from its owner
* console.log(port.owner === null) // true
* console.log(node.ports.size) // 0
* </pre>
*
* <br><b>Ports cannot live without an owner in the graph</b>
* <pre>
* console.log(graph.contains(port)) // true
* graph.remove(port.owner)
* console.log(graph.contains(port)) // false
* </pre>
*
* <br><b>Unused ports may be removed automatically</b>
* <pre>
* const node1 = graph.createNode()
* const node2 = graph.createNode()
* const port1_1 = graph.addPort(node1)
* const port1_2 = graph.addPort(node1)
* const port2 = graph.addPort(node2)
* const edge1 = graph.createEdge(port1_1, port2)
* const edge2 = graph.createEdge(port1_2, port2)
*
* // enable auto cleanup
* graph.nodeDefaults.ports.autoCleanUp = true
* graph.remove(edge1)
* // port1_1 is removed with the edge
* console.log(graph.contains(port1_1)) // false
* // port2 is not removed since edge2 still is linked to it
* console.log(graph.contains(port2))
* // disable auto cleanup
* graph.nodeDefaults.ports.autoCleanUp = false
* graph.remove(edge2)
* // no port is removed with the edge now
* console.log(graph.contains(port1_2)) // true
* console.log(graph.contains(port2)) // true
* </pre>
*
* The graph model with all relevant types and their relationships is presented in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph The Graph Model}.
*
* Using the look-up mechanism is explained in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_concepts_lookup Service Locator Pattern: Lookup}.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPort Online Documentation}
*/
export abstract class IPort {
/**
* Gets the owner of this port.
* In traditional {@link IGraph} implementations, this will be an {@link INode} and can safely be cast to one. In
* order to get to the {@link IEdge}s that connect to this instance, use {@link IGraph}'s {@link IGraph#edgesAt}
* method.
*
* Note that for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, the owner can't be changed after creation and the port has to be deleted and recreated with a new
* owner.
*
* <br><b>Relationship between port and port owner</b>
* <pre>
* // add a port to the owner
* const port = graph.addPort(owner)
*
* // the port's Owner property is set to the owner
* console.log(port.owner === owner) // true
* // the port is in its owner's Ports collection
* console.log(owner.ports.includes(port)) // true
* </pre>
*
* @see IPortOwner#ports
* @see IGraph#addPort
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPort%23owner Online Documentation}
*/
abstract readonly owner: IPortOwner | null;
/**
* Gets the style that is responsible for the visual representation of this port in a {@link CanvasComponent}.
* Note that the style instance associated with a port instance may be shared between multiple port instances and that the
* modification of this style will result in a change of the appearance of all ports that are associated with the same
* style instance.
*
* To change the style for instances of the default implementation that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#setStyle}
*
* <br><b>Setting the style of a port</b>
* <pre>
* graph.setStyle(port, portStyle)
* </pre>
*
* <br><b>Obtaining the style of a port</b>
* <pre>
* const style = port.style
* </pre>
*
* @see IGraph#setStyle
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPort%23style Online Documentation}
*/
abstract readonly style: IPortStyle;
/**
* Gets the {@link IPortLocationModelParameter} that is used to determine the location of this port.
* The port location can be calculated using the {@link IPortLocationModelParameter#model}'s {@link IPortLocationModel#getLocation} method or the methods {@link IPort#location} and
* {@link IPort#dynamicLocation}.
*
* To change the port location parameter for instances of the default implementation that were created via the factory
* methods on {@link IGraph}, use {@link IGraph#setPortLocationParameter}
*
* Note that parameters may be shared across port instances.
*
* <br><b>Setting the location parameter to a port</b>
* <pre>
* graph.setPortLocationParameter(port, FreeNodePortLocationModel.NODE_CENTER_ANCHORED)
* </pre>
*
* <br><b>Obtaining the location parameter from a port</b>
* <pre>
* const parameter = port.locationParameter
* </pre>
*
* <br><b>Setting the location of a port</b>
* <pre>
* graph.setPortLocation(port, new Point(x, y))
* </pre>
*
* <br><b>Obtaining the location of a port</b>
* <pre>
* const location = port.location
* </pre>
*
* @see IGraph#setPortLocationParameter
* @see IPort#location
* @see IPort#dynamicLocation
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPort%23locationParameter Online Documentation}
*/
abstract readonly locationParameter: IPortLocationModelParameter;
/**
Gets a snapshot of the current location of the port.
Unlike {@link IPort#dynamicLocation} this does not return a dynamic point that always refers to the current location.
@see IPort#locationParameter
@see IPortLocationModel#getLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPort%23location Online Documentation}*/
readonly location: Point;
/**
Gets a live view of the location of the port in world coordinates.
The location is the anchor for the edges, that connect to this port, however it is up to the visualization logic where
exactly the visual part of an edge will end. As this will yield a live view, it is up to the client to copy the values
if a snapshot of the state is needed. In order to modify the location of a port, use the {@link IGraph#setPortLocationParameter}
in {@link IGraph}.
@see IPort#locationParameter
@see IPortLocationModel#getLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPort%23dynamicLocation Online Documentation}*/
readonly dynamicLocation: IPoint;
static $class: Class;
private $_IPort: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPort;
}
export interface IPortOwner extends IModelItem {
}
/**
* The common interface for items in an {@link IGraph} that may own {@link IPort ports}.
* Typically this interface is actively implemented by {@link INode}'s in an {@link IGraph}. To get to the edge
* instances that connect to the ports, the {@link IGraph#edgesAt} method can be used. Alternatively for each {@link IPort}
* in {@link IPortOwner#ports}, one can use the {@link IGraph#edgesAt} method provided by {@link IGraph}. Like all items in an <code>IGraph</code>,
* this item supports the {@link ILookup#lookup} method that can be used to query additional aspects of the item.
*
* <br><b>Relationship between port and port owner</b>
* <pre>
* // add a port to the owner
* const port = graph.addPort(owner)
*
* // the port's Owner property is set to the owner
* console.log(port.owner === owner) // true
* // the port is in its owner's Ports collection
* console.log(owner.ports.includes(port)) // true
* </pre>
*
* @see INode
* @see IEdge
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortOwner Online Documentation}
*/
export abstract class IPortOwner {
/**
* Gets a collection of {@link IPort ports} that are owned by this instance.
* This gives access to a read-only live view of the ports, i.e. the collection can change over time, as well as the ports
* contained in it. If a snapshot of the current state is needed, one needs to copy the collection.
*
* To modify the port collection for instances of the default implementations that were created via the factory methods on
* {@link IGraph}, use {@link IGraph#addPort} and {@link IGraph#remove}.
*
* <br><b>Adding a port to a node.</b>
* <pre>
* // add a port at x,y with default style
* const port1 = graph.addPort(node, new Point(x, y))
*
* const portStyle = new NodeStylePortStyleAdapter()
*
* portStyle.nodeStyle = new ShapeNodeStyle({ shape: ShapeNodeShape.ELLIPSE })
* portStyle.renderSize = new Size(3, 3)
*
* // add a port at x,y with the given style and tag (user object)
* const port2 = graph.addPortAt(node, new Point(x, y), portStyle, userObject)
*
* // add a port at the center of the node with default style
* const port3 = graph.addPort(node, FreeNodePortLocationModel.NODE_CENTER_ANCHORED)
*
* // add a port at the center of the node with the given style and tag (user object)
* const port4 = graph.addPort(
*   node,
*   FreeNodePortLocationModel.NODE_CENTER_ANCHORED,
*   portStyle,
*   userObject
* )
* </pre>
*
* <br><b>Iterating over all ports of a node.</b>
* <pre>
* for (const port of node.ports) {
*   // ...
* }
* </pre>
*
* <br><b>Removing a port from a node.</b>
* <pre>
* graph.remove(port)
* </pre>
*
* @see IPort#owner
* @see IGraph#addPort
* @see IGraph#remove
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortOwner%23ports Online Documentation}
*/
abstract readonly ports: IListEnumerable<IPort>;
static $class: Class;
private $_IPortOwner: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortOwner;
}
export interface IPortCandidate {
}
/**
An interface for possible {@link IPort} candidates used by {@link IPortCandidateProvider} and the like.
Note that if an instance's {@link IPortCandidate#validity} property is {@link PortCandidateValidity#DYNAMIC} this instance {@link IPortCandidate#getPortCandidateAt}
method serves as a factory for the actual candidates.
@see IEdgeReconnectionPortCandidateProvider
@see IPortCandidateProvider
@see DefaultPortCandidate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidate Online Documentation}*/
export abstract class IPortCandidate {
/**
If the client decides to use this port candidate, this method will serve as a factory to create the instance.
If this candidate wraps an existing port, it may return that {@link IPortCandidate#port instance}. If this instance's {@link IPortCandidate#validity} is {@link PortCandidateValidity#DYNAMIC}
this method may throw an {@link Exception NotSupportedError}.
@param context The input mode context for which the port will be created. Implementations can depend on it providing an {@link IGraph}
instance in the lookup of the instance.
@returns The port instance to use if this candidate has been chosen. This method may not return <code>null</code>.
@throws {Exception} <b>(NotSupportedError)</b> If the {@link IPortCandidate#validity} is {@link PortCandidateValidity#DYNAMIC}.
@see IInputModeContext#graph
@see IPortCandidate#port
@see IPortCandidate#getPortCandidateAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidate%23createPort(IInputModeContext) Online Documentation}*/
abstract createPort(context: IInputModeContext): IPort;
/**
Returns a candidate that is derived from this instance that best fits the provided <code>location</code>.
This method needs to be called by clients if the {@link IPortCandidate#validity} of this instance is {@link PortCandidateValidity#DYNAMIC}. In
that case calling this method must return an instance whose validity is not {@link PortCandidateValidity#DYNAMIC}.
@param context The input mode context for which a concrete candidate is queried.
@param location The location for which a candidate should be returned.
@returns A {@link IPortCandidate} implementation whose {@link IPortCandidate#validity} is guaranteed to be non-{@link PortCandidateValidity#DYNAMIC}.
@see IPortCandidate#validity
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidate%23getPortCandidateAt(IInputModeContext%2CPoint) Online Documentation}*/
abstract getPortCandidateAt(context: IInputModeContext, location: Point): IPortCandidate;
/**
Gets a user defined tag for this candidate.
The interpretation of this value is implementation dependent and may be used for visual feedback or interactive
behavior.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidate%23candidateTag Online Documentation}*/
abstract readonly candidateTag: any | null;
/**
Gets whether this instance is a valid port candidate.
Implementations of {@link IPortCandidateProvider} may return sets of port candidates where some of them are marked
as invalid e.g. in order to provide visual feedback. If the value is {@link PortCandidateValidity#DYNAMIC}, method
{@link IPortCandidate#getPortCandidateAt} needs to be used to determine a concrete candidate instance.
@see IPortCandidate#getPortCandidateAt
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidate%23validity Online Documentation}*/
abstract readonly validity: PortCandidateValidity;
/**
Gets the model parameter that will be used for the {@link IPort#locationParameter} if this candidate is chosen.
Note that the value of this property is undefined if this instance's {@link IPortCandidate#validity} is {@link PortCandidateValidity#DYNAMIC}. In
this case, the parameter needs to be obtained via the {@link IPortCandidate#getPortCandidateAt} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidate%23locationParameter Online Documentation}*/
abstract readonly locationParameter: IPortLocationModelParameter;
/**
Gets the possible owner of the port to be created for this candidate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidate%23owner Online Documentation}*/
abstract readonly owner: IPortOwner;
/**
Gets the port instance that is already live in the graph if this candidate wraps an existing port or <code>null</code> otherwise.
@see IPortCandidate#createPort
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidate%23port Online Documentation}*/
abstract readonly port: IPort | null;
static $class: Class;
private $_IPortCandidate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortCandidate;
}
/**
Enumeration for the different possible {@link IPortCandidate#validity} states.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateValidity Online Documentation}*/
export enum PortCandidateValidity {
/**
The candidate is valid and can be used to connect an edge to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateValidity%23VALID Online Documentation}*/
VALID,
/**
The candidate should not be used to connect an edge to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateValidity%23INVALID Online Documentation}*/
INVALID,
/**
The candidate cannot be used to create a port, instead {@link IPortCandidate#getPortCandidateAt} needs to be called
to obtain a {@link IPortCandidate} that is either {@link PortCandidateValidity#VALID} or
{@link PortCandidateValidity#INVALID}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateValidity%23DYNAMIC Online Documentation}*/
DYNAMIC
}
/** Automatic conversion to {@link PortCandidateValidity}. */
type PortCandidateValidityStringValues = "valid" | "invalid" | "dynamic";
export interface IPortCandidateProvider {
}
/**
This interface is used to provide {@link IPortCandidate}s for a specific {@link IPortOwner}.
It is used in the context of the items that will later become the owner ({@link IPortOwner}) of the ports obtained
from the candidates. They are mainly used during edge creation where there is no real edge yet (only the dummy edge) and
thus they need to use the opposite {@link IPortCandidate} to understand the context.
Implementations of this interface may be queried using the {@link ILookup#lookup} method of {@link IPortOwner}
implementations like {@link INode}.
Note that this interface is similar to the interface {@link IEdgeReconnectionPortCandidateProvider} in the sense
that they will both provide {@link IPortCandidate} instances in a certain query context. They are used in different
query contexts, though. The two interfaces can come together in concrete implementations of
{@link IEdgeReconnectionPortCandidateProvider}: an edge could potentially be connected to ports at any given {@link IPortOwner}
and thus the {@link IPortCandidateProvider} interface can be queried from the respective {@link IPortOwner}
that the edge will be reconnected to.
@see NodeDecorator#portCandidateProviderDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider Online Documentation}*/
export abstract class IPortCandidateProvider {
/**
* Creates a generic composite implementation for the {@link IPortCandidateProvider} interface.
* <b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
*
* This method can directly be used as factory for a {@link LookupDecorator}
*
* <pre>
* graph.decorator.nodeDecorator.portCandidateProviderDecorator.setFactory(node =>
*   IPortCandidateProvider.combine(
*     IPortCandidateProvider.fromNodeCenter(node),
*     IPortCandidateProvider.fromShapeGeometry(node, 0.25, 0.75)
*   )
* )
* </pre>
*
* @param providers The providers to create the composite from.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23combine(IPortCandidateProvider) Online Documentation}
*/
static combine(...providers: IPortCandidateProvider[]): IPortCandidateProvider;
/**
* Creates a generic composite implementation for the {@link IPortCandidateProvider} interface.
* <b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
*
* This method can directly be used as factory for a {@link LookupDecorator}
*
* <pre>
* graph.decorator.nodeDecorator.portCandidateProviderDecorator.setFactory(node =>
*   IPortCandidateProvider.combine(
*     IPortCandidateProvider.fromNodeCenter(node),
*     IPortCandidateProvider.fromShapeGeometry(node, 0.25, 0.75)
*   )
* )
* </pre>
*
* @param providers The providers to create the composite from.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23combine(IEnumerable%3CIPortCandidateProvider%3E) Online Documentation}
*/
static combine(providers: IEnumerable<IPortCandidateProvider> | IEnumerableConvertible<IPortCandidateProvider>): IPortCandidateProvider;
/**
Creates an implementation of the interface {@link IPortCandidateProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link IPortCandidateProvider} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IPortCandidateProvider} interface.
@returns An instance of the {@link IPortCandidateProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
getSourcePortCandidates(context: IInputModeContext, target: IPortCandidate): IEnumerable<IPortCandidate>;
getTargetPortCandidates(context: IInputModeContext, source: IPortCandidate): IEnumerable<IPortCandidate>;
getAllSourcePortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
getAllTargetPortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
}): IPortCandidateProvider;
/**
Creates an implementation of an {@link IPortCandidateProvider} that provides only the given port candidates.
<b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
@param candidates The port candidates the provider should provide.
@returns A new {@link IPortCandidateProvider}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23fromCandidates(IPortCandidate) Online Documentation}*/
static fromCandidates(...candidates: IPortCandidate[]): PortCandidateProviderBase;
/**
Creates an implementation of an {@link IPortCandidateProvider} that provides only the given port candidates.
<b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
@param candidates The port candidates the provider should provide.
@returns A new {@link IPortCandidateProvider}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23fromCandidates(IEnumerable%3CIPortCandidate%3E) Online Documentation}*/
static fromCandidates(candidates: IEnumerable<IPortCandidate> | IEnumerableConvertible<IPortCandidate>): PortCandidateProviderBase;
/**
* Creates an implementation of the {@link IPortCandidateProvider} interface that returns the ports that exist in the
* given {@link IPortOwner#ports}'s {@link IPortOwner} collection.
* <b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
*
* This method can directly be used as factory for a {@link LookupDecorator}
*
* <pre>
* graph.decorator.nodeDecorator.portCandidateProviderDecorator.setFactory(node =>
*   IPortCandidateProvider.fromExistingPorts(node)
* )
* </pre>
*
* @param owner The owner of the port collection.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23fromExistingPorts(IPortOwner) Online Documentation}
*/
static fromExistingPorts(owner: IPortOwner): IPortCandidateProvider;
/**
* Creates a trivial implementation of an {@link IPortCandidateProvider} that always returns exactly one candidate
* that is centered at the node's {@link INode#layout}.
* <b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
*
* This method can directly be used as factory for a {@link LookupDecorator}
*
* <pre>
* graph.decorator.nodeDecorator.portCandidateProviderDecorator.setFactory(node =>
*   IPortCandidateProvider.fromNodeCenter(node)
* )
* </pre>
*
* @param node The node to get the layout's center from.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23fromNodeCenter(INode) Online Documentation}
*/
static fromNodeCenter(node: INode): PortCandidateProviderBase;
/**
* Creates a simple implementation of an {@link IPortCandidateProvider} that always returns a candidate that uses the
* default {@link IPortDefaults#locationParameter} for the corresponding {@link IPortDefaults}.
* <b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
*
* This method can directly be used as factory for a {@link LookupDecorator}
*
* <pre>
* graph.decorator.nodeDecorator.portCandidateProviderDecorator.setFactory(node =>
*   IPortCandidateProvider.fromPortDefaults(node)
* )
* </pre>
*
* @param owner
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23fromPortDefaults(IPortOwner) Online Documentation}
*/
static fromPortDefaults(owner: IPortOwner): IPortCandidateProvider;
/**
* Creates an implementation of {@link IPortCandidateProvider} that works for {@link IPortOwner} implementations
* that have {@link IShapeGeometry} instances in their lookup.
* <b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
*
* <pre>
* graph.decorator.nodeDecorator.portCandidateProviderDecorator.setFactory(node =>
*   IPortCandidateProvider.fromShapeGeometry(node, 0.25, 0.75)
* )
* </pre>
*
* @param owner The owner to receive the shape geometry from
* @param minimumSegmentLength the minimum length a segment needs to have in order to be used to add port candidates.
* @param ratios A number of double values that are interpreted as ratio values between 0.0 and 1.0. If none are given, the provider
* inserts a port candidate at the center of each straight line segment.
* @param addExistingPortsEnabled determines whether {@link PortCandidateProviderBase#addExistingPorts existing ports should be added to the list of ports}.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23fromShapeGeometry(IPortOwner%2Cboolean%2Cnumber%2Cnumber) Online Documentation}
*/
static fromShapeGeometry(owner: IPortOwner, addExistingPortsEnabled: boolean, minimumSegmentLength: number, ...ratios: number[]): PortCandidateProviderBase;
/**
* Creates an implementation of {@link IPortCandidateProvider} that works for {@link IPortOwner} implementations
* that have {@link IShapeGeometry} instances in their lookup.
* <b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
*
* <pre>
* graph.decorator.nodeDecorator.portCandidateProviderDecorator.setFactory(node =>
*   IPortCandidateProvider.fromShapeGeometry(node, 0.25, 0.75)
* )
* </pre>
*
* @param owner The owner to receive the shape geometry from
* @param ratios A number of double values that are interpreted as ratio values between 0.0 and 1.0. If none are given, the provider
* inserts a port candidate at the center of each straight line segment.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23fromShapeGeometry(IPortOwner%2Cnumber) Online Documentation}
*/
static fromShapeGeometry(owner: IPortOwner, ...ratios: number[]): PortCandidateProviderBase;
/**
* Creates a simple implementation of an {@link IPortCandidateProvider} that returns unoccupied ports at a given
* entity.
* <b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
*
* This method can directly be used as factory for a {@link LookupDecorator}
*
* <pre>
* graph.decorator.nodeDecorator.portCandidateProviderDecorator.setFactory(node =>
*   IPortCandidateProvider.fromUnoccupiedPorts(node)
* )
* </pre>
*
* @param owner The owner to query the ports from.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23fromUnoccupiedPorts(IPortOwner) Online Documentation}
*/
static fromUnoccupiedPorts(owner: IPortOwner): IPortCandidateProvider;
/**
Returns all source port candidates that belong to the context of this provider.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param context The context for which the candidates should be provided.
@returns An enumerable over all source port candidates that are associated with the current context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23getAllSourcePortCandidates(IInputModeContext) Online Documentation}*/
abstract getAllSourcePortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
/**
Returns all target port candidates that belong to the context of this provider.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param context The context for which the candidates should be provided.
@returns An enumerable over all target port candidates that are associated with the current context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23getAllTargetPortCandidates(IInputModeContext) Online Documentation}*/
abstract getAllTargetPortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
/**
Returns all port candidates that apply for the provided opposite port candidate.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param target The opposite port candidate.
@param context The context for which the candidates should be provided.
@returns An enumerable over all port candidates that are associated with the given opposite port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23getSourcePortCandidates(IInputModeContext%2CIPortCandidate) Online Documentation}*/
abstract getSourcePortCandidates(context: IInputModeContext, target: IPortCandidate): IEnumerable<IPortCandidate>;
/**
Returns all port candidates that apply for the provided opposite port candidate.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param context The context for which the candidates should be provided.
@param source The opposite port candidate.
@returns An enumerable over all port candidates that are associated with the given opposite port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23getTargetPortCandidates(IInputModeContext%2CIPortCandidate) Online Documentation}*/
abstract getTargetPortCandidates(context: IInputModeContext, source: IPortCandidate): IEnumerable<IPortCandidate>;
/**
* A generic implementation of the {@link IPortCandidateProvider} interface that provides no candidates.
* <b>Warning:</b> This field is not available unless the module <b>view-editor</b> is loaded.
*
* <pre>
* graph.decorator.nodeDecorator.portCandidateProviderDecorator.setImplementation(
*   IPortCandidateProvider.NO_CANDIDATES
* )
* </pre>
*
* @const
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateProvider%23NO_CANDIDATES Online Documentation}
*/
static NO_CANDIDATES: IPortCandidateProvider;
static $class: Class;
private $_IPortCandidateProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortCandidateProvider;
}
export interface IEdgeReconnectionPortCandidateProvider {
}
/**
This interface can be used to query {@link IPortCandidate}s for a specific existing {@link IEdge}
As the name implies this is only used in the context of reconnecting existing edges. It is queried for existing edges to
find out about possible alternative port candidates that given edge could connect to. It is used during gestures where
an edge is reconnected to another port(candidate), i.e. when the source or target port handle of an edge is moved and
the source or target port properties of the edge get set to the new {@link IPortCandidate}s.
Implementations of this interface can be queried using the {@link ILookup#lookup} method of an {@link IEdge}
implementation. This interface provides alternative candidates for the {@link IEdge#sourcePort} or {@link IEdge#targetPort} of an edge.
Note that this interface is similar to the interface {@link IPortCandidateProvider} in the sense that they will
both provide {@link IPortCandidate} instances in a certain query context. They are used in different query
contexts, though. The two interfaces can come together in concrete implementations of
{@link IEdgeReconnectionPortCandidateProvider}: an edge could potentially be connected to ports at any given {@link IPortOwner}
and thus the {@link IPortCandidateProvider} interface can be queried from the respective {@link IPortOwner}
that the edge will be reconnected to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReconnectionPortCandidateProvider Online Documentation}*/
export abstract class IEdgeReconnectionPortCandidateProvider {
/**
Creates an implementation of the interface {@link IEdgeReconnectionPortCandidateProvider} from the given
definition.
The given object must contain exactly the abstract members of the {@link IEdgeReconnectionPortCandidateProvider}
interface. No member may be omitted and no additional members can be added. While additional custom members can later be
added directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IEdgeReconnectionPortCandidateProvider}
interface.
@returns An instance of the {@link IEdgeReconnectionPortCandidateProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
getSourcePortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
getTargetPortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
}): IEdgeReconnectionPortCandidateProvider;
/**
* Creates an implementation of the {@link IEdgeReconnectionPortCandidateProvider} interface that returns the
* candidates provided by the source and target port's owner's {@link IPortCandidateProvider} implementations or the
* existing edge's port, if no such provider is registered.
* This instance uses the lookup mechanism of the nodes to query the providers.
*
* <b>Warning:</b> This method is not available unless the module <b>view-editor</b> is loaded.
*
* This method can directly be used as factory for a {@link LookupDecorator}
*
* <pre>
* graph.decorator.edgeDecorator.edgeReconnectionPortCandidateProviderDecorator.setFactory(edge =>
*   IEdgeReconnectionPortCandidateProvider.fromSourceAndTarget(edge)
* )
* </pre>
*
* @param edge
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReconnectionPortCandidateProvider%23fromSourceAndTarget(IEdge) Online Documentation}
*/
static fromSourceAndTarget(edge: IEdge): IEdgeReconnectionPortCandidateProvider;
/**
Returns all source port candidates that may be used for the edge.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param context The context for which the candidates should be provided.
@returns An enumerable over all source port candidates that are associated with the current context edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReconnectionPortCandidateProvider%23getSourcePortCandidates(IInputModeContext) Online Documentation}*/
abstract getSourcePortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
/**
Returns all source port candidates that may be used for the edge.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param context The context for which the candidates should be provided.
@returns An enumerable over all target port candidates that are associated with the current context edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReconnectionPortCandidateProvider%23getTargetPortCandidates(IInputModeContext) Online Documentation}*/
abstract getTargetPortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
/**
* Creates an implementation of the {@link IEdgeReconnectionPortCandidateProvider} interface that returns a union of
* all {@link IPortCandidateProvider#getAllSourcePortCandidates source port candidates} or {@link IPortCandidateProvider#getAllTargetPortCandidates target port candidates}, respectively, of nodes in the graph.
* This implementation can be used to allow for {@link PortRelocationHandle relocating an edge's port} to any available source or target port in the
* graph.
*
* <b>Warning:</b> This field is not available unless the module <b>view-editor</b> is loaded.
*
* <pre>
* graph.decorator.edgeDecorator.edgeReconnectionPortCandidateProviderDecorator.setImplementation(
*   IEdgeReconnectionPortCandidateProvider.ALL_NODE_CANDIDATES
* )
* </pre>
*
* @const
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReconnectionPortCandidateProvider%23ALL_NODE_CANDIDATES Online Documentation}
*/
static ALL_NODE_CANDIDATES: IEdgeReconnectionPortCandidateProvider;
/**
* Creates an implementation of the {@link IEdgeReconnectionPortCandidateProvider} interface that returns a union of
* all {@link IPortCandidateProvider#getAllSourcePortCandidates source port candidates} or {@link IPortCandidateProvider#getAllTargetPortCandidates target port candidates}, respectively, of nodes and edges in the graph.
* This implementation can be used to allow for {@link PortRelocationHandle relocating an edge's port} to any available source or target port in the
* graph.
*
* <b>Warning:</b> This field is not available unless the module <b>view-editor</b> is loaded.
*
* <pre>
* graph.decorator.edgeDecorator.edgeReconnectionPortCandidateProviderDecorator.setImplementation(
*   IEdgeReconnectionPortCandidateProvider.ALL_NODE_AND_EDGE_CANDIDATES
* )
* </pre>
*
* @const
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReconnectionPortCandidateProvider%23ALL_NODE_AND_EDGE_CANDIDATES Online Documentation}
*/
static ALL_NODE_AND_EDGE_CANDIDATES: IEdgeReconnectionPortCandidateProvider;
static $class: Class;
private $_IEdgeReconnectionPortCandidateProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeReconnectionPortCandidateProvider;
}
export interface IPortDefaults {
}
/**
* Interface used by {@link IGraph} to declare and obtain the defaults for ports at nodes and edges.
* Note that changing these defaults does <i>not</i> change properties of already created model items. Rather, only items created
* after the change are affected.
*
* <br><b>Setting defaults for ports</b>
* <pre>
* // Ports on nodes and edges have different IPortDefault instances.
* // These can be retrieved from their owner type's defaults
*
* // the defaults for node ports can be set on the IPortDefaults instance
* // found at the Ports property of the node defaults
* graph.nodeDefaults.ports.autoCleanUp = false
* graph.nodeDefaults.ports.locationParameter = FreeNodePortLocationModel.NODE_CENTER_ANCHORED
*
* // the defaults for edge ports can be set on the IPortDefaults instance
* // found at the Ports property of the edge defaults
* graph.edgeDefaults.ports.autoCleanUp = false
* graph.edgeDefaults.ports.locationParameter = BendAnchoredPortLocationModel.FIRST_BEND
* </pre>
*
* @see IPort
* @see INodeDefaults#ports
* @see IEdgeDefaults#ports
* @see IGraph#nodeDefaults
* @see IGraph#edgeDefaults
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortDefaults Online Documentation}
*/
export abstract class IPortDefaults {
/**
Factory method that returns a location model parameter instance for use with newly created ports.
Most implementations will yield either, a {@link ICloneable#clone clone} of or the {@link IPortDefaults#locationParameter} property, if {@link IPortDefaults#shareLocationParameterInstance} is enabled, but they might use more complicated
logic, too.
@param owner The owner of the port that will be created.
@returns The parameter to use, which for most implementations is either a {@link ICloneable#clone clone} of or the {@link IPortDefaults#locationParameter} property, if {@link IPortDefaults#shareLocationParameterInstance} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortDefaults%23getLocationParameterInstance(IPortOwner) Online Documentation}*/
abstract getLocationParameterInstance(owner: IPortOwner): IPortLocationModelParameter;
/**
Factory method that returns a style instance for use with newly created ports.
Most implementations will yield either, a {@link ICloneable#clone clone} of or the {@link IPortDefaults#style} property, if {@link IPortDefaults#shareStyleInstance} is enabled, but they might use more complicated
logic, too.
@param owner The owner of the port that will be created.
@returns The style to use, which for most implementations is either a {@link ICloneable#clone clone} of or the {@link IPortDefaults#style} property, if {@link IPortDefaults#shareStyleInstance} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortDefaults%23getStyleInstance(IPortOwner) Online Documentation}*/
abstract getStyleInstance(owner: IPortOwner): IPortStyle;
/**
* Gets or sets the style to use for ports.
* Depending on the setting of {@link IPortDefaults#shareStyleInstance}, the {@link IPortDefaults#getStyleInstance} method should return a {@link ICloneable#clone clone} of this
* instance or the very same instance.
*
* <pre>
* graph.nodeDefaults.ports.style = new NodeStylePortStyleAdapter(
*   new ShapeNodeStyle({ shape: ShapeNodeShape.ELLIPSE })
* )
* graph.nodeDefaults.ports.locationParameter = FreeNodePortLocationModel.NODE_CENTER_ANCHORED
* </pre>
*
* @see IPortDefaults#shareStyleInstance
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortDefaults%23style Online Documentation}
*/
abstract style: IPortStyle;
/**
* Gets or sets the defaults for labels at ports.
* <pre>
* graph.nodeDefaults.ports.labels.layoutParameter = new FreePortLabelModel().createDefaultParameter()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortDefaults%23labels Online Documentation}
*/
abstract labels: ILabelDefaults;
/**
* Gets or sets whether unused ports should automatically be removed from their owners as soon as no further edge is
* connected to them.
* <br><b>AutoCleanUp: removing ports when an edge is removed</b>
* <pre>
* const node1 = graph.createNode()
* const node2 = graph.createNode()
* const port1_1 = graph.addPort(node1)
* const port1_2 = graph.addPort(node1)
* const port2 = graph.addPort(node2)
* const edge1 = graph.createEdge(port1_1, port2)
* const edge2 = graph.createEdge(port1_2, port2)
*
* // enable auto cleanup
* graph.nodeDefaults.ports.autoCleanUp = true
* graph.remove(edge1)
* // port1_1 is removed with the edge
* console.log(graph.contains(port1_1)) // false
* // port2 is not removed since edge2 still is linked to it
* console.log(graph.contains(port2))
* // disable auto cleanup
* graph.nodeDefaults.ports.autoCleanUp = false
* graph.remove(edge2)
* // no port is removed with the edge now
* console.log(graph.contains(port1_2)) // true
* console.log(graph.contains(port2)) // true
* </pre>
*
* @see IGraph#remove
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortDefaults%23autoCleanUp Online Documentation}
*/
abstract autoCleanUp: boolean;
/**
* Gets or sets a value indicating whether the {@link IPortDefaults#style} instance should be shared referentially or {@link ICloneable#clone cloned} upon a call to
* {@link IPortDefaults#getStyleInstance}.
* <br><b>Shared vs. cloned styles</b>
* <pre>
* const n1 = graph.createNodeAt(new Point(0, 0))
* const n2 = graph.createNodeAt(new Point(0, 50))
* const n3 = graph.createNodeAt(new Point(0, 100))
* const n4 = graph.createNodeAt(new Point(0, 150))
*
* const defaultStyle = new NodeStylePortStyleAdapter({
*   nodeStyle: new ShapeNodeStyle(),
*   renderSize: new Size(5, 5)
* })
* graph.nodeDefaults.ports.style = defaultStyle
*
* // share style instances
* graph.nodeDefaults.ports.shareStyleInstance = true
* const p1 = graph.addPort(n1)
* const p2 = graph.addPort(n2)
* // p1.style === defaultStyle; p2.style === defaultStyle; p1.style === p2.style;
* defaultStyle.renderSize = new Size(10, 10) // p1 and p2 both get larger
*
* // clone style instances
* graph.nodeDefaults.ports.shareStyleInstance = false
* const p3 = graph.addPort(n3)
* const p4 = graph.addPort(n4)
* // p3.style !== defaultStyle; p4.style !== defaultStyle; p3.style !== p4.style;
* defaultStyle.renderSize = new Size(20, 20) // p3 and p4 don't change their size
* </pre>
*
* @see IPortDefaults#getStyleInstance
* @see IPortDefaults#style
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortDefaults%23shareStyleInstance Online Documentation}
*/
abstract shareStyleInstance: boolean;
/**
* Gets or sets the location model parameter to use for ports.
* Depending on the setting of {@link IPortDefaults#shareLocationParameterInstance}, the {@link IPortDefaults#getLocationParameterInstance} method should return a {@link ICloneable#clone clone}
* of this instance or the very same instance.
*
* <pre>
* graph.nodeDefaults.ports.style = new NodeStylePortStyleAdapter(
*   new ShapeNodeStyle({ shape: ShapeNodeShape.ELLIPSE })
* )
* graph.nodeDefaults.ports.locationParameter = FreeNodePortLocationModel.NODE_CENTER_ANCHORED
* </pre>
*
* @see IPortDefaults#shareLocationParameterInstance
* @see IPortDefaults#getLocationParameterInstance
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortDefaults%23locationParameter Online Documentation}
*/
abstract locationParameter: IPortLocationModelParameter;
/**
Gets or sets a value indicating whether the {@link IPortDefaults#locationParameter} instance should be shared referentially or {@link ICloneable#clone cloned} upon a call to
{@link IPortDefaults#getLocationParameterInstance}.
@see IPortDefaults#getLocationParameterInstance
@see IPortDefaults#locationParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortDefaults%23shareLocationParameterInstance Online Documentation}*/
abstract shareLocationParameterInstance: boolean;
static $class: Class;
private $_IPortDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortDefaults;
}
export interface IPortLocationModel extends ILookup {
}
/**
Defines the geometry of {@link IPort} instances with respect to specific
{@link IPortLocationModelParameter}s.
In addition to defining the location of a port with respect to a specific
{@link IPortLocationModelParameter}, a port location model is typically the factory of the model parameters that
are compatible with it, too.
Typically, the port location model defines a compilation of valid locations for ports, and a specific model parameter
specifies an exact geometry for a port.
By design, the same instance of a port location model can be shared by multiple port location model parameters.
Similarly, the same instance of a port location model parameter can be shared by multiple ports.
This interface supports the {@link ILookup#lookup} method that can be used to query additional aspects of the
implementation.
@see IPortLocationModelParameter
@see IPort
@see IGraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortLocationModel Online Documentation}*/
export abstract class IPortLocationModel {
/**
Factory method that creates a parameter for the given port that tries to match the provided location in absolute world
coordinates.
@param owner The port owner that will own the port for which the parameter shall be created.
@param location The location in the world coordinate system that should be matched as best as possible.
@returns A new instance that can be used to describe the location of an {@link IPort} at the given <code>owner</code>.
@throws {Exception} <b>(ArgumentError)</b> <code>location</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortLocationModel%23createParameter(IPortOwner%2CPoint) Online Documentation}*/
abstract createParameter(owner: IPortOwner, location: Point): IPortLocationModelParameter;
/**
Provides a {@link ILookup lookup context} for the given combination of port and parameter.
@param port The port to use in the context.
@param locationParameter The parameter to use for the port in the context.
@returns An implementation of the {@link ILookup} interface that can be used to query additional aspects of the
port/parameter combination.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortLocationModel%23getContext(IPort%2CIPortLocationModelParameter) Online Documentation}*/
abstract getContext(port: IPort, locationParameter: IPortLocationModelParameter): ILookup;
/**
Determines the location of the port for the given parameter.
@param port The port to determine the location for.
@param locationParameter The parameter to use. The parameter can be expected to be created by this instance having the {@link IPortLocationModelParameter#model} property set to this
instance..
@returns The calculated location of the port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortLocationModel%23getLocation(IPort%2CIPortLocationModelParameter) Online Documentation}*/
abstract getLocation(port: IPort, locationParameter: IPortLocationModelParameter): Point;
static $class: Class;
private $_IPortLocationModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortLocationModel;
}
export interface IPortLocationModelParameter extends ICloneable {
}
/**
Defines a state that can be used to {@link IPortLocationModel#getLocation determine the location of a port} with a {@link IPortLocationModel}.
Typically, implementations of this interface are created by factory methods of the corresponding {@link IPortLocationModelParameter#model}.
This interface extends the {@link ICloneable} interface. This allows clients to obtain a persistent copy of the
current state of this parameter. Immutable parameter implementations may return themselves.
@see IPortLocationModel
@see IPort
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortLocationModelParameter Online Documentation}*/
export abstract class IPortLocationModelParameter {
/**
Determines whether this parameter can be used for ports at the given <code>owner</code>.
A parameter must not necessarily support all types of ports. This method returns whether it supports ports at the given
<code>owner</code>.
@param owner The port owner to test.
@returns <code>true</code> if this instance can be used for ports of the given port owner, <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortLocationModelParameter%23supports(IPortOwner) Online Documentation}*/
abstract supports(owner: IPortOwner): boolean;
/**
Gets the model that created this parameter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortLocationModelParameter%23model Online Documentation}*/
abstract readonly model: IPortLocationModel;
static $class: Class;
private $_IPortLocationModelParameter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortLocationModelParameter;
}
export interface IPortSelectionTester {
}
/**
Interface that can be used to find ports at specific locations in the canvas.
This interface is mostly provided through the {@link ILookup#lookup} of {@link IPortOwner} implementations
such as {@link INode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortSelectionTester Online Documentation}*/
export abstract class IPortSelectionTester {
/**
Creates an implementation of the interface {@link IPortSelectionTester} from the given definition.
The given object must contain exactly the abstract members of the {@link IPortSelectionTester} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IPortSelectionTester} interface.
@returns An instance of the {@link IPortSelectionTester} interface based on the given definition.*/
static create(interfaceDefinition: {
getHitPort(context: IInputModeContext, location: Point): IPort;
getPortsInBox(context: IInputModeContext, rectangle: Rect): IEnumerable<IPort>;
}): IPortSelectionTester;
/**
Returns the port at the given world coordinate position or <code>null</code> if there is no such port.
@param context The input mode context to use for querying the position.
@param location The coordinates of the position in the world coordinate system.
@returns The port at the position or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortSelectionTester%23getHitPort(IInputModeContext%2CPoint) Online Documentation}*/
abstract getHitPort(context: IInputModeContext, location: Point): IPort | null;
/**
Returns the ports for the given marquee rectangle.
@param context The input mode context to use for querying the position.
@param rectangle The marquee selection box in the world coordinate system.
@returns An enumerable over the ports inside the marquee selection box.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortSelectionTester%23getPortsInBox(IInputModeContext%2CRect) Online Documentation}*/
abstract getPortsInBox(context: IInputModeContext, rectangle: Rect): IEnumerable<IPort>;
static $class: Class;
private $_IPortSelectionTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortSelectionTester;
}
export interface InsideOutsidePortLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider {
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A label model for labels at node ports which places the label either inside or outside of the owner node of the port,
according to the node border which is closest to the port position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsideOutsidePortLabelModel Online Documentation}*/
export class InsideOutsidePortLabelModel {
/**
Creates a new instance of this model with default values.
@param [options] The parameters to pass.
@param [options.distance] The distance between the owner {@link IPort} and the closest border of the {@link ILabel} This option sets the {@link InsideOutsidePortLabelModel#distance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsideOutsidePortLabelModel Online Documentation}*/
constructor(options?: {
distance?: number;
});
/**
Creates a parameter which places the label outside of the {@link IPort}'s owner node.
@returns A parameter which places the label outside of the {@link IPort}'s owner node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsideOutsidePortLabelModel%23createDefaultParameter() Online Documentation}*/
createDefaultParameter(): ILabelModelParameter;
/**
Creates a parameter which places the label inside of the {@link IPort}'s owner node.
@returns A parameter which places the label inside of the {@link IPort}'s owner node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsideOutsidePortLabelModel%23createInsideParameter() Online Documentation}*/
createInsideParameter(): ILabelModelParameter;
/**
Creates a parameter which places the label outside of the {@link IPort}'s owner node.
@returns A parameter which places the label outside of the {@link IPort}'s owner node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsideOutsidePortLabelModel%23createOutsideParameter() Online Documentation}*/
createOutsideParameter(): ILabelModelParameter;
/**
Gets or sets the distance between the owner {@link IPort} and the closest border of the {@link ILabel}
The default value is 5.
@default 5.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsideOutsidePortLabelModel%23distance Online Documentation}*/
distance: number;
static $class: Class;
private $_InsideOutsidePortLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is InsideOutsidePortLabelModel;
}
/**
Enumeration of the possible positions of the {@link InteriorStretchLabelModel}.
This field can be used as an argument for the {@link InteriorStretchLabelModel#createParameter} factory method.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModelPosition Online Documentation}*/
export enum InteriorStretchLabelModelPosition {
/**
Encodes a position at the top side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModelPosition%23NORTH Online Documentation}*/
NORTH,
/**
Encodes a position at the right side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModelPosition%23EAST Online Documentation}*/
EAST,
/**
Encodes a position at the bottom side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModelPosition%23SOUTH Online Documentation}*/
SOUTH,
/**
Encodes a position at the left side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModelPosition%23WEST Online Documentation}*/
WEST,
/**
Encodes a position at the center of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModelPosition%23CENTER Online Documentation}*/
CENTER,
/**
Encodes a position at the vertical center with the preferred height of the label and the width of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModelPosition%23CENTER_HORIZONTAL Online Documentation}*/
CENTER_HORIZONTAL,
/**
Encodes a position at the horizontal center with the preferred width of the label and the height of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModelPosition%23CENTER_VERTICAL Online Documentation}*/
CENTER_VERTICAL
}
/** Automatic conversion to {@link InteriorStretchLabelModelPosition}. */
type InteriorStretchLabelModelPositionStringValues = "north" | "east" | "south" | "west" | "center" | "center-horizontal" | "center-vertical";
export interface InteriorStretchLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A label model for node labels that lie inside of the node's {@link INode#layout} bounds and have been stretched to fit the node's size.
This label model supports 5 positions inside of the layout's bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel Online Documentation}*/
export class InteriorStretchLabelModel {
/**
Creates a new instance of this model with empty insets.
@param [options] The parameters to pass.
@param [options.insets] The insets to use within the node's {@link INode#layout}. This option sets the {@link InteriorStretchLabelModel#insets} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel Online Documentation}*/
constructor(options?: {
insets?: Insets | InsetsConvertible;
});
/**
Creates the parameter for the given position.
@param position The position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23createParameter(InteriorStretchLabelModelPosition) Online Documentation}*/
createParameter(position: InteriorStretchLabelModelPosition | InteriorStretchLabelModelPositionStringValues): ILabelModelParameter;
/**
Returns the minimum size this model would require for the node owner of the label if the the given label was used with
the provided parameter.
@param label The label to use.
@param layoutParameter The parameter to use.
@returns The minimum size requirements for this label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23getMinimumNodeSize(ILabel%2CILabelModelParameter) Online Documentation}*/
getMinimumNodeSize(label: ILabel, layoutParameter: ILabelModelParameter): Size;
/**
Returns the insets this model would require for the node owner of the label if the the given label was used with the
provided parameter.
@param label The label to use.
@param layoutParameter The parameter to use.
@returns The insets rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23getNodeInsets(ILabel%2CILabelModelParameter) Online Documentation}*/
getNodeInsets(label: ILabel, layoutParameter: ILabelModelParameter): Insets;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instance's {@link InteriorStretchLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23NORTH Online Documentation}*/
static NORTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instance's {@link InteriorStretchLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23EAST Online Documentation}*/
static EAST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instance's {@link InteriorStretchLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23SOUTH Online Documentation}*/
static SOUTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instance's {@link InteriorStretchLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23WEST Online Documentation}*/
static WEST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instance's {@link InteriorStretchLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23CENTER Online Documentation}*/
static CENTER: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Although the {@link ILabelModelParameter#model} can be cast to a {@link InteriorStretchLabelModel}, trying to change that instance's {@link InteriorStretchLabelModel#insets} will raise an
{@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23CENTER_HORIZONTAL Online Documentation}*/
static CENTER_HORIZONTAL: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Although the {@link ILabelModelParameter#model} can be cast to a {@link InteriorStretchLabelModel}, trying to change that instance's {@link InteriorStretchLabelModel#insets} will raise an
{@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23CENTER_VERTICAL Online Documentation}*/
static CENTER_VERTICAL: ILabelModelParameter;
/**
Gets or sets the insets to use within the node's {@link INode#layout}.
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorStretchLabelModel%23insets Online Documentation}*/
insets: Insets;
static $class: Class;
private $_InteriorStretchLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is InteriorStretchLabelModel;
}
/**
Enumeration of the possible positions of the {@link InteriorLabelModel}.
This field can be used as an argument for the {@link InteriorLabelModel#createParameter} factory method.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition Online Documentation}*/
export enum InteriorLabelModelPosition {
/**
Encodes a position at the top side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition%23NORTH Online Documentation}*/
NORTH,
/**
Encodes a position at the right side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition%23EAST Online Documentation}*/
EAST,
/**
Encodes a position at the bottom side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition%23SOUTH Online Documentation}*/
SOUTH,
/**
Encodes a position at the left side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition%23WEST Online Documentation}*/
WEST,
/**
Encodes a position at the upper right corner of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition%23NORTH_EAST Online Documentation}*/
NORTH_EAST,
/**
Encodes a position at the lower right corner of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition%23SOUTH_EAST Online Documentation}*/
SOUTH_EAST,
/**
Encodes a position at the upper left corner of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition%23NORTH_WEST Online Documentation}*/
NORTH_WEST,
/**
Encodes a position at the lower left corner of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition%23SOUTH_WEST Online Documentation}*/
SOUTH_WEST,
/**
Encodes a position at the center of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModelPosition%23CENTER Online Documentation}*/
CENTER
}
/** Automatic conversion to {@link InteriorLabelModelPosition}. */
type InteriorLabelModelPositionStringValues = "north" | "east" | "south" | "west" | "north-east" | "south-east" | "north-west" | "south-west" | "center";
export interface InteriorLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A label model for node labels that lie inside of the node's {@link INode#layout} bounds.
This label model supports nine positions inside of the layout's bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel Online Documentation}*/
export class InteriorLabelModel {
/**
Creates a new instance of this model with empty insets.
@param [options] The parameters to pass.
@param [options.insets] The insets to use within the node's {@link INode#layout}. This option sets the {@link InteriorLabelModel#insets} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel Online Documentation}*/
constructor(options?: {
insets?: Insets | InsetsConvertible;
});
/**
Creates the parameter for the given position.
@param position The position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23createParameter(InteriorLabelModelPosition) Online Documentation}*/
createParameter(position: InteriorLabelModelPosition | InteriorLabelModelPositionStringValues): ILabelModelParameter;
/**
Returns the minimum size this model would require for the node owner of the label if the the given label was used with
the provided parameter.
@param label The label to use.
@param layoutParameter The parameter to use.
@returns The minimum size requirements for this label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23getMinimumNodeSize(ILabel%2CILabelModelParameter) Online Documentation}*/
getMinimumNodeSize(label: ILabel, layoutParameter: ILabelModelParameter): Size;
/**
Returns the insets this model would require for the node owner of the label if the the given label was used with the
provided parameter.
@param label The label to use.
@param layoutParameter The parameter to use.
@returns The insets rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23getNodeInsets(ILabel%2CILabelModelParameter) Online Documentation}*/
getNodeInsets(label: ILabel, layoutParameter: ILabelModelParameter): Insets;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link InteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23NORTH Online Documentation}*/
static NORTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link InteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23EAST Online Documentation}*/
static EAST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link InteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23SOUTH Online Documentation}*/
static SOUTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link InteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23SOUTH_EAST Online Documentation}*/
static SOUTH_EAST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link InteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23SOUTH_WEST Online Documentation}*/
static SOUTH_WEST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link InteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23WEST Online Documentation}*/
static WEST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link InteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23NORTH_EAST Online Documentation}*/
static NORTH_EAST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link InteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23NORTH_WEST Online Documentation}*/
static NORTH_WEST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link InteriorLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23CENTER Online Documentation}*/
static CENTER: ILabelModelParameter;
/**
Gets or sets the insets to use within the node's {@link INode#layout}.
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteriorLabelModel%23insets Online Documentation}*/
insets: Insets;
static $class: Class;
private $_InteriorLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is InteriorLabelModel;
}
/**
Policy to determines at which visual layer node, edge and port labels shall be inserted by the
{@link GraphModelManager}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayerPolicy Online Documentation}*/
export enum LabelLayerPolicy {
/**
The labels will be visualized directly on top of their owner.
If the owner changes its layer or z-order the labels are changing with it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayerPolicy%23AT_OWNER Online Documentation}*/
AT_OWNER,
/**
All labels will be visualized in one of three separate label layers.
Node labels are placed in {@link GraphModelManager#nodeLabelGroup}, edge labels in {@link GraphModelManager#edgeLabelGroup} and port labels in {@link GraphModelManager#portLabelGroup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayerPolicy%23SEPARATE_LAYER Online Documentation}*/
SEPARATE_LAYER
}
/** Automatic conversion to {@link LabelLayerPolicy}. */
type LabelLayerPolicyStringValues = "at-owner" | "separate-layer";
export interface MapperMetadata extends YObject {
}
/**
The meta data that is associated with {@link IMapper}s in a {@link IMapperRegistry}.
@see IMapperRegistry#getMapperMetadata
@see IMapperRegistry#setMapperMetadata
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperMetadata Online Documentation}*/
export class MapperMetadata {
/**
Initializes a new instance of the {@link MapperMetadata} class.
@param keyType Type of the keys.
@param valueType Type of the values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperMetadata%23MapperMetadata(Class%2CClass) Online Documentation}*/
constructor(keyType: Class, valueType: Class);
/**
Initializes a new instance of the {@link MapperMetadata} from the given instance class.
@param metadata The metadata to copy key and value type from.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperMetadata%23MapperMetadata(MapperMetadata) Online Documentation}*/
constructor(metadata: MapperMetadata);
/**
Creates an instance for the specified mapper.
Type parameter <code>TKey</code>: The type of the keys.
Type parameter <code>TValue</code>: The type of the values.
@param valueType The type of the values.
@param keyType The type of the keys.
@param mapper The mapper instance to create the metadata for.
@returns An instance that uses the {@link MapperMetadata#keyType} and {@link MapperMetadata#valueType} of the mapper.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperMetadata%23create(Class%2CClass%2CIMapper%3CTKey%2CTValue%3E) Online Documentation}*/
static create<TKey, TValue>(keyType: Class, valueType: Class, mapper: IMapper<TKey, TValue> | IMapperConvertible<TKey, TValue>): MapperMetadata;
/**
The type of element the associated mapper accepts as keys for the mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperMetadata%23keyType Online Documentation}*/
readonly keyType: Class;
/**
The type of element the associated mapper accepts as Values for the mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperMetadata%23valueType Online Documentation}*/
readonly valueType: Class;
static $class: Class;
private $_MapperMetadata: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MapperMetadata;
}
export interface MapperRegistry extends YObject, IMapperRegistry {
/**
@inheritdoc*/
addMapper<K, V>(keyType: Class, valueType: Class, tag: any, mapper: IMapper<K, V> | IMapperConvertible<K, V>): void;
/**
@inheritdoc*/
getMapper<K, V>(tag: any): IMapper<K, V> | null;
/**
@inheritdoc*/
getMapperMetadata(tag: any): MapperMetadata | null;
/**
@inheritdoc*/
removeMapper(tag: any): void;
/**
@inheritdoc*/
setMapperMetadata(tag: any, metadata: MapperMetadata): void;
/**
@inheritdoc*/
readonly registeredTags: IEnumerable<any>;
} /**
A simple {@link HashMap}-based implementation of the {@link IMapperRegistry} interface.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperRegistry Online Documentation}*/
export class MapperRegistry {
constructor();
static $class: Class;
private $_MapperRegistry: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MapperRegistry;
}
/**
Determines the position of a label in the {@link NinePositionsEdgeLabelModel}
This enumeration can be used for the factory method {@link NinePositionsEdgeLabelModel#createParameter}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition Online Documentation}*/
export enum NinePositionsEdgeLabelModelPosition {
/**
Encodes a label position at the source above the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition%23SOURCE_ABOVE Online Documentation}*/
SOURCE_ABOVE,
/**
Encodes a label position at the center above the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition%23CENTER_ABOVE Online Documentation}*/
CENTER_ABOVE,
/**
Encodes a label position at the target above the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition%23TARGET_ABOVE Online Documentation}*/
TARGET_ABOVE,
/**
Encodes a label position at the source of the edge that lies on the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition%23SOURCE_CENTERED Online Documentation}*/
SOURCE_CENTERED,
/**
Encodes a label position at the center of the edge that lies on the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition%23CENTER_CENTERED Online Documentation}*/
CENTER_CENTERED,
/**
Encodes a label position at the source of the edge that lies on the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition%23TARGET_CENTERED Online Documentation}*/
TARGET_CENTERED,
/**
Encodes a label position at the source below the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition%23SOURCE_BELOW Online Documentation}*/
SOURCE_BELOW,
/**
Encodes a label position at the center below the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition%23CENTER_BELOW Online Documentation}*/
CENTER_BELOW,
/**
Encodes a label position at the target below the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModelPosition%23TARGET_BELOW Online Documentation}*/
TARGET_BELOW
}
/** Automatic conversion to {@link NinePositionsEdgeLabelModelPosition}. */
type NinePositionsEdgeLabelModelPositionStringValues = "source-above" | "center-above" | "target-above" | "source-centered" | "center-centered" | "target-centered" | "source-below" | "center-below" | "target-below";
export interface NinePositionsEdgeLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
An edge label model that knows nine different label positions.
The possible positions are near the source port, at the center of the edge, or near the target port, each above, below,
or centered on the edge. The distance to the edge and the absolute angular orientation of the label can be specified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel Online Documentation}*/
export class NinePositionsEdgeLabelModel {
/**
Initializes a new instance of the {@link NinePositionsEdgeLabelModel} class.
@param [distance=10] The distance.
@param [angle=0] The angle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel Online Documentation}*/
constructor(distance?: number, angle?: number);
/**
Initializes a new instance of the {@link NinePositionsEdgeLabelModel} class.
@param options The parameters to pass.
@param [options.distance=10] The distance.
@param [options.angle=0] The angle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel Online Documentation}*/
constructor(options: {
distance?: number;
angle?: number;
});
/**
Create a model parameter for the specified <code>position</code>
@param position The position for this parameter
@returns a model parameter for the specified <code>position</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23createParameter(NinePositionsEdgeLabelModelPosition) Online Documentation}*/
createParameter(position: NinePositionsEdgeLabelModelPosition | NinePositionsEdgeLabelModelPositionStringValues): ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link NinePositionsEdgeLabelModel#angle} or {@link NinePositionsEdgeLabelModel#distance} properties will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23SOURCE_ABOVE Online Documentation}*/
static SOURCE_ABOVE: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link NinePositionsEdgeLabelModel#angle} or {@link NinePositionsEdgeLabelModel#distance} properties will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23CENTER_ABOVE Online Documentation}*/
static CENTER_ABOVE: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link NinePositionsEdgeLabelModel#angle} or {@link NinePositionsEdgeLabelModel#distance} properties will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23TARGET_ABOVE Online Documentation}*/
static TARGET_ABOVE: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link NinePositionsEdgeLabelModel#angle} or {@link NinePositionsEdgeLabelModel#distance} properties will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23SOURCE_CENTERED Online Documentation}*/
static SOURCE_CENTERED: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link NinePositionsEdgeLabelModel#angle} or {@link NinePositionsEdgeLabelModel#distance} properties will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23CENTER_CENTERED Online Documentation}*/
static CENTER_CENTERED: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link NinePositionsEdgeLabelModel#angle} or {@link NinePositionsEdgeLabelModel#distance} properties will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23TARGET_CENTERED Online Documentation}*/
static TARGET_CENTERED: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link NinePositionsEdgeLabelModel#angle} or {@link NinePositionsEdgeLabelModel#distance} properties will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23SOURCE_BELOW Online Documentation}*/
static SOURCE_BELOW: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link NinePositionsEdgeLabelModel#angle} or {@link NinePositionsEdgeLabelModel#distance} properties will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23CENTER_BELOW Online Documentation}*/
static CENTER_BELOW: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link NinePositionsEdgeLabelModel#angle} or {@link NinePositionsEdgeLabelModel#distance} properties will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23TARGET_BELOW Online Documentation}*/
static TARGET_BELOW: ILabelModelParameter;
/**
Gets or sets the angle for the rotation of the labels.
A value of <code>0</code> makes the label appear horizontally aligned.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23angle Online Documentation}*/
angle: number;
/**
Gets or sets the distance of the labels above or below the edge.
@default 10.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/NinePositionsEdgeLabelModel%23distance Online Documentation}*/
distance: number;
static $class: Class;
private $_NinePositionsEdgeLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NinePositionsEdgeLabelModel;
}
/**
Policy to determines at which visual layer node and edge ports shall be inserted by the {@link GraphModelManager}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLayerPolicy Online Documentation}*/
export enum PortLayerPolicy {
/**
The ports will be visualized directly on top of their owner.
If the owner changes its layer or z-order the ports are changing with it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLayerPolicy%23AT_OWNER Online Documentation}*/
AT_OWNER,
/**
All ports will be visualized in the {@link GraphModelManager#portGroup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLayerPolicy%23SEPARATE_LAYER Online Documentation}*/
SEPARATE_LAYER
}
/** Automatic conversion to {@link PortLayerPolicy}. */
type PortLayerPolicyStringValues = "at-owner" | "separate-layer";
export interface SandwichLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A node label model that knows only two possible positions.
The label will be placed centered on top or below the node's {@link INode#layout} using a specifiable {@link SandwichLabelModel#yOffset}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SandwichLabelModel Online Documentation}*/
export class SandwichLabelModel {
/**
Initializes a new instance of the {@link SandwichLabelModel} class with zero offset.
@param [options] The parameters to pass.
@param [options.yOffset] The distance between the node's {@link INode#layout} and the label's {@link ILabel#layout layout}. This option sets the {@link SandwichLabelModel#yOffset} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SandwichLabelModel Online Documentation}*/
constructor(options?: {
yOffset?: number;
});
/**
Creates a parameter for the north side of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SandwichLabelModel%23createNorthParameter() Online Documentation}*/
createNorthParameter(): ILabelModelParameter;
/**
Creates a parameter for the south side of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SandwichLabelModel%23createSouthParameter() Online Documentation}*/
createSouthParameter(): ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link SandwichLabelModel#yOffset} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SandwichLabelModel%23NORTH Online Documentation}*/
static NORTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link SandwichLabelModel#yOffset} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SandwichLabelModel%23SOUTH Online Documentation}*/
static SOUTH: ILabelModelParameter;
/**
Gets or sets the distance between the node's {@link INode#layout} and the label's {@link ILabel#layout layout}.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/SandwichLabelModel%23yOffset Online Documentation}*/
yOffset: number;
static $class: Class;
private $_SandwichLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SandwichLabelModel;
}
export interface SegmentRatioPortLocationModel extends YObject, IPortLocationModel {
/**
@inheritdoc*/
createParameter(owner: IPortOwner, location: Point): IPortLocationModelParameter;
/**
@inheritdoc*/
getContext(port: IPort, locationParameter: IPortLocationModelParameter): ILookup;
/**
@inheritdoc*/
getLocation(port: IPort, locationParameter: IPortLocationModelParameter): Point;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A port location model that places the port on a certain edge segment at a specified ratio.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentRatioPortLocationModel Online Documentation}*/
export class SegmentRatioPortLocationModel {
/**
Creates the a new parameter for the specified segment index and ratio.
@param ratio The ratio for the segment, with <code>0</code> being at the start of the segment and <code>1</code> at the end.
@param segmentIndex The index of the segment.
@returns A parameter for the given ratio and segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentRatioPortLocationModel%23createFromSource(number%2Cnumber) Online Documentation}*/
createFromSource(ratio: number, segmentIndex: number): IPortLocationModelParameter;
/**
Creates the a new parameter for the specified segment index counted from the target end of the edge and the provided
ratio.
@param ratio The ratio for the segment, with <code>0</code> being at the start of the segment and <code>1</code> at the end.
@param segmentIndex The index of the segment counted from the target end.
@returns A parameter for the given ratio and segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentRatioPortLocationModel%23createFromTarget(number%2Cnumber) Online Documentation}*/
createFromTarget(ratio: number, segmentIndex: number): IPortLocationModelParameter;
/**
A static immutable global instance of this class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentRatioPortLocationModel%23INSTANCE Online Documentation}*/
static INSTANCE: SegmentRatioPortLocationModel;
static $class: Class;
private $_SegmentRatioPortLocationModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SegmentRatioPortLocationModel;
}
export interface SmartEdgeLabelModel extends YObject, ILabelModel, ILabelModelParameterFinder {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
findBestParameter(label: ILabel, model: ILabelModel, layout: IOrientedRectangle): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
An edge label model that allows the placement at any position.
Similar to
{@link EdgeSegmentLabelModel}, a position is specified by an edge segment and a ratio on that segment, amongst
others. During changes of the edge path, the label keeps its relative location with respect to the bends of the
reference segment in a similar way.
In contrast to {@link EdgeSegmentLabelModel}, the distance from the edge path is not a property of the model but of
the particular parameter. In addition, this class does not implement {@link ILabelModelParameterProvider} and
therefore, labels with this model can be moved freely and are not restricted to a fixed set of candidates at a given
distance from the edge.
If {@link SmartEdgeLabelModel#autoRotation} is enabled, labels are automatically rotated according to the angle of the corresponding reference edge segment.
During movements, labels with this model snap to notable positions if the {@link MoveLabelInputMode} provides a {@link LabelSnapContext}
that is enabled and has a suitable configuration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmartEdgeLabelModel Online Documentation}*/
export class SmartEdgeLabelModel {
/**
Initializes a new instance of this class.
@param [options] The parameters to pass.
@param [options.angle] The rotation angle of all labels with this model. This option sets the {@link SmartEdgeLabelModel#angle} property on the created object.
@param [options.autoRotation] Whether or not edge labels are automatically rotated according to the angle of the corresponding reference edge segment. This option sets the {@link SmartEdgeLabelModel#autoRotation} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmartEdgeLabelModel Online Documentation}*/
constructor(options?: {
angle?: number;
autoRotation?: boolean;
});
/**
Creates a parameter that measures the provided segment index from the source side of the edge path.
@param segmentIndex The zero-based index of the segment beginning from the source side.
@param [distance=0] The distance between the label's box and the edge's path. The default is 0.
@param [segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the source side of the
segment, a ratio of <code>1.0</code> at the target side. Ratios lesser than <code>0.0</code> or greater than <code>1.0</code> will be interpreted as absolute
values in world coordinates. The default is 0.5.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmartEdgeLabelModel%23createParameterFromSource(number%2Cnumber%2Cnumber) Online Documentation}*/
createParameterFromSource(segmentIndex: number, distance?: number, segmentRatio?: number): ILabelModelParameter;
/**
Creates a parameter that measures the provided segment index from the source side of the edge path.
@param options The parameters to pass.
@param options.segmentIndex The zero-based index of the segment beginning from the source side.
@param [options.distance=0] The distance between the label's box and the edge's path. The default is 0.
@param [options.segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the source side of the
segment, a ratio of <code>1.0</code> at the target side. Ratios lesser than <code>0.0</code> or greater than <code>1.0</code> will be interpreted as absolute
values in world coordinates. The default is 0.5.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmartEdgeLabelModel%23createParameterFromSource(number%2Cnumber%2Cnumber) Online Documentation}*/
createParameterFromSource(options: {
segmentIndex: number;
distance?: number;
segmentRatio?: number;
}): ILabelModelParameter;
/**
Creates a parameter that measures the provided segment index from the target side of the edge path.
@param segmentIndex The zero-based index of the segment beginning from the target side.
@param [distance=0] The distance between the label's box and the edge's path. The default is 0.
@param [segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the target side of the
segment, a ratio of <code>1.0</code> at the source side. Ratios lesser than <code>0.0</code> or greater than <code>1.0</code> will be interpreted as absolute
values in world coordinates. The default is 0.5.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmartEdgeLabelModel%23createParameterFromTarget(number%2Cnumber%2Cnumber) Online Documentation}*/
createParameterFromTarget(segmentIndex: number, distance?: number, segmentRatio?: number): ILabelModelParameter;
/**
Creates a parameter that measures the provided segment index from the target side of the edge path.
@param options The parameters to pass.
@param options.segmentIndex The zero-based index of the segment beginning from the target side.
@param [options.distance=0] The distance between the label's box and the edge's path. The default is 0.
@param [options.segmentRatio=0.5] The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the target side of the
segment, a ratio of <code>1.0</code> at the source side. Ratios lesser than <code>0.0</code> or greater than <code>1.0</code> will be interpreted as absolute
values in world coordinates. The default is 0.5.
@returns A label parameter that describes the provided parameters for this model instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmartEdgeLabelModel%23createParameterFromTarget(number%2Cnumber%2Cnumber) Online Documentation}*/
createParameterFromTarget(options: {
segmentIndex: number;
distance?: number;
segmentRatio?: number;
}): ILabelModelParameter;
/**
Gets the distance from the edge path described by the specified parameter.
@param layoutParameter The parameter to get the edge path distance for.
@returns The distance from the edge path
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmartEdgeLabelModel%23getDistance(ILabelModelParameter) Online Documentation}*/
getDistance(layoutParameter: ILabelModelParameter): number;
/**
Gets or sets whether or not edge labels are automatically rotated according to the angle of the corresponding reference
edge segment.
By default, this feature is enabled.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmartEdgeLabelModel%23autoRotation Online Documentation}*/
autoRotation: boolean;
/**
Gets or sets the rotation angle of all labels with this model.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/SmartEdgeLabelModel%23angle Online Documentation}*/
angle: number;
static $class: Class;
private $_SmartEdgeLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SmartEdgeLabelModel;
}
/**
Flaggable enumeration that is used by {@link SmartEdgeLabelModel} to indicate the position.
This enumeration is mainly used by the XAML serialization mechanisms.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderParameterLocation Online Documentation}*/
export enum SliderParameterLocation {
/**
Left of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderParameterLocation%23LEFT Online Documentation}*/
LEFT,
/**
Right of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderParameterLocation%23RIGHT Online Documentation}*/
RIGHT,
/**
Measured from the source end.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderParameterLocation%23FROM_SOURCE Online Documentation}*/
FROM_SOURCE,
/**
Measured from the target end.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderParameterLocation%23FROM_TARGET Online Documentation}*/
FROM_TARGET,
/**
"Center" segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderParameterLocation%23CENTER Online Documentation}*/
CENTER
}
type SliderParameterLocationStringValues = "left" | "right" | "from-source" | "from-target" | "center";
/** Automatic conversion to {@link SliderParameterLocation}. */
type SliderParameterLocationConvertible = SliderParameterLocationStringValues | SliderParameterLocationStringValues[];
/**
An enumeration for use in various properties that can take a bitwise combination of stripe types
The enumeration can be used to create a bitwise combination of the types.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeTypes Online Documentation}*/
export enum StripeTypes {
/**
Constant for rows.
@see StripeTypes#ROW
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeTypes%23ROW Online Documentation}*/
ROW,
/**
Constant for columns.
@see StripeTypes#COLUMN
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeTypes%23COLUMN Online Documentation}*/
COLUMN,
/**
Constant for all item types.
This is a bitwise combination of all other types declared in this enum.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeTypes%23ALL Online Documentation}*/
ALL,
/**
Constant for no item type.
This is a constant where no type bit has been set, i.e. it is <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeTypes%23NONE Online Documentation}*/
NONE
}
type StripeTypesStringValues = "row" | "column" | "all" | "none";
/** Automatic conversion to {@link StripeTypes}. */
type StripeTypesConvertible = StripeTypesStringValues | StripeTypesStringValues[];
export interface PortDefaults extends YObject, IPortDefaults {
/**
@inheritdoc*/
getLocationParameterInstance(owner: IPortOwner): IPortLocationModelParameter;
/**
@inheritdoc*/
getStyleInstance(owner: IPortOwner): IPortStyle;
/**
@inheritdoc*/
labels: ILabelDefaults;
/**
@inheritdoc*/
locationParameter: IPortLocationModelParameter;
/**
@inheritdoc*/
style: IPortStyle;
} /**
A canonical implementation of the {@link IPortDefaults} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDefaults Online Documentation}*/
export class PortDefaults {
/**
Initializes a new instance of the {@link PortDefaults} class using a trivial location model parameter.
@param [options] The parameters to pass.
@param [options.autoCleanUp] Sets the autoCleanUp property.
@param [options.locationParameter] Sets the locationParameter property.
@param [options.shareLocationParameterInstance] Sets the shareLocationParameterInstance property.
@param [options.style] Sets the style property.
@param [options.shareStyleInstance] Sets the shareStyleInstance property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDefaults Online Documentation}*/
constructor(options?: {
autoCleanUp?: boolean;
locationParameter?: IPortLocationModelParameter;
shareLocationParameterInstance?: boolean;
style?: IPortStyle;
shareStyleInstance?: boolean;
});
/**
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDefaults%23autoCleanUp Online Documentation}*/
autoCleanUp: boolean;
/**
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDefaults%23shareLocationParameterInstance Online Documentation}*/
shareLocationParameterInstance: boolean;
/**
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDefaults%23shareStyleInstance Online Documentation}*/
shareStyleInstance: boolean;
static $class: Class;
private $_PortDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortDefaults;
}
export interface NodeDefaults extends YObject, INodeDefaults {
/**
@inheritdoc*/
getStyleInstance(): INodeStyle;
/**
@inheritdoc*/
size: Size;
/**
@inheritdoc*/
labels: ILabelDefaults;
/**
@inheritdoc*/
ports: IPortDefaults;
/**
@inheritdoc*/
style: INodeStyle;
} /**
A canonical implementation of the {@link INodeDefaults} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDefaults Online Documentation}*/
export class NodeDefaults {
/**
Initializes a new instance of the {@link NodeDefaults} class using default properties.
@param [options] The parameters to pass.
@param [options.size] Sets the size property.
@param [options.labels] Sets the labels property.
@param [options.ports] Sets the ports property.
@param [options.style] Sets the style property.
@param [options.shareStyleInstance] Sets the shareStyleInstance property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDefaults Online Documentation}*/
constructor(options?: {
size?: Size | SizeConvertible;
labels?: ILabelDefaults;
ports?: IPortDefaults;
style?: INodeStyle;
shareStyleInstance?: boolean;
});
/**
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDefaults%23shareStyleInstance Online Documentation}*/
shareStyleInstance: boolean;
static $class: Class;
private $_NodeDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeDefaults;
}
export interface EdgeDefaults extends YObject, IEdgeDefaults {
/**
@inheritdoc*/
getStyleInstance(): IEdgeStyle;
/**
@inheritdoc*/
labels: ILabelDefaults;
/**
@inheritdoc*/
ports: IPortDefaults;
/**
@inheritdoc*/
style: IEdgeStyle;
} /**
A canonical implementation of the {@link IEdgeDefaults} interface
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDefaults Online Documentation}*/
export class EdgeDefaults {
/**
Initializes a new instance of the {@link EdgeDefaults} class with default edge style and label and port defaults.
@param [options] The parameters to pass.
@param [options.labels] Sets the labels property.
@param [options.ports] Sets the ports property.
@param [options.style] Sets the style property.
@param [options.shareStyleInstance] Sets the shareStyleInstance property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDefaults Online Documentation}*/
constructor(options?: {
labels?: ILabelDefaults;
ports?: IPortDefaults;
style?: IEdgeStyle;
shareStyleInstance?: boolean;
});
/**
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDefaults%23shareStyleInstance Online Documentation}*/
shareStyleInstance: boolean;
static $class: Class;
private $_EdgeDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeDefaults;
}
export interface LabelDefaults extends YObject, ILabelDefaults {
/**
@inheritdoc*/
getLayoutParameterInstance(owner: ILabelOwner): ILabelModelParameter;
/**
@inheritdoc*/
getStyleInstance(owner: ILabelOwner): ILabelStyle;
/**
@inheritdoc*/
autoAdjustPreferredSize: boolean;
/**
@inheritdoc*/
shareLayoutParameterInstance: boolean;
/**
@inheritdoc*/
layoutParameter: ILabelModelParameter;
/**
@inheritdoc*/
style: ILabelStyle;
/**
@inheritdoc*/
shareStyleInstance: boolean;
} /**
A canonical implementation of the {@link ILabelDefaults} interface
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDefaults Online Documentation}*/
export class LabelDefaults {
/**
Initializes a new instance of the {@link LabelDefaults} class using a trivial parameter.
@param [options] The parameters to pass.
@param [options.autoAdjustPreferredSize] Sets the autoAdjustPreferredSize property.
@param [options.shareLayoutParameterInstance] Sets the shareLayoutParameterInstance property.
@param [options.layoutParameter] Sets the layoutParameter property.
@param [options.shareStyleInstance] Sets the shareStyleInstance property.
@param [options.style] Sets the style property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDefaults Online Documentation}*/
constructor(options?: {
autoAdjustPreferredSize?: boolean;
shareLayoutParameterInstance?: boolean;
layoutParameter?: ILabelModelParameter;
shareStyleInstance?: boolean;
style?: ILabelStyle;
});
static $class: Class;
private $_LabelDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelDefaults;
}
/**
Defines common default arrow types for {@link Arrow}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArrowType Online Documentation}*/
export enum ArrowType {
/**
Default arrow
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArrowType%23DEFAULT Online Documentation}*/
DEFAULT,
/**
Simple arrow
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArrowType%23SIMPLE Online Documentation}*/
SIMPLE,
/**
Short arrow
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArrowType%23SHORT Online Documentation}*/
SHORT,
/**
Diamond shaped arrow
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArrowType%23DIAMOND Online Documentation}*/
DIAMOND,
/**
No arrow
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArrowType%23NONE Online Documentation}*/
NONE,
/**
Circular shaped arrow
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArrowType%23CIRCLE Online Documentation}*/
CIRCLE,
/**
Cross shaped arrow
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArrowType%23CROSS Online Documentation}*/
CROSS,
/**
Triangular shaped arrow
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArrowType%23TRIANGLE Online Documentation}*/
TRIANGLE
}
/** Automatic conversion to {@link ArrowType}. */
type ArrowTypeStringValues = "default" | "simple" | "short" | "diamond" | "none" | "circle" | "cross" | "triangle";
export interface EdgeDecorationInstaller extends YObject, ISelectionIndicatorInstaller, IHighlightIndicatorInstaller, IFocusIndicatorInstaller {
}
/**
An implementation of the {@link ICanvasObjectInstaller} interface that can decorate the visual appearance of an
{@link IEdge}.
This will install an {@link ICanvasObject} during {@link ICanvasObjectInstaller#addCanvasObject installation}, that will render the path and the bends of an edge
using a {@link EdgeDecorationInstaller#getStroke pen} for the path and a {@link EdgeDecorationInstaller#getBendDrawing drawing} for the bends.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorationInstaller Online Documentation}*/
export abstract class EdgeDecorationInstaller {
/**
Installs a rendering for the <code>item</code> if it is an {@link IEdge} that will use the {@link EdgeDecorationInstaller#getBendDrawing}
and {@link EdgeDecorationInstaller#getStroke} to render the path and bends.
@param context The canvas context.
@param group The {@link ICanvasObjectGroup} to add the new canvas object to.
@param item The item to install.
@returns The canvas object representing the item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorationInstaller%23addCanvasObject(ICanvasContext%2CICanvasObjectGroup%2CObject) Online Documentation}*/
addCanvasObject(context: ICanvasContext, group: ICanvasObjectGroup, item: any | null): ICanvasObject | null;
/**
Retrieves the drawing of the bend for the context.
This implementation will use the {@link EdgeSelectionIndicatorInstaller#BEND_TEMPLATE_KEY} resource to find a {@link IVisualTemplate}
instance. If <code>null</code> is yielded, a default drawing will be used.
@param canvas The canvas for which the drawing shall be returned.
@param edge The edge for which the rendering will be used.
@returns The drawing instance to use for the rendering of the bends in the given context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorationInstaller%23getBendDrawing(CanvasComponent%2CIEdge) Online Documentation}*/
abstract getBendDrawing(canvas: CanvasComponent, edge: IEdge): IVisualTemplate | null;
/**
Retrieves the {@link Stroke} for the context.
This implementation will use the {@link EdgeSelectionIndicatorInstaller#STROKE_KEY} resource to find a {@link Stroke}
instance. If <code>null</code> is yielded, a default {@link Stroke} will be used.
@param edge The edge for which the rendering will be used.
@param canvas The canvas for which the {@link Stroke} shall be returned.
@returns The {@link Stroke} instance to use for the rendering of the edge in the given context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorationInstaller%23getStroke(CanvasComponent%2CIEdge) Online Documentation}*/
abstract getStroke(canvas: CanvasComponent, edge: IEdge): Stroke | null;
/**
Gets or sets a value indicating whether to draw the decoration in in view coordinates, as opposed to intermediate
coordinates.
This is useful with {@link CanvasComponent#projection}s that have smaller scaling factors, which may result in the edge decoration not being visible in
some directions. This setting has no visible effect, unless a {@link CanvasComponent#projection} is set. Setting a new value has no effect on currently
visible edge decorations.
The default is <code>true</code> to draw the edge decoration in view coordinates.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDecorationInstaller%23useViewCoordinates Online Documentation}*/
useViewCoordinates: boolean;
static $class: Class;
private $_EdgeDecorationInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeDecorationInstaller;
}
export interface EdgeSelectionIndicatorInstaller extends EdgeDecorationInstaller {
}
/**
An implementation of the {@link ICanvasObjectInstaller} interface that can install the selection outline of an
{@link IEdge}.
This will {@link EdgeDecorationInstaller#addCanvasObject install} an {@link ICanvasObject} that will render the path and the bends of an edge using a {@link EdgeSelectionIndicatorInstaller#getStroke pen} for the
path and a {@link EdgeSelectionIndicatorInstaller#getBendDrawing drawing} for the bends.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSelectionIndicatorInstaller Online Documentation}*/
export class EdgeSelectionIndicatorInstaller {
constructor();
/**
Retrieves the drawing of the bend for the context.
This implementation will use the {@link EdgeSelectionIndicatorInstaller#BEND_TEMPLATE_KEY} resource to find a {@link IVisualTemplate}
instance. If <code>null</code> is yielded, a default drawing will be used.
@param canvas The canvas for which the drawing shall be returned.
@param edge The edge for which the rendering will be used.
@returns The drawing instance to use for the rendering of the bends in the given context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSelectionIndicatorInstaller%23getBendDrawing(CanvasComponent%2CIEdge) Online Documentation}*/
getBendDrawing(canvas: CanvasComponent, edge: IEdge): IVisualTemplate | null;
/**
Retrieves the {@link Stroke} for the context.
This implementation will use the {@link EdgeSelectionIndicatorInstaller#STROKE_KEY} resource to find a {@link Stroke}
instance. If <code>null</code> is yielded, a default {@link Stroke} will be used.
@param edge The edge for which the rendering will be used.
@param canvas The canvas for which the {@link Stroke} shall be returned.
@returns The {@link Stroke} instance to use for the rendering of the edge in the given context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSelectionIndicatorInstaller%23getStroke(CanvasComponent%2CIEdge) Online Documentation}*/
getStroke(canvas: CanvasComponent, edge: IEdge): Stroke | null;
/**
A {@link string} that will be used to find the {@link Stroke} that will be used to draw the path of the edge.
The default selection template provides the <code>yfiles-selection-template</code> and <code>yfiles-edge-selection-template</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSelectionIndicatorInstaller%23STROKE_KEY Online Documentation}*/
static STROKE_KEY: string;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} that will be used to draw the bends of the edge.
The default template has the <code>yfiles-bend-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSelectionIndicatorInstaller%23BEND_TEMPLATE_KEY Online Documentation}*/
static BEND_TEMPLATE_KEY: string;
static $class: Class;
private $_EdgeSelectionIndicatorInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeSelectionIndicatorInstaller;
}
export interface EdgeFocusIndicatorInstaller extends EdgeDecorationInstaller {
}
/**
An specialized subclass of the {@link EdgeSelectionIndicatorInstaller} that can install the focus indicator of an
{@link IEdge}.
This will {@link EdgeDecorationInstaller#addCanvasObject install} an {@link ICanvasObject} that will render the path and the bends of an edge using a {@link EdgeFocusIndicatorInstaller#getStroke pen} for the
path and a {@link EdgeFocusIndicatorInstaller#getBendDrawing drawing} for the bends.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFocusIndicatorInstaller Online Documentation}*/
export class EdgeFocusIndicatorInstaller {
constructor();
/**
Retrieves the drawing of the bend for the context.
This implementation will use the {@link EdgeFocusIndicatorInstaller#BEND_TEMPLATE_KEY} resource to find a {@link IVisualTemplate}
instance. If <code>null</code> is yielded, a default drawing will be used.
@param canvas The canvas for which the drawing shall be returned.
@param edge The edge for which the rendering will be used.
@returns The drawing instance to use for the rendering of the bends in the given context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFocusIndicatorInstaller%23getBendDrawing(CanvasComponent%2CIEdge) Online Documentation}*/
getBendDrawing(canvas: CanvasComponent, edge: IEdge): IVisualTemplate | null;
/**
Retrieves the {@link Stroke} for the context.
This implementation will use the {@link EdgeFocusIndicatorInstaller#STROKE_KEY} resource to find a {@link Stroke}
instance. If <code>null</code> is yielded, a default {@link Stroke} will be used.
@param edge The edge for which the rendering will be used.
@param canvas The canvas for which the {@link Stroke} shall be returned.
@returns The {@link Stroke} instance to use for the rendering of the edge in the given context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFocusIndicatorInstaller%23getStroke(CanvasComponent%2CIEdge) Online Documentation}*/
getStroke(canvas: CanvasComponent, edge: IEdge): Stroke | null;
/**
A {@link string} that will be used to find the {@link Stroke} that will be used to draw the path of the edge.
The default focus template provides the <code>yfiles-focus-template</code> and <code>yfiles-edge-focus-template</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFocusIndicatorInstaller%23STROKE_KEY Online Documentation}*/
static STROKE_KEY: string;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} that will be used to draw the bends of the edge.
There is no default visualization for bends of the focus indicator.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFocusIndicatorInstaller%23BEND_TEMPLATE_KEY Online Documentation}*/
static BEND_TEMPLATE_KEY: string;
static $class: Class;
private $_EdgeFocusIndicatorInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeFocusIndicatorInstaller;
}
export interface EdgeHighlightIndicatorInstaller extends EdgeDecorationInstaller {
}
/**
An specialized subclass of the {@link EdgeSelectionIndicatorInstaller} that can install the highlight rendering of
an {@link IEdge}.
This will {@link EdgeDecorationInstaller#addCanvasObject install} an {@link ICanvasObject} that will render the path and the bends of an edge using a {@link EdgeHighlightIndicatorInstaller#getStroke pen} for the
path and a {@link EdgeHighlightIndicatorInstaller#getBendDrawing drawing} for the bends.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeHighlightIndicatorInstaller Online Documentation}*/
export class EdgeHighlightIndicatorInstaller {
constructor();
/**
Retrieves the drawing of the bend for the context.
This implementation will use the {@link EdgeHighlightIndicatorInstaller#BEND_TEMPLATE_KEY} resource to find a {@link IVisualTemplate}
instance. If <code>null</code> is yielded, a default drawing will be used.
@param canvas The context for which the drawing shall be returned.
@param edge The edge for which the rendering will be used.
@returns The drawing instance to use for the rendering of the bends in the given context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeHighlightIndicatorInstaller%23getBendDrawing(CanvasComponent%2CIEdge) Online Documentation}*/
getBendDrawing(canvas: CanvasComponent, edge: IEdge): IVisualTemplate | null;
/**
Retrieves the {@link Stroke} for the context.
This implementation will use the {@link EdgeHighlightIndicatorInstaller#STROKE_KEY} resource to find a {@link Stroke}
instance. If <code>null</code> is yielded, a default {@link Stroke} will be used.
@param edge The edge for which the rendering will be used.
@param canvas The canvas for which the {@link Stroke} shall be returned.
@returns The {@link Stroke} instance to use for the rendering of the edge in the given context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeHighlightIndicatorInstaller%23getStroke(CanvasComponent%2CIEdge) Online Documentation}*/
getStroke(canvas: CanvasComponent, edge: IEdge): Stroke | null;
/**
A {@link string} that will be used to find the {@link Stroke} that will be used to draw the path of the edge.
The default highlight template provides the <code>yfiles-highlight-template</code> and <code>yfiles-edge-highlight-template</code> CSS classes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeHighlightIndicatorInstaller%23STROKE_KEY Online Documentation}*/
static STROKE_KEY: string;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} that will be used to draw the bends of the edge.
There is no default visualization for bends of the highlight indicator.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeHighlightIndicatorInstaller%23BEND_TEMPLATE_KEY Online Documentation}*/
static BEND_TEMPLATE_KEY: string;
static $class: Class;
private $_EdgeHighlightIndicatorInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeHighlightIndicatorInstaller;
}
export interface EdgeStyleDecorationInstaller extends YObject, ISelectionIndicatorInstaller, IHighlightIndicatorInstaller, IFocusIndicatorInstaller {
}
/**
Allows the use of an {@link IEdgeStyle} to render the selection, highlight or focus indicator of edges.
The renderer of the {@link EdgeStyleDecorationInstaller#edgeStyle} used by this class must create {@link SvgVisual}s.
The {@link EdgeStyleDecorationInstaller#zoomPolicy} property defines how the zoom level affects the rendering of the indicator. It can either scale according to the
zoom level similar to regular graph items or have always to same thickness regardless of the zoom, similar to the
default yFiles indicators.
@see NodeStyleDecorationInstaller
@see LabelStyleDecorationInstaller
@see PortStyleDecorationInstaller
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleDecorationInstaller Online Documentation}*/
export class EdgeStyleDecorationInstaller {
/**
Create a new instance of this class.
@param [options] The parameters to pass.
@param [options.edgeStyle] The style to use for the rendering. This option sets the {@link EdgeStyleDecorationInstaller#edgeStyle} property on the created object.
@param [options.zoomPolicy] How the style is affected by the current zoom level. This option sets the {@link EdgeStyleDecorationInstaller#zoomPolicy} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleDecorationInstaller Online Documentation}*/
constructor(options?: {
edgeStyle?: IEdgeStyle;
zoomPolicy?: StyleDecorationZoomPolicy | StyleDecorationZoomPolicyStringValues;
});
/**
Installs a rendering for the <code>item</code> if it is an {@link IEdge}.
@param context The canvas context.
@param group The canvas object group to add the newly created {@link ICanvasObject} to.
@param item The item to install.
@returns The canvas object representing the item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleDecorationInstaller%23addCanvasObject(ICanvasContext%2CICanvasObjectGroup%2CObject) Online Documentation}*/
addCanvasObject(context: ICanvasContext, group: ICanvasObjectGroup, item: any | null): ICanvasObject | null;
/**
Gets or sets how the style is affected by the current zoom level.
Changes of the value of this property are propagated to all styles created by this instance and become immediately
visible.
The default value is {@link StyleDecorationZoomPolicy#VIEW_COORDINATES}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleDecorationInstaller%23zoomPolicy Online Documentation}*/
zoomPolicy: StyleDecorationZoomPolicy;
/**
Gets or sets the style to use for the rendering.
Changes of the value of this property are <b>not</b> propagated to already created styles.
The default value is an instance of {@link PolylineEdgeStyle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleDecorationInstaller%23edgeStyle Online Documentation}*/
edgeStyle: IEdgeStyle;
static $class: Class;
private $_EdgeStyleDecorationInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeStyleDecorationInstaller;
}
export interface GraphOverviewCanvasVisualCreator extends YObject, IVisualCreator {
/**
@inheritdoc*/
createVisual(ctx: IRenderContext): Visual | null;
/**
@inheritdoc*/
updateVisual(ctx: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
An {@link IVisualCreator} for use in a {@link CanvasComponent} that renders a {@link IGraph} instance in a
sloppy overview style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewCanvasVisualCreator Online Documentation}*/
export class GraphOverviewCanvasVisualCreator {
/**
Creates a new instance for the given graph.
@param graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewCanvasVisualCreator Online Documentation}*/
constructor(graph: IGraph);
/**
Callback that paints the specified <code>edge</code>.
@param renderContext
@param ctx
@param edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewCanvasVisualCreator%23paintEdge(IRenderContext%2CCanvasRenderingContext2D%2CIEdge) Online Documentation}*/
paintEdge(renderContext: IRenderContext, ctx: CanvasRenderingContext2D, edge: IEdge): void;
/**
Callback method that paints the specified <code>node</code>.
@param renderContext
@param ctx
@param node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewCanvasVisualCreator%23paintGroupNode(IRenderContext%2CCanvasRenderingContext2D%2CINode) Online Documentation}*/
paintGroupNode(renderContext: IRenderContext, ctx: CanvasRenderingContext2D, node: INode): void;
/**
Callback method that paints the specified <code>node</code>.
@param renderContext
@param ctx
@param node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewCanvasVisualCreator%23paintNode(IRenderContext%2CCanvasRenderingContext2D%2CINode) Online Documentation}*/
paintNode(renderContext: IRenderContext, ctx: CanvasRenderingContext2D, node: INode): void;
static $class: Class;
private $_GraphOverviewCanvasVisualCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphOverviewCanvasVisualCreator;
}
export interface GraphOverviewWebGLVisualCreator extends YObject, IVisualCreator {
/**
@inheritdoc*/
createVisual(ctx: IRenderContext): Visual | null;
/**
@inheritdoc*/
updateVisual(ctx: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
An {@link IVisualCreator} for use in a {@link CanvasComponent} that renders an {@link IGraph} instance in a
sloppy overview style using WebGL.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewWebGLVisualCreator Online Documentation}*/
export class GraphOverviewWebGLVisualCreator {
/**
Creates a new instance for the given graph.
@param graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewWebGLVisualCreator Online Documentation}*/
constructor(graph: IGraph);
static $class: Class;
private $_GraphOverviewWebGLVisualCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphOverviewWebGLVisualCreator;
}
export interface IEdgePathCropper {
}
/**
Interface used by the framework mainly for calculating the visible path of an edge.
This interface is queried through the {@link ILookup#lookup} of {@link IPort} instances to calculate the
visible portion of an edge path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgePathCropper Online Documentation}*/
export abstract class IEdgePathCropper {
/**
Creates an implementation of the interface {@link IEdgePathCropper} by using the given function as implementation
for its {@link IEdgePathCropper#cropEdgePath} method.
@param cropEdgePath A function for {@link IEdgePathCropper}'s single abstract method {@link IEdgePathCropper#cropEdgePath}.
@returns An instance of the {@link IEdgePathCropper} interface based on the given function.*/
static create(cropEdgePath: (edge: IEdge, atSource: boolean, arrow: IArrow | null, path: GeneralPath) => GeneralPath): IEdgePathCropper;
/**
Crops the provided <code>path</code> at one end of an edge.
@param edge The edge whose path is to be cropped.
@param atSource Whether to crop the source or target side of the path.
@param arrow The arrow that is used at the end of the edge.
@param path The path to crop.
@returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgePathCropper%23cropEdgePath(IEdge%2Cboolean%2CIArrow%2CGeneralPath) Online Documentation}*/
abstract cropEdgePath(edge: IEdge, atSource: boolean, arrow: IArrow | null, path: GeneralPath): GeneralPath;
/**
This singleton implements the same cropping behavior as {@link DefaultEdgePathCropper} and is immutable.
It crops an edge's path at the node bounds of the source or target side with respect to the given arrow.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgePathCropper%23INSTANCE Online Documentation}*/
static INSTANCE: IEdgePathCropper;
static $class: Class;
private $_IEdgePathCropper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgePathCropper;
}
export interface IGroupBoundsCalculator {
}
/**
Interface for implementations that can calculate the bounds of a group node in a grouped graph.
Implementations of this interface will be queried from {@link INode}'s {@link ILookup#lookup} method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGroupBoundsCalculator Online Documentation}*/
export abstract class IGroupBoundsCalculator {
/**
Calculates the bounds of the {@link INode#layout} for the given <code>groupNode</code> that is a group node in the <code>graph</code>.
@param graph The graph which holds the <code>groupNode</code>.
@param groupNode The node to calculate the bounds for.
@returns The minimum bounds to use for the given group node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGroupBoundsCalculator%23calculateBounds(IGraph%2CINode) Online Documentation}*/
abstract calculateBounds(graph: IGraph, groupNode: INode): Rect;
/**
Creates an implementation of the interface {@link IGroupBoundsCalculator} by using the given function as
implementation for its {@link IGroupBoundsCalculator#calculateBounds} method.
@param calculateBounds A function for {@link IGroupBoundsCalculator}'s single abstract method
{@link IGroupBoundsCalculator#calculateBounds}.
@returns An instance of the {@link IGroupBoundsCalculator} interface based on the given function.*/
static create(calculateBounds: (graph: IGraph, groupNode: INode) => Rect): IGroupBoundsCalculator;
static $class: Class;
private $_IGroupBoundsCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGroupBoundsCalculator;
}
export interface VoidLabelStyle extends YObject, ILabelStyle {
}
/**
A void implementation of a label style that does nothing and behaves like an invisible style.
This class implements the singleton pattern. Use the shared static {@link VoidLabelStyle#INSTANCE} of this class
instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link ILabel#style} property of an
{@link ILabel}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyle Online Documentation}*/
export class VoidLabelStyle {
/**
Returns <code>this</code>.
@returns <code>this</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyle%23clone() Online Documentation}*/
clone(): this;
/**
The singleton instance of this style.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyle%23INSTANCE Online Documentation}*/
static INSTANCE: VoidLabelStyle;
/**
Gets the {@link VoidLabelStyleRenderer#INSTANCE VoidLabelStyleRenderer instance}.
@see ILabelStyle#renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyle%23renderer Online Documentation}*/
readonly renderer: ILabelStyleRenderer;
static $class: Class;
private $_VoidLabelStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidLabelStyle;
}
export interface VoidLabelStyleRenderer extends YObject, ILabelStyleRenderer {
}
/**
A void implementation of a label style renderer that does nothing.
This class implements the singleton pattern. Use the shared static {@link VoidLabelStyleRenderer#INSTANCE} of this
class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link ILabelStyle#renderer} property of an
{@link ILabelStyle}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyleRenderer Online Documentation}*/
export class VoidLabelStyleRenderer {
/**
Yields the {@link IBoundsProvider#EMPTY} that will return empty bounds.
@param label
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyleRenderer%23getBoundsProvider(ILabel%2CILabelStyle) Online Documentation}*/
getBoundsProvider(label: ILabel, style: ILabelStyle): IBoundsProvider;
/**
Yields a lookup that will only return {@link LassoTestables#NEVER} if an {@link ILassoTestable} is queried.
@param label
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyleRenderer%23getContext(ILabel%2CILabelStyle) Online Documentation}*/
getContext(label: ILabel, style: ILabelStyle): ILookup;
/**
Yields the {@link IHitTestable#NEVER} that will always report misses.
@param label
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyleRenderer%23getHitTestable(ILabel%2CILabelStyle) Online Documentation}*/
getHitTestable(label: ILabel, style: ILabelStyle): IHitTestable;
/**
Yields the {@link IMarqueeTestable#NEVER} that will always report misses.
@param label
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyleRenderer%23getMarqueeTestable(ILabel%2CILabelStyle) Online Documentation}*/
getMarqueeTestable(label: ILabel, style: ILabelStyle): IMarqueeTestable;
/**
Yields {@link Size#EMPTY}.
@param label
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyleRenderer%23getPreferredSize(ILabel%2CILabelStyle) Online Documentation}*/
getPreferredSize(label: ILabel, style: ILabelStyle): Size;
/**
Yields the {@link IVisibilityTestable#NEVER} that will always claim invisibility.
@param label
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyleRenderer%23getVisibilityTestable(ILabel%2CILabelStyle) Online Documentation}*/
getVisibilityTestable(label: ILabel, style: ILabelStyle): IVisibilityTestable;
/**
Yields the {@link VoidVisualCreator#INSTANCE} that will do nothing.
@param label
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyleRenderer%23getVisualCreator(ILabel%2CILabelStyle) Online Documentation}*/
getVisualCreator(label: ILabel, style: ILabelStyle): IVisualCreator;
/**
The singleton instance of this renderer.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidLabelStyleRenderer%23INSTANCE Online Documentation}*/
static INSTANCE: ILabelStyleRenderer;
static $class: Class;
private $_VoidLabelStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidLabelStyleRenderer;
}
export interface ILabelStyle extends ICloneable {
}
/**
Style implementation for {@link ILabel} instances in an {@link IGraph}.
Implementations are responsible for rendering {@link ILabel} instances in a {@link CanvasComponent}. It is up
to the implementation to interpret the visual appearance of a label. The framework uses the associated, possibly shared
{@link ILabelStyle#renderer} to perform the actual rendering of this style for a given label. This interface extends the {@link ICloneable}
interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
implementations may return themselves.
Working with styles (and their style renderers) is explained in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/view_visualization-graph-elements Visualization of Graph Elements}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyle Online Documentation}*/
export abstract class ILabelStyle {
/**
* Gets the renderer implementation that can be queried for implementations that provide details about the visual
* appearance and visual behavior for a given label and this style instance.
* The idiom for retrieving, e.g. an {@link IVisualCreator} implementation for a given style is:
*
* <pre>
* const creator = style.renderer.getVisualCreator(label, style)
* const visual = creator.createVisual(context)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyle%23renderer Online Documentation}
*/
abstract readonly renderer: ILabelStyleRenderer;
static $class: Class;
private $_ILabelStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelStyle;
}
export interface ILabelStyleRenderer {
}
/**
Interface used by {@link ILabelStyle} implementations to query the actual {@link IVisualCreator},
{@link IVisibilityTestable}, {@link IBoundsProvider}, {@link IHitTestable}, and {@link IMarqueeTestable}
implementations for a label and an associated style.
Implementations of this interface should be designed to be sharable between multiple style instances and should
therefore not contain style specific state.
@see ILabelStyle
@see ILabel
@see INodeStyleRenderer
@see IEdgeStyleRenderer
@see IPortStyleRenderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyleRenderer Online Documentation}*/
export abstract class ILabelStyleRenderer {
/**
Gets an implementation of the {@link IBoundsProvider} interface that can handle the provided label and its
associated style.
This method may return a flyweight implementation.
@param label The label to provide an instance for
@param style The style to use for the calculating the painting bounds
@returns An implementation that may be used to subsequently query the label's painting bounds. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyleRenderer%23getBoundsProvider(ILabel%2CILabelStyle) Online Documentation}*/
abstract getBoundsProvider(label: ILabel, style: ILabelStyle): IBoundsProvider;
/**
Gets a temporary context instance that can be used to query additional information for the label's style.
Implementations may return {@link ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
@param label The label to provide a context instance for.
@param style The style to use for the context.
@returns An non-<code>null</code> lookup implementation.
@see ILookup#EMPTY
@see ILookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyleRenderer%23getContext(ILabel%2CILabelStyle) Online Documentation}*/
abstract getContext(label: ILabel, style: ILabelStyle): ILookup;
/**
Gets an implementation of the {@link IHitTestable} interface that can handle the provided label and its associated
style.
This method may return a flyweight implementation.
@param label The label to provide an instance for
@param style The style to use for the querying hit tests
@returns An implementation that may be used to subsequently perform hit tests. Clients should not cache this instance and must
always call this method immediately before using the value returned. This enables the use of the flyweight design
pattern for implementations. This method may return <code>null</code> to indicate that the label cannot be hit tested.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyleRenderer%23getHitTestable(ILabel%2CILabelStyle) Online Documentation}*/
abstract getHitTestable(label: ILabel, style: ILabelStyle): IHitTestable;
/**
Gets an implementation of the {@link IMarqueeTestable} interface that can handle the provided label and its
associated style.
This method may return a flyweight implementation.
@param label The label to provide an instance for
@param style The style to use for the querying marquee intersection test.
@returns An implementation that may be used to subsequently query the marquee intersections. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyleRenderer%23getMarqueeTestable(ILabel%2CILabelStyle) Online Documentation}*/
abstract getMarqueeTestable(label: ILabel, style: ILabelStyle): IMarqueeTestable;
/**
Calculates the {@link ILabel#preferredSize preferred size} of a given label using the associated style.
@param label The label to determine the preferred size for
@param style The style instance that uses this instance as its {@link ILabelStyle#renderer}
@returns A size that can be used as the {@link ILabel#preferredSize} if this renderer renders the label using the associated style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyleRenderer%23getPreferredSize(ILabel%2CILabelStyle) Online Documentation}*/
abstract getPreferredSize(label: ILabel, style: ILabelStyle): Size;
/**
Gets an implementation of the {@link IVisibilityTestable} interface that can handle the provided label and its
associated style.
This method may return a flyweight implementation.
@param label The label to provide an instance for
@param style The style to use for the testing the visibility
@returns An implementation that may be used to subsequently query the label's visibility. Clients should not cache this instance
and must always call this method immediately before using the value returned. This enables the use of the flyweight
design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyleRenderer%23getVisibilityTestable(ILabel%2CILabelStyle) Online Documentation}*/
abstract getVisibilityTestable(label: ILabel, style: ILabelStyle): IVisibilityTestable;
/**
Gets an implementation of the {@link IVisualCreator} interface that can handle the provided label and its associated
style.
This method may return a flyweight implementation, but never <code>null</code>.
@param label The label to provide an instance for
@param style The style to use for the creation of the visual
@returns An implementation that may be used to subsequently create or update the visual for the label. Clients should not cache
this instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link VoidVisualCreator#INSTANCE void} implementation
instead.
@see VoidVisualCreator#INSTANCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelStyleRenderer%23getVisualCreator(ILabel%2CILabelStyle) Online Documentation}*/
abstract getVisualCreator(label: ILabel, style: ILabelStyle): IVisualCreator;
static $class: Class;
private $_ILabelStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelStyleRenderer;
}
export interface INodeInsetsProvider {
}
/**
Interface for implementations that yield insets for a given {@link INode}.
Implementations of this interface will be queried from an {@link INode}'s {@link ILookup#lookup} method. The
default implementation of a graph uses this instance by default to determine the insets of a group node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeInsetsProvider Online Documentation}*/
export abstract class INodeInsetsProvider {
/**
Creates an implementation of the interface {@link INodeInsetsProvider} by using the given function as
implementation for its {@link INodeInsetsProvider#getInsets} method.
@param getInsets A function for {@link INodeInsetsProvider}'s single abstract method {@link INodeInsetsProvider#getInsets}.
@returns An instance of the {@link INodeInsetsProvider} interface based on the given function.*/
static create(getInsets: (node: INode) => Insets): INodeInsetsProvider;
/**
Returns the insets for the given <code>node</code>.
@param node The node to yield insets for.
@returns The insets for the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeInsetsProvider%23getInsets(INode) Online Documentation}*/
abstract getInsets(node: INode): Insets;
static $class: Class;
private $_INodeInsetsProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeInsetsProvider;
}
export interface VoidShapeGeometry extends YObject, IShapeGeometry {
}
/**
A void implementation of the {@link IShapeGeometry} interface that does nothing and behaves like an empty
geometry.
This class implements the singleton pattern. Use the shared static {@link VoidShapeGeometry#INSTANCE} of this
class instead of <code>null</code> were <code>null</code> is not allowed. The {@link VoidNodeStyleRenderer} does so for example.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidShapeGeometry Online Documentation}*/
export class VoidShapeGeometry {
/**
This implementation always returns <code>null</code> (a {@link T} with no value).
@param inner The first point of the line that is inside the shape.
@param outer The second point of the line that is outside the shape.
@returns A {@link T} with no value.
@see IShapeGeometry#getIntersection
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidShapeGeometry%23getIntersection(Point%2CPoint) Online Documentation}*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
This implementation returns an empty path.
This means that the outline never intersects any line or contains any point.
@returns The empty path.
@see IShapeGeometry#getOutline
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidShapeGeometry%23getOutline() Online Documentation}*/
getOutline(): GeneralPath | null;
/**
This implementation always returns <code>false</code>.
@param location The point to test.
@returns <code>false</code>
@see IShapeGeometry#isInside
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidShapeGeometry%23isInside(Point) Online Documentation}*/
isInside(location: Point): boolean;
/**
The {@link VoidShapeGeometry} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidShapeGeometry%23INSTANCE Online Documentation}*/
static INSTANCE: IShapeGeometry;
static $class: Class;
private $_VoidShapeGeometry: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidShapeGeometry;
}
export interface VoidNodeStyle extends YObject, INodeStyle {
}
/**
A void implementation of a node style that does nothing and behaves like an invisible style.
This class implements the singleton pattern. Use the shared static {@link VoidNodeStyle#INSTANCE} of this class
instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link INode#style} property of an
{@link INode}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyle Online Documentation}*/
export class VoidNodeStyle {
/**
Returns <code>this</code>.
@returns <code>this</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyle%23clone() Online Documentation}*/
clone(): this;
/**
The {@link VoidNodeStyle} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyle%23INSTANCE Online Documentation}*/
static INSTANCE: VoidNodeStyle;
/**
Gets the {@link VoidNodeStyleRenderer#INSTANCE VoidNodeStyleRenderer instance}.
@see INodeStyle#renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyle%23renderer Online Documentation}*/
readonly renderer: INodeStyleRenderer;
static $class: Class;
private $_VoidNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidNodeStyle;
}
export interface VoidNodeStyleRenderer extends YObject, INodeStyleRenderer {
}
/**
A void implementation of a node style renderer that does nothing.
This class implements the singleton pattern. Use the shared static {@link VoidNodeStyleRenderer#INSTANCE} of this
class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link INodeStyle#renderer} property of an
{@link INodeStyle}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyleRenderer Online Documentation}*/
export class VoidNodeStyleRenderer {
/**
Yields the {@link IBoundsProvider#EMPTY} that will return empty bounds.
@param node
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyleRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Yields a lookup that will only return {@link LassoTestables#NEVER} if an {@link ILassoTestable} is queried.
@param node
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyleRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
Yields the {@link IHitTestable#NEVER} that will always report misses.
@param node
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyleRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Yields the {@link IMarqueeTestable#NEVER} that will always report misses.
@param node
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyleRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
This implementation always returns the {@link VoidShapeGeometry#INSTANCE VoidShapeGeometry instance}.
@param node The node to provide an instance for
@param style The style to use for the rendering
@returns The {@link VoidShapeGeometry#INSTANCE VoidShapeGeometry instance}.
@see INodeStyleRenderer#getShapeGeometry
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyleRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Yields the {@link IVisibilityTestable#NEVER} that will always claim invisibility.
@param node
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyleRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Yields the {@link VoidVisualCreator#INSTANCE} that will do nothing.
@param node
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyleRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
The {@link VoidNodeStyleRenderer} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidNodeStyleRenderer%23INSTANCE Online Documentation}*/
static INSTANCE: INodeStyleRenderer;
static $class: Class;
private $_VoidNodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidNodeStyleRenderer;
}
export interface INodeStyle extends ICloneable {
}
/**
Style implementation for {@link INode} instances in an {@link IGraph}.
Implementations are responsible for rendering {@link INode} instances in a {@link CanvasComponent}. It is up to
the implementation to interpret the visual appearance of a node. The framework uses the associated, possibly shared {@link INodeStyle#renderer} to
perform the actual rendering of this style for a given node. This interface extends the {@link ICloneable}
interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
implementations may return themselves.
Working with styles (and their style renderers) is explained in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/view_visualization-graph-elements Visualization of Graph Elements}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyle Online Documentation}*/
export abstract class INodeStyle {
/**
* Gets the renderer implementation that can be queried for implementations that provide details about the visual
* appearance and visual behavior for a given node and this style instance.
* The idiom for retrieving, e.g. an {@link IVisualCreator} implementation for a given style is:
*
* <pre>
* const creator = style.renderer.getVisualCreator(node, style)
* const visual = creator.createVisual(context)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyle%23renderer Online Documentation}
*/
abstract readonly renderer: INodeStyleRenderer;
static $class: Class;
private $_INodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeStyle;
}
export interface INodeStyleRenderer {
}
/**
Interface used by {@link INodeStyle} implementations that can be used to query the actual
{@link IVisualCreator}, {@link IBoundsProvider}, {@link IHitTestable}, and {@link IMarqueeTestable}
implementations for a node and an associated style.
Implementations of this interface should be designed to be sharable between multiple style instances and should
therefore not contain style specific state.
@see INodeStyle
@see INode
@see IEdgeStyleRenderer
@see ILabelStyleRenderer
@see IPortStyleRenderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyleRenderer Online Documentation}*/
export abstract class INodeStyleRenderer {
/**
Gets an implementation of the {@link IBoundsProvider} interface that can handle the provided node and its associated
style.
This method may return a flyweight implementation.
@param node The node to provide an instance for
@param style The style to use for the calculating the painting bounds
@returns An implementation that may be used to subsequently query the node's painting bounds. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyleRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
abstract getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Gets a temporary context instance that can be used to query additional information for the node's style.
Implementations may return {@link ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
@param node The node to provide a context instance for.
@param style The style to use for the context.
@returns An non-<code>null</code> lookup implementation.
@see ILookup#EMPTY
@see ILookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyleRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
abstract getContext(node: INode, style: INodeStyle): ILookup;
/**
Gets an implementation of the {@link IHitTestable} interface that can handle the provided node and its associated
style.
This method may return a flyweight implementation.
@param node The node to provide an instance for
@param style The style to use for the querying hit tests
@returns An implementation that may be used to subsequently perform hit tests. Clients should not cache this instance and must
always call this method immediately before using the value returned. This enables the use of the flyweight design
pattern for implementations. This method may return <code>null</code> to indicate that the node cannot be hit tested.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyleRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
abstract getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Gets an implementation of the {@link IMarqueeTestable} interface that can handle the provided node and its
associated style.
This method may return a flyweight implementation.
@param node The node to provide an instance for
@param style The style to use for the querying marquee intersection test.
@returns An implementation that may be used to subsequently query the marquee intersections. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyleRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
abstract getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
Gets an implementation of the {@link IShapeGeometry} interface that can handle the provided node and its
associated style.
This method may return a flyweight implementation.
@param node The node to provide an instance for
@param style The style to use for the rendering
@returns An implementation that may be used to subsequently query geometry information from. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyleRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
abstract getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Gets an implementation of the {@link IVisibilityTestable} interface that can handle the provided node and its
associated style.
This method may return a flyweight implementation.
@param node The node to provide an instance for
@param style The style to use for the testing the visibility
@returns An implementation that may be used to subsequently query the node's visibility. Clients should not cache this instance
and must always call this method immediately before using the value returned. This enables the use of the flyweight
design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyleRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
abstract getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Gets an implementation of the {@link IVisualCreator} interface that can handle the provided node and its associated
style.
This method may return a flyweight implementation, but never <code>null</code>.
@param node The node to provide an instance for
@param style The style to use for the creation of the visual
@returns An implementation that may be used to subsequently create or update the visual for the node. Clients should not cache
this instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link VoidVisualCreator#INSTANCE void} implementation
instead.
@see VoidVisualCreator#INSTANCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeStyleRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
abstract getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
static $class: Class;
private $_INodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeStyleRenderer;
}
export interface IPathGeometry {
}
/**
Interface that is used mainly by {@link IEdgeStyleRenderer} implementations to reveal geometric details about the
visualization of an edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPathGeometry Online Documentation}*/
export abstract class IPathGeometry {
/**
Creates an implementation of the interface {@link IPathGeometry} from the given definition.
The given object must contain exactly the abstract members of the {@link IPathGeometry} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IPathGeometry} interface.
@returns An instance of the {@link IPathGeometry} interface based on the given definition.*/
static create(interfaceDefinition: {
getTangent(ratio: number): Tangent;
getTangentForSegment(segmentIndex: number, ratio: number): Tangent;
getSegmentCount(): number;
getPath(): GeneralPath;
}): IPathGeometry;
/**
Returns a representation of the visible path of the edge in form of a {@link GeneralPath}
@returns An instance that describes the visible path or <code>null</code> if this is not applicable for the current geometry.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPathGeometry%23getPath() Online Documentation}*/
abstract getPath(): GeneralPath | null;
/**
Returns the number of "segments" this edge's path consists of.
@returns the number of segments or <code>-1</code> if there is no such thing as a segment for this edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPathGeometry%23getSegmentCount() Online Documentation}*/
abstract getSegmentCount(): number;
/**
Calculates the tangent on the edge's path at the given ratio point.
@param ratio a value in [0,1] where 0 is the source's end and 1 is at the target's end of the visible edge path
@returns The tangential line, if any exists.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPathGeometry%23getTangent(number) Online Documentation}*/
abstract getTangent(ratio: number): Tangent | null;
/**
Calculates the tangent on the edge's path at the given ratio point for the given segment.
@param segmentIndex the segment to use for the calculation
@param ratio a value in [0,1] where 0 is the source's end and 1 is at the target's end of the segment
@returns The tangential line, if any exists.
@see IPathGeometry#getTangent
@see IPathGeometry#getSegmentCount
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPathGeometry%23getTangentForSegment(number%2Cnumber) Online Documentation}*/
abstract getTangentForSegment(segmentIndex: number, ratio: number): Tangent | null;
static $class: Class;
private $_IPathGeometry: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPathGeometry;
}
export interface VoidPortStyleRenderer extends YObject, IPortStyleRenderer {
}
/**
A void implementation of a port style renderer that does nothing.
This class implements the singleton pattern. Use the shared static {@link VoidPortStyleRenderer#INSTANCE} of this
class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link IPortStyle#renderer} property of an
{@link IPortStyle}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyleRenderer Online Documentation}*/
export class VoidPortStyleRenderer {
/**
Yields the {@link IBoundsProvider#EMPTY} that will return empty bounds.
@param port
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyleRenderer%23getBoundsProvider(IPort%2CIPortStyle) Online Documentation}*/
getBoundsProvider(port: IPort, style: IPortStyle): IBoundsProvider;
/**
Yields a lookup that will only return {@link LassoTestables#NEVER} if an {@link ILassoTestable} is queried.
@param port
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyleRenderer%23getContext(IPort%2CIPortStyle) Online Documentation}*/
getContext(port: IPort, style: IPortStyle): ILookup;
/**
Yields the {@link IHitTestable#NEVER} that will always report misses.
@param port
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyleRenderer%23getHitTestable(IPort%2CIPortStyle) Online Documentation}*/
getHitTestable(port: IPort, style: IPortStyle): IHitTestable;
/**
Yields the {@link IMarqueeTestable#NEVER} that will always report misses.
@param port
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyleRenderer%23getMarqueeTestable(IPort%2CIPortStyle) Online Documentation}*/
getMarqueeTestable(port: IPort, style: IPortStyle): IMarqueeTestable;
/**
Yields the {@link IVisibilityTestable#NEVER} that will always claim invisibility.
@param port
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyleRenderer%23getVisibilityTestable(IPort%2CIPortStyle) Online Documentation}*/
getVisibilityTestable(port: IPort, style: IPortStyle): IVisibilityTestable;
/**
Yields the {@link VoidVisualCreator#INSTANCE} that will do nothing.
@param port
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyleRenderer%23getVisualCreator(IPort%2CIPortStyle) Online Documentation}*/
getVisualCreator(port: IPort, style: IPortStyle): IVisualCreator;
/**
The {@link VoidPortStyleRenderer} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyleRenderer%23INSTANCE Online Documentation}*/
static INSTANCE: IPortStyleRenderer;
static $class: Class;
private $_VoidPortStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidPortStyleRenderer;
}
export interface VoidPortStyle extends YObject, IPortStyle {
}
/**
A void implementation of a port style that does nothing and behaves like an invisible style.
This class implements the singleton pattern. Use the shared static {@link VoidPortStyle#INSTANCE} of this class
instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link IPort#style} property of an
{@link IPort}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyle Online Documentation}*/
export class VoidPortStyle {
/**
Returns <code>this</code>.
@returns <code>this</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyle%23clone() Online Documentation}*/
clone(): this;
/**
The {@link VoidPortStyle} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyle%23INSTANCE Online Documentation}*/
static INSTANCE: VoidPortStyle;
/**
Gets the {@link VoidPortStyleRenderer#INSTANCE VoidPortStyleRenderer instance}.
@see IPortStyle#renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidPortStyle%23renderer Online Documentation}*/
readonly renderer: IPortStyleRenderer;
static $class: Class;
private $_VoidPortStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidPortStyle;
}
export interface IPortStyle extends ICloneable {
}
/**
Style implementation for {@link IPort} instances in an {@link IGraph}.
Implementations are responsible for rendering {@link IPort} instances in a {@link CanvasComponent}. It is up to
the implementation to interpret the visual appearance of a port. The framework uses the associated, possibly shared {@link IPortStyle#renderer} to
perform the actual rendering of this style for a given port. This interface extends the {@link ICloneable}
interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
implementations may return themselves.
Working with styles (and their style renderers) is explained in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/view_visualization-graph-elements Visualization of Graph Elements}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortStyle Online Documentation}*/
export abstract class IPortStyle {
/**
* Gets the renderer implementation that can be queried for implementations that provide details about the visual
* appearance and visual behavior for a given port and this style instance.
* The idiom for retrieving, e.g. an {@link IVisualCreator} implementation for a given style is:
*
* <pre>
* const creator = style.renderer.getVisualCreator(port, style)
* const visual = creator.createVisual(context)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IPortStyle%23renderer Online Documentation}
*/
abstract readonly renderer: IPortStyleRenderer;
static $class: Class;
private $_IPortStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortStyle;
}
export interface IPortStyleRenderer {
}
/**
Interface used by {@link IPortStyle} implementations that can be used to query the actual
{@link IVisualCreator}, {@link IBoundsProvider}, {@link IHitTestable}, {@link IVisibilityTestable}, and
{@link IMarqueeTestable} implementations for a port and an associated style.
Implementations of this interface should be designed to be sharable between multiple style instances and should
therefore not contain style specific state.
@see IPortStyle
@see IPort
@see INodeStyleRenderer
@see IEdgeStyleRenderer
@see ILabelStyleRenderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortStyleRenderer Online Documentation}*/
export abstract class IPortStyleRenderer {
/**
Gets an implementation of the {@link IBoundsProvider} interface that can handle the provided port and its associated
style.
This method may return a flyweight implementation.
@param port The port to provide an instance for
@param style The style to use for the calculating the painting bounds
@returns An implementation that may be used to subsequently query the port's painting bounds. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortStyleRenderer%23getBoundsProvider(IPort%2CIPortStyle) Online Documentation}*/
abstract getBoundsProvider(port: IPort, style: IPortStyle): IBoundsProvider;
/**
Gets a temporary context instance that can be used to query additional information for the port's style.
Implementations may return {@link ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
@param port The port to provide a context instance for.
@param style The style to use for the context.
@returns An non-<code>null</code> lookup implementation.
@see ILookup#EMPTY
@see ILookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortStyleRenderer%23getContext(IPort%2CIPortStyle) Online Documentation}*/
abstract getContext(port: IPort, style: IPortStyle): ILookup;
/**
Gets an implementation of the {@link IHitTestable} interface that can handle the provided port and its associated
style.
This method may return a flyweight implementation.
@param port The port to provide an instance for
@param style The style to use for the querying hit tests
@returns An implementation that may be used to subsequently perform hit tests. Clients should not cache this instance and must
always call this method immediately before using the value returned. This enables the use of the flyweight design
pattern for implementations. This method may return <code>null</code> to indicate that the port cannot be hit tested.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortStyleRenderer%23getHitTestable(IPort%2CIPortStyle) Online Documentation}*/
abstract getHitTestable(port: IPort, style: IPortStyle): IHitTestable;
/**
Gets an implementation of the {@link IMarqueeTestable} interface that can handle the provided port and its
associated style.
This method may return a flyweight implementation.
@param port The port to provide an instance for
@param style The style to use for the querying marquee intersection test.
@returns An implementation that may be used to subsequently query the marquee intersections. Clients should not cache this
instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortStyleRenderer%23getMarqueeTestable(IPort%2CIPortStyle) Online Documentation}*/
abstract getMarqueeTestable(port: IPort, style: IPortStyle): IMarqueeTestable;
/**
Gets an implementation of the {@link IVisibilityTestable} interface that can handle the provided port and its
associated style.
This method may return a flyweight implementation.
@param port The port to provide an instance for
@param style The style to use for the testing the visibility
@returns An implementation that may be used to subsequently query the port's visibility. Clients should not cache this instance
and must always call this method immediately before using the value returned. This enables the use of the flyweight
design pattern for implementations
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortStyleRenderer%23getVisibilityTestable(IPort%2CIPortStyle) Online Documentation}*/
abstract getVisibilityTestable(port: IPort, style: IPortStyle): IVisibilityTestable;
/**
Gets an implementation of the {@link IVisualCreator} interface that can handle the provided port and its associated
style.
This method may return a flyweight implementation, but never <code>null</code>.
@param port The port to provide an instance for
@param style The style to use for the creation of the visual
@returns An implementation that may be used to subsequently create or update the visual for the port. Clients should not cache
this instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link VoidVisualCreator#INSTANCE void} implementation
instead.
@see VoidVisualCreator#INSTANCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortStyleRenderer%23getVisualCreator(IPort%2CIPortStyle) Online Documentation}*/
abstract getVisualCreator(port: IPort, style: IPortStyle): IVisualCreator;
static $class: Class;
private $_IPortStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortStyleRenderer;
}
export interface IShapeGeometry {
}
/**
Interface that describes the geometry of a shape which has an interior and an exterior.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IShapeGeometry Online Documentation}*/
export abstract class IShapeGeometry {
/**
Creates an implementation of the interface {@link IShapeGeometry} from the given definition.
The given object must contain exactly the abstract members of the {@link IShapeGeometry} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IShapeGeometry} interface.
@returns An instance of the {@link IShapeGeometry} interface based on the given definition.*/
static create(interfaceDefinition: {
getIntersection(inner: Point, outer: Point): Point;
isInside(location: Point): boolean;
getOutline(): GeneralPath;
}): IShapeGeometry;
/**
Returns the intersection for the given line with this shape's geometry.
@param inner The first point of the line that is inside the shape.
@param outer The second point of the line that is outside the shape.
@returns The coordinates of the intersection point, if an intersection was found.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IShapeGeometry%23getIntersection(Point%2CPoint) Online Documentation}*/
abstract getIntersection(inner: Point, outer: Point): Point | null;
/**
Returns the outline of the shape or <code>null</code>.
If <code>null</code> is returned callers may assume the canonical outline as result. For nodes this is the node's {@link INode#layout}.
@returns The outline or <code>null</code> if no outline can be provided.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IShapeGeometry%23getOutline() Online Documentation}*/
abstract getOutline(): GeneralPath | null;
/**
Checks whether the given coordinate is deemed to lie within the shape's geometric bounds.
@param location The point to test.
@returns True if the point lies within the shape.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IShapeGeometry%23isInside(Point) Online Documentation}*/
abstract isInside(location: Point): boolean;
static $class: Class;
private $_IShapeGeometry: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IShapeGeometry;
}
export interface LabelStyleDecorationInstaller extends YObject, ISelectionIndicatorInstaller, IHighlightIndicatorInstaller, IFocusIndicatorInstaller {
}
/**
Allows the use of an {@link ILabelStyle} to render the selection, highlight or focus indicator of labels.
The renderer of the {@link LabelStyleDecorationInstaller#labelStyle} used by this class must create {@link SvgVisual}s.
The {@link LabelStyleDecorationInstaller#zoomPolicy} property defines how the zoom level affects the rendering of the indicator. It can either scale according to the
zoom level similar to regular graph items or have always to same thickness regardless of the zoom, similar to the
default yFiles indicators.
@see EdgeStyleDecorationInstaller
@see NodeStyleDecorationInstaller
@see PortStyleDecorationInstaller
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleDecorationInstaller Online Documentation}*/
export class LabelStyleDecorationInstaller {
/**
Create a new instance of this class.
@param [options] The parameters to pass.
@param [options.labelStyle] The style to use for the rendering. This option sets the {@link LabelStyleDecorationInstaller#labelStyle} property on the created object.
@param [options.margins] The margins around the label layout. This option sets the {@link LabelStyleDecorationInstaller#margins} property on the created object.
@param [options.zoomPolicy] How the style is affected by the current zoom level. This option sets the {@link LabelStyleDecorationInstaller#zoomPolicy} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleDecorationInstaller Online Documentation}*/
constructor(options?: {
labelStyle?: ILabelStyle;
margins?: Insets | InsetsConvertible;
zoomPolicy?: StyleDecorationZoomPolicy | StyleDecorationZoomPolicyStringValues;
});
/**
Installs a rendering for the <code>item</code> if it is an {@link ILabel}.
@param context The canvas context.
@param group The canvas object group to add the newly created canvas object to.
@param item The item to install.
@returns The canvas object representing the item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleDecorationInstaller%23addCanvasObject(ICanvasContext%2CICanvasObjectGroup%2CObject) Online Documentation}*/
addCanvasObject(context: ICanvasContext, group: ICanvasObjectGroup, item: any | null): ICanvasObject | null;
/**
Gets or sets how the style is affected by the current zoom level.
Changes of the value of this property are propagated to all styles created by this instance and become immediately
visible.
The default value is {@link StyleDecorationZoomPolicy#VIEW_COORDINATES}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleDecorationInstaller%23zoomPolicy Online Documentation}*/
zoomPolicy: StyleDecorationZoomPolicy;
/**
Gets or sets the margins around the label layout.
Actually, the bounds rendered by the style is the original label layout enlarged by these margins.
Changes of the value of this property are propagated to all styles created by this instance and become immediately
visible.
The default margins have a value of <code>5</code> for all four sides.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleDecorationInstaller%23margins Online Documentation}*/
margins: Insets;
/**
Gets or sets the style to use for the rendering.
Changes of the value of this property are <b>not</b> propagated to already created styles.
The default value is an instance of {@link DefaultLabelStyle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleDecorationInstaller%23labelStyle Online Documentation}*/
labelStyle: ILabelStyle;
static $class: Class;
private $_LabelStyleDecorationInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelStyleDecorationInstaller;
}
export interface NodeStyleDecorationInstaller extends YObject, ISelectionIndicatorInstaller, IHighlightIndicatorInstaller, IFocusIndicatorInstaller {
}
/**
Allows the use of an {@link INodeStyle} to render the selection, highlight or focus indicator of nodes.
The renderer of the {@link NodeStyleDecorationInstaller#nodeStyle} used by this class must create {@link SvgVisual}s.
The {@link NodeStyleDecorationInstaller#zoomPolicy} property defines how the zoom level affects the rendering of the indicator. It can either scale according to the
zoom level similar to regular graph items or have always to same thickness regardless of the zoom, similar to the
default yFiles indicators.
@see EdgeStyleDecorationInstaller
@see LabelStyleDecorationInstaller
@see PortStyleDecorationInstaller
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleDecorationInstaller Online Documentation}*/
export class NodeStyleDecorationInstaller {
/**
Create a new instance of this class.
@param [options] The parameters to pass.
@param [options.nodeStyle] The style to use for the rendering. This option sets the {@link NodeStyleDecorationInstaller#nodeStyle} property on the created object.
@param [options.margins] The margins around the node layout. This option sets the {@link NodeStyleDecorationInstaller#margins} property on the created object.
@param [options.zoomPolicy] How the style is affected by the current zoom level. This option sets the {@link NodeStyleDecorationInstaller#zoomPolicy} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleDecorationInstaller Online Documentation}*/
constructor(options?: {
nodeStyle?: INodeStyle;
margins?: Insets | InsetsConvertible;
zoomPolicy?: StyleDecorationZoomPolicy | StyleDecorationZoomPolicyStringValues;
});
/**
Installs a rendering for the <code>item</code> if it is an {@link INode}.
@param context The canvas context.
@param group The canvas object group to add the newly created canvas object to.
@param item The item to install.
@returns The canvas object representing the item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleDecorationInstaller%23addCanvasObject(ICanvasContext%2CICanvasObjectGroup%2CObject) Online Documentation}*/
addCanvasObject(context: ICanvasContext, group: ICanvasObjectGroup, item: any | null): ICanvasObject | null;
/**
Gets or sets how the style is affected by the current zoom level.
Changes of the value of this property are propagated to all styles created by this instance and become immediately
visible.
The default value is {@link StyleDecorationZoomPolicy#VIEW_COORDINATES}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleDecorationInstaller%23zoomPolicy Online Documentation}*/
zoomPolicy: StyleDecorationZoomPolicy;
/**
Gets or sets the margins around the node layout.
Actually, the bounds rendered by the style is the original node layout enlarged by these margins.
Changes of the value of this property are propagated to all styles created by this instance and become immediately
visible.
The default margins have a value of <code>5</code> for all four sides.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleDecorationInstaller%23margins Online Documentation}*/
margins: Insets;
/**
Gets or sets the style to use for the rendering.
Changes of the value of this property are <b>not</b> propagated to already created styles.
The default value is an instance of {@link ShapeNodeStyle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleDecorationInstaller%23nodeStyle Online Documentation}*/
nodeStyle: INodeStyle;
static $class: Class;
private $_NodeStyleDecorationInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeStyleDecorationInstaller;
}
export interface GraphOverviewSvgVisualCreator extends YObject, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
An {@link IVisualCreator} for use in a {@link CanvasComponent} that renders a {@link IGraph} instance in a
sloppy overview style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator Online Documentation}*/
export class GraphOverviewSvgVisualCreator {
/**
Creates a new instance for the given graph.
@param graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator Online Documentation}*/
constructor(graph: IGraph);
/**
Returns and/or configures a edge style for the given edge to render.
@param edge The edge to render.
@returns The style or <code>null</code> if no visualization should be drawn.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator%23getEdgeStyle(IEdge) Online Documentation}*/
getEdgeStyle(edge: IEdge): IEdgeStyle | null;
/**
Obtains the {@link IVisualCreator} for the given edge.
@param context The context.
@param edge The edge.
@returns The visual creator that is obtained from the {@link GraphOverviewSvgVisualCreator#getEdgeStyle}'s {@link IEdgeStyleRenderer#getVisualCreator}
method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator%23getEdgeVisualCreator(IRenderContext%2CIEdge) Online Documentation}*/
getEdgeVisualCreator(context: IRenderContext, edge: IEdge): IVisualCreator | null;
/**
Returns and/or configures a group node style for the given node to render.
@param node The group node to render.
@returns The style or <code>null</code> if no visualization should be drawn.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator%23getGroupNodeStyle(INode) Online Documentation}*/
getGroupNodeStyle(node: INode): INodeStyle | null;
/**
Obtains the {@link IVisualCreator} for the given group node.
@param context The context.
@param node The group node.
@returns The visual creator that is obtained from the {@link GraphOverviewSvgVisualCreator#getGroupNodeStyle}'s {@link INodeStyleRenderer#getVisualCreator}
method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator%23getGroupNodeVisualCreator(IRenderContext%2CINode) Online Documentation}*/
getGroupNodeVisualCreator(context: IRenderContext, node: INode): IVisualCreator | null;
/**
Returns and/or configures a node style for the given node to render.
@param node The node to render.
@returns The style or <code>null</code> if no visualization should be drawn.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator%23getNodeStyle(INode) Online Documentation}*/
getNodeStyle(node: INode): INodeStyle | null;
/**
Obtains the {@link IVisualCreator} for the given node.
@param context The context.
@param node The node.
@returns The visual creator that is obtained from the {@link GraphOverviewSvgVisualCreator#getNodeStyle}'s {@link INodeStyleRenderer#getVisualCreator}
method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator%23getNodeVisualCreator(IRenderContext%2CINode) Online Documentation}*/
getNodeVisualCreator(context: IRenderContext, node: INode): IVisualCreator | null;
/**
Gets or sets the overview node style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator%23nodeStyle Online Documentation}*/
nodeStyle: INodeStyle | null;
/**
Gets or sets the overview group node style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator%23groupNodeStyle Online Documentation}*/
groupNodeStyle: INodeStyle | null;
/**
Gets or sets the overview edge style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphOverviewSvgVisualCreator%23edgeStyle Online Documentation}*/
edgeStyle: IEdgeStyle | null;
static $class: Class;
private $_GraphOverviewSvgVisualCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphOverviewSvgVisualCreator;
}
export interface PortStyleDecorationInstaller extends YObject, ISelectionIndicatorInstaller, IHighlightIndicatorInstaller, IFocusIndicatorInstaller {
}
/**
Allows the use of an {@link IPortStyle} to render the selection, highlight or focus indicator of ports.
The renderer of the {@link PortStyleDecorationInstaller#portStyle} used by this class must create {@link SvgVisual}s.
The {@link PortStyleDecorationInstaller#zoomPolicy} property defines how the zoom level affects the rendering of the indicator. It can either scale according to the
zoom level similar to regular graph items or have always to same thickness regardless of the zoom, similar to the
default yFiles indicators.
@see NodeStyleDecorationInstaller
@see EdgeStyleDecorationInstaller
@see LabelStyleDecorationInstaller
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleDecorationInstaller Online Documentation}*/
export class PortStyleDecorationInstaller {
/**
Create a new instance of this class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleDecorationInstaller Online Documentation}*/
constructor();
/**
Installs a rendering for the <code>item</code> if it is an {@link IPort}.
@param context The canvas context.
@param group The canvas object group to add the newly created canvas object to.
@param item The item to install.
@returns The canvas object representing the item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleDecorationInstaller%23addCanvasObject(ICanvasContext%2CICanvasObjectGroup%2CObject) Online Documentation}*/
addCanvasObject(context: ICanvasContext, group: ICanvasObjectGroup, item: any | null): ICanvasObject | null;
/**
Gets or sets how the style is affected by the current zoom level.
Changes of the value of this property are propagated to all styles created by this instance and become immediately
visible.
The default value is {@link StyleDecorationZoomPolicy#VIEW_COORDINATES}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleDecorationInstaller%23zoomPolicy Online Documentation}*/
zoomPolicy: StyleDecorationZoomPolicy;
/**
Gets or sets the style to use for the rendering.
Changes of the value of this property are <b>not</b> propagated to already created styles.
The default value is an instance of {@link NodeStylePortStyleAdapter} with {@link ShapeNodeShape#ELLIPSE ellipse} as shape and a {@link NodeStylePortStyleAdapter#renderSize size} of
<code>(10, 10)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleDecorationInstaller%23portStyle Online Documentation}*/
portStyle: IPortStyle;
static $class: Class;
private $_PortStyleDecorationInstaller: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortStyleDecorationInstaller;
}
/**
Specifies how the zoom level affects the rendering of styles that are used as selection, highlight or focus indicator.
@see NodeStyleDecorationInstaller
@see EdgeStyleDecorationInstaller
@see LabelStyleDecorationInstaller
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StyleDecorationZoomPolicy Online Documentation}*/
export enum StyleDecorationZoomPolicy {
/**
The visual is rendered in the view coordinate space and doesn't scale with the zoom level.
This is similar to the default node resize handles, for example.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StyleDecorationZoomPolicy%23VIEW_COORDINATES Online Documentation}*/
VIEW_COORDINATES,
/**
The visual is rendered in the world coordinate space and scales with the zoom level like a regular graph item
visualization, for example a node style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StyleDecorationZoomPolicy%23WORLD_COORDINATES Online Documentation}*/
WORLD_COORDINATES,
/**
Uses WorldCoordinates rendering for zoom level >= 1 and ViewCoordinates rendering for zoom level &lt; 1.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StyleDecorationZoomPolicy%23MIXED Online Documentation}*/
MIXED
}
/** Automatic conversion to {@link StyleDecorationZoomPolicy}. */
type StyleDecorationZoomPolicyStringValues = "view-coordinates" | "world-coordinates" | "mixed";
export interface Fill extends YObject, ICloneable {
}
/**
* Represents any object which fills an area.
* The fill that is defined by instances of this type can be applied to an SVG element using either the static helper {@link Fill#setFill}
* or the instance method {@link Fill#applyTo}.
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* CSS color strings are converted to {@link SolidColorFill}s. For example:
*
* <pre>
* 'blue'
* '#369'
* '#336699'
* 'rgb(51, 102, 153)'
* 'rgba(51, 102, 153, 0.5)'
* 'hsl(225, 98%, 60%)'
* 'hsla(225, 98%, 60%, 0.5)'
* </pre>
*
* @see Fill#setFill
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Fill Online Documentation}
*/
export abstract class Fill {
/**
Assigns the fill defined by this instance to the given SVG <code>element</code>.
If this {@link Fill} is a gradient and implements the {@link ISvgDefsCreator} interface, it will be added to the
<code>defs</code> section of the SVG document per the
<code>context</code>. The fill of the given element will be set to a reference to the gradient element.
This method is very similar in functionality to its static method counter part
{@link Fill#setFill}. However since this method is an instance method, it cannot be called on empty/<code>null</code> fills. If
you are unsure whether the reference actually points to an instance, use the static helper method instead.
@param element The element whose fill should be set.
@param context The context.
@see Fill#setFill
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23applyTo(SVGElement%2CICanvasContext) Online Documentation}*/
applyTo(element: SVGElement, context: ICanvasContext): void;
/**
Clones this instance by returning a {@link YObject#memberwiseClone} or this in case this instance is already {@link Fill#isFrozen frozen}.
@returns An instance of the same type as this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23clone() Online Documentation}*/
clone(): this;
/**
Freezes this instance.
Freezing an object makes it immutable, which can result in performance enhancements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23freeze() Online Documentation}*/
freeze(): void;
/**
Creates a {@link Fill} instance from the given fill-like object by performing automatic type conversion.
@param fillLike The object to convert to a {@link Fill}.
@returns Either <code>fillLike</code> if it is already a {@link Fill} or a new instance, initialized to the values found in <code>fillLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23from(Fill) Online Documentation}*/
static from(fillLike: Fill | FillConvertible): Fill;
/**
Determines whether this {@link Fill} is equal to the specified object.
@param fill The other {@link Fill} instance to compare.
@returns <code>true</code>, if this {@link SolidColorFill} is equal to the other object; <code>false</code>, otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23hasSameValue(Fill) Online Documentation}*/
abstract hasSameValue(fill: Fill | null | FillConvertible): boolean;
/**
Determines if this instance is frozen.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23isFrozen() Online Documentation}*/
isFrozen(): boolean;
/**
Sets the {@link Fill} on a given SVG element as the fill.
If this {@link Fill} is a gradient and implements the {@link ISvgDefsCreator} interface, it will be added to the
<code>defs</code> section of the SVG document per the
<code>context</code>. The fill of the given element will be set to a reference to the gradient element.
This method is very similar in functionality to its instance method counter part {@link Fill#applyTo}. However being
static, it also allows for the case where the <code>fill</code> is <code>null</code>.
@param fill The fill that should be applied. If <code>null</code>, the element's fill is set to <code>none</code>.
@param element The SVG element whose fill should be set.
@param context The context to use for managing possible <code>defs</code> entries.
@see Fill#applyTo
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23setFill(Fill%2CSVGElement%2CICanvasContext) Online Documentation}*/
static setFill(fill: Fill | null | FillConvertible, element: SVGElement, context: ICanvasContext): void;
/**
Gets a {@link SolidColorFill} for the color AliceBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23ALICE_BLUE Online Documentation}*/
static readonly ALICE_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color AntiqueWhite.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23ANTIQUE_WHITE Online Documentation}*/
static readonly ANTIQUE_WHITE: Fill;
/**
Gets a {@link SolidColorFill} for the color Aqua.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23AQUA Online Documentation}*/
static readonly AQUA: Fill;
/**
Gets a {@link SolidColorFill} for the color Aquamarine.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23AQUAMARINE Online Documentation}*/
static readonly AQUAMARINE: Fill;
/**
Gets a {@link SolidColorFill} for the color Azure.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23AZURE Online Documentation}*/
static readonly AZURE: Fill;
/**
Gets a {@link SolidColorFill} for the color Beige.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23BEIGE Online Documentation}*/
static readonly BEIGE: Fill;
/**
Gets a {@link SolidColorFill} for the color Bisque.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23BISQUE Online Documentation}*/
static readonly BISQUE: Fill;
/**
Gets a {@link SolidColorFill} for the color Black.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23BLACK Online Documentation}*/
static readonly BLACK: Fill;
/**
Gets a {@link SolidColorFill} for the color BlanchedAlmond.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23BLANCHED_ALMOND Online Documentation}*/
static readonly BLANCHED_ALMOND: Fill;
/**
Gets a {@link SolidColorFill} for the color Blue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23BLUE Online Documentation}*/
static readonly BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color BlueViolet.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23BLUE_VIOLET Online Documentation}*/
static readonly BLUE_VIOLET: Fill;
/**
Gets a {@link SolidColorFill} for the color Brown.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23BROWN Online Documentation}*/
static readonly BROWN: Fill;
/**
Gets a {@link SolidColorFill} for the color BurlyWood.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23BURLY_WOOD Online Documentation}*/
static readonly BURLY_WOOD: Fill;
/**
Gets a {@link SolidColorFill} for the color CadetBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23CADET_BLUE Online Documentation}*/
static readonly CADET_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color Chartreuse.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23CHARTREUSE Online Documentation}*/
static readonly CHARTREUSE: Fill;
/**
Gets a {@link SolidColorFill} for the color Chocolate.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23CHOCOLATE Online Documentation}*/
static readonly CHOCOLATE: Fill;
/**
Gets a {@link SolidColorFill} for the color Coral.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23CORAL Online Documentation}*/
static readonly CORAL: Fill;
/**
Gets a {@link SolidColorFill} for the color CornflowerBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23CORNFLOWER_BLUE Online Documentation}*/
static readonly CORNFLOWER_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color Cornsilk.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23CORNSILK Online Documentation}*/
static readonly CORNSILK: Fill;
/**
Gets a {@link SolidColorFill} for the color Crimson.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23CRIMSON Online Documentation}*/
static readonly CRIMSON: Fill;
/**
Gets a {@link SolidColorFill} for the color Cyan.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23CYAN Online Documentation}*/
static readonly CYAN: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_BLUE Online Documentation}*/
static readonly DARK_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkCyan.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_CYAN Online Documentation}*/
static readonly DARK_CYAN: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkGoldenrod.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_GOLDENROD Online Documentation}*/
static readonly DARK_GOLDENROD: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkGray.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_GRAY Online Documentation}*/
static readonly DARK_GRAY: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_GREEN Online Documentation}*/
static readonly DARK_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkKhaki.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_KHAKI Online Documentation}*/
static readonly DARK_KHAKI: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkMagenta.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_MAGENTA Online Documentation}*/
static readonly DARK_MAGENTA: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkOliveGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_OLIVE_GREEN Online Documentation}*/
static readonly DARK_OLIVE_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkOrange.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_ORANGE Online Documentation}*/
static readonly DARK_ORANGE: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkOrchid.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_ORCHID Online Documentation}*/
static readonly DARK_ORCHID: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkRed.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_RED Online Documentation}*/
static readonly DARK_RED: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkSalmon.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_SALMON Online Documentation}*/
static readonly DARK_SALMON: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkSeaGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_SEA_GREEN Online Documentation}*/
static readonly DARK_SEA_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkSlateBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_SLATE_BLUE Online Documentation}*/
static readonly DARK_SLATE_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkSlateGray.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_SLATE_GRAY Online Documentation}*/
static readonly DARK_SLATE_GRAY: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkTurquoise.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_TURQUOISE Online Documentation}*/
static readonly DARK_TURQUOISE: Fill;
/**
Gets a {@link SolidColorFill} for the color DarkViolet.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DARK_VIOLET Online Documentation}*/
static readonly DARK_VIOLET: Fill;
/**
Gets a {@link SolidColorFill} for the color DeepPink.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DEEP_PINK Online Documentation}*/
static readonly DEEP_PINK: Fill;
/**
Gets a {@link SolidColorFill} for the color DeepSkyBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DEEP_SKY_BLUE Online Documentation}*/
static readonly DEEP_SKY_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color DimGray.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DIM_GRAY Online Documentation}*/
static readonly DIM_GRAY: Fill;
/**
Gets a {@link SolidColorFill} for the color DodgerBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23DODGER_BLUE Online Documentation}*/
static readonly DODGER_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color Firebrick.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23FIREBRICK Online Documentation}*/
static readonly FIREBRICK: Fill;
/**
Gets a {@link SolidColorFill} for the color FloralWhite.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23FLORAL_WHITE Online Documentation}*/
static readonly FLORAL_WHITE: Fill;
/**
Gets a {@link SolidColorFill} for the color ForestGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23FOREST_GREEN Online Documentation}*/
static readonly FOREST_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color Fuchsia.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23FUCHSIA Online Documentation}*/
static readonly FUCHSIA: Fill;
/**
Gets a {@link SolidColorFill} for the color Gainsboro.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23GAINSBORO Online Documentation}*/
static readonly GAINSBORO: Fill;
/**
Gets a {@link SolidColorFill} for the color GhostWhite.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23GHOST_WHITE Online Documentation}*/
static readonly GHOST_WHITE: Fill;
/**
Gets a {@link SolidColorFill} for the color Gold.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23GOLD Online Documentation}*/
static readonly GOLD: Fill;
/**
Gets a {@link SolidColorFill} for the color Goldenrod.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23GOLDENROD Online Documentation}*/
static readonly GOLDENROD: Fill;
/**
Gets a {@link SolidColorFill} for the color Gray.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23GRAY Online Documentation}*/
static readonly GRAY: Fill;
/**
Gets a {@link SolidColorFill} for the color Green.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23GREEN Online Documentation}*/
static readonly GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color GreenYellow.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23GREEN_YELLOW Online Documentation}*/
static readonly GREEN_YELLOW: Fill;
/**
Gets a {@link SolidColorFill} for the color Honeydew.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23HONEYDEW Online Documentation}*/
static readonly HONEYDEW: Fill;
/**
Gets a {@link SolidColorFill} for the color HotPink.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23HOT_PINK Online Documentation}*/
static readonly HOT_PINK: Fill;
/**
Gets a {@link SolidColorFill} for the color IndianRed.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23INDIAN_RED Online Documentation}*/
static readonly INDIAN_RED: Fill;
/**
Gets a {@link SolidColorFill} for the color Indigo.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23INDIGO Online Documentation}*/
static readonly INDIGO: Fill;
/**
Gets a {@link SolidColorFill} for the color Ivory.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23IVORY Online Documentation}*/
static readonly IVORY: Fill;
/**
Gets a {@link SolidColorFill} for the color Khaki.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23KHAKI Online Documentation}*/
static readonly KHAKI: Fill;
/**
Gets a {@link SolidColorFill} for the color Lavender.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LAVENDER Online Documentation}*/
static readonly LAVENDER: Fill;
/**
Gets a {@link SolidColorFill} for the color LavenderBlush.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LAVENDER_BLUSH Online Documentation}*/
static readonly LAVENDER_BLUSH: Fill;
/**
Gets a {@link SolidColorFill} for the color LawnGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LAWN_GREEN Online Documentation}*/
static readonly LAWN_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color LemonChiffon.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LEMON_CHIFFON Online Documentation}*/
static readonly LEMON_CHIFFON: Fill;
/**
Gets a {@link SolidColorFill} for the color LightBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_BLUE Online Documentation}*/
static readonly LIGHT_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color LightCoral.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_CORAL Online Documentation}*/
static readonly LIGHT_CORAL: Fill;
/**
Gets a {@link SolidColorFill} for the color LightCyan.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_CYAN Online Documentation}*/
static readonly LIGHT_CYAN: Fill;
/**
Gets a {@link SolidColorFill} for the color LightGoldenrodYellow.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_GOLDENROD_YELLOW Online Documentation}*/
static readonly LIGHT_GOLDENROD_YELLOW: Fill;
/**
Gets a {@link SolidColorFill} for the color LightGray.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_GRAY Online Documentation}*/
static readonly LIGHT_GRAY: Fill;
/**
Gets a {@link SolidColorFill} for the color LightGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_GREEN Online Documentation}*/
static readonly LIGHT_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color LightPink.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_PINK Online Documentation}*/
static readonly LIGHT_PINK: Fill;
/**
Gets a {@link SolidColorFill} for the color LightSalmon.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_SALMON Online Documentation}*/
static readonly LIGHT_SALMON: Fill;
/**
Gets a {@link SolidColorFill} for the color LightSeaGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_SEA_GREEN Online Documentation}*/
static readonly LIGHT_SEA_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color LightSkyBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_SKY_BLUE Online Documentation}*/
static readonly LIGHT_SKY_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color LightSlateGray.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_SLATE_GRAY Online Documentation}*/
static readonly LIGHT_SLATE_GRAY: Fill;
/**
Gets a {@link SolidColorFill} for the color LightSteelBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_STEEL_BLUE Online Documentation}*/
static readonly LIGHT_STEEL_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color LightYellow.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIGHT_YELLOW Online Documentation}*/
static readonly LIGHT_YELLOW: Fill;
/**
Gets a {@link SolidColorFill} for the color Lime.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIME Online Documentation}*/
static readonly LIME: Fill;
/**
Gets a {@link SolidColorFill} for the color LimeGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LIME_GREEN Online Documentation}*/
static readonly LIME_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color Linen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23LINEN Online Documentation}*/
static readonly LINEN: Fill;
/**
Gets a {@link SolidColorFill} for the color Magenta.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MAGENTA Online Documentation}*/
static readonly MAGENTA: Fill;
/**
Gets a {@link SolidColorFill} for the color Maroon.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MAROON Online Documentation}*/
static readonly MAROON: Fill;
/**
Gets a {@link SolidColorFill} for the color MediumAquamarine.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MEDIUM_AQUAMARINE Online Documentation}*/
static readonly MEDIUM_AQUAMARINE: Fill;
/**
Gets a {@link SolidColorFill} for the color MediumBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MEDIUM_BLUE Online Documentation}*/
static readonly MEDIUM_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color MediumOrchid.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MEDIUM_ORCHID Online Documentation}*/
static readonly MEDIUM_ORCHID: Fill;
/**
Gets a {@link SolidColorFill} for the color MediumPurple.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MEDIUM_PURPLE Online Documentation}*/
static readonly MEDIUM_PURPLE: Fill;
/**
Gets a {@link SolidColorFill} for the color MediumSeaGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MEDIUM_SEA_GREEN Online Documentation}*/
static readonly MEDIUM_SEA_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color MediumSlateBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MEDIUM_SLATE_BLUE Online Documentation}*/
static readonly MEDIUM_SLATE_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color MediumSpringGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MEDIUM_SPRING_GREEN Online Documentation}*/
static readonly MEDIUM_SPRING_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color MediumTurquoise.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MEDIUM_TURQUOISE Online Documentation}*/
static readonly MEDIUM_TURQUOISE: Fill;
/**
Gets a {@link SolidColorFill} for the color MediumVioletRed.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MEDIUM_VIOLET_RED Online Documentation}*/
static readonly MEDIUM_VIOLET_RED: Fill;
/**
Gets a {@link SolidColorFill} for the color MidnightBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MIDNIGHT_BLUE Online Documentation}*/
static readonly MIDNIGHT_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color MintCream.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MINT_CREAM Online Documentation}*/
static readonly MINT_CREAM: Fill;
/**
Gets a {@link SolidColorFill} for the color MistyRose.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MISTY_ROSE Online Documentation}*/
static readonly MISTY_ROSE: Fill;
/**
Gets a {@link SolidColorFill} for the color Moccasin.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23MOCCASIN Online Documentation}*/
static readonly MOCCASIN: Fill;
/**
Gets a {@link SolidColorFill} for the color NavajoWhite.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23NAVAJO_WHITE Online Documentation}*/
static readonly NAVAJO_WHITE: Fill;
/**
Gets a {@link SolidColorFill} for the color Navy.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23NAVY Online Documentation}*/
static readonly NAVY: Fill;
/**
Gets a {@link SolidColorFill} for the color OldLace.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23OLD_LACE Online Documentation}*/
static readonly OLD_LACE: Fill;
/**
Gets a {@link SolidColorFill} for the color Olive.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23OLIVE Online Documentation}*/
static readonly OLIVE: Fill;
/**
Gets a {@link SolidColorFill} for the color OliveDrab.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23OLIVE_DRAB Online Documentation}*/
static readonly OLIVE_DRAB: Fill;
/**
Gets a {@link SolidColorFill} for the color Orange.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23ORANGE Online Documentation}*/
static readonly ORANGE: Fill;
/**
Gets a {@link SolidColorFill} for the color OrangeRed.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23ORANGE_RED Online Documentation}*/
static readonly ORANGE_RED: Fill;
/**
Gets a {@link SolidColorFill} for the color Orchid.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23ORCHID Online Documentation}*/
static readonly ORCHID: Fill;
/**
Gets a {@link SolidColorFill} for the color PaleGoldenrod.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PALE_GOLDENROD Online Documentation}*/
static readonly PALE_GOLDENROD: Fill;
/**
Gets a {@link SolidColorFill} for the color PaleGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PALE_GREEN Online Documentation}*/
static readonly PALE_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color PaleTurquoise.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PALE_TURQUOISE Online Documentation}*/
static readonly PALE_TURQUOISE: Fill;
/**
Gets a {@link SolidColorFill} for the color PaleVioletRed.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PALE_VIOLET_RED Online Documentation}*/
static readonly PALE_VIOLET_RED: Fill;
/**
Gets a {@link SolidColorFill} for the color PapayaWhip.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PAPAYA_WHIP Online Documentation}*/
static readonly PAPAYA_WHIP: Fill;
/**
Gets a {@link SolidColorFill} for the color PeachPuff.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PEACH_PUFF Online Documentation}*/
static readonly PEACH_PUFF: Fill;
/**
Gets a {@link SolidColorFill} for the color Peru.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PERU Online Documentation}*/
static readonly PERU: Fill;
/**
Gets a {@link SolidColorFill} for the color Pink.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PINK Online Documentation}*/
static readonly PINK: Fill;
/**
Gets a {@link SolidColorFill} for the color Plum.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PLUM Online Documentation}*/
static readonly PLUM: Fill;
/**
Gets a {@link SolidColorFill} for the color PowderBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23POWDER_BLUE Online Documentation}*/
static readonly POWDER_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color Purple.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23PURPLE Online Documentation}*/
static readonly PURPLE: Fill;
/**
Gets a {@link SolidColorFill} for the color Red.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23RED Online Documentation}*/
static readonly RED: Fill;
/**
Gets a {@link SolidColorFill} for the color RosyBrown.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23ROSY_BROWN Online Documentation}*/
static readonly ROSY_BROWN: Fill;
/**
Gets a {@link SolidColorFill} for the color RoyalBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23ROYAL_BLUE Online Documentation}*/
static readonly ROYAL_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color SaddleBrown.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SADDLE_BROWN Online Documentation}*/
static readonly SADDLE_BROWN: Fill;
/**
Gets a {@link SolidColorFill} for the color Salmon.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SALMON Online Documentation}*/
static readonly SALMON: Fill;
/**
Gets a {@link SolidColorFill} for the color SandyBrown.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SANDY_BROWN Online Documentation}*/
static readonly SANDY_BROWN: Fill;
/**
Gets a {@link SolidColorFill} for the color SeaGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SEA_GREEN Online Documentation}*/
static readonly SEA_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color SeaShell.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SEA_SHELL Online Documentation}*/
static readonly SEA_SHELL: Fill;
/**
Gets a {@link SolidColorFill} for the color Sienna.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SIENNA Online Documentation}*/
static readonly SIENNA: Fill;
/**
Gets a {@link SolidColorFill} for the color Silver.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SILVER Online Documentation}*/
static readonly SILVER: Fill;
/**
Gets a {@link SolidColorFill} for the color SkyBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SKY_BLUE Online Documentation}*/
static readonly SKY_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color SlateBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SLATE_BLUE Online Documentation}*/
static readonly SLATE_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color SlateGray.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SLATE_GRAY Online Documentation}*/
static readonly SLATE_GRAY: Fill;
/**
Gets a {@link SolidColorFill} for the color Snow.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SNOW Online Documentation}*/
static readonly SNOW: Fill;
/**
Gets a {@link SolidColorFill} for the color SpringGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23SPRING_GREEN Online Documentation}*/
static readonly SPRING_GREEN: Fill;
/**
Gets a {@link SolidColorFill} for the color SteelBlue.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23STEEL_BLUE Online Documentation}*/
static readonly STEEL_BLUE: Fill;
/**
Gets a {@link SolidColorFill} for the color Tan.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23TAN Online Documentation}*/
static readonly TAN: Fill;
/**
Gets a {@link SolidColorFill} for the color Teal.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23TEAL Online Documentation}*/
static readonly TEAL: Fill;
/**
Gets a {@link SolidColorFill} for the color Thistle.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23THISTLE Online Documentation}*/
static readonly THISTLE: Fill;
/**
Gets a {@link SolidColorFill} for the color Tomato.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23TOMATO Online Documentation}*/
static readonly TOMATO: Fill;
/**
Gets a {@link SolidColorFill} for the color Transparent.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23TRANSPARENT Online Documentation}*/
static readonly TRANSPARENT: Fill;
/**
Gets a {@link SolidColorFill} for the color Turquoise.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23TURQUOISE Online Documentation}*/
static readonly TURQUOISE: Fill;
/**
Gets a {@link SolidColorFill} for the color Violet.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23VIOLET Online Documentation}*/
static readonly VIOLET: Fill;
/**
Gets a {@link SolidColorFill} for the color Wheat.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23WHEAT Online Documentation}*/
static readonly WHEAT: Fill;
/**
Gets a {@link SolidColorFill} for the color White.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23WHITE Online Documentation}*/
static readonly WHITE: Fill;
/**
Gets a {@link SolidColorFill} for the color WhiteSmoke.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23WHITE_SMOKE Online Documentation}*/
static readonly WHITE_SMOKE: Fill;
/**
Gets a {@link SolidColorFill} for the color Yellow.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23YELLOW Online Documentation}*/
static readonly YELLOW: Fill;
/**
Gets a {@link SolidColorFill} for the color YellowGreen.
This property will yield a {@link Fill#isFrozen frozen} shared immutable value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Fill%23YELLOW_GREEN Online Documentation}*/
static readonly YELLOW_GREEN: Fill;
static $class: Class;
private $_Fill: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Fill;
}
/** Automatic conversion to {@link Fill}. */
type FillConvertible = string;
export interface Color extends YObject {
}
/**
* Represents an ARGB (alpha, red, green, blue) encoded color object.
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* CSS color strings are converted to {@link Color}s. For example:
*
* <pre>
* 'blue'
* '#369'
* '#336699'
* 'rgb(51, 102, 153)'
* 'rgba(51, 102, 153, 0.5)'
* 'hsl(225, 98%, 60%)'
* 'hsla(225, 98%, 60%, 0.5)'
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Color Online Documentation}
*/
export class Color {
/**
Initializes a new instance of the {@link Color} class.
@param [a=255] The alpha value; <code>255</code> by default.
@param [r=0] The red value; <code>0</code> by default.
@param [g=0] The green value; <code>0</code> by default
@param [b=0] The blue value; <code>0</code> by default.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color Online Documentation}*/
constructor(r?: number, g?: number, b?: number, a?: number);
/**
Initializes a new instance of the {@link Color} class.
@param options The parameters to pass.
@param [options.a=255] The alpha value; <code>255</code> by default.
@param [options.r=0] The red value; <code>0</code> by default.
@param [options.g=0] The green value; <code>0</code> by default
@param [options.b=0] The blue value; <code>0</code> by default.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color Online Documentation}*/
constructor(options: {
r?: number;
g?: number;
b?: number;
a?: number;
});
/**
Determines whether this color is equal to the specified object.
@param obj The other.
@returns <code>true</code>, if this color is equal to the other object; <code>false</code>, otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23equals(Object) Online Documentation}*/
equals(obj: any | null): boolean;
/**
Creates a {@link Color} instance from the given color-like object by performing automatic type conversion.
@param colorLike The object to convert to a {@link Color}.
@returns Either <code>colorLike</code> if it is already a {@link Color} or a new instance, initialized to the values found in <code>colorLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23from(Color) Online Documentation}*/
static from(colorLike: Color | ColorConvertible): Color;
/**
Creates a new color instance from ARGB values.
All values must be between 0 and 255.
@param a The alpha value between 0 and 255.
@param r The red value between 0 and 255.
@param g The green value between 0 and 255.
@param b The blue value between 0 and 255.
@returns A new color instance with the given values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23fromArgb(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static fromArgb(a: number, r: number, g: number, b: number): Color;
/**
Creates a new color instance from HSL and A values.
This works similar to the CSS hsla function.
@param h The alpha value between 0 and 1.
@param s The red value between 0 and 1.
@param l The green value between 0 and 1.
@param [a=1] The blue value between 0 and 1. 1 if omitted
@returns A new color instance with the given values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23fromHSLA(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static fromHSLA(h: number, s: number, l: number, a?: number): Color;
/**
Creates a new color instance from RGB and A values.
This works similar to the CSS rgba function.
@param [a=1] The alpha fractional value between 0 and 1. 1 if omitted
@param r The red value between 0 and 255.
@param g The green value between 0 and 255.
@param b The blue value between 0 and 255.
@returns A new color instance with the given values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23fromRGBA(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static fromRGBA(r: number, g: number, b: number, a?: number): Color;
/**
Returns a hash code for this instance.
@returns A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Gets the value of the red component of the color.
The value is between 0 and 255. A higher value represents a higher part of the color is made up of this component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23r Online Documentation}*/
readonly r: number;
/**
Gets the value of the green component of the color.
The value is between 0 and 255. A higher value represents a higher part of the color is made up of this component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23g Online Documentation}*/
readonly g: number;
/**
Gets the value of the blue component of the color.
The value is between 0 and 255. A higher value represents a higher part of the color is made up of this component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23b Online Documentation}*/
readonly b: number;
/**
Gets the value of the alpha component of the color.
The value is between 0 and 255. If this component is 0, then the color will be transparent.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23a Online Documentation}*/
readonly a: number;
/**
Yields a new {@link Color} for the wellknown name AliceBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23ALICE_BLUE Online Documentation}*/
static readonly ALICE_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name AntiqueWhite.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23ANTIQUE_WHITE Online Documentation}*/
static readonly ANTIQUE_WHITE: Color;
/**
Yields a new {@link Color} for the wellknown name Aqua.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23AQUA Online Documentation}*/
static readonly AQUA: Color;
/**
Yields a new {@link Color} for the wellknown name Aquamarine.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23AQUAMARINE Online Documentation}*/
static readonly AQUAMARINE: Color;
/**
Yields a new {@link Color} for the wellknown name Azure.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23AZURE Online Documentation}*/
static readonly AZURE: Color;
/**
Yields a new {@link Color} for the wellknown name Beige.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23BEIGE Online Documentation}*/
static readonly BEIGE: Color;
/**
Yields a new {@link Color} for the wellknown name Bisque.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23BISQUE Online Documentation}*/
static readonly BISQUE: Color;
/**
Yields a new {@link Color} for the wellknown name Black.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23BLACK Online Documentation}*/
static readonly BLACK: Color;
/**
Yields a new {@link Color} for the wellknown name BlanchedAlmond.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23BLANCHED_ALMOND Online Documentation}*/
static readonly BLANCHED_ALMOND: Color;
/**
Yields a new {@link Color} for the wellknown name Blue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23BLUE Online Documentation}*/
static readonly BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name BlueViolet.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23BLUE_VIOLET Online Documentation}*/
static readonly BLUE_VIOLET: Color;
/**
Yields a new {@link Color} for the wellknown name Brown.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23BROWN Online Documentation}*/
static readonly BROWN: Color;
/**
Yields a new {@link Color} for the wellknown name BurlyWood.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23BURLY_WOOD Online Documentation}*/
static readonly BURLY_WOOD: Color;
/**
Yields a new {@link Color} for the wellknown name CadetBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23CADET_BLUE Online Documentation}*/
static readonly CADET_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name Chartreuse.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23CHARTREUSE Online Documentation}*/
static readonly CHARTREUSE: Color;
/**
Yields a new {@link Color} for the wellknown name Chocolate.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23CHOCOLATE Online Documentation}*/
static readonly CHOCOLATE: Color;
/**
Yields a new {@link Color} for the wellknown name Coral.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23CORAL Online Documentation}*/
static readonly CORAL: Color;
/**
Yields a new {@link Color} for the wellknown name CornflowerBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23CORNFLOWER_BLUE Online Documentation}*/
static readonly CORNFLOWER_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name Cornsilk.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23CORNSILK Online Documentation}*/
static readonly CORNSILK: Color;
/**
Yields a new {@link Color} for the wellknown name Crimson.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23CRIMSON Online Documentation}*/
static readonly CRIMSON: Color;
/**
Yields a new {@link Color} for the wellknown name Cyan.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23CYAN Online Documentation}*/
static readonly CYAN: Color;
/**
Yields a new {@link Color} for the wellknown name DarkBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_BLUE Online Documentation}*/
static readonly DARK_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name DarkCyan.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_CYAN Online Documentation}*/
static readonly DARK_CYAN: Color;
/**
Yields a new {@link Color} for the wellknown name DarkGoldenrod.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_GOLDENROD Online Documentation}*/
static readonly DARK_GOLDENROD: Color;
/**
Yields a new {@link Color} for the wellknown name DarkGray.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_GRAY Online Documentation}*/
static readonly DARK_GRAY: Color;
/**
Yields a new {@link Color} for the wellknown name DarkGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_GREEN Online Documentation}*/
static readonly DARK_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name DarkKhaki.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_KHAKI Online Documentation}*/
static readonly DARK_KHAKI: Color;
/**
Yields a new {@link Color} for the wellknown name DarkMagenta.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_MAGENTA Online Documentation}*/
static readonly DARK_MAGENTA: Color;
/**
Yields a new {@link Color} for the wellknown name DarkOliveGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_OLIVE_GREEN Online Documentation}*/
static readonly DARK_OLIVE_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name DarkOrange.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_ORANGE Online Documentation}*/
static readonly DARK_ORANGE: Color;
/**
Yields a new {@link Color} for the wellknown name DarkOrchid.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_ORCHID Online Documentation}*/
static readonly DARK_ORCHID: Color;
/**
Yields a new {@link Color} for the wellknown name DarkRed.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_RED Online Documentation}*/
static readonly DARK_RED: Color;
/**
Yields a new {@link Color} for the wellknown name DarkSalmon.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_SALMON Online Documentation}*/
static readonly DARK_SALMON: Color;
/**
Yields a new {@link Color} for the wellknown name DarkSeaGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_SEA_GREEN Online Documentation}*/
static readonly DARK_SEA_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name DarkSlateBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_SLATE_BLUE Online Documentation}*/
static readonly DARK_SLATE_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name DarkSlateGray.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_SLATE_GRAY Online Documentation}*/
static readonly DARK_SLATE_GRAY: Color;
/**
Yields a new {@link Color} for the wellknown name DarkTurquoise.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_TURQUOISE Online Documentation}*/
static readonly DARK_TURQUOISE: Color;
/**
Yields a new {@link Color} for the wellknown name DarkViolet.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DARK_VIOLET Online Documentation}*/
static readonly DARK_VIOLET: Color;
/**
Yields a new {@link Color} for the wellknown name DeepPink.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DEEP_PINK Online Documentation}*/
static readonly DEEP_PINK: Color;
/**
Yields a new {@link Color} for the wellknown name DeepSkyBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DEEP_SKY_BLUE Online Documentation}*/
static readonly DEEP_SKY_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name DimGray.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DIM_GRAY Online Documentation}*/
static readonly DIM_GRAY: Color;
/**
Yields a new {@link Color} for the wellknown name DodgerBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23DODGER_BLUE Online Documentation}*/
static readonly DODGER_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name Firebrick.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23FIREBRICK Online Documentation}*/
static readonly FIREBRICK: Color;
/**
Yields a new {@link Color} for the wellknown name FloralWhite.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23FLORAL_WHITE Online Documentation}*/
static readonly FLORAL_WHITE: Color;
/**
Yields a new {@link Color} for the wellknown name ForestGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23FOREST_GREEN Online Documentation}*/
static readonly FOREST_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name Fuchsia.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23FUCHSIA Online Documentation}*/
static readonly FUCHSIA: Color;
/**
Yields a new {@link Color} for the wellknown name Gainsboro.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23GAINSBORO Online Documentation}*/
static readonly GAINSBORO: Color;
/**
Yields a new {@link Color} for the wellknown name GhostWhite.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23GHOST_WHITE Online Documentation}*/
static readonly GHOST_WHITE: Color;
/**
Yields a new {@link Color} for the wellknown name Gold.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23GOLD Online Documentation}*/
static readonly GOLD: Color;
/**
Yields a new {@link Color} for the wellknown name Goldenrod.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23GOLDENROD Online Documentation}*/
static readonly GOLDENROD: Color;
/**
Yields a new {@link Color} for the wellknown name Gray.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23GRAY Online Documentation}*/
static readonly GRAY: Color;
/**
Yields a new {@link Color} for the wellknown name Green.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23GREEN Online Documentation}*/
static readonly GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name GreenYellow.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23GREEN_YELLOW Online Documentation}*/
static readonly GREEN_YELLOW: Color;
/**
Yields a new {@link Color} for the wellknown name Honeydew.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23HONEYDEW Online Documentation}*/
static readonly HONEYDEW: Color;
/**
Yields a new {@link Color} for the wellknown name HotPink.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23HOT_PINK Online Documentation}*/
static readonly HOT_PINK: Color;
/**
Yields a new {@link Color} for the wellknown name IndianRed.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23INDIAN_RED Online Documentation}*/
static readonly INDIAN_RED: Color;
/**
Yields a new {@link Color} for the wellknown name Indigo.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23INDIGO Online Documentation}*/
static readonly INDIGO: Color;
/**
Yields a new {@link Color} for the wellknown name Ivory.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23IVORY Online Documentation}*/
static readonly IVORY: Color;
/**
Yields a new {@link Color} for the wellknown name Khaki.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23KHAKI Online Documentation}*/
static readonly KHAKI: Color;
/**
Yields a new {@link Color} for the wellknown name Lavender.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LAVENDER Online Documentation}*/
static readonly LAVENDER: Color;
/**
Yields a new {@link Color} for the wellknown name LavenderBlush.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LAVENDER_BLUSH Online Documentation}*/
static readonly LAVENDER_BLUSH: Color;
/**
Yields a new {@link Color} for the wellknown name LawnGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LAWN_GREEN Online Documentation}*/
static readonly LAWN_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name LemonChiffon.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LEMON_CHIFFON Online Documentation}*/
static readonly LEMON_CHIFFON: Color;
/**
Yields a new {@link Color} for the wellknown name LightBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_BLUE Online Documentation}*/
static readonly LIGHT_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name LightCoral.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_CORAL Online Documentation}*/
static readonly LIGHT_CORAL: Color;
/**
Yields a new {@link Color} for the wellknown name LightCyan.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_CYAN Online Documentation}*/
static readonly LIGHT_CYAN: Color;
/**
Yields a new {@link Color} for the wellknown name LightGoldenrodYellow.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_GOLDENROD_YELLOW Online Documentation}*/
static readonly LIGHT_GOLDENROD_YELLOW: Color;
/**
Yields a new {@link Color} for the wellknown name LightGray.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_GRAY Online Documentation}*/
static readonly LIGHT_GRAY: Color;
/**
Yields a new {@link Color} for the wellknown name LightGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_GREEN Online Documentation}*/
static readonly LIGHT_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name LightPink.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_PINK Online Documentation}*/
static readonly LIGHT_PINK: Color;
/**
Yields a new {@link Color} for the wellknown name LightSalmon.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_SALMON Online Documentation}*/
static readonly LIGHT_SALMON: Color;
/**
Yields a new {@link Color} for the wellknown name LightSeaGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_SEA_GREEN Online Documentation}*/
static readonly LIGHT_SEA_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name LightSkyBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_SKY_BLUE Online Documentation}*/
static readonly LIGHT_SKY_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name LightSlateGray.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_SLATE_GRAY Online Documentation}*/
static readonly LIGHT_SLATE_GRAY: Color;
/**
Yields a new {@link Color} for the wellknown name LightSteelBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_STEEL_BLUE Online Documentation}*/
static readonly LIGHT_STEEL_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name LightYellow.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIGHT_YELLOW Online Documentation}*/
static readonly LIGHT_YELLOW: Color;
/**
Yields a new {@link Color} for the wellknown name Lime.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIME Online Documentation}*/
static readonly LIME: Color;
/**
Yields a new {@link Color} for the wellknown name LimeGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LIME_GREEN Online Documentation}*/
static readonly LIME_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name Linen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23LINEN Online Documentation}*/
static readonly LINEN: Color;
/**
Yields a new {@link Color} for the wellknown name Magenta.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MAGENTA Online Documentation}*/
static readonly MAGENTA: Color;
/**
Yields a new {@link Color} for the wellknown name Maroon.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MAROON Online Documentation}*/
static readonly MAROON: Color;
/**
Yields a new {@link Color} for the wellknown name MediumAquamarine.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MEDIUM_AQUAMARINE Online Documentation}*/
static readonly MEDIUM_AQUAMARINE: Color;
/**
Yields a new {@link Color} for the wellknown name MediumBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MEDIUM_BLUE Online Documentation}*/
static readonly MEDIUM_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name MediumOrchid.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MEDIUM_ORCHID Online Documentation}*/
static readonly MEDIUM_ORCHID: Color;
/**
Yields a new {@link Color} for the wellknown name MediumPurple.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MEDIUM_PURPLE Online Documentation}*/
static readonly MEDIUM_PURPLE: Color;
/**
Yields a new {@link Color} for the wellknown name MediumSeaGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MEDIUM_SEA_GREEN Online Documentation}*/
static readonly MEDIUM_SEA_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name MediumSlateBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MEDIUM_SLATE_BLUE Online Documentation}*/
static readonly MEDIUM_SLATE_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name MediumSpringGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MEDIUM_SPRING_GREEN Online Documentation}*/
static readonly MEDIUM_SPRING_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name MediumTurquoise.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MEDIUM_TURQUOISE Online Documentation}*/
static readonly MEDIUM_TURQUOISE: Color;
/**
Yields a new {@link Color} for the wellknown name MediumVioletRed.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MEDIUM_VIOLET_RED Online Documentation}*/
static readonly MEDIUM_VIOLET_RED: Color;
/**
Yields a new {@link Color} for the wellknown name MidnightBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MIDNIGHT_BLUE Online Documentation}*/
static readonly MIDNIGHT_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name MintCream.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MINT_CREAM Online Documentation}*/
static readonly MINT_CREAM: Color;
/**
Yields a new {@link Color} for the wellknown name MistyRose.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MISTY_ROSE Online Documentation}*/
static readonly MISTY_ROSE: Color;
/**
Yields a new {@link Color} for the wellknown name Moccasin.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23MOCCASIN Online Documentation}*/
static readonly MOCCASIN: Color;
/**
Yields a new {@link Color} for the wellknown name NavajoWhite.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23NAVAJO_WHITE Online Documentation}*/
static readonly NAVAJO_WHITE: Color;
/**
Yields a new {@link Color} for the wellknown name Navy.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23NAVY Online Documentation}*/
static readonly NAVY: Color;
/**
Yields a new {@link Color} for the wellknown name OldLace.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23OLD_LACE Online Documentation}*/
static readonly OLD_LACE: Color;
/**
Yields a new {@link Color} for the wellknown name Olive.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23OLIVE Online Documentation}*/
static readonly OLIVE: Color;
/**
Yields a new {@link Color} for the wellknown name OliveDrab.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23OLIVE_DRAB Online Documentation}*/
static readonly OLIVE_DRAB: Color;
/**
Yields a new {@link Color} for the wellknown name Orange.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23ORANGE Online Documentation}*/
static readonly ORANGE: Color;
/**
Yields a new {@link Color} for the wellknown name OrangeRed.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23ORANGE_RED Online Documentation}*/
static readonly ORANGE_RED: Color;
/**
Yields a new {@link Color} for the wellknown name Orchid.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23ORCHID Online Documentation}*/
static readonly ORCHID: Color;
/**
Yields a new {@link Color} for the wellknown name PaleGoldenrod.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PALE_GOLDENROD Online Documentation}*/
static readonly PALE_GOLDENROD: Color;
/**
Yields a new {@link Color} for the wellknown name PaleGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PALE_GREEN Online Documentation}*/
static readonly PALE_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name PaleTurquoise.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PALE_TURQUOISE Online Documentation}*/
static readonly PALE_TURQUOISE: Color;
/**
Yields a new {@link Color} for the wellknown name PaleVioletRed.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PALE_VIOLET_RED Online Documentation}*/
static readonly PALE_VIOLET_RED: Color;
/**
Yields a new {@link Color} for the wellknown name PapayaWhip.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PAPAYA_WHIP Online Documentation}*/
static readonly PAPAYA_WHIP: Color;
/**
Yields a new {@link Color} for the wellknown name PeachPuff.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PEACH_PUFF Online Documentation}*/
static readonly PEACH_PUFF: Color;
/**
Yields a new {@link Color} for the wellknown name Peru.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PERU Online Documentation}*/
static readonly PERU: Color;
/**
Yields a new {@link Color} for the wellknown name Pink.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PINK Online Documentation}*/
static readonly PINK: Color;
/**
Yields a new {@link Color} for the wellknown name Plum.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PLUM Online Documentation}*/
static readonly PLUM: Color;
/**
Yields a new {@link Color} for the wellknown name PowderBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23POWDER_BLUE Online Documentation}*/
static readonly POWDER_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name Purple.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23PURPLE Online Documentation}*/
static readonly PURPLE: Color;
/**
Yields a new {@link Color} for the wellknown name Red.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23RED Online Documentation}*/
static readonly RED: Color;
/**
Yields a new {@link Color} for the wellknown name RosyBrown.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23ROSY_BROWN Online Documentation}*/
static readonly ROSY_BROWN: Color;
/**
Yields a new {@link Color} for the wellknown name RoyalBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23ROYAL_BLUE Online Documentation}*/
static readonly ROYAL_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name SaddleBrown.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SADDLE_BROWN Online Documentation}*/
static readonly SADDLE_BROWN: Color;
/**
Yields a new {@link Color} for the wellknown name Salmon.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SALMON Online Documentation}*/
static readonly SALMON: Color;
/**
Yields a new {@link Color} for the wellknown name SandyBrown.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SANDY_BROWN Online Documentation}*/
static readonly SANDY_BROWN: Color;
/**
Yields a new {@link Color} for the wellknown name SeaGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SEA_GREEN Online Documentation}*/
static readonly SEA_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name SeaShell.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SEA_SHELL Online Documentation}*/
static readonly SEA_SHELL: Color;
/**
Yields a new {@link Color} for the wellknown name Sienna.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SIENNA Online Documentation}*/
static readonly SIENNA: Color;
/**
Yields a new {@link Color} for the wellknown name Silver.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SILVER Online Documentation}*/
static readonly SILVER: Color;
/**
Yields a new {@link Color} for the wellknown name SkyBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SKY_BLUE Online Documentation}*/
static readonly SKY_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name SlateBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SLATE_BLUE Online Documentation}*/
static readonly SLATE_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name SlateGray.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SLATE_GRAY Online Documentation}*/
static readonly SLATE_GRAY: Color;
/**
Yields a new {@link Color} for the wellknown name Snow.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SNOW Online Documentation}*/
static readonly SNOW: Color;
/**
Yields a new {@link Color} for the wellknown name SpringGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23SPRING_GREEN Online Documentation}*/
static readonly SPRING_GREEN: Color;
/**
Yields a new {@link Color} for the wellknown name SteelBlue.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23STEEL_BLUE Online Documentation}*/
static readonly STEEL_BLUE: Color;
/**
Yields a new {@link Color} for the wellknown name Tan.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23TAN Online Documentation}*/
static readonly TAN: Color;
/**
Yields a new {@link Color} for the wellknown name Teal.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23TEAL Online Documentation}*/
static readonly TEAL: Color;
/**
Yields a new {@link Color} for the wellknown name Thistle.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23THISTLE Online Documentation}*/
static readonly THISTLE: Color;
/**
Yields a new {@link Color} for the wellknown name Tomato.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23TOMATO Online Documentation}*/
static readonly TOMATO: Color;
/**
Yields a new {@link Color} for the wellknown name Transparent.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23TRANSPARENT Online Documentation}*/
static readonly TRANSPARENT: Color;
/**
Yields a new {@link Color} for the wellknown name Turquoise.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23TURQUOISE Online Documentation}*/
static readonly TURQUOISE: Color;
/**
Yields a new {@link Color} for the wellknown name Violet.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23VIOLET Online Documentation}*/
static readonly VIOLET: Color;
/**
Yields a new {@link Color} for the wellknown name Wheat.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23WHEAT Online Documentation}*/
static readonly WHEAT: Color;
/**
Yields a new {@link Color} for the wellknown name White.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23WHITE Online Documentation}*/
static readonly WHITE: Color;
/**
Yields a new {@link Color} for the wellknown name WhiteSmoke.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23WHITE_SMOKE Online Documentation}*/
static readonly WHITE_SMOKE: Color;
/**
Yields a new {@link Color} for the wellknown name Yellow.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23YELLOW Online Documentation}*/
static readonly YELLOW: Color;
/**
Yields a new {@link Color} for the wellknown name YellowGreen.
This method will yield the same value for every call because it is immutable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Color%23YELLOW_GREEN Online Documentation}*/
static readonly YELLOW_GREEN: Color;
static $class: Class;
private $_Color: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Color;
}
/** Automatic conversion to {@link Color}. */
type ColorConvertible = string;
export interface ColorExtension extends MarkupExtension {
/**
@inheritdoc*/
provideValue(serviceProvider: ILookup | null): any | null;
} /**
The markup extension for {@link Color}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColorExtension Online Documentation}*/
export class ColorExtension {
/**
Creates a new instance of this class for the given color
@param color
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColorExtension%23ColorExtension(Color) Online Documentation}*/
constructor(color: Color | ColorConvertible);
/**
Creates a new instance of this class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColorExtension%23ColorExtension() Online Documentation}*/
constructor();
/**
Returns the value of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColorExtension%23value Online Documentation}*/
value: string;
static $class: Class;
private $_ColorExtension: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ColorExtension;
}
export interface Cursor extends YObject {
}
/**
Represents a cursor object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor Online Documentation}*/
export class Cursor {
/**
Initializes a new {@link Cursor} instance using the provided url and an alternative.
@param url The url should point to a file in the ".cur" format with a size of at most 32x32 pixels to ensure compatibility across
browsers.
@param alternative Cursors can be chained but the last alternative needs to be one of the predefined cursors.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23Cursor(string%2CCursor) Online Documentation}*/
constructor(url: string, alternative: Cursor | CursorConvertible);
/**
Returns the CSS text representation for the cursor and its alternatives.
@returns The CSS text representation for the cursor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23getSystemCursor() Online Documentation}*/
getSystemCursor(): string;
/**
Gets the invisible cursor.
This corresponds to the <code>none</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: none;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23NONE Online Documentation}*/
static readonly NONE: Cursor;
/**
Gets the default cursor.
This corresponds to the <code>default</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: default;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23DEFAULT Online Documentation}*/
static readonly DEFAULT: Cursor;
/**
Gets the cursor with a pointing hand, as used for example on a hyperlink.
This corresponds to the <code>pointer</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: pointer;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23POINTER Online Documentation}*/
static readonly POINTER: Cursor;
/**
Gets the cursor that indicates that help is available for the object under the cursor.
This corresponds to the <code>help</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: help;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23HELP Online Documentation}*/
static readonly HELP: Cursor;
/**
Gets the cursor that indicates progress.
This corresponds to the <code>progress</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: progress;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23PROGRESS Online Documentation}*/
static readonly PROGRESS: Cursor;
/**
Gets the text selection cursor.
This corresponds to the <code>text</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: text;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23TEXT Online Documentation}*/
static readonly TEXT: Cursor;
/**
Gets the wait cursor.
This corresponds to the <code>wait</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: wait;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23WAIT Online Documentation}*/
static readonly WAIT: Cursor;
/**
Gets the crosshair cursor.
This corresponds to the <code>crosshair</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: crosshair;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23CROSSHAIR Online Documentation}*/
static readonly CROSSHAIR: Cursor;
/**
Gets the move cursor.
This corresponds to the <code>move</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: move;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23MOVE Online Documentation}*/
static readonly MOVE: Cursor;
/**
Gets a cursor that indicates that an edge at the east will be moved.
Depending on the browser this might just be a horizontal resize cursor.
This corresponds to the <code>e-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: e-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23E_RESIZE Online Documentation}*/
static readonly E_RESIZE: Cursor;
/**
Gets a cursor that indicates that an edge at the north will be moved.
Depending on the browser this might just be a vertical resize cursor.
This corresponds to the <code>n-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: n-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23N_RESIZE Online Documentation}*/
static readonly N_RESIZE: Cursor;
/**
Gets a cursor that indicates that edges at the north and east will be moved.
Depending on the browser this might just be the diagonal resize cursor for NE to SW resizes.
This corresponds to the <code>ne-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: ne-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23NE_RESIZE Online Documentation}*/
static readonly NE_RESIZE: Cursor;
/**
Gets a cursor that indicates that edges at the north and west will be moved.
Depending on the browser this might just be the diagonal resize cursor for NW to SE resizes.
This corresponds to the <code>nw-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: nw-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23NW_RESIZE Online Documentation}*/
static readonly NW_RESIZE: Cursor;
/**
Gets a cursor that indicates that the edge at the south will be moved.
Depending on the browser this might just be a vertical resize cursor.
This corresponds to the <code>s-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: s-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23S_RESIZE Online Documentation}*/
static readonly S_RESIZE: Cursor;
/**
Gets a cursor that indicates that edges at the south and east will be moved.
Depending on the browser this might just be the diagonal resize cursor for NW to SE resizes.
This corresponds to the <code>se-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: se-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23SE_RESIZE Online Documentation}*/
static readonly SE_RESIZE: Cursor;
/**
Gets a cursor that indicates that edges at the south and west will be moved.
Depending on the browser this might just be the diagonal resize cursor for NE to SW resizes.
This corresponds to the <code>sw-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: sw-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23SW_RESIZE Online Documentation}*/
static readonly SW_RESIZE: Cursor;
/**
Gets a cursor that indicates that the edge at the west will be moved.
Depending on the browser this might just be a horizontal resize cursor.
This corresponds to the <code>w-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: w-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23W_RESIZE Online Documentation}*/
static readonly W_RESIZE: Cursor;
/**
Gets the vertical resize cursor.
This corresponds to the <code>ns-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: ns-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23NS_RESIZE Online Documentation}*/
static readonly NS_RESIZE: Cursor;
/**
Gets the diagonal resize cursor for NW to SE resizes.
This corresponds to the <code>nwse-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: nwse-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23NWSE_RESIZE Online Documentation}*/
static readonly NWSE_RESIZE: Cursor;
/**
Gets the diagonal resize cursor for NE to SW resizes.
This corresponds to the <code>nesw-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: nesw-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23NESW_RESIZE Online Documentation}*/
static readonly NESW_RESIZE: Cursor;
/**
Gets the horizontal resize cursor.
This corresponds to the <code>ew-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: ew-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23EW_RESIZE Online Documentation}*/
static readonly EW_RESIZE: Cursor;
/**
Gets the cursor for resizing columns.
This corresponds to the <code>col-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: col-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23COL_RESIZE Online Documentation}*/
static readonly COL_RESIZE: Cursor;
/**
Gets the cursor for resizing rows.
This corresponds to the <code>row-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: row-resize;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23ROW_RESIZE Online Documentation}*/
static readonly ROW_RESIZE: Cursor;
/**
Gets the cursor where the browser determines the actual cursor to be displayed.
This corresponds to the <code>auto</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: auto;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23AUTO Online Documentation}*/
static readonly AUTO: Cursor;
/**
Gets the cursor to indicate that there is a context menu available.
This corresponds to the <code>context-menu</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: context-menu;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23CONTEXT_MENU Online Documentation}*/
static readonly CONTEXT_MENU: Cursor;
/**
Gets the cursor to indicate selection of a table cell..
This corresponds to the <code>cell</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: cell;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23CELL Online Documentation}*/
static readonly CELL: Cursor;
/**
Gets the vertical text selection cursor.
This corresponds to the <code>vertical-text</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: vertical-text;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23VERTICAL_TEXT Online Documentation}*/
static readonly VERTICAL_TEXT: Cursor;
/**
Gets the cursor to be shown when an alias or shortcut can be created.
This corresponds to the <code>alias</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: alias;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23ALIAS Online Documentation}*/
static readonly ALIAS: Cursor;
/**
Gets the cursor to be shown when something can be copied (often used for drag &amp; drop).
This corresponds to the <code>copy</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: copy;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23COPY Online Documentation}*/
static readonly COPY: Cursor;
/**
Gets the cursor to be shown where nothing can be dropped in a drag &amp; drop operation.
This corresponds to the <code>no-drop</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: no-drop;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23NO_DROP Online Documentation}*/
static readonly NO_DROP: Cursor;
/**
Gets the cursor to be shown when an action cannot be done.
This corresponds to the <code>not-allowed</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: not-allowed;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23NOT_ALLOWED Online Documentation}*/
static readonly NOT_ALLOWED: Cursor;
/**
Gets the cursor indicating that something can be scrolled or panned.
This corresponds to the <code>all-scroll</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: all-scroll;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23ALL_SCROLL Online Documentation}*/
static readonly ALL_SCROLL: Cursor;
/**
Gets the cursor for zooming in.
This corresponds to the <code>zoom-in</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: zoom-in;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23ZOOM_IN Online Documentation}*/
static readonly ZOOM_IN: Cursor;
/**
Gets the cursor for zooming out.
This corresponds to the <code>zoom-out</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: zoom-out;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23ZOOM_OUT Online Documentation}*/
static readonly ZOOM_OUT: Cursor;
/**
Gets the cursor indicating that something can be grabbed (usually in conjunction with {@link Cursor#GRABBING}).
This corresponds to the <code>grab</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: grab;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23GRAB Online Documentation}*/
static readonly GRAB: Cursor;
/**
Gets the cursor indicating that something is currently being grabbed (usually in conjunction with {@link Cursor#GRAB}).
This corresponds to the <code>grabbing</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: grabbing;" />
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursor%23GRABBING Online Documentation}*/
static readonly GRABBING: Cursor;
static $class: Class;
private $_Cursor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Cursor;
}
/** Automatic conversion to {@link Cursor}. */
type CursorConvertible = string;
export interface DashStyle extends YObject {
}
/**
* A container for {@link DashStyle#dashes} and an {@link DashStyle#offset}.
* All of these values are in terms of multiples of the the thickness of a
* {@link Stroke}. This is different from the corresponding properties of a SVG stroke.
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* CSS border-style strings are converted to {@link DashStyle}s. For example:
*
* <pre>
* 'solid'
* 'dash'
* 'dot'
* 'dash-dot'
* 'dash-dot-dot'
* '5 10'
* '5, 10'
* </pre>
*
* It is also possible to specify an {@link Array<any>} or {@link number} to define the pattern of dashes:
*
* <pre>
* [5, 10]
* 10
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle Online Documentation}
*/
export class DashStyle {
/**
Initializes a new instance of the {@link DashStyle} class using the provided dashes and offset.
@param dashes The dashes.
@param offset The offset.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23DashStyle(Array%3Cnumber%3E%2Cnumber) Online Documentation}*/
constructor(dashes: number[] | null, offset: number);
/**
Initializes a new instance of the {@link DashStyle} class using an empty {@link DashStyle#dashes} and zero {@link DashStyle#offset}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23DashStyle() Online Documentation}*/
constructor();
/**
Clones this instance by returning a {@link YObject#memberwiseClone} or this in case this instance is already {@link DashStyle#isFrozen frozen}.
@returns An instance of the same type as this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23clone() Online Documentation}*/
clone(): any;
/**
Freezes this instance.
Freezing an object makes it immutable, which can result in performance enhancements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23freeze() Online Documentation}*/
freeze(): void;
/**
Creates a {@link DashStyle} instance from the given dash-like object by performing automatic type conversion.
@param dashLike The object to convert to a {@link DashStyle}.
@returns Either <code>dashLike</code> if it is already a {@link DashStyle} or a new instance, initialized to the values found in
<code>dashLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23from(DashStyle) Online Documentation}*/
static from(dashLike: DashStyle | DashStyleConvertible): DashStyle;
/**
Determines if this instance is frozen.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23isFrozen() Online Documentation}*/
isFrozen(): boolean;
/**
Gets or sets the dashes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23dashes Online Documentation}*/
dashes: List<number>;
/**
Gets or sets the offset.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23offset Online Documentation}*/
offset: number;
/**
Gets the dash style with a single dash.
This style is designed for dash caps that extend into the 'gaps' like {@link LineCap#SQUARE} and
{@link LineCap#ROUND}. A pen with this style doesn't look as expected if its dash cap is {@link LineCap#FLAT}.
Since {@link LineCap#FLAT} is the default value, you should explicitly set another value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23DASH Online Documentation}*/
static readonly DASH: DashStyle;
/**
Gets the dash style with a single dash and dot.
This style is designed for dash caps that extend into the 'gaps' like {@link LineCap#SQUARE} and
{@link LineCap#ROUND}. A pen with this style doesn't contain visible dots if its dash cap is
{@link LineCap#FLAT}. Since {@link LineCap#FLAT} is the default value, you should explicitly set another value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23DASH_DOT Online Documentation}*/
static readonly DASH_DOT: DashStyle;
/**
Gets the dash style with a single dash and two dots.
This style is designed for dash caps that extend into the 'gaps' like {@link LineCap#SQUARE} and
{@link LineCap#ROUND}. A pen with this style doesn't contain visible dots if its dash cap is
{@link LineCap#FLAT}. Since {@link LineCap#FLAT} is the default value, you should explicitly set another value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23DASH_DOT_DOT Online Documentation}*/
static readonly DASH_DOT_DOT: DashStyle;
/**
Gets the dash style with a single dot.
This style is designed for dash caps that extend into the 'gaps' like {@link LineCap#SQUARE} and
{@link LineCap#ROUND}. A pen with this style doesn't contain visible dots if its dash cap is
{@link LineCap#FLAT}. Since {@link LineCap#FLAT} is the default value, you should explicitly set another value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23DOT Online Documentation}*/
static readonly DOT: DashStyle;
/**
Gets the solid dash style with no dashes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DashStyle%23SOLID Online Documentation}*/
static readonly SOLID: DashStyle;
static $class: Class;
private $_DashStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DashStyle;
}
/** Automatic conversion to {@link DashStyle}. */
type DashStyleConvertible = string | number[];
export interface IVisualTemplate {
}
/**
Template factory for creating visual representations of arbitrary data objects.
Implementations of this class have the purpose of creating a {@link SvgVisual} that serves as the visual
representation of the object associated with this instance. This special-purpose interface is only used in some places
where the user interface can be customized. The {@link IVisualCreator} interface is the main interface used by the
{@link CanvasComponent}.
<h2>Note</h2>
In most cases, the visual is created with {@link IVisualTemplate#createVisual} and translated internally to the
proper location. {@link IVisualTemplate#updateVisual} is only called if the object bounds change. In some cases, {@link IVisualTemplate#updateVisual}
may never be called, e.g. for handle templates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisualTemplate Online Documentation}*/
export abstract class IVisualTemplate {
/**
Creates an implementation of the interface {@link IVisualTemplate} from the given definition.
The given object must contain exactly the abstract members of the {@link IVisualTemplate} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IVisualTemplate} interface.
@returns An instance of the {@link IVisualTemplate} interface based on the given definition.*/
static create(interfaceDefinition: {
createVisual(context: IRenderContext, bounds: Rect, dataObject: any | null): SvgVisual;
updateVisual(context: IRenderContext, oldVisual: SvgVisual, bounds: Rect, dataObject: any | null): SvgVisual;
}): IVisualTemplate;
/**
This method is called by the framework to create a {@link SvgVisual} with the given <code>bounds</code>.
{@link CanvasComponent} uses this interface through the {@link ICanvasObjectDescriptor} to populate the visual
canvas object tree.
@param context The context that describes where the visual will be used.
@param bounds The current bounds to use for the visual.
@param dataObject The data object to visualize.
@returns The visual to include in the canvas object visual tree. This may be <code>null</code>.
@see IVisualTemplate#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisualTemplate%23createVisual(IRenderContext%2CRect%2CObject) Online Documentation}*/
abstract createVisual(context: IRenderContext, bounds: Rect, dataObject: any | null): SvgVisual | null;
/**
This method updates or replaces a previously created {@link SvgVisual} to adjust to the changed <code>bounds</code>.
The {@link CanvasComponent} uses this method to give implementations a chance to update an existing Visual that has
previously been created by the same instance during a call to {@link IVisualTemplate#createVisual}. Implementation
may update the <code>oldVisual</code> and return that same reference, or create a new visual and return the new instance or <code>null</code>.
<h2>Note</h2>
This method is only called if the <code>bounds</code> have changed. in some cases, it might never be called. It should not be used to
update the visual based on contextual information. Instead, you should use a custom {@link ICanvasObjectDescriptor}
for the visualized object.
@param context The context that describes where the visual will be used in.
@param oldVisual The visual instance that had been returned the last time the {@link IVisualTemplate#createVisual} method was called
on this instance.
@param bounds The current bounds to use for the visual.
@param dataObject The data object to visualize.
@returns <code>oldVisual</code>, if this instance modified the visual, or a new visual that should replace the existing one in the canvas
object visual tree.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IVisualTemplate%23updateVisual(IRenderContext%2CSvgVisual%2CRect%2CObject) Online Documentation}*/
abstract updateVisual(context: IRenderContext, oldVisual: SvgVisual, bounds: Rect, dataObject: any | null): SvgVisual | null;
static $class: Class;
private $_IVisualTemplate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IVisualTemplate;
}
export interface DragDropItem extends YObject {
}
/**
The default implementation of a data object for the {@link DropInputMode}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropItem Online Documentation}*/
export class DragDropItem {
/**
Initializes a new instance of the {@link DragDropItem} class and sets the provided data item for the specified
format.
@param [format=null] The format.
@param [data=null] The data item.
@see DragDropItem#setData
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropItem Online Documentation}*/
constructor(format?: string | null, data?: any | null);
/**
Initializes a new instance of the {@link DragDropItem} class and sets the provided data item for the specified
format.
@param options The parameters to pass.
@param [options.format=null] The format.
@param [options.data=null] The data item.
@see DragDropItem#setData
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropItem Online Documentation}*/
constructor(options: {
format?: string | null;
data?: any | null;
});
/**
Gets the data item for the given format.
@param format The format.
@returns The data item that is stored for this format, or <code>null</code> if no such item exists.
@see DragDropItem#setData
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropItem%23getData(string) Online Documentation}*/
getData(format: string): any | null;
/**
Sets the data item for the given format.
@param format The format.
@param data The data item.
@see DragDropItem#getData
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropItem%23setData(string%2CObject) Online Documentation}*/
setData(format: string, data: any): void;
/**
Gets the types of the data items stored in this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropItem%23types Online Documentation}*/
readonly types: string[] | null;
static $class: Class;
private $_DragDropItem: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DragDropItem;
}
/**
Enumeration for use with the {@link DragSource#startDrag} method that specifies the allowed actions.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects Online Documentation}*/
export enum DragDropEffects {
/**
Indicates that the drop target cannot be used for dropping the data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects%23NONE Online Documentation}*/
NONE,
/**
Indicates that the data can be copied to the drop target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects%23COPY Online Documentation}*/
COPY,
/**
Indicates that the data can be moved to the drop target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects%23MOVE Online Documentation}*/
MOVE,
/**
Indicates that the data can be linked to the drop target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects%23LINK Online Documentation}*/
LINK,
/**
Indicates that the data can be copied or linked to the drop target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects%23COPY_LINK Online Documentation}*/
COPY_LINK,
/**
Indicates that the data can be linked or copied to the drop target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects%23LINK_MOVE Online Documentation}*/
LINK_MOVE,
/**
Indicates that the data can be copied or moved to the drop target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects%23COPY_MOVE Online Documentation}*/
COPY_MOVE,
/**
Indicates that the value is uninitialized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects%23UNINITIALIZED Online Documentation}*/
UNINITIALIZED,
/**
A combination of all of the flags.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragDropEffects%23ALL Online Documentation}*/
ALL
}
/** Automatic conversion to {@link DragDropEffects}. */
type DragDropEffectsStringValues = "none" | "copy" | "move" | "link" | "copy-link" | "link-move" | "copy-move" | "uninitialized" | "all";
export interface DragEventArgs extends EventArgs {
}
/**
The event argument that will be used by {@link DropTarget} to notify handlers of the various events during a drag
and drop operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragEventArgs Online Documentation}*/
export class DragEventArgs {
/**
Gets the position of the current drag operation in the coordinates system of the provided element.
@param element The element for which the coordinates should be used.
@returns The position in the coordinates system of <code>element</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragEventArgs%23getPosition(HTMLElement) Online Documentation}*/
getPosition(element: HTMLElement | null): Point;
/**
Gets or sets the effects that the current handler can realize.
Setting this value will automatically mark this instance as {@link DragEventArgs#handled}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragEventArgs%23dropEffect Online Documentation}*/
dropEffect: DragDropEffects;
/**
Gets the key states relevant during the drag and drop operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragEventArgs%23keyStates Online Documentation}*/
readonly keyStates: DragDropKeyStates;
/**
Gets the data that is associated with the drag.
@see DragSource#startDrag
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragEventArgs%23item Online Documentation}*/
readonly item: DragDropItem;
/**
Gets the allowed effects for the current drag operation as specified by the {@link DragSource}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragEventArgs%23effectAllowed Online Documentation}*/
readonly effectAllowed: DragDropEffects;
/**
* Gets or sets a value indicating whether this {@link DragEventArgs} has been handled.
* This property needs to be adjusted by handlers to tell the engine that the event has been handled.
*
* Note that this property needs to be set before the function synchronously returns. This means that setting the value in
* an <code>async</code> function after the first <code>await</code> or in an asynchronous callback, it cannot not be seen by the emitter of the
* event and the sender will consider the event as not being handled.
*
* <pre>
* // wrong:
* mode.addDragEnterListener(async (sender, args) => {
*   await something()
*   args.handled = true  // will be set *after* all other event listeners have been invoked, only
* })
*
* // correct:
* mode.addDragEnterListener(async (sender, args) => {
*   args.handled = true // will be set immediately and 'mode' will see the change
*   await something()
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/DragEventArgs%23handled Online Documentation}
*/
handled: boolean;
static $class: Class;
private $_DragEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DragEventArgs;
}
/**
Specifies the method that is used to fill the outside of the gradient vector of a {@link LinearGradient} or
{@link RadialGradient}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientSpreadMethod Online Documentation}*/
export enum GradientSpreadMethod {
/**
The outside is filled with the terminal colors of the gradient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientSpreadMethod%23PAD Online Documentation}*/
PAD,
/**
The outside is filled by repeating the gradient but in alternating directions, until the area is filled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientSpreadMethod%23REFLECT Online Documentation}*/
REFLECT,
/**
The outside is filled by repeating the original gradient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientSpreadMethod%23REPEAT Online Documentation}*/
REPEAT
}
/** Automatic conversion to {@link GradientSpreadMethod}. */
type GradientSpreadMethodStringValues = "pad" | "reflect" | "repeat";
export interface GradientStop extends YObject, ICloneable {
}
/**
Specifies the properties of a transition point of a gradient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop Online Documentation}*/
export class GradientStop {
/**
Creates a new transition point of a gradient with the given rgba values and offset.
The rgba values must be between 0 and 255.
@param r The red value.
@param g The green value.
@param b The blue value.
@param [a=255] The alpha value, 255 by default.
@param [offset=0] The offset defined as ratio from 0 to 1. Default value is 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop%23GradientStop(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(r: number, g: number, b: number, a?: number, offset?: number);
/**
Creates a new transition point of a gradient with the given rgba values and offset.
The rgba values must be between 0 and 255.
@param options The parameters to pass.
@param options.r The red value.
@param options.g The green value.
@param options.b The blue value.
@param [options.a=255] The alpha value, 255 by default.
@param [options.offset=0] The offset defined as ratio from 0 to 1. Default value is 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop%23GradientStop(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(options: {
r: number;
g: number;
b: number;
a?: number;
offset?: number;
});
/**
Creates a new transition point of a gradient with the given color and offset.
@param [color=null] The stop's color.
@param [offset=0] The offset defined as ratio from 0 to 1. Default value is 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop%23GradientStop(Color%2Cnumber) Online Documentation}*/
constructor(color?: Color | null | ColorConvertible, offset?: number);
/**
Creates a new transition point of a gradient with the given color and offset.
@param options The parameters to pass.
@param [options.color=null] The stop's color.
@param [options.offset=0] The offset defined as ratio from 0 to 1. Default value is 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop%23GradientStop(Color%2Cnumber) Online Documentation}*/
constructor(options: {
color?: Color | null | ColorConvertible;
offset?: number;
});
/**
Clones this instance by returning a {@link YObject#memberwiseClone} or this in case this instance is already {@link GradientStop#isFrozen frozen}.
@returns An instance of the same type as this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop%23clone() Online Documentation}*/
clone(): this;
/**
Freezes this instance.
Freezing an object makes it immutable, which can result in performance enhancements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop%23freeze() Online Documentation}*/
freeze(): void;
/**
Determines if this instance is frozen.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop%23isFrozen() Online Documentation}*/
isFrozen(): boolean;
/**
Gets or sets the color of this transition point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop%23color Online Documentation}*/
color: Color;
/**
Gets or sets the offset of this transition point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GradientStop%23offset Online Documentation}*/
offset: number;
static $class: Class;
private $_GradientStop: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GradientStop;
}
export interface ICommand {
}
/**
Base interface for a command that can be executed.
Implementations of this interface are provided by the library. Use {@link ICommand#createCommand} to conveniently
create your own implementation of this interface that can then be used with
{@link KeyboardInputMode#addCommandBinding}.
@see ICommand#createCommand
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand Online Documentation}*/
export abstract class ICommand {
/**
Determines whether this command be executed with respect to the given parameter and target.
@param [parameter=null] The parameter or <code>null</code>
@param [target=null] The target to execute the command on or <code>null</code>
@returns Whether {@link ICommand#execute} can be called with the given <code>parameter</code> and <code>target</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23canExecute(Object%2CObject) Online Documentation}*/
abstract canExecute(parameter?: any | null, target?: any | null): boolean;
/**
Determines whether this command be executed with respect to the given parameter and target.
@param options The parameters to pass.
@param [options.parameter=null] The parameter or <code>null</code>
@param [options.target=null] The target to execute the command on or <code>null</code>
@returns Whether {@link ICommand#execute} can be called with the given <code>parameter</code> and <code>target</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23canExecute(Object%2CObject) Online Documentation}*/
abstract canExecute(options: {
parameter?: any | null;
target?: any | null;
}): boolean;
/**
Creates a new {@link ICommand} instance that does nothing.
The return value serves as a token and does not in itself contain any command handling logic. Use {@link KeyboardInputMode#addCommandBinding}
to associate command handlers with the instance returned.
@param [name=null] The optional name for the command.
@returns A new {@link ICommand} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23createCommand(string) Online Documentation}*/
static createCommand(name?: string | null): ICommand;
/**
Executes this command with respect to the given parameter and target.
@param [parameter=null] The parameter or <code>null</code>
@param [target=null] The target to execute the command on or <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23execute(Object%2CObject) Online Documentation}*/
abstract execute(parameter?: any | null, target?: any | null): void;
/**
Executes this command with respect to the given parameter and target.
@param options The parameters to pass.
@param [options.parameter=null] The parameter or <code>null</code>
@param [options.target=null] The target to execute the command on or <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23execute(Object%2CObject) Online Documentation}*/
abstract execute(options: {
parameter?: any | null;
target?: any | null;
}): void;
/**
Suggests a re-query of the {@link ICommand#canExecute executability} of all commands that are known to the system to adjust the UI to properly
reflect the state.
This will enqueue a delayed query into the event queue.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23invalidateRequerySuggested() Online Documentation}*/
static invalidateRequerySuggested(): void;
/**
Adds the given listener for the <code>CanExecuteChanged</code> event that occurs when the result of a call to {@link ICommand#canExecute}
may yield a different result.
@param listener The listener to add.
@see ICommand#removeCanExecuteChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23CanExecuteChanged Online Documentation}*/
abstract addCanExecuteChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>CanExecuteChanged</code> event that occurs when the result of a call to {@link ICommand#canExecute}
may yield a different result.
@param listener The listener to remove.
@see ICommand#addCanExecuteChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23CanExecuteChanged Online Documentation}*/
abstract removeCanExecuteChangedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
A {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link GraphInputMode#setSelected select} an item.
If the parameter to this command is an {@link IModelItem} instance, then that item will be selected. If the
parameter is <code>null</code> and there is exactly one selected item, then that item remains selected. Otherwise the {@link GraphComponent#currentItem current item}
will be selected.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SELECT_ITEM Online Documentation}*/
static SELECT_ITEM: ICommand;
/**
A {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link GraphInputMode#setSelected toggle the selection state} of
an item.
If the parameter to this command is an {@link IModelItem} instance, then the selection state of that item will be
toggled. If the parameter is <code>null</code> and there is exactly one selected item, then the selected item will be deselected.
Otherwise the selection state of {@link GraphComponent#currentItem current item} will be toggled.
The default shortcut for this command is Ctrl-Space.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23TOGGLE_ITEM_SELECTION Online Documentation}*/
static TOGGLE_ITEM_SELECTION: ICommand;
/**
A {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link GraphInputMode#setSelected deselect} an item.
If the parameter to this command is an {@link IModelItem} instance, then that item will be deselected. If the
parameter is <code>null</code> and there is exactly one selected item, then the selected item will be deselected.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23DESELECT_ITEM Online Documentation}*/
static DESELECT_ITEM: ICommand;
/**
A {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link GraphEditorInputMode#editLabel edit} an label.
We first determine the item to edit: If the parameter to this command is an {@link IModelItem} instance, then we
will use that item. If the parameter is <code>null</code> and there is exactly one selected item, then we will use the selected
item.
Then we determine the label to edit from the given item: If the item is an {@link ILabel} instance, then that label
will be edited. If the item is an {@link ILabelOwner} instance with an editable label, then that label will be
edited. If the item is an {@link ILabelOwner} instance with no editable label, then an new label will be added and
edited. Otherwise this will trigger the {@link GraphEditorInputMode#onEditLabel} if {@link GraphEditorInputMode}
is used.
The default shortcut for this command is F2.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23EDIT_LABEL Online Documentation}*/
static EDIT_LABEL: ICommand;
/**
A {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link GraphEditorInputMode#addLabel add another label} to the {@link ILabelOwner}
that is specified by the command parameter.
If the parameter to this command is an {@link ILabelOwner} instance, then we will use that item. If the parameter
is <code>null</code> and there is exactly one selected {@link ILabelOwner}, then we will use the selected item. If we there is
no {@link ILabelOwner} found and the {@link GraphEditorInputMode} is used, this will trigger the
{@link GraphEditorInputMode#onAddLabel}.
The default shortcut for this command is Shift+F2.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23ADD_LABEL Online Documentation}*/
static ADD_LABEL: ICommand;
/**
An {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to
{@link GraphEditorInputMode#groupSelection group the currently selected items}.
The default shortcut for this command is Ctrl+G.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23GROUP_SELECTION Online Documentation}*/
static GROUP_SELECTION: ICommand;
/**
A {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to
{@link GraphEditorInputMode#adjustGroupNodeSize adjust the size of group nodes}.
If the parameter to this command is <code>null</code>, the sizes of the currently selected group nodes are adjusted. If the parameter
is an {@link INode} instance, the size of that group node will be adjusted. If the parameter is an {@link IEnumerable} instance, then
the sizes of all group nodes in that enumeration are adjusted.
The default shortcut for this command is Ctrl-Shift-G.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23ADJUST_GROUP_NODE_SIZE Online Documentation}*/
static ADJUST_GROUP_NODE_SIZE: ICommand;
/**
An {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to
{@link GraphEditorInputMode#ungroupSelection ungroup the currently selected items}.
The default shortcut for this command is Ctrl+U.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23UNGROUP_SELECTION Online Documentation}*/
static UNGROUP_SELECTION: ICommand;
/**
An {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link ISelectionModel#clear clear the selection}.
The default shortcut for this command is Ctrl+Shift+A.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23DESELECT_ALL Online Documentation}*/
static DESELECT_ALL: ICommand;
/**
An {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link NavigationInputMode#expandGroup expand} group nodes.
If the parameter to this command is <code>null</code>, the currently selected group nodes are expanded. If the parameter is an {@link INode}
instance, that node will be expanded. If the parameter is an {@link IEnumerable} instance, then all group nodes in that enumeration are
expanded.
The default shortcut for this command is Ctrl+Add or Ctrl+Oemplus.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23EXPAND_GROUP Online Documentation}*/
static EXPAND_GROUP: ICommand;
/**
An {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link NavigationInputMode#collapseGroup collapse} group nodes.
If the parameter to this command is <code>null</code>, the currently selected group nodes are collapsed. If the parameter is an {@link INode}
instance, that group node will be collapsed. If the parameter is an {@link IEnumerable} instance, then all group nodes in that enumeration
are collapsed.
The default shortcut for this command is Ctrl+Subtract or Ctrl+OemMinus.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23COLLAPSE_GROUP Online Documentation}*/
static COLLAPSE_GROUP: ICommand;
/**
An {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link NavigationInputMode#toggleExpansionState toggle} the
collapsed/expanded state of currently selected group nodes.
If the parameter to this command is <code>null</code>, the currently selected group nodes are toggled. If the parameter is an {@link INode}
instance, that group node will be toggled. If the parameter is an {@link IEnumerable} instance, then all group nodes in that enumeration
are toggled.
The default shortcut for this command is Ctrl+Multiply.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23TOGGLE_EXPANSION_STATE Online Documentation}*/
static TOGGLE_EXPANSION_STATE: ICommand;
/**
An {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link IFoldingView#localRoot switch the local root} of the
current {@link IFoldingView} view to the given group node.
If the parameter to this command is <code>null</code>, the currently selected group node is entered. If the parameter is an {@link INode}
instance that is a group node, that group node will be entered.
The default shortcut for this command is Ctrl+Enter.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23ENTER_GROUP Online Documentation}*/
static ENTER_GROUP: ICommand;
/**
An {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link IFoldingView#localRoot exit the current local root}
of the current {@link IFoldingView} view to the given parent view.
The default shortcut for this command is Ctrl+Backspace.
@see KeyboardInputMode#addCommandBinding
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23EXIT_GROUP Online Documentation}*/
static EXIT_GROUP: ICommand;
/**
A {@link ICommand} that {@link CreateEdgeInputMode} instances can execute if they have been {@link IInputMode#install installed} in a
{@link CanvasComponent}.
If this command gets executed, the command parameter is inspected to see if it is set to either an
{@link IPortCandidate}, an {@link IPort}, or an {@link INode} that is an element in the currently edited
graph instance. If so, method {@link CreateEdgeInputMode#doStartEdgeCreation} will be invoked and the user can
finish the edge creation gesture.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23BEGIN_EDGE_CREATION Online Documentation}*/
static BEGIN_EDGE_CREATION: ICommand;
/**
An {@link ICommand} that can be {@link ICommand#execute executed} from within the {@link GraphComponent} to {@link GraphClipboard#duplicate duplicate items} of the
current {@link IGraph}.
The default shortcut for this command is Ctrl+D.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23DUPLICATE Online Documentation}*/
static DUPLICATE: ICommand;
/**
An {@link ICommand} that reverses the direction of edges.
{@link GraphEditorInputMode} can execute this command if it is installed in a {@link GraphComponent}. The
command parameter specifies the edges to be reversed and can be an {@link IEdge}, an {@link IEnumerable IEnumerable&lt;IEdge>} or <code>null</code>. If
the parameter is <code>null</code> the currently selected edges will be reversed.
The default shortcut for this command is Ctrl+R.
@see GraphEditorInputMode#reverseEdges
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23REVERSE_EDGE Online Documentation}*/
static REVERSE_EDGE: ICommand;
/**
An {@link ICommand} to print.
This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
sense in the context of the application, e.g. printing the currently visible portion of the graph.
The default shortcut for this command is Ctrl+P.
@see ICommand#PRINT_PREVIEW
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23PRINT Online Documentation}*/
static PRINT: ICommand;
/**
An {@link ICommand} to close a document.
This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
sense in the context of the application, e.g. closing the current window.
There is no default shortcut for this command.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23CLOSE Online Documentation}*/
static CLOSE: ICommand;
/**
An {@link ICommand} to open help.
This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
sense in the context of the application, e.g. showing the online help.
The default shortcut for this command is F1.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23HELP Online Documentation}*/
static HELP: ICommand;
/**
An {@link ICommand} to show properties of an item.
This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
sense in the context of the application.
There is no default shortcut for this command.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23PROPERTIES Online Documentation}*/
static PROPERTIES: ICommand;
/**
An {@link ICommand} to create a new document.
This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
sense in the context of the application, e.g. creating a new graph.
The default shortcut for this command is Ctrl+N.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23NEW Online Documentation}*/
static NEW: ICommand;
/**
An {@link ICommand} to open a document.
{@link GraphComponent} handles this command to open a dialog prompting to open a GraphML file.
The default shortcut for this command is Ctrl+O.
@see ICommand#SAVE
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23OPEN Online Documentation}*/
static OPEN: ICommand;
/**
An {@link ICommand} to select all items.
{@link GraphEditorInputMode} and {@link GraphViewerInputMode} handle this command to select all items in the
graph.
The default shortcut for this command is Ctrl+A.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SELECT_ALL Online Documentation}*/
static SELECT_ALL: ICommand;
/**
An {@link ICommand} to delete an item.
{@link GraphEditorInputMode} handles this command to delete the currently selected items.
The default shortcut for this command is Del.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23DELETE Online Documentation}*/
static DELETE: ICommand;
/**
An {@link ICommand} to save an item.
{@link GraphComponent} handles this command to open a dialog prompting to save the graph as a GraphML file, if the
graph was never saved before, and simply saving to the previous file name if it was.
The default shortcut for this command is Ctrl+S.
@see ICommand#OPEN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SAVE Online Documentation}*/
static SAVE: ICommand;
/**
An {@link ICommand} to cut an item and place it in the clipboard.
{@link GraphEditorInputMode} handles this command in conjunction with {@link GraphClipboard}.
The default shortcuts for this command are Ctrl+X and Shift+Del.
@see ICommand#COPY
@see ICommand#PASTE
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23CUT Online Documentation}*/
static CUT: ICommand;
/**
An {@link ICommand} to copy to the clipboard.
{@link GraphViewerInputMode} and {@link GraphEditorInputMode} handle this command in conjunction with
{@link GraphClipboard}.
The default shortcuts for this command are Ctrl+C and Ctrl+Ins.
@see ICommand#CUT
@see ICommand#PASTE
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23COPY Online Documentation}*/
static COPY: ICommand;
/**
An {@link ICommand} to paste from the clipboard.
{@link GraphEditorInputMode} handles this command in conjunction with {@link GraphClipboard}. The command
parameter can be an {@link IPoint} to specify a location for pasting.
The default shortcuts for this command are Ctrl+V and Shift+Ins.
@see ICommand#CUT
@see ICommand#COPY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23PASTE Online Documentation}*/
static PASTE: ICommand;
/**
An {@link ICommand} to undo the previous operation.
{@link GraphEditorInputMode} handles this command in conjunction with {@link UndoEngine}.
The default shortcut for this command is Ctrl+Z.
@see ICommand#REDO
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23UNDO Online Documentation}*/
static UNDO: ICommand;
/**
An {@link ICommand} to redo a previously undone operation.
{@link GraphEditorInputMode} handles this command in conjunction with {@link UndoEngine}.
The default shortcut for this command is Ctrl+Y.
@see ICommand#UNDO
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23REDO Online Documentation}*/
static REDO: ICommand;
/**
An {@link ICommand} that indicates the intention to print and show a preview of what to print beforehand.
This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
sense in the context of the application.
The default shortcut for this command is Ctrl+F2.
@see ICommand#PRINT
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23PRINT_PREVIEW Online Documentation}*/
static PRINT_PREVIEW: ICommand;
/**
An {@link ICommand} to scroll up.
{@link CanvasComponent} handles this command to move the viewport upwards. By default this is done by a full
viewport height. The parameter can be a factor that is applied to the viewport height to move the viewport by a
different amount.
The default shortcut for this command is Page Up.
@see ICommand#SCROLL_PAGE_DOWN
@see ICommand#SCROLL_PAGE_LEFT
@see ICommand#SCROLL_PAGE_RIGHT
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SCROLL_PAGE_UP Online Documentation}*/
static SCROLL_PAGE_UP: ICommand;
/**
An {@link ICommand} to scroll down.
{@link CanvasComponent} handles this command to move the viewport downwards. By default this is done by a full
viewport height. The parameter can be a factor that is applied to the viewport height to move the viewport by a
different amount.
The default shortcut for this command is Page Down.
@see ICommand#SCROLL_PAGE_UP
@see ICommand#SCROLL_PAGE_LEFT
@see ICommand#SCROLL_PAGE_RIGHT
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SCROLL_PAGE_DOWN Online Documentation}*/
static SCROLL_PAGE_DOWN: ICommand;
/**
An {@link ICommand} to scroll left.
{@link CanvasComponent} handles this command to move the viewport to the left. By default this is done by a full
viewport width. The parameter can be a factor that is applied to the viewport width to move the viewport by a different
amount.
There is no default shortcut for this command.
@see ICommand#SCROLL_PAGE_UP
@see ICommand#SCROLL_PAGE_DOWN
@see ICommand#SCROLL_PAGE_RIGHT
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SCROLL_PAGE_LEFT Online Documentation}*/
static SCROLL_PAGE_LEFT: ICommand;
/**
An {@link ICommand} to scroll right.
{@link CanvasComponent} handles this command to move the viewport to the right. By default this is done by a full
viewport width. The parameter can be a factor that is applied to the viewport width to move the viewport by a different
amount.
There is no default shortcut for this command.
@see ICommand#SCROLL_PAGE_UP
@see ICommand#SCROLL_PAGE_DOWN
@see ICommand#SCROLL_PAGE_LEFT
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SCROLL_PAGE_RIGHT Online Documentation}*/
static SCROLL_PAGE_RIGHT: ICommand;
/**
An {@link ICommand} to move the selection left.
{@link NavigationInputMode} handles this command to select the next item to the left.
The default shortcut for this command is Arrow Left.
@see ICommand#MOVE_UP
@see ICommand#MOVE_DOWN
@see ICommand#MOVE_RIGHT
@see ICommand#MOVE_TO_PAGE_UP
@see ICommand#MOVE_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_LEFT Online Documentation}*/
static MOVE_LEFT: ICommand;
/**
An {@link ICommand} to move the selection right.
{@link NavigationInputMode} handles this command to select the next item to the right.
The default shortcut for this command is Arrow Right.
@see ICommand#MOVE_UP
@see ICommand#MOVE_DOWN
@see ICommand#MOVE_LEFT
@see ICommand#MOVE_TO_PAGE_UP
@see ICommand#MOVE_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_RIGHT Online Documentation}*/
static MOVE_RIGHT: ICommand;
/**
An {@link ICommand} to move the selection up.
{@link NavigationInputMode} handles this command to select the next item above.
The default shortcut for this command is Arrow Up.
@see ICommand#MOVE_DOWN
@see ICommand#MOVE_LEFT
@see ICommand#MOVE_RIGHT
@see ICommand#MOVE_TO_PAGE_UP
@see ICommand#MOVE_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_UP Online Documentation}*/
static MOVE_UP: ICommand;
/**
An {@link ICommand} to move the selection down.
{@link NavigationInputMode} handles this command to select the next item below.
The default shortcut for this command is Arrow Down.
@see ICommand#MOVE_UP
@see ICommand#MOVE_LEFT
@see ICommand#MOVE_RIGHT
@see ICommand#MOVE_TO_PAGE_UP
@see ICommand#MOVE_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_DOWN Online Documentation}*/
static MOVE_DOWN: ICommand;
/**
An {@link ICommand} to move the selection up a page.
{@link NavigationInputMode} handles this command to select the parent node, if possible.
The default shortcut for this command is Page Up.
@see ICommand#MOVE_UP
@see ICommand#MOVE_DOWN
@see ICommand#MOVE_LEFT
@see ICommand#MOVE_RIGHT
@see ICommand#MOVE_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_TO_PAGE_UP Online Documentation}*/
static MOVE_TO_PAGE_UP: ICommand;
/**
An {@link ICommand} to move the selection down a page.
{@link NavigationInputMode} handles this command to select the first child node, if possible.
The default shortcut for this command is Page Down.
@see ICommand#MOVE_UP
@see ICommand#MOVE_DOWN
@see ICommand#MOVE_LEFT
@see ICommand#MOVE_RIGHT
@see ICommand#MOVE_TO_PAGE_UP
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_TO_PAGE_DOWN Online Documentation}*/
static MOVE_TO_PAGE_DOWN: ICommand;
/**
An {@link ICommand} to move the focus backwards.
{@link NavigationInputMode} handles this command to focus the next item to the left.
The default shortcut for this command is Ctrl+Arrow Left.
@see ICommand#MOVE_FOCUS_FORWARD
@see ICommand#MOVE_FOCUS_UP
@see ICommand#MOVE_FOCUS_DOWN
@see ICommand#MOVE_FOCUS_PAGE_UP
@see ICommand#MOVE_FOCUS_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_FOCUS_BACK Online Documentation}*/
static MOVE_FOCUS_BACK: ICommand;
/**
An {@link ICommand} to move the focus forwards.
{@link NavigationInputMode} handles this command to focus the next item to the right.
The default shortcut for this command is Ctrl+Arrow Right.
@see ICommand#MOVE_FOCUS_BACK
@see ICommand#MOVE_FOCUS_UP
@see ICommand#MOVE_FOCUS_DOWN
@see ICommand#MOVE_FOCUS_PAGE_UP
@see ICommand#MOVE_FOCUS_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_FOCUS_FORWARD Online Documentation}*/
static MOVE_FOCUS_FORWARD: ICommand;
/**
An {@link ICommand} to move the focus upwards.
{@link NavigationInputMode} handles this command to focus the next item above.
The default shortcut for this command is Ctrl+Arrow Up.
@see ICommand#MOVE_FOCUS_BACK
@see ICommand#MOVE_FOCUS_FORWARD
@see ICommand#MOVE_FOCUS_DOWN
@see ICommand#MOVE_FOCUS_PAGE_UP
@see ICommand#MOVE_FOCUS_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_FOCUS_UP Online Documentation}*/
static MOVE_FOCUS_UP: ICommand;
/**
An {@link ICommand} to move the focus downwards.
{@link NavigationInputMode} handles this command to focus the next item below.
The default shortcut for this command is Ctrl+Arrow Down.
@see ICommand#MOVE_FOCUS_BACK
@see ICommand#MOVE_FOCUS_FORWARD
@see ICommand#MOVE_FOCUS_UP
@see ICommand#MOVE_FOCUS_PAGE_UP
@see ICommand#MOVE_FOCUS_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_FOCUS_DOWN Online Documentation}*/
static MOVE_FOCUS_DOWN: ICommand;
/**
An {@link ICommand} to move the focus up a page.
{@link NavigationInputMode} handles this command to focus the parent node, if possible.
The default shortcut for this command is Ctrl+Page Up.
@see ICommand#MOVE_FOCUS_BACK
@see ICommand#MOVE_FOCUS_FORWARD
@see ICommand#MOVE_FOCUS_UP
@see ICommand#MOVE_FOCUS_DOWN
@see ICommand#MOVE_FOCUS_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_FOCUS_PAGE_UP Online Documentation}*/
static MOVE_FOCUS_PAGE_UP: ICommand;
/**
An {@link ICommand} to move the focus up a page.
{@link NavigationInputMode} handles this command to focus the first child node, if possible.
The default shortcut for this command is Ctrl+Page Down.
@see ICommand#MOVE_FOCUS_BACK
@see ICommand#MOVE_FOCUS_FORWARD
@see ICommand#MOVE_FOCUS_UP
@see ICommand#MOVE_FOCUS_DOWN
@see ICommand#MOVE_FOCUS_PAGE_UP
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23MOVE_FOCUS_PAGE_DOWN Online Documentation}*/
static MOVE_FOCUS_PAGE_DOWN: ICommand;
/**
An {@link ICommand} to extend the selection to the left.
{@link NavigationInputMode} handles this command to extend the selection with the next item to the left.
The default shortcut for this command is Shift+Arrow Left.
@see ICommand#EXTEND_SELECTION_UP
@see ICommand#EXTEND_SELECTION_DOWN
@see ICommand#EXTEND_SELECTION_RIGHT
@see ICommand#SELECT_TO_PAGE_UP
@see ICommand#SELECT_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23EXTEND_SELECTION_LEFT Online Documentation}*/
static EXTEND_SELECTION_LEFT: ICommand;
/**
An {@link ICommand} to extend the selection to the right.
{@link NavigationInputMode} handles this command to extend the selection with the next item to the right.
The default shortcut for this command is Shift+Arrow Right.
@see ICommand#EXTEND_SELECTION_UP
@see ICommand#EXTEND_SELECTION_DOWN
@see ICommand#EXTEND_SELECTION_LEFT
@see ICommand#SELECT_TO_PAGE_UP
@see ICommand#SELECT_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23EXTEND_SELECTION_RIGHT Online Documentation}*/
static EXTEND_SELECTION_RIGHT: ICommand;
/**
An {@link ICommand} to extend the selection upwards.
{@link NavigationInputMode} handles this command to extend the selection with the next item above.
The default shortcut for this command is Shift+Arrow Up.
@see ICommand#EXTEND_SELECTION_DOWN
@see ICommand#EXTEND_SELECTION_LEFT
@see ICommand#EXTEND_SELECTION_RIGHT
@see ICommand#SELECT_TO_PAGE_UP
@see ICommand#SELECT_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23EXTEND_SELECTION_UP Online Documentation}*/
static EXTEND_SELECTION_UP: ICommand;
/**
An {@link ICommand} to extend the selection downwards.
{@link NavigationInputMode} handles this command to extend the selection with the next item below.
The default shortcut for this command is Shift+Arrow Down.
@see ICommand#EXTEND_SELECTION_UP
@see ICommand#EXTEND_SELECTION_LEFT
@see ICommand#EXTEND_SELECTION_RIGHT
@see ICommand#SELECT_TO_PAGE_UP
@see ICommand#SELECT_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23EXTEND_SELECTION_DOWN Online Documentation}*/
static EXTEND_SELECTION_DOWN: ICommand;
/**
An {@link ICommand} to extend the selection a page upwards.
{@link NavigationInputMode} handles this command to extend the selection with the parent node, if possible.
The default shortcut for this command is Shift+Page Up.
@see ICommand#EXTEND_SELECTION_UP
@see ICommand#EXTEND_SELECTION_DOWN
@see ICommand#EXTEND_SELECTION_LEFT
@see ICommand#EXTEND_SELECTION_RIGHT
@see ICommand#SELECT_TO_PAGE_DOWN
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SELECT_TO_PAGE_UP Online Documentation}*/
static SELECT_TO_PAGE_UP: ICommand;
/**
An {@link ICommand} to extend the selection a page downwards.
{@link NavigationInputMode} handles this command to extend the selection with the first child node, if possible.
The default shortcut for this command is Shift+Page Down.
@see ICommand#EXTEND_SELECTION_UP
@see ICommand#EXTEND_SELECTION_DOWN
@see ICommand#EXTEND_SELECTION_LEFT
@see ICommand#EXTEND_SELECTION_RIGHT
@see ICommand#SELECT_TO_PAGE_UP
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SELECT_TO_PAGE_DOWN Online Documentation}*/
static SELECT_TO_PAGE_DOWN: ICommand;
/**
An {@link ICommand} to increase the zoom level.
{@link CanvasComponent} handles this command to increase the zoom level by 20%. A different factor from the default
<code>1.2</code> may be given as the command parameter.
There is no default shortcut for this command.
@see ICommand#DECREASE_ZOOM
@see ICommand#ZOOM
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23INCREASE_ZOOM Online Documentation}*/
static INCREASE_ZOOM: ICommand;
/**
An {@link ICommand} to decrease the zoom level.
{@link CanvasComponent} handles this command to decrease the zoom level by 20%. A different factor from the default
<code>1.2</code> may be given as the command parameter.
There is no default shortcut for this command.
@see ICommand#INCREASE_ZOOM
@see ICommand#ZOOM
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23DECREASE_ZOOM Online Documentation}*/
static DECREASE_ZOOM: ICommand;
/**
An {@link ICommand} to set the zoom level.
{@link CanvasComponent} handles this command to change the viewport. If the parameter is a {@link Point},
the viewport will be centered as that point while not changing the zoom level. If the parameter is a
{@link Rect}, the viewport will encompass that rectangle and change the zoom level accordingly. If the parameter
is an instance of {@link ILookup} which returns an instance of {@link IBoundsProvider}, that {@link IBoundsProvider}
will be used to obtain a rectangle, which will be used in the same manner as the {@link Rect} mentioned before.
This can be used to zoom to a specific
{@link IModelItem}. Finally, if the parameter is a number, it will be used as the new zoom level to set.
There is no default shortcut for this command.
@see ICommand#INCREASE_ZOOM
@see ICommand#DECREASE_ZOOM
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23ZOOM Online Documentation}*/
static ZOOM: ICommand;
/**
An {@link ICommand} that invokes {@link GraphComponent#currentItem} if executed.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23SET_CURRENT_ITEM Online Documentation}*/
static SET_CURRENT_ITEM: ICommand;
/**
The {@link ICommand} that performs the FitGraphBounds(InsetsD) action.
The parameter can be either an {@link Insets} or convertible that will be used to create an appropriately sized
insets object from.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23FIT_GRAPH_BOUNDS Online Documentation}*/
static FIT_GRAPH_BOUNDS: ICommand;
/**
An {@link ICommand} that changes the {@link CanvasComponent#zoom} and the {@link CanvasComponent#viewPoint} in an animated fashion such that the {@link GraphComponent#currentItem} becomes fully visible.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23ZOOM_TO_CURRENT_ITEM Online Documentation}*/
static ZOOM_TO_CURRENT_ITEM: ICommand;
/**
An {@link ICommand} that invokes {@link CanvasComponent#fitContent} if executed.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23FIT_CONTENT Online Documentation}*/
static FIT_CONTENT: ICommand;
/**
An {@link ICommand} that invokes {@link CanvasComponent#updateContentRect} if executed.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23UPDATE_CONTENT_RECT Online Documentation}*/
static UPDATE_CONTENT_RECT: ICommand;
/**
A {@link ICommand} that raises the visual representations of {@link IModelItem}s one step to the front.
Raises the {@link GraphModelManager#getMainCanvasObject main canvas objects} of {@link IModelItem}s in their {@link ICanvasObject#group parent group} one step to the front.
{@link GraphEditorInputMode} can execute this command if it is installed in a
{@link GraphComponent}. As command parameter the following values are accepted:
<ul>
<li><code>null</code>, in which case the {@link GraphComponent#selection} is raised or the {@link GraphComponent#currentItem} if {@link GraphEditorInputMode#useCurrentItemForCommands} is set.
<li>a single {@link IModelItem} that shall be raised.
<li>an {@link IEnumerable} which shall be raised.
</ul>
@see GraphModelManager#raise
@see GraphEditorInputMode#raiseSelection
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23RAISE Online Documentation}*/
static RAISE: ICommand;
/**
A {@link ICommand} that lowers the visual representations of {@link IModelItem}s one step to back.
Lowers the {@link GraphModelManager#getMainCanvasObject main canvas objects} of {@link IModelItem}s in their {@link ICanvasObject#group parent group} one step to back.
{@link GraphEditorInputMode} can execute this command if it is installed in a {@link GraphComponent} and. As
command parameter the following values are accepted:
<ul>
<li><code>null</code>, in which case the {@link GraphComponent#selection} is lowered or the {@link GraphComponent#currentItem} if {@link GraphEditorInputMode#useCurrentItemForCommands} is set.
<li>a single {@link IModelItem} that shall be lowered.
<li>an {@link IEnumerable} which shall be lowered.
</ul>
@see GraphModelManager#lower
@see GraphEditorInputMode#lowerSelection
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23LOWER Online Documentation}*/
static LOWER: ICommand;
/**
A {@link ICommand} that brings the visual representations of {@link IModelItem}s to the front.
Brings the {@link GraphModelManager#getMainCanvasObject main canvas objects} of {@link IModelItem}s in their {@link ICanvasObject#group parent group} to the front.
{@link GraphEditorInputMode} can execute this command if it is installed in a
{@link GraphComponent}. As command parameter the following values are accepted:
<ul>
<li><code>null</code>, in which case the {@link GraphComponent#selection} is brought to the front or the {@link GraphComponent#currentItem} if {@link GraphEditorInputMode#useCurrentItemForCommands} is set.
<li>a single {@link IModelItem} that shall be brought to the front.
<li>an {@link IEnumerable} which shall be brought to the front.
</ul>
@see GraphModelManager#toFront
@see GraphEditorInputMode#selectionToFront
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23TO_FRONT Online Documentation}*/
static TO_FRONT: ICommand;
/**
A {@link ICommand} that brings the visual representations of {@link IModelItem}s to back.
Brings the {@link GraphModelManager#getMainCanvasObject main canvas objects} of {@link IModelItem}s in their {@link ICanvasObject#group parent group} to back.
{@link GraphEditorInputMode} can execute this command if it is installed in a
{@link GraphComponent}. As command parameter the following values are accepted:
<ul>
<li><code>null</code>, in which case the {@link GraphComponent#selection} is brought to back or the {@link GraphComponent#currentItem} if {@link GraphEditorInputMode#useCurrentItemForCommands} is set.
<li>a single {@link IModelItem} that shall be brought to back.
<li>an {@link IEnumerable} which shall be brought to back.
</ul>
@see GraphModelManager#toBack
@see GraphEditorInputMode#selectionToBack
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICommand%23TO_BACK Online Documentation}*/
static TO_BACK: ICommand;
static $class: Class;
private $_ICommand: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ICommand;
}
export interface ISvgDefsCreator {
}
/**
Classes that implement this interface can be placed in the SVGs defs section by the {@link SvgDefsManager}.
The {@link SvgDefsManager} can then create and update an element that can be used with an SVG use element. The
interface is also used to help the {@link SvgDefsManager} to find unused elements that can be removed from the defs
section.
When an element is created using this interface, the {@link SvgDefsManager} assigns an ID to it that is required to
reference it through an SVG use element. The ID that was assigned to it can be queried using the {@link ICanvasContext#getDefsId}
method when the {@link ISvgDefsCreator} object is passed in that was used to create the element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISvgDefsCreator Online Documentation}*/
export abstract class ISvgDefsCreator {
/**
Checks if the specified node references the element represented by this object.
@param context The current canvas context.
@param node The SVG node.
@param id The defs id that has been stored for this instance by the provided context.
@returns <code>true</code> if <code>node</code> references the element represented by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISvgDefsCreator%23accept(ICanvasContext%2CNode%2Cstring) Online Documentation}*/
abstract accept(context: ICanvasContext, node: Node, id: string): boolean;
/**
Creates an implementation of the interface {@link ISvgDefsCreator} from the given definition.
The given object must contain exactly the abstract members of the {@link ISvgDefsCreator} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link ISvgDefsCreator} interface.
@returns An instance of the {@link ISvgDefsCreator} interface based on the given definition.*/
static create(interfaceDefinition: {
updateDefsElement(context: ICanvasContext, oldElement: SVGElement): void;
createDefsElement(context: ICanvasContext): SVGElement;
accept(context: ICanvasContext, node: Node, id: string): boolean;
}): ISvgDefsCreator;
/**
Creates the element that is put into the defs section of the SVG element.
@param context The canvas context that can be used to register brushes etc.
@returns An {@link SVGElement} that can be put into the defs section of the SVG element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISvgDefsCreator%23createDefsElement(ICanvasContext) Online Documentation}*/
abstract createDefsElement(context: ICanvasContext): SVGElement;
/**
Determine whether the given element has an attribute with the given name that references the id in the defs element.
@param element The element to inspect
@param attributeName The attribute to check
@param defsId The id in the defs section
@returns Whether the attribute is equal to "url(#{defsId})"
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISvgDefsCreator%23isAttributeReference(Element%2Cstring%2Cstring) Online Documentation}*/
static isAttributeReference(element: Element, attributeName: string, defsId: string): boolean;
/**
Checks whether the given DOM <code>node</code> is an SVG <code>use</code> element that references an element with the specified <code>id</code> in the local
DOM.
Checks if <code>node</code> matches &lt;use href="#{id}">. If the given <code>id</code> does not start with a '#', it is prepended automatically.
@param node The DOM node to inspect.
@param id The id of the referenced element.
@returns <code>true</code> if the specified element references the element; otherwise, <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISvgDefsCreator%23isUseReference(Node%2Cstring) Online Documentation}*/
static isUseReference(node: Node | null, id: string): boolean;
/**
Updates the defs element with the current gradient data.
@param context
@param oldElement
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISvgDefsCreator%23updateDefsElement(ICanvasContext%2CSVGElement) Online Documentation}*/
abstract updateDefsElement(context: ICanvasContext, oldElement: SVGElement): void;
static $class: Class;
private $_ISvgDefsCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ISvgDefsCreator;
}
export interface IValueSerializerContext extends ILookup {
}
/**
Support interface used by {@link ValueSerializer}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IValueSerializerContext Online Documentation}*/
export abstract class IValueSerializerContext {
/**
Gets the value serializer for a specific {@link Property}.
@param property The descriptor of the property.
@returns A serializer or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IValueSerializerContext%23getValueSerializerFor(Property) Online Documentation}*/
abstract getValueSerializerFor(property: Property): ValueSerializer | null;
/**
Gets the value serializer for a specific type.
@param type The type to obtain a serializer for.
@returns A serializer or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IValueSerializerContext%23getValueSerializerFor(Class) Online Documentation}*/
abstract getValueSerializerFor(type: Class): ValueSerializer | null;
static $class: Class;
private $_IValueSerializerContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IValueSerializerContext;
}
/**
This enumeration contains the valid values for the <code>image-rendering</code> property of SVG image elements.
This property provides is a recommendation to the SVG rendering engine about how to optimize the shape rendering. See
the SVG specification for details.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageRenderingType Online Documentation}*/
export enum ImageRenderingType {
/**
Indicates that the rendering engine should use a balanced setup with a slight preference of quality.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageRenderingType%23AUTO Online Documentation}*/
AUTO,
/**
Indicates that the rendering speed should be optimized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageRenderingType%23OPTIMIZE_SPEED Online Documentation}*/
OPTIMIZE_SPEED,
/**
Indicates that the rendering quality should be optimized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageRenderingType%23OPTIMIZE_QUALITY Online Documentation}*/
OPTIMIZE_QUALITY,
/**
Indicates that the setting for the rendering quality should be inherited from the parent.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageRenderingType%23INHERIT Online Documentation}*/
INHERIT
}
/** Automatic conversion to {@link ImageRenderingType}. */
type ImageRenderingTypeStringValues = "auto" | "optimize-speed" | "optimize-quality" | "inherit";
/**
Possible values of the key.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key Online Documentation}*/
export enum Key {
/**
The Escape key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23ESCAPE Online Documentation}*/
ESCAPE,
/**
The Delete key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23DELETE Online Documentation}*/
DELETE,
/**
The Shift key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23SHIFT Online Documentation}*/
SHIFT,
/**
The Ctrl key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23CTRL Online Documentation}*/
CTRL,
/**
The Enter key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23ENTER Online Documentation}*/
ENTER,
/**
The Space key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23SPACE Online Documentation}*/
SPACE,
/**
The None key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23NONE Online Documentation}*/
NONE,
/**
The Alt key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23ALT Online Documentation}*/
ALT,
/**
The PageUp key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23PAGE_UP Online Documentation}*/
PAGE_UP,
/**
The PageDown key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23PAGE_DOWN Online Documentation}*/
PAGE_DOWN,
/**
The Insert key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23INSERT Online Documentation}*/
INSERT,
/**
The Back key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23BACK Online Documentation}*/
BACK,
/**
The Add key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23ADD Online Documentation}*/
ADD,
/**
The Subtract key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23SUBTRACT Online Documentation}*/
SUBTRACT,
/**
The F1 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F1 Online Documentation}*/
F1,
/**
The F2 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F2 Online Documentation}*/
F2,
/**
The F3 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F3 Online Documentation}*/
F3,
/**
The F4 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F4 Online Documentation}*/
F4,
/**
The F5 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F5 Online Documentation}*/
F5,
/**
The F6 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F6 Online Documentation}*/
F6,
/**
The F7 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F7 Online Documentation}*/
F7,
/**
The F8 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F8 Online Documentation}*/
F8,
/**
The F9 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F9 Online Documentation}*/
F9,
/**
The F10 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F10 Online Documentation}*/
F10,
/**
The F11 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F11 Online Documentation}*/
F11,
/**
The F12 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F12 Online Documentation}*/
F12,
/**
The ArrowLeft key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23ARROW_LEFT Online Documentation}*/
ARROW_LEFT,
/**
The ArrowUp key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23ARROW_UP Online Documentation}*/
ARROW_UP,
/**
The ArrowRight key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23ARROW_RIGHT Online Documentation}*/
ARROW_RIGHT,
/**
The ArrowDown key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23ARROW_DOWN Online Documentation}*/
ARROW_DOWN,
/**
The 0 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D0 Online Documentation}*/
D0,
/**
The 1 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D1 Online Documentation}*/
D1,
/**
The 2 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D2 Online Documentation}*/
D2,
/**
The 3 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D3 Online Documentation}*/
D3,
/**
The 4 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D4 Online Documentation}*/
D4,
/**
The 5 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D5 Online Documentation}*/
D5,
/**
The 6 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D6 Online Documentation}*/
D6,
/**
The 7 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D7 Online Documentation}*/
D7,
/**
The 8 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D8 Online Documentation}*/
D8,
/**
The 9 key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D9 Online Documentation}*/
D9,
/**
The A key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23A Online Documentation}*/
A,
/**
The B key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23B Online Documentation}*/
B,
/**
The C key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23C Online Documentation}*/
C,
/**
The D key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23D Online Documentation}*/
D,
/**
The E key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23E Online Documentation}*/
E,
/**
The F key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23F Online Documentation}*/
F,
/**
The G key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23G Online Documentation}*/
G,
/**
The H key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23H Online Documentation}*/
H,
/**
The I key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23I Online Documentation}*/
I,
/**
The J key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23J Online Documentation}*/
J,
/**
The K key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23K Online Documentation}*/
K,
/**
The L key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23L Online Documentation}*/
L,
/**
The M key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23M Online Documentation}*/
M,
/**
The N key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N Online Documentation}*/
N,
/**
The O key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23O Online Documentation}*/
O,
/**
The P key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23P Online Documentation}*/
P,
/**
The Q key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23Q Online Documentation}*/
Q,
/**
The R key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23R Online Documentation}*/
R,
/**
The S key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23S Online Documentation}*/
S,
/**
The T key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23T Online Documentation}*/
T,
/**
The U key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23U Online Documentation}*/
U,
/**
The V key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23V Online Documentation}*/
V,
/**
The W key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23W Online Documentation}*/
W,
/**
The X key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23X Online Documentation}*/
X,
/**
The Y key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23Y Online Documentation}*/
Y,
/**
The Z key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23Z Online Documentation}*/
Z,
/**
The Tab key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23TAB Online Documentation}*/
TAB,
/**
The Pause key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23PAUSE Online Documentation}*/
PAUSE,
/**
The Home key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23HOME Online Documentation}*/
HOME,
/**
The End key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23END Online Documentation}*/
END,
/**
The 0 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N0 Online Documentation}*/
N0,
/**
The 1 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N1 Online Documentation}*/
N1,
/**
The 2 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N2 Online Documentation}*/
N2,
/**
The 3 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N3 Online Documentation}*/
N3,
/**
The 4 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N4 Online Documentation}*/
N4,
/**
The 5 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N5 Online Documentation}*/
N5,
/**
The 6 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N6 Online Documentation}*/
N6,
/**
The 7 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N7 Online Documentation}*/
N7,
/**
The 8 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N8 Online Documentation}*/
N8,
/**
The 9 key of the numpad.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23N9 Online Documentation}*/
N9,
/**
The Multiply key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23MULTIPLY Online Documentation}*/
MULTIPLY,
/**
The Caps key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23CAPS Online Documentation}*/
CAPS,
/**
The Decimal key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23DECIMAL Online Documentation}*/
DECIMAL,
/**
The Divide key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23DIVIDE Online Documentation}*/
DIVIDE,
/**
The Semicolon key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23SEMICOLON Online Documentation}*/
SEMICOLON,
/**
The Equal key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23EQUAL Online Documentation}*/
EQUAL,
/**
The Comma key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23COMMA Online Documentation}*/
COMMA,
/**
The Dash key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23DASH Online Documentation}*/
DASH,
/**
The Period key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23PERIOD Online Documentation}*/
PERIOD,
/**
The Accent key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23ACCENT Online Documentation}*/
ACCENT,
/**
The OpenBracket key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23OPEN_BRACKET Online Documentation}*/
OPEN_BRACKET,
/**
The BackSlash key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23BACK_SLASH Online Documentation}*/
BACK_SLASH,
/**
The CloseBracket key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23CLOSE_BRACKET Online Documentation}*/
CLOSE_BRACKET,
/**
The SingleQuote key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Key%23SINGLE_QUOTE Online Documentation}*/
SINGLE_QUOTE
}
/** Automatic conversion to {@link Key}. */
type KeyStringValues = "escape" | "delete" | "shift" | "ctrl" | "enter" | "space" | "none" | "alt" | "page-up" | "page-down" | "insert" | "back" | "add" | "subtract" | "f1" | "f2" | "f3" | "f4" | "f5" | "f6" | "f7" | "f8" | "f9" | "f10" | "f11" | "f12" | "arrow-left" | "arrow-up" | "arrow-right" | "arrow-down" | "d0" | "d1" | "d2" | "d3" | "d4" | "d5" | "d6" | "d7" | "d8" | "d9" | "a" | "b" | "c" | "d" | "e" | "f" | "g" | "h" | "i" | "j" | "k" | "l" | "m" | "n" | "o" | "p" | "q" | "r" | "s" | "t" | "u" | "v" | "w" | "x" | "y" | "z" | "tab" | "pause" | "home" | "end" | "n0" | "n1" | "n2" | "n3" | "n4" | "n5" | "n6" | "n7" | "n8" | "n9" | "multiply" | "caps" | "decimal" | "divide" | "semicolon" | "equal" | "comma" | "dash" | "period" | "accent" | "open-bracket" | "back-slash" | "close-bracket" | "single-quote";
/**
The type of a {@link KeyEventArgs}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventType Online Documentation}*/
export enum KeyEventType {
/**
Neither Pressed, nor Typed, nor Released.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventType%23NONE Online Documentation}*/
NONE,
/**
The <code>keydown</code> event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventType%23DOWN Online Documentation}*/
DOWN,
/**
The <code>keypress</code> event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventType%23PRESS Online Documentation}*/
PRESS,
/**
The <code>keyup</code> event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventType%23UP Online Documentation}*/
UP
}
/** Automatic conversion to {@link KeyEventType}. */
type KeyEventTypeStringValues = "none" | "down" | "press" | "up";
export interface KeyEventArgs extends EventArgs {
}
/**
A wrapper for the native keyboard event which abstracts away incompatibilities in browser implementations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs Online Documentation}*/
export class KeyEventArgs {
/**
Creates a new instance of this class from a native keyboard event.
@param evt The native keyboard event.
@param isPrintable if set to <code>true</code> [is printable].
@param type The type of the event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs Online Documentation}*/
constructor(evt: KeyboardEvent, isPrintable: boolean, type: KeyEventType | KeyEventTypeStringValues);
/**
Sets this event's {@link KeyEventArgs#defaultPrevented} property to <code>true</code> signalling that the event has been properly handled and any default behaviors should
be prevented.
This will call the corresponding method on the {@link KeyEventArgs#originalEvent original native event}.
@see KeyEventArgs#defaultPrevented
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23preventDefault() Online Documentation}*/
preventDefault(): void;
/**
Gets the original DOM event associated with this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23originalEvent Online Documentation}*/
originalEvent: Event;
/**
Gets the original source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23originalSource Online Documentation}*/
originalSource: Node | null;
/**
Gets the type of the event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23eventType Online Documentation}*/
eventType: KeyEventType;
/**
Gets the key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23key Online Documentation}*/
key: Key;
/**
Gets the charcode of the original event, if the original event was a key press.
Otherwise returns 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23charCode Online Documentation}*/
charCode: number;
/**
Gets the modifiers.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23modifiers Online Documentation}*/
modifiers: ModifierKeys;
/**
Gets a value indicating whether the alt modifier is pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23alt Online Documentation}*/
alt: boolean;
/**
Gets a value indicating whether the shift modifier is pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23shift Online Documentation}*/
shift: boolean;
/**
Gets a value indicating whether the control modifier is pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23control Online Documentation}*/
control: boolean;
/**
Gets a value indicating whether the meta modifier is pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23meta Online Documentation}*/
meta: boolean;
/**
Gets a value indicating whether the default behaviors for this event have been prevented by a previous call to
{@link KeyEventArgs#preventDefault}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventArgs%23defaultPrevented Online Documentation}*/
readonly defaultPrevented: boolean;
static $class: Class;
private $_KeyEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is KeyEventArgs;
}
export interface LinearGradient extends Fill, ISvgDefsCreator {
/**
@inheritdoc*/
accept(context: ICanvasContext, item: Node, id: string): boolean;
} /**
Fills an area with a linear gradient.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient Online Documentation}*/
export class LinearGradient {
/**
Initializes a new instance of the {@link LinearGradient} class.
@param [options] The parameters to pass.
@param options.gradientStops An array of {@link GradientStop}s. Each item will be added to the {@link LinearGradient#gradientStops} collection.
@param [options.startPoint] The start point of this gradient. This option sets the {@link LinearGradient#startPoint} property on the created object.
@param [options.endPoint] The end point of this gradient. This option sets the {@link LinearGradient#endPoint} property on the created object.
@param [options.spreadMethod] The spread method of this gradient. This option sets the {@link LinearGradient#spreadMethod} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23LinearGradient() Online Documentation}*/
constructor(options?: {
startPoint?: Point | PointConvertible;
endPoint?: Point | PointConvertible;
spreadMethod?: GradientSpreadMethod | GradientSpreadMethodStringValues;
gradientStops?: Array<any>;
});
/**
Creates a new SVG gradient that corresponds to this gradient brush.
@param context
@returns A new SVG gradient that corresponds to this gradient brush
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23createDefsElement(ICanvasContext) Online Documentation}*/
createDefsElement(context: ICanvasContext): SVGElement;
/**
Determines whether this {@link LinearGradient} is equal to the specified object.
@param fill The other {@link Fill} instance to compare.
@returns <code>true</code>, if this {@link LinearGradient} is equal to the other object; <code>false</code>, otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23hasSameValue(Fill) Online Documentation}*/
hasSameValue(fill: Fill | null | FillConvertible): boolean;
/**
Creates a new SVG gradient that corresponds to this gradient brush.
@returns A new SVG gradient that corresponds to this gradient brush
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23toSvgGradient() Online Documentation}*/
toSvgGradient(): SVGElement;
/**
Updates the SVG gradient within the <code>defs</code> section of the SVG document if it has changed.
@param context The context.
@param oldElement The old element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23updateDefsElement(ICanvasContext%2CSVGElement) Online Documentation}*/
updateDefsElement(context: ICanvasContext, oldElement: SVGElement): void;
/**
Updates the SVG gradient within the <code>defs</code> section of the SVG document if it has changed.
@param oldElement The old element.
@param context The context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23updateSvgGradient(SVGElement%2CICanvasContext) Online Documentation}*/
updateSvgGradient(oldElement: SVGElement, context: ICanvasContext): void;
/**
Gets or sets the spread method of this gradient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23spreadMethod Online Documentation}*/
spreadMethod: GradientSpreadMethod;
/**
Gets or sets the start point of this gradient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23startPoint Online Documentation}*/
startPoint: Point;
/**
Gets or sets the end point of this gradient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23endPoint Online Documentation}*/
endPoint: Point;
/**
Gets or sets the gradient stops of this brush.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinearGradient%23gradientStops Online Documentation}*/
gradientStops: List<GradientStop>;
static $class: Class;
private $_LinearGradient: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LinearGradient;
}
export interface MarkupExtension extends YObject {
}
/**
Provides a base class for all XAML markup extensions.
Markup extensions return objects based on string attribute values or markup elements in XAML and/or can be used to
serialize classes that do not fulfill XAML serialization constraints.
<b>Warning:</b> It is essential for custom class serialization to register both regular classes and {@link MarkupExtension}
classes via AddXamlNamespaceMapping(string,string,Type).
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarkupExtension Online Documentation}*/
export abstract class MarkupExtension {
/**
Returns an object that is set as the value of the target property for this markup extension.
@param serviceProvider An object that can provide services for the markup extension.
@returns The object value to set on the property where the extension is applied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarkupExtension%23provideValue(ILookup) Online Documentation}*/
abstract provideValue(serviceProvider: ILookup | null): any | null;
static $class: Class;
private $_MarkupExtension: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MarkupExtension;
}
/**
Flag value that indicates which modifier keys were pressed.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModifierKeys Online Documentation}*/
export enum ModifierKeys {
/**
Indicates that no modifier key was pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModifierKeys%23NONE Online Documentation}*/
NONE,
/**
Indicates that the control key was pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModifierKeys%23CONTROL Online Documentation}*/
CONTROL,
/**
Indicates that the shift key was pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModifierKeys%23SHIFT Online Documentation}*/
SHIFT,
/**
Indicates that the alt key was pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModifierKeys%23ALT Online Documentation}*/
ALT,
/**
Indicates that the windows (or meta) key was pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ModifierKeys%23WINDOWS Online Documentation}*/
WINDOWS
}
type ModifierKeysStringValues = "none" | "control" | "shift" | "alt" | "windows";
/** Automatic conversion to {@link ModifierKeys}. */
type ModifierKeysConvertible = ModifierKeysStringValues | ModifierKeysStringValues[];
export interface PatternFill extends Fill, ISvgDefsCreator {
/**
@inheritdoc*/
accept(context: ICanvasContext, node: Node, id: string): boolean;
/**
@inheritdoc*/
createDefsElement(context: ICanvasContext): SVGElement;
/**
@inheritdoc*/
updateDefsElement(context: ICanvasContext, oldElement: SVGElement): void;
} /**
Fills an area with a SVG pattern.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PatternFill Online Documentation}*/
export class PatternFill {
/**
Initializes a new instance of the {@link PatternFill} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PatternFill Online Documentation}*/
constructor();
/**
Determines whether this {@link PatternFill} is equal to the specified object.
@param fill The other {@link Fill} instance to compare.
@returns <code>true</code>, if this {@link SolidColorFill} is equal to the other object; <code>false</code>, otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PatternFill%23hasSameValue(Fill) Online Documentation}*/
hasSameValue(fill: Fill | null | FillConvertible): boolean;
/**
Gets or sets the content.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PatternFill%23content Online Documentation}*/
content: SvgVisual;
/**
Gets or sets the SVG view box of this pattern brush.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PatternFill%23viewBox Online Documentation}*/
viewBox: Rect;
/**
Gets or sets the origin of this pattern brush.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PatternFill%23origin Online Documentation}*/
origin: Point;
/**
Gets or sets the size of this pattern brush.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PatternFill%23size Online Documentation}*/
size: Size;
static $class: Class;
private $_PatternFill: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PatternFill;
}
export interface Stroke extends YObject, ICloneable {
}
/**
* A utility class that holds all properties of a {@link Stroke}.
* The stroke that is defined by instances of this type can be applied to an SVG element using either the static helper {@link Stroke#setStroke}
* or the instance method {@link Stroke#applyTo}.
*
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* The strings applying to this scheme are converted to {@link Stroke}s:
*
* <pre>
* '[thickness] [type] color'
*
* thickness = thickness in px|thin|medium|thick
* type = solid|dashed|dotted
* color = CSS color strings
* </pre>
*
* Valid color strings are the same as for {@link Color}. Square brackets mark optional declarations.
*
* Examples:
*
* <pre>
* '3px dashed blue'
* 'thick blue'
* 'dotted blue'
* </pre>
*
* @see Stroke#setStroke
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke Online Documentation}
*/
export class Stroke {
/**
Initializes a new instance of the {@link Stroke} class using the given rgba values and thickness.
The rgba values must be between 0 and 255.
@param r The red value.
@param g The green value.
@param b The blue value.
@param [a=255] The alpha value, 255 by default.
@param [thickness=1.0] The thickness, 1 by default.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23Stroke(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(r: number, g: number, b: number, a?: number, thickness?: number);
/**
Initializes a new instance of the {@link Stroke} class using the given rgba values and thickness.
The rgba values must be between 0 and 255.
@param options The parameters to pass.
@param options.r The red value.
@param options.g The green value.
@param options.b The blue value.
@param [options.a=255] The alpha value, 255 by default.
@param [options.thickness=1.0] The thickness, 1 by default.
@param [options.fill] The {@link Stroke#fill} for the {@link Stroke}. This option sets the {@link Stroke#fill} property on the created object.
@param [options.lineCap] The dash cap. This option sets the {@link Stroke#lineCap} property on the created object.
@param [options.lineJoin] The line join property. This option sets the {@link Stroke#lineJoin} property on the created object.
@param [options.dashStyle] The dash style for the {@link Stroke}. This option sets the {@link Stroke#dashStyle} property on the created object.
@param [options.miterLimit] The miter limit. This option sets the {@link Stroke#miterLimit} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23Stroke(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(options: {
r: number;
g: number;
b: number;
a?: number;
thickness?: number;
fill?: Fill | null | FillConvertible;
lineCap?: LineCap | LineCapStringValues;
lineJoin?: LineJoin | LineJoinStringValues;
dashStyle?: DashStyle | DashStyleConvertible;
miterLimit?: number;
});
/**
Initializes a new instance of the {@link Stroke} class using the given {@link Stroke#fill} and thickness.
@param [fill=null] The {@link Stroke#fill}.
@param [thickness=1.0] The thickness.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23Stroke(Fill%2Cnumber) Online Documentation}*/
constructor(fill?: Fill | null | FillConvertible, thickness?: number);
/**
Initializes a new instance of the {@link Stroke} class using the given {@link Stroke#fill} and thickness.
@param options The parameters to pass.
@param [options.fill=null] The {@link Stroke#fill}.
@param [options.thickness=1.0] The thickness.
@param [options.lineCap] The dash cap. This option sets the {@link Stroke#lineCap} property on the created object.
@param [options.lineJoin] The line join property. This option sets the {@link Stroke#lineJoin} property on the created object.
@param [options.dashStyle] The dash style for the {@link Stroke}. This option sets the {@link Stroke#dashStyle} property on the created object.
@param [options.miterLimit] The miter limit. This option sets the {@link Stroke#miterLimit} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23Stroke(Fill%2Cnumber) Online Documentation}*/
constructor(options: {
fill?: Fill | null | FillConvertible;
thickness?: number;
lineCap?: LineCap | LineCapStringValues;
lineJoin?: LineJoin | LineJoinStringValues;
dashStyle?: DashStyle | DashStyleConvertible;
miterLimit?: number;
});
/**
Assigns the stroke defined by this instance to the given SVG <code>element</code>.
This method is very similar in functionality to its static method counter part
{@link Stroke#setStroke}. However since this method is an instance method, it cannot be called on non-existing/<code>null</code>
strokes. If you are unsure whether the reference actually points to an instance, use the static helper method instead.
@param element The element whose fill should be set.
@param context The context.
@see Stroke#setStroke
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23applyTo(SVGElement%2CICanvasContext) Online Documentation}*/
applyTo(element: SVGElement, context: ICanvasContext): void;
/**
Clones this instance by returning a {@link YObject#memberwiseClone} or this in case this instance is already {@link Stroke#isFrozen frozen}.
@returns An instance of the same type as this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23clone() Online Documentation}*/
clone(): this;
/**
Clones the current value of this instance to a new unfrozen {@link Stroke}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23cloneCurrentValue() Online Documentation}*/
cloneCurrentValue(): Stroke;
/**
Freezes this instance.
Freezing an object makes it immutable, which can result in performance enhancements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23freeze() Online Documentation}*/
freeze(): void;
/**
Creates a {@link Stroke} instance from the given stroke-like object by performing automatic type conversion.
@param strokeLike The object to convert to a {@link Stroke}.
@returns Either <code>strokeLike</code> if it is already a {@link Stroke} or a new instance, initialized to the values found in
<code>strokeLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23from(Stroke) Online Documentation}*/
static from(strokeLike: Stroke | StrokeConvertible): Stroke;
/**
Determines if this instance is frozen.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23isFrozen() Online Documentation}*/
isFrozen(): boolean;
/**
Sets the {@link Stroke} on a given SVG element as the stroke.
This method is very similar in functionality to its instance method counter part {@link Stroke#applyTo}. However
being static, it also allows for the case where the <code>stroke</code> is <code>null</code>.
@param stroke The stroke that should be applied. If <code>null</code>, the element's stroke is removed.
@param context The context to use for managing possible <code>defs</code> entries.
@param element The element whose stroke should be set.
@see Stroke#applyTo
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23setStroke(Stroke%2CSVGElement%2CICanvasContext) Online Documentation}*/
static setStroke(stroke: Stroke | null | StrokeConvertible, element: SVGElement, context: ICanvasContext): void;
/**
Gets or sets the {@link Stroke#fill} for the {@link Stroke}.
@default 'Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23fill Online Documentation}*/
fill: Fill | null;
/**
Gets or sets the dash cap.
@default LineCap.FLAT
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23lineCap Online Documentation}*/
lineCap: LineCap;
/**
Gets or sets the line join property.
@default LineJoin.MITER
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23lineJoin Online Documentation}*/
lineJoin: LineJoin;
/**
Gets or sets the dash style for the {@link Stroke}.
The visual appearance of the 'dashes' and 'gaps' specified by this property depends on the value of the property {@link Stroke#lineCap}, too.
The predefined dash styles are designed for dash caps that extend into the 'gaps' like {@link LineCap#SQUARE} and
{@link LineCap#ROUND}. With {@link LineCap#FLAT}, the dashes might not look like expected, for example {@link DashStyle#DOT} will not
be visible at all. Since {@link LineCap#FLAT} is the default value, you should explicitly set another value in this
case.
@default 'Solid'
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23dashStyle Online Documentation}*/
dashStyle: DashStyle;
/**
Gets or sets the miter limit.
@default 10.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23miterLimit Online Documentation}*/
miterLimit: number;
/**
Gets or sets the thickness of the stroke.
@default 1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23thickness Online Documentation}*/
thickness: number;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#ALICE_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23ALICE_BLUE Online Documentation}*/
static readonly ALICE_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#ANTIQUE_WHITE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23ANTIQUE_WHITE Online Documentation}*/
static readonly ANTIQUE_WHITE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#AQUA} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23AQUA Online Documentation}*/
static readonly AQUA: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#AQUAMARINE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23AQUAMARINE Online Documentation}*/
static readonly AQUAMARINE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#AZURE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23AZURE Online Documentation}*/
static readonly AZURE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#BEIGE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23BEIGE Online Documentation}*/
static readonly BEIGE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#BISQUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23BISQUE Online Documentation}*/
static readonly BISQUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#BLACK} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23BLACK Online Documentation}*/
static readonly BLACK: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#BLANCHED_ALMOND} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23BLANCHED_ALMOND Online Documentation}*/
static readonly BLANCHED_ALMOND: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23BLUE Online Documentation}*/
static readonly BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#BLUE_VIOLET} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23BLUE_VIOLET Online Documentation}*/
static readonly BLUE_VIOLET: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#BROWN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23BROWN Online Documentation}*/
static readonly BROWN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#BURLY_WOOD} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23BURLY_WOOD Online Documentation}*/
static readonly BURLY_WOOD: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#CADET_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23CADET_BLUE Online Documentation}*/
static readonly CADET_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#CHARTREUSE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23CHARTREUSE Online Documentation}*/
static readonly CHARTREUSE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#CHOCOLATE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23CHOCOLATE Online Documentation}*/
static readonly CHOCOLATE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#CORAL} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23CORAL Online Documentation}*/
static readonly CORAL: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#CORNFLOWER_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23CORNFLOWER_BLUE Online Documentation}*/
static readonly CORNFLOWER_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#CORNSILK} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23CORNSILK Online Documentation}*/
static readonly CORNSILK: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#CRIMSON} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23CRIMSON Online Documentation}*/
static readonly CRIMSON: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#CYAN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23CYAN Online Documentation}*/
static readonly CYAN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_BLUE Online Documentation}*/
static readonly DARK_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_CYAN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_CYAN Online Documentation}*/
static readonly DARK_CYAN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_GOLDENROD} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_GOLDENROD Online Documentation}*/
static readonly DARK_GOLDENROD: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_GRAY} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_GRAY Online Documentation}*/
static readonly DARK_GRAY: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_GREEN Online Documentation}*/
static readonly DARK_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_KHAKI} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_KHAKI Online Documentation}*/
static readonly DARK_KHAKI: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_MAGENTA} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_MAGENTA Online Documentation}*/
static readonly DARK_MAGENTA: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_OLIVE_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_OLIVE_GREEN Online Documentation}*/
static readonly DARK_OLIVE_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_ORANGE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_ORANGE Online Documentation}*/
static readonly DARK_ORANGE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_ORCHID} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_ORCHID Online Documentation}*/
static readonly DARK_ORCHID: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_RED} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_RED Online Documentation}*/
static readonly DARK_RED: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_SALMON} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_SALMON Online Documentation}*/
static readonly DARK_SALMON: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_SEA_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_SEA_GREEN Online Documentation}*/
static readonly DARK_SEA_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_SLATE_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_SLATE_BLUE Online Documentation}*/
static readonly DARK_SLATE_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_SLATE_GRAY} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_SLATE_GRAY Online Documentation}*/
static readonly DARK_SLATE_GRAY: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_TURQUOISE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_TURQUOISE Online Documentation}*/
static readonly DARK_TURQUOISE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DARK_VIOLET} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DARK_VIOLET Online Documentation}*/
static readonly DARK_VIOLET: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DEEP_PINK} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DEEP_PINK Online Documentation}*/
static readonly DEEP_PINK: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DEEP_SKY_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DEEP_SKY_BLUE Online Documentation}*/
static readonly DEEP_SKY_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DIM_GRAY} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DIM_GRAY Online Documentation}*/
static readonly DIM_GRAY: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#DODGER_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23DODGER_BLUE Online Documentation}*/
static readonly DODGER_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#FIREBRICK} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23FIREBRICK Online Documentation}*/
static readonly FIREBRICK: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#FLORAL_WHITE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23FLORAL_WHITE Online Documentation}*/
static readonly FLORAL_WHITE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#FOREST_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23FOREST_GREEN Online Documentation}*/
static readonly FOREST_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#FUCHSIA} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23FUCHSIA Online Documentation}*/
static readonly FUCHSIA: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#GAINSBORO} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23GAINSBORO Online Documentation}*/
static readonly GAINSBORO: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#GHOST_WHITE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23GHOST_WHITE Online Documentation}*/
static readonly GHOST_WHITE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#GOLD} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23GOLD Online Documentation}*/
static readonly GOLD: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#GOLDENROD} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23GOLDENROD Online Documentation}*/
static readonly GOLDENROD: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#GRAY} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23GRAY Online Documentation}*/
static readonly GRAY: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23GREEN Online Documentation}*/
static readonly GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#GREEN_YELLOW} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23GREEN_YELLOW Online Documentation}*/
static readonly GREEN_YELLOW: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#HONEYDEW} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23HONEYDEW Online Documentation}*/
static readonly HONEYDEW: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#HOT_PINK} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23HOT_PINK Online Documentation}*/
static readonly HOT_PINK: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#INDIAN_RED} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23INDIAN_RED Online Documentation}*/
static readonly INDIAN_RED: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#INDIGO} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23INDIGO Online Documentation}*/
static readonly INDIGO: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#IVORY} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23IVORY Online Documentation}*/
static readonly IVORY: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#KHAKI} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23KHAKI Online Documentation}*/
static readonly KHAKI: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LAVENDER} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LAVENDER Online Documentation}*/
static readonly LAVENDER: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LAVENDER_BLUSH} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LAVENDER_BLUSH Online Documentation}*/
static readonly LAVENDER_BLUSH: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LAWN_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LAWN_GREEN Online Documentation}*/
static readonly LAWN_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LEMON_CHIFFON} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LEMON_CHIFFON Online Documentation}*/
static readonly LEMON_CHIFFON: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_BLUE Online Documentation}*/
static readonly LIGHT_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_CORAL} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_CORAL Online Documentation}*/
static readonly LIGHT_CORAL: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_CYAN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_CYAN Online Documentation}*/
static readonly LIGHT_CYAN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_GOLDENROD_YELLOW} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_GOLDENROD_YELLOW Online Documentation}*/
static readonly LIGHT_GOLDENROD_YELLOW: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_GRAY} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_GRAY Online Documentation}*/
static readonly LIGHT_GRAY: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_GREEN Online Documentation}*/
static readonly LIGHT_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_PINK} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_PINK Online Documentation}*/
static readonly LIGHT_PINK: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_SALMON} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_SALMON Online Documentation}*/
static readonly LIGHT_SALMON: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_SEA_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_SEA_GREEN Online Documentation}*/
static readonly LIGHT_SEA_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_SKY_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_SKY_BLUE Online Documentation}*/
static readonly LIGHT_SKY_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_SLATE_GRAY} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_SLATE_GRAY Online Documentation}*/
static readonly LIGHT_SLATE_GRAY: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_STEEL_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_STEEL_BLUE Online Documentation}*/
static readonly LIGHT_STEEL_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIGHT_YELLOW} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIGHT_YELLOW Online Documentation}*/
static readonly LIGHT_YELLOW: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIME} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIME Online Documentation}*/
static readonly LIME: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LIME_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LIME_GREEN Online Documentation}*/
static readonly LIME_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#LINEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23LINEN Online Documentation}*/
static readonly LINEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MAGENTA} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MAGENTA Online Documentation}*/
static readonly MAGENTA: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MAROON} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MAROON Online Documentation}*/
static readonly MAROON: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MEDIUM_AQUAMARINE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MEDIUM_AQUAMARINE Online Documentation}*/
static readonly MEDIUM_AQUAMARINE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MEDIUM_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MEDIUM_BLUE Online Documentation}*/
static readonly MEDIUM_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MEDIUM_ORCHID} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MEDIUM_ORCHID Online Documentation}*/
static readonly MEDIUM_ORCHID: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MEDIUM_PURPLE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MEDIUM_PURPLE Online Documentation}*/
static readonly MEDIUM_PURPLE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MEDIUM_SEA_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MEDIUM_SEA_GREEN Online Documentation}*/
static readonly MEDIUM_SEA_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MEDIUM_SLATE_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MEDIUM_SLATE_BLUE Online Documentation}*/
static readonly MEDIUM_SLATE_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MEDIUM_SPRING_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MEDIUM_SPRING_GREEN Online Documentation}*/
static readonly MEDIUM_SPRING_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MEDIUM_TURQUOISE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MEDIUM_TURQUOISE Online Documentation}*/
static readonly MEDIUM_TURQUOISE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MEDIUM_VIOLET_RED} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MEDIUM_VIOLET_RED Online Documentation}*/
static readonly MEDIUM_VIOLET_RED: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MIDNIGHT_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MIDNIGHT_BLUE Online Documentation}*/
static readonly MIDNIGHT_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MINT_CREAM} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MINT_CREAM Online Documentation}*/
static readonly MINT_CREAM: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MISTY_ROSE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MISTY_ROSE Online Documentation}*/
static readonly MISTY_ROSE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#MOCCASIN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23MOCCASIN Online Documentation}*/
static readonly MOCCASIN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#NAVAJO_WHITE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23NAVAJO_WHITE Online Documentation}*/
static readonly NAVAJO_WHITE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#NAVY} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23NAVY Online Documentation}*/
static readonly NAVY: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#OLD_LACE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23OLD_LACE Online Documentation}*/
static readonly OLD_LACE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#OLIVE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23OLIVE Online Documentation}*/
static readonly OLIVE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#OLIVE_DRAB} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23OLIVE_DRAB Online Documentation}*/
static readonly OLIVE_DRAB: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#ORANGE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23ORANGE Online Documentation}*/
static readonly ORANGE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#ORANGE_RED} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23ORANGE_RED Online Documentation}*/
static readonly ORANGE_RED: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#ORCHID} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23ORCHID Online Documentation}*/
static readonly ORCHID: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PALE_GOLDENROD} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PALE_GOLDENROD Online Documentation}*/
static readonly PALE_GOLDENROD: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PALE_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PALE_GREEN Online Documentation}*/
static readonly PALE_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PALE_TURQUOISE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PALE_TURQUOISE Online Documentation}*/
static readonly PALE_TURQUOISE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PALE_VIOLET_RED} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PALE_VIOLET_RED Online Documentation}*/
static readonly PALE_VIOLET_RED: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PAPAYA_WHIP} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PAPAYA_WHIP Online Documentation}*/
static readonly PAPAYA_WHIP: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PEACH_PUFF} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PEACH_PUFF Online Documentation}*/
static readonly PEACH_PUFF: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PERU} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PERU Online Documentation}*/
static readonly PERU: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PINK} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PINK Online Documentation}*/
static readonly PINK: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PLUM} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PLUM Online Documentation}*/
static readonly PLUM: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#POWDER_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23POWDER_BLUE Online Documentation}*/
static readonly POWDER_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#PURPLE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23PURPLE Online Documentation}*/
static readonly PURPLE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#RED} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23RED Online Documentation}*/
static readonly RED: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#ROSY_BROWN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23ROSY_BROWN Online Documentation}*/
static readonly ROSY_BROWN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#ROYAL_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23ROYAL_BLUE Online Documentation}*/
static readonly ROYAL_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SADDLE_BROWN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SADDLE_BROWN Online Documentation}*/
static readonly SADDLE_BROWN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SALMON} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SALMON Online Documentation}*/
static readonly SALMON: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SANDY_BROWN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SANDY_BROWN Online Documentation}*/
static readonly SANDY_BROWN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SEA_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SEA_GREEN Online Documentation}*/
static readonly SEA_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SEA_SHELL} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SEA_SHELL Online Documentation}*/
static readonly SEA_SHELL: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SIENNA} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SIENNA Online Documentation}*/
static readonly SIENNA: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SILVER} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SILVER Online Documentation}*/
static readonly SILVER: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SKY_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SKY_BLUE Online Documentation}*/
static readonly SKY_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SLATE_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SLATE_BLUE Online Documentation}*/
static readonly SLATE_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SLATE_GRAY} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SLATE_GRAY Online Documentation}*/
static readonly SLATE_GRAY: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SNOW} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SNOW Online Documentation}*/
static readonly SNOW: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#SPRING_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23SPRING_GREEN Online Documentation}*/
static readonly SPRING_GREEN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#STEEL_BLUE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23STEEL_BLUE Online Documentation}*/
static readonly STEEL_BLUE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#TAN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23TAN Online Documentation}*/
static readonly TAN: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#TEAL} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23TEAL Online Documentation}*/
static readonly TEAL: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#THISTLE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23THISTLE Online Documentation}*/
static readonly THISTLE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#TOMATO} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23TOMATO Online Documentation}*/
static readonly TOMATO: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#TRANSPARENT} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23TRANSPARENT Online Documentation}*/
static readonly TRANSPARENT: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#TURQUOISE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23TURQUOISE Online Documentation}*/
static readonly TURQUOISE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#VIOLET} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23VIOLET Online Documentation}*/
static readonly VIOLET: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#WHEAT} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23WHEAT Online Documentation}*/
static readonly WHEAT: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#WHITE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23WHITE Online Documentation}*/
static readonly WHITE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#WHITE_SMOKE} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23WHITE_SMOKE Online Documentation}*/
static readonly WHITE_SMOKE: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#YELLOW} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23YELLOW Online Documentation}*/
static readonly YELLOW: Stroke;
/**
Gets a {@link Stroke#isFrozen frozen} {@link Stroke} that uses {@link Fill#YELLOW_GREEN} as its {@link Stroke#fill} and has a {@link Stroke#thickness} of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Stroke%23YELLOW_GREEN Online Documentation}*/
static readonly YELLOW_GREEN: Stroke;
static $class: Class;
private $_Stroke: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Stroke;
}
/** Automatic conversion to {@link Stroke}. */
type StrokeConvertible = string;
/**
Possible values for the line cap of a {@link Stroke}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineCap Online Documentation}*/
export enum LineCap {
/**
Indicates that the line cap of a {@link Stroke} should be squared.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineCap%23SQUARE Online Documentation}*/
SQUARE,
/**
Indicates that the line cap of a {@link Stroke} should be flat.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineCap%23FLAT Online Documentation}*/
FLAT,
/**
Indicates that the line cap of a {@link Stroke} should be a semicircle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineCap%23ROUND Online Documentation}*/
ROUND
}
/** Automatic conversion to {@link LineCap}. */
type LineCapStringValues = "square" | "flat" | "round";
/**
Possible values for the line join of a {@link Stroke}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineJoin Online Documentation}*/
export enum LineJoin {
/**
Indicates that the line joins of a {@link Stroke} should be beveled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineJoin%23BEVEL Online Documentation}*/
BEVEL,
/**
Indicates that the line joins of a {@link Stroke} should be rounded.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineJoin%23ROUND Online Documentation}*/
ROUND,
/**
Indicates that the line joins of a {@link Stroke} should be sharp.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineJoin%23MITER Online Documentation}*/
MITER
}
/** Automatic conversion to {@link LineJoin}. */
type LineJoinStringValues = "bevel" | "round" | "miter";
export interface RadialGradient extends Fill, ISvgDefsCreator {
/**
@inheritdoc*/
accept(context: ICanvasContext, item: Node, id: string): boolean;
} /**
Fills an area with a radial gradient.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient Online Documentation}*/
export class RadialGradient {
/**
Initializes a new instance of the {@link RadialGradient} class.
@param [options] The parameters to pass.
@param options.gradientStops An array of {@link GradientStop}s. Each item will be added to the {@link RadialGradient#gradientStops} collection.
@param [options.center] The center of this radial gradient. This option sets the {@link RadialGradient#center} property on the created object.
@param [options.radiusX] The radius of the largest circle of this radial gradient in horizontal direction. This option sets the {@link RadialGradient#radiusX} property on the created object.
@param [options.radiusY] The radius of the largest circle of this radial gradient in vertical direction. This option sets the {@link RadialGradient#radiusY} property on the created object.
@param [options.spreadMethod] The spread method of this gradient. This option sets the {@link RadialGradient#spreadMethod} property on the created object.
@param [options.gradientOrigin] The origin of this radial gradient. This option sets the {@link RadialGradient#gradientOrigin} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient Online Documentation}*/
constructor(options?: {
center?: Point | PointConvertible;
radiusX?: number;
radiusY?: number;
spreadMethod?: GradientSpreadMethod | GradientSpreadMethodStringValues;
gradientOrigin?: Point | PointConvertible;
gradientStops?: Array<any>;
});
/**
Creates a new SVG gradient that corresponds to this gradient brush.
@param context
@returns A new SVG gradient that corresponds to this gradient brush
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23createDefsElement(ICanvasContext) Online Documentation}*/
createDefsElement(context: ICanvasContext): SVGElement;
/**
Determines whether this {@link RadialGradient} is equal to the specified object.
@param fill The other {@link Fill} instance to compare.
@returns <code>true</code>, if this {@link SolidColorFill} is equal to the other object; <code>false</code>, otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23hasSameValue(Fill) Online Documentation}*/
hasSameValue(fill: Fill | null | FillConvertible): boolean;
/**
Creates a new SVG gradient that corresponds to this gradient brush.
@returns A new SVG gradient that corresponds to this gradient brush
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23toSvgGradient() Online Documentation}*/
toSvgGradient(): SVGElement;
/**
Updates the SVG gradient within the <code>defs</code> section of the SVG document if it has changed.
@param context The context.
@param oldElement The old element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23updateDefsElement(ICanvasContext%2CSVGElement) Online Documentation}*/
updateDefsElement(context: ICanvasContext, oldElement: SVGElement): void;
/**
Updates the SVG gradient within the <code>defs</code> section of the SVG document if it has changed.
@param oldElement The old element.
@param context The context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23updateSvgGradient(SVGElement%2CICanvasContext) Online Documentation}*/
updateSvgGradient(oldElement: SVGElement, context: ICanvasContext): void;
/**
Gets or sets the center of this radial gradient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23center Online Documentation}*/
center: Point;
/**
Gets or sets the origin of this radial gradient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23gradientOrigin Online Documentation}*/
gradientOrigin: Point;
/**
Gets or sets the radius of the largest circle of this radial gradient in horizontal direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23radiusX Online Documentation}*/
radiusX: number;
/**
Gets or sets the radius of the largest circle of this radial gradient in vertical direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23radiusY Online Documentation}*/
radiusY: number;
/**
Gets or sets the spread method of this gradient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23spreadMethod Online Documentation}*/
spreadMethod: GradientSpreadMethod;
/**
Gets or sets the gradient stops of this brush.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialGradient%23gradientStops Online Documentation}*/
gradientStops: List<GradientStop>;
static $class: Class;
private $_RadialGradient: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RadialGradient;
}
export interface HtmlCanvasVisual extends Visual {
}
/**
Base class for {@link Visual}s that use HTML5's Canvas for the visualization.
Note that instances of this class cannot be used anywhere where a {@link Visual} is required. The visualization is
implemented by the {@link HtmlCanvasVisual#paint} method. This means that this kind of visual does not work in a {@link SvgVisualGroup}
and wrapping the visual will not work as there is no {@link SVGElement} that could be put into another SVG container. These kind
of visuals can be used at the top-level of a {@link CanvasComponent}'s {@link ICanvasObject}s, only.
@see HtmlCanvasVisual#paint
@see {@link https://docs.yworks.com/yfileshtml/#/api/HtmlCanvasVisual Online Documentation}*/
export abstract class HtmlCanvasVisual {
constructor();
/**
Paints onto the context using HTML5 Canvas operations.
Implementations should not destroy the context's state, but should make sure to restore the state to the previously
active state. This is especially true for the transformation and clip.
@param renderContext The render context of the {@link CanvasComponent}
@param ctx The HTML5 Canvas context to use for rendering.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HtmlCanvasVisual%23paint(IRenderContext%2CCanvasRenderingContext2D) Online Documentation}*/
abstract paint(renderContext: IRenderContext, ctx: CanvasRenderingContext2D): void;
static $class: Class;
private $_HtmlCanvasVisual: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HtmlCanvasVisual;
}
/**
This enumeration contains the valid values for the <code>shape-rendering</code> property of SVG shape elements.
This property provides is a recommendation to the SVG rendering engine about how to optimize the shape rendering. See
the SVG specification for details.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeRenderingType Online Documentation}*/
export enum ShapeRenderingType {
/**
Indicates that the rendering engine should use a balanced setup with a slight preference of geometric precision.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeRenderingType%23AUTO Online Documentation}*/
AUTO,
/**
Indicates that the rendering speed should be optimized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeRenderingType%23OPTIMIZE_SPEED Online Documentation}*/
OPTIMIZE_SPEED,
/**
Indicates that the rendering should be optimized for crisp edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeRenderingType%23CRISP_EDGES Online Documentation}*/
CRISP_EDGES,
/**
Indicates that the rendering should be optimized for geometric precision.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeRenderingType%23GEOMETRIC_PRECISION Online Documentation}*/
GEOMETRIC_PRECISION,
/**
Indicates that the setting for the rendering quality should be inherited from the parent.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeRenderingType%23INHERIT Online Documentation}*/
INHERIT
}
/** Automatic conversion to {@link ShapeRenderingType}. */
type ShapeRenderingTypeStringValues = "auto" | "optimize-speed" | "crisp-edges" | "geometric-precision" | "inherit";
export interface SolidColorFill extends Fill {
}
/**
Fills an area with a specified solid color.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SolidColorFill Online Documentation}*/
export class SolidColorFill {
/**
Initializes a new instance of the {@link SolidColorFill} class with the specified RGBA values.
All values must be between 0 and 255.
New instances of this type are automatically {@link Fill#isFrozen frozen} since they are immutable anyway.
@param r The red value.
@param g The green value.
@param b The blue value.
@param [a=255] The alpha value, 255 by default.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SolidColorFill%23SolidColorFill(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(r: number, g: number, b: number, a?: number);
/**
Initializes a new instance of the {@link SolidColorFill} class with the specified color.
New instances of this type are automatically {@link Fill#isFrozen frozen} since they are immutable anyway.
@param [color=null] The solid color with which the area is filled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SolidColorFill%23SolidColorFill(Color) Online Documentation}*/
constructor(color?: Color | ColorConvertible);
/**
Freezes this instance.
New instances of this type are automatically {@link Fill#isFrozen frozen} since they are immutable anyway. Calling this method again has no
effect.
Freezing an object makes it immutable, which can result in performance improvements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SolidColorFill%23freeze() Online Documentation}*/
freeze(): void;
/**
Determines whether this {@link SolidColorFill} is equal to the specified object.
@param fill The other {@link Fill} instance to compare.
@returns <code>true</code>, if this {@link SolidColorFill} is equal to the other object; <code>false</code>, otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SolidColorFill%23hasSameValue(Fill) Online Documentation}*/
hasSameValue(fill: Fill | null | FillConvertible): boolean;
/**
Gets the solid color of this fill.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SolidColorFill%23color Online Documentation}*/
readonly color: Color;
static $class: Class;
private $_SolidColorFill: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SolidColorFill;
}
export interface SvgVisual extends Visual {
}
/**
Wraps a {@link SVGElement} as {@link Visual}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisual Online Documentation}*/
export class SvgVisual {
/**
Initializes a new instance of the {@link SvgVisual} class which wraps the specified SVG element.
@param element The SVG element which this object wraps.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisual Online Documentation}*/
constructor(element: SVGElement);
/**
Sets the transform to a scale transform using the given parameters.
This implementation uses caching and the baseVal property of the <code>transform</code> attribute if possible and thus performs
better than using the plain DOM API.
@param transformable The SVG element that should be translated.
@param scaleX The vertical scale.
@param scaleY The horizontal scale.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisual%23setScale(Element%2Cnumber%2Cnumber) Online Documentation}*/
static setScale(transformable: Element, scaleX: number, scaleY: number): void;
/**
Sets the transform to a translate transform using the given parameters.
This implementation uses caching and the baseVal property of the <code>transform</code> attribute if possible and thus performs
better than using the plain DOM API.
@param transformable The SVG element that should be translated.
@param dx The offset on the x-axis.
@param dy The offset on the y-axis.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisual%23setTranslate(Element%2Cnumber%2Cnumber) Online Documentation}*/
static setTranslate(transformable: Element, dx: number, dy: number): void;
/**
Gets or sets the SVG element that is wrapped by this object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SvgVisual%23svgElement Online Documentation}*/
svgElement: SVGElement;
static $class: Class;
private $_SvgVisual: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SvgVisual;
}
/**
Possible values for the horizontal alignment of text.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HorizontalTextAlignment Online Documentation}*/
export enum HorizontalTextAlignment {
/**
Indicates that the text should be centered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HorizontalTextAlignment%23CENTER Online Documentation}*/
CENTER,
/**
Indicates that the text should be aligned to the left.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HorizontalTextAlignment%23LEFT Online Documentation}*/
LEFT,
/**
Indicates that the text should be aligned to the right.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HorizontalTextAlignment%23RIGHT Online Documentation}*/
RIGHT
}
/** Automatic conversion to {@link HorizontalTextAlignment}. */
type HorizontalTextAlignmentStringValues = "center" | "left" | "right";
/**
Enumeration for text decoration for use in {@link Font}
The enumeration can be used to create bitwise combination of the types.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextDecoration Online Documentation}*/
export enum TextDecoration {
/**
No text decoration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextDecoration%23NONE Online Documentation}*/
NONE,
/**
A line beneath the text.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextDecoration%23UNDERLINE Online Documentation}*/
UNDERLINE,
/**
A line above the text.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextDecoration%23OVERLINE Online Documentation}*/
OVERLINE,
/**
A line through the middle of the text.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextDecoration%23LINE_THROUGH Online Documentation}*/
LINE_THROUGH,
/**
Blinking text.
Blinking text is discouraged and text might simply not blink in user agents that conform to the CSS spec.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextDecoration%23BLINK Online Documentation}*/
BLINK
}
type TextDecorationStringValues = "none" | "underline" | "overline" | "line-through" | "blink";
/** Automatic conversion to {@link TextDecoration}. */
type TextDecorationConvertible = TextDecorationStringValues | TextDecorationStringValues[];
/**
Specifies how text is trimmed when it overflows its desired box.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextWrapping Online Documentation}*/
export enum TextWrapping {
/**
The text should not be trimmed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextWrapping%23NONE Online Documentation}*/
NONE,
/**
The text could be wrapped and clipped after any character.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextWrapping%23CHARACTER Online Documentation}*/
CHARACTER,
/**
The text could be wrapped and clipped after any character and an ellipsis is added to indicate clipped text.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextWrapping%23CHARACTER_ELLIPSIS Online Documentation}*/
CHARACTER_ELLIPSIS,
/**
The text could be wrapped and clipped at word boundaries.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextWrapping%23WORD Online Documentation}*/
WORD,
/**
The text could be wrapped and clipped at word boundaries and an ellipsis is added to indicate trimmed text.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextWrapping%23WORD_ELLIPSIS Online Documentation}*/
WORD_ELLIPSIS
}
/** Automatic conversion to {@link TextWrapping}. */
type TextWrappingStringValues = "none" | "character" | "character-ellipsis" | "word" | "word-ellipsis";
export interface ToolTip extends YObject {
}
/**
Displays a string or any HTML element as tooltip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip Online Documentation}*/
export class ToolTip {
/**
Creates a new instance.
This implementation calls {@link ToolTip#createToolTip} to create the HTML element to display as tooltip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip Online Documentation}*/
constructor();
/**
Creates the {@link HTMLDivElement HTML element} in which the content is displayed.
This implementation creates a simple HTML <code>div</code> element with the CSS class <code>yfiles-tooltip</code>.
@returns A HTML div element.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip%23createToolTip() Online Documentation}*/
createToolTip(): HTMLDivElement;
/**
Sets the content to the HTML element that is displayed as tool tip.
This implementation adds a <code>string</code> as <code>innerHTML</code> of the {@link ToolTip#div}. A <code>HTMLElement</code> is added as child to the {@link ToolTip#div}.
Subclasses which intend to use a more complex element as tool tip have to override this method to set the content
properly.
@param oldContent The content to remove.
@param newContent The content to add.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip%23onContentChanged(Object%2CObject) Online Documentation}*/
onContentChanged(oldContent: any | null, newContent: any | null): void;
/**
Updates the current tooltip location in the page.
Call this method to update the tooltip location when the content changes while it is open.
Calling this method re-evaluates {@link ToolTip#desiredSize} of this tooltip before calculating the new position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip%23updateLocation() Online Documentation}*/
updateLocation(): void;
/**
Gets the div element that represents this {@link ToolTip}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip%23div Online Documentation}*/
div: HTMLDivElement;
/**
Specifies whether the tool tip is visible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip%23isOpen Online Documentation}*/
isOpen: boolean;
/**
Gets the top position of the tool tip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip%23verticalOffset Online Documentation}*/
verticalOffset: number;
/**
Gets the left position of the tool tip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip%23horizontalOffset Online Documentation}*/
horizontalOffset: number;
/**
Gets the desired size of this tooltip.
This property returns the offsetWidth and offsetHeight of the {@link ToolTip#div}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip%23desiredSize Online Documentation}*/
desiredSize: Size;
/**
Gets or sets the content of this tooltip.
The content can be either a string or any HTML element.
This setter invokes the method {@link ToolTip#onContentChanged} if the content is changed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ToolTip%23content Online Documentation}*/
content: any | null;
static $class: Class;
private $_ToolTip: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ToolTip;
}
export interface Font extends YObject {
}
/**
* Describes a font using with size, family, style, weight, and line spacing.
* This is a convertible type that can be used with the following notation(s) in parameter lists, parameter objects or
* setters.
*
* CSS font shorthand strings are converted to {@link Font}s. For example:
*
* <pre>
* 'bold 16px sans-serif'
* 'italic Tahoma'
* 'normal xx-large Arial'
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Font Online Documentation}
*/
export class Font {
/**
Initializes a new instance of the {@link Font} class with size 10, "Arial" family, normal weight and style and line
spacing <code>0.5</code>, or the given values.
@param [fontFamily=null] A comma separated list of font-family names.
@param [fontSize=10] The font size in px.
@param [fontStyle=null] The font style.
@param [fontWeight=null] The font weight.
@param [lineSpacing=0.5] The line spacing which is interpreted as additional spacing of the line height.
@param [textDecoration=null] The text decoration.
@see Font#createCopy
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font Online Documentation}*/
constructor(fontFamily?: string | null, fontSize?: number, fontStyle?: FontStyle | null | FontStyleStringValues, fontWeight?: FontWeight | null | FontWeightStringValues, lineSpacing?: number, textDecoration?: TextDecoration | null | TextDecorationConvertible);
/**
Initializes a new instance of the {@link Font} class with size 10, "Arial" family, normal weight and style and line
spacing <code>0.5</code>, or the given values.
@param options The parameters to pass.
@param [options.fontFamily=null] A comma separated list of font-family names.
@param [options.fontSize=10] The font size in px.
@param [options.fontStyle=null] The font style.
@param [options.fontWeight=null] The font weight.
@param [options.lineSpacing=0.5] The line spacing which is interpreted as additional spacing of the line height.
@param [options.textDecoration=null] The text decoration.
@see Font#createCopy
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font Online Documentation}*/
constructor(options: {
fontFamily?: string | null;
fontSize?: number;
fontStyle?: FontStyle | null | FontStyleStringValues;
fontWeight?: FontWeight | null | FontWeightStringValues;
lineSpacing?: number;
textDecoration?: TextDecoration | null | TextDecorationConvertible;
});
/**
Assigns the text properties of this instance to the given text element.
@param textElement The text element to assign set the {@link Font} on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23applyTo(SVGTextElement) Online Documentation}*/
applyTo(textElement: SVGTextElement): void;
/**
Creates a copy with the values of this instance or optionally the explicitly specified values.
@param [fontFamily=null] The font family to use for the copy.
@param [fontSize=-1] The font size to use for the copy.
@param [fontStyle=null] The font style to use for the copy.
@param [fontWeight=null] The font weight to use for the copy.
@param [lineSpacing=-1] The line spacing to use for the copy.
@param [textDecoration=null] The text decoration to use for the copy.
@returns A new {@link Font} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23createCopy(string%2Cnumber%2CFontStyle%2CFontWeight%2Cnumber%2CTextDecoration) Online Documentation}*/
createCopy(fontFamily?: string | null, fontSize?: number, fontStyle?: FontStyle | null | FontStyleStringValues, fontWeight?: FontWeight | null | FontWeightStringValues, lineSpacing?: number, textDecoration?: TextDecoration | null | TextDecorationConvertible): Font;
/**
Creates a copy with the values of this instance or optionally the explicitly specified values.
@param options The parameters to pass.
@param [options.fontFamily=null] The font family to use for the copy.
@param [options.fontSize=-1] The font size to use for the copy.
@param [options.fontStyle=null] The font style to use for the copy.
@param [options.fontWeight=null] The font weight to use for the copy.
@param [options.lineSpacing=-1] The line spacing to use for the copy.
@param [options.textDecoration=null] The text decoration to use for the copy.
@returns A new {@link Font} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23createCopy(string%2Cnumber%2CFontStyle%2CFontWeight%2Cnumber%2CTextDecoration) Online Documentation}*/
createCopy(options: {
fontFamily?: string | null;
fontSize?: number;
fontStyle?: FontStyle | null | FontStyleStringValues;
fontWeight?: FontWeight | null | FontWeightStringValues;
lineSpacing?: number;
textDecoration?: TextDecoration | null | TextDecorationConvertible;
}): Font;
/**
Creates a {@link Font} instance from the given font-like object by performing automatic type conversion.
@param typefaceLike The object to convert to a {@link Font}.
@returns Either <code>typefaceLike</code> if it is already a {@link Font} or a new instance, initialized to the values found in
<code>typefaceLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23from(Font) Online Documentation}*/
static from(typefaceLike: Font | FontConvertible): Font;
/**
Gets the size of the font.
The font size is interpreted as px.
@default 10
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23fontSize Online Documentation}*/
fontSize: number;
/**
Gets the font family.
If the given font family string contains quotation marks, it is assumed to follow the <a href="https://www.w3.org/TR/SVG11/text.html#FontFamilyProperty">W3C fontFamiliy value specification</a> regarding additional quotes,
listing multiple font family names or using generic family names. Otherwise, non-generic font family names will be
automatically quoted in order to avoid invalid font family name tokens.
@default 'Arial'
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23fontFamily Online Documentation}*/
fontFamily: string;
/**
Gets the font style.
@default FontStyle.NORMAL
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23fontStyle Online Documentation}*/
fontStyle: FontStyle;
/**
Gets the font weight.
@default FontWeight.NORMAL
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23fontWeight Online Documentation}*/
fontWeight: FontWeight;
/**
Gets the line spacing.
The value is interpreted as a fraction of the line height in EM values.
@default 0.5
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23lineSpacing Online Documentation}*/
lineSpacing: number;
/**
Gets the text decoration.
@default TextDecoration.NONE
@see {@link https://docs.yworks.com/yfileshtml/#/api/Font%23textDecoration Online Documentation}*/
textDecoration: TextDecoration;
static $class: Class;
private $_Font: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Font;
}
/** Automatic conversion to {@link Font}. */
type FontConvertible = string;
/**
Enumeration values for font styles for use in {@link Font}
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontStyle Online Documentation}*/
export enum FontStyle {
/**
The Normal font style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontStyle%23NORMAL Online Documentation}*/
NORMAL,
/**
The Italic font style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontStyle%23ITALIC Online Documentation}*/
ITALIC,
/**
The Oblique font style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontStyle%23OBLIQUE Online Documentation}*/
OBLIQUE,
/**
The Inherit font style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontStyle%23INHERIT Online Documentation}*/
INHERIT
}
/** Automatic conversion to {@link FontStyle}. */
type FontStyleStringValues = "normal" | "italic" | "oblique" | "inherit";
/**
Enumeration values for symbolic font weights for use in {@link Font}
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight Online Documentation}*/
export enum FontWeight {
/**
The Normal font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23NORMAL Online Documentation}*/
NORMAL,
/**
The Bold font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23BOLD Online Documentation}*/
BOLD,
/**
The Bolder font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23BOLDER Online Documentation}*/
BOLDER,
/**
The Lighter font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23LIGHTER Online Documentation}*/
LIGHTER,
/**
The Item100 font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23ITEM100 Online Documentation}*/
ITEM100,
/**
The Item200 font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23ITEM200 Online Documentation}*/
ITEM200,
/**
The Item300 font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23ITEM300 Online Documentation}*/
ITEM300,
/**
The Item400 font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23ITEM400 Online Documentation}*/
ITEM400,
/**
The Item500 font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23ITEM500 Online Documentation}*/
ITEM500,
/**
The Item600 font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23ITEM600 Online Documentation}*/
ITEM600,
/**
The Item700 font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23ITEM700 Online Documentation}*/
ITEM700,
/**
The Item800 font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23ITEM800 Online Documentation}*/
ITEM800,
/**
The Item900 font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23ITEM900 Online Documentation}*/
ITEM900,
/**
The Inherit font weight.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FontWeight%23INHERIT Online Documentation}*/
INHERIT
}
/** Automatic conversion to {@link FontWeight}. */
type FontWeightStringValues = "normal" | "bold" | "bolder" | "lighter" | "item100" | "item200" | "item300" | "item400" | "item500" | "item600" | "item700" | "item800" | "item900" | "inherit";
export interface ValueSerializer extends YObject {
}
/**
Helper classes used during the XAML serialization of instances.
Using this class instances can be written in xml attributes as strings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ValueSerializer Online Documentation}*/
export class ValueSerializer {
/**
Determines whether this instance can convert the specified value from the given string.
@param value The string value to convert.
@param context The context.
@returns <code>true</code> if this instance can convert the specified value from the string; otherwise, <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ValueSerializer%23canConvertFromString(string%2CIValueSerializerContext) Online Documentation}*/
canConvertFromString(value: string, context: IValueSerializerContext | null): boolean;
/**
Determines whether this instance can convert the specified value to a string.
@param value The value.
@param context The context.
@returns <code>true</code> if this instance can convert the specified value to a string; otherwise, <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ValueSerializer%23canConvertToString(Object%2CIValueSerializerContext) Online Documentation}*/
canConvertToString(value: any | null, context: IValueSerializerContext | null): boolean;
/**
Converts the string to an appropriate instance.
@param value The value to convert to an object.
@param context The context.
@returns The converted object
@see {@link https://docs.yworks.com/yfileshtml/#/api/ValueSerializer%23convertFromString(string%2CIValueSerializerContext) Online Documentation}*/
convertFromString(value: string, context: IValueSerializerContext | null): any | null;
/**
Converts the given value to string.
@param o The value.
@param context The context.
@returns The value as string representation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ValueSerializer%23convertToString(Object%2CIValueSerializerContext) Online Documentation}*/
convertToString(o: any | null, context: IValueSerializerContext | null): string;
/**
Gets the {@link ValueSerializer} implementation for the given type.
@param type The type.
@param context The context.
@returns The serializer to use for the type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ValueSerializer%23getSerializerFor(Class%2CIValueSerializerContext) Online Documentation}*/
static getSerializerFor(type: Class, context: IValueSerializerContext): ValueSerializer | null;
/**
Gets the {@link ValueSerializer} implementation for the given descriptor.
@param property The descriptor.
@param context The context.
@returns The serializer to use for the descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ValueSerializer%23getSerializerFor(Property%2CIValueSerializerContext) Online Documentation}*/
static getSerializerFor(property: Property, context: IValueSerializerContext): ValueSerializer | null;
static $class: Class;
private $_ValueSerializer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ValueSerializer;
}
/**
Possible values for vertical alignment of objects.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VerticalTextAlignment Online Documentation}*/
export enum VerticalTextAlignment {
/**
Indicates that the object should be placed in the center of another object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/VerticalTextAlignment%23CENTER Online Documentation}*/
CENTER,
/**
Indicates that the object should be placed at the top of another object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/VerticalTextAlignment%23TOP Online Documentation}*/
TOP,
/**
Indicates that the object should be placed at the bottom of another object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/VerticalTextAlignment%23BOTTOM Online Documentation}*/
BOTTOM
}
/** Automatic conversion to {@link VerticalTextAlignment}. */
type VerticalTextAlignmentStringValues = "center" | "top" | "bottom";
export interface Visual extends YObject {
}
/**
Base class for all objects that support drawing in the context of a {@link CanvasComponent}.
@see SvgVisual
@see HtmlCanvasVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/Visual Online Documentation}*/
export abstract class Visual {
static $class: Class;
private $_Visual: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Visual;
}
export interface WebGLVisual extends Visual {
}
/**
Base class for {@link Visual}s that use WebGL for the visualization.
Note that instances of this class cannot be used anywhere where a {@link Visual} is required. The visualization is
implemented by the {@link WebGLVisual#render} method. This means that this kind of visual does not work in a {@link SvgVisualGroup}
and wrapping the visual will not work as there is no {@link SVGElement} that could be put into another SVG container. These kind
of visuals can be used at the top-level of a {@link CanvasComponent}'s {@link ICanvasObject}s, only.
@see WebGLVisual#render
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLVisual Online Documentation}*/
export abstract class WebGLVisual {
constructor();
/**
Renders into the context using WebGL operations.
Implementations should not destroy the context's state, but should make sure to restore the state to the previously
active state.
@param ctx The render context of the {@link CanvasComponent}
@param gl The WebGL Rendering context to use for rendering.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLVisual%23render(IRenderContext%2CWebGLRenderingContext) Online Documentation}*/
abstract render(ctx: IRenderContext, gl: WebGLRenderingContext): void;
static $class: Class;
private $_WebGLVisual: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WebGLVisual;
}
export interface EventRecognizers extends YObject {
}
/**
* Provides implementation singletons and utility methods for simple event recognizers.
* EventRecognizers are convertible: strings like <code>"TOUCH_DOWN_PRIMARY || (LEFT_DOWN &amp;&amp; SHIFT_IS_DOWN)"</code> are automatically
* converted to recognizer functions when used in parameter lists, parameter objects, or setters.
*
* Valid EventRecognizer strings follow this grammar:
*
* <pre>
* EXPR = RECOGNIZER
*      | '(' EXPR ')'
*      | '!' EXPR
*      | EXPR '&&' EXPR
*      | EXPR '||' EXPR
* </pre>
*
* Whitespace is ignored and <code>RECOGNIZER</code> is one of the EventRecognizer constants defined in
* {@link MouseEventRecognizers}, {@link KeyEventRecognizers}, or {@link TouchEventRecognizers}. The
* operators work mostly as in JavaScript. <code>&amp;&amp;</code> and <code>||</code> have the same operator precedence, though. This means that, e.g., <code>A || B &amp;&amp; C</code>
* is - unlike JavaScript - bracketed <code>(A || B) &amp;&amp; C</code> and not <code>A || (B &amp;&amp; C)</code>.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EventRecognizers Online Documentation}
*/
export class EventRecognizers {
/**
Creates an instance that is the logical 'and' combination of the provided recognizers.
@param recognizers The recognizers to perform the logical operation on.
@returns An instance that is the result of the logical 'and' operation of the provided instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EventRecognizers%23createAndRecognizer(function(Object%2CEventArgs)%3Aboolean) Online Documentation}*/
static createAndRecognizer(...recognizers: ((eventSource: any | null, evt: EventArgs | null) => boolean)[]): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Creates an instance that is the logical 'or' combination of the provided recognizers.
@param recognizers The recognizers to perform the logical operation on.
@returns An instance that is the result of the logical 'or' operation of the provided instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EventRecognizers%23createOrRecognizer(function(Object%2CEventArgs)%3Aboolean) Online Documentation}*/
static createOrRecognizer(...recognizers: ((eventSource: any | null, evt: EventArgs | null) => boolean)[]): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
* Creates an event recognizer instance that yields the boolean inverse of the given event recognizer.
* Recognizes left mouse down events without the Shift key pressed.
*
* <pre>
* const moveInputMode = graphEditorInputMode.moveInputMode
* moveInputMode.hoverRecognizer = EventRecognizers.createAndRecognizer(
*   MouseEventRecognizers.LEFT_IS_DOWN,
*   (sender, args) => !KeyEventRecognizers.SHIFT_IS_DOWN(sender, args)
* )
* </pre>
*
* @param recognizer The recognizer to invert.
* @returns An implementation that performs the logical negation for the argument.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EventRecognizers%23inverse(function(Object%2CEventArgs)%3Aboolean) Online Documentation}
*/
static inverse(recognizer: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean)): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that will always return true;
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EventRecognizers%23ALWAYS Online Documentation}*/
static ALWAYS: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that will always return false;
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EventRecognizers%23NEVER Online Documentation}*/
static NEVER: (eventSource: any | null, evt: EventArgs | null) => boolean;
static $class: Class;
private $_EventRecognizers: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EventRecognizers;
}
export interface KeyEventRecognizers extends YObject {
}
/**
Provides commonly used predefined event recognizers that work on {@link KeyEventArgs}.
EventRecognizers are convertible, see {@link EventRecognizers}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers Online Documentation}*/
export class KeyEventRecognizers {
/**
Creates an event recognizer for a given key and type using a given set of modifiers.
@param type The type of the event.
@param value The key that is subject of the event.
@param [modifiers=ModifierKeys.NONE] The state of the modifiers that must be set. This is {@link ModifierKeys#NONE} if not set.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23create(KeyEventType%2CKey%2CModifierKeys) Online Documentation}*/
static create(type: KeyEventType | KeyEventTypeStringValues, value: Key | KeyStringValues, modifiers?: ModifierKeys | ModifierKeysConvertible): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Creates an event recognizer that matches if the specified keys are currently down.
@param key The key to create the recognizer for.
@param [modifiers=ModifierKeys.NONE] The modifier keys that should be down for the recognizer.
@returns An event recognizer that uses the gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23createIsDown(Key%2CModifierKeys) Online Documentation}*/
static createIsDown(key: Key | KeyStringValues, modifiers?: ModifierKeys | ModifierKeysConvertible): (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>keypress</code> of the Escape key.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23ESCAPE_PRESS Online Documentation}*/
static ESCAPE_PRESS: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>keydown</code> of any key.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23KEY_DOWN Online Documentation}*/
static KEY_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>keypress</code> of any key.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23KEY_PRESS Online Documentation}*/
static KEY_PRESS: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>keyup</code> of any key.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23KEY_UP Online Documentation}*/
static KEY_UP: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>keydown</code> of the Escape key.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23ESCAPE_DOWN Online Documentation}*/
static ESCAPE_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>keydown</code> of the Delete key.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23DELETE_DOWN Online Documentation}*/
static DELETE_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>keyup</code> of the Escape key.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23ESCAPE_UP Online Documentation}*/
static ESCAPE_UP: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that recognizes when the Shift modifier is active.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23SHIFT_IS_DOWN Online Documentation}*/
static SHIFT_IS_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that recognizes when the Alt modifier is active.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23ALT_IS_DOWN Online Documentation}*/
static ALT_IS_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that recognizes when the Ctrl modifier is active.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23CTRL_IS_DOWN Online Documentation}*/
static CTRL_IS_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that recognizes when the Meta modifier is active.
Meta maps to the Windows key on Windows platforms and the Command key on Mac.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23META_IS_DOWN Online Documentation}*/
static META_IS_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that recognizes when the Shift modifier is being pressed.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23SHIFT_DOWN Online Documentation}*/
static SHIFT_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that recognizes when the Shift modifier is being released.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23SHIFT_UP Online Documentation}*/
static SHIFT_UP: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that recognizes when the Ctrl modifier is being pressed.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23CTRL_DOWN Online Documentation}*/
static CTRL_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that recognizes when the Ctrl modifier is being released.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/KeyEventRecognizers%23CTRL_UP Online Documentation}*/
static CTRL_UP: (eventSource: any | null, evt: EventArgs | null) => boolean;
static $class: Class;
private $_KeyEventRecognizers: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is KeyEventRecognizers;
}
/** Automatic conversion to {@link EventRecognizer}. */
type EventRecognizerConvertible = string;
/**
Enumeration of the possible {@link MouseEventArgs} types.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes Online Documentation}*/
export enum MouseEventTypes {
/**
Not a mouse event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23NONE Online Documentation}*/
NONE,
/**
The mouse has been moved.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23MOVE Online Documentation}*/
MOVE,
/**
The mouse has been dragged, which means it has been moved while at least one button has been pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23DRAG Online Documentation}*/
DRAG,
/**
A button has been pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23DOWN Online Documentation}*/
DOWN,
/**
A button has been released.
If there was no drag event since the last press event, this will result in a click.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23UP Online Documentation}*/
UP,
/**
The mouse has entered the control's bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23ENTER Online Documentation}*/
ENTER,
/**
The mouse has exited the control's bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23LEAVE Online Documentation}*/
LEAVE,
/**
The mouse wheel has been turned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23WHEEL Online Documentation}*/
WHEEL,
/**
A mouse click has been recognized.
This is the case if a press has directly been followed by a release.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23CLICK Online Documentation}*/
CLICK,
/**
Mouse input capture has been lost.
This can happen during mouse drags, if any button is pressed and the focus is taken from the application or transferred
to another component by other means then moving the mouse.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventTypes%23LOST_CAPTURE Online Documentation}*/
LOST_CAPTURE
}
/** Automatic conversion to {@link MouseEventTypes}. */
type MouseEventTypesStringValues = "none" | "move" | "drag" | "down" | "up" | "enter" | "leave" | "wheel" | "click" | "lost-capture";
export interface MouseEventArgs extends EventArgs {
}
/**
Complex event arguments that is used by the {@link CanvasComponent} to indicate mouse events in world coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs Online Documentation}*/
export class MouseEventArgs {
/**
Constructs a new mouse event argument.
@param originalEvent The original DOM event.
@param eventType The type of the event
@param location the position of the mouse in world coordinates at the time of the event
@param [changedButtons=MouseButtons.NONE] the mouse buttons that have changed state if the type of event is {@link MouseEventTypes#DOWN} or
{@link MouseEventTypes#UP}
@param [changedModifiers=ModifierKeys.NONE] the modifier Keys that have been changed
@param [buttons=MouseButtons.NONE] the current state of the MouseButtons
@param [modifiers=ModifierKeys.NONE] the current state of the modifier Keys
@param [clickCount=0] the number of clicks this event represents, 2 for a double click
@param [mouseWheelDelta=0] the signed number of vertical mouse wheel turn units
@param [mouseWheelDeltaX=0] the signed number of horizontal mouse wheel turn units
@param [scrollAmount=0] The number of lines to scroll, if <code>scrollType</code> is {@link MouseWheelDeltaMode#LINES}.
@param [scrollType=MouseWheelDeltaMode.LINES] The type of scrolling performed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23MouseEventArgs(Event%2CMouseEventTypes%2CPoint%2CMouseButtons%2CModifierKeys%2CMouseButtons%2CModifierKeys%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2CMouseWheelDeltaMode) Online Documentation}*/
constructor(originalEvent: Event | null, eventType: MouseEventTypes | MouseEventTypesStringValues, location: Point | PointConvertible, changedButtons?: MouseButtons | MouseButtonsConvertible, changedModifiers?: ModifierKeys | ModifierKeysConvertible, buttons?: MouseButtons | MouseButtonsConvertible, modifiers?: ModifierKeys | ModifierKeysConvertible, clickCount?: number, mouseWheelDelta?: number, mouseWheelDeltaX?: number, scrollAmount?: number, scrollType?: MouseWheelDeltaMode | MouseWheelDeltaModeStringValues);
/**
Constructs a new mouse event argument.
@param options The parameters to pass.
@param options.originalEvent The original DOM event.
@param options.eventType The type of the event
@param options.location the position of the mouse in world coordinates at the time of the event
@param [options.changedButtons=MouseButtons.NONE] the mouse buttons that have changed state if the type of event is {@link MouseEventTypes#DOWN} or
{@link MouseEventTypes#UP}
@param [options.changedModifiers=ModifierKeys.NONE] the modifier Keys that have been changed
@param [options.buttons=MouseButtons.NONE] the current state of the MouseButtons
@param [options.modifiers=ModifierKeys.NONE] the current state of the modifier Keys
@param [options.clickCount=0] the number of clicks this event represents, 2 for a double click
@param [options.mouseWheelDelta=0] the signed number of vertical mouse wheel turn units
@param [options.mouseWheelDeltaX=0] the signed number of horizontal mouse wheel turn units
@param [options.scrollAmount=0] The number of lines to scroll, if <code>scrollType</code> is {@link MouseWheelDeltaMode#LINES}.
@param [options.scrollType=MouseWheelDeltaMode.LINES] The type of scrolling performed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23MouseEventArgs(Event%2CMouseEventTypes%2CPoint%2CMouseButtons%2CModifierKeys%2CMouseButtons%2CModifierKeys%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2CMouseWheelDeltaMode) Online Documentation}*/
constructor(options: {
originalEvent: Event | null;
eventType: MouseEventTypes | MouseEventTypesStringValues;
location: Point | PointConvertible;
changedButtons?: MouseButtons | MouseButtonsConvertible;
changedModifiers?: ModifierKeys | ModifierKeysConvertible;
buttons?: MouseButtons | MouseButtonsConvertible;
modifiers?: ModifierKeys | ModifierKeysConvertible;
clickCount?: number;
mouseWheelDelta?: number;
mouseWheelDeltaX?: number;
scrollAmount?: number;
scrollType?: MouseWheelDeltaMode | MouseWheelDeltaModeStringValues;
});
/**
Sets this event's {@link MouseEventArgs#defaultPrevented} property to <code>true</code> signaling that the event has been properly handled and any default behaviors should
be prevented.
This will call the corresponding method on the {@link MouseEventArgs#originalEvent original native event}.
@see MouseEventArgs#defaultPrevented
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23preventDefault() Online Documentation}*/
preventDefault(): void;
/**
Provides a human readable string representation of this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23toString() Online Documentation}*/
toString(): string;
/**
Gets the type of this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23eventType Online Documentation}*/
readonly eventType: MouseEventTypes;
/**
Gets the coordinates in the world coordinate space associated with this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23location Online Documentation}*/
readonly location: Point;
/**
Gets the number of clicks the current event represents if it is of type {@link MouseEventTypes#CLICK}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23clickCount Online Documentation}*/
readonly clickCount: number;
/**
Gets the signed number of vertical mouse wheel turn units.
Note that this can, for high-precision mouse wheels, include fractional turns.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23wheelDelta Online Documentation}*/
readonly wheelDelta: number;
/**
Gets the signed number of horizontal mouse wheel turn units.
Note that this can, for high-precision mouse wheels, include fractional turns.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23wheelDeltaX Online Documentation}*/
readonly wheelDeltaX: number;
/**
Gets the distance to scroll at once when {@link MouseEventArgs#deltaMode} is {@link MouseWheelDeltaMode#LINES}.
This is a system setting that only applies to scrolling (not zooming) and usually translates to the number of small
scroll increments per scroll wheel notch.
This property has a value of <code>1</code> if {@link MouseEventArgs#deltaMode} is {@link MouseWheelDeltaMode#PAGE}, as it is not applicable in that case.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23scrollAmount Online Documentation}*/
readonly scrollAmount: number;
/**
Gets the scrolling behavior.
This is a system setting that only applies to scrolling (not zooming) and can be either
{@link MouseWheelDeltaMode#LINES}, in which case {@link MouseEventArgs#scrollAmount} will contain the number of "lines" to scroll with each scroll
wheel notch, or {@link MouseWheelDeltaMode#PAGE} in which case a single scroll wheel notch should scroll a "page" at
a time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23deltaMode Online Documentation}*/
readonly deltaMode: MouseWheelDeltaMode;
/**
Gets the modifier keys that have changed for this event.
@see MouseEventArgs#modifiers
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23changedModifiers Online Documentation}*/
readonly changedModifiers: ModifierKeys;
/**
Gets the mouse buttons that have changed for this event.
@see MouseEventArgs#buttons
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23changedButtons Online Documentation}*/
readonly changedButtons: MouseButtons;
/**
Gets the state of the modifier keys at the time of the event creation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23modifiers Online Documentation}*/
readonly modifiers: ModifierKeys;
/**
Gets the state of the mouse buttons at the time of the event creation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23buttons Online Documentation}*/
readonly buttons: MouseButtons;
/**
Gets the original DOM event associated with this event.
In the case of pen input, the value is a <code>PointerEvent</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23originalEvent Online Documentation}*/
originalEvent: Event | null;
/**
Gets a value indicating whether the default behaviors for this event have been prevented by a previous call to
{@link MouseEventArgs#preventDefault}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventArgs%23defaultPrevented Online Documentation}*/
readonly defaultPrevented: boolean;
static $class: Class;
private $_MouseEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MouseEventArgs;
}
/**
An enumeration containing values for different mouse wheel scroll behavior.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseWheelDeltaMode Online Documentation}*/
export enum MouseWheelDeltaMode {
/**
Scrolling should scroll a set number of lines per mouse wheel notch.
@see MouseEventArgs#scrollAmount
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseWheelDeltaMode%23LINES Online Documentation}*/
LINES,
/**
Scrolling should scroll a single "page" per mouse wheel notch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseWheelDeltaMode%23PAGE Online Documentation}*/
PAGE
}
/** Automatic conversion to {@link MouseWheelDeltaMode}. */
type MouseWheelDeltaModeStringValues = "lines" | "page";
export interface MouseEventRecognizers extends YObject {
}
/**
Provides commonly used predefined event recognizers that deal with {@link MouseEventArgs}.
EventRecognizers are convertible, see {@link EventRecognizers}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers Online Documentation}*/
export class MouseEventRecognizers {
/**
An event recognizer that identifies mouse movements that are not drags.
@see MouseEventRecognizers#DRAG
@see MouseEventRecognizers#MOVE_OR_DRAG
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23MOVE Online Documentation}*/
static MOVE: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies mouse movements.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23MOVE_OR_DRAG Online Documentation}*/
static MOVE_OR_DRAG: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>mousedown</code> events.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23DOWN Online Documentation}*/
static DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>mouseup</code> events.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23UP Online Documentation}*/
static UP: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies mouse input capture lost events.
This event will occur often, especially after all mouse buttons have been released. More relevant for
{@link IInputMode}s is the {@link MouseEventRecognizers#LOST_CAPTURE_DURING_DRAG} event recognizer.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23LOST_CAPTURE Online Documentation}*/
static LOST_CAPTURE: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies mouse input capture lost events while at least one mouse button was pressed.
Since the mouse capture has been lost, the necessary {@link MouseEventTypes#UP} events will never be triggered. This
recognizer is useful for identifying this situation.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23LOST_CAPTURE_DURING_DRAG Online Documentation}*/
static LOST_CAPTURE_DURING_DRAG: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies left mouse button clicks.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23LEFT_CLICK Online Documentation}*/
static LEFT_CLICK: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies button clicks.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23CLICK Online Documentation}*/
static CLICK: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies button multi-clicks.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23MULTI_CLICK Online Documentation}*/
static MULTI_CLICK: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies right mouse button clicks.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23RIGHT_CLICK Online Documentation}*/
static RIGHT_CLICK: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies middle mouse button clicks.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23MIDDLE_CLICK Online Documentation}*/
static MIDDLE_CLICK: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies left mouse button double clicks.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23LEFT_DOUBLE_CLICK Online Documentation}*/
static LEFT_DOUBLE_CLICK: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies right mouse button double clicks.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23RIGHT_DOUBLE_CLICK Online Documentation}*/
static RIGHT_DOUBLE_CLICK: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies middle mouse button double clicks.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23MIDDLE_DOUBLE_CLICK Online Documentation}*/
static MIDDLE_DOUBLE_CLICK: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>mousedown</code> events of left mouse button.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23LEFT_DOWN Online Documentation}*/
static LEFT_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>mouseup</code> events of left mouse button.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23LEFT_UP Online Documentation}*/
static LEFT_UP: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>mousedown</code> events of right mouse button.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23RIGHT_DOWN Online Documentation}*/
static RIGHT_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies <code>mouseup</code> events of right mouse button.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23RIGHT_UP Online Documentation}*/
static RIGHT_UP: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies mouse drag events.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23DRAG Online Documentation}*/
static DRAG: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies left mouse button drags.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23LEFT_DRAG Online Documentation}*/
static LEFT_DRAG: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies when the has left the control.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23LEAVE Online Documentation}*/
static LEAVE: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies when the mouse has entered the control.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23ENTER Online Documentation}*/
static ENTER: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies right mouse button drags.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23RIGHT_DRAG Online Documentation}*/
static RIGHT_DRAG: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies middle mouse button drags.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23MIDDLE_DRAG Online Documentation}*/
static MIDDLE_DRAG: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies if any mouse button is currently down.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23IS_DOWN Online Documentation}*/
static IS_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies if the left mouse button is currently down.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23LEFT_IS_DOWN Online Documentation}*/
static LEFT_IS_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies if the right mouse button is currently down.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23RIGHT_IS_DOWN Online Documentation}*/
static RIGHT_IS_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies if the middle mouse button is currently down.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MouseEventRecognizers%23MIDDLE_IS_DOWN Online Documentation}*/
static MIDDLE_IS_DOWN: (eventSource: any | null, evt: EventArgs | null) => boolean;
static $class: Class;
private $_MouseEventRecognizers: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MouseEventRecognizers;
}
/**
Enumeration of the possible {@link TouchEventArgs} types.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes Online Documentation}*/
export enum TouchEventTypes {
/**
Not a touch event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes%23NONE Online Documentation}*/
NONE,
/**
A pointer has been moved.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes%23MOVE Online Documentation}*/
MOVE,
/**
A touch device went down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes%23DOWN Online Documentation}*/
DOWN,
/**
A touch device went up.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes%23UP Online Documentation}*/
UP,
/**
A touch device has entered the control's bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes%23ENTER Online Documentation}*/
ENTER,
/**
A touch device has exited the control's bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes%23LEAVE Online Documentation}*/
LEAVE,
/**
A touch tap has been recognized.
This is the case if a down has directly been followed by an up.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes%23TAP Online Documentation}*/
TAP,
/**
A long press has been recognized.
This is the case if a down has been held a certain amount of time without a move.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes%23LONG_PRESS Online Documentation}*/
LONG_PRESS,
/**
Touch capture has been lost while the device was down.
This can be the case if the application or control is minimized or sent to the background or similar actions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventTypes%23LOST_CAPTURE Online Documentation}*/
LOST_CAPTURE
}
/** Automatic conversion to {@link TouchEventTypes}. */
type TouchEventTypesStringValues = "none" | "move" | "down" | "up" | "enter" | "leave" | "tap" | "long-press" | "lost-capture";
export interface TouchEventArgs extends EventArgs {
}
/**
Complex event arguments that is used by the {@link CanvasComponent} to indicate touch events in world coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs Online Documentation}*/
export class TouchEventArgs {
/**
Constructs a new touch event argument.
@param originalEvent The original JavaScript event, if available.
@param eventType The type of the event
@param location The position of the touch device in world coordinates at the time of the event
@param device The device the event was triggered by
@param modifiers The current state of the modifier keys
@param changedModifiers The modifier keys that have been changed
@param tapCount the number of taps this event represents, 2 for a double tap
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23TouchEventArgs(Event%2CTouchEventTypes%2CPoint%2CTouchDevice%2CModifierKeys%2CModifierKeys%2Cnumber) Online Documentation}*/
constructor(originalEvent: Event | null, eventType: TouchEventTypes | TouchEventTypesStringValues, location: Point | PointConvertible, device: TouchDevice, modifiers: ModifierKeys | ModifierKeysConvertible, changedModifiers: ModifierKeys | ModifierKeysConvertible, tapCount: number);
/**
Constructs a new touch event argument.
@param originalEvent The original JavaScript event, if available.
@param eventType The type of the event
@param location The position of the touch device in world coordinates at the time of the event
@param device The device the event was triggered by
@param modifiers The current state of the modifier keys
@param changedModifiers The modifier keys that have been changed
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23TouchEventArgs(Event%2CTouchEventTypes%2CPoint%2CTouchDevice%2CModifierKeys%2CModifierKeys) Online Documentation}*/
constructor(originalEvent: Event | null, eventType: TouchEventTypes | TouchEventTypesStringValues, location: Point | PointConvertible, device: TouchDevice, modifiers: ModifierKeys | ModifierKeysConvertible, changedModifiers: ModifierKeys | ModifierKeysConvertible);
/**
Sets this event's {@link TouchEventArgs#defaultPrevented} property to <code>true</code> signalling that the event has been properly handled and any default behaviors should
be prevented.
This will call the corresponding method on the {@link TouchEventArgs#originalEvent original native event}.
@see TouchEventArgs#defaultPrevented
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23preventDefault() Online Documentation}*/
preventDefault(): void;
/**
Provides a human readable string representation of this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23toString() Online Documentation}*/
toString(): string;
/**
Gets the type of this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23eventType Online Documentation}*/
readonly eventType: TouchEventTypes;
/**
Returns the location represented by this event in world coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23location Online Documentation}*/
readonly location: Point;
/**
Returns the touch device associated with this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23device Online Documentation}*/
readonly device: TouchDevice;
/**
Returns the number of taps the current event represents if it is of type {@link TouchEventTypes#TAP}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23tapCount Online Documentation}*/
readonly tapCount: number;
/**
Gets the modifier keys that have changed for this event.
@see TouchEventArgs#modifiers
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23changedModifiers Online Documentation}*/
readonly changedModifiers: ModifierKeys;
/**
Gets the state of the modifier keys at the time of the event creation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23modifiers Online Documentation}*/
readonly modifiers: ModifierKeys;
/**
Gets the original DOM event associated with this event.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23originalEvent Online Documentation}*/
readonly originalEvent: Event | null;
/**
Gets a value indicating whether the default behaviors for this event have been prevented by a previous call to
{@link TouchEventArgs#preventDefault}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventArgs%23defaultPrevented Online Documentation}*/
readonly defaultPrevented: boolean;
static $class: Class;
private $_TouchEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TouchEventArgs;
}
export interface TouchDevice extends YObject {
}
/**
The representation of a touch point on the screen.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchDevice Online Documentation}*/
export class TouchDevice {
/**
Gets the device's index
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchDevice%23deviceIndex Online Documentation}*/
readonly deviceIndex: number;
/**
Gets a value indicating whether this device is the primary one.
If {@link TouchDevice#deviceIndex} is <code>0</code> this property has the value <code>true</code>, otherwise <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchDevice%23isPrimaryDevice Online Documentation}*/
readonly isPrimaryDevice: boolean;
/**
Gets the last device coordinate in the view coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchDevice%23lastViewCoordinate Online Documentation}*/
lastViewCoordinate: Point;
/**
Gets the time of the last {@link TouchEventArgs event} that was raised by this device.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchDevice%23lastEventTimeJS Online Documentation}*/
readonly lastEventTimeJS: Date;
/**
Indicates if this device is currently down or not.
Default is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchDevice%23isDown Online Documentation}*/
isDown: boolean;
static $class: Class;
private $_TouchDevice: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TouchDevice;
}
export interface TouchEventRecognizers extends YObject {
}
/**
Provides commonly used predefined event recognizers that deal with {@link TouchEventArgs}.
EventRecognizers are convertible, see {@link EventRecognizers}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers Online Documentation}*/
export class TouchEventRecognizers {
/**
An event recognizer that identifies touch movements of the primary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_MOVE_PRIMARY Online Documentation}*/
static TOUCH_MOVE_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies the loss of touch capture for the any device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23LOST_CAPTURE Online Documentation}*/
static LOST_CAPTURE: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies the loss of touch capture for the primary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23LOST_CAPTURE_PRIMARY Online Documentation}*/
static LOST_CAPTURE_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies the loss of touch capture for the secondary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23LOST_CAPTURE_SECONDARY Online Documentation}*/
static LOST_CAPTURE_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch movements of the primary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_DOWN_PRIMARY Online Documentation}*/
static TOUCH_DOWN_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch movements of the primary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_UP_PRIMARY Online Documentation}*/
static TOUCH_UP_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch taps of the primary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_TAP_PRIMARY Online Documentation}*/
static TOUCH_TAP_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch double-taps of the primary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_DOUBLE_TAP_PRIMARY Online Documentation}*/
static TOUCH_DOUBLE_TAP_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch multi-taps of the primary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_MULTI_TAP_PRIMARY Online Documentation}*/
static TOUCH_MULTI_TAP_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies when the primary device has left the control.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_LEAVE_PRIMARY Online Documentation}*/
static TOUCH_LEAVE_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies when the primary device has entered the control.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_ENTER_PRIMARY Online Documentation}*/
static TOUCH_ENTER_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies long presses of the primary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_LONG_PRESS_PRIMARY Online Documentation}*/
static TOUCH_LONG_PRESS_PRIMARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch movements of the secondary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_MOVE_SECONDARY Online Documentation}*/
static TOUCH_MOVE_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch movements of the secondary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_DOWN_SECONDARY Online Documentation}*/
static TOUCH_DOWN_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch movements of the secondary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_UP_SECONDARY Online Documentation}*/
static TOUCH_UP_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch taps of the secondary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_TAP_SECONDARY Online Documentation}*/
static TOUCH_TAP_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch double-taps of the secondary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_DOUBLE_TAP_SECONDARY Online Documentation}*/
static TOUCH_DOUBLE_TAP_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies touch multi-taps of the secondary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_MULTI_TAP_SECONDARY Online Documentation}*/
static TOUCH_MULTI_TAP_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies when the secondary device has left the control.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_LEAVE_SECONDARY Online Documentation}*/
static TOUCH_LEAVE_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies when the secondary device has entered the control.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_ENTER_SECONDARY Online Documentation}*/
static TOUCH_ENTER_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
An event recognizer that identifies long presses of the secondary device.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TouchEventRecognizers%23TOUCH_LONG_PRESS_SECONDARY Online Documentation}*/
static TOUCH_LONG_PRESS_SECONDARY: (eventSource: any | null, evt: EventArgs | null) => boolean;
static $class: Class;
private $_TouchEventRecognizers: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TouchEventRecognizers;
}
export interface CompositeUndoUnit extends YObject, IUndoUnit {
}
/**
A group of {@link IUndoUnit}s that are undone/redone as a single unit.
Clients can use this class to build an {@link IUndoUnit} that groups together multiple {@link IUndoUnit}s. The
{@link IUndoUnit}s added to this unit are undone/redone in the order in which they were added. This implementation
will try to {@link IUndoUnit#tryMergeUnit merge} or {@link IUndoUnit#tryReplaceUnit replace} units that are {@link CompositeUndoUnit#add added}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit Online Documentation}*/
export class CompositeUndoUnit {
/**
Creates a new instance using the provided names.
@param [undoName=null]
@param [redoName=null]
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit Online Documentation}*/
constructor(undoName?: string | null, redoName?: string | null);
/**
Creates a new instance using the provided names.
@param options The parameters to pass.
@param [options.undoName=null]
@param [options.redoName=null]
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit Online Documentation}*/
constructor(options: {
undoName?: string | null;
redoName?: string | null;
});
/**
Adds a new {@link IUndoUnit} to the queue.
@param unit The unit of work to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23add(IUndoUnit) Online Documentation}*/
add(unit: IUndoUnit): void;
/**
Determines whether a call to {@link CompositeUndoUnit#redo} can be made.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23canRedo() Online Documentation}*/
canRedo(): boolean;
/**
Determines whether a call to {@link CompositeUndoUnit#undo} can be made.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23canUndo() Online Documentation}*/
canUndo(): boolean;
/**
Clears the internal collection of units.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23clear() Online Documentation}*/
clear(): void;
/**
Disposes this unit and all of its contents.
This unit cannot be used anymore after this has been invoked.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23dispose() Online Documentation}*/
dispose(): void;
/**
Redoes the next {@link IUndoUnit}.
@throws {Exception} <b>(NotSupportedError)</b> If an undo operation is already in progress.
@throws {Exception} <b>(Error)</b> If {@link CompositeUndoUnit#canRedo} would yield <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23redo() Online Documentation}*/
redo(): void;
/**
Tries to remove a given unit from this compound unit.
This method will throw an exception if this unit has been undone or an undo or redo is in progress or this unit has
already been disposed. This method will not automatically {@link IUndoUnit#dispose Dispose} removed units.
@param unit The unit to remove.
@returns <code>true</code> iff the unit has been removed from this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23removeUnit(IUndoUnit) Online Documentation}*/
removeUnit(unit: IUndoUnit): boolean;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23toString() Online Documentation}*/
toString(): string;
/**
Undoes the next {@link IUndoUnit}.
@throws {Exception} <b>(NotSupportedError)</b> If an undo operation is already in progress.
@throws {Exception} <b>(Error)</b> If {@link CompositeUndoUnit#canUndo} would yield <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23undo() Online Documentation}*/
undo(): void;
/**
Gets the number of units that are contained in this compound unit.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23count Online Documentation}*/
readonly count: number;
/**
Gets or sets the current redo name.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23redoName Online Documentation}*/
redoName: string;
/**
Gets or sets the current redo name.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23undoName Online Documentation}*/
undoName: string;
/**
Gets whether this instance has been {@link CompositeUndoUnit#dispose disposed.}
An instance that has been disposed cannot be {@link CompositeUndoUnit#undo undone} or {@link CompositeUndoUnit#redo redone}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeUndoUnit%23disposed Online Documentation}*/
readonly disposed: boolean;
static $class: Class;
private $_CompositeUndoUnit: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CompositeUndoUnit;
}
export interface QueryContinueDragEventArgs extends EventArgs {
}
/**
The event argument class that is used by {@link DragSource#addQueryContinueDragListener QueryContinueDrag}
during drags.
The {@link QueryContinueDragEventArgs#action} property can be used to signal {@link DragAction#CONTINUE}, {@link DragAction#CANCEL} or {@link DragAction#DROP}
for the current drag operation.
The {@link QueryContinueDragEventArgs#dropTarget} property contains the current {@link DropTarget} during the drag, which may be null if there is currently no
{@link DropTarget}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryContinueDragEventArgs Online Documentation}*/
export class QueryContinueDragEventArgs {
/**
Gets or sets the action.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryContinueDragEventArgs%23action Online Documentation}*/
action: DragAction;
/**
Gets a value indicating whether the escape key had been pressed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryContinueDragEventArgs%23escapePressed Online Documentation}*/
escapePressed: boolean;
/**
Gets the current state of the keys.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryContinueDragEventArgs%23keyStates Online Documentation}*/
keyStates: DragDropKeyStates;
/**
* Gets or sets a value indicating whether this {@link QueryContinueDragEventArgs} is handled.
* Note that this property needs to be set before the function synchronously returns. This means that setting the value in
* an <code>async</code> function after the first <code>await</code> or in an asynchronous callback, it cannot not be seen by the emitter of the
* event and the sender will consider the event as not being handled.
*
* <pre>
* // wrong:
* dragSource.addQueryContinueDragListener(async (sender, args) => {
*   await something()
*   args.handled = true  // will be set *after* all other event listeners have been invoked, only
* })
*
* // correct:
* dragSource.addQueryContinueDragListener(async (sender, args) => {
*   args.handled = true // will be set immediately and 'mode' will see the change
*   await something()
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/QueryContinueDragEventArgs%23handled Online Documentation}
*/
handled: boolean;
/**
Gets the current {@link QueryContinueDragEventArgs#dropTarget} or null of there is no drop target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryContinueDragEventArgs%23dropTarget Online Documentation}*/
dropTarget: DropTarget | null;
static $class: Class;
private $_QueryContinueDragEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is QueryContinueDragEventArgs;
}
/**
Holds the possible drag actions.
@see QueryContinueDragEventArgs#action
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragAction Online Documentation}*/
export enum DragAction {
/**
Whether the drag should be continued.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragAction%23CONTINUE Online Documentation}*/
CONTINUE,
/**
Whether the drag should be finalized by an immediate drop.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragAction%23DROP Online Documentation}*/
DROP,
/**
Whether the drag operation should be canceled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragAction%23CANCEL Online Documentation}*/
CANCEL
}
/** Automatic conversion to {@link DragAction}. */
type DragActionStringValues = "continue" | "drop" | "cancel";
export interface DragSource extends YObject {
}
/**
Helper class for drag and drop support that is used for the source of drag and drop operations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource Online Documentation}*/
export class DragSource {
/**
Initializes a new instance of the {@link DragSource} class setting the {@link DragSource#source} property to the given element.
@param source The source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource Online Documentation}*/
constructor(source: HTMLElement | SVGElement);
/**
Cleans up by removing the association with the {@link DragSource#source} element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23cleanUp() Online Documentation}*/
cleanUp(): void;
/**
Raises the {@link DragSource#addQueryContinueDragListener QueryContinueDrag} event.
If the event is not {@link QueryContinueDragEventArgs#handled handled}, a default logic will be applied: the drag operation is canceled if the mouse button that
started the drag is released or escape has been pressed, otherwise the drag will be {@link DragAction#CONTINUE continued}.
@param evt The {@link QueryContinueDragEventArgs} instance containing the event data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23onQueryContinueDrag(QueryContinueDragEventArgs) Online Documentation}*/
onQueryContinueDrag(evt: QueryContinueDragEventArgs): void;
/**
Initiates a new drag operation using the provided drag data and effects.
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param dragData The data to use for the drag.
@param [dragDropEffects=DragDropEffects.ALL] The allowed effects.
@param [useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23startDrag(DragDropItem%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
startDrag(dragData: DragDropItem, dragDropEffects?: DragDropEffects | DragDropEffectsStringValues, useCssCursors?: boolean, dragPreview?: HTMLElement | SVGElement | null): void;
/**
Initiates a new drag operation using the provided drag data and effects.
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param options The parameters to pass.
@param options.dragData The data to use for the drag.
@param [options.dragDropEffects=DragDropEffects.ALL] The allowed effects.
@param [options.useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [options.dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23startDrag(DragDropItem%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
startDrag(options: {
dragData: DragDropItem;
dragDropEffects?: DragDropEffects;
useCssCursors?: boolean;
dragPreview?: HTMLElement | SVGElement | null;
}): void;
/**
Adds the given listener for the <code>QueryContinueDrag</code> event that occurs when the state of the {@link DragDropKeyStates}
or the {@link DropTarget} has changed to query the source whether the drag operation should be continued.
Handlers should adjust the {@link QueryContinueDragEventArgs#handled} and {@link QueryContinueDragEventArgs#action} properties accordingly.
@param listener The listener to add.
@see DragSource#removeQueryContinueDragListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23QueryContinueDrag Online Documentation}*/
addQueryContinueDragListener(listener: (sender: any, evt: QueryContinueDragEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryContinueDrag</code> event that occurs when the state of the {@link DragDropKeyStates}
or the {@link DropTarget} has changed to query the source whether the drag operation should be continued.
Handlers should adjust the {@link QueryContinueDragEventArgs#handled} and {@link QueryContinueDragEventArgs#action} properties accordingly.
@param listener The listener to remove.
@see DragSource#addQueryContinueDragListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23QueryContinueDrag Online Documentation}*/
removeQueryContinueDragListener(listener: (sender: any, evt: QueryContinueDragEventArgs) => void): void;
/**
Gets or sets the source element from which drags can originate.
If the {@link DragSource#allowDrag} property is enabled, this will automatically register mouse event handlers on the element that will recognize
default drag gestures (press followed by drag) to {@link DragSource#startDrag initiate a drag and drop operation}.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23source Online Documentation}*/
source: HTMLElement | SVGElement | null;
/**
Gets or sets a value indicating whether default drag gestures should be automatically recognized on the {@link DragSource#source}.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23allowDrag Online Documentation}*/
allowDrag: boolean;
/**
Gets or sets the data that will be used for the {@link DragSource#allowDrag automatic drag operation}.
If this data is not a {@link DragDropItem}, the {@link DropInputMode expected format} of the automatic drag operation will be <code>code</code>.
@see DragSource#effectAllowed
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23item Online Documentation}*/
item: any | null;
/**
Gets or sets the {@link DragDropEffects} that will be used for the {@link DragSource#allowDrag automatic drag operation}.
The effects, the default value is {@link DragDropEffects#ALL}.
@see DragSource#item
@default DragDropEffects.ALL
@see {@link https://docs.yworks.com/yfileshtml/#/api/DragSource%23effectAllowed Online Documentation}*/
effectAllowed: DragDropEffects;
static $class: Class;
private $_DragSource: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DragSource;
}
export interface DropTarget extends YObject {
}
/**
Helper class that plays the role of the target during a drag and drop operation that is initiated using {@link DragSource#startDrag}
in a {@link DragSource}.
During the drag operation, the current drop target can be obtained through the {@link QueryContinueDragEventArgs}
which is fired with the {@link DragSource#addQueryContinueDragListener QueryContinueDrag} event, when the drop
target changes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget Online Documentation}*/
export class DropTarget {
/**
Initializes a new instance of the {@link DropTarget} class.
Use the In order to make this instance available to the drag and drop mechanisms, the {@link DropTarget#setDropTarget}
method should be used.
@see DropTarget#setDropTarget
@see DropTarget#getDropTarget
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget Online Documentation}*/
constructor();
/**
Gets the {@link DropTarget} instance associated with the given element.
@param element The element to get the drop target instance of.
@returns The associated instance or <code>null</code>.
@see DropTarget#setDropTarget
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23getDropTarget(Element) Online Documentation}*/
static getDropTarget(element: Element): DropTarget | null;
/**
Sets the drop target for the given element.
@param element The element to register the drop target instance with.
@param dropTarget The instance to attach to the element or <code>null</code> to remove the association from the element.
@see DropTarget#getDropTarget
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23setDropTarget(Element%2CDropTarget) Online Documentation}*/
static setDropTarget(element: Element, dropTarget: DropTarget | null): void;
/**
Adds the given listener for the <code>DragEnter</code> event that occurs when a drag enters the drop target.
The handler may adjust the {@link DragEventArgs#dropEffect} and call {@link DragEventArgs#handled}.
@param listener The listener to add.
@see DropTarget#removeDragEnterListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23DragEnter Online Documentation}*/
addDragEnterListener(listener: (sender: any, evt: DragEventArgs) => void): void;
/**
Removes the given listener for the <code>DragEnter</code> event that occurs when a drag enters the drop target.
The handler may adjust the {@link DragEventArgs#dropEffect} and call {@link DragEventArgs#handled}.
@param listener The listener to remove.
@see DropTarget#addDragEnterListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23DragEnter Online Documentation}*/
removeDragEnterListener(listener: (sender: any, evt: DragEventArgs) => void): void;
/**
Adds the given listener for the <code>DragLeave</code> event that occurs when a drag leaves the drop target.
The handler may adjust the {@link DragEventArgs#dropEffect} and call {@link DragEventArgs#handled}.
@param listener The listener to add.
@see DropTarget#removeDragLeaveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23DragLeave Online Documentation}*/
addDragLeaveListener(listener: (sender: any, evt: DragEventArgs) => void): void;
/**
Removes the given listener for the <code>DragLeave</code> event that occurs when a drag leaves the drop target.
The handler may adjust the {@link DragEventArgs#dropEffect} and call {@link DragEventArgs#handled}.
@param listener The listener to remove.
@see DropTarget#addDragLeaveListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23DragLeave Online Documentation}*/
removeDragLeaveListener(listener: (sender: any, evt: DragEventArgs) => void): void;
/**
Adds the given listener for the <code>DragOver</code> event that occurs when a drag is being performed over the drop target.
The handler may adjust the {@link DragEventArgs#dropEffect} and call {@link DragEventArgs#handled}.
@param listener The listener to add.
@see DropTarget#removeDragOverListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23DragOver Online Documentation}*/
addDragOverListener(listener: (sender: any, evt: DragEventArgs) => void): void;
/**
Removes the given listener for the <code>DragOver</code> event that occurs when a drag is being performed over the drop target.
The handler may adjust the {@link DragEventArgs#dropEffect} and call {@link DragEventArgs#handled}.
@param listener The listener to remove.
@see DropTarget#addDragOverListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23DragOver Online Documentation}*/
removeDragOverListener(listener: (sender: any, evt: DragEventArgs) => void): void;
/**
Adds the given listener for the <code>Drop</code> event that occurs when the drop operation is performed on the drop target.
The handler may adjust the {@link DragEventArgs#dropEffect} and call {@link DragEventArgs#handled}.
@param listener The listener to add.
@see DropTarget#removeDropListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23Drop Online Documentation}*/
addDropListener(listener: (sender: any, evt: DragEventArgs) => void): void;
/**
Removes the given listener for the <code>Drop</code> event that occurs when the drop operation is performed on the drop target.
The handler may adjust the {@link DragEventArgs#dropEffect} and call {@link DragEventArgs#handled}.
@param listener The listener to remove.
@see DropTarget#addDropListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23Drop Online Documentation}*/
removeDropListener(listener: (sender: any, evt: DragEventArgs) => void): void;
/**
Gets or sets a value indicating whether dropping to this target is currently allowed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23allowDrop Online Documentation}*/
allowDrop: boolean;
/**
The element that is registered for this drop target.
If this drop target holds a {@link CanvasComponent}, {@link CanvasComponent#getComponent} can be used to
retrieve the actual {@link CanvasComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DropTarget%23element Online Documentation}*/
element: Element | null;
static $class: Class;
private $_DropTarget: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DropTarget;
}
export interface Future<T> extends YObject, IPropertyObservable {
}
/**
A helper holder class that can contain a {@link Future#value} of a specific type.
Typically {@link Future#hasValue} will be <code>false</code> until at a later point in time {@link Future#onValueDetermined} is called and the {@link Future#value} and {@link Future#hasValue} properties are being
updated and the {@link Future#addPropertyChangedListener PropertyChanged} event is triggered.
Type parameter <code>T</code>: The type of the {@link Future#value}
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Future Online Documentation}*/
export class Future<T> {
/**
Called when the {@link Future#value} property of this future has been determined.
This will set the {@link Future#hasValue} property to <code>true</code> and assign the {@link Future#value} property. Also the corresponding {@link Future#addPropertyChangedListener PropertyChanged} events
will be triggered if necessary.
@param value The value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Future%23onValueDetermined(T) Online Documentation}*/
onValueDetermined(value: T | null): void;
/**
Gets the value of this future, which may only be queried if {@link Future#hasValue} yields <code>true</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If this property is queried although {@link Future#hasValue} returns <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Future%23value Online Documentation}*/
value: T | null;
/**
Gets a value indicating whether this instance has been {@link Future#onValueDetermined assigned} a {@link Future#value}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Future%23hasValue Online Documentation}*/
readonly hasValue: boolean;
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
This event will be triggered for {@link Future#hasValue} changes and changes of the {@link Future#value} property. Note that if the determined future value is the
same as the default, the event will only be triggered for the {@link Future#hasValue} property.
@param listener The listener to add.
@see Future#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/Future%23PropertyChanged Online Documentation}*/
addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
This event will be triggered for {@link Future#hasValue} changes and changes of the {@link Future#value} property. Note that if the determined future value is the
same as the default, the event will only be triggered for the {@link Future#hasValue} property.
@param listener The listener to remove.
@see Future#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/Future%23PropertyChanged Online Documentation}*/
removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
static $class: Class;
private $_Future: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is Future<T>;
}
export interface ICompoundEdit {
}
/**
This interface is a means to bracket several
{@link IUndoUnit}s, that is a collection of units that are treated as one (for example they are always undone or
redone as a single unit).
{@link ICompoundEdit}s have two states: <i>active</i> and <i>settled</i>. Whenever an {@link ICompoundEdit} is retrieved from the {@link UndoEngine}
or the graph instance, it automatically is <i>active</i>. While an edit is in the <i>active</i> state, it gathers {@link IUndoUnit}s until
the {@link ICompoundEdit#commit} or {@link ICompoundEdit#cancel} methods are called. Calling the {@link ICompoundEdit#commit}
method commits all gathered {@link IUndoUnit}s to the {@link UndoEngine}, while canceling will {@link IUndoUnit#dispose dispose} them.
Calling either method will put the edit in the <i>settled</i> state. In this state, the edit does not gather units anymore and is not
reusable.
{@link ICompoundEdit}s can also be nested, in this case the changes of the inner edit are not committed directly to
the {@link UndoEngine} but to the outer edit. Only when the outer edit is committed, the units are committed to the
{@link UndoEngine}.
@see UndoEngine
@see IUndoUnit
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICompoundEdit Online Documentation}*/
export abstract class ICompoundEdit {
/**
This method cancels the compound editing process.
The recorded {@link IUndoUnit}s are {@link IUndoUnit#dispose disposed} and the edit is not usable anymore.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICompoundEdit%23cancel() Online Documentation}*/
abstract cancel(): void;
/**
This method commits the compound editing process.
The {@link IUndoUnit}s are enqueued into the corresponding {@link UndoEngine} instance and the edit is not
usable anymore.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICompoundEdit%23commit() Online Documentation}*/
abstract commit(): void;
static $class: Class;
private $_ICompoundEdit: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ICompoundEdit;
}
export interface ILookup {
}
/**
The basic lookup interface that allows for retrieving implementations for different aspects of the current instance.
The lookup idiom allows for dynamically providing, adding, changing, and removing implementation aspects of an instance
that would not be possible to achieve using ordinary object oriented methods.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookup Online Documentation}*/
export abstract class ILookup {
/**
Creates an implementation of the interface {@link ILookup} by using the given function as implementation for its {@link ILookup#lookup}
method.
@param lookup A function for {@link ILookup}'s single abstract method {@link ILookup#lookup}.
@returns An instance of the {@link ILookup} interface based on the given function.*/
static create(lookup: (type: Class) => any): ILookup;
/**
Creates an {@link ILookup} implementation that wraps an existing {@link ILookup} instance using an additional
lookup implementations.
For each lookup call that the <code>additionalLookup</code> yields <code>null</code>, the <code>basicLookup</code> will be queried instead.
@param basicLookup the fallback lookup implementation
@param additionalLookup the lookup that has precedence over the fallback
@returns an implementation that decorates the first with the latter
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookup%23createCascadingLookup(ILookup%2CILookup) Online Documentation}*/
static createCascadingLookup(basicLookup: ILookup, additionalLookup: ILookup): ILookup;
/**
Creates a dynamic {@link ILookup} implementation that for each type that the given subject can be assigned to
yields that subject.
@param subject the subject to yield if it is assignable to the type in the lookup query
@returns an instance that will either yield <code>subject</code> or <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookup%23createDynamicLookup(Object) Online Documentation}*/
static createDynamicLookup(subject: any): ILookup;
/**
Creates a simple mutable {@link ILookup} implementation that is backed by an {@link IMap}.
The {@link ILookup#lookup lookup} method uses the specified type as key when delegating to the <code>backingDictionary</code>.
@param backingDictionary The backing {@link HashMap} that is used in the {@link ILookup#lookup lookup} calls.
@returns A simple mutable {@link ILookup} implementation that is backed by a dictionary.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookup%23createMapLookup(HashMap%3CClass%2CObject%3E) Online Documentation}*/
static createMapLookup(backingDictionary: HashMap<Class, any>): ILookup;
/**
Creates a simple {@link ILookup} implementation that yields <code>subject</code> if <code>type</code> is queried.
@param subject the subject to yield
@param type the type that should yield the subject
@returns An {@link ILookup} implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookup%23createSingleLookup(Object%2CClass) Online Documentation}*/
static createSingleLookup(subject: any | null, type: Class): ILookup;
/**
Returns an instance that implements the given type or <code>null</code>.
Typically, this method will be called in order to obtain a different view or aspect of the current instance. This is
quite similar to casting or using a super type or interface of this instance, but is not limited to inheritance or
compile time constraints. An instance implementing this method is not required to return non-<code>null</code> implementations for
the types, nor does it have to return the same instance any time. Also it depends on the type and context whether the
instance returned stays up to date or needs to be reobtained for subsequent use.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
@param type The type for which an instance shall be returned.
@returns an instance that is assignable to type or <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookup%23lookup(Class) Online Documentation}*/
abstract lookup(type: Class): Object | null;
/**
An <code>ILookup</code> instance that always returns <code>null</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookup%23EMPTY Online Documentation}*/
static EMPTY: ILookup;
static $class: Class;
private $_ILookup: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILookup;
}
export interface IContextLookup {
}
/**
An interface for instances that can provide a {@link ILookup} capabilities for a given object.
This is useful in the case where lookup functionality is not intrinsic to an item or where an existing object cannot
implement the interface or when an item needs to be decorated externally with new lookup functionality.
@see ILookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookup Online Documentation}*/
export abstract class IContextLookup {
/**
Tries to create or retrieve an implementation of the given type for a given item.
@param item the item to lookup a type for
@param type the type to look up
@returns an implementation or <code>null</code>
@see ILookup#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookup%23contextLookup(Object%2CClass) Online Documentation}*/
abstract contextLookup(item: Object, type: Class): Object | null;
/**
Creates an implementation of the interface {@link IContextLookup} by using the given function as implementation for
its {@link IContextLookup#contextLookup} method.
@param contextLookup A function for {@link IContextLookup}'s single abstract method {@link IContextLookup#contextLookup}.
@returns An instance of the {@link IContextLookup} interface based on the given function.*/
static create(contextLookup: (item: any, type: Class) => any): IContextLookup;
/**
An <code>ILookupContext</code> instance that always returns <code>null</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookup%23EMPTY_CONTEXT_LOOKUP Online Documentation}*/
static EMPTY_CONTEXT_LOOKUP: IContextLookup | null;
static $class: Class;
private $_IContextLookup: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IContextLookup;
}
export interface IContextLookupChainLink extends IContextLookup {
}
/**
An interface for instances that can be used to chain lookup operations.
Implementations will perform the lookup in the {@link IContextLookup#contextLookup} method and delegate to the next
implementation in the chain if they cannot satisfy the request.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookupChainLink Online Documentation}*/
export abstract class IContextLookupChainLink {
/**
Factory method for a {@link IContextLookupChainLink lookup chain element} that simply adds an entry to a lookup chain.
@param type The type to add to the lookup.
@param instance The instance to return if the <code>type</code> is being looked up.
@returns An instance that can be used in a lookup chain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookupChainLink%23addingLookupChainLink(Class%2CObject) Online Documentation}*/
static addingLookupChainLink(type: Class, instance: any | null): IContextLookupChainLink;
/**
Factory method for a {@link IContextLookupChainLink lookup chain element} that adds an {@link ILookup} layer to a lookup chain.
If the lookup returns <code>null</code>, the call will be propagated to the chain.
@param lookup The lookup to use.
@returns An instance that can be used in a lookup chain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookupChainLink%23addingLookupChainLink(ILookup) Online Documentation}*/
static addingLookupChainLink(lookup: ILookup): IContextLookupChainLink;
/**
Factory method for a {@link IContextLookupChainLink lookup chain element} that adds an dynamic instance to a lookup chain.
The instance returned will check whether <code>instance</code> is of the queried type and will then return the instance.
@param instance The instance to query the type from dynamically.
@returns An instance that can be used in a lookup chain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookupChainLink%23addingLookupChainLink(Object) Online Documentation}*/
static addingLookupChainLink(instance: any): IContextLookupChainLink;
/**
Creates a lookup chain link that is backed by the given <code>callback</code>.
If the <code>callback</code> returns <code>null</code> the request is passed to the chain.
@param callback The callback to invoke.
@returns The implementation of the link that uses the <code>callback</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookupChainLink%23createContextLookupChainLink(function(Object%2CClass)%3AObject) Online Documentation}*/
static createContextLookupChainLink(callback: (subject: any | null, type: Class | null) => any | null): IContextLookupChainLink;
/**
Factory method for a {@link IContextLookupChainLink lookup chain element} that uses a function of type <code>function(</code><code>TContext</code><code>)</code> to yield a <code>TResult</code> for a specific <code>TContext</code>.
If the resulting link is {@link IContextLookup#contextLookup queried} for <code>TResult</code> and the context is of type <code>TContext</code>, the <code>factory</code> method will be queried and the result
will be returned. Otherwise the {@link IContextLookupChainLink#setNext next} chain link will be delegated the request to.
Type parameter <code>TContext</code>.
Type parameter <code>TResult</code>.
@param factory The factory delegate to use that yields the result for a given context.
@returns An instance that can be used in a lookup chain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookupChainLink%23factoryLookupChainLink(function(TContext)%3ATResult) Online Documentation}*/
static factoryLookupChainLink<TContext, TResult>(contextType: Class, resultType: Class, factory: (arg: TContext) => TResult): IContextLookupChainLink;
/**
Factory method for a {@link IContextLookupChainLink lookup chain element} that removes the entry from the lookup chain for a given type.
@param type The type to hide from the lookup.
@returns An instance that can be used in a lookup chain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookupChainLink%23hidingLookupChainLink(Class) Online Documentation}*/
static hidingLookupChainLink(type: Class): IContextLookupChainLink;
/**
This method is called by the framework to register the fallback lookup implementation that should be used during a call
to {@link IContextLookup#contextLookup}.
If the implementation cannot satisfy the query, it will use the provided context as a fallback. Note that
implementations can also use the results returned by the next lookup and decorate it appropriately.
@param next The context to use as a fallback.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IContextLookupChainLink%23setNext(IContextLookup) Online Documentation}*/
abstract setNext(next: IContextLookup): void;
static $class: Class;
private $_IContextLookupChainLink: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IContextLookupChainLink;
}
export interface ILookupDecorator {
}
/**
Interface that can be used to decorate the lookup result for a set of types.
@see ILookup
@see IContextLookupChainLink
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookupDecorator Online Documentation}*/
export abstract class ILookupDecorator {
/**
Adds a lookup implementation for a given type using a function of type <code>function(TContext, TResult)</code>.
This method is closely related to the {@link ILookupDecorator#add} method, however it passes the result of the
remaining chain to the delegate for convenient wrapping of the result. Note that if the implementation does not depend
on the previous result in order to wrap it, it is better to use that variant, in order to avoid the possibly costly
evaluation of the remainder of the lookup chain.
Type parameter <code>TContext</code>: The type of the context subject.
Type parameter <code>TResult</code>: The type of the query, which is a base type or the type of the result.
@param wrapperFactory The factory delegate that will be used to satisfy queries of type <code>TResult</code>
@param [decorateNull=true] Whether to actually decorate <code>null</code> results of remainder of the chain. If this is set to <code>false</code>, the <code>wrapperFactory</code> will
never be called with <code>null</code> as the second argument but the result of the query for this chain link will be the <code>null</code>
value.
@returns The {@link IContextLookupChainLink} instance as returned by the {@link ILookupDecorator#addLookup} call or
<code>null</code>, if either <code>decorator</code> was <code>null</code>, or it {@link ILookupDecorator#canDecorate could not decorate} the <code>TContext</code> type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookupDecorator%23add(function(TContext%2CTResult)%3ATResult%2Cboolean) Online Documentation}*/
add<TContext, TResult>(contextType: Class, resultType: Class, wrapperFactory: (arg1: TContext, arg2: TResult | null) => TResult | null, decorateNull?: boolean): IContextLookupChainLink | null;
/**
Adds a lookup implementation for a given type using a func.
Type parameter <code>TContext</code>: The type of the context subject.
Type parameter <code>TResult</code>: The type of the query, which is a base type or the type of the result.
@param factory The factory delegate that will be used to satisfy queries of type <code>TResult</code>.
@param [nullIsFallback=true] Whether to treat <code>null</code>-results of the <code>factory</code> as hints to use the remainder of the chain link or to actually yield the
value as the final result.
@returns The {@link IContextLookupChainLink} instance as returned by the {@link ILookupDecorator#addLookup} call or
<code>null</code>, if either <code>decorator</code> was <code>null</code>, or it {@link ILookupDecorator#canDecorate could not decorate} the <code>TContext</code> type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookupDecorator%23add(function(TContext)%3ATResult%2Cboolean) Online Documentation}*/
add<TContext, TResult>(contextType: Class, resultType: Class, factory: (arg: TContext) => TResult | null, nullIsFallback?: boolean): IContextLookupChainLink | null;
/**
Add a lookup implementation for a given type that always yields a constant result.
Type parameter <code>TContext</code>: The type of the context subject.
Type parameter <code>TResult</code>: The type of the query, which is a base type or the type of the result.
@param result The constant value to yield if the chain is queried for an implementation of type <code>TResult</code>.
@returns The {@link IContextLookupChainLink} instance as returned by the {@link ILookupDecorator#addLookup} call or
<code>null</code>, if either <code>decorator</code> was <code>null</code>, or it {@link ILookupDecorator#canDecorate could not decorate} the <code>TContext</code> type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookupDecorator%23addConstant(TResult) Online Documentation}*/
addConstant<TContext, TResult>(contextType: Class, resultType: Class, result: TResult | null): IContextLookupChainLink | null;
/**
Adds a lookup chain element to the chain of lookups for a given type.
The lookup chain is context specific. This method will not generally decorate the lookup of all instances of type <code>t</code>, but
only those that stand in context of this instance. This method will only succeed if {@link ILookupDecorator#canDecorate}
yields <code>true</code> for <code>t</code>.
@param t The type to decorate the lookup.
@param lookup The chain element to add to the lookup of the given type.
@see ILookupDecorator#removeLookup
@see ILookupDecorator#add
@see ILookupDecorator#add
@see ILookupDecorator#addConstant
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookupDecorator%23addLookup(Class%2CIContextLookupChainLink) Online Documentation}*/
abstract addLookup(t: Class, lookup: IContextLookupChainLink): void;
/**
Determines whether this instance can be used to decorate the lookup for a certain type.
@param t The type to decorate the lookup for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookupDecorator%23canDecorate(Class) Online Documentation}*/
abstract canDecorate(t: Class): boolean;
/**
Removes a previously registered lookup chain element from the chain of lookups for a given type.
@param t The type to remove the decorator from.
@param lookup The chain element to remove from the lookup of the given type.
@see ILookupDecorator#addLookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILookupDecorator%23removeLookup(Class%2CIContextLookupChainLink) Online Documentation}*/
abstract removeLookup(t: Class, lookup: IContextLookupChainLink): void;
static $class: Class;
private $_ILookupDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILookupDecorator;
}
export interface ITagOwner {
}
/**
Interface implemented by items that can be tagged with arbitrary objects.
The object held by implementations of this interface is usually a user defined object. Implementations don't depend on
this tag. They just serve as a storage for the tag that will be used by other parts of the implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITagOwner Online Documentation}*/
export abstract class ITagOwner {
/**
* Gets or sets the tag associated with this instance.
* The tag is an optional user defined object which can be used to store arbitrary data related to this item. The item
* itself does not normally depend on the tag associated with it. It serves as storage for the object only.
*
* <br><b>Setting a model item's tag</b>
* <pre>
* owner.tag = newTag
* </pre>
*
* <br><b>Getting the tag from a model item</b>
* <pre>
* const tag = owner.tag
* </pre>
*
* Tags are presented in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph_model-connecting_business_models The Graph Model}.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ITagOwner%23tag Online Documentation}
*/
abstract tag: any | null;
static $class: Class;
private $_ITagOwner: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ITagOwner;
}
export interface LookupChain extends YObject, IContextLookup, IPropertyObservable {
}
/**
An implementation of a chain of {@link IContextLookupChainLink}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain Online Documentation}*/
export class LookupChain {
constructor();
/**
A convenience method that adds a {@link IContextLookupChainLink} to the given chain that is based on a function of
type <code>function(TContext, TResult)</code>.
Depending on <code>decorateNull</code>, <code>null</code> results of the underlying chain will be passed to the
<code>wrapperFactory</code>, or simply yielded as the final result.
Type parameter <code>TContext</code>: The type of the context.
Type parameter <code>TResult</code>: The type of the result.
@param resultType The type of the result.
@param contextType The type of the context.
@param wrapperFactory The wrapper factory that will be called for queries to <code>TResult</code> to wrap the result of the query to the remainder of the chain.
@param [decorateNull=false] if set to <code>true</code> <code>null</code> return values of the remainder of the chain will be passed to the wrapper factory so that it can
decorate the null, otherwise null will be yielded as the final result.
@returns The link that has been registered with the chain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23add(Class%2CClass%2Cfunction(TContext%2CTResult)%3ATResult%2Cboolean) Online Documentation}*/
add<TContext, TResult>(contextType: Class, resultType: Class, wrapperFactory: (arg1: TContext, arg2: TResult | null) => TResult | null, decorateNull?: boolean): IContextLookupChainLink;
/**
A convenience method that adds a {@link IContextLookupChainLink} to the given chain that is based on a function of
type <code>function(</code><code>TContext</code><code>)</code>.
Depending on <code>nullIsFallback</code>, <code>null</code> return values of the <code>factory</code> will be interpreted as the final result or indicate to
the implementation that the chain should be queried for the result.
Type parameter <code>TContext</code>: The type of the context.
Type parameter <code>TResult</code>: The type of the result.
@param resultType The type of the result.
@param contextType The type of the context.
@param factory The factory that will be called for queries to <code>TResult</code>.
@param [nullIsFallback=true] If set to <code>true</code>, <code>null</code> return values of the factory method will be interpreted as the final result, otherwise the request
will propagate down the chain.
@returns The link that has been registered with the chain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23add(Class%2CClass%2Cfunction(TContext)%3ATResult%2Cboolean) Online Documentation}*/
add<TContext, TResult>(contextType: Class, resultType: Class, factory: (arg: TContext) => TResult | null, nullIsFallback?: boolean): IContextLookupChainLink;
/**
Adds the given link to the end of the chain.
@param lookup The link to add.
@see LookupChain#remove
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23add(IContextLookupChainLink) Online Documentation}*/
add(lookup: IContextLookupChainLink): void;
/**
{@link LookupChain#add Adds} a lookup implementation that always yields a constant <code>result</code> to the chain.
Type parameter <code>TResult</code>: The type of the query, which is a base type or the type of the result.
@param result The constant value to yield if the chain is queried for an implementation of type <code>TResult</code>.
@returns The {@link IContextLookupChainLink} instance that has been added to the chain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23addConstant(TResult) Online Documentation}*/
addConstant<TResult>(resultType: Class, result: TResult | null): IContextLookupChainLink;
/**
Performs the actual lookup operation.
This call will traverse the chain to satisfy the query.
@param item The item to use as the context.
@param type The type of the query.
@returns The result of the query.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23contextLookup(Object%2CClass) Online Documentation}*/
contextLookup(item: Object, type: Class): Object | null;
/**
Can be called by client code to trigger the {@link LookupChain#addPropertyChangedListener PropertyChanged}
event manually.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23notifyChange() Online Documentation}*/
notifyChange(): void;
/**
Notifies client code that the lookup of this instance might have changed.
@param evt The arguments
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23onPropertyChanged(PropertyChangedEventArgs) Online Documentation}*/
onPropertyChanged(evt: PropertyChangedEventArgs): void;
/**
Removes the link that has previously been added to this instance from this chain.
@param link The link to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23remove(IContextLookupChainLink) Online Documentation}*/
remove(link: IContextLookupChainLink): void;
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when the lookup of this instance has changed for some
or all of the types or items.
@param listener The listener to add.
@see LookupChain#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23PropertyChanged Online Documentation}*/
addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when the lookup of this instance has changed for
some or all of the types or items.
@param listener The listener to remove.
@see LookupChain#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LookupChain%23PropertyChanged Online Documentation}*/
removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
static $class: Class;
private $_LookupChain: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LookupChain;
}
export interface UndoEngine extends YObject, IPropertyObservable {
}
/**
* The main class to provide undo and redo functionality.
* <br><b>How to enable the undo support</b>
* <pre>
* graph.undoEngineEnabled = true
* </pre>
*
* The {@link UndoEngine} is in charge of {@link UndoEngine#beginCompoundEdit recording}, {@link UndoEngine#addUnit managing} and {@link UndoEngine#undo executing} single units of work that can be undone
* and redone, represented by the interface {@link IUndoUnit}.
*
* <br><b>How to get the UndoEngine from the IGraph</b>
* <pre>
* const engine = graph.undoEngine
* </pre>
*
* These units are <b>managed</b> in a queue-like structure and can be added manually to the queue by calling the {@link UndoEngine#addUnit}
* method.
*
* <br><b>How to add an undo unit to the engine</b>
* <pre>
* engine.addUnit(myUnit)
* </pre>
*
* <b>Executing</b> the units is done by client code or commands by calling {@link UndoEngine#undo} or
* {@link UndoEngine#redo}. The engine automatically manages the queue so that calling these two methods will
* consistently move the units in the queue.
*
* The {@link UndoEngine#beginCompoundEdit} method allows to <b>record</b> units by bracketing several changes in an
* {@link ICompoundEdit}. Implementations of the {@link ICompoundEdit} interface record all subsequent changes
* until it is {@link ICompoundEdit#commit committed} and therefore all recorded units are added to the engine. For more information and examples see
* {@link ICompoundEdit}.
*
* <br><b>Using an ICompoundEdit</b>
* <pre>
* const edit = engine.beginCompoundEdit(undoName, redoName)
*
* // add custom undo units or change the graph structure programmatically here
*
* if (!success) {
*   // if we don't want the changes to be done after all, then we need to cancel the edit
*   edit.cancel()
* }
* </pre>
*
* Depending on the properties {@link UndoEngine#mergeUnits} and {@link UndoEngine#autoMergeTime}, the engine tries to collapse added {@link IUndoUnit}s if possible. This enhances
* the performance and reduces the required memory for undo changes. For example, when a node is moved interactively,
* instead of saving every position that the node had when being dragged the engine will only hold one {@link IUndoUnit}
* at the end to undo the entire movement.
*
* <b>Customizing Undo/Redo</b>
*
* <ul>
* <li>In general, to be able to undo or redo certain work client code can <b>implement {@link IUndoUnit} or subclass
*     {@link UndoUnitBase UndoUnitBase}</b>. The client application needs to encapsulate
* changes in this implementation and provide the logic to completely undo and redo these changes. Instances of your custom
* {@link IUndoUnit} implementations need to be added to the {@link UndoEngine} manually via
* {@link UndoEngine#addUnit}.</li>
* <li>In certain cases it is difficult or not efficient to track and save the changes between certain states, and better to
* save the states themselves and to restore these. For example when there are potentially many intermediate steps between
* two states of interest and it would be complicated to produce {@link IUndoUnit}s for each intermediate step and
* merge them in the end. In such cases client code should <b>implement {@link IMementoSupport}</b> instead. See the documentation of {@link IMementoSupport}
* for examples and guidelines on how to implement this memento design pattern concept.</li>
* <li><b>It is generally not necessary to subclass {@link UndoEngine}</b> unless you want to customize the internal handling of the units, which is both complicated and prone to errors. In most
* use cases it is sufficient to provide custom {@link IUndoUnit}s or {@link IMementoSupport}.</li>
* </ul>
* @see IUndoUnit
* @see UndoUnitBase
* @see IMementoSupport
* @see ICompoundEdit
* @see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine Online Documentation}
*/
export class UndoEngine {
/**
Initializes a new instance of the {@link UndoEngine} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine Online Documentation}*/
constructor();
/**
Adds a new {@link IUndoUnit} to the queue.
This implementation will automatically group multiple units into a single unit if the time since the last add is less
than {@link UndoEngine#autoMergeTime}.
@param unit The unit of work to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23addUnit(IUndoUnit) Online Documentation}*/
addUnit(unit: IUndoUnit): void;
/**
Begins a compound edit that will use the provided name.
This will create a new edit that can independently be {@link ICompoundEdit#cancel canceled} or {@link ICompoundEdit#commit committed}. Note that only if the outer-most instances
is committed, the corresponding {@link IUndoUnit} units will be enqueued into this instance.
@param undoName The undo name to use for the edit.
@param redoName The redo name to use for the edit.
@returns A compound edit implementation that needs to be {@link ICompoundEdit#cancel canceled} or {@link ICompoundEdit#commit committed} later.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23beginCompoundEdit(string%2Cstring) Online Documentation}*/
beginCompoundEdit(undoName: string | null, redoName: string | null): ICompoundEdit;
/**
Determines whether a call to {@link UndoEngine#redo} can be made.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23canRedo() Online Documentation}*/
canRedo(): boolean;
/**
Determines whether a call to {@link UndoEngine#undo} can be made.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23canUndo() Online Documentation}*/
canUndo(): boolean;
/**
Clears the internal queue and {@link IUndoUnit#dispose disposes} all units in it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23clear() Online Documentation}*/
clear(): void;
/**
Returns a token that can be used to store and compare the state of the undo queue.
E.g. an application can retrieve the token once the user has saved his document. Comparing the token returned by this
instance with another one retrieved at a later point in time enables the application to determine whether the document
is in the same state.
@returns An object that can be checked against other tokens via the {@link YObject#equals} method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23getToken() Online Documentation}*/
getToken(): any;
/**
This will trigger the corresponding {@link UndoEngine#addPropertyChangedListener PropertyChanged} event.
@param name The name of the property that changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23onPropertyChanged(string) Online Documentation}*/
onPropertyChanged(name: string): void;
/**
Redoes the next {@link IUndoUnit}.
@throws {Exception} <b>(InvalidOperationError)</b> If an undo operation is already in progress or if {@link UndoEngine#canRedo} would yield <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23redo() Online Documentation}*/
redo(): void;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23toString() Online Documentation}*/
toString(): string;
/**
Undoes the next {@link IUndoUnit}.
@throws {Exception} <b>(InvalidOperationError)</b> If an undo operation is already in progress or if {@link UndoEngine#canUndo} would yield <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23undo() Online Documentation}*/
undo(): void;
/**
Gets or sets a value that indicates whether or not this instance should try to merge newly added units.
If <code>true</code> this instance will try to {@link IUndoUnit#tryMergeUnit merge} or {@link IUndoUnit#tryReplaceUnit replace} units in the queue. Note that when the {@link UndoEngine#autoMergeTime} property is set to {@link TimeSpan#ZERO}
and this property is set to <code>true</code>, the engine will always try to merge incoming units. Otherwise the {@link UndoEngine#autoMergeTime} dictates whether
the engine will try to merge a unit or not.
@see UndoEngine#autoMergeTime
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23mergeUnits Online Documentation}*/
mergeUnits: boolean;
/**
Gets or sets the duration during which the engine will try to merge newly added units.
If this is set to {@link TimeSpan#ZERO}, the engine will try to merge incoming units depending on the {@link UndoEngine#mergeUnits} property.
Otherwise, if the time span between the last added unit and the new unit exceeds the set value, the engine does not try
to merge the units regardless what the {@link UndoEngine#mergeUnits} property is set to. Likewise, if the time span is smaller than the set value,
the engine will always try to merge units.
@see UndoEngine#mergeUnits
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23autoMergeTime Online Documentation}*/
autoMergeTime: TimeSpan;
/**
Gets or sets the maximum size of the undo queue this instance is managing.
A size of <code>0</code> effectively disables this implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23size Online Documentation}*/
size: number;
/**
Gets the {@link IUndoUnit#undoName} of the next {@link UndoEngine#undo} operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23undoName Online Documentation}*/
readonly undoName: string;
/**
Gets the {@link IUndoUnit#redoName} of the next {@link UndoEngine#redo} operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23redoName Online Documentation}*/
readonly redoName: string;
/**
Gets whether this instance is currently performing an undo operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23performingUndo Online Documentation}*/
readonly performingUndo: boolean;
/**
Gets whether this instance is currently performing a redo operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23performingRedo Online Documentation}*/
readonly performingRedo: boolean;
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when {@link UndoEngine#canUndo},
{@link UndoEngine#canRedo}, {@link UndoEngine#undoName}, or {@link UndoEngine#redoName} changed its value.
@param listener The listener to add.
@see UndoEngine#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23PropertyChanged Online Documentation}*/
addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when {@link UndoEngine#canUndo},
{@link UndoEngine#canRedo}, {@link UndoEngine#undoName}, or {@link UndoEngine#redoName} changed its value.
@param listener The listener to remove.
@see UndoEngine#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23PropertyChanged Online Documentation}*/
removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Adds the given listener for the <code>UnitUndone</code> event that occurs when the engine has successfully executed the {@link IUndoUnit#undo}
operation of an {@link IUndoUnit}.
@param listener The listener to add.
@see UndoEngine#removeUnitUndoneListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23UnitUndone Online Documentation}*/
addUnitUndoneListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>UnitUndone</code> event that occurs when the engine has successfully executed the {@link IUndoUnit#undo}
operation of an {@link IUndoUnit}.
@param listener The listener to remove.
@see UndoEngine#addUnitUndoneListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23UnitUndone Online Documentation}*/
removeUnitUndoneListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>UnitRedone</code> event that occurs when the engine has successfully executed the {@link IUndoUnit#redo}
operation of an {@link IUndoUnit}.
@param listener The listener to add.
@see UndoEngine#removeUnitRedoneListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23UnitRedone Online Documentation}*/
addUnitRedoneListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>UnitRedone</code> event that occurs when the engine has successfully executed the {@link IUndoUnit#redo}
operation of an {@link IUndoUnit}.
@param listener The listener to remove.
@see UndoEngine#addUnitRedoneListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoEngine%23UnitRedone Online Documentation}*/
removeUnitRedoneListener(listener: (sender: any, evt: EventArgs) => void): void;
static $class: Class;
private $_UndoEngine: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is UndoEngine;
}
export interface IUndoUnit {
}
/**
Represents a unit of work that can be undone and redone.
Clients should use this interface when certain actions, changes or events should be monitored and have undoability
provided for them. If you simply want to track the state of certain items between two states of interest, you should
consider using {@link IMementoSupport mementos} instead.
The central methods of this interface are {@link IUndoUnit#undo} and {@link IUndoUnit#redo} which contain the
logic to undo or redo a unit of work. Keep in mind that undo/redo are sensible operations and should maintain a
consistent state before and after each operation since they may be executed potentially often one after another.
The requirement for an {@link IUndoUnit} is that when a program is in a certain state and a call to {@link IUndoUnit#undo}
is followed by a call to {@link IUndoUnit#redo} then the program is in the exact same state as before (the same
holds true for the other way around).
A default implementation of this interface is the abstract class {@link UndoUnitBase} which forces only the
implementation of the undo/redo logic and defaults the other methods. Clients should primarily extend this class instead
of implementing the whole interface.
IUndoUnits are managed by the {@link UndoEngine}. Custom units can always be added to the engine using the method
{@link UndoEngine#addUnit}. Also consider to use one of the following implementations: {@link DelegateUndoUnit} takes delegates for the
undo/redo operations and {@link CompositeUndoUnit} creates a bracketing unit comprising several other units.
Also note that in order to keep a consistent state, methods of objects of this type should not be called by client code
directly but use the {@link UndoEngine} instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IUndoUnit Online Documentation}*/
export abstract class IUndoUnit {
/**
Called by the {@link UndoEngine} and client code to let the unit dispose of references aren't needed anymore.
When this method is called, the other methods of this interface will not be called anymore by the undo engine. The undo
engine calls this method on instances that it will not use anymore to enable the instances to release instances and
memory actively.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IUndoUnit%23dispose() Online Documentation}*/
abstract dispose(): void;
/**
Redoes the work that is represented by this unit.
Undo/redo are sensible operations and should maintain a consistent state before and after each operation since they may
be executed potentially often one after another.
@see IUndoUnit#redoName
@see {@link https://docs.yworks.com/yfileshtml/#/api/IUndoUnit%23redo() Online Documentation}*/
abstract redo(): void;
/**
Tries to merge the given unit with this one.
This method is called when <code>this</code> unit is the head of an {@link UndoEngine}'s or {@link CompositeUndoUnit}'s
queue and a new unit is added. It is meant to try to incorporate the change of the given <code>unit</code> into <code>this</code> and if
successful return <code>true</code>. This should be the case when the end state of <code>this</code> unit is equal to the start state of the given
<code>unit</code>.
For example, if <code>this</code> unit is the head of the queue and represents the work from state <code>A</code> to state <code>B</code> and the given unit
the work from state <code>B</code> to state <code>C</code>, then this method should try to make <code>this</code> unit shift from state <code>A</code> to state <code>C</code>. If this
is successful, the method is expected to return <code>true</code>. The other unit is then {@link IUndoUnit#dispose disposed} by the {@link UndoEngine}
afterwards.
Clients don't necessarily have to implement this method if the unit doesn't happen very often. In fact, the default
implementation of {@link UndoUnitBase#tryMergeUnit} simply returns <code>false</code>. Implementing this method faithfully will
cause multiple units of work to inseparably appear as one and undoing/redoing it will undo/redo the work of all
collapsed units. Depending on the situation this may be reasonable, for example when there are potentially a lot of
changes where not every intermediate step is required to be recorded. In this case implementing this method faithfully
will greatly improve the performance and reduce the required amount of memory of the undo process. If you want to group
together multiple units as a single block but still want to be able to separate each step, consider to use a {@link CompositeUndoUnit}
instead.
@param unit The unit to incorporate that happened after this unit.
@returns Whether the state change of <code>unit</code> has been incorporated into this unit and <code>unit</code> can be disposed of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IUndoUnit%23tryMergeUnit(IUndoUnit) Online Documentation}*/
abstract tryMergeUnit(unit: IUndoUnit): boolean;
/**
Tries to replace the given unit with this one.
This method is called if a newly added unit couldn't be {@link IUndoUnit#tryMergeUnit merged} with the head unit of the queue (i.e. returned false).
Instead of trying to merge a newly added unit, this method tries to replace the given unit (which is the head of the
queue) with <code>this</code> by incorporating the change of the given <code>unit</code> into <code>this</code> and if successful return <code>true</code>. This should be
the case when the start state of <code>this</code> unit is equal to the end state of the given <code>unit</code>.
For example, if the given <code>unit</code> represents the work from state <code>A</code> to state <code>B</code> and <code>this</code> unit the work from state <code>B</code> to state
<code>C</code>, then this method should try to make <code>this</code> unit shift from state <code>A</code> to state <code>C</code>. If this is successful, the method is
expected to return <code>true</code>. The given <code>unit</code> is then {@link IUndoUnit#dispose disposed} by the {@link UndoEngine} afterwards and replaced with <code>this</code>
unit in the queue.
Clients don't necessarily have to implement this method if the unit doesn't happen very often. In fact, the default
implementation of {@link UndoUnitBase#tryReplaceUnit} simply returns <code>false</code>. Implementing this method faithfully
will cause multiple units of work to inseparably appear as one and undoing/redoing it will undo/redo the work of all
collapsed units. Depending on the situation this may be reasonable, for example when there are potentially a lot of
changes where not every intermediate step is required to be recorded. In this case implementing this method faithfully
will greatly improve the performance and reduce the required amount of memory of the undo process. If you want to group
together multiple units as a single block but still want to be able to separate each step, consider to use a {@link CompositeUndoUnit}
instead.
@param unit The unit to incorporate that happened before this unit.
@returns Whether the state change of <code>this</code> unit has been incorporated into the given <code>unit</code> and <code>this</code> can be disposed of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IUndoUnit%23tryReplaceUnit(IUndoUnit) Online Documentation}*/
abstract tryReplaceUnit(unit: IUndoUnit): boolean;
/**
Undoes the work that is represented by this unit.
Undo/redo are sensible operations and should maintain a consistent state before and after each operation since they may
be executed potentially often one after another.
@see IUndoUnit#undoName
@see {@link https://docs.yworks.com/yfileshtml/#/api/IUndoUnit%23undo() Online Documentation}*/
abstract undo(): void;
/**
Gets the name of the undo unit.
Depending on the implementation and context this might be a human readable representation of the undo action or a
symbolic name that needs localization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IUndoUnit%23undoName Online Documentation}*/
abstract readonly undoName: string;
/**
Gets the name of the redo unit.
Depending on the implementation and context this might be a human readable representation of the redo action or a
symbolic name that needs localization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IUndoUnit%23redoName Online Documentation}*/
abstract readonly redoName: string;
static $class: Class;
private $_IUndoUnit: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IUndoUnit;
}
export interface IMementoSupport {
}
/**
* This interface provides access to the memento design pattern to provide undoability for arbitrary models.
* By implementing this interface as well as {@link ILookup}, clients can add undoability for changes to their model
* classes. The yFiles undo mechanism uses the return value of {@link IMementoSupport#getState} to retrieve a state of
* an item at the beginning of the compound editing process. When the process ends, another state will be retrieved of the
* same item and compared to the original state. If they differ, an {@link IUndoUnit} is created that uses the {@link IMementoSupport#applyState}
* method to apply either state to the item in case of {@link IUndoUnit#undo undo} or {@link IUndoUnit#redo redo}. This represents an abstraction to the undo mechanism
* where it is only needed to define "states" of items and hides the more complicated mechanism of creating and inserting
* {@link IUndoUnit}s.
*
* The following is an example implementation of an item that is being managed using {@link IMementoSupport}:
*
* <br><b>A sample business object that implements ILookup to return IMementoSupport</b>
* <pre>
* class Employee extends BaseClass(ILookup) {
*   constructor(name, position, age) {
*     super()
*     this._name = name
*     this.position = position
*     this.age = age
*   }
*
*   get name() {
*     return this._name
*   }
*
*   lookup(type) {
*     if (type === IMementoSupport.$class) {
*       return new EmployeeMementoSupport()
*     }
*     return null
*   }
* }
* </pre>
*
* A collection of items from this type can then be watched using the following code snippet:
*
* <br><b>Using an ICompoundEdit</b>
* <pre>
* const edit = graph.beginEdit(undoName, redoName, listWithMyEmployeesToWatch)
*
* // changes to the employees are done here
*
* if (!success) {
*   // if we don't want the changes to be done after all, then we need to cancel the edit
*   edit.cancel()
* }
* </pre>
*
* Implementing the {@link IMementoSupport} interface is quite unrestrained, the type of the state returned by {@link IMementoSupport#getState}
* method can by anything as long as the {@link IMementoSupport#applyState} and {@link IMementoSupport#stateEquals}
* methods can deal with it:
*
* <br><b>Sample implementation of IMementoSupport</b>
* <pre>
* class EmployeeMementoSupport extends BaseClass(IMementoSupport) {
*   getState(subject) {
*     if (subject instanceof Employee) {
*       return new EmployeeState(subject.position, subject.age)
*     }
*     return null
*   }
*
*   applyState(subject, state) {
*     if (subject instanceof Employee && state instanceof EmployeeState) {
*       subject.position = state.position
*       subject.age = state.age
*     }
*   }
*
*   stateEquals(state1, state2) {
*     if (state1 instanceof EmployeeState && state2 instanceof EmployeeState) {
*       return state1.position === state2.position && state1.age === state2.age
*     }
*     return state1 === state2
*   }
* }
* class EmployeeState {
*   constructor(position, age) {
*     this._position = position
*     this._age = age
*   }
*
*   get position() {
*     return this._position
*   }
*
*   get age() {
*     return this._age
*   }
* }
* </pre>
*
* In summary, use this concept when you want to track the state of items during certain operations for undo/redo. This is
* efficient if it's easier to handle an item's state than the changes to the item themselves. If you want to focus on the
* changes or on certain events, you should use custom {@link IUndoUnit} implementations instead.
*
* @see ILookup
* @see ICompoundEdit
* @see UndoEngine
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IMementoSupport Online Documentation}
*/
export abstract class IMementoSupport {
/**
Reapplies a previously queried state object to a given subject.
The state object has been created using the {@link IMementoSupport#getState} method.
@param subject The subject to modify the state.
@param state The state object as returned by {@link IMementoSupport#getState}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMementoSupport%23applyState(Object%2CObject) Online Documentation}*/
abstract applyState(subject: any, state: any): void;
/**
Creates an implementation of the interface {@link IMementoSupport} from the given definition.
The given object must contain exactly the abstract members of the {@link IMementoSupport} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IMementoSupport} interface.
@returns An instance of the {@link IMementoSupport} interface based on the given definition.*/
static create(interfaceDefinition: {
getState(subject: any): any;
applyState(subject: any, state: any): void;
stateEquals(state1: any, state2: any): boolean;
}): IMementoSupport;
/**
Retrieves an object representing the state at the moment this method is called.
The returned state may be reapplied later to the subject in the {@link IMementoSupport#applyState} method.
@param subject The subject to read the state from
@returns An object that describes the current state of <code>subject</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMementoSupport%23getState(Object) Online Documentation}*/
abstract getState(subject: any): any;
/**
Determines whether two state objects are equal.
The undo mechanism in yFiles calls this method to determine if the subject has changed and whether an {@link IUndoUnit}
should be created for these changes. If this method returns <code>false</code>, an {@link IUndoUnit} is created for the two
states that can reapply either state (for either undo or redo). If this method returns <code>true</code> the state is considered not
to have changed and no undo unit will be created. Conservative implementations my simply return <code>false</code>.
@param state1 The first state as obtained from {@link IMementoSupport#getState}
@param state2 The second state as obtained from {@link IMementoSupport#getState}
@returns <code>true</code> if the states are equal; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IMementoSupport%23stateEquals(Object%2CObject) Online Documentation}*/
abstract stateEquals(state1: any, state2: any): boolean;
static $class: Class;
private $_IMementoSupport: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IMementoSupport;
}
export interface DelegateUndoUnit<T> extends UndoUnitBase {
}
/**
An implementation of the {@link IUndoUnit} interface that uses simple delegates for the actual undo and redo
operations.
Type parameter <code>T</code>: The type of the single parameter that is passed to the delegate.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegateUndoUnit Online Documentation}*/
export class DelegateUndoUnit<T> {
/**
Creates a new instance using the provided name and a undo and redo handler as well as the parameters to pass to the
delegates.
@param undoName The name of the undo.
@param undo The undo handler delegate.
@param redo The redo handler delegate.
@param undoParam The undo parameter to pass to the delegate.
@param redoParam The redo parameter to pass to the delegate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegateUndoUnit%23DelegateUndoUnit(string%2Cfunction(T)%3Avoid%2Cfunction(T)%3Avoid%2CT%2CT) Online Documentation}*/
constructor(undoName: string, undo: (param: T) => void, redo: (param: T) => void, undoParam: T, redoParam: T);
/**
Creates a new instance using the provided name and a undo/redo handler as well as the parameters to pass to the
delegates.
@param undoName The name of the undo.
@param undoRedo The undo and redo handler delegate.
@param undoParam The undo parameter to pass to the delegate.
@param redoParam The redo parameter to pass to the delegate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegateUndoUnit%23DelegateUndoUnit(string%2Cfunction(T)%3Avoid%2CT%2CT) Online Documentation}*/
constructor(undoName: string, undoRedo: (param: T) => void, undoParam: T, redoParam: T);
/**
Calls the redo delegate using the redo parameter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegateUndoUnit%23redo() Online Documentation}*/
redo(): void;
/**
Calls the undo delegate using the undo parameter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegateUndoUnit%23undo() Online Documentation}*/
undo(): void;
static $class: Class;
private $_DelegateUndoUnit: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is DelegateUndoUnit<T>;
}
export interface UndoUnitBase extends YObject, IUndoUnit {
/**
@inheritdoc*/
dispose(): void;
/**
@inheritdoc*/
tryMergeUnit(unit: IUndoUnit): boolean;
/**
@inheritdoc*/
tryReplaceUnit(unit: IUndoUnit): boolean;
/**
@inheritdoc*/
readonly undoName: string;
/**
@inheritdoc*/
readonly redoName: string;
} /**
An abstract base implementation of {@link IUndoUnit}.
This class defaults most methods required by the {@link IUndoUnit} interface. Subclasses only have to implement the
undo/redo logic. Most notably, {@link IUndoUnit#tryMergeUnit} and {@link IUndoUnit#tryReplaceUnit} are
implemented to return
<code>false</code>. Subclasses are of course allowed to override more methods if clients want to enable additional functionality.
Use this class for simple custom implementations of {@link IUndoUnit}.
@see IUndoUnit
@see UndoEngine
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoUnitBase Online Documentation}*/
export abstract class UndoUnitBase {
/**
Creates a new instance using the given name as undo and redo name.
@param undoName The undo name.
@param [redoName=null] The redo name.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoUnitBase Online Documentation}*/
constructor(undoName: string, redoName?: string | null);
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/UndoUnitBase%23toString() Online Documentation}*/
toString(): string;
static $class: Class;
private $_UndoUnitBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is UndoUnitBase;
}
export interface AdjacencyGraphBuilder extends YObject {
}
/**
* Populates a graph from custom data where node data items know about their predecessors and/or successors.
* This class can be used when the data consists of one or more collections of nodes, each of which knows its neighbors,
* and optionally, groups. The methods {@link AdjacencyGraphBuilder#createNodesSource},
* {@link AdjacencyGraphBuilder#createGroupNodesSource}, {@link AdjacencyNodesSource#createSuccessorsSource}, {@link AdjacencyNodesSource#createPredecessorsSource},
* {@link AdjacencyNodesSource#createOutEdgesSource}, and {@link AdjacencyNodesSource#createInEdgesSource} create new source collections from which nodes and group nodes will be
* created.
*
* Generally, using the {@link AdjacencyGraphBuilder} class consists of a few basic steps:
*
* <ol>
* <li>Create an {@link AdjacencyGraphBuilder} and optionally, provide an {@link IGraph} on which it should
* operate.
*
* <pre>
* const adjacencyBuilder = new AdjacencyGraphBuilder(graph)
* </pre>
* </li>
* <li>Create one or more {@link AdjacencyGraphBuilder#createNodesSource nodes sources} from which the {@link AdjacencyGraphBuilder} creates the initial nodes in the
* graph. By adding {@link AdjacencyGraphBuilder#createGroupNodesSource group node sources}, group nodes ar created.
*
* <pre>
* const orangeData = getAdjacencyNodesData()
* // create source for initial nodes (orange nodes)
* const orangeNodesSource = adjacencyBuilder.createNodesSource(
*   orangeData, // nodes data
*   orangeDataItem => orangeDataItem.nodeId
* ) // node id provider
* </pre>
* </li>
* <li>To create neighbor nodes and edges to or from them, {@link AdjacencyNodesSource#createSuccessorsSource create new successor sources} or {@link AdjacencyNodesSource#createPredecessorsSource predecessor sources} as neighbor
* sources of the initial sources.
*
* By {@link AdjacencyNodesSource#addSuccessorsSource adding existing successor sources} or {@link AdjacencyNodesSource#addPredecessorsSource predecessor sources}, edges can be defined between existing sources.
*
* <pre>
* // create an EdgeCreator for edges adjacent to orange nodes
* const orangeEdgeCreator = new EdgeCreator()
* // create a nodes source for the green nodes as successor of orangeNodesSource
* const greenNodesSource = orangeNodesSource.createSuccessorsSource(
*   orangeNodeDataItem => orangeNodeDataItem.successorNodes,
*   orangeEdgeCreator
* )
* // add the same source also as predecessor
* orangeNodesSource.addPredecessorsSource(
*   orangeNodeDataItem => orangeNodeDataItem.predecessorNodes,
*   greenNodesSource,
*   orangeEdgeCreator
* )
* </pre>
*
* By {@link AdjacencyNodesSource#addSuccessorIds adding existing successor ids} or
* {@link AdjacencyNodesSource#addPredecessorIds predecessor ids}, edges can be defined between the nodes of existing sources where the neighbors are defined by their
* id.
*
* <pre>
* const greenEdgeCreator = new EdgeCreator()
* // register EdgeCreator for edges to orange successor nodes created by orangeNodesSource
* greenNodesSource.addSuccessorIds(
*   greenNodeDataItem => greenNodeDataItem.successorNodeIds,
*   greenEdgeCreator
* )
* // register EdgeCreator for edges to orange predecessor nodes created by orangeNodesSource
* greenNodesSource.addPredecessorIds(
*   greenNodeDataItem => greenNodeDataItem.predecessorNodeIds,
*   greenEdgeCreator
* )
* </pre>
*
* Using the methods mentioned above, edges are always created implicitly between the nodes of an {@link AdjacencyNodesSource} and their added
* neighbors providing the data items of the nodes to the {@link EdgeCreator}.
*
* When the data items of an {@link AdjacencyNodesSource} don't know their neighbor items directly but know their outgoing and incoming edges which in
* turn know the data items of the opposite nodes, the corresponding methods {@link AdjacencyNodesSource#createOutEdgesSource}, {@link AdjacencyNodesSource#createInEdgesSource},
* {@link AdjacencyNodesSource#addOutEdgesSource}, {@link AdjacencyNodesSource#addInEdgesSource}, {@link AdjacencyNodesSource#addOutEdgesSourceToId}, and {@link AdjacencyNodesSource#addInEdgesSourceToId} can be used that provide the
* specified edge data items to the {@link EdgeCreator}.
*
* <pre>
* // create and register EdgeCreator for purple edges
* const purpleEdgeCreator = new EdgeCreator()
* orangeNodesSource.addOutEdgesSource(
*   nodeDataItem => nodeDataItem.purpleOutEdges,
*   purpleEdgeDataItem => purpleEdgeDataItem.greenTargetNode,
*   greenNodesSource, // use existing nodesSource for target nodes
*   purpleEdgeCreator
* )
* // create and register EdgeCreator for yellow edges
* orangeNodesSource.addOutEdgesSourceToId(
*   orangeNodeDataItem => orangeNodeDataItem.yellowOutEdges,
*   yellowEdgeDataItem => yellowEdgeDataItem.targetNodeId,
*   new EdgeCreator()
* )
* </pre>
* </li>
* <li>Each {@link AdjacencyNodesSource} provides a {@link NodeCreator} that allows for further specification of how the items from this source should be created. You can
* provide {@link NodeCreator#defaults} for the default styling but also bind more specific styling based on the actual data item with the {@link NodeCreator#styleProvider} and {@link NodeCreator#styleBindings}.
*
* <pre>
* orangeNodesSource.nodeCreator.defaults.shareStyleInstance = false
* orangeNodesSource.nodeCreator.defaults.style = new ShapeNodeStyle({
*   stroke: 'darkRed',
*   fill: 'orange'
* })
* // nodes without successor should have a diamond shape
* orangeNodesSource.nodeCreator.styleBindings.addBinding('shape', redDataItem =>
*   redDataItem.successorNodes.any() ? 'roundRectangle' : 'diamond'
* )
* </pre>
*
* It also allows for obtaining layout information from the data item ({@link NodeCreator#layoutProvider}), as well as further specification of the created
* node's {@link ITagOwner#tag} ({@link NodeCreator#tagProvider}).</li>
* <li>The edge creation can be adjusted by passing suitable {@link EdgeCreator} instances when adding a neighbor source. It allows to specify the
* styling of the created edges ({@link EdgeCreator#defaults}, {@link EdgeCreator#styleProvider}, {@link EdgeCreator#styleBindings}) and the edges' {@link ITagOwner#tag}s ({@link EdgeCreator#tagProvider}).
*
* <pre>
* greenEdgeCreator.defaults.style = new PolylineEdgeStyle({ stroke: 'forestGreen' })
* </pre>
* </li>
* <li>Optionally, labels can be added by providing one or multiple {@link NodeCreator#createLabelBinding label bindings}. If there is a varying number of labels per
* node, {@link NodeCreator#createLabelsSource} can be used, instead. Similar methods are also available on the {@link EdgeCreator}.
*
* The {@link LabelsSource} provides a {@link LabelCreator} that allows for adjusting all aspects of the label creation, like style, text, size, and tag.
*
* <pre>
* const edgeLabelCreator = purpleEdgeCreator.createLabelBinding(
*   purpleEdgeData => purpleEdgeData.connectionType
* )
* edgeLabelCreator.defaults.style = new DefaultLabelStyle({
*   backgroundFill: 'white',
*   backgroundStroke: 'purple',
*   textSize: 8
* })
* </pre>
* </li>
* <li>Call {@link AdjacencyGraphBuilder#buildGraph} to populate the graph. You can apply a layout algorithm afterward
* to make the graph look nice.
*
* <pre>
* // Build a graph initially
* adjacencyBuilder.buildGraph()
* // Apply a layout in a subsequent step
* graph.applyLayout(new HierarchicLayout())
* </pre>
* </li>
* <li>If your items or collections change later, call {@link AdjacencyGraphBuilder#updateGraph} to make the changes
* visible in the graph. If the data item instances have changed, you can call
* {@link AdjacencyGraphBuilder#setData}.
*
* <pre>
* // Set the new data collections:
* adjacencyBuilder.setData(orangeNodesSource, getAdjacencyNodesData())
* // Update a graph after the business data has changed
* adjacencyBuilder.updateGraph()
* // Apply a layout in a subsequent step
* graph.applyLayout(new HierarchicLayout())
* </pre>
* </li>
* </ol>
* You can further customize how nodes, groups, and edges are created by adding event handlers to the various events and
* modifying the items there. This can be used to modify items in ways that are not directly supported by the available
* bindings or defaults. There are creation and update events for all items, which allow for separate customization when
* nodes, groups, and edges are created or updated. For completely static graphs, where {@link AdjacencyGraphBuilder#updateGraph}
* is not needed, the update events can be safely ignored.
*
* <h2>Note</h2>
*
* This class serves as a convenient way to create graphs. Some aspects to look out for:
*
* <ul>
* <li>{@link AdjacencyGraphBuilder#buildGraph} does not clear the graph. If needed, call {@link IGraph#clear} to
* empty the graph before building it anew.</li>
* <li>The {@link AdjacencyGraphBuilder} ignores and preserves items manually created on the graph in between calls to
* {@link AdjacencyGraphBuilder#updateGraph}.</li>
* </ul>
* If you need to extensively modify your data in order to fit the requirements of the
* {@link AdjacencyGraphBuilder}, it's often better to write the code interfacing with the graph by hand instead of
* relying on {@link AdjacencyGraphBuilder}.
*
* <h2>Note</h2>
*
* {@link AdjacencyGraphBuilder#updateGraph} merely updates the graph structure. Any other bound data must be
* updated explicitly if necessary. Calling the respective <code>update</code> methods on the <code>creators</code> (e.g. {@link NodeCreator#updateStyle}), resolves the
* <code>provider</code> (e.g. {@link NodeCreator#styleProvider}) and applies the <code>bindings</code> (e.g.
* {@link NodeCreator#applyStyleBindings}). The latter can also be applied solely.
* <br />
* For example, to update any aspect of node creation:
*
* <pre>
* // evaluate the providers and apply respective bindings
* nodesSource.nodeCreator.addNodeUpdatedListener((sender, evt) => {
*   nodesSource.nodeCreator.updateLayout(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateStyle(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateTag(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateLabels(evt.graph, evt.item, evt.dataItem)
* })
*
* builder.updateGraph()
* </pre>
*
* The different graph builders are discussed in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph_builder Creating a Graph from Business Data}. Class
* <code>AdjacencyGraphBuilder</code>, in particular, is topic of section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph_builder-AdjacencyGraphBuilder AdjacencyGraphBuilder}.
*
* @see GraphBuilder
* @see TreeBuilder
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder Online Documentation}
*/
export class AdjacencyGraphBuilder {
/**
Initializes a new instance of the {@link AdjacencyGraphBuilder} class that operates on the given graph.
The <code>graph</code> will not be cleared when {@link AdjacencyGraphBuilder#buildGraph} is called. To clear the graph
beforehand, call {@link IGraph#clear}.
@param [graph=null] An {@link IGraph} instance on which this builder operates or <code>null</code> if a new {@link DefaultGraph} should be
created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder Online Documentation}*/
constructor(graph?: IGraph | null);
/**
Binds a collection of data items to the given <code>nodesSource</code>.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param data The collection of objects that is bound to the source.
@param nodesSource The source to which the data is bound.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23addNodesSource(Object%2CAdjacencyNodesSource%3CTDataItem%3E) Online Documentation}*/
addNodesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, nodesSource: AdjacencyNodesSource<TDataItem>): void;
/**
Populates the graph with items generated from the bound data.
This graph is not cleared before the new items are added. To clear the graph, explicitly call {@link IGraph#clear}
beforehand.
@returns The {@link AdjacencyGraphBuilder#graph} of this builder populated by the defined sources.
@see AdjacencyGraphBuilder#updateGraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23buildGraph() Online Documentation}*/
buildGraph(): IGraph;
/**
Creates a new {@link AdjacencyNodesSource} and binds a collection of group node data items to it.
The {@link NodeCreator} of the returned {@link AdjacencyNodesSource} can be used to further specify creation properties of these items, e.g. style, layout, etc.
Data items of this collection are created as group nodes in the {@link AdjacencyGraphBuilder#graph}.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param data The collection of objects to iterate and create the group nodes from.
@param [idProvider=null] An optional function that yields an ID for each element in the <code>data</code>. This ID is used by {@link NodesSource#parentIdProvider}, {@link AdjacencyNodesSource#addSuccessorIds}, and {@link AdjacencyNodesSource#addPredecessorIds}
to resolve the parent, source, or target nodes. The ID is also used to identify nodes during
{@link AdjacencyGraphBuilder#updateGraph}.
@returns A new {@link AdjacencyNodesSource} instance that can be used to further customize the creation of nodes, e.g. provide specific {@link NodeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23createGroupNodesSource(Object%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createGroupNodesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, idProvider?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>): AdjacencyNodesSource<TDataItem>;
/**
Creates a new {@link AdjacencyNodesSource} and binds a collection of node data items to it.
The {@link NodeCreator} of the returned {@link AdjacencyNodesSource} can be used to further specify creation properties of these items, e.g. style, layout, etc.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param data The collection of objects to iterate and create the nodes from.
@param [idProvider=null] An optional function that yields an ID for each element in the <code>data</code>. This ID is used by {@link NodesSource#parentIdProvider}, {@link AdjacencyNodesSource#addSuccessorIds}, and {@link AdjacencyNodesSource#addPredecessorIds}
to resolve the parent, source, or target nodes. The ID is also used to identify nodes during
{@link AdjacencyGraphBuilder#updateGraph}.
@returns A new {@link AdjacencyNodesSource} instance that can be used to further customize the creation of nodes, e.g. provide specific {@link NodeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23createNodesSource(Object%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createNodesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, idProvider?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>): AdjacencyNodesSource<TDataItem>;
/**
Triggers the {@link AdjacencyGraphBuilder#addEdgeCreatedListener EdgeCreated} event.
@param edge The edge that has been created.
@param dataItem The data item from which the edge has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23onEdgeCreated(IEdge%2CObject) Online Documentation}*/
onEdgeCreated(edge: IEdge, dataItem: any): void;
/**
Triggers the {@link AdjacencyGraphBuilder#addEdgeRemovedListener EdgeRemoved} event.
@param edge The edge that has been removed.
@param dataItem The corresponding data item of the removed edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23onEdgeRemoved(IEdge%2CObject) Online Documentation}*/
onEdgeRemoved(edge: IEdge, dataItem: any): void;
/**
Triggers the {@link AdjacencyGraphBuilder#addEdgeUpdatedListener EdgeUpdated} event.
@param edge The edge that has been updated.
@param dataItem The data item with which the edge has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23onEdgeUpdated(IEdge%2CObject) Online Documentation}*/
onEdgeUpdated(edge: IEdge, dataItem: any): void;
/**
Triggers the {@link AdjacencyGraphBuilder#addLabelAddedListener LabelAdded} event.
@param label The label that has been added.
@param dataItem The data item from which the label has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23onLabelAdded(ILabel%2CObject) Online Documentation}*/
onLabelAdded(label: ILabel, dataItem: any): void;
/**
Triggers the {@link AdjacencyGraphBuilder#addLabelRemovedListener LabelRemoved} event.
@param label The label that has been removed.
@param dataItem The corresponding data item of the removed label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23onLabelRemoved(ILabel%2CObject) Online Documentation}*/
onLabelRemoved(label: ILabel, dataItem: any): void;
/**
Triggers the {@link AdjacencyGraphBuilder#addLabelUpdatedListener LabelUpdated} event.
@param label The label that has been updated.
@param dataItem The data item with which the label has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23onLabelUpdated(ILabel%2CObject) Online Documentation}*/
onLabelUpdated(label: ILabel, dataItem: any): void;
/**
Triggers the {@link AdjacencyGraphBuilder#addNodeCreatedListener NodeCreated} event.
@param node The node that has been created.
@param dataItem The data item from which the node has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23onNodeCreated(INode%2CObject) Online Documentation}*/
onNodeCreated(node: INode, dataItem: any): void;
/**
Triggers the {@link AdjacencyGraphBuilder#addNodeRemovedListener NodeRemoved} event.
@param node The node that has been removed.
@param dataItem The corresponding data item of the removed node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23onNodeRemoved(INode%2CObject) Online Documentation}*/
onNodeRemoved(node: INode, dataItem: any): void;
/**
Triggers the {@link AdjacencyGraphBuilder#addNodeUpdatedListener NodeUpdated} event.
@param node The node that has been updated.
@param dataItem The data item with which the node has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23onNodeUpdated(INode%2CObject) Online Documentation}*/
onNodeUpdated(node: INode, dataItem: any): void;
/**
Binds a new data collection to an {@link AdjacencyNodesSource}, replacing the old one.
This can be used to assign an entirely different collection instance to an existing <code>nodesSource</code>, such that {@link AdjacencyGraphBuilder#updateGraph}
considers the new collection.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param nodesSource The source whose data source should be re-assigned.
@param data The collection of objects that is specified for the given source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23setData(AdjacencyNodesSource%3CTDataItem%3E%2CObject) Online Documentation}*/
setData<TDataItem>(nodesSource: AdjacencyNodesSource<TDataItem>, data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}): void;
/**
* Updates the graph after changes in the bound data.
* Graph elements corresponding to objects that are still present in the source collections are kept, new graph elements
* are created for new objects in the collections, and obsolete ones are removed.
*
* <h2>Note</h2>
*
* {@link AdjacencyGraphBuilder#updateGraph} merely updates the graph structure. Any other bound data must be
* updated explicitly if necessary. Calling the respective <code>update</code> methods on the <code>creators</code> (e.g. {@link NodeCreator#updateStyle}), resolves the
* <code>provider</code> (e.g. {@link NodeCreator#styleProvider}) and applies the <code>bindings</code> (e.g.
* {@link NodeCreator#applyStyleBindings}). The latter can also be applied solely.
* <br />
* For example, to update any aspect of node creation:
*
* <pre>
* // evaluate the providers and apply respective bindings
* nodesSource.nodeCreator.addNodeUpdatedListener((sender, evt) => {
*   nodesSource.nodeCreator.updateLayout(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateStyle(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateTag(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateLabels(evt.graph, evt.item, evt.dataItem)
* })
*
* builder.updateGraph()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23updateGraph() Online Documentation}
*/
updateGraph(): void;
/**
Gets the {@link IGraph graph} used by this builder.
The style defaults of this graph are considered in the {@link NodeCreator}, {@link EdgeCreator} and {@link LabelCreator} of the sources that are created on this
{@link AdjacencyGraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23graph Online Documentation}*/
readonly graph: IGraph;
/**
Adds the given listener for the <code>NodeCreated</code> event that occurs when a node has been created by any one of the
{@link AdjacencyGraphBuilder#createNodesSource nodes sources}.
This event can be used to further customize the created node.
New nodes are created either in response to calling {@link AdjacencyGraphBuilder#buildGraph}, or in response to
calling {@link AdjacencyGraphBuilder#updateGraph} when there are new items in the {@link NodesSource nodes sources}.
@param listener The listener to add.
@see AdjacencyGraphBuilder#addNodeUpdatedListener
@see AdjacencyGraphBuilder#addNodeRemovedListener
@see AdjacencyGraphBuilder#removeNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23NodeCreated Online Documentation}*/
addNodeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeCreated</code> event that occurs when a node has been created by any one of the
{@link AdjacencyGraphBuilder#createNodesSource nodes sources}.
This event can be used to further customize the created node.
New nodes are created either in response to calling {@link AdjacencyGraphBuilder#buildGraph}, or in response to
calling {@link AdjacencyGraphBuilder#updateGraph} when there are new items in the {@link NodesSource nodes sources}.
@param listener The listener to remove.
@see AdjacencyGraphBuilder#addNodeUpdatedListener
@see AdjacencyGraphBuilder#addNodeRemovedListener
@see AdjacencyGraphBuilder#addNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23NodeCreated Online Documentation}*/
removeNodeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>NodeUpdated</code> event that occurs when a node has been updated.
This event can be used to further customize the updated node.
Nodes are updated in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to add.
@see AdjacencyGraphBuilder#addNodeCreatedListener
@see AdjacencyGraphBuilder#addNodeRemovedListener
@see AdjacencyGraphBuilder#removeNodeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23NodeUpdated Online Documentation}*/
addNodeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeUpdated</code> event that occurs when a node has been updated.
This event can be used to further customize the updated node.
Nodes are updated in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to remove.
@see AdjacencyGraphBuilder#addNodeCreatedListener
@see AdjacencyGraphBuilder#addNodeRemovedListener
@see AdjacencyGraphBuilder#addNodeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23NodeUpdated Online Documentation}*/
removeNodeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed from any one of the
{@link AdjacencyGraphBuilder#createNodesSource nodes sources}.
This event can be used to further handle the removal of the removed node.
Nodes are removed in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to add.
@see AdjacencyGraphBuilder#addNodeCreatedListener
@see AdjacencyGraphBuilder#addNodeRemovedListener
@see AdjacencyGraphBuilder#removeNodeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23NodeRemoved Online Documentation}*/
addNodeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed from any one of the
{@link AdjacencyGraphBuilder#createNodesSource nodes sources}.
This event can be used to further handle the removal of the removed node.
Nodes are removed in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to remove.
@see AdjacencyGraphBuilder#addNodeCreatedListener
@see AdjacencyGraphBuilder#addNodeRemovedListener
@see AdjacencyGraphBuilder#addNodeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23NodeRemoved Online Documentation}*/
removeNodeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created by any one of
{@link AdjacencyNodesSource}'s add or create methods.
This event can be used to further customize the created edge.
New edges are created either in response to calling {@link AdjacencyGraphBuilder#buildGraph}, or in response to
calling {@link AdjacencyGraphBuilder#updateGraph} when there are new items in the edges sources.
@param listener The listener to add.
@see AdjacencyGraphBuilder#addEdgeUpdatedListener
@see AdjacencyGraphBuilder#addEdgeRemovedListener
@see AdjacencyGraphBuilder#removeEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23EdgeCreated Online Documentation}*/
addEdgeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created by any one of
{@link AdjacencyNodesSource}'s add or create methods.
This event can be used to further customize the created edge.
New edges are created either in response to calling {@link AdjacencyGraphBuilder#buildGraph}, or in response to
calling {@link AdjacencyGraphBuilder#updateGraph} when there are new items in the edges sources.
@param listener The listener to remove.
@see AdjacencyGraphBuilder#addEdgeUpdatedListener
@see AdjacencyGraphBuilder#addEdgeRemovedListener
@see AdjacencyGraphBuilder#addEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23EdgeCreated Online Documentation}*/
removeEdgeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>EdgeUpdated</code> event that occurs when an edge has been updated.
This event can be used to further customize the updated edge.
Edges are updated in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to add.
@see AdjacencyGraphBuilder#addEdgeCreatedListener
@see AdjacencyGraphBuilder#addEdgeRemovedListener
@see AdjacencyGraphBuilder#removeEdgeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23EdgeUpdated Online Documentation}*/
addEdgeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeUpdated</code> event that occurs when an edge has been updated.
This event can be used to further customize the updated edge.
Edges are updated in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to remove.
@see AdjacencyGraphBuilder#addEdgeCreatedListener
@see AdjacencyGraphBuilder#addEdgeRemovedListener
@see AdjacencyGraphBuilder#addEdgeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23EdgeUpdated Online Documentation}*/
removeEdgeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
This event can be used to further handle the removal of the removed edge.
Edges are removed in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to add.
@see AdjacencyGraphBuilder#addEdgeCreatedListener
@see AdjacencyGraphBuilder#addEdgeUpdatedListener
@see AdjacencyGraphBuilder#removeEdgeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23EdgeRemoved Online Documentation}*/
addEdgeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
This event can be used to further handle the removal of the removed edge.
Edges are removed in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to remove.
@see AdjacencyGraphBuilder#addEdgeCreatedListener
@see AdjacencyGraphBuilder#addEdgeUpdatedListener
@see AdjacencyGraphBuilder#addEdgeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23EdgeRemoved Online Documentation}*/
removeEdgeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to a node or edge.
This event can be used to further customize the created label.
New labels are added in response to calling {@link AdjacencyGraphBuilder#buildGraph}, or in response to calling
{@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to add.
@see AdjacencyGraphBuilder#addLabelUpdatedListener
@see AdjacencyGraphBuilder#addLabelRemovedListener
@see AdjacencyGraphBuilder#removeLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23LabelAdded Online Documentation}*/
addLabelAddedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to a node or edge.
This event can be used to further customize the created label.
New labels are added in response to calling {@link AdjacencyGraphBuilder#buildGraph}, or in response to calling
{@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to remove.
@see AdjacencyGraphBuilder#addLabelUpdatedListener
@see AdjacencyGraphBuilder#addLabelRemovedListener
@see AdjacencyGraphBuilder#addLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23LabelAdded Online Documentation}*/
removeLabelAddedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Adds the given listener for the <code>LabelUpdated</code> event that occurs when a node or edge label has been updated.
This event can be used to further customize the updated label.
Labels are updated in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to add.
@see AdjacencyGraphBuilder#addLabelAddedListener
@see AdjacencyGraphBuilder#addLabelRemovedListener
@see AdjacencyGraphBuilder#removeLabelUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23LabelUpdated Online Documentation}*/
addLabelUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelUpdated</code> event that occurs when a node or edge label has been updated.
This event can be used to further customize the updated label.
Labels are updated in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to remove.
@see AdjacencyGraphBuilder#addLabelAddedListener
@see AdjacencyGraphBuilder#addLabelRemovedListener
@see AdjacencyGraphBuilder#addLabelUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23LabelUpdated Online Documentation}*/
removeLabelUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Adds the given listener for the <code>LabelRemoved</code> event that occurs when a node or edge label has been removed.
This event can be used to further handle the removal of the removed label.
Labels are removed in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to add.
@see AdjacencyGraphBuilder#addLabelAddedListener
@see AdjacencyGraphBuilder#addLabelUpdatedListener
@see AdjacencyGraphBuilder#removeLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23LabelRemoved Online Documentation}*/
addLabelRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelRemoved</code> event that occurs when a node or edge label has been removed.
This event can be used to further handle the removal of the removed label.
Labels are removed in response to calling {@link AdjacencyGraphBuilder#updateGraph}.
@param listener The listener to remove.
@see AdjacencyGraphBuilder#addLabelAddedListener
@see AdjacencyGraphBuilder#addLabelUpdatedListener
@see AdjacencyGraphBuilder#addLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyGraphBuilder%23LabelRemoved Online Documentation}*/
removeLabelRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
static $class: Class;
private $_AdjacencyGraphBuilder: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AdjacencyGraphBuilder;
}
export interface AdjacencyNodesSource<TDataItem> extends NodesSource<TDataItem> {
}
/**
A class that accepts a collection of objects from the {@link AdjacencyGraphBuilder} to build
{@link INode}s from.
It manages the structural aspects of the graph creation, while the associated {@link NodesSource#nodeCreator} is responsible for the properties of the
created items.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource Online Documentation}*/
export class AdjacencyNodesSource<TDataItem> {
/**
Creates a new source with the given <code>nodeCreator</code>.
@param nodeCreator The creator that is used to create the nodes of this source.
@param [idProvider=null] An optional function that yields an id for each element of the associated data collection in the
{@link AdjacencyGraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource Online Documentation}*/
constructor(nodeCreator: NodeCreator<TDataItem>, idProvider?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>);
/**
Registers a provider for outgoing edge data items from which edges are created using the provided <code>edgeCreator</code>.
The <code>edgeDataProvider</code> provides a collection of outgoing edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
The <code>sourceDataProvider</code> takes such an edge data item and provides the corresponding source node data item.
The edge items function as edge objects for the <code>edgeCreator</code>.
Type parameter <code>TEdgeDataItem</code>: The type of the edge data items.
Type parameter <code>TPredecessorDataItem</code>: The type of the predecessor data items.
@param edgeDataProvider A function that yields the edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param sourceDataProvider A function that yields the source data item for each edge data item provided by <code>edgeDataProvider</code>.
@param sourceNodesSource The nodes source for the source data items.
@param edgeCreator The creator that is used to create the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23addInEdgesSource(function(TDataItem)%3AObject%2Cfunction(TEdgeDataItem)%3ATPredecessorDataItem%2CAdjacencyNodesSource%3CTPredecessorDataItem%3E%2CEdgeCreator%3CTEdgeDataItem%3E) Online Documentation}*/
addInEdgesSource<TEdgeDataItem, TPredecessorDataItem>(edgeDataProvider: ((data: TDataItem) => TEdgeDataItem[] | Iterable<TEdgeDataItem> | Map<any, TEdgeDataItem> | {
[id: string]: TEdgeDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TEdgeDataItem[] | Iterable<TEdgeDataItem> | Map<any, TEdgeDataItem> | {
[id: string]: TEdgeDataItem;
}>, sourceDataProvider: ProviderConvertible<TEdgeDataItem, TPredecessorDataItem> | ((dataItem: TEdgeDataItem) => TPredecessorDataItem | null), sourceNodesSource: AdjacencyNodesSource<TPredecessorDataItem>, edgeCreator: EdgeCreator<TEdgeDataItem>): void;
/**
Registers a provider for outgoing edge data items from which edges are created using the provided <code>edgeCreator</code>.
The <code>edgeDataProvider</code> provides a collection of outgoing edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
The <code>sourceIdProvider</code> takes such an edge data item and provides the corresponding source node id.
The edge items function as edge objects for the <code>edgeCreator</code>.
Type parameter <code>TEdgeDataItem</code>: The type of the edge data items.
@param edgeDataProvider A function that yields the edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param sourceIdProvider A function that yields the source node id for each edge data item provided by <code>edgeDataProvider</code>.
@param edgeCreator The creator that is used to create the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23addInEdgesSourceToId(function(TDataItem)%3AObject%2Cfunction(TEdgeDataItem)%3AObject%2CEdgeCreator%3CTEdgeDataItem%3E) Online Documentation}*/
addInEdgesSourceToId<TEdgeDataItem>(edgeDataProvider: ((data: TDataItem) => TEdgeDataItem[] | Iterable<TEdgeDataItem> | Map<any, TEdgeDataItem> | {
[id: string]: TEdgeDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TEdgeDataItem[] | Iterable<TEdgeDataItem> | Map<any, TEdgeDataItem> | {
[id: string]: TEdgeDataItem;
}>, sourceIdProvider: ProviderConvertible<TEdgeDataItem, any> | ((dataItem: TEdgeDataItem) => any | null), edgeCreator: EdgeCreator<TEdgeDataItem>): void;
/**
Registers a provider for outgoing edge data items from which edges are created using the provided <code>edgeCreator</code>.
The <code>edgeDataProvider</code> provides a collection of outgoing edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
The <code>targetDataProvider</code> takes such an edge data item and provides the corresponding target node data item.
The edge items function as edge objects for the <code>edgeCreator</code>.
Type parameter <code>TEdgeDataItem</code>: The type of the edge data items.
Type parameter <code>TSuccessorDataItem</code>: The type of the successor data items.
@param edgeDataProvider A function that yields the edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param targetDataProvider A function that yields the target data item for each edge data item provided by <code>edgeDataProvider</code>.
@param targetNodesSource The nodes source for the target data items.
@param edgeCreator The creator that is used to create the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23addOutEdgesSource(function(TDataItem)%3AObject%2Cfunction(TEdgeDataItem)%3ATSuccessorDataItem%2CAdjacencyNodesSource%3CTSuccessorDataItem%3E%2CEdgeCreator%3CTEdgeDataItem%3E) Online Documentation}*/
addOutEdgesSource<TEdgeDataItem, TSuccessorDataItem>(edgeDataProvider: ((data: TDataItem) => TEdgeDataItem[] | Iterable<TEdgeDataItem> | Map<any, TEdgeDataItem> | {
[id: string]: TEdgeDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TEdgeDataItem[] | Iterable<TEdgeDataItem> | Map<any, TEdgeDataItem> | {
[id: string]: TEdgeDataItem;
}>, targetDataProvider: ProviderConvertible<TEdgeDataItem, TSuccessorDataItem> | ((dataItem: TEdgeDataItem) => TSuccessorDataItem | null), targetNodesSource: AdjacencyNodesSource<TSuccessorDataItem>, edgeCreator: EdgeCreator<TEdgeDataItem>): void;
/**
Registers a provider for outgoing edge data items from which edges are created using the provided <code>edgeCreator</code>.
The <code>edgeDataProvider</code> provides a collection of outgoing edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
The <code>targetIdProvider</code> takes such an edge data item and provides the corresponding target node id.
The edge items function as edge objects for the <code>edgeCreator</code>.
Type parameter <code>TEdgeDataItem</code>: The type of the edge data items.
@param edgeDataProvider A function that yields the edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param targetIdProvider A function that yields the target node id for each edge data item provided by <code>edgeDataProvider</code>.
@param edgeCreator The creator that is used to create the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23addOutEdgesSourceToId(function(TDataItem)%3AObject%2Cfunction(TEdgeDataItem)%3AObject%2CEdgeCreator%3CTEdgeDataItem%3E) Online Documentation}*/
addOutEdgesSourceToId<TEdgeDataItem>(edgeDataProvider: ((data: TDataItem) => TEdgeDataItem[] | Iterable<TEdgeDataItem> | Map<any, TEdgeDataItem> | {
[id: string]: TEdgeDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TEdgeDataItem[] | Iterable<TEdgeDataItem> | Map<any, TEdgeDataItem> | {
[id: string]: TEdgeDataItem;
}>, targetIdProvider: ProviderConvertible<TEdgeDataItem, any> | ((dataItem: TEdgeDataItem) => any | null), edgeCreator: EdgeCreator<TEdgeDataItem>): void;
/**
Registers a provider for source node ids to which edges are created using the provided <code>edgeCreator</code>.
The <code>predecessorIdsProvider</code> provides a collection of source node ids for each node data item associated with this {@link AdjacencyNodesSource}.
The node items of this source function as data items for the <code>edgeCreator</code>.
@param predecessorIdsProvider A function that yields the source node ids for each node data item associated with this {@link AdjacencyNodesSource}.
@param edgeCreator The creator that is used to create the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23addPredecessorIds(function(TDataItem)%3AIEnumerable%3CObject%3E%2CEdgeCreator%3CTDataItem%3E) Online Documentation}*/
addPredecessorIds(predecessorIdsProvider: ProviderConvertible<TDataItem, IEnumerable<any>> | ((dataItem: TDataItem) => IEnumerable<any> | null), edgeCreator: EdgeCreator<TDataItem>): void;
/**
Registers a provider for node data items associated with a <code>predecessorNodesSource</code> to which edges are created using the
provided <code>edgeCreator</code>.
The <code>predecessorsProvider</code> provides a collection of source node data items for each node data item associated with this {@link AdjacencyNodesSource}.
The node items of this source function as data items for the <code>edgeCreator</code>.
Type parameter <code>TPredecessorDataItem</code>: The type of the predecessor data items.
@param predecessorsProvider A function that yields the predecessor data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param predecessorNodesSource The nodes source for the predecessor data items.
@param edgeCreator The creator that is used to create the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23addPredecessorsSource(function(TDataItem)%3AObject%2CAdjacencyNodesSource%3CTPredecessorDataItem%3E%2CEdgeCreator%3CTDataItem%3E) Online Documentation}*/
addPredecessorsSource<TPredecessorDataItem>(predecessorsProvider: ((data: TDataItem) => TPredecessorDataItem[] | Iterable<TPredecessorDataItem> | Map<any, TPredecessorDataItem> | {
[id: string]: TPredecessorDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TPredecessorDataItem[] | Iterable<TPredecessorDataItem> | Map<any, TPredecessorDataItem> | {
[id: string]: TPredecessorDataItem;
}>, predecessorNodesSource: AdjacencyNodesSource<TPredecessorDataItem>, edgeCreator: EdgeCreator<TDataItem>): void;
/**
Registers a provider for target node ids to which edges are created using the provided <code>edgeCreator</code>.
The <code>successorIdsProvider</code> provides a collection of target node ids for each node data item associated with this {@link AdjacencyNodesSource}.
The node items of this source function as data items for the <code>edgeCreator</code>.
@param successorIdsProvider A function that yields the target node ids for each node data item associated with this {@link AdjacencyNodesSource}.
@param edgeCreator The creator that is used to create the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23addSuccessorIds(function(TDataItem)%3AIEnumerable%3CObject%3E%2CEdgeCreator%3CTDataItem%3E) Online Documentation}*/
addSuccessorIds(successorIdsProvider: ProviderConvertible<TDataItem, IEnumerable<any>> | ((dataItem: TDataItem) => IEnumerable<any> | null), edgeCreator: EdgeCreator<TDataItem>): void;
/**
Registers a provider for node data items associated with a <code>successorNodesSource</code> to which edges are created using the
provided <code>edgeCreator</code>.
The <code>successorsProvider</code> provides a collection of target node data items for each node data item associated with this {@link AdjacencyNodesSource}.
The node items of this source function as data items for the <code>edgeCreator</code>.
Type parameter <code>TSuccessorDataItem</code>: The type of the successor data items.
@param successorsProvider A function that yields the successor data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param successorNodesSource The nodes source for the successor data items.
@param edgeCreator The creator that is used to create the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23addSuccessorsSource(function(TDataItem)%3AObject%2CAdjacencyNodesSource%3CTSuccessorDataItem%3E%2CEdgeCreator%3CTDataItem%3E) Online Documentation}*/
addSuccessorsSource<TSuccessorDataItem>(successorsProvider: ((data: TDataItem) => TSuccessorDataItem[] | Iterable<TSuccessorDataItem> | Map<any, TSuccessorDataItem> | {
[id: string]: TSuccessorDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TSuccessorDataItem[] | Iterable<TSuccessorDataItem> | Map<any, TSuccessorDataItem> | {
[id: string]: TSuccessorDataItem;
}>, successorNodesSource: AdjacencyNodesSource<TSuccessorDataItem>, edgeCreator: EdgeCreator<TDataItem>): void;
/**
Registers a provider for outgoing edge data items whose <code>sourceDataProvider</code> function as node objects for the returned {@link AdjacencyNodesSource}.
The <code>edgeDataProvider</code> provides a collection of outgoing edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
The <code>sourceDataProvider</code> takes such an edge data item and provides the corresponding source node data item.
The edge items function as edge objects for the <code>edgeCreator</code>.
The {@link NodeCreator#defaults node defaults} of this newly created source cascade with the defaults of this source.
Type parameter <code>TEdgeDataItem</code>: The type of the edge data items.
Type parameter <code>TPredecessorDataItem</code>: The type of the predecessor data items in the source.
@param edgeDataProvider A function that yields the edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param sourceDataProvider A function that yields the source node data items for each edge data item provided by <code>edgeDataProvider</code>.
@param edgeCreator The creator that is used to create the edges.
@param [sourceIdProvider=null] An optional function that yields an id for each source node data item.
@returns A new {@link AdjacencyNodesSource} for the node data items returned by <code>sourceDataProvider</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23createInEdgesSource(function(TDataItem)%3AObject%2Cfunction(TEdgeDataItem)%3ATPredecessorDataItem%2CEdgeCreator%3CTEdgeDataItem%3E%2Cfunction(TPredecessorDataItem%2CObject)%3AObject) Online Documentation}*/
createInEdgesSource<TEdgeDataItem, TPredecessorDataItem>(edgeDataProvider: ((data: TDataItem) => TPredecessorDataItem[] | Iterable<TPredecessorDataItem> | Map<any, TPredecessorDataItem> | {
[id: string]: TPredecessorDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TPredecessorDataItem[] | Iterable<TPredecessorDataItem> | Map<any, TPredecessorDataItem> | {
[id: string]: TPredecessorDataItem;
}>, sourceDataProvider: ProviderConvertible<TEdgeDataItem, TPredecessorDataItem> | ((dataItem: TEdgeDataItem) => TPredecessorDataItem | null), edgeCreator: EdgeCreator<TEdgeDataItem>, sourceIdProvider?: ((dataItem: TPredecessorDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TPredecessorDataItem>): AdjacencyNodesSource<TPredecessorDataItem>;
/**
Registers a provider for outgoing edge data items whose <code>targetDataProvider</code> function as node objects for the returned {@link AdjacencyNodesSource}.
The <code>edgeDataProvider</code> provides a collection of outgoing edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
The <code>targetDataProvider</code> takes such an edge data item and provides the corresponding target node data item.
The edge items function as edge objects for the <code>edgeCreator</code>.
The {@link NodeCreator#defaults node defaults} of this newly created source cascade with the defaults of this source.
Type parameter <code>TEdgeDataItem</code>: The type of the edge data items.
Type parameter <code>TSuccessorDataItem</code>: The type of the successor data items in the source.
@param edgeDataProvider A function that yields the edge data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param targetDataProvider A function that yields the target node data items for each edge data item provided by <code>edgeDataProvider</code>.
@param edgeCreator The creator that is used to create the edges.
@param [targetIdProvider=null] An optional function that yields an id for each target node data item.
@returns A new {@link AdjacencyNodesSource} for the node data items returned by <code>targetDataProvider</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23createOutEdgesSource(function(TDataItem)%3AObject%2Cfunction(TEdgeDataItem)%3ATSuccessorDataItem%2CEdgeCreator%3CTEdgeDataItem%3E%2Cfunction(TSuccessorDataItem%2CObject)%3AObject) Online Documentation}*/
createOutEdgesSource<TEdgeDataItem, TSuccessorDataItem>(edgeDataProvider: ((data: TDataItem) => TSuccessorDataItem[] | Iterable<TSuccessorDataItem> | Map<any, TSuccessorDataItem> | {
[id: string]: TSuccessorDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TSuccessorDataItem[] | Iterable<TSuccessorDataItem> | Map<any, TSuccessorDataItem> | {
[id: string]: TSuccessorDataItem;
}>, targetDataProvider: ProviderConvertible<TEdgeDataItem, TSuccessorDataItem> | ((dataItem: TEdgeDataItem) => TSuccessorDataItem | null), edgeCreator: EdgeCreator<TEdgeDataItem>, targetIdProvider?: ((dataItem: TSuccessorDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TSuccessorDataItem>): AdjacencyNodesSource<TSuccessorDataItem>;
/**
Registers a provider for node data items functioning as node objects for the returned {@link AdjacencyNodesSource}.
The <code>predecessorsProvider</code> provides a collection of source node data items for each node data item associated with this {@link AdjacencyNodesSource}.
The node items of this source function as data items for the <code>edgeCreator</code>.
The {@link NodeCreator#defaults node defaults} of this newly created source cascade with the defaults of this source.
Type parameter <code>TPredecessorDataItem</code>: The type of the predecessor data items in the source.
@param predecessorsProvider A function that yields the predecessor data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param edgeCreator The creator that is used to create the edges.
@param [idProvider=null] An optional function that yields an id for each source node data item.
@returns A new {@link AdjacencyNodesSource} for the node data items returned by <code>predecessorsProvider</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23createPredecessorsSource(function(TDataItem)%3AObject%2CEdgeCreator%3CTDataItem%3E%2Cfunction(TPredecessorDataItem%2CObject)%3AObject) Online Documentation}*/
createPredecessorsSource<TPredecessorDataItem>(predecessorsProvider: ((data: TDataItem) => TPredecessorDataItem[] | Iterable<TPredecessorDataItem> | Map<any, TPredecessorDataItem> | {
[id: string]: TPredecessorDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TPredecessorDataItem[] | Iterable<TPredecessorDataItem> | Map<any, TPredecessorDataItem> | {
[id: string]: TPredecessorDataItem;
}>, edgeCreator: EdgeCreator<TDataItem>, idProvider?: ((dataItem: TPredecessorDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TPredecessorDataItem>): AdjacencyNodesSource<TPredecessorDataItem>;
/**
Registers a provider for node data items functioning as node objects for the returned {@link AdjacencyNodesSource}.
The <code>successorsProvider</code> provides a collection of target node data items for each node data item associated with this {@link AdjacencyNodesSource}.
The node items of this source function as data items for the <code>edgeCreator</code>.
The {@link NodeCreator#defaults node defaults} of this newly created source cascade with the defaults of this source.
Type parameter <code>TSuccessorDataItem</code>: The type of the successor data items in the source.
@param successorsProvider A function that yields the successor data items for each node data item associated with this {@link AdjacencyNodesSource}.
@param edgeCreator The creator that is used to create the edges.
@param [idProvider=null] An optional function that yields an id for each target node data item.
@returns A new {@link AdjacencyNodesSource} for the node data items returned by <code>successorsProvider</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjacencyNodesSource%23createSuccessorsSource(function(TDataItem)%3AObject%2CEdgeCreator%3CTDataItem%3E%2Cfunction(TSuccessorDataItem%2CObject)%3AObject) Online Documentation}*/
createSuccessorsSource<TSuccessorDataItem>(successorsProvider: ((data: TDataItem) => TSuccessorDataItem[] | Iterable<TSuccessorDataItem> | Map<any, TSuccessorDataItem> | {
[id: string]: TSuccessorDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TSuccessorDataItem[] | Iterable<TSuccessorDataItem> | Map<any, TSuccessorDataItem> | {
[id: string]: TSuccessorDataItem;
}>, edgeCreator: EdgeCreator<TDataItem>, idProvider?: ((dataItem: TSuccessorDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TSuccessorDataItem>): AdjacencyNodesSource<TSuccessorDataItem>;
static $class: Class;
private $_AdjacencyNodesSource: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDataItem>(o: any): o is AdjacencyNodesSource<TDataItem>;
}
/** Automatic conversion to {@link Provider}. */
type ProviderConvertible<TDataItem, TValue> = ((data: TDataItem) => TValue | undefined) | string;
/** Automatic conversion to {@link IdProvider}. */
type IdProviderConvertible<TDataItem> = string;
export interface ObjectBindings<TDataItem> extends YObject {
}
/**
* A container for providers for specified properties of an object that provide the values that can later be {@link ObjectBindings#applyBindings applied} to
* these properties.
* {@link ObjectBindings} are used in NodeCreator.{@link NodeCreator#styleBindings}, EdgeCreator.{@link EdgeCreator#styleBindings} and LabelCreator.{@link LabelCreator#styleBindings} and can be used to configure the properties of the
* {@link INodeStyle}, {@link IEdgeStyle} or {@link ILabelStyle} assigned to a node, edge or label.
*
* The following sample shows how to use {@link NodeCreator#styleBindings} to configure the style of a node:
*
* <pre>
* nodesSource.nodeCreator.defaults.shareStyleInstance = false
* nodesSource.nodeCreator.defaults.style = new ShapeNodeStyle({
*   stroke: 'darkOrange',
*   fill: 'lightYellow',
*   shape: 'roundRectangle'
* })
* nodesSource.nodeCreator.styleBindings.addBinding('stroke', employee =>
*   employee.position.contains('Chief') ? 'darkRed' : 'darkOrange'
* )
* nodesSource.nodeCreator.styleBindings.addBinding('shape', employee =>
*   employee.freelancer ? 'hexagon' : 'roundRectangle'
* )
* </pre>
*
* Type parameter <code>TDataItem</code>: The type of the data item these bindings will use to resolve the values from.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ObjectBindings Online Documentation}
*/
export class ObjectBindings<TDataItem> {
constructor();
/**
Registers another binding between the property with the <code>propertyName</code> and a <code>provider</code> that provides the value for this
property.
When {@link ObjectBindings#applyBindings} is called later, each binding that was previously registered via {@link ObjectBindings#addBinding} is applied to the given
target object:
<ul>
<li>First the <code>provider</code> is called which returns the new value of the property.
<li>Second the property of the target object which matches the <code>propertyName</code> is determined.
<li>Finally the value returned by the provider is set on this property.
</ul>
Type parameter <code>TValue</code>: The type of the property.
@param propertyName The name of the property.
@param provider The provider that will be evaluated during {@link ObjectBindings#applyBindings}.
@see ObjectBindings#applyBindings
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObjectBindings%23addBinding(string%2Cfunction(TDataItem)%3ATValue) Online Documentation}*/
addBinding<TValue>(propertyName: string, provider: ProviderConvertible<TDataItem, TValue> | ((dataItem: TDataItem) => TValue | null)): void;
/**
Applies all bindings {@link ObjectBindings#addBinding added} earlier to the <code>target</code> using the <code>dataItem</code> in the provider of the binding.
Applying each binding involves several steps:
<ul>
<li>First the provider of the binding is called with the <code>dataItem</code> and returns the new value of the property.
<li>Second the property of the <code>target</code> which matches the property name of the binding is determined.
<li>Finally the value returned by the provider is set on this property.
</ul>
@param target The target object to whose properties the bindings shall be applied.
@param dataItem The data item the provider of each binding is called with.
@see ObjectBindings#addBinding
@see {@link https://docs.yworks.com/yfileshtml/#/api/ObjectBindings%23applyBindings(Object%2CTDataItem) Online Documentation}*/
applyBindings(target: any, dataItem: TDataItem): void;
static $class: Class;
private $_ObjectBindings: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDataItem>(o: any): o is ObjectBindings<TDataItem>;
}
/** Automatic conversion to {@link ObjectBindings}. */
type ObjectBindingsConvertible<TDataItem> = {
[propertyName: string]: (data: TDataItem) => any;
};
export interface EdgeCreator<TDataItem> extends YObject {
}
/**
This class can be used to create and update {@link IEdge}s based on data items.
These data items can be any business or arbitrary data which should be represented by edges.
{@link EdgeCreator} allows for conveniently binding properties of the data item to the properties of the created {@link IEdge} to
control aspects like the styling.
This class can be used both standalone as well as in conjunction with any of the <code>GraphBuilders</code>, e.g.
{@link GraphBuilder}.
In the context of the {@link GraphBuilder}, it is responsible to create the items from the associated {@link EdgesSource}s.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator Online Documentation}*/
export class EdgeCreator<TDataItem> {
/**
Creates a new creator for edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator Online Documentation}*/
constructor();
/**
Adds the given <code>labelsSource</code> to this instance.
Type parameter <code>TLabelDataItem</code>: The type of the data items in the source.
@param dataProvider A function that provides the label data items from the data item.
@param labelsSource The source to which the data is added.
@see EdgeCreator#createLabelsSource
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23addLabelsSource(function(TDataItem)%3AObject%2CLabelsSource%3CTLabelDataItem%3E) Online Documentation}*/
addLabelsSource<TLabelDataItem>(dataProvider: ((data: TDataItem) => TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
}>, labelsSource: LabelsSource<TLabelDataItem>): void;
/**
Resolves the {@link EdgeCreator#styleBindings} against the given <code>dataItem</code> and applies them to the given <code>edge</code> style.
The {@link EdgeCreator#styleProvider} is not evaluated by this method.
@param graph The managed graph.
@param edge The edge to update.
@param dataItem The data item that is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23applyStyleBindings(IGraph%2CIEdge%2CTDataItem) Online Documentation}*/
applyStyleBindings(graph: IGraph, edge: IEdge, dataItem: TDataItem): void;
/**
Creates an edge in the graph with the values of the bindings resolved against the <code>dataItem</code>.
This method determines the values to pass on to {@link EdgeCreator#createEdgeCore}.
@param graph The graph to create the edge in.
@param source The source node.
@param target The target node.
@param dataItem The data item from which the edge is created.
@returns The newly created edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23createEdge(IGraph%2CINode%2CINode%2CTDataItem) Online Documentation}*/
createEdge(graph: IGraph, source: INode, target: INode, dataItem: TDataItem): IEdge;
/**
Called from {@link EdgeCreator#createEdge} and performs the actual edge creation in the graph.
@param graph The graph to create the edge in.
@param source The source node.
@param target The target node.
@param style The style of the edge.
@param tag The tag of the edge.
@returns The newly created edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23createEdgeCore(IGraph%2CINode%2CINode%2CIEdgeStyle%2CObject) Online Documentation}*/
createEdgeCore(graph: IGraph, source: INode, target: INode, style: IEdgeStyle, tag: any | null): IEdge;
/**
Creates a binding for labels that will be added to edges created by this instance.
Use this method over {@link EdgeCreator#createLabelsSource} if the created edges have a predetermined amount of labels. Note that the item
type of the returned {@link LabelCreator} is the same as for this {@link EdgeCreator}.
@param [text=null] An optional provider for the {@link ILabel#text} property.
@returns A new {@link LabelCreator} instance that can be further configured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23createLabelBinding(function(TDataItem)%3Astring) Online Documentation}*/
createLabelBinding(text?: ((dataItem: TDataItem) => string | null) | null | ProviderConvertible<TDataItem, string>): LabelCreator<TDataItem>;
/**
Creates a binding for labels that will be added to edges created by this instance.
Use this method over {@link EdgeCreator#createLabelsSource} if the created edges have a predetermined amount of labels. Note that the item
type of the returned {@link LabelCreator} is the same as for this {@link EdgeCreator}.
@param options The parameters to pass.
@param [options.text=null] An optional provider for the {@link ILabel#text} property.
@param [options.defaults] A set of {@link ILabelDefaults} that will be used for the labels. This option sets the {@link LabelCreator#defaults} property on the created object.
@param [options.layoutParameter] An optional provider of a label model parameter that will be used as the label's {@link ILabel#layoutParameter} for the source data item. This option sets the {@link LabelCreator#layoutParameterProvider} property on the created object.
@param [options.style] An optional provider of an {@link ILabelStyle} instance that will be used as the label's {@link ILabel#style} for the source data item. This option sets the {@link LabelCreator#styleProvider} property on the created object.
@param [options.preferredSize] An optional provider of the size that will be used as the label's {@link ILabel#preferredSize} for the source data item. This option sets the {@link LabelCreator#preferredSizeProvider} property on the created object.
@param [options.tag] An optional provider of an object that will be used as the label's {@link ITagOwner#tag} for the source data item. This option sets the {@link LabelCreator#tagProvider} property on the created object.
@param [options.styleBindings] An optional set of bindings that will be {@link LabelCreator#applyStyleBindings applied} to the label's {@link ILabel#style}. This option sets the {@link LabelCreator#styleBindings} property on the created object.
@param [options.preferredSizeBindings] An optional binding that will be {@link LabelCreator#applyPreferredSizeBindings applied} to the label. This option sets the {@link LabelCreator#preferredSizeBindings} property on the created object.
@returns A new {@link LabelCreator} instance that can be further configured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23createLabelBinding(function(TDataItem)%3Astring) Online Documentation}*/
createLabelBinding(options: {
text?: ((dataItem: TDataItem) => string | null) | null | ProviderConvertible<TDataItem, string>;
defaults?: ILabelDefaults;
layoutParameter?: ((dataItem: TDataItem) => ILabelModelParameter | null) | null | ProviderConvertible<TDataItem, ILabelModelParameter>;
style?: ((dataItem: TDataItem) => ILabelStyle | null) | null | ProviderConvertible<TDataItem, ILabelStyle>;
preferredSize?: ((dataItem: TDataItem) => Size | null) | null | ProviderConvertible<TDataItem, Size>;
tag?: ((dataItem: TDataItem) => any | null) | null | ProviderConvertible<TDataItem, any>;
styleBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
preferredSizeBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
}): LabelCreator<TDataItem>;
/**
Creates a new source of label data items that will be added to edges created by this instance.
Use this method over {@link EdgeCreator#createLabelBinding} if the created edges have a varying number of labels. Note that the <code>data</code>
introduces a new item type for the returned {@link LabelsSource}.
Type parameter <code>TLabelDataItem</code>: The type of the data items in the source.
@param data A function that provides a collection of label data items from the edge data item.
@param [id=null] An optional function that yields an id for each label data item that is provided from the label data provider. This id
is used to identify the labels during {@link EdgeCreator#updateLabels}.
@returns A new {@link LabelsSource} instance whose {@link LabelCreator} can be configured further.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23createLabelsSource(function(TDataItem)%3AObject%2Cfunction(TLabelDataItem%2CObject)%3AObject) Online Documentation}*/
createLabelsSource<TLabelDataItem>(data: ((data: TDataItem) => TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
}>, id?: ((dataItem: TLabelDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TLabelDataItem>): LabelsSource<TLabelDataItem>;
/**
Creates a new source of label data items that will be added to edges created by this instance.
Use this method over {@link EdgeCreator#createLabelBinding} if the created edges have a varying number of labels. Note that the <code>data</code>
introduces a new item type for the returned {@link LabelsSource}.
Type parameter <code>TLabelDataItem</code>: The type of the data items in the source.
@param options The parameters to pass.
@param options.data A function that provides a collection of label data items from the edge data item.
@param [options.id=null] An optional function that yields an id for each label data item that is provided from the label data provider. This id
is used to identify the labels during {@link EdgeCreator#updateLabels}.
@param [options.defaults] A set of {@link ILabelDefaults} that will be used for the labels. This option sets the {@link LabelCreator#defaults} property on the created object.
@param [options.text] An optional provider of the text that will be used as the label's {@link ILabel#text} for the source data item. This option sets the {@link LabelCreator#textProvider} property on the created object.
@param [options.layoutParameter] An optional provider of a label model parameter that will be used as the label's {@link ILabel#layoutParameter} for the source data item. This option sets the {@link LabelCreator#layoutParameterProvider} property on the created object.
@param [options.preferredSize] An optional provider of the size that will be used as the label's {@link ILabel#preferredSize} for the source data item. This option sets the {@link LabelCreator#preferredSizeProvider} property on the created object.
@param [options.style] An optional provider of an {@link ILabelStyle} instance that will be used as the label's {@link ILabel#style} for the source data item. This option sets the {@link LabelCreator#styleProvider} property on the created object.
@param [options.tag] An optional provider of an object that will be used as the label's {@link ITagOwner#tag} for the source data item. This option sets the {@link LabelCreator#tagProvider} property on the created object.
@param [options.styleBindings] An optional set of bindings that will be {@link LabelCreator#applyStyleBindings applied} to the label's {@link ILabel#style}. This option sets the {@link LabelCreator#styleBindings} property on the created object.
@param [options.preferredSizeBindings] An optional binding that will be {@link LabelCreator#applyPreferredSizeBindings applied} to the label. This option sets the {@link LabelCreator#preferredSizeBindings} property on the created object.
@returns A new {@link LabelsSource} instance whose {@link LabelCreator} can be configured further.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23createLabelsSource(function(TDataItem)%3AObject%2Cfunction(TLabelDataItem%2CObject)%3AObject) Online Documentation}*/
createLabelsSource<TLabelDataItem>(options: {
data: ((data: TDataItem) => TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
}>;
id?: ((dataItem: TLabelDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TLabelDataItem>;
defaults?: ILabelDefaults;
text?: ((dataItem: any) => string | null) | null | ProviderConvertible<any, string>;
layoutParameter?: ((dataItem: any) => ILabelModelParameter | null) | null | ProviderConvertible<any, ILabelModelParameter>;
preferredSize?: ((dataItem: any) => Size | null) | null | ProviderConvertible<any, Size>;
style?: ((dataItem: any) => ILabelStyle | null) | null | ProviderConvertible<any, ILabelStyle>;
tag?: ((dataItem: any) => any | null) | null | ProviderConvertible<any, any>;
styleBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
preferredSizeBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
}): LabelsSource<TLabelDataItem>;
/**
Obtains an {@link IEdgeStyle} instance or <code>null</code> by resolving the {@link EdgeCreator#styleProvider}.
In case of <code>null</code>, the {@link EdgeCreator#defaults} are used.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case the defaults should be used.
@see EdgeCreator#styleProvider
@see EdgeCreator#styleBindings
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23getStyle(TDataItem) Online Documentation}*/
getStyle(dataItem: TDataItem): IEdgeStyle | null;
/**
Obtains an <code>object</code> to use as {@link ITagOwner#tag} by resolving the {@link EdgeCreator#tagProvider} on the data item.
The tag can be any value and does not need to be related to the original source data item or it's ID.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case the defaults should be used.
@see EdgeCreator#tagProvider
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23getTag(TDataItem) Online Documentation}*/
getTag(dataItem: TDataItem): any | null;
/**
Resolves the {@link EdgeCreator#styleProvider} and then {@link EdgeCreator#applyStyleBindings applies the style bindings}.
This method is called by {@link EdgeCreator#updateStyle}.
@param graph The managed graph.
@param edge The edge to update.
@param dataItem The data item that is used.
@returns The updated style instance. Depending on {@link EdgeCreator#styleProvider} it is either a new instance or the edge's style instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23getUpdatedStyle(IGraph%2CIEdge%2CTDataItem) Online Documentation}*/
getUpdatedStyle(graph: IGraph, edge: IEdge, dataItem: TDataItem): IEdgeStyle;
/**
Resolves the {@link EdgeCreator#tagProvider} on the given data item.
This method is called by {@link EdgeCreator#updateTag}. By default, it only delegates to {@link EdgeCreator#getTag}.
@param graph The managed graph.
@param edge The edge for which the new tag should be obtained.
@param dataItem The data item that is used.
@returns Returns the updated tag object.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23getUpdatedTag(IGraph%2CIEdge%2CTDataItem) Online Documentation}*/
getUpdatedTag(graph: IGraph, edge: IEdge, dataItem: TDataItem): any | null;
/**
Triggers the {@link EdgeCreator#addEdgeCreatedListener EdgeCreated} event.
@param graph The graph in which the edge has been created.
@param edge The edge that has been created.
@param dataItem The data item from which the edge has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23onEdgeCreated(IGraph%2CIEdge%2CTDataItem) Online Documentation}*/
onEdgeCreated(graph: IGraph, edge: IEdge, dataItem: TDataItem): void;
/**
Triggers the {@link EdgeCreator#addEdgeUpdatedListener EdgeUpdated} event.
@param graph The graph in which the edge has been updated.
@param edge The edge that has been updated.
@param dataItem The data item with which the edge has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23onEdgeUpdated(IGraph%2CIEdge%2CTDataItem) Online Documentation}*/
onEdgeUpdated(graph: IGraph, edge: IEdge, dataItem: TDataItem): void;
/**
Updates the edge.
By default, this method only fires the {@link EdgeCreator#addEdgeUpdatedListener EdgeUpdated} event.
To update other aspects of the edge instance, call the respective update methods when the edge is {@link EdgeCreator#addEdgeUpdatedListener updated} or overwrite
this method.
@param graph The graph that contains the edge.
@param edge The edge to update.
@param dataItem The data item with which the edge should be updated.
@see EdgeCreator#updateStyle
@see EdgeCreator#updateLabels
@see EdgeCreator#updateTag
@see EdgeCreator#applyStyleBindings
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23updateEdge(IGraph%2CIEdge%2CTDataItem) Online Documentation}*/
updateEdge(graph: IGraph, edge: IEdge, dataItem: TDataItem): void;
/**
Can be used to update the labels of the edge that have been added with this {@link EdgeCreator}.
This only adds / removes labels based on the bound data. To update bound properties for the labels, call the respective
update methods on the {@link LabelCreator}.
@param graph The managed graph.
@param edge The edge whose labels to update.
@param dataItem The edge data item with which the edge's labels should be updated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23updateLabels(IGraph%2CIEdge%2CTDataItem) Online Documentation}*/
updateLabels(graph: IGraph, edge: IEdge, dataItem: TDataItem): void;
/**
Updates the style of the <code>edge</code> with the given <code>dataItem</code> by calling {@link EdgeCreator#getUpdatedStyle} and applying the style to the edge in
the graph.
@param graph The managed graph.
@param edge The edge to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23updateStyle(IGraph%2CIEdge%2CTDataItem) Online Documentation}*/
updateStyle(graph: IGraph, edge: IEdge, dataItem: TDataItem): void;
/**
Updates the {@link ITagOwner#tag} of the <code>edge</code> with the given <code>dataItem</code> by calling {@link EdgeCreator#getUpdatedTag} and setting the new <code>tag</code> on the edge.
@param graph The managed graph.
@param edge The edge to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23updateTag(IGraph%2CIEdge%2CTDataItem) Online Documentation}*/
updateTag(graph: IGraph, edge: IEdge, dataItem: TDataItem): void;
/**
Gets or sets an optional provider of an object that will be used as the edges' {@link ITagOwner#tag} for the source data item.
By default the source data item itself will be used as the tag.
@see EdgeCreator#getTag
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23tagProvider Online Documentation}*/
tagProvider: ((dataItem: TDataItem) => any | null) | null;
/**
Gets or sets an optional provider of an {@link IEdgeStyle} instance that will be used as the edge's {@link IEdge#style} for the
source data item.
By default the {@link EdgeCreator#defaults}' {@link IEdgeDefaults#style} will be used. Note that {@link IEdgeDefaults#shareStyleInstance} will be ignored if this provider returns a value. Use the {@link EdgeCreator#styleBindings} to reconfigure
properties of the style instances.
@see EdgeCreator#getStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23styleProvider Online Documentation}*/
styleProvider: ((dataItem: TDataItem) => IEdgeStyle | null) | null;
/**
Gets or sets an optional set of bindings that will be {@link EdgeCreator#applyStyleBindings applied} to the edge's {@link IEdge#style}.
Note that bindings will affect the properties of the instance created by {@link EdgeCreator#getStyle}. If the style is obtained from the {@link EdgeCreator#defaults}
with {@link IEdgeDefaults#shareStyleInstance} set to <code>true</code>, the properties of the same instance are overwritten each time. To get fresh copies, set {@link IEdgeDefaults#shareStyleInstance} to <code>false</code>.
@see EdgeCreator#applyStyleBindings
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23styleBindings Online Documentation}*/
styleBindings: ObjectBindings<TDataItem>;
/**
Gets or sets a set of {@link IEdgeDefaults} that will be used for the edges and edge labels.
The defaults of instances of this class created by one of the <code>GraphBuilders</code>, e.g. {@link GraphBuilder}, cascade
with the defaults of the GraphBuilder's {@link GraphBuilder#graph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23defaults Online Documentation}*/
defaults: IEdgeDefaults;
/**
Adds the given listener for the <code>EdgeCreated</code> event that occurs when a edge has been created.
@param listener The listener to add.
@see EdgeCreator#addEdgeUpdatedListener
@see EdgeCreator#removeEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23EdgeCreated Online Documentation}*/
addEdgeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, TDataItem>) => void): void;
/**
Removes the given listener for the <code>EdgeCreated</code> event that occurs when a edge has been created.
@param listener The listener to remove.
@see EdgeCreator#addEdgeUpdatedListener
@see EdgeCreator#addEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23EdgeCreated Online Documentation}*/
removeEdgeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, TDataItem>) => void): void;
/**
Adds the given listener for the <code>EdgeUpdated</code> event that occurs when a edge has been updated.
@param listener The listener to add.
@see EdgeCreator#addEdgeCreatedListener
@see EdgeCreator#removeEdgeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23EdgeUpdated Online Documentation}*/
addEdgeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, TDataItem>) => void): void;
/**
Removes the given listener for the <code>EdgeUpdated</code> event that occurs when a edge has been updated.
@param listener The listener to remove.
@see EdgeCreator#addEdgeCreatedListener
@see EdgeCreator#addEdgeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCreator%23EdgeUpdated Online Documentation}*/
removeEdgeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, TDataItem>) => void): void;
static $class: Class;
private $_EdgeCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDataItem>(o: any): o is EdgeCreator<TDataItem>;
}
export interface EdgesSource<TDataItem> extends YObject {
}
/**
A class that accepts a collection of objects from the {@link GraphBuilder} to build {@link IEdge}s from.
It manages the structural aspects of the graph creation, while the associated {@link EdgesSource#edgeCreator} is responsible for the properties of the
created items.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgesSource Online Documentation}*/
export class EdgesSource<TDataItem> {
/**
Creates a new source with the given <code>edgeCreator</code>.
@param edgeCreator The creator that is used to create the edges of this source.
@param sourceIdProvider A function that yields a source node id for each element of the associated data collection in the
{@link GraphBuilder}.
@param targetIdProvider A function that yields a target node id for each element of the associated data collection in the
{@link GraphBuilder}.
@param [idProvider=null] An optional function that yields an id for each element of the associated data collection in the
{@link GraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgesSource Online Documentation}*/
constructor(edgeCreator: EdgeCreator<TDataItem>, sourceIdProvider: ProviderConvertible<TDataItem, any> | ((dataItem: TDataItem) => any | null), targetIdProvider: ProviderConvertible<TDataItem, any> | ((dataItem: TDataItem) => any | null), idProvider?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>);
/**
Obtains the source node id for each data item for which an edge is created.
This implementation uses the {@link EdgesSource#sourceIdProvider} to obtain the value from the data item.
@param dataItem A single data item from the the associated data collection in the {@link GraphBuilder}.
@returns The source node id to which the {@link EdgesSource#sourceIdProvider} resolves with the data item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgesSource%23getSourceId(TDataItem) Online Documentation}*/
getSourceId(dataItem: TDataItem): any | null;
/**
Obtains the target node id for each data item for which an edge is created.
This implementation uses the {@link EdgesSource#targetIdProvider} to obtain the value from the data item.
@param dataItem A single data item from the the associated data collection in the {@link GraphBuilder}.
@returns The target node id to which the {@link EdgesSource#targetIdProvider} resolves with the data item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgesSource%23getTargetId(TDataItem) Online Documentation}*/
getTargetId(dataItem: TDataItem): any | null;
/**
Gets the provider that yields a source node id for each element of the associated data collection in the
{@link GraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgesSource%23sourceIdProvider Online Documentation}*/
readonly sourceIdProvider: (dataItem: TDataItem) => any | null;
/**
Gets the provider that yields a target node id for each element of the associated data collection in the
{@link GraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgesSource%23targetIdProvider Online Documentation}*/
readonly targetIdProvider: (dataItem: TDataItem) => any | null;
/**
Gets or sets a provider that yields an id for each element of the associated data collection in the
{@link GraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgesSource%23idProvider Online Documentation}*/
idProvider: ((dataItem: TDataItem, canonicalId: any | null) => any) | null;
/**
Gets or sets the {@link EdgeCreator} for this source which can be used to adjust the creation behavior.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgesSource%23edgeCreator Online Documentation}*/
edgeCreator: EdgeCreator<TDataItem>;
static $class: Class;
private $_EdgesSource: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDataItem>(o: any): o is EdgesSource<TDataItem>;
}
export interface GraphBuilder extends YObject {
}
/**
* Populates a graph from custom data.
* This class can be used when the data consists of one or more collections of nodes, edges, and optionally, groups. The
* methods {@link GraphBuilder#createNodesSource}, {@link GraphBuilder#createGroupNodesSource}, and {@link GraphBuilder#createEdgesSource}
* bind the data to source objects from which nodes, groups, and edges will be created.
*
* Generally, using the {@link GraphBuilder} class consists of a few basic steps:
*
* <ol>
* <li>Create a {@link GraphBuilder} and optionally, provide an {@link IGraph} on which it should operate.
*
* <pre>
* const graphBuilder = new GraphBuilder(graph)
* </pre>
* </li>
* <li>Create one or more <code>sources</code> from which the {@link GraphBuilder} should create the graph structure
* ({@link GraphBuilder#createNodesSource}, {@link GraphBuilder#createGroupNodesSource},
* {@link GraphBuilder#createEdgesSource}).
*
* To create edges from the data, provide {@link EdgesSource#sourceIdProvider} and {@link EdgesSource#targetIdProvider} with {@link GraphBuilder#createEdgesSource} and a respective <code>idProvider</code>
* for the {@link NodesSource} to tell the {@link GraphBuilder} how the data is structured.
*
* <pre>
* const nodesData = getNodesData()
* const nodesSource = graphBuilder.createNodesSource(
*   nodesData, // nodes data
*   nodeDataItem => nodeDataItem.nodeId // node id provider
* )
* const edgesData = getEdgesData()
* const edgesSource = graphBuilder.createEdgesSource(
*   edgesData,
*   edgeDataItem => edgeDataItem.sourceNodeId,
*   edgeDataItem => edgeDataItem.targetNodeId
* )
* </pre>
* </li>
* <li>Each {@link NodesSource} provides a {@link NodeCreator} that allows for further specification of how the items from this source should be created. You can
* provide {@link NodeCreator#defaults} for the default styling but also bind more specific styling based on the actual data item with the {@link NodeCreator#styleProvider} and {@link NodeCreator#styleBindings}.
*
* <pre>
* nodesSource.nodeCreator.defaults.shareStyleInstance = false
* nodesSource.nodeCreator.defaults.style = new ShapeNodeStyle({
*   stroke: 'darkOrange',
*   fill: 'lightYellow',
*   shape: 'roundRectangle'
* })
* nodesSource.nodeCreator.styleBindings.addBinding('stroke', employee =>
*   employee.position.contains('Chief') ? 'darkRed' : 'darkOrange'
* )
* nodesSource.nodeCreator.styleBindings.addBinding('shape', employee =>
*   employee.freelancer ? 'hexagon' : 'roundRectangle'
* )
* </pre>
*
* It also allows for applying layout information from the data item ({@link NodeCreator#layoutProvider}), as well as further specification of the created
* node's {@link ITagOwner#tag} ({@link NodeCreator#tagProvider}).</li>
* <li>Similarly, each {@link EdgesSource} provides an {@link EdgeCreator} that allows for specifying the styling of the created items ({@link EdgeCreator#defaults}, {@link EdgeCreator#styleProvider}, {@link EdgeCreator#styleBindings}) and the edge's {@link ITagOwner#tag} ({@link EdgeCreator#tagProvider}).
*
* <pre>
* edgesSource.edgeCreator.defaults.Style = new PolylineEdgeStyle({ smoothingLength: 20 })
* </pre>
* </li>
* <li>Optionally, labels can be added by providing one or multiple {@link NodeCreator#createLabelBinding label bindings}. If there is a varying number of labels per
* node, {@link NodeCreator#createLabelsSource} can be used, instead. Similar methods are also available on the {@link EdgeCreator}.
*
* The {@link LabelsSource} provides a {@link LabelCreator} that allows for adjusting all aspects of the label creation, like style, text, size, and tag.
*
* <pre>
* const edgeLabelCreator = edgesSource.edgeCreator.createLabelBinding(
*   edgeDataItem => edgeDataItem.name
* )
* edgeLabelCreator.defaults.style = new DefaultLabelStyle({
*   backgroundFill: 'rgb(225,242,253)',
*   backgroundStroke: 'lightskyblue',
*   textSize: 8
* })
* </pre>
* </li>
* <li>Call {@link GraphBuilder#buildGraph} to populate the graph. You can apply a layout algorithm afterward to make
* the graph look nice.
*
* <pre>
* // Build a graph initially
* graphBuilder.buildGraph()
* // Apply a layout in a subsequent step
* graph.applyLayout(new OrganicLayout())
* </pre>
* </li>
* <li>If your items or collections change later, call {@link GraphBuilder#updateGraph} to make the changes visible in
* the graph. If the data item instances have changed, you can call {@link GraphBuilder#setData} or
* {@link GraphBuilder#setData}.
*
* <pre>
* // Set the new data collections:
* graphBuilder.setData(nodesSource, getNodesData())
* graphBuilder.setData(edgesSource, getEdgesData())
* // Update a graph after the business data has changed
* graphBuilder.updateGraph()
* // Apply a layout in a subsequent step
* graph.applyLayout(new OrganicLayout())
* </pre>
* </li>
* </ol>
* You can further customize how nodes, groups, and edges are created by adding event handlers to the various events and
* modifying the items there. This can be used to modify items in ways that are not directly supported by the available
* bindings or defaults. There are creation and update events for all items, which allow for separate customization when
* nodes, groups, and edges are created or updated. For completely static graphs, where {@link GraphBuilder#updateGraph}
* is not needed, the update events can be safely ignored.
*
* <h2>Note</h2>
*
* This class serves as a convenient way to create general graphs. Some aspects to look out for:
*
* <ul>
* <li>{@link GraphBuilder#buildGraph} does not clear the graph. If needed, call {@link IGraph#clear} to empty the
* graph before building it anew.</li>
* <li>The {@link GraphBuilder} ignores and preserves items manually created on the graph in between calls to
* {@link GraphBuilder#updateGraph}.</li>
* </ul>
* If you need to extensively modify your data in order to fit the requirements of the {@link GraphBuilder}, it's
* often better to write the code interfacing with the graph by hand instead of relying on {@link GraphBuilder}.
*
* <h2>Note</h2>
*
* {@link GraphBuilder#updateGraph} merely updates the graph structure. Any other bound data must be updated
* explicitly if necessary. Calling the respective <code>update</code> methods on the <code>creators</code> (e.g. {@link NodeCreator#updateStyle}), resolves the <code>provider</code>
* (e.g. {@link NodeCreator#styleProvider}) and applies the <code>bindings</code> (e.g.
* {@link NodeCreator#applyStyleBindings}). The latter can also be applied solely.
* <br />
* For example, to update any aspect of node creation:
*
* <pre>
* // evaluate the providers and apply respective bindings
* nodesSource.nodeCreator.addNodeUpdatedListener((sender, evt) => {
*   nodesSource.nodeCreator.updateLayout(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateStyle(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateTag(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateLabels(evt.graph, evt.item, evt.dataItem)
* })
*
* builder.updateGraph()
* </pre>
*
* The different graph builders are discussed in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph_builder Creating a Graph from Business Data}. Class <code>GraphBuilder</code>, in
* particular, is topic of section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph_builder-GraphBuilder GraphBuilder}.
*
* @see TreeBuilder
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder Online Documentation}
*/
export class GraphBuilder {
/**
Initializes a new instance of the {@link GraphBuilder} class that operates on the given graph.
The <code>graph</code> will not be cleared when {@link GraphBuilder#buildGraph} is called. To clear the graph beforehand, call
{@link IGraph#clear}.
@param [graph=null] An {@link IGraph} instance on which this builder operates or <code>null</code> if a new {@link DefaultGraph} should be
created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder Online Documentation}*/
constructor(graph?: IGraph | null);
/**
Initializes a new instance of the {@link GraphBuilder} class that operates on the given graph.
The <code>graph</code> will not be cleared when {@link GraphBuilder#buildGraph} is called. To clear the graph beforehand, call
{@link IGraph#clear}.
@param options The parameters to pass.
@param [options.graph=null] An {@link IGraph} instance on which this builder operates or <code>null</code> if a new {@link DefaultGraph} should be
created.
@param [options.nodes]
@param [options.edges]
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder Online Documentation}*/
constructor(options: {
graph?: IGraph | null;
nodes?: Array<any>;
edges?: Array<any>;
});
/**
Binds a collection of data items to the given <code>edgesSource</code>.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param data The collection of objects that is bound to the source.
@param edgesSource The source to which the data is bound.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23addEdgesSource(Object%2CEdgesSource%3CTDataItem%3E) Online Documentation}*/
addEdgesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, edgesSource: EdgesSource<TDataItem>): void;
/**
Binds a collection of data items to the given <code>nodesSource</code>.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param data The collection of objects that is bound to the source.
@param nodesSource The source to which the data is bound.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23addNodesSource(Object%2CNodesSource%3CTDataItem%3E) Online Documentation}*/
addNodesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, nodesSource: NodesSource<TDataItem>): void;
/**
Populates the graph with items generated from the bound data.
This graph is not cleared before the new items are added. To clear the graph, explicitly call {@link IGraph#clear}
beforehand.
@returns The {@link GraphBuilder#graph} of this builder populated by the defined sources.
@see GraphBuilder#updateGraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23buildGraph() Online Documentation}*/
buildGraph(): IGraph;
/**
Creates a new {@link EdgesSource} and binds a collection of edge data items to it.
The {@link EdgeCreator} of the returned {@link EdgesSource} can be used to further specify creation properties of these items, e.g. style, tag, etc.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param data The collection of objects to iterate and create the edges from.
@param sourceId A provider that defines the source of the edge in the graph structure. Must resolve to the specified node id.
@param targetId A provider that defines the target of the edge in the graph structure. Must resolve to the specified node id.
@param [id=null] An optional function that yields an id for each element in the <code>data</code>. This id is used to identify the edges during
{@link GraphBuilder#updateGraph}.
@returns A new {@link EdgesSource} instance that can be used to further customize the creation of edges, e.g. provide specific {@link EdgeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23createEdgesSource(Object%2Cfunction(TDataItem)%3AObject%2Cfunction(TDataItem)%3AObject%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createEdgesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, sourceId: ProviderConvertible<TDataItem, any> | ((dataItem: TDataItem) => any | null), targetId: ProviderConvertible<TDataItem, any> | ((dataItem: TDataItem) => any | null), id?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>): EdgesSource<TDataItem>;
/**
Creates a new {@link EdgesSource} and binds a collection of edge data items to it.
The {@link EdgeCreator} of the returned {@link EdgesSource} can be used to further specify creation properties of these items, e.g. style, tag, etc.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param options The parameters to pass.
@param options.data The collection of objects to iterate and create the edges from.
@param options.sourceId A provider that defines the source of the edge in the graph structure. Must resolve to the specified node id.
@param options.targetId A provider that defines the target of the edge in the graph structure. Must resolve to the specified node id.
@param [options.id=null] An optional function that yields an id for each element in the <code>data</code>. This id is used to identify the edges during
{@link GraphBuilder#updateGraph}.
@param [options.defaults] A set of {@link IEdgeDefaults} that will be used for the edges and edge labels. This option sets the {@link EdgeCreator#defaults} property on the created object.
@param [options.style] An optional provider of an {@link IEdgeStyle} instance that will be used as the edge's {@link IEdge#style} for the source data item. This option sets the {@link EdgeCreator#styleProvider} property on the created object.
@param [options.tag] An optional provider of an object that will be used as the edges' {@link ITagOwner#tag} for the source data item. This option sets the {@link EdgeCreator#tagProvider} property on the created object.
@param [options.styleBindings] An optional set of bindings that will be {@link EdgeCreator#applyStyleBindings applied} to the edge's {@link IEdge#style}. This option sets the {@link EdgeCreator#styleBindings} property on the created object.
@param [options.labels]
@returns A new {@link EdgesSource} instance that can be used to further customize the creation of edges, e.g. provide specific {@link EdgeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23createEdgesSource(Object%2Cfunction(TDataItem)%3AObject%2Cfunction(TDataItem)%3AObject%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createEdgesSource<TDataItem>(options: {
data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
};
sourceId: ProviderConvertible<TDataItem, any> | ((dataItem: TDataItem) => any | null);
targetId: ProviderConvertible<TDataItem, any> | ((dataItem: TDataItem) => any | null);
id?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>;
defaults?: IEdgeDefaults;
style?: ((dataItem: any) => IEdgeStyle | null) | null | ProviderConvertible<any, IEdgeStyle>;
tag?: ((dataItem: any) => any | null) | null | ProviderConvertible<any, any>;
styleBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
labels?: Array<any>;
}): EdgesSource<TDataItem>;
/**
Creates a new {@link NodesSource} and binds a collection of group node data items to it.
The {@link NodeCreator} of the returned {@link NodesSource} can be used to further specify creation properties of these items, e.g. style, layout, etc.
Data items of this collection are created as group nodes in the {@link GraphBuilder#graph}.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param data The collection of objects to iterate and create the group nodes from.
@param id A function that yields an ID for each element in the <code>data</code>. If the data has no associated IDs, <code>null</code> may be used as well
in which case the builder uses the objects themselves as identifiers internally.
<br />
This ID is used by {@link NodesSource#parentIdProvider} and {@link EdgesSource#sourceIdProvider} and {@link EdgesSource#targetIdProvider} to resolve the parent, source, or target nodes. The ID is also used to identify nodes
during {@link GraphBuilder#updateGraph}.
@returns A new {@link NodesSource} instance that can be used to further customize the creation of nodes, e.g. provide specific {@link NodeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23createGroupNodesSource(Object%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createGroupNodesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, id: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>): NodesSource<TDataItem>;
/**
Creates a new {@link NodesSource} and binds a collection of group node data items to it.
The {@link NodeCreator} of the returned {@link NodesSource} can be used to further specify creation properties of these items, e.g. style, layout, etc.
Data items of this collection are created as group nodes in the {@link GraphBuilder#graph}.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param options The parameters to pass.
@param options.data The collection of objects to iterate and create the group nodes from.
@param options.id A function that yields an ID for each element in the <code>data</code>. If the data has no associated IDs, <code>null</code> may be used as well
in which case the builder uses the objects themselves as identifiers internally.
<br />
This ID is used by {@link NodesSource#parentIdProvider} and {@link EdgesSource#sourceIdProvider} and {@link EdgesSource#targetIdProvider} to resolve the parent, source, or target nodes. The ID is also used to identify nodes
during {@link GraphBuilder#updateGraph}.
@param [options.parentId] A provider that yields a parent id for each element of the associated data collection in the {@link GraphBuilder}. This option sets the {@link NodesSource#parentIdProvider} property on the created object.
@param [options.defaults] A set of {@link INodeDefaults} that will be used for the nodes and node labels. This option sets the {@link NodeCreator#defaults} property on the created object.
@param [options.layout] An optional provider of a {@link Rect} that will be used as the node's {@link INode#layout} for the source data item. This option sets the {@link NodeCreator#layoutProvider} property on the created object.
@param [options.style] An optional provider of an {@link INodeStyle} instance that will be used as the node's {@link INode#style} for the source data item. This option sets the {@link NodeCreator#styleProvider} property on the created object.
@param [options.tag] An optional provider of an object that will be used as the node's {@link ITagOwner#tag} for the source data item. This option sets the {@link NodeCreator#tagProvider} property on the created object.
@param [options.styleBindings] An optional set of bindings that will be {@link NodeCreator#applyStyleBindings applied} to the node's {@link INode#style}. This option sets the {@link NodeCreator#styleBindings} property on the created object.
@param [options.layoutBindings] An optional set of bindings that will be {@link NodeCreator#applyStyleBindings applied} to the node's {@link INode#style}. This option sets the {@link NodeCreator#layoutBindings} property on the created object.
@param [options.labels]
@returns A new {@link NodesSource} instance that can be used to further customize the creation of nodes, e.g. provide specific {@link NodeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23createGroupNodesSource(Object%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createGroupNodesSource<TDataItem>(options: {
data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
};
id: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>;
parentId?: ((dataItem: TDataItem) => any | null) | null | ProviderConvertible<TDataItem, any>;
defaults?: INodeDefaults;
layout?: ((dataItem: any) => Rect) | null | ProviderConvertible<any, Rect>;
style?: ((dataItem: any) => INodeStyle | null) | null | ProviderConvertible<any, INodeStyle>;
tag?: ((dataItem: any) => any | null) | null | ProviderConvertible<any, any>;
styleBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
layoutBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
labels?: Array<any>;
}): NodesSource<TDataItem>;
/**
Creates a new {@link NodesSource} and binds a collection of node data items to it.
The {@link NodeCreator} of the returned {@link NodesSource} can be used to further specify creation properties of these items, e.g. style, layout, etc.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param data The collection of objects to iterate and create the nodes from.
@param id A function that yields an ID for each element in the <code>data</code>. If the data has no associated IDs, <code>null</code> may be used as well
in which case the builder uses the objects themselves as identifiers internally.
<br />
This ID is used by {@link NodesSource#parentIdProvider} and {@link EdgesSource#sourceIdProvider} and {@link EdgesSource#targetIdProvider} to resolve the parent, source, or target nodes. The ID is also used to identify nodes
during {@link GraphBuilder#updateGraph}.
@returns A new {@link NodesSource} instance that can be used to further customize the creation of nodes, e.g. provide specific {@link NodeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23createNodesSource(Object%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createNodesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, id: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>): NodesSource<TDataItem>;
/**
Creates a new {@link NodesSource} and binds a collection of node data items to it.
The {@link NodeCreator} of the returned {@link NodesSource} can be used to further specify creation properties of these items, e.g. style, layout, etc.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param options The parameters to pass.
@param options.data The collection of objects to iterate and create the nodes from.
@param options.id A function that yields an ID for each element in the <code>data</code>. If the data has no associated IDs, <code>null</code> may be used as well
in which case the builder uses the objects themselves as identifiers internally.
<br />
This ID is used by {@link NodesSource#parentIdProvider} and {@link EdgesSource#sourceIdProvider} and {@link EdgesSource#targetIdProvider} to resolve the parent, source, or target nodes. The ID is also used to identify nodes
during {@link GraphBuilder#updateGraph}.
@param [options.parentId] A provider that yields a parent id for each element of the associated data collection in the {@link GraphBuilder}. This option sets the {@link NodesSource#parentIdProvider} property on the created object.
@param [options.defaults] A set of {@link INodeDefaults} that will be used for the nodes and node labels. This option sets the {@link NodeCreator#defaults} property on the created object.
@param [options.layout] An optional provider of a {@link Rect} that will be used as the node's {@link INode#layout} for the source data item. This option sets the {@link NodeCreator#layoutProvider} property on the created object.
@param [options.style] An optional provider of an {@link INodeStyle} instance that will be used as the node's {@link INode#style} for the source data item. This option sets the {@link NodeCreator#styleProvider} property on the created object.
@param [options.tag] An optional provider of an object that will be used as the node's {@link ITagOwner#tag} for the source data item. This option sets the {@link NodeCreator#tagProvider} property on the created object.
@param [options.styleBindings] An optional set of bindings that will be {@link NodeCreator#applyStyleBindings applied} to the node's {@link INode#style}. This option sets the {@link NodeCreator#styleBindings} property on the created object.
@param [options.layoutBindings] An optional set of bindings that will be {@link NodeCreator#applyStyleBindings applied} to the node's {@link INode#style}. This option sets the {@link NodeCreator#layoutBindings} property on the created object.
@param [options.labels]
@returns A new {@link NodesSource} instance that can be used to further customize the creation of nodes, e.g. provide specific {@link NodeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23createNodesSource(Object%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createNodesSource<TDataItem>(options: {
data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
};
id: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>;
parentId?: ((dataItem: TDataItem) => any | null) | null | ProviderConvertible<TDataItem, any>;
defaults?: INodeDefaults;
layout?: ((dataItem: any) => Rect) | null | ProviderConvertible<any, Rect>;
style?: ((dataItem: any) => INodeStyle | null) | null | ProviderConvertible<any, INodeStyle>;
tag?: ((dataItem: any) => any | null) | null | ProviderConvertible<any, any>;
styleBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
layoutBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
labels?: Array<any>;
}): NodesSource<TDataItem>;
/**
Triggers the {@link GraphBuilder#addEdgeCreatedListener EdgeCreated} event.
@param edge The edge that has been created.
@param dataItem The data item from which the edge has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23onEdgeCreated(IEdge%2CObject) Online Documentation}*/
onEdgeCreated(edge: IEdge, dataItem: any): void;
/**
Triggers the {@link GraphBuilder#addEdgeRemovedListener EdgeRemoved} event.
@param edge The edge that has been removed.
@param dataItem The corresponding data item of the removed edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23onEdgeRemoved(IEdge%2CObject) Online Documentation}*/
onEdgeRemoved(edge: IEdge, dataItem: any): void;
/**
Triggers the {@link GraphBuilder#addEdgeUpdatedListener EdgeUpdated} event.
@param edge The edge that has been updated.
@param dataItem The data item with which the edge has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23onEdgeUpdated(IEdge%2CObject) Online Documentation}*/
onEdgeUpdated(edge: IEdge, dataItem: any): void;
/**
Triggers the {@link GraphBuilder#addLabelAddedListener LabelAdded} event.
@param label The label that has been added.
@param dataItem The data item from which the label has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23onLabelAdded(ILabel%2CObject) Online Documentation}*/
onLabelAdded(label: ILabel, dataItem: any): void;
/**
Triggers the {@link GraphBuilder#addLabelRemovedListener LabelRemoved} event.
@param label The label that has been removed.
@param dataItem The corresponding data item of the removed label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23onLabelRemoved(ILabel%2CObject) Online Documentation}*/
onLabelRemoved(label: ILabel, dataItem: any): void;
/**
Triggers the {@link GraphBuilder#addLabelUpdatedListener LabelUpdated} event.
@param label The label that has been updated.
@param dataItem The data item with which the label has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23onLabelUpdated(ILabel%2CObject) Online Documentation}*/
onLabelUpdated(label: ILabel, dataItem: any): void;
/**
Triggers the {@link GraphBuilder#addNodeCreatedListener NodeCreated} event.
@param node The node that has been created.
@param dataItem The data item from which the node has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23onNodeCreated(INode%2CObject) Online Documentation}*/
onNodeCreated(node: INode, dataItem: any): void;
/**
Triggers the {@link GraphBuilder#addNodeRemovedListener NodeRemoved} event.
@param node The node that has been removed.
@param dataItem The corresponding data item of the removed node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23onNodeRemoved(INode%2CObject) Online Documentation}*/
onNodeRemoved(node: INode, dataItem: any): void;
/**
Triggers the {@link GraphBuilder#addNodeUpdatedListener NodeUpdated} event.
@param node The node that has been updated.
@param dataItem The data item with which the node has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23onNodeUpdated(INode%2CObject) Online Documentation}*/
onNodeUpdated(node: INode, dataItem: any): void;
/**
Binds a new data collection to a {@link EdgesSource}, replacing the old one.
This can be used to assign an entirely different collection instance to an existing <code>edgesSource</code>, such that {@link GraphBuilder#updateGraph}
considers the new collection.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param edgesSource The source whose data source should be re-assigned.
@param data The collection of objects that is specified for the given source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23setData(EdgesSource%3CTDataItem%3E%2CObject) Online Documentation}*/
setData<TDataItem>(edgesSource: EdgesSource<TDataItem>, data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}): void;
/**
Binds a new data collection to a {@link NodesSource}, replacing the old one.
This can be used to assign an entirely different collection instance to an existing <code>nodesSource</code>, such that {@link GraphBuilder#updateGraph}
considers the new collection.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param nodesSource The source whose data source should be re-assigned.
@param data The collection of objects that is specified for the given source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23setData(NodesSource%3CTDataItem%3E%2CObject) Online Documentation}*/
setData<TDataItem>(nodesSource: NodesSource<TDataItem>, data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}): void;
/**
* Updates the graph after changes in the bound data.
* Graph elements corresponding to objects that are still present in the source collections are kept, new graph elements
* are created for new objects in the collections, and obsolete ones are removed.
*
* <h2>Note</h2>
*
* {@link GraphBuilder#updateGraph} merely updates the graph structure. Any other bound data must be updated
* explicitly if necessary. Calling the respective <code>update</code> methods on the <code>creators</code> (e.g. {@link NodeCreator#updateStyle}), resolves the <code>provider</code>
* (e.g. {@link NodeCreator#styleProvider}) and applies the <code>bindings</code> (e.g.
* {@link NodeCreator#applyStyleBindings}). The latter can also be applied solely.
* <br />
* For example, to update any aspect of node creation:
*
* <pre>
* // evaluate the providers and apply respective bindings
* nodesSource.nodeCreator.addNodeUpdatedListener((sender, evt) => {
*   nodesSource.nodeCreator.updateLayout(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateStyle(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateTag(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateLabels(evt.graph, evt.item, evt.dataItem)
* })
*
* builder.updateGraph()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23updateGraph() Online Documentation}
*/
updateGraph(): void;
/**
Gets the {@link IGraph graph} used by this builder.
The style defaults of this graph are considered in the {@link NodeCreator}, {@link EdgeCreator} and {@link LabelCreator} of the sources that are created on this
{@link GraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23graph Online Documentation}*/
readonly graph: IGraph;
/**
Adds the given listener for the <code>NodeCreated</code> event that occurs when a node has been created by any one of the
{@link GraphBuilder#createNodesSource nodes sources}.
This event can be used to further customize the created node.
New nodes are created either in response to calling {@link GraphBuilder#buildGraph}, or in response to calling {@link GraphBuilder#updateGraph}
when there are new items in the {@link NodesSource nodes sources}.
@param listener The listener to add.
@see GraphBuilder#addNodeUpdatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#removeNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23NodeCreated Online Documentation}*/
addNodeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeCreated</code> event that occurs when a node has been created by any one of the
{@link GraphBuilder#createNodesSource nodes sources}.
This event can be used to further customize the created node.
New nodes are created either in response to calling {@link GraphBuilder#buildGraph}, or in response to calling {@link GraphBuilder#updateGraph}
when there are new items in the {@link NodesSource nodes sources}.
@param listener The listener to remove.
@see GraphBuilder#addNodeUpdatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#addNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23NodeCreated Online Documentation}*/
removeNodeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>NodeUpdated</code> event that occurs when a node has been updated.
This event can be used to further customize the created node.
Nodes are updated in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to add.
@see GraphBuilder#addNodeCreatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#removeNodeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23NodeUpdated Online Documentation}*/
addNodeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeUpdated</code> event that occurs when a node has been updated.
This event can be used to further customize the created node.
Nodes are updated in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to remove.
@see GraphBuilder#addNodeCreatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#addNodeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23NodeUpdated Online Documentation}*/
removeNodeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed from any one of the
{@link GraphBuilder#createNodesSource nodes sources}.
This event can be used to further customize the created node.
Nodes are removed in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to add.
@see GraphBuilder#addNodeCreatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#removeNodeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23NodeRemoved Online Documentation}*/
addNodeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed from any one of the
{@link GraphBuilder#createNodesSource nodes sources}.
This event can be used to further customize the created node.
Nodes are removed in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to remove.
@see GraphBuilder#addNodeCreatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#addNodeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23NodeRemoved Online Documentation}*/
removeNodeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created by any one of the
{@link GraphBuilder#createEdgesSource edges sources}.
This event can be used to further customize the created node.
New edges are created either in response to calling {@link GraphBuilder#buildGraph}, or in response to calling {@link GraphBuilder#updateGraph}
when there are new items in the {@link EdgesSource edges sources}.
@param listener The listener to add.
@see GraphBuilder#addEdgeUpdatedListener
@see GraphBuilder#addEdgeRemovedListener
@see GraphBuilder#removeEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23EdgeCreated Online Documentation}*/
addEdgeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created by any one of the
{@link GraphBuilder#createEdgesSource edges sources}.
This event can be used to further customize the created node.
New edges are created either in response to calling {@link GraphBuilder#buildGraph}, or in response to calling {@link GraphBuilder#updateGraph}
when there are new items in the {@link EdgesSource edges sources}.
@param listener The listener to remove.
@see GraphBuilder#addEdgeUpdatedListener
@see GraphBuilder#addEdgeRemovedListener
@see GraphBuilder#addEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23EdgeCreated Online Documentation}*/
removeEdgeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>EdgeUpdated</code> event that occurs when an edge has been updated.
This event can be used to further customize the created node.
Edges are updated in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to add.
@see GraphBuilder#addEdgeCreatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#removeEdgeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23EdgeUpdated Online Documentation}*/
addEdgeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeUpdated</code> event that occurs when an edge has been updated.
This event can be used to further customize the created node.
Edges are updated in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to remove.
@see GraphBuilder#addEdgeCreatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#addEdgeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23EdgeUpdated Online Documentation}*/
removeEdgeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
This event can be used to further customize the created node.
Edges are removed in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to add.
@see GraphBuilder#addEdgeUpdatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#removeEdgeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23EdgeRemoved Online Documentation}*/
addEdgeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
This event can be used to further customize the created node.
Edges are removed in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to remove.
@see GraphBuilder#addEdgeUpdatedListener
@see GraphBuilder#addNodeRemovedListener
@see GraphBuilder#addEdgeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23EdgeRemoved Online Documentation}*/
removeEdgeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to a node or edge.
This event can be used to further customize the created label.
New labels are added in response to calling {@link GraphBuilder#buildGraph}, or in response to calling
{@link GraphBuilder#updateGraph}.
@param listener The listener to add.
@see GraphBuilder#addLabelUpdatedListener
@see GraphBuilder#addLabelRemovedListener
@see GraphBuilder#removeLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23LabelAdded Online Documentation}*/
addLabelAddedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to a node or edge.
This event can be used to further customize the created label.
New labels are added in response to calling {@link GraphBuilder#buildGraph}, or in response to calling
{@link GraphBuilder#updateGraph}.
@param listener The listener to remove.
@see GraphBuilder#addLabelUpdatedListener
@see GraphBuilder#addLabelRemovedListener
@see GraphBuilder#addLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23LabelAdded Online Documentation}*/
removeLabelAddedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Adds the given listener for the <code>LabelUpdated</code> event that occurs when a node or edge label has been updated.
This event can be used to further customize the created label.
Labels are updated in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to add.
@see GraphBuilder#addLabelAddedListener
@see GraphBuilder#addLabelRemovedListener
@see GraphBuilder#removeLabelUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23LabelUpdated Online Documentation}*/
addLabelUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelUpdated</code> event that occurs when a node or edge label has been updated.
This event can be used to further customize the created label.
Labels are updated in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to remove.
@see GraphBuilder#addLabelAddedListener
@see GraphBuilder#addLabelRemovedListener
@see GraphBuilder#addLabelUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23LabelUpdated Online Documentation}*/
removeLabelUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Adds the given listener for the <code>LabelRemoved</code> event that occurs when a node or edge label has been removed.
This event can be used to further customize the created label.
Labels are removed in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to add.
@see GraphBuilder#addLabelAddedListener
@see GraphBuilder#addLabelUpdatedListener
@see GraphBuilder#removeLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23LabelRemoved Online Documentation}*/
addLabelRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelRemoved</code> event that occurs when a node or edge label has been removed.
This event can be used to further customize the created label.
Labels are removed in response to calling {@link GraphBuilder#updateGraph}.
@param listener The listener to remove.
@see GraphBuilder#addLabelAddedListener
@see GraphBuilder#addLabelUpdatedListener
@see GraphBuilder#addLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilder%23LabelRemoved Online Documentation}*/
removeLabelRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
static $class: Class;
private $_GraphBuilder: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphBuilder;
}
export interface GraphBuilderItemEventArgs<TItem, TDataItem> extends ItemEventArgs<TItem> {
}
/**
Event arguments for item events in {@link GraphBuilder}, {@link AdjacencyGraphBuilder}, and
{@link TreeBuilder}.
Type parameter <code>TItem</code>: The type of the item contained in the argument.
Type parameter <code>TDataItem</code>: The type of object that the item was created from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilderItemEventArgs Online Documentation}*/
export class GraphBuilderItemEventArgs<TItem, TDataItem> {
/**
Creates a new instance of the {@link GraphBuilderItemEventArgs} class with the given graph, item, and source object.
@param graph The graph that can be used to modify <code>item</code>.
@param item The item created from <code>dataItem</code>.
@param dataItem The object <code>item</code> was created from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilderItemEventArgs Online Documentation}*/
constructor(graph: IGraph, item: TItem, dataItem: TDataItem | null);
/**
Gets the graph that can be used to modify the {@link ItemEventArgs#item}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilderItemEventArgs%23graph Online Documentation}*/
graph: IGraph;
/**
Gets the object the {@link ItemEventArgs#item} has been created from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphBuilderItemEventArgs%23dataItem Online Documentation}*/
dataItem: TDataItem;
static $class: Class;
private $_GraphBuilderItemEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TItem, TDataItem>(o: any): o is GraphBuilderItemEventArgs<TItem, TDataItem>;
}
export interface LabelCreator<TDataItem> extends YObject {
}
/**
This class can be used to create and update {@link ILabel}s based on data items.
These data items can be any business or arbitrary data which should be represented by labels.
{@link LabelCreator} allows to conveniently bind properties of the data item to the properties of the created {@link ILabel} to control
aspects like layout and styling.
In the context of the {@link GraphBuilder}, it is responsible to create the labels from the associated {@link NodesSource}s and {@link EdgesSource}s.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator Online Documentation}*/
export class LabelCreator<TDataItem> {
/**
Creates a new creator for labels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator Online Documentation}*/
constructor();
/**
Adds a label on the given {@link ILabelOwner} with the values of the bindings resolved against the <code>dataItem</code>.
This method will use the various helper methods in this class to determine the values for the label's properties.
@param graph The graph.
@param owner The owner to which the label should be added.
@param dataItem The data item for which to create the label.
@returns The newly added label.
@see LabelCreator#getLayoutParameter
@see LabelCreator#getPreferredSize
@see LabelCreator#getText
@see LabelCreator#getStyle
@see LabelCreator#getTag
@see LabelCreator#applyPreferredSizeBindings
@see LabelCreator#applyStyleBindings
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23addLabel(IGraph%2CILabelOwner%2CTDataItem) Online Documentation}*/
addLabel(graph: IGraph, owner: ILabelOwner, dataItem: TDataItem): ILabel;
/**
Called from {@link LabelCreator#addLabel} and performs the actual label creation in the graph.
@param graph The graph to create the node in.
@param owner The owner to which the label should be added.
@param text The label's text.
@param layoutParameter The layout parameter of the label.
@param style The style of the label.
@param preferredSize The preferred size of the label.
@param tag The tag of the node.
@returns The newly added label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23addLabelCore(IGraph%2CILabelOwner%2Cstring%2CILabelModelParameter%2CILabelStyle%2CSize%2CObject) Online Documentation}*/
addLabelCore(graph: IGraph, owner: ILabelOwner, text: string, layoutParameter: ILabelModelParameter, style: ILabelStyle, preferredSize: Size | null, tag: any | null): ILabel;
/**
Resolves the {@link LabelCreator#preferredSizeBindings} against the given <code>dataItem</code> and applies them to the given <code>label</code> preferredSize.
The {@link LabelCreator#preferredSizeProvider} is not evaluated by this method.
@param graph The managed graph.
@param label The label to update.
@param dataItem The data item that is used.
@see LabelCreator#updatePreferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23applyPreferredSizeBindings(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
applyPreferredSizeBindings(graph: IGraph, label: ILabel, dataItem: TDataItem): void;
/**
Resolves the {@link LabelCreator#styleBindings} against the given <code>dataItem</code> and applies them to the given <code>label</code> style.
The {@link LabelCreator#styleProvider} is not evaluated by this method.
@param graph The managed graph.
@param label The label to update.
@param dataItem The data item that is used.
@see LabelCreator#updateStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23applyStyleBindings(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
applyStyleBindings(graph: IGraph, label: ILabel, dataItem: TDataItem): void;
/**
Obtains a {@link ILabel#layoutParameter} instance or <code>null</code> by resolving {@link LabelCreator#layoutParameterProvider} on the data item.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case the defaults should be used.
@see LabelCreator#layoutParameterProvider
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getLayoutParameter(TDataItem) Online Documentation}*/
getLayoutParameter(dataItem: TDataItem): ILabelModelParameter | null;
/**
Obtains a {@link ILabel#preferredSize} or <code>null</code> by resolving the {@link LabelCreator#preferredSizeProvider} on the data item.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case the defaults should be used.
@see LabelCreator#preferredSizeProvider
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getPreferredSize(TDataItem) Online Documentation}*/
getPreferredSize(dataItem: TDataItem): Size | null;
/**
Obtains an {@link ILabelStyle} instance or <code>null</code> by resolving the {@link LabelCreator#styleProvider}.
In case of <code>null</code>, the {@link LabelCreator#defaults} are used.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case the defaults should be used.
@see LabelCreator#styleProvider
@see LabelCreator#styleBindings
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getStyle(TDataItem) Online Documentation}*/
getStyle(dataItem: TDataItem): ILabelStyle | null;
/**
Obtains an <code>object</code> to use as {@link ITagOwner#tag} by resolving the {@link LabelCreator#tagProvider} on the data item.
The tag can be any value and does not need to be related to the original source data item or it's ID.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case the defaults should be used.
@see LabelCreator#tagProvider
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getTag(TDataItem) Online Documentation}*/
getTag(dataItem: TDataItem): any | null;
/**
Obtains a <code>string</code> to use as {@link ILabel#text} or <code>null</code> by resolving the {@link LabelCreator#textProvider} on the data item.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case no text should be added.
@see LabelCreator#textProvider
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getText(TDataItem) Online Documentation}*/
getText(dataItem: TDataItem): string | null;
/**
Resolves the {@link LabelCreator#layoutParameterProvider} on the given data item.
By default, it only delegates to {@link LabelCreator#getLayoutParameter}.
This method is called by {@link LabelCreator#updateLayoutParameter}.
@param graph The managed graph.
@param label The label to update.
@param dataItem The data item that is used.
@returns Returns the new layout parameter.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getUpdatedLayoutParameter(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
getUpdatedLayoutParameter(graph: IGraph, label: ILabel, dataItem: TDataItem): ILabelModelParameter;
/**
Resolves the {@link LabelCreator#preferredSizeProvider} and then {@link LabelCreator#applyPreferredSizeBindings applies the preferred size bindings}.
This method is called by {@link LabelCreator#updatePreferredSize}.
@param graph The managed graph.
@param label The label for which the size should be updated.
@param dataItem The data item that is used.
@returns Returns the label's new preferred size.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getUpdatedPreferredSize(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
getUpdatedPreferredSize(graph: IGraph, label: ILabel, dataItem: TDataItem): Size;
/**
Resolves the {@link LabelCreator#styleProvider} and then {@link LabelCreator#applyStyleBindings applies the style bindings}.
This method is called by {@link LabelCreator#updateStyle}.
@param graph The managed graph.
@param label The label to update.
@param dataItem The data item that is used.
@returns The updated style instance. Depending on {@link LabelCreator#styleProvider} it is either a new instance or the label's style instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getUpdatedStyle(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
getUpdatedStyle(graph: IGraph, label: ILabel, dataItem: TDataItem): ILabelStyle;
/**
Resolves the {@link LabelCreator#tagProvider} on the given data item.
By default, it only delegates to {@link LabelCreator#getTag}.
This method is called by {@link LabelCreator#updateLayoutParameter}.
@param graph The managed graph.
@param label The label for which the new tag should be obtained.
@param dataItem The data item that is used.
@returns Returns the updated tag object.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getUpdatedTag(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
getUpdatedTag(graph: IGraph, label: ILabel, dataItem: TDataItem): any | null;
/**
Resolves the {@link LabelCreator#textProvider} on the given data item.
By default, it only delegates to {@link LabelCreator#getText}.
This method is called by {@link LabelCreator#updateLayoutParameter}.
@param graph The managed graph.
@param label The label for which the new text should be obtained.
@param dataItem The data item that is used.
@returns Returns the new text.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23getUpdatedText(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
getUpdatedText(graph: IGraph, label: ILabel, dataItem: TDataItem): string;
/**
Triggers the {@link LabelCreator#addLabelAddedListener LabelAdded} event.
@param graph The graph in which the label has been added.
@param label The label that has been added.
@param dataItem The data item from which the label has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23onLabelAdded(IGraph%2CTDataItem%2CILabel) Online Documentation}*/
onLabelAdded(graph: IGraph, dataItem: TDataItem, label: ILabel): void;
/**
Triggers the {@link LabelCreator#addLabelUpdatedListener LabelUpdated} event.
@param graph The graph in which the label has been updated.
@param label The label that has been updated.
@param dataItem The data item with which the label has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23onLabelUpdated(IGraph%2CTDataItem%2CILabel) Online Documentation}*/
onLabelUpdated(graph: IGraph, dataItem: TDataItem, label: ILabel): void;
/**
Updates the edge.
By default, this method only fires the {@link LabelCreator#addLabelUpdatedListener LabelUpdated} event.
To update other aspects of the label instance, call the respective update methods when the label is {@link LabelCreator#addLabelUpdatedListener updated} or overwrite
this method.
@param graph The graph that contains the edge.
@param label The label to update.
@param dataItem The data item with which the label should be updated.
@see LabelCreator#updateLayoutParameter
@see LabelCreator#updatePreferredSize
@see LabelCreator#updateStyle
@see LabelCreator#updateText
@see LabelCreator#updateTag
@see LabelCreator#applyStyleBindings
@see LabelCreator#applyPreferredSizeBindings
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23updateLabel(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
updateLabel(graph: IGraph, label: ILabel, dataItem: TDataItem): void;
/**
Updates the {@link ILabel#layoutParameter} of the <code>label</code> with the given <code>dataItem</code> by calling {@link LabelCreator#getUpdatedLayoutParameter} and setting the new parameter on
the label.
@param graph The managed graph.
@param label The label to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23updateLayoutParameter(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
updateLayoutParameter(graph: IGraph, label: ILabel, dataItem: TDataItem): void;
/**
Updates the {@link ILabel#preferredSize} of the <code>label</code> with the given <code>dataItem</code> by calling {@link LabelCreator#getUpdatedPreferredSize} and setting the new size on the
label.
@param graph The managed graph.
@param label The label to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23updatePreferredSize(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
updatePreferredSize(graph: IGraph, label: ILabel, dataItem: TDataItem): void;
/**
Updates the style of the <code>label</code> with the given <code>dataItem</code> by calling {@link LabelCreator#getUpdatedStyle} and applying the style to the node in
the graph.
@param graph The managed graph.
@param label The label to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23updateStyle(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
updateStyle(graph: IGraph, label: ILabel, dataItem: TDataItem): void;
/**
Updates the {@link ITagOwner#tag} of the <code>label</code> with the given <code>dataItem</code> by calling {@link LabelCreator#getUpdatedTag} and setting the new <code>tag</code> on the label.
@param graph The managed graph.
@param label The label to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23updateTag(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
updateTag(graph: IGraph, label: ILabel, dataItem: TDataItem): void;
/**
Updates the {@link ILabel#text} of the <code>label</code> with the given <code>dataItem</code> by calling {@link LabelCreator#getUpdatedText} and setting the new text on the label.
@param graph The managed graph.
@param label The label to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23updateText(IGraph%2CILabel%2CTDataItem) Online Documentation}*/
updateText(graph: IGraph, label: ILabel, dataItem: TDataItem): void;
/**
Gets or sets an optional provider of the text that will be used as the label's {@link ILabel#text} for the source data item.
@see LabelCreator#getText
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23textProvider Online Documentation}*/
textProvider: ((dataItem: TDataItem) => string | null) | null;
/**
Gets or sets an optional provider of the size that will be used as the label's {@link ILabel#preferredSize} for the source data item.
@see LabelCreator#getPreferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23preferredSizeProvider Online Documentation}*/
preferredSizeProvider: ((dataItem: TDataItem) => Size | null) | null;
/**
Gets or sets an optional provider of an object that will be used as the label's {@link ITagOwner#tag} for the source data item.
By default the source data item itself will be used as the tag.
@see LabelCreator#getTag
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23tagProvider Online Documentation}*/
tagProvider: ((dataItem: TDataItem) => any | null) | null;
/**
Gets or sets an optional provider of an {@link ILabelStyle} instance that will be used as the label's {@link ILabel#style} for the
source data item.
By default the {@link LabelCreator#defaults}' {@link ILabelDefaults#style} will be used. Note that {@link ILabelDefaults#shareStyleInstance} will be ignored if this provider returns a value. Use the {@link LabelCreator#styleBindings} to reconfigure
properties of the style instances.
@see LabelCreator#getStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23styleProvider Online Documentation}*/
styleProvider: ((dataItem: TDataItem) => ILabelStyle | null) | null;
/**
Gets or sets an optional provider of a label model parameter that will be used as the label's {@link ILabel#layoutParameter} for the source data
item.
@see LabelCreator#getLayoutParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23layoutParameterProvider Online Documentation}*/
layoutParameterProvider: ((dataItem: TDataItem) => ILabelModelParameter | null) | null;
/**
Gets or sets an optional set of bindings that will be {@link LabelCreator#applyStyleBindings applied} to the label's {@link ILabel#style}.
Note that bindings will affect the properties of the instance created by {@link LabelCreator#getStyle}. If the style is obtained from the {@link LabelCreator#defaults}
with {@link ILabelDefaults#shareStyleInstance} set to <code>true</code>, the properties of the same instance are overwritten each time. To get fresh copies, set {@link ILabelDefaults#shareStyleInstance} to <code>false</code>.
@see LabelCreator#applyStyleBindings
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23styleBindings Online Documentation}*/
styleBindings: ObjectBindings<TDataItem>;
/**
Gets or sets an optional binding that will be {@link LabelCreator#applyPreferredSizeBindings applied} to the label.
@see LabelCreator#applyPreferredSizeBindings
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23preferredSizeBindings Online Documentation}*/
preferredSizeBindings: ObjectBindings<TDataItem>;
/**
Gets or sets a set of {@link ILabelDefaults} that will be used for the labels.
The defaults of instances of this class created by e.g. {@link NodeCreator#createLabelBinding}, cascade with the defaults of the {@link NodeCreator}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23defaults Online Documentation}*/
defaults: ILabelDefaults;
/**
Adds the given listener for the <code>LabelAdded</code> event that occurs when a label has been added.
@param listener The listener to add.
@see LabelCreator#addLabelUpdatedListener
@see LabelCreator#removeLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23LabelAdded Online Documentation}*/
addLabelAddedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, TDataItem>) => void): void;
/**
Removes the given listener for the <code>LabelAdded</code> event that occurs when a label has been added.
@param listener The listener to remove.
@see LabelCreator#addLabelUpdatedListener
@see LabelCreator#addLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23LabelAdded Online Documentation}*/
removeLabelAddedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, TDataItem>) => void): void;
/**
Adds the given listener for the <code>LabelUpdated</code> event that occurs when a label has been updated.
@param listener The listener to add.
@see LabelCreator#addLabelAddedListener
@see LabelCreator#removeLabelUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23LabelUpdated Online Documentation}*/
addLabelUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, TDataItem>) => void): void;
/**
Removes the given listener for the <code>LabelUpdated</code> event that occurs when a label has been updated.
@param listener The listener to remove.
@see LabelCreator#addLabelAddedListener
@see LabelCreator#addLabelUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCreator%23LabelUpdated Online Documentation}*/
removeLabelUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, TDataItem>) => void): void;
static $class: Class;
private $_LabelCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDataItem>(o: any): o is LabelCreator<TDataItem>;
}
export interface LabelsSource<TDataItem> extends YObject {
}
/**
A class that accepts a collection of objects from the {@link NodeCreator} or {@link EdgeCreator} to build {@link ILabel}s from.
Allows to further customize the creation behavior through the provided {@link LabelsSource#labelCreator}.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelsSource Online Documentation}*/
export class LabelsSource<TDataItem> {
/**
Creates a new source with the given <code>labelCreator</code>.
@param labelCreator The creator that is used to create the edges of this source.
@param [idProvider=null] An optional function that yields an id for each element of the associated data collection in the
{@link GraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelsSource Online Documentation}*/
constructor(labelCreator: LabelCreator<TDataItem>, idProvider?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>);
/**
Gets or sets the {@link LabelCreator} for this source which can be used to adjust the creation behavior.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelsSource%23labelCreator Online Documentation}*/
labelCreator: LabelCreator<TDataItem>;
/**
Gets or sets a provider that yields an id for each element of the associated data collection in the
{@link GraphBuilder}.
The id is used to identify labels when updating the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelsSource%23idProvider Online Documentation}*/
idProvider: ((dataItem: TDataItem, canonicalId: any | null) => any) | null;
static $class: Class;
private $_LabelsSource: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDataItem>(o: any): o is LabelsSource<TDataItem>;
}
export interface NodeCreator<TDataItem> extends YObject {
}
/**
This class can be used to create and update {@link INode}s based on data items.
These data items can be any business or arbitrary data which should be represented by nodes.
{@link NodeCreator} allows for conveniently binding properties of the data item to the properties of the created {@link INode} to
control aspects like layout and styling.
This class can be used both standalone as well as in conjunction with any of the <code>GraphBuilders</code>, e.g.
{@link GraphBuilder}.
In the context of the {@link GraphBuilder}, it is responsible to create the items from the associated {@link NodesSource}s.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator Online Documentation}*/
export class NodeCreator<TDataItem> {
/**
Creates a new creator for nodes or {@link IGraph#isGroupNode group nodes}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator Online Documentation}*/
constructor();
/**
Adds the given <code>labelsSource</code> to this instance.
Type parameter <code>TLabelDataItem</code>: The type of the data items in the source.
@param dataProvider A function that provides the label data items from the data item.
@param labelsSource The source to which the data is added.
@see NodeCreator#createLabelsSource
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23addLabelsSource(function(TDataItem)%3AObject%2CLabelsSource%3CTLabelDataItem%3E) Online Documentation}*/
addLabelsSource<TLabelDataItem>(dataProvider: ((data: TDataItem) => TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
}>, labelsSource: LabelsSource<TLabelDataItem>): void;
/**
Resolves the {@link NodeCreator#layoutBindings} against the given <code>dataItem</code> and applies them on the given <code>node</code> layout.
The {@link NodeCreator#layoutProvider} is not evaluated by this method.
@param graph The managed graph.
@param node The node to update.
@param dataItem The data item that is used.
@see NodeCreator#updateLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23applyLayoutBindings(IGraph%2CINode%2CTDataItem) Online Documentation}*/
applyLayoutBindings(graph: IGraph, node: INode, dataItem: TDataItem): void;
/**
Resolves the {@link NodeCreator#styleBindings} against the given <code>dataItem</code> and applies them to the <code>node</code> style.
The {@link NodeCreator#styleProvider} is not evaluated by this method.
@param graph The managed graph.
@param node The node to update.
@param dataItem The data item that is used.
@see NodeCreator#updateStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23applyStyleBindings(IGraph%2CINode%2CTDataItem) Online Documentation}*/
applyStyleBindings(graph: IGraph, node: INode, dataItem: TDataItem): void;
/**
Creates a binding for labels that will be added to nodes created by this instance.
Use this method over {@link NodeCreator#createLabelsSource} if the created nodes have a predetermined amount of labels. Note that the item
type of the returned {@link LabelCreator} is the same as for this {@link NodeCreator}.
@param [text=null] An optional provider for the {@link ILabel#text} property.
@returns A new {@link LabelCreator} instance that can be further configured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23createLabelBinding(function(TDataItem)%3Astring) Online Documentation}*/
createLabelBinding(text?: ((dataItem: TDataItem) => string | null) | null | ProviderConvertible<TDataItem, string>): LabelCreator<TDataItem>;
/**
Creates a binding for labels that will be added to nodes created by this instance.
Use this method over {@link NodeCreator#createLabelsSource} if the created nodes have a predetermined amount of labels. Note that the item
type of the returned {@link LabelCreator} is the same as for this {@link NodeCreator}.
@param options The parameters to pass.
@param [options.text=null] An optional provider for the {@link ILabel#text} property.
@param [options.defaults] A set of {@link ILabelDefaults} that will be used for the labels. This option sets the {@link LabelCreator#defaults} property on the created object.
@param [options.layoutParameter] An optional provider of a label model parameter that will be used as the label's {@link ILabel#layoutParameter} for the source data item. This option sets the {@link LabelCreator#layoutParameterProvider} property on the created object.
@param [options.style] An optional provider of an {@link ILabelStyle} instance that will be used as the label's {@link ILabel#style} for the source data item. This option sets the {@link LabelCreator#styleProvider} property on the created object.
@param [options.preferredSize] An optional provider of the size that will be used as the label's {@link ILabel#preferredSize} for the source data item. This option sets the {@link LabelCreator#preferredSizeProvider} property on the created object.
@param [options.tag] An optional provider of an object that will be used as the label's {@link ITagOwner#tag} for the source data item. This option sets the {@link LabelCreator#tagProvider} property on the created object.
@param [options.styleBindings] An optional set of bindings that will be {@link LabelCreator#applyStyleBindings applied} to the label's {@link ILabel#style}. This option sets the {@link LabelCreator#styleBindings} property on the created object.
@param [options.preferredSizeBindings] An optional binding that will be {@link LabelCreator#applyPreferredSizeBindings applied} to the label. This option sets the {@link LabelCreator#preferredSizeBindings} property on the created object.
@returns A new {@link LabelCreator} instance that can be further configured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23createLabelBinding(function(TDataItem)%3Astring) Online Documentation}*/
createLabelBinding(options: {
text?: ((dataItem: TDataItem) => string | null) | null | ProviderConvertible<TDataItem, string>;
defaults?: ILabelDefaults;
layoutParameter?: ((dataItem: TDataItem) => ILabelModelParameter | null) | null | ProviderConvertible<TDataItem, ILabelModelParameter>;
style?: ((dataItem: TDataItem) => ILabelStyle | null) | null | ProviderConvertible<TDataItem, ILabelStyle>;
preferredSize?: ((dataItem: TDataItem) => Size | null) | null | ProviderConvertible<TDataItem, Size>;
tag?: ((dataItem: TDataItem) => any | null) | null | ProviderConvertible<TDataItem, any>;
styleBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
preferredSizeBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
}): LabelCreator<TDataItem>;
/**
Creates a new source of label data items that will be added to nodes created by this instance.
Use this method over {@link NodeCreator#createLabelBinding} if the created nodes have a varying number of labels. Note that the <code>data</code>
introduces a new item type for the returned {@link LabelsSource}.
Type parameter <code>TLabelDataItem</code>: The type of the data items in the source.
@param data A function that provides a collection of label data items from the node data item.
@param [id=null] An optional function that yields an id for each label data item that is provided from the label data provider. This id
is used to identify the labels during {@link NodeCreator#updateLabels}.
@returns A new {@link LabelsSource} instance whose {@link LabelCreator} can be configured further.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23createLabelsSource(function(TDataItem)%3AObject%2Cfunction(TLabelDataItem%2CObject)%3AObject) Online Documentation}*/
createLabelsSource<TLabelDataItem>(data: ((data: TDataItem) => TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
}>, id?: ((dataItem: TLabelDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TLabelDataItem>): LabelsSource<TLabelDataItem>;
/**
Creates a new source of label data items that will be added to nodes created by this instance.
Use this method over {@link NodeCreator#createLabelBinding} if the created nodes have a varying number of labels. Note that the <code>data</code>
introduces a new item type for the returned {@link LabelsSource}.
Type parameter <code>TLabelDataItem</code>: The type of the data items in the source.
@param options The parameters to pass.
@param options.data A function that provides a collection of label data items from the node data item.
@param [options.id=null] An optional function that yields an id for each label data item that is provided from the label data provider. This id
is used to identify the labels during {@link NodeCreator#updateLabels}.
@param [options.defaults] A set of {@link ILabelDefaults} that will be used for the labels. This option sets the {@link LabelCreator#defaults} property on the created object.
@param [options.text] An optional provider of the text that will be used as the label's {@link ILabel#text} for the source data item. This option sets the {@link LabelCreator#textProvider} property on the created object.
@param [options.layoutParameter] An optional provider of a label model parameter that will be used as the label's {@link ILabel#layoutParameter} for the source data item. This option sets the {@link LabelCreator#layoutParameterProvider} property on the created object.
@param [options.preferredSize] An optional provider of the size that will be used as the label's {@link ILabel#preferredSize} for the source data item. This option sets the {@link LabelCreator#preferredSizeProvider} property on the created object.
@param [options.style] An optional provider of an {@link ILabelStyle} instance that will be used as the label's {@link ILabel#style} for the source data item. This option sets the {@link LabelCreator#styleProvider} property on the created object.
@param [options.tag] An optional provider of an object that will be used as the label's {@link ITagOwner#tag} for the source data item. This option sets the {@link LabelCreator#tagProvider} property on the created object.
@param [options.styleBindings] An optional set of bindings that will be {@link LabelCreator#applyStyleBindings applied} to the label's {@link ILabel#style}. This option sets the {@link LabelCreator#styleBindings} property on the created object.
@param [options.preferredSizeBindings] An optional binding that will be {@link LabelCreator#applyPreferredSizeBindings applied} to the label. This option sets the {@link LabelCreator#preferredSizeBindings} property on the created object.
@returns A new {@link LabelsSource} instance whose {@link LabelCreator} can be configured further.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23createLabelsSource(function(TDataItem)%3AObject%2Cfunction(TLabelDataItem%2CObject)%3AObject) Online Documentation}*/
createLabelsSource<TLabelDataItem>(options: {
data: ((data: TDataItem) => TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TLabelDataItem[] | Iterable<TLabelDataItem> | Map<any, TLabelDataItem> | {
[id: string]: TLabelDataItem;
}>;
id?: ((dataItem: TLabelDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TLabelDataItem>;
defaults?: ILabelDefaults;
text?: ((dataItem: any) => string | null) | null | ProviderConvertible<any, string>;
layoutParameter?: ((dataItem: any) => ILabelModelParameter | null) | null | ProviderConvertible<any, ILabelModelParameter>;
preferredSize?: ((dataItem: any) => Size | null) | null | ProviderConvertible<any, Size>;
style?: ((dataItem: any) => ILabelStyle | null) | null | ProviderConvertible<any, ILabelStyle>;
tag?: ((dataItem: any) => any | null) | null | ProviderConvertible<any, any>;
styleBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
preferredSizeBindings?: ObjectBindings<any> | ObjectBindingsConvertible<any>;
}): LabelsSource<TLabelDataItem>;
/**
Creates a node in the graph with the values of the bindings resolved against the <code>dataItem</code>.
This method will use the various helper methods in this class to determine the values for the node's {@link INode#layout}, {@link INode#style}, and
{@link ITagOwner#tag}. It will also add the labels to the node.
@param graph The graph to create the node in.
@param parent The parent node for the newly created node.
@param dataItem The data item for which to create the node.
@returns The newly created node.
@see NodeCreator#isGroupPredicate
@see NodeCreator#getLayout
@see NodeCreator#getStyle
@see NodeCreator#getTag
@see NodeCreator#applyLayoutBindings
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23createNode(IGraph%2CINode%2CTDataItem) Online Documentation}*/
createNode(graph: IGraph, parent: INode | null, dataItem: TDataItem): INode;
/**
Called from {@link NodeCreator#createNode} and performs the actual node creation in the graph.
@param graph The graph to create the node in.
@param groupNode Whether to create a group node.
@param parent Optional parent of the node.
@param layout The layout of the node.
@param style The style of the node.
@param tag The tag of the node.
@returns The newly created node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23createNodeCore(IGraph%2Cboolean%2CINode%2CRect%2CINodeStyle%2CObject) Online Documentation}*/
createNodeCore(graph: IGraph, groupNode: boolean, parent: INode | null, layout: Rect, style: INodeStyle, tag: any | null): INode;
/**
Obtains a {@link Rect} to use as {@link INode#layout} or <code>null</code> by resolving the {@link NodeCreator#layoutProvider} and afterwards applying the {@link NodeCreator#layoutBindings}.
In case of <code>null</code>, the {@link NodeCreator#defaults} are used.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case the defaults should be used.
@see NodeCreator#layoutProvider
@see NodeCreator#layoutBindings
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23getLayout(TDataItem) Online Documentation}*/
getLayout(dataItem: TDataItem): Rect | null;
/**
Obtains an {@link INodeStyle} instance or <code>null</code> by resolving the {@link NodeCreator#styleProvider}.
In case of <code>null</code>, the {@link NodeCreator#defaults} are used.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case the defaults should be used.
@see NodeCreator#styleProvider
@see NodeCreator#styleBindings
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23getStyle(TDataItem) Online Documentation}*/
getStyle(dataItem: TDataItem): INodeStyle | null;
/**
Obtains an <code>object</code> to use as {@link ITagOwner#tag} by resolving the {@link NodeCreator#tagProvider} on the data item.
The tag can be any value and does not need to be related to the original source data item or it's ID.
@param dataItem The data item on which the provider is resolved.
@returns The value to use or <code>null</code> in case the defaults should be used.
@see NodeCreator#tagProvider
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23getTag(TDataItem) Online Documentation}*/
getTag(dataItem: TDataItem): any | null;
/**
Resolves the {@link NodeCreator#layoutProvider} and then {@link NodeCreator#applyLayoutBindings applies the layout bindings}.
This method is called by {@link NodeCreator#updateLayout}.
@param graph The managed graph.
@param node The node to update.
@param dataItem The data item that is used.
@returns An updated {@link Rect} based on the given data item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23getUpdatedLayout(IGraph%2CINode%2CTDataItem) Online Documentation}*/
getUpdatedLayout(graph: IGraph, node: INode, dataItem: TDataItem): Rect;
/**
Resolves the {@link NodeCreator#styleProvider} and then {@link NodeCreator#applyStyleBindings applies the style bindings}.
This method is called by {@link NodeCreator#updateStyle}.
@param graph The managed graph.
@param node The node to update.
@param dataItem The data item that is used.
@returns The updated style instance. Depending on {@link NodeCreator#getStyle} it is either a new instance or the node's style instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23getUpdatedStyle(IGraph%2CINode%2CTDataItem) Online Documentation}*/
getUpdatedStyle(graph: IGraph, node: INode, dataItem: TDataItem): INodeStyle;
/**
Resolves the {@link NodeCreator#tagProvider} on the given data item.
By default, it only delegates to {@link NodeCreator#getTag}.
This method is called by {@link NodeCreator#updateTag}.
@param graph The managed graph.
@param node The node for which the new tag should be obtained.
@param dataItem The data item that is used.
@returns Returns the updated tag object.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23getUpdatedTag(IGraph%2CINode%2CTDataItem) Online Documentation}*/
getUpdatedTag(graph: IGraph, node: INode, dataItem: TDataItem): any | null;
/**
Triggers the {@link NodeCreator#addNodeCreatedListener NodeCreated} event.
@param graph The graph in which the node has been created.
@param node The node that has been created.
@param dataItem The data item from which the node has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23onNodeCreated(IGraph%2CINode%2CTDataItem) Online Documentation}*/
onNodeCreated(graph: IGraph, node: INode, dataItem: TDataItem): void;
/**
Triggers the {@link NodeCreator#addNodeUpdatedListener NodeUpdated} event.
@param graph The graph in which the node has been updated.
@param node The node that has been updated.
@param dataItem The data item with which the node has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23onNodeUpdated(IGraph%2CINode%2CTDataItem) Online Documentation}*/
onNodeUpdated(graph: IGraph, node: INode, dataItem: TDataItem): void;
/**
Can be used to update the labels of the node that have been added with this {@link NodeCreator}.
This only adds / removes labels based on the bound data. To update bound properties for the labels, call the respective
update methods on the {@link LabelCreator}.
@param graph The managed graph.
@param node The node whose labels to update.
@param dataItem The node data item with which the node's labels should be updated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23updateLabels(IGraph%2CINode%2CTDataItem) Online Documentation}*/
updateLabels(graph: IGraph, node: INode, dataItem: TDataItem): void;
/**
Updates the layout of the <code>node</code> in the graph with the given layout information of the given <code>dataItem</code> by calling {@link NodeCreator#getUpdatedLayout}
and applying it to the node.
@param graph The managed graph.
@param node The node to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23updateLayout(IGraph%2CINode%2CTDataItem) Online Documentation}*/
updateLayout(graph: IGraph, node: INode, dataItem: TDataItem): void;
/**
Updates the node.
By default, this method only performs a structural update of the node, i.e. it {@link DefaultGraph#setParent sets a new parent} and updates its
{@link IGraph#isGroupNode group node state}.
To update other aspects of the node instance, call the respective update methods when the node is {@link NodeCreator#addNodeUpdatedListener updated} or overwrite
this method.
@param graph The graph that contains the node.
@param node The node to update.
@param parent The new parent of the node.
@param dataItem The data item with which the node should be updated.
@see NodeCreator#updateLayout
@see NodeCreator#updateStyle
@see NodeCreator#updateTag
@see NodeCreator#updateLabels
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23updateNode(IGraph%2CINode%2CINode%2CTDataItem) Online Documentation}*/
updateNode(graph: IGraph, node: INode, parent: INode | null, dataItem: TDataItem): void;
/**
Updates the style of the <code>node</code> with the given <code>dataItem</code> by calling {@link NodeCreator#getUpdatedStyle} and applying the style to the node in
the graph.
@param graph The managed graph.
@param node The node to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23updateStyle(IGraph%2CINode%2CTDataItem) Online Documentation}*/
updateStyle(graph: IGraph, node: INode, dataItem: TDataItem): void;
/**
Updates the {@link ITagOwner#tag} of the <code>node</code> with the given <code>dataItem</code> by calling {@link NodeCreator#getUpdatedTag} and setting the new <code>tag</code> on the node.
@param graph The managed graph.
@param node The node to update.
@param dataItem The data item that is used for the update.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23updateTag(IGraph%2CINode%2CTDataItem) Online Documentation}*/
updateTag(graph: IGraph, node: INode, dataItem: TDataItem): void;
/**
A predicate that determines whether the node created for a data item shall be a {@link IGraph#isGroupNode group node} or not.
When this predicate returns <code>true</code> but the node will not have any children associated, it will still be created as
{@link IGraph#isGroupNode group node}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23isGroupPredicate Online Documentation}*/
isGroupPredicate: (obj: TDataItem) => boolean;
/**
Gets or sets an optional provider of a {@link Rect} that will be used as the node's {@link INode#layout} for the source data item.
The {@link NodeCreator#layoutBindings} may be used to bind each of the X, Y, {@link ISize#width} and {@link ISize#height} values, separately. If no layout is provided, the {@link NodeCreator#defaults}' {@link INodeDefaults#size} will be used and
the node will be located at the origin.
@see NodeCreator#getLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23layoutProvider Online Documentation}*/
layoutProvider: ((dataItem: TDataItem) => Rect) | null;
/**
Gets or sets an optional provider of an object that will be used as the node's {@link ITagOwner#tag} for the source data item.
By default the source data item itself will be used as the tag.
@see NodeCreator#getTag
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23tagProvider Online Documentation}*/
tagProvider: ((dataItem: TDataItem) => any | null) | null;
/**
Gets or sets an optional provider of an {@link INodeStyle} instance that will be used as the node's {@link INode#style} for the
source data item.
By default the {@link NodeCreator#defaults}' {@link INodeDefaults#style} will be used. Note that {@link INodeDefaults#shareStyleInstance} will be ignored if this provider returns a value. Use the {@link NodeCreator#styleBindings} to reconfigure
properties of the style instances.
@see NodeCreator#getStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23styleProvider Online Documentation}*/
styleProvider: ((dataItem: TDataItem) => INodeStyle | null) | null;
/**
Gets or sets an optional set of bindings that will be {@link NodeCreator#applyStyleBindings applied} to the node's {@link INode#style}.
Note that bindings will affect the properties of the instance created by {@link NodeCreator#getStyle}. If the style is obtained from the {@link NodeCreator#defaults}
with {@link INodeDefaults#shareStyleInstance} set to <code>true</code>, the properties of the same instance are overwritten each time. To get fresh copies, set {@link INodeDefaults#shareStyleInstance} to <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23styleBindings Online Documentation}*/
styleBindings: ObjectBindings<TDataItem>;
/**
Gets or sets an optional set of bindings that will be {@link NodeCreator#applyStyleBindings applied} to the node's {@link INode#style}.
@see NodeCreator#applyStyleBindings
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23layoutBindings Online Documentation}*/
layoutBindings: ObjectBindings<TDataItem>;
/**
Gets or sets a set of {@link INodeDefaults} that will be used for the nodes and node labels.
The defaults of instances of this class created by one of the <code>GraphBuilders</code>, e.g. {@link GraphBuilder}, cascade
with the defaults of the GraphBuilder's {@link GraphBuilder#graph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23defaults Online Documentation}*/
defaults: INodeDefaults;
/**
Adds the given listener for the <code>NodeCreated</code> event that occurs when a node has been created.
@param listener The listener to add.
@see NodeCreator#addNodeUpdatedListener
@see NodeCreator#removeNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23NodeCreated Online Documentation}*/
addNodeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, TDataItem>) => void): void;
/**
Removes the given listener for the <code>NodeCreated</code> event that occurs when a node has been created.
@param listener The listener to remove.
@see NodeCreator#addNodeUpdatedListener
@see NodeCreator#addNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23NodeCreated Online Documentation}*/
removeNodeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, TDataItem>) => void): void;
/**
Adds the given listener for the <code>NodeUpdated</code> event that occurs when a node has been updated.
@param listener The listener to add.
@see NodeCreator#addNodeCreatedListener
@see NodeCreator#removeNodeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23NodeUpdated Online Documentation}*/
addNodeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, TDataItem>) => void): void;
/**
Removes the given listener for the <code>NodeUpdated</code> event that occurs when a node has been updated.
@param listener The listener to remove.
@see NodeCreator#addNodeCreatedListener
@see NodeCreator#addNodeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeCreator%23NodeUpdated Online Documentation}*/
removeNodeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, TDataItem>) => void): void;
static $class: Class;
private $_NodeCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDataItem>(o: any): o is NodeCreator<TDataItem>;
}
export interface NodesSource<TDataItem> extends YObject {
}
/**
A class that accepts a collection of objects from the {@link GraphBuilder} to build {@link INode}s from.
It manages the structural aspects of the graph creation, while the associated {@link NodesSource#nodeCreator} is responsible for the properties of the
created items.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodesSource Online Documentation}*/
export class NodesSource<TDataItem> {
/**
Creates a new source with the given <code>nodeCreator</code>.
@param nodeCreator The creator that is used to create the nodes of this source.
@param [idProvider=null] An optional function that yields an id for each element of the associated data collection in the
{@link GraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodesSource Online Documentation}*/
constructor(nodeCreator: NodeCreator<TDataItem>, idProvider?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>);
/**
Optionally obtains the id for the {@link IGraph#getParent parent} or <code>null</code> given each data item for which a node is created.
This implementation uses the {@link NodesSource#parentIdProvider} to obtain the value from the data item.
@param dataItem A single data item from the the associated data collection in the {@link GraphBuilder}.
@returns The id of another node or <code>null</code> in case the node should be put in the root of the hierarchy.
@see NodesSource#parentIdProvider
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodesSource%23getParentId(TDataItem) Online Documentation}*/
getParentId(dataItem: TDataItem): any | null;
/**
Gets or sets the {@link NodeCreator} for this source which can be used to adjust the creation behavior.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodesSource%23nodeCreator Online Documentation}*/
nodeCreator: NodeCreator<TDataItem>;
/**
Gets or sets a provider that yields an id for each element of the associated data collection in the
{@link GraphBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodesSource%23idProvider Online Documentation}*/
idProvider: ((dataItem: TDataItem, canonicalId: any | null) => any) | null;
/**
Gets or sets a provider that yields a parent id for each element of the associated data collection in the
{@link GraphBuilder}.
While this is optional, it is required to specify a parent/child node grouping relationship.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodesSource%23parentIdProvider Online Documentation}*/
parentIdProvider: ((dataItem: TDataItem) => any | null) | null;
static $class: Class;
private $_NodesSource: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDataItem>(o: any): o is NodesSource<TDataItem>;
}
export interface TreeBuilder extends YObject {
}
/**
* Populates a graph from custom data where objects corresponding to nodes have a parent-child relationship.
* This class can be used when the data consists of one or more collections of nodes, each of which knows its child nodes,
* and optionally, groups. The methods {@link TreeBuilder#createRootNodesSource},
* {@link TreeBuilder#createRootGroupNodesSource}, and {@link TreeNodesSource#createChildNodesSource} create new source collections from
* which nodes and group nodes will be created.
*
* Generally, using the {@link TreeBuilder} class consists of a few basic steps:
*
* <ol>
* <li>Create a {@link TreeBuilder} and optionally, provide an {@link IGraph} on which it should operate.
*
* <pre>
* const treeBuilder = new TreeBuilder(graph)
* </pre>
* </li>
* <li>Create one or more {@link TreeBuilder#createRootNodesSource root node sources} from which the {@link TreeBuilder} creates the root nodes in the tree. Group
* nodes can be created by adding {@link TreeBuilder#createRootGroupNodesSource group node sources}.
*
* <pre>
* const redData = getRootNodesData()
* // create source for root nodes (red nodes)
* const redNodesSource = treeBuilder.createRootNodesSource(redData)
* </pre>
* </li>
* <li>To create child nodes, {@link TreeNodesSource#createChildNodesSource create new sources} as child sources of the root sources. By {@link TreeNodesSource#addChildNodesSource adding existing sources} as child
* sources, the recursive structure of the tree can be defined:
*
* <pre>
* // create source for children of red nodes (blue nodes) as child source of redNodeSource
* const blueNodesSource = redNodesSource.createChildNodesSource(n => n.childrenBlue)
* // tie the knot
* blueNodesSource.addChildNodesSource(n => n.childrenRed, redNodesSource)
* </pre>
*
* Edges are created by the implicit parent-child relationship between the sources. For each child node, the {@link TreeBuilder}
* creates an edge between the child node and its (tree) parent providing the child node data item to the {@link TreeNodesSource#parentEdgeCreator}.</li>
* <li>Each {@link TreeNodesSource} provides a {@link NodeCreator} that allows for further specification of how the items from this source should be created. You can
* provide {@link NodeCreator#defaults} for the default styling but also bind more specific styling based on the actual data item with the {@link NodeCreator#styleProvider} and {@link NodeCreator#styleBindings}.
*
* <pre>
* redNodesSource.nodeCreator.defaults.shareStyleInstance = false
* redNodesSource.nodeCreator.defaults.style = new ShapeNodeStyle({
*   stroke: 'darkred',
*   fill: 'orange'
* })
* // leaf nodes should have a diamond shape
* redNodesSource.nodeCreator.styleBindings.addBinding('shape', redDataItem =>
*   redDataItem.childrenBlue.any() ? 'roundRectangle' : 'diamond'
* )
* </pre>
*
* It also allows for obtaining layout information from the data item ({@link NodeCreator#layoutProvider}), as well as further specification of the created
* node's {@link ITagOwner#tag} ({@link NodeCreator#tagProvider}).</li>
* <li>The edge creation can be adjusted by accessing the {@link TreeNodesSource#parentEdgeCreator} that allows for specifying the styling of the created edges ({@link EdgeCreator#defaults}, {@link EdgeCreator#styleProvider}, {@link EdgeCreator#styleBindings})
* and the edges' {@link ITagOwner#tag}s ({@link EdgeCreator#tagProvider}).
*
* <pre>
* blueNodesSource.parentEdgeCreator.defaults.style = new PolylineEdgeStyle({
*   stroke: 'cornflowerBlue'
* })
* </pre>
* </li>
* <li>Optionally, labels can be added by providing one or multiple {@link NodeCreator#createLabelBinding label bindings}. If there is a varying number of labels per
* node, {@link NodeCreator#createLabelsSource} can be used, instead. Similar methods are also available on the {@link EdgeCreator}.
*
* The {@link LabelsSource} provides a {@link LabelCreator} that allows for adjusting all aspects of the label creation, like style, text, size, and tag.
*
* <pre>
* const edgeLabelCreator = blueNodesSource.parentEdgeCreator.createLabelBinding(
*   blueDataItem => blueDataItem.name
* )
* edgeLabelCreator.defaults.style = new DefaultLabelStyle({
*   backgroundFill: 'rgb(225,242,253)',
*   backgroundStroke: 'lightSkyBlue',
*   textSize: 8
* })
* </pre>
* </li>
* <li>Call {@link TreeBuilder#buildGraph} to populate the graph. You can apply a layout algorithm afterward to make the
* graph look nice.
*
* <pre>
* // Build a graph initially
* treeBuilder.buildGraph()
* // Apply a layout in a subsequent step
* graph.applyLayout(new TreeLayout())
* </pre>
* </li>
* <li>If your items or collections change later, call {@link TreeBuilder#updateGraph} to make the changes visible in
* the graph. If the data item instances have changed, you can call {@link TreeBuilder#setData}.
*
* <pre>
* // Set the new data collections:
* treeBuilder.setData(redNodesSource, getRootNodesData())
* // Update a graph after the business data has changed
* treeBuilder.updateGraph()
* // Apply a layout in a subsequent step
* graph.applyLayout(new TreeLayout())
* </pre>
* </li>
* </ol>
* You can further customize how nodes, groups, and edges are created by adding event handlers to the various events and
* modifying the items there. This can be used to modify items in ways that are not directly supported by the available
* bindings or defaults. There are creation and update events for all items, which allow for separate customization when
* nodes, groups, and edges are created or updated. For completely static graphs, where {@link TreeBuilder#updateGraph}
* is not needed, the update events can be safely ignored.
*
* <h2>Note</h2>
*
* This class serves as a convenient way to create trees. Some aspects to look out for:
*
* <ul>
* <li>{@link TreeBuilder#buildGraph} does not clear the graph. If needed, call {@link IGraph#clear} to empty the
* graph before building it anew.</li>
* <li>The {@link TreeBuilder} ignores and preserves items manually created on the graph in between calls to
* {@link TreeBuilder#updateGraph}.</li>
* </ul>
* If you need to extensively modify your data in order to fit the requirements of the {@link TreeBuilder}, it's
* often better to write the code interfacing with the graph by hand instead of relying on {@link TreeBuilder}.
*
* <h2>Note</h2>
*
* {@link TreeBuilder#updateGraph} merely updates the graph structure. Any other bound data must be updated
* explicitly if necessary. Calling the respective <code>update</code> methods on the <code>creators</code> (e.g. {@link NodeCreator#updateStyle}), resolves the <code>provider</code>
* (e.g. {@link NodeCreator#styleProvider}) and applies the <code>bindings</code> (e.g.
* {@link NodeCreator#applyStyleBindings}). The latter can also be applied solely.
* <br />
* For example, to update any aspect of node creation:
*
* <pre>
* // evaluate the providers and apply respective bindings
* nodesSource.nodeCreator.addNodeUpdatedListener((sender, evt) => {
*   nodesSource.nodeCreator.updateLayout(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateStyle(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateTag(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateLabels(evt.graph, evt.item, evt.dataItem)
* })
*
* builder.updateGraph()
* </pre>
*
* The different graph builders are discussed in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph_builder Creating a Graph from Business Data}. Class <code>TreeBuilder</code>, in
* particular, is topic of section {@link https://docs.yworks.com/yfileshtml/#/dguide/graph_builder-TreeBuilder TreeBuilder}.
*
* @see GraphBuilder
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder Online Documentation}
*/
export class TreeBuilder {
/**
Initializes a new instance of the {@link TreeBuilder} class that operates on the given graph.
@param [graph=null] An {@link IGraph} instance on which this builder operates or <code>null</code> if a new {@link DefaultGraph} should be
created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder Online Documentation}*/
constructor(graph?: IGraph | null);
/**
Binds a collection of root data items to the given <code>nodesSource</code>.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param data The collection of objects that is bound to the source.
@param nodesSource The source to which the data is bound.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23addRootNodesSource(Object%2CTreeNodesSource%3CTDataItem%3E) Online Documentation}*/
addRootNodesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, nodesSource: TreeNodesSource<TDataItem>): void;
/**
Populates the graph with items generated from the bound data.
This graph is not cleared before the new items are added. To clear the graph, explicitly call {@link IGraph#clear}
beforehand.
@returns The {@link TreeBuilder#graph} of this builder populated by the defined sources.
@see TreeBuilder#updateGraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23buildGraph() Online Documentation}*/
buildGraph(): IGraph;
/**
Registers a collection of root group data items functioning as entities for the {@link NodeCreator} of the returned {@link TreeNodesSource}.
Type parameter <code>TDataItem</code>: The type of the root data items in the source.
@param data The collection of objects to iterate and create the group root nodes from.
@param [idProvider=null] An optional function that yields an id for each element in the <code>data</code>.
@returns A new {@link TreeNodesSource} instance that can be used to further customize the creation, e.g. provide specific {@link NodeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23createRootGroupNodesSource(Object%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createRootGroupNodesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, idProvider?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>): TreeNodesSource<TDataItem>;
/**
Registers a collection of root data items functioning as entities for the {@link NodeCreator} of the returned {@link TreeNodesSource}.
Type parameter <code>TDataItem</code>: The type of the root data items in the source.
@param data The collection of objects to iterate and create the root nodes from.
@param [idProvider=null] An optional function that yields an id for each element in the <code>data</code>.
@returns A new {@link TreeNodesSource} instance that can be used to further customize the creation, e.g. provide specific {@link NodeCreator#defaults style defaults}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23createRootNodesSource(Object%2Cfunction(TDataItem%2CObject)%3AObject) Online Documentation}*/
createRootNodesSource<TDataItem>(data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}, idProvider?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>): TreeNodesSource<TDataItem>;
/**
Triggers the {@link TreeBuilder#addEdgeCreatedListener EdgeCreated} event.
@param edge The edge that has been created.
@param dataItem The data item from which the edge has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23onEdgeCreated(IEdge%2CObject) Online Documentation}*/
onEdgeCreated(edge: IEdge, dataItem: any): void;
/**
Triggers the {@link TreeBuilder#addEdgeRemovedListener EdgeRemoved} event.
@param edge The edge that has been removed.
@param dataItem The corresponding data item of the removed edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23onEdgeRemoved(IEdge%2CObject) Online Documentation}*/
onEdgeRemoved(edge: IEdge, dataItem: any): void;
/**
Triggers the {@link TreeBuilder#addEdgeUpdatedListener EdgeUpdated} event.
@param edge The edge that has been updated.
@param dataItem The data item with which the edge has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23onEdgeUpdated(IEdge%2CObject) Online Documentation}*/
onEdgeUpdated(edge: IEdge, dataItem: any): void;
/**
Triggers the {@link TreeBuilder#addLabelAddedListener LabelAdded} event.
@param label The label that has been added.
@param dataItem The data item from which the label has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23onLabelAdded(ILabel%2CObject) Online Documentation}*/
onLabelAdded(label: ILabel, dataItem: any): void;
/**
Triggers the {@link TreeBuilder#addLabelRemovedListener LabelRemoved} event.
@param label The label that has been removed.
@param dataItem The corresponding data item of the removed label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23onLabelRemoved(ILabel%2CObject) Online Documentation}*/
onLabelRemoved(label: ILabel, dataItem: any): void;
/**
Triggers the {@link TreeBuilder#addLabelUpdatedListener LabelUpdated} event.
@param label The label that has been updated.
@param dataItem The data item with which the label has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23onLabelUpdated(ILabel%2CObject) Online Documentation}*/
onLabelUpdated(label: ILabel, dataItem: any): void;
/**
Triggers the {@link TreeBuilder#addNodeCreatedListener NodeCreated} event.
@param node The node that has been created.
@param dataItem The data item from which the node has been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23onNodeCreated(INode%2CObject) Online Documentation}*/
onNodeCreated(node: INode, dataItem: any): void;
/**
Triggers the {@link TreeBuilder#addNodeRemovedListener NodeRemoved} event.
@param node The node that has been removed.
@param dataItem The corresponding data item of the removed node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23onNodeRemoved(INode%2CObject) Online Documentation}*/
onNodeRemoved(node: INode, dataItem: any): void;
/**
Triggers the {@link TreeBuilder#addNodeUpdatedListener NodeUpdated} event.
@param node The node that has been updated.
@param dataItem The data item with which the node has been updated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23onNodeUpdated(INode%2CObject) Online Documentation}*/
onNodeUpdated(node: INode, dataItem: any): void;
/**
Binds a new data collection to a {@link NodesSource}, replacing the old one.
This can be used to assign an entirely different collection instance to an existing <code>nodesSource</code>, such that {@link TreeBuilder#updateGraph}
considers the new collection.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@param nodesSource The source whose data source should be re-assigned.
@param data The collection of objects that is specified for the given source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23setData(NodesSource%3CTDataItem%3E%2CObject) Online Documentation}*/
setData<TDataItem>(nodesSource: NodesSource<TDataItem>, data: TDataItem[] | Iterable<TDataItem> | Map<any, TDataItem> | {
[id: string]: TDataItem;
}): void;
/**
* Updates the graph after changes in the bound data.
* Graph elements corresponding to objects that are still present in the source collections are kept, new graph elements
* are created for new objects in the collections, and obsolete ones are removed.
*
* <h2>Note</h2>
*
* {@link TreeBuilder#updateGraph} merely updates the graph structure. Any other bound data must be updated
* explicitly if necessary. Calling the respective <code>update</code> methods on the <code>creators</code> (e.g. {@link NodeCreator#updateStyle}), resolves the <code>provider</code>
* (e.g. {@link NodeCreator#styleProvider}) and applies the <code>bindings</code> (e.g.
* {@link NodeCreator#applyStyleBindings}). The latter can also be applied solely.
* <br />
* For example, to update any aspect of node creation:
*
* <pre>
* // evaluate the providers and apply respective bindings
* nodesSource.nodeCreator.addNodeUpdatedListener((sender, evt) => {
*   nodesSource.nodeCreator.updateLayout(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateStyle(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateTag(evt.graph, evt.item, evt.dataItem)
*   nodesSource.nodeCreator.updateLabels(evt.graph, evt.item, evt.dataItem)
* })
*
* builder.updateGraph()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23updateGraph() Online Documentation}
*/
updateGraph(): void;
/**
Gets the {@link IGraph graph} used by this builder.
The style defaults of this graph are considered in the {@link NodeCreator}, {@link EdgeCreator} and {@link LabelCreator} of the sources that are created on this
{@link TreeBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23graph Online Documentation}*/
readonly graph: IGraph;
/**
Adds the given listener for the <code>NodeCreated</code> event that occurs when a node has been created.
This event can be used to further customize the created node.
New nodes are created either in response to calling {@link TreeBuilder#buildGraph}, or in response to calling
{@link TreeBuilder#updateGraph}.
@param listener The listener to add.
@see TreeBuilder#addNodeUpdatedListener
@see TreeBuilder#addNodeRemovedListener
@see TreeBuilder#removeNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23NodeCreated Online Documentation}*/
addNodeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeCreated</code> event that occurs when a node has been created.
This event can be used to further customize the created node.
New nodes are created either in response to calling {@link TreeBuilder#buildGraph}, or in response to calling
{@link TreeBuilder#updateGraph}.
@param listener The listener to remove.
@see TreeBuilder#addNodeUpdatedListener
@see TreeBuilder#addNodeRemovedListener
@see TreeBuilder#addNodeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23NodeCreated Online Documentation}*/
removeNodeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>NodeUpdated</code> event that occurs when a node has been updated.
This event can be used to further customize the created node.
@param listener The listener to add.
@see TreeBuilder#addNodeCreatedListener
@see TreeBuilder#addNodeRemovedListener
@see TreeBuilder#removeNodeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23NodeUpdated Online Documentation}*/
addNodeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeUpdated</code> event that occurs when a node has been updated.
This event can be used to further customize the created node.
@param listener The listener to remove.
@see TreeBuilder#addNodeCreatedListener
@see TreeBuilder#addNodeRemovedListener
@see TreeBuilder#addNodeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23NodeUpdated Online Documentation}*/
removeNodeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed.
This event can be used to further customize the created node.
Nodes are removed in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to add.
@see TreeBuilder#addNodeCreatedListener
@see TreeBuilder#addNodeUpdatedListener
@see TreeBuilder#removeNodeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23NodeRemoved Online Documentation}*/
addNodeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Removes the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed.
This event can be used to further customize the created node.
Nodes are removed in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to remove.
@see TreeBuilder#addNodeCreatedListener
@see TreeBuilder#addNodeUpdatedListener
@see TreeBuilder#addNodeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23NodeRemoved Online Documentation}*/
removeNodeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<INode, any>) => void): void;
/**
Adds the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created.
This event can be used to further customize the created node.
New edges are created either in response to calling {@link TreeBuilder#buildGraph}, or in response to calling
{@link TreeBuilder#updateGraph}.
@param listener The listener to add.
@see TreeBuilder#addEdgeUpdatedListener
@see TreeBuilder#addEdgeRemovedListener
@see TreeBuilder#removeEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23EdgeCreated Online Documentation}*/
addEdgeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created.
This event can be used to further customize the created node.
New edges are created either in response to calling {@link TreeBuilder#buildGraph}, or in response to calling
{@link TreeBuilder#updateGraph}.
@param listener The listener to remove.
@see TreeBuilder#addEdgeUpdatedListener
@see TreeBuilder#addEdgeRemovedListener
@see TreeBuilder#addEdgeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23EdgeCreated Online Documentation}*/
removeEdgeCreatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>EdgeUpdated</code> event that occurs when an edge has been updated.
This event can be used to further customize the created node.
Edges are updated in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to add.
@see TreeBuilder#addEdgeCreatedListener
@see TreeBuilder#addNodeRemovedListener
@see TreeBuilder#removeEdgeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23EdgeUpdated Online Documentation}*/
addEdgeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeUpdated</code> event that occurs when an edge has been updated.
This event can be used to further customize the created node.
Edges are updated in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to remove.
@see TreeBuilder#addEdgeCreatedListener
@see TreeBuilder#addNodeRemovedListener
@see TreeBuilder#addEdgeUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23EdgeUpdated Online Documentation}*/
removeEdgeUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
This event can be used to further customize the created node.
Edges are removed in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to add.
@see TreeBuilder#addEdgeUpdatedListener
@see TreeBuilder#addNodeRemovedListener
@see TreeBuilder#removeEdgeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23EdgeRemoved Online Documentation}*/
addEdgeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Removes the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
This event can be used to further customize the created node.
Edges are removed in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to remove.
@see TreeBuilder#addEdgeUpdatedListener
@see TreeBuilder#addNodeRemovedListener
@see TreeBuilder#addEdgeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23EdgeRemoved Online Documentation}*/
removeEdgeRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<IEdge, any>) => void): void;
/**
Adds the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to a node or edge.
This event can be used to further customize the created label.
New labels are added in response to calling {@link TreeBuilder#buildGraph}, or in response to calling
{@link TreeBuilder#updateGraph}.
@param listener The listener to add.
@see TreeBuilder#addLabelUpdatedListener
@see TreeBuilder#addLabelRemovedListener
@see TreeBuilder#removeLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23LabelAdded Online Documentation}*/
addLabelAddedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to a node or edge.
This event can be used to further customize the created label.
New labels are added in response to calling {@link TreeBuilder#buildGraph}, or in response to calling
{@link TreeBuilder#updateGraph}.
@param listener The listener to remove.
@see TreeBuilder#addLabelUpdatedListener
@see TreeBuilder#addLabelRemovedListener
@see TreeBuilder#addLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23LabelAdded Online Documentation}*/
removeLabelAddedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Adds the given listener for the <code>LabelUpdated</code> event that occurs when a node or edge label has been updated.
This event can be used to further customize the created label.
Labels are updated in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to add.
@see TreeBuilder#addLabelAddedListener
@see TreeBuilder#addLabelRemovedListener
@see TreeBuilder#removeLabelUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23LabelUpdated Online Documentation}*/
addLabelUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelUpdated</code> event that occurs when a node or edge label has been updated.
This event can be used to further customize the created label.
Labels are updated in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to remove.
@see TreeBuilder#addLabelAddedListener
@see TreeBuilder#addLabelRemovedListener
@see TreeBuilder#addLabelUpdatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23LabelUpdated Online Documentation}*/
removeLabelUpdatedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Adds the given listener for the <code>LabelRemoved</code> event that occurs when a node or edge label has been removed.
This event can be used to further customize the created label.
Labels are removed in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to add.
@see TreeBuilder#addLabelAddedListener
@see TreeBuilder#addLabelUpdatedListener
@see TreeBuilder#removeLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23LabelRemoved Online Documentation}*/
addLabelRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
/**
Removes the given listener for the <code>LabelRemoved</code> event that occurs when a node or edge label has been removed.
This event can be used to further customize the created label.
Labels are removed in response to calling {@link TreeBuilder#updateGraph}.
@param listener The listener to remove.
@see TreeBuilder#addLabelAddedListener
@see TreeBuilder#addLabelUpdatedListener
@see TreeBuilder#addLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeBuilder%23LabelRemoved Online Documentation}*/
removeLabelRemovedListener(listener: (sender: any, evt: GraphBuilderItemEventArgs<ILabel, any>) => void): void;
static $class: Class;
private $_TreeBuilder: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeBuilder;
}
export interface TreeNodesSource<TDataItem> extends NodesSource<TDataItem> {
}
/**
A class that accepts a collection of objects from the {@link TreeBuilder} to build {@link INode}s from.
It manages the structural aspects of the graph creation, while the associated {@link NodesSource#nodeCreator} is responsible for the properties of the
created items.
Type parameter <code>TDataItem</code>: The type of the data items in the source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeNodesSource Online Documentation}*/
export class TreeNodesSource<TDataItem> {
/**
Creates a new source with the given <code>nodeCreator</code> and <code>parentEdgeCreator</code>.
@param nodeCreator The creator that is used to create the nodes of this source.
@param parentEdgeCreator The creator that is used to create the edges of this source.
@param [idProvider=null] An optional function that yields an id for each element of the associated data collection in the
{@link TreeBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeNodesSource Online Documentation}*/
constructor(nodeCreator: NodeCreator<TDataItem>, parentEdgeCreator: EdgeCreator<TDataItem>, idProvider?: ((dataItem: TDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TDataItem>);
/**
Binds a collection of child data items to the given <code>nodesSource</code>.
Type parameter <code>TChildDataItem</code>: The type of the child data items in the source.
@param childDataProvider A function that yields a child data item for each element of the associated data collection in the
{@link TreeBuilder}.
@param nodesSource The child node source to which the child data is bound.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeNodesSource%23addChildNodesSource(function(TDataItem)%3AObject%2CTreeNodesSource%3CTChildDataItem%3E) Online Documentation}*/
addChildNodesSource<TChildDataItem>(childDataProvider: ((data: TDataItem) => TChildDataItem[] | Iterable<TChildDataItem> | Map<any, TChildDataItem> | {
[id: string]: TChildDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TChildDataItem[] | Iterable<TChildDataItem> | Map<any, TChildDataItem> | {
[id: string]: TChildDataItem;
}>, nodesSource: TreeNodesSource<TChildDataItem>): void;
/**
Registers a collection of child data items functioning as child entities for the {@link NodeCreator} of the returned {@link TreeNodesSource}.
The {@link NodeCreator#defaults node defaults} and {@link EdgeCreator#defaults edge defaults} of this newly created source cascade with the defaults of this source.
Type parameter <code>TChildDataItem</code>: The type of the child data items in the source.
@param childDataProvider A function that yields a child data item for each element of the associated data collection in the
{@link TreeBuilder}.
@param [idProvider=null] An optional function that yields an id for each element of the associated data collection in the
{@link TreeBuilder}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeNodesSource%23createChildNodesSource(function(TDataItem)%3AObject%2Cfunction(TChildDataItem%2CObject)%3AObject) Online Documentation}*/
createChildNodesSource<TChildDataItem>(childDataProvider: ((data: TDataItem) => TChildDataItem[] | Iterable<TChildDataItem> | Map<any, TChildDataItem> | {
[id: string]: TChildDataItem;
} | null | undefined) | ProviderConvertible<TDataItem, TChildDataItem[] | Iterable<TChildDataItem> | Map<any, TChildDataItem> | {
[id: string]: TChildDataItem;
}>, idProvider?: ((dataItem: TChildDataItem, canonicalId: any | null) => any) | null | IdProviderConvertible<TChildDataItem>): TreeNodesSource<TChildDataItem>;
/**
Gets or sets the {@link NodeCreator} for this source which can be used to adjust the creation behavior of the edges between child nodes and
their parents.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeNodesSource%23parentEdgeCreator Online Documentation}*/
parentEdgeCreator: EdgeCreator<TDataItem>;
static $class: Class;
private $_TreeNodesSource: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TDataItem>(o: any): o is TreeNodesSource<TDataItem>;
}
export interface CollapsibleNodeStyleDecorator extends YObject, INodeStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
Decorates an arbitrary node styles with an additional collapse/expand handle.
This style uses the {@link CollapsibleNodeStyleDecoratorRenderer} to adorn an existing {@link INodeStyle}
with a <code>Button</code> that can then be used to trigger the {@link ICommand#TOGGLE_EXPANSION_STATE}.
By default, this handle is rendered as a plus ("+") sign when in collapsed state, and a minus ("-") sign when in
expanded state.
The collapse button can be styled using the CSS class selectors 'yfiles-collapsebutton', 'yfiles-collapsebutton-checked'
and 'yfiles-collapsebutton-unchecked'.
@see CollapsibleNodeStyleDecoratorRenderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecorator Online Documentation}*/
export class CollapsibleNodeStyleDecorator {
/**
Initializes a new instance of the {@link CollapsibleNodeStyleDecorator} class using the provided style for the {@link CollapsibleNodeStyleDecorator#wrapped}
property and the provided {@link CollapsibleNodeStyleDecoratorRenderer} as renderer.
@param wrapped The decorated style.
@param [renderer=null] An instance of {@link CollapsibleNodeStyleDecoratorRenderer} to render this style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecorator%23CollapsibleNodeStyleDecorator(INodeStyle%2CCollapsibleNodeStyleDecoratorRenderer) Online Documentation}*/
constructor(wrapped: INodeStyle, renderer?: CollapsibleNodeStyleDecoratorRenderer | null);
/**
Initializes a new instance of the {@link CollapsibleNodeStyleDecorator} class using the provided style for the {@link CollapsibleNodeStyleDecorator#wrapped}
property and the provided {@link CollapsibleNodeStyleDecoratorRenderer} as renderer.
@param options The parameters to pass.
@param options.wrapped The decorated style.
@param [options.renderer=null] An instance of {@link CollapsibleNodeStyleDecoratorRenderer} to render this style.
@param [options.buttonPlacement] An {@link ILabelModelParameter} that determines the placement of the button for toggling the expanded state. This option sets the {@link CollapsibleNodeStyleDecorator#buttonPlacement} property on the created object.
@param [options.insets] The insets to use for the {@link INodeInsetsProvider} that will be reported by the {@link CollapsibleNodeStyleDecoratorRenderer}'s {@link ILookup}. This option sets the {@link CollapsibleNodeStyleDecorator#insets} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecorator%23CollapsibleNodeStyleDecorator(INodeStyle%2CCollapsibleNodeStyleDecoratorRenderer) Online Documentation}*/
constructor(options: {
wrapped: INodeStyle;
renderer?: CollapsibleNodeStyleDecoratorRenderer | null;
buttonPlacement?: ILabelModelParameter;
insets?: Insets | InsetsConvertible;
});
/**
Initializes a new instance of the {@link CollapsibleNodeStyleDecorator} class.
The {@link CollapsibleNodeStyleDecorator#wrapped} property will be initialized with a {@link ShapeNodeStyle}.
@param [options] The parameters to pass.
@param [options.buttonPlacement] An {@link ILabelModelParameter} that determines the placement of the button for toggling the expanded state. This option sets the {@link CollapsibleNodeStyleDecorator#buttonPlacement} property on the created object.
@param [options.insets] The insets to use for the {@link INodeInsetsProvider} that will be reported by the {@link CollapsibleNodeStyleDecoratorRenderer}'s {@link ILookup}. This option sets the {@link CollapsibleNodeStyleDecorator#insets} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecorator%23CollapsibleNodeStyleDecorator() Online Documentation}*/
constructor(options?: {
buttonPlacement?: ILabelModelParameter;
insets?: Insets | InsetsConvertible;
});
/**
Gets or sets an {@link ILabelModelParameter} that determines the placement of the button for toggling the expanded
state.
The {@link ILabelModelParameter} should {@link ILabelModelParameter#supports support} labels bound to an {@link INode} because the button will be
positioned as if it was an {@link ILabel} owned by the {@link INode} that uses this style.
@default 'yWorks.yFiles.UI.LabelModels.InteriorLabelModel.NorthWest'
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecorator%23buttonPlacement Online Documentation}*/
buttonPlacement: ILabelModelParameter;
/**
Gets or sets the insets to use for the {@link INodeInsetsProvider} that will be reported by the
{@link CollapsibleNodeStyleDecoratorRenderer}'s {@link ILookup}.
This effectively influences the way the bounds of a group node are being calculated. The default {@link IGroupBoundsCalculator}
implementation will use these insets to determine the minimum size of a group node given a set of contained nodes.
@default '5, 16, 5, 5'
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecorator%23insets Online Documentation}*/
insets: Insets;
/**
Gets or sets the wrapped node style that will be used to perform the actual rendering of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecorator%23wrapped Online Documentation}*/
wrapped: INodeStyle;
static $class: Class;
private $_CollapsibleNodeStyleDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CollapsibleNodeStyleDecorator;
}
export interface CollapsibleNodeStyleDecoratorRenderer extends YObject, INodeStyleRenderer, IShapeGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
{@link INodeStyleRenderer} implementation that can be used in conjunction with {@link CollapsibleNodeStyleDecorator}
instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer Online Documentation}*/
export class CollapsibleNodeStyleDecoratorRenderer {
/**
Initializes a new instance of the {@link CollapsibleNodeStyleDecoratorRenderer} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer Online Documentation}*/
constructor();
/**
Adds the toggle expansion state command to the given button visual.
This method adds event listeners for click and tap events to the given button visual that execute
{@link ICommand#TOGGLE_EXPANSION_STATE}. It is called by
{@link CollapsibleNodeStyleDecoratorRenderer#createButton}.
@param button The button visual to add the event listeners to.
@param currentNode The group node whose state is to be toggled.
@param context The context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23addToggleExpansionStateCommand(SvgVisual%2CINode%2CIRenderContext) Online Documentation}*/
static addToggleExpansionStateCommand(button: SvgVisual, currentNode: INode, context: IRenderContext): void;
/**
Prepares this instance for subsequent calls after the style and node have been initialized.
Upon invocation the {@link CollapsibleNodeStyleDecoratorRenderer#style} and {@link CollapsibleNodeStyleDecoratorRenderer#node} properties have been populated by the
{@link CollapsibleNodeStyleDecoratorRenderer#getVisualCreator},
{@link CollapsibleNodeStyleDecoratorRenderer#getBoundsProvider},
{@link CollapsibleNodeStyleDecoratorRenderer#getHitTestable}, or {@link CollapsibleNodeStyleDecoratorRenderer#getMarqueeTestable}
methods.
This is an empty implementation. Subclasses might have to override this method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Creates the button visual.
This method is called from {@link CollapsibleNodeStyleDecoratorRenderer#createVisual} to create the button. Custom
implementations can override this method to use a custom button visualization.
This implementation adds the {@link ICommand#TOGGLE_EXPANSION_STATE} command to the button. The created button is
automatically unchecked if the group node is {@link IFoldingView#isExpanded expanded} and vice versa.
@param context The context.
@param expanded Specifies whether the button visual is created for the expanded or the collapsed state.
@param size The button size.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23createButton(IRenderContext%2Cboolean%2CSize) Online Documentation}*/
createButton(context: IRenderContext, expanded: boolean, size: Size): SvgVisual;
/**
Creates the visual for the given context.
This method uses {@link CollapsibleNodeStyleDecoratorRenderer#createButton} to create the togglebutton instance.
It will then use the {@link CollapsibleNodeStyleDecoratorRenderer#getButtonLocationParameter} to determine the
placement of the button and use the {@link CollapsibleNodeStyleDecorator#wrapped wrapped style} to create the visual for the actual node.
@param context
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23createVisual(IRenderContext) Online Documentation}*/
createVisual(context: IRenderContext): Visual | null;
/**
Delegates to the {@link CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle} and takes the button into account.
@param context
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getBounds(ICanvasContext) Online Documentation}*/
getBounds(context: ICanvasContext): Rect;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link CollapsibleNodeStyleDecoratorRenderer#configure} and
returns <code>this</code>.
@param node The node to retrieve the bounds provider for. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#style} property.
@returns <code>this</code>
@see CollapsibleNodeStyleDecoratorRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Yields the {@link CollapsibleNodeStyleDecorator#buttonPlacement} for the current {@link CollapsibleNodeStyleDecoratorRenderer#style}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getButtonLocationParameter() Online Documentation}*/
getButtonLocationParameter(): ILabelModelParameter;
/**
Returns the size of new buttons.
@returns The size of new buttons
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getButtonSize() Online Documentation}*/
getButtonSize(): Size;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link CollapsibleNodeStyleDecoratorRenderer#configure} and
returns <code>this</code>.
As this method may be called often it will not automatically call
{@link CollapsibleNodeStyleDecoratorRenderer#configure}, instead subclasses should ensure that in the {@link CollapsibleNodeStyleDecoratorRenderer#lookup}
method call they should call {@link CollapsibleNodeStyleDecoratorRenderer#configure} only if needed, i.e. if they
decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
@param node The node to query the context for. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#style} property.
@returns <code>this</code>
@see CollapsibleNodeStyleDecoratorRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link CollapsibleNodeStyleDecoratorRenderer#configure} and
returns <code>this</code>.
@param node The node to query hit test with. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#style} property.
@returns <code>this</code>
@see CollapsibleNodeStyleDecoratorRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Delegates to the {@link CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
@param inner
@param outer
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getIntersection(Point%2CPoint) Online Documentation}*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link CollapsibleNodeStyleDecoratorRenderer#configure} and
returns <code>this</code>.
@param node The node to query marquee intersection tests. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#style} property.
@returns <code>this</code>
@see CollapsibleNodeStyleDecoratorRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
Delegates to the {@link CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getOutline() Online Documentation}*/
getOutline(): GeneralPath | null;
/**
Configures the {@link CollapsibleNodeStyleDecoratorRenderer#style} and {@link CollapsibleNodeStyleDecoratorRenderer#node} properties, calls {@link CollapsibleNodeStyleDecoratorRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the shape geometry for. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#style} property.
@returns <code>this</code>
@see CollapsibleNodeStyleDecoratorRenderer#isInside
@see CollapsibleNodeStyleDecoratorRenderer#getIntersection
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link CollapsibleNodeStyleDecoratorRenderer#configure} and
returns <code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call
{@link CollapsibleNodeStyleDecoratorRenderer#configure}. If the subclass implementation depends on this instance
to be configured, it needs to call <code>Configure</code> in {@link CollapsibleNodeStyleDecoratorRenderer#isVisible}.
@param node The node to query visibility tests. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#style} property.
@returns <code>this</code>
@see CollapsibleNodeStyleDecoratorRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link CollapsibleNodeStyleDecoratorRenderer#configure} and
returns <code>this</code>.
@param node The node to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link CollapsibleNodeStyleDecoratorRenderer#style} property.
@returns <code>this</code>
@see CollapsibleNodeStyleDecoratorRenderer#createVisual
@see CollapsibleNodeStyleDecoratorRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
Yields the {@link CollapsibleNodeStyleDecorator#wrapped} property for the current {@link CollapsibleNodeStyleDecoratorRenderer#style}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23getWrappedStyle() Online Documentation}*/
getWrappedStyle(): INodeStyle;
/**
Delegates to the {@link CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle} and checks the button.
@param context
@param location
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23isHit(IInputModeContext%2CPoint) Online Documentation}*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
Delegates to the {@link CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
@param context
@param rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23isInBox(IInputModeContext%2CRect) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
Delegates to the {@link CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
@param location
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23isInside(Point) Online Documentation}*/
isInside(location: Point): boolean;
/**
Delegates to the {@link CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
@param context
@param rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Updates the button visual.
This method is called from {@link CollapsibleNodeStyleDecoratorRenderer#updateVisual} to update an existing button
or to create a new one. Custom implementations can override this method to use a custom button visualization.
@param context The context.
@param expanded Specifies whether the button visual is updated for the expanded or the collapsed state.
@param size The button size.
@param oldButton The old button visual.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23updateButton(IRenderContext%2Cboolean%2CSize%2CVisual) Online Documentation}*/
updateButton(context: IRenderContext, expanded: boolean, size: Size, oldButton: Visual | null): SvgVisual;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23style Online Documentation}*/
style: CollapsibleNodeStyleDecorator;
/**
Gets or sets the currently configured node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23node Online Documentation}*/
node: INode;
/**
Gets the {@link INode#layout} of the node this renderer is currently configured for.
Returns <code>Node.Layout</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CollapsibleNodeStyleDecoratorRenderer%23layout Online Documentation}*/
readonly layout: IRectangle;
static $class: Class;
private $_CollapsibleNodeStyleDecoratorRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CollapsibleNodeStyleDecoratorRenderer;
}
export interface EdgeStyleBase extends YObject, IEdgeStyle {
}
/**
An abstract base class that makes it possible to easily implement a custom {@link IEdgeStyle}.
The only method that needs to be implemented by subclasses is {@link EdgeStyleBase#createVisual}, however to
improve rendering performance it is highly recommended to implement at least
{@link EdgeStyleBase#updateVisual}, too.
This implementation differs from the straightforward {@link IEdgeStyle} implementation in that there is no visible
separation between the style and its
{@link IEdgeStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
the style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase Online Documentation}*/
export abstract class EdgeStyleBase {
/**
Initializes a new instance of the {@link EdgeStyleBase} class.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase Online Documentation}*/
constructor();
/**
Adds the arrows to a given container.
@param context The context for the rendering.
@param group The container to which the arrows should be added.
@param edge The edge that is being rendered.
@param edgePath The edge path.
@param sourceArrow The source arrow.
@param targetArrow The target arrow.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23addArrows(IRenderContext%2CSVGGElement%2CIEdge%2CGeneralPath%2CIArrow%2CIArrow) Online Documentation}*/
addArrows(context: IRenderContext, group: SVGGElement, edge: IEdge, edgePath: GeneralPath, sourceArrow: IArrow, targetArrow: IArrow): void;
/**
Creates a new object that is a copy of the current instance.
Immutable subclasses should consider returning <code>this</code>.
@returns A new object that is a copy of this instance using {@link YObject#memberwiseClone}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23clone() Online Documentation}*/
clone(): this;
/**
Creates the visual.
This method is called in response to a {@link IVisualCreator#createVisual} call to the instance that has been
queried from the {@link EdgeStyleBase#renderer}.
@param context The render context.
@param edge The edge to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see EdgeStyleBase#updateVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23createVisual(IRenderContext%2CIEdge) Online Documentation}*/
abstract createVisual(context: IRenderContext, edge: IEdge): Visual | null;
/**
Crops the edge's path at the nodes.
This implementation uses the {@link IEdgePathCropper} instances found in the {@link ILookup#lookup} of the
source and target port of the edge to perform the actual cropping.
@param edge The edge that is being rendered.
@param path The path that should be cropped.
@param sourceArrow The source arrow instance.
@param targetArrow The target arrow instance.
@returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23cropPath(IEdge%2CIArrow%2CIArrow%2CGeneralPath) Online Documentation}*/
cropPath(edge: IEdge, sourceArrow: IArrow, targetArrow: IArrow, path: GeneralPath): GeneralPath | null;
/**
Gets the bounds of the visual for the edge in the given context.
This method is called in response to a {@link IBoundsProvider#getBounds} call to the instance that has been queried
from the {@link EdgeStyleBase#renderer}. This implementation simply yields a {@link Rect rectangle} containing. the locations of the {@link IEdge#sourcePort source port} and the {@link IEdge#targetPort target port}
of the edge and the locations of all its {@link IEdge#bends bends}.
@param context The canvas context.
@param edge The edge to which this style instance is assigned.
@returns The visual bounds of the visual representation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23getBounds(ICanvasContext%2CIEdge) Online Documentation}*/
getBounds(context: ICanvasContext, edge: IEdge): Rect;
/**
Gets the path of the edge.
@param edge The edge.
@returns The path.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23getPath(IEdge) Online Documentation}*/
getPath(edge: IEdge): GeneralPath | null;
/**
Gets the number of segments of the edge.
@param edge The edge.
@returns The segment count.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23getSegmentCount(IEdge) Online Documentation}*/
getSegmentCount(edge: IEdge): number;
/**
Calculates the source arrow anchor and direction for a a given arrow and path.
@param path The path of the edge.
@param arrow The arrow.
@returns The anchor and directional vector of the arrow, if any exist.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23getSourceArrowAnchor(GeneralPath%2CIArrow) Online Documentation}*/
getSourceArrowAnchor(path: GeneralPath, arrow: IArrow): Tangent | null;
/**
Gets the tangent to the edge at the specified ratio and the corresponding touch point.
@param edge The edge.
@param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of the path of the edge.
@returns The tangent, if any exists.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23getTangent(IEdge%2Cnumber) Online Documentation}*/
getTangent(edge: IEdge, ratio: number): Tangent | null;
/**
Gets the tangent to the edge at the specified ratio of a segment of the edge and the corresponding touch point.
@param edge The edge.
@param segmentIndex Index of the segment of the edge.
@param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of the segment of the edge.
@returns The tangent, if any exists.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23getTangentForSegment(IEdge%2Cnumber%2Cnumber) Online Documentation}*/
getTangentForSegment(edge: IEdge, segmentIndex: number, ratio: number): Tangent | null;
/**
Calculates the target arrow anchor and direction for a a given arrow and path.
@param path The path of the edge.
@param arrow The arrow.
@returns The anchor and directional vector of the arrow, if any exist.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23getTargetArrowAnchor(GeneralPath%2CIArrow) Online Documentation}*/
getTargetArrowAnchor(path: GeneralPath, arrow: IArrow): Tangent | null;
/**
Determines whether the visual representation of the edge has been hit at the given location.
This method is called in response to a {@link IHitTestable#isHit} call to the instance that has been queried from
the {@link EdgeStyleBase#renderer}.
This implementation returns the result of the {@link Point#hitsPolyline} method of class {@link Point}
with the polygonal line defined by the source port, the target port and the bends of the edge and the {@link ICanvasContext#hitTestRadius} of the
{@link ICanvasContext canvas context}.
@param context The input mode context.
@param location The point to test.
@param edge The edge to which this style instance is assigned.
@returns <code>true</code> if the specified edge representation is hit; <code>false</code> otherwise.
@see Point#hitsPolyline
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23isHit(IInputModeContext%2CPoint%2CIEdge) Online Documentation}*/
isHit(context: IInputModeContext, location: Point, edge: IEdge): boolean;
/**
Determines whether the visualization for the specified edge is included in the marquee selection.
This method is called in response to a {@link IMarqueeTestable#isInBox} call to the instance that has been queried
from the {@link EdgeStyleBase#renderer}.
This implementation returns the result of the {@link Rect#intersectsPolyline} method of class {@link Rect}
with the polygonal line defined by the source port, the target port and the bends of the edge.
@param context The input mode context.
@param rectangle The marquee selection box.
@param edge The edge to which this style instance is assigned.
@returns <code>true</code> if the specified edge is visible is selected by the marquee rectangle; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23isInBox(IInputModeContext%2CRect%2CIEdge) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect, edge: IEdge): boolean;
/**
Determines whether the visualization for the specified edge is included in the lasso selection.
This method is called in response to a {@link ILassoTestable#isInPath} call to the instance that has been queried
from the {@link EdgeStyleBase#renderer}.
This implementation uses the results of the {@link GeneralPath#findLineIntersection} method with the polygonal
line defined by the source port, the target port and the bends of the edge.
@param context The input mode context.
@param path The lasso selection path.
@param edge The edge to which this style instance is assigned.
@returns <code>true</code> if the specified edge is visible is selected by the lasso path; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23isInPath(IInputModeContext%2CGeneralPath%2CIEdge) Online Documentation}*/
isInPath(context: IInputModeContext, path: GeneralPath, edge: IEdge): boolean;
/**
Determines whether the visualization for the specified edge is visible in the context.
This method is called in response to a {@link IVisibilityTestable#isVisible} call to the instance that has been
queried from the {@link EdgeStyleBase#renderer}. This implementation simply tests whether the {@link EdgeStyleBase#getBounds bounds} intersect the clip.
@param context The canvas context.
@param rectangle The clipping rectangle.
@param edge The edge to which this style instance is assigned.
@returns <code>true</code> if the specified edge is visible in the clipping rectangle; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23isVisible(ICanvasContext%2CRect%2CIEdge) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect, edge: IEdge): boolean;
/**
Performs the {@link ILookup#lookup} operation for the {@link IEdgeStyleRenderer#getContext} that has been
queried from the {@link EdgeStyleBase#renderer}.
This implementation yields <code>null</code> for everything but:
<ul>
<li>{@link IHitTestable}
<li>{@link IVisualCreator}
<li>{@link IBoundsProvider}
<li>{@link IVisibilityTestable}
<li>{@link IMarqueeTestable}
<li>{@link ILookup}
<li>{@link IPathGeometry}
</ul>
For these interfaces an implementation will be returned that delegates to the methods in this instance.
@param edge The edge to use for the context lookup.
@param type The type to query.
@returns An implementation of the <code>type</code> or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23lookup(IEdge%2CClass) Online Documentation}*/
lookup(edge: IEdge, type: Class): any | null;
/**
Updates the arrows in a given container.
@param context The context for the rendering.
@param group The container to which the arrows should be added.
@param edge The edge that is being rendered.
@param edgePath The edge path.
@param sourceArrow The source arrow.
@param targetArrow The target arrow.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23updateArrows(IRenderContext%2CSVGGElement%2CIEdge%2CGeneralPath%2CIArrow%2CIArrow) Online Documentation}*/
updateArrows(context: IRenderContext, group: SVGGElement, edge: IEdge, edgePath: GeneralPath, sourceArrow: IArrow, targetArrow: IArrow): void;
/**
Updates the visual previously created by {@link EdgeStyleBase#createVisual}.
This method is called in response to a {@link IVisualCreator#updateVisual} call to the instance that has been
queried from the {@link EdgeStyleBase#renderer}. This implementation simply delegates to {@link EdgeStyleBase#createVisual} so subclasses should
override to improve rendering performance.
@param context The render context.
@param oldVisual The visual that has been created in the call to {@link EdgeStyleBase#createVisual}.
@param edge The edge to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see EdgeStyleBase#createVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23updateVisual(IRenderContext%2CVisual%2CIEdge) Online Documentation}*/
updateVisual(context: IRenderContext, oldVisual: Visual, edge: IEdge): Visual | null;
/**
Gets the renderer implementation for this instance.
The private implementation will delegate all API calls back to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeStyleBase%23renderer Online Documentation}*/
readonly renderer: IEdgeStyleRenderer;
static $class: Class;
private $_EdgeStyleBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeStyleBase;
}
export interface LabelStyleBase extends YObject, ILabelStyle {
}
/**
An abstract base class that makes it possible to easily implement a custom {@link ILabelStyle}.
Only {@link LabelStyleBase#createVisual} and {@link LabelStyleBase#getPreferredSize} need to be implemented
by subclasses, however to improve rendering performance it is highly recommended to implement at least
{@link LabelStyleBase#updateVisual}, too.
This implementation differs from the straightforward {@link ILabelStyle} implementation in that there is no
visible separation between the style and its
{@link ILabelStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
the style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase Online Documentation}*/
export abstract class LabelStyleBase {
/**
Initializes a new instance of the {@link LabelStyleBase} class.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase Online Documentation}*/
constructor();
/**
Creates a new object that is a copy of the current instance.
Immutable subclasses should consider returning <code>this</code>.
@returns A new object that is a copy of this instance using {@link YObject#memberwiseClone}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23clone() Online Documentation}*/
clone(): this;
/**
Creates a transform matrix that can be applied to a {@link Visual} to arrange it according to the given <code>layout</code> and <code>autoFlip</code>
rule.
@param context The render context.
@param layout The layout used to create the transform matrix.
@param autoFlip Whether to automatically flip the matrix by 180 degrees, if the result would be pointing downwards.
@returns The transform matrix.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23createLayoutTransform(IRenderContext%2CIOrientedRectangle%2Cboolean) Online Documentation}*/
static createLayoutTransform(context: IRenderContext, layout: IOrientedRectangle, autoFlip: boolean): Matrix;
/**
Creates a transform matrix that can be applied to a {@link Visual} to arrange it according to the given <code>layout</code> and <code>autoFlip</code>
rule.
This method doesn't take the value of the {@link CanvasComponent#projection} property into account. If you're using a non-identity {@link CanvasComponent#projection}, use the other
overload with the {@link IRenderContext} parameter instead.
@param layout The layout used to create the transform matrix.
@param autoFlip Whether to automatically flip the matrix by 180 degrees, if the result would be pointing downwards.
@returns The transform matrix (with projection).
@deprecated Use the overload with the render context parameter instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23createLayoutTransform(IOrientedRectangle%2Cboolean) Online Documentation}*/
static createLayoutTransform(layout: IOrientedRectangle, autoFlip: boolean): Matrix;
/**
Creates the visual.
This method is called in response to a {@link IVisualCreator#createVisual} call to the instance that has been
queried from the {@link LabelStyleBase#renderer}.
@param context The render context.
@param label The label to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see LabelStyleBase#updateVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23createVisual(IRenderContext%2CILabel) Online Documentation}*/
abstract createVisual(context: IRenderContext, label: ILabel): Visual | null;
/**
Gets the bounds of the visual for the label in the given context.
This method is called in response to a {@link IBoundsProvider#getBounds} call to the instance that has been queried
from the {@link LabelStyleBase#renderer}.
This implementation simply yields the {@link IOrientedRectangle#bounds bounds} of the {@link ILabel#layout layout} of the given label.
@param context The canvas context.
@param label The label to which this style instance is assigned.
@returns The visual bounds of the visual representation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23getBounds(ICanvasContext%2CILabel) Online Documentation}*/
getBounds(context: ICanvasContext, label: ILabel): Rect;
/**
Returns the preferred {@link Size size} of the label.
@param label The label to which this style instance is assigned.
@returns The preferred size.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23getPreferredSize(ILabel) Online Documentation}*/
abstract getPreferredSize(label: ILabel): Size;
/**
Determines whether the visual representation of the label has been hit at the given location.
This method is called in response to a {@link IHitTestable#isHit} call to the instance that has been queried from
the {@link LabelStyleBase#renderer}.
This implementation returns the result of the {@link IOrientedRectangle#hits} method invoked for the {@link ILabel#layout layout} of
the label, the given {@link Point point} and the {@link ICanvasContext#hitTestRadius hit test radius} of the {@link ICanvasContext canvas context}.
@param context The input mode context.
@param location The point to test.
@param label The label to which this style instance is assigned.
@returns <code>true</code> if the specified label representation is hit; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23isHit(IInputModeContext%2CPoint%2CILabel) Online Documentation}*/
isHit(context: IInputModeContext, location: Point, label: ILabel): boolean;
/**
Determines whether the visualization for the specified label is included in the marquee selection.
This method is called in response to a {@link IMarqueeTestable#isInBox} call to the instance that has been queried
from the {@link LabelStyleBase#renderer}.
This implementation returns the result of the {@link Rect#intersects} method invoked on <code>rectangle</code> for the {@link ILabel#layout layout}
of the label and the <code>context</code>.
@param context The input mode context.
@param rectangle The marquee selection box.
@param label The label to which this style instance is assigned.
@returns <code>true</code> if the specified label is visible is selected by the marquee rectangle; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23isInBox(IInputModeContext%2CRect%2CILabel) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect, label: ILabel): boolean;
/**
Determines whether the visualization for the specified label is included in the lasso selection.
This method is called in response to a {@link ILassoTestable#isInPath} call to the instance that has been queried
from the {@link LabelStyleBase#renderer}.
@param context The input mode context.
@param path The lasso selection path.
@param label The label to which this style instance is assigned.
@returns <code>true</code> if the specified label is visible is selected by the lasso path; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23isInPath(IInputModeContext%2CGeneralPath%2CILabel) Online Documentation}*/
isInPath(context: IInputModeContext, path: GeneralPath, label: ILabel): boolean;
/**
Determines whether the visualization for the specified label is visible in the context.
This method is called in response to a {@link IVisibilityTestable#isVisible} call to the instance that has been
queried from the {@link LabelStyleBase#renderer}.
This implementation simply tests whether the {@link LabelStyleBase#getBounds bounds} intersect the clip.
@param context The canvas context.
@param rectangle The clipping rectangle.
@param label The label to which this style instance is assigned.
@returns <code>true</code> if the specified label is visible in the clipping rectangle; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23isVisible(ICanvasContext%2CRect%2CILabel) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect, label: ILabel): boolean;
/**
Performs the {@link ILookup#lookup} operation for the {@link ILabelStyleRenderer#getContext} that has been
queried from the {@link LabelStyleBase#renderer}.
This implementation yields <code>null</code> for everything but:
<ul>
<li>{@link IHitTestable}
<li>{@link IVisualCreator}
<li>{@link IBoundsProvider}
<li>{@link IVisibilityTestable}
<li>{@link IMarqueeTestable}
<li>{@link ILookup}
</ul>
For these interfaces an implementation will be returned that delegates to the methods in this instance.
@param label The label to use for the context lookup.
@param type The type to query.
@returns An implementation of the <code>type</code> or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23lookup(ILabel%2CClass) Online Documentation}*/
lookup(label: ILabel, type: Class): any | null;
/**
Updates the visual previously created by {@link LabelStyleBase#createVisual}.
This method is called in response to a {@link IVisualCreator#updateVisual} call to the instance that has been
queried from the {@link LabelStyleBase#renderer}. This implementation simply delegates to {@link LabelStyleBase#createVisual} so subclasses
should override to improve rendering performance.
@param context The render context.
@param oldVisual The visual that has been created in the call to {@link LabelStyleBase#createVisual}.
@param label The label to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see LabelStyleBase#createVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23updateVisual(IRenderContext%2CVisual%2CILabel) Online Documentation}*/
updateVisual(context: IRenderContext, oldVisual: Visual, label: ILabel): Visual | null;
/**
Gets the renderer implementation for this instance.
The private implementation will delegate all API calls back to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelStyleBase%23renderer Online Documentation}*/
readonly renderer: ILabelStyleRenderer;
static $class: Class;
private $_LabelStyleBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelStyleBase;
}
export interface NodeStyleBase extends YObject, INodeStyle {
}
/**
* An abstract base class that makes it possible to easily implement a custom {@link INodeStyle}.
* The only method that needs to be implemented by subclasses is {@link NodeStyleBase#createVisual}, however to
* improve rendering performance it is highly recommended to implement at least
* {@link NodeStyleBase#updateVisual}, too.
*
* This implementation differs from the straightforward {@link INodeStyle} implementation in that there is no visible
* separation between the style and its
* {@link INodeStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
* the style instance.
*
* A very simple subclass of {@link NodeStyleBase} only needs a {@link NodeStyleBase#createVisual}
* implementation:
*
* <pre>
* /// <summary>
* /// A custom node style that displays a node always as an exact circle inscribed in the node's
* /// layout rectangle.
* /// </summary>
*
* class CircleNodeStyle extends NodeStyleBase
* // We inherit from NodeStyleBase<Ellipse> because the Visual we return from
* // CreateVisual will be an Ellipse
* {
*   // The only method we actually *have* to implement
*
*   /**
*    * @param {IRenderContext} context
*    * @param {INode} node
*    * @return {Visual}
*    *‍/
*   createVisual(context, node) {
*     const size = Math.min(node.layout.width, node.layout.height)
*
*     // we create a circular Element
*     const circle = document.createElementNS('http://www.w3.org/2000/svg', 'ellipse')
*     circle.setAttribute('rx', size / 2)
*     circle.setAttribute('ry', size / 2)
*     circle.setAttribute('stroke', 'black')
*     circle.setAttribute('fill', 'red')
*
*     // We now need to tell yFiles where to place the circle. This is done with
*     SvgVisual.setTranslate(circle, node.layout.x, node.layout.y)
*     return new SvgVisual(circle)
*   }
* }
* </pre>
*
* <pre>
* /**
*  * @param {IRenderContext} context
*  * @param {Visual} oldVisual
*  * @param {INode} node
*  *
*  * @return {Visual}
*  *‍/
* updateVisual(context, oldVisual, node) {
*   // Our style does not have any options that can change, so the only thing left
*   // to do here is to update location and size via SetCanvasArrangeRect
*   const size = Math.min(node.layout.width, node.layout.height)
*
*   const element = oldVisual.svgElement
*
*   element.setAttribute('rx', size / 2)
*   element.setAttribute('ry', size / 2)
*   SvgVisual.setTranslate(element, node.layout.x, node.layout.y)
*
*   return oldVisual
* }
* </pre>
*
* <pre>
* /**
*  * @param {IInputModeContext} context
*  * @param {Point} location
*  * @param {INode} node
*  *‍/
* isHit(context, location, node) {
*   const size = Math.min(node.layout.width, node.layout.height)
*   const circleBounds = new Rect(node.layout.center, new Size(size, size))
*   // Also observe the hit-test radius from the context so that clicking just outside
*   // the bounds of the circle will still register as a hit.
*   // This is less important with node styles, but for edges that are usually just
*   // displayed as a line, it's very hard to hit that line otherwise.
*   return GeomUtilities.ellipseContains(circleBounds, location, context.hitTestRadius)
*
*   // In our case, since we know our visualization is always a circle, we also could have done the math
*   // ourselves:
*
*   // The point only hits the circle when the distance to the circle's center is less than
*   // its radius (plus the hit-test radius again):
*
*   // return node.layout.center.distanceTo(location) <= size / 2 + context.hitTestRadius;
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase Online Documentation}
*/
export abstract class NodeStyleBase {
/**
Initializes a new instance of the {@link NodeStyleBase} class.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase Online Documentation}*/
constructor();
/**
Creates a new object that is a copy of the current instance.
Immutable subclasses should consider returning <code>this</code>.
@returns A new object that is a copy of this instance using {@link YObject#memberwiseClone}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23clone() Online Documentation}*/
clone(): this;
/**
Creates the visual representation for <code>node</code>.
This method is called in response to a {@link IVisualCreator#createVisual} call to the instance that has been
queried from the {@link NodeStyleBase#renderer}.
@param context The render context.
@param node The node to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see NodeStyleBase#updateVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23createVisual(IRenderContext%2CINode) Online Documentation}*/
abstract createVisual(context: IRenderContext, node: INode): Visual | null;
/**
* Gets the bounds of the visual for the node in the given context.
* This method is called in response to a {@link IBoundsProvider#getBounds} call to the instance that has been queried
* from the {@link NodeStyleBase#renderer}. This implementation simply yields the {@link INode#layout}.
*
* Usually the bounds only have to be changed if the node visualization doesn't fit exactly into the node's layout
* rectangle. For example, if we know that our style exceeds the node's layout on the lower right by 3 pixels (e.g. due to
* a drop-shadow), this method's implementation could look like follows:
*
* <pre>
* /**
*  * @param {ICanvasContext} context
*  * @param {INode} node
*  *‍/
* getBounds(context, node) {
*   return node.layout.getEnlarged(new Insets(0, 0, 3, 3))
* }
* </pre>
*
* @param context The canvas context.
* @param node The node to which this style instance is assigned.
* @returns The visual bounds of the visual representation.
* @protected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23getBounds(ICanvasContext%2CINode) Online Documentation}
*/
getBounds(context: ICanvasContext, node: INode): Rect;
/**
Gets the intersection of a line with the visual representation of the node.
This method is called in response to a {@link IShapeGeometry#getIntersection} call to the instance that has been
queried from the {@link NodeStyleBase#renderer}. This implementation simply uses the {@link NodeStyleBase#getOutline outline} to determine the intersection or the {@link INode#layout} if the outline is
<code>null</code>. If it is feasible to determine the intersection point for the current shape, this method should be implemented in
addition to {@link NodeStyleBase#getOutline} to improve performance.
@param node The node to which this style instance is assigned.
@param inner The coordinates of a point lying {@link NodeStyleBase#isInside inside} the shape.
@param outer The coordinates of a point lying outside the shape.
@returns The intersection point if one has been found or <code>null</code>, otherwise.
@see NodeStyleBase#isInside
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23getIntersection(INode%2CPoint%2CPoint) Online Documentation}*/
getIntersection(node: INode, inner: Point, outer: Point): Point | null;
/**
Gets the outline of the visual style.
This implementation yields <code>null</code> to indicate that the {@link INode#layout} depicts the outline. Implementing this method influences the
behavior of {@link NodeStyleBase#isInside} and {@link NodeStyleBase#getIntersection} since the default
implementations delegate to it.
@param node The node to which this style instance is assigned.
@returns The outline of the visual representation or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23getOutline(INode) Online Documentation}*/
getOutline(node: INode): GeneralPath | null;
/**
* Determines whether the visual representation of the node has been hit at the given location.
* This method is called in response to a {@link IHitTestable#isHit} call to the instance that has been queried from
* the {@link NodeStyleBase#renderer}. This implementation uses the {@link NodeStyleBase#getOutline outline} to determine whether the node has been hit.
*
* <pre>
* /**
*  * @param {IInputModeContext} context
*  * @param {Point} location
*  * @param {INode} node
*  *‍/
* isHit(context, location, node) {
*   // size of the cross bars
*   const size = 30
*   // model the hit-test by building two overlapping rectangles for the cross
*   const center = node.layout.center
*   const rect1 = new Rect(center, new Size(node.layout.width, size))
*   const rect2 = new Rect(center, new Size(size, node.layout.height))
*
*   // if either of them contains the point, the hit-test succeeds
*   return rect1.contains(location) || rect2.contains(location)
* }
* </pre>
*
* @param context The canvas context.
* @param location The point to test.
* @param node The node to which this style instance is assigned.
* @returns <code>true</code> if the specified node representation is hit; <code>false</code> otherwise.
* @protected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23isHit(IInputModeContext%2CPoint%2CINode) Online Documentation}
*/
isHit(context: IInputModeContext, location: Point, node: INode): boolean;
/**
* Determines whether the visualization for the specified node is included in the marquee selection.
* This method is called in response to a {@link IMarqueeTestable#isInBox} call to the instance that has been queried
* from the {@link NodeStyleBase#renderer}. This implementation simply tests whether the {@link NodeStyleBase#getBounds bounds} intersect the marquee box.
*
* The following example implementation shows how marquee selection can be customized to require the node to be completely
* inside the marquee rectangle instead of only intersecting it:
*
* <pre>
* /**
*  * @param {IInputModeContext} context
*  * @param {Rect} rectangle
*  * @param {INode} node
*  *‍/
* isInBox(context, rectangle, node) {
*   const layout = node.layout
*   // Require the top-left and bottom-right corners to be inside the rectangle.
*   // That way the complete node must be inside.
*   return rectangle.contains(layout.topLeft) && rectangle.contains(layout.bottomRight)
* }
* </pre>
*
* @param context The input mode context.
* @param rectangle The marquee selection box.
* @param node The node to which this style instance is assigned.
* @returns <code>true</code> if the specified node is visible is selected by the marquee rectangle; <code>false</code> otherwise.
* @protected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23isInBox(IInputModeContext%2CRect%2CINode) Online Documentation}
*/
isInBox(context: IInputModeContext, rectangle: Rect, node: INode): boolean;
/**
Determines whether the visualization for the specified node is included in the lasso selection.
This method is called in response to a {@link ILassoTestable#isInPath} call to the instance that has been queried
from the {@link NodeStyleBase#renderer}. This implementation simply tests whether the {@link NodeStyleBase#getBounds bounds} intersect the lasso path.
@param context The input mode context.
@param path The lasso selection path.
@param node The node to which this style instance is assigned.
@returns <code>true</code> if the specified node is visible is selected by the lasso path <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23isInPath(IInputModeContext%2CGeneralPath%2CINode) Online Documentation}*/
isInPath(context: IInputModeContext, path: GeneralPath, node: INode): boolean;
/**
Determines whether the provided point is geometrically inside the visual bounds of the node.
This method is called in response to a {@link IShapeGeometry#isInside} call to the instance that has been queried
from the {@link NodeStyleBase#renderer}. This implementation simply uses the {@link NodeStyleBase#getOutline outline} to determine whether the point is contained or the {@link INode#layout} if the
outline is <code>null</code>. If it is feasible to determine whether a given point lies inside the shape, this method should be
implemented in addition to {@link NodeStyleBase#getOutline} to improve performance.
@param node The node to which this style instance is assigned.
@param location The point to test.
@returns Whether the point is considered to lie inside the shape.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23isInside(INode%2CPoint) Online Documentation}*/
isInside(node: INode, location: Point): boolean;
/**
* Determines whether the visualization for the specified node is visible in the context.
* This method is called in response to a {@link IVisibilityTestable#isVisible} call to the instance that has been
* queried from the {@link NodeStyleBase#renderer}. This implementation simply tests whether the {@link NodeStyleBase#getBounds bounds} intersect the clip.
*
* Visibility is important to consider when the node's visualization exceeds the node's layout rectangle, e.g. due to a
* drop shadow. For example, if we know that our style exceeds the node's layout on the lower right by 3 pixels, this
* method's implementation could look like follows:
*
* <pre>
* /**
*  * @param {ICanvasContext} context
*  * @param {Rect} rectangle
*  * @param {INode} node
*  *‍/
* isVisible(context, rectangle, node) {
*   const enlargedNodeLayout = node.layout.getEnlarged(new Insets(0, 0, 3, 3))
*   // The return value is simply whether any part of enlargedNodeLayout is visible in rectangle
*   return rectangle.intersects(enlargedNodeLayout)
* }
* </pre>
*
* In many cases, {@link NodeStyleBase#isVisible} is closely aligned with {@link NodeStyleBase#getBounds}, so {@link NodeStyleBase#isVisible}
* may actually be implemented by delegating to {@link NodeStyleBase#getBounds} as follows:
*
* <pre>
* /**
*  * @param {ICanvasContext} context
*  * @param {Rect} rectangle
*  * @param {INode} node
*  *‍/
* isVisible(context, rectangle, node) {
*   return this.getBounds(context, node).intersects(rectangle)
* }
* </pre>
*
* In fact, that's the default implementation for {@link NodeStyleBase}.
*
* @param context The canvas context.
* @param rectangle The clipping rectangle.
* @param node The node to which this style instance is assigned.
* @returns <code>true</code> if the specified node is visible in the clipping rectangle; <code>false</code> otherwise.
* @protected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23isVisible(ICanvasContext%2CRect%2CINode) Online Documentation}
*/
isVisible(context: ICanvasContext, rectangle: Rect, node: INode): boolean;
/**
Performs the {@link ILookup#lookup} operation for the {@link INodeStyleRenderer#getContext} that has been
queried from the {@link NodeStyleBase#renderer}.
This implementation yields <code>null</code> for everything but:
<ul>
<li>{@link IHitTestable}
<li>{@link IVisualCreator}
<li>{@link IBoundsProvider}
<li>{@link IVisibilityTestable}
<li>{@link IMarqueeTestable}
<li>{@link ILookup}
<li>{@link IShapeGeometry}
</ul>
For these interfaces an implementation will be returned that delegates to the methods in this instance.
@param node The node to use for the context lookup.
@param type The type to query.
@returns An implementation of the <code>type</code> or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23lookup(INode%2CClass) Online Documentation}*/
lookup(node: INode, type: Class): any | null;
/**
Updates a visual representation for <code>node</code> previously created by {@link NodeStyleBase#createVisual}.
This method is called in response to a {@link IVisualCreator#updateVisual} call to the instance that has been
queried from the {@link NodeStyleBase#renderer}. This implementation simply delegates to {@link NodeStyleBase#createVisual} so subclasses should
override to improve rendering performance.
@param context The render context.
@param oldVisual The visual that has been created in the call to {@link NodeStyleBase#createVisual}.
@param node The node to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see NodeStyleBase#createVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23updateVisual(IRenderContext%2CVisual%2CINode) Online Documentation}*/
updateVisual(context: IRenderContext, oldVisual: Visual, node: INode): Visual | null;
/**
Gets the renderer implementation for this instance.
The private implementation will delegate all API calls back to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleBase%23renderer Online Documentation}*/
readonly renderer: INodeStyleRenderer;
static $class: Class;
private $_NodeStyleBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeStyleBase;
}
export interface NodeStyleLabelStyleAdapter extends YObject, ILabelStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: ILabelStyleRenderer;
} /**
A label style decorator that uses a node style instance to render the background and a label style instance to render
the foreground of a label.
The renderer of the {@link NodeStyleLabelStyleAdapter#nodeStyle} used by this class must create {@link SvgVisual}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleLabelStyleAdapter Online Documentation}*/
export class NodeStyleLabelStyleAdapter {
/**
Creates a label style that uses the provided node style to render the background and the label style to render the
foreground of this style.
Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
instance.
@param [nodeStyle=null] The style to use for rendering the background of the label.
@param [labelStyle=null] The style to use for rendering the foreground of the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleLabelStyleAdapter Online Documentation}*/
constructor(nodeStyle?: INodeStyle, labelStyle?: ILabelStyle);
/**
Creates a label style that uses the provided node style to render the background and the label style to render the
foreground of this style.
Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
instance.
@param options The parameters to pass.
@param [options.nodeStyle=null] The style to use for rendering the background of the label.
@param [options.labelStyle=null] The style to use for rendering the foreground of the label.
@param [options.autoFlip] A value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented downwards, otherwise. This option sets the {@link NodeStyleLabelStyleAdapter#autoFlip} property on the created object.
@param [options.labelStyleInsets] The insets to apply for the {@link NodeStyleLabelStyleAdapter#labelStyle} as margins. This option sets the {@link NodeStyleLabelStyleAdapter#labelStyleInsets} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleLabelStyleAdapter Online Documentation}*/
constructor(options: {
nodeStyle?: INodeStyle;
labelStyle?: ILabelStyle;
autoFlip?: boolean;
labelStyleInsets?: Insets | InsetsConvertible;
});
/**
Gets the {@link INodeStyle} that is used for rendering the background of the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleLabelStyleAdapter%23nodeStyle Online Documentation}*/
nodeStyle: INodeStyle;
/**
Gets the {@link ILabelStyle} that is used for rendering the foreground of the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleLabelStyleAdapter%23labelStyle Online Documentation}*/
labelStyle: ILabelStyle;
/**
Gets a value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented
downwards, otherwise.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleLabelStyleAdapter%23autoFlip Online Documentation}*/
autoFlip: boolean;
/**
Gets or sets the insets to apply for the {@link NodeStyleLabelStyleAdapter#labelStyle} as margins.
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleLabelStyleAdapter%23labelStyleInsets Online Documentation}*/
labelStyleInsets: Insets;
static $class: Class;
private $_NodeStyleLabelStyleAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeStyleLabelStyleAdapter;
}
export interface NodeStylePortStyleAdapter extends YObject, IPortStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: IPortStyleRenderer;
} /**
A port style decorator that uses a node style instance to render the port.
The renderer of the {@link NodeStylePortStyleAdapter#nodeStyle} used by this class must create {@link SvgVisual}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStylePortStyleAdapter Online Documentation}*/
export class NodeStylePortStyleAdapter {
/**
Creates a port style that uses the provided node style to render the port.
Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
instance.
@param [nodeStyle=null] The style to use for rendering the port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStylePortStyleAdapter Online Documentation}*/
constructor(nodeStyle?: INodeStyle);
/**
Creates a port style that uses the provided node style to render the port.
Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
instance.
@param options The parameters to pass.
@param [options.nodeStyle=null] The style to use for rendering the port.
@param [options.renderSize] The size of the port's visualization. This option sets the {@link NodeStylePortStyleAdapter#renderSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStylePortStyleAdapter Online Documentation}*/
constructor(options: {
nodeStyle?: INodeStyle;
renderSize?: Size | SizeConvertible;
});
/**
Gets the {@link INodeStyle} that is used for rendering the port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStylePortStyleAdapter%23nodeStyle Online Documentation}*/
nodeStyle: INodeStyle;
/**
Gets or sets the size of the port's visualization.
Unlike nodes, ports have no intrinsic size, so a suitable size for the visualization has to be given here.
The default value is (5,5).
@default '5,5'
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStylePortStyleAdapter%23renderSize Online Documentation}*/
renderSize: Size;
/**
Gets or sets an offset for the port visualization.
This will cause the visualization to be shifted by the given amount. This can be used to have a port visualization that
is not centered on the port's location, e.g. to get a port visualization that extends outside of the node, while the
port lies on the node outline.
The default value is (0,0).
@default '0,0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStylePortStyleAdapter%23offset Online Documentation}*/
offset: Point;
static $class: Class;
private $_NodeStylePortStyleAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeStylePortStyleAdapter;
}
export interface PortStyleBase extends YObject, IPortStyle {
}
/**
An abstract base class that makes it possible to easily implement a custom {@link IPortStyle}.
Only {@link PortStyleBase#createVisual} and {@link PortStyleBase#getBounds} need to be implemented by
subclasses, however to improve rendering performance it is highly recommended to implement at least
{@link PortStyleBase#updateVisual}, too.
This implementation differs from the straightforward {@link IPortStyle} implementation in that there is no visible
separation between the style and its
{@link IPortStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
the style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase Online Documentation}*/
export abstract class PortStyleBase {
/**
Initializes a new instance of the {@link PortStyleBase} class.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase Online Documentation}*/
constructor();
/**
Creates a new object that is a copy of the current instance.
Immutable subclasses should consider returning <code>this</code>.
@returns A new object that is a copy of this instance using {@link YObject#memberwiseClone}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23clone() Online Documentation}*/
clone(): this;
/**
Creates the visual.
This method is called in response to a {@link IVisualCreator#createVisual} call to the instance that has been
queried from the {@link PortStyleBase#renderer}.
@param context The render context.
@param port The port to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see PortStyleBase#updateVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23createVisual(IRenderContext%2CIPort) Online Documentation}*/
abstract createVisual(context: IRenderContext, port: IPort): Visual | null;
/**
Returns the bounds of the visual for the port in the given context.
@param context The canvas context.
@param port The port to which this style instance is assigned.
@returns The visual bounds of the visual representation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23getBounds(ICanvasContext%2CIPort) Online Documentation}*/
abstract getBounds(context: ICanvasContext, port: IPort): Rect;
/**
Determines whether the visual representation of the port has been hit at the given location.
This method is called in response to a {@link IHitTestable#isHit} call to the instance that has been queried from
the {@link PortStyleBase#renderer}.
This implementation uses the {@link PortStyleBase#getBounds bounds} to determine whether the port has been hit.
@param context The input mode context.
@param location The point to test.
@param port The port to which this style instance is assigned.
@returns <code>true</code> if the specified port representation is hit; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23isHit(IInputModeContext%2CPoint%2CIPort) Online Documentation}*/
isHit(context: IInputModeContext, location: Point, port: IPort): boolean;
/**
Determines whether the visualization for the specified port is included in the marquee selection.
This method is called in response to a {@link IMarqueeTestable#isInBox} call to the instance that has been queried
from the {@link PortStyleBase#renderer}.
This implementation simply tests whether the {@link PortStyleBase#getBounds bounds} intersect the marquee box.
@param context The input mode context.
@param rectangle The marquee selection box.
@param port The port to which this style instance is assigned.
@returns <code>true</code> if the specified port is visible is selected by the marquee rectangle; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23isInBox(IInputModeContext%2CRect%2CIPort) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect, port: IPort): boolean;
/**
Determines whether the visualization for the specified port is included in the lasso selection.
This method is called in response to a {@link ILassoTestable#isInPath} call to the instance that has been queried
from the {@link PortStyleBase#renderer}.
This implementation simply tests whether the {@link PortStyleBase#getBounds bounds} intersect the lasso path.
@param context The input mode context.
@param path The lasso selection path.
@param port The port to which this style instance is assigned.
@returns <code>true</code> if the specified port is visible is selected by the lasso path; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23isInPath(IInputModeContext%2CGeneralPath%2CIPort) Online Documentation}*/
isInPath(context: IInputModeContext, path: GeneralPath, port: IPort): boolean;
/**
Determines whether the visualization for the specified port is visible in the context.
This method is called in response to a {@link IVisibilityTestable#isVisible} call to the instance that has been
queried from the {@link PortStyleBase#renderer}. This implementation simply tests whether the {@link PortStyleBase#getBounds bounds} intersect the clip.
@param context The canvas context.
@param rectangle The clipping rectangle.
@param port The port to which this style instance is assigned.
@returns <code>true</code> if the specified port is visible in the clipping rectangle; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23isVisible(ICanvasContext%2CRect%2CIPort) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect, port: IPort): boolean;
/**
Performs the {@link ILookup#lookup} operation for the {@link IPortStyleRenderer#getContext} that has been
queried from the {@link PortStyleBase#renderer}.
This implementation yields <code>null</code> for everything but:
<ul>
<li>{@link IHitTestable}
<li>{@link IVisualCreator}
<li>{@link IBoundsProvider}
<li>{@link IVisibilityTestable}
<li>{@link IMarqueeTestable}
<li>{@link ILookup}
</ul>
For these interfaces an implementation will be returned that delegates to the methods in this instance.
@param port The port to use for the context lookup.
@param type The type to query.
@returns An implementation of the <code>type</code> or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23lookup(IPort%2CClass) Online Documentation}*/
lookup(port: IPort, type: Class): any | null;
/**
Updates the visual previously created by {@link PortStyleBase#createVisual}.
This method is called in response to a {@link IVisualCreator#updateVisual} call to the instance that has been
queried from the {@link PortStyleBase#renderer}. This implementation simply delegates to {@link PortStyleBase#createVisual} so subclasses should
override to improve rendering performance.
@param context The render context.
@param oldVisual The visual that has been created in the call to {@link PortStyleBase#createVisual}.
@param port The port to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see PortStyleBase#createVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23updateVisual(IRenderContext%2CVisual%2CIPort) Online Documentation}*/
updateVisual(context: IRenderContext, oldVisual: Visual, port: IPort): Visual | null;
/**
Gets the renderer implementation for this instance.
The private implementation will delegate all API calls back to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyleBase%23renderer Online Documentation}*/
readonly renderer: IPortStyleRenderer;
static $class: Class;
private $_PortStyleBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortStyleBase;
}
export interface TextRenderSupport extends YObject {
}
/**
A utility class that provides {@link TextRenderSupport#measureText text size measurement} and {@link TextRenderSupport#addText text placement} functionality, for instance, for label rendering
and measurement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextRenderSupport Online Documentation}*/
export class TextRenderSupport {
/**
Add the text content to the provided SVG <code>text</code> element.
The <code>rightToLeft</code> does not render an ellipsis in Internet Explorer 11 and lower for {@link TextWrapping#CHARACTER_ELLIPSIS}
and {@link TextWrapping#WORD_ELLIPSIS} because of a browser issue that renders the ellipsis at the wrong location.
@param targetElement An SVG element to add the provided text to. When using <code>rightToLeft</code>, it is expected as <code>SVGGElement</code> otherwise as
<code>SVGTextElement</code>.
@param text The text content to add to the provided text element (may contain newline characters).
@param font The {@link Font} that defines the font properties to apply to the added text.
@param [maximumSize=null] The bounds that shouldn't be exceeded when placing the text. When omitted, the default is an unlimited size.
@param [wrapping=TextWrapping.NONE] The {@link TextWrapping} policy to apply when the text exceeds the provided <code>maximumSize</code>. When omitted, the default
is {@link TextWrapping#NONE}.
@param [measurePolicy=TextMeasurePolicy.AUTOMATIC] The measure policy to use for measuring the size of the text. When omitted, the default is
{@link TextMeasurePolicy#AUTOMATIC}.
@param [rightToLeft=false] Whether to use right-to-left text direction.
@returns The text that was actually placed in the <code>targetElement</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextRenderSupport%23addText(SVGElement%2Cstring%2CFont%2CSize%2CTextWrapping%2CTextMeasurePolicy%2Cboolean) Online Documentation}*/
static addText(targetElement: SVGElement, text: string, font: Font, maximumSize?: Size | null, wrapping?: TextWrapping | TextWrappingStringValues, measurePolicy?: TextMeasurePolicy | TextMeasurePolicyStringValues, rightToLeft?: boolean): string;
/**
Add the text content to the provided SVG <code>text</code> element.
The <code>rightToLeft</code> does not render an ellipsis in Internet Explorer 11 and lower for {@link TextWrapping#CHARACTER_ELLIPSIS}
and {@link TextWrapping#WORD_ELLIPSIS} because of a browser issue that renders the ellipsis at the wrong location.
@param options The parameters to pass.
@param options.targetElement An SVG element to add the provided text to. When using <code>rightToLeft</code>, it is expected as <code>SVGGElement</code> otherwise as
<code>SVGTextElement</code>.
@param options.text The text content to add to the provided text element (may contain newline characters).
@param options.font The {@link Font} that defines the font properties to apply to the added text.
@param [options.maximumSize=null] The bounds that shouldn't be exceeded when placing the text. When omitted, the default is an unlimited size.
@param [options.wrapping=TextWrapping.NONE] The {@link TextWrapping} policy to apply when the text exceeds the provided <code>maximumSize</code>. When omitted, the default
is {@link TextWrapping#NONE}.
@param [options.measurePolicy=TextMeasurePolicy.AUTOMATIC] The measure policy to use for measuring the size of the text. When omitted, the default is
{@link TextMeasurePolicy#AUTOMATIC}.
@param [options.rightToLeft=false] Whether to use right-to-left text direction.
@returns The text that was actually placed in the <code>targetElement</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextRenderSupport%23addText(SVGElement%2Cstring%2CFont%2CSize%2CTextWrapping%2CTextMeasurePolicy%2Cboolean) Online Documentation}*/
static addText(options: {
targetElement: SVGElement;
text: string;
font: Font;
maximumSize?: Size | null;
wrapping?: TextWrapping;
measurePolicy?: TextMeasurePolicy;
rightToLeft?: boolean;
}): string;
/**
Calculate the width and height required to render the provided text using the provided {@link Font}.
@param text The text that should be measured.
@param font The {@link Font} to apply to the text before measuring.
@param [maximumSize=null] The bounds that shouldn't be exceeded when placing the text. When omitted, the default is an unlimited size.
@param [wrapping=TextWrapping.NONE] The {@link TextWrapping} policy to apply when the text exceeds the provided <code>maximumSize</code>. When omitted, the default
is {@link TextWrapping#NONE}.
@param [measurePolicy=TextMeasurePolicy.AUTOMATIC] The measure policy to use for measuring the size of the text; {@link TextMeasurePolicy#AUTOMATIC} per default.
@returns The size of the measured text.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextRenderSupport%23measureText(string%2CFont%2CSize%2CTextWrapping%2CTextMeasurePolicy) Online Documentation}*/
static measureText(text: string, font: Font, maximumSize?: Size | null, wrapping?: TextWrapping | TextWrappingStringValues, measurePolicy?: TextMeasurePolicy | TextMeasurePolicyStringValues): Size;
/**
Calculate the width and height required to render the provided text using the provided {@link Font}.
@param options The parameters to pass.
@param options.text The text that should be measured.
@param options.font The {@link Font} to apply to the text before measuring.
@param [options.maximumSize=null] The bounds that shouldn't be exceeded when placing the text. When omitted, the default is an unlimited size.
@param [options.wrapping=TextWrapping.NONE] The {@link TextWrapping} policy to apply when the text exceeds the provided <code>maximumSize</code>. When omitted, the default
is {@link TextWrapping#NONE}.
@param [options.measurePolicy=TextMeasurePolicy.AUTOMATIC] The measure policy to use for measuring the size of the text; {@link TextMeasurePolicy#AUTOMATIC} per default.
@returns The size of the measured text.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextRenderSupport%23measureText(string%2CFont%2CSize%2CTextWrapping%2CTextMeasurePolicy) Online Documentation}*/
static measureText(options: {
text: string;
font: Font;
maximumSize?: Size | null;
wrapping?: TextWrapping;
measurePolicy?: TextMeasurePolicy;
}): Size;
static $class: Class;
private $_TextRenderSupport: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TextRenderSupport;
}
/**
Possible text measurement modes.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextMeasurePolicy Online Documentation}*/
export enum TextMeasurePolicy {
/**
The measure policy will be determined heuristically.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextMeasurePolicy%23AUTOMATIC Online Documentation}*/
AUTOMATIC,
/**
Use <a href="http://www.w3.org/TR/SVG11/types.html#__svg__SVGLocatable__getBBox" target="_blank">getBBox()</a> to measure the size of text elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextMeasurePolicy%23SVG Online Documentation}*/
SVG,
/**
Use <a href="http://www.whatwg.org/specs/web-apps/current-work/multipage/the-canvas-element.html#dom-context-2d-measuretext" target="_blank">CanvasRenderingContext2D.measureText()</a> to measure the width of text elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TextMeasurePolicy%23CANVAS Online Documentation}*/
CANVAS
}
/** Automatic conversion to {@link TextMeasurePolicy}. */
type TextMeasurePolicyStringValues = "automatic" | "svg" | "canvas";
export interface DefaultLabelStyle extends YObject, ILabelStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: ILabelStyleRenderer;
} /**
A basic label style which renders the label's {@link ILabel#text} and optionally a {@link DefaultLabelStyle#backgroundFill background} and {@link DefaultLabelStyle#backgroundStroke border}.
This style uses a {@link DefaultLabelStyleRenderer} to create its visualization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle Online Documentation}*/
export class DefaultLabelStyle {
/**
Creates a new instance using the provided renderer.
@param renderer The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23DefaultLabelStyle(ILabelStyleRenderer) Online Documentation}*/
constructor(renderer: ILabelStyleRenderer);
/**
Creates a new instance using the provided renderer.
@param options The parameters to pass.
@param options.renderer The custom renderer for this style instance.
@param [options.backgroundFill] The brush to use for the background box of the label. This option sets the {@link DefaultLabelStyle#backgroundFill} property on the created object.
@param [options.verticalTextAlignment] The vertical text alignment to use if the label is assigned more space than needed. This option sets the {@link DefaultLabelStyle#verticalTextAlignment} property on the created object.
@param [options.horizontalTextAlignment] The text alignment. This option sets the {@link DefaultLabelStyle#horizontalTextAlignment} property on the created object.
@param [options.wrapping] The text trimming. This option sets the {@link DefaultLabelStyle#wrapping} property on the created object.
@param [options.clipText] A value that determines whether text should be clipped. This option sets the {@link DefaultLabelStyle#clipText} property on the created object.
@param [options.backgroundStroke] The pen to use for the background box of the label. This option sets the {@link DefaultLabelStyle#backgroundStroke} property on the created object.
@param [options.font] The typeface to use for the label. This option sets the {@link DefaultLabelStyle#font} property on the created object.
@param [options.autoFlip] A value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented downwards, otherwise. This option sets the {@link DefaultLabelStyle#autoFlip} property on the created object.
@param [options.textFill] The brush to use for the label's text. This option sets the {@link DefaultLabelStyle#textFill} property on the created object.
@param [options.textSize] The size of the text. This option sets the {@link DefaultLabelStyle#textSize} property on the created object.
@param [options.insets] The insets for the label. This option sets the {@link DefaultLabelStyle#insets} property on the created object.
@param [options.minimumSize] The minimum preferred size allowed for the label. This option sets the {@link DefaultLabelStyle#minimumSize} property on the created object.
@param [options.maximumSize] The maximum preferred size allowed for the label. This option sets the {@link DefaultLabelStyle#maximumSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23DefaultLabelStyle(ILabelStyleRenderer) Online Documentation}*/
constructor(options: {
renderer: ILabelStyleRenderer;
backgroundFill?: Fill | null | FillConvertible;
verticalTextAlignment?: VerticalTextAlignment | VerticalTextAlignmentStringValues;
horizontalTextAlignment?: HorizontalTextAlignment | HorizontalTextAlignmentStringValues;
wrapping?: TextWrapping | TextWrappingStringValues;
clipText?: boolean;
backgroundStroke?: Stroke | null | StrokeConvertible;
font?: Font | FontConvertible;
autoFlip?: boolean;
textFill?: Fill | FillConvertible;
textSize?: number;
insets?: Insets | InsetsConvertible;
minimumSize?: Size | SizeConvertible;
maximumSize?: Size | SizeConvertible;
});
/**
Creates a new instance using {@link DefaultLabelStyleRenderer} as renderer.
@param [options] The parameters to pass.
@param [options.backgroundFill] The brush to use for the background box of the label. This option sets the {@link DefaultLabelStyle#backgroundFill} property on the created object.
@param [options.verticalTextAlignment] The vertical text alignment to use if the label is assigned more space than needed. This option sets the {@link DefaultLabelStyle#verticalTextAlignment} property on the created object.
@param [options.horizontalTextAlignment] The text alignment. This option sets the {@link DefaultLabelStyle#horizontalTextAlignment} property on the created object.
@param [options.wrapping] The text trimming. This option sets the {@link DefaultLabelStyle#wrapping} property on the created object.
@param [options.clipText] A value that determines whether text should be clipped. This option sets the {@link DefaultLabelStyle#clipText} property on the created object.
@param [options.backgroundStroke] The pen to use for the background box of the label. This option sets the {@link DefaultLabelStyle#backgroundStroke} property on the created object.
@param [options.font] The typeface to use for the label. This option sets the {@link DefaultLabelStyle#font} property on the created object.
@param [options.autoFlip] A value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented downwards, otherwise. This option sets the {@link DefaultLabelStyle#autoFlip} property on the created object.
@param [options.textFill] The brush to use for the label's text. This option sets the {@link DefaultLabelStyle#textFill} property on the created object.
@param [options.textSize] The size of the text. This option sets the {@link DefaultLabelStyle#textSize} property on the created object.
@param [options.insets] The insets for the label. This option sets the {@link DefaultLabelStyle#insets} property on the created object.
@param [options.minimumSize] The minimum preferred size allowed for the label. This option sets the {@link DefaultLabelStyle#minimumSize} property on the created object.
@param [options.maximumSize] The maximum preferred size allowed for the label. This option sets the {@link DefaultLabelStyle#maximumSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23DefaultLabelStyle() Online Documentation}*/
constructor(options?: {
backgroundFill?: Fill | null | FillConvertible;
verticalTextAlignment?: VerticalTextAlignment | VerticalTextAlignmentStringValues;
horizontalTextAlignment?: HorizontalTextAlignment | HorizontalTextAlignmentStringValues;
wrapping?: TextWrapping | TextWrappingStringValues;
clipText?: boolean;
backgroundStroke?: Stroke | null | StrokeConvertible;
font?: Font | FontConvertible;
autoFlip?: boolean;
textFill?: Fill | FillConvertible;
textSize?: number;
insets?: Insets | InsetsConvertible;
minimumSize?: Size | SizeConvertible;
maximumSize?: Size | SizeConvertible;
});
/**
Gets or sets the brush to use for the background box of the label.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23backgroundFill Online Documentation}*/
backgroundFill: Fill | null;
/**
Gets or sets the vertical text alignment to use if the label is assigned more space than needed.
The default value is {@link VerticalTextAlignment#TOP}.
@default VerticalTextAlignment.TOP
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23verticalTextAlignment Online Documentation}*/
verticalTextAlignment: VerticalTextAlignment;
/**
Gets or sets the text alignment.
The default value is {@link HorizontalTextAlignment#LEFT}.
@default HorizontalTextAlignment.LEFT
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23horizontalTextAlignment Online Documentation}*/
horizontalTextAlignment: HorizontalTextAlignment;
/**
Gets or sets the text trimming.
The default value is {@link TextWrapping#NONE}.
@default TextWrapping.NONE
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23wrapping Online Documentation}*/
wrapping: TextWrapping;
/**
Gets or sets a value that determines whether text should be clipped.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23clipText Online Documentation}*/
clipText: boolean;
/**
Gets or sets the pen to use for the background box of the label.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23backgroundStroke Online Documentation}*/
backgroundStroke: Stroke | null;
/**
Gets or sets the typeface to use for the label.
The default value is an instance of {@link DefaultLabelStyle#font} for <code>Arial</code>".
@throws {Exception} <b>(ArgumentError)</b> value cannot be null!
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23font Online Documentation}*/
font: Font;
/**
Gets or sets a value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented
downwards, otherwise.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23autoFlip Online Documentation}*/
autoFlip: boolean;
/**
Gets or sets the brush to use for the label's text.
The default value is {@link Fill#BLACK}.
@throws {Exception} <b>(ArgumentError)</b> value cannot be null!
@default 'Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23textFill Online Documentation}*/
textFill: Fill;
/**
Gets or sets the size of the text.
This property delegates to the size of the {@link DefaultLabelStyle#font} property.
The default value is <code>12.0d</code>.
@default 12
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23textSize Online Documentation}*/
textSize: number;
/**
Gets or sets the insets for the label.
The insets define the distance between the label border and the label content.
The default value is {@link Insets#EMPTY}.
@see DefaultLabelStyle#minimumSize
@see DefaultLabelStyle#maximumSize
@default 'Empty'
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23insets Online Documentation}*/
insets: Insets;
/**
Gets or sets the minimum preferred size allowed for the label.
This property only changes the preferred size calculated by {@link DefaultLabelStyleRenderer#getPreferredSize} and
thus has no effect for {@link ILabelModel}s that ignore the preferred size, such as
{@link InteriorStretchLabelModel}.
The default value is {@link Size#ZERO}.
@see DefaultLabelStyle#maximumSize
@see DefaultLabelStyle#insets
@default 'Zero'
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23minimumSize Online Documentation}*/
minimumSize: Size;
/**
Gets or sets the maximum preferred size allowed for the label.
If {@link DefaultLabelStyle#wrapping wrapping} is enabled and the text exceeds the maximum width allowed, then the text is wrapped.
This property only changes the preferred size calculated by {@link DefaultLabelStyleRenderer#getPreferredSize} and
thus has no effect for {@link ILabelModel}s that ignore the preferred size, such as
{@link InteriorStretchLabelModel}.
The default value is {@link Size#INFINITE}.
@see DefaultLabelStyle#minimumSize
@see DefaultLabelStyle#insets
@default 'Infinite'
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23maximumSize Online Documentation}*/
maximumSize: Size;
/**
Gets or sets a value that is added to the preferred width and height of the label.
When storing a graph on one platform and loading it on another one with a slightly different font rendering
implementation or font, it often happens that text gets truncated because the preferred size is just one or two pixels
too small on that platform. This can be avoided by slightly enlarging the labels on the platform where the graph is
stored.
Values should be <code>>=0d</code>.
The default value is <code>0d</code>.
@default 0
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23enlargementAbsolute Online Documentation}*/
static enlargementAbsolute: number;
/**
Gets or sets a value that multiplies the preferred width and height of the label.
When storing a graph on one platform and loading it on another one with a slightly different font rendering
implementation or font, it often happens that text gets truncated because the preferredSize is just one or two pixels
too small on that platform. This can be avoided by slightly enlarging the labels on the platform where the graph is
stored.
Values should be <code>>=1d</code>.
The default value is <code>1d</code>.
@default 1
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyle%23enlargementRelative Online Documentation}*/
static enlargementRelative: number;
static $class: Class;
private $_DefaultLabelStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultLabelStyle;
}
export interface IconLabelStyle extends YObject, ILabelStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: ILabelStyleRenderer;
} /**
A label style that draws an icon in addition to the text.
This style draws the icon at the location specified by {@link IconLabelStyle#iconPlacement} and delegates the drawing of the label's text to the {@link IconLabelStyle#wrapped} style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyle Online Documentation}*/
export class IconLabelStyle {
/**
Creates a new instance using the provided icon and renderer.
@param [icon=null] The icon to draw.
@param [renderer=null] The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyle Online Documentation}*/
constructor(icon?: string | null, renderer?: ILabelStyleRenderer | null);
/**
Creates a new instance using the provided icon and renderer.
@param options The parameters to pass.
@param [options.icon=null] The icon to draw.
@param [options.renderer=null] The custom renderer for this style instance.
@param [options.iconSize] The size of the icon to paint for the label. This option sets the {@link IconLabelStyle#iconSize} property on the created object.
@param [options.iconPlacement] The parameter that specifies the icon placement. This option sets the {@link IconLabelStyle#iconPlacement} property on the created object.
@param [options.wrapped] The style that renders the text of the label. This option sets the {@link IconLabelStyle#wrapped} property on the created object.
@param [options.wrappedInsets] The insets that should be applied for the {@link ILabel#layout layout} of the {@link IconLabelStyle#wrapped wrapped style} rendering. This option sets the {@link IconLabelStyle#wrappedInsets} property on the created object.
@param [options.autoFlip] A value indicating whether to automatically flip the rendering should the {@link ILabel#layout layout}'s {@link IOrientedRectangle#upY up vector} point downwards. This option sets the {@link IconLabelStyle#autoFlip} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyle Online Documentation}*/
constructor(options: {
icon?: string | null;
renderer?: ILabelStyleRenderer | null;
iconSize?: Size | SizeConvertible;
iconPlacement?: ILabelModelParameter;
wrapped?: ILabelStyle;
wrappedInsets?: Insets | InsetsConvertible;
autoFlip?: boolean;
});
/**
Gets or sets the URL of the icon.
The URL can either be a relative URL, an absolute URL, or a Data URI. Relative URLs are resolved relative to the
document. Eventually, this string becomes the value of the href attribute of an SVG image element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyle%23icon Online Documentation}*/
icon: string | null;
/**
Gets or sets the size of the icon to paint for the label.
The default value is {@link Size#EMPTY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyle%23iconSize Online Documentation}*/
iconSize: Size;
/**
Gets or sets the parameter that specifies the icon placement.
All valid node label model parameters can be used. The label's {@link ILabel#layout layout} will be interpreted as the a node's layout and the
icon will be placed relative to that layout as if it was a node's label.
The default value is {@link ExteriorLabelModel#WEST}.
@throws {Exception} <b>(ArgumentError)</b> <code>value</code> is null.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyle%23iconPlacement Online Documentation}*/
iconPlacement: ILabelModelParameter;
/**
Gets or sets the style that renders the text of the label.
This style will be rendered before the icon and is responsible for rendering the {@link ILabel#text}, since {@link IconLabelStyleRenderer}
will only render the {@link IconLabelStyle#icon}.
The default value is {@link DefaultLabelStyle}.
@throws {Exception} <b>(ArgumentError)</b> <code>value</code> is null.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyle%23wrapped Online Documentation}*/
wrapped: ILabelStyle;
/**
Gets or sets the insets that should be applied for the {@link ILabel#layout layout} of the {@link IconLabelStyle#wrapped wrapped style} rendering.
The {@link IconLabelStyleRenderer#getPreferredSize} will take the preferred size of the {@link IconLabelStyle#wrapped}'s {@link ILabelStyleRenderer}
and add these insets to it. During the {@link IVisualCreator rendering} these insets will be used to offset the rendering of the wrapped
style.
The default insets are empty.
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyle%23wrappedInsets Online Documentation}*/
wrappedInsets: Insets;
/**
Gets a value indicating whether to automatically flip the rendering should the {@link ILabel#layout layout}'s {@link IOrientedRectangle#upY up vector} point downwards.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyle%23autoFlip Online Documentation}*/
autoFlip: boolean;
static $class: Class;
private $_IconLabelStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IconLabelStyle;
}
export interface DefaultLabelStyleRenderer extends YObject, ILabelStyleRenderer, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
getPreferredSize(label: ILabel, style: ILabelStyle): Size;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
A simple {@link ILabelStyleRenderer} implementation that can handle {@link DefaultLabelStyle} instances.
This implementation will just draw the label's {@link ILabel#text} using the properties provided by {@link DefaultLabelStyle} into
the label's {@link ILabel#layout layout} area.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer Online Documentation}*/
export class DefaultLabelStyleRenderer {
/**
Creates a new default instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer Online Documentation}*/
constructor();
/**
Add the text content to the provided SVG text element.
@param textContainer An SVG element to add the provided text to. It is a <code>SVGGElement</code> when using {@link CanvasComponent#rightToLeft}, otherwise a <code>SVGTextElement</code>.
@param font The {@link Font} that defines the font properties to apply to the added text.
@param text The text content to add to the provided text element (may contain newline characters).
@param maxSize The bounds that shouldn't be exceeded when placing the text; <code>null</code> or not provided means unbound.
@param wrapping The {@link TextWrapping} policy to apply when the text exceeds the provided <code>maxSize</code>.
@param rightToLeft Whether to use right-to-left text direction.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23addTextElements(SVGElement%2CFont%2Cstring%2CSize%2CTextWrapping%2Cboolean) Online Documentation}*/
addTextElements(textContainer: SVGElement, font: Font, text: string, maxSize: Size, wrapping: TextWrapping | TextWrappingStringValues, rightToLeft: boolean): string;
/**
Prepares this instance for subsequent calls after the style and label have been initialized.
Upon invocation the {@link DefaultLabelStyleRenderer#style} and {@link DefaultLabelStyleRenderer#label} properties have been populated by the
{@link DefaultLabelStyleRenderer#getVisualCreator}, {@link DefaultLabelStyleRenderer#getBoundsProvider},
{@link DefaultLabelStyleRenderer#getHitTestable}, or {@link DefaultLabelStyleRenderer#getMarqueeTestable}
methods.
Stores the {@link ILabel#layout layout} into {@link DefaultLabelStyleRenderer#layout}. Subclasses which to override this method should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Configures the <code>style</code> and <code>label</code> parameters, calls {@link DefaultLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to retrieve the bounds provider for. The value will be stored in the {@link DefaultLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link DefaultLabelStyleRenderer#style} property.
@returns <code>this</code>
@see DefaultLabelStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getBoundsProvider(ILabel%2CILabelStyle) Online Documentation}*/
getBoundsProvider(label: ILabel, style: ILabelStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>label</code> parameters, does <b>not</b> call {@link DefaultLabelStyleRenderer#configure} and returns
<code>this</code>.
As this method may be called often it will not automatically call {@link DefaultLabelStyleRenderer#configure},
instead subclasses should ensure that in the {@link DefaultLabelStyleRenderer#lookup} method call they should call
{@link DefaultLabelStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an instance that
depends on a correctly configured <code>this</code>.
@param label The label to query the context for. The value will be stored in the {@link DefaultLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link DefaultLabelStyleRenderer#style} property.
@returns <code>this</code>
@see DefaultLabelStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getContext(ILabel%2CILabelStyle) Online Documentation}*/
getContext(label: ILabel, style: ILabelStyle): ILookup;
/**
Configures the <code>style</code> and <code>label</code> parameters, calls {@link DefaultLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to query hit test with. The value will be stored in the {@link DefaultLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link DefaultLabelStyleRenderer#style} property.
@returns <code>this</code>
@see DefaultLabelStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getHitTestable(ILabel%2CILabelStyle) Online Documentation}*/
getHitTestable(label: ILabel, style: ILabelStyle): IHitTestable;
/**
Determines how the text should be aligned within the assigned label bounds.
@returns This value as obtained from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getHorizontalTextAlignment() Online Documentation}*/
getHorizontalTextAlignment(): HorizontalTextAlignment;
/**
Configures the <code>style</code> and <code>label</code> parameters, calls {@link DefaultLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to query marquee intersection tests. The value will be stored in the {@link DefaultLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link DefaultLabelStyleRenderer#style} property.
@returns <code>this</code>
@see DefaultLabelStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getMarqueeTestable(ILabel%2CILabelStyle) Online Documentation}*/
getMarqueeTestable(label: ILabel, style: ILabelStyle): IMarqueeTestable;
/**
Calculates the preferred size given the current state of the renderer.
The size that is needed by the text is enlarged by a {@link DefaultLabelStyle#enlargementRelative relative} and an {@link DefaultLabelStyle#enlargementAbsolute absolute} value.
The implementation then adds the {@link DefaultLabelStyle#insets} and adjusts the size to respect the {@link DefaultLabelStyle#minimumSize minimum} and {@link DefaultLabelStyle#maximumSize maximum} allowed size.
At the end of the calculation, the size is {@link DefaultLabelStyleRenderer#quantizePreferredSize quantized} to avoid unnecessary floating point precision.
@returns The size as suggested by this renderer.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getPreferredSizeCore() Online Documentation}*/
getPreferredSizeCore(): Size;
/**
Determines how the text should be aligned vertically within the assigned label bounds.
@returns This value as obtained from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getVerticalTextAlignment() Online Documentation}*/
getVerticalTextAlignment(): VerticalTextAlignment;
/**
Configures the <code>style</code> and <code>label</code> parameters, does <b>not</b> call {@link DefaultLabelStyleRenderer#configure} and returns
<code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link DefaultLabelStyleRenderer#configure}. If
the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link DefaultLabelStyleRenderer#isVisible}.
@param label The label to query visibility tests. The value will be stored in the {@link DefaultLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link DefaultLabelStyleRenderer#style} property.
@returns <code>this</code>
@see DefaultLabelStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getVisibilityTestable(ILabel%2CILabelStyle) Online Documentation}*/
getVisibilityTestable(label: ILabel, style: ILabelStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>label</code> parameters, calls {@link DefaultLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link DefaultLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link DefaultLabelStyleRenderer#style} property.
@returns <code>this</code>
@see DefaultLabelStyleRenderer#createVisual
@see DefaultLabelStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getVisualCreator(ILabel%2CILabelStyle) Online Documentation}*/
getVisualCreator(label: ILabel, style: ILabelStyle): IVisualCreator;
/**
Determines how the text should be trimmed to fit the assigned label bounds.
@returns This value as obtained from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23getWrapping() Online Documentation}*/
getWrapping(): TextWrapping;
/**
Delegates to {@link DefaultLabelStyle#autoFlip}
@returns Whether to flip the drawing if it is upside down.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23isAutoFlip() Online Documentation}*/
isAutoFlip(): boolean;
/**
Determines whether text should be forced to be clipped inside the rectangle.
@returns This value as obtained from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23isClippingText() Online Documentation}*/
isClippingText(): boolean;
/**
Uses the {@link ILabel#layout} to determine whether the clip intersects.
@param context
@param rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Implements the {@link ILookup} interface.
This method will be used by default if {@link DefaultLabelStyleRenderer#getContext} has been queried for a lookup
implementation. Note that it cannot be assumed that {@link DefaultLabelStyleRenderer#configure} has already been
invoked. However, normally {@link DefaultLabelStyleRenderer#label} and {@link DefaultLabelStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link DefaultLabelStyleRenderer#configure}
on success and return <code>this</code>.
@param type The type to query for.
@returns An implementation or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Calculate the width and height required to render the provided text using the provided {@link Font}.
@param text The text that should be measured.
@param font The {@link Font} to apply to the text before measuring.
@param maximumSize The maximum available size for the text.
@returns The text size.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23measureText(string%2CFont%2CSize) Online Documentation}*/
measureText(text: string, font: Font, maximumSize: Size): Size;
/**
Quantizes the result of the preferred size calculation.
The benefit of quantization is that coordinates are easier to recognize in a debugger, labels end up with sizes that are
easier to read, GraphML files become smaller because less space is wasted on unnecessary floating point precision, and
numerical instability in certain cases is eliminated.
The current implementation just {@link Math#ceil rounds up} width and height to integral values.
@param size The preferred size
@returns The quantized preferred size.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23quantizePreferredSize(Size) Online Documentation}*/
quantizePreferredSize(size: Size): Size;
/**
Gets or sets the current {@link ILabel#layout layout}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23layout Online Documentation}*/
layout: IOrientedRectangle;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23style Online Documentation}*/
style: DefaultLabelStyle;
/**
Gets or sets the currently configured label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLabelStyleRenderer%23label Online Documentation}*/
label: ILabel;
static $class: Class;
private $_DefaultLabelStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultLabelStyleRenderer;
}
export interface IconLabelStyleRenderer extends YObject, ILabelStyleRenderer, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, ILassoTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
getPreferredSize(label: ILabel, style: ILabelStyle): Size;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
isInPath(context: IInputModeContext, lassoPath: GeneralPath): boolean;
/**
@inheritdoc*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
An {@link ILabelStyleRenderer} that can handle {@link IconLabelStyle} instances.
This implementation renders the label's inner style and an icon.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer Online Documentation}*/
export class IconLabelStyleRenderer {
/**
Creates a new instance of this renderer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer Online Documentation}*/
constructor();
/**
Prepares this instance for subsequent calls after the style and label have been initialized.
Upon invocation the {@link IconLabelStyleRenderer#style} and {@link IconLabelStyleRenderer#label} properties have been populated by the {@link IconLabelStyleRenderer#getVisualCreator},
{@link IconLabelStyleRenderer#getBoundsProvider}, {@link IconLabelStyleRenderer#getHitTestable}, or {@link IconLabelStyleRenderer#getMarqueeTestable}
methods.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Configures the <code>style</code> and <code>label</code> parameters, calls {@link IconLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to retrieve the bounds provider for. The value will be stored in the {@link IconLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link IconLabelStyleRenderer#style} property.
@returns <code>this</code>
@see IconLabelStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getBoundsProvider(ILabel%2CILabelStyle) Online Documentation}*/
getBoundsProvider(label: ILabel, style: ILabelStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>label</code> parameters, does <b>not</b> call {@link IconLabelStyleRenderer#configure} and returns <code>this</code>.
As this method may be called often it will not automatically call {@link IconLabelStyleRenderer#configure},
instead subclasses should ensure that in the {@link IconLabelStyleRenderer#lookup} method call they should call {@link IconLabelStyleRenderer#configure}
only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
@param label The label to query the context for. The value will be stored in the {@link IconLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link IconLabelStyleRenderer#style} property.
@returns <code>this</code>
@see IconLabelStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getContext(ILabel%2CILabelStyle) Online Documentation}*/
getContext(label: ILabel, style: ILabelStyle): ILookup;
/**
Configures the <code>style</code> and <code>label</code> parameters, calls {@link IconLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to query hit test with. The value will be stored in the {@link IconLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link IconLabelStyleRenderer#style} property.
@returns <code>this</code>
@see IconLabelStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getHitTestable(ILabel%2CILabelStyle) Online Documentation}*/
getHitTestable(label: ILabel, style: ILabelStyle): IHitTestable;
/**
Retrieves the {@link IconLabelStyle#icon} from the {@link IconLabelStyleRenderer#style}.
@returns The icon.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getIcon() Online Documentation}*/
getIcon(): string | null;
/**
Retrieves the {@link IconLabelStyle#iconPlacement} from the {@link IconLabelStyleRenderer#style}.
@returns The placement parameter.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getIconPlacement() Online Documentation}*/
getIconPlacement(): ILabelModelParameter;
/**
Gets the size of the icon from the {@link IconLabelStyle#iconSize} property.
@returns The size to use for the icon.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getIconSize() Online Documentation}*/
getIconSize(): Size;
/**
Configures the <code>style</code> and <code>label</code> parameters, calls {@link IconLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to query marquee intersection tests. The value will be stored in the {@link IconLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link IconLabelStyleRenderer#style} property.
@returns <code>this</code>
@see IconLabelStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getMarqueeTestable(ILabel%2CILabelStyle) Online Documentation}*/
getMarqueeTestable(label: ILabel, style: ILabelStyle): IMarqueeTestable;
/**
Calculates the preferred size given the current state of the renderer.
@returns The size as suggested by this renderer.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getPreferredSizeCore() Online Documentation}*/
getPreferredSizeCore(): Size;
/**
Configures the <code>style</code> and <code>label</code> parameters, does <b>not</b> call {@link IconLabelStyleRenderer#configure} and returns <code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link IconLabelStyleRenderer#configure}. If the
subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link IconLabelStyleRenderer#isVisible}.
@param label The label to query visibility tests. The value will be stored in the {@link IconLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link IconLabelStyleRenderer#style} property.
@returns <code>this</code>
@see IconLabelStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getVisibilityTestable(ILabel%2CILabelStyle) Online Documentation}*/
getVisibilityTestable(label: ILabel, style: ILabelStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>label</code> parameters, calls {@link IconLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link IconLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link IconLabelStyleRenderer#style} property.
@returns <code>this</code>
@see IconLabelStyleRenderer#createVisual
@see IconLabelStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getVisualCreator(ILabel%2CILabelStyle) Online Documentation}*/
getVisualCreator(label: ILabel, style: ILabelStyle): IVisualCreator;
/**
Retrieves the {@link IconLabelStyle#wrapped} from the {@link IconLabelStyleRenderer#style}.
@returns The style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getWrappedStyle() Online Documentation}*/
getWrappedStyle(): ILabelStyle;
/**
Retrieves the {@link IconLabelStyle#wrappedInsets} from the {@link IconLabelStyleRenderer#style}.
@returns The insets.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23getWrappedStyleInsets() Online Documentation}*/
getWrappedStyleInsets(): Insets;
/**
Retrieves the {@link IconLabelStyle#autoFlip} property from the {@link IconLabelStyleRenderer#style}.
@returns The autoflip property value.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23isAutoFlip() Online Documentation}*/
isAutoFlip(): boolean;
/**
Implements the {@link ILookup} interface.
This method will be used by default if {@link IconLabelStyleRenderer#getContext} has been queried for a lookup
implementation. Note that it cannot be assumed that {@link IconLabelStyleRenderer#configure} has already been
invoked. However, normally {@link IconLabelStyleRenderer#label} and {@link IconLabelStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link IconLabelStyleRenderer#configure}
on success and return <code>this</code>.
@param type The type to query for.
@returns An implementation or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Returns whether the icon bounds should be considered for hit testing.
@returns true
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23shouldHitTestIcon() Online Documentation}*/
shouldHitTestIcon(): boolean;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23style Online Documentation}*/
style: IconLabelStyle;
/**
Gets or sets the currently configured label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IconLabelStyleRenderer%23label Online Documentation}*/
label: ILabel;
static $class: Class;
private $_IconLabelStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IconLabelStyleRenderer;
}
export interface GeneralPathNodeStyle extends YObject, INodeStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
A node style that can display node shapes with a boundary that is defined by a {@link GeneralPath} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyle Online Documentation}*/
export class GeneralPathNodeStyle {
/**
Creates a new instance using the specified path and a custom renderer instance.
@param [path=null] The path that defines the boundary of this style. The path must be defined in such a way that the point (0,0) represents
the top left of a node's {@link INode#layout} and the point (1,1) the lower right. The path will be scaled accordingly to the node's size.
If not specified, a simple rectangular path is used.
@param [renderer=null] Custom renderer instance for this style. If <code>null</code> a new instance of the {@link GeneralPathNodeStyleRenderer default implementation} will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyle Online Documentation}*/
constructor(path?: GeneralPath | null, renderer?: GeneralPathNodeStyleRenderer | null);
/**
Creates a new instance using the specified path and a custom renderer instance.
@param options The parameters to pass.
@param [options.path=null] The path that defines the boundary of this style. The path must be defined in such a way that the point (0,0) represents
the top left of a node's {@link INode#layout} and the point (1,1) the lower right. The path will be scaled accordingly to the node's size.
If not specified, a simple rectangular path is used.
@param [options.renderer=null] Custom renderer instance for this style. If <code>null</code> a new instance of the {@link GeneralPathNodeStyleRenderer default implementation} will be used.
@param [options.fill] The {@link Fill} that is used to draw the shape. This option sets the {@link GeneralPathNodeStyle#fill} property on the created object.
@param [options.stroke] The {@link Stroke} that is used to draw the shape. This option sets the {@link GeneralPathNodeStyle#stroke} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyle Online Documentation}*/
constructor(options: {
path?: GeneralPath | null;
renderer?: GeneralPathNodeStyleRenderer | null;
fill?: Fill | null | FillConvertible;
stroke?: Stroke | null | StrokeConvertible;
});
/**
Get or sets the path that defines the boundary of the shape.
The path must be defined in such a way that the point (0,0) represents the top left of a node's {@link INode#layout} and the point (1,1) the
lower right. The path will be scaled accordingly to the node's size.
The default is a simple rectangular path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyle%23path Online Documentation}*/
path: GeneralPath | null;
/**
Gets or sets the {@link Fill} that is used to draw the shape.
The default value is {@link Fill#WHITE}.
@default 'System.Drawing.Brushes.White'
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyle%23fill Online Documentation}*/
fill: Fill | null;
/**
Gets or sets the {@link Stroke} that is used to draw the shape.
The default value is {@link Stroke#BLACK}.
@default 'System.Drawing.Pens.Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyle%23stroke Online Documentation}*/
stroke: Stroke | null;
static $class: Class;
private $_GeneralPathNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GeneralPathNodeStyle;
}
export interface GeneralPathNodeStyleRenderer extends YObject, INodeStyleRenderer, IShapeGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, ILassoTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
@inheritdoc*/
getOutline(): GeneralPath | null;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
isInPath(context: IInputModeContext, lassoPath: GeneralPath): boolean;
/**
@inheritdoc*/
isInside(location: Point): boolean;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
An {@link INodeStyleRenderer} implementation that can handle {@link GeneralPathNodeStyle} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer Online Documentation}*/
export class GeneralPathNodeStyleRenderer {
/**
Initializes a new instance of the {@link GeneralPathNodeStyleRenderer} class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer Online Documentation}*/
constructor();
/**
Prepares this instance for subsequent calls after the style and node have been initialized.
Upon invocation the {@link GeneralPathNodeStyleRenderer#style} and {@link GeneralPathNodeStyleRenderer#node} properties have been populated by the
{@link GeneralPathNodeStyleRenderer#getVisualCreator},
{@link GeneralPathNodeStyleRenderer#getBoundsProvider}, {@link GeneralPathNodeStyleRenderer#getHitTestable},
or {@link GeneralPathNodeStyleRenderer#getMarqueeTestable} methods.
Subclasses which override this method must call the base method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the bounds provider for. The value will be stored in the {@link GeneralPathNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link GeneralPathNodeStyleRenderer#style} property.
@returns <code>this</code>
@see GeneralPathNodeStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link GeneralPathNodeStyleRenderer#configure} and returns
<code>this</code>.
As this method may be called often it will not automatically call {@link GeneralPathNodeStyleRenderer#configure},
instead subclasses should ensure that in the {@link GeneralPathNodeStyleRenderer#lookup} method call they should
call {@link GeneralPathNodeStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an
instance that depends on a correctly configured <code>this</code>.
@param node The node to query the context for. The value will be stored in the {@link GeneralPathNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link GeneralPathNodeStyleRenderer#style} property.
@returns <code>this</code>
@see GeneralPathNodeStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query hit test with. The value will be stored in the {@link GeneralPathNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link GeneralPathNodeStyleRenderer#style} property.
@returns <code>this</code>
@see GeneralPathNodeStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query marquee intersection tests. The value will be stored in the {@link GeneralPathNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link GeneralPathNodeStyleRenderer#style} property.
@returns <code>this</code>
@see GeneralPathNodeStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
Configures the {@link GeneralPathNodeStyleRenderer#style} and {@link GeneralPathNodeStyleRenderer#node} properties, calls {@link GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the shape geometry for. The value will be stored in the {@link GeneralPathNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link GeneralPathNodeStyleRenderer#style} property.
@returns <code>this</code>
@see GeneralPathNodeStyleRenderer#isInside
@see GeneralPathNodeStyleRenderer#getIntersection
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link GeneralPathNodeStyleRenderer#configure} and returns
<code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link GeneralPathNodeStyleRenderer#configure}. If
the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link GeneralPathNodeStyleRenderer#isVisible}.
@param node The node to query visibility tests. The value will be stored in the {@link GeneralPathNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link GeneralPathNodeStyleRenderer#style} property.
@returns <code>this</code>
@see GeneralPathNodeStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link GeneralPathNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link GeneralPathNodeStyleRenderer#style} property.
@returns <code>this</code>
@see GeneralPathNodeStyleRenderer#createVisual
@see GeneralPathNodeStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
Determines visibility by checking for an intersection with the {@link GeneralPathNodeStyleRenderer#layout}.
@param context The context.
@param rectangle The current clip
@returns Whether the {@link GeneralPathNodeStyleRenderer#layout} is visible in the clip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Implements the {@link ILookup} interface.
This method will be used by default if {@link GeneralPathNodeStyleRenderer#getContext} has been queried for a
lookup implementation. Note that it cannot be assumed that {@link GeneralPathNodeStyleRenderer#configure} has
already been invoked. However, normally {@link GeneralPathNodeStyleRenderer#node} and {@link GeneralPathNodeStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>.
Subclass implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful
lookup. This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link GeneralPathNodeStyleRenderer#configure}
on success and return <code>this</code>.
@param type The type to query for.
@returns An implementation or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23style Online Documentation}*/
style: GeneralPathNodeStyle | null;
/**
Gets or sets the currently configured node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23node Online Documentation}*/
node: INode | null;
/**
Gets the {@link INode#layout} of the node this renderer is currently configured for.
Returns <code>Node.Layout</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GeneralPathNodeStyleRenderer%23layout Online Documentation}*/
readonly layout: IRectangle;
static $class: Class;
private $_GeneralPathNodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GeneralPathNodeStyleRenderer;
}
export interface MarkupLabelStyle extends DefaultLabelStyle {
}
/**
* A label style that interprets the label's {@link ILabel#text} as markup and renders the label using rich text features.
* This style supports a certain subset of HTML, CSS styling, and additionally some color tags. The supported markup
* elements are:
*
* <ul>
* <li><b>&lt;span></b> – A stylable element.</li>
* <li><b>&lt;p></b> – A paragraph element.</li>
* <li><b>&lt;pre></b> – A container for pre-formatted text.</li>
* <li><b>&lt;br></b> – Adds a line break</li>
* <li><b>&lt;em></b> – Italic text</li>
* <li><b>&lt;strong></b> – Bold text</li>
* <li><b>&lt;del></b> – Strikethrough text</li>
* <li><b>&lt;u></b> – Underline text</li>
* <li><b>&lt;h1>–&lt;h6></b> – Heading 1—6</li>
* <li><b>&lt;code></b> – Monospace text</li>
* <li><b>&lt;small></b> – Small font-size</li>
* <li><b>&lt;large></b> – Large font-size</li>
* <li><b>&lt;sub></b> – Subscript text</li>
* <li><b>&lt;sup></b> – Superscript text</li>
* </ul>
* Please note that &lt;sub> and &lt;sup> may not be supported by all browsers.
*
* To add line breaks, use the &lt;br> tag. Newline characters in the label text are ignored.
*
* Additionally, HTML color names are supported as tag names, e.g. &lt;red> produces red text:
*
* <pre>
* <red>Red text</red>
* </pre>
*
* Tags can be nested. For example, a span can contain an &lt;em> tag.
*
* <pre>
* <span>Text with <em>italic</em> part.</span>
* </pre>
*
* Tags can be styled with CSS. Please note that this style transforms all elements in the label text into styled SVG
* &lt;tspan> elements. Therefore, your CSS style has to apply to SVG text. This example shows how to create semi-transparent
* text using CSS styling.
*
* <pre>
* <span style="opacity: 0.5;">Semi-transparent text</span>
* </pre>
*
* Please make sure that your CSS does not change the font size, since text height changed by CSS is not considered in the
* text measuring. To change the overall font size, use the {@link DefaultLabelStyle#font} property.
*
* Also, tags can be given CSS classes. All tags already expose classes that can be used to style all occurrences of these
* elements.
*
* <ul>
* <li><b>&lt;span></b> – yfiles-label-span</li>
* <li><b>&lt;p></b> – yfiles-label-p</li>
* <li><b>&lt;pre></b> – yfiles-label-pre</li>
* <li><b>&lt;em></b> – yfiles-label-em</li>
* <li><b>&lt;strong></b> – yfiles-label-strong</li>
* <li><b>&lt;del></b> – yfiles-label-del</li>
* <li><b>&lt;u></b> – yfiles-label-u</li>
* <li><b>&lt;h1>-&lt;h6></b> – yfiles-label-h[1-6]</li>
* <li><b>&lt;code></b> – yfiles-label-code</li>
* <li><b>&lt;small></b> – yfiles-label-small</li>
* <li><b>&lt;large></b> – yfiles-label-large</li>
* <li><b>&lt;sub></b> – yfiles-label-sub</li>
* <li><b>&lt;sup></b> – yfiles-label-sup</li>
* <li><b>&lt;[colorname]></b> – yfiles-label-[colorname]</li>
* </ul>
* This style does not support right-to-left direction.
*
* @see MarkupLabelStyleRenderer
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MarkupLabelStyle Online Documentation}
*/
export class MarkupLabelStyle {
/**
Creates a new instance using the provided renderer.
@param renderer The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarkupLabelStyle%23MarkupLabelStyle(ILabelStyleRenderer) Online Documentation}*/
constructor(renderer: ILabelStyleRenderer);
/**
Creates a new instance using the provided renderer.
@param options The parameters to pass.
@param options.renderer The custom renderer for this style instance.
@param [options.backgroundFill] The brush to use for the background box of the label. This option sets the {@link DefaultLabelStyle#backgroundFill} property on the created object.
@param [options.verticalTextAlignment] The vertical text alignment to use if the label is assigned more space than needed. This option sets the {@link DefaultLabelStyle#verticalTextAlignment} property on the created object.
@param [options.horizontalTextAlignment] The text alignment. This option sets the {@link DefaultLabelStyle#horizontalTextAlignment} property on the created object.
@param [options.wrapping] The text trimming. This option sets the {@link DefaultLabelStyle#wrapping} property on the created object.
@param [options.clipText] A value that determines whether text should be clipped. This option sets the {@link DefaultLabelStyle#clipText} property on the created object.
@param [options.backgroundStroke] The pen to use for the background box of the label. This option sets the {@link DefaultLabelStyle#backgroundStroke} property on the created object.
@param [options.font] The typeface to use for the label. This option sets the {@link DefaultLabelStyle#font} property on the created object.
@param [options.autoFlip] A value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented downwards, otherwise. This option sets the {@link DefaultLabelStyle#autoFlip} property on the created object.
@param [options.textFill] The brush to use for the label's text. This option sets the {@link DefaultLabelStyle#textFill} property on the created object.
@param [options.textSize] The size of the text. This option sets the {@link DefaultLabelStyle#textSize} property on the created object.
@param [options.insets] The insets for the label. This option sets the {@link DefaultLabelStyle#insets} property on the created object.
@param [options.minimumSize] The minimum preferred size allowed for the label. This option sets the {@link DefaultLabelStyle#minimumSize} property on the created object.
@param [options.maximumSize] The maximum preferred size allowed for the label. This option sets the {@link DefaultLabelStyle#maximumSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarkupLabelStyle%23MarkupLabelStyle(ILabelStyleRenderer) Online Documentation}*/
constructor(options: {
renderer: ILabelStyleRenderer;
backgroundFill?: Fill | null | FillConvertible;
verticalTextAlignment?: VerticalTextAlignment | VerticalTextAlignmentStringValues;
horizontalTextAlignment?: HorizontalTextAlignment | HorizontalTextAlignmentStringValues;
wrapping?: TextWrapping | TextWrappingStringValues;
clipText?: boolean;
backgroundStroke?: Stroke | null | StrokeConvertible;
font?: Font | FontConvertible;
autoFlip?: boolean;
textFill?: Fill | FillConvertible;
textSize?: number;
insets?: Insets | InsetsConvertible;
minimumSize?: Size | SizeConvertible;
maximumSize?: Size | SizeConvertible;
});
/**
Creates a new instance using {@link MarkupLabelStyleRenderer} as renderer.
@param [options] The parameters to pass.
@param [options.backgroundFill] The brush to use for the background box of the label. This option sets the {@link DefaultLabelStyle#backgroundFill} property on the created object.
@param [options.verticalTextAlignment] The vertical text alignment to use if the label is assigned more space than needed. This option sets the {@link DefaultLabelStyle#verticalTextAlignment} property on the created object.
@param [options.horizontalTextAlignment] The text alignment. This option sets the {@link DefaultLabelStyle#horizontalTextAlignment} property on the created object.
@param [options.wrapping] The text trimming. This option sets the {@link DefaultLabelStyle#wrapping} property on the created object.
@param [options.clipText] A value that determines whether text should be clipped. This option sets the {@link DefaultLabelStyle#clipText} property on the created object.
@param [options.backgroundStroke] The pen to use for the background box of the label. This option sets the {@link DefaultLabelStyle#backgroundStroke} property on the created object.
@param [options.font] The typeface to use for the label. This option sets the {@link DefaultLabelStyle#font} property on the created object.
@param [options.autoFlip] A value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented downwards, otherwise. This option sets the {@link DefaultLabelStyle#autoFlip} property on the created object.
@param [options.textFill] The brush to use for the label's text. This option sets the {@link DefaultLabelStyle#textFill} property on the created object.
@param [options.textSize] The size of the text. This option sets the {@link DefaultLabelStyle#textSize} property on the created object.
@param [options.insets] The insets for the label. This option sets the {@link DefaultLabelStyle#insets} property on the created object.
@param [options.minimumSize] The minimum preferred size allowed for the label. This option sets the {@link DefaultLabelStyle#minimumSize} property on the created object.
@param [options.maximumSize] The maximum preferred size allowed for the label. This option sets the {@link DefaultLabelStyle#maximumSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarkupLabelStyle%23MarkupLabelStyle() Online Documentation}*/
constructor(options?: {
backgroundFill?: Fill | null | FillConvertible;
verticalTextAlignment?: VerticalTextAlignment | VerticalTextAlignmentStringValues;
horizontalTextAlignment?: HorizontalTextAlignment | HorizontalTextAlignmentStringValues;
wrapping?: TextWrapping | TextWrappingStringValues;
clipText?: boolean;
backgroundStroke?: Stroke | null | StrokeConvertible;
font?: Font | FontConvertible;
autoFlip?: boolean;
textFill?: Fill | FillConvertible;
textSize?: number;
insets?: Insets | InsetsConvertible;
minimumSize?: Size | SizeConvertible;
maximumSize?: Size | SizeConvertible;
});
static $class: Class;
private $_MarkupLabelStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MarkupLabelStyle;
}
export interface MarkupLabelStyleRenderer extends DefaultLabelStyleRenderer {
}
/**
An {@link ILabelStyleRenderer} implementation that can handle {@link MarkupLabelStyle} instances.
@see MarkupLabelStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/MarkupLabelStyleRenderer Online Documentation}*/
export class MarkupLabelStyleRenderer {
constructor();
static $class: Class;
private $_MarkupLabelStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MarkupLabelStyleRenderer;
}
export interface PathBasedEdgeStyleRenderer<TStyle extends IEdgeStyle> extends YObject, IEdgeStyleRenderer, IPathGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, ILassoTestable, IHitTestable, ILookup, IVisualCreator, IObstacleProvider {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
getPath(): GeneralPath | null;
/**
@inheritdoc*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
isInPath(context: IInputModeContext, path: GeneralPath): boolean;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
An abstract base class for {@link IEdgeStyleRenderer} implementations that are based on the calculation of a
{@link GeneralPath}.
Type parameter <code>TStyle</code>: The exact type of style to use for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer Online Documentation}*/
export abstract class PathBasedEdgeStyleRenderer<TStyle extends IEdgeStyle> {
/**
Creates a new instance of PathBasedEdgeStyleRenderer.
@param edgeStyleType The exact type of style to use for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer Online Documentation}*/
constructor(edgeStyleType: Class);
/**
Prepares this instance for subsequent calls after the style and edge have been initialized.
Upon invocation the {@link PathBasedEdgeStyleRenderer#style} and {@link PathBasedEdgeStyleRenderer#edge} properties have been populated by the {@link PathBasedEdgeStyleRenderer#getVisualCreator}, {@link PathBasedEdgeStyleRenderer#getBoundsProvider}, {@link PathBasedEdgeStyleRenderer#getHitTestable},
or {@link PathBasedEdgeStyleRenderer#getMarqueeTestable} methods.
Subclasses which override this method have to call the base implementation, first.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
This method should create the basic uncropped path given the control points.
@returns A path that has to be cropped, yet.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23createPath() Online Documentation}*/
abstract createPath(): GeneralPath;
/**
Creates a rectangular self loop path.
@param coordinates The coordinates of the path points, alternating x and y values.
@see PathBasedEdgeStyleRenderer#createSelfLoopPath
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23createRectangleSelfLoop(Array%3Cnumber%3E) Online Documentation}*/
static createRectangleSelfLoop(coordinates: number[]): GeneralPath;
/**
Creates a rectangular self loop path.
@param sx The x coordinate of the source port.
@param sy The y coordinate of the source port.
@param oppositeX The x coordinate of the single control point.
@param oppositeY The y coordinate of the single control point.
@param tx The x coordinate of the target port.
@param ty The y coordinate of the target port.
@returns A path that describes a rectangular self loop.
@see PathBasedEdgeStyleRenderer#createSelfLoopPath
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23createRectangleSelfLoop(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static createRectangleSelfLoop(sx: number, sy: number, oppositeX: number, oppositeY: number, tx: number, ty: number): GeneralPath;
/**
Creates a round self loop path.
@param sx The x coordinate of the source port.
@param sy The y coordinate of the source port.
@param oppositeX The x coordinate of the single control point.
@param oppositeY The y coordinate of the single control point.
@param tx The x coordinate of the target port.
@param ty The y coordinate of the target port.
@returns A path that describes a round self loop.
@see PathBasedEdgeStyleRenderer#createSelfLoopPath
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23createRoundSelfLoop(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static createRoundSelfLoop(sx: number, sy: number, oppositeX: number, oppositeY: number, tx: number, ty: number): GeneralPath;
/**
Special implementation that will create a self loop path.
Called if the {@link PathBasedEdgeStyleRenderer#edge} has the same source and target {@link IPortOwner port owner} and less than two {@link IEdge#bends bends}. The returned path ends at the source
and target ports and will be {@link PathBasedEdgeStyleRenderer#cropPath cropped} at the node bounds later.
@param roundSelfLoop Whether to create a round self loop path. If <code>false</code> a rectangular path will be created.
@returns A path for a self loop.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23createSelfLoopPath(boolean) Online Documentation}*/
createSelfLoopPath(roundSelfLoop: boolean): GeneralPath;
/**
Crops the edge's path at the nodes.
This implementation uses the {@link IEdgePathCropper} instances found in the {@link ILookup#lookup} of the
source and target port of the edge to perform the actual cropping.
@param path The path that should be cropped.
@returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23cropPath(GeneralPath) Online Documentation}*/
cropPath(path: GeneralPath): GeneralPath;
/**
Configures the <code>style</code> and <code>edge</code> parameters, calls {@link PathBasedEdgeStyleRenderer#configure} and returns <code>this</code>.
@param edge The edge to retrieve the bounds provider for. The value will be stored in the {@link PathBasedEdgeStyleRenderer#edge} property.
@param style The style to associate with the edge. The value will be stored in the {@link PathBasedEdgeStyleRenderer#style} property.
@returns <code>this</code>
@see PathBasedEdgeStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getBoundsProvider(IEdge%2CIEdgeStyle) Online Documentation}*/
getBoundsProvider(edge: IEdge, style: IEdgeStyle): IBoundsProvider;
/**
Yields the {@link IBridgeCreator} to use for {@link PathBasedEdgeStyleRenderer#addBridges adding bridges} to the {@link PathBasedEdgeStyleRenderer#createPath created path}.
The return value of this method (which may be <code>null</code>) will be passed to the {@link BridgeManager}'s {@link BridgeManager#addBridges}
method.
@returns This implementation returns <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getBridgeCreator() Online Documentation}*/
getBridgeCreator(): IBridgeCreator | null;
/**
Configures the <code>style</code> and <code>edge</code> parameters, does <b>not</b> call {@link PathBasedEdgeStyleRenderer#configure} and returns <code>this</code>.
As this method may be called often it will not automatically call {@link PathBasedEdgeStyleRenderer#configure}, instead subclasses should ensure that in
the {@link PathBasedEdgeStyleRenderer#lookup} method call they should call {@link PathBasedEdgeStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an instance
that depends on a correctly configured <code>this</code>.
@param edge The edge to query the context for. The value will be stored in the {@link PathBasedEdgeStyleRenderer#edge} property.
@param style The style to associate with the edge. The value will be stored in the {@link PathBasedEdgeStyleRenderer#style} property.
@returns <code>this</code>
@see PathBasedEdgeStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getContext(IEdge%2CIEdgeStyle) Online Documentation}*/
getContext(edge: IEdge, style: IEdgeStyle): ILookup;
/**
Configures the <code>style</code> and <code>edge</code> parameters and returns <code>this</code>.
This implementation does not call {@link PathBasedEdgeStyleRenderer#configure}. If subclasses depend on a configured instance in an override of {@link PathBasedEdgeStyleRenderer#isHit},
they need to override this method, too and call {@link PathBasedEdgeStyleRenderer#configure} if the base call returns a non-<code>null</code> value.
@param edge
@param style
@see PathBasedEdgeStyleRenderer#isHit
@see PathBasedEdgeStyleRenderer#configure
@see IEdgeStyleRenderer#getHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getHitTestable(IEdge%2CIEdgeStyle) Online Documentation}*/
getHitTestable(edge: IEdge, style: IEdgeStyle): IHitTestable;
/**
Configures the <code>style</code> and <code>edge</code> parameters, calls {@link PathBasedEdgeStyleRenderer#configure} and returns <code>this</code>.
@param edge The edge to query marquee intersection tests. The value will be stored in the {@link PathBasedEdgeStyleRenderer#edge} property.
@param style The style to associate with the edge. The value will be stored in the {@link PathBasedEdgeStyleRenderer#style} property.
@returns <code>this</code>
@see PathBasedEdgeStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getMarqueeTestable(IEdge%2CIEdgeStyle) Online Documentation}*/
getMarqueeTestable(edge: IEdge, style: IEdgeStyle): IMarqueeTestable;
/**
Returns {@link PathBasedEdgeStyleRenderer#createPath} unless {@link PathBasedEdgeStyleRenderer#isVisible} yields <code>false</code> for the provided context and {@link IRenderContext#clip}.
@param context The context to yield the obstacles for.
@returns Either <code>null</code> or the result of {@link PathBasedEdgeStyleRenderer#createPath}.
@see IObstacleProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getObstacles(IRenderContext) Online Documentation}*/
getObstacles(context: IRenderContext): GeneralPath | null;
/**
Gets an implementation of the {@link IPathGeometry} interface that can handle the provided edge and its associated
style.
@param edge The edge to provide an instance for
@param style The style to use for the painting
@returns <code>this</code>
@see PathBasedEdgeStyleRenderer#configure
@see PathBasedEdgeStyleRenderer#getTangent
@see PathBasedEdgeStyleRenderer#getTangentForSegment
@see PathBasedEdgeStyleRenderer#getSegmentCount
@see PathBasedEdgeStyleRenderer#getPath
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getPathGeometry(IEdge%2CIEdgeStyle) Online Documentation}*/
getPathGeometry(edge: IEdge, style: IEdgeStyle): IPathGeometry;
/**
Returns the number of "segments" this edge's path consists of.
This method is part of the {@link IPathGeometry} interface, that is implemented by this class. This interface will
be returned by {@link PathBasedEdgeStyleRenderer#getPathGeometry}.
@returns the number of segments or <code>-1</code> if there is no such thing as a segment for this edge.
@see PathBasedEdgeStyleRenderer#getPathGeometry
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getSegmentCount() Online Documentation}*/
abstract getSegmentCount(): number;
/**
Gets the value that determines the radius of the smoothing arcs that can be added to the path at the bends.
This implementation returns <code>0.0d</code>, which effectively turns off path smoothing.
@returns The radius of the arcs to use at the bends of the path.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getSmoothingLength() Online Documentation}*/
getSmoothingLength(): number;
/**
Gets the source arrow from the style.
@returns The arrow to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getSourceArrow() Online Documentation}*/
abstract getSourceArrow(): IArrow | null;
/**
Calculate and update the anchor and the source arrow's direction vector.
@param arrow The arrow to calculate the anchor for.
@returns The anchor and directional vector of the arrow, if any exist.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getSourceArrowAnchor(IArrow) Online Documentation}*/
getSourceArrowAnchor(arrow: IArrow): Tangent | null;
/**
Gets the {@link Stroke} to use to render the path.
@returns The {@link Stroke} or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getStroke() Online Documentation}*/
abstract getStroke(): Stroke | null;
/**
Calculates the tangent on the edge's path at the given ratio point.
This method is part of the {@link IPathGeometry} interface, that is implemented by this class. This interface will
be returned by {@link PathBasedEdgeStyleRenderer#getPathGeometry}.
@param ratio a value in [0,1] where 0 is the source's end and 1 is at the target's end of the visible edge path
@returns The tangential line, if any exists.
@see PathBasedEdgeStyleRenderer#getPathGeometry
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getTangent(number) Online Documentation}*/
abstract getTangent(ratio: number): Tangent | null;
/**
Calculates the tangent on the edge's path at the given ratio point for the given segment.
This method is part of the {@link IPathGeometry} interface, that is implemented by this class. This interface will
be returned by {@link PathBasedEdgeStyleRenderer#getPathGeometry}.
@param segmentIndex the segment to use for the calculation
@param ratio a value in [0,1] where 0 is the source's end and 1 is at the target's end of the segment
@returns The tangential line, if any exists.
@see PathBasedEdgeStyleRenderer#getTangent
@see PathBasedEdgeStyleRenderer#getSegmentCount
@see PathBasedEdgeStyleRenderer#getPathGeometry
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getTangentForSegment(number%2Cnumber) Online Documentation}*/
abstract getTangentForSegment(segmentIndex: number, ratio: number): Tangent | null;
/**
Gets the target arrow from the style.
@returns The arrow to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getTargetArrow() Online Documentation}*/
abstract getTargetArrow(): IArrow | null;
/**
Calculate and update the anchor and the target arrow's direction vector.
@param arrow The arrow to calculate the anchor for.
@returns The anchor and directional vector of the arrow, if any exist.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getTargetArrowAnchor(IArrow) Online Documentation}*/
getTargetArrowAnchor(arrow: IArrow): Tangent | null;
/**
Configures the <code>style</code> and <code>edge</code> parameters, does <b>not</b> call {@link PathBasedEdgeStyleRenderer#configure} and returns <code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link PathBasedEdgeStyleRenderer#configure}. If the subclass implementation depends on
this instance to be configured, it needs to call <code>Configure</code> in {@link PathBasedEdgeStyleRenderer#isVisible}.
@param edge The edge to query visibility tests. The value will be stored in the {@link PathBasedEdgeStyleRenderer#edge} property.
@param style The style to associate with the edge. The value will be stored in the {@link PathBasedEdgeStyleRenderer#style} property.
@returns <code>this</code>
@see PathBasedEdgeStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getVisibilityTestable(IEdge%2CIEdgeStyle) Online Documentation}*/
getVisibilityTestable(edge: IEdge, style: IEdgeStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>edge</code> parameters, calls {@link PathBasedEdgeStyleRenderer#configure} and returns <code>this</code>.
@param edge The edge to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link PathBasedEdgeStyleRenderer#edge} property.
@param style The style to associate with the edge. The value will be stored in the {@link PathBasedEdgeStyleRenderer#style} property.
@returns <code>this</code>
@see PathBasedEdgeStyleRenderer#createVisual
@see PathBasedEdgeStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23getVisualCreator(IEdge%2CIEdgeStyle) Online Documentation}*/
getVisualCreator(edge: IEdge, style: IEdgeStyle): IVisualCreator;
/**
This method is overridden for performance reasons.
This implementation uses {@link PathBasedEdgeStyleRenderer#isVisible} as an early exit indicator. If {@link PathBasedEdgeStyleRenderer#isVisible} returns <code>false</code> for a certain rectangle, the
instance will not be configured but a non-hit will be assumed. Otherwise this instance will be configured and the
standard path based hit test routine
Subclasses should not depend on the fact that {@link PathBasedEdgeStyleRenderer#configure} has already been called. This may not be the case. If your
subclass depends on Configure being called, override {@link PathBasedEdgeStyleRenderer#getHitTestable} and call {@link PathBasedEdgeStyleRenderer#configure} after the base class call.
@param context
@param location
@see PathBasedEdgeStyleRenderer#getHitTestable
@see PathBasedEdgeStyleRenderer#configure
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23isHit(IInputModeContext%2CPoint) Online Documentation}*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
Calls {@link PathBasedEdgeStyleRenderer#configure} and test the {@link PathBasedEdgeStyleRenderer#edge} using the {@link PathBasedEdgeStyleRenderer#style} for a visible intersection with the clip bounds.
This simple implementation uses {@link PathBasedEdgeStyleRenderer#getBounds} to determine the visibility. Subclasses may choose to override this behavior
for improved performance but need to <b>make sure to call {@link PathBasedEdgeStyleRenderer#configure}</b> if they depend on a configured instance.
@param context the current canvas context
@param rectangle the clip bounds
@returns <code>true</code> if the edge may be painted within the clip bounds.
@see PathBasedEdgeStyleRenderer#getVisibilityTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
The path instance that this instance is working with.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23path Online Documentation}*/
path: GeneralPath | null;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23style Online Documentation}*/
style: TStyle;
/**
Gets or sets the currently configured edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23edge Online Documentation}*/
edge: IEdge;
/**
Gets whether the rendered path should use the {@link BridgeManager} to {@link BridgeManager#addBridges add bridges to it.}
@see PathBasedEdgeStyleRenderer#getBridgeCreator
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23addBridges Online Documentation}*/
readonly addBridges: boolean;
/**
Gets or sets the distance between the node's layout and its self loop control points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23selfLoopDistance Online Documentation}*/
selfLoopDistance: number;
/**
Gets or sets the policy with which smoothing is applied to the path when {@link PathBasedEdgeStyleRenderer#getSmoothingLength} returns a value larger than
0.
The default value is {@link SmoothingPolicy#SYMMETRIC}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23smoothingPolicy Online Documentation}*/
smoothingPolicy: SmoothingPolicy;
/**
Gets or sets a value indicating whether the first and last segments of the path should always begin and end with a
straight line segment, respectively.
Given high enough values for
{@link PathBasedEdgeStyleRenderer#getSmoothingLength}, the first and last segments may become completely curved. This property prevents this from
happening, e.g. to prevent arrowheads from colliding with the smoothed edge path.
The default value is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathBasedEdgeStyleRenderer%23straightEndsForSmoothing Online Documentation}*/
straightEndsForSmoothing: boolean;
static $class: Class;
private $_PathBasedEdgeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TStyle extends IEdgeStyle>(o: any): o is PathBasedEdgeStyleRenderer<TStyle>;
}
export interface PolylineEdgeStyleRenderer extends PathBasedEdgeStyleRenderer<PolylineEdgeStyle> {
/**
@inheritdoc*/
createPath(): GeneralPath;
/**
@inheritdoc*/
getSegmentCount(): number;
/**
@inheritdoc*/
getStroke(): Stroke | null;
/**
@inheritdoc*/
getTangent(ratio: number): Tangent | null;
/**
@inheritdoc*/
getTangentForSegment(segmentIndex: number, ratio: number): Tangent | null;
/**
@inheritdoc*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
} /**
An {@link IEdgeStyleRenderer} that will render {@link PolylineEdgeStyle} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyleRenderer Online Documentation}*/
export class PolylineEdgeStyleRenderer {
constructor();
/**
Gets the value that determines the radius of the smoothing arcs that can be added to the path at the bends.
This implementation returns {@link PolylineEdgeStyle#smoothingLength}.
@returns The radius of the arcs to use at the bends of the path.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyleRenderer%23getSmoothingLength() Online Documentation}*/
getSmoothingLength(): number;
/**
Gets the source arrow from the style via {@link PolylineEdgeStyle#sourceArrow}.
@returns The arrow to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyleRenderer%23getSourceArrow() Online Documentation}*/
getSourceArrow(): IArrow | null;
/**
Gets the target arrow from the style via {@link PolylineEdgeStyle#targetArrow}.
@returns The arrow to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyleRenderer%23getTargetArrow() Online Documentation}*/
getTargetArrow(): IArrow | null;
static $class: Class;
private $_PolylineEdgeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PolylineEdgeStyleRenderer;
}
export interface PolylineEdgeStyle extends YObject, IEdgeStyle {
}
/**
A polygonal {@link IEdgeStyle} where straight line segments are used to connect the bends.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle Online Documentation}*/
export class PolylineEdgeStyle {
/**
Creates a new instance using the provided renderer.
@param renderer The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle%23PolylineEdgeStyle(PolylineEdgeStyleRenderer) Online Documentation}*/
constructor(renderer: PolylineEdgeStyleRenderer);
/**
Creates a new instance using the provided renderer.
@param options The parameters to pass.
@param options.renderer The custom renderer for this style instance.
@param [options.stroke] The Pen for the line. This option sets the {@link PolylineEdgeStyle#stroke} property on the created object.
@param [options.smoothingLength] The smoothing length used for creating smooth bends. This option sets the {@link PolylineEdgeStyle#smoothingLength} property on the created object.
@param [options.sourceArrow] The visual arrow at the source end of edges that use this style. This option sets the {@link PolylineEdgeStyle#sourceArrow} property on the created object.
@param [options.targetArrow] The visual arrow at the target end of edges that use this style. This option sets the {@link PolylineEdgeStyle#targetArrow} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle%23PolylineEdgeStyle(PolylineEdgeStyleRenderer) Online Documentation}*/
constructor(options: {
renderer: PolylineEdgeStyleRenderer;
stroke?: Stroke | null | StrokeConvertible;
smoothingLength?: number;
sourceArrow?: IArrow | IArrowConvertible;
targetArrow?: IArrow | IArrowConvertible;
});
/**
Creates a new instance using {@link PolylineEdgeStyleRenderer} as renderer.
@param [options] The parameters to pass.
@param [options.stroke] The Pen for the line. This option sets the {@link PolylineEdgeStyle#stroke} property on the created object.
@param [options.smoothingLength] The smoothing length used for creating smooth bends. This option sets the {@link PolylineEdgeStyle#smoothingLength} property on the created object.
@param [options.sourceArrow] The visual arrow at the source end of edges that use this style. This option sets the {@link PolylineEdgeStyle#sourceArrow} property on the created object.
@param [options.targetArrow] The visual arrow at the target end of edges that use this style. This option sets the {@link PolylineEdgeStyle#targetArrow} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle%23PolylineEdgeStyle() Online Documentation}*/
constructor(options?: {
stroke?: Stroke | null | StrokeConvertible;
smoothingLength?: number;
sourceArrow?: IArrow | IArrowConvertible;
targetArrow?: IArrow | IArrowConvertible;
});
/**
Performs a {@link YObject#memberwiseClone}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle%23clone() Online Documentation}*/
clone(): this;
/**
Gets or sets the Pen for the line.
The default value is {@link Stroke#BLACK}.
@default 'System.Drawing.Pens.Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle%23stroke Online Documentation}*/
stroke: Stroke | null;
/**
Gets or sets the smoothing length used for creating smooth bends.
A value of <code>0.0d</code> will disable smoothing.
The default value is <code>0</code>.
@default 0.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle%23smoothingLength Online Documentation}*/
smoothingLength: number;
/**
Gets or sets the visual arrow at the target end of edges that use this style.
Arrow instances may be shared between multiple style instances.
This property may not be <code>null</code>. To have no target arrow set, use {@link IArrow#NONE} instead.
Note that changes to the properties of an arrow may not be immediately visible. It is therefore recommended to assign a
new arrow instance to this property if you want to change aspects of the target arrow visualization.
The default value is {@link IArrow#NONE}.
@default 'None'
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle%23targetArrow Online Documentation}*/
targetArrow: IArrow;
/**
Gets or sets the visual arrow at the source end of edges that use this style.
Arrow instances may be shared between multiple style instances.
This property may not be <code>null</code>. To have no source arrow set, use {@link IArrow#NONE} instead.
Note that changes to the properties of an arrow may not be immediately visible. It is therefore recommended to assign a
new arrow instance to this property if you want to change aspects of the source arrow visualization.
The default value is {@link IArrow#NONE}.
@default 'None'
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle%23sourceArrow Online Documentation}*/
sourceArrow: IArrow;
/**
Gets the renderer implementation that has been supplied to this instance in the constructor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeStyle%23renderer Online Documentation}*/
readonly renderer: IEdgeStyleRenderer;
static $class: Class;
private $_PolylineEdgeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PolylineEdgeStyle;
}
/**
This enumeration predefines some often used shapes that can be used together with {@link ShapeNodeStyle}
instances
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape Online Documentation}*/
export enum ShapeNodeShape {
/**
A rectangular shape
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23RECTANGLE Online Documentation}*/
RECTANGLE,
/**
A rectangular shape with rounded edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23ROUND_RECTANGLE Online Documentation}*/
ROUND_RECTANGLE,
/**
An elliptical shape
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23ELLIPSE Online Documentation}*/
ELLIPSE,
/**
A triangular shape that points to the top
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23TRIANGLE Online Documentation}*/
TRIANGLE,
/**
A triangular shape that points to the bottom
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23TRIANGLE2 Online Documentation}*/
TRIANGLE2,
/**
A rectangle that is sheared in the horizontal direction to the right
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23SHEARED_RECTANGLE Online Documentation}*/
SHEARED_RECTANGLE,
/**
A rectangle that is sheared in the horizontal direction to the left
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23SHEARED_RECTANGLE2 Online Documentation}*/
SHEARED_RECTANGLE2,
/**
A trapezoid shape that is smaller at the bottom
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23TRAPEZ Online Documentation}*/
TRAPEZ,
/**
A trapezoid shape that is smaller at the top
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23TRAPEZ2 Online Documentation}*/
TRAPEZ2,
/**
A 5-star shape
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23STAR5 Online Documentation}*/
STAR5,
/**
A 6-star shape
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23STAR6 Online Documentation}*/
STAR6,
/**
An 8-star shape
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23STAR8 Online Documentation}*/
STAR8,
/**
An arrow like shape that points to the right
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23FAT_ARROW Online Documentation}*/
FAT_ARROW,
/**
An arrow like shape that points to the left
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23FAT_ARROW2 Online Documentation}*/
FAT_ARROW2,
/**
A symmetric parallelogram shape that has sloped edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23DIAMOND Online Documentation}*/
DIAMOND,
/**
A regular eight-sided shape,
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23OCTAGON Online Documentation}*/
OCTAGON,
/**
A regular six-sided shape,
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeShape%23HEXAGON Online Documentation}*/
HEXAGON
}
/** Automatic conversion to {@link ShapeNodeShape}. */
type ShapeNodeShapeStringValues = "rectangle" | "round-rectangle" | "ellipse" | "triangle" | "triangle2" | "sheared-rectangle" | "sheared-rectangle2" | "trapez" | "trapez2" | "star5" | "star6" | "star8" | "fat-arrow" | "fat-arrow2" | "diamond" | "octagon" | "hexagon";
export interface ShapeNodeStyleRenderer extends YObject, INodeStyleRenderer, IShapeGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
@inheritdoc*/
getOutline(): GeneralPath | null;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isInside(location: Point): boolean;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
Default renderer implementation that can be used for {@link ShapeNodeStyle} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer Online Documentation}*/
export class ShapeNodeStyleRenderer {
constructor();
/**
Prepares this instance for subsequent calls after the style and node have been initialized.
Upon invocation the {@link ShapeNodeStyleRenderer#style} and {@link ShapeNodeStyleRenderer#node} properties have been populated by the {@link ShapeNodeStyleRenderer#getVisualCreator},
{@link ShapeNodeStyleRenderer#getBoundsProvider}, {@link ShapeNodeStyleRenderer#getHitTestable}, or {@link ShapeNodeStyleRenderer#getMarqueeTestable}
methods.
Subclasses which override this method must call the base method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Returns the layout bounds as an upper bound for the painting bound.
This implementation simply sets the {@link ShapeNodeStyleRenderer#layout}'s bounds to the scratch variable and returns.
@param context the context to calculate the bounds for
@returns the bounds or <code>null</code> to indicate an unbound area
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getBounds(ICanvasContext) Online Documentation}*/
getBounds(context: ICanvasContext): Rect;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the bounds provider for. The value will be stored in the {@link ShapeNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShapeNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShapeNodeStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
As this method may be called often it will not automatically call {@link ShapeNodeStyleRenderer#configure},
instead subclasses should ensure that in the {@link ShapeNodeStyleRenderer#lookup} method call they should call {@link ShapeNodeStyleRenderer#configure}
only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
@param node The node to query the context for. The value will be stored in the {@link ShapeNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShapeNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShapeNodeStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
Get the {@link Fill} that is currently used by this renderer.
This implementation retrieves the {@link Fill} from the owning style
@returns The {@link Fill} that is currently used by this renderer.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getFill() Online Documentation}*/
getFill(): Fill | null;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query hit test with. The value will be stored in the {@link ShapeNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShapeNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShapeNodeStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query marquee intersection tests. The value will be stored in the {@link ShapeNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShapeNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShapeNodeStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
Get the shape that is currently used by this renderer.
This implementation retrieves the shape from the owning style
@returns The shape that is currently used by this renderer.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getShape() Online Documentation}*/
getShape(): ShapeNodeShape;
/**
Configures the {@link ShapeNodeStyleRenderer#style} and {@link ShapeNodeStyleRenderer#node} properties, calls {@link ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the shape geometry for. The value will be stored in the {@link ShapeNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShapeNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShapeNodeStyleRenderer#isInside
@see ShapeNodeStyleRenderer#getIntersection
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Get the {@link Stroke} that is currently used by this renderer.
This implementation retrieves the {@link Stroke} from the owning style
@returns The {@link Stroke} that is currently used by this renderer.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getStroke() Online Documentation}*/
getStroke(): Stroke | null;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link ShapeNodeStyleRenderer#configure}. If the
subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link ShapeNodeStyleRenderer#isVisible}.
@param node The node to query visibility tests. The value will be stored in the {@link ShapeNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShapeNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShapeNodeStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link ShapeNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShapeNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShapeNodeStyleRenderer#createVisual
@see ShapeNodeStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
Tests the {@link ShapeNodeStyleRenderer#node node} using the current {@link ShapeNodeStyleRenderer#style} for an intersection with the given box.
This implementation uses the {@link ShapeNodeStyleRenderer#layout} to determine whether the node is in the box.
@param context the current input mode context
@param rectangle the box describing the marquee's bounds
@returns <code>true</code> if the node is considered to be captured by the marquee
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23isInBox(IInputModeContext%2CRect) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
Determines visibility by checking for an intersection with the {@link ShapeNodeStyleRenderer#layout}.
@param context The context.
@param rectangle The current clip
@returns Whether the {@link ShapeNodeStyleRenderer#layout} is visible in the clip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Implements the {@link ILookup} interface.
This method will be used by default if {@link ShapeNodeStyleRenderer#getContext} has been queried for a lookup
implementation. Note that it cannot be assumed that {@link ShapeNodeStyleRenderer#configure} has already been
invoked. However, normally {@link ShapeNodeStyleRenderer#node} and {@link ShapeNodeStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link ShapeNodeStyleRenderer#configure}
on success and return <code>this</code>.
@param type The type to query for.
@returns An implementation or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23style Online Documentation}*/
style: ShapeNodeStyle;
/**
Gets or sets the currently configured node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23node Online Documentation}*/
node: INode;
/**
Gets the {@link INode#layout} of the node this renderer is currently configured for.
Returns <code>Node.Layout</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23layout Online Documentation}*/
readonly layout: IRectangle;
/**
Gets or sets the radius that is used to draw rounded edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyleRenderer%23roundRectArcRadius Online Documentation}*/
roundRectArcRadius: number;
static $class: Class;
private $_ShapeNodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ShapeNodeStyleRenderer;
}
export interface ShapeNodeStyle extends YObject, INodeStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
Node style implementation that can show predefined shapes of type {@link ShapeNodeShape}.
This style can be used together with {@link ShapeNodeStyleRenderer} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyle Online Documentation}*/
export class ShapeNodeStyle {
/**
Creates a new instance using the provided renderer.
@param renderer Custom renderer instance for this style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyle%23ShapeNodeStyle(ShapeNodeStyleRenderer) Online Documentation}*/
constructor(renderer: ShapeNodeStyleRenderer);
/**
Creates a new instance using the provided renderer.
@param options The parameters to pass.
@param options.renderer Custom renderer instance for this style.
@param [options.fill] The {@link ShapeNodeStyle#fill} for this style. This option sets the {@link ShapeNodeStyle#fill} property on the created object.
@param [options.stroke] The {@link ShapeNodeStyle#stroke} for this style. This option sets the {@link ShapeNodeStyle#stroke} property on the created object.
@param [options.shape] The shape for this style. This option sets the {@link ShapeNodeStyle#shape} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyle%23ShapeNodeStyle(ShapeNodeStyleRenderer) Online Documentation}*/
constructor(options: {
renderer: ShapeNodeStyleRenderer;
fill?: Fill | null | FillConvertible;
stroke?: Stroke | null | StrokeConvertible;
shape?: ShapeNodeShape | ShapeNodeShapeStringValues;
});
/**
Creates a new instance using {@link ShapeNodeStyleRenderer} as renderer.
@param [options] The parameters to pass.
@param [options.fill] The {@link ShapeNodeStyle#fill} for this style. This option sets the {@link ShapeNodeStyle#fill} property on the created object.
@param [options.stroke] The {@link ShapeNodeStyle#stroke} for this style. This option sets the {@link ShapeNodeStyle#stroke} property on the created object.
@param [options.shape] The shape for this style. This option sets the {@link ShapeNodeStyle#shape} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyle%23ShapeNodeStyle() Online Documentation}*/
constructor(options?: {
fill?: Fill | null | FillConvertible;
stroke?: Stroke | null | StrokeConvertible;
shape?: ShapeNodeShape | ShapeNodeShapeStringValues;
});
/**
Gets or sets the {@link ShapeNodeStyle#stroke} for this style.
The default value is {@link Stroke#BLACK}.
@default 'System.Drawing.Pens.Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyle%23stroke Online Documentation}*/
stroke: Stroke | null;
/**
Gets or sets the {@link ShapeNodeStyle#fill} for this style.
The default value is {@link Fill#WHITE}.
@default 'System.Drawing.Brushes.White'
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyle%23fill Online Documentation}*/
fill: Fill | null;
/**
Gets or sets the shape for this style.
The default value is {@link ShapeNodeShape#RECTANGLE}.
@default ShapeNodeShape.RECTANGLE
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShapeNodeStyle%23shape Online Documentation}*/
shape: ShapeNodeShape;
static $class: Class;
private $_ShapeNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ShapeNodeStyle;
}
export interface FoldingEdgeConverterBase extends YObject, IFoldingEdgeConverter {
}
/**
An abstract basic implementation of the {@link IFoldingEdgeConverter} interface that may be derived from to create
a customized {@link FoldingManager#foldingEdgeConverter}.
This class provides default mechanisms and callbacks that can be used to set another style for folding edges, for the
ports of folding edges, synchronizing the first label of a folding edge with the master edge, etc. Subclasses need to
implement the {@link FoldingEdgeConverterBase#addFoldingEdge} method, only.
@see DefaultFoldingEdgeConverter
@see FoldingManager
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase Online Documentation}*/
export abstract class FoldingEdgeConverterBase {
/**
Creates the {@link IEdgeStyle} for use in {@link FoldingEdgeConverterBase#initializeFoldingEdgeStyle}.
@param foldingView The folding view instance.
@param foldingEdge The folding edge in the view whose {@link IEdge#style} should be initialized.
@param masterEdges The edges that are being represented by the folding edge.
@returns The {@link FoldingEdgeConverterBase#foldingEdgeStyle} or a {@link ICloneable#clone} of it depending on the {@link FoldingEdgeConverterBase#cloneEdgeStyle} property. This method may return <code>null</code> to indicate that
the default style should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23createEdgeStyle(IFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
createEdgeStyle(foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): IEdgeStyle | null;
/**
Creates the {@link ILabelModelParameter} for use in
{@link FoldingEdgeConverterBase#initializeFoldingEdgeLabels}.
@param foldingView The folding view instance.
@param localLabel The local label in the view whose {@link ILabel#layoutParameter} should be initialized. May be <code>null</code> if the label layout parameter is requested for the
label creation.
@param masterLabel The label that is being represented by the folding label.
@returns The {@link FoldingEdgeConverterBase#labelLayoutParameter} or a {@link ICloneable#clone} of it depending on the {@link FoldingEdgeConverterBase#cloneLabelLayoutParameter} property. This method may return <code>null</code> to indicate that
the default label layout parameter should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23createLabelLayoutParameter(IFoldingView%2CILabel%2CILabel) Online Documentation}*/
createLabelLayoutParameter(foldingView: IFoldingView, localLabel: ILabel | null, masterLabel: ILabel): ILabelModelParameter | null;
/**
Creates the {@link ILabelStyle} for use in {@link FoldingEdgeConverterBase#initializeFoldingEdgeLabels}.
@param foldingView The folding view instance.
@param localLabel The local label in the view whose {@link ILabel#style} should be initialized. May be <code>null</code> if the label style is requested for the label
creation.
@param masterLabel The label that is being represented by the folding label.
@returns The {@link FoldingEdgeConverterBase#labelStyle} or a {@link ICloneable#clone} of it depending on the {@link FoldingEdgeConverterBase#cloneLabelStyle} property. This method may return <code>null</code> to indicate that
the default style should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23createLabelStyle(IFoldingView%2CILabel%2CILabel) Online Documentation}*/
createLabelStyle(foldingView: IFoldingView, localLabel: ILabel | null, masterLabel: ILabel): ILabelStyle | null;
/**
Gets the preferred size for use in {@link FoldingEdgeConverterBase#initializeFoldingEdgeLabels} and
{@link FoldingEdgeConverterBase#synchronizeLabels}.
This implementation returns the preferred size of <code>masterLabel</code>.
@param masterLabel The label that is being represented by the folding label.
@returns The preferred size of the given label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23createPreferredLabelSize(ILabel) Online Documentation}*/
createPreferredLabelSize(masterLabel: ILabel): Size;
/**
Creates the {@link IPortLocationModelParameter} of the source port for use in
{@link FoldingEdgeConverterBase#initializeFoldingEdgePorts}.
@param foldingView The folding view instance.
@param localEdge The local edge in the view whose source port's {@link IPort#locationParameter} should be initialized.
@param masterEdges The master edges that the folding edge represents.
@returns The {@link FoldingEdgeConverterBase#sourcePortLocationParameter} or a {@link ICloneable#clone} of it depending on the {@link FoldingEdgeConverterBase#cloneSourcePortLocationParameter} property. This method may return <code>null</code> to indicate that
the default port location parameter should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23createSourcePortLocationParameter(IFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
createSourcePortLocationParameter(foldingView: IFoldingView, localEdge: IEdge, masterEdges: IList<IEdge>): IPortLocationModelParameter | null;
/**
Creates the {@link IPortStyle} of the source port for use in
{@link FoldingEdgeConverterBase#initializeFoldingEdgePorts}.
@param foldingView The folding view instance.
@param localEdge The local edge in the view whose source port's {@link IPort#style} should be initialized.
@param masterEdges The master edges that the folding edge represents.
@returns The {@link FoldingEdgeConverterBase#sourcePortStyle} or a {@link ICloneable#clone} of it depending on the {@link FoldingEdgeConverterBase#cloneSourcePortStyle} property. This method may return <code>null</code> to indicate that
the default style should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23createSourcePortStyle(IFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
createSourcePortStyle(foldingView: IFoldingView, localEdge: IEdge, masterEdges: IList<IEdge>): IPortStyle | null;
/**
Creates the {@link IPortLocationModelParameter} of the target port for use in
{@link FoldingEdgeConverterBase#initializeFoldingEdgePorts}.
@param foldingView The folding view instance.
@param localEdge The local edge in the view whose target port's {@link IPort#locationParameter} should be initialized.
@param masterEdges The master edges that the folding edge represents.
@returns The {@link FoldingEdgeConverterBase#targetPortLocationParameter} or a {@link ICloneable#clone} of it depending on the {@link FoldingEdgeConverterBase#cloneTargetPortLocationParameter} property. This method may return <code>null</code> to indicate that
the default port location parameter should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23createTargetPortLocationParameter(IFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
createTargetPortLocationParameter(foldingView: IFoldingView, localEdge: IEdge, masterEdges: IList<IEdge>): IPortLocationModelParameter | null;
/**
Creates the {@link IPortStyle} of the target port for use in
{@link FoldingEdgeConverterBase#initializeFoldingEdgePorts}.
@param foldingView The folding view instance.
@param localEdge The local edge in the view whose target port's {@link IPort#style} should be initialized.
@param masterEdges The master edges that the folding edge represents.
@returns The {@link FoldingEdgeConverterBase#targetPortStyle} or a {@link ICloneable#clone} of it depending on the {@link FoldingEdgeConverterBase#cloneTargetPortStyle} property. This method may return <code>null</code> to indicate that
the default style should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23createTargetPortStyle(IFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
createTargetPortStyle(foldingView: IFoldingView, localEdge: IEdge, masterEdges: IList<IEdge>): IPortStyle | null;
/**
Initializes the bends of the folding edge.
This will call the {@link FoldingEdgeState#clearBends} callback if {@link FoldingEdgeConverterBase#resetBends} is enabled.
@param state The viewstate to change.
@param foldingView The folding view instance that has triggered the {@link FoldingEdgeConverterBase#initializeFoldingEdgeState} call.
@param foldingEdge The folding edge in the view that may be changed using the <code>state</code>.
@param masterEdges The master edges that the folding edge represents.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23initializeFoldingEdgeBends(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
initializeFoldingEdgeBends(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
/**
Initializes the initial labels of the folding edge.
This will first {@link FoldingLabelOwnerState#clearLabels clear all the labels} and then try to copy the first label of the single master edge if {@link FoldingEdgeConverterBase#copyFirstLabel} is enabled. It
will call {@link FoldingEdgeConverterBase#createLabelStyle} and {@link FoldingEdgeConverterBase#createLabelLayoutParameter}
and use the returned style and layout parameter if they are non-<code>null</code> when {@link FoldingEdgeState#addLabel adding the label} on the folding edge.
@param state The viewstate to change.
@param foldingView The folding view instance that has triggered the {@link FoldingEdgeConverterBase#initializeFoldingEdgeState} call.
@param foldingEdge The folding edge in the view that may be changed using the <code>state</code>.
@param masterEdges The master edges that the folding edge represents.
@see FoldingEdgeConverterBase#labelStyle
@see FoldingEdgeConverterBase#labelLayoutParameter
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23initializeFoldingEdgeLabels(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
initializeFoldingEdgeLabels(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
/**
Initializes the ports of the folding edge.
If the source port of the <code>foldingEdge</code> {@link IFoldingView#isInFoldingState is in folding state}, this implementation calls {@link FoldingEdgeConverterBase#createSourcePortStyle}
and {@link FoldingEdgeConverterBase#createSourcePortLocationParameter} and {@link FoldingPortState#style assigns the style} and {@link FoldingPortState#locationParameter location parameter}
to the folding source port if non-<code>null</code> values have been returned. The same holds true for the target port where the {@link FoldingEdgeConverterBase#createTargetPortStyle}
and {@link FoldingEdgeConverterBase#createTargetPortLocationParameter} callbacks are used.
@param state The viewState to change.
@param foldingView The folding view instance that has triggered the {@link FoldingEdgeConverterBase#initializeFoldingEdgeState} call.
@param foldingEdge The folding edge in the view that may be changed using the <code>state</code>.
@param masterEdges The master edges that the folding edge represents.
@see FoldingEdgeConverterBase#sourcePortStyle
@see FoldingEdgeConverterBase#targetPortStyle
@see FoldingEdgeConverterBase#sourcePortLocationParameter
@see FoldingEdgeConverterBase#targetPortLocationParameter
@see FoldingEdgeConverterBase#createSourcePortStyle
@see FoldingEdgeConverterBase#createTargetPortStyle
@see FoldingEdgeConverterBase#createSourcePortLocationParameter
@see FoldingEdgeConverterBase#createTargetPortLocationParameter
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23initializeFoldingEdgePorts(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
initializeFoldingEdgePorts(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
/**
Implements the {@link IFoldingEdgeConverter#initializeFoldingEdgeState} method and initializes the folding edge
appearance.
This method calls {@link FoldingEdgeConverterBase#initializeFoldingEdgeStyle},
{@link FoldingEdgeConverterBase#initializeFoldingEdgePorts},
{@link FoldingEdgeConverterBase#initializeFoldingEdgeBends},
{@link FoldingEdgeConverterBase#initializeFoldingEdgeLabels}.
@param state
@param foldingView
@param foldingEdge
@param masterEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23initializeFoldingEdgeState(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
initializeFoldingEdgeState(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
/**
Initializes the {@link FoldingEdgeState#style style property} of the folding edge.
This implementation calls {@link FoldingEdgeConverterBase#createEdgeStyle} and {@link FoldingEdgeState#style assigns} the style to the folding
edge if a non-<code>null</code> value has been returned.
@param state The viewstate to change.
@param foldingView The folding view instance that has triggered the {@link FoldingEdgeConverterBase#initializeFoldingEdgeState} call.
@param foldingEdge The folding edge in the view that may be changed using the <code>state</code>.
@param masterEdges The master edges that the folding edge represents.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23initializeFoldingEdgeStyle(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
initializeFoldingEdgeStyle(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
/**
Called by {@link FoldingEdgeConverterBase#updateFoldingEdgeState} to synchronize the first label if {@link FoldingEdgeConverterBase#copyFirstLabel} is enabled.
This will adjust the label's text, preferred size and tag properties or remove the label if there is no master label or
there is more than one master edge.
@param state The viewstate of the edge to change.
@param foldingView The folding view.
@param foldingEdge The folding edge.
@param masterEdges The master edges.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23synchronizeLabels(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
synchronizeLabels(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
/**
Implements the {@link IFoldingEdgeConverter#updateFoldingEdgeState} method and changes the folding edge
appearance.
This method calls {@link FoldingEdgeConverterBase#synchronizeLabels}, only.
@param state
@param foldingView
@param foldingEdge
@param masterEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23updateFoldingEdgeState(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
updateFoldingEdgeState(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
/**
Gets or sets a value indicating whether the {@link FoldingEdgeConverterBase#foldingEdgeStyle} instance should be assigned as a {@link ICloneable#clone clone} or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23cloneEdgeStyle Online Documentation}*/
cloneEdgeStyle: boolean;
/**
Gets or sets the {@link IPortStyle} to use for the folding source port.
A value of <code>null</code> will leave the style property unmodified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23sourcePortStyle Online Documentation}*/
sourcePortStyle: IPortStyle | null;
/**
Gets or sets a value indicating whether the {@link FoldingEdgeConverterBase#sourcePortStyle} instance should be assigned as a {@link ICloneable#clone clone} or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23cloneSourcePortStyle Online Documentation}*/
cloneSourcePortStyle: boolean;
/**
Gets or sets the {@link IPortStyle} to use for the folding target port.
A value of <code>null</code> will leave the style property unmodified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23targetPortStyle Online Documentation}*/
targetPortStyle: IPortStyle | null;
/**
Gets or sets a value indicating whether the {@link FoldingEdgeConverterBase#targetPortStyle} instance should be assigned as a {@link ICloneable#clone}clone or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23cloneTargetPortStyle Online Documentation}*/
cloneTargetPortStyle: boolean;
/**
Gets or sets the {@link IPortLocationModelParameter} to use for the source port of the folding edge.
Since the appearance of the folding edge may totally differ from the appearance of the master edge, the location of the
source port may also be different. This property can be used to assign a different specific location parameter to the
source port if the edge {@link IFoldingView#isInFoldingState is in folding state}. A value of <code>null</code> will leave the location parameter property unmodified. The
default is <code>null</code>
@see FoldingEdgeConverterBase#createSourcePortLocationParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23sourcePortLocationParameter Online Documentation}*/
sourcePortLocationParameter: IPortLocationModelParameter | null;
/**
Gets or sets a value indicating whether the {@link FoldingEdgeConverterBase#sourcePortLocationParameter} instance should be assigned as a {@link ICloneable#clone}clone or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23cloneSourcePortLocationParameter Online Documentation}*/
cloneSourcePortLocationParameter: boolean;
/**
Gets or sets the {@link IPortLocationModelParameter} to use for the target port of the folding edge.
Since the appearance of the folding edge may totally differ from the appearance of the master edge, the location of the
target port may also be different. This property can be used to assign a different specific location parameter to the
target port if the edge {@link IFoldingView#isInFoldingState is in folding state}. A value of <code>null</code> will leave the location parameter property unmodified. The
default is <code>null</code>
@see FoldingEdgeConverterBase#createTargetPortLocationParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23targetPortLocationParameter Online Documentation}*/
targetPortLocationParameter: IPortLocationModelParameter | null;
/**
Gets or sets a value indicating whether the {@link FoldingEdgeConverterBase#targetPortLocationParameter} instance should be assigned as a {@link ICloneable#clone}clone or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23cloneTargetPortLocationParameter Online Documentation}*/
cloneTargetPortLocationParameter: boolean;
/**
Gets or sets the {@link ILabelStyle} to use for the first label of the folding edge.
This will only affect the folding edge if the {@link FoldingEdgeConverterBase#copyFirstLabel} property is set to <code>true</code>.
A value of <code>null</code> will leave the style property unmodified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23labelStyle Online Documentation}*/
labelStyle: ILabelStyle | null;
/**
Gets or sets a value indicating whether the {@link FoldingEdgeConverterBase#labelStyle} instance should be assigned as a {@link ICloneable#clone}clone or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23cloneLabelStyle Online Documentation}*/
cloneLabelStyle: boolean;
/**
Gets or sets the {@link ILabelModelParameter} to use for the first label of the folding edge.
This will only affect the folding edge if the {@link FoldingEdgeConverterBase#copyFirstLabel} property is set to <code>true</code>.
A value of <code>null</code> will leave the label model parameter property unmodified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23labelLayoutParameter Online Documentation}*/
labelLayoutParameter: ILabelModelParameter | null;
/**
Gets or sets a value indicating whether the {@link FoldingEdgeConverterBase#labelLayoutParameter} instance should be assigned as a {@link ICloneable#clone}clone or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23cloneLabelLayoutParameter Online Documentation}*/
cloneLabelLayoutParameter: boolean;
/**
Gets or sets the {@link IEdgeStyle} to use for the folding edge.
Depending on the {@link FoldingEdgeConverterBase#cloneEdgeStyle} property this implementation will assign a {@link ICloneable#clone} to the property or this
instance.
A value of <code>null</code> (the default) will leave the style property unmodified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23foldingEdgeStyle Online Documentation}*/
foldingEdgeStyle: IEdgeStyle | null;
/**
Gets or sets a value indicating whether to reset and clear the bends of a folding edge initially.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23resetBends Online Documentation}*/
resetBends: boolean;
/**
Gets or sets a value indicating whether the first label of the {@link IFoldingView#getMasterItem master edge} should be recreated for the folding edge.
This setting can be used to initially create a copy of the first label of the master edge (if any) and subsequently
synchronize the {@link ILabel#text} property with the master's edge label text.
@see FoldingEdgeConverterBase#labelStyle
@see FoldingEdgeConverterBase#labelLayoutParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeConverterBase%23copyFirstLabel Online Documentation}*/
copyFirstLabel: boolean;
static $class: Class;
private $_FoldingEdgeConverterBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FoldingEdgeConverterBase;
}
export interface DefaultFoldingEdgeConverter extends FoldingEdgeConverterBase {
}
/**
The default implementation of the {@link IFoldingEdgeConverter} that is used by the {@link FoldingManager}
initially and can be customized to suit the application's needs.
@see FoldingEdgeConverterBase
@see DefaultFoldingEdgeConverter#reuseMasterPorts
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFoldingEdgeConverter Online Documentation}*/
export class DefaultFoldingEdgeConverter {
/**
Initializes a new instance of this class with default values.
@param [options] The parameters to pass.
@param [options.reuseMasterPorts] A property that determines whether this implementation should reuse the ports of the {@link FoldingManager#masterGraph} at non-{@link IFoldingView#isInFoldingState folder} nodes for the folding edges. This option sets the {@link DefaultFoldingEdgeConverter#reuseMasterPorts} property on the created object.
@param [options.reuseFolderNodePorts] A property that determines whether this implementation should reuse the ports of {@link IFoldingView#isInFoldingState folder} nodes for the folding edges. This option sets the {@link DefaultFoldingEdgeConverter#reuseFolderNodePorts} property on the created object.
@param [options.cloneEdgeStyle] A value indicating whether the {@link FoldingEdgeConverterBase#foldingEdgeStyle} instance should be assigned as a {@link ICloneable#clone clone} or not. This option sets the {@link FoldingEdgeConverterBase#cloneEdgeStyle} property on the created object.
@param [options.sourcePortStyle] The {@link IPortStyle} to use for the folding source port. This option sets the {@link FoldingEdgeConverterBase#sourcePortStyle} property on the created object.
@param [options.cloneSourcePortStyle] A value indicating whether the {@link FoldingEdgeConverterBase#sourcePortStyle} instance should be assigned as a {@link ICloneable#clone clone} or not. This option sets the {@link FoldingEdgeConverterBase#cloneSourcePortStyle} property on the created object.
@param [options.targetPortStyle] The {@link IPortStyle} to use for the folding target port. This option sets the {@link FoldingEdgeConverterBase#targetPortStyle} property on the created object.
@param [options.cloneTargetPortStyle] A value indicating whether the {@link FoldingEdgeConverterBase#targetPortStyle} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneTargetPortStyle} property on the created object.
@param [options.sourcePortLocationParameter] The {@link IPortLocationModelParameter} to use for the source port of the folding edge. This option sets the {@link FoldingEdgeConverterBase#sourcePortLocationParameter} property on the created object.
@param [options.cloneSourcePortLocationParameter] A value indicating whether the {@link FoldingEdgeConverterBase#sourcePortLocationParameter} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneSourcePortLocationParameter} property on the created object.
@param [options.targetPortLocationParameter] The {@link IPortLocationModelParameter} to use for the target port of the folding edge. This option sets the {@link FoldingEdgeConverterBase#targetPortLocationParameter} property on the created object.
@param [options.cloneTargetPortLocationParameter] A value indicating whether the {@link FoldingEdgeConverterBase#targetPortLocationParameter} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneTargetPortLocationParameter} property on the created object.
@param [options.labelStyle] The {@link ILabelStyle} to use for the first label of the folding edge. This option sets the {@link FoldingEdgeConverterBase#labelStyle} property on the created object.
@param [options.cloneLabelStyle] A value indicating whether the {@link FoldingEdgeConverterBase#labelStyle} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneLabelStyle} property on the created object.
@param [options.labelLayoutParameter] The {@link ILabelModelParameter} to use for the first label of the folding edge. This option sets the {@link FoldingEdgeConverterBase#labelLayoutParameter} property on the created object.
@param [options.cloneLabelLayoutParameter] A value indicating whether the {@link FoldingEdgeConverterBase#labelLayoutParameter} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneLabelLayoutParameter} property on the created object.
@param [options.foldingEdgeStyle] The {@link IEdgeStyle} to use for the folding edge. This option sets the {@link FoldingEdgeConverterBase#foldingEdgeStyle} property on the created object.
@param [options.resetBends] A value indicating whether to reset and clear the bends of a folding edge initially. This option sets the {@link FoldingEdgeConverterBase#resetBends} property on the created object.
@param [options.copyFirstLabel] A value indicating whether the first label of the {@link IFoldingView#getMasterItem master edge} should be recreated for the folding edge. This option sets the {@link FoldingEdgeConverterBase#copyFirstLabel} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFoldingEdgeConverter Online Documentation}*/
constructor(options?: {
reuseMasterPorts?: boolean;
reuseFolderNodePorts?: boolean;
cloneEdgeStyle?: boolean;
sourcePortStyle?: IPortStyle | null;
cloneSourcePortStyle?: boolean;
targetPortStyle?: IPortStyle | null;
cloneTargetPortStyle?: boolean;
sourcePortLocationParameter?: IPortLocationModelParameter | null;
cloneSourcePortLocationParameter?: boolean;
targetPortLocationParameter?: IPortLocationModelParameter | null;
cloneTargetPortLocationParameter?: boolean;
labelStyle?: ILabelStyle | null;
cloneLabelStyle?: boolean;
labelLayoutParameter?: ILabelModelParameter | null;
cloneLabelLayoutParameter?: boolean;
foldingEdgeStyle?: IEdgeStyle | null;
resetBends?: boolean;
copyFirstLabel?: boolean;
});
/**
Actually adds the folding edge as a {@link IFoldingEdgeFactory#addAsSeparateEdge separate edge} to the view, reusing existing {@link IFoldingView#getViewItem view} ports of the original source and
target port, if {@link DefaultFoldingEdgeConverter#reuseMasterPorts} is enabled and that is possible for the given edge.
@param factory The {@link IFoldingEdgeFactory} implementation.
@param foldingView The instance into which the folding edge is going to be inserted.
@param masterEdge The edge from the {@link FoldingManager#masterGraph} that needs to be represented by a folding edge.
@param source The source node that belongs to the {@link IFoldingView} graph that will act as the local view node of the actual
source node.
@param sourceIsCollapsed Determines whether the <code>source</code> is currently a folder node.
@param target The target node that belongs to the {@link IFoldingView} graph that will act as the local view node of the actual
target node.
@param targetIsCollapsed Determines whether the <code>target</code> is currently a folder node.
@returns The edge as returned by the call to {@link IFoldingEdgeFactory#addAsSeparateEdge}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFoldingEdgeConverter%23addFoldingEdge(IFoldingEdgeFactory%2CIFoldingView%2CIEdge%2CINode%2Cboolean%2CINode%2Cboolean) Online Documentation}*/
addFoldingEdge(factory: IFoldingEdgeFactory, foldingView: IFoldingView, masterEdge: IEdge, source: INode, sourceIsCollapsed: boolean, target: INode, targetIsCollapsed: boolean): IEdge | null;
/**
Gets or sets a property that determines whether this implementation should reuse the ports of the {@link FoldingManager#masterGraph} at non-{@link IFoldingView#isInFoldingState folder} nodes
for the folding edges.
The {@link IFoldingView view} can be configured to use separate port instances for folding edges. This makes it possible to assign different
styles and locations to source and target ports of folding edges, which may be desirable if the style and geometry of
folding edges differs a lot from the style of the master edges. Folding edges may of course still connect to non-folder
nodes at one of their ports. Setting this property to <code>true</code> will make the folding edge connect to the {@link IFoldingView#getViewItem view} port instance.
This property can be set to <code>true</code>, iff the graph model used is port-centric, i.e. each edge connects to specific ports
and even folding edges should still connect to those specific view ports. The default value is <code>false</code>
@see DefaultFoldingEdgeConverter#reuseFolderNodePorts
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFoldingEdgeConverter%23reuseMasterPorts Online Documentation}*/
reuseMasterPorts: boolean;
/**
Gets or sets a property that determines whether this implementation should reuse the ports of {@link IFoldingView#isInFoldingState folder} nodes for the
folding edges.
The {@link IFoldingView view} can be configured to use separate port instances for folding edges. This makes it possible to assign different
styles and locations to source and target ports of folding edges, which may be desirable if the style and geometry of
folding edges differs a lot from the style of the master edges. Folder nodes ({@link IFoldingView#collapse collapsed} group nodes), will have all of
their master's ports {@link IFoldingView#getViewItem represented} in the view. This property can be used to make folding edges that connect to the master
ports connect to these folding port instances instead of creating extra folding ports for the edges. Note that ports can
only be automatically reused by folding edges if they still connect to the same nodes. Otherwise the view will have to
create new folding ports to let the edges connect to the representing adjacent nodes. This property can be set to <code>true</code>,
iff the graph model used is port-centric, i.e. each edge connects to specific ports and even folding edges should still
connect to those specific port instances, if possible. The default value is <code>false</code>.
@see DefaultFoldingEdgeConverter#reuseMasterPorts
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFoldingEdgeConverter%23reuseFolderNodePorts Online Documentation}*/
reuseFolderNodePorts: boolean;
static $class: Class;
private $_DefaultFoldingEdgeConverter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultFoldingEdgeConverter;
}
export interface DefaultFolderNodeConverter extends YObject, IFolderNodeConverter {
}
/**
* Simple default implementation of the {@link IFolderNodeConverter} interface that is used by the
* {@link FoldingManager}'s {@link FoldingManager#folderNodeConverter} property.
* This class has a couple of properties that can be adjusted to customize the behavior. Also, it contains a number of
* callback methods for conveniently overriding the implementation in custom sub classes.
*
* <pre>
* manager.folderNodeConverter = new DefaultFolderNodeConverter({
*   copyFirstLabel: true,
*   folderNodeStyle: new ShapeNodeStyle({ fill: Fill.GRAY })
* })
* </pre>
*
* @see FoldingManager
* @see IFolderNodeConverter
* @see DefaultFoldingEdgeConverter
* @see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter Online Documentation}
*/
export class DefaultFolderNodeConverter {
/**
Initializes a new instance of this class using the values provided by the <code>folderNodeDefaults</code> parameter.
@param folderNodeDefaults
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23DefaultFolderNodeConverter(INodeDefaults) Online Documentation}*/
constructor(folderNodeDefaults: INodeDefaults);
/**
Initializes a new instance of this class with default values.
@param [options] The parameters to pass.
@param [options.folderNodeStyle] The {@link INodeStyle} to use for the {@link IFoldingView#collapse collapsed} group nodes. This option sets the {@link DefaultFolderNodeConverter#folderNodeStyle} property on the created object.
@param [options.copyFirstLabel] A value indicating whether the first label of the {@link IFoldingView#getMasterItem master group node} should be recreated for the collapsed group node instance. This option sets the {@link DefaultFolderNodeConverter#copyFirstLabel} property on the created object.
@param [options.labelStyle] The {@link ILabelStyle} to use for the first label of the collapsed group nodes. This option sets the {@link DefaultFolderNodeConverter#labelStyle} property on the created object.
@param [options.labelLayoutParameter] The {@link ILabelModelParameter} to use for the first label of the collapsed group node. This option sets the {@link DefaultFolderNodeConverter#labelLayoutParameter} property on the created object.
@param [options.portStyle] The {@link IPortStyle} to use for the ports at the collapsed group node that {@link IFoldingView#getViewItem represent} the ports of the {@link IFoldingView#getMasterItem master group node} in the collapsed state. This option sets the {@link DefaultFolderNodeConverter#portStyle} property on the created object.
@param [options.folderNodeSize] The initial size of the {@link INode#layout} of the collapsed group node that will be assigned during the {@link IFolderNodeConverter#initializeFolderNodeState creation} of the appearance of the collapsed group node. This option sets the {@link DefaultFolderNodeConverter#folderNodeSize} property on the created object.
@param [options.portLocationParameter] The {@link IPortLocationModelParameter} to use for the ports at the collapsed group node that {@link IFoldingView#getViewItem represent} the ports of the {@link IFoldingView#getMasterItem master group node} in the collapsed state. This option sets the {@link DefaultFolderNodeConverter#portLocationParameter} property on the created object.
@param [options.cloneNodeStyle] A value indicating whether the {@link DefaultFolderNodeConverter#folderNodeStyle} instance should be assigned as a {@link ICloneable#clone clone} or not. This option sets the {@link DefaultFolderNodeConverter#cloneNodeStyle} property on the created object.
@param [options.clonePortStyle] A value indicating whether the {@link DefaultFolderNodeConverter#portStyle} instance should be assigned as a {@link ICloneable#clone clone} or not. This option sets the {@link DefaultFolderNodeConverter#clonePortStyle} property on the created object.
@param [options.clonePortLocationParameter] A value indicating whether the {@link DefaultFolderNodeConverter#portLocationParameter} instance should be assigned as a {@link ICloneable#clone clone} or not. This option sets the {@link DefaultFolderNodeConverter#clonePortLocationParameter} property on the created object.
@param [options.cloneLabelStyle] A value indicating whether the {@link DefaultFolderNodeConverter#labelStyle} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link DefaultFolderNodeConverter#cloneLabelStyle} property on the created object.
@param [options.cloneLabelLayoutParameter] A value indicating whether the {@link DefaultFolderNodeConverter#labelLayoutParameter} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link DefaultFolderNodeConverter#cloneLabelLayoutParameter} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23DefaultFolderNodeConverter() Online Documentation}*/
constructor(options?: {
folderNodeStyle?: INodeStyle | null;
copyFirstLabel?: boolean;
labelStyle?: ILabelStyle | null;
labelLayoutParameter?: ILabelModelParameter | null;
portStyle?: IPortStyle | null;
folderNodeSize?: Size | null | SizeConvertible;
portLocationParameter?: IPortLocationModelParameter | null;
cloneNodeStyle?: boolean;
clonePortStyle?: boolean;
clonePortLocationParameter?: boolean;
cloneLabelStyle?: boolean;
cloneLabelLayoutParameter?: boolean;
});
/**
Creates the {@link ILabelModelParameter} for use in
{@link DefaultFolderNodeConverter#initializeFolderNodeLabels}.
@param foldingView The folding view instance.
@param localLabel The local label in the view whose {@link ILabel#layoutParameter} should be initialized. May be <code>null</code> if the label layout parameter is requested for the
label creation.
@param masterLabel The label that is being represented by the folding label.
@returns The {@link DefaultFolderNodeConverter#labelLayoutParameter} or a {@link ICloneable#clone} of it depending on the {@link DefaultFolderNodeConverter#cloneLabelLayoutParameter} property. This method may return <code>null</code> to indicate that
the default label layout parameter should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23createLabelLayoutParameter(IFoldingView%2CILabel%2CILabel) Online Documentation}*/
createLabelLayoutParameter(foldingView: IFoldingView, localLabel: ILabel | null, masterLabel: ILabel): ILabelModelParameter | null;
/**
Creates the {@link ILabelStyle} for use in {@link DefaultFolderNodeConverter#initializeFolderNodeLabels}.
@param foldingView The folding view instance.
@param localLabel The local label in the view whose {@link ILabel#style} should be initialized. May be <code>null</code> if the label style is requested for the label
creation.
@param masterLabel The label that is being represented by the folding label.
@returns The {@link DefaultFolderNodeConverter#labelStyle} or a {@link ICloneable#clone} of it depending on the {@link DefaultFolderNodeConverter#cloneLabelStyle} property. This method may return <code>null</code> to indicate that
the default style should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23createLabelStyle(IFoldingView%2CILabel%2CILabel) Online Documentation}*/
createLabelStyle(foldingView: IFoldingView, localLabel: ILabel | null, masterLabel: ILabel): ILabelStyle | null;
/**
Creates the {@link INodeStyle} for use in {@link DefaultFolderNodeConverter#initializeFolderNodeStyle}.
@param foldingView The folding view instance.
@param viewNode The local folder node in the view whose {@link INode#style} should be initialized.
@param masterNode The node that is being represented by the folder node.
@returns The {@link DefaultFolderNodeConverter#folderNodeStyle} or a {@link ICloneable#clone} of it depending on the {@link DefaultFolderNodeConverter#cloneNodeStyle} property. This method may return <code>null</code> to indicate that
the default style should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23createNodeStyle(IFoldingView%2CINode%2CINode) Online Documentation}*/
createNodeStyle(foldingView: IFoldingView, viewNode: INode, masterNode: INode): INodeStyle | null;
/**
Creates the {@link IPortLocationModelParameter} for use in
{@link DefaultFolderNodeConverter#initializeFolderNodePorts}.
@param foldingView The folding view instance.
@param localPort The local port in the view whose {@link IPort#locationParameter} should be initialized.
@param masterPort The port that is being represented by the folding port.
@returns The {@link DefaultFolderNodeConverter#portLocationParameter} or a {@link ICloneable#clone} of it depending on the {@link DefaultFolderNodeConverter#clonePortLocationParameter} property. This method may return <code>null</code> to indicate that
the default port location parameter should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23createPortLocationParameter(IFoldingView%2CIPort%2CIPort) Online Documentation}*/
createPortLocationParameter(foldingView: IFoldingView, localPort: IPort, masterPort: IPort): IPortLocationModelParameter | null;
/**
Creates the {@link IPortStyle} for use in {@link DefaultFolderNodeConverter#initializeFolderNodePorts}.
@param foldingView The folding view instance.
@param localPort The local port in the view whose {@link IPort#style} should be initialized.
@param masterPort The port that is being represented by the folding port.
@returns The {@link DefaultFolderNodeConverter#portStyle} or a {@link ICloneable#clone} of it depending on the {@link DefaultFolderNodeConverter#clonePortStyle} property. This method may return <code>null</code> to indicate that
the default style should not be changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23createPortStyle(IFoldingView%2CIPort%2CIPort) Online Documentation}*/
createPortStyle(foldingView: IFoldingView, localPort: IPort, masterPort: IPort): IPortStyle | null;
/**
Gets the preferred size for use in {@link DefaultFolderNodeConverter#initializeFolderNodeLabels} and
{@link DefaultFolderNodeConverter#synchronizeLabels}.
This implementation returns the preferred size of <code>masterLabel</code>.
@param masterLabel The label that is being represented by the folding label.
@returns The preferred size of the given label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23createPreferredLabelSize(ILabel) Online Documentation}*/
createPreferredLabelSize(masterLabel: ILabel): Size;
/**
Initializes the initial labels of the collapsed group node.
This will try to copy the first label of the master node if {@link DefaultFolderNodeConverter#copyFirstLabel} is enabled. It will call {@link DefaultFolderNodeConverter#createLabelStyle}
and {@link DefaultFolderNodeConverter#createLabelLayoutParameter} and use the returned style and layout parameter
if they are non-<code>null</code> when {@link FolderNodeState#addLabel adding the label} on the folder node.
@param state The node view state whose labels should be synchronized.
@param foldingView The folding view instance that has triggered the {@link DefaultFolderNodeConverter#initializeFolderNodeState}
call.
@param viewNode The collapsed group node in the view that may be changed using the {@link FolderNodeState}.
@param masterNode The master node that the folder node represents.
@see DefaultFolderNodeConverter#labelStyle
@see DefaultFolderNodeConverter#labelLayoutParameter
@see FoldingManager#getFolderNodeState
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23initializeFolderNodeLabels(FolderNodeState%2CIFoldingView%2CINode%2CINode) Online Documentation}*/
initializeFolderNodeLabels(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
/**
Initializes the {@link INode#layout} of the collapsed group node.
This implementation will {@link FolderNodeState#layout set the bounds} of the nodes in such a way that the size will be that of the {@link DefaultFolderNodeConverter#folderNodeSize} property (if
non-<code>null</code>) but not smaller than the {@link INodeSizeConstraintProvider#getMinimumSize minimum} size that is reported by a potential {@link INodeSizeConstraintProvider}
for {@link INode}s that has been found in the {@link ILookup} of the <code>viewNode</code>.
@param state The node view state whose layout should be synchronized.
@param foldingView The folding view instance.
@param viewNode The local collapsed group node to change.
@param masterNode The master node that is represented by the local group node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23initializeFolderNodeLayout(FolderNodeState%2CIFoldingView%2CINode%2CINode) Online Documentation}*/
initializeFolderNodeLayout(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
/**
Called by {@link DefaultFolderNodeConverter#initializeFolderNodeState} to initialize the appearance of the {@link IFoldingView#getViewItem representatives}
of the master ports at the collapsed group node.
This method will call {@link DefaultFolderNodeConverter#createPortStyle} and {@link DefaultFolderNodeConverter#createPortLocationParameter}
for each port at the group node and assign the returned style and location parameter if they are non-<code>null</code>.
@param state The node view state whose ports should be synchronized.
@param foldingView The folding view instance.
@param viewNode The local group node.
@param masterNode The master group node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23initializeFolderNodePorts(FolderNodeState%2CIFoldingView%2CINode%2CINode) Online Documentation}*/
initializeFolderNodePorts(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
/**
Implements the {@link IFolderNodeConverter#initializeFolderNodeState} method and initializes the appearance of the
collapsed group node.
This method calls {@link DefaultFolderNodeConverter#initializeFolderNodeStyle},
{@link DefaultFolderNodeConverter#initializeFolderNodeLabels},
{@link DefaultFolderNodeConverter#initializeFolderNodeLayout}, and
{@link DefaultFolderNodeConverter#initializeFolderNodePorts}.
@param state
@param foldingView
@param viewNode
@param masterNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23initializeFolderNodeState(FolderNodeState%2CIFoldingView%2CINode%2CINode) Online Documentation}*/
initializeFolderNodeState(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
/**
Initializes the {@link FolderNodeState#style style property} of the collapsed group node.
This implementation calls {@link DefaultFolderNodeConverter#createNodeStyle} and {@link FolderNodeState#style assigns} the style to the group
node if a non-<code>null</code> value has been returned.
@param state The node view state whose style should be synchronized.
@param foldingView The folding view instance that has triggered the {@link DefaultFolderNodeConverter#initializeFolderNodeState}
call.
@param viewNode The local node in the view that may be changed using the {@link FolderNodeState}.
@param masterNode The master node that the local folder node represents.
@see FoldingManager#getFolderNodeState
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23initializeFolderNodeStyle(FolderNodeState%2CIFoldingView%2CINode%2CINode) Online Documentation}*/
initializeFolderNodeStyle(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
/**
Called by {@link DefaultFolderNodeConverter#updateFolderNodeState} to synchronize the first label if {@link DefaultFolderNodeConverter#copyFirstLabel} is enabled.
This will adjust the label text property or remove the label if there is no master label.
@param state The node view state whose labels should be synchronized.
@param foldingView The folding view.
@param viewNode The local node instance.
@param masterNode The master node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23synchronizeLabels(FolderNodeState%2CIFoldingView%2CINode%2CINode) Online Documentation}*/
synchronizeLabels(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
/**
Implements the {@link IFolderNodeConverter#updateFolderNodeState} method and changes the folder node appearance.
This method calls {@link DefaultFolderNodeConverter#synchronizeLabels}, only.
@param state
@param foldingView
@param viewNode
@param masterNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23updateFolderNodeState(FolderNodeState%2CIFoldingView%2CINode%2CINode) Online Documentation}*/
updateFolderNodeState(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
/**
Gets or sets the {@link INodeStyle} to use for the {@link IFoldingView#collapse collapsed} group nodes.
Depending on the {@link DefaultFolderNodeConverter#cloneNodeStyle} property this implementation will assign a {@link ICloneable#clone} to the property or this
instance. The default is <code>null</code>
A value of <code>null</code> will leave the style property unmodified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23folderNodeStyle Online Documentation}*/
folderNodeStyle: INodeStyle | null;
/**
Gets or sets a value indicating whether the first label of the {@link IFoldingView#getMasterItem master group node} should be recreated for the collapsed
group node instance.
This setting can be used to initially create a copy of the first label of the master group node (if any) and
subsequently synchronize the {@link ILabel#text} property with the master's node label text. Set it to <code>true</code> if the first label should be
copied; <code>false</code> otherwise. The default is <code>false</code>.
@see DefaultFolderNodeConverter#labelStyle
@see DefaultFolderNodeConverter#labelLayoutParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23copyFirstLabel Online Documentation}*/
copyFirstLabel: boolean;
/**
Gets or sets the {@link ILabelStyle} to use for the first label of the collapsed group nodes.
This will only affect the collapsed group node if the {@link DefaultFolderNodeConverter#copyFirstLabel} property is set to <code>true</code>. The default is <code>null</code>
A value of <code>null</code> will leave the style property unmodified.
@see DefaultFolderNodeConverter#copyFirstLabel
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23labelStyle Online Documentation}*/
labelStyle: ILabelStyle | null;
/**
Gets or sets the {@link ILabelModelParameter} to use for the first label of the collapsed group node.
This will only affect the folder node if the {@link DefaultFolderNodeConverter#copyFirstLabel} property is set to <code>true</code>. The default is <code>null</code>
A value of <code>null</code> will leave the label model parameter property unmodified.
@see DefaultFolderNodeConverter#copyFirstLabel
@see DefaultFolderNodeConverter#initializeFolderNodeLabels
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23labelLayoutParameter Online Documentation}*/
labelLayoutParameter: ILabelModelParameter | null;
/**
Gets or sets the {@link IPortStyle} to use for the ports at the collapsed group node that {@link IFoldingView#getViewItem represent} the ports of
the {@link IFoldingView#getMasterItem master group node} in the collapsed state.
Since the appearance of the collapsed group node may totally differ from the appearance of the expanded group node, the
appearance of the ports may also be different. This property can be used to assign a different specific style to the
ports if the node is in collapsed state. A value of <code>null</code> will leave the style property unmodified. The default is <code>null</code>
@see DefaultFolderNodeConverter#createPortStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23portStyle Online Documentation}*/
portStyle: IPortStyle | null;
/**
Sets the initial size of the {@link INode#layout} of the collapsed group node that will be assigned during the {@link IFolderNodeConverter#initializeFolderNodeState creation} of the appearance of
the collapsed group node.
Since the {@link INode#layout} of an {@link IFoldingView#expand expanded} group node normally encompasses the child nodes, it will have quite a large size. Since the
contents of a collapsed group node are not visible, there is no need for the collapsed group node to encompass the area
of its children. Thus the size can be different and most of the time should be smaller. Setting this property to non-<code>null</code>
values will make the initial layout of the collapsed node become the given size assigned. The default is <code>null</code>
@see DefaultFolderNodeConverter#initializeFolderNodeLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23folderNodeSize Online Documentation}*/
folderNodeSize: Size | null;
/**
Gets or sets the {@link IPortLocationModelParameter} to use for the ports at the collapsed group node that {@link IFoldingView#getViewItem represent}
the ports of the {@link IFoldingView#getMasterItem master group node} in the collapsed state.
Since the appearance of the collapsed group node may totally differ from the appearance of the expanded group node, the
appearance of the ports may also be different. This property can be used to assign a different specific location
parameter to the ports if the node is in collapsed state. A value of <code>null</code> will leave the location parameter property
unmodified. The default is <code>null</code>
@see DefaultFolderNodeConverter#createPortLocationParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23portLocationParameter Online Documentation}*/
portLocationParameter: IPortLocationModelParameter | null;
/**
Gets or sets a value indicating whether the {@link DefaultFolderNodeConverter#folderNodeStyle} instance should be assigned as a {@link ICloneable#clone clone} or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23cloneNodeStyle Online Documentation}*/
cloneNodeStyle: boolean;
/**
Gets or sets a value indicating whether the {@link DefaultFolderNodeConverter#portStyle} instance should be assigned as a {@link ICloneable#clone clone} or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23clonePortStyle Online Documentation}*/
clonePortStyle: boolean;
/**
Gets or sets a value indicating whether the {@link DefaultFolderNodeConverter#portLocationParameter} instance should be assigned as a {@link ICloneable#clone clone} or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23clonePortLocationParameter Online Documentation}*/
clonePortLocationParameter: boolean;
/**
Gets or sets a value indicating whether the {@link DefaultFolderNodeConverter#labelStyle} instance should be assigned as a {@link ICloneable#clone}clone or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23cloneLabelStyle Online Documentation}*/
cloneLabelStyle: boolean;
/**
Gets or sets a value indicating whether the {@link DefaultFolderNodeConverter#labelLayoutParameter} instance should be assigned as a {@link ICloneable#clone}clone or not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultFolderNodeConverter%23cloneLabelLayoutParameter Online Documentation}*/
cloneLabelLayoutParameter: boolean;
static $class: Class;
private $_DefaultFolderNodeConverter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultFolderNodeConverter;
}
export interface ExcludingFoldingEdgeConverter extends YObject, IFoldingEdgeConverter {
}
/**
A simple implementation of the {@link IFoldingEdgeConverter} interface for use in the {@link FoldingManager#foldingEdgeConverter} property, that will prevent
folding edges from appearing in the {@link IFoldingView view}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExcludingFoldingEdgeConverter Online Documentation}*/
export class ExcludingFoldingEdgeConverter {
/**
Always calls {@link IFoldingEdgeFactory#excludeFoldingEdge}
@param factory
@param foldingView
@param masterEdge
@param source
@param sourceIsCollapsed
@param target
@param targetIsCollapsed
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExcludingFoldingEdgeConverter%23addFoldingEdge(IFoldingEdgeFactory%2CIFoldingView%2CIEdge%2CINode%2Cboolean%2CINode%2Cboolean) Online Documentation}*/
addFoldingEdge(factory: IFoldingEdgeFactory, foldingView: IFoldingView, masterEdge: IEdge, source: INode, sourceIsCollapsed: boolean, target: INode, targetIsCollapsed: boolean): IEdge | null;
/**
Does nothing and should not normally be called by the view since all folding edges are excluded from it.
@param state
@param foldingView
@param foldingEdge
@param masterEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExcludingFoldingEdgeConverter%23initializeFoldingEdgeState(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
initializeFoldingEdgeState(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
/**
Does nothing and should not normally be called by the view since all folding edges are excluded from it.
@param state
@param foldingView
@param foldingEdge
@param masterEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExcludingFoldingEdgeConverter%23updateFoldingEdgeState(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
updateFoldingEdgeState(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
static $class: Class;
private $_ExcludingFoldingEdgeConverter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ExcludingFoldingEdgeConverter;
}
export interface MergingFoldingEdgeConverter extends FoldingEdgeConverterBase {
}
/**
* A simple implementation of the {@link IFoldingEdgeConverter} interface that will make the {@link IFoldingView view} contain at most one
* folding edge between each pair of nodes by letting a single folding edge represent all master edges.
* This implementation by default, will have the {@link FoldingEdgeConverterBase#sourcePortLocationParameter} and the {@link FoldingEdgeConverterBase#targetPortLocationParameter} properties set to {@link FreeNodePortLocationModel#NODE_CENTER_ANCHORED}
* and the {@link FoldingEdgeConverterBase#resetBends} property set to <code>true</code>.
*
* <pre>
* manager.foldingEdgeConverter = new MergingFoldingEdgeConverter({
*   copyFirstLabel: true,
*   foldingEdgeStyle: new PolylineEdgeStyle({ stroke: Stroke.GRAY })
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MergingFoldingEdgeConverter Online Documentation}
*/
export class MergingFoldingEdgeConverter {
/**
Initializes a new instance of the {@link MergingFoldingEdgeConverter} class.
@param [options] The parameters to pass.
@param [options.ignoreEdgeDirection] A value indicating whether edge direction should be ignored for the determination of the {@link IFoldingEdgeFactory#getExistingFoldingEdges existing folding edges}. This option sets the {@link MergingFoldingEdgeConverter#ignoreEdgeDirection} property on the created object.
@param [options.cloneEdgeStyle] A value indicating whether the {@link FoldingEdgeConverterBase#foldingEdgeStyle} instance should be assigned as a {@link ICloneable#clone clone} or not. This option sets the {@link FoldingEdgeConverterBase#cloneEdgeStyle} property on the created object.
@param [options.sourcePortStyle] The {@link IPortStyle} to use for the folding source port. This option sets the {@link FoldingEdgeConverterBase#sourcePortStyle} property on the created object.
@param [options.cloneSourcePortStyle] A value indicating whether the {@link FoldingEdgeConverterBase#sourcePortStyle} instance should be assigned as a {@link ICloneable#clone clone} or not. This option sets the {@link FoldingEdgeConverterBase#cloneSourcePortStyle} property on the created object.
@param [options.targetPortStyle] The {@link IPortStyle} to use for the folding target port. This option sets the {@link FoldingEdgeConverterBase#targetPortStyle} property on the created object.
@param [options.cloneTargetPortStyle] A value indicating whether the {@link FoldingEdgeConverterBase#targetPortStyle} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneTargetPortStyle} property on the created object.
@param [options.sourcePortLocationParameter] The {@link IPortLocationModelParameter} to use for the source port of the folding edge. This option sets the {@link FoldingEdgeConverterBase#sourcePortLocationParameter} property on the created object.
@param [options.cloneSourcePortLocationParameter] A value indicating whether the {@link FoldingEdgeConverterBase#sourcePortLocationParameter} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneSourcePortLocationParameter} property on the created object.
@param [options.targetPortLocationParameter] The {@link IPortLocationModelParameter} to use for the target port of the folding edge. This option sets the {@link FoldingEdgeConverterBase#targetPortLocationParameter} property on the created object.
@param [options.cloneTargetPortLocationParameter] A value indicating whether the {@link FoldingEdgeConverterBase#targetPortLocationParameter} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneTargetPortLocationParameter} property on the created object.
@param [options.labelStyle] The {@link ILabelStyle} to use for the first label of the folding edge. This option sets the {@link FoldingEdgeConverterBase#labelStyle} property on the created object.
@param [options.cloneLabelStyle] A value indicating whether the {@link FoldingEdgeConverterBase#labelStyle} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneLabelStyle} property on the created object.
@param [options.labelLayoutParameter] The {@link ILabelModelParameter} to use for the first label of the folding edge. This option sets the {@link FoldingEdgeConverterBase#labelLayoutParameter} property on the created object.
@param [options.cloneLabelLayoutParameter] A value indicating whether the {@link FoldingEdgeConverterBase#labelLayoutParameter} instance should be assigned as a {@link ICloneable#clone}clone or not. This option sets the {@link FoldingEdgeConverterBase#cloneLabelLayoutParameter} property on the created object.
@param [options.foldingEdgeStyle] The {@link IEdgeStyle} to use for the folding edge. This option sets the {@link FoldingEdgeConverterBase#foldingEdgeStyle} property on the created object.
@param [options.resetBends] A value indicating whether to reset and clear the bends of a folding edge initially. This option sets the {@link FoldingEdgeConverterBase#resetBends} property on the created object.
@param [options.copyFirstLabel] A value indicating whether the first label of the {@link IFoldingView#getMasterItem master edge} should be recreated for the folding edge. This option sets the {@link FoldingEdgeConverterBase#copyFirstLabel} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingFoldingEdgeConverter Online Documentation}*/
constructor(options?: {
ignoreEdgeDirection?: boolean;
cloneEdgeStyle?: boolean;
sourcePortStyle?: IPortStyle | null;
cloneSourcePortStyle?: boolean;
targetPortStyle?: IPortStyle | null;
cloneTargetPortStyle?: boolean;
sourcePortLocationParameter?: IPortLocationModelParameter | null;
cloneSourcePortLocationParameter?: boolean;
targetPortLocationParameter?: IPortLocationModelParameter | null;
cloneTargetPortLocationParameter?: boolean;
labelStyle?: ILabelStyle | null;
cloneLabelStyle?: boolean;
labelLayoutParameter?: ILabelModelParameter | null;
cloneLabelLayoutParameter?: boolean;
foldingEdgeStyle?: IEdgeStyle | null;
resetBends?: boolean;
copyFirstLabel?: boolean;
});
/**
Adds the first separate edge to the source and target node pair using the {@link IFoldingEdgeFactory#addAsSeparateEdge}
method.
@param factory
@param foldingView
@param masterEdge
@param viewSourceNode
@param sourceIsCollapsed
@param targetSourceNode
@param targetIsCollapsed
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingFoldingEdgeConverter%23addFirstSeparateEdge(IFoldingEdgeFactory%2CIFoldingView%2CIEdge%2CINode%2Cboolean%2CINode%2Cboolean) Online Documentation}*/
addFirstSeparateEdge(factory: IFoldingEdgeFactory, foldingView: IFoldingView, masterEdge: IEdge, viewSourceNode: INode, sourceIsCollapsed: boolean, targetSourceNode: INode, targetIsCollapsed: boolean): IEdge;
/**
Tries to add the folding edge to an {@link IFoldingEdgeFactory#getExistingFoldingEdges existing} folding edge, considering the {@link MergingFoldingEdgeConverter#ignoreEdgeDirection} property.
If this is the first folding edge that connects the source and target node, this method will use the {@link MergingFoldingEdgeConverter#addFirstSeparateEdge}
factory to create the initial representative.
@param factory
@param foldingView
@param masterEdge
@param source
@param sourceIsCollapsed
@param target
@param targetIsCollapsed
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingFoldingEdgeConverter%23addFoldingEdge(IFoldingEdgeFactory%2CIFoldingView%2CIEdge%2CINode%2Cboolean%2CINode%2Cboolean) Online Documentation}*/
addFoldingEdge(factory: IFoldingEdgeFactory, foldingView: IFoldingView, masterEdge: IEdge, source: INode, sourceIsCollapsed: boolean, target: INode, targetIsCollapsed: boolean): IEdge | null;
/**
Gets or sets a value indicating whether edge direction should be ignored for the determination of the
{@link IFoldingEdgeFactory#getExistingFoldingEdges existing folding edges}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingFoldingEdgeConverter%23ignoreEdgeDirection Online Documentation}*/
ignoreEdgeDirection: boolean;
static $class: Class;
private $_MergingFoldingEdgeConverter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MergingFoldingEdgeConverter;
}
export interface FoldingManager extends YObject {
}
/**
* This class adds folding support to {@link IGraph} implementations.
* Folding is an extension to the grouping concept. It adds the functionality of {@link IFoldingView#collapse collapsing} (closing) and {@link IFoldingView#expand expanding}
* (opening) group nodes. Collapsing a group node means that its content is hidden and the group node itself becomes a
* normal (i.e. non-group or leaf) node.
*
* In yFiles folding works with two graph instances: a {@link FoldingManager#masterGraph master graph} which is is the complete unfolded graph and the so
* called {@link IFoldingView#graph view graph} which supports collapsing and expanding. This view graph is a modified and managed copy of the master
* graph. To make sure that the state of both graphs is correctly synchronized, yFiles provides the types {@link FoldingManager}
* and the {@link IFoldingView}.
*
* The {@link FoldingManager} holds the {@link FoldingManager#masterGraph master graph} and synchronizes changes between the master graph and the
* different {@link IFoldingView}s. Therefore, it also serves as see {@link FoldingManager#createFoldingView factory} for instances of
* {@link IFoldingView}. In turn, each {@link IFoldingView} instance manages an {@link IGraph}
* implementation that provides a view of a subset of the {@link FoldingManager#masterGraph} where {@link IGraph#isGroupNode group nodes} can appear as {@link IFoldingView#collapse collapsed} group nodes with their
* contents hidden. The views are synchronized with the {@link FoldingManager#masterGraph}, but may contain additional {@link FoldingManager#getFolderNodeState view state}.
*
* <br><b>Creating a graph where nodes can be collapsed and expanded</b>
* <pre>
* // first create the manager
* const manager = new FoldingManager()
* // the manager serves as factory to create views
* const foldingView = manager.createFoldingView()
* // each view contains a folding-enabled graph: the view graph
* const viewGraph = foldingView.graph
* // the view graph is the graph which should actually be displayed
* graphComponent.graph = viewGraph
* </pre>
*
* {@link IFoldingView#collapse Collapsed} group nodes in a {@link IFoldingView view}, as well as their adjacent edges, maintain a separate set of {@link ILabel labels}, {@link IPort ports}, {@link IBend bends}, and
* {@link INode#layout geometry}. This view-local state is shared between all views that have been {@link FoldingManager#createFoldingView created} by the same manager. It can be
* obtained and modified via the methods {@link FoldingManager#getFoldingEdgeState} for folding edges and {@link FoldingManager#getFolderNodeState}
* for folder nodes. The {@link FoldingManager} manages the varying states nodes and edges can have in different views
* and during different collapsed/expanded states of the nodes. Group nodes can appear as {@link IFoldingView#collapse collapsed} group nodes with their
* contents hidden. Edges that connect to elements inside collapsed group nodes can be {@link FoldingManager#foldingEdgeConverter configured} to be represented by {@link FoldingEdgeStateId folding edges}
* in the views. The process of converting one or more true edges from the {@link FoldingManager#masterGraph} to zero or more folding edges can be
* customized, as well as the process of {@link FoldingManager#folderNodeConverter converting} an expanded group node to a collapsed group node.
*
* Class {@link FoldingManager}'s central role for folding support is discussed in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/folding-technical_background Folding Revisited}. A
* brief description of the class's API, set-up, etc. is given in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/folding-working_with_folding Working with Folding}.
*
* @see IFoldingView
* @see FoldingManager#createFoldingView
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager Online Documentation}
*/
export class FoldingManager {
/**
* Creates a manager instance for the given {@link FoldingManager#masterGraph} to create views on top of it that support folding operations.
* The given <code>masterGraph</code> may already contain nodes.
*
* <br><b>Setting up folding with a new graph instance</b>
* <pre>
* const manager = new FoldingManager()
* graphComponent.graph = manager.createFoldingView().graph
* </pre>
*
* <br><b>Setting up folding, reusing a given graph instance</b>
* <pre>
* const manager = new FoldingManager(graphComponent.graph)
* graphComponent.graph = manager.createFoldingView().graph
* </pre>
*
* @param masterGraph The graph to use as the {@link FoldingManager#masterGraph}.
* @throws {Exception} <b>(ArgumentError)</b> <code>masterGraph</code> is null.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23FoldingManager(IGraph) Online Documentation}
*/
constructor(masterGraph: IGraph);
/**
* Creates a manager instance for the given {@link FoldingManager#masterGraph} to create views on top of it that support folding operations.
* The given <code>masterGraph</code> may already contain nodes.
*
* <br><b>Setting up folding with a new graph instance</b>
* <pre>
* const manager = new FoldingManager()
* graphComponent.graph = manager.createFoldingView().graph
* </pre>
*
* <br><b>Setting up folding, reusing a given graph instance</b>
* <pre>
* const manager = new FoldingManager(graphComponent.graph)
* graphComponent.graph = manager.createFoldingView().graph
* </pre>
*
* @param options The parameters to pass.
* @param options.masterGraph The graph to use as the {@link FoldingManager#masterGraph}.
* @param [options.foldingEdgeConverter] The {@link IFoldingEdgeConverter} implementation that is used to create/convert and modify the {@link FoldingEdgeStateId folding edges} inside the view instances. This option sets the {@link FoldingManager#foldingEdgeConverter} property on the created object.
* @param [options.folderNodeConverter] The {@link IFolderNodeConverter} implementation that is used to create/convert and modify the folder nodes inside the view instances. This option sets the {@link FoldingManager#folderNodeConverter} property on the created object.
* @throws {Exception} <b>(ArgumentError)</b> <code>masterGraph</code> is null.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23FoldingManager(IGraph) Online Documentation}
*/
constructor(options: {
masterGraph: IGraph;
foldingEdgeConverter?: IFoldingEdgeConverter;
folderNodeConverter?: IFolderNodeConverter;
});
/**
* Creates a new manager with a default backing graph implementation.
* Use {@link FoldingManager#createFoldingView} to create a view that supports the {@link IFoldingView} interface
* for displaying and working with a collapsible graph structure.
*
* This constructor will set a new empty {@link DefaultGraph} as {@link FoldingManager#masterGraph}.
*
* <br><b>Setting up folding with a new graph instance</b>
* <pre>
* const manager = new FoldingManager()
* graphComponent.graph = manager.createFoldingView().graph
* </pre>
*
* <br><b>Setting up folding, reusing a given graph instance</b>
* <pre>
* const manager = new FoldingManager(graphComponent.graph)
* graphComponent.graph = manager.createFoldingView().graph
* </pre>
*
* @param [options] The parameters to pass.
* @param [options.foldingEdgeConverter] The {@link IFoldingEdgeConverter} implementation that is used to create/convert and modify the {@link FoldingEdgeStateId folding edges} inside the view instances. This option sets the {@link FoldingManager#foldingEdgeConverter} property on the created object.
* @param [options.folderNodeConverter] The {@link IFolderNodeConverter} implementation that is used to create/convert and modify the folder nodes inside the view instances. This option sets the {@link FoldingManager#folderNodeConverter} property on the created object.
* @see FoldingManager#createFoldingView
* @see FoldingManager#masterGraph
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23FoldingManager() Online Documentation}
*/
constructor(options?: {
foldingEdgeConverter?: IFoldingEdgeConverter;
folderNodeConverter?: IFolderNodeConverter;
});
/**
* Creates a separate view instance of the {@link FoldingManager#masterGraph} which is a synchronized copy of a subset of the items from the master graph.
* This method will create a new instance of an {@link IGraph} that is constructed as a synchronized copy of the {@link FoldingManager#masterGraph}.
* That instance can be obtained from the {@link IFoldingView#graph} property of the returned {@link IFoldingView} implementation. The graph
* instance will provide an {@link IFoldingView} instance in its {@link ILookup#lookup} mechanism:
*
* This instance can be used to gain access to this manager instance and the folding operations.
*
* Note that the entities that are contained in the returned graph instance are not the same instances as the original
* "master" entities in the {@link FoldingManager#masterGraph}. However they share the same properties (style, {@link ITagOwner#tag tag}, {@link ILabelModelParameter parameters}, and {@link INode#layout geometry}). For collapsed
* group nodes and view edges, there is a separate set of properties available for each possible state combination. The {@link FoldingManager#getFoldingEdgeState}
* method for edges and the {@link FoldingManager#getFolderNodeState} method for nodes provide access to that state
* while it is not being displayed in a view.
*
* <br><b>Create a folding enabled graph</b>
* <pre>
* // first create the manager
* const manager = new FoldingManager()
* // the manager serves as factory to create views
* const foldingView = manager.createFoldingView()
* // each view contains a folding-enabled graph: the view graph
* const viewGraph = foldingView.graph
* // the view graph is the graph which should actually be displayed
* graphComponent.graph = viewGraph
* </pre>
*
* If the {@link FoldingManager#masterGraph} already contains graph elements the created view and its view graph will contain these initially, too.
*
* <br><b>Create a folding enabled graph with initial elements</b>
* <pre>
* const manager = new FoldingManager(graphComponent.graph)
* graphComponent.graph = manager.createFoldingView().graph
* </pre>
*
* All group nodes are expanded by default. The <code>isExpanded</code> parameter allows for providing a predicate function which can
* decide for each node whether it is expanded or collapsed. The following example uses a predicate which returns <code>false</code> for
* all nodes. Therefore, all nodes in the view graph will be collapsed initially.
*
* <br><b>Create a folding enabled graph with all groups being collapsed initially</b>
* <pre>
* const manager = new FoldingManager(graphComponent.graph)
* graphComponent.graph = manager.createFoldingView({ isExpanded: node => false }).graph
* </pre>
*
* @param [root=null] A group node or the root of the {@link FoldingManager#masterGraph} whose contents will be displayed in the view.
* @param [isExpanded=null] A predicate that can be provided to the view that will be used as a callback to determine the initial {@link IFoldingView#isExpanded expansion state} of
* group nodes in the view.
* @returns A new graph instance that represents a synchronized view on the {@link FoldingManager#masterGraph}.
* @see FoldingManager#prepareFoldingView
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23createFoldingView(INode%2Cfunction(INode)%3Aboolean) Online Documentation}
*/
createFoldingView(root?: INode | null, isExpanded?: ((obj: INode) => boolean) | null): IFoldingView;
/**
* Creates a separate view instance of the {@link FoldingManager#masterGraph} which is a synchronized copy of a subset of the items from the master graph.
* This method will create a new instance of an {@link IGraph} that is constructed as a synchronized copy of the {@link FoldingManager#masterGraph}.
* That instance can be obtained from the {@link IFoldingView#graph} property of the returned {@link IFoldingView} implementation. The graph
* instance will provide an {@link IFoldingView} instance in its {@link ILookup#lookup} mechanism:
*
* This instance can be used to gain access to this manager instance and the folding operations.
*
* Note that the entities that are contained in the returned graph instance are not the same instances as the original
* "master" entities in the {@link FoldingManager#masterGraph}. However they share the same properties (style, {@link ITagOwner#tag tag}, {@link ILabelModelParameter parameters}, and {@link INode#layout geometry}). For collapsed
* group nodes and view edges, there is a separate set of properties available for each possible state combination. The {@link FoldingManager#getFoldingEdgeState}
* method for edges and the {@link FoldingManager#getFolderNodeState} method for nodes provide access to that state
* while it is not being displayed in a view.
*
* <br><b>Create a folding enabled graph</b>
* <pre>
* // first create the manager
* const manager = new FoldingManager()
* // the manager serves as factory to create views
* const foldingView = manager.createFoldingView()
* // each view contains a folding-enabled graph: the view graph
* const viewGraph = foldingView.graph
* // the view graph is the graph which should actually be displayed
* graphComponent.graph = viewGraph
* </pre>
*
* If the {@link FoldingManager#masterGraph} already contains graph elements the created view and its view graph will contain these initially, too.
*
* <br><b>Create a folding enabled graph with initial elements</b>
* <pre>
* const manager = new FoldingManager(graphComponent.graph)
* graphComponent.graph = manager.createFoldingView().graph
* </pre>
*
* All group nodes are expanded by default. The <code>isExpanded</code> parameter allows for providing a predicate function which can
* decide for each node whether it is expanded or collapsed. The following example uses a predicate which returns <code>false</code> for
* all nodes. Therefore, all nodes in the view graph will be collapsed initially.
*
* <br><b>Create a folding enabled graph with all groups being collapsed initially</b>
* <pre>
* const manager = new FoldingManager(graphComponent.graph)
* graphComponent.graph = manager.createFoldingView({ isExpanded: node => false }).graph
* </pre>
*
* @param options The parameters to pass.
* @param [options.root=null] A group node or the root of the {@link FoldingManager#masterGraph} whose contents will be displayed in the view.
* @param [options.isExpanded=null] A predicate that can be provided to the view that will be used as a callback to determine the initial {@link IFoldingView#isExpanded expansion state} of
* group nodes in the view.
* @returns A new graph instance that represents a synchronized view on the {@link FoldingManager#masterGraph}.
* @see FoldingManager#prepareFoldingView
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23createFoldingView(INode%2Cfunction(INode)%3Aboolean) Online Documentation}
*/
createFoldingView(options: {
root?: INode | null;
isExpanded?: ((obj: INode) => boolean) | null;
}): IFoldingView;
/**
Disposes of this instance.
This will disconnect the {@link FoldingManager#masterGraph} from the previously created {@link FoldingManager#createFoldingView managed views}. Also each view will be {@link IFoldingView#dispose disposed}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23dispose() Online Documentation}*/
dispose(): void;
/**
* Provides access to the folder node state of a node.
* This method can be used to query the collapsed state of a node, even if the node is currently not present in any managed
* view or the node is in fact expanded in all views.
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* state.style = new PanelNodeStyle()
* </pre>
*
* @param masterNode The node in the {@link FoldingManager#masterGraph} for which the state should be returned.
* @returns A state holder implementation of a node, which is not part of any graph.
* @throws {Exception} <b>(ArgumentError)</b> The master node does not belong to the managed graph.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23getFolderNodeState(INode) Online Documentation}
*/
getFolderNodeState(masterNode: INode): FolderNodeState;
/**
* Return a data container implementation that is not part of any actual {@link IGraph} that describes the state of
* the edge for the given {@link FoldingEdgeStateId}.
* This method can be used to query the state of an edge that is either not currently visible in any {@link FoldingManager#views view} or connects to
* different source and target nodes because of the {@link IFoldingView#collapse collapsed} states of the nodes and their parents in the view.
*
* <pre>
* const state = manager.getFoldingEdgeState(
*   new FoldingEdgeStateId(masterEdge, masterSource, true, masterTarget, false)
* )
* state.style = new PolylineEdgeStyle({ stroke: Stroke.RED })
* </pre>
*
* <pre>
* const state = manager.getFoldingEdgeState(
*   new FoldingEdgeStateId(masterEdge, masterSource, true, masterTarget, false)
* )
* const portState = state.sourcePort
* portState.locationParameter = FreeNodePortLocationModel.NODE_RIGHT_ANCHORED
* </pre>
*
* @param id The ID of the folding edge for which the folding state should be returned.
* @returns A {@link FoldingEdgeState} that can be used to query and change the properties of the edge.
* @throws {Exception} <b>(ArgumentError)</b> if the <code>id</code> doesn't represent a dummy edge state which is managed by this manager.
* @see FoldingManager#getFolderNodeState
* @see FoldingEdgeStateId
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23getFoldingEdgeState(FoldingEdgeStateId) Online Documentation}
*/
getFoldingEdgeState(id: FoldingEdgeStateId): FoldingEdgeState;
/**
Determines whether for the given <code>masterNode</code> this manager has stored associated folder node state.
The manager does not necessarily have folder node state for folder nodes if that specific state has never been requested
by a {@link IFoldingView view} because it has never been visualized. This method indicates whether there is state information available for
the given master node.
@param masterNode The master node for which the state is requested.
@returns Whether {@link FoldingManager#getFolderNodeState} would return existing state information.
@see FoldingManager#getFolderNodeState
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23hasFolderNodeState(INode) Online Documentation}*/
hasFolderNodeState(masterNode: INode): boolean;
/**
Determines whether for the given {@link FoldingEdgeStateId} this manager has stored associated view local state.
The manager does not necessarily have a {@link FoldingEdgeState} for folding elements if that specific state has
never been requested by a {@link IFoldingView} because it has never been visualized. This method indicates whether
there is state information available for the given id.
@param id The id for which state is requested.
@returns Whether {@link FoldingManager#getFoldingEdgeState} would return existing state information.
@see FoldingManager#getFoldingEdgeState
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23hasFoldingEdgeState(FoldingEdgeStateId) Online Documentation}*/
hasFoldingEdgeState(id: FoldingEdgeStateId): boolean;
/**
Can be overridden by subclasses to prepare the {@link IFoldingView} instance right before it will be initialized
with a copy of the elements in the initial graph.
Subclasses could override this method to register additional {@link IMapper} instances with the graph instance or add
{@link IContextLookupChainLink}s via the {@link ILookupDecorator} of the {@link IFoldingView#graph} to the instance if they are
essential for the initial creation of the elements in the view instance. Overriding methods should always consider to
call the base class implementation, first.
@param foldingView The graph view that has just been created but is still empty.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23prepareFoldingView(IFoldingView) Online Documentation}*/
prepareFoldingView(foldingView: IFoldingView): void;
/**
Triggers a call to {@link IFolderNodeConverter#updateFolderNodeState} for all view nodes that {@link IFoldingView#getViewItem represent} the given <code>masterNode</code>
in its collapsed state.
In all {@link FoldingManager#views} that are currrently showing the node in collapsed state, the {@link FoldingManager#getFolderNodeState view-local state} will be updated via a call to the
{@link IFolderNodeConverter}'s {@link IFolderNodeConverter#updateFolderNodeState} method.
Note that all {@link FoldingManager#views} share the same {@link FolderNodeState} for the <code>masterNode</code>.
This method is useful in situations where a change in the data on the master graph did not automatically trigger a call
to the converter. Using this method, the state of the collapsed nodes in the views can be synchronized according to the
logic in the {@link IFolderNodeConverter} implementation.
@param masterNode The master node that belongs to the {@link FoldingManager#masterGraph}.
@throws {Exception} <b>(ArgumentError)</b> If the provided master node does not belong to the master graph.
@see FoldingManager#updateFoldingEdgeStates
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23updateFolderNodeStates(INode) Online Documentation}*/
updateFolderNodeStates(masterNode: INode): void;
/**
Triggers a call to {@link IFoldingEdgeConverter#updateFoldingEdgeState} for all folder edges that {@link IFoldingView#getViewItem represent} the
given <code>masterEdge</code>.
In all {@link FoldingManager#views} that are currrently showing the view edge for the provided master edge, the {@link FoldingManager#getFoldingEdgeState view-local state} will be updated via
a call to the {@link IFoldingEdgeConverter}'s {@link IFoldingEdgeConverter#updateFoldingEdgeState} method.
This method is useful in situations where a change in the data on the master graph did not automatically trigger a call
to the converter. Using this method, the state of the representing edges in the views can be synchronized according to
the logic in the {@link IFoldingEdgeConverter} implementation.
@param masterEdge The master edge that belongs to the {@link FoldingManager#masterGraph}.
@throws {Exception} <b>(ArgumentError)</b> If the provided master edge does not belong to the master graph.
@see FoldingManager#updateFolderNodeStates
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23updateFoldingEdgeStates(IEdge) Online Documentation}*/
updateFoldingEdgeStates(masterEdge: IEdge): void;
/**
* Gets or sets the {@link IFoldingEdgeConverter} implementation that is used to create/convert and modify the {@link FoldingEdgeStateId folding edges}
* inside the view instances.
* The converter instance will be used to {@link IFoldingEdgeConverter#initializeFoldingEdgeState create the initial} and {@link IFoldingEdgeConverter#updateFoldingEdgeState change} the appearance of the folding edges as soon as they
* appear in a view.
*
* <pre>
* manager.foldingEdgeConverter = new MergingFoldingEdgeConverter({
*   copyFirstLabel: true,
*   foldingEdgeStyle: new PolylineEdgeStyle({ stroke: Stroke.GRAY })
* })
* </pre>
*
* @throws {Exception} <b>(ArgumentError)</b> <code>value</code> is null.
* @see DefaultFoldingEdgeConverter
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23foldingEdgeConverter Online Documentation}
*/
foldingEdgeConverter: IFoldingEdgeConverter;
/**
* Gets or sets the {@link IFolderNodeConverter} implementation that is used to create/convert and modify the folder
* nodes inside the view instances.
* The converter instance will be used to {@link IFolderNodeConverter#initializeFolderNodeState create the initial} and {@link IFolderNodeConverter#updateFolderNodeState change} the appearance of the folder nodes as soon as they
* appear in a view.
*
* <pre>
* manager.folderNodeConverter = new DefaultFolderNodeConverter({
*   copyFirstLabel: true,
*   folderNodeStyle: new ShapeNodeStyle({ fill: Fill.GRAY })
* })
* </pre>
*
* @see DefaultFolderNodeConverter
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23folderNodeConverter Online Documentation}
*/
folderNodeConverter: IFolderNodeConverter;
/**
Gets the master {@link IGraph} instance that holds the complete model in expanded group node state.
This graph contains all non-folding edges and expanded group nodes, no folding edges and no collapsed group nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23masterGraph Online Documentation}*/
readonly masterGraph: IGraph;
/**
Gets all currently known {@link FoldingManager#createFoldingView view instances} that are managed by this instance.
@see IFoldingView#graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingManager%23views Online Documentation}*/
readonly views: IEnumerable<IFoldingView>;
static $class: Class;
private $_FoldingManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FoldingManager;
}
export interface FoldingBendState extends YObject {
}
/**
A data container representing the state of a bend of a {@link FoldingEdgeState}.
When the properties of the state are changed, these changes are reflected immediately in the {@link FoldingManager#views}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingBendState Online Documentation}*/
export class FoldingBendState {
/**
Returns an {@link IBend} instance which represents this bend state.
The returned bend is not present in any graph but may be passed to {@link IFoldingView#getViewItem} to get the bend
state's view bend in the {@link IFoldingView#graph folding view graph}.
@returns An {@link IBend} instance which represents this bend state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingBendState%23asBend() Online Documentation}*/
asBend(): IBend;
/**
Gets or sets the location of the bend.
@throws {Exception} <b>(ArgumentError)</b> <code>value</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingBendState%23location Online Documentation}*/
location: Point;
/**
Gets or sets the tag of the bend.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingBendState%23tag Online Documentation}*/
tag: any | null;
/**
Gets the bend's index in the {@link FoldingEdgeState#bends} of its owner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingBendState%23index Online Documentation}*/
readonly index: number;
static $class: Class;
private $_FoldingBendState: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FoldingBendState;
}
export interface FoldingEdgeState extends FoldingLabelOwnerState {
/**
@inheritdoc*/
addLabel(text: string, layoutParameter: ILabelModelParameter, style: ILabelStyle, preferredSize: Size | SizeConvertible, tag: any | null): FoldingLabelState;
} /**
* A data container representing the state of an edge whose source and/or target node are collapsed.
* When the properties of the state are changed, these changes are reflected immediately in the {@link FoldingManager#views}.
*
* <pre>
* const state = manager.getFoldingEdgeState(
*   new FoldingEdgeStateId(masterEdge, masterSource, true, masterTarget, false)
* )
* state.style = new PolylineEdgeStyle({ stroke: Stroke.RED })
* </pre>
*
* <pre>
* const state = manager.getFoldingEdgeState(
*   new FoldingEdgeStateId(masterEdge, masterSource, true, masterTarget, false)
* )
* const portState = state.sourcePort
* portState.locationParameter = FreeNodePortLocationModel.NODE_RIGHT_ANCHORED
* </pre>
*
* @see FoldingEdgeStateId
* @see FoldingManager#getFoldingEdgeState
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState Online Documentation}
*/
export class FoldingEdgeState {
/**
Adds a new {@link FoldingBendState} to this edge state at the given <code>index</code> and <code>location</code>.
@param location The location of the new bend state.
@param [index=-1] The index the bend state shall be inserted in the
{@link FoldingEdgeState#bends}; a negative value (which is the default) indicates that the bend should be appended to the end of the list of bends.
@returns The newly added bend state.
@throws {Exception} <b>(ArgumentError)</b> <code>location</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23addBend(Point%2Cnumber) Online Documentation}*/
addBend(location: Point | PointConvertible, index?: number): FoldingBendState;
/**
Returns an {@link IEdge} instance which represents this edge state.
The returned edge is not present in any graph but may be passed to {@link IFoldingView#getViewItem} to get the edge
state's view edge in the {@link IFoldingView#graph folding view graph}.
@returns An {@link IEdge} instance which represents this edge state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23asEdge() Online Documentation}*/
asEdge(): IEdge;
/**
Removes all {@link FoldingEdgeState#bends}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23clearBends() Online Documentation}*/
clearBends(): void;
/**
Removes the <code>label</code> from its owning {@link FoldingLabelOwnerState}.
@param label The label to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23remove(FoldingLabelState) Online Documentation}*/
remove(label: FoldingLabelState): void;
/**
Removes the <code>bend</code> from this edge state.
@param bend The bend to remove.
@throws {Exception} <b>(ArgumentError)</b> Thrown if the <code>bend</code> doesn't belong to this edge state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23remove(FoldingBendState) Online Documentation}*/
remove(bend: FoldingBendState): void;
/**
Gets or sets the tag of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23tag Online Documentation}*/
tag: any | null;
/**
Gets the {@link FoldingBendState}s of this state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23bends Online Documentation}*/
readonly bends: IListEnumerable<FoldingBendState>;
/**
Gets the {@link FoldingPortState} which represents the source port of this edge state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23sourcePort Online Documentation}*/
readonly sourcePort: FoldingPortState;
/**
Gets the {@link FoldingPortState} which represents the target port of this edge state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23targetPort Online Documentation}*/
readonly targetPort: FoldingPortState;
/**
Gets or sets the style of this edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingEdgeState%23style Online Documentation}*/
style: IEdgeStyle;
static $class: Class;
private $_FoldingEdgeState: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FoldingEdgeState;
}
export interface FoldingLabelState extends YObject {
}
/**
* A data container representing the state of a label at a {@link FolderNodeState} or {@link FoldingEdgeState}.
* When the properties of the state are changed, these changes are reflected immediately in the {@link FoldingManager#views}.
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const labelState = state.labels.get(0)
* labelState.text = 'New Label Text'
* </pre>
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const style = new DefaultLabelStyle()
* state.addLabel('Folder Label', InteriorLabelModel.NORTH_EAST, style, new Size(20, 10), null)
* </pre>
*
* @see FoldingLabelOwnerState#labels
* @see FoldingLabelOwnerState#addLabel
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelState Online Documentation}
*/
export class FoldingLabelState {
/**
Returns an {@link ILabel} instance which represents this label state.
The returned label is not present in any graph but may be passed to {@link IFoldingView#getViewItem} to get the
label state's view label in the {@link IFoldingView#graph folding view graph}.
@returns An {@link ILabel} instance which represents this label state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelState%23asLabel() Online Documentation}*/
asLabel(): ILabel;
/**
Gets or sets the style of this label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelState%23style Online Documentation}*/
style: ILabelStyle;
/**
Gets or sets the preferred size of the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelState%23preferredSize Online Documentation}*/
preferredSize: Size;
/**
Gets or sets the label's text.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelState%23text Online Documentation}*/
text: string;
/**
Gets or sets the {@link ILabelModelParameter} used to describe the label's layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelState%23layoutParameter Online Documentation}*/
layoutParameter: ILabelModelParameter;
/**
Gets or sets the tag of the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelState%23tag Online Documentation}*/
tag: any | null;
static $class: Class;
private $_FoldingLabelState: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FoldingLabelState;
}
export interface FoldingLabelOwnerState extends YObject {
}
/**
* An abstract base class for data containers representing the state of model items which can own labels.
* Developers should not extend this class. Only instances of its subclasses {@link FolderNodeState} and {@link FoldingEdgeState}
* are supported.
*
* When the properties of the state are changed, these changes are reflected immediately in the {@link FoldingManager#views}.
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const labelState = state.labels.get(0)
* labelState.text = 'New Label Text'
* </pre>
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const style = new DefaultLabelStyle()
* state.addLabel('Folder Label', InteriorLabelModel.NORTH_EAST, style, new Size(20, 10), null)
* </pre>
*
* @see FolderNodeState
* @see FoldingEdgeState
* @see FoldingLabelState
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelOwnerState Online Documentation}
*/
export abstract class FoldingLabelOwnerState {
constructor();
/**
* Adds a {@link FoldingLabelState} to this state.
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const style = new DefaultLabelStyle()
* state.addLabel('Folder Label', InteriorLabelModel.NORTH_EAST, style, new Size(20, 10), null)
* </pre>
*
* @param text The text of the new label.
* @param layoutParameter The layout parameter of the new label.
* @param style The style of the new label.
* @param preferredSize The preferred size of the label.
* @param tag The tag of the label.
* @returns The newly added {@link FoldingLabelState}.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelOwnerState%23addLabel(string%2CILabelModelParameter%2CILabelStyle%2CSize%2CObject) Online Documentation}
*/
abstract addLabel(text: string, layoutParameter: ILabelModelParameter, style: ILabelStyle, preferredSize: Size | SizeConvertible, tag: any | null): FoldingLabelState;
/**
Removes all {@link FoldingLabelOwnerState#labels}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelOwnerState%23clearLabels() Online Documentation}*/
clearLabels(): void;
/**
* Gets the {@link FoldingLabelState}s owned by this state.
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const labelState = state.labels.get(0)
* labelState.text = 'New Label Text'
* </pre>
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const labelState = state.labels.get(0)
* labelState.text = 'New Label Text'
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingLabelOwnerState%23labels Online Documentation}
*/
readonly labels: IListEnumerable<FoldingLabelState>;
static $class: Class;
private $_FoldingLabelOwnerState: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FoldingLabelOwnerState;
}
export interface FoldingPortState extends FoldingLabelOwnerState {
}
/**
* A data container representing the state of a port at a {@link FolderNodeState} or {@link FoldingEdgeState}.
* When the properties of the state are changed, these changes are reflected immediately in the {@link FoldingManager#views}.
*
* Ports at folder nodes should be retrieved from the {@link FolderNodeState#ports} collection of its owner.
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const portState = state.ports.get(0)
* portState.locationParameter = FreeNodePortLocationModel.NODE_TOP_ANCHORED
* </pre>
*
* The source and target ports of an edge should be retrieved from the {@link FoldingEdgeState#sourcePort} and {@link FoldingEdgeState#targetPort} properties of the edge.
*
* <pre>
* const state = manager.getFoldingEdgeState(
*   new FoldingEdgeStateId(masterEdge, masterSource, true, masterTarget, false)
* )
* const portState = state.sourcePort
* portState.locationParameter = FreeNodePortLocationModel.NODE_RIGHT_ANCHORED
* </pre>
*
* @see FoldingEdgeState#sourcePort
* @see FoldingEdgeState#targetPort
* @see FolderNodeState#ports
* @see FolderNodeState#getFoldingPortState
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingPortState Online Documentation}
*/
export abstract class FoldingPortState {
/**
Returns an {@link IPort} instance which represents this port state.
The returned port is not present in any graph but may be passed to {@link IFoldingView#getViewItem} to get the port
state's view port in the {@link IFoldingView#graph folding view graph}.
@returns An {@link IPort} instance which represents this port state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingPortState%23asPort() Online Documentation}*/
abstract asPort(): IPort;
/**
Gets or sets the style of this port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingPortState%23style Online Documentation}*/
style: IPortStyle;
/**
Gets or sets the {@link IPortLocationModelParameter} describing the position of this port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingPortState%23locationParameter Online Documentation}*/
locationParameter: IPortLocationModelParameter;
/**
Gets or sets the tag of the port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FoldingPortState%23tag Online Documentation}*/
abstract tag: any | null;
static $class: Class;
private $_FoldingPortState: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FoldingPortState;
}
export interface FolderNodeState extends FoldingLabelOwnerState {
/**
@inheritdoc*/
addLabel(text: string, layoutParameter: ILabelModelParameter, style: ILabelStyle, preferredSize: Size | SizeConvertible, tag: any | null): FoldingLabelState;
} /**
* A data container representing the collapsed state of a node.
* When the properties of the state are changed, these changes are reflected immediately in the {@link FoldingManager#views}.
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* state.style = new PanelNodeStyle()
* </pre>
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const labelState = state.labels.get(0)
* labelState.text = 'New Label Text'
* </pre>
*
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const style = new DefaultLabelStyle()
* state.addLabel('Folder Label', InteriorLabelModel.NORTH_EAST, style, new Size(20, 10), null)
* </pre>
*
* @see FoldingManager#getFolderNodeState
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FolderNodeState Online Documentation}
*/
export class FolderNodeState {
/**
Returns an {@link INode} instance which represents this folder node state.
The returned node is not present in any graph but may be passed to {@link IFoldingView#getViewItem} to get the node
state's folder node in the {@link IFoldingView#graph folding view graph}.
@returns An {@link INode} instance which represents this folder node state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FolderNodeState%23asNode() Online Documentation}*/
asNode(): INode;
/**
Provides access to the folding state of a master port.
This method can be used to query the folding state of a port at the folder which is represented by this state.
@param masterPort The master port for which the state should be returned.
@returns A state holder implementation of the <code>masterPort</code>, which reflects the port in the node's collapsed state.
@see FoldingPortState#style
@see FoldingPortState#locationParameter
@see {@link https://docs.yworks.com/yfileshtml/#/api/FolderNodeState%23getFoldingPortState(IPort) Online Documentation}*/
getFoldingPortState(masterPort: IPort): FoldingPortState;
/**
Removes the <code>label</code> from its owning {@link FoldingLabelOwnerState}.
@param label The label to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FolderNodeState%23remove(FoldingLabelState) Online Documentation}*/
remove(label: FoldingLabelState): void;
/**
Gets or sets the layout of the node.
@throws {Exception} <b>(ArgumentError)</b> <code>value</code> contains one or more NaN values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FolderNodeState%23layout Online Documentation}*/
layout: IRectangle;
/**
* Gets or sets the style of this node.
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* state.style = new PanelNodeStyle()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FolderNodeState%23style Online Documentation}
*/
style: INodeStyle;
/**
Gets or sets the tag of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FolderNodeState%23tag Online Documentation}*/
tag: any | null;
/**
* Gets the {@link FoldingPortState}s owned by this state.
* <pre>
* const state = manager.getFolderNodeState(masterNode)
* const portState = state.ports.get(0)
* portState.locationParameter = FreeNodePortLocationModel.NODE_TOP_ANCHORED
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FolderNodeState%23ports Online Documentation}
*/
readonly ports: IListEnumerable<FoldingPortState>;
static $class: Class;
private $_FolderNodeState: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FolderNodeState;
}
export interface IFoldingEdgeConverter {
}
/**
* Interface used by {@link FoldingManager} and {@link IFoldingView} which is responsible for managing edges
* between folder nodes.
* Since {@link FoldingManager#createFoldingView} creates views that can potentially contain edges that connect to
* nodes which have a different set of attributes, these edges might need to have a separate set of attributes, too.
* Implementations of this interface are being called by the
* {@link IFoldingView}s to configure these kind of folding edges.
*
* <pre>
* manager.folderNodeConverter = new DefaultFolderNodeConverter({
*   copyFirstLabel: true,
*   folderNodeStyle: new ShapeNodeStyle({ fill: Fill.GRAY })
* })
* </pre>
*
* @see DefaultFoldingEdgeConverter
* @see FoldingManager#foldingEdgeConverter
* @see FoldingEdgeStateId
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingEdgeConverter Online Documentation}
*/
export abstract class IFoldingEdgeConverter {
/**
This method gets called by the {@link IFoldingView} implementation to determine whether a given edge in the {@link FoldingManager#masterGraph} should
be represented by a folding edge in the given view.
The implementation will be passed in a <code>factory</code> object that implements the {@link IFoldingEdgeFactory} interface,
which needs to be used by the implementation to communicate the result of the query. The implementation needs to either
call {@link IFoldingEdgeFactory#addAsSeparateEdge}, {@link IFoldingEdgeFactory#addToExistingFoldingEdge}, or {@link IFoldingEdgeFactory#excludeFoldingEdge}
and return the values that these implementation yields to the caller. The implementation can optionally query the {@link IFoldingEdgeFactory#getExistingFoldingEdges existing folding edges}
between the source and target node and decide whether to add the edge {@link IFoldingEdgeFactory#addToExistingFoldingEdge to the existing folding edge}.
@param factory The {@link IFoldingEdgeFactory} implementation that needs to be called in order to communicate the results of this
query.
@param foldingView The instance into which the folding edge is going to be inserted.
@param masterEdge The edge from the {@link FoldingManager#masterGraph} that needs to be represented by a folding edge. Note that you may not return this instance.
@param source The source node that belongs to the {@link IFoldingView} graph that will act as the local view node of the actual
source node.
@param sourceIsCollapsed Determines whether the <code>source</code> is currently a folder node.
@param target The target node that belongs to the {@link IFoldingView} graph that will act as the local view node of the actual
target node.
@param targetIsCollapsed Determines whether the <code>target</code> is currently a folder node.
@returns The edge as returned by {@link IFoldingEdgeFactory#addAsSeparateEdge}, or
{@link IFoldingEdgeFactory#addToExistingFoldingEdge}, or <code>null</code> if the edge will be {@link IFoldingEdgeFactory#excludeFoldingEdge excluded} from the view.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingEdgeConverter%23addFoldingEdge(IFoldingEdgeFactory%2CIFoldingView%2CIEdge%2CINode%2Cboolean%2CINode%2Cboolean) Online Documentation}*/
abstract addFoldingEdge(factory: IFoldingEdgeFactory, foldingView: IFoldingView, masterEdge: IEdge, source: INode, sourceIsCollapsed: boolean, target: INode, targetIsCollapsed: boolean): IEdge | null;
/**
Creates an implementation of the interface {@link IFoldingEdgeConverter} from the given definition.
The given object must contain exactly the abstract members of the {@link IFoldingEdgeConverter} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IFoldingEdgeConverter} interface.
@returns An instance of the {@link IFoldingEdgeConverter} interface based on the given definition.*/
static create(interfaceDefinition: {
addFoldingEdge(factory: IFoldingEdgeFactory, foldingView: IFoldingView, masterEdge: IEdge, source: INode, sourceIsCollapsed: boolean, target: INode, targetIsCollapsed: boolean): IEdge;
updateFoldingEdgeState(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
initializeFoldingEdgeState(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
}): IFoldingEdgeConverter;
/**
Triggered by the {@link IFoldingView view} to initially create the appearance of a folding edge.
Implementations may not use the {@link IFoldingView}'s {@link IFoldingView#graph} instance to modify the <code>foldingEdge</code>, because this would
create and enqueue undo events. Rather the properties provided by the <code>state</code> implementation must be set.
@param state The {@link FoldingEdgeState} of the edge to change.
@param foldingView The graph to which the folding edge belongs.
@param foldingEdge The folding edge for which the appearance shall be determined.
@param masterEdges The list of {@link IFoldingView#getMasterEdges master edges}, that the folding edge initially represents.
@see IFoldingEdgeConverter#updateFoldingEdgeState
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingEdgeConverter%23initializeFoldingEdgeState(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
abstract initializeFoldingEdgeState(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
/**
Triggered by the {@link IFoldingView view} to adjust the appearance of a folding edge
For instance, it can be used to reflect a state change in the {@link IFoldingView#getMasterEdges master edges} that make up that folding edge.
Implementations may not use the {@link IFoldingView}'s {@link IFoldingView#graph} instance to modify the <code>foldingEdge</code>, because this would
create and enqueue undo events. Rather the properties provided by the <code>state</code> implementation must be set.
@param state The {@link FoldingEdgeState} of the edge to change.
@param foldingView The graph to which the folding edge belongs.
@param foldingEdge The folding edge for which the appearance might need an update.
@param masterEdges The list of {@link IFoldingView#getMasterEdges master edges}, that the folding edge currently represents.
@see IFoldingEdgeConverter#initializeFoldingEdgeState
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingEdgeConverter%23updateFoldingEdgeState(FoldingEdgeState%2CIFoldingView%2CIEdge%2CIList%3CIEdge%3E) Online Documentation}*/
abstract updateFoldingEdgeState(state: FoldingEdgeState, foldingView: IFoldingView, foldingEdge: IEdge, masterEdges: IList<IEdge>): void;
static $class: Class;
private $_IFoldingEdgeConverter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IFoldingEdgeConverter;
}
export interface IFolderNodeConverter {
}
/**
* Interface used by {@link FoldingManager} and {@link IFoldingView} which is responsible for managing the
* appearance of folder nodes.
* Since {@link FoldingManager#createFoldingView} creates views that can potentially contain nodes which have a
* different set of attributes, these attributes need to be determined somehow. Implementations of this interface are being
* called by the {@link IFoldingView view}s to configure the appearance of these {@link IFoldingView#isInFoldingState folder nodes}.
*
* <pre>
* manager.folderNodeConverter = new DefaultFolderNodeConverter({
*   copyFirstLabel: true,
*   folderNodeStyle: new ShapeNodeStyle({ fill: Fill.GRAY })
* })
* </pre>
*
* @see DefaultFolderNodeConverter
* @see FoldingManager#folderNodeConverter
* @see IFoldingView#isInFoldingState
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IFolderNodeConverter Online Documentation}
*/
export abstract class IFolderNodeConverter {
/**
Creates an implementation of the interface {@link IFolderNodeConverter} from the given definition.
The given object must contain exactly the abstract members of the {@link IFolderNodeConverter} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IFolderNodeConverter} interface.
@returns An instance of the {@link IFolderNodeConverter} interface based on the given definition.*/
static create(interfaceDefinition: {
updateFolderNodeState(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
initializeFolderNodeState(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
}): IFolderNodeConverter;
/**
Called by {@link IFoldingView view} implementations to initially create the appearance of a {@link IFoldingView#isInFoldingState folder} node.
This method is called by the view to allow for initially creating the appearance of a local folder node in the view when
it is included in the view for the first time. This can be used, e.g. to initialize the labels or style properties with
the corresponding properties of the <code>masterNode</code>.
By default the <code>state</code> is an exact copy of the <code>masterNode</code>, including layout, style, labels, and ports.
Note that changing the appearance has to be done on <code>state</code>.
@param state The folder node state to initialize.
@param foldingView The graph instance for which the folder node has been created.
@param viewNode The folder node instance in the view
@param masterNode The node that represents the {@link IFoldingView#getMasterItem} of the viewstate to change.
@see IFolderNodeConverter#updateFolderNodeState
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFolderNodeConverter%23initializeFolderNodeState(FolderNodeState%2CIFoldingView%2CINode%2CINode) Online Documentation}*/
abstract initializeFolderNodeState(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
/**
Called by {@link IFoldingView view} implementations to change the appearance of a {@link IFoldingView#isInFoldingState folder} node.
This method is called by the view to allow for adjusting the appearance of a local folder node in the view when the {@link IFoldingView#getMasterItem master}
node for it has changed properties. This can be used, e.g. to synchronize the label or style properties with the
corresponding properties of the <code>masterNode</code>.
Note that changing the appearance has to be done on <code>state</code>.
@param state The folder node state to update.
@param foldingView The graph instance for which the folder node can be changed.
@param viewNode The folder node instance in the view
@param masterNode The node that represents the {@link IFoldingView#getMasterItem} of the viewstate to change.
@see IFolderNodeConverter#initializeFolderNodeState
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFolderNodeConverter%23updateFolderNodeState(FolderNodeState%2CIFoldingView%2CINode%2CINode) Online Documentation}*/
abstract updateFolderNodeState(state: FolderNodeState, foldingView: IFoldingView, viewNode: INode, masterNode: INode): void;
static $class: Class;
private $_IFolderNodeConverter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IFolderNodeConverter;
}
export interface IFoldingEdgeFactory {
}
/**
The interface of the factory that is passed to implementations of the {@link IFoldingEdgeConverter}'s {@link IFoldingEdgeConverter#addFoldingEdge}
method.
Implementors of the {@link IFoldingEdgeConverter} need to use this interface to tell the calling {@link IFoldingView}
implementation how to add the folding edge to the given view. They need to call exactly one of the following methods for
each factory:
<ul>
<li><b>{@link IFoldingEdgeFactory#addAsSeparateEdge}</b> – Call this to add a separate folding edge for the given master edge to the view. Pass the return value of that method to
the caller.
<li><b>{@link IFoldingEdgeFactory#addToExistingFoldingEdge}</b> – Call this to reuse a folding edge that is already part of the view as a representative for the master edge. Pass the
return value of that method to the caller.
<li><b>{@link IFoldingEdgeFactory#excludeFoldingEdge}</b> – Call this in order not to add a folding edge for the master edge to the view at all. Pass <code>null</code> to the caller.
</ul>
@see IFoldingEdgeConverter#addFoldingEdge
@see IFoldingEdgeConverter
@see FoldingManager#foldingEdgeConverter
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingEdgeFactory Online Documentation}*/
export abstract class IFoldingEdgeFactory {
/**
Tells the {@link IFoldingView} implementation to add the edge provided to the {@link IFoldingEdgeConverter#addFoldingEdge}
method as a separate folding edge to the view at the given ports.
This tells the view that there is currently no {@link IFoldingEdgeFactory#getExistingFoldingEdges existing folding edge} in the view to which the master edge should be
added to. Instead the view should add a separate folding edge that represents the master edge. Use the result of this
method call as the return value for the {@link IFoldingEdgeConverter#addFoldingEdge} call. Note that the returned
folding edge should not be modified by this code, instead the {@link IFoldingEdgeConverter#initializeFoldingEdgeState}
and {@link IFoldingEdgeConverter#updateFoldingEdgeState} callbacks need to be used to change the appearance of the
folding edge. Existing ports at the provided source and target nodes may be specified to further influence the folding
edge creation.
@param sourcePort The source port at the source node in the view to use or <code>null</code> if an extra {@link IFoldingView#isInFoldingState folding state} port should be added for the
edge to connect to at the source.
@param targetPort The target port at the target node in the view to use or <code>null</code> if an extra {@link IFoldingView#isInFoldingState folding state} port should be added for the
edge to connect to at the target.
@returns The folding edge that has been created in the view. Don't change the properties of the edge directly.
@see IFoldingEdgeConverter
@see IFoldingView#getViewItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingEdgeFactory%23addAsSeparateEdge(IPort%2CIPort) Online Documentation}*/
abstract addAsSeparateEdge(sourcePort: IPort | null, targetPort: IPort | null): IEdge;
/**
Tells the {@link IFoldingView} implementation to logically add the edge provided to the {@link IFoldingEdgeConverter#addFoldingEdge}
method to a folding edge that already exists in the view.
This tells the view that there is already an {@link IFoldingEdgeFactory#getExistingFoldingEdges existing folding edge} in the view to which the master edge should be added
logically added to. The given <code>foldingEdge</code> will have the master edge added to its collection of {@link IFoldingView#getMasterEdges master edges}. Note that
the returned folding edge should not be modified by this code, instead the {@link IFoldingEdgeConverter#updateFoldingEdgeState}
callback needs to be used to change the appearance of the folding edge. Implementations can use the {@link IFoldingEdgeFactory#getExistingFoldingEdges}
method to retrieve the current set of existing folding edges that interconnect the same source and target nodes. The
range of valid parameters is constrained to this collection.
@param foldingEdge One of the {@link IFoldingEdgeFactory#getExistingFoldingEdges existing folding edges} from the view which has been chosen to represent the master edge in this view.
@returns The folding edge that represents the edge in the view, which is the same as the <code>foldingEdge</code> parameter. Don't change the
properties of the edge directly.
@see IFoldingEdgeConverter
@see IFoldingEdgeFactory#getExistingFoldingEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingEdgeFactory%23addToExistingFoldingEdge(IEdge) Online Documentation}*/
abstract addToExistingFoldingEdge(foldingEdge: IEdge): IEdge;
/**
Creates an implementation of the interface {@link IFoldingEdgeFactory} from the given definition.
The given object must contain exactly the abstract members of the {@link IFoldingEdgeFactory} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IFoldingEdgeFactory} interface.
@returns An instance of the {@link IFoldingEdgeFactory} interface based on the given definition.*/
static create(interfaceDefinition: {
addAsSeparateEdge(sourcePort: IPort | null, targetPort: IPort | null): IEdge;
addToExistingFoldingEdge(foldingEdge: IEdge): IEdge;
excludeFoldingEdge(): IEdge;
getExistingFoldingEdges(ignoreDirection: boolean): IEnumerable<IEdge>;
}): IFoldingEdgeFactory;
/**
Tells the {@link IFoldingView} implementation not to add a folding edge for the edge provided to the {@link IFoldingEdgeConverter#addFoldingEdge}
method in this view.
This tells the view that there should not be a representation of the master edge in this view. If this method is called,
<code>null</code> should be returned as the result of a {@link IFoldingEdgeConverter#addFoldingEdge} call, which is the same as
the return value of this method. So it is possible to just pass the return value on to the caller.
@returns This method will always return <code>null</code>. This is for symmetry reasons to be easily exchangeable with the other methods.
@see IFoldingEdgeConverter
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingEdgeFactory%23excludeFoldingEdge() Online Documentation}*/
abstract excludeFoldingEdge(): IEdge | null;
/**
Helper methods that can be used by the implementation of the {@link IFoldingEdgeConverter#addFoldingEdge} method to
determine the folding edges that are currently part of the view and interconnect the same nodes as the folding edge
would, if it was included in the view.
This method will yield all {@link IFoldingView#isInFoldingState folding edges} that interconnect the same source and target node as that the folding edge
would, if it was included in this view. One of the items in the returned collection can be passed to the {@link IFoldingEdgeFactory#addToExistingFoldingEdge}
method.
@param ignoreDirection Whether the direction of the edges should be ignored so that reverse edges are included in the enumeration, too.
@returns An collection of currently existing folding edges in the view that interconnects the same source and target node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFoldingEdgeFactory%23getExistingFoldingEdges(boolean) Online Documentation}*/
abstract getExistingFoldingEdges(ignoreDirection: boolean): IEnumerable<IEdge>;
static $class: Class;
private $_IFoldingEdgeFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IFoldingEdgeFactory;
}
export interface GraphMLIOHandler extends YObject {
}
/**
Reads and writes {@link IGraph graphs} from/to the GraphML format.
For more information about the GraphML file format please refer to the <a href="http://graphml.graphdrawing.org/primer/graphml-primer.html" target="_blank">GraphML Primer</a>.
<b>Warning:</b> It is essential for custom class serialization to register both regular classes and {@link MarkupExtension}
classes via {@link GraphMLIOHandler#addXamlNamespaceMapping}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler Online Documentation}*/
export class GraphMLIOHandler {
/**
Creates a new instance of {@link GraphMLIOHandler}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler Online Documentation}*/
constructor();
/**
Register <code>graphData</code> as an input data target for GraphML data that is present at the graph level.
This method can optionally use a custom event handler for the actual deserialization.
This methods creates and configures the necessary handler itself.
Type parameter <code>TValue</code>: The type of the mapper values
@param dataType The type of the mapper values
@param keyDefinitionPredicate Predicate function to select the correct GraphML attribute
@param graphData Delegate that stores the graph data dynamically
@param [deserializationCallback=null] custom deserialization callback.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addGraphInputData(Class%2Cfunction(Element)%3Aboolean%2Cfunction(IGraph%2CTValue)%3Avoid%2Cfunction(Object%2CHandleDeserializationEventArgs)%3Avoid) Online Documentation}*/
addGraphInputData<TValue>(dataType: Class, keyDefinitionPredicate: (obj: Element) => boolean, graphData: (arg1: IGraph | null, arg2: TValue) => void, deserializationCallback?: ((sender: any | null, evt: HandleDeserializationEventArgs | null) => void) | null): void;
/**
Register <code>graphData</code> as an input data target for GraphML data that is present at the graph level.
This method can optionally use a custom event handler for the actual deserialization.
This methods creates and configures the necessary handler itself.
Type parameter <code>TValue</code>: The type of the mapper values
@param dataType The type of the mapper values
@param name The name of the attribute
@param [uri=null] The optional URI of the key definition to disambiguate keys.
@param graphData Delegate that stores the graph data dynamically
@param [deserializationCallback=null] custom deserialization callback.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addGraphInputData(Class%2Cstring%2Cfunction(IGraph%2CTValue)%3Avoid%2Cfunction(Object%2CHandleDeserializationEventArgs)%3Avoid%2Cstring) Online Documentation}*/
addGraphInputData<TValue>(dataType: Class, name: string | null, graphData: (arg1: IGraph | null, arg2: TValue) => void, deserializationCallback?: ((sender: any | null, evt: HandleDeserializationEventArgs | null) => void) | null, uri?: string | null): void;
/**
Register <code>graphData</code> as an input data target for GraphML data that is present at the graph level.
This method can optionally use a custom event handler for the actual deserialization.
This methods creates and configures the necessary handler itself.
Type parameter <code>TValue</code>: The type of the mapper values
@param options The parameters to pass.
@param options.dataType The type of the mapper values
@param options.name The name of the attribute
@param [options.uri=null] The optional URI of the key definition to disambiguate keys.
@param options.graphData Delegate that stores the graph data dynamically
@param [options.deserializationCallback=null] custom deserialization callback.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addGraphInputData(Class%2Cstring%2Cfunction(IGraph%2CTValue)%3Avoid%2Cfunction(Object%2CHandleDeserializationEventArgs)%3Avoid%2Cstring) Online Documentation}*/
addGraphInputData<TValue>(options: {
dataType: Class;
name: string | null;
graphData: (arg1: IGraph, arg2: TValue) => void;
deserializationCallback?: ((sender: any | null, evt: HandleDeserializationEventArgs) => void) | null;
uri?: string | null;
}): void;
/**
Register <code>graphData</code> as an output data source for data that is written at the graph level.
This method can optionally use a custom event handler for the actual serialization.
This methods creates and configures the necessary handler itself.
Type parameter <code>TValue</code>: The type of the mapper values
@param dataType The type of the mapper values
@param name The name of the attribute
@param [uri=null] The optional URI to add to the key definition to disambiguate keys.
@param graphData Delegate that retrieves the graph data dynamically
@param [serializationCallback=null] custom serialization callback.
@param [declaredKeyType=null] The key type to write in the declaration. If this is omitted or <code>null</code>, the type is inferred automatically from <code>TValue</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addGraphOutputData(Class%2Cstring%2Cfunction(IGraph)%3ATValue%2Cfunction(Object%2CHandleSerializationEventArgs)%3Avoid%2CKeyType%2Cstring) Online Documentation}*/
addGraphOutputData<TValue>(dataType: Class, name: string | null, graphData: (arg: IGraph) => TValue, serializationCallback?: ((sender: any | null, evt: HandleSerializationEventArgs | null) => void) | null, declaredKeyType?: KeyType | null, uri?: string | null): void;
/**
Register <code>graphData</code> as an output data source for data that is written at the graph level.
This method can optionally use a custom event handler for the actual serialization.
This methods creates and configures the necessary handler itself.
Type parameter <code>TValue</code>: The type of the mapper values
@param options The parameters to pass.
@param options.dataType The type of the mapper values
@param options.name The name of the attribute
@param [options.uri=null] The optional URI to add to the key definition to disambiguate keys.
@param options.graphData Delegate that retrieves the graph data dynamically
@param [options.serializationCallback=null] custom serialization callback.
@param [options.declaredKeyType=null] The key type to write in the declaration. If this is omitted or <code>null</code>, the type is inferred automatically from <code>TValue</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addGraphOutputData(Class%2Cstring%2Cfunction(IGraph)%3ATValue%2Cfunction(Object%2CHandleSerializationEventArgs)%3Avoid%2CKeyType%2Cstring) Online Documentation}*/
addGraphOutputData<TValue>(options: {
dataType: Class;
name: string | null;
graphData: (arg: IGraph) => TValue;
serializationCallback?: ((sender: any | null, evt: HandleSerializationEventArgs) => void) | null;
declaredKeyType?: KeyType | null;
uri?: string | null;
}): void;
/**
Register <code>graphData</code> as an output data source for data that is written at the graph level.
This method can optionally use a custom event handler for the actual serialization.
This methods creates and configures the necessary handler itself.
Type parameter <code>TValue</code>: The type of the mapper values
@param dataType The type of the mapper values
@param name The name of the attribute
@param [uri=null] The optional URI to add to the key definition to disambiguate keys.
@param graphData the data that is written at the top level
@param [serializationCallback=null] custom serialization callback.
@param [declaredKeyType=null] The key type to write in the declaration. If this is omitted or <code>null</code>, the type is inferred automatically from <code>TValue</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addGraphOutputData(Class%2Cstring%2CTValue%2Cfunction(Object%2CHandleSerializationEventArgs)%3Avoid%2CKeyType%2Cstring) Online Documentation}*/
addGraphOutputData<TValue>(dataType: Class, name: string | null, graphData: TValue, serializationCallback?: ((sender: any | null, evt: HandleSerializationEventArgs | null) => void) | null, declaredKeyType?: KeyType | null, uri?: string | null): void;
/**
Register <code>graphData</code> as an output data source for data that is written at the graph level.
This method can optionally use a custom event handler for the actual serialization.
This methods creates and configures the necessary handler itself.
Type parameter <code>TValue</code>: The type of the mapper values
@param options The parameters to pass.
@param options.dataType The type of the mapper values
@param options.name The name of the attribute
@param [options.uri=null] The optional URI to add to the key definition to disambiguate keys.
@param options.graphData the data that is written at the top level
@param [options.serializationCallback=null] custom serialization callback.
@param [options.declaredKeyType=null] The key type to write in the declaration. If this is omitted or <code>null</code>, the type is inferred automatically from <code>TValue</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addGraphOutputData(Class%2Cstring%2CTValue%2Cfunction(Object%2CHandleSerializationEventArgs)%3Avoid%2CKeyType%2Cstring) Online Documentation}*/
addGraphOutputData<TValue>(options: {
dataType: Class;
name: string | null;
graphData: TValue;
serializationCallback?: ((sender: any | null, evt: HandleSerializationEventArgs) => void) | null;
declaredKeyType?: KeyType | null;
uri?: string | null;
}): void;
/**
Create a {@link Future} instance that wraps an {@link IInputHandler} that is dynamically created by <code>factory</code>.
The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link Future#hasValue}.
The validity is determined by <code>keyDefinitionPredicate</code>. This methods creates and configures the necessary handler itself.
@param keyDefinitionPredicate The predicate that matches the key definition.
@param factory A factory to create {@link IInputHandler} instances.
@returns A {@link Future} instance that wraps an {@link IInputHandler} that actually parses the data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addInputHandlerFactory(function(Element)%3Aboolean%2CIGenericInputHandlerFactory) Online Documentation}*/
addInputHandlerFactory(keyDefinitionPredicate: (obj: Element) => boolean, factory: IGenericInputHandlerFactory): Future<IInputHandler>;
/**
Register a {@link IMapper} instance for use as an input data target.
This method uses a custom event handler for the actual deserialization.
This methods creates and configures the necessary handler itself.
Type parameter <code>TKey</code>: The type of the mapper keys
Type parameter <code>TData</code>: The type of the mapper values
@param dataType The type of the mapper values
@param keyType The type of the mapper keys
@param keyDefinitionPredicate The predicate function that will determine whether to create an {@link IInputHandler} for the given key
definition.
@param mapper the mapper that serves as data source
@param deserializationCallback custom deserialization callback.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addInputMapper(Class%2CClass%2Cfunction(Element)%3Aboolean%2CIMapper%3CTKey%2CTData%3E%2Cfunction(Object%2CHandleDeserializationEventArgs)%3Avoid) Online Documentation}*/
addInputMapper<TKey, TData>(keyType: Class, dataType: Class, keyDefinitionPredicate: (obj: Element) => boolean, mapper: IMapper<TKey, TData> | IMapperConvertible<TKey, TData>, deserializationCallback: ((sender: any | null, evt: HandleDeserializationEventArgs | null) => void) | null): void;
/**
Register a {@link IMapper} instance for use as an input data target.
This methods creates and configures the necessary handler itself.
Type parameter <code>TKey</code>: The type of the mapper keys
Type parameter <code>TData</code>: The type of the mapper values
@param dataType The type of the mapper values
@param keyType The type of the mapper keys
@param name The name of the attribute
@param mapper the mapper that serves as data source
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addInputMapper(Class%2CClass%2Cstring%2CIMapper%3CTKey%2CTData%3E) Online Documentation}*/
addInputMapper<TKey, TData>(keyType: Class, dataType: Class, name: string, mapper: IMapper<TKey, TData> | IMapperConvertible<TKey, TData>): void;
/**
Create a {@link Future} instance that wraps an {@link IMapper} for use as an input data target.
The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link Future#hasValue}
This methods creates and configures the necessary handler itself.
Type parameter <code>TKey</code>: The type of the mapper keys
Type parameter <code>TData</code>: The type of the mapper values
@param dataType The type of the mapper values
@param keyType The type of the mapper keys
@param deserializationCallback custom deserialization callback.
@param keyDefinitionPredicate The predicate function that will determine whether to create an {@link IInputHandler} for the given key
definition.
@returns A {@link Future} instance that wraps an {@link IMapper} that actually holds the data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addInputMapperFuture(Class%2CClass%2Cfunction(Element)%3Aboolean%2Cfunction(Object%2CHandleDeserializationEventArgs)%3Avoid) Online Documentation}*/
addInputMapperFuture<TKey, TData>(keyType: Class, dataType: Class, keyDefinitionPredicate: (obj: Element) => boolean, deserializationCallback: ((sender: any | null, evt: HandleDeserializationEventArgs | null) => void) | null): Future<IMapper<TKey, TData>>;
/**
Create a {@link Future} instance that wraps an {@link IMapper} for use as an input data target.
The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link Future#hasValue}
This methods creates and configures the necessary handler itself.
Type parameter <code>TKey</code>: The type of the mapper keys
Type parameter <code>TData</code>: The type of the mapper values
@param dataType The type of the mapper values
@param keyType The type of the mapper keys
@param name The name of the attribute
@returns A {@link Future} instance that wraps an {@link IMapper} that actually holds the data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addInputMapperFuture(Class%2CClass%2Cstring) Online Documentation}*/
addInputMapperFuture<TKey, TData>(keyType: Class, dataType: Class, name: string): Future<IMapper<TKey, TData>>;
/**
Adds a namespace to the header of a GraphML file.
@param namespaceURI The namespace URI
@param shortName The namespace prefix
@throws {Exception} <b>(ArgumentError)</b> If <code>namespaceURI</code> is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addNamespace(string%2Cstring) Online Documentation}*/
addNamespace(namespaceURI: string, shortName: string | null): void;
/**
Register a {@link IMapper} instance for use as an output data source.
This method uses a custom event handler for the actual serialization.
This methods creates and configures the necessary handler itself.
Type parameter <code>TModelItem</code>: The type of the mapper keys
Type parameter <code>TValue</code>: The type of the mapper values
@param dataType The type of the mapper values
@param modelItemType The type of the mapper keys
@param name The name of the attribute
@param uri The URI to add to the key definition to disambiguate keys.
@param mapper the mapper that serves as data source
@param serializationCallback custom serialization callback.
@param declaredKeyType The key type to write in the declaration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addOutputMapper(Class%2CClass%2Cstring%2Cstring%2CIMapper%3CTModelItem%2CTValue%3E%2Cfunction(Object%2CHandleSerializationEventArgs)%3Avoid%2CKeyType) Online Documentation}*/
addOutputMapper<TModelItem, TValue>(modelItemType: Class, dataType: Class, name: string | null, uri: string | null, mapper: IMapper<TModelItem, TValue> | IMapperConvertible<TModelItem, TValue>, serializationCallback: ((sender: any | null, evt: HandleSerializationEventArgs | null) => void) | null, declaredKeyType: KeyType | KeyTypeStringValues): void;
/**
Register a {@link IMapper} instance for use as an output data source
This methods creates and configures the necessary handler itself
Type parameter <code>TModelItem</code>: The type of the mapper keys
Type parameter <code>TValue</code>: The type of the mapper values
@param dataType The type of the mapper values
@param modelItemType The type of the mapper keys
@param name The name of the attribute
@param mapper the mapper that serves as data source
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addOutputMapper(Class%2CClass%2Cstring%2CIMapper%3CTModelItem%2CTValue%3E) Online Documentation}*/
addOutputMapper<TModelItem, TValue>(modelItemType: Class, dataType: Class, name: string | null, mapper: IMapper<TModelItem, TValue> | IMapperConvertible<TModelItem, TValue>): void;
/**
Create a {@link Future} instance that wraps an {@link IInputHandler} that stores the data in an {@link IMapper} that will be {@link GraphMLIOHandler#createMapper created} in the
graph's {@link IMapperRegistry}.
The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link Future#hasValue}.
This methods creates and configures the necessary handler itself and {@link GraphMLIOHandler#createMapper creates} an {@link IMapper} in the {@link IMapperRegistry}
with name <code>tag</code>.
Type parameter <code>TKey</code>.
Type parameter <code>TData</code>.
@param dataType
@param keyType
@param tag The name of the key in the {@link IMapperRegistry}.
@param keyDefinitionPredicate A predicate to determine whether the key definition is valid.
@param deserializationCallback The custom deserialization callback, can be <code>null</code> which will result in default deserialization.
@returns A {@link Future} instance that wraps an {@link IMapper} that actually holds the data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addRegistryInputMapper(Class%2CClass%2CObject%2Cfunction(Element)%3Aboolean%2Cfunction(Object%2CHandleDeserializationEventArgs)%3Avoid) Online Documentation}*/
addRegistryInputMapper<TKey, TData>(keyType: Class, dataType: Class, tag: any, keyDefinitionPredicate: (obj: Element) => boolean, deserializationCallback: ((sender: any | null, evt: HandleDeserializationEventArgs | null) => void) | null): Future<IMapper<TKey, TData>>;
/**
Create a {@link Future} instance that wraps an {@link IInputHandler} that stores the data in an {@link IMapper} that will be {@link GraphMLIOHandler#createMapper created} in the
graph's {@link IMapperRegistry}.
The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link Future#hasValue}.
The attribute must be registered under the attribute name <code>name</code>. This methods creates and configures the necessary
handler itself and {@link GraphMLIOHandler#createMapper creates} an {@link IMapper} in the {@link IMapperRegistry} with name <code>name</code>.
Type parameter <code>TKey</code>.
Type parameter <code>TData</code>.
@param dataType
@param keyType
@param name The name of the GraphML attribute and the key in the {@link IMapperRegistry}.
@returns A {@link Future} instance that wraps an {@link IMapper} that actually holds the data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addRegistryInputMapper(Class%2CClass%2Cstring) Online Documentation}*/
addRegistryInputMapper<TKey, TData>(keyType: Class, dataType: Class, name: string): Future<IMapper<TKey, TData>>;
/**
Use a registered {@link IMapper} instance as source for the given GraphML attribute.
Type parameter <code>TModelItem</code>.
Type parameter <code>TValue</code>.
@param valueType
@param modelItemType
@param name The GraphML name of the attribute
@param uri The URI to add to the key definition to disambiguate keys.
@param tag The tag name of the mapper in the {@link IMapperRegistry}
@param serializationCallback custom serialization callback.
@param declaredKeyType The key type to write in the declaration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addRegistryOutputMapper(Class%2CClass%2Cstring%2Cstring%2CObject%2Cfunction(Object%2CHandleSerializationEventArgs)%3Avoid%2CKeyType) Online Documentation}*/
addRegistryOutputMapper<TModelItem, TValue>(modelItemType: Class, valueType: Class, name: string | null, uri: string | null, tag: any, serializationCallback: ((sender: any | null, evt: HandleSerializationEventArgs | null) => void) | null, declaredKeyType: KeyType | KeyTypeStringValues): void;
/**
Use a registered {@link IMapper} instance as source for the given GraphML attribute.
Type parameter <code>TModelItem</code>.
Type parameter <code>TValue</code>.
@param valueType
@param modelItemType
@param name The GraphML name of the attribute
@param tag The tag name of the mapper in the {@link IMapperRegistry}
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addRegistryOutputMapper(Class%2CClass%2Cstring%2CObject) Online Documentation}*/
addRegistryOutputMapper<TModelItem, TValue>(modelItemType: Class, valueType: Class, name: string | null, tag: any): void;
/**
Use a registered {@link IMapper} instance as source for the given GraphML attribute.
The type arguments of the mapper are determined automatically.
@param name The GraphML name of the attribute
@param uri The optional URI of the attribute
@param tag The tag name of the mapper in the {@link IMapperRegistry}
@param serializationCallback An optional serialization handler
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addRegistryOutputMapper(string%2Cstring%2CObject%2Cfunction(Object%2CHandleSerializationEventArgs)%3Avoid) Online Documentation}*/
addRegistryOutputMapper(name: string | null, uri: string | null, tag: any, serializationCallback: ((sender: any | null, evt: HandleSerializationEventArgs | null) => void) | null): void;
/**
Use a registered {@link IMapper} instance as source for the given GraphML attribute.
The type arguments of the mapper are determined automatically.
@param name The GraphML name of the attribute
@param tag The tag name of the mapper in the {@link IMapperRegistry}
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addRegistryOutputMapper(string%2CObject) Online Documentation}*/
addRegistryOutputMapper(name: string | null, tag: any): void;
/**
Adds a schema location to the header of a GraphML file.
@param schemaNamespace The namespace URI for this schema location
@param schemaLocation The schema location
@throws {Exception} <b>(ArgumentError)</b> If <code>schemaNamespace</code> is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addSchemaLocation(string%2Cstring) Online Documentation}*/
addSchemaLocation(schemaNamespace: string, schemaLocation: string | null): void;
/**
Add a namespace mapping between an XML namespace and a JavaScript <code>module</code>.
The module is an object where the keys are XML tag names and the values are the types to be mapped.
This is used both for writing and parsing objects.
@param xmlNamespace The XML namespace that corresponds to <code>module</code>.
@param module The module that is mapped to the XML namespace. The keys are the names of the XML tags. The values are the types.
@see IXamlNameMapper
@see GraphMLIOHandler#createXamlNameMapper
@see GraphMLIOHandler#addXamlNamespaceMapping
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addXamlNamespaceMapping(string%2CObject) Online Documentation}*/
addXamlNamespaceMapping(xmlNamespace: string, module: any): void;
/**
Add a namespace mapping between a fully qualified XML name (consisting of <code>xmlNamespace</code> and <code>tagName</code>) and a type.
Either of <code>xmlNamespace</code> or <code>tagName</code> may be <code>null</code>, meaning that the default rules for determining the namespace and/or tag
name of <code>type</code> are used.
This is used both for writing and parsing objects.
@param xmlNamespace The XML namespace that corresponds to <code>type</code>. If <code>null</code>, the namespace will be determined based on the JavaScript module
that contains <code>type</code>
@param tagName The XML tag name that corresponds to <code>type</code>. If <code>null</code>, the name will be determined from the type name automatically.
@param type The type that is mapped to the fully qualified XML name.
@see IXamlNameMapper
@see GraphMLIOHandler#createXamlNameMapper
@see GraphMLIOHandler#addXamlNamespaceMapping
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23addXamlNamespaceMapping(string%2Cstring%2CClass) Online Documentation}*/
addXamlNamespaceMapping(xmlNamespace: string | null, tagName: string | null, type: Class): void;
/**
Configure the predefined handlers for the {@link GraphMLParser#addHandleDeserializationListener HandleDeserialization}
event.
Except for text content and primitive types, this implementation delegates all work to
{@link GraphMLIOHandler#handleXamlDeserialization}
This method is called for each parse process.
@param graph The graph that will be parsed.
@param parser The parser to configure.
@see GraphMLIOHandler#handlePrimitivesDeserialization
@see GraphMLIOHandler#handleXamlDeserialization
@see GraphMLIOHandler#handleTextNodeDeserialization
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureDeserializationHandlers(IGraph%2CGraphMLParser) Online Documentation}*/
configureDeserializationHandlers(graph: IGraph, parser: GraphMLParser): void;
/**
Configures the {@link GraphMLIOHandler#deserializationPropertyOverrides deserialization property overrides} for the given parser.
This method is called for each parse process.
@param parser The parser to set the deserialization properties.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureDeserializationPropertyOverrides(GraphMLParser) Online Documentation}*/
configureDeserializationPropertyOverrides(parser: GraphMLParser): void;
/**
Configures a given instance of {@link GraphMLParser} that can handle the given graph.
This method is called by {@link GraphMLIOHandler#createGraphMLParser}.
@param parser The parser to configure
@param graph The graph that should be filled by the parser
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureGraphMLParser(GraphMLParser%2CIGraph) Online Documentation}*/
configureGraphMLParser(parser: GraphMLParser, graph: IGraph): void;
/**
Configures a given {@link GraphMLWriter} that is used for writing <code>graph</code>.
This method is called by {@link GraphMLIOHandler#createGraphMLWriter}.
@param writer The writer to configure
@param graph The graph that will be written.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureGraphMLWriter(GraphMLWriter%2CIGraph) Online Documentation}*/
configureGraphMLWriter(writer: GraphMLWriter, graph: IGraph): void;
/**
Configures default event handlers for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event.
This implementation configures several input handlers for the standard yFiles for HTML GraphML attributes (styles,
folding state, labels, geometry).
This method is called for each parse process.
@param parser The parser that should be configured.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureInputHandlers(GraphMLParser) Online Documentation}*/
configureInputHandlers(parser: GraphMLParser): void;
/**
Configures <code>writer</code> to use the namespace declarations set with {@link GraphMLIOHandler#addNamespace}.
In addition to the namespace declarations provided with {@link GraphMLIOHandler#addNamespace}, the default {@link GraphMLWriter}
implementation also automatically places all namespace declarations that have occurred
This method is called for each write process.
@param writer The writer to configure.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureNamespaces(GraphMLWriter) Online Documentation}*/
configureNamespaces(writer: GraphMLWriter): void;
/**
Configures default event handlers for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event.
This implementation configures several output handlers for the standard yFiles GraphML attributes (styles, folding
state, labels, geometry).
This method is called for each write process.
@param graph The graph that will be written.
@param writer The writer that should be configured.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureOutputHandlers(IGraph%2CGraphMLWriter) Online Documentation}*/
configureOutputHandlers(graph: IGraph, writer: GraphMLWriter): void;
/**
Configures the lookup of the given parser.
This method is called for each parse process.
@param parser The parser to configure.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureParserLookup(GraphMLParser) Online Documentation}*/
configureParserLookup(parser: GraphMLParser): void;
/**
Configures <code>writer</code> to use the schema locations set with {@link GraphMLIOHandler#addSchemaLocation}.
This method is called for each write process.
@param writer The writer to configure.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureSchemaLocations(GraphMLWriter) Online Documentation}*/
configureSchemaLocations(writer: GraphMLWriter): void;
/**
Configures the predefined handlers for the {@link GraphMLWriter#addHandleSerializationListener HandleSerialization}
event.
This method is called for each write process.
@param graph The graph that will be written.
@param writer The writer to configure.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureSerializationHandlers(IGraph%2CGraphMLWriter) Online Documentation}*/
configureSerializationHandlers(graph: IGraph, writer: GraphMLWriter): void;
/**
Configure important serialization properties from properties of {@link GraphMLIOHandler}.
This method is called for each write process.
@param graph The graph to write.
@param writer The writer to configure.
@see GraphMLWriter#setSerializationProperty
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureSerializationProperties(IGraph%2CGraphMLWriter) Online Documentation}*/
configureSerializationProperties(graph: IGraph, writer: GraphMLWriter): void;
/**
Configures the {@link GraphMLIOHandler#serializationPropertyOverrides serialization property overrides} for the given writer.
This method is called for each write process.
@param writer The writer to set the serialization properties.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureSerializationPropertyOverrides(GraphMLWriter) Online Documentation}*/
configureSerializationPropertyOverrides(writer: GraphMLWriter): void;
/**
Configure the lookup map of <code>writer</code>.
This method is called for each write process.
@param graph The graph that will be written.
@param writer The writer to configure.
@see GraphMLWriter#setLookup
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureWriterLookup(IGraph%2CGraphMLWriter) Online Documentation}*/
configureWriterLookup(graph: IGraph, writer: GraphMLWriter): void;
/**
Configure <code>writer</code>.
This method is called for each write process.
This implementation does nothing.
@param writer The writer to configure.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23configureXMLWriter(IXmlWriter) Online Documentation}*/
configureXMLWriter(writer: IXmlWriter): void;
/**
Creates the default value serializer context and returns it.
This factory method can be used to provide custom {@link ValueSerializer}s in cases where no {@link GraphMLAttribute#valueSerializer} can be used.
@returns An implementation of the {@link IValueSerializerContext} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createDefaultValueSerializerContext() Online Documentation}*/
createDefaultValueSerializerContext(): IValueSerializerContext;
/**
Create an {@link IGraphElementFactory} instance that creates the graph elements for the <code>graph</code> instance.
This method is called for each parse process.
@param parser The parser to use.
@param graph The graph that is populated.
@returns An {@link IGraphElementFactory} instance that is configured for the given parameters.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createGraphElementFactory(GraphMLParser%2CIGraph) Online Documentation}*/
createGraphElementFactory(parser: GraphMLParser, graph: IGraph): IGraphElementFactory;
/**
Create a suitable instance of {@link GraphMLParser} that can handle the given graph.
This method is called for each parse process.
It delegates the configuration of the parser to {@link GraphMLIOHandler#configureGraphMLParser}.
@param graph The graph that should be filled by this instance
@returns a suitable instance of {@link GraphMLParser} that can handle the given graph
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createGraphMLParser(IGraph) Online Documentation}*/
createGraphMLParser(graph: IGraph): GraphMLParser;
/**
Create and configure a new {@link GraphMLWriter} that is used for writing <code>graph</code>.
This method is called for each write process. It delegates the configuration of the writer to
{@link GraphMLIOHandler#configureGraphMLWriter}.
@param graph The graph that will be written.
@returns A new {@link GraphMLWriter} instance that is configured for <code>graph</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createGraphMLWriter(IGraph) Online Documentation}*/
createGraphMLWriter(graph: IGraph): GraphMLWriter;
/**
Called by {@link GraphMLIOHandler#addRegistryInputMapper} and its overloads to creates the mapper instance.
This method creates the mapper instance, unless there is an existing mapper in the target graph with a matching type. In
this case the existing mapper is returned. In all other cases a new mapper will be created and returned. If a mapper is
already registered with the same <code>registryTag</code> but wrong types, it will be replaced.
Type parameter <code>TKey</code>: The type of the keys.
Type parameter <code>TData</code>: The type of the data.
@param dataType The type of the data.
@param keyType The type of the keys.
@param context
@param registryTag The tag for the mapper in the registry.
@returns The mapper instance to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createMapper(Class%2CClass%2CIParseContext%2CObject) Online Documentation}*/
createMapper<TKey, TData>(keyType: Class, dataType: Class, context: IParseContext, registryTag: any): IMapper<TKey, TData>;
/**
Creates an {@link IInputHandler} implementations that matches the provided <code>TKey</code> and <code>TData</code>.
This is a factory method that creates an instance of several predefined {@link IInputHandler} implementations
that match the provided <code>TKey</code> and <code>TData</code>.
Type parameter <code>TKey</code>: The type of the mapper keys
Type parameter <code>TData</code>: The type of the mapper entries
@param dataType The type of the mapper entries
@param modelItemType The type of the mapper keys
@param mapper The mapper to use.
@param deserializationCallback Optional deserialization callback for complex data.
@returns A suitable {@link MapperInputHandler} implementation
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createMapperInputHandler(Class%2CClass%2CIMapper%3CTKey%2CTData%3E%2Cfunction(Object%2CHandleDeserializationEventArgs)%3Avoid) Online Documentation}*/
static createMapperInputHandler<TKey, TData>(modelItemType: Class, dataType: Class, mapper: IMapper<TKey, TData> | null | IMapperConvertible<TKey, TData>, deserializationCallback: ((sender: any | null, evt: HandleDeserializationEventArgs | null) => void) | null): MapperInputHandler<TKey, TData>;
/**
Creates an {@link IOutputHandler} implementation that matches the provided <code>TKey</code> and <code>TData</code>.
This is a factory method that creates an instance of several predefined {@link IOutputHandler} implementations
that match the provided <code>TKey</code> and <code>TData</code>.
Type parameter <code>TKey</code>: The type of the mapper keys
Type parameter <code>TData</code>: The type of the mapper entries
@param dataType The type of the mapper entries
@param modelItemType The type of the mapper keys
@param name The name of the graphml attribute
@param mapper The mapper to use.
@param serializationCallback Optional serialization callback for complex data.
@returns A suitable {@link MapperOutputHandler} implementation
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createMapperOutputHandler(Class%2CClass%2Cstring%2CIMapper%3CTKey%2CTData%3E%2Cfunction(Object%2CHandleSerializationEventArgs)%3Avoid) Online Documentation}*/
static createMapperOutputHandler<TKey, TData>(modelItemType: Class, dataType: Class, name: string | null, mapper: IMapper<TKey, TData> | null | IMapperConvertible<TKey, TData>, serializationCallback: ((sender: any | null, evt: HandleSerializationEventArgs | null) => void) | null): MapperOutputHandler<TKey, TData>;
/**
Configure mappings from CLR to XML names.
This implementation scans all loaded assemblies for XmlnsDefinitionAttributes and returns a suitable instance of
{@link IXamlNameMapper}.
This method is called for each read or write process.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createXamlNameMapper() Online Documentation}*/
createXamlNameMapper(): IXamlNameMapper | null;
/**
Configure mappings from XML namespaces to prefixes.
This method is called for each write process.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createXamlPrefixMapper() Online Documentation}*/
createXamlPrefixMapper(): IXamlPrefixMapper;
/**
Factory method that creates and configures a suitable {@link IXmlWriter} implementation.
This method is called for each write process. The {@link IXmlWriter} implementation is used by default for all
low-level write operations.
@returns A suitable {@link IXmlWriter} instance for the serialization
@see IWriteContext#writer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23createXMLWriter() Online Documentation}*/
createXMLWriter(): IXmlWriter;
/**
Handle the serialization of arrays.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} is an array. This handler is registered by default.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleArraySerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleArraySerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handle the serialization for enums.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleEnumSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleEnumSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handler for the serialization of types that extend FrameworkTemplate.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleJSONDeserialization(Object%2CHandleDeserializationEventArgs) Online Documentation}*/
handleJSONDeserialization(sender: any | null, evt: HandleDeserializationEventArgs): void;
/**
Handle the serialization of plain JavaScript object that can be serialized to JSON.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} can be converted to JSON data. This serializer is only called from element property syntax and
is registered by default.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleJSONSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleJSONSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handle the serialization of {@link IEnumerable} instances.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} is an IEnumerable instance. This handler is registered by default.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleListSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleListSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handle the serialization of {@link MarkupExtension} convertible instances.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} can be converted to a {@link MarkupExtension} instance. This handler is registered by
default.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleMarkupExtensionBasedSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleMarkupExtensionBasedSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handle the serialization of <code>null</code> references.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} is a <code>null</code> reference. This handler is registered by default and is only used to write <code>null</code>
references in XML element syntax.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleNullSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleNullSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handle the serialization of primitive types, enums and strings.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handlePrimitivesDeserialization(Object%2CHandleDeserializationEventArgs) Online Documentation}*/
handlePrimitivesDeserialization(sender: any | null, evt: HandleDeserializationEventArgs): void;
/**
Handle the serialization of primitive types, enums and strings.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handlePrimitivesSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handlePrimitivesSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handle the serialization of {@link IReferenceHandle} instances.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} is a {@link IReferenceHandle} instance. This handler is registered by default and should
not be removed if {@link GraphMLIOHandler#writeSharedReferences} is enabled.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleReferenceHandleSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleReferenceHandleSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handle the serialization of instances that can be serialized through a {@link ValueSerializer}.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} can be converted to a string with a
{@link ValueSerializer}. This serializer is only called from element property syntax and is registered by
default.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleTextConvertibleSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleTextConvertibleSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handler for the {@link GraphMLParser#addHandleDeserializationListener HandleDeserialization} event.
@param source The source of the event.
@param evt The event arguments.
@see HandleDeserializationEventArgs
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleTextNodeDeserialization(Object%2CHandleDeserializationEventArgs) Online Documentation}*/
handleTextNodeDeserialization(source: any | null, evt: HandleDeserializationEventArgs): void;
/**
Handle the serialization of TypeExtensions.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} is an array. This handler is registered by default.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleTypeExtensionSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleTypeExtensionSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handle the serialization of Types.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} is an array. This handler is registered by default.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleTypeSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleTypeSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handle the serialization of <code>undefined</code> references.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} is an <code>undefined</code> reference. This handler is registered by default and is only used to write <code>undefined</code>
references in XML element syntax.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleUndefinedSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleUndefinedSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Handler for the {@link GraphMLParser#addHandleDeserializationListener HandleDeserialization} event.
@param source The source of the event.
@param evt The event arguments.
@see HandleDeserializationEventArgs
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleXamlDeserialization(Object%2CHandleDeserializationEventArgs) Online Documentation}*/
handleXamlDeserialization(source: any | null, evt: HandleDeserializationEventArgs): void;
/**
Handle the serialization of complex objects that are not handled by other callbacks.
This sets {@link HandleSerializationEventArgs#handled} to <code>true</code> iff {@link HandleSerializationEventArgs#item} is not a <code>null</code> reference. This handler is registered by default and uses {@link XamlSerializer}
to create XAML output.
@param sender The origin of the event.
@param evt The event handler arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23handleXamlSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
handleXamlSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Checks whether the <code>name</code> attribute in <code>definition</code> matches <code>name</code>
@param definition The key definition to check
@param name The name that should be matched
@returns <code>true</code> if the name attribute in <code>definition</code> matches <code>name</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23matchesName(Element%2Cstring) Online Documentation}*/
static matchesName(definition: Element, name: string | null): boolean;
/**
Checks whether the <code>scope</code> attribute in <code>definition</code> matches <code>scope</code>
@param definition The key definition to check
@param scope The scope that should be matched
@returns <code>true</code> if the scope attribute in <code>definition</code> matches <code>scope</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23matchesScope(Element%2CKeyScope) Online Documentation}*/
static matchesScope(definition: Element, scope: KeyScope | KeyScopeStringValues): boolean;
/**
Checks whether the <code>type</code> attribute in <code>definition</code> matches <code>type</code>
@param definition The key definition to check
@param type The type that should be matched
@returns <code>true</code> if the type attribute in <code>definition</code> matches <code>type</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23matchesType(Element%2CKeyType) Online Documentation}*/
static matchesType(definition: Element, type: KeyType | KeyTypeStringValues): boolean;
/**
Checks whether the <code>uri</code> attribute in <code>definition</code> matches <code>uri</code>
@param definition The key definition to check
@param uri The uri that should be matched
@returns <code>true</code> if the uri attribute in <code>definition</code> matches <code>uri</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23matchesUri(Element%2Cstring) Online Documentation}*/
static matchesUri(definition: Element, uri: string | null): boolean;
/**
Event handler for the {@link GraphMLParser#addHandleDeserializationListener HandleDeserialization} event.
This event handler is used to propagate the {@link GraphMLParser#addHandleDeserializationListener HandleDeserialization}
event to subscribers of
{@link GraphMLIOHandler#addHandleDeserializationListener HandleDeserialization}. This implementation
just calls {@link GraphMLIOHandler#onHandleDeserialization}, which in turn raises the {@link GraphMLIOHandler#addHandleDeserializationListener HandleDeserialization}
event.
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onGraphMLParserHandleDeserialization(Object%2CHandleDeserializationEventArgs) Online Documentation}*/
onGraphMLParserHandleDeserialization(sender: any, evt: HandleDeserializationEventArgs): void;
/**
Event handler for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers} event.
This event handler is used to propagate the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event to subscribers of {@link GraphMLIOHandler#addQueryInputHandlersListener QueryInputHandlers}.
This implementation just calls {@link GraphMLIOHandler#onQueryInputHandlers}, which in turn raises the {@link GraphMLIOHandler#addQueryInputHandlersListener QueryInputHandlers}
event.
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onGraphMLParserQueryInputHandlers(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
onGraphMLParserQueryInputHandlers(sender: any, evt: QueryInputHandlersEventArgs): void;
/**
Event handler for the {@link GraphMLWriter#addHandleSerializationListener HandleSerialization} event.
This event handler is used to propagate the {@link GraphMLWriter#addHandleSerializationListener HandleSerialization}
event to subscribers of {@link GraphMLIOHandler#addHandleSerializationListener HandleSerialization}.
This implementation just calls {@link GraphMLIOHandler#onHandleSerialization}, which in turn raises the {@link GraphMLIOHandler#addHandleSerializationListener HandleSerialization}
event.
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onGraphMLWriterHandleSerialization(Object%2CHandleSerializationEventArgs) Online Documentation}*/
onGraphMLWriterHandleSerialization(sender: any | null, evt: HandleSerializationEventArgs): void;
/**
Event handler for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers} event.
This event handler is used to propagate the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event to subscribers of {@link GraphMLIOHandler#addQueryOutputHandlersListener QueryOutputHandlers}.
This implementation just calls {@link GraphMLIOHandler#onQueryOutputHandlers}, which in turn raises the {@link GraphMLIOHandler#addQueryOutputHandlersListener QueryOutputHandlers}
event.
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onGraphMLWriterQueryOutputHandlers(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
onGraphMLWriterQueryOutputHandlers(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Raises the {@link GraphMLIOHandler#addHandleDeserializationListener HandleDeserialization} event.
@param evt The arguments for the {@link GraphMLIOHandler#addHandleDeserializationListener HandleDeserialization}
event.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onHandleDeserialization(HandleDeserializationEventArgs) Online Documentation}*/
onHandleDeserialization(evt: HandleDeserializationEventArgs): void;
/**
Raises the {@link GraphMLIOHandler#addHandleSerializationListener HandleSerialization} event.
@param evt The arguments for the {@link GraphMLIOHandler#addHandleSerializationListener HandleSerialization}
event.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onHandleSerialization(HandleSerializationEventArgs) Online Documentation}*/
onHandleSerialization(evt: HandleSerializationEventArgs): void;
/**
Fires the {@link GraphMLIOHandler#addOverrideResolveReferenceListener OverrideResolveReference} event.
@param evt The arguments that get passed to the {@link GraphMLIOHandler#addOverrideResolveReferenceListener OverrideResolveReference}
event.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onOverrideResolveReference(ResolveReferenceEventArgs) Online Documentation}*/
onOverrideResolveReference(evt: ResolveReferenceEventArgs): void;
/**
Raises the {@link GraphMLIOHandler#addParsedListener Parsed} event.
This method is called when the document has been parsed and after {@link IParseEvents#addDocumentParsedListener DocumentParsed}
is called.
@param evt The {@link ParseEventArgs} instance containing the event data.
@see GraphMLIOHandler#addParsedListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onParsed(ParseEventArgs) Online Documentation}*/
onParsed(evt: ParseEventArgs): void;
/**
Raises the {@link GraphMLIOHandler#addParsingListener Parsing} event.
This method is called just before the parsing starts and before {@link IParseEvents#addDocumentParsingListener DocumentParsing}
is called.
@param evt The {@link ParseEventArgs} instance containing the event data.
@see GraphMLIOHandler#addParsingListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onParsing(ParseEventArgs) Online Documentation}*/
onParsing(evt: ParseEventArgs): void;
/**
Raises the {@link GraphMLIOHandler#addQueryInputHandlersListener QueryInputHandlers} event.
@param evt The arguments for the {@link GraphMLIOHandler#addQueryInputHandlersListener QueryInputHandlers}
event.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onQueryInputHandlers(QueryInputHandlersEventArgs) Online Documentation}*/
onQueryInputHandlers(evt: QueryInputHandlersEventArgs): void;
/**
Raises the {@link GraphMLIOHandler#addQueryOutputHandlersListener QueryOutputHandlers} event.
@param evt The arguments for the {@link GraphMLIOHandler#addQueryOutputHandlersListener QueryOutputHandlers}
event.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onQueryOutputHandlers(QueryOutputHandlersEventArgs) Online Documentation}*/
onQueryOutputHandlers(evt: QueryOutputHandlersEventArgs): void;
/**
Fires the {@link GraphMLIOHandler#addQueryReferenceIdListener QueryReferenceId} event.
@param evt The arguments that get passed to the {@link GraphMLIOHandler#addQueryReferenceIdListener QueryReferenceId}
event.
@throws {Exception} <b>(ArgumentError)</b> <code>args</code> is null.
@see GraphMLWriter#onQueryReferenceId
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onQueryReferenceId(QueryReferenceIdEventArgs) Online Documentation}*/
onQueryReferenceId(evt: QueryReferenceIdEventArgs): void;
/**
Fires the {@link GraphMLIOHandler#addResolveReferenceListener ResolveReference} event.
@param evt The arguments that get passed to the {@link GraphMLIOHandler#addResolveReferenceListener ResolveReference}
event.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onResolveReference(ResolveReferenceEventArgs) Online Documentation}*/
onResolveReference(evt: ResolveReferenceEventArgs): void;
/**
Raises the {@link GraphMLIOHandler#addWritingListener Writing} event.
This method is called just before the writing starts and before {@link IWriteEvents#addDocumentWritingListener DocumentWriting}
is called.
@param evt The {@link WriteEventArgs} instance containing the event data.
@see GraphMLIOHandler#addWritingListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onWriting(WriteEventArgs) Online Documentation}*/
onWriting(evt: WriteEventArgs): void;
/**
Raises the {@link GraphMLIOHandler#addWrittenListener Written} event.
This method is called just after the writing finishes and after {@link IWriteEvents#addDocumentWrittenListener DocumentWritten}
is called.
@param evt The {@link WriteEventArgs} instance containing the event data.
@see GraphMLIOHandler#addWrittenListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23onWritten(WriteEventArgs) Online Documentation}*/
onWritten(evt: WriteEventArgs): void;
/**
Read GraphML from an existing XML document.
@param graph The graph that is to be populated with nodes and edges as read from the document.
@param document The XML document. <b>Note</b>: If you are using Internet Explorer 9 and {@link XMLHttpRequest}s to retrieve the document, you may not use
the value from the {@link XMLHttpRequest#responseXML} property. Please parse {@link XMLHttpRequest#responseText} instead and use the
result. For further reference see
{@link https://docs.microsoft.com/en-us/previous-versions/windows/internet-explorer/ie-developer/compatibility/hh180177(v=vs.85)}.
@throws {Exception} <b>(ArgumentError)</b> <code>document</code> is null.
@throws {Exception} <b>(ArgumentError)</b> <code>graph</code> is null.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23readFromDocument(IGraph%2CDocument) Online Documentation}*/
readFromDocument(graph: IGraph, document: Document): Promise<IGraph>;
/**
Convenience method that imports the <code>graph</code> from an XML data provided in a string <code>data</code>.
@param graph The Graph object that is to be populated with nodes and edges as read from the GraphML data.
@param data A string that contains GraphML data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23readFromGraphMLText(IGraph%2Cstring) Online Documentation}*/
readFromGraphMLText(graph: IGraph, data: string): Promise<IGraph>;
/**
Reads GraphML content from the given <code>url</code> and populates the <code>graph</code>.
Note that this is an asynchronous method that returns immediately. Use the returned Promise to get notified when the
actual parsing has completed.
@param graph The graph that is to be populated with nodes and edges as read from the URL.
@param url The URL of the file to be read.
@throws {Exception} <b>(ArgumentError)</b> <code>url</code> is null.
@throws {Exception} <b>(ArgumentError)</b> <code>graph</code> is null.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23readFromURL(IGraph%2Cstring) Online Documentation}*/
readFromURL(graph: IGraph, url: string): Promise<IGraph>;
/**
Predefined input handler that reads edge bends.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerEdgeBendsInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerEdgeBendsInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes edge bends.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerEdgeBendsOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerEdgeBendsOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads edge labels.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerEdgeLabelsInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerEdgeLabelsInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes edge labels.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerEdgeLabelsOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerEdgeLabelsOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads edge styles.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerEdgeStyleInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerEdgeStyleInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes edge styles.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerEdgeStyleOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerEdgeStyleOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads {@link IGraph} graph default settings.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerGraphSettingsInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerGraphSettingsInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes the settings of a graph.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event.
@param sender
@param evt
@see SerializationProperties#DISABLE_GRAPH_SETTINGS
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerGraphSettingsOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerGraphSettingsOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads node labels.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerNodeLabelsInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerNodeLabelsInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes node labels.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerNodeLabelsOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerNodeLabelsOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads node layouts.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerNodeLayoutInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerNodeLayoutInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes node layouts.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerNodeLayoutOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerNodeLayoutOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads node styles.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerNodeStyleInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerNodeStyleInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes node styles.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerNodeStyleOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerNodeStyleOutputHandler(sender: any, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads port labels.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerPortLabelsInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerPortLabelsInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes port labels.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerPortLabelsOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerPortLabelsOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads port locations.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerPortLocationInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerPortLocationInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes port locations.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerPortLocationOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerPortLocationOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads port styles.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerPortStyleInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerPortStyleInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes port styles.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerPortStyleOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerPortStyleOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Predefined input handler that reads {@link ITagOwner} tags.
This handler is by default registered for the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerTagInputHandler(Object%2CQueryInputHandlersEventArgs) Online Documentation}*/
registerTagInputHandler(sender: any | null, evt: QueryInputHandlersEventArgs): void;
/**
Predefined output handler that writes {@link ITagOwner} tags.
This handler is by default registered for the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23registerTagOutputHandler(Object%2CQueryOutputHandlersEventArgs) Online Documentation}*/
registerTagOutputHandler(sender: any | null, evt: QueryOutputHandlersEventArgs): void;
/**
Writes the given graph object to a GraphML file.
@param graph The Graph object that is to be written to a GraphML file.
@returns A string containing the GraphML data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23write(IGraph) Online Documentation}*/
write(graph: IGraph): Promise<string>;
/**
Gets or sets a value indicating whether schema descriptions should be written.
The default is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23writeXMLSchema Online Documentation}*/
writeXMLSchema: boolean;
/**
Gets or sets a value indicating whether to support object sharing on output.
If enabled, multiple object references will be correctly represented in the output document, otherwise, each object
reference will be serialized independently.
<b>Note:</b> <i>Reading</i> shared references is always supported.
The default is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23writeSharedReferences Online Documentation}*/
writeSharedReferences: boolean;
/**
Gets or sets a value indicating whether to clear an existing graph instance before reading
If set
to<code>true</code>, an existing graph instance will be cleared before the graph is read into this instance, otherwise the loaded
graph will be merged into the existing one.
The default is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23clearGraphBeforeRead Online Documentation}*/
clearGraphBeforeRead: boolean;
/**
Adds the given listener for the <code>OverrideResolveReference</code> event that occurs before inline references are evaluated.
Subscribe to this event to customize resolving of GraphML object references. This event is fired before the framework
tries to resolve internal references and can therefore be used to override internal references, for example.
@param listener The listener to add.
@see GraphMLIOHandler#addResolveReferenceListener
@see GraphMLIOHandler#removeOverrideResolveReferenceListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23OverrideResolveReference Online Documentation}*/
addOverrideResolveReferenceListener(listener: (sender: any, evt: ResolveReferenceEventArgs) => void): void;
/**
Removes the given listener for the <code>OverrideResolveReference</code> event that occurs before inline references are evaluated.
Subscribe to this event to customize resolving of GraphML object references. This event is fired before the framework
tries to resolve internal references and can therefore be used to override internal references, for example.
@param listener The listener to remove.
@see GraphMLIOHandler#addResolveReferenceListener
@see GraphMLIOHandler#addOverrideResolveReferenceListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23OverrideResolveReference Online Documentation}*/
removeOverrideResolveReferenceListener(listener: (sender: any, evt: ResolveReferenceEventArgs) => void): void;
/**
Adds the given listener for the <code>ResolveReference</code> event that occurs after references are evaluated.
Subscribe to this event to customize resolving of GraphML object references. This event is fired when the framework
could not resolve an object reference by an internal one.
@param listener The listener to add.
@see GraphMLIOHandler#addOverrideResolveReferenceListener
@see GraphMLIOHandler#removeResolveReferenceListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23ResolveReference Online Documentation}*/
addResolveReferenceListener(listener: (sender: any, evt: ResolveReferenceEventArgs) => void): void;
/**
Removes the given listener for the <code>ResolveReference</code> event that occurs after references are evaluated.
Subscribe to this event to customize resolving of GraphML object references. This event is fired when the framework
could not resolve an object reference by an internal one.
@param listener The listener to remove.
@see GraphMLIOHandler#addOverrideResolveReferenceListener
@see GraphMLIOHandler#addResolveReferenceListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23ResolveReference Online Documentation}*/
removeResolveReferenceListener(listener: (sender: any, evt: ResolveReferenceEventArgs) => void): void;
/**
Adds the given listener for the <code>QueryReferenceId</code> event that occurs when a reference is queried.
This event can be used to provide names of external or internal references for objects.
The reference targets are not serialized to the GraphML file if they are set to
{@link GraphMLReferenceType#EXTERNAL}. To resolve these references when parsing, you typically have to subscribe
to the {@link GraphMLIOHandler#addResolveReferenceListener ResolveReference} event.
@param listener The listener to add.
@see QueryReferenceIdEventArgs#referenceId
@see QueryReferenceIdEventArgs#referenceType
@see GraphMLIOHandler#removeQueryReferenceIdListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryReferenceId Online Documentation}*/
addQueryReferenceIdListener(listener: (sender: any, evt: QueryReferenceIdEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryReferenceId</code> event that occurs when a reference is queried.
This event can be used to provide names of external or internal references for objects.
The reference targets are not serialized to the GraphML file if they are set to
{@link GraphMLReferenceType#EXTERNAL}. To resolve these references when parsing, you typically have to subscribe
to the {@link GraphMLIOHandler#addResolveReferenceListener ResolveReference} event.
@param listener The listener to remove.
@see QueryReferenceIdEventArgs#referenceId
@see QueryReferenceIdEventArgs#referenceType
@see GraphMLIOHandler#addQueryReferenceIdListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryReferenceId Online Documentation}*/
removeQueryReferenceIdListener(listener: (sender: any, evt: QueryReferenceIdEventArgs) => void): void;
/**
Adds the given listener for the <code>Written</code> event that occurs when the writing of a document has been finished.
@param listener The listener to add.
@see GraphMLIOHandler#onWritten
@see GraphMLWriter#addWrittenListener
@see GraphMLIOHandler#removeWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23Written Online Documentation}*/
addWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>Written</code> event that occurs when the writing of a document has been finished.
@param listener The listener to remove.
@see GraphMLIOHandler#onWritten
@see GraphMLWriter#addWrittenListener
@see GraphMLIOHandler#addWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23Written Online Documentation}*/
removeWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>Writing</code> event that occurs when the writing of a document is just about to begin.
This event can be used to register to the fine-grained events available in the {@link WriteEventArgs#context}'s {@link IWriteContext#writeEvents} and to configure the writing process
on a per write case.
@param listener The listener to add.
@see GraphMLIOHandler#onWriting
@see GraphMLWriter#addWritingListener
@see GraphMLIOHandler#removeWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23Writing Online Documentation}*/
addWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>Writing</code> event that occurs when the writing of a document is just about to begin.
This event can be used to register to the fine-grained events available in the {@link WriteEventArgs#context}'s {@link IWriteContext#writeEvents} and to configure the writing process
on a per write case.
@param listener The listener to remove.
@see GraphMLIOHandler#onWriting
@see GraphMLWriter#addWritingListener
@see GraphMLIOHandler#addWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23Writing Online Documentation}*/
removeWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Gets the mapper for the overrides of the serialization properties.
Using this mapper {@link IWriteContext#getSerializationProperty serialization properties} can be specified.
@see GraphMLIOHandler#configureSerializationPropertyOverrides
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23serializationPropertyOverrides Online Documentation}*/
serializationPropertyOverrides: IMap<string, any>;
/**
Gets the mapper for the overrides of the deserialization properties.
Using this mapper {@link IParseContext#getDeserializationProperty deserialization properties} can be specified.
@see GraphMLIOHandler#configureDeserializationPropertyOverrides
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23deserializationPropertyOverrides Online Documentation}*/
deserializationPropertyOverrides: IMap<string, any>;
/**
Adds the given listener for the <code>QueryType</code> event that occurs when a type is queried.
This event can be used to override type resolution for XML names in XAML.
@param listener The listener to add.
@see QueryTypeEventArgs
@see GraphMLIOHandler#removeQueryTypeListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryType Online Documentation}*/
addQueryTypeListener(listener: (sender: any, evt: QueryTypeEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryType</code> event that occurs when a type is queried.
This event can be used to override type resolution for XML names in XAML.
@param listener The listener to remove.
@see QueryTypeEventArgs
@see GraphMLIOHandler#addQueryTypeListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryType Online Documentation}*/
removeQueryTypeListener(listener: (sender: any, evt: QueryTypeEventArgs) => void): void;
/**
Adds the given listener for the <code>QueryName</code> event that occurs when a name is queried.
This event can be used to override name mappings to XML names in XAML.
If the event handler determines that it map resolve {@link QueryNameEventArgs#type} to a xml name, it should place the result into the {@link QueryNameEventArgs#result} property and
thus mark the event as {@link QueryNameEventArgs#handled}.
It is valid to provide only part of the XML name by setting the other part to null. In that case, the default name
determination rules are used for that part.
@param listener The listener to add.
@see GraphMLIOHandler#removeQueryNameListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryName Online Documentation}*/
addQueryNameListener(listener: (sender: any, evt: QueryNameEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryName</code> event that occurs when a name is queried.
This event can be used to override name mappings to XML names in XAML.
If the event handler determines that it map resolve {@link QueryNameEventArgs#type} to a xml name, it should place the result into the {@link QueryNameEventArgs#result} property and
thus mark the event as {@link QueryNameEventArgs#handled}.
It is valid to provide only part of the XML name by setting the other part to null. In that case, the default name
determination rules are used for that part.
@param listener The listener to remove.
@see GraphMLIOHandler#addQueryNameListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryName Online Documentation}*/
removeQueryNameListener(listener: (sender: any, evt: QueryNameEventArgs) => void): void;
/**
Adds the given listener for the <code>HandleSerialization</code> event that occurs when XML content is about to be serialized.
Subscribe to this event to provide custom serialization handling for XML content.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
The current item is provided by {@link HandleSerializationEventArgs#item}.
@param listener The listener to add.
@see GraphMLIOHandler#removeHandleSerializationListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23HandleSerialization Online Documentation}*/
addHandleSerializationListener(listener: (sender: any, evt: HandleSerializationEventArgs) => void): void;
/**
Removes the given listener for the <code>HandleSerialization</code> event that occurs when XML content is about to be serialized.
Subscribe to this event to provide custom serialization handling for XML content.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
The current item is provided by {@link HandleSerializationEventArgs#item}.
@param listener The listener to remove.
@see GraphMLIOHandler#addHandleSerializationListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23HandleSerialization Online Documentation}*/
removeHandleSerializationListener(listener: (sender: any, evt: HandleSerializationEventArgs) => void): void;
/**
Adds the given listener for the <code>QueryOutputHandlers</code> event that occurs when a GraphML attribute is about to be written.
Subscribe to this event to dynamically register one or more {@link IOutputHandler} instance(s) for a given
GraphML attribute key definition.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
If you want to register an {@link IOutputHandler} for this GraphML attribute, you can add the handler with
{@link QueryOutputHandlersEventArgs#addOutputHandler}.
@param listener The listener to add.
@see GraphMLIOHandler#removeQueryOutputHandlersListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryOutputHandlers Online Documentation}*/
addQueryOutputHandlersListener(listener: (sender: any, evt: QueryOutputHandlersEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryOutputHandlers</code> event that occurs when a GraphML attribute is about to be
written.
Subscribe to this event to dynamically register one or more {@link IOutputHandler} instance(s) for a given
GraphML attribute key definition.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
If you want to register an {@link IOutputHandler} for this GraphML attribute, you can add the handler with
{@link QueryOutputHandlersEventArgs#addOutputHandler}.
@param listener The listener to remove.
@see GraphMLIOHandler#addQueryOutputHandlersListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryOutputHandlers Online Documentation}*/
removeQueryOutputHandlersListener(listener: (sender: any, evt: QueryOutputHandlersEventArgs) => void): void;
/**
Adds the given listener for the <code>Parsed</code> event that occurs when the the document has been parsed.
@param listener The listener to add.
@see GraphMLIOHandler#onParsed
@see GraphMLParser#addParsedListener
@see GraphMLIOHandler#removeParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23Parsed Online Documentation}*/
addParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>Parsed</code> event that occurs when the the document has been parsed.
@param listener The listener to remove.
@see GraphMLIOHandler#onParsed
@see GraphMLParser#addParsedListener
@see GraphMLIOHandler#addParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23Parsed Online Documentation}*/
removeParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>Parsing</code> event that occurs when the parsing of a document is just about to begin.
This event can be used to register to the fine-grained events available in the {@link ParseEventArgs#context}'s {@link IParseContext#parseEvents} and to configure the parsing process
on a per parse case.
@param listener The listener to add.
@see GraphMLIOHandler#onParsing
@see GraphMLParser#addParsingListener
@see GraphMLIOHandler#removeParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23Parsing Online Documentation}*/
addParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>Parsing</code> event that occurs when the parsing of a document is just about to begin.
This event can be used to register to the fine-grained events available in the {@link ParseEventArgs#context}'s {@link IParseContext#parseEvents} and to configure the parsing process
on a per parse case.
@param listener The listener to remove.
@see GraphMLIOHandler#onParsing
@see GraphMLParser#addParsingListener
@see GraphMLIOHandler#addParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23Parsing Online Documentation}*/
removeParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>QueryInputHandlers</code> event that occurs when a GraphML attribute is about to be read.
Subscribe to this event to dynamically register one or more {@link IInputHandler} instance(s) for a given GraphML
attribute key definition.
The current key definition is provided by {@link QueryInputHandlersEventArgs#keyDefinition}. If you want to register an {@link IInputHandler} for this GraphML
attribute, you can add the handler with {@link QueryInputHandlersEventArgs#addInputHandler} method.
Implementations should also consider the {@link QueryInputHandlersEventArgs#handled} property.
@param listener The listener to add.
@see GraphMLParser#addQueryInputHandlersListener
@see GraphMLIOHandler#removeQueryInputHandlersListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryInputHandlers Online Documentation}*/
addQueryInputHandlersListener(listener: (sender: any, evt: QueryInputHandlersEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryInputHandlers</code> event that occurs when a GraphML attribute is about to be read.
Subscribe to this event to dynamically register one or more {@link IInputHandler} instance(s) for a given GraphML
attribute key definition.
The current key definition is provided by {@link QueryInputHandlersEventArgs#keyDefinition}. If you want to register an {@link IInputHandler} for this GraphML
attribute, you can add the handler with {@link QueryInputHandlersEventArgs#addInputHandler} method.
Implementations should also consider the {@link QueryInputHandlersEventArgs#handled} property.
@param listener The listener to remove.
@see GraphMLParser#addQueryInputHandlersListener
@see GraphMLIOHandler#addQueryInputHandlersListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23QueryInputHandlers Online Documentation}*/
removeQueryInputHandlersListener(listener: (sender: any, evt: QueryInputHandlersEventArgs) => void): void;
/**
Adds the given listener for the <code>HandleDeserialization</code> event that occurs when XML content is about to be deserialized.
Subscribe to this event to provide custom deserialization handling for XML content.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
The current XML content is provided by {@link HandleDeserializationEventArgs#xmlNode}. If you want to return deserialized content, set the value of the {@link HandleDeserializationEventArgs#result} property.
@param listener The listener to add.
@see GraphMLParser#addHandleDeserializationListener
@see GraphMLIOHandler#removeHandleDeserializationListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23HandleDeserialization Online Documentation}*/
addHandleDeserializationListener(listener: (sender: any, evt: HandleDeserializationEventArgs) => void): void;
/**
Removes the given listener for the <code>HandleDeserialization</code> event that occurs when XML content is about to be
deserialized.
Subscribe to this event to provide custom deserialization handling for XML content.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
The current XML content is provided by {@link HandleDeserializationEventArgs#xmlNode}. If you want to return deserialized content, set the value of the {@link HandleDeserializationEventArgs#result} property.
@param listener The listener to remove.
@see GraphMLParser#addHandleDeserializationListener
@see GraphMLIOHandler#addHandleDeserializationListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLIOHandler%23HandleDeserialization Online Documentation}*/
removeHandleDeserializationListener(listener: (sender: any, evt: HandleDeserializationEventArgs) => void): void;
static $class: Class;
private $_GraphMLIOHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphMLIOHandler;
}
export interface IGenericInputHandlerFactory {
}
/**
Interface for factory classes for {@link IInputHandler} creation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGenericInputHandlerFactory Online Documentation}*/
export abstract class IGenericInputHandlerFactory {
/**
Creates an implementation of the interface {@link IGenericInputHandlerFactory} by using the given function as
implementation for its {@link IGenericInputHandlerFactory#createInputHandler} method.
@param createInputHandler A function for {@link IGenericInputHandlerFactory}'s single abstract method
{@link IGenericInputHandlerFactory#createInputHandler}.
@returns An instance of the {@link IGenericInputHandlerFactory} interface based on the given function.*/
static create(createInputHandler: (keyType: Class, valueType: Class, evt: QueryInputHandlersEventArgs) => IInputHandler): IGenericInputHandlerFactory;
/**
Create an {@link IInputHandler} instance based on the given <code>evt</code>.
@param keyType The key type as inferred by the caller from the GraphML contents.
@param valueType The type of the value as inferred by the caller from the GraphML contents.
@param evt The event argument that is used like in
{@link GraphMLIOHandler#addQueryInputHandlersListener QueryInputHandlers}
@returns The specific input handler to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGenericInputHandlerFactory%23createInputHandler(Class%2CClass%2CQueryInputHandlersEventArgs) Online Documentation}*/
abstract createInputHandler(keyType: Class, valueType: Class, evt: QueryInputHandlersEventArgs): IInputHandler | null;
static $class: Class;
private $_IGenericInputHandlerFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGenericInputHandlerFactory;
}
export interface GraphMLSupport extends YObject {
}
/**
Provides support for opening and saving graphs from and to GraphML.
The methods {@link GraphMLSupport#openFile} and {@link GraphMLSupport#saveFile} open and save GraphML.
In addition, the {@link ICommand#OPEN} and {@link ICommand#SAVE} commands can be enabled by setting a non-{@link StorageLocation#NONE}
storage location and {@link GraphMLSupport#registerCommands registering the commands} with a {@link GraphComponent}. The constructor of this class registers
these commands if a {@link GraphComponent} is passed to it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport Online Documentation}*/
export class GraphMLSupport {
/**
* Creates a new instance of this class.
* If <code>graphComponent</code> is given, the method {@link GraphMLSupport#registerCommands} is called to register the {@link ICommand#OPEN}
* and {@link ICommand#SAVE} commands with that {@link GraphComponent}.
*
* <br><b>Create new GraphML support</b>
* <pre>
* const gs = new GraphMLSupport({
*   graphComponent,
*   // configure to load and save to the file system
*   storageLocation: StorageLocation.FILE_SYSTEM
* })
*
* // Now the OPEN and SAVE commands are enabled, for example
* ICommand.SAVE.execute(null, graphComponent)
* </pre>
*
* @param [graphComponent=null] The {@link GraphComponent} that the {@link ICommand#OPEN} and {@link ICommand#SAVE} commands are
* registered on.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport Online Documentation}
*/
constructor(graphComponent?: GraphComponent | null);
/**
* Creates a new instance of this class.
* If <code>graphComponent</code> is given, the method {@link GraphMLSupport#registerCommands} is called to register the {@link ICommand#OPEN}
* and {@link ICommand#SAVE} commands with that {@link GraphComponent}.
*
* <br><b>Create new GraphML support</b>
* <pre>
* const gs = new GraphMLSupport({
*   graphComponent,
*   // configure to load and save to the file system
*   storageLocation: StorageLocation.FILE_SYSTEM
* })
*
* // Now the OPEN and SAVE commands are enabled, for example
* ICommand.SAVE.execute(null, graphComponent)
* </pre>
*
* @param options The parameters to pass.
* @param [options.graphComponent=null] The {@link GraphComponent} that the {@link ICommand#OPEN} and {@link ICommand#SAVE} commands are
* registered on.
* @param [options.storageLocation] A value indicating the type of storage used by the {@link ICommand#OPEN} and {@link ICommand#SAVE} commands. This option sets the {@link GraphMLSupport#storageLocation} property on the created object.
* @param [options.graphMLIOHandler] The {@link GraphMLSupport#graphMLIOHandler} to use for the various I/O methods in this instance. This option sets the {@link GraphMLSupport#graphMLIOHandler} property on the created object.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport Online Documentation}
*/
constructor(options: {
graphComponent?: GraphComponent | null;
storageLocation?: StorageLocation | StorageLocationStringValues;
graphMLIOHandler?: GraphMLIOHandler;
});
/**
Creates the {@link HTMLInputElement} that is used by the {@link GraphMLSupport#openFile} methods.
@returns The pre-configured element to use.
@see GraphMLSupport#openFile
@see GraphMLSupport#openFile
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport%23createOpenFileInputElement() Online Documentation}*/
createOpenFileInputElement(): HTMLInputElement;
/**
Creates the {@link HTMLElement} that is used by the {@link GraphMLSupport#saveFile} method.
If {@link GraphMLSupport#storageLocation} is set to {@link StorageLocation#FILE_SYSTEM} but saving to a file is not supported by the user's browser,
this method is called to create the element to which the GraphML content is appended.
@returns The pre-configured element to use or <code>null</code> if element creation failed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport%23createSaveFileTextElement() Online Documentation}*/
createSaveFileTextElement(): HTMLElement | null;
/**
Shows a {@link GraphMLSupport#createOpenFileInputElement open file dialog} and imports the selected GraphML file into the graph of the provided <code>graphComponent</code>.
This method is called in response to the {@link ICommand#OPEN} command if {@link GraphMLSupport#storageLocation} is set to a value different from
{@link StorageLocation#NONE}. It delegates to {@link GraphMLSupport#openFile}, then calls {@link GraphComponent#fitGraphBounds}
and clears the {@link UndoEngine} of the <code>graphComponent</code>.
Any exception thrown during loading of the GraphML file will reject the promise.
@param graphComponent The GraphML file is imported into the graph of this {@link GraphComponent}.
@param [storageLocation=null] The {@link GraphMLSupport#storageLocation} to open from. If not provided, the value of the {@link GraphMLSupport#storageLocation} property is used.
@returns A promise that will resolve to the loaded graph instance or reject upon exceptions during loading. Note that if the file
open dialog is canceled by the user, the promise will stay unresolved because there is no API available that can be used
to determine whether the user has pressed cancel. Depending on the UA implementation it may be possible to listen to
focus or other input device events on the page to detect when the user has closed the file open dialog.
@see GraphMLSupport#storageLocation
@see GraphMLSupport#saveFile
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport%23openFile(GraphComponent%2CStorageLocation) Online Documentation}*/
openFile(graphComponent: GraphComponent, storageLocation?: StorageLocation | null | StorageLocationStringValues): Promise<IGraph>;
/**
Shows a {@link GraphMLSupport#createOpenFileInputElement open file dialog} and imports the selected GraphML file into the provided <code>graph</code>.
Any exception thrown during loading of the GraphML file will reject the promise.
@param graph The GraphML file is imported into this graph.
@param [storageLocation=null] The {@link GraphMLSupport#storageLocation} to open from. If not provided, the value of the {@link GraphMLSupport#storageLocation} property is used.
@returns A promise that will resolve to the loaded graph instance or reject upon exceptions during loading. Note that if the file
open dialog is canceled by the user, the promise will stay unresolved because there is no API available that can be used
to determine whether the user has pressed cancel. Depending on the UA implementation it may be possible to listen to
focus or other input device events on the page to detect when the user has closed the file open dialog.
@see GraphMLSupport#storageLocation
@see GraphMLSupport#saveFile
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport%23openFile(IGraph%2CStorageLocation) Online Documentation}*/
openFile(graph: IGraph, storageLocation?: StorageLocation | null | StorageLocationStringValues): Promise<IGraph>;
/**
Registers the {@link ICommand#OPEN} and {@link ICommand#SAVE} commands on the given <code>graphComponent</code> with the
current {@link GraphMLSupport#storageLocation}.
@param graphComponent The {@link GraphComponent} to register the commands on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport%23registerCommands(GraphComponent) Online Documentation}*/
registerCommands(graphComponent: GraphComponent): void;
/**
Exports the graph as GraphML and saves it to the provided storage location.
This method is called in response to the {@link ICommand#SAVE} command if {@link GraphMLSupport#storageLocation} /// is set to a value different from
{@link StorageLocation#NONE}.
Any exception thrown during loading of the GraphML file will be caught and an error message displayed to the user. For
more control over error behavior it is recommended to use {@link GraphMLSupport#graphMLIOHandler} directly, e.g. from a custom command handler.
@param graph The graph to save as GraphML.
@param [storageLocation=null] The {@link GraphMLSupport#storageLocation} to open from. If not provided, the value of the {@link GraphMLSupport#storageLocation} property is used.
@returns A promise that will either resolve to the GraphML string representation of the graph or reject upon exceptions during
the save operation.
@see GraphMLSupport#storageLocation
@see GraphMLSupport#openFile
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport%23saveFile(IGraph%2CStorageLocation) Online Documentation}*/
saveFile(graph: IGraph, storageLocation?: StorageLocation | null | StorageLocationStringValues): Promise<string>;
/**
Gets or sets the {@link GraphMLSupport#graphMLIOHandler} to use for the various I/O methods in this instance.
A new instance of {@link GraphMLSupport#graphMLIOHandler} is created on first access if the property has not been set.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport%23graphMLIOHandler Online Documentation}*/
graphMLIOHandler: GraphMLIOHandler;
/**
Gets or sets a value indicating the type of storage used by the {@link ICommand#OPEN} and {@link ICommand#SAVE}
commands.
By default this feature is set to {@link StorageLocation#NONE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLSupport%23storageLocation Online Documentation}*/
storageLocation: StorageLocation;
static $class: Class;
private $_GraphMLSupport: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphMLSupport;
}
/**
Describes the storage location of the file operations.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StorageLocation Online Documentation}*/
export enum StorageLocation {
/**
No location at all, effectively disabling storage operations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StorageLocation%23NONE Online Documentation}*/
NONE,
/**
The local storage as described by the W3C Web Storage API.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StorageLocation%23LOCAL_STORAGE Online Documentation}*/
LOCAL_STORAGE,
/**
The session storage as described by the W3C Web Storage API.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StorageLocation%23SESSION_STORAGE Online Documentation}*/
SESSION_STORAGE,
/**
The file system.
Opening a file from the file system works in all supported browsers. Saving a file is supported in Firefox 28+, Chrome
38+, Opera 25+, IE 10+, recent versions of the related mobile browsers and Android Browser 4.4.4. If saving to a file is
not supported, the file contents is shown in a new browser window instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StorageLocation%23FILE_SYSTEM Online Documentation}*/
FILE_SYSTEM
}
/** Automatic conversion to {@link StorageLocation}. */
type StorageLocationStringValues = "none" | "local-storage" | "session-storage" | "file-system";
export interface ChildParseContext extends YObject, IParseContext {
/**
@inheritdoc*/
deserializeCore<T>(targetType: Class, context: IParseContext, targetNode: Node): T;
/**
@inheritdoc*/
getCurrent<T>(itemType: Class): T | null;
/**
@inheritdoc*/
getDeserializationProperty(key: string): any | null;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
readonly objectStack: IListEnumerable<any>;
/**
@inheritdoc*/
readonly parseEvents: IParseEvents;
/**
@inheritdoc*/
readonly graph: IGraph;
} /**
An implementation of {@link IParseContext} that delegates most calls to a {@link ChildParseContext#parentContext}, but adds the capability to locally
add {@link ChildParseContext#lookup} results and {@link ChildParseContext#getDeserializationProperty deserialization properties}
Whenever it is necessary to locally modify some context properties, a new instance of this class should be created that
wraps an existing one.
@see ChildParseContext#setDeserializationProperty
@see ChildParseContext#setLookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildParseContext Online Documentation}*/
export class ChildParseContext {
/**
Initializes a new instance of the {@link ChildParseContext} class that delegates to the provided context and adds
the <code>currentObject</code> to the {@link ChildParseContext#objectStack}.
@param parentContext The parent context.
@param currentObject The current object to add to the {@link ChildParseContext#objectStack} and return in {@link ChildParseContext#getCurrent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildParseContext%23ChildParseContext(IParseContext%2CObject) Online Documentation}*/
constructor(parentContext: IParseContext, currentObject: any | null);
/**
Initializes a new instance of the {@link ChildParseContext} class that delegates to the provided context.
@param parentContext The parent context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildParseContext%23ChildParseContext(IParseContext) Online Documentation}*/
constructor(parentContext: IParseContext);
/**
Removes a previously locally {@link ChildParseContext#setDeserializationProperty set} deserialization property.
@param key The key for which the local property override should be removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildParseContext%23removeDeserializationProperty(string) Online Documentation}*/
removeDeserializationProperty(key: string): void;
/**
Removes a local lookup override that has been added using {@link ChildParseContext#setLookup}
Type parameter <code>T</code>: The type key to remove.
@param tType The type key to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildParseContext%23removeLookup(Class) Online Documentation}*/
removeLookup<T>(tType: Class): void;
/**
Sets the local deserialization property for the given key to the value.
@param key The key to override.
@param value The value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildParseContext%23setDeserializationProperty(string%2CObject) Online Documentation}*/
setDeserializationProperty(key: string, value: any | null): void;
/**
Adds another lookup result to the local lookup override.
Type parameter <code>T</code>: The type to use as a key for the lookup.
@param tType The type to use as a key for the lookup.
@param value The value to provide for queries to <code>T</code>.
@see ChildParseContext#removeLookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildParseContext%23setLookup(Class%2CT) Online Documentation}*/
setLookup<T>(tType: Class, value: T): void;
/**
Gets the parent context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildParseContext%23parentContext Online Documentation}*/
parentContext: IParseContext;
static $class: Class;
private $_ChildParseContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ChildParseContext;
}
export interface CreationProperties extends YObject, IMapper<any, any> {
}
/**
A simple mutable implementation of {@link IMapper} that is typed for {@link Object} keys and values.
This class is used by the {@link GraphMLParser} to store properties of an item before it has been created. An
instance of this class can be found in the {@link ILookup#lookup} of the {@link IParseContext} during the
parsing of an item. Custom parsers may populate the dictionary with various properties that can then be used by the {@link IGraphElementFactory}
during the creation of the item.
@see IGraphElementFactory
@see GraphMLParser
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties Online Documentation}*/
export class CreationProperties {
constructor();
/**
Clears all entries from this property store.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23clear() Online Documentation}*/
clear(): void;
/**
Gets the value associated with the specified key.
@param key The key.
@returns The value stored for the key.
@see CreationProperties#set
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23get(Object) Online Documentation}*/
get(key: any): any | null;
/**
Removes the associated entry for the given key
@param key The entry key to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23removeValue(Object) Online Documentation}*/
removeValue(key: any): void;
/**
Sets the value associated with the specified key.
@param key The key.
@param value The value stored for the key.
@see CreationProperties#get
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23set(Object%2CObject) Online Documentation}*/
set(key: any, value: any | null): void;
/**
Creation property key that tags group node information.
Stores values of type {@link boolean}.
@see IGraphElementFactory#createNode
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23IS_GROUP_NODE Online Documentation}*/
static IS_GROUP_NODE: string;
/**
Creation property key that tags label information.
Stores values of type {@link IEnumerable} with {@link ILabel} template instances.
@see IGraphElementFactory
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23LABELS Online Documentation}*/
static LABELS: string;
/**
Creation property key that tags node layout information.
Stores values of type {@link Rect}.
@see IGraphElementFactory#createNode
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23LAYOUT Online Documentation}*/
static LAYOUT: string;
/**
Creation property key that tags style information.
Stores values of type {@link INodeStyle}, {@link IEdgeStyle}, and {@link IPortStyle} respectively.
@see IGraphElementFactory
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23STYLE Online Documentation}*/
static STYLE: string;
/**
Creation property key that tags {@link ITagOwner#tag tag} object information.
Stores values of type {@link Object}.
@see IGraphElementFactory
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23TAG Online Documentation}*/
static TAG: string;
/**
Creation property key that tags bend information.
Stores values of type {@link IEnumerable} with {@link IBend} template instances.
@see IGraphElementFactory#createEdge
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23BENDS Online Documentation}*/
static BENDS: string;
/**
Creation property key that tags {@link IPortLocationModelParameter port location} information.
Stores values of type {@link IPortLocationModelParameter}.
@see IGraphElementFactory#addPort
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23PORT_LOCATION_MODEL_PARAMETER Online Documentation}*/
static PORT_LOCATION_MODEL_PARAMETER: string;
/**
Gets all the entries stored by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreationProperties%23entries Online Documentation}*/
readonly entries: IEnumerable<MapEntry<any, any>>;
static $class: Class;
private $_CreationProperties: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CreationProperties;
}
export interface GraphMLParser extends YObject {
}
/**
Core parser class for GraphML.
This class allows for low-level customization of the parsing process. Usually, it is used by one of
{@link GraphMLIOHandler}'s <code>readFrom</code> methods which calls one of the various <code>GraphMLParser.parseFrom</code> method
variants. It should rarely be necessary to use this class directly.
@see GraphMLIOHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser Online Documentation}*/
export class GraphMLParser {
constructor();
/**
Create the initial {@link IParseContext} instance.
@param graph The graph that is parsed.
@returns An {@link IParseContext} instance that is suitable for <code>graph</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23createParseContext(IGraph) Online Documentation}*/
createParseContext(graph: IGraph): IParseContext;
/**
Deserialize the content of <code>element</code>.
Typically, this method is called only indirectly by calls to {@link IParseContext#deserializeCore}. This
implementation calls {@link GraphMLParser#onHandleDeserialization}, which in turn raises the {@link GraphMLParser#addHandleDeserializationListener HandleDeserialization}
event.
Type parameter <code>T</code>: The desired type of the result.
@param targetType The desired type of the result.
@param context The current parse context.
@param element The XML content that should be deserialized.
@returns An instance of <code>T</code> that is represented by <code>element</code>.
@see IParseContext#deserializeCore
@see GraphMLParser#addHandleDeserializationListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23deserialize(Class%2CIParseContext%2CNode) Online Documentation}*/
deserialize<T>(targetType: Class, context: IParseContext, element: Node): T;
/**
Retrieve a deserialization property that has been set by {@link GraphMLParser#setDeserializationProperty}.
@param key The key for the property.
@see IParseContext#getDeserializationProperty
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23getDeserializationProperty(string) Online Documentation}*/
getDeserializationProperty(key: string): any | null;
/**
Dynamically retrieve all {@link IInputHandler} instances that process a GraphML attribute with key definition
<code>keyDefinition</code>.
Typically, this method is called only indirectly by the parse process. This implementation calls
{@link GraphMLParser#onQueryInputHandlers}, which in turn raises the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event.
@param context The current parse context.
@param keyDefinition The key definition element of the GraphML attribute.
@returns A list of {@link IInputHandler IInputHandlers} that claim to be responsible for handling the given GraphML attribute.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23getInputHandlers(IParseContext%2CElement) Online Documentation}*/
getInputHandlers(context: IParseContext, keyDefinition: Element): IList<IInputHandler>;
/**
Dynamically retrieve an instance of <code>type</code>.
@param type The type for which an implementation is needed.
@returns An implementation of <code>type</code>, or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23lookup(Class) Online Documentation}*/
lookup(type: Class): any | null;
/**
Fires the {@link GraphMLParser#addHandleDeserializationListener HandleDeserialization} event.
@param evt The arguments that get passed to the {@link GraphMLParser#addHandleDeserializationListener HandleDeserialization}
event.
@see GraphMLIOHandler#onHandleDeserialization
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23onHandleDeserialization(HandleDeserializationEventArgs) Online Documentation}*/
onHandleDeserialization(evt: HandleDeserializationEventArgs): void;
/**
Raises the {@link GraphMLParser#addParsedListener Parsed} event.
This method is called when the document has been parsed and after {@link IParseEvents#addDocumentParsedListener DocumentParsed}
is called.
@param evt The {@link ParseEventArgs} instance containing the event data.
@see GraphMLParser#addParsedListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23onParsed(ParseEventArgs) Online Documentation}*/
onParsed(evt: ParseEventArgs): void;
/**
Raises the {@link GraphMLParser#addParsingListener Parsing} event.
This method is called just before the parsing starts and before {@link IParseEvents#addDocumentParsingListener DocumentParsing}
is called.
@param evt The {@link ParseEventArgs} instance containing the event data.
@see GraphMLParser#addParsingListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23onParsing(ParseEventArgs) Online Documentation}*/
onParsing(evt: ParseEventArgs): void;
/**
Fires the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers} event.
@param evt The arguments that get passed to the {@link GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
event.
@see GraphMLIOHandler#onQueryInputHandlers
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23onQueryInputHandlers(QueryInputHandlersEventArgs) Online Documentation}*/
onQueryInputHandlers(evt: QueryInputHandlersEventArgs): void;
/**
Parse the XML document <code>document</code> into an <code>graph</code> instance, using <code>elementFactory</code> to create the elements.
Typically, this method is called indirectly from {@link GraphMLIOHandler#readFromDocument}.
@param document The XML document instance to parse. <b>Note</b>: If you are using Internet Explorer 9 and {@link XMLHttpRequest}s to retrieve the document,
you may not use the value from the {@link XMLHttpRequest#responseXML} property. Please parse {@link XMLHttpRequest#responseText} instead
and use the result. For further reference see
{@link https://docs.microsoft.com/en-us/previous-versions/windows/internet-explorer/ie-developer/compatibility/hh180177(v=vs.85)}.
@param graph The graph instance that is populated.
@param elementFactory The {@link IGraphElementFactory} instance that is used to create the elements.
@see GraphMLIOHandler#readFromDocument
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23parseFromDocument(IGraph%2CDocument%2CIGraphElementFactory) Online Documentation}*/
parseFromDocument(graph: IGraph, document: Document, elementFactory: IGraphElementFactory): Promise<IGraph>;
/**
Parse the input <code>url</code> into an <code>graph</code> instance, using <code>elementFactory</code> to create the elements.
Typically, this method is called indirectly from {@link GraphMLIOHandler#readFromURL}.
@param url The URL to the input to parse.
@param graph The graph instance that is populated.
@param elementFactory The {@link IGraphElementFactory} instance that is used to create the elements.
@see GraphMLIOHandler#readFromGraphMLText
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23parseFromURL(IGraph%2Cstring%2CIGraphElementFactory) Online Documentation}*/
parseFromURL(graph: IGraph, url: string, elementFactory: IGraphElementFactory): Promise<IGraph>;
/**
Remove a deserialization property that has been set by {@link GraphMLParser#setDeserializationProperty}.
@param key The key for the property.
@see IParseContext#getDeserializationProperty
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23removeDeserializationProperty(string) Online Documentation}*/
removeDeserializationProperty(key: string): void;
/**
Remove an implementation of <code>T</code> that has been set with {@link GraphMLParser#setLookup}.
Type parameter <code>T</code>.
@param tType
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23removeLookup(Class) Online Documentation}*/
removeLookup<T>(tType: Class): void;
/**
Set a deserialization property that allows to fine tune the parsing process.
@param key The key for the property.
@param value The property value.
@see IParseContext#getDeserializationProperty
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23setDeserializationProperty(string%2CObject) Online Documentation}*/
setDeserializationProperty(key: string, value: any | null): void;
/**
Register an implementation of <code>T</code> for use with {@link GraphMLParser#lookup}.
Type parameter <code>T</code>: The type of <code>instance</code>.
@param tType The type of <code>instance</code>.
@param instance The implementation of <code>T</code> for lookup retrieval.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23setLookup(Class%2CT) Online Documentation}*/
setLookup<T>(tType: Class, instance: T | null): void;
/**
Adds the given listener for the <code>QueryInputHandlers</code> event that occurs when {@link IInputHandler}s are queried.
Subscribe to this event to dynamically register one or more {@link IInputHandler} instance(s) for a given GraphML
attribute key definition.
The current key definition is provided by {@link QueryInputHandlersEventArgs#keyDefinition}. If you want to register an {@link IInputHandler} for this GraphML
attribute, you can add the handler with {@link QueryInputHandlersEventArgs#addInputHandler}.
@param listener The listener to add.
@see GraphMLIOHandler#addQueryInputHandlersListener
@see GraphMLParser#removeQueryInputHandlersListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23QueryInputHandlers Online Documentation}*/
addQueryInputHandlersListener(listener: (sender: any, evt: QueryInputHandlersEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryInputHandlers</code> event that occurs when {@link IInputHandler}s are queried.
Subscribe to this event to dynamically register one or more {@link IInputHandler} instance(s) for a given GraphML
attribute key definition.
The current key definition is provided by {@link QueryInputHandlersEventArgs#keyDefinition}. If you want to register an {@link IInputHandler} for this GraphML
attribute, you can add the handler with {@link QueryInputHandlersEventArgs#addInputHandler}.
@param listener The listener to remove.
@see GraphMLIOHandler#addQueryInputHandlersListener
@see GraphMLParser#addQueryInputHandlersListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23QueryInputHandlers Online Documentation}*/
removeQueryInputHandlersListener(listener: (sender: any, evt: QueryInputHandlersEventArgs) => void): void;
/**
Adds the given listener for the <code>HandleDeserialization</code> event that occurs when XML content is about to be deserialized.
Subscribe to this event to provide custom deserialization handling for XML content.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
The current XML content is provided by {@link HandleDeserializationEventArgs#xmlNode}. If you want to return deserialized content, set the value of the {@link HandleDeserializationEventArgs#result} property.
@param listener The listener to add.
@see GraphMLIOHandler#addHandleDeserializationListener
@see GraphMLParser#removeHandleDeserializationListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23HandleDeserialization Online Documentation}*/
addHandleDeserializationListener(listener: (sender: any, evt: HandleDeserializationEventArgs) => void): void;
/**
Removes the given listener for the <code>HandleDeserialization</code> event that occurs when XML content is about to be
deserialized.
Subscribe to this event to provide custom deserialization handling for XML content.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
The current XML content is provided by {@link HandleDeserializationEventArgs#xmlNode}. If you want to return deserialized content, set the value of the {@link HandleDeserializationEventArgs#result} property.
@param listener The listener to remove.
@see GraphMLIOHandler#addHandleDeserializationListener
@see GraphMLParser#addHandleDeserializationListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23HandleDeserialization Online Documentation}*/
removeHandleDeserializationListener(listener: (sender: any, evt: HandleDeserializationEventArgs) => void): void;
/**
Adds the given listener for the <code>Parsed</code> event that occurs when the the document has been parsed.
@param listener The listener to add.
@see GraphMLParser#onParsed
@see GraphMLParser#removeParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23Parsed Online Documentation}*/
addParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>Parsed</code> event that occurs when the the document has been parsed.
@param listener The listener to remove.
@see GraphMLParser#onParsed
@see GraphMLParser#addParsedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23Parsed Online Documentation}*/
removeParsedListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Adds the given listener for the <code>Parsing</code> event that occurs when the parsing of the document is just about to begin.
This event can be used to register to the fine-grained events available in the {@link ParseEventArgs#context}'s {@link IParseContext#parseEvents} and the configure the parsing process
on a per parse case.
@param listener The listener to add.
@see GraphMLParser#onParsing
@see GraphMLParser#removeParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23Parsing Online Documentation}*/
addParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
/**
Removes the given listener for the <code>Parsing</code> event that occurs when the parsing of the document is just about to begin.
This event can be used to register to the fine-grained events available in the {@link ParseEventArgs#context}'s {@link IParseContext#parseEvents} and the configure the parsing process
on a per parse case.
@param listener The listener to remove.
@see GraphMLParser#onParsing
@see GraphMLParser#addParsingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParser%23Parsing Online Documentation}*/
removeParsingListener(listener: (sender: any, evt: ParseEventArgs) => void): void;
static $class: Class;
private $_GraphMLParser: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphMLParser;
}
export interface HandleDeserializationEventArgs extends EventArgs {
}
/**
The event arguments used by {@link GraphMLIOHandler#addHandleDeserializationListener HandleDeserialization}
and {@link GraphMLParser#addHandleDeserializationListener HandleDeserialization} to let registered
deserialization code perform the deserialization.
If the event handler determines that it can deserialize the {@link HandleDeserializationEventArgs#xmlNode}, it should place the result into the {@link HandleDeserializationEventArgs#result} property and thus
mark the event as {@link HandleDeserializationEventArgs#handled}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleDeserializationEventArgs Online Documentation}*/
export class HandleDeserializationEventArgs {
/**
Initializes a new instance of the {@link HandleDeserializationEventArgs} class.
@param context The context in which serialization should take place.
@param xmlNode The XML element that contains the data to deserialize.
@param targetType The required {@link HandleDeserializationEventArgs#targetType target type} of the {@link HandleDeserializationEventArgs#result}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleDeserializationEventArgs Online Documentation}*/
constructor(context: IParseContext, xmlNode: Node, targetType: Class | null);
/**
Gets the type of the resulting instance that is required by the requesting deserializer or <code>null</code> if the target type is
not known in advance.
@see IParseContext#deserializeCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleDeserializationEventArgs%23targetType Online Documentation}*/
targetType: Class | null;
/**
Gets or sets the result of the deserialization, which is <code>null</code> initially.
Setting this property will automatically set the {@link HandleDeserializationEventArgs#handled} property to <code>true</code>.
@throws {Exception} <b>(ArgumentError)</b> If the value is not assignable to {@link HandleDeserializationEventArgs#targetType}
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleDeserializationEventArgs%23result Online Documentation}*/
result: any | null;
/**
Gets or sets a value indicating whether this {@link HandleDeserializationEventArgs} is handled.
Setting this property to <code>true</code> indicates to the event source that the {@link HandleDeserializationEventArgs#result} has been assigned and that the event should not be
propagated to further listeners.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleDeserializationEventArgs%23handled Online Documentation}*/
handled: boolean;
/**
Gets the context in which the {@link HandleDeserializationEventArgs#xmlNode} shall be deserialized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleDeserializationEventArgs%23context Online Documentation}*/
context: IParseContext;
/**
Gets the XML node that contains the data to deserialize.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleDeserializationEventArgs%23xmlNode Online Documentation}*/
xmlNode: Node;
static $class: Class;
private $_HandleDeserializationEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HandleDeserializationEventArgs;
}
export interface IGraphElementFactory {
}
/**
This is the interface for a factory that is used for creating the constituents of an {@link IGraph} that are
encountered during the parsing of {@link GraphMLParser GraphML}
@see GraphMLIOHandler#createGraphElementFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementFactory Online Documentation}*/
export abstract class IGraphElementFactory {
/**
Adds a new port to the given {@link IPortOwner}.
@param context The context from which the graph, the {@link CreationProperties}, and the {@link IParseContext#objectStack} can be queried.
@param owner The owner of the port.
@returns A newly created port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementFactory%23addPort(IParseContext%2CIPortOwner) Online Documentation}*/
abstract addPort(context: IParseContext, owner: IPortOwner): IPort;
/**
Creates an implementation of the interface {@link IGraphElementFactory} from the given definition.
The given object must contain exactly the abstract members of the {@link IGraphElementFactory} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IGraphElementFactory} interface.
@returns An instance of the {@link IGraphElementFactory} interface based on the given definition.*/
static create(interfaceDefinition: {
createNode(context: IParseContext): INode;
createEdge(context: IParseContext, sourceNode: INode, sourcePort: IPort | null, targetNode: INode, targetPort: IPort | null): IEdge;
addPort(context: IParseContext, owner: IPortOwner): IPort;
}): IGraphElementFactory;
/**
Creates an edge for the given context and the provided source and target nodes or ports.
@param context The context from which the graph, the {@link CreationProperties}, and the {@link IParseContext#objectStack} can be queried.
@param sourceNode The source node for the edge.
@param sourcePort The source port for the edge, if specified, otherwise <code>null</code>.
@param targetNode The target node for the edge.
@param targetPort The target port for the edge, if specified, otherwise <code>null</code>.
@returns A newly created edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementFactory%23createEdge(IParseContext%2CINode%2CIPort%2CINode%2CIPort) Online Documentation}*/
abstract createEdge(context: IParseContext, sourceNode: INode, sourcePort: IPort | null, targetNode: INode, targetPort: IPort | null): IEdge;
/**
Creates a node for the given context.
@param context The context from which the graph, the {@link CreationProperties}, and the {@link IParseContext#objectStack} can be queried.
@returns A newly created node
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementFactory%23createNode(IParseContext) Online Documentation}*/
abstract createNode(context: IParseContext): INode;
static $class: Class;
private $_IGraphElementFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGraphElementFactory;
}
export interface IGraphElementIdAcceptor {
}
/**
Interface for classes that can store the GraphML id attribute of GraphML core elements
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdAcceptor Online Documentation}*/
export abstract class IGraphElementIdAcceptor {
/**
Creates an implementation of the interface {@link IGraphElementIdAcceptor} from the given definition.
The given object must contain exactly the abstract members of the {@link IGraphElementIdAcceptor} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IGraphElementIdAcceptor} interface.
@returns An instance of the {@link IGraphElementIdAcceptor} interface based on the given definition.*/
static create(interfaceDefinition: {
storeGraphId(context: IParseContext, graph: IGraph, id: string): void;
storeNodeId(context: IParseContext, node: INode, id: string): void;
storeEdgeId(context: IParseContext, edge: IEdge, id: string): void;
storePortId(context: IParseContext, port: IPort, id: string): void;
}): IGraphElementIdAcceptor;
/**
Store the value of the id attribute for the given edge.
@param context The current parse context.
@param edge The current edge element
@param id The id of the edge's XML representation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdAcceptor%23storeEdgeId(IParseContext%2CIEdge%2Cstring) Online Documentation}*/
abstract storeEdgeId(context: IParseContext, edge: IEdge, id: string): void;
/**
Store the value of the id attribute for the given graph.
@param context The current parse context.
@param graph The current graph element
@param id The id of the graph's XML representation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdAcceptor%23storeGraphId(IParseContext%2CIGraph%2Cstring) Online Documentation}*/
abstract storeGraphId(context: IParseContext, graph: IGraph, id: string): void;
/**
Store the value of the id attribute for the given node.
@param context The current parse context.
@param node The current node element
@param id The id of the node's XML representation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdAcceptor%23storeNodeId(IParseContext%2CINode%2Cstring) Online Documentation}*/
abstract storeNodeId(context: IParseContext, node: INode, id: string): void;
/**
Store the value of the id attribute for the given port.
@param context The current parse context.
@param port The current port element
@param id The id of the port's XML representation
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdAcceptor%23storePortId(IParseContext%2CIPort%2Cstring) Online Documentation}*/
abstract storePortId(context: IParseContext, port: IPort, id: string): void;
static $class: Class;
private $_IGraphElementIdAcceptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGraphElementIdAcceptor;
}
export interface GraphElementIdAcceptor extends YObject, IGraphElementIdAcceptor, IGraphElementResolver {
/**
@inheritdoc*/
resolveEdge(context: IParseContext, id: string): IEdge | null;
/**
@inheritdoc*/
resolveGraph(context: IParseContext, id: string): IGraph | null;
/**
@inheritdoc*/
resolveNode(context: IParseContext, id: string): INode | null;
/**
@inheritdoc*/
resolvePort(context: IParseContext, ownerId: string, id: string): IPort | null;
/**
@inheritdoc*/
storeEdgeId(context: IParseContext, edge: IEdge, id: string): void;
/**
@inheritdoc*/
storeGraphId(context: IParseContext, graph: IGraph, id: string): void;
/**
@inheritdoc*/
storeNodeId(context: IParseContext, node: INode, id: string): void;
/**
@inheritdoc*/
storePortId(context: IParseContext, port: IPort, id: string): void;
} /**
Simple implementation of {@link IGraphElementIdAcceptor} and {@link IGraphElementIdProvider} that stores
all ids in {@link IMapper} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphElementIdAcceptor Online Documentation}*/
export class GraphElementIdAcceptor {
constructor();
/**
Factory method to create a default implementation of {@link IGraphElementIdProvider} that delegates to <code>fallback</code>
and uses the values stored by this {@link GraphElementIdAcceptor} instance, if possible.
@param fallback The fallback.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphElementIdAcceptor%23createIdProvider(IGraphElementIdProvider) Online Documentation}*/
createIdProvider(fallback: IGraphElementIdProvider): IGraphElementIdProvider;
/**
Gets a mapping between all GraphML ids for <code>&lt;graph></code> elements and the corresponding {@link IGraph} instances
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphElementIdAcceptor%23graphIds Online Documentation}*/
readonly graphIds: IMap<IGraph, string>;
/**
Gets a mapping between all GraphML ids for <code>&lt;node></code> elements and the corresponding {@link INode} instances
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphElementIdAcceptor%23nodeIds Online Documentation}*/
readonly nodeIds: IMap<INode, string>;
/**
Gets a mapping between all GraphML ids for <code>&lt;edge></code> elements and the corresponding {@link IEdge} instances
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphElementIdAcceptor%23edgeIds Online Documentation}*/
readonly edgeIds: IMap<IEdge, string>;
/**
Gets a mapping between all GraphML ids for <code>&lt;port></code> elements and the corresponding {@link IPort} instances
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphElementIdAcceptor%23portIds Online Documentation}*/
readonly portIds: IMap<IPort, string>;
static $class: Class;
private $_GraphElementIdAcceptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphElementIdAcceptor;
}
export interface IGraphElementResolver {
}
/**
Interface for classes can resolve graph elements based on their GraphML id attribute(s).
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementResolver Online Documentation}*/
export abstract class IGraphElementResolver {
/**
Creates an implementation of the interface {@link IGraphElementResolver} from the given definition.
The given object must contain exactly the abstract members of the {@link IGraphElementResolver} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IGraphElementResolver} interface.
@returns An instance of the {@link IGraphElementResolver} interface based on the given definition.*/
static create(interfaceDefinition: {
resolveNode(context: IParseContext, id: string): INode;
resolveGraph(context: IParseContext, id: string): IGraph;
resolvePort(context: IParseContext, ownerId: string, id: string): IPort;
resolveEdge(context: IParseContext, id: string): IEdge;
}): IGraphElementResolver;
/**
Resolve the GraphML <code>id</code> to an {@link IEdge} instance.
@param context The current parse context.
@param id The id that should be resolved.
@returns The {@link IEdge} that is defined by this GraphML <code>id</code>, or <code>null</code> if no such edge exists yet.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementResolver%23resolveEdge(IParseContext%2Cstring) Online Documentation}*/
abstract resolveEdge(context: IParseContext, id: string): IEdge | null;
/**
Resolve the GraphML <code>id</code> to an {@link IGraph} instance.
@param context The current parse context.
@param id The id that should be resolved.
@returns The {@link IGraph} that is defined by this GraphML <code>id</code>, or <code>null</code> if no such graph exists yet.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementResolver%23resolveGraph(IParseContext%2Cstring) Online Documentation}*/
abstract resolveGraph(context: IParseContext, id: string): IGraph | null;
/**
Resolve the GraphML <code>id</code> to an {@link INode} instance.
@param context The current parse context.
@param id The id that should be resolved.
@returns The {@link INode} that is defined by this GraphML <code>id</code>, or <code>null</code> if no such node exists yet.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementResolver%23resolveNode(IParseContext%2Cstring) Online Documentation}*/
abstract resolveNode(context: IParseContext, id: string): INode | null;
/**
Resolve the GraphML <code>id</code> to an {@link IPort} instance.
The port owner is specified by the <code>ownerId</code> attribute.
@param context The current parse context.
@param ownerId The GraphML id of the port owner.
@param id The id that should be resolved.
@returns The {@link IPort} that is defined by the GraphML ids, or <code>null</code> if no such port exists yet.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementResolver%23resolvePort(IParseContext%2Cstring%2Cstring) Online Documentation}*/
abstract resolvePort(context: IParseContext, ownerId: string, id: string): IPort | null;
static $class: Class;
private $_IGraphElementResolver: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGraphElementResolver;
}
export interface IInputHandler {
}
/**
This class defines a handler for key/data elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputHandler Online Documentation}*/
export abstract class IInputHandler {
/**
This method is invoked when no data tag is defined, and the default value should be applied.
@param context the current parse context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputHandler%23applyDefault(IParseContext) Online Documentation}*/
abstract applyDefault(context: IParseContext): void;
/**
Creates an implementation of the interface {@link IInputHandler} from the given definition.
The given object must contain exactly the abstract members of the {@link IInputHandler} interface. No member may
be omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IInputHandler} interface.
@returns An instance of the {@link IInputHandler} interface based on the given definition.*/
static create(interfaceDefinition: {
parseData(context: IParseContext, node: Node): void;
applyDefault(context: IParseContext): void;
precedence: ParsePrecedence;
}): IInputHandler;
/**
This method is invoked each time a data element with matching key is processed.
@param context the current parse context.
@param node the DOM node representing the data element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputHandler%23parseData(IParseContext%2CNode) Online Documentation}*/
abstract parseData(context: IParseContext, node: Node): void;
/**
Gets the parse precedence that is associated with this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IInputHandler%23precedence Online Documentation}*/
abstract readonly precedence: ParsePrecedence;
static $class: Class;
private $_IInputHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IInputHandler;
}
/**
This enumeration defines the valid priorities for {@link IInputHandler} processing.
@see IInputHandler#precedence
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParsePrecedence Online Documentation}*/
export enum ParsePrecedence {
/**
The data is parsed as early as possible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParsePrecedence%23FIRST Online Documentation}*/
FIRST,
/**
The data is parsed before its owner GraphML element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParsePrecedence%23BEFORE_OWNER Online Documentation}*/
BEFORE_OWNER,
/**
The data is parsed after its owner GraphML element, but before any non-data child content.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParsePrecedence%23BEFORE_CHILDREN Online Documentation}*/
BEFORE_CHILDREN,
/**
Default parse priority.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParsePrecedence%23DEFAULT Online Documentation}*/
DEFAULT,
/**
The data is parsed after any non-data child content, but before parsing the owner GraphML element is finished.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParsePrecedence%23AFTER_CHILDREN Online Documentation}*/
AFTER_CHILDREN,
/**
The data is parsed directly after parsing the owner GraphML element is finished.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParsePrecedence%23AFTER_OWNER Online Documentation}*/
AFTER_OWNER,
/**
The data is parsed as late as possible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParsePrecedence%23LAST Online Documentation}*/
LAST
}
/** Automatic conversion to {@link ParsePrecedence}. */
type ParsePrecedenceStringValues = "first" | "before-owner" | "before-children" | "default" | "after-children" | "after-owner" | "last";
export interface InputHandlerBase<TKey, TData> extends YObject, IInputHandler {
}
/**
An abstract base implementation of the {@link IInputHandler} interface.
Type parameter <code>TKey</code>: The type of the key, i.e. the items that are associated with the data in the GraphML.
Type parameter <code>TData</code>: The type of the data that is associated with the keys.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase Online Documentation}*/
export abstract class InputHandlerBase<TKey, TData> {
/**
Initializes a new instance of the {@link InputHandlerBase} class.
The {@link InputHandlerBase#precedence} gets the initial value assigned to <code>precedence</code>
@param keyType The type of the key, i.e. the items that are associated with the data in the GraphML.
@param dataType The type of the data that is associated with the keys.
@param precedence The precedence to assign to the {@link InputHandlerBase#precedence} property.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23InputHandlerBase(ParsePrecedence) Online Documentation}*/
constructor(keyType: Class, dataType: Class, precedence: ParsePrecedence | ParsePrecedenceStringValues);
/**
Initializes a new instance of the {@link InputHandlerBase} class.
The {@link InputHandlerBase#precedence} has an initial value of {@link ParsePrecedence#DEFAULT}.
@param keyType The type of the key, i.e. the items that are associated with the data in the GraphML.
@param dataType The type of the data that is associated with the keys.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23InputHandlerBase() Online Documentation}*/
constructor(keyType: Class, dataType: Class);
/**
This method is invoked when no data tag is defined, and the default value should be applied.
This implementation delegates to {@link InputHandlerBase#setValue} if {@link InputHandlerBase#defaultExists a default exists} using the {@link InputHandlerBase#defaultValue}.
@param context the current parse context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23applyDefault(IParseContext) Online Documentation}*/
applyDefault(context: IParseContext): void;
/**
Initializes this instance from the GraphML key definition.
This implementation looks for the GraphML default element and delegates to {@link InputHandlerBase#parseDataCore} if a default is found to store
the {@link InputHandlerBase#defaultValue}.
@param context The context.
@param definition The definition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23initializeFromKeyDefinition(IParseContext%2CElement) Online Documentation}*/
initializeFromKeyDefinition(context: IParseContext, definition: Element): void;
/**
This method is invoked each time a data element with matching key is processed.
If the {@link IParseContext#getCurrent current element} is of the same type as this <code>TKey</code>, this method delegates the actual parsing to {@link InputHandlerBase#parseDataCore} and
{@link InputHandlerBase#setValue}
@param context the current parse context.
@param node the DOM node representing the data element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23parseData(IParseContext%2CNode) Online Documentation}*/
parseData(context: IParseContext, node: Node): void;
/**
Abstract method callback that actually parses the data.
@param context The context.
@param node The node that contains the data. This is either the GraphML 'data' element or the 'default' element.
@returns The parsed data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23parseDataCore(IParseContext%2CNode) Online Documentation}*/
abstract parseDataCore(context: IParseContext, node: Node): TData | null;
/**
Abstract method callback that actually stores the value with the keys.
@param context The context.
@param key The key.
@param data The data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23setValue(IParseContext%2CTKey%2CTData) Online Documentation}*/
abstract setValue(context: IParseContext, key: TKey, data: TData): void;
/**
Gets or sets a value indicating whether the default value in the key definition should be parsed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23parseDefaultValue Online Documentation}*/
parseDefaultValue: boolean;
/**
Gets or sets the default value that will be applied to the keys where the data attribute is missing.
If any default value is set (including <code>null</code>), {@link InputHandlerBase#defaultExists} is automatically set to true.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23defaultValue Online Documentation}*/
defaultValue: TData | null;
/**
Gets a value indicating whether a default value has been set, either through a key definition element or explicitly by
assigning a value to the {@link InputHandlerBase#defaultValue} property.
If this property is set to <code>true</code>, the {@link InputHandlerBase#applyDefault} method will call the {@link InputHandlerBase#setValue} method using the {@link InputHandlerBase#defaultValue} as the value
parameter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23defaultExists Online Documentation}*/
defaultExists: boolean;
/**
Gets or sets the parse precedence that is associated with this handler.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InputHandlerBase%23precedence Online Documentation}*/
precedence: ParsePrecedence;
static $class: Class;
private $_InputHandlerBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TKey, TData>(o: any): o is InputHandlerBase<TKey, TData>;
}
export interface MapperInputHandler<TKey, TData> extends InputHandlerBase<TKey, TData> {
}
/**
An {@link IInputHandler} implementation that uses an {@link IMapper} instance to associate the values with the keys.
This implementation can be provided an
{@link HandleDeserializationEventArgs}-handler to perform the deserialization. If no such value is provided then
the default deserialization is performed.
The parsed values are stored in a {@link IMapper}.
Type parameter <code>TKey</code>: The type of the key.
Type parameter <code>TData</code>: The type of the value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler Online Documentation}*/
export class MapperInputHandler<TKey, TData> {
/**
Initializes a new instance of the {@link MapperInputHandler} class that uses the event handler for deserialization and suggests the provided
target type for the deserializer.
@param keyType The type of the key.
@param dataType The type of the value.
@param deserializer
@param [deserializerTargetType=null]
@see MapperInputHandler#deserializerTargetType
@see MapperInputHandler#deserializer
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23MapperInputHandler(function(Object%2CHandleDeserializationEventArgs)%3Avoid%2CClass) Online Documentation}*/
constructor(keyType: Class, dataType: Class, deserializer: ((sender: any | null, evt: HandleDeserializationEventArgs | null) => void) | null, deserializerTargetType?: Class | null);
/**
Initializes a new instance of the {@link MapperInputHandler} class that uses the {@link IParseContext} for deserialization.
The {@link IInputHandler#precedence} has an initial value of {@link ParsePrecedence#DEFAULT}.
@param keyType The type of the key.
@param dataType The type of the value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23MapperInputHandler() Online Documentation}*/
constructor(keyType: Class, dataType: Class);
/**
Overridden to take the {@link MapperInputHandler#overrideDefaultValue} property into account.
@param context The context for the parse operation.
@see MapperInputHandler#overrideDefaultValue
@see InputHandlerBase#applyDefault
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23applyDefault(IParseContext) Online Documentation}*/
applyDefault(context: IParseContext): void;
/**
Gets the node to pass to {@link MapperInputHandler#deserializer} or {@link IParseContext#deserializeCore IParseContext.Deserialize} respectively, given the 'data' or 'default' element from the
GraphML.
This method is called by {@link MapperInputHandler#parseDataCore} to determine the node to pass to the deserialization mechanism.
This implementation returns <code>node</code> iff {@link MapperInputHandler#useParentElementForDeserialization} is <code>true</code> (i.e. the <code>'data'</code> or <code>'default'</code> node itself), otherwise the first child of
the <code>node</code> that is not a comment or processing instruction.
@param context The context within which the node is parsed.
@param node The 'data' or 'default' node.
@returns The node that should be deserialized.
@see MapperInputHandler#useParentElementForDeserialization
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23getDeserializationXmlNode(IParseContext%2CNode) Online Documentation}*/
getDeserializationXmlNode(context: IParseContext, node: Node): Node | null;
/**
Parses the data using either the {@link MapperInputHandler#deserializer} or the {@link IParseContext#deserializeCore} call.
@param context The context.
@param node The node that contains the data. This is either the GraphML <code>'data'</code> element or the <code>'default'</code> element.
@returns The parsed data.
@throws {Exception} <b>(DeserializationNotSupportedError)</b> <code>DeserializationNotSupportedException</code> if {@link MapperInputHandler#deserializer} could not handle the node.
@see MapperInputHandler#getDeserializationXmlNode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23parseDataCore(IParseContext%2CNode) Online Documentation}*/
parseDataCore(context: IParseContext, node: Node): TData | null;
/**
This method uses the {@link MapperInputHandler#mapper} to associate the data with the key.
@param context The context.
@param key The key.
@param data The data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23setValue(IParseContext%2CTKey%2CTData) Online Documentation}*/
setValue(context: IParseContext, key: TKey, data: TData): void;
/**
Gets or sets the mapper to use for storing the data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23mapper Online Documentation}*/
mapper: IMapper<TKey, TData>;
/**
Gets or sets a value indicating whether values that are applied via the {@link InputHandlerBase#applyDefault} mechanism and are equal to the
value that is already present in the mapper will explicitly be overridden in the mapper.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23overrideDefaultValue Online Documentation}*/
overrideDefaultValue: boolean;
/**
Gets or sets whether {@link MapperInputHandler#getDeserializationXmlNode} returns either the <code>'data'</code>/<code>'default'</code> element itself or its content.
The default value is <code>false</code>, meaning that {@link MapperInputHandler#getDeserializationXmlNode} should return the content of the <code>'data'</code>/<code>'default'</code>
node.
@see MapperInputHandler#getDeserializationXmlNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23useParentElementForDeserialization Online Documentation}*/
useParentElementForDeserialization: boolean;
/**
Gets or sets the deserializer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23deserializer Online Documentation}*/
deserializer: ((sender: any | null, evt: HandleDeserializationEventArgs | null) => void) | null;
/**
Gets or sets the target deserialization type for the {@link HandleDeserializationEventArgs#targetType} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperInputHandler%23deserializerTargetType Online Documentation}*/
deserializerTargetType: Class | null;
static $class: Class;
private $_MapperInputHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TKey, TData>(o: any): o is MapperInputHandler<TKey, TData>;
}
export interface QueryInputHandlersEventArgs extends EventArgs {
}
/**
Specialized {@link EventArgs} that are used for dynamic {@link IInputHandler} registration.
@see GraphMLIOHandler#addQueryInputHandlersListener
@see GraphMLParser#addQueryInputHandlersListener
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryInputHandlersEventArgs Online Documentation}*/
export class QueryInputHandlersEventArgs {
/**
Create a new instance.
@param context The current parse context.
@param keyDefinition The current key definition
@param handlers Stores the registered {@link IInputHandler} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryInputHandlersEventArgs Online Documentation}*/
constructor(context: IParseContext, keyDefinition: Element, handlers: IList<IInputHandler> | IListConvertible<IInputHandler>);
/**
Register an {@link IInputHandler} instance for the current {@link QueryInputHandlersEventArgs#keyDefinition}.
Calling this method automatically sets {@link QueryInputHandlersEventArgs#handled} to <code>true</code>.
@param handler
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryInputHandlersEventArgs%23addInputHandler(IInputHandler) Online Documentation}*/
addInputHandler(handler: IInputHandler): void;
/**
Gets the key definition of the GraphML attribute.
This XML element should be queried to determine if an {@link IInputHandler} should be registered with
{@link QueryInputHandlersEventArgs#addInputHandler}
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryInputHandlersEventArgs%23keyDefinition Online Documentation}*/
keyDefinition: Element;
/**
Gets or sets whether the current GraphML attribute has been processed.
This is automatically set by calling
{@link QueryInputHandlersEventArgs#addInputHandler}, so it should be seldom necessary to set this property
manually.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryInputHandlersEventArgs%23handled Online Documentation}*/
handled: boolean;
/**
Gets the {@link IParseContext} instance that can queried for further information.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryInputHandlersEventArgs%23context Online Documentation}*/
context: IParseContext;
/**
Gets the {@link IInputHandler}s that have been registered on this event argument instance so far.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryInputHandlersEventArgs%23handlers Online Documentation}*/
readonly handlers: IEnumerable<IInputHandler>;
static $class: Class;
private $_QueryInputHandlersEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is QueryInputHandlersEventArgs;
}
export interface ChildWriteContext extends YObject, IWriteContext {
/**
@inheritdoc*/
getCurrent<T>(itemType: Class): T | null;
/**
@inheritdoc*/
getSerializationProperty(key: string): any | null;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
serializeCore<T>(targetType: Class, context: IWriteContext, item: T | null): void;
/**
@inheritdoc*/
readonly objectStack: IListEnumerable<any>;
/**
@inheritdoc*/
readonly writeEvents: IWriteEvents;
/**
@inheritdoc*/
readonly graph: IGraph;
} /**
An {@link IWriteContext} implementation for use in the context of {@link GraphMLWriter} that wraps a given
instance and delegates most of the calls to it but allows for modifying {@link ChildWriteContext#setLookup lookup} and {@link ChildWriteContext#setSerializationProperty serialization properties}.
Whenever it is necessary to locally modify some context properties, a new instance of this class should be created that
wraps an existing one.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildWriteContext Online Documentation}*/
export class ChildWriteContext {
/**
Initializes a new instance of the {@link ChildWriteContext} class that delegates to the provided context and
returns the given value in {@link ChildWriteContext#getCurrent}.
@param parentContext The parent context to delegate calls to.
@param currentObject The current object to return in {@link ChildWriteContext#getCurrent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildWriteContext%23ChildWriteContext(IWriteContext%2CObject) Online Documentation}*/
constructor(parentContext: IWriteContext, currentObject: any | null);
/**
Initializes a new instance of the {@link ChildWriteContext} class that delegates to the given parent context.
@param parentContext The parent context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildWriteContext%23ChildWriteContext(IWriteContext) Online Documentation}*/
constructor(parentContext: IWriteContext);
/**
Removes a local lookup override that has been added using {@link ChildWriteContext#setLookup}
Type parameter <code>T</code>: The type key to remove.
@param tType The type key to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildWriteContext%23removeLookup(Class) Online Documentation}*/
removeLookup<T>(tType: Class): void;
/**
Removes a previously locally {@link ChildWriteContext#setSerializationProperty set} serialization property.
@param key The key for which the local property override should be removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildWriteContext%23removeSerializationProperty(string) Online Documentation}*/
removeSerializationProperty(key: string): void;
/**
Adds another lookup result to the local lookup override.
Type parameter <code>T</code>: The type to use as a key for the lookup.
@param tType The type to use as a key for the lookup.
@param value The value to provide for queries to <code>T</code>.
@see ChildWriteContext#removeLookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildWriteContext%23setLookup(Class%2CT) Online Documentation}*/
setLookup<T>(tType: Class, value: T | null): void;
/**
Sets the local serialization property for the given key to the value.
@param key The key to override.
@param value The value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildWriteContext%23setSerializationProperty(string%2CObject) Online Documentation}*/
setSerializationProperty(key: string, value: any | null): void;
/**
Gets the parent context to which most calls are delegated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildWriteContext%23parentContext Online Documentation}*/
readonly parentContext: IWriteContext;
/**
Gets or sets the writer.
If a <code>null</code> writer is set, this implementation will delegate to the {@link ChildWriteContext#parentContext}'s {@link IWriteContext#writer}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildWriteContext%23writer Online Documentation}*/
writer: IXmlWriter;
static $class: Class;
private $_ChildWriteContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ChildWriteContext;
}
export interface GraphMLWriter extends YObject {
}
/**
Core writer class for GraphML.
This class allows for low-level customization of the writing process. Usually, it is used by
{@link GraphMLIOHandler}'s <code>write</code> method which calls
<code>GraphMLWriter.write</code>. It should rarely be necessary to use this class directly.
@see GraphMLIOHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter Online Documentation}*/
export class GraphMLWriter {
constructor();
/**
Create the initial {@link IWriteContext} instance.
@param graph The graph that is written.
@param writer The core {@link IXmlWriter} instance that does the actual writing of XML content.
@returns An {@link IWriteContext} instance that is suitable for <code>graph</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23createWriteContext(IGraph%2CIXmlWriter) Online Documentation}*/
createWriteContext(graph: IGraph, writer: IXmlWriter): IWriteContext;
/**
Dynamically retrieve all {@link IOutputHandler} that should be used for the current write process.
Typically, this method is called only indirectly by the write process. This implementation calls
{@link GraphMLWriter#onQueryOutputHandlers}, which in turn raises the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event.
@param context The current write context.
@param scope The scope to get output handlers for.
@returns A dictionary from handler definition ids to {@link IOutputHandler IOutputHandlers}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23getOutputHandlers(IWriteContext%2CKeyScope) Online Documentation}*/
getOutputHandlers(context: IWriteContext, scope: KeyScope | KeyScopeStringValues): IEnumerable<IOutputHandler>;
/**
Retrieve a serialization property that has been set by {@link GraphMLWriter#setSerializationProperty}.
@param key The key for the property.
@see IWriteContext#getSerializationProperty
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23getSerializationProperty(string) Online Documentation}*/
getSerializationProperty(key: string): any | null;
/**
Dynamically retrieve an instance of <code>type</code>.
@param type The type for which an implementation is needed.
@returns An implementation of <code>type</code>, or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23lookup(Class) Online Documentation}*/
lookup(type: Class): any | null;
/**
Fires the {@link GraphMLWriter#addHandleSerializationListener HandleSerialization} event.
@param evt The arguments that get passed to the {@link GraphMLWriter#addHandleSerializationListener HandleSerialization}
event.
@see GraphMLIOHandler#onHandleSerialization
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23onHandleSerialization(HandleSerializationEventArgs) Online Documentation}*/
onHandleSerialization(evt: HandleSerializationEventArgs): void;
/**
Fires the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers} event.
@param evt The arguments that get passed to the {@link GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
event.
@see GraphMLIOHandler#onQueryOutputHandlers
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23onQueryOutputHandlers(QueryOutputHandlersEventArgs) Online Documentation}*/
onQueryOutputHandlers(evt: QueryOutputHandlersEventArgs): void;
/**
Fires the {@link GraphMLWriter#addQueryReferenceIdListener QueryReferenceId} event.
@param evt The arguments that get passed to the {@link GraphMLWriter#addQueryReferenceIdListener QueryReferenceId}
event.
@see GraphMLIOHandler#onQueryReferenceId
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23onQueryReferenceId(QueryReferenceIdEventArgs) Online Documentation}*/
onQueryReferenceId(evt: QueryReferenceIdEventArgs): void;
/**
Raises the {@link GraphMLWriter#addWritingListener Writing} event.
This method is called just before the writing starts and before {@link IWriteEvents#addDocumentWritingListener DocumentWriting}
is called for the first time.
@param evt The {@link WriteEventArgs} instance containing the event data.
@see GraphMLWriter#addWritingListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23onWriting(WriteEventArgs) Online Documentation}*/
onWriting(evt: WriteEventArgs): void;
/**
Raises the {@link GraphMLWriter#addWrittenListener Written} event.
This method is called just before the writing has been finished and after {@link IWriteEvents#addDocumentWrittenListener DocumentWritten}
is called for the last time.
@param evt The {@link WriteEventArgs} instance containing the event data.
@see GraphMLWriter#addWrittenListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23onWritten(WriteEventArgs) Online Documentation}*/
onWritten(evt: WriteEventArgs): void;
/**
Remove an implementation of <code>T</code> that has been set with {@link GraphMLWriter#setLookup}.
Type parameter <code>T</code>.
@param tType
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23removeLookup(Class) Online Documentation}*/
removeLookup<T>(tType: Class): void;
/**
Remove a serialization property that has been set by {@link GraphMLWriter#setSerializationProperty}.
@param key The key for the property.
@see IWriteContext#getSerializationProperty
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23removeSerializationProperty(string) Online Documentation}*/
removeSerializationProperty(key: string): void;
/**
Serialize <code>item</code>.
Typically, this method is called only indirectly by calls to {@link IWriteContext#serializeCore}. This
implementation calls {@link GraphMLWriter#onHandleSerialization}, which in turn raises the {@link GraphMLWriter#addHandleSerializationListener HandleSerialization}
event.
Type parameter <code>T</code>: The desired type of the result.
@param targetType The desired type of the result.
@param context The current parse context.
@param item The object that should be serialized.
@throws {Exception} <b>(SerializationNotSupportedError)</b> <code>SerializationNotSupportedException</code> if <code>item</code> cannot be serialized.
@see IWriteContext#serializeCore
@see GraphMLWriter#addHandleSerializationListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23serialize(Class%2CIWriteContext%2CT) Online Documentation}*/
serialize<T>(targetType: Class, context: IWriteContext, item: T | null): void;
/**
Register an implementation of <code>T</code> for use with {@link GraphMLWriter#lookup}.
Type parameter <code>T</code>: The type of <code>instance</code>.
@param tType The type of <code>instance</code>.
@param instance The implementation of <code>T</code> for lookup retrieval.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23setLookup(Class%2CT) Online Documentation}*/
setLookup<T>(tType: Class, instance: T | null): void;
/**
Set a serialization property that allows to fine tune the write process.
@param key The key for the property.
@param value The property value.
@see IWriteContext#getSerializationProperty
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23setSerializationProperty(string%2CObject) Online Documentation}*/
setSerializationProperty(key: string, value: any | null): void;
/**
Write the GraphML representation of <code>graph</code>, using <code>writer</code> for the actual XML generation.
Typically, this method is called indirectly from {@link GraphMLIOHandler#write}.
@param graph The graph instance that is populated.
@param writer The writer that actually writes the XML content.
@see GraphMLIOHandler#write
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23write(IGraph%2CIXmlWriter) Online Documentation}*/
write(graph: IGraph, writer: IXmlWriter): void;
/**
Adds the given listener for the <code>HandleSerialization</code> event that occurs when XML content is about to be serialized.
Subscribe to this event to provide custom serialization handling for XML content.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
The current item is provided by {@link HandleSerializationEventArgs#item}.
@param listener The listener to add.
@see GraphMLIOHandler#addHandleSerializationListener
@see GraphMLWriter#removeHandleSerializationListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23HandleSerialization Online Documentation}*/
addHandleSerializationListener(listener: (sender: any, evt: HandleSerializationEventArgs) => void): void;
/**
Removes the given listener for the <code>HandleSerialization</code> event that occurs when XML content is about to be serialized.
Subscribe to this event to provide custom serialization handling for XML content.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
The current item is provided by {@link HandleSerializationEventArgs#item}.
@param listener The listener to remove.
@see GraphMLIOHandler#addHandleSerializationListener
@see GraphMLWriter#addHandleSerializationListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23HandleSerialization Online Documentation}*/
removeHandleSerializationListener(listener: (sender: any, evt: HandleSerializationEventArgs) => void): void;
/**
Adds the given listener for the <code>QueryOutputHandlers</code> event that occurs when {@link IOutputHandler}s are queried.
Subscribe to this event to dynamically register one or more {@link IOutputHandler} instance(s) for a given
GraphML attribute key definition.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
If you want to register an {@link IOutputHandler} for this GraphML attribute, you can add the handler with
{@link QueryOutputHandlersEventArgs#addOutputHandler}.
@param listener The listener to add.
@see GraphMLIOHandler#addQueryOutputHandlersListener
@see GraphMLWriter#removeQueryOutputHandlersListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23QueryOutputHandlers Online Documentation}*/
addQueryOutputHandlersListener(listener: (sender: any, evt: QueryOutputHandlersEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryOutputHandlers</code> event that occurs when
{@link IOutputHandler}s are queried.
Subscribe to this event to dynamically register one or more {@link IOutputHandler} instance(s) for a given
GraphML attribute key definition.
The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
earlier.
If you want to register an {@link IOutputHandler} for this GraphML attribute, you can add the handler with
{@link QueryOutputHandlersEventArgs#addOutputHandler}.
@param listener The listener to remove.
@see GraphMLIOHandler#addQueryOutputHandlersListener
@see GraphMLWriter#addQueryOutputHandlersListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23QueryOutputHandlers Online Documentation}*/
removeQueryOutputHandlersListener(listener: (sender: any, evt: QueryOutputHandlersEventArgs) => void): void;
/**
Adds the given listener for the <code>QueryReferenceId</code> event that occurs when a reference is queried.
This event can be used to provide names of external references for objects.
These reference targets are not serialized to the GraphML file.
@param listener The listener to add.
@see GraphMLIOHandler#addQueryReferenceIdListener
@see GraphMLWriter#removeQueryReferenceIdListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23QueryReferenceId Online Documentation}*/
addQueryReferenceIdListener(listener: (sender: any, evt: QueryReferenceIdEventArgs) => void): void;
/**
Removes the given listener for the <code>QueryReferenceId</code> event that occurs when a reference is queried.
This event can be used to provide names of external references for objects.
These reference targets are not serialized to the GraphML file.
@param listener The listener to remove.
@see GraphMLIOHandler#addQueryReferenceIdListener
@see GraphMLWriter#addQueryReferenceIdListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23QueryReferenceId Online Documentation}*/
removeQueryReferenceIdListener(listener: (sender: any, evt: QueryReferenceIdEventArgs) => void): void;
/**
Gets the events in the internal {@link IWriteEvents} implementation.
You can subscribe to any of the events in the returned {@link IWriteEvents} instance to react to various steps in
the writing process
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23writeEvents Online Documentation}*/
readonly writeEvents: IWriteEvents;
/**
Adds the given listener for the <code>Written</code> event that occurs when the writing of a document has just been finished.
@param listener The listener to add.
@see GraphMLWriter#onWritten
@see GraphMLWriter#removeWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23Written Online Documentation}*/
addWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>Written</code> event that occurs when the writing of a document has just been finished.
@param listener The listener to remove.
@see GraphMLWriter#onWritten
@see GraphMLWriter#addWrittenListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23Written Online Documentation}*/
removeWrittenListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Adds the given listener for the <code>Writing</code> event that occurs when the writing of a document is just about to begin.
This event can be used to register to the fine-grained events available in the {@link WriteEventArgs#context}'s {@link IWriteContext#writeEvents} and to configure the writing process
on a per write case.
@param listener The listener to add.
@see GraphMLWriter#onWriting
@see GraphMLWriter#removeWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23Writing Online Documentation}*/
addWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
/**
Removes the given listener for the <code>Writing</code> event that occurs when the writing of a document is just about to begin.
This event can be used to register to the fine-grained events available in the {@link WriteEventArgs#context}'s {@link IWriteContext#writeEvents} and to configure the writing process
on a per write case.
@param listener The listener to remove.
@see GraphMLWriter#onWriting
@see GraphMLWriter#addWritingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriter%23Writing Online Documentation}*/
removeWritingListener(listener: (sender: any, evt: WriteEventArgs) => void): void;
static $class: Class;
private $_GraphMLWriter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphMLWriter;
}
export interface HandleSerializationEventArgs extends EventArgs {
}
/**
The event arguments used by {@link GraphMLIOHandler#addHandleSerializationListener HandleSerialization}
and {@link GraphMLWriter#addHandleSerializationListener HandleSerialization} to let registered
serialization code perform the serialization of items.
If the event handler determines that it can serialize the {@link HandleSerializationEventArgs#item}, it should use the {@link HandleSerializationEventArgs#writer} for output and mark the event as {@link HandleSerializationEventArgs#handled} so that
other code will not perform serialization, too.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleSerializationEventArgs Online Documentation}*/
export class HandleSerializationEventArgs {
/**
Initializes a new instance of the {@link HandleSerializationEventArgs} class using the provided context to write
the <code>item</code> using the given type for serialization.
@param context The context to use for writing.
@param item The item to to serialize.
@param [serializationType=null] The type that should be used for serialization, which needs to be a super type or interface of the <code>item</code> or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleSerializationEventArgs Online Documentation}*/
constructor(context: IWriteContext, item: any | null, serializationType?: Class | null);
/**
Replaces the {@link HandleSerializationEventArgs#item} using a substitute for serialization.
This method can be used to replace the {@link HandleSerializationEventArgs#item} and {@link HandleSerializationEventArgs#sourceType} property so that the serialization chain can be traversed with a
substituted item. If this method has been called the {@link HandleSerializationEventArgs#handled} property should not be set to <code>true</code>.
@param item The substitute to assign to the {@link HandleSerializationEventArgs#item} property.
@param serializationType The serialization type to assign to the {@link HandleSerializationEventArgs#sourceType} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleSerializationEventArgs%23replaceItem(Object%2CClass) Online Documentation}*/
replaceItem(item: any | null, serializationType: Class | null): void;
/**
Gets the item that should be serialized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleSerializationEventArgs%23item Online Documentation}*/
item: any | null;
/**
Gets the type in the inheritance of the {@link HandleSerializationEventArgs#item} that should be used for serialization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleSerializationEventArgs%23sourceType Online Documentation}*/
sourceType: Class | null;
/**
Gets or sets a value indicating whether this {@link HandleSerializationEventArgs} is handled.
Setting this property to <code>true</code> indicates to the event source that the {@link HandleSerializationEventArgs#item} has been successfully serialized using the {@link HandleSerializationEventArgs#writer} and no
other code needs to handle the serialization request, anymore.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleSerializationEventArgs%23handled Online Documentation}*/
handled: boolean;
/**
Gets the context to be used for writing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleSerializationEventArgs%23context Online Documentation}*/
context: IWriteContext;
/**
Gets the writer for writing the xml output.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleSerializationEventArgs%23writer Online Documentation}*/
readonly writer: IXmlWriter;
static $class: Class;
private $_HandleSerializationEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HandleSerializationEventArgs;
}
export interface IGraphElementIdProvider {
}
/**
Interface for classes that provide custom id's for core GraphML elements
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdProvider Online Documentation}*/
export abstract class IGraphElementIdProvider {
/**
Get an ID for the specified <code>edge</code> object
@param context The current write context, for further information to the provider
@param edge The current graph object
@returns an ID for the specified edge object
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdProvider%23getEdgeId(IWriteContext%2CIEdge) Online Documentation}*/
abstract getEdgeId(context: IWriteContext, edge: IEdge): string;
/**
Get an ID for the specified <code>graph</code> object
@param context The current write context, for further information to the provider
@param graph The current graph object
@returns an ID for the specified graph object
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdProvider%23getGraphId(IWriteContext%2CIGraph) Online Documentation}*/
abstract getGraphId(context: IWriteContext, graph: IGraph): string;
/**
Get an ID for the specified <code>node</code> object
@param context The current write context, for further information to the provider
@param node The current graph object
@returns an ID for the specified graph object
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdProvider%23getNodeId(IWriteContext%2CINode) Online Documentation}*/
abstract getNodeId(context: IWriteContext, node: INode): string;
/**
Get an ID for the specified <code>port</code> object
@param context The current write context, for further information to the provider
@param port The current port object
@returns an ID for the specified port object
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphElementIdProvider%23getPortId(IWriteContext%2CIPort) Online Documentation}*/
abstract getPortId(context: IWriteContext, port: IPort): string;
static $class: Class;
private $_IGraphElementIdProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGraphElementIdProvider;
}
export interface MapperOutputHandler<TKey, TData> extends OutputHandlerBase<TKey, TData> {
}
/**
An implementation of the {@link OutputHandlerBase} class, that uses an {@link IMapper} to retrieve the values to output for each item and a {@link MapperOutputHandler#serializer} callback for the
serialization or delegates to the {@link IWriteContext#serializeCore} callback.
Type parameter <code>TKey</code>: The type of the keys.
Type parameter <code>TData</code>: The type of the data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperOutputHandler Online Documentation}*/
export class MapperOutputHandler<TKey, TData> {
/**
Initializes a new instance of the {@link MapperOutputHandler} class using the provided key scope, the name, and the key type.
@param keyType The type of the keys.
@param dataType The type of the data.
@param scope The scope for the key definition attributes.
@param name The name for the key definition attributes.
@param type The type for the key definition attributes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperOutputHandler Online Documentation}*/
constructor(keyType: Class, dataType: Class, scope: KeyScope | KeyScopeStringValues, name: string | null, type: KeyType | KeyTypeStringValues);
/**
Obtains the data for the given key.
@param context The context.
@param key The key.
@returns The data that is associated with the key using the {@link MapperOutputHandler#mapper}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperOutputHandler%23getValue(IWriteContext%2CTKey) Online Documentation}*/
getValue(context: IWriteContext, key: TKey): TData | null;
/**
Writes the contents of the key definition.
This method will additionally write the {@link MapperOutputHandler#mapperMetadata} to the key definition content, if any has been set.
@param context The context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperOutputHandler%23writeKeyDefinitionContent(IWriteContext) Online Documentation}*/
writeKeyDefinitionContent(context: IWriteContext): void;
/**
Performs the actual writing of the data using either the {@link MapperOutputHandler#serializer} or the {@link IWriteContext#serializeCore} method.
@param context The context to use for writing.
@param data The data to write.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperOutputHandler%23writeValueCore(IWriteContext%2CTData) Online Documentation}*/
writeValueCore(context: IWriteContext, data: TData | null): void;
/**
Gets or sets the mapper metadata to serialize to the key definition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperOutputHandler%23mapperMetadata Online Documentation}*/
mapperMetadata: MapperMetadata | null;
/**
Gets or sets the mapper to use for querying the data for each item.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperOutputHandler%23mapper Online Documentation}*/
mapper: IMapper<TKey, TData>;
/**
Gets or sets the serialization callback to use for serializing the value in {@link MapperOutputHandler#writeValueCore}.
@see MapperOutputHandler#sourceType
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperOutputHandler%23serializer Online Documentation}*/
serializer: ((sender: any | null, evt: HandleSerializationEventArgs | null) => void) | null;
/**
Gets or sets the source type of the serialization.
This type will be used as the {@link HandleSerializationEventArgs#sourceType}.
@see MapperOutputHandler#serializer
@see HandleSerializationEventArgs
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperOutputHandler%23sourceType Online Documentation}*/
sourceType: Class | null;
static $class: Class;
private $_MapperOutputHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TKey, TData>(o: any): o is MapperOutputHandler<TKey, TData>;
}
export interface OutputHandlerBase<TKey, TData> extends YObject, IOutputHandler {
/**
@inheritdoc*/
readonly keyDefinitionAttributes: IEnumerable<GraphMLXmlAttribute>;
/**
@inheritdoc*/
readonly dataTagAttributes: IEnumerable<GraphMLXmlAttribute>;
} /**
Abstract base implementation of the {@link IOutputHandler} interface.
Type parameter <code>TKey</code>: The type of the keys.
Type parameter <code>TData</code>: The type of the data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase Online Documentation}*/
export abstract class OutputHandlerBase<TKey, TData> {
/**
Initializes a new instance of the {@link OutputHandlerBase} class using the provided scope, name, and type for the key definition.
@param keyType The type of the keys.
@param dataType The type of the data.
@param scope The scope.
@param name The name to use for the key definition.
@param type The type to use for the key definition.
@see OutputHandlerBase#setKeyDefinitionUri
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase Online Documentation}*/
constructor(keyType: Class, dataType: Class, scope: KeyScope | KeyScopeStringValues, name: string | null, type: KeyType | KeyTypeStringValues);
/**
Obtains the data for the given key.
@param context The context.
@param key The key.
@returns The data that is associated with the key.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23getValue(IWriteContext%2CTKey) Online Documentation}*/
abstract getValue(context: IWriteContext, key: TKey): TData | null;
/**
Determines whether in the current context, the value is the default value and therefore no data element needs to be
written.
This implementation will return <code>false</code> if no {@link OutputHandlerBase#defaultExists default has been set}. Otherwise it will use {@link OutputHandlerBase#getValue} and compare the result
with the {@link OutputHandlerBase#defaultValue} to determine whether they are equal and therefore no data needs to be written.
@param context The context.
@returns <code>true</code> if for the current context the default value applies and therefore no data element needs to be written.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23isDefaultValue(IWriteContext) Online Documentation}*/
isDefaultValue(context: IWriteContext): boolean;
/**
Sets the name for the key definition.
@param name The name of the key element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23setKeyDefinitionName(string) Online Documentation}*/
setKeyDefinitionName(name: string | null): void;
/**
Sets the scope for the key definition.
Note that the official GraphML schema does not support the {@link KeyScope#GRAPHML} so this implementation will
silently ignore this which will result in {@link KeyScope#ALL} implicitly being defined.
@param scope The scope of the key element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23setKeyDefinitionScope(KeyScope) Online Documentation}*/
setKeyDefinitionScope(scope: KeyScope | KeyScopeStringValues): void;
/**
Sets the key type for the key definition.
@param type The type of the key element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23setKeyDefinitionType(KeyType) Online Documentation}*/
setKeyDefinitionType(type: KeyType | KeyTypeStringValues): void;
/**
Sets an optional URI for the key definition.
By default, no URI is set.
@param uri The URI of the key element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23setKeyDefinitionUri(string) Online Documentation}*/
setKeyDefinitionUri(uri: string | null): void;
/**
Writes the contents of the key definition.
Depending on whether {@link OutputHandlerBase#writeKeyDefault} is enabled and {@link OutputHandlerBase#defaultExists} is <code>true</code>, this method will write the GraphML 'default' element and use {@link OutputHandlerBase#writeValueCore}
to serialize the {@link OutputHandlerBase#defaultValue}.
@param context The context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23writeKeyDefinitionContent(IWriteContext) Online Documentation}*/
writeKeyDefinitionContent(context: IWriteContext): void;
/**
Actually writes the value for the current context.
This method will use {@link OutputHandlerBase#getValue} to obtain the data and delegate to {@link OutputHandlerBase#writeValueCore} to perform the actual serialization.
@param context The context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23writeValue(IWriteContext) Online Documentation}*/
writeValue(context: IWriteContext): void;
/**
Performs the actual writing of the data.
@param context The context.
@param data The data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23writeValueCore(IWriteContext%2CTData) Online Documentation}*/
abstract writeValueCore(context: IWriteContext, data: TData | null): void;
/**
Gets or sets the default value.
Setting this value also sets the {@link OutputHandlerBase#defaultExists} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23defaultValue Online Documentation}*/
defaultValue: TData | null;
/**
Gets or sets a value indicating whether a default value has been assigned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23defaultExists Online Documentation}*/
defaultExists: boolean;
/**
Gets or sets the write precedence for the handler.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23precedence Online Documentation}*/
precedence: WritePrecedence;
/**
Gets or sets a value indicating whether to write the key default.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputHandlerBase%23writeKeyDefault Online Documentation}*/
writeKeyDefault: boolean;
static $class: Class;
private $_OutputHandlerBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TKey, TData>(o: any): o is OutputHandlerBase<TKey, TData>;
}
export interface QueryOutputHandlersEventArgs extends EventArgs {
}
/**
Event argument implementation that is used by the {@link GraphMLIOHandler#addQueryOutputHandlersListener QueryOutputHandlers}
event.
These event arguments can be used to add more {@link IOutputHandler} implementations to the {@link GraphMLWriter}
dynamically at the time of writing, by adding them to the {@link QueryOutputHandlersEventArgs#handlers} collection.
@see GraphMLIOHandler
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryOutputHandlersEventArgs Online Documentation}*/
export class QueryOutputHandlersEventArgs {
/**
Initializes a new instance of the {@link QueryOutputHandlersEventArgs} class.
@param context The context to assign to the {@link QueryOutputHandlersEventArgs#context} property.
@param scope The scope where the handlers should be added.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryOutputHandlersEventArgs Online Documentation}*/
constructor(context: IWriteContext, scope: KeyScope | KeyScopeStringValues);
/**
Adds the given output handler to the set of registered output handlers for the given scope.
@param handler The handler to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryOutputHandlersEventArgs%23addOutputHandler(IOutputHandler) Online Documentation}*/
addOutputHandler(handler: IOutputHandler): void;
/**
Gets the scope where the handlers should be added.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryOutputHandlersEventArgs%23scope Online Documentation}*/
readonly scope: KeyScope;
/**
Gets the {@link IOutputHandler}s that have been registered on this event argument instance so far.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryOutputHandlersEventArgs%23handlers Online Documentation}*/
readonly handlers: IEnumerable<IOutputHandler>;
/**
Gets the context for which the handlers are queried.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryOutputHandlersEventArgs%23context Online Documentation}*/
readonly context: IWriteContext;
static $class: Class;
private $_QueryOutputHandlersEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is QueryOutputHandlersEventArgs;
}
export interface XmlWriter extends YObject, IXmlWriter {
/**
@inheritdoc*/
flushDocument(): void;
/**
@inheritdoc*/
getXmlString(): string;
/**
@inheritdoc*/
writeAttributeNS(prefix: string | null, localName: string, ns: string | null, value: string): IXmlWriter;
/**
@inheritdoc*/
writeCData(content: string): IXmlWriter;
/**
@inheritdoc*/
writeComment(comment: string): IXmlWriter;
/**
@inheritdoc*/
writeDocumentFragment(fragment: Document): IXmlWriter;
/**
@inheritdoc*/
writeEndDocument(): void;
/**
@inheritdoc*/
writeEndElement(): IXmlWriter;
/**
@inheritdoc*/
writeProcessingInstruction(target: string, data: string): IXmlWriter;
/**
@inheritdoc*/
writeStartDocument(): IXmlWriter;
/**
@inheritdoc*/
writeStartElement(prefix: string | null, localName: string, ns: string | null): IXmlWriter;
/**
@inheritdoc*/
writeStartElement(localName: string, ns: string | null): IXmlWriter;
/**
@inheritdoc*/
writeString(s: string): IXmlWriter;
} /**
Default implementation of {@link IXmlWriter}.
This writer does not support random access on the output document.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlWriter Online Documentation}*/
export class XmlWriter {
/**
Create a new instance with enabled {@link XmlWriter#indent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlWriter Online Documentation}*/
constructor();
/**
Closes the current element.
@param addEndTag Specifies whether to add an end tag or to write a self-closing element.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlWriter%23closeCurrentElement(boolean) Online Documentation}*/
closeCurrentElement(addEndTag: boolean): void;
/**
Factory method that creates the namespace manager for this instance.
@returns A new manager instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlWriter%23createNamespaceManager() Online Documentation}*/
createNamespaceManager(): IXmlNamespaceManager;
/**
This property controls whether the output should be properly indented.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlWriter%23indent Online Documentation}*/
indent: boolean;
/**
Gets the namespace manager associated with this writer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XmlWriter%23namespaceManager Online Documentation}*/
readonly namespaceManager: IXmlNamespaceManager;
static $class: Class;
private $_XmlWriter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is XmlWriter;
}
export interface IReferenceHandle {
}
/**
Interface that allows a mark and identify shared resources.
@see IReferenceHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReferenceHandle Online Documentation}*/
export abstract class IReferenceHandle {
/**
Creates an implementation of the interface {@link IReferenceHandle} from the given definition.
The given object must contain exactly the abstract members of the {@link IReferenceHandle} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IReferenceHandle} interface.
@returns An instance of the {@link IReferenceHandle} interface based on the given definition.*/
static create(interfaceDefinition: {
referenceType: GraphMLReferenceType;
subject: any;
id: string;
}): IReferenceHandle;
/**
Gets the type of a resource (external or internal)
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReferenceHandle%23referenceType Online Documentation}*/
abstract readonly referenceType: GraphMLReferenceType;
/**
Gets the resource that is encapsulated by the handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReferenceHandle%23subject Online Documentation}*/
abstract readonly subject: any;
/**
Gets the id that identifies the resource in the GraphML document.
Note that this value is ignored for internal references.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReferenceHandle%23id Online Documentation}*/
abstract readonly id: string;
static $class: Class;
private $_IReferenceHandle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IReferenceHandle;
}
/**
The sharing states of objects registered at an {@link IReferenceHandler} via
{@link IReferenceHandler#registerResource}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SharingState Online Documentation}*/
export enum SharingState {
/**
The registered object shall not be shared.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SharingState%23UNSHARED Online Documentation}*/
UNSHARED,
/**
The object has been registered the first time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SharingState%23FIRST_SEEN Online Documentation}*/
FIRST_SEEN,
/**
The object has been registered multiple times and will be shared.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SharingState%23DUPLICATE Online Documentation}*/
DUPLICATE
}
/** Automatic conversion to {@link SharingState}. */
type SharingStateStringValues = "unshared" | "first-seen" | "duplicate";
export interface IReferenceHandler {
}
/**
This interface defines methods to write and configure shared sections in the resulting GraphML output
This interface provides infrastructure for both external and internal (i.e. stored in the GraphML document) references.
Usually, it is not necessary to use this interface directly from client code.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReferenceHandler Online Documentation}*/
export abstract class IReferenceHandler {
/**
Creates an implementation of the interface {@link IReferenceHandler} from the given definition.
The given object must contain exactly the abstract members of the {@link IReferenceHandler} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IReferenceHandler} interface.
@returns An instance of the {@link IReferenceHandler} interface based on the given definition.*/
static create(interfaceDefinition: {
registerResource(context: IWriteContext, subject: any): SharingState;
markAsUnshared(context: IWriteContext, subject: any): void;
getHandle(context: IWriteContext, subject: any | null): IReferenceHandle;
internalHandles: IEnumerable<IReferenceHandle>;
}): IReferenceHandler;
/**
Gets the reference handle for a given subject.
@param subject The subject.
@param context The current write context.
@returns A handle for the resource <code>subject</code>, or <code>null</code> if the resource is not registered.
@see IReferenceHandle
@see IReferenceHandle#subject
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReferenceHandler%23getHandle(IWriteContext%2CObject) Online Documentation}*/
abstract getHandle(context: IWriteContext, subject: any | null): IReferenceHandle | null;
/**
Unregister a shared resource.
This allows to explicitly mark a resource as unshared even if it is referenced multiple times in the object graph. If a
resource has already been registered with {@link IReferenceHandler#registerResource}, it will be unmarked. In
addition, subsequent calls to {@link IReferenceHandler#registerResource} for the same reference won't mark the
resource as shared again.
@param context The current write context.
@param subject The resource to register.
@see IReferenceHandler#registerResource
@see GraphMLAttribute#shareable
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReferenceHandler%23markAsUnshared(IWriteContext%2CObject) Online Documentation}*/
abstract markAsUnshared(context: IWriteContext, subject: any): void;
/**
Register a shared resource.
@param context The current write context.
@param subject The resource to register.
@returns <code>true</code> iff the resource needs to be serialized; otherwise the resource is either a duplicate resource or is declared
externally and does not need any additional handling.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReferenceHandler%23registerResource(IWriteContext%2CObject) Online Documentation}*/
abstract registerResource(context: IWriteContext, subject: any): SharingState;
/**
Gets a collection of all registered reference handles that should be written to the GraphML shared data section.
@see GraphMLIOHandler#addQueryReferenceIdListener
@see GraphMLWriter#addQueryReferenceIdListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReferenceHandler%23internalHandles Online Documentation}*/
abstract readonly internalHandles: IEnumerable<IReferenceHandle>;
static $class: Class;
private $_IReferenceHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IReferenceHandler;
}
/**
Specifies the type of the shared resource.
@see IReferenceHandle#referenceType
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLReferenceType Online Documentation}*/
export enum GraphMLReferenceType {
/**
Specifies an internal resource, that is the serialized form of the resource is stored in the GraphML document itself.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLReferenceType%23INTERNAL Online Documentation}*/
INTERNAL,
/**
Specifies an external resource, that is the resource description is not stored in the GraphML document itself.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLReferenceType%23EXTERNAL Online Documentation}*/
EXTERNAL
}
/** Automatic conversion to {@link GraphMLReferenceType}. */
type GraphMLReferenceTypeStringValues = "internal" | "external";
export interface ResolveReferenceEventArgs extends EventArgs {
}
/**
Event arguments for calls to the {@link GraphMLIOHandler#addResolveReferenceListener ResolveReference}
and {@link GraphMLIOHandler#addOverrideResolveReferenceListener OverrideResolveReference} events.
@see GraphMLIOHandler#addResolveReferenceListener
@see GraphMLIOHandler#addOverrideResolveReferenceListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResolveReferenceEventArgs Online Documentation}*/
export class ResolveReferenceEventArgs {
/**
Creates a new instance of {@link ResolveReferenceEventArgs}.
The {@link ResolveReferenceEventArgs#handled} property is set to <code>false</code>, the {@link ResolveReferenceEventArgs#value} property is set to <code>null</code>
@param context The current parse context that can be used by the event handler for additional information.
@param referenceId The reference id that should evaluated by the event handler.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResolveReferenceEventArgs Online Documentation}*/
constructor(context: IParseContext, referenceId: string);
/**
Gets the reference id that should evaluated by the event handler.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResolveReferenceEventArgs%23referenceId Online Documentation}*/
referenceId: string;
/**
Gets or sets the reference value that is encapsulated by this instance.
Setting this property automatically sets {@link ResolveReferenceEventArgs#handled} to <code>true</code>.
@see ResolveReferenceEventArgs#handled
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResolveReferenceEventArgs%23value Online Documentation}*/
value: any;
/**
Gets whether the last event handler invocation has actually handled the event.
This property is set implicitly by {@link ResolveReferenceEventArgs#value}.
@see ResolveReferenceEventArgs#value
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResolveReferenceEventArgs%23handled Online Documentation}*/
handled: boolean;
/**
Gets the current parse context that can be used by the event handler for additional information.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResolveReferenceEventArgs%23context Online Documentation}*/
context: IParseContext;
static $class: Class;
private $_ResolveReferenceEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ResolveReferenceEventArgs;
}
export interface QueryReferenceIdEventArgs extends EventArgs {
}
/**
Event arguments for calls to the {@link GraphMLIOHandler#addQueryReferenceIdListener QueryReferenceId}
or {@link GraphMLWriter#addQueryReferenceIdListener QueryReferenceId} events.
Event handling code should inspect the {@link QueryReferenceIdEventArgs#value} property and decide whether to set the {@link QueryReferenceIdEventArgs#referenceId} and/or {@link QueryReferenceIdEventArgs#referenceType} property in order to {@link QueryReferenceIdEventArgs#handled handle} the
event.
@see GraphMLIOHandler#addQueryReferenceIdListener
@see GraphMLWriter#addQueryReferenceIdListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryReferenceIdEventArgs Online Documentation}*/
export class QueryReferenceIdEventArgs {
/**
Creates a new instance.
The {@link QueryReferenceIdEventArgs#handled} property is set to <code>false</code>, the {@link QueryReferenceIdEventArgs#referenceId} is set to <code>null</code>.
@param context The current write context that can be used by the event handler for additional information.
@param value The referenced object instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryReferenceIdEventArgs Online Documentation}*/
constructor(context: IWriteContext, value: any);
/**
Gets or sets the reference id that should be evaluated by the event handler.
Setting this property to a non-null string automatically sets {@link QueryReferenceIdEventArgs#handled} to <code>true</code>.
@see QueryReferenceIdEventArgs#handled
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryReferenceIdEventArgs%23referenceId Online Documentation}*/
referenceId: string | null;
/**
Gets or sets the type of the reference being queried.
Setting this property automatically sets {@link QueryReferenceIdEventArgs#handled} to <code>true</code>. By default this property is {@link GraphMLReferenceType#EXTERNAL}
so {@link QueryReferenceIdEventArgs#value}s that are given a {@link QueryReferenceIdEventArgs#referenceId} are treated as external references that are not serialized to the GraphML. Setting this property
to {@link GraphMLReferenceType#INTERNAL} and specifying a {@link QueryReferenceIdEventArgs#referenceId} will make the reference mechanism use the provided
reference but will cause it to serialize the value to the GraphML, nevertheless.
@see QueryReferenceIdEventArgs#referenceId
@see QueryReferenceIdEventArgs#handled
@default GraphMLReferenceType.EXTERNAL
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryReferenceIdEventArgs%23referenceType Online Documentation}*/
referenceType: GraphMLReferenceType;
/**
Gets the reference value that is encapsulated by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryReferenceIdEventArgs%23value Online Documentation}*/
value: any;
/**
Gets whether the last event handler invocation has actually handled the event.
This property is set implicitly by {@link QueryReferenceIdEventArgs#referenceId}.
@see QueryReferenceIdEventArgs#referenceId
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryReferenceIdEventArgs%23handled Online Documentation}*/
handled: boolean;
/**
Gets the current write context that can be used by the event handler for additional information.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryReferenceIdEventArgs%23context Online Documentation}*/
context: IWriteContext;
static $class: Class;
private $_QueryReferenceIdEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is QueryReferenceIdEventArgs;
}
export interface GraphMLParseValueSerializerContext extends YObject, IValueSerializerContext {
/**
@inheritdoc*/
getValueSerializerFor(property: Property): ValueSerializer | null;
/**
@inheritdoc*/
getValueSerializerFor(type: Class): ValueSerializer | null;
} /**
Special implementation of {@link IValueSerializerContext} that should be used in a GraphML parse context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParseValueSerializerContext Online Documentation}*/
export class GraphMLParseValueSerializerContext {
/**
Creates a new instance that wraps <code>context</code>
@param context The current write context
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParseValueSerializerContext Online Documentation}*/
constructor(context: IParseContext);
/**
Returns the wrapped {@link IParseContext} for this serviceType, otherwise <code>null</code>
@param serviceType The serviceType
@returns the wrapped {@link IParseContext} for this serviceType, otherwise <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParseValueSerializerContext%23lookup(Class) Online Documentation}*/
lookup(serviceType: Class): Object | null;
/**
Gets or sets the parent object if such an object exists currently.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLParseValueSerializerContext%23owner Online Documentation}*/
owner: Object | null;
static $class: Class;
private $_GraphMLParseValueSerializerContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphMLParseValueSerializerContext;
}
export interface GraphMLWriteValueSerializerContext extends YObject, IValueSerializerContext {
/**
@inheritdoc*/
getValueSerializerFor(property: Property): ValueSerializer | null;
/**
@inheritdoc*/
getValueSerializerFor(type: Class): ValueSerializer | null;
} /**
Special implementation of {@link IValueSerializerContext} that should be used in a GraphML write context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriteValueSerializerContext Online Documentation}*/
export class GraphMLWriteValueSerializerContext {
/**
Creates a new instance that wraps <code>context</code>
@param context The current write context
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriteValueSerializerContext Online Documentation}*/
constructor(context: IWriteContext);
/**
Returns the wrapped {@link IWriteContext} for this serviceType, otherwise <code>null</code>
@param serviceType The serviceType
@returns the wrapped {@link IWriteContext} for this serviceType, otherwise <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriteValueSerializerContext%23lookup(Class) Online Documentation}*/
lookup(serviceType: Class): Object | null;
/**
Gets or sets the parent object if such an object exists currently.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphMLWriteValueSerializerContext%23owner Online Documentation}*/
owner: Object | null;
static $class: Class;
private $_GraphMLWriteValueSerializerContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphMLWriteValueSerializerContext;
}
export interface QueryNameEventArgs extends EventArgs {
}
/**
The event arguments used by {@link GraphMLIOHandler#addQueryNameListener QueryName} to
override automatic name determination in GraphML.
If the event handler determines that it map resolve {@link QueryNameEventArgs#type} to a xml name, it should place the result into the {@link QueryNameEventArgs#result} property and
thus mark the event as {@link QueryNameEventArgs#handled}.
It is valid to provide only part of the XML name by setting the other part to null. In that case, the default name
determination rules are used for that part.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryNameEventArgs Online Documentation}*/
export class QueryNameEventArgs {
/**
Initializes a new instance of the {@link QueryNameEventArgs} class.
@param context The context in which the lookup shall take place.
@param type The type that should be mapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryNameEventArgs Online Documentation}*/
constructor(context: IWriteContext, type: Class);
/**
Gets or sets the result of the name query, which is <code>null</code> initially.
Setting this property will automatically set the {@link QueryNameEventArgs#handled} property to <code>true</code>.
It is valid to provide only part of the XML name by setting the other part to null. In that case, the default name
determination rules are used for that part.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryNameEventArgs%23result Online Documentation}*/
result: XmlName | null;
/**
Gets or sets a value indicating whether this {@link QueryNameEventArgs} is handled.
Setting this property to <code>true</code> indicates to the event source that the {@link QueryNameEventArgs#result} has been assigned and that the event should not be
propagated to further listeners.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryNameEventArgs%23handled Online Documentation}*/
handled: boolean;
/**
Gets the type for which the {@link QueryNameEventArgs#result XML name} is queried.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryNameEventArgs%23type Online Documentation}*/
readonly type: Class;
/**
Gets the context in which the {@link QueryNameEventArgs#type} shall mapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryNameEventArgs%23context Online Documentation}*/
readonly context: IWriteContext;
static $class: Class;
private $_QueryNameEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is QueryNameEventArgs;
}
export interface QueryTypeEventArgs extends EventArgs {
}
/**
The event arguments used by {@link GraphMLIOHandler#addQueryTypeListener QueryType} to
override automatic type resolution in GraphML.
If the event handler determines that it can resolve {@link QueryTypeEventArgs#xmlName} to a type, it should place the result into the {@link QueryTypeEventArgs#result} property and thus
mark the event as {@link QueryTypeEventArgs#handled}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryTypeEventArgs Online Documentation}*/
export class QueryTypeEventArgs {
/**
Initializes a new instance of the {@link QueryTypeEventArgs} class.
@param context The context in the lookup should take place.
@param xName The type that should be mapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryTypeEventArgs Online Documentation}*/
constructor(context: IParseContext, xName: XmlName);
/**
Gets or sets the result of the type query, which is <code>null</code> initially.
Setting this property will automatically set the {@link QueryTypeEventArgs#handled} property to <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryTypeEventArgs%23result Online Documentation}*/
result: Class | null;
/**
Gets or sets a value indicating whether this {@link QueryTypeEventArgs} is handled.
Setting this property to <code>true</code> indicates to the event source that the {@link QueryTypeEventArgs#result} has been assigned and that the event should not be
propagated to further listeners.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryTypeEventArgs%23handled Online Documentation}*/
handled: boolean;
/**
Gets the fully qualified xml name that should be resolved to a type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryTypeEventArgs%23xmlName Online Documentation}*/
readonly xmlName: XmlName;
/**
Gets the context in which the {@link Class} shall mapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/QueryTypeEventArgs%23context Online Documentation}*/
readonly context: IParseContext;
static $class: Class;
private $_QueryTypeEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is QueryTypeEventArgs;
}
export interface XamlReader extends YObject {
}
/**
An implementation that can read (a subset of) XAML formatted XML content.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlReader Online Documentation}*/
export class XamlReader {
/**
Deserializes the specified element node in the context to an instance.
@param context The context.
@param element The XML element node.
@returns The instance that is described in the XML.
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlReader%23deserialize(IParseContext%2CElement) Online Documentation}*/
deserialize(context: IParseContext, element: Element): any | null;
/**
A singleton instance of this class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlReader%23INSTANCE Online Documentation}*/
static INSTANCE: XamlReader;
static $class: Class;
private $_XamlReader: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is XamlReader;
}
export interface IXamlPrefixMapper {
}
/**
Interface for mapping XAML namespaces to XML namespace prefixes and vice versa.
This interface is only used to specify mappings between namespaces that are used for serialization or deserialization
with classes {@link XamlSerializer} and
{@link XamlReader}. If no prefix mapping is specified for a namespace, a default XML namespace/prefix mapping is
used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXamlPrefixMapper Online Documentation}*/
export abstract class IXamlPrefixMapper {
/**
Gets the namespace for the given prefix.
@param prefix The prefix.
@returns The namespace.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXamlPrefixMapper%23getNamespaceOfPrefix(string) Online Documentation}*/
abstract getNamespaceOfPrefix(prefix: string | null): string | null;
/**
Gets the prefix of namespace.
@param ns The namespace.
@returns The prefix to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IXamlPrefixMapper%23getPrefixOfNamespace(string) Online Documentation}*/
abstract getPrefixOfNamespace(ns: string): string | null;
static $class: Class;
private $_IXamlPrefixMapper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IXamlPrefixMapper;
}
export interface XamlSerializer extends YObject {
}
/**
Serializer that produces XAML conforming output from the target object's reflection data.
By default, the resulting XAML output follows the rules for normal XAML output. In addition, serialization can be fine
tuned by applying {@link GraphMLAttribute} to the target object's class or some of its members.
Note that usually the type should be excluded from obfuscation, or its name should be declared with
{@link GraphMLAttribute#name}. The same holds for specific members.
@see GraphMLAttribute
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlSerializer Online Documentation}*/
export class XamlSerializer {
/**
Serialize the given object to a {@link IXmlWriter}
This implementation already provides the toplevel XML element with an appropriate tag name.
@param context The current write context
@param subject The actual object
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlSerializer%23serialize(IWriteContext%2CObject) Online Documentation}*/
serialize(context: IWriteContext, subject: any): void;
/**
A singleton instance of this class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/XamlSerializer%23INSTANCE Online Documentation}*/
static INSTANCE: XamlSerializer;
static $class: Class;
private $_XamlSerializer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is XamlSerializer;
}
export interface CreateBendInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} used for recognizing the gesture of creating {@link IBend}s in an {@link IGraph}.
* This mode is used by {@link GraphEditorInputMode} but can be used stand-alone, too. Client code should register
* with the {@link CreateBendInputMode#addBendCreatedListener BendCreated} event to perform
* additional actions after the bend has been created.
*
* This mode is {@link CreateBendInputMode#exclusive} by default.
*
* Typically the <code>CreateBendInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} and can be retrieved
* from the {@link GraphEditorInputMode#createBendInputMode} property.
*
* <br><b>Getting the CreateBendInputMode from its parent input mode</b>
* <pre>
* const createBendInputMode = mode.createBendInputMode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode Online Documentation}
*/
export class CreateBendInputMode {
/**
Creates a new instance that will use the {@link IGraph} from the {@link CreateBendInputMode#inputModeContext} to create bends in.
@see CreateBendInputMode#graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode Online Documentation}*/
constructor();
/**
Called when a bend creation gesture can be started at the current mouse pointer location.
This implementation sets the {@link ConcurrencyController#preferredCursor} to {@link CreateBendInputMode#armedCursor}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23arm() Online Documentation}*/
arm(): void;
/**
Cancels the editing of this mode.
This implementation {@link ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
{@link CreateBendInputMode#onCanceled}.
@see CreateBendInputMode#onCanceled
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Creates the bend.
The bend is created using the hit edge's {@link IBendCreator} implementation, as obtained from the edge's
{@link ILookup#lookup}.
@param edge The hit edge.
@param location The coordinates of the hit.
@returns The created bend or <code>null</code>.
@see CreateBendInputMode#createInputModeContext
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23createBend(IEdge%2CPoint) Online Documentation}*/
createBend(edge: IEdge, location: Point): IBend | null;
/**
Creates an {@link IInputModeContext} for use with the upcoming {@link IBendCreator#createBend} call in
{@link CreateBendInputMode#createBend}.
@returns An instance of {@link IInputModeContext}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23createInputModeContext() Online Documentation}*/
createInputModeContext(): IInputModeContext;
/**
Called when a bend creation gesture can no longer be started at the current mouse pointer location.
This implementation sets the {@link ConcurrencyController#preferredCursor} back to <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23disarm() Online Documentation}*/
disarm(): void;
/**
Updates the {@link CreateBendInputMode#splitOrthogonalSegment} property and then delegates to {@link CreateBendInputMode#createBend}.
This method triggers the {@link CreateBendInputMode#addBendCreatedListener BendCreated} event
if a bend is created.
@param edge The edge to create a bend for.
@param dragLocation The location at which the user initiated the gesture.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23dragSegment(IEdge%2CPoint) Online Documentation}*/
dragSegment(edge: IEdge, dragLocation: Point): void;
/**
Finds the edge at the given coordinate.
@param location The coordinates.
@returns The edge that has been hit.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23getEdge(Point) Online Documentation}*/
getEdge(location: Point): IEdge | null;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link CreateBendInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Determines whether the current location is valid to begin a bend creation gesture.
@param source
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23isValidBegin(Object%2CEventArgs) Online Documentation}*/
isValidBegin(source: any, evt: EventArgs): boolean;
/**
Triggers the {@link CreateBendInputMode#addBendCreatedListener BendCreated} event.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23onBendCreated(BendEventArgs) Online Documentation}*/
onBendCreated(evt: BendEventArgs): void;
/**
Called after {@link CreateBendInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Triggers the {@link CreateBendInputMode#addDragCanceledListener DragCanceled} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23onDragCanceled(InputModeEventArgs) Online Documentation}*/
onDragCanceled(evt: InputModeEventArgs): void;
/**
Triggered at the end of each drag.
This method triggers the {@link CreateBendInputMode#addDraggedListener Dragged} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23onDragged(InputModeEventArgs) Online Documentation}*/
onDragged(evt: InputModeEventArgs): void;
/**
Triggered at the start of each drag.
This method triggers the {@link CreateBendInputMode#addDraggingListener Dragging} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23onDragging(InputModeEventArgs) Online Documentation}*/
onDragging(evt: InputModeEventArgs): void;
/**
Called after {@link CreateBendInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link CreateBendInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link CreateBendInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the CreateBendInputMode on its parent input mode</b>
* <pre>
* mode.createBendInputMode.enabled = false
* </pre>
*
* Note that in order to disable interactive bend creation one has to disable bend creation during edge creation, too:
*
* <br><b>Disabling the bend creation during edge creation and on existing edges</b>
* <pre>
* // mode is an instance of GraphEditorInputMode
* mode.createBendInputMode.enabled = false
* mode.createEdgeInputMode.allowCreateBend = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link CreateBendInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Adds the given listener for the <code>BendCreated</code> event that occurs once a bend creation gesture has been recognized.
@param listener The listener to add.
@see CreateBendInputMode#removeBendCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23BendCreated Online Documentation}*/
addBendCreatedListener(listener: (sender: any, evt: BendEventArgs) => void): void;
/**
Removes the given listener for the <code>BendCreated</code> event that occurs once a bend creation gesture has been recognized.
@param listener The listener to remove.
@see CreateBendInputMode#addBendCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23BendCreated Online Documentation}*/
removeBendCreatedListener(listener: (sender: any, evt: BendEventArgs) => void): void;
/**
Gets or sets the cursor to use when this mode is in "armed" state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23armedCursor Online Documentation}*/
armedCursor: Cursor | null;
/**
Gets or sets an event recognizer that determines when the user begins to move the selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23pressedRecognizer Online Documentation}*/
pressedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets an event recognizer that determines when the user is moving the selection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23draggedRecognizer Online Documentation}*/
draggedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets an event recognizer that determines when the user has finished the move.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23releasedRecognizer Online Documentation}*/
releasedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets an event recognizer that determines when the user begins to move the selection via touch.
The default value is {@link TouchEventRecognizers#TOUCH_LONG_PRESS_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23prepareRecognizerTouch Online Documentation}*/
prepareRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets an event recognizer that determines when the user begins actually moves the selection via touch.
The default value is {@link TouchEventRecognizers#TOUCH_MOVE_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23movedRecognizerTouch Online Documentation}*/
movedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets an event recognizer that determines when the finalized the gesture via touch.
The default value is {@link TouchEventRecognizers#TOUCH_UP_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23releasedRecognizerTouch Online Documentation}*/
releasedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the {@link IHitTestable} that determines where bends may be created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23beginHitTestable Online Documentation}*/
beginHitTestable: IHitTestable;
/**
Gets the location where the bend creation gesture was initialized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23location Online Documentation}*/
readonly location: Point;
/**
Gets the graph this mode is acting upon.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23graph Online Documentation}*/
readonly graph: IGraph | null;
/**
Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to add.
@see CreateBendInputMode#removeDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23Dragging Online Documentation}*/
addDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to remove.
@see CreateBendInputMode#addDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23Dragging Online Documentation}*/
removeDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to add.
@see CreateBendInputMode#removeDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23Dragged Online Documentation}*/
addDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to remove.
@see CreateBendInputMode#addDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23Dragged Online Documentation}*/
removeDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to add.
@see CreateBendInputMode#removeDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23DragCanceled Online Documentation}*/
addDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to remove.
@see CreateBendInputMode#addDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23DragCanceled Online Documentation}*/
removeDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Gets or sets an event recognizer that determines whether the bend creation should be treated as a split segment
operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23splitOrthogonalSegmentRecognizer Online Documentation}*/
splitOrthogonalSegmentRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets a value indicating whether an orthogonal segment should be split by the last bend creation.
This property can be read by implementations of {@link IBendCreator} if they retrieve this instance from their {@link IInputModeContext#parentInputMode}. If
this property is
<code>true</code>, implementation are advised to treat the bend creation as a gesture that splits the segment into two.
@see CreateBendInputMode#splitOrthogonalSegmentRecognizer
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23splitOrthogonalSegment Online Documentation}*/
splitOrthogonalSegment: boolean;
/**
Gets or sets a property that determines whether this mode should use an existing bend in {@link CreateBendInputMode#createBend}
if there is one found at the given location.
The default is <code>true</code>
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateBendInputMode%23useExistingBend Online Documentation}*/
useExistingBend: boolean;
static $class: Class;
private $_CreateBendInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CreateBendInputMode;
}
export interface EditLabelHelper extends YObject, IEditLabelHelper {
}
/**
Simple default implementation of the {@link IEditLabelHelper} interface.
This class provided a number of factory methods that can be overwritten to change the default behavior which is modeled
after the behavior used by {@link GraphEditorInputMode}'s {@link GraphEditorInputMode#createLabel} and
{@link GraphEditorInputMode#editLabel}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EditLabelHelper Online Documentation}*/
export class EditLabelHelper {
/**
Determines the label model parameter to use for the label, depending on the <code>owner</code>.
This method will obtain the default parameter instances from the {@link EditLabelHelper#graph} considering the type of the current <code>owner</code>.
@param context The input mode context.
@param owner The label's owner.
@returns The layout parameter to use or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EditLabelHelper%23getLabelParameter(IInputModeContext%2CILabelOwner) Online Documentation}*/
getLabelParameter(context: IInputModeContext, owner: ILabelOwner): ILabelModelParameter | null;
/**
Determines the style to use for the label, depending on the <code>owner</code>.
This method will obtain the default style instances from the {@link EditLabelHelper#graph} considering the type of the current <code>owner</code>.
@param context The input mode context.
@param owner The label's owner.
@returns The style to use or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EditLabelHelper%23getLabelStyle(IInputModeContext%2CILabelOwner) Online Documentation}*/
getLabelStyle(context: IInputModeContext, owner: ILabelOwner): ILabelStyle | null;
/**
Gets the preferred size for the label.
This implementation yields <code>null</code> to indicate that the preferred size should be calculated after the text has been
entered.
@param context The input mode context.
@param label The label to get the preferred size from.
@returns The size to use or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EditLabelHelper%23getPreferredSize(IInputModeContext%2CILabel) Online Documentation}*/
getPreferredSize(context: IInputModeContext, label: ILabel): Size | null;
/**
Helper method that will be called for {@link ILabelOwner}s.
This method delegates to {@link EditLabelHelper#getLabelParameter}, {@link EditLabelHelper#getLabelStyle}, and
{@link EditLabelHelper#getPreferredSize} to set the appropriate properties on <code>evt</code> and sets the {@link LabelEditingEventArgs#handled} property to
<code>true</code>. Label creation thus is always possible.
@param evt The original event arguments.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EditLabelHelper%23onLabelAdding(LabelEditingEventArgs) Online Documentation}*/
onLabelAdding(evt: LabelEditingEventArgs): void;
/**
Helper method that will be called for {@link ILabelOwner}s.
This method returns the first of the {@link ILabelOwner#labels} that belong to the {@link LabelEditingEventArgs#owner}, if set. Otherwise, it delegates to
{@link EditLabelHelper#getLabelParameter}, {@link EditLabelHelper#getLabelStyle}, and {@link EditLabelHelper#getPreferredSize}
to set the appropriate properties on <code>evt</code>.
The arguments are always set as {@link LabelEditingEventArgs#handled}, so label editing is always possible.
@param evt The original event arguments.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EditLabelHelper%23onLabelEditing(LabelEditingEventArgs) Online Documentation}*/
onLabelEditing(evt: LabelEditingEventArgs): void;
/**
Gets or sets the graph instance to retrieve the default values from.
If this instance is not set, the implementation will try to retrieve the graph from the {@link IInputModeContext}'s
{@link ILookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EditLabelHelper%23graph Online Documentation}*/
graph: IGraph | null;
static $class: Class;
private $_EditLabelHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EditLabelHelper;
}
/**
Used by {@link CreateEdgeInputMode} to determine the {@link EdgeDirectionPolicy direction} in which edges get created.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDirectionPolicy Online Documentation}*/
export enum EdgeDirectionPolicy {
/**
The default policy where edges can only be created from source to target port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDirectionPolicy%23START_AT_SOURCE Online Documentation}*/
START_AT_SOURCE,
/**
A policy where edges can only be created from target to source port in {@link CreateEdgeInputMode#reversedEdgeCreation reverse} direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDirectionPolicy%23START_AT_TARGET Online Documentation}*/
START_AT_TARGET,
/**
A policy where edges will be created in the same direction as last time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDirectionPolicy%23KEEP_DIRECTION Online Documentation}*/
KEEP_DIRECTION,
/**
* A policy where edges will be created in the direction depending on whether the creation started at a {@link IPortCandidateProvider#getAllSourcePortCandidates source} or {@link IPortCandidateProvider#getAllTargetPortCandidates target}
* {@link IPortCandidate}.
* With this policy {@link CreateEdgeInputMode} queries both {@link IPortCandidateProvider#getAllSourcePortCandidates}
* and {@link IPortCandidateProvider#getAllTargetPortCandidates} for a port candidate to start edge creation from. If
* the chosen candidate has been returned by the latter method the edge will be created in reversed order.
*
* This requires a custom IPortCandidateProvider which implements both {@link IPortCandidateProvider#getAllSourcePortCandidates}
* and {@link IPortCandidateProvider#getAllTargetPortCandidates} to return candidates to start edge creation at the
* source port and at the target port, respectively.
*
* <pre>
* public class TagSensitivePortCandidateProvider : IPortCandidateProvider
* {
*   private readonly IPortOwner owner;
*
*   public TagSensitivePortCandidateProvider(IPortOwner owner) {
*     this.owner = owner;
*   }
*
*   public IEnumerable<IPortCandidate> GetSourcePortCandidates(IInputModeContext context) {
*     return from port in owner.Ports where "Source".Equals(port.Tag) select new DefaultPortCandidate(port);
*   }
*
*   public IEnumerable<IPortCandidate> GetTargetPortCandidates(IInputModeContext context) {
*     return from port in owner.Ports where "Target".Equals(port.Tag) select new DefaultPortCandidate(port);
*   }
*
*   ...
*
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDirectionPolicy%23DETERMINE_FROM_PORT_CANDIDATES Online Documentation}
*/
DETERMINE_FROM_PORT_CANDIDATES
}
/** Automatic conversion to {@link EdgeDirectionPolicy}. */
type EdgeDirectionPolicyStringValues = "start-at-source" | "start-at-target" | "keep-direction" | "determine-from-port-candidates";
export interface CreateEdgeInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
* An {@link IInputMode} for interactively creating edges between nodes and edges in an {@link IGraph} displayed
* in a {@link CanvasComponent} or {@link GraphComponent}.
* This class provides a {@link CreateEdgeInputMode#edgeCreator factory callback} that the final edge creation will be delegated to. The styling of the newly
* created edge can be customized via the {@link CreateEdgeInputMode#edgeDefaults}. Defaults can be automatically adjusted in response to the various events that
* will be triggered by this instance during the edge creation. Use the {@link CreateEdgeInputMode#dummyEdgeGraph} property to modify the appearance of the dummy
* edge. This class uses {@link IPortCandidateProvider} instances found in the {@link ILookup} of the {@link INode nodes} in
* the graph to display port candidates and to finally create the edges.
*
* This mode is {@link CreateEdgeInputMode#exclusive} by default.
*
* Typically the <code>CreateEdgeInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} and can be retrieved
* from the {@link GraphEditorInputMode#createEdgeInputMode} property.
*
* <br><b>Getting the CreateEdgeInputMode from its parent input mode</b>
* <pre>
* const createEdgeInputMode = mode.createEdgeInputMode
* </pre>
*
* An overview of this input mode is given in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/interaction-creating_edges Creating Edges}. Additionally, customization of edge creation is
* discussed in section {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_interaction_creating_edges#customizing_interaction_creating_edges Customizing User Interaction: Creating Edges}.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode Online Documentation}
*/
export class CreateEdgeInputMode {
/**
Creates a new instance that will use the {@link IGraph} from the {@link CreateEdgeInputMode#inputModeContext} to create edges in.
Setting the {@link CreateEdgeInputMode#graph} property can be used to optionally set a different graph to work on.
@see CreateEdgeInputMode#graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode Online Documentation}*/
constructor();
/**
Cancels the editing of this mode.
This implementation {@link ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
{@link CreateEdgeInputMode#onCanceled}.
@see CreateEdgeInputMode#onCanceled
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Creates the dummy edge that will be displayed by the input mode during the creation.
The edge must be part of the {@link CreateEdgeInputMode#dummyEdgeGraph} and therefore be created by one of its factory methods {@link IGraph#createEdge}.
@returns An {@link IEdge} implementation that can be used as a dummy.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23createDummyEdge() Online Documentation}*/
createDummyEdge(): IEdge;
/**
Factory method that will create the dummy graph to hold the preview of the {@link CreateEdgeInputMode#dummyEdge}.
Note that the instance returned by this instance must accept any port as valid source and target ports for it's {@link IGraph#setEdgePorts}
method. This will allow the preview edge to be connected to the actual {@link CreateEdgeInputMode#graph}.
@returns A graph instance that will be rendered as a preview.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23createDummyEdgeGraph() Online Documentation}*/
createDummyEdgeGraph(): IGraph;
/**
Creates the dummy node which is used as target for the {@link CreateEdgeInputMode#dummyEdge} during the creation.
The node must be part of the {@link CreateEdgeInputMode#dummyEdgeGraph} and therefore be created by one of its factory methods {@link IGraph#createNode}.
This implementation creates a node at origin using the {@link IGraph#nodeDefaults} of the {@link CreateEdgeInputMode#dummyEdgeGraph} (zero size and invisible).
@returns An {@link INode} implementation that can be used as a dummy.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23createDummyTargetNode() Online Documentation}*/
createDummyTargetNode(): INode;
/**
Creates the dummy port which is used as target for the {@link CreateEdgeInputMode#dummyEdge} during the creation.
The port must be part of the {@link CreateEdgeInputMode#dummyEdgeGraph} and therefore be created by its factory method {@link IGraph#addPort} with the {@link CreateEdgeInputMode#dummyTargetNode} as
owner.
This port will only be used if the edge is not connected to a valid target.
The default implementation creates an invisible port at the center of the {@link CreateEdgeInputMode#dummyTargetNode} with the
{@link FreeNodePortLocationModel}.
@returns An {@link IPort} implementation that can be used as a dummy.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23createDummyTargetNodePort() Online Documentation}*/
createDummyTargetNodePort(): IPort;
/**
Called at the end of the edge creation process if both {@link CreateEdgeInputMode#sourcePortCandidate} and {@link CreateEdgeInputMode#targetPortCandidate} have been set.
This method will first {@link IPortCandidate#createPort create the new ports} if necessary and then create the edge via the {@link CreateEdgeInputMode#edgeCreator} callback to the ports. After
the ports are created the corresponding {@link CreateEdgeInputMode#addPortAddedListener PortAdded}
events are triggered. If the edge creation is canceled by the creator callback, newly created ports will be removed
again.
@param graph The graph to create the edge for.
@param sourcePortCandidate The candidate to use for the source end of the edge. Usually the {@link CreateEdgeInputMode#sourcePortCandidate}. In case {@link CreateEdgeInputMode#reversedEdgeCreation reverse edge creation}, though, the value of
{@link CreateEdgeInputMode#targetPortCandidate}.
@param targetPortCandidate The candidate to use for the target end of the edge. Usually the {@link CreateEdgeInputMode#targetPortCandidate}. In case {@link CreateEdgeInputMode#reversedEdgeCreation reverse edge creation}, though, the value of
{@link CreateEdgeInputMode#sourcePortCandidate}.
@returns The edge created or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23createEdge(IGraph%2CIPortCandidate%2CIPortCandidate) Online Documentation}*/
createEdge(graph: IGraph, sourcePortCandidate: IPortCandidate, targetPortCandidate: IPortCandidate | null): IEdge | null;
/**
Creates an {@link IInputModeContext} for use with the port candidates queries for the upcoming edge creation
operation and the actual edge creation
@returns An instance of {@link IInputModeContext} that is configured for this mode.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23createEdgeCreationInputModeContext() Online Documentation}*/
createEdgeCreationInputModeContext(): IInputModeContext;
/**
Synthetically starts the interactive edge creation process using the provided {@link IPortCandidate} as the source
port.
This instance needs to be {@link IInputMode#install installed} and {@link CreateEdgeInputMode#enabled} and it needs to be able to {@link ConcurrencyController#canRequestMutex request the input mutex} in order to proceed. If any
of these conditions are not satisfied, this method will throw an {@link Exception InvalidOperationError}.
@param sourcePortCandidate The source port candidate to use for the edge creation.
@param [initialTargetLocation=null] The initial location that is passed to {@link CreateEdgeInputMode#updateTargetLocation}.
@returns A task that represents the asynchronous edge creation. The result of the task contains the newly created edge or <code>null</code> if
the creation was canceled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23doStartEdgeCreation(IPortCandidate%2CIPoint) Online Documentation}*/
doStartEdgeCreation(sourcePortCandidate: IPortCandidate, initialTargetLocation?: IPoint | null | IPointConvertible): Promise<IEdge>;
/**
Finds the closest {@link PortCandidateValidity#VALID} candidate given a number of
{@link IPortCandidate}s and a location in world coordinates.
@param candidates The candidates to find the closest from.
@param location The location of the mouse in world coordinates.
@returns The candidate to use or <code>null</code> if no candidate satisfies the needs
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getClosestCandidate(IEnumerable%3CIPortCandidate%3E%2CPoint) Online Documentation}*/
getClosestCandidate(candidates: IEnumerable<IPortCandidate>, location: Point): IPortCandidate | null;
/**
Finds the closest candidate given a number of {@link IPortCandidate}s and a location in world coordinates.
@param candidates The candidates to find the closest from.
@param location The location of the pointer in world coordinates.
@returns The candidate to use or <code>null</code> if no candidate satisfies the needs
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getClosestSourceCandidate(IEnumerable%3CIPortCandidate%3E%2CPoint) Online Documentation}*/
getClosestSourceCandidate(candidates: IEnumerable<IPortCandidate>, location: Point): IPortCandidate | null;
/**
Finds the closest target candidate given a number of {@link IPortCandidate}s and a location in world coordinates.
@param candidates The candidates to find the closest from.
@param location The location of the mouse in world coordinates.
@returns The candidate to use or <code>null</code> if no candidate satisfies the needs or {@link CreateEdgeInputMode#enforceBendCreationRecognizer} yields <code>true</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getClosestTargetCandidate(IEnumerable%3CIPortCandidate%3E%2CPoint) Online Documentation}*/
getClosestTargetCandidate(candidates: IEnumerable<IPortCandidate>, location: Point): IPortCandidate | null;
/**
Gets the snap lines lines that are induced by the current dummy edge.
@param context The graph snap context.
@param provider The dummy edge segment snap line provider.
@returns The snap lines induced by the current dummy edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getDummyEdgeSnapLines(GraphSnapContext%2CISnapLineProvider) Online Documentation}*/
getDummyEdgeSnapLines(context: GraphSnapContext, provider: ISnapLineProvider): IEnumerable<OrthogonalSnapLine>;
/**
Calculates direction of first edge segment of the DummyNode.
An edge starting in an border will always be orthogonal to the border's outer line. An edge starting in a corner will
point the way the user dragged the mouse. An edge starting in the inner of the node will be directed the way dragged
unless another edge that is close to the new edge ({@link CreateEdgeInputMode#preferredMinimalEdgeDistance}) already went that way.
@returns A {@link EdgeSegmentDirection} defining the direction of the first edge segment.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getFirstSegmentDirection() Online Documentation}*/
getFirstSegmentDirection(): EdgeSegmentDirection;
/**
Retrieves the port owner at a given position in world coordinates.
If there is a {@link IHitTester} for type {@link INode} in the lookup of this mode's {@link CreateEdgeInputMode#inputModeContext} then this instance will be used for the query.
Otherwise this implementation calls the {@link INodeStyleRenderer#getHitTestable} method of the nodes to find hit
nodes and uses an {@link IComparer} to determine the first hit node.
@param location The position in world coordinates.
@returns The node or <code>null</code> if no suitable node was found.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getPortOwner(Point) Online Documentation}*/
getPortOwner(location: Point): IPortOwner | null;
/**
Retrieves the port owners from the graph in the order of their importance.
This implementation only considers nodes in the graph unless {@link CreateEdgeInputMode#allowEdgeToEdgeConnections} is set to
<code>true</code>, in which case edges are also added to the enumerable.
@returns An enumerable over all {@link IPortOwner}s in this graph
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getPortOwners() Online Documentation}*/
getPortOwners(): IEnumerable<IPortOwner>;
/**
Finds the source {@link IPortOwner port owner} at the specified location.
This always is the source {@link INode node} if {@link CreateEdgeInputMode#allowEdgeToEdgeConnections} is set to <code>false</code>.
This default implementation simply delegates to {@link CreateEdgeInputMode#getPortOwner}.
@param sourceLocation The location of the mouse at the beginning of the gesture.
@returns The port owner to use for the source of the edge creation or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getSource(Point) Online Documentation}*/
getSource(sourceLocation: Point): IPortOwner | null;
/**
Finds the best matching source port {@link IPortCandidate} for the given <code>candidates</code> at the provided <code>location</code>.
The <code>resolveCandidates</code> parameter determines whether dynamic port candidates should be {@link CreateEdgeInputMode#resolveCandidates resolved} with respect to <code>location</code>
or simply be discarded.
Note that "source" refers to the port candidate where the edge creation started. In case of {@link CreateEdgeInputMode#reversedEdgeCreation reversed edge creation} this
will actually become the target port of the edge.
@param candidates The candidates.
@param location The location where the gesture was initiated.
@param resolveCandidates Determines whether {@link PortCandidateValidity#DYNAMIC} ports should be resolved with respect to the <code>location</code> or
not.
@returns The candidate to use or <code>null</code>.
@see CreateEdgeInputMode#resolveSourcePortCandidates
@see CreateEdgeInputMode#portCandidateResolutionRecognizer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getSourcePortCandidate(IEnumerable%3CIPortCandidate%3E%2CPoint%2Cboolean) Online Documentation}*/
getSourcePortCandidate(candidates: IEnumerable<IPortCandidate>, location: Point, resolveCandidates: boolean): IPortCandidate | null;
/**
Finds the source {@link IPortCandidateProvider port candidate provider} at the specified location.
This default implementation simply delegates to {@link CreateEdgeInputMode#getSource} to get a port owner and
returns the {@link IPortCandidateProvider} returned by that port owner's {@link ILookup#lookup}.
@param sourceLocation The location of the mouse at the beginning of the gesture.
@returns The port candidate provider to use for the source of the edge creation or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getSourcePortCandidateProvider(Point) Online Documentation}*/
getSourcePortCandidateProvider(sourceLocation: Point): IPortCandidateProvider | null;
/**
Gets the source port candidates for the given <code>location</code>.
This implementation first tries to get {@link CreateEdgeInputMode#getSourcePortCandidateProvider port candidate provider} for the given <code>location</code>. If a provider is found its method
{@link IPortCandidateProvider#getAllSourcePortCandidates} is called to get the candidates.
If no owner or provider is found, <code>null</code> is returned.
Note that "source" refers to the port candidate where the edge creation started. In case of {@link CreateEdgeInputMode#reversedEdgeCreation reversed edge creation} this
will actually become the target port of the edge.
@param location The location to get the port candidates for.
@returns The port candidates for the given location or <code>null</code> if there are none.
@see CreateEdgeInputMode#getSource
@see CreateEdgeInputMode#getSourcePortCandidateProvider
@see IPortCandidateProvider#getAllSourcePortCandidates
@see IPortCandidateProvider#getAllTargetPortCandidates
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getSourcePortCandidates(Point) Online Documentation}*/
getSourcePortCandidates(location: Point): IEnumerable<IPortCandidate> | null;
/**
Finds the target {@link IPortOwner port owner} at the specified location.
This always is a target {@link INode node} if {@link CreateEdgeInputMode#allowEdgeToEdgeConnections} is set to <code>false</code>.
This default implementation simply delegates to {@link CreateEdgeInputMode#getPortOwner}.
Note that "target" refers to the end of the edge where creation finishes. In case of {@link CreateEdgeInputMode#reversedEdgeCreation reversed edge creation} this will
actually become the source end of the created edge.
@param targetLocation The location of the pointer at the beginning of the gesture.
@returns The port owner or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getTarget(Point) Online Documentation}*/
getTarget(targetLocation: Point): IPortOwner | null;
/**
Retrieves the {@link IPortCandidateProvider} instance that provides the possible candidates for the target of the
edge given the current source candidate.
This implementation queries all nodes in the graph for {@link CreateEdgeInputMode#lookupTargetPortCandidateProvider their provider} and returns a composite unless {@link CreateEdgeInputMode#useHitItemsCandidatesOnly} is set to <code>true</code> in
which case the {@link CreateEdgeInputMode#lookupTargetPortCandidateProvider} is queried using the current {@link CreateEdgeInputMode#getTarget target}.
Note that "target" refers to the port candidate where the edge creation finishes. In case of {@link CreateEdgeInputMode#reversedEdgeCreation reversed edge creation} this
will actually become the {@link IEdge#sourcePort source port} of the edge.
@param sourcePortCandidate The candidate at the other end of the edge that has been chosen for the edge creation. In case of {@link CreateEdgeInputMode#reversedEdgeCreation reversed edge creation}
this is actually the candidate for the {@link IEdge#targetPort target end}of the edge.
@param location The location of the mouse.
@returns The provider to get the port candidates from.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getTargetPortCandidateProvider(IPortCandidate%2CPoint) Online Documentation}*/
getTargetPortCandidateProvider(sourcePortCandidate: IPortCandidate, location: Point): IPortCandidateProvider | null;
/**
Retrieves the target port candidates for a given location in world coordinates.
The <code>resolveCandidates</code> parameter determines whether {@link PortCandidateValidity#DYNAMIC dynamic} port candidates should be {@link CreateEdgeInputMode#resolveCandidates resolved} with respect to <code>location</code>
or simply be discarded.
This implementation delegates to {@link CreateEdgeInputMode#getTargetPortCandidateProvider} or returns an empty
enumerable if no provider has been found.
Note that "target" refers to the port candidate where the edge creation finishes. In case of {@link CreateEdgeInputMode#reversedEdgeCreation reversed edge creation} this
will actually become the source port of the edge.
@param location The location of the mouse in world coordinates.
@param resolveCandidates Determines whether {@link PortCandidateValidity#DYNAMIC} ports should be resolved with respect to the <code>location</code> or
not.
@returns A possibly empty enumeration over all target port candidates.
@see CreateEdgeInputMode#sourcePortCandidate
@see CreateEdgeInputMode#resolveTargetPortCandidates
@see CreateEdgeInputMode#portCandidateResolutionRecognizer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23getTargetPortCandidates(Point%2Cboolean) Online Documentation}*/
getTargetPortCandidates(location: Point, resolveCandidates: boolean): IEnumerable<IPortCandidate>;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link CreateEdgeInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Retrieves the port candidate provider instance given a source port owner.
This implementations uses the <code>sourceOwner</code>'s {@link ILookup#lookup} to retrieve an {@link IPortCandidateProvider}
implementation.
@param sourceOwner The {@link IPortOwner} to find the candidates for.
@returns A provider instance or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23lookupSourcePortCandidateProvider(IPortOwner) Online Documentation}*/
lookupSourcePortCandidateProvider(sourceOwner: IPortOwner): IPortCandidateProvider | null;
/**
Retrieves the port candidate provider instance given a source port candidate and a possible target item.
This implementations uses the <code>targetOwner</code>'s {@link ILookup#lookup} to retrieve an {@link IPortCandidateProvider}
implementation.
@param sourcePortCandidate The current source port candidate.
@param targetOwner The {@link IPortOwner} to find the candidates for.
@param location The location of the mouse.
@returns A provider instance or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23lookupTargetPortCandidateProvider(IPortCandidate%2CIPortOwner%2CPoint) Online Documentation}*/
lookupTargetPortCandidateProvider(sourcePortCandidate: IPortCandidate | null, targetOwner: IPortOwner, location: Point): IPortCandidateProvider | null;
/**
Measures the distance between a port <code>candidate</code> and a given <code>location</code>.
If a {@link CanvasComponent#projection} is set, this method returns the distance in view coordinates. Otherwise, the distance in world coordinates is
returned.
@param candidate The port candidate.
@param location The location in world coordinates.
@returns The distance between the <code>candidate</code> and the <code>location</code> (in world coordinates if no {@link CanvasComponent#projection} is set; in view coordinates if it has.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23measureDistance(IPortCandidate%2CPoint) Online Documentation}*/
measureDistance(candidate: IPortCandidate, location: Point): number;
/**
Called after {@link CreateEdgeInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController} property of the installed {@link ConcurrencyController} has
been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Called after the edge has been finalized.
This method will trigger the {@link CreateEdgeInputMode#addEdgeCreatedListener EdgeCreated}
event.
@param evt The event argument holding the edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onEdgeCreated(EdgeEventArgs) Online Documentation}*/
onEdgeCreated(evt: EdgeEventArgs): void;
/**
Raises the {@link CreateEdgeInputMode#addEdgeCreationStartedListener EdgeCreationStarted} event when
the gesture for creating an edge has been initialized.
@param evt The {@link EdgeEventArgs} instance containing the dummy edge that will be used during edge creation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onEdgeCreationStarted(EdgeEventArgs) Online Documentation}*/
onEdgeCreationStarted(evt: EdgeEventArgs): void;
/**
Triggers the {@link CreateEdgeInputMode#addGestureCanceledListener GestureCanceled} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onGestureCanceled(InputModeEventArgs) Online Documentation}*/
onGestureCanceled(evt: InputModeEventArgs): void;
/**
Triggers the {@link CreateEdgeInputMode#addGestureCancelingListener GestureCanceling} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onGestureCanceling(InputModeEventArgs) Online Documentation}*/
onGestureCanceling(evt: InputModeEventArgs): void;
/**
Called once the drag has been finalized.
This method triggers the {@link CreateEdgeInputMode#addGestureFinishedListener GestureFinished}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onGestureFinished(InputModeEventArgs) Online Documentation}*/
onGestureFinished(evt: InputModeEventArgs): void;
/**
Called before the drag will be finalized.
This method triggers the {@link CreateEdgeInputMode#addGestureFinishingListener GestureFinishing}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onGestureFinishing(InputModeEventArgs) Online Documentation}*/
onGestureFinishing(evt: InputModeEventArgs): void;
/**
Triggers the {@link CreateEdgeInputMode#addGestureStartedListener GestureStarted} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onGestureStarted(InputModeEventArgs) Online Documentation}*/
onGestureStarted(evt: InputModeEventArgs): void;
/**
Triggers the {@link CreateEdgeInputMode#addGestureStartingListener GestureStarting} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onGestureStarting(InputModeEventArgs) Online Documentation}*/
onGestureStarting(evt: InputModeEventArgs): void;
/**
Called at the end of each drag.
This method triggers the {@link CreateEdgeInputMode#addMovedListener Moved} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onMoved(InputModeEventArgs) Online Documentation}*/
onMoved(evt: InputModeEventArgs): void;
/**
Called at the start of each drag.
This method triggers the {@link CreateEdgeInputMode#addMovingListener Moving} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onMoving(InputModeEventArgs) Online Documentation}*/
onMoving(evt: InputModeEventArgs): void;
/**
Raises the {@link CreateEdgeInputMode#addPortAddedListener PortAdded} event if the mode has
{@link IGraph#addPort added a port} for the source or target node to complete the edge creation.
@param evt The {@link ItemEventArgs} instance containing the port that has been added.
@see CreateEdgeInputMode#addPortAddedListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onPortAdded(ItemEventArgs%3CIPort%3E) Online Documentation}*/
onPortAdded(evt: ItemEventArgs<IPort>): void;
/**
Called when the {@link CreateEdgeInputMode#reversedEdgeCreation} property changes and the edge creation is {@link CreateEdgeInputMode#isCreationInProgress in progress}.
This implementation swaps source and target port of the dummy edge and adjusts the bends by reversing their locations.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onReversedEdgeCreationChanged() Online Documentation}*/
onReversedEdgeCreationChanged(): void;
/**
Triggers the {@link CreateEdgeInputMode#addSourcePortCandidateChangedListener SourcePortCandidateChanged}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onSourcePortCandidateChanged(ItemEventArgs%3CIPortCandidate%3E) Online Documentation}*/
onSourcePortCandidateChanged(evt: ItemEventArgs<IPortCandidate>): void;
/**
Called after {@link CreateEdgeInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Triggers the {@link CreateEdgeInputMode#addTargetPortCandidateChangedListener TargetPortCandidateChanged}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23onTargetPortCandidateChanged(ItemEventArgs%3CIPortCandidate%3E) Online Documentation}*/
onTargetPortCandidateChanged(evt: ItemEventArgs<IPortCandidate>): void;
/**
Sets the position of the tip of the edge during creation.
Has to set the {@link CreateEdgeInputMode#dragPoint} and the center of the {@link CreateEdgeInputMode#dummyTargetNode} to <code>newLocation</code>.
@param newLocation The location of the tip of the edge.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23relocateDragPoint(Point) Online Documentation}*/
relocateDragPoint(newLocation: Point): void;
/**
Returns an enumerable over the given candidates where no instance has {@link PortCandidateValidity#DYNAMIC} {@link IPortCandidate#validity}.
If the location has a value, method {@link IPortCandidate#getPortCandidateAt} is used to resolve the dynamic
candidate to a real candidate, otherwise that candidate is discarded.
@param candidates The candidates to possibly resolve.
@param location The location to resolve dynamic candidates against or <code>null</code> if they should be discarded instead.
@returns An enumerable of non-{@link PortCandidateValidity#DYNAMIC} port candidates.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23resolveCandidates(IEnumerable%3CIPortCandidate%3E%2CPoint) Online Documentation}*/
resolveCandidates(candidates: IEnumerable<IPortCandidate>, location: Point | null): IEnumerable<IPortCandidate>;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Updates the {@link CreateEdgeInputMode#dummyEdge} to reflect the current source port candidate.
This implementation creates a self loop of the dummy edge if the <code>sourcePortCandidate</code> is the same as the owner of the {@link CreateEdgeInputMode#targetPortCandidate}.
In case of {@link CreateEdgeInputMode#reversedEdgeCreation edge creation in reversed direction} the updated port is the {@link IEdge#targetPort} of the created edge.
@param sourcePortCandidate The new owner target port candidate.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23updateDummyEdgeSourcePort(IPortCandidate) Online Documentation}*/
updateDummyEdgeSourcePort(sourcePortCandidate: IPortCandidate | null): void;
/**
Updates the {@link CreateEdgeInputMode#dummyEdge} to reflect the current target port candidate.
This implementation creates a self loop of the dummy edge if the <code>targetPortCandidate</code> is the same as the owner of the {@link CreateEdgeInputMode#sourcePortCandidate}.
In case of {@link CreateEdgeInputMode#reversedEdgeCreation edge creation in reversed direction} the updated port is the {@link IEdge#sourcePort} of the created edge.
@param targetPortCandidate The new owner target port candidate.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23updateDummyEdgeTargetPort(IPortCandidate) Online Documentation}*/
updateDummyEdgeTargetPort(targetPortCandidate: IPortCandidate | null): void;
/**
Draws the highlight for the owner of the current {@link CreateEdgeInputMode#targetPortCandidate}.
This implementation retrieves the {@link HighlightIndicatorManager} from the {@link CreateEdgeInputMode#inputModeContext}'s {@link ILookup}.
Overriders have to take care of removal of the highlight of the <code>oldCandidate</code>'s owner, too.
@param oldCandidate The port candidate whose owner is currently highlighted.
@param newCandidate The port candidate whose owner should be highlighted.
@see CreateEdgeInputMode#showTargetHighlight
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23updateTargetHighlight(IPortCandidate%2CIPortCandidate) Online Documentation}*/
updateTargetHighlight(oldCandidate: IPortCandidate | null, newCandidate: IPortCandidate | null): void;
/**
Updates the drag point.
This method is called when the current target location for the creation gesture is changed.
@param location The new coordinates.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23updateTargetLocation(Point) Online Documentation}*/
updateTargetLocation(location: Point): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link CreateEdgeInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
* Gets or sets the enabled state of this input mode.
* Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
* {@link CreateEdgeInputMode#controller} so a disabled instance should never try to acquire the input mutex.
*
* <br><b>Disabling the CreateEdgeInputMode on its parent input mode</b>
* <pre>
* mode.createEdgeInputMode.enabled = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23enabled Online Documentation}
*/
enabled: boolean;
/**
Gets the installed {@link CreateEdgeInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Gets or sets the cursor that indicates a valid place to begin an edge creation.
The default cursor is {@link Cursor#POINTER}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23validBeginCursor Online Documentation}*/
validBeginCursor: Cursor | null;
/**
Gets or sets the cursor that indicates a valid place to create a bend.
The default cursor is {@link Cursor#DEFAULT}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23validBendCursor Online Documentation}*/
validBendCursor: Cursor | null;
/**
Gets or sets the cursor that is used while a new edge is created when there is no {@link CreateEdgeInputMode#validBendHitTestable valid location} to create a bend.
The default cursor is {@link Cursor#DEFAULT}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23dragCursor Online Documentation}*/
dragCursor: Cursor | null;
/**
Gets or sets the cursor that indicates a valid place to finish creation.
The default cursor is {@link Cursor#POINTER}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23validEndCursor Online Documentation}*/
validEndCursor: Cursor | null;
/**
Gets or sets event recognizer that temporarily disables snapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23disableSnappingRecognizer Online Documentation}*/
disableSnappingRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets event recognizer that reenables temporarily disabled snapping.
@see CreateEdgeInputMode#disableSnappingRecognizer
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23enableSnappingRecognizer Online Documentation}*/
enableSnappingRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that toggles the orientation of the first orthogonally created segment.
@see CreateEdgeInputMode#orthogonalEdgeCreation
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23toggleSegmentOrientationRecognizer Online Documentation}*/
toggleSegmentOrientationRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
* Gets or sets the event recognizer that recognizes gesture which toggles the {@link CreateEdgeInputMode#reversedEdgeCreation direction} of the created edge.
* The default instance never triggers.
*
* <pre>
* createEdgeInputMode.toggleDirectionRecognizer = KeyEventRecognizers.SHIFT_DOWN
* </pre>
*
* @see CreateEdgeInputMode#edgeDirectionPolicy
* @see CreateEdgeInputMode#onReversedEdgeCreationChanged
* @see CreateEdgeInputMode#reversedEdgeCreation
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23toggleDirectionRecognizer Online Documentation}
*/
toggleDirectionRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that recognizes the finishing gesture that is used to end the edge creation on the
target node.
The default is a combination of {@link MouseEventRecognizers#LEFT_DOWN} and
{@link MouseEventRecognizers#LEFT_UP}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23finishRecognizer Online Documentation}*/
finishRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that recognizes the starting gesture that is used to prepare the edge creation on the
source node.
The default is {@link MouseEventRecognizers#LEFT_DOWN}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23prepareRecognizer Online Documentation}*/
prepareRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that recognizes the initial dragging gesture that is used to initiate the creation
after the preparation.
The default is {@link MouseEventRecognizers#DRAG}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23draggedRecognizer Online Documentation}*/
draggedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that recognizes the movement of the end point of the edge.
The default is {@link MouseEventRecognizers#MOVE_OR_DRAG}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23endPointMovedRecognizer Online Documentation}*/
endPointMovedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to create a bend.
The default is {@link MouseEventRecognizers#LEFT_UP}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23createBendRecognizer Online Documentation}*/
createBendRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to cancel edge creation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23cancelRecognizer Online Documentation}*/
cancelRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to remove bends.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23removeBendRecognizer Online Documentation}*/
removeBendRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether dynamic port candidates should be resolved.
By default this is {@link KeyEventRecognizers#SHIFT_IS_DOWN}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23portCandidateResolutionRecognizer Online Documentation}*/
portCandidateResolutionRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that recognizes the finishing gesture that is used to end the edge creation on the
target node by a touch gesture.
The default is {@link TouchEventRecognizers#TOUCH_UP_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23finishRecognizerTouch Online Documentation}*/
finishRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that recognizes the starting gesture that is used to prepare the edge creation on the
source node via touch.
The default is {@link TouchEventRecognizers#TOUCH_LONG_PRESS_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23prepareRecognizerTouch Online Documentation}*/
prepareRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that recognizes the initial dragging gesture that is used to initiate the creation
after the preparation via touch.
The default is {@link TouchEventRecognizers#TOUCH_MOVE_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23draggedRecognizerTouch Online Documentation}*/
draggedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to create a bend via touch.
The default is {@link TouchEventRecognizers#TOUCH_UP_PRIMARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23createBendRecognizerTouch Online Documentation}*/
createBendRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to cancel edge creation via touch.
The default is {@link TouchEventRecognizers#TOUCH_MULTI_TAP_SECONDARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23cancelRecognizerTouch Online Documentation}*/
cancelRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer determines whether to remove bends via touch.
The default is {@link TouchEventRecognizers#TOUCH_LONG_PRESS_SECONDARY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23removeBendRecognizerTouch Online Documentation}*/
removeBendRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets whether and where to display possible port candidates during the creation of the edge.
Setting this property to {@link ShowPortCandidates#SOURCE} or {@link ShowPortCandidates#ALL} will result in
displaying source port candidates for each node the mouse is located over. For a {@link CreateEdgeInputMode} as
child of a default {@link GraphEditorInputMode} selected nodes will rather be moved than serve as source of an edge
creation. To disable showing port candidates for selected nodes developers might set appropriate {@link CreateEdgeInputMode#showSourcePortCandidatesHitTestable} and {@link CreateEdgeInputMode#beginHitTestable}s. An example {@link IHitTestable}
is shown there.
The default is {@link ShowPortCandidates#TARGET}.
@default ShowPortCandidates.TARGET
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23showPortCandidates Online Documentation}*/
showPortCandidates: ShowPortCandidates;
/**
* Gets or sets whether or not edges are allowed to connect to other edges.
* The default is <code>false</code>.
*
* <pre>
* createEdgeInputMode.allowEdgeToEdgeConnections = true
* </pre>
*
* @default false
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23allowEdgeToEdgeConnections Online Documentation}
*/
allowEdgeToEdgeConnections: boolean;
/**
Gets or sets the distance in the view coordinate system that determines how far the mouse should be snapped to a port
candidate if {@link CreateEdgeInputMode#snapToTargetCandidate} is enabled.
The default value is <code>20</code>.
@default 20.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23snapDistance Online Documentation}*/
snapDistance: number;
/**
Gets or sets the {@link CreateEdgeInputMode#snapContext} which manages snapping model items to certain coordinates (for instance, other items).
If set to <code>null</code> (the default) this input mode tries to obtain the {@link CreateEdgeInputMode#snapContext} from the {@link IInputModeContext}. To explicitly
disable snapping, a {@link CreateEdgeInputMode#snapContext} implementation that does nothing has to be set to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23snapContext Online Documentation}*/
snapContext: SnapContext | null;
/**
Gets or sets the {@link ICanvasObjectDescriptor} that is used for visualizing the
{@link IPortCandidate}s in the canvas during the edit.
This descriptor is queried for the {@link IVisualCreator} to obtain the visualizations for the candidates that are
not currently closest. By default an instance of {@link DefaultPortCandidateDescriptor} is used.
@see CreateEdgeInputMode#closestCandidateDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23candidateDescriptor Online Documentation}*/
candidateDescriptor: ICanvasObjectDescriptor;
/**
Gets or sets the {@link ICanvasObjectDescriptor} that is used for visualizing the closest {@link IPortCandidate}
in the canvas during the edit.
This descriptor is queried for the {@link IVisualCreator} to obtain the visualization for the currently closest
candidate. By default an instance of {@link DefaultPortCandidateDescriptor} is used.
@see CreateEdgeInputMode#candidateDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23closestCandidateDescriptor Online Documentation}*/
closestCandidateDescriptor: ICanvasObjectDescriptor;
/**
Gets or sets the distance in the view coordinate system that determines how far the mouse should be snapped to a
horizontal or vertical line.
The default value is <code>5</code>.
@default 5.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23orthogonalSnapDistance Online Documentation}*/
orthogonalSnapDistance: number;
/**
Gets or sets a property that determines whether the mouse should be forced to snap to the nearest valid port candidate
if the mouse hovers over a target node.
If the nearest port candidate is farther away than {@link CreateEdgeInputMode#snapDistance} and the mouse hovers over a valid {@link CreateEdgeInputMode#getTarget target}, this property will cause
the mouse to snap to the nearest valid candidate. The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23forceSnapToCandidate Online Documentation}*/
forceSnapToCandidate: boolean;
/**
Gets or sets a property that determines whether this mode should use only the {@link CreateEdgeInputMode#getTargetPortCandidates target port candidates} of the node the
mouse currently hovers over.
The default is <code>true</code>.
@see CreateEdgeInputMode#showPortCandidates
@see CreateEdgeInputMode#getTarget
@see CreateEdgeInputMode#getTargetPortCandidates
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23useHitItemsCandidatesOnly Online Documentation}*/
useHitItemsCandidatesOnly: boolean;
/**
Gets or sets the the policy that controls whether edges will be create orthogonally.
By default orthogonal edge creation is enabled if orthogonal edge editing is enabled on the {@link GraphEditorInputMode parent input mode}.
@see GraphEditorInputMode#orthogonalEdgeEditingContext
@see OrthogonalEdgeEditingContext#enabled
@default OrthogonalEdgeEditingPolicy.AUTO
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23orthogonalEdgeCreation Online Documentation}*/
orthogonalEdgeCreation: OrthogonalEdgeEditingPolicy;
/**
Gets or sets the preferred minimal distance of orthogonal edges.
This information is needed when deciding which way to route the first segment of an orthogonal edge.
@default 10.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23preferredMinimalEdgeDistance Online Documentation}*/
preferredMinimalEdgeDistance: number;
/**
Gets or sets a value indicating whether {@link PortCandidateValidity#DYNAMIC} ports that are part of the {@link CreateEdgeInputMode#getSourcePortCandidates source port candidates}
should be resolved for the current mouse location.
By default this feature is enabled and the {@link CreateEdgeInputMode#portCandidateResolutionRecognizer} is configured so that the resolution mechanism is used if the user holds the
shift modifier during the gesture.
Note that "source" refers to the port candidate where the edge creation started. In case of {@link CreateEdgeInputMode#reversedEdgeCreation reversed edge creation} this
will actually become the target port of the edge.
@see CreateEdgeInputMode#portCandidateResolutionRecognizer
@see CreateEdgeInputMode#resolveCandidates
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23resolveSourcePortCandidates Online Documentation}*/
resolveSourcePortCandidates: boolean;
/**
Gets or sets a value indicating whether {@link PortCandidateValidity#DYNAMIC} ports that are part of the {@link CreateEdgeInputMode#getTargetPortCandidates target port candidates}
should be resolved for the current mouse location.
By default this feature is enabled and the {@link CreateEdgeInputMode#portCandidateResolutionRecognizer} is configured so that the resolution mechanism is used if the user holds the
shift modifier during the gesture.
@see CreateEdgeInputMode#portCandidateResolutionRecognizer
@see CreateEdgeInputMode#resolveCandidates
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23resolveTargetPortCandidates Online Documentation}*/
resolveTargetPortCandidates: boolean;
/**
Gets or sets a value that determines how the direction of a newly created edge is determined.
This property is {@link EdgeDirectionPolicy#START_AT_SOURCE} by default which means that edges are created from the
source to the target. Setting the value to {@link EdgeDirectionPolicy#DETERMINE_FROM_PORT_CANDIDATES} will query
both {@link IPortCandidateProvider#getAllSourcePortCandidates source} and {@link IPortCandidateProvider#getAllTargetPortCandidates target} port candidates during {@link CreateEdgeInputMode#getSourcePortCandidates}. Depending on
which candidate is chosen in {@link CreateEdgeInputMode#getSourcePortCandidates} the edge creation will either be {@link CreateEdgeInputMode#reversedEdgeCreation reversed}
or not.
Setting the property to another value during edge creation will not affect the current edge. Set {@link CreateEdgeInputMode#reversedEdgeCreation} to control the
direction of a currently active edge creation.
@default EdgeDirectionPolicy.START_AT_SOURCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23edgeDirectionPolicy Online Documentation}*/
edgeDirectionPolicy: EdgeDirectionPolicy;
/**
* Gets or sets a value indicating whether the current edge creation should be created from {@link IEdge#targetPort its target port} to
* {@link IEdge#sourcePort its source port}.
* This property is automatically set at the start of the edge creation according to the value of {@link CreateEdgeInputMode#edgeDirectionPolicy}. It can be set to a
* different value during edge creation to change the direction of the created edge immediately. Note that the currently
* assigned {@link CreateEdgeInputMode#sourcePortCandidate} will stay the same, then, and is not queried again from the
* {@link CreateEdgeInputMode#getSourcePortCandidates}
*
* Setting the {@link CreateEdgeInputMode#reversedEdgeCreation} in a listener to the {@link CreateEdgeInputMode#addSourcePortCandidateChangedListener SourcePortCandidateChanged}
* event allows for fully customizing the edge creation direction. The example shows how to set the direction according to
* the candidate's {@link ITagOwner#tag}.
*
* <pre>
* createEdgeInputMode.addSourcePortCandidateChangedListener((sender, args) => {
*   createEdgeInputMode.reversedEdgeCreation = args.item.owner.tag === 'createFromTarget'
* })
* </pre>
*
* Setting this value as shown above will override the {@link CreateEdgeInputMode#edgeDirectionPolicy}.
*
* @see CreateEdgeInputMode#edgeDirectionPolicy
* @default false
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23reversedEdgeCreation Online Documentation}
*/
reversedEdgeCreation: boolean;
/**
Gets or sets the width of a node's border which is taken into account when deciding which way to route the first segment
of an orthogonal edge.
An edge starting inside a given margin is always pointing away from it.
@default 0.1
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23nodeBorderWidthRatio Online Documentation}*/
nodeBorderWidthRatio: number;
/**
* Gets or sets the ports based edge creation callback.
* The default creator will delegate to {@link IGraph#createEdge} and obtain the tag from the {@link CreateEdgeInputMode#dummyEdge}. It will then copy over
* all bends, labels, and ports from the {@link CreateEdgeInputMode#dummyEdge}.
*
* <br><b>A custom edge creator</b>
* <pre>
* // set a custom edge creator which creates a label on the new edge
* createEdgeInputMode.edgeCreator = (
*   context,
*   graph,
*   sourceCandidate,
*   targetCandidate,
*   dummyEdge
* ) => {
*   // get the source and target ports from the candidates
*   const sourcePort = sourceCandidate.port !== null ? sourceCandidate.createPort(context) : null
*   const targetPort = targetCandidate.port !== null ? targetCandidate.createPort(context) : null
*   // create the edge between the source and target port
*   const edge = graph.createEdge(sourcePort, targetPort, dummyEdge.style)
*   // create a label
*   graph.addLabel(edge, 'A new edge')
*   // return the created edge
*   return edge
* }
* </pre>
*
* @throws {Exception} <b>(ArgumentError)</b> If the argument is <code>null</code>
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23edgeCreator Online Documentation}
*/
edgeCreator: (context: IInputModeContext, graph: IGraph, sourcePortCandidate: IPortCandidate, targetPortCandidate: IPortCandidate | null, templateEdge: IEdge) => IEdge | null;
/**
* Gets or sets whether or not to allow the creation of self loops.
* The default is <code>true</code>, which enables that an edge is connected to a {@link IEdge#sourcePort} and {@link IEdge#targetPort} that have the same {@link IPort#owner}.
*
* <pre>
* createEdgeInputMode.allowSelfloops = true
* </pre>
*
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23allowSelfloops Online Documentation}
*/
allowSelfloops: boolean;
/**
Gets or sets whether an edge creation gesture will be canceled when the gesture is ended on an invalid target and no
bend may be created there.
If bend creation is allowed at the target location (i.e. {@link CreateEdgeInputMode#validBendHitTestable} returns true), the gesture will always continue by creating a
bend. Otherwise, if this property is <code>true</code>, the gesture will be canceled automatically.
The default is <code>false</code>, meaning that the gesture must always be canceled explicitly.
@see CreateEdgeInputMode#allowCreateBend
@see CreateEdgeInputMode#validBendHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23cancelGestureOnInvalidTarget Online Documentation}*/
cancelGestureOnInvalidTarget: boolean;
/**
* Gets or sets whether or not to allow the creation of bends.
* The default is <code>true</code>.
*
* To disable bend creation during edge creation and on existing edges one has to disable the {@link GraphEditorInputMode#createBendInputMode}, too.
*
* <br><b>Disabling the bend creation during edge creation and on existing edges</b>
* <pre>
* // mode is an instance of GraphEditorInputMode
* mode.createBendInputMode.enabled = false
* mode.createEdgeInputMode.allowCreateBend = false
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23allowCreateBend Online Documentation}
*/
allowCreateBend: boolean;
/**
Gets or sets the candidate for the {@link IEdge#sourcePort} of the edge to be created.
This property will be populated automatically when edge creation has started.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23sourcePortCandidate Online Documentation}*/
sourcePortCandidate: IPortCandidate | null;
/**
Gets or sets the current candidate for the {@link IEdge#targetPort} of the edge to be created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23targetPortCandidate Online Documentation}*/
targetPortCandidate: IPortCandidate | null;
/**
Gets or sets whether the owner of the current {@link CreateEdgeInputMode#targetPortCandidate} should be highlighted.
The default is <code>true</code>.
@see CreateEdgeInputMode#updateTargetHighlight
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23showTargetHighlight Online Documentation}*/
showTargetHighlight: boolean;
/**
Adds the given listener for the <code>SourcePortCandidateChanged</code> event that occurs when the value of the {@link CreateEdgeInputMode#sourcePortCandidate} property changes.
@param listener The listener to add.
@see CreateEdgeInputMode#removeSourcePortCandidateChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23SourcePortCandidateChanged Online Documentation}*/
addSourcePortCandidateChangedListener(listener: (sender: any, evt: ItemEventArgs<IPortCandidate>) => void): void;
/**
Removes the given listener for the <code>SourcePortCandidateChanged</code> event that occurs when the value of the {@link CreateEdgeInputMode#sourcePortCandidate} property
changes.
@param listener The listener to remove.
@see CreateEdgeInputMode#addSourcePortCandidateChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23SourcePortCandidateChanged Online Documentation}*/
removeSourcePortCandidateChangedListener(listener: (sender: any, evt: ItemEventArgs<IPortCandidate>) => void): void;
/**
Adds the given listener for the <code>TargetPortCandidateChanged</code> event that occurs when the value of the {@link CreateEdgeInputMode#targetPortCandidate} property changes.
@param listener The listener to add.
@see CreateEdgeInputMode#removeTargetPortCandidateChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23TargetPortCandidateChanged Online Documentation}*/
addTargetPortCandidateChangedListener(listener: (sender: any, evt: ItemEventArgs<IPortCandidate>) => void): void;
/**
Removes the given listener for the <code>TargetPortCandidateChanged</code> event that occurs when the value of the {@link CreateEdgeInputMode#targetPortCandidate} property
changes.
@param listener The listener to remove.
@see CreateEdgeInputMode#addTargetPortCandidateChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23TargetPortCandidateChanged Online Documentation}*/
removeTargetPortCandidateChangedListener(listener: (sender: any, evt: ItemEventArgs<IPortCandidate>) => void): void;
/**
Gets or sets whether after calls to {@link CreateEdgeInputMode#createEdge} the current undo edit should be canceled
if the method returns <code>null</code>.
This should be set to <code>false</code> if you want to perform some undoable actions in these methods and/or in the {@link CreateEdgeInputMode#edgeCreator} callback, but
need to return <code>null</code> from any of these methods. Default value is <code>true</code>, meaning that returning <code>null</code> results in canceling
the composite undo entry.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23cancelEditImplicitly Online Documentation}*/
cancelEditImplicitly: boolean;
/**
* Adds the given listener for the <code>EdgeCreated</code> event that occurs after an edge has been created by this mode.
* <pre>
* createEdgeInputMode.addEdgeCreatedListener((sender, args) =>
*   createEdgeInputMode.graph.applyLayout(new EdgeRouter())
* )
* </pre>
*
* @param listener The listener to add.
* @see CreateEdgeInputMode#onEdgeCreated
* @see CreateEdgeInputMode#removeEdgeCreatedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23EdgeCreated Online Documentation}
*/
addEdgeCreatedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
* Removes the given listener for the <code>EdgeCreated</code> event that occurs after an edge has been created by this mode.
* <pre>
* createEdgeInputMode.addEdgeCreatedListener((sender, args) =>
*   createEdgeInputMode.graph.applyLayout(new EdgeRouter())
* )
* </pre>
*
* @param listener The listener to remove.
* @see CreateEdgeInputMode#onEdgeCreated
* @see CreateEdgeInputMode#addEdgeCreatedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23EdgeCreated Online Documentation}
*/
removeEdgeCreatedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
Adds the given listener for the <code>PortAdded</code> event that occurs when this instance {@link IGraph#addPort adds a port} to the source or target node
during completion of the edge creation gesture.
@param listener The listener to add.
@see CreateEdgeInputMode#removePortAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23PortAdded Online Documentation}*/
addPortAddedListener(listener: (sender: any, evt: ItemEventArgs<IPort>) => void): void;
/**
Removes the given listener for the <code>PortAdded</code> event that occurs when this instance {@link IGraph#addPort adds a port} to the source or target
node during completion of the edge creation gesture.
@param listener The listener to remove.
@see CreateEdgeInputMode#addPortAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23PortAdded Online Documentation}*/
removePortAddedListener(listener: (sender: any, evt: ItemEventArgs<IPort>) => void): void;
/**
Gets the dummy edge instance that will be used to render a preview of the edge to be created.
This instance will be used for rendering the edge during the gesture. If this property is unset initially, factory
method {@link CreateEdgeInputMode#createDummyEdge} will be called.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23dummyEdge Online Documentation}*/
readonly dummyEdge: IEdge;
/**
Gets the dummy graph instance that will hold the items that will render a preview of the newly created edge.
The dummy edge graph can be used to change the appearance and other properties of the dummy edge, either when starting a
gesture or during it in response to certain events. Possible actions are:
<ul>
<li>Changing the style of the dummy edge.
<li>Adding bends to the dummy edge.
<li>Adding labels to the dummy edge.
<li>Changing style and other properties of the added labels.
</ul>
All changes made to the dummy edge during the gesture will be persisted to the created edge in the original graph. This
behavior can be changed by setting a custom {@link CreateEdgeInputMode#edgeCreator}.
The dummy edge graph only allows operations on the dummy edge.
Changing style of the dummy edge's source or target port has no effect. The style for these is controlled by the
{@link IPortCandidate}s and hence by an {@link IPortCandidateProvider}.
If the property is not set initially, method {@link CreateEdgeInputMode#createDummyEdgeGraph} will be used as the
factory method.
@see CreateEdgeInputMode#dummyEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23dummyEdgeGraph Online Documentation}*/
readonly dummyEdgeGraph: IGraph;
/**
Gets or sets the appearance of newly created edges.
This property directly delegates to the {@link IGraph#edgeDefaults} of the {@link CreateEdgeInputMode#dummyEdgeGraph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23edgeDefaults Online Documentation}*/
edgeDefaults: IEdgeDefaults;
/**
The {@link INode} which is used as target node for the temporarily created {@link CreateEdgeInputMode#dummyEdge} during edge creation.
By default, this node is invisible.
The node is created upon first access by method {@link CreateEdgeInputMode#createDummyTargetNode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23dummyTargetNode Online Documentation}*/
readonly dummyTargetNode: INode;
/**
The {@link IPort} which is used as target port for the temporarily created {@link CreateEdgeInputMode#dummyEdge} during edge creation.
This port's owner is the {@link CreateEdgeInputMode#dummyTargetNode}. This port is used as target port of the {@link CreateEdgeInputMode#dummyEdge} unless the dummy edge snaps to a valid {@link CreateEdgeInputMode#targetPortCandidate} in which
case the edge is connected to the port represented by that candidate.
The port is created upon first access by method {@link CreateEdgeInputMode#createDummyTargetNodePort}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23dummyTargetNodePort Online Documentation}*/
readonly dummyTargetNodePort: IPort;
/**
Gets or sets whether or not the edge's end point should snap to the current target port candidate during the creation of
the edge.
This will make the edge's end point point to the position of the target port candidate. This will result in the preview
edge looking exactly like the edge after creation. By default this feature is enabled.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23snapToTargetCandidate Online Documentation}*/
snapToTargetCandidate: boolean;
/**
* Gets or sets a {@link IHitTestable} that determines whether to show source port candidates if the cursor hovers
* over the provided location.
* If {@link CreateEdgeInputMode#showPortCandidates} is set to {@link ShowPortCandidates#SOURCE} or {@link ShowPortCandidates#ALL} the source port candidates
* will be shown as soon as this testable returns <code>true</code>.
*
* The default implementation returns <code>true</code> if the cursor is located over a port owner.
*
* To avoid showing port candidates for selected nodes (which will by default be moved instead of being considered as
* source of an edge) developers might set a hit testable which excludes selected nodes.
*
* <pre>
* class UnselectedNodesHitTestable extends IHitTestable {
*   /**
*    *  @param {IInputModeContext} context
*    *  @param {Point} location
*    *‍/
*   isHit(context, location) {
*     /** @type {GraphComponent} *‍/
*     const graphComponent = context.canvasComponent
*     return graphComponent.graph.nodes.some(
*       node =>
*         !graphComponent.selection.isSelected(node) &&
*         node.style.renderer.getHitTestable(node, node.style).isHit(context, location)
*     )
*   }
* }
* </pre>
*
* Note that the same hit testable should be set to {@link CreateEdgeInputMode#beginHitTestable}, too.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23showSourcePortCandidatesHitTestable Online Documentation}
*/
showSourcePortCandidatesHitTestable: IHitTestable;
/**
* Gets or sets a {@link IHitTestable} that determines whether it is valid to start an edge creation gesture here.
* The default implementation returns <code>true</code> if the cursor is located over a port owner.
*
* To avoid showing port candidates for selected nodes (which will by default be moved instead of being considered as
* source of an edge) developers might set a hit testable which excludes selected nodes.
*
* <pre>
* class UnselectedNodesHitTestable extends IHitTestable {
*   /**
*    *  @param {IInputModeContext} context
*    *  @param {Point} location
*    *‍/
*   isHit(context, location) {
*     /** @type {GraphComponent} *‍/
*     const graphComponent = context.canvasComponent
*     return graphComponent.graph.nodes.some(
*       node =>
*         !graphComponent.selection.isSelected(node) &&
*         node.style.renderer.getHitTestable(node, node.style).isHit(context, location)
*     )
*   }
* }
* </pre>
*
* Note that the same hit testable should be set to {@link CreateEdgeInputMode#showSourcePortCandidatesHitTestable}, too.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23beginHitTestable Online Documentation}
*/
beginHitTestable: IHitTestable;
/**
Gets or sets a {@link IHitTestable} that determines whether it is valid to finish an edge creation gesture here if
valid {@link IPortCandidate}s are available.
This hit testable is only queried if {@link CreateEdgeInputMode#forceSnapToCandidate} is disabled.
If this {@link IHitTestable} returns <code>true</code> a valid port candidate is searched. If and only if one is found the
current location is considered as possible end for an edge creation. If this {@link IHitTestable} returns <code>false</code> {@link CreateEdgeInputMode#prematureEndHitTestable} is
queried.
The default implementation returns <code>true</code> if there is a port owner at the given location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23endHitTestable Online Documentation}*/
endHitTestable: IHitTestable;
/**
Gets or sets a {@link IHitTestable} that determines, whether it is valid to finish an edge creation gesture here
even if no {@link IPortCandidate}s are available.
This hit testable is only queried if {@link CreateEdgeInputMode#forceSnapToCandidate} is disabled and {@link CreateEdgeInputMode#endHitTestable} returns <code>false</code> or no valid port candidate is found.
If this {@link IHitTestable} returns <code>true</code> the current location is considered as possible end for an edge creation,
even if no valid {@link IPortCandidate} is available as target port candidate.
The default implementation always returns <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23prematureEndHitTestable Online Documentation}*/
prematureEndHitTestable: IHitTestable;
/**
Gets or sets a {@link IHitTestable} that determines whether it is valid to create a bend here.
The default implementation returns always <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23validBendHitTestable Online Documentation}*/
validBendHitTestable: IHitTestable;
/**
* Gets or sets whether edge creation should only start if the pointer is located directly over a valid port candidate.
* If set to <code>false</code> edge creation can start if the pointer is located anywhere over a port owner. In this case the closest
* valid port candidate will be chosen as source port.
*
* If set to <code>true</code> edge creation can only start if the pointer is located over a valid port candidate. This candidate will
* be chosen as source port. The radius within which a port is considered as hit can be set using {@link CreateEdgeInputMode#sourcePortCandidateHitRadius}. Showing source port
* candidates can be enabled using {@link CreateEdgeInputMode#showPortCandidates}.
*
* Default is <code>false</code>.
*
* A possible use case is to support edge creation at selected nodes without disabling the {@link GraphEditorInputMode#moveInputMode}. The following example sets a
* higher priority to the {@link GraphEditorInputMode#createEdgeInputMode} but restricts possible start points to valid port candidates. That way, if a user drags from a
* valid port candidate, edge creation will start. If he drags from other regions of the selected node, the node will be
* moved:
*
* <pre>
* graphEditorInputMode.moveInputMode.priority = 45
* graphEditorInputMode.createEdgeInputMode.priority = 40
* graphEditorInputMode.createEdgeInputMode.showPortCandidates = ShowPortCandidates.ALL
* graphEditorInputMode.createEdgeInputMode.startOverCandidateOnly = true
* </pre>
*
* @see CreateEdgeInputMode#sourcePortCandidateHitRadius
* @see CreateEdgeInputMode#showPortCandidates
* @default false
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23startOverCandidateOnly Online Documentation}
*/
startOverCandidateOnly: boolean;
/**
Gets or sets the radius from within a valid port candidate will be considered as hit.
The radius is in world coordinates.
This value will be considered if {@link CreateEdgeInputMode#startOverCandidateOnly} is set to <code>true</code>.
Default is 5.
@see CreateEdgeInputMode#startOverCandidateOnly
@default 5.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23sourcePortCandidateHitRadius Online Documentation}*/
sourcePortCandidateHitRadius: number;
/**
Gets a copy of the starting point at which the edge creation gesture started.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23startPoint Online Documentation}*/
readonly startPoint: Point;
/**
Gets the dynamically updated dragging point for the edge creation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23dragPoint Online Documentation}*/
readonly dragPoint: IPoint;
/**
Gets whether currently an edge creation is in progress.
This property yields <code>true</code> after the edge creation has been {@link CreateEdgeInputMode#addGestureStartedListener started} and before it has been {@link CreateEdgeInputMode#cancel cancelled} or finalized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23isCreationInProgress Online Documentation}*/
readonly isCreationInProgress: boolean;
/**
Adds the given listener for the <code>GestureFinishing</code> event that occurs before the gesture will be finished.
@param listener The listener to add.
@see CreateEdgeInputMode#removeGestureFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureFinishing Online Documentation}*/
addGestureFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>GestureFinishing</code> event that occurs before the gesture will be finished.
@param listener The listener to remove.
@see CreateEdgeInputMode#addGestureFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureFinishing Online Documentation}*/
removeGestureFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Adds the given listener for the <code>GestureFinished</code> event that occurs once the gesture has been finished.
* <pre>
* createEdgeInputMode.addGestureStartedListener(
*   (sender, args) => (statusLabel.content = 'Edge creation in progress...')
* )
* createEdgeInputMode.addGestureCanceledListener((sender, args) => (statusLabel.content = ''))
* createEdgeInputMode.addGestureFinishedListener((sender, args) => (statusLabel.content = ''))
* </pre>
*
* @param listener The listener to add.
* @see CreateEdgeInputMode#removeGestureFinishedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureFinished Online Documentation}
*/
addGestureFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Removes the given listener for the <code>GestureFinished</code> event that occurs once the gesture has been finished.
* <pre>
* createEdgeInputMode.addGestureStartedListener(
*   (sender, args) => (statusLabel.content = 'Edge creation in progress...')
* )
* createEdgeInputMode.addGestureCanceledListener((sender, args) => (statusLabel.content = ''))
* createEdgeInputMode.addGestureFinishedListener((sender, args) => (statusLabel.content = ''))
* </pre>
*
* @param listener The listener to remove.
* @see CreateEdgeInputMode#addGestureFinishedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureFinished Online Documentation}
*/
removeGestureFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>GestureStarting</code> event that occurs once the actual creation gesture is starting, that is
when starting to drag the edge from its source node.
@param listener The listener to add.
@see CreateEdgeInputMode#removeGestureStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureStarting Online Documentation}*/
addGestureStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>GestureStarting</code> event that occurs once the actual creation gesture is starting, that
is when starting to drag the edge from its source node.
@param listener The listener to remove.
@see CreateEdgeInputMode#addGestureStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureStarting Online Documentation}*/
removeGestureStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Adds the given listener for the <code>GestureStarted</code> event that occurs once the actual creation gesture is initialized and has
* started, that is when the edge is dragged from its source node.
* <pre>
* createEdgeInputMode.addGestureStartedListener((sender, args) => {
*   createEdgeInputMode.dummyEdgeGraph.setStyle(
*     createEdgeInputMode.dummyTargetNode,
*     new ShinyPlateNodeStyle({ fill: Fill.ORANGE })
*   )
* })
* </pre>
*
* <pre>
* createEdgeInputMode.addGestureStartedListener(
*   (sender, args) => (statusLabel.content = 'Edge creation in progress...')
* )
* createEdgeInputMode.addGestureCanceledListener((sender, args) => (statusLabel.content = ''))
* createEdgeInputMode.addGestureFinishedListener((sender, args) => (statusLabel.content = ''))
* </pre>
*
* @param listener The listener to add.
* @see CreateEdgeInputMode#removeGestureStartedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureStarted Online Documentation}
*/
addGestureStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Removes the given listener for the <code>GestureStarted</code> event that occurs once the actual creation gesture is initialized and
* has started, that is when the edge is dragged from its source node.
* <pre>
* createEdgeInputMode.addGestureStartedListener((sender, args) => {
*   createEdgeInputMode.dummyEdgeGraph.setStyle(
*     createEdgeInputMode.dummyTargetNode,
*     new ShinyPlateNodeStyle({ fill: Fill.ORANGE })
*   )
* })
* </pre>
*
* <pre>
* createEdgeInputMode.addGestureStartedListener(
*   (sender, args) => (statusLabel.content = 'Edge creation in progress...')
* )
* createEdgeInputMode.addGestureCanceledListener((sender, args) => (statusLabel.content = ''))
* createEdgeInputMode.addGestureFinishedListener((sender, args) => (statusLabel.content = ''))
* </pre>
*
* @param listener The listener to remove.
* @see CreateEdgeInputMode#addGestureStartedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureStarted Online Documentation}
*/
removeGestureStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Moving</code> event that occurs at the start of every drag or move.
@param listener The listener to add.
@see CreateEdgeInputMode#removeMovingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23Moving Online Documentation}*/
addMovingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Moving</code> event that occurs at the start of every drag or move.
@param listener The listener to remove.
@see CreateEdgeInputMode#addMovingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23Moving Online Documentation}*/
removeMovingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Moved</code> event that occurs at the end of every drag or move.
@param listener The listener to add.
@see CreateEdgeInputMode#removeMovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23Moved Online Documentation}*/
addMovedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Moved</code> event that occurs at the end of every drag or move.
@param listener The listener to remove.
@see CreateEdgeInputMode#addMovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23Moved Online Documentation}*/
removeMovedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Adds the given listener for the <code>GestureCanceled</code> event that occurs when the gesture has been canceled.
* <pre>
* createEdgeInputMode.addGestureStartedListener(
*   (sender, args) => (statusLabel.content = 'Edge creation in progress...')
* )
* createEdgeInputMode.addGestureCanceledListener((sender, args) => (statusLabel.content = ''))
* createEdgeInputMode.addGestureFinishedListener((sender, args) => (statusLabel.content = ''))
* </pre>
*
* @param listener The listener to add.
* @see CreateEdgeInputMode#removeGestureCanceledListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureCanceled Online Documentation}
*/
addGestureCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Removes the given listener for the <code>GestureCanceled</code> event that occurs when the gesture has been canceled.
* <pre>
* createEdgeInputMode.addGestureStartedListener(
*   (sender, args) => (statusLabel.content = 'Edge creation in progress...')
* )
* createEdgeInputMode.addGestureCanceledListener((sender, args) => (statusLabel.content = ''))
* createEdgeInputMode.addGestureFinishedListener((sender, args) => (statusLabel.content = ''))
* </pre>
*
* @param listener The listener to remove.
* @see CreateEdgeInputMode#addGestureCanceledListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureCanceled Online Documentation}
*/
removeGestureCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>GestureCanceling</code> event that occurs before the gesture will be canceled.
@param listener The listener to add.
@see CreateEdgeInputMode#removeGestureCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureCanceling Online Documentation}*/
addGestureCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>GestureCanceling</code> event that occurs before the gesture will be canceled.
@param listener The listener to remove.
@see CreateEdgeInputMode#addGestureCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23GestureCanceling Online Documentation}*/
removeGestureCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
* Adds the given listener for the <code>EdgeCreationStarted</code> event that occurs when the edge creation has started, that is when
* the cursor is dragged over the source node.
* The edge that is stored in the {@link ItemEventArgs#item} property is the {@link CreateEdgeInputMode#dummyEdge} that is used during creation.
*
* <pre>
* createEdgeInputMode.addGestureStartedListener((sender, args) => {
*   const polylineEdgeStyle = createEdgeInputMode.dummyEdge.style
*   if (polylineEdgeStyle !== null) {
*     polylineEdgeStyle.stroke = Stroke.RED
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see CreateEdgeInputMode#removeEdgeCreationStartedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23EdgeCreationStarted Online Documentation}
*/
addEdgeCreationStartedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
* Removes the given listener for the <code>EdgeCreationStarted</code> event that occurs when the edge creation has started, that is
* when the cursor is dragged over the source node.
* The edge that is stored in the {@link ItemEventArgs#item} property is the {@link CreateEdgeInputMode#dummyEdge} that is used during creation.
*
* <pre>
* createEdgeInputMode.addGestureStartedListener((sender, args) => {
*   const polylineEdgeStyle = createEdgeInputMode.dummyEdge.style
*   if (polylineEdgeStyle !== null) {
*     polylineEdgeStyle.stroke = Stroke.RED
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see CreateEdgeInputMode#addEdgeCreationStartedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23EdgeCreationStarted Online Documentation}
*/
removeEdgeCreationStartedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
Gets or sets the event recognizer that will be queried to decide if bend creation is enforced.
The default is {@link KeyEventRecognizers#CTRL_IS_DOWN} or {@link KeyEventRecognizers#META_IS_DOWN}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23enforceBendCreationRecognizer Online Documentation}*/
enforceBendCreationRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets the graph this mode is acting upon.
@throws {Exception} <b>(NotSupportedError)</b> If there is no input mode context available or the context does not contain the graph in the lookup.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CreateEdgeInputMode%23graph Online Documentation}*/
readonly graph: IGraph;
static $class: Class;
private $_CreateEdgeInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CreateEdgeInputMode;
}
/**
Defines the direction of an edge segment
This enumeration is used by {@link CreateEdgeInputMode#getFirstSegmentDirection}
@see CreateEdgeInputMode#getFirstSegmentDirection
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentDirection Online Documentation}*/
export enum EdgeSegmentDirection {
/**
The edge segment is going into north direction
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentDirection%23NORTH Online Documentation}*/
NORTH,
/**
The edge segment is going into east direction
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentDirection%23EAST Online Documentation}*/
EAST,
/**
The edge segment is going into south direction
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentDirection%23SOUTH Online Documentation}*/
SOUTH,
/**
The edge segment is going into west direction
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSegmentDirection%23WEST Online Documentation}*/
WEST
}
/** Automatic conversion to {@link EdgeSegmentDirection}. */
type EdgeSegmentDirectionStringValues = "north" | "east" | "south" | "west";
/**
Defines whether to show the visualization of source and target port candidates during edge creation.
This enumeration is used by {@link CreateEdgeInputMode#showPortCandidates}
@see CreateEdgeInputMode#showPortCandidates
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShowPortCandidates Online Documentation}*/
export enum ShowPortCandidates {
/**
Don't show any port candidates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShowPortCandidates%23NONE Online Documentation}*/
NONE,
/**
Show port candidates to start the edge creation at.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShowPortCandidates%23SOURCE Online Documentation}*/
SOURCE,
/**
Show port candidates for possible ends of the newly created edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShowPortCandidates%23TARGET Online Documentation}*/
TARGET,
/**
Show port candidates for both the start and the end of the newly created edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShowPortCandidates%23ALL Online Documentation}*/
ALL
}
type ShowPortCandidatesStringValues = "none" | "source" | "target" | "all";
/** Automatic conversion to {@link ShowPortCandidates}. */
type ShowPortCandidatesConvertible = ShowPortCandidatesStringValues | ShowPortCandidatesStringValues[];
/**
Policy to determine how the {@link CanvasComponent#contentRect content rectangle} of the {@link GraphComponent} should be recalculated after interactive
graph changes.
@see GraphEditorInputMode#adjustContentRectPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjustContentRectPolicy Online Documentation}*/
export enum AdjustContentRectPolicy {
/**
The content rectangle is not adjusted automatically after interactive changes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjustContentRectPolicy%23NEVER Online Documentation}*/
NEVER,
/**
The new content rectangle is recalculated with the union of the previous content rectangle and the new
{@link GraphComponent#fitGraphBounds bounds of the graph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjustContentRectPolicy%23UNION Online Documentation}*/
UNION,
/**
The new content rectangle is recalculated with the new {@link GraphComponent#fitGraphBounds bounds of the graph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AdjustContentRectPolicy%23ALWAYS Online Documentation}*/
ALWAYS
}
/** Automatic conversion to {@link AdjustContentRectPolicy}. */
type AdjustContentRectPolicyStringValues = "never" | "union" | "always";
export interface GraphEditorInputMode extends GraphInputMode {
/**
@inheritdoc*/
uninstall(context: IInputModeContext): void;
} /**
* A complex {@link IInputMode} that can be used in a {@link GraphComponent} to edit an {@link IGraph}
* instance.
* This class delegates most of the work to minor {@link IInputMode} implementations. However it contains a number of
* properties that influence the general behavior:
*
* <ul>
* <li><b>{@link GraphInputMode#selectableItems}</b> – Determines the types of items that can be selected at all.</li>
* <li><b>{@link GraphInputMode#selectablePredicate}</b> – Allows to further constrain {@link GraphInputMode#selectableItems} with a predicate.</li>
* <li><b>{@link GraphInputMode#marqueeSelectableItems}</b> – Determines the types of items that can be selected with {@link MarqueeSelectionInputMode marquee selection} and {@link LassoSelectionInputMode lasso selection}.</li>
* <li><b>{@link GraphInputMode#clickSelectableItems}</b> – Determines the types of items that can be selected by {@link ClickInputMode clicking them}.</li>
* <li><b>{@link GraphInputMode#detailSelectionRecognizer}</b> – Determines when to use a detail click selection mode, which allows for selecting the item that is directly under the
* mouse without taking into account the {@link GraphInputMode#clickHitTestOrder}.</li>
* <li><b>{@link GraphInputMode#clickHitTestOrder}</b> – Determines the priorities for hit tests during mouse clicks to determine which items should get {@link GraphInputMode#addItemClickedListener click events}.</li>
* <li><b>{@link GraphInputMode#doubleClickHitTestOrder}</b> – Determines the priorities for hit tests when {@link GraphInputMode#addItemDoubleClickedListener double-clicking items}.</li>
* <li><b>{@link GraphEditorInputMode#movableItems}</b> – Determines the type of the items that can be {@link GraphEditorInputMode#moveInputMode moved} with the mouse.</li>
* <li><b>{@link GraphEditorInputMode#showHandleItems}</b> – Determines the type of the items whose {@link IHandle}s should be shown and movable by the {@link GraphEditorInputMode#handleInputMode}.</li>
* <li><b>{@link GraphEditorInputMode#deletableItems}</b> – Determines the type of the items that can be deleted by {@link GraphEditorInputMode#deleteSelection}.</li>
* <li><b>{@link GraphEditorInputMode#deletablePredicate}</b> – Allows to further constrain {@link GraphEditorInputMode#deletableItems} with a predicate.</li>
* <li><b>{@link GraphEditorInputMode#labelEditableItems}</b> – Determines the type of the items that can have their labels edited.</li>
* <li><b>{@link GraphEditorInputMode#allowGroupingOperations}</b> – Determines whether this input mode should allow grouping operations at all.</li>
* <li><b>{@link GraphEditorInputMode#allowGroupSelection}</b> – Determines whether this input mode should allow {@link GraphEditorInputMode#groupSelection grouping the current selection}.</li>
* <li><b>{@link GraphEditorInputMode#allowUngroupSelection}</b> – Determines whether this input mode should allow {@link GraphEditorInputMode#ungroupSelection ungrouping the current selection}.</li>
* <li><b>{@link GraphEditorInputMode#allowClearSelection}</b> – Determines whether this input mode should allow {@link GraphInputMode#clearSelection clearing the selection}. This has implications both for the {@link ICommand#DESELECT_ALL}
* and for click/marquee selection which usually clear the selection unless the {@link GraphInputMode#multiSelectionRecognizer} is pressed.</li>
* <li><b>{@link GraphInputMode#allowClipboardOperations}</b> – Determines whether this input mode should enable {@link GraphClipboard graph clipboard operations.}</li>
* <li><b>{@link GraphEditorInputMode#allowDuplicate}</b> – Determines whether this input mode should enable {@link GraphClipboard graph clipboard's} {@link GraphClipboard#duplicate} operation.</li>
* <li><b>{@link GraphEditorInputMode#allowEditLabel}</b> – Determines whether this input mode should allow editing labels.</li>
* <li><b>{@link GraphEditorInputMode#allowReparentNodes}</b> – Determines whether this input mode should allow {@link IReparentNodeHandler reparenting nodes to other groups.}</li>
* <li><b>{@link GraphEditorInputMode#allowReparentToNonGroupNodes}</b> – Determines whether this input mode should allow
* {@link IReparentNodeHandler reparenting nodes to other nodes which are not group nodes, transforming these into group nodes.}</li>
* <li><b>{@link GraphEditorInputMode#allowAdjustGroupNodeSize}</b> – Determines whether the {@link ICommand#ADJUST_GROUP_NODE_SIZE} command should be handled.</li>
* <li><b>{@link GraphEditorInputMode#allowCreateNode}</b> – Determines whether this input mode should allow {@link GraphEditorInputMode#nodeCreator creating new nodes} via simple mouse clicks.</li>
* <li><b>{@link GraphEditorInputMode#allowCreateEdge}</b> – Determines whether {@link GraphEditorInputMode#createEdgeInputMode} should be {@link CreateEdgeInputMode#enabled} to allow for creating new edges.</li>
* <li><b>{@link GraphEditorInputMode#allowEditLabelOnDoubleClick}</b> – Determines whether labels should be editable on otherwise unhandled double clicks.</li>
* <li><b>{@link GraphEditorInputMode#allowReverseEdge}</b> – Determines whether edges can be reversed.</li>
* <li><b>{@link GraphEditorInputMode#allowCreateBend}</b> – Determines whether the {@link GraphEditorInputMode#createBendInputMode} should be {@link CreateBendInputMode#enabled} to allow for creating new bends.</li>
* <li><b>{@link NavigationInputMode#allowEnterGroup}</b> – Determines whether the mode allows for {@link NavigationInputMode#enterGroup entering} group nodes in {@link IFoldingView}s.</li>
* <li><b>{@link NavigationInputMode#allowExitGroup}</b> – Determines whether the mode allows for {@link NavigationInputMode#exitGroup exiting} the current group node in {@link IFoldingView}s.</li>
* <li><b>{@link NavigationInputMode#allowCollapseGroup}</b> – Determines whether the mode allows for {@link NavigationInputMode#collapseGroup collapsing} group nodes in {@link IFoldingView}s.</li>
* <li><b>{@link NavigationInputMode#allowExpandGroup}</b> – Determines whether the mode allows for {@link NavigationInputMode#expandGroup expanding} group nodes in {@link IFoldingView}s.</li>
* <li><b>{@link ItemHoverInputMode#hoverItems}</b> – Determines which items should be reported as being hovered through the {@link ItemHoverInputMode#addHoveredItemChangedListener corresponding event} in this instance's
* {@link ItemHoverInputMode}.</li>
* <li><b>{@link GraphInputMode#shouldInstallCommand}</b> – Determines which of the built-in {@link ICommand}s this input mode should provide and install.</li>
* </ul>
* By default, {@link GraphEditorInputMode} supports the following commands. Most of them have default keybindings:
*
* <ul>
* <li><b>{@link ICommand#UNDO} (Ctrl+Z)</b> – Undoes the last operation. Can be disabled with {@link GraphEditorInputMode#allowUndoOperations}.</li>
* <li><b>{@link ICommand#REDO} (Ctrl+Y)</b> – Redoes the last operation that was undone. Can be disabled with {@link GraphEditorInputMode#allowUndoOperations}.</li>
* <li><b>{@link ICommand#SELECT_ALL} (Ctrl+A)</b> – Selects all items. Can be configured with {@link GraphInputMode#selectableItems} and {@link GraphInputMode#selectablePredicate}.</li>
* <li><b>{@link ICommand#DESELECT_ALL} (Ctrl+Shift+A)</b> – Clears the current selection. Can be disabled with {@link GraphEditorInputMode#allowClearSelection}.</li>
* <li><b>{@link ICommand#CUT} (Ctrl+X, Shift+Del)</b> – Cuts the current selection and places it in the clipboard. Can be disabled with {@link GraphInputMode#allowClipboardOperations}.</li>
* <li><b>{@link ICommand#COPY} (Ctrl+C, Ctrl+Ins)</b> – Copies the current selection to the clipboard. Can be disabled with {@link GraphInputMode#allowClipboardOperations}.</li>
* <li><b>{@link ICommand#PASTE} (Ctrl+V, Shift+Ins)</b> – Pastes the current clipboard contents into the graph. Can be disabled with {@link GraphInputMode#allowClipboardOperations}.</li>
* <li><b>{@link ICommand#DUPLICATE} (Ctrl+D)</b> – Duplicates the current selection. Can be disabled with {@link GraphEditorInputMode#allowDuplicate}.</li>
* <li><b>{@link ICommand#DELETE} (Del)</b> – Deletes the current selection. Can be configured with {@link GraphEditorInputMode#deletablePredicate} and {@link GraphEditorInputMode#deletableItems}.</li>
* <li><b>{@link ICommand#ADD_LABEL} (Shift+F2)</b> – Adds a new label to the selected item. Can be configured with {@link GraphEditorInputMode#labelEditableItems} and disabled with {@link GraphEditorInputMode#allowAddLabel}.</li>
* <li><b>{@link ICommand#EDIT_LABEL} (F2)</b> – Edits an existing label or adds a new one to the selected item. Can be configured with {@link GraphEditorInputMode#labelEditableItems} and disabled with {@link GraphEditorInputMode#allowEditLabel}.</li>
* <li><b>{@link ICommand#REVERSE_EDGE} (Ctrl+R)</b> – Reverses the direction of the selected edges. Can be disabled with {@link GraphEditorInputMode#allowReverseEdge}.</li>
* <li><b>{@link ICommand#GROUP_SELECTION} (Ctrl+G)</b> – Creates a new group node and places the currently selected nodes into it. Can be disabled with {@link GraphEditorInputMode#allowGroupingOperations} and {@link GraphEditorInputMode#allowGroupSelection}.</li>
* <li><b>{@link ICommand#UNGROUP_SELECTION} (Ctrl+U)</b> – Makes each of the currently selected nodes a direct child of the root group. Can be disabled with {@link GraphEditorInputMode#allowGroupingOperations} and {@link GraphEditorInputMode#allowUngroupSelection}.</li>
* <li><b>{@link ICommand#ADJUST_GROUP_NODE_SIZE} (Ctrl+Shift+G)</b> – Adjusts the sizes of the selected group nodes to fit exactly around their contents. Can be disabled with {@link GraphEditorInputMode#allowGroupingOperations} and {@link GraphEditorInputMode#allowAdjustGroupNodeSize}.</li>
* <li><b>{@link ICommand#RAISE}</b> – Raises the visual representations of {@link IModelItem}s.</li>
* <li><b>{@link ICommand#LOWER}</b> – Lowers the visual representations of {@link IModelItem}s.</li>
* <li><b>{@link ICommand#TO_FRONT}</b> – Raises the visual representations of {@link IModelItem}s to the front.</li>
* <li><b>{@link ICommand#TO_BACK}</b> – Lowers the visual representations of {@link IModelItem}s to the back.</li>
* </ul>
* All commands can also be selectively disabled by using the {@link GraphInputMode#availableCommands} collection. Furthermore, there are a few commands provided
* by {@link GraphInputMode#navigationInputMode}:
*
* <ul>
* <li><b>{@link ICommand#COLLAPSE_GROUP} (Ctrl+Minus)</b> – Collapses the currently selected group nodes. Can be disabled with {@link NavigationInputMode#allowCollapseGroup} of {@link GraphInputMode#navigationInputMode}.</li>
* <li><b>{@link ICommand#EXPAND_GROUP} (Ctrl+Plus)</b> – Expands the currently selected group nodes. Can be disabled with {@link NavigationInputMode#allowExpandGroup} of {@link GraphInputMode#navigationInputMode}.</li>
* <li><b>{@link ICommand#TOGGLE_EXPANSION_STATE} (Ctrl+Multiply)</b> – Collapses expanded group nodes and expands collapsed group nodes in the current selection. Can be disabled with {@link NavigationInputMode#allowCollapseGroup} and {@link NavigationInputMode#allowExpandGroup} of
* {@link GraphInputMode#navigationInputMode}.</li>
* <li><b>{@link ICommand#ENTER_GROUP} (Ctrl+Enter)</b> – Replaces the currently displayed graph with the contents of the selected group node. Can be disabled with {@link NavigationInputMode#allowEnterGroup} of {@link GraphInputMode#navigationInputMode}.</li>
* <li><b>{@link ICommand#EXIT_GROUP} (Ctrl+Backspace)</b> – Replaces the currently displayed graph with the contents of the parent of the group node whose contents are currently
* displayed. Can be disabled with {@link NavigationInputMode#allowExitGroup} of {@link GraphInputMode#navigationInputMode}.</li>
* </ul>
* {@link GraphEditorInputMode} can be configured to automatically {@link GraphEditorInputMode#adjustContentRect adjust the content rectangle} of the {@link GraphEditorInputMode#graphComponent} whenever the
* graph is changed using gestures.
*
* This class contains a number of public methods that can be used to programmatically perform the corresponding actions,
* too.
*
* Setting the {@link GraphEditorInputMode#nodeCreator} to another instance or <code>null</code> changes the way nodes are created. Also the various minor {@link IInputMode}
* instances can be disabled, to change the behavior, too.
*
* This input mode manages a set of specialized input modes, each handling a specific part of the interaction with the
* graph. The following list details those along with their default
* {@link IInputMode#priority priorities}. Lower priorities come earlier when handling events.
*
* <ul>
* <li><b>{@link WaitInputMode} (−1)</b> – Disables this input mode when layouts or animations run.</li>
* <li><b>{@link GraphEditorInputMode#handleInputMode} (0)</b> – Deals with displaying handles on or around items and interacting with them.</li>
* <li><b>{@link KeyboardInputMode} (0)</b> – Handles commands and arbitrary keyboard shortcuts.</li>
* <li><b>{@link ClickInputMode} (10)</b> – Handles mouse clicks and double-clicks.</li>
* <li><b>{@link TapInputMode} (20)</b> – Handles taps and double-tap.</li>
* <li><b>{@link MoveViewportInputMode} (39)</b> – Allows panning the viewport.</li>
* <li><b>{@link GraphEditorInputMode#moveLabelInputMode} (39)</b> – Allows moving labels to new positions.</li>
* <li><b>{@link GraphEditorInputMode#moveInputMode} (40)</b> – Allows moving items that are not labels.</li>
* <li><b>{@link GraphEditorInputMode#moveUnselectedInputMode} (41)</b> – Allows moving unselected items.</li>
* <li><b>{@link GraphEditorInputMode#createBendInputMode} (42)</b> – Allows creating bends on edges.</li>
* <li><b>{@link GraphEditorInputMode#createEdgeInputMode} (45)</b> – Allows creating edges</li>
* <li><b>{@link LassoSelectionInputMode} (45)</b> – Allows dragging a lasso path to select items within it.</li>
* <li><b>{@link MarqueeSelectionInputMode} (50)</b> – Allows dragging a rectangle to select items within it.</li>
* <li><b>{@link ItemHoverInputMode} (55)</b> – Provides events to indicate that the mouse pointer hovers over an item.</li>
* <li><b>{@link NavigationInputMode} (55)</b> – Provides general navigation facilities, such as navigating from one item to another with the arrow keys, collapsing,
* expanding, entering or exiting groups, as well as providing commands for those tasks.</li>
* <li><b>{@link ContextMenuInputMode} (60)</b> – Handles querying the context menu contents based on a location as well as preparing the menu for display.</li>
* <li><b>{@link GraphEditorInputMode#nodeDropInputMode} (70)</b> – Allows dropping nodes on the {@link GraphEditorInputMode#graphComponent}, e.g. for dragging items from a palette.</li>
* <li><b>{@link GraphEditorInputMode#labelDropInputMode} (70)</b> – Allows dropping labels on the {@link GraphEditorInputMode#graphComponent}, e.g. for dragging items from a palette.</li>
* <li><b>{@link GraphEditorInputMode#portDropInputMode} (70)</b> – Allows dropping ports on the {@link GraphEditorInputMode#graphComponent}, e.g. for dragging items from a palette.</li>
* <li><b>{@link MouseHoverInputMode} (100)</b> – Handles tool tips.</li>
* <li><b>{@link GraphEditorInputMode#textEditorInputMode} (100)</b> – Handles editing labels.</li>
* </ul>
* To enable single selection mode, disable {@link MarqueeSelectionInputMode} and set {@link GraphInputMode#multiSelectionRecognizer} to
* {@link EventRecognizers#NEVER}. Remove {@link ICommand#SELECT_ALL} and {@link ICommand#TOGGLE_ITEM_SELECTION}
* from {@link GraphInputMode#availableCommands} and from {@link GraphInputMode#navigationInputMode}.{@link NavigationInputMode#availableCommands}. An example can be found in the Single Selection Demo.
*
* The <code>GraphEditorInputMode</code> can be installed by setting it as {@link CanvasComponent#inputMode} on the {@link GraphEditorInputMode#graphComponent} it handles.
*
* <br><b>Installing a configured GraphEditorInputMode</b>
* <pre>
* graphComponent.inputMode = new GraphEditorInputMode({
*   allowGroupingOperations: true,
*   snapContext: new GraphSnapContext(),
*   labelSnapContext: new LabelSnapContext(),
*   orthogonalEdgeEditingContext: new OrthogonalEdgeEditingContext({ enabled: false })
* })
* </pre>
*
* Note that the <code>GraphEditorInputMode</code> also supports grouping operations. This functionality, however, is disabled by
* default and has to be enabled. The following example shows how to enable grouping operations on the <code>GraphEditorInputMode</code>
* together with some detail settings:
*
* <br><b>Configuring grouping operations</b>
* <pre>
* // first and foremost: enable grouping operations at all
* geim.allowGroupingOperations = true
* // support grouping but not ungrouping the currently selected nodes
* geim.allowGroupSelection = true
* geim.allowUngroupSelection = false
* // forbid adjusting the size with Ctrl-Shift-G
* geim.allowAdjustGroupNodeSize = false
* // allow adding nodes to other group nodes by moving
* geim.allowReparentNodes = true
* // also allow to add nodes to non-group nodes, transforming these into groups
* geim.allowReparentToNonGroupNodes = true
* // support navigation like opening and closing groups
* geim.navigationInputMode.allowExpandGroup = true
* geim.navigationInputMode.allowCollapseGroup = true
* // forbid entering and exiting groups
* geim.navigationInputMode.allowEnterGroup = false
* geim.navigationInputMode.allowExitGroup = false
* </pre>
*
* Selecting multiple items is supported by default. Enabling selection for only one or no selected item at once is
* described in the {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_interaction_single_selection Developer's Guide} and shown in the following example:
*
* <pre>
* // disable marquee and lasso selection
* geim.marqueeSelectionInputMode.enabled = false
* geim.lassoSelectionInputMode.enabled = false
* // disable multi selection with Ctrl-Click
* geim.multiSelectionRecognizer = EventRecognizers.NEVER
*
* // deactivate commands that can lead to multi selection
* geim.availableCommands.remove(ICommand.TOGGLE_ITEM_SELECTION)
* geim.availableCommands.remove(ICommand.SELECT_ALL)
*
* geim.navigationInputMode.availableCommands.remove(ICommand.EXTEND_SELECTION_LEFT)
* geim.navigationInputMode.availableCommands.remove(ICommand.EXTEND_SELECTION_UP)
* geim.navigationInputMode.availableCommands.remove(ICommand.EXTEND_SELECTION_DOWN)
* geim.navigationInputMode.availableCommands.remove(ICommand.EXTEND_SELECTION_RIGHT)
*
* // disable selection of (possibly multiple) items
* mode.pasteSelectableItems = GraphItemTypes.None
* </pre>
*
* An overview of this input mode is given in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/interaction-geim Editing Graphs}. Additionally, specific support for graph
* hierarchies is discussed in {@link https://docs.yworks.com/yfileshtml/#/dguide/interaction-grouping Supported User Interactions: Grouping}.
*
* @see GraphEditorInputMode#createEdgeInputMode
* @see GraphEditorInputMode#createBendInputMode
* @see GraphEditorInputMode#moveLabelInputMode
* @see GraphEditorInputMode#nodeDropInputMode
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode Online Documentation}
*/
export class GraphEditorInputMode {
/**
Creates a new instance that is not bound to an existing graph or selection, initially.
The {@link IGraph} and {@link IGraphSelection} instances are obtained later from the {@link IInputModeContext}
once this mode gets {@link GraphEditorInputMode#install installed} in a suitable canvas control.
@param [options] The parameters to pass.
@param [options.orthogonalEdgeEditingContext] The orthogonal edge editing context. This option sets the {@link GraphEditorInputMode#orthogonalEdgeEditingContext} property on the created object.
@param [options.orthogonalBendRemoval] The policy that controls whether removing bends from an orthogonal edge will keep the edge orthogonal. This option sets the {@link GraphEditorInputMode#orthogonalBendRemoval} property on the created object.
@param [options.reparentNodeHandler] The {@link IReparentNodeHandler} that is used for moving nodes out of group nodes or into other group nodes. This option sets the {@link GraphEditorInputMode#reparentNodeHandler} property on the created object.
@param [options.hitTester] The hitTester property. This option sets the {@link GraphEditorInputMode#hitTester} property on the created object.
@param [options.pasteSelectableItems] Which types of items should be selected after {@link GraphEditorInputMode#paste} or {@link GraphEditorInputMode#duplicateSelection}. This option sets the {@link GraphEditorInputMode#pasteSelectableItems} property on the created object.
@param [options.smartPasteSelection] Whether nodes, edges, labels, and ports should only be selected after {@link GraphEditorInputMode#paste} or {@link GraphEditorInputMode#duplicateSelection} if they were selected when they were initially copied into the clipboard. This option sets the {@link GraphEditorInputMode#smartPasteSelection} property on the created object.
@param [options.movableItems] Which types of items should be movable using the {@link GraphEditorInputMode#moveInputMode} or {@link GraphEditorInputMode#moveLabelInputMode} respectively in case of labels. This option sets the {@link GraphEditorInputMode#movableItems} property on the created object.
@param [options.deletableItems] Which types of items may be deleted using the {@link GraphEditorInputMode#deleteSelection} action. This option sets the {@link GraphEditorInputMode#deletableItems} property on the created object.
@param [options.deletablePredicate] A predicate that is queried to determine whether an {@link IModelItem} can be deleted or not. This option sets the {@link GraphEditorInputMode#deletablePredicate} property on the created object.
@param [options.labelEditableItems] Which types of items may have their {@link GraphEditorInputMode#onEditLabel labels edited}. This option sets the {@link GraphEditorInputMode#labelEditableItems} property on the created object.
@param [options.handleInputMode] The {@link HandleInputMode} as child input mode. This option sets the {@link GraphEditorInputMode#handleInputMode} property on the created object.
@param [options.showHandleItems] Which types of items should have their {@link IHandle}s shown. This option sets the {@link GraphEditorInputMode#showHandleItems} property on the created object.
@param [options.useCurrentItemForCommands] A value indicating whether to use the {@link GraphComponent#currentItem} as a fallback for the commands if no item is provided in the parameter and the current selection is empty. This option sets the {@link GraphEditorInputMode#useCurrentItemForCommands} property on the created object.
@param [options.pasteDelta] The offset for paste operation if clipboard operations are enabled. This option sets the {@link GraphEditorInputMode#pasteDelta} property on the created object.
@param [options.allowEditLabel] A value determining whether the {@link ICommand#EDIT_LABEL} command should be handled. This option sets the {@link GraphEditorInputMode#allowEditLabel} property on the created object.
@param [options.allowEditLabelOnDoubleClick] A value determining whether double-clicking should start label editing. This option sets the {@link GraphEditorInputMode#allowEditLabelOnDoubleClick} property on the created object.
@param [options.allowAddLabel] A value determining whether the {@link ICommand#ADD_LABEL} command should be handled. This option sets the {@link GraphEditorInputMode#allowAddLabel} property on the created object.
@param [options.textEditorInputModeConfigurator] A handler which allows for configuring the {@link GraphEditorInputMode#textEditorInputMode} during adding or editing labels. This option sets the {@link GraphEditorInputMode#textEditorInputModeConfigurator} property on the created object.
@param [options.moveInputMode] The {@link MoveInputMode} as child mode. This option sets the {@link GraphEditorInputMode#moveInputMode} property on the created object.
@param [options.moveUnselectedInputMode] The {@link MoveInputMode} which handles unselected graph items. This option sets the {@link GraphEditorInputMode#moveUnselectedInputMode} property on the created object.
@param [options.adjustContentRectPolicy] A property that determines whether and how this instance should automatically adjust the {@link CanvasComponent#contentRect} if the graph changes. This option sets the {@link GraphEditorInputMode#adjustContentRectPolicy} property on the created object.
@param [options.contentRectMargins] The margins to use for {@link CanvasComponent#updateContentRect} in {@link GraphEditorInputMode#adjustContentRect} calls. This option sets the {@link GraphEditorInputMode#contentRectMargins} property on the created object.
@param [options.snapContext] The {@link SnapContext} instance that handles interactive snapping of elements during drag operations like movements. This option sets the {@link GraphEditorInputMode#snapContext} property on the created object.
@param [options.labelSnapContext] The {@link LabelSnapContext} instance that handles interactive snapping of {@link ILabel}s to their owner during drag operations like movements. This option sets the {@link GraphEditorInputMode#labelSnapContext} property on the created object.
@param [options.moveLabelInputMode] The {@link MoveLabelInputMode} which handles interactive moving {@link ILabel}s. This option sets the {@link GraphEditorInputMode#moveLabelInputMode} property on the created object.
@param [options.allowGroupingOperations] A value indicating whether grouping operations like {@link GraphEditorInputMode#allowGroupSelection grouping selected nodes} or {@link GraphEditorInputMode#allowReparentNodes moving nodes into group nodes} should be enabled. This option sets the {@link GraphEditorInputMode#allowGroupingOperations} property on the created object.
@param [options.allowGroupSelection] A value determining whether the {@link ICommand#GROUP_SELECTION} command should be handled. This option sets the {@link GraphEditorInputMode#allowGroupSelection} property on the created object.
@param [options.allowAdjustGroupNodeSize] A value determining whether the {@link ICommand#ADJUST_GROUP_NODE_SIZE} command should be handled. This option sets the {@link GraphEditorInputMode#allowAdjustGroupNodeSize} property on the created object.
@param [options.allowUngroupSelection] A value determining whether the {@link ICommand#UNGROUP_SELECTION} command should be handled. This option sets the {@link GraphEditorInputMode#allowUngroupSelection} property on the created object.
@param [options.allowClearSelection] A value determining whether the {@link ICommand#DESELECT_ALL} command should be handled. This option sets the {@link GraphEditorInputMode#allowClearSelection} property on the created object.
@param [options.allowPaste] A value determining whether the {@link ICommand#PASTE} command should be handled. This option sets the {@link GraphEditorInputMode#allowPaste} property on the created object.
@param [options.allowDuplicate] A value determining whether the {@link ICommand#DUPLICATE} command should be handled. This option sets the {@link GraphEditorInputMode#allowDuplicate} property on the created object.
@param [options.allowUndoOperations] A value indicating whether undo operations, that is, {@link ICommand#UNDO} and {@link ICommand#REDO} should be enabled. This option sets the {@link GraphEditorInputMode#allowUndoOperations} property on the created object.
@param [options.allowReparentNodes] A value indicating whether moving nodes out of group nodes or into other group nodes is allowed. This option sets the {@link GraphEditorInputMode#allowReparentNodes} property on the created object.
@param [options.allowReparentToNonGroupNodes] A value indicating whether moving nodes into {@link IGraph#isGroupNode non-group nodes} is allowed to convert them into group nodes. This option sets the {@link GraphEditorInputMode#allowReparentToNonGroupNodes} property on the created object.
@param [options.allowCreateNode] A value determining whether {@link GraphEditorInputMode#createNode node creation} by clicking on an empty canvas location is enabled. This option sets the {@link GraphEditorInputMode#allowCreateNode} property on the created object.
@param [options.allowCreateBend] A value determining whether {@link GraphEditorInputMode#createBendInputMode bend creation} should be {@link CreateBendInputMode#enabled enabled}. This option sets the {@link GraphEditorInputMode#allowCreateBend} property on the created object.
@param [options.allowCreateEdge] A value determining whether {@link GraphEditorInputMode#createEdgeInputMode edge creation} should be {@link CreateEdgeInputMode#enabled enabled}. This option sets the {@link GraphEditorInputMode#allowCreateEdge} property on the created object.
@param [options.allowReverseEdge] A value indicating whether the {@link ICommand#REVERSE_EDGE} command should be handled. This option sets the {@link GraphEditorInputMode#allowReverseEdge} property on the created object.
@param [options.nodeCreator] The callback that is responsible for creating a new node, for instance, in response to a mouse click. This option sets the {@link GraphEditorInputMode#nodeCreator} property on the created object.
@param [options.textEditorInputMode] The {@link TextEditorInputMode} which handles label editing. This option sets the {@link GraphEditorInputMode#textEditorInputMode} property on the created object.
@param [options.createEdgeInputMode] The {@link CreateEdgeInputMode} which handles interactive edge creation. This option sets the {@link GraphEditorInputMode#createEdgeInputMode} property on the created object.
@param [options.autoRemoveEmptyLabels] A value indicating whether this mode should automatically {@link IGraph#remove remove labels} from the graph when a label text has been edited and the label text is empty. This option sets the {@link GraphEditorInputMode#autoRemoveEmptyLabels} property on the created object.
@param [options.autoSelectSelfloopBends] A property that determines whether clicking on a node should automatically select all bends of self loops adjacent to that node. This option sets the {@link GraphEditorInputMode#autoSelectSelfloopBends} property on the created object.
@param [options.hideLabelDuringEditing] A property that determines whether the label should be hidden while it is edited. This option sets the {@link GraphEditorInputMode#hideLabelDuringEditing} property on the created object.
@param [options.createBendInputMode] The {@link CreateBendInputMode} which handles interactive bend creation on existing edges. This option sets the {@link GraphEditorInputMode#createBendInputMode} property on the created object.
@param [options.nodeDropInputMode] The {@link NodeDropInputMode} that is responsible for handling drag and drop operations of nodes onto the {@link GraphEditorInputMode#graphComponent}. This option sets the {@link GraphEditorInputMode#nodeDropInputMode} property on the created object.
@param [options.labelDropInputMode] The {@link LabelDropInputMode} that is responsible for handling drag and drop operations of labels onto the {@link GraphEditorInputMode#graphComponent}. This option sets the {@link GraphEditorInputMode#labelDropInputMode} property on the created object.
@param [options.portDropInputMode] The {@link PortDropInputMode} that is responsible for handling drag and drop operations of ports onto the {@link GraphEditorInputMode#graphComponent}. This option sets the {@link GraphEditorInputMode#portDropInputMode} property on the created object.
@param [options.waiting] Whether the user interaction should be blocked. This option sets the {@link GraphEditorInputMode#waiting} property on the created object.
@param [options.clickHitTestOrder] The order of the types of items that should be used to determine what item has been clicked or tapped. This option sets the {@link GraphInputMode#clickHitTestOrder} property on the created object.
@param [options.doubleClickHitTestOrder] The order of the types of items that should be used to determine what item has been double-clicked or double-tapped. This option sets the {@link GraphInputMode#doubleClickHitTestOrder} property on the created object.
@param [options.detailSelectionRecognizer] The event recognizer that determines if a click should select the item that is currently visible under the mouse cursor, without giving more important items higher priority. This option sets the {@link GraphInputMode#detailSelectionRecognizer} property on the created object.
@param [options.cyclicSelectionRecognizer] The event recognizer that determines if a click should cycle through all the items that are currently under the mouse cursor. This option sets the {@link GraphInputMode#cyclicSelectionRecognizer} property on the created object.
@param [options.multiSelectionRecognizer] The event recognizer that determines if a click is deemed a multi selection gesture. This option sets the {@link GraphInputMode#multiSelectionRecognizer} property on the created object.
@param [options.clickableItems] Which types of items should be reported through the {@link GraphInputMode#addItemClickedListener item click events}. This option sets the {@link GraphInputMode#clickableItems} property on the created object.
@param [options.focusableItems] The items that can be given focus via the {@link GraphInputMode#setCurrentItem} method. This option sets the {@link GraphInputMode#focusableItems} property on the created object.
@param [options.selectableItems] Which types of items should be selectable at all. This option sets the {@link GraphInputMode#selectableItems} property on the created object.
@param [options.selectablePredicate] A predicate that is queried to determine whether a given {@link IModelItem} is selectable. This option sets the {@link GraphInputMode#selectablePredicate} property on the created object.
@param [options.clickSelectableItems] Which types of items should be selectable by mouse clicks. This option sets the {@link GraphInputMode#clickSelectableItems} property on the created object.
@param [options.marqueeSelectableItems] Which types of items should be selected during {@link GraphInputMode#marqueeSelect marquee selections} or {@link GraphInputMode#lassoSelect lasso selection}. This option sets the {@link GraphInputMode#marqueeSelectableItems} property on the created object.
@param [options.ignoreVoidStyles] A value indicating whether void styles ({@link VoidNodeStyle}, {@link VoidEdgeStyle}, {@link VoidLabelStyle}, and {@link VoidPortStyle}) should be ignored when selecting or focusing items. This option sets the {@link GraphInputMode#ignoreVoidStyles} property on the created object.
@param [options.contextMenuInputMode] The {@link ContextMenuInputMode} which handles context menus on a {@link CanvasComponent}. This option sets the {@link GraphInputMode#contextMenuInputMode} property on the created object.
@param [options.contextMenuItems] The types of items a context menu should be queried for. This option sets the {@link GraphInputMode#contextMenuItems} property on the created object.
@param [options.mouseHoverInputMode] The {@link MouseHoverInputMode} which is responsible for displaying Tooltips on a {@link CanvasComponent}. This option sets the {@link GraphInputMode#mouseHoverInputMode} property on the created object.
@param [options.toolTipItems] The types of the items that should be queried a tool tip for. This option sets the {@link GraphInputMode#toolTipItems} property on the created object.
@param [options.clickInputMode] The {@link ClickInputMode} that is used by this instance to determine clicks. This option sets the {@link GraphInputMode#clickInputMode} property on the created object.
@param [options.tapInputMode] The {@link TapInputMode} which detects single and multiple taps on a {@link CanvasComponent}. This option sets the {@link GraphInputMode#tapInputMode} property on the created object.
@param [options.marqueeSelectionInputMode] The {@link MarqueeSelectionInputMode} which handles the selection of multiple elements by drawing a rectangle around them. This option sets the {@link GraphInputMode#marqueeSelectionInputMode} property on the created object.
@param [options.lassoSelectionInputMode] The {@link LassoSelectionInputMode} which handles the selection of multiple elements by drawing a polygon around them. This option sets the {@link GraphInputMode#lassoSelectionInputMode} property on the created object.
@param [options.navigationInputMode] The {@link NavigationInputMode} which is responsible for navigating and traversing the elements in the {@link IGraph}. This option sets the {@link GraphInputMode#navigationInputMode} property on the created object.
@param [options.waitInputMode] The {@link WaitInputMode} that is provided by this instance for those who need to make use of it. This option sets the {@link GraphInputMode#waitInputMode} property on the created object.
@param [options.itemHoverInputMode] The {@link ItemHoverInputMode} that is provided by this instance to detect when the cursor hovers over a graph item. This option sets the {@link GraphInputMode#itemHoverInputMode} property on the created object.
@param [options.moveViewportInputMode] The {@link MoveViewportInputMode} associated with this instance. This option sets the {@link GraphInputMode#moveViewportInputMode} property on the created object.
@param [options.keyboardInputMode] The {@link KeyboardInputMode} associated with this instance. This option sets the {@link GraphInputMode#keyboardInputMode} property on the created object.
@param [options.allowClipboardOperations] A property that determines whether clipboard operations with the usual shortcuts are enabled on the canvas. This option sets the {@link GraphInputMode#allowClipboardOperations} property on the created object.
@param [options.exclusive] A value indicating whether this mode will be the only one running when it has the mutex. This option sets the {@link MultiplexingInputMode#exclusive} property on the created object.
@param [options.enabled] The enabled state of this input mode. This option sets the {@link MultiplexingInputMode#enabled} property on the created object.
@param [options.priority] Sets the priority property.
@param [options.defaultCursor] The cursor to use whenever no child mode prefers a different cursor. This option sets the {@link MultiplexingInputMode#defaultCursor} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode Online Documentation}*/
constructor(options?: {
orthogonalEdgeEditingContext?: OrthogonalEdgeEditingContext | null;
orthogonalBendRemoval?: OrthogonalEdgeEditingPolicy | OrthogonalEdgeEditingPolicyStringValues;
reparentNodeHandler?: IReparentNodeHandler;
hitTester?: IHitTester<IModelItem>;
pasteSelectableItems?: GraphItemTypes | GraphItemTypesConvertible;
smartPasteSelection?: boolean;
movableItems?: GraphItemTypes | GraphItemTypesConvertible;
deletableItems?: GraphItemTypes | GraphItemTypesConvertible;
deletablePredicate?: ((obj: IModelItem) => boolean) | null;
labelEditableItems?: GraphItemTypes | GraphItemTypesConvertible;
handleInputMode?: HandleInputMode;
showHandleItems?: GraphItemTypes | GraphItemTypesConvertible;
useCurrentItemForCommands?: boolean;
pasteDelta?: Point | PointConvertible;
allowEditLabel?: boolean;
allowEditLabelOnDoubleClick?: boolean;
allowAddLabel?: boolean;
textEditorInputModeConfigurator?: ((context: IInputModeContext, mode: TextEditorInputMode, label: ILabel) => void) | null;
moveInputMode?: MoveInputMode;
moveUnselectedInputMode?: MoveInputMode;
adjustContentRectPolicy?: AdjustContentRectPolicy | AdjustContentRectPolicyStringValues;
contentRectMargins?: Insets | InsetsConvertible;
snapContext?: SnapContext | null;
labelSnapContext?: SnapContext | null;
moveLabelInputMode?: MoveLabelInputMode;
allowGroupingOperations?: boolean;
allowGroupSelection?: boolean;
allowAdjustGroupNodeSize?: boolean;
allowUngroupSelection?: boolean;
allowClearSelection?: boolean;
allowPaste?: boolean;
allowDuplicate?: boolean;
allowUndoOperations?: boolean;
allowReparentNodes?: boolean;
allowReparentToNonGroupNodes?: boolean;
allowCreateNode?: boolean;
allowCreateBend?: boolean;
allowCreateEdge?: boolean;
allowReverseEdge?: boolean;
nodeCreator?: ((context: IInputModeContext, graph: IGraph, location: Point, parent: INode | null) => INode | null) | null;
textEditorInputMode?: TextEditorInputMode;
createEdgeInputMode?: CreateEdgeInputMode;
autoRemoveEmptyLabels?: boolean;
autoSelectSelfloopBends?: boolean;
hideLabelDuringEditing?: boolean;
createBendInputMode?: CreateBendInputMode;
nodeDropInputMode?: NodeDropInputMode;
labelDropInputMode?: LabelDropInputMode;
portDropInputMode?: PortDropInputMode;
waiting?: boolean;
clickHitTestOrder?: GraphItemTypes[];
doubleClickHitTestOrder?: GraphItemTypes[];
detailSelectionRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
cyclicSelectionRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
multiSelectionRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
clickableItems?: GraphItemTypes | GraphItemTypesConvertible;
focusableItems?: GraphItemTypes | GraphItemTypesConvertible;
selectableItems?: GraphItemTypes | GraphItemTypesConvertible;
selectablePredicate?: ((obj: IModelItem) => boolean) | null;
clickSelectableItems?: GraphItemTypes | GraphItemTypesConvertible;
marqueeSelectableItems?: GraphItemTypes | GraphItemTypesConvertible;
ignoreVoidStyles?: boolean;
contextMenuInputMode?: ContextMenuInputMode;
contextMenuItems?: GraphItemTypes | GraphItemTypesConvertible;
mouseHoverInputMode?: MouseHoverInputMode;
toolTipItems?: GraphItemTypes | GraphItemTypesConvertible;
clickInputMode?: ClickInputMode;
tapInputMode?: TapInputMode;
marqueeSelectionInputMode?: MarqueeSelectionInputMode;
lassoSelectionInputMode?: LassoSelectionInputMode;
navigationInputMode?: NavigationInputMode;
waitInputMode?: WaitInputMode;
itemHoverInputMode?: ItemHoverInputMode;
moveViewportInputMode?: MoveViewportInputMode;
keyboardInputMode?: KeyboardInputMode;
allowClipboardOperations?: boolean;
exclusive?: boolean;
enabled?: boolean;
priority?: number;
defaultCursor?: Cursor | null | CursorConvertible;
});
/**
Adds a label to the given <code>labelOwner</code> after the {@link GraphEditorInputMode#textEditorInputMode} has successfully finished editing after
{@link GraphEditorInputMode#addLabel}.
Labels with empty text will not be added if {@link GraphEditorInputMode#autoRemoveEmptyLabels} it set to <code>true</code>. Also this method will query the label's {@link ILabel#owner} for an {@link INodeSizeConstraintProvider}
if the owner is an {@link INode} and will make sure that the size constraints are still kept after adding the label
by calling {@link GraphEditorInputMode#adjustToSizeConstraints}.
@param labelOwner The owner to add the label to.
@param newText The text of the new label.
@param [preferredSize=null] The preferred size of the new label.
@param [param=null] The label model parameter of the new label.
@param [style=null] The style of the new label.
@param [tag=null] The tag of the new Label.
@returns The newly created label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23addLabel(ILabelOwner%2Cstring%2CILabelModelParameter%2CILabelStyle%2CSize%2CObject) Online Documentation}*/
addLabel(labelOwner: ILabelOwner, newText: string, param?: ILabelModelParameter | null, style?: ILabelStyle | null, preferredSize?: Size | null | SizeConvertible, tag?: any | null): ILabel | null;
/**
Adds a label to the given <code>labelOwner</code> after the {@link GraphEditorInputMode#textEditorInputMode} has successfully finished editing after
{@link GraphEditorInputMode#addLabel}.
Labels with empty text will not be added if {@link GraphEditorInputMode#autoRemoveEmptyLabels} it set to <code>true</code>. Also this method will query the label's {@link ILabel#owner} for an {@link INodeSizeConstraintProvider}
if the owner is an {@link INode} and will make sure that the size constraints are still kept after adding the label
by calling {@link GraphEditorInputMode#adjustToSizeConstraints}.
@param options The parameters to pass.
@param options.labelOwner The owner to add the label to.
@param options.newText The text of the new label.
@param [options.preferredSize=null] The preferred size of the new label.
@param [options.param=null] The label model parameter of the new label.
@param [options.style=null] The style of the new label.
@param [options.tag=null] The tag of the new Label.
@returns The newly created label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23addLabel(ILabelOwner%2Cstring%2CILabelModelParameter%2CILabelStyle%2CSize%2CObject) Online Documentation}*/
addLabel(options: {
labelOwner: ILabelOwner;
newText: string;
param?: ILabelModelParameter | null;
style?: ILabelStyle | null;
preferredSize?: Size | null | SizeConvertible;
tag?: any | null;
}): ILabel | null;
/**
Adds a new label to the given item interactively.
This will activate the {@link GraphEditorInputMode#textEditorInputMode} to let the user interactively enter the label's text and unless the user cancels label creation
the returned {@link Promise} will contain the label.
The text that the user enters may be {@link GraphEditorInputMode#addValidateLabelTextListener validated} before the label is actually added.
This method works nearly identical to {@link GraphEditorInputMode#createLabel}. The only difference is when either
the {@link GraphEditorInputMode#addLabelAddingListener LabelAdding} event is handled, or <code>owner</code>
has an
{@link IEditLabelHelper}, and via those instead of adding a label an existing one should be edited. This method
will edit that label instead of adding a new one.
@param owner The item to add a label to.
@returns A Promise that fulfills with either the newly created label instance or <code>null</code> if the {@link TextEditorInputMode#addEditingCanceledListener editing was canceled} or the text did
not {@link GraphEditorInputMode#addValidateLabelTextListener validate}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23addLabel(ILabelOwner) Online Documentation}*/
addLabel(owner: ILabelOwner): Promise<ILabel>;
/**
Updates the {@link CanvasComponent#contentRect} using {@link CanvasComponent#updateContentRect} and {@link GraphEditorInputMode#contentRectMargins}
This method is called whenever the content changes and takes the {@link GraphEditorInputMode#adjustContentRectPolicy} into account.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23adjustContentRect() Online Documentation}*/
adjustContentRect(): void;
/**
{@link IGraph#adjustGroupNodeLayout Adjusts the size} of the group nodes in the enumerable.
This action is executed in response to the {@link ICommand#ADJUST_GROUP_NODE_SIZE} if {@link GraphEditorInputMode#allowAdjustGroupNodeSize} is set to <code>true</code>.
@param nodes The nodes to adjust their sizes.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23adjustGroupNodeSize(IEnumerable%3CINode%3E) Online Documentation}*/
adjustGroupNodeSize(nodes: IEnumerable<INode>): void;
/**
{@link IGraph#adjustGroupNodeLayout Adjusts the size} of the selected group nodes.
This method delegates to {@link GraphEditorInputMode#adjustGroupNodeSize} with the {@link IGraphSelection#selectedNodes currently selected nodes}.
@see GraphEditorInputMode#adjustGroupNodeSize
@see ICommand#ADJUST_GROUP_NODE_SIZE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23adjustGroupNodeSizes() Online Documentation}*/
adjustGroupNodeSizes(): void;
/**
Adjusts the node's {@link INode#layout} to adhere to possible {@link INodeSizeConstraintProvider size constraints}.
This method will use the {@link INodeSizeConstraintProvider} for {@link INode} that can be found in their {@link ILookup}
to make sure they are still valid. The actual work of this method will be delegated to
{@link GraphEditorInputMode#setNodeLayout}.
@param node The node to possibly adjust the size of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23adjustToSizeConstraints(INode) Online Documentation}*/
adjustToSizeConstraints(node: INode): void;
/**
Creates a node on click.
This method is only called if no item has been hit and {@link GraphInputMode#clickClearSelection} returned <code>false</code>.
@param context The input mode context.
@param location The location of the click.
@returns <code>true</code> if a node was successfully created, <code>false</code> otherwise.
@see GraphEditorInputMode#shouldClickCreateNode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23clickCreateNode(IInputModeContext%2CPoint) Online Documentation}*/
clickCreateNode(context: IInputModeContext, location: Point): boolean;
/**
Closes the label editor if it is currently open.
@param cancel Whether to cancel the editor and discard changes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23closeLabelEditor(boolean) Online Documentation}*/
closeLabelEditor(cancel: boolean): void;
/**
Factory method for the {@link GraphEditorInputMode#createBendInputMode} property.
This method will be called upon first access to the {@link GraphEditorInputMode#createBendInputMode} property.
@returns a new instance of <code>CreateBendInputMode</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createCreateBendInputMode() Online Documentation}*/
createCreateBendInputMode(): CreateBendInputMode;
/**
Factory method for the {@link GraphEditorInputMode#createEdgeInputMode} property.
This method will be called upon first access to the {@link GraphEditorInputMode#createEdgeInputMode} property.
@returns a new instance of CreateEdgeInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createCreateEdgeInputMode() Online Documentation}*/
createCreateEdgeInputMode(): CreateEdgeInputMode;
/**
Factory method for the {@link GraphEditorInputMode#handleInputMode} property.
This method will be called upon first access to the {@link GraphEditorInputMode#handleInputMode} property.
@returns a new instance of HandleInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createHandleInputMode() Online Documentation}*/
createHandleInputMode(): HandleInputMode;
/**
Interactively creates a new label for the provided label owner.
This method will invoke the text editor that will let the user edit the text of the label. If the user commits the label
text the label will be added to the label owner.
The text that the user enters may be {@link GraphEditorInputMode#addValidateLabelTextListener validated} before the label is actually added.
This method works nearly identical to {@link GraphEditorInputMode#addLabel}. The only difference is when either the
{@link GraphEditorInputMode#addLabelAddingListener LabelAdding} event is handled, or <code>owner</code>
has an
{@link IEditLabelHelper}, and via those instead of adding a label an existing one should be edited. This method
will do nothing instead of editing an existing label or adding a new one in that case.
@param owner The item to create a new label for.
@returns A Promise that fulfills either with the edited label, or with <code>null</code> if {@link TextEditorInputMode#addEditingCanceledListener editing was canceled} or the text did not
{@link GraphEditorInputMode#addValidateLabelTextListener validate}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createLabel(ILabelOwner) Online Documentation}*/
createLabel(owner: ILabelOwner): Promise<ILabel>;
/**
Uses the {@link GraphEditorInputMode#textEditorInputMode text editor} to actually create a new label.
The label is added to the <code>evt</code>'s {@link LabelEditingEventArgs#owner}.
This method is only called when label creation is not {@link LabelEditingEventArgs#cancel forbidden} after querying {@link GraphEditorInputMode#addLabelAddingListener LabelAdding}
and the label owner's {@link IEditLabelHelper}.
@param evt The information for label creation.
@returns A Promise that fulfills with either the newly created label instance or <code>null</code> if the {@link TextEditorInputMode#addEditingCanceledListener editing was canceled} or the text did
not {@link GraphEditorInputMode#addValidateLabelTextListener validate}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createLabelCore(LabelEditingEventArgs) Online Documentation}*/
createLabelCore(evt: LabelEditingEventArgs): Promise<ILabel>;
/**
Factory method that creates the {@link GraphEditorInputMode#labelDropInputMode} lazily the first time the property is accessed.
@returns A plain new instance of the {@link LabelDropInputMode} type, which is initially {@link DropInputMode#enabled disabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createLabelDropInputMode() Online Documentation}*/
createLabelDropInputMode(): LabelDropInputMode;
/**
Factory method for the {@link GraphEditorInputMode#moveInputMode} property.
This method will be called upon first access to the {@link MoveInputMode} property.
@returns a new instance of MoveInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createMoveInputMode() Online Documentation}*/
createMoveInputMode(): MoveInputMode;
/**
Factory method for the {@link GraphEditorInputMode#moveLabelInputMode} property.
This method will be called upon first access to the {@link GraphEditorInputMode#moveLabelInputMode} property.
@returns a new instance of {@link MoveLabelInputMode}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createMoveLabelInputMode() Online Documentation}*/
createMoveLabelInputMode(): MoveLabelInputMode;
/**
Factory method for the {@link GraphEditorInputMode#moveUnselectedInputMode} property.
This method will be called upon first access to the {@link GraphEditorInputMode#moveUnselectedInputMode} property.
@returns a new instance of MoveInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createMoveUnselectedInputMode() Online Documentation}*/
createMoveUnselectedInputMode(): MoveInputMode;
/**
Factory method for the {@link MoveViewportInputMode} property.
This method will be called upon first access to the {@link MoveViewportInputMode} property.
This implementation will set the {@link MoveInputMode#pressedRecognizer} to react on left mouse button pressed with Ctrl pressed.
@returns a new instance of {@link MoveViewportInputMode}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createMoveViewportInputMode() Online Documentation}*/
createMoveViewportInputMode(): MoveViewportInputMode;
/**
Creates a node given a certain click point.
This implementation delegates to the current node creation callback or simply returns <code>null</code> if there is no such callback
or {@link GraphEditorInputMode#allowCreateNode} yields <code>false</code>. Finally, it tries to {@link GraphInputMode#setCurrentItem set the node as the current item}.
@param location The point where the mouse had been clicked.
@returns The newly created node or <code>null</code>.
@see GraphEditorInputMode#onNodeCreated
@see GraphInputMode#setCurrentItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createNode(Point) Online Documentation}*/
createNode(location: Point): INode | null;
/**
Factory method that creates the {@link GraphEditorInputMode#nodeDropInputMode} lazily the first time the property is accessed.
@returns A plain new instance of the {@link NodeDropInputMode} type, which is initially {@link DropInputMode#enabled disabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createNodeDropInputMode() Online Documentation}*/
createNodeDropInputMode(): NodeDropInputMode;
/**
Factory method that creates the {@link GraphEditorInputMode#portDropInputMode} lazily the first time the property is accessed.
@returns A plain new instance of the {@link PortDropInputMode} type, which is initially {@link DropInputMode#enabled disabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createPortDropInputMode() Online Documentation}*/
createPortDropInputMode(): PortDropInputMode;
/**
Factory method for the {@link GraphEditorInputMode#reparentNodeHandler} property.
This method will be called upon first access to the {@link GraphEditorInputMode#reparentNodeHandler} property.
@returns a new instance of {@link GraphEditorInputMode#reparentNodeHandler}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createReparentNodeHandler() Online Documentation}*/
createReparentNodeHandler(): IReparentNodeHandler;
/**
Factory method for the {@link GraphEditorInputMode#textEditorInputMode} property.
This method will be called upon first access to the {@link GraphEditorInputMode#textEditorInputMode} property.
@returns a new instance of TextEditorInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createTextEditorInputMode() Online Documentation}*/
createTextEditorInputMode(): TextEditorInputMode;
/**
* Cuts and inserts the currently selected elements to the clipboard.
* <pre>
* mode.cut()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23cut() Online Documentation}
*/
cut(): void;
/**
This method deletes the currently selected elements.
If the {@link GraphSelection} is non-empty this implementation triggers the {@link GraphEditorInputMode#addDeletingSelectionListener DeletingSelection}
event, possibly a number of {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
events, and a final {@link GraphEditorInputMode#deleteSelection} event. Note that {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
will not be called for items that are removed implicitly, e.g. if the event is triggered for a node, its labels, ports,
and adjacent edges will not be reported separately.
@see GraphEditorInputMode#addDeletingSelectionListener
@see GraphEditorInputMode#addDeletedItemListener
@see GraphEditorInputMode#addDeletedSelectionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23deleteSelection() Online Documentation}*/
deleteSelection(): void;
/**
Actually performs a double-click on the given item.
This will raise the {@link GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked} and {@link GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked}
(depending on the button), and {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
events. If neither event was handled, label editing will be started, if {@link GraphEditorInputMode#allowEditLabelOnDoubleClick} is <code>true</code>
@param item The item that has been double-clicked.
@param evt The original event arguments for the click. Setting its {@link ClickEventArgs#handled} property to <code>true</code> will indicate that the double-click was
handled and events on other items in the same location are no longer raised.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23doubleClick(IModelItem%2CClickEventArgs) Online Documentation}*/
doubleClick(item: IModelItem, evt: ClickEventArgs): void;
/**
Actually performs a double-tap on the given item.
This will raise the {@link GraphInputMode#addItemDoubleTappedListener ItemDoubleTapped} and {@link GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
events. If neither event was handled, label editing will be started, if {@link GraphEditorInputMode#allowEditLabelOnDoubleClick} is <code>true</code>
@param item The item that has been double-tapped.
@param evt The original event arguments for the tap. Setting its {@link TapEventArgs#handled} property to <code>true</code> will indicate that the double-tap was handled
and events on other items in the same location are no longer raised.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23doubleTap(IModelItem%2CTapEventArgs) Online Documentation}*/
doubleTap(item: IModelItem, evt: TapEventArgs): void;
/**
Initiates the {@link GraphEditorInputMode#handleInputMode} to drag the given bend.
@param bend The bend to drag.
@see GraphEditorInputMode#onCreateBendInputModeBendCreated
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23dragBend(IBend) Online Documentation}*/
dragBend(bend: IBend): void;
/**
* Duplicates the selected elements.
* <pre>
* mode.duplicateSelection()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23duplicateSelection() Online Documentation}
*/
duplicateSelection(): void;
/**
Starts editing the given label.
This implementation uses the {@link TextEditorInputMode} to display an editor to edit the label. The text that the
user enters may be {@link GraphEditorInputMode#addValidateLabelTextListener validated} before the label is actually edited.
The label to edit can be a
{@link SimpleLabel}, which will be edited as usual, except that the label text changes will be written directly
back into the instance.
This method will raise the {@link GraphEditorInputMode#addLabelEditingListener LabelEditing}
event and query the {@link IEditLabelHelper} for the label and its owner, but will ignore the result except for the
{@link LabelEditingEventArgs#textEditorInputModeConfigurator} property on the {@link LabelEditingEventArgs}. Thus editing a label via this method cannot be prevented by event
handlers or {@link IEditLabelHelper}s.
@param label The label to edit.
@returns A Promise that fulfills either with the edited label, or with <code>null</code> if {@link TextEditorInputMode#addEditingCanceledListener editing was canceled} or the text did not
{@link GraphEditorInputMode#addValidateLabelTextListener validate}.
@see GraphEditorInputMode#onLabelTextEdited
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23editLabel(ILabel) Online Documentation}*/
editLabel(label: ILabel): Promise<ILabel>;
/**
Core label edit method - opens a new {@link GraphEditorInputMode#textEditorInputMode} input field.
This method is only called if label editing has not been {@link LabelEditingEventArgs#cancel forbidden} after querying {@link GraphEditorInputMode#addLabelEditingListener LabelEditing}
or the label's or the label owner's {@link IEditLabelHelper}.
@param label The label to edit.
@param configurator A callback to configure the {@link GraphEditorInputMode#textEditorInputMode} before the label is edited.
@returns A promise that resolves with either the label's text or a <code>null</code> if the text edit was canceled.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23editLabelCore(ILabel%2Cfunction(IInputModeContext%2CTextEditorInputMode%2CILabel)%3Avoid) Online Documentation}*/
editLabelCore(label: ILabel, configurator: ((context: IInputModeContext, mode: TextEditorInputMode, label: ILabel) => void) | null): Promise<string>;
/**
Starts label editing by executing {@link ICommand#EDIT_LABEL }.
The method should return whether the request was satisfied.
@param item The item whose label or the label itself that should be edited upon the double click gesture.
@returns <code>true</code> iff the request was handled.
@see GraphInputMode#doubleClick
@see GraphEditorInputMode#allowEditLabelOnDoubleClick
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23editLabelOnDoubleClick(IModelItem) Online Documentation}*/
editLabelOnDoubleClick(item: IModelItem): boolean;
/**
Creates a new group for all of the currently selected elements.
This method will also clear the selection and select the newly created group node.
The default shortcut for this is Ctrl+G.
@returns The newly created group node or <code>null</code>.
@see ICommand#GROUP_SELECTION
@see GraphInputMode#clearSelection
@see GraphInputMode#setSelected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23groupSelection() Online Documentation}*/
groupSelection(): INode | null;
/**
Installs this mode into the provided context.
@param context The context to install this mode in and retrieve the graph and selection instance from.
@param controller The {@link ConcurrencyController} for this mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
* Lowers the visual representations of all {@link GraphComponent#selection selected} {@link IModelItem}s below their current predecessors.
* <pre>
* mode.lowerSelection()
* </pre>
*
* @see ICommand#LOWER
* @see GraphModelManager#lower
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23lowerSelection() Online Documentation}
*/
lowerSelection(): void;
/**
Invoked if the shift+F2 key is pressed and {@link GraphEditorInputMode#allowAddLabel} is set to <code>true</code>.
This method determines the label owner to add to and delegates to {@link GraphEditorInputMode#createLabelCore}.
This method also raises a {@link GraphEditorInputMode#addLabelAddingListener LabelAdding}
event and queries potential {@link ILabelOwner}'s {@link IEditLabelHelper}'s {@link IEditLabelHelper#onLabelAdding}
methods.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onAddLabel() Online Documentation}*/
onAddLabel(): boolean;
/**
Called as soon as {@link CreateBendInputMode} created a new bend.
This method selects the bend and starts the {@link GraphEditorInputMode#handleInputMode} using the handle for the newly created bend. The bend is created using the
hit edge's {@link IBendCreator} implementation, as obtained from the edge's {@link ILookup#lookup} and calls
{@link GraphEditorInputMode#dragBend}. If the subsequent drag is canceled by the user, this call will remove the
bend again.
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onCreateBendInputModeBendCreated(Object%2CItemEventArgs%3CIBend%3E) Online Documentation}*/
onCreateBendInputModeBendCreated(sender: any, evt: ItemEventArgs<IBend>): void;
/**
* Called when the {@link GraphEditorInputMode#createBendInputMode} property value changes and after initialization of the property.
* <br><b>Disabling the CreateBendInputMode on its parent input mode</b>
* <pre>
* mode.createBendInputMode.enabled = false
* </pre>
*
* @param oldMode the old value, which may be <code>null</code> the first time
* @param newMode the new value
* @protected
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onCreateBendInputModeChanged(CreateBendInputMode%2CCreateBendInputMode) Online Documentation}
*/
onCreateBendInputModeChanged(oldMode: CreateBendInputMode | null, newMode: CreateBendInputMode): void;
/**
Called when the {@link GraphEditorInputMode#createEdgeInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onCreateEdgeInputModeChanged(CreateEdgeInputMode%2CCreateEdgeInputMode) Online Documentation}*/
onCreateEdgeInputModeChanged(oldMode: CreateEdgeInputMode | null, newMode: CreateEdgeInputMode): void;
/**
Raises the {@link GraphEditorInputMode#addDeletedItemListener DeletedItem} event.
@param evt The instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onDeletedItem(ItemEventArgs%3CIModelItem%3E) Online Documentation}*/
onDeletedItem(evt: ItemEventArgs<IModelItem>): void;
/**
Raises the {@link GraphEditorInputMode#addDeletedSelectionListener DeletedSelection} event.
@param evt The {@link SelectionEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onDeletedSelection(SelectionEventArgs%3CIModelItem%3E) Online Documentation}*/
onDeletedSelection(evt: SelectionEventArgs<IModelItem>): void;
/**
Raises the {@link GraphEditorInputMode#addDeletingSelectionListener DeletingSelection} event.
@param evt The {@link SelectionEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onDeletingSelection(SelectionEventArgs%3CIModelItem%3E) Online Documentation}*/
onDeletingSelection(evt: SelectionEventArgs<IModelItem>): void;
/**
Invoked after an edge's source and/or target ports have been changed as the result of an input gesture.
Dispatches the {@link GraphEditorInputMode#addEdgePortsChangedListener EdgePortsChanged} event.
@param evt The {@link EdgeEventArgs} for the
{@link GraphEditorInputMode#addEdgePortsChangedListener EdgePortsChanged}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onEdgePortsChanged(EdgeEventArgs) Online Documentation}*/
onEdgePortsChanged(evt: EdgeEventArgs): void;
/**
Invoked if the F2 key is pressed and {@link GraphEditorInputMode#allowEditLabel} is set to <code>true</code>.
This method determines the label to edit and delegates to either {@link GraphEditorInputMode#editLabel} or {@link GraphEditorInputMode#createLabel}
if no label could be found.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onEditLabel() Online Documentation}*/
onEditLabel(): boolean;
/**
Called when the {@link GraphEditorInputMode#graphComponent} property changes.
@param oldGraphComponent The old control.
@param newGraphComponent The new control.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onGraphComponentChanged(GraphComponent%2CGraphComponent) Online Documentation}*/
onGraphComponentChanged(oldGraphComponent: GraphComponent | null, newGraphComponent: GraphComponent | null): void;
/**
Called when the {@link GraphSelection} property changes.
This will update the internal state of the modes.
@param oldSelection The old selection instance.
@param newSelection The new selection instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onGraphSelectionChanged(IGraphSelection%2CIGraphSelection) Online Documentation}*/
onGraphSelectionChanged(oldSelection: IGraphSelection | null, newSelection: IGraphSelection): void;
/**
Called when the {@link GraphEditorInputMode#handleInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onHandleInputModeChanged(HandleInputMode%2CHandleInputMode) Online Documentation}*/
onHandleInputModeChanged(oldMode: HandleInputMode | null, newMode: HandleInputMode): void;
/**
Raises the {@link GraphEditorInputMode#addLabelAddedListener LabelAdded} event.
@param evt The {@link ItemEventArgs} instance that contains the {@link ILabel} that has been added.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelAdded(LabelEventArgs) Online Documentation}*/
onLabelAdded(evt: LabelEventArgs): void;
/**
Raises the {@link GraphEditorInputMode#addLabelAddingListener LabelAdding} event.
Invoking the event handlers stops once one event marks the <code>evt</code> as handled.
@param evt The event arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelAdding(LabelEditingEventArgs) Online Documentation}*/
onLabelAdding(evt: LabelEditingEventArgs): void;
/**
Called when the {@link GraphEditorInputMode#labelDropInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelDropInputModeChanged(LabelDropInputMode%2CLabelDropInputMode) Online Documentation}*/
onLabelDropInputModeChanged(oldMode: LabelDropInputMode | null, newMode: LabelDropInputMode): void;
/**
Called when the {@link GraphEditorInputMode#labelDropInputMode}'s {@link ItemDropInputMode#addItemCreatedListener ItemCreated} event is triggered.
This implementation {@link GraphEditorInputMode#adjustContentRect adjusts the content rect}, triggers the {@link GraphEditorInputMode#addLabelAddedListener LabelAdded}
event and tries to set the newly created node as the {@link GraphInputMode#setCurrentItem current item}.
@param sender The sender.
@param evt The event argument instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelDropInputModeLabelCreated(Object%2CItemEventArgs%3CILabel%3E) Online Documentation}*/
onLabelDropInputModeLabelCreated(sender: any, evt: ItemEventArgs<ILabel>): void;
/**
Raises the {@link GraphEditorInputMode#addLabelEditingListener LabelEditing} event.
Invoking the event handlers stops once one event marks the <code>evt</code> as handled.
@param evt The event arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelEditing(LabelEditingEventArgs) Online Documentation}*/
onLabelEditing(evt: LabelEditingEventArgs): void;
/**
Called when the {@link GraphEditorInputMode#labelSnapContext} property context changed.
@param oldContext The old snap context.
@param newContext The new snap context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelSnapContextChanged(SnapContext%2CSnapContext) Online Documentation}*/
onLabelSnapContextChanged(oldContext: SnapContext | null, newContext: SnapContext | null): void;
/**
Raises the {@link GraphEditorInputMode#addLabelTextChangedListener LabelTextChanged} event.
@param evt The {@link ItemEventArgs} instance that contains the {@link ILabel} that has changed the text.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelTextChanged(LabelEventArgs) Online Documentation}*/
onLabelTextChanged(evt: LabelEventArgs): void;
/**
Called when the text of a label has been {@link GraphEditorInputMode#editLabel edited}.
This method {@link GraphEditorInputMode#addValidateLabelTextListener validates the label text} and if successful calls {@link GraphEditorInputMode#setLabelText}.
@param label The label that was edited.
@param text The new text.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelTextEdited(ILabel%2Cstring) Online Documentation}*/
onLabelTextEdited(label: ILabel, text: string): void;
/**
Raises the {@link GraphEditorInputMode#addLabelTextEditingCanceledListener LabelTextEditingCanceled}
event.
@param evt The {@link LabelEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelTextEditingCanceled(LabelEventArgs) Online Documentation}*/
onLabelTextEditingCanceled(evt: LabelEventArgs): void;
/**
Raises the {@link GraphEditorInputMode#addLabelTextEditingStartedListener LabelTextEditingStarted}
event.
@param evt The {@link LabelEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onLabelTextEditingStarted(LabelEventArgs) Online Documentation}*/
onLabelTextEditingStarted(evt: LabelEventArgs): void;
/**
Called when the {@link GraphEditorInputMode#moveInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onMoveInputModeChanged(MoveInputMode%2CMoveInputMode) Online Documentation}*/
onMoveInputModeChanged(oldMode: MoveInputMode | null, newMode: MoveInputMode): void;
/**
Called when the {@link GraphEditorInputMode#moveLabelInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onMoveLabelInputModeChanged(MoveLabelInputMode%2CMoveLabelInputMode) Online Documentation}*/
onMoveLabelInputModeChanged(oldMode: MoveLabelInputMode | null, newMode: MoveLabelInputMode): void;
/**
Called when the {@link GraphEditorInputMode#moveUnselectedInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onMoveUnselectedInputModeChanged(MoveInputMode%2CMoveInputMode) Online Documentation}*/
onMoveUnselectedInputModeChanged(oldMode: MoveInputMode | null, newMode: MoveInputMode): void;
/**
Raises the {@link GraphEditorInputMode#addNodeCreatedListener NodeCreated} event.
Note that if {@link NodeDropInputMode#allowFolderNodeAsParent} is set to <code>true</code>, the reported node can actually be part of the {@link IFoldingView master graph}.
@param evt The {@link ItemEventArgs} instance containing the created node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onNodeCreated(ItemEventArgs%3CINode%3E) Online Documentation}*/
onNodeCreated(evt: ItemEventArgs<INode>): void;
/**
Called when the {@link GraphEditorInputMode#nodeDropInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onNodeDropInputModeChanged(NodeDropInputMode%2CNodeDropInputMode) Online Documentation}*/
onNodeDropInputModeChanged(oldMode: NodeDropInputMode | null, newMode: NodeDropInputMode): void;
/**
Called when the {@link GraphEditorInputMode#nodeDropInputMode}'s {@link ItemDropInputMode#addItemCreatedListener ItemCreated} event is triggered.
This implementation {@link GraphEditorInputMode#adjustContentRect adjusts the content rect}, triggers the {@link GraphEditorInputMode#addNodeCreatedListener NodeCreated}
event and tries to set the newly created node as the {@link GraphInputMode#setCurrentItem current item}.
@param sender The sender.
@param evt The event argument instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onNodeDropInputModeNodeCreated(Object%2CItemEventArgs%3CINode%3E) Online Documentation}*/
onNodeDropInputModeNodeCreated(sender: any, evt: ItemEventArgs<INode>): void;
/**
Raises the {@link GraphEditorInputMode#addNodeReparentedListener NodeReparented} event.
@param evt The {@link NodeEventArgs} instance containing the event data.
@see GraphEditorInputMode#reparentNodeHandler
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onNodeReparented(NodeEventArgs) Online Documentation}*/
onNodeReparented(evt: NodeEventArgs): void;
/**
Raises the {@link GraphEditorInputMode#addPortAddedListener PortAdded} event.
@param evt The {@link ItemEventArgs} instance that contains the {@link IPort} that has been added.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onPortAdded(PortEventArgs) Online Documentation}*/
onPortAdded(evt: PortEventArgs): void;
/**
Called when the {@link GraphEditorInputMode#portDropInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onPortDropInputModeChanged(PortDropInputMode%2CPortDropInputMode) Online Documentation}*/
onPortDropInputModeChanged(oldMode: PortDropInputMode | null, newMode: PortDropInputMode): void;
/**
Called when the {@link GraphEditorInputMode#portDropInputMode}'s {@link ItemDropInputMode#addItemCreatedListener ItemCreated} event is triggered.
This implementation {@link GraphEditorInputMode#adjustContentRect adjusts the content rect}, triggers the {@link GraphEditorInputMode#addPortAddedListener PortAdded}
event and tries to set the newly created node as the {@link GraphInputMode#setCurrentItem current item}.
@param sender The sender.
@param evt The event argument instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onPortDropInputModePortCreated(Object%2CItemEventArgs%3CIPort%3E) Online Documentation}*/
onPortDropInputModePortCreated(sender: any, evt: ItemEventArgs<IPort>): void;
/**
Called when the {@link GraphEditorInputMode#reparentNodeHandler} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onReparentNodeHandlerChanged(IReparentNodeHandler%2CIReparentNodeHandler) Online Documentation}*/
onReparentNodeHandlerChanged(oldMode: IReparentNodeHandler | null, newMode: IReparentNodeHandler): void;
/**
Called when the {@link GraphEditorInputMode#snapContext} property context changed.
@param oldContext The old snap context.
@param newContext The new snap context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onSnapContextChanged(SnapContext%2CSnapContext) Online Documentation}*/
onSnapContextChanged(oldContext: SnapContext | null, newContext: SnapContext | null): void;
/**
Called when the {@link GraphEditorInputMode#textEditorInputMode} property value changes and after initialization of the property.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onTextEditorInputModeChanged(TextEditorInputMode%2CTextEditorInputMode) Online Documentation}*/
onTextEditorInputModeChanged(oldMode: TextEditorInputMode | null, newMode: TextEditorInputMode): void;
/**
Raises the {@link GraphEditorInputMode#addValidateLabelTextListener ValidateLabelText} event.
@param evt The {@link LabelTextValidatingEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23onValidateLabelText(LabelTextValidatingEventArgs) Online Documentation}*/
onValidateLabelText(evt: LabelTextValidatingEventArgs): void;
/**
* Pastes the current clipboard contents.
* <pre>
* mode.paste()
* </pre>
*
* @see GraphEditorInputMode#pasteAtLocation
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23paste() Online Documentation}
*/
paste(): void;
/**
* Pastes the current clipboard contents at the given location.
* The location is at the center of the bounds of the items that are pasted. This method reconfigures {@link GraphClipboard#pasteDelta} and calls
* {@link GraphEditorInputMode#paste}.
*
* <pre>
* mode.pasteAtLocation(mode.graphComponent.lastEventLocation)
* </pre>
*
* @param location The location at which the center of the clipboard's contents should be positioned.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23pasteAtLocation(Point) Online Documentation}
*/
pasteAtLocation(location: Point): void;
/**
* Raises the visual representations of all {@link GraphComponent#selection selected} {@link IModelItem}s above their current successors.
* <pre>
* mode.raiseSelection()
* </pre>
*
* @see ICommand#RAISE
* @see GraphModelManager#raise
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23raiseSelection() Online Documentation}
*/
raiseSelection(): void;
/**
Redoes the last undone operation.
Delegates to the {@link IGraph}'s {@link UndoEngine}'s {@link UndoEngine#redo} method.
This method will return without doing anything if undo has not been enabled on the {@link GraphEditorInputMode#graphComponent}'s {@link GraphComponent#graph} or if the {@link UndoEngine}
{@link UndoEngine#canRedo cannot redo} operations.
@see GraphEditorInputMode#undo
@see UndoEngine#redo
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23redo() Online Documentation}*/
redo(): void;
/**
Re-evaluate all visible handles.
This causes a query to all implementations of {@link IHandle}, {@link IHandleProvider}, and {@link IReshapeHandleProvider}
from the {@link ILookup lookup}see> of selected items.
This method is called when the {@link GraphEditorInputMode#showHandleItems} property is changed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23requeryHandles() Online Documentation}*/
requeryHandles(): void;
/**
Reverses the given edges.
The edge will only be reversed if {@link GraphEditorInputMode#allowReverseEdge} is set to <code>true</code> and {@link GraphEditorInputMode#shouldReverseEdge} returns <code>true</code>
for the edge. A {@link GraphEditorInputMode#addEdgePortsChangedListener EdgePortsChanged} event
will be dispatched after the edge has been reversed.
@param edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23reverseEdge(IEdge) Online Documentation}*/
reverseEdge(edge: IEdge): void;
/**
Reverses the given edges.
The edges will only be reversed if {@link GraphEditorInputMode#allowReverseEdge} is set to <code>true</code> and {@link GraphEditorInputMode#shouldReverseEdge} returns <code>true</code>
for the edge.
For each edge a {@link GraphEditorInputMode#addEdgePortsChangedListener EdgePortsChanged} event
will be dispatched.
@param edges
@see ICommand#REVERSE_EDGE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23reverseEdges(IEnumerable%3CIEdge%3E) Online Documentation}*/
reverseEdges(edges: IEnumerable<IEdge>): void;
/**
Reverses the selected edges.
This implementation invokes {@link GraphEditorInputMode#reverseEdges} providing the {@link GraphSelection selected edges}.
The edges will only be reversed if {@link GraphEditorInputMode#shouldReverseEdge} returns <code>true</code> for each edge.
For each edge an {@link GraphEditorInputMode#addEdgePortsChangedListener EdgePortsChanged} event
will be dispatched.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23reverseSelectedEdges() Online Documentation}*/
reverseSelectedEdges(): void;
/**
* Lowers the visual representations of all {@link GraphComponent#selection selected} {@link IModelItem}s to the back.
* <pre>
* mode.selectionToBack()
* </pre>
*
* @see ICommand#TO_BACK
* @see GraphModelManager#toBack
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23selectionToBack() Online Documentation}
*/
selectionToBack(): void;
/**
* Raises the visual representations of all {@link GraphComponent#selection selected} {@link IModelItem}s to the front.
* <pre>
* mode.selectionToFront()
* </pre>
*
* @see ICommand#TO_FRONT
* @see GraphModelManager#toFront
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23selectionToFront() Online Documentation}
*/
selectionToFront(): void;
/**
Selects the node and possible bends of selfloop edges.
This method delegates to {@link GraphInputMode#setSelected} for the node and all adjacent self loop bends.
@param node The node to select.
@see GraphEditorInputMode#autoSelectSelfloopBends
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23selectNodeAndSelfloopBends(INode) Online Documentation}*/
selectNodeAndSelfloopBends(node: INode): void;
/**
Sets the label's text after it has been {@link GraphEditorInputMode#editLabel edited}.
Depending on {@link GraphEditorInputMode#autoRemoveEmptyLabels} it will either set the label's text or remove it from the graph. Also this method will query the label's {@link ILabel#owner}
for an {@link INodeSizeConstraintProvider} if the owner is an {@link INode} and will make sure that the size
constraints are still kept if the {@link ILabel#preferredSize} changes due to the edit by calling
{@link GraphEditorInputMode#adjustToSizeConstraints}.
@param label The label to set the text or remove.
@param text The new text.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23setLabelText(ILabel%2Cstring) Online Documentation}*/
setLabelText(label: ILabel, text: string): void;
/**
Updates the bounds of the node.
This method differs from a simple call to {@link IGraph#setNodeLayout} in a number of ways:
<ul>
<li>An undo unit is enqueued.
<li>The node is reshaped interactively using the {@link IReshapeHandler} implementation found in its lookup. This
enables correctly adjusting e.g. affected orthogonal edges and parent group nodes.
<li>{@link GraphEditorInputMode#adjustContentRect} is called.
</ul>
@param node The node to reshape.
@param layout The new node layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23setNodeLayout(INode%2CRect) Online Documentation}*/
setNodeLayout(node: INode, layout: Rect): void;
/**
Predicate that determines whether a label may be added interactively to <code>item</code>
This is overridden if the item has a {@link IEditLabelHelper} that returns <code>false</code> for its {@link IEditLabelHelper#onLabelAdding}
method.
@param item The item to query
@returns <code>true</code> iff a label may be added
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23shouldAddLabel(IModelItem) Online Documentation}*/
shouldAddLabel(item: IModelItem): boolean;
/**
Specifies whether a node should be created by
{@link ClickInputMode#addClickedListener Clicked}.
This implementation returns false when {@link ContextMenuInputMode#swallowCloseClick} is <code>true</code> and the click has closed a context menu.
@param context The input mode context.
@param clickPoint The location of the click.
@returns Whether {@link GraphEditorInputMode#clickCreateNode} should be called to create a node.
@see GraphEditorInputMode#clickCreateNode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23shouldClickCreateNode(IInputModeContext%2CPoint) Online Documentation}*/
shouldClickCreateNode(context: IInputModeContext, clickPoint: Point): boolean;
/**
Called to determine whether the given item should be deleted during {@link GraphEditorInputMode#deleteSelection}.
This implementation uses and honors the {@link GraphEditorInputMode#deletableItems} and {@link GraphEditorInputMode#deletablePredicate} properties.
@param item The item.
@returns Whether to delete that item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23shouldDelete(IModelItem) Online Documentation}*/
shouldDelete(item: IModelItem): boolean;
/**
Called to determine whether the label or the labels of the provided item should be edited in response to {@link GraphEditorInputMode#onEditLabel}
or {@link GraphEditorInputMode#onAddLabel}.
@param item The item.
@returns Whether to edit the label or the labels for that item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23shouldEditLabel(IModelItem) Online Documentation}*/
shouldEditLabel(item: IModelItem): boolean;
/**
Determines whether or not moving the item is allowed.
This predicate determines whether an item can be moved by <b>any</b> appropriate input mode, e.g by default it also evaluates to <code>true</code>
for {@link ILabel}s which are moved by {@link GraphEditorInputMode#moveLabelInputMode} instead of {@link GraphEditorInputMode#moveInputMode}.
This implementation returns the result of the {@link GraphEditorInputMode#movableItems} property for the given item.
@param item The item to check.
@returns Whether or not moving the item is allowed.
@see GraphEditorInputMode#movableItems
@see GraphEditorInputMode#moveInputMode
@see GraphEditorInputMode#moveLabelInputMode
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23shouldMove(IModelItem) Online Documentation}*/
shouldMove(item: IModelItem): boolean;
/**
Called to determine whether the given edge should be reversed by {@link GraphEditorInputMode#reverseEdge},
{@link GraphEditorInputMode#reverseEdges}, or {@link GraphEditorInputMode#reverseEdges}.
This implementation returns the value of {@link GraphEditorInputMode#allowReverseEdge}.
@param edge The edge which should be reversed.
@returns <code>true</code> if the edge should be reversed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23shouldReverseEdge(IEdge) Online Documentation}*/
shouldReverseEdge(edge: IEdge): boolean;
/**
Can be overridden to determine which items should be selected after {@link GraphEditorInputMode#paste} or
{@link GraphEditorInputMode#duplicateSelection}.
This implementation returns <code>true</code> for the {@link GraphEditorInputMode#pasteSelectableItems} items.
@param item The item to check.
@returns Whether to select the item after {@link GraphEditorInputMode#paste} or
{@link GraphEditorInputMode#duplicateSelection}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23shouldSelectAfterPaste(IModelItem) Online Documentation}*/
shouldSelectAfterPaste(item: IModelItem): boolean;
/**
Determines whether to show the {@link GraphEditorInputMode#handleInputMode handles} for the given item.
@param item The item to check
@returns Whether to show the item based on the setting of the corresponding {@link GraphEditorInputMode#showHandleItems}, property.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23shouldShowHandles(IModelItem) Online Documentation}*/
shouldShowHandles(item: IModelItem): boolean;
/**
Snaps the node to the grid using the {@link IGridConstraintProvider} for {@link INode}s queried from the given context.
This method is called by this instance whenever new nodes are created using the UI.
@param context
@param node
@see MultiplexingInputMode#createChildInputModeContext
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23snapToGrid(IInputModeContext%2CINode) Online Documentation}*/
snapToGrid(context: IInputModeContext, node: INode): void;
/**
Undoes the last operation.
Delegates to the {@link IGraph}'s {@link UndoEngine}'s {@link UndoEngine#undo} method.
This method will return without doing anything if undo has not been enabled on the {@link GraphEditorInputMode#graphComponent}'s {@link GraphComponent#graph} or if the {@link UndoEngine}
{@link UndoEngine#canUndo cannot undo} operations.
@see GraphEditorInputMode#redo
@see UndoEngine#undo
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23undo() Online Documentation}*/
undo(): void;
/**
Makes each of the currently selected nodes a direct child of the root group.
The default shortcut for this is Ctrl+U.
@see ICommand#UNGROUP_SELECTION
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23ungroupSelection() Online Documentation}*/
ungroupSelection(): void;
/**
An {@link IPositionHandler} combining all position handlers of {@link GraphSelection selected} model items that are {@link GraphEditorInputMode#shouldMove movable}.
This handler retrieves the {@link GraphEditorInputMode} from the {@link ILookup#lookup} in
{@link IDragHandler#initializeDrag}. All items in its {@link GraphSelection selection} that are {@link GraphEditorInputMode#shouldMove movable} are queried for their {@link IPositionHandler}
or {@link IHandle} and these drag handlers are combined afterwards.
This handler is the default {@link MoveInputMode#positionHandler} of {@link GraphEditorInputMode#moveInputMode} and any {@link MoveInputMode} using this position handler has to be a child mode
of an {@link GraphEditorInputMode}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23SELECTED_MOVABLES_POSITION_HANDLER Online Documentation}*/
static SELECTED_MOVABLES_POSITION_HANDLER: IPositionHandler;
/**
An {@link IHitTestable} that tests if any {@link GraphSelection selected} model item that is {@link GraphEditorInputMode#shouldMove movable} is hit.
This testable retrieves the {@link GraphEditorInputMode} from the {@link ILookup#lookup} in
{@link IHitTestable#isHit}. All items in its {@link GraphSelection selection} that are {@link GraphEditorInputMode#shouldMove movable} and provide an {@link IPositionHandler}
are queried for their {@link IHitTestable}
This testable is the default {@link MoveInputMode#hitTestable} of {@link GraphEditorInputMode#moveInputMode} and any {@link MoveInputMode} using this testable has to be a child mode of an
{@link GraphEditorInputMode}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23SELECTED_MOVABLES_HIT_TESTABLE Online Documentation}*/
static SELECTED_MOVABLES_HIT_TESTABLE: IHitTestable;
/**
An event handler setting the {@link IPositionHandler} of the {@link GraphEditorInputMode#shouldMove movable} model item that is hit at the {@link QueryPositionHandlerEventArgs#queryLocation query location}.
This handler retrieves the {@link GraphEditorInputMode} from the {@link ILookup#lookup} and tests if any item
{@link IHitTester hit} at the {@link QueryPositionHandlerEventArgs#queryLocation query location} is {@link GraphEditorInputMode#shouldMove movable} and provides an {@link IPositionHandler}. If such an {@link IPositionHandler}
is found, it is set as {@link QueryPositionHandlerEventArgs#positionHandler}.
This handler is added per default to the {@link MoveInputMode#addQueryPositionHandlerListener QueryPositionHandler}
event of {@link GraphEditorInputMode#moveUnselectedInputMode}. Any {@link MoveInputMode} using this query handler has to be a child mode of an
{@link GraphEditorInputMode}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23UNSELECTED_MOVABLES_QUERY_POSITION_HANDLER Online Documentation}*/
static UNSELECTED_MOVABLES_QUERY_POSITION_HANDLER: (sender: any | null, evt: QueryPositionHandlerEventArgs | null) => void;
/**
An {@link IHitTestable} that tests if any model item that is {@link GraphEditorInputMode#shouldMove movable} is hit.
This testable retrieves the {@link GraphEditorInputMode} from the {@link ILookup#lookup} in
{@link IHitTestable#isHit}. and tests if any {@link IHitTester hit} items that is {@link GraphEditorInputMode#shouldMove movable} provides an {@link IPositionHandler}.
This testable is the default {@link MoveInputMode#hitTestable} of {@link GraphEditorInputMode#moveUnselectedInputMode}. Any {@link MoveInputMode} using this testable has to be a child mode of an
{@link GraphEditorInputMode}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23UNSELECTED_MOVABLES_HIT_TESTABLE Online Documentation}*/
static UNSELECTED_MOVABLES_HIT_TESTABLE: IHitTestable;
/**
* Gets or sets the orthogonal edge editing context.
* Orthogonal edge editing is enabled by setting an instance of {@link OrthogonalEdgeEditingContext}.
*
* The default value is <code>null</code>.
*
* <br><b>Enabling orthogonal edge editing</b>
* <pre>
* mode.orthogonalEdgeEditingContext = new OrthogonalEdgeEditingContext()
* </pre>
*
* @default null
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23orthogonalEdgeEditingContext Online Documentation}
*/
orthogonalEdgeEditingContext: OrthogonalEdgeEditingContext | null;
/**
Gets or sets the policy that controls whether removing bends from an orthogonal edge will keep the edge orthogonal.
By default this feature is enabled automatically if an {@link OrthogonalEdgeEditingContext} is set. (Default is
{@link OrthogonalEdgeEditingPolicy#AUTO}).
@see GraphEditorInputMode#orthogonalEdgeEditingContext
@default OrthogonalEdgeEditingPolicy.AUTO
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23orthogonalBendRemoval Online Documentation}*/
orthogonalBendRemoval: OrthogonalEdgeEditingPolicy;
/**
Gets or sets the {@link IReparentNodeHandler} that is used for moving nodes out of group nodes or into other group
nodes.
This setting has no effect if reparenting nodes is generally disallowed by setting {@link GraphEditorInputMode#allowReparentNodes} to <code>false</code>.
Child input modes will use this {@link IReparentNodeHandler} as well, since it is added to
{@link GraphEditorInputMode#childInputModeContextLookup}.
If not already initialized, the factory method {@link GraphEditorInputMode#createReparentNodeHandler} will be
called on first access. Upon changes, the {@link GraphEditorInputMode#onReparentNodeHandlerChanged} method will be
called.
@see GraphEditorInputMode#allowReparentNodes
@see ReparentNodeHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23reparentNodeHandler Online Documentation}*/
reparentNodeHandler: IReparentNodeHandler;
/**
Gets or sets the hitTester property.
The enumerator will be queried for hit tests if the user clicks on the canvas.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23hitTester Online Documentation}*/
hitTester: IHitTester<IModelItem>;
/**
* Gets or sets which types of items should be selected after {@link GraphEditorInputMode#paste} or
* {@link GraphEditorInputMode#duplicateSelection}.
* The default is {@link GraphItemTypes#ALL}.
*
* <pre>
* mode.pasteSelectableItems = GraphItemTypes.NODE | GraphItemTypes.BEND
* </pre>
*
* @see GraphEditorInputMode#shouldSelectAfterPaste
* @see GraphEditorInputMode#smartPasteSelection
* @see GraphInputMode#setSelected
* @default GraphItemTypes.ALL
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23pasteSelectableItems Online Documentation}
*/
pasteSelectableItems: GraphItemTypes;
/**
* Gets or sets whether nodes, edges, labels, and ports should only be selected after {@link GraphEditorInputMode#paste}
* or {@link GraphEditorInputMode#duplicateSelection} if they were selected when they were initially copied into the
* clipboard.
* Disabling this feature will select all {@link GraphEditorInputMode#pasteSelectableItems} after a paste or duplicate operation.
*
* The default is <code>true</code>.
*
* <pre>
* mode.smartPasteSelection = false
* </pre>
*
* @see GraphEditorInputMode#shouldSelectAfterPaste
* @see GraphEditorInputMode#pasteSelectableItems
* @see GraphInputMode#setSelected
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23smartPasteSelection Online Documentation}
*/
smartPasteSelection: boolean;
/**
* Gets or sets which types of items should be movable using the {@link GraphEditorInputMode#moveInputMode} or {@link GraphEditorInputMode#moveLabelInputMode} respectively in case of labels.
* The default is {@link GraphItemTypes#ALL}.
*
* <br><b>Defining movable items as bitwise combination of the GraphItemTypes</b>
* <pre>
* mode.movableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE
* </pre>
*
* @see GraphEditorInputMode#shouldMove
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23movableItems Online Documentation}
*/
movableItems: GraphItemTypes;
/**
* Gets or sets which types of items may be deleted using the {@link GraphEditorInputMode#deleteSelection} action.
* The default is {@link GraphItemTypes#ALL}.
*
* <br><b>Defining items which can be deleted as bitwise combination of the GraphItemTypes</b>
* <pre>
* mode.deletableItems = GraphItemTypes.NODE | GraphItemTypes.EDGE
* </pre>
*
* @see GraphEditorInputMode#shouldDelete
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23deletableItems Online Documentation}
*/
deletableItems: GraphItemTypes;
/**
* Gets or sets a predicate that is queried to determine whether an {@link IModelItem} can be deleted or not.
* This allows to further restrict which items are deletable, compared to {@link GraphEditorInputMode#deletableItems}. If this property is set to <code>null</code> the effect is
* the same as a predicate that always returns <code>true</code>.
*
* In the following example items are only deletable if their tag is the string "I am deletable":
*
* <pre>
* mode.selectablePredicate = item => item.tag === 'I am selectable'
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23deletablePredicate Online Documentation}
*/
deletablePredicate: ((obj: IModelItem) => boolean) | null;
/**
* Gets or sets which types of items may have their {@link GraphEditorInputMode#onEditLabel labels edited}.
* The default is {@link GraphItemTypes#LABEL_OWNER}|{@link GraphItemTypes#LABEL}, which allows for editing the
* labels of {@link ILabelOwner}s, and existing {@link ILabel}s.
*
* <br><b>Configure label editing</b>
* <pre>
* // forbid adding new labels
* mode.allowAddLabel = false
* // allow editing existing labels
* mode.allowEditLabel = true
* // start label editing on double click
* mode.allowEditLabelOnDoubleClick = true
* // only support editing node labels
* mode.labelEditableItems = GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL
* </pre>
*
* @see GraphEditorInputMode#shouldEditLabel
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23labelEditableItems Online Documentation}
*/
labelEditableItems: GraphItemTypes;
/**
* Gets or sets the {@link HandleInputMode} as child input mode.
* If no value has been set explicitly, the factory method {@link GraphEditorInputMode#createHandleInputMode} will be
* called upon first access. Upon change the {@link GraphEditorInputMode#onHandleInputModeChanged} method will be
* called.
*
* By default this input mode has a priority of <code>0</code>.
*
* <br><b>Disabling the HandleInputMode on its parent input mode</b>
* <pre>
* mode.handleInputMode.enabled = false
* </pre>
*
* If one needs to execute some code after some items have been moved by the {@link HandleInputMode} he can register a
* handler to its {@link HandleInputMode#addDragFinishedListener DragFinished} event. The moved items
* can be retrieved from the {@link HandleInputMode#affectedItems} property:
*
* <pre>
* mode.handleInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.handleInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23handleInputMode Online Documentation}
*/
handleInputMode: HandleInputMode;
/**
* Gets or sets which types of items should have their {@link IHandle}s shown.
* The default is {@link GraphItemTypes#ALL}.
*
* <br><b>Defining items which provide handles as bitwise combination of the GraphItemTypes</b>
* <pre>
* mode.showHandleItems = GraphItemTypes.NODE | GraphItemTypes.EDGE
* </pre>
*
* @see GraphEditorInputMode#shouldShowHandles
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23showHandleItems Online Documentation}
*/
showHandleItems: GraphItemTypes;
/**
Gets or sets a value indicating whether to use the {@link GraphComponent#currentItem} as a fallback for the commands if no item is provided in the
parameter and the current selection is empty.
<code>true</code> if the current item should be used as a fallback, <code>false</code> otherwise.
This applies to the following commands:
<ul>
<li>{@link ICommand#ADD_LABEL}
<li>{@link ICommand#EDIT_LABEL}
<li>{@link ICommand#ADJUST_GROUP_NODE_SIZE}
<li>{@link ICommand#RAISE}
<li>{@link ICommand#LOWER}
<li>{@link ICommand#TO_FRONT}
<li>{@link ICommand#TO_BACK}
</ul>
Default is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23useCurrentItemForCommands Online Documentation}*/
useCurrentItemForCommands: boolean;
/**
Adds the given listener for the <code>ElementsPasted</code> event that occurs when the {@link GraphEditorInputMode#paste}
operation finished successfully.
@param listener The listener to add.
@see GraphEditorInputMode#removeElementsPastedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23ElementsPasted Online Documentation}*/
addElementsPastedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ElementsPasted</code> event that occurs when the {@link GraphEditorInputMode#paste}
operation finished successfully.
@param listener The listener to remove.
@see GraphEditorInputMode#addElementsPastedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23ElementsPasted Online Documentation}*/
removeElementsPastedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Adds the given listener for the <code>ElementsDuplicated</code> event that occurs when the {@link GraphEditorInputMode#duplicateSelection}
operation finished successfully.
@param listener The listener to add.
@see GraphEditorInputMode#removeElementsDuplicatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23ElementsDuplicated Online Documentation}*/
addElementsDuplicatedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ElementsDuplicated</code> event that occurs when the {@link GraphEditorInputMode#duplicateSelection}
operation finished successfully.
@param listener The listener to remove.
@see GraphEditorInputMode#addElementsDuplicatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23ElementsDuplicated Online Documentation}*/
removeElementsDuplicatedListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
* Gets or sets the offset for paste operation if clipboard operations are enabled.
* The offset is applied to avoid that newly pasted elements are pasted exactly over their original.
*
* Default is (15, 15).
*
* Setting the {@link GraphEditorInputMode#pasteDelta} to 0,0 will prevent the pasted elements from being moved from their original location.
*
* <pre>
* mode.pasteDelta = Point.ORIGIN
* </pre>
*
* @default '15,15'
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23pasteDelta Online Documentation}
*/
pasteDelta: Point;
/**
Adds the given listener for the <code>ElementsCut</code> event that occurs when the {@link GraphEditorInputMode#cut} operation
finished successfully.
@param listener The listener to add.
@see GraphEditorInputMode#removeElementsCutListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23ElementsCut Online Documentation}*/
addElementsCutListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
Removes the given listener for the <code>ElementsCut</code> event that occurs when the {@link GraphEditorInputMode#cut}
operation finished successfully.
@param listener The listener to remove.
@see GraphEditorInputMode#addElementsCutListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23ElementsCut Online Documentation}*/
removeElementsCutListener(listener: (sender: any, evt: EventArgs) => void): void;
/**
* Gets or sets a value determining whether the {@link ICommand#EDIT_LABEL} command should be handled.
* If this flag is set to <code>true</code> pressing F2 will start the label editor. By default this feature is enabled.
*
* <br><b>Configure label editing</b>
* <pre>
* // forbid adding new labels
* mode.allowAddLabel = false
* // allow editing existing labels
* mode.allowEditLabel = true
* // start label editing on double click
* mode.allowEditLabelOnDoubleClick = true
* // only support editing node labels
* mode.labelEditableItems = GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL
* </pre>
*
* @see GraphEditorInputMode#editLabel
* @see GraphEditorInputMode#onEditLabel
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowEditLabel Online Documentation}
*/
allowEditLabel: boolean;
/**
* Gets or sets a value determining whether double-clicking should start label editing.
* If enabled, double-clicking an item will automatically execute the {@link ICommand#EDIT_LABEL} command.
*
* This setting has no effect if label editing is generally disallowed by setting {@link GraphEditorInputMode#allowEditLabel} to <code>false</code>.
*
* The default value is <code>true</code>.
*
* <br><b>Configure label editing</b>
* <pre>
* // forbid adding new labels
* mode.allowAddLabel = false
* // allow editing existing labels
* mode.allowEditLabel = true
* // start label editing on double click
* mode.allowEditLabelOnDoubleClick = true
* // only support editing node labels
* mode.labelEditableItems = GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL
* </pre>
*
* @see GraphEditorInputMode#allowEditLabel
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowEditLabelOnDoubleClick Online Documentation}
*/
allowEditLabelOnDoubleClick: boolean;
/**
* Gets or sets a value determining whether the {@link ICommand#ADD_LABEL} command should be handled.
* If this flag is set to <code>true</code> pressing Shift+F2 will start the label editor. By default this feature is enabled.
*
* <br><b>Configure label editing</b>
* <pre>
* // forbid adding new labels
* mode.allowAddLabel = false
* // allow editing existing labels
* mode.allowEditLabel = true
* // start label editing on double click
* mode.allowEditLabelOnDoubleClick = true
* // only support editing node labels
* mode.labelEditableItems = GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL
* </pre>
*
* @see GraphEditorInputMode#createLabel
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowAddLabel Online Documentation}
*/
allowAddLabel: boolean;
/**
Adds the given listener for the <code>LabelAdded</code> event that occurs when this mode has triggered the addition of an
{@link ILabel}, for instance, in response to {@link GraphEditorInputMode#createLabel}.
@param listener The listener to add.
@see GraphEditorInputMode#removeLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelAdded Online Documentation}*/
addLabelAddedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelAdded</code> event that occurs when this mode has triggered the addition of an
{@link ILabel}, for instance, in response to {@link GraphEditorInputMode#createLabel}.
@param listener The listener to remove.
@see GraphEditorInputMode#addLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelAdded Online Documentation}*/
removeLabelAddedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
* Gets or sets a handler which allows for configuring the {@link GraphEditorInputMode#textEditorInputMode} during adding or editing labels.
* If this handler is set it will be called during adding or editing labels after the defaults are set on {@link GraphEditorInputMode#textEditorInputMode} but before a {@link LabelEditingEventArgs#textEditorInputModeConfigurator}
* which is obtained from {@link GraphEditorInputMode#addLabelAddingListener LabelAdding},
* {@link GraphEditorInputMode#addLabelEditingListener LabelEditing}, or an {@link IEditLabelHelper}
* is called. Therefore, the settings which are done in this handler override the defaults but might be overridden
* themselves by a subsequently called handler.
*
* <pre>
* mode.textEditorInputModeConfigurator = (context, inputMode, label) => {
*   // anchor the text editor centered above the label
*   inputMode.anchor = new Point(0.5, 1)
*   // always place the editor horizontal
*   inputMode.upVector = new Point(0, -1)
*   // place the editor relative to the label's center
*   inputMode.location = label.layout.orientedRectangleCenter
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23textEditorInputModeConfigurator Online Documentation}
*/
textEditorInputModeConfigurator: ((context: IInputModeContext, mode: TextEditorInputMode, label: ILabel) => void) | null;
/**
Adds the given listener for the <code>LabelTextEditingStarted</code> event that occurs when the actual label editing process is about
to start.
This allows to customize the label editing process.
@param listener The listener to add.
@see GraphEditorInputMode#addValidateLabelTextListener
@see GraphEditorInputMode#removeLabelTextEditingStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelTextEditingStarted Online Documentation}*/
addLabelTextEditingStartedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelTextEditingStarted</code> event that occurs when the actual label editing process is
about to start.
This allows to customize the label editing process.
@param listener The listener to remove.
@see GraphEditorInputMode#addValidateLabelTextListener
@see GraphEditorInputMode#addLabelTextEditingStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelTextEditingStarted Online Documentation}*/
removeLabelTextEditingStartedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Adds the given listener for the <code>LabelTextEditingCanceled</code> event that occurs when the actual label editing process is
canceled.
This allows to unconfigure temporary customizations for the actual label editing process.
@param listener The listener to add.
@see GraphEditorInputMode#addValidateLabelTextListener
@see GraphEditorInputMode#removeLabelTextEditingCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelTextEditingCanceled Online Documentation}*/
addLabelTextEditingCanceledListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelTextEditingCanceled</code> event that occurs when the actual label editing process is
canceled.
This allows to unconfigure temporary customizations for the actual label editing process.
@param listener The listener to remove.
@see GraphEditorInputMode#addValidateLabelTextListener
@see GraphEditorInputMode#addLabelTextEditingCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelTextEditingCanceled Online Documentation}*/
removeLabelTextEditingCanceledListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
* Adds the given listener for the <code>ValidateLabelText</code> event that occurs when a label that is about to be {@link GraphEditorInputMode#addLabel added} or {@link GraphEditorInputMode#editLabel edited}.
* This event that can be used to validate the label text before inserting/updating the actual label. Set {@link LabelTextValidatingEventArgs#cancel} to <code>true</code> to
* reject the currently edited label's {@link LabelTextValidatingEventArgs#newText}.
*
* Note that in the case of {@link GraphEditorInputMode#addLabel} the {@link LabelTextValidatingEventArgs#label} is not part of the current graph but only a
* dummy instance.
*
* Labels with tag "int label" should be rejected if their text cannot be parsed into an integer value:
*
* <pre>
* mode.addValidateLabelTextListener((o, args) => {
*   if (args.label.tag === 'int label') {
*     args.cancel = Number.isNaN(parseInt(args.newText))
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphEditorInputMode#onValidateLabelText
* @see GraphEditorInputMode#removeValidateLabelTextListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23ValidateLabelText Online Documentation}
*/
addValidateLabelTextListener(listener: (sender: any, evt: LabelTextValidatingEventArgs) => void): void;
/**
* Removes the given listener for the <code>ValidateLabelText</code> event that occurs when a label that is about to be {@link GraphEditorInputMode#addLabel added} or
* {@link GraphEditorInputMode#editLabel edited}.
* This event that can be used to validate the label text before inserting/updating the actual label. Set {@link LabelTextValidatingEventArgs#cancel} to <code>true</code> to
* reject the currently edited label's {@link LabelTextValidatingEventArgs#newText}.
*
* Note that in the case of {@link GraphEditorInputMode#addLabel} the {@link LabelTextValidatingEventArgs#label} is not part of the current graph but only a
* dummy instance.
*
* Labels with tag "int label" should be rejected if their text cannot be parsed into an integer value:
*
* <pre>
* mode.addValidateLabelTextListener((o, args) => {
*   if (args.label.tag === 'int label') {
*     args.cancel = Number.isNaN(parseInt(args.newText))
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphEditorInputMode#onValidateLabelText
* @see GraphEditorInputMode#addValidateLabelTextListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23ValidateLabelText Online Documentation}
*/
removeValidateLabelTextListener(listener: (sender: any, evt: LabelTextValidatingEventArgs) => void): void;
/**
Adds the given listener for the <code>LabelTextChanged</code> event that occurs when this mode has triggered the edit of an
{@link ILabel}, for instance, in response to {@link GraphEditorInputMode#editLabel}.
@param listener The listener to add.
@see GraphEditorInputMode#removeLabelTextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelTextChanged Online Documentation}*/
addLabelTextChangedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelTextChanged</code> event that occurs when this mode has triggered the edit of an
{@link ILabel}, for instance, in response to {@link GraphEditorInputMode#editLabel}.
@param listener The listener to remove.
@see GraphEditorInputMode#addLabelTextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelTextChanged Online Documentation}*/
removeLabelTextChangedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
* Adds the given listener for the <code>DeletingSelection</code> event that occurs just before the {@link GraphEditorInputMode#deleteSelection}
* method starts its work and will be followed by any number of {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
* events and finalized by a {@link GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
* event.
* This event is raised before the deletion starts. It might be used to take a snapshot of the items to delete since there
* is no access to these afterwards.
*
* The {@link GraphEditorInputMode#addDeletedItemListener DeletedItem} is raised for each item
* after it has been removed. The {@link GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
* event is raised after all items have been removed.
*
* The deleted items are neither in the selection nor in the graph. One has to retrieve them before deletion:
*
* <pre>
* let selectedItems = new List()
*
* mode.addDeletingSelectionListener((sender, args) => {
*   selectedItems = args.selection.toList()
* })
*
* mode.addDeletedSelectionListener((sender, args) => {
*   // the deleted items are the items which were selected before deletion
*   // and are not in the current selection
*   const list = args.selection.toList()
*   const deletedItems = selectedItems.filter(item => !list.includes(item))
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphEditorInputMode#deleteSelection
* @see GraphEditorInputMode#removeDeletingSelectionListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23DeletingSelection Online Documentation}
*/
addDeletingSelectionListener(listener: (sender: any, evt: SelectionEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>DeletingSelection</code> event that occurs just before the {@link GraphEditorInputMode#deleteSelection}
* method starts its work and will be followed by any number of {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
* events and finalized by a {@link GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
* event.
* This event is raised before the deletion starts. It might be used to take a snapshot of the items to delete since there
* is no access to these afterwards.
*
* The {@link GraphEditorInputMode#addDeletedItemListener DeletedItem} is raised for each item
* after it has been removed. The {@link GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
* event is raised after all items have been removed.
*
* The deleted items are neither in the selection nor in the graph. One has to retrieve them before deletion:
*
* <pre>
* let selectedItems = new List()
*
* mode.addDeletingSelectionListener((sender, args) => {
*   selectedItems = args.selection.toList()
* })
*
* mode.addDeletedSelectionListener((sender, args) => {
*   // the deleted items are the items which were selected before deletion
*   // and are not in the current selection
*   const list = args.selection.toList()
*   const deletedItems = selectedItems.filter(item => !list.includes(item))
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphEditorInputMode#deleteSelection
* @see GraphEditorInputMode#addDeletingSelectionListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23DeletingSelection Online Documentation}
*/
removeDeletingSelectionListener(listener: (sender: any, evt: SelectionEventArgs<IModelItem>) => void): void;
/**
* Adds the given listener for the <code>DeletedItem</code> event that occurs when an item has been deleted interactively by this mode.
* Note that this event will not be triggered for items that are removed implicitly, e.g. if the event is triggered for a
* node, its labels, ports, and adjacent edges will not necessarily be reported separately. In order to be notified of any
* removal events, the {@link IGraph} events should be used instead.
*
* This event is raised after each selected item have been removed. To clean up after deletion, e.g. by applying a new
* automatic layout, use the {@link GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
* event which is raised only once after all items have been deleted.
*
* The {@link GraphEditorInputMode#addDeletingSelectionListener DeletingSelection} event is raised
* before all items have been removed. The {@link GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
* event is raised after all items have been removed.
*
* In addition to the deleted item and its type, the {@link ItemEventArgs} provide information about the status before deletion.
*
* <pre>
* mode.addDeletedItemListener((sender, e) => {
*   if (e instanceof NodeEventArgs) {
*     console.log('Old parent: ' + e.parent)
*   } else if (e instanceof EdgeEventArgs) {
*     console.log('Old source port owner: ' + e.sourcePortOwner)
*     console.log('Old target port owner: ' + e.targetPortOwner)
*   } else if (e instanceof PortEventArgs) {
*     console.log('Old port owner: ' + e.owner)
*   } else if (e instanceof LabelEventArgs) {
*     console.log('Old label owner: ' + e.owner)
*   } else if (e instanceof BendEventArgs) {
*     console.log('Old bend owner: ' + e.owner)
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphEditorInputMode#deleteSelection
* @see GraphEditorInputMode#addDeletingSelectionListener
* @see GraphEditorInputMode#addDeletedSelectionListener
* @see GraphEditorInputMode#removeDeletedItemListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23DeletedItem Online Documentation}
*/
addDeletedItemListener(listener: (sender: any, evt: ItemEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>DeletedItem</code> event that occurs when an item has been deleted interactively by this
* mode.
* Note that this event will not be triggered for items that are removed implicitly, e.g. if the event is triggered for a
* node, its labels, ports, and adjacent edges will not necessarily be reported separately. In order to be notified of any
* removal events, the {@link IGraph} events should be used instead.
*
* This event is raised after each selected item have been removed. To clean up after deletion, e.g. by applying a new
* automatic layout, use the {@link GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
* event which is raised only once after all items have been deleted.
*
* The {@link GraphEditorInputMode#addDeletingSelectionListener DeletingSelection} event is raised
* before all items have been removed. The {@link GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
* event is raised after all items have been removed.
*
* In addition to the deleted item and its type, the {@link ItemEventArgs} provide information about the status before deletion.
*
* <pre>
* mode.addDeletedItemListener((sender, e) => {
*   if (e instanceof NodeEventArgs) {
*     console.log('Old parent: ' + e.parent)
*   } else if (e instanceof EdgeEventArgs) {
*     console.log('Old source port owner: ' + e.sourcePortOwner)
*     console.log('Old target port owner: ' + e.targetPortOwner)
*   } else if (e instanceof PortEventArgs) {
*     console.log('Old port owner: ' + e.owner)
*   } else if (e instanceof LabelEventArgs) {
*     console.log('Old label owner: ' + e.owner)
*   } else if (e instanceof BendEventArgs) {
*     console.log('Old bend owner: ' + e.owner)
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphEditorInputMode#deleteSelection
* @see GraphEditorInputMode#addDeletingSelectionListener
* @see GraphEditorInputMode#addDeletedSelectionListener
* @see GraphEditorInputMode#addDeletedItemListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23DeletedItem Online Documentation}
*/
removeDeletedItemListener(listener: (sender: any, evt: ItemEventArgs<IModelItem>) => void): void;
/**
* Adds the given listener for the <code>DeletedSelection</code> event that occurs after all selected items have been removed in
* {@link GraphEditorInputMode#deleteSelection}.
* This event is raised after all selected items have been removed. It is recommended to register to this event if one
* wants to clean up after deletion, e.g. by applying a new automatic layout.
*
* The {@link GraphEditorInputMode#addDeletingSelectionListener DeletingSelection} event is raised
* before all items have been removed. The {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
* is raised for each item after it has been removed.
*
* Note that all items which have been removed are not in the {@link SelectionEventArgs#selection} anymore.
*
* Applying a layout after the selected items have been deleted;
*
* <pre>
* mode.addDeletedSelectionListener((sender, args) => mode.graph.applyLayout(new HierarchicLayout()))
* </pre>
*
* The deleted items are neither in the selection nor in the graph. One has to retrieve them before deletion:
*
* <pre>
* let selectedItems = new List()
*
* mode.addDeletingSelectionListener((sender, args) => {
*   selectedItems = args.selection.toList()
* })
*
* mode.addDeletedSelectionListener((sender, args) => {
*   // the deleted items are the items which were selected before deletion
*   // and are not in the current selection
*   const list = args.selection.toList()
*   const deletedItems = selectedItems.filter(item => !list.includes(item))
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphEditorInputMode#deleteSelection
* @see GraphEditorInputMode#removeDeletedSelectionListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23DeletedSelection Online Documentation}
*/
addDeletedSelectionListener(listener: (sender: any, evt: SelectionEventArgs<IModelItem>) => void): void;
/**
* Removes the given listener for the <code>DeletedSelection</code> event that occurs after all selected items have been removed in
* {@link GraphEditorInputMode#deleteSelection}.
* This event is raised after all selected items have been removed. It is recommended to register to this event if one
* wants to clean up after deletion, e.g. by applying a new automatic layout.
*
* The {@link GraphEditorInputMode#addDeletingSelectionListener DeletingSelection} event is raised
* before all items have been removed. The {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
* is raised for each item after it has been removed.
*
* Note that all items which have been removed are not in the {@link SelectionEventArgs#selection} anymore.
*
* Applying a layout after the selected items have been deleted;
*
* <pre>
* mode.addDeletedSelectionListener((sender, args) => mode.graph.applyLayout(new HierarchicLayout()))
* </pre>
*
* The deleted items are neither in the selection nor in the graph. One has to retrieve them before deletion:
*
* <pre>
* let selectedItems = new List()
*
* mode.addDeletingSelectionListener((sender, args) => {
*   selectedItems = args.selection.toList()
* })
*
* mode.addDeletedSelectionListener((sender, args) => {
*   // the deleted items are the items which were selected before deletion
*   // and are not in the current selection
*   const list = args.selection.toList()
*   const deletedItems = selectedItems.filter(item => !list.includes(item))
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphEditorInputMode#deleteSelection
* @see GraphEditorInputMode#addDeletedSelectionListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23DeletedSelection Online Documentation}
*/
removeDeletedSelectionListener(listener: (sender: any, evt: SelectionEventArgs<IModelItem>) => void): void;
/**
* Gets or sets the {@link MoveInputMode} as child mode.
* If no value has been set explicitly, the factory method {@link GraphEditorInputMode#createMoveInputMode} will be
* called upon first access. Upon change the {@link GraphEditorInputMode#onMoveInputModeChanged} method will be
* called. The newly created input mode is configured to handle selected items.
*
* The {@link GraphEditorInputMode#moveInputMode} does not handle labels, the {@link GraphEditorInputMode#moveLabelInputMode} is used instead. By default, it only handles selected items. The {@link GraphEditorInputMode#moveUnselectedInputMode} can be enabled to
* move unselected items, too.
*
* The {@link GraphEditorInputMode#moveInputMode} respects {@link GraphEditorInputMode#movableItems} and method {@link GraphEditorInputMode#shouldMove}.
*
* By default this input mode has a priority of <code>40</code>.
*
* <br><b>Disabling the MoveInputMode on its parent input mode</b>
* <pre>
* mode.moveInputMode.enabled = false
* </pre>
*
* If one needs to execute some code after some items have been moved by the {@link MoveInputMode} he can register a
* handler to its {@link MoveInputMode#addDragFinishedListener DragFinished} event. The moved items can
* be retrieved from the {@link MoveInputMode#affectedItems} property:
*
* <pre>
* mode.moveInputMode.addDragFinishedListener((sender, args) => {
*   for (const item of mode.moveInputMode.affectedItems) {
*     // these items have been moved
*   }
* })
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see GraphEditorInputMode#shouldMove
* @see GraphEditorInputMode#movableItems
* @see GraphEditorInputMode#moveUnselectedInputMode
* @see GraphEditorInputMode#moveLabelInputMode
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23moveInputMode Online Documentation}
*/
moveInputMode: MoveInputMode;
/**
* Gets or sets the {@link MoveInputMode} which handles unselected graph items.
* If no value has been set explicitly, the factory method {@link GraphEditorInputMode#createMoveUnselectedInputMode}
* will be called upon first access. Upon change the {@link GraphEditorInputMode#onMoveUnselectedInputModeChanged}
* method will be called.
*
* The {@link GraphEditorInputMode#moveUnselectedInputMode} does not handle labels, the {@link GraphEditorInputMode#moveLabelInputMode} is used instead.
*
* The {@link GraphEditorInputMode#moveUnselectedInputMode} respects {@link GraphEditorInputMode#movableItems} and method {@link GraphEditorInputMode#shouldMove}.
*
* By default this input mode has a priority of <code>41</code> and is {@link MoveInputMode#enabled disabled}.
*
* <br><b>Enabling the MoveUnselectedInputMode on its parent input mode</b>
* <pre>
* mode.moveUnselectedInputMode.enabled = true
* </pre>
*
* By default the <code>MoveUnselectedInputMode</code> interferes with the {@link GraphEditorInputMode#createEdgeInputMode}. A possible solution is to set a higher priority to the <code>MoveUnselectedInputMode</code>
* than to the {@link GraphEditorInputMode#createEdgeInputMode} and to add the ability to temporarily disable it. The following example shows how to configure the <code>MoveUnselectedInputMode</code>
* to be disabled when pressing the Shift key:
*
* <pre>
* mode.moveInputMode.enabled = false
* // Enable the move input mode for unselected items
* const moveUnselectedInputMode = mode.moveUnselectedInputMode
* moveUnselectedInputMode.enabled = true
*
* // Use a modifier recognizer for shift key not held down.
* moveUnselectedInputMode.pressedRecognizer = EventRecognizers.createAndRecognizer(
*   moveUnselectedInputMode.pressedRecognizer,
*   (sender, args) => !KeyEventRecognizers.SHIFT_IS_DOWN(sender, args)
* )
* moveUnselectedInputMode.hoverRecognizer = EventRecognizers.createAndRecognizer(
*   moveUnselectedInputMode.hoverRecognizer,
*   (sender, args) => !KeyEventRecognizers.SHIFT_IS_DOWN(sender, args)
* )
* // Set a higher priority than CreateEdgeInputMode
* moveUnselectedInputMode.priority = mode.createEdgeInputMode.priority - 1
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see GraphEditorInputMode#shouldMove
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23moveUnselectedInputMode Online Documentation}
*/
moveUnselectedInputMode: MoveInputMode;
/**
Gets or sets a property that determines whether and how this instance should automatically adjust the {@link CanvasComponent#contentRect} if the graph
changes.
The {@link GraphEditorInputMode#adjustContentRect} method is called whenever the content changes. The default is
{@link AdjustContentRectPolicy#UNION}.
@default AdjustContentRectPolicy.UNION
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23adjustContentRectPolicy Online Documentation}*/
adjustContentRectPolicy: AdjustContentRectPolicy;
/**
Gets or sets the margins to use for {@link CanvasComponent#updateContentRect} in {@link GraphEditorInputMode#adjustContentRect}
calls.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23contentRectMargins Online Documentation}*/
contentRectMargins: Insets;
/**
Gets the {@link GraphEditorInputMode#graphComponent} instance this mode is working on or <code>null</code>.
This property is set when this mode is {@link GraphEditorInputMode#install}ed in a {@link GraphEditorInputMode#graphComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23graphComponent Online Documentation}*/
readonly graphComponent: GraphComponent | null;
/**
* Gets or sets the {@link SnapContext} instance that handles interactive snapping of elements during drag operations
* like movements.
* To enable snapping one has to set an instance of {@link SnapContext}, in most cases {@link GraphSnapContext}.
*
* Setting this property to a {@link SnapContext} which is {@link SnapContext#enabled disabled} will disable snapping, whereas setting it to <code>null</code>
* will make the child modes use the context from their {@link IInputModeContext}s, if available.
*
* Upon change the {@link GraphEditorInputMode#onSnapContextChanged} method is called, which will as a side effect
* configure the snap context to display the {@link SnapResult}s in the {@link CanvasComponent}.
*
* Note that label snapping is controlled by another context which has to be set separately: {@link GraphEditorInputMode#labelSnapContext}.
*
* Default is <code>null</code>.
*
* <br><b>Setting the snap context</b>
* <pre>
* mode.snapContext = new GraphSnapContext()
* </pre>
*
* If one wants to enable snapping for labels, too, he has to set the {@link GraphEditorInputMode#labelSnapContext}, too:
*
* <br><b>Setting the snap context for both graph item and label snapping</b>
* <pre>
* mode.snapContext = new GraphSnapContext()
* mode.labelSnapContext = new LabelSnapContext()
* </pre>
*
* @see GraphSnapContext
* @see GraphEditorInputMode#labelSnapContext
* @see LabelSnapContext
* @default null
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23snapContext Online Documentation}
*/
snapContext: SnapContext | null;
/**
* Gets or sets the {@link LabelSnapContext} instance that handles interactive snapping of
* {@link ILabel}s to their owner during drag operations like movements.
* To enable snapping for labels one has to set an instance of {@link SnapContext}, in most cases
* {@link LabelSnapContext}. Setting this property is not necessary if moving labels is not allowed or if the used {@link ILabelModel label models}
* do not support snapping or allow for discrete positions, anyway.
*
* Setting this property to a {@link SnapContext} which is {@link SnapContext#enabled disabled} will disable snapping, whereas setting it to <code>null</code>
* will make the child modes use the context from their {@link IInputModeContext}s, if available. Upon change the {@link GraphEditorInputMode#onLabelSnapContextChanged}
* method is called, which will as a side effect configure the snap context to display the {@link SnapResult}s in the
* {@link CanvasComponent}.
*
* Default is <code>null</code>.
*
* <br><b>Setting the snap context for both graph item and label snapping</b>
* <pre>
* mode.snapContext = new GraphSnapContext()
* mode.labelSnapContext = new LabelSnapContext()
* </pre>
*
* If one wants to enable snapping for labels only, he can omit setting the {@link GraphEditorInputMode#snapContext}:
*
* <br><b>Setting the snap context</b>
* <pre>
* mode.labelSnapContext = new LabelSnapContext()
* </pre>
*
* @see LabelSnapContext
* @see GraphEditorInputMode#snapContext
* @default null
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23labelSnapContext Online Documentation}
*/
labelSnapContext: SnapContext | null;
/**
* Gets or sets the {@link MoveLabelInputMode} which handles interactive moving {@link ILabel}s.
* This mode is responsible for moving labels to another candidate position. If no value has been set explicitly, the
* factory method {@link GraphEditorInputMode#createMoveLabelInputMode} will be called upon first access. Upon change
* the {@link GraphEditorInputMode#onMoveLabelInputModeChanged} method will be called.
*
* By default this input mode has a priority of <code>39</code>.
*
* <br><b>Disabling the MoveLabelInputMode on its parent input mode</b>
* <pre>
* mode.moveLabelInputMode.enabled = false
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see GraphEditorInputMode#shouldMove
* @see GraphEditorInputMode#moveInputMode
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23moveLabelInputMode Online Documentation}
*/
moveLabelInputMode: MoveLabelInputMode;
/**
* Gets or sets a value indicating whether grouping operations like {@link GraphEditorInputMode#allowGroupSelection grouping selected nodes} or {@link GraphEditorInputMode#allowReparentNodes moving nodes into group nodes}
* should be enabled.
* Setting this property to <code>false</code> effectively disables handling of all grouping {@link ICommand commands}. If this property is set to <code>true</code>
* the related properties {@link GraphEditorInputMode#allowGroupSelection}, {@link GraphEditorInputMode#allowUngroupSelection}, and {@link GraphEditorInputMode#allowReparentNodes} allow for selectively enabling or disabling only certain grouping operations.
*
* The default value is <code>false</code>.
*
* The following example shows how to enable grouping operations on the <code>GraphEditorInputMode</code> together with some detail
* settings:
*
* <pre>
* // first and foremost: enable grouping operations at all
* geim.allowGroupingOperations = true
* // support grouping but not ungrouping the currently selected nodes
* geim.allowGroupSelection = true
* geim.allowUngroupSelection = false
* // forbid adjusting the size with Ctrl-Shift-G
* geim.allowAdjustGroupNodeSize = false
* // allow adding nodes to other group nodes by moving
* geim.allowReparentNodes = true
* // also allow to add nodes to non-group nodes, transforming these into groups
* geim.allowReparentToNonGroupNodes = true
* </pre>
*
* @see GraphEditorInputMode#allowGroupSelection
* @see GraphEditorInputMode#allowUngroupSelection
* @see GraphEditorInputMode#allowAdjustGroupNodeSize
* @see GraphEditorInputMode#allowReparentNodes
* @default false
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowGroupingOperations Online Documentation}
*/
allowGroupingOperations: boolean;
/**
* Gets or sets a value determining whether the {@link ICommand#GROUP_SELECTION} command should be handled.
* This setting has no effect if grouping operations are generally disallowed by setting {@link GraphEditorInputMode#allowGroupingOperations} to <code>false</code>.
*
* The default value is <code>true</code>.
*
* The following example shows how to configure the grouping support on the <code>GraphEditorInputMode</code>. Note that general
* grouping support has to be enabled explicitly with {@link GraphEditorInputMode#allowGroupingOperations}:
*
* <pre>
* // first and foremost: enable grouping operations at all
* geim.allowGroupingOperations = true
* // support grouping but not ungrouping the currently selected nodes
* geim.allowGroupSelection = true
* geim.allowUngroupSelection = false
* // forbid adjusting the size with Ctrl-Shift-G
* geim.allowAdjustGroupNodeSize = false
* // allow adding nodes to other group nodes by moving
* geim.allowReparentNodes = true
* // also allow to add nodes to non-group nodes, transforming these into groups
* geim.allowReparentToNonGroupNodes = true
* </pre>
*
* @see GraphEditorInputMode#allowGroupingOperations
* @see GraphEditorInputMode#groupSelection
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowGroupSelection Online Documentation}
*/
allowGroupSelection: boolean;
/**
* Gets or sets a value determining whether the {@link ICommand#ADJUST_GROUP_NODE_SIZE} command should be handled.
* This setting has no effect if grouping operations are generally disallowed by setting {@link GraphEditorInputMode#allowGroupingOperations} to <code>false</code>.
*
* The default value is <code>true</code>.
*
* The following example shows how to configure the grouping support on the <code>GraphEditorInputMode</code>. Note that general
* grouping support has to be enabled explicitly with {@link GraphEditorInputMode#allowGroupingOperations}:
*
* <pre>
* // first and foremost: enable grouping operations at all
* geim.allowGroupingOperations = true
* // support grouping but not ungrouping the currently selected nodes
* geim.allowGroupSelection = true
* geim.allowUngroupSelection = false
* // forbid adjusting the size with Ctrl-Shift-G
* geim.allowAdjustGroupNodeSize = false
* // allow adding nodes to other group nodes by moving
* geim.allowReparentNodes = true
* // also allow to add nodes to non-group nodes, transforming these into groups
* geim.allowReparentToNonGroupNodes = true
* </pre>
*
* @see GraphEditorInputMode#allowGroupingOperations
* @see GraphEditorInputMode#adjustGroupNodeSizes
* @see GraphEditorInputMode#adjustGroupNodeSize
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowAdjustGroupNodeSize Online Documentation}
*/
allowAdjustGroupNodeSize: boolean;
/**
* Gets or sets a value determining whether the {@link ICommand#UNGROUP_SELECTION} command should be handled.
* This setting has no effect if grouping operations are generally disallowed by setting {@link GraphEditorInputMode#allowGroupingOperations} to <code>false</code>.
*
* The default value is <code>true</code>.
*
* The following example shows how to configure the grouping support on the <code>GraphEditorInputMode</code>. Note that general
* grouping support has to be enabled explicitly with {@link GraphEditorInputMode#allowGroupingOperations}:
*
* <pre>
* // first and foremost: enable grouping operations at all
* geim.allowGroupingOperations = true
* // support grouping but not ungrouping the currently selected nodes
* geim.allowGroupSelection = true
* geim.allowUngroupSelection = false
* // forbid adjusting the size with Ctrl-Shift-G
* geim.allowAdjustGroupNodeSize = false
* // allow adding nodes to other group nodes by moving
* geim.allowReparentNodes = true
* // also allow to add nodes to non-group nodes, transforming these into groups
* geim.allowReparentToNonGroupNodes = true
* </pre>
*
* @see GraphEditorInputMode#allowGroupingOperations
* @see GraphEditorInputMode#ungroupSelection
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowUngroupSelection Online Documentation}
*/
allowUngroupSelection: boolean;
/**
* Gets or sets a value determining whether the {@link ICommand#DESELECT_ALL} command should be handled.
* The default value is <code>true</code>.
*
* <pre>
* mode.allowClearSelection = true
* </pre>
*
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowClearSelection Online Documentation}
*/
allowClearSelection: boolean;
/**
* Gets or sets a value determining whether the {@link ICommand#PASTE} command should be handled.
* This setting has no effect if clipboard operations are generally disallowed by setting {@link GraphInputMode#allowClipboardOperations} to <code>false</code>.
*
* The default value is <code>true</code>.
*
* <pre>
* mode.allowClipboardOperations = true
* mode.allowDuplicate = true
* mode.allowPaste = true
* </pre>
*
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowPaste Online Documentation}
*/
allowPaste: boolean;
/**
* Gets or sets a value determining whether the {@link ICommand#DUPLICATE} command should be handled.
* This setting has no effect if clipboard operations are generally disallowed by setting {@link GraphInputMode#allowClipboardOperations} to <code>false</code>.
*
* The default value is <code>true</code>.
*
* <pre>
* mode.allowClipboardOperations = true
* mode.allowDuplicate = true
* mode.allowPaste = true
* </pre>
*
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowDuplicate Online Documentation}
*/
allowDuplicate: boolean;
/**
Gets or sets a value indicating whether undo operations, that is, {@link ICommand#UNDO} and {@link ICommand#REDO}
should be enabled.
Note that even if this feature is enabled, an {@link UndoEngine undo engine} needs to be available in the {@link InputModeBase#inputModeContext} in order to work.
Setting this property to <code>false</code> effectively disables handling of the commands {@link ICommand#UNDO} and
{@link ICommand#REDO}. Note that this property is not respected when calling {@link GraphEditorInputMode#undo}
or {@link GraphEditorInputMode#redo} directly.
The default value is <code>true</code>.
@see GraphEditorInputMode#undo
@see GraphEditorInputMode#redo
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowUndoOperations Online Documentation}*/
allowUndoOperations: boolean;
/**
* Gets or sets a value indicating whether moving nodes out of group nodes or into other group nodes is allowed.
* If set to <code>false</code> the {@link GraphEditorInputMode#reparentNodeHandler} will effectively be disabled and not called anymore for reparenting operations.
*
* This setting has no effect if grouping operations are generally disallowed by setting {@link GraphEditorInputMode#allowGroupingOperations} to <code>false</code>.
*
* The default value is <code>true</code>.
*
* The following example shows how to configure the grouping support on the <code>GraphEditorInputMode</code>. Note that general
* grouping support has to be enabled explicitly with {@link GraphEditorInputMode#allowGroupingOperations}:
*
* <pre>
* // first and foremost: enable grouping operations at all
* geim.allowGroupingOperations = true
* // support grouping but not ungrouping the currently selected nodes
* geim.allowGroupSelection = true
* geim.allowUngroupSelection = false
* // forbid adjusting the size with Ctrl-Shift-G
* geim.allowAdjustGroupNodeSize = false
* // allow adding nodes to other group nodes by moving
* geim.allowReparentNodes = true
* // also allow to add nodes to non-group nodes, transforming these into groups
* geim.allowReparentToNonGroupNodes = true
* </pre>
*
* @see GraphEditorInputMode#reparentNodeHandler
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowReparentNodes Online Documentation}
*/
allowReparentNodes: boolean;
/**
* Gets or sets a value indicating whether moving nodes into {@link IGraph#isGroupNode non-group nodes} is allowed to convert them into group nodes.
* If {@link GraphEditorInputMode#allowReparentNodes reparenting nodes is allowed} in general and this property is enabled, users may reparent nodes not only to groups and
* folders but also to non-group nodes. Still, the decision about the final acceptance of the reparenting and the actual
* execution are delegated to the {@link GraphEditorInputMode#reparentNodeHandler}.
*
* This setting has no effect if grouping operations are generally disallowed by setting {@link GraphEditorInputMode#allowGroupingOperations} to <code>false</code>.
*
* The default value is <code>false</code>.
*
* The following example shows how to configure the grouping support on the <code>GraphEditorInputMode</code>. Note that general
* grouping support has to be enabled explicitly with {@link GraphEditorInputMode#allowGroupingOperations}:
*
* <pre>
* // first and foremost: enable grouping operations at all
* geim.allowGroupingOperations = true
* // support grouping but not ungrouping the currently selected nodes
* geim.allowGroupSelection = true
* geim.allowUngroupSelection = false
* // forbid adjusting the size with Ctrl-Shift-G
* geim.allowAdjustGroupNodeSize = false
* // allow adding nodes to other group nodes by moving
* geim.allowReparentNodes = true
* // also allow to add nodes to non-group nodes, transforming these into groups
* geim.allowReparentToNonGroupNodes = true
* </pre>
*
* @see GraphEditorInputMode#allowGroupingOperations
* @default false
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowReparentToNonGroupNodes Online Documentation}
*/
allowReparentToNonGroupNodes: boolean;
/**
Adds the given listener for the <code>NodeReparented</code> event that occurs when a node has been reparented interactively.
This is triggered whenever the {@link GraphEditorInputMode#reparentNodeHandler}'s {@link IReparentNodeHandler#reparent} method has been called to reparent a node
interactively.
@param listener The listener to add.
@see GraphEditorInputMode#reparentNodeHandler
@see GraphEditorInputMode#removeNodeReparentedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23NodeReparented Online Documentation}*/
addNodeReparentedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
Removes the given listener for the <code>NodeReparented</code> event that occurs when a node has been reparented interactively.
This is triggered whenever the {@link GraphEditorInputMode#reparentNodeHandler}'s {@link IReparentNodeHandler#reparent} method has been called to reparent a node
interactively.
@param listener The listener to remove.
@see GraphEditorInputMode#reparentNodeHandler
@see GraphEditorInputMode#addNodeReparentedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23NodeReparented Online Documentation}*/
removeNodeReparentedListener(listener: (sender: any, evt: NodeEventArgs) => void): void;
/**
* Adds the given listener for the <code>NodeCreated</code> event that occurs when this mode has created a node in response to user
* interaction.
* This event is triggered if the node is created using a click gesture via {@link GraphEditorInputMode#createNode},
* via a drag and drop operation that was completed by {@link GraphEditorInputMode#nodeDropInputMode}, or by grouping nodes via {@link ICommand#GROUP_SELECTION}.
* Note that if {@link NodeDropInputMode#allowFolderNodeAsParent} is set to <code>true</code>, the reported node can actually be part of the {@link IFoldingView master graph}.
*
* Applying a layout each time the user created a new node
*
* <pre>
* mode.addNodeCreatedListener((sender, args) => mode.graph.applyLayout(new HierarchicLayout()))
* </pre>
*
* @param listener The listener to add.
* @see GraphEditorInputMode#createNode
* @see GraphEditorInputMode#removeNodeCreatedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23NodeCreated Online Documentation}
*/
addNodeCreatedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
* Removes the given listener for the <code>NodeCreated</code> event that occurs when this mode has created a node in response to user
* interaction.
* This event is triggered if the node is created using a click gesture via {@link GraphEditorInputMode#createNode},
* via a drag and drop operation that was completed by {@link GraphEditorInputMode#nodeDropInputMode}, or by grouping nodes via {@link ICommand#GROUP_SELECTION}.
* Note that if {@link NodeDropInputMode#allowFolderNodeAsParent} is set to <code>true</code>, the reported node can actually be part of the {@link IFoldingView master graph}.
*
* Applying a layout each time the user created a new node
*
* <pre>
* mode.addNodeCreatedListener((sender, args) => mode.graph.applyLayout(new HierarchicLayout()))
* </pre>
*
* @param listener The listener to remove.
* @see GraphEditorInputMode#createNode
* @see GraphEditorInputMode#addNodeCreatedListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23NodeCreated Online Documentation}
*/
removeNodeCreatedListener(listener: (sender: any, evt: ItemEventArgs<INode>) => void): void;
/**
* Gets or sets a value determining whether {@link GraphEditorInputMode#createNode node creation} by clicking on an empty canvas location is enabled.
* To disable node creation via drag and drop gestures, the {@link GraphEditorInputMode#nodeDropInputMode} must be disabled, too. However, it is disabled by default.
*
* The default value is <code>true</code>.
*
* <br><b>Enabling or disabling item creation</b>
* <pre>
* mode.allowCreateBend = true
* mode.allowCreateEdge = true
* mode.allowCreateNode = true
* </pre>
*
* @see GraphEditorInputMode#nodeCreator
* @see GraphEditorInputMode#createNode
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowCreateNode Online Documentation}
*/
allowCreateNode: boolean;
/**
* Gets or sets a value determining whether {@link GraphEditorInputMode#createBendInputMode bend creation} should be {@link CreateBendInputMode#enabled enabled}.
* This property delegates to {@link GraphEditorInputMode#createBendInputMode}'s {@link CreateBendInputMode#enabled} property.
*
* The default value is <code>true</code>.
*
* <br><b>Enabling or disabling item creation</b>
* <pre>
* mode.allowCreateBend = true
* mode.allowCreateEdge = true
* mode.allowCreateNode = true
* </pre>
*
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowCreateBend Online Documentation}
*/
allowCreateBend: boolean;
/**
* Gets or sets a value determining whether {@link GraphEditorInputMode#createEdgeInputMode edge creation} should be {@link CreateEdgeInputMode#enabled enabled}.
* This property delegates to {@link GraphEditorInputMode#createEdgeInputMode}'s {@link CreateEdgeInputMode#enabled} property.
*
* The default value is <code>true</code>.
*
* <br><b>Enabling or disabling item creation</b>
* <pre>
* mode.allowCreateBend = true
* mode.allowCreateEdge = true
* mode.allowCreateNode = true
* </pre>
*
* @default true
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowCreateEdge Online Documentation}
*/
allowCreateEdge: boolean;
/**
Gets or sets a value indicating whether the {@link ICommand#REVERSE_EDGE} command should be handled.
If set to <code>false</code> the methods {@link GraphEditorInputMode#reverseEdge},
{@link GraphEditorInputMode#reverseEdges}, and {@link GraphEditorInputMode#reverseSelectedEdges} do nothing.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23allowReverseEdge Online Documentation}*/
allowReverseEdge: boolean;
/**
Adds the given listener for the <code>EdgePortsChanged</code> event that occurs after an edge's source and/or target ports have been
changed as the result of an input gesture.
@param listener The listener to add.
@see EdgeEventArgs
@see GraphEditorInputMode#reverseSelectedEdges
@see GraphEditorInputMode#reverseEdges
@see GraphEditorInputMode#reverseEdge
@see GraphEditorInputMode#removeEdgePortsChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23EdgePortsChanged Online Documentation}*/
addEdgePortsChangedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
Removes the given listener for the <code>EdgePortsChanged</code> event that occurs after an edge's source and/or target ports have
been changed as the result of an input gesture.
@param listener The listener to remove.
@see EdgeEventArgs
@see GraphEditorInputMode#reverseSelectedEdges
@see GraphEditorInputMode#reverseEdges
@see GraphEditorInputMode#reverseEdge
@see GraphEditorInputMode#addEdgePortsChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23EdgePortsChanged Online Documentation}*/
removeEdgePortsChangedListener(listener: (sender: any, evt: EdgeEventArgs) => void): void;
/**
* Gets or sets the callback that is responsible for creating a new node, for instance, in response to a mouse click.
* A <code>null</code> value will disable node creation using mouse clicks, just as setting {@link GraphEditorInputMode#allowCreateNode} to <code>false</code>.
*
* <pre>
* // set a custom node creator which creates a label on the new node
* graphEditorInputMode.nodeCreator = (context, graph, location, parent) => {
*   // create a node at the location with the given parent and the default size
*   const node = graph.createNode(parent, new Rect(location, graph.nodeDefaults.size))
*   // add a label
*   graph.addLabel(node, 'A new node')
*   // return the new node
*   return node
* }
* </pre>
*
* @see GraphEditorInputMode#createNode
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23nodeCreator Online Documentation}
*/
nodeCreator: ((context: IInputModeContext, graph: IGraph, location: Point, parent: INode | null) => INode | null) | null;
/**
* Gets or sets the {@link TextEditorInputMode} which handles label editing.
* If no value has been set explicitly, the factory method {@link GraphEditorInputMode#createTextEditorInputMode} will
* be called upon first access. Upon change the {@link GraphEditorInputMode#onTextEditorInputModeChanged} method will
* be called.
*
* By default this input mode has a priority of <code>100</code>.
*
* <br><b>Disabling the TextEditorInputMode on its parent input mode</b>
* <pre>
* mode.textEditorInputMode.enabled = false
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23textEditorInputMode Online Documentation}
*/
textEditorInputMode: TextEditorInputMode;
/**
* Gets or sets the {@link CreateEdgeInputMode} which handles interactive edge creation.
* If no value has been set explicitly, the factory method {@link GraphEditorInputMode#createCreateEdgeInputMode} will
* be called upon first access. Upon change the {@link GraphEditorInputMode#onCreateEdgeInputModeChanged} method will
* be called.
*
* By default this input mode has a priority of <code>45</code>.
*
* <br><b>Disabling the CreateEdgeInputMode on its parent input mode</b>
* <pre>
* mode.createEdgeInputMode.enabled = false
* </pre>
*
* An overview of this input mode is given in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/interaction-creating_edges Creating Edges}. Additionally, customization of edge creation is
* discussed in section {@link https://docs.yworks.com/yfileshtml/#/dguide/customizing_interaction_creating_edges#customizing_interaction_creating_edges Customizing User Interaction: Creating Edges}.
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createEdgeInputMode Online Documentation}
*/
createEdgeInputMode: CreateEdgeInputMode;
/**
Gets or sets a value indicating whether this mode should automatically {@link IGraph#remove remove labels} from the graph when a label text
has been edited and the label text is empty.
<code>true</code> if empty labels should be removed after text editing, <code>false</code> otherwise. {@link GraphEditorInputMode#addDeletedItemListener DeletedItem}
is dispatched after a label has been removed.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23autoRemoveEmptyLabels Online Documentation}*/
autoRemoveEmptyLabels: boolean;
/**
Gets or sets a property that determines whether clicking on a node should automatically select all bends of self loops
adjacent to that node.
The default value is <code>true</code>.
@see GraphInputMode#clickSelectableItems
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23autoSelectSelfloopBends Online Documentation}*/
autoSelectSelfloopBends: boolean;
/**
Gets or sets a property that determines whether the label should be hidden while it is edited.
If enabled, any selection, focus, and highlight visualizations of the label are hidden along with the label
visualization itself.
Even if enabled, the label <b>text</b> is still visible in the label text editor.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23hideLabelDuringEditing Online Documentation}*/
hideLabelDuringEditing: boolean;
/**
* Gets or sets the {@link CreateBendInputMode} which handles interactive bend creation on existing edges.
* If no value has been set explicitly, the factory method {@link GraphEditorInputMode#createCreateBendInputMode} will
* be called upon first access. Upon change the {@link GraphEditorInputMode#onCreateBendInputModeChanged} method will
* be called.
*
* By default this input mode has a priority of <code>42</code>.
*
* <br><b>Disabling the CreateBendInputMode on its parent input mode</b>
* <pre>
* mode.createBendInputMode.enabled = false
* </pre>
*
* Note that in order to diable interactive bend creation one has to disable bend creation during edge creation, too:
*
* <pre>
* // mode is an instance of GraphEditorInputMode
* mode.createBendInputMode.enabled = false
* mode.createEdgeInputMode.allowCreateBend = false
* </pre>
*
* By default, dragging on selected edges will move the edge. In order to start bend creation on selected edges, too, the {@link CreateBendInputMode}
* has to be set to a higher priority than the {@link MoveInputMode} which handles moving the edge.
*
* <pre>
* mode.createBendInputMode.priority = mode.moveInputMode.priority - 1
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23createBendInputMode Online Documentation}
*/
createBendInputMode: CreateBendInputMode;
/**
* Gets or sets the {@link NodeDropInputMode} that is responsible for handling drag and drop operations of nodes onto
* the {@link GraphEditorInputMode#graphComponent}.
* The mode by default is {@link DropInputMode#enabled disabled} and needs to be enabled to work, first. If no value has been set explicitly, the factory
* method {@link GraphEditorInputMode#createNodeDropInputMode} will be called upon first access. Upon change the {@link GraphEditorInputMode#onNodeDropInputModeChanged}
* method will be called.
*
* By default this input mode has a priority of <code>70</code>.
*
* <br><b>Disabling the NodeDropInputMode on its parent input mode</b>
* <pre>
* mode.nodeDropInputMode.enabled = false
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23nodeDropInputMode Online Documentation}
*/
nodeDropInputMode: NodeDropInputMode;
/**
* Gets or sets the {@link LabelDropInputMode} that is responsible for handling drag and drop operations of labels
* onto the {@link GraphEditorInputMode#graphComponent}.
* The mode by default is {@link DropInputMode#enabled disabled} and needs to be enabled to work, first. If no value has been set explicitly, the factory
* method {@link GraphEditorInputMode#createLabelDropInputMode} will be called upon first access. Upon change the {@link GraphEditorInputMode#onLabelDropInputModeChanged}
* method will be called.
*
* By default this input mode has a priority of <code>70</code>.
*
* <br><b>Disabling the LabelDropInputMode on its parent input mode</b>
* <pre>
* mode.labelDropInputMode.enabled = false
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23labelDropInputMode Online Documentation}
*/
labelDropInputMode: LabelDropInputMode;
/**
* Gets or sets the {@link PortDropInputMode} that is responsible for handling drag and drop operations of ports onto
* the {@link GraphEditorInputMode#graphComponent}.
* The mode by default is {@link DropInputMode#enabled disabled} and needs to be enabled to work, first. If no value has been set explicitly, the factory
* method {@link GraphEditorInputMode#createPortDropInputMode} will be called upon first access. Upon change the {@link GraphEditorInputMode#onPortDropInputModeChanged}
* method will be called.
*
* By default this input mode has a priority of <code>70</code>.
*
* <br><b>Disabling the PortDropInputMode on its parent input mode</b>
* <pre>
* mode.portDropInputMode.enabled = false
* </pre>
*
* @throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
* {@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23portDropInputMode Online Documentation}
*/
portDropInputMode: PortDropInputMode;
/**
Adds the given listener for the <code>PortAdded</code> event that occurs when this mode has triggered the addition of an
{@link IPort}, for instance, in response to {@link GraphEditorInputMode#portDropInputMode}.
@param listener The listener to add.
@see GraphEditorInputMode#removePortAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23PortAdded Online Documentation}*/
addPortAddedListener(listener: (sender: any, evt: PortEventArgs) => void): void;
/**
Removes the given listener for the <code>PortAdded</code> event that occurs when this mode has triggered the addition of an
{@link IPort}, for instance, in response to {@link GraphEditorInputMode#portDropInputMode}.
@param listener The listener to remove.
@see GraphEditorInputMode#addPortAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23PortAdded Online Documentation}*/
removePortAddedListener(listener: (sender: any, evt: PortEventArgs) => void): void;
/**
* Gets or sets whether the user interaction should be blocked.
* Setting this property to <code>true</code> will {@link WaitInputMode#startWaiting start the waiting process}. Setting it to <code>false</code> will {@link WaitInputMode#endWaiting end the waiting}.
*
* Blocking the user interaction is done by the {@link WaitInputMode}.
*
* <br><b>Blocking user interaction during an asynchronous process</b>
* <pre>
* graphEditorInputMode.waiting = true
* longRunningAsyncProcess().then(() => {
*   graphEditorInputMode.waiting = false
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23waiting Online Documentation}
*/
waiting: boolean;
/**
* Adds the given listener for the <code>LabelAdding</code> event that occurs when a label is about to be added.
* Event handlers for this event can customize the behavior for adding a new label extensively. Adding a label can be
* forbidden entirely, or the properties of the added label can be changed, by setting the appropriate properties on the
* {@link LabelEditingEventArgs}. It is even possible to forbid adding a label, but edit an existing one instead.
*
* <pre>
* mode.addLabelAddingListener((source, args) => {
*   if (args.owner.labels.size >= 1) {
*     // don't add a second label
*     args.cancel = true
*   } else {
*     // the new label should be red
*     args.style = new DefaultLabelStyle({ backgroundFill: Fill.RED })
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to add.
* @see GraphEditorInputMode#removeLabelAddingListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelAdding Online Documentation}
*/
addLabelAddingListener(listener: (sender: any, evt: LabelEditingEventArgs) => void): void;
/**
* Removes the given listener for the <code>LabelAdding</code> event that occurs when a label is about to be added.
* Event handlers for this event can customize the behavior for adding a new label extensively. Adding a label can be
* forbidden entirely, or the properties of the added label can be changed, by setting the appropriate properties on the
* {@link LabelEditingEventArgs}. It is even possible to forbid adding a label, but edit an existing one instead.
*
* <pre>
* mode.addLabelAddingListener((source, args) => {
*   if (args.owner.labels.size >= 1) {
*     // don't add a second label
*     args.cancel = true
*   } else {
*     // the new label should be red
*     args.style = new DefaultLabelStyle({ backgroundFill: Fill.RED })
*     args.handled = true
*   }
* })
* </pre>
*
* @param listener The listener to remove.
* @see GraphEditorInputMode#addLabelAddingListener
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelAdding Online Documentation}
*/
removeLabelAddingListener(listener: (sender: any, evt: LabelEditingEventArgs) => void): void;
/**
Adds the given listener for the <code>LabelEditing</code> event that occurs when a label is about to be edited.
Event handlers for this event can customize the behavior for editing an existing label. Editing can be forbidden
entirely, or redirected to a different label, even on a different owner. In case editing existing labels is unwanted,
event handlers can also specify that instead of editing a label, a new one should be added.
@param listener The listener to add.
@see GraphEditorInputMode#removeLabelEditingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelEditing Online Documentation}*/
addLabelEditingListener(listener: (sender: any, evt: LabelEditingEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelEditing</code> event that occurs when a label is about to be edited.
Event handlers for this event can customize the behavior for editing an existing label. Editing can be forbidden
entirely, or redirected to a different label, even on a different owner. In case editing existing labels is unwanted,
event handlers can also specify that instead of editing a label, a new one should be added.
@param listener The listener to remove.
@see GraphEditorInputMode#addLabelEditingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphEditorInputMode%23LabelEditing Online Documentation}*/
removeLabelEditingListener(listener: (sender: any, evt: LabelEditingEventArgs) => void): void;
static $class: Class;
private $_GraphEditorInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphEditorInputMode;
}
export interface ItemDropInputMode<T> extends DropInputMode {
}
/**
A {@link DropInputMode} providing a preview of the dragged item.
This input mode {@link ItemDropInputMode#showPreview can visualize a preview} of the {@link ItemDropInputMode#draggedItem dragged item} during the drag operation, and supports {@link ItemDropInputMode#snappingEnabled snapping} via the {@link ItemDropInputMode#snapContext}
and {@link ItemDropInputMode#highlightDropTarget highlighting} of the {@link ItemDropInputMode#dropTarget} via a {@link HighlightIndicatorManager}.
Type parameter <code>T</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode Online Documentation}*/
export abstract class ItemDropInputMode<T> {
/**
Initializes a new instance of the {@link ItemDropInputMode} class for the expected data type.
This instance fires events only if a drag operation carries data of the expected type.
@param expectedType A list of types that are used in {@link DragDropItem#getData}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23ItemDropInputMode(Array%3Cstring%3E) Online Documentation}*/
constructor(expectedType: string[]);
/**
Initializes a new instance of the {@link ItemDropInputMode} class for the expected data type.
This instance fires events only if a drag operation carries data of the expected type.
@param expectedType The type that is used in {@link DragDropItem#getData}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23ItemDropInputMode(string) Online Documentation}*/
constructor(expectedType: string);
/**
Cleanup method that calls {@link ItemDropInputMode#cleanUpSnapContext}, {@link ItemDropInputMode#cleanUpPreview} and {@link ItemDropInputMode#cleanUpDropTarget}.
This method should be called for cleanup actions after a drag/drop gesture has been finished or canceled.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23cleanUp() Online Documentation}*/
cleanUp(): void;
/**
Cleans up the drop target and its highlighting.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23cleanUpDropTarget() Online Documentation}*/
cleanUpDropTarget(): void;
/**
Cleans up the item preview.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23cleanUpPreview() Online Documentation}*/
cleanUpPreview(): void;
/**
Cleans up the snap context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23cleanUpSnapContext() Online Documentation}*/
cleanUpSnapContext(): void;
/**
Callback registered on the {@link SnapContext} that collects {@link SnapResult}s for the dragged element.
The default implementation doesn't collect any snap results.
@param source The {@link SnapContext} this callback is registered at.
@param evt The arguments describing the movement {@link SnapResult}s shall be collected for.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23collectSnapResults(Object%2CCollectSnapResultsEventArgs) Online Documentation}*/
collectSnapResults(source: any, evt: CollectSnapResultsEventArgs): void;
/**
Returns the drop target at the specified location.
The drop target is highlighted, if {@link ItemDropInputMode#highlightDropTarget} is enabled. Per default, <code>null</code> is returned.
@param dragLocation The location to return the drop target for.
@returns The drop target at the specified location or <code>null</code> if no drop target can be found.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23getDropTarget(Point) Online Documentation}*/
getDropTarget(dragLocation: Point): IModelItem | null;
/**
Initializes the drop target.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23initializeDropTarget() Online Documentation}*/
initializeDropTarget(): void;
/**
Initializes the item preview.
This method is called by {@link ItemDropInputMode#onDragEntered}. If a preview {@link ItemDropInputMode#showPreview shall be displayed}, the {@link ItemDropInputMode#previewGraph preview graph} is initialized and {@link ItemDropInputMode#populatePreviewGraph}
is called.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23initializePreview() Online Documentation}*/
initializePreview(): void;
/**
Initializes the snapping context.
This method is called by {@link ItemDropInputMode#onDragEntered}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23initializeSnapContext() Online Documentation}*/
initializeSnapContext(): void;
/**
Called once a drag has entered the canvas.
Calls {@link ItemDropInputMode#initializeSnapContext}, {@link ItemDropInputMode#initializePreview} and {@link ItemDropInputMode#initializeDropTarget}.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23onDragEntered(InputModeEventArgs) Online Documentation}*/
onDragEntered(evt: InputModeEventArgs): void;
/**
Called whenever a new item is created
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23onItemCreated(ItemEventArgs%3CT%3E) Online Documentation}*/
onItemCreated(evt: ItemEventArgs<T>): void;
/**
Subclasses shall fill the specified graph that is used to preview the dragged item.
@param previewGraph The preview graph to fill.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23populatePreviewGraph(IGraph) Online Documentation}*/
populatePreviewGraph(previewGraph: IGraph): void;
/**
Sets the {@link DropInputMode#setDragLocation drag location} and calls {@link ItemDropInputMode#updatePreview} to update the layout of item preview based on the mouse coordinates and
the results of the {@link ItemDropInputMode#snapContext}.
@param location
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23setDragLocation(Point) Online Documentation}*/
setDragLocation(location: Point): void;
/**
Calls {@link ItemDropInputMode#getDropTarget} for {@link ItemDropInputMode#snappedMousePosition} and sets the returned item as {@link ItemDropInputMode#dropTarget}.
If {@link ItemDropInputMode#highlightDropTarget} is enabled, the highlight is updated as well.
@param dragLocation The location to update the drop target for.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23updateDropTarget(Point) Online Documentation}*/
updateDropTarget(dragLocation: Point): void;
/**
Subclasses shall update the {@link ItemDropInputMode#previewGraph preview graph} so the dragged item is displayed at the specified <code>dragLocation</code>.
@param previewGraph The preview graph to update.
@param dragLocation The current drag location.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23updatePreview(IGraph%2CPoint) Online Documentation}*/
updatePreview(previewGraph: IGraph, dragLocation: Point): void;
/**
Adds the given listener for the <code>ItemCreated</code> event that occurs when a new item gets created by this input mode.
@param listener The listener to add.
@see ItemDropInputMode#removeItemCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23ItemCreated Online Documentation}*/
addItemCreatedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Removes the given listener for the <code>ItemCreated</code> event that occurs when a new item gets created by this input mode.
@param listener The listener to remove.
@see ItemDropInputMode#addItemCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23ItemCreated Online Documentation}*/
removeItemCreatedListener(listener: (sender: any, evt: ItemEventArgs<T>) => void): void;
/**
Gets or sets whether a preview of the dragged element is displayed during the drag
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23showPreview Online Documentation}*/
showPreview: boolean;
/**
Gets or sets whether dragged items are snapped.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23snappingEnabled Online Documentation}*/
snappingEnabled: boolean;
/**
Gets or sets the event recognizer that determines whether to disable snapping temporarily.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23disableSnappingRecognizer Online Documentation}*/
disableSnappingRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets whether the drop target is highlighted.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23highlightDropTarget Online Documentation}*/
highlightDropTarget: boolean;
/**
Gets or sets the {@link ItemDropInputMode#snapContext} which is used to snap the dragged element during the drag.
If set to <code>null</code> (the default) this input mode tries to obtain the {@link ItemDropInputMode#snapContext} from the {@link IInputModeContext}. To explicitly
disable snapping, a {@link ItemDropInputMode#snapContext} implementation that does nothing has to be set to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23snapContext Online Documentation}*/
snapContext: SnapContext | null;
/**
Gets the current snapped mouse position during drag operations.
The position is returned in world coordinates according to the {@link CanvasComponent} into which this input mode is
installed. If {@link ItemDropInputMode#snappingEnabled} is false, this value corresponds with {@link DropInputMode#mousePosition}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23snappedMousePosition Online Documentation}*/
snappedMousePosition: Point;
/**
Gets the drop target at {@link ItemDropInputMode#snappedMousePosition}
@see ItemDropInputMode#highlightDropTarget
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23dropTarget Online Documentation}*/
dropTarget: IModelItem | null;
/**
Gets the currently dragged item.
This implementation simply tries to cast the {@link DropInputMode#dropData} to the type of the dragged items.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23draggedItem Online Documentation}*/
draggedItem: T | null;
/**
Gets or sets the callback for item creation.
Called by method {@link ItemDropInputMode#onDragDropped}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23itemCreator Online Documentation}*/
itemCreator: ((context: IInputModeContext, graph: IGraph, dropData: any, dropTarget: IModelItem | null, dropLocation: Point) => T | null) | null;
/**
Gets the graph displayed as item preview.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemDropInputMode%23previewGraph Online Documentation}*/
readonly previewGraph: IGraph | null;
static $class: Class;
private $_ItemDropInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ItemDropInputMode<T>;
}
export interface LabelDropInputMode extends ItemDropInputMode<ILabel> {
}
/**
* An {@link ItemDropInputMode} specialized to the drag and drop of {@link ILabel}s.
* This input mode {@link ItemDropInputMode#showPreview can visualize a preview} of the {@link ItemDropInputMode#draggedItem dragged item} during the drag operation, and supports {@link ItemDropInputMode#snappingEnabled snapping} via the {@link ItemDropInputMode#snapContext}
* and {@link ItemDropInputMode#highlightDropTarget highlighting} of the {@link ItemDropInputMode#dropTarget} via a {@link HighlightIndicatorManager}.
*
* Typically the <code>LabelDropInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} and can be retrieved
* from the {@link GraphEditorInputMode#labelDropInputMode} property.
*
* <br><b>Getting the LabelDropInputMode from its parent input mode</b>
* <pre>
* const labelDropInputMode = mode.labelDropInputMode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode Online Documentation}
*/
export class LabelDropInputMode {
/**
Constructs a new instance of class {@link DropInputMode} for the expected data type.
Not used in this class since data type is always
{@link ILabel}. This constructor only exists for usage in child class.
@param expectedType
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23LabelDropInputMode(Array%3Cstring%3E) Online Documentation}*/
constructor(expectedType: string[]);
/**
Constructs a new instance of class {@link DropInputMode} for the expected data type.
Not used in this class since data type is always
{@link ILabel}. This constructor only exists for usage in child class.
@param expectedType
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23LabelDropInputMode(string) Online Documentation}*/
constructor(expectedType: string);
/**
Constructs a new instance of class {@link LabelDropInputMode} for the type {@link ILabel}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23LabelDropInputMode() Online Documentation}*/
constructor();
/**
Creates the label in the graph after it's been dropped.
This method is called by the {@link ItemDropInputMode#itemCreator} that is set as default on this class.
@param context The context for which the label should be created.
@param graph The {@link IGraph Graph} in which to create the label.
@param label The label that was dragged and should therefore be created.
@param dropTarget The {@link ILabelOwner} on which the label is dropped.
@param parameter The {@link ILabelModelParameter} of the new label.
@returns a newly created label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23createLabel(IInputModeContext%2CIGraph%2CILabel%2CILabelOwner%2CILabelModelParameter) Online Documentation}*/
createLabel(context: IInputModeContext, graph: IGraph | null, label: ILabel, dropTarget: ILabelOwner | null, parameter: ILabelModelParameter | null): ILabel | null;
/**
Looks for a {@link LabelDropInputMode#isValidLabelOwnerPredicate valid label owner} at the <code>dragLocation</code>.
@param dragLocation
@returns A {@link ILabelOwner label owner} or <code>null</code> if no valid label owner was found at the drag location.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23getDropTarget(Point) Online Documentation}*/
getDropTarget(dragLocation: Point): IModelItem | null;
/**
Get the layout the dragged label currently has.
@param dragLocation The current drag location
@param draggedLabel The dragged label.
@returns The layout for the dragged label
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23getLabelLayout(Point%2CILabel) Online Documentation}*/
getLabelLayout(dragLocation: Point, draggedLabel: ILabel): IOrientedRectangle;
/**
Determine the {@link ILabelModelParameter} the label should have if it were dropped at <code>draggedLocation</code> over <code>owner</code>.
If {@link LabelDropInputMode#useBestMatchingParameter} is <code>true</code>, this implementation returns just the {@link ILabel#layoutParameter} of the <code>draggedLabel</code> if it can be used for <code>owner</code> and <code>null</code> otherwise.
The <code>draggedLocation</code> is ignored in that case.
If {@link LabelDropInputMode#useBestMatchingParameter} is <code>false</code>, this implementation will use the model of the <code>draggedLabel</code> if it is compatible with <code>owner</code> to {@link ILabelModelParameterFinder#findBestParameter determine} the
parameter that corresponds best to the current <code>draggedLocation</code>.
@param owner The label owner where the label would be created. May be <code>null</code>.
@param draggedLabel The label that is currently dragged.
@param draggedLocation The current drag location.
@returns A {@link ILabelModelParameter} for <code>draggedLabel</code>, or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23getNewLabelModelParameter(ILabelOwner%2CILabel%2CPoint) Online Documentation}*/
getNewLabelModelParameter(owner: ILabelOwner | null, draggedLabel: ILabel, draggedLocation: Point): ILabelModelParameter | null;
/**
Default backing method for {@link LabelDropInputMode#isValidLabelOwnerPredicate}
This implementation checks whether a valid label model parameter could be found for the combination of <code>owner</code> and
<code>draggedLabel</code>, and whether the label has an {@link IEditLabelHelper} that forbids adding new labels.
@param owner The label owner where the label would be created. May be <code>null</code>.
@param draggedLabel The label that is currently dragged.
@returns <code>true</code> iff <code>owner</code> can be used to drop a label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23isValidLabelOwner(ILabelOwner%2CILabel) Online Documentation}*/
isValidLabelOwner(owner: ILabelOwner | null, draggedLabel: ILabel | null): boolean;
/**
Starts a drag and drop operation for a {@link ILabel}.
This method creates a {@link DragSource}, and then invokes {@link DragSource#startDrag} with a {@link DragDropItem}
that contains the <code>label</code> and the corresponding type.
If {@link LabelDropInputMode#useBestMatchingParameter} is
<code>false</code>, then the label needs a correct preferred size. Additionally, the label owner needs a correct size for label
parameters which consider the owner's geometry (e.g. stretch label models).
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param dragSource The source of the drag operation.
@param label The label to drag.
@param [dragDropEffects=DragDropEffects.ALL] The allowed drag drop effects.
@param [useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see DragSource#startDrag
@see DragDropItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23startDrag(Element%2CILabel%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
static startDrag(dragSource: HTMLElement | SVGElement, label: ILabel, dragDropEffects?: DragDropEffects | DragDropEffectsStringValues, useCssCursors?: boolean, dragPreview?: HTMLElement | SVGElement | null): DragSource;
/**
Starts a drag and drop operation for a {@link ILabel}.
This method creates a {@link DragSource}, and then invokes {@link DragSource#startDrag} with a {@link DragDropItem}
that contains the <code>label</code> and the corresponding type.
If {@link LabelDropInputMode#useBestMatchingParameter} is
<code>false</code>, then the label needs a correct preferred size. Additionally, the label owner needs a correct size for label
parameters which consider the owner's geometry (e.g. stretch label models).
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param options The parameters to pass.
@param options.dragSource The source of the drag operation.
@param options.label The label to drag.
@param [options.dragDropEffects=DragDropEffects.ALL] The allowed drag drop effects.
@param [options.useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [options.dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see DragSource#startDrag
@see DragDropItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23startDrag(Element%2CILabel%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
static startDrag(options: {
dragSource: HTMLElement | SVGElement;
label: ILabel;
dragDropEffects?: DragDropEffects | DragDropEffectsStringValues;
useCssCursors?: boolean;
dragPreview?: HTMLElement | SVGElement | null;
}): DragSource;
/**
Gets the currently dragged {@link IModelItem} instance.
This implementation simply tries to cast the {@link DropInputMode#dropData} to {@link ILabel}.
If native drag and drop is used, the value of this property is always <code>null</code>. This is because the protected mode prevents
access to the actual data during <code>dragover</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23draggedItem Online Documentation}*/
draggedItem: ILabel | null;
/**
Gets or sets the callback for testing whether an existing label owner in the graph can be used as owner for the
currently dragged label.
The default implementation checks whether a valid label model parameter could be found for the combination of label
owner and dragged label, and whether the label has an {@link IEditLabelHelper} that forbids adding new labels.
Note that during native drag and drop, {@link LabelDropInputMode#draggedItem} is always null and cannot be used for this check.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23isValidLabelOwnerPredicate Online Documentation}*/
isValidLabelOwnerPredicate: (arg1: ILabelOwner | null, arg2: ILabel | null) => boolean;
/**
Determines whether the default implementation of {@link LabelDropInputMode#getNewLabelModelParameter} should
directly use the parameter stored in the dragged item or whether the current drop location should be also taken into
account to find a parameter that best matches the drag location.
Default value is <code>false</code>, which means to use the parameter directly
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23useBestMatchingParameter Online Documentation}*/
useBestMatchingParameter: boolean;
/**
Whether to automatically raise the {@link ICommand#EDIT_LABEL} command after a label has been created.
This property is disabled by default
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelDropInputMode%23autoEditLabel Online Documentation}*/
autoEditLabel: boolean;
static $class: Class;
private $_LabelDropInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelDropInputMode;
}
export interface LabelPositionHandler extends YObject, IPositionHandler {
/**
@inheritdoc*/
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
/**
@inheritdoc*/
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
initializeDrag(context: IInputModeContext): void;
/**
@inheritdoc*/
readonly location: IPoint;
} /**
An implementation of {@link IPositionHandler} that shows the various label position candidates and lets the user
move a label to one of those candidate positions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler Online Documentation}*/
export class LabelPositionHandler {
/**
Initializes a new instance of the {@link LabelPositionHandler} class.
@param label The label that shall be moved.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler Online Documentation}*/
constructor(label: ILabel);
/**
Creates an {@link ICanvasObjectDescriptor} to display the given candidate position.
@param context The context for which the descriptor is created.
@param highlight Whether to highlight the candidate. This will be <code>true</code> for the current position and <code>false</code> for the others.
@returns An implementation that will render the given candidate.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23createCandidateDescriptor(IInputModeContext%2Cboolean) Online Documentation}*/
createCandidateDescriptor(context: IInputModeContext, highlight: boolean): ICanvasObjectDescriptor;
/**
Calculates a distance value between a candidate rectangle and the mouse location.
@param rectangle The rectangle candidate.
@param location The mouse location.
@returns A value indicating the closeness of the mouse to the rectangle.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23getDistance(IOrientedRectangle%2CPoint) Online Documentation}*/
getDistance(rectangle: IOrientedRectangle, location: Point): number;
/**
Gets the candidates for the given label.
@param label The label.
@returns A possibly empty enumerator over possible label parameter candidates.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23getParameterCandidates(ILabel) Online Documentation}*/
getParameterCandidates(label: ILabel): IEnumerable<ILabelModelParameter>;
/**
Actually applies the given parameter at the end of the gesture.
@param label The label to set the parameter for.
@param layoutParameter The new parameter.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23setLayoutParameter(ILabel%2CILabelModelParameter) Online Documentation}*/
setLayoutParameter(label: ILabel, layoutParameter: ILabelModelParameter): void;
/**
Called by clients to set the position to the given coordinates.
@param location The new location.
@see IDragHandler#location
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23setPosition(Point) Online Documentation}*/
setPosition(location: Point): void;
/**
Determines whether to use a {@link ILabelModelParameterFinder} to find the best candidate.
This implementation returns <code>true</code> if the {@link LabelPositionHandler#useFinder} property is <code>true</code> and the control key is pressed.
@param context The context that is currently being used - may be <code>null</code> if the method is called without context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23useParameterFinder(IInputModeContext) Online Documentation}*/
useParameterFinder(context: IInputModeContext | null): boolean;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} that will be used to represent the rectangular selection.
The default template has the <code>yfiles-label-position-candidate-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23CANDIDATE_TEMPLATE_KEY Online Documentation}*/
static CANDIDATE_TEMPLATE_KEY: string;
/**
A {@link string} that will be used to find the {@link IVisualTemplate} that will be used to represent the rectangular selection.
The default template has the <code>yfiles-label-position-highlight-template</code> CSS class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23HIGHLIGHT_TEMPLATE_KEY Online Documentation}*/
static HIGHLIGHT_TEMPLATE_KEY: string;
/**
Gets or sets the template to use for showing candidates.
This property is not used if the {@link LabelPositionHandler#visualization} is set to {@link Visualization#NONE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23candidateTemplate Online Documentation}*/
candidateTemplate: IVisualTemplate;
/**
Gets or sets the template to use for showing highlighted candidates.
This property is only used if the {@link LabelPositionHandler#visualization} is set to {@link Visualization#GHOST}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23highlightTemplate Online Documentation}*/
highlightTemplate: IVisualTemplate;
/**
Gets the graph instance from the {@link IInputModeContext} that has been passed to the {@link LabelPositionHandler#initializeDrag}
method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23graph Online Documentation}*/
readonly graph: IGraph | null;
/**
Gets the label instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23label Online Documentation}*/
readonly label: ILabel;
/**
Gets or sets a property that determines whether the handler may use a {@link ILabelModelParameterFinder} to assign
arbitrary positions.
The default value is <code>true</code>
@see LabelPositionHandler#useParameterFinder
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23useFinder Online Documentation}*/
useFinder: boolean;
/**
Gets or sets how the moved label should be shown during the gesture.
This property must not be changed during the gesture.
Default is {@link Visualization#DUMMY}.
@default Visualization.DUMMY
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPositionHandler%23visualization Online Documentation}*/
visualization: Visualization;
static $class: Class;
private $_LabelPositionHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelPositionHandler;
}
export interface LabelSnapContextHelper extends YObject, ILabelSnapContextHelper {
/**
@inheritdoc*/
collectSnapResults(context: LabelSnapContext, evt: CollectSnapResultsEventArgs, suggestedLayout: IOrientedRectangle, label: ILabel): void;
} /**
The default label snap context helper that provides the snap lines and the snap results to the {@link LabelSnapContext}
during dragging of labels.
@see LabelSnapContext
@see LabelDecorator#labelSnapContextHelperDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContextHelper Online Documentation}*/
export class LabelSnapContextHelper {
/**
Adds snap lines to the <code>snapContext</code> that are parallel to the path segments of the owner edge at the given distance.
This method is not used by this class to create its snap lines. Instead, it can be called by custom sub-classes to
create this kind of snap lines.
@param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
the context explicitly.
@param inputModeContext The context of the input mode that handles the dragging.
@param label The edge label that is dragged.
@param distance The distance of the added snap lines from the edge path.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContextHelper%23addEdgePathSnapLines(LabelSnapContext%2CIInputModeContext%2CILabel%2Cnumber) Online Documentation}*/
addEdgePathSnapLines(snapContext: LabelSnapContext, inputModeContext: IInputModeContext, label: ILabel, distance: number): void;
/**
Adds snap lines to the <code>snapContext</code> that are parallel to the path segments of the owner edge at particular distances.
The corresponding settings of the <code>snapContext</code> specify which snap lines are actually created.
@param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
the context explicitly.
@param inputModeContext The context of the input mode that handles the dragging.
@param label The edge label that is dragged.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContextHelper%23addEdgePathSnapLines(LabelSnapContext%2CIInputModeContext%2CILabel) Online Documentation}*/
addEdgePathSnapLines(snapContext: LabelSnapContext, inputModeContext: IInputModeContext, label: ILabel): void;
/**
Adds two snap line for the initial center location of the label, one in the direction of the up vector, the other
orthogonal to that direction.
This method is called by {@link LabelSnapContextHelper#addSnapLines} to add the snap lines for the initial position
label location.
@param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
the context explicitly.
@param inputModeContext The context of the input mode that handles the dragging.
@param label The label that is dragged.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContextHelper%23addInitialLocationSnapLines(LabelSnapContext%2CIInputModeContext%2CILabel) Online Documentation}*/
addInitialLocationSnapLines(snapContext: LabelSnapContext, inputModeContext: IInputModeContext, label: ILabel): void;
/**
Adds snap lines to the <code>snapContext</code> that are parallel to the owner node's border at particular distances.
The corresponding settings of the <code>snapContext</code> specify which snap lines are actually created.
@param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
the context explicitly.
@param inputModeContext The context of the input mode that handles the dragging.
@param label The node label that is dragged.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContextHelper%23addNodeShapeSnapLines(LabelSnapContext%2CIInputModeContext%2CILabel) Online Documentation}*/
addNodeShapeSnapLines(snapContext: LabelSnapContext, inputModeContext: IInputModeContext, label: ILabel): void;
/**
Adds horizontal and vertical snap lines to the <code>snapContext</code> that are originating from the label's owner port if the
label's owner is a port.
Snap lines are only created if the label's owner is a {@link IPort} and {@link LabelSnapContext#collectPortSnapLines} of the <code>snapContext</code> is set tot <code>true</code>.
@param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
the context explicitly.
@param inputModeContext The context of the input mode that handles the dragging.
@param label The port label that is dragged.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContextHelper%23addPortSnapLines(LabelSnapContext%2CIInputModeContext%2CILabel) Online Documentation}*/
addPortSnapLines(snapContext: LabelSnapContext, inputModeContext: IInputModeContext, label: ILabel): void;
/**
Called during {@link IDragHandler#initializeDrag initialization} of a <code>label</code> dragging to add {@link SnapLine snap lines} to which the label can potentially snap to the
<code>snapContext</code>.
@param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
the context explicitly.
@param inputModeContext The context of the input mode that handles the dragging.
@param label The label that is dragged.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContextHelper%23addSnapLines(LabelSnapContext%2CIInputModeContext%2CILabel) Online Documentation}*/
addSnapLines(snapContext: LabelSnapContext, inputModeContext: IInputModeContext, label: ILabel): void;
/**
Returns the static shared instance of this class.
Since this implementation does not carry any state, this instance can safely be used even in multi-threaded
environments.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSnapContextHelper%23INSTANCE Online Documentation}*/
static INSTANCE: LabelSnapContextHelper;
static $class: Class;
private $_LabelSnapContextHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelSnapContextHelper;
}
export interface LabelTextValidatingEventArgs extends InputModeEventArgs {
}
/**
Event arguments for the {@link GraphEditorInputMode#addValidateLabelTextListener ValidateLabelText}
event.
Event handlers can set the {@link LabelTextValidatingEventArgs#cancel} property to <code>true</code> to cancel label editing if validation failed, or set the {@link LabelTextValidatingEventArgs#newText} property to a
changed, validated value.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelTextValidatingEventArgs Online Documentation}*/
export class LabelTextValidatingEventArgs {
/**
Initializes a new instance of the {@link LabelTextValidatingEventArgs} class.
@param context The context.
@param label The label.
@param newText The initial new text as entered by the user.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelTextValidatingEventArgs Online Documentation}*/
constructor(context: IInputModeContext, label: ILabel, newText: string);
/**
* Gets the label that is being edited.
* Note that the label might not belong to a graph if it is a dummy for a label that is about to be created.
*
* Labels with tag "int label" should be rejected if their text cannot be parsed into an integer value:
*
* <pre>
* mode.addValidateLabelTextListener((o, args) => {
*   if (args.label.tag === 'int label') {
*     args.cancel = Number.isNaN(parseInt(args.newText))
*   }
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelTextValidatingEventArgs%23label Online Documentation}
*/
label: ILabel;
/**
* Gets or sets the new text to use for the label.
* Labels with tag "int label" should be rejected if their text cannot be parsed into an integer value:
*
* <pre>
* mode.addValidateLabelTextListener((o, args) => {
*   if (args.label.tag === 'int label') {
*     args.cancel = Number.isNaN(parseInt(args.newText))
*   }
* })
* </pre>
*
* @see LabelTextValidatingEventArgs#cancel
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelTextValidatingEventArgs%23newText Online Documentation}
*/
newText: string;
/**
* Gets or sets a value indicating whether the edit should be canceled.
* Labels with tag "int label" should be rejected if their text cannot be parsed into an integer value:
*
* <pre>
* mode.addValidateLabelTextListener((o, args) => {
*   if (args.label.tag === 'int label') {
*     args.cancel = Number.isNaN(parseInt(args.newText))
*   }
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelTextValidatingEventArgs%23cancel Online Documentation}
*/
cancel: boolean;
static $class: Class;
private $_LabelTextValidatingEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelTextValidatingEventArgs;
}
export interface MoveLabelInputMode extends MoveInputMode {
}
/**
* An {@link IInputMode} that can be used to drag a {@link ILabel} in a {@link GraphComponent}.
* This implementation shows the possible candidate positions for a given label and allows the user to drag the label to
* one of these positions.
*
* Note that the {@link MoveInputMode#positionHandler} property is not used by this instance. Developers who want to set their own {@link IPositionHandler}
* implementations have to either decorate the label's {@link ILookup} or override
* {@link MoveLabelInputMode#createLabelPositionHandler}.
*
* Typically the <code>MoveLabelInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} and can be retrieved
* from the {@link GraphEditorInputMode#moveLabelInputMode} property.
*
* <br><b>Getting the MoveLabelInputMode from its parent input mode</b>
* <pre>
* const moveLabelInputMode = mode.moveLabelInputMode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode Online Documentation}
*/
export class MoveLabelInputMode {
/**
Creates a new instance that will move labels in the graph retrieved from the {@link MoveInputMode#inputModeContext}.
@see MoveLabelInputMode#graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode Online Documentation}*/
constructor();
/**
Creates the {@link IPositionHandler position handler} for the given label used in {@link MoveLabelInputMode#isValidLabelHit}.
First, this implementation checks for a position handler in the label's
{@link ILookup lookup}. If this fails, it creates a new label position handler for the given label.
@param label The label to create a position handler for.
@returns A position handler for the given label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23createLabelPositionHandler(ILabel) Online Documentation}*/
createLabelPositionHandler(label: ILabel): IPositionHandler;
/**
Looks for a label that has been hit at the specified position.
If {@link MoveLabelInputMode#allowUnselected moving unselected labels are allowed}, this implementation enumerates the labels at the given <code>location</code>. The first
label where {@link MoveLabelInputMode#shouldMove} returns <code>true</code> is returned. If there is no such label, <code>null</code> is
returned.
If {@link MoveLabelInputMode#allowUnselected only selected labels can be moved}, this implementation checks the {@link MoveLabelInputMode#graphSelection} to see if there is exactly one selected label.
This label is returned if it is hit by the mouse and {@link MoveLabelInputMode#shouldMove} returns <code>true</code>. Otherwise,
<code>null</code> is returned.
@param context The context for the hit test.
@param location The coordinates for the hit test.
@returns The label hit at the given location or <code>null</code>.
@see MoveLabelInputMode#isValidLabelHit
@see MoveLabelInputMode#shouldMove
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23getHitLabel(IInputModeContext%2CPoint) Online Documentation}*/
getHitLabel(context: IInputModeContext, location: Point): ILabel | null;
/**
Determines whether or not a valid label has been hit at the specified position.
This implementation calls {@link MoveLabelInputMode#getHitLabel} to find the label at the specified location. If a
label is hit, the {@link IPositionHandler} is created by
{@link MoveLabelInputMode#createLabelPositionHandler}.
@param context The context for the hit test.
@param location The coordinates for the hit test.
@returns <code>true</code> if a valid label is hit and the label moving should be started here. Otherwise, <code>false</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23isValidLabelHit(IInputModeContext%2CPoint) Online Documentation}*/
isValidLabelHit(context: IInputModeContext, location: Point): boolean;
/**
Clears the {@link MoveInputMode#positionHandler} property.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23onDragCanceled(InputModeEventArgs) Online Documentation}*/
onDragCanceled(evt: InputModeEventArgs): void;
/**
Clears the {@link MoveInputMode#positionHandler} property.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23onDragFinished(InputModeEventArgs) Online Documentation}*/
onDragFinished(evt: InputModeEventArgs): void;
/**
Registers the {@link MoveInputMode#positionHandler} created during the last {@link MoveLabelInputMode#createLabelPositionHandler} call.
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23onDragStarting(InputModeEventArgs) Online Documentation}*/
onDragStarting(evt: InputModeEventArgs): void;
/**
Determines whether or not a label is allowed to be moved.
This implementation returns <code>false</code> if the parent input mode {@link GraphEditorInputMode#shouldMove forbids} moving the label. Overriding implementations should
query this implementation as base if this feature is desired.
@param context The context.
@param label The label to be checked.
@returns <code>true</code> if the label is allowed to move.
@see MoveLabelInputMode#getHitLabel
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23shouldMove(IInputModeContext%2CILabel) Online Documentation}*/
shouldMove(context: IInputModeContext, label: ILabel): boolean;
/**
Gets or sets a property that determines whether or not moving unselected labels is allowed.
The default value is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23allowUnselected Online Documentation}*/
allowUnselected: boolean;
/**
Gets or sets a value indicating whether or not using the {@link ILabelModelParameterFinder} to assign new label
model parameters is allowed.
The default is <code>true</code>
This property has no effect if an {@link IPositionHandler} is available in the label's {@link ILookup lookup}.
@see MoveLabelInputMode#createLabelPositionHandler
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23useLabelModelParameterFinder Online Documentation}*/
useLabelModelParameterFinder: boolean;
/**
Gets or sets how the moved label should be shown during the gesture.
Changing this property during a gesture will have no effect.
Default is {@link Visualization#DUMMY}.
This property has no effect if an {@link IPositionHandler} is available in the label's {@link ILookup lookup}.
@default Visualization.DUMMY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23visualization Online Documentation}*/
visualization: Visualization;
/**
Gets the label that is currently moved or <code>null</code> if there is no such label.
The value of this property is not <code>null</code> only if this mode is currently active and performs a move operation, that is
between the {@link MoveInputMode#addDragStartingListener DragStarting} and the {@link MoveInputMode#addDragFinishedListener DragFinished}
event handlers.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23movedLabel Online Documentation}*/
movedLabel: ILabel | null;
/**
Gets the {@link IGraphSelection} this mode is acting upon.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23graphSelection Online Documentation}*/
readonly graphSelection: IGraphSelection | null;
/**
Gets the {@link IGraph} this mode is acting upon.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MoveLabelInputMode%23graph Online Documentation}*/
readonly graph: IGraph | null;
static $class: Class;
private $_MoveLabelInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MoveLabelInputMode;
}
export interface NodeDropInputMode extends ItemDropInputMode<INode> {
}
/**
* An {@link ItemDropInputMode} specialized to the drag and drop of {@link INode}s.
* By default, this input mode copies the layout, style, tag, ports, and labels of the {@link ItemDropInputMode#draggedItem dragged item} to {@link ItemDropInputMode#itemCreator the new node}.
*
* It {@link ItemDropInputMode#showPreview can visualize a preview} of the {@link ItemDropInputMode#draggedItem dragged item} during the drag operation, and supports {@link ItemDropInputMode#snappingEnabled snapping} via the {@link ItemDropInputMode#snapContext} and {@link ItemDropInputMode#highlightDropTarget highlighting}
* of the {@link ItemDropInputMode#dropTarget} via a {@link HighlightIndicatorManager}. In addition, it supports dragging nodes into {@link IGraph#isGroupNode groups} and, optionally, {@link IFoldingView#collapse folders}. In this case the {@link NodeDropInputMode#isValidParentPredicate} will be
* queried if the dropped upon node is actually a valid parent.
*
* Typically the <code>NodeDropInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} and can be retrieved
* from the {@link GraphEditorInputMode#nodeDropInputMode} property.
*
* <br><b>Getting the NodeDropInputMode from its parent input mode</b>
* <pre>
* const nodeDropInputMode = mode.nodeDropInputMode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode Online Documentation}
*/
export class NodeDropInputMode {
/**
Initializes a new instance of the {@link NodeDropInputMode} class for the expected data type.
@param expectedType A list of types that are used in {@link NodeDropInputMode#startDrag}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23NodeDropInputMode(Array%3Cstring%3E) Online Documentation}*/
constructor(expectedType: string[]);
/**
Initializes a new instance of the {@link NodeDropInputMode} class for the expected data type.
@param expectedType The type that is used in {@link NodeDropInputMode#startDrag}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23NodeDropInputMode(string) Online Documentation}*/
constructor(expectedType: string);
/**
Initializes a new instance of the {@link NodeDropInputMode} class for {@link INode}s.
This instance fires events only if the type of the data of the drag operation is the full name of the {@link INode}
type. Such a drag operation can be started for example with {@link NodeDropInputMode#startDrag}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23NodeDropInputMode() Online Documentation}*/
constructor();
/**
Creates the node in the graph after it's been dropped.
This method is called by the {@link ItemDropInputMode#itemCreator} that is set as default on this class.
@param context The context for which the node should be created.
@param graph The {@link IGraph Graph} in which to create the item.
@param node The node that was dragged and should therefore be created.
@param dropTarget The {@link IModelItem} on which the node is dropped.
@param layout The bounds of the new node.
@returns a newly created node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23createNode(IInputModeContext%2CIGraph%2CINode%2CIModelItem%2CRect) Online Documentation}*/
createNode(context: IInputModeContext, graph: IGraph, node: INode, dropTarget: IModelItem | null, layout: Rect): INode | null;
/**
Calculates the layout of the new node.
In this implementation the mouse location is used as center of the node. Can be overridden in child class to implement a
different layout.
@param mouseLocation Current mouse position
@param size Size of the node
@returns a {@link Rect} with the given size and the mouse location as center.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23getNodeLayout(Point%2CSize) Online Documentation}*/
getNodeLayout(mouseLocation: Point, size: Size): Rect;
/**
Starts a drag and drop operation for a {@link INode}.
This method creates a {@link DragSource}, and then invokes {@link DragSource#startDrag} with a {@link DragDropItem}
that contains the <code>node</code> and the corresponding type.
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param dragSource The source of the drag operation.
@param node The node to drag.
@param [dragDropEffects=DragDropEffects.ALL] The allowed drag drop effects.
@param [useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see DragSource#startDrag
@see DragDropItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23startDrag(Element%2CINode%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
static startDrag(dragSource: HTMLElement | SVGElement, node: INode, dragDropEffects?: DragDropEffects | DragDropEffectsStringValues, useCssCursors?: boolean, dragPreview?: HTMLElement | SVGElement | null): DragSource;
/**
Starts a drag and drop operation for a {@link INode}.
This method creates a {@link DragSource}, and then invokes {@link DragSource#startDrag} with a {@link DragDropItem}
that contains the <code>node</code> and the corresponding type.
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param options The parameters to pass.
@param options.dragSource The source of the drag operation.
@param options.node The node to drag.
@param [options.dragDropEffects=DragDropEffects.ALL] The allowed drag drop effects.
@param [options.useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [options.dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see DragSource#startDrag
@see DragDropItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23startDrag(Element%2CINode%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
static startDrag(options: {
dragSource: HTMLElement | SVGElement;
node: INode;
dragDropEffects?: DragDropEffects | DragDropEffectsStringValues;
useCssCursors?: boolean;
dragPreview?: HTMLElement | SVGElement | null;
}): DragSource;
/**
Gets or sets the callback for testing whether a {@link ItemDropInputMode#draggedItem dragged node} should be created as a {@link IGraph#isGroupNode group node}.
The default implementation returns <code>false</code> always. Can be set to a different value to implement custom group node
detection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23isGroupNodePredicate Online Documentation}*/
isGroupNodePredicate: (obj: INode) => boolean;
/**
Gets or sets the callback for testing whether an existing node in the graph can be used as a valid parent node in the
context of grouping.
The default implementation returns <code>true</code> always. Can be set to a different value to implement custom group node
detection. Note that the node passed to the predicate may be <code>null</code> to indicate that the node will be placed at the root
of the hierarchy.
@see NodeDropInputMode#allowNonGroupNodeAsParent
@see NodeDropInputMode#allowFolderNodeAsParent
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23isValidParentPredicate Online Documentation}*/
isValidParentPredicate: (obj: INode) => boolean;
/**
Gets or sets a value indicating whether nodes can be dropped on {@link IFoldingView#collapse collapsed} folder nodes.
If this property is set to <code>true</code>, dropping nodes on collapsed folder nodes will create the node inside the folder node in
the master graph. In that case the {@link ItemDropInputMode#addItemCreatedListener ItemCreated} event will yield the node in the {@link FoldingManager#masterGraph master graph}. The node will
not be {@link IGraph#contains contained} in the currently visible graph. By default this feature is disabled. In any case the {@link NodeDropInputMode#isValidParentPredicate} will be queried
additionally if the node is a valid parent.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23allowFolderNodeAsParent Online Documentation}*/
allowFolderNodeAsParent: boolean;
/**
Gets or sets a value indicating whether nodes can be dropped onto nodes that are no {@link IGraph#isGroupNode group nodes}.
If this property is enabled, dropping a node on a non-group node will effectively convert the non-group node to a real
group node. By default this feature is disabled. In any case the {@link NodeDropInputMode#isValidParentPredicate} will be queried additionally if the node is a valid
parent.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23allowNonGroupNodeAsParent Online Documentation}*/
allowNonGroupNodeAsParent: boolean;
/**
Gets the currently dragged {@link IModelItem} instance.
This implementation simply tries to cast the {@link DropInputMode#dropData} to {@link INode}.
If native drag and drop is used, the value of this property is always <code>null</code>. This is because the protected mode prevents
access to the actual data during <code>dragover</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDropInputMode%23draggedItem Online Documentation}*/
draggedItem: INode | null;
static $class: Class;
private $_NodeDropInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeDropInputMode;
}
export interface NodeReshapeHandleProvider extends ReshapeHandleProviderBase {
}
/**
* Default implementation of the {@link IReshapeHandleProvider} interface for nodes.
* The actual reshaping is delegated to an {@link IReshapeHandler}.
*
* <br><b>Decorate lookup to customize node resizing</b>
* <pre>
* // decorate reshape handle provider
* const nodeDecorator = graphComponent.graph.decorator.nodeDecorator
* nodeDecorator.reshapeHandleProviderDecorator.setFactory(node => {
*   // lookup the reshape handler that performs the actual resizing
*   /** @type {IReshapeHandler} *‍/
*   const reshapeHandler = node.lookup(IReshapeHandler.$class)
*   // create a provider for handles at the node corners
*   const handleProvider = new NodeReshapeHandleProvider(
*     node,
*     reshapeHandler,
*     HandlePositions.CORNERS
*   )
*   // use a different reshape policy
*   handleProvider.reshapePolicy = ReshapePolicy.MAXIMUM
*   return handleProvider
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandleProvider Online Documentation}
*/
export class NodeReshapeHandleProvider {
/**
Creates a default instance that provide handles for each of the <code>handlePositions</code> given for the <code>node</code>.
@param node The node to provide reshape handles for.
@param reshapeHandler The reshape handler to delegate the actual reshaping to.
@param handlePositions A bitwise combination of all handle positions this instance should provide a handle for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandleProvider Online Documentation}*/
constructor(node: INode, reshapeHandler: IReshapeHandler, handlePositions: HandlePositions | HandlePositionsConvertible);
/**
Provides a {@link NodeReshapeHandlerHandle} that uses the {@link NodeReshapeHandleProvider#reshapeHandler} to perform the actual reshaping.
The {@link ReshapeHandleProviderBase#minimumSize}, {@link ReshapeHandleProviderBase#maximumSize}, {@link ReshapeHandleProviderBase#minimumEnclosedArea}, {@link ReshapeHandleProviderBase#maximumBoundingArea}, {@link NodeReshapeHandleProvider#ratioReshapeRecognizer} and {@link NodeReshapeHandleProvider#centerReshapeRecognizer} are propagated to the {@link NodeReshapeHandlerHandle}.
The {@link NodeReshapeHandleProvider#reshapePolicy} is only propagated if the <code>position</code> is neither {@link HandlePositions#HORIZONTAL} nor
{@link HandlePositions#VERTICAL}. If <code>position</code> is {@link HandlePositions#HORIZONTAL}, {@link ReshapePolicy#HORIZONTAL}
is set. If <code>position</code> is {@link HandlePositions#VERTICAL}, {@link ReshapePolicy#VERTICAL} is set.
This method may be called for each possible single position contained in the set as returned by
{@link IReshapeHandleProvider#getAvailableHandles}.
@param inputModeContext The context for which the handles are queried.
@param position The position to provide an instance for.
@returns A handle implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandleProvider%23getHandle(IInputModeContext%2CHandlePositions) Online Documentation}*/
getHandle(inputModeContext: IInputModeContext, position: HandlePositions | HandlePositionsConvertible): IHandle;
/**
Gets the node to provide reshape handles for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandleProvider%23node Online Documentation}*/
readonly node: INode;
/**
Gets the reshape handler to delegate the actual reshaping to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandleProvider%23reshapeHandler Online Documentation}*/
readonly reshapeHandler: IReshapeHandler;
/**
* Gets or sets the event recognizer that determines if the width/height ratio of the reshapeable is kept during the
* reshape gesture.
* The default value is {@link KeyEventRecognizers#SHIFT_IS_DOWN}.
*
* <br><b>Always keeping the node ratio</b>
* <pre>
* // always keep the aspect ratio
* provider.ratioReshapeRecognizer = EventRecognizers.ALWAYS
* </pre>
*
* @see NodeReshapeHandleProvider#centerReshapeRecognizer
* @see NodeReshapeHandleProvider#reshapePolicy
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandleProvider%23ratioReshapeRecognizer Online Documentation}
*/
ratioReshapeRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
* Gets or sets the event recognizer that determines if the center of the reshapeable is kept during the reshape gesture.
* The default value is {@link KeyEventRecognizers#ALT_IS_DOWN}.
*
* <br><b>Always keeping the node center</b>
* <pre>
* // always keep the center of the node
* provider.centerReshapeRecognizer = EventRecognizers.ALWAYS
* </pre>
*
* @see NodeReshapeHandleProvider#ratioReshapeRecognizer
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandleProvider%23centerReshapeRecognizer Online Documentation}
*/
centerReshapeRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
* Gets or sets the policy that determines how the mouse location is interpreted when the reshape gesture shall keep the
* aspect ratio.
* The {@link NodeReshapeHandleProvider#ratioReshapeRecognizer} is used to determine whether the aspect ratio shall be kept at all.
*
* {@link NodeReshapeHandleProvider#getHandle} propagates this policy value to the {@link ReshapeHandlerHandle#reshapePolicy} of its
* {@link HandlePositions#NORTH_WEST}, {@link HandlePositions#NORTH_EAST}, {@link HandlePositions#SOUTH_WEST}
* and {@link HandlePositions#SOUTH_EAST} handles. For {@link HandlePositions#WEST} and {@link HandlePositions#EAST}
* handles, {@link ReshapePolicy#HORIZONTAL} is used. For {@link HandlePositions#NORTH} and {@link HandlePositions#SOUTH}
* handles, {@link ReshapePolicy#VERTICAL} is used.
*
* The default value is {@link ReshapePolicy#PROJECTION}.
*
* <br><b>Extend the node bounds to the mouse location</b>
* <pre>
* // when keeping the aspect ratio extend the node bounds to the mouse location
* provider.reshapePolicy = ReshapePolicy.MAXIMUM
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandleProvider%23reshapePolicy Online Documentation}
*/
reshapePolicy: ReshapePolicy;
static $class: Class;
private $_NodeReshapeHandleProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeReshapeHandleProvider;
}
export interface NodeReshapeHandlerHandle extends ReshapeHandlerHandle {
/**
@inheritdoc*/
cancelDrag(inputModeContext: IInputModeContext, originalLocation: Point): void;
/**
@inheritdoc*/
dragFinished(inputModeContext: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
handleMove(inputModeContext: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
initializeDrag(inputModeContext: IInputModeContext): void;
} /**
* The default {@link IHandle} implementation for nodes that uses {@link IReshapeHandler} for the actual
* reshaping.
* In addition to the {@link ReshapeHandlerHandle} this implementation supports snapping if the node provides an {@link INodeReshapeSnapResultProvider}
* in its lookup and an active {@link GraphSnapContext} is found.
*
* Furthermore orthogonal edges attached to the node are kept orthogonal during the reshape gesture if an active {@link OrthogonalEdgeEditingContext}
* is found.
*
* <br><b>Provider with custom settings for south east handle</b>
* <pre>
* class MyNodeReshapeHandleProvider extends NodeReshapeHandleProvider {
*   /**
*    * @param {INode} node
*    * @param {IReshapeHandler} reshapeHandler
*    *‍/
*   constructor(node, reshapeHandler) {
*     super(node, reshapeHandler, HandlePositions.CORNERS)
*   }
*
*   /**
*    * @param {IInputModeContext} context
*    * @param {HandlePositions} position
*    * @returns {IHandle}
*    *‍/
*   getHandle(inputModeContext, position) {
*     const handle = new NodeReshapeHandlerHandle(this.node, this.reshapeHandler, position)
*     if (position === HandlePositions.SOUTH_EAST) {
*       // south east handle should always keep aspect ratio
*       handle.reshapePolicy = ReshapePolicy.PROJECTION
*       handle.ratioReshapeRecognizer = EventRecognizers.ALWAYS
*     }
*     return handle
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandlerHandle Online Documentation}
*/
export class NodeReshapeHandlerHandle {
/**
Default constructor that creates a handle for the given position to reshape the <code>node</code> using the given <code>reshapeHandler</code>.
@param node The node to reshape with this handle.
@param reshapeHandler The reshapeable to write the changes to.
@param position The position of the handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandlerHandle Online Documentation}*/
constructor(node: INode, reshapeHandler: IReshapeHandler, position: HandlePositions | HandlePositionsConvertible);
/**
Gets the node to reshape with this handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeHandlerHandle%23node Online Documentation}*/
readonly node: INode;
static $class: Class;
private $_NodeReshapeHandlerHandle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeReshapeHandlerHandle;
}
export interface OrthogonalEdgeEditingContext extends YObject {
}
/**
Used to interactively reshape orthogonal edge paths.
This class will be queried from the {@link IInputModeContext} of implementations of {@link IPositionHandler},
{@link IHandle}, and {@link IReshapeHandler} that support orthogonal editing of edges.
In a first step, the edit (which is a movement of one or more {@link IPort}s, {@link IBend}s, {@link IEdge edge ends}, or
{@link IPortOwner}s) is being {@link OrthogonalEdgeEditingContext#initializeDrag initialized} before anything has been changed. Then, client code that seeks to modify
the graph, uses the methods {@link OrthogonalEdgeEditingContext#addExplicitlyMovedBend},
{@link OrthogonalEdgeEditingContext#addMovedEdgeEnd}, {@link OrthogonalEdgeEditingContext#addMovedPort}, and
{@link OrthogonalEdgeEditingContext#addTransformedPortOwner}. In the next step, the {@link OrthogonalEdgeEditingContext#dragInitialized drag is being initialized} and
all affected edges will be {@link OrthogonalEdgeEditingContext#prepareEdgePaths prepared} so that the editing will not destroy the orthogonality. Interested editors now get
the chance to register and negotiate {@link OrthogonalEdgeEditingContext#addImplicitlyMovedBend implicitly moved} bends. Then the actual editing is performed and the edit is either
{@link OrthogonalEdgeEditingContext#cancelDrag canceled}, in which case the {@link OrthogonalEdgeEditingContext#removeAddedBends added bends are removed} or the edit is {@link OrthogonalEdgeEditingContext#dragFinished finished} and the affected edges are being
{@link OrthogonalEdgeEditingContext#cleanUp cleaned up}.
@see IOrthogonalEdgeHelper
@see OrthogonalEdgeEditingContext#enabled
@see OrthogonalEdgeEditingContext#movePorts
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext Online Documentation}*/
export class OrthogonalEdgeEditingContext {
/**
Initializes a new instance of the {@link OrthogonalEdgeEditingContext} class that is initially {@link OrthogonalEdgeEditingContext#enabled}.
@param [options] The parameters to pass.
@param [options.enabled] A value indicating whether orthogonal edge editing is enabled at all. This option sets the {@link OrthogonalEdgeEditingContext#enabled} property on the created object.
@param [options.movePorts] A value indicating ports are allowed to be moved at all. This option sets the {@link OrthogonalEdgeEditingContext#movePorts} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext Online Documentation}*/
constructor(options?: {
enabled?: boolean;
movePorts?: boolean;
});
/**
Registers the provided bend with this instance so that it will be considered explicitly moved for this edit.
{@link IDragHandler} implementations that are being used to explicitly move a bend during the edit should use this
method to register their edit while they are being {@link IDragHandler#initializeDrag initialized}.
@param movementInfo The movement info that describes the movement of the bend.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23addExplicitlyMovedBend(MovementInfo) Online Documentation}*/
addExplicitlyMovedBend(movementInfo: MovementInfo): void;
/**
Registers the provided bend as being moved implicitly for this edit.
The instance will be returned by {@link OrthogonalEdgeEditingContext#createImplicitlyMovedBendInfo} and the {@link MovementInfo#moveType} will
be combined with the requested move type.
@param movementInfo The movement info for the bend.
@returns The info that has been registered with this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23addImplicitlyMovedBend(MovementInfo) Online Documentation}*/
addImplicitlyMovedBend(movementInfo: MovementInfo): MovementInfo;
/**
Registers the provided end of the edge with this instance so that it will be considered explicitly moved for this edit.
{@link IDragHandler} implementations that are being used to explicitly move the end of an edge during the edit
should use this method to register their edit while they are being {@link IDragHandler#initializeDrag initialized}.
@param movementInfo The movement info that describes the movement of the end of the edge. The {@link MovementInfo#movedItem} must be an {@link IEdge}.
@param sourceEnd <code>true</code> if the <code>movementInfo</code> represents the source end of the edge.
@see IEdgePortHandleProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23addMovedEdgeEnd(MovementInfo%2Cboolean) Online Documentation}*/
addMovedEdgeEnd(movementInfo: MovementInfo, sourceEnd: boolean): void;
/**
Registers the provided port with this instance so that it will be considered explicitly moved for this edit.
{@link IDragHandler} implementations that are being used to explicitly move a bend during the edit should use this
method to register their edit while they are being {@link IDragHandler#initializeDrag initialized}.
@param movementInfo The movement info that describes the movement of the port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23addMovedPort(MovementInfo) Online Documentation}*/
addMovedPort(movementInfo: MovementInfo): void;
/**
Adds an {@link IPortOwner} that is being transformed somehow during the edit so that the attached {@link IPortOwner#ports ports} will be
moved in an irregular ({@link MoveTypes#ARBITRARY_MOVE non-linear}) way.
Normally this will be nodes that are being resized or moved in a non-linear way, or edges, whose bends or ports are
being changed.
@param owner The item that will be transformed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23addTransformedPortOwner(IPortOwner) Online Documentation}*/
addTransformedPortOwner(owner: IPortOwner): void;
/**
Called by client edit code when a drag has been canceled.
Resets all internal state lists and sets {@link OrthogonalEdgeEditingContext#isInitialized} back to <code>false</code>. Also {@link OrthogonalEdgeEditingContext#removeAddedBends temporarily added bends are removed} and {@link OrthogonalEdgeEditingContext#addCleanedUpListener CleanedUp}
will be triggered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23cancelDrag() Online Documentation}*/
cancelDrag(): void;
/**
Performs clean up procedures.
This is called in response to {@link OrthogonalEdgeEditingContext#cancelDrag} and {@link OrthogonalEdgeEditingContext#dragFinished}
as well as initially during {@link OrthogonalEdgeEditingContext#initializeDrag}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23cleanUp() Online Documentation}*/
cleanUp(): void;
/**
Cleans up the edge's path after a successfully {@link OrthogonalEdgeEditingContext#dragFinished finished drag}.
This implementations delegates to the {@link IOrthogonalEdgeHelper}'s {@link IOrthogonalEdgeHelper#cleanUpEdge}
method.
@param graph The graph in which the edge resides.
@param edge The modified edge.
@see OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23cleanUpEdgePath(IGraph%2CIEdge) Online Documentation}*/
cleanUpEdgePath(graph: IGraph, edge: IEdge): void;
/**
Called after a successfully {@link OrthogonalEdgeEditingContext#dragFinished finished drag} to clean up artifacts of modified edges.
@param graph The graph.
@see OrthogonalEdgeEditingContext#cleanUpEdgePath
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23cleanUpEdgePaths(IGraph) Online Documentation}*/
cleanUpEdgePaths(graph: IGraph): void;
/**
Creates or obtains a previously created {@link MovementInfo} for the provided implicitly moved bend.
This method is queried by client {@link IDragHandler} implementations that have been initialized for the current
edit that need to move adjacent bends implicitly. Implementations should pass as the parameters the movement information
that controls the vertical or horizontal movement of the bend implicitly.
@param bend The implicitly moved bend.
@param verticalAdjacentInfo The movement info that implicitly constrains the vertical movement of the bend or <code>null</code>.
@param horizontalAdjacentInfo The movement info that implicitly constrains the horizontal movement of the bend or <code>null</code>.
@returns The info to be used by clients that requested it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23createImplicitlyMovedBendInfo(IBend%2CMovementInfo%2CMovementInfo) Online Documentation}*/
createImplicitlyMovedBendInfo(bend: IBend, horizontalAdjacentInfo: MovementInfo | null, verticalAdjacentInfo: MovementInfo | null): MovementInfo | null;
/**
Creates an {@link OrthogonalEdgeDragHandler} that keeps the orthogonal edges attached to <code>node</code> orthogonal while the
node is moved or reshaped interactively.
If no enabled {@link OrthogonalEdgeEditingContext} can be found in the {@link ILookup#lookup} of the <code>context</code>,
<code>null</code> is returned.
In each move/reshape step, {@link OrthogonalEdgeDragHandler#handleMove} has to be called. When the gesture is
canceled or finished, {@link OrthogonalEdgeDragHandler#cancelDrag} or {@link OrthogonalEdgeDragHandler#finishDrag}
has to be called.
@param context The input mode context which is editing the node.
@param node The node that is moved or reshaped.
@param linearPortMovement <code>true</code> if ports on the node move linear with the mouse movement, <code>false</code> otherwise.
@returns An {@link OrthogonalEdgeDragHandler} for the node or <code>null</code> if no enabled {@link OrthogonalEdgeEditingContext}
was found.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23createOrthogonalEdgeDragHandler(IInputModeContext%2CINode%2Cboolean) Online Documentation}*/
static createOrthogonalEdgeDragHandler(context: IInputModeContext, node: INode, linearPortMovement: boolean): OrthogonalEdgeDragHandler | null;
/**
Called when a drag has been successfully finished.
This method will perform the necessary clean up and {@link OrthogonalEdgeEditingContext#cleanUpEdgePaths clean up edge paths}.
@see OrthogonalEdgeEditingContext#addCleanedUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23dragFinished() Online Documentation}*/
dragFinished(): void;
/**
Needs to be called by client editing code after {@link OrthogonalEdgeEditingContext#initializeDrag} has been called
and all {@link IDragHandler}s have been initialized.
This method will update the {@link OrthogonalEdgeEditingContext#isInitializing} and {@link OrthogonalEdgeEditingContext#isInitialized} properties accordingly and will {@link OrthogonalEdgeEditingContext#prepareEdgePaths prepare} the edge paths. Finally, the {@link OrthogonalEdgeEditingContext#addInitializedListener Initialized}
event is raised so that registered handlers can perform their post-initialization process.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23dragInitialized() Online Documentation}*/
dragInitialized(): void;
/**
Gets the {@link MovementInfo movement infos} that describe the orthogonal path of the edge.
@param edge The edge to obtain the infos for.
@returns An array that contains for the first entry the source end of the edge, then all bends and then as the final entry the
target end of the edge. For the bends, this can be be <code>null</code> values if nothing is known about the bend, yet.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23getMovementInfos(IEdge) Online Documentation}*/
getMovementInfos(edge: IEdge): IListEnumerable<MovementInfo>;
/**
Gets the {@link IOrthogonalEdgeHelper} instance associated with the given edge.
This implementation obtains the helper instance from the {@link ILookup#lookup} of the edge.
@param edge The edge to obtain the helper instance for.
@returns The helper instance to use or <code>null</code>, in which case the default behavior will be used.
@see OrthogonalEdgeHelper
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23getOrthogonalEdgeHelper(IEdge) Online Documentation}*/
getOrthogonalEdgeHelper(edge: IEdge): IOrthogonalEdgeHelper | null;
/**
Gets the declared segment orientation for the provided segment at the given edge.
This implementation uses the {@link OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper GetOrthogonalEdgeHelper()}.{@link IOrthogonalEdgeHelper#getSegmentOrientation GetSegmentOrientation} method to yield the orientation.
@param edge The edge to determine the orientation of the segment.
@param segmentIndex Index of the segment.
@returns The orientation of the segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23getSegmentOrientation(IEdge%2Cnumber) Online Documentation}*/
getSegmentOrientation(edge: IEdge, segmentIndex: number): SegmentOrientation;
/**
Called by client code when a drag is started about to be started.
This code needs to be called before any of the {@link IDragHandler} implementations are initialized so that they
can then register the items they are going to modify with this instance. After this method has been called, the handlers
that perform the actual edit need to be initialized and as soon as this has been done, {@link OrthogonalEdgeEditingContext#dragInitialized}
should be called.
@param context The context in which the edit is going to be performed.
@throws {Exception} <b>(InvalidOperationError)</b> If this context is already {@link OrthogonalEdgeEditingContext#addInitializedListener Initialized}
or currently
{@link OrthogonalEdgeEditingContext#addInitializingListener Initializing}.
@see OrthogonalEdgeEditingContext#dragInitialized
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23initializeDrag(IInputModeContext) Online Documentation}*/
initializeDrag(context: IInputModeContext): void;
/**
Called by
{@link IDragHandler}s and the like to determines whether the given edge is orthogonally edited edge in the
specified input mode context.
This implementation will always yield <code>false</code> if it is {@link OrthogonalEdgeEditingContext#enabled disabled}. Otherwise {@link OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper}
will be used to delegate the query to {@link IOrthogonalEdgeHelper#shouldEditOrthogonally}.
@param context The input mode context which is editing the edge.
@param edge The edge that will be edited.
@returns <code>true</code> if this edge should be orthogonally edited for the specified input mode context; <code>false</code> otherwise.
@see OrthogonalEdgeEditingContext#enabled
@see IOrthogonalEdgeHelper#shouldEditOrthogonally
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23isOrthogonallyEditedEdge(IInputModeContext%2CIEdge) Online Documentation}*/
isOrthogonallyEditedEdge(context: IInputModeContext, edge: IEdge): boolean;
/**
Locks the movement of the ports of the edges so that {@link OrthogonalEdgeEditingContext#shouldMoveEndImplicitly}
will yield <code>false</code> for the provided edge during the current edit.
This state will be reset after the current edit.
@param edge The edge to lock the ports of.
@see OrthogonalEdgeEditingContext#lockedPortEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23lockPortMovement(IEdge) Online Documentation}*/
lockPortMovement(edge: IEdge): void;
/**
Raises the {@link OrthogonalEdgeEditingContext#addCleanedUpListener CleanedUp}
event.
@param evt The {@link InputModeEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23onCleanedUp(InputModeEventArgs) Online Documentation}*/
onCleanedUp(evt: InputModeEventArgs): void;
/**
Raises the {@link OrthogonalEdgeEditingContext#addInitializedListener Initialized}
event.
@param evt The {@link InputModeEventArgs} instance containing the event data.
@see OrthogonalEdgeEditingContext#addInitializedListener
@see OrthogonalEdgeEditingContext#dragInitialized
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23onInitialized(InputModeEventArgs) Online Documentation}*/
onInitialized(evt: InputModeEventArgs): void;
/**
Raises the {@link OrthogonalEdgeEditingContext#addInitializingListener Initializing}
event.
@param evt The {@link InputModeEventArgs} instance containing the event data.
@see OrthogonalEdgeEditingContext#addInitializingListener
@see OrthogonalEdgeEditingContext#initializeDrag
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23onInitializing(InputModeEventArgs) Online Documentation}*/
onInitializing(evt: InputModeEventArgs): void;
/**
Called during {@link OrthogonalEdgeEditingContext#dragInitialized} to prepare the edge paths for orthogonal
editing.
@param graph The graph that contains the edges to be edited.
@see OrthogonalEdgeEditingContext#prepareOrthogonalEdge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23prepareEdgePaths(IGraph) Online Documentation}*/
prepareEdgePaths(graph: IGraph): void;
/**
Helper method that inspects and prepares an orthogonal edge for the upcoming edit process.
This method inspects an edge and possibly inserts new bends into an edge to assure that during editing the orthogonality
won't be lost.
@param graph The graph to use for modifying the bends.
@param edge The edge to inspect.
@param orientations The orientations of the segments - the array is of length (edge.Bends.Count + 1).
@param infos The {@link MovementInfo} instances for the source end, the bends, and the target end. This array contains {@link MovementInfo}
instances representing either edge ends or ports at the first and last position and {@link MovementInfo}s or <code>null</code>
entries for the bends. The length thus is (edge.Bends.Count + 2) and <code>null</code> entries indicate that there is no information
about the movement (in case the bend will only be moved implicitly later, if at all).
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23prepareOrthogonalEdge(IGraph%2CIEdge%2CIListEnumerable%3CSegmentOrientation%3E%2CIListEnumerable%3CMovementInfo%3E) Online Documentation}*/
prepareOrthogonalEdge(graph: IGraph, edge: IEdge, orientations: IListEnumerable<SegmentOrientation>, infos: IListEnumerable<MovementInfo>): void;
/**
Can be used by subclasses during {@link OrthogonalEdgeEditingContext#prepareOrthogonalEdge} to register added bends
that can later be {@link OrthogonalEdgeEditingContext#removeAddedBends removed} in case the operation is canceled.
This method will only record the addition of bends if this instance is {@link OrthogonalEdgeEditingContext#isInitialized initialized} or {@link OrthogonalEdgeEditingContext#addInitializingListener still initializing}.
@param bend The bend that has been added to guarantee orthogonality.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23registerAddedBend(IBend) Online Documentation}*/
registerAddedBend(bend: IBend): void;
/**
Removes previously {@link OrthogonalEdgeEditingContext#registerAddedBend registered} added bends.
@param graph The graph to use for removing the bends.
@param addedBends The added bends.
@see OrthogonalEdgeEditingContext#addedBends
@see OrthogonalEdgeEditingContext#registerAddedBend
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23removeAddedBends(IGraph%2CIEnumerable%3CIBend%3E) Online Documentation}*/
removeAddedBends(graph: IGraph, addedBends: IEnumerable<IBend>): void;
/**
Called by {@link IDragHandler}s and the like to determines whether the specified end of the provided <code>edge</code> should be
moved implicitly.
If an item is next to the end of an edge and the last segment should be kept orthogonal, this may only be possible if
the end of the edge is moved, too. This can be achieved by moving the edge to another {@link IPort}, or by moving
the port instance itself. This depends on the implementation of the {@link IEdgePortHandleProvider} for the given
edge. This implementation will delegate to the {@link IOrthogonalEdgeHelper#shouldMoveEndImplicitly} method of the
{@link IOrthogonalEdgeHelper} instance returned by {@link OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper}
unless the side of the edge is connected to an {@link IPortOwner} instance that is marked as {@link OrthogonalEdgeEditingContext#addTransformedPortOwner transformed} during the
edit.
@param edge The edge for which it should be determined whether the edge end can be moved.
@param sourceSide if set to <code>true</code> the source side of the end is queried, else the target side.
@returns <code>true</code> if the specified side of the edge can be moved; <code>false</code> otherwise, in which case the segment should be split to
maintain orthogonality.
@see OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper
@see IOrthogonalEdgeHelper#shouldMoveEndImplicitly
@see OrthogonalEdgeEditingContext#movePorts
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23shouldMoveEndImplicitly(IEdge%2Cboolean) Online Documentation}*/
shouldMoveEndImplicitly(edge: IEdge, sourceSide: boolean): boolean;
/**
Adds the given listener for the <code>CleanedUp</code> event that occurs when the recent edit operation has been cleaned up.
@param listener The listener to add.
@see OrthogonalEdgeEditingContext#cleanUp
@see OrthogonalEdgeEditingContext#onCleanedUp
@see OrthogonalEdgeEditingContext#removeCleanedUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23CleanedUp Online Documentation}*/
addCleanedUpListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>CleanedUp</code> event that occurs when the recent edit operation has been cleaned up.
@param listener The listener to remove.
@see OrthogonalEdgeEditingContext#cleanUp
@see OrthogonalEdgeEditingContext#onCleanedUp
@see OrthogonalEdgeEditingContext#addCleanedUpListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23CleanedUp Online Documentation}*/
removeCleanedUpListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Initializing</code> event that occurs when the edit is about to be initialized.
@param listener The listener to add.
@see OrthogonalEdgeEditingContext#initializeDrag
@see OrthogonalEdgeEditingContext#onInitializing
@see OrthogonalEdgeEditingContext#removeInitializingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23Initializing Online Documentation}*/
addInitializingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Initializing</code> event that occurs when the edit is about to be initialized.
@param listener The listener to remove.
@see OrthogonalEdgeEditingContext#initializeDrag
@see OrthogonalEdgeEditingContext#onInitializing
@see OrthogonalEdgeEditingContext#addInitializingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23Initializing Online Documentation}*/
removeInitializingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Initialized</code> event that occurs when the edit has been initialized.
@param listener The listener to add.
@see OrthogonalEdgeEditingContext#dragInitialized
@see OrthogonalEdgeEditingContext#onInitialized
@see OrthogonalEdgeEditingContext#removeInitializedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23Initialized Online Documentation}*/
addInitializedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Initialized</code> event that occurs when the edit has been initialized.
@param listener The listener to remove.
@see OrthogonalEdgeEditingContext#dragInitialized
@see OrthogonalEdgeEditingContext#onInitialized
@see OrthogonalEdgeEditingContext#addInitializedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23Initialized Online Documentation}*/
removeInitializedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Gets or sets a value indicating whether this instance is currently initializing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23isInitializing Online Documentation}*/
isInitializing: boolean;
/**
Gets or sets a value indicating whether orthogonal edge editing is enabled at all.
The default value is <code>true</code>.
@see OrthogonalEdgeEditingContext#isOrthogonallyEditedEdge
@see IOrthogonalEdgeHelper#shouldEditOrthogonally
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23enabled Online Documentation}*/
enabled: boolean;
/**
Gets or sets a value indicating ports are allowed to be moved at all.
The default value is <code>true</code>.
@see OrthogonalEdgeEditingContext#shouldMoveEndImplicitly
@see IOrthogonalEdgeHelper#shouldMoveEndImplicitly
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23movePorts Online Documentation}*/
movePorts: boolean;
/**
Gets the current input mode context that is in effect for the current edit.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23currentInputModeContext Online Documentation}*/
readonly currentInputModeContext: IInputModeContext | null;
/**
Gets or sets whether the context is initialized.
The context is initialized between the calls to {@link OrthogonalEdgeEditingContext#initializeDrag} and {@link OrthogonalEdgeEditingContext#dragFinished}
or {@link OrthogonalEdgeEditingContext#cancelDrag}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23isInitialized Online Documentation}*/
isInitialized: boolean;
/**
Gets the previously {@link OrthogonalEdgeEditingContext#registerAddedBend registered added} bends.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23addedBends Online Documentation}*/
readonly addedBends: IEnumerable<IBend>;
/**
Gets the enumeration of {@link IPortOwner} instances that are being transformed during the edit.
For elements in the enumerable the attached {@link IPortOwner#ports} are considered to be moved in an irregular way during the edit.
@see MoveTypes#ARBITRARY_MOVE
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23transformedPortOwners Online Documentation}*/
readonly transformedPortOwners: IEnumerable<IPortOwner>;
/**
Gets the enumeration of {@link IEdge} instances whose ports have been locked at source and target end.
For elements in the enumerable the end points should not be moved and thus {@link OrthogonalEdgeEditingContext#shouldMoveEndImplicitly}
returns false for these edges. Note that this collection is reset for each edit.
@see MoveTypes#ARBITRARY_MOVE
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeEditingContext%23lockedPortEdges Online Documentation}*/
readonly lockedPortEdges: IEnumerable<IEdge>;
static $class: Class;
private $_OrthogonalEdgeEditingContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalEdgeEditingContext;
}
export interface OrthogonalEdgeDragHandler extends YObject {
}
/**
This class can be used to keep the orthogonal edges attached to an {@link INode} orthogonal while the node is moved
or reshaped interactively.
Instances of this class can be created via {@link OrthogonalEdgeEditingContext#createOrthogonalEdgeDragHandler}.
In each move/reshape step of the node this handler has been created for, {@link OrthogonalEdgeDragHandler#handleMove}
has to be called. When the gesture is canceled or finished, {@link OrthogonalEdgeDragHandler#cancelDrag} or {@link OrthogonalEdgeDragHandler#finishDrag}
has to be called.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeDragHandler Online Documentation}*/
export class OrthogonalEdgeDragHandler {
/**
Cancels the gesture and resets the port and bend locations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeDragHandler%23cancelDrag() Online Documentation}*/
cancelDrag(): void;
/**
Finishes the gesture and cleans up temporary bends that were created during the gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeDragHandler%23finishDrag() Online Documentation}*/
finishDrag(): void;
/**
Updates the port locations and bend locations of orthogonal edges at these ports.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeDragHandler%23handleMove() Online Documentation}*/
handleMove(): void;
static $class: Class;
private $_OrthogonalEdgeDragHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalEdgeDragHandler;
}
export interface OrthogonalEdgeHelper extends YObject, IOrthogonalEdgeHelper {
}
/**
Default implementation of the {@link IOrthogonalEdgeHelper}
This class inspects the edges in question and infers the
{@link SegmentOrientation}s of the segments from the current geometry of the edge path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeHelper Online Documentation}*/
export class OrthogonalEdgeHelper {
/**
Returns whether the given bend can be removed.
This implementation always returns <code>true</code>.
@param context The input mode context.
@param bend The bend that is designated for removal.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeHelper%23canRemoveBend(IInputModeContext%2CIBend) Online Documentation}*/
canRemoveBend(context: IInputModeContext, bend: IBend): boolean;
/**
Invoked after the provided edge has been edited orthogonally.
The default implementation in this class removes duplicate and collinear bends, as well as segments of zero length.
@param context The input mode context which edited the edge.
@param graph The graph to use for modifying the edge instance.
@param edge The edge to clean up the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeHelper%23cleanUpEdge(IInputModeContext%2CIGraph%2CIEdge) Online Documentation}*/
cleanUpEdge(context: IInputModeContext, graph: IGraph, edge: IEdge): void;
/**
Gets the orientation of the given segment by looking at the geometry of the segment.
If the segment has a zero length, the adjacent segments will be inspected to infer the orientation such that the
orientation toggles between {@link SegmentOrientation#VERTICAL} and {@link SegmentOrientation#HORIZONTAL}.
@param context The input mode context in which the orientation is needed.
@param edge The edge to inspect.
@param segmentIndex The index of the segment.
@returns The declared orientation of the segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeHelper%23getSegmentOrientation(IInputModeContext%2CIEdge%2Cnumber) Online Documentation}*/
getSegmentOrientation(context: IInputModeContext, edge: IEdge, segmentIndex: number): SegmentOrientation;
/**
Determines whether the provided edge should be edited orthogonally in the specified input mode context.
If this method returns <code>false</code>, the other methods will not be queried at all.
The default implementation in this class always returns <code>true</code>.
@param context The input mode context in which the edge is about to be edited.
@param edge The edge to inspect.
@returns <code>true</code> if the edge should be orthogonally edited in the specified input mode context; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeHelper%23shouldEditOrthogonally(IInputModeContext%2CIEdge) Online Documentation}*/
shouldEditOrthogonally(context: IInputModeContext, edge: IEdge): boolean;
/**
Determines whether this end of the provided edge can be moved in the input mode context.
This information is required to determine whether an edge's first or last segment needs to be split or the adjacent edge
end can be moved along with the other end of the segment.
The default implementation in this class always returns <code>false</code>.
@param context The input mode context in which the segment is edited.
@param edge The edge to inspect.
@param sourceEnd if set to <code>true</code> the source end of the edge is queried, otherwise the target end.
@returns <code>true</code> if the end of the edge can be moved for the specified input mode context; <code>false</code> otherwise, in which case the
corresponding segment needs to be split to keep the segment orientation orthogonal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalEdgeHelper%23shouldMoveEndImplicitly(IInputModeContext%2CIEdge%2Cboolean) Online Documentation}*/
shouldMoveEndImplicitly(context: IInputModeContext, edge: IEdge, sourceEnd: boolean): boolean;
static $class: Class;
private $_OrthogonalEdgeHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalEdgeHelper;
}
export interface PortDropInputMode extends ItemDropInputMode<IPort> {
}
/**
* An {@link ItemDropInputMode} specialized to the drag and drop of {@link IPort}s.
* This input mode {@link ItemDropInputMode#showPreview can visualize a preview} of the {@link ItemDropInputMode#draggedItem dragged item} during the drag operation, and supports {@link ItemDropInputMode#snappingEnabled snapping} via the {@link ItemDropInputMode#snapContext}
* and {@link ItemDropInputMode#highlightDropTarget highlighting} of the {@link ItemDropInputMode#dropTarget} via a {@link HighlightIndicatorManager}.
*
* Typically the <code>PortDropInputMode</code> is installed as child mode of a {@link GraphEditorInputMode} and can be retrieved
* from the {@link GraphEditorInputMode#portDropInputMode} property.
*
* <br><b>Getting the PortDropInputMode from its parent input mode</b>
* <pre>
* const portDropInputMode = mode.portDropInputMode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode Online Documentation}
*/
export class PortDropInputMode {
/**
Constructs a new instance of class {@link DropInputMode} for the expected data type.
Not used in this class since data type is always
{@link IPort}. This constructor only exists for usage in child class.
@param expectedType
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23PortDropInputMode(Array%3Cstring%3E) Online Documentation}*/
constructor(expectedType: string[]);
/**
Constructs a new instance of class {@link DropInputMode} for the expected data type.
Not used in this class since data type is always
{@link IPort}. This constructor only exists for usage in child class.
@param expectedType
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23PortDropInputMode(string) Online Documentation}*/
constructor(expectedType: string);
/**
Constructs a new instance of class {@link PortDropInputMode} for the type {@link IPort}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23PortDropInputMode() Online Documentation}*/
constructor();
/**
Creates the port in the graph after it's been dropped.
This method is called by the {@link ItemDropInputMode#itemCreator} that is set as default on this class.
@param context The context for which the port should be created.
@param graph The {@link IGraph Graph} in which to create the port.
@param draggedPort The port that was dragged and should therefore be created.
@param dropTarget The {@link IPortOwner} on which the port is dropped.
@param parameter The {@link IPortLocationModelParameter} of the new port.
@returns a newly created label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23createPort(IInputModeContext%2CIGraph%2CIPort%2CIPortOwner%2CIPortLocationModelParameter) Online Documentation}*/
createPort(context: IInputModeContext, graph: IGraph | null, draggedPort: IPort, dropTarget: IPortOwner | null, parameter: IPortLocationModelParameter | null): IPort | null;
/**
Looks for a {@link PortDropInputMode#isValidPortOwnerPredicate valid port owner} at the <code>dragLocation</code>.
@param dragLocation
@returns A {@link IPortOwner port owner} or <code>null</code> if no valid port owner was found at the drop location.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23getDropTarget(Point) Online Documentation}*/
getDropTarget(dragLocation: Point): IModelItem | null;
/**
Determine the {@link IPortLocationModelParameter} the port should have if it were dropped at <code>dragLocation</code> over
<code>owner</code>.
If {@link PortDropInputMode#useBestMatchingParameter} is <code>true</code>, this implementation returns just the {@link IPort#locationParameter} of the <code>draggedPort</code> if it can be used for <code>owner</code> and <code>null</code> otherwise.
The <code>dragLocation</code> is ignored in that case.
If {@link PortDropInputMode#useBestMatchingParameter} is <code>false</code>, this implementation will use the model of the <code>draggedPort</code> if it is compatible with <code>owner</code> to determine the
parameter that corresponds best to the current <code>dragLocation</code>.
@param owner The port owner where the port would be created. May be <code>null</code>.
@param draggedPort The port that is currently dragged.
@param dragLocation The current drag location.
@returns A {@link IPortLocationModelParameter} for <code>draggedPort</code>, or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23getNewPortLocationModelParameter(IPortOwner%2CIPort%2CPoint) Online Documentation}*/
getNewPortLocationModelParameter(owner: IPortOwner | null, draggedPort: IPort, dragLocation: Point): IPortLocationModelParameter | null;
/**
Default backing method for {@link PortDropInputMode#isValidPortOwnerPredicate}
This implementation checks whether a valid port location model parameter could be found for the combination of <code>owner</code> and
<code>draggedPort</code>.
@param owner The port owner where the port would be created. May be <code>null</code>.
@param draggedPort The port that is currently dragged.
@returns <code>true</code> iff <code>owner</code> can be used to drop a port.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23isValidPortOwner(IPortOwner%2CIPort) Online Documentation}*/
isValidPortOwner(owner: IPortOwner | null, draggedPort: IPort | null): boolean;
/**
Starts a drag and drop operation for a {@link IPort}.
This method creates a {@link DragSource}, and then invokes {@link DragSource#startDrag} with a {@link DragDropItem}
that contains the <code>port</code> and the corresponding type.
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param dragSource The source of the drag operation.
@param port The port to drag.
@param [dragDropEffects=DragDropEffects.ALL] The allowed drag drop effects.
@param [useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see DragSource#startDrag
@see DragDropItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23startDrag(Element%2CIPort%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
static startDrag(dragSource: HTMLElement | SVGElement, port: IPort, dragDropEffects?: DragDropEffects | DragDropEffectsStringValues, useCssCursors?: boolean, dragPreview?: HTMLElement | SVGElement | null): DragSource;
/**
Starts a drag and drop operation for a {@link IPort}.
This method creates a {@link DragSource}, and then invokes {@link DragSource#startDrag} with a {@link DragDropItem}
that contains the <code>port</code> and the corresponding type.
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param options The parameters to pass.
@param options.dragSource The source of the drag operation.
@param options.port The port to drag.
@param [options.dragDropEffects=DragDropEffects.ALL] The allowed drag drop effects.
@param [options.useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [options.dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see DragSource#startDrag
@see DragDropItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23startDrag(Element%2CIPort%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
static startDrag(options: {
dragSource: HTMLElement | SVGElement;
port: IPort;
dragDropEffects?: DragDropEffects | DragDropEffectsStringValues;
useCssCursors?: boolean;
dragPreview?: HTMLElement | SVGElement | null;
}): DragSource;
/**
Gets the currently dragged {@link IModelItem} instance.
This implementation simply tries to cast the {@link DropInputMode#dropData} to {@link IPort}.
If native drag and drop is used, the value of this property is always <code>null</code>. This is because the protected mode prevents
access to the actual data during <code>dragover</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23draggedItem Online Documentation}*/
draggedItem: IPort | null;
/**
Gets or sets the callback for testing whether an existing port owner in the graph can be used as owner for the currently
dragged port.
The default implementation checks whether a valid port model parameter could be found for the combination of port owner
and dragged port.
Note that during native drag and drop, {@link PortDropInputMode#draggedItem} is always null and cannot be used for this check.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23isValidPortOwnerPredicate Online Documentation}*/
isValidPortOwnerPredicate: (arg1: IPortOwner | null, arg2: IPort | null) => boolean;
/**
Determines whether the default implementation of {@link PortDropInputMode#getNewPortLocationModelParameter} should
directly use the parameter stored in the dragged item or whether the current drop location should be also taken into
account.
Default value is <code>false</code>, which means to use the parameter directly
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDropInputMode%23useBestMatchingParameter Online Documentation}*/
useBestMatchingParameter: boolean;
static $class: Class;
private $_PortDropInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortDropInputMode;
}
export interface PortLocationModelParameterHandle extends YObject, IHandle {
/**
@inheritdoc*/
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
/**
@inheritdoc*/
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
initializeDrag(context: IInputModeContext): void;
/**
@inheritdoc*/
readonly location: IPoint;
} /**
An implementation of the {@link IHandle} that can be used to move an {@link IPort} interactively by assigning
new {@link IPortLocationModelParameter}s to the {@link IPort#locationParameter}
This class also provides snapping facilities and can be used in conjunction with both
{@link OrthogonalEdgeEditingContext}, as well as {@link GraphSnapContext}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle Online Documentation}*/
export class PortLocationModelParameterHandle {
/**
Initializes a new instance of the {@link PortLocationModelParameterHandle} class to change the {@link IPortLocationModelParameter}
of the given <code>port</code>
@param port The port to adjust the location parameter of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle Online Documentation}*/
constructor(port: IPort);
/**
Called during the drag to collect snap results.
This implementation will delegate to the {@link IPortSnapResultProvider} that has been received from the port this
instance is bound to.
@param source The source of the event.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle%23collectSnapResults(Object%2CCollectSnapResultsEventArgs) Online Documentation}*/
collectSnapResults(source: any, evt: CollectSnapResultsEventArgs): void;
/**
Gets the graph to use for {@link IGraph#setPortLocationParameter setting the parameter} from the context.
@param context The input mode context to use.
@returns The graph or <code>null</code> if the graph could not be obtained.
@see IInputModeContext#graph
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle%23getGraph(IInputModeContext) Online Documentation}*/
getGraph(context: IInputModeContext): IGraph | null;
/**
Determines the type of the movement for the given port.
This is used for the {@link MovementInfo} that is passed to the {@link OrthogonalEdgeEditingContext#addMovedPort}
method.
@param port The port to determine the move type for.
@returns This implementation returns {@link MoveTypes#LINEAR_MOVE} for models of type {@link FreeNodePortLocationModel}
and {@link MoveTypes#ARBITRARY_MOVE} for all others.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle%23getMoveType(IPort) Online Documentation}*/
getMoveType(port: IPort): MoveTypes;
/**
Obtains the new parameter for the given location and port.
@param port The port to obtain a parameter for.
@param model The model to use.
@param location The new location.
@returns The new parameter to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle%23getNewParameter(IPort%2CIPortLocationModel%2CPoint) Online Documentation}*/
getNewParameter(port: IPort, model: IPortLocationModel, location: Point): IPortLocationModelParameter;
/**
Applies the parameter.
@param graph The graph to use for applying.
@param port The port to apply the parameter to.
@param locationParameter The new parameter to apply.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle%23setParameter(IGraph%2CIPort%2CIPortLocationModelParameter) Online Documentation}*/
setParameter(graph: IGraph, port: IPort, locationParameter: IPortLocationModelParameter): void;
/**
Gets the port this instance acts on.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle%23port Online Documentation}*/
readonly port: IPort;
/**
Gets or sets the type of the handle that can be used by the rendering engine to render types differently.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle%23type Online Documentation}*/
type: HandleTypes;
/**
Gets or sets the cursor to display when the mouse hovers over or drags this handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLocationModelParameterHandle%23cursor Online Documentation}*/
cursor: Cursor;
static $class: Class;
private $_PortLocationModelParameterHandle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortLocationModelParameterHandle;
}
export interface ReparentNodeHandler extends YObject, IReparentNodeHandler {
/**
@inheritdoc*/
isReparentGesture(context: IInputModeContext, node: INode): boolean;
/**
@inheritdoc*/
isValidParent(context: IInputModeContext, node: INode, newParent: INode | null): boolean;
/**
@inheritdoc*/
reparent(context: IInputModeContext, node: INode, newParent: INode | null): void;
/**
@inheritdoc*/
shouldReparent(context: IInputModeContext, node: INode): boolean;
} /**
Simple default implementation of {@link IReparentNodeHandler} that treats shift key presses as reparent gestures
and allows for reparenting all nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentNodeHandler Online Documentation}*/
export class ReparentNodeHandler {
constructor();
/**
Gets or sets the event recognizer that determines whether to reparent a node.
The default is {@link KeyEventRecognizers#SHIFT_IS_DOWN}
@see ReparentNodeHandler#isReparentGesture
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentNodeHandler%23reparentRecognizer Online Documentation}*/
reparentRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
static $class: Class;
private $_ReparentNodeHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReparentNodeHandler;
}
export interface NodeSnapLineProvider extends YObject, ISnapLineProvider {
}
/**
An implementation of the {@link ISnapLineProvider} interface that provides {@link OrthogonalSnapLine}s for
{@link INode}s.
This implementation uses the {@link GraphSnapContext} to create snap lines around the geometric border of the
node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapLineProvider Online Documentation}*/
export class NodeSnapLineProvider {
/**
Called by {@link NodeSnapLineProvider#addSnapLines} the collect snap lines for the centers of the node.
This method is called by {@link NodeSnapLineProvider#addSnapLines} and by default used the {@link NodeSnapLineProvider#createCenterSnapLine}
to add them to the {@link CollectGraphSnapLinesEventArgs#addFixedNodeSnapLine fixed node} and {@link CollectGraphSnapLinesEventArgs#addFixedSegmentSnapLine fixed segment} collections
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param evt The argument to use for adding snap lines.
@param node The node to add snap lines for.
@param layout The layout of the node to use for adding the snap lines.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapLineProvider%23addCenterSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CINode%2CRect) Online Documentation}*/
addCenterSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, node: INode, layout: Rect): void;
/**
Adds the fixed snap lines around the border of the nodes' <code>layout</code>.
This method is called by {@link NodeSnapLineProvider#addSnapLines}
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param evt The argument to use for adding snap lines.
@param node The node to add snap lines for.
@param layout The layout of the node to use for adding the snap lines.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapLineProvider%23addFixedNodeBorderSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CINode%2CRect) Online Documentation}*/
addFixedNodeBorderSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, node: INode, layout: Rect): void;
/**
Adds the fixed snap lines around the node for other nodes to snap to using the {@link GraphSnapContext#nodeToNodeDistance}.
This method is called by {@link NodeSnapLineProvider#addSnapLines} and uses the {@link GraphSnapContext#nodeToNodeDistance} to determine the location of the
segments to add around the <code>layout</code>.
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param evt The argument to use for adding snap lines.
@param node The node to add snap lines for.
@param layout The layout of the node to use for adding the snap lines.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapLineProvider%23addNodeToNodeSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CINode%2CRect) Online Documentation}*/
addNodeToNodeSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, node: INode, layout: Rect): void;
/**
Adds the fixed snap lines around the node for edge segments to snap to.
This method is called by {@link NodeSnapLineProvider#addSnapLines} and uses the {@link GraphSnapContext#nodeToEdgeDistance} to determine the location of the
segments to add around the <code>layout</code>.
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param evt The argument to use for adding snap lines.
@param node The node to add snap lines for.
@param layout The layout of the node to use for adding the snap lines.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapLineProvider%23addNodeToSegmentSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CINode%2CRect) Online Documentation}*/
addNodeToSegmentSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, node: INode, layout: Rect): void;
/**
Called by {@link NodeSnapLineProvider#addSnapLines} the collect snap lines to snap ports to at the provided node.
This method is called by {@link NodeSnapLineProvider#addSnapLines}
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param evt The argument to use for adding snap lines.
@param node The node to add snap lines for.
@param layout The layout of the node to use for adding the snap lines.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapLineProvider%23addPortSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CINode%2CRect) Online Documentation}*/
addPortSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, node: INode, layout: Rect): void;
/**
Called by the {@link GraphSnapContext} when a {@link GraphSnapContext#initializeDrag drag} is about to start.
This means that the provided <code>item</code>, which can be cast to {@link INode} will not be moved during the edit and thus
can contribute to the set of fixed snap lines. This implementation delegates to
{@link NodeSnapLineProvider#addFixedNodeBorderSnapLines}, {@link NodeSnapLineProvider#addPortSnapLines},
{@link NodeSnapLineProvider#addNodeToNodeSnapLines}, and {@link NodeSnapLineProvider#addCenterSnapLines}.
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param evt The argument to use for adding snap lines.
@param item The item to add snap lines for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapLineProvider%23addSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CIModelItem) Online Documentation}*/
addSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, item: IModelItem): void;
/**
Creates a snap line for the center of nodes.
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param node The node to add snap lines for.
@param orientation The snap line orientation.
@param location The location of the snap line.
@returns A snap line that other centers can snap to.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapLineProvider%23createCenterSnapLine(GraphSnapContext%2CINode%2CSnapLineOrientation%2CPoint) Online Documentation}*/
createCenterSnapLine(context: GraphSnapContext, node: INode, orientation: SnapLineOrientation | SnapLineOrientationStringValues, location: Point): OrthogonalSnapLine;
static $class: Class;
private $_NodeSnapLineProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeSnapLineProvider;
}
export interface EdgeSnapLineProvider extends YObject, ISnapLineProvider {
}
/**
Simple implementation of the {@link ISnapLineProvider} interface that can be used to add snap lines for
orthogonally routed {@link IEdge}s.
This implementation can be put into the {@link ILookup#lookup} of an {@link IEdge}.
@see EdgeDecorator#snapLineProviderDecorator
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSnapLineProvider Online Documentation}*/
export class EdgeSnapLineProvider {
/**
Adds horizontal snap lines for a horizontally oriented fixed segment snap lines.
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param edge The edge to add snap lines for.
@param segmentStart The start of the segment.
@param evt The argument to use for adding snap lines.
@param segmentEnd The end of the segment.
@see EdgeSnapLineProvider#addVerticalSegmentSnapLines
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSnapLineProvider%23addHorizontalSegmentSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CIEdge%2CPoint%2CPoint) Online Documentation}*/
addHorizontalSegmentSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, edge: IEdge, segmentStart: Point, segmentEnd: Point): void;
/**
Adds the snap lines for the given model item to the <code>evt</code>.
This implementation queries the {@link GraphSnapContext#getMovementInfos movement information} for the provided edge to determine which edge segments stay fixed.
For each fixed segment, it will call {@link EdgeSnapLineProvider#addVerticalSegmentSnapLines} or {@link EdgeSnapLineProvider#addHorizontalSegmentSnapLines}
respectively.
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param item The item to add snap lines for.
@param evt The argument to use for adding snap lines.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSnapLineProvider%23addSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CIModelItem) Online Documentation}*/
addSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, item: IModelItem): void;
/**
Adds vertical snap lines for a vertically oriented fixed segment snap lines.
@param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
context explicitly.
@param edge The edge to add snap lines for.
@param segmentStart The start of the segment.
@param segmentEnd The end of the segment.
@param evt The argument to use for adding snap lines.
@see EdgeSnapLineProvider#addHorizontalSegmentSnapLines
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeSnapLineProvider%23addVerticalSegmentSnapLines(GraphSnapContext%2CCollectGraphSnapLinesEventArgs%2CIEdge%2CPoint%2CPoint) Online Documentation}*/
addVerticalSegmentSnapLines(context: GraphSnapContext, evt: CollectGraphSnapLinesEventArgs, edge: IEdge, segmentStart: Point, segmentEnd: Point): void;
static $class: Class;
private $_EdgeSnapLineProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeSnapLineProvider;
}
export interface NodeSnapResultProvider extends YObject, INodeSnapResultProvider {
/**
@inheritdoc*/
collectSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, suggestedLayout: Rect, node: INode): void;
} /**
The default implementation of the {@link INodeSnapResultProvider} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapResultProvider Online Documentation}*/
export class NodeSnapResultProvider {
/**
Calculates a {@link SnapResult} and {@link CollectSnapResultsEventArgs#addSnapResult adds it} to the argument.
This method delegates to {@link NodeSnapResultProvider#addGridSnapResultCore} using {@link SnapContext#gridSnapType} and {@link SnapPolicy#TO_NEAREST}
as the additional arguments. This method snaps the <code>location</code> to the next grid point or grid lines that are determined by
{@link GraphSnapContext#nodeGridConstraintProvider} or obtained from the {@link InputModeEventArgs#context}.
@param context The context in which the snapping is performed.
@param evt The arguments to add the results to.
@param location A location in the layout of the suggested layout of the node that will be snapped to a grid point location.
@param node The node that is currently being processed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapResultProvider%23addGridSnapResult(GraphSnapContext%2CCollectSnapResultsEventArgs%2CPoint%2CINode) Online Documentation}*/
addGridSnapResult(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, location: Point, node: INode): void;
/**
Calculates a {@link SnapResult} and {@link CollectSnapResultsEventArgs#addSnapResult adds it} to the argument.
This method snaps the <code>location</code> to the next grid point or grid lines that are determined by {@link GraphSnapContext#nodeGridConstraintProvider} or obtained from the {@link InputModeEventArgs#context}.
@param context The context in which the snapping is performed.
@param evt The arguments to add the results to.
@param location A location in the layout of the suggested layout of the node that will be snapped to a grid point location.
@param node The node that is currently being processed.
@param gridSnapType The type of snapping that should be performed.
@param xSnapPolicy How to snap the x coordinate.
@param ySnapPolicy How to snap the y coordinate.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapResultProvider%23addGridSnapResultCore(GraphSnapContext%2CCollectSnapResultsEventArgs%2CPoint%2CINode%2CGridSnapTypes%2CSnapPolicy%2CSnapPolicy) Online Documentation}*/
addGridSnapResultCore(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, location: Point, node: INode, gridSnapType: GridSnapTypes | GridSnapTypesConvertible, xSnapPolicy: SnapPolicy | SnapPolicyStringValues, ySnapPolicy: SnapPolicy | SnapPolicyStringValues): void;
/**
Collects snap results that snap the node to a grid and {@link CollectSnapResultsEventArgs#addSnapResult adds them} to the argument.
This implementation simply delegates to {@link NodeSnapResultProvider#addGridSnapResult} using the center of the
<code>suggestedLayout</code>.
@param context The context in which the snapping is performed.
@param evt The arguments to add the results to.
@param suggestedLayout The layout of the node if it would move without snapping.
@param node The node that is currently being processed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapResultProvider%23collectGridSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CRect%2CINode) Online Documentation}*/
collectGridSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, suggestedLayout: Rect, node: INode): void;
/**
Collects the results for the given layout for all snap lines in question.
For each snap line that could be snapped to, this method calls
{@link NodeSnapResultProvider#collectSnapResultsForSnapLine}.
@param context The context for which the results are being queried.
@param evt The {@link CollectSnapResultsEventArgs} instance to which the results should be {@link CollectSnapResultsEventArgs#addSnapResult added}.
@param suggestedLayout The suggested layout of the node.
@param node The node that is being dragged.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapResultProvider%23collectSnapLineSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CRect%2CINode) Online Documentation}*/
collectSnapLineSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, suggestedLayout: Rect, node: INode): void;
/**
Verifies whether the node will snap to the given snap line.
If the node will snap, a {@link SnapResult} for a {@link SnapLine} will be created and added to the event
argument.
@param evt The arguments to add the results to.
@param snapLine The snap line to test snapping for.
@param suggestedLayout The layout of the node if it would move without snapping.
@param node The node that is currently being processed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapResultProvider%23collectSnapResultsForSnapLine(CollectSnapResultsEventArgs%2COrthogonalSnapLine%2CRect%2CINode) Online Documentation}*/
collectSnapResultsForSnapLine(evt: CollectSnapResultsEventArgs, snapLine: OrthogonalSnapLine, suggestedLayout: Rect, node: INode): void;
/**
Yields the static shared instance of this class.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeSnapResultProvider%23INSTANCE Online Documentation}*/
static INSTANCE: INodeSnapResultProvider;
static $class: Class;
private $_NodeSnapResultProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeSnapResultProvider;
}
export interface NodeReshapeSnapResultProvider extends YObject, INodeReshapeSnapResultProvider {
}
/**
The default implementation of the {@link INodeReshapeSnapResultProvider} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider Online Documentation}*/
export class NodeReshapeSnapResultProvider {
/**
Adds a snap result for a location that snaps to a grid line.
This method is called by {@link NodeReshapeSnapResultProvider#collectGridSnapResults} for each grid line snap.
@param context The context in which the snapping is performed.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data.
@param node The node that is being reshaped.
@param snapLineType The type of snap line.
@param snapLineLocation The point at which the snap line is anchored.
@param delta The amount the mouse needs to be moved in order to get to the snapping location.
@param snappedLocation The location of the mouse that is associated with the grid point.
@param from Where the snap line starts.
@param to Where the snap line ends.
@param snapLinePoint A point of interest on the snap line that can be highlighted.
@param preferredSnapType Whether the cursor should preferably snap horizontally or vertically.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23addGridLineSnapResult(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2CSnapLineSnapTypes%2CIPoint%2CPoint%2CPoint%2Cnumber%2Cnumber%2CPoint%2CSnapTypes) Online Documentation}*/
addGridLineSnapResult(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, snapLineType: SnapLineSnapTypes | SnapLineSnapTypesConvertible, snapLineLocation: IPoint, delta: Point, snappedLocation: Point, from: number, to: number, snapLinePoint: Point, preferredSnapType: SnapTypes | SnapTypesConvertible): void;
/**
Adds a snap result for a location that snaps to the grid.
This method is called by {@link NodeReshapeSnapResultProvider#collectGridSnapResults} for each grid snap.
@param context The context in which the snapping is performed.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data.
@param node The node that is being reshaped.
@param provider The provider that is associated with the grid snapping.
@param gridSnappedPoint The point on the grid that should be highlighted.
@param snappedLocation The location of the mouse that is associated with the grid point.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23addGridSnapResult(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2CIGridConstraintProvider%3CINode%3E%2CPoint%2CPoint) Online Documentation}*/
addGridSnapResult(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, provider: IGridConstraintProvider<INode>, gridSnappedPoint: Point, snappedLocation: Point): void;
/**
Adds a snap result for "same size" to the collect event argument.
This method is called by {@link NodeReshapeSnapResultProvider#collectSameSizeSnapResults} for each result found.
@param context The context in which the snapping is performed.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data. This will be used to {@link CollectSnapResultsEventArgs#addSnapResult add} the result
to.
@param node The node that is being reshaped.
@param orientation The orientation of the result - indicates whether width or height have been snapped.
@param rectangles The rectangles that have the same width or height as the node will - this does not yet includes the rectangle of the
node being reshaped.
@param finalSize The target size to which the node will snap.
@param snapDelta The snap delta that the mouse needs to be moved in order to snap.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23addSameSizeSnapResult(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2CSnapLineOrientation%2CIEnumerable%3CRect%3E%2Cnumber%2CPoint) Online Documentation}*/
addSameSizeSnapResult(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, orientation: SnapLineOrientation | SnapLineOrientationStringValues, rectangles: IEnumerable<Rect>, finalSize: number, snapDelta: Point): void;
/**
Adds a snap result for a location that snaps the node's bounds to a snap line.
This method is called by {@link NodeReshapeSnapResultProvider#collectSnapLineResults} for each snap line snap.
@param context The context in which the snapping is performed.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data.
@param node The node that is being reshaped.
@param snapLine The snap line to which the node can be snapped
@param delta The amount the mouse needs to be moved in order to get to the snapping location.
@param snapPoint The point of interest that can be highlighted on the snap line.
@param preferredSnapType Whether the cursor should preferably snap horizontally or vertically.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23addSnaplineSnapResult(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2COrthogonalSnapLine%2CPoint%2CPoint%2CSnapTypes) Online Documentation}*/
addSnaplineSnapResult(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, snapLine: OrthogonalSnapLine, delta: Point, snapPoint: Point, preferredSnapType: SnapTypes | SnapTypesConvertible): void;
/**
Collects the snap results for the grid.
@param context The context in which the snapping is performed.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data.
@param node The node that is being reshaped.
@param reshapeContext The reshape context that describes how the node's layout is reshaped.
@param suggestedLayout The suggested layout of the node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23collectGridSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2CReshapeRectangleContext%2CRect) Online Documentation}*/
collectGridSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, reshapeContext: ReshapeRectangleContext, suggestedLayout: Rect): void;
/**
Collects the snap results that make the node the same size as other elements.
This method ultimately delegates to {@link NodeReshapeSnapResultProvider#addSameSizeSnapResult} to add the results
to the event argument.
@param context The context in which the snapping is performed.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data.
@param node The node that is being reshaped.
@param reshapeContext The reshape context that contains information about the nature of the resize.
@param suggestedLayout The layout of the node as it would be if the mouse location would not be snapped.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23collectSameSizeSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2CReshapeRectangleContext%2CRect) Online Documentation}*/
collectSameSizeSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, reshapeContext: ReshapeRectangleContext, suggestedLayout: Rect): void;
/**
Collects the snap results that makes the node's bounds snap to the {@link GraphSnapContext#getFixedNodeSnapLines fixed node snap lines}.
This method ultimately delegates to {@link NodeReshapeSnapResultProvider#addSnaplineSnapResult} to add the results
to the event argument. Method {@link NodeReshapeSnapResultProvider#getSnapLines} is used to query the snap lines
that will be checked by this method.
@param context The context in which the snapping is performed.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data.
@param node The node that is being reshaped.
@param reshapeContext The reshape context that contains information about the nature of the resize.
@param suggestedLayout The layout of the node as it would be if the mouse location would not be snapped.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23collectSnapLineResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2CReshapeRectangleContext%2CRect) Online Documentation}*/
collectSnapLineResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, reshapeContext: ReshapeRectangleContext, suggestedLayout: Rect): void;
/**
Called when a node is {@link IDragHandler#handleMove dragged} to add {@link SnapResult}s for
{@link OrthogonalSnapLine}s or sizes to which this node can potentially snap during resizing.
This implementation delegates to the following methods in order:
<ul>
<li>{@link NodeReshapeSnapResultProvider#collectGridSnapResults}
<li>{@link NodeReshapeSnapResultProvider#collectSameSizeSnapResults}
<li>{@link NodeReshapeSnapResultProvider#collectSnapLineResults}
</ul>
@param context The snap context which manages the snap lines and the settings.
@param evt The event argument to obtain the context from and add the results to.
@param node The node that is being reshaped.
@param reshapeContext Carries information about the reshape process.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23collectSnapResults(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2CReshapeRectangleContext) Online Documentation}*/
collectSnapResults(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, reshapeContext: ReshapeRectangleContext): void;
/**
Helper method for method {@link NodeReshapeSnapResultProvider#collectSnapLineResults} that gets all the {@link SnapLine}
that should be checked for the currently moved node.
This method aggregates the {@link GraphSnapContext#getFixedNodeSnapLines fixed nodes' snaplines} and the {@link GraphSnapContext#getAdditionalSnapLines additional snap lines} that are relevant to the node.
@param context The context in which the snapping is performed.
@param evt The {@link CollectSnapResultsEventArgs} instance containing the event data.
@param node The node that is being reshaped.
@param reshapeContext The reshape context that contains information about the nature of the resize.
@param suggestedLayout The layout of the node as it would be if the mouse location would not be snapped.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23getSnapLines(GraphSnapContext%2CCollectSnapResultsEventArgs%2CINode%2CReshapeRectangleContext%2CRect) Online Documentation}*/
getSnapLines(context: GraphSnapContext, evt: CollectSnapResultsEventArgs, node: INode, reshapeContext: ReshapeRectangleContext, suggestedLayout: Rect): IEnumerable<SnapLine>;
/**
A shared singleton instance of this type.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeReshapeSnapResultProvider%23INSTANCE Online Documentation}*/
static INSTANCE: INodeReshapeSnapResultProvider;
static $class: Class;
private $_NodeReshapeSnapResultProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeReshapeSnapResultProvider;
}
export interface DefaultBendCreator extends YObject, IBendCreator {
/**
@inheritdoc*/
createBend(context: IInputModeContext, graph: IGraph, edge: IEdge, location: Point): number;
} /**
Simple default implementations of the {@link IBendCreator} interface used during the creation of bends in the UI.
An instance of this class can be found in the {@link ILookup#lookup} of an edge, e.g.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultBendCreator Online Documentation}*/
export class DefaultBendCreator {
static $class: Class;
private $_DefaultBendCreator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultBendCreator;
}
export interface PortCandidateProviderBase extends YObject, IPortCandidateProvider {
}
/**
An abstract base implementation of the {@link IPortCandidateProvider} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase Online Documentation}*/
export abstract class PortCandidateProviderBase {
/**
Adds candidates for all existing {@link IPortOwner#ports} of the <code>owner</code> to the provided list.
@param owner The port owner whose ports should be added.
@param list The list to add candidates for existing ports to.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23addExistingPorts(IPortOwner%2CIList%3CIPortCandidate%3E) Online Documentation}*/
addExistingPorts(owner: IPortOwner, list: IList<IPortCandidate>): void;
/**
Factory method that creates a simple candidate that will use the {@link PortCandidateProviderBase#createInstance}
method of this instance to delegate {@link IPortCandidate#createPort} queries to.
@param locationParameter The location of the candidate. This instance is assigned by reference to the candidate.
@param owner The owner to use for the candidate.
@param [valid=true] Whether the port should be {@link PortCandidateValidity#VALID}.
@returns A candidate whose {@link IPortCandidate#createPort} method delegates to this instance's
{@link PortCandidateProviderBase#createInstance}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23createCandidate(IPortOwner%2CIPortLocationModelParameter%2Cboolean) Online Documentation}*/
createCandidate(owner: IPortOwner, locationParameter: IPortLocationModelParameter, valid?: boolean): DefaultPortCandidate;
/**
Callback method used by the ports created using the factory methods
{@link PortCandidateProviderBase#createCandidate}.
This method can be overridden by subclasses to perform more sophisticated port creation logic.
@param context The context in which the instance is created.
@param candidate The candidate to create a port for.
@returns The new port.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23createInstance(IInputModeContext%2CDefaultPortCandidate) Online Documentation}*/
createInstance(context: IInputModeContext, candidate: DefaultPortCandidate): IPort;
/**
Returns all source port candidates that belong to the context of this provider.
<h2>Note</h2>
This implementation simply delegates to {@link PortCandidateProviderBase#getPortCandidates}.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param context
@returns An enumerable over all source port candidates that are associated with the current context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23getAllSourcePortCandidates(IInputModeContext) Online Documentation}*/
getAllSourcePortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
/**
Returns all target port candidates that belong to the context of this provider.
<h2>Note</h2>
This implementation simply delegates to {@link PortCandidateProviderBase#getPortCandidates}.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param context
@returns An enumerable over all target port candidates that are associated with the current context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23getAllTargetPortCandidates(IInputModeContext) Online Documentation}*/
getAllTargetPortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
/**
Callback method used by the ports created using the factory methods
{@link PortCandidateProviderBase#createCandidate}.
This method can be overridden by subclasses to perform more sophisticated candidate creation logic.
@param context The context in which the instance is created.
@param candidate The candidate to create a port for.
@param suggestedLocation The location to get a candidate for.
@returns The new candidate whose {@link IPortCandidate#validity} may not be {@link PortCandidateValidity#DYNAMIC}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23getPortCandidateAt(IInputModeContext%2CDefaultPortCandidate%2CPoint) Online Documentation}*/
getPortCandidateAt(context: IInputModeContext, candidate: DefaultPortCandidate, suggestedLocation: Point): IPortCandidate;
/**
Creates an enumeration of possibly port candidates.
This method is used as a callback by most of the getter methods in this class. Subclasses should override this method to
provide the same candidates for all use-cases.
@param context The context for which the candidates should be provided.
@returns An enumerable collection of port candidates.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23getPortCandidates(IInputModeContext) Online Documentation}*/
abstract getPortCandidates(context: IInputModeContext): IEnumerable<IPortCandidate>;
/**
Returns all port candidates that apply for the provided opposite port candidate.
<h2>Note</h2>
This implementation simply delegates to {@link PortCandidateProviderBase#getPortCandidates}.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param context
@param target
@returns An enumerable over all port candidates that are associated with the given opposite port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23getSourcePortCandidates(IInputModeContext%2CIPortCandidate) Online Documentation}*/
getSourcePortCandidates(context: IInputModeContext, target: IPortCandidate): IEnumerable<IPortCandidate>;
/**
Returns all port candidates that apply for the provided opposite port candidate.
<h2>Note</h2>
This implementation simply delegates to {@link PortCandidateProviderBase#getPortCandidates}.
The enumerable may both contain {@link PortCandidateValidity valid, invalid, and dynamic} port candidates.
@param context
@param source
@returns An enumerable over all port candidates that are associated with the given opposite port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23getTargetPortCandidates(IInputModeContext%2CIPortCandidate) Online Documentation}*/
getTargetPortCandidates(context: IInputModeContext, source: IPortCandidate): IEnumerable<IPortCandidate>;
/**
Gets or sets the style to use for the creation of the port in {@link PortCandidateProviderBase#createInstance}.
A value of <code>null</code> will make the implementation use the corresponding current default port style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23style Online Documentation}*/
style: IPortStyle | null;
/**
Gets or sets the tag to assign to the port in {@link PortCandidateProviderBase#createInstance}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateProviderBase%23tag Online Documentation}*/
tag: any | null;
static $class: Class;
private $_PortCandidateProviderBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortCandidateProviderBase;
}
export interface PortRelocationHandle extends YObject, IHandle {
/**
@inheritdoc*/
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
/**
@inheritdoc*/
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
initializeDrag(context: IInputModeContext): void;
/**
@inheritdoc*/
type: HandleTypes;
/**
@inheritdoc*/
cursor: Cursor;
} /**
* A sophisticated implementation of the {@link IHandle} interface that lets the user change an edge's {@link IEdge#sourcePort source} and
* {@link IEdge#targetPort target port}.
* This handle will move one of the edge's end points visually to another {@link IPort} or {@link IPortCandidate}
* and upon finalization of the gesture will {@link PortRelocationHandle#setPorts set the new ports} for the edge. This class heavily relies on the
* implementation of the {@link IEdgeReconnectionPortCandidateProvider} that is queried from the {@link IEdge}
* this handle is acting on.
*
* Instances of this {@link IHandle} are provided by the {@link PortRelocationHandleProvider}. The most commonly
* used settings are available on the {@link PortRelocationHandleProvider}, too:
*
* <pre>
* graph.decorator.edgeDecorator.edgePortHandleProviderDecorator.setFactory(edge => {
*   const portRelocationHandleProvider = new PortRelocationHandleProvider(graph, edge)
*   portRelocationHandleProvider.visualization = Visualization.LIVE
*   portRelocationHandleProvider.showPortCandidates = false
*   return portRelocationHandleProvider
* })
* </pre>
*
* For less frequently used settings a custom {@link PortRelocationHandleProvider} has to be provided and its {@link PortRelocationHandleProvider#createPortRelocationHandle}
* method has to be overridden to provide the configured {@link PortRelocationHandle}:
*
* <pre>
* graph.decorator.edgeDecorator.edgePortHandleProviderDecorator.setFactory(edge => {
*   const portRelocationHandleProvider = new MyHandleProvider(graph, edge)
*   portRelocationHandleProvider.visualization = Visualization.LIVE
*   portRelocationHandleProvider.showPortCandidates = false
*   return portRelocationHandleProvider
* })
*
* class MyHandleProvider extends PortRelocationHandleProvider {
*   /**
*    * @param {IGraph} graph
*    * @param {IEdge} edge
*    *‍/
*   constructor(graph, edge) {
*     super(graph, edge)
*   }
*
*   /**
*    * @param {IGraph} graph
*    * @param {IEdge} edge
*    * @param {boolean} sourceEnd
*    *‍/
*   createPortRelocationHandle(graph, edge, sourceEnd) {
*     const portRelocationHandle = new PortRelocationHandle(graph, edge, sourceEnd)
*     portRelocationHandle.addExistingPort = true
*     return portRelocationHandle
*   }
* }
* </pre>
*
* @see PortRelocationHandle#getPortCandidates
* @see EdgeDecorator#edgeReconnectionPortCandidateProviderDecorator
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle Online Documentation}
*/
export class PortRelocationHandle {
/**
Creates a new instance of the {@link PortRelocationHandle} class.
@param graph The graph or <code>null</code>. If the graph instance is not specified, the handle will try to receive the graph from the {@link IInputModeContext}
it is passed in during {@link PortRelocationHandle#initializeDrag}.
@param edge The edge.
@param sourceEnd if set to <code>true</code> the source end will be subject to relocation, otherwise it will be the target port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle Online Documentation}*/
constructor(graph: IGraph | null, edge: IEdge, sourceEnd: boolean);
/**
Factory method that creates the {@link ICanvasObjectDescriptor} that will be used to paint the "current"
{@link IPortCandidate}.
@returns The descriptor to use for the rendering in the {@link CanvasComponent}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23createCurrentPortCandidateDescriptor() Online Documentation}*/
createCurrentPortCandidateDescriptor(): ICanvasObjectDescriptor;
/**
Factory method that creates the dummy edge that will be shown during the drag operation.
@param edge The edge to be represented by the dummy edge.
@returns The dummy edge instance to use.
@see PortRelocationHandle#visualization
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23createDummyEdge(IEdge) Online Documentation}*/
createDummyEdge(edge: IEdge): SimpleEdge;
/**
Factory method that creates a dummy {@link IVisualCreator} for the dummy edge during the drag operation.
Only used if {@link PortRelocationHandle#visualization} is set to {@link Visualization#DUMMY}.
@param dummy The dummy to create an {@link IVisualCreator} for.
@returns An {@link IVisualCreator} that shows the dummy.
@see PortRelocationHandle#visualization
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23createDummyEdgeVisualCreator(SimpleEdge) Online Documentation}*/
createDummyEdgeVisualCreator(dummy: SimpleEdge): IVisualCreator;
/**
Factory method that creates the candidate for the existing port.
@param port The port to create a candidate for.
@returns The candidate or <code>null</code>.
@see PortRelocationHandle#getPortCandidates
@see PortRelocationHandle#addExistingPort
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23createExistingPortCandidate(IPort) Online Documentation}*/
createExistingPortCandidate(port: IPort): IPortCandidate | null;
/**
Factory method that creates the {@link ICanvasObjectDescriptor} that will be used to paint the
{@link IPortCandidate}s.
@returns The descriptor to use for the rendering in the {@link CanvasComponent}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23createPortCandidateDescriptor() Online Documentation}*/
createPortCandidateDescriptor(): ICanvasObjectDescriptor;
/**
Filters the given <code>allCandidates</code> to return the subset of candidates which are relevant at the given <code>location</code>.
If {@link PortRelocationHandle#showHitPortOwnerCandidatesOnly} is set to <code>true</code> this implementation returns only the candidates of the port owner at the given location. Otherwise it
returns the entire <code>allCandidates</code>.
Developers can override this method to apply a different filter, e.g. all port candidates in a defined radius around the
<code>location</code>.
@param context The context that is used to retrieve the candidates for.
@param location The current location.
@param allCandidates All possible port candidates.
@returns The filtered list of port candidates.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23filterCandidates(IInputModeContext%2CPoint%2CIEnumerable%3CIPortCandidate%3E) Online Documentation}*/
filterCandidates(context: IInputModeContext, location: Point, allCandidates: IEnumerable<IPortCandidate>): IEnumerable<IPortCandidate>;
/**
Finds the closest candidate from the given set of candidates.
@param context The context in for which the closest handle candidate is sought.
@param location The location for which to find a candidate.
@param candidates The list of possible candidates.
@returns The closes candidate or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23getClosestCandidate(IInputModeContext%2CPoint%2CIEnumerable%3CIPortCandidate%3E) Online Documentation}*/
getClosestCandidate(context: IInputModeContext, location: Point, candidates: IEnumerable<IPortCandidate>): IPortCandidate | null;
/**
Gets the {@link IPortOwner} at the given <code>location</code>.
@param context The input mode context.
@param location The location to find a port owner at.
@returns The {@link IPortOwner} at the given location or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23getCurrentOwner(IInputModeContext%2CPoint) Online Documentation}*/
getCurrentOwner(context: IInputModeContext, location: Point): IPortOwner | null;
/**
Gets the graph to use from the context.
@param context The input mode context.
@returns A graph instance or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23getGraph(IInputModeContext) Online Documentation}*/
getGraph(context: IInputModeContext | null): IGraph | null;
/**
Called during {@link PortRelocationHandle#dragFinished} to actually get the new port from the chosen candidate.
This implementation returns the result of {@link IPortCandidate#createPort} or calls {@link IGraph#addPort} if
the result was <code>null</code>.
@param context
@param portCandidate The candidate to get a port instance from.
@param suggestedLocation The suggested location for the port.
@returns A non-<code>null</code> port instance.
@see PortRelocationHandle#setPorts
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23getPort(IInputModeContext%2CIPortCandidate%2CPoint) Online Documentation}*/
getPort(context: IInputModeContext, portCandidate: IPortCandidate, suggestedLocation: Point | null): IPort;
/**
Gets the possible candidates for the given edge.
This implementation uses the {@link IEdgeReconnectionPortCandidateProvider} from the edge's {@link ILookup#lookup}
to retrieve the candidate sets. If {@link PortRelocationHandle#addExistingPort} is enabled, the existing port will be part of the candidates.
@param context The context that is used to retrieve the candidates for.
@param edge The edge.
@param sourcePort Whether to look for source port candidates.
@returns A non-<code>null</code> enumerable over the candidates.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23getPortCandidates(IInputModeContext%2CIEdge%2Cboolean) Online Documentation}*/
getPortCandidates(context: IInputModeContext, edge: IEdge, sourcePort: boolean): IEnumerable<IPortCandidate>;
/**
Retrieves the port candidates during {@link PortRelocationHandle#handleMove}.
This implementation returns candidates which were collected in {@link PortRelocationHandle#initializeDrag}.
Developers who want to newly determine the candidates with each move might override this method to delegate to {@link PortRelocationHandle#getPortCandidates}
instead.
@param context The context that is used to retrieve the candidates for.
@param edge The edge.
@param sourcePort Whether to look for source port candidates.
@returns A non-<code>null</code> enumerable over the candidates.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23getPortCandidatesDuringMove(IInputModeContext%2CIEdge%2Cboolean) Online Documentation}*/
getPortCandidatesDuringMove(context: IInputModeContext, edge: IEdge, sourcePort: boolean): IEnumerable<IPortCandidate>;
/**
Hides the original edge that during the drag operation.
Only called if {@link PortRelocationHandle#visualization} is set to {@link Visualization#DUMMY}.
@param edge The edge to hide.
@param context The context to get the canvas the edge should be hidden from.
@see PortRelocationHandle#unhideOriginalEdge
@see PortRelocationHandle#visualization
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23hideOriginalEdge(IInputModeContext%2CIEdge) Online Documentation}*/
hideOriginalEdge(context: IInputModeContext, edge: IEdge): void;
/**
Determines whether {@link PortRelocationHandle#resolveCandidate port candidate resolution} is enabled for the current gesture.
This implementation checks whether the the parent input mode is an instance of {@link HandleInputMode}, the {@link PortRelocationHandle#portCandidateResolutionRecognizer}
recognizes its {@link MouseEventArgs}, and the {@link HandleInputMode#currentHandle}'s location is equal to this {@link PortRelocationHandle#location}.
@param context The context to inspect.
@returns <code>true</code> if port candidates may be resolved; <code>false</code> otherwise.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23isPortCandidateResolutionEnabled(IInputModeContext) Online Documentation}*/
isPortCandidateResolutionEnabled(context: IInputModeContext): boolean;
/**
Tries to {@link IPortCandidate#getPortCandidateAt resolve} a dynamic port candidate for the given location.
@param context The context in which the candidate is resolved.
@param portCandidate The candidate.
@param location The current location.
@returns A candidate.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23resolveCandidate(IInputModeContext%2CIPortCandidate%2CPoint) Online Documentation}*/
resolveCandidate(context: IInputModeContext, portCandidate: IPortCandidate, location: Point): IPortCandidate;
/**
Updates the closest candidate for visual feedback.
@param portCandidate
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23setClosestCandidate(IPortCandidate) Online Documentation}*/
setClosestCandidate(portCandidate: IPortCandidate | null): void;
/**
Triggered by {@link PortRelocationHandle#dragFinished} to actually change the port.
@param context The context.
@param edge The edge to change ports.
@param setSourcePort Whether to set the source port. <code>false</code> for target ports.
@param portCandidate The candidate that has been chosen.
@param suggestedLocation The suggested location for the port.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23setPort(IInputModeContext%2CIEdge%2Cboolean%2CIPortCandidate%2CPoint) Online Documentation}*/
setPort(context: IInputModeContext, edge: IEdge, setSourcePort: boolean, portCandidate: IPortCandidate, suggestedLocation: Point | null): void;
/**
Finally sets the ports for the edge to the new values.
@param context The context in which the ports are set.
@param edge The edge to set the ports for.
@param sourcePort The (possibly) new source port.
@param targetPort The (possibly) new target port.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23setPorts(IInputModeContext%2CIEdge%2CIPort%2CIPort) Online Documentation}*/
setPorts(context: IInputModeContext, edge: IEdge, sourcePort: IPort, targetPort: IPort): void;
/**
Updates the position of the handle.
@param location The new location of the handle.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23setPosition(Point) Online Documentation}*/
setPosition(location: Point): void;
/**
Updates the dummy edge's visual appearance to reflect the new port candidate.
@param dummy The dummy edge.
@param source Whether to update the source or target port.
@param portCandidate The new candidate to indicate.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23setToPortCandidate(SimpleEdge%2Cboolean%2CIPortCandidate) Online Documentation}*/
setToPortCandidate(dummy: SimpleEdge, source: boolean, portCandidate: IPortCandidate): void;
/**
Unhides the original edge that was hidden during the drag operation.
@param edge The edge to unhide.
@param canvas The canvas the edge was hidden from.
@see PortRelocationHandle#hideOriginalEdge
@see PortRelocationHandle#visualization
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23unhideOriginalEdge(IEdge%2CCanvasComponent) Online Documentation}*/
unhideOriginalEdge(edge: IEdge, canvas: CanvasComponent | null): void;
/**
Draws the highlight for the owner of the current port candidate.
This implementation retrieves the {@link HighlightIndicatorManager} from the {@link PortRelocationHandle#inputModeContext}'s {@link ILookup}.
Overriders have to take care of removal of the highlight of the <code>oldCandidate</code>'s owner, too.
@param oldCandidate The port candidate whose owner is currently highlighted.
@param newCandidate The port candidate whose owner should be highlighted.
@see PortRelocationHandle#showTargetHighlight
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23updateHighlight(IPortCandidate%2CIPortCandidate) Online Documentation}*/
updateHighlight(oldCandidate: IPortCandidate | null, newCandidate: IPortCandidate | null): void;
/**
A {@link string} to define the pen for the visualization if {@link PortRelocationHandle#visualization} is set to {@link Visualization#GHOST}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23GHOST_VISUALIZATION_STROKE_KEY Online Documentation}*/
static GHOST_VISUALIZATION_STROKE_KEY: string;
/**
Gets or sets a value indicating whether to show the candidates of hit {@link IPortOwner}s only.
@see PortRelocationHandleProvider#showHitPortOwnerCandidatesOnly
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23showHitPortOwnerCandidatesOnly Online Documentation}*/
showHitPortOwnerCandidatesOnly: boolean;
/**
Gets or sets a value indicating whether to show port candidates.
@see PortRelocationHandleProvider#showPortCandidates
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23showPortCandidates Online Documentation}*/
showPortCandidates: boolean;
/**
Gets or sets the maximum distance the pointer may be away of a candidate in order to be snapped to the candidate.
The distance is measured in view coordinates. The default value is <code>50.0</code>.
@see PortRelocationHandleProvider#maximumSnapDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23maximumSnapDistance Online Documentation}*/
maximumSnapDistance: number;
/**
Gets or sets whether or not edges are allowed to connect to other edges.
The default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23allowEdgeToEdgeConnections Online Documentation}*/
allowEdgeToEdgeConnections: boolean;
/**
Gets or sets whether or not cyclic port dependencies are allowed.
If {@link PortRelocationHandle#allowEdgeToEdgeConnections edge to edge connections} are allowed by this instance, this property determines whether
{@link PortRelocationHandle#getClosestCandidate}s are will yield candidates that would result in the edge being
modified would create a cyclic dependency. The default is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23allowCyclicPortDependencies Online Documentation}*/
allowCyclicPortDependencies: boolean;
/**
Gets or sets a property that determines whether the existing port should be available as a possible port candidate,
too.
The default is <code>true</code>. If this property is set to <code>false</code>, {@link PortRelocationHandle#getPortCandidates} will return
only the ports provided by the {@link IPortCandidateProvider} in the node's lookup.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23addExistingPort Online Documentation}*/
addExistingPort: boolean;
/**
Gets or sets whether the owner of the current {@link IPortCandidate} should be highlighted.
The default is <code>true</code>.
@see PortRelocationHandle#updateHighlight
@see PortRelocationHandleProvider#showTargetHighlight
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23showTargetHighlight Online Documentation}*/
showTargetHighlight: boolean;
/**
Gets or sets a value indicating whether a {@link PortCandidateValidity#DYNAMIC dynamic port candidate} should be resolved for the current mouse location.
By default this feature is enabled and the {@link PortRelocationHandle#portCandidateResolutionRecognizer} is configured so that the resolution mechanism is used if the user {@link KeyEventRecognizers#SHIFT_IS_DOWN holds the shift modifier}
during the gesture.
@see PortRelocationHandle#portCandidateResolutionRecognizer
@see PortRelocationHandle#resolveCandidate
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23resolvePortCandidates Online Documentation}*/
resolvePortCandidates: boolean;
/**
Gets or sets the event recognizer that determines whether to resolve a {@link PortCandidateValidity#DYNAMIC dynamic port candidate}.
Note that this property has no effect if {@link PortRelocationHandle#resolvePortCandidates} is set to <code>false</code>.
By default this is set to {@link KeyEventRecognizers#SHIFT_IS_DOWN}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23portCandidateResolutionRecognizer Online Documentation}*/
portCandidateResolutionRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets a value indicating whether the source end of the edge is handled by this instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23sourceEnd Online Documentation}*/
readonly sourceEnd: boolean;
/**
Gets the edge this instance acts upon.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23edge Online Documentation}*/
readonly edge: IEdge;
/**
Gets the dummy edge this instance uses during the edit.
Only used if {@link PortRelocationHandle#visualization} is set to {@link Visualization#DUMMY}.
@see PortRelocationHandle#visualization
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23dummyEdge Online Documentation}*/
readonly dummyEdge: SimpleEdge | null;
/**
Gets or sets how the changed edge should be shown during the gesture.
This property must not be changed during the gesture.
If this property is set to {@link Visualization#LIVE} it is crucial that {@link PortRelocationHandle constructor}'s <code>graph</code> parameter is not <code>null</code>
and its <code>edge</code> parameter is an edge which is contained in that graph.
Default is {@link Visualization#DUMMY}.
@see PortRelocationHandleProvider#visualization
@default Visualization.DUMMY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23visualization Online Documentation}*/
visualization: Visualization;
/**
Gets or sets the context to operate on.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23inputModeContext Online Documentation}*/
inputModeContext: IInputModeContext | null;
/**
Gets a view of the location of the item.
This implementation returns <code>this</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandle%23location Online Documentation}*/
readonly location: IPoint;
static $class: Class;
private $_PortRelocationHandle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortRelocationHandle;
}
export interface PortRelocationHandleProvider extends YObject, IHandleProvider, IEdgePortHandleProvider {
/**
@inheritdoc*/
getHandles(context: IInputModeContext): IEnumerable<IHandle>;
} /**
* Simple implementation of a {@link IHandleProvider} that can be used for {@link IEdge}s.
* It will return the {@link IHandle}s for the {@link IEdge#sourcePort source} and {@link IEdge#targetPort target} ports using {@link PortRelocationHandle}s.
*
* The properties {@link PortRelocationHandleProvider#showHitPortOwnerCandidatesOnly}, {@link PortRelocationHandleProvider#maximumSnapDistance} and {@link PortRelocationHandleProvider#visualization} will only be used in {@link PortRelocationHandleProvider#createPortRelocationHandle} to
* configure the created {@link PortRelocationHandle}s.
*
* Note that this implementation also explicitly implements the {@link IEdgePortHandleProvider} and that it is
* recommended to decorate the {@link IEdgePortHandleProvider} interface with edges in order to influence the handles
* of the source and target port of the edge instead of directly decorating and implementing the {@link IHandleProvider}
* interface. The default implementation of the {@link IHandleProvider} interface in the lookup of the
* {@link IEdge}s (see {@link DefaultGraph#DEFAULT_EDGE_LOOKUP}) will use the {@link IEdgeReconnectionPortCandidateProvider} interface to collect the
* source and target port handles.
*
* This implementation provides instances of {@link PortRelocationHandle}. The most commonly used settings for {@link PortRelocationHandle}
* are available here, too:
*
* <pre>
* graph.decorator.edgeDecorator.edgePortHandleProviderDecorator.setFactory(edge => {
*   const portRelocationHandleProvider = new PortRelocationHandleProvider(graph, edge)
*   portRelocationHandleProvider.visualization = Visualization.LIVE
*   portRelocationHandleProvider.showPortCandidates = false
*   return portRelocationHandleProvider
* })
* </pre>
*
* For less frequently used settings a custom {@link PortRelocationHandleProvider} has to be provided and its {@link PortRelocationHandleProvider#createPortRelocationHandle}
* method has to be overridden to provide the configured {@link PortRelocationHandle}:
*
* <pre>
* graph.decorator.edgeDecorator.edgePortHandleProviderDecorator.setFactory(edge => {
*   const portRelocationHandleProvider = new MyHandleProvider(graph, edge)
*   portRelocationHandleProvider.visualization = Visualization.LIVE
*   portRelocationHandleProvider.showPortCandidates = false
*   return portRelocationHandleProvider
* })
*
* class MyHandleProvider extends PortRelocationHandleProvider {
*   /**
*    * @param {IGraph} graph
*    * @param {IEdge} edge
*    *‍/
*   constructor(graph, edge) {
*     super(graph, edge)
*   }
*
*   /**
*    * @param {IGraph} graph
*    * @param {IEdge} edge
*    * @param {boolean} sourceEnd
*    *‍/
*   createPortRelocationHandle(graph, edge, sourceEnd) {
*     const portRelocationHandle = new PortRelocationHandle(graph, edge, sourceEnd)
*     portRelocationHandle.addExistingPort = true
*     return portRelocationHandle
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider Online Documentation}
*/
export class PortRelocationHandleProvider {
/**
Initializes a new instance of the {@link PortRelocationHandleProvider} class using the given graph and edge.
@param graph The graph or null. If the graph is null, the handles will try to receive the graph from the {@link IInputModeContext}
it is used in.
@param edge The edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider Online Documentation}*/
constructor(graph: IGraph | null, edge: IEdge);
/**
Factory method that creates the handle for this provider.
@param graph The graph that contains the edge.
@param edge The edge.
@param sourcePort if set to <code>true</code> the handle for the source port is sought.
@returns A handle or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider%23createPortRelocationHandle(IGraph%2CIEdge%2Cboolean) Online Documentation}*/
createPortRelocationHandle(graph: IGraph | null, edge: IEdge, sourcePort: boolean): IHandle | null;
/**
Gets or sets a value indicating whether to show the candidates of hit {@link IPortOwner}s only.
@see PortRelocationHandle#showHitPortOwnerCandidatesOnly
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider%23showHitPortOwnerCandidatesOnly Online Documentation}*/
showHitPortOwnerCandidatesOnly: boolean;
/**
Gets or sets a value indicating whether to show port candidates.
@see PortRelocationHandle#showPortCandidates
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider%23showPortCandidates Online Documentation}*/
showPortCandidates: boolean;
/**
Gets or sets whether the owner of the current {@link IPortCandidate} should be highlighted.
The default is <code>true</code>.
@see PortRelocationHandle#showTargetHighlight
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider%23showTargetHighlight Online Documentation}*/
showTargetHighlight: boolean;
/**
Gets or sets the maximum distance the pointer may be away of a candidate in order to be snapped to the candidate.
The distance is measured in view coordinates. The default value is <code>50.0</code>.
@see PortRelocationHandle#maximumSnapDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider%23maximumSnapDistance Online Documentation}*/
maximumSnapDistance: number;
/**
Gets or sets how the changed edge should be shown during the gesture.
If this property is set to {@link Visualization#LIVE} it is crucial that {@link PortRelocationHandleProvider#graph}
is not <code>null</code> and {@link PortRelocationHandleProvider#edge} is contained in that graph.
Default is {@link Visualization#DUMMY}.
@see PortRelocationHandle#visualization
@default Visualization.DUMMY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider%23visualization Online Documentation}*/
visualization: Visualization;
/**
The graph this provider is working on.
@protected
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider%23graph Online Documentation}*/
graph: IGraph | null;
/**
The edge this provider is working on.
@protected
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortRelocationHandleProvider%23edge Online Documentation}*/
edge: IEdge;
static $class: Class;
private $_PortRelocationHandleProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortRelocationHandleProvider;
}
export interface PortsHandleProvider extends YObject, IHandleProvider {
}
/**
A simple implementation of an {@link IHandleProvider} that returns all handles in the lookup of the ports owned by
the given owner.
Using this class to display port handles will result in exceptions if the underlying port instances are removed from the
graph while the handles are displayed. This can happen, for example, through <code>Undo</code> or if ports are removed
programmatically with
{@link IGraph#remove}. In that case, you need to query the handles again before interacting with the graph, for
example by temporarily unselecting and reselecting the port-owning nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortsHandleProvider Online Documentation}*/
export class PortsHandleProvider {
/**
Creates an instance using the given owner as the provider for the ports.
@param owner
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortsHandleProvider Online Documentation}*/
constructor(owner: IPortOwner);
/**
Retrieves the handle implementation from the port's lookup.
@param port
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortsHandleProvider%23getHandle(IPort) Online Documentation}*/
getHandle(port: IPort): IHandle | null;
/**
Returns a collection of zero or more {@link IHandle} implementations that are associated with the ports of this
context.
@param context
@returns A collection of handles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortsHandleProvider%23getHandles(IInputModeContext) Online Documentation}*/
getHandles(context: IInputModeContext): IEnumerable<IHandle>;
/**
The owner of the ports.
@protected
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortsHandleProvider%23portOwner Online Documentation}*/
portOwner: IPortOwner;
static $class: Class;
private $_PortsHandleProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortsHandleProvider;
}
/**
Enumeration with different options on how to handle previews during a gesture.
Used by some input gestures (e.g. edge relocation and label movement) to determine the way the changed item is
visualized during the gesture.
@see PortRelocationHandle
@see MoveLabelInputMode
@see LabelPositionHandler
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Visualization Online Documentation}*/
export enum Visualization {
/**
No preview during the gesture, although some sort of visual feedback may be present.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Visualization%23NONE Online Documentation}*/
NONE,
/**
A primitive visualization is displayed during the gesture.
The unchanged original is still visible during the gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Visualization%23GHOST Online Documentation}*/
GHOST,
/**
A dummy item is displayed during the gesture, while the original one is hidden.
This option has better compatibility than {@link Visualization#LIVE} since the graph is not changed during the
gesture, but may cause visual glitches in some circumstances, especially with items that depend on the one that is
currently manipulated. In many cases, however, this option still looks accurate enough.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Visualization%23DUMMY Online Documentation}*/
DUMMY,
/**
The item is changed directly during the gesture.
This options offers the best visual fidelity and during the gesture everything should be an accurate preview of how
things look after finishing it. However, since the actual item is changed, this may cause graph events to be raised
during the gesture.
Generally, if you listen to events on {@link IGraph}, this option should best be avoided.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Visualization%23LIVE Online Documentation}*/
LIVE
}
/** Automatic conversion to {@link Visualization}. */
type VisualizationStringValues = "none" | "ghost" | "dummy" | "live";
export interface ResizeStripeInputMode extends YObject, IInputMode {
/**
@inheritdoc*/
tryStop(): boolean;
/**
@inheritdoc*/
priority: number;
/**
@inheritdoc*/
readonly inputModeContext: IInputModeContext | null;
} /**
Specialized input mode that is used to resize a stripe by dragging one of its borders.
By default, adjacent stripes are moved. To redistribute the space instead, use {@link ResizeStripeInputMode#resizeNeighborsRecognizer}. Also, by default the content of the
stripes is either moved or constrains the minimum available sizes for the stripe resize operations. To completely ignore
all content nodes, use {@link ResizeStripeInputMode#ignoreContentRecognizer}.
This mode is {@link ResizeStripeInputMode#exclusive} by default.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode Online Documentation}*/
export class ResizeStripeInputMode {
/**
Creates a new instance of this input mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode Online Documentation}*/
constructor();
/**
Cancels the editing of this mode.
This implementation {@link ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
{@link ResizeStripeInputMode#onCanceled}.
@see ResizeStripeInputMode#onCanceled
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23cancel() Online Documentation}*/
cancel(): void;
/**
Installs this mode into the given context that is provided by the canvas.
In general a mode can only be installed into a single canvas at all times.
This method is called to initialize this instance. Subclasses should override this method to register the corresponding
event handler delegates for the various input events they need to register with.
Overriding implementations should call the base implementation, first.
@param context The context that this instance shall be installed into. The same instance will be passed to this instance during
{@link IInputMode#uninstall}. A reference to the context may be kept and queried during the time the mode is
installed.
@param controller The {@link ResizeStripeInputMode#controller} for this mode.
@see IInputMode#uninstall
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23install(IInputModeContext%2CConcurrencyController) Online Documentation}*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
Called to determine whether it is valid to begin the gesture
This implementation checks whether a stripe border has been hit at the latest {@link MouseEventArgs}
@param source
@param evt
@returns true iff a stripe border has been hit at the latest {@link MouseEventArgs}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23isValidBegin(Object%2CEventArgs) Online Documentation}*/
isValidBegin(source: any, evt: EventArgs): boolean;
/**
Called after {@link ResizeStripeInputMode#cancel} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been canceled.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onCanceled() Online Documentation}*/
onCanceled(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>true</code>.
Can be overridden in subclasses to perform additional actions after the mode has been activated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onConcurrencyControllerActivated() Online Documentation}*/
onConcurrencyControllerActivated(): void;
/**
Called after the {@link ConcurrencyController#active} property of the installed {@link ConcurrencyController} has been set to <code>false</code>.
Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
Overriding implementations should call the base implementation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onConcurrencyControllerDeactivated() Online Documentation}*/
onConcurrencyControllerDeactivated(): void;
/**
Triggers the {@link ResizeStripeInputMode#addDragCanceledListener DragCanceled} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onDragCanceled(InputModeEventArgs) Online Documentation}*/
onDragCanceled(evt: InputModeEventArgs): void;
/**
Triggers the {@link ResizeStripeInputMode#addDragCancelingListener DragCanceling} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onDragCanceling(InputModeEventArgs) Online Documentation}*/
onDragCanceling(evt: InputModeEventArgs): void;
/**
Called once the drag has been finalized.
This method triggers the {@link ResizeStripeInputMode#addDragFinishedListener DragFinished}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onDragFinished(InputModeEventArgs) Online Documentation}*/
onDragFinished(evt: InputModeEventArgs): void;
/**
Called before the drag will be finalized.
This method triggers the {@link ResizeStripeInputMode#addDragFinishingListener DragFinishing}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onDragFinishing(InputModeEventArgs) Online Documentation}*/
onDragFinishing(evt: InputModeEventArgs): void;
/**
Called at the end of each drag.
This method triggers the {@link ResizeStripeInputMode#addDraggedListener Dragged}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onDragged(InputModeEventArgs) Online Documentation}*/
onDragged(evt: InputModeEventArgs): void;
/**
Called at the start of each drag.
This method triggers the {@link ResizeStripeInputMode#addDraggingListener Dragging}
event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onDragging(InputModeEventArgs) Online Documentation}*/
onDragging(evt: InputModeEventArgs): void;
/**
Triggers the {@link ResizeStripeInputMode#addDragStartedListener DragStarted} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onDragStarted(InputModeEventArgs) Online Documentation}*/
onDragStarted(evt: InputModeEventArgs): void;
/**
Triggers the {@link ResizeStripeInputMode#addDragStartingListener DragStarting} event.
@param evt The event argument that contains context information.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onDragStarting(InputModeEventArgs) Online Documentation}*/
onDragStarting(evt: InputModeEventArgs): void;
/**
Called after {@link ResizeStripeInputMode#tryStop} has been called.
Can be overridden in subclasses to perform additional actions after the mode has been stopped.
This implementation does nothing.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23onStopped() Online Documentation}*/
onStopped(): void;
/**
Uninstalls this mode from the given context.
This code should clean up all changes made to the canvas in the {@link IInputMode#install} method. After a mode has
been uninstalled it can be installed again into the same or another canvas.
Overriding implementations should call the base implementation after their own code.
@param context The context to deregister from. This is the same instance that had been passed to {@link IInputMode#install} during
installation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23uninstall(IInputModeContext) Online Documentation}*/
uninstall(context: IInputModeContext): void;
/**
Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
The value of this property will be delegated to the {@link ConcurrencyController#exclusive} property of the {@link ResizeStripeInputMode#controller}.
If this mode is marked as exclusive and has the mutex, all other modes {@link MultiplexingInputMode#add added} to the same {@link MultiplexingInputMode}
will be deactivated. Otherwise it will always run concurrently with all other modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23exclusive Online Documentation}*/
exclusive: boolean;
/**
Gets or sets the enabled state of this input mode.
Clients can use this property to disable or reenable this instance. This will set the {@link ConcurrencyController#enabled Enabled} property of the installed
{@link ResizeStripeInputMode#controller} so a disabled instance should never try to acquire the input mutex.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23enabled Online Documentation}*/
enabled: boolean;
/**
Gets the installed {@link ResizeStripeInputMode#controller}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23controller Online Documentation}*/
readonly controller: ConcurrencyController | null;
/**
Gets or sets the event recognizer that determines whether to start resizing the stripe via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23pressedRecognizerTouch Online Documentation}*/
pressedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to move a handle via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23draggedRecognizerTouch Online Documentation}*/
draggedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish resizing the stripe via touch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23releasedRecognizerTouch Online Documentation}*/
releasedRecognizerTouch: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to start resizing the stripe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23pressedRecognizer Online Documentation}*/
pressedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to move a handle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23draggedRecognizer Online Documentation}*/
draggedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to cancel the resize operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23cancelRecognizer Online Documentation}*/
cancelRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to finish resizing the stripe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23releasedRecognizer Online Documentation}*/
releasedRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to resize adjacent stripes instead of moving them.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23resizeNeighborsRecognizer Online Documentation}*/
resizeNeighborsRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets the event recognizer that determines whether to ignore the content of all stripes instead of moving them or
considering for minimum sizes.
If this recognizer recognizes its gesture, the behavior will be to completely ignore all stripe contents.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23ignoreContentRecognizer Online Documentation}*/
ignoreContentRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets a value indicating whether a drag operation is currently in progress.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23isDragging Online Documentation}*/
isDragging: boolean;
/**
Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to add.
@see ResizeStripeInputMode#removeDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragFinishing Online Documentation}*/
addDragFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
@param listener The listener to remove.
@see ResizeStripeInputMode#addDragFinishingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragFinishing Online Documentation}*/
removeDragFinishingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
@param listener The listener to add.
@see ResizeStripeInputMode#removeDragFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragFinished Online Documentation}*/
addDragFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
@param listener The listener to remove.
@see ResizeStripeInputMode#addDragFinishedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragFinished Online Documentation}*/
removeDragFinishedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to add.
@see ResizeStripeInputMode#removeDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragStarting Online Documentation}*/
addDragStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
@param listener The listener to remove.
@see ResizeStripeInputMode#addDragStartingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragStarting Online Documentation}*/
removeDragStartingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to add.
@see ResizeStripeInputMode#removeDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragStarted Online Documentation}*/
addDragStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
@param listener The listener to remove.
@see ResizeStripeInputMode#addDragStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragStarted Online Documentation}*/
removeDragStartedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to add.
@see ResizeStripeInputMode#removeDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23Dragging Online Documentation}*/
addDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
@param listener The listener to remove.
@see ResizeStripeInputMode#addDraggingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23Dragging Online Documentation}*/
removeDraggingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to add.
@see ResizeStripeInputMode#removeDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23Dragged Online Documentation}*/
addDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
@param listener The listener to remove.
@see ResizeStripeInputMode#addDraggedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23Dragged Online Documentation}*/
removeDraggedListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to add.
@see ResizeStripeInputMode#removeDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragCanceled Online Documentation}*/
addDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
@param listener The listener to remove.
@see ResizeStripeInputMode#addDragCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragCanceled Online Documentation}*/
removeDragCanceledListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to add.
@see ResizeStripeInputMode#removeDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragCanceling Online Documentation}*/
addDragCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
/**
Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
@param listener The listener to remove.
@see ResizeStripeInputMode#addDragCancelingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResizeStripeInputMode%23DragCanceling Online Documentation}*/
removeDragCancelingListener(listener: (sender: any, evt: InputModeEventArgs) => void): void;
static $class: Class;
private $_ResizeStripeInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ResizeStripeInputMode;
}
export interface IReparentStripeHandler {
}
/**
Interface used for implementations that recognize, approve and disapprove stripe reparenting gestures, as well as
actually performs the reparenting.
@see ReparentStripePositionHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReparentStripeHandler Online Documentation}*/
export abstract class IReparentStripeHandler {
/**
Creates an implementation of the interface {@link IReparentStripeHandler} from the given definition.
The given object must contain exactly the abstract members of the {@link IReparentStripeHandler} interface. No
member may be omitted and no additional members can be added. While additional custom members can later be added
directly on the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IReparentStripeHandler} interface.
@returns An instance of the {@link IReparentStripeHandler} interface based on the given definition.*/
static create(interfaceDefinition: {
shouldReparent(context: IInputModeContext, stripe: IStripe): boolean;
isValidParent(context: IInputModeContext, stripe: IStripe, newParent: IStripe, index: number, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues): boolean;
reparent(context: IInputModeContext, movedStripe: IStripe, newParent: IStripe, index: number, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues): void;
}): IReparentStripeHandler;
/**
Determines whether the provided node may be reparented to a <code>newParent</code>.
This method is called to determine whether a {@link IReparentStripeHandler#reparent} operation should actually be
executed.
@param context The context that provides information about the user input.
@param stripe The stripe that will be reparented.
@param newParent The potential new parent.
@param index The index where the stripe would be {@link IReparentStripeHandler#reparent reparented}
@param reparentPosition Where to place the stripe as a result of the gesture that would be used for the {@link IReparentStripeHandler#reparent }
operation.
@returns Whether <code>newParent</code> is a valid new parent for <code>stripe</code> for the given index and gesture.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReparentStripeHandler%23isValidParent(IInputModeContext%2CIStripe%2CIStripe%2Cnumber%2CStripeReparentPolicy) Online Documentation}*/
abstract isValidParent(context: IInputModeContext, stripe: IStripe, newParent: IStripe, index: number, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues): boolean;
/**
Performs the actual reparenting after the reparent gesture has been finalized.
Implementations should use {@link ITable#setParent} or {@link ITable#setParent} to set the parent of <code>movedStripe</code>
to <code>newParent</code>.
@param context The context that provides information about the user input.
@param movedStripe The stripe that will be reparented.
@param newParent The potential new parent.
@param index The index where the stripe should be inserted.
@param reparentPosition Where to place the stripe as a result of the gesture that triggered the reparent operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReparentStripeHandler%23reparent(IInputModeContext%2CIStripe%2CIStripe%2Cnumber%2CStripeReparentPolicy) Online Documentation}*/
abstract reparent(context: IInputModeContext, movedStripe: IStripe, newParent: IStripe, index: number, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues): void;
/**
Determines whether the user may detach the given stripe from its current parent in order to reparent it.
@param context The context that provides information about the user input.
@param stripe The stripe that is about to be detached from its current parent.
@returns Whether the stripe may be detached and reparented.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IReparentStripeHandler%23shouldReparent(IInputModeContext%2CIStripe) Online Documentation}*/
abstract shouldReparent(context: IInputModeContext, stripe: IStripe): boolean;
static $class: Class;
private $_IReparentStripeHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IReparentStripeHandler;
}
export interface ReparentStripeHandler extends YObject, IReparentStripeHandler {
}
/**
Default implementation of the {@link IReparentStripeHandler} interface.
This implementation allows to specify a maximal nesting depth for reparent operations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripeHandler Online Documentation}*/
export class ReparentStripeHandler {
constructor();
/**
Adjust the size of the source or the target stripe.
This implementation adjusts the size of <code>stripe</code> to <code>Math.Max(originalStripeSize, originalParentSize)</code> if <code>reparentPosition</code>
is an {@link StripeReparentPolicy#ADD_CHILD} operation
@param context The context that provides information about the user input.
@param stripe The stripe that will be reparented.
@param newParent The potential new parent.
@param index The index where the stripe should be inserted.
@param reparentPosition Where to place the stripe as a result of the gesture that triggered the reparent operation.
@param originalStripeSize The original size of <code>stripe</code>
@param originalParentSize The original size of <code>newParent</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripeHandler%23adjustSize(IInputModeContext%2CIStripe%2CIStripe%2CStripeReparentPolicy%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
adjustSize(context: IInputModeContext, stripe: IStripe, newParent: IStripe, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues, index: number, originalStripeSize: number, originalParentSize: number): void;
/**
Checks the constraints imposed by {@link ReparentStripeHandler#maxRowLevel} or {@link ReparentStripeHandler#maxColumnLevel} for a valid gesture.
@param context The context that provides information about the user input.
@param stripe The stripe that will be reparented.
@param newParent The potential new parent.
@param index The index where the stripe would be {@link ReparentStripeHandler#reparent reparented}
@param reparentPosition Where to place the stripe as a result of the gesture that would be used for the {@link ReparentStripeHandler#reparent }
operation.
@returns <code>true</code> iff the resulting nesting depth is smaller than {@link ReparentStripeHandler#maxRowLevel} or {@link ReparentStripeHandler#maxColumnLevel}, or if the nesting depth would not increase by the operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripeHandler%23isValidParent(IInputModeContext%2CIStripe%2CIStripe%2Cnumber%2CStripeReparentPolicy) Online Documentation}*/
isValidParent(context: IInputModeContext, stripe: IStripe, newParent: IStripe, index: number, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues): boolean;
/**
Performs the actual reparenting after the reparent gesture has been finalized.
This implementation adjusts the size of the moved and/or the target stripe by calling {@link ReparentStripeHandler#adjustSize}
.
@param context The context that provides information about the user input.
@param stripe The stripe that will be reparented.
@param newParent The potential new parent.
@param index The index where the stripe should be inserted.
@param reparentPosition Where to place the stripe as a result of the gesture that triggered the reparent operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripeHandler%23reparent(IInputModeContext%2CIStripe%2CIStripe%2Cnumber%2CStripeReparentPolicy) Online Documentation}*/
reparent(context: IInputModeContext, stripe: IStripe, newParent: IStripe, index: number, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues): void;
/**
Determines whether the user may detach the given stripe from its current parent in order to reparent it.
@param context The context that provides information about the user input.
@param stripe The stripe that is about to be detached from its current parent.
@returns This implementation returns always <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripeHandler%23shouldReparent(IInputModeContext%2CIStripe) Online Documentation}*/
shouldReparent(context: IInputModeContext, stripe: IStripe): boolean;
/**
Gets or sets the maximum nesting level the row hierarchy may acquire during a reparent gesture.
Note that it is always possible to decrease the nesting depth, even if the resulting depth would still be too high. This
allows to interactively reduce an invalid nesting depth in multiple steps. The default value is
<code>0x7FFFFFFF</code>, which effectively means an unlimited nesting depth.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripeHandler%23maxRowLevel Online Documentation}*/
maxRowLevel: number;
/**
Gets or sets the maximum nesting level the column hierarchy may acquire during a reparent gesture.
Note that it is always possible to decrease the nesting depth, even if the resulting depth would still be too high. This
allows to interactively reduce an invalid nesting depth in multiple steps. The default value is
<code>0x7FFFFFFF</code>, which effectively means an unlimited nesting depth.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripeHandler%23maxColumnLevel Online Documentation}*/
maxColumnLevel: number;
static $class: Class;
private $_ReparentStripeHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReparentStripeHandler;
}
export interface ReparentStripeInputMode extends MoveInputMode {
}
/**
Specialized input mode that is used to reparent a stripe inside an {@link ITable} structure.
This implementation delegates most work to an instance of {@link ReparentStripePositionHandler}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripeInputMode Online Documentation}*/
export class ReparentStripeInputMode {
constructor();
static $class: Class;
private $_ReparentStripeInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReparentStripeInputMode;
}
export interface ReparentStripePositionHandler extends YObject, IPositionHandler {
/**
@inheritdoc*/
cancelDrag(context: IInputModeContext, originalLocation: Point): void;
/**
@inheritdoc*/
dragFinished(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
handleMove(context: IInputModeContext, originalLocation: Point, newLocation: Point): void;
/**
@inheritdoc*/
initializeDrag(context: IInputModeContext): void;
/**
@inheritdoc*/
readonly location: IPoint;
} /**
Customized {@link IPositionHandler} implementation that moves a stripe in the table hierarchy when the stripe is
dragged.
This class is used by default by both {@link ReparentStripeInputMode} and {@link StripeDropInputMode} to
perform the actual reparenting and insertion.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler Online Documentation}*/
export class ReparentStripePositionHandler {
/**
Creates a new instance of this position handler which moves <code>movedStripe</code>
@param movedStripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler Online Documentation}*/
constructor(movedStripe: IStripe);
/**
Create a ghost object for the source region of the reparent gesture.
The default implementation uses the return value of {@link IStripeInputVisualizationHelper#getVisualCreator} with
type {@link StripeVisualizationType#DRAG_SOURCE}.
@param context The current input mode context
@returns A canvas object that is used to visualize the target region.
@see ReparentStripePositionHandler#updateSourceGhost
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23createSourceGhost(IInputModeContext) Online Documentation}*/
createSourceGhost(context: IInputModeContext): ICanvasObject | null;
/**
Create a ghost visualization for the target region of the reparent gesture.
The default implementation uses the return value of {@link IStripeInputVisualizationHelper#getVisualCreator} with
type {@link StripeVisualizationType#DROP_TARGET}.
@param context The current input mode context
@param stripe The stripe for which the visualization should be created.
@returns A canvas object that is used to visualize the target region.
@see ReparentStripePositionHandler#updateTargetGhost
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23createTargetGhost(IInputModeContext%2CIStripe) Online Documentation}*/
createTargetGhost(context: IInputModeContext, stripe: IStripe | null): ICanvasObject | null;
/**
Determine the reparent gesture that would result from the given parameters.
The target region has already been determined by {@link ReparentStripePositionHandler#getTargetSubregion}
@param context The current input mode context
@param location The current drag location
@param sourceStripe The stripe that is moved
@param targetSubregion The stripe subregion for the target.
@param targetBounds The target bounds
@returns A {@link StripeReparentPolicy} that specifies the operation to perform.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23determineGesture(IInputModeContext%2CIPoint%2CIStripe%2CStripeSubregion%2CRect) Online Documentation}*/
determineGesture(context: IInputModeContext, location: IPoint, sourceStripe: IStripe, targetSubregion: StripeSubregion | null, targetBounds: Rect): StripeReparentPolicy;
/**
Finds the target stripe subregion at <code>location</code> and its associated owner node.
@param location The hit location.
@returns A {@link StripeSubregion} that lies at <code>location</code>, or null if no such region could be found.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23getTargetSubregion(Point) Online Documentation}*/
getTargetSubregion(location: Point): StripeSubregion | null;
/**
Called whenever a change in the oldTable hierarchy occurs through this position handler
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23onStripeChanged(StripeEventArgs) Online Documentation}*/
onStripeChanged(evt: StripeEventArgs): void;
/**
Called by clients to set the position to the given coordinates.
@param location The new location.
@see IDragHandler#location
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23setPosition(Point) Online Documentation}*/
setPosition(location: Point): void;
/**
Updates the source ghost visualization after a drag.
The default does not change <code>sourceGhost</code>.
@param context The current input mode context.
@param sourceGhost The canvas object that represents the source ghost.
@param movedStripe The {@link IStripe} which is moved by this reparent gesture.
@param reparentPosition Where to place the stripe after reparenting.
@returns An updated canvas object for the source ghost.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23updateSourceGhost(IInputModeContext%2CICanvasObject%2CIStripe%2CStripeReparentPolicy) Online Documentation}*/
updateSourceGhost(context: IInputModeContext, sourceGhost: ICanvasObject | null, movedStripe: IStripe | null, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues): ICanvasObject | null;
/**
Modifies the target ghost visualization after a drag.
The default implementation only hides <code>targetGhost</code> iff <code>reparentPosition</code> is {@link StripeReparentPolicy#INVALID}.
@param context The current input mode context.
@param targetGhost The canvas object that represents the target ghost.
@param targetStripe The target for the reparent gesture.
@param reparentPosition The mode the describes the results of the reparent gesture.
@param targetBounds The current visualization bounds that have been computed with
{@link ReparentStripePositionHandler#updateTargetGhostBounds}.
@returns An updated canvas object for the target ghost.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23updateTargetGhost(IInputModeContext%2CICanvasObject%2CIStripe%2CStripeReparentPolicy%2CRect) Online Documentation}*/
updateTargetGhost(context: IInputModeContext, targetGhost: ICanvasObject | null, targetStripe: IStripe | null, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues, targetBounds: Rect): ICanvasObject | null;
/**
Updates the visualization bounds for the target visualization depending on the provided values.
@param originalTargetBounds The target bounds prior to this method call.
@param reparentPosition Where to place the stripe after the reparenting operations according to the reparent gesture.
@param targetStripe The target stripe which has been determined by the input mode.
@returns Updated bounds for the target visualization.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23updateTargetGhostBounds(Rect%2CStripeReparentPolicy%2CIStripe) Online Documentation}*/
updateTargetGhostBounds(originalTargetBounds: Rect, reparentPosition: StripeReparentPolicy | StripeReparentPolicyStringValues, targetStripe: IStripe): Rect;
/**
Gets the moved stripe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23movedStripe Online Documentation}*/
movedStripe: IStripe | null;
/**
Gets the current {@link IInputModeContext}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23inputModeContext Online Documentation}*/
readonly inputModeContext: IInputModeContext | null;
/**
Adds the given listener for the <code>StripeChanged</code> event that occurs when a stripe state is changed by this input mode.
@param listener The listener to add.
@see ReparentStripePositionHandler#removeStripeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23StripeChanged Online Documentation}*/
addStripeChangedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
/**
Removes the given listener for the <code>StripeChanged</code> event that occurs when a stripe state is changed by this input mode.
@param listener The listener to remove.
@see ReparentStripePositionHandler#addStripeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReparentStripePositionHandler%23StripeChanged Online Documentation}*/
removeStripeChangedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
static $class: Class;
private $_ReparentStripePositionHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReparentStripePositionHandler;
}
export interface StripeDropInputMode extends ItemDropInputMode<IStripe> {
/**
@inheritdoc*/
cancel(): void;
} /**
An {@link ItemDropInputMode} specialized to the drag and drop of {@link IStripe}s.
This input mode {@link ItemDropInputMode#showPreview can visualize a preview} of the {@link ItemDropInputMode#draggedItem dragged item} during the drag operation.
@see StripeDropInputMode#startDrag
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode Online Documentation}*/
export class StripeDropInputMode {
/**
Initializes a new instance of the {@link StripeDropInputMode} class for the expected data type.
You might want to adjust the {@link StripeDropInputMode#draggedItem} getter, too.
@param expectedType A list of types that are used in {@link StripeDropInputMode#startDrag}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23StripeDropInputMode(Array%3Cstring%3E) Online Documentation}*/
constructor(expectedType: string[]);
/**
Initializes a new instance of the {@link StripeDropInputMode} class for the expected data type.
You might want to adjust the {@link StripeDropInputMode#draggedItem} getter, too.
@param expectedType The type that is used in {@link StripeDropInputMode#startDrag}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23StripeDropInputMode(string) Online Documentation}*/
constructor(expectedType: string);
/**
Initializes a new instance of the {@link NodeDropInputMode} class for {@link INode}s.
This instance fires events only if the type of the data of the drag operation is one of the full names of types
{@link IStripe}, {@link IRow}, and {@link IColumn}. Such a drag operation can be started for example with
{@link StripeDropInputMode#startDrag}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23StripeDropInputMode() Online Documentation}*/
constructor();
/**
Create a preview version of the table that is temporarily used to show the dragged stripe.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23createPreviewTable() Online Documentation}*/
createPreviewTable(): ITable;
/**
Create a new stripe as the result of a drag and drop gesture.
This method is called by the {@link ItemDropInputMode#itemCreator} that is set as default on this class.
@param newParent The parent for which to create the stripe.
@param context The context for which the stripe should be created.
@param draggedStripe The dragged stripe.
@returns a newly created stripe.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23createStripe(IInputModeContext%2CIStripe%2CIStripe) Online Documentation}*/
createStripe(context: IInputModeContext, newParent: IStripe, draggedStripe: IStripe | null): IStripe | null;
/**
Calculates the layout of the new table.
In this implementation the mouse location is used as center of the table. Can be overridden in child class to implement
a different layout.
@param location Current mouse position
@param size Size of the table
@returns a {@link Rect} with the given size and the mouse location as center.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23getPreviewTableLayout(Point%2CSize) Online Documentation}*/
getPreviewTableLayout(location: Point, size: Size): Rect;
/**
Called whenever a new stripe is created
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23onStripeCreated(ItemEventArgs%3CIStripe%3E) Online Documentation}*/
onStripeCreated(evt: ItemEventArgs<IStripe>): void;
/**
Starts a drag and drop operation for an {@link IStripe}.
This method creates a {@link DragSource}, and then invokes {@link DragSource#startDrag} with a {@link DragDropItem}
that contains the <code>stripe</code> and the corresponding type.
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param dragSource The source of the drag operation.
@param stripe The stripe to drag.
@param [dragDropEffects=DragDropEffects.ALL] The allowed drag drop effects.
@param [useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see DragSource#startDrag
@see DragDropItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23startDrag(Element%2CIStripe%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
static startDrag(dragSource: HTMLElement | SVGElement, stripe: IStripe, dragDropEffects?: DragDropEffects | DragDropEffectsStringValues, useCssCursors?: boolean, dragPreview?: HTMLElement | SVGElement | null): DragSource;
/**
Starts a drag and drop operation for an {@link IStripe}.
This method creates a {@link DragSource}, and then invokes {@link DragSource#startDrag} with a {@link DragDropItem}
that contains the <code>stripe</code> and the corresponding type.
If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
depending on the allowed drag-drop operations for these elements:
<ul>
<li><code>yfiles-cursor-dragdrop-move</code>
<li><code>yfiles-cursor-dragdrop-link</code>
<li><code>yfiles-cursor-dragdrop-all</code>
<li><code>yfiles-cursor-dragdrop-no-drop</code>
</ul>
@param options The parameters to pass.
@param options.dragSource The source of the drag operation.
@param options.stripe The stripe to drag.
@param [options.dragDropEffects=DragDropEffects.ALL] The allowed drag drop effects.
@param [options.useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
@param [options.dragPreview=null] Optional element that should be shown during dragging, e.g. outside of the
{@link GraphComponent}. Note that this feature requires support for CSS pointer-events and should not be used in
browsers that do not support it.
@see DragSource#startDrag
@see DragDropItem
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23startDrag(Element%2CIStripe%2CDragDropEffects%2Cboolean%2CElement) Online Documentation}*/
static startDrag(options: {
dragSource: HTMLElement | SVGElement;
stripe: IStripe;
dragDropEffects?: DragDropEffects | DragDropEffectsStringValues;
useCssCursors?: boolean;
dragPreview?: HTMLElement | SVGElement | null;
}): DragSource;
/**
Gets the currently dragged {@link IModelItem} instance.
This implementation simply tries to cast the {@link DropInputMode#dropData} to {@link INode}.
If native drag and drop is used, the value of this property is always <code>null</code>. This is because the protected mode prevents
access to the actual data during <code>dragover</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23draggedItem Online Documentation}*/
draggedItem: IStripe | null;
/**
Adds the given listener for the <code>StripeCreated</code> event that occurs when a new stripe gets created by this input mode.
@param listener The listener to add.
@see StripeDropInputMode#removeStripeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23StripeCreated Online Documentation}*/
addStripeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IStripe>) => void): void;
/**
Removes the given listener for the <code>StripeCreated</code> event that occurs when a new stripe gets created by this input mode.
@param listener The listener to remove.
@see StripeDropInputMode#addStripeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDropInputMode%23StripeCreated Online Documentation}*/
removeStripeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IStripe>) => void): void;
static $class: Class;
private $_StripeDropInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StripeDropInputMode;
}
export interface TableEditorInputMode extends MultiplexingInputMode {
/**
@inheritdoc*/
install(context: IInputModeContext, controller: ConcurrencyController): void;
/**
@inheritdoc*/
uninstall(context: IInputModeContext): void;
} /**
Input mode that can be used to interactively edit a table.
This mode can either be used as a stand alone input mode, or as a child mode of
{@link GraphEditorInputMode}. In that case, some child modes of this implementation are disabled:
<ul>
<li>{@link TableEditorInputMode#clickInputMode}
<li>{@link TableEditorInputMode#textEditorInputMode}
<li>{@link TableEditorInputMode#keyboardInputMode}
</ul>
In addition, the {@link TableEditorInputMode#stripeSelection stripe selection} is optionally synchronized with the {@link GraphInputMode#graphSelection}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode Online Documentation}*/
export class TableEditorInputMode {
/**
Default constructor.
@param [options] The parameters to pass.
@param [options.resizeStripeInputMode] The {@link ResizeStripeInputMode} that is used to change the size of a stripe interactively. This option sets the {@link TableEditorInputMode#resizeStripeInputMode} property on the created object.
@param [options.reparentStripeHandler] The {@link TableEditorInputMode#reparentStripeHandler} property. This option sets the {@link TableEditorInputMode#reparentStripeHandler} property on the created object.
@param [options.reparentStripeInputMode] The {@link ReparentStripeInputMode} that is used to change a stripes position in the table hierarchy. This option sets the {@link TableEditorInputMode#reparentStripeInputMode} property on the created object.
@param [options.stripeDropInputMode] The {@link TableEditorInputMode#stripeDropInputMode} property that is responsible for handling drag and drop operations of stripes onto the Canvas. This option sets the {@link TableEditorInputMode#stripeDropInputMode} property on the created object.
@param [options.keyboardInputMode] The {@link TableEditorInputMode#keyboardInputMode}. This option sets the {@link TableEditorInputMode#keyboardInputMode} property on the created object.
@param [options.textEditorInputMode] The {@link TableEditorInputMode#textEditorInputMode}. This option sets the {@link TableEditorInputMode#textEditorInputMode} property on the created object.
@param [options.labelEditableItems] A property that determines which types of items may have their {@link TableEditorInputMode#editLabel labels edited}. This option sets the {@link TableEditorInputMode#labelEditableItems} property on the created object.
@param [options.autoRemoveEmptyLabels] A value indicating whether this mode should automatically {@link ITable#remove remove labels} from the table when a label text has been edited and the label text is empty. This option sets the {@link TableEditorInputMode#autoRemoveEmptyLabels} property on the created object.
@param [options.clickInputMode] The {@link TableEditorInputMode#clickInputMode}. This option sets the {@link TableEditorInputMode#clickInputMode} property on the created object.
@param [options.deletableItems] A property that determines which types of items may be deleted using the {@link TableEditorInputMode#deleteSelection} action. This option sets the {@link TableEditorInputMode#deletableItems} property on the created object.
@param [options.selectableItems] A property that determines which types of items should be selectable at all. This option sets the {@link TableEditorInputMode#selectableItems} property on the created object.
@param [options.clickSelectableItems] A property that determines which types of items should be selectable through {@link TableEditorInputMode#onClickInputModeClicked mouse clicks.}. This option sets the {@link TableEditorInputMode#clickSelectableItems} property on the created object.
@param [options.allowEditLabel] Whether the label editor will be automatically invoked if the F2 key is pressed. This option sets the {@link TableEditorInputMode#allowEditLabel} property on the created object.
@param [options.allowAddLabel] Whether the label editor will be automatically invoked if the Shift+F2 key is pressed. This option sets the {@link TableEditorInputMode#allowAddLabel} property on the created object.
@param [options.multiSelectionRecognizer] The event recognizer that determines whether a click is deemed a multi selection gesture. This option sets the {@link TableEditorInputMode#multiSelectionRecognizer} property on the created object.
@param [options.clickSelectableRegions] Clicks on which {@link StripeSubregionTypes}(s) should toggle a stripe selection state. This option sets the {@link TableEditorInputMode#clickSelectableRegions} property on the created object.
@param [options.clickableItems] A property that determines for which types of items should mouse clicks be handled by this mode. This option sets the {@link TableEditorInputMode#clickableItems} property on the created object.
@param [options.clickableRegions] Clicks on which {@link StripeSubregionTypes}(s) react to mouse clicks. This option sets the {@link TableEditorInputMode#clickableRegions} property on the created object.
@param [options.synchronizeWithGraphSelection] A value indicating whether {@link TableEditorInputMode#stripeSelection} should be synchronized with the {@link GraphInputMode#graphSelection} if this instance is installed as a child of a {@link GraphEditorInputMode} instance. This option sets the {@link TableEditorInputMode#synchronizeWithGraphSelection} property on the created object.
@param [options.allowMixedSelection] Whether to allow mixed selections of graph elements and table elements This option sets the {@link TableEditorInputMode#allowMixedSelection} property on the created object.
@param [options.allowEditLabelOnDoubleClick] A value determining whether double-clicking should start label editing. This option sets the {@link TableEditorInputMode#allowEditLabelOnDoubleClick} property on the created object.
@param [options.exclusive] A value indicating whether this mode will be the only one running when it has the mutex. This option sets the {@link MultiplexingInputMode#exclusive} property on the created object.
@param [options.enabled] The enabled state of this input mode. This option sets the {@link MultiplexingInputMode#enabled} property on the created object.
@param [options.priority] Sets the priority property.
@param [options.defaultCursor] The cursor to use whenever no child mode prefers a different cursor. This option sets the {@link MultiplexingInputMode#defaultCursor} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode Online Documentation}*/
constructor(options?: {
resizeStripeInputMode?: ResizeStripeInputMode;
reparentStripeHandler?: IReparentStripeHandler;
reparentStripeInputMode?: ReparentStripeInputMode;
stripeDropInputMode?: StripeDropInputMode;
keyboardInputMode?: KeyboardInputMode;
textEditorInputMode?: TextEditorInputMode;
labelEditableItems?: StripeTypes | StripeTypesConvertible;
autoRemoveEmptyLabels?: boolean;
clickInputMode?: ClickInputMode;
deletableItems?: StripeTypes | StripeTypesConvertible;
selectableItems?: StripeTypes | StripeTypesConvertible;
clickSelectableItems?: StripeTypes | StripeTypesConvertible;
allowEditLabel?: boolean;
allowAddLabel?: boolean;
multiSelectionRecognizer?: EventRecognizerConvertible | ((eventSource: any | null, evt: EventArgs | null) => boolean);
clickSelectableRegions?: StripeSubregionTypes | StripeSubregionTypesConvertible;
clickableItems?: StripeTypes | StripeTypesConvertible;
clickableRegions?: StripeSubregionTypes | StripeSubregionTypesConvertible;
synchronizeWithGraphSelection?: boolean;
allowMixedSelection?: boolean;
allowEditLabelOnDoubleClick?: boolean;
exclusive?: boolean;
enabled?: boolean;
priority?: number;
defaultCursor?: Cursor | null | CursorConvertible;
});
/**
Adds a new label to the given stripe interactively.
This will activate the {@link TableEditorInputMode#textEditorInputMode} to let the user interactively enter the label's text and unless the user cancels label creation
the returned {@link Promise} will contain the label.
The text that the user enters may be {@link TableEditorInputMode#addValidateLabelTextListener validated} before the label is actually added.
This method is similar to {@link TableEditorInputMode#createLabel}. It differs only when either the {@link TableEditorInputMode#addLabelAddingListener LabelAdding}
event is handled or the <code>owner</code> has an {@link IEditLabelHelper} requests editing an existing label instead of
creating a new one. In this case, this method will edit that label instead of adding a new one.
@param owner The item.
@returns A Promise that fulfills with either the newly created label or <code>null</code> if the {@link TextEditorInputMode#addEditingCanceledListener editing was canceled} or the text did not
{@link TableEditorInputMode#addValidateLabelTextListener validate}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23addLabel(ILabelOwner) Online Documentation}*/
addLabel(owner: ILabelOwner): Promise<ILabel>;
/**
Clears the current selection.
@see TableEditorInputMode#selectableItems
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23clearSelection() Online Documentation}*/
clearSelection(): void;
/**
Actually performs the click on the given item.
Before selection {@link TableEditorInputMode#clearSelection the current selection will be cleared}, unless the {@link TableEditorInputMode#multiSelectionRecognizer} is triggered.
@param tableHitTestResult Information about the item that has been clicked.
@param evt The original event arguments for the click. Setting its {@link ClickEventArgs#handled} property to <code>true</code> will indicate that the click was handled. By
default this happens when the clicked item is either selected or focused.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23click(TableHitTestResult%2CClickEventArgs) Online Documentation}*/
click(tableHitTestResult: TableHitTestResult, evt: ClickEventArgs): void;
/**
Factory method that creates the {@link TableEditorInputMode#keyboardInputMode} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23createClickInputMode() Online Documentation}*/
createClickInputMode(): ClickInputMode;
/**
Factory method that creates the {@link TableEditorInputMode#keyboardInputMode} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23createKeyboardInputMode() Online Documentation}*/
createKeyboardInputMode(): KeyboardInputMode;
/**
Interactively creates a new label for the provided stripe.
This method will invoke the text editor that will let the user edit the text of the label. If the user commits the label
text the label will be added to the label owner.
The text that the user enters may be {@link TableEditorInputMode#addValidateLabelTextListener validated} before the label is actually added.
This method is similar to {@link TableEditorInputMode#addLabel}. It differs only when either the {@link TableEditorInputMode#addLabelAddingListener LabelAdding}
event is handled or the <code>stripe</code> has an {@link IEditLabelHelper} requests editing an existing label instead of
creating a new one. In both cases, this method will do nothing and neither adds a new label nor edits an existing one.
@param stripe The item to create a new label for.
@returns A Promise for the label that will be resolved with the newly created label or a <code>null</code> if the creation was canceled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23createLabel(IStripe) Online Documentation}*/
createLabel(stripe: IStripe): Promise<ILabel>;
/**
Factory method for the {@link TableEditorInputMode#reparentStripeHandler} property.
This method will be called upon first access to the {@link TableEditorInputMode#reparentStripeHandler} property.
@returns a new instance of {@link IReparentStripeHandler}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23createReparentStripeHandler() Online Documentation}*/
createReparentStripeHandler(): IReparentStripeHandler;
/**
Factory method that creates the {@link TableEditorInputMode#reparentStripeInputMode} lazily the first time the property is accessed.
@returns A plain new instance of the {@link ReparentStripeInputMode} type, which is initially {@link MoveInputMode#enabled enabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23createReparentStripeInputMode() Online Documentation}*/
createReparentStripeInputMode(): ReparentStripeInputMode;
/**
Factory method that creates the {@link TableEditorInputMode#resizeStripeInputMode} lazily the first time the property is accessed.
@returns A plain new instance of the {@link ResizeStripeInputMode} type, which is initially {@link ResizeStripeInputMode#enabled enabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23createResizeStripeInputMode() Online Documentation}*/
createResizeStripeInputMode(): ResizeStripeInputMode;
/**
Factory method that creates the {@link TableEditorInputMode#stripeDropInputMode} lazily the first time the property is accessed.
@returns A plain new instance of the {@link StripeDropInputMode} type, which is initially {@link DropInputMode#enabled disabled}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23createStripeDropInputMode() Online Documentation}*/
createStripeDropInputMode(): StripeDropInputMode;
/**
Create a new instance of {@link IStripeSelection} the first time property {@link TableEditorInputMode#stripeSelection} is accessed.
By default, the {@link IStripeSelection#allowMixedSelection} property is <code>false</code>.
@returns A new {@link IStripeSelection} instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23createStripeSelection() Online Documentation}*/
createStripeSelection(): IStripeSelection;
/**
Factory method for the {@link TableEditorInputMode#textEditorInputMode} property.
This method will be called upon first access to the {@link TableEditorInputMode#textEditorInputMode} property.
@returns a new instance of {@link TextEditorInputMode}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23createTextEditorInputMode() Online Documentation}*/
createTextEditorInputMode(): TextEditorInputMode;
/**
This method deletes the currently selected stripes.
If the {@link TableEditorInputMode#stripeSelection} is non-empty this implementation triggers the {@link TableEditorInputMode#addDeletingSelectionListener DeletingSelection}
event, possibly a number of {@link TableEditorInputMode#addDeletedItemListener DeletedItem}
events, and a final {@link TableEditorInputMode#deleteSelection} event.
@see TableEditorInputMode#addDeletingSelectionListener
@see TableEditorInputMode#addDeletedItemListener
@see TableEditorInputMode#addDeletedSelectionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23deleteSelection() Online Documentation}*/
deleteSelection(): void;
/**
Deletes a single stripe
@param stripe The stripe to delete.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23deleteStripe(IStripe) Online Documentation}*/
deleteStripe(stripe: IStripe): void;
/**
Actually performs a double-click on the given item.
By default, this will start label editing if {@link TableEditorInputMode#allowEditLabelOnDoubleClick} is <code>true</code>
@param tableHitTestResult The item that has been double-clicked.
@param evt The original event arguments for the click. Setting its {@link ClickEventArgs#handled} property to <code>true</code> will indicate that the double-click was
handled and events on other items in the same location are no longer raised.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23doubleClick(TableHitTestResult%2CClickEventArgs) Online Documentation}*/
doubleClick(tableHitTestResult: TableHitTestResult, evt: ClickEventArgs): void;
/**
Actually performs a double-tap on the given item.
By default, this will start label editing if {@link TableEditorInputMode#allowEditLabelOnDoubleClick} is <code>true</code>
@param tableHitTestResult The item that has been double-tapped.
@param evt The original event arguments for the tap. Setting its {@link TapEventArgs#handled} property to <code>true</code> will indicate that the double-tap was handled
and events on other items in the same location are no longer raised.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23doubleTap(TableHitTestResult%2CTapEventArgs) Online Documentation}*/
doubleTap(tableHitTestResult: TableHitTestResult, evt: TapEventArgs): void;
/**
Starts editing of the given label.
The label must be bound to an {@link IStripe} instance for this call to succeed.
This implementation uses the {@link TableEditorInputMode#textEditorInputMode} to display an editor to edit the label. The text that the user enters may be {@link TableEditorInputMode#addValidateLabelTextListener validated}
before the label is actually edited.
This method will raise the {@link TableEditorInputMode#addLabelEditingListener LabelEditing}
event and query the {@link IEditLabelHelper} for the label and its owner, but will ignore the result except for the
{@link LabelEditingEventArgs#textEditorInputModeConfigurator} property on the {@link LabelEditingEventArgs}. Thus editing a label via this method cannot be prevented by event
handlers or {@link IEditLabelHelper}s.
@param label The label to edit.
@returns A Promise of the label that will be resolved with the label or a <code>null</code> if the text edit was canceled.
@see TableEditorInputMode#onLabelTextEdited
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23editLabel(ILabel) Online Documentation}*/
editLabel(label: ILabel): Promise<ILabel>;
/**
Starts label editing by executing {@link ICommand#EDIT_LABEL }.
The method should return whether the request was satisfied.
@param item The item whose label or the label itself that should be edited upon the double click gesture.
@returns <code>true</code> iff the request was handled.
@see TableEditorInputMode#doubleClick
@see TableEditorInputMode#doubleTap
@see TableEditorInputMode#allowEditLabelOnDoubleClick
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23editLabelOnDoubleClick(IModelItem) Online Documentation}*/
editLabelOnDoubleClick(item: IModelItem): boolean;
/**
Used as a callback to find the items underneath a certain point.
@param location The location to test.
@param stripeTypes An enumeration value of {@link StripeTypes} to specify the stripe type.
@param subregionTypes An enumeration value of {@link StripeSubregionTypes} to further restrict the stripe region.
@param [filter=null] Additional predicate to further restrict the hit test results.
@returns The stripe subregions that have been found for the location or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23findStripe(Point%2CStripeTypes%2CStripeSubregionTypes%2Cfunction(StripeSubregion)%3Aboolean) Online Documentation}*/
findStripe(location: Point, stripeTypes: StripeTypes, subregionTypes: StripeSubregionTypes, filter?: ((obj: StripeSubregion) => boolean) | null): StripeSubregion | null;
/**
Used as a callback to find the items underneath a certain point.
@param location The location to test.
@param stripeTypes An enumeration value of {@link StripeTypes} to specify the stripe type.
@param subregionTypes An enumeration value of {@link StripeSubregionTypes} to further restrict the stripe region.
@param [filter=null] Additional predicate to further restrict the hit test results.
@returns The stripe subregions that have been found for the location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23findStripes(Point%2CStripeTypes%2CStripeSubregionTypes%2Cfunction(StripeSubregion)%3Aboolean) Online Documentation}*/
findStripes(location: Point, stripeTypes: StripeTypes, subregionTypes: StripeSubregionTypes, filter?: ((obj: StripeSubregion) => boolean) | null): IEnumerable<StripeSubregion>;
/**
Inserts a new stripe as child of <code>owner</code>
This method automatically resizes the table node and moves all content in neighboring stripes accordingly. The stripe is
created with an initial <code>size</code> and otherwise the default settings of the table where <code>owner</code> belongs to. If <code>size</code> is entered
as <code>0</code>, the owners default size is used.
@param owner The parent of the new stripe.
@param index The index where to insert the new stripe.
@param [size=null] The initial size of the stripe
@returns A new stripe at the given index unless <code>owner</code> is not associated with a {@link IStripe#table}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23insertChild(IStripe%2Cnumber%2Cnumber) Online Documentation}*/
insertChild(owner: IStripe, index: number, size?: number | null): IStripe | null;
/**
Invoked if the shift+F2 key is pressed and {@link TableEditorInputMode#allowAddLabel} is set to <code>true</code>.
This method determines the label owner to add to and delegates to {@link TableEditorInputMode#createLabel}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onAddLabel() Online Documentation}*/
onAddLabel(): boolean;
/**
Called when the {@link TableEditorInputMode#clickInputMode} property value changes and after initialization of the field.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onClickInputModeChanged(ClickInputMode%2CClickInputMode) Online Documentation}*/
onClickInputModeChanged(oldMode: ClickInputMode | null, newMode: ClickInputMode): void;
/**
Called once {@link TableEditorInputMode#clickInputMode} triggers the {@link ClickInputMode#addClickedListener Clicked} event.
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onClickInputModeClicked(Object%2CClickEventArgs) Online Documentation}*/
onClickInputModeClicked(sender: any, evt: ClickEventArgs): void;
/**
Called once {@link TableEditorInputMode#clickInputMode} triggers the {@link ClickInputMode#addDoubleClickedListener DoubleClicked} event.
@param sender
@param evt
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onClickInputModeDoubleClicked(Object%2CClickEventArgs) Online Documentation}*/
onClickInputModeDoubleClicked(sender: any, evt: ClickEventArgs): void;
/**
Raises the {@link TableEditorInputMode#addDeletedItemListener DeletedItem} event.
@param evt The instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onDeletedItem(ItemEventArgs%3CIModelItem%3E) Online Documentation}*/
onDeletedItem(evt: ItemEventArgs<IModelItem>): void;
/**
Raises the {@link TableEditorInputMode#addDeletedSelectionListener DeletedSelection} event.
@param evt The {@link SelectionEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onDeletedSelection(SelectionEventArgs%3CIStripe%3E) Online Documentation}*/
onDeletedSelection(evt: SelectionEventArgs<IStripe>): void;
/**
Raises the {@link TableEditorInputMode#addDeletingSelectionListener DeletingSelection} event.
@param evt The {@link SelectionEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onDeletingSelection(SelectionEventArgs%3CIStripe%3E) Online Documentation}*/
onDeletingSelection(evt: SelectionEventArgs<IStripe>): void;
/**
Invoked if the F2 key is pressed and {@link TableEditorInputMode#allowEditLabel} is set to <code>true</code>.
This method determines the label to edit and delegates to either {@link TableEditorInputMode#editLabel} or {@link TableEditorInputMode#createLabel}
if no label could be found.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onEditLabel() Online Documentation}*/
onEditLabel(): boolean;
/**
Raises the {@link TableEditorInputMode#addItemClickedListener ItemClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link TableItemClickedEventArgs} instance that contains the item that has been clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onItemClicked(TableItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemClicked(evt: TableItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link TableItemClickedEventArgs} instance that contains the item that has been double clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onItemDoubleClicked(TableItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemDoubleClicked(evt: TableItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link TableEditorInputMode#addItemDoubleTappedListener ItemDoubleTapped} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link TapEventArgs#handled} property to <code>true</code>.
@param evt The {@link TableItemTappedEventArgs} instance that contains the item that has been double tapped.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onItemDoubleTapped(TableItemTappedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemDoubleTapped(evt: TableItemTappedEventArgs<IModelItem>): void;
/**
Raises the {@link TableEditorInputMode#addItemLeftClickedListener ItemLeftClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link TableItemClickedEventArgs} instance that contains the item that has been clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onItemLeftClicked(TableItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemLeftClicked(evt: TableItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link TableEditorInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link TableItemClickedEventArgs} instance that contains the item that has been double clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onItemLeftDoubleClicked(TableItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemLeftDoubleClicked(evt: TableItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link TableEditorInputMode#addItemRightClickedListener ItemRightClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link ItemClickedEventArgs} instance that contains the item that has been clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onItemRightClicked(TableItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemRightClicked(evt: TableItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link TableEditorInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link ClickEventArgs#handled} property to <code>true</code>.
@param evt The {@link TableItemClickedEventArgs} instance that contains the item that has been double clicked.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onItemRightDoubleClicked(TableItemClickedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemRightDoubleClicked(evt: TableItemClickedEventArgs<IModelItem>): void;
/**
Raises the {@link TableEditorInputMode#addItemTappedListener ItemTapped} event.
This implementation will stop the event propagation as soon as an event handler has set the {@link TapEventArgs#handled} property to <code>true</code>.
@param evt The {@link TableItemTappedEventArgs} instance that contains the item that has been tapped.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onItemTapped(TableItemTappedEventArgs%3CIModelItem%3E) Online Documentation}*/
onItemTapped(evt: TableItemTappedEventArgs<IModelItem>): void;
/**
Called when the {@link TableEditorInputMode#keyboardInputMode} changed.
@param oldMode The old {@link TableEditorInputMode#keyboardInputMode}.
@param newMode The new {@link TableEditorInputMode#keyboardInputMode}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onKeyboardInputModeChanged(KeyboardInputMode%2CKeyboardInputMode) Online Documentation}*/
onKeyboardInputModeChanged(oldMode: KeyboardInputMode | null, newMode: KeyboardInputMode): void;
/**
Raises the {@link TableEditorInputMode#addLabelAddedListener LabelAdded} event.
@param evt The {@link ItemEventArgs} instance that contains the {@link ILabel} that has been added.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onLabelAdded(LabelEventArgs) Online Documentation}*/
onLabelAdded(evt: LabelEventArgs): void;
/**
Raises the {@link TableEditorInputMode#addLabelAddingListener LabelAdding} event.
Invoking the event handlers stops once one event marks the <code>evt</code> as handled.
@param evt The event arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onLabelAdding(LabelEditingEventArgs) Online Documentation}*/
onLabelAdding(evt: LabelEditingEventArgs): void;
/**
Raises the {@link TableEditorInputMode#addLabelEditingListener LabelEditing} event.
Invoking the event handlers stops once one event marks the <code>evt</code> as handled.
@param evt The event arguments.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onLabelEditing(LabelEditingEventArgs) Online Documentation}*/
onLabelEditing(evt: LabelEditingEventArgs): void;
/**
Raises the {@link TableEditorInputMode#addLabelTextChangedListener LabelTextChanged} event.
@param evt The {@link ItemEventArgs} instance that contains the {@link ILabel} that has changed the text.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onLabelTextChanged(ItemEventArgs%3CILabel%3E) Online Documentation}*/
onLabelTextChanged(evt: ItemEventArgs<ILabel>): void;
/**
Called when the text of a label has been {@link TableEditorInputMode#editLabel edited}.
This method {@link TableEditorInputMode#addValidateLabelTextListener validates the label text} and if successful sets the label text.
@param label The label that was edited.
@param text The new text.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onLabelTextEdited(ILabel%2Cstring) Online Documentation}*/
onLabelTextEdited(label: ILabel, text: string): void;
/**
Raises the {@link TableEditorInputMode#addLabelTextEditingCanceledListener LabelTextEditingCanceled}
event.
@param evt The {@link LabelEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onLabelTextEditingCanceled(LabelEventArgs) Online Documentation}*/
onLabelTextEditingCanceled(evt: LabelEventArgs): void;
/**
Raises the {@link TableEditorInputMode#addLabelTextEditingStartedListener LabelTextEditingStarted}
event.
@param evt The {@link LabelEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onLabelTextEditingStarted(LabelEventArgs) Online Documentation}*/
onLabelTextEditingStarted(evt: LabelEventArgs): void;
/**
Called when the {@link TableEditorInputMode#reparentStripeHandler} property value changes and after initialization of the field.
@param oldHandler the old value, which may be <code>null</code> the first time
@param newHandler the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onReparentStripeHandlerChanged(IReparentStripeHandler%2CIReparentStripeHandler) Online Documentation}*/
onReparentStripeHandlerChanged(oldHandler: IReparentStripeHandler | null, newHandler: IReparentStripeHandler): void;
/**
Called when the {@link TableEditorInputMode#reparentStripeInputMode} property value changes and after initialization of the field.
@param oldHandler the old value, which may be <code>null</code> the first time
@param newHandler the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onReparentStripeInputModeChanged(ReparentStripeInputMode%2CReparentStripeInputMode) Online Documentation}*/
onReparentStripeInputModeChanged(oldHandler: ReparentStripeInputMode | null, newHandler: ReparentStripeInputMode): void;
/**
Called when the {@link TableEditorInputMode#resizeStripeInputMode} property value changes and after initialization of the field.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onResizeStripeInputModeChanged(ResizeStripeInputMode%2CResizeStripeInputMode) Online Documentation}*/
onResizeStripeInputModeChanged(oldMode: ResizeStripeInputMode | null, newMode: ResizeStripeInputMode): void;
/**
Called when the {@link TableEditorInputMode#stripeDropInputMode} property value changes and after initialization of the field.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onStripeDropInputModeChanged(StripeDropInputMode%2CStripeDropInputMode) Online Documentation}*/
onStripeDropInputModeChanged(oldMode: StripeDropInputMode | null, newMode: StripeDropInputMode): void;
/**
Called when the {@link TableEditorInputMode#textEditorInputMode} property value changes and after initialization of the field.
@param oldMode the old value, which may be <code>null</code> the first time
@param newMode the new value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onTextEditorInputModeChanged(TextEditorInputMode%2CTextEditorInputMode) Online Documentation}*/
onTextEditorInputModeChanged(oldMode: TextEditorInputMode | null, newMode: TextEditorInputMode): void;
/**
Raises the {@link TableEditorInputMode#addValidateLabelTextListener ValidateLabelText} event.
@param evt The {@link LabelTextValidatingEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23onValidateLabelText(LabelTextValidatingEventArgs) Online Documentation}*/
onValidateLabelText(evt: LabelTextValidatingEventArgs): void;
/**
Clears the current selection and selects all stripes in all table instances.
By default, this method will be called in response to a Ctrl-A-key press recognized by {@link TableEditorInputMode#keyboardInputMode}.
@see ICommand#SELECT_ALL
@see TableEditorInputMode#selectableItems
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23selectAll() Online Documentation}*/
selectAll(): void;
/**
Predicate that determines whether a label may be added interactively to <code>item</code>
This is overridden if the item has a {@link IEditLabelHelper} that returns <code>false</code> for its {@link IEditLabelHelper#onLabelAdding}
method.
@param item The item to query
@returns <code>true</code> iff a label may be added
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23shouldAddLabel(IModelItem) Online Documentation}*/
shouldAddLabel(item: IModelItem): boolean;
/**
Called to determine whether the item described by <code>tableHitTestResult</code> can be {@link TableEditorInputMode#click clicked}.
This implementation uses the {@link TableEditorInputMode#clickableItems} and {@link TableEditorInputMode#clickableRegions} value to determine whether the item should be clicked. If this mode is installed as
child mode of a {@link GraphEditorInputMode}, it also detects clicks on labels if {@link GraphEditorInputMode}
allows them.
@param tableHitTestResult Information about the item to check.
@returns Whether the item should be {@link TableEditorInputMode#click clicked} in response to a {@link ClickInputMode#addClickedListener detected mouse click}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23shouldClick(TableHitTestResult) Online Documentation}*/
shouldClick(tableHitTestResult: TableHitTestResult): boolean;
/**
Determines whether the given item should be deleted during {@link TableEditorInputMode#deleteSelection}.
This implementation deletes an item if it is of one of the {@link TableEditorInputMode#deletableItems} type and if it is not the last stripe of a given type in the
table (e.g. the only row or the only column).
@param stripe The item.
@returns Whether to delete that item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23shouldDelete(IStripe) Online Documentation}*/
shouldDelete(stripe: IStripe): boolean;
/**
Called to determine whether the item described by <code>tableHitTestResult</code> can be {@link TableEditorInputMode#doubleClick double clicked}.
This implementation uses the {@link TableEditorInputMode#shouldClick} method to determine whether the item should
be double clicked.
@param tableHitTestResult Information about the item to check.
@returns Whether the item should be {@link TableEditorInputMode#doubleClick double clicked} in response to a {@link ClickInputMode#addDoubleClickedListener detected double click}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23shouldDoubleClick(TableHitTestResult) Online Documentation}*/
shouldDoubleClick(tableHitTestResult: TableHitTestResult): boolean;
/**
Determines whether the label or the labels of the provided item should be edited in response to a command or explicit
method call to {@link TableEditorInputMode#editLabel} or {@link TableEditorInputMode#createLabel}.
@param item The item.
@returns Whether to edit the label or the labels for that item.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23shouldEditLabel(IModelItem) Online Documentation}*/
shouldEditLabel(item: IModelItem): boolean;
/**
Called by {@link TableEditorInputMode#onKeyboardInputModeChanged} to determine which of the built-in
{@link ICommand}s to install.
This implementation unconditionally returns <code>true</code>, subclasses may override this method to adjust the behavior.
@param command The command to install.
@returns Whether to install this command.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23shouldInstallCommand(ICommand) Online Documentation}*/
shouldInstallCommand(command: ICommand): boolean;
/**
Actually performs the tap on the given item.
Before selection {@link TableEditorInputMode#clearSelection the current selection will be cleared}.
@param tableHitTestResult Information about the item that has been tapped.
@param evt The original event arguments for the tap. Setting its {@link TapEventArgs#handled} property to <code>true</code> will indicate that the tap was handled. By
default this happens when the tapped item is either selected or focused.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23tap(TableHitTestResult%2CTapEventArgs) Online Documentation}*/
tap(tableHitTestResult: TableHitTestResult, evt: TapEventArgs): void;
/**
Gets or sets the {@link ResizeStripeInputMode} that is used to change the size of a stripe interactively.
The mode by default is {@link ResizeStripeInputMode#enabled enabled} with a priority of <code>0</code>. If the field has not yet been initialized upon first access, the
factory method {@link TableEditorInputMode#createResizeStripeInputMode} will be called. Upon change the {@link TableEditorInputMode#onResizeStripeInputModeChanged}
method will be called.
@throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
{@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23resizeStripeInputMode Online Documentation}*/
resizeStripeInputMode: ResizeStripeInputMode;
/**
Gets or sets the {@link TableEditorInputMode#reparentStripeHandler} property.
This handler will be delegated to by an implementation of the {@link IReparentStripeHandler} interface that this
class puts into the {@link TableEditorInputMode#childInputModeContextLookup}. If the field has not yet been
initialized upon first access, the factory method {@link TableEditorInputMode#createReparentStripeHandler} will be
called. Upon change the {@link TableEditorInputMode#onReparentStripeHandlerChanged} method will be called.
@see ReparentStripeHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23reparentStripeHandler Online Documentation}*/
reparentStripeHandler: IReparentStripeHandler;
/**
Gets or sets the {@link ReparentStripeInputMode} that is used to change a stripes position in the table hierarchy.
The mode by default is {@link MoveInputMode#enabled enabled} with a priority of <code>0</code>. If the field has not yet been initialized upon first access, the
factory method {@link TableEditorInputMode#createReparentStripeInputMode} will be called. Upon change the {@link TableEditorInputMode#onReparentStripeInputModeChanged}
method will be called.
@throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
{@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23reparentStripeInputMode Online Documentation}*/
reparentStripeInputMode: ReparentStripeInputMode;
/**
Gets or sets the {@link TableEditorInputMode#stripeDropInputMode} property that is responsible for handling drag and drop operations of stripes onto the Canvas.
The mode by default is {@link DropInputMode#enabled disabled} and needs to be enabled to work, first. If the field has not yet been initialized upon
first access, the factory method {@link TableEditorInputMode#createStripeDropInputMode} will be called. Upon change
the {@link TableEditorInputMode#onStripeDropInputModeChanged} method will be called.
By default this input mode has a priority of <code>70</code>.
@throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
{@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23stripeDropInputMode Online Documentation}*/
stripeDropInputMode: StripeDropInputMode;
/**
Gets or sets the {@link TableEditorInputMode#keyboardInputMode}.
If the field has not yet been initialized upon first access, the factory method {@link TableEditorInputMode#createKeyboardInputMode}
will be called. Upon change the {@link TableEditorInputMode#onKeyboardInputModeChanged} method will be called. If
the parent {@link TableEditorInputMode} instance is installed as child mode of a {@link GraphEditorInputMode}
instance, this mode is <b>not</b> used, since all work is delegated to the parent's {@link GraphInputMode#keyboardInputMode} instance. This implies that all modifications
made to the property value have no effect as long as the parent mode is child of a {@link GraphEditorInputMode}
instance.
By default this input mode has a priority of <code>0</code>.
@throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
{@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23keyboardInputMode Online Documentation}*/
keyboardInputMode: KeyboardInputMode;
/**
Gets or sets the {@link TableEditorInputMode#textEditorInputMode}.
If the field has not yet been initialized upon first access, the factory method {@link TableEditorInputMode#createTextEditorInputMode}
will be called. Upon change the {@link TableEditorInputMode#onTextEditorInputModeChanged} method will be called. If
the parent {@link TableEditorInputMode} instance is installed as child mode of a {@link GraphEditorInputMode}
instance, this mode is <b>not</b> used, since all work is delegated to the parent's {@link GraphEditorInputMode#textEditorInputMode} instance. This implies that all modifications
made to the property value have no effect as long as the parent mode is child of a {@link GraphEditorInputMode}
instance.
By default this input mode has a priority of <code>100</code>.
@throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
{@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23textEditorInputMode Online Documentation}*/
textEditorInputMode: TextEditorInputMode;
/**
Gets or set a property that determines which types of items may have their {@link TableEditorInputMode#editLabel labels edited}.
The default is {@link StripeTypes#ALL} which allows for editing the labels of all {@link ILabelOwner}s, and
existing {@link ILabel}s.
@see TableEditorInputMode#shouldEditLabel
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23labelEditableItems Online Documentation}*/
labelEditableItems: StripeTypes;
/**
Adds the given listener for the <code>LabelAdding</code> event that occurs when a label is about to be added.
Event handlers for this event can customize the behavior for adding a new label extensively. Adding a label can be
forbidden entirely, or the properties of the added label can be changed, by setting the appropriate properties on the
{@link LabelEditingEventArgs}. It is even possible to forbid adding a label, but edit an existing one instead.
@param listener The listener to add.
@see TableEditorInputMode#removeLabelAddingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelAdding Online Documentation}*/
addLabelAddingListener(listener: (sender: any, evt: LabelEditingEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelAdding</code> event that occurs when a label is about to be added.
Event handlers for this event can customize the behavior for adding a new label extensively. Adding a label can be
forbidden entirely, or the properties of the added label can be changed, by setting the appropriate properties on the
{@link LabelEditingEventArgs}. It is even possible to forbid adding a label, but edit an existing one instead.
@param listener The listener to remove.
@see TableEditorInputMode#addLabelAddingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelAdding Online Documentation}*/
removeLabelAddingListener(listener: (sender: any, evt: LabelEditingEventArgs) => void): void;
/**
Adds the given listener for the <code>LabelEditing</code> event that occurs when a label is about to be edited.
Event handlers for this event can customize the behavior for editing an existing label. Editing can be forbidden
entirely, or redirected to a different label, even on a different owner. In case editing existing labels is unwanted,
event handlers can also specify that instead of editing a label, a new one should be added.
@param listener The listener to add.
@see TableEditorInputMode#removeLabelEditingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelEditing Online Documentation}*/
addLabelEditingListener(listener: (sender: any, evt: LabelEditingEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelEditing</code> event that occurs when a label is about to be edited.
Event handlers for this event can customize the behavior for editing an existing label. Editing can be forbidden
entirely, or redirected to a different label, even on a different owner. In case editing existing labels is unwanted,
event handlers can also specify that instead of editing a label, a new one should be added.
@param listener The listener to remove.
@see TableEditorInputMode#addLabelEditingListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelEditing Online Documentation}*/
removeLabelEditingListener(listener: (sender: any, evt: LabelEditingEventArgs) => void): void;
/**
Adds the given listener for the <code>LabelAdded</code> event that occurs when this mode has triggered the addition of an
{@link ILabel}, for instance, in response to {@link TableEditorInputMode#createLabel}.
@param listener The listener to add.
@see TableEditorInputMode#removeLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelAdded Online Documentation}*/
addLabelAddedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelAdded</code> event that occurs when this mode has triggered the addition of an
{@link ILabel}, for instance, in response to {@link TableEditorInputMode#createLabel}.
@param listener The listener to remove.
@see TableEditorInputMode#addLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelAdded Online Documentation}*/
removeLabelAddedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Adds the given listener for the <code>LabelTextChanged</code> event that occurs when this mode has triggered the edit of an
{@link ILabel}, for instance, in response to {@link TableEditorInputMode#editLabel}.
@param listener The listener to add.
@see TableEditorInputMode#removeLabelTextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelTextChanged Online Documentation}*/
addLabelTextChangedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Removes the given listener for the <code>LabelTextChanged</code> event that occurs when this mode has triggered the edit of an
{@link ILabel}, for instance, in response to {@link TableEditorInputMode#editLabel}.
@param listener The listener to remove.
@see TableEditorInputMode#addLabelTextChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelTextChanged Online Documentation}*/
removeLabelTextChangedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Adds the given listener for the <code>LabelTextEditingStarted</code> event that occurs when the actual label editing process is about
to start.
This allows to customize the actual label editing process further.
@param listener The listener to add.
@see TableEditorInputMode#onValidateLabelText
@see TableEditorInputMode#removeLabelTextEditingStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelTextEditingStarted Online Documentation}*/
addLabelTextEditingStartedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Removes the given listener for the <code>LabelTextEditingStarted</code> event that occurs when the actual label editing process is
about to start.
This allows to customize the actual label editing process further.
@param listener The listener to remove.
@see TableEditorInputMode#onValidateLabelText
@see TableEditorInputMode#addLabelTextEditingStartedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelTextEditingStarted Online Documentation}*/
removeLabelTextEditingStartedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Adds the given listener for the <code>LabelTextEditingCanceled</code> event that occurs when the actual label editing process is
canceled.
This allows to revert temporary customizations for the actual label editing process.
@param listener The listener to add.
@see TableEditorInputMode#onValidateLabelText
@see TableEditorInputMode#removeLabelTextEditingCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelTextEditingCanceled Online Documentation}*/
addLabelTextEditingCanceledListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Removes the given listener for the <code>LabelTextEditingCanceled</code> event that occurs when the actual label editing process is
canceled.
This allows to revert temporary customizations for the actual label editing process.
@param listener The listener to remove.
@see TableEditorInputMode#onValidateLabelText
@see TableEditorInputMode#addLabelTextEditingCanceledListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23LabelTextEditingCanceled Online Documentation}*/
removeLabelTextEditingCanceledListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Adds the given listener for the <code>ValidateLabelText</code> event that occurs after label editing to validate the label text for a
label that is about to be added or edited.
Note that the {@link LabelTextValidatingEventArgs#label} may only a dummy instance, i.e. not bound to a live stripe instance.
@param listener The listener to add.
@see TableEditorInputMode#onValidateLabelText
@see TableEditorInputMode#removeValidateLabelTextListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ValidateLabelText Online Documentation}*/
addValidateLabelTextListener(listener: (sender: any, evt: LabelTextValidatingEventArgs) => void): void;
/**
Removes the given listener for the <code>ValidateLabelText</code> event that occurs after label editing to validate the label text
for a label that is about to be added or edited.
Note that the {@link LabelTextValidatingEventArgs#label} may only a dummy instance, i.e. not bound to a live stripe instance.
@param listener The listener to remove.
@see TableEditorInputMode#onValidateLabelText
@see TableEditorInputMode#addValidateLabelTextListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ValidateLabelText Online Documentation}*/
removeValidateLabelTextListener(listener: (sender: any, evt: LabelTextValidatingEventArgs) => void): void;
/**
Gets or sets a value indicating whether this mode should automatically {@link ITable#remove remove labels} from the table when a label text
has been edited and the label text is empty.
If the {@link TableEditorInputMode} instance is installed as child mode of a {@link GraphEditorInputMode}
instance, auto removal is only enabled allowed if both the value of this property and the value of {@link GraphEditorInputMode#autoRemoveEmptyLabels} are <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23autoRemoveEmptyLabels Online Documentation}*/
autoRemoveEmptyLabels: boolean;
/**
Gets or sets the {@link TableEditorInputMode#clickInputMode}.
If the field has not yet been initialized upon first access, the factory method {@link TableEditorInputMode#createClickInputMode}
will be called. Upon change the {@link TableEditorInputMode#onClickInputModeChanged} method will be called. If the
parent {@link TableEditorInputMode} instance is installed as child mode of a {@link GraphEditorInputMode}
instance, this mode is <b>not</b> used, since all work is delegated to the parent's {@link GraphInputMode#clickInputMode} instance. This implies that all modifications
made to the property value have no effect as long as the parent mode is child of a {@link GraphEditorInputMode}
instance.
By default this input mode has a priority of <code>0</code>.
@throws {Exception} <b>(NotSupportedError)</b> If an attempt is made to change the instance while this mode is {@link IInputMode#install installed}. To exchange a mode, first
{@link IInputMode#uninstall}, then {@link IInputMode#install reinstall} to ensure that all data is initialized correctly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23clickInputMode Online Documentation}*/
clickInputMode: ClickInputMode;
/**
Adds the given listener for the <code>DeletingSelection</code> event that occurs just before the {@link TableEditorInputMode#deleteSelection}
method starts its work and will be followed by any number of {@link TableEditorInputMode#addDeletedItemListener DeletedItem}
events and finalized by a {@link TableEditorInputMode#addDeletedSelectionListener DeletedSelection}
event.
@param listener The listener to add.
@see TableEditorInputMode#deleteSelection
@see TableEditorInputMode#removeDeletingSelectionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23DeletingSelection Online Documentation}*/
addDeletingSelectionListener(listener: (sender: any, evt: SelectionEventArgs<IStripe>) => void): void;
/**
Removes the given listener for the <code>DeletingSelection</code> event that occurs just before the {@link TableEditorInputMode#deleteSelection}
method starts its work and will be followed by any number of {@link TableEditorInputMode#addDeletedItemListener DeletedItem}
events and finalized by a {@link TableEditorInputMode#addDeletedSelectionListener DeletedSelection}
event.
@param listener The listener to remove.
@see TableEditorInputMode#deleteSelection
@see TableEditorInputMode#addDeletingSelectionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23DeletingSelection Online Documentation}*/
removeDeletingSelectionListener(listener: (sender: any, evt: SelectionEventArgs<IStripe>) => void): void;
/**
Adds the given listener for the <code>DeletedItem</code> event that occurs when an item has been deleted interactively by this mode.
@param listener The listener to add.
@see TableEditorInputMode#removeDeletedItemListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23DeletedItem Online Documentation}*/
addDeletedItemListener(listener: (sender: any, evt: ItemEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>DeletedItem</code> event that occurs when an item has been deleted interactively by this
mode.
@param listener The listener to remove.
@see TableEditorInputMode#addDeletedItemListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23DeletedItem Online Documentation}*/
removeDeletedItemListener(listener: (sender: any, evt: ItemEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>DeletedSelection</code> event that occurs after all selected items have been removed in
{@link TableEditorInputMode#deleteSelection}.
@param listener The listener to add.
@see TableEditorInputMode#deleteSelection
@see TableEditorInputMode#removeDeletedSelectionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23DeletedSelection Online Documentation}*/
addDeletedSelectionListener(listener: (sender: any, evt: SelectionEventArgs<IStripe>) => void): void;
/**
Removes the given listener for the <code>DeletedSelection</code> event that occurs after all selected items have been removed in
{@link TableEditorInputMode#deleteSelection}.
@param listener The listener to remove.
@see TableEditorInputMode#deleteSelection
@see TableEditorInputMode#addDeletedSelectionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23DeletedSelection Online Documentation}*/
removeDeletedSelectionListener(listener: (sender: any, evt: SelectionEventArgs<IStripe>) => void): void;
/**
Gets or set a property that determines which types of items may be deleted using the {@link TableEditorInputMode#deleteSelection}
action.
The default is {@link GraphItemTypes#ALL}.
@see TableEditorInputMode#shouldDelete
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23deletableItems Online Documentation}*/
deletableItems: StripeTypes;
/**
Gets or set a property that determines which types of items should be selectable at all.
The default is {@link StripeTypes#ALL}.
@see GraphInputMode#setSelected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23selectableItems Online Documentation}*/
selectableItems: StripeTypes;
/**
Gets or set a property that determines which types of items should be selectable through {@link TableEditorInputMode#onClickInputModeClicked mouse clicks.}.
The default is {@link StripeTypes#ALL}.
@see GraphInputMode#setSelected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23clickSelectableItems Online Documentation}*/
clickSelectableItems: StripeTypes;
/**
Gets or sets whether the label editor will be automatically invoked if the F2 key is pressed.
If the {@link TableEditorInputMode} instance is installed as child mode of a {@link GraphEditorInputMode}
instance, editing a label is only allowed if both the value of this property and the value of {@link GraphEditorInputMode#allowEditLabel} are
<code>true</code>. If label editing is allowed, pressing F2 will start the label editor. By default this feature is enabled.
@see TableEditorInputMode#editLabel
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23allowEditLabel Online Documentation}*/
allowEditLabel: boolean;
/**
Gets or sets whether the label editor will be automatically invoked if the Shift+F2 key is pressed.
If the {@link TableEditorInputMode} instance is installed as child mode of a {@link GraphEditorInputMode}
instance, adding a label is only allowed if both the value of this property and the value of {@link GraphEditorInputMode#allowAddLabel} are <code>true</code>. If label adding
is allowed, <code>true</code> pressing Shift+F2 will start the label editor. By default this feature is enabled.
@see TableEditorInputMode#createLabel
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23allowAddLabel Online Documentation}*/
allowAddLabel: boolean;
/**
Gets or sets the event recognizer that determines whether a click is deemed a multi selection gesture.
The default is a combination of {@link KeyEventRecognizers#CTRL_IS_DOWN} or
{@link KeyEventRecognizers#META_IS_DOWN}. If the {@link TableEditorInputMode} instance is installed as child
mode of a {@link GraphEditorInputMode} instance, the value of {@link GraphInputMode#multiSelectionRecognizer} is used instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23multiSelectionRecognizer Online Documentation}*/
multiSelectionRecognizer: (eventSource: any | null, evt: EventArgs | null) => boolean;
/**
Gets or sets clicks on which {@link StripeSubregionTypes}(s) should toggle a stripe selection state.
This is independent to {@link TableEditorInputMode#clickSelectableItems}. By default, only clicks on a stripe {@link StripeSubregionTypes#HEADER} are handled.
@default StripeSubregionTypes.HEADER
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23clickSelectableRegions Online Documentation}*/
clickSelectableRegions: StripeSubregionTypes;
/**
Gets or set a property that determines for which types of items should mouse clicks be handled by this mode.
The default is {@link StripeTypes#ALL}.
@see GraphInputMode#setSelected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23clickableItems Online Documentation}*/
clickableItems: StripeTypes;
/**
Gets or sets clicks on which {@link StripeSubregionTypes}(s) react to mouse clicks.
This is independent to {@link TableEditorInputMode#clickSelectableItems}. Default value is {@link StripeSubregionTypes#HEADER}.
@default StripeSubregionTypes.HEADER
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23clickableRegions Online Documentation}*/
clickableRegions: StripeSubregionTypes;
/**
Gets or sets a value indicating whether {@link TableEditorInputMode#stripeSelection} should be synchronized with the {@link GraphInputMode#graphSelection} if this instance is installed as a child of a
{@link GraphEditorInputMode} instance.
This feature needs to be disabled temporarily if the {@link GraphComponent#selection} is changed.
By default, this feature is enabled.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23synchronizeWithGraphSelection Online Documentation}*/
synchronizeWithGraphSelection: boolean;
/**
Gets or sets whether to allow mixed selections of graph elements and table elements
Default is <code>true</code>. Changing this property only has an effect if {@link TableEditorInputMode#synchronizeWithGraphSelection} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23allowMixedSelection Online Documentation}*/
allowMixedSelection: boolean;
/**
Gets the {@link IStripeSelection selected stripes}
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23stripeSelection Online Documentation}*/
readonly stripeSelection: IStripeSelection;
/**
Gets or sets a value determining whether double-clicking should start label editing.
If enabled, double-clicking an item will automatically execute the {@link ICommand#EDIT_LABEL} command.
This setting has no effect if label editing is generally disallowed by setting {@link TableEditorInputMode#allowEditLabel} to <code>false</code>.
If the {@link TableEditorInputMode} instance is installed as child mode of a {@link GraphEditorInputMode}
instance, editing a label by double clicks is only allowed if both the value of this property and the value of {@link GraphEditorInputMode#allowEditLabelOnDoubleClick} are
<code>true</code>.
The default value is <code>true</code>.
@see TableEditorInputMode#allowEditLabel
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23allowEditLabelOnDoubleClick Online Documentation}*/
allowEditLabelOnDoubleClick: boolean;
/**
Adds the given listener for the <code>ItemClicked</code> event that occurs when an item has been clicked or tapped.
If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
This event will also be raised for {@link TableEditorInputMode#tap taps}.
Button-specific and input type specific events
({@link TableEditorInputMode#addItemLeftClickedListener ItemLeftClicked}, {@link TableEditorInputMode#addItemRightClickedListener ItemRightClicked}
and {@link TableEditorInputMode#addItemTappedListener ItemTapped}) will be raised before
this event. If one of those is marked {@link ClickEventArgs#handled} this event will not be raised anymore.
@param listener The listener to add.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemLeftClickedListener
@see TableEditorInputMode#addItemRightClickedListener
@see TableEditorInputMode#addItemTappedListener
@see TableEditorInputMode#removeItemClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemClicked Online Documentation}*/
addItemClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>ItemClicked</code> event that occurs when an item has been clicked or tapped.
If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
This event will also be raised for {@link TableEditorInputMode#tap taps}.
Button-specific and input type specific events
({@link TableEditorInputMode#addItemLeftClickedListener ItemLeftClicked}, {@link TableEditorInputMode#addItemRightClickedListener ItemRightClicked}
and {@link TableEditorInputMode#addItemTappedListener ItemTapped}) will be raised before
this event. If one of those is marked {@link ClickEventArgs#handled} this event will not be raised anymore.
@param listener The listener to remove.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemLeftClickedListener
@see TableEditorInputMode#addItemRightClickedListener
@see TableEditorInputMode#addItemTappedListener
@see TableEditorInputMode#addItemClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemClicked Online Documentation}*/
removeItemClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>ItemLeftClicked</code> event that occurs when an item has been left clicked.
This event will be raised before the {@link TableEditorInputMode#addItemClickedListener ItemClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemClickedListener ItemClicked} event will not be
raised in that case.
@param listener The listener to add.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemClickedListener
@see TableEditorInputMode#addItemRightClickedListener
@see TableEditorInputMode#addItemTappedListener
@see TableEditorInputMode#addItemLeftDoubleClickedListener
@see TableEditorInputMode#removeItemLeftClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemLeftClicked Online Documentation}*/
addItemLeftClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>ItemLeftClicked</code> event that occurs when an item has been left clicked.
This event will be raised before the {@link TableEditorInputMode#addItemClickedListener ItemClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemClickedListener ItemClicked} event will not be
raised in that case.
@param listener The listener to remove.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemClickedListener
@see TableEditorInputMode#addItemRightClickedListener
@see TableEditorInputMode#addItemTappedListener
@see TableEditorInputMode#addItemLeftDoubleClickedListener
@see TableEditorInputMode#addItemLeftClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemLeftClicked Online Documentation}*/
removeItemLeftClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>ItemRightClicked</code> event that occurs when an item has been right clicked.
This event will be raised before the {@link TableEditorInputMode#addItemClickedListener ItemClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemClickedListener ItemClicked} event will not be
raised in that case.
@param listener The listener to add.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemClickedListener
@see TableEditorInputMode#addItemLeftClickedListener
@see TableEditorInputMode#addItemTappedListener
@see TableEditorInputMode#addItemRightDoubleClickedListener
@see TableEditorInputMode#removeItemRightClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemRightClicked Online Documentation}*/
addItemRightClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>ItemRightClicked</code> event that occurs when an item has been right clicked.
This event will be raised before the {@link TableEditorInputMode#addItemClickedListener ItemClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemClickedListener ItemClicked} event will not be
raised in that case.
@param listener The listener to remove.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemClickedListener
@see TableEditorInputMode#addItemLeftClickedListener
@see TableEditorInputMode#addItemTappedListener
@see TableEditorInputMode#addItemRightDoubleClickedListener
@see TableEditorInputMode#addItemRightClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemRightClicked Online Documentation}*/
removeItemRightClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>ItemTapped</code> event that occurs when an item has been tapped.
This event will be raised before the {@link TableEditorInputMode#addItemClickedListener ItemClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemClickedListener ItemClicked} event will not be
raised in that case.
@param listener The listener to add.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemClickedListener
@see TableEditorInputMode#addItemLeftClickedListener
@see TableEditorInputMode#addItemRightClickedListener
@see TableEditorInputMode#addItemDoubleTappedListener
@see TableEditorInputMode#removeItemTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemTapped Online Documentation}*/
addItemTappedListener(listener: (sender: any, evt: TableItemTappedEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>ItemTapped</code> event that occurs when an item has been tapped.
This event will be raised before the {@link TableEditorInputMode#addItemClickedListener ItemClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemClickedListener ItemClicked} event will not be
raised in that case.
@param listener The listener to remove.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemClickedListener
@see TableEditorInputMode#addItemLeftClickedListener
@see TableEditorInputMode#addItemRightClickedListener
@see TableEditorInputMode#addItemDoubleTappedListener
@see TableEditorInputMode#addItemTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemTapped Online Documentation}*/
removeItemTappedListener(listener: (sender: any, evt: TableItemTappedEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>ItemDoubleClicked</code> event that occurs when an item has been double clicked.
If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
Depending on the value of the {@link ClickInputMode#doubleClickPolicy} property of {@link TableEditorInputMode#clickInputMode} this event may be preceded by no, one, or two {@link TableEditorInputMode#addItemClickedListener ItemClicked}
events (as well as the corresponding button-specific event).
Button-specific and input-type specific events
({@link TableEditorInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked}, {@link TableEditorInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked}
and {@link TableEditorInputMode#addItemDoubleTappedListener ItemDoubleTapped}) will be raised
before this event. If one of those is marked {@link ClickEventArgs#handled} this event will not be raised anymore.
@param listener The listener to add.
@see TableEditorInputMode#addItemClickedListener
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemLeftDoubleClickedListener
@see TableEditorInputMode#addItemRightDoubleClickedListener
@see TableEditorInputMode#addItemDoubleTappedListener
@see TableEditorInputMode#removeItemDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemDoubleClicked Online Documentation}*/
addItemDoubleClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>ItemDoubleClicked</code> event that occurs when an item has been double clicked.
If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
Depending on the value of the {@link ClickInputMode#doubleClickPolicy} property of {@link TableEditorInputMode#clickInputMode} this event may be preceded by no, one, or two {@link TableEditorInputMode#addItemClickedListener ItemClicked}
events (as well as the corresponding button-specific event).
Button-specific and input-type specific events
({@link TableEditorInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked}, {@link TableEditorInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked}
and {@link TableEditorInputMode#addItemDoubleTappedListener ItemDoubleTapped}) will be raised
before this event. If one of those is marked {@link ClickEventArgs#handled} this event will not be raised anymore.
@param listener The listener to remove.
@see TableEditorInputMode#addItemClickedListener
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemLeftDoubleClickedListener
@see TableEditorInputMode#addItemRightDoubleClickedListener
@see TableEditorInputMode#addItemDoubleTappedListener
@see TableEditorInputMode#addItemDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemDoubleClicked Online Documentation}*/
removeItemDoubleClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>ItemLeftDoubleClicked</code> event that occurs when an item has been left double clicked.
This event will be raised before the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not
be raised in that case.
@param listener The listener to add.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemRightDoubleClickedListener
@see TableEditorInputMode#addItemDoubleTappedListener
@see TableEditorInputMode#removeItemLeftDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemLeftDoubleClicked Online Documentation}*/
addItemLeftDoubleClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>ItemLeftDoubleClicked</code> event that occurs when an item has been left double clicked.
This event will be raised before the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not
be raised in that case.
@param listener The listener to remove.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemRightDoubleClickedListener
@see TableEditorInputMode#addItemDoubleTappedListener
@see TableEditorInputMode#addItemLeftDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemLeftDoubleClicked Online Documentation}*/
removeItemLeftDoubleClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>ItemRightDoubleClicked</code> event that occurs when an item has been right double clicked.
This event will be raised before the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not
be raised in that case.
@param listener The listener to add.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemLeftDoubleClickedListener
@see TableEditorInputMode#addItemDoubleTappedListener
@see TableEditorInputMode#removeItemRightDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemRightDoubleClicked Online Documentation}*/
addItemRightDoubleClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>ItemRightDoubleClicked</code> event that occurs when an item has been right double clicked.
This event will be raised before the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not
be raised in that case.
@param listener The listener to remove.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemLeftDoubleClickedListener
@see TableEditorInputMode#addItemDoubleTappedListener
@see TableEditorInputMode#addItemRightDoubleClickedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemRightDoubleClicked Online Documentation}*/
removeItemRightDoubleClickedListener(listener: (sender: any, evt: TableItemClickedEventArgs<IModelItem>) => void): void;
/**
Adds the given listener for the <code>ItemDoubleTapped</code> event that occurs when an item has been double tapped.
This event will be raised before the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not
be raised in that case.
@param listener The listener to add.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemLeftDoubleClickedListener
@see TableEditorInputMode#addItemRightDoubleClickedListener
@see TableEditorInputMode#removeItemDoubleTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemDoubleTapped Online Documentation}*/
addItemDoubleTappedListener(listener: (sender: any, evt: TableItemTappedEventArgs<IModelItem>) => void): void;
/**
Removes the given listener for the <code>ItemDoubleTapped</code> event that occurs when an item has been double tapped.
This event will be raised before the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked}
event. If one of the event handlers sets the {@link ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore. This also means
that the {@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not
be raised in that case.
@param listener The listener to remove.
@see TableEditorInputMode#clickableItems
@see TableEditorInputMode#addItemDoubleClickedListener
@see TableEditorInputMode#addItemLeftDoubleClickedListener
@see TableEditorInputMode#addItemRightDoubleClickedListener
@see TableEditorInputMode#addItemDoubleTappedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableEditorInputMode%23ItemDoubleTapped Online Documentation}*/
removeItemDoubleTappedListener(listener: (sender: any, evt: TableItemTappedEventArgs<IModelItem>) => void): void;
static $class: Class;
private $_TableEditorInputMode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TableEditorInputMode;
}
export interface TableItemClickedEventArgs<T> extends ItemClickedEventArgs<T> {
}
/**
Provides data for the {@link TableEditorInputMode#addItemClickedListener ItemClicked},
{@link TableEditorInputMode#addItemDoubleClickedListener ItemDoubleClicked},
{@link TableEditorInputMode#addItemLeftClickedListener ItemLeftClicked},
{@link TableEditorInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked}, {@link TableEditorInputMode#addItemRightClickedListener ItemRightClicked}
and {@link TableEditorInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked} events.
Type parameter <code>T</code>: The type of the item this event carries.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableItemClickedEventArgs Online Documentation}*/
export class TableItemClickedEventArgs<T> {
/**
Initializes a new instance of the {@link TableItemClickedEventArgs} class.
@param item The item which is the subject of the event.
@param tableNode The table node where <code>item</code> is associated.
@param context The context in which the click or tap occurred.
@param location The location.
@param modifiers State of the modifiers.
@param mouseButtons The mouse button(s) that have changed.
@param clickCount The number of clicks.
@param region The specific region on a stripe where the click has occured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableItemClickedEventArgs Online Documentation}*/
constructor(item: T, region: StripeSubregion | null, tableNode: INode | null, context: IInputModeContext, location: Point | PointConvertible, modifiers: ModifierKeys | ModifierKeysConvertible, mouseButtons: MouseButtons | MouseButtonsConvertible, clickCount: number);
/**
The table node where {@link ItemClickedEventArgs#item} is associated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableItemClickedEventArgs%23tableNode Online Documentation}*/
readonly tableNode: INode | null;
/**
The specific region on a stripe where the click has occured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableItemClickedEventArgs%23region Online Documentation}*/
readonly region: StripeSubregion | null;
static $class: Class;
private $_TableItemClickedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is TableItemClickedEventArgs<T>;
}
export interface TableItemTappedEventArgs<T> extends ItemTappedEventArgs<T> {
}
/**
Provides data for the {@link TableEditorInputMode#addItemTappedListener ItemTapped} and {@link TableEditorInputMode#addItemDoubleTappedListener ItemDoubleTapped}
events.
Type parameter <code>T</code>: The type of the item this event carries.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableItemTappedEventArgs Online Documentation}*/
export class TableItemTappedEventArgs<T> {
/**
Initializes a new instance of the {@link TableItemTappedEventArgs} class.
@param item The item which is the subject of the event.
@param tableNode The table node where <code>item</code> is associated.
@param context The context in which the click or tap occurred.
@param location The location.
@param clickCount The number of clicks.
@param region The specific region on a stripe where the click has occured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableItemTappedEventArgs Online Documentation}*/
constructor(item: T, region: StripeSubregion | null, tableNode: INode | null, context: IInputModeContext, location: Point | PointConvertible, clickCount: number);
/**
The table node where {@link ItemTappedEventArgs#item} is associated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableItemTappedEventArgs%23tableNode Online Documentation}*/
readonly tableNode: INode | null;
/**
The specific region on a stripe where the click has occured.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableItemTappedEventArgs%23region Online Documentation}*/
readonly region: StripeSubregion | null;
static $class: Class;
private $_TableItemTappedEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is TableItemTappedEventArgs<T>;
}
export interface TableHitTestResult extends YObject {
}
/**
Support class to describe hit test results in the context of {@link TableEditorInputMode} interactions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableHitTestResult Online Documentation}*/
export class TableHitTestResult {
/**
The node where {@link TableHitTestResult#item} is directly or indirectly bound to.
Since the locations of table items are always relative to the table origin, this can be used to determine the absolute
location of the hit item without having to perform the whole hit test again.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableHitTestResult%23tableNode Online Documentation}*/
readonly tableNode: INode | null;
/**
The item that is actually hit.
Currently, this is either an {@link ILabel} bound to an {@link IStripe}, or an {@link IStripe} itself.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableHitTestResult%23item Online Documentation}*/
readonly item: IModelItem | null;
/**
Specific information about the region on a stripe where the hit was detected.
This can be used to e.g. handle hits on a stripe header differently from other hits.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableHitTestResult%23region Online Documentation}*/
readonly region: StripeSubregion | null;
static $class: Class;
private $_TableHitTestResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TableHitTestResult;
}
export interface TableReshapeHandler extends YObject, IReshapeHandler {
/**
@inheritdoc*/
cancelReshape(context: IInputModeContext, originalBounds: Rect): void;
/**
@inheritdoc*/
handleReshape(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
/**
@inheritdoc*/
initializeReshape(context: IInputModeContext): void;
/**
@inheritdoc*/
reshapeFinished(context: IInputModeContext, originalBounds: Rect, newBounds: Rect): void;
/**
@inheritdoc*/
readonly bounds: IRectangle;
} /**
{@link IReshapeHandler} implementation that constrains node resizes so that minimum size constraints of an
associated {@link ITable} are satisfied and which resizes the outer
{@link IStripe}s of such a table when the node is resized.
The constraints are applied in addition to any constraints imposed by the {@link IReshapeHandler} which is wrapped
by this implementation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableReshapeHandler Online Documentation}*/
export class TableReshapeHandler {
/**
Creates a new instance of this Handler which wraps an existing instance of <code>coreHandler</code> and uses <code>table</code> for additional
constraints.
In addition, <code>table</code>'s outer stripes are resized if necessary.
<code>table</code> should be associated to <code>tableNode</code>
@param coreHandler The handler that is wrapped by this instance.
@param tableNode The node for which this handler is used
@param table The table that is used for additional constraints.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableReshapeHandler Online Documentation}*/
constructor(coreHandler: IReshapeHandler, tableNode: INode, table: ITable);
/**
Gets the core reshape handler wrapped by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableReshapeHandler%23coreHandler Online Documentation}*/
readonly coreHandler: IReshapeHandler;
/**
Gets the node this handler is used for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableReshapeHandler%23tableNode Online Documentation}*/
readonly tableNode: INode;
/**
Gets the table used for additional constraints.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableReshapeHandler%23table Online Documentation}*/
readonly table: ITable;
static $class: Class;
private $_TableReshapeHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TableReshapeHandler;
}
export interface TemplateStripeStyleBase extends YObject, IStripeStyle, IPropertyObservable, ICloneable {
}
/**
Base class for stripe styles that use a SVG snippet as template for the visualization of {@link IStripe}s.
@see TemplateStripeStyle
@see StringTemplateStripeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase Online Documentation}*/
export abstract class TemplateStripeStyleBase {
/**
Constructs a new instance using the provided <code>renderer</code>.
@param renderer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23TemplateStripeStyleBase(TemplateStripeStyleRenderer) Online Documentation}*/
constructor(renderer: TemplateStripeStyleRenderer);
/**
Constructs a new instance of this style with default values.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23TemplateStripeStyleBase() Online Documentation}*/
constructor();
/**
Performs a shallow clone of this instance, except for the value of {@link TemplateStripeStyleBase#styleTag} which is cloned, too, if it can be cloned.
@returns A clone of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23clone() Online Documentation}*/
clone(): this;
/**
Factory method that is called by the {@link TemplateStripeStyleRenderer} to create the visual that will be used
for the display of the stripe.
@param context The context for which the visual should be created.
@param stripe The stripe that will be rendered.
@returns The readily styled visual.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23createTemplate(IRenderContext%2CIStripe) Online Documentation}*/
createTemplate(context: IRenderContext, stripe: IStripe): SvgVisual;
/**
Loads all template script elements that have not been parsed, yet, and creates the respective template cache objects.
This method can be called to perform the template loading at a certain point in time, for example during application
startup. By default, a template is loaded lazily when the first visual using this template is built.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23loadAllTemplates() Online Documentation}*/
static loadAllTemplates(): void;
/**
Raises the {@link TemplateStripeStyleBase#addPropertyChangedListener PropertyChanged}
event.
@param evt The {@link PropertyChangedEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23onPropertyChanged(PropertyChangedEventArgs) Online Documentation}*/
onPropertyChanged(evt: PropertyChangedEventArgs): void;
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
@param listener The listener to add.
@see TemplateStripeStyleBase#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23PropertyChanged Online Documentation}*/
addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
@param listener The listener to remove.
@see TemplateStripeStyleBase#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23PropertyChanged Online Documentation}*/
removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the tag that is associated with this style instance.
The various style renderer implementations can use this tag to make it accessible in the context of the instantiated
templates and controls. The framework does not make any assumptions about the value of this property. It is there for
convenient use in the process of data binding the templates. This property supports {@link IPropertyObservable property change notification} and can
thus be data bound easily.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23styleTag Online Documentation}*/
styleTag: any | null;
/**
Gets or sets an implementation of {@link IContextLookup} that can be used to satisfy queries that are made to the
implementation which is returned by calls to {@link IStripeStyleRenderer#getContext} that are made on the style
renderer that is associated with this style instance.
This can be used to conveniently customize the lookup behavior of the style renderers that are associated with this
instance. The default value is {@link IContextLookup#EMPTY_CONTEXT_LOOKUP}
@default 'Lookups.EmptyContextLookup'
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23contextLookup Online Documentation}*/
contextLookup: IContextLookup;
/**
Convenience getter that yields the {@link TemplateStripeStyleRenderer} that is used by this instance.
@see IStripeStyle#renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleBase%23renderer Online Documentation}*/
readonly renderer: IStripeStyleRenderer;
static $class: Class;
private $_TemplateStripeStyleBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplateStripeStyleBase;
}
export interface TemplateStripeStyle extends TemplateStripeStyleBase {
}
/**
An {@link IStripeStyle} implementation that uses an SVG snippet as a template for the visualization of
{@link IStripe}s.
The {@link TemplateStripeStyle#styleResourceKey} is used to determine the template that will be applied to the {@link Visual}. The template must be specified in
the DOM in a script tag with <code>type='text/yfiles-template'</code>.
The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
value to the value of the property with the given name in the node's tag. The tag itself can be bound with
<code>attribute="{Binding}"</code>. If the bound object provides the function <code>addPropertyChangedListener(listener)</code> or implements
{@link IPropertyObservable}, bindings to properties of the business object are dynamically updated if the registered
listeners are called on property change. Business objects can be made observable by calling
{@link TemplateStripeStyle#makeObservable}.
TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to a set of
well-defined properties of the node that are provided by the style. The available template binding properties are:
<ul>
<li>width: The width of the graph element
<li>height: The height of the graph element
<li>bounds: The bounding rectangle of the graph element
<li>canvas: The {@link GraphComponent} the element is rendered in
<li>zoom: The current zoom level of the {@link GraphComponent}
<li>itemSelected: Whether the element is selected
<li>itemFocused: Whether the element has the focus
<li>itemHighlighted: Whether the element is highlighted
<li>styleTag: The tag associated with the element's style instance
</ul>
Bindings can specify converters and converter parameters. Converters are simple functions with the signature
<code>function(value:Object, parameter:Object):Object</code>. A converter has to be registered under its name either in the {@link TemplateStripeStyle#CONVERTERS converter store}
or with the <code>window</code> object before it can be used.
<code>TextWrapConverter</code> is a built-in converter function which emits wrapped text as SVG elements. It can be used to display
wrapped text in a template style. To use the <code>TextWrapConverter</code> the template style must include a &lt;g> element that binds
a property containing the text to be wrapped to <code>data-content</code>. <code>TextWrapConverter</code> must be specified as the converter
function. The converter’s parameters are passed using a shorthand notation for width, height, font, trimming and color.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/_text_wrapping_in_a_template_style Text Wrapping in a Template Style} for more details.
Depending on the current state, the following CSS classes are added to the root element of the template:
<ul>
<li><code>yfiles-selected</code>
<li><code>yfiles-focused</code>
<li><code>yfiles-highlighted</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyle Online Documentation}*/
export class TemplateStripeStyle {
/**
Creates a new instance using the given resource key and a custom renderer instance.
@param [styleResourceKey=null] The key of the style resource that will be used for the component.
@param [renderer=null] The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyle Online Documentation}*/
constructor(styleResourceKey?: string | null, renderer?: TemplateStripeStyleRenderer);
/**
Creates a new instance using the given resource key and a custom renderer instance.
@param options The parameters to pass.
@param [options.styleResourceKey=null] The key of the style resource that will be used for the component.
@param [options.renderer=null] The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyle Online Documentation}*/
constructor(options: {
styleResourceKey?: string | null;
renderer?: TemplateStripeStyleRenderer;
});
/**
Makes an object observable to bindings in template styles.
Bindings to observable objects react to changes in the bound data. This enables dynamic styling of graph items that
reacts to changes of the underlying business data of each graph item.
This method adds the functions <code>firePropertyChanged</code>, <code>addPropertyChangedListener</code> and <code>removePropertyChangedListener</code> to the
given object. Call <code>firePropertyChanged(propertyName: string)</code> each time a bound property value changes in order to update
the related bindings.
@param dataObject The object to make observable.
@returns The object that was passed in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyle%23makeObservable(Object) Online Documentation}*/
static makeObservable(dataObject: Object): Object;
/**
Gets or sets the {@link string} that will be used to look up the template that is applied to the component.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyle%23styleResourceKey Online Documentation}*/
styleResourceKey: string | null;
/**
Gets the storage object for binding converters.
Binding converter functions can be stored in this object. Once a converter function name is resolved during template
parsing, the converter function is located in this object first. If it is not found in this object, it is located on the
window object. The binding converter store is shared among all template styles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyle%23CONVERTERS Online Documentation}*/
static readonly CONVERTERS: any;
static $class: Class;
private $_TemplateStripeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplateStripeStyle;
}
export interface StringTemplateStripeStyle extends TemplateStripeStyleBase {
}
/**
A {@link IStripeStyle} that uses an SVG snippet as template for the visualization of {@link IStripe}s.
In contrast to {@link TemplateStripeStyle}, the SVG template used by this style must be encoded as string and
assigned to the {@link StringTemplateStripeStyle#svgContent} property.
It is vital that the {@link StringTemplateStripeStyle#svgContent SVG snippet} does not declare the SVG namespace itself. Its content is parsed in a dummy namespace
in order to avoid immediate validation of the result. The resulting SVG elements are automatically created in the SVG
namespace before they are added to the DOM.
The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
value to the value of the property with the given name in the node's tag. The tag itself can be bound with
<code>attribute="{Binding}"</code>. If the bound object provides the function <code>addPropertyChangedListener(listener)</code> or implements
{@link IPropertyObservable}, bindings to properties of the business object are dynamically updated if the registered
listeners are called on property change. Business objects can be made observable by calling
{@link StringTemplateStripeStyle#makeObservable}.
TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to a set of
well-defined properties of the node that are provided by the style. The available template binding properties are:
<ul>
<li>width: The width of the graph element
<li>height: The height of the graph element
<li>bounds: The bounding rectangle of the graph element
<li>canvas: The {@link GraphComponent} the element is rendered in
<li>zoom: The current zoom level of the {@link GraphComponent}
<li>itemSelected: Whether the element is selected
<li>itemFocused: Whether the element has the focus
<li>itemHighlighted: Whether the element is highlighted
<li>styleTag: The tag associated with the element's style instance
</ul>
Bindings can specify converters and converter parameters. Converters are simple functions with the signature
<code>function(value:Object, parameter:Object):Object</code>. A converter has to be registered under its name either in the {@link StringTemplateStripeStyle#CONVERTERS converter store}
or with the <code>window</code> object before it can be used.
<code>TextWrapConverter</code> is a built-in converter function which emits wrapped text as SVG elements. It can be used to display
wrapped text in a template style. To use the <code>TextWrapConverter</code> the template style must include a &lt;g> element that binds
a property containing the text to be wrapped to <code>data-content</code>. <code>TextWrapConverter</code> must be specified as the converter
function. The converter’s parameters are passed using a shorthand notation for width, height, font, trimming and color.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/_text_wrapping_in_a_template_style Text Wrapping in a Template Style} for more details.
Depending on the current state, the following CSS classes are added to the root element of the template:
<ul>
<li><code>yfiles-selected</code>
<li><code>yfiles-focused</code>
<li><code>yfiles-highlighted</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateStripeStyle Online Documentation}*/
export class StringTemplateStripeStyle {
/**
Creates a new instance using the given SVG template and a custom renderer instance.
@param [svgContent=null] The SVG snippet that will be used as a template for the component.
@param [renderer=null] The custom renderer for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateStripeStyle Online Documentation}*/
constructor(svgContent?: string | null, renderer?: TemplateStripeStyleRenderer);
/**
Creates a new instance using the given SVG template and a custom renderer instance.
@param options The parameters to pass.
@param [options.svgContent=null] The SVG snippet that will be used as a template for the component.
@param [options.renderer=null] The custom renderer for this instance.
@param [options.styleTag] The tag that is associated with this style instance. This option sets the {@link TemplateStripeStyleBase#styleTag} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateStripeStyle Online Documentation}*/
constructor(options: {
svgContent?: string | null;
renderer?: TemplateStripeStyleRenderer;
styleTag?: any | null;
});
/**
Makes an object observable to bindings in template styles.
Bindings to observable objects react to changes in the bound data. This enables dynamic styling of graph items that
reacts to changes of the underlying business data of each graph item.
This method adds the functions <code>firePropertyChanged</code>, <code>addPropertyChangedListener</code> and <code>removePropertyChangedListener</code> to the
given object. Call <code>firePropertyChanged(propertyName: string)</code> each time a bound property value changes in order to update
the related bindings.
@param dataObject The object to make observable.
@returns The object that was passed in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateStripeStyle%23makeObservable(Object) Online Documentation}*/
static makeObservable(dataObject: Object): Object;
/**
Gets or sets the SVG content that is the template for the stripe visualization.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateStripeStyle%23svgContent Online Documentation}*/
svgContent: string | null;
/**
Specifies whether code contained in the templates should be included in the DOM.
Script tags and event listeners can be used to inject malicious code. This property should only be enabled if the
templates originate from trusted sources. If the application allows to load external GraphML files, this property should
probably not be enabled.
This setting is shared by {@link StringTemplateNodeStyle}, StringLabelControlPortStyle and
{@link StringTemplatePortStyle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateStripeStyle%23trusted Online Documentation}*/
static trusted: boolean;
/**
Gets the storage object for binding converters.
Binding converter functions can be stored in this object. Once a converter function name is resolved during template
parsing, the converter function is located in this object first. If it is not found in this object, it is located on the
window object. The binding converter store is shared among all template styles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateStripeStyle%23CONVERTERS Online Documentation}*/
static readonly CONVERTERS: any;
static $class: Class;
private $_StringTemplateStripeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StringTemplateStripeStyle;
}
export interface TemplateStripeStyleRenderer extends YObject, IStripeStyleRenderer, ILookup, IVisualCreator {
}
/**
An implementation of the {@link IStripeStyleRenderer} interface that can render {@link TemplateStripeStyle}
instances.
<h2>Note</h2>
It is not recommended to inherit from this class because it is not guaranteed that the visuals are updated correctly in
this case.
@see TemplateStripeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer Online Documentation}*/
export class TemplateStripeStyleRenderer {
/**
Prepares this instance for subsequent calls after the style and stripe have been initialized.
Upon invocation the {@link TemplateStripeStyleRenderer#style} and {@link TemplateStripeStyleRenderer#stripe} properties have been populated by the {@link TemplateStripeStyleRenderer#getVisualCreator}
or {@link TemplateStripeStyleRenderer#getContext} methods.
This is an empty implementation. Subclasses might have to override this method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Creates the visual by delegating to CreateControl and then initializing the control with the context.
@param context The context for the creation.
@returns The control that is used for rendering the stripe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23createVisual(IRenderContext) Online Documentation}*/
createVisual(context: IRenderContext): Visual | null;
/**
Configures the <code>style</code> and <code>stripe</code> parameters, does <b>not</b> call {@link TemplateStripeStyleRenderer#configure} and returns
<code>this</code>.
As this method may be called often it will not automatically call {@link TemplateStripeStyleRenderer#configure},
instead subclasses should ensure that in the {@link TemplateStripeStyleRenderer#lookup} method call they should
call {@link TemplateStripeStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an
instance that depends on a correctly configured <code>this</code>.
@param stripe The stripe to query the context for. The value will be stored in the {@link TemplateStripeStyleRenderer#stripe} property.
@param style The style to associate with the stripe. The value will be stored in the {@link TemplateStripeStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateStripeStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23getContext(IStripe%2CIStripeStyle) Online Documentation}*/
getContext(stripe: IStripe, style: IStripeStyle): ILookup;
/**
Gets the context by delegating to {@link TemplateStripeStyleBase#contextLookup}.
@param style
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23getContextLookup(TemplateStripeStyleBase) Online Documentation}*/
getContextLookup(style: TemplateStripeStyleBase): IContextLookup;
/**
Configures the <code>style</code> and <code>stripe</code> parameters, calls {@link TemplateStripeStyleRenderer#configure} and returns <code>this</code>.
@param stripe The stripe to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link TemplateStripeStyleRenderer#stripe} property.
@param style The style to associate with the stripe. The value will be stored in the {@link TemplateStripeStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateStripeStyleRenderer#createVisual
@see TemplateStripeStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23getVisualCreator(IStripe%2CIStripeStyle) Online Documentation}*/
getVisualCreator(stripe: IStripe, style: IStripeStyle): IVisualCreator;
/**
Implementation of the {@link ILookup#lookup} method.
Delegates to the lookup returned by GetContextLookup(StripeControlStripeStyle), first. Last, it returns this class if it
implements the queried <code>type</code>.
@param type The type to query an instance for.
@returns The implementation or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Updates the component correspondingly.
@param context The context for the creation.
@param oldVisual The currently rendered visual.
@returns The control that is used for rendering the stripe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23updateVisual(IRenderContext%2CVisual) Online Documentation}*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23style Online Documentation}*/
style: TemplateStripeStyleBase;
/**
Gets or sets the currently configured stripe.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23stripe Online Documentation}*/
stripe: IStripe;
/**
Gets the {@link IStripe#layout} of the stripe this renderer is currently configured for.
Returns <code>Stripe.Layout</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateStripeStyleRenderer%23layout Online Documentation}*/
readonly layout: IRectangle;
static $class: Class;
private $_TemplateStripeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplateStripeStyleRenderer;
}
export interface TableNodeStyle extends YObject, INodeStyle, IPropertyObservable {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
An {@link INodeStyle} for {@link ITable tables}.
This implementations stores the table instance and therefore, its instances cannot be shared among different table
instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle Online Documentation}*/
export class TableNodeStyle {
/**
Constructor with a custom renderer implementation that uses <code>table</code> as backing table model.
@param [table=null] The table instance that defines the tabular structure.
@param [renderer=null] Custom renderer instance for this style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle Online Documentation}*/
constructor(table?: ITable, renderer?: TableNodeStyleRenderer);
/**
Constructor with a custom renderer implementation that uses <code>table</code> as backing table model.
@param options The parameters to pass.
@param [options.table=null] The table instance that defines the tabular structure.
@param [options.renderer=null] Custom renderer instance for this style.
@param [options.backgroundStyle] The style that is used to draw the background of the table. This option sets the {@link TableNodeStyle#backgroundStyle} property on the created object.
@param [options.tableRenderingOrder] The order in which rows and columns are rendered. This option sets the {@link TableNodeStyle#tableRenderingOrder} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle Online Documentation}*/
constructor(options: {
table?: ITable;
renderer?: TableNodeStyleRenderer;
backgroundStyle?: INodeStyle | null;
tableRenderingOrder?: TableRenderingOrder | TableRenderingOrderStringValues;
});
/**
Create a copy of the background style when this style instance is cloned
@param style The original background node style
@returns This implementation always returns the original <code>style</code> instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle%23copyBackgroundStyle(INodeStyle) Online Documentation}*/
copyBackgroundStyle(style: INodeStyle | null): INodeStyle | null;
/**
Create a copy of the table when this style instance is cloned
@param table The original table instance
@returns This implementation always tries to {@link ICloneable#clone} clone the original <code>table</code> instance, if possible,
otherwise it returns <code>table</code> unchanged.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle%23copyTable(ITable) Online Documentation}*/
copyTable(table: ITable): ITable;
/**
Raises the {@link TableNodeStyle#addPropertyChangedListener PropertyChanged} event.
Called when the value of the {@link TableNodeStyle#table} property changes.
@param evt The {@link PropertyChangedEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle%23onPropertyChanged(PropertyChangedEventArgs) Online Documentation}*/
onPropertyChanged(evt: PropertyChangedEventArgs): void;
/**
Gets or sets the order in which rows and columns are rendered.
@default TableRenderingOrder.COLUMNS_FIRST
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle%23tableRenderingOrder Online Documentation}*/
tableRenderingOrder: TableRenderingOrder;
/**
Gets or sets the style that is used to draw the background of the table.
By default, the background style is a {@link VoidNodeStyle} and nothing is drawn.
@default 'TableNodeStyle.defaultBackgroundStyle'
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle%23backgroundStyle Online Documentation}*/
backgroundStyle: INodeStyle | null;
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when the value of the {@link TableNodeStyle#table} property changes.
@param listener The listener to add.
@see TableNodeStyle#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle%23PropertyChanged Online Documentation}*/
addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when the value of the {@link TableNodeStyle#table} property changes.
@param listener The listener to remove.
@see TableNodeStyle#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle%23PropertyChanged Online Documentation}*/
removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the {@link ITable} instance that defines the tabular structure.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyle%23table Online Documentation}*/
table: ITable;
static $class: Class;
private $_TableNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TableNodeStyle;
}
export interface TableNodeStyleRenderer extends YObject, INodeStyleRenderer, IShapeGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
@inheritdoc*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
@inheritdoc*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
@inheritdoc*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
@inheritdoc*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
@inheritdoc*/
getOutline(): GeneralPath | null;
/**
@inheritdoc*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
@inheritdoc*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
@inheritdoc*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
isInside(location: Point): boolean;
/**
@inheritdoc*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
Default renderer implementation that can be used for {@link TableNodeStyle} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyleRenderer Online Documentation}*/
export class TableNodeStyleRenderer {
/**
Prepares this instance for subsequent calls after the style and node have been initialized.
Upon invocation the {@link TableNodeStyleRenderer#style} and {@link TableNodeStyleRenderer#node} properties have been populated by the {@link TableNodeStyleRenderer#getVisualCreator},
{@link TableNodeStyleRenderer#getBoundsProvider}, {@link TableNodeStyleRenderer#getHitTestable}, or {@link TableNodeStyleRenderer#getMarqueeTestable}
methods.
This is an empty implementation. Subclasses might have to override this method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Get the {@link INodeStyle background style} that is currently used by this renderer.
This implementation retrieves the {@link TableNodeStyle#backgroundStyle} from the owning style
@returns The {@link INodeStyle background style} that is currently used by this renderer.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyleRenderer%23getBackgroundStyle() Online Documentation}*/
getBackgroundStyle(): INodeStyle | null;
/**
Gets the table object that should be used for rendering.
This implementation delegates to the Table provided by the {@link TableNodeStyle#table} property.
@returns The table that should be used for the rendering process.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyleRenderer%23getTable() Online Documentation}*/
getTable(): ITable | null;
/**
Get the {@link TableRenderingOrder} that is currently used by this renderer.
This implementation retrieves the {@link TableRenderingOrder} from the owning style
@returns The {@link TableRenderingOrder} that is currently used by this renderer.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyleRenderer%23getTableRenderingOrder() Online Documentation}*/
getTableRenderingOrder(): TableRenderingOrder;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyleRenderer%23style Online Documentation}*/
style: TableNodeStyle;
/**
Gets or sets the currently configured node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableNodeStyleRenderer%23node Online Documentation}*/
node: INode;
static $class: Class;
private $_TableNodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TableNodeStyleRenderer;
}
export interface TableAnimation extends YObject, IAnimation {
/**
@inheritdoc*/
animate(time: number): void;
/**
@inheritdoc*/
cleanUp(): void;
/**
@inheritdoc*/
initialize(): void;
} /**
An animation for the {@link IStripe stripes} in an {@link ITable}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableAnimation Online Documentation}*/
export class TableAnimation {
/**
Initializes a new instance of the {@link TableAnimation} class for the given table, the column widths, and row
heights.
Note that only the sizes of
<b>leaf stripes</b>, i.e. those without child stripes, are considered. The sizes of a stripe with children is implicitly determined by its
contents.
@param table The table to animate.
@param columnLayout The sizes of the <b>leaf</b> columns, in natural order.
@param rowLayout The sizes of the <b>leaf</b> rows, in natural order.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableAnimation Online Documentation}*/
constructor(table: ITable, columnLayout: number[], rowLayout: number[]);
/**
Create the animation for a single {@link IStripe}
@param stripe The stripe
@param targetSize The target size of the stripe
@returns An animation that morphs the stripe size
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableAnimation%23createStripeAnimation(IStripe%2Cnumber) Online Documentation}*/
createStripeAnimation(stripe: IStripe, targetSize: number): IAnimation;
/**
Gets or sets the preferred duration of the animation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableAnimation%23preferredDuration Online Documentation}*/
preferredDuration: TimeSpan;
static $class: Class;
private $_TableAnimation: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TableAnimation;
}
/**
Enumeration for the possible results of a reparent gesture.
This enumeration determines where to place the stripe as result of the reparent gesture.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeReparentPolicy Online Documentation}*/
export enum StripeReparentPolicy {
/**
Add the moved stripe as a new child of the target stripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeReparentPolicy%23ADD_CHILD Online Documentation}*/
ADD_CHILD,
/**
Insert the moved stripe before the target stripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeReparentPolicy%23INSERT_BEFORE Online Documentation}*/
INSERT_BEFORE,
/**
Insert the moved stripe after the target stripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeReparentPolicy%23INSERT_AFTER Online Documentation}*/
INSERT_AFTER,
/**
The current gesture would result in an invalid operation (such as trying to make a stripe to its own child).
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeReparentPolicy%23INVALID Online Documentation}*/
INVALID
}
/** Automatic conversion to {@link StripeReparentPolicy}. */
type StripeReparentPolicyStringValues = "add-child" | "insert-before" | "insert-after" | "invalid";
export interface IStripe extends ILabelOwner {
}
/**
This interface defines a child element of a table or a table element container
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe Online Documentation}*/
export abstract class IStripe {
/**
Returns the layout of <code>stripe</code> in absolute coordinates.
Since {@link IStripe#layout} is relative to the owning node's left corner, you need to provide an <code>node</code> node to calculate the absolute layout.
@param node The node relative to which the stripe layout is calculated. This should be the same node where {@link IStripe#table} for <code>stripe</code> is associated
to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23getAbsoluteBounds(INode) Online Documentation}*/
getAbsoluteBounds(node: INode): Rect;
/**
Gets the insets for this stripe.
These insets are applied in addition to any implicit insets provided by the child stripes. To find out the actual insets
that are currently in effect, use {@link IStripe#actualInsets}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23insets Online Documentation}*/
abstract readonly insets: Insets;
/**
Gets a live view of the relative layout of the stripe.
The layout of a stripe is a rectangle in the coordinate system that is relative to the upper left corner of the parent.
Since this method will yield a live view, it is up to the client to copy the values of the instance if a snapshot of the
state is needed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23layout Online Documentation}*/
abstract readonly layout: IRectangle;
/**
Gets the style for this stripe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23style Online Documentation}*/
abstract readonly style: IStripeStyle;
/**
Gets the current changeable dimension of the stripe.
This is always the stripe size which is not implicitly determined by other stripes, i.e. the height for {@link IRow}
instances and the width for {@link IColumn} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23size Online Documentation}*/
abstract readonly size: number;
/**
Gets the minimal permissible value for {@link IStripe#size}.
This value determines how much the stripe may be shrunk by interactive manipulation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23minimumSize Online Documentation}*/
abstract readonly minimumSize: number;
/**
Gets the {@link ITable} instance where this stripe is installed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23table Online Documentation}*/
abstract readonly table: ITable | null;
/**
Checks whether <code>stripe</code> is the root item of a stripe hierarchy.
@see IStripe#root
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23isRoot Online Documentation}*/
readonly isRoot: boolean;
/**
Gets the index of <code>stripe</code> in its {@link IStripe#parentStripe} collection of {@link IStripe#childStripes}.
The <code>stripe</code> is the <b>GetIndex</b>th child of its parent.
@see IColumn#childColumns
@see IRow#childRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23index Online Documentation}*/
readonly index: number;
/**
Returns the actual minimum size a stripe can acquire.
This is the maximum of {@link IStripe#minimumSize} and the horizontal or vertical {@link IStripe#insets}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23actualMinSize Online Documentation}*/
readonly actualMinSize: number;
/**
Gets the direct children of <code>stripe</code> in a table hierarchy.
For {@link IColumn} instances, this returns {@link IColumn#childColumns}, for {@link IRow} instances, this returns {@link IRow#childRows}
@see IColumn#childColumns
@see IRow#childRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23childStripes Online Documentation}*/
readonly childStripes: IEnumerable<IStripe>;
/**
Gets the parent of <code>stripe</code> in a table hierarchy.
For {@link IColumn} instances, this returns {@link IColumn#parentColumn}, for {@link IRow} instances, this returns {@link IRow#parentRow}
@see IColumn#parentColumn
@see IRow#parentRow
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23parentStripe Online Documentation}*/
readonly parentStripe: IStripe | null;
/**
Gets the root of the stripe hierarchy of <code>stripe</code> in a table hierarchy.
If the stripe is part of an {@link ITable} instance, this returns {@link ITable#rootColumn} for {@link IColumn} instances and {@link IRow#parentRow} for {@link ITable#rootRow}
instances. Otherwise <code>null</code> is returned.
@see ITable#rootColumn
@see ITable#rootRow
@see IStripe#isRoot
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23root Online Documentation}*/
readonly root: IStripe | null;
/**
Returns a collection of all descendants of <code>stripe</code> that are leaves and therefore don't have any child stripes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23leaves Online Documentation}*/
readonly leaves: IEnumerable<IStripe>;
/**
Returns a collection of all descendants of <code>stripe</code>.
The descendants are returned in top to bottom order.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23descendants Online Documentation}*/
readonly descendants: IEnumerable<IStripe>;
/**
Gets the insets that are currently in effect for this stripe.
This value is determined by the accumulated insets of all ancestor, sibling and descendant insets. These insets may be
larger than {@link IStripe#insets}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23actualInsets Online Documentation}*/
readonly actualInsets: Insets;
/**
The current actual dimension of the stripe.
This is always the stripe size which is actually used for the stripe. For leaf stripes, this is the same as
{@link IStripe#size}, otherwise, it is the accumulated size of all descendants with taking nested insets into account.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripe%23actualSize Online Documentation}*/
readonly actualSize: number;
static $class: Class;
private $_IStripe: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IStripe;
}
export interface IStripeHitTestHelper {
}
/**
Helper interface that allows to bundle {@link IHitTestable} implementations for the main regions of a {@link IStripe}
instance.
The default {@link IRow} and {@link IColumn} implementations provide default implementations of this
interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeHitTestHelper Online Documentation}*/
export abstract class IStripeHitTestHelper {
/**
Return a {@link IHitTestable} that should report hits on the far border region of the stripe.
The far border is the region that is used for the resize gesture in {@link ResizeStripeInputMode} to expand the
stripe to the bottom or the right. This is typically the bottom border for an {@link IRow} instance and the right
border for an {@link IColumn}.
@returns a {@link IHitTestable} that should report hits on the far border region of the stripe.
@see StripeSubregion
@see StripeSubregionTypes#FAR_BORDER
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeHitTestHelper%23getFarBorderHitTestable() Online Documentation}*/
abstract getFarBorderHitTestable(): IHitTestable;
/**
Return a {@link IHitTestable} that should report hits on the leading header region of the stripe.
The leading header is the region that is used by default for click selection and as start location for a stripe drag.
Typically, this is the area occupied by the left insets for an {@link IRow} instance and by the top insets for an {@link IColumn}
instance.
@returns a {@link IHitTestable} that should report hits on the header region of the stripe.
@see StripeSubregion
@see StripeSubregionTypes#HEADER
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeHitTestHelper%23getLeadingHeaderHitTestable() Online Documentation}*/
abstract getLeadingHeaderHitTestable(): IHitTestable;
/**
Return a {@link IHitTestable} that should report hits on the near border region of the stripe.
The near border is the region that is used for the resize gesture in {@link ResizeStripeInputMode} to expand the
stripe to the top or the left. This is typically the upper border for an {@link IRow} instance and the left border
for an {@link IColumn}.
@returns a {@link IHitTestable} that should report hits on the near border region of the stripe.
@see StripeSubregion
@see StripeSubregionTypes#NEAR_BORDER
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeHitTestHelper%23getNearBorderHitTestable() Online Documentation}*/
abstract getNearBorderHitTestable(): IHitTestable;
/**
Return a {@link IHitTestable} that should report hits on the whole stripe.
@returns a {@link IHitTestable} that should report hits on the whole stripe.
@see StripeSubregion
@see StripeSubregionTypes#STRIPE
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeHitTestHelper%23getStripeHitTestable() Online Documentation}*/
abstract getStripeHitTestable(): IHitTestable;
/**
Return a {@link IHitTestable} that should report hits on the trailing header region of the stripe.
The trailing header is the region that is used by default for click selection and as start location for a stripe drag.
Typically, this is the area occupied by the right insets for an {@link IRow} instance and by the bottom insets for
an {@link IColumn} instance.
@returns a {@link IHitTestable} that should report hits on the header region of the stripe.
@see StripeSubregion
@see StripeSubregionTypes#HEADER
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeHitTestHelper%23getTrailingHeaderHitTestable() Online Documentation}*/
abstract getTrailingHeaderHitTestable(): IHitTestable;
static $class: Class;
private $_IStripeHitTestHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IStripeHitTestHelper;
}
/**
Enumeration that specifies the various hot spot regions of an {@link IStripe} instance.
Members of this enumeration can be combined to specify composite regions.
@see StripeSubregion
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregionTypes Online Documentation}*/
export enum StripeSubregionTypes {
/**
No active hotspot region.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregionTypes%23NONE Online Documentation}*/
NONE,
/**
The hot spot region includes the stripe itself.
@see IStripeHitTestHelper#getStripeHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregionTypes%23STRIPE Online Documentation}*/
STRIPE,
/**
The hot spot region includes the leading stripe header.
The leading header is the region that is used by default for click selection and as start location for a stripe drag.
Typically, this is the area occupied by the left insets for an {@link IRow} instance and by the top insets for an {@link IColumn}
instance.
@see IStripeHitTestHelper#getLeadingHeaderHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregionTypes%23LEADING_HEADER Online Documentation}*/
LEADING_HEADER,
/**
The hot spot region includes the trailing stripe header.
The trailing header is the region that is used by default for click selection and as start location for a stripe drag.
Typically, this is the area occupied by the right insets for an {@link IRow} instance and by the bottom insets for
an {@link IColumn} instance.
@see IStripeHitTestHelper#getTrailingHeaderHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregionTypes%23TRAILING_HEADER Online Documentation}*/
TRAILING_HEADER,
/**
The hot spot region includes both stripe headers.
This is a bitwise combination of {@link StripeSubregionTypes#LEADING_HEADER} and
{@link StripeSubregionTypes#TRAILING_HEADER}
@see IStripeHitTestHelper#getTrailingHeaderHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregionTypes%23HEADER Online Documentation}*/
HEADER,
/**
The hot spot region includes the near-border region of the stripe.
The near border is the region that is used for the resize gesture in {@link ResizeStripeInputMode} to expand the
stripe to the top or the left. This is typically the upper border for an {@link IRow} instance and the left border
for an {@link IColumn}.
@see IStripeHitTestHelper#getNearBorderHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregionTypes%23NEAR_BORDER Online Documentation}*/
NEAR_BORDER,
/**
The hot spot region includes the stripe far-border region.
The far border is the region that is used for the resize gesture in {@link ResizeStripeInputMode} to expand the
stripe to the bottom or the right. This is typically the bottom border for an {@link IRow} instance and the right
border for an {@link IColumn}.
@see IStripeHitTestHelper#getFarBorderHitTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregionTypes%23FAR_BORDER Online Documentation}*/
FAR_BORDER,
/**
Flag for any stripe region.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregionTypes%23ALL Online Documentation}*/
ALL
}
type StripeSubregionTypesStringValues = "none" | "stripe" | "leading-header" | "trailing-header" | "header" | "near-border" | "far-border" | "all";
/** Automatic conversion to {@link StripeSubregionTypes}. */
type StripeSubregionTypesConvertible = StripeSubregionTypesStringValues | StripeSubregionTypesStringValues[];
export interface StripeSubregion extends YObject {
}
/**
Further describes the result of a hit test on a stripe or table.
Instances of this class are returned by the default {@link StripeHitTester} implementation.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregion Online Documentation}*/
export class StripeSubregion {
/**
Create a new instance that describes the specific subregion.
@param subregionType The specific subregion
@param stripe The stripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregion Online Documentation}*/
constructor(subregionType: StripeSubregionTypes | StripeSubregionTypesConvertible, stripe: IStripe);
/**
Gets the specific subregion type that this instance describes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregion%23subregionType Online Documentation}*/
readonly subregionType: StripeSubregionTypes;
/**
Gets the stripe that this instance describes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregion%23stripe Online Documentation}*/
readonly stripe: IStripe;
/**
Gets the node the region is associated to.
This property is non-<code>null</code> only if this instance was returned by
{@link ReparentStripePositionHandler#getTargetSubregion}. Otherwise, especially if this instance was returned by
{@link ITable#findStripe}, it is <code>null</code>. In this case, you can get the table from the {@link StripeSubregion#stripe} of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSubregion%23targetTableNode Online Documentation}*/
targetTableNode: INode | null;
static $class: Class;
private $_StripeSubregion: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StripeSubregion;
}
export interface DefaultStripeInputVisualizationHelper extends YObject, IStripeInputVisualizationHelper {
}
/**
Default implementation of {@link IStripeInputVisualizationHelper} that uses
{@link IVisualTemplate}s for the actual visualization.
If you want to change the global behavior, you can just provide a different {@link IVisualTemplate} for the various
predefined template keys in the application resources. If you want to override the visualization for a specific stripe
only, assign a different {@link IVisualTemplate} to the instance that is already present in the stripe's lookup.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper Online Documentation}*/
export class DefaultStripeInputVisualizationHelper {
/**
Default constructor
@param stripe The stripe for which the visualizations are intended
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper Online Documentation}*/
constructor(stripe: IStripe);
/**
Provide a {@link IVisualTemplate} that is used to visualize <code>type</code>.
@param context The input mode context. Allows you to access the {@link CanvasComponent} to retrieve the resources.
@param type The visualization type
@returns A {@link IVisualTemplate} that is used to visualize <code>type</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23findTemplate(IInputModeContext%2CStripeVisualizationType) Online Documentation}*/
findTemplate(context: IInputModeContext, type: StripeVisualizationType | StripeVisualizationTypeStringValues): IVisualTemplate | null;
/**
Return an {@link IVisualCreator} that is used to visualize the input operation specified by <code>type</code>.
This implementation uses {@link IVisualTemplate}s for the actual visualization.
@param context The input mode context which provides further information.
@param tableNode The node where the stripe's owner is currently bound to.
@param type The actual visualization type
@returns A {@link IVisualCreator} instance that renders a representation for the current operation.
@see DefaultStripeInputVisualizationHelper#resizeStripeTemplate
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23getVisualCreator(IInputModeContext%2CINode%2CStripeVisualizationType) Online Documentation}*/
getVisualCreator(context: IInputModeContext, tableNode: INode, type: StripeVisualizationType | StripeVisualizationTypeStringValues): IVisualCreator;
/**
A {@link string} that can be used to store a {@link IVisualTemplate} that can be used to create the visual that will be used to
draw the highlight for the stripe that is resized.
This template is used for {@link StripeVisualizationType#RESIZE}
The default template has the <code>yfiles-stripe-resize-template</code> CSS class.
@see DefaultStripeInputVisualizationHelper#getVisualCreator
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23RESIZE_STRIPE_TEMPLATE_KEY Online Documentation}*/
static RESIZE_STRIPE_TEMPLATE_KEY: string;
/**
Gets or sets the {@link IVisualTemplate} that is used to visualize the current stripe resize operation.
This template is used for {@link StripeVisualizationType#RESIZE}
@see DefaultStripeInputVisualizationHelper#getVisualCreator
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23resizeStripeTemplate Online Documentation}*/
resizeStripeTemplate: IVisualTemplate | null;
/**
A {@link string} that can be used to store a {@link IVisualTemplate} that can be used to create the visual that will be used to
draw the highlight for the stripe that is selected.
This template is used for {@link StripeVisualizationType#SELECTION}
The default template has the <code>yfiles-stripe-selection-template</code> CSS class.
@see DefaultStripeInputVisualizationHelper#getVisualCreator
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23SELECTED_STRIPE_TEMPLATE_KEY Online Documentation}*/
static SELECTED_STRIPE_TEMPLATE_KEY: string;
/**
Gets or sets the {@link IVisualTemplate} that is used to visualize the current stripe selection operation.
This template is used for {@link StripeVisualizationType#SELECTION}
@see DefaultStripeInputVisualizationHelper#getVisualCreator
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23selectedStripeTemplate Online Documentation}*/
selectedStripeTemplate: IVisualTemplate | null;
/**
A {@link string} that can be used to store a {@link IVisualTemplate} that can be used to create the visual that will be used to
draw the highlight for the current source of a stripe reparent operation.
This template is used for {@link StripeVisualizationType#DRAG_SOURCE}
The default template has the <code>yfiles-stripe-drag-template</code> CSS class.
@see DefaultStripeInputVisualizationHelper#getVisualCreator
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23DRAG_SOURCE_STRIPE_TEMPLATE_KEY Online Documentation}*/
static DRAG_SOURCE_STRIPE_TEMPLATE_KEY: string;
/**
Gets or sets the {@link IVisualTemplate} that is used to visualize the current source of a stripe reparent
operation.
This template is used for {@link StripeVisualizationType#DRAG_SOURCE}
@see DefaultStripeInputVisualizationHelper#getVisualCreator
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23dragSourceStripeTemplate Online Documentation}*/
dragSourceStripeTemplate: IVisualTemplate | null;
/**
A {@link string} that can be used to store a {@link IVisualTemplate} that can be used to create the visual that will be used to
draw the highlight for the current target of a stripe reparent operation.
This template is used for {@link StripeVisualizationType#DROP_TARGET}
The default template has the <code>yfiles-highlight-template</code> and <code>yfiles-stripe-highlight-template</code> CSS classes.
@see DefaultStripeInputVisualizationHelper#getVisualCreator
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23DROP_TARGET_STRIPE_TEMPLATE_KEY Online Documentation}*/
static DROP_TARGET_STRIPE_TEMPLATE_KEY: string;
/**
Gets or sets the {@link IVisualTemplate} that is used to visualize the current target of a stripe reparent
operation.
This template is used for {@link StripeVisualizationType#DROP_TARGET}
@see DefaultStripeInputVisualizationHelper#getVisualCreator
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultStripeInputVisualizationHelper%23dropTargetStripeTemplate Online Documentation}*/
dropTargetStripeTemplate: IVisualTemplate | null;
static $class: Class;
private $_DefaultStripeInputVisualizationHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultStripeInputVisualizationHelper;
}
/**
Enumeration constants for the various visualization types used in conjunction with
{@link IStripeInputVisualizationHelper}
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeVisualizationType Online Documentation}*/
export enum StripeVisualizationType {
/**
A visualization for a selected stripe or stripe label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeVisualizationType%23SELECTION Online Documentation}*/
SELECTION,
/**
A visualization for the source stripe of a reparent operation.
@see ReparentStripeInputMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeVisualizationType%23DRAG_SOURCE Online Documentation}*/
DRAG_SOURCE,
/**
A visualization for the target stripe of a reparent or drag and drop operation.
@see ReparentStripeInputMode
@see StripeDropInputMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeVisualizationType%23DROP_TARGET Online Documentation}*/
DROP_TARGET,
/**
A visualization for resize operations that are triggered for example by {@link ResizeStripeInputMode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeVisualizationType%23RESIZE Online Documentation}*/
RESIZE
}
/** Automatic conversion to {@link StripeVisualizationType}. */
type StripeVisualizationTypeStringValues = "selection" | "drag-source" | "drop-target" | "resize";
export interface IStripeInputVisualizationHelper {
}
/**
This interface allows to change the visualization for different input gestures.
A stripe should provide an implementation of this interface in its lookup which is queried by the various table related
input modes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeInputVisualizationHelper Online Documentation}*/
export abstract class IStripeInputVisualizationHelper {
/**
Creates an implementation of the interface {@link IStripeInputVisualizationHelper} by using the given function as
implementation for its {@link IStripeInputVisualizationHelper#getVisualCreator} method.
@param getVisualCreator A function for {@link IStripeInputVisualizationHelper}'s single abstract method
{@link IStripeInputVisualizationHelper#getVisualCreator}.
@returns An instance of the {@link IStripeInputVisualizationHelper} interface based on the given function.*/
static create(getVisualCreator: (context: IInputModeContext, tableNode: INode, type: StripeVisualizationType) => IVisualCreator): IStripeInputVisualizationHelper;
/**
Return an {@link IVisualCreator} that is used to visualize the input operation specified by <code>type</code>.
@param context The input mode context which provides further information.
@param tableNode The node where the stripe's owner is currently bound to.
@param type The actual visualization type
@returns A {@link IVisualCreator} instance that renders a representation for the current operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeInputVisualizationHelper%23getVisualCreator(IInputModeContext%2CINode%2CStripeVisualizationType) Online Documentation}*/
abstract getVisualCreator(context: IInputModeContext, tableNode: INode, type: StripeVisualizationType | StripeVisualizationTypeStringValues): IVisualCreator;
static $class: Class;
private $_IStripeInputVisualizationHelper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IStripeInputVisualizationHelper;
}
export interface ITable extends ILookup {
}
/**
Central interface which provides support for tabular data presentation.
This interface supports a presentation of data where the nodes of a diagram are organized in a tabular way, i.e., where
each node is associated to a specific row and column in the grid-like structure of a table. The {@link ITable}
together with its associated interfaces {@link IColumn} and {@link IRow} represents the model for the tabular
presentation (or table, for short). It also provides means for altering the model, in particular creation ({@link ITable#createChildColumn}
and {@link ITable#createChildRow}) and {@link ITable#remove removal} of columns and rows but also changing their size or appearance.
The table structure is backed by a group node. As a consequence, the nodes which should be presented in a tabular
fashion need to be set up as children of this group node. Using a group node to hold the content nodes has several
advantages: For example, when the group node is moved, the child nodes move accordingly, thus maintaining the visual
clue that they are contained in the table group node.
Note that the table structure and its content nodes are only loosely coupled: the node's center coordinates determine
the row and column that it belongs to.
Associating an {@link ITable} instance with its backing group node can most conveniently be done by setting a {@link TableNodeStyle}
instance as style of the associated group node. This style manages the visualization of the individual rows and columns.
It also associates its {@link TableNodeStyle#table} with the group node by decorating its {@link ILookup#lookup} to return the table instance.
User interaction is supported by the {@link TableEditorInputMode} which provides support for interactively resizing
rows and columns, re-parenting them, or editing their labels.
The table model with all relevant types and their relationships is presented in detail in the section {@link https://docs.yworks.com/yfileshtml/#/dguide/tables Tables}.
@see IStripe
@see IColumn
@see IRow
@see Table
@see TableEditorInputMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable Online Documentation}*/
export abstract class ITable {
/**
Add a label to the given owner using the text as the initial label text and label model parameter and style.
@param owner the owner to add the label to.
@param text the initial text of the label
@param [layoutParameter=null] The label model parameter instance to use.
@param [style=null] The style to use for the label
@param [preferredSize=null] The initial values to use for the {@link ILabel#preferredSize}.
@param [tag=null] the initial {@link ITagOwner#tag} to assign.
@returns the newly created label
@see ITable#setLabelText
@see ITable#setLabelLayoutParameter
@see ITable#setStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23addLabel(IStripe%2Cstring%2CILabelModelParameter%2CILabelStyle%2CSize%2CObject) Online Documentation}*/
abstract addLabel(owner: IStripe, text: string, layoutParameter?: ILabelModelParameter | null, style?: ILabelStyle | null, preferredSize?: Size | null | SizeConvertible, tag?: any | null): ILabel;
/**
Add a label to the given owner using the text as the initial label text and label model parameter and style.
@param options The parameters to pass.
@param options.owner the owner to add the label to.
@param options.text the initial text of the label
@param [options.layoutParameter=null] The label model parameter instance to use.
@param [options.style=null] The style to use for the label
@param [options.preferredSize=null] The initial values to use for the {@link ILabel#preferredSize}.
@param [options.tag=null] the initial {@link ITagOwner#tag} to assign.
@returns the newly created label
@see ITable#setLabelText
@see ITable#setLabelLayoutParameter
@see ITable#setStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23addLabel(IStripe%2Cstring%2CILabelModelParameter%2CILabelStyle%2CSize%2CObject) Online Documentation}*/
abstract addLabel(options: {
owner: IStripe;
text: string;
layoutParameter?: ILabelModelParameter | null;
style?: ILabelStyle | null;
preferredSize?: Size | null | SizeConvertible;
tag?: any | null;
}): ILabel;
/**
Adjusts the {@link ILabel#preferredSize} property of a label to fit the suggested size of its {@link ILabelStyleRenderer}.
This implementation uses the {@link ILabel#style style's} renderer for the label to determine the preferred rendering size. This is useful
after the label's content or style have been changed.
@param label The label to adjust the size for.
@see ILabelDefaults#autoAdjustPreferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23adjustLabelPreferredSize(ILabel) Online Documentation}*/
adjustLabelPreferredSize(label: ILabel): void;
/**
Calculates the preferred size of a label with the given properties.
@param stripe The item that will own the label.
@param style The label style.
@param layoutParameter The label model parameter.
@param text The text.
@param tag The tag for the label.
@returns The size as calculated by the {@link ILabelStyleRenderer}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23calculateLabelPreferredSize(IStripe%2CILabelStyle%2CILabelModelParameter%2Cstring%2CObject) Online Documentation}*/
calculateLabelPreferredSize(stripe: IStripe, style: ILabelStyle, layoutParameter: ILabelModelParameter, text: string, tag: any | null): Size;
/**
Removes all stripes from a table.
All existing rows and columns are cleared from <code>table</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23clear() Online Documentation}*/
clear(): void;
/**
Create a new column as the <code>index</code>th child of <code>owner</code> with the given parameters.
@param owner The owner of the new column.
@param [index=null] The position in the child list.
@param [width=null] The actual width of the column.
@param [minWidth=null] The minimal width of the column.
@param [insets=null] The insets of the column.
@param [style=null] The style of the column.
@param [tag=null] The tag of the column
@returns A new column instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createChildColumn(IColumn%2Cnumber%2Cnumber%2CInsets%2CIStripeStyle%2CObject%2Cnumber) Online Documentation}*/
abstract createChildColumn(owner: IColumn | null, width?: number | null, minWidth?: number | null, insets?: Insets | null | InsetsConvertible, style?: IStripeStyle | null, tag?: any | null, index?: number | null): IColumn;
/**
Create a new column as the <code>index</code>th child of <code>owner</code> with the given parameters.
@param options The parameters to pass.
@param options.owner The owner of the new column.
@param [options.index=null] The position in the child list.
@param [options.width=null] The actual width of the column.
@param [options.minWidth=null] The minimal width of the column.
@param [options.insets=null] The insets of the column.
@param [options.style=null] The style of the column.
@param [options.tag=null] The tag of the column
@param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link ITable#addLabel}
method.
@param options.childColumns An array of rows to create as children of this row. Each item will be passed to the {@link ITable#createChildColumn}
method.
@returns A new column instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createChildColumn(IColumn%2Cnumber%2Cnumber%2CInsets%2CIStripeStyle%2CObject%2Cnumber) Online Documentation}*/
abstract createChildColumn(options: {
owner: IColumn | null;
width?: number | null;
minWidth?: number | null;
insets?: Insets | null | InsetsConvertible;
style?: IStripeStyle | null;
tag?: any | null;
index?: number | null;
labels?: Array<any>;
childColumns?: Array<any>;
}): IColumn;
/**
Create a new row as the <code>index</code>th child of <code>owner</code> with the given parameters.
@param owner The owner of the new row.
@param [index=null] The position in the child list.
@param [height=null] The actual height of the row.
@param [minHeight=null] The minimal height of the row.
@param [insets=null] The insets of the row.
@param [style=null] The style of the row.
@param [tag=null] The tag of the row
@returns A new row instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createChildRow(IRow%2Cnumber%2Cnumber%2CInsets%2CIStripeStyle%2CObject%2Cnumber) Online Documentation}*/
abstract createChildRow(owner: IRow | null, height?: number | null, minHeight?: number | null, insets?: Insets | null | InsetsConvertible, style?: IStripeStyle | null, tag?: any | null, index?: number | null): IRow;
/**
Create a new row as the <code>index</code>th child of <code>owner</code> with the given parameters.
@param options The parameters to pass.
@param options.owner The owner of the new row.
@param [options.index=null] The position in the child list.
@param [options.height=null] The actual height of the row.
@param [options.minHeight=null] The minimal height of the row.
@param [options.insets=null] The insets of the row.
@param [options.style=null] The style of the row.
@param [options.tag=null] The tag of the row
@param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link ITable#addLabel}
method.
@param options.childRows An array of rows to create as children of this row. Each item will be passed to the {@link ITable#createChildRow}
method.
@returns A new row instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createChildRow(IRow%2Cnumber%2Cnumber%2CInsets%2CIStripeStyle%2CObject%2Cnumber) Online Documentation}*/
abstract createChildRow(options: {
owner: IRow | null;
height?: number | null;
minHeight?: number | null;
insets?: Insets | null | InsetsConvertible;
style?: IStripeStyle | null;
tag?: any | null;
index?: number | null;
labels?: Array<any>;
childRows?: Array<any>;
}): IRow;
/**
Creates and returns a column as last child of <code>table</code>.
The column will be a part of this table after the method returns. This will trigger the corresponding events.
@param [width=null] The actual width of the column.
@param [minWidth=null] The minimal width of the column.
@param [insets=null] The insets of the column.
@param [style=null] The style of the column.
@param [tag=null] The tag of the column
@param [index=null] The position in the child list.
@returns A newly created column instance
@see ITable#addStripeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createColumn(number%2Cnumber%2CInsets%2CIStripeStyle%2CObject%2Cnumber) Online Documentation}*/
createColumn(width?: number | null, minWidth?: number | null, insets?: Insets | null | InsetsConvertible, style?: IStripeStyle | null, tag?: any | null, index?: number | null): IColumn;
/**
Creates and returns a column as last child of <code>table</code>.
The column will be a part of this table after the method returns. This will trigger the corresponding events.
@param options The parameters to pass.
@param [options.width=null] The actual width of the column.
@param [options.minWidth=null] The minimal width of the column.
@param [options.insets=null] The insets of the column.
@param [options.style=null] The style of the column.
@param [options.tag=null] The tag of the column
@param [options.index=null] The position in the child list.
@returns A newly created column instance
@see ITable#addStripeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createColumn(number%2Cnumber%2CInsets%2CIStripeStyle%2CObject%2Cnumber) Online Documentation}*/
createColumn(options: {
width?: number | null;
minWidth?: number | null;
insets?: Insets | null | InsetsConvertible;
style?: IStripeStyle | null;
tag?: any | null;
index?: number | null;
}): IColumn;
/**
Creates a <code>columns</code> x <code>rows</code>.
All existing rows and columns are cleared from <code>table</code>
@param columns The number of columns to test.
@param rows The number of rows to test.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createGrid(number%2Cnumber) Online Documentation}*/
createGrid(columns: number, rows: number): void;
/**
Creates the label model parameter for a given {@link ILabelOwner}.
This implementation uses the label defaults for the <code>table</code> to {@link ILabelDefaults#getLayoutParameterInstance obtain the parameter instance}.
@param stripe The item that is the {@link ILabel#owner} of the label in question.
@returns The default label model parameter to use for newly created labels at the item.
@see ILabelDefaults#getLayoutParameterInstance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createLabelLayoutParameter(IStripe) Online Documentation}*/
createLabelLayoutParameter(stripe: IStripe): ILabelModelParameter;
/**
Creates an instance of the {@link ILabelDefaults#style default label style} for the given <code>stripe</code>.
Delegates to the {@link ILabelDefaults#getStyleInstance} method for the given {@link IStripe stripe}.
@param stripe The item the newly created label will belong to.
@returns The style instance to use for newly created stripes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createLabelStyle(IStripe) Online Documentation}*/
createLabelStyle(stripe: IStripe): ILabelStyle;
/**
Creates and returns a row as last child of <code>table</code>.
The row will be a part of this table after the method returns. This will trigger the corresponding events.
@param [height=null] The actual height of the row.
@param [minHeight=null] The minimal height of the row.
@param [insets=null] The insets of the row.
@param [style=null] The style of the row.
@param [tag=null] The tag of the row
@param [index=null] The position in the child list.
@returns A newly created row instance
@see ITable#addStripeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createRow(number%2Cnumber%2CInsets%2CIStripeStyle%2CObject%2Cnumber) Online Documentation}*/
createRow(height?: number | null, minHeight?: number | null, insets?: Insets | null | InsetsConvertible, style?: IStripeStyle | null, tag?: any | null, index?: number | null): IRow;
/**
Creates and returns a row as last child of <code>table</code>.
The row will be a part of this table after the method returns. This will trigger the corresponding events.
@param options The parameters to pass.
@param [options.height=null] The actual height of the row.
@param [options.minHeight=null] The minimal height of the row.
@param [options.insets=null] The insets of the row.
@param [options.style=null] The style of the row.
@param [options.tag=null] The tag of the row
@param [options.index=null] The position in the child list.
@returns A newly created row instance
@see ITable#addStripeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23createRow(number%2Cnumber%2CInsets%2CIStripeStyle%2CObject%2Cnumber) Online Documentation}*/
createRow(options: {
height?: number | null;
minHeight?: number | null;
insets?: Insets | null | InsetsConvertible;
style?: IStripeStyle | null;
tag?: any | null;
index?: number | null;
}): IRow;
/**
Finds a column underneath a certain point.
The column are tested in bottom up order.
@param node The node this table is currently bound to
@param location The location to test.
@param [filter=null] Additional predicate to further restrict the results.
@returns The topmost column that has been found for the location or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23findColumn(INode%2CPoint%2Cfunction(IStripe)%3Aboolean) Online Documentation}*/
findColumn(node: INode, location: Point | PointConvertible, filter?: ((obj: IStripe) => boolean) | null): IColumn | null;
/**
Finds a row underneath a certain point.
The rows are tested in bottom up order.
@param node The node this table is currently bound to
@param location The location to test.
@param [filter=null] Additional predicate to further restrict the results.
@returns The topmost row that has been found for the location or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23findRow(INode%2CPoint%2Cfunction(IStripe)%3Aboolean) Online Documentation}*/
findRow(node: INode, location: Point | PointConvertible, filter?: ((obj: IStripe) => boolean) | null): IRow | null;
/**
Finds a stripe underneath a certain point.
@param location The location to test.
@param stripeTypes An enumeration value of {@link StripeTypes} to specify the stripe type.
@param subregionTypes An enumeration value of {@link StripeSubregionTypes} to further restrict the stripe region.
@param canvas The canvas that provides necessary context for hit determination, such as {@link ICanvasContext#hitTestRadius}.
@param [filter=null] Additional predicate to further restrict the hit test results.
@returns The stripe subregions that have been found for the location or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23findStripe(Point%2CStripeTypes%2CStripeSubregionTypes%2CCanvasComponent%2Cfunction(StripeSubregion)%3Aboolean) Online Documentation}*/
static findStripe(location: Point | PointConvertible, stripeTypes: StripeTypes | StripeTypesConvertible, subregionTypes: StripeSubregionTypes | StripeSubregionTypesConvertible, canvas: CanvasComponent, filter?: ((obj: StripeSubregion) => boolean) | null): StripeSubregion | null;
/**
Finds all stripes underneath a certain point.
The stripes are always returned in bottom up, rows first order. Note that this method does not take any {@link ICanvasContext#hitTestRadius} into account.
@param node The node this table is currently bound to
@param location The location to test.
@param stripeTypes An enumeration value of {@link StripeTypes} to specify the stripe type.
@param [filter=null] Additional predicate to further restrict the hit test results.
@returns The stripes that have been found for the location or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23findStripes(INode%2CPoint%2CStripeTypes%2Cfunction(IStripe)%3Aboolean) Online Documentation}*/
findStripes(node: INode, location: Point | PointConvertible, stripeTypes: StripeTypes | StripeTypesConvertible, filter?: ((obj: IStripe) => boolean) | null): IEnumerable<IStripe>;
/**
Used as a callback to find the items underneath a certain point.
@param location The location to test.
@param stripeTypes An enumeration value of {@link StripeTypes} to specify the stripe type.
@param subregionTypes An enumeration value of {@link StripeSubregionTypes} to further restrict the stripe region.
@param canvas The canvas that provides necessary context for hit determination, such as {@link ICanvasContext#hitTestRadius}.
@param [filter=null] Additional predicate to further restrict the hit test results.
@returns The stripe subregions that have been found for the location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23findStripes(Point%2CStripeTypes%2CStripeSubregionTypes%2CCanvasComponent%2Cfunction(StripeSubregion)%3Aboolean) Online Documentation}*/
static findStripes(location: Point | PointConvertible, stripeTypes: StripeTypes | StripeTypesConvertible, subregionTypes: StripeSubregionTypes | StripeSubregionTypesConvertible, canvas: CanvasComponent, filter?: ((obj: StripeSubregion) => boolean) | null): IEnumerable<StripeSubregion>;
/**
Calculates the relative bounds of a table cell that is determined by <code>column</code> and <code>row</code>
Both <code>column</code> and <code>row</code> must live in the provided <code>table</code> instance. Otherwise, an {@link Exception ArgumentError} is thrown.
@param column The column that determines the cell
@param row The row that determines the cell
@returns The relative bounds of a the cell determined by <code>column</code> and <code>row</code>
@throws {Exception} <b>(ArgumentError)</b> If <code>column</code> and <code>row</code> are part of a different table instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23getCellBounds(IColumn%2CIRow) Online Documentation}*/
getCellBounds(column: IColumn, row: IRow): Rect;
/**
Calculates the absolute bounds of a table cell that is determined by <code>column</code> and <code>row</code>
Both <code>column</code> and <code>row</code> must live in the same {@link ITable} instance that is also present in the <code>tableNode</code>s
{@link ILookup}. Otherwise, an {@link Exception ArgumentError} is thrown.
@param tableNode The node where a table is bound to via the node's lookup
@param column The column that determines the cell
@param row The row that determines the cell
@returns The absolute bounds of a the cell determined by <code>column</code> and <code>row</code>
@throws {Exception} <b>(ArgumentError)</b> If no table is present in the {@link ILookup} of <code>tableNode</code>, or if <code>column</code> and <code>row</code> are part of a different table
instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23getCellBounds(INode%2CIColumn%2CIRow) Online Documentation}*/
static getCellBounds(tableNode: INode, column: IColumn, row: IRow): Rect;
/**
Places <code>node</code> in the center of the table cell determined by <code>column</code> and <code>row</code>.
Both <code>column</code> and <code>row</code> must live in the same {@link ITable} instance that is also present in the <code>tableNode</code>s
{@link ILookup}. Otherwise, an {@link Exception ArgumentError} is thrown. The cell bounds are determined by
{@link ITable#getCellBounds}.
If <code>reparent</code> is <code>true</code>, the node is made a child node of <code>tableNode</code> if it is no descendant of the table node hierarchy yet.
Otherwise, the grouping hierarchy is not changed.
If <code>alwaysMove</code> is
<code>true</code>, the node is moved even if its center is already contained in the cell, otherwise only when the node layout center
is currently outside of the cell bounds.
This method does not change the size of the target stripes if they would be too small to contain the whole node layout.
Callers must increase the stripe size before or after calling this method, if necessary.
@param graph The graph instance where the move should be performed
@param node The node that should be moved into the given cell.
@param tableNode The node where a table is bound to via the node's lookup
@param column The column that determines the cell
@param row The row that determines the cell
@param [reparent=true] Whether <code>node</code> should also be made part of <code>tableNode</code>'s grouping hierarchy if it is not yet contained in it.
@param [alwaysMove=false] Whether <code>node</code> should be moved even if it is already placed in the cell.
@throws {Exception} <b>(ArgumentError)</b> If no table is present in the {@link ILookup} of <code>tableNode</code>, or if <code>column</code> and <code>row</code> are part of a different table
instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23placeNodeInCell(IGraph%2CINode%2CINode%2CIColumn%2CIRow%2Cboolean%2Cboolean) Online Documentation}*/
static placeNodeInCell(graph: IGraph, node: INode, tableNode: INode, column: IColumn, row: IRow, reparent?: boolean, alwaysMove?: boolean): void;
/**
Places <code>node</code> in the center of the table cell determined by <code>column</code> and <code>row</code>.
Both <code>column</code> and <code>row</code> must live in the same {@link ITable} instance that is also present in the <code>tableNode</code>s
{@link ILookup}. Otherwise, an {@link Exception ArgumentError} is thrown. The cell bounds are determined by
{@link ITable#getCellBounds}.
If <code>reparent</code> is <code>true</code>, the node is made a child node of <code>tableNode</code> if it is no descendant of the table node hierarchy yet.
Otherwise, the grouping hierarchy is not changed.
If <code>alwaysMove</code> is
<code>true</code>, the node is moved even if its center is already contained in the cell, otherwise only when the node layout center
is currently outside of the cell bounds.
This method does not change the size of the target stripes if they would be too small to contain the whole node layout.
Callers must increase the stripe size before or after calling this method, if necessary.
@param options The parameters to pass.
@param options.graph The graph instance where the move should be performed
@param options.node The node that should be moved into the given cell.
@param options.tableNode The node where a table is bound to via the node's lookup
@param options.column The column that determines the cell
@param options.row The row that determines the cell
@param [options.reparent=true] Whether <code>node</code> should also be made part of <code>tableNode</code>'s grouping hierarchy if it is not yet contained in it.
@param [options.alwaysMove=false] Whether <code>node</code> should be moved even if it is already placed in the cell.
@throws {Exception} <b>(ArgumentError)</b> If no table is present in the {@link ILookup} of <code>tableNode</code>, or if <code>column</code> and <code>row</code> are part of a different table
instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23placeNodeInCell(IGraph%2CINode%2CINode%2CIColumn%2CIRow%2Cboolean%2Cboolean) Online Documentation}*/
static placeNodeInCell(options: {
graph: IGraph;
node: INode;
tableNode: INode;
column: IColumn;
row: IRow;
reparent?: boolean;
alwaysMove?: boolean;
}): void;
/**
Removes <code>stripe</code> from its parent container.
This method reparents all children of <code>stripe</code> to the parent of the stripe.
@param stripe The stripe to remove
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23remove(IStripe) Online Documentation}*/
abstract remove(stripe: IStripe): void;
/**
Removes the given label from its owner.
This will trigger the corresponding event.
@param label the label to remove
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23remove(ILabel) Online Documentation}*/
abstract remove(label: ILabel): void;
/**
{@link ITable#remove Removes} <code>stripe</code> and all of its descendants from <code>table</code>.
This method does not resize any stripes, use {@link ITable#removeRecursivelyWithResize} instead.
@param stripe The stripe to remove
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23removeRecursively(IStripe) Online Documentation}*/
removeRecursively(stripe: IStripe): void;
/**
{@link ITable#removeWithResize Removes} the given <code>stripe</code> and all of its descendants from <code>table</code> and resizes all affected stripes so that the table size
does not change if possible.
@param stripe The stripe to remove
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23removeRecursivelyWithResize(IStripe) Online Documentation}*/
removeRecursivelyWithResize(stripe: IStripe): void;
/**
{@link ITable#remove Removes} <code>stripe</code> from <code>table</code> and resizes all affected stripes so that the table size does not change if possible.
@param stripe The stripe to remove
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23removeWithResize(IStripe) Online Documentation}*/
removeWithResize(stripe: IStripe): void;
/**
Sets the label model parameter for the given label.
@param label The label.
@param layoutParameter The new parameter.
@throws {Exception} <b>(ArgumentError)</b> If the parameter cannot be used for this label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setLabelLayoutParameter(ILabel%2CILabelModelParameter) Online Documentation}*/
abstract setLabelLayoutParameter(label: ILabel, layoutParameter: ILabelModelParameter): void;
/**
Sets the preferred size of the label.
@param label The label.
@param preferredSize The new preferred size.
@see ILabel#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setLabelPreferredSize(ILabel%2CSize) Online Documentation}*/
abstract setLabelPreferredSize(label: ILabel, preferredSize: Size): void;
/**
Sets the label text of the given label.
@param label the label to modify
@param text the new text of the label
@see ILabel#text
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setLabelText(ILabel%2Cstring) Online Documentation}*/
abstract setLabelText(label: ILabel, text: string): void;
/**
Sets the minimum size of <code>stripe</code>.
This method sets the {@link IStripe#minimumSize} of <code>stripe</code>.
@param stripe The stripe to change
@param minimumSize The new minimum size of the stripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setMinimumSize(IStripe%2Cnumber) Online Documentation}*/
abstract setMinimumSize(stripe: IStripe, minimumSize: number): void;
/**
Sets <code>owner</code> as new parent of <code>column</code>, inserting <code>column</code> at the given <code>index</code>.
The column will be inserted at position <code>index</code> in the child list of the new parent.
@param owner
@param column
@param [index=null]
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setParent(IColumn%2CIColumn%2Cnumber) Online Documentation}*/
abstract setParent(owner: IColumn, column: IColumn, index?: number | null): void;
/**
Sets <code>owner</code> as new parent of <code>row</code>, inserting <code>row</code> at the given <code>index</code>.
The row will be inserted at position <code>index</code> in the child list of the new parent.
@param owner
@param row
@param [index=null]
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setParent(IRow%2CIRow%2Cnumber) Online Documentation}*/
abstract setParent(owner: IRow, row: IRow, index?: number | null): void;
/**
Sets the actual size of <code>stripe</code>.
This method sets the {@link IStripe#size} of <code>stripe</code>.
@param stripe The stripe to change
@param size The new size of the stripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setSize(IStripe%2Cnumber) Online Documentation}*/
abstract setSize(stripe: IStripe, size: number): void;
/**
Sets the insets of <code>stripe</code>.
@param stripe The stripe to change
@param insets The new insets of the stripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setStripeInsets(IStripe%2CInsets) Online Documentation}*/
abstract setStripeInsets(stripe: IStripe, insets: Insets | InsetsConvertible): void;
/**
Sets the style of <code>stripe</code>.
@param stripe The stripe to change
@param style The new style of the stripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setStyle(IStripe%2CIStripeStyle) Online Documentation}*/
abstract setStyle(stripe: IStripe, style: IStripeStyle): void;
/**
Assigns the given style instance by reference to the label.
Style instances can be shared.
@param label The label that will be assigned the new style
@param style The style instance that will be assigned to the label.
@see ILabel#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23setStyle(ILabel%2CILabelStyle) Online Documentation}*/
abstract setStyle(label: ILabel, style: ILabelStyle): void;
/**
Gets or sets the insets for this table.
These insets are applied in addition to any implicit insets provided by the child stripes.
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23insets Online Documentation}*/
abstract insets: Insets;
/**
Gets the layout of the table, relative to an owner node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23layout Online Documentation}*/
abstract readonly layout: IRectangle;
/**
Gets or sets the defaults for rows.
The settings that are obtained from the instance influence newly created elements only. Setting different defaults later
does not influence existing elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23rowDefaults Online Documentation}*/
abstract rowDefaults: IStripeDefaults;
/**
Gets or sets the defaults for Columns.
The settings that are obtained from the instance influence newly created elements only. Setting different defaults later
does not influence existing elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23columnDefaults Online Documentation}*/
abstract columnDefaults: IStripeDefaults;
/**
Gets the virtual root of the row hierarchy
This row should not be queried except for the {@link IRow#parentRow} and the associated {@link IStripe#table}
@see ITable#setParent
@see ITable#createChildRow
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23rootRow Online Documentation}*/
abstract readonly rootRow: IRow;
/**
Gets the virtual root of the column hierarchy
This column should not be queried except for the {@link IColumn#parentColumn} and the associated {@link IStripe#table}
@see ITable#setParent
@see ITable#createChildColumn
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23rootColumn Online Documentation}*/
abstract readonly rootColumn: IColumn;
/**
Adds the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to this graph instance.
@param listener The listener to add.
@see ITable#addLabel
@see ITable#removeLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23LabelAdded Online Documentation}*/
abstract addLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Removes the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to this graph instance.
@param listener The listener to remove.
@see ITable#addLabel
@see ITable#addLabelAddedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23LabelAdded Online Documentation}*/
abstract removeLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Adds the given listener for the <code>LabelRemoved</code> event that occurs when a label has been removed from this graph instance.
This event will also be triggered, prior to the removal of the {@link ILabel#owner owner} of the label.
Implementations may choose to use the {@link LabelEventArgs} to carry additional label owner information. The {@link LabelEventArgs#owner}
property will be set to the the owner of the label that owned it before the removal.
@param listener The listener to add.
@see ITable#remove
@see ITable#removeLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23LabelRemoved Online Documentation}*/
abstract addLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Removes the given listener for the <code>LabelRemoved</code> event that occurs when a label has been removed from this graph
instance.
This event will also be triggered, prior to the removal of the {@link ILabel#owner owner} of the label.
Implementations may choose to use the {@link LabelEventArgs} to carry additional label owner information. The {@link LabelEventArgs#owner}
property will be set to the the owner of the label that owned it before the removal.
@param listener The listener to remove.
@see ITable#remove
@see ITable#addLabelRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23LabelRemoved Online Documentation}*/
abstract removeLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
Adds the given listener for the <code>LabelChanged</code> event that occurs when a label has been changed, for instance, if its style
has been replaced.
@param listener The listener to add.
@see ITable#setStyle
@see ITable#removeLabelChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23LabelChanged Online Documentation}*/
abstract addLabelChangedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Removes the given listener for the <code>LabelChanged</code> event that occurs when a label has been changed, for instance, if its
style has been replaced.
@param listener The listener to remove.
@see ITable#setStyle
@see ITable#addLabelChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23LabelChanged Online Documentation}*/
abstract removeLabelChangedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
Adds the given listener for the <code>StripeCreated</code> event that occurs when a stripe has been added.
@param listener The listener to add.
@see ITable#createChildRow
@see ITable#createChildColumn
@see ITable#removeStripeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23StripeCreated Online Documentation}*/
abstract addStripeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IStripe>) => void): void;
/**
Removes the given listener for the <code>StripeCreated</code> event that occurs when a stripe has been added.
@param listener The listener to remove.
@see ITable#createChildRow
@see ITable#createChildColumn
@see ITable#addStripeCreatedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23StripeCreated Online Documentation}*/
abstract removeStripeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IStripe>) => void): void;
/**
Adds the given listener for the <code>StripeRemoved</code> event that occurs when a stripe has been removed from its parent.
@param listener The listener to add.
@see ITable#remove
@see ITable#removeStripeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23StripeRemoved Online Documentation}*/
abstract addStripeRemovedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
/**
Removes the given listener for the <code>StripeRemoved</code> event that occurs when a stripe has been removed from its parent.
@param listener The listener to remove.
@see ITable#remove
@see ITable#addStripeRemovedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23StripeRemoved Online Documentation}*/
abstract removeStripeRemovedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
/**
Adds the given listener for the <code>StripeChanged</code> event that occurs when a stripe has been changed, for instance, if its
style has been replaced.
@param listener The listener to add.
@see ITable#setStyle
@see ITable#setStripeInsets
@see ITable#setSize
@see ITable#setMinimumSize
@see ITable#removeStripeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23StripeChanged Online Documentation}*/
abstract addStripeChangedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
/**
Removes the given listener for the <code>StripeChanged</code> event that occurs when a stripe has been changed, for instance, if its
style has been replaced.
@param listener The listener to remove.
@see ITable#setStyle
@see ITable#setStripeInsets
@see ITable#setSize
@see ITable#setMinimumSize
@see ITable#addStripeChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23StripeChanged Online Documentation}*/
abstract removeStripeChangedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
/**
Return the accumulated insets for the table.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23accumulatedInsets Online Documentation}*/
readonly accumulatedInsets: Insets;
/**
Gets a {@link TableDecorator} instance for use with this table.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITable%23decorator Online Documentation}*/
readonly decorator: TableDecorator;
static $class: Class;
private $_ITable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ITable;
}
export interface IRow extends IStripe {
}
/**
Interface that describes a row in a {@link ITable} instance.
Default implementations of this interface are created by the factory methods in the {@link ITable} interface and
its implementations, therefore, it usually shouldn't be necessary to implement this interface yourself.
@see ITable#createChildRow
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRow Online Documentation}*/
export abstract class IRow {
/**
Gets the parent of this row.
In a table, each row has a parent, except for the {@link ITable#rootRow} hierarchy root.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRow%23parentRow Online Documentation}*/
abstract readonly parentRow: IRow | null;
/**
Gets the child rows.
This collection may be empty. Each entry in this collection has this instance as {@link IRow#parentRow}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IRow%23childRows Online Documentation}*/
abstract readonly childRows: IEnumerable<IRow>;
static $class: Class;
private $_IRow: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IRow;
}
export interface IColumn extends IStripe {
}
/**
Interface that describes a column in a {@link ITable} instance.
Default implementations of this interface are created by the factory methods in the {@link ITable} interface and
its implementations, therefore, it usually shouldn't be necessary to implement this interface yourself.
@see ITable#createChildColumn
@see {@link https://docs.yworks.com/yfileshtml/#/api/IColumn Online Documentation}*/
export abstract class IColumn {
/**
Gets the parent of this column.
In a table, each column has a parent, except for the {@link ITable#rootColumn} hierarchy root.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IColumn%23parentColumn Online Documentation}*/
abstract readonly parentColumn: IColumn | null;
/**
Gets the child columns.
This collection may be empty. Each entry in this collection has this instance as {@link IColumn#parentColumn}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IColumn%23childColumns Online Documentation}*/
abstract readonly childColumns: IEnumerable<IColumn>;
static $class: Class;
private $_IColumn: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IColumn;
}
export interface NodeLabelModelStripeLabelModelAdapter extends YObject, ILabelModel {
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
Allows parameters from {@link ILabelModel}s for nodes to be used for {@link IStripe}s in a
{@link ITable}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelModelStripeLabelModelAdapter Online Documentation}*/
export class NodeLabelModelStripeLabelModelAdapter {
/**
Creates a parameter that delegates to the given <code>parameter</code> of a {@link ILabelModel} for nodes.
@param parameter The parameter of a {@link ILabelModel} that supports nodes.
@returns A parameter to position the stripe label as if the stripe were a node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelModelStripeLabelModelAdapter%23createParameter(ILabelModelParameter) Online Documentation}*/
createParameter(parameter: ILabelModelParameter): ILabelModelParameter;
/**
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelModelStripeLabelModelAdapter%23INSTANCE Online Documentation}*/
static INSTANCE: NodeLabelModelStripeLabelModelAdapter;
static $class: Class;
private $_NodeLabelModelStripeLabelModelAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeLabelModelStripeLabelModelAdapter;
}
/**
Enumeration of the possible positions of the {@link InteriorLabelModel}.
This field can be used as an argument for the {@link InteriorLabelModel#createParameter} factory method.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModelPosition Online Documentation}*/
export enum StretchStripeLabelModelPosition {
/**
Encodes a position at the top side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModelPosition%23NORTH Online Documentation}*/
NORTH,
/**
Encodes a position at the right side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModelPosition%23EAST Online Documentation}*/
EAST,
/**
Encodes a position at the bottom side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModelPosition%23SOUTH Online Documentation}*/
SOUTH,
/**
Encodes a position at the left side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModelPosition%23WEST Online Documentation}*/
WEST
}
/** Automatic conversion to {@link StretchStripeLabelModelPosition}. */
type StretchStripeLabelModelPositionStringValues = "north" | "east" | "south" | "west";
export interface StretchStripeLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A label model for labels that is especially tailored to be used as a model for the four default positions where the
label of a stripe may lie.
This label model supports four positions inside of the stripe bounds. The east and west positions are rotated
automatically. The {@link ILabel#owner} is expected to be an {@link IStripe} instance, i.e. the parameter supports only {@link IStripe}
instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel Online Documentation}*/
export class StretchStripeLabelModel {
/**
Creates a new instance of this model with empty insets.
@param [options] The parameters to pass.
@param [options.insets] The insets to use within the stripe's {@link IStripe#layout}. This option sets the {@link StretchStripeLabelModel#insets} property on the created object.
@param [options.useActualInsets] Whether or not to use the {@link IStripe#actualInsets actual insets} or the normal {@link IStripe#insets} of a stripe for centering the label inside the stripe header area. This option sets the {@link StretchStripeLabelModel#useActualInsets} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel Online Documentation}*/
constructor(options?: {
insets?: Insets | InsetsConvertible;
useActualInsets?: boolean;
});
/**
Creates the parameter for the given position.
@param position The position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel%23createParameter(StretchStripeLabelModelPosition) Online Documentation}*/
createParameter(position: StretchStripeLabelModelPosition | StretchStripeLabelModelPositionStringValues): ILabelModelParameter;
/**
Calculates the geometry in form of an {@link IOrientedRectangle} for a given label using the given model
parameter.
The geometry is calculated in table coordinates.
@param label the label to calculate the geometry for
@param layoutParameter A parameter that has been created by this model. This is typically the parameter that yielded this instance through its
{@link ILabelModelParameter#model} property.
@returns An instance that describes the geometry. This is typically an instance designed as a flyweight, so clients should not
cache the instance but store the values if they need a snapshot for later use
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel%23getGeometry(ILabel%2CILabelModelParameter) Online Documentation}*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link StretchStripeLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel%23NORTH Online Documentation}*/
static NORTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link StretchStripeLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel%23SOUTH Online Documentation}*/
static SOUTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link StretchStripeLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel%23EAST Online Documentation}*/
static EAST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link StretchStripeLabelModel#insets} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel%23WEST Online Documentation}*/
static WEST: ILabelModelParameter;
/**
Gets or sets the insets to use within the stripe's {@link IStripe#layout}.
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel%23insets Online Documentation}*/
insets: Insets;
/**
Gets or sets whether or not to use the {@link IStripe#actualInsets actual insets} or the normal {@link IStripe#insets} of a stripe for centering the label inside the
stripe header area.
If <code>true</code> the header insets are used. Default is <code>false</code>
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/StretchStripeLabelModel%23useActualInsets Online Documentation}*/
useActualInsets: boolean;
static $class: Class;
private $_StretchStripeLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StretchStripeLabelModel;
}
export interface StripeEventArgs extends ItemEventArgs<IStripe> {
}
/**
A specialized subclass of the {@link ItemEventArgs} class that is bound to the {@link IStripe} type and carries additional {@link IStripe owner}
information.
Since for some events ({@link ITable#addStripeRemovedListener StripeRemoved}) it is unclear what {@link IStripe#parentStripe parent} the
stripe was owned by before the event, this event can be used to carry exactly that information.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeEventArgs Online Documentation}*/
export class StripeEventArgs {
/**
Initializes a new instance of the {@link StripeEventArgs} class.
@param stripe The stripe to assign to the {@link ItemEventArgs#item} property.
@param parentStripe The previous parent of the stripe.
@param table The table the stripe belonged to previously.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeEventArgs Online Documentation}*/
constructor(stripe: IStripe, parentStripe: IStripe, table: ITable | null);
/**
Gets the parent of the stripe that owned the stripe before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeEventArgs%23parentStripe Online Documentation}*/
readonly parentStripe: IStripe;
/**
Gets the table that owned the stripe before the event happened.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeEventArgs%23table Online Documentation}*/
readonly table: ITable | null;
static $class: Class;
private $_StripeEventArgs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StripeEventArgs;
}
/**
Enumeration of the possible positions of the {@link InteriorLabelModel}.
This field can be used as an argument for the {@link InteriorLabelModel#createParameter} factory method.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModelPosition Online Documentation}*/
export enum StripeLabelModelPosition {
/**
Encodes a position at the top side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModelPosition%23NORTH Online Documentation}*/
NORTH,
/**
Encodes a position at the right side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModelPosition%23EAST Online Documentation}*/
EAST,
/**
Encodes a position at the bottom side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModelPosition%23SOUTH Online Documentation}*/
SOUTH,
/**
Encodes a position at the left side of the node interior
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModelPosition%23WEST Online Documentation}*/
WEST
}
/** Automatic conversion to {@link StripeLabelModelPosition}. */
type StripeLabelModelPositionStringValues = "north" | "east" | "south" | "west";
export interface StripeLabelModel extends YObject, ILabelModel, ILabelModelParameterProvider {
/**
@inheritdoc*/
createDefaultParameter(): ILabelModelParameter;
/**
@inheritdoc*/
getContext(label: ILabel, layoutParameter: ILabelModelParameter): ILookup;
/**
@inheritdoc*/
getParameters(label: ILabel, model: ILabelModel): IEnumerable<ILabelModelParameter>;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
A label model for labels that is especially tailored to be used as a model for the four default positions where the
label of a stripe may lie.
This label model supports four positions inside of the stripe bounds. The east and west positions are rotated
automatically. The {@link ILabel#owner} is expected to be an {@link IStripe} instance, i.e. the parameter supports only {@link IStripe}
instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel Online Documentation}*/
export class StripeLabelModel {
/**
Creates a new instance of this model with empty insets.
@param [options] The parameters to pass.
@param [options.ratio] The ratio how far the label should be positioned from the border in the header area. This option sets the {@link StripeLabelModel#ratio} property on the created object.
@param [options.useActualInsets] A value indicating whether to use the {@link IStripe#actualInsets actual insets} or the normal {@link IStripe#insets} of a stripe for centering the label inside the stripe header area. This option sets the {@link StripeLabelModel#useActualInsets} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel Online Documentation}*/
constructor(options?: {
ratio?: number;
useActualInsets?: boolean;
});
/**
Creates the parameter for the given position.
@param position The position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel%23createParameter(StripeLabelModelPosition) Online Documentation}*/
createParameter(position: StripeLabelModelPosition | StripeLabelModelPositionStringValues): ILabelModelParameter;
/**
Calculates the geometry in form of an {@link IOrientedRectangle} for a given label using the given model
parameter.
The geometry is calculated in table coordinates, i.e. relative to the upper left corner of the {@link ITable#layout table}.
@param label the label to calculate the geometry for
@param layoutParameter A parameter that has been created by this model. This is typically the parameter that yielded this instance through its
{@link ILabelModelParameter#model} property.
@returns An instance that describes the geometry. This is typically an instance designed as a flyweight, so clients should not
cache the instance but store the values if they need a snapshot for later use
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel%23getGeometry(ILabel%2CILabelModelParameter) Online Documentation}*/
getGeometry(label: ILabel, layoutParameter: ILabelModelParameter): IOrientedRectangle;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link StripeLabelModel#ratio} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel%23NORTH Online Documentation}*/
static NORTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link StripeLabelModel#ratio} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel%23SOUTH Online Documentation}*/
static SOUTH: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link StripeLabelModel#ratio} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel%23EAST Online Documentation}*/
static EAST: ILabelModelParameter;
/**
A predefined parameter instance that can be shared between label instances.
Trying to change that instances' {@link StripeLabelModel#ratio} will raise an {@link Exception InvalidOperationError}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel%23WEST Online Documentation}*/
static WEST: ILabelModelParameter;
/**
Gets or sets a value indicating whether to use the {@link IStripe#actualInsets actual insets} or the normal {@link IStripe#insets} of a stripe for centering the label
inside the stripe header area.
If <code>true</code> the header insets are used. Default is <code>false</code>
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel%23useActualInsets Online Documentation}*/
useActualInsets: boolean;
/**
Gets or sets the ratio how far the label should be positioned from the border in the header area.
0 means the label is positioned on the outer border, 1 means it is positioned at the inner border of the header or inset
area. Default value is 0.5
@default 0.5
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelModel%23ratio Online Documentation}*/
ratio: number;
static $class: Class;
private $_StripeLabelModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StripeLabelModel;
}
export interface IStripeSelection extends ISelectionModel<IStripe> {
}
/**
Manages the selection state of items in an {@link ITable} instance.
This interface provides access to the {@link ISelectionModel} instances that manage the selection of rows and columns. The generic methods that
use {@link IModelItem} parameters delegate to the corresponding domain-specific selection models.
@see ISelectionModel
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeSelection Online Documentation}*/
export abstract class IStripeSelection {
/**
Gets an {@link ISelectionModel} of the selected rows.
This is the row part of the selection that this instance is a composite of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeSelection%23selectedRows Online Documentation}*/
abstract readonly selectedRows: ISelectionModel<IRow>;
/**
Gets an {@link ISelectionModel} of the selected columns.
This is the column part of the selection that this instance is a composite of.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeSelection%23selectedColumns Online Documentation}*/
abstract readonly selectedColumns: ISelectionModel<IColumn>;
/**
Gets or sets whether to allow mixed row/column selections
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeSelection%23allowMixedSelection Online Documentation}*/
abstract allowMixedSelection: boolean;
/**
Gets or sets whether to allow the selection to span multiple table instances
The default is <code>false</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeSelection%23allowCrossTableSelection Online Documentation}*/
abstract allowCrossTableSelection: boolean;
static $class: Class;
private $_IStripeSelection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IStripeSelection;
}
export interface StripeSelection extends YObject, IStripeSelection {
/**
@inheritdoc*/
clear(): void;
/**
@inheritdoc*/
getEnumerator(): IEnumerator<IStripe>;
/**
@inheritdoc*/
setSelected(item: IStripe, selected: boolean): void;
/**
@inheritdoc*/
readonly selectedRows: ISelectionModel<IRow>;
/**
@inheritdoc*/
readonly selectedColumns: ISelectionModel<IColumn>;
/**
@inheritdoc*/
addItemSelectionChangedListener(listener: (sender: any, evt: ItemSelectionChangedEventArgs<IStripe>) => void): void;
/**
@inheritdoc*/
removeItemSelectionChangedListener(listener: (sender: any, evt: ItemSelectionChangedEventArgs<IStripe>) => void): void;
} /**
Default implementation of the {@link IStripeSelection} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSelection Online Documentation}*/
export class StripeSelection {
/**
Default constructor that creates empty selection models.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSelection Online Documentation}*/
constructor();
/**
Return whether <code>stripe</code> is currently selected
@param stripe The stripe to test
@returns <code>true</code> iff <code>stripe</code> is currently selected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSelection%23isSelected(IStripe) Online Documentation}*/
isSelected(stripe: IStripe): boolean;
/**
Raises the {@link StripeSelection#addItemSelectionChangedListener ItemSelectionChanged} event
@param evt The parameters for the event
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSelection%23onItemSelectionChanged(ItemSelectionChangedEventArgs%3CIStripe%3E) Online Documentation}*/
onItemSelectionChanged(evt: ItemSelectionChangedEventArgs<IStripe>): void;
/**
Gets or sets whether to allow mixed row/column selections
The default is <code>false</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSelection%23allowMixedSelection Online Documentation}*/
allowMixedSelection: boolean;
/**
Gets or sets whether to allow the selection to span multiple table instances
The default is <code>true</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeSelection%23allowCrossTableSelection Online Documentation}*/
allowCrossTableSelection: boolean;
static $class: Class;
private $_StripeSelection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StripeSelection;
}
export interface Table extends YObject, ITable, ICloneable {
/**
@inheritdoc*/
addLabel(owner: IStripe, text: string, layoutParameter?: ILabelModelParameter | null, style?: ILabelStyle | null, preferredSize?: Size | null | SizeConvertible, tag?: any | null): ILabel;
/**
@inheritdoc*/
addLabel(options: {
owner: IStripe;
text: string;
layoutParameter?: ILabelModelParameter | null;
style?: ILabelStyle | null;
preferredSize?: Size | null | SizeConvertible;
tag?: any | null;
}): ILabel;
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
createChildColumn(owner: IColumn | null, width?: number | null, minWidth?: number | null, insets?: Insets | null | InsetsConvertible, style?: IStripeStyle | null, tag?: any | null, index?: number | null): IColumn;
/**
@inheritdoc*/
createChildColumn(options: {
owner: IColumn | null;
width?: number | null;
minWidth?: number | null;
insets?: Insets | null | InsetsConvertible;
style?: IStripeStyle | null;
tag?: any | null;
index?: number | null;
}): IColumn;
/**
@inheritdoc*/
createChildRow(owner: IRow | null, height?: number | null, minHeight?: number | null, insets?: Insets | null | InsetsConvertible, style?: IStripeStyle | null, tag?: any | null, index?: number | null): IRow;
/**
@inheritdoc*/
createChildRow(options: {
owner: IRow | null;
height?: number | null;
minHeight?: number | null;
insets?: Insets | null | InsetsConvertible;
style?: IStripeStyle | null;
tag?: any | null;
index?: number | null;
}): IRow;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
Removes the given label from its owner.
This will trigger the corresponding event.
@param label the label to remove
@throws {Exception} <b>(ArgumentError)</b> If the label is not associated with this table instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23remove(ILabel) Online Documentation}*/
remove(label: ILabel): void;
/**
@inheritdoc*/
remove(stripe: IStripe): void;
/**
@inheritdoc*/
setMinimumSize(stripe: IStripe, minimumSize: number): void;
/**
@inheritdoc*/
setParent(owner: IColumn, column: IColumn, index?: number | null): void;
/**
@inheritdoc*/
setParent(owner: IRow, row: IRow, index?: number | null): void;
/**
@inheritdoc*/
setSize(stripe: IStripe, size: number): void;
/**
@inheritdoc*/
setStripeInsets(stripe: IStripe, insets: Insets | InsetsConvertible): void;
/**
Assigns the given style instance by reference to the label.
Style instances can be shared.
@param label The label that will be assigned the new style
@param style The style instance that will be assigned to the label.
@throws {Exception} <b>(ArgumentError)</b> If the label is not associated with this table instance.
@see ILabel#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23setStyle(ILabel%2CILabelStyle) Online Documentation}*/
setStyle(label: ILabel, style: ILabelStyle): void;
/**
@inheritdoc*/
setStyle(stripe: IStripe, style: IStripeStyle): void;
/**
@inheritdoc*/
readonly rootRow: IRow;
/**
@inheritdoc*/
readonly rootColumn: IColumn;
/**
@inheritdoc*/
addStripeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IStripe>) => void): void;
/**
@inheritdoc*/
removeStripeCreatedListener(listener: (sender: any, evt: ItemEventArgs<IStripe>) => void): void;
/**
@inheritdoc*/
addStripeRemovedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
/**
@inheritdoc*/
removeStripeRemovedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
/**
@inheritdoc*/
addStripeChangedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
/**
@inheritdoc*/
removeStripeChangedListener(listener: (sender: any, evt: StripeEventArgs) => void): void;
/**
@inheritdoc*/
addLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
@inheritdoc*/
removeLabelAddedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
@inheritdoc*/
addLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
@inheritdoc*/
removeLabelRemovedListener(listener: (sender: any, evt: LabelEventArgs) => void): void;
/**
@inheritdoc*/
addLabelChangedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
/**
@inheritdoc*/
removeLabelChangedListener(listener: (sender: any, evt: ItemEventArgs<ILabel>) => void): void;
} /**
Default implementation of the {@link ITable} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table Online Documentation}*/
export class Table {
/**
Default constructor that creates an empty table
@param [options] The parameters to pass.
@param [options.insets] Sets the insets property.
@param [options.rowDefaults] The defaults for rows. This option sets the {@link Table#rowDefaults} property on the created object.
@param [options.columnDefaults] The defaults for Columns. This option sets the {@link Table#columnDefaults} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table Online Documentation}*/
constructor(options?: {
insets?: Insets | InsetsConvertible;
rowDefaults?: IStripeDefaults;
columnDefaults?: IStripeDefaults;
});
/**
Called during {@link Table#clone} to create a copy of <code>column</code>
If <code>column</code> implements the {@link ICloneable} interface, the <code>column</code> instance is
{@link ICloneable#clone}d, otherwise, the original instance is returned unchanged.
@param column The column that should be copied during a clone operation.
@returns A clone of <code>column</code> if possible, otherwise the original reference to <code>column</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23copyColumn(IColumn) Online Documentation}*/
copyColumn(column: IColumn): IColumn;
/**
Called during {@link Table#clone} to create a copy of <code>originalDefaults</code>
This implementation returns an unchanged reference to <code>originalDefaults</code>. If you need to create a deep clone of
<code>originalDefaults</code>, you'll have to perform the cloning yourself.
@param originalDefaults The default settings that should be copied during a clone operation.
@returns The original reference to <code>originalDefaults</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23copyDefaults(IStripeDefaults) Online Documentation}*/
copyDefaults(originalDefaults: IStripeDefaults): IStripeDefaults;
/**
Called during {@link Table#clone} to create a copy of <code>row</code>
If <code>row</code> implements the {@link ICloneable} interface, the <code>row</code> instance is
{@link ICloneable#clone}d, otherwise, the original instance is returned unchanged.
@param row The row that should be copied during a clone operation.
@returns A clone of <code>row</code> if possible, otherwise the original reference to <code>row</code>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23copyRow(IRow) Online Documentation}*/
copyRow(row: IRow): IRow;
/**
Creates the Column defaults.
@returns A new instance of the {@link StripeDefaults} class which is used for {@link Table#createChildColumn newly created columns}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23createColumnDefaults() Online Documentation}*/
createColumnDefaults(): IStripeDefaults;
/**
Creates the row defaults.
@returns A new instance of the {@link StripeDefaults} class that is used for {@link Table#createChildRow newly created rows}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23createRowDefaults() Online Documentation}*/
createRowDefaults(): IStripeDefaults;
/**
Installs the undo support for use with all tables that are bound to nodes in <code>graph</code>.
This method queries the <code>graph</code> for its undo support dynamically whenever undo support is queried in the table. In
contrast to {@link Table#installStaticUndoSupport} the undo support installed with this method will thus always be
the one which is the currently available support for the <code>graph</code>.
@param graph The graph where the nodes exist or will be created/changed and where the undo support should come from.
@see Table#uninstallDynamicUndoSupport
@see Table#installStaticUndoSupport
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23installDynamicUndoSupport(IGraph) Online Documentation}*/
static installDynamicUndoSupport(graph: IGraph): void;
/**
Installs the undo support for use with all tables that are bound to nodes in <code>graph</code>.
This methods installs the undo support which manages the <code>graph</code> at the time this method is called. In contrast to {@link Table#installDynamicUndoSupport}
the installed undo support will not change when the <code>graph</code>'s undo support changes.
This method adds undo support both for existing nodes in <code>graph</code> as well as for newly created or changed nodes.
@param graph The graph where the nodes exist or will be created/changed.
@see Table#uninstallStaticUndoSupport
@see Table#installDynamicUndoSupport
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23installStaticUndoSupport(IGraph) Online Documentation}*/
static installStaticUndoSupport(graph: IGraph): void;
/**
Called after a label has been added to a stripe.
This method raises the {@link Table#addLabelAddedListener LabelAdded} event.
@param label The label that has just been added.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23onLabelAdded(ILabel) Online Documentation}*/
onLabelAdded(label: ILabel): void;
/**
Called after a label has changed.
This method raises the {@link Table#addLabelChangedListener LabelChanged} event.
@param label The label that has changed.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23onLabelChanged(ILabel) Online Documentation}*/
onLabelChanged(label: ILabel): void;
/**
Called after a label has been removed from its stripe.
This method raises the {@link Table#addLabelRemovedListener LabelRemoved} event.
@param label The label that has just been removed.
@param owner The old owner of the label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23onLabelRemoved(ILabelOwner%2CILabel) Online Documentation}*/
onLabelRemoved(owner: ILabelOwner, label: ILabel): void;
/**
Called after a stripe has changed.
This will trigger the {@link Table#addStripeChangedListener StripeChanged} event.
@param owner The original owner
@param stripe The stripe that has changed.
@param oldTable The original table owner, in case the stripe has been moved between different table instances
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23onStripeChanged(IStripe%2CIStripe%2CITable) Online Documentation}*/
onStripeChanged(owner: IStripe, stripe: IStripe, oldTable: ITable): void;
/**
Called after a stripe has been created and added to a table.
This method raises the {@link Table#addStripeCreatedListener StripeCreated} event.
@param stripe The stripe that has just been created.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23onStripeCreated(IStripe) Online Documentation}*/
onStripeCreated(stripe: IStripe): void;
/**
Called after a stripe has been removed from its owner.
This method raises the {@link Table#addStripeRemovedListener StripeRemoved} event.
@param stripe The stripe that has just been removed.
@param owner The old owner of the stripe.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23onStripeRemoved(IStripe%2CIStripe) Online Documentation}*/
onStripeRemoved(owner: IStripe, stripe: IStripe): void;
/**
Sets the label model parameter for the given label.
@param label The label.
@param layoutParameter The new parameter.
@throws {Exception} <b>(ArgumentError)</b> If the parameter cannot be used for this label or if the label is not associated with this table instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23setLabelLayoutParameter(ILabel%2CILabelModelParameter) Online Documentation}*/
setLabelLayoutParameter(label: ILabel, layoutParameter: ILabelModelParameter): void;
/**
Sets the preferred size of the label.
@param label The label.
@param preferredSize The new preferred size.
@throws {Exception} <b>(ArgumentError)</b> If the label is not associated with this table instance.
@see ILabel#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23setLabelPreferredSize(ILabel%2CSize) Online Documentation}*/
setLabelPreferredSize(label: ILabel, preferredSize: Size): void;
/**
Sets the label text of the given label.
@param label the label to modify
@param text the new text of the label
@throws {Exception} <b>(ArgumentError)</b> If the label is not associated with this table instance.
@see ILabel#text
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23setLabelText(ILabel%2Cstring) Online Documentation}*/
setLabelText(label: ILabel, text: string): void;
/**
Uninstalls the undo support that has previously been installed with {@link Table#installDynamicUndoSupport}
@param graph The graph where the tables are installed for which the undo support should be cleared.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23uninstallDynamicUndoSupport(IGraph) Online Documentation}*/
static uninstallDynamicUndoSupport(graph: IGraph): void;
/**
Uninstalls the undo support that has previously been installed with {@link Table#installStaticUndoSupport}
@param graph The graph where the tables are installed for which the undo support should be cleared.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23uninstallStaticUndoSupport(IGraph) Online Documentation}*/
static uninstallStaticUndoSupport(graph: IGraph): void;
/**
Gets an {@link IContextLookup} that provides the default implementations returned by an
{@link IStripe}'s lookup.
The provided default implementations can for example be used as fallback when the stripe lookup is wrapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23DEFAULT_STRIPE_LOOKUP Online Documentation}*/
static DEFAULT_STRIPE_LOOKUP: IContextLookup;
/**
Gets an {@link IContextLookup} that provides the default implementations returned by an
{@link IColumn}'s lookup.
The provided default implementations can for example be used as fallback when the column lookup is wrapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23DEFAULT_COLUMN_LOOKUP Online Documentation}*/
static DEFAULT_COLUMN_LOOKUP: IContextLookup;
/**
Gets an {@link IContextLookup} that provides the default implementations returned by an
{@link IRow}'s lookup.
The provided default implementations can for example be used as fallback when the row lookup is wrapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23DEFAULT_ROW_LOOKUP Online Documentation}*/
static DEFAULT_ROW_LOOKUP: IContextLookup;
/**
Gets the toplevel {@link IRow}s in this table.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23rows Online Documentation}*/
readonly rows: IEnumerable<IRow>;
/**
Gets the toplevel {@link IColumn}s in this table.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23columns Online Documentation}*/
readonly columns: IEnumerable<IColumn>;
/**
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23insets Online Documentation}*/
insets: Insets;
/**
Gets the layout of the table, relative to an owner node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23layout Online Documentation}*/
readonly layout: IRectangle;
/**
Gets or sets the defaults for rows.
The settings that are obtained from the instance influence newly created elements only. Setting different defaults later
does not influence existing elements.
If no value is set explicitly a default will be created using {@link Table#createRowDefaults} upon first read
access.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23rowDefaults Online Documentation}*/
rowDefaults: IStripeDefaults;
/**
Gets or sets the defaults for Columns.
The settings that are obtained from the instance influence newly created elements only. Setting different defaults later
does not influence existing elements.
If no value is set explicitly a default will be created using {@link Table#createColumnDefaults} upon first read
access.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Table%23columnDefaults Online Documentation}*/
columnDefaults: IStripeDefaults;
static $class: Class;
private $_Table: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Table;
}
export interface IStripeDefaults {
}
/**
Interface used by {@link ITable} and the like to declare and obtain the defaults for stripes and their labels.
@see ITable#rowDefaults
@see ITable#columnDefaults
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeDefaults Online Documentation}*/
export abstract class IStripeDefaults {
/**
Factory method that returns a style instance for use with newly created stripes.
Most implementations will yield either, a {@link ICloneable#clone clone} of or the {@link IStripeDefaults#style} property, if {@link IStripeDefaults#shareStyleInstance} is enabled, but they might use more complicated
logic, too.
@returns The style to use, which for most implementations is either a {@link ICloneable#clone clone} of or the {@link IStripeDefaults#style} property, if {@link IStripeDefaults#shareStyleInstance} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeDefaults%23getStyleInstance() Online Documentation}*/
abstract getStyleInstance(): IStripeStyle;
/**
Gets or sets the default insets for a stripe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeDefaults%23insets Online Documentation}*/
abstract insets: Insets;
/**
Gets or sets the default stripe size.
@see IStripe#size
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeDefaults%23size Online Documentation}*/
abstract size: number;
/**
Gets or sets the default minimum stripe size.
@see IStripe#minimumSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeDefaults%23minimumSize Online Documentation}*/
abstract minimumSize: number;
/**
Gets or sets the defaults for labels at stripes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeDefaults%23labels Online Documentation}*/
abstract labels: ILabelDefaults;
/**
Gets or sets the style to use for stripes.
Depending on the setting of {@link IStripeDefaults#shareStyleInstance}, the {@link IStripeDefaults#getStyleInstance} method should return a {@link ICloneable#clone clone} of this
instance or the very same instance.
@see IStripeDefaults#shareStyleInstance
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeDefaults%23style Online Documentation}*/
abstract style: IStripeStyle;
/**
Gets or sets a value indicating whether the {@link IStripeDefaults#style} instance should be shared referentially or {@link ICloneable#clone cloned} upon a call to
{@link IStripeDefaults#getStyleInstance}.
@see IStripeDefaults#getStyleInstance
@see IStripeDefaults#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeDefaults%23shareStyleInstance Online Documentation}*/
abstract shareStyleInstance: boolean;
static $class: Class;
private $_IStripeDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IStripeDefaults;
}
export interface StripeDefaults extends YObject, IStripeDefaults {
/**
@inheritdoc*/
getStyleInstance(): IStripeStyle;
/**
@inheritdoc*/
labels: ILabelDefaults;
/**
@inheritdoc*/
style: IStripeStyle;
} /**
A canonical implementation of the {@link IStripeDefaults} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDefaults Online Documentation}*/
export class StripeDefaults {
/**
Default constructor
This constructor sets the style to {@link VoidNodeStyle#INSTANCE} and the insets to {@link Insets#EMPTY}
@param [options] The parameters to pass.
@param [options.size] The default stripe size. This option sets the {@link StripeDefaults#size} property on the created object.
@param [options.minimumSize] The default minimum stripe size. This option sets the {@link StripeDefaults#minimumSize} property on the created object.
@param [options.labels] Sets the labels property.
@param [options.style] Sets the style property.
@param [options.shareStyleInstance] Sets the shareStyleInstance property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDefaults Online Documentation}*/
constructor(options?: {
size?: number;
minimumSize?: number;
labels?: ILabelDefaults;
style?: IStripeStyle;
shareStyleInstance?: boolean;
});
/**
Gets or sets the default insets for a stripe.
@default '0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDefaults%23insets Online Documentation}*/
insets: Insets;
/**
Gets or sets the default stripe size.
@see IStripe#size
@default 100
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDefaults%23size Online Documentation}*/
size: number;
/**
Gets or sets the default minimum stripe size.
@see IStripe#minimumSize
@default 10
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDefaults%23minimumSize Online Documentation}*/
minimumSize: number;
/**
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDefaults%23shareStyleInstance Online Documentation}*/
shareStyleInstance: boolean;
static $class: Class;
private $_StripeDefaults: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StripeDefaults;
}
export interface TableDecorator extends YObject {
}
/**
Facilitates decorating {@link IModelItem}s from an {@link ITable} instance.
This class serves as a factory for lookup decorators for the various items that make up an {@link ITable}. It
obtains the {@link ILookupDecorator} from the {@link ITable}'s {@link ILookup#lookup} and passes that to
the various specialized decorator classes {@link StripeDecorator}, {@link StripeDecorator} and {@link StripeLabelDecorator}.
@see TableDecorator#rowDecorator
@see TableDecorator#columnDecorator
@see StripeDecorator
@see TableDecorator#stripeLabelDecorator
@see StripeLabelDecorator
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableDecorator Online Documentation}*/
export class TableDecorator {
/**
Initializes a new instance of the {@link TableDecorator} class.
@param table The table.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableDecorator Online Documentation}*/
constructor(table: ITable);
/**
Gets a {@link StripeDecorator} instance that can be used to decorate the {@link IRow} instances in the table that this decorator has been
created for.
This will obtain the {@link ILookupDecorator} instance from the {@link ITable}'s {@link ILookup#lookup}
method and use it to initialize the {@link StripeDecorator}
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableDecorator%23rowDecorator Online Documentation}*/
readonly rowDecorator: StripeDecorator<IRow>;
/**
Gets a {@link StripeDecorator} instance that can be used to decorate the {@link IColumn} instances in the table that this decorator has
been created for.
This will obtain the {@link ILookupDecorator} instance from the {@link ITable}'s {@link ILookup#lookup}
method and use it to initialize the {@link StripeDecorator}
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableDecorator%23columnDecorator Online Documentation}*/
readonly columnDecorator: StripeDecorator<IColumn>;
/**
Gets a {@link StripeLabelDecorator} instance that can be used to decorate the {@link ILabel} instances in the
table that this decorator has been created for.
This will obtain the {@link ILookupDecorator} instance from the {@link ITable}'s {@link ILookup#lookup}
method and use it to initialize the {@link StripeLabelDecorator}
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableDecorator%23stripeLabelDecorator Online Documentation}*/
readonly stripeLabelDecorator: StripeLabelDecorator;
static $class: Class;
private $_TableDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TableDecorator;
}
export interface StripeDecorator<TStripe extends IStripe> extends YObject {
}
/**
Helps in {@link ILookupDecorator#addLookup decorating} the {@link ILookup#lookup} method of {@link IStripe} instances by providing access to
decorators for the most common types.
Predefined {@link LookupDecorator} are made available through properties like e.g.
{@link StripeDecorator#positionHandlerDecorator}. Note that the list of possible decorations that are available using this class is not exhaustive in any way. This is
merely a way to provide access to those interfaces that are more commonly being decorated by the developer.
Typically, a configured instance of this class is available through {@link TableDecorator#columnDecorator} and {@link TableDecorator#rowDecorator} on the {@link TableDecorator} which is
returned by {@link ITable#decorator} on an {@link ITable}.
This decorator can be used to conveniently {@link ILookupDecorator decorate the lookup} of {@link IStripe}s in an {@link ITable}. It
serves as a factory for predefined {@link LookupDecorator} instances that are specifically useful for the {@link IStripe} type.
Type parameter <code>TStripe</code>.
@see TableDecorator
@see LookupDecorator
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDecorator Online Documentation}*/
export class StripeDecorator<TStripe extends IStripe> {
/**
Initializes a new instance of the {@link StripeDecorator} class.
@param type
@param decorator The decorator to use, which can be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDecorator Online Documentation}*/
constructor(type: Class, decorator: ILookupDecorator | null);
/**
Creates a {@link LookupDecorator} for <code>TStripe</code>s that can be used to decorate <code>TInterface</code> types of the labels.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(type: Class, decorateNulls?: boolean, nullIsFallback?: boolean): LookupDecorator<TStripe, TInterface>;
/**
Creates a {@link LookupDecorator} for <code>TStripe</code>s that can be used to decorate <code>TInterface</code> types of the labels.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param options The parameters to pass.
@param options.type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(options: {
type: Class;
decorateNulls?: boolean;
nullIsFallback?: boolean;
}): LookupDecorator<TStripe, TInterface>;
/**
Removes an {@link IContextLookupChainLink} again that has been added by one of the various decorators for stripes.
@param lookup The lookup chain link to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDecorator%23remove(IContextLookupChainLink) Online Documentation}*/
remove(lookup: IContextLookupChainLink): void;
/**
Gets a {@link LookupDecorator} that decorates the {@link IPositionHandler} type for the type of items this instance has been created for.
Custom {@link IPositionHandler} implementations can be used to restrict or enhance the way the user moves elements
interactively in the editor. This interface is mainly used by the {@link ReparentStripeInputMode}. This is a
convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link ITable}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDecorator%23positionHandlerDecorator Online Documentation}*/
readonly positionHandlerDecorator: LookupDecorator<TStripe, IPositionHandler>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IStripeInputVisualizationHelper} type for the type of items this instance has
been created for.
Custom {@link IStripeInputVisualizationHelper} implementations can be used change the way the look of an item is
represented during the default interactions in a {@link CanvasComponent}. This interface is mainly used by the
{@link TableEditorInputMode}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link ITable}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDecorator%23inputVisualizationDecorator Online Documentation}*/
readonly inputVisualizationDecorator: LookupDecorator<TStripe, IStripeInputVisualizationHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IEditLabelHelper} type for the <code>TStripe</code>s this instance has been created for.
Custom {@link IEditLabelHelper} implementations can be used to change or customize interactive label editing and
label creation. This interface is mainly used by the {@link TableEditorInputMode} which uses the callbacks from the
implementation to determine the label to add or edit, as well as to customize the {@link TextEditorInputMode}
appearance for the upcoming edit. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link ITable}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeDecorator%23editLabelHelperDecorator Online Documentation}*/
readonly editLabelHelperDecorator: LookupDecorator<TStripe, IEditLabelHelper>;
static $class: Class;
private $_StripeDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TStripe extends IStripe>(o: any): o is StripeDecorator<TStripe>;
}
export interface StripeLabelDecorator extends YObject {
}
/**
Helps in {@link ILookupDecorator#addLookup decorating} the {@link ILookup#lookup} method of {@link ILabel} instances by providing access to
decorators for the most common types.
Predefined {@link LookupDecorator} are made available through properties like e.g.
{@link StripeLabelDecorator#highlightDecorator}. Note that the list of possible decorations that are available using this class is not exhaustive in any way. This is
merely a way to provide access to those interfaces that are more commonly being decorated by the developer.
Typically, a configured instance of this class is available through {@link TableDecorator#stripeLabelDecorator} on the {@link TableDecorator} which is
returned by {@link ITable#decorator} on an {@link ITable}.
This decorator can be used to conveniently {@link ILookupDecorator decorate the lookup} of {@link ILabel}s in an {@link ITable}. It
serves as a factory for predefined {@link LookupDecorator} instances that are specifically useful for the {@link ILabel} type.
@see TableDecorator
@see LookupDecorator
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator Online Documentation}*/
export class StripeLabelDecorator {
/**
Initializes a new instance of the {@link StripeLabelDecorator} class.
@param decorator The decorator to use, which can be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator Online Documentation}*/
constructor(decorator: ILookupDecorator | null);
/**
Creates a {@link LookupDecorator} for {@link ILabel}s that can be used to decorate <code>TInterface</code> types of the labels.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(type: Class, decorateNulls?: boolean, nullIsFallback?: boolean): LookupDecorator<ILabel, TInterface>;
/**
Creates a {@link LookupDecorator} for {@link ILabel}s that can be used to decorate <code>TInterface</code> types of the labels.
<h2>Note</h2>
Note that the <code>$class</code> property on JavaScript classes inheriting from yFiles classes is only valid once an instance of
this subclass was created. Call fixType, first or instantiate the class at least once.
Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param options The parameters to pass.
@param options.type The type of the interface that should be decorated of the instances' {@link ILookup#lookup}.
@param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
@param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
@returns A new {@link LookupDecorator} for the specified interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator%23getDecoratorFor(Class%2Cboolean%2Cboolean) Online Documentation}*/
getDecoratorFor<TInterface>(options: {
type: Class;
decorateNulls?: boolean;
nullIsFallback?: boolean;
}): LookupDecorator<ILabel, TInterface>;
/**
Removes an {@link IContextLookupChainLink} again that has been added by one of the various decorators for stripe
labels.
@param lookup The lookup chain link to remove.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator%23remove(IContextLookupChainLink) Online Documentation}*/
remove(lookup: IContextLookupChainLink): void;
/**
Gets a {@link LookupDecorator} that decorates the {@link ISelectionIndicatorInstaller} type for the type of items this instance has been
created for.
Custom {@link ISelectionIndicatorInstaller} implementations can be used change the way the selection of an item is
represented in a {@link CanvasComponent}. This interface is mainly used by the {@link SelectionIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link ITable}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator%23selectionDecorator Online Documentation}*/
readonly selectionDecorator: LookupDecorator<ILabel, ISelectionIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IHighlightIndicatorInstaller} type for the type of items this instance has been
created for.
Custom {@link IHighlightIndicatorInstaller} implementations can be used change the way the highlighting of an item
is represented in a {@link CanvasComponent}. This interface is mainly used by the {@link HighlightIndicatorManager} in the
{@link GraphComponent}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link ITable}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator%23highlightDecorator Online Documentation}*/
readonly highlightDecorator: LookupDecorator<ILabel, IHighlightIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IFocusIndicatorInstaller} type for the type of items this instance has been
created for.
Custom {@link IFocusIndicatorInstaller} implementations can be used change the way the focus of an item is
represented in a {@link GraphComponent}. This interface is mainly used by the {@link FocusIndicatorManager} in the {@link GraphComponent}.
This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is provided by the
{@link ITable}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator%23focusIndicatorDecorator Online Documentation}*/
readonly focusIndicatorDecorator: LookupDecorator<ILabel, IFocusIndicatorInstaller>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILabelModelParameterFinder} type for the
{@link ILabel}s this instance has been created for.
Custom {@link ILabelModelParameterFinder} implementations can be used change the way {@link ILabel}s can be
dragged interactively by the user. It helps in determining the right {@link ILabelModelParameter} for a given
location that has been chosen by the user. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link ITable}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator%23labelModelParameterFinderDecorator Online Documentation}*/
readonly labelModelParameterFinderDecorator: LookupDecorator<ILabel, ILabelModelParameterFinder>;
/**
Gets a {@link LookupDecorator} that decorates the {@link IEditLabelHelper} type for the
{@link ILabel}s this instance has been created for.
Custom {@link IEditLabelHelper} implementations can be used to change or customize interactive label editing. This
interface is mainly used by the {@link TableEditorInputMode} which uses the callbacks from the implementation to
determine whether the label can be edited, as well as to customize the {@link TextEditorInputMode} appearance for
the upcoming edit. This is a convenient alternative to the direct usage of the {@link ILookupDecorator} that is
provided by the {@link ITable}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator%23editLabelHelperDecorator Online Documentation}*/
readonly editLabelHelperDecorator: LookupDecorator<ILabel, IEditLabelHelper>;
/**
Gets a {@link LookupDecorator} that decorates the {@link ILabelModelParameterProvider} type for the
{@link ILabel}s this instance has been created for.
Custom {@link ILabelModelParameterProvider} implementations can be used to change the set of possible
{@link ILabelModelParameter}s that the user can assign to a label interactively by dragging it in a
{@link GraphComponent}. This is a convenient alternative to the direct usage of the {@link ILookupDecorator}
that is provided by the {@link ITable}'s {@link ILookup#lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeLabelDecorator%23labelModelParameterProviderDecorator Online Documentation}*/
readonly labelModelParameterProviderDecorator: LookupDecorator<ILabel, ILabelModelParameterProvider>;
static $class: Class;
private $_StripeLabelDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StripeLabelDecorator;
}
export interface IStripeStyle extends ICloneable {
}
/**
Style implementation for {@link IStripe} instances in an {@link ITable}.
It is up to the implementation to interpret the visual appearance of a stripe. The framework uses the associated,
possibly shared {@link IStripeStyle#renderer} to perform the actual rendering of this style for a given stripe. This interface extends the {@link ICloneable}
interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
implementations may return themselves.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeStyle Online Documentation}*/
export abstract class IStripeStyle {
/**
* Gets the renderer implementation that can be queried for implementations that provide details about the visual
* appearance and visual behavior for a given stripe and this style instance.
* The idiom for retrieving, e.g. an {@link IVisualCreator} implementation for a given stripe is:
*
* <pre>
* const creator = style.renderer.getVisualCreator(stripe, style)
* const visual = creator.createVisual(context)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeStyle%23renderer Online Documentation}
*/
abstract readonly renderer: IStripeStyleRenderer;
static $class: Class;
private $_IStripeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IStripeStyle;
}
export interface IStripeStyleRenderer {
}
/**
Interface used by {@link IStripeStyle} implementations that can be used to query the actual {@link IVisualCreator}
implementation for a stripe and an associated style.
Implementations of this interface should be designed to be sharable between multiple style instances and should
therefore not contain style specific state.
@see IStripeStyle
@see IStripe
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeStyleRenderer Online Documentation}*/
export abstract class IStripeStyleRenderer {
/**
Gets a temporary context instance that can be used to query additional information for the stripe's style.
Implementations may return {@link ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
@param stripe The stripe to provide a context instance for.
@param style The style to use for the context.
@returns An non-<code>null</code> lookup implementation.
@see ILookup#EMPTY
@see ILookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeStyleRenderer%23getContext(IStripe%2CIStripeStyle) Online Documentation}*/
abstract getContext(stripe: IStripe, style: IStripeStyle): ILookup;
/**
Gets an implementation of the {@link IVisualCreator} interface that can handle the provided stripe and its
associated style.
This method may return a flyweight implementation, but never <code>null</code>.
@param stripe The stripe to provide an instance for
@param style The style to use for the creation of the visual
@returns An implementation that may be used to subsequently create or update the visual for the stripe. Clients should not cache
this instance and must always call this method immediately before using the value returned. This enables the use of the
flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link VoidVisualCreator#INSTANCE void} implementation
instead.
@see VoidVisualCreator#INSTANCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/IStripeStyleRenderer%23getVisualCreator(IStripe%2CIStripeStyle) Online Documentation}*/
abstract getVisualCreator(stripe: IStripe, style: IStripeStyle): IVisualCreator;
static $class: Class;
private $_IStripeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IStripeStyleRenderer;
}
export interface VoidStripeStyle extends YObject, IStripeStyle {
}
/**
A void implementation of a stripe style that does nothing and behaves like an invisible style.
This class implements the singleton pattern. Use the shared static {@link VoidStripeStyle#INSTANCE} of this class
instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link IStripe#style} property of an
{@link IStripe}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidStripeStyle Online Documentation}*/
export class VoidStripeStyle {
/**
Returns <code>this</code>.
@returns <code>this</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidStripeStyle%23clone() Online Documentation}*/
clone(): this;
/**
The {@link VoidStripeStyle} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidStripeStyle%23INSTANCE Online Documentation}*/
static INSTANCE: VoidStripeStyle;
/**
Gets the {@link VoidStripeStyleRenderer#INSTANCE VoidStripeStyleRenderer instance}.
@see IStripeStyle#renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidStripeStyle%23renderer Online Documentation}*/
readonly renderer: IStripeStyleRenderer;
static $class: Class;
private $_VoidStripeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidStripeStyle;
}
export interface VoidStripeStyleRenderer extends YObject, IStripeStyleRenderer {
}
/**
A void implementation of a stripe style renderer that does nothing.
This class implements the singleton pattern. Use the shared static {@link VoidStripeStyleRenderer#INSTANCE} of
this class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link IStripeStyle#renderer} property of an
{@link IStripeStyle}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidStripeStyleRenderer Online Documentation}*/
export class VoidStripeStyleRenderer {
/**
Yields the {@link ILookup#EMPTY} that will not yield anything.
@param stripe
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidStripeStyleRenderer%23getContext(IStripe%2CIStripeStyle) Online Documentation}*/
getContext(stripe: IStripe, style: IStripeStyle): ILookup;
/**
Yields the {@link VoidVisualCreator#INSTANCE} that will do nothing.
@param stripe
@param style
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidStripeStyleRenderer%23getVisualCreator(IStripe%2CIStripeStyle) Online Documentation}*/
getVisualCreator(stripe: IStripe, style: IStripeStyle): IVisualCreator;
/**
The {@link VoidStripeStyleRenderer} singleton.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/VoidStripeStyleRenderer%23INSTANCE Online Documentation}*/
static INSTANCE: IStripeStyleRenderer;
static $class: Class;
private $_VoidStripeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is VoidStripeStyleRenderer;
}
export interface NodeStyleStripeStyleAdapter extends YObject, IStripeStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: IStripeStyleRenderer;
} /**
A stripe style decorator that uses a node style instance to render the stripe.
The renderer of the {@link NodeStyleStripeStyleAdapter#nodeStyle} used by this class must create {@link SvgVisual}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleStripeStyleAdapter Online Documentation}*/
export class NodeStyleStripeStyleAdapter {
/**
Creates a stripe style that uses the provided node style to render the stripe.
Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
instance.
@param [nodeStyle=null] The node style to use for rendering the stripe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleStripeStyleAdapter Online Documentation}*/
constructor(nodeStyle?: INodeStyle);
/**
Gets the {@link INodeStyle} that is used for rendering the stripe.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeStyleStripeStyleAdapter%23nodeStyle Online Documentation}*/
nodeStyle: INodeStyle;
static $class: Class;
private $_NodeStyleStripeStyleAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeStyleStripeStyleAdapter;
}
/**
Enumeration that is used to specify the drawing order for rows and columns
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableRenderingOrder Online Documentation}*/
export enum TableRenderingOrder {
/**
Draw columns, then rows
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableRenderingOrder%23COLUMNS_FIRST Online Documentation}*/
COLUMNS_FIRST,
/**
Draw rows, then columns
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableRenderingOrder%23ROWS_FIRST Online Documentation}*/
ROWS_FIRST
}
/** Automatic conversion to {@link TableRenderingOrder}. */
type TableRenderingOrderStringValues = "columns-first" | "rows-first";
export interface StripeHitTester extends YObject {
}
/**
Support class that allows to iterate over hits on table elements or subregions of table elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeHitTester Online Documentation}*/
export class StripeHitTester {
/**
Create a new instance for a specific table instance that use the order specified by <code>tableRenderingOrder</code> to determine the
order of the hit test results.
@param table The table to use
@param tableRenderingOrder The hit test order. This determines the order in which stripes are returned by
{@link StripeHitTester#enumerateHits}
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeHitTester Online Documentation}*/
constructor(table: ITable, tableRenderingOrder: TableRenderingOrder | TableRenderingOrderStringValues);
/**
Return a collection of hits at <code>location</code>.
This implementation returns the hits in the following order:
<ol>
<li>First all {@link IRow}s or {@link IColumn}s at the given location, in the order specified by the {@link StripeHitTester#tableRenderingOrder} property.
<li>For each stripe type, the stripes are returned in bottom up order, i.e. children are returned before their parents.
<li>For each stripe, the subregions are returned in the order {@link StripeSubregionTypes#NEAR_BORDER},
{@link StripeSubregionTypes#FAR_BORDER}, {@link StripeSubregionTypes#LEADING_HEADER},
{@link StripeSubregionTypes#TRAILING_HEADER},{@link StripeSubregionTypes#STRIPE}, e.g. a header hit is always
returned before a hit on the whole stripe.
</ol>
@param context The input mode context to use.
@param location The location in absolute coordinates.
@param node The node where the table instance is currently bound to.
@returns A collection of {@link StripeSubregion}s at <code>location</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeHitTester%23enumerateHits(IInputModeContext%2CPoint%2CINode) Online Documentation}*/
enumerateHits(context: IInputModeContext, location: Point | PointConvertible, node: INode): IEnumerable<StripeSubregion>;
/**
Gets the hit test order that is currently used by this instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeHitTester%23tableRenderingOrder Online Documentation}*/
readonly tableRenderingOrder: TableRenderingOrder;
static $class: Class;
private $_StripeHitTester: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StripeHitTester;
}
export interface StripeStyleBase extends YObject, IStripeStyle {
}
/**
An abstract base class that makes it possible to easily implement a custom {@link IStripeStyle}.
The only method that needs to be implemented by subclasses is {@link StripeStyleBase#createVisual}, however to
improve rendering performance it is highly recommended to implement at least
{@link StripeStyleBase#updateVisual}, too.
This implementation differs from the straightforward {@link IStripeStyle} implementation in that there is no
visible separation between the style and its
{@link IStripeStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
the style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeStyleBase Online Documentation}*/
export abstract class StripeStyleBase {
/**
Initializes a new instance of the {@link StripeStyleBase} class.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeStyleBase Online Documentation}*/
constructor();
/**
Creates a new object that is a copy of the current instance.
Immutable subclasses should consider returning <code>this</code>.
@returns A new object that is a copy of this instance using {@link YObject#memberwiseClone}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeStyleBase%23clone() Online Documentation}*/
clone(): this;
/**
Creates the visual.
This method is called in response to a {@link IVisualCreator#createVisual} call to the instance that has been
queried from the {@link StripeStyleBase#renderer}.
@param context
@param stripe The stripe to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see StripeStyleBase#updateVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeStyleBase%23createVisual(IRenderContext%2CIStripe) Online Documentation}*/
abstract createVisual(context: IRenderContext, stripe: IStripe): Visual | null;
/**
Performs the {@link ILookup#lookup} operation for the {@link IStripeStyleRenderer#getContext} that has been
queried from the {@link StripeStyleBase#renderer}.
This implementation yields <code>null</code> for everything but:
<ul>
<li>{@link IVisualCreator}
<li>{@link ILookup}
</ul>
For these interfaces an implementation will be returned that delegates to the methods in this instance.
@param stripe The stripe to use for the context lookup.
@param type The type to query.
@returns An implementation of the <code>type</code> or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeStyleBase%23lookup(IStripe%2CClass) Online Documentation}*/
lookup(stripe: IStripe, type: Class): Object | null;
/**
Updates the visual previously created by {@link StripeStyleBase#createVisual}.
This method is called in response to a {@link IVisualCreator#updateVisual} call to the instance that has been
queried from the {@link StripeStyleBase#renderer}. This implementation simply delegates to {@link StripeStyleBase#createVisual} so subclasses
should override to improve rendering performance.
@param context
@param oldVisual The visual that has been created in the call to {@link StripeStyleBase#createVisual}.
@param stripe The stripe to which this style instance is assigned.
@returns The visual as required by the {@link IVisualCreator#createVisual} interface.
@see StripeStyleBase#createVisual
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeStyleBase%23updateVisual(IRenderContext%2CVisual%2CIStripe) Online Documentation}*/
updateVisual(context: IRenderContext, oldVisual: Visual, stripe: IStripe): Visual | null;
/**
Gets the renderer implementation for this instance.
The private implementation will delegate all API calls back to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StripeStyleBase%23renderer Online Documentation}*/
readonly renderer: IStripeStyleRenderer;
static $class: Class;
private $_StripeStyleBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StripeStyleBase;
}
export interface ArcEdgeStyleRenderer extends PathBasedEdgeStyleRenderer<ArcEdgeStyle> {
/**
@inheritdoc*/
createPath(): GeneralPath;
/**
@inheritdoc*/
getSegmentCount(): number;
/**
@inheritdoc*/
getStroke(): Stroke | null;
/**
@inheritdoc*/
getTangent(ratio: number): Tangent | null;
/**
@inheritdoc*/
getTangentForSegment(segmentIndex: number, ratio: number): Tangent | null;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
Default renderer implementation for {@link ArcEdgeStyle}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyleRenderer Online Documentation}*/
export class ArcEdgeStyleRenderer {
constructor();
/**
Retrieves the {@link ArcEdgeStyle#height} of the style.
@returns the height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyleRenderer%23getHeight() Online Documentation}*/
getHeight(): number;
/**
Gets the source arrow from the style via {@link ArcEdgeStyle#sourceArrow}.
@returns The arrow to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyleRenderer%23getSourceArrow() Online Documentation}*/
getSourceArrow(): IArrow | null;
/**
Gets the target arrow from the style via {@link ArcEdgeStyle#targetArrow}.
@returns The arrow to use.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyleRenderer%23getTargetArrow() Online Documentation}*/
getTargetArrow(): IArrow | null;
/**
Gets a value indicating whether this {@link ArcEdgeStyleRenderer#getHeight} should be interpreted as an absolute
or relative value.
@returns <code>true</code> if the height value should be interpreted as a fixed value, otherwise, <code>false</code>.
@see ArcEdgeStyleRenderer#getHeight
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyleRenderer%23isFixedHeight() Online Documentation}*/
isFixedHeight(): boolean;
/**
Overridden for performance reasons.
@param context
@param rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Gets whether bridges are added to the edge path.
Overridden to return <code>false</code> since the flattening interferes with its own obstacles.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyleRenderer%23addBridges Online Documentation}*/
readonly addBridges: boolean;
static $class: Class;
private $_ArcEdgeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ArcEdgeStyleRenderer;
}
export interface ArcEdgeStyle extends YObject, IEdgeStyle {
}
/**
Provides an arc representation for an edge.
The rendering does not take into account bends of an edge, instead the arc's height is determined as a fixed ratio or
fixed height.
This style uses an {@link ArcEdgeStyleRenderer} for its visualization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle Online Documentation}*/
export class ArcEdgeStyle {
/**
Creates a new instance using the provided renderer.
@param renderer The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23ArcEdgeStyle(ArcEdgeStyleRenderer) Online Documentation}*/
constructor(renderer: ArcEdgeStyleRenderer);
/**
Creates a new instance using the provided renderer.
@param options The parameters to pass.
@param options.renderer The custom renderer for this style instance.
@param [options.stroke] The {@link ArcEdgeStyle#stroke} that is used to draw the arc This option sets the {@link ArcEdgeStyle#stroke} property on the created object.
@param [options.height] The "height" of the arc. This option sets the {@link ArcEdgeStyle#height} property on the created object.
@param [options.fixedHeight] A value indicating whether this {@link ArcEdgeStyle} interprets the {@link ArcEdgeStyle#height} value as an absolute or relative value. This option sets the {@link ArcEdgeStyle#fixedHeight} property on the created object.
@param [options.provideHeightHandle] A value indicating whether the {@link ArcEdgeStyleRenderer} should provide an {@link IHandle} if queried for the {@link IHandleProvider} implementation that allows for adjusting the {@link ArcEdgeStyle#height} of this instance. This option sets the {@link ArcEdgeStyle#provideHeightHandle} property on the created object.
@param [options.targetArrow] The visual arrow at the target end of edges that use this style. This option sets the {@link ArcEdgeStyle#targetArrow} property on the created object.
@param [options.sourceArrow] The visual arrow at the source end of edges that use this style. This option sets the {@link ArcEdgeStyle#sourceArrow} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23ArcEdgeStyle(ArcEdgeStyleRenderer) Online Documentation}*/
constructor(options: {
renderer: ArcEdgeStyleRenderer;
stroke?: Stroke | null | StrokeConvertible;
height?: number;
fixedHeight?: boolean;
provideHeightHandle?: boolean;
targetArrow?: IArrow | IArrowConvertible;
sourceArrow?: IArrow | IArrowConvertible;
});
/**
Creates a new instance using {@link ArcEdgeStyleRenderer} as renderer.
@param [options] The parameters to pass.
@param [options.stroke] The {@link ArcEdgeStyle#stroke} that is used to draw the arc This option sets the {@link ArcEdgeStyle#stroke} property on the created object.
@param [options.height] The "height" of the arc. This option sets the {@link ArcEdgeStyle#height} property on the created object.
@param [options.fixedHeight] A value indicating whether this {@link ArcEdgeStyle} interprets the {@link ArcEdgeStyle#height} value as an absolute or relative value. This option sets the {@link ArcEdgeStyle#fixedHeight} property on the created object.
@param [options.provideHeightHandle] A value indicating whether the {@link ArcEdgeStyleRenderer} should provide an {@link IHandle} if queried for the {@link IHandleProvider} implementation that allows for adjusting the {@link ArcEdgeStyle#height} of this instance. This option sets the {@link ArcEdgeStyle#provideHeightHandle} property on the created object.
@param [options.targetArrow] The visual arrow at the target end of edges that use this style. This option sets the {@link ArcEdgeStyle#targetArrow} property on the created object.
@param [options.sourceArrow] The visual arrow at the source end of edges that use this style. This option sets the {@link ArcEdgeStyle#sourceArrow} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23ArcEdgeStyle() Online Documentation}*/
constructor(options?: {
stroke?: Stroke | null | StrokeConvertible;
height?: number;
fixedHeight?: boolean;
provideHeightHandle?: boolean;
targetArrow?: IArrow | IArrowConvertible;
sourceArrow?: IArrow | IArrowConvertible;
});
/**
Performs a {@link YObject#memberwiseClone}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23clone() Online Documentation}*/
clone(): this;
/**
Gets or sets the {@link ArcEdgeStyle#stroke} that is used to draw the arc
The default value is {@link Stroke#BLACK}.
@default 'System.Drawing.Pens.Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23stroke Online Documentation}*/
stroke: Stroke | null;
/**
Gets or sets the "height" of the arc.
Depending on the setting of {@link ArcEdgeStyle#fixedHeight}, this value is interpreted differently: If {@link ArcEdgeStyle#fixedHeight} is disabled, the height of the arc will depend
on it's width. The width will be multiplied by this value to obtain the height. If the {@link ArcEdgeStyle#fixedHeight} feature is enabled, this value
will be interpreted as the absolute height.
The default value is <code>0</code>.
@default 0
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23height Online Documentation}*/
height: number;
/**
Gets or sets a value indicating whether this {@link ArcEdgeStyle} interprets the {@link ArcEdgeStyle#height} value as an absolute or relative
value.
The default value is <code>true</code>.
@see ArcEdgeStyle#height
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23fixedHeight Online Documentation}*/
fixedHeight: boolean;
/**
Gets or sets a value indicating whether the {@link ArcEdgeStyleRenderer} should provide an {@link IHandle} if
queried for the {@link IHandleProvider} implementation that allows for adjusting the {@link ArcEdgeStyle#height} of this instance.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23provideHeightHandle Online Documentation}*/
provideHeightHandle: boolean;
/**
Gets or sets the visual arrow at the target end of edges that use this style.
Arrow instances may be shared between multiple style instances.
Note that changes to the properties of an arrow may not be immediately visible. It is therefore recommended to assign a
new arrow instance to this property if you want to change aspects of the target arrow visualization.
The default value is {@link IArrow#NONE}.
@default 'None'
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23targetArrow Online Documentation}*/
targetArrow: IArrow;
/**
Gets or sets the visual arrow at the source end of edges that use this style.
Arrow instances may be shared between multiple style instances.
Note that changes to the properties of an arrow may not be immediately visible. It is therefore recommended to assign a
new arrow instance to this property if you want to change aspects of the source arrow visualization.
The default value is {@link IArrow#NONE}.
@default 'None'
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23sourceArrow Online Documentation}*/
sourceArrow: IArrow;
/**
Gets the renderer implementation that has been supplied to this instance in the constructor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ArcEdgeStyle%23renderer Online Documentation}*/
readonly renderer: IEdgeStyleRenderer;
static $class: Class;
private $_ArcEdgeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ArcEdgeStyle;
}
export interface BevelNodeStyleRenderer extends YObject, INodeStyleRenderer, IShapeGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
@inheritdoc*/
getOutline(): GeneralPath | null;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isInside(location: Point): boolean;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
A {@link INodeStyle} {@link INodeStyleRenderer renderer} implementation that draws a rounded rectangle with a bevel border in a 'shiny plate'
fashion.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer Online Documentation}*/
export class BevelNodeStyleRenderer {
/**
Prepares this instance for subsequent calls after the style and node have been initialized.
Upon invocation the {@link BevelNodeStyleRenderer#style} and {@link BevelNodeStyleRenderer#node} properties have been populated by the {@link BevelNodeStyleRenderer#getVisualCreator},
{@link BevelNodeStyleRenderer#getBoundsProvider}, {@link BevelNodeStyleRenderer#getHitTestable}, or {@link BevelNodeStyleRenderer#getMarqueeTestable}
methods.
This is an empty implementation. Subclasses might have to override this method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Returns the layout bounds as an upper bound for the painting bound.
This implementation simply sets the {@link BevelNodeStyleRenderer#layout}'s bounds to the scratch variable and returns.
@param context the context to calculate the bounds for
@returns the bounds or <code>null</code> to indicate an unbound area
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23getBounds(ICanvasContext) Online Documentation}*/
getBounds(context: ICanvasContext): Rect;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link BevelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the bounds provider for. The value will be stored in the {@link BevelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link BevelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see BevelNodeStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link BevelNodeStyleRenderer#configure} and returns <code>this</code>.
As this method may be called often it will not automatically call {@link BevelNodeStyleRenderer#configure},
instead subclasses should ensure that in the {@link BevelNodeStyleRenderer#lookup} method call they should call {@link BevelNodeStyleRenderer#configure}
only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
@param node The node to query the context for. The value will be stored in the {@link BevelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link BevelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see BevelNodeStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link BevelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query hit test with. The value will be stored in the {@link BevelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link BevelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see BevelNodeStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link BevelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query marquee intersection tests. The value will be stored in the {@link BevelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link BevelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see BevelNodeStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
Configures the {@link BevelNodeStyleRenderer#style} and {@link BevelNodeStyleRenderer#node} properties, calls {@link BevelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the shape geometry for. The value will be stored in the {@link BevelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link BevelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see BevelNodeStyleRenderer#isInside
@see BevelNodeStyleRenderer#getIntersection
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link BevelNodeStyleRenderer#configure} and returns <code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link BevelNodeStyleRenderer#configure}. If the
subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link BevelNodeStyleRenderer#isVisible}.
@param node The node to query visibility tests. The value will be stored in the {@link BevelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link BevelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see BevelNodeStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link BevelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link BevelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link BevelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see BevelNodeStyleRenderer#createVisual
@see BevelNodeStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
Tests the {@link BevelNodeStyleRenderer#node node} using the current {@link BevelNodeStyleRenderer#style} for an intersection with the given box.
This implementation uses the {@link BevelNodeStyleRenderer#layout} to determine whether the node is in the box.
@param context the current input mode context
@param rectangle the box describing the marquee's bounds
@returns <code>true</code> if the node is considered to be captured by the marquee
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23isInBox(IInputModeContext%2CRect) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
Determines visibility by checking for an intersection with the {@link BevelNodeStyleRenderer#layout}.
@param context The context.
@param rectangle The current clip
@returns Whether the {@link BevelNodeStyleRenderer#layout} is visible in the clip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23style Online Documentation}*/
style: BevelNodeStyle;
/**
Gets or sets the currently configured node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23node Online Documentation}*/
node: INode;
/**
Gets the {@link INode#layout} of the node this renderer is currently configured for.
Returns <code>Node.Layout</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23layout Online Documentation}*/
readonly layout: IRectangle;
/**
Gets the color from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23color Online Documentation}*/
readonly color: Color;
/**
Gets the inset from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23inset Online Documentation}*/
readonly inset: number;
/**
Gets the radius to use from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23radius Online Documentation}*/
readonly radius: number;
/**
Gets or sets a property that determines whether this instance should provide an {@link IHandle} for editing the
radius of the shape.
The default is <code>false</code>. This implementation will add an appropriate {@link IHandleProvider} to the {@link BevelNodeStyleRenderer#lookup}
of this instance if this feature is enabled.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyleRenderer%23provideRadiusHandle Online Documentation}*/
provideRadiusHandle: boolean;
static $class: Class;
private $_BevelNodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BevelNodeStyleRenderer;
}
export interface BevelNodeStyle extends YObject, INodeStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
An implementation of the {@link INodeStyle} that can be used to create rectangular nodes with rounded corners, a
bevel border, and a shining background.
Optionally, this style renders a drop shadow by itself.
This style uses the {@link BevelNodeStyleRenderer} to visualize a node.
@see BevelNodeStyleRenderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyle Online Documentation}*/
export class BevelNodeStyle {
/**
Creates a new instance using the provided renderer to share.
@param renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyle%23BevelNodeStyle(BevelNodeStyleRenderer) Online Documentation}*/
constructor(renderer: BevelNodeStyleRenderer);
/**
Creates a new instance using the provided renderer to share.
@param options The parameters to pass.
@param options.renderer
@param [options.inset] The insets to use for the bevel. This option sets the {@link BevelNodeStyle#inset} property on the created object.
@param [options.radius] The radius of the corner of the rounded rectangle. This option sets the {@link BevelNodeStyle#radius} property on the created object.
@param [options.color] The base color to use. This option sets the {@link BevelNodeStyle#color} property on the created object.
@param [options.drawShadow] A value indicating whether to draw a drop shadow. This option sets the {@link BevelNodeStyle#drawShadow} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyle%23BevelNodeStyle(BevelNodeStyleRenderer) Online Documentation}*/
constructor(options: {
renderer: BevelNodeStyleRenderer;
inset?: number;
radius?: number;
color?: Color | ColorConvertible;
drawShadow?: boolean;
});
/**
Creates a new instance using {@link BevelNodeStyleRenderer} as renderer to share.
@param [options] The parameters to pass.
@param [options.inset] The insets to use for the bevel. This option sets the {@link BevelNodeStyle#inset} property on the created object.
@param [options.radius] The radius of the corner of the rounded rectangle. This option sets the {@link BevelNodeStyle#radius} property on the created object.
@param [options.color] The base color to use. This option sets the {@link BevelNodeStyle#color} property on the created object.
@param [options.drawShadow] A value indicating whether to draw a drop shadow. This option sets the {@link BevelNodeStyle#drawShadow} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyle%23BevelNodeStyle() Online Documentation}*/
constructor(options?: {
inset?: number;
radius?: number;
color?: Color | ColorConvertible;
drawShadow?: boolean;
});
/**
Gets or sets the insets to use for the bevel.
The default value is <code>3</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyle%23inset Online Documentation}*/
inset: number;
/**
Gets or sets the radius of the corner of the rounded rectangle.
The default value is <code>10</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyle%23radius Online Documentation}*/
radius: number;
/**
Gets or sets the base color to use.
The default value is {@link Color#BLACK}.
@default 'Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyle%23color Online Documentation}*/
color: Color;
/**
Gets or sets a value indicating whether to draw a drop shadow.
The default value is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/BevelNodeStyle%23drawShadow Online Documentation}*/
drawShadow: boolean;
static $class: Class;
private $_BevelNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BevelNodeStyle;
}
export interface BezierEdgeStyle extends YObject, IEdgeStyle {
}
/**
Provides an edge representation as a cubic bezier path.
The rendering is composed of a sequence of cubic bezier segments. The bends together with the source and target port
locations of the edge serve as control points for this path:
<ul>
<li>If there are only two control points (i.e. just source and target port and no bends), a straight line is rendered
<li>Otherwise: the number of control points is internally padded to be 1 (mod 3) by duplicating the target port location, if
necessary
<li>Consecutive 4-tuples of control points describe a cubic segment of the path
<li>Each consecutive pair of 4-tuples share their last resp. first control point.
<li>Control points are <b>not</b> automatically aligned to form smooth connections between segments.
</ul>
This style uses an {@link BezierEdgeStyleRenderer} for its visualization.
This style disables interactive bend creation and orthogonal editing and does not add bridges to its path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyle Online Documentation}*/
export class BezierEdgeStyle {
/**
Creates a new instance using the provided renderer.
@param renderer The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyle%23BezierEdgeStyle(BezierEdgeStyleRenderer) Online Documentation}*/
constructor(renderer: BezierEdgeStyleRenderer);
/**
Creates a new instance using the provided renderer.
@param options The parameters to pass.
@param options.renderer The custom renderer for this style instance.
@param [options.stroke] The Pen that is used to draw the path. This option sets the {@link BezierEdgeStyle#stroke} property on the created object.
@param [options.targetArrow] The visual arrow at the target end of edges that use this style. This option sets the {@link BezierEdgeStyle#targetArrow} property on the created object.
@param [options.sourceArrow] The visual arrow at the source end of edges that use this style. This option sets the {@link BezierEdgeStyle#sourceArrow} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyle%23BezierEdgeStyle(BezierEdgeStyleRenderer) Online Documentation}*/
constructor(options: {
renderer: BezierEdgeStyleRenderer;
stroke?: Stroke | null | StrokeConvertible;
targetArrow?: IArrow | IArrowConvertible;
sourceArrow?: IArrow | IArrowConvertible;
});
/**
Creates a new instance using {@link BezierEdgeStyleRenderer} as renderer.
@param [options] The parameters to pass.
@param [options.stroke] The Pen that is used to draw the path. This option sets the {@link BezierEdgeStyle#stroke} property on the created object.
@param [options.targetArrow] The visual arrow at the target end of edges that use this style. This option sets the {@link BezierEdgeStyle#targetArrow} property on the created object.
@param [options.sourceArrow] The visual arrow at the source end of edges that use this style. This option sets the {@link BezierEdgeStyle#sourceArrow} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyle%23BezierEdgeStyle() Online Documentation}*/
constructor(options?: {
stroke?: Stroke | null | StrokeConvertible;
targetArrow?: IArrow | IArrowConvertible;
sourceArrow?: IArrow | IArrowConvertible;
});
/**
Performs a {@link YObject#memberwiseClone}
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyle%23clone() Online Documentation}*/
clone(): this;
/**
Gets or sets the Pen that is used to draw the path.
The default value is {@link Stroke#BLACK}.
@default 'System.Drawing.Pens.Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyle%23stroke Online Documentation}*/
stroke: Stroke | null;
/**
Gets or sets the visual arrow at the target end of edges that use this style.
Arrow instances may be shared between multiple style instances.
Note that changes to the properties of an arrow may not be immediately visible. It is therefore recommended to assign a
new arrow instance to this property if you want to change aspects of the target arrow visualization.
The default value is {@link IArrow#NONE}.
@default 'None'
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyle%23targetArrow Online Documentation}*/
targetArrow: IArrow;
/**
Gets or sets the visual arrow at the source end of edges that use this style.
Arrow instances may be shared between multiple style instances.
Note that changes to the properties of an arrow may not be immediately visible. It is therefore recommended to assign a
new arrow instance to this property if you want to change aspects of the source arrow visualization.
The default value is {@link IArrow#NONE}.
@default 'None'
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyle%23sourceArrow Online Documentation}*/
sourceArrow: IArrow;
/**
Gets the renderer implementation that has been supplied to this instance in the constructor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyle%23renderer Online Documentation}*/
readonly renderer: IEdgeStyleRenderer;
static $class: Class;
private $_BezierEdgeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BezierEdgeStyle;
}
export interface BezierEdgeStyleRenderer extends PathBasedEdgeStyleRenderer<BezierEdgeStyle> {
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
getSourceArrow(): IArrow | null;
/**
@inheritdoc*/
getStroke(): Stroke | null;
/**
@inheritdoc*/
getTangent(ratio: number): Tangent | null;
/**
@inheritdoc*/
getTangentForSegment(segmentIndex: number, ratio: number): Tangent | null;
/**
@inheritdoc*/
getTargetArrow(): IArrow | null;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
} /**
Default renderer implementation for {@link BezierEdgeStyle}
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyleRenderer Online Documentation}*/
export class BezierEdgeStyleRenderer {
constructor();
/**
Converts the edge bends into a cubic bezier path.
The rendering is composed of a sequence of cubic bezier segments. The bends together with the source and target port
locations of the edge serve as control points for this path:
<ul>
<li>If there are only two control points (i.e. just source and target port and no bends), a straight line is rendered
<li>Otherwise: the number of control points is internally padded to be 1 (mod 3) by duplicating the target port location, if
necessary
<li>Consecutive 4-tuples of control points describe a cubic segment of the path
<li>Each consecutive pair of 4-tuples share their last resp. first control point.
<li>Control points are <b>not</b> automatically aligned to form smooth connections between segments.
</ul>
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyleRenderer%23createPath() Online Documentation}*/
createPath(): GeneralPath;
/**
Returns the number of line, cubic and close operations in the renderer's path
@returns The number of line, cubic and close operations in the renderer's path, or 0 if no path exists.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyleRenderer%23getSegmentCount() Online Documentation}*/
getSegmentCount(): number;
/**
Overridden to always return <code>false</code>, since no bridges can be added in a meainingful way to a bezier path.
The bezier path can still generate obstacles for <b>other</b> edges.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BezierEdgeStyleRenderer%23addBridges Online Documentation}*/
readonly addBridges: boolean;
static $class: Class;
private $_BezierEdgeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BezierEdgeStyleRenderer;
}
export interface ImageNodeStyleRenderer extends YObject, INodeStyleRenderer, IShapeGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, ILassoTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
A style renderer implementation that can be used together with {@link ImageNodeStyle} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer Online Documentation}*/
export class ImageNodeStyleRenderer {
constructor();
/**
Prepares this instance for subsequent calls after the style and node have been initialized.
Upon invocation the {@link ImageNodeStyleRenderer#style} and {@link ImageNodeStyleRenderer#node} properties have been populated by the {@link ImageNodeStyleRenderer#getVisualCreator},
{@link ImageNodeStyleRenderer#getBoundsProvider}, {@link ImageNodeStyleRenderer#getHitTestable}, or {@link ImageNodeStyleRenderer#getMarqueeTestable}
methods.
This is an empty implementation. Subclasses might have to override this method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Returns the layout bounds as an upper bound for the painting bound.
This implementation simply sets the {@link ImageNodeStyleRenderer#layout}'s bounds to the scratch variable and returns.
@param context the context to calculate the bounds for
@returns the bounds or <code>null</code> to indicate an unbound area
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getBounds(ICanvasContext) Online Documentation}*/
getBounds(context: ICanvasContext): Rect;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ImageNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the bounds provider for. The value will be stored in the {@link ImageNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ImageNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ImageNodeStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link ImageNodeStyleRenderer#configure} and returns <code>this</code>.
As this method may be called often it will not automatically call {@link ImageNodeStyleRenderer#configure},
instead subclasses should ensure that in the {@link ImageNodeStyleRenderer#lookup} method call they should call {@link ImageNodeStyleRenderer#configure}
only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
@param node The node to query the context for. The value will be stored in the {@link ImageNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ImageNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ImageNodeStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
Get the style's fallback image.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getFallbackImage() Online Documentation}*/
getFallbackImage(): string | null;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ImageNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query hit test with. The value will be stored in the {@link ImageNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ImageNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ImageNodeStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Get the image that is currently used to render the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getImage() Online Documentation}*/
getImage(): string;
/**
Tries to use the {@link ImageNodeStyle#normalizedOutline} to perform the intersection calculation, otherwise reverts to default (rectangular) behavior.
@param inner
@param outer
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getIntersection(Point%2CPoint) Online Documentation}*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ImageNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query marquee intersection tests. The value will be stored in the {@link ImageNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ImageNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ImageNodeStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
Tries to use the {@link ImageNodeStyle#normalizedOutline} to create the outline shape path, otherwise reverts to default (rectangular) behavior.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getOutline() Online Documentation}*/
getOutline(): GeneralPath | null;
/**
Gets the value that determines whether the aspect ratio of the image should be preserved.
@returns false
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getPreserveAspectRatio() Online Documentation}*/
getPreserveAspectRatio(): boolean;
/**
Configures the {@link ImageNodeStyleRenderer#style} and {@link ImageNodeStyleRenderer#node} properties, calls {@link ImageNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the shape geometry for. The value will be stored in the {@link ImageNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ImageNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ImageNodeStyleRenderer#isInside
@see ImageNodeStyleRenderer#getIntersection
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link ImageNodeStyleRenderer#configure} and returns <code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link ImageNodeStyleRenderer#configure}. If the
subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link ImageNodeStyleRenderer#isVisible}.
@param node The node to query visibility tests. The value will be stored in the {@link ImageNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ImageNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ImageNodeStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ImageNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link ImageNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ImageNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ImageNodeStyleRenderer#createVisual
@see ImageNodeStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
Tries to use {@link ImageNodeStyle#normalizedOutline} to perform the hit test analysis, otherwise reverts to default (rectangular) behavior.
@param context the context the hit test is performed in
@param location the coordinates in world coordinate system
@returns whether something has been hit
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23isHit(IInputModeContext%2CPoint) Online Documentation}*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
Tries to use the {@link ImageNodeStyle#normalizedOutline} to perform the marquee intersection analysis, otherwise reverts to default (rectangular) behavior.
@param context
@param rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23isInBox(IInputModeContext%2CRect) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
Tries to use the {@link ImageNodeStyle#normalizedOutline} to perform the lasso intersection analysis, otherwise reverts to default (rectangular) behavior.
@param context
@param lassoPath
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23isInPath(IInputModeContext%2CGeneralPath) Online Documentation}*/
isInPath(context: IInputModeContext, lassoPath: GeneralPath): boolean;
/**
Tries to use the {@link ImageNodeStyle#normalizedOutline} to perform the contains test, otherwise reverts to default (rectangular) behavior.
@param location
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23isInside(Point) Online Documentation}*/
isInside(location: Point): boolean;
/**
Determines visibility by checking for an intersection with the {@link ImageNodeStyleRenderer#layout}.
@param context The context.
@param rectangle The current clip
@returns Whether the {@link ImageNodeStyleRenderer#layout} is visible in the clip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Implements the {@link ILookup} interface.
This method will be used by default if {@link ImageNodeStyleRenderer#getContext} has been queried for a lookup
implementation. Note that it cannot be assumed that {@link ImageNodeStyleRenderer#configure} has already been
invoked. However, normally {@link ImageNodeStyleRenderer#node} and {@link ImageNodeStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link ImageNodeStyleRenderer#configure}
on success and return <code>this</code>.
@param type The type to query for.
@returns An implementation or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23style Online Documentation}*/
style: ImageNodeStyle;
/**
Gets or sets the currently configured node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23node Online Documentation}*/
node: INode;
/**
Gets the {@link INode#layout} of the node this renderer is currently configured for.
Returns <code>Node.Layout</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23layout Online Documentation}*/
readonly layout: IRectangle;
/**
A {@link string} that will be used to find the fallback image url that will be used to render the node, if the style does not
explicitly specify a {@link ImageNodeStyle#fallbackImage fallback image}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyleRenderer%23TEMPLATE_KEY Online Documentation}*/
static TEMPLATE_KEY: string;
static $class: Class;
private $_ImageNodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ImageNodeStyleRenderer;
}
export interface ImageNodeStyle extends YObject, INodeStyle {
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
A node style that can use an image for the visual representation of a node.
This style can be used together with {@link ImageNodeStyleRenderer} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyle Online Documentation}*/
export class ImageNodeStyle {
/**
Creates a new instance that uses the specified image URL, fallback image URL and the given renderer instance.
{@link ImageNodeStyle} uses the SVG &lt;image> element to render images and therefore supports all image types that
are supported by the user's browser. The <code>image</code> is used as the href attribute of the &lt;image> element. If the <code>image</code> cannot
be loaded, the <code>fallbackImage</code> is used instead. Both should either be a valid absolute URL, or an URL that is relative to
the webpage that contains the {@link GraphComponent}, or null.
@param [image=null] The image URL.
@param [fallbackImage=null] The fallback image URL.
@param [renderer=null] The renderer used by this instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyle Online Documentation}*/
constructor(image?: string | null, fallbackImage?: string | null, renderer?: ImageNodeStyleRenderer);
/**
Creates a new instance that uses the specified image URL, fallback image URL and the given renderer instance.
{@link ImageNodeStyle} uses the SVG &lt;image> element to render images and therefore supports all image types that
are supported by the user's browser. The <code>image</code> is used as the href attribute of the &lt;image> element. If the <code>image</code> cannot
be loaded, the <code>fallbackImage</code> is used instead. Both should either be a valid absolute URL, or an URL that is relative to
the webpage that contains the {@link GraphComponent}, or null.
@param options The parameters to pass.
@param [options.image=null] The image URL.
@param [options.fallbackImage=null] The fallback image URL.
@param [options.renderer=null] The renderer used by this instance
@param [options.normalizedOutline] The normalized outline of the image that will be displayed. This option sets the {@link ImageNodeStyle#normalizedOutline} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyle Online Documentation}*/
constructor(options: {
image?: string | null;
fallbackImage?: string | null;
renderer?: ImageNodeStyleRenderer;
normalizedOutline?: GeneralPath | null;
});
/**
This implementation performs a shallow copy with respect to the {@link ImageNodeStyle#image} property.
If clients need to have a deep copy of this instance, they need to manually clone and reassign the {@link ImageNodeStyle#image} to the clone.
@returns A shallow copy of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyle%23clone() Online Documentation}*/
clone(): this;
/**
Gets or sets the image that is used by the style instance.
The default value is <code>null</code>.
{@link ImageNodeStyle} uses the SVG &lt;image> element to render images and therefore supports all image types that
are supported by the user's browser. The {@link ImageNodeStyle#image} property of the style is used as the href attribute of the image. It should
either be a valid absolute URL, or an URL that is relative to the webpage that contains the
{@link GraphComponent}, or null.
If the {@link ImageNodeStyle#image} fails to load, the {@link ImageNodeStyle#fallbackImage} is used instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyle%23image Online Documentation}*/
image: string | null;
/**
Gets or sets the fallback image that is used by the style instance if the {@link ImageNodeStyle#image} is null.
The default value is <code>null</code>.
{@link ImageNodeStyle} uses the SVG &lt;image> element to render images and therefore supports all image types that
are supported by the user's browser. The {@link ImageNodeStyle#fallbackImage} property of the style is used as the href attribute of the image if the {@link ImageNodeStyle#image} fails
to load. It should either be a valid absolute URL, or an URL that is relative to the webpage that contains the
{@link GraphComponent}, or null.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyle%23fallbackImage Online Documentation}*/
fallbackImage: string | null;
/**
Gets or sets the normalized outline of the image that will be displayed.
This shape will be used to satisfy requests to {@link IMarqueeTestable#isInBox} and {@link IHitTestable#isHit}
and may be left <code>null</code> to indicate default (rectangular) behavior.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/ImageNodeStyle%23normalizedOutline Online Documentation}*/
normalizedOutline: GeneralPath | null;
static $class: Class;
private $_ImageNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ImageNodeStyle;
}
export interface PanelNodeStyle extends YObject, INodeStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
An implementation of {@link INodeStyle} that draws a simple floating panel with a slight gradient, a thin border
and optionally a drop shadow.
This style can be used to create the visual representation for group nodes, for example. By default, it a drop shadow by
itself.
This style uses the {@link PanelNodeStyleRenderer} to visualize a node.
@see PanelNodeStyleRenderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyle Online Documentation}*/
export class PanelNodeStyle {
/**
Creates a new instance using the provided renderer to share.
@param renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyle%23PanelNodeStyle(PanelNodeStyleRenderer) Online Documentation}*/
constructor(renderer: PanelNodeStyleRenderer);
/**
Creates a new instance using the provided renderer to share.
@param options The parameters to pass.
@param options.renderer
@param [options.color] The base color to use. This option sets the {@link PanelNodeStyle#color} property on the created object.
@param [options.insets] The instance to use that provides the insets for this style. This option sets the {@link PanelNodeStyle#insets} property on the created object.
@param [options.labelInsetsColor] The base color to use for drawing the label insets background. This option sets the {@link PanelNodeStyle#labelInsetsColor} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyle%23PanelNodeStyle(PanelNodeStyleRenderer) Online Documentation}*/
constructor(options: {
renderer: PanelNodeStyleRenderer;
color?: Color | ColorConvertible;
insets?: Insets | InsetsConvertible;
labelInsetsColor?: Color | ColorConvertible;
});
/**
Creates a new instance using {@link PanelNodeStyleRenderer} as renderer to share.
@param [options] The parameters to pass.
@param [options.color] The base color to use. This option sets the {@link PanelNodeStyle#color} property on the created object.
@param [options.insets] The instance to use that provides the insets for this style. This option sets the {@link PanelNodeStyle#insets} property on the created object.
@param [options.labelInsetsColor] The base color to use for drawing the label insets background. This option sets the {@link PanelNodeStyle#labelInsetsColor} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyle%23PanelNodeStyle() Online Documentation}*/
constructor(options?: {
color?: Color | ColorConvertible;
insets?: Insets | InsetsConvertible;
labelInsetsColor?: Color | ColorConvertible;
});
/**
Gets or sets the base color to use.
The default value is {@link Color#BLACK}.
@default 'Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyle%23color Online Documentation}*/
color: Color;
/**
Gets or sets the base color to use for drawing the label insets background.
Setting this to <code>null</code> effectively disables label insets background coloring.
The default value is {@link Color#LIGHT_GRAY}.
@default 'LightGray'
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyle%23labelInsetsColor Online Documentation}*/
labelInsetsColor: Color;
/**
Gets or sets the instance to use that provides the insets for this style.
The {@link PanelNodeStyleRenderer} will use these insets and return them via an {@link INodeInsetsProvider}
if such an instance is queried through the {@link INodeStyleRenderer#getContext context lookup}.
The default insets are set to (5,5,5,5).
@see INodeInsetsProvider
@default '5, 5, 5, 5'
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyle%23insets Online Documentation}*/
insets: Insets;
static $class: Class;
private $_PanelNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PanelNodeStyle;
}
export interface PanelNodeStyleRenderer extends YObject, INodeStyleRenderer, IShapeGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
A {@link INodeStyle} {@link INodeStyleRenderer renderer} implementation that draws a simple floating panel with a slight gradient, a thin
border and a simple drop shadow.
@see PanelNodeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer Online Documentation}*/
export class PanelNodeStyleRenderer {
/**
Prepares this instance for subsequent calls after the style and node have been initialized.
Upon invocation the {@link PanelNodeStyleRenderer#style} and {@link PanelNodeStyleRenderer#node} properties have been populated by the {@link PanelNodeStyleRenderer#getVisualCreator},
{@link PanelNodeStyleRenderer#getBoundsProvider}, {@link PanelNodeStyleRenderer#getHitTestable}, or {@link PanelNodeStyleRenderer#getMarqueeTestable}
methods.
This is an empty implementation. Subclasses might have to override this method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Determines whether to draw the drop shadow.
@returns <code>true</code>, always.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23drawShadow() Online Documentation}*/
drawShadow(): boolean;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link PanelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the bounds provider for. The value will be stored in the {@link PanelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link PanelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see PanelNodeStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link PanelNodeStyleRenderer#configure} and returns <code>this</code>.
As this method may be called often it will not automatically call {@link PanelNodeStyleRenderer#configure},
instead subclasses should ensure that in the {@link PanelNodeStyleRenderer#lookup} method call they should call {@link PanelNodeStyleRenderer#configure}
only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
@param node The node to query the context for. The value will be stored in the {@link PanelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link PanelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see PanelNodeStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link PanelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query hit test with. The value will be stored in the {@link PanelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link PanelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see PanelNodeStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Finds an intersection for the current {@link PanelNodeStyleRenderer#node node} using the {@link PanelNodeStyleRenderer#style}.
This implementation returns the intersection for the {@link PanelNodeStyleRenderer#layout}.
@param inner
@param outer
@returns True if an intersection was actually found
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23getIntersection(Point%2CPoint) Online Documentation}*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link PanelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query marquee intersection tests. The value will be stored in the {@link PanelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link PanelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see PanelNodeStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
Returns the outline of the shape using the node's {@link PanelNodeStyleRenderer#layout}.
@returns The outline of the bounds of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23getOutline() Online Documentation}*/
getOutline(): GeneralPath | null;
/**
Configures the {@link PanelNodeStyleRenderer#style} and {@link PanelNodeStyleRenderer#node} properties, calls {@link PanelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the shape geometry for. The value will be stored in the {@link PanelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link PanelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see PanelNodeStyleRenderer#isInside
@see PanelNodeStyleRenderer#getIntersection
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link PanelNodeStyleRenderer#configure} and returns <code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link PanelNodeStyleRenderer#configure}. If the
subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link PanelNodeStyleRenderer#isVisible}.
@param node The node to query visibility tests. The value will be stored in the {@link PanelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link PanelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see PanelNodeStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link PanelNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link PanelNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link PanelNodeStyleRenderer#style} property.
@returns <code>this</code>
@see PanelNodeStyleRenderer#createVisual
@see PanelNodeStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
Hit tests the {@link PanelNodeStyleRenderer#node node} using the current {@link PanelNodeStyleRenderer#style}.
This implementation uses the {@link PanelNodeStyleRenderer#layout} to determine whether the node has been hit. The check is delegated to {@link Rect#containsWithEps}
using {@link ICanvasContext#hitTestRadius} as the last argument.
@param context the context the hit test is performed in
@param location the coordinates in world coordinate system
@returns whether something has been hit
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23isHit(IInputModeContext%2CPoint) Online Documentation}*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
Tests the {@link PanelNodeStyleRenderer#node node} using the current {@link PanelNodeStyleRenderer#style} for an intersection with the given box.
This implementation uses the {@link PanelNodeStyleRenderer#layout} to determine whether the node is in the box.
@param context the current input mode context
@param rectangle the box describing the marquee's bounds
@returns <code>true</code> if the node is considered to be captured by the marquee
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23isInBox(IInputModeContext%2CRect) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
Checks whether the given coordinate lies within the shape's geometric bounds.
This implementation checks for containment in the {@link PanelNodeStyleRenderer#layout}.
@param location
@returns True if the point lies within the geometry of the rendering.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23isInside(Point) Online Documentation}*/
isInside(location: Point): boolean;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23style Online Documentation}*/
style: PanelNodeStyle;
/**
Gets or sets the currently configured node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23node Online Documentation}*/
node: INode;
/**
Gets the {@link INode#layout} of the node this renderer is currently configured for.
Returns <code>Node.Layout</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23layout Online Documentation}*/
readonly layout: IRectangle;
/**
Gets the color from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PanelNodeStyleRenderer%23color Online Documentation}*/
readonly color: Color;
static $class: Class;
private $_PanelNodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PanelNodeStyleRenderer;
}
export interface ShadowNodeStyleDecorator extends YObject, INodeStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
Decorates other node styles with a smooth drop shadow.
This implementation uses an SVG filter effect. This may cause performance issues for larger nodes or high zoom factors.
In this case it might be more appropriate to implement a drop shadow implementation using canvas drawing and displaying
the canvas content using an <code>image</code> element with <code>canvas.toDataURL</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShadowNodeStyleDecorator Online Documentation}*/
export class ShadowNodeStyleDecorator {
/**
Creates a new instance wrapping the provided style.
The provided instance is used by reference, so subsequent changes to the style will affect the appearance of this
style.
@param [wrapped=null] The style to wrap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShadowNodeStyleDecorator Online Documentation}*/
constructor(wrapped?: INodeStyle);
/**
Gets or sets the wrapped node style instance by reference.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShadowNodeStyleDecorator%23wrapped Online Documentation}*/
wrapped: INodeStyle;
static $class: Class;
private $_ShadowNodeStyleDecorator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ShadowNodeStyleDecorator;
}
export interface ShinyPlateNodeStyle extends YObject, INodeStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
Implementation of the {@link INodeStyle} which creates rectangular nodes with rounded corners and a highlighted
('shiny plate') interior.
By default, this style renders a drop shadow by itself.
This style uses the {@link ShinyPlateNodeStyleRenderer} to visualize a node.
@see ShinyPlateNodeStyleRenderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyle Online Documentation}*/
export class ShinyPlateNodeStyle {
/**
Creates a new instance using the provided renderer.
@param renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyle%23ShinyPlateNodeStyle(ShinyPlateNodeStyleRenderer) Online Documentation}*/
constructor(renderer: ShinyPlateNodeStyleRenderer);
/**
Creates a new instance using the provided renderer.
@param options The parameters to pass.
@param options.renderer
@param [options.fill] The background brush for this style. This option sets the {@link ShinyPlateNodeStyle#fill} property on the created object.
@param [options.drawShadow] A value indicating whether to draw a drop shadow. This option sets the {@link ShinyPlateNodeStyle#drawShadow} property on the created object.
@param [options.insets] The instance to use that provides the insets for this style. This option sets the {@link ShinyPlateNodeStyle#insets} property on the created object.
@param [options.stroke] The {@link ShinyPlateNodeStyle#stroke} to use for the outline. This option sets the {@link ShinyPlateNodeStyle#stroke} property on the created object.
@param [options.radius] The corner radius of the rounded rectangle This option sets the {@link ShinyPlateNodeStyle#radius} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyle%23ShinyPlateNodeStyle(ShinyPlateNodeStyleRenderer) Online Documentation}*/
constructor(options: {
renderer: ShinyPlateNodeStyleRenderer;
fill?: Fill | null | FillConvertible;
drawShadow?: boolean;
insets?: Insets | InsetsConvertible;
stroke?: Stroke | null | StrokeConvertible;
radius?: number;
});
/**
Creates a new instance using {@link ShinyPlateNodeStyleRenderer} as renderer.
@param [options] The parameters to pass.
@param [options.fill] The background brush for this style. This option sets the {@link ShinyPlateNodeStyle#fill} property on the created object.
@param [options.drawShadow] A value indicating whether to draw a drop shadow. This option sets the {@link ShinyPlateNodeStyle#drawShadow} property on the created object.
@param [options.insets] The instance to use that provides the insets for this style. This option sets the {@link ShinyPlateNodeStyle#insets} property on the created object.
@param [options.stroke] The {@link ShinyPlateNodeStyle#stroke} to use for the outline. This option sets the {@link ShinyPlateNodeStyle#stroke} property on the created object.
@param [options.radius] The corner radius of the rounded rectangle This option sets the {@link ShinyPlateNodeStyle#radius} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyle%23ShinyPlateNodeStyle() Online Documentation}*/
constructor(options?: {
fill?: Fill | null | FillConvertible;
drawShadow?: boolean;
insets?: Insets | InsetsConvertible;
stroke?: Stroke | null | StrokeConvertible;
radius?: number;
});
/**
Gets or sets the background brush for this style.
The default value is {@link Fill#BLACK}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyle%23fill Online Documentation}*/
fill: Fill | null;
/**
Gets or sets the {@link ShinyPlateNodeStyle#stroke} to use for the outline.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyle%23stroke Online Documentation}*/
stroke: Stroke | null;
/**
Gets or sets the corner radius of the rounded rectangle
The default value is <code>5</code>.
@default 5
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyle%23radius Online Documentation}*/
radius: number;
/**
Gets or sets a value indicating whether to draw a drop shadow.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyle%23drawShadow Online Documentation}*/
drawShadow: boolean;
/**
Gets or sets the instance to use that provides the insets for this style.
The {@link ShinyPlateNodeStyleRenderer} will use these insets and return them via an {@link INodeInsetsProvider}
if such an instance is queried through the {@link INodeStyleRenderer#getContext context lookup}.
The default insets are set to (5,5,5,5).
@see INodeInsetsProvider
@default '5,5,5,5'
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyle%23insets Online Documentation}*/
insets: Insets;
static $class: Class;
private $_ShinyPlateNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ShinyPlateNodeStyle;
}
export interface ShinyPlateNodeStyleRenderer extends YObject, INodeStyleRenderer, IShapeGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
createVisual(context: IRenderContext): Visual | null;
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
@inheritdoc*/
getOutline(): GeneralPath | null;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isInside(location: Point): boolean;
/**
@inheritdoc*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
lookup(type: Class): Object | null;
/**
@inheritdoc*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
} /**
A {@link INodeStyle} {@link INodeStyleRenderer renderer} implementation that draws a simple floating shinyPlate with a slight gradient, a
thin border and a simple drop shadow.
@see ShinyPlateNodeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer Online Documentation}*/
export class ShinyPlateNodeStyleRenderer {
/**
Prepares this instance for subsequent calls after the style and node have been initialized.
Upon invocation the {@link ShinyPlateNodeStyleRenderer#style} and {@link ShinyPlateNodeStyleRenderer#node} properties have been populated by the
{@link ShinyPlateNodeStyleRenderer#getVisualCreator}, {@link ShinyPlateNodeStyleRenderer#getBoundsProvider},
{@link ShinyPlateNodeStyleRenderer#getHitTestable}, or {@link ShinyPlateNodeStyleRenderer#getMarqueeTestable}
methods.
Subclasses which override this method must call the base method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Determines whether to draw the drop shadow.
@returns {@link ShinyPlateNodeStyle#drawShadow}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23drawShadow() Online Documentation}*/
drawShadow(): boolean;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the bounds provider for. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShinyPlateNodeStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link ShinyPlateNodeStyleRenderer#configure} and returns
<code>this</code>.
As this method may be called often it will not automatically call {@link ShinyPlateNodeStyleRenderer#configure},
instead subclasses should ensure that in the {@link ShinyPlateNodeStyleRenderer#lookup} method call they should
call {@link ShinyPlateNodeStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an
instance that depends on a correctly configured <code>this</code>.
@param node The node to query the context for. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShinyPlateNodeStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query hit test with. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShinyPlateNodeStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to query marquee intersection tests. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShinyPlateNodeStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
Configures the {@link ShinyPlateNodeStyleRenderer#style} and {@link ShinyPlateNodeStyleRenderer#node} properties, calls {@link ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the shape geometry for. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShinyPlateNodeStyleRenderer#isInside
@see ShinyPlateNodeStyleRenderer#getIntersection
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link ShinyPlateNodeStyleRenderer#configure} and returns
<code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link ShinyPlateNodeStyleRenderer#configure}. If
the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link ShinyPlateNodeStyleRenderer#isVisible}.
@param node The node to query visibility tests. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShinyPlateNodeStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link ShinyPlateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see ShinyPlateNodeStyleRenderer#createVisual
@see ShinyPlateNodeStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
Tests the {@link ShinyPlateNodeStyleRenderer#node node} using the current {@link ShinyPlateNodeStyleRenderer#style} for an intersection with the given box.
This implementation uses the {@link ShinyPlateNodeStyleRenderer#layout} to determine whether the node is in the box.
@param context the current input mode context
@param rectangle the box describing the marquee's bounds
@returns <code>true</code> if the node is considered to be captured by the marquee
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23isInBox(IInputModeContext%2CRect) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23style Online Documentation}*/
style: ShinyPlateNodeStyle;
/**
Gets or sets the currently configured node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23node Online Documentation}*/
node: INode;
/**
Gets the {@link INode#layout} of the node this renderer is currently configured for.
Returns <code>Node.Layout</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23layout Online Documentation}*/
readonly layout: IRectangle;
/**
Gets the {@link ShinyPlateNodeStyleRenderer#fill} from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23fill Online Documentation}*/
readonly fill: Fill | null;
/**
Gets the {@link ShinyPlateNodeStyleRenderer#stroke} from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23stroke Online Documentation}*/
readonly stroke: Stroke | null;
/**
Gets the radius from the style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShinyPlateNodeStyleRenderer%23radius Online Documentation}*/
readonly radius: number;
static $class: Class;
private $_ShinyPlateNodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ShinyPlateNodeStyleRenderer;
}
export interface WebGLImageNodeStyle extends YObject, INodeStyle {
/**
@inheritdoc*/
renderer: INodeStyleRenderer;
} /**
A node style that uses WebGL rendering and displays an image for the visual representation of a node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLImageNodeStyle Online Documentation}*/
export class WebGLImageNodeStyle {
/**
Creates a new instance that uses the specified image.
@param [image=null] The image source, i.e. everything that one could use to specify the <code>src</code> of an &lt;img> element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLImageNodeStyle Online Documentation}*/
constructor(image?: string | null);
/**
Creates a new instance that uses the specified image.
@param options The parameters to pass.
@param [options.image=null] The image source, i.e. everything that one could use to specify the <code>src</code> of an &lt;img> element.
@param [options.normalizedOutline] The normalized outline of the image that will be displayed. This option sets the {@link WebGLImageNodeStyle#normalizedOutline} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLImageNodeStyle Online Documentation}*/
constructor(options: {
image?: string | null;
normalizedOutline?: GeneralPath | null;
});
/**
This implementation performs a shallow copy with respect to the {@link WebGLImageNodeStyle#image} property.
If clients need to have a deep copy of this instance, they need to manually clone and reassign the {@link WebGLImageNodeStyle#image} to the clone.
@returns A shallow copy of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLImageNodeStyle%23clone() Online Documentation}*/
clone(): this;
/**
Gets or sets the image reference.
The value can be everything that one could use to specify the <code>src</code> of an &lt;img> element. Note that the reference must
adhere to the same-origin policy. The default value is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLImageNodeStyle%23image Online Documentation}*/
image: string | null;
/**
Gets or sets the normalized outline of the image that will be displayed.
This shape will be used to satisfy requests to {@link IMarqueeTestable#isInBox} and {@link IHitTestable#isHit}
and may be left <code>null</code> to indicate default (rectangular) behavior.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLImageNodeStyle%23normalizedOutline Online Documentation}*/
normalizedOutline: GeneralPath | null;
static $class: Class;
private $_WebGLImageNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WebGLImageNodeStyle;
}
export interface WebGLPolylineEdgeStyle extends YObject, IEdgeStyle {
}
/**
An edge style that uses WebGL rendering and displays edges as polygonal straight line segments that connect the bends.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle Online Documentation}*/
export class WebGLPolylineEdgeStyle {
/**
Creates a new instance of this edge style.
@param [options] The parameters to pass.
@param [options.color] The {@link WebGLPolylineEdgeStyle#color} of the line. This option sets the {@link WebGLPolylineEdgeStyle#color} property on the created object.
@param [options.thickness] The {@link WebGLPolylineEdgeStyle#thickness} of the line. This option sets the {@link WebGLPolylineEdgeStyle#thickness} property on the created object.
@param [options.cropPath] Whether the line path will be cropped at the source and target node bounds, potentially leaving space for the arrowheads. This option sets the {@link WebGLPolylineEdgeStyle#cropPath} property on the created object.
@param [options.targetArrow] Whether a target arrow should be drawn. This option sets the {@link WebGLPolylineEdgeStyle#targetArrow} property on the created object.
@param [options.sourceArrow] Whether a source arrow should be drawn. This option sets the {@link WebGLPolylineEdgeStyle#sourceArrow} property on the created object.
@param [options.targetArrowScale] The factor by which the target arrow is scaled. This option sets the {@link WebGLPolylineEdgeStyle#targetArrowScale} property on the created object.
@param [options.targetArrowRatio] The ratio between the target arrow width and the edge width. This option sets the {@link WebGLPolylineEdgeStyle#targetArrowRatio} property on the created object.
@param [options.sourceArrowScale] The factor by which the source arrow is scaled. This option sets the {@link WebGLPolylineEdgeStyle#sourceArrowScale} property on the created object.
@param [options.sourceArrowRatio] The ratio between the source arrow width and the edge width. This option sets the {@link WebGLPolylineEdgeStyle#sourceArrowRatio} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle Online Documentation}*/
constructor(options?: {
color?: Color | ColorConvertible;
thickness?: number;
cropPath?: boolean;
targetArrow?: boolean;
sourceArrow?: boolean;
targetArrowScale?: number;
targetArrowRatio?: number;
sourceArrowScale?: number;
sourceArrowRatio?: number;
});
/**
Performs a {@link YObject#memberwiseClone}
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23clone() Online Documentation}*/
clone(): this;
/**
Gets or sets the {@link WebGLPolylineEdgeStyle#color} of the line.
The default value is {@link Color#BLACK}.
@default 'yWorks.Controls.Colors.Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23color Online Documentation}*/
color: Color;
/**
Gets or sets the {@link WebGLPolylineEdgeStyle#thickness} of the line.
The default value is <code>1.0</code>.
@default '1'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23thickness Online Documentation}*/
thickness: number;
/**
Gets or sets whether the line path will be cropped at the source and target node bounds, potentially leaving space for
the arrowheads.
The default value is <code>false</code> since cropping is an expensive operation.
@default 'false'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23cropPath Online Documentation}*/
cropPath: boolean;
/**
Gets or sets whether a target arrow should be drawn.
The default value is <code>false</code>. Note that for correct arrow rendering, {@link WebGLPolylineEdgeStyle#cropPath} has to be enabled.
@default 'false'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23targetArrow Online Documentation}*/
targetArrow: boolean;
/**
Gets or sets whether a source arrow should be drawn.
The default value is <code>false</code>. Note that for correct arrow rendering, {@link WebGLPolylineEdgeStyle#cropPath} has to be enabled.
@default 'false'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23sourceArrow Online Documentation}*/
sourceArrow: boolean;
/**
Gets or sets the factor by which the target arrow is scaled.
The default value is <code>1.0</code>.
@default '1'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23targetArrowScale Online Documentation}*/
targetArrowScale: number;
/**
Gets or sets the ratio between the target arrow width and the edge width.
The arrow width is calculated as: arrow width = edge thickness + edge thickness * arrow ratio. The default value is
<code>1.0</code>.
@default '1'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23targetArrowRatio Online Documentation}*/
targetArrowRatio: number;
/**
Gets or sets the factor by which the source arrow is scaled.
The default value is <code>1.0</code>.
@default '1'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23sourceArrowScale Online Documentation}*/
sourceArrowScale: number;
/**
Gets or sets the ratio between the source arrow width and the edge width.
The arrow width is calculated as: arrow width = edge thickness + edge thickness * arrow ratio. The default value is
<code>1.0</code>.
@default '1'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23sourceArrowRatio Online Documentation}*/
sourceArrowRatio: number;
/**
Gets the renderer implementation that has been supplied to this instance in the constructor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLPolylineEdgeStyle%23renderer Online Documentation}*/
renderer: IEdgeStyleRenderer;
static $class: Class;
private $_WebGLPolylineEdgeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WebGLPolylineEdgeStyle;
}
export interface WebGLShapeNodeStyle extends YObject, INodeStyle {
/**
@inheritdoc*/
clone(): this;
/**
@inheritdoc*/
renderer: INodeStyleRenderer;
} /**
A node style that uses WebGL rendering and displays nodes as geometric shapes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLShapeNodeStyle Online Documentation}*/
export class WebGLShapeNodeStyle {
/**
Creates a new instance of this node style.
@param [options] The parameters to pass.
@param [options.color] The {@link WebGLShapeNodeStyle#color} of this style. This option sets the {@link WebGLShapeNodeStyle#color} property on the created object.
@param [options.shape] The shape of this style. This option sets the {@link WebGLShapeNodeStyle#shape} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLShapeNodeStyle Online Documentation}*/
constructor(options?: {
color?: Color | ColorConvertible;
shape?: ShapeNodeShape | ShapeNodeShapeStringValues;
});
/**
Gets or sets the {@link WebGLShapeNodeStyle#color} of this style.
The default value is {@link Color#WHITE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLShapeNodeStyle%23color Online Documentation}*/
color: Color;
/**
Gets or sets the shape of this style.
The default value is {@link ShapeNodeShape#RECTANGLE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLShapeNodeStyle%23shape Online Documentation}*/
shape: ShapeNodeShape;
static $class: Class;
private $_WebGLShapeNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WebGLShapeNodeStyle;
}
export interface WebGLTaperedEdgeStyle extends YObject, IEdgeStyle {
/**
@inheritdoc*/
renderer: IEdgeStyleRenderer;
} /**
An edge style that uses WebGL rendering and displays edges as tapered straight lines that connect the centers of the
source and target nodes.
This style ignores the location of bends and the exact location of the ports and can only be used for edges at nodes. It
doesn't work for edges whose source or target port owner is an edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLTaperedEdgeStyle Online Documentation}*/
export class WebGLTaperedEdgeStyle {
/**
Creates a new instance of this style.
@param [options] The parameters to pass.
@param [options.color] The {@link Color} of the line. This option sets the {@link WebGLTaperedEdgeStyle#color} property on the created object.
@param [options.thickness] The thickness at the source end of the line. This option sets the {@link WebGLTaperedEdgeStyle#thickness} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLTaperedEdgeStyle Online Documentation}*/
constructor(options?: {
color?: Color | ColorConvertible;
thickness?: number;
});
/**
Performs a {@link YObject#memberwiseClone}
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLTaperedEdgeStyle%23clone() Online Documentation}*/
clone(): this;
/**
Gets or sets the {@link Color} of the line.
The default value is {@link Color#BLACK}.
@default 'yWorks.Controls.Colors.Black'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLTaperedEdgeStyle%23color Online Documentation}*/
color: Color;
/**
Gets or sets the thickness at the source end of the line.
The default value is 5.
@default '5'
@see {@link https://docs.yworks.com/yfileshtml/#/api/WebGLTaperedEdgeStyle%23thickness Online Documentation}*/
thickness: number;
static $class: Class;
private $_WebGLTaperedEdgeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WebGLTaperedEdgeStyle;
}
export interface ITemplateStyleBindingContext extends IPropertyObservable {
}
/**
Exposes properties of a graph element to which a <code>TemplateBinding</code> can be bound.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext Online Documentation}*/
export abstract class ITemplateStyleBindingContext {
/**
Gets or sets the bounds of the visual.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23bounds Online Documentation}*/
abstract readonly bounds: Rect;
/**
Gets the width of the visual.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23width Online Documentation}*/
abstract readonly width: number;
/**
Gets the height of the visual.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23height Online Documentation}*/
abstract readonly height: number;
/**
Gets or sets the {@link CanvasComponent} instance this instance is used in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23canvasComponent Online Documentation}*/
abstract readonly canvasComponent: CanvasComponent | null;
/**
Gets or sets a value indicating whether the item is currently selected.
This state is bound to the {@link IGraphSelection} which is used in the {@link ITemplateStyleBindingContext#canvasComponent} this visual is being used in. Setting this
value will write through to the {@link IGraphSelection}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23itemSelected Online Documentation}*/
abstract itemSelected: boolean;
/**
Gets or sets a value indicating whether the item is the currently focused item in the {@link GraphComponent}.
This state is bound to the {@link GraphComponent#currentItem} which is used in the {@link ITemplateStyleBindingContext#canvasComponent} this visual is being used in. Setting this value will write through to
the {@link GraphComponent#currentItem}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23itemFocused Online Documentation}*/
abstract itemFocused: boolean;
/**
Gets or sets a value indicating whether the item is currently highlighted.
This state is bound to the {@link HighlightIndicatorManager} which is used in the {@link ITemplateStyleBindingContext#canvasComponent} this visual is being used in. Setting this value will write through to
the {@link HighlightIndicatorManager}'s {@link HighlightIndicatorManager#selectionModel}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23itemHighlighted Online Documentation}*/
abstract itemHighlighted: boolean;
/**
Gets or sets the item that is rendered by this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23item Online Documentation}*/
abstract readonly item: IModelItem;
/**
Gets or sets the style tag that is associated with the style that created this visual.
This property can be used by implementers as a convenient property to pass data to the visual.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23styleTag Online Documentation}*/
abstract readonly styleTag: any | null;
/**
Gets the current zoom level of the {@link CanvasComponent}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITemplateStyleBindingContext%23zoom Online Documentation}*/
abstract readonly zoom: number;
static $class: Class;
private $_ITemplateStyleBindingContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ITemplateStyleBindingContext;
}
export interface ILabelTemplateStyleBindingContext extends ITemplateStyleBindingContext {
}
/**
Exposes additional properties of a label to which a <code>TemplateBinding</code> in a label template can be bound.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelTemplateStyleBindingContext Online Documentation}*/
export abstract class ILabelTemplateStyleBindingContext {
/**
Gets or sets the label text property that is populated using the {@link ILabel}'s {@link ILabel#text} property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelTemplateStyleBindingContext%23labelText Online Documentation}*/
abstract labelText: string;
/**
Gets a value indicating whether this instance has been flipped so that it points upwards.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelTemplateStyleBindingContext%23isFlipped Online Documentation}*/
abstract readonly isFlipped: boolean;
/**
Gets a value indicating whether this instance is currently rendered upside down.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelTemplateStyleBindingContext%23isUpsideDown Online Documentation}*/
abstract readonly isUpsideDown: boolean;
static $class: Class;
private $_ILabelTemplateStyleBindingContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelTemplateStyleBindingContext;
}
export interface TemplateLabelStyleBase extends YObject, ILabelStyle, IPropertyObservable, ICloneable {
/**
@inheritdoc*/
readonly renderer: ILabelStyleRenderer;
} /**
Base class for label styles that use a SVG snippet as template for the visualization of {@link ILabel}s.
@see TemplateLabelStyle
@see StringTemplateLabelStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase Online Documentation}*/
export abstract class TemplateLabelStyleBase {
/**
Constructs a new instance using the provided <code>renderer</code>.
@param renderer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23TemplateLabelStyleBase(TemplateLabelStyleRenderer) Online Documentation}*/
constructor(renderer: TemplateLabelStyleRenderer | null);
/**
Constructs a new instance of this style with default values.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23TemplateLabelStyleBase() Online Documentation}*/
constructor();
/**
Performs a shallow clone of this instance, except for the value of {@link TemplateLabelStyleBase#styleTag} which is cloned, too, if it can be cloned.
@returns A clone of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23clone() Online Documentation}*/
clone(): this;
/**
Factory method that is called by the {@link TemplateLabelStyleRenderer} to create the visual that will be used for
the display of the label.
@param context The context for which the visual should be created.
@param label The label that will be rendered.
@returns The readily styled visual.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23createTemplate(IRenderContext%2CILabel) Online Documentation}*/
createTemplate(context: IRenderContext, label: ILabel): SvgVisual;
/**
Determines the preferred size of the label if this style was applied.
@param context The context for which the size should be calculated.
@param label The label to determine the preferred size of.
@returns The preferred size.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23getPreferredSize(IRenderContext%2CILabel) Online Documentation}*/
getPreferredSize(context: IRenderContext | null, label: ILabel): Size;
/**
Loads all template script elements that have not been parsed, yet, and creates the respective template cache objects.
This method can be called to perform the template loading at a certain point in time, for example during application
startup. By default, a template is loaded lazily when the first visual using this template is built.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23loadAllTemplates() Online Documentation}*/
static loadAllTemplates(): void;
/**
Raises the {@link TemplateLabelStyleBase#addPropertyChangedListener PropertyChanged} event.
@param evt The {@link PropertyChangedEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23onPropertyChanged(PropertyChangedEventArgs) Online Documentation}*/
onPropertyChanged(evt: PropertyChangedEventArgs): void;
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
@param listener The listener to add.
@see TemplateLabelStyleBase#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23PropertyChanged Online Documentation}*/
addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
@param listener The listener to remove.
@see TemplateLabelStyleBase#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23PropertyChanged Online Documentation}*/
removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the tag that is associated with this style instance.
The various style renderer implementations can use this tag to make it accessible in the context of the instantiated
templates and controls. The framework does not make any assumptions about the value of this property. It is there for
convenient use in the process of data binding the templates. This property supports {@link IPropertyObservable property change notification} and can
thus be data bound easily.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23styleTag Online Documentation}*/
styleTag: any | null;
/**
Gets or sets an implementation of {@link IContextLookup} that can be used to satisfy queries that are made to the
implementation which is returned by calls to {@link ILabelStyleRenderer#getContext} that are made on the style
renderer that is associated with this style instance.
This can be used to conveniently customize the lookup behavior of the style renderers that are associated with this
instance. The default value is {@link IContextLookup#EMPTY_CONTEXT_LOOKUP}
@default 'Lookups.EmptyContextLookup'
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23contextLookup Online Documentation}*/
contextLookup: IContextLookup;
/**
Gets or sets the normalized outline of a label that will be displayed using the created visual.
The shape will automatically arranged and transformed to have the bounds of the label. This shape will be used to
satisfy requests to {@link IMarqueeTestable#isInBox} and {@link IHitTestable#isHit} and may be left <code>null</code> to
indicate default (rectangular) behavior.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23normalizedOutline Online Documentation}*/
normalizedOutline: GeneralPath | null;
/**
Gets or sets a value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if
otherwise it would be rendered upside-down.
The default value is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23autoFlip Online Documentation}*/
autoFlip: boolean;
/**
Gets or sets the preferred size of the labels that use this style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleBase%23preferredSize Online Documentation}*/
preferredSize: Size;
static $class: Class;
private $_TemplateLabelStyleBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplateLabelStyleBase;
}
export interface TemplateLabelStyle extends TemplateLabelStyleBase {
}
/**
An {@link ILabelStyle} that uses an SVG snippet as template for the visualization of {@link ILabel}s.
This template style uses the {@link TemplateLabelStyle#styleResourceKey} property to determine the template that will be applied to the {@link Visual}. The
template must be specified in the DOM in a script element with the attribute <code>type='text/yfiles-template'</code>. See the {@link StringTemplateLabelStyle}
class for a template style that gets its template as string.
The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
value to the value of the property with the given name in the label's tag. The tag itself can be bound with
<code>attribute="{Binding}"</code>. If the bound object provides the function <code>addPropertyChangedListener(listener)</code> or implements
{@link IPropertyObservable}, bindings to properties of the business object are dynamically updated if the registered
listeners are called on property change. Business objects can be made observable by calling
{@link TemplateLabelStyle#makeObservable}.
TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to one of the
properties of the label that are exposed by the {@link ILabelTemplateStyleBindingContext} interface.
Bindings can specify converters and converter parameters. Converters are simple functions with the signature
<code>function(value:Object, parameter:Object):Object</code>. A converter has to be registered under its name either in the {@link TemplateLabelStyle#CONVERTERS converter store}
or with the <code>window</code> object before it can be used.
<code>TextWrapConverter</code> is a built-in converter function which emits wrapped text as SVG elements. It can be used to display
wrapped text in a template style. To use the <code>TextWrapConverter</code> the template style must include a &lt;g> element that binds
a property containing the text to be wrapped to <code>data-content</code>. <code>TextWrapConverter</code> must be specified as the converter
function. The converter’s parameters are passed using a shorthand notation for width, height, font, trimming and color.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/_text_wrapping_in_a_template_style Text Wrapping in a Template Style} for more details.
Depending on the current state, the following CSS classes are added to the root element of the template:
<ul>
<li><code>yfiles-selected</code>
<li><code>yfiles-focused</code>
<li><code>yfiles-highlighted</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyle Online Documentation}*/
export class TemplateLabelStyle {
/**
Creates a new instance using the given render template id and a custom renderer instance.
@param [renderTemplateId=null] The id of the SVG snippet that will be used for the component.
@param [renderer=null] The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyle Online Documentation}*/
constructor(renderTemplateId?: string | null, renderer?: TemplateLabelStyleRenderer);
/**
Creates a new instance using the given render template id and a custom renderer instance.
@param options The parameters to pass.
@param [options.renderTemplateId=null] The id of the SVG snippet that will be used for the component.
@param [options.renderer=null] The custom renderer for this style instance.
@param [options.autoFlip] A value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if otherwise it would be rendered upside-down. This option sets the {@link TemplateLabelStyleBase#autoFlip} property on the created object.
@param [options.normalizedOutline] The normalized outline of a label that will be displayed using the created visual. This option sets the {@link TemplateLabelStyleBase#normalizedOutline} property on the created object.
@param [options.preferredSize] The preferred size of the labels that use this style. This option sets the {@link TemplateLabelStyleBase#preferredSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyle Online Documentation}*/
constructor(options: {
renderTemplateId?: string | null;
renderer?: TemplateLabelStyleRenderer;
autoFlip?: boolean;
normalizedOutline?: GeneralPath | null;
preferredSize?: Size | SizeConvertible;
});
/**
Makes an object observable to bindings in template styles.
Bindings to observable objects react to changes in the bound data. This enables dynamic styling of graph items that
reacts to changes of the underlying business data of each graph item.
This method adds the functions <code>firePropertyChanged</code>, <code>addPropertyChangedListener</code> and <code>removePropertyChangedListener</code> to the
given object. Call <code>firePropertyChanged(propertyName: string)</code> each time a bound property value changes in order to update
the related bindings.
@param dataObject The object to make observable.
@returns The object that was passed in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyle%23makeObservable(Object) Online Documentation}*/
static makeObservable(dataObject: Object): Object;
/**
Gets or sets the {@link string} that will be used to look up the template that is applied to the component.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyle%23styleResourceKey Online Documentation}*/
styleResourceKey: string | null;
/**
Gets the storage object for binding converters.
Binding converter functions can be stored in this object. Once a converter function name is resolved during template
parsing, the converter function is located in this object first. If it is not found in this object, it is located on the
window object. The binding converter store is shared among all template styles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyle%23CONVERTERS Online Documentation}*/
static readonly CONVERTERS: any;
static $class: Class;
private $_TemplateLabelStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplateLabelStyle;
}
export interface StringTemplateLabelStyle extends TemplateLabelStyleBase {
}
/**
An {@link ILabelStyle} that uses an SVG snippet as template for the visualization of {@link ILabel}s.
In contrast to {@link TemplateLabelStyle}, the SVG template used by this style must be encoded as string and
assigned to the {@link StringTemplateLabelStyle#svgContent} property.
It is vital that the {@link StringTemplateLabelStyle#svgContent SVG snippet} does not declare the SVG namespace itself. Its content is parsed in a dummy namespace
in order to avoid immediate validation of the result. The resulting SVG elements are automatically created in the SVG
namespace before they are added to the DOM.
The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
value to the value of the property with the given name in the label's tag. The tag itself can be bound with
<code>attribute="{Binding}"</code>. If the bound object provides the function <code>addPropertyChangedListener(listener)</code> or implements
{@link IPropertyObservable}, bindings to properties of the business object are dynamically updated if the registered
listeners are called on property change. Business objects can be made observable by calling
{@link StringTemplateLabelStyle#makeObservable}.
TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to one of the
properties of the label that are exposed by the {@link ILabelTemplateStyleBindingContext} interface.
Bindings can specify converters and converter parameters. Converters are simple functions with the signature
<code>function(value:Object, parameter:Object):Object</code>. A converter has to be registered under its name either in the {@link StringTemplateLabelStyle#CONVERTERS converter store}
or with the <code>window</code> object before it can be used.
<code>TextWrapConverter</code> is a built-in converter function which emits wrapped text as SVG elements. It can be used to display
wrapped text in a template style. To use the <code>TextWrapConverter</code> the template style must include a &lt;g> element that binds
a property containing the text to be wrapped to <code>data-content</code>. <code>TextWrapConverter</code> must be specified as the converter
function. The converter’s parameters are passed using a shorthand notation for width, height, font, trimming and color.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/_text_wrapping_in_a_template_style Text Wrapping in a Template Style} for more details.
Depending on the current state, the following CSS classes are added to the root element of the template:
<ul>
<li><code>yfiles-selected</code>
<li><code>yfiles-focused</code>
<li><code>yfiles-highlighted</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateLabelStyle Online Documentation}*/
export class StringTemplateLabelStyle {
/**
Creates a new instance using the provided renderer.
@param [svgContent=null] The SVG snippet to use as template.
@param [renderer=null] The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateLabelStyle Online Documentation}*/
constructor(svgContent?: string | null, renderer?: TemplateLabelStyleRenderer);
/**
Creates a new instance using the provided renderer.
@param options The parameters to pass.
@param [options.svgContent=null] The SVG snippet to use as template.
@param [options.renderer=null] The custom renderer for this style instance.
@param [options.autoFlip] A value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if otherwise it would be rendered upside-down. This option sets the {@link TemplateLabelStyleBase#autoFlip} property on the created object.
@param [options.normalizedOutline] The normalized outline of a label that will be displayed using the created visual. This option sets the {@link TemplateLabelStyleBase#normalizedOutline} property on the created object.
@param [options.preferredSize] The preferred size of the labels that use this style. This option sets the {@link TemplateLabelStyleBase#preferredSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateLabelStyle Online Documentation}*/
constructor(options: {
svgContent?: string | null;
renderer?: TemplateLabelStyleRenderer;
autoFlip?: boolean;
normalizedOutline?: GeneralPath | null;
preferredSize?: Size | SizeConvertible;
});
/**
Makes an object observable to bindings in template styles.
Bindings to observable objects react to changes in the bound data. This enables dynamic styling of graph items that
reacts to changes of the underlying business data of each graph item.
This method adds the functions <code>firePropertyChanged</code>, <code>addPropertyChangedListener</code> and <code>removePropertyChangedListener</code> to the
given object. Call <code>firePropertyChanged(propertyName: string)</code> each time a bound property value changes in order to update
the related bindings.
@param dataObject The object to make observable.
@returns The object that was passed in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateLabelStyle%23makeObservable(Object) Online Documentation}*/
static makeObservable(dataObject: Object): Object;
/**
Gets or sets the SVG content that is the template for the label visualization.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateLabelStyle%23svgContent Online Documentation}*/
svgContent: string | null;
/**
Specifies whether code contained in the templates should be included in the DOM.
Script tags and event listeners can be used to inject malicious code. This property should only be enabled if the
templates originate from trusted sources. If the application allows to load external GraphML files, this property should
probably not be enabled.
This setting is shared by {@link StringTemplateNodeStyle}, {@link StringTemplatePortStyle} and
StringStripeControlPortStyle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateLabelStyle%23trusted Online Documentation}*/
static trusted: boolean;
/**
Gets the storage object for binding converters.
Binding converter functions can be stored in this object. Once a converter function name is resolved during template
parsing, the converter function is located in this object first. If it is not found in this object, it is located on the
window object. The binding converter store is shared among all template styles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateLabelStyle%23CONVERTERS Online Documentation}*/
static readonly CONVERTERS: any;
static $class: Class;
private $_StringTemplateLabelStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StringTemplateLabelStyle;
}
export interface TemplateLabelStyleRenderer extends YObject, ILabelStyleRenderer, IVisualCreator, IBoundsProvider, IHitTestable, IMarqueeTestable, IVisibilityTestable, ILookup {
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
} /**
An implementation of the {@link ILabelStyleRenderer} interface that can render {@link TemplateLabelStyleBase}
instances.
<h2>Note</h2>
It is not recommended to inherit from this class because it is not guaranteed that the visuals are updated correctly in
this case.
@see TemplateLabelStyle
@see StringTemplateLabelStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer Online Documentation}*/
export class TemplateLabelStyleRenderer {
/**
Stores the {@link ILabel#layout layout}.
Subclasses should override this method, call the super implementation and configure their painting entities.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Creates the visual by delegating to {@link TemplateLabelStyleBase#createTemplate} and then initializing the
control with the context.
@param context The context for the creation.
@returns The visual that is used for rendering the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23createVisual(IRenderContext) Online Documentation}*/
createVisual(context: IRenderContext): Visual | null;
/**
Configures the {@link TemplateLabelStyleRenderer#style} and {@link TemplateLabelStyleRenderer#label} properties, calls {@link TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to retrieve the bounds provider for. The value will be stored in the {@link TemplateLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link TemplateLabelStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateLabelStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getBoundsProvider(ILabel%2CILabelStyle) Online Documentation}*/
getBoundsProvider(label: ILabel, style: ILabelStyle): IBoundsProvider;
/**
Configures the {@link TemplateLabelStyleRenderer#style} and {@link TemplateLabelStyleRenderer#label} properties, does <b>not</b> call {@link TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
As this method may be called often it will not automatically call {@link TemplateLabelStyleRenderer#configure},
instead subclasses should ensure that in the {@link TemplateLabelStyleRenderer#lookup} method call they should
call {@link TemplateLabelStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an
instance that depends on a correctly configured <code>this</code>.
@param label The label to query the context for. The value will be stored in the {@link TemplateLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link TemplateLabelStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateLabelStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getContext(ILabel%2CILabelStyle) Online Documentation}*/
getContext(label: ILabel, style: ILabelStyle): ILookup;
/**
Configures the {@link TemplateLabelStyleRenderer#style} and {@link TemplateLabelStyleRenderer#label} properties, calls {@link TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to query hit test with. The value will be stored in the {@link TemplateLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link TemplateLabelStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateLabelStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getHitTestable(ILabel%2CILabelStyle) Online Documentation}*/
getHitTestable(label: ILabel, style: ILabelStyle): IHitTestable;
/**
Configures the {@link TemplateLabelStyleRenderer#style} and {@link TemplateLabelStyleRenderer#label} properties, calls {@link TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to query marquee intersection tests. The value will be stored in the {@link TemplateLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link TemplateLabelStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateLabelStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getMarqueeTestable(ILabel%2CILabelStyle) Online Documentation}*/
getMarqueeTestable(label: ILabel, style: ILabelStyle): IMarqueeTestable;
/**
Gets the normalized outline by delegating to {@link TemplateLabelStyleBase#normalizedOutline}.
@param style
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getNormalizedOutline(TemplateLabelStyleBase) Online Documentation}*/
getNormalizedOutline(style: TemplateLabelStyleBase): GeneralPath | null;
/**
Calculates the preferred size for the given label and style.
@param label The label instance.
@param style The style instance to apply.
@returns The preferred size for the given label and style.
@see TemplateLabelStyleRenderer#getPreferredSizeCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getPreferredSize(ILabel%2CILabelStyle) Online Documentation}*/
getPreferredSize(label: ILabel, style: ILabelStyle): Size;
/**
Gets the size of the preferred for the current configuration using the provided context.
@param context The render context.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getPreferredSizeCore(IRenderContext) Online Documentation}*/
getPreferredSizeCore(context: IRenderContext | null): Size;
/**
Calculates the preferred size for the given label and style.
@param context
@param label The label instance.
@param style The style instance to apply.
@returns The preferred size for the given label and style.
@see TemplateLabelStyleRenderer#getPreferredSizeCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getPreferredSizeWithContext(IRenderContext%2CILabel%2CILabelStyle) Online Documentation}*/
getPreferredSizeWithContext(context: IRenderContext | null, label: ILabel, style: ILabelStyle): Size;
/**
Configures the {@link TemplateLabelStyleRenderer#style} and {@link TemplateLabelStyleRenderer#label} properties, does <b>not</b> call {@link TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link TemplateLabelStyleRenderer#configure}. If
the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link TemplateLabelStyleRenderer#isVisible}.
@param label The label to query visibility tests. The value will be stored in the {@link TemplateLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link TemplateLabelStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateLabelStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getVisibilityTestable(ILabel%2CILabelStyle) Online Documentation}*/
getVisibilityTestable(label: ILabel, style: ILabelStyle): IVisibilityTestable;
/**
Configures the {@link TemplateLabelStyleRenderer#style} and {@link TemplateLabelStyleRenderer#label} properties, calls {@link TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
@param label The label to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link TemplateLabelStyleRenderer#label} property.
@param style The style to associate with the label. The value will be stored in the {@link TemplateLabelStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateLabelStyleRenderer#createVisual
@see TemplateLabelStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23getVisualCreator(ILabel%2CILabelStyle) Online Documentation}*/
getVisualCreator(label: ILabel, style: ILabelStyle): IVisualCreator;
/**
Delegates to the {@link TemplateLabelStyleBase}'s {@link TemplateLabelStyleBase#autoFlip} property.
@param style
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23isAutoFlip(TemplateLabelStyleBase) Online Documentation}*/
isAutoFlip(style: TemplateLabelStyleBase): boolean;
/**
Uses the {@link ILabel#layout layout} to determine whether the clip intersects.
@param context
@param rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Delegates to the {@link TemplateLabelStyleBase#contextLookup} of the {@link ILabelStyle}.
@param type The type to query an instance for.
@returns The implementation or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Delegates to the {@link TemplateLabelStyleBase}'s {@link TemplateLabelStyleBase#contextLookup}.
@param style The style to look up the context for.
@param label The current label.
@param type The type to look up.
@returns The result from the context or <code>null</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23lookupContext(TemplateLabelStyleBase%2CILabel%2CClass) Online Documentation}*/
lookupContext(style: TemplateLabelStyleBase, label: ILabel, type: Class): any | null;
/**
Updates the visual correspondingly.
@param context The context for the creation.
@param oldVisual The currently rendered visual.
@returns The visual that is used for rendering the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23updateVisual(IRenderContext%2CVisual) Online Documentation}*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
/**
Gets the layout of the current label.
This value is set during {@link TemplateLabelStyleRenderer#configure} to the label's {@link ILabel#layout layout}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23layout Online Documentation}*/
layout: IOrientedRectangle;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23style Online Documentation}*/
style: TemplateLabelStyleBase;
/**
Gets or sets the currently configured label.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateLabelStyleRenderer%23label Online Documentation}*/
label: ILabel;
static $class: Class;
private $_TemplateLabelStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplateLabelStyleRenderer;
}
export interface TemplateNodeStyleBase extends YObject, INodeStyle, IPropertyObservable, ICloneable {
/**
@inheritdoc*/
readonly renderer: INodeStyleRenderer;
} /**
Base class for node styles that use a SVG snippet as template for the visualization of {@link INode}s.
@see TemplateNodeStyle
@see StringTemplateNodeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase Online Documentation}*/
export abstract class TemplateNodeStyleBase {
/**
Constructs a new instance using the provided <code>renderer</code>.
@param renderer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23TemplateNodeStyleBase(TemplateNodeStyleRenderer) Online Documentation}*/
constructor(renderer: TemplateNodeStyleRenderer);
/**
Constructs a new instance of this style with default values.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23TemplateNodeStyleBase() Online Documentation}*/
constructor();
/**
Performs a shallow clone of this instance, except for the value of {@link TemplateNodeStyleBase#styleTag} which is cloned, too, if it can be cloned.
@returns A clone of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23clone() Online Documentation}*/
clone(): this;
/**
Factory method that is called by the {@link TemplateNodeStyleRenderer} to create the visual that will be used for
the display of the node.
@param context The context for which the visual should be created.
@param node The node that will be rendered.
@returns The readily styled visual.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23createTemplate(IRenderContext%2CINode) Online Documentation}*/
createTemplate(context: IRenderContext, node: INode): SvgVisual;
/**
Loads all template script elements that have not been parsed, yet, and creates the respective template cache objects.
This method can be called to perform the template loading at a certain point in time, for example during application
startup. By default, a template is loaded lazily when the first visual using this template is built.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23loadAllTemplates() Online Documentation}*/
static loadAllTemplates(): void;
/**
Raises the {@link TemplateNodeStyleBase#addPropertyChangedListener PropertyChanged} event.
@param evt The {@link PropertyChangedEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23onPropertyChanged(PropertyChangedEventArgs) Online Documentation}*/
onPropertyChanged(evt: PropertyChangedEventArgs): void;
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
@param listener The listener to add.
@see TemplateNodeStyleBase#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23PropertyChanged Online Documentation}*/
addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
@param listener The listener to remove.
@see TemplateNodeStyleBase#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23PropertyChanged Online Documentation}*/
removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the tag that is associated with this style instance.
The various style renderer implementations can use this tag to make it accessible in the context of the instantiated
templates and controls. The framework does not make any assumptions about the value of this property. It is there for
convenient use in the process of data binding the templates. This property supports {@link IPropertyObservable property change notification} and can
thus be data bound easily.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23styleTag Online Documentation}*/
styleTag: any | null;
/**
Gets or sets an implementation of {@link IContextLookup} that can be used to satisfy queries that are made to the
implementation which is returned by calls to {@link INodeStyleRenderer#getContext} that are made on the style
renderer that is associated with this style instance.
This can be used to conveniently customize the lookup behavior of the style renderers that are associated with this
instance. E.g. it is possible to provide customized implementations of {@link ISelectionIndicatorInstaller},
{@link IHighlightIndicatorInstaller}, {@link IHandleProvider}, {@link INodeSizeConstraintProvider}, and
{@link INodeInsetsProvider}. Simple style implementations can use {@link IContextLookup#EMPTY_CONTEXT_LOOKUP}
but may not use <code>null</code> for this property. The default value is {@link IContextLookup#EMPTY_CONTEXT_LOOKUP}
@default 'Lookups.EmptyContextLookup'
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23contextLookup Online Documentation}*/
contextLookup: IContextLookup;
/**
Gets or sets the insets the {@link INodeInsetsProvider} implementation for {@link INode} should yield for this
instance.
This is especially useful if this style is used for {@link IGraph#createGroupNode group nodes}, as they require insets for the calculation of the
bounds. The insets can be used to make sure that child nodes do not cover visual elements of the style inside the
insets. The default value is an {@link Insets} insets with <code>5</code> for all insets.
@see INodeInsetsProvider
@see INodeStyleRenderer#getContext
@default '5'
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23insets Online Documentation}*/
insets: Insets;
/**
Gets or sets the minimum size an {@link INodeSizeConstraintProvider} implementation for {@link INode} should
yield.
The default value is {@link Size#EMPTY}.
@see INodeSizeConstraintProvider
@default 'Empty'
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23minimumSize Online Documentation}*/
minimumSize: Size;
/**
Gets or sets the normalized outline of a node that will be displayed using the created visual.
This shape will be used to satisfy requests to {@link IMarqueeTestable#isInBox}, {@link IHitTestable#isHit},
as well as for {@link IShapeGeometry} methods (which affect edge cropping at the node outline).
This property may be left at or set to <code>null</code> to indicate default (rectangular) behavior.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleBase%23normalizedOutline Online Documentation}*/
normalizedOutline: GeneralPath | null;
static $class: Class;
private $_TemplateNodeStyleBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplateNodeStyleBase;
}
export interface TemplateNodeStyle extends TemplateNodeStyleBase {
}
/**
An {@link INodeStyle} that uses an SVG snippet as a template for the visualization of {@link INode}s.
This template style uses the {@link TemplateNodeStyle#styleResourceKey} property to determine the template that will be applied to the {@link Visual}. The
template must be specified in the DOM in a script element with the attribute <code>type='text/yfiles-template'</code>. See the {@link StringTemplateNodeStyle}
class for a template style that gets its template as string.
The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
value to the value of the property with the given name in the node's tag. The tag itself can be bound with
<code>attribute="{Binding}"</code>. If the bound object provides the function <code>addPropertyChangedListener(listener)</code> or implements
{@link IPropertyObservable}, bindings to properties of the business object are dynamically updated if the registered
listeners are called on property change. Business objects can be made observable by calling
{@link TemplateNodeStyle#makeObservable}.
TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to one of the
properties of the node that are exposed by the {@link ITemplateStyleBindingContext} interface.
Bindings can specify converters and converter parameters. Converters are simple functions with the signature
<code>function(value:Object, parameter:Object):Object</code>. A converter has to be registered under its name either in the {@link TemplateNodeStyle#CONVERTERS converter store}
or with the <code>window</code> object before it can be used.
<code>TextWrapConverter</code> is a built-in converter function which emits wrapped text as SVG elements. It can be used to display
wrapped text in a template style. To use the <code>TextWrapConverter</code> the template style must include a &lt;g> element that binds
a property containing the text to be wrapped to <code>data-content</code>. <code>TextWrapConverter</code> must be specified as the converter
function. The converter’s parameters are passed using a shorthand notation for width, height, font, trimming and color.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/_text_wrapping_in_a_template_style Text Wrapping in a Template Style} for more details.
Depending on the current state, the following CSS classes are added to the root element of the template:
<ul>
<li><code>yfiles-selected</code>
<li><code>yfiles-focused</code>
<li><code>yfiles-highlighted</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyle Online Documentation}*/
export class TemplateNodeStyle {
/**
Creates a new instance using the given render template id and a custom renderer instance.
@param [renderTemplateId=null] The id of the SVG snippet that will be used for the component.
@param [renderer=null] The custom renderer for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyle Online Documentation}*/
constructor(renderTemplateId?: string | null, renderer?: TemplateNodeStyleRenderer);
/**
Creates a new instance using the given render template id and a custom renderer instance.
@param options The parameters to pass.
@param [options.renderTemplateId=null] The id of the SVG snippet that will be used for the component.
@param [options.renderer=null] The custom renderer for this instance.
@param [options.insets] The insets the {@link INodeInsetsProvider} implementation for {@link INode} should yield for this instance. This option sets the {@link TemplateNodeStyleBase#insets} property on the created object.
@param [options.minimumSize] The minimum size an {@link INodeSizeConstraintProvider} implementation for {@link INode} should yield. This option sets the {@link TemplateNodeStyleBase#minimumSize} property on the created object.
@param [options.normalizedOutline] The normalized outline of a node that will be displayed using the created visual. This option sets the {@link TemplateNodeStyleBase#normalizedOutline} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyle Online Documentation}*/
constructor(options: {
renderTemplateId?: string | null;
renderer?: TemplateNodeStyleRenderer;
insets?: Insets | InsetsConvertible;
minimumSize?: Size | SizeConvertible;
normalizedOutline?: GeneralPath | null;
});
/**
Makes an object observable to bindings in template styles.
Bindings to observable objects react to changes in the bound data. This enables dynamic styling of graph items that
reacts to changes of the underlying business data of each graph item.
This method adds the functions <code>firePropertyChanged</code>, <code>addPropertyChangedListener</code> and <code>removePropertyChangedListener</code> to the
given object. Call <code>firePropertyChanged(propertyName: string)</code> each time a bound property value changes in order to update
the related bindings.
@param dataObject The object to make observable.
@returns The object that was passed in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyle%23makeObservable(Object) Online Documentation}*/
static makeObservable(dataObject: Object): Object;
/**
Gets or sets the {@link string} that will be used to look up the template that is applied to the component.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyle%23styleResourceKey Online Documentation}*/
styleResourceKey: string | null;
/**
Gets the storage object for binding converters.
Binding converter functions can be stored in this object. Once a converter function name is resolved during template
parsing, the converter function is located in this object first. If it is not found in this object, it is located on the
window object. The binding converter store is shared among all template styles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyle%23CONVERTERS Online Documentation}*/
static readonly CONVERTERS: any;
static $class: Class;
private $_TemplateNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplateNodeStyle;
}
export interface StringTemplateNodeStyle extends TemplateNodeStyleBase {
}
/**
An {@link INodeStyle} that uses an SVG snippet as template for the visualization of {@link INode}s.
In contrast to {@link TemplateNodeStyle}, the SVG template used by this style must be encoded as string and
assigned to the {@link StringTemplateNodeStyle#svgContent} property.
It is vital that the {@link StringTemplateNodeStyle#svgContent SVG snippet} does not declare the SVG namespace itself. Its content is parsed in a dummy namespace
in order to avoid immediate validation of the result. The resulting SVG elements are automatically created in the SVG
namespace before they are added to the DOM.
The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
value to the value of the property with the given name in the node's tag. The tag itself can be bound with
<code>attribute="{Binding}"</code>. If the bound object provides the function <code>addPropertyChangedListener(listener)</code> or implements
{@link IPropertyObservable}, bindings to properties of the business object are dynamically updated if the registered
listeners are called on property change. Business objects can be made observable by calling
{@link StringTemplateNodeStyle#makeObservable}.
TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to one of the
properties of the node that are exposed by the {@link ITemplateStyleBindingContext} interface.
Bindings can specify converters and converter parameters. Converters are simple functions with the signature
<code>function(value:Object, parameter:Object):Object</code>. A converter has to be registered under its name either in the {@link StringTemplateNodeStyle#CONVERTERS converter store}
or with the <code>window</code> object before it can be used.
<code>TextWrapConverter</code> is a built-in converter function which emits wrapped text as SVG elements. It can be used to display
wrapped text in a template style. To use the <code>TextWrapConverter</code> the template style must include a &lt;g> element that binds
a property containing the text to be wrapped to <code>data-content</code>. <code>TextWrapConverter</code> must be specified as the converter
function. The converter’s parameters are passed using a shorthand notation for width, height, font, trimming and color.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/_text_wrapping_in_a_template_style Text Wrapping in a Template Style} for more details.
Depending on the current state, the following CSS classes are added to the root element of the template:
<ul>
<li><code>yfiles-selected</code>
<li><code>yfiles-focused</code>
<li><code>yfiles-highlighted</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateNodeStyle Online Documentation}*/
export class StringTemplateNodeStyle {
/**
Creates a new instance using the given SVG template and a custom renderer instance.
@param [svgContent=null] The SVG snippet that will be used as a template for the component.
@param [renderer=null] The custom renderer for this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateNodeStyle Online Documentation}*/
constructor(svgContent?: string | null, renderer?: TemplateNodeStyleRenderer);
/**
Creates a new instance using the given SVG template and a custom renderer instance.
@param options The parameters to pass.
@param [options.svgContent=null] The SVG snippet that will be used as a template for the component.
@param [options.renderer=null] The custom renderer for this instance.
@param [options.insets] The insets the {@link INodeInsetsProvider} implementation for {@link INode} should yield for this instance. This option sets the {@link TemplateNodeStyleBase#insets} property on the created object.
@param [options.minimumSize] The minimum size an {@link INodeSizeConstraintProvider} implementation for {@link INode} should yield. This option sets the {@link TemplateNodeStyleBase#minimumSize} property on the created object.
@param [options.normalizedOutline] The normalized outline of a node that will be displayed using the created visual. This option sets the {@link TemplateNodeStyleBase#normalizedOutline} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateNodeStyle Online Documentation}*/
constructor(options: {
svgContent?: string | null;
renderer?: TemplateNodeStyleRenderer;
insets?: Insets | InsetsConvertible;
minimumSize?: Size | SizeConvertible;
normalizedOutline?: GeneralPath | null;
});
/**
Makes an object observable to bindings in template styles.
Bindings to observable objects react to changes in the bound data. This enables dynamic styling of graph items that
reacts to changes of the underlying business data of each graph item.
This method adds the functions <code>firePropertyChanged</code>, <code>addPropertyChangedListener</code> and <code>removePropertyChangedListener</code> to the
given object. Call <code>firePropertyChanged(propertyName: string)</code> each time a bound property value changes in order to update
the related bindings.
@param dataObject The object to make observable.
@returns The object that was passed in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateNodeStyle%23makeObservable(Object) Online Documentation}*/
static makeObservable(dataObject: Object): Object;
/**
Gets or sets the SVG content that is the template for the node visualization.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateNodeStyle%23svgContent Online Documentation}*/
svgContent: string | null;
/**
Specifies whether code contained in the templates should be included in the DOM.
Script tags and event listeners can be used to inject malicious code. This property should only be enabled if the
templates originate from trusted sources. If the application allows to load external GraphML files, this property should
probably not be enabled.
This setting is shared by {@link StringTemplateLabelStyle}, {@link StringTemplatePortStyle} and
StringStripeControlPortStyle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateNodeStyle%23trusted Online Documentation}*/
static trusted: boolean;
/**
Gets the storage object for binding converters.
Binding converter functions can be stored in this object. Once a converter function name is resolved during template
parsing, the converter function is located in this object first. If it is not found in this object, it is located on the
window object. The binding converter store is shared among all template styles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplateNodeStyle%23CONVERTERS Online Documentation}*/
static readonly CONVERTERS: any;
static $class: Class;
private $_StringTemplateNodeStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StringTemplateNodeStyle;
}
export interface TemplateNodeStyleRenderer extends YObject, INodeStyleRenderer, IShapeGeometry, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, ILassoTestable, IHitTestable, ILookup, IVisualCreator {
}
/**
An implementation of the {@link INodeStyleRenderer} interface that can render {@link TemplateNodeStyleBase}
instances.
<h2>Note</h2>
It is not recommended to inherit from this class because it is not guaranteed that the visuals are updated correctly in
this case.
@see TemplateNodeStyle
@see StringTemplateNodeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer Online Documentation}*/
export class TemplateNodeStyleRenderer {
/**
Prepares this instance for subsequent calls after the style and node have been initialized.
Upon invocation the {@link TemplateNodeStyleRenderer#style} and {@link TemplateNodeStyleRenderer#node} properties have been populated by the
{@link TemplateNodeStyleRenderer#getVisualCreator}, {@link TemplateNodeStyleRenderer#getBoundsProvider},
{@link TemplateNodeStyleRenderer#getHitTestable}, or {@link TemplateNodeStyleRenderer#getMarqueeTestable}
methods.
This is an empty implementation. Subclasses might have to override this method.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Creates the visual by delegating to {@link TemplateNodeStyleBase#createTemplate} and then initializing the control
with the context.
@param context The context for the creation.
@returns The visual that is used for rendering the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23createVisual(IRenderContext) Online Documentation}*/
createVisual(context: IRenderContext): Visual | null;
/**
Returns the layout bounds as an upper bound for the painting bound.
This implementation simply sets the {@link TemplateNodeStyleRenderer#layout}'s bounds to the scratch variable and returns.
@param context the context to calculate the bounds for
@returns the bounds or <code>null</code> to indicate an unbound area
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getBounds(ICanvasContext) Online Documentation}*/
getBounds(context: ICanvasContext): Rect;
/**
Provides a {@link IBoundsProvider bounds calculation} implementation for the given style and node.
If {@link TemplateNodeStyleBase#contextLookup} provides an implementation of the {@link IBoundsProvider} interface than this instance will be returned,
otherwise the default (rectangular) behavior will be used.
@param node The node to query the provider for.
@param style The style whose provider is queried.
@returns An implementation that can calculate visual bounds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getBoundsProvider(INode%2CINodeStyle) Online Documentation}*/
getBoundsProvider(node: INode, style: INodeStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link TemplateNodeStyleRenderer#configure} and returns
<code>this</code>.
As this method may be called often it will not automatically call {@link TemplateNodeStyleRenderer#configure},
instead subclasses should ensure that in the {@link TemplateNodeStyleRenderer#lookup} method call they should call
{@link TemplateNodeStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an instance that
depends on a correctly configured <code>this</code>.
@param node The node to query the context for. The value will be stored in the {@link TemplateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link TemplateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateNodeStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getContext(INode%2CINodeStyle) Online Documentation}*/
getContext(node: INode, style: INodeStyle): ILookup;
/**
Gets the context by delegating to {@link TemplateNodeStyleBase#contextLookup}.
@param style
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getContextLookup(TemplateNodeStyleBase) Online Documentation}*/
getContextLookup(style: TemplateNodeStyleBase): IContextLookup;
/**
Provides a {@link IHitTestable hit testing} implementation for the given style and node.
If {@link TemplateNodeStyleBase#contextLookup} provides an implementation of the {@link IHitTestable} interface than this instance will be returned, otherwise
the {@link TemplateNodeStyleRenderer#getNormalizedOutline outline shape} will be used to perform the hit testing.
@param node The node to query the tester for.
@param style The style whose tester is queried.
@returns An implementation that can perform the hit testing.
@see TemplateNodeStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getHitTestable(INode%2CINodeStyle) Online Documentation}*/
getHitTestable(node: INode, style: INodeStyle): IHitTestable;
/**
Gets the insets by delegating to {@link TemplateNodeStyleBase#insets}.
@param style
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getInsets(TemplateNodeStyleBase) Online Documentation}*/
getInsets(style: TemplateNodeStyleBase): Insets;
/**
Tries to use the {@link TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to perform the intersection calculation, otherwise reverts to default (rectangular)
behavior.
@param inner
@param outer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getIntersection(Point%2CPoint) Online Documentation}*/
getIntersection(inner: Point, outer: Point): Point | null;
/**
Provides a {@link IMarqueeTestable marquee testing} implementation for the given style and node.
If {@link TemplateNodeStyleBase#contextLookup} provides an implementation of the {@link IMarqueeTestable} interface than this instance will be returned,
otherwise the {@link TemplateNodeStyleRenderer#getNormalizedOutline outline shape} will be used to determine the marquee intersection test with the visual representation of
the node.
@param node The node to query the tester for.
@param style The style whose tester is queried.
@returns An implementation that can test for marquee intersections.
@see TemplateNodeStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getMarqueeTestable(INode%2CINodeStyle) Online Documentation}*/
getMarqueeTestable(node: INode, style: INodeStyle): IMarqueeTestable;
/**
Gets the minimum size by delegating to {@link TemplateNodeStyleBase#minimumSize}.
@param style
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getMinimumSize(TemplateNodeStyleBase) Online Documentation}*/
getMinimumSize(style: TemplateNodeStyleBase): Size;
/**
Gets the normalized outline by delegating to {@link TemplateNodeStyleBase#normalizedOutline}.
@param style
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getNormalizedOutline(TemplateNodeStyleBase) Online Documentation}*/
getNormalizedOutline(style: TemplateNodeStyleBase): GeneralPath | null;
/**
Tries to use the {@link TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to create the outline shape path, otherwise reverts to default (rectangular) behavior.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getOutline() Online Documentation}*/
getOutline(): GeneralPath | null;
/**
Provides the geometry for the visual representation.
If {@link TemplateNodeStyleBase#contextLookup} provides an implementation of the {@link IShapeGeometry} interface than this instance will be returned,
otherwise the {@link TemplateNodeStyleRenderer#getNormalizedOutline outline shape} will be used to determine the outline of the visual representation of a node.
@param node The node to query the geometry for.
@param style The style for which the geometry is queried.
@returns An implementation that describes the outline geometry of the shape.
@see TemplateNodeStyleRenderer#isInside
@see TemplateNodeStyleRenderer#getOutline
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getShapeGeometry(INode%2CINodeStyle) Online Documentation}*/
getShapeGeometry(node: INode, style: INodeStyle): IShapeGeometry;
/**
Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link TemplateNodeStyleRenderer#configure} and returns
<code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link TemplateNodeStyleRenderer#configure}. If
the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link TemplateNodeStyleRenderer#isVisible}.
@param node The node to query visibility tests. The value will be stored in the {@link TemplateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link TemplateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateNodeStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getVisibilityTestable(INode%2CINodeStyle) Online Documentation}*/
getVisibilityTestable(node: INode, style: INodeStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>node</code> parameters, calls {@link TemplateNodeStyleRenderer#configure} and returns <code>this</code>.
@param node The node to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link TemplateNodeStyleRenderer#node} property.
@param style The style to associate with the node. The value will be stored in the {@link TemplateNodeStyleRenderer#style} property.
@returns <code>this</code>
@see TemplateNodeStyleRenderer#createVisual
@see TemplateNodeStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23getVisualCreator(INode%2CINodeStyle) Online Documentation}*/
getVisualCreator(node: INode, style: INodeStyle): IVisualCreator;
/**
Tries to use the {@link TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to perform the hit test analysis, otherwise reverts to default (rectangular) behavior.
@param context the context the hit test is performed in
@param location the hit point in world coordinates
@returns whether something has been hit
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23isHit(IInputModeContext%2CPoint) Online Documentation}*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
Tries to use the {@link TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to perform the marquee intersection analysis, otherwise reverts to default (rectangular)
behavior.
@param context
@param rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23isInBox(IInputModeContext%2CRect) Online Documentation}*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
Tries to use the {@link TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to perform the lasso intersection analysis, otherwise reverts to default (rectangular)
behavior.
@param context
@param lassoPath
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23isInPath(IInputModeContext%2CGeneralPath) Online Documentation}*/
isInPath(context: IInputModeContext, lassoPath: GeneralPath): boolean;
/**
Tries to use the {@link TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to perform the contains test, otherwise reverts to default (rectangular) behavior.
@param location
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23isInside(Point) Online Documentation}*/
isInside(location: Point): boolean;
/**
Determines visibility by checking for an intersection with the {@link TemplateNodeStyleRenderer#layout}.
@param context The context.
@param rectangle The current clip
@returns Whether the {@link TemplateNodeStyleRenderer#layout} is visible in the clip.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Implementation of the {@link ILookup#lookup} method.
Delegates to the lookup returned by {@link TemplateNodeStyleRenderer#getContextLookup}, first. Returns special {@link INodeInsetsProvider}
or {@link INodeSizeConstraintProvider} implementations if asked for. Last, it returns this class if it implements
the queried <code>type</code>.
@param type The type to query an instance for.
@returns The implementation or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Updates the component correspondingly.
@param context The context for the creation.
@param oldVisual The currently rendered visual.
@returns The visual that is used for rendering the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23updateVisual(IRenderContext%2CVisual) Online Documentation}*/
updateVisual(context: IRenderContext, oldVisual: Visual | null): Visual | null;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23style Online Documentation}*/
style: TemplateNodeStyleBase;
/**
Gets or sets the currently configured node.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23node Online Documentation}*/
node: INode;
/**
Gets the {@link INode#layout} of the node this renderer is currently configured for.
Returns <code>Node.Layout</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplateNodeStyleRenderer%23layout Online Documentation}*/
readonly layout: IRectangle;
static $class: Class;
private $_TemplateNodeStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplateNodeStyleRenderer;
}
export interface TemplatePortStyleBase extends YObject, IPortStyle, IPropertyObservable, ICloneable {
/**
@inheritdoc*/
readonly renderer: IPortStyleRenderer;
} /**
Base class for port styles that use a SVG snippet as template for the visualization of {@link IPort}s.
@see TemplatePortStyle
@see TemplatePortStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase Online Documentation}*/
export abstract class TemplatePortStyleBase {
/**
Constructs a new instance using the provided <code>renderer</code>.
@param renderer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase Online Documentation}*/
constructor(renderer: TemplatePortStyleRenderer);
/**
Performs a shallow clone of this instance, except for the value of {@link TemplatePortStyleBase#styleTag} which is cloned, too, if it can be cloned.
@returns A clone of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23clone() Online Documentation}*/
clone(): this;
/**
Factory method that is called by the {@link TemplatePortStyleRenderer} to create the visual that will be used for
the display of the port.
@param context The context for which the visual should be created.
@param port The port that will be rendered.
@returns The readily styled visual.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23createTemplate(IRenderContext%2CIPort) Online Documentation}*/
createTemplate(context: IRenderContext, port: IPort): SvgVisual;
/**
Loads all template script elements that have not been parsed, yet, and creates the respective template cache objects.
This method can be called to perform the template loading at a certain point in time, for example during application
startup. By default, a template is loaded lazily when the first visual using this template is built.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23loadAllTemplates() Online Documentation}*/
static loadAllTemplates(): void;
/**
Raises the {@link TemplatePortStyleBase#addPropertyChangedListener PropertyChanged} event.
@param evt The {@link PropertyChangedEventArgs} instance containing the event data.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23onPropertyChanged(PropertyChangedEventArgs) Online Documentation}*/
onPropertyChanged(evt: PropertyChangedEventArgs): void;
/**
Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
@param listener The listener to add.
@see TemplatePortStyleBase#removePropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23PropertyChanged Online Documentation}*/
addPropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
@param listener The listener to remove.
@see TemplatePortStyleBase#addPropertyChangedListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23PropertyChanged Online Documentation}*/
removePropertyChangedListener(listener: (sender: any, args: PropertyChangedEventArgs) => void): void;
/**
Gets or sets the tag that is associated with this style instance.
The various style renderer implementations can use this tag to make it accessible in the context of the instantiated
templates and controls. The framework does not make any assumptions about the value of this property. It is there for
convenient use in the process of data binding the templates. This property supports {@link IPropertyObservable property change notification} and can
thus be data bound easily.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23styleTag Online Documentation}*/
styleTag: any | null;
/**
Gets or sets an implementation of {@link IContextLookup} that can be used to satisfy queries that are made to the
implementation which is returned by calls to {@link IPortStyleRenderer#getContext} that are made on the style
renderer that is associated with this style instance.
This can be used to conveniently customize the lookup behavior of the style renderers that are associated with this
instance. E.g. it is possible to provide customized implementations of {@link ISelectionIndicatorInstaller},
{@link IHighlightIndicatorInstaller}, {@link IHandleProvider}, {@link INodeSizeConstraintProvider}, and
{@link INodeInsetsProvider}. Simple style implementations can use {@link IContextLookup#EMPTY_CONTEXT_LOOKUP}
but may not use <code>null</code> for this property. The default value is {@link IContextLookup#EMPTY_CONTEXT_LOOKUP}
@default 'Lookups.EmptyContextLookup'
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23contextLookup Online Documentation}*/
contextLookup: IContextLookup;
/**
Gets or sets the {@link Size size} of the port that will be displayed using the {@link TemplatePortStyleBase#createTemplate created visual}.
This size will be used to satisfy requests to {@link IMarqueeTestable#isInBox},
{@link ILassoTestable#isInPath}, {@link IBoundsProvider#getBounds}, and
{@link IHitTestable#isHit}. The default value is (5,5).
@default '5,5'
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23renderSize Online Documentation}*/
renderSize: Size;
/**
Gets or sets an offset for the port visualization.
This offset will be used to satisfy requests to {@link IMarqueeTestable#isInBox},
{@link ILassoTestable#isInPath}, {@link IBoundsProvider#getBounds}, and
{@link IHitTestable#isHit}. This can be used to have a port visualization that is not centered on the port's
location, e.g. to get a port visualization that extends outside of the node, while the port lies on the node outline.
The default value is (0,0).
@default '0,0'
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23offset Online Documentation}*/
offset: Point;
/**
Gets or sets a path that specifies the normalized outline of a port that will be displayed using the {@link TemplatePortStyleBase#createTemplate created visual}.
The shape will automatically be arranged and transformed to have the bounds of the port. This shape will be used to
satisfy requests to {@link IMarqueeTestable#isInBox} and {@link IHitTestable#isHit} and may be left <code>null</code> to
indicate default (rectangular) behavior.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleBase%23normalizedOutline Online Documentation}*/
normalizedOutline: GeneralPath | null;
static $class: Class;
private $_TemplatePortStyleBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplatePortStyleBase;
}
export interface TemplatePortStyle extends TemplatePortStyleBase {
}
/**
An {@link IPortStyle} that uses an SVG snippet as template for the visualization of {@link IPort}s.
This template style uses the {@link TemplatePortStyle#styleResourceKey} property to determine the template that will be applied to the {@link Visual}. The
template must be specified in the DOM in a script element with the attribute <code>type='text/yfiles-template'</code>. See the {@link StringTemplatePortStyle}
class for a template style that gets its template as string.
Note that the {@link TemplatePortStyleBase#renderSize} should be specified to match the size of the visual's template.
The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
value to the value of the property with the given name in the port's tag. The tag itself can be bound with
<code>attribute="{Binding}"</code>. If the bound object provides the function <code>addPropertyChangedListener(listener)</code> or implements
{@link IPropertyObservable}, bindings to properties of the business object are dynamically updated if the registered
listeners are called on property change. Business objects can be made observable by calling
{@link TemplatePortStyle#makeObservable}.
TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to one of the
properties of the port that are exposed by the {@link ITemplateStyleBindingContext} interface.
Bindings can specify converters and converter parameters. Converters are simple functions with the signature
<code>function(value:Object, parameter:Object):Object</code>. A converter has to be registered under its name either in the {@link TemplatePortStyle#CONVERTERS converter store}
or with the <code>window</code> object before it can be used.
<code>TextWrapConverter</code> is a built-in converter function which emits wrapped text as SVG elements. It can be used to display
wrapped text in a template style. To use the <code>TextWrapConverter</code> the template style must include a &lt;g> element that binds
a property containing the text to be wrapped to <code>data-content</code>. <code>TextWrapConverter</code> must be specified as the converter
function. The converter’s parameters are passed using a shorthand notation for width, height, font, trimming and color.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/_text_wrapping_in_a_template_style Text Wrapping in a Template Style} for more details.
Depending on the current state, the following CSS classes are added to the root element of the template:
<ul>
<li><code>yfiles-selected</code>
<li><code>yfiles-focused</code>
<li><code>yfiles-highlighted</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyle Online Documentation}*/
export class TemplatePortStyle {
/**
Creates a new instance using the given render template id and a custom renderer instance.
@param [renderTemplateId=null] The id of the SVG snippet that will be used for the component.
@param [renderer=null] The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyle Online Documentation}*/
constructor(renderTemplateId?: string | null, renderer?: TemplatePortStyleRenderer);
/**
Creates a new instance using the given render template id and a custom renderer instance.
@param options The parameters to pass.
@param [options.renderTemplateId=null] The id of the SVG snippet that will be used for the component.
@param [options.renderer=null] The custom renderer for this style instance.
@param [options.normalizedOutline] A path that specifies the normalized outline of a port that will be displayed using the {@link TemplatePortStyleBase#createTemplate created visual}. This option sets the {@link TemplatePortStyleBase#normalizedOutline} property on the created object.
@param [options.renderSize] The {@link Size size} of the port that will be displayed using the {@link TemplatePortStyleBase#createTemplate created visual}. This option sets the {@link TemplatePortStyleBase#renderSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyle Online Documentation}*/
constructor(options: {
renderTemplateId?: string | null;
renderer?: TemplatePortStyleRenderer;
normalizedOutline?: GeneralPath | null;
renderSize?: Size | SizeConvertible;
});
/**
Makes an object observable to bindings in template styles.
Bindings to observable objects react to changes in the bound data. This enables dynamic styling of graph items that
reacts to changes of the underlying business data of each graph item.
This method adds the functions <code>firePropertyChanged</code>, <code>addPropertyChangedListener</code> and <code>removePropertyChangedListener</code> to the
given object. Call <code>firePropertyChanged(propertyName: string)</code> each time a bound property value changes in order to update
the related bindings.
@param dataObject The object to make observable.
@returns The object that was passed in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyle%23makeObservable(Object) Online Documentation}*/
static makeObservable(dataObject: Object): Object;
/**
Gets or sets the {@link string} that will be used to look up the template that is applied to the component.
The default value is <code>null</code>.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyle%23styleResourceKey Online Documentation}*/
styleResourceKey: string | null;
/**
Gets the storage object for binding converters.
Binding converter functions can be stored in this object. Once a converter function name is resolved during template
parsing, the converter function is located in this object first. If it is not found in this object, it is located on the
window object. The binding converter store is shared among all template styles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyle%23CONVERTERS Online Documentation}*/
static readonly CONVERTERS: any;
static $class: Class;
private $_TemplatePortStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplatePortStyle;
}
export interface TemplatePortStyleRenderer extends YObject, IPortStyleRenderer, IBoundsProvider, IVisibilityTestable, IMarqueeTestable, ILassoTestable, IHitTestable, ILookup, IVisualCreator {
/**
@inheritdoc*/
getBounds(context: ICanvasContext): Rect;
/**
@inheritdoc*/
isHit(context: IInputModeContext, location: Point): boolean;
/**
@inheritdoc*/
isInBox(context: IInputModeContext, rectangle: Rect): boolean;
/**
@inheritdoc*/
isInPath(context: IInputModeContext, lassoPath: GeneralPath): boolean;
} /**
An implementation of the {@link IPortStyleRenderer} interface that can render {@link TemplatePortStyleBase}
instances.
<h2>Note</h2>
It is not recommended to inherit from this class because it is not guaranteed that the visuals are updated correctly in
this case.
@see TemplatePortStyle
@see StringTemplatePortStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer Online Documentation}*/
export class TemplatePortStyleRenderer {
constructor();
/**
Prepares this instance for subsequent calls after the style and port have been initialized.
Upon invocation the {@link TemplatePortStyleRenderer#style} and {@link TemplatePortStyleRenderer#port} properties have been populated by the
{@link TemplatePortStyleRenderer#getVisualCreator}, {@link TemplatePortStyleRenderer#getBoundsProvider},
{@link TemplatePortStyleRenderer#getHitTestable}, or {@link TemplatePortStyleRenderer#getMarqueeTestable}
methods.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23configure() Online Documentation}*/
configure(): void;
/**
Creates the visual by delegating to {@link TemplatePortStyleBase#createTemplate} and then initializing the visual
with the context.
@param context The context for the creation.
@returns The visual that is used for rendering the port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23createVisual(IRenderContext) Online Documentation}*/
createVisual(context: IRenderContext): Visual | null;
/**
Configures the <code>style</code> and <code>port</code> parameters, calls {@link TemplatePortStyleRenderer#configure} and returns <code>this</code>.
@param port The port to retrieve the bounds provider for. The value will be stored in the {@link TemplatePortStyleRenderer#port} property.
@param style The style to associate with the port. The value will be stored in the {@link TemplatePortStyleRenderer#style} property.
@returns <code>this</code>
@see TemplatePortStyleRenderer#getBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23getBoundsProvider(IPort%2CIPortStyle) Online Documentation}*/
getBoundsProvider(port: IPort, style: IPortStyle): IBoundsProvider;
/**
Configures the <code>style</code> and <code>port</code> parameters, does <b>not</b> call {@link TemplatePortStyleRenderer#configure} and returns
<code>this</code>.
As this method may be called often it will not automatically call {@link TemplatePortStyleRenderer#configure},
instead subclasses should ensure that in the {@link TemplatePortStyleRenderer#lookup} method call they should call
{@link TemplatePortStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an instance that
depends on a correctly configured <code>this</code>.
@param port The port to query the context for. The value will be stored in the {@link TemplatePortStyleRenderer#port} property.
@param style The style to associate with the port. The value will be stored in the {@link TemplatePortStyleRenderer#style} property.
@returns <code>this</code>
@see TemplatePortStyleRenderer#lookup
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23getContext(IPort%2CIPortStyle) Online Documentation}*/
getContext(port: IPort, style: IPortStyle): ILookup;
/**
Configures the <code>style</code> and <code>port</code> parameters, calls {@link TemplatePortStyleRenderer#configure} and returns <code>this</code>.
@param port The port to query hit test with. The value will be stored in the {@link TemplatePortStyleRenderer#port} property.
@param style The style to associate with the port. The value will be stored in the {@link TemplatePortStyleRenderer#style} property.
@returns <code>this</code>
@see TemplatePortStyleRenderer#isHit
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23getHitTestable(IPort%2CIPortStyle) Online Documentation}*/
getHitTestable(port: IPort, style: IPortStyle): IHitTestable;
/**
Configures the <code>style</code> and <code>port</code> parameters, calls {@link TemplatePortStyleRenderer#configure} and returns <code>this</code>.
@param port The port to query marquee intersection tests. The value will be stored in the {@link TemplatePortStyleRenderer#port} property.
@param style The style to associate with the port. The value will be stored in the {@link TemplatePortStyleRenderer#style} property.
@returns <code>this</code>
@see TemplatePortStyleRenderer#isInBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23getMarqueeTestable(IPort%2CIPortStyle) Online Documentation}*/
getMarqueeTestable(port: IPort, style: IPortStyle): IMarqueeTestable;
/**
Gets the normalized outline by delegating to {@link TemplatePortStyleBase#normalizedOutline}.
@param style
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23getNormalizedOutline(TemplatePortStyleBase) Online Documentation}*/
getNormalizedOutline(style: TemplatePortStyleBase): GeneralPath | null;
/**
Gets the offset for the visualization.
@returns The Offset.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23getOffset() Online Documentation}*/
getOffset(): Point;
/**
Gets the size of the rendering from the style.
@returns The {@link TemplatePortStyleBase#renderSize}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23getRenderSize() Online Documentation}*/
getRenderSize(): Size;
/**
Configures the <code>style</code> and <code>port</code> parameters, does <b>not</b> call {@link TemplatePortStyleRenderer#configure} and returns
<code>this</code>.
Unlike most of the other methods this implementation does <b>not</b> call {@link TemplatePortStyleRenderer#configure}. If
the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
{@link TemplatePortStyleRenderer#isVisible}.
@param port The port to query visibility tests. The value will be stored in the {@link TemplatePortStyleRenderer#port} property.
@param style The style to associate with the port. The value will be stored in the {@link TemplatePortStyleRenderer#style} property.
@returns <code>this</code>
@see TemplatePortStyleRenderer#isVisible
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23getVisibilityTestable(IPort%2CIPortStyle) Online Documentation}*/
getVisibilityTestable(port: IPort, style: IPortStyle): IVisibilityTestable;
/**
Configures the <code>style</code> and <code>port</code> parameters, calls {@link TemplatePortStyleRenderer#configure} and returns <code>this</code>.
@param port The port to retrieve the {@link IVisualCreator} for. The value will be stored in the {@link TemplatePortStyleRenderer#port} property.
@param style The style to associate with the port. The value will be stored in the {@link TemplatePortStyleRenderer#style} property.
@returns <code>this</code>
@see TemplatePortStyleRenderer#createVisual
@see TemplatePortStyleRenderer#updateVisual
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23getVisualCreator(IPort%2CIPortStyle) Online Documentation}*/
getVisualCreator(port: IPort, style: IPortStyle): IVisualCreator;
/**
Calls {@link TemplatePortStyleRenderer#configure} and test the {@link TemplatePortStyleRenderer#port} using the {@link TemplatePortStyleRenderer#style} for a visible intersection with the
clip bounds.
This simple implementation uses {@link TemplatePortStyleRenderer#getBounds} to determine the visibility.
Subclasses may choose to override this behavior for improved performance but need to <b>make sure to call {@link TemplatePortStyleRenderer#configure}</b> if they depend on a configured
instance.
@param context the current canvas context
@param rectangle the clip bounds
@returns <code>true</code> if the port may be painted within the clip bounds.
@see TemplatePortStyleRenderer#getVisibilityTestable
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23isVisible(ICanvasContext%2CRect) Online Documentation}*/
isVisible(context: ICanvasContext, rectangle: Rect): boolean;
/**
Implements the {@link ILookup} interface.
This method will be used by default if {@link TemplatePortStyleRenderer#getContext} has been queried for a lookup
implementation. Note that it cannot be assumed that {@link TemplatePortStyleRenderer#configure} has already been
invoked. However, normally {@link TemplatePortStyleRenderer#port} and {@link TemplatePortStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link TemplatePortStyleRenderer#configure}
on success and return <code>this</code>.
@param type The type to query for.
@returns An implementation or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23lookup(Class) Online Documentation}*/
lookup(type: Class): Object | null;
/**
Delegates to the {@link TemplatePortStyleBase}'s {@link TemplatePortStyleBase#contextLookup}.
@param style
@param port
@param type
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23lookupContext(TemplatePortStyleBase%2CIPort%2CClass) Online Documentation}*/
lookupContext(style: TemplatePortStyleBase, port: IPort, type: Class): any | null;
/**
Updates the visual correspondingly.
@param context The context for the creation.
@param oldVisual The currently rendered visual.
@returns The visual that is used for rendering the port.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23updateVisual(IRenderContext%2CVisual) Online Documentation}*/
updateVisual(context: IRenderContext | null, oldVisual: Visual): Visual | null;
/**
Gets the center of the port.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23center Online Documentation}*/
readonly center: Point;
/**
Gets or sets the currently configured style.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23style Online Documentation}*/
style: TemplatePortStyleBase;
/**
Gets or sets the currently configured port.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemplatePortStyleRenderer%23port Online Documentation}*/
port: IPort;
static $class: Class;
private $_TemplatePortStyleRenderer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemplatePortStyleRenderer;
}
export interface StringTemplatePortStyle extends TemplatePortStyleBase {
}
/**
An {@link IPortStyle} that uses an SVG snippet as template for the visualization of {@link IPort}s.
In contrast to {@link TemplatePortStyle}, the SVG template used by this style must be encoded as string and
assigned to the {@link StringTemplatePortStyle#svgContent} property. Note that the RenderSize should be specified to match the size of the visual's template.
It is vital that the {@link StringTemplatePortStyle#svgContent SVG snippet} does not declare the SVG namespace itself. Its content is parsed in a dummy namespace
in order to avoid immediate validation of the result. The resulting SVG elements are automatically created in the SVG
namespace before they are added to the DOM.
The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
value to the value of the property with the given name in the port's tag. The tag itself can be bound with
<code>attribute="{Binding}"</code>. If the bound object provides the function <code>addPropertyChangedListener(listener)</code> or implements
{@link IPropertyObservable}, bindings to properties of the business object are dynamically updated if the registered
listeners are called on property change. Business objects can be made observable by calling
{@link StringTemplatePortStyle#makeObservable}.
TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to one of the
properties of the port that are exposed by the {@link ITemplateStyleBindingContext} interface.
Bindings can specify converters and converter parameters. Converters are simple functions with the signature
<code>function(value:Object, parameter:Object):Object</code>. A converter has to be registered under its name either in the {@link StringTemplatePortStyle#CONVERTERS converter store}
or with the <code>window</code> object before it can be used.
<code>TextWrapConverter</code> is a built-in converter function which emits wrapped text as SVG elements. It can be used to display
wrapped text in a template style. To use the <code>TextWrapConverter</code> the template style must include a &lt;g> element that binds
a property containing the text to be wrapped to <code>data-content</code>. <code>TextWrapConverter</code> must be specified as the converter
function. The converter’s parameters are passed using a shorthand notation for width, height, font, trimming and color.
See {@link https://docs.yworks.com/yfileshtml/#/dguide/_text_wrapping_in_a_template_style Text Wrapping in a Template Style} for more details.
Depending on the current state, the following CSS classes are added to the root element of the template depending:
<ul>
<li><code>yfiles-selected</code>
<li><code>yfiles-focused</code>
<li><code>yfiles-highlighted</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplatePortStyle Online Documentation}*/
export class StringTemplatePortStyle {
/**
Creates a new instance using the provided renderer.
@param [svgContent=null] The SVG snippet to use as template.
@param [renderer=null] The custom renderer for this style instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplatePortStyle Online Documentation}*/
constructor(svgContent?: string | null, renderer?: TemplatePortStyleRenderer);
/**
Creates a new instance using the provided renderer.
@param options The parameters to pass.
@param [options.svgContent=null] The SVG snippet to use as template.
@param [options.renderer=null] The custom renderer for this style instance.
@param [options.normalizedOutline] A path that specifies the normalized outline of a port that will be displayed using the {@link TemplatePortStyleBase#createTemplate created visual}. This option sets the {@link TemplatePortStyleBase#normalizedOutline} property on the created object.
@param [options.renderSize] The {@link Size size} of the port that will be displayed using the {@link TemplatePortStyleBase#createTemplate created visual}. This option sets the {@link TemplatePortStyleBase#renderSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplatePortStyle Online Documentation}*/
constructor(options: {
svgContent?: string | null;
renderer?: TemplatePortStyleRenderer;
normalizedOutline?: GeneralPath | null;
renderSize?: Size | SizeConvertible;
});
/**
Makes an object observable to bindings in template styles.
Bindings to observable objects react to changes in the bound data. This enables dynamic styling of graph items that
reacts to changes of the underlying business data of each graph item.
This method adds the functions <code>firePropertyChanged</code>, <code>addPropertyChangedListener</code> and <code>removePropertyChangedListener</code> to the
given object. Call <code>firePropertyChanged(propertyName: string)</code> each time a bound property value changes in order to update
the related bindings.
@param dataObject The object to make observable.
@returns The object that was passed in.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplatePortStyle%23makeObservable(Object) Online Documentation}*/
static makeObservable(dataObject: Object): Object;
/**
Gets or sets the SVG content that is the template for the port visualization.
@default null
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplatePortStyle%23svgContent Online Documentation}*/
svgContent: string | null;
/**
Specifies whether code contained in the templates should be included in the DOM.
Script tags and event listeners can be used to inject malicious code. This property should only be enabled if the
templates originate from trusted sources. If the application allows to load external GraphML files, this property should
probably not be enabled.
This setting is shared by {@link StringTemplateNodeStyle}, {@link StringTemplateLabelStyle} and
StringStripeControlPortStyle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplatePortStyle%23trusted Online Documentation}*/
static trusted: boolean;
/**
Gets the storage object for binding converters.
Binding converter functions can be stored in this object. Once a converter function name is resolved during template
parsing, the converter function is located in this object first. If it is not found in this object, it is located on the
window object. The binding converter store is shared among all template styles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StringTemplatePortStyle%23CONVERTERS Online Documentation}*/
static readonly CONVERTERS: any;
static $class: Class;
private $_StringTemplatePortStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StringTemplatePortStyle;
}
export interface AbortHandler extends YObject {
}
/**
This class provides a means for early termination of graph algorithms.
Instances of this class may be attached to and retrieved from a graph and may receive requests for stopping and
canceling an algorithm that is currently executed on the given graph.
<b>Client Code Usage</b>
An instance of this class can be attached to a graph using method {@link AbortHandler#createForGraph}. The
handler's {@link AbortHandler#stopDuration} and {@link AbortHandler#cancelDuration} methods can be used to automatically stop or cancel an algorithm after a specified period of time has
elapsed. Algorithms can check for these requests and handle them appropriately.
<ul>
<li><b>Stop</b> – The algorithm should terminate gracefully, delivering a consistent result. Although the termination will be early, it
will usually not be immediate.
<li><b>Cancel</b> – The algorithm should terminate immediately and all work done so far will be discarded.
<b>IMPORTANT:</b> It is not guaranteed that the processed graph will be in a consistent state after cancellation. For this reason, it is
strongly recommended to cancel only algorithms that work on copies of the real graph structure such as layout algorithms
running in buffered mode. Furthermore, the state of the used {@link ILayoutAlgorithm} instance may become
corrupted. Hence, a new instance has to be created after each cancellation.
</ul>
If a graph with an attached handler is processed by multiple algorithms (or multiple times by the same algorithm), the
attached handler has to be {@link AbortHandler#reset} between algorithm runs. Otherwise, previous requests for
early termination may lead to an undesired early termination of the next algorithm run.
<b>Usage in Algorithms</b>
Algorithms have to retrieve an instance of this class from the graph that is processed using method
{@link AbortHandler#getFromGraph}. Then, the algorithm needs to query the retrieved instance of this class for
stop or cancel requests using method {@link AbortHandler#check}.
Alternatively, convenience method {@link AbortHandler#check} for one-time checks is available. For performance
critical code that checks repeatedly, it is recommended to follow the first approach, though.
When handling a stop request, algorithms should ensure that the resulting graph is still in a consistent state.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler Online Documentation}*/
export class AbortHandler {
/**
Creates a new {@link AbortHandler} instance.
@see AbortHandler#stopDuration
@see AbortHandler#cancelDuration
@see AbortHandler#reset
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler Online Documentation}*/
constructor();
/**
Determines whether or not an algorithm should terminate immediately.
This method returns <code>true</code> if the algorithm should terminate gracefully and ensures that the processed graph remains in a
consistent state.
@returns <code>true</code>, if the algorithm should terminate immediately, <code>false</code> otherwise
@throws {Exception} <b>(AlgorithmAbortedError)</b> if the algorithm should terminate immediately
@see AbortHandler#stop
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23check() Online Documentation}*/
check(): boolean;
/**
Determines whether or not an algorithm should terminate immediately.
This method returns <code>true</code> if the algorithm should terminate gracefully and ensures that the processed graph remains in a
consistent state.
This convenience method is meant for one-time checks only. For performance critical code that needs to check repeatedly,
it is recommended to retrieve the given graph's attached handler once and only call the handler's {@link AbortHandler#check}
method repeatedly.
@param graph
@returns <code>true</code>, if the algorithm should stop immediately while still providing some valid result, <code>false</code> otherwise
@throws {Exception} <b>(AlgorithmAbortedError)</b> if the algorithm should terminate immediately
@throws {Exception} <b>(NullReferenceError)</b> if the given graph is <code>null</code>
@see AbortHandler#check
@see AbortHandler#stop
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23check(Graph) Online Documentation}*/
static check(graph: Graph | null): boolean;
/**
Attaches the {@link AbortHandler} instance of the given source graph to the target graph as well.
<h2>Note</h2>
If there is an {@link AbortHandler} attached to the given target graph, this method will silently replace that
instance with the one attached to the source graph.
@param source the graph whose handler is attached to the target graph
@param target the graph to which the handler of the source graph is attached
@throws {Exception} <b>(NullReferenceError)</b> if the given source is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23copyHandler(Graph%2CGraph) Online Documentation}*/
static copyHandler(source: Graph | null, target: Graph | null): void;
/**
Creates an {@link AbortHandler handler} instance and attaches it to the given graph.
If the given graph has already an attached handler instance, this is the instance that will be returned and this method
will not create a new {@link AbortHandler}.
This method should be called by client code prior to starting a graph algorithm that may be terminated early.
@param graph the graph to which the handler will be attached
@returns the {@link AbortHandler} instance for the given graph
@throws {Exception} <b>(NullReferenceError)</b> if the given graph is <code>null</code>.
@see AbortHandler#hasHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23createForGraph(Graph) Online Documentation}*/
static createForGraph(graph: Graph | null): AbortHandler | null;
/**
Returns an {@link AbortHandler} instance for the given graph.
If {@link AbortHandler#createForGraph} has been used for attaching a new handler to the given graph, this is
the instance that will be returned. Otherwise, a non-functional instance is returned whose methods do nothing. Use {@link AbortHandler#hasHandler}
to check whether or not a handler has been already attached to the given graph.
@param graph the given graph
@returns an {@link AbortHandler} for the given graph or a non-functional instance if no handler has been previously
created
@throws {Exception} <b>(NullReferenceError)</b> if the given graph is <code>null</code>
@see AbortHandler#createForGraph
@see AbortHandler#hasHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23getFromGraph(Graph) Online Documentation}*/
static getFromGraph(graph: Graph | null): AbortHandler | null;
/**
Determines whether or not an {@link AbortHandler} instance is attached to the given graph.
@param graph the given graph
@returns <code>true</code> if a handler is attached to the given graph, <code>false</code> otherwise
@throws {Exception} <b>(NullReferenceError)</b> if the given graph is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23hasHandler(Graph) Online Documentation}*/
static hasHandler(graph: Graph | null): boolean;
/**
Removes any attached {@link AbortHandler} instance from the given graph.
@param graph the given graph
@throws {Exception} <b>(NullReferenceError)</b> if the given graph is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23removeFromGraph(Graph) Online Documentation}*/
static removeFromGraph(graph: Graph | null): void;
/**
Resets the state of the {@link AbortHandler handler}.
Resetting the {@link AbortHandler handler} discards any previous stop or cancel requests. Moreover, the handler's internal timestamp that is
used for determining whether or not an algorithm should be stopped or cancelled automatically is reset as well.
This method should be called whenever a graph with an attached handler is processed an additional time to prevent
previous requests for early termination to result in an undesired early termination of the next algorithm run.
@see AbortHandler#cancelDuration
@see AbortHandler#stopDuration
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23reset() Online Documentation}*/
reset(): void;
/**
Schedules a stop request.
Algorithms that detect stop requests should terminate gracefully and ensure that the processed graph remains in a
consistent state.
If a Cancel request has already been scheduled for this handler, the stop request is ignored. If a Cancel request is
scheduled later on, the stop request is overridden.
@see AbortHandler#check
@see AbortHandler#cancelDuration
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23stop() Online Documentation}*/
stop(): void;
/**
Determines the remaining time (in milliseconds) until an algorithm that {@link AbortHandler#check checks} this handler is cancelled automatically.
@returns the remaining time until the algorithm is cancelled automatically.
@see AbortHandler#cancelDuration
@see AbortHandler#reset
@see AbortHandler#cancelDuration
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23timeToCancel() Online Documentation}*/
timeToCancel(): TimeSpan;
/**
Determines the remaining time until an algorithm that {@link AbortHandler#check checks} this handler is stopped automatically.
If the {@link AbortHandler#stopDuration stop duration} is less than or equal to zero, <code>-1</code> will be returned which means that the algorithm may run
unrestricted.
@returns the remaining time until the algorithm is stopped automatically or <code>-1</code> if the algorithm may run unrestricted
@see AbortHandler#stopDuration
@see AbortHandler#reset
@see AbortHandler#stop
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23timeToStop() Online Documentation}*/
timeToStop(): TimeSpan;
/**
Data provider key for attaching an AbortHandler instance to a graph
Only instances of {@link AbortHandler} should be assigned to this {@link IDataProvider}, otherwise a
{@link Exception} will occur. Layout algorithms will use the attached handler to check for requests to cancel or
stop the layout process.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23ABORT_HANDLER_DP_KEY Online Documentation}*/
static ABORT_HANDLER_DP_KEY: GraphDpKey<AbortHandler> | null;
/**
Returns whether or not a stop request was scheduled explicitly with the {@link AbortHandler#stop} method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23stopRequested Online Documentation}*/
readonly stopRequested: boolean;
/**
Gets or sets the duration an algorithm may run before being stopped automatically.
An algorithm is terminated gracefully, if the time in between {@link AbortHandler creating} or {@link AbortHandler#reset resetting} this handler and calling {@link AbortHandler#check}
exceeds the stop duration.
<h2>Note</h2>
Automatic termination will only occur for positive values.
@throws {Exception} <b>(ArgumentError)</b> if the duration is negative
@see AbortHandler#timeToStop
@see AbortHandler#reset
@see AbortHandler#stop
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23stopDuration Online Documentation}*/
stopDuration: TimeSpan;
/**
Gets or sets the duration an algorithm may run before being cancelled automatically.
An algorithm is terminated immediately, if the time in between {@link AbortHandler creating} or {@link AbortHandler#reset resetting} this handler and calling {@link AbortHandler#check}
exceeds the cancel duration.
<h2>Note</h2>
Automatic termination will only occur for positive values.
@throws {Exception} <b>(ArgumentError)</b> if the duration is negative
@see AbortHandler#reset
@see AbortHandler#cancelDuration
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23cancelDuration Online Documentation}*/
cancelDuration: TimeSpan;
/**
Gets whether or not methods {@link AbortHandler#check} or {@link AbortHandler#check} were called
after a stop or cancel event.
More precisely, it returns <code>true</code> if one of the check methods either threw an {@link Exception AlgorithmAbortedError} or returned <code>true</code> to
indicate that the calling algorithm should terminate gracefully. Otherwise, this method returns <code>false</code>.
@see AbortHandler#check
@see AbortHandler#check
@see {@link https://docs.yworks.com/yfileshtml/#/api/AbortHandler%23checkFailed Online Documentation}*/
readonly checkFailed: boolean;
static $class: Class;
private $_AbortHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AbortHandler;
}
export interface AggregationInfo extends YObject {
}
/**
The result of a {@link NodeAggregationAlgorithm#aggregate} run is a hierarchical nested clustering structure.
Such a structure can be modeled by a directed tree which we encode by means of a set of {@link AggregationInfo}
instances.
More precisely, each node of the original graph is mapped to a unique {@link AggregationInfo} instance. The
mapping is stored in the {@link INodeMap} that is passed to method
{@link NodeAggregationAlgorithm#aggregate}. Each {@link AggregationInfo} has a reference to its {@link AggregationInfo#parentAggregation parent} which
induces a tree structure. There is always exactly one {@link AggregationInfo} without a parent that represents
the root of the tree. Property {@link NodeAggregationAlgorithm#nodesOnlyOnLeaves} allows to specify whether nodes are only mapped to leaves of the tree structure or if
they can also be mapped to inner nodes.
Note that there are {@link AggregationInfo} instances that are not associated with a node of the original
graph. These instances always represent an inner element of the tree structure.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AggregationInfo Online Documentation}*/
export class AggregationInfo {
/**
Gets the {@link AggregationInfo parent info} of this info.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AggregationInfo%23parentAggregation Online Documentation}*/
readonly parentAggregation: AggregationInfo | null;
/**
Gets the number of nodes of the original graph that are descendants of this aggregate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AggregationInfo%23descendantCount Online Documentation}*/
readonly descendantCount: number;
/**
Gets the sum of the weights of nodes of the original graph that are descendants of this aggregate.
<h2>Note</h2>
The weight of a node corresponds to its custom weight (see {@link NodeAggregationAlgorithm#NODE_WEIGHT_DP_KEY}) or, if
no custom weights are specified, to its {@link CentralityAlgorithm#pageRank}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AggregationInfo%23descendantWeightSum Online Documentation}*/
readonly descendantWeightSum: number;
/**
Gets the maximum weight of a node of the original graph that is a descendant of this aggregate.
<h2>Note</h2>
The weight of a node corresponds to its custom weight (see {@link NodeAggregationAlgorithm#NODE_WEIGHT_DP_KEY}) or, if
no custom weights are specified, to its {@link CentralityAlgorithm#pageRank}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AggregationInfo%23maximumDescendantWeight Online Documentation}*/
readonly maximumDescendantWeight: number;
static $class: Class;
private $_AggregationInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AggregationInfo;
}
export interface BfsAlgorithm extends YObject {
}
/**
This class provides services that center around breadth first search (BFS).
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To run a breadth-first search on {@link IGraph}
instances use {@link Bfs} instead.
Breadth first search starts at a given set of nodes and explores the neighboring nodes first, before visiting the next
level neighbors.
A breadth first search run can be either directed or undirected. All methods require a list of nodes that are considered
as the core nodes from which the breadth first search starts. These nodes can be identified either with a {@link YNodeList}
or with a {@link IDataProvider} that returns <code>true</code> for core nodes and <code>false</code> for all other nodes. The output is
given as an array of {@link YNodeList}s each of which contains the nodes of a particular layer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsAlgorithm Online Documentation}*/
export class BfsAlgorithm {
/**
Returns the layers of nodes constructed by a directed/undirected breadth first search where the maximum number of layers
is restricted.
<b>Note:</b> This method works with instances of {@link Graph}. To run a breadth-first search on {@link IGraph}
instances use {@link Bfs} instead.
The first of these layers contains all nodes within the given {@link YNodeList}. These nodes are the core nodes
from which a breadth first search to the other nodes starts. The algorithm fills the provided {@link INodeMap}
with an integer indicating the layer index of each node (all core nodes have layer index <code>0</code>).
In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
<h2>Note</h2>
If the number of layers is restricted, the returned {@link YNodeList} may only contain a subset of nodes (i.e.,
only those with layer index &lt; max layer count).
@param graph the given graph
@param coreNodes the list of core nodes
@param direction one of the predefined direction specifiers
@param layerIDMap the {@link INodeMap} that will be filled during the BFS execution and holds the zero-based index of the BFS
layer to which each node belongs or <code>-1</code> if the node is not reachable
@param [maxLayerCount=0] the maximum number of layers that will be returned or <code>0</code> if all layers are required
@returns an array of {@link YNodeList}s each of which contains the nodes of a particular layer
@throws {Exception} <b>(ArgumentError)</b> if the given direction is not supported
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsAlgorithm%23getLayers(Graph%2CYNodeList%2CTraversalDirection%2CINodeMap%2Cnumber) Online Documentation}*/
static getLayers(graph: Graph, coreNodes: YNodeList, direction: TraversalDirection | TraversalDirectionStringValues, layerIDMap: INodeMap, maxLayerCount?: number): YNodeList[] | null;
/**
Returns the layers of nodes constructed by a directed/undirected breadth first search where the maximum number of layers
is restricted.
<b>Note:</b> This method works with instances of {@link Graph}. To run a breadth-first search on {@link IGraph}
instances use {@link Bfs} instead.
The first of these layers contains all nodes within the given {@link YNodeList}. These nodes are the core nodes
from which either a directed or undirected breadth first search to the other nodes starts. The algorithm fills the
provided {@link INodeMap} with an integer indicating the layer of each node.
In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
<h2>Note</h2>
If the number of layers is restricted, the returned {@link YNodeList} may only contain a subset of nodes (i.e.,
only those with layer &lt;= max layer).
@param graph the given graph
@param coreNodes the list of core nodes from which the BFS starts
@param directed <code>true</code> if the graph should be considered directed, <code>false</code> otherwise
@param layerIDMap the {@link INodeMap} that will be filled during the BFS execution and holds the zero-based index of the BFS
layer to which each node belongs or <code>-1</code> if the node is not reachable
@param [maxLayers=0] the number of layers that will be returned or <code>0</code> if all layers are required
@returns an array of {@link YNodeList}s each of which contains the nodes of a particular layer
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsAlgorithm%23getLayers(Graph%2CYNodeList%2Cboolean%2CINodeMap%2Cnumber) Online Documentation}*/
static getLayers(graph: Graph, coreNodes: YNodeList, directed: boolean, layerIDMap: INodeMap, maxLayers?: number): YNodeList[] | null;
/**
Returns the layers of nodes constructed by a breadth first search.
<b>Note:</b> This method works with instances of {@link Graph}. To run a breadth-first search on {@link IGraph}
instances use {@link Bfs} instead.
The first of these layers contains all nodes within the given {@link YNodeList}. These nodes are the core nodes
from which an undirected breadth first search to the other nodes starts. The algorithm fills the provided {@link INodeMap}
with an integer indicating the layer of each node.
In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
@param graph the given graph
@param coreNodes the list of core nodes from which the BFS starts
@param layerIDMap the {@link INodeMap} that will be filled during the BFS execution and holds the zero-based index of the BFS
layer to which each node belongs or <code>-1</code> if the node is not reachable
@returns an array of {@link YNodeList}s each of which contains the nodes of a particular layer
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsAlgorithm%23getLayers(Graph%2CYNodeList%2CINodeMap) Online Documentation}*/
static getLayers(graph: Graph, coreNodes: YNodeList, layerIDMap: INodeMap): YNodeList[] | null;
/**
Returns the layers of nodes constructed by a breadth first search.
<b>Note:</b> This method works with instances of {@link Graph}. To run a breadth-first search on {@link IGraph}
instances use {@link Bfs} instead.
The first of these layers contains all nodes for which the given {@link IDataProvider} returns <code>true</code>. These
nodes are the core nodes from which an undirected breadth first search to the other nodes starts. The algorithm fills
the provided {@link INodeMap} with an integer indicating the layer of each node.
In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
@param graph the given graph
@param isCoreNode the {@link IDataProvider} that contains the nodes from which the BFS starts; core nodes are marked with a <code>true</code>
value
@param layerIDMap the {@link INodeMap} that will be filled during the BFS execution and holds the zero-based index of the BFS
layer to which each node belongs or <code>-1</code> if the node is not reachable
@returns an array of {@link YNodeList}s each of which contains the nodes of a particular layer
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsAlgorithm%23getLayers(Graph%2CIDataProvider%2CINodeMap) Online Documentation}*/
static getLayers(graph: Graph, isCoreNode: IDataProvider, layerIDMap: INodeMap): YNodeList[] | null;
/**
Returns the layers of nodes calculated by a breadth first search.
<b>Note:</b> This method works with instances of {@link Graph}. To run a breadth-first search on {@link IGraph}
instances use {@link Bfs} instead.
The first of these layers contains all nodes within the given
{@link YNodeList}. These nodes are the core nodes from which an undirected breadth first search to the other
nodes starts.
In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
@param graph the given graph
@param coreNodes the list of core nodes from which the BFS starts
@returns an array of {@link YNodeList}s each of which contains the nodes of a particular layer
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsAlgorithm%23getLayers(Graph%2CYNodeList) Online Documentation}*/
static getLayers(graph: Graph, coreNodes: YNodeList): YNodeList[] | null;
/**
Returns the layers of nodes constructed by a breadth first search.
<b>Note:</b> This method works with instances of {@link Graph}. To run a breadth-first search on {@link IGraph}
instances use {@link Bfs} instead.
The first of these layers contains all nodes for which the given {@link IDataProvider} returns
<code>true</code>. These nodes are the core nodes from which an undirected breadth first search to the other nodes starts.
In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
@param graph the given graph
@param isCoreNode the {@link IDataProvider} that contains the nodes from which the BFS starts; core nodes are marked with a <code>true</code>
value
@returns an array of {@link YNodeList}s each of which contains the nodes of a particular layer
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsAlgorithm%23getLayers(Graph%2CIDataProvider) Online Documentation}*/
static getLayers(graph: Graph, isCoreNode: IDataProvider): YNodeList[] | null;
static $class: Class;
private $_Bfs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BfsAlgorithm;
}
export interface BipartitionAlgorithm extends YObject {
}
/**
This class provides methods to determine whether a graph is bipartite and to obtain the corresponding partitions.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To determine a bipartition of an {@link IGraph}
use {@link Bipartition} instead.
A <b>bipartite</b> graph is a graph whose nodes can be partitioned into two sets such that each edges connects two nodes of different
sets. In other words, there are no edges connecting nodes that belong to the same partition.
The two sets/partitions are represented by two {@link Object} constants i.e., {@link BipartitionAlgorithm#RED} or
{@link BipartitionAlgorithm#BLUE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionAlgorithm Online Documentation}*/
export class BipartitionAlgorithm {
/**
Calculates a bipartition of the given graph, if one exists.
<b>Note:</b> This method works with instances of {@link Graph}. To determine a bipartition of an {@link IGraph} use {@link Bipartition}
instead.
If the graph is bipartite, then for all nodes of the given graph either {@link BipartitionAlgorithm#RED} or {@link BipartitionAlgorithm#BLUE}
objects will be set in the given {@link INodeMap}, depending on the partition to which each node belongs.
@param graph the given graph
@param markMap the {@link INodeMap} that will be filled during the BFS execution and returns the partition (either {@link BipartitionAlgorithm#RED}
or {@link BipartitionAlgorithm#BLUE}) to which each node belongs
@returns <code>true</code> if the graph is bipartite, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionAlgorithm%23getBipartition(Graph%2CINodeMap) Online Documentation}*/
static getBipartition(graph: Graph | null, markMap: INodeMap | null): boolean;
/**
Determines whether or not the given graph is bipartite.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is bipartite
use {@link GraphStructureAnalyzer#isBipartite} or {@link Bipartition} instead.
@param graph the given graph
@returns <code>true</code> if the graph is bipartite, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionAlgorithm%23isBipartite(Graph) Online Documentation}*/
static isBipartite(graph: Graph | null): boolean;
/**
A constant for marking a node that belongs to the red partition.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionAlgorithm%23RED Online Documentation}*/
static RED: Object | null;
/**
A constant for marking a node that belongs to the blue partition.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionAlgorithm%23BLUE Online Documentation}*/
static BLUE: Object | null;
static $class: Class;
private $_Bipartitions: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BipartitionAlgorithm;
}
export interface CentralityAlgorithm extends YObject {
}
/**
This class provides methods to determine various centrality indices of nodes or edges of a graph.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To calculate centrality measures for {@link IGraph}
instances use one of the following classes instead:
<ul>
<li>{@link BetweennessCentrality}
<li>{@link ClosenessCentrality}
<li>{@link GraphCentrality}
<li>{@link DegreeCentrality}
<li>{@link WeightCentrality}
<li>{@link EigenvectorCentrality}
<li>{@link PageRank}
</ul>
Centrality indices serve to quantify an intuitive feeling that in most networks some nodes or edges are "more central"
than others. The provided methods assign a double value to each node or edge of a graph that represents its centrality.
The higher an assigned value, the more central the element is considered by the algorithm.
Also, this class provides convenience methods that normalize the returned centrality values such that they lie within
the interval <code>[0,1]</code>.
<b>Definitions</b>
<ul>
<li><b>Betweenness centrality</b> is a measure for how often a node/edge lies on a shortest path between each pair of nodes in the graph.
<li><b>Closeness centrality</b> is the reciprocal of the sum of shortest path distances of a node to all other nodes in the graph.
<li><b>Graph centrality</b> is the reciprocal of the maximum of all shortest path distances from a node to all other nodes in the graph.
<li><b>Degree centrality</b> is the number of the incoming, outgoing or overall edges incident to a node (measures incoming, outgoing and overall
degree).
<li><b>Weight centrality</b> measures the weight associated with incoming, outgoing, or all edges of a node.
<li><b>Eigenvector centrality</b> measures the influence of a node by means of a dominant eigenvector of the graph's adjacency matrix.
<li><b>Page rank</b> measures the relative importance of a node in the graph which is determined based on the edges adjacent to a node. The
basic idea is that nodes that are linked to by many other nodes are important.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm Online Documentation}*/
export class CentralityAlgorithm {
/**
Computes the closeness centrality for the nodes of a graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate closeness centrality for edges in an {@link IGraph}
use {@link ClosenessCentrality} instead.
<i>Closeness centrality</i> is defined as the reciprocal of the sum of shortest path distances of a node to all other nodes in the graph.
Therefore, a node with high closeness centrality has short distances to all other nodes of a graph. If the sum of the
shortest path distances is <code>0</code>, the closeness of a node is set to {@link Number#POSITIVE_INFINITY}.
<h2>Note</h2>
For non-connected graphs, the centrality values of all nodes will be zero, since the distance to some nodes is
infinite.
@param graph the input graph
@param closeness the {@link INodeMap} that will be filled during the execution and returns a double value (centrality) for each
node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param edgeCosts the {@link IDataProvider} that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23closenessCentrality(Graph%2CINodeMap%2Cboolean%2CIDataProvider) Online Documentation}*/
static closenessCentrality(graph: Graph | null, closeness: INodeMap | null, directed: boolean, edgeCosts: IDataProvider | null): void;
/**
Computes the degree centrality for the nodes of a given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate degree centrality for edges in an {@link IGraph}
use {@link DegreeCentrality} instead.
<i>Degree centrality</i> is the number of the incoming, outgoing or overall edges incident to a node (measures incoming, outgoing and overall
degree).
@param graph the input graph
@param centrality the {@link INodeMap} that will be filled during the execution and returns a double value (centrality) for each
node
@param considerInEdges <code>true</code> if the incoming edges should be considered, <code>false</code> otherwise
@param considerOutEdges <code>true</code> if the outgoing edges should be considered, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23degreeCentrality(Graph%2CINodeMap%2Cboolean%2Cboolean) Online Documentation}*/
static degreeCentrality(graph: Graph | null, centrality: INodeMap | null, considerInEdges: boolean, considerOutEdges: boolean): void;
/**
Computes betweenness centrality for each edge of a given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate betweenness centrality for nodes and edges in
an {@link IGraph} use {@link BetweennessCentrality} instead.
<i>Betweenness centrality</i> is a measure for how often an edge lies on a shortest path between each pair of nodes in the graph. Removing a central
edge will cause many shortest paths to change.
@param graph the input graph
@param centrality the {@link IEdgeMap} that will be filled during the execution and returns a double value (centrality) for each
edge
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param edgeCosts the {@link IDataProvider} that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost;
for invalid input values the algorithm uses cost <code>1.0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23edgeBetweenness(Graph%2CIEdgeMap%2Cboolean%2CIDataProvider) Online Documentation}*/
static edgeBetweenness(graph: Graph | null, centrality: IEdgeMap | null, directed: boolean, edgeCosts: IDataProvider | null): void;
/**
Computes an eigenvector centrality for each node of a given undirected graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate the eigenvector centrality for nodes in an {@link IGraph}
use {@link EigenvectorCentrality} instead.
The centrality values are scaled so that the largest centrality value is <code>1.0</code>.
<h2>Note</h2>
For some input graphs the power iteration method that is used to calculate the dominant eigenvector doesn't converge
and, thus, this method doesn't find a valid solution. This is indicated by the return value of this method. In such
cases, we recommend to switch to the {@link CentralityAlgorithm#pageRank} algorithm.
@param graph the undirected input graph
@param centralityMap a node map that can store values of type <code>double</code>. It is used for returning the resulting centrality values (in the
interval <code>[0,1]</code>) for each node
@param [precision=0.001] specifies the precision used during the calculation of the power iteration method (i.e. the maximum possible difference
to consider two values as equal)
@returns whether or not an eigenvector could be found and <code>centralityMap</code> contains valid entries
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23eigenvectorCentrality(Graph%2CINodeMap%2Cnumber) Online Documentation}*/
static eigenvectorCentrality(graph: Graph | null, centralityMap: INodeMap | null, precision?: number): boolean;
/**
Computes the graph centrality for the nodes of a graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate graph centrality for edges in an {@link IGraph}
use {@link GraphCentrality} instead.
<i>Graph centrality</i> is defined as the reciprocal of the maximum of all shortest path distances from a node to all other nodes in the graph.
Nodes with high graph centrality have short distances to all other nodes in the graph.
<h2>Note</h2>
For non-connected graphs, the centrality values of all nodes will be zero since the distance to some nodes is infinite.
@param graph the input graph
@param centrality the {@link INodeMap} that will be filled during the execution and returns a double value (centrality) for each
node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param edgeCosts the {@link IDataProvider} that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23graphCentrality(Graph%2CINodeMap%2Cboolean%2CIDataProvider) Online Documentation}*/
static graphCentrality(graph: Graph | null, centrality: INodeMap | null, directed: boolean, edgeCosts: IDataProvider | null): void;
/**
Computes betweenness centrality for each node of a given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate betweenness centrality for nodes and edges in
an {@link IGraph} use {@link BetweennessCentrality} instead.
<i>Betweenness centrality</i> is a measure for how often a node lies on a shortest path between each pair of nodes in the graph. Removing a central
edge will cause many shortest paths to change.
@param graph the input graph
@param centrality the {@link INodeMap} that will be filled during the execution and returns a double value (centrality) for each
node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param edgeCosts the {@link IDataProvider} that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost;
for invalid input values the algorithm uses cost <code>1.0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23nodeBetweenness(Graph%2CINodeMap%2Cboolean%2CIDataProvider) Online Documentation}*/
static nodeBetweenness(graph: Graph | null, centrality: INodeMap | null, directed: boolean, edgeCosts: IDataProvider | null): void;
/**
Computes betweenness centrality for each node and edge of a given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate betweenness centrality for nodes and edges in
an {@link IGraph} use {@link BetweennessCentrality} instead.
<i>Betweenness centrality</i> is a measure for how often a node/edge lies on a shortest path between each pair of nodes in the graph. Removing a
central node/edge will cause many shortest paths to change.
@param graph the input graph
@param nodeCentrality the {@link INodeMap} that will be filled during the execution and returns a double value (centrality) for each
node
@param edgeCentrality the {@link IEdgeMap} that will be filled during the execution and returns a double value (centrality) for each
edge
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param edgeCosts the {@link IDataProvider} that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost;
for invalid input values the algorithm uses cost <code>1.0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23nodeEdgeBetweenness(Graph%2CINodeMap%2CIEdgeMap%2Cboolean%2CIDataProvider) Online Documentation}*/
static nodeEdgeBetweenness(graph: Graph | null, nodeCentrality: INodeMap | null, edgeCentrality: IEdgeMap | null, directed: boolean, edgeCosts: IDataProvider | null): void;
/**
Normalizes the <code>double</code> values of a given {@link IEdgeMap} by dividing each of them by the maximum of all values
(maximum norm).
<h2>Note</h2>
If the maximum value is <code>Double.POSITIVE_INFINITY</code>, all values other than <code>Double.POSITIVE_INFINITY</code> are set to <code>0</code>.
@param graph the input graph
@param map the {@link IEdgeMap} that will be filled during the execution and returns a double value from <code>[0,1]</code> interval
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23normalizeEdgeMap(Graph%2CIEdgeMap) Online Documentation}*/
static normalizeEdgeMap(graph: Graph | null, map: IEdgeMap | null): void;
/**
Normalizes the <code>double</code> values of a given {@link INodeMap} by dividing each of them by the maximum of all values
(maximum norm).
<h2>Note</h2>
If the maximum value is <code>Double.POSITIVE_INFINITY</code>, all values other than <code>Double.POSITIVE_INFINITY</code> are set to <code>0</code>.
@param graph the input graph
@param map the {@link INodeMap} that will be filled during the execution and returns a double value from <code>[0,1]</code> interval
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23normalizeNodeMap(Graph%2CINodeMap) Online Documentation}*/
static normalizeNodeMap(graph: Graph | null, map: INodeMap | null): void;
/**
Computes page rank values for all nodes based on their attached edges.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate the page rank for nodes in an {@link IGraph}
use {@link PageRank} instead.
The original algorithm was proposed by Sergey Brin and Lawrence Page as PageRank algorithm and refined later in several
papers:
S. Brin and L. Page. The anatomy of a large-scale hypertextual Web search
engine.
<br />
Computer Networks and ISDN Systems, 30(1-7):107-117
<br />
1998.
The algorithm starts by initializing the rank value for each node (see {@link IDataProvider} <code>initialPageRank</code>).
After that it uses multiple iterations to propagate the rank of a node to its successors. The base factor for each
successor is <code>1</code>. These base factors are multiplied by edge weights (see data provider <code>edgeWeight</code> and node weights (see
data provider <code>nodeWeight</code> if specified.
The final node factors are scaled afterwards to sum up to <code>1</code> so the overall rank stays the same after each iteration. The
old page rank of the node multiplied with these scaled factors are then distributed to the successors.
When a node can't distribute its rank, e.g. because it has no successors, its rank is distributed between all nodes
scaled by their weight (see data provider <code>nodeWeight</code>). Furthermore a <code>damping factor</code> is used so that a share of each
nodes' rank isn't propagated to the successors but also distributed between all nodes.
After each iteration, the relative change of each node's page rank ((newRank - oldRank)/oldRank) is calculated. If all
page rank changes are below the specified <code>precision</code> or if the maximal iterations are reached, the algorithm stops.
The <code>edgeDirectedness</code> {@link IDataProvider} defines the direction of the edges. This in consequence defines the
propagation direction of ranks. Edges with a positive directedness value propagate the rank from the source to the
target node - this is also the default if the given data provider is <code>null</code>. Edges which have a directedness of <code>0</code> are
considered to be undirected so that propagation happens from source to target and from target to source. Edges with a
negative directedness are considered to have the reverse direction such that they propagate ranks only from target to
source.
<h2>Note</h2>
If {@link IDataProvider} <code>initialPageRank</code> is <code>null</code>, the algorithm assumes that all nodes have an initial rank of
<code>1.0</code>.
<h2>Note</h2>
If {@link IDataProvider} <code>nodeWeight</code> is <code>null</code>, the algorithm assumes that all nodes have weight <code>1.0</code>.
<h2>Note</h2>
If {@link IDataProvider} <code>edgeWeight</code> is <code>null</code>, the algorithm assumes that all edges have weight <code>1.0</code>.
<h2>Note</h2>
If {@link IDataProvider} <code>edgeDirectedness</code> is <code>null</code>, the algorithm assumes that all edges have directedness
<code>1.0</code>, i.e. are directed from source to target.
@param graph the graph to calculate the ranks for
@param [pageRank=null] the {@link INodeMap} that will be filled during the execution with the rank value for each node
@param [initialPageRank=null] The {@link IDataProvider} that specifies the initial page rank for each node
@param [nodeWeight=null] the {@link IDataProvider} that specifies the weight for each node
@param [edgeWeight=null] the {@link IDataProvider} that specifies the weight for each edge
@param [edgeDirectedness=null] {@link IDataProvider} that stores the directedness of the edges
@param [dampingFactor=0.85] a value between <code>0</code> and <code>1</code> denoting the factor of a node's rank that shall be distributed to its successors (a good default
value is <code>0.85</code>)
@param [precision=0.001] the non-negative precision of the calculated page ranks (a good default value is <code>0.001</code>)
@returns the sum of all node ranks
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23pageRank(Graph%2CINodeMap%2CIDataProvider%2CIDataProvider%2CIDataProvider%2CIDataProvider%2Cnumber%2Cnumber) Online Documentation}*/
static pageRank(graph: Graph | null, pageRank?: INodeMap | null, initialPageRank?: IDataProvider | null, nodeWeight?: IDataProvider | null, edgeWeight?: IDataProvider | null, edgeDirectedness?: IDataProvider | null, dampingFactor?: number, precision?: number): number;
/**
Computes page rank values for all nodes based on their attached edges.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate the page rank for nodes in an {@link IGraph}
use {@link PageRank} instead.
The original algorithm was proposed by Sergey Brin and Lawrence Page as PageRank algorithm and refined later in several
papers:
S. Brin and L. Page. The anatomy of a large-scale hypertextual Web search
engine.
<br />
Computer Networks and ISDN Systems, 30(1-7):107-117
<br />
1998.
The algorithm starts by initializing the rank value for each node (see {@link IDataProvider} <code>initialPageRank</code>).
After that it uses multiple iterations to propagate the rank of a node to its successors. The base factor for each
successor is <code>1</code>. These base factors are multiplied by edge weights (see data provider <code>edgeWeight</code> and node weights (see
data provider <code>nodeWeight</code> if specified.
The final node factors are scaled afterwards to sum up to <code>1</code> so the overall rank stays the same after each iteration. The
old page rank of the node multiplied with these scaled factors are then distributed to the successors.
When a node can't distribute its rank, e.g. because it has no successors, its rank is distributed between all nodes
scaled by their weight (see data provider <code>nodeWeight</code>). Furthermore a <code>damping factor</code> is used so that a share of each
nodes' rank isn't propagated to the successors but also distributed between all nodes.
After each iteration, the relative change of each node's page rank ((newRank - oldRank)/oldRank) is calculated. If all
page rank changes are below the specified <code>precision</code> or if the maximal iterations are reached, the algorithm stops.
The <code>edgeDirectedness</code> {@link IDataProvider} defines the direction of the edges. This in consequence defines the
propagation direction of ranks. Edges with a positive directedness value propagate the rank from the source to the
target node - this is also the default if the given data provider is <code>null</code>. Edges which have a directedness of <code>0</code> are
considered to be undirected so that propagation happens from source to target and from target to source. Edges with a
negative directedness are considered to have the reverse direction such that they propagate ranks only from target to
source.
<h2>Note</h2>
If {@link IDataProvider} <code>initialPageRank</code> is <code>null</code>, the algorithm assumes that all nodes have an initial rank of
<code>1.0</code>.
<h2>Note</h2>
If {@link IDataProvider} <code>nodeWeight</code> is <code>null</code>, the algorithm assumes that all nodes have weight <code>1.0</code>.
<h2>Note</h2>
If {@link IDataProvider} <code>edgeWeight</code> is <code>null</code>, the algorithm assumes that all edges have weight <code>1.0</code>.
<h2>Note</h2>
If {@link IDataProvider} <code>edgeDirectedness</code> is <code>null</code>, the algorithm assumes that all edges have directedness
<code>1.0</code>, i.e. are directed from source to target.
@param options The parameters to pass.
@param options.graph the graph to calculate the ranks for
@param [options.pageRank=null] the {@link INodeMap} that will be filled during the execution with the rank value for each node
@param [options.initialPageRank=null] The {@link IDataProvider} that specifies the initial page rank for each node
@param [options.nodeWeight=null] the {@link IDataProvider} that specifies the weight for each node
@param [options.edgeWeight=null] the {@link IDataProvider} that specifies the weight for each edge
@param [options.edgeDirectedness=null] {@link IDataProvider} that stores the directedness of the edges
@param [options.dampingFactor=0.85] a value between <code>0</code> and <code>1</code> denoting the factor of a node's rank that shall be distributed to its successors (a good default
value is <code>0.85</code>)
@param [options.precision=0.001] the non-negative precision of the calculated page ranks (a good default value is <code>0.001</code>)
@returns the sum of all node ranks
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23pageRank(Graph%2CINodeMap%2CIDataProvider%2CIDataProvider%2CIDataProvider%2CIDataProvider%2Cnumber%2Cnumber) Online Documentation}*/
static pageRank(options: {
graph: Graph | null;
pageRank?: INodeMap | null;
initialPageRank?: IDataProvider | null;
nodeWeight?: IDataProvider | null;
edgeWeight?: IDataProvider | null;
edgeDirectedness?: IDataProvider | null;
dampingFactor?: number;
precision?: number;
}): number;
/**
Computes the weight centrality for the nodes of a graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate weight centrality for edges in an {@link IGraph}
use {@link WeightCentrality} instead.
<i>Weight centrality</i> measures the weight associated with incoming, outgoing, or all edges of a node.
<h2>Note</h2>
Weight centrality degenerates to degree centrality when the edges have uniform weight. In particular, when parameter <code>edgeWeights</code>
is <code>null</code>, then {@link CentralityAlgorithm#degreeCentrality degreeCentrality} is invoked instead.
@param graph the input graph
@param centrality the {@link INodeMap} that will be filled during the execution and returns a double value (centrality) for each
node
@param considerInEdges <code>true</code> if the incoming edges should be considered, <code>false</code> otherwise
@param considerOutEdges <code>true</code> if the outgoing edges should be considered, <code>false</code> otherwise
@param edgeWeights the {@link IDataProvider} that returns a positive double value (weight) or <code>null</code> if the edges are considered to
have uniform weight of <code>1.0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CentralityAlgorithm%23weightCentrality(Graph%2CINodeMap%2Cboolean%2Cboolean%2CIDataProvider) Online Documentation}*/
static weightCentrality(graph: Graph | null, centrality: INodeMap | null, considerInEdges: boolean, considerOutEdges: boolean, edgeWeights: IDataProvider | null): void;
static $class: Class;
private $_Centrality: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CentralityAlgorithm;
}
export interface CycleAlgorithm extends YObject {
}
/**
This class is responsible for finding cycles within a graph that have certain properties.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To determine cycles in an {@link IGraph}
instance use one of the following classes instead:
<ul>
<li>{@link Cycle}
<li>{@link CycleEdges}
<li>{@link FeedbackEdgeSet}
</ul>
<b>Definitions</b>
<ul>
<li>An edge path with vertices <code>v0, v1, v2, ... , vk</code> (with k > 0 and (vi-1,vi), 0 &lt; i &lt;= k is edge of the given graph) is
called <b>simple</b> if no vertex appears more than once.
<li>An edge path forms a <b>cycle</b> if <code>v0 = vk</code>.
<li>A cycle is called <b>simple</b> if no vertex appears more than once.
<li>A graph that contains no cycles is called <b>acyclic</b>.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleAlgorithm Online Documentation}*/
export class CycleAlgorithm {
/**
Returns an {@link EdgeList} that contains all the edges that are part of at least one directed or undirected
simple cycle.
<b>Note:</b> This method works with instances of {@link Graph}. To determine all cycle edges in an {@link IGraph} use
{@link CycleEdges} instead.
<h2>Note</h2>
Self-loops are always considered to be cycle edges.
@param graph the input graph
@param directed <code>true</code> if the graph should be considered directed, <code>false</code> otherwise
@returns an {@link EdgeList} that contains all the edges that are part of at least one directed or undirected simple
cycle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleAlgorithm%23findAllCycleEdges(Graph%2Cboolean) Online Documentation}*/
static findAllCycleEdges(graph: Graph | null, directed: boolean): EdgeList | null;
/**
Returns an {@link EdgeList} that contains the edges of a cycle found in the given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To find a cycle in an {@link IGraph} use {@link Cycle}
instead.
The edges are returned in the order they appear in the detected cycle.
If the returned cycle is empty, no cycle has been found in the given graph.
@param graph the given graph
@param directed <code>true</code> if the graph should be considered directed, <code>false</code> otherwise
@returns an {@link EdgeList} containing the edges of a cycle or an empty {@link EdgeList} if the graph is
acyclic
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleAlgorithm%23findCycle(Graph%2Cboolean) Online Documentation}*/
static findCycle(graph: Graph | null, directed: boolean): EdgeList | null;
/**
Marks the edges of a given graph whose removal or reversal would make the graph acyclic while trying to minimize the
cost associated with the marked edges.
<b>Note:</b> This method works with instances of {@link Graph}. To determine feedback edges in an {@link IGraph} use
{@link FeedbackEdgeSet} instead.
This minimization is performed heuristically, since it is a well known hard problem to come up with an optimal
solution.
The costs are assigned using a {@link IDataProvider} that holds a non-negative double value for each edge.
@param graph the given graph
@param cycleEdges the {@link IEdgeMap} that will be filled during the execution and returns whether an edge is a detected cycle
edge
@param [costDP=null] the {@link IDataProvider} that holds the non-negative {@link number} reversal cost for each edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleAlgorithm%23findCycleEdges(Graph%2CIEdgeMap%2CIDataProvider) Online Documentation}*/
static findCycleEdges(graph: Graph | null, cycleEdges: IEdgeMap | null, costDP?: IDataProvider | null): void;
/**
Marks the edges of a given graph whose removal or reversal would make the graph acyclic based on a depth first search.
<b>Note:</b> This method works with instances of {@link Graph}. To determine feedback edges in an {@link IGraph} use
{@link FeedbackEdgeSet} instead.
The number of marked cycle edges is expected to be slightly greater than when using
{@link CycleAlgorithm#findCycleEdges}. The advantage of this method is that the result set is more stable when edges
are added or removed over time.
@param graph the given graph
@param cycleEdges the {@link IEdgeMap} that will be filled during the execution and returns a boolean value indicating whether
or not an edge is a detected cycle edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleAlgorithm%23findCycleEdgesDFS(Graph%2CIEdgeMap) Online Documentation}*/
static findCycleEdgesDFS(graph: Graph | null, cycleEdges: IEdgeMap | null): void;
static $class: Class;
private $_Cycles: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CycleAlgorithm;
}
export interface Dart extends YObject {
}
/**
Represents a dart of a face of a {@link PlanarEmbedding}.
For each edge there exist two darts, one that represents the edge in its original direction (i.e., from source to
target) and one that represents its reverse. Each dart is associated with a {@link Dart#face face}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dart Online Documentation}*/
export class Dart {
/**
Gets the edge associated with this dart.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dart%23associatedEdge Online Documentation}*/
readonly associatedEdge: Edge | null;
/**
Gets whether this dart represents the original edge or its reverse.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dart%23reversed Online Documentation}*/
readonly reversed: boolean;
/**
Gets the opposite dart.
The opposite dart is associated with the same edge as this dart but with a different {@link Dart#reversed direction}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dart%23oppositeDart Online Documentation}*/
readonly oppositeDart: Dart | null;
/**
Gets the face list associated with this dart.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dart%23face Online Documentation}*/
readonly face: IList<Dart> | null;
static $class: Class;
private $_Dart: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Dart;
}
export interface DfsAlgorithm extends YObject {
}
/**
Framework class for implementing depth first search (DFS) based algorithms.
A depth first search starts from a specified node and traverses the neighbors within a branch as deeply as possible
(i.e., without visiting any vertex twice) before backtracking.
A depth first search traversal induces a DFS forest that may consist of more than one DFS tree. Based on these trees,
the edges are divided into four categories (see example):
<ul>
<li><b>Tree Edges:</b> The edges of the DFS forest.
<li><b>Back edges:</b> Non-tree edges that connect nodes with their ancestors in the DFS tree. Self-loops are also considered as back edges.
<li><b>Forward edges:</b> Edges that connect nodes of the DFS tree to one of their descendants.
<li><b>Cross edges:</b> All other edges.
</ul>
During a DFS run each node of the graph can be in one of the following states:
<ul>
<li><b>Undiscovered:</b> A node that has not been visited yet.
<li><b>Discovered but not finished:</b> A node that has been already visited, but has not been completed yet, i.e. it is still part of an active path of the
DFS tree.
<li><b>Finished:</b> A node that has been completed, i.e. it has been visited before and is not part of an active path of the DFS tree
anymore.
</ul>
Graph algorithms, which are based on a depth first search, can extend this class and override appropriate callback
methods provided by this class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm Online Documentation}*/
export class DfsAlgorithm {
/**
Creates a new {@link DfsAlgorithm} instance with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm Online Documentation}*/
constructor();
/**
Cancels the depth first search.
It may be overridden to support custom implementations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23cancel() Online Documentation}*/
cancel(): void;
/**
Callback method that will be invoked whenever DFS continues its search at a new root node.
By default, this method does nothing. It may be overridden to support custom implementations.
@param v the new root node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23lookFurther(YNode) Online Documentation}*/
lookFurther(v: YNode | null): void;
/**
Callback method that will be invoked after the DFS has returned from the given node.
By default, this method does nothing. It may be overridden to support custom implementations.
@param edge the given edge
@param node the node that has been reached via the given edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23postTraverse(Edge%2CYNode) Online Documentation}*/
postTraverse(edge: Edge | null, node: YNode | null): void;
/**
Callback method that will be invoked whenever a node visit has been completed.
By default, this method does nothing. It may be overridden to support custom implementations.
@param node the given node
@param dfsNumber the DFS number of the given node
@param compNumber the completion number of the given node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23postVisit(YNode%2Cnumber%2Cnumber) Online Documentation}*/
postVisit(node: YNode | null, dfsNumber: number, compNumber: number): void;
/**
Callback method that will be invoked if the given edge will be considered the first (and only) time during the DFS.
By default, this method does nothing. It may be overridden to support custom implementations.
@param edge the given edge
@param node the node to be visited next only if <code>treeEdge == true</code>
@param treeEdge <code>true</code> if the <code>node</code> will be visited, <code>false</code> otherwise
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23preTraverse(Edge%2CYNode%2Cboolean) Online Documentation}*/
preTraverse(edge: Edge | null, node: YNode | null, treeEdge: boolean): void;
/**
Callback method that will be invoked whenever a formerly unvisited node gets visited for the first time.
By default, this method does nothing. It may be overridden to support custom implementations.
@param node the given node
@param dfsNumber the DFS number of the given node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23preVisit(YNode%2Cnumber) Online Documentation}*/
preVisit(node: YNode | null, dfsNumber: number): void;
/**
Starts a depth first search from a given {@link YNode} of the input graph.
<b>Warning:</b> If the given <code>start</code> node is <code>null</code>, this method returns silently.
@param graph the input graph
@param start the given start node
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23start(Graph%2CYNode) Online Documentation}*/
start(graph: Graph | null, start: YNode | null): void;
/**
Starts a depth first search on the given graph.
The first node of the graph will be visited first.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23start(Graph) Online Documentation}*/
start(graph: Graph | null): void;
/**
A {@link INodeMap} that holds for each {@link YNode} an {@link Object} indicating the current state of the given
node as it is visited by this algorithm.
Each node will be assigned one of the {@link DfsAlgorithm#WHITE WHITE}, {@link DfsAlgorithm#GRAY GRAY} or {@link DfsAlgorithm#BLACK BLACK} objects that describe the state of the nodes.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23stateMap Online Documentation}*/
stateMap: INodeMap | null;
/**
A constant indicating a node has not been visited yet.
@protected
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23WHITE Online Documentation}*/
static WHITE: Object | null;
/**
A constant indicating a node has already been visited, but has not been completed yet.
Such a node is still part of an active path of the DFS tree.
@protected
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23GRAY Online Documentation}*/
static GRAY: Object | null;
/**
A constant indicating a node has been completed.
Such a node has been visited before and is no longer part of an active path of the DFS tree.
@protected
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23BLACK Online Documentation}*/
static BLACK: Object | null;
/**
Sets whether or not to interpret the edges of the graph as directed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23directedMode Online Documentation}*/
directedMode: boolean;
/**
Sets whether or not to continue the depth first search after all nodes reachable from the first node have been visited.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DfsAlgorithm%23lookFurtherMode Online Documentation}*/
lookFurtherMode: boolean;
static $class: Class;
private $_Dfs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DfsAlgorithm;
}
export interface DpKeyBase<TValue> extends YObject {
}
/**
This is the base class of all look-up keys that are used to register {@link IDataProvider} with a
{@link Graph}.
Type parameter <code>TValue</code>: The type of the values that are returned by the {@link IDataProvider} registered with this class.
@see Graph#addDataProvider
@see Graph#getDataProvider
@see Graph#removeDataProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyBase Online Documentation}*/
export class DpKeyBase<TValue> {
/**
Initializes a new instance of the {@link DpKeyBase} class.
@param valueType The type of the values that are returned by the {@link IDataProvider} registered with this class.
@param declaringType Type that declares this key.
@param name The name of this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyBase Online Documentation}*/
constructor(valueType: Class, declaringType: Class | null, name: string | null);
/**
Determines whether the specified {@link Object} is equal to this instance.
@param obj The {@link Object} to compare with this instance.
@returns <code>true</code> if the specified {@link Object} is equal to this instance; otherwise, <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyBase%23equals(Object) Online Documentation}*/
equals(obj: any | null): boolean;
/**
Returns whether this key is equal to the specified <code>other</code>.
@param other The other key to compare with this instance.
@returns <code>true</code> if the specified key is equal to this instance; otherwise, <code>false</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyBase%23equalsCore(DpKeyBase%3CTValue%3E) Online Documentation}*/
equalsCore(other: DpKeyBase<TValue> | null): boolean;
/**
Returns a hash code for this instance.
@returns A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyBase%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Gets the type that declares this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyBase%23declaringType Online Documentation}*/
declaringType: Class | null;
/**
Gets the name of this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyBase%23name Online Documentation}*/
name: string | null;
static $class: Class;
private $_DpKeyBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TValue>(o: any): o is DpKeyBase<TValue>;
}
export interface EdgeDpKey<TValue> extends DpKeyBase<TValue> {
}
/**
This class is used as look-up key for registering {@link IDataProvider} for {@link Edge}s with a
{@link Graph}.
Type parameter <code>TValue</code>: The type of the values that are returned by the {@link IDataProvider} registered with this class.
@see Graph#addDataProvider
@see Graph#getDataProvider
@see Graph#removeDataProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDpKey Online Documentation}*/
export class EdgeDpKey<TValue> {
/**
Initializes a new instance of the {@link EdgeDpKey} class.
@param valueType The type of the values that are returned by the {@link IDataProvider} registered with this class.
@param declaringType Type that declares this key.
@param name The name of this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDpKey Online Documentation}*/
constructor(valueType: Class, declaringType: Class | null, name: string | null);
static $class: Class;
private $_EdgeDpKey: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TValue>(o: any): o is EdgeDpKey<TValue>;
}
export interface GraphDpKey<TValue> extends DpKeyBase<TValue> {
}
/**
This class is used as look-up key for registering {@link IDataProvider} for the {@link Graph} itself
with a graph.
Type parameter <code>TValue</code>: The type of the values that are returned by the {@link IDataProvider} registered with this class.
@see Graph#addDataProvider
@see Graph#getDataProvider
@see Graph#removeDataProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphDpKey Online Documentation}*/
export class GraphDpKey<TValue> {
/**
Initializes a new instance of the {@link GraphDpKey} class.
@param valueType The type of the values that are returned by the {@link IDataProvider} registered with this class.
@param declaringType Type that declares this key.
@param name The name of this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphDpKey Online Documentation}*/
constructor(valueType: Class, declaringType: Class | null, name: string | null);
static $class: Class;
private $_GraphDpKey: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TValue>(o: any): o is GraphDpKey<TValue>;
}
export interface GraphObjectDpKey<TValue> extends DpKeyBase<TValue> {
}
/**
This class is used as look-up key for registering {@link IDataProvider} for {@link GraphObject}s
with a {@link Graph}.
Type parameter <code>TValue</code>: The type of the values that are returned by the {@link IDataProvider} registered with this class.
@see Graph#addDataProvider
@see Graph#getDataProvider
@see Graph#removeDataProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphObjectDpKey Online Documentation}*/
export class GraphObjectDpKey<TValue> {
/**
Initializes a new instance of the {@link GraphObjectDpKey} class.
@param valueType The type of the values that are returned by the {@link IDataProvider} registered with this class.
@param declaringType Type that declares this key.
@param name The name of this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphObjectDpKey Online Documentation}*/
constructor(valueType: Class, declaringType: Class | null, name: string | null);
static $class: Class;
private $_GraphObjectDpKey: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TValue>(o: any): o is GraphObjectDpKey<TValue>;
}
export interface IEdgeLabelLayoutDpKey<TValue> extends DpKeyBase<TValue> {
}
/**
This class is used as look-up key for registering {@link IDataProvider} for IEdgeLabelLayouts with a
{@link Graph}.
Type parameter <code>TValue</code>: The type of the values that are returned by the {@link IDataProvider} registered with this class.
@see Graph#addDataProvider
@see Graph#getDataProvider
@see Graph#removeDataProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayoutDpKey Online Documentation}*/
export class IEdgeLabelLayoutDpKey<TValue> {
/**
Initializes a new instance of the {@link IEdgeLabelLayoutDpKey} class.
@param valueType The type of the values that are returned by the {@link IDataProvider} registered with this class.
@param declaringType Type that declares this key.
@param name The name of this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayoutDpKey Online Documentation}*/
constructor(valueType: Class, declaringType: Class | null, name: string | null);
static $class: Class;
private $_IEdgeLabelLayoutDpKey: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TValue>(o: any): o is IEdgeLabelLayoutDpKey<TValue>;
}
export interface INodeLabelLayoutDpKey<TValue> extends DpKeyBase<TValue> {
}
/**
This class is used as look-up key for registering {@link IDataProvider} for INodeLabelLayouts with a
{@link Graph}.
Type parameter <code>TValue</code>: The type of the values that are returned by the {@link IDataProvider} registered with this class.
@see Graph#addDataProvider
@see Graph#getDataProvider
@see Graph#removeDataProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelLayoutDpKey Online Documentation}*/
export class INodeLabelLayoutDpKey<TValue> {
/**
Initializes a new instance of the {@link INodeLabelLayoutDpKey} class.
@param valueType The type of the values that are returned by the {@link IDataProvider} registered with this class.
@param declaringType Type that declares this key.
@param name The name of this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelLayoutDpKey Online Documentation}*/
constructor(valueType: Class, declaringType: Class | null, name: string | null);
static $class: Class;
private $_INodeLabelLayoutDpKey: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TValue>(o: any): o is INodeLabelLayoutDpKey<TValue>;
}
export interface ILabelLayoutDpKey<TValue> extends DpKeyBase<TValue> {
}
/**
This class is used as look-up key for registering {@link IDataProvider} for ILabelLayouts with a
{@link Graph}.
Type parameter <code>TValue</code>: The type of the values that are returned by the {@link IDataProvider} registered with this class.
@see Graph#addDataProvider
@see Graph#getDataProvider
@see Graph#removeDataProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutDpKey Online Documentation}*/
export class ILabelLayoutDpKey<TValue> {
/**
Initializes a new instance of the {@link ILabelLayoutDpKey} class.
@param valueType The type of the values that are returned by the {@link IDataProvider} registered with this class.
@param declaringType Type that declares this key.
@param name The name of this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutDpKey Online Documentation}*/
constructor(valueType: Class, declaringType: Class | null, name: string | null);
static $class: Class;
private $_ILabelLayoutDpKey: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TValue>(o: any): o is ILabelLayoutDpKey<TValue>;
}
export interface NodeDpKey<TValue> extends DpKeyBase<TValue> {
}
/**
This class is used as look-up key for registering {@link IDataProvider} for {@link YNode}s with a
{@link Graph}.
Type parameter <code>TValue</code>: The type of the values that are returned by the {@link IDataProvider} registered with this class.
@see Graph#addDataProvider
@see Graph#getDataProvider
@see Graph#removeDataProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDpKey Online Documentation}*/
export class NodeDpKey<TValue> {
/**
Initializes a new instance of the {@link NodeDpKey} class.
@param valueType The type of the values that are returned by the {@link IDataProvider} registered with this class.
@param declaringType Type that declares this key.
@param name The name of this key.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDpKey Online Documentation}*/
constructor(valueType: Class, declaringType: Class | null, name: string | null);
static $class: Class;
private $_NodeDpKey: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TValue>(o: any): o is NodeDpKey<TValue>;
}
export interface Edge extends GraphObject {
}
/**
Represents an edge, i.e., a directed connection between two nodes (represented by instances of class
{@link YNode}) in the directed graph data type {@link Graph}.
The <i>directed</i> stems from the fact that an edge has a distinct source node and a distinct target node. Using pair notation, an
edge would be written as (<i>&lt;source node></i>, <i>&lt;target node></i>).
Most notably, an edge provides access to its source node ({@link Edge#source}) and its target node ({@link Edge#target}). Note that an edge can have the same
node as its source and target. Such an edge is then called "self-loop" and method {@link Edge#selfLoop} yields <code>true</code>.
<b>Important:</b> Class Graph is the single authority for any structural changes to the graph data type. Specifically, this means that
there is no way to create or delete a node or an edge without using an actual Graph instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge Online Documentation}*/
export class Edge {
/**
Creates a new edge that belongs to the given graph.
The new edge <code>e</code> has source node <code>v</code> and target node <code>w</code>.
Edge <code>e</code> is inserted in such a way that an iteration over the edges at node <code>v</code> returns <code>e</code>
<ul>
<li>after <code>e1</code>, if <code>d1 == AFTER</code>
<li>before <code>e1</code>, if <code>d1 == BEFORE</code>,
</ul>
and an iteration over the edges at <code>w</code> returns <code>e</code>
<ul>
<li>after <code>e2</code>, if <code>d2 == AFTER</code>
<li>before <code>e2</code>, if <code>d2 == BEFORE</code>.
</ul>
@param v The source node of the edge.
@param e1 An edge with source node <code>v</code>.
@param w The target node of the edge.
@param e2 An edge with target node <code>w</code>.
@param d1 One of the object insertion specifiers {@link GraphElementInsertion#BEFORE} or
{@link GraphElementInsertion#AFTER}.
@param d2 One of the object insertion specifiers {@link GraphElementInsertion#BEFORE} or
{@link GraphElementInsertion#AFTER}.
@param g
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge Online Documentation}*/
constructor(g: Graph, v: YNode, e1: Edge | null, w: YNode, e2: Edge | null, d1: GraphElementInsertion | GraphElementInsertionStringValues, d2: GraphElementInsertion | GraphElementInsertionStringValues);
/**
Creates a copy of this edge that will be inserted into the given graph connecting the given source and target nodes.
@param g The graph the created edge will belong to.
@param v The source node of the created edge.
@param w The target node of the created edge.
@returns The newly created Edge object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23createCopy(Graph%2CYNode%2CYNode) Online Documentation}*/
createCopy(g: Graph, v: YNode, w: YNode): Edge;
/**
Callback method that is invoked from a graph just before this edge will be reinserted into that graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23onReinsert() Online Documentation}*/
onReinsert(): void;
/**
Returns the node at the opposite edge end with respect to the given node.
Note that self-loops have the same node at both edge ends.
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23opposite(YNode) Online Documentation}*/
opposite(v: YNode | null): YNode;
/**
Returns a String representation of this edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23toString() Online Documentation}*/
toString(): string;
/**
Gets the graph this edge belongs to.
If the edge does not belong to a graph, because it was removed or hidden from it, this method returns <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23graph Online Documentation}*/
readonly graph: Graph | null;
/**
Gets the index of this edge within its graph G.
Edge indices represent the ordering of standard edge iteration on G. The value of an index is <code> >= 0</code> and
<code> &lt; G.edgeCount()</code>.
Note that indices are subject to change whenever the sequence of edges in a graph is modified by either removing,
hiding, reinserting, or unhiding an edge, or by explicitly changing its position in the sequence.
@see Graph#removeEdge
@see Graph#hide
@see Graph#reInsertEdge
@see Graph#unhide
@see Graph#moveToFirst
@see Graph#moveToLast
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23index Online Documentation}*/
readonly index: number;
/**
Gets the source node connected to this edge.
@see Edge#target
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23source Online Documentation}*/
readonly source: YNode;
/**
Gets the target node connected to this edge.
@see Edge#source
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23target Online Documentation}*/
readonly target: YNode;
/**
Gets <code>true</code> if and only if this edge is a self-loop.
An edge is called a self-loop, if it is adjacent to only one node, i.e., source node and target node are the same.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23selfLoop Online Documentation}*/
readonly selfLoop: boolean;
/**
Gets the successor of this edge in the list of outgoing edges at its source node.
If this edge is the last outgoing edge at its source node, then <code>null</code> is returned.
@see Edge#prevOutEdge
@see Edge#nextInEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23nextOutEdge Online Documentation}*/
readonly nextOutEdge: Edge | null;
/**
Gets the successor of this edge in the list of incoming edges at its target node.
If this edge is the last incoming edge at its target node, then <code>null</code> is returned.
@see Edge#prevInEdge
@see Edge#nextOutEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23nextInEdge Online Documentation}*/
readonly nextInEdge: Edge | null;
/**
Gets the predecessor of this edge in the list of outgoing edges at its source node.
If this edge is the first outgoing edge at its source node, then <code>null</code> is returned.
@see Edge#nextOutEdge
@see Edge#prevInEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23prevOutEdge Online Documentation}*/
readonly prevOutEdge: Edge | null;
/**
Gets the predecessor of this edge in the list of incoming edges at its target node.
If this edge is the first incoming edge at its target node, then <code>null</code> is returned.
@see Edge#nextInEdge
@see Edge#prevOutEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/Edge%23prevInEdge Online Documentation}*/
readonly prevInEdge: Edge | null;
static $class: Class;
private $_Edge: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Edge;
}
export interface EdgeList extends YList {
}
/**
Specialized list implementation for instances of type {@link Edge}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList Online Documentation}*/
export class EdgeList {
/**
Creates a list that is initialized with a given other {@link EdgeList}.
@param edgeList
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23EdgeList(EdgeList) Online Documentation}*/
constructor(edgeList: EdgeList);
/**
Creates a list that is initialized with a single edge provided.
@param e
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23EdgeList(Edge) Online Documentation}*/
constructor(e: Edge);
/**
Creates a list that is initialized with those edges from the given EdgeCursor object for which the given data provider
returns <code>true</code> upon calling its {@link IDataProvider#getBoolean getBool} method.
@param ec An edge cursor providing edges that should be added to this list.
@param predicate A data provider that acts as a inclusion predicate for each edge accessible by the given edge cursor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23EdgeList(IEdgeCursor%2CIDataProvider) Online Documentation}*/
constructor(ec: IEdgeCursor, predicate: IDataProvider);
/**
Creates a list that is initialized with the edges provided by the given array of edges.
@param a
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23EdgeList(Array%3CEdge%3E) Online Documentation}*/
constructor(a: Edge[]);
/**
Creates a list that is initialized with the edges provided by the given {@link IEdgeCursor} object.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23EdgeList(IEdgeCursor) Online Documentation}*/
constructor(c: IEdgeCursor);
/**
Creates an empty edge list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23EdgeList() Online Documentation}*/
constructor();
/**
Returns an edge cursor for this edge list.
@returns An edge cursor granting access to the edges within this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23edges() Online Documentation}*/
edges(): IEdgeCursor;
/**
Returns the first edge in this list, or <code>null</code> when the list is empty.
@returns The first edge in the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23firstEdge() Online Documentation}*/
firstEdge(): Edge | null;
/**
Returns an enumerator for this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23getEnumerator() Online Documentation}*/
getEnumerator(): IEnumerator<Edge>;
/**
Returns the last edge in this list, or <code>null</code> when the list is empty.
@returns The last edge in the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23lastEdge() Online Documentation}*/
lastEdge(): Edge | null;
/**
Removes the first edge from this list and returns it.
@returns The first edge from the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23popEdge() Online Documentation}*/
popEdge(): Edge | null;
/**
Returns an edge array containing all elements of this list in the canonical order.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeList%23toEdgeArray() Online Documentation}*/
toEdgeArray(): Edge[];
static $class: Class;
private $_EdgeList: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeList;
}
/**
@see BfsAlgorithm#getLayers
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TraversalDirection Online Documentation}*/
export enum TraversalDirection {
/**
An edge direction specifier for incoming edges.
@see BfsAlgorithm#getLayers
@see {@link https://docs.yworks.com/yfileshtml/#/api/TraversalDirection%23PREDECESSOR Online Documentation}*/
PREDECESSOR,
/**
An edge direction specifier for outgoing edges.
@see BfsAlgorithm#getLayers
@see {@link https://docs.yworks.com/yfileshtml/#/api/TraversalDirection%23SUCCESSOR Online Documentation}*/
SUCCESSOR,
/**
An edge direction specifier for both incoming and outgoing edges.
@see BfsAlgorithm#getLayers
@see {@link https://docs.yworks.com/yfileshtml/#/api/TraversalDirection%23BOTH Online Documentation}*/
BOTH
}
/** Automatic conversion to {@link TraversalDirection}. */
type TraversalDirectionStringValues = "predecessor" | "successor" | "both";
/**
Object insertion specifier.
@see Edge
@see Graph#changeEdge
@see Graph#changeEdge
@see Graph#createEdge
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphElementInsertion Online Documentation}*/
export enum GraphElementInsertion {
/**
Object insertion specifier.
An object gets inserted before another one.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphElementInsertion%23BEFORE Online Documentation}*/
BEFORE,
/**
Object insertion specifier.
An object gets inserted after another one.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphElementInsertion%23AFTER Online Documentation}*/
AFTER
}
/** Automatic conversion to {@link GraphElementInsertion}. */
type GraphElementInsertionStringValues = "before" | "after";
/**
Provides constants for different metrics used in {@link KMeansClustering} and
{@link GroupAlgorithm#kMeansClustering}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DistanceMetric Online Documentation}*/
export enum DistanceMetric {
/**
Specifies the euclidean distance metric or L² metric.
This is calculated by <code>sqrt((p1.x - p2.x)^2 + (p1.y - p2.y)^2)</code>.
<h2>Note</h2>
When this value is set to the {@link KMeansClustering#metric} property, it has the same effect as the {@link DistanceMetric#EUCLIDEAN_SQUARED}
value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DistanceMetric%23EUCLIDEAN Online Documentation}*/
EUCLIDEAN,
/**
Specifies the (squared) euclidean distance metric.
This is calculated by <code>(p1.x - p2.x)^2 + (p1.y - p2.y)^2</code>.
<h2>Note</h2>
When this value is set to the {@link KMeansClustering#metric} property, it has the same effect as the {@link DistanceMetric#EUCLIDEAN}
value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DistanceMetric%23EUCLIDEAN_SQUARED Online Documentation}*/
EUCLIDEAN_SQUARED,
/**
Specifies the Manhattan distance metric or L¹ metric.
This is calculated by <code>abs(p1.x - p2.x) + abs(p1.y - p2.y)</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/DistanceMetric%23MANHATTAN Online Documentation}*/
MANHATTAN,
/**
Specifies the Chebychev distance metric or L<sub>∞</sub> metric.
This is calculated by <code>max(p1.x - p2.x) + abs(p1.y - p2.y)</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/DistanceMetric%23CHEBYCHEV Online Documentation}*/
CHEBYCHEV
}
/** Automatic conversion to {@link DistanceMetric}. */
type DistanceMetricStringValues = "euclidean" | "euclidean-squared" | "manhattan" | "chebychev";
/**
A specifier for average-linkage (or mean-linkage) clustering.
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Linkage Online Documentation}*/
export enum Linkage {
/**
A specifier for single-linkage (or minimum-linkage) clustering.
The distance between two clusters is determined by the <i>minimum</i> distance. Clusters thus are as close as their <i>closest</i> pair of nodes.
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see {@link https://docs.yworks.com/yfileshtml/#/api/Linkage%23SINGLE Online Documentation}*/
SINGLE,
/**
A specifier for complete-linkage (or maximum-linkage) clustering.
The distance between two clusters is determined by the <i>maximum</i> distance. Clusters thus are as close as the pair of nodes that
are <i>furthest away</i> from each other.
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see {@link https://docs.yworks.com/yfileshtml/#/api/Linkage%23COMPLETE Online Documentation}*/
COMPLETE,
/**
A specifier for average-linkage (or mean-linkage) clustering.
The distance between two clusters is the <i>average</i> distance between each pair of nodes.
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see {@link https://docs.yworks.com/yfileshtml/#/api/Linkage%23AVERAGE Online Documentation}*/
AVERAGE
}
/** Automatic conversion to {@link Linkage}. */
type LinkageStringValues = "single" | "complete" | "average";
/**
@see NodeAggregationAlgorithm#nodeTypeHandlingPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationNodeTypeHandlingPolicy Online Documentation}*/
export enum NodeAggregationNodeTypeHandlingPolicy {
/**
A node type handling policy that specifies that the types are ignored.
@see NodeAggregationAlgorithm#nodeTypeHandlingPolicy
@see NodeAggregationAlgorithm#NODE_TYPE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationNodeTypeHandlingPolicy%23IGNORE Online Documentation}*/
IGNORE,
/**
A node type handling policy that specifies that nodes of different types are always separated right at the root.
Separating at the root means that the top-level cluster of the aggregation is the only one that may (indirectly) contain
nodes of different types. The only exception to this rules is if there are
{@link GroupingKeys#GROUP_DP_KEY group nodes}, because a group is a kind of user-specified cluster and, thus, may contain nodes with different types.
@see NodeAggregationAlgorithm#nodeTypeHandlingPolicy
@see NodeAggregationAlgorithm#NODE_TYPE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationNodeTypeHandlingPolicy%23SEPARATE_AT_ROOT Online Documentation}*/
SEPARATE_AT_ROOT,
/**
A node type handling policy that specifies that nodes of different types are always separated at the leaves.
More precisely, all direct children of a cluster of the aggregation are associated with the same type. Note that, for
this policy, the algorithm may assign nodes without type to any cluster (even though the cluster already has some nodes
of a specific type).
@see NodeAggregationAlgorithm#nodeTypeHandlingPolicy
@see NodeAggregationAlgorithm#NODE_TYPE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationNodeTypeHandlingPolicy%23SEPARATE_AT_LEAVES Online Documentation}*/
SEPARATE_AT_LEAVES
}
/** Automatic conversion to {@link NodeAggregationNodeTypeHandlingPolicy}. */
type NodeAggregationNodeTypeHandlingPolicyStringValues = "ignore" | "separate-at-root" | "separate-at-leaves";
/**
@see NodeAggregationAlgorithm#aggregationPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationPolicy Online Documentation}*/
export enum NodeAggregationPolicy {
/**
An aggregation policy that specifies that nodes are aggregated according to their structure, that is, the connectivity.
@see NodeAggregationAlgorithm#aggregationPolicy
@see NodeAggregationAlgorithm#NODE_CENTER_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationPolicy%23STRUCTURAL Online Documentation}*/
STRUCTURAL,
/**
An aggregation policy that specifies that nodes are aggregated according to their geometric position.
More precisely, nodes that are placed close to each other are more likely to be contained in the same cluster. The
coordinates of the nodes have to be specified by the user by means of a {@link IDataProvider} registered to
the graph with key {@link NodeAggregationAlgorithm#NODE_CENTER_DP_KEY}.
<h2>Note</h2>
Using this policy, the edges are completely ignored during the aggregation.
@see NodeAggregationAlgorithm#aggregationPolicy
@see NodeAggregationAlgorithm#NODE_CENTER_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationPolicy%23GEOMETRIC Online Documentation}*/
GEOMETRIC
}
/** Automatic conversion to {@link NodeAggregationPolicy}. */
type NodeAggregationPolicyStringValues = "structural" | "geometric";
export interface AffineLine extends YObject {
}
/**
This class represents a line in the 2D-dimensional affine space.
The line is defined by the equation ax + by + c = 0
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine Online Documentation}*/
export class AffineLine {
/**
Creates an affine line which is defined by two points.
@param p1
@param p2
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine%23AffineLine(YPoint%2CYPoint) Online Documentation}*/
constructor(p1: YPoint, p2: YPoint);
/**
Creates an affine line which is defined by a point and a vector.
@param p1
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine%23AffineLine(YPoint%2CYVector) Online Documentation}*/
constructor(p1: YPoint, v: YVector);
/**
Returns the crossing of two lines.
If the lines are parallel, <code>null</code> is returned.
@param l1
@param l2
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine%23getCrossing(AffineLine%2CAffineLine) Online Documentation}*/
static getCrossing(l1: AffineLine | null, l2: AffineLine | null): YPoint | null;
/**
Projects an point on the line in direction of the X-axis.
@param p
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine%23getXProjection(YPoint) Online Documentation}*/
getXProjection(p: YPoint | null): YPoint | null;
/**
Projects an point on the line in direction of the Y-axis.
@param p
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine%23getYProjection(YPoint) Online Documentation}*/
getYProjection(p: YPoint | null): YPoint | null;
/**
Returns the equation of the line as String
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine%23toString() Online Documentation}*/
toString(): string;
/**
Gets a from ax+by+c = 0
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine%23a Online Documentation}*/
readonly a: number;
/**
Gets b from ax+by+c = 0
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine%23b Online Documentation}*/
readonly b: number;
/**
Gets c from ax+by+c = 0
@see {@link https://docs.yworks.com/yfileshtml/#/api/AffineLine%23c Online Documentation}*/
readonly c: number;
static $class: Class;
private $_AffineLine: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AffineLine;
}
export interface BorderLineSegment extends YObject {
}
/**
The handle of a segment of a borderline.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSegment Online Documentation}*/
export class BorderLineSegment {
/**
Returns the segment's value at the given position.
Note: In case the position lies outside the segments range, the calculated value might be invalid. As the segment is not
aware of any offsets the position also must not include any offsets.
@param position the position the value is retrieved for.
@returns the segment's value at the given position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSegment%23getValueAt(number) Online Documentation}*/
getValueAt(position: number): number;
/**
Returns the next segment or <code>null</code> if there is no such segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSegment%23next() Online Documentation}*/
next(): BorderLineSegment | null;
/**
Returns the previous segment or <code>null</code> if there is no such segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSegment%23prev() Online Documentation}*/
prev(): BorderLineSegment | null;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSegment%23toString() Online Documentation}*/
toString(): string;
/**
Gets the end of this segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSegment%23end Online Documentation}*/
readonly end: number;
static $class: Class;
private $_BorderLineSegment: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BorderLineSegment;
}
export interface BorderLine extends YObject {
}
/**
This class can be used to easily model an orthogonal border line or sky-line.
It provides methods for measuring the distance between different {@link BorderLine} instances, merging
multiple instances, modifying and efficiently moving them around.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine Online Documentation}*/
export class BorderLine {
/**
Creates a new {@link BorderLine} from a single segment.
@param min the beginning of this borderline
@param max the ending of this borderline
@param valueAtMin the value of the segment at the beginning of this borderline
@param valueAtMax the value of the segment at the ending of this borderline
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23BorderLine(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(min: number, max: number, valueAtMin: number, valueAtMax: number);
/**
Creates a new {@link BorderLine} from a single segment.
@param min the beginning of this borderline
@param max the ending of this borderline
@param value the value of the segment
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23BorderLine(number%2Cnumber%2Cnumber) Online Documentation}*/
constructor(min: number, max: number, value: number);
/**
Creates a new {@link BorderLine} with the given value from -{@link Number#MAX_VALUE} to +{@link Number#MAX_VALUE}.
@param value the value of the segment
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23BorderLine(number) Online Documentation}*/
constructor(value: number);
/**
Adds the given offset to the segments' positions.
This method has complexity O(1).
@param delta the delta to add to the positions
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23addOffset(number) Online Documentation}*/
addOffset(delta: number): void;
/**
Adds the given offset to the current values of the whole borderline.
This method has complexity O(1).
@param delta the delta to add to the values
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23addValueOffset(number) Online Documentation}*/
addValueOffset(delta: number): void;
/**
Convenience method that copies the actual data from the given argument to this instance.
@param other the argument to retrieve the values from
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23adoptValues(BorderLine) Online Documentation}*/
adoptValues(other: BorderLine | null): void;
/**
Creates a copy of this borderline.
Optionally negates the values or offsets.
@param negateValues whether the values are negated
@param negateOffsets whether the offsets are negated
@returns the copy of the borderline
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23createCopy(boolean%2Cboolean) Online Documentation}*/
createCopy(negateValues: boolean, negateOffsets: boolean): BorderLine | null;
/**
Merges this borderline with the given borderline using the "maximum" policy.
That means the resulting borderline will have greater value of both borderline on each position. If you imagine each
borderline as a the upper border of a plane, the resulting borderline will be the upper border of the merged planes.
@param other the other borderline
@returns a new borderline that is the result of the merge
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23createMax(BorderLine) Online Documentation}*/
createMax(other: BorderLine | null): BorderLine | null;
/**
Merges this borderline with the given borderline using the "minimum" policy.
That means the resulting borderline will have smaller value of both borderline on each position. If you imagine each
borderline as a the lower border of a plane, the resulting borderline will be the lower border of the merged planes.
@param other the other borderline
@returns a new borderline that is the result of the merge
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23createMin(BorderLine) Online Documentation}*/
createMin(other: BorderLine | null): BorderLine | null;
/**
Returns the first segment or <code>null</code> if there is no such segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23firstSegment() Online Documentation}*/
firstSegment(): BorderLineSegment | null;
/**
Calculates the minimal distance between this borderline and the other one.
The other one is treated as if the values were all greater.
@param greater
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getDistanceTo(BorderLine) Online Documentation}*/
getDistanceTo(greater: BorderLine | null): number;
/**
Returns the maximum position of the given segment.
@param s the segment
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getMax(BorderLineSegment) Online Documentation}*/
getMax(s: BorderLineSegment | null): number;
/**
Calculates the maximum value in the interval from->to.
<h2>Note</h2>
If the <code>from</code> value is larger or equal to the <code>to</code> value, the algorithm always returns -{@link Number#MAX_VALUE}.
@param from
@param to
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getMaxValue(number%2Cnumber) Online Documentation}*/
getMaxValue(from: number, to: number): number;
/**
Returns the minimum position of the given segment.
@param s the segment
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getMin(BorderLineSegment) Online Documentation}*/
getMin(s: BorderLineSegment | null): number;
/**
Calculates the minimum value in the interval from->to.
<h2>Note</h2>
If the <code>from</code> value is larger or equal to the <code>to</code> value, the algorithm always returns {@link Number#MAX_VALUE}.
@param from
@param to
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getMinValue(number%2Cnumber) Online Documentation}*/
getMinValue(from: number, to: number): number;
/**
Returns the segment at the given position.
@param pos the position
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getSegmentAt(number) Online Documentation}*/
getSegmentAt(pos: number): BorderLineSegment | null;
/**
Returns the slope of the given segment.
@param s the segment
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getSlope(BorderLineSegment) Online Documentation}*/
getSlope(s: BorderLineSegment | null): number;
/**
Returns the value of the minimum of the given segment.
If the segment's slope is 0, it's the value of the whole segment. In case the slope differs from 0, it's the value of
the start of the slope.
@param s the segment
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getValue(BorderLineSegment) Online Documentation}*/
getValue(s: BorderLineSegment | null): number;
/**
Returns the value that is set on this borderline at the specified position.
The position must lie within the range of the segment.
@param segment The segment whose value shall be returned.
@param pos the position where the value will be retrieved.
@returns the value
@throws {Exception} <b>(ArgumentError)</b> if <code>pos</code> is outside the segment's range.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getValueAt(BorderLineSegment%2Cnumber) Online Documentation}*/
getValueAt(segment: BorderLineSegment, pos: number): number;
/**
Returns the value that is set on this borderline at the specified position.
The position must lie within the range of the segment that is stored in <code>cell</code>.
@param cell The list cell containing the segment whose value shall be returned.
@param pos the position
@returns the value
@throws {Exception} <b>(ArgumentError)</b> if <code>pos</code> is outside the segment's range that is stored in <code>cell</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getValueAt(ListCell%2Cnumber) Online Documentation}*/
getValueAt(cell: ListCell, pos: number): number;
/**
Returns the value that is set on this borderline at the specified position.
@param pos the position
@returns the value
@throws {Exception} <b>(IndexOutOfRangeError)</b> if the position is outside of the borderline.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23getValueAt(number) Online Documentation}*/
getValueAt(pos: number): number;
/**
Grows this borderline horizontally, so that the {@link BorderLine#getValueAt values} of the borderline stay the same however their {@link BorderLine#getMin start} and {@link BorderLine#getMax end}
points are moved in the direction of <code>toMin</code> and <code>toMax</code>.
This is useful for scenarios where a borderline is needed that consists of an enlarged border.
Note that this method normalizes the segments, i.e., it transforms each segment with slope != 0 to a segment with slope
== 0.
@param toMin the delta by which the border should be extended towards -Infinity
@param toMax the delta by which the border should be extended towards +Infinity
@param positive whether the borderline should be interpreted to point in positive direction. This influences the direction into which a
segment's border is extended.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23grow(number%2Cnumber%2Cboolean) Online Documentation}*/
grow(toMin: number, toMax: number, positive: boolean): void;
/**
Returns the last segment or <code>null</code> if there is no such segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23lastSegment() Online Documentation}*/
lastSegment(): BorderLineSegment | null;
/**
Merges this borderline with the given borderline using the "maximum" policy.
@param other the other borderline
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23mergeWithMax(BorderLine) Online Documentation}*/
mergeWithMax(other: BorderLine | null): void;
/**
Merges this borderline with the given borderline using the "minimum" policy.
@param other the other borderline
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23mergeWithMin(BorderLine) Online Documentation}*/
mergeWithMin(other: BorderLine | null): void;
/**
Returns the next segment or <code>null</code> if there is no such segment.
@param s
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23next(BorderLineSegment) Online Documentation}*/
next(s: BorderLineSegment | null): BorderLineSegment | null;
/**
Returns the previous segment or <code>null</code> if there is no such segment.
@param s
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23prev(BorderLineSegment) Online Documentation}*/
prev(s: BorderLineSegment | null): BorderLineSegment | null;
/**
Assures that all values in the given interval are greater or equal than the given value.
@param min the lower end of the interval
@param max the upper end of the interval
@param value the smallest possible value for the interval
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23setMaxValue(number%2Cnumber%2Cnumber) Online Documentation}*/
setMaxValue(min: number, max: number, value: number): void;
/**
Assures that all values in the given interval are less or equal than the given value.
@param min the lower end of the interval
@param max the upper end of the interval
@param value the greatest possible value for the interval
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23setMinValue(number%2Cnumber%2Cnumber) Online Documentation}*/
setMinValue(min: number, max: number, value: number): void;
/**
Sets a specific interval to a slope starting at a given value.
@param min the left side of the interval.
@param max the right side of the interval.
@param value the value at <code>min</code> where the slope starts.
@param slope the slope of the segment in the given interval.
@throws {Exception} <b>(ArgumentError)</b> if <code>min</code> is greater than <code>max</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23setSloped(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
setSloped(min: number, max: number, value: number, slope: number): void;
/**
Sets a specific interval described by <code>min</code> and <code>max</code> to a given value.
@param min the left side of the interval.
@param max the right side of the interval.
@param value the value for the whole interval.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23setValue(number%2Cnumber%2Cnumber) Online Documentation}*/
setValue(min: number, max: number, value: number): void;
/**
Returns a lengthy String representation of this borderline.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23toString() Online Documentation}*/
toString(): string;
/**
Gets the smallest position of this borderline
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23min Online Documentation}*/
readonly min: number;
/**
Gets the greatest position of this borderline
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23max Online Documentation}*/
readonly max: number;
/**
Gets the minimum value that is set on this borderline
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23minValue Online Documentation}*/
readonly minValue: number;
/**
Gets the maximum value that is set on this borderline
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLine%23maxValue Online Documentation}*/
readonly maxValue: number;
static $class: Class;
private $_BorderLine: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BorderLine;
}
export interface Geom extends YObject {
}
/**
This class provides useful geometric primitives and advanced geometric algorithms.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom Online Documentation}*/
export class Geom {
/**
Calculates the convex hull for a set of points.
@param points a list of {@link YPoint} objects
@returns a list of {@link YPoint} objects that constitute the convex hull of the given points. The list contains points
in counter clockwise order around the hull. The first point is the one with the smallest <code>x</code> coordinate. If two such
points exist then of these points the one with the smallest <code>y</code> coordinate is chosen as the first one.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23calcConvexHull(YList) Online Documentation}*/
static calcConvexHull(points: YList | null): YList | null;
/**
Calculates the intersection point of two <b>affine</b> lines.
Each line is given by two points.
@param p1 one point on the first line.
@param p2 another point on the first line.
@param p3 one point on the second line.
@param p4 another point on the second line.
@returns the intersection point of the specified lines or <code>null</code> if there is no intersection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23calcIntersection(YPoint%2CYPoint%2CYPoint%2CYPoint) Online Documentation}*/
static calcIntersection(p1: YPoint, p2: YPoint, p3: YPoint, p4: YPoint): YPoint | null;
/**
Calculates the intersection point of two affine lines.
Each line is given by a point and a direction vector.
@param p1 origin point of the first line.
@param d1 direction vector of the first line.
@param p2 origin point of the second line.
@param d2 direction vector of the second line.
@returns the intersection point of the specified lines or <code>null</code> if there is no intersection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23calcIntersection(YPoint%2CYVector%2CYPoint%2CYVector) Online Documentation}*/
static calcIntersection(p1: YPoint, d1: YVector, p2: YPoint, d2: YVector): YPoint | null;
/**
Calculates the intersection point of two <b>affine</b> lines.
Each line is given by the coordinates of two points.
@param x1 x-coordinate of one point on the first line.
@param y1 y-coordinate of one point on the first line.
@param x2 x-coordinate of another point on the first line.
@param y2 y-coordinate of another point on the first line.
@param x3 x-coordinate of one point on the second line.
@param y3 y-coordinate of one point on the second line.
@param x4 x-coordinate of another point on the second line.
@param y4 y-coordinate of another point on the second line.
@returns the intersection point of the specified lines or <code>null</code> if there is no intersection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23calcIntersection(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static calcIntersection(x1: number, y1: number, x2: number, y2: number, x3: number, y3: number, x4: number, y4: number): YPoint | null;
/**
Intersects the pair of specified source <code>Rectangle2D</code> objects and puts the result into the specified destination <code>Rectangle2D</code>
object.
If one or both of the source rectangles have negative width or height, the resulting rectangle will be located at (0,0)
with a width and height of -1. One of the source rectangles can also be the destination to avoid creating a third
Rectangle2D object, but in this case the original points of this source rectangle will be overwritten by this method.
@param r1 the first of a pair of <code>Rectangle2D</code> objects to be intersected with each other
@param r2 the second of a pair of <code>Rectangle2D</code> objects to be intersected with each other
@param dest the <code>Rectangle2D</code> that holds the results of the intersection of <code>r1</code> and <code>r2</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23calcIntersection(Rectangle2D%2CRectangle2D%2CRectangle2D) Online Documentation}*/
static calcIntersection(r1: Rectangle2D | null, r2: Rectangle2D | null, dest: Rectangle2D | null): Rectangle2D | null;
/**
Unions the pair of source <code>Rectangle2D</code> objects and puts the result into the specified destination <code>Rectangle2D</code> object.
If one of the source rectangles has negative width or height, it is excluded from the union. If both source rectangles
have negative width or height, the destination rectangle will become a copy of <code>r1</code>. One of the source rectangles can also
be the destination to avoid creating a third Rectangle2D object, but in this case the original points of this source
rectangle will be overwritten by this method. If the destination is <code>null</code>, a new <code>Rectangle2D</code> is created.
@param r1 the first of a pair of <code>Rectangle2D</code> objects to be combined with each other
@param r2 the second of a pair of <code>Rectangle2D</code> objects to be combined with each other
@param dest the <code>Rectangle2D</code> that holds the results of the union of <code>r1</code> and <code>r2</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23calcUnion(Rectangle2D%2CRectangle2D%2CRectangle2D) Online Documentation}*/
static calcUnion(r1: Rectangle2D | null, r2: Rectangle2D | null, dest: Rectangle2D | null): Rectangle2D | null;
/**
Returns <code>true</code> iff the given points are collinear, that is, all three points lie on a common line.
Same as {@link Geom#orientation orientation(p,q,r) == 0}
@param p
@param q
@param r
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23collinear(YPoint%2CYPoint%2CYPoint) Online Documentation}*/
static collinear(p: YPoint | null, q: YPoint | null, r: YPoint | null): boolean;
/**
Determines the distance of the point <code>p</code> to the line segment <code>[l1, l2]</code>.
@param pointX the x coordinate of p
@param pointY the y coordinate of p
@param lineX1 the x coordinate of l1
@param lineY1 the y coordinate of l1
@param lineX2 the x coordinate of l2
@param lineY2 the y coordinate of l2
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23distanceToLineSegment(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static distanceToLineSegment(pointX: number, pointY: number, lineX1: number, lineY1: number, lineX2: number, lineY2: number): number;
/**
Same as {@link Geom#orientation orientation(p,q,r) > 0}
@param p
@param q
@param r
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23leftTurn(YPoint%2CYPoint%2CYPoint) Online Documentation}*/
static leftTurn(p: YPoint | null, q: YPoint | null, r: YPoint | null): boolean;
/**
Returns whether the two lines defined by the given coordinates intersect or not.
@param x1
@param y1
@param x2
@param y2
@param x3
@param y3
@param x4
@param y4
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23linesIntersect(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static linesIntersect(x1: number, y1: number, x2: number, y2: number, x3: number, y3: number, x4: number, y4: number): boolean;
/**
Same as {@link Geom#orientation} with double values as arguments.
@param px
@param py
@param qx
@param qy
@param rx
@param ry
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23orientation(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static orientation(px: number, py: number, qx: number, qy: number, rx: number, ry: number): number;
/**
Returns the orientation of point <code>r</code> relative to the directed line from point <code>p</code> to point <code>q</code>.
The given tuple of points is said to have positive orientation if <code>p</code> and <code>q</code> are distinct and <code>r</code> lies to the left of the
oriented line passing through <code>p</code> and <code>q</code> and oriented from <code>p</code> to <code>q</code>.
The tuple is said to have negative orientation if <code>p</code> and <code>q</code> are distinct and <code>r</code> lies to the right of the line, and the
tuple is said to have orientation zero if the three points are collinear.
@param p
@param q
@param r
@returns <b>+1</b> in the case of positive orientation, <b>-1</b> in the case of negative orientation and <b>0</b> in the case of zero orientation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23orientation(YPoint%2CYPoint%2CYPoint) Online Documentation}*/
static orientation(p: YPoint | null, q: YPoint | null, r: YPoint | null): number;
/**
Determines the projection of the point <code>p</code> onto the line segment <code>[l1, l2]</code>.
The resulting point is
<ul>
<li>the orthogonal projection of <code>p</code> onto the line through <code>l1</code> and <code>l2</code>, iff the projection lies on the line segment <code>[l1, l2]</code>
<li>the end point of the line segment <code>[l1, l2]</code> that is closest to <code>p</code>, otherwise
</ul>
@param pointX the x coordinate of p
@param pointY the y coordinate of p
@param lineX1 the x coordinate of l1
@param lineY1 the y coordinate of l1
@param lineX2 the x coordinate of l2
@param lineY2 the y coordinate of l2
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23projection(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static projection(pointX: number, pointY: number, lineX1: number, lineY1: number, lineX2: number, lineY2: number): YPoint | null;
/**
Same as {@link Geom#orientation orientation(p,q,r) &lt; 0}
@param p
@param q
@param r
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23rightTurn(YPoint%2CYPoint%2CYPoint) Online Documentation}*/
static rightTurn(p: YPoint | null, q: YPoint | null, r: YPoint | null): boolean;
/**
Returns +1 if point <code>d</code> lies left of the directed circle through points <code>a</code>, <code>b</code>, and <code>c</code>, 0 if <code>a,b,c</code> and <code>d</code> are cocircular, and
-1 otherwise.
@param a
@param b
@param c
@param d
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23sideOfCircle(YPoint%2CYPoint%2CYPoint%2CYPoint) Online Documentation}*/
static sideOfCircle(a: YPoint | null, b: YPoint | null, c: YPoint | null, d: YPoint | null): number;
/**
Converts the given degree value from radian to angular
@param angrad
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23toDegrees(number) Online Documentation}*/
static toDegrees(angrad: number): number;
/**
Converts the given degree value from angular to radian.
@param angdeg
@see {@link https://docs.yworks.com/yfileshtml/#/api/Geom%23toRadians(number) Online Documentation}*/
static toRadians(angdeg: number): number;
static $class: Class;
private $_Geom: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Geom;
}
export interface ILineSegmentCursor extends ICursor {
}
/**
This is an interface for a sequence of instances of {@link LineSegment}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILineSegmentCursor Online Documentation}*/
export abstract class ILineSegmentCursor {
/**
Gets the instance of {@link LineSegment} the cursor is currently pointing on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILineSegmentCursor%23lineSegment Online Documentation}*/
abstract readonly lineSegment: LineSegment | null;
static $class: Class;
private $_ILineSegmentCursor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILineSegmentCursor;
}
export interface IPlaneObject {
}
/**
This interface describes a 2-dimensional object which has a finite bounding box.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPlaneObject Online Documentation}*/
export abstract class IPlaneObject {
/**
Gets the smallest Rectangle which contains the object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPlaneObject%23boundingBox Online Documentation}*/
abstract readonly boundingBox: YRectangle | null;
static $class: Class;
private $_IPlaneObject: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPlaneObject;
}
export interface IPointCursor extends ICursor {
}
/**
This is an interface for a sequence of instances of {@link YPoint}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPointCursor Online Documentation}*/
export abstract class IPointCursor {
/**
Gets the instance of {@link YPoint} the cursor is currently pointing on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPointCursor%23point Online Documentation}*/
abstract readonly point: YPoint | null;
static $class: Class;
private $_IPointCursor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPointCursor;
}
export interface YInsets extends YObject, ICloneable {
/**
@inheritdoc*/
clone(): this;
} /**
Double-precision immutable insets representation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets Online Documentation}*/
export class YInsets {
/**
Creates a new instance with the given values
@param top The new top inset value
@param left The new left inset value
@param bottom The new bottom inset value
@param right The new right inset value
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets Online Documentation}*/
constructor(top: number, left: number, bottom: number, right: number);
/**
Returns whether this insets object and <code>o</code> are equal.
Two insets are considered equal if the have the same values for each side.
@param o The object to test for equality
@returns <code>true</code> if both objects are equal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets%23equals(Object) Online Documentation}*/
equals(o: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Creates a {@link Insets} from a given {@link YInsets}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Insets}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets%23toInsets() Online Documentation}*/
toInsets(): Insets;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets%23toString() Online Documentation}*/
toString(): string;
/**
The inset from the top.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets%23top Online Documentation}*/
top: number;
/**
The inset from the left.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets%23left Online Documentation}*/
left: number;
/**
The inset from the bottom.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets%23bottom Online Documentation}*/
bottom: number;
/**
The inset from the right.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/YInsets%23right Online Documentation}*/
right: number;
static $class: Class;
private $_Insets: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YInsets;
}
export interface IIntersectionHandler {
}
/**
An instance of this interface handles intersections found by the <code>IntersectionAlgorithm</code>,
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIntersectionHandler Online Documentation}*/
export abstract class IIntersectionHandler {
/**
This method is called at every intersection.
@param a
@param b
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIntersectionHandler%23checkIntersection(Object%2CObject) Online Documentation}*/
abstract checkIntersection(a: any | null, b: any | null): void;
/**
Creates an implementation of the interface {@link IIntersectionHandler} by using the given function as
implementation for its {@link IIntersectionHandler#checkIntersection} method.
@param checkIntersection A function for {@link IIntersectionHandler}'s single abstract method
{@link IIntersectionHandler#checkIntersection}.
@returns An instance of the {@link IIntersectionHandler} interface based on the given function.*/
static create(checkIntersection: (a: any | null, b: any | null) => void): IIntersectionHandler;
static $class: Class;
private $_IIntersectionHandler: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IIntersectionHandler;
}
export interface IntersectionAlgorithm extends YObject {
}
/**
This class calculates the intersection of rectangles in the plane with the help of a sweep-line algorithm.
The complexity is <code>O(n log n + s)</code> where <code>n</code> is the number of rectangles and <code>s</code> the number of intersections.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IntersectionAlgorithm Online Documentation}*/
export class IntersectionAlgorithm {
/**
Calculates the intersections of rectangles in the plane.
Every found intersection is reported to an
<code>IntersectionHandler</code>. Rectangles with negative size are completely ignored by this implementation (i.e. never generate
intersections)
@param objects a list of <code>PlaneObject</code> objects.
@param iHandler intersections are reported to this class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IntersectionAlgorithm%23intersect(YList%2CIIntersectionHandler) Online Documentation}*/
static intersect(objects: YList | null, iHandler: IIntersectionHandler | null): void;
static $class: Class;
private $_IntersectionAlgorithm: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IntersectionAlgorithm;
}
export interface LineSegment extends YObject, IPlaneObject {
}
/**
This class represents a line segment in the plane.
A line segment is defined by its two end points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment Online Documentation}*/
export class LineSegment {
/**
Returns a new {@link LineSegment}.
@param p1 the first end point of the line segment.
@param p2 the second end point of the line segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment Online Documentation}*/
constructor(p1: YPoint | null, p2: YPoint | null);
/**
Checks whether a line segment intersects a paraxial box.
Implemented using the Cohen-Sutherland algorithm.
@param boxX1 x-coordinate of the upper left corner of the box
@param boxY1 y-coordinate of the upper left corner of the box
@param boxX2 x-coordinate of the lower right corner of the box
@param boxY2 y-coordinate of the lower right corner of the box
@param sX1 x-coordinate of the first end point of the line segment
@param sY1 y-coordinate of the first end point of the line segment
@param sX2 x-coordinate of the second end point of the line segment
@param sY2 y-coordinate of the second end point of the line segment
@returns <code>true</code> if the line segment intersects the box, <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23boxIntersectsSegment(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static boxIntersectsSegment(boxX1: number, boxY1: number, boxX2: number, boxY2: number, sX1: number, sY1: number, sX2: number, sY2: number): boolean;
/**
Checks whether a line segment intersects a box.
Implemented using the Cohen-Sutherland algorithm.
@param box A rectangle
@param x1 x-coordinate of the first end point of the line segment
@param y1 y-coordinate of the first end point of the line segment
@param x2 x-coordinate of the second end point of the line segment
@param y2 y-coordinate of the second end point of the line segment
@returns <code>true</code> if the line segment intersects the box, <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23boxIntersectsSegment(YRectangle%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static boxIntersectsSegment(box: YRectangle | null, x1: number, y1: number, x2: number, y2: number): boolean;
/**
Checks whether a line segment intersects a box.
@param box A rectangle.
@param s first end point of the line segment.
@param t second end point of the line segment.
@returns <code>true</code> if the line segments intersects the box, <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23boxIntersectsSegment(YRectangle%2CYPoint%2CYPoint) Online Documentation}*/
static boxIntersectsSegment(box: YRectangle | null, s: YPoint | null, t: YPoint | null): boolean;
/**
Checks whether a given point lies on this line segment.
@param point an arbitrary point.
@returns <code>true</code> if the line segments intersects the box, <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23contains(YPoint) Online Documentation}*/
contains(point: YPoint | null): boolean;
/**
Returns intersection point between the two line segments, if there is one or <code>null</code> if the two line segments do not
intersect.
@param s1 first line segment
@param s2 second line segment
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23getIntersection(LineSegment%2CLineSegment) Online Documentation}*/
static getIntersection(s1: LineSegment | null, s2: LineSegment | null): YPoint | null;
/**
Checks whether the line segment intersects a point.
@param p a point
@returns <code>true</code> if the line segments intersects the given point, <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23intersects(YPoint) Online Documentation}*/
intersects(p: YPoint): boolean;
/**
Checks whether the line segment intersects a box.
@param box A rectangle.
@returns <code>true</code> if the line segments intersects the box, <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23intersects(YRectangle) Online Documentation}*/
intersects(box: YRectangle): boolean;
/**
Determines if the given points define a horizontal line segment.
This methods allows for y-coordinate differences of up to 1e-8.
@param p1
@param p2
@returns <code>true</code> if the given points define a horizontal line segment; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23isHorizontalSegment(YPoint%2CYPoint) Online Documentation}*/
static isHorizontalSegment(p1: YPoint | null, p2: YPoint | null): boolean;
/**
Returns if the projection on the X axis of the line segment covers a certain point on the X Axis.
@param x
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23isInXIntervall(number) Online Documentation}*/
isInXIntervall(x: number): boolean;
/**
Returns if the projection on the Y axis of the line segment covers a certain point on the Y Axis.
@param y
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23isInYIntervall(number) Online Documentation}*/
isInYIntervall(y: number): boolean;
/**
Determines if the given points define a vertical line segment.
This methods allows for x-coordinate differences of up to 1e-8.
@param p1
@param p2
@returns <code>true</code> if the given points define a vertical line segment; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23isVerticalSegment(YPoint%2CYPoint) Online Documentation}*/
static isVerticalSegment(p1: YPoint | null, p2: YPoint | null): boolean;
/**
Returns the length of the line segment, this is the value of the Euclidean norm.
@returns an value > 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23length() Online Documentation}*/
length(): number;
/**
Returns the affine line defined by the end points of the line segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23toAffineLine() Online Documentation}*/
toAffineLine(): AffineLine | null;
/**
String representation of the line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23toString() Online Documentation}*/
toString(): string;
/**
Returns the vector pointing from the first end point to the second end point of the line segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23toYVector() Online Documentation}*/
toYVector(): YVector | null;
/**
Gets if this segment is considered vertical, that is, the x values of the end points differ by at most 1e-8.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23isVertical Online Documentation}*/
readonly isVertical: boolean;
/**
Gets if this segment is considered horizontal, that is, the y values of the end points differ by at most 1e-8.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23isHorizontal Online Documentation}*/
readonly isHorizontal: boolean;
/**
Gets the first end point of the line segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23firstEndPoint Online Documentation}*/
readonly firstEndPoint: YPoint | null;
/**
Gets the second end point of the line segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23secondEndPoint Online Documentation}*/
readonly secondEndPoint: YPoint | null;
/**
Gets the y value of the line on x coordinate 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23xOffset Online Documentation}*/
readonly xOffset: number;
/**
Gets the slope of the line segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23slope Online Documentation}*/
readonly slope: number;
/**
Gets the smallest Rectangle which contains the object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23boundingBox Online Documentation}*/
readonly boundingBox: YRectangle | null;
/**
Gets the distance from start to end point in x-coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23deltaX Online Documentation}*/
readonly deltaX: number;
/**
Gets the distance from start to end point in y-coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LineSegment%23deltaY Online Documentation}*/
readonly deltaY: number;
static $class: Class;
private $_LineSegment: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LineSegment;
}
export interface Point2D extends YObject, ICloneable {
}
/**
Represents a two-dimensional point located at <code>(x, y)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D Online Documentation}*/
export class Point2D {
/**
Creates a new instance at the given location.
@param x The x coordinate of the new point
@param y The x coordinate of the new point
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23Point2D(number%2Cnumber) Online Documentation}*/
constructor(x: number, y: number);
/**
Creates a new instance that is located at <code>(0, 0)</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23Point2D() Online Documentation}*/
constructor();
/**
Creates a new instance of this class that has the same values as this objects.
@returns a new instance of this class that has the same values as this objects.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23clone() Online Documentation}*/
clone(): this;
/**
Calculates the distance between this points and another one, given as its coordinates.
@param px The x coordinate of the second point.
@param py The y coordinate of the second point.
@returns the distance between this points and another one, given as its coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23distance(number%2Cnumber) Online Documentation}*/
distance(px: number, py: number): number;
/**
Calculates the distance between this points and another one.
@param p The second point.
@returns the distance between this points and another one.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23distance(Point2D) Online Documentation}*/
distance(p: Point2D | null): number;
/**
Calculates the distance between two points, given as their coordinates.
@param x1 The x coordinate of the first point.
@param y1 The y coordinate of the first point.
@param x2 The x coordinate of the second point.
@param y2 The y coordinate of the second point.
@returns the distance between two points, given as their coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23distance(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static distance(x1: number, y1: number, x2: number, y2: number): number;
/**
Calculates the squared distance between this points and another one, given as its coordinates.
@param px The x coordinate of the second point.
@param py The y coordinate of the second point.
@returns the squared distance between this points and another one, given as its coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23distanceSq(number%2Cnumber) Online Documentation}*/
distanceSq(px: number, py: number): number;
/**
Calculates the squared distance between this points and another one.
@param p The second point.
@returns the squared distance between this points and another one.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23distanceSq(Point2D) Online Documentation}*/
distanceSq(p: Point2D | null): number;
/**
Calculates the squared distance between two points, given as their coordinates.
@param x1 The x coordinate of the first point.
@param y1 The y coordinate of the first point.
@param x2 The x coordinate of the second point.
@param y2 The y coordinate of the second point.
@returns the squared distance between two points, given as their coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23distanceSq(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static distanceSq(x1: number, y1: number, x2: number, y2: number): number;
/**
Returns whether this point and <code>obj</code> are equal.
Two points are considered equal if the have the same location.
@param obj The object to test for equality
@returns <code>true</code> if both objects are equal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23equals(Object) Online Documentation}*/
equals(obj: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Sets the location of this point to the given coordinates.
@param x the new x coordinate
@param y the new y coordinate
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23setLocation(number%2Cnumber) Online Documentation}*/
setLocation(x: number, y: number): void;
/**
Creates a {@link Point} from a given {@link Point2D}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Point}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23toPoint() Online Documentation}*/
toPoint(): Point;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23toString() Online Documentation}*/
toString(): string;
/**
Sets the location of this point to the same values as for the given point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23location Online Documentation}*/
location: Point2D | null;
/**
Gets the x coordinate of this point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23x Online Documentation}*/
readonly x: number;
/**
Gets the y coordinate of this point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Point2D%23y Online Documentation}*/
readonly y: number;
static $class: Class;
private $_Point2D: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Point2D;
}
export interface Rectangle2D extends YObject, ICloneable {
}
/**
Represents a two-dimensional rectangle of size <code>(width x height)</code>, located at the point <code>(x, y)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D Online Documentation}*/
export class Rectangle2D {
/**
Creates a new rectangle of the specified size at the specified location.
@param x the x-coordinate of the upper left corner of the created rectangle.
@param y the y-coordinate of the upper left corner of the created rectangle.
@param width the width of the created rectangle.
@param height the height of the created rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23Rectangle2D(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(x: number, y: number, width: number, height: number);
/**
Creates a new rectangle of size <code>(0 x 0)</code> at the location <code>(0, 0)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23Rectangle2D() Online Documentation}*/
constructor();
/**
Adds a point, specified by its coordinates, to this rectangle.
The rectangle will be grown if necessary. Note that for points that would lie on the right or bottom border of the
rectangle, {@link Rectangle2D#contains} will still return <code>false</code> for the added point.
@param px the x coordinate of the point to add
@param py the y coordinate of the point to add
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23add(number%2Cnumber) Online Documentation}*/
add(px: number, py: number): void;
/**
Adds a rectangle to this rectangle.
The rectangle will be grown to the union of both rectangles
@param r the rectangle to add
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23add(Rectangle2D) Online Documentation}*/
add(r: Rectangle2D | null): void;
/**
Creates a new instance of this class that has the same values as this objects.
@returns a new instance of this class that has the same values as this objects.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23clone() Online Documentation}*/
clone(): this;
/**
Checks whether this rectangle completely contains the second specified rectangle.
@param x the x coordinate of the second rectangle.
@param y the y coordinate of the second rectangle.
@param width the width coordinate of the second rectangle.
@param height the height coordinate of the second rectangle.
@returns <code>true</code> if this rectangle contains the second one
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23contains(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
contains(x: number, y: number, width: number, height: number): boolean;
/**
Checks if the point specified by the given coordinates is contained in this rectangle.
@param x the x-coordinate of the point.
@param y the y-coordinate of the point.
@returns <code>true</code> if the specified point is contained in this rectangle; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23contains(number%2Cnumber) Online Documentation}*/
contains(x: number, y: number): boolean;
/**
Checks whether this rectangle completely contains the second specified rectangle.
@param rect the rectangle to check for containment
@returns <code>true</code> if this rectangle contains the second one
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23contains(Rectangle2D) Online Documentation}*/
contains(rect: Rectangle2D | null): boolean;
/**
Calculates the intersection of this rectangle with the given rectangle and returns the result as new rectangle.
@param r a rectangle to intersect with this rectangle.
@returns a new rectangle that represents the calculated intersection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23createIntersection(Rectangle2D) Online Documentation}*/
createIntersection(r: Rectangle2D | null): Rectangle2D | null;
/**
Calculates the union of this rectangle with the given rectangle and returns the result as new rectangle.
@param r a rectangle to union with this rectangle.
@returns a new rectangle that represents the calculated union.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23createUnion(Rectangle2D) Online Documentation}*/
createUnion(r: Rectangle2D | null): Rectangle2D | null;
/**
Returns whether this rectangle and <code>obj</code> are equal.
Two rectangles are considered equal if the have the same location and size.
@param obj The object to test for equality
@returns <code>true</code> if both objects are equal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23equals(Object) Online Documentation}*/
equals(obj: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Calculates the intersection of the first and second specified rectangle and sets the resulting location and size to the
third rectangle.
@param src1 the first rectangle to intersect.
@param src2 the second rectangle to intersect.
@param dst the rectangle to which the result is set.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23intersect(Rectangle2D%2CRectangle2D%2CRectangle2D) Online Documentation}*/
static intersect(src1: Rectangle2D | null, src2: Rectangle2D | null, dst: Rectangle2D | null): void;
/**
Checks whether this rectangle and the second specified rectangle intersect.
@param x the x coordinate of the second rectangle.
@param y the y coordinate of the second rectangle.
@param width the width coordinate of the second rectangle.
@param height the height coordinate of the second rectangle.
@returns <code>true</code> if both rectangles intersect
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23intersects(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
intersects(x: number, y: number, width: number, height: number): boolean;
/**
Checks whether this rectangle and the second specified rectangle intersect.
@param rect the rectangle to check for intersection
@returns <code>true</code> if both rectangles intersect
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23intersects(Rectangle2D) Online Documentation}*/
intersects(rect: Rectangle2D | null): boolean;
/**
Checks if the line segment specified by the given coordinates intersects this rectangle.
@param x1 the x-coordinate of the first end point of the line.
@param y1 the y-coordinate of the first end point of the line.
@param x2 the x-coordinate of the other end point of the line.
@param y2 the y-coordinate of the other end point of the line.
@returns <code>true</code> if the specified line intersects this rectangle; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23intersectsLine(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
intersectsLine(x1: number, y1: number, x2: number, y2: number): boolean;
/**
Sets the outer bounds of this rectangle based on the specified location and size.
@param x the new x-coordinate of the upper left corner.
@param y the new y-coordinate of the upper left corner.
@param width the new width.
@param height the new height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23setFrame(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
setFrame(x: number, y: number, width: number, height: number): void;
/**
Sets the outer bounds of this rectangle based on the specified new center and corner.
@param centerX the x-coordinate of the new center of this rectangle.
@param centerY the y-coordinate of the new center of this rectangle.
@param cornerX the x-coordinate of any new corner of this rectangle.
@param cornerY the y-coordinate of any new corner of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23setFrameFromCenter(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
setFrameFromCenter(centerX: number, centerY: number, cornerX: number, cornerY: number): void;
/**
Sets the outer bounds of this rectangle based on the specified new center and corner.
@param center the new center of this rectangle.
@param corner any new corner of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23setFrameFromCenter(Point2D%2CPoint2D) Online Documentation}*/
setFrameFromCenter(center: Point2D | null, corner: Point2D | null): void;
/**
Sets the outer bounds of this rectangle based on the end points of one of its diagonals.
@param x1 the x-coordinate of the first end point of a diagonal.
@param y1 the y-coordinate of the first end point of a diagonal.
@param x2 the x-coordinate of the other end point of a diagonal.
@param y2 the y-coordinate of the other end point of a diagonal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23setFrameFromDiagonal(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
setFrameFromDiagonal(x1: number, y1: number, x2: number, y2: number): void;
/**
Sets the outer bounds of this rectangle based on the end points of one of its diagonals.
@param p1 the first end point of a diagonal.
@param p2 the other end point of a diagonal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23setFrameFromDiagonal(Point2D%2CPoint2D) Online Documentation}*/
setFrameFromDiagonal(p1: Point2D | null, p2: Point2D | null): void;
/**
Sets the location and size of this rectangle to the specified values.
@param x the new x-coordinate of the upper left corner.
@param y the new y-coordinate of the upper left corner.
@param width the new width.
@param height the new height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23setRect(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
setRect(x: number, y: number, width: number, height: number): void;
/**
Creates a {@link Rect} from a given {@link Rectangle2D}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Rect}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23toRect() Online Documentation}*/
toRect(): Rect;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23toString() Online Documentation}*/
toString(): string;
/**
Calculates the union of the first and second specified rectangle and sets the resulting location and size to the third
rectangle.
@param src1 the first rectangle to union.
@param src2 the second rectangle to union.
@param dst the rectangle to which the result is set.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23union(Rectangle2D%2CRectangle2D%2CRectangle2D) Online Documentation}*/
static union(src1: Rectangle2D | null, src2: Rectangle2D | null, dst: Rectangle2D | null): void;
/**
Gets or sets the outer bounds of this rectangle in double coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23frame Online Documentation}*/
frame: Rectangle2D | null;
/**
Sets the location and size of this rectangle to be similar to the specified rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23rect Online Documentation}*/
rect: Rectangle2D | null;
/**
Gets the x coordinate of the upper left corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23x Online Documentation}*/
readonly x: number;
/**
Gets the y coordinate of the upper left corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23y Online Documentation}*/
readonly y: number;
/**
Gets the width of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23width Online Documentation}*/
readonly width: number;
/**
Gets the height of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23height Online Documentation}*/
readonly height: number;
/**
Gets whether this instance is empty, that is, covers no area.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23empty Online Documentation}*/
readonly empty: boolean;
/**
Gets the smallest x coordinate of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23minX Online Documentation}*/
readonly minX: number;
/**
Gets the smallest y coordinate of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23minY Online Documentation}*/
readonly minY: number;
/**
Gets the largest x coordinate of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23maxX Online Documentation}*/
readonly maxX: number;
/**
Gets the largest y coordinate of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23maxY Online Documentation}*/
readonly maxY: number;
/**
Gets the x coordinate of the center point of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23centerX Online Documentation}*/
readonly centerX: number;
/**
Gets the y coordinate of the center point of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23centerY Online Documentation}*/
readonly centerY: number;
/**
Gets the bounds of this instance in double precision.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Rectangle2D%23bounds2D Online Documentation}*/
readonly bounds2D: Rectangle2D | null;
static $class: Class;
private $_Rectangle2D: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Rectangle2D;
}
export interface TriangulationAlgorithm extends YObject {
}
/**
This class provides algorithms for the triangulation of point sets in the plane.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TriangulationAlgorithm Online Documentation}*/
export class TriangulationAlgorithm {
/**
Computes a Delauney triangulation of the given points.
A Delauney triangulation is a triangulation such that none of the given points is inside the circumcircle of any of the
calculated triangles.
The calculated triangulation is represented by an embedded graph, i.e. to each edge there exists a reverse edge and the
outedges around each node are in embedded order. The returned edge and the (optional) reverseEdgeMap can be used to
construct all faces of the plane graph and to determine its outer face.
@param result a graph whose nodes represent the points that need to be triangulated.
@param pointData must provide the location (YPoint) for each node in the given graph.
@param revMap a node map that will contain for each edge its reverse edge. If this argument is <code>null</code> then no reverse edge information
will be available.
@returns an edge on the outer face of the result graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TriangulationAlgorithm%23calcDelauneyTriangulation(Graph%2CIDataProvider%2CIEdgeMap) Online Documentation}*/
static calcDelauneyTriangulation(result: Graph | null, pointData: IDataProvider | null, revMap: IEdgeMap | null): Edge | null;
/**
Computes a triangulation of the given points.
The calculated triangulation is represented by an embedded graph, i.e. to each edge there exists a reverse edge and the
outedges around each node are in embedded order. The returned edge and the (optional) reverseEdgeMap can be used to
construct all faces of the plane graph and to determine its outer face.
@param points the point set to be triangulated. The points must be provided as a YList of YPoints.
@param result the resulting triangulation
@param resultMap the node map that forms the link between a point and a node.
@param reverseEdgeMap a node map that will contain for each edge its reverse edge. If this argument is <code>null</code> then no reverse edge information
will be available.
@returns an edge on the outer face of the result graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TriangulationAlgorithm%23triangulatePoints(YList%2CGraph%2CINodeMap%2CIEdgeMap) Online Documentation}*/
static triangulatePoints(points: YList | null, result: Graph | null, resultMap: INodeMap | null, reverseEdgeMap: IEdgeMap | null): Edge | null;
/**
Computes a triangulation of the given points.
The calculated triangulation is represented by an embedded graph, i.e. to each edge there exists a reverse edge and the
outedges around each node are in embedded order. The returned edge and the (optional) reverseEdgeMap can be used to
construct all faces of the plane graph and to determine its outer face.
@param result a graph whose nodes represent the points that need to be triangulated.
@param pointData must provide the location (YPoint) for each node in the given graph.
@param reverseEdgeMap a node map that will contain for each edge its reverse edge. If this argument is <code>null</code> then no reverse edge information
will be available.
@returns an edge on the outer face of the result graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TriangulationAlgorithm%23triangulatePoints(Graph%2CIDataProvider%2CIEdgeMap) Online Documentation}*/
static triangulatePoints(result: Graph | null, pointData: IDataProvider | null, reverseEdgeMap: IEdgeMap | null): Edge | null;
static $class: Class;
private $_Triangulator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TriangulationAlgorithm;
}
export interface YDimension extends YObject, IComparable {
/**
@inheritdoc*/
compareTo(o: any | null): number;
} /**
This class represents the size of an object.
An instance of this class implements the immutable design pattern.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YDimension Online Documentation}*/
export class YDimension {
/**
Creates a new YDimension2D object for given size.
@param width
@param height
@see {@link https://docs.yworks.com/yfileshtml/#/api/YDimension Online Documentation}*/
constructor(width: number, height: number);
/**
Tests a dimension to equality to another dimension.
@param o
@see {@link https://docs.yworks.com/yfileshtml/#/api/YDimension%23equals(Object) Online Documentation}*/
equals(o: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/YDimension%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Creates a {@link Size} from a given {@link YDimension}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Size}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YDimension%23toSize() Online Documentation}*/
toSize(): Size;
/**
Returns the size in the form: "W: width H: height"
@see {@link https://docs.yworks.com/yfileshtml/#/api/YDimension%23toString() Online Documentation}*/
toString(): string;
/**
Gets the width of the dimension object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YDimension%23width Online Documentation}*/
readonly width: number;
/**
Gets the height of the dimension object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YDimension%23height Online Documentation}*/
readonly height: number;
static $class: Class;
private $_YDimension: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YDimension;
}
export interface YOrientedRectangle extends YObject, IPlaneObject {
}
/**
An oriented rectangle in 2D coordinate space with double precision coordinates.
The rectangle's <code>height</code> extends from its <em>{@link YOrientedRectangle#anchor anchor point}</em> in the direction of its <em>up
vector</em> ({@link YOrientedRectangle#upX ux}, {@link YOrientedRectangle#upY uy}). Its <code>width</code> extends from its {@link YOrientedRectangle#anchor anchor point} in
direction <code>(-uy, ux)</code> (i.e. perpendicular to the <em>up vector</em>). This means that an oriented rectangle with anchor point <code>(0, 0)</code> width
<code>100</code>, height <code>10</code>, and up vector <code>(0, -1)</code> is a paraxial rectangle with upper left corner <code>(0, -10)</code> and lower right corner
<code>(100, 0)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle Online Documentation}*/
export class YOrientedRectangle {
/**
Creates a new instance using the provided rectangle's values to initialize anchor, size, and up vector.
@param rect the provided rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23YOrientedRectangle(YOrientedRectangle) Online Documentation}*/
constructor(rect: YOrientedRectangle);
/**
Creates a new instance using the provided rectangle's values to initialize anchor and size.
The oriented rectangle's up vector will be <code>(0, -1)</code>.
@param rect the provided rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23YOrientedRectangle(YRectangle) Online Documentation}*/
constructor(rect: YRectangle);
/**
Creates a new instance using the provided values to initialize anchor, size, and up vector.
@param anchorX The x coordinate of the anchor of the oriented rectangle.
@param anchorY The y coordinate of the anchor of the oriented rectangle.
@param width The width of the rectangle.
@param height The height of the rectangle.
@param upX The x component of the up vector.
@param upY The y component of the up vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23YOrientedRectangle(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(anchorX: number, anchorY: number, width: number, height: number, upX: number, upY: number);
/**
Creates a new instance using the provided values to initialize anchor and size.
The oriented rectangle's up vector will be <code>(0, -1)</code>.
@param anchorX The x coordinate of the anchor of the oriented rectangle.
@param anchorY The y coordinate of the anchor of the oriented rectangle.
@param width The width of the rectangle.
@param height The height of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23YOrientedRectangle(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(anchorX: number, anchorY: number, width: number, height: number);
/**
Creates a new instance using the provided values to initialize anchor, size, and up vector.
@param position The provider for the dynamic anchor of this instance.
@param size The provider for the dynamic size of this instance.
@param upVector The up vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23YOrientedRectangle(YPoint%2CYDimension%2CYVector) Online Documentation}*/
constructor(position: YPoint | null, size: YDimension | null, upVector: YVector | null);
/**
Creates a new instance using the provided values to initialize the anchor and size.
The oriented rectangle's up vector will be <code>(0, -1)</code>.
@param anchor The provider for the dynamic anchor of this instance.
@param size The provider for the dynamic size of this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23YOrientedRectangle(YPoint%2CYDimension) Online Documentation}*/
constructor(anchor: YPoint | null, size: YDimension | null);
/**
Copies the actual values from the given <code>OrientedRectangle</code> to this instance.
@param other the <code>OrientedRectangle</code> to retrieve the values from
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23adoptValues(YOrientedRectangle) Online Documentation}*/
adoptValues(other: YOrientedRectangle | null): void;
/**
Determines the four corner points of an oriented rectangle.
@param rect The rectangle to determine the bounds.
@returns the array of corner points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23calcPoints(YOrientedRectangle) Online Documentation}*/
static calcPoints(rect: YOrientedRectangle | null): YPoint[] | null;
/**
Determines the coordinates of the four corners of an oriented rectangle.
@param rect The rectangle to determine the bounds.
@returns the array of coordinates of the corner points in which each even index has the x-coordinate and each odd index the
y-coordinate
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23calcPointsInDouble(YOrientedRectangle) Online Documentation}*/
static calcPointsInDouble(rect: YOrientedRectangle | null): number[] | null;
/**
Determines whether or not the specified point lies inside this oriented rectangle.
@param x the x-coordinate of the point to check.
@param y the y-coordinate of the point to check.
@param [closed=false] if <code>true</code>, all points on the border of the rectangle are considered to be <em>contained</em>.
@returns <code>true</code> iff the specified point lies inside; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23contains(number%2Cnumber%2Cboolean) Online Documentation}*/
contains(x: number, y: number, closed?: boolean): boolean;
/**
Determines whether the given rectangle r1 contains rectangle r2, using an epsilon value.
@param r1 The first rectangle.
@param r2 The second rectangle.
@param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
or equal to that value, it will be considered a hit.
@returns true iff the r1 contains r2.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23contains(YOrientedRectangle%2CYOrientedRectangle%2Cnumber) Online Documentation}*/
static contains(r1: YOrientedRectangle, r2: YOrientedRectangle, eps: number): boolean;
/**
Determines whether the given oriented rectangle contains the provided point, using an epsilon value.
@param rect The rectangle.
@param p The point to test.
@param eps fuzziness range. A positive value allows for fuzzy hit testing. If a point lies outside the given rectangle, but its
distance is less than or equal to that value, it will be considered a hit.
@returns <code>true</code> if the point lies inside the rectangle; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23contains(YOrientedRectangle%2CYPoint%2Cnumber) Online Documentation}*/
static contains(rect: YOrientedRectangle, p: YPoint, eps: number): boolean;
/**
Determines whether the given oriented rectangle contains the provided point, using an epsilon value.
@param rect The rectangle.
@param x x-coordinate of the point to test.
@param y y-coordinate of the point to test.
@param eps fuzziness range. A positive value allows for fuzzy hit testing. If a point lies outside the given rectangle, but its
distance is less than or equal to that value, it will be considered a hit.
@returns <code>true</code> if the point lies inside the rectangle; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23contains(YOrientedRectangle%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static contains(rect: YOrientedRectangle | null, x: number, y: number, eps: number): boolean;
/**
Determines whether another object is equal to this one.
@param o the other object to compare for equality
@returns <code>true</code> if <code>o</code> is considered equal to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23equals(Object) Online Documentation}*/
equals(o: any | null): boolean;
/**
Creates a new <code>OrientedRectangle</code> instance whose anchor point is moved by the specified distance values, but has the same
width, height, and up vector as this rectangle.
@param dx the distance to move the anchor point in x-direction. A positive value means "move" to the right, a negative value means
"move" to the left.
@param dy the distance to move the anchor point in y-direction. A positive value means "move" downwards, a negative value means
"move" upwards.
@returns a new <code>OrientedRectangle</code> instance whose anchor point is moved by the specified distance values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23getMovedInstance(number%2Cnumber) Online Documentation}*/
getMovedInstance(dx: number, dy: number): YOrientedRectangle | null;
/**
Creates a new <code>OrientedRectangle</code> instance that has the specified width and height, but has the same anchor point and up
vector as this rectangle.
@param width the width of the new rectangle.
@param height the height of the new rectangle.
@returns a new <code>OrientedRectangle</code> instance that has the specified width and height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23getResizedInstance(number%2Cnumber) Online Documentation}*/
getResizedInstance(width: number, height: number): YOrientedRectangle | null;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Determines an intersection point of the specified oriented rectangle and the specified line segment.
Note: there might be more than one intersection point. However this method only returns one intersection point or <code>null</code>
if there is no intersection.
@param rect
@param line
@param eps
@returns an intersection point of the specified oriented rectangle and the specified line segment or <code>null</code> if the rectangle and
the segment do not intersect.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23intersectionPoint(YOrientedRectangle%2CLineSegment%2Cnumber) Online Documentation}*/
static intersectionPoint(rect: YOrientedRectangle | null, line: LineSegment | null, eps: number): YPoint | null;
/**
Determines whether or not the specified oriented rectangle and the specified line segment intersect.
@param rect
@param line
@param eps
@returns <code>true</code> if the rectangle and the segment intersect and <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23intersects(YOrientedRectangle%2CLineSegment%2Cnumber) Online Documentation}*/
static intersects(rect: YOrientedRectangle, line: LineSegment, eps: number): boolean;
/**
Determines whether a rectangle intersects an oriented rectangle, given an epsilon.
@param orientedRectangle The oriented rectangle to test.
@param rectangle The rectangle to test.
@param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
or equal to that value, it will be considered a hit.
@returns Whether they have a non-empty intersection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23intersects(YOrientedRectangle%2CYRectangle%2Cnumber) Online Documentation}*/
static intersects(orientedRectangle: YOrientedRectangle, rectangle: YRectangle, eps: number): boolean;
/**
Moves this rectangle by applying the offset to the anchor.
@param dx The x offset to move the rectangle's position by.
@param dy The y offset to move the rectangle's position by.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23moveBy(number%2Cnumber) Online Documentation}*/
moveBy(dx: number, dy: number): void;
/**
Sets the anchor of this rectangle.
@param x the new x-coordinate of the anchor point.
@param y the new y-coordinate of the anchor point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23setAnchor(number%2Cnumber) Online Documentation}*/
setAnchor(x: number, y: number): void;
/**
Sets the anchor of this oriented rectangle so that the center of the rectangle coincides with the given coordinate
pair.
@param cx The x coordinate of the center.
@param cy The y coordinate of the center.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23setCenter(number%2Cnumber) Online Documentation}*/
setCenter(cx: number, cy: number): void;
/**
Sets the size of this rectangle.
@param width the new width.
@param height the new height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23setSize(number%2Cnumber) Online Documentation}*/
setSize(width: number, height: number): void;
/**
Sets the components of the up vector to the new values.
@param upX The x component of the normalized up vector.
@param upY The y component of the normalized up vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23setUpVector(number%2Cnumber) Online Documentation}*/
setUpVector(upX: number, upY: number): void;
/**
Creates an immutable {@link IOrientedRectangle} from a given {@link YOrientedRectangle}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link IOrientedRectangle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23toImmutableOrientedRectangle() Online Documentation}*/
toImmutableOrientedRectangle(): IOrientedRectangle;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23toString() Online Documentation}*/
toString(): string;
/**
Gets whether this instance has negative width or height.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23empty Online Documentation}*/
readonly empty: boolean;
/**
Gets or sets the anchor of this oriented rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23anchor Online Documentation}*/
anchor: YPoint | null;
/**
Gets the x-coordinate of this rectangle's anchor point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23anchorX Online Documentation}*/
readonly anchorX: number;
/**
Gets the y-coordinate of this rectangle's anchor point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23anchorY Online Documentation}*/
readonly anchorY: number;
/**
Gets or sets the size of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23size Online Documentation}*/
size: YDimension | null;
/**
Gets the width of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23width Online Documentation}*/
readonly width: number;
/**
Gets the height of this rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23height Online Documentation}*/
readonly height: number;
/**
Gets the x-component of this rectangle's up vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23upX Online Documentation}*/
readonly upX: number;
/**
Gets the y-component of this rectangle's up vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23upY Online Documentation}*/
readonly upY: number;
/**
Gets or sets the angle (measured in radians) of this rectangle.
The angle of an oriented rectangle is the angle between the vector <code>(0, -1)</code> and the rectangle's up vector in counter
clockwise order. An angle of 0 means the up vector points up in direction <code>(0, -1)</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23angle Online Documentation}*/
angle: number;
/**
Gets or sets the current center of this oriented rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23center Online Documentation}*/
center: YPoint | null;
/**
Calculates the paraxial bounding box of this oriented rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YOrientedRectangle%23boundingBox Online Documentation}*/
readonly boundingBox: YRectangle | null;
static $class: Class;
private $_YOrientedRectangle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YOrientedRectangle;
}
export interface YPoint extends YObject, IComparable {
}
/**
This class represents a point in the plane with double coordinates.
This class implements the immutable design pattern.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint Online Documentation}*/
export class YPoint {
/**
Creates a new {@link YPoint} object for a given position.
@param x the x coordinate of the point.
@param y the y coordinate of the point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23YPoint(number%2Cnumber) Online Documentation}*/
constructor(x: number, y: number);
/**
Creates a new {@link YPoint} at location (0,0)
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23YPoint() Online Documentation}*/
constructor();
/**
Adds two points and returns the result.
@param p1 an arbitrary instance of YPoint.
@param p2 an arbitrary instance of YPoint.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23add(YPoint%2CYPoint) Online Documentation}*/
static add(p1: YPoint | null, p2: YPoint | null): YPoint | null;
/**
Comparable implementation.
YPoints are ordered by ascending x-coordinates. If the x-coordinates of two points equal, then these points are ordered
by ascending y-coordinates.
@param o
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23compareTo(Object) Online Documentation}*/
compareTo(o: any | null): number;
/**
Returns the euclidean distance between two points.
@param x1 x-coordinate of first point
@param y1 y-coordinate of first point
@param x2 x-coordinate of second point
@param y2 y-coordinate of second point
@returns the euclidean distance between first and second point
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23distance(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static distance(x1: number, y1: number, x2: number, y2: number): number;
/**
Returns the euclidean distance between two points.
@param p1 an arbitrary point
@param p2 an arbitrary point
@returns the Euclidean distance between p1 and p2.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23distance(YPoint%2CYPoint) Online Documentation}*/
static distance(p1: YPoint | null, p2: YPoint | null): number;
/**
Returns the euclidean distance between this point and a given point.
@param x the x coordinate of an arbitrary point
@param y the y coordinate of an arbitrary point
@returns the Euclidean distance between this point and the point (x,y).
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23distanceTo(number%2Cnumber) Online Documentation}*/
distanceTo(x: number, y: number): number;
/**
Returns the euclidean distance between this point and a given point.
@param p an arbitrary point
@returns the Euclidean distance between this point and p.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23distanceTo(YPoint) Online Documentation}*/
distanceTo(p: YPoint | null): number;
/**
Tests a point to equality to another point.
This test returns true if the <code>o</code> is also an instance of YPoint and has the same coordinates as the instance on which
equals is invoked.
@param o an arbitrary instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23equals(Object) Online Documentation}*/
equals(o: any | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Returns a point that geometrically lies in in the middle of the line formed by the given points.
@param p1 an arbitrary instance of YPoint.
@param p2 an arbitrary instance of YPoint.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23midPoint(YPoint%2CYPoint) Online Documentation}*/
static midPoint(p1: YPoint | null, p2: YPoint | null): YPoint | null;
/**
Returns the point, got by moving this point to another position.
@param x the value which is added on the x-coordinate of the point.
@param y the value which is added on the y-coordinate of the point.
@returns a new instance of YPoint which is the result of the moving operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23moveBy(number%2Cnumber) Online Documentation}*/
moveBy(x: number, y: number): YPoint | null;
/**
Subtracts two points (p1 - p2) and returns the result.
@param p1 an arbitrary instance of YPoint.
@param p2 an arbitrary instance of YPoint.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23subtract(YPoint%2CYPoint) Online Documentation}*/
static subtract(p1: YPoint | null, p2: YPoint | null): YPoint | null;
/**
Returns a copy of the given point with exchanged x- and y-coordinates.
@param p an arbitrary instance of YPoint.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23swap(YPoint) Online Documentation}*/
static swap(p: YPoint | null): YPoint | null;
/**
Creates a {@link Point} from a given {@link YPoint}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Point}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23toPoint() Online Documentation}*/
toPoint(): Point;
/**
Returns the coordinates of the point as string.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23toString() Online Documentation}*/
toString(): string;
/**
A {@link YPoint} constant with coordinates (0,0).
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23ORIGIN Online Documentation}*/
static ORIGIN: YPoint | null;
/**
Gets the x-coordinate of the point object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23x Online Documentation}*/
readonly x: number;
/**
Gets the y-coordinate of the point object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPoint%23y Online Documentation}*/
readonly y: number;
static $class: Class;
private $_YPoint: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YPoint;
}
export interface YPointPath extends YObject {
}
/**
This class represents an ordered list of points in the plane.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath Online Documentation}*/
export class YPointPath {
/**
Creates a new path from an array of points.
@param path
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23YPointPath(Array%3CYPoint%3E) Online Documentation}*/
constructor(path: YPoint[]);
/**
Creates a new path from a list of points.
@param l a list of {@link YPoint} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23YPointPath(IList%3CObject%3E) Online Documentation}*/
constructor(l: IList<any>);
/**
Creates a new empty path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23YPointPath() Online Documentation}*/
constructor();
/**
Calculate the (geometric) length of the path.
The length of the path is the sum of lengths of all line segments making up the path.
@returns the (geometric) length of the path
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23calculateLength() Online Documentation}*/
calculateLength(): number;
/**
Create a point path with reverse ordering of the points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23createReverse() Online Documentation}*/
createReverse(): YPointPath | null;
/**
Get the points in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23cursor() Online Documentation}*/
cursor(): ICursor | null;
/**
Returns a line segment in the path.
@param i
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23getLineSegment(number) Online Documentation}*/
getLineSegment(i: number): LineSegment | null;
/**
Get the number of points in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23length() Online Documentation}*/
length(): number;
/**
Get the points in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23lineSegments() Online Documentation}*/
lineSegments(): ILineSegmentCursor | null;
/**
Get the points in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23points() Online Documentation}*/
points(): IPointCursor | null;
/**
Get the points in the list as array.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23toArray() Online Documentation}*/
toArray(): YPoint[] | null;
/**
Get the points in the path as list.
@returns a list of {@link YPoint} instances.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23toList() Online Documentation}*/
toList(): IList<any> | null;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23toString() Online Documentation}*/
toString(): string;
/**
Defines a path with no points.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23EMPTY_PATH Online Documentation}*/
static EMPTY_PATH: YPointPath | null;
/**
Gets the first point in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23first Online Documentation}*/
readonly first: YPoint | null;
/**
Gets the last point in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23last Online Documentation}*/
readonly last: YPoint | null;
/**
Gets the number of line segments in the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YPointPath%23lineSegmentCount Online Documentation}*/
readonly lineSegmentCount: number;
static $class: Class;
private $_YPointPath: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YPointPath;
}
export interface YRectangle extends YDimension, IPlaneObject {
/**
@inheritdoc*/
compareTo(o: any | null): number;
} /**
This class defines a rectangle and provides utility methods for it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle Online Documentation}*/
export class YRectangle {
/**
Creates a new rectangle with given upper left corner and size.
@param x x-coordinate of upper left corner of the rectangle.
@param y y-coordinate of upper left corner of the rectangle.
@param width width of the rectangle.
@param height height of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23YRectangle(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(x: number, y: number, width: number, height: number);
/**
Creates a new rectangle with given upper left corner and size.
@param pos upper left corner of the rectangle.
@param size size of the rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23YRectangle(YPoint%2CYDimension) Online Documentation}*/
constructor(pos: YPoint | null, size: YDimension | null);
/**
Creates a new rectangle with upper left corner (0,0) and size (0,0).
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23YRectangle() Online Documentation}*/
constructor();
/**
Checks whether or not this <code>YRectangle</code> contains the given rectangle.
@param p
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23contains(YRectangle) Online Documentation}*/
contains(p: YRectangle): boolean;
/**
Checks whether or not this <code>YRectangle</code> contains the given point.
@param p
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23contains(YPoint) Online Documentation}*/
contains(p: YPoint): boolean;
/**
Checks whether or not this <code>YRectangle</code> contains the rectangle defined by the given frame.
@param x
@param y
@param width
@param height
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23contains(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
contains(x: number, y: number, width: number, height: number): boolean;
/**
Checks whether or not this <code>YRectangle</code> contains the given point.
@param x the x-coordinate of the point to check.
@param y the x-coordinate of the point to check.
@returns <code>true</code> if the point lies inside the rectangle; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23contains(number%2Cnumber) Online Documentation}*/
contains(x: number, y: number): boolean;
/**
Determines whether the specified rectangle contains the specified point.
@param rx the x-coordinate of the upper left corner of the rectangle.
@param ry the y-coordinate of the upper left corner of the rectangle.
@param rw the width of the rectangle.
@param rh the height of the rectangle.
@param x the x-coordinate of the point to check.
@param y the x-coordinate of the point to check.
@param [closed=false] if <code>true</code>, all points on the border of the rectangle are considered to be <em>contained</em>.
@returns <code>true</code> if the point lies inside the rectangle; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23contains(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cboolean) Online Documentation}*/
static contains(rx: number, ry: number, rw: number, rh: number, x: number, y: number, closed?: boolean): boolean;
/**
Returns the Euclidean distance to the passed rectangle.
If they overlap, the distance is 0.
@param other the second rectangle.
@returns the distance to the given rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23getEuclideanDistance(YRectangle) Online Documentation}*/
getEuclideanDistance(other: YRectangle | null): number;
/**
Returns the Manhattan distance to the passed rectangle.
If they overlap the distance is 0.
@param other the second rectangle.
@returns the distance to the given rectangle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23getManhattanDistance(YRectangle) Online Documentation}*/
getManhattanDistance(other: YRectangle | null): number;
/**
Returns whether or not the given rectangles intersect.
@param r1
@param r2
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23intersects(YRectangle%2CYRectangle) Online Documentation}*/
static intersects(r1: YRectangle | null, r2: YRectangle | null): boolean;
/**
Creates a {@link Rect} from a given {@link YRectangle}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Rect}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23toRect() Online Documentation}*/
toRect(): Rect;
/**
Returns a string representation of this rectangle
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23toString() Online Documentation}*/
toString(): string;
/**
Gets x-coordinate of upper left corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23x Online Documentation}*/
readonly x: number;
/**
Gets y-coordinate of upper left corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23y Online Documentation}*/
readonly y: number;
/**
Gets coordinates of upper left corner.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23location Online Documentation}*/
readonly location: YPoint | null;
/**
Gets this object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YRectangle%23boundingBox Online Documentation}*/
readonly boundingBox: YRectangle | null;
static $class: Class;
private $_YRectangle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YRectangle;
}
export interface YVector extends YObject {
}
/**
This class represents a vector in the 2-dimensional real vector space.
This vector is an ordered 2 tuple and is defined by two doubles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector Online Documentation}*/
export class YVector {
/**
Creates a new vector, whose direction is given by two points.
The vector is defined by <code>p1</code> - <code>p2</code>.
@param p1 The first point.
@param [p2=null] The second point. If no point is spcified, <code>(0,0)</code> is used instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23YVector(YPoint%2CYPoint) Online Documentation}*/
constructor(p1: YPoint, p2?: YPoint | null);
/**
Creates a new vector which is a copy of another vector.
@param v the vector, whose values are copied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23YVector(YVector) Online Documentation}*/
constructor(v: YVector);
/**
Creates a new vector, whose direction is given by two points.
The vector is defined by <code>(x1 - x2, y1 - y2)</code>.
@param x1 the X-coordinate of the first point.
@param y1 the Y-coordinate of the first point.
@param x2 the X-coordinate of the second point.
@param y2 the Y-coordinate of the second point.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23YVector(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(x1: number, y1: number, x2: number, y2: number);
/**
Creates a new vector with given direction.
@param dx the first coordinate
@param dy the second coordinate
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23YVector(number%2Cnumber) Online Documentation}*/
constructor(dx: number, dy: number);
/**
Adds a vector to this vector.
@param v the vector to add.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23add(YVector) Online Documentation}*/
add(v: YVector | null): void;
/**
Adds the vector to a point and returns the resulting point.
@param p a point.
@param v the vector to add to the point.
@returns p+v
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23add(YPoint%2CYVector) Online Documentation}*/
static add(p: YPoint, v: YVector): YPoint | null;
/**
Adds two vectors and returns the result.
@param v first vector to sum.
@param w second vector to sum.
@returns v+w
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23add(YVector%2CYVector) Online Documentation}*/
static add(v: YVector, w: YVector): YVector | null;
/**
Returns the result of the addition of two angles between <code>0</code> and <code>2*Pi</code>.
The result is calculated modulo <code>2*Pi</code>.
@param a1 a value in <code>[0,2Pi)</code>.
@param a2 a value in <code>[0,2Pi)</code>.
@returns The sum of a1 and a2 modulo <code>2*Pi</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23addAngle(number%2Cnumber) Online Documentation}*/
static addAngle(a1: number, a2: number): number;
/**
Returns the angle (measured in radians) between two vectors in clockwise order (with regards to screen coordinates) from
v1 to v2.
Screen coordinates mean positive x-direction is from left to right and positive y-direction is from top to bottom.
@param v1
@param v2
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23angle(YVector%2CYVector) Online Documentation}*/
static angle(v1: YVector | null, v2: YVector | null): number;
/**
Returns this vector with unit length.
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23getNormal(YVector) Online Documentation}*/
static getNormal(v: YVector | null): YVector | null;
/**
Returns the length of the vector, this is the value of the euclidean norm.
@returns a value > 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23length() Online Documentation}*/
length(): number;
/**
Assigns unit length to the vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23norm() Online Documentation}*/
norm(): void;
/**
Returns the vector which is orthogonal to the given one and has unit length.
@param v a vector.
@returns a vector which is orthogonal to v with unit length.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23orthoNormal(YVector) Online Documentation}*/
static orthoNormal(v: YVector | null): YVector | null;
/**
Returns true if vector v1 is on the right side of v2.
@param v1
@param v2
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23rightOf(YVector%2CYVector) Online Documentation}*/
static rightOf(v1: YVector | null, v2: YVector | null): boolean;
/**
Returns a new <code>YVector</code> instance that is obtained by rotating this vector by the given angle (measured in radians) in
clockwise direction (with regards to screen coordinates).
Screen coordinates mean positive x-direction is from left to right and positive y-direction is from top to bottom.
@param angle the angle of rotation in radians.
@returns the rotated vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23rotate(number) Online Documentation}*/
rotate(angle: number): YVector | null;
/**
Returns the value of the scalar product of two vectors.
@param v1 the first vector.
@param v2 the second vector.
@returns <code>v1.x * v2.x + v1.y * v2.y</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23scalarProduct(YVector%2CYVector) Online Documentation}*/
static scalarProduct(v1: YVector | null, v2: YVector | null): number;
/**
Scales the vector by an factor.
@param factor the scale factor, with which the length is multiplied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23scale(number) Online Documentation}*/
scale(factor: number): void;
/**
Creates a {@link Point} from a given {@link YVector}.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@returns The {@link Point}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23toPoint() Online Documentation}*/
toPoint(): Point;
/**
Returns a string representation of this vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23toString() Online Documentation}*/
toString(): string;
/**
Gets the first coordinate of the vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23x Online Documentation}*/
readonly x: number;
/**
Gets the second coordinate of the vector.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YVector%23y Online Documentation}*/
readonly y: number;
static $class: Class;
private $_YVector: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YVector;
}
export interface Graph extends YObject {
}
/**
This class implements a directed graph structure.
Basically, a directed graph consists of a set of objects called "nodes" (represented by instances of class
{@link YNode}) and a set of node pairs which are called "edges" (represented by instances of class
{@link Edge}).
The <i>directed</i> stems from the fact that all edges in the graph have direction, i.e., they have a distinct source node and a
distinct target node. Using the aforementioned pair notation, an edge would be written as (<i>&lt;source node></i>, <i>&lt;target node></i>).
Class Graph presents a proper data type that provides support for all essential operations like element creation,
removal, access, and iteration.
<b>Important:</b> Class Graph is the single authority for any structural changes to the graph data type. Specifically, this means that
there is no way to create or delete a node or an edge without using an actual Graph instance.
Furthermore, this class is also responsible for providing access to its elements. This is done by means of bidirectional
cursors that present a read-only view on the node set (interface {@link INodeCursor}) and edge set (interface
{@link IEdgeCursor}).
This class provides direct support for the notion of data accessors. It allows to register so-called data providers
(implementations of interface
{@link IDataProvider}) that hold arbitrary data which is associated to its nodes and/or edges.
Also, it serves as a factory to create so-called maps ({@link INodeMap},
{@link IEdgeMap}) that can be utilized to bind arbitrary data to nodes and edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph Online Documentation}*/
export class Graph {
/**
Instantiates a new {@link Graph} object as a partial copy of the given graph.
Only the subgraph induced by the given cursor will be copied to the new Graph instance. If no cursor is specified, the
complete graph is copied. Values bound to the argument graph via node and edge keys are available in the new {@link Graph}
instance with the keys registered with <code>graph</code>. Only references to these values are copied.
@param graph The graph to be (partially) copied.
@param [subNodes=null] A cursor to iterate over the nodes that actually induce the subgraph to be copied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23Graph(Graph%2CICursor) Online Documentation}*/
constructor(graph: Graph, subNodes?: ICursor | null);
/**
Instantiates an empty {@link Graph} object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23Graph() Online Documentation}*/
constructor();
/**
Registers the given data provider using the given look-up key.
If there is already a data provider registered with that key, then it will be overwritten with the new one.
@param providerKey
@param data
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23addDataProvider(Object%2CIDataProvider) Online Documentation}*/
addDataProvider(providerKey: any, data: IDataProvider): void;
/**
Redefines an edge's end points.
Edge <code>e</code> has source node <code>v := sourceReference.source()</code> or <code>v := newSource</code>, if <code>sourceReference == null</code> and target node <code>w := targetReference.target()</code>
or <code>w := newTarget</code>, if <code>targetReference == null</code>.
Edge <code>e</code> is inserted in such a way that an iteration over the edges at <code>v</code> returns <code>e</code>
<ul>
<li>after <code>sourceReference</code>, if <code>sourceD == AFTER</code>
<li>before <code>sourceReference</code>, if <code>sourceD == BEFORE</code>,
</ul>
and an iteration over the edges at <code>w</code> returns <code>e</code>
<ul>
<li>after <code>targetReference</code>, if <code>targetD == AFTER</code>
<li>before <code>targetReference</code>, if <code>targetD == BEFORE</code>.
</ul>
@param e The edge to be changed.
@param newSource The new source node.
@param sourceReference Reference edge for insertion at the new source node.
@param sourceD One of the object insertion specifiers {@link GraphElementInsertion#BEFORE} or
{@link GraphElementInsertion#AFTER}.
@param newTarget The new target node.
@param targetReference Reference edge for insertion at the new target node.
@param targetD One of the object insertion specifiers {@link GraphElementInsertion#BEFORE} or
{@link GraphElementInsertion#AFTER}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23changeEdge(Edge%2CYNode%2CEdge%2CGraphElementInsertion%2CYNode%2CEdge%2CGraphElementInsertion) Online Documentation}*/
changeEdge(e: Edge, newSource: YNode, sourceReference: Edge, sourceD: GraphElementInsertion | GraphElementInsertionStringValues, newTarget: YNode, targetReference: Edge, targetD: GraphElementInsertion | GraphElementInsertionStringValues): void;
/**
Redefines an edge's end points.
Edge <code>e</code> has source node <code>v := e1.source()</code> and target node <code>w := e2.target()</code>.
Edge <code>e</code> is inserted in such a way that an iteration over the edges at <code>v</code> returns <code>e</code>
<ul>
<li>after <code>e1</code>, if <code>d1 == AFTER</code>
<li>before <code>e1</code>, if <code>d1 == BEFORE</code>,
</ul>
and an iteration over the edges at <code>w</code> returns <code>e</code>
<ul>
<li>after <code>e2</code>, if <code>d2 == AFTER</code>
<li>before <code>e2</code>, if <code>d2 == BEFORE</code>.
</ul>
@param e The edge to be changed.
@param e1 Reference edge for insertion at a new source node.
@param e2 Reference edge for insertion at a new target node.
@param d1 One of the object insertion specifiers {@link GraphElementInsertion#BEFORE} or
{@link GraphElementInsertion#AFTER}.
@param d2 One of the object insertion specifiers {@link GraphElementInsertion#BEFORE} or
{@link GraphElementInsertion#AFTER}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23changeEdge(Edge%2CEdge%2CEdge%2CGraphElementInsertion%2CGraphElementInsertion) Online Documentation}*/
changeEdge(e: Edge, e1: Edge, e2: Edge, d1: GraphElementInsertion | GraphElementInsertionStringValues, d2: GraphElementInsertion | GraphElementInsertionStringValues): void;
/**
Redefines an edge's end points.
The edge is appended to the lists of incoming and outgoing edges at the given source node and target node,
respectively.
@param e The edge to be changed.
@param newSource The new source node of the given edge.
@param newTarget The new target node of the given edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23changeEdge(Edge%2CYNode%2CYNode) Online Documentation}*/
changeEdge(e: Edge, newSource: YNode, newTarget: YNode): void;
/**
Removes all nodes and edges from this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23clear() Online Documentation}*/
clear(): void;
/**
Whether or not this graph contains the given edge.
@param e
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23contains(Edge) Online Documentation}*/
contains(e: Edge): boolean;
/**
Whether or not this graph contains the given node.
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23contains(YNode) Online Documentation}*/
contains(v: YNode): boolean;
/**
Returns whether or not this graph contains an edge that connects the given nodes.
@param source The source node.
@param target The target node.
@see YNode#getEdgeTo
@see YNode#getEdgeFrom
@see YNode#getEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23containsEdge(YNode%2CYNode) Online Documentation}*/
containsEdge(source: YNode, target: YNode): boolean;
/**
Creates a copy of this graph.
Invokes {@link Graph#Graph}.
@returns The newly created Graph object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23createCopy() Online Documentation}*/
createCopy(): Graph;
/**
Creates a new edge in this graph to be ordered before or after a given edge.
The new edge <code>e</code> has source node <code>v</code> and target node <code>w</code>, i.e., would be written as edge <code>e = (v, w)</code>.
Edge <code>e</code> is inserted in such a way that an iteration over the edges at node <code>v</code> returns <code>e</code>
<ul>
<li>after <code>e1</code>, if <code>d1 == AFTER</code>
<li>before <code>e1</code>, if <code>d1 == BEFORE</code>,
</ul>
and an iteration over the edges at <code>w</code> returns <code>e</code>
<ul>
<li>after <code>e2</code>, if <code>d2 == AFTER</code>
<li>before <code>e2</code>, if <code>d2 == BEFORE</code>.
</ul>
@param v The source node of the edge.
@param e1 An edge with source node <code>v</code>.
@param w The target node of the edge.
@param e2 An edge with target node <code>w</code>.
@param d1 One of the object insertion specifiers {@link GraphElementInsertion#BEFORE} or
{@link GraphElementInsertion#AFTER}.
@param d2 One of the object insertion specifiers {@link GraphElementInsertion#BEFORE} or
{@link GraphElementInsertion#AFTER}.
@returns The newly created Edge object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23createEdge(YNode%2CEdge%2CYNode%2CEdge%2CGraphElementInsertion%2CGraphElementInsertion) Online Documentation}*/
createEdge(v: YNode, e1: Edge | null, w: YNode, e2: Edge | null, d1: GraphElementInsertion | GraphElementInsertionStringValues, d2: GraphElementInsertion | GraphElementInsertionStringValues): Edge;
/**
Creates a new edge in this graph.
The new edge has source node <code>v</code> and target node <code>w</code>, i.e., would be written as edge <code>e = (v, w)</code>.
The edge is appended to the lists of incoming and outgoing edges at the source node and target node, respectively.
@param v The source node of the edge.
@param w The target node of the edge.
@returns The newly created Edge object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23createEdge(YNode%2CYNode) Online Documentation}*/
createEdge(v: YNode, w: YNode): Edge;
/**
Returns a newly created edge map that is valid for the edges in this graph.
The implementation returned by this method can be used for any edge that is part of this Graph instance at any point of
time, i.e., it is safe to modify the graph structure (add and remove nodes and edges) freely.
The implementation returned uses <code>O(m)</code> memory at all times and provides true <code>O(1)</code> read and write access for each edge.
In order to release the resources held by this map, {@link Graph#disposeEdgeMap} has to be called.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23createEdgeMap() Online Documentation}*/
createEdgeMap(): IEdgeMap;
/**
Creates an empty base object of the same type as this graph.
Subclasses should override this method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23createGraph() Online Documentation}*/
createGraph(): Graph;
/**
Creates a new node in this graph.
@returns The newly created Node object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23createNode() Online Documentation}*/
createNode(): YNode;
/**
Returns a newly created node map that is valid for the nodes in this graph.
The implementation returned by this method can be used for any node that is part of this Graph instance at any point of
time, i.e., it is safe to modify the graph structure (add and remove nodes and edges) freely.
The implementation returned uses <code>O(n)</code> memory at all times and provides true <code>O(1)</code> read and write access for each node.
In order to release the resources held by this map, {@link Graph#disposeNodeMap} has to be called.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23createNodeMap() Online Documentation}*/
createNodeMap(): INodeMap;
/**
Informs the graph that the given edge map is no longer needed.
This method is used for EdgeMap implementations that have been obtained using the {@link Graph#createEdgeMap}
factory method.
Calling this method will destroy the edge map and associated resources can be freed. It is strongly recommended to
dispose of all edge maps that are not needed anymore using this method.
@param map
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23disposeEdgeMap(IEdgeMap) Online Documentation}*/
disposeEdgeMap(map: IEdgeMap | null): void;
/**
Informs the graph that the given node map is no longer needed.
This method is used for NodeMap implementations that have been obtained using the {@link Graph#createNodeMap}
factory method.
Calling this method will destroy the node map and associated resources can be freed. It is strongly recommended to
dispose of all node maps that are not needed anymore using this method.
@param map
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23disposeNodeMap(INodeMap) Online Documentation}*/
disposeNodeMap(map: INodeMap | null): void;
/**
Low-level iteration support for adjacent edges.
@param v
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23firstOutEdge(YNode) Online Documentation}*/
static firstOutEdge(v: YNode): Edge | null;
/**
Returns the data provider that is registered with the graph using the given look-up key.
The look-up domain of a returned data provider normally consists of either the nodes of the graph, or its edges, or
both.
@param providerKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23getDataProvider(Object) Online Documentation}*/
getDataProvider(providerKey: any): IDataProvider | null;
/**
Returns an array containing all edges of this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23getEdgeArray() Online Documentation}*/
getEdgeArray(): Edge[];
/**
Provides access to the edges of the graph.
@returns An EdgeCursor to iterate over the edges in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23getEdgeCursor() Online Documentation}*/
getEdgeCursor(): IEdgeCursor;
/**
Returns an array containing all nodes of this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23getNodeArray() Online Documentation}*/
getNodeArray(): YNode[];
/**
Provides access to the nodes of the graph.
@returns A NodeCursor to iterate over the nodes in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23getNodeCursor() Online Documentation}*/
getNodeCursor(): INodeCursor;
/**
Hides the given node from this graph.
Hiding a node means to (temporarily) remove the node from the graph.
Generally, hiding should <i>only</i> be used in the sense of temporarily removing an object that will be reinserted shortly after.
To reinsert a hidden node use {@link Graph#unhide}.
@param v
@see Graph#hide
@see Graph#unhide
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23hide(YNode) Online Documentation}*/
hide(v: YNode): void;
/**
Hides the given edge from this graph.
Hiding an edge means to (temporarily) remove the edge from the graph.
Generally, hiding should <i>only</i> be used in the sense of temporarily removing an object that will be reinserted shortly after.
To reinsert a hidden edge use {@link Graph#unhide}.
@param e
@see Graph#hide
@see Graph#unhide
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23hide(Edge) Online Documentation}*/
hide(e: Edge): void;
/**
Moves the given edge to the first position within the sequence of edges in this graph.
@param e
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23moveToFirst(Edge) Online Documentation}*/
moveToFirst(e: Edge): void;
/**
Moves the given node to the first position within the sequence of nodes in this graph.
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23moveToFirst(YNode) Online Documentation}*/
moveToFirst(v: YNode): void;
/**
Moves the given edge to the last position within the sequence of edges in this graph.
@param e
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23moveToLast(Edge) Online Documentation}*/
moveToLast(e: Edge): void;
/**
Moves the given node to the last position within the sequence of nodes in this graph.
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23moveToLast(YNode) Online Documentation}*/
moveToLast(v: YNode): void;
/**
For internal debugging purposes only.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23printNodeSlotSize() Online Documentation}*/
printNodeSlotSize(): void;
/**
Reinserts a formerly removed edge into this graph.
The reinserted edge is appended to the sequence of edges in this graph, i.e., normally, its new position does not match
the position before its removal. The same holds for the edge's positions in the list of incoming and outgoing edges at
its source node and target node, respectively.
Note that reinserting an edge whose source/target is not in the graph (e.g., because it's currently hidden/removed)
causes an exception. Hence, in such cases, you first have to unhide/reinsert the corresponding endpoints.
@param e The edge to be reinserted.
@see Graph#removeEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23reInsertEdge(Edge) Online Documentation}*/
reInsertEdge(e: Edge): void;
/**
Reinserts a formerly removed node into this graph.
The reinserted node is appended to the sequence of nodes in this graph, i.e., normally, its new position does not match
the position before its removal.
@param v The node to be reinserted.
@see Graph#removeNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23reInsertNode(YNode) Online Documentation}*/
reInsertNode(v: YNode): void;
/**
Removes the data provider that is registered using the given look-up key.
@param providerKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23removeDataProvider(Object) Online Documentation}*/
removeDataProvider(providerKey: any): void;
/**
Removes the given edge from this graph.
The edge will be deselected before it gets removed.
@param e The edge to be removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23removeEdge(Edge) Online Documentation}*/
removeEdge(e: Edge): void;
/**
Removes the given node from this graph.
All edges connecting to the given node are removed as well (preceding the actual node removal).
The node will be deselected before it gets removed.
@param v The node to be removed from this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23removeNode(YNode) Online Documentation}*/
removeNode(v: YNode): void;
/**
Reverses the given edge.
This operation exchanges source and target node of the edge.
@param e
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23reverseEdge(Edge) Online Documentation}*/
reverseEdge(e: Edge): void;
/**
Sorts incoming and outgoing edges at each node of the graph.
If a given comparator is <code>null</code>, then the corresponding edges (i.e., incoming/outgoing) will not be sorted. This sorts the
order of the edges as returned by {@link YNode#getOutEdgeCursor} and {@link YNode#getInEdgeCursor}
respectively.
@param inComparer The comparator used for the incoming edges at each node.
@param outComparer The comparator used for the outgoing edges at each node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23sortEdges(IComparer%3CObject%3E%2CIComparer%3CObject%3E) Online Documentation}*/
sortEdges(inComparer: IComparer<any> | null, outComparer: IComparer<any> | null): void;
/**
Sorts the internally held list of edges.
If the given comparator is <code>null</code>, then the edges will not be sorted. This list determines the order of the edges as
returned by {@link Graph#getEdgeCursor}.
@param comparer The comparator used for the edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23sortEdges(IComparer%3CObject%3E) Online Documentation}*/
sortEdges(comparer: IComparer<any> | null): void;
/**
Sorts the internally held list of nodes.
If the given comparator is <code>null</code>, then the nodes will not be sorted. This list determines the order of the nodes as
returned by {@link Graph#getNodeCursor}.
@param comparer The comparator used for the nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23sortNodes(IComparer%3CObject%3E) Online Documentation}*/
sortNodes(comparer: IComparer<any> | null): void;
/**
Returns a String representation of this graph.
The result contains the String representations of all nodes followed by the String representations of all edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23toString() Online Documentation}*/
toString(): string;
/**
Unhides the given node in this graph.
Unhiding a node means to reinsert a node that was formerly hidden from this graph by a call to
{@link Graph#hide}.
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23unhide(YNode) Online Documentation}*/
unhide(v: YNode): void;
/**
Unhides the given edge in this graph.
Unhiding an edge means to reinsert an edge that was formerly hidden from this graph by a call to
{@link Graph#hide}.
Note that unhiding an edge whose source/target is not in the graph (e.g., because it's currently hidden/removed) causes
an exception. Hence, in such cases, you first have to unhide/reinsert the corresponding endpoints.
@param e
@see Graph#hide
@see Graph#unhide
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23unhide(Edge) Online Documentation}*/
unhide(e: Edge): void;
/**
Gets the number of nodes in this graph.
Same as {@link Graph#nodeCount}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23n Online Documentation}*/
readonly n: number;
/**
Gets the number of nodes in this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23nodeCount Online Documentation}*/
readonly nodeCount: number;
/**
Gets the number of edges in this graph.
Same as {@link Graph#edgeCount}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23e Online Documentation}*/
readonly e: number;
/**
Gets the number of edges in this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23edgeCount Online Documentation}*/
readonly edgeCount: number;
/**
Gets <code>true</code> if this graph contains no nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23empty Online Documentation}*/
readonly empty: boolean;
/**
Gets the first node in this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23firstNode Online Documentation}*/
readonly firstNode: YNode | null;
/**
Gets the first edge in this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23firstEdge Online Documentation}*/
readonly firstEdge: Edge | null;
/**
Gets the last node in this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23lastNode Online Documentation}*/
readonly lastNode: YNode | null;
/**
Gets the last edge in this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23lastEdge Online Documentation}*/
readonly lastEdge: Edge | null;
/**
Gets all node maps that have been created by this graph but have not yet been disposed.
@see Graph#createNodeMap
@see Graph#disposeNodeMap
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23registeredNodeMaps Online Documentation}*/
readonly registeredNodeMaps: INodeMap[];
/**
Gets all edge maps that have been created by this graph but have not yet been disposed.
@see Graph#createEdgeMap
@see Graph#disposeEdgeMap
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23registeredEdgeMaps Online Documentation}*/
readonly registeredEdgeMaps: IEdgeMap[];
/**
Gets an array of all data provider look-up keys that are registered with this graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23dataProviderKeys Online Documentation}*/
readonly dataProviderKeys: any[];
/**
Yields a dynamic {@link IEnumerable} for
{@link YNode}s that can be used to iterate over the nodes that are contained in this instance.
This is a live enumerable and will thus reflect the current state of the graph. Note that changes to the graph structure
during the traversal should be carried out with great care.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23nodes Online Documentation}*/
readonly nodes: IEnumerable<YNode> | null;
/**
Yields a dynamic {@link IEnumerable} for
{@link Edge}s that can be used to iterate over the edges that are contained in this instance.
This is a live enumerable and will thus reflect the current state of the graph. Note that changes to the graph structure
during the traversal should be carried out with great care.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Graph%23edges Online Documentation}*/
readonly edges: IEnumerable<Edge> | null;
static $class: Class;
private $_Graph: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Graph;
}
export interface GraphChecker extends YObject {
}
/**
This class provides methods that check structural properties of a given graph.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To analyze structural properties of {@link IGraph}
instances use {@link GraphStructureAnalyzer} instead.
<b>Definitions</b>
<ul>
<li><b>Cycle:</b> An edge path with vertices <code>v0, v1, v2, ... , vk</code> forms a cycle if <code>v0 = vk</code> and consists of at least one edge.
<li><b>Acyclic graph:</b> A graph that contains no directed cycle.
<li><b>Cyclic graph:</b> A graph that contains a directed cycle.
<li><b>Connected graph:</b> A graph in which there exists an undirected path of edges between every pair of nodes.
<li><b>Strongly connected graph:</b> A graph in which there exists a directed path between each pair of nodes.
<li><b>Biconnected graph:</b> A graph that has no cut vertex or articulation point (i.e., a node whose removal disconnects the graph).
<li><b>Bipartite graph:</b> A graph whose nodes can be partitioned into two sets such that each edge connects two nodes of different sets.
<li><b>Tree graph:</b> An acyclic graph, in which any pair of vertices is connected through a path. If one vertex of a tree is distinguished
from the other vertices, then the tree is called <b>rooted tree</b>.
<li><b>N-ary tree graph:</b> A directed rooted tree where each node has a maximum of <code>n</code> children.
<li><b>Forest graph:</b> A graph whose connected components are trees.
<li><b>Simple graph:</b> A graph that contains no self-loops and parallel edges.
<li><b>Planar graph:</b> A graph that can be drawn on the plane without edge crossings.
<li><b>Average degree</b>: The number of edges in comparison to the number of nodes.
<li><b>Average weighted degree</b>: The sum of the edge weights in comparison to the number of nodes.
<li><b>Diameter</b>:The maximum eccentricity of any vertex in the graph i.e., the length of the longest shortest path between any two
vertices.
<li><b>Density</b>: The ratio of edges of the graph to the maximum possible number of edges.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker Online Documentation}*/
export class GraphChecker {
/**
Computes the average degree of a given graph.
The average degree measures the number of edges in comparison to the number of nodes and is defined as:
<ul>
<li><code>graph.E() / graph.N()</code> for directed graphs
<li><code>2 * graph.E() / graph.N()</code> for undirected graphs
</ul>
<h2>Note</h2>
Parallel edges and self-loops are both included in the calculation.
@param graph the input graph
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@returns the average degree of the given graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23getAverageDegree(Graph%2Cboolean) Online Documentation}*/
static getAverageDegree(graph: Graph | null, directed: boolean): number;
/**
Computes the average weighted degree of a given graph.
The average weighted degree measures the sum of the edge weights in comparison to the number of nodes and is defined
as:
<ul>
<li><code>edge_weight_sum / graph.N()</code> for directed graphs
<li><code>2 * edge_weight_sum / graph.N()</code> for undirected graphs
</ul>
<h2>Note</h2>
Parallel edges and self-loops are both included in the calculation.
@param graph the input graph
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param edgeWeight {@link IDataProvider} that provides the weights of the edges
@returns the average weighted degree of the given graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23getAverageWeightedDegree(Graph%2Cboolean%2CIDataProvider) Online Documentation}*/
static getAverageWeightedDegree(graph: Graph | null, directed: boolean, edgeWeight: IDataProvider | null): number;
/**
Computes the density of a given simple graph.
The density is the ratio of edges of the graph to the maximum possible number of edges and equals to:
<ul>
<li><code>graph.E() / (graph.N() * (graph.N() - 1))</code> for directed simple graphs
<li><code>2 * graph.E() / (graph.N() * (graph.N() - 1))</code> for undirected simple graphs
</ul>
<h2>Note</h2>
This algorithm ignores self-loops and parallel edges.
@param graph the input graph
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@returns the density of the given graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23getDensity(Graph%2Cboolean) Online Documentation}*/
static getDensity(graph: Graph | null, directed: boolean): number;
/**
Computes the diameter of a given graph.
The diameter is the maximum eccentricity of any vertex in the graph and equals to the length of the longest of the
shortest path between any two vertices.
<h2>Note</h2>
Parallel edges are also included in the calculation.
<h2>Note</h2>
For the calculation, an algorithm that solved the all-pairs shortest path problem is used. Depending on the structure of
the given graph and the values of the given edge costs, the algorithm internally delegates its work to the algorithm
with the theoretically best running time. Please note that theory does not necessarily reflect practice.
@param graph the input graph
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param edgeCost {@link IDataProvider} that provides the costs of the edges
@returns the diameter of the given graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23getDiameter(Graph%2Cboolean%2CIDataProvider) Online Documentation}*/
static getDiameter(graph: Graph | null, directed: boolean, edgeCost: IDataProvider | null): number;
/**
Checks whether or not the given directed graph is acyclic.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is acyclic use
{@link GraphStructureAnalyzer#isAcyclic} instead.
A graph is called <i>acyclic</i> if it contains no directed cycle.
<h2>Note</h2>
<code>isAcyclic(graph) &lt;=> !isCyclic(graph)</code>.
@param graph the given graph
@returns <code>true</code> if the graph is acyclic, <code>false</code>, otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isAcyclic(Graph) Online Documentation}*/
static isAcyclic(graph: Graph | null): boolean;
/**
Checks whether or not the given undirected graph is biconnected.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is biconnected
use {@link GraphStructureAnalyzer#isBiconnected} instead.
A graph is called <i>biconnected</i> if it has no cut vertex or articulation point, i.e., no node whose removal disconnects the graph.
@param graph the given undirected graph
@returns <code>true</code> if the graph is biconnected, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isBiconnected(Graph) Online Documentation}*/
static isBiconnected(graph: Graph | null): boolean;
/**
Checks whether or not the given undirected graph is bipartite.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is bipartite
use {@link GraphStructureAnalyzer#isAcyclic} instead.
A graph is called <i>bipartite</i> if its nodes can be partitioned into two sets such that each edge connects two nodes of different
sets.
@param graph the given undirected graph
@returns <code>true</code> if the graph is bipartite, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isBipartite(Graph) Online Documentation}*/
static isBipartite(graph: Graph | null): boolean;
/**
Checks whether or not the given graph is connected.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is connected
use {@link GraphStructureAnalyzer#isConnected} instead.
A graph is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
@param graph the given graph
@returns <code>true</code> if the graph is connected, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isConnected(Graph) Online Documentation}*/
static isConnected(graph: Graph | null): boolean;
/**
Checks whether or not the given directed graph is cyclic.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is cyclic use
{@link GraphStructureAnalyzer#isCyclic} instead.
A graph is called <i>cyclic</i> if it contains a directed cycle.
<h2>Note</h2>
<code>isCyclic(graph) &lt;=> !isAcyclic(graph)</code>.
@param graph the given graph
@returns <code>true</code> if the graph is cyclic, <code>false</code>, otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isCyclic(Graph) Online Documentation}*/
static isCyclic(graph: Graph | null): boolean;
/**
Checks whether the given graph is a forest.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is a forest
use {@link GraphStructureAnalyzer#isForest} instead.
A graph is a <i>forest</i> if its connected components are trees.
@param graph the given graph
@returns <code>true</code> if the graph is a forest, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isForest(Graph) Online Documentation}*/
static isForest(graph: Graph | null): boolean;
/**
Checks whether or not the given undirected graph contains no multiple edges.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} has multiple
edges use {@link GraphStructureAnalyzer#hasMultipleEdges} instead.
The method returns <code>true</code> if the graph contains no two distinct edges <code>e1, e2</code> that connect the same pairs of nodes in
either direction.
<h2>Note</h2>
<code>isMultipleEdgeFree(graph) => isSimple(graph)</code>.
@param graph the given undirected graph
@returns <code>true</code> if the graph contains no multiple edges, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isMultipleEdgeFree(Graph) Online Documentation}*/
static isMultipleEdgeFree(graph: Graph | null): boolean;
/**
Checks whether or not the given graph is a directed rooted tree where each node has a maximum of <code>n</code> children.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is an <i>n</i>-ary
tree use {@link GraphStructureAnalyzer#isNaryTree} instead.
@param graph the given graph
@param n
@returns <code>true</code> if the graph is a directed rooted tree where each node has at most <code>n</code> children, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isNaryTree(Graph%2Cnumber) Online Documentation}*/
static isNaryTree(graph: Graph | null, n: number): boolean;
/**
Checks whether or not the given directed graph is a directed rooted tree.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is a directed
tree use {@link GraphStructureAnalyzer#isTree} instead.
<h2>Note</h2>
<code>isRootedTree(graph) => isTree(graph)</code>
@param graph the given graph
@returns <code>true</code> if the graph is a directed rooted tree, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isRootedTree(Graph) Online Documentation}*/
static isRootedTree(graph: Graph | null): boolean;
/**
Checks whether or not the given graph contains no self-loops.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} has self-loops
use {@link GraphStructureAnalyzer#hasSelfLoops} instead.
<h2>Note</h2>
<code>!isSelfLoopFree(graph) => !isAcyclic(graph)</code>
@param graph the given graph
@returns <code>true</code> if the graph contains no self-loops, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isSelfLoopFree(Graph) Online Documentation}*/
static isSelfLoopFree(graph: Graph | null): boolean;
/**
Checks whether or not the given directed graph is simple.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is simple use
{@link GraphStructureAnalyzer#isAcyclic} instead.
A graph is called <i>simple</i> if it contains no two distinct edges <code>e1, e2</code> where
<code>e1.source() == e2.source() &amp;&amp; e1.target() == e2.target()</code>.
<h2>Note</h2>
<code>isMultipleEdgeFree(graph) => isSimple(graph)</code>.
@param graph the given directed graph
@returns <code>true</code> if the graph is simple, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isSimple(Graph) Online Documentation}*/
static isSimple(graph: Graph | null): boolean;
/**
Checks whether or not the given directed graph is strongly connected.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is strongly
connected use {@link GraphStructureAnalyzer#isStronglyConnected} instead.
A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
@param graph the given directed graph
@returns <code>true</code> if the graph is strongly connected, <code>false</code>, otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isStronglyConnected(Graph) Online Documentation}*/
static isStronglyConnected(graph: Graph | null): boolean;
/**
Checks whether or not the given graph is an undirected tree.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is an
undirected tree use {@link GraphStructureAnalyzer#isTree} instead.
<h2>Note</h2>
<code>isRootedTree(graph) => isTree(graph)</code>
@param graph the given graph
@returns <code>true</code> if the graph is an undirected tree, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphChecker%23isTree(Graph) Online Documentation}*/
static isTree(graph: Graph | null): boolean;
static $class: Class;
private $_GraphChecker: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphChecker;
}
export interface GraphConnectivity extends YObject {
}
/**
This class provides algorithms for determining certain connectivity components within a graph.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To determine connectivity of {@link IGraph}
instances use one of the following classes instead:
<ul>
<li>{@link ConnectedComponents}
<li>{@link StronglyConnectedComponents}
<li>{@link BiconnectedComponents}
<li>{@link Neighborhood}
<li>{@link Reachability}
</ul>
It also provides convenience methods for working with these components.
<b>Definitions</b>
<ul>
<li><b>Connected graph:</b> A graph is called connected if there exists an undirected path of edges between every pair of nodes.
<li><b>Strongly connected graph:</b> A graph is called strongly connected if there exists a directed path between each pair of nodes.
<li><b>Biconnected graph:</b> A graph is called biconnected if it has no cut vertex or articulation point (i.e., a node whose removal disconnects the
remaining graph).
<li><b>KCore:</b> The <code>k-cores</code> of a graph are the maximal connected subgraphs in which all nodes have degree greater or equal to <code>k</code>.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity Online Documentation}*/
export class GraphConnectivity {
/**
Calculates the biconnected components and the articulation points of a given undirected graph and returns the number of
biconnected components.
<b>Note:</b> This method works with instances of {@link Graph}. To determine biconnected components and articulation
points in an {@link IGraph} use {@link BiconnectedComponents} instead.
Articulation points are returned in the form of a {@link INodeMap} that returns for each node a boolean value
indicating whether or not it is an articulation point.
<h2>Note</h2>
Self-loops do not belong to any biconnected component. Therefore, the component index for self-loops is always <code>-1</code>.
@param graph the input graph
@param compNum the {@link IEdgeMap} that will be filled during the execution and returns the zero-based index of the
biconnected component to which each edge belongs or <code>-1</code> for self-loops
@param [aPoint=null] the {@link INodeMap} that will be filled during the execution and returns a boolean value indicating whether
or not a given node is an articulation point
@returns the number of biconnected components
@see GraphConnectivity#biconnectedComponents
@see GraphConnectivity#biconnectedComponents
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23biconnectedComponents(Graph%2CIEdgeMap%2CINodeMap) Online Documentation}*/
static biconnectedComponents(graph: Graph | null, compNum: IEdgeMap | null, aPoint?: INodeMap | null): number;
/**
Calculates the biconnected components of a given undirected graph.
<b>Note:</b> This method works with instances of {@link Graph}. To determine biconnected components and articulation
points in an {@link IGraph} use {@link BiconnectedComponents} instead.
The result is returned as an array of
{@link EdgeList}s each containing all edges that belong to the same biconnected component.
<h2>Note</h2>
Self-loops do not belong to any biconnected component. Therefore, no self-loops are included in the returned
{@link EdgeList}s.
@param graph the input graph
@returns an array of {@link EdgeList}s each containing all edges that belong to the same biconnected component
@see GraphConnectivity#biconnectedComponents
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23biconnectedComponents(Graph) Online Documentation}*/
static biconnectedComponents(graph: Graph | null): EdgeList[] | null;
/**
Calculates the connected components of a given graph and returns their number.
<b>Note:</b> This method works with instances of {@link Graph}. To determine connected components in an {@link IGraph}
use {@link ConnectedComponents} instead.
A graph <code>G</code> is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
The connected components of a graph are the maximal connected subgraphs of which the graph consists.
@param compNum the {@link INodeMap} that will be filled during the execution and returns the zero-based index of the
connected component to which each node belongs
@param graph
@returns the number of connected components of the given graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23connectedComponents(Graph%2CINodeMap) Online Documentation}*/
static connectedComponents(graph: Graph | null, compNum: INodeMap | null): number;
/**
Calculates the connected components of a given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To determine connected components in an {@link IGraph}
use {@link ConnectedComponents} instead.
A graph <code>G</code> is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
The connected components of a graph are the maximal connected subgraphs of which the graph consists.
@param graph the input graph
@returns an array of {@link YNodeList}s each of which contains the nodes that belong to the same connected component
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23connectedComponents(Graph) Online Documentation}*/
static connectedComponents(graph: Graph | null): YNodeList[] | null;
/**
Determines the direct or indirect neighbors of a given set of nodes.
<b>Note:</b> This method works with instances of {@link Graph}. To determine direct and indirect neighbors of nodes in an
{@link IGraph} use {@link IGraph#neighbors} (direct neighbors only) or {@link Neighborhood} instead.
<ul>
<li>A <i>direct</i> neighbor of a node is directly connected by an edge to that node.
<li>An <i>indirect</i> neighbor of a node is directly connected to another direct or indirect neighbor of a node.
</ul>
The order of the returned nodes is determined by a breadth first search. No start node will be part of the resulting
set.
To obtain the result, an integer value should be given as input that limits the distance between a start node and a
returned node. For all returned nodes there must be a path to a start node that has a length equal to or smaller than
this distance.
Setting the maximum distance to <code>1</code> will only yield the direct neighbors of all start nodes. On the other hand, setting
the maximum distance to <code>graph.N()</code> or larger will yield all neighbors of all start nodes.
@param graph
@param startNodes
@param maxDistance
@returns a {@link YNodeList} that contains all direct and indirect neighbors of a node
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23getNeighbors(Graph%2CYNodeList%2Cnumber) Online Documentation}*/
static getNeighbors(graph: Graph | null, startNodes: YNodeList | null, maxDistance: number): YNodeList | null;
/**
Determines the direct or indirect predecessors of a given list of nodes.
<b>Note:</b> This method works with instances of {@link Graph}. To determine direct and indirect predecessors of nodes in
an {@link IGraph} use {@link IGraph#predecessors} (direct predecessors only) or {@link Neighborhood}
instead.
<ul>
<li>A <i>direct</i> predecessor of a node is the source node of an incoming edge connected to a node.
<li>An <i>indirect</i> predecessor of a node is a direct predecessor to another predecessor of a node.
</ul>
The order of the returned nodes is determined by a breadth first search. No start node will be part of the resulting
set.
To obtain the result, an integer value should be given as input that limits the distance between a start node and a
returned node. For all returned nodes there must be a path to a start node that has a length equal to or smaller than
this distance.
Setting the maximum distance to <code>1</code> will only yield the direct predecessors of all start nodes. On the other hand, setting
the maximum distance to <code>graph.N()</code> or larger will yield all predecessors of all start nodes.
@param graph the given graph
@param startNodes a {@link YNodeList} containing the nodes from which the search starts
@param maxDistance an integer value that limits the distance between a start node and a returned node
@returns a {@link YNodeList} that contains all direct and indirect predecessors of a node
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23getPredecessors(Graph%2CYNodeList%2Cnumber) Online Documentation}*/
static getPredecessors(graph: Graph | null, startNodes: YNodeList | null, maxDistance: number): YNodeList | null;
/**
Determines the direct or indirect successors of a given list of nodes.
<b>Note:</b> This method works with instances of {@link Graph}. To determine direct and indirect successors of nodes in an
{@link IGraph} use {@link IGraph#successors} (direct successors only) or {@link Neighborhood}
instead.
<ul>
<li>A <i>direct</i> successor of a node is the target node of an outgoing edge connected to a node.
<li>An <i>indirect</i> successor of a node is a direct successor to another successor of a node.
</ul>
The order of the returned nodes is determined by a breadth first search. No start node will be part of the resulting
set.
To obtain the result, an integer value should be given as input that limits the distance between a start node and a
returned node. For all returned nodes there must be a path to a start node that has a length equal to or smaller than
this distance.
Setting the maximum distance to <code>1</code> will only yield the direct successors of all start nodes. On the other hand, setting
the maximum distance to <code>graph.N()</code> or larger will yield all successors of all start nodes.
@param graph the given graph
@param startNodes a {@link YNodeList} containing the nodes from which the search starts
@param maxDistance an integer value that limits the distance between a start node and a returned node
@returns a {@link YNodeList} that contains all direct and indirect successors of a node
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23getSuccessors(Graph%2CYNodeList%2Cnumber) Online Documentation}*/
static getSuccessors(graph: Graph | null, startNodes: YNodeList | null, maxDistance: number): YNodeList | null;
/**
Checks whether or not the given undirected graph is biconnected.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is biconnected
use {@link GraphStructureAnalyzer#isBiconnected} instead.
A graph is called <i>biconnected</i> if it has no cut vertex or articulation point, i.e., no node whose removal disconnects the graph.
@param graph the given undirected graph
@returns <code>true</code> if the graph is biconnected, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23isBiconnected(Graph) Online Documentation}*/
static isBiconnected(graph: Graph | null): boolean;
/**
Checks whether or not the given graph is connected.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is connected
use {@link GraphStructureAnalyzer#isConnected} instead.
A graph is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
@param graph the given graph
@returns <code>true</code> if the graph is connected, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23isConnected(Graph) Online Documentation}*/
static isConnected(graph: Graph | null): boolean;
/**
Checks whether or not the given directed graph is strongly connected.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is strongly
connected use {@link GraphStructureAnalyzer#isStronglyConnected} instead.
A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
@param graph the given directed graph
@returns <code>true</code> if the graph is strongly connected, <code>false</code>, otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23isStronglyConnected(Graph) Online Documentation}*/
static isStronglyConnected(graph: Graph | null): boolean;
/**
Calculates the k-core for a given undirected input graph and k value.
The k-core of an undirected input graph consists of the subgraph components where each node has at least degree <code>k</code>. The
algorithm has runtime complexity <code>O((n+m)log(n))</code> and requires linear space.
<h2>Note</h2>
This algorithm doesn't modify the input graph but returns the nodes of the k-core (list may be empty).
<h2>Note</h2>
Self-loops are ignored.
@param graph the undirected input graph
@param k the minimum degree (k-core) of a node in the resulting subgraph components
@returns a list that contains the nodes of the k-core (list may be empty)
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23kCore(Graph%2Cnumber) Online Documentation}*/
static kCore(graph: Graph | null, k: number): YNodeList | null;
/**
Calculates the k-cores of an undirected input graph.
The k-core of an undirected input graph consists of the subgraph components where each node has at least degree <code>k</code>. The
algorithm has runtime complexity <code>O((n+m)log(n))</code> and requires linear space.
<h2>Note</h2>
For a given <code>k</code>, the k-core of a graph can consists of all nodes with <code>kValue</code> value greater or equal to <code>k</code>.
<h2>Note</h2>
Self-loops are ignored.
@param graph the undirected input graph
@param kValue stores the largest <code>k</code> for which a node is contained in the k-core
@returns the largest <code>k</code> for which the k-core of the graph is not empty
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23kCore(Graph%2CINodeMap) Online Documentation}*/
static kCore(graph: Graph | null, kValue: INodeMap | null): number;
/**
Makes the given graph biconnected by inserting a minimum number of edges in the graph.
The given graph is considered to be undirected.
@param graph the input graph
@returns an {@link EdgeList} containing the edges added to the graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23makeBiconnected(Graph) Online Documentation}*/
static makeBiconnected(graph: Graph | null): EdgeList | null;
/**
Makes a graph connected by adding additional edges to the graph.
The number of edges that will be added equals the number of separate components of the original graph minus <code>1</code>.
@param graph the input graph
@returns an {@link EdgeList} containing the edges added to the graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23makeConnected(Graph) Online Documentation}*/
static makeConnected(graph: Graph | null): EdgeList | null;
/**
Determines the set of nodes that are reachable from a given node when a set of edges that cannot be traversed is
specified.
<b>Note:</b> This method works with instances of {@link Graph}. To determine reachability of nodes in an {@link IGraph}
use {@link Reachability} instead.
The result is based on a depth first search.
@param graph the given graph
@param start the node from which the search starts
@param directed <code>true</code> if the edges should be traversed from source to target, <code>false</code> if edges can be traversed in both directions
@param [forbidden=null] an array that holds for each {@link Edge} a {@link boolean} value indicating whether or not an edge can be traversed; an edge
<code>e</code> is marked as forbidden if <code>forbidden[e.index()] == true</code>
@param reached an array that will be filled during the execution and returns for each {@link YNode} a {@link boolean} value based on whether
the node can be reached during the DFS; if a node <code>v</code> is reachable, then <code>reached[v.index()] = true</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23reachable(Graph%2CYNode%2Cboolean%2CArray%3Cboolean%3E%2CArray%3Cboolean%3E) Online Documentation}*/
static reachable(graph: Graph | null, start: YNode | null, directed: boolean, reached: boolean[] | null, forbidden?: boolean[] | null): void;
/**
Calculates the strongly connected components of a given graph and returns their number.
<b>Note:</b> This method works with instances of {@link Graph}. To determine strongly connected components in an {@link IGraph}
use {@link StronglyConnectedComponents} instead.
A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
The strongly connected components of a graph are the strongly connected subgraphs of which it consists.
@param graph the input graph
@param compNum the {@link INodeMap} that will be filled during the execution and returns the zero-based index of the
connected component to which each node belongs
@returns the number of strongly connected components of the given graph
@see GraphConnectivity#stronglyConnectedComponents
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23stronglyConnectedComponents(Graph%2CINodeMap) Online Documentation}*/
static stronglyConnectedComponents(graph: Graph | null, compNum: INodeMap | null): number;
/**
Calculates the strongly connected components of a given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To determine strongly connected components in an {@link IGraph}
use {@link StronglyConnectedComponents} instead.
A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
The strongly connected components of a graph are the strongly connected subgraphs of which it consists.
@param graph the input graph
@returns an array of
{@link YNodeList}s each of which contains the nodes that belong to the same strongly connected component
@see GraphConnectivity#stronglyConnectedComponents
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23stronglyConnectedComponents(Graph) Online Documentation}*/
static stronglyConnectedComponents(graph: Graph | null): YNodeList[] | null;
/**
Transforms the return values of {@link GraphConnectivity#biconnectedComponents} to an array of
{@link EdgeList}s, like it is returned by {@link GraphConnectivity#biconnectedComponents}.
@param graph the input graph
@param compNum the {@link IEdgeMap} that will be filled during the execution and returns the zero-based index of the
connected component to which each edge belongs
@param maxCompNum the maximum number of biconnected components
@returns an array of {@link EdgeList}s each containing all edges that belong to the same biconnected component
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23toEdgeListArray(Graph%2CIEdgeMap%2Cnumber) Online Documentation}*/
static toEdgeListArray(graph: Graph | null, compNum: IEdgeMap | null, maxCompNum: number): EdgeList[] | null;
/**
Transforms the return values of method {@link GraphConnectivity#connectedComponents} to an array of
{@link YNodeList}s, like it is returned by {@link GraphConnectivity#connectedComponents}.
@param graph the input graph
@param compNum the {@link INodeMap} that will be filled during the execution and returns the zero-based index of the
connected component to which each node belongs
@param maxCompNum the maximum number of connected components
@returns an array of {@link YNodeList}s each of which contains the nodes that belong to the same connected component
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphConnectivity%23toNodeListArray(Graph%2CINodeMap%2Cnumber) Online Documentation}*/
static toNodeListArray(graph: Graph | null, compNum: INodeMap | null, maxCompNum: number): YNodeList[] | null;
static $class: Class;
private $_GraphConnectivity: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphConnectivity;
}
export interface GraphObject extends YObject {
}
/**
Common base type for both {@link YNode} and {@link Edge}.
This type does not add public functionality to its base type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphObject Online Documentation}*/
export class GraphObject {
static $class: Class;
private $_GraphObject: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphObject;
}
export interface Dendrogram extends YObject {
}
/**
This class provides the result of hierarchical clustering algorithms by means of a binary tree structure.
The dendrogram is created using the agglomerative strategy (i.e., a bottom-up approach) based on the following steps:
<ul>
<li>At the beginning (i.e., at level
<code>0</code>), each node belongs to its own cluster. At this point, each node of the original graph (the one that will be
clustered) is mapped to one of the dendrogram nodes that represent a leaf node (i.e., a node with no children).
<li>At each step, pairs of clusters are merged while moving up to the hierarchy. In this case, a new dendrogram node is
created and the corresponding dissimilarity value between the two merged clusters can be obtained using
{@link Dendrogram#getDissimilarityValue}. The dissimilarity values are monotone increasing starting from zero
which corresponds to the leaf nodes of the dendrogram.
<li>The algorithm continues until all nodes belong to the same cluster. At this point, the root of the dendrogram is reached
and can be obtained using method {@link Dendrogram#root}.
</ul>
The dendrogram is a binary tree that is directed from the root to the leaves. This means that one can iterate the
dendrogram starting with the {@link Dendrogram#root root node}, moving on to its {@link Dendrogram#getChildren children}, then moving on to their children and so on.
From each node of the dendrogram, it is possible to retrieve the set of nodes of the original graph that belong to this
dendrogram node using {@link Dendrogram#getClusterNodes}. This method returns an array of
{@link YNodeList}s that contain the nodes of all clusters that are merged when this dendrogram node was
created. For example, say that clusters <code>C1 = {v1, v2}</code> and <code>C2 = {v3, v4}</code> are merged, where <code>{v1, v2, v3, v4}</code> are nodes of
the original graph. Say that dendrogram node <code>u</code> is created by this merging. Then, the cluster nodes of <code>u</code> will be returned
as two {@link YNodeList}s where the first contains nodes <code>v1, v2</code> and the second <code>v3, v4</code>.
<b>Warning:</b> Only the {@link Dendrogram#isLeaf leaf nodes} of the dendrogram are mapped with a node of the original graph. This means that for all other inner
nodes of the dendrogram, method {@link Dendrogram#getOriginalNode} returns <code>null</code>.
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram Online Documentation}*/
export class Dendrogram {
/**
Returns a {@link YNodeList} that contains the child nodes of the given parent node.
If the given node is a {@link Dendrogram#isLeaf leaf} node, the returned list will be empty.
@param parent the parent node for which to retrieve the child nodes
@returns the list of child nodes of the given parent node
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23getChildren(YNode) Online Documentation}*/
getChildren(parent: YNode | null): YNodeList | null;
/**
Returns an array of
{@link YNodeList}s that contain the nodes of the original graph that are associated with the given dendrogram
node.
Each such {@link YNodeList} represents nodes that belong to the same cluster. For example, say that clusters <code>C1 = {v1, v2}</code>
and <code>C2 = {v3, v4}</code> are merged, where <code>{v1, v2, v3, v4}</code> are nodes of the original graph. Say that dendrogram node <code>u</code> is
created by this merging. Then, the cluster nodes of <code>u</code> will be returned as two {@link YNodeList}s where the
first contains nodes <code>v1, v2</code> and the second <code>v3, v4</code>.
<b>Warning:</b> This method expects a {@link YNode} that belongs to the dendrogram representation and not to the original
graph.
@param node the given dendrogram node
@returns an array of
{@link YNodeList}s that contain the nodes of the original graph that are associated with the given dendrogram
node
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23getClusterNodes(YNode) Online Documentation}*/
getClusterNodes(node: YNode | null): YNodeList[] | null;
/**
Returns the dissimilarity value associated with the given node of the dendrogram.
<b>Warning:</b> This method expects a {@link YNode} that belongs to the dendrogram representation and not to the original
graph.
@param node the given dendrogram node
@returns the dissimilarity value associated with the given dendrogram node
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23getDissimilarityValue(YNode) Online Documentation}*/
getDissimilarityValue(node: YNode | null): number;
/**
Returns the level of the given node of the dendrogram.
<b>Warning:</b> This method expects a {@link YNode} that belongs to the dendrogram representation and not to the original
graph.
@param node the given dendrogram node
@returns the level of the given dendrogram node
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23getLevel(YNode) Online Documentation}*/
getLevel(node: YNode | null): number;
/**
Returns the node of the dendrogram that belongs to the given level index.
If the given level index is greater than the {@link Dendrogram#levelCount number of levels}, then this method will return <code>null</code>.
@param level the given level index
@returns the node of the dendrogram that belongs to the given level index or <code>null</code> if there exists no level with the given index.
@throws {Exception} <b>(ArgumentError)</b> if the given level is negative or greater than the {@link Dendrogram#nodeCount node count} minus one.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23getNodeAtLevel(number) Online Documentation}*/
getNodeAtLevel(level: number): YNode | null;
/**
Returns the node of original graph that is mapped with the given leaf node of the dendrogram.
<b>Warning:</b> This method is meant to be called only for the leaf nodes of the dendrogram and returns <code>null</code> if the given node is not a
leaf node.
@param node the given leaf node
@returns the node of original graph that is mapped with the given leaf node or <code>null</code> if the given node is an inner dendrogram
node
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23getOriginalNode(YNode) Online Documentation}*/
getOriginalNode(node: YNode | null): YNode | null;
/**
Returns whether or not the given dendrogram node is a leaf node.
A leaf node has no further children and represents a node of the original graph. The original node can be retrieved via
{@link Dendrogram#getOriginalNode}.
<b>Warning:</b> This method expects a {@link YNode} that belongs to the dendrogram representation and not to the original
graph.
@param node a dendrogram node
@returns <code>true</code> if the given node is a leaf node, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23isLeaf(YNode) Online Documentation}*/
isLeaf(node: YNode | null): boolean;
/**
Gets the root node of the dendrogram.
A dendrogram traversal can be implemented by starting from this node, then moving on to its children using method
{@link Dendrogram#getChildren}, and so on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23root Online Documentation}*/
readonly root: YNode | null;
/**
Gets the number of nodes in this dendrogram.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23nodeCount Online Documentation}*/
readonly nodeCount: number;
/**
Gets the number of levels this dendrogram has.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Dendrogram%23levelCount Online Documentation}*/
readonly levelCount: number;
static $class: Class;
private $_Dendrogram: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Dendrogram;
}
export interface INodeDistanceProvider {
}
/**
An interface that determines the distance between two nodes of a graph.
It is used by the hierarchical clustering algorithm in order to determine the distance between any two nodes of a
graph.
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see GroupAlgorithm#hierarchicalClustering
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeDistanceProvider Online Documentation}*/
export abstract class INodeDistanceProvider {
/**
Creates an implementation of the interface {@link INodeDistanceProvider} by using the given function as
implementation for its {@link INodeDistanceProvider#getDistance} method.
@param getDistance A function for {@link INodeDistanceProvider}'s single abstract method
{@link INodeDistanceProvider#getDistance}.
@returns An instance of the {@link INodeDistanceProvider} interface based on the given function.*/
static create(getDistance: (n1: YNode | null, n2: YNode | null) => number): INodeDistanceProvider;
/**
Returns the distance between two given nodes of a graph.
Implementations of this method have to guarantee that the returned value is greater or equal to zero. Furthermore, the
distances between two nodes have to be symmetric, i.e., the distance between node <code>n1</code> and <code>n2</code> is equal to the distance
between node <code>n2</code> and <code>n1</code>.
@param n1 the first node
@param n2 the second node
@returns the distance between the two given nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeDistanceProvider%23getDistance(YNode%2CYNode) Online Documentation}*/
abstract getDistance(n1: YNode | null, n2: YNode | null): number;
static $class: Class;
private $_INodeDistanceProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeDistanceProvider;
}
export interface GroupAlgorithm extends YObject {
}
/**
This class provides methods for automatically partitioning nodes of a graph into groups.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To partition an {@link IGraph} into clusters
use one of the following classes instead:
<ul>
<li>{@link EdgeBetweennessClustering}
<li>{@link BiconnectedComponentClustering}
<li>{@link KMeansClustering}
<li>{@link HierarchicalClustering}
<li>{@link LouvainModularityClustering}
<li>{@link LabelPropagationClustering}
<li>{@link ClusteringCoefficient}
</ul>
Partitions can be defined using edge betweenness centrality, biconnectivity, k-means clustering, hierarchical clustering
or community detection algorithms like label propagation or louvain modularity.
This class also contains methods that provide clustering-related information like the average clustering coefficient,
the average weighted coefficient and the modularity of a graph network.
<b>Definitions</b>
<ul>
<li><b>Betweenness centrality</b> is a measure for how often a node lies on a shortest path between each pair of nodes in the graph.
<li><b>Biconnected graph</b> is a graph that has no cut vertex or articulation point (i.e., a node whose removal disconnects the graph).
<li><b>K-means clustering algorithm</b> partitions the nodes of a graph into <code>k</code>-clusters based on their positions on the plane and a given distance metric.
<li><b>Hierarchical clustering</b> creates a hierarchy of clusters in a bottom-to-top approach based on some distance metric and linkage.
<li><b>Label Propagation</b> algorithm iteratively assigns a label to each node. The label of a node is set to the most frequent label among its
neighbors.
Our implementation is based on the description in: "Near linear time algorithm to detect community structures in
large-scale networks" by U. N. Raghavan, R. Albert, and S. Kumara in Phys. Rev. E 76, 036106, 2007
<li><b>Louvain Modularity</b> algorithm iteratively tries to construct communities by moving nodes from their current community to another until the
modularity is locally optimized.
Our implementation is based on the description in: "Fast unfolding of communities in large networks" by V.D. Blondel,
J.L. Guillaume, R. Lambiotte and E. Lefebvre, in Journal of Statistical Mechanics: Theory and Experiment 2008 (10),
P10008 (12pp).
<li><b>Average clustering coefficient</b> is the average of all the local clustering coefficient values of a graph. The <b>local clustering coefficient</b> of a node depends on the number of edges
between its neighbors. A value of <code>1</code> indicates that the node's neighbors form a clique (i.e., there is an edge between
each pair of neighbors).
<li><b>Average weighted clustering coefficient</b> is the average of all the local clustering coefficient values of a graph where each local value of a node is weighted
by the maximum possible number of edges between its neighbors.
<li><b>Modularity</b> is the fraction of edges that belong to a community minus the expected fraction if these edges were distributed at
random, see https://en.wikipedia.org/wiki/Modularity_(networks).
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm Online Documentation}*/
export class GroupAlgorithm {
/**
This method partitions the graph by analyzing its <i>biconnected components</i>.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} into clusters by its
biconnected components use {@link BiconnectedComponentClustering} instead.
Nodes will be grouped such that the nodes within each group are biconnected. Nodes that belong to multiple biconnected
components will be assigned to exactly one of these components.
<h2>Note</h2>
Biconnected components are defined for undirected graphs only. As a consequence, this algorithm ignores self-loops and
isolated nodes with only self-loop edges or no edges at all are not assigned to any group, i.e. the <code>groupID</code> for such a
node will be <code>null</code>.
@param graph the input graph
@param groupIDs the {@link INodeMap} that will be filled during the execution and returns an integer value (cluster ID) for
each node
@returns the resulting number of different groups
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23biconnectedComponentGrouping(Graph%2CINodeMap) Online Documentation}*/
static biconnectedComponentGrouping(graph: Graph | null, groupIDs: INodeMap | null): number;
/**
Partitions the graph into groups using <i>edge betweenness clustering</i> proposed by Girvan and Newman.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} into clusters with
edge-betweenness clustering use {@link EdgeBetweennessClustering} instead.
In each iteration the edge with the highest betweenness centrality is removed from the graph. The method stops, if there
are no more edges to remove or if the requested maximum number of groups is found. The clustering with the best quality
reached during the process is returned.
The algorithm includes several heuristic speed-up techniques available through the quality/time ratio. For the highest
quality setting, it is used almost unmodified. The fast betweenness approximation of Brandes and Pich (<i>Centrality Estimation in Large Networks</i>) is employed for
values around <code>0.5</code>. Typically, this results in a tiny decrease in quality but a large speed-up and is the recommended
setting. To achieve the lowest running time, a local betweenness calculation is used (Gregory: <i>Local Betweenness for Finding Communities in Networks</i>).
The method requires the maximum number of groups that will be returned. The smaller this value is, the faster the
overall computation time. The upper bound on the number of groups is
<code>graph.N()</code>. Also, the number of returned groups is never smaller than the number of connected components of the graph.
@param graph the input graph
@param clusterIDs the {@link INodeMap} that will be filled during the execution and returns an integer value (cluster ID) for
each node
@param qualityTimeRatio a value between <code>0.0</code> (low quality, fast) and <code>1.0</code> (high quality, slow); the recommended value is <code>0.5</code>
@param minGroupCount the minimum number of groups that will be returned
@param maxGroupCount the maximum number of groups that will be returned
@param refine <code>true</code> if the algorithm refines the current grouping, <code>false</code> if the algorithm discards the current grouping
@returns the resulting number of different groups
@throws {Exception} <b>(ArgumentError)</b> if <code>minGroupCount > maxGroupCount</code> or <code>minGroupCount > graph.N()</code> or <code>maxGroupCount &lt;= 0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23edgeBetweennessClustering(Graph%2CINodeMap%2Cnumber%2Cnumber%2Cnumber%2Cboolean) Online Documentation}*/
static edgeBetweennessClustering(graph: Graph | null, clusterIDs: INodeMap | null, qualityTimeRatio: number, minGroupCount: number, maxGroupCount: number, refine: boolean): number;
/**
Partitions the graph into groups using <i>edge betweenness centrality</i>.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} into clusters with
edge-betweenness clustering use {@link EdgeBetweennessClustering} instead.
In each iteration the edge with the highest betweenness centrality is removed from the graph. The method stops, if there
are no more edges to remove. The clustering with the best quality reached during the process will be returned.
The method requires the maximum number of groups that will be returned. The smaller this value is, the faster the
overall computation time. The upper bound on the number of groups is
<code>graph.N()</code>. Also, the number of returned groups is never smaller than the number of connected components of the graph.
@param graph the input graph
@param clusterIDs the {@link INodeMap} that will be filled during the execution and returns an integer value (cluster ID) for
each node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param minGroupCount the minimum number of groups that will be returned
@param maxGroupCount the maximum number of groups that will be returned
@param edgeCosts the {@link IDataProvider} that holds a positive {@link number} cost or <code>null</code> if the edges of the graph are considered to be
of equal cost
@returns the resulting number of different groups
@throws {Exception} <b>(ArgumentError)</b> if <code>minGroupCount > maxGroupCount</code> or <code>minGroupCount > graph.N()</code> or <code>maxGroupCount &lt;= 0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23edgeBetweennessClustering(Graph%2CINodeMap%2Cboolean%2Cnumber%2Cnumber%2CIDataProvider) Online Documentation}*/
static edgeBetweennessClustering(graph: Graph | null, clusterIDs: INodeMap | null, directed: boolean, minGroupCount: number, maxGroupCount: number, edgeCosts: IDataProvider | null): number;
/**
Calculates the local clustering coefficient for each node and returns the average clustering coefficient.
<b>Note:</b> This method works with instances of {@link Graph}. To measure the clustering coefficients in an {@link IGraph}
use {@link ClusteringCoefficient} instead.
The clustering coefficient measures the degree to which the nodes of a network tend to cluster together, see
https://en.wikipedia.org/wiki/Clustering_coefficient. More precisely, for a node n, the local clustering coefficient is
the actual number of edges between the neighbors of n divided by the maximum possible number of such edges. Hence, it is
always a {@link number} value between <code>0.0</code> and <code>1.0</code>.
<h2>Note</h2>
The algorithm ignores self-loops and parallel edges. Group nodes are handled like common nodes.
@param graph the input graph
@param coefficientMap a map that returns the clustering coefficient for each node
@param directed whether or not the graph is directed
@returns returns the average clustering coefficient
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23getClusteringCoefficient(Graph%2CINodeMap%2Cboolean) Online Documentation}*/
static getClusteringCoefficient(graph: Graph | null, coefficientMap: INodeMap | null, directed: boolean): number;
/**
Computes the modularity of a given graph.
The modularity value estimates the quality of the partition of the nodes into the given communities and is a value
between
<code>[-1/2,1]</code>. High modularity values denote that nodes with dense connections are placed in the same community, while nodes
with sparse connections are assigned to different communities.
To compute the modularity, a {@link IDataProvider} <code>communityIndex</code> is needed that returns an integer value
representing the community index for each node.
If no {@link IDataProvider} for the edge weights is given, the algorithm will do the calculation assuming that
all edges have edge weight equals to <code>1.0</code>. Note that in the case where, all edges have edge weights equal to <code>0.0</code>, the
modularity will be <code>0.0</code>, too.
@param graph The input graph.
@param communityIndex The community index for each node (nodes with same index are associated with the same community).
@param [edgeWeight=null] The weight of the edges.
If no {@link IDataProvider} is given, an equal weight of <code>1.0</code> will be assumed for all edges.
@returns the modularity of a given graph
@throws {Exception} <b>(ArgumentError)</b> if no {@link IDataProvider} <code>communityIndex</code> for the community indices is specified
@see GroupAlgorithm#louvainModularity
@see GroupAlgorithm#labelPropagation
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23getModularity(Graph%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static getModularity(graph: Graph | null, communityIndex: IDataProvider | null, edgeWeight?: IDataProvider | null): number;
/**
Partitions the graph into clusters based on hierarchical clustering, while the dendrogram is cut based on a given
maximum number of clusters.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} using hierarchical
clustering use {@link HierarchicalClustering} instead.
The clustering is performed using the agglomerative strategy i.e., a bottom-up approach according to which at the
beginning each node belongs to its own cluster. At each step pairs of clusters are merged while moving up to the
hierarchy. The dissimilarity between clusters is determined based on the given linkage and the given node distances. The
algorithm continues until all nodes belong to the same cluster.
The result will be given based on the given maximum number of clusters value that is used for cutting the hierarchical
tree at a point such that the number of remaining clusters equals to this value.
The maximum number of clusters needs to be greater than zero and less than the number of the nodes of the graph.
<b>Warning:</b> If the {@link INodeDistanceProvider} object returns a negative distance value for a pair of nodes, zero will
be used instead.
@param graph the input graph
@param maxCluster the maximum number of clusters that determines where to cut the hierarchic tree into clusters
@param clusterIDs the {@link INodeMap} that will be filled during the execution and returns an integer value (cluster ID) for
each node
@param distances a given {@link INodeDistanceProvider} object that determines the distance between any two graph nodes
@param linkage one of the predefined linkage values
@returns the resulting number of clusters
@throws {Exception} <b>(ArgumentError)</b> if an unknown linkage is given or if the maximum number of clusters is less than or equal to zero or greater than the
number of nodes of the graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23hierarchicalClustering(Graph%2Cnumber%2CINodeMap%2CINodeDistanceProvider%2CLinkage) Online Documentation}*/
static hierarchicalClustering(graph: Graph | null, maxCluster: number, clusterIDs: INodeMap | null, distances: INodeDistanceProvider | null, linkage: Linkage | LinkageStringValues): number;
/**
Partitions the graph into clusters based on hierarchical clustering, while the dendrogram is cut based on a given
cut-off value.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} using hierarchical
clustering use {@link HierarchicalClustering} instead.
The clustering is performed using the agglomerative strategy i.e., a bottom-up approach according to which at the
beginning each node belongs to its own cluster. At each step pairs of clusters are merged while moving up to the
hierarchy. The dissimilarity between clusters is determined based on the given linkage and the given node distances. The
algorithm continues until all nodes belong to the same cluster.
The result will be given based on the given cut-off value that is used for cutting the hierarchical tree at a point such
that the dissimilarity values of the nodes that remain at the dendrogram are less than this value.
<b>Warning:</b> The cut-off value should be greater that zero. If a negative value is given, zero will be used instead.
<b>Warning:</b> If the {@link INodeDistanceProvider} object returns a negative distance value for a pair of nodes, zero will
be used instead.
@param graph the input graph
@param clusterIDs the {@link INodeMap} that will be filled during the execution and returns an integer value (cluster ID) for
each node
@param distances a given {@link INodeDistanceProvider} object that determines the distance between any two nodes
@param linkage one of the predefined linkage values
@param cutOff the cut-off value that determines where to cut the hierarchic tree into clusters
@returns the resulting number of clusters
@throws {Exception} <b>(ArgumentError)</b> if an unknown linkage is used
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23hierarchicalClustering(Graph%2CINodeMap%2CINodeDistanceProvider%2CLinkage%2Cnumber) Online Documentation}*/
static hierarchicalClustering(graph: Graph | null, clusterIDs: INodeMap | null, distances: INodeDistanceProvider | null, linkage: Linkage | LinkageStringValues, cutOff: number): number;
/**
Partitions the graph into clusters based on hierarchical clustering.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} using hierarchical
clustering use {@link HierarchicalClustering} instead.
The clustering is performed using the agglomerative strategy i.e., a bottom-up approach according to which at the
beginning each node belongs to its own cluster. At each step pairs of clusters are merged while moving up to the
hierarchy. The dissimilarity between clusters is determined based on the given linkage and the given node distances
metric. The algorithm continues until all nodes belong to the same cluster.
The result is returned as a {@link Dendrogram} object which represents the result of the clustering algorithm
as a binary tree structure. It can easily be traversed by starting from the {@link Dendrogram#root root node} and moving on to nodes of the next
level via method {@link Dendrogram#getChildren}.
<b>Warning:</b> If the {@link INodeDistanceProvider} object returns a negative distance value for a pair of nodes, zero will
be used instead.
@param graph the input graph
@param distances a given {@link INodeDistanceProvider} object that determines the distance between any two nodes
@param linkage one of the predefined linkage values
@returns a {@link Dendrogram} which represents the result of the clustering as a binary tree
@throws {Exception} <b>(ArgumentError)</b> if an unknown linkage is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23hierarchicalClustering(Graph%2CINodeDistanceProvider%2CLinkage) Online Documentation}*/
static hierarchicalClustering(graph: Graph | null, distances: INodeDistanceProvider | null, linkage: Linkage | LinkageStringValues): Dendrogram | null;
/**
Partitions the graph into clusters using k-means clustering algorithm.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} using <i>k</i>-means
clustering use {@link KMeansClustering} instead.
The nodes of the graph will be partitioned in <code>k</code> clusters based on their positions such that their distance from the
cluster's mean (centroid) is minimized.
The distance can be defined using diverse metrics as euclidean distance, euclidean-squared distance, manhattan distance
or chebychev distance.
<h2>Note</h2>
If the given number of clusters is greater than the number of nodes of the graph, number <code>k</code> will be set equal to <code>2</code>.
<b>Warning:</b> If the number of given centroids is smaller than <code>k</code> or if no centroids are given, random initial centroids will be
assigned for all clusters.
@param graph the input graph
@param clusterIDs the {@link INodeMap} that will be filled during the execution and returns an integer value (cluster ID) for
each node
@param nodePositions the {@link IDataProvider} that holds a {@link YPoint point} representing the current position of each node in the graph
@param distanceMetric one of the predefined distance metrics
@param k the number of clusters
@param [iterations=100] the maximum number of iterations performed by the algorithm for convergence
@param [centroids=null] the initial centroids
@returns the number of resulting (non-empty) clusters
@throws {Exception} <b>(ArgumentError)</b> if the given distance metric is not supported
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23kMeansClustering(Graph%2CINodeMap%2CIDataProvider%2CDistanceMetric%2Cnumber%2Cnumber%2CArray%3CYPoint%3E) Online Documentation}*/
static kMeansClustering(graph: Graph | null, clusterIDs: INodeMap | null, nodePositions: IDataProvider | null, distanceMetric: DistanceMetric | DistanceMetricStringValues, k: number, iterations?: number, centroids?: YPoint[] | null): number;
/**
Partitions the graph into clusters using k-means clustering algorithm.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} using <i>k</i>-means
clustering use {@link KMeansClustering} instead.
The nodes of the graph will be partitioned in <code>k</code> clusters based on their positions such that their distance from the
cluster's mean (centroid) is minimized.
The distance can be defined using diverse metrics as euclidean distance, euclidean-squared distance, manhattan distance
or chebychev distance.
<h2>Note</h2>
If the given number of clusters is greater than the number of nodes of the graph, number <code>k</code> will be set equal to <code>2</code>.
<b>Warning:</b> If the number of given centroids is smaller than <code>k</code> or if no centroids are given, random initial centroids will be
assigned for all clusters.
@param options The parameters to pass.
@param options.graph the input graph
@param options.clusterIDs the {@link INodeMap} that will be filled during the execution and returns an integer value (cluster ID) for
each node
@param options.nodePositions the {@link IDataProvider} that holds a {@link YPoint point} representing the current position of each node in the graph
@param options.distanceMetric one of the predefined distance metrics
@param options.k the number of clusters
@param [options.iterations=100] the maximum number of iterations performed by the algorithm for convergence
@param [options.centroids=null] the initial centroids
@returns the number of resulting (non-empty) clusters
@throws {Exception} <b>(ArgumentError)</b> if the given distance metric is not supported
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23kMeansClustering(Graph%2CINodeMap%2CIDataProvider%2CDistanceMetric%2Cnumber%2Cnumber%2CArray%3CYPoint%3E) Online Documentation}*/
static kMeansClustering(options: {
graph: Graph | null;
clusterIDs: INodeMap | null;
nodePositions: IDataProvider | null;
distanceMetric: DistanceMetric;
k: number;
iterations?: number;
centroids?: YPoint[] | null;
}): number;
/**
Detects the communities in the specified input graph by applying a label propagation algorithm.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} into clusters using
label propagation use {@link LabelPropagationClustering} instead.
This algorithm iteratively assigns a label to each node. The label of a node is set to the most frequent label among its
neighbors. If there are multiple candidates the algorithm randomly chooses one of them. After applying the algorithm,
all nodes with the same label (i.e. nodes for which {@link INodeMap} <code>finalLabel</code> returns the same integer
value) belong to the same community.
For a given node and neighbor, the weight of the neighbors label is
<code>nodeWeight(neighbor) * edgeWeight((node, neighbor))</code>. In addition, the overall weight of a specific label is the sum of
such weights for all neighbors with matching label.
At the start of the algorithm, each node is associated with the label returned by method {@link IDataProvider#getInt}
of <code>initialLabel</code>. Negative values are ignored, i.e., at the start there may be nodes without label. This implies that
there may be results, where some of the nodes are not associated with labels, too. For such nodes method GetInt of <code>finalLabel</code>
returns <code>-1</code>. If no <code>initialLabel</code> provider is given each node starts with a unique label value.
If {@link IDataProvider} <code>edgeDirectedness</code> is specified, the algorithm only considers a subset of the neighbors
when determining the label of a node. More precisely, it only considers the neighbors connected by edges to a node n for
which
<ul>
<li>the edge's directedness is greater than <code>0.0</code> and the edge is incoming (i.e. n is the edge's target),
<li>the edge's directedness is smaller than <code>0.0</code> and the edge is outgoing (i.e. n is the edge's source),
<li>or the edge's directedness is <code>0.0</code> (i.e. the edge is considered to be <b>undirected</b>).
</ul>
<h2>Note</h2>
A node with a self-loop is considered to be a neighbor of itself. Hence, the node's label is also included when
calculating the most frequent label among the neighbors. If you want to prevent this, you can specify suitable edge
weights. Setting the weight of self-loops to <code>0.0</code> results in ignoring them.
<h2>Note</h2>
If none of the neighbors of a node is associated with a label with positive weight, the node keeps its current label.
<h2>Note</h2>
If no <code>edgeDirectedness</code> provider is specified, the algorithm assumes that all edges are undirected (i.e. have
directedness <code>0.0</code>).
@param graph the undirected input graph
@param finalLabel the {@link INodeMap} that returns the integer labels of each node after applying the algorithm where nodes
without a label receive the value <code>-1</code>
@param [initialLabel=null] {@link IDataProvider} that stores the initial integer labels of each node (negative values indicate that a
node is not associated with an initial label).
If no provider is specified, each node starts with a unique label.
@param [nodeWeight=null] {@link IDataProvider} that stores the weights of the nodes.
If no provider is specified, the algorithm assumes that all nodes have the weight <code>1.0</code>.
@param [edgeWeight=null] {@link IDataProvider} that stores the weights of the edges.
If no provider is specified, the algorithm assumes that all edges have the weight <code>1.0</code>.
@param [edgeDirectedness=null] {@link IDataProvider} that stores the directedness of the edges.
If no provider is specified, the algorithm assumes that all edges are undirected (i.e. have directedness <code>0.0</code>).
@returns the number of resulting communities
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23labelPropagation(Graph%2CINodeMap%2CIDataProvider%2CIDataProvider%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static labelPropagation(graph: Graph | null, finalLabel: INodeMap | null, initialLabel?: IDataProvider | null, nodeWeight?: IDataProvider | null, edgeWeight?: IDataProvider | null, edgeDirectedness?: IDataProvider | null): number;
/**
Detects the communities in the specified input graph by applying a label propagation algorithm.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} into clusters using
label propagation use {@link LabelPropagationClustering} instead.
This algorithm iteratively assigns a label to each node. The label of a node is set to the most frequent label among its
neighbors. If there are multiple candidates the algorithm randomly chooses one of them. After applying the algorithm,
all nodes with the same label (i.e. nodes for which {@link INodeMap} <code>finalLabel</code> returns the same integer
value) belong to the same community.
For a given node and neighbor, the weight of the neighbors label is
<code>nodeWeight(neighbor) * edgeWeight((node, neighbor))</code>. In addition, the overall weight of a specific label is the sum of
such weights for all neighbors with matching label.
At the start of the algorithm, each node is associated with the label returned by method {@link IDataProvider#getInt}
of <code>initialLabel</code>. Negative values are ignored, i.e., at the start there may be nodes without label. This implies that
there may be results, where some of the nodes are not associated with labels, too. For such nodes method GetInt of <code>finalLabel</code>
returns <code>-1</code>. If no <code>initialLabel</code> provider is given each node starts with a unique label value.
If {@link IDataProvider} <code>edgeDirectedness</code> is specified, the algorithm only considers a subset of the neighbors
when determining the label of a node. More precisely, it only considers the neighbors connected by edges to a node n for
which
<ul>
<li>the edge's directedness is greater than <code>0.0</code> and the edge is incoming (i.e. n is the edge's target),
<li>the edge's directedness is smaller than <code>0.0</code> and the edge is outgoing (i.e. n is the edge's source),
<li>or the edge's directedness is <code>0.0</code> (i.e. the edge is considered to be <b>undirected</b>).
</ul>
<h2>Note</h2>
A node with a self-loop is considered to be a neighbor of itself. Hence, the node's label is also included when
calculating the most frequent label among the neighbors. If you want to prevent this, you can specify suitable edge
weights. Setting the weight of self-loops to <code>0.0</code> results in ignoring them.
<h2>Note</h2>
If none of the neighbors of a node is associated with a label with positive weight, the node keeps its current label.
<h2>Note</h2>
If no <code>edgeDirectedness</code> provider is specified, the algorithm assumes that all edges are undirected (i.e. have
directedness <code>0.0</code>).
@param options The parameters to pass.
@param options.graph the undirected input graph
@param options.finalLabel the {@link INodeMap} that returns the integer labels of each node after applying the algorithm where nodes
without a label receive the value <code>-1</code>
@param [options.initialLabel=null] {@link IDataProvider} that stores the initial integer labels of each node (negative values indicate that a
node is not associated with an initial label).
If no provider is specified, each node starts with a unique label.
@param [options.nodeWeight=null] {@link IDataProvider} that stores the weights of the nodes.
If no provider is specified, the algorithm assumes that all nodes have the weight <code>1.0</code>.
@param [options.edgeWeight=null] {@link IDataProvider} that stores the weights of the edges.
If no provider is specified, the algorithm assumes that all edges have the weight <code>1.0</code>.
@param [options.edgeDirectedness=null] {@link IDataProvider} that stores the directedness of the edges.
If no provider is specified, the algorithm assumes that all edges are undirected (i.e. have directedness <code>0.0</code>).
@returns the number of resulting communities
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23labelPropagation(Graph%2CINodeMap%2CIDataProvider%2CIDataProvider%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static labelPropagation(options: {
graph: Graph | null;
finalLabel: INodeMap | null;
initialLabel?: IDataProvider | null;
nodeWeight?: IDataProvider | null;
edgeWeight?: IDataProvider | null;
edgeDirectedness?: IDataProvider | null;
}): number;
/**
Detects the communities in the specified input graph by applying the louvain modularity method.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} into clusters using
louvain modularity use {@link LouvainModularityClustering} instead.
The algorithm starts by assigning each node to its own community. Then, iteratively tries to construct communities by
moving nodes from their current community to another until the modularity is locally optimized. At the next step, the
small communities found are merged to a single node and the algorithm starts from the beginning until the modularity of
the graph cannot be further improved.
The community index of a node corresponds to the index of the associated community. If there are nodes that are not
associated with a community, their index is <code>-1</code>.
If no {@link IDataProvider} for the edge weights is given, the algorithm will assume that all edges have edge
weights equal to <code>1.0</code>.
@param graph the input graph
@param communityIndex {@link INodeMap} that stores, for each node, the integer community index of the node.
@param [edgeWeight=null] {@link IDataProvider} that stores the weights of the edges.
If nothing is given, the algorithm will assume that all edges have edge weights equal to <code>1.0</code>.
@returns the number of resulting communities
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlgorithm%23louvainModularity(Graph%2CINodeMap%2CIDataProvider) Online Documentation}*/
static louvainModularity(graph: Graph | null, communityIndex: INodeMap | null, edgeWeight?: IDataProvider | null): number;
static $class: Class;
private $_Groups: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GroupAlgorithm;
}
export interface ICursor {
}
/**
A general interface for iterating over a collection of objects.
It can be regarded as a read-only view of such a collection.
A YCursor acts like a movable pointer on the elements of a collection. The pointer can be moved forward and backward and
the element currently pointed on can be accessed. The removal of elements can only be performed on the provider of the
cursor, not on the cursor itself. (That's why the cursor presents a read-only view.)
Implementations of this interface do not need to support operations marked "optional."
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICursor Online Documentation}*/
export abstract class ICursor {
/**
Moves this cursor one position forward.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICursor%23next() Online Documentation}*/
abstract next(): void;
/**
Moves this cursor one position backward (optional).
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICursor%23prev() Online Documentation}*/
abstract prev(): void;
/**
Moves this cursor to the first valid cursor position (optional).
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICursor%23toFirst() Online Documentation}*/
abstract toFirst(): void;
/**
Moves this cursor to the last valid cursor position (optional).
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICursor%23toLast() Online Documentation}*/
abstract toLast(): void;
/**
Gets <code>true</code> if the current cursor position is valid.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICursor%23ok Online Documentation}*/
abstract readonly ok: boolean;
/**
Gets the object currently pointed on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICursor%23current Online Documentation}*/
abstract readonly current: Object | null;
/**
Gets the number of elements that can be accessed with this cursor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ICursor%23size Online Documentation}*/
abstract readonly size: number;
static $class: Class;
private $_ICursor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ICursor;
}
export interface IDataAcceptor {
}
/**
A general interface for setting data.
A data acceptor associates data with data holders. It constitutes a write-only view on particular data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataAcceptor Online Documentation}*/
export abstract class IDataAcceptor {
/**
Creates an implementation of the interface {@link IDataAcceptor} from the given definition.
The given object must contain exactly the abstract members of the {@link IDataAcceptor} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IDataAcceptor} interface.
@returns An instance of the {@link IDataAcceptor} interface based on the given definition.*/
static create(interfaceDefinition: {
set(dataHolder: any | null, value: any | null): void;
setInt(dataHolder: any | null, value: number): void;
setNumber(dataHolder: any | null, value: number): void;
setBoolean(dataHolder: any | null, value: boolean): void;
}): IDataAcceptor;
/**
Sets an object value associated with the given data holder.
This method may throw an UnsupportedOperationException.
@param dataHolder
@param value
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataAcceptor%23set(Object%2CObject) Online Documentation}*/
abstract set(dataHolder: any | null, value: any | null): void;
/**
Sets a boolean value associated with the given data holder.
This method may throw an UnsupportedOperationException.
@param dataHolder
@param value
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataAcceptor%23setBoolean(Object%2Cboolean) Online Documentation}*/
abstract setBoolean(dataHolder: any | null, value: boolean): void;
/**
Sets an integer value associated with the given data holder.
This method may throw an UnsupportedOperationException.
@param dataHolder
@param value
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataAcceptor%23setInt(Object%2Cnumber) Online Documentation}*/
abstract setInt(dataHolder: any | null, value: number): void;
/**
Sets a double value associated with the given data holder.
This method may throw an UnsupportedOperationException.
@param dataHolder
@param value
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataAcceptor%23setNumber(Object%2Cnumber) Online Documentation}*/
abstract setNumber(dataHolder: any | null, value: number): void;
static $class: Class;
private $_IDataAcceptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IDataAcceptor;
}
export interface IDataMap extends IDataProvider, IDataAcceptor {
}
/**
Interface that combines the {@link IDataProvider} and {@link IDataAcceptor} interfaces.
This interface does not declare any additional methods.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataMap Online Documentation}*/
export abstract class IDataMap {
/**
Creates an implementation of the interface {@link IDataMap} from the given definition.
The given object must contain exactly the abstract members of the {@link IDataMap} interface. No member may be
omitted and no additional members can be added. While additional custom members can later be added directly on the
instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IDataMap} interface.
@returns An instance of the {@link IDataMap} interface based on the given definition.*/
static create(interfaceDefinition: {
get(dataHolder: any | null): any;
getInt(dataHolder: any | null): number;
getNumber(dataHolder: any | null): number;
getBoolean(dataHolder: any | null): boolean;
set(dataHolder: any | null, value: any | null): void;
setInt(dataHolder: any | null, value: number): void;
setNumber(dataHolder: any | null, value: number): void;
setBoolean(dataHolder: any | null, value: boolean): void;
}): IDataMap;
static $class: Class;
private $_IDataMap: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IDataMap;
}
export interface IDataProvider {
}
/**
A general interface for data provision.
A data provider grants access to data associated with one or more data holders. It constitutes a read-only view on
particular data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataProvider Online Documentation}*/
export abstract class IDataProvider {
/**
Creates an implementation of the interface {@link IDataProvider} from the given definition.
The given object must contain exactly the abstract members of the {@link IDataProvider} interface. No member
may be omitted and no additional members can be added. While additional custom members can later be added directly on
the instance, this is discouraged.
@param interfaceDefinition An object that contains exactly the abstract members of the {@link IDataProvider} interface.
@returns An instance of the {@link IDataProvider} interface based on the given definition.*/
static create(interfaceDefinition: {
get(dataHolder: any | null): any;
getInt(dataHolder: any | null): number;
getNumber(dataHolder: any | null): number;
getBoolean(dataHolder: any | null): boolean;
}): IDataProvider;
/**
Returns an object value associated with the given data holder.
This method may throw an UnsupportedOperationException.
@param dataHolder
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataProvider%23get(Object) Online Documentation}*/
abstract get(dataHolder: any | null): any | null;
/**
Returns a boolean value associated with the given data holder.
This method may throw an UnsupportedOperationException.
@param dataHolder
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataProvider%23getBoolean(Object) Online Documentation}*/
abstract getBoolean(dataHolder: any | null): boolean;
/**
Returns an integer value associated with the given data holder.
This method may throw an UnsupportedOperationException.
@param dataHolder
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataProvider%23getInt(Object) Online Documentation}*/
abstract getInt(dataHolder: any | null): number;
/**
Returns a double value associated with the given data holder.
This method may throw an UnsupportedOperationException.
@param dataHolder
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDataProvider%23getNumber(Object) Online Documentation}*/
abstract getNumber(dataHolder: any | null): number;
static $class: Class;
private $_IDataProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IDataProvider;
}
export interface IEdgeCursor extends ICursor {
}
/**
A cursor interface for iterating over edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeCursor Online Documentation}*/
export abstract class IEdgeCursor {
/**
Moves the cursor to the cyclic next element of the underlying sequence.
This is the next element if available, else it is the first element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeCursor%23cyclicNext() Online Documentation}*/
abstract cyclicNext(): void;
/**
Moves the cursor to the cyclic previous element of the underlying sequence.
This is the previous element if available, else it is the last element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeCursor%23cyclicPrev() Online Documentation}*/
abstract cyclicPrev(): void;
/**
Gets the edge at the current location of the cursor.
This method is the typed variant of {@link ICursor#current}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeCursor%23edge Online Documentation}*/
abstract readonly edge: Edge | null;
static $class: Class;
private $_IEdgeCursor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeCursor;
}
export interface IEdgeMap extends IDataProvider, IDataAcceptor, IDataMap {
}
/**
Provides access to data associated with an edge.
An edge map can be considered as a map that allows only edges as keys. Edge keys of an edge map must belong to the same
graph.
There are data access methods defined for the most common typed values as well.
The edge values are initialized with Java(TM) default values (null, 0, 0.0, false) upon initialization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeMap Online Documentation}*/
export abstract class IEdgeMap {
static $class: Class;
private $_IEdgeMap: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeMap;
}
export interface INodeCursor extends ICursor {
}
/**
A cursor interface for iterating over nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeCursor Online Documentation}*/
export abstract class INodeCursor {
/**
Moves the cursor to the cyclic next element of the underlying sequence.
This is the next element if available, else it is the first element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeCursor%23cyclicNext() Online Documentation}*/
abstract cyclicNext(): void;
/**
Moves the cursor to the cyclic previous element of the underlying sequence.
This is the previous element if available, else it is the last element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeCursor%23cyclicPrev() Online Documentation}*/
abstract cyclicPrev(): void;
/**
Gets the node at the current location of the cursor.
This method is the typed variant of {@link ICursor#current}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeCursor%23node Online Documentation}*/
abstract readonly node: YNode | null;
static $class: Class;
private $_INodeCursor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeCursor;
}
export interface INodeMap extends IDataProvider, IDataAcceptor, IDataMap {
}
/**
Provides access to data associated with a node.
A node map can be considered as a map that allows only nodes as keys. Node keys of a node map must belong to the same
graph.
There are data access methods defined for the most common typed values as well.
The node values are initialized with Java(TM) default values (null, 0, 0.0, false) upon initialization.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeMap Online Documentation}*/
export abstract class INodeMap {
static $class: Class;
private $_INodeMap: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeMap;
}
export interface INodeSequencer {
}
/**
This is a generic interface for classes that provide an ordering for the nodes of a graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeSequencer Online Documentation}*/
export abstract class INodeSequencer {
/**
Creates an implementation of the interface {@link INodeSequencer} by using the given function as
implementation for its {@link INodeSequencer#nodes} method.
@param nodes A function for {@link INodeSequencer}'s single abstract method {@link INodeSequencer#nodes}.
@returns An instance of the {@link INodeSequencer} interface based on the given function.*/
static create(nodes: (graph: Graph | null) => INodeCursor): INodeSequencer;
/**
Returns a cursor that grants access to <b>all</b> nodes of the given graph in some order.
@param graph the input graph
@returns a {@link INodeCursor} that grants access to all nodes of a graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeSequencer%23nodes(Graph) Online Documentation}*/
abstract nodes(graph: Graph | null): INodeCursor | null;
static $class: Class;
private $_INodeSequencer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeSequencer;
}
export interface IndependentSetAlgorithm extends YObject {
}
/**
This class provides methods for calculating independent sets.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To partition an {@link IGraph} into
independent sets use {@link IndependentSets} instead.
An <b>independent set</b> is a set of nodes in a graph, in which no two nodes are adjacent.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSetAlgorithm Online Documentation}*/
export class IndependentSetAlgorithm {
/**
Calculates an independent set for a given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} into independent sets
use {@link IndependentSets} instead.
A greedy heuristic is applied which tries to find a large independent set.
@param conflictGraph the input graph
@returns a {@link YNodeList} containing an independent set of nodes
@see IndependentSetAlgorithm#getIndependentSet
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSetAlgorithm%23getIndependentSet(Graph) Online Documentation}*/
static getIndependentSet(conflictGraph: Graph | null): YNodeList | null;
/**
Partitions the set of nodes of the given graph into independent sets.
<b>Note:</b> This method works with instances of {@link Graph}. To partition an {@link IGraph} into independent sets
use {@link IndependentSets} instead.
The method iteratively calls method {@link IndependentSetAlgorithm#getIndependentSet}.
@param conflictGraph the input graph
@returns an array of {@link YNodeList}s each of which contains an independent set of nodes
@see IndependentSetAlgorithm#getIndependentSet
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSetAlgorithm%23getIndependentSets(Graph) Online Documentation}*/
static getIndependentSets(conflictGraph: Graph | null): YNodeList[] | null;
static $class: Class;
private $_IndependentSets: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IndependentSetAlgorithm;
}
export interface ListCell extends YObject {
}
/**
Represents a so-called "cell" or "link" of the doubly linked list implementation {@link YList}.
It may be used to perform fast access and remove operations on that type of list.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ListCell Online Documentation}*/
export class ListCell {
/**
Returns the predecessor cell of this cell.
If there is no predecessor, then <code>null</code> is returned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ListCell%23pred() Online Documentation}*/
pred(): ListCell | null;
/**
Returns the successor cell of this cell.
If there is no successor, then <code>null</code> is returned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ListCell%23succ() Online Documentation}*/
succ(): ListCell | null;
/**
Gets or sets the element stored in this cell.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ListCell%23info Online Documentation}*/
info: Object | null;
static $class: Class;
private $_ListCell: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ListCell;
}
export interface NetworkFlowAlgorithm extends YObject {
}
/**
This class provides sophisticated algorithms for solving classical network flow problems.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To solve flow problems on {@link IGraph}
instances use {@link MaximumFlow} or {@link MinimumCostFlow} instead.
<b>Definitions</b>
<ul>
<li><b><b>Maximum flow problem:</b></b> – Given a directed graph in which each edge has a capacity and given a source node <code>s</code> and a sink node <code>t</code>, find a flow of
maximum value from <code>s</code> to <code>t</code>.
<li><b><b>Minimum cut problem:</b></b> – Given a directed graph in which each edge has a capacity and given a source node <code>s</code> and a sink node <code>t</code>, find an <code>s-t</code> cut of
minimum capacity (i.e., a set of edges of minimum capacity whose removal would disconnect <code>t</code> from <code>s</code>).
<li><b><b>Minimum cost flow problem:</b></b> – Given a directed graph in which each edge has a cost and a capacity and each node has a supply or demand, find a flow of
minimum total cost that satisfies the edge capacities and the node balances.
<li><b><b>Minimum cost maximum flow problem:</b></b> – Given a directed graph in which each edge has a cost and a capacity and given a source node <code>s</code> and a sink node <code>t</code>, find a
flow of maximum value from <code>s</code> to <code>t</code> that has the minimum total cost.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/NetworkFlowAlgorithm Online Documentation}*/
export class NetworkFlowAlgorithm {
/**
Solves a maximum flow problem using the preflow-push method.
<b>Note:</b> This method works with instances of {@link Graph}. To solve a maximum flow problem on {@link IGraph}
instances use {@link MaximumFlow} instead.
The implementation is based on
<ul>
<li>Mehlhorn, Naeher: <i>LEDA: a platform for combinatorial and geometric computing</i>, Cambridge University Press, 2000, pp. 443-488.
</ul>
The worst case running time is <code>O(mdeg * n^2 * m^(1/2))</code>, where <code>n</code> is the number of nodes in the network, <code>m</code> the number of
edges and <code>mdeg</code> the maximal degree of any node.
Edges may have infinite capacity, which is denoted by the value <code>0x7FFFFFFF</code>.
@param graph the given network
@param source the source node of the network
@param sink the sink node of the network
@param eCapDP the {@link IEdgeMap} that returns the integer capacity for each edge. <code>0x7FFFFFFF</code> denotes infinite capacity for
an edge
@param flowEM the {@link IEdgeMap} that will be filled during the execution with an integer flow for each edge
@returns the maximum flow value
@see NetworkFlowAlgorithm#calcMaxFlowMinCut
@see {@link https://docs.yworks.com/yfileshtml/#/api/NetworkFlowAlgorithm%23calcMaxFlow(Graph%2CYNode%2CYNode%2CIDataProvider%2CIEdgeMap) Online Documentation}*/
static calcMaxFlow(graph: Graph | null, source: YNode | null, sink: YNode | null, eCapDP: IDataProvider | null, flowEM: IEdgeMap | null): number;
/**
Solves a maximum flow problem using the preflow-push method but additionally marks all nodes that belong to the minimum
cut set that is associated with the source of the network.
<b>Note:</b> This method works with instances of {@link Graph}. To solve a maximum flow problem on {@link IGraph}
instances use {@link MaximumFlow} instead.
@param graph the given network
@param source the source node of the network
@param sink the sink node of the network
@param eCapDP the {@link IEdgeMap} that returns the integer capacity for each edge. <code>0x7FFFFFFF</code> denotes infinite capacity for
an edge
@param flowEM the {@link IEdgeMap} that will be filled during the execution with an integer flow for each edge
@param sourceCutNM the {@link INodeMap} that will be filled during the execution and returns a boolean value indicating whether
or not a node belongs to the cut set associated with the source of the network
@returns the maximum flow value
@see NetworkFlowAlgorithm#calcMaxFlow
@see {@link https://docs.yworks.com/yfileshtml/#/api/NetworkFlowAlgorithm%23calcMaxFlowMinCut(Graph%2CYNode%2CYNode%2CIDataProvider%2CIEdgeMap%2CINodeMap) Online Documentation}*/
static calcMaxFlowMinCut(graph: Graph | null, source: YNode | null, sink: YNode | null, eCapDP: IDataProvider | null, flowEM: IEdgeMap | null, sourceCutNM: INodeMap | null): number;
/**
Solves a minimum cost maximum flow problem.
<b>Note:</b> This method works with instances of {@link Graph}. To solve a minimum-cost flow problem on {@link IGraph}
instances use {@link MinimumCostFlow} instead.
@param graph the given network
@param s the source node of the network
@param t the sink node of the network
@param uCapDP the {@link IDataProvider} that returns the integer capacity of each edge
@param cost0DP the {@link IDataProvider} that returns a double value (cost) for each edge
@param flowEM the {@link IEdgeMap} that will be filled during the execution with an integer flow for each edge
@param dualsNM the {@link INodeMap} that will be filled during the execution with an integer value (dual value) for each node
or <code>null</code> if no such values occur; dual values are also referred as potentials
@returns the total cost of the flow
@see NetworkFlowAlgorithm#minCostFlow
@see NetworkFlowAlgorithm#minCostFlow
@see {@link https://docs.yworks.com/yfileshtml/#/api/NetworkFlowAlgorithm%23minCostFlow(Graph%2CYNode%2CYNode%2CIDataProvider%2CIDataProvider%2CIEdgeMap%2CINodeMap) Online Documentation}*/
static minCostFlow(graph: Graph, s: YNode, t: YNode, uCapDP: IDataProvider | null, cost0DP: IDataProvider | null, flowEM: IEdgeMap | null, dualsNM: INodeMap | null): number;
/**
Solves a minimum cost flow problem with a capacity scaling algorithm.
<b>Note:</b> This method works with instances of {@link Graph}. To solve a minimum-cost flow problem on {@link IGraph}
instances use {@link MinimumCostFlow} instead.
This algorithm is a variant of the successive shortest path algorithm (see Ahuja,Magnanti,Orlin: <i>Network flows</i>, Prentice Hall, 1993,
pp.320-324). It has the pseudo-polynomial running time <code>O(m*log U*(m+n log n))</code> where <code>n</code> is the number of nodes in the
network, <code>m</code> the number of edges and <code>U</code> the maximal edge capacity.
Edges may have infinite capacity, which is denoted by the value <code>0x7FFFFFFF</code>.
There are no restriction for the costs. In particular, they can also be negative.
@param graph the given network
@param lCapDP the {@link IDataProvider} that returns the integer lower bound for the capacity of each edge or <code>null</code> if no
bound is specified
@param uCapDP the {@link IDataProvider} that returns the integer upper bound for the capacity of each edge
@param cost0DP the {@link IDataProvider} that returns a double value (cost) of each edge
@param supplyDP the {@link IDataProvider} that returns the supply/demand of each node; supply is denoted by a positive value,
demand by a negative value
@param flowEM the {@link IEdgeMap} that will be filled during the execution with an integer flow for each edge
@param dualsNM the {@link INodeMap} that will be filled during the execution with an integer value (dual value) for each node
or <code>null</code> if no such values occur; dual values are also referred as potentials
@returns the total cost of the flow
@see NetworkFlowAlgorithm#minCostFlow
@see NetworkFlowAlgorithm#minCostFlow
@see {@link https://docs.yworks.com/yfileshtml/#/api/NetworkFlowAlgorithm%23minCostFlow(Graph%2CIDataProvider%2CIDataProvider%2CIDataProvider%2CIDataProvider%2CIEdgeMap%2CINodeMap) Online Documentation}*/
static minCostFlow(graph: Graph | null, lCapDP: IDataProvider | null, uCapDP: IDataProvider | null, cost0DP: IDataProvider | null, supplyDP: IDataProvider | null, flowEM: IEdgeMap | null, dualsNM: INodeMap | null): number;
/**
Uses method {@link NetworkFlowAlgorithm#minCostFlow} to solve a minimum cost flow problem.
<b>Note:</b> This method works with instances of {@link Graph}. To solve a minimum-cost flow problem on {@link IGraph}
instances use {@link MinimumCostFlow} instead.
@param graph the given network
@param uCapDP the {@link IDataProvider} that returns the integer capacity of each edge
@param cost0DP the {@link IDataProvider} that returns a double value (cost) for each edge
@param supplyDP the {@link IDataProvider} that returns the supply/demand of each node; supply is denoted by a positive value,
demand by a negative value
@param flowEM the {@link IEdgeMap} that will be filled during the execution with an integer flow for each edge
@param dualsNM the {@link INodeMap} that will be filled during the execution with an integer value (dual value) for each node
or <code>null</code> if no such values occur; dual values are also referred as potentials
@returns the total cost of the flow
@see NetworkFlowAlgorithm#minCostFlow
@see NetworkFlowAlgorithm#minCostFlow
@see {@link https://docs.yworks.com/yfileshtml/#/api/NetworkFlowAlgorithm%23minCostFlow(Graph%2CIDataProvider%2CIDataProvider%2CIDataProvider%2CIEdgeMap%2CINodeMap) Online Documentation}*/
static minCostFlow(graph: Graph | null, uCapDP: IDataProvider | null, cost0DP: IDataProvider | null, supplyDP: IDataProvider | null, flowEM: IEdgeMap | null, dualsNM: INodeMap | null): number;
static $class: Class;
private $_NetworkFlows: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NetworkFlowAlgorithm;
}
export interface YNode extends GraphObject {
}
/**
Represents a so-called node in the directed graph data type {@link Graph}.
Most notably, a node provides access to its adjacent edges (represented by instances of class
{@link Edge}). These can be distinguished into the sets of incoming and outgoing edges.
Iteration over all three sets of edges is provided by means of bidirectional cursors that present a read-only view of
the respective set ({@link YNode#getEdgeCursor}, {@link YNode#getInEdgeCursor},
{@link YNode#getOutEdgeCursor}). Also supported is iteration over all nodes at opposite ends of either incoming
edges or outgoing edges ({@link YNode#getPredecessorCursor}, {@link YNode#getSuccessorCursor}).
The number of overall edges at a node is called its degree ({@link YNode#degree}), which is the sum of incoming and outgoing edges ({@link YNode#inDegree}, {@link YNode#outDegree}).
<b>Important:</b> Class Graph is the single authority for any structural changes to the graph data type. Specifically, this means that
there is no way to create or delete a node or an edge without using an actual Graph instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode Online Documentation}*/
export class YNode {
/**
Instantiates a new {@link YNode} that will be part of the given graph.
@param g The graph that the created node will belong to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23YNode(Graph) Online Documentation}*/
constructor(g: Graph);
/**
Creates a copy of this node that will be inserted into the given graph.
@param g The graph that the created node will belong to.
@returns The newly created Node object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23createCopy(Graph) Online Documentation}*/
createCopy(g: Graph): YNode | null;
/**
Returns an edge that connects this node with the given node, if such an edge exists.
Otherwise <code>null</code> is returned.
Note that the first matching edge is returned, and that outgoing edges are tested prior to incoming edges.
@param opposite
@see YNode#getEdgeFrom
@see YNode#getEdgeTo
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23getEdge(YNode) Online Documentation}*/
getEdge(opposite: YNode): Edge | null;
/**
Returns an edge cursor for all incoming and outgoing edges at this node.
@see YNode#getInEdgeCursor
@see YNode#getOutEdgeCursor
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23getEdgeCursor() Online Documentation}*/
getEdgeCursor(): IEdgeCursor;
/**
Returns an incoming edge that connects the given node with this node, if such an edge exists.
Otherwise <code>null</code> is returned.
@param source
@see YNode#getEdge
@see YNode#getEdgeTo
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23getEdgeFrom(YNode) Online Documentation}*/
getEdgeFrom(source: YNode): Edge | null;
/**
Returns an outgoing edge that connects this node with the given node, if such an edge exists.
Otherwise <code>null</code> is returned.
@param target
@see YNode#getEdge
@see YNode#getEdgeFrom
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23getEdgeTo(YNode) Online Documentation}*/
getEdgeTo(target: YNode): Edge | null;
/**
Returns an edge cursor for incoming edges at this node.
If an edge is specified, the cursor starts at the given edge, and the cyclic sequence order is the same as returned by
{@link YNode#getInEdgeCursor}.
@param [startEdge=null] The first edge being accessed by the returned cursor.
@see YNode#getOutEdgeCursor
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23getInEdgeCursor(Edge) Online Documentation}*/
getInEdgeCursor(startEdge?: Edge | null): IEdgeCursor;
/**
Returns a node cursor for all neighbor nodes of this node.
Neighbor nodes are those at the opposite ends of both incoming and outgoing edges.
@see YNode#getPredecessorCursor
@see YNode#getSuccessorCursor
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23getNeighborCursor() Online Documentation}*/
getNeighborCursor(): INodeCursor;
/**
Returns an edge cursor for outgoing edges at this node.
If an edge is specified, the cursor starts at the given edge, and the cyclic sequence order is the same as returned by
{@link YNode#getOutEdgeCursor}.
@param [startEdge=null] The first edge being accessed by the returned cursor.
@see YNode#getInEdgeCursor
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23getOutEdgeCursor(Edge) Online Documentation}*/
getOutEdgeCursor(startEdge?: Edge | null): IEdgeCursor;
/**
Returns a node cursor for all predecessor nodes of this node.
Predecessor nodes are those at the opposite ends of incoming edges.
@see YNode#getSuccessorCursor
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23getPredecessorCursor() Online Documentation}*/
getPredecessorCursor(): INodeCursor;
/**
Returns a node cursor for all successor nodes of this node.
Successor nodes are those at the opposite ends of outgoing edges.
@see YNode#getPredecessorCursor
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23getSuccessorCursor() Online Documentation}*/
getSuccessorCursor(): INodeCursor;
/**
Sorts incoming edges at this node according to the given comparator.
@param c
@see YNode#sortOutEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23sortInEdges(IComparer%3CObject%3E) Online Documentation}*/
sortInEdges(c: IComparer<any> | null): void;
/**
Sorts outgoing edges at this node according to the given comparator.
@param c
@see YNode#sortInEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23sortOutEdges(IComparer%3CObject%3E) Online Documentation}*/
sortOutEdges(c: IComparer<any> | null): void;
/**
Returns a String representation of this node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23toString() Online Documentation}*/
toString(): string;
/**
Gets the overall number of incoming and outgoing edges at this node.
Note that self-loops are counted twice.
@see Edge
@see YNode#inDegree
@see YNode#outDegree
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23degree Online Documentation}*/
readonly degree: number;
/**
Gets the number of incoming edges at this node.
@see YNode#degree
@see YNode#outDegree
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23inDegree Online Documentation}*/
readonly inDegree: number;
/**
Gets the number of outgoing edges at this node.
@see YNode#degree
@see YNode#inDegree
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23outDegree Online Documentation}*/
readonly outDegree: number;
/**
Gets the index of this node within its graph G.
Node indices represent the ordering of standard node iteration on G. The value of an index is <code> >= 0</code> and
<code> &lt; G.nodeCount()</code>.
Note that indices are subject to change whenever the sequence of nodes in a graph is modified by either removing,
hiding, reinserting, or unhiding a node, or by explicitly changing its position in the sequence.
@see Graph#removeNode
@see Graph#hide
@see Graph#reInsertNode
@see Graph#unhide
@see Graph#moveToFirst
@see Graph#moveToLast
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23index Online Documentation}*/
readonly index: number;
/**
Gets the graph this node belongs to.
If the node does not belong to a graph, because it was removed or hidden from it, this method returns <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23graph Online Documentation}*/
readonly graph: Graph | null;
/**
Gets the first outgoing edge at this node, or <code>null</code> if it does not exist.
@see YNode#firstInEdge
@see YNode#lastOutEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23firstOutEdge Online Documentation}*/
readonly firstOutEdge: Edge | null;
/**
Gets the first incoming edge at this node, or <code>null</code> if it does not exist.
@see YNode#firstOutEdge
@see YNode#lastInEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23firstInEdge Online Documentation}*/
readonly firstInEdge: Edge | null;
/**
Gets the last outgoing edge at this node, or <code>null</code> if it does not exist.
@see YNode#firstOutEdge
@see YNode#lastInEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23lastOutEdge Online Documentation}*/
readonly lastOutEdge: Edge | null;
/**
Gets the last incoming edge at this node, or <code>null</code> if it does not exist.
@see YNode#firstInEdge
@see YNode#lastOutEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23lastInEdge Online Documentation}*/
readonly lastInEdge: Edge | null;
/**
Yields a dynamic {@link IEnumerable} for {@link Edge}s that can be used to iterate over outgoing edges at this instance.
This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
graph structure during the traversal should be carried out with great care. Note that self-loop edges are reported,
too.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23outEdges Online Documentation}*/
readonly outEdges: IEnumerable<Edge> | null;
/**
Yields a dynamic {@link IEnumerable} for {@link Edge}s that can be used to iterate over ingoing edges at this instance.
This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
graph structure during the traversal should be carried out with great care. Note that self-loop edges are reported,
too.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23inEdges Online Documentation}*/
readonly inEdges: IEnumerable<Edge> | null;
/**
Yields a dynamic {@link IEnumerable} for {@link Edge}s that can be used to iterate over the adjacent edges at this instance.
This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
graph structure during the traversal should be carried out with great care. Note that self-loop edges are reported twice
(as in edge and as out edge).
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23edges Online Documentation}*/
readonly edges: IEnumerable<Edge> | null;
/**
Yields a dynamic {@link IEnumerable} for
{@link YNode}s that can be used to iterate over the opposite sides of adjacent outgoing edges at this
instance.
This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
graph structure during the traversal should be carried out with great care. Note that for self-loop edges this node
itself will be reported as a successor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23successors Online Documentation}*/
readonly successors: IEnumerable<YNode> | null;
/**
Yields a dynamic {@link IEnumerable} for
{@link YNode}s that can be used to iterate over the opposite sides of adjacent incoming edges at this
instance.
This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
graph structure during the traversal should be carried out with great care. Note that for self-loop edges this node
itself will be reported as a predecessor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23predecessors Online Documentation}*/
readonly predecessors: IEnumerable<YNode> | null;
/**
Yields a dynamic {@link IEnumerable} for
{@link YNode}s that can be used to iterate over the opposite sides of adjacent adjacent edges at this
instance.
This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
graph structure during the traversal should be carried out with great care. Note that for self-loop edges this node
itself will be reported as a neighbor, twice.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNode%23neighbors Online Documentation}*/
readonly neighbors: IEnumerable<YNode> | null;
static $class: Class;
private $_Node: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YNode;
}
export interface NodeAggregationAlgorithm extends YObject {
}
/**
This class realizes an algorithm that {@link NodeAggregationAlgorithm#aggregate aggregates nodes} and creates a hierarchical clustering structure subject to
user-specified constraints like the {@link NodeAggregationAlgorithm#nodeTypeHandlingPolicy type of nodes} as well as the preferred {@link NodeAggregationAlgorithm#minimumClusterSize minimum} and {@link NodeAggregationAlgorithm#maximumClusterSize maximum} size of a cluster.
The result of the aggregation can be used to (interactively) visualize parts of large graphs.
Note that the resulting clustering structure corresponds to a directed rooted tree which we encode by means of a set of
{@link AggregationInfo} instances. More precisely, each node of the original graph is mapped to a unique {@link AggregationInfo}
instance. This mapping is stored in the {@link INodeMap} that is passed to method
{@link NodeAggregationAlgorithm#aggregate}. Each {@link AggregationInfo} has a reference to its {@link AggregationInfo#parentAggregation parent} which
induces a directed tree structure. There is always exactly one {@link AggregationInfo} without a parent that
represents the root of the tree. Property {@link NodeAggregationAlgorithm#nodesOnlyOnLeaves} allows to specify whether nodes are only mapped to leaves of the tree
structure or if they can also be mapped to inner nodes.
Property {@link NodeAggregationAlgorithm#aggregationPolicy} allows to specify whether the algorithm should consider {@link NodeAggregationPolicy#STRUCTURAL structural properties} (i.e., considering the
connectivity) for the aggregation or {@link NodeAggregationPolicy#GEOMETRIC geometric properties} (i.e., the distance between nodes). For the second policy, the
coordinates of the nodes have to be specified by the user by means of a {@link IDataProvider} registered with
key {@link NodeAggregationAlgorithm#NODE_CENTER_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm Online Documentation}*/
export class NodeAggregationAlgorithm {
/**
Creates a new {@link NodeAggregationAlgorithm} instance with default settings.
@param [options] The parameters to pass.
@param [options.nodesOnlyOnLeaves] Whether or not all original nodes are only mapped to leaves of the directed rooted aggregation tree that represents the hierarchical clustering structure. This option sets the {@link NodeAggregationAlgorithm#nodesOnlyOnLeaves} property on the created object.
@param [options.maximumDuration] The maximum duration in milliseconds that this layout algorithm is allowed to run. This option sets the {@link NodeAggregationAlgorithm#maximumDuration} property on the created object.
@param [options.nodeTypeHandlingPolicy] The policy for handling nodes of different type. This option sets the {@link NodeAggregationAlgorithm#nodeTypeHandlingPolicy} property on the created object.
@param [options.aggregationPolicy] The policy applied for determining the clusters. This option sets the {@link NodeAggregationAlgorithm#aggregationPolicy} property on the created object.
@param [options.multiThreadingAllowed] Whether or not the layout algorithm may use multi-threading to reduce the running time. This option sets the {@link NodeAggregationAlgorithm#multiThreadingAllowed} property on the created object.
@param [options.minimumClusterSize] The preferred minimum number of elements contained in a cluster. This option sets the {@link NodeAggregationAlgorithm#minimumClusterSize} property on the created object.
@param [options.maximumClusterSize] The preferred maximum number of elements contained in a cluster. This option sets the {@link NodeAggregationAlgorithm#maximumClusterSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm Online Documentation}*/
constructor(options?: {
nodesOnlyOnLeaves?: boolean;
maximumDuration?: number;
nodeTypeHandlingPolicy?: NodeAggregationNodeTypeHandlingPolicy | NodeAggregationNodeTypeHandlingPolicyStringValues;
aggregationPolicy?: NodeAggregationPolicy | NodeAggregationPolicyStringValues;
multiThreadingAllowed?: boolean;
minimumClusterSize?: number;
maximumClusterSize?: number;
});
/**
Starts the node aggregation for the specified graph.
The aggregation result corresponds to a directed rooted tree which we encode by means of a set of {@link AggregationInfo}
instances. More precisely, each node of the original graph is mapped to a unique {@link AggregationInfo}
instance that is stored in the given {@link INodeMap} <code>aggregationResult</code>. Each {@link AggregationInfo}
has a reference to its {@link AggregationInfo#parentAggregation parent} which induces a directed tree structure. There is always exactly one {@link AggregationInfo}
without a parent that represents the root of the tree.
@param graph the input graph
@param aggregationResult stores the result of the aggregation
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23aggregate(Graph%2CINodeMap) Online Documentation}*/
aggregate(graph: Graph | null, aggregationResult: INodeMap | null): void;
/**
Data provider key for specifying the top-level nodes of the aggregation info
Top-level nodes are directly contained in the root cluster of the aggregation result or in direct children of the root
if there are top-level nodes with different types (nodes of different type are not allowed to be directly contained in
the same cluster). The only exception to this rules is if there are
{@link GroupingKeys#GROUP_DP_KEY group nodes}, because a group is a kind of user-specified cluster and, thus, a top-level node contained in a group is
always placed in the cluster associated with that group.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23TOP_LEVEL_NODES_DP_KEY Online Documentation}*/
static TOP_LEVEL_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for specifying the coordinates of the nodes
If the {@link NodeAggregationAlgorithm#aggregationPolicy aggregation policy} is set to
{@link NodeAggregationPolicy#GEOMETRIC}, these coordinates are considered during the aggregation of nodes.
<h2>Note</h2>
The coordinates are only considered if the {@link NodeAggregationAlgorithm#aggregationPolicy aggregation policy} is set to
{@link NodeAggregationPolicy#GEOMETRIC}.
@see NodeAggregationAlgorithm#aggregationPolicy
@see NodeAggregationPolicy#GEOMETRIC
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23NODE_CENTER_DP_KEY Online Documentation}*/
static NODE_CENTER_DP_KEY: NodeDpKey<YPoint> | null;
/**
Data provider key for specifying the type of the nodes
Nodes mapped to equal objects are considered to be of the same type. Property {@link NodeAggregationAlgorithm#nodeTypeHandlingPolicy} specifies how the algorithm should handle
nodes of different type.
@see NodeAggregationAlgorithm#nodeTypeHandlingPolicy
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23NODE_TYPE_DP_KEY Online Documentation}*/
static NODE_TYPE_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for specifying the (non-negative) weights of the nodes
Depending on the exact setting, the algorithm applies multiple different aggregation approaches and chooses the best
one. If nodes have weights, it prefers aggregation results where nodes with higher weight are closer to the root of the
aggregation hierarchy.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23NODE_WEIGHT_DP_KEY Online Documentation}*/
static NODE_WEIGHT_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for specifying the (non-negative) weights of the edges
If the {@link NodeAggregationAlgorithm#aggregationPolicy aggregation policy} is set to
{@link NodeAggregationPolicy#STRUCTURAL}, nodes connected by edges with high weights are more likely to be
clustered together.
<h2>Note</h2>
Edge weights are only considered if the {@link NodeAggregationAlgorithm#aggregationPolicy aggregation policy} is set to
{@link NodeAggregationPolicy#STRUCTURAL}.
@see NodeAggregationAlgorithm#aggregationPolicy
@see NodeAggregationPolicy#STRUCTURAL
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23EDGE_WEIGHT_DP_KEY Online Documentation}*/
static EDGE_WEIGHT_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for specifying the directedness of edges
Generally, for this node aggregation algorithm, the effect of specifying an edge directedness is quite low. It is mainly
used for the detection of substructures, see
{@link Substructures}. A substructure is only identified as such if all edges are either undirected or
consistently directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the algorithm assumes that all edges have
directedness <code>0.0</code>.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23EDGE_DIRECTEDNESS_DP_KEY Online Documentation}*/
static EDGE_DIRECTEDNESS_DP_KEY: EdgeDpKey<number> | null;
/**
Gets or sets whether or not all original nodes are only mapped to leaves of the directed rooted aggregation tree that
represents the hierarchical clustering structure.
If this property is disabled, nodes can also represent inner nodes and, thus, contain other node elements. This may
allow a more efficient representation of the resulting tree structure. For example, for a star-like structure in the
input graph, the root of the star can directly represent the parent aggregate of the star. Otherwise, an additional
virtual tree node must be inserted.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23nodesOnlyOnLeaves Online Documentation}*/
nodesOnlyOnLeaves: boolean;
/**
Gets or sets the maximum duration in milliseconds that this layout algorithm is allowed to run.
The duration needs to be non-negative.
<h2>Note</h2>
Restricting the maximum duration may result in a lower aggregation quality. Furthermore, the real runtime may exceed the
maximum duration since the algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the specified duration has a negative value
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the policy for handling nodes of different type.
@throws {Exception} <b>(ArgumentError)</b> if an unknown policy is provided
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23nodeTypeHandlingPolicy Online Documentation}*/
nodeTypeHandlingPolicy: NodeAggregationNodeTypeHandlingPolicy;
/**
Gets or sets the policy applied for determining the clusters.
@throws {Exception} <b>(ArgumentError)</b> if an unknown policy is provided
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23aggregationPolicy Online Documentation}*/
aggregationPolicy: NodeAggregationPolicy;
/**
Gets or sets whether or not the layout algorithm may use multi-threading to reduce the running time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23multiThreadingAllowed Online Documentation}*/
multiThreadingAllowed: boolean;
/**
Gets or sets the preferred minimum number of elements contained in a cluster.
The minimum allowed value for this property is <code>1</code>.
<h2>Note</h2>
The algorithm doesn't guarantee that the found solution observes this value.
@throws {Exception} <b>(ArgumentError)</b> if the minimum cluster size is less than 1
@see NodeAggregationAlgorithm#maximumClusterSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23minimumClusterSize Online Documentation}*/
minimumClusterSize: number;
/**
Gets or sets the preferred maximum number of elements contained in a cluster.
The minimum allowed value for this property is <code>2</code>.
<h2>Note</h2>
The algorithm doesn't guarantee that the found solution observes this value.
@throws {Exception} <b>(ArgumentError)</b> if the maximum cluster size is less than 2
@see NodeAggregationAlgorithm#minimumClusterSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationAlgorithm%23maximumClusterSize Online Documentation}*/
maximumClusterSize: number;
static $class: Class;
private $_NodeAggregation: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeAggregationAlgorithm;
}
export interface YNodeList extends YList {
}
/**
Specialized list implementation for instances of type {@link YNode}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList Online Documentation}*/
export class YNodeList {
/**
Creates a list that is initialized with the entries of the given list.
@param list the values are added to the new list
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23YNodeList(YNodeList) Online Documentation}*/
constructor(list: YNodeList);
/**
Creates a list that is initialized with a single node provided.
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23YNodeList(YNode) Online Documentation}*/
constructor(v: YNode);
/**
Creates a list that is initialized with the nodes provided by the given array of nodes.
@param a
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23YNodeList(Array%3CYNode%3E) Online Documentation}*/
constructor(a: YNode[]);
/**
Creates a list that is initialized with those nodes from the given NodeCursor object for which the given data provider
returns <code>true</code> upon calling its {@link IDataProvider#getBoolean getBool} method.
@param nc A node cursor providing nodes that should be added to this list.
@param predicate A data provider that acts as a inclusion predicate for each node accessible by the given node cursor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23YNodeList(INodeCursor%2CIDataProvider) Online Documentation}*/
constructor(nc: INodeCursor, predicate: IDataProvider);
/**
Creates a list that is initialized with the nodes provided by the given NodeCursor object.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23YNodeList(INodeCursor) Online Documentation}*/
constructor(c: INodeCursor);
/**
Creates an empty node list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23YNodeList() Online Documentation}*/
constructor();
/**
Returns the first node in this list, or <code>null</code> when the list is empty.
@returns The first node in the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23firstNode() Online Documentation}*/
firstNode(): YNode | null;
/**
Returns an enumerator for this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23getEnumerator() Online Documentation}*/
getEnumerator(): IEnumerator<YNode>;
/**
Returns the last node in this list, or <code>null</code> when the list is empty.
@returns The last node in the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23lastNode() Online Documentation}*/
lastNode(): YNode | null;
/**
Returns a node cursor for this node list.
@returns A node cursor granting access to the nodes within this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23nodes() Online Documentation}*/
nodes(): INodeCursor;
/**
Removes the first node from this list and returns it.
@returns The first node from the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23popNode() Online Documentation}*/
popNode(): YNode | null;
/**
Returns a node array containing all elements of this list in the canonical order.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNodeList%23toNodeArray() Online Documentation}*/
toNodeArray(): YNode[];
static $class: Class;
private $_NodeList: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YNodeList;
}
export interface NodeOrderAlgorithm extends YObject {
}
/**
This class provides algorithms that order the nodes of a graph using specific criteria.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To retrieve a topological order of nodes in an {@link IGraph}
use {@link GraphStructureAnalyzer#getTopologicalOrder} instead.
<b>Definitions</b>
<ul>
<li><b>A topological ordering</b> of the nodes of a directed graph is a linear ordering of the nodes such that for each directed edge <code>(u,v)</code>, node <code>u</code> lies
before <code>v</code> in the ordering.
<li>A <b>DFS completion ordering</b> of the nodes of a graph is a node ordering identical to the order of node completion events in a {@link DfsAlgorithm depth first search}.
<li>An <b>st-ordering</b> <code>(v_1,v_2,....,v_n)</code> of a biconnected graph is a node ordering which guarantees that:
<ul>
<li>Source node <code>s</code> and sink node <code>t</code> are connected by an edge.
<li>For each node <code>v_i</code> in the ordering other than <code>s</code> or <code>t</code>, there are neighbors <code>v_j</code> and <code>v_k</code> with <code>j &lt; i</code> and <code>k > i</code>.
</ul>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm Online Documentation}*/
export class NodeOrderAlgorithm {
/**
Calculates an ordering of the nodes identical to the order of node completion events in a depth first search.
This ordering is a reversed topological ordering in case the input graph is acyclic.
@param graph the input graph
@param order an array of {@link number}s that returns for each {@link YNode} <code>v</code>, its zero-based index within the calculated ordering, i.e.,
<code>order[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
@see NodeOrderAlgorithm#topological
@see NodeOrderAlgorithm#dfsCompletion
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm%23dfsCompletion(Graph%2CArray%3Cnumber%3E) Online Documentation}*/
static dfsCompletion(graph: Graph | null, order: number[] | null): void;
/**
Calculates an ordering of the nodes identical to the order of node completion events in a depth first search.
Like {@link NodeOrderAlgorithm#dfsCompletion} but the result is returned as a {@link YNodeList}.
This ordering is a reversed topological ordering in case the input graph is acyclic.
@param graph the input graph
@returns a {@link YNodeList} containing the nodes of the graph in the order identical to the order of node completion
events in a depth first search
@see NodeOrderAlgorithm#topological
@see NodeOrderAlgorithm#dfsCompletion
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm%23dfsCompletion(Graph) Online Documentation}*/
static dfsCompletion(graph: Graph | null): YNodeList | null;
/**
Assigns an <code>st</code>-ordering to the nodes of a biconnected graph given the edge between source node <code>s</code> and sink node <code>t</code>.
An <b><code>st</code>-ordering</b> <code>(v_1,v_2,....,v_n)</code> of a biconnected graph is a node ordering which guarantees that:
<ul>
<li>Source node <code>s</code> and sink node <code>t</code> are connected by an edge.
<li>For each node <code>v_i</code> in the ordering other than <code>s</code> or <code>t</code>, there are neighbors <code>v_j</code> and <code>v_k</code> with <code>j &lt; i</code> and <code>k > i</code>.
</ul>
@param graph the input graph
@param stOrder an array of {@link number}s that will be filled during the execution and returns for each {@link YNode} <code>v</code>, its zero-based
index within the calculated ordering, i.e., <code>stOrder[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
@param [stEdge=null] an {@link Edge} that connects source node <code>s</code> and sink node <code>t</code>
@see NodeOrderAlgorithm#st
@see NodeOrderAlgorithm#st
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm%23st(Graph%2CArray%3Cnumber%3E%2CEdge) Online Documentation}*/
static st(graph: Graph | null, stOrder: number[] | null, stEdge?: Edge | null): void;
/**
Assigns an <code>st</code>-ordering to the nodes of a biconnected graph.
Like {@link NodeOrderAlgorithm#st} but the result is returned as a {@link YNodeList}.
An <b><code>st</code>-ordering</b> <code>(v_1,v_2,....,v_n)</code> of a biconnected graph is a node ordering which guarantees that:
<ul>
<li>Source node <code>s</code> and sink node <code>t</code> are connected by an edge.
<li>For each node <code>v_i</code> in the ordering other than <code>s</code> or <code>t</code>, there are neighbors <code>v_j</code> and <code>v_k</code> with <code>j &lt; i</code> and <code>k > i</code>.
</ul>
@param graph the input graph
@returns a {@link YNodeList} containing the nodes of the graph in the order defined by the <code>st</code>-ordering
@see NodeOrderAlgorithm#st
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm%23st(Graph) Online Documentation}*/
static st(graph: Graph | null): YNodeList | null;
/**
Converts an array-based result returned by a method of this class to a {@link YNodeList} that contains all
nodes in the order provided by the given array.
@param graph the input graph
@param order an array of {@link number}s that will be filled during the execution and returns for each {@link YNode} <code>v</code>, its zero-based
index within the calculated ordering, i.e., <code>order[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
@returns a {@link YNodeList} containing the nodes of the graph in the order provided by the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm%23toNodeList(Graph%2CArray%3Cnumber%3E) Online Documentation}*/
static toNodeList(graph: Graph | null, order: number[] | null): YNodeList | null;
/**
Copies an array-based result returned by a method of this class to a {@link INodeMap} that will provide values
of basic type <code>int</code>.
@param graph the input graph
@param order an array of {@link number}s that returns for each {@link YNode} <code>v</code>, its zero-based index within the calculated ordering, i.e.,
<code>order[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
@param result the {@link INodeMap} that will be filled during the execution with the zero-based index of each node within
the calculated ordering
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm%23toNodeMap(Graph%2CArray%3Cnumber%3E%2CINodeMap) Online Documentation}*/
static toNodeMap(graph: Graph | null, order: number[] | null, result: INodeMap | null): void;
/**
Copies a {@link YNodeList}-based result returned by a method of this class to a {@link INodeMap} that
will provide values of basic type <code>int</code>.
@param order a {@link YNodeList} containing the nodes of the graph in the appropriate order
@param result the {@link INodeMap} that will be filled during the execution with the zero-based index of each node within
the calculated ordering
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm%23toNodeMap(YNodeList%2CINodeMap) Online Documentation}*/
static toNodeMap(order: YNodeList | null, result: INodeMap | null): void;
/**
Assigns a topological ordering to the nodes of a directed acyclic graph.
<b>Note:</b> This method works with instances of {@link Graph}. To retrieve a topological order of nodes in an {@link IGraph}
use {@link GraphStructureAnalyzer#getTopologicalOrder} instead.
A topological ordering of the nodes of a directed graph is a linear ordering of the nodes such that for each directed
edge <code>(u,v)</code>, node <code>u</code> lies before <code>v</code> in the ordering.
<b>Warning:</b> If the given graph is not acyclic, this method returns <code>false</code> leaving the contents of the result array <code>order</code>
unspecified.
@param graph the input graph
@param order an array of {@link number}s that will be filled during the execution and returns for each {@link YNode} <code>v</code>, its zero-based
index within the calculated ordering, i.e., <code>order[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
@returns <code>true</code> if the graph is acyclic, <code>false</code> otherwise
@see NodeOrderAlgorithm#topological
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm%23topological(Graph%2CArray%3Cnumber%3E) Online Documentation}*/
static topological(graph: Graph | null, order: number[] | null): boolean;
/**
Returns a topological ordering of the nodes of a directed acyclic graph.
<b>Note:</b> This method works with instances of {@link Graph}. To retrieve a topological order of nodes in an {@link IGraph}
use {@link GraphStructureAnalyzer#getTopologicalOrder} instead.
@param graph the input graph
@returns a {@link YNodeList} containing the nodes of the graph in the order they appear in the topological ordering
@throws {Exception} <b>(ArgumentError)</b> if the graph is cyclic
@see NodeOrderAlgorithm#topological
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderAlgorithm%23topological(Graph) Online Documentation}*/
static topological(graph: Graph | null): YNodeList | null;
static $class: Class;
private $_NodeOrders: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeOrderAlgorithm;
}
export interface PathAlgorithm extends YObject {
}
/**
This class provides methods for finding paths within a graph that have certain properties.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To work with various types of paths in an {@link IGraph}
use one of the following classes instead:
<ul>
<li>{@link Chains}
<li>{@link LongestPath}
<li>{@link Paths}
</ul>
<b>Definitions</b>
<ul>
<li>A <b>path</b> of length <code>k</code> from node <code>u</code> to node <code>v</code> in a graph is a sequence <code>v0, v1, v2, ... , vk</code> of nodes such that <code>u = v0, v = vk</code> and
<code>(vi-1, vi)</code> in <code>E</code>, for each <code>i = 1, 2, .. , k</code>.
<li>A path is called <b>simple</b> if no node appears twice.
<li>A <b>chain</b> is a path of maximum length in which each internal node has degree <code>2</code>.
<li>The <b>longest path problem</b> is the problem of finding a simple path of maximum length in a given graph. The longest path problem is
<code>NP</code>-hard for undirected graphs, but can be solved in linear time for directed acyclic graphs.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm Online Documentation}*/
export class PathAlgorithm {
/**
Constructs a {@link YNodeList path of nodes} from a given {@link EdgeList path of edges}.
The returned node path has length <code>path.size()+1</code>, if the given path is not empty. Otherwise, the returned path will be
empty. The <code>i</code>-th node in the returned path will be either source or target node of the <code>i</code>-th edge in the given path.
@param path the given {@link EdgeList path of edges}
@returns a {@link YNodeList path of nodes} from the given {@link EdgeList path of edges}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23constructNodePath(EdgeList) Online Documentation}*/
static constructNodePath(path: EdgeList | null): YNodeList | null;
/**
Returns all chains present in the given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To find all chains in an {@link IGraph} use {@link Chains}
instead.
A chain is a path of maximum length in which each internal node has degree <code>2</code>.
The internal nodes on directed chains all have in-degree <code>1</code> and out-degree <code>1</code>.
<h2>Note</h2>
Method {@link PathAlgorithm#constructNodePath} can be used for converting an edge path to a node path.
@param graph the input graph
@param directed <code>true</code> if the chain should be considered as directed, <code>false</code> otherwise
@returns an array of {@link EdgeList}s each of which contains the edges (at least two) that make up a chain
@see PathAlgorithm#constructNodePath
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findAllChains(Graph%2Cboolean) Online Documentation}*/
static findAllChains(graph: Graph | null, directed: boolean): EdgeList[] | null;
/**
A variant of {@link PathAlgorithm#findAllPaths} which returns all simple directed or undirected paths between two
given nodes and, additionally, allows to specify a filter for the paths to be returned.
<b>Note:</b> This method works with instances of {@link Graph}. To find all paths between two nodes in an {@link IGraph}
use {@link Paths} instead.
@param graph the input graph
@param startNode the given start node
@param endNode the given end node
@param directed <code>true</code> if the path should be considered as directed, <code>false</code> otherwise
@param filter a predicate that accepts or rejects a found {@link EdgeList} and adds it to the result
@returns an array of {@link EdgeList}s each of which represents a path between the start and end node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findAllPaths(Graph%2CYNode%2CYNode%2Cboolean%2Cfunction(EdgeList)%3Aboolean) Online Documentation}*/
static findAllPaths(graph: Graph | null, startNode: YNode | null, endNode: YNode | null, directed: boolean, filter: ((obj: EdgeList) => boolean) | null): EdgeList[] | null;
/**
Returns all simple directed or undirected paths that connect a <code>start</code> node with an <code>end</code> node.
<b>Note:</b> This method works with instances of {@link Graph}. To find all paths between two nodes in an {@link IGraph}
use {@link Paths} instead.
@param graph the input graph
@param startNode the given start node
@param endNode the given end node
@param directed <code>true</code> if the path should be considered as directed, <code>false</code> otherwise
@returns an array of {@link EdgeList}s each of which represents a path between the start and end node
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findAllPaths(Graph%2CYNode%2CYNode%2Cboolean) Online Documentation}*/
static findAllPaths(graph: Graph | null, startNode: YNode | null, endNode: YNode | null, directed: boolean): EdgeList[] | null;
/**
Finds all edges that belong to a directed path from a <code>start</code> node to an <code>end</code> node.
<b>Note:</b> This method works with instances of {@link Graph}. To find all paths between two nodes in an {@link IGraph}
use {@link Paths} instead.
@param graph the input graph
@param startNode the given start node
@param endNode the given end node
@param pathEdges the {@link IEdgeMap} that will be filled during the execution with a boolean value indicating whether or not
an edge belongs to a path connecting the two given nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findAllPaths(Graph%2CYNode%2CYNode%2CIEdgeMap) Online Documentation}*/
static findAllPaths(graph: Graph | null, startNode: YNode | null, endNode: YNode | null, pathEdges: IEdgeMap | null): void;
/**
A variant of
{@link PathAlgorithm#findAllPaths}, which returns all simple directed or undirected paths between two given nodes as a
special cursor that calculates the next path in the sequence, only when needed.
<b>Note:</b> This method works with instances of {@link Graph}. To find all paths between two nodes in an {@link IGraph}
use {@link Paths} instead.
The returned cursor only supports the operation {@link ICursor#ok}, {@link ICursor#current}, {@link ICursor#size} and {@link ICursor#next}.
@param graph the input graph
@param startNode the given start node
@param endNode the given end node
@param directed <code>true</code> if the path should be considered as directed, <code>false</code> otherwise
@returns a {@link ICursor} that calculates the next path in the sequence
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findAllPathsCursor(Graph%2CYNode%2CYNode%2Cboolean) Online Documentation}*/
static findAllPathsCursor(graph: Graph | null, startNode: YNode | null, endNode: YNode | null, directed: boolean): ICursor | null;
/**
Returns the longest directed path in a given acyclic weighted graph.
<b>Note:</b> This method works with instances of {@link Graph}. To find the longest path in an {@link IGraph} use {@link LongestPath}
with negative weights instead.
All edges of the graph have an integral length associated with them. The longest path is defined as one of all directed
paths within the graph for which the edge lengths of all contained edges sum up to a maximum.
@param graph a directed acyclic graph
@param [edgeLength=null] the {@link IDataProvider} that returns the non-negative integer length of each edge
@returns an {@link EdgeList} containing the edges of the longest directed path
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findLongestPath(Graph%2CIDataProvider) Online Documentation}*/
static findLongestPath(graph: Graph | null, edgeLength?: IDataProvider | null): EdgeList | null;
/**
Calculates the longest path from a given node to all other node in a given directed acyclic graph.
<b>Note:</b> This method works with instances of {@link Graph}. To find the longest paths from one node to all other nodes
in an {@link IGraph} use {@link SingleSourceShortestPaths} with negative edge costs instead.
@param graph a directed acyclic graph
@param startNode the node for which the distances are calculated
@param dist the {@link IEdgeMap} that returns the distance (i.e. weight) of type double for each edge
@param maxDist the {@link INodeMap} that will be filled during the execution and holds the maximum distance between the given
node and all other nodes
@param predicate the {@link IEdgeMap} that returns a boolean value indicating whether or not an edge should be considered
during the path search
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findLongestPaths(Graph%2CYNode%2CIEdgeMap%2CINodeMap%2CIEdgeMap) Online Documentation}*/
static findLongestPaths(graph: Graph | null, startNode: YNode | null, dist: IEdgeMap | null, maxDist: INodeMap | null, predicate: IEdgeMap | null): void;
/**
Returns an {@link EdgeList} containing the edges of an undirected simple path within the given graph.
The edges are returned in the order that they appear in the found path.
A heuristic is used for finding a path that is long. It is not guaranteed, though, that the returned path is actually
the longest path within the given graph, since that is a well known hard problem.
@param graph the given graph
@returns an {@link EdgeList} containing the edges of an undirected simple path
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findLongPath(Graph) Online Documentation}*/
static findLongPath(graph: Graph | null): EdgeList | null;
/**
Returns whether or not a directed path from a start node to another node in an acyclic graph exists.
<b>Note:</b> This method works with instances of {@link Graph}. To find a path between two nodes in an {@link IGraph}
use {@link Paths} instead.
@param graph an acyclic graph which contains the two nodes
@param topSort a {@link YNodeList list} of nodes sorted in topological order
@param predicate the {@link IEdgeMap} that returns a boolean value indicating whether or not an edge should be considered
during the path search
@param startNode
@param endNode
@returns <code>true</code> if a directed path from a start node to another node exists, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findPath(Graph%2CYNodeList%2CYNode%2CYNode%2CIEdgeMap) Online Documentation}*/
static findPath(graph: Graph | null, topSort: YNodeList | null, startNode: YNode | null, endNode: YNode | null, predicate: IEdgeMap | null): boolean;
/**
Returns an {@link EdgeList} containing the edges of a path from the given start node to the given end node, if
such a path exists.
<b>Note:</b> This method works with instances of {@link Graph}. To find a path between two nodes in an {@link IGraph}
use {@link Paths} instead.
The edges are returned in the order that they appear in the found path. If the returned path is empty, no path between
the given nodes was found.
@param graph the input graph
@param startNode the first node of the path
@param endNode the last node of the path
@param directed <code>true</code> if the path should be directed, <code>false</code> otherwise
@returns an {@link EdgeList} containing the path edges between the start node and the end node
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathAlgorithm%23findPath(Graph%2CYNode%2CYNode%2Cboolean) Online Documentation}*/
static findPath(graph: Graph | null, startNode: YNode | null, endNode: YNode | null, directed: boolean): EdgeList | null;
static $class: Class;
private $_Paths: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PathAlgorithm;
}
export interface PlanarEmbedding extends YObject {
}
/**
Represents an embedding of a planar graph.
A planar embedding is represented by a {@link PlanarEmbedding#faces list of faces}. Each face is represented by a list of {@link Dart darts} which describe the
border of the face.
<b>Warning:</b> Querying the planar embedding after changes to the underlying graph structure may produce exceptions.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlanarEmbedding Online Documentation}*/
export class PlanarEmbedding {
/**
Creates a new embedding for the specified planar graph.
@param graph a planar graph
@throws {Exception} <b>(ArgumentError)</b> if the specified graph is not planar
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlanarEmbedding Online Documentation}*/
constructor(graph: Graph | null);
/**
Returns the cyclic next dart of the given dart.
@param dart a dart
@returns the cyclic next dart of the given dart
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlanarEmbedding%23getCyclicNext(Dart) Online Documentation}*/
getCyclicNext(dart: Dart | null): Dart | null;
/**
Returns the cyclic previous dart of the given dart.
@param dart a dart
@returns the cyclic previous dart of the given dart
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlanarEmbedding%23getCyclicPrevious(Dart) Online Documentation}*/
getCyclicPrevious(dart: Dart | null): Dart | null;
/**
Returns the two darts associated with the given edge.
@param edge an edge
@returns the two darts associated with the given edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlanarEmbedding%23getDarts(Edge) Online Documentation}*/
getDarts(edge: Edge | null): Dart[] | null;
/**
Returns a {@link IList} containing the (cyclic ordered) outgoing darts of the specified node.
@param node a node
@returns a {@link IList} containing the (cyclic ordered) outgoing darts
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlanarEmbedding%23getOutgoingDarts(YNode) Online Documentation}*/
getOutgoingDarts(node: YNode | null): IList<Dart> | null;
/**
Return whether or not the given graph is planar.
@param graph the input graph
@returns <code>true</code> if the given graph is planar, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlanarEmbedding%23isPlanar(Graph) Online Documentation}*/
static isPlanar(graph: Graph | null): boolean;
/**
Gets a {@link IList} of face lists.
More precisely, each entry of this list is another list that contains all {@link Dart darts} of a face.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlanarEmbedding%23faces Online Documentation}*/
readonly faces: IList<IList<Dart>> | null;
/**
Gets the {@link IList} that represents the outer face.
<h2>Note</h2>
The returned list is also contained in the {@link PlanarEmbedding#faces face lists}.
@see PlanarEmbedding#faces
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlanarEmbedding%23outerFace Online Documentation}*/
readonly outerFace: IList<Dart> | null;
static $class: Class;
private $_PlanarEmbedding: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PlanarEmbedding;
}
export interface RankAssignmentAlgorithm extends YObject {
}
/**
This class provides algorithms for solving the rank assignment problem.
<b>Definitions</b>
Let <code>G=(V,E)</code> be a directed acyclic graph. Let <code>length(e)</code> denote the minimum length and <code>weight(e)</code> the weight of an edge <code>e</code>.
The <b>rank assignment problem</b> is the problem of finding integer values <code>rank(v)</code> for all <code>v</code> in <code>V</code>, such that:
<ul>
<li><code>rank(v) - rank(w) >= length(v,w)</code>, for all <code>(v,w)</code> in <code>E</code> and,
<li>the sum <code>∑(weight(v,w) * (rank(v) - rank(w)))</code> over all <code>(v,w)</code> in <code>E</code> is minimized.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/RankAssignmentAlgorithm Online Documentation}*/
export class RankAssignmentAlgorithm {
/**
Like {@link RankAssignmentAlgorithm#simple}, but arrays are used instead of {@link INodeMap}s and
{@link IEdgeMap}s.
Minimum edge length and weights should be non-negative.
<b>Warning:</b> If the algorithm exceeds the maximum duration, the result may be invalid (not a valid ranking).
<b>Warning:</b> If the algorithm exceeds the maximum duration, the result may be invalid (not a valid ranking).
<h2>Note</h2>
The calculated result is a valid but not optimal solution of the rank assignment problem.
@param graph the input graph in which all the edges have directions, such that <code>rank[source] &lt; rank[target]</code> and
<code>rank[target] - rank[source] >= minlength[edge]</code>
@param rank an array that will be filled with the ranking <code>r</code> of each node <code>v</code> such that <code>rank[v.index] == r</code>
@param minLength an array holding a non-negative value <code>len</code> of each edge <code>e</code> such that <code>minLength[e.index] == len</code>
@param [maximalDuration=0x7FFFFFFFFFFFFFFF] a preferred time limit for the algorithm (in milliseconds)
@returns the number of layers
@see RankAssignmentAlgorithm#simple
@see {@link https://docs.yworks.com/yfileshtml/#/api/RankAssignmentAlgorithm%23simple(Graph%2CArray%3Cnumber%3E%2CArray%3Cnumber%3E%2Cnumber) Online Documentation}*/
static simple(graph: Graph, rank: number[], minLength: number[], maximalDuration?: number): number;
/**
This method quickly calculates a tight tree given a maximum time duration for the algorithm.
The algorithm is using a highly optimized version of Gansner's algorithm:
<ul>
<li>E.R. Gansner et al., <i>A Technique for Drawing Directed Graphs</i>, IEEE Transactions on Software Engineering, Vol.19, No.3, March 1993.
</ul>
Minimum edge length and weights should be non-negative.
<b>Warning:</b> If the algorithm exceeds the maximum duration, the result may be invalid (not a valid ranking).
<h2>Note</h2>
The calculated result is a valid but not optimal solution of the rank assignment problem.
@param graph the input graph in which all the edges have directions, such that <code>rank[source] &lt; rank[target]</code> and
<code>rank[target] - rank[source] >= minlength[edge]</code>
@param rank the {@link INodeMap} that will be filled during the execution and returns the integer ranking of each node
@param minLength the {@link IEdgeMap} that returns an integer value (minimum/tight length) of each edge
@param [maximalDuration=0x7FFFFFFFFFFFFFFF] a preferred time limit for the algorithm (in milliseconds)
@see RankAssignmentAlgorithm#simple
@see {@link https://docs.yworks.com/yfileshtml/#/api/RankAssignmentAlgorithm%23simple(Graph%2CINodeMap%2CIEdgeMap%2Cnumber) Online Documentation}*/
static simple(graph: Graph, rank: INodeMap, minLength: IEdgeMap, maximalDuration?: number): number;
/**
Similar to {@link RankAssignmentAlgorithm#simplex} but, additionally, it is possible to provide a valid initial tree
solution for the problem.
Minimum edge length and weights should be non-negative.
<b>Warning:</b> If the algorithm exceeds the maximum duration, the result may not be optimal.
@param graph the given graph
@param layer the {@link INodeMap} that will be filled during the execution and returns the zero-based ranking index for
each node
@param w the {@link IDataProvider} that returns an integer value (weight) of each edge
@param minLength the {@link IDataProvider} that returns an integer value (minimum length) of each edge
@param tree the {@link IEdgeMap} that returns a boolean value indicating whether or not an edge is a tree edge
@param root the given root node of the tree solution
@param validRanking <code>true</code> if the argument <code>layer</code> contains a valid ranking, <code>false</code> otherwise
@param [maximalDuration=0x7FFFFFFFFFFFFFFF] a preferred time limit for the algorithm (in milliseconds)
@returns the number of layers
@see RankAssignmentAlgorithm#simplex
@see RankAssignmentAlgorithm#simplex
@see {@link https://docs.yworks.com/yfileshtml/#/api/RankAssignmentAlgorithm%23simplex(Graph%2CINodeMap%2CIDataProvider%2CIDataProvider%2CIEdgeMap%2CYNode%2Cboolean%2Cnumber) Online Documentation}*/
static simplex(graph: Graph | null, layer: INodeMap | null, w: IDataProvider | null, minLength: IDataProvider | null, tree: IEdgeMap | null, root: YNode | null, validRanking: boolean, maximalDuration?: number): number;
/**
Solves the rank assignment problem using the simplex method given a maximum time duration for the algorithm.
This method assigns a minimum rank to the nodes in a acyclic graph.
Although its time complexity has not been proven polynomial, in practice it takes few iterations and runs quickly.
The algorithm is based on:
<ul>
<li>E.R. Gansner et al., <i>A Technique for Drawing Directed Graphs</i>, IEEE Transactions on Software Engineering, Vol.19, No.3, March 1993.
</ul>
Minimum edge length and weights should be non-negative.
<b>Warning:</b> If the algorithm exceeds the maximum duration, the result may not be optimal.
@param graph the given graph
@param layer the {@link INodeMap} that will be filled during the execution and returns the zero-based ranking index for
each node
@param w the {@link IDataProvider} that returns an integer value (weight) of each edge
@param minLength the {@link IDataProvider} that returns an integer value (minimum length) of each edge
@param [maximalDuration=0x7FFFFFFFFFFFFFFF] a preferred time limit for the algorithm (in milliseconds)
@returns the number of layers
@see RankAssignmentAlgorithm#simplex
@see RankAssignmentAlgorithm#simplex
@see {@link https://docs.yworks.com/yfileshtml/#/api/RankAssignmentAlgorithm%23simplex(Graph%2CINodeMap%2CIDataProvider%2CIDataProvider%2Cnumber) Online Documentation}*/
static simplex(graph: Graph | null, layer: INodeMap | null, w: IDataProvider | null, minLength: IDataProvider | null, maximalDuration?: number): number;
static $class: Class;
private $_RankAssignments: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RankAssignmentAlgorithm;
}
export interface ShortestPathAlgorithm extends YObject {
}
/**
This class provides diverse algorithms and helper methods for solving the shortest path problem on weighted graphs.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To find shortest paths in an {@link IGraph}
use one of the following classes instead:
<ul>
<li>{@link ShortestPath}
<li>{@link SingleSourceShortestPaths}
<li>{@link AllPairsShortestPaths}
</ul>
<b>Definitions</b>
Given a weighted directed/undirected graph:
<ul>
<li>The <b>shortest path problem</b> is the problem of finding a shortest path between <b>a source node <code>s</code>
and a target node <code>t</code></b> such that the sum of the edge costs is minimized.
<li>The <b>k-shortest path problem</b> is the problem of finding <b><code>k</code> shortest paths</b> between <b>a source
node <code>s</code> and a target node <code>t</code></b> such that the sum of the edge costs is minimized.
<li>The <b>single-source shortest path problem</b> is the problem of finding shortest paths from <b>a source node
<code>s</code> to all</b> other nodes such that the sum of the edge costs is minimized.
<li>The <b>single-source single-sink shortest path problem</b> is the problem of finding shortest paths from <b>a source node <code>s</code> to a target node <code>t</code></b> such that the sum of the edge costs is minimized.
<li>The <b>all-pairs shortest path problem</b> is the problem of finding shortest paths between <b>every pair of
nodes</b> such that the sum of the edge costs is minimized.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm Online Documentation}*/
export class ShortestPathAlgorithm {
/**
Solves the single-source shortest path problem for <b>acyclic</b> directed graphs.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths from one node to all other
nodes in an {@link IGraph} use {@link SingleSourceShortestPaths} instead.
Each edge is associated with an arbitrary double value that represents the cost of that edge.
This method yields the shortest distance from a given node <code>s</code> to all other nodes.
@param graph the input graph
@param s the node from which the shortest path search starts
@param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
@param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
<code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
@param [pred=null] an array of {@link Edge}s that will be filled during the execution and returns for each node <code>t</code> the shortest
path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
@returns <code>true</code> if the input graph is acyclic, <code>false</code> otherwise
@see ShortestPathAlgorithm#acyclic
@see ShortestPathAlgorithm#constructNodePath
@see ShortestPathAlgorithm#constructEdgePath
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23acyclic(Graph%2CYNode%2CArray%3Cnumber%3E%2CArray%3Cnumber%3E%2CArray%3CEdge%3E) Online Documentation}*/
static acyclic(graph: Graph, s: YNode, cost: number[], dist: number[], pred?: Edge[] | null): boolean;
/**
Solves the single-source shortest path problem for <b>acyclic</b> directed graphs.
Each edge is associated with an arbitrary double value that represents the cost of that edge.
This method yields the shortest distance from a given node <code>s</code> to all other nodes.
@param graph the input graph
@param s the node from which the shortest path search starts
@param cost the {@link IDataProvider} that returns the double value (cost) for traversing each edge
@param dist the {@link INodeMap} that will be filled during the execution and returns a double value (shortest distance)
from node <code>s</code> to all other nodes or {@link Number#POSITIVE_INFINITY} if no such paths exist
@param pred the {@link INodeMap} that will be filled during the execution and returns for each node <code>t</code> the last edge on the
shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
@returns <code>true</code> if the input graph is acyclic, <code>false</code> otherwise
@see ShortestPathAlgorithm#acyclic
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23acyclic(Graph%2CYNode%2CIDataProvider%2CINodeMap%2CINodeMap) Online Documentation}*/
static acyclic(graph: Graph, s: YNode, cost: IDataProvider, dist: INodeMap, pred: INodeMap): boolean;
/**
This method solves the all-pairs shortest path problem for graphs with arbitrary edge costs.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths between all pairs of nodes in
an {@link IGraph} use {@link AllPairsShortestPaths} instead.
If the given graph contains a negative-cost cycle, then <code>false</code> is returned and the values returned in <code>dist</code> are left
unspecified.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
@param dist an array of values that will be filled during the execution and returns the shortest path distances from all pairs of
nodes <code>s</code> and <code>t</code> in the graph. The distance from <code>s</code> to <code>t</code> is <code>dist[s.index()][t.index()]</code>. If there is no path from <code>s</code> to <code>t</code>,
then <code>dist[s.index()][t.index()] == Double.POSITIVE_INFINITY</code>.
@returns <code>true</code> if the given graph does not contain a negative-cost cycle, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23allPairs(Graph%2Cboolean%2CArray%3Cnumber%3E%2CArray%3CArray%3Cnumber%3E%3E) Online Documentation}*/
static allPairs(graph: Graph | null, directed: boolean, cost: number[] | null, dist: number[][] | null): boolean;
/**
Solves the single-source single-sink shortest path problem for arbitrary graphs using an implementation of the
well-known a-star algorithm (A*).
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths between two nodes in an {@link IGraph}
use {@link ShortestPath} instead. Use the {@link ShortestPath#heuristic} property to use A*.
This method is like {@link ShortestPathAlgorithm#aStar}, with the difference that it does not return the shortest path
but the cost of the shortest path. To construct the actual shortest path use {@link ShortestPathAlgorithm#constructEdgePath}
or {@link ShortestPathAlgorithm#constructNodePath}.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge. The costs should be non-negative.
Furthermore, the algorithm uses so-called <em>heuristic costs</em> for each node. The heuristic cost for a node <code>n</code> is the estimated cost to reach
the sink node <code>t</code> from <code>n</code>. The heuristic has a crucial impact on the performance of the shortest path search:
<ul>
<li>If the heuristic is 'good', i.e., heuristic costs are close to the actual costs, then the algorithm is able to avoid
unprofitable search paths and move faster towards the target. Using a good heuristic, this algorithm outperforms other
shortest-path algorithms.
<li>The heuristic must be <em>admissible</em>, which means that it must never overestimate the actual cost to reach the target node. Otherwise,
it is not guaranteed that the algorithm actually finds the shortest path. Use {@link ShortestPathAlgorithm#dijkstra}
if you have no proper problem-specific heuristic.
</ul>
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the source node
@param t the sink node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost an array of double values that represent the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
@param heuristicCost an array of double values that represent an estimate to reach the sink node <code>t</code>. The value of <code>heuristicCost[n.index()]</code> is
the estimated heuristic cost to reach node <code>t</code> from node <code>n</code>
@param pred an array of {@link Edge}s that will be filled during the execution and holds for each node <code>t</code> the shortest path
edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest path
from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
@returns the shortest distance (cost) between <code>s</code> and <code>t</code> if a path between these two nodes exists or {@link Number#POSITIVE_INFINITY}
otherwise
@see ShortestPathAlgorithm#aStar
@see ShortestPathAlgorithm#aStar
@see ShortestPathAlgorithm#aStar
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23aStar(Graph%2CYNode%2CYNode%2Cboolean%2CArray%3Cnumber%3E%2CArray%3Cnumber%3E%2CArray%3CEdge%3E) Online Documentation}*/
static aStar(graph: Graph, s: YNode, t: YNode, directed: boolean, cost: number[], heuristicCost: number[], pred: Edge[]): number;
/**
Solves the single-source single-sink shortest path problem for arbitrary graphs using an implementation of the
well-known a-star algorithm (A*).
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths between two nodes in an {@link IGraph}
use {@link ShortestPath} instead. Use the {@link ShortestPath#heuristic} property to use A*.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge. The costs should be non-negative.
Furthermore, the algorithm uses so-called <em>heuristic costs</em> for each node. The heuristic cost for a node <code>n</code> is the estimated cost to reach
the sink node <code>t</code> from <code>n</code>. The heuristic has a crucial impact on the performance of the shortest path search:
<ul>
<li>If the heuristic is 'good', i.e., heuristic costs are close to the actual costs, then the algorithm is able to avoid
unprofitable search paths and move faster towards the target. Using a good heuristic, this algorithm outperforms other
shortest-path algorithms.
<li>The heuristic must be <em>admissible</em>, which means that it must never overestimate the actual cost to reach the target node. Otherwise,
it is not guaranteed that the algorithm actually finds the shortest path. Use {@link ShortestPathAlgorithm#dijkstra}
if you have no proper problem-specific heuristic.
</ul>
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the source node
@param t the sink node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost an array of double values that represent the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
@param heuristicCost an array of double values that represent an estimate to reach the sink node <code>t</code>. The value of <code>heuristicCost[n.index()]</code> is
the estimated heuristic cost to reach node <code>t</code> from node <code>n</code>
@returns the list of edges constituting the shortest path from the source node <code>s</code> to the sink node <code>t</code> or an empty list if there is
no such path
@see ShortestPathAlgorithm#aStar
@see ShortestPathAlgorithm#aStar
@see ShortestPathAlgorithm#aStar
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23aStar(Graph%2CYNode%2CYNode%2Cboolean%2CArray%3Cnumber%3E%2CArray%3Cnumber%3E) Online Documentation}*/
static aStar(graph: Graph, s: YNode, t: YNode, directed: boolean, cost: number[], heuristicCost: number[]): EdgeList | null;
/**
Solves the single-source single-sink shortest path problem for arbitrary graphs using an implementation of the
well-known a-star algorithm (A*).
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths between two nodes in an {@link IGraph}
use {@link ShortestPath} instead. Use the {@link ShortestPath#heuristic} property to use A*.
This method is like {@link ShortestPathAlgorithm#aStar}, with the only difference that it takes
{@link IDataProvider}s and a {@link INodeMap} as input instead of arrays.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge. The costs should be non-negative.
Furthermore, the algorithm uses so-called <em>heuristic costs</em> for each node. The heuristic cost for a node <code>n</code> is the estimated cost to reach
the sink node <code>t</code> from <code>n</code>. The heuristic has a crucial impact on the performance of the shortest path search:
<ul>
<li>If the heuristic is 'good', i.e., heuristic costs are close to the actual costs, then the algorithm is able to avoid
unprofitable search paths and move faster towards the target. Using a good heuristic, this algorithm outperforms other
shortest-path algorithms.
<li>The heuristic must be <em>admissible</em>, which means that it must never overestimate the actual cost to reach the target node. Otherwise,
it is not guaranteed that the algorithm actually finds the shortest path. Use {@link ShortestPathAlgorithm#dijkstra}
if you have no proper problem-specific heuristic.
</ul>
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the source node
@param t the sink node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost the {@link IDataProvider} that returns the cost for traversing an edge
@param heuristicCost the {@link IDataProvider} that yields for each node the heuristic cost estimate to reach the sink node <code>t</code>
starting from the node
@param pred the map that will be filled during the execution and returns for each node <code>t</code> the last edge on the shortest path from <code>s</code>
to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
@returns the shortest distance (cost) between <code>s</code> and <code>t</code> if a path between these two nodes exists or {@link Number#POSITIVE_INFINITY}
otherwise
@see ShortestPathAlgorithm#aStar
@see ShortestPathAlgorithm#aStar
@see ShortestPathAlgorithm#aStar
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23aStar(Graph%2CYNode%2CYNode%2Cboolean%2CIDataProvider%2CIDataProvider%2CINodeMap) Online Documentation}*/
static aStar(graph: Graph, s: YNode, t: YNode, directed: boolean, cost: IDataProvider, heuristicCost: IDataProvider, pred: INodeMap): number;
/**
Solves the single-source single-sink shortest path problem for arbitrary graphs using an implementation of the
well-known a-star algorithm (A*).
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths between two nodes in an {@link IGraph}
use {@link ShortestPath} instead. Use the {@link ShortestPath#heuristic} property to use A*.
This method is like {@link ShortestPathAlgorithm#aStar}, with the only difference that it takes
{@link IDataProvider}s as parameters instead of arrays.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge. The costs should be non-negative.
Furthermore, the algorithm uses so-called <em>heuristic costs</em> for each node. The heuristic cost for a node <code>n</code> is the estimated cost to reach
the sink node <code>t</code> from <code>n</code>. The heuristic has a crucial impact on the performance of the shortest path search:
<ul>
<li>If the heuristic is 'good', i.e., heuristic costs are close to the actual costs, then the algorithm is able to avoid
unprofitable search paths and move faster towards the target. Using a good heuristic, this algorithm outperforms other
shortest-path algorithms.
<li>The heuristic must be <em>admissible</em>, which means that it must never overestimate the actual cost to reach the target node. Otherwise,
it is not guaranteed that the algorithm actually finds the shortest path. Use {@link ShortestPathAlgorithm#dijkstra}
if you have no proper problem-specific heuristic.
</ul>
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the source node
@param t the sink node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost the {@link IDataProvider} that returns the cost for traversing an edge
@param heuristicCost the {@link IDataProvider} that yields for each node the heuristic cost estimate to reach the sink node <code>t</code>
starting from the node
@returns the list of edges constituting the shortest path from the source node <code>s</code> to the sink node <code>t</code> or an empty list if there is
no such path
@see ShortestPathAlgorithm#aStar
@see ShortestPathAlgorithm#aStar
@see ShortestPathAlgorithm#aStar
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23aStar(Graph%2CYNode%2CYNode%2Cboolean%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static aStar(graph: Graph, s: YNode, t: YNode, directed: boolean, cost: IDataProvider, heuristicCost: IDataProvider): EdgeList | null;
/**
Solves the single-source shortest path problem for arbitrary graphs.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths from one node to all other
nodes in an {@link IGraph} use {@link SingleSourceShortestPaths} instead.
Each edge is associated with an arbitrary double value that represents the cost of this edge.
In case the given weighted graph contains no <b>negative-cost cycles</b>, this method will yield the shortest distance from a given node <code>s</code> to all
other nodes. If, on the other hand, the given graph contains negative-cost cycles, this method will yield no reasonable
result which will be indicated by the return value <code>false</code>.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
<b>Warning:</b>
If the graph is considered undirected, each edge with negative costs induces a negative-cost cycle and, thus, the
algorithm doesn't return a valid result. Hence, for undirected graphs, all costs must not be negative.
@param graph the input graph
@param s the source node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost the {@link IDataProvider} that returns the double value (cost) for traversing each edge
@param dist the {@link INodeMap} that will be filled during the execution and returns a double value (shortest distance)
from node <code>s</code> to all other nodes or {@link Number#POSITIVE_INFINITY} if no such paths exist
@param pred the {@link INodeMap} that will be filled during the execution and returns for each node <code>t</code> the last edge on the
shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
@returns <code>true</code> if the algorithm found a valid result, <code>false</code> otherwise (i.e., graph contains negative-cost cycles)
@see ShortestPathAlgorithm#bellmanFord
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23bellmanFord(Graph%2CYNode%2Cboolean%2CIDataProvider%2CINodeMap%2CINodeMap) Online Documentation}*/
static bellmanFord(graph: Graph, s: YNode, directed: boolean, cost: IDataProvider, dist: INodeMap, pred: INodeMap): boolean;
/**
Solves the single-source shortest path problem for arbitrary graphs.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths from one node to all other
nodes in an {@link IGraph} use {@link SingleSourceShortestPaths} instead.
Each edge is associated with an arbitrary double value that represents the cost of this edge.
In case the given weighted graph contains no <b>negative cost cycles</b>, this method will yield the shortest distance from a given node <code>s</code> to all
other nodes. If, on the other hand, the given graph contains negative-cost cycles, this method will yield no reasonable
result which will be indicated by the return value <code>false</code>.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
<b>Warning:</b>
If the graph is considered undirected, each edge with negative costs induces a negative-cost cycle and, thus, the
algorithm doesn't return a valid result. Hence, for undirected graphs, all costs must not be negative.
@param graph the input graph
@param s the node from which the shortest path search starts
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
@param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
<code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
@param [pred=null] an array of {@link Edge}s that will be filled during the execution and returns for each node <code>t</code> the shortest
path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
@returns <code>true</code> if the algorithm found a valid result, <code>false</code> otherwise (i.e., graph contains negative-cost cycles)
@see ShortestPathAlgorithm#constructNodePath
@see ShortestPathAlgorithm#constructEdgePath
@see ShortestPathAlgorithm#bellmanFord
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23bellmanFord(Graph%2CYNode%2Cboolean%2CArray%3Cnumber%3E%2CArray%3Cnumber%3E%2CArray%3CEdge%3E) Online Documentation}*/
static bellmanFord(graph: Graph | null, s: YNode | null, directed: boolean, cost: number[] | null, dist: number[] | null, pred?: Edge[] | null): boolean;
/**
Convenience method that constructs an explicit {@link EdgeList path of edges} from the result returned by one of the shortest paths
methods defined in this class.
If there is no path from node <code>s</code> to <code>t</code>, then an empty list is returned.
@param s the start node of the shortest path which must be the same start node that was specified when <code>pred</code> was calculated
@param t the target node of the path
@param pred an array of {@link Edge}s that will be filled during the execution and returns for each node <code>t</code> the shortest
path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
@returns a {@link EdgeList list} containing the edges on the shortest path from <code>s</code> to <code>t</code> in the correct order
@see ShortestPathAlgorithm#constructEdgePath
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23constructEdgePath(YNode%2CYNode%2CArray%3CEdge%3E) Online Documentation}*/
static constructEdgePath(s: YNode, t: YNode, pred: Edge[]): EdgeList | null;
/**
Like {@link ShortestPathAlgorithm#constructEdgePath} but the path edges are given by a
{@link IDataProvider}.
If there is no path from node <code>s</code> to <code>t</code>, then an empty list is returned.
@param s the start node of the shortest path which must be the same start node that was specified when <code>pred</code> was calculated
@param t the target node of the path
@param pred the {@link INodeMap} that will be filled during the execution and returns for each node <code>t</code> the last edge on the
shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
@returns a {@link EdgeList list} containing the edges on the shortest path from <code>s</code> to <code>t</code> in the correct order
@see ShortestPathAlgorithm#constructEdgePath
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23constructEdgePath(YNode%2CYNode%2CIDataProvider) Online Documentation}*/
static constructEdgePath(s: YNode, t: YNode, pred: IDataProvider): EdgeList | null;
/**
Convenience method that constructs an explicit {@link YNodeList path of nodes} from the result returned by one of the shortest paths
methods defined in this class.
If there is no path from node <code>s</code> to <code>t</code>, then an empty list is returned.
@param s the start node of the shortest path which must be the same start node that was specified when <code>pred</code> was calculated
@param t the target node of the path
@param pred an array of {@link Edge}s that will be filled during the execution and returns for each node <code>t</code> the shortest
path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
@returns a {@link YNodeList list} containing the nodes on the shortest path from <code>s</code> to <code>t</code> in the correct order
@see ShortestPathAlgorithm#constructNodePath
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23constructNodePath(YNode%2CYNode%2CArray%3CEdge%3E) Online Documentation}*/
static constructNodePath(s: YNode, t: YNode, pred: Edge[]): YNodeList | null;
/**
Like {@link ShortestPathAlgorithm#constructNodePath} but the path edges are given by a
{@link IDataProvider}.
If there is no path from node <code>s</code> to <code>t</code>, then an empty list is returned.
@param s the start node of the shortest path which must be the same start node that was specified when <code>pred</code> was calculated
@param t the target node of the path
@param pred the {@link INodeMap} that will be filled during the execution and returns for each node <code>t</code> the last edge on the
shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
@returns a {@link YNodeList list} containing the nodes on the shortest path from <code>s</code> to <code>t</code> in the correct order
@see ShortestPathAlgorithm#constructNodePath
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23constructNodePath(YNode%2CYNode%2CIDataProvider) Online Documentation}*/
static constructNodePath(s: YNode, t: YNode, pred: IDataProvider): YNodeList | null;
/**
Solves the single-source shortest path problem for arbitrary graphs and additionally, this method yields the path edges
of each calculated shortest path.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths from one node to all other
nodes in an {@link IGraph} use {@link SingleSourceShortestPaths} instead.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
This method yields the shortest distance from a given node <code>s</code> to all other nodes.
The costs should be non-negative.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the node from which the shortest path search starts
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
@param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
<code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
@param [pred=null] an array of {@link Edge}s that will be filled during the execution and returns for each node <code>t</code> the shortest
path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
@see ShortestPathAlgorithm#constructNodePath
@see ShortestPathAlgorithm#constructEdgePath
@see ShortestPathAlgorithm#dijkstra
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23dijkstra(Graph%2CYNode%2Cboolean%2CArray%3Cnumber%3E%2CArray%3Cnumber%3E%2CArray%3CEdge%3E) Online Documentation}*/
static dijkstra(graph: Graph, s: YNode, directed: boolean, cost: number[], dist: number[], pred?: Edge[] | null): void;
/**
Solves the single-source shortest path problem for arbitrary graphs.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths from one node to all other
nodes in an {@link IGraph} use {@link SingleSourceShortestPaths} instead.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
This method yields the shortest distance from a given node <code>s</code> to all other nodes.
The costs should be non-negative.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the node from which the shortest path search starts
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost the {@link IDataProvider} that returns the double value (cost) for traversing each edge
@param dist the {@link INodeMap} that will be filled during the execution and returns a double value (shortest distance)
from node <code>s</code> to all other nodes or {@link Number#POSITIVE_INFINITY} if no such paths exist
@param pred the {@link INodeMap} that will be filled during the execution and returns for each node <code>t</code> the last edge on the
shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
@see ShortestPathAlgorithm#constructNodePath
@see ShortestPathAlgorithm#constructEdgePath
@see ShortestPathAlgorithm#dijkstra
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23dijkstra(Graph%2CYNode%2Cboolean%2CIDataProvider%2CINodeMap%2CINodeMap) Online Documentation}*/
static dijkstra(graph: Graph, s: YNode, directed: boolean, cost: IDataProvider, dist: INodeMap, pred: INodeMap): void;
/**
Finds all nodes and edges that belong to a shortest path from a <code>start</code> node to a set of target nodes in the graph not
farther away than a given distance.
This method assumes that each edge of the input graph has a <b>cost of 1.0</b>.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
<b>Warning:</b> Shortest paths longer than <code>maxLength</code> will not be considered.
@param graph the input graph
@param start the start node
@param targetMap the {@link IDataProvider} that returns a boolean value indicating whether or not a node belongs to the set of
target nodes
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param maxLength the maximum edge length of the shortest paths
@param pathEdges a {@link EdgeList list} that will be filled during the execution and returns the edges on the shortest path from <code>s</code> to <code>t</code> in the correct
order
@param pathNodes a {@link YNodeList list} that will be filled during the execution and returns the nodes on the shortest path from <code>s</code> to <code>t</code> in the correct
order
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23findShortestUniformPaths(Graph%2CYNode%2CIDataProvider%2Cboolean%2Cnumber%2CEdgeList%2CYNodeList) Online Documentation}*/
static findShortestUniformPaths(graph: Graph | null, start: YNode | null, targetMap: IDataProvider | null, directed: boolean, maxLength: number, pathEdges: EdgeList | null, pathNodes: YNodeList | null): void;
/**
Marks all edges that belong to a shortest path from <code>start</code> node to <code>target</code> node.
This method assumes that each edge of the input graph has a <b>cost of 1.0</b>.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param start the start node
@param end the target node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param pathMap the {@link IEdgeMap} that will be filled during the execution and returns a boolean value indicating whether
or not the edge belongs to a shortest path connecting the two nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23findShortestUniformPaths(Graph%2CYNode%2CYNode%2Cboolean%2CIEdgeMap) Online Documentation}*/
static findShortestUniformPaths(graph: Graph | null, start: YNode | null, end: YNode | null, directed: boolean, pathMap: IEdgeMap | null): void;
/**
This method finds the <code>k</code> shortest paths connecting a pair of nodes in a directed graph with non-negative edge costs.
The result will be returned as a list of {@link EdgeList} objects.
The cost should be non-negative.
<h2>Note</h2>
The returned paths are not required to be simple, i.e. they may contain a node or an edge multiple times.
<h2>Note</h2>
The returned list may contain less than <code>k</code> paths in case there are fewer directed paths between the start and target
node.
@param graph the input graph
@param costDP the {@link IDataProvider} that returns a double value (cost) for traversing each edge
@param start the given start node
@param end the given target node
@param k a non-negative integer value
@returns a {@link YList list} of {@link EdgeList} objects each of which represents a path from <code>start</code> node to <code>target</code> node. The <code>i</code>-th
path in the list contains the <code>i</code>-th shortest path between the <code>start</code> and <code>target</code> node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23kShortestPaths(Graph%2CIDataProvider%2CYNode%2CYNode%2Cnumber) Online Documentation}*/
static kShortestPaths(graph: Graph | null, costDP: IDataProvider | null, start: YNode | null, end: YNode | null, k: number): YList | null;
/**
A variant of {@link ShortestPathAlgorithm#kShortestPaths} that returns the result as a special cursor that calculates
the next path in the sequence only when needed.
The returned cursor only supports the operation {@link ICursor#ok}, {@link ICursor#current} and {@link ICursor#next}.
The cost should be non-negative.
<h2>Note</h2>
The returned paths are not required to be simple, i.e. they may contain a node or an edge multiple times.
<h2>Note</h2>
The returned list may contain less than <code>k</code> paths in case there are fewer directed paths between the start and target
node.
@param graph the input graph
@param costDP the {@link IDataProvider} that returns a double value (cost) for traversing each edge
@param start the given start node
@param end the given target node
@param k a non-negative integer value
@returns a {@link ICursor cursor} that calculates the next path in the sequence only when needed
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23kShortestPathsCursor(Graph%2CIDataProvider%2CYNode%2CYNode%2Cnumber) Online Documentation}*/
static kShortestPathsCursor(graph: Graph | null, costDP: IDataProvider | null, start: YNode | null, end: YNode | null, k: number): ICursor | null;
/**
Returns two edge-disjoint paths in a non-negatively weighted directed graph, such that both paths connect nodes <code>s</code> and <code>t</code>
and have minimum total length.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param source the source node of the shortest pair
@param target the target node of the shortest pair
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param costDP the {@link IDataProvider} that returns a double value (cost) for traversing each edge
@returns a two-dimensional array of {@link EdgeList}s holding the resulting edge-disjoint paths or <code>null</code> if no such
edge-disjoint paths exist
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23shortestPair(Graph%2CYNode%2CYNode%2Cboolean%2CIDataProvider) Online Documentation}*/
static shortestPair(graph: Graph | null, source: YNode | null, target: YNode | null, directed: boolean, costDP: IDataProvider | null): EdgeList[] | null;
/**
This method solves the single-source shortest path problem for arbitrary graphs.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths from one node to all other
nodes in an {@link IGraph} use {@link SingleSourceShortestPaths} instead.
<h2>Note</h2>
Depending on the structure of the given graph and the values of the given edge costs it delegates its work to the
algorithm with the theoretically best running time. Please note that theory does not necessarily reflect practice.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the node from which the shortest path search starts
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
@param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
<code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
@param [pred=null] an array of {@link Edge}s that will be filled during the execution and returns for each node <code>t</code> the shortest
path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
@returns <code>true</code> if the weighted graph does not contain a negative-cost cycle, <code>false</code> otherwise
@see ShortestPathAlgorithm#constructNodePath
@see ShortestPathAlgorithm#constructEdgePath
@see ShortestPathAlgorithm#singleSource
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23singleSource(Graph%2CYNode%2Cboolean%2CArray%3Cnumber%3E%2CArray%3Cnumber%3E%2CArray%3CEdge%3E) Online Documentation}*/
static singleSource(graph: Graph, s: YNode, directed: boolean, cost: number[], dist: number[], pred?: Edge[] | null): boolean;
/**
This method solves the single-source shortest path problem for arbitrary graphs.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths from one node to all other
nodes in an {@link IGraph} use {@link SingleSourceShortestPaths} instead.
<h2>Note</h2>
Depending on the structure of the given graph and the values of the given edge costs it delegates its work to the
algorithm with the theoretically best running time. Please note that theory does not necessarily reflect practice.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the source node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost the {@link IDataProvider} that returns the double value (cost) for traversing each edge
@param dist the {@link INodeMap} that will be filled during the execution and returns a double value (shortest distance)
from node <code>s</code> to all other nodes or {@link Number#POSITIVE_INFINITY} if no such paths exist
@param pred the {@link INodeMap} that will be filled during the execution and returns for each node <code>t</code> the last edge on the
shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
@returns <code>true</code> if the weighted graph does not contain a negative-cost cycle, <code>false</code> otherwise
@see ShortestPathAlgorithm#constructNodePath
@see ShortestPathAlgorithm#constructEdgePath
@see ShortestPathAlgorithm#singleSource
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23singleSource(Graph%2CYNode%2Cboolean%2CIDataProvider%2CINodeMap%2CINodeMap) Online Documentation}*/
static singleSource(graph: Graph, s: YNode, directed: boolean, cost: IDataProvider, dist: INodeMap, pred: INodeMap): boolean;
/**
This method solves the single-source single-sink shortest path problem for arbitrary graphs.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths between two nodes in an {@link IGraph}
use {@link ShortestPath} instead.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
This method returns the shortest distance from node <code>s</code> to node
<code>t</code>. It also returns information to construct the actual path between these two nodes.
The costs should be non-negative.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the source node
@param t the sink node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
@param pred an array of {@link Edge}s that will be filled during the execution and returns for each node <code>t</code> the shortest
path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
@returns the distance between <code>s</code>and <code>t</code> if a path between these two nodes exists or {@link Number#POSITIVE_INFINITY} otherwise
@see ShortestPathAlgorithm#constructNodePath
@see ShortestPathAlgorithm#constructEdgePath
@see ShortestPathAlgorithm#singleSourceSingleSink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23singleSourceSingleSink(Graph%2CYNode%2CYNode%2Cboolean%2CArray%3Cnumber%3E%2CArray%3CEdge%3E) Online Documentation}*/
static singleSourceSingleSink(graph: Graph, s: YNode, t: YNode, directed: boolean, cost: number[], pred: Edge[]): number;
/**
Like {@link ShortestPathAlgorithm#singleSourceSingleSink} but uses {@link INodeMap}s and
{@link IDataProvider}s instead of arrays.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths between two nodes in an {@link IGraph}
use {@link ShortestPath} instead.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
The costs should be non-negative.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the source node
@param t the sink node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost the {@link IDataProvider} that returns the double value (cost) for traversing each edge
@param pred the {@link INodeMap} that will be filled during the execution and returns for each node <code>t</code> the last edge on the
shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
@returns the distance between <code>s</code>and <code>t</code> if a path between these two nodes exists or {@link Number#POSITIVE_INFINITY} otherwise
@see ShortestPathAlgorithm#singleSourceSingleSink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23singleSourceSingleSink(Graph%2CYNode%2CYNode%2Cboolean%2CIDataProvider%2CINodeMap) Online Documentation}*/
static singleSourceSingleSink(graph: Graph, s: YNode, t: YNode, directed: boolean, cost: IDataProvider, pred: INodeMap): number;
/**
Similar to {@link ShortestPathAlgorithm#singleSourceSingleSink} but instead of returning the shortest distance between
the source and sink the actual shortest edge path between these nodes will be returned.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths between two nodes in an {@link IGraph}
use {@link ShortestPath} instead.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
If the returned path is empty, then there is no path between the nodes.
The costs should be non-negative.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the source node
@param t the sink node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
@returns a shortest {@link EdgeList path of edges} between source and sink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23singleSourceSingleSink(Graph%2CYNode%2CYNode%2Cboolean%2CArray%3Cnumber%3E) Online Documentation}*/
static singleSourceSingleSink(graph: Graph, s: YNode, t: YNode, directed: boolean, cost: number[]): EdgeList | null;
/**
Similar to {@link ShortestPathAlgorithm#singleSourceSingleSink} but instead of returning the shortest distance between
the source and sink the actual shortest edge path between these nodes will be returned.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths between two nodes in an {@link IGraph}
use {@link ShortestPath} instead.
Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
If the returned path is empty, then there is no path between the nodes.
The costs should be non-negative.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the source node
@param t the sink node
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param cost the {@link IDataProvider} that returns the double value (cost) for traversing each edge
@returns a shortest {@link EdgeList path of edges} between source and sink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see ShortestPathAlgorithm#singleSourceSingleSink
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23singleSourceSingleSink(Graph%2CYNode%2CYNode%2Cboolean%2CIDataProvider) Online Documentation}*/
static singleSourceSingleSink(graph: Graph, s: YNode, t: YNode, directed: boolean, cost: IDataProvider): EdgeList | null;
/**
Solves the single-source shortest path problem for arbitrary graphs in which each edge has a <b>uniform cost of
<code>1.0</code></b>.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths from one node to all other
nodes in an {@link IGraph} use {@link SingleSourceShortestPaths} instead.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the node from which the shortest path search starts
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
<code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
@param [pred=null] an array of {@link Edge}s that will be filled during the execution and returns for each node <code>t</code> the shortest
path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
@see ShortestPathAlgorithm#constructNodePath
@see ShortestPathAlgorithm#constructEdgePath
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23uniform(Graph%2CYNode%2Cboolean%2CArray%3Cnumber%3E%2CArray%3CEdge%3E) Online Documentation}*/
static uniform(graph: Graph, s: YNode, directed: boolean, dist: number[], pred?: Edge[] | null): void;
/**
Like {@link ShortestPathAlgorithm#uniform} but uses {@link INodeMap}s instead of arrays.
<b>Note:</b> This method works with instances of {@link Graph}. To find the shortest paths from one node to all other
nodes in an {@link IGraph} use {@link SingleSourceShortestPaths} instead.
<h2>Note</h2>
If the graph is considered undirected, each edge can be traversed in both directions and the returned shortest paths
can, thus, be undirected.
@param graph the input graph
@param s the node from which the shortest path search starts
@param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
@param dist the {@link INodeMap} that will be filled during the execution and returns a double value (shortest distance)
from node <code>s</code> to all other nodes or {@link Number#POSITIVE_INFINITY} if no such paths exist
@param pred the {@link INodeMap} that will be filled during the execution and returns for each node <code>t</code> the last edge on the
shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
@see ShortestPathAlgorithm#uniform
@see ShortestPathAlgorithm#constructNodePath
@see ShortestPathAlgorithm#constructEdgePath
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23uniform(Graph%2CYNode%2Cboolean%2CINodeMap%2CINodeMap) Online Documentation}*/
static uniform(graph: Graph | null, s: YNode | null, directed: boolean, dist: INodeMap | null, pred: INodeMap | null): void;
/**
Convenience method that returns an array containing uniform edge costs of <code>1.0</code> for each edge of the given graph.
@param graph the input graph
@returns an array that contains uniform edge costs of value <code>1.0</code> (i.e., for each edge <code>e: cost[e.index()] == 1.0</code>)
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathAlgorithm%23uniformCost(Graph) Online Documentation}*/
static uniformCost(graph: Graph | null): number[] | null;
static $class: Class;
private $_ShortestPaths: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ShortestPathAlgorithm;
}
export interface SortingAlgorithm extends YObject {
}
/**
This class provides methods for efficiently sorting graph elements in graph structures.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SortingAlgorithm Online Documentation}*/
export class SortingAlgorithm {
/**
Sorts the nodes of a given graph by degree in ascending order.
@param graph the given graph
@returns an array of {@link YNode}s sorted by degree in ascending order
@see {@link https://docs.yworks.com/yfileshtml/#/api/SortingAlgorithm%23sortNodesByDegree(Graph) Online Documentation}*/
static sortNodesByDegree(graph: Graph | null): YNode[] | null;
/**
Sorts the nodes of a given graph by an integer key associated with each node through a {@link IDataProvider}.
The nodes are sorted in ascending order.
@param graph the given graph
@param keys the {@link IDataProvider} that returns the keys based on which the nodes will be sorted
@returns an array of {@link YNode}s sorted by an integer key in ascending order
@see {@link https://docs.yworks.com/yfileshtml/#/api/SortingAlgorithm%23sortNodesByIntKey(Graph%2CIDataProvider) Online Documentation}*/
static sortNodesByIntKey(graph: Graph | null, keys: IDataProvider | null): YNode[] | null;
static $class: Class;
private $_Sorting: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SortingAlgorithm;
}
export interface SpanningTreeAlgorithm extends YObject {
}
/**
This class provides (minimum) spanning tree algorithms for graphs.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To find a (minimum) spanning tree for an {@link IGraph}
use {@link SpanningTree} instead.<b>Definitions</b>
<ul>
<li>A <b>spanning tree</b> of an undirected connected graph is a subset of its edges that induce a tree that connects all nodes of the graph.
<li>A <b>minimum spanning tree</b> of a weighted connected graph is a spanning tree whose edges have minimum overall cost among all spanning trees of
that graph.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTreeAlgorithm Online Documentation}*/
export class SpanningTreeAlgorithm {
/**
Returns the overall cost of a previously calculated minimum spanning tree.
@param treeEdges the given {@link EdgeList list} of edges that form a minimum spanning tree
@param edgeCost the {@link IDataProvider} that returns a double value (cost) for each tree edge
@returns the overall cost of the tree edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTreeAlgorithm%23cost(EdgeList%2CIDataProvider) Online Documentation}*/
static cost(treeEdges: EdgeList | null, edgeCost: IDataProvider | null): number;
/**
Calculates a minimum spanning tree for the given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To find a minimum spanning tree for an {@link IGraph}
use {@link SpanningTree} instead.
The implementation is based on an algorithm originally published in:
<ul>
<li>J.B. Kruskal. On the shortest spanning subtree of a graph and the traveling salesman problem. <i>Proceedings of the American Mathematical Society</i>, pages 48-50, 1956.
</ul>
@param graph the input graph
@param cost the {@link IDataProvider} that returns a double value (cost) for each edge
@returns a {@link EdgeList list} containing the edges that form the minimum spanning tree
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTreeAlgorithm%23kruskal(Graph%2CIDataProvider) Online Documentation}*/
static kruskal(graph: Graph | null, cost: IDataProvider | null): EdgeList | null;
/**
Calculates a minimum spanning tree for the given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To find a minimum spanning tree for an {@link IGraph}
use {@link SpanningTree} instead.
Currently, the result is obtained by calling {@link SpanningTreeAlgorithm#prim}.
@param graph the input graph
@param cost the {@link IDataProvider} that returns a double value (cost) for each edge
@returns a {@link EdgeList list} containing the edges that form the minimum spanning tree
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTreeAlgorithm%23minimum(Graph%2CIDataProvider) Online Documentation}*/
static minimum(graph: Graph | null, cost: IDataProvider | null): EdgeList | null;
/**
Calculates a minimum spanning tree for the given graph.
<b>Note:</b> This method works with instances of {@link Graph}. To find a minimum spanning tree for an {@link IGraph}
use {@link SpanningTree} instead.
The implementation is based on an algorithm originally published in:
<ul>
<li>R.C. Prim. Shortest connection networks and some generalizations. <i>Bell System Technical Journal</i>, 36:1389-1401, 1957.
</ul>
@param graph the input graph
@param cost the {@link IDataProvider} that returns a double value (cost) for each edge
@returns a {@link EdgeList list} containing the edges that form the minimum spanning tree
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTreeAlgorithm%23prim(Graph%2CIDataProvider) Online Documentation}*/
static prim(graph: Graph | null, cost: IDataProvider | null): EdgeList | null;
/**
Calculates a spanning tree for the given graph in which each edge has a uniform cost of <code>1.0</code>.
<b>Note:</b> This method works with instances of {@link Graph}. To find a minimum spanning tree for an {@link IGraph}
use {@link SpanningTree} instead.
@param graph the input graph
@returns a {@link EdgeList list} containing the edges that form the minimum spanning tree
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTreeAlgorithm%23uniform(Graph) Online Documentation}*/
static uniform(graph: Graph | null): EdgeList | null;
static $class: Class;
private $_SpanningTrees: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SpanningTreeAlgorithm;
}
export interface Substructure extends YObject {
}
/**
Represents a substructure of a graph, like a tree, chain, star, clique or tree.
Class {@link Substructures} is able to detect such structures and returns them in a list of {@link Substructure}
instances.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructure Online Documentation}*/
export class Substructure {
/**
Gets the nodes that belong to this substructure.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructure%23nodes Online Documentation}*/
readonly nodes: YNodeList | null;
/**
Gets the edges that belong to this substructure.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructure%23edges Online Documentation}*/
readonly edges: EdgeList | null;
/**
Gets the edges connecting a node in the substructure with one outside of it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructure%23interEdges Online Documentation}*/
readonly interEdges: EdgeList | null;
static $class: Class;
private $_Substructure: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Substructure;
}
export interface Substructures extends YObject {
}
/**
This class allows to detect different substructures within the input graph, for example, tree, star, chain and cycle
structures.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures Online Documentation}*/
export class Substructures {
/**
Returns a list of {@link Substructure}s that represent the chains in the specified graph with at least <code>minimumSize</code>
nodes.
A chain only consists of elements with the same <code>edgeDirectedness</code> and <code>nodeType</code>. More precisely, the <code>edgeDirectedness</code> is
considered as follows: A substructure is only identified as such if all edges are either undirected or consistently
directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
The smallest <code>minimumSize</code> value that could be considered is <code>3</code>.
<h2>Note</h2>
If no <code>edgeDirectedness</code> is specified, all edges are treated as undirected. Furthermore, if no <code>nodeType</code> is specified, all
node are considered to be of the same type.
@param graph the input graph
@param minimumSize the minimum number of nodes of a chain (chains with less nodes are ignored)
@param [nodeType=null] the type of each node (nodes returning the equal objects are considered to be of the same type)
@param [edgeDirectedness=null] the directedness of each edge
@returns a list of {@link Substructure}s that represent the chains
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures%23getChains(Graph%2Cnumber%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static getChains(graph: Graph | null, minimumSize: number, nodeType?: IDataProvider | null, edgeDirectedness?: IDataProvider | null): IList<any> | null;
/**
Returns a list of {@link Substructure}s that represent the chains in the specified graph with at least <code>minimumSize</code>
nodes.
A chain only consists of elements with the same <code>edgeDirectedness</code> and <code>nodeType</code>. More precisely, the <code>edgeDirectedness</code> is
considered as follows: A substructure is only identified as such if all edges are either undirected or consistently
directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
The smallest <code>minimumSize</code> value that could be considered is <code>3</code>.
<h2>Note</h2>
If no <code>edgeDirectedness</code> is specified, all edges are treated as undirected. Furthermore, if no <code>nodeType</code> is specified, all
node are considered to be of the same type.
@param options The parameters to pass.
@param options.graph the input graph
@param options.minimumSize the minimum number of nodes of a chain (chains with less nodes are ignored)
@param [options.nodeType=null] the type of each node (nodes returning the equal objects are considered to be of the same type)
@param [options.edgeDirectedness=null] the directedness of each edge
@returns a list of {@link Substructure}s that represent the chains
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures%23getChains(Graph%2Cnumber%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static getChains(options: {
graph: Graph | null;
minimumSize: number;
nodeType?: IDataProvider | null;
edgeDirectedness?: IDataProvider | null;
}): IList<any> | null;
/**
Returns a list of {@link Substructure}s that represent the (undirected) cliques in the specified graph with at
least <code>minimumSize</code> nodes.
A clique is a subset of nodes of the same type (see <code>nodeType</code>) that are all adjacent (i.e., connected) to each other.
Note that finding a maximum clique is NP-hard. Hence, we only use a simple heuristic approach that doesn't guarantee to
find all/the largest clique. It only identifies cliques that have at least one node with at most one non-clique
neighbor. Furthermore, a node can only be contained in a single clique, i.e., the returned cliques are non-overlapping.
<h2>Note</h2>
A clique only consists of elements with the same <code>nodeType</code>, i.e., nodes associated with equal objects. If no <code>nodeType</code> is
specified, all node are considered to be of the same type.
<h2>Note</h2>
The smallest <code>minimumSize</code> value that could be considered is <code>3</code>.
@param graph the input graph
@param minimumSize the minimum size of a clique (cliques with less nodes are ignored)
@param [nodeType=null] the type of each node (nodes returning the equal objects are considered to be of the same type). If none is provided,
all nodes are considered to be of the same type
@returns a list of {@link Substructure}s that represent the cliques
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures%23getCliques(Graph%2Cnumber%2CIDataProvider) Online Documentation}*/
static getCliques(graph: Graph | null, minimumSize: number, nodeType?: IDataProvider | null): IList<any> | null;
/**
Returns a list of {@link Substructure}s that represent the cycles in the specified graph with at least <code>minimumSize</code>
nodes.
A <em>cycle</em> is a simple edge path in which the first and last nodes are identical. The algorithm only considers cycles in which
at most one edge connects a cycle node with the rest of the graph (i.e. isolated cycles that are connected with one
edge). Furthermore, a cycle only consists of elements with the same <code>edgeDirectedness</code> and <code>nodeType</code>.
The <code>edgeDirectedness</code> is considered as follows: A substructure is only identified as such if all edges are either
undirected or consistently directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
The smallest <code>minimumSize</code> value that could be considered is <code>3</code>.
<h2>Note</h2>
If no <code>edgeDirectedness</code> is specified, all edges are treated as undirected. Furthermore, if no <code>nodeType</code> is specified, all
node are considered to be of the same type.
@param graph the input graph
@param minimumSize the minimum number of nodes of a cycle (cycles with less nodes are ignored)
@param [nodeType=null] the type of each node (nodes returning the equal objects are considered to be of the same type)
@param [edgeDirectedness=null] the directedness of each edge
@returns a list of {@link Substructure}s that represent the cycles
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures%23getCycles(Graph%2Cnumber%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static getCycles(graph: Graph | null, minimumSize: number, nodeType?: IDataProvider | null, edgeDirectedness?: IDataProvider | null): IList<any> | null;
/**
Returns a list of {@link Substructure}s that represent the cycles in the specified graph with at least <code>minimumSize</code>
nodes.
A <em>cycle</em> is a simple edge path in which the first and last nodes are identical. The algorithm only considers cycles in which
at most one edge connects a cycle node with the rest of the graph (i.e. isolated cycles that are connected with one
edge). Furthermore, a cycle only consists of elements with the same <code>edgeDirectedness</code> and <code>nodeType</code>.
The <code>edgeDirectedness</code> is considered as follows: A substructure is only identified as such if all edges are either
undirected or consistently directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
The smallest <code>minimumSize</code> value that could be considered is <code>3</code>.
<h2>Note</h2>
If no <code>edgeDirectedness</code> is specified, all edges are treated as undirected. Furthermore, if no <code>nodeType</code> is specified, all
node are considered to be of the same type.
@param options The parameters to pass.
@param options.graph the input graph
@param options.minimumSize the minimum number of nodes of a cycle (cycles with less nodes are ignored)
@param [options.nodeType=null] the type of each node (nodes returning the equal objects are considered to be of the same type)
@param [options.edgeDirectedness=null] the directedness of each edge
@returns a list of {@link Substructure}s that represent the cycles
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures%23getCycles(Graph%2Cnumber%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static getCycles(options: {
graph: Graph | null;
minimumSize: number;
nodeType?: IDataProvider | null;
edgeDirectedness?: IDataProvider | null;
}): IList<any> | null;
/**
Returns a list of {@link Substructure}s that represent the stars in the specified graph with at least <code>minimumSize</code>
nodes.
A <em>star</em> consists of a <code>root</code> that is connected to multiple nodes with degree one.
Since a star only consists of elements with the same <code>edgeDirectedness</code> and
<code>nodeType</code>, a root may be associated with different stars. In this case, the algorithm only returns the largest star.
The <code>edgeDirectedness</code> is considered as follows: A substructure is only identified as such if all edges are either
undirected or consistently directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
The smallest <code>minimumSize</code> value that could be considered is <code>2</code>.
<h2>Note</h2>
If no <code>edgeDirectedness</code> is specified, all edges are treated as undirected. Furthermore, if no <code>nodeType</code> is specified, all
node are considered to be of the same type.
@param graph the input graph
@param minimumSize the minimum number of degree one nodes of a star (stars with less nodes are ignored)
@param [nodeType=null] the type of each node (nodes returning the equal objects are considered to be of the same type)
@param [edgeDirectedness=null] the directedness of each edge
@returns a list of {@link Substructure}s that represent the stars
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures%23getStars(Graph%2Cnumber%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static getStars(graph: Graph | null, minimumSize: number, nodeType?: IDataProvider | null, edgeDirectedness?: IDataProvider | null): IList<any> | null;
/**
Returns a list of {@link Substructure}s that represent the stars in the specified graph with at least <code>minimumSize</code>
nodes.
A <em>star</em> consists of a <code>root</code> that is connected to multiple nodes with degree one.
Since a star only consists of elements with the same <code>edgeDirectedness</code> and
<code>nodeType</code>, a root may be associated with different stars. In this case, the algorithm only returns the largest star.
The <code>edgeDirectedness</code> is considered as follows: A substructure is only identified as such if all edges are either
undirected or consistently directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
The smallest <code>minimumSize</code> value that could be considered is <code>2</code>.
<h2>Note</h2>
If no <code>edgeDirectedness</code> is specified, all edges are treated as undirected. Furthermore, if no <code>nodeType</code> is specified, all
node are considered to be of the same type.
@param options The parameters to pass.
@param options.graph the input graph
@param options.minimumSize the minimum number of degree one nodes of a star (stars with less nodes are ignored)
@param [options.nodeType=null] the type of each node (nodes returning the equal objects are considered to be of the same type)
@param [options.edgeDirectedness=null] the directedness of each edge
@returns a list of {@link Substructure}s that represent the stars
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures%23getStars(Graph%2Cnumber%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static getStars(options: {
graph: Graph | null;
minimumSize: number;
nodeType?: IDataProvider | null;
edgeDirectedness?: IDataProvider | null;
}): IList<any> | null;
/**
Returns a list of {@link Substructure}s that represent the trees in the specified graph with at least <code>minimumSize</code>
nodes.
The <code>root</code> of a subtree is the only node which may have non-tree edges. Furthermore, a subtree only consists of elements
with the same <code>nodeType</code>.
The <code>edgeDirectedness</code> is considered as follows: A substructure is only identified as such if all edges are either
undirected or consistently directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
The smallest <code>minimumSize</code> value that could be considered is <code>3</code>.
<h2>Note</h2>
A chain or star structure is also considered to be a tree.
<h2>Note</h2>
If no <code>edgeDirectedness</code> is specified, all edges are treated as undirected. Furthermore, if no <code>nodeType</code> is specified, all
node are considered to be of the same type.
@param graph the input graph
@param minimumSize the minimum size of a tree (trees with less nodes are ignored)
@param [nodeType=null] the type of each node (nodes returning the equal objects are considered to be of the same type)
@param [edgeDirectedness=null] the directedness of each edge
@returns a list of {@link Substructure}s that represent the subtrees
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures%23getTrees(Graph%2Cnumber%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static getTrees(graph: Graph | null, minimumSize: number, nodeType?: IDataProvider | null, edgeDirectedness?: IDataProvider | null): IList<any> | null;
/**
Returns a list of {@link Substructure}s that represent the trees in the specified graph with at least <code>minimumSize</code>
nodes.
The <code>root</code> of a subtree is the only node which may have non-tree edges. Furthermore, a subtree only consists of elements
with the same <code>nodeType</code>.
The <code>edgeDirectedness</code> is considered as follows: A substructure is only identified as such if all edges are either
undirected or consistently directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
The smallest <code>minimumSize</code> value that could be considered is <code>3</code>.
<h2>Note</h2>
A chain or star structure is also considered to be a tree.
<h2>Note</h2>
If no <code>edgeDirectedness</code> is specified, all edges are treated as undirected. Furthermore, if no <code>nodeType</code> is specified, all
node are considered to be of the same type.
@param options The parameters to pass.
@param options.graph the input graph
@param options.minimumSize the minimum size of a tree (trees with less nodes are ignored)
@param [options.nodeType=null] the type of each node (nodes returning the equal objects are considered to be of the same type)
@param [options.edgeDirectedness=null] the directedness of each edge
@returns a list of {@link Substructure}s that represent the subtrees
@see {@link https://docs.yworks.com/yfileshtml/#/api/Substructures%23getTrees(Graph%2Cnumber%2CIDataProvider%2CIDataProvider) Online Documentation}*/
static getTrees(options: {
graph: Graph | null;
minimumSize: number;
nodeType?: IDataProvider | null;
edgeDirectedness?: IDataProvider | null;
}): IList<any> | null;
static $class: Class;
private $_Substructures: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Substructures;
}
export interface TransitivityAlgorithm extends YObject {
}
/**
This class provides algorithms to compute reachability information for directed, acyclic graphs.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To calculate the transitive closure or reduction
of an {@link IGraph} use {@link TransitiveClosure} or {@link TransitiveReduction} instead.
<b>Definitions</b>
<ul>
<li><b>Reflexive, transitive closure:</b> Let <code>G = (V,E)</code> be a directed acyclic graph. The <b>reflexive, transitive closure</b> of <code>G</code> is a graph which contains edge <code>(v,w)</code> only if there exists a path
from <code>v</code> to <code>w</code> in <code>G</code>.
<li><b>Transitive reduction:</b> Let <code>G = (V,E)</code> be a directed acyclic graph. The <b>transitive reduction</b> of <code>G</code> is a graph which contains edge <code>(v,w)</code> only if there exists no path
from <code>v</code> to <code>w</code> in <code>G</code> of length <code>2</code> or more.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitivityAlgorithm Online Documentation}*/
export class TransitivityAlgorithm {
/**
Calculates the transitive closure for a directed acyclic graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate the transitive closure for an {@link IGraph}
use {@link TransitiveClosure} instead.
Given a <code>G = (V,E)</code> be a directed acyclic graph. The <i>reflexive, transitive closure</i> of <code>G</code> is a graph which contains edge <code>(v,w)</code> only if there exists a
path from <code>v</code> to <code>w</code> in <code>G</code>.
<h2>Note</h2>
This implementation produces the transitive closure and not the reflexive, transitive closure of the specified graph,
since no self-loops are added to the specified graph.
@param graph the input graph to which this method will add transitive edges, if necessary
@param [addedEdges=null] a {@link EdgeList list} that will be filled during the execution and contains the edges that have been added to the graph by this method
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitivityAlgorithm%23transitiveClosure(Graph%2CEdgeList) Online Documentation}*/
static transitiveClosure(graph: Graph | null, addedEdges?: EdgeList | null): void;
/**
Creates the transitive edges that connect the <code>visible</code> nodes in the specified graph.
The algorithm creates a <code>transitive edge</code> between two visible nodes if:
<ul>
<li>there is a (directed/undirected) path between the nodes using only invisible nodes as intermediate nodes
<li>and there is not already an edge between the two nodes
</ul>
@param graph the original graph that contains both the visible and invisible nodes
@param visibleNode specifies which nodes should be considered as <code>visible</code>
@param directed whether or not the edges should be considered as directed
@returns the {@link EdgeList} of created transitive edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitivityAlgorithm%23transitiveEdges(Graph%2CIDataProvider%2Cboolean) Online Documentation}*/
static transitiveEdges(graph: Graph | null, visibleNode: IDataProvider | null, directed: boolean): EdgeList | null;
/**
Calculates the transitive reduction for a directed acyclic graph.
<b>Note:</b> This method works with instances of {@link Graph}. To calculate the transitive reduction for an {@link IGraph}
use {@link TransitiveReduction} instead.
The transitive edges in the graph will be removed by this method.
Given <code>G = (V,E)</code> be a directed acyclic graph. The <i>transitive reduction</i> of <code>G</code> is a graph which contains edge <code>(v,w)</code> only if there exists no path
from <code>v</code> to <code>w</code> in <code>G</code> of length <code>2</code> or more.
<b>Warning:</b> This implementation is costly in terms of memory, since a (<code>n</code> x <code>n</code>)-matrix is allocated to store reach data.
@param graph the input graph
@param [transitiveEdges=null] a {@link EdgeList list} that will be filled during the execution and contains all transitive edges of the given graph; removal of these
edges will yield the transitive reduction of the graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitivityAlgorithm%23transitiveReduction(Graph%2CEdgeList) Online Documentation}*/
static transitiveReduction(graph: Graph | null, transitiveEdges?: EdgeList | null): void;
static $class: Class;
private $_Transitivity: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TransitivityAlgorithm;
}
export interface TreeAlgorithm extends YObject {
}
/**
This class provides diverse algorithms and services for tree-structured graphs or subgraphs.
<b>Note:</b> Methods of this class work with instances of {@link Graph}. To analyze {@link IGraph} instances that are
directed trees use {@link TreeAnalyzer} instead.
<b>Definitions</b>
<ul>
<li><b>Tree:</b> An acyclic graph, in which any pair of vertices (nodes) is connected through a path. If one vertex of a tree is
distinguished from the other vertices, then this vertex is called the <b>root</b> and the tree is called a <b>rooted tree</b>.
<li><b>Directed rooted tree:</b> A rooted tree where edges are directed from the root to the leaves.
<li><b>Depth:</b> The depth of a vertex in a rooted tree is the number of edges of the unique path between this vertex and the root.
<li><b>Parent:</b> In a rooted tree a vertex <code>v</code> is called parent of a vertex <code>w</code> if <code>v</code> and <code>w</code> are adjacent (i.e. connected by an edge) and the
unique path between <code>w</code> and the root contains <code>v</code>. Note that each vertex except the root has exactly one parent.
<li><b>Children:</b> In a rooted tree a vertex <code>w</code> is called child of a vertex <code>v</code> if <code>v</code> is the parent of <code>w</code>. A vertex may have several children.
<li><b>N-ary tree:</b> A directed rooted tree where each node has a maximum of <code>n</code> children.
<li><b>Forest:</b> A graph whose connected components are trees.
<li><b>Leaf:</b> A leaf <code>v</code> is a node with out-degree (i.e., the number of edges having <code>v</code> as a target) zero if the input is a directed
rooted tree, and a node with degree (i.e., the number of edges incident to <code>v</code>) one, otherwise.
<li><b>Subtree:</b> A subtree of a tree <code>T</code> is a subgraph of <code>T</code> which is also a tree.
<li><b>Nearest or Lowest or Least common ancestor:</b> The nearest common ancestor of two nodes <code>u</code> and <code>v</code> in a tree graph is the shared ancestor of <code>u</code> and <code>v</code> that is located
farthest from the root.
<li><b>Eccentricity:</b> The eccentricity of a tree node is the maximum distance to any other node.
<li><b>Center node:</b> The center of a tree is the set of nodes that have minimal eccentricity.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm Online Documentation}*/
export class TreeAlgorithm {
/**
Collects all nodes of the subtree starting with root.
<b>Note:</b> This method works with instances of {@link Graph}. To retrieve a subtree of a directed tree in an {@link IGraph}
use {@link TreeAnalyzer#getSubtree} instead.
@param root the root of the subtree
@param nodes the list that is filled with the nodes of the subtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23collectSubtree(YNode%2CYNodeList) Online Documentation}*/
static collectSubtree(root: YNode | null, nodes: YNodeList | null): void;
/**
Converts the given tree to a directed rooted tree with the given node as root element by reversing some edges.
A list of all reversed edges will be returned by this method.
@param tree the given tree
@param [root=null] the given root element
@returns an {@link EdgeList} containing the reversed edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23directTree(Graph%2CYNode) Online Documentation}*/
static directTree(tree: Graph | null, root?: YNode | null): EdgeList | null;
/**
Returns the center node of an undirected tree.
The center node has the property of inducing a minimum depth tree when being used as the root of that tree.
@param tree the given undirected tree
@returns the center node of the given undirected tree
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getCenterRoot(Graph) Online Documentation}*/
static getCenterRoot(tree: Graph | null): YNode | null;
/**
Returns all leaf nodes of the given tree.
<b>Note:</b> This method works with instances of {@link Graph}. To retrieve all leaf nodes of a directed tree in an {@link IGraph}
use {@link TreeAnalyzer#getLeafNodes} instead.
A leaf node is a node with <code>outdegree == 0</code> if the input is a directed rooted tree, and a node with
<code>degree == 1</code>, otherwise.
@param tree the given tree
@param directedRootedTree <code>true</code> if the algorithm should consider the tree as directed, <code>false</code> otherwise
@returns a {@link YNodeList} that contains all leaf nodes of the given tree
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getLeafNodes(Graph%2Cboolean) Online Documentation}*/
static getLeafNodes(tree: Graph | null, directedRootedTree: boolean): YNodeList | null;
/**
Returns the nearest common ancestor of a subset of nodes within a directed rooted tree.
<b>Note:</b> This method works with instances of {@link Graph}. To determine the nearest common ancestor of nodes in a
directed tree in an {@link IGraph} use GetNearestCommonAncestor instead.
It is not part of the given subset.
@param tree the given directed rooted tree
@param root the root of the tree
@param rootedDownward <code>true</code> if the tree is directed from the root to the leaves, <code>false</code> otherwise
@param nodes the subset of nodes
@returns the {@link YNode nearest common ancestor} of the given subset of nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getNearestCommonAncestor(Graph%2CYNode%2Cboolean%2CYNodeList) Online Documentation}*/
static getNearestCommonAncestor(tree: Graph | null, root: YNode | null, rootedDownward: boolean, nodes: YNodeList | null): YNode | null;
/**
Returns a possible root for the given (undirected) tree.
<b>Note:</b> This method works with instances of {@link Graph}. To determine the root of a directed tree in an {@link IGraph}
use {@link TreeAnalyzer#getRoot} instead.
More precisely:
<ul>
<li>If the input is a directed rooted tree or reversed directed rooted tree, it returns the corresponding root node.
<li>If the input is a tree, the method returns a maximum weight center node as defined in
{@link TreeAlgorithm#getWeightedCenterNode}.
<li>If the input is not a tree, a node with <code>indegree == 0</code> (or <code>outdegree == 0</code>) is returned.
</ul>
@param tree the given tree
@returns a possible root for the given tree
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getRoot(Graph) Online Documentation}*/
static getRoot(tree: Graph | null): YNode | null;
/**
Returns the depths of each subtree of a rooted directed tree.
@param tree a rooted directed tree graph
@param subtreeDepthMap the {@link INodeMap} that will be filled during the execution with the depth of the subtree rooted at each
node
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getSubTreeDepths(Graph%2CINodeMap) Online Documentation}*/
static getSubTreeDepths(tree: Graph | null, subtreeDepthMap: INodeMap | null): void;
/**
Returns the size (number of nodes) of each subtree of a rooted directed tree.
@param tree a rooted directed tree graph
@param subtreeSizeMap the {@link INodeMap} that will be filled during the execution with the size of the subtree rooted at each
node
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getSubTreeSizes(Graph%2CINodeMap) Online Documentation}*/
static getSubTreeSizes(tree: Graph | null, subtreeSizeMap: INodeMap | null): void;
/**
Returns an array of {@link EdgeList} objects each containing edges that belong to a maximal directed subtree
of the given graph.
This method can also be applied to the result obtained by
{@link TreeAlgorithm#getUndirectedTreeNodes}. In this case, the subtrees are considered to be undirected.
@param graph the given graph
@param [treeNodes=null] an array of {@link YNodeList}s previously calculated by {@link TreeAlgorithm#getTreeNodes}
@returns an array of {@link EdgeList} objects each containing edges that belong to a maximal subtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getTreeEdges(Graph%2CArray%3CYNodeList%3E) Online Documentation}*/
static getTreeEdges(graph: Graph | null, treeNodes?: YNodeList[] | null): EdgeList[] | null;
/**
Returns an array of {@link YNodeList} objects each containing nodes that belong to a maximal directed subtree
of the given graph.
For each list of tree nodes, the first node element is the root of a tree. On each such root, all outgoing edges connect
to nodes in the subtree and each in-degree of the root is at least two.
@param graph the given graph
@returns an array of {@link YNodeList} objects each containing nodes that belong to a maximal directed subtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getTreeNodes(Graph) Online Documentation}*/
static getTreeNodes(graph: Graph | null): YNodeList[] | null;
/**
Returns an array of {@link YNodeList} objects each containing nodes that belong to a maximal undirected subtree
of the given graph.
For each list of tree nodes, the first node is the only node of the subtree that may be incident to non-tree edges.
@param graph the given graph
@returns an array of {@link YNodeList} objects each containing nodes that belong to a maximal undirected subtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getUndirectedTreeNodes(Graph) Online Documentation}*/
static getUndirectedTreeNodes(graph: Graph | null): YNodeList[] | null;
/**
Finds a node used by the greatest number of all (undirected) paths interconnecting all nodes with each other.
The number of paths per node are stored in the given {@link INodeMap}.
@param tree the given tree
@param [intWeight=null] the {@link INodeMap} that holds the number of paths per node
@returns a {@link YNode} used by the greatest number of all undirected paths
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23getWeightedCenterNode(Graph%2CINodeMap) Online Documentation}*/
static getWeightedCenterNode(tree: Graph | null, intWeight?: INodeMap | null): YNode | null;
/**
Checks whether or not the given graph is a forest.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is a forest
use {@link GraphStructureAnalyzer#isForest} instead.
If
<code>directedRootedTree == true</code>, each component has to be a directed rooted tree. Otherwise, each component has to be an
undirected tree.
<h2>Note</h2>
<code>isForest(graph, true) => isForest(graph, false)</code>
@param graph the given graph
@param directedRootedTree <code>true</code> if the algorithm should check for directed rooted trees, <code>false</code> otherwise
@returns <code>true</code> if the given graph is a forest, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23isForest(Graph%2Cboolean) Online Documentation}*/
static isForest(graph: Graph | null, directedRootedTree: boolean): boolean;
/**
Checks whether or not the given graph is a forest, that is, a graph whose connected components are directed rooted
trees.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is a forest
use {@link GraphStructureAnalyzer#isForest} instead.
@param graph the given graph
@returns <code>true</code> if the given graph is a forest, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23isForest(Graph) Online Documentation}*/
static isForest(graph: Graph | null): boolean;
/**
Checks whether or not the given graph is a directed rooted tree in which each node has a maximum of <code>n</code> children.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is an <i>n</i>-ary
tree use {@link GraphStructureAnalyzer#isNaryTree} instead.
@param graph the given graph
@param n the allowed maximum of children
@returns <code>true</code> if the given graph is a n-ary tree, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23isNaryTree(Graph%2Cnumber) Online Documentation}*/
static isNaryTree(graph: Graph | null, n: number): boolean;
/**
Checks whether or not the given graph is a directed rooted tree.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is a directed
tree use {@link GraphStructureAnalyzer#isTree} instead.
<h2>Note</h2>
<code>isRootedTree(graph) => isTree(graph)</code>
@param graph the given graph
@returns <code>true</code> if the given graph is a directed rooted tree, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23isRootedTree(Graph) Online Documentation}*/
static isRootedTree(graph: Graph | null): boolean;
/**
Checks whether or not the given graph is an undirected tree.
<b>Note:</b> This method works with instances of {@link Graph}. To determine whether an {@link IGraph} is a tree use
{@link GraphStructureAnalyzer#isTree} instead.
<h2>Note</h2>
<code>isRootedTree(graph) => isTree(graph)</code>.
@param graph the given graph
@returns <code>true</code> if the given graph is an undirected tree, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAlgorithm%23isTree(Graph) Online Documentation}*/
static isTree(graph: Graph | null): boolean;
static $class: Class;
private $_Trees: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeAlgorithm;
}
export interface Comparers extends YObject {
}
/**
This class provides access to some Comparator instances that are commonly used in yFiles.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers Online Documentation}*/
export class Comparers {
/**
Compares the specified floating point numbers.
Returns a negative integer, zero, or a positive integer as the first argument is less than, equal to, or greater than
the second.
<b>Warning:</b> This method has no special handling for <code>NaN</code>. If you need <code>NaN</code>-safe comparison, implement your own comparison instead.
@param d1 the first number to compare.
@param d2 the second number to compare.
@returns a negative integer, zero, or a positive integer as the first argument is less than, equal to, or greater than the
second.
@see IComparer#compare
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers%23compare(number%2Cnumber) Online Documentation}*/
static compare(d1: number, d2: number): number;
/**
Returns a {@link IComparer} that compares to Objects of type Comparable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers%23createComparableComparer() Online Documentation}*/
static createComparableComparer(): IComparer<any> | null;
/**
Returns a {@link IComparer} that compares objects of arbitrary type.
Two objects are compared by comparing the {@link IComparable} instances the given data provider returns for each of
these objects.
@param dp a data provider that must return a {@link IComparable} for each object that is being compared by this comparator.
@returns a {@link IComparer} that compares arbitrary objects.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers%23createComparableDataComparer(IDataProvider) Online Documentation}*/
static createComparableDataComparer(dp: IDataProvider | null): IComparer<any> | null;
/**
Returns a {@link IComparer} that compares objects of arbitrary type.
Two objects are compared by comparing the int value the given data provider returns for each of these objects.
@param dp a data provider that must return an int value for each object that is being compared by this comparator.
@returns a {@link IComparer} that compares arbitrary objects.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers%23createIntDataComparer(IDataProvider) Online Documentation}*/
static createIntDataComparer(dp: IDataProvider | null): IComparer<any> | null;
/**
Returns a {@link IComparer} that compares objects of type {@link Edge}.
Two edges are compared by comparing their source nodes. Each source node <code>e.source()</code> in turn is compared by the int value
provided by the given data provider: <code>dp.getInt(e.source())</code>.
@param dp a data provider that must return an int value for the source node of each edge being compared.
@returns a {@link IComparer} that compares edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers%23createIntDataSourceComparer(IDataProvider) Online Documentation}*/
static createIntDataSourceComparer(dp: IDataProvider | null): IComparer<any> | null;
/**
Returns a {@link IComparer} that compares objects of type {@link Edge}.
Two edges are compared by comparing their target nodes. Each target node <code>e.target()</code> in turn is compared by the int value
provided by the given data provider: <code>dp.getInt(e.target())</code>.
@param dp a data provider that must return an int value for the target node of each edge being compared.
@returns a {@link IComparer} that compares edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers%23createIntDataTargetComparer(IDataProvider) Online Documentation}*/
static createIntDataTargetComparer(dp: IDataProvider | null): IComparer<any> | null;
/**
Returns a {@link IComparer} that compares objects of arbitrary type.
Two objects are compared by comparing the double value the given data provider returns for each of these objects.
@param dp a data provider that must return a double value for each object that is being compared by this comparator.
@returns a {@link IComparer} that compares arbitrary objects.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers%23createNumberDataComparer(IDataProvider) Online Documentation}*/
static createNumberDataComparer(dp: IDataProvider | null): IComparer<any> | null;
/**
Returns a {@link IComparer} that compares objects of type {@link Edge}.
Two edges are compared by comparing their source nodes. Each source node <code>e.source()</code> in turn is compared by the double
value provided by the given data provider: <code>dp.getDouble(e.source())</code>.
@param dp a data provider that must return a double value for the source node of each edge being compared.
@returns a {@link IComparer} that compares edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers%23createNumberDataSourceComparer(IDataProvider) Online Documentation}*/
static createNumberDataSourceComparer(dp: IDataProvider | null): IComparer<any> | null;
/**
Returns a {@link IComparer} that compares objects of type {@link Edge}.
Two edges are compared by comparing their target nodes. Each target node <code>e.target()</code> in turn is compared by the double
value provided by the given data provider: <code>dp.getDouble(e.target())</code>.
@param dp a data provider that must return a double value for the target node of each edge being compared.
@returns a {@link IComparer} that compares edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Comparers%23createNumberDataTargetComparer(IDataProvider) Online Documentation}*/
static createNumberDataTargetComparer(dp: IDataProvider | null): IComparer<any> | null;
static $class: Class;
private $_Comparers: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Comparers;
}
export interface Cursors extends YObject {
}
/**
Provides utility methods for working with {@link ICursor cursors}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursors Online Documentation}*/
export class Cursors {
/**
Creates a new cursor that provides a logical view on the concatenation of the two given cursors.
@param c1 - first concatenation argument
@param c2 - second concatenation argument
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursors%23concatenate(ICursor%2CICursor) Online Documentation}*/
static concatenate(c1: ICursor | null, c2: ICursor | null): ICursor | null;
/**
Creates a cursor view of the given collection.
Creates an ICursor view of the given collection.
Note that the returned cursor does not support the {@link ICursor#prev prev} and {@link ICursor#toLast toLast} operations.
Type parameter <code>T</code>.
@param c The collection.
@returns The cursor view of the given collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursors%23createCursor(ICollection%3CT%3E) Online Documentation}*/
static createCursor<T>(c: ICollection<T> | null | ICollectionConvertible<T>): ICursor | null;
/**
Creates a cursor view of the given collection.
Note that the returned cursor does not support the operations {@link ICursor#prev prev}, {@link ICursor#toLast toLast}, and
{@link IEdgeCursor#cyclicPrev}.
Type parameter <code>T</code>.
@param c The collection.
@returns an {@link IEdgeCursor} view of the given collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursors%23createEdgeCursor(ICollection%3CT%3E) Online Documentation}*/
static createEdgeCursor<T>(c: ICollection<T> | null | ICollectionConvertible<T>): IEdgeCursor | null;
/**
Creates a cursor view of the given collection.
Note that the returned cursor does not support the operations {@link ICursor#prev prev}, {@link ICursor#toLast toLast}, and
{@link INodeCursor#cyclicPrev}.
Type parameter <code>T</code>.
@param c The collection.
@returns an {@link INodeCursor} view of the given collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursors%23createNodeCursor(ICollection%3CT%3E) Online Documentation}*/
static createNodeCursor<T>(c: ICollection<T> | null | ICollectionConvertible<T>): INodeCursor | null;
/**
Creates or fills an array with the values provided by the cursor
@param cursor the cursor
@param dest the array to fill with the values or <code>null</code> if the method should create the array itself
@returns dest or a newly created array filled with the values from cursor
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cursors%23toArray(ICursor%2CArray%3CObject%3E) Online Documentation}*/
static toArray(cursor: ICursor | null, dest: any[] | null): any[] | null;
static $class: Class;
private $_Cursors: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Cursors;
}
export interface DataProviderBase extends YObject, IDataProvider {
}
/**
An abstract adapter class for providing data.
The data provision methods in this class throw a {@link Exception NotSupportedError} and {@link DataProviderBase#defined}
always returns <code>false</code>.
This class exists as a convenience for creating data provider objects.
Extend this class to provide either typed or untyped data for a certain lookup domain.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviderBase Online Documentation}*/
export abstract class DataProviderBase {
/**
Returns <code>false</code> for all data holders.
Subclasses should override this method to make clear for which data holders there is a value accessible via this data
provider.
@param dataHolder
@returns <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviderBase%23defined(Object) Online Documentation}*/
defined(dataHolder: any | null): boolean;
/**
Subclasses may override this method to provide access to object values.
@param dataHolder
@throws {Exception} <b>(NotSupportedError)</b> unless overwritten.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviderBase%23get(Object) Online Documentation}*/
get(dataHolder: any | null): any | null;
/**
Subclasses may override this method to provide access to boolean values.
@param dataHolder
@throws {Exception} <b>(NotSupportedError)</b> unless overwritten.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviderBase%23getBoolean(Object) Online Documentation}*/
getBoolean(dataHolder: any | null): boolean;
/**
Subclasses may override this method to provide access to integer values.
@param dataHolder
@throws {Exception} <b>(NotSupportedError)</b> unless overwritten.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviderBase%23getInt(Object) Online Documentation}*/
getInt(dataHolder: any | null): number;
/**
Subclasses may override this method to provide access to double values.
@param dataHolder
@throws {Exception} <b>(NotSupportedError)</b> unless overwritten.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviderBase%23getNumber(Object) Online Documentation}*/
getNumber(dataHolder: any | null): number;
static $class: Class;
private $_DataProviderAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DataProviderBase;
}
export interface DataProviders extends YObject {
}
/**
This class provides convenience and transformation services for {@link IDataProvider}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders Online Documentation}*/
export class DataProviders {
/**
Returns a {@link IDataProvider} that returns the given value for each key.
@param data constant Object data returned by the created data provider.
@returns a data provider view of a single value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createConstantDataProvider(Object) Online Documentation}*/
static createConstantDataProvider(data: any | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of an {@link Object} array defined for edges.
The {@link Object} value <code>data[edge.index()]</code> will be returned by the data provider upon the method call <code>get(edge)</code>.
@param data array data for each edge of a static graph
@returns a data provider view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createEdgeDataProvider(Array%3CObject%3E) Online Documentation}*/
static createEdgeDataProvider(data: any[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of a double, int, boolean and {@link Object} array defined for edges.
The double value <code>doubleData[edge.index()]</code> will be returned by the data provider upon the method call <code>getDouble(edge)</code>.
The int value <code>intData[edge.index()]</code> will be returned by the data provider upon the method call <code>getInt(edge)</code>.
The boolean value <code>boolData[edge.index()]</code> will be returned by the data provider upon the method call <code>getBool(edge)</code>.
The {@link Object} value <code>objectData[edge.index()]</code> will be returned by the data provider upon the method call <code>get(edge)</code>.
@param doubleData double data for each edge of a static graph
@param intData int data for each edge of a static graph
@param boolData boolean data for each edge of a static graph
@param objectData {@link Object} data for each edge of a static graph
@returns a data provider view of the given arrays
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createEdgeDataProviderForArrays(Array%3Cnumber%3E%2CArray%3Cnumber%3E%2CArray%3Cboolean%3E%2CArray%3CObject%3E) Online Documentation}*/
static createEdgeDataProviderForArrays(doubleData: number[] | null, intData: number[] | null, boolData: boolean[] | null, objectData: any[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of a boolean array defined for edges.
The boolean value <code>data[edge.index()]</code> will be returned by the data provider upon the method call <code>getBool(edge)</code>.
@param data array data for each edge of a static graph
@returns a data provider view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createEdgeDataProviderForBoolean(Array%3Cboolean%3E) Online Documentation}*/
static createEdgeDataProviderForBoolean(data: boolean[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of an int array defined for edges.
The int value <code>data[edge.index()]</code> will be returned by the data provider upon the method call <code>getInt(edge)</code>.
@param data array data for each edge of a static graph
@returns a data provider view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createEdgeDataProviderForInt(Array%3Cnumber%3E) Online Documentation}*/
static createEdgeDataProviderForInt(data: number[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of a double array defined for edges.
The double value <code>data[edge.index()]</code> will be returned by the data provider upon the method call <code>getDouble(edge)</code>.
@param data array data for each edge of a static graph
@returns a data provider view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createEdgeDataProviderForNumber(Array%3Cnumber%3E) Online Documentation}*/
static createEdgeDataProviderForNumber(data: number[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} that returns the negated boolean values provided by another data provider.
@param data
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createNegatedDataProvider(IDataProvider) Online Documentation}*/
static createNegatedDataProvider(data: IDataProvider | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of an {@link Object} array defined for nodes.
The {@link Object} value <code>data[node.index()]</code> will be returned by the data provider upon the method call <code>get(node)</code>.
@param data array data for each node of a static graph
@returns a data provider view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createNodeDataProvider(Array%3CObject%3E) Online Documentation}*/
static createNodeDataProvider(data: any[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of a boolean array defined for nodes.
The boolean value <code>data[node.index()]</code> will be returned by the data provider upon the method call <code>getBool(node)</code>.
@param data array data for each node of a static graph
@returns a data provider view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createNodeDataProviderForBoolean(Array%3Cboolean%3E) Online Documentation}*/
static createNodeDataProviderForBoolean(data: boolean[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of an int array defined for nodes.
The int value <code>data[node.index()]</code> will be returned by the data provider upon the method call <code>getInt(node)</code>.
@param data array data for each node of a static graph
@returns a data provider view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createNodeDataProviderForInt(Array%3Cnumber%3E) Online Documentation}*/
static createNodeDataProviderForInt(data: number[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of a double array defined for nodes.
The double value <code>data[node.index()]</code> will be returned by the data provider upon the method call <code>getDouble(node)</code>.
@param data array data for each node of a static graph
@returns a data provider view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createNodeDataProviderForNumber(Array%3Cnumber%3E) Online Documentation}*/
static createNodeDataProviderForNumber(data: number[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} view of a double, int, boolean and {@link Object} array defined for nodes.
The double value <code>doubleData[node.index()]</code> will be returned by the data provider upon the method call <code>getDouble(node)</code>.
The int value <code>intData[node.index()]</code> will be returned by the data provider upon the method call <code>getInt(node)</code>.
The boolean value <code>boolData[node.index()]</code> will be returned by the data provider upon the method call <code>getBool(node)</code>.
The {@link Object} value <code>objectData[node.index()]</code> will be returned by the data provider upon the method call <code>get(node)</code>.
@param doubleData double data for each node of a static graph
@param intData int data for each node of a static graph
@param boolData boolean data for each node of a static graph
@param objectData {@link Object} data for each node of a static graph
@returns a data provider view of the given arrays
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createNodeDataProviderWithArrays(Array%3Cnumber%3E%2CArray%3Cnumber%3E%2CArray%3Cboolean%3E%2CArray%3CObject%3E) Online Documentation}*/
static createNodeDataProviderWithArrays(doubleData: number[] | null, intData: number[] | null, boolData: boolean[] | null, objectData: any[] | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} for edges that return the data provider values bound to their source nodes.
@param nodeData
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createSourceDataProvider(IDataProvider) Online Documentation}*/
static createSourceDataProvider(nodeData: IDataProvider | null): IDataProvider | null;
/**
Returns a {@link IDataProvider} for edges that return the data provider values bound to their target nodes.
@param nodeData
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataProviders%23createTargetDataProvider(IDataProvider) Online Documentation}*/
static createTargetDataProvider(nodeData: IDataProvider | null): IDataProvider | null;
static $class: Class;
private $_DataProviders: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DataProviders;
}
export interface GraphPartitionManager extends YObject {
}
/**
Provides functionality to hide and unhide partitions of nodes and their adjacent edges of a graph temporarily for
algorithmic operations.
This class can be used to temporarily hide away certain elements of a graph and to unhide that parts at a later time
again. Instances of this class keep track of graph elements that were hidden from a graph in order to make them visible
again at a later time.
Note that this class should not be used to hide elements from a Graph2D for pure hiding purposes. Since this class will
by default prevent the graph instance from firing events, other code might cease to work correctly. Use this class for
short term removal of nodes and edges, only.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager Online Documentation}*/
export class GraphPartitionManager {
/**
Instantiates a new {@link GraphPartitionManager} for the given graph.
All non-static hiding and unhiding methods will refer to the given graph.
@param graph
@param partitionId
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager Online Documentation}*/
constructor(graph: Graph | null, partitionId: IDataProvider | null);
/**
Assures that only nodes are visible in the graph that are associated with the given partitionId.
@param partitionId the partitionId for the nodes that will be made visible
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23displayPartition(Object) Online Documentation}*/
displayPartition(partitionId: any | null): void;
/**
Hides the given list of nodes from the graph.
The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
@param nl
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hide(YNodeList) Online Documentation}*/
hide(nl: YNodeList): void;
/**
Hides the given list of edges from the graph.
The hidden edges will be stored so that they can be unhidden again at a later time.
@param el
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hide(EdgeList) Online Documentation}*/
hide(el: EdgeList): void;
/**
Hides the given edge from the graph.
The hidden edge will be stored so that they can be unhidden again at a later time.
@param e
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hide(Edge) Online Documentation}*/
hide(e: Edge): void;
/**
Hides the given node and all it's adjacent edges from the graph.
The hidden elements will be stored so that they can be unhidden again at a later time.
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hide(YNode) Online Documentation}*/
hide(v: YNode): void;
/**
Hides all nodes and edges from this graph.
The hidden elements will be stored so that they can be unhidden again at a later time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hideAll() Online Documentation}*/
hideAll(): void;
/**
Hides the given edges from the graph.
The hidden edges will be stored so that they can be unhidden again at a later time.
@param ec
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hideEdgeCursor(IEdgeCursor) Online Documentation}*/
hideEdgeCursor(ec: IEdgeCursor | null): void;
/**
Hides all edges from this graph.
The hidden edges will be stored so that they can be unhidden again at a later time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hideEdges() Online Documentation}*/
hideEdges(): void;
/**
Hides the given elements from the graph.
The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
@param cursor
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hideItemCursor(ICursor) Online Documentation}*/
hideItemCursor(cursor: ICursor | null): void;
/**
Hides multiple edges from the graph.
If there are multiple edges connecting two nodes then all but one (representative) of these edges will be hidden. The
hidden edges will be stored so that they can be unhidden again at a later time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hideMultipleEdges() Online Documentation}*/
hideMultipleEdges(): void;
/**
Hides the given nodes from the graph.
The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
@param nc
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hideNodeCursor(INodeCursor) Online Documentation}*/
hideNodeCursor(nc: INodeCursor | null): void;
/**
Hides nodes and adjacent edges that have the given partitionId associated.
@param partitionId the id
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hidePartition(Object) Online Documentation}*/
hidePartition(partitionId: any | null): void;
/**
Hides all self-loop edges from this graph.
The hidden edges will be stored so that they can be unhidden again at a later time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23hideSelfLoops() Online Documentation}*/
hideSelfLoops(): void;
/**
Initializes internal data structures using the new DataProvider.
This method must also be called whenever the content of the given DataProvider changes.
@param partitionId the data provider that holds the partitionIds for all elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23initPartitions(IDataProvider) Online Documentation}*/
initPartitions(partitionId: IDataProvider | null): void;
/**
Hides all self-loops and multiple edges from the graph.
The overall effect of this method is that the minimum number of edges are hidden from the graph such that it contains no
self-loops and no multiple edges anymore. The hidden edges will be stored so that they can be unhidden again at a later
time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23simplifyGraph() Online Documentation}*/
simplifyGraph(): void;
/**
This method will be called whenever the partition manager is requested to unhide the given node from the graph.
@param v
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23unhide(YNode) Online Documentation}*/
unhide(v: YNode): void;
/**
This method will be called whenever the partition manager is requested to unhide the given edge from the graph.
@param e
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23unhide(Edge) Online Documentation}*/
unhide(e: Edge): void;
/**
Unhides all formerly hidden elements in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23unhideAll() Online Documentation}*/
unhideAll(): void;
/**
Unhides all formerly hidden edges in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23unhideEdges() Online Documentation}*/
unhideEdges(): void;
/**
Unhides all formerly hidden nodes in the graph.
Note that this method does not unhide hidden edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23unhideNodes() Online Documentation}*/
unhideNodes(): void;
/**
Unhides nodes that have the given partitionId associated.
@param partitionId the id
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23unhidePartition(Object) Online Documentation}*/
unhidePartition(partitionId: any | null): void;
/**
Gets the <code>Graph</code> for which this partition manager was created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionManager%23graph Online Documentation}*/
readonly graph: Graph | null;
static $class: Class;
private $_GraphPartitionManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphPartitionManager;
}
export interface LayoutGraphHider extends YObject {
}
/**
Provides functionality to hide and unhide nodes and edges of a graph temporarily for algorithmic operations.
This class can be used to temporarily hide away certain elements of a graph and to unhide that parts at a later time
again. Instances of this class keep track of graph elements that were hidden from a graph in order to make them visible
again at a later time.
Note that this class should not be used to hide elements from a Graph2D for pure hiding purposes. Since this class will
by default prevent the graph instance from firing events, other code might cease to work correctly. Use this class for
short term removal of nodes and edges, only.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider Online Documentation}*/
export class LayoutGraphHider {
/**
Instantiates a new GraphHider for the given graph.
All non-static hiding and unhiding methods will refer to the given graph.
@param g
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider Online Documentation}*/
constructor(g: Graph | null);
/**
The edges that are currently hidden
@returns a cursor of the currently hidden edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23getHiddenEdgeCursor() Online Documentation}*/
getHiddenEdgeCursor(): IEdgeCursor | null;
/**
The nodes that are currently hidden
@returns a cursor of the currently hidden nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23getHiddenNodeCursor() Online Documentation}*/
getHiddenNodeCursor(): INodeCursor | null;
/**
Hides the given list of nodes from the graph.
The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
@param nl
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hide(YNodeList) Online Documentation}*/
hide(nl: YNodeList): void;
/**
Hides the given list of edges from the graph.
The hidden edges will be stored so that they can be unhidden again at a later time.
@param el
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hide(EdgeList) Online Documentation}*/
hide(el: EdgeList): void;
/**
Hides the given edge from the graph.
The hidden edge will be stored so that they can be unhidden again at a later time.
@param e
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hide(Edge) Online Documentation}*/
hide(e: Edge): void;
/**
Hides the given node and all it's adjacent edges from the graph.
The hidden elements will be stored so that they can be unhidden again at a later time.
@param v
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hide(YNode) Online Documentation}*/
hide(v: YNode): void;
/**
Hides all nodes and edges from this graph.
The hidden elements will be stored so that they can be unhidden again at a later time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hideAll() Online Documentation}*/
hideAll(): void;
/**
Hides the given edges from the graph.
The hidden edges will be stored so that they can be unhidden again at a later time.
@param ec
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hideEdgeCursor(IEdgeCursor) Online Documentation}*/
hideEdgeCursor(ec: IEdgeCursor | null): void;
/**
Hides all edges from this graph.
The hidden edges will be stored so that they can be unhidden again at a later time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hideEdges() Online Documentation}*/
hideEdges(): void;
/**
Hides the given elements from the graph.
The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
@param cursor
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hideItemCursor(ICursor) Online Documentation}*/
hideItemCursor(cursor: ICursor | null): void;
/**
Hides multiple edges from the graph.
If there are multiple edges connecting two nodes then all but one (representative) of these edges will be hidden. The
hidden edges will be stored so that they can be unhidden again at a later time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hideMultipleEdges() Online Documentation}*/
hideMultipleEdges(): void;
/**
Hides the given nodes from the graph.
The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
@param nc
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hideNodeCursor(INodeCursor) Online Documentation}*/
hideNodeCursor(nc: INodeCursor | null): void;
/**
Hides all self-loop edges from this graph.
The hidden edges will be stored so that they can be unhidden again at a later time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hideSelfLoops() Online Documentation}*/
hideSelfLoops(): void;
/**
Hides the subgraph induced by the given edges from the given graph.
The induced subgraph defined by the given edges consists of the given edges and all nodes that are solely connected to
the rest of the graph by the given edges.
@param graph
@param ec
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hideSubgraph(Graph%2CIEdgeCursor) Online Documentation}*/
static hideSubgraph(graph: Graph | null, ec: IEdgeCursor | null): void;
/**
Hides all self-loops and multiple edges from the graph.
The overall effect of this method is that the minimum number of edges are hidden from the graph such that it contains no
self-loops and no multiple edges anymore. The hidden edges will be stored so that they can be unhidden again at a later
time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23simplifyGraph() Online Documentation}*/
simplifyGraph(): void;
/**
This method will be called whenever the hider is requested to unhide the given node from the graph.
@param v
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhide(YNode) Online Documentation}*/
unhide(v: YNode): void;
/**
This method will be called whenever the hider is requested to unhide the given edge from the graph.
@param e
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhide(Edge) Online Documentation}*/
unhide(e: Edge): void;
/**
Unhides all formerly hidden elements in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhideAll() Online Documentation}*/
unhideAll(): void;
/**
Unhides the given edge.
Also updates {@link LayoutGraphHider#hiddenEdges}.
Note that unhiding an edge whose source/target is not in the graph (e.g., because it's currently hidden/removed) causes
an exception. Hence, in such cases, you first have to unhide/reinsert the corresponding endpoints.
@param e the edge that will be unhidden
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhideEdge(Edge) Online Documentation}*/
unhideEdge(e: Edge | null): void;
/**
Unhides the given edges.
Also updates {@link LayoutGraphHider#hiddenEdges}.
Note that unhiding an edge whose source/target is not in the graph (e.g., because it's currently hidden/removed) causes
an exception. Hence, in such cases, you first have to unhide/reinsert the corresponding endpoints.
@param edges the edges that will be unhidden
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhideEdges(EdgeList) Online Documentation}*/
unhideEdges(edges: EdgeList | null): void;
/**
Unhides all formerly hidden edges in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhideEdges() Online Documentation}*/
unhideEdges(): void;
/**
Unhides the given node and if requested its adjacent edges.
Also updates {@link LayoutGraphHider#hiddenNodes} and {@link LayoutGraphHider#hiddenEdges}.
@param v the node that will be unhidden
@param unhideAdjacentEdges whether of not to unhide previously hidden edges connected at <code>v</code> whose other end point is not hidden, i.e. it is part of
the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhideNode(YNode%2Cboolean) Online Documentation}*/
unhideNode(v: YNode | null, unhideAdjacentEdges: boolean): void;
/**
Unhides the given nodes and if requested its adjacent edges.
Also updates {@link LayoutGraphHider#hiddenNodes} and {@link LayoutGraphHider#hiddenEdges}.
@param nodes the nodes that will be unhidden
@param unhideAdjacentEdges whether of not to unhide previously hidden edges connected at the given nodes whose other end point is not hidden, i.e.
it is part of the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhideNodes(YNodeList%2Cboolean) Online Documentation}*/
unhideNodes(nodes: YNodeList | null, unhideAdjacentEdges: boolean): void;
/**
Unhides all formerly hidden nodes in the graph.
Note that this method does not unhide hidden edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhideNodes() Online Documentation}*/
unhideNodes(): void;
/**
Unhides the subgraph induced by the given edges in the given graph.
The induced subgraph defined by the given edges consists of the given edges and all nodes that are either source or
target of at least one of the given edges.
Parts of the subgraph that are already contained in the given graph will not be unhidden and pose no problem to this
method.
@param graph
@param ec
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23unhideSubgraph(Graph%2CIEdgeCursor) Online Documentation}*/
static unhideSubgraph(graph: Graph | null, ec: IEdgeCursor | null): void;
/**
holds the list of the hidden edges in stack order
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hiddenEdges Online Documentation}*/
hiddenEdges: EdgeList | null;
/**
holds the list of the hidden nodes in stack order
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23hiddenNodes Online Documentation}*/
hiddenNodes: YNodeList | null;
/**
Gets or sets whether or not this hider should fire graph events.
By default the hider does not fire graph events.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23fireGraphEvents Online Documentation}*/
fireGraphEvents: boolean;
/**
Gets the <code>Graph</code> for which this <code>GraphHider</code> was created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphHider%23graph Online Documentation}*/
readonly graph: Graph | null;
static $class: Class;
private $_LayoutGraphHider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutGraphHider;
}
export interface Maps extends YObject {
}
/**
This class provides convenience and transformation services for {@link INodeMap} and
{@link IEdgeMap}, and other layout-related collection types.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps Online Documentation}*/
export class Maps {
/**
Create a {@link IDataMap} view of the given map.
Accessing basic value types is solved by storing the corresponding wrapper types Double, Integer and Boolean within the
given map.
@param map
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createDataMap(IMap%3CObject%2CObject%3E) Online Documentation}*/
static createDataMap(map: IMap<any, any>): IDataMap;
/**
Create an {@link IEdgeMap} view of the given map.
Accessing basic value types is solved by storing the corresponding wrapper types Double, Integer and Boolean within the
given map.
@param map
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createEdgeMap(IMap%3CObject%2CObject%3E) Online Documentation}*/
static createEdgeMap(map: IMap<any, any>): IEdgeMap;
/**
Creates a {@link IDataMap} that is based on hashing.
The preconditions specified in java.util.HashMap apply for the keys and values of this map.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHashedDataMap() Online Documentation}*/
static createHashedDataMap(): IDataMap;
/**
Creates an {@link IEdgeMap} that is based on hashing.
The preconditions specified in java.util.HashMap apply for the keys and values of this map.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHashedEdgeMap() Online Documentation}*/
static createHashedEdgeMap(): IEdgeMap;
/**
Creates a {@link INodeMap} that is based on hashing.
The preconditions specified in java.util.HashMap apply for the keys and values of this map.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHashedNodeMap() Online Documentation}*/
static createHashedNodeMap(): INodeMap;
/**
Factory method that creates a {@link IMap} that is a special hash map implementation mainly used by the layout algorithms.
Type parameter <code>K</code>.
Type parameter <code>V</code>.
@param [entries=null] The items to optionally populate the set with initially.
@returns A new instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHashMap(ICollection%3CMapEntry%3CK%2CV%3E%3E) Online Documentation}*/
static createHashMap<K, V>(entries?: ICollection<MapEntry<K, V>> | null | ICollectionConvertible<MapEntry<K, V>>): IMap<K, V> | null;
/**
Factory method that creates an {@link ICollection} implementation with set semantics that is a special hash map based implementation mainly
used by the layout algorithms.
Type parameter <code>T</code>.
@param [items=null] The items to optionally populate the set with initially.
@returns A new instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHashSet(IEnumerable%3CT%3E) Online Documentation}*/
static createHashSet<T>(items?: IEnumerable<T> | null | IEnumerableConvertible<T>): ICollection<T> | null;
/**
Creates a resettable high performance map for values of type <code>double</code>.
A high performance map is a facade to another map which will allow to reset all bound values to a default value in
constant time.
@param backingProvider
@param backingAcceptor
@param defaultValue
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHighPerformanceDoubleMap(IDataProvider%2CIDataAcceptor%2Cnumber) Online Documentation}*/
static createHighPerformanceDoubleMap(backingProvider: IDataProvider | null, backingAcceptor: IDataAcceptor | null, defaultValue: number): INodeMap | null;
/**
Creates a resettable high performance map for values of type <code>double</code>.
A high performance map is a facade to another map which will allow to reset all bound values to a default value in
constant time.
@param backingMap
@param defaultValue
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHighPerformanceDoubleMap(IDataMap%2Cnumber) Online Documentation}*/
static createHighPerformanceDoubleMap(backingMap: IDataMap | null, defaultValue: number): INodeMap | null;
/**
Creates a resettable high performance map for values of type <code>int</code>.
A high performance map is a facade to another map which will allow to reset all bound values to a default value in
constant time.
@param backingProvider
@param backingAcceptor
@param defaultValue
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHighPerformanceIntMap(IDataProvider%2CIDataAcceptor%2Cnumber) Online Documentation}*/
static createHighPerformanceIntMap(backingProvider: IDataProvider | null, backingAcceptor: IDataAcceptor | null, defaultValue: number): INodeMap | null;
/**
Creates a resettable high performance map for values of type <code>int</code>.
A high performance map is a facade to another map which will allow to reset all bound values to a default value in
constant time.
@param backingMap
@param defaultValue
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHighPerformanceIntMap(IDataMap%2Cnumber) Online Documentation}*/
static createHighPerformanceIntMap(backingMap: IDataMap | null, defaultValue: number): INodeMap | null;
/**
Creates a resettable high performance map for values of type <code>Object</code>.
A high performance map is a facade to another map which will allow to reset all bound values to a default value in
constant time.
@param backingProvider
@param backingAcceptor
@param defaultValue
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHighPerformanceMap(IDataProvider%2CIDataAcceptor%2CObject) Online Documentation}*/
static createHighPerformanceMap(backingProvider: IDataProvider | null, backingAcceptor: IDataAcceptor | null, defaultValue: any | null): INodeMap | null;
/**
Creates a resettable high performance map for values of type <code>Object</code>.
A high performance map is a facade to another map which will allow to reset all bound values to a default value in
constant time.
@param backingMap
@param defaultValue
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createHighPerformanceMap(IDataMap%2CObject) Online Documentation}*/
static createHighPerformanceMap(backingMap: IDataMap | null, defaultValue: any | null): INodeMap | null;
/**
Returns a {@link IEdgeMap} view of an {@link Object} array defined for edges.
The {@link Object} value <code>data[edge.index()]</code> will be accessed by the {@link IEdgeMap} upon the method calls <code>get(edge)</code> and
<code>set(edge,value)</code>.
<b>Warning!</b> The indices of the accessed edges must not change during the use of this map.
@param data array data for each edge of a static graph
@returns a {@link IEdgeMap} view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexEdgeMap(Array%3CObject%3E) Online Documentation}*/
static createIndexEdgeMap(data: any[]): IEdgeMap;
/**
Returns a {@link IEdgeMap} view of a boolean array defined for edges.
The boolean value <code>data[edge.index()]</code> will be accessed by the {@link IEdgeMap} upon the method calls <code>getBool(edge)</code>
and <code>setBool(edge,value)</code>.
<b>Warning!</b> The indices of the accessed edges must not change during the use of this map.
@param data array data for each edge of a static graph
@returns a {@link IEdgeMap} view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexEdgeMapForBoolean(Array%3Cboolean%3E) Online Documentation}*/
static createIndexEdgeMapForBoolean(data: boolean[]): IEdgeMap;
/**
Returns a {@link IEdgeMap} view of an int array defined for edges.
The int value <code>data[edge.index()]</code> will be accessed by the {@link IEdgeMap} upon the method calls <code>getInt(edge)</code>
and <code>setInt(edge,value)</code>.
<b>Warning!</b> The indices of the accessed edges must not change during the use of this map.
@param data array data for each edge of a static graph
@returns a {@link IEdgeMap} view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexEdgeMapForInt(Array%3Cnumber%3E) Online Documentation}*/
static createIndexEdgeMapForInt(data: number[]): IEdgeMap;
/**
Returns a {@link IEdgeMap} view of a double array defined for edges.
The double value <code>data[edge.index()]</code> will be accessed by the {@link IEdgeMap} upon the method calls <code>getDouble(edge)</code>
and <code>setDouble(edge,value)</code>.
<b>Warning!</b> The indices of the accessed edges must not change during the use of this map.
@param data array data for each edge of a static graph
@returns a {@link IEdgeMap} view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexEdgeMapForNumber(Array%3Cnumber%3E) Online Documentation}*/
static createIndexEdgeMapForNumber(data: number[]): IEdgeMap;
/**
Returns a {@link IEdgeMap} view of a double, int, boolean and {@link Object} array defined for edges.
The double value <code>doubleData[edge.index()]</code> will be accessed by the {@link IEdgeMap} upon the method calls <code>getDouble(edge)</code>
and <code>setDouble(edge,value)</code>.
The int value <code>intData[edge.index()]</code> will be accessed by the {@link IEdgeMap} upon the method calls <code>getInt(edge)</code>
and <code>setInt(edge,value)</code>.
The boolean value <code>boolData[edge.index()]</code> will be accessed by the {@link IEdgeMap} upon the method calls <code>getBool(edge)</code>
and <code>setBool(edge,value)</code>.
The {@link Object} value <code>objectData[edge.index()]</code> will be accessed by the {@link IEdgeMap} upon the method calls <code>get(edge)</code>
and <code>set(edge,value)</code>.
<b>Warning!</b> The indices of the accessed edges must not change during the use of this map.
@param doubleData double data for each edge of a static graph
@param intData int data for each edge of a static graph
@param boolData boolean data for each edge of a static graph
@param objectData {@link Object} data for each edge of a static graph
@returns a {@link IEdgeMap} view of the given arrays
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexEdgeMapFromArrays(Array%3Cnumber%3E%2CArray%3Cnumber%3E%2CArray%3Cboolean%3E%2CArray%3CObject%3E) Online Documentation}*/
static createIndexEdgeMapFromArrays(doubleData: number[], intData: number[], boolData: boolean[], objectData: any[]): IEdgeMap;
/**
Returns a {@link INodeMap} view of an {@link Object} array defined for nodes.
The {@link Object} value <code>data[node.index()]</code> will be accessed by the NodeMap upon the method calls <code>get(node)</code> and <code>set(node,value)</code>. <b>Warning!</b> The
indices of the accessed nodes must not change during the use of this map.
@param data array data for each node of a static graph
@returns a {@link INodeMap} view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexNodeMap(Array%3CObject%3E) Online Documentation}*/
static createIndexNodeMap(data: any[]): INodeMap;
/**
Returns a {@link INodeMap} view of a boolean array defined for nodes.
The boolean value <code>data[node.index()]</code> will be accessed by the {@link INodeMap} upon the method calls <code>getBool(node)</code>
and <code>setBool(node,value)</code>.
@param data array data for each node of a static graph
@returns a {@link INodeMap} view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexNodeMapForBoolean(Array%3Cboolean%3E) Online Documentation}*/
static createIndexNodeMapForBoolean(data: boolean[]): INodeMap;
/**
Returns a {@link INodeMap} view of an int array defined for nodes.
The int value <code>data[node.index()]</code> will be accessed by the {@link INodeMap} upon the method calls <code>getInt(node)</code>
and <code>setInt(node,value)</code>. <b>Warning!</b> The indices of the accessed nodes must not change during the use of this map.
@param data array data for each node of a static graph
@returns a {@link INodeMap} view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexNodeMapForInt(Array%3Cnumber%3E) Online Documentation}*/
static createIndexNodeMapForInt(data: number[]): INodeMap;
/**
Returns a {@link INodeMap} view of a double array defined for nodes.
The double value <code>data[node.index()]</code> will be accessed by the {@link INodeMap} upon the method calls <code>getDouble(node)</code>
and <code>setDouble(node,value)</code>.
<b>Warning!</b> The indices of the accessed nodes must not change during the use of this map.
@param data array data for each node of a static graph
@returns a {@link INodeMap} view of the given array
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexNodeMapForNumber(Array%3Cnumber%3E) Online Documentation}*/
static createIndexNodeMapForNumber(data: number[]): INodeMap;
/**
Returns a {@link INodeMap} view of a double, int, boolean and {@link Object} array defined for nodes.
The double value <code>doubleData[node.index()]</code> will be accessed by the {@link INodeMap} upon the method calls <code>getDouble(node)</code>
and <code>setDouble(node,value)</code>.
The int value <code>intData[node.index()]</code> will be accessed by the {@link INodeMap} upon the method calls <code>getInt(node)</code>
and <code>setInt(node,value)</code>.
The boolean value <code>boolData[node.index()]</code> will be accessed by the {@link INodeMap} upon the method calls <code>getBool(node)</code>
and <code>setBool(node,value)</code>.
The {@link Object} value <code>objectData[node.index()]</code> will be accessed by the {@link INodeMap} upon the method calls <code>get(node)</code>
and <code>set(node,value)</code>.
<b>Warning!</b> The indices of the accessed nodes must not change during the use of this map.
@param doubleData double data for each node of a static graph
@param intData int data for each node of a static graph
@param boolData boolean data for each node of a static graph
@param objectData {@link Object} data for each node of a static graph
@returns a {@link INodeMap} view of the given arrays
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createIndexNodeMapFromArrays(Array%3Cnumber%3E%2CArray%3Cnumber%3E%2CArray%3Cboolean%3E%2CArray%3CObject%3E) Online Documentation}*/
static createIndexNodeMapFromArrays(doubleData: number[], intData: number[], boolData: boolean[], objectData: any[]): INodeMap;
/**
Create a {@link INodeMap} view of the given map.
Accessing basic value types is solved by storing the corresponding wrapper types Double, Integer and Boolean within the
given map.
@param map
@see {@link https://docs.yworks.com/yfileshtml/#/api/Maps%23createNodeMap(IMap%3CObject%2CObject%3E) Online Documentation}*/
static createNodeMap(map: IMap<any, any>): INodeMap;
static $class: Class;
private $_Maps: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Maps;
}
export interface YList extends YObject, IList<any> {
}
/**
An implementation of a doubly linked list that provides direct access to the cells that store the elements.
The cells are represented by class {@link ListCell}.
This class supports fast access and removal operations, specifically, it is possible to remove an element in constant
time (i.e. O(1)) given a reference to its list cell.
It also supports iteration over the elements either by using the list cells directly (methods {@link YList#firstCell}/{@link YList#lastCell} together with
{@link YList#succCell}/{@link YList#predCell}, respectively) or by means of a cursor
({@link YList#cursor}).
Furthermore, this list offers its own {@link YList#sort} method. Note that this class also provides all
relevant methods to use the list like a stack data type.
This implementation permits <code>null</code> as values. The {@link YList#cursor} returned by instances of this class is
not fail fast.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList Online Documentation}*/
export class YList {
/**
Creates a list that is initialized with the elements provided by the given array of objects.
@param a
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23YList(Array%3CObject%3E) Online Documentation}*/
constructor(a: any[]);
/**
Creates a list that is initialized with those elements from the given YCursor object for which the given data provider
returns <code>true</code> upon calling its {@link IDataProvider#getBoolean getBool} method.
@param c A cursor providing objects that should be added to this list.
@param predicate A data provider that acts as a inclusion predicate for each object accessible by the given cursor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23YList(ICursor%2CIDataProvider) Online Documentation}*/
constructor(c: ICursor, predicate: IDataProvider);
/**
Creates a list that is initialized with the elements provided by the given Collection object.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23YList(IEnumerable%3CObject%3E) Online Documentation}*/
constructor(c: IEnumerable<any>);
/**
Creates a list that is initialized with the elements provided by the given YCursor object.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23YList(ICursor) Online Documentation}*/
constructor(c: ICursor);
/**
Creates an empty doubly linked list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23YList() Online Documentation}*/
constructor();
/**
Adds all elements provided by the given collection to this list at the specified position.
Elements currently at that position are shifted to the right.
@param index index at which the first element of the specified collection is placed
@param c the collection that contains the elements to be added
@returns <code>true</code> if this list was changed, <code>false</code> otherwise
@throws {Exception} <b>(IndexOutOfRangeError)</b> if the index is out of range, that is, negative or larger than the size of this list
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23addAll(number%2CICollection%3CObject%3E) Online Documentation}*/
addAll(index: number, c: ICollection<any> | ICollectionConvertible<any>): boolean;
/**
Appends all elements provided by the given cursor to this list.
The cursor will be moved from its given position to the end.
Be aware that a statement like <code>aList.append(aList.cursor())</code> results in an infinite recursion.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23addAll(ICursor) Online Documentation}*/
addAll(c: ICursor): void;
/**
Appends all elements provided by the given collection to this list.
@param collection
@returns Whether there have been elements appended.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23addAll(ICollection%3CObject%3E) Online Documentation}*/
addAll(collection: ICollection<any>): boolean;
/**
Inserts the given object at the head of this list.
@param o
@returns The newly created ListCell object that stores the given object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23addFirst(Object) Online Documentation}*/
addFirst(o: any | null): ListCell | null;
/**
Adds a formerly removed {@link ListCell} object at the head of this list.
<b>Attention:</b> If the {@link ListCell} object is still part of any list, then that list will be corrupted afterwards.
@param cell A list cell which is not part of any list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23addFirstCell(ListCell) Online Documentation}*/
addFirstCell(cell: ListCell): void;
/**
Inserts the given object at the tail of this list.
@param o
@returns The newly created ListCell object that stores the given object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23addLast(Object) Online Documentation}*/
addLast(o: any | null): ListCell | null;
/**
Adds a formerly removed {@link ListCell} object at the tail of this list.
<b>Attention:</b> If the {@link ListCell} object is still part of any list, then that list will be corrupted afterwards.
@param cell A list cell which is not part of any list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23addLastCell(ListCell) Online Documentation}*/
addLastCell(cell: ListCell): void;
/**
Removes all elements from this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23clear() Online Documentation}*/
clear(): void;
/**
Whether or not this list contains all the elements in the given collection.
Equality of elements is defined by the {@link YObject#equals} method.
@param collection
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23containsAll(ICollection%3CObject%3E) Online Documentation}*/
containsAll(collection: ICollection<any> | ICollectionConvertible<any>): boolean;
/**
Copies the elements of this collection to an array, starting at the specified array index.
@param array The one-dimensional array that is the destination of the elements copied from this collection.
@param arrayIndex The zero-based index in <code>array</code> at which copying begins.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23copyTo(Array%3CObject%3E%2Cnumber) Online Documentation}*/
copyTo(array: any[], arrayIndex: number): void;
/**
Returns a cursor for this list.
All cursor operations are supported. This cursor implementation is not fail-fast and continues to work if this list is
modified during the traversal as long as the current ListCell the cursor points at is this in this list or has been
removed from this list but has not been added to another instance since then.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23cursor() Online Documentation}*/
cursor(): ICursor;
/**
Returns the cyclic predecessor cell of the given list cell.
The last cell is returned as the cyclic predecessor of the first list cell.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23cyclicPred(ListCell) Online Documentation}*/
cyclicPred(c: ListCell): ListCell | null;
/**
Returns the cyclic successor cell of the given list cell.
The first cell is returned as the cyclic successor of the last list cell.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23cyclicSucc(ListCell) Online Documentation}*/
cyclicSucc(c: ListCell): ListCell | null;
/**
Returns the i-th element of this list.
@param i
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23elementAt(number) Online Documentation}*/
elementAt(i: number): Object;
/**
Determines whether another object is equal to this one.
@param other the other object to compare for equality
@returns <code>true</code> if <code>other</code> is considered equal to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Returns the {@link ListCell} where object <code>o</code> is stored.
This operation returns <code>null</code>, if no such cell exists. Equality of elements is defined by the {@link YObject#equals}
method. The first element in the list that matches that criteria is returned.
@param o
@returns the ListCell that contains the element or <code>null</code> if no such ListCell was found
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23findCell(Object) Online Documentation}*/
findCell(o: any | null): ListCell | null;
/**
Gets the element at the specified index.
@param index The zero-based index of the element to get or set.
@returns The element at the specified index.
@see YList#set
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23get(number) Online Documentation}*/
get(index: number): any;
/**
Gets the cell at the given index.
@param index the zero-based index of the cell in this list.
@returns The cell.
@throws {Exception} <b>(IndexOutOfRangeError)</b> if the index is negative or greater or equal than the {@link YList#size}
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23getCell(number) Online Documentation}*/
getCell(index: number): ListCell;
/**
Returns an enumerator that iterates through this collection.
@returns An {@link IEnumerator} that can be used to iterate through this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23getEnumerator() Online Documentation}*/
getEnumerator(): IEnumerator<any>;
/**
Returns the element stored in the given list cell.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23getInfo(ListCell) Online Documentation}*/
getInfo(c: ListCell): Object | null;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Whether or not this list contains the given element.
Equality of elements is defined by the {@link YObject#equals} method.
@param o
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23includes(Object) Online Documentation}*/
includes(o: any): boolean;
/**
Inserts the given <code>element</code> into this list at the given <code>index</code>.
The (newly created) list cell that stores the object is inserted right before the given index.
@param index The index that specifies the insertion position.
@param element The element to insert.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23insert(number%2CObject) Online Documentation}*/
insert(index: number, element: any): void;
/**
Inserts the given object into this list with respect to a given reference list cell.
The (newly created) list cell that stores the object is inserted right after the reference list cell <code>refCell</code>.
If <code>refCell == null</code>, the given object is inserted at the head of the list.
@param o The object to be inserted.
@param refCell The list cell used to reference the position.
@returns The newly created ListCell object that stores object <code>o</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23insertAfter(Object%2CListCell) Online Documentation}*/
insertAfter(o: any | null, refCell: ListCell): ListCell | null;
/**
Inserts the given object into this list with respect to a given reference list cell.
The (newly created) list cell that stores the object is inserted right before the reference list cell <code>refCell</code>.
If <code>refCell == null</code>, the given object is appended to the list.
@param o The object to be inserted.
@param refCell The list cell used to reference the position.
@returns The newly created ListCell object that stores object <code>o</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23insertBefore(Object%2CListCell) Online Documentation}*/
insertBefore(o: any | null, refCell: ListCell): ListCell | null;
/**
Inserts a formerly removed {@link ListCell} object into this list with respect to a given reference list
cell.
The {@link ListCell} object is inserted right after the reference list cell <code>refCell</code>.
<b>Attention:</b> If the {@link ListCell} object is still part of any list, then that list will be corrupted afterwards.
@param cellToInsert A list cell which is not part of any list.
@param refCell The list cell used to reference the position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23insertCellAfter(ListCell%2CListCell) Online Documentation}*/
insertCellAfter(cellToInsert: ListCell, refCell: ListCell): void;
/**
Inserts a formerly removed {@link ListCell} object into this list with respect to a given reference list
cell.
The {@link ListCell} object is inserted right before the reference list cell <code>refCell</code>.
<b>Attention:</b> If the {@link ListCell} object is still part of any list, then that list will be corrupted afterwards.
@param cellToInsert A list cell which is not part of any list.
@param refCell The list cell used to reference the position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23insertCellBefore(ListCell%2CListCell) Online Documentation}*/
insertCellBefore(cellToInsert: ListCell, refCell: ListCell): void;
/**
Checks whether this list contains elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23isEmpty() Online Documentation}*/
isEmpty(): boolean;
/**
Returns the zero-based index of the last occurence of the given element in this list.
@param o The element.
@returns The zero-based index of the last occurence of the given element in this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23lastIndexOf(Object) Online Documentation}*/
lastIndexOf(o: any | null): number;
/**
Equivalent to {@link YList#firstObject}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23peek() Online Documentation}*/
peek(): Object | null;
/**
Returns the predecessor cell of the given list cell.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23predCell(ListCell) Online Documentation}*/
predCell(c: ListCell): ListCell | null;
/**
Removes the given object from this list.
Only the first element for which equality to <code>o</code> holds gets removed.
@param o
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23remove(Object) Online Documentation}*/
remove(o: any): boolean;
/**
Removes the given collection of objects from this list.
@param collection
@returns Whether there have been elements removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23removeAll(ICollection%3CObject%3E) Online Documentation}*/
removeAll(collection: ICollection<any> | ICollectionConvertible<any>): boolean;
/**
Removes the element at the position specified by the given <code>index</code>.
@param index The index that specifies the element to remove.
@returns The element at the specified index.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23removeAt(number) Online Documentation}*/
removeAt(index: number): void;
/**
Removes the element pointed to by the given YCursor object.
@param c
@returns The removed element.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23removeAtCursor(ICursor) Online Documentation}*/
removeAtCursor(c: ICursor): any | null;
/**
Removes the given list cell, and hence the element stored in it, from this list.
@param c
@returns The element that is stored in the removed cell.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23removeCell(ListCell) Online Documentation}*/
removeCell(c: ListCell): Object | null;
/**
Retains only those elements in this list which are contained in the given collection.
@param collection
@returns Whether there have been elements removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23retainAll(ICollection%3CObject%3E) Online Documentation}*/
retainAll(collection: ICollection<any> | ICollectionConvertible<any>): boolean;
/**
Reverses the sequence of elements in this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23reverse() Online Documentation}*/
reverse(): void;
/**
Sets the element at the specified index.
@param index The zero-based index of the element to get or set.
@param value The element at the specified index.
@see YList#get
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23set(number%2CObject) Online Documentation}*/
set(index: number, value: any): void;
/**
Updates the element stored in the given list cell with the given object.
@param c
@param value
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23setInfo(ListCell%2CObject) Online Documentation}*/
setInfo(c: ListCell, value: any | null): void;
/**
Sorts the elements in this list according to the given comparator.
NOTE: The elements will be assigned to different list cells by this method.
@param comparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23sort(IComparer%3CObject%3E) Online Documentation}*/
sort(comparer: IComparer<any> | null): void;
/**
Sorts the elements in this list into ascending order, according to their natural ordering.
All elements must implement the {@link IComparable} interface. Furthermore, all elements in this list must be
mutually comparable (that is, <code>e1.compareTo(e2)</code> must not throw a ClassCastException for any elements <code>e1</code> and <code>e2</code> in this
list).
NOTE: The elements will be assigned to different list cells by this method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23sort() Online Documentation}*/
sort(): void;
/**
Transfers the contents of the given list to the end of this list.
The given list will be empty after this operation.
Note that this operation transfers the list cells of the given list to this list. No new list cells are created by this
operation.
@param list
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23splice(YList) Online Documentation}*/
splice(list: YList): void;
/**
Returns the successor cell of the given list cell.
@param c
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23succCell(ListCell) Online Documentation}*/
succCell(c: ListCell): ListCell | null;
/**
Returns an array representation of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23toArray() Online Documentation}*/
toArray(): any[];
/**
Returns a string representation of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23toString() Online Documentation}*/
toString(): string;
/**
Gets the number of elements in this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23size Online Documentation}*/
readonly size: number;
/**
Gets the first element of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23firstObject Online Documentation}*/
readonly firstObject: Object | null;
/**
Gets the last element of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23lastObject Online Documentation}*/
readonly lastObject: Object | null;
/**
Gets the first cell of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23firstCell Online Documentation}*/
readonly firstCell: ListCell | null;
/**
Gets the last cell of this list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YList%23lastCell Online Documentation}*/
readonly lastCell: ListCell | null;
static $class: Class;
private $_YList: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YList;
}
export interface GroupingKeys extends YObject {
}
/**
This interface defines {@link IDataProvider} keys for those layout algorithms that can handle hierarchically
grouped graphs.
There are two groups of {@link IDataProvider} keys. {@link IDataProvider}s registered with
{@link GroupingKeys#NODE_ID_DP_KEY}, {@link GroupingKeys#GROUP_DP_KEY} and {@link GroupingKeys#PARENT_NODE_ID_DP_KEY}
define the relationship of nodes within the hierarchy. A parent node is defined for each node in the graph. This parent
is the containing group node. For top-level nodes, this parent is <code>null</code>.
{@link GroupingKeys#GROUP_NODE_INSETS_DP_KEY} and {@link GroupingKeys#MINIMUM_NODE_SIZE_DP_KEY} are used for
registering
{@link IDataProvider}s that add layout information to group nodes such as the insets and the minimum size
respectively.
@see LayoutGroupingSupport
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingKeys Online Documentation}*/
export class GroupingKeys {
/**
Data provider key for assigning a unique ID for each node
The IDs of the nodes need to differ based on {@link YObject#hashCode} and {@link YObject#equals}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingKeys%23NODE_ID_DP_KEY Online Documentation}*/
static NODE_ID_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for describing the hierarchy of nodes in the graph
@see GroupingKeys#NODE_ID_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingKeys%23PARENT_NODE_ID_DP_KEY Online Documentation}*/
static PARENT_NODE_ID_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for distinguishing normal nodes from group nodes
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingKeys%23GROUP_DP_KEY Online Documentation}*/
static GROUP_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for assigning insets for group nodes
Insets reserve space around the contents of a group node. Labels and other visual decorations may be placed inside this
space.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingKeys%23GROUP_NODE_INSETS_DP_KEY Online Documentation}*/
static GROUP_NODE_INSETS_DP_KEY: NodeDpKey<YInsets> | null;
/**
Data provider key for assigning minimum sizes for group nodes
<h2>Note</h2>
Most common {@link ILayoutAlgorithm layout algorithms} ignore the minimum size for an empty group node, i.e., a node that is marked as group but
has no children assigned to it.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingKeys%23MINIMUM_NODE_SIZE_DP_KEY Online Documentation}*/
static MINIMUM_NODE_SIZE_DP_KEY: NodeDpKey<YDimension> | null;
static $class: Class;
private $_GroupingKeys: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GroupingKeys;
}
export interface LayoutGroupingSupport extends YObject {
}
/**
This class provides convenient and efficient access to the hierarchic graph structure.
The grouping information is provided through {@link IDataProvider}s registered with the graph with keys
{@link GroupingKeys#NODE_ID_DP_KEY}, {@link GroupingKeys#PARENT_NODE_ID_DP_KEY} and
{@link GroupingKeys#GROUP_DP_KEY}. It describes the hierarchy as a tree where a group node is the parent of the
nodes that it contains.
<b>Warning:</b> Once an instance of {@link LayoutGroupingSupport} is not needed anymore, it should be {@link LayoutGroupingSupport#dispose disposed of}.
<b>Warning:</b> If the state of the graph changes (e.g. new nodes are added), current {@link LayoutGroupingSupport} should be {@link LayoutGroupingSupport#dispose disposed of}
and replaced by a new one.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport Online Documentation}*/
export class LayoutGroupingSupport {
/**
Creates a new {@link LayoutGroupingSupport} instance that represents the hierarchy of the graph.
The hierarchy information is cached during the constructor call. Later modifications to the graph are not considered and
may lead to errors when using this {@link LayoutGroupingSupport} instance.
<h2>Note</h2>
Since {@link LayoutGroupingSupport} holds the current state of the graph instance, it should be {@link LayoutGroupingSupport#dispose disposed of} once it is
not needed anymore.
@param graph the graph instance from which the hierarchy information is obtained
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport Online Documentation}*/
constructor(graph: Graph | null);
/**
Creates a {@link INodeMap} to store hierarchy information for each node.
This method is called in {@link LayoutGroupingSupport#init} and may be overridden to customize how the {@link INodeMap}
is created.
@param graph the current graph
@returns the new instance of {@link INodeMap}
@see LayoutGroupingSupport#init
@see LayoutGroupingSupport#disposeInfoMap
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23createInfoMap(Graph) Online Documentation}*/
createInfoMap(graph: Graph | null): INodeMap | null;
/**
Disposes of all internal data structures held by this {@link LayoutGroupingSupport instance}.
{@link LayoutGroupingSupport} should be disposed of when it is not needed anymore, or if the state of the graph changes
(e.g. new nodes are added).
<h2>Note</h2>
After disposal this instance becomes useless. For subsequent group handling, a new {@link LayoutGroupingSupport}
instance needs to be created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23dispose() Online Documentation}*/
dispose(): void;
/**
Disposes of the {@link INodeMap} created to store hierarchy information for each node.
This method is called in {@link LayoutGroupingSupport#dispose} and may be overridden to revert changes from
{@link LayoutGroupingSupport#createInfoMap}.
@param graph the current graph
@param infoMap the {@link INodeMap} which contains the hierarchy information for each node
@see LayoutGroupingSupport#dispose
@see LayoutGroupingSupport#createInfoMap
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23disposeInfoMap(Graph%2CINodeMap) Online Documentation}*/
disposeInfoMap(graph: Graph | null, infoMap: INodeMap | null): void;
/**
Returns the direct children of the given group node.
If the {@link LayoutGroupingSupport#root virtual root of the hierarchy tree} is passed, all top-level nodes are returned.
@param parent the group node
@returns a set of child nodes of the given parent node
@see LayoutGroupingSupport#root
@see LayoutGroupingSupport#getParent
@see LayoutGroupingSupport#getDescendants
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23getChildren(YNode) Online Documentation}*/
getChildren(parent: YNode | null): YNodeList | null;
/**
Returns all descendants of the given group node.
All nodes in the subtree rooted at this group node are collected recursively.
If the {@link LayoutGroupingSupport#root virtual root of the hierarchy tree} is passed, all nodes in the graph are returned.
@param parent the group node
@returns a set of nodes that belong to a subtree of the hierarchy tree rooted at the given parent
@see LayoutGroupingSupport#root
@see LayoutGroupingSupport#getParent
@see LayoutGroupingSupport#getChildren
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23getDescendants(YNode) Online Documentation}*/
getDescendants(parent: YNode | null): YNodeList | null;
/**
Returns all edges crossing the bounds of the given group node such that their targets are inside the group node while
their sources lie outside the group node.
@param group the group node whose bounds are crossed
@returns a set of edges that cross the bounds of the group node starting outside the group node
@see LayoutGroupingSupport#getEdgesGoingOut
@see LayoutGroupingSupport#isNormalEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23getEdgesGoingIn(YNode) Online Documentation}*/
getEdgesGoingIn(group: YNode | null): EdgeList | null;
/**
Returns all edges crossing the bounds of the given group node such that their sources are inside the group node while
their targets lie outside the group node.
@param group the group node whose bounds are crossed
@returns a set of edges that cross the bounds of the group node starting inside the group node
@see LayoutGroupingSupport#getEdgesGoingIn
@see LayoutGroupingSupport#isNormalEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23getEdgesGoingOut(YNode) Online Documentation}*/
getEdgesGoingOut(group: YNode | null): EdgeList | null;
/**
Returns the nearest common ancestor of the given nodes in the hierarchy structure.
The nearest common ancestor is the local root of the smallest subtree in the hierarchy tree that contains both nodes.
This group is the first node that contains these two nodes. If no such node exists, the {@link LayoutGroupingSupport#root virtual root} is returned.
Common ancestors in special cases:
<ul>
<li><code>node1 == node2</code>: the parent of <code>node1</code>/<code>node2</code>
<li><code>node1 is parent of node2</code>: the parent of <code>node1</code>
<li><code>node2 is parent of node1</code>: the parent of <code>node2</code>
</ul>
@param node1 one of the two nodes whose common ancestor is determined
@param node2 one of the two nodes whose common ancestor is determined
@returns the nearest common ancestor node or {@link LayoutGroupingSupport#root}
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23getNearestCommonAncestor(YNode%2CYNode) Online Documentation}*/
getNearestCommonAncestor(node1: YNode | null, node2: YNode | null): YNode | null;
/**
Returns the parent for the given node.
The parent is the containing group node.
@param node the node for which the parent is determined
@returns the parent of the given node or <code>null</code> for a top-level node
@throws {Exception} <b>(ArgumentError)</b> if the given node is <code>null</code>
@see LayoutGroupingSupport#getChildren
@see LayoutGroupingSupport#getDescendants
@see LayoutGroupingSupport#getNearestCommonAncestor
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23getParent(YNode) Online Documentation}*/
getParent(node: YNode | null): YNode | null;
/**
Returns an immediate child of the group node that represents the given node.
If this node is already a child of the group node, the group node will be its representative. In case the node is
located further down in the hierarchy, the child of the group node, which is also an ancestor of the node, is used.
There is no representative if the node is not a descendant of the group.
@param node the node for which a representative is searched
@param group the group node to which the representative should belong
@returns the representative or <code>null</code> if the node is not a descendant of the group
@throws {Exception} <b>(ArgumentError)</b> if the given node is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23getRepresentative(YNode%2CYNode) Online Documentation}*/
getRepresentative(node: YNode | null, group: YNode | null): YNode | null;
/**
Determines whether or not the given node is a group node with children.
Normal nodes never have children.
@param node the potential parent
@returns <code>true</code> if the given node has children, <code>false</code> otherwise
@see LayoutGroupingSupport#getChildren
@see LayoutGroupingSupport#isGroupNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23hasChildren(YNode) Online Documentation}*/
hasChildren(node: YNode | null): boolean;
/**
Initializes internal data structures.
This method is called from {@link LayoutGroupingSupport} and retrieves the state of the hierarchic graph at the moment
of invocation. It may be overridden to append additional information used for group handling.
<h2>Note</h2>
This method creates a state that should be {@link LayoutGroupingSupport#dispose disposed of} if this instance of {@link LayoutGroupingSupport} is not needed
anymore.
@see LayoutGroupingSupport#dispose
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23init() Online Documentation}*/
init(): void;
/**
Returns whether or not the given graph is flat.
A graph is considered to be <i>flat</i> if there exist no group nodes or there exist group nodes but none of them has children.
@param graph the input graph
@returns <code>true</code> if there are no groups or only empty groups in the graph, <code>false</code> otherwise
@see LayoutGroupingSupport#isGrouped
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23isFlat(Graph) Online Documentation}*/
static isFlat(graph: Graph | null): boolean;
/**
Returns whether or not the given graph is grouped.
A graph is considered to be grouped if there are {@link IDataProvider}s registered with the graph with keys
{@link GroupingKeys#GROUP_DP_KEY}, {@link GroupingKeys#NODE_ID_DP_KEY} and
{@link GroupingKeys#PARENT_NODE_ID_DP_KEY}.
@param graph the input graph
@returns <code>true</code> if the graph contains hierarchic information, <code>false</code> otherwise
@see LayoutGroupingSupport#isFlat
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23isGrouped(Graph) Online Documentation}*/
static isGrouped(graph: Graph | null): boolean;
/**
Determines whether or not the given node is a group node.
While group nodes may have children, normal nodes never have children.
@param node the potential group node
@returns <code>true</code> if the given node is a group node, <code>false</code> otherwise
@see LayoutGroupingSupport#hasChildren
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23isGroupNode(YNode) Online Documentation}*/
isGroupNode(node: YNode | null): boolean;
/**
Determines whether or not the given edge is a normal edge as opposed to an edge that crosses any group node bounds.
Source and target of a normal edge are part of the same hierarchy level and share the same parent node.
@param edge the edge to check
@returns <code>true</code> if the given edge is normal, <code>false</code> otherwise
@see LayoutGroupingSupport#getEdgesGoingIn
@see LayoutGroupingSupport#getEdgesGoingOut
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23isNormalEdge(Edge) Online Documentation}*/
isNormalEdge(edge: Edge | null): boolean;
/**
Gets the {@link Graph} instance for which this {@link LayoutGroupingSupport} object provides hierarchy
information.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23graph Online Documentation}*/
readonly graph: Graph | null;
/**
Gets a {@link YNode} instance associated with the virtual root of the hierarchy tree.
This instance can be given to the {@link LayoutGroupingSupport#getChildren} to obtain the nodes of the top level of the
hierarchy.
<h2>Note</h2>
This {@link YNode} instance is not part of the graph.
@see LayoutGroupingSupport#getChildren
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGroupingSupport%23root Online Documentation}*/
readonly root: YNode | null;
static $class: Class;
private $_GroupingSupport: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutGroupingSupport;
}
export interface BendConverter extends YObject, ILayoutStage {
}
/**
{@link BendConverter} replaces edge bends with dummy nodes for the {@link BendConverter#coreLayout core layout algorithm}.
<b>Concept</b>
Replacing the bends works in three steps:
<ol>
<li>Replacing the bends and the edge segments between them with dummy nodes and edges
<li>Invoking the {@link BendConverter#coreLayout core layout algorithm} on the altered graph
<li>Recreating the bends and removing all dummy elements
</ol>
When the dummy elements have been inserted, edge information about {@link BendConverter#adoptPortConstraints port constraints}, {@link BendConverter#adoptEdgeGroups edge groups} and {@link BendConverter#adoptAffectedEdges selection state} can
be adopted from the original edges. That way, the {@link BendConverter#coreLayout core layout algorithm} can access the mentioned information.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter Online Documentation}*/
export class BendConverter {
/**
Creates a new {@link BendConverter} instance with the given size as the width and height for the dummy nodes.
@param size the size for the dummy nodes
@throws {Exception} <b>(ArgumentError)</b> if the given size is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23BendConverter(number) Online Documentation}*/
constructor(size: number);
/**
Creates a new instance of {@link BendConverter} with the given {@link BendConverter#coreLayout core layout algorithm}.
@param core the core layout algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23BendConverter(ILayoutAlgorithm) Online Documentation}*/
constructor(core: ILayoutAlgorithm | null);
/**
Creates a new {@link BendConverter} instance with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23BendConverter() Online Documentation}*/
constructor();
/**
Adds information to newly created dummy nodes.
This is a callback method that will be invoked after new elements for the given edge have been added to the given graph
by {@link BendConverter#prepare}. At this point, the original edge is still in the graph.
The current implementation does nothing.
@param graph the input graph
@param edge the edge whose bends were replaced by the given dummy nodes
@param dummyNodes the dummy nodes representing the bends of the edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23addedPathForEdge(LayoutGraph%2CEdge%2CYNodeList) Online Documentation}*/
addedPathForEdge(graph: LayoutGraph | null, edge: Edge | null, dummyNodes: YNodeList | null): void;
/**
Replaces all bends in the current {@link BendConverter#DEFAULT_AFFECTED_EDGES_DP_KEY scope} with dummy nodes before invoking the {@link BendConverter#coreLayout core layout algorithm}.
After the {@link BendConverter#coreLayout core layout algorithm} has finished, the edges between the dummy nodes are reconnected and the dummy nodes are
replaced by bends again.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Prepares the graph before calling the {@link BendConverter#coreLayout core layout algorithm}.
The bends of the edges belonging to the current scope are replaced by dummy nodes and the segments between those dummy
nodes are replaced by dummy edges.
Edge-specific information like {@link PortConstraint}s, edge groups or selection state can be applied to the dummy
edges to keep this information for the {@link BendConverter#coreLayout core layout algorithm}.
This method is called by {@link BendConverter#applyLayout} before invoking the
{@link BendConverter#coreLayout core layout algorithm}. It may be overridden to extend the edge-specific information that should be adopted by the dummy
edges.
<h2>Note</h2>
All changes to the graph should be reverted in {@link BendConverter#restore}.
@param graph the input graph
@see BendConverter#restore
@see BendConverter#adoptAffectedEdges
@see BendConverter#adoptPortConstraints
@see BendConverter#adoptEdgeGroups
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23prepare(LayoutGraph) Online Documentation}*/
prepare(graph: LayoutGraph | null): void;
/**
Restores the structure of the graph after the {@link BendConverter#coreLayout core layout algorithm} has finished.
This method is called by {@link BendConverter#applyLayout} after invoking the {@link BendConverter#coreLayout core layout algorithm}. It may be
overridden to clean up all additional information added in {@link BendConverter#prepare}.
<h2>Note</h2>
All changes to the graph made in {@link BendConverter#prepare} should be reverted by this method.
@param graph the graph arranged by the {@link BendConverter#coreLayout core layout algorithm}
@see BendConverter#prepare
@see BendConverter#adoptAffectedEdges
@see BendConverter#adoptPortConstraints
@see BendConverter#adoptEdgeGroups
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23restore(LayoutGraph) Online Documentation}*/
restore(graph: LayoutGraph | null): void;
/**
Data provider key for specifying the set of edges for which this stage will replace bends
Only the bends of the marked edges will be replaced with dummy nodes.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the bends of all edges are replaced.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23DEFAULT_AFFECTED_EDGES_DP_KEY Online Documentation}*/
static DEFAULT_AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets the core layout algorithm that is wrapped by this {@link ILayoutStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
/**
Gets or sets whether or not edge grouping information of the original edge should be adopted by its replacement edges.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23adoptEdgeGroups Online Documentation}*/
adoptEdgeGroups: boolean;
/**
Gets or sets whether or not the associated
{@link PortConstraint}s of the original edge should be adopted by its replacement edges.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23adoptPortConstraints Online Documentation}*/
adoptPortConstraints: boolean;
/**
Gets or sets whether or not the selection state of the original edge should be adopted by its replacement edges.
The selection state is looked up in a {@link IDataProvider} registered with the key returned by {@link BendConverter#affectedEdgesDpKey}.
@see BendConverter#affectedEdgesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23adoptAffectedEdges Online Documentation}*/
adoptAffectedEdges: boolean;
/**
Gets or sets the key to register a {@link IDataProvider} which identifies edges as <em>selected</em>.
If {@link BendConverter#adoptAffectedEdges the selection is conserved}, the selection state described by the {@link IDataProvider} registered with this
key is applied to the according dummy edges.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see BendConverter#adoptAffectedEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/BendConverter%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
static $class: Class;
private $_BendConverter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BendConverter;
}
export interface BufferedLayout extends LayoutStageBase {
}
/**
A {@link BufferedLayout} is a hull algorithm that invokes its {@link LayoutStageBase#coreLayout core layout algorithm} on a copy of the input graph.
After the layout process has finished, the calculated layout will be written back to the original graph.
This class also provides the possibility to perform arbitrary layout algorithms merely on a graph interface plus
associated graph layout. This comes in handy if an application has its own graph and only provides a graph interface
adapter to the <em>yFiles</em> graph layout machinery.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BufferedLayout Online Documentation}*/
export class BufferedLayout {
/**
Creates a new {@link BufferedLayout} instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayouter=null] the core layout routine
@see {@link https://docs.yworks.com/yfileshtml/#/api/BufferedLayout Online Documentation}*/
constructor(coreLayouter?: ILayoutAlgorithm | null);
/**
Applies the layout calculated by the {@link LayoutStageBase#coreLayout core layout algorithm} to the input graph.
The layout calculation is invoked on a copy of the given graph. Then, the calculated layout will be written back to the
given graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/BufferedLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
static $class: Class;
private $_BufferedLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BufferedLayout;
}
export interface ComponentLayout extends LayoutStageBase {
}
/**
A {@link ComponentLayout} arranges the connected components of a graph.
<b>Layout Style</b>
The components can be arranged using different {@link ComponentLayout#style styles}. All styles except {@link ComponentArrangementStyles#NONE}
place the components without overlaps.
<b>Concept</b>
{@link ComponentLayout} is a {@link ILayoutStage} that can wrap another {@link ILayoutAlgorithm layout algorithm}. That way, it
allows handling disconnected graphs for the wrapped {@link LayoutStageBase#coreLayout core layout algorithm}.
The following steps outline the concept of {@link ComponentLayout}:
<ol>
<li>Determine the connected components of the graph
<li>Hide all graph components
<li>Apply the following steps to each component separately
<ol>
<li>Unhide the component
<li>Apply the {@link LayoutStageBase#coreLayout core layout algorithm} to the component
<li>Hide the component
</ol>
<li>Unhide all graph components
<li>{@link ComponentLayout#componentArrangement Optionally arrange} the components
</ol>
<b>Features</b>
To arrange the subgraphs of the components, {@link ComponentLayout} uses the specified {@link LayoutStageBase#coreLayout core layout algorithm}. If
there is no {@link LayoutStageBase#coreLayout core layout algorithm} specified, it will keep the locations in the subgraph and arrange the components as
they are.
Hierarchically grouped graphs are handled in a special way. The contents of a group node will always belong to the same
component as the group node itself. To change that behavior {@link ComponentLayout#considerGrouping grouping} can be disabled.
By default, the components consist of the connected nodes in a graph. To choose custom subgraphs to form components,
register a {@link IDataProvider} with {@link ComponentLayout#COMPONENT_ID_DP_KEY} and assign component
IDs to the nodes.
@see MultiStageLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout Online Documentation}*/
export class ComponentLayout {
/**
Creates a new {@link ComponentLayout} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algortihm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link ComponentLayout} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algortihm.
@param [options.preferredSize] The preferred size of the layout. This option sets the {@link ComponentLayout#preferredSize} property on the created object.
@param [options.considerLabels] Whether or not to take node and edge labels into account when calculating the bounding box of the graph components. This option sets the {@link ComponentLayout#considerLabels} property on the created object.
@param [options.componentArrangement] Whether or not the separately arranged components of the input graph should be arranged relative to each other. This option sets the {@link ComponentLayout#componentArrangement} property on the created object.
@param [options.gridSpacing] The current grid spacing. This option sets the {@link ComponentLayout#gridSpacing} property on the created object.
@param [options.componentSpacing] The distance between the bounding boxes of the components. This option sets the {@link ComponentLayout#componentSpacing} property on the created object.
@param [options.considerGrouping] Whether or not grouping information bound to the graph should be considered when determining the graph components. This option sets the {@link ComponentLayout#considerGrouping} property on the created object.
@param [options.style] How the components are arranged. This option sets the {@link ComponentLayout#style} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
preferredSize?: YDimension | null;
considerLabels?: boolean;
componentArrangement?: boolean;
gridSpacing?: number;
componentSpacing?: number;
considerGrouping?: boolean;
style?: ComponentArrangementStyles | ComponentArrangementStylesConvertible;
});
/**
Delegates the layout calculation for each component separately to the {@link LayoutStageBase#coreLayout core layout algorithm} and optionally arranges the
components.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Produces a component graph layout.
This method is called by {@link ComponentLayout#applyLayout} in case {@link ComponentLayout#componentArrangement component arrangement} is enabled. It moves
the graph's components such that their bounding boxes do not overlap. Subclasses may want to override this method to
introduce custom component arrangement styles.
<h2>Note</h2>
Depending on the chosen arrangement strategy, this layout algorithm takes the ratio of the overall layout area into
account. {@link ComponentArrangementStyles#NONE}, {@link ComponentArrangementStyles#SINGLE_ROW} and {@link ComponentArrangementStyles#SINGLE_COLUMN}
are excluded.
@param graph the input graph
@param nodes the nodes of the components; the i-th list contains the nodes of the i-th component
@param edges the edges of the components; the i-th list contains the edges of the i-th component
@param bbox the bounds of the components; the i-th rectangle describes the bounding box of the i-th component
@param boxes the extended bounds of the components; the i-th rectangle describes the bounding box of the i-th component extended by
the spacing between components. The method arranges these boxes in such a way that they do not overlap. Then, the i-th
graph component must be placed inside the i-th box
@see ComponentLayout#arrangeFields
@see LayoutGraphUtilities#arrangeRectangleRows
@see LayoutGraphUtilities#arrangeRectangleMultiRows
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23arrangeComponents(LayoutGraph%2CArray%3CYNodeList%3E%2CArray%3CEdgeList%3E%2CArray%3CYRectangle%3E%2CArray%3CRectangle2D%3E) Online Documentation}*/
arrangeComponents(graph: LayoutGraph | null, nodes: YNodeList[] | null, edges: EdgeList[] | null, bbox: YRectangle[] | null, boxes: Rectangle2D[] | null): void;
/**
Arranges the bounding boxes of the components.
This method is called by {@link ComponentLayout#arrangeComponents} if the {@link ComponentLayout#style style} is set to
{@link ComponentArrangementStyles#PACKED_RECTANGLE},
{@link ComponentArrangementStyles#PACKED_COMPACT_RECTANGLE}, {@link ComponentArrangementStyles#PACKED_CIRCLE}
or
{@link ComponentArrangementStyles#PACKED_COMPACT_CIRCLE}. It may be overridden to adjust the component arrangement
strategy of the mentioned styles.
@param graph the input graph
@param nodes the nodes of the components; the i-th list contains the nodes of the i-th component
@param edges the edges of the components; the i-th list contains the edges of the i-th component
@param bbox the bounds of the components; the i-th rectangle describes the bounding box of the i-th component
@param boxes the extended bounds of the components; the i-th rectangle describes the bounding box of the i-th component
@param circular <code>true</code> if the arrangement should be circular, <code>false</code> if it should be rectangular
@param fill <code>true</code> if it is allowed to place components in empty spaces <em>inside other
components</em>, <code>false</code> otherwise
@param fromSketch <code>true</code> if the initial coordinates should be considered, <code>false</code> otherwise
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23arrangeFields(LayoutGraph%2CArray%3CYNodeList%3E%2CArray%3CEdgeList%3E%2CArray%3CYRectangle%3E%2CArray%3CRectangle2D%3E%2Cboolean%2Cboolean%2Cboolean) Online Documentation}*/
arrangeFields(graph: LayoutGraph | null, nodes: YNodeList[] | null, edges: EdgeList[] | null, bbox: YRectangle[] | null, boxes: Rectangle2D[] | null, circular: boolean, fill: boolean, fromSketch: boolean): void;
/**
Calculates the bounding box of a graph component including {@link NodeHalo}s.
This method will be invoked for each component of the graph. Depending on the state of property
{@link ComponentLayout#considerLabels}, the returned bounding box will also include node and edge labels. It may be overridden to extend the bounds to reserve
space for other elements.
@param graph the subgraph containing the nodes and edges of a component
@returns the bounding box of the component
@see ComponentLayout#considerLabels
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23calculateBounds(LayoutGraph) Online Documentation}*/
calculateBounds(graph: LayoutGraph | null): Rectangle2D | null;
/**
Determines which nodes belong to the same graph component.
This implementation uses the graph connectivity to sort the nodes into different components. Nodes that are not
connected by a path will be in separate components.
This method is called by {@link ComponentLayout#applyLayout}. It may be overridden to choose another approach to
find components that will be passed to the
{@link LayoutStageBase#coreLayout core layout algorithm}. However, most of the layout algorithms cannot handle disconnected graphs. Also, edges between
custom components will be ignored.
<h2>Note</h2>
Additionally to normal connectedness, a group node and all of its children will belong to the same component.
@param graph the input graph
@param compNumber a map that will be filled with the zero-based index of the component to which the node belongs
@returns the number of connected components of this graph
@see GraphConnectivity#connectedComponents
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23findGraphComponents(LayoutGraph%2CINodeMap) Online Documentation}*/
findGraphComponents(graph: LayoutGraph | null, compNumber: INodeMap | null): number;
/**
Moves the subgraph containing the given nodes and edges to the specified origin.
This method is called by {@link ComponentLayout#arrangeComponents} and {@link ComponentLayout#arrangeFields}
to move the components to overlap-free positions.
@param graph the input graph
@param nodes the nodes in the moving subgraph
@param edges the edges in the moving subgraph
@param origin the new origin of the graph
@param rectangle the current bounds of the subgraph
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23setOrigin(LayoutGraph%2CYNodeList%2CEdgeList%2CYPoint%2CYRectangle) Online Documentation}*/
setOrigin(graph: LayoutGraph | null, nodes: YNodeList | null, edges: EdgeList | null, origin: YPoint | null, rectangle: YRectangle | null): void;
/**
Data provider key for specifying which nodes should be arranged
If no {@link IDataProvider} is registered with this key, all components will be laid out by the core layout
algorithm.
<h2>Note</h2>
Components will only be arranged if at least one of the nodes that make up the component is marked.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23AFFECTED_COMPONENTS_DP_KEY Online Documentation}*/
static AFFECTED_COMPONENTS_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for specifying custom graph components
<h2>Note</h2>
The component IDs are sorted and the associated components are then handled in the resulting order. This way it is
possible to influence the ordering of components, if they, for example, should be placed in a column/row while keeping a
specific order.
<b>Warning:</b> Most layout algorithms cannot handle disconnected components (only the {@link HierarchicLayout}, {@link PartitionLayout}
and {@link EdgeRouter} can handle such components). Hence, you should not assign the same component ID to nodes of
different {@link GraphConnectivity#connectedComponents connected components} for algorithms that cannot handle the disconnected components.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23COMPONENT_ID_DP_KEY Online Documentation}*/
static COMPONENT_ID_DP_KEY: NodeDpKey<IComparable> | null;
/**
Gets or sets the preferred size of the layout.
The layout size also defines the desired aspect ratio (width/height).
The width and height need to be greater than zero.
<b>Warning:</b> {@link ComponentArrangementStyles#NONE}, {@link ComponentArrangementStyles#SINGLE_ROW} and {@link ComponentArrangementStyles#SINGLE_COLUMN}
do not consider the preferred layout size.
@throws {Exception} <b>(ArgumentError)</b> if the specified width or height is negative or zero.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23preferredSize Online Documentation}*/
preferredSize: YDimension | null;
/**
Gets or sets whether or not to take node and edge labels into account when calculating the bounding box of the graph
components.
@see ComponentLayout#calculateBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23considerLabels Online Documentation}*/
considerLabels: boolean;
/**
Gets or sets whether or not the separately arranged components of the input graph should be arranged relative to each
other.
If enabled, the components are arranged using a specific {@link ComponentLayout#style style} without producing overlaps between components (except for
{@link ComponentArrangementStyles#NONE}). Otherwise, the layout algorithm will keep the components at their
location. Then, the components may overlap.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23componentArrangement Online Documentation}*/
componentArrangement: boolean;
/**
Gets or sets the current grid spacing.
Components will be moved by multiples of this value, thus keeping their offset to the grid. That way, components or
parts of them that were placed on a grid before, will stay on their original grid. The grid spacing also influences the
distance between the components.
The spacing needs to be a non-negative value. If the grid spacing is set to <code>0</code>, the grid won't be considered at all.
<h2>Note</h2>
This option keeps existing grid locations intact and won't correct the location of a component with respect to the grid
specified here.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23gridSpacing Online Documentation}*/
gridSpacing: number;
/**
Gets or sets the distance between the bounding boxes of the components.
The spacing needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23componentSpacing Online Documentation}*/
componentSpacing: number;
/**
Gets or sets whether or not grouping information bound to the graph should be considered when determining the graph
components.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23considerGrouping Online Documentation}*/
considerGrouping: boolean;
/**
Gets or sets how the components are arranged.
<b>Warning:</b> The style will have no effect if {@link ComponentLayout#componentArrangement component arrangement} is disabled.
@throws {Exception} <b>(ArgumentError)</b> if the specified style is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayout%23style Online Documentation}*/
style: ComponentArrangementStyles;
static $class: Class;
private $_ComponentLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ComponentLayout;
}
export interface CompositeLayoutStage extends LayoutStageBase {
}
/**
{@link CompositeLayoutStage} allows for combining multiple {@link ILayoutStage}s.
The {@link ILayoutStage}s are organized in a pipeline where additional {@link ILayoutStage}s can be {@link CompositeLayoutStage#prependStage prepended}
or {@link CompositeLayoutStage#appendStage appended}.
Each {@link ILayoutStage} in the pipeline executes its pre-processing code before calling its successor
{@link ILayoutStage}. Then, after all subsequent
{@link ILayoutStage}s have finished, it will execute its post-processing code.
Note that the pipeline only works when all {@link ILayoutStage}s call their core layout algorithm which is the
next {@link ILayoutStage} in the pipeline.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutStage Online Documentation}*/
export class CompositeLayoutStage {
/**
Creates a new {@link CompositeLayoutStage} instance with two {@link ILayoutStage}s in the pipeline.
The inner {@link ILayoutStage} will be wrapped by the outer {@link ILayoutStage}.
@param outerStage the wrapping {@link ILayoutStage}
@param innerStage the wrapped {@link ILayoutStage}
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutStage%23CompositeLayoutStage(ILayoutStage%2CILayoutStage) Online Documentation}*/
constructor(outerStage: ILayoutStage | null, innerStage: ILayoutStage | null);
/**
Creates a new {@link CompositeLayoutStage} instance with an empty pipeline.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutStage%23CompositeLayoutStage() Online Documentation}*/
constructor();
/**
Appends a {@link ILayoutStage} to the previously added {@link ILayoutStage}s.
The added {@link ILayoutStage} will be invoked just before the {@link LayoutStageBase#coreLayout core layout algorithm} will be invoked. Its
pre-processing code will be executed after the already added {@link ILayoutStage}s' pre-processing code, while its
post-processing code will be executed before the post-processing code of all the other {@link ILayoutStage}s.
@param stage the stage to append
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutStage%23appendStage(ILayoutStage) Online Documentation}*/
appendStage(stage: ILayoutStage | null): void;
/**
Executes the pipeline of {@link ILayoutStage} and the core layout algorithm which apply the layout to the graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Prepends a {@link ILayoutStage} to the previously added {@link ILayoutStage}s.
The added {@link ILayoutStage} will be invoked before any other {@link ILayoutStage}. Its pre-processing
code will be executed before the already added {@link ILayoutStage}s' pre-processing code, while its
post-processing code will be executed after the post-processing code of all the other {@link ILayoutStage}s.
@param stage the stage to prepend
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutStage%23prependStage(ILayoutStage) Online Documentation}*/
prependStage(stage: ILayoutStage | null): void;
/**
Gets the complete pipeline of {@link ILayoutStage}s that were added to this {@link CompositeLayoutStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutStage%23layoutStages Online Documentation}*/
readonly layoutStages: IList<any> | null;
static $class: Class;
private $_CompositeLayoutStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CompositeLayoutStage;
}
export interface CopiedLayoutGraph extends LayoutGraph {
/**
Returns the layout information for all labels associated with the given edge of the copied graph.
If there is no layout information for a label, yet, a new implementation of {@link IEdgeLabelLayout} is created.
@param copiedEdge the edge
@returns the label layout information for all labels of the given edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getLabelLayout(Edge) Online Documentation}*/
getLabelLayout(copiedEdge: Edge): IEdgeLabelLayout[];
/**
Returns the layout information for all labels associated with the given node of the copied graph.
If there is no layout information for a label, yet, a new implementation of {@link INodeLabelLayout} is created.
@param copiedNode the node
@returns the label layout information for all labels of the given node
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getLabelLayout(YNode) Online Documentation}*/
getLabelLayout(copiedNode: YNode): INodeLabelLayout[];
/**
Returns the layout information for the given edge of the copied graph.
If there is no layout information for this edge, yet, a new implementation of {@link IEdgeLayout} is returned.
@param copiedEdge the edge
@returns the layout information for the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getLayout(Edge) Online Documentation}*/
getLayout(copiedEdge: Edge): IEdgeLayout;
/**
Returns the layout information for the given node of the copied graph.
If there is no layout information for this node, yet, a new implementation of {@link INodeLayout} is returned.
@param copiedNode the node
@returns the layout information for the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getLayout(YNode) Online Documentation}*/
getLayout(copiedNode: YNode): INodeLayout;
} /**
A {@link CopiedLayoutGraph} is a {@link LayoutGraph} that serves as a <b>copy of another graph</b> with layout information.
A {@link CopiedLayoutGraph} comes to use if a {@link ILayoutAlgorithm layout algorithm} should not run directly on the original graph.
Using a copy, the calculated layout will not be applied to the original graph before the layout algorithm finished
successfully. This can <em>prevent inconsistencies</em> in case something goes wrong during layout calculations.
<b>Concept</b>
{@link CopiedLayoutGraph} contains corresponding nodes and edges for each node and edge in the original graph and
copies the labels of the original nodes and edges. It also delegates all
{@link IDataProvider}s registered with the original graph.
To preserve the relations between the original elements and the copied elements in this graph, {@link CopiedLayoutGraph}
updates mappings between those elements.
If there are structural changes to the original graph, they can be transferred to this copied graph using
{@link CopiedLayoutGraph#syncStructure}. Then the copied graph will be altered, adding and removing elements,
until it corresponds to the original again.
Method {@link CopiedLayoutGraph#commitLayoutToOriginalGraph} will transfer updated layout information to the
original graph. The original graph will stay unchanged until this method is called.
@see BufferedLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph Online Documentation}*/
export class CopiedLayoutGraph {
/**
Creates a new {@link CopiedLayoutGraph} instance which is a copy of the given graph.
@param graph the original graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23CopiedLayoutGraph(LayoutGraph) Online Documentation}*/
constructor(graph: LayoutGraph);
/**
Writes the current layout information of all elements of this copied graph to the original graph.
<h2>Note</h2>
This works only if the original graph structure was not changed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23commitLayoutToOriginalGraph() Online Documentation}*/
commitLayoutToOriginalGraph(): void;
/**
Creates an edge in the copied graph that corresponds to the given original edge.
The edge will connect to the copies of the source and target of the original edge. All labels of the original edge will
be also copied and assigned to this edge.
The mappings are updated. Hence, it is possible to look up the newly created edge by using {@link CopiedLayoutGraph#getCopiedEdge}
or the original edge by using {@link CopiedLayoutGraph#getOriginalEdge}.
This method can be used for keeping the {@link CopiedLayoutGraph} in sync with the original graph.
@param originalEdge the original edge
@returns the copy of the original edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23createCopiedEdge(Object) Online Documentation}*/
createCopiedEdge(originalEdge: any): Edge;
/**
Creates a node in the copied graph that corresponds to the given original node.
All labels of the original node will be copied and assigned to this node.
The mappings are updated. Hence, it is possible to look up the newly created node by using {@link CopiedLayoutGraph#getCopiedNode}
or the original node by using {@link CopiedLayoutGraph#getOriginalNode}.
This method can be used for keeping the {@link CopiedLayoutGraph} in sync with the original graph.
@param originalNode the original node
@returns the copy of the original node
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23createCopiedNode(Object) Online Documentation}*/
createCopiedNode(originalNode: any): YNode;
/**
Creates a new {@link ILabelLayoutFactory} for this {@link CopiedLayoutGraph}.
Subclasses overriding this method must provide an instance that manages the mapping between copied and original
{@link ILabelLayout}s.
@returns the new label layout factory
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23createLabelFactory() Online Documentation}*/
createLabelFactory(): ILabelLayoutFactory;
/**
Returns the copied edge that corresponds to the given original edge.
@param originalEdge an edge of the original graph whose copy resides in this graph
@returns an edge in this graph that is the copy of the given original edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getCopiedEdge(Object) Online Documentation}*/
getCopiedEdge(originalEdge: any): Edge | null;
/**
Returns the copied node that corresponds to the given original node.
@param originalNode a node of the original graph whose copy resides in this graph
@returns a node in this graph that is the copy of the given original node
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getCopiedNode(Object) Online Documentation}*/
getCopiedNode(originalNode: any): YNode | null;
/**
Returns the original edge that corresponds to the given copied edge.
@param copiedEdge the edge in this graph that is a copy of the returned edge
@returns the edge in the original graph whose copy is the given edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getOriginalEdge(Edge) Online Documentation}*/
getOriginalEdge(copiedEdge: Edge): Object | null;
/**
Returns the original node that corresponds to the given copied node.
@param copiedNode the node in this graph that is a copy of the returned node
@returns the node in the original graph whose copy is the given node
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getOriginalNode(YNode) Online Documentation}*/
getOriginalNode(copiedNode: YNode): Object | null;
/**
Returns the edge of the copied graph to which the given {@link IEdgeLabelLayout} belongs.
@param labelLayout a label layout information of an edge in the copied graph
@returns the edge that belongs to the given label layout
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getOwnerEdge(IEdgeLabelLayout) Online Documentation}*/
getOwnerEdge(labelLayout: IEdgeLabelLayout): Edge;
/**
Returns the node of the copied graph to which the given {@link INodeLabelLayout} belongs.
@param labelLayout a label layout information of a node in the copied graph
@returns the node that belongs to the given label layout
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23getOwnerNode(INodeLabelLayout) Online Documentation}*/
getOwnerNode(labelLayout: INodeLabelLayout): YNode;
/**
Synchronizes the structure of the {@link CopiedLayoutGraph} with the actual structure of the underlying original
graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CopiedLayoutGraph%23syncStructure() Online Documentation}*/
syncStructure(): void;
static $class: Class;
private $_CopiedLayoutGraph: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CopiedLayoutGraph;
}
export interface CurveFittingLayoutStage extends LayoutStageBase {
}
/**
{@link CurveFittingLayoutStage} fits a piecewise <em>cubic bezier curve</em> to given arbitrary edge paths.
<b>Concept</b>
The {@link LayoutGraph#getPath path} of an edge, i.e., its start and end point as well as the intermediate control points, are interpreted as the
set of input points. A cubic bezier curve is then fitted to these points.
After the fitting, the points of the processed edges can be interpreted as cubic bezier control points. Each four
consecutive points form a bezier curve, where the first and last of the four points represent the start and end of the
curve and the second and third point are the control points defining how the curve looks. The second and third point do
not necessarily lie on the actual curve.
There are two exceptions to the stated rules:
<ul>
<li>If an input edge only consists of a source and a target point, it will be ignored by this stage.
<li>Self-loop edges with only one intermediate control point are also ignored. Note that self-loops with more control points
are, however, handled by this stage.
</ul>
<b>Features</b>
The {@link CurveFittingLayoutStage#maximumError maximum allowed error} for the fitting process can be selected to control how high the precision should be. A high
allowed error means that the approximation process has a lot of freedom. Curves are only loosely bound to the original
edge path. If this is not desired, e.g., because no new intersection with other graph elements should be introduced,
then a rather low error value should be specified. The lower the error value, the larger the number of bend points of an
approximated edge will be.
It is possible to define the set of edges that should be processed by this layout stage. Therefore, only paths of a
sub-set of edges can be changed. Use a {@link IDataProvider} registered with the specified {@link CurveFittingLayoutStage#affectedEdgesDpKey selection key} to
mark edges which should be part of the considered sub-set.
<h2>Note</h2>
If more than three consecutive points of an edge path are collinear, the stage always fits a bezier piece configured
such that it represents a straight line to the collinear points (without the first and last point of the collinear
part).
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveFittingLayoutStage Online Documentation}*/
export class CurveFittingLayoutStage {
/**
Creates a new {@link CurveFittingLayoutStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveFittingLayoutStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link CurveFittingLayoutStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.maximumError] The maximum allowed error for the fitting process. This option sets the {@link CurveFittingLayoutStage#maximumError} property on the created object.
@param [options.straightControlPointRatio] The ratio that defines the distance between the intermediate control points and the end points of a cubic segment which actually represents a straight line, i.e., in case the four points are collinear. This option sets the {@link CurveFittingLayoutStage#straightControlPointRatio} property on the created object.
@param [options.affectedEdgesDpKey] The {@link IDataProvider} key to look up the selection state of the edges that defines whether or not an edge's path should be approximated by a bezier curve. This option sets the {@link CurveFittingLayoutStage#affectedEdgesDpKey} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveFittingLayoutStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
maximumError?: number;
straightControlPointRatio?: number;
affectedEdgesDpKey?: any | null;
});
/**
Changes the edge paths of selected edges of the given graph such that they represent piecewise cubic bezier curves which
approximate the original points of the path.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveFittingLayoutStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for selecting edges for curve fitting
<h2>Note</h2>
If there is no {@link IDataProvider} registered with this key, a curve is fitted to all edges.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveFittingLayoutStage%23DEFAULT_AFFECTED_EDGES_DP_KEY Online Documentation}*/
static DEFAULT_AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets the maximum allowed error for the fitting process.
The error is defined as the minimum distance between an original input control point (i.e. bend) of and edge and the
fitted bezier curve.
The maximum error must be greater than or equal to zero.
<h2>Note</h2>
It is not recommended to set the maximum error to really small values, e.g., zero or one. Doing so produces a high
number of bezier pieces and, thus, a high number of control points for each edge.
@throws {Exception} <b>(ArgumentError)</b> if the specified maximum error is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveFittingLayoutStage%23maximumError Online Documentation}*/
maximumError: number;
/**
Gets or sets the ratio that defines the distance between the intermediate control points and the end points of a cubic
segment which actually represents a straight line, i.e., in case the four points are collinear.
<ul>
<li><code>0.33</code> - the second and third control point are equally distributed between te first and fourth point.
<li><code>0.0</code> - the first and second control point as well as the third and fourth point are equal.
<li><code>0.5</code> - the second and third control point are equal and exactly the mid-point between the first and fourth point.
</ul>
The control point ratio is defined to be within the interval <code>[0.0, 0.5]</code>.
@throws {Exception} <b>(ArgumentError)</b> if the specified ratio is outside the interval <code>[0.0, 0.5]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveFittingLayoutStage%23straightControlPointRatio Online Documentation}*/
straightControlPointRatio: number;
/**
Gets or sets the {@link IDataProvider} key to look up the selection state of the edges that defines whether or
not an edge's path should be approximated by a bezier curve.
If there is a {@link IDataProvider} registered with the input graph with this key, then only marked edges are
processed by this layout stage. Otherwise, all edges are processed.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IDataProvider} key is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveFittingLayoutStage%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
static $class: Class;
private $_CurveFittingLayoutStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CurveFittingLayoutStage;
}
export interface DefaultLayoutGraph extends LayoutGraph {
/**
@inheritdoc*/
createLabelFactory(): ILabelLayoutFactory;
/**
@inheritdoc*/
getLabelLayout(edge: Edge): IEdgeLabelLayout[];
/**
@inheritdoc*/
getLabelLayout(node: YNode): INodeLabelLayout[];
/**
@inheritdoc*/
getLayout(edge: Edge): IEdgeLayout;
/**
@inheritdoc*/
getLayout(node: YNode): INodeLayout;
/**
@inheritdoc*/
getOwnerEdge(labelLayout: IEdgeLabelLayout): Edge;
/**
@inheritdoc*/
getOwnerNode(labelLayout: INodeLabelLayout): YNode;
} /**
{@link DefaultLayoutGraph} is a default implementation of {@link LayoutGraph} which holds the complete
layout information about the graph and its elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph Online Documentation}*/
export class DefaultLayoutGraph {
/**
Creates a new {@link DefaultLayoutGraph} instance which is a copy of the given subgraph.
@param graph the original graph
@param nodeSubset the nodes that induce a subgraph of the original graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23DefaultLayoutGraph(LayoutGraph%2CICursor) Online Documentation}*/
constructor(graph: LayoutGraph, nodeSubset: ICursor);
/**
Creates a new {@link DefaultLayoutGraph} instance which is a copy of the given graph.
@param argGraph the original graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23DefaultLayoutGraph(LayoutGraph) Online Documentation}*/
constructor(argGraph: LayoutGraph);
/**
Creates a new {@link DefaultLayoutGraph} instance with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23DefaultLayoutGraph() Online Documentation}*/
constructor();
/**
Creates a new implementation of {@link IEdgeLayout}.
This method is called by {@link DefaultLayoutGraph#getLayout} if there is no layout information assigned to an
edge, yet. It may be overridden to create custom {@link IEdgeLayout}s.
@returns the layout of an edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23createEdgeLayout() Online Documentation}*/
createEdgeLayout(): IEdgeLayout;
/**
Creates a new implementation of {@link INodeLayout}.
This method is called by {@link DefaultLayoutGraph#getLayout} if there is no layout information assigned to a
node, yet. It may be overridden to create custom {@link INodeLayout}s.
@returns the layout of a node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23createNodeLayout() Online Documentation}*/
createNodeLayout(): INodeLayout;
/**
Specifies the layout information for all associated labels of the given edge.
The layout information consists of the location, size and orientation of the label.
@param edge the edge
@param layout the layout information for the labels of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23setLabelLayout(Edge%2CArray%3CIEdgeLabelLayout%3E) Online Documentation}*/
setLabelLayout(edge: Edge, layout: IEdgeLabelLayout[]): void;
/**
Specifies the layout information for all associated labels of the given node.
The layout information consists of the location, size and orientation of the label.
@param node the node
@param layout the layout information for the labels of the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23setLabelLayout(YNode%2CArray%3CINodeLabelLayout%3E) Online Documentation}*/
setLabelLayout(node: YNode, layout: INodeLabelLayout[]): void;
/**
Specifies the layout information for all associated labels of the given node in case there is only one label.
The layout information consists of the location, size and orientation of the label.
@param node the node
@param layout the layout information for the label of the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23setLabelLayout(YNode%2CINodeLabelLayout) Online Documentation}*/
setLabelLayout(node: YNode, layout: INodeLabelLayout): void;
/**
Specifies the layout information for the given edge.
The layout information consists of the locations of the control and end points of the edge.
@param edge the edge
@param layout the layout information for the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23setLayout(Edge%2CIEdgeLayout) Online Documentation}*/
setLayout(edge: Edge, layout: IEdgeLayout): void;
/**
Specifies the layout information for the given node.
The layout information consists of the location and size of the node.
@param node the node
@param layout the layout information for the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23setLayout(YNode%2CINodeLayout) Online Documentation}*/
setLayout(node: YNode, layout: INodeLayout): void;
/**
A {@link INodeMap} associating {@link INodeLabelLayout}s with the nodes of this graph.
@see DefaultLayoutGraph#setLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23nodeLabelMap Online Documentation}*/
nodeLabelMap: INodeMap;
/**
An {@link IEdgeMap} associating {@link IEdgeLabelLayout}s with the edges of this graph.
@see DefaultLayoutGraph#setLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23edgeLabelMap Online Documentation}*/
edgeLabelMap: IEdgeMap;
/**
A {@link IMap} associating the owner of a node label with the node label itself.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23nodeLabelFeatureMap Online Documentation}*/
nodeLabelFeatureMap: IMap<any, any>;
/**
A {@link IMap} associating the owner of an edge label with the edge label itself.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayoutGraph%23edgeLabelFeatureMap Online Documentation}*/
edgeLabelFeatureMap: IMap<any, any>;
static $class: Class;
private $_DefaultLayoutGraph: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultLayoutGraph;
}
export interface Direction extends YObject {
}
/**
{@link Direction} provides four main directions and some methods to work with them.
<h2>Note</h2>
There is exactly one immutable object for each direction.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction Online Documentation}*/
export class Direction {
/**
Returns the direction encoded as an integer value.
@returns <code>0</code> for direction {@link Direction#UP}, <code>1</code> for {@link Direction#RIGHT}, <code>2</code> for {@link Direction#DOWN}, and
<code>3</code> for {@link Direction#LEFT}
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23getDirection() Online Documentation}*/
getDirection(): number;
/**
Returns a string representation of the direction.
@returns ^ for {@link Direction#UP}, > for {@link Direction#RIGHT}, v for {@link Direction#DOWN}, and &lt; for
{@link Direction#LEFT}.
@throws {Exception} <b>(ArgumentError)</b> if the current direction value is invalid
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23toString() Online Documentation}*/
toString(): string;
/**
Decodes the up direction.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23UP Online Documentation}*/
static UP: Direction | null;
/**
Decodes the right direction.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23RIGHT Online Documentation}*/
static RIGHT: Direction | null;
/**
Decodes the down direction.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23DOWN Online Documentation}*/
static DOWN: Direction | null;
/**
Decodes the left direction.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23LEFT Online Documentation}*/
static LEFT: Direction | null;
/**
Gets the direction that follows this direction in clockwise order.
The new direction is turned <code>90</code> degrees in clockwise direction compared to this instance.
@throws {Exception} <b>(ArgumentError)</b> if the current direction value is invalid
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23turnCW Online Documentation}*/
readonly turnCW: Direction | null;
/**
Gets the direction that follows this direction in counterclockwise order.
The new direction is turned <code>90</code> degrees in counterclockwise direction compared to this instance.
@throws {Exception} <b>(ArgumentError)</b> if the current direction value is invalid
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23turnCCW Online Documentation}*/
readonly turnCCW: Direction | null;
/**
Gets the mirror direction.
The new direction is turned <code>180</code> degrees compared to this instance.
@throws {Exception} <b>(ArgumentError)</b> if the current direction value is invalid
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23mirror Online Documentation}*/
readonly mirror: Direction | null;
/**
Gets whether or not this direction is horizontal ({@link Direction#RIGHT} or {@link Direction#LEFT}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23horizontal Online Documentation}*/
readonly horizontal: boolean;
/**
Gets whether or not this direction is vertical ({@link Direction#UP} or {@link Direction#DOWN}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/Direction%23vertical Online Documentation}*/
readonly vertical: boolean;
static $class: Class;
private $_Direction: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Direction;
}
export interface DiscreteEdgeLabelLayoutModel extends YObject, IEdgeLabelLayoutModel {
/**
@inheritdoc*/
createModelParameter(labelBounds: YOrientedRectangle | null, edgeLayout: IEdgeLayout | null, sourceNode: INodeLayout | null, targetNode: INodeLayout | null): Object | null;
/**
@inheritdoc*/
getLabelCandidates(label: IEdgeLabelLayout | null, edgeLayout: IEdgeLayout | null, sourceNode: INodeLayout | null, targetNode: INodeLayout | null): YList | null;
/**
@inheritdoc*/
getLabelPlacement(labelSize: YDimension | null, edgeLayout: IEdgeLayout | null, sourceNode: INodeLayout | null, targetNode: INodeLayout | null, param: any | null): YOrientedRectangle | null;
} /**
{@link DiscreteEdgeLabelLayoutModel} describes the placement of rotated labels at some predefined positions along
the edge.
The predefined positions in this model
It's possible to specify a distance value that controls the distance between label and edge.
Furthermore, there's the possibility to mask out arbitrary edge label candidates. This can either be done by specifying
predefined candidate masks or combining several label positions with a logical <code>or</code>-operation to a user-defined mask.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel Online Documentation}*/
export class DiscreteEdgeLabelLayoutModel {
/**
Creates a new instance of {@link DiscreteEdgeLabelLayoutModel} using the given candidate mask.
The candidates mask can either describe multiple or single valid positions.
@param candidateMask the position mask that defines the allowed positions for an edge label
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23DiscreteEdgeLabelLayoutModel(DiscreteEdgeLabelPositions) Online Documentation}*/
constructor(candidateMask: DiscreteEdgeLabelPositions | DiscreteEdgeLabelPositionsConvertible);
/**
Creates a new instance of {@link DiscreteEdgeLabelLayoutModel} using the given candidate mask.
The candidates mask can either describe multiple or single valid positions.
@param options The parameters to pass.
@param options.candidateMask the position mask that defines the allowed positions for an edge label
@param [options.positionRelativeToSegment] Whether or not the label position mask should be interpreted relative to the edge segment. This option sets the {@link DiscreteEdgeLabelLayoutModel#positionRelativeToSegment} property on the created object.
@param [options.autoRotation] Whether or not edge labels are automatically rotated according to the angle of the corresponding reference edge segment. This option sets the {@link DiscreteEdgeLabelLayoutModel#autoRotation} property on the created object.
@param [options.autoFlipping] Whether or not edge labels get flipped if they would be upside down in their current position. This option sets the {@link DiscreteEdgeLabelLayoutModel#autoFlipping} property on the created object.
@param [options.angle] The angle (measured in radians) of the label model. This option sets the {@link DiscreteEdgeLabelLayoutModel#angle} property on the created object.
@param [options.distance] The distance between the label's bounding box and the edge path. This option sets the {@link DiscreteEdgeLabelLayoutModel#distance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23DiscreteEdgeLabelLayoutModel(DiscreteEdgeLabelPositions) Online Documentation}*/
constructor(options: {
candidateMask: DiscreteEdgeLabelPositions | DiscreteEdgeLabelPositionsConvertible;
positionRelativeToSegment?: boolean;
autoRotation?: boolean;
autoFlipping?: boolean;
angle?: number;
distance?: number;
});
/**
Creates a new instance of {@link DiscreteEdgeLabelLayoutModel} with default settings.
@param [options] The parameters to pass.
@param [options.positionRelativeToSegment] Whether or not the label position mask should be interpreted relative to the edge segment. This option sets the {@link DiscreteEdgeLabelLayoutModel#positionRelativeToSegment} property on the created object.
@param [options.autoRotation] Whether or not edge labels are automatically rotated according to the angle of the corresponding reference edge segment. This option sets the {@link DiscreteEdgeLabelLayoutModel#autoRotation} property on the created object.
@param [options.autoFlipping] Whether or not edge labels get flipped if they would be upside down in their current position. This option sets the {@link DiscreteEdgeLabelLayoutModel#autoFlipping} property on the created object.
@param [options.angle] The angle (measured in radians) of the label model. This option sets the {@link DiscreteEdgeLabelLayoutModel#angle} property on the created object.
@param [options.distance] The distance between the label's bounding box and the edge path. This option sets the {@link DiscreteEdgeLabelLayoutModel#distance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23DiscreteEdgeLabelLayoutModel() Online Documentation}*/
constructor(options?: {
positionRelativeToSegment?: boolean;
autoRotation?: boolean;
autoFlipping?: boolean;
angle?: number;
distance?: number;
});
/**
Returns a model parameter that encodes the specified position.
This model parameter can be passed to {@link DiscreteEdgeLabelLayoutModel#getLabelPlacement} to determine the
label's position.
@param position one of the valid positions
@returns a model parameter that encodes the specified position
@throws {Exception} <b>(ArgumentError)</b> if the specified position is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23createPositionParameter(number) Online Documentation}*/
static createPositionParameter(position: number): any | null;
/**
Returns the oriented box of the label for the given label position.
This method is called by {@link DiscreteEdgeLabelLayoutModel#getLabelPlacement} and {@link DiscreteEdgeLabelLayoutModel#getLabelCandidates}
to retrieve a valid position.
@param labelSize the size of the label that should be placed
@param edgeLayout the layout of the edge to which the label belongs
@param sourceLayout the layout of the source node of the label-owning edge
@param targetLayout the layout of the target node of the label-owning edge
@param position the label position that is valid in this model
@returns the oriented box of the label
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23getLabelPlacementForPosition(YDimension%2CIEdgeLayout%2CINodeLayout%2CINodeLayout%2CDiscreteEdgeLabelPositions) Online Documentation}*/
getLabelPlacementForPosition(labelSize: YDimension | null, edgeLayout: IEdgeLayout | null, sourceLayout: INodeLayout | null, targetLayout: INodeLayout | null, position: DiscreteEdgeLabelPositions | DiscreteEdgeLabelPositionsConvertible): YOrientedRectangle | null;
/**
Returns the position specifier that is encoded by the given model parameter.
@param parameter the model parameter
@returns the position specifier
@throws {Exception} <b>(ArgumentError)</b> if the specified model parameter is not valid for this model
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23getPosition(Object) Online Documentation}*/
static getPosition(parameter: any | null): number;
/**
Checks whether or not the given model parameter encodes a valid edge label position for this model.
If the model parameter describes a position that is accepted by the candidate mask, this is a valid parameter.
<h2>Note</h2>
In order to be valid, the parameter should be created by this label model.
@param parameter the model parameter
@returns <code>true</code> if the label position described by the given model parameter is allowed, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23isParameterValid(Object) Online Documentation}*/
isParameterValid(parameter: any | null): boolean;
/**
Gets the candidate mask which specifies the valid positions for edge labels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23candidateMask Online Documentation}*/
readonly candidateMask: DiscreteEdgeLabelPositions;
/**
Gets or sets whether or not the label position mask should be interpreted relative to the edge segment.
If this option is disabled, the position mask is interpreted in a geometric sense.
<h2>Note</h2>
This option is only relevant for non-center positions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23positionRelativeToSegment Online Documentation}*/
positionRelativeToSegment: boolean;
/**
Gets or sets whether or not edge labels are automatically rotated according to the angle of the corresponding reference
edge segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23autoRotation Online Documentation}*/
autoRotation: boolean;
/**
Gets or sets whether or not edge labels get flipped if they would be upside down in their current position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23autoFlipping Online Documentation}*/
autoFlipping: boolean;
/**
Gets or sets the angle (measured in radians) of the label model.
The angle is applied in clockwise direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23angle Online Documentation}*/
angle: number;
/**
Gets or sets the distance between the label's bounding box and the edge path.
The distance must be a non-negative value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23distance Online Documentation}*/
distance: number;
/**
Gets a model parameter that encodes the default position of this model's allowed edge label positions.
Default positions are (in descending order):
<ul>
<li>{@link DiscreteEdgeLabelPositions#CENTER}
<li>{@link DiscreteEdgeLabelPositions#SOURCE_CENTER}
<li>{@link DiscreteEdgeLabelPositions#TAIL}
<li>{@link DiscreteEdgeLabelPositions#SOURCE_TAIL}
</ul>
Descending order means that whenever two or more of the above default positions are part of the allowed positions, then
the model parameter encodes the one that is listed first.
<h2>Note</h2>
The model parameter encodes {@link DiscreteEdgeLabelPositions#CENTER} when none of the above default positions is
part of the allowed positions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelLayoutModel%23defaultParameter Online Documentation}*/
readonly defaultParameter: Object | null;
static $class: Class;
private $_DiscreteEdgeLabelLayoutModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DiscreteEdgeLabelLayoutModel;
}
export interface DiscreteNodeLabelLayoutModel extends YObject, INodeLabelLayoutModel {
/**
@inheritdoc*/
createModelParameter(labelBounds: YOrientedRectangle | null, nodeLayout: INodeLayout | null): Object | null;
/**
@inheritdoc*/
getLabelCandidates(labelLayout: INodeLabelLayout | null, nodeLayout: INodeLayout | null): YList | null;
/**
@inheritdoc*/
getLabelPlacement(labelSize: YDimension | null, nodeLayout: INodeLayout | null, parameter: any | null): YOrientedRectangle | null;
} /**
{@link DiscreteNodeLabelLayoutModel} allows placing labels at eight positions around a node and at nine positions
inside the node.
It's possible to specify an insets value that controls the distance between label and node.
Furthermore, there's the possibility to mask out arbitrary node label candidates. This can either be done by specifying
predefined candidate masks or a combination of valid positions of this model using a logical <code>or</code>-operation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelLayoutModel Online Documentation}*/
export class DiscreteNodeLabelLayoutModel {
/**
Creates a new instance of {@link DiscreteNodeLabelLayoutModel} using the given candidate mask and insets.
The specified mask can also describe a single position.
@param candidateMask the position mask that defines the allowed positions for a node label
@param [inset=4.0] the inset value that defines the internal label distance to the node's border
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelLayoutModel%23DiscreteNodeLabelLayoutModel(DiscreteNodeLabelPositions%2Cnumber) Online Documentation}*/
constructor(candidateMask: DiscreteNodeLabelPositions | DiscreteNodeLabelPositionsConvertible, inset?: number);
/**
Creates a new instance of {@link DiscreteNodeLabelLayoutModel} using the given candidate mask and insets.
The specified mask can also describe a single position.
@param options The parameters to pass.
@param options.candidateMask the position mask that defines the allowed positions for a node label
@param [options.inset=4.0] the inset value that defines the internal label distance to the node's border
@param [options.distance] The distance between the label's bounding box and the node. This option sets the {@link DiscreteNodeLabelLayoutModel#distance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelLayoutModel%23DiscreteNodeLabelLayoutModel(DiscreteNodeLabelPositions%2Cnumber) Online Documentation}*/
constructor(options: {
candidateMask: DiscreteNodeLabelPositions | DiscreteNodeLabelPositionsConvertible;
inset?: number;
distance?: number;
});
/**
Creates a new instance of {@link DiscreteNodeLabelLayoutModel} with default settings.
@param [options] The parameters to pass.
@param [options.distance] The distance between the label's bounding box and the node. This option sets the {@link DiscreteNodeLabelLayoutModel#distance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelLayoutModel%23DiscreteNodeLabelLayoutModel() Online Documentation}*/
constructor(options?: {
distance?: number;
});
/**
Returns the oriented box of the label for the given label position.
This method is called by {@link DiscreteNodeLabelLayoutModel#getLabelPlacement} and {@link DiscreteNodeLabelLayoutModel#getLabelCandidates}
to retrieve a valid position.
@param labelSize the size of the label that should be placed
@param nodeLayout the layout of the node to which the label belongs
@param position the label position that is valid in this model
@returns the oriented box of the label
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelLayoutModel%23getLabelPlacementForPosition(YDimension%2CINodeLayout%2CDiscreteNodeLabelPositions) Online Documentation}*/
getLabelPlacementForPosition(labelSize: YDimension | null, nodeLayout: INodeLayout | null, position: DiscreteNodeLabelPositions | DiscreteNodeLabelPositionsConvertible): YOrientedRectangle | null;
/**
Checks whether or not the given model parameter encodes a valid node label position for this model.
If the model parameter describes a position that is accepted by the candidate mask, this is a valid parameter.
<h2>Note</h2>
In order to be valid, the parameter should be created by this label model.
@param parameter the model parameter
@returns <code>true</code> if the label position described by the given model parameter is allowed, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelLayoutModel%23isParameterValid(Object) Online Documentation}*/
isParameterValid(parameter: any | null): boolean;
/**
Gets the candidate mask which specifies the valid positions for node labels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelLayoutModel%23candidateMask Online Documentation}*/
readonly candidateMask: DiscreteNodeLabelPositions;
/**
Gets or sets the distance between the label's bounding box and the node.
The distance must be a non-negative value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelLayoutModel%23distance Online Documentation}*/
distance: number;
/**
Gets a model parameter that encodes the default position of this model's allowed node label positions.
Default positions are (in descending order):
<ul>
<li>{@link DiscreteNodeLabelPositions#CENTER}
<li>{@link DiscreteNodeLabelPositions#NORTH_EAST}
<li>{@link DiscreteNodeLabelPositions#NORTH}
<li>{@link DiscreteNodeLabelPositions#EAST}
</ul>
Descending order means that whenever two or more of the above default positions are part of the allowed positions, then
the model parameter encodes the one that is listed first.
<h2>Note</h2>
The model parameter encodes {@link DiscreteNodeLabelPositions#CENTER} when none of the above default positions is
part of the allowed positions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelLayoutModel%23defaultParameter Online Documentation}*/
readonly defaultParameter: Object | null;
static $class: Class;
private $_DiscreteNodeLabelLayoutModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DiscreteNodeLabelLayoutModel;
}
export interface EdgeBundleDescriptor extends YObject {
}
/**
An {@link EdgeBundleDescriptor} defines the {@link EdgeBundling bundling} settings of an edge.
The defined settings will be considered by {@link ILayoutAlgorithm layout algorithms} which support edge bundling.
Each {@link EdgeBundleDescriptor} can either be specified for a single edge or shared by several edges. It is
assigned to an edge by registering a {@link IDataProvider} with key {@link EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}
to the input graph.
<h2>Note</h2>
This descriptor does not represent an ID to define different bundles; two edges associated with different descriptor
instances might still be bundled. The descriptor's purpose is to control the basic bundling property of an edge, e.g.,
whether it should be bundled at all and if the edge is directed or undirected.
@see EdgeBundling
@see EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleDescriptor Online Documentation}*/
export class EdgeBundleDescriptor {
/**
Creates a new {@link EdgeBundleDescriptor} representing a copy of the given other descriptor instance.
@param descriptor the descriptor to copy
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleDescriptor%23EdgeBundleDescriptor(EdgeBundleDescriptor) Online Documentation}*/
constructor(descriptor: EdgeBundleDescriptor | null);
/**
Creates a new {@link EdgeBundleDescriptor} with default settings.
@param [options] The parameters to pass.
@param [options.considerDirection] Whether or not the direction of the edge associated to this descriptor should be considered. This option sets the {@link EdgeBundleDescriptor#considerDirection} property on the created object.
@param [options.bundled] Whether or not an edge associated to this descriptor should be bundled. This option sets the {@link EdgeBundleDescriptor#bundled} property on the created object.
@param [options.bezierFitting] Whether or not a bezier curve should be fit to an edge associated to this descriptor. This option sets the {@link EdgeBundleDescriptor#bezierFitting} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleDescriptor%23EdgeBundleDescriptor() Online Documentation}*/
constructor(options?: {
considerDirection?: boolean;
bundled?: boolean;
bezierFitting?: boolean;
});
/**
Gets or sets whether or not the direction of the edge associated to this descriptor should be considered.
During the bundling procedure, the edges are bundled based on their direction such that the incoming edges of a node are
bundled separately from its outgoing edges. If the direction of edges adjacent to a node is not considered, then they
are not bundled separately.
If the direction of some edges is taken into consideration while the direction of some others (adjacent to the same
node) is ignored, the undirected edges are also bundled separately. This is done to distinguish them from the incoming
and outgoing edges of the particular node.
<h2>Note</h2>
For
{@link EdgeBundlingStage}, this property does not affect the result because all edges are always considered to be
undirected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleDescriptor%23considerDirection Online Documentation}*/
considerDirection: boolean;
/**
Gets or sets whether or not an edge associated to this descriptor should be bundled.
<b>Warning:</b> If an edge should not be bundled, the other settings of this descriptor will be ignored.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleDescriptor%23bundled Online Documentation}*/
bundled: boolean;
/**
Gets or sets whether or not a bezier curve should be fit to an edge associated to this descriptor.
If disabled, the edges resemble B-splines where points of the path are actually curve points. Therefore, this
representation requires a high number of bends to model curves.
If this feature is enabled, a bezier curve is fitted to the start and end point as well as the intermediate control
points of the edge. After the fitting, the points of the edge can be interpreted as cubic bezier control points. Each
four consecutive points form a bezier curve, where the first and last of the four points represent the start and end of
the curve and the second and third point are the control points defining how the curve looks. The second and third point
do not necessarily lie on the actual curve.
<h2>Note</h2>
When visualizing a fitted edge, the control points should not simply be connected by straight-line segments. This may
lead to bad results, i.e., wrong paths. The points need to be interpreted as cubic bezier control points as described
above.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleDescriptor%23bezierFitting Online Documentation}*/
bezierFitting: boolean;
static $class: Class;
private $_EdgeBundleDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeBundleDescriptor;
}
export interface EdgeBundling extends YObject {
}
/**
An {@link EdgeBundling} instance defines if and how the edges of a graph should be bundled by a
{@link ILayoutAlgorithm layout algorithm}, given that the algorithm supports edge bundling.
Bundling together multiple edges means that their common parts are to some degree merged into a bundled part. At the
source and target point, the edges are again clearly split. Edge bundling is useful to increase the readability of graph
drawings with a high number of edges that connect a comparably small number of nodes. Without bundling, such drawings
can often contain visual clutter and feature bad readability.
The bundling {@link EdgeBundling#bundlingStrength strength} and {@link EdgeBundling#bundlingQuality quality} are global settings for the bundling process. On the other hand, each edge can get
specific settings by assigning a {@link EdgeBundleDescriptor} to it. A {@link IDataProvider} can be
registered with the input graph with key {@link EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY} to assign descriptors
to edges. This allows, for example, to define which edges should actually be bundled.
The following layout algorithms/stages currently support bundling.
<ul>
<li>{@link CircularLayout}
<li>{@link RadialLayout}
<li>{@link TreeReductionStage}: only for graphs that do not contain multi-parent structures
<li>{@link EdgeBundlingStage}: a generic bundling stage, any core layout algorithm may be used
</ul>
@see EdgeBundleDescriptor
@see CircularLayout#edgeBundling
@see RadialLayout#edgeBundling
@see TreeReductionStage#edgeBundling
@see EdgeBundlingStage#edgeBundling
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundling Online Documentation}*/
export class EdgeBundling {
/**
Creates a new {@link EdgeBundling} instance with the given {@link EdgeBundling#bundlingStrength bundling strength} and {@link EdgeBundling#bundlingQuality bundling quality}.
@param [bundlingStrength=0.95] the bundling strength from the interval <code>[0,1]</code>
@param [bundlingQuality=0.5] the bundling quality from the interval <code>[0,1]</code>
@throws {Exception} <b>(ArgumentError)</b> if the given bundling strength or quality is not within <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundling Online Documentation}*/
constructor(bundlingStrength?: number, bundlingQuality?: number);
/**
Creates a new {@link EdgeBundling} instance with the given {@link EdgeBundling#bundlingStrength bundling strength} and {@link EdgeBundling#bundlingQuality bundling quality}.
@param options The parameters to pass.
@param [options.bundlingStrength=0.95] the bundling strength from the interval <code>[0,1]</code>
@param [options.bundlingQuality=0.5] the bundling quality from the interval <code>[0,1]</code>
@param [options.defaultBundleDescriptor] The default {@link EdgeBundleDescriptor} used for all edges which do not have a specific descriptor assigned via a {@link IDataProvider} registered with key {@link EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}. This option sets the {@link EdgeBundling#defaultBundleDescriptor} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the given bundling strength or quality is not within <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundling Online Documentation}*/
constructor(options: {
bundlingStrength?: number;
bundlingQuality?: number;
defaultBundleDescriptor?: EdgeBundleDescriptor | null;
});
/**
Data provider key for specifying individual bundling settings for edges
If no {@link EdgeBundleDescriptor} is mapped to an edge, the {@link EdgeBundling#defaultBundleDescriptor default descriptor} is used by the layout algorithm to
obtain the bundling setup of that edge.
@see EdgeBundling#defaultBundleDescriptor
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundling%23EDGE_BUNDLE_DESCRIPTOR_DP_KEY Online Documentation}*/
static EDGE_BUNDLE_DESCRIPTOR_DP_KEY: EdgeDpKey<EdgeBundleDescriptor> | null;
/**
Gets or sets the desired quality of the calculated edge bundling.
Higher quality indicates that the bundling procedure uses more sophisticated methods to compute the actual bundles and
the routing of the edges. If the applied layout algorithm is the {@link CircularLayout}, this may lead to a
significantly higher running time, especially for large graphs. On the other hand, a low quality indicates that run-time
is more important than highly optimized bundling results, leading to, for example, more crossings between different
bundles (for {@link CircularLayout}) or completely straight-line un-bundled edges for the
{@link EdgeBundlingStage}.
The quality is defined to lie within <code>[0,1]</code>, where higher values stand for higher quality.
<b>Warning:</b> This feature has an effect if the applied layout algorithm is the {@link CircularLayout} or the
{@link EdgeBundlingStage}.
@throws {Exception} <b>(ArgumentError)</b> if the given quality value is not within <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundling%23bundlingQuality Online Documentation}*/
bundlingQuality: number;
/**
Gets or sets the strength of the edge bundling.
The strength controls how tightly the edges are bundled and influences the shape of the curves of bundled edges. Low
values lead to only slightly bundled edges; results will mostly show individual node-to-node connectivity information.
High values show the connectivity on a higher level, strongly bundling edges together and generating highly curved edge
paths.
The bundling strength is defined as a value from the interval <code>[0,1]</code>.
<h2>Note</h2>
A bundling strength of <code>0</code> leads to straight-line edges where edges are basically not bundled.
<b>Warning:</b> For the
{@link EdgeBundlingStage}, high values can significantly affect the performance of the algorithm, especially for
large graphs.
@throws {Exception} <b>(ArgumentError)</b> if the given strength is less than <code>0</code> or greater than <code>1</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundling%23bundlingStrength Online Documentation}*/
bundlingStrength: number;
/**
Gets or sets the default {@link EdgeBundleDescriptor} used for all edges which do not have a specific descriptor
assigned via a {@link IDataProvider} registered with key
{@link EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}.
@throws {Exception} <b>(ArgumentError)</b> if the given descriptor is <code>null</code>
@see EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundling%23defaultBundleDescriptor Online Documentation}*/
defaultBundleDescriptor: EdgeBundleDescriptor | null;
static $class: Class;
private $_EdgeBundling: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeBundling;
}
export interface EdgeLabelCandidate extends LabelCandidate {
}
/**
An {@link EdgeLabelCandidate} describes one valid placement for an edge label considering the label model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelCandidate Online Documentation}*/
export class EdgeLabelCandidate {
/**
Creates a new instance of {@link EdgeLabelCandidate} described by location, size, model parameter and internal
flag.
@param pos the location of the upper left corner of the candidate
@param size the size of the candidate
@param param the parameters of the label model associated with this candidate
@param owner the label associated with the candidate
@param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelCandidate%23EdgeLabelCandidate(YPoint%2CYDimension%2CObject%2CIEdgeLabelLayout%2Cboolean) Online Documentation}*/
constructor(pos: YPoint | null, size: YDimension | null, param: any | null, owner: IEdgeLabelLayout | null, internal?: boolean);
/**
Creates a new instance of {@link EdgeLabelCandidate} described by its box, model parameter and internal flag.
@param labelBox the box that specifies the candidate's size and position
@param param the parameters of the label model associated with this candidate
@param owner the label associated with the candidate
@param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelCandidate%23EdgeLabelCandidate(YOrientedRectangle%2CObject%2CIEdgeLabelLayout%2Cboolean) Online Documentation}*/
constructor(labelBox: YOrientedRectangle | null, param: any | null, owner: IEdgeLabelLayout | null, internal?: boolean);
static $class: Class;
private $_EdgeLabelCandidate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeLabelCandidate;
}
export interface EdgeLabelOrientationSupport extends YObject {
}
/**
{@link EdgeLabelOrientationSupport} provides orientation and mirroring support for {@link ILayoutAlgorithm layout algorithms} with
integrated edge labeling.
To calculate the position and orientation of edge labels whose placements are defined with the
{@link PreferredPlacementDescriptor}, the {@link OrientationLayout} would need the direction of the
associated edge. Since this direction is not available to the {@link OrientationLayout}, because it is a wrapper,
the calculation must be done in the core layout algorithm itself with the help of the
{@link EdgeLabelOrientationSupport}.
Important: Since layout algorithms with integrated edge labeling cannot deal with ambiguous preferred placement
descriptors (combinations of the same setting, e.g., source or target for the {@link PreferredPlacementDescriptor#placeAlongEdge}), {@link EdgeLabelOrientationSupport#preProcessLabel}
removes combinations and replaces the original descriptor with a <em>non-ambiguous</em> one that is used during the layout calculation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport Online Documentation}*/
export class EdgeLabelOrientationSupport {
/**
Creates a new {@link EdgeLabelOrientationSupport} instance.
<h2>Note</h2>
Since this class handles the orientation of the edge labels, the core layout algorithm should disable the edge label
calculation in its {@link OrientationLayout}.
@see OrientationLayout#considerEdgeLabels
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport Online Documentation}*/
constructor();
/**
Creates and returns a non-ambiguous {@link PreferredPlacementDescriptor} of the given potentially ambiguous one.
The created descriptor will be one without any combinations of settings, e.g., source or target for the {@link PreferredPlacementDescriptor#placeAlongEdge}.
@param ambiguous an ambiguous {@link PreferredPlacementDescriptor}
@returns a non-ambiguous version of the given ambiguous {@link PreferredPlacementDescriptor}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23createNonAmbiguous(PreferredPlacementDescriptor) Online Documentation}*/
createNonAmbiguous(ambiguous: PreferredPlacementDescriptor | null): PreferredPlacementDescriptor | null;
/**
Calculates the up vector of an edge label considering the preferred placement of the label as well as the direction of
the edge segment the label is attached to.
@param descriptor the data describing the preferred label orientation
@param segmentDirection the direction the edge segment is pointing to
@returns the up vector for the edge label that considers the preferred placement of the label as well as the direction of the
edge segment the label is attached to
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23getEdgeLabelUpVector(PreferredPlacementDescriptor%2CDirection) Online Documentation}*/
static getEdgeLabelUpVector(descriptor: PreferredPlacementDescriptor | null, segmentDirection: Direction | null): YVector | null;
/**
Calculates the up vector for a label whose right vector is rotated clockwise by the given angle.
The up vector starts in the lower-left corner of the label and points up. If the label is paraxial it is <code>(0, -1)</code>. The
right vector starts at the same point and points right. In a paraxial label it is <code>(1, 0)</code>.
@param rightVectorAngle the angle in radians the right vector is rotated by
@returns the up vector for a label whose right vector is rotated clockwise by the given angle
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23getLabelUpVector(number) Online Documentation}*/
static getLabelUpVector(rightVectorAngle: number): YVector | null;
/**
Transforms a direction of the layout to the oriented layout direction.
@param direction direction in the layout
@param orientation the orientation of the layout
@param mirrorMask the mask to determine whether or not the orientation is mirrored
@returns the direction in the oriented layout
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23getOrientedDirection(Direction%2CLayoutOrientation%2CMirrorModes) Online Documentation}*/
static getOrientedDirection(direction: Direction | null, orientation: LayoutOrientation | LayoutOrientationStringValues, mirrorMask: MirrorModes | MirrorModesConvertible): Direction | null;
/**
Returns whether or not the orientation is mirrored.
@param orientation the orientation of the layout
@param mirrorMask the mask to determine whether or not the orientation is mirrored
@returns <code>true</code> if the orientation is mirrored, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23isOrientationMirrored(LayoutOrientation%2CMirrorModes) Online Documentation}*/
static isOrientationMirrored(orientation: LayoutOrientation | LayoutOrientationStringValues, mirrorMask: MirrorModes | MirrorModesConvertible): boolean;
/**
Marks the given port label for a non-port placement (fallback).
The given edge label's {@link PreferredPlacementDescriptor} will internally be changed such that its {@link PreferredPlacementDescriptor#placeAlongEdge placement along edge}
is either {@link LabelPlacements#AT_SOURCE} or {@link LabelPlacements#AT_TARGET}.
Algorithms that are executed after this call and before {@link EdgeLabelOrientationSupport#resetAmbiguousLabelDescriptors}
will handle port labels like normal source or target labels.
<b>Warning:</b> This method may only be called for labels that were also processed by an earlier call to
{@link EdgeLabelOrientationSupport#replaceAmbiguousLabelDescriptors}.
@param portLabel an edge label that had a {@link LabelPlacements#AT_SOURCE_PORT} or {@link LabelPlacements#AT_TARGET_PORT}
placement preference
@throws {Exception} <b>(InvalidOperationError)</b> if {@link EdgeLabelOrientationSupport#replaceAmbiguousLabelDescriptors} was never called for the graph containing
the given label.
@see LabelPlacements#AT_SOURCE_PORT
@see LabelPlacements#AT_TARGET_PORT
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23markPortLabelForFallbackPlacement(LabelLayoutData) Online Documentation}*/
markPortLabelForFallbackPlacement(portLabel: LabelLayoutData | null): void;
/**
Restores the original preferred placement and updates the label rotation according to the layout orientation.
The original {@link LabelLayoutData#preferredPlacementDescriptor} that has been replaced in {@link EdgeLabelOrientationSupport#preProcessLabel} gets restored. Then it
moves and rotates the {@link LabelLayoutData#bounds label box} in respect of the orientation.
This method is called by the core layout algorithm with the integrated edge labeling for each edge label after the
location of the label is determined.
@param graph the graph to get the orientation information from
@param label the information about the edge label whose location is determined
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23postProcessLabel(Graph%2CLabelLayoutData) Online Documentation}*/
postProcessLabel(graph: Graph | null, label: LabelLayoutData | null): void;
/**
Prepares the label for the core layout algorithm.
First, the {@link LabelLayoutData#preferredPlacementDescriptor preferred placement} of the edge label is replaced by a non-ambiguous {@link PreferredPlacementDescriptor}
that has only relative references. This descriptor is orientation independent and can be used in the core layout
algorithm. Second, the method rotates the {@link LabelLayoutData#bounds label box} in respect of its {@link PreferredPlacementDescriptor} and the
direction of the segment to which it belongs. After that call, the core layout algorithm can use the geometry ({@link LabelLayoutData#width} and
{@link LabelLayoutData#height}) of the label to calculate its location.
This method is called by the core layout algorithm with the integrated edge labeling for each edge label before the
layout algorithm uses the geometry of labels. It should be called as soon as the direction of the segment the label
belongs to is determined.
@param graph the graph to get the orientation information from
@param label the information about the edge label whose segment direction is determined
@param segmentDirection the direction of the segment the edge label belongs to
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23preProcessLabel(Graph%2CLabelLayoutData%2CDirection) Online Documentation}*/
preProcessLabel(graph: Graph | null, label: LabelLayoutData | null, segmentDirection: Direction | null): void;
/**
Replaces the
{@link PreferredPlacementDescriptor}s of all edge labels in the given graph with non-ambiguous descriptors.
The non-ambiguous descriptors are retrieved from the original descriptors. They describe only one of the possible
placements.
This method should be called by a layout algorithm at the beginning of the layout calculation to assure that the label's
placement is handled consistently.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23replaceAmbiguousLabelDescriptors(Graph) Online Documentation}*/
replaceAmbiguousLabelDescriptors(graph: Graph | null): void;
/**
Restores the
{@link PreferredPlacementDescriptor}s of all edge labels in the given graph with their original descriptors.
This method should be called by a layout algorithm at the end of the layout calculation to assure that the original
state of the graph is not corrupted.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23resetAmbiguousLabelDescriptors(Graph) Online Documentation}*/
resetAmbiguousLabelDescriptors(graph: Graph | null): void;
/**
Calculates and sets the up vector of an edge label considering the preferred placement of the label as well as the
direction of the edge segment the label is attached to.
@param label the data that describes the label orientation and is used for setting the new up vector
@param segmentDirection the direction the edge segment is pointing to
@see EdgeLabelOrientationSupport#getEdgeLabelUpVector
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeLabelOrientationSupport%23updateLabelOrientation(LabelLayoutData%2CDirection) Online Documentation}*/
static updateLabelOrientation(label: LabelLayoutData | null, segmentDirection: Direction | null): void;
static $class: Class;
private $_EdgeLabelOrientationSupport: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeLabelOrientationSupport;
}
/**
Routing styles for edges routed by the {@link EdgeRouter}.
@see EdgeRouterEdgeLayoutDescriptor#routingStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeRoutingStyle Online Documentation}*/
export enum EdgeRouterEdgeRoutingStyle {
/**
A routing style constant specifying that the edge should be orthogonal.
Orthogonal edges only consist of vertical and horizontal segments.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeRoutingStyle%23ORTHOGONAL Online Documentation}*/
ORTHOGONAL,
/**
A routing style constant specifying that the edge should be octilinear.
Octilinear edges consist of vertical and horizontal segments as well as segments with slope <code>-1</code> and <code>1</code>.
<b>Warning:</b> Backbone segments of a {@link EdgeRouterBusDescriptor bus} are always routed orthogonally. Bus connections that lead from the backbone to the actual
end nodes can be octilinear.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeRoutingStyle%23OCTILINEAR Online Documentation}*/
OCTILINEAR,
/**
A routing style constant specifying that the edge should be curved.
Curved edges are constructed using cubic bezier splines. As for all routing styles, overlaps with other elements are
avoided. If there is very little space for a smooth curve, it can happen that the resulting path remains orthogonal.
Therefore, the curved routing style does not produce optimal results for input graphs that offer little space for the
edge routes.
When {@link EdgeRouter#integratedEdgeLabeling integrated edge labeling} is enabled, for edge labels a straight, non-curved segment where the label is placed will
be inserted. All the settings of the {@link PreferredPlacementDescriptor} are supported. Similarly, for the {@link EdgeRouterEdgeLayoutDescriptor#minimumFirstSegmentLength minimum first segment length}
and the {@link EdgeRouterEdgeLayoutDescriptor#minimumLastSegmentLength minimum last segment length}, a straight edge segment is created to fulfill the constraints.
<h2>Note</h2>
If the bends should actually be interpreted as bezier points, i.e. to let the visualization create the spline curves,
enable {@link EdgeRouterEdgeLayoutDescriptor#createControlPoints}
<h2>Note</h2>
If the curved route should be allowed to start/end with a non-orthogonal segment and at any side of the adjacent node,
the connection style {@link CurveConnectionStyle#ORGANIC} can be specified as {@link EdgeRouterEdgeLayoutDescriptor#sourceCurveConnectionStyle} and/or {@link EdgeRouterEdgeLayoutDescriptor#targetCurveConnectionStyle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeRoutingStyle%23CURVED Online Documentation}*/
CURVED
}
/** Automatic conversion to {@link EdgeRouterEdgeRoutingStyle}. */
type EdgeRouterEdgeRoutingStyleStringValues = "orthogonal" | "octilinear" | "curved";
export interface FixNodeLayoutStage extends LayoutStageBase {
}
/**
{@link FixNodeLayoutStage} fixes the initial position of either a <em>single</em> node or the alignment of the bounding box of
several nodes (but not the positions of the individual nodes).
The absolute coordinates of the {@link FixNodeLayoutStage#calculateFixPoint fix point} are the same before and after invoking the {@link LayoutStageBase#coreLayout core layout algorithm}. However, the
relative coordinates of the nodes are altered by the {@link LayoutStageBase#coreLayout core layout algorithm}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage Online Documentation}*/
export class FixNodeLayoutStage {
/**
Creates a new {@link FixNodeLayoutStage} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link FixNodeLayoutStage} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.fixPointPolicy] The fix point calculation policy used in {@link FixNodeLayoutStage#calculateFixPoint} to determine which point in the corresponding rectangle should be considered fixed. This option sets the {@link FixNodeLayoutStage#fixPointPolicy} property on the created object.
@param [options.includingEdges] Whether or not subgraph edges are taken into account when calculating the bounding box of the fixed nodes. This option sets the {@link FixNodeLayoutStage#includingEdges} property on the created object.
@param [options.includingLabels] Whether or not label geometries are taken into account when calculating the bounding box of the fixed nodes. This option sets the {@link FixNodeLayoutStage#includingLabels} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
fixPointPolicy?: FixPointPolicy | FixPointPolicyStringValues;
includingEdges?: boolean;
includingLabels?: boolean;
});
/**
Delegates to the {@link LayoutStageBase#coreLayout core layout algorithm} and moves the fixed point back to its initial location.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Calculates the bounding box of the specified nodes.
The calculated bounds may contain {@link FixNodeLayoutStage#includingEdges edges} and/or {@link FixNodeLayoutStage#includingLabels labels}.
This method is called by {@link FixNodeLayoutStage#calculateFixPoint} and may be overridden to customize the size
of the bounds.
@param graph the graph associated with the specified nodes
@param fixed the list of nodes for which to determine the bounding box
@returns the bounding box of the specified nodes
@see FixNodeLayoutStage#includingEdges
@see FixNodeLayoutStage#includingLabels
@see FixNodeLayoutStage#calculateFixPoint
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage%23calculateBounds(LayoutGraph%2CYNodeList) Online Documentation}*/
calculateBounds(graph: LayoutGraph | null, fixed: YNodeList | null): Rectangle2D | null;
/**
Calculates the fix point for the specified nodes in the specified graph.
The default implementation calls {@link FixNodeLayoutStage#calculateBounds} and
{@link FixNodeLayoutStage#calculateFixPoint}.
This method is called by {@link FixNodeLayoutStage#applyLayout} to determine the point that will have the same
coordinates before and after invoking the
{@link LayoutStageBase#coreLayout core layout algorithm}. It may be overridden to introduce a custom policy for retrieving the fix point.
@param graph the graph associated with the specified nodes
@param fixed the list of nodes for which a fix point has to be determined
@returns the fix point for the specified nodes in the specified graph
@see FixNodeLayoutStage#fixPointPolicy
@see FixNodeLayoutStage#calculateBounds
@see FixNodeLayoutStage#calculateFixPoint
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage%23calculateFixPoint(LayoutGraph%2CYNodeList) Online Documentation}*/
calculateFixPoint(graph: LayoutGraph | null, fixed: YNodeList | null): YPoint | null;
/**
Calculates the fix point of the specified rectangle according to the policy returned by {@link FixNodeLayoutStage#fixPointPolicy}.
If the specified rectangle is <code>null</code> or the rectangle has negative <code>width</code> or negative <code>height</code> then <code>null</code> will be returned.
This method is called by
{@link FixNodeLayoutStage#calculateFixPoint}. It may be overridden to introduce a custom policy for retrieving the
fix point.
@param bounds the rectangle for which to determine the fix point
@returns the fix point of the specified rectangle or <code>null</code> if nothing should be fixed
@throws {Exception} <b>(ArgumentError)</b> if the current fix point policy is invalid
@see FixNodeLayoutStage#fixPointPolicy
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage%23calculateFixPoint(Rectangle2D) Online Documentation}*/
calculateFixPoint(bounds: Rectangle2D | null): YPoint | null;
/**
Data provider key for determining the nodes that should be considered fixed
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage%23FIXED_NODE_DP_KEY Online Documentation}*/
static FIXED_NODE_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the fix point calculation policy used in {@link FixNodeLayoutStage#calculateFixPoint} to determine
which point in the corresponding rectangle should be considered fixed.
@throws {Exception} <b>(ArgumentError)</b> if the specified policy is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage%23fixPointPolicy Online Documentation}*/
fixPointPolicy: FixPointPolicy;
/**
Gets or sets whether or not subgraph edges are taken into account when calculating the bounding box of the fixed nodes.
@see FixNodeLayoutStage#calculateBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage%23includingEdges Online Documentation}*/
includingEdges: boolean;
/**
Gets or sets whether or not label geometries are taken into account when calculating the bounding box of the fixed
nodes.
@see FixNodeLayoutStage#calculateBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutStage%23includingLabels Online Documentation}*/
includingLabels: boolean;
static $class: Class;
private $_FixNodeLayoutStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FixNodeLayoutStage;
}
export interface FixPortLocationStage extends LayoutStageBase {
}
/**
{@link FixPortLocationStage} ensures that layout algorithms that cannot handle port constraints keep the ports of
edges with strong {@link PortConstraint}s.
Without this stage, the port locations get lost.
For each edge with strong {@link PortConstraint}s, this stage simply stores the original port and restores it
after applying the {@link LayoutStageBase#coreLayout core layout algorithm}. Therefore, it either replaces the source/target port by the original port or
adds the original port to the edge path calculated by the {@link LayoutStageBase#coreLayout core layout algorithm} (see method {@link FixPortLocationStage#keepCalculatedPorts}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPortLocationStage Online Documentation}*/
export class FixPortLocationStage {
/**
Creates a new {@link FixPortLocationStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPortLocationStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link FixPortLocationStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.keepCalculatedPorts] Whether or not the final edge routes should contain the ports calculated by the {@link LayoutStageBase#coreLayout core layout algorithm}. This option sets the {@link FixPortLocationStage#keepCalculatedPorts} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPortLocationStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
keepCalculatedPorts?: boolean;
});
/**
Restores the locations of strong {@link PortConstraint}s after invoking the {@link LayoutStageBase#coreLayout core layout algorithm}.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPortLocationStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets whether or not the final edge routes should contain the ports calculated by the {@link LayoutStageBase#coreLayout core layout algorithm}.
If this option is disabled, this stage simply replaces the current source/target ports (calculated by the core layout
algorithm) by the original ports (for strong ports only). Otherwise, the calculated ports are still contained in the
edge paths.
<h2>Note</h2>
For layout algorithms that produce non straight-line edge routes this option should always be enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPortLocationStage%23keepCalculatedPorts Online Documentation}*/
keepCalculatedPorts: boolean;
static $class: Class;
private $_FixPortLocationStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FixPortLocationStage;
}
export interface FreeEdgeLabelLayoutModelParameter extends YObject {
}
/**
This model parameter encodes a label position within {@link FreeEdgeLabelLayoutModel}.
It represents every possible location in relation to the source point of the edge. The radius describes the distance of
the label center from the source point. There is an angle <code>theta</code> which defines the location on the circle with the given
radius. It is the angle between the first segment of the underlying edge and the imaginary line to the center of the
label. Finally, there is a second angle that describes the rotation of the label box.
@see FreeEdgeLabelLayoutModelParameter#setPoint
@see FreeEdgeLabelLayoutModelParameter#angle
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModelParameter Online Documentation}*/
export class FreeEdgeLabelLayoutModelParameter {
/**
Creates a new instance of {@link FreeEdgeLabelLayoutModelParameter} with the given radius, angle and rotation
angle.
@param radius the distance of the label from the source node of the underlying edge
@param theta the angle in radians between the first segment of the underlying edge and the imaginary line to the center of the label
@param [angle=0] the rotation angle of the label
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModelParameter%23FreeEdgeLabelLayoutModelParameter(number%2Cnumber%2Cnumber) Online Documentation}*/
constructor(radius: number, theta: number, angle?: number);
/**
Creates a new instance of {@link FreeEdgeLabelLayoutModelParameter} with radius <code>50</code> and angle <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModelParameter%23FreeEdgeLabelLayoutModelParameter() Online Documentation}*/
constructor();
/**
Specifies a point that represents the location of the label's center in relation to the source point of the edge.
The x-coordinate corresponds to the radius which describes the distance of the label to the source point. The
y-coordinate corresponds to the angle between the first segment of the underlying edge and the imaginary line to the
center of the label.
@param radius the distance to the source point
@param theta the location on the radius
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModelParameter%23setPoint(number%2Cnumber) Online Documentation}*/
setPoint(radius: number, theta: number): void;
/**
Returns a {@link string} representation of this model parameter.
The {@link string} contains the radius around the source node on which the label is placed and the angle that describes the location
on said radius.
@returns the {@link string} representation
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModelParameter%23toString() Online Documentation}*/
toString(): string;
/**
Gets or sets the rotation angle of the label that is placed using this model parameter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModelParameter%23angle Online Documentation}*/
angle: number;
/**
Gets a point that represents the location of the label's center in relation to the source point of the edge.
The x-coordinate corresponds to the radius which describes the distance of the label to the source point. The
y-coordinate corresponds to the angle between the first segment of the underlying edge and the imaginary line to the
center of the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModelParameter%23point Online Documentation}*/
readonly point: YPoint | null;
static $class: Class;
private $_FreeEdgeLabelLayoutModelParameter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreeEdgeLabelLayoutModelParameter;
}
export interface FreeEdgeLabelLayoutModel extends YObject, IEdgeLabelLayoutModel {
/**
@inheritdoc*/
getLabelPlacement(labelSize: YDimension | null, edgeLayout: IEdgeLayout | null, sourceNode: INodeLayout | null, targetNode: INodeLayout | null, param: any | null): YOrientedRectangle | null;
} /**
{@link FreeEdgeLabelLayoutModel} allows placing edge labels at any location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModel Online Documentation}*/
export class FreeEdgeLabelLayoutModel {
/**
Creates a new instance of {@link FreeEdgeLabelLayoutModel}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModel Online Documentation}*/
constructor();
/**
Creates a model parameter that represents the given edge label position within this model.
The created model parameter represents the current location of the given label.
This parameter can be passed to {@link FreeEdgeLabelLayoutModel#getLabelPlacement} to retrieve the current label
box.
@param labelBounds the oriented box of the label the parameter represents
@param edgeLayout the layout of the edge to which the label belongs
@param sourceLayout the layout of the source node of the label-owning edge
@param targetLayout the layout of the target node of the label-owning edge
@returns the model parameter representing the given label location
@throws {Exception} <b>(ArgumentError)</b> if one or more of the given bounds' values are invalid, i.e., <code>NaN</code>
@see FreeEdgeLabelLayoutModel#getLabelPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModel%23createModelParameter(YOrientedRectangle%2CIEdgeLayout%2CINodeLayout%2CINodeLayout) Online Documentation}*/
createModelParameter(labelBounds: YOrientedRectangle | null, edgeLayout: IEdgeLayout | null, sourceLayout: INodeLayout | null, targetLayout: INodeLayout | null): Object | null;
/**
Returns the {@link EdgeLabelCandidate} that describes the current label position.
<h2>Note</h2>
The returned list contains only one candidate which describes the current location.
@param labelLayout the label for which candidates should be generated
@param edgeLayout the layout of the edge to which the label belongs
@param sourceLayout the layout of the source node of the label-owning edge
@param targetLayout the layout of the target node of the label-owning edge
@returns a list of {@link EdgeLabelCandidate} instances
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModel%23getLabelCandidates(IEdgeLabelLayout%2CIEdgeLayout%2CINodeLayout%2CINodeLayout) Online Documentation}*/
getLabelCandidates(labelLayout: IEdgeLabelLayout | null, edgeLayout: IEdgeLayout | null, sourceLayout: INodeLayout | null, targetLayout: INodeLayout | null): YList | null;
/**
Gets the model parameter that encodes the default position of {@link FreeEdgeLabelLayoutModel}.
The distance of the label to the edge's source point will be <code>20</code>. The angle between the line to the center of the label
and the first segment is <code>20</code> radians. The label is not rotated.
This parameter can be passed to {@link FreeEdgeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
label box.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeEdgeLabelLayoutModel%23defaultParameter Online Documentation}*/
readonly defaultParameter: Object | null;
static $class: Class;
private $_FreeEdgeLabelLayoutModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreeEdgeLabelLayoutModel;
}
export interface FreeNodeLabelLayoutModelParameter extends YObject {
}
/**
The model parameter that encodes a label position within {@link FreeNodeLabelLayoutModel}.
It represents every possible location in relation to the node. The point describes the offset of the label's center to
the upper-left corner of the node.
@see FreeNodeLabelLayoutModelParameter#setPoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModelParameter Online Documentation}*/
export class FreeNodeLabelLayoutModelParameter {
/**
Creates a new instance of {@link FreeNodeLabelLayoutModelParameter} with the given offset.
@param dx the horizontal offset
@param dy the vertical offset
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModelParameter%23FreeNodeLabelLayoutModelParameter(number%2Cnumber) Online Documentation}*/
constructor(dx: number, dy: number);
/**
Creates a new instance of {@link FreeNodeLabelLayoutModelParameter}.
It has no offset, i.e., the upper-left corners of both node label and node are at the same location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModelParameter%23FreeNodeLabelLayoutModelParameter() Online Documentation}*/
constructor();
/**
Specifies the offset of the label's center to the upper-left corner of the node.
@param dx the horizontal offset
@param dy the vertical offset
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModelParameter%23setPoint(number%2Cnumber) Online Documentation}*/
setPoint(dx: number, dy: number): void;
/**
Gets the offset of the label's center to the upper-left corner of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModelParameter%23point Online Documentation}*/
readonly point: YPoint | null;
static $class: Class;
private $_FreeNodeLabelLayoutModelParameter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreeNodeLabelLayoutModelParameter;
}
export interface FreeNodeLabelLayoutModel extends YObject, INodeLabelLayoutModel {
/**
@inheritdoc*/
getLabelPlacement(labelSize: YDimension | null, nodeLayout: INodeLayout | null, param: any | null): YOrientedRectangle | null;
} /**
{@link FreeNodeLabelLayoutModel} allows placing node labels at any desired position.
The location of the label does not need to satisfy any constraints. The {@link FreeNodeLabelLayoutModel#createModelParameter model parameter} describes a fixed offset between
the upper-left corner of the node and the center of the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModel Online Documentation}*/
export class FreeNodeLabelLayoutModel {
/**
Returns a new instance of {@link FreeNodeLabelLayoutModel}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModel Online Documentation}*/
constructor();
/**
Creates a model parameter that represents the given node label position within this model.
The created model parameter represents the current location of the given label.
This parameter can be passed to {@link FreeNodeLabelLayoutModel#getLabelPlacement} to retrieve the current label
box.
@param labelBounds the bounds of the label the parameter represents
@param nodeLayout the layout of the node to which the label belongs
@returns the model parameter
@throws {Exception} <b>(ArgumentError)</b> if one or more of the given bounds' values are invalid, i.e., <code>NaN</code>
@see FreeNodeLabelLayoutModel#getLabelPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModel%23createModelParameter(YOrientedRectangle%2CINodeLayout) Online Documentation}*/
createModelParameter(labelBounds: YOrientedRectangle | null, nodeLayout: INodeLayout | null): Object | null;
/**
Returns the {@link NodeLabelCandidate} that describes the current label position.
<h2>Note</h2>
The returned list contains only one candidate which describes the current location.
@param labelLayout the label for which candidates should be generated
@param nodeLayout the layout of the node to which the label belongs
@returns a list of {@link NodeLabelCandidate} instances
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModel%23getLabelCandidates(INodeLabelLayout%2CINodeLayout) Online Documentation}*/
getLabelCandidates(labelLayout: INodeLabelLayout | null, nodeLayout: INodeLayout | null): YList | null;
/**
Gets a model parameter that encodes the default position of {@link INodeLabelLayoutModel}.
The node label's upper-left corner is placed <code>20</code> pixels to the left and <code>20</code> pixels above the node.
This parameter can be passed to {@link FreeNodeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
label box.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodeLabelLayoutModel%23defaultParameter Online Documentation}*/
readonly defaultParameter: Object | null;
static $class: Class;
private $_FreeNodeLabelLayoutModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreeNodeLabelLayoutModel;
}
/**
@see ComponentLayout#style
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles Online Documentation}*/
export enum ComponentArrangementStyles {
/**
Style specifier describing no special component arrangement at all.
Components will be centered at the same position they resided at before the layout started. If combined with the style
modifier
{@link ComponentArrangementStyles#MODIFIER_NO_OVERLAP}, components might get moved so that they don't overlap
after the layout.
@see ComponentLayout#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23NONE Online Documentation}*/
NONE,
/**
Style specifier describing a component arrangement strategy that places components in multiple rows so that the overall
aspect ratio of the whole graph gets as close to the aspect ratio of the preferred layout size as possible.
@see ComponentLayout#style
@see ComponentLayout#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23ROWS Online Documentation}*/
ROWS,
/**
Style specifier describing a component arrangement strategy that places all components next to each other in a single
row.
If combined with the style modifier
{@link ComponentArrangementStyles#MODIFIER_AS_IS}, components will be placed in the same order as they were placed
before the layout.
@see ComponentLayout#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23SINGLE_ROW Online Documentation}*/
SINGLE_ROW,
/**
Style specifier describing a component arrangement strategy that places components above each other in a single column.
If combined with the style modifier
{@link ComponentArrangementStyles#MODIFIER_AS_IS}, components will be placed in the same order as they were placed
before the layout.
@see ComponentLayout#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23SINGLE_COLUMN Online Documentation}*/
SINGLE_COLUMN,
/**
Style specifier describing a component arrangement strategy that places components in a rectangular cloud around the
biggest component.
Entities of different components will not overlap, however the bounding boxes of the components may overlap.
The aspect ratio of the rectangle is defined by the {@link ComponentLayout#preferredSize preferred size} of the graph.
@see ComponentLayout#style
@see ComponentLayout#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23PACKED_RECTANGLE Online Documentation}*/
PACKED_RECTANGLE,
/**
Style specifier describing a component arrangement strategy that places components in a rectangular cloud around the
biggest component.
Entities of different components will not overlap, however the bounding boxes of the components may overlap.
In contrast to
{@link ComponentArrangementStyles#PACKED_RECTANGLE}, components might even be placed in empty spaces inside other
components.
The aspect ratio of the rectangle is defined by the {@link ComponentLayout#preferredSize preferred size} of the graph.
@see ComponentLayout#style
@see ComponentLayout#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23PACKED_COMPACT_RECTANGLE Online Documentation}*/
PACKED_COMPACT_RECTANGLE,
/**
Style specifier describing a component arrangement strategy that places components in a circular cloud around the
biggest component.
Entities of different components will not overlap, however the bounding boxes of the components may overlap.
@see ComponentLayout#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23PACKED_CIRCLE Online Documentation}*/
PACKED_CIRCLE,
/**
Style specifier describing a component arrangement strategy that places components in a circular compact cloud around
the biggest component.
Entities of different components will not overlap, however the bounding boxes of the components may overlap.
In contrast to
{@link ComponentArrangementStyles#PACKED_CIRCLE}, components might even be placed in empty spaces inside other
components.
@see ComponentLayout#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23PACKED_COMPACT_CIRCLE Online Documentation}*/
PACKED_COMPACT_CIRCLE,
/**
Style specifier describing a component arrangement strategy that places components in multiple rows.
The strategy tries to arrange the components such that the aspect ratio of the final component placement gets as close
as possible to the aspect ratio of the preferred layout size, configurable using {@link ComponentLayout#preferredSize}.
In contrast to
{@link ComponentArrangementStyles#ROWS}, each row may also be divided into further sub-rows, which is especially
useful if there are few very large components and many significantly smaller ones.
@see ComponentLayout#style
@see ComponentLayout#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23MULTI_ROWS Online Documentation}*/
MULTI_ROWS,
/**
Style specifier describing a component arrangement strategy that places components in multiple rows.
The strategy tries to arrange the components such that the aspect ratio of the final component placement gets close to
the aspect ratio of the preferred layout size, configurable using {@link ComponentLayout#preferredSize}. But in contrast to
{@link ComponentArrangementStyles#MULTI_ROWS}, this strategy does not always try to come as close to the preferred
view ratio, if a ratio close to the preferred ratio only induces a lot of unused view space. This unused space will be
minimized.
In contrast to
{@link ComponentArrangementStyles#ROWS}, each row may also be divided into further sub-rows, which is especially
useful if there are few very large components and many significantly smaller ones.
@see ComponentLayout#style
@see ComponentLayout#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23MULTI_ROWS_COMPACT Online Documentation}*/
MULTI_ROWS_COMPACT,
/**
Style specifier describing a component arrangement strategy that places the components in {@link ComponentArrangementStyles#MULTI_ROWS}
with a restricted height.
The resulting height does not exceed the preferred height ({@link ComponentLayout#preferredSize}).
In contrast to
{@link ComponentArrangementStyles#MULTI_ROWS_HEIGHT_CONSTRAINT_COMPACT}, the result will be an arrangement which
uses as much height as possible, even if that means that all components will be arranged in one single column.
<h2>Note</h2>
The preferred width and thus the aspect ratio will be ignored.
@see ComponentLayout#style
@see ComponentLayout#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23MULTI_ROWS_HEIGHT_CONSTRAINT Online Documentation}*/
MULTI_ROWS_HEIGHT_CONSTRAINT,
/**
Style specifier describing a component arrangement strategy that places the components in {@link ComponentArrangementStyles#MULTI_ROWS}
with a restricted height and as compact as possible.
The resulting height does not exceed the preferred height ({@link ComponentLayout#preferredSize}).
In comparison with
{@link ComponentArrangementStyles#MULTI_ROWS_HEIGHT_CONSTRAINT}, the result will be compacted, which means that an
arrangement with the minimum width such that the height constraint is still fulfilled will be realized.
<h2>Note</h2>
The preferred width and thus the aspect ratio will be ignored.
@see ComponentLayout#style
@see ComponentLayout#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23MULTI_ROWS_HEIGHT_CONSTRAINT_COMPACT Online Documentation}*/
MULTI_ROWS_HEIGHT_CONSTRAINT_COMPACT,
/**
Style specifier describing a component arrangement strategy that places the components in {@link ComponentArrangementStyles#MULTI_ROWS}
with a restricted width.
The resulting width does not exceed the preferred width ({@link ComponentLayout#preferredSize}).
In contrast to {@link ComponentArrangementStyles#MULTI_ROWS_WIDTH_CONSTRAINT_COMPACT} the result will be an
arrangement which uses as much space in width as possible, even if that means that all components will be arranged in a
single row.
<h2>Note</h2>
The preferred height and thus the aspect ratio will be ignored.
@see ComponentLayout#style
@see ComponentLayout#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23MULTI_ROWS_WIDTH_CONSTRAINT Online Documentation}*/
MULTI_ROWS_WIDTH_CONSTRAINT,
/**
Style specifier describing a component arrangement strategy that places the components in {@link ComponentArrangementStyles#MULTI_ROWS}
with a restricted width and as compact as possible.
The resulting width does not exceed the preferred width ({@link ComponentLayout#preferredSize}).
In comparison with {@link ComponentArrangementStyles#MULTI_ROWS_HEIGHT_CONSTRAINT} the result will be compacted,
in order to minimize unused view space induced by components that are large in height.
<h2>Note</h2>
The preferred height and thus the aspect ratio will be ignored.
@see ComponentLayout#style
@see ComponentLayout#preferredSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23MULTI_ROWS_WIDTH_CONSTRAINT_COMPACT Online Documentation}*/
MULTI_ROWS_WIDTH_CONSTRAINT_COMPACT,
/**
A constant for masking actual style constants from style modifiers.
@see ComponentLayout#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23MASK Online Documentation}*/
MASK,
/**
A modifier that can be added to normal style specifiers to indicate that the arrangement strategy should take the
initial positions of the components into account.
Combining this modifier is supported with the following styles: {@link ComponentArrangementStyles#SINGLE_ROW},
{@link ComponentArrangementStyles#SINGLE_COLUMN}, {@link ComponentArrangementStyles#PACKED_RECTANGLE},
{@link ComponentArrangementStyles#PACKED_COMPACT_RECTANGLE}, {@link ComponentArrangementStyles#PACKED_CIRCLE}
and {@link ComponentArrangementStyles#PACKED_COMPACT_CIRCLE}.
@see ComponentLayout#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23MODIFIER_AS_IS Online Documentation}*/
MODIFIER_AS_IS,
/**
A modifier that can be added to normal style specifiers to indicate that the arrangement strategy should produce a
non-overlapping component layout.
Combining this modifier is not supported with {@link ComponentArrangementStyles#NONE}.
<h2>Note</h2>
Non-overlapping component layout (in the sense that no elements from adjacent components overlap) is inherently
supported by all styles except for {@link ComponentArrangementStyles#NONE}.
@see ComponentLayout#style
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementStyles%23MODIFIER_NO_OVERLAP Online Documentation}*/
MODIFIER_NO_OVERLAP
}
type ComponentArrangementStylesStringValues = "none" | "rows" | "single-row" | "single-column" | "packed-rectangle" | "packed-compact-rectangle" | "packed-circle" | "packed-compact-circle" | "multi-rows" | "multi-rows-compact" | "multi-rows-height-constraint" | "multi-rows-height-constraint-compact" | "multi-rows-width-constraint" | "multi-rows-width-constraint-compact" | "mask" | "modifier-as-is" | "modifier-no-overlap";
/** Automatic conversion to {@link ComponentArrangementStyles}. */
type ComponentArrangementStylesConvertible = ComponentArrangementStylesStringValues | ComponentArrangementStylesStringValues[];
/**
@see HierarchicLayoutRoutingStyle#sourceCurveConnectionStyle
@see HierarchicLayoutRoutingStyle#targetCurveConnectionStyle
@see CurveEdgeLayoutDescriptor#sourceCurveConnectionStyle
@see CurveEdgeLayoutDescriptor#targetCurveConnectionStyle
@see EdgeRouterEdgeLayoutDescriptor#sourceCurveConnectionStyle
@see EdgeRouterEdgeLayoutDescriptor#targetCurveConnectionStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveConnectionStyle Online Documentation}*/
export enum CurveConnectionStyle {
/**
Curve connection style that keeps the port and the first/last segment that the layout algorithm would produce without
curve routing.
<ul>
<li>For the {@link HierarchicLayout} the edge ports are distributed according to the specified {@link HierarchicLayoutNodeLayoutDescriptor#portAssignment} and the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumFirstSegmentLength minimum first segment length}
as well as the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumLastSegmentLength minimum last segment length} are considered. This way the edges keep the original hierarchic connection
style and the first/last segment remains orthogonal.
<li>For the {@link EdgeRouter} the edge ports are distributed as usual and the the {@link EdgeRouterEdgeLayoutDescriptor#minimumFirstSegmentLength minimum first segment length} as
well as the {@link EdgeRouterEdgeLayoutDescriptor#minimumLastSegmentLength minimum last segment length} are considered. This way the edges keep the original connection style and the
first/last segment remains orthogonal.
<li>For the {@link CurveRoutingStage} the edge ports as provided by the specified {@link LayoutStageBase#coreLayout core layout algorithm} are kept,
including the starting segment provided by it. The {@link CurveEdgeLayoutDescriptor#minimumFirstSegmentLength minimum first segment length} and the {@link CurveEdgeLayoutDescriptor#minimumLastSegmentLength minimum last segment length} is
considered if the core layout did consider it too.
</ul>
@see HierarchicLayoutRoutingStyle#sourceCurveConnectionStyle
@see HierarchicLayoutRoutingStyle#targetCurveConnectionStyle
@see EdgeRouterEdgeLayoutDescriptor#sourceCurveConnectionStyle
@see EdgeRouterEdgeLayoutDescriptor#targetCurveConnectionStyle
@see CurveEdgeLayoutDescriptor#sourceCurveConnectionStyle
@see CurveEdgeLayoutDescriptor#targetCurveConnectionStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveConnectionStyle%23KEEP_PORT Online Documentation}*/
KEEP_PORT,
/**
Curve connection style that changes the part of a curved edge path such that the curves have a more organic-like and
straight start or end.
The port location is specifically amended for the curve routing such that the connecting part of the curve can be routed
directly into the required edge direction. This feature generates more direct and straighter routes that have less
direction turns. In consequence, the resulting layouts may be more compact overall as less space for the routing is
required.
The side where the port ends up at the source/target node may be changed with respect to the side the layout algorithm
initially chooses. It is not changed if there are port constraints defined for it (i.e. {@link PortConstraint}s or
{@link PortCandidate}s). This includes implicit constraints for the port due to port and edge grouping. Similarly,
this style can also not be applied if there is an edge label or
{@link NodeHalo}s that require the connecting segment to remain as it is.
In contrast to
{@link CurveConnectionStyle#KEEP_PORT}, this style does not guarantee to respect the specified minimum first and
last segment length settings.
@see HierarchicLayoutRoutingStyle#sourceCurveConnectionStyle
@see HierarchicLayoutRoutingStyle#targetCurveConnectionStyle
@see EdgeRouterEdgeLayoutDescriptor#sourceCurveConnectionStyle
@see EdgeRouterEdgeLayoutDescriptor#targetCurveConnectionStyle
@see CurveEdgeLayoutDescriptor#sourceCurveConnectionStyle
@see CurveEdgeLayoutDescriptor#targetCurveConnectionStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveConnectionStyle%23ORGANIC Online Documentation}*/
ORGANIC
}
/** Automatic conversion to {@link CurveConnectionStyle}. */
type CurveConnectionStyleStringValues = "keep-port" | "organic";
/**
@see DiscreteEdgeLabelLayoutModel#DiscreteEdgeLabelLayoutModel
@see DiscreteEdgeLabelLayoutModel#candidateMask
@see DiscreteEdgeLabelLayoutModel#getLabelPlacementForPosition
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions Online Documentation}*/
export enum DiscreteEdgeLabelPositions {
/**
Position specifier that describes a label placement near the source, left/above the edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23SOURCE_HEAD Online Documentation}*/
SOURCE_HEAD,
/**
Position specifier that describes a label placement near the middle of the edge, left/above the edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23HEAD Online Documentation}*/
HEAD,
/**
Position specifier that describes a label placement near the target, left/above the edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23TARGET_HEAD Online Documentation}*/
TARGET_HEAD,
/**
Position specifier that describes a label placement near the source, right/below the edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23SOURCE_TAIL Online Documentation}*/
SOURCE_TAIL,
/**
Position specifier that describes a label placement near the middle of the edge, right/below the edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23TAIL Online Documentation}*/
TAIL,
/**
Position specifier that describes a label placement near the target, right/below the edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23TARGET_TAIL Online Documentation}*/
TARGET_TAIL,
/**
Position specifier that describes a label placement near the source, directly on the edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23SOURCE_CENTER Online Documentation}*/
SOURCE_CENTER,
/**
Position specifier that describes a label placement near the middle of the edge, directly on the edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23CENTER Online Documentation}*/
CENTER,
/**
Position specifier that describes a label placement near the target, directly on the edge's path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23TARGET_CENTER Online Documentation}*/
TARGET_CENTER,
/**
Position mask that constrains the allowed positions to the two locations near the middle of the edge beside the edge's
path.
@see DiscreteEdgeLabelPositions#HEAD
@see DiscreteEdgeLabelPositions#TAIL
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23TWO_POS Online Documentation}*/
TWO_POS,
/**
Position mask that constrains the allowed positions to the {@link DiscreteEdgeLabelPositions#CENTER} of the edge's
path.
@see DiscreteEdgeLabelPositions#CENTER
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23CENTERED Online Documentation}*/
CENTERED,
/**
Position mask that constrains the allowed positions to a set of six positions beside the edge path.
Two of the positions are near the source and two others are near the target. The remaining positions are in the middle
of the edge.
@see DiscreteEdgeLabelPositions#SOURCE_HEAD
@see DiscreteEdgeLabelPositions#SOURCE_TAIL
@see DiscreteEdgeLabelPositions#HEAD
@see DiscreteEdgeLabelPositions#TAIL
@see DiscreteEdgeLabelPositions#TARGET_HEAD
@see DiscreteEdgeLabelPositions#TARGET_TAIL
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23SIX_POS Online Documentation}*/
SIX_POS,
/**
Position mask that constrains the allowed positions to a set of three positions directly on the edge's path.
@see DiscreteEdgeLabelPositions#SOURCE_CENTER
@see DiscreteEdgeLabelPositions#CENTER
@see DiscreteEdgeLabelPositions#TARGET_CENTER
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteEdgeLabelPositions%23THREE_CENTER Online Documentation}*/
THREE_CENTER
}
type DiscreteEdgeLabelPositionsStringValues = "source-head" | "head" | "target-head" | "source-tail" | "tail" | "target-tail" | "source-center" | "center" | "target-center" | "two-pos" | "centered" | "six-pos" | "three-center";
/** Automatic conversion to {@link DiscreteEdgeLabelPositions}. */
type DiscreteEdgeLabelPositionsConvertible = DiscreteEdgeLabelPositionsStringValues | DiscreteEdgeLabelPositionsStringValues[];
/**
@see DiscreteNodeLabelLayoutModel#DiscreteNodeLabelLayoutModel
@see DiscreteNodeLabelLayoutModel#candidateMask
@see DiscreteNodeLabelLayoutModel#getLabelPlacementForPosition
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions Online Documentation}*/
export enum DiscreteNodeLabelPositions {
/**
Position specifier that describes a label placement north of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23NORTH Online Documentation}*/
NORTH,
/**
Position specifier that describes a label placement north-west of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23NORTH_WEST Online Documentation}*/
NORTH_WEST,
/**
Position specifier that describes a label placement north-east of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23NORTH_EAST Online Documentation}*/
NORTH_EAST,
/**
Position specifier that describes a label placement east of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23EAST Online Documentation}*/
EAST,
/**
Position specifier that describes a label placement west of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23WEST Online Documentation}*/
WEST,
/**
Position specifier that describes a label placement south of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23SOUTH Online Documentation}*/
SOUTH,
/**
Position specifier that describes a label placement south-west of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23SOUTH_WEST Online Documentation}*/
SOUTH_WEST,
/**
Position specifier that describes a label placement south-east of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23SOUTH_EAST Online Documentation}*/
SOUTH_EAST,
/**
Position specifier that describes a label placement at the center of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23CENTER Online Documentation}*/
CENTER,
/**
Position specifier that describes a label placement at the bottom inside the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23BOTTOM Online Documentation}*/
BOTTOM,
/**
Position specifier that describes a label placement at the top inside the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23TOP Online Documentation}*/
TOP,
/**
Position specifier that describes a label placement at the left inside the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23LEFT Online Documentation}*/
LEFT,
/**
Position specifier that describes a label placement at the right inside the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23RIGHT Online Documentation}*/
RIGHT,
/**
Position specifier that describes a label placement at the top-left inside the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23TOP_LEFT Online Documentation}*/
TOP_LEFT,
/**
Position specifier that describes a label placement at the top-right inside the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23TOP_RIGHT Online Documentation}*/
TOP_RIGHT,
/**
Position specifier that describes a label placement at the bottom-left inside the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23BOTTOM_LEFT Online Documentation}*/
BOTTOM_LEFT,
/**
Position specifier that describes a label placement at the bottom-right inside the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23BOTTOM_RIGHT Online Documentation}*/
BOTTOM_RIGHT,
/**
Position mask that constrains the positions to {@link DiscreteNodeLabelPositions#NORTH above} and {@link DiscreteNodeLabelPositions#SOUTH below} the node.
@see DiscreteNodeLabelPositions#NORTH
@see DiscreteNodeLabelPositions#SOUTH
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23SANDWICH_MASK Online Documentation}*/
SANDWICH_MASK,
/**
Position mask that constrains the positions to the nine node-internal positions.
@see DiscreteNodeLabelPositions#CENTER
@see DiscreteNodeLabelPositions#TOP
@see DiscreteNodeLabelPositions#BOTTOM
@see DiscreteNodeLabelPositions#LEFT
@see DiscreteNodeLabelPositions#RIGHT
@see DiscreteNodeLabelPositions#TOP_LEFT
@see DiscreteNodeLabelPositions#TOP_RIGHT
@see DiscreteNodeLabelPositions#BOTTOM_LEFT
@see DiscreteNodeLabelPositions#BOTTOM_RIGHT
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23INTERNAL_MASK Online Documentation}*/
INTERNAL_MASK,
/**
Position mask that constrains the positions to the node's four sides.
@see DiscreteNodeLabelPositions#EAST
@see DiscreteNodeLabelPositions#WEST
@see DiscreteNodeLabelPositions#NORTH
@see DiscreteNodeLabelPositions#SOUTH
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23SIDES_MASK Online Documentation}*/
SIDES_MASK,
/**
Position mask that constrains the positions to the corners outside the node.
@see DiscreteNodeLabelPositions#NORTH_EAST
@see DiscreteNodeLabelPositions#NORTH_WEST
@see DiscreteNodeLabelPositions#SOUTH_EAST
@see DiscreteNodeLabelPositions#SOUTH_WEST
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23CORNER_MASK Online Documentation}*/
CORNER_MASK,
/**
Position mask that constraints the positions to the eight node-external positions.
@see DiscreteNodeLabelPositions#NORTH
@see DiscreteNodeLabelPositions#EAST
@see DiscreteNodeLabelPositions#SOUTH
@see DiscreteNodeLabelPositions#WEST
@see DiscreteNodeLabelPositions#NORTH_EAST
@see DiscreteNodeLabelPositions#NORTH_WEST
@see DiscreteNodeLabelPositions#SOUTH_EAST
@see DiscreteNodeLabelPositions#SOUTH_WEST
@see {@link https://docs.yworks.com/yfileshtml/#/api/DiscreteNodeLabelPositions%23EIGHT_POS_MASK Online Documentation}*/
EIGHT_POS_MASK
}
type DiscreteNodeLabelPositionsStringValues = "north" | "north-west" | "north-east" | "east" | "west" | "south" | "south-west" | "south-east" | "center" | "bottom" | "top" | "left" | "right" | "top-left" | "top-right" | "bottom-left" | "bottom-right" | "sandwich-mask" | "internal-mask" | "sides-mask" | "corner-mask" | "eight-pos-mask";
/** Automatic conversion to {@link DiscreteNodeLabelPositions}. */
type DiscreteNodeLabelPositionsConvertible = DiscreteNodeLabelPositionsStringValues | DiscreteNodeLabelPositionsStringValues[];
/**
Fix point calculation policy specifying that the <em>center</em> of a rectangle containing all fixed nodes is considered the fix
point.
@see FixNodeLayoutStage#fixPointPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPointPolicy Online Documentation}*/
export enum FixPointPolicy {
/**
Fix point calculation policy specifying that the <em>center</em> of a rectangle containing all fixed nodes is considered the fix
point.
@see FixNodeLayoutStage#fixPointPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPointPolicy%23CENTER Online Documentation}*/
CENTER,
/**
Fix point calculation policy specifying that the <em>upper left corner</em> of a rectangle containing all fixed nodes is considered the fix
point.
@see FixNodeLayoutStage#fixPointPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPointPolicy%23UPPER_LEFT Online Documentation}*/
UPPER_LEFT,
/**
Fix point calculation policy specifying that the <em>upper right corner</em> of a rectangle containing all fixed nodes is considered the fix
point.
@see FixNodeLayoutStage#fixPointPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPointPolicy%23UPPER_RIGHT Online Documentation}*/
UPPER_RIGHT,
/**
Fix point calculation policy specifying that the <em>lower right corner</em> of a rectangle containing all fixed nodes is considered the fix
point.
@see FixNodeLayoutStage#fixPointPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPointPolicy%23LOWER_RIGHT Online Documentation}*/
LOWER_RIGHT,
/**
Fix point calculation policy specifying that the <em>lower left corner</em> of a rectangle containing all fixed nodes is considered the fix
point.
@see FixNodeLayoutStage#fixPointPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPointPolicy%23LOWER_LEFT Online Documentation}*/
LOWER_LEFT
}
/** Automatic conversion to {@link FixPointPolicy}. */
type FixPointPolicyStringValues = "center" | "upper-left" | "upper-right" | "lower-right" | "lower-left";
/**
@see PreferredPlacementDescriptor#newSharedInstance
@see PreferredPlacementDescriptor#placeAlongEdge
@see PreferredPlacementDescriptor#sideOfEdge
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements Online Documentation}*/
export enum LabelPlacements {
/**
Preferred placement specifier for edge labels which expresses that the label should be placed near the source node of an
edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23AT_SOURCE Online Documentation}*/
AT_SOURCE,
/**
Preferred placement specifier for edge labels which expresses that the label should be placed near the target node of an
edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23AT_TARGET Online Documentation}*/
AT_TARGET,
/**
Preferred placement specifier for edge labels which expresses that the label should be placed near the center of the
edge path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23AT_CENTER Online Documentation}*/
AT_CENTER,
/**
Preferred placement specifier for edge labels which expresses that the label should be placed directly at the source
port of the edge.
<b>Warning:</b> Placement at the source port is currently only supported by {@link HierarchicLayout} when enabling
{@link HierarchicLayout#integratedEdgeLabeling integrated edge labeling}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23AT_SOURCE_PORT Online Documentation}*/
AT_SOURCE_PORT,
/**
Preferred placement specifier for edge labels which expresses that the label should be placed directly at the target
port of the edge.
<b>Warning:</b> Placement at the target port is currently only supported by {@link HierarchicLayout} when enabling
{@link HierarchicLayout#integratedEdgeLabeling integrated edge labeling}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23AT_TARGET_PORT Online Documentation}*/
AT_TARGET_PORT,
/**
Placement specifier mask that masks all placement specifiers that describe the placement along the edge.
<h2>Note</h2>
To determine the preferred placement along the edge, this mask can be used to hide the side specifiers that may also be
encoded in the preferred placement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23ALONG_EDGE_MASK Online Documentation}*/
ALONG_EDGE_MASK,
/**
Preferred placement specifier for edge labels which expresses that the label should be placed on the path of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23ON_EDGE Online Documentation}*/
ON_EDGE,
/**
Preferred placement specifier for edge labels which expresses that the label should be placed on the left hand side of
the edge path.
The side is interpreted relative to the direction of the edge, if no different {@link PreferredPlacementDescriptor#sideReference side reference} is defined.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23LEFT_OF_EDGE Online Documentation}*/
LEFT_OF_EDGE,
/**
Preferred placement specifier for edge labels which expresses that the label should be placed on the right hand side of
the edge path.
The side is interpreted relative to the direction of the edge, if no different {@link PreferredPlacementDescriptor#sideReference side reference} is defined.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23RIGHT_OF_EDGE Online Documentation}*/
RIGHT_OF_EDGE,
/**
Placement specifier mask that masks all placement specifiers that describe the side of the edge.
<h2>Note</h2>
To determine the preferred side of the edge, this mask can be used to hide the placement specifiers that may also be
encoded in the preferred placement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23ON_SIDE_OF_EDGE_MASK Online Documentation}*/
ON_SIDE_OF_EDGE_MASK,
/**
Preferred placement specifier for edge labels which expresses that the label can be placed anywhere along the edge or on
any side of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPlacements%23ANYWHERE Online Documentation}*/
ANYWHERE
}
type LabelPlacementsStringValues = "at-source" | "at-target" | "at-center" | "at-source-port" | "at-target-port" | "along-edge-mask" | "on-edge" | "left-of-edge" | "right-of-edge" | "on-side-of-edge-mask" | "anywhere";
/** Automatic conversion to {@link LabelPlacements}. */
type LabelPlacementsConvertible = LabelPlacementsStringValues | LabelPlacementsStringValues[];
/**
Orientation specifier which defines that the main layout orientation is from top to bottom.
@see EdgeLabelOrientationSupport#getOrientedDirection
@see EdgeLabelOrientationSupport#isOrientationMirrored
@see PortConstraintOptimizerBase#layoutOrientation
@see MultiStageLayout#layoutOrientation
@see OrientationLayout#OrientationLayout
@see OrientationLayout#isOrientationMirrored
@see OrientationLayout#orientation
@see PortCandidate#getDirectionForLayoutOrientation
@see PortCandidate#getXOffsetForLayoutOrientation
@see PortCandidate#getXOffsetForLayoutOrientation
@see PortCandidate#getYOffsetForLayoutOrientation
@see PortCandidate#getYOffsetForLayoutOrientation
@see PortCandidate#toPortConstraintForLayoutOrientation
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutOrientation Online Documentation}*/
export enum LayoutOrientation {
/**
Orientation specifier which defines that the main layout orientation is from top to bottom.
This is the default for all layout algorithms.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutOrientation%23TOP_TO_BOTTOM Online Documentation}*/
TOP_TO_BOTTOM,
/**
Orientation specifier which defines that the main layout orientation is from left to right.
The layout will be rotated by <code>90</code> degrees counterclockwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutOrientation%23LEFT_TO_RIGHT Online Documentation}*/
LEFT_TO_RIGHT,
/**
Orientation specifier which defines that the main layout orientation is from right to left.
The layout will be rotated by <code>90</code> degrees clockwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutOrientation%23RIGHT_TO_LEFT Online Documentation}*/
RIGHT_TO_LEFT,
/**
Orientation specifier which defines that the main layout orientation is from bottom to top.
The layout will be flipped along the x-axis.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutOrientation%23BOTTOM_TO_TOP Online Documentation}*/
BOTTOM_TO_TOP
}
/** Automatic conversion to {@link LayoutOrientation}. */
type LayoutOrientationStringValues = "top-to-bottom" | "left-to-right" | "right-to-left" | "bottom-to-top";
/**
Row alignment specifier which defines that each row will be aligned with its center.
@see LayoutGraphUtilities#arrangeRectangleMultiRows
@see LayoutGraphUtilities#arrangeRectangleRows
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowAlignment Online Documentation}*/
export enum RowAlignment {
/**
Row alignment specifier which defines that each row will be aligned with its left border.
@see LayoutGraphUtilities#arrangeRectangleRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowAlignment%23LEADING Online Documentation}*/
LEADING,
/**
Row alignment specifier which defines that each row will be aligned with its right border.
@see LayoutGraphUtilities#arrangeRectangleRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowAlignment%23TRAILING Online Documentation}*/
TRAILING,
/**
Row alignment specifier which defines that each row will be aligned with its left border and its right border.
Rectangles in smaller rows will be distributed over the row, so that the row aligns with both sides.
@see LayoutGraphUtilities#arrangeRectangleRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowAlignment%23JUSTIFIED Online Documentation}*/
JUSTIFIED,
/**
Row alignment specifier which defines that each row will be aligned with its center.
@see LayoutGraphUtilities#arrangeRectangleRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowAlignment%23CENTER Online Documentation}*/
CENTER
}
/** Automatic conversion to {@link RowAlignment}. */
type RowAlignmentStringValues = "leading" | "trailing" | "justified" | "center";
/**
@see LayoutGraphUtilities#arrangeRectangleMultiRows
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiRowConstraint Online Documentation}*/
export enum MultiRowConstraint {
/**
Constraint specifier for a multi-row rectangle arrangement that is unconstrained.
Both, height and width, are not limited in size. The rectangles will be arranged in a way that the view ratio defined by
the preferred height and width will be achieved.
@see LayoutGraphUtilities#arrangeRectangleMultiRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiRowConstraint%23NONE Online Documentation}*/
NONE,
/**
Constraint specifier for a multi-row rectangle arrangement that constrains the height of the layout.
Limits the height of the arrangement to the given preferred height value. If some rectangles are higher than allowed,
then they are placed nevertheless, but the constraint will still be considered for the rest and not weakened to the
height of the higher rectangles. Having one rectangle being higher than the constraint does always result in
arrangements consisting only of a single top-level row.
@see LayoutGraphUtilities#arrangeRectangleMultiRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiRowConstraint%23HEIGHT Online Documentation}*/
HEIGHT,
/**
Constraint specifier for a multi-row rectangle arrangement that constrains the width of the layout.
Limits the width of the arrangement to the given preferred width value. If a rectangle is wider than allowed, then it
will nevertheless be placed (in a row containing the rectangle alone), but the constraint will be still considered for
other rows and not weakened.
@see LayoutGraphUtilities#arrangeRectangleMultiRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiRowConstraint%23WIDTH Online Documentation}*/
WIDTH
}
/** Automatic conversion to {@link MultiRowConstraint}. */
type MultiRowConstraintStringValues = "none" | "height" | "width";
/**
@see LayoutGraphUtilities#determineEdgeFlowDirection
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFlow Online Documentation}*/
export enum EdgeFlow {
/**
Edge flow specifier which defines that the flow direction of most edges is up.
@see LayoutGraphUtilities#determineEdgeFlowDirection
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFlow%23UP Online Documentation}*/
UP,
/**
Edge flow specifier which defines that the flow direction of most edges is down.
@see LayoutGraphUtilities#determineEdgeFlowDirection
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFlow%23DOWN Online Documentation}*/
DOWN,
/**
Edge flow specifier which defines that the flow direction of most edges is left.
@see LayoutGraphUtilities#determineEdgeFlowDirection
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFlow%23LEFT Online Documentation}*/
LEFT,
/**
Edge flow specifier which defines that the flow direction of most edges is right.
@see LayoutGraphUtilities#determineEdgeFlowDirection
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFlow%23RIGHT Online Documentation}*/
RIGHT,
/**
Edge flow specifier that defines that there is no main flow direction.
@see LayoutGraphUtilities#determineEdgeFlowDirection
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeFlow%23NONE Online Documentation}*/
NONE
}
/** Automatic conversion to {@link EdgeFlow}. */
type EdgeFlowStringValues = "up" | "down" | "left" | "right" | "none";
/**
@see LayoutGraphUtilities#alignNodeLayouts
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignment Online Documentation}*/
export enum NodeAlignment {
/**
Node alignment policy that results in top-aligned nodes for vertical alignment and left-aligned nodes for horizontal
alignments.
@see LayoutGraphUtilities#alignNodeLayouts
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignment%23LEADING Online Documentation}*/
LEADING,
/**
Node alignment policy that results in center-aligned nodes.
@see LayoutGraphUtilities#alignNodeLayouts
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignment%23CENTERED Online Documentation}*/
CENTERED,
/**
Node alignment policy that results in bottom-aligned nodes for vertical alignment and right-aligned nodes for horizontal
alignments.
@see LayoutGraphUtilities#alignNodeLayouts
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignment%23TRAILING Online Documentation}*/
TRAILING,
/**
Node alignment policy that results in vertically or horizontally distributed nodes.
In this context, <em>distributed</em> means that the vertical (or horizontal) extensions of nodes do not overlap, independent of their
horizontal (or vertical) position.
@see LayoutGraphUtilities#alignNodeLayouts
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAlignment%23DISTRIBUTED Online Documentation}*/
DISTRIBUTED
}
/** Automatic conversion to {@link NodeAlignment}. */
type NodeAlignmentStringValues = "leading" | "centered" | "trailing" | "distributed";
/**
@see EdgeLabelOrientationSupport#getOrientedDirection
@see EdgeLabelOrientationSupport#isOrientationMirrored
@see HierarchicLayout#getMirrorMode
@see PortConstraintOptimizerBase#mirrorMode
@see OrientationLayout#mirrorMode
@see PortCandidate#getXOffsetForLayoutOrientation
@see PortCandidate#getYOffsetForLayoutOrientation
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MirrorModes Online Documentation}*/
export enum MirrorModes {
/**
Mirroring specifier that can be used for preventing any mirroring of layout orientations.
@see OrientationLayout#mirrorMode
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/MirrorModes%23NONE Online Documentation}*/
NONE,
/**
Mirroring specifier that defines that layout orientation {@link LayoutOrientation#TOP_TO_BOTTOM} will be
mirrored.
This specifier can be used for creating a mirror mask that defines which layout orientations shall be mirrored at their
corresponding axis. It can be combined with the other mirroring specifiers to mirror several orientations.
@see OrientationLayout#mirrorMode
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/MirrorModes%23TOP_TO_BOTTOM Online Documentation}*/
TOP_TO_BOTTOM,
/**
Mirroring specifier that defines that layout orientation {@link LayoutOrientation#RIGHT_TO_LEFT} will be
mirrored.
This specifier can be used for creating a mirror mask that defines which layout orientations shall be mirrored at their
corresponding axis. It can be combined with the other mirroring specifiers to mirror several orientations.
@see OrientationLayout#mirrorMode
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/MirrorModes%23RIGHT_TO_LEFT Online Documentation}*/
RIGHT_TO_LEFT,
/**
Mirroring specifier that defines that layout orientation {@link LayoutOrientation#BOTTOM_TO_TOP} will be
mirrored.
This specifier can be used for creating a mirror mask that defines which layout orientations shall be mirrored at their
corresponding axis. It can be combined with the other mirroring specifiers to mirror several orientations.
@see OrientationLayout#mirrorMode
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/MirrorModes%23BOTTOM_TO_TOP Online Documentation}*/
BOTTOM_TO_TOP,
/**
Mirroring specifier that defines that layout orientation {@link LayoutOrientation#LEFT_TO_RIGHT} will be
mirrored.
This specifier can be used to create a mirror mask that defines which layout orientations shall be mirrored at their
corresponding axis. It can be combined with the other mirroring specifiers to mirror several orientations.
@see OrientationLayout#mirrorMode
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/MirrorModes%23LEFT_TO_RIGHT Online Documentation}*/
LEFT_TO_RIGHT
}
type MirrorModesStringValues = "none" | "top-to-bottom" | "right-to-left" | "bottom-to-top" | "left-to-right";
/** Automatic conversion to {@link MirrorModes}. */
type MirrorModesConvertible = MirrorModesStringValues | MirrorModesStringValues[];
/**
@see PortCandidate#createCandidate
@see PortCandidate#createCandidate
@see PortCandidate#direction
@see PortCandidate#getDirectionForLayoutOrientation
@see PortCandidate#isInDirection
@see IPortCandidateMatcher#findMatchingCandidate
@see IPortCandidateMatcher#findMatchingCandidate
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections Online Documentation}*/
export enum PortDirections {
/**
A port direction specifier indicating a port on the north (i.e., top) side of a node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections%23NORTH Online Documentation}*/
NORTH,
/**
A port direction specifier indicating a port on the east (i.e., right) side of a node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections%23EAST Online Documentation}*/
EAST,
/**
A port direction specifier indicating a port on the west (i.e., left) side of a node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections%23WEST Online Documentation}*/
WEST,
/**
A port direction specifier indicating a port on the south (i.e., bottom) side of a node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections%23SOUTH Online Documentation}*/
SOUTH,
/**
A port direction specifier indicating a port lying in the direction of the main flow of the edge or of the overall
layout.
The exact direction is domain-specific.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections%23WITH_THE_FLOW Online Documentation}*/
WITH_THE_FLOW,
/**
A port direction specifier indicating a port lying in the opposite direction of the main flow of the edge or of the
overall layout.
The exact direction is domain-specific.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections%23AGAINST_THE_FLOW Online Documentation}*/
AGAINST_THE_FLOW,
/**
A port direction specifier indicating a port lying left of the direction of the main flow of the edge or of the overall
layout.
The exact direction is domain-specific.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections%23LEFT_IN_FLOW Online Documentation}*/
LEFT_IN_FLOW,
/**
A port direction specifier indicating a port lying right of the direction of the main flow of the edge or of the overall
layout.
The exact direction is domain-specific.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections%23RIGHT_IN_FLOW Online Documentation}*/
RIGHT_IN_FLOW,
/**
A combination of all possible direction specifiers indicating that edges can connect to any side of a node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortDirections%23ANY Online Documentation}*/
ANY
}
type PortDirectionsStringValues = "north" | "east" | "west" | "south" | "with-the-flow" | "against-the-flow" | "left-in-flow" | "right-in-flow" | "any";
/** Automatic conversion to {@link PortDirections}. */
type PortDirectionsConvertible = PortDirectionsStringValues | PortDirectionsStringValues[];
/**
@see PortConstraint#create
@see PortConstraint#side
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortSide Online Documentation}*/
export enum PortSide {
/**
A weak constraint indicating that edges can connect to any side of a node.
@see PortConstraint#create
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortSide%23ANY Online Documentation}*/
ANY,
/**
A weak constraint indicating that edges can connect to the north side (top side) of a node.
@see PortConstraint#create
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortSide%23NORTH Online Documentation}*/
NORTH,
/**
A weak constraint indicating that edges can connect to the south side (bottom side) of a node.
@see PortConstraint#create
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortSide%23SOUTH Online Documentation}*/
SOUTH,
/**
A weak constraint indicating that edges can connect to the east side (right side) of a node.
@see PortConstraint#create
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortSide%23EAST Online Documentation}*/
EAST,
/**
A weak constraint indicating that edges can connect to the west side (left side) of a node.
@see PortConstraint#create
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortSide%23WEST Online Documentation}*/
WEST
}
/** Automatic conversion to {@link PortSide}. */
type PortSideStringValues = "any" | "north" | "south" | "east" | "west";
/**
@see PreferredPlacementDescriptor#sideReference
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSideReferences Online Documentation}*/
export enum LabelSideReferences {
/**
Side placement specifier for edge labels which expresses that the preferred side of the edge is interpreted relative to
the edge flow.
@see PreferredPlacementDescriptor#sideReference
@see PreferredPlacementDescriptor#sideOfEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSideReferences%23RELATIVE_TO_EDGE_FLOW Online Documentation}*/
RELATIVE_TO_EDGE_FLOW,
/**
Side placement specifier for edge labels which defines that the preferred side for the label is interpreted
independently of the edge direction placing {@link LabelPlacements#LEFT_OF_EDGE left labels} above horizontal segments.
For vertical segments the label is placed {@link LabelPlacements#LEFT_OF_EDGE left} or {@link LabelPlacements#RIGHT_OF_EDGE right} of the edge. In case the segment is horizontal, the label is
placed above the segment if it should be left of the edge, otherwise below the segment.
@see PreferredPlacementDescriptor#sideReference
@see PreferredPlacementDescriptor#sideOfEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSideReferences%23ABSOLUTE_WITH_LEFT_IN_NORTH Online Documentation}*/
ABSOLUTE_WITH_LEFT_IN_NORTH,
/**
Side placement specifier for edge labels which defines that the preferred side for the label is interpreted
independently of the edge direction placing {@link LabelPlacements#RIGHT_OF_EDGE left labels} above horizontal segments.
For vertical segments the label is placed {@link LabelPlacements#LEFT_OF_EDGE left} or {@link LabelPlacements#RIGHT_OF_EDGE right} of the edge. In case the segment is horizontal, the label is
placed above the segment if it should be right of the edge, otherwise below the segment.
@see PreferredPlacementDescriptor#sideReference
@see PreferredPlacementDescriptor#sideOfEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelSideReferences%23ABSOLUTE_WITH_RIGHT_IN_NORTH Online Documentation}*/
ABSOLUTE_WITH_RIGHT_IN_NORTH
}
type LabelSideReferencesStringValues = "relative-to-edge-flow" | "absolute-with-left-in-north" | "absolute-with-right-in-north";
/** Automatic conversion to {@link LabelSideReferences}. */
type LabelSideReferencesConvertible = LabelSideReferencesStringValues | LabelSideReferencesStringValues[];
/**
Angle interpretation specifier for edge labels which expresses that the rotation angle should be interpreted relative to
the edge slope.
@see PreferredPlacementDescriptor#angleReference
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelAngleReferences Online Documentation}*/
export enum LabelAngleReferences {
/**
Angle interpretation specifier for edge labels which expresses that the rotation angle should be interpreted absolute
and not relative to the edge slope.
@see PreferredPlacementDescriptor#angleReference
@see PreferredPlacementDescriptor#angle
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelAngleReferences%23ABSOLUTE Online Documentation}*/
ABSOLUTE,
/**
Angle interpretation specifier for edge labels which expresses that the rotation angle should be interpreted relative to
the edge slope.
@see PreferredPlacementDescriptor#angleReference
@see PreferredPlacementDescriptor#angle
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelAngleReferences%23RELATIVE_TO_EDGE_FLOW Online Documentation}*/
RELATIVE_TO_EDGE_FLOW
}
type LabelAngleReferencesStringValues = "absolute" | "relative-to-edge-flow";
/** Automatic conversion to {@link LabelAngleReferences}. */
type LabelAngleReferencesConvertible = LabelAngleReferencesStringValues | LabelAngleReferencesStringValues[];
/**
Angle interpretation specifier which expresses that the rotation angle should be interpreted <em>clockwise</em>.
@see PreferredPlacementDescriptor#angleRotationOnRightSide
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelAngleOnRightSideRotations Online Documentation}*/
export enum LabelAngleOnRightSideRotations {
/**
Angle interpretation specifier which expresses that the rotation angle should be interpreted <em>clockwise</em>.
Therefore, edge labels whose descriptor {@link PreferredPlacementDescriptor#isRightOfEdge} will be co-rotating with the angle of labels that are placed left of or
centered on the edge.
@see PreferredPlacementDescriptor#angleRotationOnRightSide
@see PreferredPlacementDescriptor#angle
@see PreferredPlacementDescriptor#sideOfEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelAngleOnRightSideRotations%23CLOCKWISE Online Documentation}*/
CLOCKWISE,
/**
Angle interpretation specifier which expresses that the rotation angle should be interpreted <em>counter-clockwise</em>.
Therefore, edge labels whose descriptor {@link PreferredPlacementDescriptor#isRightOfEdge} will be counter-rotating with the angle of labels that are placed left of or
centered on the edge.
@see PreferredPlacementDescriptor#angleRotationOnRightSide
@see PreferredPlacementDescriptor#angle
@see PreferredPlacementDescriptor#sideOfEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelAngleOnRightSideRotations%23COUNTER_CLOCKWISE Online Documentation}*/
COUNTER_CLOCKWISE
}
type LabelAngleOnRightSideRotationsStringValues = "clockwise" | "counter-clockwise";
/** Automatic conversion to {@link LabelAngleOnRightSideRotations}. */
type LabelAngleOnRightSideRotationsConvertible = LabelAngleOnRightSideRotationsStringValues | LabelAngleOnRightSideRotationsStringValues[];
/**
Angle offset specifier for edge labels which expresses that no additional offset should be added to the angle.
@see PreferredPlacementDescriptor#angleOffsetOnRightSide
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelAngleOnRightSideOffsets Online Documentation}*/
export enum LabelAngleOnRightSideOffsets {
/**
Angle offset specifier for edge labels which expresses that no additional offset should be added to the angle.
@see PreferredPlacementDescriptor#angleOffsetOnRightSide
@see PreferredPlacementDescriptor#angle
@see PreferredPlacementDescriptor#sideOfEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelAngleOnRightSideOffsets%23NONE Online Documentation}*/
NONE,
/**
Angle offset specifier for edge labels which expresses that an additional offset of <code>180</code> degrees should be added to the
angle.
That way, point-symmetric label placements left and right of the edge can be attained.
@see PreferredPlacementDescriptor#angleOffsetOnRightSide
@see PreferredPlacementDescriptor#angle
@see PreferredPlacementDescriptor#sideOfEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelAngleOnRightSideOffsets%23SEMI Online Documentation}*/
SEMI
}
type LabelAngleOnRightSideOffsetsStringValues = "none" | "semi";
/** Automatic conversion to {@link LabelAngleOnRightSideOffsets}. */
type LabelAngleOnRightSideOffsetsConvertible = LabelAngleOnRightSideOffsetsStringValues | LabelAngleOnRightSideOffsetsStringValues[];
/**
Slider mode specifier which describes continuous label positions directly on the edge path.
@see SliderEdgeLabelLayoutModel
@see SliderEdgeLabelLayoutModel#mode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderMode Online Documentation}*/
export enum SliderMode {
/**
Slider mode specifier which describes continuous label positions directly on the edge path.
@see SliderEdgeLabelLayoutModel#mode
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderMode%23CENTER Online Documentation}*/
CENTER,
/**
Slider mode specifier which describes continuous label positions along the sides of the edge path.
@see SliderEdgeLabelLayoutModel#mode
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderMode%23SIDE Online Documentation}*/
SIDE,
/**
Slider mode specifier which describes continuous label positions along one side of the edge path.
<h2>Note</h2>
The side of the slider is determined by the
{@link SliderEdgeLabelLayoutModel#distance distance}. A positive distance will result in a right slider while negative distance will result in a left slider. Those
sides are defined relative to the segment's direction.
@see SliderEdgeLabelLayoutModel#mode
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderMode%23SINGLE_SIDE Online Documentation}*/
SINGLE_SIDE
}
/** Automatic conversion to {@link SliderMode}. */
type SliderModeStringValues = "center" | "side" | "single-side";
/**
Self-loop style specifier which defines that the routes of self-loops are rounded.
@see SelfLoopRouter#layoutStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopStyle Online Documentation}*/
export enum SelfLoopStyle {
/**
Self-loop style specifier which defines that the routes of self-loops are orthogonal.
The routes consist of an alternating sequence of horizontal and vertical line segments.
@see SelfLoopRouter#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopStyle%23ORTHOGONAL Online Documentation}*/
ORTHOGONAL,
/**
Self-loop style specifier which defines that the routes of self-loops are rounded.
The {@link SelfLoopRouter#lineDistance distance between incident self-loops} and {@link SelfLoopRouter#cornerApproximationPointsCount rounding of the corners} can be customized for rounded self-loops.
@see SelfLoopRouter#layoutStyle
@see SelfLoopRouter#lineDistance
@see SelfLoopRouter#cornerApproximationPointsCount
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopStyle%23ROUNDED Online Documentation}*/
ROUNDED
}
/** Automatic conversion to {@link SelfLoopStyle}. */
type SelfLoopStyleStringValues = "orthogonal" | "rounded";
/**
@see Swimlanes#arrangeSwimlanes
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlanesMode Online Documentation}*/
export enum SwimlanesMode {
/**
Swimlane ordering mode specifier defining that node grouping is ignored during the swimlane ordering optimization.
@see Swimlanes#arrangeSwimlanes
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlanesMode%23IGNORE_GROUPS Online Documentation}*/
IGNORE_GROUPS,
/**
Swimlane ordering mode specifier defining that only node grouping is considered during the swimlane ordering
optimization.
@see Swimlanes#arrangeSwimlanes
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlanesMode%23ONLY_GROUPS Online Documentation}*/
ONLY_GROUPS,
/**
Swimlane ordering mode specifier defining that both node grouping and edge length are considered during the swimlane
ordering optimization.
@see Swimlanes#arrangeSwimlanes
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlanesMode%23MIXED Online Documentation}*/
MIXED
}
/** Automatic conversion to {@link SwimlanesMode}. */
type SwimlanesModeStringValues = "ignore-groups" | "only-groups" | "mixed";
export interface GivenCoordinatesStage extends LayoutStageBase {
}
/**
{@link GivenCoordinatesStage} changes node locations and sizes as well as edge paths to user-specified values
before invoking the {@link LayoutStageBase#coreLayout core layout algorithm}.
Use {@link IDataProvider}-key {@link GivenCoordinatesStage#NODE_LOCATION_DP_KEY} to specify the
locations (top-left coordinate) of nodes, {@link IDataProvider}-key {@link GivenCoordinatesStage#NODE_SIZE_DP_KEY}
to specify the size of nodes and {@link IDataProvider}-key {@link GivenCoordinatesStage#EDGE_PATH_DP_KEY}
to specify the path of edges (i.e. the port locations and the bends of edges).
A possible application for this stage are animated incremental layouts where the position of non-incremental elements
should be changed/restored without affecting the start position of theses elements in the animation. Therefore, this
stage could also be used together with {@link PlaceNodesAtBarycenterStage}.
<h2>Note</h2>
Even though this stage moves nodes and paths to the specified coordinates, the {@link LayoutStageBase#coreLayout core layout algorithm} may change these
positions.
@see PlaceNodesAtBarycenterStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStage Online Documentation}*/
export class GivenCoordinatesStage {
/**
Creates a new {@link GivenCoordinatesStage} instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] the core layout algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Changes node locations and edge paths to user-specified values and then invokes the core layout algorithm.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for specifying the initial location of nodes
Each node can be mapped to an instance of {@link YPoint}. Before calling the {@link LayoutStageBase#coreLayout core layout algorithm}, this stage
places the top-left coordinate of each node on the associated
{@link YPoint}. The location of nodes without associated value isn't changed by this stage (but may be changed
by the core layout algorithm).
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStage%23NODE_LOCATION_DP_KEY Online Documentation}*/
static NODE_LOCATION_DP_KEY: NodeDpKey<YPoint> | null;
/**
Data provider key for specifying the size of nodes
Each node can be mapped to an instance of {@link YDimension}. Before calling the {@link LayoutStageBase#coreLayout core layout algorithm}, this
stage sets the size of each node to the specified
{@link YDimension size}. The size of nodes without associated value isn't changed by this stage.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStage%23NODE_SIZE_DP_KEY Online Documentation}*/
static NODE_SIZE_DP_KEY: NodeDpKey<YDimension> | null;
/**
Data provider key for specifying the initial path of edges
Each edge can be mapped to an instance of {@link YPointPath}. Before calling the {@link LayoutStageBase#coreLayout core layout algorithm}, this
stage sets the path of each edge to the associated
{@link YPointPath}. The path must contain the source port location, followed by the bends (if any), followed
by the target port location.
If an empty path is specified, the current path will be reset: all bends are cleared and the current source/target port
locations are kept. The path of edges without associated value isn't changed by this stage (but may be changed by the
core layout algorithm).
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStage%23EDGE_PATH_DP_KEY Online Documentation}*/
static EDGE_PATH_DP_KEY: EdgeDpKey<YPointPath> | null;
static $class: Class;
private $_GivenCoordinatesStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GivenCoordinatesStage;
}
export interface GraphLayoutLineWrapper extends LayoutStageBase {
}
/**
{@link GraphLayoutLineWrapper} can line-wrap or column-wrap a graph layout.
This {@link ILayoutStage} supports both wrapping a layout at a given width (height) and wrapping a layout
automatically at some width (height) such that the resulting overall aspect ratio is close to a desired {@link GraphLayoutLineWrapper#targetRatio target ratio}.
The {@link GraphLayoutLineWrapper} can be used on an existing graph layout or on the result of the
{@link LayoutStageBase#coreLayout core layout algorithm}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper Online Documentation}*/
export class GraphLayoutLineWrapper {
/**
Creates a new {@link GraphLayoutLineWrapper} instance with default settings.
@param [options] The parameters to pass.
@param [options.edgeSpacing] The space that should be kept between edges. This option sets the {@link GraphLayoutLineWrapper#edgeSpacing} property on the created object.
@param [options.columnMode] Whether this {@link ILayoutStage} wraps lines/rows or columns. This option sets the {@link GraphLayoutLineWrapper#columnMode} property on the created object.
@param [options.spacing] The space between adjacent lines of the wrapped graph layout. This option sets the {@link GraphLayoutLineWrapper#spacing} property on the created object.
@param [options.mirror] Whether or not every other line is mirrored. This option sets the {@link GraphLayoutLineWrapper#mirror} property on the created object.
@param [options.fixedWidthLineBreaks] Whether the algorithm should use the {@link GraphLayoutLineWrapper#fixedWidth fixed width} or {@link GraphLayoutLineWrapper#targetRatio target aspect ratio} to determine line breaks. This option sets the {@link GraphLayoutLineWrapper#fixedWidthLineBreaks} property on the created object.
@param [options.targetRatio] The desired target aspect ratio for the layout. This option sets the {@link GraphLayoutLineWrapper#targetRatio} property on the created object.
@param [options.fixedWidth] The desired width of the lines. This option sets the {@link GraphLayoutLineWrapper#fixedWidth} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper Online Documentation}*/
constructor(options?: {
edgeSpacing?: number;
columnMode?: boolean;
spacing?: number;
mirror?: boolean;
fixedWidthLineBreaks?: boolean;
targetRatio?: number;
fixedWidth?: number;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Cuts the layout resulting from the {@link LayoutStageBase#coreLayout core layout algorithm} at a specified width/height and shifts the remaining graph into
the next row/column until the layout has the desired width/height.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the space that should be kept between edges.
The spacing must be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is a negative value
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper%23edgeSpacing Online Documentation}*/
edgeSpacing: number;
/**
Gets or sets whether this {@link ILayoutStage} wraps lines/rows or columns.
If this mode is enabled, the layout is wrapped vertically and new columns are appended to the right of the first column.
Otherwise, the layout is wrapped horizontally and new rows are appended below the first row.
Also, the {@link GraphLayoutLineWrapper#fixedWidth} property will be interpreted as a fixed height if this mode is enabled.
@see GraphLayoutLineWrapper#fixedWidth
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper%23columnMode Online Documentation}*/
columnMode: boolean;
/**
Gets or sets the space between adjacent lines of the wrapped graph layout.
If the {@link GraphLayoutLineWrapper#columnMode column mode} is enabled, this spacing defines the distance between adjacent columns.
The spacing should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper%23spacing Online Documentation}*/
spacing: number;
/**
Gets or sets whether or not every other line is mirrored.
When breaking a line and placing the remaining line below, this remainder can be rotated by <code>180</code> degrees. Then, the edges
connecting both lines
<em>stay short</em>. In case the remainder is not rotated, the edges are routed between the two lines to connect to the front of the
non-mirrored line.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper%23mirror Online Documentation}*/
mirror: boolean;
/**
Gets or sets whether the algorithm should use the {@link GraphLayoutLineWrapper#fixedWidth fixed width} or {@link GraphLayoutLineWrapper#targetRatio target aspect ratio} to determine line breaks.
@see GraphLayoutLineWrapper#fixedWidth
@see GraphLayoutLineWrapper#targetRatio
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper%23fixedWidthLineBreaks Online Documentation}*/
fixedWidthLineBreaks: boolean;
/**
Gets or sets the desired target aspect ratio for the layout.
The aspect ratio needs to be greater than <code>0</code>.
<b>Warning:</b> The specified width is only considered if {@link GraphLayoutLineWrapper#fixedWidthLineBreaks} returns <code>false</code>.
@throws {Exception} <b>(ArgumentError)</b> if the target ratio is <code>0</code> or less
@see GraphLayoutLineWrapper#fixedWidthLineBreaks
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper%23targetRatio Online Documentation}*/
targetRatio: number;
/**
Gets or sets the desired width of the lines.
The {@link GraphLayoutLineWrapper} breaks the lines at the specified width.
In {@link GraphLayoutLineWrapper#columnMode column mode}, the fixed width is interpreted as fixed height for a column.
<h2>Note</h2>
The algorithm will not necessarily be able to satisfy very small values, since the nodes need to fit completely into a
line.
<b>Warning:</b> The specified width is only considered if {@link GraphLayoutLineWrapper#fixedWidthLineBreaks} returns <code>true</code>.
@see GraphLayoutLineWrapper#fixedWidthLineBreaks
@see GraphLayoutLineWrapper#columnMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphLayoutLineWrapper%23fixedWidth Online Documentation}*/
fixedWidth: number;
static $class: Class;
private $_GraphLayoutLineWrapper: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphLayoutLineWrapper;
}
export interface ColumnDescriptor extends YObject, IComparable {
}
/**
This class represents a column of a {@link PartitionGrid partition grid} structure.
The results of the calculation of the geometry of the column will be placed into the instances of this class after the
layout.
@see PartitionGrid
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor Online Documentation}*/
export class ColumnDescriptor {
/**
Compares this {@link ColumnDescriptor} instance with a given other {@link ColumnDescriptor} instance based
on the indices of the two instances.
@param o the {@link ColumnDescriptor} to compare to
@returns <code>-1</code>, <code>0</code> or <code>1</code> if this {@link ColumnDescriptor} is less than, equal to, or greater than the given other
{@link ColumnDescriptor}
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23compareTo(Object) Online Documentation}*/
compareTo(o: any | null): number;
/**
Gets the index of the column within the {@link PartitionGrid partition grid}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23index Online Documentation}*/
readonly index: number;
/**
Gets or sets the minimum width of the column.
The minimum width is defined to be at least <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum width is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23minimumWidth Online Documentation}*/
minimumWidth: number;
/**
Gets or sets the left column inset in which no element will lie in the resulting layout.
The column inset must be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the left inset is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23leftInset Online Documentation}*/
leftInset: number;
/**
Gets or sets the right column inset in which no element will lie in the resulting layout.
The column inset must be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the right inset is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23rightInset Online Documentation}*/
rightInset: number;
/**
Gets or sets the computed width of the column after the layout has been calculated.
The computed width needs to be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the computed width is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23computedWidth Online Documentation}*/
computedWidth: number;
/**
Gets or sets the original width of the column.
The original width should be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the original width is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23originalWidth Online Documentation}*/
originalWidth: number;
/**
Gets or sets the original position (smallest x-coordinate) of the column.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23originalPosition Online Documentation}*/
originalPosition: number;
/**
Gets or sets the computed position (smallest x-coordinate) of the column after the layout has been calculated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23computedPosition Online Documentation}*/
computedPosition: number;
/**
Gets or sets the tightness factor of this column.
This value must lie within the interval <code>[0,1]</code>.
The larger the value, the more the column will be forced to obtain its minimum width:
<ul>
<li>a value of <code>0.0d</code> will disable the compression of the column.
<li>a value of <code>1.0d</code> will try to force the column to obtain its specified {@link ColumnDescriptor#minimumWidth minimum width}.
</ul>
@throws {Exception} <b>(ArgumentError)</b> if the tightness value is outside the interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23tightness Online Documentation}*/
tightness: number;
/**
Gets or sets whether the index of the column is fixed or it should be chosen automatically in order to minimize edge
lengths.
For all columns where this property is enabled, the relative ordering given by the indices is preserved. The remaining
columns may be sorted again so that the overall edge lengths are minimized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ColumnDescriptor%23indexFixed Online Documentation}*/
indexFixed: boolean;
static $class: Class;
private $_ColumnDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ColumnDescriptor;
}
export interface GenericPartitionGridStage extends LayoutStageBase {
}
/**
This {@link ILayoutStage} offers generic support for {@link PartitionGrid partition grid structures}.
The stage performs the following steps:
<ol>
<li>First it determines sub-components such that each component only contains nodes associated with the same
{@link PartitionCellId grid cell}. Nodes without associated grid cell are temporarily assigned to a suitable cell.
<li>Now this stage applies the specified {@link LayoutStageBase#coreLayout core layout algorithm} to each sub-component and, afterwards, reassembles the global
partition grid structure. Since the sub-components do not contain nodes of different cells, the core layout algorithm <b>doesn't require support for partition grid on its own</b>.
<li>Finally, this stage routes edges that connect nodes of different sub-components by means of the specified
{@link GenericPartitionGridStage#interEdgeRouter inter-edge router}. Note that such edges are temporarily hidden during the core layout runs.
</ol>
Disabling property {@link GenericPartitionGridStage#splitMultiCellComponents} allows to switch to a mode where connected components are not split into smaller sub-components.
Hence, a component may contain nodes of different partition cells and, thus, the specified {@link LayoutStageBase#coreLayout core layout algorithm} has to
support partition grid structures by itself. In this case, this stage doesn't offer a generic partition grid support
(since the core layout algorithm already has to support such structures). Instead it may speed up the layout calculation
since the core layout algorithm is applied to different smaller graphs instead of a single, larger input graph.
@see PartitionGrid
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStage Online Documentation}*/
export class GenericPartitionGridStage {
/**
Creates a new instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] the core layout algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] the core layout algorithm
@param [options.considerLabels] Whether or not to take node and edge labels into account when calculating the bounding box of the partition cells. This option sets the {@link GenericPartitionGridStage#considerLabels} property on the created object.
@param [options.interEdgeRouter] The current edge routing algorithm for handling inter-edges. This option sets the {@link GenericPartitionGridStage#interEdgeRouter} property on the created object.
@param [options.interEdgesDpKey] The key for marking the inter-edges to be routed. This option sets the {@link GenericPartitionGridStage#interEdgesDpKey} property on the created object.
@param [options.splitMultiCellComponents] Whether or not the algorithm should split connected components that have nodes associated with different partition cells. This option sets the {@link GenericPartitionGridStage#splitMultiCellComponents} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
considerLabels?: boolean;
interEdgeRouter?: ILayoutAlgorithm | null;
interEdgesDpKey?: any | null;
splitMultiCellComponents?: boolean;
});
/**
Starts the layout.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for explicitly marking (some) edges that should not be considered when calculating the sub-components
This key is only considered if property {@link GenericPartitionGridStage#splitMultiCellComponents} is disabled. The marked edges, the so-called split-edges, are temporarily
removed during the calculation of the sub-components and, finally, routed with the specified
{@link GenericPartitionGridStage#interEdgeRouter inter-edge router}. This allows to control how the graph is split into smaller graph components. Recall that with this
setting a component may contain nodes of different partition cells.
@see GenericPartitionGridStage#interEdgeRouter
@see GenericPartitionGridStage#splitMultiCellComponents
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStage%23SPLIT_EDGES_DP_KEY Online Documentation}*/
static SPLIT_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets whether or not to take node and edge labels into account when calculating the bounding box of the partition
cells.
<h2>Note</h2>
The labels are only considered if option {@link GenericPartitionGridStage#splitMultiCellComponents} is enabled. Otherwise the proper calculation of the partition cell box is up
to the {@link LayoutStageBase#coreLayout core layout algorithm}.
@see GenericPartitionGridStage#splitMultiCellComponents
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStage%23considerLabels Online Documentation}*/
considerLabels: boolean;
/**
Gets or sets the current edge routing algorithm for handling inter-edges.
If option {@link GenericPartitionGridStage#splitMultiCellComponents} is enabled, components that contain nodes associated with different {@link PartitionCellId partition cells} are split into
sub-components. Edges that connect nodes of different sub-components are called inter-edges and routed with this edge
routing algorithm.
It is required that a suitable {@link GenericPartitionGridStage#interEdgesDpKey selection key} is specified. The same selection key must be used for setting the scope for
the edge routing algorithm.
<h2>Note</h2>
If no router is set, all inter-edges are routed as straight lines.
@see GenericPartitionGridStage#interEdgesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStage%23interEdgeRouter Online Documentation}*/
interEdgeRouter: ILayoutAlgorithm | null;
/**
Gets or sets the key for marking the inter-edges to be routed.
The key should be used by the specified {@link GenericPartitionGridStage#interEdgeRouter inter-edge routing algorithm} to obtain the edges that must be routed. This stage
automatically marks these edges and registers the {@link IDataProvider} using the specified key.
<h2>Note</h2>
If there is already a {@link IDataAcceptor} registered with this key, this stage uses it to mark the edges
instead of adding a new {@link IDataProvider}.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see GenericPartitionGridStage#interEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStage%23interEdgesDpKey Online Documentation}*/
interEdgesDpKey: any | null;
/**
Gets or sets whether or not the algorithm should split connected components that have nodes associated with different
partition cells.
If this option is enabled, such components are split into sub-components that only include nodes associated with the
same {@link PartitionCellId grid cell}. Hence, the {@link LayoutStageBase#coreLayout core layout algorithm} doesn't see the {@link PartitionGrid partition grid} at all. Edges that connect nodes of
different sub-components are temporarily hidden and routed afterwards by means of the {@link GenericPartitionGridStage#interEdgeRouter inter-edge router}.
If this option is disabled, the layout algorithm determines independent components and calls the {@link LayoutStageBase#coreLayout core layout algorithm}
for each of them. Note that two connected components are only independent if there is no group node that contains a node
of each of them and if the two sub-partition grids induced by the connected components do no have a common {@link ColumnDescriptor}
or
{@link RowDescriptor}. The core layout algorithm is called once for each independent component and, thus, has to
support partition grid structures.
<b>Warning:</b> If this option is disabled, the {@link LayoutStageBase#coreLayout core layout algorithm} has to support partition grid structures by itself.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStage%23splitMultiCellComponents Online Documentation}*/
splitMultiCellComponents: boolean;
static $class: Class;
private $_GenericPartitionGridStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GenericPartitionGridStage;
}
export interface PartitionCellIdEntry extends YObject, IComparable {
}
/**
This class represents a single partition cell, i.e., a pair consisting of a row and a column.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellIdEntry Online Documentation}*/
export class PartitionCellIdEntry {
/**
Compares this {@link PartitionCellIdEntry} instance with a given other {@link PartitionCellIdEntry}
instance.
The comparison is based on the indices of the {@link PartitionCellIdEntry#row rows} and {@link PartitionCellIdEntry#column columns} of the two pairs. A {@link PartitionCellIdEntry} is
considered to be smaller than another pair if its {@link RowDescriptor#index row index} is smaller than the row index of the other pair. If the row
indices are equal, the {@link PartitionCellIdEntry} with the smaller {@link ColumnDescriptor#index column index} is said to be smaller. Hence,
two {@link PartitionCellIdEntry} instances are equal, if the row and column indices are equal.
@param o the {@link PartitionCellIdEntry} to be compared
@returns <code>-1</code>, <code>0</code> or <code>1</code> if this {@link PartitionCellIdEntry} is smaller than, equal to, or greater than the given other {@link PartitionCellIdEntry}
instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellIdEntry%23compareTo(Object) Online Documentation}*/
compareTo(o: any | null): number;
/**
Compares this {@link PartitionCellIdEntry} instance with a given other {@link PartitionCellIdEntry} instance
based on their associated {@link ColumnDescriptor} and {@link RowDescriptor} instances.
@param o the given {@link PartitionCellIdEntry} instance
@returns <code>true</code> if the two objects are equal, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellIdEntry%23equals(Object) Online Documentation}*/
equals(o: any | null): boolean;
/**
Returns the hash code value for this {@link PartitionCellIdEntry} instance.
@returns the hash code value for this {@link PartitionCellIdEntry} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellIdEntry%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Gets the {@link RowDescriptor} associated with this {@link PartitionCellId partition cell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellIdEntry%23row Online Documentation}*/
readonly row: RowDescriptor | null;
/**
Gets the {@link ColumnDescriptor} associated with this {@link PartitionCellId partition cell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellIdEntry%23column Online Documentation}*/
readonly column: ColumnDescriptor | null;
static $class: Class;
private $_PartitionCellIdEntry: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionCellIdEntry;
}
export interface PartitionCellId extends YObject {
}
/**
{@link PartitionCellId} represents an identifier for partition cells of a {@link PartitionGrid partition grid} structure.
@see PartitionGrid
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellId Online Documentation}*/
export class PartitionCellId {
/**
Compares this {@link PartitionCellId} instance with a given other {@link PartitionCellId} instance.
@param o the given {@link PartitionCellId} instance
@returns <code>true</code> if the two objects are equal, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellId%23equals(Object) Online Documentation}*/
equals(o: any | null): boolean;
/**
Returns the hash code value for this {@link PartitionCellId} instance.
@returns the hash code value for this {@link PartitionCellId} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellId%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Gets a {@link ICollection collection} of elements of type {@link PartitionCellIdEntry}.
Each pair consists of a {@link ColumnDescriptor column} and a {@link RowDescriptor row} defining a single {@link PartitionCellId partition cell}.
<h2>Note</h2>
A {@link PartitionCellId partition cell} identifier that covers more than one partition cell can only be assigned to group nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellId%23cells Online Documentation}*/
readonly cells: IEnumerable<PartitionCellIdEntry> | null;
/**
Gets whether or not this {@link PartitionCellId} represents a multi-cell, i.e., a cell spanning multiple
rows/columns.
This means that it was created using {@link PartitionGrid#createCellSpanId},
{@link PartitionGrid#createCellSpanId}, {@link PartitionGrid#createCellSpanId}, {@link PartitionGrid#createColumnSpanId}
or {@link PartitionGrid#createRowSpanId}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellId%23isSpanning Online Documentation}*/
readonly isSpanning: boolean;
/**
Gets the first {@link RowDescriptor row} associated with this {@link PartitionCellId partition cell} identifier.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellId%23row Online Documentation}*/
readonly row: RowDescriptor | null;
/**
Gets the first {@link ColumnDescriptor column} associated with this {@link PartitionCellId partition cell} identifier.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellId%23column Online Documentation}*/
readonly column: ColumnDescriptor | null;
static $class: Class;
private $_PartitionCellId: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionCellId;
}
export interface PartitionGrid extends YObject {
}
/**
This class represents a partition grid that partitions the drawing area into rectangular partition cells.
Such a structure allows, for example, to model vertical/horizontal swimlanes which are often used in flow diagrams to
separate logical units.
The grid consists of rows and columns that are created using the corresponding constructors of this class or methods {@link PartitionGrid#addRow}
and {@link PartitionGrid#addColumn}. The {@link PartitionGrid} object has to be registered with the graph
using the {@link IDataProvider} key {@link PartitionGrid#PARTITION_GRID_DP_KEY}.
Each node can be assigned to a {@link PartitionCellId} which represents a set of partition cells. The mapping has
to be registered with the graph using {@link IDataProvider} key
{@link PartitionGrid#PARTITION_CELL_ID_DP_KEY}. Simple partition cell identifiers which represent a single
partition cell, i.e., a row/column pair, can be created using method {@link PartitionGrid#createCellId} or
{@link PartitionGrid#createCellId}.
Nodes can also be mapped to a partition cell identifier that represents a multi-cell, i.e., a cell spanning multiple
columns and rows. Please note, however, that multi-cells are not allowed to overlap each other. This means that the user
has to ensure that each partition cell (row/column pair) is associated with at most one partition cell identifier. The
multi-cell identifier can be created using methods {@link PartitionGrid#createCellSpanId},
{@link PartitionGrid#createCellSpanId}, {@link PartitionGrid#createCellSpanId},
{@link PartitionGrid#createColumnSpanId}, and {@link PartitionGrid#createRowSpanId}.
<b>Warning:</b> If at least one node is mapped to a multi-cell, enabling option {@link PartitionGrid#optimizeColumnOrder} or {@link PartitionGrid#optimizeRowOrder} has no effect.
<h2>Note</h2>
The partition cell identifier of a group node is not inherited by its non-group descendant. Hence, each non-group
descendant has to be manually assigned to a partition cell identifier. In addition, a group node mapped to a column/row
span partition cell identifier represents the associated column/row. Thus, its boundary corresponds to that of the
column/row and edges incident to such a group node are not considered during the layout. Since the boundary of a group
node always encompasses all of its children, don't assign a partition cell identifier to layout the group as normal.
<h2>Note</h2>
Partition grid structures are currently supported by the {@link HierarchicLayout}, {@link OrganicLayout}
and {@link EdgeRouter}. The {@link RecursiveGroupLayout} allows to define a local partition grid for each
recursive group node, so that the grid is supported if the responsible group layout algorithm is one of the
aforementioned. Furthermore, layout stage {@link GenericPartitionGridStage} offers generic partition grid
support.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid Online Documentation}*/
export class PartitionGrid {
/**
Creates an instance of {@link PartitionGrid} with the given number of rows and columns as well as with given
insets and minimum sizes.
@param [rowCount=0] the number of rows
@param [columnCount=0] the number of columns
@param [rowInsets=0] the top and bottom insets of the rows
@param [columnInsets=0] the left and right insets of the columns
@param [minRowSize=0] the minimum row height
@param [minColumnSize=0] the minimum column width
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid Online Documentation}*/
constructor(rowCount?: number, columnCount?: number, rowInsets?: number, columnInsets?: number, minRowSize?: number, minColumnSize?: number);
/**
Creates an instance of {@link PartitionGrid} with the given number of rows and columns as well as with given
insets and minimum sizes.
@param options The parameters to pass.
@param [options.rowCount=0] the number of rows
@param [options.columnCount=0] the number of columns
@param [options.rowInsets=0] the top and bottom insets of the rows
@param [options.columnInsets=0] the left and right insets of the columns
@param [options.minRowSize=0] the minimum row height
@param [options.minColumnSize=0] the minimum column width
@param [options.optimizeColumnOrder] Whether or not the order of the columns should be chosen automatically to minimize edge lengths. This option sets the {@link PartitionGrid#optimizeColumnOrder} property on the created object.
@param [options.optimizeRowOrder] Whether or not the order of the rows should be chosen automatically to minimize edge lengths. This option sets the {@link PartitionGrid#optimizeRowOrder} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid Online Documentation}*/
constructor(options: {
rowCount?: number;
columnCount?: number;
rowInsets?: number;
columnInsets?: number;
minRowSize?: number;
minColumnSize?: number;
optimizeColumnOrder?: boolean;
optimizeRowOrder?: boolean;
});
/**
Adds a new {@link ColumnDescriptor column} to the {@link PartitionGrid} instance.
The index of the column (as returned by {@link ColumnDescriptor#index}) is equal to the size of the list returned by {@link PartitionGrid#columns}.
@returns a new {@link ColumnDescriptor} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23addColumn() Online Documentation}*/
addColumn(): ColumnDescriptor | null;
/**
Adds a new {@link RowDescriptor row} to the {@link PartitionGrid} instance.
The index of the row (as returned by {@link RowDescriptor#index}) is equal to the size of the list returned by {@link PartitionGrid#rows}.
@returns a new {@link RowDescriptor} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23addRow() Online Documentation}*/
addRow(): RowDescriptor | null;
/**
Creates a {@link PartitionCellId partition cell} identifier that represents the cell defined by the given column and row index.
@param rowIndex the row index of the row defining the cell
@param columnIndex the column index of the column defining the cell
@returns a {@link PartitionCellId} instance
@throws {Exception} <b>(ArgumentError)</b> if the given column/row index is not valid
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23createCellId(number%2Cnumber) Online Documentation}*/
createCellId(rowIndex: number, columnIndex: number): PartitionCellId | null;
/**
Creates a {@link PartitionCellId partition cell} identifier that represents the cell defined by the given {@link ColumnDescriptor column} and {@link RowDescriptor row}.
@param rd the row defining the cell
@param cd the column defining the cell
@returns a {@link PartitionCellId} instance
@throws {Exception} <b>(ArgumentError)</b> if the given column/row is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23createCellId(RowDescriptor%2CColumnDescriptor) Online Documentation}*/
createCellId(rd: RowDescriptor | null, cd: ColumnDescriptor | null): PartitionCellId | null;
/**
Creates a {@link PartitionCellId partition cell} identifier that represents a multi-cell spanning multiple columns and rows.
The partition cell identifier represents all cells defined by a row between <code>fromRow</code> and <code>toRow</code> (both including) and a
column between <code>fromRow</code> and <code>toCol</code> (both including).
A group node mapped to such a partition cell identifier represents the multi-cell, i.e., its boundary corresponds to the
smallest rectangle containing all partition cells specified by the partition cell identifier. The boundary does not
depend on the partition cells associated with the group's descendants. Hence, each non-group descendant has to be
manually assigned to one of the partition cells. Furthermore, edges incident to such a group node are not considered
during the layout.
@param fromRow the topmost row defining the cell
@param fromCol the leftmost column defining the cell
@param toRow the bottommost row defining the cell
@param toCol the rightmost column defining the cell
@returns a {@link PartitionCellId} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23createCellSpanId(RowDescriptor%2CColumnDescriptor%2CRowDescriptor%2CColumnDescriptor) Online Documentation}*/
createCellSpanId(fromRow: RowDescriptor | null, fromCol: ColumnDescriptor | null, toRow: RowDescriptor | null, toCol: ColumnDescriptor | null): PartitionCellId | null;
/**
Creates a {@link PartitionCellId partition cell} identifier that represents a multi-cell spanning multiple columns and rows.
The partition cell identifier represents all cells defined by a row with index between <code>fromRowIndex</code> and <code>toRowIndex</code> (both
including) and a column with index between <code>fromColIndex</code> and <code>toColIndex</code> (both including).
A group node mapped to such a partition cell identifier represents the multi-cell, i.e., its boundary corresponds to the
smallest rectangle containing all partition cells specified by the partition cell identifier. The boundary does not
depend on the partition cells associated with the group's descendants. Hence, each non-group descendant has to be
manually assigned to one of the partition cells. Furthermore, edges incident to such a group node are not considered
during the layout.
@param fromRowIndex the row index of the topmost row defining the cell
@param fromColIndex the column index of the leftmost column defining the cell
@param toRowIndex the row index of the bottommost row defining the cell
@param toColIndex the column index of the rightmost column defining the cell
@returns a {@link PartitionCellId} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23createCellSpanId(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
createCellSpanId(fromRowIndex: number, fromColIndex: number, toRowIndex: number, toColIndex: number): PartitionCellId | null;
/**
Creates a {@link PartitionCellId partition cell} identifier that represents a multi-cell spanning multiple columns and rows.
The partition cell identifier represents all cells spanned by the columns of <code>columnList</code> and rows of <code>rowList</code>.
A group node mapped to such a partition cell identifier represents the multi-cell, i.e., its boundary corresponds to the
smallest rectangle containing all partition cells specified by the partition cell identifier. The boundary does not
depend on the partition cells associated with the group's descendants. Hence, each non-group descendant has to be
manually assigned to one of the partition cells. Furthermore, edges incident to such a group node are not considered
during the layout.
@param rowList the {@link RowDescriptor rows} defining the cell
@param columnList the {@link ColumnDescriptor columns} defining the cell
@returns a {@link PartitionCellId} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23createCellSpanId(ICollection%3CObject%3E%2CICollection%3CObject%3E) Online Documentation}*/
createCellSpanId(rowList: ICollection<any> | null | ICollectionConvertible<any>, columnList: ICollection<any> | null | ICollectionConvertible<any>): PartitionCellId | null;
/**
Creates a {@link PartitionCellId partition cell} identifier that represents a cell spanning a whole column.
A group node mapped to such a partition cell identifier represents the column, i.e., its enclosing boundary corresponds
to that of the column. The boundary does not depend on the partition cells associated with the group's descendants.
Hence, each non-group descendant has to be manually assigned to one of the partition cells. Furthermore, edges incident
to such a group node are not considered during the layout.
@param columnIndex the column index of the column defining the cell
@returns a {@link PartitionCellId} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23createColumnSpanId(number) Online Documentation}*/
createColumnSpanId(columnIndex: number): PartitionCellId | null;
/**
Creates a {@link PartitionCellId partition cell} identifier that represents a cell spanning a whole row.
A group node mapped to such a partition cell identifier represents the row, i.e., its enclosing boundary corresponds to
that of the row. The boundary does not depend on the partition cells associated with the group's descendants. Hence,
each non-group descendant has to be manually assigned to one of the partition cells. Furthermore, edges incident to such
a group node are not considered during the layout.
@param rowIndex the row index of the row defining the cell
@returns a {@link PartitionCellId} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23createRowSpanId(number) Online Documentation}*/
createRowSpanId(rowIndex: number): PartitionCellId | null;
/**
Finalizes the orientation changes and is called at the end of the {@link OrientationLayout} stage.
Points are transferred to their final positions through {@link OrientationLayout#completeTransform}.
@param orientationLayouter an {@link OrientationLayout} instance
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23finalizeOrientationChange(OrientationLayout) Online Documentation}*/
finalizeOrientationChange(orientationLayouter: OrientationLayout | null): void;
/**
Returns the {@link ColumnDescriptor column} with the given index.
@param index the index of the column to be returned
@returns the {@link ColumnDescriptor column} with the given index or <code>null</code> if no such column exists
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23getColumn(number) Online Documentation}*/
getColumn(index: number): ColumnDescriptor | null;
/**
Returns the {@link PartitionGrid} instance associated with the given graph.
@param graph the given graph
@returns the {@link PartitionGrid} of the graph or <code>null</code> if the graph has no associated partition grid
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23getPartitionGrid(Graph) Online Documentation}*/
static getPartitionGrid(graph: Graph | null): PartitionGrid | null;
/**
Returns the {@link RowDescriptor row} with the given index.
@param index the index of the row to be returned
@returns the {@link RowDescriptor row} with the given index or <code>null</code> if no such row exists
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23getRow(number) Online Documentation}*/
getRow(index: number): RowDescriptor | null;
/**
Prepares for the orientation changes and is called at the beginning of the {@link OrientationLayout} stage.
Original points are transferred to their temporary positions through {@link OrientationLayout#prepareTransform}.
@param orientationLayouter an {@link OrientationLayout} instance
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23prepareOrientationChange(OrientationLayout) Online Documentation}*/
prepareOrientationChange(orientationLayouter: OrientationLayout | null): void;
/**
Data provider key for storing the partition grid structure
If the {@link IDataProvider} contains a partition grid structure, this structure will be considered during
layout (provided that the layout algorithm supports such constraints).
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23PARTITION_GRID_DP_KEY Online Documentation}*/
static PARTITION_GRID_DP_KEY: GraphDpKey<PartitionGrid> | null;
/**
Data provider key for storing a set of partition cells for each node
Each node is placed inside the columns/rows defined by the corresponding {@link PartitionCellId} identifier.
Instances can be shared among multiple nodes, but don't have to be shared.
<h2>Note</h2>
Multi-cell identifiers (i.e., an identifier that represents multiple columns/rows) are not allowed to overlap each
other.
@see PartitionCellId
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23PARTITION_CELL_ID_DP_KEY Online Documentation}*/
static PARTITION_CELL_ID_DP_KEY: NodeDpKey<PartitionCellId> | null;
/**
Gets or sets whether or not the order of the columns should be chosen automatically to minimize edge lengths.
For all columns where {@link ColumnDescriptor#indexFixed} is set to
<code>true</code>, the relative ordering given by the indices is preserved. The remaining columns may be sorted again so that the
overall edge lengths are minimized.
<b>Warning:</b>
If at least one node is mapped to a multi-cell (i.e., a cell spanning multiple columns/rows), enabling this option has
no effect.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23optimizeColumnOrder Online Documentation}*/
optimizeColumnOrder: boolean;
/**
Gets or sets whether or not the order of the rows should be chosen automatically to minimize edge lengths.
For all rows where {@link RowDescriptor#indexFixed} is set to
<code>true</code>, the relative ordering given by the indices is preserved. The remaining rows may be sorted again so that the
overall edge lengths are minimized.
<b>Warning:</b>
If at least one node is mapped to a multi-cell (i.e., a cell spanning multiple columns/rows), enabling this option has
no effect.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23optimizeRowOrder Online Documentation}*/
optimizeRowOrder: boolean;
/**
Gets the rows of the {@link PartitionGrid partition grid}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23rows Online Documentation}*/
readonly rows: IEnumerable<RowDescriptor> | null;
/**
Gets the columns of the {@link PartitionGrid partition grid}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGrid%23columns Online Documentation}*/
readonly columns: IEnumerable<ColumnDescriptor> | null;
static $class: Class;
private $_PartitionGrid: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionGrid;
}
export interface RowDescriptor extends YObject, IComparable {
}
/**
This class represents a row of a {@link PartitionGrid partition grid} structure.
The results of the calculation of the geometry of the row will be placed into the instances of this class after the
layout.
@see PartitionGrid
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor Online Documentation}*/
export class RowDescriptor {
/**
Compares this {@link RowDescriptor} instance with a given other {@link RowDescriptor} instance based on the
indices of the two instances.
@param o the {@link RowDescriptor} to compare to
@returns <code>-1</code>, <code>0</code> or <code>1</code> if this {@link RowDescriptor} is less than, equal to, or greater than the given other
{@link RowDescriptor}
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23compareTo(Object) Online Documentation}*/
compareTo(o: any | null): number;
/**
Gets the index of the row within the {@link PartitionGrid partition grid}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23index Online Documentation}*/
readonly index: number;
/**
Gets or sets the minimum height of the row.
The minimum height is defined to be at least <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum height is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23minimumHeight Online Documentation}*/
minimumHeight: number;
/**
Gets or sets the top inset in which no element will lie in the resulting layout.
The inset value needs to be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the given top inset is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23topInset Online Documentation}*/
topInset: number;
/**
Gets or sets the bottom inset in which no element will lie in the resulting layout.
The inset value needs to be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the given bottom inset is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23bottomInset Online Documentation}*/
bottomInset: number;
/**
Gets or sets the computed height of the row after the layout has been calculated.
The computed height needs to be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the computed height is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23computedHeight Online Documentation}*/
computedHeight: number;
/**
Gets or sets the original position (smallest y-coordinate) of the row.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23originalPosition Online Documentation}*/
originalPosition: number;
/**
Gets or sets the original height of the row.
The original height needs to be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the original height is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23originalHeight Online Documentation}*/
originalHeight: number;
/**
Gets or sets the computed position (smallest y-coordinate) of the row after the layout has been calculated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23computedPosition Online Documentation}*/
computedPosition: number;
/**
Gets or sets the tightness factor of this row.
This value must lie within the interval <code>[0,1]</code>.
The larger the value, the more the row will be forced to obtain its minimum height:
<ul>
<li>a value of <code>0.0d</code> will disable the compression of the row.
<li>a value of <code>1.0d</code> will try to force the row to obtain its specified {@link RowDescriptor#minimumHeight minimum height}.
</ul>
@throws {Exception} <b>(ArgumentError)</b> if the tightness value is outside the interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23tightness Online Documentation}*/
tightness: number;
/**
Gets or sets whether the index of the row is fixed or it should be chosen automatically in order to minimize edge
lengths.
For all rows where this property is enabled, the relative ordering given by the indices is preserved. The remaining rows
may be sorted again so that the overall edge lengths are minimized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RowDescriptor%23indexFixed Online Documentation}*/
indexFixed: boolean;
static $class: Class;
private $_RowDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RowDescriptor;
}
/**
@see FixGroupLayoutStage#interEdgeRoutingStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterEdgeRoutingStyle Online Documentation}*/
export enum InterEdgeRoutingStyle {
/**
An inter-edge routing style based on which the edge layout of inter-edges will be adopted from the
{@link LayoutStageBase#coreLayout core layout algorithm}.
@see FixGroupLayoutStage#interEdgeRoutingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterEdgeRoutingStyle%23ADOPT Online Documentation}*/
ADOPT,
/**
An inter-edge routing style based on which inter-edges are routed in an orthogonal style.
@see FixGroupLayoutStage#interEdgeRoutingStyle
@see FixGroupLayoutStage#interEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterEdgeRoutingStyle%23ORTHOGONAL Online Documentation}*/
ORTHOGONAL,
/**
An inter-edge routing style based on which inter-edges are routed as straight lines.
@see FixGroupLayoutStage#interEdgeRoutingStyle
@see FixGroupLayoutStage#interEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterEdgeRoutingStyle%23STRAIGHTLINE Online Documentation}*/
STRAIGHTLINE
}
/** Automatic conversion to {@link InterEdgeRoutingStyle}. */
type InterEdgeRoutingStyleStringValues = "adopt" | "orthogonal" | "straightline";
export interface HideGroupsStage extends LayoutStageBase {
}
/**
A {@link ILayoutStage layout stage} that hides the group nodes of hierarchically grouped graphs.
This stage removes all group nodes and adjacent edges from the graph before passing it to the
{@link LayoutStageBase#coreLayout core layout algorithm}. After the core layout algorithm has arranged the graph, the hidden elements will be reinserted.
This stage is used by layout algorithms that cannot handle grouped graphs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HideGroupsStage Online Documentation}*/
export class HideGroupsStage {
/**
Creates an instance of {@link HideGroupsStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HideGroupsStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates an instance of {@link HideGroupsStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.resetEdgePaths] Whether or not this stage resets the paths of edges incident to group nodes. This option sets the {@link HideGroupsStage#resetEdgePaths} property on the created object.
@param [options.hidingEmptyGroupNodes] Whether or not empty group nodes, i.e., group nodes without children, will be hidden by this stage. This option sets the {@link HideGroupsStage#hidingEmptyGroupNodes} property on the created object.
@param [options.groupBoundsCalculator] The {@link ILayoutGroupBoundsCalculator} instance for calculating the sizes of group nodes. This option sets the {@link HideGroupsStage#groupBoundsCalculator} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HideGroupsStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
resetEdgePaths?: boolean;
hidingEmptyGroupNodes?: boolean;
groupBoundsCalculator?: ILayoutGroupBoundsCalculator | null;
});
/**
Calculates the layout based on the given core layout algorithm.
Before calling the core layout algorithm, group nodes and their adjacent edges will be hidden. After the layout has been
calculated, {@link HideGroupsStage} will restore the graph.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/HideGroupsStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Hides the group nodes and their incident edges of the input graph.
Furthermore, this method removes all grouping-related {@link IDataProvider}s from the input graph.
@param graph the input graph
@see GroupingKeys#NODE_ID_DP_KEY
@see GroupingKeys#PARENT_NODE_ID_DP_KEY
@see GroupingKeys#GROUP_DP_KEY
@see HideGroupsStage#unhideGroupNodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/HideGroupsStage%23hideGroupNodes(LayoutGraph) Online Documentation}*/
hideGroupNodes(graph: LayoutGraph | null): void;
/**
Un-hides the group nodes and their incident edges of the input graph that were previously hidden using method
{@link HideGroupsStage#hideGroupNodes}.
Furthermore, this method restores all previously removed grouping related
{@link IDataProvider}s on the input graph.
@param graph the given graph
@see GroupingKeys#NODE_ID_DP_KEY
@see GroupingKeys#PARENT_NODE_ID_DP_KEY
@see GroupingKeys#GROUP_DP_KEY
@see HideGroupsStage#hideGroupNodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/HideGroupsStage%23unhideGroupNodes(LayoutGraph) Online Documentation}*/
unhideGroupNodes(graph: LayoutGraph | null): void;
/**
Gets or sets whether or not this stage resets the paths of edges incident to group nodes.
Resetting an edge path removes all bends and moves the edge's port on a group to the center.
<h2>Note</h2>
Since this stage may change the size of group nodes (see
{@link HideGroupsStage#groupBoundsCalculator}), keeping the original edge paths often produces weird routes for edges at group nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HideGroupsStage%23resetEdgePaths Online Documentation}*/
resetEdgePaths: boolean;
/**
Gets or sets whether or not empty group nodes, i.e., group nodes without children, will be hidden by this stage.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HideGroupsStage%23hidingEmptyGroupNodes Online Documentation}*/
hidingEmptyGroupNodes: boolean;
/**
Gets or sets the {@link ILayoutGroupBoundsCalculator} instance for calculating the sizes of group nodes.
For every group which is in its open state, the {@link ILayoutGroupBoundsCalculator} will be asked to calculate the
bounds for the group's child nodes. The resulting size will be used during the ongoing layout calculation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HideGroupsStage%23groupBoundsCalculator Online Documentation}*/
groupBoundsCalculator: ILayoutGroupBoundsCalculator | null;
static $class: Class;
private $_HideGroupsStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HideGroupsStage;
}
export interface ILayoutGroupBoundsCalculator {
}
/**
A {@link ILayoutGroupBoundsCalculator} is responsible for calculating the bounds of a group node during a layout
process.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutGroupBoundsCalculator Online Documentation}*/
export abstract class ILayoutGroupBoundsCalculator {
/**
Calculates the bounds of the given group node that contains the given list of child nodes.
This method is called during the layout process.
@param graph the input graph
@param groupNode the group node whose bounds will be calculated
@param children a {@link YNodeList} containing the nodes that reside inside the group node
@returns a {@link Rectangle2D} instance that describes the bounds of the group node
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutGroupBoundsCalculator%23calculateBounds(LayoutGraph%2CYNode%2CYNodeList) Online Documentation}*/
abstract calculateBounds(graph: LayoutGraph | null, groupNode: YNode | null, children: YNodeList | null): Rectangle2D | null;
/**
Creates an implementation of the interface {@link ILayoutGroupBoundsCalculator} by using the given function as
implementation for its {@link ILayoutGroupBoundsCalculator#calculateBounds} method.
@param calculateBounds A function for {@link ILayoutGroupBoundsCalculator}'s single abstract method
{@link ILayoutGroupBoundsCalculator#calculateBounds}.
@returns An instance of the {@link ILayoutGroupBoundsCalculator} interface based on the given function.*/
static create(calculateBounds: (graph: LayoutGraph | null, groupNode: YNode | null, children: YNodeList | null) => Rectangle2D): ILayoutGroupBoundsCalculator;
static $class: Class;
private $_IGroupBoundsCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayoutGroupBoundsCalculator;
}
export interface InsetsGroupBoundsCalculator extends YObject, ILayoutGroupBoundsCalculator {
}
/**
This {@link ILayoutGroupBoundsCalculator} calculates the bounds of group nodes and allows to specify custom insets for
each group node.
{@link YInsets Insets} are provided by a {@link IDataProvider} that must have been registered with the graph using the
specified {@link InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key} prior to the invocation of the layout algorithm.
<b>Warning:</b> This implementation is intended to be used with class {@link RecursiveGroupLayout} and assumes that the graph
given to method {@link InsetsGroupBoundsCalculator#calculateBounds} only contains the content of the specified
group (which is always true if it is called within the {@link RecursiveGroupLayout}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator Online Documentation}*/
export class InsetsGroupBoundsCalculator {
/**
Constructs an instance of {@link InsetsGroupBoundsCalculator} using the given {@link IDataProvider} key
as {@link InsetsGroupBoundsCalculator#groupNodeInsetsDpKey group node insets key} and the given insets as {@link InsetsGroupBoundsCalculator#defaultInsets default insets}.
@param [groupNodeInsetsDPKey=null] the {@link IDataProvider} key to use as {@link InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key}
@param [defaultInsets=null] the insets to use as {@link InsetsGroupBoundsCalculator#defaultInsets default insets}
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator Online Documentation}*/
constructor(groupNodeInsetsDPKey?: any | null, defaultInsets?: YInsets | null);
/**
Constructs an instance of {@link InsetsGroupBoundsCalculator} using the given {@link IDataProvider} key
as {@link InsetsGroupBoundsCalculator#groupNodeInsetsDpKey group node insets key} and the given insets as {@link InsetsGroupBoundsCalculator#defaultInsets default insets}.
@param options The parameters to pass.
@param [options.groupNodeInsetsDPKey=null] the {@link IDataProvider} key to use as {@link InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key}
@param [options.defaultInsets=null] the insets to use as {@link InsetsGroupBoundsCalculator#defaultInsets default insets}
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator Online Documentation}*/
constructor(options: {
groupNodeInsetsDPKey?: any | null;
defaultInsets?: YInsets | null;
});
/**
Calculates the bounds of the given group node and enlarges the bounds by the insets associated with the group node.
The {@link YInsets insets} associated with the group node will be retrieved from a {@link IDataProvider} registered with the
graph with the current {@link InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key}.
<b>Warning:</b> This implementation is intended to be used with class {@link RecursiveGroupLayout} and assumes that the graph
given to method {@link InsetsGroupBoundsCalculator#calculateBounds} only contains the content of the specified
group (which is always true if it is called within the {@link RecursiveGroupLayout}).
@param graph the input graph
@param groupNode the group node whose bounds will be calculated
@param children a {@link YNodeList} containing the nodes that reside inside the group node
@returns a {@link Rectangle2D} instance that describes the bounds of the group node
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator%23calculateBounds(LayoutGraph%2CYNode%2CYNodeList) Online Documentation}*/
calculateBounds(graph: LayoutGraph | null, groupNode: YNode | null, children: YNodeList | null): Rectangle2D | null;
/**
Enlarges the given graph bounds by the insets associated with the given group node.
The {@link YInsets insets} associated with the group node will be retrieved from a {@link IDataProvider} registered with the
graph with the current {@link InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key}.
This method is called while {@link InsetsGroupBoundsCalculator#calculateBounds calculating the group bounds} to realize the support for custom insets. It may be overridden
to change the way specific insets influence the group node bounds.
@param graph the input graph
@param groupNode the group node whose bounds should be enlarged
@param children the list of nodes that the given group node contains
@param bounds a {@link Rectangle2D} instance representing the bounds of the given group node
@returns a {@link Rectangle2D} instance representing the enlarged bounds of the given group node
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator%23createEnlargedBounds(LayoutGraph%2CYNode%2CYNodeList%2CRectangle2D) Online Documentation}*/
createEnlargedBounds(graph: LayoutGraph | null, groupNode: YNode | null, children: YNodeList | null, bounds: Rectangle2D | null): Rectangle2D | null;
/**
Gets or sets whether or not the labels of nodes contained in a group are considered while calculating the bounds of the
group node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets whether or not the labels of edges contained in a group are considered while calculating the bounds of the
group node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator%23considerEdgeLabels Online Documentation}*/
considerEdgeLabels: boolean;
/**
Gets or sets whether or not the calculation of the bounds of a group node considers the {@link NodeHalo NodeHalos} associated with the
group's child nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator%23considerNodeHalos Online Documentation}*/
considerNodeHalos: boolean;
/**
Gets or sets the default insets that will be used if the {@link IDataProvider} registered with the {@link InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key}
does not contain a mapping for a specific node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator%23defaultInsets Online Documentation}*/
defaultInsets: YInsets | null;
/**
Gets or sets the key to register a {@link IDataProvider} that is used to access the insets for the group
nodes.
The {@link IDataProvider} registered with this key must provide a mapping from each group node to a {@link YInsets}
(or {@link YInsets}) instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InsetsGroupBoundsCalculator%23groupNodeInsetsDpKey Online Documentation}*/
groupNodeInsetsDpKey: any | null;
static $class: Class;
private $_InsetsGroupBoundsCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is InsetsGroupBoundsCalculator;
}
export interface IsolatedGroupComponentLayout extends ComponentLayout {
}
/**
A special {@link ComponentLayout} implementation that allows to lay out isolated subgraphs within a group as
separate components.
{@link OrthogonalLayout OrthogonalLayout} uses this layout stage to arrange components.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IsolatedGroupComponentLayout Online Documentation}*/
export class IsolatedGroupComponentLayout {
/**
Creates an instance of {@link IsolatedGroupComponentLayout} with the given layout algorithm as
{@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] the core layout algorithm that will be invoked for each component of the graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/IsolatedGroupComponentLayout Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates an instance of {@link IsolatedGroupComponentLayout} with the given layout algorithm as
{@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] the core layout algorithm that will be invoked for each component of the graph
@param [options.preferredSize] The preferred size of the layout. This option sets the {@link ComponentLayout#preferredSize} property on the created object.
@param [options.considerLabels] Whether or not to take node and edge labels into account when calculating the bounding box of the graph components. This option sets the {@link ComponentLayout#considerLabels} property on the created object.
@param [options.componentArrangement] Whether or not the separately arranged components of the input graph should be arranged relative to each other. This option sets the {@link ComponentLayout#componentArrangement} property on the created object.
@param [options.gridSpacing] The current grid spacing. This option sets the {@link ComponentLayout#gridSpacing} property on the created object.
@param [options.componentSpacing] The distance between the bounding boxes of the components. This option sets the {@link ComponentLayout#componentSpacing} property on the created object.
@param [options.considerGrouping] Whether or not grouping information bound to the graph should be considered when determining the graph components. This option sets the {@link ComponentLayout#considerGrouping} property on the created object.
@param [options.style] How the components are arranged. This option sets the {@link ComponentLayout#style} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IsolatedGroupComponentLayout Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
preferredSize?: YDimension | null;
considerLabels?: boolean;
componentArrangement?: boolean;
gridSpacing?: number;
componentSpacing?: number;
considerGrouping?: boolean;
style?: ComponentArrangementStyles | ComponentArrangementStylesConvertible;
});
/**
Calculates the layout based on the current {@link LayoutStageBase#coreLayout core layout algorithm}.
Before calling the core layout algorithm, the {@link IsolatedGroupComponentLayout} identifies the isolated
subgraphs within a group and treats them as separate components.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/IsolatedGroupComponentLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Determines the nodes that belong to the same graph component.
Unlike
{@link ComponentLayout}, this implementation additionally identifies isolated subgraphs within a group as separate
components.
@param graph the input graph
@param compNumber the {@link INodeMap} that will be filled by the layout algorithm, such that it contains the zero-based index
of the component number to which each node belongs
@returns the number of separate components
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/IsolatedGroupComponentLayout%23findIsolatedGraphComponents(LayoutGraph%2CINodeMap) Online Documentation}*/
findIsolatedGraphComponents(graph: LayoutGraph | null, compNumber: INodeMap | null): number;
static $class: Class;
private $_IsolatedGroupComponentLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IsolatedGroupComponentLayout;
}
export interface MinimumSizeGroupBoundsCalculator extends InsetsGroupBoundsCalculator {
}
/**
This {@link ILayoutGroupBoundsCalculator} allows custom insets for group nodes and guarantees that their provided
minimum sizes will be maintained.
As a subclass of {@link InsetsGroupBoundsCalculator}, this class supports custom insets for each group node.
Furthermore, it guarantees that a desired minimum size is maintained. The minimum sizes are provided by a {@link IDataProvider}
registered with the graph using the specified {@link MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey minimum node size key}.
<b>Warning:</b> This implementation is intended to be used with class {@link RecursiveGroupLayout} and assumes that the graph
given to method {@link MinimumSizeGroupBoundsCalculator#calculateBounds} only contains the content of the
specified group (which is always true if it is called within the {@link RecursiveGroupLayout}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumSizeGroupBoundsCalculator Online Documentation}*/
export class MinimumSizeGroupBoundsCalculator {
/**
Creates a new instance of {@link MinimumSizeGroupBoundsCalculator} with the given key as {@link MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey minimum node size key}.
@param minSizeDataProviderKey the {@link IDataProvider} key to use as {@link MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey minimum size key}
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumSizeGroupBoundsCalculator%23MinimumSizeGroupBoundsCalculator(Object) Online Documentation}*/
constructor(minSizeDataProviderKey: any | null);
/**
Creates a new instance of {@link MinimumSizeGroupBoundsCalculator} with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumSizeGroupBoundsCalculator%23MinimumSizeGroupBoundsCalculator() Online Documentation}*/
constructor();
/**
Calculates the bounds of the given group node, enlarges the bounds by the insets associated with the group node and
makes sure that the size of the group node obeys the associated minimum size.
The {@link YInsets insets} associated with the group node will be retrieved from a {@link IDataProvider} registered with the
graph with the current {@link InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key}.
The minimum sizes associated with the group node will be retrieved from a {@link IDataProvider} registered
with the graph with the current {@link MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey minimum size key}.
<b>Warning:</b> This implementation is intended to be used with class {@link RecursiveGroupLayout} and assumes that the graph
given to method {@link MinimumSizeGroupBoundsCalculator#calculateBounds} only contains the content of the
specified group (which is always true if it is called within the {@link RecursiveGroupLayout}).
@param graph the input graph
@param groupNode the group node whose bounds will be calculated
@param children a {@link YNodeList} containing the nodes that reside inside the group node
@returns a {@link Rectangle2D} instance that describes the bounds of the group node
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumSizeGroupBoundsCalculator%23calculateBounds(LayoutGraph%2CYNode%2CYNodeList) Online Documentation}*/
calculateBounds(graph: LayoutGraph | null, groupNode: YNode | null, children: YNodeList | null): Rectangle2D | null;
/**
Gets or sets how the nodes are resized horizontally if the calculated width is smaller than the preferred minimum width
of a group node.
The alignment is defined to be a value between <code>0</code> and <code>1</code>, such that:
<ul>
<li>A value of <code>0</code> means that the nodes grow to the right while the left border remains unchanged.
<li>A value of <code>1</code> means that the nodes grow to the left while the right border remains unchanged.
</ul>
@throws {Exception} <b>(ArgumentError)</b> if the given value does not lie within <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumSizeGroupBoundsCalculator%23xAlignment Online Documentation}*/
xAlignment: number;
/**
Gets or sets how the nodes are resized vertically if the calculated height is smaller than the preferred minimum height
of a group node.
The alignment is defined to be a value between <code>0</code> and <code>1</code>, such that:
<ul>
<li>A value of <code>0</code> means that the nodes grow downwards while the top border remains unchanged.
<li>A value of <code>1</code> means that the nodes grow upwards while the bottom border remains unchanged.
</ul>
@throws {Exception} <b>(ArgumentError)</b> if the given value does not lie within <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumSizeGroupBoundsCalculator%23yAlignment Online Documentation}*/
yAlignment: number;
/**
Gets or sets the key to register a {@link IDataProvider} that is used to access the minimum sizes for the
group nodes.
The {@link IDataProvider} registered with this key must provide a mapping from each group node to a {@link YDimension}
object defining the minimum width and height of the group node.
@see MinimumSizeGroupBoundsCalculator#defaultMinimumNodeSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumSizeGroupBoundsCalculator%23minimumNodeSizeDpKey Online Documentation}*/
minimumNodeSizeDpKey: any | null;
/**
Gets or sets the default minimum size (width and height) for group nodes.
The default minimum size is used if the {@link IDataProvider} registered with the {@link MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey minimum size key} does not
contain a mapping for a group node.
@throws {Exception} <b>(ArgumentError)</b> if the given {@link YDimension} is <code>null</code>
@see MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumSizeGroupBoundsCalculator%23defaultMinimumNodeSize Online Documentation}*/
defaultMinimumNodeSize: YDimension | null;
static $class: Class;
private $_MinimumSizeGroupBoundsCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MinimumSizeGroupBoundsCalculator;
}
export interface RecursiveGroupLayout extends LayoutStageBase {
}
/**
This layout algorithm recursively traverses a hierarchically organized graph in a bottom-up fashion and applies a
specified layout algorithm to the contents (direct children) of each group node.
<b>Layout Style</b>
The way a graph is arranged depends on the {@link ILayoutAlgorithm layout algorithms} which are applied to the different group nodes. {@link RecursiveGroupLayout}
is able to produce different layout styles for the content of each group node.
This layout algorithm can be either applied if a {@link ILayoutAlgorithm layout algorithm} cannot handle grouped graphs by itself or if the
content of (some) group nodes should be arranged differently.
{@link RecursiveGroupLayout} uses a hierarchy tree representation of the grouped graph in which the content nodes
are the children of their containing group node. That way, it can traverse the tree recursively while arranging only the
direct children of each group node. The layout algorithm starts by arranging the leaves in the hierarchy tree, then
works its way up to the root computing the layout for each group node in the tree.
<b>A) Group Handling</b> All nodes other than the direct children are temporarily hidden. The layout algorithm performs two steps for each group
node.
<ol>
<li>It arranges the direct children using either the {@link LayoutStageBase#coreLayout core layout algorithm} or a special layout algorithm retrieved from a {@link IDataProvider}
registered with
{@link RecursiveGroupLayout#GROUP_NODE_LAYOUT_DP_KEY}. The content of group nodes among the children is already
arranged at this time and will be ignored. These group nodes are handled like normal nodes with a size that encloses the
content.
The following two special cases should be considered. If <code>null</code> is specified as layout algorithm for a group, the
recursion will be disabled and the group is handled non-recursively: the group itself and its content is arranged by the
algorithm specified for the nearest ancestor which has a non-<code>null</code> algorithm mapped to it. Importantly, this must be
distinguished from the second case where the layout of the group content should remain unchanged. This is achieved when
the group is associated with {@link RecursiveGroupLayout#NULL_LAYOUT} as responsible layout algorithm (or any
another algorithm that does not change the layout).
<li>Then {@link RecursiveGroupLayout} computes the final size of the group node using an implementation of
{@link ILayoutGroupBoundsCalculator}. Customized {@link ILayoutGroupBoundsCalculator}s can be specified using
{@link RecursiveGroupLayout#groupBoundsCalculator}. Aside from the resulting layout, this size is used in the following iteration.
</ol>
<b>B) Top-Level Hierarchy</b>
After a layout is applied to all group nodes, the layout algorithm uses the {@link LayoutStageBase#coreLayout core layout algorithm} to arrange the top
level hierarchy. Note that {@link RecursiveGroupLayout} can run without a core layout. In this case no layout is
calculated for the top level hierarchy. Still, for group nodes with a non-<code>null</code> algorithm, the bounds are adjusted to fit
their respective contents.
<b>C) Inter-Edge Routing</b>
Finally, routes for the edges whose source node is located at a different hierarchy level than its target node are
computed. The edge routing algorithm for these so-called inter-edges can be {@link RecursiveGroupLayout#interEdgeRouter customized}.
<b>Features</b>
There are two alternatives for applying different layout styles to the contents of group nodes:
<ol>
<li>Mapping each group node to a corresponding {@link ILayoutAlgorithm layout algorithm} by registering a {@link IDataProvider} with key
{@link RecursiveGroupLayout#GROUP_NODE_LAYOUT_DP_KEY}. The content of the hierarchy root is arranged with the
{@link LayoutStageBase#coreLayout core layout algorithm}.
<li>Using {@link LayoutMultiplexer} as {@link LayoutStageBase#coreLayout core layout algorithm}.
</ol>
Since {@link RecursiveGroupLayout} delegates the actual arrangement of the graph to other layout algorithms, it
will support the same features as the currently used layout algorithm.
The improvement of the routing of inter-edges is based on the {@link RecursiveGroupLayout#autoAssignPortCandidates insertion} of {@link PortCandidate}s or the {@link RecursiveGroupLayout#replacePortConstraints conversion}
of {@link PortConstraint}s into {@link PortCandidate}s. Hence, they only work well if the applied layout
algorithm supports {@link PortCandidate}s.
This algorithm also provides a {@link RecursiveGroupLayout#fromSketchMode From Sketch mode} that should be activated if the applied layout algorithm runs in From
Sketch mode, too. Otherwise, the initial coordinates may not be considered correctly.
It is also possible to apply individual layout styles to different sub-graphs using this algorithm <em>without actually defining group nodes</em>: Use {@link TemporaryGroupNodeInsertionStage}
and specify {@link RecursiveGroupLayout} as its core layout algorithm. The stage allows to define components (i.e.
sub-graphs) which are internally enclosed by a temporary group node. To assign a specific {@link ILayoutAlgorithm}
instance for a component, simply map the component Id to the desired {@link ILayoutAlgorithm} in a {@link IDataProvider}
registered with key {@link TemporaryGroupNodeInsertionStage#COMPONENT_LAYOUT_ALGORITHM_DP_KEY}. It is then not
necessary to use key
{@link RecursiveGroupLayout#GROUP_NODE_LAYOUT_DP_KEY}. Note, however, that only non-nested components are
supported by the mentioned stage.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout Online Documentation}*/
export class RecursiveGroupLayout {
/**
Creates a new instance of {@link RecursiveGroupLayout} with default settings using the given {@link ILayoutAlgorithm layout algorithm} and
{@link ILayoutGroupBoundsCalculator} implementation.
If the given {@link ILayoutGroupBoundsCalculator} is <code>null</code>, the {@link MinimumSizeGroupBoundsCalculator} will be
used instead.
@param [coreLayout=null] the layout algorithm that is applied in each step of the recursion
@param [groupBoundsCalculator=null] the {@link ILayoutGroupBoundsCalculator} for calculating group sizes
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null, groupBoundsCalculator?: ILayoutGroupBoundsCalculator | null);
/**
Creates a new instance of {@link RecursiveGroupLayout} with default settings using the given {@link ILayoutAlgorithm layout algorithm} and
{@link ILayoutGroupBoundsCalculator} implementation.
If the given {@link ILayoutGroupBoundsCalculator} is <code>null</code>, the {@link MinimumSizeGroupBoundsCalculator} will be
used instead.
@param options The parameters to pass.
@param [options.coreLayout=null] the layout algorithm that is applied in each step of the recursion
@param [options.groupBoundsCalculator=null] the {@link ILayoutGroupBoundsCalculator} for calculating group sizes
@param [options.fromSketchMode] Whether or not to consider the initial coordinates of the graph elements. This option sets the {@link RecursiveGroupLayout#fromSketchMode} property on the created object.
@param [options.autoAssignPortCandidates] Whether or not temporary {@link PortCandidate}s are inserted to improve the routing of inter-edges. This option sets the {@link RecursiveGroupLayout#autoAssignPortCandidates} property on the created object.
@param [options.replacePortConstraints] Whether or not {@link PortConstraint}s of inter-edges are temporarily replaced by {@link PortCandidate}s. This option sets the {@link RecursiveGroupLayout#replacePortConstraints} property on the created object.
@param [options.considerEmptyGroups] Whether empty group nodes are handled like group nodes with content or like normal nodes. This option sets the {@link RecursiveGroupLayout#considerEmptyGroups} property on the created object.
@param [options.interEdgeRouter] The current edge routing algorithm for handling inter-edges. This option sets the {@link RecursiveGroupLayout#interEdgeRouter} property on the created object.
@param [options.interEdgesDpKey] The key for marking the inter-edges to be routed. This option sets the {@link RecursiveGroupLayout#interEdgesDpKey} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
groupBoundsCalculator?: ILayoutGroupBoundsCalculator | null;
fromSketchMode?: boolean;
autoAssignPortCandidates?: boolean;
replacePortConstraints?: boolean;
considerEmptyGroups?: boolean;
interEdgeRouter?: ILayoutAlgorithm | null;
interEdgesDpKey?: any | null;
});
/**
Invokes a recursive traversal through the grouping hierarchy of the given graph during which the specified {@link RecursiveGroupLayout#GROUP_NODE_LAYOUT_DP_KEY layout algorithms}
are applied to the content of the groups.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Reroutes the given inter-edges using the current {@link RecursiveGroupLayout#interEdgeRouter edge routing algorithm}.
This method is called after calculating the overall layout when the positions of all nodes and normal edges are fixed.
If no {@link RecursiveGroupLayout#interEdgeRouter inter-edge router} is specified, this method {@link LayoutGraphUtilities#resetPath resets the path} of all inter-edges that don't connect to the proper
location within the group. This may happen for inter-edges without
{@link PortCandidate}s or if the applied layout algorithm doesn't support such constraints.
@param graph the input graph
@param interEdges the edges which traverse the boundary of a group node
@see RecursiveGroupLayout#autoAssignPortCandidates
@see RecursiveGroupLayout#replacePortConstraints
@see RecursiveGroupLayout#interEdgeRouter
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23routeInterEdges(LayoutGraph%2CEdgeList) Online Documentation}*/
routeInterEdges(graph: LayoutGraph | null, interEdges: EdgeList | null): void;
/**
Data provider key for specifying a local partition grid for each group node
Each recursively handled group node can get its own local {@link PartitionGrid}. The grid is then valid for the
child nodes of the group and only visible for the {@link ILayoutAlgorithm layout algorithm} that is responsible for the group's content. Thus, to
properly consider the grid it is required that the responsible algorithm supports the partition grid structure.
The mapping of the nodes to the {@link PartitionCellId partition cells} is still defined by a single {@link IDataProvider} registered
with the graph with key {@link PartitionGrid#PARTITION_CELL_ID_DP_KEY}. When {@link PartitionGrid#createCellId creating the cell id} it is important
to call the creation method on the correct partition grid instance.
In addition to the local partition grids it is also allowed to have a global grid which is defined as usual via key
{@link PartitionGrid#PARTITION_GRID_DP_KEY}. This grid is handled by the core layout algorithm responsible for the
top-level hierarchy.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23GROUP_NODE_PARTITION_GRID_DP_KEY Online Documentation}*/
static GROUP_NODE_PARTITION_GRID_DP_KEY: NodeDpKey<PartitionGrid> | null;
/**
Data provider key for arranging the content of each group node with an individual layout algorithm
The specified {@link ILayoutAlgorithm layout algorithm} instance is applied to the content of the group node. To arrange the top level elements
the {@link LayoutStageBase#coreLayout core layout algorithm} is used.
The core layout is also applied to group nodes if there is no {@link IDataProvider} registered. Importantly,
this must be distinguished from the case that there is a provider but it returns <code>null</code> for a group. Then, {@link RecursiveGroupLayout}
handles the corresponding group node non-recursively. The group node and its content is arranged using the {@link ILayoutAlgorithm layout algorithm}
instance specified for the nearest ancestor of the group node which is associated with an algorithm.
@see RecursiveGroupLayout#NULL_LAYOUT
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23GROUP_NODE_LAYOUT_DP_KEY Online Documentation}*/
static GROUP_NODE_LAYOUT_DP_KEY: NodeDpKey<ILayoutAlgorithm> | null;
/**
Data provider key for assigning source split ids to edges connecting to group nodes
The edges will be aligned with edges that connect to the same group node and have the same split id at their source
(preferably) or target.
<h2>Note</h2>
Marking edges with split ids will only provide good results if {@link HierarchicLayout} is used as core layout
algorithm. Also, the edges need to be {@link HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting routed directly from the content of a group to the group itself}.
<h2>Note</h2>
When a source split id is assigned to an edge that crosses a group node's border, it will be split automatically for
layout. In that case the {@link HierarchicLayout}'s {@link HierarchicLayout#edgeLayoutDescriptor default edge layout descriptor} must have {@link HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting direct edge routing from the content of a group to the group itself}
enabled for good results.
<b>Warning:</b>
In case a core layout algorithm is used that cannot handle groups, it will fail if it needs to layout a graph containing
an edge with a split id.
@see RecursiveGroupLayout#TARGET_SPLIT_ID_DP_KEY
@see HierarchicLayout
@see HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23SOURCE_SPLIT_ID_DP_KEY Online Documentation}*/
static SOURCE_SPLIT_ID_DP_KEY: EdgeDpKey<any> | null;
/**
Data provider key for assigning target split ids to edges connecting to group nodes
The edges will be aligned with edges that connect to the same group node and have the same split id at their source or
target (preferably).
<h2>Note</h2>
Marking edges with split ids will only provide good results if {@link HierarchicLayout} is used as core layout
algorithm. Also, the edges need to be {@link HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting routed directly from the content of a group to the group itself}.
<b>Warning:</b>
In case a core layout algorithm is used that cannot handle groups, it will fail if it needs to layout a graph containing
an edge with a split id.
@see RecursiveGroupLayout#SOURCE_SPLIT_ID_DP_KEY
@see HierarchicLayout
@see HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23TARGET_SPLIT_ID_DP_KEY Online Documentation}*/
static TARGET_SPLIT_ID_DP_KEY: EdgeDpKey<any> | null;
/**
A constant that represents a {@link ILayoutAlgorithm} implementation that does nothing.
This implementation can be assigned to group nodes to keep their content unchanged. The layout algorithm will still
calculate the sizes of the group nodes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23NULL_LAYOUT Online Documentation}*/
static NULL_LAYOUT: ILayoutAlgorithm | null;
/**
Gets or sets whether or not to consider the initial coordinates of the graph elements.
When using the initial coordinates, {@link RecursiveGroupLayout} sets the coordinates of the nodes to their
initial position before the corresponding {@link ILayoutAlgorithm layout algorithm} is called.
<h2>Note</h2>
This option should be enabled if {@link RecursiveGroupLayout} uses a {@link ILayoutAlgorithm layout algorithm} that runs in From Sketch
mode.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23fromSketchMode Online Documentation}*/
fromSketchMode: boolean;
/**
Gets or sets whether or not temporary {@link PortCandidate}s are inserted to improve the routing of inter-edges.
If enabled, {@link RecursiveGroupLayout} will insert {@link PortCandidate}s for all inter-edges that cross a
group node border. Those {@link PortCandidate}s are located at the relative position of the real source/target
node. Inter-edges that connect to such {@link PortCandidate}s will be routed when the layout of the containing
group node is calculated and will not be {@link RecursiveGroupLayout#routeInterEdges rerouted} later. This may produce more suitable edge routes but cannot prevent
edges from crossing nodes.
Without temporary or user specified {@link PortCandidate}s, inter-edges will always end at the border/center of
the corresponding group node. Thus, they are rerouted afterwards using an {@link RecursiveGroupLayout#interEdgeRouter edge routing algorithm}.
<h2>Note</h2>
Predefined
{@link PortCandidate}s are always satisfied depending on the used layout algorithm, even if this option is
disabled.
<b>Warning:</b> Adding temporary {@link PortCandidate}s will only have an effect if the layout algorithm supports them.
@see RecursiveGroupLayout#routeInterEdges
@see RecursiveGroupLayout#interEdgeRouter
@see PortCandidate
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23autoAssignPortCandidates Online Documentation}*/
autoAssignPortCandidates: boolean;
/**
Gets or sets whether or not {@link PortConstraint}s of inter-edges are temporarily replaced by
{@link PortCandidate}s.
If disabled, inter-edges will always end at the border/center of the corresponding group node, even if those edges have
port constraints. Thus, they are {@link RecursiveGroupLayout#routeInterEdges rerouted} later without considering the constraint. Enabling this settings may produce
more suitable edge routes but cannot prevent edges from crossing nodes.
Port candidates are automatically redirected to their original location. Hence, enabling this option may produce more
suitable edge routes if the layout algorithm applied to the content of a group node can handle port candidates.
<h2>Note</h2>
Predefined
{@link PortCandidate}s are always satisfied depending on the used layout algorithm, even if this option is
disabled.
<b>Warning:</b> Temporarily replacing the {@link PortConstraint}s will only have an effect if the layout algorithm supports
{@link PortCandidate}s.
@see RecursiveGroupLayout#routeInterEdges
@see PortCandidate
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23replacePortConstraints Online Documentation}*/
replacePortConstraints: boolean;
/**
Gets or sets whether empty group nodes are handled like group nodes with content or like normal nodes.
If they are handled like other group nodes, {@link RecursiveGroupLayout} will resize them according to their
(non-existing) content. This results in small empty group nodes. Handled like normal nodes, empty group nodes will keep
their initial size.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23considerEmptyGroups Online Documentation}*/
considerEmptyGroups: boolean;
/**
Gets or sets the current edge routing algorithm for handling inter-edges.
During layout, edges that connect from outside a group node to the content inside (inter-edges) are temporarily
connected to the group node itself. Hence, these edges have to be routed after restoring the original graph structure
using this edge routing algorithm.
It is required that a suitable {@link RecursiveGroupLayout#interEdgesDpKey selection key} is specified. The same selection key must be used for setting the sphere of
action for the edge router.
@see RecursiveGroupLayout#interEdgesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23interEdgeRouter Online Documentation}*/
interEdgeRouter: ILayoutAlgorithm | null;
/**
Gets or sets the key for marking the inter-edges to be routed.
The key should be used by the specified {@link RecursiveGroupLayout#interEdgeRouter inter-edge routing algorithm} to obtain the edges to be routed. This layouter
automatically marks these edges and registers the {@link IDataProvider} using the specified key.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see RecursiveGroupLayout#interEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23interEdgesDpKey Online Documentation}*/
interEdgesDpKey: any | null;
/**
Gets or sets a {@link ILayoutGroupBoundsCalculator} which computes the sizes of all group nodes.
This {@link ILayoutGroupBoundsCalculator} is used each time after calculating the layout for a content graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayout%23groupBoundsCalculator Online Documentation}*/
groupBoundsCalculator: ILayoutGroupBoundsCalculator | null;
static $class: Class;
private $_RecursiveGroupLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RecursiveGroupLayout;
}
export interface HandleNaNCoordinatesStage extends LayoutStageBase {
}
/**
A {@link ILayoutStage} that temporarily removes all nodes and edges with <code>NaN</code>-coordinates.
This <code>layout stage</code> may be necessary when a {@link ILayoutAlgorithm} takes the initial coordinates into account. In
this case initial coordinates containing <code>NaN</code>-values, can cause exceptions or strange results.
<code>Layouter</code>s that consider initial coordinates:
<ul>
<li>{@link HierarchicLayout} in incremental mode
<li>{@link OrthogonalLayout} with activated from sketch drawing
<li>{@link OrganicLayout} for a subset of nodes
<li>{@link TreeLayout}, {@link ClassicTreeLayout} when using a from-sketch comparator.
<li>{@link BalloonLayout} in incremental mode
<li>{@link PartialLayout}
<li>all edge routing algorithms: {@link EdgeRouter}, {@link ChannelEdgeRouter}, {@link BusRouter},
{@link OrganicEdgeRouter}
</ul>
Since this stage only hides the invalid graph elements and doesn't apply any new coordinates, it is not defined where
those nodes and edges will be placed within the calculated layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleNaNCoordinatesStage Online Documentation}*/
export class HandleNaNCoordinatesStage {
constructor();
/**
Hides all graph elements with <code>NaN</code>-coordinates during {@link LayoutStageBase#applyLayoutCore core layout} and restores them afterwards.
@param graph the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HandleNaNCoordinatesStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
static $class: Class;
private $_HandleNaNCoordinatesStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HandleNaNCoordinatesStage;
}
export interface IEdgeLabelLayout extends ILabelLayout {
}
/**
{@link IEdgeLabelLayout} defines the properties of one edge label from the perspective of a labeling algorithm.
Note: To avoid inconsistencies, changing the {@link ILabelLayout#modelParameter model parameter} should always be combined with updating the corresponding
{@link ILabelLayout#orientedBox oriented box}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayout Online Documentation}*/
export abstract class IEdgeLabelLayout {
/**
Gets the label model that describes the valid locations for this label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayout%23labelModel Online Documentation}*/
abstract readonly labelModel: IEdgeLabelLayoutModel | null;
/**
Gets the preferred placement for this edge label, which can be considered by {@link ILayoutAlgorithm layout algorithms}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayout%23preferredPlacementDescriptor Online Documentation}*/
abstract readonly preferredPlacementDescriptor: PreferredPlacementDescriptor | null;
static $class: Class;
private $_IEdgeLabelLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeLabelLayout;
}
export interface IEdgeLabelLayoutModel {
}
/**
This interface defines the properties of the model associated with an {@link IEdgeLabelLayout}.
An {@link IEdgeLabelLayoutModel} provides a set of possible <em>candidates</em> for the placement of an edge label. It also defines a
<em>parameter</em> that describes the current location of the label.
It is important that all state information is encapsulated in the {@link IEdgeLabelLayoutModel#createModelParameter model parameter}. {@link IEdgeLabelLayoutModel}
instances may be shared between multiple edge labels but produce different parameters for labels with different
locations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayoutModel Online Documentation}*/
export abstract class IEdgeLabelLayoutModel {
/**
Creates a model parameter that represents the given edge label position within this model.
The created model parameter is the closest parameter representation of the given label location that can be achieved
within this model.
This parameter can be passed to {@link IEdgeLabelLayoutModel#getLabelPlacement} to retrieve the current label
box.
A model parameter can be an arbitrary {@link Object}. However it must contain all information to allow restoring of the encoded
location with this {@link IEdgeLabelLayoutModel}.
@param labelBounds the oriented box of the label, encoding the label location for which the parameter should be created
@param edgeLayout the layout of the edge to which the label belongs
@param sourceLayout the layout of the source node of the label-owning edge
@param targetLayout the layout of the target node of the label-owning edge
@returns the model parameter representing the given label location
@see IEdgeLabelLayoutModel#getLabelPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayoutModel%23createModelParameter(YOrientedRectangle%2CIEdgeLayout%2CINodeLayout%2CINodeLayout) Online Documentation}*/
abstract createModelParameter(labelBounds: YOrientedRectangle | null, edgeLayout: IEdgeLayout | null, sourceLayout: INodeLayout | null, targetLayout: INodeLayout | null): Object | null;
/**
Returns all {@link EdgeLabelCandidate}s that describe valid label positions within this model.
In case the implementing {@link IEdgeLabelLayoutModel} allows every possible location, this method always returns
the current location as a {@link LabelCandidate}.
@param labelLayout the label for which candidates should be generated
@param edgeLayout the layout of the edge to which the label belongs
@param sourceLayout the layout of the source node of the label-owning edge
@param targetLayout the layout of the target node of the label-owning edge
@returns a list of {@link EdgeLabelCandidate} instances
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayoutModel%23getLabelCandidates(IEdgeLabelLayout%2CIEdgeLayout%2CINodeLayout%2CINodeLayout) Online Documentation}*/
abstract getLabelCandidates(labelLayout: IEdgeLabelLayout | null, edgeLayout: IEdgeLayout | null, sourceLayout: INodeLayout | null, targetLayout: INodeLayout | null): YList | null;
/**
Returns the oriented box of the label for the position encoded by the given model parameter.
<h2>Note</h2>
The provided parameter must have been generated by this model.
@param labelSize the width and height of the label
@param edgeLayout the layout of the edge to which the label belongs
@param sourceLayout the layout of the source node of the label-owning edge
@param targetLayout the layout of the target node of the label-owning edge
@param parameter the model parameter that describes the abstract position of the label within this model
@returns the oriented bounds of the label
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayoutModel%23getLabelPlacement(YDimension%2CIEdgeLayout%2CINodeLayout%2CINodeLayout%2CObject) Online Documentation}*/
abstract getLabelPlacement(labelSize: YDimension | null, edgeLayout: IEdgeLayout | null, sourceLayout: INodeLayout | null, targetLayout: INodeLayout | null, parameter: any | null): YOrientedRectangle | null;
/**
Gets a model parameter that encodes the default position of this model's allowed edge label positions.
This parameter can be passed to {@link IEdgeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
label box.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelLayoutModel%23defaultParameter Online Documentation}*/
abstract readonly defaultParameter: Object | null;
static $class: Class;
private $_IEdgeLabelLayoutModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeLabelLayoutModel;
}
export interface IEdgeLayout {
}
/**
An {@link IEdgeLayout} encapsulates the layout information for an edge.
The layout information consists of the two end points of the edge layout and a sequence of control points that determine
the visual path of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLayout Online Documentation}*/
export abstract class IEdgeLayout {
/**
Adds a control point to the end of the control point sequence.
@param x the absolute x-coordinate of the new control point
@param y the absolute y-coordinate of the new control point
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLayout%23addPoint(number%2Cnumber) Online Documentation}*/
abstract addPoint(x: number, y: number): void;
/**
Removes all control points from this edge layout.
The {@link IEdgeLayout} still keeps the {@link IEdgeLayout#sourcePoint source point} and the {@link IEdgeLayout#targetPoint target point}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLayout%23clearPoints() Online Documentation}*/
abstract clearPoints(): void;
/**
Returns the control point at the given position in the sequence of control points.
The first control point in the sequence has index <code>0</code> and the last control point has index {@link IEdgeLayout#pointCount}
<code>-1</code>.
@param index the position of the control point in the control point sequence
@returns the absolute coordinates of the control point at the given index
@see IEdgeLayout#setPoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLayout%23getPoint(number) Online Documentation}*/
abstract getPoint(index: number): YPoint;
/**
Returns the number of control points of the edge.
The source and target points are not included in the point count.
@returns the number of control points
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLayout%23pointCount() Online Documentation}*/
abstract pointCount(): number;
/**
Sets the absolute coordinates of the control point at the given position in the sequence of control points.
The first control point in the sequence has index <code>0</code> and the last control point has index {@link IEdgeLayout#pointCount}
<code>-1</code>.
@param index the position of the changing control point in the sequence
@param x the new absolute x-coordinate of the control point at the given index
@param y the new absolute y-coordinate of the control point at the given index
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLayout%23setPoint(number%2Cnumber%2Cnumber) Online Documentation}*/
abstract setPoint(index: number, x: number, y: number): void;
/**
Gets or sets the relative coordinates of the first end point of this layout.
These coordinates are relative to the center location of the edge's source node. To obtain the absolute coordinates of
the end point, the relative coordinates have to be added to the center coordinates of the source
{@link INodeLayout}.
@see LayoutGraph#getCenterX
@see LayoutGraph#getCenterY
@see INodeLayout#x
@see INodeLayout#y
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLayout%23sourcePoint Online Documentation}*/
abstract sourcePoint: YPoint;
/**
Gets or sets the relative coordinates of the second end point of this layout.
These coordinates are relative to the center location of the edge's target node. To obtain the absolute coordinates of
the end point, the relative coordinates have to be added to the center coordinates of the target
{@link INodeLayout}.
@see LayoutGraph#getCenterX
@see LayoutGraph#getCenterY
@see INodeLayout#x
@see INodeLayout#y
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLayout%23targetPoint Online Documentation}*/
abstract targetPoint: YPoint;
static $class: Class;
private $_IEdgeLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeLayout;
}
export interface IIntersectionCalculator {
}
/**
An {@link IIntersectionCalculator} is responsible for calculating the intersection point between an edge and the
visual representation of its source or target node.
This interface is especially useful in conjunction with {@link PortCalculator} when adjusting the final port
locations after the layout is calculated.
<h2>Note</h2>
If the layout is executed using a {@link LayoutExecutor} and the {@link LayoutExecutor#portAdjustmentPolicy} is set to another value than {@link PortAdjustmentPolicy#NEVER}
an {@link IIntersectionCalculator} which querys the node's {@link IShapeGeometry} for the node's {@link IShapeGeometry#getOutline}
is automatically used.
@see PortCalculator
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIntersectionCalculator Online Documentation}*/
export abstract class IIntersectionCalculator {
/**
Calculates an intersection point between the edge and the given {@link INodeLayout}.
Implementing classes use a point on the connecting edge segment (specified with an offset to the node's center) along
with a direction pointing to the node and calculate the intersection between this line and the node.
@param nodeLayout the layout information of the node
@param xOffset the x-coordinate of a point on the last line segment relative to the node's center
@param yOffset the y-coordinate of a point on the last line segment relative to the node's center
@param dx the x-component of the normalized direction vector indicating the direction of the segment pointing towards the node
@param dy the y-component of the normalized direction vector indicating the direction of the segment pointing towards the node
@returns the intersection coordinates relative to the node's center or <code>null</code> if no such intersection was found
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIntersectionCalculator%23calculateIntersectionPoint(INodeLayout%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
abstract calculateIntersectionPoint(nodeLayout: INodeLayout | null, xOffset: number, yOffset: number, dx: number, dy: number): YPoint | null;
/**
Creates an implementation of the interface {@link IIntersectionCalculator} by using the given function as
implementation for its {@link IIntersectionCalculator#calculateIntersectionPoint} method.
@param calculateIntersectionPoint A function for {@link IIntersectionCalculator}'s single abstract method
{@link IIntersectionCalculator#calculateIntersectionPoint}.
@returns An instance of the {@link IIntersectionCalculator} interface based on the given function.*/
static create(calculateIntersectionPoint: (nodeLayout: INodeLayout | null, xOffset: number, yOffset: number, dx: number, dy: number) => YPoint): IIntersectionCalculator;
static $class: Class;
private $_IIntersectionCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IIntersectionCalculator;
}
export interface IntersectionCalculatorKeys extends YObject {
}
/**
Helper class that declares several keys for {@link IDataProvider} that provide
{@link IIntersectionCalculator}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IntersectionCalculatorKeys Online Documentation}*/
export class IntersectionCalculatorKeys {
/**
Data provider key for providing an individual intersection calculation for each edge source in the graph
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IntersectionCalculatorKeys%23SOURCE_INTERSECTION_CALCULATOR_DP_KEY Online Documentation}*/
static SOURCE_INTERSECTION_CALCULATOR_DP_KEY: EdgeDpKey<IIntersectionCalculator> | null;
/**
Data provider key for providing an individual intersection calculation for each edge target in the graph
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/IntersectionCalculatorKeys%23TARGET_INTERSECTION_CALCULATOR_DP_KEY Online Documentation}*/
static TARGET_INTERSECTION_CALCULATOR_DP_KEY: EdgeDpKey<IIntersectionCalculator> | null;
static $class: Class;
private $_IntersectionCalculatorKeys: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IntersectionCalculatorKeys;
}
export interface ILabelLayout {
}
/**
{@link ILabelLayout} defines the properties of a label from the perspective of a labeling algorithm.
Both node and edge labels need to implement this interface. It describes the size and position of a label.
Note: To avoid inconsistencies, changing the {@link ILabelLayout#modelParameter model parameter} should always be combined with updating the corresponding
{@link ILabelLayout#orientedBox oriented box}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayout Online Documentation}*/
export abstract class ILabelLayout {
/**
Gets the bounding box of the label.
The bounding box is the smallest, paraxial rectangle that contains the whole label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayout%23boundingBox Online Documentation}*/
abstract readonly boundingBox: YRectangle | null;
/**
Gets the box of the label.
This box matches the area that is covered by the label. The {@link YOrientedRectangle#setUpVector up vector} of the rectangle encodes the orientation of the
label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayout%23orientedBox Online Documentation}*/
abstract readonly orientedBox: YOrientedRectangle | null;
/**
Gets or sets the label model parameter that describes the position of this label.
The model parameter must fit the according label model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayout%23modelParameter Online Documentation}*/
abstract modelParameter: any | null;
static $class: Class;
private $_ILabelLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelLayout;
}
export interface ILabelLayoutFactory {
}
/**
{@link ILabelLayoutFactory} provides methods to create, add and remove node/edge labels for
{@link LayoutGraph}s.
It can be used, e.g., to temporarily add labels during layout calculation.
Note that original labels which are removed during a layout calculation have to be re-added when the layout calculation
is done.
An instance of this factory can be obtained with method {@link LayoutGraphUtilities#getLabelFactory}. This method
uses the {@link LayoutGraph#createLabelFactory} method to create the factory.
Note that each factory instance is bound to a specific graph instance, i.e., you can only create, add and remove labels
for the graph which was given to method {@link LayoutGraphUtilities#getLabelFactory}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory Online Documentation}*/
export abstract class ILabelLayoutFactory {
/**
Adds the given {@link IEdgeLabelLayout} to the given edge.
The given {@link ILabelLayout} has to be created with method {@link ILabelLayoutFactory#createLabelLayout}
or {@link ILabelLayoutFactory#createLabelLayout}.
<h2>Note</h2>
Implementations of this method must ensure that re-adding temporarily removed original labels during {@link BufferedLayout buffered layout calculations}
automatically restores the original order of labels of the given edge. Otherwise, the layout calculation will most
likely produce incorrect results or break.
@param edge the edge
@param labelLayout the edge label layout that should be added to the given edge
@throws {Exception} <b>(ArgumentError)</b> if the given label layout has already been added to the graph
@see ILabelLayoutFactory#createLabelLayout
@see ILabelLayoutFactory#createLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory%23addLabelLayout(Edge%2CIEdgeLabelLayout) Online Documentation}*/
abstract addLabelLayout(edge: Edge, labelLayout: IEdgeLabelLayout): void;
/**
Adds the given node label layout to the given node.
The given label layout has to be created with method {@link ILabelLayoutFactory#createLabelLayout} or
{@link ILabelLayoutFactory#createLabelLayout}.
<h2>Note</h2>
Implementations of this method must ensure that re-adding temporarily removed original labels during {@link BufferedLayout buffered layout calculations}
automatically restores the original order of labels at the given node. Otherwise, the layout calculation will most
likely produce incorrect results.
@param node the node
@param labelLayout the node label layout that should be added to the given node
@throws {Exception} <b>(ArgumentError)</b> if the given label layout has already been added to the graph
@see ILabelLayoutFactory#createLabelLayout
@see ILabelLayoutFactory#createLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory%23addLabelLayout(YNode%2CINodeLabelLayout) Online Documentation}*/
abstract addLabelLayout(node: YNode, labelLayout: INodeLabelLayout | null): void;
/**
Creates a new {@link ILabelLayout} for the given edge.
<h2>Note</h2>
Calling this method has no effect on the edge's labels. The returned {@link ILabelLayout} has to be added to the
edge using method {@link ILabelLayoutFactory#addLabelLayout}.
<h2>Note</h2>
The oriented box associated with the newly created edge label layout (see method
{@link ILabelLayout#orientedBox}) may differ from the given oriented box, i.e., the box associated with the label layout corresponds to the closest box
(with respect to the given box) that is valid in the specified label model.
@param edge the edge
@param orientedBox the box specifying the size and location of the label
@param model specifies the model of the created edge label
@param descriptor specifies the preferred placement descriptor of the created label. If this value is
<code>null</code>, the default descriptor is used
@returns a new edge label layout
@see ILabelLayoutFactory#addLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory%23createLabelLayout(Edge%2CYOrientedRectangle%2CIEdgeLabelLayoutModel%2CPreferredPlacementDescriptor) Online Documentation}*/
abstract createLabelLayout(edge: Edge, orientedBox: YOrientedRectangle, model: IEdgeLabelLayoutModel, descriptor: PreferredPlacementDescriptor | null): IEdgeLabelLayout | null;
/**
Creates a new {@link ILabelLayout} for the given node.
<h2>Note</h2>
Calling this method has no effect on the node's labels. The returned label layout has to be added to the node using
method {@link ILabelLayoutFactory#addLabelLayout}.
<h2>Note</h2>
The oriented box associated with the newly created node label layout (see method
{@link ILabelLayout#orientedBox}) may differ from the given oriented box, i.e., the box associated with the label layout corresponds to the closest box
(with respect to the given box) that is valid in the specified label model.
@param node the node
@param orientedBox the box specifying the size and location of the label
@param model specifies the model of the created node label
@returns a new node label layout
@see ILabelLayoutFactory#addLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory%23createLabelLayout(YNode%2CYOrientedRectangle%2CINodeLabelLayoutModel) Online Documentation}*/
abstract createLabelLayout(node: YNode, orientedBox: YOrientedRectangle, model: INodeLabelLayoutModel): INodeLabelLayout | null;
/**
Creates a new {@link ILabelLayout} for the given edge.
This method should create a {@link ILabelLayout} with an {@link IEdgeLabelLayoutModel} that supports the
desired oriented label box exactly as given.
<h2>Note</h2>
Calling this method has no effect on the edge's labels. The returned label layout has to be added to the edge using
method {@link ILabelLayoutFactory#addLabelLayout}.
@param edge the edge
@param orientedBox the box specifying the size and location of the label
@returns a new edge label layout
@see ILabelLayoutFactory#addLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory%23createLabelLayout(Edge%2CYOrientedRectangle) Online Documentation}*/
abstract createLabelLayout(edge: Edge, orientedBox: YOrientedRectangle): IEdgeLabelLayout | null;
/**
Creates a new {@link ILabelLayout} for the given node.
This method should create a {@link ILabelLayout} with a {@link INodeLabelLayoutModel} that supports the
desired oriented label box exactly as given.
<h2>Note</h2>
Calling this method has no effect on the node's labels. The returned label layout has to be added to the node using
method {@link ILabelLayoutFactory#addLabelLayout}.
@param node the node
@param orientedBox the box specifying the size and location of the label
@returns a new node label layout
@see ILabelLayoutFactory#addLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory%23createLabelLayout(YNode%2CYOrientedRectangle) Online Documentation}*/
abstract createLabelLayout(node: YNode, orientedBox: YOrientedRectangle | null): INodeLabelLayout | null;
/**
Removes the given {@link IEdgeLabelLayout} from the given edge.
<h2>Note</h2>
Original labels which are removed during a layout calculation have to be re-added when the layout calculation is done.
@param edge the edge
@param labelLayout the label layout that should be removed from the given edge
@see ILabelLayoutFactory#addLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory%23removeLabelLayout(Edge%2CIEdgeLabelLayout) Online Documentation}*/
abstract removeLabelLayout(edge: Edge, labelLayout: IEdgeLabelLayout): void;
/**
Removes the given node label layout from the given node.
<h2>Note</h2>
Original labels which are removed during a layout calculation have to be re-added when the layout calculation is done.
@param node the node
@param labelLayout the label layout that should be removed from the given node
@see ILabelLayoutFactory#addLabelLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory%23removeLabelLayout(YNode%2CINodeLabelLayout) Online Documentation}*/
abstract removeLabelLayout(node: YNode, labelLayout: INodeLabelLayout | null): void;
/**
Gets the graph associated with this label factory instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILabelLayoutFactory%23graph Online Documentation}*/
abstract readonly graph: LayoutGraph | null;
static $class: Class;
private $_ILabelLayoutFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILabelLayoutFactory;
}
export interface ILayerConstraint {
}
/**
This interface describes a layering constraint.
Such constraints can be given as additional input to the {@link HierarchicLayout}.
@see ILayerConstraintFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraint Online Documentation}*/
export abstract class ILayerConstraint {
/**
Gets or sets the priority of a constraint.
This value is used for resolving conflicting constraints. A constraint with a higher priority is more likely to be
considered.
The priority has to be greater than <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraint%23priority Online Documentation}*/
abstract priority: number;
static $class: Class;
private $_ILayerConstraint: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayerConstraint;
}
export interface ILayerConstraintFactory {
}
/**
This interface provides methods for creating layering constraints.
<b>Warning:</b> If you manually register a {@link IDataProvider} with the graph with key
{@link LayoutKeys#NODE_ID_DP_KEY}, you must use the corresponding node IDs stored in this {@link IDataProvider}
as arguments for all methods that create a constraint. Otherwise, you can just use the node instances themselves.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory Online Documentation}*/
export abstract class ILayerConstraintFactory {
/**
Adds a constraint that forces the {@link YNode} with ID <code>aboveId</code> to lie at least <code>minDistance</code> layers above the {@link YNode}
with ID <code>referenceId</code> with a given weight penalty for larger layer differences.
The minimum distance includes potentially empty layers that are removed by the {@link ILayerer} instance. In
that case, the actual layer difference may be smaller than the minimum distance.
@param referenceId the ID of the reference node
@param aboveId the ID of the node that should lie above
@param minDistance the minimum layer distance between the node and its reference node
@param [weight=0] the weight penalty for larger layer differences
@returns a {@link ILayerConstraint} object that represents the constraint
@see ILayerConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23addPlaceNodeAboveConstraint(Object%2CObject%2Cnumber%2Cnumber) Online Documentation}*/
abstract addPlaceNodeAboveConstraint(referenceId: any | null, aboveId: any | null, minDistance: number, weight?: number): ILayerConstraint | null;
/**
Adds a constraint that forces the {@link YNode} with ID <code>aboveId</code> to lie above the {@link YNode} with ID
<code>referenceId</code>.
@param referenceId the ID of the reference node
@param aboveId the ID of the node that should lie above
@returns a {@link ILayerConstraint} object that represents the constraint
@see ILayerConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23addPlaceNodeAboveConstraint(Object%2CObject) Online Documentation}*/
abstract addPlaceNodeAboveConstraint(referenceId: any | null, aboveId: any | null): ILayerConstraint | null;
/**
Adds a constraint that places a {@link YNode} on the bottom layer.
@param nodeId the ID of the node that should lie on the bottom layer
@returns a {@link ILayerConstraint} object that represents the constraint
@see ILayerConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23addPlaceNodeAtBottomConstraint(Object) Online Documentation}*/
abstract addPlaceNodeAtBottomConstraint(nodeId: any | null): ILayerConstraint | null;
/**
Adds a constraint that places a {@link YNode} on the topmost layer.
@param nodeId the ID of the node that should lie on the top layer
@returns a {@link ILayerConstraint} object that represents the constraint
@see ILayerConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23addPlaceNodeAtTopConstraint(Object) Online Documentation}*/
abstract addPlaceNodeAtTopConstraint(nodeId: any | null): ILayerConstraint | null;
/**
Adds a constraint that forces the {@link YNode} with ID <code>belowId</code> to lie at least <code>minDistance</code> layers below the {@link YNode}
with ID <code>referenceId</code> with a given weight penalty for larger layer differences.
The minimum distance includes potentially empty layers that are removed by the {@link ILayerer} instance. In
that case, the actual layer difference may be smaller than the minimum distance.
@param referenceId the ID of the reference node
@param belowId the ID of the node that should lie below
@param minDistance the minimum layer distance between the node and its reference node
@param [weight=0] the weight penalty for larger layer differences
@returns a {@link ILayerConstraint} object that represents the constraint
@see ILayerConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23addPlaceNodeBelowConstraint(Object%2CObject%2Cnumber%2Cnumber) Online Documentation}*/
abstract addPlaceNodeBelowConstraint(referenceId: any | null, belowId: any | null, minDistance: number, weight?: number): ILayerConstraint | null;
/**
Adds a constraint that forces a {@link YNode} with ID <code>belowId</code> to lie below the {@link YNode} with ID
<code>referenceId</code>.
@param referenceId the ID of the reference node
@param belowId the ID of the node that should lie below
@returns a {@link ILayerConstraint} object that represents the constraint
@see ILayerConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23addPlaceNodeBelowConstraint(Object%2CObject) Online Documentation}*/
abstract addPlaceNodeBelowConstraint(referenceId: any | null, belowId: any | null): ILayerConstraint | null;
/**
Adds a constraint that forces the {@link YNode} with ID <code>sameLayerId</code> to lie in the same layer as the {@link YNode}
with ID <code>referenceId</code>.
@param referenceId the ID of the reference node
@param sameLayerId the ID of the node that should lie in the same layer
@returns a {@link ILayerConstraint} object that represents the constraint
@see ILayerConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23addPlaceNodeInSameLayerConstraint(Object%2CObject) Online Documentation}*/
abstract addPlaceNodeInSameLayerConstraint(referenceId: any | null, sameLayerId: any | null): ILayerConstraint | null;
/**
Disposes this factory.
This method may have to be called when this factory is not needed anymore, i.e. after the layout has been calculated.
The documentation of the method or class providing this factory should state when this is necessary. Calling this method
also clears all constraints.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23dispose() Online Documentation}*/
abstract dispose(): void;
/**
Clears all constraints for a given {@link YNode}.
@param nodeId the ID of the node for which all constraints should be cleared
@see ILayerConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23removeConstraints(Object) Online Documentation}*/
abstract removeConstraints(nodeId: any | null): void;
/**
Gets a token that allows to bind a constraint factory to a graph instance after creation.
This method should only be used if the constraint factory is not bound to a graph instance initially. It allows to bind
this factory to a graph instance after creation. See the factory methods that create instances of this interface for a
description.
@see HierarchicLayout#createLayerConstraintFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerConstraintFactory%23memento Online Documentation}*/
abstract readonly memento: any | null;
static $class: Class;
private $_ILayerConstraintFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayerConstraintFactory;
}
export interface ILayoutAlgorithm {
}
/**
{@link ILayoutAlgorithm} describes the general interface for algorithms that perform a layout process on a
{@link LayoutGraph}.
The main method for invoking the layout calculation is
{@link ILayoutAlgorithm#applyLayout}. Implementing classes will arrange the graph there.
This class also provides keys to register {@link IDataProvider}s with the graph that contain information about
the nodes and edges in the graph. {@link IDataProvider}s registered with {@link LayoutKeys#NODE_ID_DP_KEY}
and {@link LayoutKeys#EDGE_ID_DP_KEY} contain unique identifiers for both nodes and edges. These identifiers allow
for ensuring consistent results even if the order of nodes and edges was changed between two layout runs.
The selection state of nodes and edges is stored in {@link IDataProvider}s registered with the keys {@link LayoutKeys#AFFECTED_NODES_DP_KEY}
and
{@link LayoutKeys#AFFECTED_EDGES_DP_KEY}. Based on this information, the layout algorithm may reduce its
calculations to the selected subset of nodes or edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutAlgorithm Online Documentation}*/
export abstract class ILayoutAlgorithm {
/**
Main layout routine that assigns new layout information to the given graph.
The call to this routine will only succeed if the layout algorithm can handle the input graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutAlgorithm%23applyLayout(LayoutGraph) Online Documentation}*/
abstract applyLayout(graph: LayoutGraph): void;
static $class: Class;
private $_ILayoutAlgorithm: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayoutAlgorithm;
}
export interface LayoutKeys extends YObject {
}
/**
Helper class that provides unique identifiers for some general purpose {@link IDataProvider data providers}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutKeys Online Documentation}*/
export class LayoutKeys {
/**
Data provider key for looking up a unique identifier for each node in a graph
Layout algorithms may use this information to provide consistent layouts for multiple runs.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutKeys%23NODE_ID_DP_KEY Online Documentation}*/
static NODE_ID_DP_KEY: NodeDpKey<any>;
/**
Data provider key for looking up a unique identifier for each edge in a graph
Layout algorithms may use this information to provide consistent layouts for multiple runs.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutKeys%23EDGE_ID_DP_KEY Online Documentation}*/
static EDGE_ID_DP_KEY: EdgeDpKey<any>;
/**
Data provider key for looking up the selected state of the nodes in the graph
A layout algorithm can retrieve a {@link IDataProvider} registered with this key to apply a special handling
to the selected nodes. For example, the layout algorithm may restrict its scope, i.e., it only arranges the selected
nodes.
However, it is often more convenient to use a specific {@link IDataProvider} key for this purpose, e.g., if
you want to combine two layout algorithms of the same kind which have to operate on different subsets of the graph.
Thus, these algorithms may provide a method like <code>setSubgraphNodesDpKey(Object)</code> that customizes the {@link IDataProvider}
key allowing to specify different sets of selected nodes for nested layout algorithms.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutKeys%23AFFECTED_NODES_DP_KEY Online Documentation}*/
static AFFECTED_NODES_DP_KEY: NodeDpKey<boolean>;
/**
Data provider key for looking up the selected state of the edges in the graph
A layout algorithm can retrieve a {@link IDataProvider} registered with this key to apply a special handling
to the selected edges. For example, the layout algorithm may restrict its scope, i.e., it only routes the selected
edges.
However, it is often more convenient to use a specific {@link IDataProvider} key for this purpose, e.g., if
you want to combine two layout algorithms of the same kind which have to operate on different subsets of the graph.
Thus, these algorithms may provide a method like <code>setSubgraphEdgesDpKey(Object)</code> that customizes the {@link IDataProvider}
key allowing to specify different sets of selected edges for nested layout algorithms.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutKeys%23AFFECTED_EDGES_DP_KEY Online Documentation}*/
static AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean>;
static $class: Class;
private $_LayoutKeys: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutKeys;
}
export interface ILayoutStage extends ILayoutAlgorithm {
}
/**
A {@link ILayoutStage} is a layout algorithm that represents a stage of a larger layout process.
It delegates the core layout process to another {@link ILayoutAlgorithm layout algorithm}. Implementing classes may perform pre-processing steps
before the {@link ILayoutStage#coreLayout core layout algorithm} gets called and post-processing steps afterwards.
@see MultiStageLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutStage Online Documentation}*/
export abstract class ILayoutStage {
/**
Gets or sets the core layout algorithm.
This algorithm is wrapped by this stage. It is invoked in {@link ILayoutAlgorithm#applyLayout}. The {@link ILayoutStage}
may add pre- and post-processing steps before and after calling the core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutStage%23coreLayout Online Documentation}*/
abstract coreLayout: ILayoutAlgorithm | null;
static $class: Class;
private $_ILayoutStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayoutStage;
}
export interface INodeLabelLayout extends ILabelLayout {
}
/**
{@link INodeLabelLayout} defines the properties of one node label from the perspective of a labeling algorithm.
Note: To avoid inconsistencies, changing the {@link ILabelLayout#modelParameter model parameter} should always be combined with updating the corresponding
{@link ILabelLayout#orientedBox oriented box}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelLayout Online Documentation}*/
export abstract class INodeLabelLayout {
/**
Gets the label model that describes the valid locations for this label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelLayout%23labelModel Online Documentation}*/
abstract readonly labelModel: INodeLabelLayoutModel | null;
static $class: Class;
private $_INodeLabelLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeLabelLayout;
}
export interface INodeLabelLayoutModel {
}
/**
{@link INodeLabelLayoutModel} defines the properties of the model associated with a
{@link INodeLabelLayout}.
A {@link INodeLabelLayoutModel} provides a set of possible <em>candidates</em> for the placement of a node label. It also defines a <em>parameter</em>
that describes the current location of the label.
It is important that all state information is encapsulated in the {@link INodeLabelLayoutModel#createModelParameter model parameter}. {@link INodeLabelLayoutModel}
instances may be shared between multiple node labels but produce different parameters for labels with different
locations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelLayoutModel Online Documentation}*/
export abstract class INodeLabelLayoutModel {
/**
Creates a model parameter that represents the given node label position within this model.
The created model parameter is the closest parameter representation of the given label location that can be achieved
within this model.
This parameter can be passed to {@link INodeLabelLayoutModel#getLabelPlacement} to retrieve the current label
box.
A model parameter can be an arbitrary {@link Object}. However it must contain all information to allow restoring of the encoded
location with this {@link INodeLabelLayoutModel}.
@param labelBounds the box of the label, encoding the label location for which the parameter should be created
@param nodeLayout the layout of the node to which the label belongs
@returns the model parameter representing the given label location
@see INodeLabelLayoutModel#getLabelPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelLayoutModel%23createModelParameter(YOrientedRectangle%2CINodeLayout) Online Documentation}*/
abstract createModelParameter(labelBounds: YOrientedRectangle | null, nodeLayout: INodeLayout | null): Object | null;
/**
Returns all {@link NodeLabelCandidate}s that describe valid label positions within this model.
In case the implementing {@link INodeLabelLayoutModel} allows every possible location, this method always returns
the current location as a {@link LabelCandidate}.
@param labelLayout the label for which candidates should be generated
@param nodeLayout the layout of the node to which the label belongs
@returns a list of {@link NodeLabelCandidate} instances
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelLayoutModel%23getLabelCandidates(INodeLabelLayout%2CINodeLayout) Online Documentation}*/
abstract getLabelCandidates(labelLayout: INodeLabelLayout | null, nodeLayout: INodeLayout | null): YList | null;
/**
Returns the bounds of the label for the position encoded by the given model parameter.
<h2>Note</h2>
The provided parameter must have been generated by this model.
@param labelSize the width and height of the label
@param nodeLayout the layout of the node to which the label belongs
@param param the model parameter that describes the abstract position of the label within this model
@returns the oriented box of the label
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelLayoutModel%23getLabelPlacement(YDimension%2CINodeLayout%2CObject) Online Documentation}*/
abstract getLabelPlacement(labelSize: YDimension | null, nodeLayout: INodeLayout | null, param: any | null): YOrientedRectangle | null;
/**
Gets a model parameter that encodes the default position of this model's allowed node label positions.
This parameter can be passed to {@link INodeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
label box.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelLayoutModel%23defaultParameter Online Documentation}*/
abstract readonly defaultParameter: Object | null;
static $class: Class;
private $_INodeLabelLayoutModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeLabelLayoutModel;
}
export interface INodeLayout {
}
/**
A {@link INodeLayout} encapsulates the layout information for a node.
The layout information consists of the size and position of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLayout Online Documentation}*/
export abstract class INodeLayout {
/**
Sets the coordinates of the upper-left corner of the node.
@param x the new x-coordinate of the node
@param y the new y-coordinate of the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLayout%23setLocation(number%2Cnumber) Online Documentation}*/
abstract setLocation(x: number, y: number): void;
/**
Sets the size of the node.
@param width the new width of the node
@param height the new height of the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLayout%23setSize(number%2Cnumber) Online Documentation}*/
abstract setSize(width: number, height: number): void;
/**
Gets the x-coordinate of the upper-left corner of the node.
The location of the node is described by the coordinates of the upper-left corner of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLayout%23x Online Documentation}*/
abstract readonly x: number;
/**
Gets the y-coordinate of the upper-left corner of the node.
The location of the node is described by the coordinates of the upper-left corner of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLayout%23y Online Documentation}*/
abstract readonly y: number;
/**
Gets the width of the node.
The width of the node is the horizontal distance the node spans.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLayout%23width Online Documentation}*/
abstract readonly width: number;
/**
Gets the height of the node.
The height of the node is the vertical distance the node spans.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLayout%23height Online Documentation}*/
abstract readonly height: number;
static $class: Class;
private $_INodeLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeLayout;
}
export interface IPartitionInterEdgeRouter {
}
/**
An {@link IPartitionInterEdgeRouter} is responsible for routing inter-edges.
An inter-edge is an edge that connects nodes that belong to different partitions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartitionInterEdgeRouter Online Documentation}*/
export abstract class IPartitionInterEdgeRouter {
/**
Routes inter-edges using {@link ChannelEdgeRouter}.
<b>Warning:</b> This method is not available unless the module <b>router-other</b> is loaded.
@param [router=null] The router instance to use for the edge routing. If not specified, a internal instance is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartitionInterEdgeRouter%23createChannelInterEdgeRouter(ChannelEdgeRouter) Online Documentation}*/
static createChannelInterEdgeRouter(router?: ChannelEdgeRouter | null): IPartitionInterEdgeRouter | null;
/**
Routes inter-edges using {@link EdgeRouter}.
<b>Warning:</b> This method is not available unless the module <b>router-polyline</b> is loaded.
@param [router=null] The router instance to use for the edge routing. If not specified, a internal instance is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartitionInterEdgeRouter%23createPolylineInterEdgeRouter(EdgeRouter) Online Documentation}*/
static createPolylineInterEdgeRouter(router?: EdgeRouter | null): IPartitionInterEdgeRouter | null;
/**
Routes inter-edges of the partitioned graph.
@param graph the input graph
@param partitionIDDP the partition ID for each node in the graph
@param interEdgeDP the map that marks inter-edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartitionInterEdgeRouter%23routeInterEdges(LayoutGraph%2CIDataProvider%2CIDataProvider) Online Documentation}*/
abstract routeInterEdges(graph: LayoutGraph | null, partitionIDDP: IDataProvider | null, interEdgeDP: IDataProvider | null): void;
/**
Gets or sets whether or not to route inter-edges only.
Inter-edges are edges that connect nodes from different partitions. If this feature is disabled, all edges are routed by
this routing algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartitionInterEdgeRouter%23routeInterEdgesOnly Online Documentation}*/
abstract routeInterEdgesOnly: boolean;
static $class: Class;
private $_IPartitionInterEdgeRouter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPartitionInterEdgeRouter;
}
export interface IProfitModel {
}
/**
A {@link IProfitModel} ranks
{@link LabelCandidate}s used for prioritizing certain label locations over others.
Generic labeling algorithm {@link GenericLabeling} will use the ranking to decide which
{@link LabelCandidate}s to eliminate.
{@link LabelCandidate}s with a high profit will be preferred over
{@link LabelCandidate}s with a low profit.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IProfitModel Online Documentation}*/
export abstract class IProfitModel {
/**
Creates an implementation of the interface {@link IProfitModel} by using the given function as implementation for
its {@link IProfitModel#getProfit} method.
@param getProfit A function for {@link IProfitModel}'s single abstract method {@link IProfitModel#getProfit}.
@returns An instance of the {@link IProfitModel} interface based on the given function.*/
static create(getProfit: (candidate: LabelCandidate | null) => number): IProfitModel;
/**
Returns the profit for placing a label using the given {@link LabelCandidate}.
Higher profit means better candidates. Hence, there is a higher probability that the candidate is chosen by a labeling
algorithm.
Profits need to have a value between <code>0</code> and <code>1</code>.
@param candidate the candidate
@returns the profit of the candidate
@see {@link https://docs.yworks.com/yfileshtml/#/api/IProfitModel%23getProfit(LabelCandidate) Online Documentation}*/
abstract getProfit(candidate: LabelCandidate | null): number;
static $class: Class;
private $_IProfitModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IProfitModel;
}
export interface ISequenceConstraintFactory {
}
/**
This interface specifies the general contract for factory classes that can be used to associate sequence constraints
with a graph.
Sequence constraints affect the sequence of nodes within a {@link ILayer layer}.
A {@link ISequenceConstraintFactory} has to be {@link ISequenceConstraintFactory#dispose disposed of} after use. Disposing of the factory will also
remove all constraints previously specified for the factory's associated graph.
<h2>Note</h2>
<ul>
<li>Sequence constraints cannot be used together with {@link SwimlaneDescriptor swimlanes}.
<li>If you manually register with the graph a {@link IDataProvider} with key
{@link LayoutKeys#NODE_ID_DP_KEY}, you must use the corresponding node IDs stored in this {@link IDataProvider}
as arguments for all methods that create a constraint. Otherwise, you can just use the node instances themselves.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISequenceConstraintFactory Online Documentation}*/
export abstract class ISequenceConstraintFactory {
/**
Adds a constraint that forces the {@link YNode} or {@link Edge} with id <code>after</code> to lie after the {@link YNode}
or {@link Edge} with id <code>reference</code>.
<h2>Note</h2>
A constraint is only considered if both elements share at least one layer. An edge is associated with each layer that it
spans, i.e., each layer between it's source and target (excluding the source and target layer).
<h2>Note</h2>
A constraint is only considered if both elements share at least one layer. An edge constraint is associated with each
layer that the edge spans including the edge's source and target (e.g., an after constraint between two edges also
implies that the source/target of the <code>after</code> edge should be placed after that of the <code>reference</code> edge, if possible).
Constraints of edges with {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} are ignored.
<h2>Note</h2>
If there are groups or swimlanes, there are implicit constraints on the edges, e.g., the edge path never leaves the
bounding box of the group that is the common ancestor of the edge's endpoints. Furthermore, if both endpoints of an edge
are associated with the same swimlane, the route does never leave this lane. Such constraints cannot be affected by
sequence constraints.
@param referenceId the ID of the reference node or edge
@param afterId the ID of the node or edge that should be placed after the reference element
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISequenceConstraintFactory%23addPlaceAfterConstraint(Object%2CObject) Online Documentation}*/
abstract addPlaceAfterConstraint(referenceId: any | null, afterId: any | null): void;
/**
Adds a constraint that places a {@link YNode} or {@link Edge} at the start of the sequence.
<h2>Note</h2>
For edges, this constraint applies to each layer that the edge spans including the edge's source and target. Edges with
{@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} are ignored.
<h2>Note</h2>
If there are groups or swimlanes, there are implicit constraints on the edges, e.g., the edge path never leaves the
bounding box of the group that is the common ancestor of the edge's endpoints. Furthermore, if both endpoints of an edge
are associated with the same swimlane, the route does never leave this lane. Such constraints cannot be affected by
sequence constraints.
@param id the ID of the node or edge that should be placed at the start
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISequenceConstraintFactory%23addPlaceAtHeadConstraint(Object) Online Documentation}*/
abstract addPlaceAtHeadConstraint(id: any | null): void;
/**
Adds a constraint that places a {@link YNode} or {@link Edge} at the end of the sequence.
<h2>Note</h2>
For edges, this constraint applies to each layer that the edge spans including the edge's source and target. Edges with
{@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} are ignored.
<h2>Note</h2>
If there are groups or swimlanes, there are implicit constraints on the edges, e.g., the edge path never leaves the
bounding box of the group that is the common ancestor of the edge's endpoints. Furthermore, if both endpoints of an edge
are associated with the same swimlane, the route does never leave this lane. Such constraints cannot be affected by
sequence constraints.
@param id the ID of the node or edge that should be placed at the end
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISequenceConstraintFactory%23addPlaceAtTailConstraint(Object) Online Documentation}*/
abstract addPlaceAtTailConstraint(id: any | null): void;
/**
Adds a constraint that forces the {@link YNode} or {@link Edge} with id <code>before</code> to lie before the {@link YNode}
or {@link Edge} with id <code>reference</code>.
<h2>Note</h2>
A constraint is only considered if both elements share at least one layer. An edge is associated with each layer that it
spans, i.e., each layer between it's source and target (excluding the source and target layer).
<h2>Note</h2>
A constraint is only considered if both elements share at least one layer. An edge constraint is associated with each
layer that the edge spans including the edge's source and target (e.g., a before constraint between two edges also
implies that the source/target of the <code>before</code> edge should be placed before that of the <code>reference</code> edge, if possible).
Constraints of edges with {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} are ignored.
<h2>Note</h2>
If there are groups or swimlanes, there are implicit constraints on the edges, e.g., the edge path never leaves the
bounding box of the group that is the common ancestor of the edge's endpoints. Furthermore, if both endpoints of an edge
are associated with the same swimlane, the route does never leave this lane. Such constraints cannot be affected by
sequence constraints.
@param referenceId the ID of the reference node or edge
@param beforeId the ID of the node or edge that should be placed before the reference element
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISequenceConstraintFactory%23addPlaceBeforeConstraint(Object%2CObject) Online Documentation}*/
abstract addPlaceBeforeConstraint(referenceId: any | null, beforeId: any | null): void;
/**
Disposes of the {@link ISequenceConstraintFactory}.
This method may have be called when the factory is not needed anymore, i.e. after the layout has been calculated. The
documentation of the method or class providing this factory should state when this is necessary. Calling this method
also clears all constraints.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISequenceConstraintFactory%23dispose() Online Documentation}*/
abstract dispose(): void;
/**
Gets a token that allows to bind a constraint factory to a graph instance after creation.
This method should only be used if the constraint factory is not bound to a graph instance initially. It allows to bind
this factory to a graph instance after creation.
@see HierarchicLayoutCore#createSequenceConstraintFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISequenceConstraintFactory%23memento Online Documentation}*/
abstract readonly memento: any | null;
static $class: Class;
private $_ISequenceConstraintFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ISequenceConstraintFactory;
}
export interface LabelCandidate extends YObject, IPlaneObject {
}
/**
A {@link LabelCandidate} describes one valid placement for a label considering the label model.
Generic labeling algorithms place labels according to their label model. To get valid locations for the label, the label
model generates various
{@link LabelCandidate}s. One of these candidates will be chosen to provide the actual placement of the label.
@see INodeLabelLayoutModel
@see IEdgeLabelLayoutModel
@see GenericLabeling
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate Online Documentation}*/
export class LabelCandidate {
/**
Creates a new instance of {@link LabelCandidate} described by location, size, model parameter and internal flag.
@param pos the location of the upper left corner of the candidate
@param size the size of the candidate
@param param the parameters of the label model associated with this candidate
@param owner the label associated with the candidate
@param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23LabelCandidate(YPoint%2CYDimension%2CObject%2CILabelLayout%2Cboolean) Online Documentation}*/
constructor(pos: YPoint | null, size: YDimension | null, param: any | null, owner: ILabelLayout | null, internal?: boolean);
/**
Creates a new instance of {@link LabelCandidate} described by its box, model parameter and internal flag.
@param labelBox the box that specifies the candidate's size and position
@param param the parameters of the label model associated with this candidate
@param owner the label associated with the candidate
@param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23LabelCandidate(YOrientedRectangle%2CObject%2CILabelLayout%2Cboolean) Online Documentation}*/
constructor(labelBox: YOrientedRectangle | null, param: any | null, owner: ILabelLayout | null, internal?: boolean);
/**
Replaces the model parameter and box of the owner with the model parameter and box of this candidate.
When this candidate is chosen to provide the placement of the label, this method transfers the model parameter and the
candidate box to the {@link ILabelLayout}.
@see LabelCandidate#owner
@see LabelCandidate#modelParameter
@see LabelCandidate#box
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23propagate() Online Documentation}*/
propagate(): void;
/**
Returns a {@link string} representation of this {@link LabelCandidate}.
The {@link string} contains the owning label and the coordinates of the candidate's bounds.
@returns a representation of the candidate
@see LabelCandidate#owner
@see LabelCandidate#location
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23toString() Online Documentation}*/
toString(): string;
/**
Gets or sets the custom profit for this candidate.
The profit describes the priority of this candidate with respect to other candidates. Labeling algorithms will use the
profit to decide which candidate to choose.
The value is restricted to the interval <code>[0,1]</code>, where <code>1</code> specifies the highest possible profit.
<h2>Note</h2>
The value is only considered if the labeling algorithm uses an {@link MISLabelingBase#optimizationStrategy optimization strategy}.
@throws {Exception} <b>(ArgumentError)</b> if the specified value lies outside of <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23customProfit Online Documentation}*/
customProfit: number;
/**
Gets the model parameter that was used by the underlying model to generate this label candidate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23modelParameter Online Documentation}*/
readonly modelParameter: any | null;
/**
Gets the coordinates of the upper-left corner of the candidate's bounds.
The bounds are the smallest paraxial box that contains the candidate.
@see LabelCandidate#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23location Online Documentation}*/
readonly location: YPoint | null;
/**
Gets the width and height of the candidate's bounds.
The bounds are the smallest paraxial box that contains the candidate.
@see LabelCandidate#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23size Online Documentation}*/
readonly size: YDimension | null;
/**
Gets the minimum x-coordinate of the candidate's bounds.
The bounds are the smallest paraxial box that contains the candidate.
@see LabelCandidate#location
@see LabelCandidate#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23x Online Documentation}*/
readonly x: number;
/**
Gets the minimum y-coordinate of the candidate's bounds.
The bounds are the smallest paraxial box that contains the candidate.
@see LabelCandidate#location
@see LabelCandidate#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23y Online Documentation}*/
readonly y: number;
/**
Gets the width of the candidate's bounds.
The bounds are the smallest paraxial box that contains the candidate.
@see LabelCandidate#size
@see LabelCandidate#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23width Online Documentation}*/
readonly width: number;
/**
Gets the height of the candidate's bounding box.
The bounding box is the smallest paraxial box that contains the candidate.
@see LabelCandidate#size
@see LabelCandidate#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23height Online Documentation}*/
readonly height: number;
/**
Gets the bounding box of this candidate.
The bounding box describes the smallest paraxial box that contains the candidate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23boundingBox Online Documentation}*/
readonly boundingBox: YRectangle | null;
/**
Gets the oriented box of this candidate.
If the label is rotated, this box will be oriented accordingly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23box Online Documentation}*/
readonly box: YOrientedRectangle | null;
/**
Gets the {@link ILabelLayout} for which this candidate was created.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23owner Online Documentation}*/
readonly owner: ILabelLayout | null;
/**
Gets whether or not this candidate describes an internal label.
Internal labels reside inside the boundaries of a node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23internal Online Documentation}*/
readonly internal: boolean;
/**
Gets or sets a penalty for a candidate position that overlaps the box of a node.
The penalty should be a factor between <code>0</code> and <code>1</code>, where <code>0</code> means that an overlap is considered unimportant and <code>1</code> means that
an overlap is considered severe.
@throws {Exception} <b>(ArgumentError)</b> if the specified penalty is outside <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23nodeOverlapPenalty Online Documentation}*/
nodeOverlapPenalty: number;
/**
Gets or sets a penalty for a candidate position that overlaps the path of an edge.
The penalty should be a factor between <code>0</code> and <code>1</code>, where <code>0</code> means that an overlap is considered unimportant and <code>1</code> means that
an overlap is considered severe.
@throws {Exception} <b>(ArgumentError)</b> if the specified penalty is outside <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23edgeOverlapPenalty Online Documentation}*/
edgeOverlapPenalty: number;
/**
Gets the sum of node overlap penalty and edge overlap penalty.
@see LabelCandidate#edgeOverlapPenalty
@see LabelCandidate#nodeOverlapPenalty
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23overlapPenalty Online Documentation}*/
readonly overlapPenalty: number;
/**
Gets whether or not this candidate's box had already been chosen by the labeling algorithm.
If this candidate is propagated, its model parameter and box were transferred to the {@link LabelCandidate#owner label}.
@see LabelCandidate#propagate
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelCandidate%23propagated Online Documentation}*/
readonly propagated: boolean;
static $class: Class;
private $_LabelCandidate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelCandidate;
}
export interface LabelLayoutBase extends YObject, ILabelLayout {
}
/**
{@link LabelLayoutBase} is the default implementation of the {@link ILabelLayout} interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutBase Online Documentation}*/
export class LabelLayoutBase {
/**
Creates a new {@link LabelLayoutBase} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutBase Online Documentation}*/
constructor();
/**
Sets the box of the label.
This box matches the area that is covered by the label. The {@link YOrientedRectangle#setUpVector up vector} of the rectangle encodes the orientation of the
label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutBase%23orientedBox Online Documentation}*/
orientedBox: YOrientedRectangle | null;
/**
Sets the bounding box of the label.
The bounding box is the smallest, paraxial rectangle that contains the whole label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutBase%23boundingBox Online Documentation}*/
boundingBox: YRectangle | null;
/**
Gets or sets the label model parameter that describes the position of this label.
The model parameter must fit the according label model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutBase%23modelParameter Online Documentation}*/
modelParameter: any | null;
static $class: Class;
private $_LabelLayoutBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelLayoutBase;
}
export interface LabelLayoutData extends YObject {
}
/**
{@link LabelLayoutData} encapsulates layout information for a label.
The information is used by clients to inform label-aware {@link ILayoutAlgorithm layout algorithms} about labeling constraints like label size
and preferred placement.
Label-aware algorithms like {@link HierarchicLayout} return the calculated label positions by assigning the
corresponding box to the provided {@link LabelLayoutData} instances.
{@link IDataProvider DataProviders} are used for associating an array of {@link LabelLayoutData} instances with either the nodes or the
edges of a {@link LayoutGraph}. The {@link IDataProvider DataProviders} must be registered with the input graph using one of the keys
defined in {@link LabelLayoutKeys}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData Online Documentation}*/
export class LabelLayoutData {
/**
Creates a new instance of {@link LabelLayoutData} for a label with the given width and height.
@param width the width of the label
@param height the height of the label
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23LabelLayoutData(number%2Cnumber) Online Documentation}*/
constructor(width: number, height: number);
/**
Creates a new instance of {@link LabelLayoutData} for a label with the given oriented box and preferred placement
descriptor.
@param bounds the box of the label
@param [preferredPlacement=null] a preferred placement descriptor
@throws {Exception} <b>(ArgumentError)</b> if the specified descriptor is <code>null</code>
@see PreferredPlacementDescriptor
@see LabelLayoutData#preferredPlacementDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23LabelLayoutData(YOrientedRectangle%2CPreferredPlacementDescriptor) Online Documentation}*/
constructor(bounds: YOrientedRectangle | null, preferredPlacement?: PreferredPlacementDescriptor | null);
/**
Specifies the coordinates of the upper-left corner of the bounding box of the label.
@param x the new x-coordinate of the upper-left corner
@param y the new y-coordinate of the upper-left corner
@see LabelLayoutData#x
@see LabelLayoutData#y
@see YOrientedRectangle#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23setLocation(number%2Cnumber) Online Documentation}*/
setLocation(x: number, y: number): void;
/**
Specifies the width and height of the {@link LabelLayoutData#bounds oriented box} of this label.
<h2>Note</h2>
{@link LabelLayoutData#width} and {@link LabelLayoutData#height} in contrast yield the width and height of the paraxial bounding box of the oriented box. Both sizes only coincide
in the case the oriented box is not rotated.
@param width the width of the oriented box
@param height the height of the oriented box
@see LabelLayoutData#bounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23setSize(number%2Cnumber) Online Documentation}*/
setSize(width: number, height: number): void;
/**
Returns a {@link string} representation of {@link LabelLayoutData}.
The {@link string} contains the bounds of the label and the preferred placement
@returns the {@link string} representation of {@link LabelLayoutData}
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23toString() Online Documentation}*/
toString(): string;
/**
Gets or sets the {@link YOrientedRectangle oriented box} of the label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23bounds Online Documentation}*/
bounds: YOrientedRectangle | null;
/**
Gets the width of the label's bounding box.
@see YOrientedRectangle#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23width Online Documentation}*/
readonly width: number;
/**
Gets the height of the label's bounding box.
@see YOrientedRectangle#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23height Online Documentation}*/
readonly height: number;
/**
Gets the x-coordinate of the upper-left corner of the label's bounding box.
@see YOrientedRectangle#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23x Online Documentation}*/
readonly x: number;
/**
Gets the y-coordinate of the upper-left corner of the label's bounding box.
@see YOrientedRectangle#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23y Online Documentation}*/
readonly y: number;
/**
Gets or sets the preferred placement of this label.
@throws {Exception} <b>(ArgumentError)</b> if the specified descriptor is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutData%23preferredPlacementDescriptor Online Documentation}*/
preferredPlacementDescriptor: PreferredPlacementDescriptor | null;
static $class: Class;
private $_LabelLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelLayoutData;
}
export interface LabelLayoutKeys extends YObject {
}
/**
{@link LabelLayoutKeys} are used for associating
{@link IDataProvider}s that contain label-specific information for edge and node labels.
Label-aware layout algorithms like {@link HierarchicLayout} may look for such {@link IDataProvider}s
that contain {@link LabelLayoutData} for each edge.
Instead of setting up {@link LabelLayoutData} and {@link IDataProvider}s one can also use {@link LabelLayoutTranslator}
to automatically translate information provided by the standard label layout classes {@link IEdgeLabelLayout} and
{@link INodeLabelLayout} to layout data accessible through the keys defined below.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutKeys Online Documentation}*/
export class LabelLayoutKeys {
/**
Data provider key for accessing label information of each edge in the input graph
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutKeys%23EDGE_LABEL_LAYOUT_DP_KEY Online Documentation}*/
static EDGE_LABEL_LAYOUT_DP_KEY: EdgeDpKey<LabelLayoutData[]> | null;
/**
Data provider key for accessing label information of each node in the input graph
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutKeys%23NODE_LABEL_LAYOUT_DP_KEY Online Documentation}*/
static NODE_LABEL_LAYOUT_DP_KEY: NodeDpKey<LabelLayoutData[]> | null;
/**
Data provider key for marking labels that should be ignored by the layout algorithm
A label is ignored if there is a {@link IDataProvider} registered with this key which returns <code>true</code> for the
given {@link ILabelLayout}.
The {@link LabelLayoutTranslator} only translates labels that should not be ignored.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutKeys%23IGNORED_LABELS_DP_KEY Online Documentation}*/
static IGNORED_LABELS_DP_KEY: ILabelLayoutDpKey<boolean> | null;
static $class: Class;
private $_LabelLayoutKeys: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelLayoutKeys;
}
export interface LabelLayoutTranslator extends YObject, ILayoutStage {
}
/**
{@link LabelLayoutTranslator} automatically translates label layout information provided by the standard label
layout classes {@link IEdgeLabelLayout} and {@link INodeLabelLayout} to layout data of type {@link LabelLayoutData}
that is accessible by the keys defined in class {@link LabelLayoutKeys}.
One can use this {@link ILayoutStage} as label layout algorithm of a label-aware layout algorithm like
{@link HierarchicLayout}. This can be done by assigning the stage by an appropriate call to {@link MultiStageLayout#labeling setLabelLayouter}.
When writing the {@link LabelLayoutData}, the {@link LabelLayoutData#bounds boxes} are set relative. In case of a node label, the box's {@link YOrientedRectangle#anchor anchor} is
relative to the center of the owning node. In case of an edge label the box's anchor is relative to the center of the
source node of the owning edge.
If {@link LabelLayoutTranslator#writeBackNodeLabels writing back node labels} and/or {@link LabelLayoutTranslator#writeBackEdgeLabels writing back edge labels} is enabled, the {@link LabelLayoutData} is transferred
back to the {@link INodeLabelLayout}s and/or {@link IEdgeLabelLayout}s.
By default, the {@link LabelLayoutData#bounds boxes} are now interpreted absolute but by activating {@link LabelLayoutTranslator#writeBackRelativeNodeLabelLocation} and/or {@link LabelLayoutTranslator#writeBackRelativeEdgeLabelLocation} this can be changed to interpret them
relative as explained above.
Note that care must be taken on the choice of the label model that is used by the classes {@link IEdgeLabelLayout}
and {@link INodeLabelLayout}. The calculated label positions must be consistent with the label positions allowed
by the label model. The best label layout results are achieved by choosing {@link FreeEdgeLabelLayoutModel} for
edge layouts and {@link FreeNodeLabelLayoutModel} for node layouts.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator Online Documentation}*/
export class LabelLayoutTranslator {
/**
Creates a new {@link LabelLayoutTranslator} instance with default settings.
@param [options] The parameters to pass.
@param [options.coreLayout] The core layout algorithm that is wrapped by this {@link ILayoutStage}. This option sets the {@link LabelLayoutTranslator#coreLayout} property on the created object.
@param [options.autoFlipping] Whether or not this stage should automatically flip edge label content depending on the label's rotation angle. This option sets the {@link LabelLayoutTranslator#autoFlipping} property on the created object.
@param [options.resettingNodeLabelOrientation] Whether or not to reset the orientation of node labels. This option sets the {@link LabelLayoutTranslator#resettingNodeLabelOrientation} property on the created object.
@param [options.resettingEdgeLabelOrientation] Whether or not to reset the orientation of edge labels. This option sets the {@link LabelLayoutTranslator#resettingEdgeLabelOrientation} property on the created object.
@param [options.writeBackEdgeLabels] Whether or not edge label information is written back to the model after core layout. This option sets the {@link LabelLayoutTranslator#writeBackEdgeLabels} property on the created object.
@param [options.writeBackRelativeEdgeLabelLocation] Whether or not edge label {@link LabelLayoutData#bounds boxes} are interpreted relative to the edge when writing them back to the model. This option sets the {@link LabelLayoutTranslator#writeBackRelativeEdgeLabelLocation} property on the created object.
@param [options.writeBackNodeLabels] Whether or not node label information is written back to the model after the core layout. This option sets the {@link LabelLayoutTranslator#writeBackNodeLabels} property on the created object.
@param [options.writeBackRelativeNodeLabelLocation] Whether or not node label {@link LabelLayoutData#bounds bounds} are interpreted relative to the node when writing them back to the model. This option sets the {@link LabelLayoutTranslator#writeBackRelativeNodeLabelLocation} property on the created object.
@param [options.translateNodeLabels] Whether or not node label information is translated. This option sets the {@link LabelLayoutTranslator#translateNodeLabels} property on the created object.
@param [options.translateEdgeLabels] Whether or not edge label information is translated. This option sets the {@link LabelLayoutTranslator#translateEdgeLabels} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator Online Documentation}*/
constructor(options?: {
coreLayout?: ILayoutAlgorithm | null;
autoFlipping?: boolean;
resettingNodeLabelOrientation?: boolean;
resettingEdgeLabelOrientation?: boolean;
writeBackEdgeLabels?: boolean;
writeBackRelativeEdgeLabelLocation?: boolean;
writeBackNodeLabels?: boolean;
writeBackRelativeNodeLabelLocation?: boolean;
translateNodeLabels?: boolean;
translateEdgeLabels?: boolean;
});
/**
Translates traditional {@link ILabelLayout} information to {@link IDataProvider}-based
{@link LabelLayoutData}.
Afterwards, the calculated layout data will be written back to the original label layout.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the core layout algorithm that is wrapped by this {@link ILayoutStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
/**
Gets or sets whether or not this stage should automatically flip edge label content depending on the label's rotation
angle.
More precisely, if this option is enabled and the up vector of the {@link YOrientedRectangle} associated with
an {@link IEdgeLabelLayout} points downwards ({@link YOrientedRectangle#upY upY} > 0), this {@link ILayoutStage} automatically flips the
label. It rotates the label by <code>180</code> degrees without changing the label's center.
@see LayoutGraphUtilities#autoFlipBox
@see ILabelLayout#orientedBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23autoFlipping Online Documentation}*/
autoFlipping: boolean;
/**
Gets or sets whether or not to reset the orientation of node labels.
If this option is enabled, the up vector of the corresponding oriented box is set to <code>(0,-1)</code> before adding it to the
{@link IDataProvider}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23resettingNodeLabelOrientation Online Documentation}*/
resettingNodeLabelOrientation: boolean;
/**
Gets or sets whether or not to reset the orientation of edge labels.
If this option is enabled, the up vector of the corresponding oriented box is set to <code>(0,-1)</code> before adding it to the
{@link IDataProvider}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23resettingEdgeLabelOrientation Online Documentation}*/
resettingEdgeLabelOrientation: boolean;
/**
Gets or sets whether or not edge label information is written back to the model after core layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23writeBackEdgeLabels Online Documentation}*/
writeBackEdgeLabels: boolean;
/**
Gets or sets whether or not edge label {@link LabelLayoutData#bounds boxes} are interpreted relative to the edge when writing them back to the model.
<b>Warning:</b> This option is only considered if {@link LabelLayoutTranslator#writeBackEdgeLabels} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23writeBackRelativeEdgeLabelLocation Online Documentation}*/
writeBackRelativeEdgeLabelLocation: boolean;
/**
Gets or sets whether or not node label information is written back to the model after the core layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23writeBackNodeLabels Online Documentation}*/
writeBackNodeLabels: boolean;
/**
Gets or sets whether or not node label {@link LabelLayoutData#bounds bounds} are interpreted relative to the node when writing them back to the model.
<b>Warning:</b> This option is only considered if {@link LabelLayoutTranslator#writeBackNodeLabels} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23writeBackRelativeNodeLabelLocation Online Documentation}*/
writeBackRelativeNodeLabelLocation: boolean;
/**
Gets or sets whether or not node label information is translated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23translateNodeLabels Online Documentation}*/
translateNodeLabels: boolean;
/**
Gets or sets whether or not edge label information is translated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelLayoutTranslator%23translateEdgeLabels Online Documentation}*/
translateEdgeLabels: boolean;
static $class: Class;
private $_LabelLayoutTranslator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelLayoutTranslator;
}
/**
@see MISLabelingBase#optimizationStrategy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/OptimizationStrategy Online Documentation}*/
export enum OptimizationStrategy {
/**
An optimization strategy aiming at a good balance between the available optimization options.
@see MISLabelingBase#optimizationStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/OptimizationStrategy%23BALANCED Online Documentation}*/
BALANCED,
/**
An optimization strategy that especially reduces overlaps between labels and nodes as well as labels and node halos.
@see MISLabelingBase#optimizationStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/OptimizationStrategy%23NODE_OVERLAP Online Documentation}*/
NODE_OVERLAP,
/**
An optimization strategy that especially reduces overlaps between labels.
@see MISLabelingBase#optimizationStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/OptimizationStrategy%23LABEL_OVERLAP Online Documentation}*/
LABEL_OVERLAP,
/**
An optimization strategy that especially reduces overlaps between labels and edges.
@see MISLabelingBase#optimizationStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/OptimizationStrategy%23EDGE_OVERLAP Online Documentation}*/
EDGE_OVERLAP,
/**
An optimization strategy that mainly tries to satisfy the preferences described by a {@link PreferredPlacementDescriptor}
associated with edge labels.
@see MISLabelingBase#optimizationStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/OptimizationStrategy%23PREFERRED_PLACEMENT Online Documentation}*/
PREFERRED_PLACEMENT,
/**
Use no optimization strategy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OptimizationStrategy%23NONE Online Documentation}*/
NONE
}
/** Automatic conversion to {@link OptimizationStrategy}. */
type OptimizationStrategyStringValues = "balanced" | "node-overlap" | "label-overlap" | "edge-overlap" | "preferred-placement" | "none";
export interface GenericLabeling extends MISLabelingBase {
}
/**
A generic labeling algorithm for placing the labels of a graph.
This algorithm can switch between two internal implementations. By default, it generates high quality label placements
even for difficult instances. This especially holds if the label models allow a large number of different
{@link LabelCandidate}s, i.e., there is a high potential for optimizations.
If the duration of the calculation is of utmost importance, the internal algorithm can be changed to a simpler
implementation by setting {@link GenericLabeling#maximumDuration} to <code>0</code>. On the downside, the results will not be as good as the ones of the default mode.
In default mode, this algorithm reduces the labeling problem to the maximum independent set (MIS) problem and solves the
problem using simulated annealing. It is inspired by the article from Christensen, Marks and Shieber: <i>A General
Cartographic Labelling Algorithm</i>.
It is recommended to use {@link IEdgeLabelLayoutModel}s and
{@link INodeLabelLayoutModel}s which allow free positioning of labels to achieve best results with this generic
labeling algorithm.
This algorithm works according to the general labeling concept defined by {@link LabelingBase}.
@see LabelingBase
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericLabeling Online Documentation}*/
export class GenericLabeling {
/**
Creates a new instance of {@link GenericLabeling} with default settings.
@param [options] The parameters to pass.
@param [options.deterministic] Whether or not this algorithm behaves deterministically. This option sets the {@link GenericLabeling#deterministic} property on the created object.
@param [options.maximumDuration] The time limit for this algorithm in milliseconds. This option sets the {@link GenericLabeling#maximumDuration} property on the created object.
@param [options.customProfitModelRatio] The ratio between the internal profit (ip) and the profit computed using the specified {@link LabelingBase#profitModel profit model} (sp). This option sets the {@link MISLabelingBase#customProfitModelRatio} property on the created object.
@param [options.optimizationStrategy] The optimization strategy which defines the importance of criteria when optimizing labeling results. This option sets the {@link MISLabelingBase#optimizationStrategy} property on the created object.
@param [options.removeNodeOverlaps] Whether or not {@link LabelCandidate label candidates} that overlap with nodes are removed. This option sets the {@link MISLabelingBase#removeNodeOverlaps} property on the created object.
@param [options.removeEdgeOverlaps] Whether or not {@link LabelCandidate label candidates} that overlap with edges are removed. This option sets the {@link MISLabelingBase#removeEdgeOverlaps} property on the created object.
@param [options.reduceAmbiguity] Whether or not the number of ambiguous label placements is reduced by applying an additional optimization step. This option sets the {@link MISLabelingBase#reduceAmbiguity} property on the created object.
@param [options.profitModel] The {@link IProfitModel} for ranking the {@link LabelCandidate}s for labels. This option sets the {@link LabelingBase#profitModel} property on the created object.
@param [options.moveInternalNodeLabels] Whether or not <em>internal</em> node labels are allowed to move. This option sets the {@link LabelingBase#moveInternalNodeLabels} property on the created object.
@param [options.reduceLabelOverlaps] Whether or not a post-processing step to reduce the number of label overlaps is applied. This option sets the {@link LabelingBase#reduceLabelOverlaps} property on the created object.
@param [options.placeNodeLabels] Whether or not labels assigned to nodes are placed. This option sets the {@link LabelingBase#placeNodeLabels} property on the created object.
@param [options.placeEdgeLabels] Whether or not labels assigned to edges are placed. This option sets the {@link LabelingBase#placeEdgeLabels} property on the created object.
@param [options.affectedLabelsDpKey] The {@link IDataProvider} key to mark labels as selected for placement. This option sets the {@link LabelingBase#affectedLabelsDpKey} property on the created object.
@param [options.autoFlipping] Whether or not edge labels are automatically flipped if otherwise they would be upside-down. This option sets the {@link LabelingBase#autoFlipping} property on the created object.
@param [options.edgeGroupOverlapAllowed] Whether or not edge labels may overlap with edges belonging to the same edge group as the label's edge. This option sets the {@link LabelingBase#edgeGroupOverlapAllowed} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericLabeling Online Documentation}*/
constructor(options?: {
deterministic?: boolean;
maximumDuration?: number;
customProfitModelRatio?: number;
optimizationStrategy?: OptimizationStrategy | OptimizationStrategyStringValues;
removeNodeOverlaps?: boolean;
removeEdgeOverlaps?: boolean;
reduceAmbiguity?: boolean;
profitModel?: IProfitModel | null;
moveInternalNodeLabels?: boolean;
reduceLabelOverlaps?: boolean;
placeNodeLabels?: boolean;
placeEdgeLabels?: boolean;
affectedLabelsDpKey?: any | null;
autoFlipping?: boolean;
edgeGroupOverlapAllowed?: boolean;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Gets or sets whether or not this algorithm behaves deterministically.
If enabled, it produces the same results given the same input graph and settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericLabeling%23deterministic Online Documentation}*/
deterministic: boolean;
/**
Gets or sets the time limit for this algorithm in milliseconds.
<code>0x7FFFFFFF</code> denotes that there is no time limit. Values have to be greater than or equal to <code>0</code>.
<h2>Note</h2>
Restricting the maximum duration may lead to a lower result quality. Furthermore, the algorithm can not guarantee to
strictly observe the specified time limit.
<h2>Note</h2>
If the {@link GenericLabeling#maximumDuration maximum duration} is set to <code>0</code>, a simple but fast internal algorithm is used.
@throws {Exception} <b>(ArgumentError)</b> if the given maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericLabeling%23maximumDuration Online Documentation}*/
maximumDuration: number;
static $class: Class;
private $_GenericLabeling: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GenericLabeling;
}
export interface LabelingBase extends LayoutStageBase {
}
/**
An abstract base class for generic labeling algorithms which place labels of graphs.
Labeling algorithms compute positions for labels in a given graph. The main goal is to place labels such that they do
not overlap with other graph elements. Labeling algorithms do not modify positions of nodes and edges.
<b>Concept</b>
Generic labeling algorithms use the model associated with a label (i.e. {@link INodeLabelLayout#labelModel node label model} for nodes and {@link IEdgeLabelLayout#labelModel edge label model}
for edges). The models are necessary to compute a set of {@link LabelCandidate}s, i.e., candidate positions for a
label. Then, one <i>best matching</i> candidate from the set will be selected. This selection depends on several preferences, e.g., if {@link LabelingBase#removeNodeOverlaps overlaps with nodes}
are not allowed, candidates encoding overlapping positions will be ignored.
After one {@link LabelCandidate} has been chosen, the label model's parameter is used for encoding the label
position. Therefore, the result of a generic labeling algorithm is one model parameter per label. The parameter is {@link ILabelLayout#modelParameter stored with the label}
and expresses the label position with respect to the underlying label model.
<b>Features</b>
This class provides a general frame, offering functionality which can be shared by other generic labeling algorithms.
To only place labels or a subset of labels, the main labeling methods {@link LabelingBase#label}, {@link LabelingBase#label}
and {@link LabelingBase#label} are offered.
As this class represents an {@link LayoutStageBase}, it allows usage as a generic labeling stage. By combining it
with a
{@link LayoutStageBase#coreLayout core layouter}, the labeling will take place after the core layouter was executed
({@link LabelingBase#applyLayout}). This is especially useful if the core layouter does not support label
handling.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase Online Documentation}*/
export abstract class LabelingBase {
/**
Creates a new {@link LabelingBase} instance with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase Online Documentation}*/
constructor();
/**
Places the labels in the graph after first executing the {@link LayoutStageBase#coreLayout core layouter}.
<h2>Note</h2>
Calls method {@link LabelingBase#label} to perform the actual labeling.
@param graph the input graph
@see LabelingBase#label
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Returns the profit for placing a {@link LabelCandidate} with respect to the current {@link LabelingBase#profitModel profit model}.
Method {@link IProfitModel#getProfit} on the current profit model will be invoked to compute the actual profit
value.
The higher the profit (rank) of a candidate is, the more likely it will be chosen as actual position by the algorithm.
@param candidate a label candidate
@returns the profit value between <code>0</code> and <code>1</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23getProfit(LabelCandidate) Online Documentation}*/
getProfit(candidate: LabelCandidate | null): number;
/**
Places the labels of the input graph restricting the placement to labels contained in the given lists.
The {@link LabelingBase#affectedLabelsDpKey selection} will not be considered; the given lists will define the selection of labels that are to be placed.
<b>Warning:</b> If {@link LabelingBase#placeNodeLabels node labeling} or {@link LabelingBase#placeEdgeLabels edge labeling} is disabled, then the labels of the respective given list will <b>not</b> be placed.
@param graph the input graph
@param nodeLabels a list of {@link INodeLabelLayout}s defining the set of node labels that will be placed
@param edgeLabels a list of {@link IEdgeLabelLayout}s defining the set of edge labels that will be placed
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23label(LayoutGraph%2CYList%2CYList) Online Documentation}*/
label(graph: LayoutGraph | null, nodeLabels: YList | null, edgeLabels: YList | null): void;
/**
Places the labels of the input graph using a {@link IDataProvider} registered to the input graph with the
given key for determining which labels to place.
Instead of considering the current {@link LabelingBase#affectedLabelsDpKey selection key}, the {@link IDataProvider} registered with the given key will
be used for selecting labels. Invoking this method has the same effect as first {@link LabelingBase#affectedLabelsDpKey specifying the key} as selection key and
then invoking {@link LabelingBase#label}.
@param graph The input graph.
@param [key=null] The optional {@link IDataProvider} key for label selection. If not specified, the current {@link LabelingBase#affectedLabelsDpKey selection key} is
used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23label(LayoutGraph%2CObject) Online Documentation}*/
label(graph: LayoutGraph | null, key?: any | null): void;
/**
Data provider key for mapping models to edge/node labels
The {@link INodeLabelLayoutModel}s and {@link IEdgeLabelLayoutModel}s linked to
{@link INodeLabelLayout}s and {@link IEdgeLabelLayout}s by this {@link IDataProvider} are taken as
replacement models. This means that the original model is internally replaced by another model when computing valid
positions for the label. A model parameter that fits the calculated position of the replacement model best is set for
the original label model.
<h2>Note</h2>
If there is no {@link IDataProvider} registered with this key, then the original models will be used.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23LABEL_MODEL_DP_KEY Online Documentation}*/
static LABEL_MODEL_DP_KEY: ILabelLayoutDpKey<any> | null;
/**
Gets or sets the {@link IProfitModel} for ranking the {@link LabelCandidate}s for labels.
The profit model is used when {@link LabelingBase#getProfit calculating the profit} of a candidate.
The higher the profit (rank) of a candidate is, the more likely it will be chosen as actual position by the algorithm.
@see IProfitModel
@see SimpleProfitModel
@see LabelingBase#getProfit
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23profitModel Online Documentation}*/
profitModel: IProfitModel | null;
/**
Gets or sets whether or not <em>internal</em> node labels are allowed to move.
A node label is <em>internal</em> if the label's boundaries are completely within the node's boundaries.
<b>Warning:</b> This feature only works in conjunction with {@link INodeLabelLayoutModel}s that restrict the positioning of
labels (i.e. <em>non-free</em> label models).
<h2>Note</h2>
This setting will only have an effect if {@link LabelingBase#placeNodeLabels node labeling} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23moveInternalNodeLabels Online Documentation}*/
moveInternalNodeLabels: boolean;
/**
Gets or sets whether or not a post-processing step to reduce the number of label overlaps is applied.
<h2>Note</h2>
The overlap reduction process is only applied to edge labels whose {@link IEdgeLabelLayoutModel} is of type {@link FreeEdgeLabelLayoutModel}
and node labels whose {@link INodeLabelLayoutModel} is of type {@link FreeNodeLabelLayoutModel}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23reduceLabelOverlaps Online Documentation}*/
reduceLabelOverlaps: boolean;
/**
Gets or sets whether or not {@link LabelCandidate label candidates} that overlap with nodes are removed.
If overlapping candidates are not removed, they will be considered but get a penalty. Therefore, it is still less likely
that an overlapping candidate is finally chosen.
The detection and removal of labels that overlap with nodes may increase the runtime of this algorithm.
<h2>Note</h2>
If this feature is enabled and all candidates of a label overlap with some node, then the label model's default
parameter will be used to determine the actual placement ({@link INodeLabelLayoutModel#defaultParameter} for node labels and {@link IEdgeLabelLayoutModel#defaultParameter} for edge labels).
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23removeNodeOverlaps Online Documentation}*/
removeNodeOverlaps: boolean;
/**
Gets or sets whether or not {@link LabelCandidate label candidates} that overlap with edges are removed.
If overlapping candidates are not removed, they will be considered but get a penalty. Therefore, it is still less likely
that an overlapping candidate is finally chosen.
The detection and removal of labels that overlap with edges may increase the runtime of this algorithm.
<h2>Note</h2>
If this feature is enabled and all candidates of a label overlap with some edge, then the label model's default
parameter will be used to determine the actual placement ({@link INodeLabelLayoutModel#defaultParameter} for node labels and {@link IEdgeLabelLayoutModel#defaultParameter} for edge labels).
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23removeEdgeOverlaps Online Documentation}*/
removeEdgeOverlaps: boolean;
/**
Gets or sets whether or not labels assigned to nodes are placed.
<b>Warning:</b> This setting has higher priority than any custom label {@link LabelingBase#affectedLabelsDpKey selection}. If it is disabled, node labels marked via the
selection will still be ignored. If enabled, then the {@link LabelingBase#affectedLabelsDpKey selection} may determine whether a label is placed or not.
@see LabelingBase#affectedLabelsDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23placeNodeLabels Online Documentation}*/
placeNodeLabels: boolean;
/**
Gets or sets whether or not labels assigned to edges are placed.
<b>Warning:</b> This setting has higher priority than any custom label {@link LabelingBase#affectedLabelsDpKey selection}. If it is disabled, edge labels marked via the
selection will still be ignored. If enabled, then the {@link LabelingBase#affectedLabelsDpKey selection} may determine whether a label is placed or not.
@see LabelingBase#affectedLabelsDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23placeEdgeLabels Online Documentation}*/
placeEdgeLabels: boolean;
/**
Gets or sets the {@link IDataProvider} key to mark labels as selected for placement.
If a {@link IDataProvider} is registered with this key, only the selected labels will be placed, while all
other labels are considered fixed.
The registered {@link IDataProvider} needs to map from {@link ILabelLayout} to {@link boolean} where <code>true</code> indicates
that a label should be placed and <code>false</code> indicates that a label should be ignored.
<b>Warning:</b> The specified selection will be ignored when invoking the algorithm via {@link LabelingBase#label} or
{@link LabelingBase#label}. Furthermore, if {@link LabelingBase#placeNodeLabels node labeling} or {@link LabelingBase#placeEdgeLabels edge labeling} is disabled, the selection does not
have any effect on the respective labels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23affectedLabelsDpKey Online Documentation}*/
affectedLabelsDpKey: any | null;
/**
Gets or sets whether or not edge labels are automatically flipped if otherwise they would be upside-down.
<b>Warning:</b> This feature has an effect only if labels have a <em>free</em> model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23autoFlipping Online Documentation}*/
autoFlipping: boolean;
/**
Gets or sets whether or not edge labels may overlap with edges belonging to the same edge group as the label's edge.
An edge group is a set of edges which share a common bus near the target or source node. They are defined using
{@link IDataProvider}s registered with {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY} and
{@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingBase%23edgeGroupOverlapAllowed Online Documentation}*/
edgeGroupOverlapAllowed: boolean;
static $class: Class;
private $_LabelingBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelingBase;
}
export interface MISLabelingBase extends LabelingBase {
}
/**
A base class for generic labeling algorithms which solve the labeling problem by reducing it to the maximum independent
set (MIS) problem.
Different optimization strategies are available; an optimization strategy defines which criteria the algorithm should
try to optimize. For example, reducing the number of overlaps between labels and nodes may be considered more important
than reducing the number of overlaps between labels and edges.
@see GenericLabeling
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase Online Documentation}*/
export abstract class MISLabelingBase {
/**
Creates a new {@link MISLabelingBase} instance with default settings.
@param [options] The parameters to pass.
@param [options.customProfitModelRatio] The ratio between the internal profit (ip) and the profit computed using the specified {@link LabelingBase#profitModel profit model} (sp). This option sets the {@link MISLabelingBase#customProfitModelRatio} property on the created object.
@param [options.optimizationStrategy] The optimization strategy which defines the importance of criteria when optimizing labeling results. This option sets the {@link MISLabelingBase#optimizationStrategy} property on the created object.
@param [options.removeNodeOverlaps] Whether or not {@link LabelCandidate label candidates} that overlap with nodes are removed. This option sets the {@link MISLabelingBase#removeNodeOverlaps} property on the created object.
@param [options.removeEdgeOverlaps] Whether or not {@link LabelCandidate label candidates} that overlap with edges are removed. This option sets the {@link MISLabelingBase#removeEdgeOverlaps} property on the created object.
@param [options.reduceAmbiguity] Whether or not the number of ambiguous label placements is reduced by applying an additional optimization step. This option sets the {@link MISLabelingBase#reduceAmbiguity} property on the created object.
@param [options.profitModel] The {@link IProfitModel} for ranking the {@link LabelCandidate}s for labels. This option sets the {@link LabelingBase#profitModel} property on the created object.
@param [options.moveInternalNodeLabels] Whether or not <em>internal</em> node labels are allowed to move. This option sets the {@link LabelingBase#moveInternalNodeLabels} property on the created object.
@param [options.reduceLabelOverlaps] Whether or not a post-processing step to reduce the number of label overlaps is applied. This option sets the {@link LabelingBase#reduceLabelOverlaps} property on the created object.
@param [options.placeNodeLabels] Whether or not labels assigned to nodes are placed. This option sets the {@link LabelingBase#placeNodeLabels} property on the created object.
@param [options.placeEdgeLabels] Whether or not labels assigned to edges are placed. This option sets the {@link LabelingBase#placeEdgeLabels} property on the created object.
@param [options.affectedLabelsDpKey] The {@link IDataProvider} key to mark labels as selected for placement. This option sets the {@link LabelingBase#affectedLabelsDpKey} property on the created object.
@param [options.autoFlipping] Whether or not edge labels are automatically flipped if otherwise they would be upside-down. This option sets the {@link LabelingBase#autoFlipping} property on the created object.
@param [options.edgeGroupOverlapAllowed] Whether or not edge labels may overlap with edges belonging to the same edge group as the label's edge. This option sets the {@link LabelingBase#edgeGroupOverlapAllowed} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase Online Documentation}*/
constructor(options?: {
customProfitModelRatio?: number;
optimizationStrategy?: OptimizationStrategy | OptimizationStrategyStringValues;
removeNodeOverlaps?: boolean;
removeEdgeOverlaps?: boolean;
reduceAmbiguity?: boolean;
profitModel?: IProfitModel | null;
moveInternalNodeLabels?: boolean;
reduceLabelOverlaps?: boolean;
placeNodeLabels?: boolean;
placeEdgeLabels?: boolean;
affectedLabelsDpKey?: any | null;
autoFlipping?: boolean;
edgeGroupOverlapAllowed?: boolean;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Returns a {@link INodeMap} which assigns a profit value to each node in the {@link MISLabelingBase#conflictGraph conflict graph}.
As the conflict graph's nodes represent {@link LabelCandidate}s, this mapping gives the profit value of label
candidates. The assigned value is defined as the difference between the {@link LabelingBase#getProfit profit induced by the profit model} and the
candidate's {@link LabelCandidate#overlapPenalty overlap penalty}.
The returned map is a mapping from each {@link YNode} (representing a label candidate) in the {@link MISLabelingBase#conflictGraph}
to a {@link number} representing the profit value of the candidate.
@returns a mapping from nodes (i.e. label candidates) to their profit value
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23assignProfit() Online Documentation}*/
assignProfit(): INodeMap | null;
/**
Creates the edges in the conflict graph, i.e., one edge between two nodes if the corresponding
{@link LabelCandidate}s intersect.
The nodes of the {@link MISLabelingBase#conflictGraph conflict graph} represent
{@link LabelCandidate}s. An edge between candidates signals that they overlap. A maximum independent set will be
computed on the conflict graph to choose candidates such that no two candidates overlap.
This method may be overridden to change the structure of the {@link MISLabelingBase#conflictGraph conflict graph}. Edges between two
{@link LabelCandidate}s in the conflict graph signal that the two candidates should not be selected together. By
overriding this method, arbitrary reasons for indicating that two label candidates should not be chosen at the same time
can be modeled.
<h2>Note</h2>
This method will calculate all intersections among label candidates of the conflict graph. During this process, methods
{@link MISLabelingBase#foundEdgeOverlap}, {@link MISLabelingBase#foundHaloOverlap}, {@link MISLabelingBase#foundLabelOverlap}
and {@link MISLabelingBase#foundNodeOverlap} will be invoked when finding an intersection of a certain type. The
mentioned methods will influence the {@link LabelingBase#getProfit profit} of label candidates. This should be kept in mind when overriding this
method.
@see MISLabelingBase#conflictGraph
@see MISLabelingBase#nodesToBoxes
@see MISLabelingBase#boxesToNodes
@see MISLabelingBase#nodesToID
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23createEdges() Online Documentation}*/
createEdges(): void;
/**
Indicates that an overlap between a {@link LabelCandidate} and an {@link Edge} of the input graph has
been found.
This method is called when finding overlaps while {@link MISLabelingBase#createEdges creating edges} of the {@link MISLabelingBase#conflictGraph conflict graph}. It will store a factor
indicating how much the two elements overlap. The factor influences the {@link LabelingBase#getProfit profit} assigned to the given label candidate.
This method may be overridden to realize a custom strategy for reacting to overlaps between label candidates and edges.
<h2>Note</h2>
This method has no effect if {@link OptimizationStrategy#NONE} is chosen as {@link MISLabelingBase#optimizationStrategy optimization strategy}.
@param labelCandidate the {@link LabelCandidate} overlapping with the given {@link Edge}
@param edge the {@link Edge} overlapping with the given {@link LabelCandidate}
@param eSegment the {@link LineSegment} of the given edge overlapping with the given candidate
@see MISLabelingBase#conflictGraph
@see MISLabelingBase#createEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23foundEdgeOverlap(LabelCandidate%2CEdge%2CLineSegment) Online Documentation}*/
foundEdgeOverlap(labelCandidate: LabelCandidate | null, edge: Edge | null, eSegment: LineSegment | null): void;
/**
Indicates that an overlap between a {@link LabelCandidate} and a {@link NodeHalo} of the input graph has
been found.
This method is called when finding overlaps while {@link MISLabelingBase#createEdges creating edges} of the {@link MISLabelingBase#conflictGraph conflict graph}. It will store a factor
indicating how much the two elements overlap. The factor influences the {@link LabelingBase#getProfit profit} assigned to the given label candidate.
This method may be overridden to realize a custom strategy for reacting to overlaps between label candidates and node
halos.
<h2>Note</h2>
This method has no effect if {@link OptimizationStrategy#NONE} is chosen as {@link MISLabelingBase#optimizationStrategy optimization strategy}.
@param labelCandidate the {@link LabelCandidate} overlapping with a node halo
@param node the {@link YNode} whose {@link NodeHalo} is overlapping with the given label candidate
@param haloRect the bounding box of the {@link NodeHalo} overlapping with the given label candidate
@see MISLabelingBase#conflictGraph
@see MISLabelingBase#createEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23foundHaloOverlap(LabelCandidate%2CYNode%2CYRectangle) Online Documentation}*/
foundHaloOverlap(labelCandidate: LabelCandidate | null, node: YNode | null, haloRect: YRectangle | null): void;
/**
Indicates that an overlap between two {@link LabelCandidate}s has been found.
This method is called when finding overlaps while {@link MISLabelingBase#createEdges creating edges} of the
{@link MISLabelingBase#conflictGraph conflict graph}. It will store a factor indicating how much the two candidates overlap. The factor influences the penalty
assigned when both candidates are chosen, i.e., the penalty for the corresponding overlap.
This method may be overridden to realize a custom strategy for reacting to overlaps among
{@link LabelCandidate}s.
<h2>Note</h2>
This method has no effect if {@link OptimizationStrategy#NONE} is chosen as {@link MISLabelingBase#optimizationStrategy optimization strategy}.
@param candidate1 the first overlapping {@link LabelCandidate}
@param candidate2 the second overlapping {@link LabelCandidate}
@param edge the {@link Edge} in {@link MISLabelingBase#conflictGraph} representing the found overlap
@see MISLabelingBase#conflictGraph
@see MISLabelingBase#createEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23foundLabelOverlap(LabelCandidate%2CLabelCandidate%2CEdge) Online Documentation}*/
foundLabelOverlap(candidate1: LabelCandidate | null, candidate2: LabelCandidate | null, edge: Edge | null): void;
/**
Indicates that an overlap between a {@link LabelCandidate} and a {@link YNode} of the input graph has
been found.
This method is called when finding overlaps while {@link MISLabelingBase#createEdges creating edges} of the {@link MISLabelingBase#conflictGraph conflict graph}. It will store a factor
indicating how much the two elements overlap. The factor influences the {@link LabelingBase#getProfit profit} assigned to the given label candidate.
This method may be overridden to realize a custom strategy for reacting to overlaps between label candidates and nodes.
<h2>Note</h2>
This method has no effect if {@link OptimizationStrategy#NONE} is chosen as {@link MISLabelingBase#optimizationStrategy optimization strategy}.
@param labelCandidate the {@link LabelCandidate} overlapping with the given node
@param node the {@link YNode} overlapping with the given label candidate
@param nodeBox the bounding box of the given node
@see MISLabelingBase#conflictGraph
@see MISLabelingBase#createEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23foundNodeOverlap(LabelCandidate%2CYNode%2CYRectangle) Online Documentation}*/
foundNodeOverlap(labelCandidate: LabelCandidate | null, node: YNode | null, nodeBox: YRectangle | null): void;
/**
The input graph that will be labeled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23graph Online Documentation}*/
graph: LayoutGraph | null;
/**
The conflict graph modeling
{@link LabelCandidate}s as nodes and edges between them as conflicts, i.e., overlaps among candidates.
@see MISLabelingBase#createEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23conflictGraph Online Documentation}*/
conflictGraph: Graph | null;
/**
The mapping from each node in the {@link MISLabelingBase#conflictGraph} to the corresponding {@link LabelCandidate}
instance.
@see MISLabelingBase#conflictGraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23nodesToBoxes Online Documentation}*/
nodesToBoxes: INodeMap | null;
/**
The mapping from the {@link LabelCandidate}s to the corresponding nodes in the
{@link MISLabelingBase#conflictGraph}.
@see MISLabelingBase#conflictGraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23boxesToNodes Online Documentation}*/
boxesToNodes: IMap<any, any> | null;
/**
The mapping from nodes in the {@link MISLabelingBase#conflictGraph} to a corresponding integer value (ID).
The ID denotes the actual label which a {@link LabelCandidate} (i.e. a node in the conflict graph) belongs to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23nodesToID Online Documentation}*/
nodesToID: INodeMap | null;
/**
Gets or sets the ratio between the internal profit (ip) and the profit computed using the specified {@link LabelingBase#profitModel profit model} (sp).
This ratio defines how to weight the two profit values (ip) and (sp). The overall ratio is then computed as
<code>ratio * sp + (1 - ratio) * ip</code>. The profit of a {@link LabelCandidate} defines how likely it is that the candidate
will be chosen as actual label position.
The ratio is defined to be a value from the interval <code>[0,1]</code>.
<b>Warning:</b> This ratio has no effect if the {@link LabelingBase#profitModel profit model} is <code>null</code> or if {@link OptimizationStrategy#NONE} is chosen as
{@link MISLabelingBase#optimizationStrategy optimization strategy}.
@throws {Exception} <b>(ArgumentError)</b> if the given ratio is negative or larger than <code>1</code>
@see LabelingBase#profitModel
@see MISLabelingBase#optimizationStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23customProfitModelRatio Online Documentation}*/
customProfitModelRatio: number;
/**
Gets or sets the optimization strategy which defines the importance of criteria when optimizing labeling results.
Depending on the strategy, criteria like label-node overlaps, label-label overlaps and others are more or less
important. For example, if the number of overlaps between labels and nodes is the most important criterion for the
result, strategy {@link OptimizationStrategy#NODE_OVERLAP} should be chosen.
@throws {Exception} <b>(ArgumentError)</b> if the given strategy is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23optimizationStrategy Online Documentation}*/
optimizationStrategy: OptimizationStrategy;
/**
Gets or sets whether or not {@link LabelCandidate label candidates} that overlap with nodes are removed.
If overlapping candidates are not removed, they will be considered but get a penalty. Therefore, it is still less likely
that an overlapping candidate is finally chosen.
The detection and removal of labels that overlap with nodes may increase the runtime of this algorithm.
<h2>Note</h2>
If this feature is enabled and all candidates of a label overlap with some node, then the label model's default
parameter will be used to determine the actual placement ({@link INodeLabelLayoutModel#defaultParameter} for node labels and {@link IEdgeLabelLayoutModel#defaultParameter} for edge labels).
<b>Warning:</b> It is recommended to enable this feature only in conjunction with {@link OptimizationStrategy#NONE} set as {@link MISLabelingBase#optimizationStrategy optimization}
strategy, i.e., without optimization. If used together with optimization, the optimization strategy may be restricted
too much and not deliver as good results as it could otherwise do.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23removeNodeOverlaps Online Documentation}*/
removeNodeOverlaps: boolean;
/**
Gets or sets whether or not {@link LabelCandidate label candidates} that overlap with edges are removed.
If overlapping candidates are not removed, they will be considered but get a penalty. Therefore, it is still less likely
that an overlapping candidate is finally chosen.
The detection and removal of labels that overlap with edges may increase the runtime of this algorithm.
<h2>Note</h2>
If this feature is enabled and all candidates of a label overlap with some edge, then the label model's default
parameter will be used to determine the actual placement ({@link INodeLabelLayoutModel#defaultParameter} for node labels and {@link IEdgeLabelLayoutModel#defaultParameter} for edge labels).
<b>Warning:</b> It is recommended to enable this feature only in conjunction with {@link OptimizationStrategy#NONE} set as {@link MISLabelingBase#optimizationStrategy optimization}
strategy, i.e., without optimization. If used together with optimization, the optimization strategy may be restricted
too much and not deliver as good results as it could otherwise do.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23removeEdgeOverlaps Online Documentation}*/
removeEdgeOverlaps: boolean;
/**
Gets or sets whether or not the number of ambiguous label placements is reduced by applying an additional optimization
step.
A label position is considered to be ambiguous if it might not be possible to identify to which graph element the label
belongs. For example, an edge label placed in between two edges is ambiguous.
Enabling this reduction step does not guarantee that no ambiguous placements are selected. The algorithm will try to
avoid them if other good positions without ambiguity are available. Other aspects like {@link PreferredPlacementDescriptor preferred placement} for edge
labels will still be more important than the reduction of ambiguity.
<h2>Note</h2>
Applying the ambiguity reduction step may significantly increase the running time of the labeling algorithm, especially
for graphs with a large number of labels and label models with a high number of {@link LabelCandidate}s.
<b>Warning:</b> This setting has no effect if {@link OptimizationStrategy#NONE} is selected as {@link MISLabelingBase#optimizationStrategy optimization strategy}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MISLabelingBase%23reduceAmbiguity Online Documentation}*/
reduceAmbiguity: boolean;
static $class: Class;
private $_MISLabelingBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MISLabelingBase;
}
export interface LayoutGraph extends Graph {
}
/**
A {@link LayoutGraph} is a {@link Graph} with attached layout information that basically represents a
drawing of a graph.
It provides access to the layout information for nodes, edges and labels. This layout information consists of
coordinates for the elements.
{@link ILayoutAlgorithm Layout algorithms} work on {@link LayoutGraph}s and arrange their nodes and edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph Online Documentation}*/
export abstract class LayoutGraph {
/**
Creates a new {@link LayoutGraph} which is a copy of the given subgraph.
@param graph the original graph
@param nodeSubset the nodes that induce the subgraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23LayoutGraph(LayoutGraph%2CICursor) Online Documentation}*/
constructor(graph: LayoutGraph, nodeSubset: ICursor);
/**
Creates a new {@link LayoutGraph} which is a copy of the given subgraph.
@param subgraph the original subgraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23LayoutGraph(LayoutGraph) Online Documentation}*/
constructor(subgraph: LayoutGraph);
/**
Creates a new, empty {@link LayoutGraph} with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23LayoutGraph() Online Documentation}*/
constructor();
/**
Creates a new {@link ILabelLayoutFactory} for this {@link LayoutGraph} that can be used for copying layout
information of labels.
@returns the new {@link ILabelLayoutFactory}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23createLabelFactory() Online Documentation}*/
abstract createLabelFactory(): ILabelLayoutFactory;
/**
Returns the rectangle describing the bounding box of the given node.
@param node the node
@returns the bounds of the node
@see LayoutGraph#getLocation
@see LayoutGraph#getSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getBoundingBox(YNode) Online Documentation}*/
getBoundingBox(node: YNode): YRectangle;
/**
Returns the center coordinates of the given node.
@param node the node
@returns the location of the node's center
@see INodeLayout
@see LayoutGraph#setCenter
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getCenter(YNode) Online Documentation}*/
getCenter(node: YNode): YPoint;
/**
Returns the x-coordinate of the given node's center.
@param node the node
@returns the x-value of the node's center coordinate
@see INodeLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getCenterX(YNode) Online Documentation}*/
getCenterX(node: YNode): number;
/**
Returns the y-coordinate of the given node's center.
@param node the node
@returns the y-value of the node's center coordinate
@see INodeLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getCenterY(YNode) Online Documentation}*/
getCenterY(node: YNode): number;
/**
Returns the height of the given node.
@param node the node
@returns the height of the node
@see INodeLayout#height
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getHeight(YNode) Online Documentation}*/
getHeight(node: YNode): number;
/**
Returns the layout information for all labels associated with the given edge.
The layout information consists of the location, size and orientation of the label.
@param edge the edge
@returns the layout information for the edge's labels
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getLabelLayout(Edge) Online Documentation}*/
abstract getLabelLayout(edge: Edge): IEdgeLabelLayout[];
/**
Returns the layout information for all labels associated with the given node.
The layout information consists of the location, size and orientation of the label.
@param node the node
@returns the layout information for the node's labels
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getLabelLayout(YNode) Online Documentation}*/
abstract getLabelLayout(node: YNode): INodeLabelLayout[];
/**
Returns the layout information for the given edge.
The layout information consists of the locations of the control points and end points of the edge.
@param edge the edge
@returns the layout information for the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getLayout(Edge) Online Documentation}*/
abstract getLayout(edge: Edge): IEdgeLayout;
/**
Returns the layout information for the given node.
The layout information consists of the location and size of the node.
@param node the node
@returns the layout information for the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getLayout(YNode) Online Documentation}*/
abstract getLayout(node: YNode): INodeLayout;
/**
Returns the coordinates of the given node's upper-left corner.
@param node the node
@returns the location of the node's upper-left corner
@see INodeLayout#x
@see INodeLayout#y
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getLocation(YNode) Online Documentation}*/
getLocation(node: YNode): YPoint;
/**
Returns the edge to which the given {@link IEdgeLabelLayout} belongs.
@param labelLayout the label's layout information
@returns the edge that owns the label
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getOwnerEdge(IEdgeLabelLayout) Online Documentation}*/
abstract getOwnerEdge(labelLayout: IEdgeLabelLayout): Edge;
/**
Returns the node to which the given {@link INodeLabelLayout} belongs.
@param labelLayout the label's layout information
@returns the node that owns the label
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getOwnerNode(INodeLabelLayout) Online Documentation}*/
abstract getOwnerNode(labelLayout: INodeLabelLayout): YNode;
/**
Returns the path of an edge, including the ports.
The path of an edge is the position of the source port, followed by the positions of the bends, followed by the position
of the target port.
@param edge the edge
@returns the path of the given edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getPath(Edge) Online Documentation}*/
getPath(edge: Edge): YPointPath;
/**
Returns the path of an edge, including the ports.
The path of an edge is the position of the source port, followed by the positions of the bends, followed by the position
of the target port.
@param edge the edge
@returns a list of {@link YPoint}s representing the path of the given edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getPathList(Edge) Online Documentation}*/
getPathList(edge: Edge): YList;
/**
Returns the control points of an edge.
<h2>Note</h2>
The source and target port are not included in this path.
@param edge the edge
@returns a list of {@link YPoint}s representing the sequence of control points of the edge
@see IEdgeLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getPointList(Edge) Online Documentation}*/
getPointList(edge: Edge): YList;
/**
Returns the control points of an edge.
<h2>Note</h2>
The source and target port are not included in this path.
@param edge the edge
@returns a {@link YPointPath} representing the sequence of control points of the edge
@see IEdgeLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getPoints(Edge) Online Documentation}*/
getPoints(edge: Edge): YPointPath;
/**
Returns the size of the given node.
@param node the node
@returns the size of the node
@see INodeLayout#width
@see INodeLayout#height
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getSize(YNode) Online Documentation}*/
getSize(node: YNode): YDimension;
/**
Returns the absolute coordinates of the source point of the given edge.
@param edge the edge
@returns the absolute source point coordinates of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getSourcePointAbs(Edge) Online Documentation}*/
getSourcePointAbs(edge: Edge): YPoint;
/**
Returns the relative coordinates of the source point of the given edge.
The coordinates are interpreted relative to the center of the source node.
@param edge the edge
@returns the relative source point coordinates of the edge
@see IEdgeLayout#sourcePoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getSourcePointRel(Edge) Online Documentation}*/
getSourcePointRel(edge: Edge): YPoint;
/**
Returns the absolute coordinates of the target point of the given edge.
@param edge the edge
@returns the absolute target point coordinates of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getTargetPointAbs(Edge) Online Documentation}*/
getTargetPointAbs(edge: Edge): YPoint;
/**
Returns the relative coordinates of the target point of the given edge.
The coordinates are interpreted relative to the center of the target node.
@param edge the edge
@returns the relative target point coordinates of the edge
@see IEdgeLayout#targetPoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getTargetPointRel(Edge) Online Documentation}*/
getTargetPointRel(edge: Edge): YPoint;
/**
Returns the width of the given node.
@param node the node
@returns the width of the node
@see INodeLayout#width
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getWidth(YNode) Online Documentation}*/
getWidth(node: YNode): number;
/**
Returns the x-coordinate of the given node's upper-left corner.
@param node the node
@returns the x-value of the node's upper-left corner
@see INodeLayout#x
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getX(YNode) Online Documentation}*/
getX(node: YNode): number;
/**
Returns the y-coordinate of the given node's upper-left corner.
@param node the node
@returns the y-value of the node's upper-left corner
@see INodeLayout#y
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23getY(YNode) Online Documentation}*/
getY(node: YNode): number;
/**
Moves the upper-left corner of a node by a given vector.
@param node the node
@param dx the x-component of the vector
@param dy the y-component of the vector
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23moveBy(YNode%2Cnumber%2Cnumber) Online Documentation}*/
moveBy(node: YNode, dx: number, dy: number): void;
/**
Specifies the center coordinates of the given node.
@param node the node
@param position the new location of the node's center
@see INodeLayout
@see LayoutGraph#setCenter
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setCenter(YNode%2CYPoint) Online Documentation}*/
setCenter(node: YNode, position: YPoint): void;
/**
Specifies the center coordinates of the given node.
@param node the node
@param x the new x-coordinate of the node's center
@param y the new y-coordinate of the node's center
@see INodeLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setCenter(YNode%2Cnumber%2Cnumber) Online Documentation}*/
setCenter(node: YNode, x: number, y: number): void;
/**
Specifies the two end points of the given edge in absolute coordinates.
@param edge the edge
@param source the port on the source side of the edge
@param target the port on the target side of the edge
@see LayoutGraph#setSourcePointAbs
@see LayoutGraph#setTargetPointAbs
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setEndPointsAbs(Edge%2CYPoint%2CYPoint) Online Documentation}*/
setEndPointsAbs(edge: Edge, source: YPoint, target: YPoint): void;
/**
Specifies the coordinates of the upper-left corner of the given node.
@param node the node
@param position the new location of the upper-left corner
@see INodeLayout#setLocation
@see LayoutGraph#setLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setLocation(YNode%2CYPoint) Online Documentation}*/
setLocation(node: YNode, position: YPoint): void;
/**
Specifies the coordinates of the upper-left corner of the given node.
@param node the node
@param x the new x-coordinate of the upper-left corner
@param y the new y-coordinate of the upper-left corner
@see INodeLayout#setLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setLocation(YNode%2Cnumber%2Cnumber) Online Documentation}*/
setLocation(node: YNode, x: number, y: number): void;
/**
Sets the path of the given edge.
The path of an edge is the position of the source port, followed by the positions of the bends, followed by the position
of the target port.
Both source and target port are expected to be given in absolute coordinates.
<h2>Note</h2>
Previously specified coordinates will be cleared.
@param edge an edge in the graph
@param path the list of {@link YPoint}s representing the new path sequence of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setPath(Edge%2CYList) Online Documentation}*/
setPath(edge: Edge, path: YList): void;
/**
Sets the path of the given edge.
The path of an edge is the position of the source port, followed by the positions of the bends, followed by the position
of the target port.
Both source and target port are expected to be given in absolute coordinates.
<h2>Note</h2>
Previously specified coordinates will be cleared.
@param edge an edge in the graph
@param path the new path sequence of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setPath(Edge%2CYPointPath) Online Documentation}*/
setPath(edge: Edge, path: YPointPath): void;
/**
Sets the control points of the given edge.
The source and target points of the edge will not be modified.
<h2>Note</h2>
Previously specified points will be cleared.
@param edge an edge in the graph
@param points the list of {@link YPoint}s representing the new sequence of control points
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setPoints(Edge%2CYList) Online Documentation}*/
setPoints(edge: Edge, points: YList): void;
/**
Sets the control points of the given edge.
The source and target points of the edge will not be modified.
<h2>Note</h2>
Previously specified points will be cleared.
@param edge an edge in the graph
@param points the new sequence of control points
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setPoints(Edge%2CYPointPath) Online Documentation}*/
setPoints(edge: Edge, points: YPointPath): void;
/**
Specifies the size of the given node.
@param node the node
@param size the new size of the node
@see LayoutGraph#setSize
@see INodeLayout#setSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setSize(YNode%2CYDimension) Online Documentation}*/
setSize(node: YNode, size: YDimension): void;
/**
Specifies the width and height of the given node.
@param node the node
@param width the new width of the node
@param height the new height of the node
@see INodeLayout#setSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setSize(YNode%2Cnumber%2Cnumber) Online Documentation}*/
setSize(node: YNode, width: number, height: number): void;
/**
Specifies the absolute coordinates of the source point of the given edge.
@param edge the edge
@param point the new absolute source point coordinates of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setSourcePointAbs(Edge%2CYPoint) Online Documentation}*/
setSourcePointAbs(edge: Edge, point: YPoint): void;
/**
Specifies the relative coordinates of the source point of the given edge.
The coordinates are interpreted relative to the center of the source node.
@param edge the edge
@param point the new relative source point coordinates of the edge
@see IEdgeLayout#sourcePoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setSourcePointRel(Edge%2CYPoint) Online Documentation}*/
setSourcePointRel(edge: Edge, point: YPoint): void;
/**
Specifies the absolute coordinates of the target point of the given edge.
@param edge the edge
@param point the new absolute target point coordinates of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setTargetPointAbs(Edge%2CYPoint) Online Documentation}*/
setTargetPointAbs(edge: Edge, point: YPoint): void;
/**
Specifies the relative coordinates of the target point of the given edge.
The coordinates are interpreted relative to the center of the target node.
@param edge the edge
@param point the new relative target point coordinates of the edge
@see IEdgeLayout#targetPoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23setTargetPointRel(Edge%2CYPoint) Online Documentation}*/
setTargetPointRel(edge: Edge, point: YPoint): void;
/**
Gets a list of all edges in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraph%23edgeList Online Documentation}*/
readonly edgeList: EdgeList;
static $class: Class;
private $_LayoutGraph: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutGraph;
}
export interface LayoutGraphUtilities extends YObject {
}
/**
The {@link LayoutGraphUtilities} provides several helper and utility functions for {@link LayoutGraph} and
the {@link ILayoutAlgorithm layout algorithms}.
@see LayoutGraph
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities Online Documentation}*/
export class LayoutGraphUtilities {
/**
Aligns the specified nodes either vertically or horizontally according to the specified node alignment mode.
For vertical arrangements, nodes are either {@link NodeAlignment#LEADING top-aligned}, {@link NodeAlignment#CENTERED center-aligned}, {@link NodeAlignment#TRAILING bottom-aligned}, or {@link NodeAlignment#DISTRIBUTED vertically distributed}.
For horizontal arrangements, nodes are either {@link NodeAlignment#LEADING left-aligned}, {@link NodeAlignment#CENTERED center-aligned}, {@link NodeAlignment#TRAILING right-aligned}, or {@link NodeAlignment#DISTRIBUTED horizontally distributed}.
In this context, <em>distributed</em> means that the specified nodes are placed in such a way that the projections of their bounds onto the
vertical or horizontal coordinate axis do not overlap. For example, after vertically distributing nodes, <code>(n1.y + n1.height) &lt;= n2.y</code>
for two consecutively placed nodes <code>n1</code> and <code>n2</code>.
@param graph the graph that contains the specified nodes
@param nodes the subset of the specified graph's nodes that is arranged
@param referenceNode if <code>null</code>, the common coordinate for aligning the specified nodes is calculated as the the vertical minimum (top-aligned),
the vertical center, the vertical maximum (bottom-aligned), the horizontal minimum (left-aligned), the horizontal
center, or the horizontal maximum (right-aligned) of the bounds of the specified nodes; otherwise the appropriate
coordinate of the given reference node's bounds is used as common coordinate for aligning the specified nodes. The
reference node is ignored when {@link NodeAlignment#DISTRIBUTED distributing} nodes.
@param vertical <code>true</code> if nodes should be aligned (or distributed) according to their y-coordinates; otherwise nodes are aligned (or
distributed) according to their x-coordinates.
@param mode one of the valid alignment modes
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23alignNodeLayouts(LayoutGraph%2CYNodeList%2CYNode%2Cboolean%2CNodeAlignment) Online Documentation}*/
static alignNodeLayouts(graph: LayoutGraph | null, nodes: YNodeList | null, referenceNode: YNode | null, vertical: boolean, mode: NodeAlignment | NodeAlignmentStringValues): void;
/**
Reassigns the bounds of the rectangles in a grid to fit in a large rectangle with an aspect ratio close to the specified
aspect ratio.
The aspect ratio must be greater than <code>0</code>.
@param rectangles the rectangles whose coordinates will be modified
@param finalRect the exact dimension will be placed in this rectangle
@param aspectRatio the preferred aspect ratio of the grid
@returns the dimension of the grid
@throws {Exception} <b>(ArgumentError)</b> if the given aspect ratio is less than or equal to zero
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23arrangeRectangleGrid(Array%3CRectangle2D%3E%2CRectangle2D%2Cnumber) Online Documentation}*/
static arrangeRectangleGrid(rectangles: Rectangle2D[] | null, finalRect: Rectangle2D | null, aspectRatio: number): YDimension | null;
/**
Reassigns the bounds of the rectangles in multiple rows to fit the preferred width and height.
The rectangles will be placed within this bound, arranged in rows, where each row may be split into more sub-rows. This
often leads to more compact results than provided by the procedure used in
{@link LayoutGraphUtilities#arrangeRectangleRows}. A row will be subdivided into more rows, if two successive
rectangles to be placed in the row could be placed stacked within this row without making the row larger in height.
The preferred width and height should be greater than <code>0</code>, otherwise the nodes are not arranged.
@param rectangles the rectangles to arrange (coordinates will be assigned)
@param finalRect the exact resulting dimension will be stored in this rectangle
@param preferredWidth the preferred width of the result rectangle
@param preferredHeight the preferred height of the result rectangle
@param compaction <code>true</code> if the algorithm tries to always minimize the resulting dimension, whilst respecting the given constraints, <code>false</code>
otherwise
@param constraintMode the width or height constraints to consider
@param [alignment=RowAlignment.LEADING] the alignment inside a row
@returns the number of rows used
@see LayoutGraphUtilities#arrangeRectangleMultiRows
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23arrangeRectangleMultiRows(Array%3CRectangle2D%3E%2CRectangle2D%2Cnumber%2Cnumber%2Cboolean%2CMultiRowConstraint%2CRowAlignment) Online Documentation}*/
static arrangeRectangleMultiRows(rectangles: Rectangle2D[] | null, finalRect: Rectangle2D | null, preferredWidth: number, preferredHeight: number, compaction: boolean, constraintMode: MultiRowConstraint | MultiRowConstraintStringValues, alignment?: RowAlignment | RowAlignmentStringValues): number;
/**
Reassigns the bounds of the rectangles to fit in a large rectangle with an aspect ratio close to the specified aspect
ratio.
This is achieved by arranging the rectangles in rows.
The aspect ratio must be greater than <code>0</code>.
@param rectangles the rectangles whose coordinates will be modified
@param finalRect the exact dimension of the resulting rectangle
@param aspectRatio the preferred aspect ratio of the resulting rectangle
@param [alignment=RowAlignment.LEADING] one of the valid row alignments
@returns the number of used rows
@throws {Exception} <b>(ArgumentError)</b> if the aspect ratio is <code>0</code> or less
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23arrangeRectangleRows(Array%3CRectangle2D%3E%2CRectangle2D%2Cnumber%2CRowAlignment) Online Documentation}*/
static arrangeRectangleRows(rectangles: Rectangle2D[] | null, finalRect: Rectangle2D | null, aspectRatio: number, alignment?: RowAlignment | RowAlignmentStringValues): number;
/**
Applies the reversed layout of the first edge to the second edge.
@param graph the graph to which both edges belong
@param edge1 the edge which adopts the points
@param edge2 the template
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23assignReverseLayout(LayoutGraph%2CEdge%2CEdge) Online Documentation}*/
static assignReverseLayout(graph: LayoutGraph | null, edge1: Edge | null, edge2: Edge | null): void;
/**
Flips the orientation of a given rectangle, if its up vector points downward <code>(</code>{@link YOrientedRectangle#upY upY}<code> > 0)</code>.
That means, it rotates the rectangle by <code>180</code> degrees without changing the rectangle's center.
@param rect the given rectangle
@returns <code>true</code> if the given rectangle was flipped, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23autoFlipBox(YOrientedRectangle) Online Documentation}*/
static autoFlipBox(rect: YOrientedRectangle | null): boolean;
/**
Clips the path of the given edge on the bounding box of the source and target points.
On a clipped edge, the source and target port will be located at the border of the corresponding node.
<h2>Note</h2>
If the source/target end of the edge's path is not contained in the source/target node layout, the algorithm just keeps
that end of the path as it is, i.e., this method does never extend the path.
@param graph the graph to which the edge belongs
@param edge the edge to clip
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23clipEdgeOnBounds(LayoutGraph%2CEdge) Online Documentation}*/
static clipEdgeOnBounds(graph: LayoutGraph | null, edge: Edge | null): void;
/**
Returns the path of an edge clipped on the bounding box of the source and target node.
On the clipped edge, the source and target port will be located at the border of the corresponding node.
<h2>Note</h2>
If the source/target end of the edge's path is not contained in the source/target node layout, the algorithm just keeps
that end of the path as it is, i.e., this method does never extend the path.
@param graph the graph to which the edge belongs
@param edge the clipped edge
@returns the clipped path of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23clipEdgePathOnBounds(LayoutGraph%2CEdge) Online Documentation}*/
static clipEdgePathOnBounds(graph: LayoutGraph | null, edge: Edge | null): YPointPath | null;
/**
Returns the path of an edge clipped on insets of the bounding box of the source and target node.
On the clipped edge, the source and target port will be located inside the node with an inset to the border of the
corresponding node.
<h2>Note</h2>
If the source/target end of the given path is not contained in the specified source/target node layout, the algorithm
just keeps that end of the path as it is, i.e., this method does never extend the path.
@param edge the clipped edge
@param source the source of the edge
@param target the target of the edge
@param [inset=0.0] the inset of the node
@returns the clipped path of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23clipEdgePathOnBounds(IEdgeLayout%2CINodeLayout%2CINodeLayout%2Cnumber) Online Documentation}*/
static clipEdgePathOnBounds(edge: IEdgeLayout | null, source: INodeLayout | null, target: INodeLayout | null, inset?: number): YPointPath | null;
/**
Clips the path of all edges in the graph on the bounding box of their source and target points.
On a clipped edge, the source and target port will be located at the border of the corresponding node.
<h2>Note</h2>
If the source/target end of an edge's path is not contained in the associated source/target node layout, the algorithm
just keeps that end of the path as it is, i.e., this method does never extend the path.
@param graph the graph
@see LayoutGraphUtilities#clipEdgeOnBounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23clipEdgesOnBounds(LayoutGraph) Online Documentation}*/
static clipEdgesOnBounds(graph: LayoutGraph | null): void;
/**
Returns the path of a given edge path clipped on the given bounding boxes of the source and target node.
<h2>Note</h2>
In contrast to methods {@link LayoutGraphUtilities#clipEdgePathOnBounds} and
{@link LayoutGraphUtilities#clipEdgePathOnBounds}, the path can explicitly be specified and is not queried from a
given graph and edge. Furthermore, this method does not consider any insets on the given boxes
<h2>Note</h2>
If the source/target end of the given path is not contained in the specified source/target rectangle, the algorithm just
keeps that end of the path as it is, i.e., this method does never extend the path.
@param path the path that will be clipped
@param sourceBox the box of the source node that the given path should be clipped at
@param targetBox the box of the target node that the given path should be clipped at
@returns the path clipped on the given nodes boxes
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23clipPathOnBounds(YPointPath%2CYRectangle%2CYRectangle) Online Documentation}*/
static clipPathOnBounds(path: YPointPath | null, sourceBox: YRectangle | null, targetBox: YRectangle | null): YPointPath | null;
/**
Determines the main direction of the edge flow by analyzing the current layout of the graph.
The method only considers edges that are marked in the given {@link IDataProvider}.
@param graph the underlying graph
@param considerEdges the edges to consider
@returns one of the predefined flow directions
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23determineEdgeFlowDirection(LayoutGraph%2CIDataProvider) Online Documentation}*/
static determineEdgeFlowDirection(graph: LayoutGraph | null, considerEdges: IDataProvider | null): EdgeFlow;
/**
Returns a {@link string} representation of the path of the given edge.
<h2>Note</h2>
The source and target ports are not included in the representation.
@param graph the graph to which the edge belongs
@param edge the edge
@returns the {@link string} that contains all control points of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23edgeLayoutString(LayoutGraph%2CEdge) Online Documentation}*/
static edgeLayoutString(graph: LayoutGraph | null, edge: Edge | null): string | null;
/**
Returns the bounding box of the given edge.
In this case, the bounding box is the smallest paraxial rectangle which contains the control points as well as the
absolute coordinates of the source and target point of the edge.
<h2>Note</h2>
The bounding box does not include edge labels.
@param graph the graph that contains the edge
@param edge the edge
@returns the bounding box of the edge
@see LayoutGraph#getSourcePointAbs
@see LayoutGraph#getTargetPointAbs
@see LayoutGraph#getLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getBoundingBox(LayoutGraph%2CEdge) Online Documentation}*/
static getBoundingBox(graph: LayoutGraph, edge: Edge): Rectangle2D | null;
/**
Returns the bounding box of the given node.
In this case, the bounding box is the smallest paraxial rectangle which contains the given node. It is identical to the
node's bounds.
<h2>Note</h2>
The bounding box does not include node labels.
@param graph the graph that contains the node
@param node the node
@returns the bounding box of the node
@see LayoutGraph#getLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getBoundingBox(LayoutGraph%2CYNode) Online Documentation}*/
static getBoundingBox(graph: LayoutGraph, node: YNode): Rectangle2D | null;
/**
Returns the bounding box of the nodes and edges accessible through the given cursors, optionally including node labels,
edge labels or {@link NodeHalo}s.
In this case, the bounding box is the smallest paraxial rectangle which contains the given nodes and the control points
as well as the source and target points of all edges. If node or edge labels should be included, it also contains their
boxes. {@link NodeHalo}s can also extend the bounding box if desired.
@param graph the graph that contains the given nodes and edges
@param nodes the nodes to include in the bounding box
@param edges the edges to include in the bounding box
@param includeNodeLabels <code>true</code> if labels of nodes should be included, <code>false</code> otherwise
@param includeEdgeLabels <code>true</code> if labels of edges should be included, <code>false</code> otherwise
@param [includeHalos=false] <code>true</code> if {@link NodeHalo}s should be included, <code>false</code> otherwise
@returns the bounding box of the given nodes and edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getBoundingBox(LayoutGraph%2CINodeCursor%2CIEdgeCursor%2Cboolean%2Cboolean%2Cboolean) Online Documentation}*/
static getBoundingBox(graph: LayoutGraph | null, nodes: INodeCursor | null, edges: IEdgeCursor | null, includeNodeLabels: boolean, includeEdgeLabels: boolean, includeHalos?: boolean): Rectangle2D | null;
/**
Returns the bounding box of the nodes and edges accessible through the given cursors, optionally including node labels
and edge labels.
In this case, the bounding box is the smallest paraxial rectangle which contains the given nodes and the control points
as well as the source and target points of all edges. If labels should be included, it also contains their boxes.
@param graph the graph that contains the given nodes and edges
@param nodes the nodes to include in the bounding box
@param edges the edges to include in the bounding box
@param includeLabels <code>true</code> if labels of nodes and edges should be included, <code>false</code> otherwise
@returns the bounding box of the given nodes and edges
@see LayoutGraphUtilities#getBoundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getBoundingBox(LayoutGraph%2CINodeCursor%2CIEdgeCursor%2Cboolean) Online Documentation}*/
static getBoundingBox(graph: LayoutGraph | null, nodes: INodeCursor | null, edges: IEdgeCursor | null, includeLabels: boolean): Rectangle2D | null;
/**
Returns the bounding box of the nodes and edges accessible through the given cursors.
In this case, the bounding box is the smallest paraxial rectangle which contains the given nodes and the control points
as well as the source and target points of all edges.
<h2>Note</h2>
The bounding box does not include node labels or edge labels.
@param graph the graph that contains the nodes and edges
@param nodes the nodes to include in the bounding box
@param edges the edges to include in the bounding box
@returns the bounding box containing the given nodes and edges
@see LayoutGraphUtilities#getBoundingBoxOfEdges
@see LayoutGraphUtilities#getBoundingBoxOfNodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getBoundingBox(LayoutGraph%2CINodeCursor%2CIEdgeCursor) Online Documentation}*/
static getBoundingBox(graph: LayoutGraph | null, nodes: INodeCursor | null, edges: IEdgeCursor | null): Rectangle2D | null;
/**
Returns the bounding box of the edges accessible through the given cursor.
In this case, the bounding box is the smallest paraxial rectangle which contains the control points as well as the
absolute coordinates of the source and target points of all given edges.
<h2>Note</h2>
The bounding box does not include edge labels.
@param graph the graph that contains the edges
@param edges the edges to include in the bounding box
@returns the bounding box containing the given edges
@see LayoutGraph#getSourcePointAbs
@see LayoutGraph#getTargetPointAbs
@see LayoutGraph#getLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getBoundingBoxOfEdges(LayoutGraph%2CIEdgeCursor) Online Documentation}*/
static getBoundingBoxOfEdges(graph: LayoutGraph, edges: IEdgeCursor | null): Rectangle2D | null;
/**
Returns the bounding box of the nodes accessible through the given cursor.
In this case, the bounding box is the smallest paraxial rectangle which contains the given nodes.
<h2>Note</h2>
The bounding box does not include node labels.
@param graph the graph that contains the nodes
@param nodes the nodes to include in the bounding box
@returns the bounding box containing the given nodes
@see LayoutGraph#getLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getBoundingBoxOfNodes(LayoutGraph%2CINodeCursor) Online Documentation}*/
static getBoundingBoxOfNodes(graph: LayoutGraph, nodes: INodeCursor | null): Rectangle2D | null;
/**
Returns the {@link ILabelLayoutFactory} for the given graph.
<h2>Note</h2>
The factory instance is bound to the specified graph instance, i.e., you can only create, add and remove labels for this
graph.
@param graph the graph
@returns the label factory of the graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getLabelFactory(LayoutGraph) Online Documentation}*/
static getLabelFactory(graph: LayoutGraph | null): ILabelLayoutFactory | null;
/**
Returns the upper-left corner of the paraxial bounding box of the given edge label as encoded by the specified model
parameter.
The parameter describes the abstract position of the label within the specified model. The parameter must have been
generated by that model.
@param model the {@link IEdgeLabelLayoutModel} used for placing the label
@param labelSize the size of the label that should be placed
@param edgeLayout the layout of the edge to which the label belongs
@param sourceLayout the layout of the edge's source node
@param targetLayout the layout of the edge's target node
@param param the model parameter
@returns the upper-left corner of the label's bounding box
@see IEdgeLabelLayoutModel#getLabelPlacement
@see YOrientedRectangle#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getLabelPlacement(IEdgeLabelLayoutModel%2CYDimension%2CIEdgeLayout%2CINodeLayout%2CINodeLayout%2CObject) Online Documentation}*/
static getLabelPlacement(model: IEdgeLabelLayoutModel | null, labelSize: YDimension | null, edgeLayout: IEdgeLayout | null, sourceLayout: INodeLayout | null, targetLayout: INodeLayout | null, param: any | null): YPoint | null;
/**
Returns the upper-left corner of the paraxial bounding box of the given node label as encoded by the specified model
parameter.
The parameter describes the abstract position of the label within the specified model. The parameter must have been
generated by that model.
@param model the {@link INodeLabelLayoutModel} used for placing the label
@param labelSize the size of the label
@param nodeLayout the layout of the node to which the label belongs
@param param the model parameter
@returns the upper-left corner of the label's bounding box
@see INodeLabelLayoutModel#getLabelPlacement
@see YOrientedRectangle#boundingBox
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getLabelPlacement(INodeLabelLayoutModel%2CYDimension%2CINodeLayout%2CObject) Online Documentation}*/
static getLabelPlacement(model: INodeLabelLayoutModel | null, labelSize: YDimension | null, nodeLayout: INodeLayout | null, param: any | null): YPoint | null;
/**
Returns the distance between the centers of the two given nodes.
@param graph the graph to which the nodes belong
@param node1 the first node
@param node2 the second node
@returns the distance between the nodes' centers
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getNodeDistance(LayoutGraph%2CYNode%2CYNode) Online Documentation}*/
static getNodeDistance(graph: LayoutGraph | null, node1: YNode | null, node2: YNode | null): number;
/**
Returns a {@link string} representation of all node positions in the graph.
@param graph the graph
@returns the {@link string} containing all node locations
@see LayoutGraph#getLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getNodePositions(LayoutGraph) Online Documentation}*/
static getNodePositions(graph: LayoutGraph | null): string | null;
/**
Returns the length of the path of the given edge.
The path starts at the source point, traverses through all control points and ends at the target point.
@param graph the graph to which the edge belongs
@param edge the edge
@returns the length of the edge's path
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23getPathLength(LayoutGraph%2CEdge) Online Documentation}*/
static getPathLength(graph: LayoutGraph | null, edge: Edge | null): number;
/**
Checks whether or not the path of an edge is outside the bounding box of the source and target node, considering the
given halo.
The edge is outside the nodes if its control points as well as the source and target point are not inside the nodes'
bounding boxes.
The nodes can be enlarged by a halo to check if the edge keeps some distance to the nodes.
@param edge the edge
@param source the source of the edge
@param target the target of the edge
@param halo the halo for the nodes
@returns <code>true</code> if the edge is outside the nodes, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23isEdgeOutsideNodes(IEdgeLayout%2CINodeLayout%2CINodeLayout%2Cnumber) Online Documentation}*/
static isEdgeOutsideNodes(edge: IEdgeLayout | null, source: INodeLayout | null, target: INodeLayout | null, halo: number): boolean;
/**
Checks whether or not the path of an edge is outside the bounding box of the source and target node.
The edge is outside the nodes if its control points as well as the source and target point are not inside the nodes'
bounding boxes.
@param graph the graph to which the edge belongs
@param edge the edge
@returns <code>true</code> if the edge is outside the nodes, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23isEdgeOutsideNodes(LayoutGraph%2CEdge) Online Documentation}*/
static isEdgeOutsideNodes(graph: LayoutGraph | null, edge: Edge | null): boolean;
/**
Determines whether or not most of the edges of the graph are routed octilinear.
The method only considers edges marked in the given {@link IDataProvider}.
<h2>Note</h2>
This method also returns <code>true</code> for orthogonal layouts.
@param graph the underlying graph
@param considerEdges the edges to consider
@returns <code>true</code> if most of the edges in the graph are routed octilinear, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23isUsingOctilinearEdgeRoutes(LayoutGraph%2CIDataProvider) Online Documentation}*/
static isUsingOctilinearEdgeRoutes(graph: LayoutGraph | null, considerEdges: IDataProvider | null): boolean;
/**
Determines whether or not most of the edges of the graph are routed orthogonally.
The method only considers edges marked in the given {@link IDataProvider}.
@param graph the underlying graph
@param considerEdges the edges to consider
@returns <code>true</code> if most of the edges in the graph are routed orthogonally, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23isUsingOrthogonalEdgeRoutes(LayoutGraph%2CIDataProvider) Online Documentation}*/
static isUsingOrthogonalEdgeRoutes(graph: LayoutGraph | null, considerEdges: IDataProvider | null): boolean;
/**
Moves all control points of the given edge by the vector <code>(dx,dy)</code>.
@param graph the graph
@param edge the edge
@param dx the horizontal distance to move
@param dy the vertical distance to move
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23moveEdge(LayoutGraph%2CEdge%2Cnumber%2Cnumber) Online Documentation}*/
static moveEdge(graph: LayoutGraph | null, edge: Edge | null, dx: number, dy: number): void;
/**
Moves the control points of all edges accessible through the given {@link IEdgeCursor} by the vector <code>(dx,dy)</code>.
@param graph the graph
@param edges the edges to be moved
@param dx the horizontal distance to move
@param dy the vertical distance to move
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23moveEdges(LayoutGraph%2CIEdgeCursor%2Cnumber%2Cnumber) Online Documentation}*/
static moveEdges(graph: LayoutGraph | null, edges: IEdgeCursor | null, dx: number, dy: number): void;
/**
Moves the given node by the vector <code>(dx,dy)</code>.
@param graph the graph
@param node the node
@param dx the horizontal distance to move
@param dy the vertical distance to move
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23moveNode(LayoutGraph%2CYNode%2Cnumber%2Cnumber) Online Documentation}*/
static moveNode(graph: LayoutGraph | null, node: YNode | null, dx: number, dy: number): void;
/**
Moves all nodes accessible through the given {@link INodeCursor} by the vector <code>(dx,dy)</code>.
@param graph the graph
@param nodes the nodes to be moved
@param dx the horizontal distance to move
@param dy the vertical distance to move
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23moveNodes(LayoutGraph%2CINodeCursor%2Cnumber%2Cnumber) Online Documentation}*/
static moveNodes(graph: LayoutGraph | null, nodes: INodeCursor | null, dx: number, dy: number): void;
/**
Moves the subgraph induced by the nodes accessible through the given {@link INodeCursor} by the vector
<code>(dx,dy)</code>.
@param graph the graph that contains the subgraph
@param nodes the nodes that induce the subgraph
@param dx the horizontal distance to move
@param dy the vertical distance to move
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23moveSubgraph(LayoutGraph%2CINodeCursor%2Cnumber%2Cnumber) Online Documentation}*/
static moveSubgraph(graph: LayoutGraph | null, nodes: INodeCursor | null, dx: number, dy: number): void;
/**
Moves the subgraph induced by the edges accessible through the given {@link IEdgeCursor} by the vector
<code>(dx,dy)</code>.
@param graph the graph that contains the subgraph
@param edges the edges that induce the subgraph
@param dx the horizontal distance to move
@param dy the vertical distance to move
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23moveSubgraphOfEdges(LayoutGraph%2CIEdgeCursor%2Cnumber%2Cnumber) Online Documentation}*/
static moveSubgraphOfEdges(graph: LayoutGraph | null, edges: IEdgeCursor | null, dx: number, dy: number): void;
/**
Checks whether or not the path of the given edge intersects the interior of a given rectangular area.
@param graph the graph to which the edge belongs
@param edge the edge
@param rect the rectangular area to check for intersection
@returns <code>true</code> if the edge path intersects with the given rectangle, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23pathIntersectsRect(LayoutGraph%2CEdge%2CRectangle2D) Online Documentation}*/
static pathIntersectsRect(graph: LayoutGraph | null, edge: Edge | null, rect: Rectangle2D | null): boolean;
/**
Removes successive bends that have the same coordinates from the given edge.
@param graph the graph
@param edge the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23removeDuplicateBends(LayoutGraph%2CEdge) Online Documentation}*/
static removeDuplicateBends(graph: LayoutGraph | null, edge: Edge | null): void;
/**
Removes successive bends that have the same coordinates from all edges in the graph.
@param graph the graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23removeDuplicateBends(LayoutGraph) Online Documentation}*/
static removeDuplicateBends(graph: LayoutGraph | null): void;
/**
Assigns a trivial path which has no control points to the specified edge.
@param graph the graph
@param edge the edge
@param [resetPorts=true] <code>true</code> if the ports of the edge should be set to the center of the corresponding nodes, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23resetPath(LayoutGraph%2CEdge%2Cboolean) Online Documentation}*/
static resetPath(graph: LayoutGraph | null, edge: Edge | null, resetPorts?: boolean): void;
/**
Assigns a trivial path which has no control points to all edges in the specified graph.
@param graph the graph
@param [resetPorts=true] <code>true</code> if the ports of the edges should be set to the center of the corresponding nodes, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23resetPaths(LayoutGraph%2Cboolean) Online Documentation}*/
static resetPaths(graph: LayoutGraph | null, resetPorts?: boolean): void;
/**
Sets the ports of all edges in the graph to the center of the nodes.
<h2>Note</h2>
The control points will not be changed.
@param graph the graph
@see LayoutGraph#setSourcePointRel
@see LayoutGraph#setTargetPointRel
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23resetPorts(LayoutGraph) Online Documentation}*/
static resetPorts(graph: LayoutGraph | null): void;
/**
Reverses the layout of an edge.
The order of the control points is reversed and source and target point are swapped.
<h2>Note</h2>
This method will not reverse the edge in the graph structure.
@param el the layout information of an edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23reverseEdgeLayout(IEdgeLayout) Online Documentation}*/
static reverseEdgeLayout(el: IEdgeLayout | null): void;
/**
Rounds the locations of the control points and ports of the given edge to integer values.
@param graph the graph to which the edge belongs
@param edge the edge whose values are rounded
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23roundEdgeLayout(LayoutGraph%2CEdge) Online Documentation}*/
static roundEdgeLayout(graph: LayoutGraph | null, edge: Edge | null): void;
/**
Rounds coordinates and sizes in the layout of an entire graph to integer values.
@param graph the graph
@see LayoutGraphUtilities#roundNodeLayout
@see LayoutGraphUtilities#roundEdgeLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23roundLayout(LayoutGraph) Online Documentation}*/
static roundLayout(graph: LayoutGraph | null): void;
/**
Rounds the location and size of the given node to integer values.
@param graph the graph to which the node belongs
@param node the node whose values are rounded
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23roundNodeLayout(LayoutGraph%2CYNode) Online Documentation}*/
static roundNodeLayout(graph: LayoutGraph | null, node: YNode | null): void;
/**
Routes a list of edges which are incident to the same nodes in parallel, optionally joining the end points.
All edges get a path that is parallel to the path of the first (leading) edge.
The position of the leading edge may be adjusted if there is an odd number of parallel edges. It will be moved from the
center to achieve a more symmetric port assignment.
@param graph the graph
@param leadingEdge the leading edge
@param edges the list of edges that will be aligned with the leading edge
@param lineDistance the distance between two edges
@param [adjustLeadingEdge=false] <code>true</code> if the leading edge should be adjusted if the size of list is odd, <code>false</code> otherwise
@param [joinEnds=false] <code>true</code> if the edges should share their end points (the ones of the leading edge), <code>false</code> otherwise
@param [absJoinDistance=0.0] the absolute distance between the end points and the beginning of the parallel segment routing if the ends are joined
@param [relJoinDistance=0.0] the relative distance, measured relative to the length of the first/last segments if the ends are joined
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23routeEdgesParallel(LayoutGraph%2CEdge%2CEdgeList%2Cnumber%2Cboolean%2Cboolean%2Cnumber%2Cnumber) Online Documentation}*/
static routeEdgesParallel(graph: LayoutGraph, leadingEdge: Edge, edges: EdgeList, lineDistance: number, adjustLeadingEdge?: boolean, joinEnds?: boolean, absJoinDistance?: number, relJoinDistance?: number): void;
/**
Routes a list of edges which are incident to the same nodes in parallel, optionally joining the end points.
All edges get a path that is parallel to the path of the first (leading) edge.
The position of the leading edge may be adjusted if there is an odd number of parallel edges. It will be moved from the
center to achieve a more symmetric port assignment.
@param options The parameters to pass.
@param options.graph the graph
@param options.leadingEdge the leading edge
@param options.edges the list of edges that will be aligned with the leading edge
@param options.lineDistance the distance between two edges
@param [options.adjustLeadingEdge=false] <code>true</code> if the leading edge should be adjusted if the size of list is odd, <code>false</code> otherwise
@param [options.joinEnds=false] <code>true</code> if the edges should share their end points (the ones of the leading edge), <code>false</code> otherwise
@param [options.absJoinDistance=0.0] the absolute distance between the end points and the beginning of the parallel segment routing if the ends are joined
@param [options.relJoinDistance=0.0] the relative distance, measured relative to the length of the first/last segments if the ends are joined
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23routeEdgesParallel(LayoutGraph%2CEdge%2CEdgeList%2Cnumber%2Cboolean%2Cboolean%2Cnumber%2Cnumber) Online Documentation}*/
static routeEdgesParallel(options: {
graph: LayoutGraph;
leadingEdge: Edge;
edges: EdgeList;
lineDistance: number;
adjustLeadingEdge?: boolean;
joinEnds?: boolean;
absJoinDistance?: number;
relJoinDistance?: number;
}): void;
/**
Routes two edges which are incident to the same nodes in parallel, optionally joining the end points.
The second edge gets a path that is parallel to the path of the first (leading) edge.
@param graph the graph
@param leadingEdge the leading edge
@param edge the edge to be adjusted
@param lineDistance the distance between the two edges
@param [joinEnds=false] <code>true</code> if the edges should share their end points (the ones of the leading edge), <code>false</code> otherwise
@param [absJoinDistance=0.0] the absolute distance between the end points and the beginning of the parallel segment routing if the ends are joined
@param [relJoinDistance=0.0] the relative distance, measured relative to the length of the first/last segments if the ends are joined
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23routeEdgesParallel(LayoutGraph%2CEdge%2CEdge%2Cnumber%2Cboolean%2Cnumber%2Cnumber) Online Documentation}*/
static routeEdgesParallel(graph: LayoutGraph, leadingEdge: Edge, edge: Edge, lineDistance: number, joinEnds?: boolean, absJoinDistance?: number, relJoinDistance?: number): void;
/**
Routes two edges which are incident to the same nodes in parallel, optionally joining the end points.
The second edge gets a path that is parallel to the path of the first (leading) edge.
@param options The parameters to pass.
@param options.graph the graph
@param options.leadingEdge the leading edge
@param options.edge the edge to be adjusted
@param options.lineDistance the distance between the two edges
@param [options.joinEnds=false] <code>true</code> if the edges should share their end points (the ones of the leading edge), <code>false</code> otherwise
@param [options.absJoinDistance=0.0] the absolute distance between the end points and the beginning of the parallel segment routing if the ends are joined
@param [options.relJoinDistance=0.0] the relative distance, measured relative to the length of the first/last segments if the ends are joined
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23routeEdgesParallel(LayoutGraph%2CEdge%2CEdge%2Cnumber%2Cboolean%2Cnumber%2Cnumber) Online Documentation}*/
static routeEdgesParallel(options: {
graph: LayoutGraph;
leadingEdge: Edge;
edge: Edge;
lineDistance: number;
joinEnds?: boolean;
absJoinDistance?: number;
relJoinDistance?: number;
}): void;
/**
Routes a self-loop.
For a self-loop edge, source and target are identical. The path of the self-loop will start at the top of the node, go
around the upper-left corner and end at the left side of the node.
@param graph the graph to which the edge belongs
@param edge the self-loop
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphUtilities%23routeSelfLoop(LayoutGraph%2CEdge) Online Documentation}*/
static routeSelfLoop(graph: LayoutGraph | null, edge: Edge | null): void;
static $class: Class;
private $_LayoutGraphUtilities: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutGraphUtilities;
}
export interface LayoutMultiplexer extends LayoutStageBase {
}
/**
A {@link LayoutMultiplexer} delegates its layout calls to a core layout algorithm that is dynamically chosen at
runtime by inspecting the nodes of the input graph.
If a {@link IDataProvider} is registered with the given graph using the look-up key
{@link LayoutMultiplexer#LAYOUT_DP_KEY}, it is used for retrieving the {@link ILayoutAlgorithm} objects that
are stored for the graph's nodes. If no such {@link IDataProvider} is registered, this layout stage's {@link LayoutStageBase#coreLayout core layout algorithm}
(if any) is used for the layout calculation of all nodes.
Note that the first non-<code>null</code> {@link ILayoutAlgorithm} instance retrieved from the {@link IDataProvider}
is used for all nodes.
The main purpose of this class is to enable the use of individual layout algorithms for different components (when used
as the core layout algorithm of {@link ComponentLayout}) or for the contents of group nodes (when used as the core
layout algorithm of {@link RecursiveGroupLayout}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutMultiplexer Online Documentation}*/
export class LayoutMultiplexer {
/**
Creates a new instance of {@link LayoutMultiplexer}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutMultiplexer Online Documentation}*/
constructor();
/**
Retrieves and invokes the {@link ILayoutAlgorithm layout algorithm} that is assigned to the graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutMultiplexer%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Returns the core layout algorithm associated with the nodes contained in the given graph.
This implementation looks up the layout algorithm in the {@link IDataProvider} registered with the key
{@link LayoutMultiplexer#LAYOUT_DP_KEY}. The first non-<code>null</code> {@link ILayoutAlgorithm layout algorithm} instance found for a node of the
given graph will be returned by this method. If no algorithm is registered at all, it will fall back to the
{@link LayoutStageBase#coreLayout default core layout algorithm}.
This method is called in {@link LayoutMultiplexer#applyLayout} and may be overridden to implement a different way
of selecting the {@link ILayoutAlgorithm layout algorithms}.
@param graph the input graph
@returns the according layout algorithm
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutMultiplexer%23getCoreLayout(LayoutGraph) Online Documentation}*/
getCoreLayout(graph: LayoutGraph | null): ILayoutAlgorithm | null;
/**
Data provider key for assigning layout algorithms to the nodes
At runtime, the layout algorithms are determined and applied to the current graph. The first non-<code>null</code> {@link ILayoutAlgorithm}
instance is used for the whole graph.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutMultiplexer%23LAYOUT_DP_KEY Online Documentation}*/
static LAYOUT_DP_KEY: NodeDpKey<ILayoutAlgorithm> | null;
static $class: Class;
private $_LayoutMultiplexer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutMultiplexer;
}
export interface LayoutStageBase extends YObject, ILayoutStage {
}
/**
Abstract base class implementing {@link ILayoutStage} that handles the management of the {@link LayoutStageBase#coreLayout core layout algorithm}.
Subclasses of this stage can use the provided methods, such that the management of the {@link LayoutStageBase#coreLayout core layout algorithm} does not
need to be considered anymore.
@see MultiStageLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutStageBase Online Documentation}*/
export abstract class LayoutStageBase {
/**
Creates a new {@link ILayoutStage} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutStageBase Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Invokes the layout process of the {@link LayoutStageBase#coreLayout core layout algorithm}.
Overriding classes may call this method during {@link ILayoutAlgorithm#applyLayout} to delegate arranging the
graph to the {@link LayoutStageBase#coreLayout core layout algorithm}. They can add pre- and post-processing code before and after this call.
@param graph the input graph
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutStageBase%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Gets or sets the core layout algorithm that is wrapped by this stage.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutStageBase%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
static $class: Class;
private $_LayoutStageBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutStageBase;
}
export interface MinimumNodeSizeStage extends LayoutStageBase {
}
/**
{@link MinimumNodeSizeStage} enforces a given minimum width/height of the nodes of a graph during the layout
process.
It temporarily enlarges nodes whose width/height is below the specified minimum values.
This {@link ILayoutStage} is especially useful to prevent that the {@link LayoutStageBase#coreLayout core layout algorithm} has to handle zero-sized
nodes or nodes with negative sizes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumNodeSizeStage Online Documentation}*/
export class MinimumNodeSizeStage {
/**
Creates a new {@link MinimumNodeSizeStage} instance using the given {@link LayoutStageBase#coreLayout core layout algorithm} along with a specific
minimum width and height for the nodes.
@param coreLayout the core layout routine
@param minWidth the minimum width of nodes that should be enforced
@param minHeight the minimum height of nodes that should be enforced
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumNodeSizeStage%23MinimumNodeSizeStage(ILayoutAlgorithm%2Cnumber%2Cnumber) Online Documentation}*/
constructor(coreLayout: ILayoutAlgorithm | null, minWidth: number, minHeight: number);
/**
Creates a new {@link MinimumNodeSizeStage} instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
The nodes will be enlarged to be at least <code>1</code> wide and <code>1</code> high.
@param coreLayout the core routine
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumNodeSizeStage%23MinimumNodeSizeStage(ILayoutAlgorithm) Online Documentation}*/
constructor(coreLayout: ILayoutAlgorithm | null);
/**
Resizes all nodes that are smaller than the specified minimum size to that minimum size, executes the {@link LayoutStageBase#coreLayout core layout algorithm}
and restores the original sizes afterwards.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumNodeSizeStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
static $class: Class;
private $_MinimumNodeSizeStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MinimumNodeSizeStage;
}
export interface MultiStageLayout extends YObject, ILayoutAlgorithm {
}
/**
An abstract base class for layout algorithms that provides services to simplify and decompose the input graph before it
is passed to the {@link MultiStageLayout#applyLayoutCore core layout algorithm} itself.
Subclasses of this layout algorithm have to provide implementations for the abstract method
{@link MultiStageLayout#applyLayoutCore}. It invokes the actual layout routine of this layout algorithm.
<b>Concept</b>
This class provides a configurable pipeline that contains {@link ILayoutStage}s that add preprocessing steps
and/or postprocessing steps to the {@link MultiStageLayout#applyLayoutCore layout algorithm}. {@link MultiStageLayout} provides some predefined
{@link ILayoutStage}s, that simplify the input graph for the layout algorithm and complement the result. It is
also possible to add custom {@link ILayoutStage}s which are executed {@link MultiStageLayout#prependStage before} or {@link MultiStageLayout#appendStage after the predefined ones}.
Each {@link ILayoutStage} in the pipeline wraps its successor. When the layout pipeline is processed, the
preprocessing code of the preceding {@link ILayoutStage} is executed before the code in the current {@link ILayoutStage}
while the postprocessing code is executed afterwards.
The pipeline of predefined {@link ILayoutStage}s of {@link MultiStageLayout} (when all are activated) causes
the following sequence of steps:
<ol>
<li>Hides the group nodes.
<li>Hides all except for the induced subgraph.
<li>Decomposes the graph into its connected components. Performs steps 4-12 on each component.
<li>Prepares for label placement.
<li>Prepares the layout for an orientation change.
<li>Hides self-loops.
<li>Hides parallel edges.
<li>Invokes the {@link MultiStageLayout#applyLayoutCore layout algorithm} on the simplified graph.
<li>Un-hides and routes parallel edges.
<li>Un-hides and routes self-loops.
<li>Completes orientation change of the layout.
<li>Assigns label positions.
<li>Arranges the connected components of the graph.
<li>Un-hides non-induced subgraph.
<li>Un-hides the group nodes.
</ol>
In order to take effect, a stage that is not already activated by default needs to be activated using the corresponding
method, e.g., {@link MultiStageLayout#hideGroupsStageEnabled}. Vice versa, a stage can also be deactivated, if necessary.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout Online Documentation}*/
export abstract class MultiStageLayout {
/**
Creates a new {@link MultiStageLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link MultiStageLayout#orientationLayoutEnabled} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link MultiStageLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link MultiStageLayout#componentLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout Online Documentation}*/
constructor(options?: {
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
orientationLayoutEnabled?: boolean;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
selfLoopRouterEnabled?: boolean;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
componentLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Appends the given {@link ILayoutStage} to the layout pipeline.
Stages that are added with this method will be invoked at the very end of the layout pipeline. This means after all
prepended {@link ILayoutStage}s, all predefined stages, and all previously appended {@link ILayoutStage}s,
but just before the invocation of the {@link MultiStageLayout#applyLayoutCore layout algorithm}.
@param stage the {@link ILayoutStage} instance to be added
@see MultiStageLayout#removeStage
@see MultiStageLayout#prependStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23appendStage(ILayoutStage) Online Documentation}*/
appendStage(stage: ILayoutStage | null): void;
/**
Calculates a layout for the given graph and applies it directly to the graph.
This method executes all activated {@link ILayoutStage}s in the layout pipeline as well as the {@link MultiStageLayout#applyLayoutCore layout algorithm}.
<b>Warning:</b>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph. This method is not side effect free in the sense that the order of edges or nodes in the input graph may change
during the layout process.
@param graph the input graph
@see MultiStageLayout#appendStage
@see MultiStageLayout#prependStage
@see MultiStageLayout#applyLayoutCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Invokes the core layout algorithm.
This method should be implemented by subclasses in order to perform the layout routine of the layout algorithm.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
abstract applyLayoutCore(graph: LayoutGraph | null): void;
/**
Checks the sizes of the nodes to be non-zero.
@param g The graph to check.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23checkNodeSize(LayoutGraph) Online Documentation}*/
checkNodeSize(g: LayoutGraph | null): void;
/**
Deactivates all predefined {@link ILayoutStage}s so that upon {@link MultiStageLayout#applyLayout} only the
{@link MultiStageLayout#applyLayoutCore layout algorithm} will be executed.
<b>Warning:</b> The {@link ILayoutStage}s prepare the graph for the
{@link MultiStageLayout#applyLayoutCore layout algorithm}, i.e., they can temporarily hide elements. If the layout algorithm cannot handle those elements, e.g.,
group nodes, it may not work well on the input graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23disableAllStages() Online Documentation}*/
disableAllStages(): void;
/**
Prepends the given {@link ILayoutStage} to the layout pipeline.
Stages that are added with this method will be invoked at the very beginning of the layout pipeline. This means before
all previously prepended {@link ILayoutStage}s, all predefined stages, and all appended
{@link ILayoutStage}s.
@param stage the {@link ILayoutStage} instance to be added
@see MultiStageLayout#removeStage
@see MultiStageLayout#appendStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23prependStage(ILayoutStage) Online Documentation}*/
prependStage(stage: ILayoutStage | null): void;
/**
Removes the given {@link ILayoutStage} from the layout pipeline.
This method can only remove {@link ILayoutStage}s that have been previously added using {@link MultiStageLayout#appendStage}
or {@link MultiStageLayout#prependStage}. Predefined {@link ILayoutStage}s can be deactivated separately.
@param stage a {@link ILayoutStage} to be removed from the layout pipeline
@see MultiStageLayout#prependStage
@see MultiStageLayout#appendStage
@see MultiStageLayout#selfLoopRouterEnabled
@see MultiStageLayout#parallelEdgeRouterEnabled
@see MultiStageLayout#orientationLayoutEnabled
@see MultiStageLayout#subgraphLayoutEnabled
@see MultiStageLayout#labelingEnabled
@see MultiStageLayout#hideGroupsStageEnabled
@see MultiStageLayout#componentLayoutEnabled
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23removeStage(ILayoutStage) Online Documentation}*/
removeStage(stage: ILayoutStage | null): void;
/**
Gets or sets the {@link ILayoutStage} that places the labels of the input graph.
<h2>Note</h2>
This {@link ILayoutStage} needs to be {@link MultiStageLayout#labelingEnabled activated} in order to take effect.
@see MultiStageLayout#labelingEnabled
@see GenericLabeling
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23labeling Online Documentation}*/
labeling: ILayoutStage | null;
/**
Gets or sets the {@link ILayoutStage} that routes self-loops.
<h2>Note</h2>
This {@link ILayoutStage} needs to be {@link MultiStageLayout#selfLoopRouterEnabled activated} in order to take effect.
<b>Warning:</b>
The default implementation temporarily removes all self-loops during core layout and routes them when inserting them
again afterwards. Custom implementations should also hide the self-loops because the core layouter might not be able to
handle them.
@see MultiStageLayout#selfLoopRouterEnabled
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23selfLoopRouter Online Documentation}*/
selfLoopRouter: ILayoutStage | null;
/**
Gets or sets the {@link ILayoutStage} that routes parallel edges.
<h2>Note</h2>
This {@link ILayoutStage} needs to be {@link MultiStageLayout#parallelEdgeRouterEnabled activated} in order to take effect.
<b>Warning:</b>
The default implementation temporarily removes all parallel edges during core layout and routes them when inserting them
again afterwards. Custom implementations should also hide the parallel edges because the core layouter might not be able
to handle them.
@see MultiStageLayout#parallelEdgeRouterEnabled
@see ParallelEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23parallelEdgeRouter Online Documentation}*/
parallelEdgeRouter: ILayoutStage | null;
/**
Gets or sets the {@link ILayoutStage} that arranges the connected components of an input graph.
<h2>Note</h2>
This {@link ILayoutStage} needs to be {@link MultiStageLayout#componentLayoutEnabled activated} in order to take effect.
<b>Warning:</b> The default implementation passes all connected components to the {@link MultiStageLayout#applyLayoutCore layout algorithm} separately. After all components are
layouted, it arranges them in relation to each other. Custom implementations should respect that the layout algorithm
may not be able to handle multiple connected components.
@see MultiStageLayout#componentLayoutEnabled
@see ComponentLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23componentLayout Online Documentation}*/
componentLayout: ILayoutStage | null;
/**
Gets or sets the {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph.
<h2>Note</h2>
This {@link ILayoutStage} needs to be {@link MultiStageLayout#subgraphLayoutEnabled activated} in order to take effect.
@see MultiStageLayout#subgraphLayoutEnabled
@see SubgraphLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23subgraphLayout Online Documentation}*/
subgraphLayout: ILayoutStage | null;
/**
Gets or sets the {@link ILayoutStage} that hides the group nodes of the input graph.
<h2>Note</h2>
This {@link ILayoutStage} needs to be {@link MultiStageLayout#hideGroupsStageEnabled activated} in order to take effect.
<b>Warning:</b>
The default implementation temporarily removes all group nodes during core layout and resizes them when inserting them
again afterwards. Custom implementations should also hide the group nodes because the core layouter might not be able to
handle them.
@see MultiStageLayout#hideGroupsStageEnabled
@see HideGroupsStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23hideGroupsStage Online Documentation}*/
hideGroupsStage: ILayoutStage | null;
/**
Gets or sets the {@link ILayoutStage} that modifies the orientation of a computed layout.
<h2>Note</h2>
In order to configure this {@link ILayoutStage} using {@link MultiStageLayout#layoutOrientation}, it is necessary that an instance of {@link OrientationLayout}
is used.
<h2>Note</h2>
This {@link ILayoutStage} needs to be {@link MultiStageLayout#orientationLayoutEnabled activated} in order to take effect.
@see MultiStageLayout#orientationLayoutEnabled
@see MultiStageLayout#layoutOrientation
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23orientationLayout Online Documentation}*/
orientationLayout: ILayoutStage | null;
/**
Gets or sets whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated.
This {@link ILayoutStage} may be deactivated if the orientation is irrelevant for the layout or if the core layout
algorithm already handles the layout orientation by itself.
<h2>Note</h2>
If the current layout orientation is
{@link LayoutOrientation#TOP_TO_BOTTOM}, the orientation of the layout will not be modified.
@see MultiStageLayout#orientationLayout
@see MultiStageLayout#layoutOrientation
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23orientationLayoutEnabled Online Documentation}*/
orientationLayoutEnabled: boolean;
/**
Gets or sets the main orientation of the layout.
This is a convenience method that configures the {@link MultiStageLayout#orientationLayout orientation layout stage}.
<b>Warning:</b> This method will take effect only if the layout stage that modifies the orientation is an instance of
{@link OrientationLayout}. It has to be overridden to support {@link ILayoutStage}s that do not extend
{@link OrientationLayout}.
@throws {Exception} <b>(ArgumentError)</b> if the specified orientation does not match a default layout orientation
@see MultiStageLayout#orientationLayout
@see MultiStageLayout#orientationLayoutEnabled
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23layoutOrientation Online Documentation}*/
layoutOrientation: LayoutOrientation;
/**
Gets or sets whether or not the {@link ILayoutStage} used for routing self-loops is activated.
This {@link ILayoutStage} should be deactivated if the {@link MultiStageLayout#applyLayoutCore layout algorithm} handles self-loops by itself.
@see MultiStageLayout#selfLoopRouter
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23selfLoopRouterEnabled Online Documentation}*/
selfLoopRouterEnabled: boolean;
/**
Gets or sets whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated.
This {@link ILayoutStage} may be activated to apply a generic labeling algorithm to the input graph. It will then
try to find the best locations for the labels in the layout calculated by the
{@link MultiStageLayout#applyLayoutCore layout algorithm}. When the stage is deactivated, the labels are ignored or placed by an integrated labeling of the
layout algorithm.
@see MultiStageLayout#labeling
@see GenericLabeling
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23labelingEnabled Online Documentation}*/
labelingEnabled: boolean;
/**
Gets or sets whether or not the {@link ILayoutStage} used for hiding group nodes is activated.
This {@link ILayoutStage} should be deactivated if the {@link MultiStageLayout#applyLayoutCore layout algorithm} handles group nodes itself.
@see MultiStageLayout#hideGroupsStage
@see HideGroupsStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23hideGroupsStageEnabled Online Documentation}*/
hideGroupsStageEnabled: boolean;
/**
Gets or sets whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated.
This {@link ILayoutStage} should be deactivated if the {@link MultiStageLayout#applyLayoutCore layout algorithm} is able to handle several connected
components.
@see MultiStageLayout#componentLayout
@see ComponentLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23componentLayoutEnabled Online Documentation}*/
componentLayoutEnabled: boolean;
/**
Gets or sets whether or not the {@link ILayoutStage} used for routing parallel edges is activated.
This {@link ILayoutStage} should be deactivated if the {@link MultiStageLayout#applyLayoutCore layout algorithm} handles parallel edges itself.
@see MultiStageLayout#parallelEdgeRouter
@see ParallelEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23parallelEdgeRouterEnabled Online Documentation}*/
parallelEdgeRouterEnabled: boolean;
/**
Gets or sets whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the
input graph is activated.
This {@link ILayoutStage} may be activated if the {@link MultiStageLayout#applyLayoutCore layout algorithm} should be applied only to a specific part of
the graph. The remaining graph will stay unchanged.
@see MultiStageLayout#subgraphLayout
@see SubgraphLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiStageLayout%23subgraphLayoutEnabled Online Documentation}*/
subgraphLayoutEnabled: boolean;
static $class: Class;
private $_MultiStageLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MultiStageLayout;
}
export interface NodeHalo extends YObject {
}
/**
A {@link NodeHalo} specifies a rectangular area around a specific node.
A {@link ILayoutAlgorithm layout algorithm} that supports
{@link NodeHalo}s, keeps this area clear of graph elements, except the node labels of this specific node and the
adjacent segments of its edges. All minimum distances to this node, which are used in the layout calculation, e.g.,
minimum first segment length, will be extended by the halo size.
This class defines a key used for registering a {@link IDataProvider} that provides {@link NodeHalo}s
for all nodes of the input graph. {@link NodeHalo}-aware layout algorithms like {@link HierarchicLayout}
look for {@link IDataProvider}s registered with this key to retrieve {@link NodeHalo} information.
The following layouters support {@link NodeHalo}s with different restrictions:
<ul>
<li>{@link HierarchicLayout}
<li>{@link OrthogonalLayout}
<li>{@link CompactOrthogonalLayout}
<li>{@link TreeLayout}
<li>{@link ClassicTreeLayout}
<li>{@link OrganicLayout}
<ul>
<li>if the scope is set to {@link OrganicLayoutScope#SUBSET} or {@link OrganicLayoutScope#MAINLY_SUBSET}, there might be some
overlapping nodes and {@link NodeHalo}s
<li>if node overlaps are allowed, {@link NodeHalo}s may also overlap
<li>edges do not consider {@link NodeHalo}s
</ul>
<li>{@link GenericLabeling}
<ul>
<li>if there is not enough space, labels may overlap with {@link NodeHalo}s
</ul>
<li>{@link EdgeRouter}
<ul>
<li>other constraints (e.g. {@link PortConstraint}s) with higher costs may cause edges to cross
{@link NodeHalo}s
</ul>
<li>{@link CircularLayout}
<ul>
<li>{@link NodeHalo} overlaps may occur if the internal {@link BalloonLayout} {@link BalloonLayout#allowOverlaps allows overlaps}
<li>edges do not consider {@link NodeHalo}s
</ul>
<li>{@link BalloonLayout}
<ul>
<li>{@link NodeHalo} overlaps may occur, if {@link BalloonLayout#allowOverlaps} is enabled
<li>edges do not consider {@link NodeHalo}s
</ul>
<li>{@link SelfLoopRouter}
<ul>
<li>if there is not enough space, self-loops may overlap with {@link NodeHalo}s
</ul>
<li>{@link RadialLayout}
<ul>
<li>non-tree edges do not consider {@link NodeHalo}s
</ul>
<li>{@link RecursiveGroupLayout}
<li>{@link ComponentLayout}
</ul>
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo Online Documentation}*/
export class NodeHalo {
/**
Creates a {@link NodeHalo} with the specified values.
The values must not be negative, {@link Number#NaN} or {@link Number#POSITIVE_INFINITY}.
@param top the {@link NodeHalo} size at the top side of the node
@param left the {@link NodeHalo} size at the left side of the node
@param bottom the {@link NodeHalo} size at the bottom side of the node
@param right the {@link NodeHalo} size at the right side of the node
@returns the newly created {@link NodeHalo}
@throws {Exception} <b>(ArgumentError)</b> if one of the values is negative, {@link Number#NaN} or {@link Number#POSITIVE_INFINITY}
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23create(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static create(top: number, left: number, bottom: number, right: number): NodeHalo | null;
/**
Creates a {@link NodeHalo} object with the specified value on each side of the node.
The value must not be negative, {@link Number#NaN} or {@link Number#POSITIVE_INFINITY}.
@param value the halo size at each side of the node
@returns the newly created {@link NodeHalo}
@throws {Exception} <b>(ArgumentError)</b> if the value is negative, {@link Number#NaN} or {@link Number#POSITIVE_INFINITY}
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23create(number) Online Documentation}*/
static create(value: number): NodeHalo | null;
/**
Determines whether another object is equal to this one.
@param o the other object to compare for equality
@returns <code>true</code> if <code>o</code> is considered equal to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23equals(Object) Online Documentation}*/
equals(o: any | null): boolean;
/**
Returns the {@link NodeHalo} associated with the given node.
If no such {@link NodeHalo} is available, this method returns a {@link NodeHalo#ZERO_HALO}.
@param graph the graph containing the given node
@param node the node whose {@link NodeHalo} will be determined
@returns the {@link NodeHalo} of the given node if available, otherwise a {@link NodeHalo#ZERO_HALO}
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23getHalo(LayoutGraph%2CYNode) Online Documentation}*/
static getHalo(graph: LayoutGraph | null, node: YNode | null): NodeHalo | null;
/**
Returns a {@link YRectangle} instance with the bounds of the given node including its {@link NodeHalo}.
This {@link YRectangle} is located at the position of the node.
If the node does not have a {@link NodeHalo} assigned, this box has the same size as the node.
@param graph the graph containing the given node
@param node the node whose {@link NodeHalo} will be determined
@returns a {@link YRectangle} instance with the bounds of the given node including its {@link NodeHalo}
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23getHaloBox(LayoutGraph%2CYNode) Online Documentation}*/
static getHaloBox(graph: LayoutGraph | null, node: YNode | null): YRectangle | null;
/**
Checks whether or not the given graph contains {@link NodeHalo} information.
This method looks for a {@link IDataProvider} that has been registered with the given graph using
{@link NodeHalo#NODE_HALO_DP_KEY}.
@param graph the input graph
@returns <code>true</code> if a {@link IDataProvider} has been registered with the according key, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23hasHalos(LayoutGraph) Online Documentation}*/
static hasHalos(graph: LayoutGraph | null): boolean;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Data provider key for assigning halos for the nodes
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23NODE_HALO_DP_KEY Online Documentation}*/
static NODE_HALO_DP_KEY: NodeDpKey<NodeHalo> | null;
/**
A constant holding a {@link NodeHalo} with zero size on each side.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23ZERO_HALO Online Documentation}*/
static ZERO_HALO: NodeHalo | null;
/**
The {@link NodeHalo} size at the top side of the node.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23top Online Documentation}*/
top: number;
/**
The {@link NodeHalo} size at the left side of the node.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23left Online Documentation}*/
left: number;
/**
The {@link NodeHalo} size at the bottom side of the node.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23bottom Online Documentation}*/
bottom: number;
/**
The {@link NodeHalo} size at the right side of the node.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeHalo%23right Online Documentation}*/
right: number;
static $class: Class;
private $_NodeHalo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeHalo;
}
export interface NodeLabelCandidate extends LabelCandidate {
}
/**
A {@link NodeLabelCandidate} describes one valid placement for a node label considering the label model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelCandidate Online Documentation}*/
export class NodeLabelCandidate {
/**
Creates a new instance of {@link NodeLabelCandidate} described by location, size, model parameter and internal
flag.
@param pos the location of the upper left corner of the candidate
@param size the size of the candidate
@param param the parameters of the label model associated with this candidate
@param owner the label associated with the candidate
@param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelCandidate%23NodeLabelCandidate(YPoint%2CYDimension%2CObject%2CINodeLabelLayout%2Cboolean) Online Documentation}*/
constructor(pos: YPoint | null, size: YDimension | null, param: any | null, owner: INodeLabelLayout | null, internal?: boolean);
/**
Creates a new instance of {@link NodeLabelCandidate} described by its box, model parameter and internal flag.
@param labelBox the box that specifies the candidate's size and position
@param param the parameters of the label model associated with this candidate
@param owner the label associated with the candidate
@param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelCandidate%23NodeLabelCandidate(YOrientedRectangle%2CObject%2CINodeLabelLayout%2Cboolean) Online Documentation}*/
constructor(labelBox: YOrientedRectangle | null, param: any | null, owner: INodeLabelLayout | null, internal?: boolean);
static $class: Class;
private $_NodeLabelCandidate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeLabelCandidate;
}
export interface NormalizeGraphElementOrderStage extends LayoutStageBase {
}
/**
{@link NormalizeGraphElementOrderStage} normalizes the order of the elements within a graph to ensure the same
order for multiple layout invocations.
Among other factors, the results produced by layout algorithms usually depend on the order of the nodes and edges within
a graph. Unfortunately, useful operations such as hiding or unhiding elements from a graph or simply invoking layout
algorithms on a graph will have the potential side effect of changing that order.
With this {@link ILayoutStage} it is possible to establish a predefined order of nodes and edges within a graph to
avoid non-deterministic layout behavior. Defining the order of nodes and edges is done by associating each node or edge
in the graph with a {@link IComparable} value using a {@link IDataProvider} registered with the key {@link NormalizeGraphElementOrderStage#NODE_COMPARABLE_DP_KEY}
or {@link NormalizeGraphElementOrderStage#EDGE_COMPARABLE_DP_KEY}, respectively.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NormalizeGraphElementOrderStage Online Documentation}*/
export class NormalizeGraphElementOrderStage {
/**
Creates a new {@link NormalizeGraphElementOrderStage} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayouter=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NormalizeGraphElementOrderStage Online Documentation}*/
constructor(coreLayouter?: ILayoutAlgorithm | null);
/**
Enforces the specified {@link NormalizeGraphElementOrderStage#NODE_COMPARABLE_DP_KEY order of nodes} and {@link NormalizeGraphElementOrderStage#EDGE_COMPARABLE_DP_KEY order of edges} before invoking the {@link LayoutStageBase#coreLayout core layout algorithm}.
@param graph the input graph
@throws {Exception} <b>(InvalidOperationError)</b> if there is no {@link IDataProvider} registered for one of the keys {@link NormalizeGraphElementOrderStage#NODE_COMPARABLE_DP_KEY}
and {@link NormalizeGraphElementOrderStage#EDGE_COMPARABLE_DP_KEY}
@see NormalizeGraphElementOrderStage#NODE_COMPARABLE_DP_KEY
@see NormalizeGraphElementOrderStage#EDGE_COMPARABLE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/NormalizeGraphElementOrderStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Assigns comparable values for each node and edge.
The compared values are induced from the current ordering of the nodes and edges within the given graph.
This is a convenient method to fill the given {@link IDataMap}s which can then be registered with the graph
with keys {@link NormalizeGraphElementOrderStage#NODE_COMPARABLE_DP_KEY} and {@link NormalizeGraphElementOrderStage#EDGE_COMPARABLE_DP_KEY}
and used for several layout calculations.
@param graph the input graph
@param comparableNodeMap the map that will be filled with comparable values for the nodes
@param comparableEdgeMap the map that will be filled with comparable values for the edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/NormalizeGraphElementOrderStage%23fillComparableMapFromGraph(Graph%2CIDataMap%2CIDataMap) Online Documentation}*/
static fillComparableMapFromGraph(graph: Graph | null, comparableNodeMap: IDataMap | null, comparableEdgeMap: IDataMap | null): void;
/**
Data provider key for defining a (global) order for nodes
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NormalizeGraphElementOrderStage%23NODE_COMPARABLE_DP_KEY Online Documentation}*/
static NODE_COMPARABLE_DP_KEY: NodeDpKey<IComparable> | null;
/**
Data provider key for defining a (global) order for edges
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NormalizeGraphElementOrderStage%23EDGE_COMPARABLE_DP_KEY Online Documentation}*/
static EDGE_COMPARABLE_DP_KEY: EdgeDpKey<IComparable> | null;
static $class: Class;
private $_NormalizeGraphElementOrderStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NormalizeGraphElementOrderStage;
}
/**
Enum constants that specify the initial placement for the {@link ClassicOrganicLayout}.
@see ClassicOrganicLayout#initialPlacement
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/InitialPlacement Online Documentation}*/
export enum InitialPlacement {
/**
Initial placement strategy for starting with randomly chosen node locations.
@see ClassicOrganicLayout#initialPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/InitialPlacement%23RANDOM Online Documentation}*/
RANDOM,
/**
Initial placement strategy for starting with all node location coordinates set to <code>0</code>.
This strategy is best if the initial placement should be maximally neutral, ignoring the existing layout of the input
graph.
@see ClassicOrganicLayout#initialPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/InitialPlacement%23ZERO Online Documentation}*/
ZERO,
/**
Initial placement strategy for starting with the node locations as they are in the input graph.
This strategy is best if the existing layout should change as little as possible.
@see ClassicOrganicLayout#initialPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/InitialPlacement%23AS_IS Online Documentation}*/
AS_IS
}
/** Automatic conversion to {@link InitialPlacement}. */
type InitialPlacementStringValues = "random" | "zero" | "as-is";
/**
Enum constants that specify how groups are handled by the {@link OrganicLayout}.
@see ClassicOrganicLayout#groupNodePolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupsPolicy Online Documentation}*/
export enum GroupsPolicy {
/**
Group handling policy for considering and arranging the nodes contained in a group such that their togetherness is
highlighted.
<h2>Note</h2>
Groups whose content should not be arranged but kept fixed can be marked by registering a {@link IDataProvider}
with key {@link ClassicOrganicLayout#FIXED_GROUP_NODES_DP_KEY}.
@see ClassicOrganicLayout#groupNodePolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupsPolicy%23LAYOUT Online Documentation}*/
LAYOUT,
/**
Group handling policy for keeping the relative positions of nodes that belong to the same group fixed.
The fixed groups will be taken into account when placing the other nodes of the graph.
By registering a {@link IDataProvider} with key
{@link ClassicOrganicLayout#FIXED_GROUP_NODES_DP_KEY}, the individual group nodes can be marked as fixed or not
fixed. If there is no data provider registered with the mentioned key, then all group nodes of the graph will be treated
as fixed.
@see ClassicOrganicLayout#groupNodePolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupsPolicy%23FIXED Online Documentation}*/
FIXED,
/**
Group handling policy for completely ignoring group nodes during the layout process.
@see ClassicOrganicLayout#groupNodePolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupsPolicy%23IGNORE Online Documentation}*/
IGNORE
}
/** Automatic conversion to {@link GroupsPolicy}. */
type GroupsPolicyStringValues = "layout" | "fixed" | "ignore";
/**
Enum constants that specify the scope for the {@link OrganicLayout} and {@link ClassicOrganicLayout}.
@see ClassicOrganicLayout#scope
@see OrganicLayout#scope
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutScope Online Documentation}*/
export enum OrganicLayoutScope {
/**
Scope mode indicating that the algorithm should place <em>all</em> nodes of the graph.
@see ClassicOrganicLayout#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutScope%23ALL Online Documentation}*/
ALL,
/**
Scope mode indicating that the algorithm should <em>mainly</em> place a subset of nodes.
The subset is determined by a boolean value returned by a {@link IDataProvider} registered with key
{@link ClassicOrganicLayout#AFFECTED_NODES_DP_KEY}. Nodes that don't belong to the subset may be moved to a
certain degree but will keep their relative positions to the other nodes outside the subset.
@see ClassicOrganicLayout#scope
@see ClassicOrganicLayout#AFFECTED_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutScope%23MAINLY_SUBSET Online Documentation}*/
MAINLY_SUBSET,
/**
Scope mode indicating that the algorithm should <em>only</em> place a subset of nodes.
The subset is determined by a boolean value returned by a {@link IDataProvider} registered with key
{@link ClassicOrganicLayout#AFFECTED_NODES_DP_KEY}. Nodes not being part of the subset will remain at their
location.
@see ClassicOrganicLayout#scope
@see ClassicOrganicLayout#AFFECTED_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutScope%23SUBSET Online Documentation}*/
SUBSET
}
/** Automatic conversion to {@link OrganicLayoutScope}. */
type OrganicLayoutScopeStringValues = "all" | "mainly-subset" | "subset";
/**
Enum constants that specify the Horizontal overlap criterion for the {@link ShuffleLayout}.
@see ShuffleLayout#horizontalOverlapCriterion
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HorizontalOverlapCriterion Online Documentation}*/
export enum HorizontalOverlapCriterion {
/**
Horizontal overlap criterion defining an overlap as horizontal if the overlapping area is greater in height than in
width.
Otherwise, if the overlap area's width is greater than or equal to its height, an overlap will be categorized as
vertical.
The area of an overlap is defined as the rectangle where two nodes intersect with each other.
@see ShuffleLayout#horizontalOverlapCriterion
@see {@link https://docs.yworks.com/yfileshtml/#/api/HorizontalOverlapCriterion%23INTERSECTION_BOX Online Documentation}*/
INTERSECTION_BOX,
/**
Horizontal overlap criterion categorizing an overlap as horizontal if the center-to-center difference between the
overlapping nodes is greater in horizontal direction (x-coordinates) than in vertical direction (y-coordinates).
Otherwise, if the center-to-center difference between two overlapping nodes is greater in vertical direction
(y-coordinates), the corresponding overlap is categorized as vertical. The same applies if the differences in vertical
and horizontal direction are equal.
@see ShuffleLayout#horizontalOverlapCriterion
@see {@link https://docs.yworks.com/yfileshtml/#/api/HorizontalOverlapCriterion%23NODE_CENTER Online Documentation}*/
NODE_CENTER,
/**
Horizontal overlap criterion categorizing an overlap as horizontal if the required movement for solving the overlap is
shorter in horizontal direction than in vertical direction.
Otherwise, an overlap will be categorized as vertical.
This criterion tries to avoid moving nodes too much because the direction for resolving overlaps will be chosen such
that the shorter movement is preferred.
@see ShuffleLayout#horizontalOverlapCriterion
@see {@link https://docs.yworks.com/yfileshtml/#/api/HorizontalOverlapCriterion%23LESS_MOVEMENT Online Documentation}*/
LESS_MOVEMENT
}
/** Automatic conversion to {@link HorizontalOverlapCriterion}. */
type HorizontalOverlapCriterionStringValues = "intersection-box" | "node-center" | "less-movement";
/**
@see OrganicLayout#chainSubstructureStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructureStyle Online Documentation}*/
export enum ChainSubstructureStyle {
/**
Substructure style specifier for chains that indicates that the algorithm does not handle such structures in a special
way.
@see OrganicLayout#chainSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructureStyle%23NONE Online Documentation}*/
NONE,
/**
Substructure style specifier for chains that leads to a compact (rotated) rectangular layout style for chains.
@see OrganicLayout#chainSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructureStyle%23RECTANGULAR Online Documentation}*/
RECTANGULAR,
/**
Substructure style specifier for chains that leads to a straight-line layout style for chains.
@see OrganicLayout#chainSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructureStyle%23STRAIGHT_LINE Online Documentation}*/
STRAIGHT_LINE
}
/** Automatic conversion to {@link ChainSubstructureStyle}. */
type ChainSubstructureStyleStringValues = "none" | "rectangular" | "straight-line";
/**
@see OrganicLayout#starSubstructureStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructureStyle Online Documentation}*/
export enum StarSubstructureStyle {
/**
Substructure style specifier for stars that indicates that the algorithm does not handle such structures in a special
way.
@see OrganicLayout#starSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructureStyle%23NONE Online Documentation}*/
NONE,
/**
Substructure style specifier for stars that leads to a radial layout style for stars where the root is placed in the
middle.
@see OrganicLayout#starSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructureStyle%23RADIAL Online Documentation}*/
RADIAL,
/**
Substructure style specifier for stars that leads to a radial layout style for stars where the root is placed on the
outside (separated from the nodes with degree one).
<h2>Note</h2>
If the edges are grouped at the root node (see {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY} and
{@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), the algorithm uses a grouped routing style.
@see OrganicLayout#starSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructureStyle%23SEPARATED_RADIAL Online Documentation}*/
SEPARATED_RADIAL,
/**
Substructure style specifier for stars that leads to a circular layout style for stars where the root is placed in the
middle.
In contrast to the radial layout style, all nodes (except the root) are placed on a single cycle.
@see OrganicLayout#starSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructureStyle%23CIRCULAR Online Documentation}*/
CIRCULAR
}
/** Automatic conversion to {@link StarSubstructureStyle}. */
type StarSubstructureStyleStringValues = "none" | "radial" | "separated-radial" | "circular";
/**
@see OrganicLayout#parallelSubstructureStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelSubstructureStyle Online Documentation}*/
export enum ParallelSubstructureStyle {
/**
Substructure style specifier for parallel structures that indicates that the algorithm does not handle such structures
in a special way.
@see OrganicLayout#parallelSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelSubstructureStyle%23NONE Online Documentation}*/
NONE,
/**
Substructure style specifier for parallel structures that leads to a (rotated) rectangular layout style for the inner
nodes.
<h2>Note</h2>
If the edges are grouped at the outer nodes (see {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY} and
{@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), the algorithm uses a grouped routing style.
@see OrganicLayout#parallelSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelSubstructureStyle%23RECTANGULAR Online Documentation}*/
RECTANGULAR,
/**
Substructure style specifier for parallel structures that leads to a radial layout style for the inner nodes.
<h2>Note</h2>
If the edges are grouped at the outer nodes (see {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY} and
{@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), the algorithm uses a grouped routing style.
@see OrganicLayout#parallelSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelSubstructureStyle%23RADIAL Online Documentation}*/
RADIAL,
/**
Substructure style specifier for parallel structures that places the inner nodes on a straight-line.
<h2>Note</h2>
If the edges are grouped at the outer nodes (see {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY} and
{@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), the algorithm uses a grouped routing style.
@see OrganicLayout#parallelSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelSubstructureStyle%23STRAIGHT_LINE Online Documentation}*/
STRAIGHT_LINE
}
/** Automatic conversion to {@link ParallelSubstructureStyle}. */
type ParallelSubstructureStyleStringValues = "none" | "rectangular" | "radial" | "straight-line";
/**
@see OrganicLayout#cycleSubstructureStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructureStyle Online Documentation}*/
export enum CycleSubstructureStyle {
/**
Substructure style specifier for cycle structures that indicates that the algorithm does not handle such structures in a
special way.
@see OrganicLayout#cycleSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructureStyle%23NONE Online Documentation}*/
NONE,
/**
Substructure style specifier for cycle structures that leads to a circular layout style.
@see OrganicLayout#cycleSubstructureStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructureStyle%23CIRCULAR Online Documentation}*/
CIRCULAR
}
/** Automatic conversion to {@link CycleSubstructureStyle}. */
type CycleSubstructureStyleStringValues = "none" | "circular";
/**
Enum constants that specify how group nodes are handled by the {@link OrganicLayout}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupNodeMode Online Documentation}*/
export enum GroupNodeMode {
/**
A mode constant for placing and resizing the group nodes and their content by the layout algorithm.
The size of the group is adjusted to the space that is needed by the placement of its contained nodes.
@see OrganicLayout#GROUP_NODE_MODE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupNodeMode%23NORMAL Online Documentation}*/
NORMAL,
/**
A mode constant for placing the group nodes and their content with respect to the original bounds of the group node.
The bounds of the group nodes are treated as fixed and cannot be exceeded by the content of the group nodes.
<b>Warning:</b> If property {@link OrganicLayout#considerNodeLabels} is enabled, property {@link OrganicLayout#nodeOverlapsAllowed} is disabled or if there are
{@link NodeHalo}s, the bounds restriction may be violated because the algorithm has to adjust the graph elements
such that there are no overlaps.
@see OrganicLayout#GROUP_NODE_MODE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupNodeMode%23FIX_BOUNDS Online Documentation}*/
FIX_BOUNDS,
/**
A mode constant for placing group nodes with fixed content.
Although these particular group nodes can move like all other ordinary group nodes, their content remains fixed relative
to the position of the group node.
<h2>Note</h2>
This implicitly fixes all descendants of the group node.
<b>Warning:</b> If property {@link OrganicLayout#considerNodeLabels} is enabled, property {@link OrganicLayout#nodeOverlapsAllowed} is disabled or if there are
{@link NodeHalo}s, the bounds restriction may be violated because the algorithm has to adjust the graph elements
such that there are no overlaps.
@see OrganicLayout#GROUP_NODE_MODE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupNodeMode%23FIX_CONTENTS Online Documentation}*/
FIX_CONTENTS
}
/** Automatic conversion to {@link GroupNodeMode}. */
type GroupNodeModeStringValues = "normal" | "fix-bounds" | "fix-contents";
export interface OrientationLayout extends LayoutStageBase {
}
/**
{@link OrientationLayout} is a {@link ILayoutStage} that changes the orientation of the layout.
<b>Layout Style</b>
There are four orientations that can be used for a layout. These orientations can exist with or without mirroring at the
x-axis or y-axis.
Changing the layout orientation is useful for layout styles that show a main direction.
<b>Concept</b>
{@link OrientationLayout} performs three basic steps.
<ol>
<li>Preparation: The sizes and locations of nodes and edge bends as well as labels and {@link PortConstraint}s are
transformed to the {@link LayoutOrientation#TOP_TO_BOTTOM} orientation.
<li>Delegation: The {@link LayoutStageBase#coreLayout core layout algorithm} is invoked and calculates the layout for orientation
{@link LayoutOrientation#TOP_TO_BOTTOM}.
<li>Completion: The result of the {@link LayoutStageBase#coreLayout core layout algorithm} is rotated and mirrored to fit the desired layout orientation.
</ol>
<b>Features</b>
{@link OrientationLayout} can either be used as a {@link ILayoutStage} wrapping a {@link ILayoutAlgorithm layout algorithm}. Then it
will change the orientation of the result of the {@link LayoutStageBase#coreLayout core layout algorithm}.
If no {@link LayoutStageBase#coreLayout core layout algorithm} is specified, {@link OrientationLayout} can work alone. It will just change the
orientation of the layout without arranging nodes and edges.
<b>Warning:</b> For subclasses of {@link MultiStageLayout}s it is highly recommended to use {@link MultiStageLayout#orientationLayout} for layout orientation changes and
not to wrap {@link MultiStageLayout} with an {@link OrientationLayout} instance. That may cause unwanted
artifacts.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout Online Documentation}*/
export class OrientationLayout {
/**
Creates a new {@link OrientationLayout} instance using the given orientation.
@param orientation the orientation specifier
@throws {Exception} <b>(ArgumentError)</b> if the specified orientation is unknown
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23OrientationLayout(LayoutOrientation) Online Documentation}*/
constructor(orientation: LayoutOrientation | LayoutOrientationStringValues);
/**
Creates a new {@link OrientationLayout} instance using the given orientation.
@param options The parameters to pass.
@param options.orientation the orientation specifier
@param [options.considerEdgeLabels] Whether or not the position of the edge labels should be changed during orientation change. This option sets the {@link OrientationLayout#considerEdgeLabels} property on the created object.
@param [options.horizontalOrientation] Whether or not the current orientation is horizontal. This option sets the {@link OrientationLayout#horizontalOrientation} property on the created object.
@param [options.mirrorMode] The mirror mask that defines which orientations this {@link OrientationLayout} shall mirror. This option sets the {@link OrientationLayout#mirrorMode} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the specified orientation is unknown
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23OrientationLayout(LayoutOrientation) Online Documentation}*/
constructor(options: {
orientation: LayoutOrientation;
considerEdgeLabels?: boolean;
horizontalOrientation?: boolean;
mirrorMode?: MirrorModes | MirrorModesConvertible;
});
/**
Creates a new {@link OrientationLayout} instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] the core layout routine
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23OrientationLayout(ILayoutAlgorithm) Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link OrientationLayout} instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] the core layout routine
@param [options.considerEdgeLabels] Whether or not the position of the edge labels should be changed during orientation change. This option sets the {@link OrientationLayout#considerEdgeLabels} property on the created object.
@param [options.horizontalOrientation] Whether or not the current orientation is horizontal. This option sets the {@link OrientationLayout#horizontalOrientation} property on the created object.
@param [options.mirrorMode] The mirror mask that defines which orientations this {@link OrientationLayout} shall mirror. This option sets the {@link OrientationLayout#mirrorMode} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23OrientationLayout(ILayoutAlgorithm) Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
considerEdgeLabels?: boolean;
horizontalOrientation?: boolean;
mirrorMode?: MirrorModes | MirrorModesConvertible;
});
/**
Prepares the graph for orientation change before calling the {@link LayoutStageBase#coreLayout core layout algorithm} and finishes the orientation change
afterwards.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Applies the desired orientation to the graph.
It performs the inverse coordinate transformation that has been applied to the layout by method
{@link OrientationLayout#prepareOrientationChange}.
This method is called by {@link OrientationLayout#applyLayout} after invoking the
{@link LayoutStageBase#coreLayout core layout algorithm}. It may be overridden to extend the orientation change to custom layout features.
@param graph the input graph
@see OrientationLayout#prepareOrientationChange
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23completeOrientationChange(LayoutGraph) Online Documentation}*/
completeOrientationChange(graph: LayoutGraph | null): void;
/**
Transforms the given point during completion.
The point is transformed from {@link LayoutOrientation#TOP_TO_BOTTOM} orientation to the desired orientation.
@param point the calculated point
@returns the transformed point
@see OrientationLayout#completeOrientationChange
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23completeTransform(YPoint) Online Documentation}*/
completeTransform(point: YPoint | null): YPoint | null;
/**
Creates {@link YInsets} that are a geometric transformation of the given {@link YInsets}.
The transform will correspond to the currently set orientation.
@param insets the original insets
@returns the oriented insets
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23createOrientedInsets(YInsets) Online Documentation}*/
createOrientedInsets(insets: YInsets | null): YInsets | null;
/**
Creates a {@link NodeHalo} that is a geometric transformation of the given {@link NodeHalo}.
The transform will correspond to the currently set orientation.
@param halo the original {@link NodeHalo}
@returns the oriented {@link NodeHalo}
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23createOrientedNodeHalo(NodeHalo) Online Documentation}*/
createOrientedNodeHalo(halo: NodeHalo | null): NodeHalo | null;
/**
Creates a {@link YDimension} that is a geometric transformation of the given size.
The transform will correspond to the currently set orientation.
@param nodeSize the original node size
@returns the oriented node size
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23createOrientedNodeSize(YDimension) Online Documentation}*/
createOrientedNodeSize(nodeSize: YDimension | null): YDimension | null;
/**
Returns whether or not this {@link OrientationLayout} instance will mirror the graph for a given layout
orientation.
Horizontal orientations get mirrored at the x-axis while vertical orientations are mirrored at the y-axis.
@param orientation the layout orientation for which to check the mirror state
@returns <code>true</code> if the layout algorithm will mirror the graph at the corresponding axis for the given layout orientation, <code>false</code>
otherwise
@see OrientationLayout#orientation
@see OrientationLayout#mirrorMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23isOrientationMirrored(LayoutOrientation) Online Documentation}*/
isOrientationMirrored(orientation: LayoutOrientation | LayoutOrientationStringValues): boolean;
/**
Prepares the layout for the desired orientation.
It will rotate and mirror the coordinates and bounds of the graph layout such that the {@link LayoutStageBase#coreLayout core layout algorithm} can perform
its calculations assuming the canonical {@link LayoutOrientation#TOP_TO_BOTTOM} orientation.
This method is called by {@link OrientationLayout#applyLayout} before invoking the
{@link LayoutStageBase#coreLayout core layout algorithm}. It might be overridden to extend the preparations to custom layout features.
@param graph the input graph
@see OrientationLayout#completeOrientationChange
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23prepareOrientationChange(LayoutGraph) Online Documentation}*/
prepareOrientationChange(graph: LayoutGraph | null): void;
/**
Transforms the given point during preparation.
The point is transformed so it matches the {@link LayoutOrientation#TOP_TO_BOTTOM} orientation.
@param point the original point
@returns the transformed point
@see OrientationLayout#prepareOrientationChange
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23prepareTransform(YPoint) Online Documentation}*/
prepareTransform(point: YPoint | null): YPoint | null;
/**
Transforms the given point during the preparation or completion step.
This method is called by {@link OrientationLayout#prepareOrientationChange} and {@link OrientationLayout#completeOrientationChange}
to get the according point location.
@param point the original/calculated point
@param prepare <code>true</code> if this method is called during preparation, <code>false</code> if it is called during completion
@returns the transformed point
@see OrientationLayout#prepareOrientationChange
@see OrientationLayout#completeOrientationChange
@see OrientationLayout#prepareTransform
@see OrientationLayout#completeTransform
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23transform(YPoint%2Cboolean) Online Documentation}*/
transform(point: YPoint | null, prepare: boolean): YPoint | null;
/**
Gets or sets the main direction of the layout.
@throws {Exception} <b>(ArgumentError)</b> if the specified orientation is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23orientation Online Documentation}*/
orientation: LayoutOrientation;
/**
Gets or sets the mirror mask that defines which orientations this {@link OrientationLayout} shall mirror.
{@link OrientationLayout#orientation Setting a layout orientation} can be seen as rotating the graph by <code>90</code>, <code>180</code> or <code>270</code> degrees. Afterwards the graph can be
mirrored at the x-axis (for horizontal layout orientations) or y-axis (for vertical layout orientations).
Which directions are mirrored can be defined by the given mask. It can combine several mirrored directions with a
logical <code>or</code>-operation.
@throws {Exception} <b>(ArgumentError)</b> if an unknown mask is specified
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23mirrorMode Online Documentation}*/
mirrorMode: MirrorModes;
/**
Gets whether or not the current orientation is horizontal.
The orientation is horizontal if it is {@link LayoutOrientation#LEFT_TO_RIGHT} or
{@link LayoutOrientation#RIGHT_TO_LEFT}.
@see OrientationLayout#orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23horizontalOrientation Online Documentation}*/
readonly horizontalOrientation: boolean;
/**
Gets or sets whether or not the position of the edge labels should be changed during orientation change.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrientationLayout%23considerEdgeLabels Online Documentation}*/
considerEdgeLabels: boolean;
static $class: Class;
private $_OrientationLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrientationLayout;
}
export interface ParallelEdgeRouter extends LayoutStageBase {
}
/**
{@link ParallelEdgeRouter} is a {@link ILayoutStage} that routes parallel edges which connect the same pair
of nodes in a graph.
<b>Layout Style</b>
The edge paths of parallel edges consist of parallel segments. At the end points, the edges can either still be parallel
or {@link ParallelEdgeRouter#joinEnds joined in one point}.
<b>Concept</b>
{@link ParallelEdgeRouter} performs four basic steps:
<ol>
<li>Remove all parallel edges from the input graph, leaving one master edge in the graph
<li>Invoke the {@link LayoutStageBase#coreLayout core layout algorithm} on the graph without parallel edges
<li>Reinsert all previously removed edges
<li>Route the parallel edges
</ol>
<b>Features</b>
{@link ParallelEdgeRouter} can either be used as a {@link ILayoutStage} wrapping a {@link ILayoutAlgorithm layout algorithm} which
cannot handle parallel edges. Then it will hide the parallel edges from this {@link LayoutStageBase#coreLayout core layout algorithm} and take over the
routing of them.
If no {@link LayoutStageBase#coreLayout core layout algorithm} is specified, {@link ParallelEdgeRouter} can work alone. It will route only the
parallel edges and keep the remaining graph unchanged.
Parallel edges can either be all edges between the same two nodes, ignoring the direction, or edges are only considered
parallel if they share the same source node and target node. Hence, they share the same {@link ParallelEdgeRouter#directedMode direction}.
To make sure that the ports of all parallel edges are on the end nodes, the first and last segments can be
{@link ParallelEdgeRouter#joinEnds joined}. However, these segments won't be parallel.
There are two ways to define the distance between two parallel edges. First, an absolute distance can be {@link ParallelEdgeRouter#lineDistance defined}.
Parallel segments will keep this distance. Second, the distance can be
{@link ParallelEdgeRouter#adaptiveLineDistances determined relative to the node bounds}. Depending on the size of the nodes and the number of parallel edges between
them, the distance between parallel edges is adjusted. This will keep the edges straight in most cases.
It is possible to select
{@link ParallelEdgeRouter#LEADING_EDGE_DP_KEY custom master edges}. All edges that are parallel to the selected edges will be temporarily removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter Online Documentation}*/
export class ParallelEdgeRouter {
/**
Creates a new {@link ParallelEdgeRouter} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see LayoutStageBase#coreLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link ParallelEdgeRouter} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.directedMode] Whether or not the direction of edges should be considered. This option sets the {@link ParallelEdgeRouter#directedMode} property on the created object.
@param [options.adaptiveLineDistances] Whether or not the distances between parallel edges should be determined automatically according to the sizes of their source and target nodes. This option sets the {@link ParallelEdgeRouter#adaptiveLineDistances} property on the created object.
@param [options.lineDistance] The distance between two adjacent edge paths that run in parallel. This option sets the {@link ParallelEdgeRouter#lineDistance} property on the created object.
@param [options.joinEnds] Whether or not to join end points of parallel edges. This option sets the {@link ParallelEdgeRouter#joinEnds} property on the created object.
@param [options.absJoinEndDistance] The absolute distance from the end point of the joined lines to the first parallel segments. This option sets the {@link ParallelEdgeRouter#absJoinEndDistance} property on the created object.
@param [options.relJoinEndDistance] The relative distance from the end point of the joined lines to the first parallel segments. This option sets the {@link ParallelEdgeRouter#relJoinEndDistance} property on the created object.
@param [options.adjustLeadingEdge] Whether or not to adjust the leading edge to obtain more symmetric results. This option sets the {@link ParallelEdgeRouter#adjustLeadingEdge} property on the created object.
@see LayoutStageBase#coreLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
directedMode?: boolean;
adaptiveLineDistances?: boolean;
lineDistance?: number;
joinEnds?: boolean;
absJoinEndDistance?: number;
relJoinEndDistance?: number;
adjustLeadingEdge?: boolean;
});
/**
Delegates the arrangement of nodes and edges to the {@link LayoutStageBase#coreLayout core layout algorithm} and routes the parallel edges.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Hides all parallel edges leaving a master edge in the graph.
{@link ParallelEdgeRouter} detects parallel edges of the given graph. From each set of parallel edges it hides all
but one edge from the given graph.
This method is called before invoking the {@link LayoutStageBase#coreLayout core layout algorithm}. It may be overridden to change the selection of leading
edges. Hidden edges should be stored in {@link ParallelEdgeRouter#hiddenEdges} to enable un-hiding later on.
@param graph the input graph
@see ParallelEdgeRouter#directedMode
@see ParallelEdgeRouter#AFFECTED_EDGES_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23findAndHideParallelEdges(Graph) Online Documentation}*/
findAndHideParallelEdges(graph: Graph | null): void;
/**
Assigns a layout to all parallel edges.
The layout for each parallel edge follows the layout of the master edge which was not removed from the
{@link LayoutStageBase#coreLayout core layout algorithm}.
This method is called when the {@link LayoutStageBase#coreLayout core layout algorithm} is finished. It may be overridden to introduce a custom routing for
parallel edges.
@param graph the graph after the core layout
@param parallelEdges the map that provides all parallel edges for each master edge
@see ParallelEdgeRouter#joinEnds
@see ParallelEdgeRouter#adaptiveLineDistances
@see ParallelEdgeRouter#adjustLeadingEdge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23layoutParallelEdges(LayoutGraph%2CIEdgeMap) Online Documentation}*/
layoutParallelEdges(graph: LayoutGraph | null, parallelEdges: IEdgeMap | null): void;
/**
Data provider key for marking edges that will be routed
<h2>Note</h2>
If no {@link IDataProvider} is registered with the given key, all edges will be routed.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23AFFECTED_EDGES_DP_KEY Online Documentation}*/
static AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Data provider key for specifying the master edges
Marked edges won't be removed for the
{@link LayoutStageBase#coreLayout core layout algorithm}. All edges parallel to the master edges will get parallel paths.
If no leading edge is specified for a bundle of parallel edges, it will be determined automatically. If there is more
than one leading edge, the first one is chosen.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23LEADING_EDGE_DP_KEY Online Documentation}*/
static LEADING_EDGE_DP_KEY: EdgeDpKey<boolean> | null;
/**
Holds the list of parallel edges that are hidden from the {@link LayoutStageBase#coreLayout core layout algorithm}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23hiddenEdges Online Documentation}*/
hiddenEdges: EdgeList | null;
/**
{@link IEdgeMap} that associates a hidden edge with the unique parallel edge not hidden from the
{@link LayoutStageBase#coreLayout core layout algorithm}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23parallelEdges Online Documentation}*/
parallelEdges: IEdgeMap | null;
/**
Gets or sets whether or not the direction of edges should be considered.
In directed mode, only edges that share the same source and target node will be routed in parallel. Edges that connect
to the same nodes but in different directions won't be considered parallel.
In undirected mode, all edges connecting the same pair of nodes will be routed in parallel.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23directedMode Online Documentation}*/
directedMode: boolean;
/**
Gets or sets whether or not the distances between parallel edges should be determined automatically according to the
sizes of their source and target nodes.
If enabled, the distances between edges are chosen such that all parallel edges can be drawn as straight lines and still
connect to the nodes' bounds.
@see ParallelEdgeRouter#lineDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23adaptiveLineDistances Online Documentation}*/
adaptiveLineDistances: boolean;
/**
Gets or sets the distance between two adjacent edge paths that run in parallel.
The distance needs to be non-negative.
<h2>Note</h2>
If {@link ParallelEdgeRouter#adaptiveLineDistances adaptive line distances} are used, the resulting distance may be less than the specified distance.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is less than <code>0</code>
@see ParallelEdgeRouter#adaptiveLineDistances
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23lineDistance Online Documentation}*/
lineDistance: number;
/**
Gets or sets whether or not to join end points of parallel edges.
If enabled, all edges end with non-parallel segments that connect at the same point to the node. Otherwise, all segments
of the edges are parallel. In this case, some edges may not connect to the nodes' bounds.
@see ParallelEdgeRouter#absJoinEndDistance
@see ParallelEdgeRouter#relJoinEndDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23joinEnds Online Documentation}*/
joinEnds: boolean;
/**
Gets or sets the absolute distance from the end point of the joined lines to the first parallel segments.
The absolute distance needs to be non-negative.
<b>Warning:</b> This distance is only considered if parallel edge paths are {@link ParallelEdgeRouter#joinEnds joined}.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see ParallelEdgeRouter#joinEnds
@see ParallelEdgeRouter#relJoinEndDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23absJoinEndDistance Online Documentation}*/
absJoinEndDistance: number;
/**
Gets or sets the relative distance from the end point of the joined lines to the first parallel segments.
The length is relative to the corresponding original segment's length. It needs to have a value in the interval <code>[0,1]</code>.
<b>Warning:</b> This distance is only considered if parallel edge paths are {@link ParallelEdgeRouter#joinEnds joined}.
@throws {Exception} <b>(ArgumentError)</b> if the specified relative distance is less than <code>0</code> or greater than <code>1</code>
@see ParallelEdgeRouter#joinEnds
@see ParallelEdgeRouter#absJoinEndDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23relJoinEndDistance Online Documentation}*/
relJoinEndDistance: number;
/**
Gets or sets whether or not to adjust the leading edge to obtain more symmetric results.
<h2>Note</h2>
Enabling this property may change the original ports of the master edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouter%23adjustLeadingEdge Online Documentation}*/
adjustLeadingEdge: boolean;
static $class: Class;
private $_ParallelEdgeRouter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ParallelEdgeRouter;
}
/**
@see ClearAreaLayout#edgeRoutingStrategy
@see PartialLayout#edgeRoutingStrategy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutEdgeRoutingStrategy Online Documentation}*/
export enum PartialLayoutEdgeRoutingStrategy {
/**
A routing strategy that produces orthogonal routes.
A route of an edge is called <i>orthogonal</i> if it only consists of vertical and horizontal segments.
@see PartialLayout#edgeRoutingStrategy
@see ClearAreaLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutEdgeRoutingStrategy%23ORTHOGONAL Online Documentation}*/
ORTHOGONAL,
/**
A routing strategy that produces straight-line routes.
@see PartialLayout#edgeRoutingStrategy
@see ClearAreaLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutEdgeRoutingStrategy%23STRAIGHTLINE Online Documentation}*/
STRAIGHTLINE,
/**
Automatically chooses a suitable routing strategy by analyzing the existing edge routes.
If, for example, all edges have orthogonal edge routes, edges that are rerouted are routed orthogonally, too.
@see PartialLayout#edgeRoutingStrategy
@see ClearAreaLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutEdgeRoutingStrategy%23AUTOMATIC Online Documentation}*/
AUTOMATIC,
/**
A routing strategy that produces organic routes for partial edges and inter-edges.
@see PartialLayout#edgeRoutingStrategy
@see ClearAreaLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutEdgeRoutingStrategy%23ORGANIC Online Documentation}*/
ORGANIC,
/**
A routing strategy that produces octilinear routes.
A route of an edge is called <i>octilinear</i> if the slope of each segment is a multiple of <code>45</code> degrees.
@see PartialLayout#edgeRoutingStrategy
@see ClearAreaLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutEdgeRoutingStrategy%23OCTILINEAR Online Documentation}*/
OCTILINEAR
}
/** Automatic conversion to {@link PartialLayoutEdgeRoutingStrategy}. */
type PartialLayoutEdgeRoutingStrategyStringValues = "orthogonal" | "straightline" | "automatic" | "organic" | "octilinear";
/**
@see ClearAreaLayout#componentAssignmentStrategy
@see FillAreaLayout#componentAssignmentStrategy
@see PartialLayout#componentAssignmentStrategy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentAssignmentStrategy Online Documentation}*/
export enum ComponentAssignmentStrategy {
/**
A component assignment strategy that assigns each node to a separate subgraph component.
@see PartialLayout#componentAssignmentStrategy
@see ClearAreaLayout#componentAssignmentStrategy
@see FillAreaLayout#componentAssignmentStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentAssignmentStrategy%23SINGLE Online Documentation}*/
SINGLE,
/**
A component assignment strategy where the subgraph components correspond to the connected components of the graph.
@see PartialLayout#componentAssignmentStrategy
@see ClearAreaLayout#componentAssignmentStrategy
@see FillAreaLayout#componentAssignmentStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentAssignmentStrategy%23CONNECTED Online Documentation}*/
CONNECTED,
/**
A component assignment strategy where the subgraph components correspond to the clusters computed by a clustering
algorithm based on {@link GroupAlgorithm#edgeBetweennessClustering edge betweenness centrality}.
@see PartialLayout#componentAssignmentStrategy
@see ClearAreaLayout#componentAssignmentStrategy
@see FillAreaLayout#componentAssignmentStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentAssignmentStrategy%23CLUSTERING Online Documentation}*/
CLUSTERING,
/**
A component assignment strategy where the subgraph components are defined by the user.
@see PartialLayout#componentAssignmentStrategy
@see ClearAreaLayout#componentAssignmentStrategy
@see FillAreaLayout#componentAssignmentStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentAssignmentStrategy%23CUSTOMIZED Online Documentation}*/
CUSTOMIZED
}
/** Automatic conversion to {@link ComponentAssignmentStrategy}. */
type ComponentAssignmentStrategyStringValues = "single" | "connected" | "clustering" | "customized";
/**
@see ClearAreaLayout#layoutOrientation
@see FillAreaLayout#layoutOrientation
@see PartialLayout#layoutOrientation
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutOrientation Online Documentation}*/
export enum PartialLayoutOrientation {
/**
Layout orientation specifier which defines that the main layout orientation is from top to bottom.
@see PartialLayout#layoutOrientation
@see ClearAreaLayout#layoutOrientation
@see FillAreaLayout#layoutOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutOrientation%23TOP_TO_BOTTOM Online Documentation}*/
TOP_TO_BOTTOM,
/**
Layout orientation specifier which defines that the main layout orientation is from bottom to top.
@see PartialLayout#layoutOrientation
@see ClearAreaLayout#layoutOrientation
@see FillAreaLayout#layoutOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutOrientation%23BOTTOM_TO_TOP Online Documentation}*/
BOTTOM_TO_TOP,
/**
Layout orientation specifier which defines that the main layout orientation is from left to right.
@see PartialLayout#layoutOrientation
@see ClearAreaLayout#layoutOrientation
@see FillAreaLayout#layoutOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutOrientation%23LEFT_TO_RIGHT Online Documentation}*/
LEFT_TO_RIGHT,
/**
Layout orientation specifier which defines that the main layout orientation is from right to left.
@see PartialLayout#layoutOrientation
@see ClearAreaLayout#layoutOrientation
@see FillAreaLayout#layoutOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutOrientation%23RIGHT_TO_LEFT Online Documentation}*/
RIGHT_TO_LEFT,
/**
Layout orientation specifier where the orientation is automatically detected.
@see PartialLayout#layoutOrientation
@see ClearAreaLayout#layoutOrientation
@see FillAreaLayout#layoutOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutOrientation%23AUTO_DETECT Online Documentation}*/
AUTO_DETECT,
/**
Layout orientation specifier where the layout orientation is completely ignored.
@see PartialLayout#layoutOrientation
@see ClearAreaLayout#layoutOrientation
@see FillAreaLayout#layoutOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutOrientation%23NONE Online Documentation}*/
NONE
}
/** Automatic conversion to {@link PartialLayoutOrientation}. */
type PartialLayoutOrientationStringValues = "top-to-bottom" | "bottom-to-top" | "left-to-right" | "right-to-left" | "auto-detect" | "none";
/**
@see ClearAreaLayout#clearAreaStrategy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaStrategy Online Documentation}*/
export enum ClearAreaStrategy {
/**
A strategy that clears the area by trying to minimize the number of moved nodes, changing the layout rather locally.
From the specified
{@link ClearAreaLayout#area}, stripes running from the area border to the graph border in all four directions are considered. The area is now
cleared by simply moving all elements inside the stripe in that direction, as far as necessary. If elements are not
completely inside the stripe but collide with moved elements, then they are moved along too. This way, the moving stripe
can quickly grow in some scenarios.
All edges where at least one node was moved are finally rerouted by the {@link ClearAreaLayout#edgeRouter routing algorithm}. This also applies for edges
that cross the area if property {@link ClearAreaLayout#considerEdges} is enabled.
<b>Warning:</b> This strategy does not consider the {@link ClearAreaLayout#componentAssignmentStrategy component assignment} setting.
<b>Warning:</b> Specifying an {@link ClearAreaLayout#AREA_GROUP_NODE_DP_KEY area group node} and a {@link PartitionGrid} structure at the same time is not supported by this
strategy. In that case the area group node is ignored, i.e. the space will not be created inside any group but the
partition grid constraints are preserved. Therefore, it is strongly discouraged to select this strategy for grouped
graphs that have a {@link PartitionGrid}.
@see ClearAreaLayout#clearAreaStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaStrategy%23LOCAL Online Documentation}*/
LOCAL,
/**
A strategy that clears the area by trying to minimize the number of moved nodes, changing the layout rather locally but
moving all necessary nodes by a uniform offset.
This strategy basically works like {@link ClearAreaStrategy#LOCAL} with the difference that nodes that are
required to move are moved uniformly, by the same location offset. Therefore, compared to {@link ClearAreaStrategy#LOCAL}
this strategy preserves more of the initial relations. On the other hand, results can become less compact. As nodes are
uniformly moved, edges where both end nodes were shifted do not need to be rerouted.
<b>Warning:</b> This strategy does not consider the {@link ClearAreaLayout#componentAssignmentStrategy component assignment} setting.
<b>Warning:</b> Specifying an {@link ClearAreaLayout#AREA_GROUP_NODE_DP_KEY area group node} and a {@link PartitionGrid} structure at the same time is not supported by this
strategy. In that case the area group node is ignored, i.e. the space will not be created inside any group but the
partition grid constraints are preserved. Therefore, it is strongly discouraged to select this strategy for grouped
graphs that have a {@link PartitionGrid}.
@see ClearAreaLayout#clearAreaStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaStrategy%23LOCAL_UNIFORM Online Documentation}*/
LOCAL_UNIFORM,
/**
A strategy that clears the area with the goal to preserve the shape of the existing edge paths, moving related elements
in common.
Preserving the shapes of the edge paths means that the direction of the edge segments is not changed and no new bends
are created. The segment lengths may change in order to generate free space. The effect of this strategy is that the
relative routes remain the same. The general aesthetics of a given layout are, ideally, preserved. The layout only
becomes sparser to free up the desired space.
This strategy models all graph elements (nodes, edges, bends, labels) the same way and unifies related elements where
appropriate. They are then moved in common in order to keep the relation among them as is.
<h2>Note</h2>
Compared to the local approaches {@link ClearAreaStrategy#LOCAL} and
{@link ClearAreaStrategy#LOCAL_UNIFORM}, this strategy usually needs to move a larger number of nodes to achieve
its goal.
@see ClearAreaLayout#clearAreaStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaStrategy%23PRESERVE_SHAPES Online Documentation}*/
PRESERVE_SHAPES,
/**
A strategy that clears the area with the goal to preserve the shape of the existing edge paths, moving related elements
in common and also moving all elements by a uniform offset.
In addition to the related strategy
{@link ClearAreaStrategy#PRESERVE_SHAPES}, this one uses a uniform offset for all elements that are moved. This
way the relative locations between all elements in the graph are preserved. On the other hand, unnecessary additional
space may be generated.
Preserving the shapes of the edge paths means that the direction of the edge segments is not changed and no new bends
are created. The segment lengths may change in order to generate free space. The effect of this strategy is that the
relative routes remain the same. The general aesthetics of a given layout are, ideally, preserved. The layout only
becomes sparser to free up the desired space.
This strategy models all graph elements (nodes, edges, bends, labels) the same way and unifies related elements where
appropriate. They are then moved in common in order to keep the relation among them as is.
<h2>Note</h2>
Compared to the local approaches {@link ClearAreaStrategy#LOCAL} and
{@link ClearAreaStrategy#LOCAL_UNIFORM}, this strategy usually needs to move a larger number of nodes to achieve
its goal.
@see ClearAreaLayout#clearAreaStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaStrategy%23PRESERVE_SHAPES_UNIFORM Online Documentation}*/
PRESERVE_SHAPES_UNIFORM,
/**
A strategy that clears the area by globally partitioning the input into two parts and moving them apart.
The existing layout is divided into two partitions somewhere along the {@link ClearAreaLayout#area area}. The division can be vertical or horizontal.
To free up the space, the two components are then moved horizontally (vertical division) or vertically (horizontal
division). Edges connecting nodes of different partitions are rerouted by the {@link ClearAreaLayout#edgeRouter routing algorithm}.
This strategy has a rather global effect on the existing layout because every element is displaced. The edge routes and
relative locations between elements that are in the same partition are not changed at all. Compared to other strategies,
more unnecessary free space may be generated as not only the exact {@link ClearAreaLayout#area area} is cleared, but a whole vertical or horizontal
stripe.
The specified {@link ClearAreaLayout#componentAssignmentStrategy component assignments} is considered such that if possible all component nodes are in the same partition.
@see ClearAreaLayout#clearAreaStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaStrategy%23GLOBAL Online Documentation}*/
GLOBAL
}
/** Automatic conversion to {@link ClearAreaStrategy}. */
type ClearAreaStrategyStringValues = "local" | "local-uniform" | "preserve-shapes" | "preserve-shapes-uniform" | "global";
/**
@see PartialLayout#subgraphPlacement
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubgraphPlacement Online Documentation}*/
export enum SubgraphPlacement {
/**
A positioning strategy which tries to place each subgraph component close to the barycenter of its graph neighbors.
@see PartialLayout#subgraphPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubgraphPlacement%23BARYCENTER Online Documentation}*/
BARYCENTER,
/**
A positioning strategy which tries to place each subgraph component close to its original position.
<h2>Note</h2>
The algorithm still tries to prevent overlaps.
@see PartialLayout#subgraphPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubgraphPlacement%23FROM_SKETCH Online Documentation}*/
FROM_SKETCH
}
/** Automatic conversion to {@link SubgraphPlacement}. */
type SubgraphPlacementStringValues = "barycenter" | "from-sketch";
export interface PartialLayout extends LayoutStageBase {
}
/**
This class represents a partial layout algorithm which changes the coordinates for a given set of graph elements (called
partial elements) only.
The location and size of the remaining elements (called fixed elements) is not allowed to be changed.<b>Layout Style</b>
This partial layout algorithm offers a kind of generic partial layout support for other existing layout algorithms.
Hence, its layout style heavily depends on the selected {@link PartialLayout#coreLayout core layout algorithm} as well as the specified {@link PartialLayout#edgeRouter edge router} or
{@link PartialLayout#edgeRoutingStrategy edge routing strategy}.
The partial layout is suitable for applications where users may incrementally add new elements to an existing drawing.
The added elements should be arranged so that they fit best possible into the given diagram without making any changes
to the already existing layout. Hence, the so-called mental map of the existing drawing is preserved.
<b>Features</b>
Similar to the layout style, the supported feature set mainly depends on the features supported by the specified {@link PartialLayout#coreLayout core layout algorithm}
as well as the specified {@link PartialLayout#edgeRouter edge router} or {@link PartialLayout#edgeRoutingStrategy edge routing strategy}. The internal step that {@link PartialLayout#placeSubgraphs places the components} can handle
group nodes as well as minimum distance constraints. Furthermore, it is able to consider node labels.
<b>Concept</b>
The layout algorithm tries to place the partial elements such that the resulting drawing (including the fixed elements)
has a good quality with respect to common graph drawing aesthetics.
The layout algorithm handles each selected graph element as partial element. For this, it looks up the {@link IDataProvider}
keys {@link PartialLayout#AFFECTED_NODES_DP_KEY} and {@link PartialLayout#AFFECTED_EDGES_DP_KEY}. Partial
node elements can be assigned to the so-called <i>subgraph components</i>. During the layout process each subgraph induced by the nodes of a
component is first laid out using the specified {@link PartialLayout#coreLayout core layout algorithm}. Then, the different components are placed
one-by-one onto the drawing area such that the number of overlaps among graph elements is small. The user can specify
different objectives for finding 'good' positions for subgraph components (see {@link PartialLayout#subgraphPlacement}), e.g., {@link SubgraphPlacement#BARYCENTER}
specifies that the component should be placed close to the barycenter of its graph neighbors and {@link SubgraphPlacement#FROM_SKETCH}
specifies that the component should be placed close to its original position.
Method {@link PartialLayout#componentAssignmentStrategy} allows to specify the strategy that assigns partial nodes to subgraph components. Possible values are
{@link ComponentAssignmentStrategy#CLUSTERING}, {@link ComponentAssignmentStrategy#CONNECTED}, {@link ComponentAssignmentStrategy#SINGLE}
and
{@link ComponentAssignmentStrategy#CUSTOMIZED}. The last value allows to use a customized component assignment.
Note that nodes of a component cannot be assigned to different group nodes.
Furthermore, the user can specify the edge routing strategy (see {@link PartialLayout#edgeRoutingStrategy}) that is used for routing partial edges and edges
between different subgraph components (so-called <i>inter-edges</i>). Possible values are {@link PartialLayoutEdgeRoutingStrategy#ORGANIC},
{@link PartialLayoutEdgeRoutingStrategy#ORTHOGONAL}, {@link PartialLayoutEdgeRoutingStrategy#STRAIGHTLINE}, {@link PartialLayoutEdgeRoutingStrategy#OCTILINEAR}
and {@link PartialLayoutEdgeRoutingStrategy#AUTOMATIC}.
<h2>Note</h2>
When a partial node should be placed inside a fixed group node, it is important that there is enough free space inside
the group. Otherwise, there may be overlapping node elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout Online Documentation}*/
export class PartialLayout {
/**
Creates a new instance of {@link PartialLayout} which uses the specified {@link ILayoutAlgorithm} instance
as the core layout algorithm.
This instance is applied to each subgraph component, see {@link PartialLayout#componentAssignmentStrategy}.
@param [coreLayout=null] the layout algorithm that is applied to the subgraph components
@see PartialLayout#componentAssignmentStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new instance of {@link PartialLayout} which uses the specified {@link ILayoutAlgorithm} instance
as the core layout algorithm.
This instance is applied to each subgraph component, see {@link PartialLayout#componentAssignmentStrategy}.
@param options The parameters to pass.
@param [options.coreLayout=null] the layout algorithm that is applied to the subgraph components
@param [options.maximumDuration] The preferred time limit (in milliseconds) for the layout algorithm. This option sets the {@link PartialLayout#maximumDuration} property on the created object.
@param [options.routeInterEdgesImmediately] Whether or not edges between different subgraph components should be routed immediately. This option sets the {@link PartialLayout#routeInterEdgesImmediately} property on the created object.
@param [options.componentCompaction] Whether or not a subgraph component may be placed within another subgraph component. This option sets the {@link PartialLayout#componentCompaction} property on the created object.
@param [options.resizeFixedGroups] Whether or not fixed (non-partial) group nodes may be resized. This option sets the {@link PartialLayout#resizeFixedGroups} property on the created object.
@param [options.subgraphPlacement] The objective used for finding 'good' positions for subgraph components. This option sets the {@link PartialLayout#subgraphPlacement} property on the created object.
@param [options.minimumNodeDistance] The minimum distance between two adjacent nodes. This option sets the {@link PartialLayout#minimumNodeDistance} property on the created object.
@param [options.considerNodeAlignment] Whether or not partial nodes should be aligned. This option sets the {@link PartialLayout#considerNodeAlignment} property on the created object.
@param [options.componentAssignmentStrategy] The strategy that assigns partial nodes to subgraph components. This option sets the {@link PartialLayout#componentAssignmentStrategy} property on the created object.
@param [options.optimizeOrientation] Whether or not a postprocessing step should be applied to reduce the number of directed edges that do not comply with the specified layout orientation. This option sets the {@link PartialLayout#optimizeOrientation} property on the created object.
@param [options.edgeRouter] The custom edge router instance that is used for partial edges and edges between different subgraph components (so-called inter-edges). This option sets the {@link PartialLayout#edgeRouter} property on the created object.
@param [options.edgeRoutingStrategy] The routing strategy that is used for partial edges and inter-edges. This option sets the {@link PartialLayout#edgeRoutingStrategy} property on the created object.
@param [options.layoutOrientation] The layout orientation that is considered during the placement of partial elements. This option sets the {@link PartialLayout#layoutOrientation} property on the created object.
@param [options.allowMirroring] Whether or not subgraph components are mirrored to improve the layout quality. This option sets the {@link PartialLayout#allowMirroring} property on the created object.
@see PartialLayout#componentAssignmentStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
maximumDuration?: number;
routeInterEdgesImmediately?: boolean;
componentCompaction?: boolean;
resizeFixedGroups?: boolean;
subgraphPlacement?: SubgraphPlacement | SubgraphPlacementStringValues;
minimumNodeDistance?: number;
considerNodeAlignment?: boolean;
componentAssignmentStrategy?: ComponentAssignmentStrategy | ComponentAssignmentStrategyStringValues;
optimizeOrientation?: boolean;
edgeRouter?: ILayoutAlgorithm | null;
edgeRoutingStrategy?: PartialLayoutEdgeRoutingStrategy | PartialLayoutEdgeRoutingStrategyStringValues;
layoutOrientation?: PartialLayoutOrientation | PartialLayoutOrientationStringValues;
allowMirroring?: boolean;
});
/**
This method calculates the partial layout.
It calls the following methods:
<ol>
<li>{@link PartialLayout#routeEdgesBetweenFixedElements}
<li>{@link PartialLayout#layoutSubgraph} for each subgraph component
<li>{@link PartialLayout#placeSubgraphs}
<li>{@link PartialLayout#routeInterEdges}
</ol>
<h2>Note</h2>
The above methods are called after applying the
{@link OrientationLayout}. Hence, they always assume that the graph is drawn from top to bottom.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
This method is called each time when edges are routed with an edge router.
Subclasses may modify the configuration of the given edge router instance.
The type of the given instance depends on the edge routing strategy, i.e., if the routing strategy is set to {@link PartialLayoutEdgeRoutingStrategy#OCTILINEAR}
or {@link PartialLayoutEdgeRoutingStrategy#ORTHOGONAL} it's an instance of {@link EdgeRouter}, if the routing strategy
is set to {@link PartialLayoutEdgeRoutingStrategy#ORGANIC} it's an instance of {@link OrganicEdgeRouter}, and, if the
routing strategy is set to {@link PartialLayoutEdgeRoutingStrategy#STRAIGHTLINE} it's an instance of a private
{@link StraightLineEdgeRouter}. If the edge routing strategy is set to
{@link PartialLayoutEdgeRoutingStrategy#AUTOMATIC}, the layout algorithm chooses one of the above strategies that best fits
the routing style of the fixed edges.
<h2>Note</h2>
If a custom edge router is set (see {@link PartialLayout#edgeRouter}), its instance corresponds to that of the parameter.
@param edgeRouter the instance used for routing the edges
@see PartialLayout#edgeRouter
@see PartialLayout#edgeRoutingStrategy
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23configureEdgeRouter(ILayoutAlgorithm) Online Documentation}*/
configureEdgeRouter(edgeRouter: ILayoutAlgorithm | null): void;
/**
This method is called during the {@link PartialLayout#applyLayout layout process} and calculates the layout for the given subgraph component using the
specified {@link PartialLayout#coreLayout core layout algorithm}.
It is called once for each subgraph component.
Subclasses may implement a custom layout strategy or add some additional data.
@param subGraph the subgraph component
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23layoutSubgraph(LayoutGraph) Online Documentation}*/
layoutSubgraph(subGraph: LayoutGraph | null): void;
/**
This method is called during the {@link PartialLayout#applyLayout layout process} and places the subgraph components one-by-one onto the drawing area.
Therefore, it considers the {@link PartialLayout#subgraphPlacement specified objective} for finding a suitable position.
Subclasses may implement a custom placement strategy or add some additional data. However, they must not modify the
given subgraph component lists.
@param graph the input graph
@param subgraphComponents each entry contains a {@link YNodeList} that induces a subgraph component
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23placeSubgraphs(LayoutGraph%2CArray%3CYNodeList%3E) Online Documentation}*/
placeSubgraphs(graph: LayoutGraph | null, subgraphComponents: YNodeList[] | null): void;
/**
This method is called during the {@link PartialLayout#applyLayout layout process} and routes all partial edges that connect two fixed elements.
It either uses a {@link PartialLayout#edgeRouter custom edge router} or an internal edge router that produces routes according to the specified
{@link PartialLayout#edgeRoutingStrategy routing strategy}.
Subclasses may implement a custom routing strategy or add some additional data.
@param graph the subgraph of the input graph induced by the fixed nodes
@param partialEdges the {@link EdgeList list of partial edges} to be routed
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23routeEdgesBetweenFixedElements(LayoutGraph%2CEdgeList) Online Documentation}*/
routeEdgesBetweenFixedElements(graph: LayoutGraph | null, partialEdges: EdgeList | null): void;
/**
This method is called during the {@link PartialLayout#applyLayout layout process} and routes all inter-edges.
<i>Inter-edges</i> are edges between different subgraph components including edges between fixed and partial elements. For the routing,
this method uses the edge router instance set with method {@link PartialLayout#edgeRouter}. If no edge router was specified by the user, it uses an
internal edge router with routing strategy {@link PartialLayout#edgeRoutingStrategy}.
Subclasses may implement a custom routing strategy or add some additional data.
@param graph the relevant subgraph
@param interEdges the {@link EdgeList list of inter-edges} to be routed
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23routeInterEdges(LayoutGraph%2CEdgeList) Online Documentation}*/
routeInterEdges(graph: LayoutGraph | null, interEdges: EdgeList | null): void;
/**
Data provider key for marking partial nodes
@see PartialLayout#AFFECTED_EDGES_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23AFFECTED_NODES_DP_KEY Online Documentation}*/
static AFFECTED_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for specifying the edges that should be considered to be directed
If a layout orientation is specified (i.e., {@link PartialLayout#layoutOrientation} is not
{@link PartialLayoutOrientation#NONE}), the algorithm tries to route directed edges such that they adhere to that
orientation.
<h2>Note</h2>
If this {@link IDataProvider} is not registered, all edges are considered to be directed.
@see PartialLayout#layoutOrientation
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23DIRECTED_EDGES_DP_KEY Online Documentation}*/
static DIRECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Data provider key for marking partial edges
<h2>Note</h2>
All edges with at least one partial endpoint are automatically considered to be partial.
@see PartialLayout#AFFECTED_NODES_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23AFFECTED_EDGES_DP_KEY Online Documentation}*/
static AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Data provider key for obtaining the edges that should be routed by the edge router
<b>Warning:</b> The associated {@link IDataProvider} must not be set by the user! The key is used by this algorithm to
temporarily add a {@link IDataProvider} that marks edges that should be routed by the specified edge router,
see {@link PartialLayout#edgeRouter}.
@see PartialLayout#edgeRouter
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23ROUTE_EDGE_DP_KEY Online Documentation}*/
static ROUTE_EDGE_DP_KEY: EdgeDpKey<boolean> | null;
/**
Data provider key for defining custom subgraph components
<h2>Note</h2>
The associated {@link IDataProvider} is only considered if {@link PartialLayout#componentAssignmentStrategy} is set to
{@link ComponentAssignmentStrategy#CUSTOMIZED}.
<b>Warning:</b> Nodes of a subgraph component cannot be assigned to different group nodes or different partition cells.
@see ComponentAssignmentStrategy#CUSTOMIZED
@see PartialLayout#componentAssignmentStrategy
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23COMPONENT_ID_DP_KEY Online Documentation}*/
static COMPONENT_ID_DP_KEY: NodeDpKey<any> | null;
/**
Gets or sets the {@link ILayoutAlgorithm} instance that is applied to each subgraph component.
More precisely, during the layout process each subgraph induced by the (partial) nodes of a component (see
{@link PartialLayout#componentAssignmentStrategy}) is first laid out using this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
/**
Gets or sets the preferred time limit (in milliseconds) for the layout algorithm.
The specified value has to be greater than or equal to <code>0</code>. If the value is <code>0x7FFFFFFF</code>, the time is not limited.
<h2>Note</h2>
Restricting the maximum duration may result in a worse layout quality. Furthermore, the real runtime may exceed the
maximum duration since the layout algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets whether or not edges between different subgraph components should be routed immediately.
If this option is enabled, edges are routed during the placement of the subgraph components, i.e., immediately after a
component is placed, its edges to other already placed components are routed. Otherwise, these edges are routed in a
separate step after placing all subgraph components. Hence, while enabling this option usually leads to shorter edge
routes, the placement of subgraph components is less compact.
@see PartialLayout#routeInterEdges
@see PartialLayout#placeSubgraphs
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23routeInterEdgesImmediately Online Documentation}*/
routeInterEdgesImmediately: boolean;
/**
Gets or sets whether or not a subgraph component may be placed within another subgraph component.
Enabling this option leads to more compact layout results but requires more runtime.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23componentCompaction Online Documentation}*/
componentCompaction: boolean;
/**
Gets or sets whether or not fixed (non-partial) group nodes may be resized.
Enabling this option may lead to better results if there are fixed group nodes, since there is more space for the
partial elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23resizeFixedGroups Online Documentation}*/
resizeFixedGroups: boolean;
/**
Gets or sets the objective used for finding 'good' positions for subgraph components.
@throws {Exception} <b>(ArgumentError)</b> if the specified strategy does not match one of the predefined positioning strategies
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23subgraphPlacement Online Documentation}*/
subgraphPlacement: SubgraphPlacement;
/**
Gets or sets the minimum distance between two adjacent nodes.
The specified value has to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23minimumNodeDistance Online Documentation}*/
minimumNodeDistance: number;
/**
Gets or sets whether or not partial nodes should be aligned.
If this option is enabled, the algorithm tries to align the center of partial nodes with other nodes.
<h2>Note</h2>
Since the specified {@link PartialLayout#coreLayout core layout algorithm} doesn't consider this option, the alignment works best if the {@link PartialLayout#componentAssignmentStrategy component assignment}
is set to {@link ComponentAssignmentStrategy#SINGLE}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23considerNodeAlignment Online Documentation}*/
considerNodeAlignment: boolean;
/**
Gets or sets the strategy that assigns partial nodes to subgraph components.
The specified {@link PartialLayout#coreLayout core layout algorithm} independently calculates the layout for each subgraph component.
<ul>
<li>{@link ComponentAssignmentStrategy#SINGLE}: each partial node is a separate subgraph component. The specified
core layout doesn't have any effect because it is applied to each single node separately.
<li>{@link ComponentAssignmentStrategy#CONNECTED}: components are defined by the connected components of the graph
induced by the partial elements.
<li>{@link ComponentAssignmentStrategy#CLUSTERING}: components are defined by edge betweenness clustering.
<li>{@link ComponentAssignmentStrategy#CUSTOMIZED}: components are user-defined. Two partial nodes are considered to
be in the same component if the {@link IDataProvider} associated with key {@link PartialLayout#COMPONENT_ID_DP_KEY}
returns the same object for both of them.
</ul>
<h2>Note</h2>
Borders of groups and partition cells always split up the components, i.e., two nodes assigned to different groups or
partition cells are always assigned to different subgraph components.
@throws {Exception} <b>(ArgumentError)</b> if the specified strategy does not match one of the predefined strategies
@see PartialLayout#COMPONENT_ID_DP_KEY
@see PartialLayout#coreLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23componentAssignmentStrategy Online Documentation}*/
componentAssignmentStrategy: ComponentAssignmentStrategy;
/**
Gets or sets whether or not a postprocessing step should be applied to reduce the number of directed edges that do not
comply with the specified layout orientation.
@see PartialLayout#layoutOrientation
@see PartialLayout#DIRECTED_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23optimizeOrientation Online Documentation}*/
optimizeOrientation: boolean;
/**
Gets or sets the custom edge router instance that is used for partial edges and edges between different subgraph
components (so-called inter-edges).
<h2>Note</h2>
Calling method {@link PartialLayout#edgeRoutingStrategy} automatically sets a new edge router instance according to the specified routing strategy.
<b>Warning:</b> Edges that have to be routed with the specified router will be marked using {@link IDataProvider} key
{@link PartialLayout#ROUTE_EDGE_DP_KEY}. Hence, the specified router has to be configured such that it observes
this key and only routes marked edges!
@see PartialLayout#ROUTE_EDGE_DP_KEY
@see PartialLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23edgeRouter Online Documentation}*/
edgeRouter: ILayoutAlgorithm | null;
/**
Gets or sets the routing strategy that is used for partial edges and inter-edges.
<i>Inter-edges</i> are edges between fixed and partial nodes as well as edges between different subgraph components. The available
strategies are listed in the following.
<ul>
<li>{@link PartialLayoutEdgeRoutingStrategy#STRAIGHTLINE} produces straight-line edge routes.
<li>{@link PartialLayoutEdgeRoutingStrategy#ORTHOGONAL} produces orthogonal edge routes.
<li>{@link PartialLayoutEdgeRoutingStrategy#OCTILINEAR} produces octilinear edge routes.
<li>{@link PartialLayoutEdgeRoutingStrategy#ORGANIC} produces organic edge routes.
<li>{@link PartialLayoutEdgeRoutingStrategy#AUTOMATIC} automatically chooses a suitable strategy for the partial edges
inter-edges, based on the existing routes of the fixed edges.
</ul>
<h2>Note</h2>
Calling this method disables a previously set customized edge router, see {@link PartialLayout#edgeRouter}.
<h2>Note</h2>
Calling this method disables a previously set customized edge router, see {@link PartialLayout#edgeRouter}.
@throws {Exception} <b>(ArgumentError)</b> if the specified strategy does not match one of the predefined routing strategies
@see PartialLayout#edgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23edgeRoutingStrategy Online Documentation}*/
edgeRoutingStrategy: PartialLayoutEdgeRoutingStrategy;
/**
Gets or sets the layout orientation that is considered during the placement of partial elements.
More precisely, the algorithm tries to place each {@link PartialLayout#componentAssignmentStrategy subgraph component} such that each predecessor of a component's node <code>v</code>
is placed before <code>v</code> and each successor after <code>v</code> with respect to the layout orientation.
<ul>
<li>{@link PartialLayoutOrientation#NONE}: the layout orientation is completely ignored.
<li>{@link PartialLayoutOrientation#AUTO_DETECT}: the layout orientation is automatically detected. The algorithm analyzes
the flow direction of fixed edges in the current drawing. If there is no common flow direction, the results are the same
as for {@link PartialLayoutOrientation#NONE}. Otherwise, the layout orientation is one of the four main orientation.
<li>{@link PartialLayoutOrientation#TOP_TO_BOTTOM}: the algorithm tries to place partial nodes (subgraph components) such
that each predecessor of a node <code>v</code> is placed above <code>v</code> and each successor below <code>v</code>.
<li>{@link PartialLayoutOrientation#BOTTOM_TO_TOP}: the algorithm tries to place partial nodes (subgraph components) such
that each predecessor of a node <code>v</code> is placed below <code>v</code> and each successor above <code>v</code>.
<li>{@link PartialLayoutOrientation#LEFT_TO_RIGHT}: the algorithm tries to place partial nodes (subgraph components) such
that each predecessor of a node <code>v</code> is placed to the left of <code>v</code> and each successor to the right of <code>v</code>.
<li>{@link PartialLayoutOrientation#RIGHT_TO_LEFT}: the algorithm tries to place partial nodes (subgraph components) such
that each predecessor of a node <code>v</code> is placed to the right of <code>v</code> and each successor to the left of <code>v</code>.
</ul>
<b>Warning:</b> This only affects the placement of nodes of different subgraph components that are connected by {@link PartialLayout#DIRECTED_EDGES_DP_KEY directed edges}. For
nodes of the same subgraph component the layout orientation depends on the chosen {@link PartialLayout#coreLayout core layout algorithm}.
@throws {Exception} <b>(ArgumentError)</b> if the specified orientation does not match one of the predefined orientations
@see PartialLayout#DIRECTED_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23layoutOrientation Online Documentation}*/
layoutOrientation: PartialLayoutOrientation;
/**
Gets or sets whether or not subgraph components are mirrored to improve the layout quality.
If enabled, the algorithm checks for each component which of the four possible mirrorings minimizes the edge length.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayout%23allowMirroring Online Documentation}*/
allowMirroring: boolean;
static $class: Class;
private $_PartialLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartialLayout;
}
export interface PlaceNodesAtBarycenterStage extends LayoutStageBase {
}
/**
{@link PlaceNodesAtBarycenterStage} temporarily hides the {@link PlaceNodesAtBarycenterStage#AFFECTED_NODES_DP_KEY affected nodes}, applies the specified core layout
algorithm (if any) to the resulting graph and, finally, reinserts the hidden nodes by placing them on the barycenter of
their neighbors using the specified {@link PlaceNodesAtBarycenterStage#AFFECTED_NODES_SIZE_DP_KEY node sizes}.
A possible application for this stage are animated incremental layouts, i.e., if there are new elements, you can first
place them near the barycenter of their existing neighbors before starting the animation. Hence, the animation will
produce smoother layouts since the new elements "start" from suitable initial positions. If the new elements should be
placed on their original coordinates and/or get their original sizes after starting the animation and before calling the
core layout, then you can use the {@link GivenCoordinatesStage}.
@see GivenCoordinatesStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStage Online Documentation}*/
export class PlaceNodesAtBarycenterStage {
/**
Creates a new {@link PlaceNodesAtBarycenterStage} instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] the core layout routine
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link PlaceNodesAtBarycenterStage} instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] the core layout routine
@param [options.removeBends] Whether or not bends of edges incident to {@link PlaceNodesAtBarycenterStage#AFFECTED_NODES_DP_KEY affected nodes} should be removed. This option sets the {@link PlaceNodesAtBarycenterStage#removeBends} property on the created object.
@param [options.resetPorts] Whether or not ports of edges incident to {@link PlaceNodesAtBarycenterStage#AFFECTED_NODES_DP_KEY affected nodes} should be set to the center of the corresponding nodes. This option sets the {@link PlaceNodesAtBarycenterStage#resetPorts} property on the created object.
@param [options.considerGrouping] Whether or not the grouping structure should be considered. This option sets the {@link PlaceNodesAtBarycenterStage#considerGrouping} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
removeBends?: boolean;
resetPorts?: boolean;
considerGrouping?: boolean;
});
/**
Starts the layout.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for determining which nodes should be placed by this stage
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStage%23AFFECTED_NODES_DP_KEY Online Documentation}*/
static AFFECTED_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for specifying the size of affected nodes
<h2>Note</h2>
Affected nodes that are not mapped to a specific size keep their current size.
<h2>Note</h2>
Values specified for non-affected nodes are ignored.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStage%23AFFECTED_NODES_SIZE_DP_KEY Online Documentation}*/
static AFFECTED_NODES_SIZE_DP_KEY: NodeDpKey<YDimension> | null;
/**
Gets or sets whether or not bends of edges incident to {@link PlaceNodesAtBarycenterStage#AFFECTED_NODES_DP_KEY affected nodes} should be removed.
@see PlaceNodesAtBarycenterStage#resetPorts
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStage%23removeBends Online Documentation}*/
removeBends: boolean;
/**
Gets or sets whether or not ports of edges incident to {@link PlaceNodesAtBarycenterStage#AFFECTED_NODES_DP_KEY affected nodes} should be set to the center of the corresponding
nodes.
<h2>Note</h2>
This property is only considered if {@link PlaceNodesAtBarycenterStage#removeBends} is enabled.
@see PlaceNodesAtBarycenterStage#removeBends
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStage%23resetPorts Online Documentation}*/
resetPorts: boolean;
/**
Gets or sets whether or not the grouping structure should be considered.
If this option is enabled, the layout algorithm considers the hierarchic grouping structure as follows:
<ul>
<li>The barycenter of an affected group depends on the coordinates of non-affected nodes outside the group that connect to
nodes inside the group.
<li>The barycenter of affected nodes is automatically moved inside the bounding box of the contained parent group.
</ul>
If this option is disabled, there is no special treatment of group nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStage%23considerGrouping Online Documentation}*/
considerGrouping: boolean;
static $class: Class;
private $_PlaceNodesAtBarycenterStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PlaceNodesAtBarycenterStage;
}
export interface EdgeRouterEdgeLayoutDescriptor extends YObject {
}
/**
This class is used by {@link EdgeRouter} to provide routing details for the edges of the graph.
An {@link EdgeRouterEdgeLayoutDescriptor} instance can be specified individually for single edges using a {@link IDataProvider}
that returns an {@link EdgeRouterEdgeLayoutDescriptor} instance for each edge of the graph, or <code>null</code> if no {@link EdgeRouterEdgeLayoutDescriptor}
is bound to an edge. The {@link IDataProvider} is registered with the graph using key
{@link EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
This class is designed such that it allows future additions of new getter methods.
<h2>Note</h2>
Not all of the values contained in this class will be considered for all kinds of edges and algorithm settings.
@see EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor Online Documentation}*/
export class EdgeRouterEdgeLayoutDescriptor {
/**
Creates a new instance of an {@link EdgeRouterEdgeLayoutDescriptor} with the default settings.
@param [options] The parameters to pass.
@param [options.minimumFirstSegmentLength] The minimum length of the first segment of the edge path (at the source node). This option sets the {@link EdgeRouterEdgeLayoutDescriptor#minimumFirstSegmentLength} property on the created object.
@param [options.minimumLastSegmentLength] The minimum length of the last segment of the edge path (at the target node). This option sets the {@link EdgeRouterEdgeLayoutDescriptor#minimumLastSegmentLength} property on the created object.
@param [options.minimumEdgeToEdgeDistance] The minimum distance between a pair of edges. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#minimumEdgeToEdgeDistance} property on the created object.
@param [options.minimumNodeCornerDistance] The minimum distance that the edge should maintain from node corners when entering or leaving the node. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#minimumNodeCornerDistance} property on the created object.
@param [options.monotonicPathRestriction] The monotonic path restrictions that should be applied. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#monotonicPathRestriction} property on the created object.
@param [options.routingStyle] The routing style for the edge associated with this descriptor. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#routingStyle} property on the created object.
@param [options.penaltySettings] The {@link PenaltySettings} used for this edge. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#penaltySettings} property on the created object.
@param [options.directGroupContentEdgeRouting] Whether or not this edge is routed directly to a group node's border if it connects the group node with one of its descendants. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#directGroupContentEdgeRouting} property on the created object.
@param [options.intermediateRoutingPoints] The list of intermediate {@link YPoint}s that need to lie on the edge route. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#intermediateRoutingPoints} property on the created object.
@param [options.createControlPoints] Whether or not the points of the resulting edge path represent cubic bezier control points. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#createControlPoints} property on the created object.
@param [options.sourceCurveConnectionStyle] The style which determines how curved edge paths connect at the source side of the edge. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#sourceCurveConnectionStyle} property on the created object.
@param [options.targetCurveConnectionStyle] The style using which determines how edge paths connect at the target side of the edge. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#targetCurveConnectionStyle} property on the created object.
@param [options.preferredOctilinearSegmentLength] The preferred length of non-orthogonal, octilinear segments. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#preferredOctilinearSegmentLength} property on the created object.
@param [options.maximumOctilinearSegmentRatio] The maximum ratio between the horizontal/vertical part of a segment and the non-orthogonal, octilinear part. This option sets the {@link EdgeRouterEdgeLayoutDescriptor#maximumOctilinearSegmentRatio} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor Online Documentation}*/
constructor(options?: {
minimumFirstSegmentLength?: number;
minimumLastSegmentLength?: number;
minimumEdgeToEdgeDistance?: number;
minimumNodeCornerDistance?: number;
monotonicPathRestriction?: MonotonicPathRestriction | MonotonicPathRestrictionStringValues;
routingStyle?: EdgeRouterEdgeRoutingStyle | EdgeRouterEdgeRoutingStyleStringValues;
penaltySettings?: PenaltySettings | null;
directGroupContentEdgeRouting?: boolean;
intermediateRoutingPoints?: IList<any> | null | IListConvertible<any>;
createControlPoints?: boolean;
sourceCurveConnectionStyle?: CurveConnectionStyle | CurveConnectionStyleStringValues;
targetCurveConnectionStyle?: CurveConnectionStyle | CurveConnectionStyleStringValues;
preferredOctilinearSegmentLength?: number;
maximumOctilinearSegmentRatio?: number;
});
/**
Creates a copy of this {@link EdgeRouterEdgeLayoutDescriptor} instance.
@returns a copy of this {@link EdgeRouterEdgeLayoutDescriptor} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23createCopy() Online Documentation}*/
createCopy(): EdgeRouterEdgeLayoutDescriptor | null;
/**
Gets or sets the minimum length of the first segment of the edge path (at the source node).
The minimum length should be greater than or equal to <code>0</code>. If a negative value is given as input, the default value, i.e.,
<code>5.0</code>, will be used instead.
<h2>Note</h2>
The final segment length also depends on the used grid size (see
{@link Grid#spacing}). If the grid is larger than the specified minimum length, then the grid size defines the actual minimum.
<h2>Note</h2>
The first segment length starts, if present, at the {@link NodeHalo halo} of the source node.
<b>Warning:</b> The first segment length may be violated for curved edge routes when using source connection style
{@link CurveConnectionStyle#ORGANIC}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23minimumFirstSegmentLength Online Documentation}*/
minimumFirstSegmentLength: number;
/**
Gets or sets the minimum length of the last segment of the edge path (at the target node).
The minimum length should be greater than or equal to <code>0</code>. If a negative value is given as input, the default value, i.e.,
<code>10.0</code>, will be used instead.
<h2>Note</h2>
The final segment length also depends on the used grid size (see
{@link Grid#spacing}). If the grid is larger than the specified minimum length, then the grid size defines the actual minimum.
<h2>Note</h2>
The last segment length ends, if present, at the {@link NodeHalo halo} of the target node.
<b>Warning:</b> The last segment length may be violated for curved edge routes when using target connection style
{@link CurveConnectionStyle#ORGANIC}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23minimumLastSegmentLength Online Documentation}*/
minimumLastSegmentLength: number;
/**
Gets or sets the minimum distance between a pair of edges.
The minimum distance should be greater than or equal to <code>0</code>. If a negative value is given as input, the default value,
i.e., <code>3.0</code>, will be used instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23minimumEdgeToEdgeDistance Online Documentation}*/
minimumEdgeToEdgeDistance: number;
/**
Gets or sets the minimum distance that the edge should maintain from node corners when entering or leaving the node.
The minimum distance should be greater than or equal to <code>0</code>. If a negative value is given as input, the default value,
i.e., <code>3.0</code>, will be used instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23minimumNodeCornerDistance Online Documentation}*/
minimumNodeCornerDistance: number;
/**
Gets or sets the monotonic path restrictions that should be applied.
Monotonic path restrictions imply that (ideally) the vertical and/or horizontal segments of an edge path are directed
from the source node towards the target node without ever changing their direction back towards the source node.
If an unknown restriction is specified, {@link MonotonicPathRestriction#NONE} will be used instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23monotonicPathRestriction Online Documentation}*/
monotonicPathRestriction: MonotonicPathRestriction;
/**
Gets or sets the routing style for the edge associated with this descriptor.
@throws {Exception} <b>(ArgumentError)</b> if an unknown routing style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23routingStyle Online Documentation}*/
routingStyle: EdgeRouterEdgeRoutingStyle;
/**
Gets or sets the {@link PenaltySettings} used for this edge.
Besides specifying a completely customized setting, the user can choose between four predefined optimization strategies:
{@link PenaltySettings#OPTIMIZATION_BALANCED}, {@link PenaltySettings#OPTIMIZATION_EDGE_BENDS}, {@link PenaltySettings#OPTIMIZATION_EDGE_CROSSINGS}
and {@link PenaltySettings#OPTIMIZATION_EDGE_LENGTHS}.
@throws {Exception} <b>(ArgumentError)</b> if the given penalty settings are <code>null</code>
@see PenaltySettings#OPTIMIZATION_BALANCED
@see PenaltySettings#OPTIMIZATION_EDGE_BENDS
@see PenaltySettings#OPTIMIZATION_EDGE_CROSSINGS
@see PenaltySettings#OPTIMIZATION_EDGE_LENGTHS
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23penaltySettings Online Documentation}*/
penaltySettings: PenaltySettings | null;
/**
Gets or sets whether or not this edge is routed directly to a group node's border if it connects the group node with one
of its descendants.
When enabled, the edge will connect to the inner side of the group node's border instead of leaving the group node and
connect from outside.
{@link PortCandidate}s as well as {@link PortConstraint}s are supported for direct content edges, including
<em>fixed</em> {@link PortCandidate}s and <em>strong</em> {@link PortConstraint}s. Note that, for example, a constraint with direction {@link PortSide#NORTH}
at the side of the enclosing group node means that the edge connects to the upper group node border <em>from inside</em> and never leaves
the group node.
<b>Warning:</b> This only applies to edges that connect to a group node from inside this group node.
<b>Warning:</b> The capacity of
{@link PortCandidateSet}s is not considered for direct group content edges. Capacity violations may occur when a
content edge shares a set with other edges.
<h2>Note</h2>
Edges that are completely overlapped by their source or target may require special attention during visualization. They
can disappear when edges are clipped at their source and target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23directGroupContentEdgeRouting Online Documentation}*/
directGroupContentEdgeRouting: boolean;
/**
Gets or sets the list of intermediate {@link YPoint}s that need to lie on the edge route.
The edge route is constructed such that all the specified intermediate points lie on the route in the order they are
specified. The route can, but must not necessarily bend at the specified points.
<b>Warning:</b>
The algorithm tries to avoid edges with self-crossings, but when specifying intermediate points it can not be guaranteed
that no such crossings occur.
<b>Warning:</b>
Consecutive points that are really close to each other can sometimes lead to low quality results and should thus be
avoided.
<h2>Note</h2>
Intermediate points are only considered for edges that are not part of a bus structure, see
{@link EdgeRouter#BUS_DESCRIPTOR_DP_KEY}.
@see EdgeRouter#BUS_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23intermediateRoutingPoints Online Documentation}*/
intermediateRoutingPoints: IList<any> | null;
/**
Gets or sets whether or not the points of the resulting edge path represent cubic bezier control points.
<ul>
<li><b>Disabled (default):</b> Each point of the path represents a normal polyline bend point.
<li><b>Enabled:</b> Four consecutive points p1, p2, p3, p4 of the resulting edge path need to be interpreted such that p1 is the start
point of the cubic bezier curve, p4 is the end point of it and p2, p3 are the two intermediate control points. If p4 is
not the last point of the edge path it is at the same time the start point of the next curve. For example, points p1,
p2, p3, p4, p5, p6, p7 form a path that consists of two curves: the curve defined by p1, p2, p3, p4 and the curve
defined by p4, p5, p6, p7.
</ul>
<h2>Note</h2>
This feature is useful when the visualization of the edge draws bezier curves instead of polyline paths.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23createControlPoints Online Documentation}*/
createControlPoints: boolean;
/**
Gets or sets the style which determines how curved edge paths connect at the source side of the edge.
@throws {Exception} <b>(ArgumentError)</b> if an unknown connection style is given
@see EdgeRouterEdgeLayoutDescriptor#targetCurveConnectionStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23sourceCurveConnectionStyle Online Documentation}*/
sourceCurveConnectionStyle: CurveConnectionStyle;
/**
Gets or sets the style using which determines how edge paths connect at the target side of the edge.
@throws {Exception} <b>(ArgumentError)</b> if an unknown connection style is given
@see EdgeRouterEdgeLayoutDescriptor#sourceCurveConnectionStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23targetCurveConnectionStyle Online Documentation}*/
targetCurveConnectionStyle: CurveConnectionStyle;
/**
Gets or sets the preferred length of non-orthogonal, octilinear segments.
If there is not enough space to use this preferred length, the segments may also be shorter.
The preferred length of non-orthogonal, octilinear segments has to be greater than or equal to <code>0</code>.
<b>Warning:</b> This setting applies only for edges with {@link EdgeRouterEdgeRoutingStyle#OCTILINEAR octilinear} routing.
@throws {Exception} <b>(ArgumentError)</b> if the preferred octilinear segment length is negative
@see EdgeRouterEdgeRoutingStyle#OCTILINEAR
@see EdgeRouterEdgeLayoutDescriptor#maximumOctilinearSegmentRatio
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23preferredOctilinearSegmentLength Online Documentation}*/
preferredOctilinearSegmentLength: number;
/**
Gets or sets the maximum ratio between the horizontal/vertical part of a segment and the non-orthogonal, octilinear
part.
When octilinear segments are added to an edge path, corners between horizontal and vertical segments are cut and
replaced by the new segment. This ratio describes the cutting points on a segment. If it is zero, the route stays
orthogonal. The value cannot be larger than <code>0.5</code> because there may be another non-orthogonal segment at the other side of
the segment. For long orthogonal segments the length of the octilinear segment is determined by the value of property {@link EdgeRouterEdgeLayoutDescriptor#preferredOctilinearSegmentLength}.
The maximum octilinear segment ratio must be between <code>0</code> and <code>0.5</code>.
<b>Warning:</b> This setting applies only for edges with {@link EdgeRouterEdgeRoutingStyle#OCTILINEAR octilinear} routing.
@throws {Exception} <b>(ArgumentError)</b> if the maximum ratio is negative or greater than <code>0.5</code>
@see EdgeRouterEdgeRoutingStyle#OCTILINEAR
@see EdgeRouterEdgeLayoutDescriptor#preferredOctilinearSegmentLength
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterEdgeLayoutDescriptor%23maximumOctilinearSegmentRatio Online Documentation}*/
maximumOctilinearSegmentRatio: number;
static $class: Class;
private $_EdgeLayoutDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeRouterEdgeLayoutDescriptor;
}
export interface PenaltySettings extends YObject {
}
/**
This class provides the cost penalties that are applied for violating restrictions during a {@link PathSearch}.
The {@link PathSearch} calculates the <em>shortest</em> path from the source to the target, where <em>shortest</em> path means the path with the
lowest costs. Costs result from violations of restrictions that are defined by the different
{@link PathSearchExtension}s. This class defines penalties for the various violations. The {@link PathSearch}
can be adapted to specific needs by varying these penalties. In order to avoid a certain violation, the appropriate
penalty must be increased.
There are some predefined penalty settings which set a different focus for the path search:
{@link PenaltySettings#OPTIMIZATION_BALANCED}, {@link PenaltySettings#OPTIMIZATION_EDGE_BENDS}, {@link PenaltySettings#OPTIMIZATION_EDGE_LENGTHS}
and {@link PenaltySettings#OPTIMIZATION_EDGE_CROSSINGS}.
<h2>Note</h2>
The penalties are not independent. They will all influence the {@link PathSearch} together. Therefore, changing a
certain penalty might not always lead to the desired effect for specific edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings Online Documentation}*/
export class PenaltySettings {
/**
Creates a new instance of {@link PenaltySettings}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings Online Documentation}*/
constructor();
/**
Creates a copy of this {@link PenaltySettings} instance.
This implementation uses method {@link PenaltySettings#newInstance} to obtain a new instance which adopts all
settings.
@returns a copy of this instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23createCopy() Online Documentation}*/
createCopy(): PenaltySettings | null;
/**
Creates a new instance of {@link PenaltySettings}.
This factory method is used by
{@link PenaltySettings#createCopy}. Subclasses can override this method in order to return a more specialized
version.
@returns a specialized {@link PenaltySettings}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23newInstance() Online Documentation}*/
newInstance(): PenaltySettings | null;
/**
Optimization strategy that balances bends and edge crossings in the edge routes.
By default, this strategy is used for the path search.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23OPTIMIZATION_BALANCED Online Documentation}*/
static OPTIMIZATION_BALANCED: PenaltySettings | null;
/**
Optimization strategy that minimizes bends in the edge routes.
Edges may cross other edges to prevent bends occurring around them.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23OPTIMIZATION_EDGE_BENDS Online Documentation}*/
static OPTIMIZATION_EDGE_BENDS: PenaltySettings | null;
/**
Optimization strategy that minimizes edge crossings in the edge routes.
This may cause more bends.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23OPTIMIZATION_EDGE_CROSSINGS Online Documentation}*/
static OPTIMIZATION_EDGE_CROSSINGS: PenaltySettings | null;
/**
Optimization strategy that minimizes the length of the edge routes.
In consequence, there may be more edge crossings.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23OPTIMIZATION_EDGE_LENGTHS Online Documentation}*/
static OPTIMIZATION_EDGE_LENGTHS: PenaltySettings | null;
/**
Gets or sets the penalty for the edge length.
This penalty will make long routes more expensive than short routes so edges will preferably be kept short.
Increasing the edge length penalty will raise the importance of short edges in relation to all other penalties while
decreasing this penalty will raise the importance of all other restrictions. A high edge length penalty will result in
routes that have less bends and more edge crossings to maintain the edge as short as possible.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23edgeLengthPenalty Online Documentation}*/
edgeLengthPenalty: number;
/**
Gets or sets the penalty for an edge bend.
This penalty will make routes with many bends more expensive than routes with few or no bends. Hence, edges will have
preferably few bends.
Increasing bend penalty will raise the importance of avoiding bends in relation to other penalties while decreasing this
penalty will raise the importance of all other restrictions. A high bend penalty will result in routes that will
preferably cross other edges instead of bending to avoid other edges. When increasing the edge bend penalty, the
resulting routes will have more edge crossings.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
<h2>Note</h2>
A very small bend penalty (less than
<code>1</code>) can lead to squiggles when the algorithm tries to avoid more expensive restrictions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23bendPenalty Online Documentation}*/
bendPenalty: number;
/**
Gets or sets the penalty for a crossing of two edges.
This penalty will make routes that cross many other edges more expensive than routes that cross few other edges or no
edges at all. Hence, edges preferably won't cross other edges.
Increasing the edge crossing penalty will raise the importance of avoiding edge crossings in relation to other
penalties, while decreasing this penalty will raise the importance of all other restrictions. When increasing the edge
crossing penalty, the resulting route will be longer and have more bends because crossing other edges needs to be
avoided.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23edgeCrossingPenalty Online Documentation}*/
edgeCrossingPenalty: number;
/**
Gets or sets the penalty for an edge crossing a regular node.
This penalty will make routes that cross normal nodes more expensive than routes that avoid crossing nodes. Hence, edges
won't preferably cross any nodes.
Increasing the node crossing penalty will raise the importance of avoiding node crossings in relation to other
penalties, while decreasing this penalty will raise the importance of all other restrictions.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23nodeCrossingPenalty Online Documentation}*/
nodeCrossingPenalty: number;
/**
Gets or sets the penalty for an edge crossing a group node.
This penalty will make routes that cross group nodes more expensive than routes that avoid crossing group nodes. Hence,
edges won't preferably cross any group nodes.
Increasing the group node crossing penalty will raise the importance of avoiding group node crossings in relation to
other penalties, while decreasing this penalty will raise the importance of all other restrictions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23groupNodeCrossingPenalty Online Documentation}*/
groupNodeCrossingPenalty: number;
/**
Gets or sets the penalty for an edge crossing a node label.
This penalty will make routes that cross node labels more expensive than routes that avoid crossing node labels. Hence,
edges won't preferably cross any node labels.
Increasing the node label crossing penalty will raise the importance of avoiding node labels crossings in relation to
other penalties, while decreasing this penalty will raise the importance of all other restrictions. A high label
crossing penalty will cause edges to bend more often to avoid passing straight through labels.
The crossing penalty can be weighted for each label individually using the
{@link EdgeRouter#LABEL_CROSSING_PENALTY_FACTOR_DP_KEY}.
Use {@link EdgeRouter#considerNodeLabels} to set whether node labels should be considered when routing the edges.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
<h2>Note</h2>
This only applies to labels that are placed outside the bounds of their owner nodes as long as the {@link PenaltySettings#nodeCrossingPenalty node crossing penalty}
is higher than the node label crossing penalty. Unless the route connects from outside a group to one of its inner
nodes, the node label will also be considered.
@see EdgeRouter#considerNodeLabels
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23nodeLabelCrossingPenalty Online Documentation}*/
nodeLabelCrossingPenalty: number;
/**
Gets or sets the penalty for an edge crossing an edge label.
This penalty will make routes that cross edge labels of fixed edges more expensive than routes that avoid crossing edge
labels, so edges won't preferably cross any edge labels that belong to fixed edges.
Increasing the edge label crossing penalty will raise the importance of avoiding edge labels crossings in relation to
other penalties, while decreasing this penalty will raise the importance of all other restrictions. A high label
crossing penalty will cause edges to bend more often to avoid passing straight through labels.
The crossing penalty can be weighted for each label individually using the
{@link EdgeRouter#LABEL_CROSSING_PENALTY_FACTOR_DP_KEY}.
Use {@link EdgeRouter#considerEdgeLabels} to set whether edge labels should be considered when routing the edges.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see EdgeRouter#considerEdgeLabels
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23edgeLabelCrossingPenalty Online Documentation}*/
edgeLabelCrossingPenalty: number;
/**
Gets or sets the penalty for violations of the minimum distance between any edge and any normal node side.
This penalty will make routes that pass too close to normal nodes more expensive than routes that maintain the specified
minimum node to edge distance.
Increasing the minimum node to edge distance penalty will raise the importance of maintaining the minimum distance to
nodes, while decreasing this penalty will raise the importance of all other restrictions. A high penalty for violating
the minimum distance between edges and nodes will produce edges that rather use a long route around nodes to maintain
the specified distance than pass through a small channel between two nodes.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see EdgeRouter#minimumNodeToEdgeDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23minimumNodeToEdgeDistancePenalty Online Documentation}*/
minimumNodeToEdgeDistancePenalty: number;
/**
Gets or sets the penalty for violations of the minimum distance between any edge and any group node side.
This penalty will make routes that pass group nodes too close more expensive than routes that maintain the specified
minimum node to edge distance.
Increasing the minimum group node to edge distance penalty will raise the importance of maintaining the minimum distance
to group nodes, while decreasing this penalty will raise the importance of all other restrictions. A high penalty for
violating the minimum distance between edges and group nodes will produce edges that rather use a long route around
group nodes to maintain the specified distance than pass through a small channel between two group nodes.
Use {@link EdgeRouter#minimumNodeToEdgeDistance} to set the minimum distance.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see EdgeRouter#minimumNodeToEdgeDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23minimumGroupNodeToEdgeDistancePenalty Online Documentation}*/
minimumGroupNodeToEdgeDistancePenalty: number;
/**
Gets or sets the penalty for violations of the minimum distance between any two edges.
This penalty will make routes that pass other edges too close more expensive than routes that maintain the specified
minimum edge to edge distance.
Increasing the minimum edge to edge distance penalty will raise the importance of maintaining the minimum distance to
other edges, while decreasing this penalty will raise the importance of all other restrictions. When this penalty has a
high value, edges will maintain the specified distance. This may also reduce the number of edges that pass through a
small channel between nodes. So, if such a channel is part of the shortest routes for many edges, some of them will take
a longer way.
Use {@link EdgeRouterEdgeLayoutDescriptor#minimumEdgeToEdgeDistance} to set the minimum distance for a certain edge to any other edge.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see EdgeRouterEdgeLayoutDescriptor#minimumEdgeToEdgeDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23minimumEdgeToEdgeDistancePenalty Online Documentation}*/
minimumEdgeToEdgeDistancePenalty: number;
/**
Gets or sets the penalty for violations of the minimum distance the edge shall maintain from node corners when entering
or leaving the node.
This penalty will make routes that end too close to the corner of their source or target node more expensive than routes
that maintain the specified minimum node corner distance.
Increasing the node corner distance penalty will raise the importance of maintaining the minimum distance to the corner
of the adjacent nodes, while decreasing this penalty will raise the importance of all other restrictions.
Use {@link EdgeRouterEdgeLayoutDescriptor#minimumNodeCornerDistance} to set the minimum node corner distance for a certain edge.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see EdgeRouterEdgeLayoutDescriptor#minimumNodeCornerDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23minimumNodeCornerDistancePenalty Online Documentation}*/
minimumNodeCornerDistancePenalty: number;
/**
Gets or sets the penalty for violations of the minimum length of the first and last segment of an edge.
This penalty will make routes whose first/last bend is too close to their source or target node more expensive than
routes that maintain the specified minimum first/last segment length.
Increasing the minimum first last segment length penalty will raise the importance of maintaining the first and last
segment as long as they fit the minimum first/last segment length, while decreasing this penalty will raise the
importance of all other restrictions.
Use {@link EdgeRouterEdgeLayoutDescriptor#minimumFirstSegmentLength} to set the minimum length of the first segment and {@link EdgeRouterEdgeLayoutDescriptor#minimumLastSegmentLength} to set the minimum length of the last segment of a certain
edge.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see EdgeRouterEdgeLayoutDescriptor#minimumFirstSegmentLength
@see EdgeRouterEdgeLayoutDescriptor#minimumLastSegmentLength
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23minimumFirstLastSegmentLengthPenalty Online Documentation}*/
minimumFirstLastSegmentLengthPenalty: number;
/**
Gets or sets the penalty for an edge bend being closer to the border of the source node, the target node or one of their
ancestor group nodes than the minimum node to edge distance.
This penalty will make routes that bend too close to their source or target node more expensive than routes that
maintain the specified minimum node to edge distance.
Increasing the bends in node to edge distance penalty will raise the importance of avoiding bends that violate the
minimum node to edge distance, while decreasing this penalty will raise the importance of all other restrictions. This
penalty supports
{@link EdgeRouter#minimumNodeToEdgeDistance}. In case the path search finally reaches the target node, the minimum node to edge distance for this node must be
violated. Penalizing bends that are too close to the node forces the edge to directly cross this distance and connect to
the node.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see EdgeRouter#minimumNodeToEdgeDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23bendsInNodeToEdgeDistancePenalty Online Documentation}*/
bendsInNodeToEdgeDistancePenalty: number;
/**
Gets or sets the penalty for violating the monotonic path restrictions of an edge.
This penalty will make routes with detours m ore expensive than routes that stay monotone.
Increasing the monotony violation penalty will raise the importance of avoiding detours in the specified direction,
while decreasing this penalty will raise the importance of all other restrictions.
Use {@link EdgeRouterEdgeLayoutDescriptor#monotonicPathRestriction} to set the monotonic path restrictions of a certain edge.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see EdgeRouterEdgeLayoutDescriptor#monotonicPathRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23monotonyViolationPenalty Online Documentation}*/
monotonyViolationPenalty: number;
/**
Gets or sets the penalty for an edge leaving and reentering in the same {@link PartitionGrid partition grid} cell.
This penalty will make routes that re-enter the same partition grid cell more expensive than routes taking a more direct
way.
Increasing the partition grid reentrance penalty will raise the importance of taking the most direct way through the
partition grid, while decreasing this penalty will raise the importance of all other restrictions.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see PartitionGrid
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23partitionGridCellReentrancePenalty Online Documentation}*/
partitionGridCellReentrancePenalty: number;
/**
Gets or sets the penalty if an edge does not comply with its {@link PortCandidate}s or
{@link PortConstraint}s.
This penalty will make routes that start or end at a point that doesn't comply with its port constraints more expensive
than routes that obey them.
Increasing the port violation penalty will raise the importance of maintaining port constraints, while decreasing this
penalty will raise the importance of all other restrictions. It is recommended to have high penalties for port violation
since the constraints lose purpose if they are disregarded.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see PortCandidate
@see PortConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23portViolationPenalty Online Documentation}*/
portViolationPenalty: number;
/**
Gets or sets the penalty for an edge with a group ID that is not grouped at source or target side.
This penalty will make routes for grouped edges that don't use partly the same path as the other edges in this group
more expensive than routes that stick to the group.
Increasing the invalid edge grouping penalty will raise the importance of grouping edges with the same group ID, while
decreasing this penalty will raise the importance of all other restrictions. As edge groups require that the route of an
edge belonging to an edge group stays partly identical to the other edges of the group, the route may not be optimal
with respect to the other restrictions. To maintain the edges together as long as possible, this penalty should be
higher than most others.
Grouped edges have the same source or target group ID assigned in the data provider registered with the graph with the {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
or {@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY} key.
The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23invalidEdgeGroupingPenalty Online Documentation}*/
invalidEdgeGroupingPenalty: number;
/**
Gets or sets the penalty for bends of a self-loop edge.
This penalty will make self-loops leave and enter the node at different sides unless they are forced to one side by
{@link PortConstraint}s or {@link PortCandidate}s.
Increasing this penalty will stronger enforce routing the self-loops between two sides of the node but it will also
require more time for determining the edge path. Decreasing the penalty may result in self-loops that only connect to
one side of the node.
By default this value is set to <code>9</code>. The value must be <code>>= 0</code>, otherwise the default value will be assigned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PenaltySettings%23singleSideSelfLoopPenalty Online Documentation}*/
singleSideSelfLoopPenalty: number;
static $class: Class;
private $_PenaltySettings: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PenaltySettings;
}
export interface PortCalculator extends LayoutStageBase, ILayoutStage {
}
/**
A {@link PortCalculator} can be applied to move ports from the nodes' rectangular bounds to the actual outline.
Layout algorithms only consider rectangular nodes even though the actual shape of a node is, for example, circular.
Hence, without adding specific strong {@link PortConstraint}s, the ports are usually placed at the border of the
nodes' {@link LayoutGraph#getBoundingBox bounding boxes} (except for some layout algorithms that produce straight-line edge routes and place the ports at
the nodes' center). A {@link PortCalculator} may wrap a {@link LayoutStageBase#coreLayout core layout algorithm} to adjust the port assignment after
the layout calculation. For the calculation of the new port offsets, the {@link PortCalculator} delegates to
instances of interface {@link IIntersectionCalculator}. To specify such an instance for the source/target of an
edge, use a {@link IDataProvider} that is registered to the graph with key {@link IntersectionCalculatorKeys#SOURCE_INTERSECTION_CALCULATOR_DP_KEY}
and {@link IntersectionCalculatorKeys#TARGET_INTERSECTION_CALCULATOR_DP_KEY}, respectively. Note that the
intersection point (i.e., port) of an edge and the associated source/target node provided by method {@link IIntersectionCalculator#calculateIntersectionPoint}
will be added to the edge's {@link IEdgeLayout}.
<h2>Note</h2>
This class will not change the coordinates of a port if it is associated with a strong port constraint.
<h2>Note</h2>
A {@link PortCalculator} will automatically be applied if the layout is executed using a {@link LayoutExecutor}
and the PortAdjustmentPolicy is set to another value than {@link PortAdjustmentPolicy#NEVER}.
@see IIntersectionCalculator
@see IntersectionCalculatorKeys#SOURCE_INTERSECTION_CALCULATOR_DP_KEY
@see IntersectionCalculatorKeys#TARGET_INTERSECTION_CALCULATOR_DP_KEY
@see PortConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCalculator Online Documentation}*/
export class PortCalculator {
/**
Creates a new {@link PortCalculator} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCalculator Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Adjusts the ports after invoking the {@link LayoutStageBase#coreLayout core layout algorithm}.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCalculator%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Performs the actual port calculation on the specified graph instance.
@param graph the input graph
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCalculator%23calculatePorts(LayoutGraph) Online Documentation}*/
calculatePorts(graph: LayoutGraph | null): void;
/**
Determines whether two points are equal.
This implementation uses {@link PortCalculator#Eps} to add some tolerance to the comparison. It is called by {@link PortCalculator#calculatePorts}
and may be overridden to change the accuracy/tolerance of the comparison.
@param x1 the x-coordinate of the first point
@param y1 the y-coordinate of the first point
@param x2 the x-coordinate of the second point
@param y2 the y-coordinate of the second point
@returns <code>true</code> if both points are considered equal, <code>false</code> otherwise
@see PortCalculator#calculatePorts
@see PortCalculator#Eps
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCalculator%23equalsEps(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
equalsEps(x1: number, y1: number, x2: number, y2: number): boolean;
/**
EPSILON used by {@link PortCalculator#equalsEps}.
By default this is <code>0.2d</code>.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCalculator%23Eps Online Documentation}*/
Eps: number;
static $class: Class;
private $_PortCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortCalculator;
}
export interface PortCandidate extends YObject {
}
/**
This class represents a candidate port on which edges can be connected to their source and/or target node.
The notion of {@link PortCandidate}s extends the one of {@link PortConstraint}s. In contrast to
{@link PortConstraint}s, they can be associated with both edges and nodes. Moreover, more than one {@link PortCandidate}
can be specified for an edge or a node.
<b>Edge</b>
{@link PortCandidate}s can be introduced to define specific sides or exact locations on which a given edge can be
connected to its source or target node.
Information about edge {@link PortCandidate}s is held by {@link IDataProvider}s that are registered with
the graph using keys {@link PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY} (to specify
{@link PortCandidate}s for an edge on the source node) and {@link PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY}
(to specify {@link PortCandidate}s for an edge on the target node). The {@link IDataProvider}s hold a {@link ICollection collection}
of {@link PortCandidate}s for each edge.
<b>Node</b> {@link PortCandidate}s determine available ports on the nodes to which edges can be connected. This means that an
edge of the graph can be connected to any of these node ports (unless edge
{@link PortCandidate}s are specified too).
To register and define node {@link PortCandidate}s refer to {@link PortCandidateSet}.
A {@link PortCandidate} is described by the following attributes:
<ul>
<li>Offsets: The X-offset (Y-offset) determines the horizontal (vertical) distance of the {@link PortCandidate}
relative to the center of the node.
<li>Fixed or free: Fixed
{@link PortCandidate}s are defined using exact coordinates, whereas coordinates of free candidates are selected
arbitrarily.
<li>Direction: Corresponds to the incoming or outgoing directions of the edges when connecting to a target or source node,
respectively.
<li>Penalty cost: Represents the cost of usage of a
{@link PortCandidate}. During the processing of an edge by a layout or routing algorithm, candidates of lower cost
will be considered first.
</ul>
If candidates are defined for nodes as well as for edges and if they should be properly matched by layout algorithms
(i.e. detected as the same candidate), then it is important that the same {@link PortCandidate} instance is
registered. It does not suffice to register candidates with the same parameter values.
Not all layout and routing algorithms are able to handle {@link PortCandidate}s. Major layout algorithms that obey
them are {@link HierarchicLayout}, {@link EdgeRouter}, {@link ChannelEdgeRouter} and
{@link BusRouter}.
<h2>Note</h2>
In case that both {@link PortConstraint}s and
{@link PortCandidate}s (on edges and/or nodes) are defined for a graph, the layout or the routing algorithm will
try to match both of them in order to find an appropriate port. The matching process depends on the actual algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate Online Documentation}*/
export class PortCandidate {
/**
Factory method that creates a fixed {@link PortCandidate} instance with the given cost of usage, direction and
offset values.
@param directionMask any combination of the default direction specifiers
@param xOffset the X-offset relative to the center of the node
@param yOffset the Y-offset relative to the center of the node
@param [cost=0.0] the penalty cost for using this {@link PortCandidate} instance
@returns a new {@link PortCandidate} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23createCandidate(number%2Cnumber%2CPortDirections%2Cnumber) Online Documentation}*/
static createCandidate(xOffset: number, yOffset: number, directionMask: PortDirections | PortDirectionsConvertible, cost?: number): PortCandidate | null;
/**
Factory method that returns a free {@link PortCandidate} instance with zero cost of usage and the direction
specified by the {@link PortConstraint#side side} of the given {@link PortConstraint}.
@param fromPortConstraint the {@link PortConstraint} whose {@link PortConstraint#side side} should be used as direction
@returns a new {@link PortCandidate} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23createCandidate(PortConstraint) Online Documentation}*/
static createCandidate(fromPortConstraint: PortConstraint | null): PortCandidate | null;
/**
Factory method that creates a free {@link PortCandidate} instance with the given direction and cost of usage.
@param directionMask any combination of the default direction specifiers
@param [cost=0] the penalty cost for using this {@link PortCandidate}
@returns a new {@link PortCandidate} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23createCandidate(PortDirections%2Cnumber) Online Documentation}*/
static createCandidate(directionMask: PortDirections | PortDirectionsConvertible, cost?: number): PortCandidate | null;
/**
Returns whether or not this {@link PortCandidate} is equal to another {@link PortCandidate} based on their
internal attributes.
@param other the reference {@link PortCandidate} object with which to compare
@returns <code>true</code> if this object is the same as the {@link PortCandidate} argument, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Returns the direction of this {@link PortCandidate} instance with respect to the given {@link OrientationLayout#orientation layout orientation}.
If the direction mask is not required to be calculated with respect to the actual {@link OrientationLayout#orientation layout orientation}, {@link PortCandidate#direction} should be used
instead.
@param layoutOrientation
@returns the direction of this {@link PortCandidate} with respect to the given layout orientation
@see PortCandidate#direction
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23getDirectionForLayoutOrientation(LayoutOrientation) Online Documentation}*/
getDirectionForLayoutOrientation(layoutOrientation: LayoutOrientation | LayoutOrientationStringValues): PortDirections;
/**
Returns the X-offset of the {@link PortCandidate} with respect to the center of the corresponding node for the
given {@link OrientationLayout#orientation layout orientation} and {@link OrientationLayout#mirrorMode mirror mask}.
The X-offset encodes the horizontal distance of the port relative to the center of a node. If the {@link PortCandidate}
is not {@link PortCandidate#fixed fixed} (i.e., free), the X-offset is equal to zero.
<h2>Note</h2>
If the X-offset is not required to be calculated for the actual {@link OrientationLayout#orientation layout orientation}, {@link PortCandidate#xOffset} should be used instead.
@param layoutOrientation the layout orientation
@param mirrorMask the mirror mask
@returns the X-offset of the port with respect to the given orientation and mirror mask
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23getXOffsetForLayoutOrientation(LayoutOrientation%2CMirrorModes) Online Documentation}*/
getXOffsetForLayoutOrientation(layoutOrientation: LayoutOrientation | LayoutOrientationStringValues, mirrorMask: MirrorModes | MirrorModesConvertible): number;
/**
Returns the X-offset of the {@link PortCandidate} with respect to the center of the corresponding node for the
given {@link OrientationLayout#orientation layout orientation}.
The X-offset encodes the horizontal distance of the port relative to the center of a node. If the {@link PortCandidate}
is not {@link PortCandidate#fixed fixed} (i.e., free), the X-offset is equal to zero.
<h2>Note</h2>
If the X-offset is not required to be calculated for the actual {@link OrientationLayout#orientation layout orientation}, {@link PortCandidate#xOffset} should be used instead.
@param layoutOrientation the layout orientation
@returns the X-offset of the {@link PortCandidate} with respect to the given layout orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23getXOffsetForLayoutOrientation(LayoutOrientation) Online Documentation}*/
getXOffsetForLayoutOrientation(layoutOrientation: LayoutOrientation | LayoutOrientationStringValues): number;
/**
Returns the Y-offset of the {@link PortCandidate} with respect to the center of the corresponding node for the
given {@link OrientationLayout#orientation layout orientation} and {@link OrientationLayout#mirrorMode mirror mask}.
The Y-offset encodes the vertical distance of the port relative to the center of a node. If the {@link PortCandidate}
is not {@link PortCandidate#fixed fixed} (i.e., free), the Y-offset is equal to zero.
<h2>Note</h2>
If the Y-offset is not required to be calculated for the actual {@link OrientationLayout#orientation layout orientation}, {@link PortCandidate#yOffset} should be used instead.
@param layoutOrientation the layout orientation
@param mirrorMask the mirror mask
@returns the Y-offset of the {@link PortCandidate} with respect to the given orientation and mirror mask
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23getYOffsetForLayoutOrientation(LayoutOrientation%2CMirrorModes) Online Documentation}*/
getYOffsetForLayoutOrientation(layoutOrientation: LayoutOrientation | LayoutOrientationStringValues, mirrorMask: MirrorModes | MirrorModesConvertible): number;
/**
Returns the Y-offset of the {@link PortCandidate} with respect to the center of the corresponding node for the
given {@link OrientationLayout#orientation layout orientation}.
The Y-offset encodes the vertical distance of the port relative to the center of a node. If the {@link PortCandidate}
is not {@link PortCandidate#fixed fixed} (i.e., free), the Y-offset is equal to zero.
<h2>Note</h2>
If the Y-offset is not required to be calculated for the actual {@link OrientationLayout#orientation layout orientation}, {@link PortCandidate#yOffset} should be used instead.
@param layoutOrientation the layout orientation
@returns the Y-offset of the {@link PortCandidate} with respect to the given layout orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23getYOffsetForLayoutOrientation(LayoutOrientation) Online Documentation}*/
getYOffsetForLayoutOrientation(layoutOrientation: LayoutOrientation | LayoutOrientationStringValues): number;
/**
Returns a hash code value for this {@link PortCandidate} instance based on the internal attributes.
@returns a hash code value
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Returns whether or not this {@link PortCandidate} lies in the given direction.
@param directionMask one of the default direction specifiers
@returns <code>true</code> if this {@link PortCandidate} lies in the given direction, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23isInDirection(PortDirections) Online Documentation}*/
isInDirection(directionMask: PortDirections | PortDirectionsConvertible): boolean;
/**
A utility method that converts the current {@link PortCandidate} instance to a {@link PortConstraint}.
The {@link PortConstraint#side side} of the created {@link PortConstraint} matches the {@link PortCandidate#direction direction} of this {@link PortCandidate}.
If this {@link PortCandidate} is {@link PortCandidate#fixed fixed}, the resulting {@link PortConstraint} will be {@link PortConstraint#strong strong}.
<h2>Note</h2>
If the matching {@link PortConstraint} should be determined with respect to a certain {@link OrientationLayout#orientation layout orientation}, method {@link PortCandidate#toPortConstraintForLayoutOrientation}
should be used instead.
@returns a {@link PortConstraint} that matches this {@link PortCandidate}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23toPortConstraint() Online Documentation}*/
toPortConstraint(): PortConstraint | null;
/**
A utility method that converts the current {@link PortCandidate} instance to a {@link PortConstraint} with
respect to the given {@link OrientationLayout#orientation layout orientation}.
The {@link PortConstraint#side side} of the created {@link PortConstraint} is the {@link PortCandidate#direction direction} of this {@link PortCandidate} interpreted
for the given orientation.
If this {@link PortCandidate} is {@link PortCandidate#fixed fixed}, the resulting {@link PortConstraint} will be {@link PortConstraint#strong strong}.
<h2>Note</h2>
If the matching {@link PortConstraint} should not be determined with respect to a certain {@link OrientationLayout#orientation layout orientation},
method {@link PortCandidate#toPortConstraint} should be used instead.
@param layoutOrientation
@returns a {@link PortConstraint} that matches this {@link PortCandidate} with respect to the given orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23toPortConstraintForLayoutOrientation(LayoutOrientation) Online Documentation}*/
toPortConstraintForLayoutOrientation(layoutOrientation: LayoutOrientation | LayoutOrientationStringValues): PortConstraint | null;
/**
Returns a string representation of this {@link PortCandidate} instance.
@returns a string representation of the {@link PortCandidate}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23toString() Online Documentation}*/
toString(): string;
/**
Data provider key for specifying source port candidates for each edge
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY Online Documentation}*/
static SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY: EdgeDpKey<ICollection<PortCandidate>> | null;
/**
Data provider key for specifying target port candidates for each edge
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY Online Documentation}*/
static TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY: EdgeDpKey<ICollection<PortCandidate>> | null;
/**
Gets the direction of this {@link PortCandidate}.
If the direction is required to be calculated with respect to the actual {@link OrientationLayout#orientation layout orientation}, {@link PortCandidate#getDirectionForLayoutOrientation}
should be used instead.
@see PortCandidate#getDirectionForLayoutOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23direction Online Documentation}*/
readonly direction: PortDirections;
/**
Gets whether or not this {@link PortCandidate} is fixed.
Fixed
{@link PortCandidate}s are defined using exact coordinates, whereas coordinates of free candidates are selected
arbitrarily.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23fixed Online Documentation}*/
readonly fixed: boolean;
/**
Gets the X-offset of the {@link PortCandidate} with respect to the center of the corresponding node.
The X-offset encodes the horizontal distance of the port relative to the center of a node. If the {@link PortCandidate}
is not {@link PortCandidate#fixed fixed} (i.e., free), the X-offset is zero.
<h2>Note</h2>
If the X-offset is required to be calculated with respect to the actual {@link OrientationLayout#orientation layout orientation}, {@link PortCandidate#getXOffsetForLayoutOrientation}
should be used instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23xOffset Online Documentation}*/
readonly xOffset: number;
/**
Gets the Y-offset of the {@link PortCandidate} with respect to the center of the corresponding node.
The Y-offset encodes the vertical distance of the port relative to the center of a node. If the {@link PortCandidate}
is not {@link PortCandidate#fixed fixed} (i.e., free), the Y-offset is zero.
<h2>Note</h2>
If the Y-offset is required to be calculated with respect to the actual {@link OrientationLayout#orientation layout orientation}, method {@link PortCandidate#getYOffsetForLayoutOrientation}
should be used instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23yOffset Online Documentation}*/
readonly yOffset: number;
/**
Gets the penalty cost for using this {@link PortCandidate} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidate%23cost Online Documentation}*/
readonly cost: number;
static $class: Class;
private $_PortCandidate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortCandidate;
}
export interface IPortCandidateMatcher {
}
/**
A {@link IPortCandidateMatcher} retrieves the best matching {@link IPortCandidateSetEntry entries} from a {@link PortCandidateSet}.
How well a candidate is matching can be decided based on its cost, its currently available capacity, its offsets or
whether or not its direction is consistent with the desired one.
To use custom implementations of this interface, method {@link PortCandidateSet#createMatcher} should be
overridden.
<h2>Note</h2>
While the {@link EdgeRouter} supports {@link PortCandidateSet}s, it doesn't support custom {@link IPortCandidateMatcher}
implementations.
<h2>Note</h2>
The {@link HierarchicLayout} supports {@link PortCandidateSet}s but uses the {@link IPortCandidateMatcher}
only to determine fallback candidates within class {@link PortCandidateOptimizer}. For an advanced
customization of the port assignment strategy users may implement a custom
{@link PortConstraintOptimizerBase}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateMatcher Online Documentation}*/
export abstract class IPortCandidateMatcher {
/**
Returns the next best matching {@link IPortCandidateSetEntry} for the given parameters.
@param edge the edge for which to find a candidate
@param source <code>true</code> if the given edge connects to its source node, <code>false</code> otherwise
@param xOffset the preferred X-offset of the port relative to the center of the node
@param yOffset the preferred Y-offset of the port relative to the center of the node
@param directionMask any combination of default directions defined in {@link PortCandidate}
@returns a matching {@link IPortCandidateSetEntry} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateMatcher%23findMatchingCandidate(Edge%2Cboolean%2Cnumber%2Cnumber%2CPortDirections) Online Documentation}*/
abstract findMatchingCandidate(edge: Edge | null, source: boolean, xOffset: number, yOffset: number, directionMask: PortDirections): IPortCandidateSetEntry | null;
/**
Returns the next best matching {@link IPortCandidateSetEntry} for the given parameters.
@param edge the edge for which to find a candidate
@param source <code>true</code> if the given edge connects to its source node, <code>false</code> otherwise
@param directionMask any combination of default directions defined in {@link PortCandidate}
@returns a matching {@link IPortCandidateSetEntry} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateMatcher%23findMatchingCandidate(Edge%2Cboolean%2CPortDirections) Online Documentation}*/
abstract findMatchingCandidate(edge: Edge | null, source: boolean, directionMask: PortDirections): IPortCandidateSetEntry | null;
/**
Returns the next best matching {@link IPortCandidateSetEntry}.
@returns a matching {@link IPortCandidateSetEntry} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateMatcher%23findMatchingCandidate() Online Documentation}*/
abstract findMatchingCandidate(): IPortCandidateSetEntry | null;
static $class: Class;
private $_IPortCandidateMatcher: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortCandidateMatcher;
}
export interface IPortCandidateSetEntry {
}
/**
This interface represents an entry of a {@link PortCandidateSet} and provides access to the actual {@link PortCandidate}
instance and the maximum number of edges allowed to connect to it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateSetEntry Online Documentation}*/
export abstract class IPortCandidateSetEntry {
/**
Gets the actual {@link PortCandidate} associated with this {@link IPortCandidateSetEntry}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateSetEntry%23portCandidate Online Documentation}*/
abstract readonly portCandidate: PortCandidate | null;
/**
Gets the maximum number of edges allowed to connect to the {@link PortCandidate} associated with this
{@link IPortCandidateSetEntry}.
<code>0x7FFFFFFF</code> indicates that the number of connections is unbounded.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortCandidateSetEntry%23connections Online Documentation}*/
abstract readonly connections: number;
static $class: Class;
private $_IPortCandidateSetEntry: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortCandidateSetEntry;
}
export interface PortCandidateSet extends YObject {
}
/**
This class associates {@link PortCandidate}s with the nodes of a graph.
{@link PortCandidate}s defined on nodes specify sides or even exact locations at which edges can be connected.
This means that an edge can be connected to any of the available node ports (unless edge {@link PortCandidate}s
are specified too). Each node {@link PortCandidate} has a capacity, i.e., a maximum number of edges allowed to
connect to it.
Information about {@link PortCandidate}s on nodes is held by a {@link IDataProvider} that stores a {@link PortCandidateSet}
for each node and is registered with the graph using key {@link PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY}.
The {@link PortCandidate}s are retrieved from the {@link PortCandidateSet} through a {@link IPortCandidateSetEntry}
that holds the information about the actual {@link PortCandidate}. This object is also used for retrieving the
best candidate from a {@link PortCandidateSet} based on the cost (lower costs are considered first) and the
available capacity of each candidate. Custom searching for the best {@link PortCandidate} can be achieved by
implementing {@link IPortCandidateMatcher} interface.
Not all layout and routing algorithms are able to handle {@link PortCandidate}s. Major layout algorithms that obey
them are {@link HierarchicLayout}, {@link EdgeRouter}, {@link ChannelEdgeRouter} and
{@link BusRouter}.
<h2>Note</h2>
In case that both {@link PortConstraint}s and
{@link PortCandidate}s (on edges and/or nodes) are defined for a graph, the layout or the routing algorithm will
try to match both of them in order to find an appropriate port. The matching process depends on the actual algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet Online Documentation}*/
export class PortCandidateSet {
/**
Creates a new empty {@link PortCandidateSet} instance.
Addition of {@link PortCandidate}s to this {@link PortCandidateSet set} can be performed using {@link PortCandidateSet#add}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet Online Documentation}*/
constructor();
/**
Adds a {@link PortCandidate} with the given capacity to this {@link PortCandidateSet}.
The capacity value defines the maximum number of edges allowed to connect to the candidate. If no limitation for this
number exists, <code>0x7FFFFFFF</code> can be used.
The capacity must be greater than zero.
@param p the {@link PortCandidate} to add
@param [connections=1] the capacity of the given port
@throws {Exception} <b>(ArgumentError)</b> if the given capacity is less than or equal to zero
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet%23add(PortCandidate%2Cnumber) Online Documentation}*/
add(p: PortCandidate | null, connections?: number): void;
/**
Creates a {@link IPortCandidateMatcher} that retrieves {@link IPortCandidateSetEntry entries} from this {@link PortCandidateSet}.
Each next best matching candidate is determined based on its cost, its current available capacity and based on whether
or not its direction is consistent with the desired one.
This method may be overridden to support custom implementations of {@link IPortCandidateMatcher}.
<h2>Note</h2>
While the {@link EdgeRouter} supports {@link PortCandidateSet}s, it doesn't support custom {@link IPortCandidateMatcher}
implementations.
<h2>Note</h2>
The {@link HierarchicLayout} supports {@link PortCandidateSet}s but uses the {@link IPortCandidateMatcher}
only to determine fallback candidates within class {@link PortCandidateOptimizer}. For an advanced
customization of the port assignment strategy users may implement a custom
{@link PortConstraintOptimizerBase}.
@returns a new {@link IPortCandidateMatcher} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet%23createMatcher() Online Documentation}*/
createMatcher(): IPortCandidateMatcher | null;
/**
Calculates the penalty cost for the usage of the given {@link IPortCandidateSetEntry entry} and the given parameters.
The cost is equal to the {@link PortCandidate#cost cost of usage} of the {@link IPortCandidateSetEntry#portCandidate associated port candidate}.
This method may be overridden to customize the calculation of the cost of usage of an {@link IPortCandidateSetEntry entry}.
@param entry the entry for which the cost is computed
@param edge the edge that will connect to the {@link PortCandidate} associated with the entry
@param source <code>true</code> if the given edge connects to its source node, <code>false</code> otherwise
@param xOffset the preferred X-offset of the port relative to the center of the node
@param yOffset the preferred Y-offset of the port relative to the center of the node
@returns the penalty cost for the usage of the given {@link IPortCandidateSetEntry}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet%23getCost(IPortCandidateSetEntry%2CEdge%2Cboolean%2Cnumber%2Cnumber) Online Documentation}*/
getCost(entry: IPortCandidateSetEntry | null, edge: Edge | null, source: boolean, xOffset: number, yOffset: number): number;
/**
Calculates the priority of the given {@link IPortCandidateSetEntry entry}.
Higher priority means that the entry will be considered earlier by a {@link IPortCandidateMatcher}.
This method may be overridden to customize the calculation of the priority of {@link IPortCandidateSetEntry entries}.
@param entry the entry for which the cost is computed
@param edge the edge that will connect to the {@link PortCandidate} associated with the entry
@param source <code>true</code> if the given edge connects to its source node, <code>false</code> otherwise
@returns the priority of the given {@link IPortCandidateSetEntry}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet%23getPriority(IPortCandidateSetEntry%2CEdge%2Cboolean) Online Documentation}*/
getPriority(entry: IPortCandidateSetEntry | null, edge: Edge | null, source: boolean): number;
/**
Removes the given {@link IPortCandidateSetEntry entry} from the list of candidate entries.
@param entry the entry to remove
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet%23remove(IPortCandidateSetEntry) Online Documentation}*/
remove(entry: IPortCandidateSetEntry | null): void;
/**
Data provider key for specifying a port candidate set for each node
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet%23NODE_PORT_CANDIDATE_SET_DP_KEY Online Documentation}*/
static NODE_PORT_CANDIDATE_SET_DP_KEY: NodeDpKey<PortCandidateSet> | null;
/**
Gets the total number of connections allowed by the {@link PortCandidate}s described by this {@link PortCandidateSet}
or <code>0x7FFFFFFF</code> if the number is unbounded.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet%23connectionCount Online Documentation}*/
readonly connectionCount: number;
/**
Gets an {@link IEnumerator} over the list of candidate {@link IPortCandidateSetEntry entries} of this {@link PortCandidateSet}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateSet%23entries Online Documentation}*/
readonly entries: IEnumerator<any> | null;
static $class: Class;
private $_PortCandidateSet: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortCandidateSet;
}
export interface PortConstraint extends YObject {
}
/**
A {@link PortConstraint} represents a constraint for the source or target port of an edge path.
It determines the location at which an edge path can be connected to its source and/or target node. There exist two
different types of {@link PortConstraint}s:
<ul>
<li><b>Weak</b> {@link PortConstraint}s specify the side where an edge should enter/leave a node.
<li><b>Strong</b> {@link PortConstraint}s specify the exact coordinates of the point where an edge should enter/leave a node.
Therefore, for edges with strong port constraints, layout algorithms supporting such constraints use the current
source/target point coordinates as returned by method {@link LayoutGraph#getSourcePointRel} and {@link LayoutGraph#getTargetPointRel}
respectively.
</ul>
Information about {@link PortConstraint}s is held by {@link IDataProvider}s that are registered with the
graph using keys {@link PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY} (to specify
{@link PortConstraint}s for an edge on its source node) and {@link PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}
(to specify {@link PortConstraint}s for an edge on its target node).
Not all layout and routing algorithms are able to handle {@link PortConstraint}s. Major layout algorithms that
obey them are {@link HierarchicLayout}, {@link TreeLayout}, {@link EdgeRouter}, {@link ChannelEdgeRouter}
and {@link BusRouter}.
<h2>Note</h2>
In case that both {@link PortConstraint}s and
{@link PortCandidate}s (on edges and/or nodes) are defined for a graph, the layout or the routing algorithm will
try to match both of them in order to find an appropriate port. The matching process depends on the actual algorithm.
<h2>Note</h2>
For edges incident to group nodes, the {@link HierarchicLayout} only considers weak
{@link PortConstraint}s.
@see PortConstraintKeys
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint Online Documentation}*/
export class PortConstraint {
/**
Creates a new {@link PortConstraint} that constrains the edge to connect to the given side of a node and that can
be either strong or weak.
If the {@link PortConstraint} is strong, port constraint aware layout algorithms keep the current source/target
point location as returned by method {@link LayoutGraph#getSourcePointRel} and {@link LayoutGraph#getTargetPointRel}
respectively.
@param side one of the default side specifiers
@param [strong=false] <code>true</code> if this {@link PortConstraint} is strong, <code>false</code> otherwise
@returns a new {@link PortConstraint} instance
@throws {Exception} <b>(ArgumentError)</b> if an unknown side is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23create(PortSide%2Cboolean) Online Documentation}*/
static create(side: PortSide | PortSideStringValues, strong?: boolean): PortConstraint | null;
/**
Returns whether or not this {@link PortConstraint} is equal to another {@link PortConstraint} based on
whether they are both strong constraints or not and the side attribute.
@param other the reference {@link PortConstraint} object with which to compare
@returns <code>true</code> if this {@link PortConstraint} is equal to the given {@link PortConstraint} argument, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Returns the source {@link PortConstraint} associated with the given edge.
To retrieve the {@link IDataProvider} that holds information about source {@link PortConstraint}s, key {@link PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}
is used.
@param graph the input graph
@param e the edge
@returns the source {@link PortConstraint} of the given edge or <code>null</code> if there is none
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23getSPC(LayoutGraph%2CEdge) Online Documentation}*/
static getSPC(graph: LayoutGraph | null, e: Edge | null): PortConstraint | null;
/**
Returns the target {@link PortConstraint} associated with the given edge.
To retrieve the {@link IDataProvider} that holds information about target {@link PortConstraint}s, key {@link PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}
is used.
@param graph the input graph
@param e the edge
@returns the target {@link PortConstraint} of the given edge or <code>null</code> if there is none
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23getTPC(LayoutGraph%2CEdge) Online Documentation}*/
static getTPC(graph: LayoutGraph | null, e: Edge | null): PortConstraint | null;
/**
Returns a hash code value for this {@link PortConstraint} instance based on whether it is a strong constraint or
not and the side attribute.
@returns a hash code value
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Checks whether or not there exist source or target {@link PortConstraint}s associated with the given edge.
To retrieve the {@link IDataProvider}s that hold information about source or target
{@link PortConstraint}s, keys {@link PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY} and {@link PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}
are used.
<h2>Note</h2>
A weak {@link PortConstraint} that allows {@link PortSide#ANY any side} is not considered to be a real constraint and ignored during
this check.
@param graph the input graph
@param e the edge
@returns <code>true</code> if the edge has source or target {@link PortConstraint}s, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23hasPC(LayoutGraph%2CEdge) Online Documentation}*/
static hasPC(graph: LayoutGraph | null, e: Edge | null): boolean;
/**
Checks whether or not there exist source {@link PortConstraint}s associated with the given edge.
To retrieve the {@link IDataProvider} that holds information about source {@link PortConstraint}s, key {@link PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}
is used.
<h2>Note</h2>
A weak {@link PortConstraint} that allows {@link PortSide#ANY any side} is not considered to be a real constraint and ignored during
this check.
@param graph the input graph
@param e the edge
@returns <code>true</code> if the edge has source {@link PortConstraint}s, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23hasSPC(LayoutGraph%2CEdge) Online Documentation}*/
static hasSPC(graph: LayoutGraph | null, e: Edge | null): boolean;
/**
Checks whether or not there exist target {@link PortConstraint}s associated with the given edge.
To retrieve the {@link IDataProvider} that holds information about target {@link PortConstraint}s, key {@link PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}
is used.
<h2>Note</h2>
A weak {@link PortConstraint} that allows {@link PortSide#ANY any side} is not considered to be a real constraint and ignored during
this check.
@param graph the input graph
@param e the edge
@returns <code>true</code> if the edge has target {@link PortConstraint}s, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23hasTPC(LayoutGraph%2CEdge) Online Documentation}*/
static hasTPC(graph: LayoutGraph | null, e: Edge | null): boolean;
/**
Returns a {@link string} representation of this {@link PortConstraint} instance.
@returns a {@link string} representation of the {@link PortConstraint}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23toString() Online Documentation}*/
toString(): string;
/**
Gets whether or not this {@link PortConstraint} instance represents a strong constraint.
A strong {@link PortConstraint} determines the exact coordinates where the edge path's end should be located.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23strong Online Documentation}*/
readonly strong: boolean;
/**
Gets the side of the node to which an edge should connect.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23side Online Documentation}*/
readonly side: PortSide;
/**
Gets whether or not this {@link PortConstraint} instance is associated with side {@link PortSide#NORTH}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23atNorth Online Documentation}*/
readonly atNorth: boolean;
/**
Gets whether or not this {@link PortConstraint} instance is associated with side {@link PortSide#SOUTH}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23atSouth Online Documentation}*/
readonly atSouth: boolean;
/**
Gets whether or not this {@link PortConstraint} instance is associated with side {@link PortSide#EAST}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23atEast Online Documentation}*/
readonly atEast: boolean;
/**
Gets whether or not this {@link PortConstraint} instance is associated with side {@link PortSide#WEST}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23atWest Online Documentation}*/
readonly atWest: boolean;
/**
Gets whether or not this {@link PortConstraint} instance is associated with side {@link PortSide#ANY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraint%23atAnySide Online Documentation}*/
readonly atAnySide: boolean;
static $class: Class;
private $_PortConstraint: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortConstraint;
}
export interface PortConstraintConfigurator extends YObject {
}
/**
The {@link PortConstraintConfigurator} provides the possibility to create
{@link PortConstraint}s according to the initial connecting side of an edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintConfigurator Online Documentation}*/
export class PortConstraintConfigurator {
/**
Creates a new {@link PortConstraintConfigurator} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintConfigurator Online Documentation}*/
constructor();
/**
Creates a {@link PortConstraint} for the given edge.
The side of the constraint is set up according to the current drawing of the edge within the given graph.
@param graph the input graph
@param edge the edge
@param source <code>true</code> if a source {@link PortConstraint} for the given edge will be created, <code>false</code> for a target
{@link PortConstraint}
@param strong <code>true</code> if a strong {@link PortConstraint} is created, <code>false</code> for a weak {@link PortConstraint}
@returns the {@link PortConstraint} for the given edge whose side is retrieved from the initial drawing
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintConfigurator%23createPortConstraintFromSketch(LayoutGraph%2CEdge%2Cboolean%2Cboolean) Online Documentation}*/
createPortConstraintFromSketch(graph: LayoutGraph | null, edge: Edge | null, source: boolean, strong: boolean): PortConstraint | null;
/**
Creates {@link PortConstraint}s for some edges of the given graph.
@param graph the input graph
@param ec the {@link IEdgeCursor} that provides access to the edges for which to create {@link PortConstraint}s
@param spcMap the {@link IEdgeMap} that will be used for storing the source {@link PortConstraint}s of the edges
@param strongSP <code>true</code> if strong source {@link PortConstraint}s should be assigned, <code>false</code> otherwise
@param tpcMap the {@link IEdgeMap} that will be used for storing the target {@link PortConstraint}s of the edges
@param strongTP <code>true</code> if strong target {@link PortConstraint}s should be assigned, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintConfigurator%23createPortConstraintsFromSketch(LayoutGraph%2CIEdgeCursor%2CIEdgeMap%2Cboolean%2CIEdgeMap%2Cboolean) Online Documentation}*/
createPortConstraintsFromSketch(graph: LayoutGraph | null, ec: IEdgeCursor | null, spcMap: IEdgeMap | null, strongSP: boolean, tpcMap: IEdgeMap | null, strongTP: boolean): void;
/**
Creates weak {@link PortConstraint}s for all edges of the given graph.
@param graph the input graph
@param spcMap the {@link IEdgeMap} that will be used for storing the source {@link PortConstraint}s of the edges
@param tpcMap the {@link IEdgeMap} that will be used for storing the target {@link PortConstraint}s of the edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintConfigurator%23createPortConstraintsFromSketch(LayoutGraph%2CIEdgeMap%2CIEdgeMap) Online Documentation}*/
createPortConstraintsFromSketch(graph: LayoutGraph | null, spcMap: IEdgeMap | null, tpcMap: IEdgeMap | null): void;
static $class: Class;
private $_PortConstraintConfigurator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortConstraintConfigurator;
}
export interface PortConstraintKeys extends YObject {
}
/**
{@link PortConstraintKeys} are used for associating {@link IDataProvider DataProviders} that contain constraints for the edges of the
input graph.
These constraints can either be {@link PortConstraint}s or edge groups.
{@link PortConstraint}s force the edges to start/end on specified sides of the node. They may also restrict ports
of edges to certain coordinates. Edge groups will bundle edges at a common source or target. They will start at the same
coordinates and may share parts of their edge routes.
Layout algorithms that support {@link PortConstraint}s or edge groups will access the
{@link IDataProvider}s using the provided keys.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintKeys Online Documentation}*/
export class PortConstraintKeys {
/**
Data provider key for providing source constraints for edges in the input graph
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintKeys%23SOURCE_PORT_CONSTRAINT_DP_KEY Online Documentation}*/
static SOURCE_PORT_CONSTRAINT_DP_KEY: EdgeDpKey<PortConstraint> | null;
/**
Data provider key for providing target constraints for edges in the input graph
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintKeys%23TARGET_PORT_CONSTRAINT_DP_KEY Online Documentation}*/
static TARGET_PORT_CONSTRAINT_DP_KEY: EdgeDpKey<PortConstraint> | null;
/**
Data provider key for assigning source group information for edges of the input graph
Edges sharing a source group identifier will share a common bus near the source or at a common source node if possible.
<h2>Note</h2>
The {@link HierarchicLayout} and the {@link EdgeRouter} are able to group incoming and outgoing edges.
Therefore, the source group identifier of the outgoing edges of a node has to match with the target group identifier of
the incoming edges.
<h2>Note</h2>
The endpoint of an edge can either be associated with a group or a port group id, but not both at the same time.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintKeys%23SOURCE_GROUP_ID_DP_KEY Online Documentation}*/
static SOURCE_GROUP_ID_DP_KEY: EdgeDpKey<any> | null;
/**
Data provider key for assigning target group information for edges of the input graph
Edges sharing a target group identifier will share a common bus near the target or at a common target node if possible.
<h2>Note</h2>
The {@link HierarchicLayout} and the {@link EdgeRouter} are able to group incoming and outgoing edges.
Therefore, the target group identifier of the incoming edges of a node has to match with the source group identifier of
the outgoing edges.
<h2>Note</h2>
The endpoint of an edge can either be associated with a group or a port group id, but not both at the same time.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintKeys%23TARGET_GROUP_ID_DP_KEY Online Documentation}*/
static TARGET_GROUP_ID_DP_KEY: EdgeDpKey<any> | null;
/**
Data provider key for assigning port group ids to the edges' source
All edges with the same port id at a node will share the same port location. However, they will be routed
independently.
<h2>Note</h2>
Currently, only the {@link HierarchicLayout} and the {@link EdgeRouter} support port grouping.
<h2>Note</h2>
The endpoint of an edge can either be associated with a group or a port group id, but not both at the same time.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintKeys%23SOURCE_PORT_GROUP_ID_DP_KEY Online Documentation}*/
static SOURCE_PORT_GROUP_ID_DP_KEY: EdgeDpKey<any> | null;
/**
Data provider key for assigning port group ids to the edges' target
All edges with the same port id at a node will share the same port location. However, they will be routed
independently.
<h2>Note</h2>
Currently, only the {@link HierarchicLayout} and the {@link EdgeRouter} support port grouping.
<h2>Note</h2>
The endpoint of an edge can either be associated with a group or a port group id, but not both at the same time.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintKeys%23TARGET_PORT_GROUP_ID_DP_KEY Online Documentation}*/
static TARGET_PORT_GROUP_ID_DP_KEY: EdgeDpKey<any> | null;
static $class: Class;
private $_PortConstraintKeys: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortConstraintKeys;
}
export interface PreferredPlacementDescriptor extends YObject {
}
/**
A {@link PreferredPlacementDescriptor} provides placement information for automatic edge labeling.
It provides several options that influence the label placement in generic labeling algorithms and layout algorithms that
support integrated label placement.
The preferred placement can describe multiple locations. For some options, like the {@link PreferredPlacementDescriptor#placeAlongEdge placement along the edge}, it is
possible to have a combination of the valid values. These values can be connected with a logical <code>or</code>-operation.
The following options describe the preferred placement:
<ul>
<li>the placement along the edge
<li>the side of the edge
<li>the reference of the side
<li>the rotation angle of the label
<li>the reference for the rotation angle
<li>the rotation behavior for label on the right side of the edge
<li>the angle offset for label on the right side of the edge
<li>the distance between the label and its edge segment
</ul>
The {@link PreferredPlacementDescriptor} will be immutable after {@link PreferredPlacementDescriptor#freeze}
is called.
A {@link PreferredPlacementDescriptor} must be set for each label and will be retrieved by the layout algorithms
calling {@link IEdgeLabelLayout#preferredPlacementDescriptor}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor Online Documentation}*/
export class PreferredPlacementDescriptor {
/**
Creates a new {@link PreferredPlacementDescriptor} instance that copies the given descriptor.
@param descriptor the {@link PreferredPlacementDescriptor} that is copied
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23PreferredPlacementDescriptor(PreferredPlacementDescriptor) Online Documentation}*/
constructor(descriptor: PreferredPlacementDescriptor | null);
/**
Creates a new {@link PreferredPlacementDescriptor} instance with default settings.
@param [options] The parameters to pass.
@param [options.placeAlongEdge] The preferred placement along the edge. This option sets the {@link PreferredPlacementDescriptor#placeAlongEdge} property on the created object.
@param [options.sideOfEdge] The preferred side of the edge. This option sets the {@link PreferredPlacementDescriptor#sideOfEdge} property on the created object.
@param [options.sideReference] How to interpret the preferred side as given by {@link PreferredPlacementDescriptor#sideOfEdge}. This option sets the {@link PreferredPlacementDescriptor#sideReference} property on the created object.
@param [options.angle] The angle (in radians) for the label rotation. This option sets the {@link PreferredPlacementDescriptor#angle} property on the created object.
@param [options.angleReference] The reference of the angle given by {@link PreferredPlacementDescriptor#angle}. This option sets the {@link PreferredPlacementDescriptor#angleReference} property on the created object.
@param [options.angleRotationOnRightSide] How the angle is applied to labels on the right side in respect of the labels of the left side of the edge. This option sets the {@link PreferredPlacementDescriptor#angleRotationOnRightSide} property on the created object.
@param [options.angleOffsetOnRightSide] The angle offset for labels that are placed on the right side of the edge. This option sets the {@link PreferredPlacementDescriptor#angleOffsetOnRightSide} property on the created object.
@param [options.distanceToEdge] The preferred distance between a label and the corresponding edge segment. This option sets the {@link PreferredPlacementDescriptor#distanceToEdge} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23PreferredPlacementDescriptor() Online Documentation}*/
constructor(options?: {
placeAlongEdge?: LabelPlacements | LabelPlacementsConvertible;
sideOfEdge?: LabelPlacements | LabelPlacementsConvertible;
sideReference?: LabelSideReferences | LabelSideReferencesConvertible;
angle?: number;
angleReference?: LabelAngleReferences | LabelAngleReferencesConvertible;
angleRotationOnRightSide?: LabelAngleOnRightSideRotations | LabelAngleOnRightSideRotationsConvertible;
angleOffsetOnRightSide?: LabelAngleOnRightSideOffsets | LabelAngleOnRightSideOffsetsConvertible;
distanceToEdge?: number;
});
/**
Clones this instance by returning a {@link YObject#memberwiseClone} or this in case this instance is already {@link PreferredPlacementDescriptor#isFrozen frozen}.
@returns An instance of the same type as this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23clone() Online Documentation}*/
clone(): any | null;
/**
Determines whether another object is equal to this one.
@param o the other object to compare for equality
@returns <code>true</code> if <code>o</code> is considered equal to this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23equals(Object) Online Documentation}*/
equals(o: any | null): boolean;
/**
Makes this {@link PreferredPlacementDescriptor} instance immutable.
If this instance is immutable, all <code>setter</code> methods will throw an {@link Exception InvalidOperationError} when invoked.
@see PreferredPlacementDescriptor#isFrozen
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23freeze() Online Documentation}*/
freeze(): void;
/**
Returns a hash code for this object.
The hash code is a numeric value that can be used to treat this object as a key in a hash table or similar data
structure.
Two objects that are considered equal must have the same hash code. However, the reverse does not hold and two objects
having the same hash code don't have to be equal. Ideally, the hash code should be roughly uniformly-distributed to
prevent hash tables from performing poorly. Calculating the hash code is also a potentially frequent operation and
should therefore be fast.
@returns the hash code for this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Returns whether or not this is an immutable instance of {@link PreferredPlacementDescriptor}.
If this instance is immutable, all <code>setter</code> methods will throw an {@link Exception InvalidOperationError} when invoked.
@returns <code>true</code> if this instance is immutable, <code>false</code> otherwise
@see PreferredPlacementDescriptor#freeze
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isFrozen() Online Documentation}*/
isFrozen(): boolean;
/**
Returns an immutable descriptor instance for the specified placement.
Descriptor instances returned by this method will use default values for all properties but {@link PreferredPlacementDescriptor#placeAlongEdge placement along edge} and
{@link PreferredPlacementDescriptor#sideOfEdge side of edge}.
@param placement a combination of valid placements along the edge and valid sides of the edge
@returns an immutable descriptor instance for the specified placement
@see PreferredPlacementDescriptor#isFrozen
@see PreferredPlacementDescriptor#placeAlongEdge
@see PreferredPlacementDescriptor#sideOfEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23newSharedInstance(LabelPlacements) Online Documentation}*/
static newSharedInstance(placement: LabelPlacements | LabelPlacementsConvertible): PreferredPlacementDescriptor | null;
/**
Returns a string representation for this object.
@returns a string describing this object
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23toString() Online Documentation}*/
toString(): string;
/**
Gets or sets the preferred placement along the edge.
<b>Warning:</b> Placements {@link LabelPlacements#AT_SOURCE_PORT} and {@link LabelPlacements#AT_TARGET_PORT} that indicate
a placement directly at the edge port are currently only supported by {@link HierarchicLayout}.
@throws {Exception} <b>(InvalidOperationError)</b> if this instance is {@link PreferredPlacementDescriptor#isFrozen immutable}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23placeAlongEdge Online Documentation}*/
placeAlongEdge: LabelPlacements;
/**
Gets or sets the preferred side of the edge.
@throws {Exception} <b>(InvalidOperationError)</b> if this instance is {@link PreferredPlacementDescriptor#isFrozen immutable}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23sideOfEdge Online Documentation}*/
sideOfEdge: LabelPlacements;
/**
Gets or sets how to interpret the preferred side as given by {@link PreferredPlacementDescriptor#sideOfEdge}.
@throws {Exception} <b>(InvalidOperationError)</b> if this instance is {@link PreferredPlacementDescriptor#isFrozen immutable}
@throws {Exception} <b>(ArgumentError)</b> if the specified side reference is invalid
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23sideReference Online Documentation}*/
sideReference: LabelSideReferences;
/**
Gets or sets the angle (in radians) for the label rotation.
How this angle is applied depends on the {@link PreferredPlacementDescriptor#angleReference angle reference}, the {@link PreferredPlacementDescriptor#angleRotationOnRightSide rotation} and the {@link PreferredPlacementDescriptor#angleOffsetOnRightSide offset} for labels on the right side of
the edge.
@throws {Exception} <b>(InvalidOperationError)</b> if this instance is {@link PreferredPlacementDescriptor#isFrozen immutable}
@see PreferredPlacementDescriptor#angleReference
@see PreferredPlacementDescriptor#angleRotationOnRightSide
@see PreferredPlacementDescriptor#angleOffsetOnRightSide
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23angle Online Documentation}*/
angle: number;
/**
Gets or sets the reference of the angle given by {@link PreferredPlacementDescriptor#angle}.
@throws {Exception} <b>(InvalidOperationError)</b> if this instance is {@link PreferredPlacementDescriptor#isFrozen immutable}
@throws {Exception} <b>(ArgumentError)</b> if the specified value is not a valid angle reference
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23angleReference Online Documentation}*/
angleReference: LabelAngleReferences;
/**
Gets or sets how the angle is applied to labels on the right side in respect of the labels of the left side of the
edge.
@throws {Exception} <b>(InvalidOperationError)</b> if this instance is {@link PreferredPlacementDescriptor#isFrozen immutable}
@throws {Exception} <b>(ArgumentError)</b> if the specified value is not a valid angle rotation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23angleRotationOnRightSide Online Documentation}*/
angleRotationOnRightSide: LabelAngleOnRightSideRotations;
/**
Gets or sets the angle offset for labels that are placed on the right side of the edge.
@throws {Exception} <b>(InvalidOperationError)</b> if this instance is {@link PreferredPlacementDescriptor#isFrozen immutable}
@throws {Exception} <b>(ArgumentError)</b> if the specified value is not valid angle offset
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23angleOffsetOnRightSide Online Documentation}*/
angleOffsetOnRightSide: LabelAngleOnRightSideOffsets;
/**
Gets or sets the preferred distance between a label and the corresponding edge segment.
If the given distance is <code>&lt; 0</code>, the distance is not fixed, i.e., it is chosen by the automatic labeling algorithm.
@throws {Exception} <b>(InvalidOperationError)</b> if this instance is {@link PreferredPlacementDescriptor#isFrozen immutable}
@throws {Exception} <b>(ArgumentError)</b> if the specified value is infinite or <code>NaN</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23distanceToEdge Online Documentation}*/
distanceToEdge: number;
/**
Gets whether or not one of the preferred placement locations for this label is at source.
@see LabelPlacements#AT_SOURCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAtSource Online Documentation}*/
readonly isAtSource: boolean;
/**
Gets whether or not one of the preferred placement locations for this label is at center.
@see LabelPlacements#AT_CENTER
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAtCenter Online Documentation}*/
readonly isAtCenter: boolean;
/**
Gets whether or not one of the preferred placements for this label is at target.
@see LabelPlacements#AT_TARGET
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAtTarget Online Documentation}*/
readonly isAtTarget: boolean;
/**
Gets whether or not one of the preferred placements for this label is at the source port.
@see LabelPlacements#AT_SOURCE_PORT
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAtSourcePort Online Documentation}*/
readonly isAtSourcePort: boolean;
/**
Gets whether or not one of the preferred placements for this label is at the target port.
@see LabelPlacements#AT_TARGET_PORT
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAtTargetPort Online Documentation}*/
readonly isAtTargetPort: boolean;
/**
Gets whether or not one of the preferred sides for this label is left of the edge.
@see LabelPlacements#LEFT_OF_EDGE
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isLeftOfEdge Online Documentation}*/
readonly isLeftOfEdge: boolean;
/**
Gets whether or not one of the preferred sides for this label is on the edge.
@see LabelPlacements#ON_EDGE
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isOnEdge Online Documentation}*/
readonly isOnEdge: boolean;
/**
Gets whether or not one of the preferred sides for this label is right of the edge.
@see LabelPlacements#RIGHT_OF_EDGE
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isRightOfEdge Online Documentation}*/
readonly isRightOfEdge: boolean;
/**
Gets whether or not one of the angle interpretations is absolute.
@see LabelAngleReferences#ABSOLUTE
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAngleAbsolute Online Documentation}*/
readonly isAngleAbsolute: boolean;
/**
Gets whether or not one of the angle interpretation is relative to the edge slope.
@see LabelAngleReferences#RELATIVE_TO_EDGE_FLOW
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAngleRelativeToEdgeFlow Online Documentation}*/
readonly isAngleRelativeToEdgeFlow: boolean;
/**
Gets whether or not one of the angle interpretations is to rotate the angle of labels that are right of the edge
co-rotating with the labels left of or centered on the edge, i.e., the angle of all labels is interpreted clockwise.
@see PreferredPlacementDescriptor#isRightOfEdge
@see LabelAngleOnRightSideRotations#CLOCKWISE
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAngleOnRightSideCoRotating Online Documentation}*/
readonly isAngleOnRightSideCoRotating: boolean;
/**
Gets whether or not one of the angle interpretations is to rotate the angle of labels that are right of the edge
counter-rotating with the labels left of or centered on the edge, i.e., the angle of labels left or centered on edge is
interpreted clockwise while the angle of labels right of edge is interpreted counter-clockwise.
@see PreferredPlacementDescriptor#isRightOfEdge
@see LabelAngleOnRightSideRotations#COUNTER_CLOCKWISE
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAngleOnRightSideCounterRotating Online Documentation}*/
readonly isAngleOnRightSideCounterRotating: boolean;
/**
Gets whether or not one of the angle interpretations is to add no angle offset to labels that are right of the edge.
@see PreferredPlacementDescriptor#isRightOfEdge
@see LabelAngleOnRightSideOffsets#NONE
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAngleOffsetOnRightSideZero Online Documentation}*/
readonly isAngleOffsetOnRightSideZero: boolean;
/**
Gets whether or not one of the angle interpretations is to add an angle offset of <code>180</code> degrees to labels that are right
of the edge.
@see PreferredPlacementDescriptor#isRightOfEdge
@see LabelAngleOnRightSideOffsets#SEMI
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isAngleOffsetOnRightSide180 Online Documentation}*/
readonly isAngleOffsetOnRightSide180: boolean;
/**
Gets whether or not the preferred side specified by {@link PreferredPlacementDescriptor#sideOfEdge} is interpreted relative to the edge flow.
@see LabelSideReferences#RELATIVE_TO_EDGE_FLOW
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isSideRelativeToEdgeFlow Online Documentation}*/
readonly isSideRelativeToEdgeFlow: boolean;
/**
Gets whether or not the preferred side specified by {@link PreferredPlacementDescriptor#sideOfEdge} is interpreted absolute, and on horizontal segments, labels with
preferred side {@link LabelPlacements#LEFT_OF_EDGE} are placed above the segment.
@see LabelSideReferences#ABSOLUTE_WITH_LEFT_IN_NORTH
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isSideAbsoluteWithLeftInNorth Online Documentation}*/
readonly isSideAbsoluteWithLeftInNorth: boolean;
/**
Gets whether or not the preferred side specified by {@link PreferredPlacementDescriptor#sideOfEdge} is interpreted absolute, and on horizontal segments, labels with
preferred side {@link LabelPlacements#RIGHT_OF_EDGE} are placed above the segment.
@see LabelSideReferences#ABSOLUTE_WITH_RIGHT_IN_NORTH
@see {@link https://docs.yworks.com/yfileshtml/#/api/PreferredPlacementDescriptor%23isSideAbsoluteWithRightInNorth Online Documentation}*/
readonly isSideAbsoluteWithRightInNorth: boolean;
static $class: Class;
private $_PreferredPlacementDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PreferredPlacementDescriptor;
}
export interface RemoveCollinearBendsStage extends LayoutStageBase {
}
/**
The {@link RemoveCollinearBendsStage} removes all collinear bends found in a graph.
The algorithm traverses each edge path from source to target and removes for each triple of consecutive collinear bends
the middle one.
A collinear bend forms a straight line with its preceding bend and its subsequent bend. The angle between the connecting
edge segments is either <code>0</code> or <code>180</code> degrees.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveCollinearBendsStage Online Documentation}*/
export class RemoveCollinearBendsStage {
/**
Creates a new {@link RemoveCollinearBendsStage} instance with default settings.
@param [options] The parameters to pass.
@param [options.scale] The scale that defines how accurate the algorithm is when it comes to comparing the coordinates of two bends. This option sets the {@link RemoveCollinearBendsStage#scale} property on the created object.
@param [options.removeStraightOnly] Whether the algorithm removes only collinear bends which lie in the middle of their neighbors or all collinear bends. This option sets the {@link RemoveCollinearBendsStage#removeStraightOnly} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveCollinearBendsStage Online Documentation}*/
constructor(options?: {
scale?: number;
removeStraightOnly?: boolean;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Removes collinear bends of the edges in the given graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveCollinearBendsStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the scale that defines how accurate the algorithm is when it comes to comparing the coordinates of two
bends.
This value determines how the values of the coordinates are rounded. If rounded coordinates lead to a collinear bend,
this bend will be removed. The larger the value, the higher the precision is.
<ul>
<li>Positive scale values: the value denotes the number of decimal positions that are considered (e.g. <code>10.2345</code> becomes <code>10.23</code>
for a scale factor of <code>2</code>).
<li>Scale value zero: coordinates are rounded to the next integer coordinates (e.g. <code>10.2345</code> becomes <code>10</code>).
<li>Negative scale values: the coordinates are rounded to the next multiple of <code>10</code>, <code>100</code> and so on (e.g. <code>10.2345</code> becomes <code>0</code> for
a scale factor of <code>-2</code>).
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveCollinearBendsStage%23scale Online Documentation}*/
scale: number;
/**
Gets or sets whether the algorithm removes only collinear bends which lie in the middle of their neighbors or all
collinear bends.
In the first case, the angle at the middle bend must be <code>180</code> degrees, while in the second case, the angle can be either <code>180</code>
degrees or <code>0</code> degrees.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveCollinearBendsStage%23removeStraightOnly Online Documentation}*/
removeStraightOnly: boolean;
static $class: Class;
private $_RemoveCollinearBendsStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RemoveCollinearBendsStage;
}
export interface ReverseEdgesStage extends LayoutStageBase {
}
/**
{@link ReverseEdgesStage} selectively reverses edges in a graph, while keeping the layout and label positions of
the reversed edges as close to the pre-reversal layout and positions as possible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStage Online Documentation}*/
export class ReverseEdgesStage {
/**
Creates a new {@link ReverseEdgesStage} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayouter=null] The core layout algorithm.
@see LayoutStageBase#coreLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStage Online Documentation}*/
constructor(coreLayouter?: ILayoutAlgorithm | null);
/**
Reverses all <em>selected</em> edges before calling the {@link LayoutStageBase#coreLayout core layout algorithm} and restores the original edge direction after the core
layout algorithm has finished.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Marks edges in a tree-structured graph component that need to be reversed to make the specified node the root of the
tree component.
Only edges in the graph component to which the specified node belongs will be marked.
@param graph the graph to which the specified node belongs
@param root the node to be considered the root of its tree component
@param reversedEdges a {@link IDataAcceptor} to mark edges for reversal
@throws {Exception} <b>(ArgumentError)</b> if the specified root node is <code>null</code> or does not belong to the specified graph
@see ReverseEdgesStage#AFFECTED_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStage%23findReversedTreeEdges(LayoutGraph%2CYNode%2CIDataAcceptor) Online Documentation}*/
static findReversedTreeEdges(graph: LayoutGraph | null, root: YNode | null, reversedEdges: IDataAcceptor | null): void;
/**
Reverses the specified edge in the specified graph.
The layout of the specified edge is reversed as specified in
{@link LayoutGraphUtilities#reverseEdgeLayout}. Moreover, the positions of all labels associated with the
specified edge are adjusted to match the pre-reversal position as closely as possible. (Depending on a label's model it
might not always be possible to achieve an exact match.)
@param graph the graph to which the specified edge belongs
@param edge the edge to be reversed
@see LayoutGraphUtilities#reverseEdgeLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStage%23reverseEdge(LayoutGraph%2CEdge) Online Documentation}*/
reverseEdge(graph: LayoutGraph | null, edge: Edge | null): void;
/**
Reverses <em>selected</em> edges in the specified graph.
<em>Selected</em> edges are marked in a {@link IDataProvider} registered with
{@link ReverseEdgesStage#AFFECTED_EDGES_DP_KEY}.
Implementation detail: This method calls {@link ReverseEdgesStage#reverseEdge} in order to reverse a <em>selected</em> edge.
@param graph the graph in which to reverse edges.
@see ReverseEdgesStage#AFFECTED_EDGES_DP_KEY
@see ReverseEdgesStage#reverseEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStage%23reverseEdges(LayoutGraph) Online Documentation}*/
reverseEdges(graph: LayoutGraph | null): void;
/**
Data provider key for selecting edges for reversal
<h2>Note</h2>
If there is no {@link IDataProvider} registered with this key, all edges are reversed.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStage%23AFFECTED_EDGES_DP_KEY Online Documentation}*/
static AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
static $class: Class;
private $_ReverseEdgesStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReverseEdgesStage;
}
/**
@see BusRouter#scope
@see CurveRoutingStage#scope
@see EdgeRouter#scope
@see PolylineLayoutStage#scope
@see StraightLineEdgeRouter#scope
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterScope Online Documentation}*/
export enum EdgeRouterScope {
/**
A scope specifier which defines that all edges of the input graph will be routed.
@see BusRouter#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterScope%23ROUTE_ALL_EDGES Online Documentation}*/
ROUTE_ALL_EDGES,
/**
A scope specifier which defines that only the selected edges of the input graph will be routed.
The selection state of an edge is determined by a boolean value returned by a {@link IDataProvider} registered
with key {@link BusRouter#affectedEdgesDpKey}.
All other non-selected edges will be considered to have fixed routes.
@see BusRouter#scope
@see BusRouter#affectedEdgesDpKey
@see BusRouter#DEFAULT_AFFECTED_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterScope%23ROUTE_AFFECTED_EDGES Online Documentation}*/
ROUTE_AFFECTED_EDGES,
/**
A scope specifier which defines that only edges incident to selected nodes will be routed.
The selection state of a node is determined by a boolean value returned by a {@link IDataProvider} registered
with key {@link CurveRoutingStage#affectedNodesDpKey}.
All other edges that are incident to non-selected nodes will be considered to have fixed routes.
@see EdgeRouter#scope
@see EdgeRouter#affectedNodesDpKey
@see CurveRoutingStage#scope
@see CurveRoutingStage#affectedNodesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterScope%23ROUTE_EDGES_AT_AFFECTED_NODES Online Documentation}*/
ROUTE_EDGES_AT_AFFECTED_NODES
}
/** Automatic conversion to {@link EdgeRouterScope}. */
type EdgeRouterScopeStringValues = "route-all-edges" | "route-affected-edges" | "route-edges-at-affected-nodes";
/**
Channel orientation specifier for routing edges vertically.
@see ChannelRoutingTool
@see ChannelRoutingTool#setChannel
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelOrientation Online Documentation}*/
export enum ChannelOrientation {
/**
Channel orientation specifier for routing edges vertically.
More precisely, the algorithm calculates the x-coordinates of the vertical middle segments of the added edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelOrientation%23VERTICAL Online Documentation}*/
VERTICAL,
/**
Channel orientation specifier for routing edges horizontally.
More precisely, the algorithm calculates the y-coordinates of the horizontal middle segments of the added edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelOrientation%23HORIZONTAL Online Documentation}*/
HORIZONTAL
}
/** Automatic conversion to {@link ChannelOrientation}. */
type ChannelOrientationStringValues = "vertical" | "horizontal";
/**
@see OrthogonalPatternEdgeRouter#monotonicPathRestriction
@see EdgeRouterEdgeLayoutDescriptor#monotonicPathRestriction
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MonotonicPathRestriction Online Documentation}*/
export enum MonotonicPathRestriction {
/**
A constant specifying that there are no monotonic edge path restrictions.
The direction of the edge segments is not restricted, hence, intermediate segments might also point in a direction other
than the one towards the target.
@see EdgeRouterEdgeLayoutDescriptor#monotonicPathRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/MonotonicPathRestriction%23NONE Online Documentation}*/
NONE,
/**
A constant specifying monotonic edge path restrictions for the vertical direction.
This implies that each vertical edge segment is directed from the source to the target. Note that if the source and
target node are located on the same y-coordinate, then this restriction can not have an effect.
@see EdgeRouterEdgeLayoutDescriptor#monotonicPathRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/MonotonicPathRestriction%23VERTICAL Online Documentation}*/
VERTICAL,
/**
A constant specifying monotonic edge path restrictions for the horizontal direction.
This implies that each horizontal edge segment is directed from the source to the target. Note that if the source and
target node are located on the same x-coordinate, then this restriction can not have an effect.
@see EdgeRouterEdgeLayoutDescriptor#monotonicPathRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/MonotonicPathRestriction%23HORIZONTAL Online Documentation}*/
HORIZONTAL,
/**
A constant specifying monotonic edge path restrictions for the horizontal and vertical direction.
This implies that each horizontal as well as each vertical edge segment is directed from the source to the target.
@see EdgeRouterEdgeLayoutDescriptor#monotonicPathRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/MonotonicPathRestriction%23BOTH Online Documentation}*/
BOTH
}
/** Automatic conversion to {@link MonotonicPathRestriction}. */
type MonotonicPathRestrictionStringValues = "none" | "vertical" | "horizontal" | "both";
export interface BusRouterBusDescriptor extends YObject {
}
/**
A descriptor for bus-related information to be associated with an edge.
It consists of a bus ID defining the bus of the associated edge, two optional edge group IDs for specifying the edge
grouping at the source and target side, respectively, and an optional boolean flag to mark the edge as fixed.
Every object except <code>null</code> can be used as a valid bus ID. Every edge of ID <code>null</code> forms a group of its own. Also, every
object except <code>null</code> can be used as a valid edge group ID. If there is no group ID defined for an edge or if <code>null</code> is set,
a default group ID is used. Therefore, all such edges ending at the same node belong to the same group.
<h2>Note</h2>
At least one of the edges of a bus has to connect two different nodes. Otherwise, if all edges are self-loops (edges
that start and end at the same node), the edges are routed separately without constituting a bus structure.
@see BusRouter#EDGE_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor Online Documentation}*/
export class BusRouterBusDescriptor {
/**
Creates a new instance of {@link BusRouterBusDescriptor} for the specified parameters.
@param busID the ID of the bus to which the associated edge belongs
@param fixed <code>true</code> if associated edge is fixed, <code>false</code> otherwise
@param sourceGroupID the group ID of the source side
@param targetGroupID the group ID of the target side
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23BusRouterBusDescriptor(Object%2Cboolean%2CObject%2CObject) Online Documentation}*/
constructor(busID: any | null, fixed: boolean, sourceGroupID: any | null, targetGroupID: any | null);
/**
Creates a new instance of {@link BusRouterBusDescriptor} for the specified parameters and default group IDs.
@param busID the ID of the bus to which the associated edge belongs
@param fixed <code>true</code> if associated edge is fixed, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23BusRouterBusDescriptor(Object%2Cboolean) Online Documentation}*/
constructor(busID: any | null, fixed: boolean);
/**
Creates a new instance of {@link BusRouterBusDescriptor} for the specified parameters and marks it as not fixed.
@param busID the ID of the bus to which the associated edge belongs
@param sourceGroupID the group ID of the source side
@param targetGroupID the group ID of the target side
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23BusRouterBusDescriptor(Object%2CObject%2CObject) Online Documentation}*/
constructor(busID: any | null, sourceGroupID: any | null, targetGroupID: any | null);
/**
Creates a new instance of {@link BusRouterBusDescriptor} for the specified bus ID which is marked as not fixed and uses the
default edge group IDs.
@param busID the ID of the bus to which the associated edge belongs
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23BusRouterBusDescriptor(Object) Online Documentation}*/
constructor(busID: any | null);
/**
Indicates whether or not some other object is "equal to" this one.
@param obj the reference object with which to compare
@returns <code>true</code> if all corresponding parameters are equal, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23equals(Object) Online Documentation}*/
equals(obj: any | null): boolean;
/**
Returns a hash code value for this {@link BusRouterBusDescriptor} instance.
@returns a hash code value
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Returns a string representation consisting of the bus ID, the fixed flag, and both group IDs.
@returns a string representation consisting of the bus ID, the fixed flag, and both group IDs
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23toString() Online Documentation}*/
toString(): string;
/**
Gets or sets the bus ID.
Edges associated with {@link BusRouterBusDescriptor descriptor instances} having the same bus ID belong to a common bus. Every edge associated with ID
<code>null</code> forms a group of its own.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23busId Online Documentation}*/
busId: any | null;
/**
Gets or sets the group ID for the source side of the edge.
Edges incident to the same node <code>v</code> but associated with different group IDs at this endpoint, use separate bus connections
at <code>v</code>.
<h2>Note</h2>
If there is no group ID defined or if <code>null</code> is set, a default group ID is used. Therefore, all associated edges ending at
the same node belong to the same group.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23sourceGroupId Online Documentation}*/
sourceGroupId: any | null;
/**
Gets or sets the group ID for the target side of the edge.
Edges incident to the same node <code>v</code> but associated with different group IDs at this endpoint, use separate bus connections
at <code>v</code>.
<h2>Note</h2>
If there is no group ID defined or if <code>null</code> is set, a default group ID is used. Therefore, all associated edges ending at
the same node belong to the same group.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23targetGroupId Online Documentation}*/
targetGroupId: any | null;
/**
Gets or sets whether or not the associated edge is fixed.
The algorithm doesn't change the route of fixed edges but tries to prevent overlaps with non-fixed edges.
<h2>Note</h2>
The structure induced by the fixed edges must be orthogonal and cycle-free.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterBusDescriptor%23fixed Online Documentation}*/
fixed: boolean;
static $class: Class;
private $_BusDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BusRouterBusDescriptor;
}
export interface ChannelRoutingTool extends YObject {
}
/**
This routing algorithm applies orthogonal routes to the edges within a channel.
Each edge that should be routed has to be specified by a unique key (e.g., the edge itself) as well as two points that
represent the two endpoints of this edge, see {@link ChannelRoutingTool#addSegment}. The orthogonal route of each
edge consists of three segments. More precisely, for an edge with source point <code>(sx,sy)</code>, target point <code>(tx,ty)</code> and {@link ChannelOrientation#VERTICAL vertical}
channel orientation, the algorithm calculates the x-coordinate of the vertical middle segment (called <code>mx</code>) that can be
obtained with method {@link ChannelRoutingTool#getChannelCoordinate} after {@link ChannelRoutingTool#route routing} the edges. Hence, the actual
route consists of points <code>(sx,sy), (mx,sy), (mx,ty)</code> and <code>(tx,ty)</code>. Analogously, if the channel orientation is {@link ChannelOrientation#HORIZONTAL horizontal},
the route consists of the points <code>(sx,sy), (sx,my), (tx,my)</code> and <code>(tx,ty)</code>.
Besides the orientation, each channel is restricted by a smallest and largest assignable coordinate. If, for example,
the orientation is {@link ChannelOrientation#VERTICAL vertical}, the {@link ChannelRoutingTool#getChannelCoordinate calculated x-coordinate} of the middle segment lies between these two values. Both
values can be changed by means of method {@link ChannelRoutingTool#setChannel}.
All edges assigned to the same coordinate are said to lie in the same sub-channel. In general, the routing algorithm
tries to assign coordinates such the number of sub-channels as well as the number of edge crossings is small.
<h2>Note</h2>
This class doesn't change the initial routes of the edges - it only calculates coordinates that can be retrieved with
method {@link ChannelRoutingTool#getChannelCoordinate} or {@link ChannelRoutingTool#getSubchannelRank}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool Online Documentation}*/
export class ChannelRoutingTool {
/**
Creates a new instance of {@link ChannelRoutingTool}.
@param min The smallest assignable coordinate within the channel.
@param max The largest assignable coordinate within the channel.
@param [orientation=ChannelOrientation.HORIZONTAL] The orientation of the channel. If omitted, the default value is {@link ChannelOrientation#HORIZONTAL}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool Online Documentation}*/
constructor(min: number, max: number, orientation?: ChannelOrientation | ChannelOrientationStringValues);
/**
Adds an edge object that belongs to an edge group to the set of elements that should be routed within the channel.
Edges having the same group id will be placed in the same sub-channel (i.e., the same coordinate).
@param key the unique id of the edge
@param groupId the id of the edge group
@param p1 the first endpoint of the edge
@param p2 the second endpoint of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23addGroupSegment(Object%2CObject%2CYPoint%2CYPoint) Online Documentation}*/
addGroupSegment(key: any | null, groupId: any | null, p1: YPoint | null, p2: YPoint | null): void;
/**
Adds an edge object with endpoint <code>p1</code> and <code>p2</code> to the set of elements that should be routed within the channel.
@param key the unique id of the edge
@param p1 the first endpoint of the edge
@param p2 the second endpoint of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23addSegment(Object%2CYPoint%2CYPoint) Online Documentation}*/
addSegment(key: any | null, p1: YPoint | null, p2: YPoint | null): void;
/**
Returns whether the given id has already been registered as a group id.
@param groupId the group id used during {@link ChannelRoutingTool#addGroupSegment adding}
@returns <code>true</code> if the key has already been registered, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23containsGroupKey(Object) Online Documentation}*/
containsGroupKey(groupId: any | null): boolean;
/**
Returns whether the given key has already been registered.
@param key the key used during {@link ChannelRoutingTool#addSegment adding}
@returns <code>true</code> if the key has already been registered, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23containsKey(Object) Online Documentation}*/
containsKey(key: any | null): boolean;
/**
Returns the calculated channel coordinate for an added edge object.
@param key the key used during {@link ChannelRoutingTool#addSegment adding} the edge object
@returns the channel coordinate of the associated edge object
@see ChannelRoutingTool#addSegment
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23getChannelCoordinate(Object) Online Documentation}*/
getChannelCoordinate(key: any | null): number;
/**
Returns the sub-channel rank for the specified edge group within the channel.
@param groupId the group id used during {@link ChannelRoutingTool#addGroupSegment adding} an edge object
@returns the sub-channel rank of the edge group
@see ChannelRoutingTool#subchannelCount
@see ChannelRoutingTool#addGroupSegment
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23getGroupSubchannelRank(Object) Online Documentation}*/
getGroupSubchannelRank(groupId: any | null): number;
/**
Returns the sub-channel rank of the specified edge object within the channel.
<h2>Note</h2>
The {@link ChannelRoutingTool#getChannelCoordinate coordinate} of an edge object increases with increasing sub-channel rank. Edge objects assigned to the same
sub-channel rank are placed on the same coordinate.
@param key the key used during {@link ChannelRoutingTool#addSegment adding} the edge object
@returns the sub-channel rank of the given edge object
@see ChannelRoutingTool#subchannelCount
@see ChannelRoutingTool#addSegment
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23getSubchannelRank(Object) Online Documentation}*/
getSubchannelRank(key: any | null): number;
/**
Routes all added edge objects within the specified channel.
The resulting sub-channels can then be queried using method {@link ChannelRoutingTool#getSubchannelRank}, while
the exact coordinates can be queried with method {@link ChannelRoutingTool#getChannelCoordinate}.
@see ChannelRoutingTool#addSegment
@see ChannelRoutingTool#addGroupSegment
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23route() Online Documentation}*/
route(): void;
/**
Specifies the channel bounds and orientation.
Calling this method removes all previously added elements.
@param min The smallest assignable coordinate within the channel.
@param max The largest assignable coordinate within the channel.
@param [orientation=ChannelOrientation.HORIZONTAL] The orientation of the channel. If omitted, the default value is {@link ChannelOrientation#HORIZONTAL}.
@see ChannelRoutingTool#addSegment
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23setChannel(number%2Cnumber%2CChannelOrientation) Online Documentation}*/
setChannel(min: number, max: number, orientation?: ChannelOrientation | ChannelOrientationStringValues): void;
/**
Gets the number of sub-channels (i.e., different coordinates assigned to the edge objects) within this channel.
A return value of <code>0</code> indicates that there are no sub-channels present.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23subchannelCount Online Documentation}*/
readonly subchannelCount: number;
/**
Gets or sets whether or not middle segments with lengths smaller than {@link ChannelRoutingTool#epsilon epsilon} are ignored.
@see ChannelRoutingTool#epsilon
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23ignoreEpsilonChannel Online Documentation}*/
ignoreEpsilonChannel: boolean;
/**
Gets or sets the current epsilon value.
If option {@link ChannelRoutingTool#ignoreEpsilonChannel} is enabled, middle segments with length smaller than this value are ignored.
@see ChannelRoutingTool#ignoreEpsilonChannel
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelRoutingTool%23epsilon Online Documentation}*/
epsilon: number;
static $class: Class;
private $_ChannelRoutingTool: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ChannelRoutingTool;
}
export interface SelfLoopRouter extends LayoutStageBase {
}
/**
A {@link SelfLoopRouter} routes the self-loops (reflexive edges) of a graph.
<b>Layout Style</b>
The nodes and normal edges are arranged by the
{@link LayoutStageBase#coreLayout core layout algorithm}. This layout algorithm only handles the paths of self-loops.
Self-loops are routed either {@link SelfLoopStyle#ORTHOGONAL orthogonal} or with {@link SelfLoopStyle#ROUNDED rounded} corners. The layout algorithm places the self-loops in the least
crowded quadrant around a node.
<b>Concept</b>
Self-loops are handled in four steps:
<ol>
<li>Remove all self-loops of the input graph
<li>Invoke the core layout algorithm on the now self-loops free graph
<li>Reinsert all formerly removed self-loops
<li>Route the self-loops of the input graph
</ol>
<b>Features</b>
{@link SelfLoopRouter} can either be used as a {@link ILayoutStage} wrapping a {@link ILayoutAlgorithm layout algorithm} which cannot
handle self-loops. Then it will hide the self-loops from this {@link LayoutStageBase#coreLayout core layout algorithm} and take over the routing of them.
If no {@link LayoutStageBase#coreLayout core layout algorithm} is specified, {@link SelfLoopRouter} can work alone. It will route only the self-loops
and keep the remaining graph unchanged.
When all self-loops already have a suitable layout, this layout algorithm can
{@link SelfLoopRouter#keepAllSelfLoopRoutes keep the self-loop paths}. It will just hide the self-loops before invoking the core layout algorithm, such that they are
not altered by it.
To only exclude some self-loops from routing, a {@link IDataProvider} registered with {@link SelfLoopRouter#KEEP_SELF_LOOP_LAYOUT_DP_KEY}
can be used for marking those self-loops.
@see MultiStageLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter Online Documentation}*/
export class SelfLoopRouter {
/**
Creates a new {@link SelfLoopRouter} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link SelfLoopRouter} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.cornerApproximationPointsCount] The number of points that are used for rounding the corners of the self-loops. This option sets the {@link SelfLoopRouter#cornerApproximationPointsCount} property on the created object.
@param [options.lineDistance] The distance between two self-loops incident to the same node. This option sets the {@link SelfLoopRouter#lineDistance} property on the created object.
@param [options.smartSelfLoopPlacement] Whether or not the algorithm chooses the side of the self-loop depending on the incident edges. This option sets the {@link SelfLoopRouter#smartSelfLoopPlacement} property on the created object.
@param [options.layoutStyle] The layout style of the self-loops. This option sets the {@link SelfLoopRouter#layoutStyle} property on the created object.
@param [options.keepAllSelfLoopRoutes] Whether or not this algorithm should keep all self-loop routes. This option sets the {@link SelfLoopRouter#keepAllSelfLoopRoutes} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
cornerApproximationPointsCount?: number;
lineDistance?: number;
smartSelfLoopPlacement?: boolean;
layoutStyle?: SelfLoopStyle | SelfLoopStyleStringValues;
keepAllSelfLoopRoutes?: boolean;
});
/**
Delegates the arrangement of nodes and edges to the {@link LayoutStageBase#coreLayout core layout algorithm} and routes the self-loops.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Calculates the routes of the self-loops in the given graph.
This method will only change the routes of self-loops that are not marked to be {@link SelfLoopRouter#KEEP_SELF_LOOP_LAYOUT_DP_KEY kept}.
This method is called by {@link SelfLoopRouter#applyLayout} after the {@link LayoutStageBase#coreLayout core layout algorithm} has arranged the graph
and all self-loops have been reinserted into the graph. It may be overridden to introduce custom self-loop styles.
@param graph the input graph
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter%23layoutSelfLoops(LayoutGraph) Online Documentation}*/
layoutSelfLoops(graph: LayoutGraph | null): void;
/**
Data provider key for determining which self-loops should keep their current path
If no {@link IDataProvider} is registered with this key, all self-loops will be routed.
@see SelfLoopRouter#keepAllSelfLoopRoutes
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter%23KEEP_SELF_LOOP_LAYOUT_DP_KEY Online Documentation}*/
static KEEP_SELF_LOOP_LAYOUT_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets the number of points that are used for rounding the corners of the self-loops.
For rounded corners, the number of points should be at least
<code>2</code>. All smaller values will produce one bend and, therefore, an orthogonal path.
The number of points for a corner must be at least <code>1</code>.
<b>Warning:</b> This option is only considered when style {@link SelfLoopStyle#ROUNDED} is used.
@throws {Exception} <b>(ArgumentError)</b> if the specified point count is less than <code>1</code>
@see SelfLoopRouter#layoutStyle
@see SelfLoopStyle#ROUNDED
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter%23cornerApproximationPointsCount Online Documentation}*/
cornerApproximationPointsCount: number;
/**
Gets or sets the distance between two self-loops incident to the same node.
The distance needs to be a non-negative value.
<b>Warning:</b> This value is only considered for layout style {@link SelfLoopStyle#ROUNDED}.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see SelfLoopStyle#ROUNDED
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter%23lineDistance Online Documentation}*/
lineDistance: number;
/**
Gets or sets whether or not the algorithm chooses the side of the self-loop depending on the incident edges.
If this option is enabled, self-loops are placed on one of the four corners of the corresponding node. For the choice of
the corner, the algorithm considers all incident edges. Otherwise, self-loops are always placed at the upper left corner
of the corresponding node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter%23smartSelfLoopPlacement Online Documentation}*/
smartSelfLoopPlacement: boolean;
/**
Gets or sets the layout style of the self-loops.
@throws {Exception} <b>(ArgumentError)</b> if the specified style is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter%23layoutStyle Online Documentation}*/
layoutStyle: SelfLoopStyle;
/**
Gets or sets whether or not this algorithm should keep all self-loop routes.
When enabled, no self-loop will be routed, instead the existing self-loop layout will be moved with its node.
<b>Warning:</b> Enabling this option will override the information in the {@link IDataProvider} registered with
{@link SelfLoopRouter#KEEP_SELF_LOOP_LAYOUT_DP_KEY}. Self-loops that were not marked will still not be routed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopRouter%23keepAllSelfLoopRoutes Online Documentation}*/
keepAllSelfLoopRoutes: boolean;
static $class: Class;
private $_SelfLoopRouter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SelfLoopRouter;
}
export interface SequentialLayout extends YObject, ILayoutAlgorithm {
}
/**
A {@link SequentialLayout} allows for chaining multiple {@link ILayoutAlgorithm layout algorithms}.
The {@link ILayoutAlgorithm layout algorithms} are invoked one after the other in the order in which they were
{@link SequentialLayout#appendLayout added to the chain}. They all run independently of each other, i.e., they only influence each other by the changed input
graph. Hence, the sequence determines the result.
One possible application would be to bundle several {@link ILayoutAlgorithm layout algorithms} and pass a single {@link ILayoutAlgorithm}
instance to a {@link BufferedLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SequentialLayout Online Documentation}*/
export class SequentialLayout {
/**
Creates a new {@link SequentialLayout} instance with an empty chain of {@link ILayoutAlgorithm layout algorithms}.
@param [options] The parameters to pass.
@param [options.layouts] The chain of {@link ILayoutAlgorithm layout algorithms}. This option sets the {@link SequentialLayout#layouts} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SequentialLayout Online Documentation}*/
constructor(options?: {
layouts?: IList<ILayoutAlgorithm> | null | IListConvertible<ILayoutAlgorithm>;
});
/**
Appends the given {@link ILayoutAlgorithm layout algorithm} to the end of the chain.
@param layouter the {@link ILayoutAlgorithm layout algorithm} to append
@see {@link https://docs.yworks.com/yfileshtml/#/api/SequentialLayout%23appendLayout(ILayoutAlgorithm) Online Documentation}*/
appendLayout(layouter: ILayoutAlgorithm | null): void;
/**
Appends the specified list of {@link ILayoutAlgorithm layout algorithms} to the end of the chain.
@param layouters the list of {@link ILayoutAlgorithm layout algorithms} to append
@see {@link https://docs.yworks.com/yfileshtml/#/api/SequentialLayout%23appendLayouts(ICollection%3CILayoutAlgorithm%3E) Online Documentation}*/
appendLayouts(layouters: ICollection<ILayoutAlgorithm> | null | ICollectionConvertible<ILayoutAlgorithm>): void;
/**
Executes all {@link ILayoutAlgorithm layout algorithms} in the chain, one after the other.
The order of the layout algorithms is defined by the order in which they were appended.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/SequentialLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Clears the current chain of {@link ILayoutAlgorithm layout algorithms} and therefore removes all layout algorithms.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SequentialLayout%23removeAll() Online Documentation}*/
removeAll(): void;
/**
Gets or sets the chain of {@link ILayoutAlgorithm layout algorithms}.
<h2>Note</h2>
The current chain is replaced by the specified layout algorithms.
@see SequentialLayout#appendLayout
@see SequentialLayout#appendLayouts
@see {@link https://docs.yworks.com/yfileshtml/#/api/SequentialLayout%23layouts Online Documentation}*/
layouts: IList<ILayoutAlgorithm> | null;
static $class: Class;
private $_SequentialLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SequentialLayout;
}
export interface ShuffleLayout extends YObject, ILayoutStage {
}
/**
This layout algorithm removes overlaps between nodes in a graph.
<em>Note:</em> The usage of {@link RemoveOverlapsStage} instead of this class is recommended for most use cases involving the
mere removal of overlaps. That stage offers a more powerful strategy to do the task.
<b>Layout Style</b>
The style of results often resembles the look of tiles which have been dropped onto each other, where tiles correspond
to nodes of the graph. The reason is that overlapping nodes are moved in order to resolve the overlap. During this
process, several nodes moving in the same direction may be stacked next to or above each other.
This algorithm does not route the edges of the input graph - edges might although be stretched due to the node
movement.
Nodes overlapping with other nodes will be moved in horizontal or vertical direction in order to remove overlaps. The
concept behind this removal step is based on a famous Russian arcade game.
<b>Features</b>
This layout stage can also be {@link ShuffleLayout#applyLayout executed} on its own, without specifying a {@link ShuffleLayout#coreLayout core layout algorithm}.
A minimum distance between nodes can be specified such that not only overlaps will be removed but nodes will keep this
specified distance to other nodes. This distance can be defined separately for each node using a {@link IDataProvider}
registered with key {@link ShuffleLayout#MINIMUM_DISTANCE_DP_KEY}. The minimum distance can also be specified
globally via {@link ShuffleLayout#minimumNodeDistance}.
To specify that specific nodes should not be moved, they can be marked as fixed using a {@link IDataProvider}
registered with key {@link ShuffleLayout#FIXED_NODE_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout Online Documentation}*/
export class ShuffleLayout {
/**
Creates a new instance of {@link ShuffleLayout} with default settings.
@param [options] The parameters to pass.
@param [options.coreLayout] The core layout algorithm. This option sets the {@link ShuffleLayout#coreLayout} property on the created object.
@param [options.horizontalOverlapCriterion] The criterion for marking an overlap as horizontal. This option sets the {@link ShuffleLayout#horizontalOverlapCriterion} property on the created object.
@param [options.minimumNodeDistance] The default minimum distance that has to be obeyed between any two nodes. This option sets the {@link ShuffleLayout#minimumNodeDistance} property on the created object.
@param [options.simpleMode] Whether or not the simple, fast layout mode of this algorithm is active. This option sets the {@link ShuffleLayout#simpleMode} property on the created object.
@param [options.barycenterMode] Whether or not the barycenter mode is used for node shuffling when removing overlaps. This option sets the {@link ShuffleLayout#barycenterMode} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout Online Documentation}*/
constructor(options?: {
coreLayout?: ILayoutAlgorithm | null;
horizontalOverlapCriterion?: HorizontalOverlapCriterion | HorizontalOverlapCriterionStringValues;
minimumNodeDistance?: number;
simpleMode?: boolean;
barycenterMode?: boolean;
});
/**
Performs the overlap removal (shuffle) algorithm on the given graph, after the {@link ShuffleLayout#coreLayout core layout algorithm} was applied to it.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for specifying a minimum distance for each node
The default minimum distance specified by {@link ShuffleLayout#minimumNodeDistance} will be ignored for a node if the {@link IDataProvider} registered
with this key contains a valid minimum distance for that node.
Minimum distance values need to be greater than <code>0</code>.
<h2>Note</h2>
If no {@link IDataProvider} with this key is registered, the minimum distance for all nodes is specified by {@link ShuffleLayout#minimumNodeDistance}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout%23MINIMUM_DISTANCE_DP_KEY Online Documentation}*/
static MINIMUM_DISTANCE_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for marking nodes as fixed
A node marked as fixed will not be moved by this algorithm but stay at its current position.
<b>Warning:</b> If two nodes overlap and both nodes are fixed, then this algorithm will not remove the corresponding overlap.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout%23FIXED_NODE_DP_KEY Online Documentation}*/
static FIXED_NODE_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the core layout algorithm.
This algorithm is wrapped by this stage. It is invoked in {@link ILayoutAlgorithm#applyLayout}. The {@link ILayoutStage}
may add pre- and post-processing steps before and after calling the core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
/**
Gets or sets the criterion for marking an overlap as horizontal.
This criterion influences how overlaps will be resolved. If an overlap is considered horizontal, it will <em>preferably</em> be solved by
moving nodes horizontally, else vertically.
<b>Warning:</b> This criterion will only have an effect if the {@link ShuffleLayout#simpleMode simple mode} is disabled.
@throws {Exception} <b>(ArgumentError)</b> if the given criterion is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout%23horizontalOverlapCriterion Online Documentation}*/
horizontalOverlapCriterion: HorizontalOverlapCriterion;
/**
Gets or sets the default minimum distance that has to be obeyed between any two nodes.
This default distance will be considered for a node if the {@link IDataProvider} registered with the graph
with key {@link ShuffleLayout#MINIMUM_DISTANCE_DP_KEY} does not contain a valid, positive distance for that node.
If there is no {@link IDataProvider} registered with the mentioned key, then this default distance will be
applied to all nodes.
The minimum distance needs to be a non-negative value.
@throws {Exception} <b>(ArgumentError)</b> if the given distance is negative
@see ShuffleLayout#MINIMUM_DISTANCE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout%23minimumNodeDistance Online Documentation}*/
minimumNodeDistance: number;
/**
Gets or sets whether or not the simple, fast layout mode of this algorithm is active.
Enabling this mode, the overlap removal step will be executed using a simpler and less sophisticated approach. All
overlaps will only be solved by moving nodes vertically. The algorithm will not try to figure out which direction might
be better for overlap removal.
The runtime will improve, but results may be of lower quality when using this mode.
<b>Warning:</b> When enabling this mode, the specified {@link ShuffleLayout#horizontalOverlapCriterion horizontal overlap criterion} will not have any effect.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout%23simpleMode Online Documentation}*/
simpleMode: boolean;
/**
Gets or sets whether or not the barycenter mode is used for node shuffling when removing overlaps.
If this mode is active, the overlap removal step will be executed two times for each direction, once with the normal
node ordering and once with the reversed ordering. Finally, the barycenter between both results will be used for
assigning the node coordinates.
Activating the barycenter mode allows more symmetric results for some graphs. However, the runtime might increase.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShuffleLayout%23barycenterMode Online Documentation}*/
barycenterMode: boolean;
static $class: Class;
private $_ShuffleLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ShuffleLayout;
}
export interface SimpleProfitModel extends YObject, IProfitModel {
/**
@inheritdoc*/
getProfit(candidate: LabelCandidate | null): number;
} /**
{@link SimpleProfitModel} implements the ranking for edge and node labels.
In this implementation node label positions have no ranking, the profit for all candidates is <code>1</code>.
The ranking of edge label positions depends on the locations described in the {@link PreferredPlacementDescriptor}
associated with an edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleProfitModel Online Documentation}*/
export class SimpleProfitModel {
/**
Creates a new instance of {@link SimpleProfitModel}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleProfitModel Online Documentation}*/
constructor();
static $class: Class;
private $_SimpleProfitModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SimpleProfitModel;
}
export interface SliderEdgeLabelLayoutModel extends YObject, IEdgeLabelLayoutModel {
/**
@inheritdoc*/
createModelParameter(labelBounds: YOrientedRectangle | null, edgeLayout: IEdgeLayout | null, sourceNode: INodeLayout | null, targetNode: INodeLayout | null): Object | null;
/**
@inheritdoc*/
getLabelPlacement(labelSize: YDimension | null, edgeLayout: IEdgeLayout | null, sourceNode: INodeLayout | null, targetNode: INodeLayout | null, para: any | null): YOrientedRectangle | null;
} /**
{@link SliderEdgeLabelLayoutModel} allows placement of labels at a set of continuous rotated positions along both
sides of an edge or directly on the edge path.
It is possible to specify distance values that control the distance between label and edge and between label and nodes.
{@link SliderMode#CENTER} mode on the left, {@link SliderMode#SIDE} mode in the middle and {@link SliderMode#SINGLE_SIDE}
on the right
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel Online Documentation}*/
export class SliderEdgeLabelLayoutModel {
/**
Creates a new instance of {@link SliderEdgeLabelLayoutModel} with the given mode.
@param mode the slider mode
@see SliderEdgeLabelLayoutModel#mode
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel Online Documentation}*/
constructor(mode: SliderMode | SliderModeStringValues);
/**
Returns all {@link EdgeLabelCandidate}s that describe valid label positions within this model.
The number of candidates and their respective locations are computed depending on the geometries of both label and
edge.
@param labelLayout the label for which candidates should be generated
@param edgeLayout the layout of the edge to which the label belongs
@param sourceLayout the layout of the source node of the label-owning edge
@param targetLayout the layout of the target node of the label-owning edge
@returns a list of {@link EdgeLabelCandidate} instances
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel%23getLabelCandidates(IEdgeLabelLayout%2CIEdgeLayout%2CINodeLayout%2CINodeLayout) Online Documentation}*/
getLabelCandidates(labelLayout: IEdgeLabelLayout | null, edgeLayout: IEdgeLayout | null, sourceLayout: INodeLayout | null, targetLayout: INodeLayout | null): YList | null;
/**
Gets the model's slider mode which determines whether the label slides {@link SliderMode#CENTER on the edge}, {@link SliderMode#SIDE along both sides of the edge} or
{@link SliderMode#SINGLE_SIDE along one side of the edge}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel%23mode Online Documentation}*/
readonly mode: SliderMode;
/**
Gets or sets whether or not the distance to the edge is interpreted relative to the edge's path.
If enabled, the label is placed to the left of the edge segment (relative to the segment direction) if the given
distance is less than <code>0</code> and to the right of the edge segment if the given distance is greater than <code>0</code>.
If disabled, the label is placed below the edge segment (in geometric sense) if the distance is less than <code>0</code> and above
the edge segment if the distance is greater than <code>0</code>.
@see SliderEdgeLabelLayoutModel#distance
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel%23distanceRelativeToEdge Online Documentation}*/
distanceRelativeToEdge: boolean;
/**
Gets or sets the distance between the label's box and the edge's path.
<h2>Note</h2>
The interpretation of positive/negative values depends on property {@link SliderEdgeLabelLayoutModel#distanceRelativeToEdge}.
@see SliderEdgeLabelLayoutModel#distanceRelativeToEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel%23distance Online Documentation}*/
distance: number;
/**
Gets or sets whether or not edge labels are automatically rotated according to the angle of the corresponding reference
edge segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel%23autoRotation Online Documentation}*/
autoRotation: boolean;
/**
Gets or sets whether or not edge labels get flipped if they would be upside down in their current position.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel%23autoFlipping Online Documentation}*/
autoFlipping: boolean;
/**
Gets or sets the angle (measured in radians) of the label model.
The angle is applied in clockwise direction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel%23angle Online Documentation}*/
angle: number;
/**
Gets a model parameter that encodes the default position of this model's allowed edge label positions.
This parameter can be passed to {@link SliderEdgeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
label box.
The default positions for the {@link SliderEdgeLabelLayoutModel#mode slider modes} are relative to the first edge segment, either at the beginning of the
segment ({@link SliderMode#SIDE} or
{@link SliderMode#SINGLE_SIDE}) or at its middle ({@link SliderMode#CENTER}).
@see SliderEdgeLabelLayoutModel#mode
@see {@link https://docs.yworks.com/yfileshtml/#/api/SliderEdgeLabelLayoutModel%23defaultParameter Online Documentation}*/
readonly defaultParameter: Object | null;
static $class: Class;
private $_SliderEdgeLabelLayoutModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SliderEdgeLabelLayoutModel;
}
export interface StraightLineEdgeRouter extends LayoutStageBase {
}
/**
This edge routing algorithm routes edges as straight-lines considering the specified {@link PortConstraint}s.
<b>Layout Style</b>
The edges connect to their ports with straight-lines. This may lead to edges crossing nodes.
<b>Concept</b>
{@link StraightLineEdgeRouter} will first invoke the {@link LayoutStageBase#coreLayout} if one exists. Then, it will remove all bends from the edges
and will place the ports either on the center of the nodes or at a location specified by a
{@link PortConstraint}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouter Online Documentation}*/
export class StraightLineEdgeRouter {
/**
Creates a new instance of {@link StraightLineEdgeRouter} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouter Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new instance of {@link StraightLineEdgeRouter} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.scope] The (sub-)set of edges to be routed. This option sets the {@link StraightLineEdgeRouter#scope} property on the created object.
@param [options.affectedNodesDpKey] The key to register a {@link IDataProvider} that provides the selected state of the nodes of the graph. This option sets the {@link StraightLineEdgeRouter#affectedNodesDpKey} property on the created object.
@param [options.affectedEdgesDpKey] The key to register a {@link IDataProvider} that provides the selected state of the edges of the graph. This option sets the {@link StraightLineEdgeRouter#affectedEdgesDpKey} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouter Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
scope?: EdgeRouterScope | EdgeRouterScopeStringValues;
affectedNodesDpKey?: any | null;
affectedEdgesDpKey?: any | null;
});
/**
Routes the edges as straight-lines from their source to their target after invoking the {@link LayoutStageBase#applyLayoutCore core layout algorithm}.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouter%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the (sub-)set of edges to be routed.
{@link StraightLineEdgeRouter} will only route those edges and leave the remaining edges unchanged.
@throws {Exception} <b>(ArgumentError)</b> if the given argument is an unknown specifier
@see StraightLineEdgeRouter#affectedEdgesDpKey
@see StraightLineEdgeRouter#affectedNodesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouter%23scope Online Documentation}*/
scope: EdgeRouterScope;
/**
Gets or sets the key to register a {@link IDataProvider} that provides the selected state of the nodes of the
graph.
If the scope only includes {@link EdgeRouterScope#ROUTE_EDGES_AT_AFFECTED_NODES edges at selected nodes}, only edges of the nodes marked in this {@link IDataProvider}
are routed while all other edges are considered to have fixed routes.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see StraightLineEdgeRouter#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouter%23affectedNodesDpKey Online Documentation}*/
affectedNodesDpKey: any | null;
/**
Gets or sets the key to register a {@link IDataProvider} that provides the selected state of the edges of the
graph.
If the scope only includes {@link EdgeRouterScope#ROUTE_AFFECTED_EDGES selected edges}, only edges of the nodes marked in this {@link IDataProvider} are
routed while all other edges are considered to have fixed routes.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see StraightLineEdgeRouter#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouter%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
static $class: Class;
private $_StraightLineEdgeRouter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StraightLineEdgeRouter;
}
export interface SubgraphLayout extends LayoutStageBase {
}
/**
A {@link SubgraphLayout} is a {@link ILayoutStage} that filters out all graph elements that are not part of
a specific subgraph.
The nodes that induce this subgraph are marked using a {@link IDataProvider} registered with a {@link SubgraphLayout#affectedNodesDpKey specific key}.
They will be arranged by the {@link LayoutStageBase#coreLayout core layout algorithm}.
<b>Warning:</b>
All excluded nodes and their incident edges keep their locations. Hence, arranging only a subgraph may lead to
overlaps.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubgraphLayout Online Documentation}*/
export class SubgraphLayout {
/**
Creates a new {@link SubgraphLayout} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubgraphLayout Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link SubgraphLayout} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.affectedNodesDpKey] The key to register a {@link IDataProvider} which marks the nodes that induce the subgraph to be arranged. This option sets the {@link SubgraphLayout#affectedNodesDpKey} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubgraphLayout Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
affectedNodesDpKey?: any | null;
});
/**
Delegates the arrangement of the {@link SubgraphLayout#affectedNodesDpKey specified subgraph} of the given graph to the {@link LayoutStageBase#coreLayout core layout algorithm}.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubgraphLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the key to register a {@link IDataProvider} which marks the nodes that induce the subgraph to be
arranged.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubgraphLayout%23affectedNodesDpKey Online Documentation}*/
affectedNodesDpKey: any | null;
static $class: Class;
private $_SubgraphLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SubgraphLayout;
}
export interface SwimlaneRepresentative extends YObject {
}
/**
A {@link SwimlaneRepresentative SwimlaneRepresentative} represents the position and arrangement mode for a swimlane.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneRepresentative Online Documentation}*/
export class SwimlaneRepresentative {
/**
Creates a new {@link SwimlaneRepresentative} instance.
@param swimlanePos denotes the current position of the represented swimlane
@param allowRearrangement <code>true</code> if the represented swimlane may be {@link SwimlaneRepresentative#allowRearrangement rearranged}, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneRepresentative Online Documentation}*/
constructor(swimlanePos: number, allowRearrangement: boolean);
/**
Gets or sets the current position of the represented swimlane.
The swimlane positions are described by a zero-based index and must not be negative.
@throws {Exception} <b>(ArgumentError)</b> if the position is smaller than <code>0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneRepresentative%23swimlanePos Online Documentation}*/
swimlanePos: number;
/**
Gets or sets whether or not the represented swimlane can be rearranged.
For all swimlanes with disabled rearrangement, the relative ordering given by {@link SwimlaneRepresentative#swimlanePos} is preserved. The other swimlanes may be
rearranged.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneRepresentative%23allowRearrangement Online Documentation}*/
allowRearrangement: boolean;
static $class: Class;
private $_SwimlaneRepresentative: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SwimlaneRepresentative;
}
export interface Swimlanes extends YObject {
}
/**
{@link Swimlanes} provides helper classes and methods to handle swimlanes.
It is used by {@link HierarchicLayout} to arrange the nodes in lanes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Swimlanes Online Documentation}*/
export class Swimlanes {
/**
Calculates an ordering of the swimlanes considering the specified ordering mode.
Depending on the swimlane mode, the ordering will try to reduce:
<ol>
<li>the number of swimlanes traversed by edges
<li>the number of swimlanes spanned by {@link LayoutGroupingSupport#isGroupNode group nodes}
</ol>
<h2>Note</h2>
A group node can span a swimlane without having a descendant in it.
<h2>Note</h2>
After calling this method, {@link SwimlaneRepresentative#swimlanePos} returns an integer value <code>(>= 0)</code> for each swimlane representative denoting the optimized
position of the associated swimlane.
@param graph the input graph
@param node2Swimlane the assignment of nodes to swimlanes
@param iterations the number of iterations performed by the heuristic.
@param mode one of the valid swimlane ordering mode specifier
@see SwimlaneRepresentative
@see LayoutGroupingSupport
@see {@link https://docs.yworks.com/yfileshtml/#/api/Swimlanes%23arrangeSwimlanes(Graph%2CIDataProvider%2Cnumber%2CSwimlanesMode) Online Documentation}*/
static arrangeSwimlanes(graph: Graph | null, node2Swimlane: IDataProvider | null, iterations: number, mode: SwimlanesMode | SwimlanesModeStringValues): void;
/**
Calculates an ordering of the swimlanes.
The ordering will try to reduce:
<ol>
<li>the number of swimlanes traversed by edges
<li>the number of swimlanes spanned by {@link LayoutGroupingSupport#isGroupNode group nodes}
</ol>
<h2>Note</h2>
A group node can span a swimlane without having a descendant in it.
<h2>Note</h2>
After calling this method, {@link SwimlaneRepresentative#swimlanePos} returns an integer value <code>(>= 0)</code> for each swimlane representative denoting the optimized
position of the associated swimlane.
@param graph the input graph
@param node2Swimlane the assignment of nodes to swimlanes
@see SwimlaneRepresentative
@see LayoutGroupingSupport
@see {@link https://docs.yworks.com/yfileshtml/#/api/Swimlanes%23arrangeSwimlanes(Graph%2CIDataProvider) Online Documentation}*/
static arrangeSwimlanes(graph: Graph | null, node2Swimlane: IDataProvider | null): void;
static $class: Class;
private $_Swimlanes: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Swimlanes;
}
/**
@see TabularLayout#layoutPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutPolicy Online Documentation}*/
export enum TabularLayoutPolicy {
/**
Layout policy that automatically chooses the table size of the layout.
The resulting table gets a dimension such that both row count and column count are similar, leading to a maximally
square table with respect to the row and column count (actual node sizes are not considered). For example, given a graph
with 14 nodes, a 4x4 grid will be created.
The nodes are arranged in the table in a row-wise fashion using the order provided by {@link TabularLayout#nodeComparer}.
<b>Warning:</b> If a {@link PartitionGrid} instance is registered that already contains {@link PartitionGrid#rows rows} and/or {@link PartitionGrid#columns columns}, the number of
rows/columns is ignored and, furthermore, the list of rows will be
<em>cleared</em>. However, the settings of the first row and the first column are used as template for the settings of the newly created
rows and columns.
@see TabularLayout#layoutPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutPolicy%23AUTO_SIZE Online Documentation}*/
AUTO_SIZE,
/**
Layout policy that adopts the size from an already registered {@link PartitionGrid} instance.
The algorithm generates a tabular layout with the number of {@link PartitionGrid#rows rows} and {@link PartitionGrid#columns columns} as specified by the registered grid
instance. The settings of the respective {@link RowDescriptor}s and
{@link ColumnDescriptor}s are considered.
Using this layout policy, it is possible to exactly define in which cell of the table a certain node should be placed by
mapping a specific {@link PartitionCellId} to a node. {@link PartitionCellId} instances can be created using
methods {@link PartitionGrid#createCellId} and {@link PartitionGrid#createCellId}. Multi-cells (see e.g.
{@link PartitionGrid#createCellSpanId}) are not allowed. The node-cell mapping must be defined in a {@link IDataProvider}
registered with the graph with key {@link PartitionGrid#PARTITION_CELL_ID_DP_KEY}.
Nodes without a specific mapping to a cell are inserted one after another in a row-wise manner in the order defined by
the specified {@link TabularLayout#nodeComparer}.
<b>Warning:</b> The registered {@link PartitionGrid} instance must be large enough for all nodes to fit into the grid. Otherwise,
a {@link Exception InvalidGraphStructureError} exception will be thrown.
@see TabularLayout#layoutPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutPolicy%23FIXED_SIZE Online Documentation}*/
FIXED_SIZE,
/**
Layout policy that uses the coordinates of the input graph to determine the table size and the mapping of the nodes to
the cells.
Enabling this feature, the current layout of the input graph will be interpreted as a sketch for the layout algorithm.
Nodes are assigned to the same row if there exists a horizontal line that crosses through both of their bounding boxes.
Similarly, nodes are assigned to the same column if there exists a vertical line that crosses through both of their
bounding boxes. Clearly, this strategy only works smoothly if the given input already resembles a rough table
arrangement.
If the given layout does not offer any division of the nodes into column and rows or contains overlapping nodes, then
the sketch is ambiguous and the algorithm will chose one valid variant.
<b>Warning:</b> If a {@link PartitionGrid} instance is registered that already contains {@link PartitionGrid#rows rows} and/or {@link PartitionGrid#columns columns}, the number of
rows/columns is ignored and, furthermore, the list of rows will be
<em>cleared</em>. However, the settings of the first row and the first column are used as template for the settings of the newly created
rows and columns.
@see TabularLayout#layoutPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutPolicy%23FROM_SKETCH Online Documentation}*/
FROM_SKETCH
}
/** Automatic conversion to {@link TabularLayoutPolicy}. */
type TabularLayoutPolicyStringValues = "auto-size" | "fixed-size" | "from-sketch";
export interface TabularLayoutNodeLayoutDescriptor extends YObject {
}
/**
This class is used by {@link TabularLayout} to provide placement details for nodes during the layout process.
It allows to configure layout settings individually for each node of the input graph.
A {@link TabularLayoutNodeLayoutDescriptor} can be specified individually for single nodes using a {@link IDataProvider}
that returns a descriptor instance for the node, or <code>null</code> if no individual descriptor is bound to a node. In the latter
case, the {@link TabularLayout#defaultNodeLayoutDescriptor default descriptor} is used. The {@link IDataProvider} must be registered with the graph with key
{@link TabularLayout#NODE_LAYOUT_DESCRIPTOR_DP_KEY}.
@see TabularLayout#NODE_LAYOUT_DESCRIPTOR_DP_KEY
@see TabularLayout#defaultNodeLayoutDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutNodeLayoutDescriptor Online Documentation}*/
export class TabularLayoutNodeLayoutDescriptor {
/**
Creates a new instance of a {@link TabularLayoutNodeLayoutDescriptor} with the default values.
@param [options] The parameters to pass.
@param [options.verticalAlignment] The vertical alignment of the node within its row. This option sets the {@link TabularLayoutNodeLayoutDescriptor#verticalAlignment} property on the created object.
@param [options.horizontalAlignment] The horizontal alignment of the node within its column. This option sets the {@link TabularLayoutNodeLayoutDescriptor#horizontalAlignment} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutNodeLayoutDescriptor Online Documentation}*/
constructor(options?: {
verticalAlignment?: number;
horizontalAlignment?: number;
});
/**
Creates a copy of this descriptor instance.
@returns a copy of this descriptor instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutNodeLayoutDescriptor%23createCopy() Online Documentation}*/
createCopy(): TabularLayoutNodeLayoutDescriptor | null;
/**
Gets or sets the vertical alignment of the node within its row.
All nodes of the same row will be aligned according to their vertical alignment setting.
<ul>
<li>A value of <code>0.0</code> corresponds to top-alignment.
<li>A value of <code>0.5</code> corresponds to center-alignment.
<li>A value of <code>1.0</code> corresponds to bottom-alignment.
</ul>
The vertical alignment value must be within <code>[0,1]</code>.
@throws {Exception} <b>(ArgumentError)</b> if the given value is not within interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutNodeLayoutDescriptor%23verticalAlignment Online Documentation}*/
verticalAlignment: number;
/**
Gets or sets the horizontal alignment of the node within its column.
All nodes of the same column will be aligned according to their horizontal alignment setting.
<ul>
<li>A value of <code>0.0</code> corresponds to left-alignment.
<li>A value of <code>0.5</code> corresponds to center-alignment.
<li>A value of <code>1.0</code> corresponds to right-alignment.
</ul>
The horizontal alignment value must be within <code>[0,1]</code>.
@throws {Exception} <b>(ArgumentError)</b> if the given value is not within interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutNodeLayoutDescriptor%23horizontalAlignment Online Documentation}*/
horizontalAlignment: number;
static $class: Class;
private $_TabularNodeLayoutDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TabularLayoutNodeLayoutDescriptor;
}
export interface TabularLayout extends YObject, ILayoutAlgorithm {
}
/**
This layout algorithm places nodes in a tabular fashion.
<b>Layout Style</b>
The nodes are placed based on an underlying {@link PartitionGrid} structure. Only nodes are arranged, edges are
not routed, but their bend points are cleared. Therefore, this algorithm is suitable if nodes should be arranged in rows
and columns in a regular fashion.
<b>Concept</b>
The table consists of {@link PartitionCellId cells} which form {@link RowDescriptor rows} and {@link ColumnDescriptor columns}. Each cell contains a single node or might also be empty. Note
that it is not required that the input graph already contains a {@link PartitionGrid} for this algorithm to
properly work. It is, however, useful if the resulting row and column geometry should be retrieved (e.g. to visualize
the table). It is mandatory to register a {@link PartitionGrid} when using layout policy
{@link TabularLayoutPolicy#FIXED_SIZE}.
Rows/columns do not overlap and are not compacted: nodes that belong to different rows are strictly separated with
respect to the covered y-coordinates range and nodes that belong to different columns are strictly separated with
respect to the covered x-coordinate range.
<b>Features</b>
It is possible to define in which cell of the table a certain node should be placed when using policy
{@link TabularLayoutPolicy#FIXED_SIZE}. See the policy documentation for more details.
The algorithm considers the following properties associated with {@link RowDescriptor}s and
{@link ColumnDescriptor}s of a {@link PartitionGrid} registered with the input graph:
<ul>
<li>{@link RowDescriptor#topInset}
<li>{@link RowDescriptor#bottomInset}
<li>{@link RowDescriptor#minimumHeight}
<li>{@link ColumnDescriptor#leftInset}
<li>{@link ColumnDescriptor#rightInset}
<li>{@link ColumnDescriptor#minimumWidth}
</ul>
Group nodes are supported but only top-level groups are handled like an actual node in the table. The content of
top-level groups is kept fix. That means that the content keeps the relative location with respect to the top-level
group but it is not recursively arranged in a tabular fashion. To achieve recursive layouts where the content is a
tabular layout too, use {@link RecursiveGroupLayout} with {@link TabularLayout} as core layout algorithm.
Generally, it is recommended that group nodes already have proper bounds enclosing all content (and their labels).
Otherwise, overlaps may occur, as group node sizes are not changed by this algorithm. {@link RecursiveGroupLayout}
with {@link RecursiveGroupLayout#NULL_LAYOUT} as core algorithm can be used to recursively assign group node
sizes.
<b>Warning:</b> The settings {@link PartitionGrid#optimizeRowOrder} and {@link PartitionGrid#optimizeColumnOrder} of a given grid instance are not considered by this layout algorithm. Furthermore, the following
properties of rows and columns are ignored as well: {@link RowDescriptor#tightness}, {@link RowDescriptor#indexFixed}, {@link ColumnDescriptor#tightness} and {@link ColumnDescriptor#indexFixed}.
@see PartitionGrid
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayout Online Documentation}*/
export class TabularLayout {
/**
Initializes a new instance of the {@link TabularLayout} class with default settings.
@param [options] The parameters to pass.
@param [options.defaultNodeLayoutDescriptor] The {@link TabularLayoutNodeLayoutDescriptor} instance used for all the nodes that do not have an individual layout descriptor. This option sets the {@link TabularLayout#defaultNodeLayoutDescriptor} property on the created object.
@param [options.layoutPolicy] The policy defining the size of the resulting tabular layout and the mapping of the nodes to the table cells. This option sets the {@link TabularLayout#layoutPolicy} property on the created object.
@param [options.considerNodeLabels] Whether or not the layout algorithm reserves space for node labels. This option sets the {@link TabularLayout#considerNodeLabels} property on the created object.
@param [options.nodeComparer] The {@link IComparer} that defines the order in which the <em>free</em> nodes are placed in the table. This option sets the {@link TabularLayout#nodeComparer} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayout Online Documentation}*/
constructor(options?: {
defaultNodeLayoutDescriptor?: TabularLayoutNodeLayoutDescriptor | null;
layoutPolicy?: TabularLayoutPolicy | TabularLayoutPolicyStringValues;
considerNodeLabels?: boolean;
nodeComparer?: IComparer<any> | null;
});
/**
Arranges the input graph in a tabular fashion.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for providing layout information for each node
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayout%23NODE_LAYOUT_DESCRIPTOR_DP_KEY Online Documentation}*/
static NODE_LAYOUT_DESCRIPTOR_DP_KEY: NodeDpKey<TabularLayoutNodeLayoutDescriptor> | null;
/**
Gets or sets the {@link TabularLayoutNodeLayoutDescriptor} instance used for all the nodes that do not have an
individual layout descriptor.
@throws {Exception} <b>(ArgumentError)</b> if the specified descriptor is <code>null</code>
@see TabularLayout#NODE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayout%23defaultNodeLayoutDescriptor Online Documentation}*/
defaultNodeLayoutDescriptor: TabularLayoutNodeLayoutDescriptor | null;
/**
Gets or sets the policy defining the size of the resulting tabular layout and the mapping of the nodes to the table
cells.
This policy is a crucial setting for the algorithm and should be chosen depending on the desired use case. Overview over
the policies:
<ul>
<li>{@link TabularLayoutPolicy#AUTO_SIZE}: no {@link PartitionGrid} instance required. The size of the table is
chosen automatically and nodes are sorted using {@link TabularLayout#nodeComparer}.
<li>{@link TabularLayoutPolicy#FIXED_SIZE}: requires a registered {@link PartitionGrid} instance that defines
the size of the tabular layout and, optionally, the mapping of nodes to specific cells.
<li>{@link TabularLayoutPolicy#FROM_SKETCH}: no {@link PartitionGrid} instance required. The size of the table
and the positioning of the nodes is solely derived from the coordinates of the input graph (sketch).
</ul>
@throws {Exception} <b>(ArgumentError)</b> if the given policy is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayout%23layoutPolicy Online Documentation}*/
layoutPolicy: TabularLayoutPolicy;
/**
Gets or sets whether or not the layout algorithm reserves space for node labels.
Internally, nodes are treated like they would have a size including their labels. Therefore, layout results may get
significantly larger if this feature is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets the {@link IComparer} that defines the order in which the <em>free</em> nodes are placed in the table.
The nodes are arranged in the table in a row-wise fashion using the order provided by this
{@link IComparer}, i.e., the first row is filled from left to right, then the second row is filled, again from left to right etc.
Nodes that are mapped to specific {@link PartitionCellId} are considered to be <em>fixed</em> and are not sorted using this {@link IComparer}.
This only applies when using policy {@link TabularLayoutPolicy#FIXED_SIZE}. Furthermore, if {@link TabularLayoutPolicy#FROM_SKETCH}
is active, this property is ignored.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayout%23nodeComparer Online Documentation}*/
nodeComparer: IComparer<any> | null;
static $class: Class;
private $_TabularLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TabularLayout;
}
export interface TemporaryGroupNodeInsertionStage extends LayoutStageBase {
}
/**
{@link TemporaryGroupNodeInsertionStage} is useful if nodes should be treated by layout algorithms as if they were
actually grouped without the need to manually create a grouped graph for the layout process.
This stage converts given component Ids into a valid, temporary grouping structure that is visible for the {@link LayoutStageBase#coreLayout core layout algorithm}
and is obeyed by it if it supports grouped graphs. Importantly, it is not possible to realize a nested grouping
structure using this stage. If nested groups are required during layout, they need to be defined the usual way by using
{@link IDataProvider} keys specified in {@link GroupingKeys}.
This stage works as follows:
<ol>
<li>A graph grouping is derived from the given
{@link TemporaryGroupNodeInsertionStage#COMPONENT_ID_DP_KEY component Ids}. All nodes with the same Id are assigned to a newly created group node that is inserted into the graph.
<li>The {@link LayoutStageBase#coreLayout core layout algorithm} is executed. It works on the graph containing the inserted group nodes.
<li>The temporarily inserted group nodes are removed, restoring the original graph structure.
</ol>
In case that the input graph is already grouped, the following should be noted:
<ul>
<li>All component nodes must be on the
<em>same hierarchy level</em>, except for the case that group nodes are themselves part of the component. However, if a group node is in a component,
then all its descendants need to be assigned to the same component too. Otherwise, an Exception is thrown.
<li>The {@link LayoutStageBase#coreLayout core layout algorithm} sees both, the newly inserted groups and the old, existing ones and does not distinguish among
them.
</ul>
@see TemporaryGroupNodeInsertionStage#COMPONENT_ID_DP_KEY
@see TemporaryGroupNodeInsertionStage#COMPONENT_LAYOUT_ALGORITHM_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemporaryGroupNodeInsertionStage Online Documentation}*/
export class TemporaryGroupNodeInsertionStage {
/**
Creates a new {@link TemporaryGroupNodeInsertionStage} with the given layout algorithm as {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [core=null] the core layout algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemporaryGroupNodeInsertionStage Online Documentation}*/
constructor(core?: ILayoutAlgorithm | null);
/**
Creates temporary group nodes based on the sub-graphs define by the {@link TemporaryGroupNodeInsertionStage#COMPONENT_ID_DP_KEY component Id} mapping, executes the core layout
algorithm, and, finally, removes the temporary group nodes.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemporaryGroupNodeInsertionStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for specifying components of the input graph
Nodes with the same component Id constitute a component and are assigned to the same, temporary, group node, which is
visible during the execution of the {@link LayoutStageBase#coreLayout core layout algorithm} and again removed afterwards.
<b>Warning:</b> It is not allowed to define nested components. If nesting is required, define a real grouping structure - see
{@link GroupingKeys}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemporaryGroupNodeInsertionStage%23COMPONENT_ID_DP_KEY Online Documentation}*/
static COMPONENT_ID_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for specifying a layout algorithm for each component node when used with the recursive group layout
stage
If this stage is used in conjunction with {@link RecursiveGroupLayout}, the layout algorithm specified here is
mapped to be responsible for the arrangement of the nodes forming a specified component. The first found {@link ILayoutAlgorithm}
instance bound to any node of a component is applied to that component. Internally, the temporary group node that
encloses the component nodes is automatically mapped in a {@link IDataProvider} registered with key
{@link RecursiveGroupLayout#GROUP_NODE_LAYOUT_DP_KEY}.
<b>Warning:</b> A mapping registered with this key has an effect only if the core layout algorithm is of type
{@link RecursiveGroupLayout}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemporaryGroupNodeInsertionStage%23COMPONENT_LAYOUT_ALGORITHM_DP_KEY Online Documentation}*/
static COMPONENT_LAYOUT_ALGORITHM_DP_KEY: NodeDpKey<ILayoutAlgorithm> | null;
static $class: Class;
private $_TemporaryGroupNodeInsertionStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemporaryGroupNodeInsertionStage;
}
/**
@see GraphTransformer#operation
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/OperationType Online Documentation}*/
export enum OperationType {
/**
Operation specifier defining that the graph will be mirrored at the x-axis.
@see GraphTransformer#operation
@see {@link https://docs.yworks.com/yfileshtml/#/api/OperationType%23MIRROR_X_AXIS Online Documentation}*/
MIRROR_X_AXIS,
/**
Operation specifier defining that the graph will be mirrored at the y-axis.
@see GraphTransformer#operation
@see {@link https://docs.yworks.com/yfileshtml/#/api/OperationType%23MIRROR_Y_AXIS Online Documentation}*/
MIRROR_Y_AXIS,
/**
Operation specifier defining that the graph will be rotated.
@see GraphTransformer#operation
@see GraphTransformer#rotationAngle
@see GraphTransformer#bestFitRotationEnabled
@see {@link https://docs.yworks.com/yfileshtml/#/api/OperationType%23ROTATE Online Documentation}*/
ROTATE,
/**
Operation specifier defining that the graph will be scaled.
@see GraphTransformer#operation
@see GraphTransformer#scaleFactor
@see GraphTransformer#setScaleFactors
@see GraphTransformer#scaleNodeSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/OperationType%23SCALE Online Documentation}*/
SCALE,
/**
Operation specifier defining that the graph will be translated.
@see GraphTransformer#operation
@see GraphTransformer#translateX
@see GraphTransformer#translateY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OperationType%23TRANSLATE Online Documentation}*/
TRANSLATE
}
/** Automatic conversion to {@link OperationType}. */
type OperationTypeStringValues = "mirror-x-axis" | "mirror-y-axis" | "rotate" | "scale" | "translate";
export interface GraphTransformer extends MultiStageLayout, ILayoutStage {
}
/**
This layout algorithm applies geometric transformations to (sub-)graphs.
<b>Layout Style</b>
There are several ways to transform the graph that include mirroring, rotating, scaling and translating.
<b>Features</b>
The layout can be {@link OperationType#SCALE scaled} with independent factors for the horizontal and vertical scaling. Optionally, the sizes of
nodes can be scaled, too.
The layout can be mirrored either on the {@link OperationType#MIRROR_X_AXIS x-axis} or the {@link OperationType#MIRROR_Y_AXIS y-axis}.
A {@link OperationType#ROTATE rotation} can be specified. Instead of {@link GraphTransformer#rotationAngle defining} a rotation angle the angle can be {@link GraphTransformer#bestFitRotationEnabled determined automatically} to fit
specific bounds.
The coordinates in the layout can be {@link OperationType#TRANSLATE translated} by a specific vector.
To only apply previously mentioned transformations to a subgraph, the {@link SubgraphLayout} needs to be {@link MultiStageLayout#subgraphLayoutEnabled enabled}.
Then the nodes in the subgraph must be marked in a {@link IDataProvider} registered with the according {@link SubgraphLayout#affectedNodesDpKey key}.
<h2>Note</h2>
Labels are not affected by the transformation. They will stay at the location that is defined by their label model.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer Online Documentation}*/
export class GraphTransformer {
/**
Creates a new instance of {@link GraphTransformer} with default settings.
@param [options] The parameters to pass.
@param [options.coreLayout] The core layout algorithm. This option sets the {@link GraphTransformer#coreLayout} property on the created object.
@param [options.operation] The transformation operation. This option sets the {@link GraphTransformer#operation} property on the created object.
@param [options.rotationAngle] The angle of rotation. This option sets the {@link GraphTransformer#rotationAngle} property on the created object.
@param [options.bestFitRotationEnabled] Whether or not to automatically choose the best fitting rotation angle. This option sets the {@link GraphTransformer#bestFitRotationEnabled} property on the created object.
@param [options.preferredHeight] The preferred height of the layout. This option sets the {@link GraphTransformer#preferredHeight} property on the created object.
@param [options.preferredWidth] The preferred width of the layout. This option sets the {@link GraphTransformer#preferredWidth} property on the created object.
@param [options.scaleNodeSize] Whether or not to also scale node sizes when performing the {@link OperationType#SCALE scale operation}. This option sets the {@link GraphTransformer#scaleNodeSize} property on the created object.
@param [options.scaleFactor] The uniform scaling factor used for the {@link OperationType#SCALE scale operation}. This option sets the {@link GraphTransformer#scaleFactor} property on the created object.
@param [options.scaleFactorY] The vertical scaling factor used for the {@link OperationType#SCALE scale operation}. This option sets the {@link GraphTransformer#scaleFactorY} property on the created object.
@param [options.scaleFactorX] The horizontal scaling factor used for the {@link OperationType#SCALE scale operation}. This option sets the {@link GraphTransformer#scaleFactorX} property on the created object.
@param [options.translateX] The horizontal translation distance. This option sets the {@link GraphTransformer#translateX} property on the created object.
@param [options.translateY] The vertical translation distance. This option sets the {@link GraphTransformer#translateY} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link MultiStageLayout#orientationLayoutEnabled} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link MultiStageLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link MultiStageLayout#componentLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer Online Documentation}*/
constructor(options?: {
coreLayout?: ILayoutAlgorithm | null;
operation?: OperationType | OperationTypeStringValues;
rotationAngle?: number;
bestFitRotationEnabled?: boolean;
preferredHeight?: number;
preferredWidth?: number;
scaleNodeSize?: boolean;
scaleFactor?: number;
scaleFactorY?: number;
scaleFactorX?: number;
translateX?: number;
translateY?: number;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
orientationLayoutEnabled?: boolean;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
selfLoopRouterEnabled?: boolean;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
componentLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Rotates the given graph so its resulting bounding box fits best to the given bounds.
@param graph the graph that is rotated
@param width the width of the preferred layout bounds
@param height the height of the preferred layout bounds
@returns the rotation angle
@see GraphTransformer#bestFitRotationEnabled
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23applyBestFitRotationAngle(LayoutGraph%2Cnumber%2Cnumber) Online Documentation}*/
static applyBestFitRotationAngle(graph: LayoutGraph | null, width: number, height: number): number;
/**
Applies the selected transformation to the given graph after invoking the optional {@link GraphTransformer#coreLayout core layout algorithm}.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Returns the rotation angle for the graph that will result in a layout that fits best to the given bounds.
@param graph the graph that is rotated
@param width the width of the preferred layout bounds
@param height the height of the preferred layout bounds
@returns the rotation angle
@see GraphTransformer#applyBestFitRotationAngle
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23findBestFitRotationAngle(LayoutGraph%2Cnumber%2Cnumber) Online Documentation}*/
static findBestFitRotationAngle(graph: LayoutGraph | null, width: number, height: number): number;
/**
Specifies the preferred size of the layout.
If performing a {@link OperationType#ROTATE rotation} and this rotation should be
{@link GraphTransformer#bestFitRotationEnabled automatically fitted}, then the given preferred size defines the aspect ratio that will be used for fitting the layout
rotation.
The preferred width and height need to be non-negative.
<b>Warning:</b> This option only has an effect if {@link GraphTransformer#bestFitRotationEnabled the rotation is fitted to the bounds}.
@param width the preferred width of the layout
@param height the preferred height of the layout
@throws {Exception} <b>(ArgumentError)</b> if the specified width or height is negative
@see GraphTransformer#preferredWidth
@see GraphTransformer#preferredHeight
@see GraphTransformer#bestFitRotationEnabled
@see GraphTransformer#operation
@see OperationType#ROTATE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23preferredLayoutSize(number%2Cnumber) Online Documentation}*/
preferredLayoutSize(width: number, height: number): void;
/**
Fits the given graph into the specified rectangular bounds.
If the bounding box fits in the given bounds, nothing is done. Otherwise, the graph is scaled and optionally moved to
fit.
<h2>Note</h2>
The algorithm does not modify the node's sizes.
<h2>Note</h2>
This operation won't have any effects, if there are nodes that are bigger than the requested bounding box size.
@param graph the graph to modify
@param x the upper left x-coordinate of the bounds
@param y the upper left y-coordinate of the bounds
@param w the width of the bounds
@param h the height of the bounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23setMaximalBounds(LayoutGraph%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static setMaximalBounds(graph: LayoutGraph | null, x: number, y: number, w: number, h: number): void;
/**
Specifies the horizontal and vertical scaling factor used for the {@link OperationType#SCALE scale operation}.
Only positive scaling factors are allowed.
<b>Warning:</b> This option only has an effect if the layout is {@link OperationType#SCALE scaled}.
@param xFactor the horizontal scaling factor
@param yFactor the vertical scaling factor
@throws {Exception} <b>(ArgumentError)</b> if one the specified scaling factors is negative or <code>0</code>
@see GraphTransformer#scaleFactorX
@see GraphTransformer#scaleFactorY
@see GraphTransformer#operation
@see OperationType#SCALE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23setScaleFactors(number%2Cnumber) Online Documentation}*/
setScaleFactors(xFactor: number, yFactor: number): void;
/**
Translates the layout coordinates by the given vector.
@param graph the graph that is translated
@param dx the horizontal distance the given graph is moved
@param dy the vertical distance the given graph is moved
@see OperationType#TRANSLATE
@see GraphTransformer#operation
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23translate(LayoutGraph%2Cnumber%2Cnumber) Online Documentation}*/
static translate(graph: LayoutGraph | null, dx: number, dy: number): void;
/**
Gets or sets the core layout algorithm.
This algorithm is wrapped by this stage. It is invoked in {@link ILayoutAlgorithm#applyLayout}. The {@link ILayoutStage}
may add pre- and post-processing steps before and after calling the core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
/**
Gets or sets the transformation operation.
@throws {Exception} <b>(ArgumentError)</b> if the specified operation is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23operation Online Documentation}*/
operation: OperationType;
/**
Gets or sets the angle of rotation.
The angle must be given in degrees.
<b>Warning:</b> This option only has an effect if the layout is {@link OperationType#ROTATE rotated}.
@see GraphTransformer#operation
@see OperationType#ROTATE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23rotationAngle Online Documentation}*/
rotationAngle: number;
/**
Gets or sets whether or not to automatically choose the best fitting rotation angle.
The angle will be chosen such that the bounding box after the rotation fits best to the {@link GraphTransformer#preferredLayoutSize preferred aspect ratio}.
<b>Warning:</b> This option only has an effect if the layout is {@link OperationType#ROTATE rotated}.
<b>Warning:</b> When this option is enabled, the layout algorithm will ignore the {@link GraphTransformer#rotationAngle specified angle}.
@see GraphTransformer#operation
@see OperationType#ROTATE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23bestFitRotationEnabled Online Documentation}*/
bestFitRotationEnabled: boolean;
/**
Gets or sets the preferred height of the layout.
This value will be used for {@link GraphTransformer#bestFitRotationEnabled fitting the rotation} to preferred bounds.
The preferred height needs to be non-negative.
<b>Warning:</b> This option only has an effect if performing a {@link OperationType#ROTATE rotation} and {@link GraphTransformer#bestFitRotationEnabled the rotation is fitted to the bounds}.
@throws {Exception} <b>(ArgumentError)</b> if the specified height is negative
@see GraphTransformer#preferredWidth
@see GraphTransformer#bestFitRotationEnabled
@see GraphTransformer#operation
@see OperationType#ROTATE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23preferredHeight Online Documentation}*/
preferredHeight: number;
/**
Gets or sets the preferred width of the layout.
This value will be used for {@link GraphTransformer#bestFitRotationEnabled fitting the rotation} to preferred bounds.
The preferred width needs to be non-negative.
<b>Warning:</b> This option only has an effect if performing a {@link OperationType#ROTATE rotation} and {@link GraphTransformer#bestFitRotationEnabled the rotation is fitted to the bounds}.
@throws {Exception} <b>(ArgumentError)</b> if the specified width is negative
@see GraphTransformer#preferredHeight
@see GraphTransformer#bestFitRotationEnabled
@see GraphTransformer#operation
@see OperationType#ROTATE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23preferredWidth Online Documentation}*/
preferredWidth: number;
/**
Gets or sets whether or not to also scale node sizes when performing the {@link OperationType#SCALE scale operation}.
<b>Warning:</b> This option only has an effect if the layout is {@link OperationType#SCALE scaled}.
@see GraphTransformer#operation
@see OperationType#SCALE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23scaleNodeSize Online Documentation}*/
scaleNodeSize: boolean;
/**
Sets the uniform scaling factor used for the {@link OperationType#SCALE scale operation}.
This factor applies vertical and horizontal.
Only positive scaling factors are allowed.
<b>Warning:</b> This option only has an effect if the layout is {@link OperationType#SCALE scaled}.
@throws {Exception} <b>(ArgumentError)</b> if the specified scaling factor is negative or <code>0</code>
@see GraphTransformer#scaleFactorX
@see GraphTransformer#scaleFactorY
@see GraphTransformer#operation
@see OperationType#SCALE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23scaleFactor Online Documentation}*/
scaleFactor: number;
/**
Gets or sets the vertical scaling factor used for the {@link OperationType#SCALE scale operation}.
Only positive scaling factors are allowed.
<b>Warning:</b> This option only has an effect if the layout is {@link OperationType#SCALE scaled}.
@throws {Exception} <b>(ArgumentError)</b> if the specified scaling factor is negative or <code>0</code>
@see GraphTransformer#scaleFactorX
@see GraphTransformer#operation
@see OperationType#SCALE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23scaleFactorY Online Documentation}*/
scaleFactorY: number;
/**
Gets or sets the horizontal scaling factor used for the {@link OperationType#SCALE scale operation}.
Only positive scaling factors are allowed.
<b>Warning:</b> This option only has an effect if the layout is {@link OperationType#SCALE scaled}.
@throws {Exception} <b>(ArgumentError)</b> if the specified scaling factor is negative or <code>0</code>
@see GraphTransformer#scaleFactorY
@see GraphTransformer#operation
@see OperationType#SCALE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23scaleFactorX Online Documentation}*/
scaleFactorX: number;
/**
Gets or sets the horizontal translation distance.
A positive value means the graph is moved to the right while, a negative value means the graph is moved to the left.
<b>Warning:</b> This option only has an effect if the layout is {@link OperationType#TRANSLATE translated}.
@see GraphTransformer#translateY
@see GraphTransformer#operation
@see OperationType#TRANSLATE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23translateX Online Documentation}*/
translateX: number;
/**
Gets or sets the vertical translation distance.
A positive value means the graph is moved downwards, while a negative value means the graph is moved upwards.
<b>Warning:</b> This option only has an effect if the layout is {@link OperationType#TRANSLATE translated}.
@see GraphTransformer#translateX
@see GraphTransformer#operation
@see OperationType#TRANSLATE
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphTransformer%23translateY Online Documentation}*/
translateY: number;
static $class: Class;
private $_GraphTransformer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphTransformer;
}
/**
@see AspectRatioNodePlacer#fillStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillStyle Online Documentation}*/
export enum FillStyle {
/**
Style specifier which defines that rows/columns are aligned with their leading side.
@see AspectRatioNodePlacer#fillStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillStyle%23LEADING Online Documentation}*/
LEADING,
/**
Style specifier which defines that rows/columns are aligned with their center.
@see AspectRatioNodePlacer#fillStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillStyle%23CENTERED Online Documentation}*/
CENTERED,
/**
Style specifier which defines that rows/columns are justified to have the same width/height.
@see AspectRatioNodePlacer#fillStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillStyle%23JUSTIFY Online Documentation}*/
JUSTIFY,
/**
Style specifier which defines that rows/columns are aligned with their trailing side.
@see AspectRatioNodePlacer#fillStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillStyle%23TRAILING Online Documentation}*/
TRAILING
}
/** Automatic conversion to {@link FillStyle}. */
type FillStyleStringValues = "leading" | "centered" | "justify" | "trailing";
/**
@see AspectRatioTreeLayout#createBends
@see AspectRatioTreeLayout#getRootPlacement
@see AspectRatioTreeLayout#rootPlacement
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootPlacement Online Documentation}*/
export enum RootPlacement {
/**
A root placement specifier for placing the root centered above its subtree, depending on the actual {@link MultiStageLayout#layoutOrientation layout orientation}.
@see AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootPlacement%23TOP Online Documentation}*/
TOP,
/**
A root placement specifier for placing the root in the upper left corner of the subtree bounds with respect to the
actual {@link MultiStageLayout#layoutOrientation layout orientation}.
Whether the root is placed entirely beside the subtree with no horizontal overlaps or entirely above the subtree without
vertical overlaps will be determined by the layout of the subtree. The layout algorithm tries to minimize the bounds of
the subtree.
@see AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
@see RootPlacement#CORNER_TOP
@see RootPlacement#CORNER_SIDE
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootPlacement%23CORNER Online Documentation}*/
CORNER,
/**
A root placement specifier for placing the root in the upper left corner of the subtree bounds with respect to the
actual {@link MultiStageLayout#layoutOrientation layout orientation}.
The root is placed entirely beside the subtree with no horizontal overlaps. This might by important if the root's height
is very large while its width is small.
@see AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootPlacement%23CORNER_SIDE Online Documentation}*/
CORNER_SIDE,
/**
A root placement specifier for placing the root of a subtree in the upper left corner of the subtree bounds with respect
to the actual {@link MultiStageLayout#layoutOrientation layout orientation}.
The root is placed entirely above the subtree with no vertical overlaps. This might by important if the root's width is
very large while its height is small.
@see AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootPlacement%23CORNER_TOP Online Documentation}*/
CORNER_TOP
}
/** Automatic conversion to {@link RootPlacement}. */
type RootPlacementStringValues = "top" | "corner" | "corner-side" | "corner-top";
/**
A direction specifier for placing the child nodes next to each other in direction of the actual
{@link MultiStageLayout#layoutOrientation layout orientation}, with the edges connecting in the direction of flow.
@see AspectRatioTreeLayout#createBends
@see AspectRatioTreeLayout#getSubtreeArrangement
@see AspectRatioTreeLayout#subtreeArrangement
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeArrangement Online Documentation}*/
export enum SubtreeArrangement {
/**
A direction specifier for placing the child nodes next to each other in direction of the actual
{@link MultiStageLayout#layoutOrientation layout orientation}, with the edges connecting in the direction of flow.
@see AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeArrangement%23HORIZONTAL Online Documentation}*/
HORIZONTAL,
/**
A direction specifier for placing the child nodes above each other in direction of the actual
{@link MultiStageLayout#layoutOrientation layout orientation}, with the edges connecting orthogonally to the direction of flow.
@see AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeArrangement%23VERTICAL Online Documentation}*/
VERTICAL
}
/** Automatic conversion to {@link SubtreeArrangement}. */
type SubtreeArrangementStringValues = "horizontal" | "vertical";
/**
@see BalloonLayout#interleavedMode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterleavedMode Online Documentation}*/
export enum InterleavedMode {
/**
A child placement mode where all child nodes are placed around their parent with equal distances.
<h2>Note</h2>
The point to which the distance to the parent is measured may vary depending on the selected {@link BalloonLayout#childAlignmentPolicy alignment policy} (e.g.
distance to node center or distance to convex hull border).
@see BalloonLayout#interleavedMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterleavedMode%23OFF Online Documentation}*/
OFF,
/**
A child placement mode where nodes are placed in an interleaved fashion, that is, with two different, alternating
distances to the parent node.
<h2>Note</h2>
If there is enough space within the desired child wedge of some node to place all children without using different
distances, interleaving will be omitted.
@see BalloonLayout#interleavedMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterleavedMode%23ALL_NODES Online Documentation}*/
ALL_NODES,
/**
A child placement mode where child nodes of selected nodes are placed in an interleaved fashion, that is, with two
different, alternating distances to the parent node.
All other nodes are placed without interleaving. The node selection can be specified by registering an appropriate {@link IDataProvider}
for key {@link BalloonLayout#INTERLEAVED_NODES_DP_KEY}.
<h2>Note</h2>
If there is enough space within the desired child wedge of a selected node to place all children without using different
distances, interleaving will be omitted.
<h2>Note</h2>
If there is no {@link IDataProvider} registered with key
{@link BalloonLayout#INTERLEAVED_NODES_DP_KEY}, then the selection of nodes whose children are arranged interleaved
will be made based on a simple heuristic.
@see BalloonLayout#interleavedMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterleavedMode%23MARKED_NODES Online Documentation}*/
MARKED_NODES
}
/** Automatic conversion to {@link InterleavedMode}. */
type InterleavedModeStringValues = "off" | "all-nodes" | "marked-nodes";
/**
@see BalloonLayout#nodeLabelingPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelingPolicy Online Documentation}*/
export enum NodeLabelingPolicy {
/**
Node labeling policy for ray-like label placement at nodes with zero or one child node.
Labels belonging to leaf nodes and nodes with exactly one successor (thus forming a sort of chain) will not be oriented
horizontal but ray-like; they get the same orientation as their nodes' incoming edge. The labels of the other nodes will
be oriented horizontally and placed at the center of the corresponding node.
@see BalloonLayout#nodeLabelingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelingPolicy%23RAY_LIKE Online Documentation}*/
RAY_LIKE,
/**
Node labeling policy for ray-like label placement at leaf nodes.
Labels corresponding to leaf nodes get the same orientation as their nodes' incoming edge and are placed outside the
node (without overlaps). The labels of the other nodes will be oriented horizontally and placed at the center of the
corresponding node.
@see BalloonLayout#nodeLabelingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelingPolicy%23RAY_LIKE_LEAVES Online Documentation}*/
RAY_LIKE_LEAVES,
/**
Node labeling policy for horizontal label placement at all nodes.
The node labels are placed at the center of the corresponding node.
@see BalloonLayout#nodeLabelingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelingPolicy%23HORIZONTAL Online Documentation}*/
HORIZONTAL
}
/** Automatic conversion to {@link NodeLabelingPolicy}. */
type NodeLabelingPolicyStringValues = "ray-like" | "ray-like-leaves" | "horizontal";
/**
@see BalloonLayout#childAlignmentPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildAlignmentPolicy Online Documentation}*/
export enum ChildAlignmentPolicy {
/**
Alignment policy to align child nodes rooted at the same parent such that the border of their convex hull has the same
distance to the parent node's center.
<h2>Note</h2>
Convex hulls of nodes are always defined as circles.
<h2>Note</h2>
Two different distances will be realized, if the {@link BalloonLayout#interleavedMode interleaved mode} is active.
@see BalloonLayout#childAlignmentPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildAlignmentPolicy%23PLAIN Online Documentation}*/
PLAIN,
/**
Alignment policy to align child nodes rooted at the same parent such that each child has the same center-to-center
distance to the parent node.
All nodes will be placed on a common radius around their parent, respective to their center coordinates.
<h2>Note</h2>
Two different radii will be realized, if the {@link BalloonLayout#interleavedMode interleaved mode} is active.
@see BalloonLayout#childAlignmentPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildAlignmentPolicy%23SAME_CENTER Online Documentation}*/
SAME_CENTER,
/**
Alignment policy for aligning child nodes such that the distances to their parent are kept short and drawings can
potentially become more compact.
This policy realizes the exact same child alignment as {@link ChildAlignmentPolicy#PLAIN} if there are no edge
labels associated with edges going to child nodes or if {@link BalloonLayout#integratedEdgeLabeling integrated edge labeling} is disabled.
If edge labels need to be considered, edge labels will be added to the convex hull of the child node to which the edge
connects. This means that child nodes connected with a labeled edge will be aligned at the border where the edge label
begins. In consequence, edge labels will be aligned with neighboring edge labels as well as neighboring child nodes if a
child node has no edge label at its incoming edge.
This alignment strategy is especially effective if large edge labels need to be considered while drawings should still
be compact and distances from parent to child nodes short.
<h2>Note</h2>
Since compactness is prioritized in conjunction with large edge labels, drawings may lack symmetry regarding node
distances.
@see BalloonLayout#childAlignmentPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildAlignmentPolicy%23COMPACT Online Documentation}*/
COMPACT,
/**
Alignment policy to align child nodes rooted at the same parent using a mixture of the other policies and aiming to
achieve symmetry and compactness at the same time.
Similar to
{@link ChildAlignmentPolicy#SAME_CENTER}, this policy aims to achieve alignments featuring equal distances between
parent and child node centers. However, a much larger distance for all child nodes caused by single large subtrees is
avoided by this policy. In such cases, the smaller subtrees may be placed on a common radius and the large subtrees are
placed with different distances.
<b>Warning:</b> Has no effect if the interleaved mode is active {@link BalloonLayout#interleavedMode}. In that case, a {@link ChildAlignmentPolicy#PLAIN plain child alignment} will be applied instead.
@see BalloonLayout#childAlignmentPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildAlignmentPolicy%23SMART Online Documentation}*/
SMART
}
/** Automatic conversion to {@link ChildAlignmentPolicy}. */
type ChildAlignmentPolicyStringValues = "plain" | "same-center" | "compact" | "smart";
/**
@see BalloonLayout#childOrderingPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildOrderingPolicy Online Documentation}*/
export enum ChildOrderingPolicy {
/**
Child ordering policy which orders child nodes depending on their subtree size such that large subtrees are placed next
to small ones in order to save space.
The actual area of a subtree is used as the subtree size. The smallest subtree is placed next to the largest subtree,
followed by the second smallest subtree and so on. This way, drawings can become more compact, because the adjacent
placement of large subtrees is avoided (which would require edges to become longer for subtrees to fit next to each
other).
@see BalloonLayout#childOrderingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildOrderingPolicy%23COMPACT Online Documentation}*/
COMPACT,
/**
Child ordering policy which sorts the child nodes according to their wedge angles.
The node with the greatest angle gets median position in the sorting order. The smaller the wedges, the farther away a
node will be placed from the median position. Using this policy, the produced results are often more symmetric.
@see BalloonLayout#childOrderingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildOrderingPolicy%23SYMMETRIC Online Documentation}*/
SYMMETRIC
}
/** Automatic conversion to {@link ChildOrderingPolicy}. */
type ChildOrderingPolicyStringValues = "compact" | "symmetric";
/**
@see BalloonLayout#rootNodePolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodePolicy Online Documentation}*/
export enum RootNodePolicy {
/**
Root node policy for choosing a node with indegree <code>0</code> as root node of the tree.
The selection of the root node will happen according to {@link TreeAlgorithm#getRoot}.
@see BalloonLayout#rootNodePolicy
@see TreeAlgorithm#getRoot
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodePolicy%23DIRECTED_ROOT Online Documentation}*/
DIRECTED_ROOT,
/**
Root node policy for choosing the center node as root node of the tree.
A {@link TreeAlgorithm#getCenterRoot center node} induces a minimum depth tree when being used as the root of that tree.
@see BalloonLayout#rootNodePolicy
@see TreeAlgorithm#getCenterRoot
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodePolicy%23CENTER_ROOT Online Documentation}*/
CENTER_ROOT,
/**
Root node policy for choosing a weighted center node as root node of the tree.
A {@link TreeAlgorithm#getWeightedCenterNode weighted center node} is a node which is part of the greatest number of all undirected paths in a graph.
@see BalloonLayout#rootNodePolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodePolicy%23WEIGHTED_CENTER_ROOT Online Documentation}*/
WEIGHTED_CENTER_ROOT,
/**
Root node policy for choosing a custom node as the root node of the tree.
The root node is specified using a {@link IDataProvider} registered with the graph with key
{@link BalloonLayout#SELECTED_ROOT_DP_KEY}.
@see BalloonLayout#rootNodePolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodePolicy%23SELECTED_ROOT Online Documentation}*/
SELECTED_ROOT
}
/** Automatic conversion to {@link RootNodePolicy}. */
type RootNodePolicyStringValues = "directed-root" | "center-root" | "weighted-center-root" | "selected-root";
/**
@see DefaultNodePlacer#DefaultNodePlacer
@see DefaultNodePlacer#DefaultNodePlacer
@see DefaultNodePlacer#DefaultNodePlacer
@see DefaultNodePlacer#childPlacement
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildPlacement Online Documentation}*/
export enum ChildPlacement {
/**
Placement specifier which defines that subtrees are placed one below the other and left of the root node.
@see DefaultNodePlacer#childPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildPlacement%23VERTICAL_TO_LEFT Online Documentation}*/
VERTICAL_TO_LEFT,
/**
Placement specifier which defines that subtrees are placed one below the other and right of the root node.
@see DefaultNodePlacer#childPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildPlacement%23VERTICAL_TO_RIGHT Online Documentation}*/
VERTICAL_TO_RIGHT,
/**
Placement specifier which defines that subtrees are placed horizontally next to each other and above the root node.
@see DefaultNodePlacer#childPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildPlacement%23HORIZONTAL_UPWARD Online Documentation}*/
HORIZONTAL_UPWARD,
/**
Placement specifier which defines that subtrees are placed horizontally next to each other and below the root node.
@see DefaultNodePlacer#childPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChildPlacement%23HORIZONTAL_DOWNWARD Online Documentation}*/
HORIZONTAL_DOWNWARD
}
/** Automatic conversion to {@link ChildPlacement}. */
type ChildPlacementStringValues = "vertical-to-left" | "vertical-to-right" | "horizontal-upward" | "horizontal-downward";
/**
@see DefaultNodePlacer#DefaultNodePlacer
@see DefaultNodePlacer#DefaultNodePlacer
@see DefaultNodePlacer#rootAlignment
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootAlignment Online Documentation}*/
export enum RootAlignment {
/**
Alignment specifier which defines that the root node is placed ahead of all its child subtrees.
Depending on the {@link DefaultNodePlacer#childPlacement child placement}, the root node is placed ahead (e.g.
{@link ChildPlacement#HORIZONTAL_DOWNWARD}: left) of the combined bounding box of all subtrees without labels.
@see DefaultNodePlacer#rootAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootAlignment%23LEADING_OFFSET Online Documentation}*/
LEADING_OFFSET,
/**
Alignment specifier which defines that the root node is placed aligned with its first subtree.
Depending on the {@link DefaultNodePlacer#childPlacement child placement}, the root node is placed aligned with the bounding box of its first (e.g.
{@link ChildPlacement#HORIZONTAL_DOWNWARD}: left) subtree.
@see DefaultNodePlacer#rootAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootAlignment%23LEADING Online Documentation}*/
LEADING,
/**
Alignment specifier which defines that the root node is placed aligned with the center of its subtrees.
@see DefaultNodePlacer#rootAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootAlignment%23CENTER Online Documentation}*/
CENTER,
/**
Alignment specifier which defines that the root node is placed at the median of the connection points to its subtrees.
@see DefaultNodePlacer#rootAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootAlignment%23MEDIAN Online Documentation}*/
MEDIAN,
/**
Alignment specifier which defines that the root node is placed aligned with its last subtree.
Depending on the {@link DefaultNodePlacer#childPlacement child placement}, the root node is placed aligned with the bounding box of its last (e.g.
{@link ChildPlacement#HORIZONTAL_DOWNWARD}: right) subtree.
@see DefaultNodePlacer#rootAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootAlignment%23TRAILING Online Documentation}*/
TRAILING,
/**
Alignment specifier which defines that the root is placed after all its subtree.
Depending on the {@link DefaultNodePlacer#childPlacement child placement}, the root node is placed after (e.g.
{@link ChildPlacement#HORIZONTAL_DOWNWARD}: right) of the combined bounding box of all subtrees without labels.
@see DefaultNodePlacer#rootAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootAlignment%23TRAILING_OFFSET Online Documentation}*/
TRAILING_OFFSET,
/**
Alignment specifier which defines that the root is placed after all its subtrees, centered on the bus.
Depending on the {@link DefaultNodePlacer#childPlacement child placement}, the root node is placed after (e.g.
{@link ChildPlacement#HORIZONTAL_DOWNWARD}: right) of the combined bounding box of all subtrees without labels. The
bus leaves directly from the root node.
<h2>Note</h2>
If there are {@link PortConstraint}s at source, the edge may bend inside the node to connect to the bus.
@see DefaultNodePlacer#rootAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootAlignment%23TRAILING_ON_BUS Online Documentation}*/
TRAILING_ON_BUS,
/**
Alignment specifier which defines that the root is placed ahead of all its subtrees, centered on the bus.
Depending on the {@link DefaultNodePlacer#childPlacement child placement}, the root node is placed ahead (e.g.
{@link ChildPlacement#HORIZONTAL_DOWNWARD}: left) of the combined bounding box of all subtrees without labels. The
bus leaves directly from the root node.
<h2>Note</h2>
If there are {@link PortConstraint}s at source, the edge may bend inside the node to connect to the bus.
@see DefaultNodePlacer#rootAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootAlignment%23LEADING_ON_BUS Online Documentation}*/
LEADING_ON_BUS
}
/** Automatic conversion to {@link RootAlignment}. */
type RootAlignmentStringValues = "leading-offset" | "leading" | "center" | "median" | "trailing" | "trailing-offset" | "trailing-on-bus" | "leading-on-bus";
/**
@see DefaultNodePlacer#DefaultNodePlacer
@see DefaultNodePlacer#routingStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutEdgeRoutingStyle Online Documentation}*/
export enum TreeLayoutEdgeRoutingStyle {
/**
Routing style specifier which defines that edge paths are routed orthogonally with the bends located in the channel
between the root node and the child nodes.
@see DefaultNodePlacer#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutEdgeRoutingStyle%23FORK Online Documentation}*/
FORK,
/**
Routing style specifier which defines that edge paths are routed orthogonally with just one bend.
Edges will leave the root node at its sides and bend exactly above the according child node. If the child node is placed
directly below the root node, the edge will bend inside of the root node and leave it at the bottom.
@see DefaultNodePlacer#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutEdgeRoutingStyle%23FORK_AT_ROOT Online Documentation}*/
FORK_AT_ROOT,
/**
Routing style specifier which defines that edge paths are routed with a straight segment to the center of the child
nodes.
<h2>Note</h2>
Edges may overlap with nodes when using this style.
@see DefaultNodePlacer#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutEdgeRoutingStyle%23STRAIGHT Online Documentation}*/
STRAIGHT,
/**
Routing style constant which defines that edge paths are routed with a straight segment to the connector of the {@link SubtreeShape}
of the child nodes.
@see DefaultNodePlacer#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutEdgeRoutingStyle%23POLYLINE Online Documentation}*/
POLYLINE
}
/** Automatic conversion to {@link TreeLayoutEdgeRoutingStyle}. */
type TreeLayoutEdgeRoutingStyleStringValues = "fork" | "fork-at-root" | "straight" | "polyline";
/**
@see DefaultTreeLayoutPortAssignment
@see DefaultTreeLayoutPortAssignment#mode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutPortAssignmentMode Online Documentation}*/
export enum TreeLayoutPortAssignmentMode {
/**
A port assignment specifier that defines that {@link PortConstraint}s are considered.
Ports are placed using the following rules:
<ul>
<li>edges without a {@link PortConstraint} are anchored at the center of the node
<li>edges with a weak {@link PortConstraint} are anchored at the center of their specified side
<li>edges with a strong {@link PortConstraint} are anchored at their initial coordinates
</ul>
If edges with
{@link PortConstraint}s are grouped, all edges in the same group will use the port coordinates of the first edge
in the group.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutPortAssignmentMode%23PORT_CONSTRAINT Online Documentation}*/
PORT_CONSTRAINT,
/**
A port assignment specifier which defines that edges are distributed evenly at the northern side of their nodes.
Grouped edges will use the same port coordinate and will be considered as a single edge when calculating the
distribution.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutPortAssignmentMode%23DISTRIBUTED_NORTH Online Documentation}*/
DISTRIBUTED_NORTH,
/**
A port assignment specifier which defines that edges are distributed evenly at the southern side of their nodes.
Grouped edges will use the same port coordinate and will be considered as a single edge when calculating the
distribution.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutPortAssignmentMode%23DISTRIBUTED_SOUTH Online Documentation}*/
DISTRIBUTED_SOUTH,
/**
A port assignment specifier which defines that edges are distributed evenly at the eastern side of their nodes.
Grouped edges will use the same port coordinate and will be considered as a single edge when calculating the
distribution.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutPortAssignmentMode%23DISTRIBUTED_EAST Online Documentation}*/
DISTRIBUTED_EAST,
/**
A port assignment specifier which defines that edges are distributed evenly at the western side of their nodes.
Grouped edges will use the same port coordinate and will be considered as a single edge when calculating the
distribution.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutPortAssignmentMode%23DISTRIBUTED_WEST Online Documentation}*/
DISTRIBUTED_WEST,
/**
A port assignment specifier which defines that all ports are reset to the center of their nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutPortAssignmentMode%23NONE Online Documentation}*/
NONE
}
/** Automatic conversion to {@link TreeLayoutPortAssignmentMode}. */
type TreeLayoutPortAssignmentModeStringValues = "port-constraint" | "distributed-north" | "distributed-south" | "distributed-east" | "distributed-west" | "none";
/**
A direction constant that describes that the last connector segment points up.
@see SubtreeShape#connectorDirection
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectorDirection Online Documentation}*/
export enum ConnectorDirection {
/**
A direction constant that describes that the last connector segment points up.
The edge from the parent will connect to the north of the {@link SubtreeShape}.
@see SubtreeShape#connectorDirection
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectorDirection%23NORTH Online Documentation}*/
NORTH,
/**
A direction constant that describes that the last connector segment points to the right.
The edge from the parent will connect to the east of the {@link SubtreeShape}.
@see SubtreeShape#connectorDirection
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectorDirection%23EAST Online Documentation}*/
EAST,
/**
A direction constant that describes that the last connector segment points down.
The edge from the parent will connect to the south of the {@link SubtreeShape}.
@see SubtreeShape#connectorDirection
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectorDirection%23SOUTH Online Documentation}*/
SOUTH,
/**
A direction constant that describes that the last connector segment points to the left.
The edge from the parent will connect to the west of the {@link SubtreeShape}.
@see SubtreeShape#connectorDirection
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectorDirection%23WEST Online Documentation}*/
WEST
}
/** Automatic conversion to {@link ConnectorDirection}. */
type ConnectorDirectionStringValues = "north" | "east" | "south" | "west";
/**
Side constant that defines the index of the eastern border line.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSide Online Documentation}*/
export enum BorderLineSide {
/**
Side constant that defines the index of the northern border line.
@see SubtreeShape#getBorderLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSide%23NORTH Online Documentation}*/
NORTH,
/**
Side constant that defines the index of the eastern border line.
@see SubtreeShape#getBorderLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSide%23EAST Online Documentation}*/
EAST,
/**
Side constant that defines the index of the southern border line.
@see SubtreeShape#getBorderLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSide%23SOUTH Online Documentation}*/
SOUTH,
/**
Side constant that defines the index of the western border line.
@see SubtreeShape#getBorderLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/BorderLineSide%23WEST Online Documentation}*/
WEST
}
/** Automatic conversion to {@link BorderLineSide}. */
type BorderLineSideStringValues = "north" | "east" | "south" | "west";
/**
A bus placement that places the bus that connects the child nodes in the center of the child subtrees.
@see GridNodePlacer#busPlacement
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusPlacement Online Documentation}*/
export enum BusPlacement {
/**
A bus placement that places the bus that connects the child nodes in a leading way, left of the child subtrees.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusPlacement%23LEADING Online Documentation}*/
LEADING,
/**
A bus placement that places the bus that connects the child nodes in a trailing way, right of the child subtrees.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusPlacement%23TRAILING Online Documentation}*/
TRAILING,
/**
A bus placement that places the bus that connects the child nodes in the center of the child subtrees.
For a row with <code>n > 1</code> children, the bus segment is placed after the <code>(n/2)th</code> child. If a row consists of a single child,
it is placed right of the bus.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusPlacement%23CENTER Online Documentation}*/
CENTER
}
/** Automatic conversion to {@link BusPlacement}. */
type BusPlacementStringValues = "leading" | "trailing" | "center";
/**
@see LayeredNodePlacer#routingStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredRoutingStyle Online Documentation}*/
export enum LayeredRoutingStyle {
/**
Routing style specifier which defines that edges consist of one straight segment.
<h2>Note</h2>
It is possible that edges overlap with nodes, depending on their size and vertical alignment. If
{@link TreeLayout#considerNodeLabels node labels are considered}, there might also be edge overlaps with node labels that are placed above the according
node.
@see LayeredNodePlacer#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredRoutingStyle%23STRAIGHT Online Documentation}*/
STRAIGHT,
/**
Routing style specifier which defines that edges are routed in an orthogonal, bus-like fashion.
@see LayeredNodePlacer#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredRoutingStyle%23ORTHOGONAL Online Documentation}*/
ORTHOGONAL
}
/** Automatic conversion to {@link LayeredRoutingStyle}. */
type LayeredRoutingStyleStringValues = "straight" | "orthogonal";
/**
@see MultiParentDescriptor#edgeStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentRoutingStyle Online Documentation}*/
export enum MultiParentRoutingStyle {
/**
Routing style for routing non-shared edge segments in a polyline fashion.
The edges connected to a multi-parent structure will share their paths until shortly above/below the multi-parent nodes.
This style only applies to the part of the edges where they split to connect to different multi-parents.
@see MultiParentDescriptor#edgeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentRoutingStyle%23POLYLINE Online Documentation}*/
POLYLINE,
/**
Routing style for routing non-shared edge segments in an orthogonal fashion.
The edges connected to a multi-parent structure will share their paths until shortly above/below the multi-parent nodes.
This style only applies to the part of the edges where they split to connect to different multi-parents.
@see MultiParentDescriptor#edgeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentRoutingStyle%23ORTHOGONAL Online Documentation}*/
ORTHOGONAL,
/**
Routing style for routing non-shared edge segments straight.
This style will route the part of the edges that doesn't share the same path directly from the center of the
multi-parent to the common point of the edges in the multi-parent structure.
The edges connected to a multi-parent structure will share their paths until shortly above/below the multi-parent nodes.
This style only applies to the part of the edges where they split to connect to different multi-parents.
@see MultiParentDescriptor#edgeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentRoutingStyle%23STRAIGHT Online Documentation}*/
STRAIGHT
}
/** Automatic conversion to {@link MultiParentRoutingStyle}. */
type MultiParentRoutingStyleStringValues = "polyline" | "orthogonal" | "straight";
/**
@see AspectRatioNodePlacer#determineChildConnector
@see AspectRatioNodePlacer#placeSubtreeOfNode
@see AssistantNodePlacer#determineChildConnector
@see AssistantNodePlacer#placeSubtree
@see AssistantNodePlacer#placeSubtreeOfNode
@see BusNodePlacer#determineChildConnector
@see BusNodePlacer#placeSubtreeOfNode
@see CompactNodePlacer#determineChildConnector
@see CompactNodePlacer#placeSubtreeOfNode
@see DefaultNodePlacer#determineChildConnector
@see DefaultNodePlacer#placeSubtreeOfNode
@see DelegatingNodePlacer#determineChildConnector
@see DelegatingNodePlacer#placeSubtree
@see DelegatingNodePlacer#placeSubtreeOfNode
@see DendrogramNodePlacer#placeSubtree
@see DoubleLineNodePlacer#determineChildConnector
@see DoubleLineNodePlacer#placeSubtreeOfNode
@see FreeNodePlacer#determineChildConnector
@see FreeNodePlacer#placeSubtreeOfNode
@see GridNodePlacer#determineChildConnector
@see GridNodePlacer#placeSubtreeOfNode
@see GroupedNodePlacer#placeSubtree
@see ITreeLayoutNodePlacer#placeSubtree
@see LayeredNodePlacer#determineChildConnector
@see LayeredNodePlacer#placeSubtreeOfNode
@see LeafNodePlacer#placeSubtree
@see LeftRightNodePlacer#determineChildConnector
@see LeftRightNodePlacer#placeSubtreeOfNode
@see NodePlacerBase#determineChildConnector
@see NodePlacerBase#placeSubtree
@see RotatableNodePlacerBase#determineChildConnector
@see RotatableNodePlacerBase#placeSubtree
@see RotatableNodePlacerBase#translateDirectionToModel
@see RotatableNodePlacerBase#translateDirectionToReal
@see SimpleNodePlacer#determineChildConnector
@see SimpleNodePlacer#placeSubtreeOfNode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentConnectorDirection Online Documentation}*/
export enum ParentConnectorDirection {
/**
Direction specifier which indicates that no connector should be calculated.
@see ITreeLayoutNodePlacer#placeSubtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentConnectorDirection%23NONE Online Documentation}*/
NONE,
/**
Direction specifier which indicates that any direction can be used for the connector to the parent node.
@see ITreeLayoutNodePlacer#placeSubtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentConnectorDirection%23ANY Online Documentation}*/
ANY,
/**
Direction specifier which indicates that the connector to the parent node should end in a segment that can be extended
by a segment that goes north.
@see ITreeLayoutNodePlacer#placeSubtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentConnectorDirection%23NORTH Online Documentation}*/
NORTH,
/**
Direction specifier which indicates that the connector to the parent node should end in a segment that can be extended
by a segment that goes east.
@see ITreeLayoutNodePlacer#placeSubtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentConnectorDirection%23EAST Online Documentation}*/
EAST,
/**
Direction specifier which indicates that the connector to the parent node should end in a segment that can be extended
by a segment that goes south.
@see ITreeLayoutNodePlacer#placeSubtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentConnectorDirection%23SOUTH Online Documentation}*/
SOUTH,
/**
Direction specifier which indicates that the connector to the parent node should end in a segment that can be extended
by a segment that goes west.
@see ITreeLayoutNodePlacer#placeSubtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParentConnectorDirection%23WEST Online Documentation}*/
WEST
}
/** Automatic conversion to {@link ParentConnectorDirection}. */
type ParentConnectorDirectionStringValues = "none" | "any" | "north" | "east" | "south" | "west";
/**
@see ClassicTreeLayout#portStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyle Online Documentation}*/
export enum PortStyle {
/**
A constant defining that ports are placed at the center of the nodes.
This constant uses <code>(0,0)</code> as port offset.
@see ClassicTreeLayout#portStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyle%23NODE_CENTER Online Documentation}*/
NODE_CENTER,
/**
A constant defining that ports are placed at the center of the border of the nodes.
@see ClassicTreeLayout#portStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyle%23BORDER_CENTER Online Documentation}*/
BORDER_CENTER,
/**
A constant defining that ports are evenly distributed along the border of the nodes.
@see ClassicTreeLayout#portStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyle%23BORDER_DISTRIBUTED Online Documentation}*/
BORDER_DISTRIBUTED,
/**
A constant defining that ports are assigned based on given {@link PortConstraint port constraints}.
@see ClassicTreeLayout#portStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortStyle%23PORT_CONSTRAINTS_AWARE Online Documentation}*/
PORT_CONSTRAINTS_AWARE
}
/** Automatic conversion to {@link PortStyle}. */
type PortStyleStringValues = "node-center" | "border-center" | "border-distributed" | "port-constraints-aware";
/**
@see ClassicTreeLayout#edgeRoutingStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayoutEdgeRoutingStyle Online Documentation}*/
export enum ClassicTreeLayoutEdgeRoutingStyle {
/**
A constant for routing the edges as straight-line segments.
@see ClassicTreeLayout#edgeRoutingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayoutEdgeRoutingStyle%23PLAIN Online Documentation}*/
PLAIN,
/**
A constant for routing the edges orthogonally in a bus-like fashion.
@see ClassicTreeLayout#edgeRoutingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayoutEdgeRoutingStyle%23ORTHOGONAL Online Documentation}*/
ORTHOGONAL
}
/** Automatic conversion to {@link ClassicTreeLayoutEdgeRoutingStyle}. */
type ClassicTreeLayoutEdgeRoutingStyleStringValues = "plain" | "orthogonal";
/**
@see ClassicTreeLayout#leafPlacement
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafPlacement Online Documentation}*/
export enum LeafPlacement {
/**
A policy for placing the leaf nodes in a stack-like fashion with balanced stack heights.
In this context, a stack-like fashion means that leaf nodes that connect to the same parent node are placed one upon the
other, resulting in horizontally compact layouts.
This policy tries to balance stack heights, i.e., for each subtree that consists only of leaf nodes, {@link LeafPlacement#LEAVES_STACKED_RIGHT}
or {@link LeafPlacement#LEAVES_STACKED_LEFT_AND_RIGHT} will be used depending on the number of leaves in the
subtree.
<b>Warning:</b> This policy has an effect only on subtrees where all children are leaf nodes.
@see ClassicTreeLayout#leafPlacement
@see ClassicTreeLayout#leafPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafPlacement%23LEAVES_STACKED Online Documentation}*/
LEAVES_STACKED,
/**
A policy for placing the leaf nodes in a stack-like fashion using a single stack left.
In this context, a stack-like fashion means that leaf nodes that connect to the same root node are placed one upon the
other, resulting in horizontally compact layouts.
This policy uses a single stack that lies to the left relative to the center of the parent node.
<b>Warning:</b> This policy has an effect only on subtrees where all children are leaf nodes.
@see ClassicTreeLayout#leafPlacement
@see ClassicTreeLayout#leafPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafPlacement%23LEAVES_STACKED_LEFT Online Documentation}*/
LEAVES_STACKED_LEFT,
/**
A policy for placing the leaf nodes in a stack-like fashion using a single stack right.
In this context, a stack-like fashion means that leaf nodes that connect to the same root node are placed one upon the
other, resulting in horizontally compact layouts.
This policy uses a single stack that lies to the right relative to the center of the parent node.
<b>Warning:</b> This policy has an effect only on subtrees where all children are leaf nodes.
@see ClassicTreeLayout#leafPlacement
@see ClassicTreeLayout#leafPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafPlacement%23LEAVES_STACKED_RIGHT Online Documentation}*/
LEAVES_STACKED_RIGHT,
/**
A policy for placing the leaf nodes in a stack-like fashion using two stacks.
In this context, a stack-like fashion means that leaf nodes that connect to the same root node are placed one upon the
other, resulting in horizontally compact layouts.
This policy distributes leaf nodes among two stacks, one to the left and one to the right relative to the center of the
parent node.
<b>Warning:</b> This policy has an effect only on subtrees where all children are leaf nodes.
@see ClassicTreeLayout#leafPlacement
@see ClassicTreeLayout#leafPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafPlacement%23LEAVES_STACKED_LEFT_AND_RIGHT Online Documentation}*/
LEAVES_STACKED_LEFT_AND_RIGHT,
/**
A policy for placing the leaf nodes in a dendrogram-like fashion.
According to this policy, all leaf nodes are placed in one layer (i.e. all leaves are placed on one horizontal line in a
top-to-bottom or in a bottom-to-top layout).
@see ClassicTreeLayout#leafPlacement
@see ClassicTreeLayout#leafPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafPlacement%23ALL_LEAVES_ON_SAME_LAYER Online Documentation}*/
ALL_LEAVES_ON_SAME_LAYER,
/**
A policy for placing the leaf nodes with the same parent in the same layer.
For example, siblings are being placed on a horizontal line in a top-to-bottom or in a bottom-to-top layout.
This policy produces vertically compact layouts.
@see ClassicTreeLayout#leafPlacement
@see ClassicTreeLayout#leafPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafPlacement%23SIBLINGS_ON_SAME_LAYER Online Documentation}*/
SIBLINGS_ON_SAME_LAYER
}
/** Automatic conversion to {@link LeafPlacement}. */
type LeafPlacementStringValues = "leaves-stacked" | "leaves-stacked-left" | "leaves-stacked-right" | "leaves-stacked-left-and-right" | "all-leaves-on-same-layer" | "siblings-on-same-layer";
/**
@see TreeMapLayout#tilingPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TilingPolicy Online Documentation}*/
export enum TilingPolicy {
/**
A tiling policy that uses the so-called 'squarify' algorithm.
It tries to generate nodes with a size close to a specified
{@link TreeMapLayout#aspectRatio aspect ratio}. This increases the readability of the tree maps.
The child ordering given by {@link TreeMapLayout#nodeComparer} is considered but it is not as predictable as for
{@link TilingPolicy#SLICE_AND_DICE}. The order is not strictly from top to bottom or from left to right, but it
might differ for different subsets of children. This is an inherent part of the squarified procedure.
The implementation follows the approach of <em>Bruls et al. [Bruls, Mark; Huizing, Kees; van Wijk, Jarke J. (2000). 'Squarified treemaps']</em>.
@see TreeMapLayout#tilingPolicy
@see TreeMapLayout#aspectRatio
@see {@link https://docs.yworks.com/yfileshtml/#/api/TilingPolicy%23SQUARIFIED Online Documentation}*/
SQUARIFIED,
/**
A tiling policy that places the child nodes one after another inside the parent node.
Child nodes are either tiled vertically (slice), one on top of the other with the width equal to the parent's width. Or
they are tiled horizontally (dice), one next to the other with the height equal to the parent's height. The tiling
strategy alternates with the hierarchy levels, e.g., level 1 uses slice, level 2 uses dice, level 3 uses slice and so
on.
This policy keeps the ordering of the children defined by {@link TreeMapLayout#nodeComparer} stable. Either children are strictly ordered from left to
right (dice) or from top to bottom (slice).
@see TreeMapLayout#tilingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/TilingPolicy%23SLICE_AND_DICE Online Documentation}*/
SLICE_AND_DICE
}
/** Automatic conversion to {@link TilingPolicy}. */
type TilingPolicyStringValues = "squarified" | "slice-and-dice";
export interface BalloonLayoutNodeInfo extends YObject {
}
/**
Encapsulates information bound to a node while the algorithm calculates a layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutNodeInfo Online Documentation}*/
export class BalloonLayoutNodeInfo {
/**
The upper angle of the subtree wedge rooted at the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutNodeInfo%23upperAngle Online Documentation}*/
upperAngle: number;
/**
The lower angle of the subtree wedge rooted at the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutNodeInfo%23lowerAngle Online Documentation}*/
lowerAngle: number;
/**
The angle of the gap to the wedge of the previous child node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutNodeInfo%23gapAngle Online Documentation}*/
gapAngle: number;
/**
The distance of the node to its root node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutNodeInfo%23dist Online Documentation}*/
dist: number;
/**
Gets the sum of all wedge angles of the subtree rooted at this node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutNodeInfo%23angleSum Online Documentation}*/
readonly angleSum: number;
static $class: Class;
private $_BalloonLayoutNodeInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BalloonLayoutNodeInfo;
}
export interface BalloonLayout extends MultiStageLayout {
}
/**
A tree layout algorithm that arranges the subtrees of the tree in a balloon-like fashion.
<b>Layout Style</b>
{@link BalloonLayout} is designed to arrange directed and undirected tree graphs. Subtrees rooted at a node are
placed in a radial fashion around their root node. All direct children of one node can be placed on a common circle
around their parent node (depending on the
{@link BalloonLayout#childAlignmentPolicy alignment policy}). Therefore, subtrees look like balloons or stars, especially if subtrees have similar sizes. The edges
of the tree are drawn as straight lines.
<b>Concept</b>
The algorithm executes the following steps:
<ol>
<li>Select a root node according to the specified {@link BalloonLayout#rootNodePolicy root policy}.
<li>Determine the placement of subtrees around the root using a bottom-up recursive approach (starting with leaf nodes).
<ul>
<li>For a leaf node: calculate the convex hull of the node.
<li>For a non-leaf node: {@link BalloonLayout#calculateChildArrangement arrange the children} rooted at the node. Children are sorted according to a specified {@link BalloonLayout#childOrderingPolicy ordering policy}
and a distance to their parent is chosen. This distance will be chosen such that subtrees rooted at the current node fit
into the
{@link BalloonLayout#preferredChildWedge preferred wedge angle}. The wedge angle defines how much radial space a subtree occupies around its parent; it is
computed using the convex hull of subtrees.
Finally, the convex hull of the subtree rooted at the current node is updated in order to include all the convex hulls
of child nodes. This is possible, because distances and wedges of the child subtrees are now known.
</ul>
<li>Assign the actual coordinates of nodes, again using a recursive approach (starting with the root node).
</ol>
<b>Features</b>
The algorithm features integrated {@link BalloonLayout#integratedEdgeLabeling edge labeling} as well as {@link BalloonLayout#integratedNodeLabeling node labeling}. Edge labels and node labels are placed
automatically without generating overlaps with other labels or graph elements. There are different {@link BalloonLayout#nodeLabelingPolicy ways} to place node
labels. Edge labeling will take the settings of {@link PreferredPlacementDescriptor} into account.
Defining a {@link BalloonLayout#preferredChildWedge preferred wedge angle} has a great influence on the layout style. Subtrees rooted at a node get a certain
amount of radial space to be placed around the parent node, such that a preferred angle close to <code>360</code> degrees will
generate drawings where subtrees look like balloons, while an angle close to <code>180</code> degrees could be chosen to get drawings
where subtrees look like semicircles.
Since it is computationally not very complex, {@link BalloonLayout} is very well suited for large tree graphs. It
performs well even for huge graphs.
This layout algorithm can only handle graphs with a tree structure. To apply it to a general graph, a {@link TreeReductionStage}
can be appended. This stage will temporarily remove some edges of the input graph until a tree is obtained. These edges
will later be reinserted and routed separately.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout Online Documentation}*/
export class BalloonLayout {
/**
Creates a new {@link BalloonLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.comparer] The {@link IComparer} instance that determines the order of the outgoing edges for each node of the tree. This option sets the {@link BalloonLayout#comparer} property on the created object.
@param [options.childOrderingPolicy] The child ordering policy for sorting the child nodes around their parents. This option sets the {@link BalloonLayout#childOrderingPolicy} property on the created object.
@param [options.minimumNodeDistance] The minimum distance to be kept between the nodes in the tree. This option sets the {@link BalloonLayout#minimumNodeDistance} property on the created object.
@param [options.fromSketchMode] Whether or not to consider the given coordinates of the input diagram when arranging the tree. This option sets the {@link BalloonLayout#fromSketchMode} property on the created object.
@param [options.rootNodePolicy] The root node selection policy of this layout algorithm. This option sets the {@link BalloonLayout#rootNodePolicy} property on the created object.
@param [options.preferredChildWedge] The preferred radial amount (wedge) in degrees that child nodes may in total occupy around their parent node. This option sets the {@link BalloonLayout#preferredChildWedge} property on the created object.
@param [options.preferredRootWedge] The preferred radial amount (wedge) in degrees that child nodes may in total occupy around the global root. This option sets the {@link BalloonLayout#preferredRootWedge} property on the created object.
@param [options.allowOverlaps] Whether or not (partially) overlapping nodes are allowed. This option sets the {@link BalloonLayout#allowOverlaps} property on the created object.
@param [options.compactnessFactor] The factor defining how compact layout results will potentially be, where a smaller factor produces potentially more compact layouts. This option sets the {@link BalloonLayout#compactnessFactor} property on the created object.
@param [options.minimumEdgeLength] The minimum length that this layout algorithm assigns to edges of the graph. This option sets the {@link BalloonLayout#minimumEdgeLength} property on the created object.
@param [options.considerNodeLabels] Whether or not the layout algorithm reserves space for node labels. This option sets the {@link BalloonLayout#considerNodeLabels} property on the created object.
@param [options.interleavedMode] The mode for child node arrangement. This option sets the {@link BalloonLayout#interleavedMode} property on the created object.
@param [options.childAlignmentPolicy] The child alignment policy for this layout algorithm. This option sets the {@link BalloonLayout#childAlignmentPolicy} property on the created object.
@param [options.integratedNodeLabeling] Whether or not the layout algorithm automatically places node labels. This option sets the {@link BalloonLayout#integratedNodeLabeling} property on the created object.
@param [options.integratedEdgeLabeling] Whether or not the layout algorithm automatically places edge labels. This option sets the {@link BalloonLayout#integratedEdgeLabeling} property on the created object.
@param [options.nodeLabelingPolicy] The policy defining how node labels are placed by the integrated node labeling mechanism (for example, the desired label orientation). This option sets the {@link BalloonLayout#nodeLabelingPolicy} property on the created object.
@param [options.nodeLabelSpacing] The distance between node labels belonging to the same node. This option sets the {@link BalloonLayout#nodeLabelSpacing} property on the created object.
@param [options.edgeLabelSpacing] The distance between edge labels belonging to the same edge as well as the distance of the edge labels to the target node of the edge. This option sets the {@link BalloonLayout#edgeLabelSpacing} property on the created object.
@param [options.chainStraighteningMode] Whether or not chains are drawn straight or not. This option sets the {@link BalloonLayout#chainStraighteningMode} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link BalloonLayout#componentLayoutEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link BalloonLayout#hideGroupsStageEnabled} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link BalloonLayout#orientationLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link BalloonLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link BalloonLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout Online Documentation}*/
constructor(options?: {
comparer?: IComparer<any> | null;
childOrderingPolicy?: ChildOrderingPolicy | ChildOrderingPolicyStringValues;
minimumNodeDistance?: number;
fromSketchMode?: boolean;
rootNodePolicy?: RootNodePolicy | RootNodePolicyStringValues;
preferredChildWedge?: number;
preferredRootWedge?: number;
allowOverlaps?: boolean;
compactnessFactor?: number;
minimumEdgeLength?: number;
considerNodeLabels?: boolean;
interleavedMode?: InterleavedMode | InterleavedModeStringValues;
childAlignmentPolicy?: ChildAlignmentPolicy | ChildAlignmentPolicyStringValues;
integratedNodeLabeling?: boolean;
integratedEdgeLabeling?: boolean;
nodeLabelingPolicy?: NodeLabelingPolicy | NodeLabelingPolicyStringValues;
nodeLabelSpacing?: number;
edgeLabelSpacing?: number;
chainStraighteningMode?: boolean;
componentLayoutEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
orientationLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
selfLoopRouterEnabled?: boolean;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
labelingEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Arranges the given graph as a tree graph in a balloon-like fashion.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph. This method is not side effect free in the sense that the order of edges or nodes in the input graph may change
during the layout process.
@param graph the input graph
@throws {Exception} <b>(InvalidGraphStructureError)</b> if the given graph is not a tree
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Calculates the wedge angle that has to be reserved for the subtree rooted at the given node scaling the {@link BalloonLayoutNodeInfo#dist distance} with
the given scale factor.
Given some {@link BalloonLayoutNodeInfo#dist distance} the {@link BalloonLayoutNodeInfo#upperAngle upper angle} and {@link BalloonLayoutNodeInfo#lowerAngle lower angle} of the wedge belonging to the subtree rooted at <code>root</code> will be
calculated and stored in the {@link BalloonLayoutNodeInfo} instance associated with <code>root</code>.
This method may be overridden to perform a custom wedge angle assignment scheme. The method is called when
{@link BalloonLayout#calculateChildArrangement arranging child nodes}. Large edge labels on the incoming edge to <code>root</code> need to be considered, if {@link BalloonLayout#integratedEdgeLabeling integrated edge labeling}
should still work properly.
<b>Warning:</b> Calling this method may change data in the node information instance associated with node <code>root</code>.
@param root the node for which the wedge angles are calculated
@param [scaleFactor=1.0] a factor to be applied to the {@link BalloonLayoutNodeInfo#dist distance} of <code>root</code>
@returns the sum of the upper and lower wedge angle of the subtree rooted at the given root node
@see BalloonLayout#calculateChildArrangement
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23calculateAngles(YNode%2Cnumber) Online Documentation}*/
calculateAngles(root: YNode | null, scaleFactor?: number): number;
/**
Calculates a child node arrangement for a given root node of the tree.
During the arrangement, child nodes of <code>root</code> will be {@link BalloonLayout#sortChildNodes sorted}. Furthermore, distances of the child nodes will be chosen
such that the wedge of the subtree of <code>root</code> fits into the preferred wedge angle, which is either defined via {@link BalloonLayout#preferredRootWedge} or {@link BalloonLayout#preferredChildWedge}.
Calculated distances are stored in {@link BalloonLayoutNodeInfo#dist}.
The angle values - upper and lower wedge angle - may also be updated during this process and stored in {@link BalloonLayoutNodeInfo#upperAngle}
and {@link BalloonLayoutNodeInfo#lowerAngle}, respectively. To compute the angles of wedges, method {@link BalloonLayout#calculateAngles}
is used.
This method may be overridden to perform a custom child node arrangement. If support for available features like {@link BalloonLayout#interleavedMode interleaving}
should be maintained, then these features need to be carefully considered during the arrangement.
@param root the node for whose children to compute an arrangement
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23calculateChildArrangement(YNode) Online Documentation}*/
calculateChildArrangement(root: YNode | null): void;
/**
Determines the root node of {@link BalloonLayout#graph} according to the chosen {@link BalloonLayout#rootNodePolicy root node policy}.
This method may be overridden to implement some other strategy for choosing the root node of the input graph. It is
called before the rest of the layout process within {@link BalloonLayout#applyLayoutCore}.
<b>Warning:</b> If the {@link BalloonLayout#rootNodePolicy root node policy} is set to {@link RootNodePolicy#SELECTED_ROOT} and more than one node is marked as
potential root node, only one will be considered as root. If no nodes are selected, the root node will be defined
according to {@link TreeAlgorithm#getRoot}.
@returns the root node of {@link BalloonLayout#graph}
@see BalloonLayout#rootNodePolicy
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23determineRoot() Online Documentation}*/
determineRoot(): YNode | null;
/**
Returns the {@link BalloonLayoutNodeInfo} object associated with the given node while the layout algorithm is
active.
The returned object contains detailed information describing the placement of a node in the layout being computed, e.g.,
a node's distance to its parent or the wedge angle of the subtree rooted at a node.
Subclasses may want to override this method to realize another node information setup. This method is called throughout
the algorithm, each time some information associated with a node needs to be retrieved or stored.
@param node the node whose information object should be retrieved
@returns the {@link BalloonLayoutNodeInfo} instance associated to the given node
@see BalloonLayoutNodeInfo
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23getInfo(YNode) Online Documentation}*/
getInfo(node: YNode | null): BalloonLayoutNodeInfo | null;
/**
Returns the preferred radial amount (wedge) in degrees that child nodes may in total occupy around the given node.
The wedge angle controls the degree to which the child nodes may radiate from the center of layout. A value close to <code>360</code>
means that the child nodes may radiate in (almost) any direction from their parent node, edge lengths can in consequence
stay rather small. On the other hand, a small value means that children are restricted to a small angle; thus, edges
lengths (and drawings) may become large.
This method returns the {@link BalloonLayout#preferredRootWedge preferred root wedge} if node <code>root</code> was selected as global root node ({@link BalloonLayout#rootNodePolicy}). Otherwise, it either
returns {@link BalloonLayout#preferredChildWedge} or if the given node has an outdegree equal to <code>2</code>, it returns the minimum of {@link BalloonLayout#preferredChildWedge} and <code>180</code>.
This method may be overridden to provide a custom child wedge function.
@param root the node to get the preferred wedge angle for
@returns the preferred wedge angle for <code>root</code> in degrees
@see BalloonLayout#preferredChildWedge
@see BalloonLayout#preferredRootWedge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23getPreferredChildWedge(YNode) Online Documentation}*/
getPreferredChildWedge(root: YNode | null): number;
/**
Sorts the child nodes (successors) of the given node.
This implementation uses the original node coordinates if {@link BalloonLayout#fromSketchMode From Sketch mode} is enabled. Otherwise it uses {@link BalloonLayout#comparer the specified comparator}
to sort the outgoing edges and thus the children of <code>root</code>. If there is no such comparator, then the sorting depends on
whether or not the child nodes are placed in an {@link BalloonLayout#interleavedMode interleaved} fashion:
<ul>
<li>Normal: Children are sorted according to the chosen {@link BalloonLayout#childOrderingPolicy child ordering policy}.
<li>Interleaved: Children are sorted such that the resulting interleaved node placement is compact, while children inducing
larger subgraphs are placed next to smaller ones.
</ul>
This method may be overridden to realize a custom child node ordering. It gets called in method {@link BalloonLayout#calculateChildArrangement}
before coordinates are assigned and just after the wedge sizes for all subtrees rooted at <code>root</code> are determined.
@param root the node whose child nodes will be sorted
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23sortChildNodes(YNode) Online Documentation}*/
sortChildNodes(root: YNode | null): void;
/**
Data provider key for marking nodes whose child nodes should be placed in an interleaved fashion
This {@link IDataProvider} allows to individually configure the interleaving feature for each node in the
graph.
<b>Warning:</b> The custom node selection induced by this {@link IDataProvider} is only considered if the interleaved mode is
set to {@link InterleavedMode#MARKED_NODES}.
@see BalloonLayout#interleavedMode
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23INTERLEAVED_NODES_DP_KEY Online Documentation}*/
static INTERLEAVED_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for marking the node that will be used as root node of the tree
<b>Warning:</b> The custom root node selection induced by this {@link IDataProvider} is only considered if the {@link BalloonLayout#rootNodePolicy root node policy}
is set to {@link RootNodePolicy#SELECTED_ROOT}.
@see BalloonLayout#rootNodePolicy
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23SELECTED_ROOT_DP_KEY Online Documentation}*/
static SELECTED_ROOT_DP_KEY: NodeDpKey<boolean> | null;
/**
The layout graph being acted upon.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23graph Online Documentation}*/
graph: LayoutGraph | null;
/**
Gets or sets the {@link IComparer} instance that determines the order of the outgoing edges for each node of the tree.
If no {@link IComparer} is specified (i.e. the {@link IComparer} is <code>null</code>), the outgoing edges will be sorted according to the current
{@link BalloonLayout#childOrderingPolicy child ordering policy}.
<b>Warning:</b> If {@link BalloonLayout#fromSketchMode From Sketch} mode is enabled, the specified comparator does not have any effect.
@see BalloonLayout#childOrderingPolicy
@see BalloonLayout#fromSketchMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23comparer Online Documentation}*/
comparer: IComparer<any> | null;
/**
Gets or sets the child ordering policy for sorting the child nodes around their parents.
The sorting policy can affect the compactness of drawings. Advantageous orderings allow adjacent subtrees to be close
together and can thus make the whole layout more compact.
<b>Warning:</b> This policy is ignored if {@link BalloonLayout#fromSketchMode From Sketch mode} is enabled.
<b>Warning:</b> This policy is ignored for nodes whose children are placed in an {@link BalloonLayout#interleavedMode interleaved} fashion.
<b>Warning:</b> This policy is ignored if a comparator for sorting edges is set, see {@link BalloonLayout#comparer}.
@throws {Exception} <b>(ArgumentError)</b> if an unknown ordering policy is given
@see BalloonLayout#fromSketchMode
@see BalloonLayout#comparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23childOrderingPolicy Online Documentation}*/
childOrderingPolicy: ChildOrderingPolicy;
/**
Gets or sets the minimum distance to be kept between the nodes in the tree.
The distance needs to be a non-negative integer value.
<b>Warning:</b> This minimum distance is not always considered if overlaps are {@link BalloonLayout#allowOverlaps allowed}.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum distance is negative
@see BalloonLayout#allowOverlaps
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23minimumNodeDistance Online Documentation}*/
minimumNodeDistance: number;
/**
Gets or sets whether or not to consider the given coordinates of the input diagram when arranging the tree.
Enabling this feature, the original circular order of child nodes around their parent nodes will be maintained according
to the initial coordinates when determining the order of the child nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23fromSketchMode Online Documentation}*/
fromSketchMode: boolean;
/**
Gets or sets the root node selection policy of this layout algorithm.
The policy determines which node is chosen as (virtual) tree root during the layout process.
@throws {Exception} <b>(ArgumentError)</b> if an unknown root node policy is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23rootNodePolicy Online Documentation}*/
rootNodePolicy: RootNodePolicy;
/**
Gets or sets the preferred radial amount (wedge) in degrees that child nodes may in total occupy around their parent
node.
The wedge angle controls the degree to which the child nodes may radiate from the center of layout. A value close to <code>360</code>
means that the child nodes may radiate in (almost) any direction from their parent node, edge lengths can in consequence
stay rather small. On the other hand, a small value means that children are restricted to a small angle; thus, edge
lengths (and drawings) may become large.
The minimum allowed wedge angle is <code>1</code> and the maximum allowed value is <code>359</code>.
<h2>Note</h2>
The angle of the designated root node of the tree is not affected by this setting. The preferred angle of the root can
be specified using {@link BalloonLayout#preferredRootWedge}.
@throws {Exception} <b>(ArgumentError)</b> if the given angle is smaller than <code>1</code> or larger than <code>359</code>
@see BalloonLayout#preferredRootWedge
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23preferredChildWedge Online Documentation}*/
preferredChildWedge: number;
/**
Gets or sets the preferred radial amount (wedge) in degrees that child nodes may in total occupy around the global
root.
This property allows to separately control the wedge angle for the designated root node of the tree, while {@link BalloonLayout#preferredChildWedge} controls the
angles of all child nodes. The root node will be determined depending on the {@link BalloonLayout#rootNodePolicy root node policy}.
The minimum allowed root wedge angle is <code>1</code> and the maximum allowed value is <code>360</code>.
@throws {Exception} <b>(ArgumentError)</b> if the given angle is smaller than <code>1</code> or larger than <code>360</code>
@see BalloonLayout#preferredChildWedge
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23preferredRootWedge Online Documentation}*/
preferredRootWedge: number;
/**
Gets or sets whether or not (partially) overlapping nodes are allowed.
If overlaps are allowed, the resulting layouts can become significantly more compact. Overlaps will mostly occur at the
borders of nodes. Nodes will not be totally covered by other nodes.
<h2>Note</h2>
The computation of the convex hull of a node depends on this setting: the outer circle with respect to the rectangle
defined by height and width of a node is taken if overlaps ar not allowed. Otherwise, the inner circle is used for the
convex hull.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23allowOverlaps Online Documentation}*/
allowOverlaps: boolean;
/**
Gets or sets the factor defining how compact layout results will potentially be, where a smaller factor produces
potentially more compact layouts.
The algorithm tries to optimize the child node arrangement around each tree node such that each subtree is as close to
its root as possible, while still fitting into the {@link BalloonLayout#preferredChildWedge preferred wedge angle} and not overlapping with adjacent subtrees.
High compactness factor values induce the optimization procedure to be less strict and accept less optimal results,
while low factor values mean that the optimization will only stop when being nearly optimal. Thus, lower values lead to
a potentially higher runtime.
The minimum factor value is <code>0.05</code> and the maximum is <code>1.0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the factor is smaller than <code>0.05</code> or greater than <code>1.0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23compactnessFactor Online Documentation}*/
compactnessFactor: number;
/**
Gets or sets the minimum length that this layout algorithm assigns to edges of the graph.
A lower minimum edge length allows generally more compact layouts. It has the highest effect if most nodes of the graph
have a low degree, as the minimum can potentially be met for such graphs.
The minimum length must be non-negative.
<h2>Note</h2>
Really small minimum edge length values (i.e. smaller than 5) are not recommended, because edges may be barely visible
in the result drawing.
@throws {Exception} <b>(ArgumentError)</b> if the given length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23minimumEdgeLength Online Documentation}*/
minimumEdgeLength: number;
/**
Gets or sets whether or not the layout algorithm reserves space for node labels.
The size of nodes will temporarily be adjusted such that the label is included. Therefore, result drawings may get
significantly larger if this feature is enabled.
<h2>Note</h2>
This option only works correctly if the {@link MultiStageLayout#layoutOrientation layout orientation} is set to {@link LayoutOrientation#TOP_TO_BOTTOM}
(which is the default). Changing the layout orientation for an undirected layout algorithm like the {@link BalloonLayout}
doesn't make sense.
<b>Warning:</b> This has an effect only if the {@link BalloonLayout#integratedNodeLabeling integrated node labeling} mechanism is disabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets the mode for child node arrangement.
Child nodes are either placed interleaved or on a single layer around their parent node. Interleaved placement means
that child nodes are placed around their common parent in two different layers in an alternating fashion. For example,
the first child is on the inner layer, the second child on the outer layer, the third one again on the inner layer,
etc.
Independent of this mode, the alignment of child nodes on the same layer is still defined by the {@link BalloonLayout#childAlignmentPolicy alignment policy}.
However, {@link ChildAlignmentPolicy#SMART} is only supported for non-interleaved arrangement.
<b>Warning:</b> Activating interleaved child arrangement for all or for selected nodes will cause the {@link BalloonLayout#childOrderingPolicy ordering policy} to have no more
effect for those nodes.
@throws {Exception} <b>(ArgumentError)</b> if an unknown mode for interleaved arrangement is given
@see BalloonLayout#childAlignmentPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23interleavedMode Online Documentation}*/
interleavedMode: InterleavedMode;
/**
Gets or sets the child alignment policy for this layout algorithm.
This policy influences the distance of child nodes to their parent nodes and the alignment of children with the same
parent.
<h2>Note</h2>
The alignment policy can have a high influence on the result if node sizes and/or subtree sizes are differing a lot. It
also has a high influence if the graph contains labels - especially large edge labels - and if {@link BalloonLayout#integratedEdgeLabeling integrated labeling} is
enabled.
@throws {Exception} <b>(ArgumentError)</b> if an unknown policy is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23childAlignmentPolicy Online Documentation}*/
childAlignmentPolicy: ChildAlignmentPolicy;
/**
Gets or sets whether or not the layout algorithm automatically places node labels.
If enabled, this layout algorithm will calculate the positions for the node labels assuring that no overlaps occur.
Different labeling strategies may be selected using {@link BalloonLayout#nodeLabelingPolicy}.
<h2>Note</h2>
If integrated labeling is enabled, then the simple {@link BalloonLayout#considerNodeLabels consideration mechanism} has no more effect.
@see BalloonLayout#nodeLabelingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23integratedNodeLabeling Online Documentation}*/
integratedNodeLabeling: boolean;
/**
Gets or sets whether or not the layout algorithm automatically places edge labels.
If enabled, this layout algorithm will calculate the positions for the edge labels assuring that no overlaps occur.
<h2>Note</h2>
When enabling integrated edge labeling, it is recommended to also enable
{@link BalloonLayout#chainStraighteningMode chain straightening}, because edge labels (especially large ones) may lead to significantly non-straight node chains.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23integratedEdgeLabeling Online Documentation}*/
integratedEdgeLabeling: boolean;
/**
Gets or sets the policy defining how node labels are placed by the integrated node labeling mechanism (for example, the
desired label orientation).
<b>Warning:</b> The policy only has an effect if {@link BalloonLayout#integratedNodeLabeling integrated node labeling} is enabled.
@throws {Exception} <b>(ArgumentError)</b> if an unknown labeling policy is given
@see BalloonLayout#integratedNodeLabeling
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23nodeLabelingPolicy Online Documentation}*/
nodeLabelingPolicy: NodeLabelingPolicy;
/**
Gets or sets the distance between node labels belonging to the same node.
It also defines the distance between labels and the node they belong to in case of label placement outside of the node
(e.g. for {@link NodeLabelingPolicy#RAY_LIKE ray-like label placement}).
The spacing must have a non-negative value.
<b>Warning:</b> The spacing value has an effect only if {@link BalloonLayout#integratedNodeLabeling integrated node labeling} is enabled.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing value is negative
@see BalloonLayout#integratedNodeLabeling
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23nodeLabelSpacing Online Documentation}*/
nodeLabelSpacing: number;
/**
Gets or sets the distance between edge labels belonging to the same edge as well as the distance of the edge labels to
the target node of the edge.
The spacing must have a non-negative value.
<b>Warning:</b> The spacing value has an effect only if {@link BalloonLayout#integratedEdgeLabeling integrated edge labeling} is enabled.
@throws {Exception} <b>(ArgumentError)</b> if the given label spacing value is negative
@see BalloonLayout#integratedEdgeLabeling
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23edgeLabelSpacing Online Documentation}*/
edgeLabelSpacing: number;
/**
Gets or sets whether or not chains are drawn straight or not.
A chain is defined as a tree node with exactly one child node. If this feature is enabled, then the incoming edge and
outgoing edge of the chain will have the same orientation, i.e., the whole chain looks straight.
Straightening all chains can lead to smoother, more symmetric results.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23chainStraighteningMode Online Documentation}*/
chainStraighteningMode: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated.
<b>Warning:</b> {@link BalloonLayout} can only handle single components. Disabling {@link ComponentLayout} will lead to
errors during execution.
@see MultiStageLayout#componentLayoutEnabled
@see MultiStageLayout#componentLayout
@see ComponentLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23componentLayoutEnabled Online Documentation}*/
componentLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for hiding group nodes is activated.
<b>Warning:</b> {@link BalloonLayout} cannot handle group nodes. Disabling {@link HideGroupsStage} will lead to errors during
execution.
@see MultiStageLayout#hideGroupsStageEnabled
@see MultiStageLayout#hideGroupsStage
@see HideGroupsStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23hideGroupsStageEnabled Online Documentation}*/
hideGroupsStageEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated.
<h2>Note</h2>
If the current layout orientation is
{@link LayoutOrientation#TOP_TO_BOTTOM}, the orientation of the layout will not be modified.
<b>Warning:</b> Based on the undirected layout style, disabling {@link OrientationLayout} has no significant effect.
@see MultiStageLayout#orientationLayoutEnabled
@see MultiStageLayout#orientationLayout
@see MultiStageLayout#layoutOrientation
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23orientationLayoutEnabled Online Documentation}*/
orientationLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing parallel edges is activated.
<b>Warning:</b> {@link BalloonLayout} cannot handle parallel edges. Disabling {@link ParallelEdgeRouter} will lead to errors
during execution.
@see MultiStageLayout#parallelEdgeRouterEnabled
@see MultiStageLayout#parallelEdgeRouter
@see ParallelEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23parallelEdgeRouterEnabled Online Documentation}*/
parallelEdgeRouterEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing self-loops is activated.
<b>Warning:</b> {@link BalloonLayout} cannot handle self-loops. Disabling {@link SelfLoopRouter} will lead to errors during
execution.
@see MultiStageLayout#selfLoopRouterEnabled
@see MultiStageLayout#selfLoopRouter
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayout%23selfLoopRouterEnabled Online Documentation}*/
selfLoopRouterEnabled: boolean;
static $class: Class;
private $_BalloonLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BalloonLayout;
}
export interface FamilyTreeLayout extends MultiStageLayout {
}
/**
This layout algorithm arranges genealogical graphs (family trees).
<b>Layout Style</b>
The family trees consist of nodes that represent individuals and nodes that represent families. Individuals are
connected via these family nodes. The graph is arranged in layers that describe generations.
<b>Concept</b>
In order to determine whether a node represents an individual or a partnership, a {@link IDataProvider} has to
be registered with key {@link FamilyTreeLayout#FAMILY_TYPE_DP_KEY}. If this {@link IDataProvider} is
missing, an {@link Exception ArgumentError} will be thrown.
This {@link IDataProvider} provides:
<ul>
<li>{@link FamilyType#MALE} for a node representing a male individual.
<li>{@link FamilyType#FEMALE} for a node representing a female individual.
<li>{@link FamilyType#FAMILY} for a node representing a family.
</ul>
All other values will be interpreted as if the node represents an individual.
A {@link FamilyType#FAMILY} node links all members of a family. It can only have two incoming edges from the
parents. Two nodes of the same type (families or individuals) cannot be directly connected. If the input graph violates
these specifications, a {@link Exception InvalidGraphStructureError} exception will be thrown.
The layout is calculated basically in two steps:
<ul>
<li>The families are laid out by the inner layouter in a compact way.
<li>The relations between these "family groups" and their children and other families are laid out by the
{@link FamilyTreeLayout#topLayout top layout algorithm}.
</ul>
<b>Features</b>
The distances between nodes {@link FamilyTreeLayout#spacingBetweenFamilyMembers in the same generation} as well as between {@link FamilyTreeLayout#offsetForFamilyNodes individuals and family nodes} can be adjusted.
Nodes in the same generation can be {@link FamilyTreeLayout#alignment aligned} at three different coordinates: top, center, bottom.
The location of {@link FamilyType#FAMILY} nodes can be {@link FamilyTreeLayout#familyNodesAlwaysBelow specified} in relation to the partners. This placement
can be set independently for {@link FamilyTreeLayout#partnerlessBelow parents}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout Online Documentation}*/
export class FamilyTreeLayout {
/**
Creates a new instance of {@link FamilyTreeLayout} with default settings.
@param [options] The parameters to pass.
@param [options.topLayout] The {@link ILayoutAlgorithm layout algorithm} that is responsible for the layout between family groups. This option sets the {@link FamilyTreeLayout#topLayout} property on the created object.
@param [options.spacingBetweenFamilyMembers] The spacing between two nodes representing members of the same family in one generation. This option sets the {@link FamilyTreeLayout#spacingBetweenFamilyMembers} property on the created object.
@param [options.offsetForFamilyNodes] The offset by which a {@link FamilyType#FAMILY} node is shifted down if its parents are not direct neighbors. This option sets the {@link FamilyTreeLayout#offsetForFamilyNodes} property on the created object.
@param [options.partnerlessBelow] Whether or not {@link FamilyType#FAMILY} nodes of individuals that have only children but no partners are placed below this individual or next to it. This option sets the {@link FamilyTreeLayout#partnerlessBelow} property on the created object.
@param [options.familyNodesAlwaysBelow] Whether or not {@link FamilyType#FAMILY} nodes are placed between or below the partners. This option sets the {@link FamilyTreeLayout#familyNodesAlwaysBelow} property on the created object.
@param [options.alignment] The vertical alignment of the individual nodes in the same generation. This option sets the {@link FamilyTreeLayout#alignment} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link FamilyTreeLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link FamilyTreeLayout#selfLoopRouterEnabled} property on the created object.
@param [options.familyMembersSortingPolicy] The order of individuals in the same generation (siblings, partners) based on their sex. This option sets the {@link FamilyTreeLayout#familyMembersSortingPolicy} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link MultiStageLayout#orientationLayoutEnabled} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link MultiStageLayout#componentLayoutEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout Online Documentation}*/
constructor(options?: {
topLayout?: ILayoutAlgorithm | null;
spacingBetweenFamilyMembers?: number;
offsetForFamilyNodes?: number;
partnerlessBelow?: boolean;
familyNodesAlwaysBelow?: boolean;
alignment?: VerticalNodeAlignment | VerticalNodeAlignmentStringValues;
parallelEdgeRouterEnabled?: boolean;
selfLoopRouterEnabled?: boolean;
familyMembersSortingPolicy?: FamilyMembersSortingPolicy | FamilyMembersSortingPolicyStringValues;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
orientationLayoutEnabled?: boolean;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
componentLayoutEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Applies a family tree layout to the given graph.
@param graph the input graph
@throws {Exception} <b>(InvalidOperationError)</b> if no {@link IDataProvider} is registered using {@link FamilyTreeLayout#FAMILY_TYPE_DP_KEY}
@throws {Exception} <b>(InvalidGraphStructureError)</b> if a {@link FamilyType#FAMILY} node has more than two parents or two nodes of the same type are linked
together
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Data provider key for assigning a node type for each node
The layout algorithm will handle the nodes according to this type.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23FAMILY_TYPE_DP_KEY Online Documentation}*/
static FAMILY_TYPE_DP_KEY: NodeDpKey<FamilyType> | null;
/**
Gets or sets the {@link ILayoutAlgorithm layout algorithm} that is responsible for the layout between family groups.
If the specified layout algorithm is <code>null</code>, fallback algorithm {@link HierarchicLayout} is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23topLayout Online Documentation}*/
topLayout: ILayoutAlgorithm | null;
/**
Gets or sets the spacing between two nodes representing members of the same family in one generation.
Values should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23spacingBetweenFamilyMembers Online Documentation}*/
spacingBetweenFamilyMembers: number;
/**
Gets or sets the offset by which a {@link FamilyType#FAMILY} node is shifted down if its parents are not direct
neighbors.
Values should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given offset is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23offsetForFamilyNodes Online Documentation}*/
offsetForFamilyNodes: number;
/**
Gets or sets whether or not {@link FamilyType#FAMILY} nodes of individuals that have only children but no
partners are placed below this individual or next to it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23partnerlessBelow Online Documentation}*/
partnerlessBelow: boolean;
/**
Gets or sets whether or not {@link FamilyType#FAMILY} nodes are placed between or below the partners.
@see FamilyType#FAMILY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23familyNodesAlwaysBelow Online Documentation}*/
familyNodesAlwaysBelow: boolean;
/**
Gets or sets the vertical alignment of the individual nodes in the same generation.
@throws {Exception} <b>(ArgumentError)</b> if the specified alignment is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23alignment Online Documentation}*/
alignment: VerticalNodeAlignment;
/**
Sets whether or not the {@link ILayoutStage} used for routing parallel edges is activated.
<b>Warning:</b> {@link FamilyTreeLayout} cannot handle parallel edges. Disabling {@link ParallelEdgeRouter} will lead to
errors during execution.
@see MultiStageLayout#parallelEdgeRouterEnabled
@see MultiStageLayout#parallelEdgeRouter
@see ParallelEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23parallelEdgeRouterEnabled Online Documentation}*/
parallelEdgeRouterEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing self-loops is activated.
<b>Warning:</b> {@link FamilyTreeLayout} cannot handle self-loops. Disabling {@link SelfLoopRouter} will lead to errors
during execution.
@see MultiStageLayout#selfLoopRouterEnabled
@see MultiStageLayout#selfLoopRouter
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23selfLoopRouterEnabled Online Documentation}*/
selfLoopRouterEnabled: boolean;
/**
Gets or sets the order of individuals in the same generation (siblings, partners) based on their sex.
@throws {Exception} <b>(ArgumentError)</b> if the specified order is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayout%23familyMembersSortingPolicy Online Documentation}*/
familyMembersSortingPolicy: FamilyMembersSortingPolicy;
static $class: Class;
private $_FamilyTreeLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FamilyTreeLayout;
}
/**
Enum constants that specify the node types for the {@link FamilyTreeLayout}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyType Online Documentation}*/
export enum FamilyType {
/**
Type specifier for nodes that connect partners with each other and their children.
@see FamilyTreeLayout#FAMILY_TYPE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyType%23FAMILY Online Documentation}*/
FAMILY,
/**
Type specifier for nodes that represent men.
@see FamilyTreeLayout#FAMILY_TYPE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyType%23MALE Online Documentation}*/
MALE,
/**
Type specifier for nodes that represent women.
@see FamilyTreeLayout#FAMILY_TYPE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyType%23FEMALE Online Documentation}*/
FEMALE
}
/** Automatic conversion to {@link FamilyType}. */
type FamilyTypeStringValues = "family" | "male" | "female";
/**
Alignment specifier that defines that all nodes in the same layer (generation) are aligned based on their top
coordinate.
@see FamilyTreeLayout#alignment
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/VerticalNodeAlignment Online Documentation}*/
export enum VerticalNodeAlignment {
/**
Alignment specifier that defines that all nodes in the same layer (generation) are aligned based on their top
coordinate.
The descriptions of the directions are based on {@link LayoutOrientation#TOP_TO_BOTTOM}.
@see FamilyTreeLayout#alignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/VerticalNodeAlignment%23TOP Online Documentation}*/
TOP,
/**
Alignment specifier that defines that all nodes in the same layer (generation) are aligned based on their center
coordinate.
The descriptions of the directions are based on {@link LayoutOrientation#TOP_TO_BOTTOM}.
@see FamilyTreeLayout#alignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/VerticalNodeAlignment%23CENTER Online Documentation}*/
CENTER,
/**
Alignment specifier that defines that all nodes in the same layer (generation) are aligned based on their bottom
coordinate.
The descriptions of the directions are based on {@link LayoutOrientation#TOP_TO_BOTTOM}.
@see FamilyTreeLayout#alignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/VerticalNodeAlignment%23BOTTOM Online Documentation}*/
BOTTOM
}
/** Automatic conversion to {@link VerticalNodeAlignment}. */
type VerticalNodeAlignmentStringValues = "top" | "center" | "bottom";
/**
Enum constants that specify how family members are sorted by the {@link FamilyTreeLayout}.
@see FamilyTreeLayout#familyMembersSortingPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyMembersSortingPolicy Online Documentation}*/
export enum FamilyMembersSortingPolicy {
/**
Order specifier that disables sorting of individuals according to their sex.
@see FamilyTreeLayout#familyMembersSortingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyMembersSortingPolicy%23NO_SORTING Online Documentation}*/
NO_SORTING,
/**
Order specifier that places female individuals before their male siblings and partners, if possible.
@see FamilyTreeLayout#familyMembersSortingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyMembersSortingPolicy%23FEMALE_FIRST Online Documentation}*/
FEMALE_FIRST,
/**
Order specifier that places female individuals before their male siblings and partners.
@see FamilyTreeLayout#familyMembersSortingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyMembersSortingPolicy%23FEMALE_ALWAYS_FIRST Online Documentation}*/
FEMALE_ALWAYS_FIRST,
/**
Order specifier that places male individuals before their female siblings and partners, if possible.
@see FamilyTreeLayout#familyMembersSortingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyMembersSortingPolicy%23MALE_FIRST Online Documentation}*/
MALE_FIRST,
/**
Order specifier that places male individuals before their female siblings and partners.
@see FamilyTreeLayout#familyMembersSortingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyMembersSortingPolicy%23MALE_ALWAYS_FIRST Online Documentation}*/
MALE_ALWAYS_FIRST
}
/** Automatic conversion to {@link FamilyMembersSortingPolicy}. */
type FamilyMembersSortingPolicyStringValues = "no-sorting" | "female-first" | "female-always-first" | "male-first" | "male-always-first";
export interface AsIsLayerer extends YObject, ILayerer {
}
/**
{@link AsIsLayerer} assigns nodes to layers by analyzing already existing node coordinates.
Nodes whose bounding boxes overlap in the main layout direction are assigned to the same layer. To influence to which
amount the boxes need to overlap, the nodes can temporarily be
{@link AsIsLayerer#nodeScalingFactor scaled}. Alternatively, a minimum and maximum size or a halo can be specified for the nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsLayerer Online Documentation}*/
export class AsIsLayerer {
/**
Creates an instance of {@link AsIsLayerer} with default settings.
@param [options] The parameters to pass.
@param [options.nodeScalingFactor] The scaling factor used for scaling the size of the nodes. This option sets the {@link AsIsLayerer#nodeScalingFactor} property on the created object.
@param [options.maximumNodeSize] The maximum size of a node used for calculating layer overlaps. This option sets the {@link AsIsLayerer#maximumNodeSize} property on the created object.
@param [options.minimumNodeSize] The minimum size of a node used for calculating layer overlaps. This option sets the {@link AsIsLayerer#minimumNodeSize} property on the created object.
@param [options.nodeHalo] The size of the halo around a node, that means the insets that are used for calculating layer overlaps. This option sets the {@link AsIsLayerer#nodeHalo} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsLayerer Online Documentation}*/
constructor(options?: {
nodeScalingFactor?: number;
maximumNodeSize?: number;
minimumNodeSize?: number;
nodeHalo?: number;
});
/**
Assigns all nodes of the graph to layers by analyzing already existing node coordinates and adds them to the {@link ILayers}
instance.
Nodes whose bounding boxes overlap in the main layout direction are assigned to the same layer. To influence to which
amount the boxes need to overlap, the nodes can temporarily be {@link AsIsLayerer#nodeScalingFactor scaled}.
@param graph the input graph
@param layers the {@link ILayers} instance that will be filled with the results of the calculation
@param ldp the {@link ILayoutDataProvider} used for querying information about the nodes and edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsLayerer%23assignLayers(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
assignLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
Callback used for calculating the lower (max) value of a given node.
@param graph the graph
@param node the node
@returns the lower (max) of a given node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsLayerer%23getMax(LayoutGraph%2CYNode) Online Documentation}*/
getMax(graph: LayoutGraph | null, node: YNode | null): number;
/**
Callback used for calculating the upper (min) value of a given node.
@param graph the graph
@param node the node
@returns the upper (min) value of a given node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsLayerer%23getMin(LayoutGraph%2CYNode) Online Documentation}*/
getMin(graph: LayoutGraph | null, node: YNode | null): number;
/**
Gets or sets the scaling factor used for scaling the size of the nodes.
Nodes are being scaled from their center. This can be used for more fuzzy layering.
The scaling factor can take positive values. Factors smaller than <code>1</code> only consider large overlaps of the nodes' bounding
boxes in the main layout direction. Factors larger than <code>1</code> also combine nodes with some distance between them.
@throws {Exception} <b>(ArgumentError)</b> if the specified value is less than <code>0</code>
@see AsIsLayerer#minimumNodeSize
@see AsIsLayerer#maximumNodeSize
@see AsIsLayerer#nodeHalo
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsLayerer%23nodeScalingFactor Online Documentation}*/
nodeScalingFactor: number;
/**
Gets or sets the maximum size of a node used for calculating layer overlaps.
This can be used for more fuzzy layering.
The maximum size can take positive values.
@throws {Exception} <b>(ArgumentError)</b> if the specified value is less than <code>0</code>
@see AsIsLayerer#nodeScalingFactor
@see AsIsLayerer#nodeHalo
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsLayerer%23maximumNodeSize Online Documentation}*/
maximumNodeSize: number;
/**
Gets or sets the minimum size of a node used for calculating layer overlaps.
This can be used for more fuzzy layering.
The minimum size can take positive values.
@throws {Exception} <b>(ArgumentError)</b> if the specified value is less than <code>0</code>
@see AsIsLayerer#nodeScalingFactor
@see AsIsLayerer#nodeHalo
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsLayerer%23minimumNodeSize Online Documentation}*/
minimumNodeSize: number;
/**
Gets or sets the size of the halo around a node, that means the insets that are used for calculating layer overlaps.
If the value of the halo is positive, it specifies a border around a node. In case it is negative, it is considered as
an inset that defines a smaller area inside the node.
This can be used for more fuzzy layering.
@see AsIsLayerer#nodeScalingFactor
@see AsIsLayerer#minimumNodeSize
@see AsIsLayerer#maximumNodeSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsLayerer%23nodeHalo Online Documentation}*/
nodeHalo: number;
static $class: Class;
private $_AsIsLayerer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AsIsLayerer;
}
export interface AsIsSequencer extends YObject, ISequencer {
}
/**
This class calculates the sequence of the nodes within each layer using the nodes' coordinates.
For {@link MultiStageLayout#layoutOrientation layout orientation}
{@link LayoutOrientation#TOP_TO_BOTTOM}, the order corresponds to that obtain when sorting the nodes according to
their x-coordinate.
An instance of this class can be set to {@link HierarchicLayout} using method {@link HierarchicLayout#fixedElementsSequencer} and {@link HierarchicLayout#fromScratchSequencer} respectively.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsSequencer Online Documentation}*/
export class AsIsSequencer {
/**
Creates a new {@link AsIsSequencer} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsSequencer Online Documentation}*/
constructor();
/**
Calculates the sequence of the nodes within each layer using the nodes' coordinates.
This method is called by {@link HierarchicLayoutCore} and finally writes back the calculated sequence using
method {@link ILayer#setNodeOrder}.
@param graph the input graph
@param layers the given {@link ILayers} instance that defines the layering structure
@param ldp the {@link ILayoutDataProvider} containing information about the nodes and edges of the graph
@param itemFactory the {@link IItemFactory} used for creating and destroying helper structures
@see {@link https://docs.yworks.com/yfileshtml/#/api/AsIsSequencer%23sequenceNodeLayers(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
sequenceNodeLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
static $class: Class;
private $_AsIsSequencer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AsIsSequencer;
}
export interface AspectRatioComponentLayerer extends YObject, ILayerer {
}
/**
This class is a wrapper {@link ILayerer} implementation that delegates the actual layering to a delegate {@link ILayerer}
object.
If the graph consists of multiple components, they will be identified and each of the components will be layered
separately using the delegate {@link ILayerer} instance. After that they will all be merged such that the
desired aspect ratio is fulfilled best.
<h2>Note</h2>
For grouped graphs, the desired aspect ratio is also considered for each group node separately.
@see AspectRatioComponentLayerer#desiredAspectRatio
@see MultiComponentLayerer
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioComponentLayerer Online Documentation}*/
export class AspectRatioComponentLayerer {
/**
Creates a new instance of {@link AspectRatioComponentLayerer} using the given delegate {@link ILayerer}
instance.
@param singleComponentLayerer
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link ILayerer} is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioComponentLayerer Online Documentation}*/
constructor(singleComponentLayerer: ILayerer | null);
/**
Creates a new instance of {@link AspectRatioComponentLayerer} using the given delegate {@link ILayerer}
instance.
@param options The parameters to pass.
@param options.singleComponentLayerer
@param [options.considerNodeSize] Whether or not the size of the nodes should be considered. This option sets the {@link AspectRatioComponentLayerer#considerNodeSize} property on the created object.
@param [options.desiredAspectRatio] The desired aspect ratio. This option sets the {@link AspectRatioComponentLayerer#desiredAspectRatio} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link ILayerer} is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioComponentLayerer Online Documentation}*/
constructor(options: {
singleComponentLayerer: ILayerer | null;
considerNodeSize?: boolean;
desiredAspectRatio?: number;
});
/**
Assigns all nodes of the graph to layers and registers them to the {@link ILayers} instance based on the given
aspect ratio.
If the graph consists of multiple components, they will be identified and each of the components will be layered
separately using the delegate {@link ILayerer} instance. After that, they will all be merged such that the
desired aspect ratio is best fulfilled.
<h2>Note</h2>
For grouped graphs, the desired aspect ratio is also considered for each group node separately.
@param graph the input graph
@param layers the {@link ILayers} instance that will be filled with the results of the calculation
@param ldp the {@link ILayoutDataProvider} used for query information about the nodes and edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioComponentLayerer%23assignLayers(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
assignLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
Gets or sets the delegate {@link ILayerer} instance.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link ILayerer} is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioComponentLayerer%23singleComponentLayerer Online Documentation}*/
singleComponentLayerer: ILayerer | null;
/**
Gets or sets whether or not the size of the nodes should be considered.
If this option is disabled, all nodes are considered to be of equal size. Hence, the given aspect ratio specifies the
ratio between the number of nodes within a layer and the overall number of layers.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioComponentLayerer%23considerNodeSize Online Documentation}*/
considerNodeSize: boolean;
/**
Gets or sets the desired aspect ratio.
If the graph consists of multiple components, they will be identified and each of the components will be layered
separately using the delegate {@link ILayerer} instance. After that, they will all be merged such that the
desired aspect ratio is best fulfilled.
The given aspect ratio should be greater or equal to <code>0</code>.
<h2>Note</h2>
For grouped graphs, the desired aspect ratio is also considered for each group node separately.
@throws {Exception} <b>(ArgumentError)</b> if the aspect ratio is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioComponentLayerer%23desiredAspectRatio Online Documentation}*/
desiredAspectRatio: number;
static $class: Class;
private $_AspectRatioComponentLayerer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AspectRatioComponentLayerer;
}
export interface BFSLayerer extends YObject, ILayerer {
}
/**
{@link BFSLayerer} uses a breadth first search for assigning layers to the nodes.
The nodes of the first layer can be freely selected (see
{@link BFSLayerer#CORE_NODES_DP_KEY}). The nodes that belong to a subsequent layer are determined as follows:
Add all yet unassigned nodes to the new layer that are connected to nodes already assigned. As a consequence, all
connected nodes will be at most one layer apart. Also, edges between nodes that belong to the same layer are possible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BFSLayerer Online Documentation}*/
export class BFSLayerer {
/**
Creates an instance of {@link BFSLayerer}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BFSLayerer Online Documentation}*/
constructor();
/**
Assigns all nodes of the graph to layers and adds them to the {@link ILayers} instance.
@param graph the input graph
@param layers the {@link ILayers} instance that will be filled with the results of the calculation
@param ldp the {@link ILayoutDataProvider} used for query information about the nodes and edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/BFSLayerer%23assignLayers(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
assignLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
Data provider key for identifying the core nodes of the BFS layering
The {@link IDataProvider} registered with this key should provide boolean values for the nodes of the graph.
The boolean value determines whether or not a node is to be placed in the first layer. If there are no nodes marked as
core nodes, then nodes with in-degree <code>0</code> are considered to be the core nodes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BFSLayerer%23CORE_NODES_DP_KEY Online Documentation}*/
static CORE_NODES_DP_KEY: NodeDpKey<boolean> | null;
static $class: Class;
private $_BFSLayerer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BFSLayerer;
}
export interface HierarchicLayoutBusDescriptor extends YObject {
}
/**
A {@link HierarchicLayoutBusDescriptor} defines settings for edges routed on a common bus and forming a bus substructure for
{@link HierarchicLayout}.
Edges mapped to the same {@link HierarchicLayoutBusDescriptor} instance, adjacent to a common root node and having the same
edge direction form a bus. The mapping is retrieved from a {@link IDataProvider} registered with the input
graph with key {@link HierarchicLayout#BUS_DESCRIPTOR_DP_KEY}.
@see HierarchicLayout#BUS_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutBusDescriptor Online Documentation}*/
export class HierarchicLayoutBusDescriptor {
/**
Creates a new instance of a {@link HierarchicLayoutBusDescriptor} with the default values.
@param [options] The parameters to pass.
@param [options.maximumNodesAfterBus] The maximum number of nodes in a layer that are placed after the common bus segment. This option sets the {@link HierarchicLayoutBusDescriptor#maximumNodesAfterBus} property on the created object.
@param [options.maximumNodesBeforeBus] The maximum number of nodes in a layer that are placed before the common bus segment. This option sets the {@link HierarchicLayoutBusDescriptor#maximumNodesBeforeBus} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutBusDescriptor Online Documentation}*/
constructor(options?: {
maximumNodesAfterBus?: number;
maximumNodesBeforeBus?: number;
});
/**
Data provider key for assigning nodes to specific layers relative to the root node of the bus
An offset value <code>i</code> specified for a bus node is interpreted relative to the root node as follows, assuming that the root
node is in layer <code>x</code>.
<ul>
<li>If the bus is placed in the layers after the root, the bus node will be placed in layer <code>x + i</code>
<li>If the bus is placed in the layers before the root, the bus node will be placed in layer <code>x - i</code>
</ul>
<b>Warning:</b> The specified {@link HierarchicLayoutBusDescriptor#maximumNodesBeforeBus number of nodes before the bus} and {@link HierarchicLayoutBusDescriptor#maximumNodesAfterBus number of nodes after the bus} might be violated if the layer indices
provided here are such that e.g., a layer contains too many nodes to fulfill the node count constraints.
<h2>Note</h2>
The provided layer offsets are normalized such that it is not possible to define empty bus layers (e.g. offset values <code>2</code>,
<code>10</code>, <code>15</code> are normalized to <code>1</code>, <code>2</code>, <code>3</code>).
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutBusDescriptor%23ROOT_OFFSET_DP_KEY Online Documentation}*/
static ROOT_OFFSET_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for assigning nodes to a specific side of the common bus
If no {@link IDataProvider} is specified with this key, the nodes will be placed in an alternating way.
<b>Warning:</b> The side assignment induced by this setting is ignored if there are any {@link ISequenceConstraintFactory sequence constraints} between the bus nodes.
<b>Warning:</b> It can happen that the specified maximum number of nodes {@link HierarchicLayoutBusDescriptor#maximumNodesBeforeBus before the bus} and {@link HierarchicLayoutBusDescriptor#maximumNodesAfterBus after the bus} is exceeded due to these side
placement constraints.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutBusDescriptor%23PLACE_BEFORE_BUS_DP_KEY Online Documentation}*/
static PLACE_BEFORE_BUS_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the maximum number of nodes in a layer that are placed before the common bus segment.
This setting allows to individually specify how the bus nodes should be distributed before/after the common bus segment
with respect to sequence of each layer. By default, both the {@link HierarchicLayoutBusDescriptor#maximumNodesBeforeBus} and the {@link HierarchicLayoutBusDescriptor#maximumNodesAfterBus} are <code>0</code> which means that the nodes are equally
distributed and the common bus segment is placed in the middle. The actual number of nodes before and after the bus is
determined automatically in that case.
<b>Warning:</b> If nodes are specifically assigned to one side using {@link HierarchicLayoutBusDescriptor#PLACE_BEFORE_BUS_DP_KEY}, but the
value of this property is set to <code>0</code>, then the maximum will be exceeded as necessary. The maximum can also be exceeded if
there are too many fixed bus nodes in the same layer (when running in {@link LayoutMode#INCREMENTAL incremental mode}) or when bus layers are
user-specified via {@link HierarchicLayoutBusDescriptor#ROOT_OFFSET_DP_KEY}.
@throws {Exception} <b>(ArgumentError)</b> if the given node count is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutBusDescriptor%23maximumNodesBeforeBus Online Documentation}*/
maximumNodesBeforeBus: number;
/**
Gets or sets the maximum number of nodes in a layer that are placed after the common bus segment.
This setting allows to individually specify how the bus nodes should be distributed before/after the common bus segment
with respect to the sequence of each layer. By default, both the {@link HierarchicLayoutBusDescriptor#maximumNodesBeforeBus} and the {@link HierarchicLayoutBusDescriptor#maximumNodesAfterBus} are <code>0</code> which means that the nodes are equally
distributed and the common bus segment is placed in the middle. The actual number of nodes before and after the bus is
determined automatically in that case.
<b>Warning:</b> If nodes are specifically assigned to one side using {@link HierarchicLayoutBusDescriptor#PLACE_BEFORE_BUS_DP_KEY}, but the
value of this property is set to <code>0</code>, then the maximum will be exceeded as necessary. The maximum can also be exceeded if
there are too many fixed bus nodes in the same layer (when running in {@link LayoutMode#INCREMENTAL incremental mode}) or when bus layers are
user-specified via {@link HierarchicLayoutBusDescriptor#ROOT_OFFSET_DP_KEY}.
@throws {Exception} <b>(ArgumentError)</b> if the given node count is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutBusDescriptor%23maximumNodesAfterBus Online Documentation}*/
maximumNodesAfterBus: number;
static $class: Class;
private $_BusDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HierarchicLayoutBusDescriptor;
}
export interface ConstraintIncrementalLayerer extends YObject, ILayerer {
}
/**
This class is a {@link ILayerer} implementation that observes relative and absolute layering constraints
defined by the {@link ILayerConstraintFactory layer constraint factory}.
Furthermore, it can handle the edge directedness as specified by {@link IDataProvider} key
{@link HierarchicLayout#EDGE_DIRECTEDNESS_DP_KEY}.
For a given graph instance, the layer constraint factory can be created using method
{@link HierarchicLayout#createLayerConstraintFactory}. The {@link ILayerer} can be used for both
common layering and layering of incremental nodes.
<h2>Note</h2>
This {@link ILayerer} is always applied automatically if the graph instance has constraints created with the
{@link ILayerConstraintFactory layering constraint factory}.
@see ILayerConstraintFactory
@see HierarchicLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstraintIncrementalLayerer Online Documentation}*/
export class ConstraintIncrementalLayerer {
/**
Creates a new {@link ConstraintIncrementalLayerer} with the given {@link ILayerer} instance.
@param coreLayerer the given {@link ILayerer} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstraintIncrementalLayerer Online Documentation}*/
constructor(coreLayerer: ILayerer | null);
/**
Creates a new {@link ConstraintIncrementalLayerer} with the given {@link ILayerer} instance.
@param options The parameters to pass.
@param options.coreLayerer the given {@link ILayerer} instance
@param [options.allowSameLayerEdges] Whether or not same-layer edges can be created by this {@link ILayerer} instance. This option sets the {@link ConstraintIncrementalLayerer#allowSameLayerEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstraintIncrementalLayerer Online Documentation}*/
constructor(options: {
coreLayerer: ILayerer | null;
allowSameLayerEdges?: boolean;
});
/**
Assigns all nodes of the graph to layers and adds them to the {@link ILayers} instance based on relative and
absolute layering constraints defined by the given layering constraints.
<h2>Note</h2>
A {@link ILayers} structure will be filled by the {@link ILayerer}. If the {@link ILayerer}
instance is used for layering incremental nodes, the {@link ILayers} structure already has to contain all
non-incremental nodes.
@param graph the input graph
@param layers the {@link ILayers} instance that will be filled with the results of the calculation
@param ldp the {@link ILayoutDataProvider} used for query information about the nodes and edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstraintIncrementalLayerer%23assignLayers(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
assignLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
Checks if the current setting of constraints is consistent, that is, the induced constraint graph has no cycles.
@throws {Exception} <b>(ArgumentError)</b> if the constraint graph is inconsistent
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstraintIncrementalLayerer%23checkConstraints() Online Documentation}*/
checkConstraints(): void;
/**
Data provider key for assigning additional edge weights
The {@link ILayerer} tries to maintain edges with higher weights short.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstraintIncrementalLayerer%23ADDITIONAL_EDGE_WEIGHT_DP_KEY Online Documentation}*/
static ADDITIONAL_EDGE_WEIGHT_DP_KEY: EdgeDpKey<number> | null;
/**
Gets or sets whether or not same-layer edges can be created by this {@link ILayerer} instance.
This only concerns edges between nodes that have no hard constraints that will force a same-layer edge (i.e. a
same-layer constraint).
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConstraintIncrementalLayerer%23allowSameLayerEdges Online Documentation}*/
allowSameLayerEdges: boolean;
static $class: Class;
private $_ConstraintIncrementalLayerer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ConstraintIncrementalLayerer;
}
export interface DefaultDrawingDistanceCalculator extends YObject, IDrawingDistanceCalculator {
/**
@inheritdoc*/
dispose(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
@inheritdoc*/
initialize(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
} /**
This class is a default {@link IDrawingDistanceCalculator} implementation that is used by {@link HierarchicLayoutCore}
to configure the {@link IHierarchicLayoutNodePlacer}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultDrawingDistanceCalculator Online Documentation}*/
export class DefaultDrawingDistanceCalculator {
/**
Creates a new instance of {@link DefaultDrawingDistanceCalculator} with default settings.
@param [options] The parameters to pass.
@param [options.adaptiveMinimumEdgeDistanceEnabled] Whether or not an adaptive minimum edge distance is used. This option sets the {@link DefaultDrawingDistanceCalculator#adaptiveMinimumEdgeDistanceEnabled} property on the created object.
@param [options.compaction] Whether or not the compaction option is enabled. This option sets the {@link DefaultDrawingDistanceCalculator#compaction} property on the created object.
@param [options.nodeToNodeDistance] The minimum distance between two {@link YNode}s. This option sets the {@link DefaultDrawingDistanceCalculator#nodeToNodeDistance} property on the created object.
@param [options.nodeToEdgeDistance] The minimum distance between a {@link YNode} and an (non-adjacent) {@link Edge}. This option sets the {@link DefaultDrawingDistanceCalculator#nodeToEdgeDistance} property on the created object.
@param [options.edgeToEdgeDistance] The minimum distance between two edges. This option sets the {@link DefaultDrawingDistanceCalculator#edgeToEdgeDistance} property on the created object.
@param [options.optimizeSwimlaneDistances] Whether or not to use an optimized minimum distance calculation for swimlane layouts. This option sets the {@link DefaultDrawingDistanceCalculator#optimizeSwimlaneDistances} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultDrawingDistanceCalculator Online Documentation}*/
constructor(options?: {
adaptiveMinimumEdgeDistanceEnabled?: boolean;
compaction?: boolean;
nodeToNodeDistance?: number;
nodeToEdgeDistance?: number;
edgeToEdgeDistance?: number;
optimizeSwimlaneDistances?: boolean;
});
/**
Determines the minimum distance between two {@link YNode}s of the same layer.
@param graph the input graph
@param layer the {@link ILayer} object that will be used during subsequent calls
@param ldp the {@link ILayoutDataProvider} used for querying information
@param left the left of the two {@link YNode}s whose minimum distance is to be determined or <code>null</code> if only the left border
of the right {@link YNode} is of interest
@param right the right of the two {@link YNode}s whose minimum distance is to be determined or <code>null</code> if only the right border
of the left {@link YNode} is of interest
@returns the minimum distances between {@link YNode}s of same layer
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultDrawingDistanceCalculator%23getMinDistance(LayoutGraph%2CILayer%2CILayoutDataProvider%2CYNode%2CYNode) Online Documentation}*/
getMinDistance(graph: LayoutGraph | null, layer: ILayer | null, ldp: ILayoutDataProvider | null, left: YNode | null, right: YNode | null): number;
/**
Gets or sets whether or not an adaptive minimum edge distance is used.
If this option is enabled the minimum distance between two adjacent edges is the minimum of the current minimum edge
distance and the distance of the edges' source/target points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultDrawingDistanceCalculator%23adaptiveMinimumEdgeDistanceEnabled Online Documentation}*/
adaptiveMinimumEdgeDistanceEnabled: boolean;
/**
Gets or sets whether or not the compaction option is enabled.
<h2>Note</h2>
This option is automatically set by the layout algorithm if option {@link SimplexNodePlacer#nodeCompaction} or {@link SimplexNodePlacer#labelCompaction} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultDrawingDistanceCalculator%23compaction Online Documentation}*/
compaction: boolean;
/**
Gets or sets the minimum distance between two {@link YNode}s.
Minimum distance should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultDrawingDistanceCalculator%23nodeToNodeDistance Online Documentation}*/
nodeToNodeDistance: number;
/**
Gets or sets the minimum distance between a {@link YNode} and an (non-adjacent) {@link Edge}.
Minimum distance should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultDrawingDistanceCalculator%23nodeToEdgeDistance Online Documentation}*/
nodeToEdgeDistance: number;
/**
Gets or sets the minimum distance between two edges.
Minimum distance should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultDrawingDistanceCalculator%23edgeToEdgeDistance Online Documentation}*/
edgeToEdgeDistance: number;
/**
Gets or sets whether or not to use an optimized minimum distance calculation for swimlane layouts.
If set to <code>true</code> this instance will report <code>0.0d</code> as the minimum distance between two nodes if they belong to different
swimlanes. This avoids unwanted feedback between different swimlanes during node placement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultDrawingDistanceCalculator%23optimizeSwimlaneDistances Online Documentation}*/
optimizeSwimlaneDistances: boolean;
static $class: Class;
private $_DefaultDrawingDistanceCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultDrawingDistanceCalculator;
}
export interface DefaultLayerSequencer extends YObject, ISequencer {
/**
@inheritdoc*/
sequenceNodeLayers(graph: LayoutGraph | null, glayers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
} /**
This class is a {@link ISequencer} implementation that performs the second phase of the Sugiyama algorithm.
It minimizes the crossings in the diagram by using either the barycentric or median heuristic.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayerSequencer Online Documentation}*/
export class DefaultLayerSequencer {
/**
Creates a new instance of {@link DefaultLayerSequencer}.
@param [options] The parameters to pass.
@param [options.transposition] Whether or not to apply an additional crossing minimization heuristic. This option sets the {@link DefaultLayerSequencer#transposition} property on the created object.
@param [options.groupTransposition] Whether or not an additional crossing minimization heuristic should be used in the presence of grouped graphs. This option sets the {@link DefaultLayerSequencer#groupTransposition} property on the created object.
@param [options.weightHeuristic] The weight heuristic that should be used. This option sets the {@link DefaultLayerSequencer#weightHeuristic} property on the created object.
@param [options.maximumDuration] The time limit (in milliseconds) set for this sequencer per execution. This option sets the {@link DefaultLayerSequencer#maximumDuration} property on the created object.
@param [options.randomizationRounds] The number of randomized rounds that this algorithm performs, if there was no optimal solution. This option sets the {@link DefaultLayerSequencer#randomizationRounds} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayerSequencer Online Documentation}*/
constructor(options?: {
transposition?: boolean;
groupTransposition?: boolean;
weightHeuristic?: WeightHeuristic | WeightHeuristicStringValues;
maximumDuration?: number;
randomizationRounds?: number;
});
/**
Returns the cost of an edge crossing between the two given edges.
The default implementation defines the crossing cost of two edges as the product of the individual crossing costs which
are retrieved from the {@link IDataProvider} registered with the graph with key
{@link HierarchicLayout#EDGE_CROSSING_COST_DP_KEY}. If no individual crossing costs are defined, all edges
have a cost of <code>1</code>, hence, this method also returns <code>1</code>.
This method may be overridden to specify costs for a specific pair of edges. The given {@link ILayoutDataProvider}
is helpful to determine the type of the edges by using method {@link ILayoutDataProvider#getEdgeData}.
Furthermore, using the original edge to decide what a crossing costs might be helpful, see
{@link HierarchicLayout#getOriginalEdge}.
@param edge1 the first edge involved in the crossing
@param edge2 the second edge involved in the crossing
@param layoutDataProvider the {@link ILayoutDataProvider} containing information about the edges and nodes
@returns the crossing cost for the two given edges
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayerSequencer%23getCrossingCost(Edge%2CEdge%2CILayoutDataProvider) Online Documentation}*/
getCrossingCost(edge1: Edge | null, edge2: Edge | null, layoutDataProvider: ILayoutDataProvider | null): number;
/**
Gets or sets whether or not to apply an additional crossing minimization heuristic.
Activating this heuristic can reduce the overall number of edge crossings. On the other hand, it may increase the
running time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayerSequencer%23transposition Online Documentation}*/
transposition: boolean;
/**
Gets or sets whether or not an additional crossing minimization heuristic should be used in the presence of grouped
graphs.
Activating this heuristic can reduce the overall number of edge crossings in grouped graphs. On the other hand, it may
increase running time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayerSequencer%23groupTransposition Online Documentation}*/
groupTransposition: boolean;
/**
Gets or sets the weight heuristic that should be used.
@throws {Exception} <b>(ArgumentError)</b> if the constant is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayerSequencer%23weightHeuristic Online Documentation}*/
weightHeuristic: WeightHeuristic;
/**
Gets or sets the time limit (in milliseconds) set for this sequencer per execution.
Values have to be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayerSequencer%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the number of randomized rounds that this algorithm performs, if there was no optimal solution.
Values have to be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if a negative value is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayerSequencer%23randomizationRounds Online Documentation}*/
randomizationRounds: number;
static $class: Class;
private $_DefaultLayerSequencer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultLayerSequencer;
}
export interface DefaultLayeredComponentsMerger extends YObject, ILayeredComponentsMerger {
/**
@inheritdoc*/
merge(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, srcLayers: ILayers | null, targetLayers: ILayers | null): void;
} /**
A default implementation of a {@link ILayeredComponentsMerger} that provides simple default behaviors.
Instances of this class are used internally by {@link HierarchicLayoutCore} and {@link MultiComponentLayerer}
when e.g. the graph is grouped or it consists of more than one connected component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayeredComponentsMerger Online Documentation}*/
export class DefaultLayeredComponentsMerger {
/**
Creates a new instance of {@link DefaultLayeredComponentsMerger} using the given merging policy.
@param policy one of the predefined merging constants
@throws {Exception} <b>(ArgumentError)</b> if an unknown policy is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayeredComponentsMerger%23DefaultLayeredComponentsMerger(MergingPolicy) Online Documentation}*/
constructor(policy: MergingPolicy | MergingPolicyStringValues);
/**
Creates a new instance of {@link DefaultLayeredComponentsMerger} with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayeredComponentsMerger%23DefaultLayeredComponentsMerger() Online Documentation}*/
constructor();
/**
Gets or sets the merging policy constant as defined in {@link DefaultLayeredComponentsMerger}.
@throws {Exception} <b>(ArgumentError)</b> if an unknown policy is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultLayeredComponentsMerger%23policy Online Documentation}*/
policy: MergingPolicy;
static $class: Class;
private $_DefaultLayeredComponentsMerger: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultLayeredComponentsMerger;
}
export interface DefaultPortAllocator extends YObject, IPortAllocator {
}
/**
This class is a default {@link IPortAllocator} implementation.
It is used by {@link HierarchicLayoutCore} after the sequencing phase.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortAllocator Online Documentation}*/
export class DefaultPortAllocator {
/**
Creates a new instance of {@link DefaultPortAllocator} with default settings.
@param [options] The parameters to pass.
@param [options.considerFixedPorts] Whether or not this {@link IPortAllocator} considers edges with strong {@link PortConstraint}s or fixed {@link PortCandidate}. This option sets the {@link DefaultPortAllocator#considerFixedPorts} property on the created object.
@param [options.defaultPortBorderGapRatio] The border gap ratio for the default port. This option sets the {@link DefaultPortAllocator#defaultPortBorderGapRatio} property on the created object.
@see DefaultPortAllocator#defaultPortBorderGapRatio
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortAllocator Online Documentation}*/
constructor(options?: {
considerFixedPorts?: boolean;
defaultPortBorderGapRatio?: number;
});
/**
Assigns source and target port coordinates to each edge of the graph.
This method assigns:
<ul>
<li>{@link PortSide#ANY} ports to appropriate sides.
<li>{@link PortConstraint#strong Weak} ports to appropriate positions between strong ones.
</ul>
This method is called by {@link HierarchicLayoutCore} after the sequencing phase.
@param graph the input graph
@param layers the given {@link ILayers} instance containing the layering structure
@param ldp the {@link ILayoutDataProvider} containing information about the nodes and edges of the graph
@param itemFactory the {@link IItemFactory} used for creating and destroying helper structures
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortAllocator%23assignPorts(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
assignPorts(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
Callback method used for determining the port border gap for each node and side.
It may be overridden for a custom calculation of port border gap on a given node.
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} containing information about the nodes and edges of the graph
@param node the given node
@param sideIndex the zero-based (top is 0) clock-wise index of the side of the node (for top-to-bottom layouts)
@param sideLength the width/height of the side
@param edgeCount the number of edges/ports that connect to this side
@returns the absolute gap on both sides of the ports
@see DefaultPortAllocator#getPortDistanceDelta
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortAllocator%23getPortBorderGap(LayoutGraph%2CILayoutDataProvider%2CYNode%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
getPortBorderGap(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, node: YNode | null, sideIndex: number, sideLength: number, edgeCount: number): number;
/**
Callback method used for determining the port border gap ratio.
See {@link DefaultPortAllocator#defaultPortBorderGapRatio} for an explanation.
It may be overridden for a custom calculation of port border gap ratio on a given node.
<b>Warning:</b> If there exists a {@link HierarchicLayoutNodeLayoutDescriptor} instance registered for the given node, this implementation
returns the same value as {@link HierarchicLayoutNodeLayoutDescriptor#getPortBorderGapRatio}. Else, returns the same value as
{@link DefaultPortAllocator#defaultPortBorderGapRatio}.
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} containing information about the nodes and edges of the graph
@param node the given node
@param sideIndex the zero-based (top is 0) clock-wise index of the side of the node (for top-to-bottom layouts)
@param sideLength the width/height of the side
@param edgeCount the number of edges/ports that connect to this side
@returns the border gap ratio
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortAllocator%23getPortBorderGapRatio(LayoutGraph%2CILayoutDataProvider%2CYNode%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
getPortBorderGapRatio(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, node: YNode | null, sideIndex: number, sideLength: number, edgeCount: number): number;
/**
Callback method used for determining the distance between two adjacent ports.
It may be overridden for a custom calculation of the distance between two adjacent ports..
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} containing information about the nodes and edges of the graph
@param node the given node
@param sideIndex the zero-based (top is 0) clock-wise index of the side of the node (for top-to-bottom layouts)
@param sideLength the width/height of the side
@param edgeCount the number of edges/ports that connect to this side
@param portBorderGap the previously calculated port border gap
@returns the absolute distance between two adjacent ports
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortAllocator%23getPortDistanceDelta(LayoutGraph%2CILayoutDataProvider%2CYNode%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
getPortDistanceDelta(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, node: YNode | null, sideIndex: number, sideLength: number, edgeCount: number, portBorderGap: number): number;
/**
Gets or sets whether or not this {@link IPortAllocator} considers edges with strong
{@link PortConstraint}s or fixed {@link PortCandidate}.
If these edges are ignored, the ports of the other edges are uniformly distributed on the sides of the nodes which may
lead to intersections with the ignored edges. On the other hand, if these edges are considered, the port assignment of
the other edges may no longer be uniform.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortAllocator%23considerFixedPorts Online Documentation}*/
considerFixedPorts: boolean;
/**
Gets or sets the border gap ratio for the default port.
This ratio determines the gap between a corner of the node and the first assigned port.
The ratio should be greater than <code>0</code>.
A value of <code>0.0d</code> results in ports being placed directly on the corner of the node (if there is more than one port).
A value of <code>0.5d</code> results in ports being distributed along the side of the node so that the distance between the corner of
the node and the first port is half as wide as the distance between two adjacent ports.
A value of <code>Double.POSITIVE_INFINITY</code> results in all ports being centered at the side in one point.
@throws {Exception} <b>(ArgumentError)</b> if the given ratio is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultPortAllocator%23defaultPortBorderGapRatio Online Documentation}*/
defaultPortBorderGapRatio: number;
static $class: Class;
private $_DefaultPortAllocator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultPortAllocator;
}
export interface HierarchicLayoutEdgeLayoutDescriptor extends YObject {
}
/**
This class is used by {@link HierarchicLayoutCore} during the various phases to provide routing details for
the edges of the graph.
An {@link HierarchicLayoutEdgeLayoutDescriptor} instance can be specified individually for single edges using a {@link IDataProvider}
that returns an {@link HierarchicLayoutEdgeLayoutDescriptor} instance for each edge of the graph, or <code>null</code> if no {@link HierarchicLayoutEdgeLayoutDescriptor}
is bound to an edge. The {@link IDataProvider} is registered with the graph using key
{@link HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
This class is designed as a class to allow future additions of new getter methods.
<h2>Note</h2>
Not all of these values will be considered for all kinds of edges and algorithm settings.
@see HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see HierarchicLayoutNodeLayoutDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor Online Documentation}*/
export class HierarchicLayoutEdgeLayoutDescriptor {
/**
Creates a new instance of an {@link HierarchicLayoutEdgeLayoutDescriptor} with the default values.
@param [options] The parameters to pass.
@param [options.minimumFirstSegmentLength] The minimum length of the first segment of the edge path (at the source node). This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumFirstSegmentLength} property on the created object.
@param [options.minimumLastSegmentLength] The minimum length of the last segment of the edge path (at the target node). This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumLastSegmentLength} property on the created object.
@param [options.minimumLength] The minimum length of an edge. This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumLength} property on the created object.
@param [options.minimumDistance] The preferred minimum distance to obstacles. This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumDistance} property on the created object.
@param [options.minimumSlope] The minimum slope that the mid segments of this edge should have, if the edge is routed polyline in vertical layouts. This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumSlope} property on the created object.
@param [options.sourcePortOptimization] Whether or not source port optimization is enabled. This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#sourcePortOptimization} property on the created object.
@param [options.targetPortOptimization] Whether or not target port optimization is enabled. This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#targetPortOptimization} property on the created object.
@param [options.routingStyle] The routing style for this edge. This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#routingStyle} property on the created object.
@param [options.minimumOctilinearSegmentLength] The minimum length of octilinear segments for this edge. This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumOctilinearSegmentLength} property on the created object.
@param [options.recursiveEdgeStyle] How edges in grouped graphs are routed between the content of different groups. This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle} property on the created object.
@param [options.directGroupContentEdgeRouting] Whether or not this edge is routed directly to a group node's border if it connects the group node with one of its descendants. This option sets the {@link HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor Online Documentation}*/
constructor(options?: {
minimumFirstSegmentLength?: number;
minimumLastSegmentLength?: number;
minimumLength?: number;
minimumDistance?: number;
minimumSlope?: number;
sourcePortOptimization?: boolean;
targetPortOptimization?: boolean;
routingStyle?: HierarchicLayoutRoutingStyle | null;
minimumOctilinearSegmentLength?: number;
recursiveEdgeStyle?: RecursiveEdgeStyle | RecursiveEdgeStyleStringValues;
directGroupContentEdgeRouting?: boolean;
});
/**
Creates a copy of this {@link HierarchicLayoutEdgeLayoutDescriptor} instance.
@returns a copy of this {@link HierarchicLayoutEdgeLayoutDescriptor} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23createCopy() Online Documentation}*/
createCopy(): HierarchicLayoutEdgeLayoutDescriptor | null;
/**
Gets or sets the minimum length of the first segment of the edge path (at the source node).
The minimum length should be greater than <code>0</code>.
If the value is <code>0.0d</code> the first segment does not need to be orthogonal.
<b>Warning:</b> The first segment length may be violated for curved edge routes when using {@link HierarchicLayoutRoutingStyle#sourceCurveConnectionStyle curve connection} style
{@link CurveConnectionStyle#ORGANIC}.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length of the first segment is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23minimumFirstSegmentLength Online Documentation}*/
minimumFirstSegmentLength: number;
/**
Gets or sets the minimum length of the last segment of the edge path (at the target node).
The minimum length should be greater than <code>0</code>.
If the value is <code>0.0d</code> the last segment does not need to be orthogonal.
<b>Warning:</b> The last segment length may be violated for curved edge routes when using {@link HierarchicLayoutRoutingStyle#targetCurveConnectionStyle curve connection} style
{@link CurveConnectionStyle#ORGANIC}.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length of the last segment is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23minimumLastSegmentLength Online Documentation}*/
minimumLastSegmentLength: number;
/**
Gets or sets the minimum length of an edge.
If this is used for polyline routing, it describes the minimum vertical length of the mid segment. If it is used for
orthogonal routing, it describes the minimum length of the whole edge.
The minimum length should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23minimumLength Online Documentation}*/
minimumLength: number;
/**
Gets or sets the preferred minimum distance to obstacles.
The minimum distance should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23minimumDistance Online Documentation}*/
minimumDistance: number;
/**
Gets or sets the minimum slope that the mid segments of this edge should have, if the edge is routed polyline in
vertical layouts.
The minimum slope should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum slope is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23minimumSlope Online Documentation}*/
minimumSlope: number;
/**
Gets or sets whether or not source port optimization is enabled.
This means that the algorithm should try to find a better source {@link PortConstraint port constraint} if the current constraint is <code>null</code> or
{@link PortSide#ANY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23sourcePortOptimization Online Documentation}*/
sourcePortOptimization: boolean;
/**
Gets or sets whether or not target port optimization is enabled.
This means that the algorithm should try to find a better target {@link PortConstraint port constraint} if the current constraint is <code>null</code> or
{@link PortSide#ANY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23targetPortOptimization Online Documentation}*/
targetPortOptimization: boolean;
/**
Gets or sets the routing style for this edge.
@see HierarchicLayoutRoutingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23routingStyle Online Documentation}*/
routingStyle: HierarchicLayoutRoutingStyle | null;
/**
Gets or sets the minimum length of octilinear segments for this edge.
<b>Warning:</b> This option has an effect only if the {@link HierarchicLayoutEdgeLayoutDescriptor#routingStyle routing style} is {@link HierarchicLayoutEdgeRoutingStyle#OCTILINEAR octilinear}.
<h2>Note</h2>
The layout algorithm cannot always maintain the specified minimum length.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length of the first segment is negative
@see HierarchicLayoutEdgeLayoutDescriptor#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23minimumOctilinearSegmentLength Online Documentation}*/
minimumOctilinearSegmentLength: number;
/**
Gets or sets how edges in grouped graphs are routed between the content of different groups.
<h2>Note</h2>
If also {@link HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting direct routing between groups and their content} is enabled, such an edge may not be strictly recursive anymore.
Edges may leave/enter group nodes left or right.
<b>Warning:</b> An edge that belongs to a {@link HierarchicLayoutBusDescriptor bus structure} can not be recursive at the same time.
<b>Warning:</b> Edge grouping is not supported for the recursive styles {@link RecursiveEdgeStyle#DIRECTED} and
{@link RecursiveEdgeStyle#UNDIRECTED}. Any edge grouping information will be ignored, including
{@link HierarchicLayout#automaticEdgeGrouping automatic groupings}.
@throws {Exception} <b>(ArgumentError)</b> if an unknown style is specified
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23recursiveEdgeStyle Online Documentation}*/
recursiveEdgeStyle: RecursiveEdgeStyle;
/**
Gets or sets whether or not this edge is routed directly to a group node's border if it connects the group node with one
of its descendants.
When enabled, the edge will connect to the inner side of the group node's border instead of leaving the group node and
connect from outside.
<b>Warning:</b> This only applies to edges that connect to a group node from inside this group node.
<b>Warning:</b> If enabled, {@link PortConstraint}s assigned to the end of an edge that connects to a group are restricted to the
two sides lying in the direction of the main flow. More precisely, for layout orientation {@link LayoutOrientation#TOP_TO_BOTTOM}
only side {@link PortSide#NORTH} and {@link PortSide#SOUTH} are allowed.
<h2>Note</h2>
Edges that are completely overlapped by their source or target may require special attention during visualization. They
can disappear when edges are clipped at their source and target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeLayoutDescriptor%23directGroupContentEdgeRouting Online Documentation}*/
directGroupContentEdgeRouting: boolean;
static $class: Class;
private $_EdgeLayoutDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HierarchicLayoutEdgeLayoutDescriptor;
}
/**
A weight assignment specifier based on a median heuristic.
@see DefaultLayerSequencer#weightHeuristic
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightHeuristic Online Documentation}*/
export enum WeightHeuristic {
/**
A weight assignment specifier based on a barycenter heuristic.
The position of a node within a layer will be determined by the barycenter of its successor (downward pass) and
predecessor (upward pass) nodes.
@see DefaultLayerSequencer#weightHeuristic
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightHeuristic%23BARYCENTER Online Documentation}*/
BARYCENTER,
/**
A weight assignment specifier based on a median heuristic.
The position of a node within a layer will be determined by the median position of its successor (downward pass) and
predecessor (upward pass) nodes.
@see DefaultLayerSequencer#weightHeuristic
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightHeuristic%23MEDIAN Online Documentation}*/
MEDIAN
}
/** Automatic conversion to {@link WeightHeuristic}. */
type WeightHeuristicStringValues = "barycenter" | "median";
/**
@see DefaultLayeredComponentsMerger#DefaultLayeredComponentsMerger
@see DefaultLayeredComponentsMerger#policy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingPolicy Online Documentation}*/
export enum MergingPolicy {
/**
A constant that adds the new layers of the source {@link ILayers layering} below the layers of the target {@link ILayers layering}.
@see DefaultLayeredComponentsMerger#policy
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingPolicy%23ADD_BELOW Online Documentation}*/
ADD_BELOW,
/**
A constant that adds the new layers of the source {@link ILayers layering} above the layers of the target {@link ILayers layering}.
@see DefaultLayeredComponentsMerger#policy
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingPolicy%23ADD_ABOVE Online Documentation}*/
ADD_ABOVE,
/**
A constant that adds the new layers of the source {@link ILayers layering} to the right of the layers of the target layering.
Both {@link ILayers layering}s will become top-aligned before the merge.
<h2>Note</h2>
The source layering is the one with the larger number of layers.
@see DefaultLayeredComponentsMerger#policy
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingPolicy%23ADD_RIGHT_TOP_ALIGNED Online Documentation}*/
ADD_RIGHT_TOP_ALIGNED,
/**
A constant that adds the new layers of the source {@link ILayers layering} to the right of the layers of the target {@link ILayers layering}.
Both {@link ILayers layering}s will become bottom-aligned before the merge.
<h2>Note</h2>
The source layering is the one with the larger number of layers.
@see DefaultLayeredComponentsMerger#policy
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingPolicy%23ADD_RIGHT_BOTTOM_ALIGNED Online Documentation}*/
ADD_RIGHT_BOTTOM_ALIGNED,
/**
A constant that adds the new layers of the source {@link ILayers layering} to the right of the layers of the target {@link ILayers layering}.
Both {@link ILayers layering}s will become center-aligned before the merge.
<h2>Note</h2>
The source layering is the one with the larger number of layers.
@see DefaultLayeredComponentsMerger#policy
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingPolicy%23ADD_RIGHT_CENTER_ALIGNED Online Documentation}*/
ADD_RIGHT_CENTER_ALIGNED,
/**
A constant that adds the new layers of the source {@link ILayers layering} to the left of the layers of the target {@link ILayers layering}.
Both {@link ILayers layering}s will become top-aligned before the merge.
<h2>Note</h2>
The source layering is the one with the larger number of layers.
@see DefaultLayeredComponentsMerger#policy
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingPolicy%23ADD_LEFT_TOP_ALIGNED Online Documentation}*/
ADD_LEFT_TOP_ALIGNED,
/**
A constant that adds the new layers of the source {@link ILayers layering} to the left of the layers of the target {@link ILayers layering}.
Both {@link ILayers layering}s will become bottom-aligned before the merge.
<h2>Note</h2>
The source layering is the one with the larger number of layers.
@see DefaultLayeredComponentsMerger#policy
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingPolicy%23ADD_LEFT_BOTTOM_ALIGNED Online Documentation}*/
ADD_LEFT_BOTTOM_ALIGNED,
/**
A constant that adds the new layers of the source {@link ILayers layering} to the left of the layers of the target {@link ILayers layering}.
Both {@link ILayers layering}s will become center-aligned before the merge.
<h2>Note</h2>
The source layering is the one with the larger number of layers.
@see DefaultLayeredComponentsMerger#policy
@see {@link https://docs.yworks.com/yfileshtml/#/api/MergingPolicy%23ADD_LEFT_CENTER_ALIGNED Online Documentation}*/
ADD_LEFT_CENTER_ALIGNED
}
/** Automatic conversion to {@link MergingPolicy}. */
type MergingPolicyStringValues = "add-below" | "add-above" | "add-right-top-aligned" | "add-right-bottom-aligned" | "add-right-center-aligned" | "add-left-top-aligned" | "add-left-bottom-aligned" | "add-left-center-aligned";
/**
@see IEdgeData#type
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDataType Online Documentation}*/
export enum EdgeDataType {
/**
A constant describing a normal edge.
@see IEdgeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDataType%23NORMAL Online Documentation}*/
NORMAL,
/**
A constant describing an edge that connects to a source group node.
@see IEdgeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDataType%23SOURCE_GROUP_NODE_CONNECTOR Online Documentation}*/
SOURCE_GROUP_NODE_CONNECTOR,
/**
A constant describing an edge that connects to a target group node.
@see IEdgeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDataType%23TARGET_GROUP_NODE_CONNECTOR Online Documentation}*/
TARGET_GROUP_NODE_CONNECTOR,
/**
A constant describing an edge that connects two group nodes.
@see IEdgeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDataType%23GROUP_NODE_INTERCONNECTOR Online Documentation}*/
GROUP_NODE_INTERCONNECTOR,
/**
A constant describing a same-layer edge that connects directly two neighboring nodes.
@see IEdgeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDataType%23DIRECT_SAME_LAYER_EDGE Online Documentation}*/
DIRECT_SAME_LAYER_EDGE,
/**
A constant describing a same-layer edge that does not connect directly two neighboring nodes.
This means that two nodes of the same layer are not connected directly with a straight-line edge, but with an edge
consisting of polyline segments.
@see IEdgeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDataType%23NON_DIRECT_SAME_LAYER_EDGE Online Documentation}*/
NON_DIRECT_SAME_LAYER_EDGE,
/**
A constant describing an edge that connects group border nodes in adjacent layers.
@see IEdgeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDataType%23BORDER_EDGE Online Documentation}*/
BORDER_EDGE,
/**
A constant describing an edge that has been added temporarily to replace an edge connected to a group node.
@see IEdgeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeDataType%23REDIRECTED_GROUP_EDGE Online Documentation}*/
REDIRECTED_GROUP_EDGE
}
/** Automatic conversion to {@link EdgeDataType}. */
type EdgeDataTypeStringValues = "normal" | "source-group-node-connector" | "target-group-node-connector" | "group-node-interconnector" | "direct-same-layer-edge" | "non-direct-same-layer-edge" | "border-edge" | "redirected-group-edge";
/**
Routing specifier that routes edges between different group contents non-recursively.
@see HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveEdgeStyle Online Documentation}*/
export enum RecursiveEdgeStyle {
/**
Routing specifier that routes edges between different group contents non-recursively.
{@link Edge}s may leave and enter group nodes at the left and right side in order to connect as directly as
possible.
@see HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveEdgeStyle%23OFF Online Documentation}*/
OFF,
/**
Routing specifier that routes edges between different group contents recursively and directed.
{@link Edge}s can only leave the groups at the bottom side and enter them at the top side.
@see HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveEdgeStyle%23DIRECTED Online Documentation}*/
DIRECTED,
/**
Routing specifier that routes edges between different group contents recursively and undirected.
{@link Edge}s can leave the groups at the bottom or top side and enter them at the bottom or top side. This
style is more compact than the directed edge style.
@see HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveEdgeStyle%23UNDIRECTED Online Documentation}*/
UNDIRECTED
}
/** Automatic conversion to {@link RecursiveEdgeStyle}. */
type RecursiveEdgeStyleStringValues = "off" | "directed" | "undirected";
/**
@see HierarchicLayout#componentArrangementPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementPolicy Online Documentation}*/
export enum ComponentArrangementPolicy {
/**
A component arrangement policy that achieves a <code>1:1</code> aspect ratio for the graph.
After layering the single components with the core layerer, the components will be merged such that an 1:1 aspect ratio
is fulfilled best.
@see HierarchicLayout#componentArrangementPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementPolicy%23COMPACT Online Documentation}*/
COMPACT,
/**
A component arrangement policy that aligns the components with their topmost layer.
After layering the single components with the core layerer, the components are aligned with their topmost layer.
@see HierarchicLayout#componentArrangementPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentArrangementPolicy%23TOPMOST Online Documentation}*/
TOPMOST
}
/** Automatic conversion to {@link ComponentArrangementPolicy}. */
type ComponentArrangementPolicyStringValues = "compact" | "topmost";
/**
@see HierarchicLayout#layoutMode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutMode Online Documentation}*/
export enum LayoutMode {
/**
A layout mode that allows to keep some nodes fixed and insert the remaining nodes incrementally.
Fixed elements will be handled by the {@link HierarchicLayout#fixedElementsSequencer fixed elements sequencer} and {@link HierarchicLayout#fixedElementsLayerer fixed elements layerer}. Incremental elements will be
added during the layering or the sequencing phase. They are handled by {@link HierarchicLayout#fromScratchLayerer from scratch layerer} and {@link HierarchicLayout#fixedElementsLayerer fixed elements layerer}
respectively.
@see IIncrementalHintsFactory#createLayerIncrementallyHint
@see HierarchicLayout#layoutMode
@see HierarchicLayout#fixedElementsSequencer
@see HierarchicLayout#fixedElementsLayerer
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutMode%23INCREMENTAL Online Documentation}*/
INCREMENTAL,
/**
A layout mode that will recompute the complete layout from scratch.
All elements will be handled by the same {@link HierarchicLayout#fromScratchLayerer layerer} and {@link HierarchicLayout#fromScratchSequencer sequencer}.
@see HierarchicLayout#layoutMode
@see HierarchicLayout#fromScratchSequencer
@see HierarchicLayout#fromScratchLayerer
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutMode%23FROM_SCRATCH Online Documentation}*/
FROM_SCRATCH
}
/** Automatic conversion to {@link LayoutMode}. */
type LayoutModeStringValues = "incremental" | "from-scratch";
/**
@see HierarchicLayout#fromScratchLayeringStrategy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutLayeringStrategy Online Documentation}*/
export enum HierarchicLayoutLayeringStrategy {
/**
A layering strategy which places each node in the highest possible layer without passing its predecessor in the
topological order of nodes.
All nodes with indegree zero will be assigned to the topmost layer of the layout. The number of separate layers will be
as small as possible.
@see HierarchicLayout#fromScratchLayeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutLayeringStrategy%23HIERARCHICAL_TOPMOST Online Documentation}*/
HIERARCHICAL_TOPMOST,
/**
A layering strategy which places each node in the optimal layer to minimize the layer distances.
The layer distance of an edge is the absolute difference between the layer numbers of its source and target node. The
layer assignment will minimize the overall sum of the layer distances of all edges in the layout.
@see HierarchicLayout#fromScratchLayeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutLayeringStrategy%23HIERARCHICAL_OPTIMAL Online Documentation}*/
HIERARCHICAL_OPTIMAL,
/**
A layering strategy which applies a heuristic to approximate the {@link HierarchicLayoutLayeringStrategy#HIERARCHICAL_OPTIMAL optimal layering}.
The quality of the result may not be as good as with {@link HierarchicLayoutLayeringStrategy#HIERARCHICAL_OPTIMAL optimal layering} but it is usually achieved faster.
@see HierarchicLayout#fromScratchLayeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutLayeringStrategy%23HIERARCHICAL_TIGHT_TREE Online Documentation}*/
HIERARCHICAL_TIGHT_TREE,
/**
A layering strategy which applies a fast heuristic that improves the layering done by {@link HierarchicLayoutLayeringStrategy#HIERARCHICAL_TOPMOST}
by shifting some nodes down.
The quality of the result is usually worse than the one produced by
{@link HierarchicLayoutLayeringStrategy#HIERARCHICAL_TIGHT_TREE}.
@see HierarchicLayout#fromScratchLayeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutLayeringStrategy%23HIERARCHICAL_DOWNSHIFT Online Documentation}*/
HIERARCHICAL_DOWNSHIFT,
/**
A layering strategy based on a breadth first search (BFS).
All edges will span at most one layer in the resulting drawing. Edges between nodes that belong to the same layer are
possible.
The nodes that will be placed in the first layer can be provided by a data provider bound to the input graph using the
key
{@link BFSLayerer#CORE_NODES_DP_KEY}. If this data provider is not given, then nodes that have no incoming
edges are placed in the first layer.
@see HierarchicLayout#fromScratchLayeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutLayeringStrategy%23BFS Online Documentation}*/
BFS,
/**
A layering strategy which derives the layers from the initial coordinates of the nodes.
It tries to find a layering that is similar to the one in the input graph. The layout algorithm may place nodes in the
same layer, even though they are connected by an edge.
In a vertically oriented layout, nodes that overlap vertically are placed in the same layer. The same holds for a
horizontally oriented layout in the case where the nodes overlap horizontally.
@see HierarchicLayout#fromScratchLayeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutLayeringStrategy%23FROM_SKETCH Online Documentation}*/
FROM_SKETCH,
/**
A layering strategy which uses a layer assignment specified by the user.
The nodes' layer assignment must be provided by a data provider bound to the input graph using the key
{@link GivenLayersLayerer#LAYER_ID_DP_KEY}.
This layering strategy allows edges whose source and target share a layer.
@see HierarchicLayout#fromScratchLayeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutLayeringStrategy%23USER_DEFINED Online Documentation}*/
USER_DEFINED,
/**
A dummy layering strategy which describes a strategy that is not part of the default strategies.
@see HierarchicLayout#fromScratchLayeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutLayeringStrategy%23UNKNOWN Online Documentation}*/
UNKNOWN
}
/** Automatic conversion to {@link HierarchicLayoutLayeringStrategy}. */
type HierarchicLayoutLayeringStrategyStringValues = "hierarchical-topmost" | "hierarchical-optimal" | "hierarchical-tight-tree" | "hierarchical-downshift" | "bfs" | "from-sketch" | "user-defined" | "unknown";
/**
A group layering alignment strategy which aligns groups and normal nodes that occupy the same layer to the top with
respect to their inner layers.
@see HierarchicLayout#groupAlignmentPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlignmentPolicy Online Documentation}*/
export enum GroupAlignmentPolicy {
/**
A group layering alignment strategy which aligns groups and normal nodes that occupy the same layer to the top with
respect to their inner layers.
<b>Warning:</b> This only applies when recursive group layering is enabled.
@see HierarchicLayout#groupAlignmentPolicy
@see HierarchicLayout#recursiveGroupLayering
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlignmentPolicy%23TOP Online Documentation}*/
TOP,
/**
A group layering alignment strategy which aligns groups and normal nodes that occupy the same layer to the center with
respect to their inner layers.
<b>Warning:</b> This only applies when recursive group layering is enabled.
@see HierarchicLayout#groupAlignmentPolicy
@see HierarchicLayout#recursiveGroupLayering
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlignmentPolicy%23CENTER Online Documentation}*/
CENTER,
/**
A group layering alignment strategy which aligns groups and normal nodes that occupy the same layer to the bottom with
respect to their inner layers.
<b>Warning:</b> This only applies when recursive group layering is enabled.
@see HierarchicLayout#groupAlignmentPolicy
@see HierarchicLayout#recursiveGroupLayering
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupAlignmentPolicy%23BOTTOM Online Documentation}*/
BOTTOM
}
/** Automatic conversion to {@link GroupAlignmentPolicy}. */
type GroupAlignmentPolicyStringValues = "top" | "center" | "bottom";
/**
@see ILayer#type
@see ILayers#insert
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayerType Online Documentation}*/
export enum LayerType {
/**
A constant describing a normal layer consisting mainly of normal nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayerType%23NORMAL Online Documentation}*/
NORMAL,
/**
A constant describing a layer consisting mainly of label nodes or dummy nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayerType%23LABEL Online Documentation}*/
LABEL,
/**
A constant describing a layer consisting of source group nodes and dummy nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayerType%23SOURCE_GROUP_NODES Online Documentation}*/
SOURCE_GROUP_NODES,
/**
A constant describing a layer consisting of target group nodes and dummy nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayerType%23TARGET_GROUP_NODES Online Documentation}*/
TARGET_GROUP_NODES,
/**
Describes a layer above a group to store the connector proxies where edges connect to groups.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayerType%23UPPER_GROUP_CONNECTOR_NODES Online Documentation}*/
UPPER_GROUP_CONNECTOR_NODES,
/**
Describes a layer below a group to store the connector proxies where edges connect to groups.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayerType%23LOWER_GROUP_CONNECTOR_NODES Online Documentation}*/
LOWER_GROUP_CONNECTOR_NODES
}
/** Automatic conversion to {@link LayerType}. */
type LayerTypeStringValues = "normal" | "label" | "source-group-nodes" | "target-group-nodes" | "upper-group-connector-nodes" | "lower-group-connector-nodes";
/**
@see IItemFactory#createGroupBorderNode
@see INodeData#type
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType Online Documentation}*/
export enum NodeDataType {
/**
A constant describing a normal {@link YNode}.
A normal node may have any degree, any size and any number of connected same-layer edges.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23NORMAL Online Documentation}*/
NORMAL,
/**
A constant describing a bend {@link YNode}.
A bend node has degree <code>2</code>, small size and no same-layer edges.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23BEND Online Documentation}*/
BEND,
/**
A constant describing a side proxy {@link YNode}.
A side proxy node, in case port constraints are used for the drawing phase, has degree
<code>1</code>, small size and no same-layer edges. The associated node is the one of which this node is the proxy.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23SIDE_PROXY Online Documentation}*/
SIDE_PROXY,
/**
A constant describing a side proxy {@link YNode} for drawing same-layer edges.
A side proxy node for drawing same-layer edges has degree
<code>0</code>, small size and one same-layer edge. The associated node is the one of which this node is the proxy. The associated
edge is the same-layer edge of which this node is the proxy.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23SAME_LAYER_SIDE_PROXY Online Documentation}*/
SAME_LAYER_SIDE_PROXY,
/**
A constant describing a label {@link YNode}.
A label node has degree
<code>2</code>, size of the labels and no same-layer edges. The borders of this node describe the layout of the labels.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23LABEL Online Documentation}*/
LABEL,
/**
A constant describing a group {@link YNode} for nested graph layouts.
This group node carries additional {@link ILayers} information.
@see INodeData#type
@see INodeData#groupLayers
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23GROUP Online Documentation}*/
GROUP,
/**
A constant describing a source group {@link YNode} for bus edges.
This node serves as a dummy node in a {@link LayerType#SOURCE_GROUP_NODES} layer.
@see INodeData#type
@see INodeData#groupId
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23SOURCE_GROUP_NODE Online Documentation}*/
SOURCE_GROUP_NODE,
/**
A constant describing a target group {@link YNode} for bus edges.
This node serves as a dummy node in a {@link LayerType#TARGET_GROUP_NODES} layer.
@see INodeData#type
@see INodeData#groupId
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23TARGET_GROUP_NODE Online Documentation}*/
TARGET_GROUP_NODE,
/**
A constant describing a back-loop proxy that belongs to the source of an edge.
A back-loop proxy that belongs to the source of an edge has degree <code>2</code>, small size and exactly one same-layer edge.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23SOURCE_BACK_LOOP_PROXY Online Documentation}*/
SOURCE_BACK_LOOP_PROXY,
/**
A constant describing a back-loop proxy that belongs to the target of an edge.
A back-loop proxy that belongs to the target of an edge has degree <code>2</code>, small size and exactly one same-layer edge.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23TARGET_BACK_LOOP_PROXY Online Documentation}*/
TARGET_BACK_LOOP_PROXY,
/**
A constant describing a same-layer edge crossing dummy {@link YNode}.
A same-layer edge crossing dummy node has degree
<code>2</code>, small size and exactly two same-layer edges. It is used for marking the crossing of the layer, if source and target
port are at opposite layer sides.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23SAME_LAYER_CENTER_NODE Online Documentation}*/
SAME_LAYER_CENTER_NODE,
/**
A constant describing a {@link YNode} used for the drawing phase to keep the distance between two nodes.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23DISTANCE_NODE Online Documentation}*/
DISTANCE_NODE,
/**
A constant describing a {@link YNode} used during the drawing phase for marking the beginning of a node group
interval.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23GROUP_BEGIN Online Documentation}*/
GROUP_BEGIN,
/**
A constant describing a {@link YNode} used during the drawing phase for marking the ending of a node group
interval.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23GROUP_END Online Documentation}*/
GROUP_END,
/**
A constant describing a {@link YNode} used during the sequencing and drawing phase for marking a dummy node
inserted to guarantee that a group is non-empty on the layer on which it is defined.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23GROUP_LAYER_DUMMY Online Documentation}*/
GROUP_LAYER_DUMMY,
/**
A constant describing a {@link YNode} used during the sequencing and drawing phase for marking a dummy node
inserted as a legal endpoint of an edge connecting to a group node.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23PROXY_FOR_EDGE_AT_GROUP Online Documentation}*/
PROXY_FOR_EDGE_AT_GROUP,
/**
A constant describing a {@link YNode} used during the sequencing and drawing phase for marking a dummy node
inserted as a legal endpoint of an edge connecting a descendant node to a group node.
It has degree <code>0</code> or <code>1</code> depending on whether or not the opposite node of the edge is assigned to the same layer. This kind
of proxy nodes is only used if there are edges that shall connect to the group node without backloops.
@see HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23PROXY_FOR_CONTENT_EDGE_AT_GROUP Online Documentation}*/
PROXY_FOR_CONTENT_EDGE_AT_GROUP,
/**
A constant describing a {@link YNode} used during sequencing and drawing phase for routing an {@link Edge}
recursively.
An {@link Edge} that connects to the content of a group node has to leave the group node at the bottom side
and enter it at the top side.
@see INodeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23RECURSIVE_EDGE_DUMMY Online Documentation}*/
RECURSIVE_EDGE_DUMMY,
/**
A constant describing a {@link YNode} that is used during the sequencing and drawing phase for the bus-style
routing of bus edges at a certain root node.
Such a node is inserted on each layer of the bus structure.
@see INodeData#type
@see HierarchicLayout#BUS_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeDataType%23BUS_STRUCTURE_DUMMY Online Documentation}*/
BUS_STRUCTURE_DUMMY
}
/** Automatic conversion to {@link NodeDataType}. */
type NodeDataTypeStringValues = "normal" | "bend" | "side-proxy" | "same-layer-side-proxy" | "label" | "group" | "source-group-node" | "target-group-node" | "source-back-loop-proxy" | "target-back-loop-proxy" | "same-layer-center-node" | "distance-node" | "group-begin" | "group-end" | "group-layer-dummy" | "proxy-for-edge-at-group" | "proxy-for-content-edge-at-group" | "recursive-edge-dummy" | "bus-structure-dummy";
/**
@see HierarchicLayoutNodeLayoutDescriptor#portAssignment
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutPortAssignmentMode Online Documentation}*/
export enum HierarchicLayoutPortAssignmentMode {
/**
A port assignment strategy that distributes ports evenly along the border of the node.
This is the default port assignment strategy of {@link HierarchicLayout}.
@see HierarchicLayoutNodeLayoutDescriptor#portAssignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutPortAssignmentMode%23DEFAULT Online Documentation}*/
DEFAULT,
/**
A port assignment strategy that places all edges on grid lines.
If there are not enough grid lines for each port, ports may overlap. In case there is no grid line available at the side
of a node, the ports are placed centered at that side.
<b>Warning:</b> This port assignment can only be used if the {@link HierarchicLayout#gridSpacing grid spacing} is greater than <code>0</code>.
@see HierarchicLayoutNodeLayoutDescriptor#portAssignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutPortAssignmentMode%23ON_GRID Online Documentation}*/
ON_GRID,
/**
A port assignment strategy that places all edges on grid lines or sub-grid lines.
If there are not enough grid lines for each port, the grid gets subdivided with sub-grid lines until each edge has space
for its port.
<b>Warning:</b> This port assignment can only be used if the {@link HierarchicLayout#gridSpacing grid spacing} is greater than <code>0</code>.
@see HierarchicLayoutNodeLayoutDescriptor#portAssignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutPortAssignmentMode%23ON_SUBGRID Online Documentation}*/
ON_SUBGRID
}
/** Automatic conversion to {@link HierarchicLayoutPortAssignmentMode}. */
type HierarchicLayoutPortAssignmentModeStringValues = "default" | "on-grid" | "on-subgrid";
/**
@see HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelMode Online Documentation}*/
export enum NodeLabelMode {
/**
A constant that prevents node labels from being considered during the layout.
It is used by {@link HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode}.
@see HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelMode%23NEVER Online Documentation}*/
NEVER,
/**
A constant that makes the algorithm consider node labels for self-loops, node placement and routing.
It is used by {@link HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode}.
@see HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelMode%23CONSIDER_FOR_SELF_LOOPS Online Documentation}*/
CONSIDER_FOR_SELF_LOOPS,
/**
A constant that makes the algorithm consider node labels for node placement and routing, but not for self-loops.
It is used by {@link HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode}.
@see HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelMode%23CONSIDER_FOR_DRAWING Online Documentation}*/
CONSIDER_FOR_DRAWING,
/**
A constant that makes the algorithm consider node labels during routing only.
It is used by {@link HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode}.
@see HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeLabelMode%23CONSIDER_FOR_ROUTING Online Documentation}*/
CONSIDER_FOR_ROUTING
}
/** Automatic conversion to {@link NodeLabelMode}. */
type NodeLabelModeStringValues = "never" | "consider-for-self-loops" | "consider-for-drawing" | "consider-for-routing";
/**
Routing styles for edges routed by the {@link HierarchicLayout}.
@see HierarchicLayoutRoutingStyle#RoutingStyle
@see HierarchicLayoutRoutingStyle#backLoopRoutingStyle
@see HierarchicLayoutRoutingStyle#defaultEdgeRoutingStyle
@see HierarchicLayoutRoutingStyle#edgeGroupRoutingStyle
@see HierarchicLayoutRoutingStyle#sameLayerEdgeRoutingStyle
@see HierarchicLayoutRoutingStyle#selfLoopRoutingStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeRoutingStyle Online Documentation}*/
export enum HierarchicLayoutEdgeRoutingStyle {
/**
A routing style constant specifying that the edges should be orthogonal.
Orthogonal edges only consist of vertical and horizontal segments.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeRoutingStyle%23ORTHOGONAL Online Documentation}*/
ORTHOGONAL,
/**
A routing style constant specifying that the edges should be octilinear.
Octilinear edges consist of vertical and horizontal segments as well as segments with slope <code>-1</code> and <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeRoutingStyle%23OCTILINEAR Online Documentation}*/
OCTILINEAR,
/**
A routing style constant specifying that the edges should be polyline.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeRoutingStyle%23POLYLINE Online Documentation}*/
POLYLINE,
/**
A routing style constant specifying that the edges should be curved.
Curved edges are constructed using cubic bezier splines. As for other routing styles, overlaps with other elements are
avoided. If there is very little space for a smooth curve, it can happen that the resulting path is polyline or almost
polyline with little curving. To reserve more space for curved edges, one can increase values of settings that influence
the distances between graph elements, e.g., {@link HierarchicLayoutEdgeLayoutDescriptor#minimumDistance}.
When {@link HierarchicLayout#integratedEdgeLabeling integrated edge labeling} is enabled, for edge labels a straight, non-curved segment where the label is placed will
be inserted. All the settings of the {@link PreferredPlacementDescriptor} are supported. Similarly, for the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumFirstSegmentLength minimum first segment length}
and the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumLastSegmentLength minimum last segment length}, a straight edge segment is created to fulfill the constraints.
<h2>Note</h2>
If the bends should actually be interpreted as bezier points, i.e. to let the visualization create the spline curves,
enable {@link HierarchicLayoutRoutingStyle#createControlPoints}
<h2>Note</h2>
If the curved route should be allowed to start/end with a non-orthogonal segment and at any side of the adjacent node,
the connection style {@link CurveConnectionStyle#ORGANIC} can be specified as {@link HierarchicLayoutRoutingStyle#sourceCurveConnectionStyle} and/or {@link HierarchicLayoutRoutingStyle#targetCurveConnectionStyle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutEdgeRoutingStyle%23CURVED Online Documentation}*/
CURVED
}
/** Automatic conversion to {@link HierarchicLayoutEdgeRoutingStyle}. */
type HierarchicLayoutEdgeRoutingStyleStringValues = "orthogonal" | "octilinear" | "polyline" | "curved";
/**
A strategy specifying no horizontal group compaction.
@see SimplexNodePlacer#groupCompactionStrategy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupCompactionPolicy Online Documentation}*/
export enum GroupCompactionPolicy {
/**
A strategy specifying no horizontal group compaction.
This means that group node contents will occupy nearly the same horizontal positions as when not grouped at all.
@see SimplexNodePlacer#groupCompactionStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupCompactionPolicy%23NONE Online Documentation}*/
NONE,
/**
A strategy specifying maximal horizontal group compaction.
This means that the node placer will try to minimize the horizontally occupied space for a group node.
<h2>Note</h2>
To obtain compact groups you should additionally disable option {@link SimplexNodePlacer#bendReduction}.
@see SimplexNodePlacer#groupCompactionStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupCompactionPolicy%23MAXIMAL Online Documentation}*/
MAXIMAL
}
/** Automatic conversion to {@link GroupCompactionPolicy}. */
type GroupCompactionPolicyStringValues = "none" | "maximal";
/**
@see TopologicalLayerer#rankingPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RankingPolicy Online Documentation}*/
export enum RankingPolicy {
/**
A ranking policy that specifies that nodes are not re-ranked after the initial topological layering step.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RankingPolicy%23NO_RERANKING Online Documentation}*/
NO_RERANKING,
/**
A ranking policy that specifies that nodes are re-ranked by an inexpensive downshift rule.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RankingPolicy%23DOWN_SHIFT Online Documentation}*/
DOWN_SHIFT,
/**
A ranking policy that specifies that nodes are re-ranked by finding a spanning tree that contains only tight (length
<code>1</code>) edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RankingPolicy%23TIGHT_TREE Online Documentation}*/
TIGHT_TREE
}
/** Automatic conversion to {@link RankingPolicy}. */
type RankingPolicyStringValues = "no-reranking" | "down-shift" | "tight-tree";
export interface GivenLayersLayerer extends YObject, ILayerer {
}
/**
This layering algorithm implementation assigns nodes to layers given a mapping of nodes to layer IDs.
The layer IDs are given by a {@link IDataProvider} that returns the integer layer ID of each node of the
graph. Nodes with the same ID are in the same layer while the layers are sorted according to their IDs such that the
smallest ID represents the top layer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenLayersLayerer Online Documentation}*/
export class GivenLayersLayerer {
/**
Creates an instance of {@link GivenLayersLayerer}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenLayersLayerer Online Documentation}*/
constructor();
/**
Assigns all nodes of the graph to layers and adds them to the {@link ILayers} instance.
@param graph the input graph
@param layers the {@link ILayers} instance that will be filled with the results of the calculation
@param ldp the {@link ILayoutDataProvider} used for querying information about the nodes and edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenLayersLayerer%23assignLayers(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
assignLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
Convenience method that removes empty layers and ensures that the smallest layer has value <code>0</code>.
@param graph the graph
@param layerId the {@link IDataProvider} that returns an integer value (layer) for each node; the first layer has the number
<code>0</code>
@param normalizedLayerId the {@link IDataAcceptor} that will be filled by the layout algorithm and holds an integer value (layer ID
after normalization) for each node
@returns the number of layers after the removal of empty layers
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenLayersLayerer%23normalize(Graph%2CIDataProvider%2CIDataAcceptor) Online Documentation}*/
normalize(graph: Graph | null, layerId: IDataProvider | null, normalizedLayerId: IDataAcceptor | null): number;
/**
Data provider key for storing the layer IDs
This {@link IDataProvider} stores for each node the layer ID to which it belongs. Nodes with the same ID are
in the same layer while the layers are sorted according to their IDs such that the smallest ID represents the top
layer.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenLayersLayerer%23LAYER_ID_DP_KEY Online Documentation}*/
static LAYER_ID_DP_KEY: NodeDpKey<number> | null;
static $class: Class;
private $_GivenLayersLayerer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GivenLayersLayerer;
}
export interface GivenSequenceSequencer extends YObject, ISequencer {
}
/**
This class is a {@link ISequencer} implementation that determines the sequence of nodes of the same {@link ILayer layer}
based on a given {@link IComparer comparator} constraint.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenSequenceSequencer Online Documentation}*/
export class GivenSequenceSequencer {
/**
Creates a new instance of {@link GivenSequenceSequencer} using a given {@link IComparer} for the sequencing.
@param [c=null]
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenSequenceSequencer Online Documentation}*/
constructor(c?: IComparer<any> | null);
/**
Calculates the sequence of the nodes within a {@link ILayers} instance based on the given {@link IComparer} instance.
This method is called by {@link HierarchicLayoutCore} during the sequencing phase and finally writes back the
calculated sequence using the {@link ILayer#setNodeOrder} method.
@param graph the input graph
@param layers the given {@link ILayers} instance containing the elements of the layers
@param ldp the {@link ILayoutDataProvider} containing information about the nodes and edges of the graph
@param itemFactory the {@link IItemFactory} used for creating and destroying helper structures
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenSequenceSequencer%23sequenceNodeLayers(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
sequenceNodeLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
Gets or sets the {@link IComparer} used by this {@link GivenSequenceSequencer} to determine the sequence of the nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenSequenceSequencer%23sequenceComparer Online Documentation}*/
sequenceComparer: IComparer<any> | null;
static $class: Class;
private $_GivenSequenceSequencer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GivenSequenceSequencer;
}
export interface HierarchicLayout extends MultiStageLayout {
}
/**
This layout algorithm arranges graphs in a hierarchic fashion.
<b>Layout Style</b>
The nodes are distributed into layers so that most of the edges point to the main layout direction. The order of the
nodes within the layers ensures that the number of edge crossings is as small as possible. There are different edge
routing styles available. Edges can be orthogonal, polyline, octilinear or curved (see {@link HierarchicLayoutRoutingStyle}
for details).
Hierarchical diagrams are commonly used for the visualization of hierarchical data, since they facilitate the
identification of dependencies and relationships among the nodes of the graph. Possible application domains are the
following: workflow visualization, call graph visualization, entity-relationship diagrams, biochemical pathways and
network management.
<b>Concept</b>
The layout algorithm runs in three main phases:
<ul>
<li><b>Layering</b> – The nodes are distributed into layers by means of {@link HierarchicLayout#fromScratchLayerer} or
{@link HierarchicLayout#fixedElementsLayerer}, respectively. If the layout orientation is top-to-bottom, the nodes in each layer are arranged horizontally while the
layers are ordered vertically top-to-bottom.
<li><b>Sequencing</b> – The order of the nodes in each layer is determined such that the number of edge crossings is as small as possible. To
specify the sequencing algorithm, use {@link HierarchicLayout#fromScratchSequencer} or {@link HierarchicLayout#fixedElementsSequencer}.
<li><b>Drawing</b> – The layout algorithm assigns the final coordinates to all nodes and routes the edges.
</ul>
<b>Features</b>
This layout algorithm is able to create hierarchic layouts from scratch or add new elements to the existing sketch
drawing incrementally. In order to add elements incrementally to the current sketch or let the algorithm optimize
certain elements in the current sketch, set the layout mode to {@link LayoutMode#INCREMENTAL}. Then register a
{@link IDataProvider} (e.g. use {@link Maps#createHashedDataMap}) with the graph using the {@link HierarchicLayout#INCREMENTAL_HINTS_DP_KEY}
DataProvider key and associate the hints obtained from the {@link IIncrementalHintsFactory} with the elements
to be added incrementally.
{@link HierarchicLayoutNodeLayoutDescriptor} and {@link HierarchicLayoutEdgeLayoutDescriptor} instances can be used for specifying
individual information (e.g. distances or routing styles) for each node and edge in the graph. The descriptors are bound
to the graph using {@link IDataProvider}s registered with {@link HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY}
or {@link HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}. If there is no descriptor assigned to some
nodes/edges, a default descriptor will be used. To set default descriptors use {@link HierarchicLayout#nodeLayoutDescriptor} and {@link HierarchicLayout#edgeLayoutDescriptor}.
{@link HierarchicLayout} supports two approaches to connect edges on a specific side or even an exact location
to a node. {@link PortConstraint}s define a single constraint for the ports of an edge. To realize more complex
port restrictions, several {@link PortCandidate}s or {@link PortCandidateSet}s can be assigned to edges or
nodes. If an edge with registered {@link PortCandidate}s connects to nodes with {@link PortCandidateSet}s,
the layouter will try to match both collections to find an appropriate port. In case there is no matching port
candidate, a {@link PortCandidate} specified for the edge is preferred. For the matching to work properly, the
candidates in both collection need to be the same <em>instances</em>. Since their simultaneous existence at the same node may be
ambiguous, it is not recommended to use a combination of {@link PortConstraint}s and {@link PortCandidate}s
in the same layout. Furthermore,
{@link PortCandidate}s with multiple directions (e.g. EAST or WEST) are not supported. To model that an edge
should connect at one of several sides, define multiple candidates instead, where each candidate has a single
direction.
The edge grouping feature of this layout algorithm is restricted to normal, hierarchic edges. Edges with {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style}
{@link RecursiveEdgeStyle#DIRECTED} or {@link RecursiveEdgeStyle#UNDIRECTED} will not be grouped.
They are also not grouped when enabling {@link HierarchicLayout#automaticEdgeGrouping automatic edge grouping}.
The port grouping feature (see {@link PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY} and
{@link PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY}) allows to specify edges that share the same port location
at their source/target. However, unlike as for edge grouping, edges with port groups will be routed independently
without sharing segments. Port grouping has similar restrictions as the edge grouping feature. In addition, port
grouping is not considered for self-loops.
<h2>Note</h2>
For edges incident to group nodes, the {@link HierarchicLayout} only considers weak
{@link PortConstraint}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout Online Documentation}*/
export class HierarchicLayout {
/**
Creates a new {@link HierarchicLayout} instance with the default settings.
@param [options] The parameters to pass.
@param [options.groupAlignmentPolicy] The group layer alignment strategy used for recursive group layering. This option sets the {@link HierarchicLayout#groupAlignmentPolicy} property on the created object.
@param [options.compactGroups] Whether or not layer compaction for recursive group layering is active. This option sets the {@link HierarchicLayout#compactGroups} property on the created object.
@param [options.componentArrangementPolicy] The policy that specifies how to arrange connected components. This option sets the {@link HierarchicLayout#componentArrangementPolicy} property on the created object.
@param [options.maximumDuration] The time limit (in milliseconds) set for the layout algorithm. This option sets the {@link HierarchicLayout#maximumDuration} property on the created object.
@param [options.recursiveGroupLayering] Whether or not groups are respected during the layering stage. This option sets the {@link HierarchicLayout#recursiveGroupLayering} property on the created object.
@param [options.gridSpacing] The equidistant spacing between the horizontal and vertical grid lines. This option sets the {@link HierarchicLayout#gridSpacing} property on the created object.
@param [options.backLoopRouting] Whether or not reversed edges should be routed as back-loops. This option sets the {@link HierarchicLayout#backLoopRouting} property on the created object.
@param [options.backLoopRoutingForSelfLoops] Whether or not self-loops should be routed in a similar manner as back-loops. This option sets the {@link HierarchicLayout#backLoopRoutingForSelfLoops} property on the created object.
@param [options.automaticEdgeGrouping] Whether or not edges are grouped automatically. This option sets the {@link HierarchicLayout#automaticEdgeGrouping} property on the created object.
@param [options.orthogonalRouting] Whether or not edges should be routed orthogonally. This option sets the {@link HierarchicLayout#orthogonalRouting} property on the created object.
@param [options.integratedEdgeLabeling] Whether or not the layout algorithm reserves space for labels and places them. This option sets the {@link HierarchicLayout#integratedEdgeLabeling} property on the created object.
@param [options.considerNodeLabels] Whether or not the layout algorithm considers node labels when calculating node positions to avoid overlaps. This option sets the {@link HierarchicLayout#considerNodeLabels} property on the created object.
@param [options.minimumLayerDistance] The minimum distance between two adjacent layers. This option sets the {@link HierarchicLayout#minimumLayerDistance} property on the created object.
@param [options.stopAfterLayering] Whether or not to stop the layout algorithm after the layering step. This option sets the {@link HierarchicLayout#stopAfterLayering} property on the created object.
@param [options.stopAfterSequencing] Whether or not to stop the layout algorithm after the sequencing step. This option sets the {@link HierarchicLayout#stopAfterSequencing} property on the created object.
@param [options.nodeToNodeDistance] The minimum distance between two adjacent nodes in one layer. This option sets the {@link HierarchicLayout#nodeToNodeDistance} property on the created object.
@param [options.nodeToEdgeDistance] The minimum distance between an edge and an adjacent node in one layer. This option sets the {@link HierarchicLayout#nodeToEdgeDistance} property on the created object.
@param [options.edgeToEdgeDistance] The minimum distance between two adjacent edges in one layer. This option sets the {@link HierarchicLayout#edgeToEdgeDistance} property on the created object.
@param [options.separateLayers] Whether or not to separate layers. This option sets the {@link HierarchicLayout#separateLayers} property on the created object.
@param [options.fromScratchLayeringStrategy] A predefined layering strategy for the {@link HierarchicLayout#fromScratchLayerer from scratch} layerer. This option sets the {@link HierarchicLayout#fromScratchLayeringStrategy} property on the created object.
@param [options.fromScratchLayerer] The {@link ILayerer} instance that obtains the layering for the nodes if the layout algorithm runs in {@link LayoutMode#FROM_SCRATCH From Scratch mode}. This option sets the {@link HierarchicLayout#fromScratchLayerer} property on the created object.
@param [options.fromScratchSequencer] The {@link ISequencer} instance that calculates the node sequence if the layout algorithm runs in {@link LayoutMode#FROM_SCRATCH From Scratch mode}. This option sets the {@link HierarchicLayout#fromScratchSequencer} property on the created object.
@param [options.nodePlacer] The {@link IHierarchicLayoutNodePlacer} instance that will calculate the final node placement of the layout. This option sets the {@link HierarchicLayout#nodePlacer} property on the created object.
@param [options.fixedElementsLayerer] The {@link ILayerer} instance that obtains the layering for fixed nodes during the incremental layout run. This option sets the {@link HierarchicLayout#fixedElementsLayerer} property on the created object.
@param [options.fixedElementsSequencer] The {@link ISequencer} instance that calculates the sequence of the fixed nodes during the incremental layout run. This option sets the {@link HierarchicLayout#fixedElementsSequencer} property on the created object.
@param [options.layoutMode] The layout mode this layouter should use for upcoming layouts. This option sets the {@link HierarchicLayout#layoutMode} property on the created object.
@param [options.edgeLayoutDescriptor] The {@link HierarchicLayoutEdgeLayoutDescriptor} instance used for all those edges that do not have a specific layout descriptor assigned. This option sets the {@link HierarchicLayout#edgeLayoutDescriptor} property on the created object.
@param [options.nodeLayoutDescriptor] The {@link HierarchicLayoutNodeLayoutDescriptor} instance used for all those nodes that do not have a specific layout descriptor assigned. This option sets the {@link HierarchicLayout#nodeLayoutDescriptor} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link MultiStageLayout#orientationLayoutEnabled} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link MultiStageLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link MultiStageLayout#componentLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout Online Documentation}*/
constructor(options?: {
groupAlignmentPolicy?: GroupAlignmentPolicy | GroupAlignmentPolicyStringValues;
compactGroups?: boolean;
componentArrangementPolicy?: ComponentArrangementPolicy | ComponentArrangementPolicyStringValues;
maximumDuration?: number;
recursiveGroupLayering?: boolean;
gridSpacing?: number;
backLoopRouting?: boolean;
backLoopRoutingForSelfLoops?: boolean;
automaticEdgeGrouping?: boolean;
orthogonalRouting?: boolean;
integratedEdgeLabeling?: boolean;
considerNodeLabels?: boolean;
minimumLayerDistance?: number;
stopAfterLayering?: boolean;
stopAfterSequencing?: boolean;
nodeToNodeDistance?: number;
nodeToEdgeDistance?: number;
edgeToEdgeDistance?: number;
separateLayers?: boolean;
fromScratchLayeringStrategy?: HierarchicLayoutLayeringStrategy | HierarchicLayoutLayeringStrategyStringValues;
fromScratchLayerer?: ILayerer | null;
fromScratchSequencer?: ISequencer | null;
nodePlacer?: IHierarchicLayoutNodePlacer | null;
fixedElementsLayerer?: ILayerer | null;
fixedElementsSequencer?: ISequencer | null;
layoutMode?: LayoutMode | LayoutModeStringValues;
edgeLayoutDescriptor?: HierarchicLayoutEdgeLayoutDescriptor | null;
nodeLayoutDescriptor?: HierarchicLayoutNodeLayoutDescriptor | null;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
orientationLayoutEnabled?: boolean;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
selfLoopRouterEnabled?: boolean;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
componentLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Calculates a hierarchic layout of the given graph.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Delegates the calculation of the hierarchic layout to a configured {@link HierarchicLayoutCore} instance.
{@link HierarchicLayout} applies its own configuration to {@link HierarchicLayoutCore} and also
prepares the graph for layout.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Configures the core layout algorithm with the settings of this {@link HierarchicLayout} instance.
This method is called by {@link HierarchicLayout#applyLayoutCore} before the actual layout is calculated. It
may be overridden in order to manually reconfigure the core layout algorithm.
This implementation will temporarily set a {@link PortCandidateOptimizer} if a {@link IDataProvider}
is registered with {@link PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY} and no {@link HierarchicLayoutCore#portConstraintOptimizer} is assigned.
<h2>Note</h2>
Everything that is added to the graph or the core layout algorithm needs to be removed in
{@link HierarchicLayout#disposeCoreLayout}.
@param graph the input graph
@param coreLayouter the given core layout algorithm instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23configureCoreLayout(LayoutGraph%2CHierarchicLayoutCore) Online Documentation}*/
configureCoreLayout(graph: LayoutGraph | null, coreLayouter: HierarchicLayoutCore | null): void;
/**
Returns a new {@link HierarchicLayoutEdgeLayoutDescriptor} instance that will be used during the various phases of the layout
algorithm to determine the drawing details of the edges of the graph.
This method may be overridden to create a new {@link HierarchicLayoutEdgeLayoutDescriptor} instance with different
configuration settings.
@returns a new {@link HierarchicLayoutEdgeLayoutDescriptor} instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23createEdgeLayoutDescriptor() Online Documentation}*/
createEdgeLayoutDescriptor(): HierarchicLayoutEdgeLayoutDescriptor | null;
/**
Returns a new {@link HierarchicLayoutCore} instance.
This method may be overridden to create a new {@link HierarchicLayoutCore} object with different configuration
settings.
This factory method provides the initial {@link HierarchicLayoutCore} instance.
@returns a new {@link HierarchicLayoutCore} instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23createHierarchicLayoutCore() Online Documentation}*/
createHierarchicLayoutCore(): HierarchicLayoutCore | null;
/**
Returns a {@link IIncrementalHintsFactory} instance that must be used to obtain hints to be associated with
graph elements that should be laid out incrementally.
Use this factory and a {@link IDataProvider} that is registered to the graph using the {@link HierarchicLayout#INCREMENTAL_HINTS_DP_KEY}
key to associate appropriate hints with the graph elements that should be laid out incrementally by the algorithm.
@returns a new {@link IIncrementalHintsFactory} instance
@see HierarchicLayoutCore#createIncrementalHintsFactory
@see HierarchicLayout#INCREMENTAL_HINTS_DP_KEY
@see Graph#addDataProvider
@see HierarchicLayout#layoutMode
@see LayoutMode#INCREMENTAL
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23createIncrementalHintsFactory() Online Documentation}*/
createIncrementalHintsFactory(): IIncrementalHintsFactory | null;
/**
Creates and registers an incremental layer constraint factory for an {@link IGraph}.
As this factory method {@link IMapperRegistry#addMapper adds} an {@link IMapper} to the <code>graph</code>, it has to be {@link ILayerConstraintFactory#dispose disposed} after the layout calculation.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@param graph The graph for which the factory is created
@returns the sequence constraint factory
@deprecated Instead of creating a layer constraint factory directly, the LayerConstraintFactory property of HierarchicLayoutData
should be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23createLayerConstraintFactory(IGraph) Online Documentation}*/
createLayerConstraintFactory(graph: IGraph): ILayerConstraintFactory;
/**
Returns a {@link ILayerConstraintFactory} instance that can be used for specifying layer constraints for the
given graph.
The instance is usually bound to {@link Graph} instance
<code>graph</code>, i.e., if the input graph for the layerer changes, a new instance must be retrieved. This instance can be used for
creating constraints for this graph instance.
You can create an instance without binding it to a graph instance initially by passing a <code>null</code> parameter. In that case,
you <b>must</b> bind the returned instance to the graph, see {@link HierarchicLayout#LAYER_CONSTRAINTS_MEMENTO_DP_KEY}
and {@link ILayerConstraintFactory#memento}.
<b>Warning:</b> {@link ILayerConstraintFactory} instances have to be {@link ILayerConstraintFactory#dispose disposed} after use if a <code>graph</code> was passed in. Disposing
the factory will also remove all constraints previously specified for the factory's associated graph. Creating layering
constraints with a disposed factory will throw an {@link Exception InvalidOperationError}.
<b>Warning:</b>
Creating a new constraint factory for the given graph will render all previously created factories (including any
constraints specified) bound to that graph useless. Therefore, a new constraint factory should only be created after
previously used factories have been properly disposed.
@param graph the input graph
@returns a new {@link ILayerConstraintFactory} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23createLayerConstraintFactory(Graph) Online Documentation}*/
createLayerConstraintFactory(graph: Graph | null): ILayerConstraintFactory | null;
/**
Returns a new {@link HierarchicLayoutNodeLayoutDescriptor} instance that will be used during the various phases of the layout
algorithm to determine the drawing details of the nodes of the graph.
This method may be overridden to create a new {@link HierarchicLayoutNodeLayoutDescriptor} instance with different
configuration settings.
@returns a new {@link HierarchicLayoutNodeLayoutDescriptor} instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23createNodeLayoutDescriptor() Online Documentation}*/
createNodeLayoutDescriptor(): HierarchicLayoutNodeLayoutDescriptor | null;
/**
Creates and registers a sequencer constraint factory for an {@link IGraph}.
As this factory method {@link IMapperRegistry#addMapper adds} an {@link IMapper} to the <code>graph</code>, it has to be {@link ISequenceConstraintFactory#dispose disposed} after the layout calculation.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@param graph The graph.
@returns the sequence constraint factory
@deprecated Instead of creating a sequence constraint factory directly, the SequenceConstraintFactory property of
HierarchicLayoutData should be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23createSequenceConstraintFactory(IGraph) Online Documentation}*/
createSequenceConstraintFactory(graph: IGraph): ISequenceConstraintFactory;
/**
Returns a {@link ISequenceConstraintFactory} instance that can be used for specifying sequence constraints for
the given graph.
For these sequence constraints to have any effect, the {@link ISequencer} that determines the in-layer node
order (<em>sequence</em>) has to support constraints. Both, {@link DefaultLayerSequencer DefaultLayerSequencer} and the incremental sequencer used internally support
sequence constraints.
<b>Warning:</b> {@link ISequenceConstraintFactory} instances have to be {@link ISequenceConstraintFactory#dispose disposed of} after use if a <code>graph</code> was passed in.
Disposing of the factory will also remove all constraints previously specified for the factory's associated graph.
Creating sequence constraints with a disposed of factory will throw an {@link Exception InvalidOperationError}.
<b>Warning:</b>
Creating a new constraint factory for a graph will render all previously created factories (including any constraints
specified) for that graph useless. Therefore a new constraint factory should only be created after previously used
factories have been properly disposed of.
@param graph the input graph
@returns a {@link ISequenceConstraintFactory} instance
@see HierarchicLayout#fromScratchSequencer
@see HierarchicLayout#fixedElementsSequencer
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23createSequenceConstraintFactory(LayoutGraph) Online Documentation}*/
createSequenceConstraintFactory(graph: LayoutGraph | null): ISequenceConstraintFactory | null;
/**
Disposes of the core layout algorithm.
This method is called by {@link HierarchicLayout#applyLayoutCore} after the actual layout is calculated. It
may be overridden in order to revert a custom configuration made in
{@link HierarchicLayout#configureCoreLayout}.
This implementation will remove the {@link PortCandidateOptimizer} that was created in case a {@link IDataProvider}
is registered with {@link PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY} and no {@link HierarchicLayoutCore#portConstraintOptimizer} was initially assigned.
<h2>Note</h2>
Everything that was added to the graph or the core layout algorithm in {@link HierarchicLayout#configureCoreLayout}
needs to be removed here.
@param graph the input graph
@param coreLayouter the given core layout algorithm instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23disposeCoreLayout(LayoutGraph%2CHierarchicLayoutCore) Online Documentation}*/
disposeCoreLayout(graph: LayoutGraph | null, coreLayouter: HierarchicLayoutCore | null): void;
/**
Returns the mirror mask of the {@link MultiStageLayout#orientationLayout orientation layouter}.
It may be overridden in order to configure a different mirror mask.
@returns the mirror mask
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23getMirrorMode() Online Documentation}*/
getMirrorMode(): MirrorModes;
/**
Returns the corresponding original edge instance for a given edge.
This convenience method allows to retrieve the original edge - which is part of the input graph - when customizing the
hierarchic layout algorithm and dealing with potentially artificial dummy edges which are inserted during the
hierarchical layout procedure.
<h2>Note</h2>
If the given edge is already an original edge, it will immediately be returned.
@param edge the edge for which the original input edge is retrieved
@param layoutDataProvider the layout data provider holding layout information of the hierarchical layout
@returns the original input edge corresponding to the given edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23getOriginalEdge(Edge%2CILayoutDataProvider) Online Documentation}*/
static getOriginalEdge(edge: Edge | null, layoutDataProvider: ILayoutDataProvider | null): Edge | null;
/**
Data provider key for specifying the directedness of edges
Generally, the hierarchic layout algorithm assigns nodes to layers such that most of the edges point in the
{@link MultiStageLayout#layoutOrientation main layout direction}. The directedness of an edge specifies whether it should comply with this strategy. More
precisely, a value of <code>1</code> means that the edge should fully comply, a value of <code>-1</code> that it should comply inversely (the edge
should point against the main layout direction), and a value of <code>0</code> means that the direction doesn't matter at all and the
endpoints of the edges may be placed at the same layer. If there are conflicting preferences, edges with higher absolute
values are more likely to point in the desired direction.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the algorithm assumes that all edges have
directedness <code>1</code>.
<b>Warning:</b> Considering the edge directedness requires a special layering algorithm. Hence, {@link HierarchicLayout}
ignores the specified {@link HierarchicLayout#fromScratchLayerer layerer} and automatically switches to the {@link ConstraintIncrementalLayerer} in such
cases.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23EDGE_DIRECTEDNESS_DP_KEY Online Documentation}*/
static EDGE_DIRECTEDNESS_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for specifying individual crossing costs of the edges
The crossing costs of the edges are considered during the crossing minimization phase. The total crossing costs are
minimized using a heuristic where the cost of a crossing between two edges is defined as the product of the edges'
crossing costs.
If no individual crossing costs are defined, each crossing has a cost of
<code>1</code>. A crossing with an edge that has a cost of zero is cost-free and such a crossing will therefore not be avoided.
<b>Warning:</b>
The implemented crossing minimization is a heuristic. Therefore, there is no guarantee that crossing a certain edge is
actually avoided, even though it has high crossing costs.
<b>Warning:</b>
Avoid using very large cost values. Due to the fact that costs need to be multiplied and summed up, this might lead to
overflows and undesired results.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23EDGE_CROSSING_COST_DP_KEY Online Documentation}*/
static EDGE_CROSSING_COST_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for specifying individual crossing costs for vertical borders of group nodes
Like
{@link HierarchicLayout#EDGE_CROSSING_COST_DP_KEY edge crossing costs}, these costs are considered during the crossing minimization phase. The total costs are minimized
using a heuristic where the cost of a crossing between a group node border and an edge is defined as the product of the
respective group border crossing cost and the edge crossing cost.
If no individual crossing costs are defined, the cost for crossing a group node border is
<code>5</code>. This means that by default, a crossing between an edge and a group node border is more expensive than a crossing
between two edges. Also note that a crossing with a group node border that has a cost of zero is cost-free and such a
crossing will therefore not be avoided.
<h2>Note</h2>
In case of a vertical layout orientation (default), these costs only apply to vertical group borders. If the layout
orientation is horizontal (e.g. left-to-right), the costs only apply to horizontal group borders.
<b>Warning:</b>
Avoid using very large cost values. Due to the fact that costs need to be multiplied and summed up, this might lead to
overflows and undesired results.
@see HierarchicLayout#EDGE_CROSSING_COST_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23GROUP_BORDER_CROSSING_COST_DP_KEY Online Documentation}*/
static GROUP_BORDER_CROSSING_COST_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for defining the priority of critical edges
Critical edges highlight different edge paths that are relevant for a user. The layouter tries to vertically align each
node pair that is connected by a critical edge. Conflicts between different critical edges are always resolved in favor
of the higher priority.
Critical edges do not automatically affect the result of the crossing minimization phase. If crossings between critical
edges should be avoided, one can assign individual crossing costs by using
{@link HierarchicLayout#EDGE_CROSSING_COST_DP_KEY}. Critical edges need to get a larger crossing cost than
other, non-critical edges. This is recommended if critical edges should not only highlight a relevant path but also
crossings with them should be avoided. Note, however, that the crossing minimization is only a heuristic and there is no
guarantee for an optimal solution in all cases.
<h2>Note</h2>
Critical edges are always connected to the center port. Hence, the edge distribution is no longer uniform.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23CRITICAL_EDGE_PRIORITY_DP_KEY Online Documentation}*/
static CRITICAL_EDGE_PRIORITY_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for specifying the thickness of the edges
The specified non-negative thickness is considered when calculating minimum distances so that there are no overlaps
between edges and other graph elements. By default, each edge has thickness <code>0</code>.
<h2>Note</h2>
The port assignments {@link HierarchicLayoutPortAssignmentMode#ON_GRID on grid} and {@link HierarchicLayoutPortAssignmentMode#ON_SUBGRID on sub-grid} ignore the thickness of edges.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23EDGE_THICKNESS_DP_KEY Online Documentation}*/
static EDGE_THICKNESS_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for specifying the bus to which an edge belongs to
All edges with the same {@link HierarchicLayoutBusDescriptor} instance adjacent to a specific node (the root) and having the
same direction are arranged in a compact, bus-like way. The {@link HierarchicLayout#EDGE_DIRECTEDNESS_DP_KEY edge directedness} is also considered here. The direction
defines the placement of the bus structure in the following way:
<ul>
<li>Bus nodes connected by out-edges are placed in the layers starting with the layer that comes directly after the layer of
the root (e.g. below the root for top-to-bottom layout orientation).
<li>Bus nodes connected by in-edges are placed in the layers ending with the layer that comes directly before the layer of
the root (e.g. above the root for top-to-bottom layout orientation).
</ul>
The bus substructures are arranged using a style that yields more compact layout results. Edges to the child nodes, the
so-called bus nodes, are routed using a shared segment that connects to the common root node. The bus nodes are arranged
using several layers above or below the root node such that the whole substructure has a compact area. By default, each
bus layer contains equally many bus nodes, bus nodes of adjacent layers are center-aligned and the shared bus segment is
routed in the middle of the nodes. This produces compact, symmetric arrangements of the bus structures.
The arrangement can also be customized for each bus individually, by using the settings offered by
{@link HierarchicLayoutBusDescriptor}. It allows to configure the number of bus nodes before and after the common bus
segment.
There apply some restrictions for bus structures:
<ul>
<li>Only nodes that are in the same {@link PartitionCellId partition grid cell} or same {@link SwimlaneDescriptor swimlane} as the root node are included in the bus
structure.
<li>The bus nodes must all belong to the same group node. Also, a group node can not be part of a bus structure; neither as
bus element nor as root.
<li>Bus structures can not be nested. That means that if a node is a bus node it can not be a root node of another bus at
the same time.
<li>For bus edges, the integrated edge labeling feature does not place port labels as close to the actual port as it does
for other edges (see {@link LabelPlacements#AT_SOURCE_PORT} and {@link LabelPlacements#AT_TARGET_PORT}).
<li>Edge grouping constraints at the side of the bus root node are ignored for bus edges; the grouping induced by the bus
structure is considered to be stronger.
<li>If in {@link LayoutMode#INCREMENTAL incremental layout mode}, the bus edges are always treated in an incremental way such that the bus-style routing
remains consistent (e.g. if a fixed bus edge contains bends that suggest another route, they are ignored). The reason is
that the bus-style layout of the structures has higher priority. Furthermore, the location of the bus segment relative
to the elements is not derived from the sketch but depends on the settings {@link HierarchicLayoutBusDescriptor#maximumNodesBeforeBus} and {@link HierarchicLayoutBusDescriptor#maximumNodesAfterBus} as well as the before/after constraints
(see
{@link HierarchicLayoutBusDescriptor#PLACE_BEFORE_BUS_DP_KEY}). This means that it can happen that a fixed bus node which was
placed before the bus in the input drawing may be placed after the bus if the bus was enlarged due to the insertion of
incremental bus nodes.
<li>It is not recommended to define buses inside a group node which has an
{@link IIncrementalHintsFactory#createIncrementalGroupHint incremental group hint}. Fixed bus nodes inside such a group are not necessarily kept fix but are treated like an
incrementally inserted node.
</ul>
<h2>Note</h2>
When two parallel edges have the same {@link HierarchicLayoutBusDescriptor}, only one edge will be treated as a bus edge.
<b>Warning:</b> If layering constraints are specified using
{@link ILayerConstraintFactory}, they are not considered for bus nodes because the bus structure is layered
independently.
<b>Warning:</b>
If bus nodes have edges to other bus nodes (from the same or from another bus), then these edges are not considered in
the same way during the layering as other edges; in consequence, it can happen that both bus nodes end up in the same
final layer and thus the edge becomes a same-layer edge. The same can happen if a group has an edge to a bus node where
the bus node is outside said group but the root of the bus is a child of it.
@see HierarchicLayoutBusDescriptor
@see HierarchicLayoutBusDescriptor#PLACE_BEFORE_BUS_DP_KEY
@see HierarchicLayoutBusDescriptor#ROOT_OFFSET_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23BUS_DESCRIPTOR_DP_KEY Online Documentation}*/
static BUS_DESCRIPTOR_DP_KEY: EdgeDpKey<HierarchicLayoutBusDescriptor> | null;
/**
Data provider key for specifying incremental hints
Incremental hints are created using an {@link HierarchicLayout#createIncrementalHintsFactory incremental hints factory}.
@see HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY
@see IIncrementalHintsFactory
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23INCREMENTAL_HINTS_DP_KEY Online Documentation}*/
static INCREMENTAL_HINTS_DP_KEY: GraphObjectDpKey<any> | null;
/**
Data acceptor key for publishing the layer IDs for all nodes in the graph
For each non-group node, the index of the node's layer will be stored in this {@link IDataAcceptor} after the
layout run.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the layer information of the nodes is dropped.
@see HierarchicLayoutCore#LAYER_INDEX_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23LAYER_INDEX_DP_KEY Online Documentation}*/
static LAYER_INDEX_DP_KEY: NodeDpKey<number> | null;
/**
Data acceptor key for publishing the index inside their layer for all nodes in the graph
For each non-group node, the index (position) of the node within its layer will be stored in this {@link IDataAcceptor}
after the layout run.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the sequence information of the nodes is dropped.
@see HierarchicLayoutCore#SEQUENCE_INDEX_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23SEQUENCE_INDEX_DP_KEY Online Documentation}*/
static SEQUENCE_INDEX_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for defining swimlanes for the nodes
The layout algorithm will arrange nodes in swimlanes according to the registered descriptors.
Layout information about the swimlanes is finally written back to the descriptor instances. Instances can be shared
among multiple nodes in the same lane.
@see HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23SWIMLANE_DESCRIPTOR_DP_KEY Online Documentation}*/
static SWIMLANE_DESCRIPTOR_DP_KEY: NodeDpKey<SwimlaneDescriptor> | null;
/**
Data provider key for associating an alternative bounds with the collapsed/expanded group
When running in {@link LayoutMode#INCREMENTAL incremental}
{@link HierarchicLayout#layoutMode layout mode}, the alternative bounds of the collapsed/expanded group will be used during the layering and sequencing
phase of the algorithm.
<h2>Note</h2>
To further improve the results you should also specify {@link HierarchicLayout#ALTERNATIVE_EDGE_PATH_DP_KEY alternative edge paths} for edges adjacent to such groups. In
addition, {@link HierarchicLayout#recursiveGroupLayering recursive group layering} should be enabled and {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} should be set to
{@link RecursiveEdgeStyle#DIRECTED}. Edges that end at a group itself or a folder node should not have {@link PortCandidate port candidates}
or {@link PortConstraint port constraints} at this end (because this may destroy the recursive directed edge style required to obtain stable
results).
<b>Warning:</b> The algorithm assumes that there is at most one node with associated alternative bounds.
@see HierarchicLayout#ALTERNATIVE_EDGE_PATH_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23ALTERNATIVE_GROUP_BOUNDS_DP_KEY Online Documentation}*/
static ALTERNATIVE_GROUP_BOUNDS_DP_KEY: NodeDpKey<YRectangle> | null;
/**
Data provider key for associating alternative paths for edges connecting to groups, group content or folder nodes
When running in {@link LayoutMode#INCREMENTAL incremental}
{@link HierarchicLayout#layoutMode layout mode}, the alternative edge paths are considered during the routing of fixed (i.e., non-incremental) edges.
The alternative paths should be used in conjunction with {@link HierarchicLayout#ALTERNATIVE_GROUP_BOUNDS_DP_KEY alternative group bounds} to achieve more stable layout results
when collapsing/expanding a group node as follows:
<ol>
<li>Collapsing: edges adjacent to the group itself and edges where one of the endpoints (source/target) lies inside the
group should get the path <em>before</em> collapsing the group as alternative path. If both endpoints are either inside or outside the
group, no alternative path is required.
<li>Expanding: edges adjacent to the expanded folder node (which is now a group) should get the path <em>before</em> expanding as
alternative path.
</ol>
@see HierarchicLayout#ALTERNATIVE_GROUP_BOUNDS_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23ALTERNATIVE_EDGE_PATH_DP_KEY Online Documentation}*/
static ALTERNATIVE_EDGE_PATH_DP_KEY: EdgeDpKey<YPointPath> | null;
/**
Data provider key for marking folder nodes
When using recursive edge styles in incremental mode, edges will also start at the bottom and end at the top of marked
folder nodes. This will keep the edge routes more stable since the connection sides won't change.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23FOLDER_NODES_DP_KEY Online Documentation}*/
static FOLDER_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for specifying custom sub-components of the input graph
A sub-component is arranged by another {@link ILayoutAlgorithm layout algorithm} and its layout is finally embedded into the global hierarchical
layout. This feature allows, for example, to highlight special sub-structures by arranging them in a different fashion.
The sub-layout algorithm must be specified by a {@link IDataProvider} registered with the graph with key
{@link HierarchicLayout#SUB_COMPONENT_LAYOUT_ALGORITHM_DP_KEY}. The first found {@link ILayoutAlgorithm}
instance bound to any node of a sub-component is applied as algorithm for that sub-component.
Carefully note that nodes of a sub-component can be connected to nodes of another sub-component as well as to nodes that
are not part of any sub-component. Edges that model these connections are so-called
<em>inter-edges</em>. For inter-edges there apply some restrictions with respect to the supported features:
<ul>
<li>Their general style can somewhat differ from other, normal hierarchical edges. They do not fully comply with all {@link HierarchicLayoutRoutingStyle}
settings.
<li>The {@link HierarchicLayout#backLoopRouting backloop routing} style is not applied for the part of an inter-edge that connects to a sub-component node.
<li>The {@link HierarchicLayoutEdgeLayoutDescriptor#minimumFirstSegmentLength minimum first segment length} and the {@link HierarchicLayoutEdgeLayoutDescriptor#minimumLastSegmentLength minimum last segment length} can not always be fulfilled for the side of an
inter-edge that lies inside a sub-component. It is still kept on the side outside of sub-components.
<li>The {@link HierarchicLayout#nodeToEdgeDistance node-to-edge distance} can not always be kept between inter-edges and nodes inside a sub-component. This will mostly
appear if a sub-component has a rather large number of inter-edges connecting to it.
<li>The {@link HierarchicLayout#integratedEdgeLabeling integrated edge labeling} does not guarantee full support for all {@link PreferredPlacementDescriptor}
configurations for inter-edges. This affects the side of the inter-edge that lies inside a sub-component and when the
label should be placed near the component node, i.e., using settings {@link LabelPlacements#AT_SOURCE},
{@link LabelPlacements#AT_SOURCE_PORT}, {@link LabelPlacements#AT_TARGET} or
{@link LabelPlacements#AT_TARGET_PORT}.
</ul>
Sub-components can be defined on grouped graphs with the following restrictions:
<ul>
<li>All sub-component nodes must be on the <em>same hierarchy level</em> if the group node itself is not part of the sub-component.
<li>If a group is assigned to a sub-component, <em>all</em> its descendants (including other group nodes) must be in that component,
too.
</ul>
When running in
{@link LayoutMode#INCREMENTAL incremental mode}, it is still up to the sub-layout algorithm to decide whether to treat a sub-component element as
incremental or fixed. If the given layout of the sub-component should be considered, then the from-sketch mode of the
used layout algorithm needs to be enabled (if such a mode is supported).
Layering and sequencing constraints are supported for sub-components in the following way: all constraints from the
sub-component nodes are transferred to the component itself. Constraints between nodes of the same component are
ignored. In the same way, a given layering provided by {@link GivenLayersLayerer} is supported for
sub-components.
<b>Warning:</b>
Sub-components may not be nested, that is, a sub-component is not allowed to contain nodes that belong to yet another
sub-component. Nesting of sub-components results in an exception.
<b>Warning:</b> When using a {@link PartitionGrid}, all nodes of a sub-component need to be assigned to the same {@link PartitionCellId}
or, alternatively, not be assigned to a cell at all. Otherwise, an exception will the thrown. Furthermore, it is not
possible that the layout of a sub-component uses another partition grid instance at the same time - in other words,
nested partition grids are not supported.
<h2>Note</h2>
For the layout of a sub-component, the specified sub-component layout algorithm is fully responsible. Thus, the
supported features and layout style are defined solely by that algorithm.
@see HierarchicLayout#SUB_COMPONENT_LAYOUT_ALGORITHM_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23SUB_COMPONENT_ID_DP_KEY Online Documentation}*/
static SUB_COMPONENT_ID_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for specifying a layout algorithm for each sub-component
A sub-component is arranged by the layout algorithm defined by this mapping and its layout is finally embedded into the
global hierarchical layout. The first non-<code>null</code> instance mapped to any sub-component node is used for the whole
sub-component. For more details about this feature and how to define sub-components, please see
{@link HierarchicLayout#SUB_COMPONENT_ID_DP_KEY}.
@see HierarchicLayout#SUB_COMPONENT_ID_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23SUB_COMPONENT_LAYOUT_ALGORITHM_DP_KEY Online Documentation}*/
static SUB_COMPONENT_LAYOUT_ALGORITHM_DP_KEY: NodeDpKey<ILayoutAlgorithm> | null;
/**
Data provider key for specifying whether or not the ports of edges incident to a specific group node should be uniformly
distributed
This setting only affects the two sides of a group node that are in flow and against the flow of the layout orientation.
For example, for orientation {@link LayoutOrientation#TOP_TO_BOTTOM} only the edges incident to the top and bottom
group node side are considered but not the edges at the left and right side.
If no {@link IDataProvider} is registered with this key, then the ports at group nodes are not explicitly
distributed in a uniform way.
The uniform group port assignment considers the {@link HierarchicLayout#edgeToEdgeDistance edge-to-edge distance} such that two ports keep the specified distance to
each other. Group nodes may in consequence be enlarged to satisfy that constraints, potentially generating less compact
layouts. The distance from the group node border to the first/last port on each group side may be influenced by using
{@link GroupingKeys#GROUP_NODE_INSETS_DP_KEY group node insets}.
Importantly, it is not guaranteed that the group node ports of a specific side can be uniformly distributed. In the
following cases ports of a specific side are not uniformly distributed:
<ol>
<li>There are edges that cross the group node border because they connect to a child node of the group.
<li>There is a self-loop at the group node which has its source and target port at the same group node side.
</ol>
<b>Warning:</b> Enabling uniform group ports for at least one group node may significantly increase the runtime of the algorithm. Also,
the uniform distribution may be skipped in case that the {@link HierarchicLayout#maximumDuration maximum duration} is restricted and there is no time left.
<h2>Note</h2>
Forcing group nodes to have uniform ports can have several side-effects and also affect other parts of the layout. In
general, results can become less compact and there might be more bends, because bends may be introduced where they would
not be necessary when ports are not uniformly distributed.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23UNIFORM_PORT_ASSIGNMENT_GROUPS_DP_KEY Online Documentation}*/
static UNIFORM_PORT_ASSIGNMENT_GROUPS_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for storing the constraint graph
A <code>v1 before v2</code> constraint is represented as an edge between the representatives of <code>v1</code> and <code>v2</code> in the constraint graph.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23SEQUENCE_CONSTRAINTS_MEMENTO_DP_KEY Online Documentation}*/
static SEQUENCE_CONSTRAINTS_MEMENTO_DP_KEY: GraphDpKey<any> | null;
/**
Data provider key for storing the constraint graph
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23LAYER_CONSTRAINTS_MEMENTO_DP_KEY Online Documentation}*/
static LAYER_CONSTRAINTS_MEMENTO_DP_KEY: GraphDpKey<any> | null;
/**
Gets or sets the group layer alignment strategy used for recursive group layering.
@throws {Exception} <b>(ArgumentError)</b> if an unknown group alignment policy is given
@see HierarchicLayout#recursiveGroupLayering
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23groupAlignmentPolicy Online Documentation}*/
groupAlignmentPolicy: GroupAlignmentPolicy;
/**
Gets or sets whether or not layer compaction for recursive group layering is active.
The number of node layers is decreased if possible without reversing edge directions. The resulting layering tries to
keep the layer span of a group node low (i.e., the height of a group for layout orientation top-to-bottom). Note that a
lower number of layers usually leads to an increased overall width (for layout orientation top-to-bottom).
<b>Warning:</b>
Group compaction only applies when recursive group layering is enabled. During non-recursive layering, groups are
ignored completely.
@see HierarchicLayout#recursiveGroupLayering
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23compactGroups Online Documentation}*/
compactGroups: boolean;
/**
Gets or sets the policy that specifies how to arrange connected components.
@throws {Exception} <b>(ArgumentError)</b> if the specified policy does not match a default component arrangement policy
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23componentArrangementPolicy Online Documentation}*/
componentArrangementPolicy: ComponentArrangementPolicy;
/**
Gets or sets the time limit (in milliseconds) set for the layout algorithm.
Values have to be greater or equal to <code>0</code>.
<h2>Note</h2>
Restricting the maximum duration may result in a lower layout quality. Furthermore, the real runtime may exceed the
maximum duration since the layout algorithm still has to find a valid solution.
<h2>Note</h2>
The layout algorithm takes also into consideration the time limits of the default {@link ILayerer}, {@link ISequencer}
and {@link IHierarchicLayoutNodePlacer} instances (if any) and will try not to exceed the given values.
<h2>Note</h2>
The layout algorithm takes also into consideration the time limits of the default {@link ILayerer}, {@link ISequencer}
and {@link IHierarchicLayoutNodePlacer} instances (if any) and will try to not exceed the given values.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets whether or not groups are respected during the layering stage.
If this option is enabled, groups are layered recursively, i.e.
<ul>
<li>nodes in the same group always occupy adjacent layers
<li>layer intervals spanned by different group nodes are either disjoint or are nested
</ul>
If it is disabled, group information is ignored for the layering step.
If the graph is flat, this setting is ignored.
<b>Warning:</b> In case group compaction is enabled, groups are layered non-recursively.
<b>Warning:</b> The recursive processing of groups restricts the valid layering results which has to be considered when specifying
{@link ILayerConstraintFactory layer constraints}. To ensure that such constraints are correctly considered, it is recommended to only specify them
between elements of the same grouping hierarchy (i.e., elements with the same parent group).
@see HierarchicLayout#compactGroups
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23recursiveGroupLayering Online Documentation}*/
recursiveGroupLayering: boolean;
/**
Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
When the spacing is negative or zero, no grid is defined, otherwise nodes and edges are placed on multiples of the grid
spacing.
<b>Warning:</b> The grid feature doesn't work together with exact (layer/sequence) coordinate hints.
@see IIncrementalHintsFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23gridSpacing Online Documentation}*/
gridSpacing: number;
/**
Gets or sets whether or not reversed edges should be routed as back-loops.
When this option is enabled, back-loops attach to the same node sides as the other edges, i.e., for layout orientation
top-to-bottom, they leave their source at the bottom and enter their target at the top.
<h2>Note</h2>
Port constraints and port candidates are still considered.
<h2>Note</h2>
The backloop routing considers the specified {@link HierarchicLayout#EDGE_DIRECTEDNESS_DP_KEY edge directedness}. This means that undirected edges (edges with a
directedness value of
<code>0</code>) are never routed as back-loops because they have no dedicated source/target node. Furthermore, for edges with a
negative directedness value the source and target are interchanged.
<b>Warning:</b> This setting is ignored for edges with {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge styles} {@link RecursiveEdgeStyle#DIRECTED} and
{@link RecursiveEdgeStyle#UNDIRECTED}, including edges where source or target are not inside a group node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23backLoopRouting Online Documentation}*/
backLoopRouting: boolean;
/**
Gets or sets whether or not self-loops should be routed in a similar manner as back-loops.
Self-loops will start at the bottom of their attached nodes and end at the top of them.
<b>Warning:</b> Back-loop routing for self-loops only applies if back-loop routing is enabled.
<h2>Note</h2>
Port constraints and port candidates are still considered.
@see HierarchicLayout#backLoopRouting
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23backLoopRoutingForSelfLoops Online Documentation}*/
backLoopRoutingForSelfLoops: boolean;
/**
Gets or sets whether or not edges are grouped automatically.
The automatic edge grouping tries to group a high number of edges without changing the semantic of the graph, i.e., it
groups edges either at a common source node or a common target node. Edge groupings often allow more compact layouts
since grouped edges are routed in a bus-style manner.
<h2>Note</h2>
Edges are only grouped at their source (target) node if they do not have {@link PortConstraint port constraints}, {@link PortCandidate port candidates} or port group
ids (see {@link PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY} and
{@link PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY}) at this node. Furthermore, edges cannot be grouped at a
node with specified {@link PortCandidateSet port candidate set}.
<b>Warning:</b> User specified edge groups are not considered.
<b>Warning:</b> This setting is ignored for edges with {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge styles} {@link RecursiveEdgeStyle#DIRECTED} and
{@link RecursiveEdgeStyle#UNDIRECTED}, including edges where source or target are not inside a group node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23automaticEdgeGrouping Online Documentation}*/
automaticEdgeGrouping: boolean;
/**
Gets or sets whether or not edges should be routed orthogonally.
When orthogonal routing is enabled, all resulting edge paths will be composed of vertical and horizontal segments,
only.
<h2>Note</h2>
This is a convenience method that sets the routing style for the {@link HierarchicLayout#edgeLayoutDescriptor default edge layout descriptor} to orthogonal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23orthogonalRouting Online Documentation}*/
orthogonalRouting: boolean;
/**
Gets or sets whether or not the layout algorithm reserves space for labels and places them.
To define the desired placement for each label add a {@link PreferredPlacementDescriptor} on
{@link IEdgeLabelLayout}.
This method is a convenience method that assures that the {@link MultiStageLayout#labeling labeling algorithm} is of type {@link LabelLayoutTranslator}
and {@link LabelLayoutTranslator#translateEdgeLabels} is set to <code>true</code>.
<b>Warning:</b> When enabling this property, any previously specified {@link MultiStageLayout#labeling labeling algorithm} will be overwritten and when disabling it, any
currently specified labeling algorithm will be disabled via property
{@link MultiStageLayout#labelingEnabled}. Therefore, it is recommended to only use this convenience property instead of manually changing the mentioned other
properties.
@throws {Exception} <b>(InvalidOperationError)</b> if integrated labeling should be enabled but the current {@link MultiStageLayout#labeling labeling algorithm} is not of type
{@link LabelLayoutTranslator}
@throws {Exception} <b>(InvalidOperationError)</b> if no properly configured {@link LabelLayoutTranslator} is registered even though integrated labeling was enabled
earlier (can happen when manually specifying the {@link MultiStageLayout#labeling labeling algorithm}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23integratedEdgeLabeling Online Documentation}*/
integratedEdgeLabeling: boolean;
/**
Gets or sets whether or not the layout algorithm considers node labels when calculating node positions to avoid
overlaps.
This method is a convenience method that assures that the {@link MultiStageLayout#labeling labeling algorithm} is of type {@link LabelLayoutTranslator}
and {@link LabelLayoutTranslator#translateNodeLabels} is set to <code>true</code>.
<b>Warning:</b>
Enabling this option may overwrite the current labeling algorithm. Hence, to combine this option with a generic edge
labeling algorithm, the generic labeling has to be applied in an additional step after calculating the layout.
@throws {Exception} <b>(InvalidOperationError)</b> if this property should be enabled but the current {@link MultiStageLayout#labeling labeling algorithm} is not of type
{@link LabelLayoutTranslator}
@throws {Exception} <b>(InvalidOperationError)</b> if no properly configured {@link LabelLayoutTranslator} is registered even though this property was enabled
earlier (can happen when manually specifying the {@link MultiStageLayout#labeling labeling algorithm}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets the minimum distance between two adjacent layers.
The specified distance should have a non-negative value.
@throws {Exception} <b>(ArgumentError)</b> if the distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23minimumLayerDistance Online Documentation}*/
minimumLayerDistance: number;
/**
Gets or sets whether or not to stop the layout algorithm after the layering step.
By then, each node will be assigned to a layer. Since the sequencing and drawing phases are skipped, the order of the
nodes within a layer matches the initial order of the nodes, edges won't be routed and the nodes won't get coordinates
assigned.
The calculated layering information can be retrieved from the {@link IDataProvider} registered with the key
{@link HierarchicLayout#LAYER_INDEX_DP_KEY}.
<h2>Note</h2>
The bounds of group nodes depend on the locations of their content and the routes of connecting edges. Therefore, they
are not yet calculated after the layering phase and will have width and height <code>1</code>. Similarly, {@link RowDescriptor rows} and {@link ColumnDescriptor columns} of a
partition grid do not get proper computed positions and computed height/width values.
@see HierarchicLayout#LAYER_INDEX_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23stopAfterLayering Online Documentation}*/
stopAfterLayering: boolean;
/**
Gets or sets whether or not to stop the layout algorithm after the sequencing step.
By then, each node will be assigned to a layer and will have a place in the sequence of nodes in this layer. Since the
drawing phase is skipped, edges won't be routed and the nodes won't get coordinates assigned.
The calculated layering and sequencing information can be retrieved from the {@link IDataProvider}s registered
with the keys {@link HierarchicLayout#LAYER_INDEX_DP_KEY} and
{@link HierarchicLayout#SEQUENCE_INDEX_DP_KEY}.
<h2>Note</h2>
The bounds of group nodes depend on the locations of their content and the routes of connecting edges. Therefore, they
are not yet calculated after the layering phase and will have width and height <code>1</code>. Similarly, {@link RowDescriptor rows} and {@link ColumnDescriptor columns} of a
partition grid do not get proper computed positions and computed height/width values.
@see HierarchicLayout#SEQUENCE_INDEX_DP_KEY
@see HierarchicLayout#LAYER_INDEX_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23stopAfterSequencing Online Documentation}*/
stopAfterSequencing: boolean;
/**
Gets or sets the minimum distance between two adjacent nodes in one layer.
All values have to be greater than or equal to <code>0</code>.
<h2>Note</h2>
If nodes should use different distances, you can map them to different {@link HierarchicLayoutNodeLayoutDescriptor} instances
(see {@link HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY}) with custom {@link HierarchicLayoutNodeLayoutDescriptor#minimumDistance minimum distance} values.
<h2>Note</h2>
This property delegates to the according property of {@link DefaultDrawingDistanceCalculator}. It must not be
used if a different {@link IDrawingDistanceCalculator} instance is {@link HierarchicLayoutCore#drawingDistanceCalculator currently used}.
@throws {Exception} <b>(ArgumentError)</b> if the distance is negative
@throws {Exception} <b>(InvalidOperationError)</b> if the current {@link IDrawingDistanceCalculator} is not an instance of
{@link DefaultDrawingDistanceCalculator}
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23nodeToNodeDistance Online Documentation}*/
nodeToNodeDistance: number;
/**
Gets or sets the minimum distance between an edge and an adjacent node in one layer.
All values have to be greater than or equal to <code>0</code>.
<h2>Note</h2>
If nodes/edges should use different distances, you can map them to different {@link HierarchicLayoutNodeLayoutDescriptor} /{@link HierarchicLayoutEdgeLayoutDescriptor}
instances (see {@link HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY} and
{@link HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}) with custom minimum distance values.
<h2>Note</h2>
This property delegates to the according property of {@link DefaultDrawingDistanceCalculator}. It must not be
used if a different {@link IDrawingDistanceCalculator} instance is {@link HierarchicLayoutCore#drawingDistanceCalculator currently used}.
@throws {Exception} <b>(ArgumentError)</b> if the distance is negative
@throws {Exception} <b>(InvalidOperationError)</b> if the current {@link IDrawingDistanceCalculator} is not an instance of
{@link DefaultDrawingDistanceCalculator}
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23nodeToEdgeDistance Online Documentation}*/
nodeToEdgeDistance: number;
/**
Gets or sets the minimum distance between two adjacent edges in one layer.
All values have to be greater than or equal to <code>0</code>.
<h2>Note</h2>
If edges should use different distances, you can map them to different {@link HierarchicLayoutEdgeLayoutDescriptor} instances
(see {@link HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}) with custom {@link HierarchicLayoutEdgeLayoutDescriptor#minimumDistance minimum distance} values.
<h2>Note</h2>
This property delegates to the according property of {@link DefaultDrawingDistanceCalculator}. It must not be
used if a different {@link IDrawingDistanceCalculator} instance is {@link HierarchicLayoutCore#drawingDistanceCalculator currently used}.
@throws {Exception} <b>(ArgumentError)</b> if the distance is negative
@throws {Exception} <b>(InvalidOperationError)</b> if the current {@link IDrawingDistanceCalculator} is not an instance of
{@link DefaultDrawingDistanceCalculator}
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23edgeToEdgeDistance Online Documentation}*/
edgeToEdgeDistance: number;
/**
Gets or sets whether or not to separate layers.
In case layers are not separated, nodes of a layer may extend into adjacent layers. Otherwise, nodes of different layers
are strictly separated, i.e., layers are placed below each other.
<h2>Note</h2>
The alignment of nodes within their layer is not affected by this property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23separateLayers Online Documentation}*/
separateLayers: boolean;
/**
Gets the {@link DefaultDrawingDistanceCalculator} that is registered with the layout algorithm by default.
@throws {Exception} <b>(InvalidOperationError)</b> if the current instance returned by {@link HierarchicLayoutCore#drawingDistanceCalculator} is not an instance of {@link DefaultDrawingDistanceCalculator}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23defaultDrawingDistanceCalculator Online Documentation}*/
readonly defaultDrawingDistanceCalculator: DefaultDrawingDistanceCalculator | null;
/**
Gets the current layout algorithm instance.
By default, this method will return a {@link HierarchicLayoutCore} instance created with
{@link HierarchicLayout#createHierarchicLayoutCore}.
@see HierarchicLayout#createHierarchicLayoutCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23hierarchicLayoutCore Online Documentation}*/
readonly hierarchicLayoutCore: HierarchicLayoutCore | null;
/**
Gets or sets a predefined layering strategy for the {@link HierarchicLayout#fromScratchLayerer from scratch} layerer.
The layouter assigns the nodes to separate layers. The nodes within each layer will be placed on the same horizontal
layer. The layers will be arranged vertically starting with the small-numbered layers.
An important layering strategy for the hierarchic layout style is called Hierarchical Layering. A hierarchical layering
tries to assign nodes to layers in a way such that as much edges of the graph as possible will point to the main layout
direction, i.e., the start nodes of the edges will be in a layer with a smaller number than the corresponding end nodes.
Also, a hierarchical layering will never put two connected nodes in the same layer.
This method wraps the internal implementations into a {@link MultiComponentLayerer} instance so that it is
possible to specify the behavior of the algorithm if the {@link MultiStageLayout#componentLayoutEnabled component layouter} is disabled.
@throws {Exception} <b>(ArgumentError)</b> if an unknown strategy is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23fromScratchLayeringStrategy Online Documentation}*/
fromScratchLayeringStrategy: HierarchicLayoutLayeringStrategy;
/**
Gets or sets the {@link ILayerer} instance that obtains the layering for the nodes if the layout algorithm
runs in {@link LayoutMode#FROM_SCRATCH From Scratch mode}.
If the graph consists of multiple components, then the {@link ILayerer} instance should be wrapped in
{@link MultiComponentLayerer}.
@throws {Exception} <b>(ArgumentError)</b> if the {@link ILayerer} is <code>null</code>
@see HierarchicLayout#layoutMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23fromScratchLayerer Online Documentation}*/
fromScratchLayerer: ILayerer | null;
/**
Gets or sets the {@link ISequencer} instance that calculates the node sequence if the layout algorithm runs in
{@link LayoutMode#FROM_SCRATCH From Scratch mode}.
@throws {Exception} <b>(ArgumentError)</b> if the {@link ISequencer} is <code>null</code>
@see HierarchicLayout#fromScratchLayerer
@see HierarchicLayout#layoutMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23fromScratchSequencer Online Documentation}*/
fromScratchSequencer: ISequencer | null;
/**
Gets or sets the {@link IHierarchicLayoutNodePlacer} instance that will calculate the final node placement of the layout.
@throws {Exception} <b>(ArgumentError)</b> if the {@link IHierarchicLayoutNodePlacer} is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23nodePlacer Online Documentation}*/
nodePlacer: IHierarchicLayoutNodePlacer | null;
/**
Gets or sets the {@link ILayerer} instance that obtains the layering for fixed nodes during the incremental
layout run.
@throws {Exception} <b>(ArgumentError)</b> if the {@link ILayerer} is <code>null</code>
@see HierarchicLayout#layoutMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23fixedElementsLayerer Online Documentation}*/
fixedElementsLayerer: ILayerer | null;
/**
Gets or sets the {@link ISequencer} instance that calculates the sequence of the fixed nodes during the
incremental layout run.
@throws {Exception} <b>(ArgumentError)</b> if the given {@link ISequencer} is <code>null</code>
@see HierarchicLayout#layoutMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23fixedElementsSequencer Online Documentation}*/
fixedElementsSequencer: ISequencer | null;
/**
Gets or sets the layout mode this layouter should use for upcoming layouts.
Depending on the mode the layout algorithm will use different {@link ILayerer} and {@link ISequencer}
implementations.
@throws {Exception} <b>(ArgumentError)</b> if the given layout mode is unknown
@see HierarchicLayout#fromScratchLayerer
@see HierarchicLayout#fromScratchSequencer
@see HierarchicLayout#fixedElementsLayerer
@see HierarchicLayout#fixedElementsSequencer
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23layoutMode Online Documentation}*/
layoutMode: LayoutMode;
/**
Gets or sets the {@link HierarchicLayoutEdgeLayoutDescriptor} instance used for all those edges that do not have a specific
layout descriptor assigned.
By default, this method will return a {@link HierarchicLayoutEdgeLayoutDescriptor} instance created with
{@link HierarchicLayout#createEdgeLayoutDescriptor}.
@throws {Exception} <b>(ArgumentError)</b> if the {@link HierarchicLayoutEdgeLayoutDescriptor} is <code>null</code>
@see HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23edgeLayoutDescriptor Online Documentation}*/
edgeLayoutDescriptor: HierarchicLayoutEdgeLayoutDescriptor | null;
/**
Gets or sets the {@link HierarchicLayoutNodeLayoutDescriptor} instance used for all those nodes that do not have a specific
layout descriptor assigned.
By default, this method will return a {@link HierarchicLayoutNodeLayoutDescriptor} instance created with
{@link HierarchicLayout#createNodeLayoutDescriptor}.
@throws {Exception} <b>(ArgumentError)</b> if the given {@link HierarchicLayoutNodeLayoutDescriptor} is <code>null</code>
@see HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY
@see HierarchicLayout#createNodeLayoutDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayout%23nodeLayoutDescriptor Online Documentation}*/
nodeLayoutDescriptor: HierarchicLayoutNodeLayoutDescriptor | null;
static $class: Class;
private $_HierarchicLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HierarchicLayout;
}
export interface HierarchicLayoutCore extends MultiStageLayout {
}
/**
This class arranges graphs in a hierarchic/layered fashion and supports complete re-layout as well as incremental graph
layout.
Note that class {@link HierarchicLayout} offers a facade to this class which is easier to use. Hence, we
strongly recommend to not directly use class {@link HierarchicLayoutCore}.
<b>Layout Style</b>
The nodes are distributed into layers such that most of the edges point to the main layout direction. The order of the
nodes within the layers ensures that the number of edge crossings is as small as possible. There are different edge
routing styles available. Edges can be orthogonal, polyline or octilinear.
Hierarchical diagrams are commonly used for the visualization of hierarchical data, since they facilitate the
identification of dependencies and relationships among the nodes of the graph. Possible application domains are the
following: workflow visualization, call graph visualization, entity-relationship diagrams, biochemical pathways and
network management.
<b>Concept</b>
The layout algorithm runs in three main phases:
<ul>
<li><b>Layering</b> – The nodes are distributed into layers by means of
{@link HierarchicLayoutCore#layerer}. If the layout orientation is top-to-bottom, the nodes in each layer are arranged horizontally while the layers are
ordered vertically top-to-bottom.
<li><b>Sequencing</b> – The order of the nodes in each layer is determined such that the number of edge crossings is as small as possible. To
specify the sequencing algorithm, use {@link HierarchicLayoutCore#sequencer}.
<li><b>Drawing</b> – The layout algorithm assigns the final coordinates to all nodes and routes the edges.
</ul>
<b>Features</b>
In order to customize this layout algorithm, modify the {@link HierarchicLayoutCore#layerer Layerer}, {@link HierarchicLayoutCore#sequencer Sequencer}, {@link HierarchicLayoutCore#portAllocator PortAllocator}, {@link HierarchicLayoutCore#drawingDistanceCalculator DrawingDistanceCalculator},
and {@link HierarchicLayoutCore#nodePlacer NodePlacer} instances.
This layout algorithm is able to incrementally add new elements to an existing drawing of a graph. In order to add
elements incrementally, register a {@link IDataProvider} with the graph using the {@link HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY}
DataProvider key and associate the hints obtained from the {@link IIncrementalHintsFactory} with the elements
to be added incrementally.
{@link HierarchicLayoutNodeLayoutDescriptor} and {@link HierarchicLayoutEdgeLayoutDescriptor} instances can be used for specifying
individual information (e.g. distances or routing styles) for each node and edge in the graph. The descriptors are bound
to the graph using {@link IDataProvider}s registered with {@link HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY}
or {@link HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
{@link HierarchicLayoutCore} supports two approaches to connect edges on a specific side or even an exact
location to a node. {@link PortConstraint}s define a single constraint for the ports of an edge. To realize more
complex port restrictions, several {@link PortCandidate}s or {@link PortCandidateSet}s can be assigned to
edges or nodes. If an edge with registered {@link PortCandidate}s connects to nodes with
{@link PortCandidateSet}s, the layouter will try to match both collections to find an appropriate port. In case
there is no matching port candidate, a {@link PortCandidate} specified for the edge is preferred. Since their
simultaneous existence at the same node may be ambiguous, it is not recommended to use a combination of
{@link PortConstraint}s and {@link PortCandidate}s in the same layout.
This algorithm specifies a {@link LabelLayoutTranslator} instance as the current {@link MultiStageLayout#labeling} and configures it such that the
node labels are passed to it by setting {@link LabelLayoutTranslator#translateNodeLabels} to <code>true</code>.
In order to enable integrated edge labeling of this algorithm, make sure {@link LabelLayoutTranslator} is
registered and edge labels are being translated and written back after the layout.
Moreover, this algorithm supports layer and sequence constraints. These constraints can be specified using a {@link ILayerConstraintFactory}
and {@link ISequenceConstraintFactory}, respectively.
@see HierarchicLayout
@see ILayerer
@see ISequencer
@see IPortAllocator
@see IDrawingDistanceCalculator
@see IHierarchicLayoutNodePlacer
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore Online Documentation}*/
export class HierarchicLayoutCore {
/**
Creates a new {@link HierarchicLayoutCore} instance with the default settings.
@param [options] The parameters to pass.
@param [options.componentLayoutEnabled] Whether or not the {@link ComponentLayout} should be enabled. This option sets the {@link HierarchicLayoutCore#componentLayoutEnabled} property on the created object.
@param [options.maximumDuration] The time limit (in milliseconds) set for the layout algorithm. This option sets the {@link HierarchicLayoutCore#maximumDuration} property on the created object.
@param [options.layerer] The {@link ILayerer} instance responsible for generating the layer assignment (first phase of the layout algorithm). This option sets the {@link HierarchicLayoutCore#layerer} property on the created object.
@param [options.sequencer] The {@link ISequencer} instance responsible for calculating the sequence of nodes within each layer (second phase of the layout algorithm). This option sets the {@link HierarchicLayoutCore#sequencer} property on the created object.
@param [options.nodePlacer] The {@link IHierarchicLayoutNodePlacer} instance responsible for the assignment of the x-(sequence) and preliminary y-coordinates of the nodes. This option sets the {@link HierarchicLayoutCore#nodePlacer} property on the created object.
@param [options.portAllocator] The {@link IPortAllocator} instance responsible for assigning port coordinates to the adjacent edges of each node. This option sets the {@link HierarchicLayoutCore#portAllocator} property on the created object.
@param [options.portConstraintOptimizer] The {@link IPortConstraintOptimizer} instance responsible for optimizing the assignment of port constraints to the adjacent edges of each node. This option sets the {@link HierarchicLayoutCore#portConstraintOptimizer} property on the created object.
@param [options.drawingDistanceCalculator] The {@link IDrawingDistanceCalculator} instance responsible for determining the minimum distances between elements that belong to the same layer. This option sets the {@link HierarchicLayoutCore#drawingDistanceCalculator} property on the created object.
@param [options.gridSpacing] The equidistant spacing between the horizontal and vertical grid lines. This option sets the {@link HierarchicLayoutCore#gridSpacing} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link MultiStageLayout#orientationLayoutEnabled} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link MultiStageLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see HierarchicLayoutCore#createLayerer
@see HierarchicLayoutCore#createIncrementalLayerer
@see HierarchicLayoutCore#createSequencer
@see HierarchicLayoutCore#createDrawingDistanceCalculator
@see HierarchicLayoutCore#createNodePlacer
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore Online Documentation}*/
constructor(options?: {
componentLayoutEnabled?: boolean;
maximumDuration?: number;
layerer?: ILayerer | null;
sequencer?: ISequencer | null;
nodePlacer?: IHierarchicLayoutNodePlacer | null;
portAllocator?: IPortAllocator | null;
portConstraintOptimizer?: IPortConstraintOptimizer | null;
drawingDistanceCalculator?: IDrawingDistanceCalculator | null;
gridSpacing?: number;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
orientationLayoutEnabled?: boolean;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
selfLoopRouterEnabled?: boolean;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Calculates a hierarchic layout of the given graph.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Factory method that is called lazily upon first usage by {@link HierarchicLayoutCore#drawingDistanceCalculator} if the given {@link IDrawingDistanceCalculator}
instance is <code>null</code>.
The default implementation returns a {@link DefaultDrawingDistanceCalculator} instance. It may be overridden
to return a custom {@link DefaultDrawingDistanceCalculator} instance.
<b>Warning:</b> Creating a custom implementation of {@link IDrawingDistanceCalculator} will make the properties {@link HierarchicLayout#nodeToNodeDistance}, {@link HierarchicLayout#nodeToEdgeDistance} and {@link HierarchicLayout#edgeToEdgeDistance}
superfluous. Those properties delegate to {@link DefaultDrawingDistanceCalculator} and will throw an {@link Exception InvalidOperationError}
if a different class is used. Adjustments of those distances need to be done directly on the custom implementation of
{@link IDrawingDistanceCalculator}.
@returns a {@link IDrawingDistanceCalculator} instance
@see HierarchicLayoutCore#drawingDistanceCalculator
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createDrawingDistanceCalculator() Online Documentation}*/
createDrawingDistanceCalculator(): IDrawingDistanceCalculator | null;
/**
Factory method called during {@link HierarchicLayoutCore#applyLayoutCore} and reates an appropriate {@link IEdgeReverser}
implementation.
By default, a suitable private implementation of {@link IEdgeReverser} is returned. It may be overridden to
return a custom {@link IEdgeReverser} implementation.
@returns an implementation
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createEdgeReverser() Online Documentation}*/
createEdgeReverser(): IEdgeReverser | null;
/**
Factory method that is called during {@link HierarchicLayoutCore#applyLayoutCore} and creates a {@link LayoutGroupingSupport}
instance in case of a grouped graph.
@param graph the input graph
@returns a {@link LayoutGroupingSupport} instance or <code>null</code> if the graph is not grouped
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createGrouping(LayoutGraph) Online Documentation}*/
createGrouping(graph: LayoutGraph | null): LayoutGroupingSupport | null;
/**
Creates an {@link IIncrementalHintsFactory} implementation used for obtaining hint objects associated with
nodes and edges of the graph prior to the invocation of the layout algorithm.
Incremental hints can be obtained from a {@link IDataProvider} that is registered with the graph using key
{@link HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY}.
@returns an {@link IIncrementalHintsFactory} instance
@see HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createIncrementalHintsFactory() Online Documentation}*/
createIncrementalHintsFactory(): IIncrementalHintsFactory | null;
/**
Factory method called during {@link HierarchicLayoutCore#applyLayoutCore} and creates an appropriate {@link ILayerer}
implementation.
By default, a {@link ConstraintIncrementalLayerer} instance with a {@link TopologicalLayerer}
instance as argument is returned. This method may be overridden to return a custom {@link ILayerer}
implementation.
@returns a {@link ILayerer} implementation
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createIncrementalLayerer() Online Documentation}*/
createIncrementalLayerer(): ILayerer | null;
/**
Callback method that is called during {@link HierarchicLayoutCore#applyLayoutCore}, creates the {@link INodeData}
and {@link IEdgeData} instances and binds them to the elements using the given {@link IItemFactory itemFactory}.
@param g the input graph
@param itemFactory the given {@link IItemFactory} instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createItemData(LayoutGraph%2CIItemFactory) Online Documentation}*/
createItemData(g: LayoutGraph | null, itemFactory: IItemFactory | null): void;
/**
Creates a {@link ILayerConstraintFactory layer constraint factory} that allows to create hints that affect the assignment of the nodes to layers.
By default, a suitable private implementation of {@link ILayerConstraintFactory} is returned.
@param graph the given graph
@returns a {@link ILayerConstraintFactory} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createLayerConstraintFactory(Graph) Online Documentation}*/
createLayerConstraintFactory(graph: Graph | null): ILayerConstraintFactory | null;
/**
Factory method that is called lazily upon first usage by {@link HierarchicLayoutCore#layerer} if no {@link ILayerer} instance is given as input.
The default implementation returns a <code>new MultiComponentLayerer(new WeightedLayerer())</code> instance. It may be overridden to
return a custom {@link ILayerer} instance.
@returns a {@link ILayerer} instance
@see HierarchicLayoutCore#layerer
@see MultiComponentLayerer
@see WeightedLayerer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createLayerer() Online Documentation}*/
createLayerer(): ILayerer | null;
/**
Factory method called during {@link HierarchicLayoutCore#applyLayoutCore} and creates an appropriate {@link ILayers}
implementation using the given {@link ILayoutDataProvider} instance.
By default, a suitable private implementation of {@link ILayers} is returned. It may be overridden to return
custom implementations of the {@link ILayers} interface.
@param ldp the given {@link ILayoutDataProvider} instance
@returns a {@link ILayers} implementation
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createLayers(ILayoutDataProvider) Online Documentation}*/
createLayers(ldp: ILayoutDataProvider | null): ILayers | null;
/**
Factory method that is called lazily upon first usage by {@link HierarchicLayoutCore#nodePlacer} if the given {@link IHierarchicLayoutNodePlacer} instance is <code>null</code>.
The default implementation returns a {@link SimplexNodePlacer} instance. It may be overridden to return a
custom {@link IHierarchicLayoutNodePlacer} instance.
@returns a {@link IHierarchicLayoutNodePlacer} instance
@see HierarchicLayoutCore#nodePlacer
@see SimplexNodePlacer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createNodePlacer() Online Documentation}*/
createNodePlacer(): IHierarchicLayoutNodePlacer | null;
/**
Factory method that is called lazily upon first usage by {@link HierarchicLayoutCore#portAllocator} if the given {@link IPortAllocator} instance is
<code>null</code>.
The default implementation returns a {@link DefaultPortAllocator} instance. It may be overridden to return a
custom {@link IPortAllocator} instance.
@returns a {@link IPortAllocator} instance
@see HierarchicLayoutCore#portAllocator
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createPortAllocator() Online Documentation}*/
createPortAllocator(): IPortAllocator | null;
/**
Factory method that is called lazily upon first usage by {@link HierarchicLayoutCore#portConstraintOptimizer} if the given {@link IPortConstraintOptimizer}
instance is <code>null</code>.
The default implementation returns <code>null</code>. It may be overridden to return a custom {@link IPortConstraintOptimizer}
instance.
@returns <code>null</code>
@see HierarchicLayoutCore#portConstraintOptimizer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createPortConstraintOptimizer() Online Documentation}*/
createPortConstraintOptimizer(): IPortConstraintOptimizer | null;
/**
Creates sequence constraints that affect the sequence of the nodes within each layer.
By default, a suitable private implementation of {@link ISequenceConstraintFactory} is returned.
@param graph the given graph
@returns a {@link ISequenceConstraintFactory} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createSequenceConstraintFactory(Graph) Online Documentation}*/
createSequenceConstraintFactory(graph: Graph | null): ISequenceConstraintFactory | null;
/**
Factory method that is called lazily upon first usage by {@link HierarchicLayoutCore#sequencer} if the given {@link ISequencer} instance is <code>null</code>.
The default implementation returns a {@link DefaultLayerSequencer} instance. It may be overridden to return a
custom {@link ISequencer} instance.
@returns a {@link ISequencer} instance
@see HierarchicLayoutCore#sequencer
@see DefaultLayerSequencer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createSequencer() Online Documentation}*/
createSequencer(): ISequencer | null;
/**
Factory method called during {@link HierarchicLayoutCore#applyLayoutCore} and creates a {@link ISequencer}
implementation that can sequence subgraphs incrementally.
By default, a suitable private implementation of {@link ISequencer} is returned. It may be overridden to
return custom implementations of the {@link ISequencer} interface.
@returns a {@link ISequencer} implementation
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createSubgraphLayerSequencer() Online Documentation}*/
createSubgraphLayerSequencer(): ISequencer | null;
/**
Provides access to implementation specific properties of the algorithms used.
Used for internal purposes.
@param key the key to a property
@returns the associated value or <code>null</code>
@see HierarchicLayoutCore#setAlgorithmProperty
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23getAlgorithmProperty(Object) Online Documentation}*/
getAlgorithmProperty(key: any | null): any | null;
/**
Callback method called during {@link HierarchicLayoutCore#applyLayoutCore} and returns a {@link IDataProvider}
that holds the {@link HierarchicLayoutEdgeLayoutDescriptor} information.
The {@link IDataProvider} holding the information about the node layout is registered with the graph using key
{@link HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
@param graph the input graph
@returns a {@link IDataProvider} instance or <code>null</code>
@see HierarchicLayoutEdgeLayoutDescriptor
@see HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23getEdgeLayoutDescriptors(LayoutGraph) Online Documentation}*/
getEdgeLayoutDescriptors(graph: LayoutGraph | null): IDataProvider | null;
/**
Callback method called during {@link HierarchicLayoutCore#applyLayoutCore} and returns a {@link IDataProvider}
that holds the incremental hints information.
The {@link IDataProvider} holding the information about incremental hints is registered with the graph using
key {@link HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY}.
@param graph the input graph
@returns a {@link IDataProvider} instance or <code>null</code>
@see HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23getIncrementalHints(LayoutGraph) Online Documentation}*/
getIncrementalHints(graph: LayoutGraph | null): IDataProvider | null;
/**
Callback method called during {@link HierarchicLayoutCore#applyLayoutCore} and returns a {@link IDataProvider}
that holds the {@link HierarchicLayoutNodeLayoutDescriptor} information.
The {@link IDataProvider} holding the information about the node layout is registered with the graph using key
{@link HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY}.
@param graph the input graph
@returns a {@link IDataProvider} instance or <code>null</code>
@see HierarchicLayoutNodeLayoutDescriptor
@see HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23getNodeLayoutDescriptors(LayoutGraph) Online Documentation}*/
getNodeLayoutDescriptors(graph: LayoutGraph | null): IDataProvider | null;
/**
Callback method called during {@link HierarchicLayoutCore#applyLayoutCore} and returns a {@link IDataProvider}
that holds the {@link SwimlaneDescriptor} information.
The {@link IDataProvider} holding the information about swimlanes is registered with the graph using key
{@link HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY}.
@param graph the input graph
@returns a {@link IDataProvider} instance or <code>null</code>
@see SwimlaneDescriptor
@see HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23getSwimLaneDescriptors(LayoutGraph) Online Documentation}*/
getSwimLaneDescriptors(graph: LayoutGraph | null): IDataProvider | null;
/**
Callback method that publishes the layering information.
The layering information is stored in a {@link IDataProvider} registered with the graph using key
{@link HierarchicLayoutCore#LAYER_INDEX_DP_KEY}.
@param graph the input graph
@param layers the {@link ILayers} implementation that provides the layering information
@see HierarchicLayoutCore#LAYER_INDEX_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23publishLayers(LayoutGraph%2CILayers) Online Documentation}*/
publishLayers(graph: LayoutGraph | null, layers: ILayers | null): void;
/**
Callback method that publishes the sequencing information.
The sequencing information is stored in a {@link IDataProvider} registered with the graph using key
{@link HierarchicLayoutCore#SEQUENCE_INDEX_DP_KEY}.
@param graph the input graph
@param layers the {@link ILayers} implementation that provides the layering and sequencing information
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@see HierarchicLayoutCore#SEQUENCE_INDEX_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23publishSequences(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
publishSequences(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
Removes the bends which are obviously not necessary.
By default, this method removes collinear bends from the graph. A bend point is considered to be collinear if it lies on
a line with its preceding and succeeding bend point. It may be overridden for a custom implementation of bend removal.
@param graph the input graph
@param [bendReductionThreshold=0.1] this method removes a bend, if its horizontal/vertical distance to the closest collinear bend is above this value
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23reduceBendCount(LayoutGraph%2Cnumber) Online Documentation}*/
reduceBendCount(graph: LayoutGraph | null, bendReductionThreshold?: number): void;
/**
Provides access to implementation specific properties of the algorithms used.
Used for internal purposes.
@param key the key to a property
@param value the value to associate with the key
@see HierarchicLayoutCore#setAlgorithmProperty
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23setAlgorithmProperty(Object%2CObject) Online Documentation}*/
setAlgorithmProperty(key: any | null, value: any | null): void;
/**
Data provider key for specifying incremental hints
Incremental hints are created using an {@link HierarchicLayoutCore#createIncrementalHintsFactory incremental hints factory}.
@see HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY
@see IIncrementalHintsFactory
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23INCREMENTAL_HINTS_DP_KEY Online Documentation}*/
static INCREMENTAL_HINTS_DP_KEY: GraphObjectDpKey<any> | null;
/**
Data provider key for providing layout information for each edge
@see IEdgeData#edgeLayoutDescriptor
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23EDGE_LAYOUT_DESCRIPTOR_DP_KEY Online Documentation}*/
static EDGE_LAYOUT_DESCRIPTOR_DP_KEY: EdgeDpKey<HierarchicLayoutEdgeLayoutDescriptor> | null;
/**
Data provider key for providing layout information for each node
@see INodeData#nodeLayoutDescriptor
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23NODE_LAYOUT_DESCRIPTOR_DP_KEY Online Documentation}*/
static NODE_LAYOUT_DESCRIPTOR_DP_KEY: NodeDpKey<HierarchicLayoutNodeLayoutDescriptor> | null;
/**
Data provider key for providing information about swimlanes
If this key is present during the layout, the layout algorithm will arrange nodes in swimlanes. The information about
the swimlanes is finally written back to the descriptor instances.
Instances can be shared among multiple nodes in the same lane, but don't have to be shared.
@see SwimlaneDescriptor
@see INodeData#swimLaneDescriptor
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23SWIMLANE_DESCRIPTOR_DP_KEY Online Documentation}*/
static SWIMLANE_DESCRIPTOR_DP_KEY: NodeDpKey<SwimlaneDescriptor> | null;
/**
Data acceptor key for publishing the layer IDs for all nodes in the graph
For each non-group node, the index of the node's layer will be stored in this {@link IDataAcceptor} after the
layout run.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the layer information of the nodes is dropped.
@see HierarchicLayout#LAYER_INDEX_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23LAYER_INDEX_DP_KEY Online Documentation}*/
static LAYER_INDEX_DP_KEY: NodeDpKey<number> | null;
/**
Data acceptor key for publishing the index inside their layer for all nodes in the graph
For each non-group node, the index (position) of the node within its layer will be stored in this {@link IDataAcceptor}
after the layout run.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the sequence information of the nodes is dropped.
@see HierarchicLayout#SEQUENCE_INDEX_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23SEQUENCE_INDEX_DP_KEY Online Documentation}*/
static SEQUENCE_INDEX_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for internal use only
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23INCREMENTAL_NODES_DP_KEY Online Documentation}*/
static INCREMENTAL_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Factory method called during the initialization of this {@link HierarchicLayoutCore} instance that creates a
{@link MultiStageLayout#labeling label layouter}.
In order to use the integrated node label awareness feature one has to use a {@link LabelLayoutTranslator}
instance and set {@link LabelLayoutTranslator#translateNodeLabels} to <code>true</code>. Likewise, in order to make use of the integrated edge labeling {@link LabelLayoutTranslator#translateEdgeLabels} must be set to <code>true</code> and also
{@link LabelLayoutTranslator#writeBackEdgeLabels} must be set to <code>true</code> (which is the default).
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23createLabelLayout Online Documentation}*/
readonly createLabelLayout: ILayoutStage | null;
/**
Gets or sets whether or not the {@link ComponentLayout} should be enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23componentLayoutEnabled Online Documentation}*/
componentLayoutEnabled: boolean;
/**
Gets or sets the time limit (in milliseconds) set for the layout algorithm.
Values have to be greater than or equal to <code>0</code>.
<h2>Note</h2>
Restricting the maximum duration may result in a lower layout quality. Furthermore, the real runtime may exceed the
maximum duration since the layout algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the {@link ILayerer} instance responsible for generating the layer assignment (first phase of the
layout algorithm).
To set the default layerer, see {@link HierarchicLayoutCore#createLayerer}.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link ILayerer} is <code>null</code>
@see HierarchicLayoutCore#createLayerer
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23layerer Online Documentation}*/
layerer: ILayerer | null;
/**
Gets or sets the {@link ISequencer} instance responsible for calculating the sequence of nodes within each
layer (second phase of the layout algorithm).
To set the default sequencer, see {@link HierarchicLayoutCore#createSequencer}.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link ISequencer} is <code>null</code>
@see HierarchicLayoutCore#createSequencer
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23sequencer Online Documentation}*/
sequencer: ISequencer | null;
/**
Gets or sets the {@link IHierarchicLayoutNodePlacer} instance responsible for the assignment of the x-(sequence) and
preliminary y-coordinates of the nodes.
To set the default node placer, see {@link HierarchicLayoutCore#createNodePlacer}.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IHierarchicLayoutNodePlacer} is <code>null</code>
@see HierarchicLayoutCore#createNodePlacer
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23nodePlacer Online Documentation}*/
nodePlacer: IHierarchicLayoutNodePlacer | null;
/**
Gets or sets the {@link IPortAllocator} instance responsible for assigning port coordinates to the adjacent
edges of each node.
To set the default port allocator, see {@link HierarchicLayoutCore#createPortAllocator}.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IPortAllocator} is <code>null</code>
@see HierarchicLayoutCore#createPortAllocator
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23portAllocator Online Documentation}*/
portAllocator: IPortAllocator | null;
/**
Gets or sets the {@link IPortConstraintOptimizer} instance responsible for optimizing the assignment of port
constraints to the adjacent edges of each node.
To set the default port allocator, see {@link HierarchicLayoutCore#createPortConstraintOptimizer}.
@see HierarchicLayoutCore#createPortConstraintOptimizer
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23portConstraintOptimizer Online Documentation}*/
portConstraintOptimizer: IPortConstraintOptimizer | null;
/**
Gets or sets the {@link IDrawingDistanceCalculator} instance responsible for determining the minimum distances
between elements that belong to the same layer.
To set the default port allocator, see {@link HierarchicLayoutCore#createDrawingDistanceCalculator}.
<b>Warning:</b> Specifying a custom implementation of {@link IDrawingDistanceCalculator} will make the properties {@link HierarchicLayout#nodeToNodeDistance}, {@link HierarchicLayout#nodeToEdgeDistance} and {@link HierarchicLayout#edgeToEdgeDistance}
superfluous. Those properties delegate to {@link DefaultDrawingDistanceCalculator} and will throw an {@link Exception InvalidOperationError}
if a different class is used. Adjustments of those distances need to be done directly on the custom implementation of
{@link IDrawingDistanceCalculator}.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IDrawingDistanceCalculator} is <code>null</code>
@see HierarchicLayoutCore#createDrawingDistanceCalculator
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23drawingDistanceCalculator Online Documentation}*/
drawingDistanceCalculator: IDrawingDistanceCalculator | null;
/**
Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
When the spacing is negative or zero, no grid is defined, otherwise nodes and edges are placed on multiples of the grid
spacing.
<b>Warning:</b> The grid feature doesn't work together with exact (layer/sequence) coordinate hints.
@see IIncrementalHintsFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutCore%23gridSpacing Online Documentation}*/
gridSpacing: number;
static $class: Class;
private $_HierarchicLayoutCore: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HierarchicLayoutCore;
}
export interface IDrawingDistanceCalculator {
}
/**
This interface determines the minimum distances between elements that belong to the same layer.
{@link IDrawingDistanceCalculator} is implemented by a helper class used by {@link HierarchicLayoutCore}
during the phase of node placement (i.e. when the final coordinates for all nodes are calculated). {@link IHierarchicLayoutNodePlacer}
uses instances of this interface to determine the minimum distances between elements that belong to the same layer.
Information about graph elements (i.e. nodes or edges) is given using a {@link ILayoutDataProvider} while
information about layers is given through {@link ILayer} (describes a single layer of the hierarchic layout)
or {@link ILayers} (manages multiple {@link ILayer layers} of the hierarchic layout).
Implementations not needed any more should be {@link IDrawingDistanceCalculator#dispose disposed of}.
The default implementation used by the {@link HierarchicLayoutCore} is
{@link DefaultDrawingDistanceCalculator}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDrawingDistanceCalculator Online Documentation}*/
export abstract class IDrawingDistanceCalculator {
/**
Disposes of internal data structures.
Implementations should release internally held data structures when they are not needed any more.
@param graph the input graph that contains all elements that are used during the node placement
@param layers the {@link ILayers} object that will be used during subsequent calls
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@see IDrawingDistanceCalculator#initialize
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDrawingDistanceCalculator%23dispose(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
abstract dispose(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
Main interface method called by {@link IHierarchicLayoutNodePlacer} instances to determine the minimum distance between
{@link YNode}s of same layer.
<h2>Note</h2>
One of {@link YNode} arguments may be <code>null</code> if its border is not of interest. In that case, only the border of
the non-<code>null</code> {@link YNode} instance should be considered.
@param graph the input graph
@param layer the {@link ILayer} object that will be used during subsequent calls
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param left the left of the two {@link YNode}s whose minimum distance is to be determined or <code>null</code> if only the left border
of the right {@link YNode} is of interest
@param right the right of the two {@link YNode}s whose minimum distance is to be determined or <code>null</code> if only the right border
of the left {@link YNode} is of interest
@returns the minimum distance between two {@link YNode}s of the same layer
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDrawingDistanceCalculator%23getMinDistance(LayoutGraph%2CILayer%2CILayoutDataProvider%2CYNode%2CYNode) Online Documentation}*/
abstract getMinDistance(graph: LayoutGraph | null, layer: ILayer | null, ldp: ILayoutDataProvider | null, left: YNode | null, right: YNode | null): number;
/**
Initializes internal data structures.
Implementations may cache lengthy calculations in the initialization phase.
<h2>Note</h2>
It is guaranteed that the graph will not be changed during subsequent calls to
{@link IDrawingDistanceCalculator#getMinDistance}.
<h2>Note</h2>
Implementations should {@link IDrawingDistanceCalculator#dispose dispose of} internally held data structures when they are not needed any more.
@param graph the input graph that contains all elements that are used during the node placement
@param layers the {@link ILayers} object that will be used during subsequent calls
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@see IDrawingDistanceCalculator#dispose
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDrawingDistanceCalculator%23initialize(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
abstract initialize(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
static $class: Class;
private $_IDrawingDistanceCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IDrawingDistanceCalculator;
}
export interface IEdgeData {
}
/**
This interface provides layout information about the edges of the graph during the layout.
{@link IEdgeData} is a multi-purpose edge descriptor for each edge, holding information about the {@link IEdgeData#type type} of an
edge, associated {@link PortConstraint port constraints}, {@link PortCandidate port candidates}, edge groups as well as the current edge state.
It is used in all phases of the {@link HierarchicLayoutCore} to store edge information.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData Online Documentation}*/
export abstract class IEdgeData {
/**
Gets the type of the {@link Edge} associated with this {@link IEdgeData} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23type Online Documentation}*/
abstract readonly type: EdgeDataType;
/**
Gets the {@link YNode} associated with this {@link IEdgeData} instance.
If no {@link YNode} is associated with this {@link IEdgeData} instance, <code>null</code> should be returned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23associatedNode Online Documentation}*/
abstract readonly associatedNode: YNode | null;
/**
Gets the {@link Edge} associated with this {@link IEdgeData} instance.
If no {@link Edge} is associated with this {@link IEdgeData} instance, <code>null</code> should be returned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23associatedEdge Online Documentation}*/
abstract readonly associatedEdge: Edge | null;
/**
Gets the {@link PortConstraint port constraints} for the source port of the {@link Edge} associated with this {@link IEdgeData}
instance.
{@link PortConstraint}s are obtained by a {@link IDataProvider} registered with the graph with key
{@link PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23sourcePortConstraint Online Documentation}*/
abstract readonly sourcePortConstraint: PortConstraint | null;
/**
Gets the ID of the edge port group at the source node to which the {@link Edge} associated with this {@link IEdgeData}
instance belongs (if any).
Information about edge port grouping at the source node is obtained by a {@link IDataProvider} registered with
the graph with key {@link PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23sourcePortGroup Online Documentation}*/
abstract readonly sourcePortGroup: any | null;
/**
Gets the {@link ICollection collection} of {@link PortCandidate}s for the source port of the {@link Edge} associated with
this {@link IEdgeData} instance.
{@link PortCandidate Source port candidates} are obtained by a {@link IDataProvider} registered with the graph with key
{@link PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23sourcePortCandidates Online Documentation}*/
abstract readonly sourcePortCandidates: ICollection<any> | null;
/**
Gets the {@link PortConstraint port constraints} for the target port of the {@link Edge} associated with this {@link IEdgeData}
instance.
{@link PortConstraint}s are obtained by a {@link IDataProvider} registered with the graph with key
{@link PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23targetPortConstraint Online Documentation}*/
abstract readonly targetPortConstraint: PortConstraint | null;
/**
Gets the ID of the edge port group at the target node to which the {@link Edge} associated with this {@link IEdgeData}
instance belongs (if any).
Information about edge port grouping at the target node is obtained by a {@link IDataProvider} registered with
the graph with key {@link PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23targetPortGroup Online Documentation}*/
abstract readonly targetPortGroup: any | null;
/**
Gets the {@link ICollection collection} of {@link PortCandidate}s for the target port of the {@link Edge} associated with
this {@link IEdgeData} instance.
{@link PortCandidate Target port candidates} are obtained by a {@link IDataProvider} registered with the graph with key
{@link PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23targetPortCandidates Online Documentation}*/
abstract readonly targetPortCandidates: ICollection<any> | null;
/**
Gets the ID of the edge group at the source node to which the {@link Edge} associated with this {@link IEdgeData}
instance belongs (if any).
Information about edge grouping at the source node is obtained by a {@link IDataProvider} registered with the
graph with key {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23sourceGroup Online Documentation}*/
abstract readonly sourceGroup: any | null;
/**
Gets the ID of the edge group at the target node to which the {@link Edge} associated with this {@link IEdgeData}
instance belongs (if any).
Information about edge grouping at the target node is obtained by a {@link IDataProvider} registered with the
graph with key {@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23targetGroup Online Documentation}*/
abstract readonly targetGroup: any | null;
/**
Gets the group that is represented by the {@link Edge} associated with this {@link IEdgeData}
instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23group Online Documentation}*/
abstract readonly group: any | null;
/**
Gets whether or not the {@link Edge} associated with this {@link IEdgeData} instance has been
reversed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23reversed Online Documentation}*/
abstract readonly reversed: boolean;
/**
Gets whether or not the {@link Edge} associated with this {@link IEdgeData} instance is an upper
same-layer edge.
An upper same-layer edge is an edge connecting two nodes of the same layer with polyline segments (not directly, with a
straight-line) and is routed above the layer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23upperSameLayerEdge Online Documentation}*/
abstract readonly upperSameLayerEdge: boolean;
/**
Gets the {@link HierarchicLayoutEdgeLayoutDescriptor} instance bound to the {@link Edge} associated with this {@link IEdgeData}
instance.
If no {@link HierarchicLayoutEdgeLayoutDescriptor} is bound to the edge, <code>null</code> should be returned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23edgeLayoutDescriptor Online Documentation}*/
abstract readonly edgeLayoutDescriptor: HierarchicLayoutEdgeLayoutDescriptor | null;
/**
Gets or sets whether or not the {@link Edge} associated with this {@link IEdgeData} instance should
be an upper same-layer edge (if it is a same-layer edge).
An upper same-layer edge is an edge connecting two nodes of the same layer with polyline segments (not directly, with a
straight-line) and is routed above the layer.
If enabled the edge will be routed above the layer, otherwise below it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23fallbackUpperSameLayerEdge Online Documentation}*/
abstract fallbackUpperSameLayerEdge: boolean;
/**
Gets the thickness of the {@link Edge} associated with this {@link IEdgeData} instance.
The thickness is considered when calculating minimum distances so that there are no overlaps between edges and other
graph elements.
The thickness must have a non-negative value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23thickness Online Documentation}*/
abstract readonly thickness: number;
/**
Gets the crossing cost of the {@link Edge} associated with this {@link IEdgeData} instance.
The crossings costs of edges are considered when minimizing the edge crossing count during the sequencing phase.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23crossingCost Online Documentation}*/
abstract readonly crossingCost: number;
/**
Gets the critical edge priority of the {@link Edge} associated with this {@link IEdgeData}
instance.
The layout algorithm tries to vertically align each node pair that is connected by a critical edge. Conflicts between
different critical edges are always resolved in favor of the edge with the higher priority.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeData%23criticalEdgePriority Online Documentation}*/
abstract readonly criticalEdgePriority: number;
static $class: Class;
private $_IEdgeData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeData;
}
export interface IEdgeReverser {
}
/**
This interface handles edges that should be reversed during the hierarchic layout process.
{@link IEdgeReverser} contains methods for reversing all (or a given edge) that point to the wrong direction,
i.e. the layer indices of the source nodes are greater than the layer indices of the target nodes. It also provides
methods to restore the edges that have been reversed at the end of the layout algorithm.
Implementations of this interface perform operations after the layering phase of the hierarchic layout algorithm.
@see HierarchicLayoutCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReverser Online Documentation}*/
export abstract class IEdgeReverser {
/**
Reverses all edges which point to the wrong direction.
It is called after the layering phase of the {@link HierarchicLayoutCore hierarchic layout algorithm}.
An edge is reversed if the layer index of its source node is greater than the layer index of its target node.
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} containing layering information about the nodes and edges of the graph
@see ILayoutDataProvider#getNodeData
@see INodeData#layer
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReverser%23normalizeEdges(LayoutGraph%2CILayoutDataProvider) Online Documentation}*/
abstract normalizeEdges(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null): void;
/**
Restores the original direction of edges marked as reversed.
It is called at the end of the {@link HierarchicLayoutCore hierarchic layout algorithm}.
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} containing information about the nodes and edges of the graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReverser%23restoreEdgeDirections(LayoutGraph%2CILayoutDataProvider) Online Documentation}*/
abstract restoreEdgeDirections(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null): void;
/**
Reverses a given edge which points to the wrong direction.
It may be called by the incremental parts of the {@link HierarchicLayoutCore hierarchic layout algorithm}.
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} containing layering information about the nodes and edges of the graph
@param edge the edge to be reversed
@see ILayoutDataProvider#getNodeData
@see INodeData#layer
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeReverser%23reverse(LayoutGraph%2CILayoutDataProvider%2CEdge) Online Documentation}*/
abstract reverse(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, edge: Edge | null): void;
static $class: Class;
private $_IEdgeReverser: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeReverser;
}
export interface IIncrementalHintsFactory {
}
/**
This interface provides factory methods for creating incremental hints for the hierarchic layout.
Incremental hints can be used for specifying that a subset of nodes (called <em>incremental</em> nodes) should be added to an existing graph
layout. A layer hint specifies that both, a node's layer and its position within the layer should be freely determined
by the algorithm. A sequence hint specifies that the algorithm should consider the current layer of a node while its
position within the layer should be freely determined by the algorithm. A node without any hint is called a <em>fixed</em> node.
Sequence hints can also be specified for edges. These <em>incremental</em> edges are inserted in a suitable position within the layers
crossed by the edges. For <em>fixed</em> edges the given input route is considered to calculate the position with the layers.
Information about incremental hints is provided by a {@link IDataProvider} registered with the graph with key
{@link HierarchicLayout#INCREMENTAL_HINTS_DP_KEY}.
An instance of a class implementing this interface can be obtained through method
{@link HierarchicLayout#createIncrementalHintsFactory}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIncrementalHintsFactory Online Documentation}*/
export abstract class IIncrementalHintsFactory {
/**
Creates a hint {@link Object} for a group {@link YNode} that should be inserted incrementally during the layering phase.
The group will be placed on a suitable position. The descendants of the group may be associated with hints created by
methods {@link IIncrementalHintsFactory#createLayerIncrementallyHint}, {@link IIncrementalHintsFactory#createSequenceIncrementallyHint}
or this method (if the descendant is an inner group).
All hints of the descendants of a group are interpreted relative to the group node. Descendants without hints maintain
their relative order within the group node (but not with elements outside the group).
The positions of groups without incremental hints depend on the position of their descendants (i.e., the group is not
interpreted as fixed - it is simply ignored).
<h2>Note</h2>
Descendants of incremental groups must not be associated with exact coordinate hints (see method
{@link IIncrementalHintsFactory#createUseExactCoordinatesHint}, {@link IIncrementalHintsFactory#createUseExactSequenceCoordinatesHint}
and {@link IIncrementalHintsFactory#createUseExactLayerCoordinatesHint}).
@param forItemId the group node to be inserted incrementally
@returns an {@link Object} that can be interpreted as a hint by {@link HierarchicLayoutCore}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIncrementalHintsFactory%23createIncrementalGroupHint(Object) Online Documentation}*/
abstract createIncrementalGroupHint(forItemId: Object | null): Object | null;
/**
Creates a hint {@link Object} for a {@link YNode} that should be inserted incrementally during the layering phase.
This will place the node in a suitable layer, possibly creating new layers.
Neighbors of this node may be also marked such that they are laid out incrementally. This makes it possible to
incrementally add whole subgraphs to the current layout.
@param forNodeId the node to be layered and sequenced incrementally
@returns an {@link Object} that can be interpreted as a hint by {@link HierarchicLayoutCore}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIncrementalHintsFactory%23createLayerIncrementallyHint(Object) Online Documentation}*/
abstract createLayerIncrementallyHint(forNodeId: Object | null): Object | null;
/**
Creates a hint {@link Object} for a {@link YNode} or {@link Edge} that should be inserted incrementally during the
sequencing phase.
All nodes that are not incrementally inserted during the layering phase (see
{@link IIncrementalHintsFactory#createLayerIncrementallyHint}) stay in their current layer. More precisely,
the default {@link HierarchicLayout#fixedElementsLayerer} implementation is an instance of {@link AsIsLayerer} that determines the layer of the "fixed"
nodes by analyzing the current node coordinates. Note that for the other nodes, the layout algorithm may insert new
layers in between the existing layers.
For edges associated with this hint (or which are incident to an incremental node), the algorithm automatically
calculates a suitable sequencing (position within the layer). For edges between two non-incremental nodes that do not
have a hint, the sequencing phase considers the current route.
@param forItemId the node or edge to be sequenced/inserted incrementally
@returns an {@link Object} that can be interpreted as a hint by {@link HierarchicLayoutCore}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIncrementalHintsFactory%23createSequenceIncrementallyHint(Object) Online Documentation}*/
abstract createSequenceIncrementallyHint(forItemId: Object | null): Object | null;
/**
Creates a hint {@link Object} for a {@link YNode} that should be inserted incrementally into the graph on its exact current
position.
As for nodes without hints, the algorithm uses the {@link HierarchicLayout#fixedElementsLayerer} instance to determine the layer of such nodes and the {@link HierarchicLayout#fixedElementsSequencer} instance to
determine the position within the layer. By default, both implementations use the current coordinates of the nodes to
derive the layer/position. However, for nodes marked with this hint, the algorithm should also preserve the exact
coordinates while for nodes without hints the coordinates may be changed (these nodes only keep their relative
positions).
<b>Warning:</b> Using exact coordinate hints may lead to drawing artifacts like node overlaps and node-edge intersections.
@param forNodeId the node to be placed on its exact current position
@returns an {@link Object} that can be interpreted as a hint by {@link HierarchicLayoutCore}
@see IIncrementalHintsFactory#createUseExactSequenceCoordinatesHint
@see IIncrementalHintsFactory#createUseExactLayerCoordinatesHint
@see SimplexNodePlacer#exactPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIncrementalHintsFactory%23createUseExactCoordinatesHint(Object) Online Documentation}*/
abstract createUseExactCoordinatesHint(forNodeId: Object | null): Object | null;
/**
Creates a hint {@link Object} for a {@link YNode} that should be inserted incrementally into the graph on its exact current
layer position.
The algorithm uses the {@link HierarchicLayout#fixedElementsLayerer} instance to determine the layer of such nodes and the {@link HierarchicLayout#fromScratchSequencer} instance to determine the sequencing.
<b>Warning:</b> Using exact coordinate hints may lead to drawing artifacts like node overlaps and node-edge intersections.
@param forNodeId the node to be placed on its exact current layer position
@returns an {@link Object} that can be interpreted as a hint by {@link HierarchicLayoutCore}
@see IIncrementalHintsFactory#createUseExactCoordinatesHint
@see IIncrementalHintsFactory#createUseExactSequenceCoordinatesHint
@see SimplexNodePlacer#exactPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIncrementalHintsFactory%23createUseExactLayerCoordinatesHint(Object) Online Documentation}*/
abstract createUseExactLayerCoordinatesHint(forNodeId: Object | null): Object | null;
/**
Creates a hint {@link Object} for a {@link YNode} that should be inserted incrementally into the graph on its exact current
sequence position.
The algorithm uses the {@link HierarchicLayout#fromScratchLayerer} instance to determine the layer of such nodes and the {@link HierarchicLayout#fixedElementsSequencer} instance to determine the sequencing.
<b>Warning:</b> Using exact coordinate hints may lead to drawing artifacts like node overlaps and node-edge intersections.
@param forNodeId the node to be placed on its exact current sequence position
@returns an {@link Object} that can be interpreted as a hint by {@link HierarchicLayoutCore}
@see IIncrementalHintsFactory#createUseExactCoordinatesHint
@see IIncrementalHintsFactory#createUseExactLayerCoordinatesHint
@see SimplexNodePlacer#exactPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/IIncrementalHintsFactory%23createUseExactSequenceCoordinatesHint(Object) Online Documentation}*/
abstract createUseExactSequenceCoordinatesHint(forNodeId: Object | null): Object | null;
static $class: Class;
private $_IIncrementalHintsFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IIncrementalHintsFactory;
}
export interface IItemFactory {
}
/**
This interface is a factory for creating and destroying helper structures used in the graph during layout.
An implementation of this interface is provided by the {@link HierarchicLayoutCore} during the layout for the
different sub-modules.
<b>Warning:</b> It is mainly for internal use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory Online Documentation}*/
export abstract class IItemFactory {
/**
Converts a given {@link YNode} to a label node.
@param dummyNode the given {@link YNode} to be converted
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23convertToLabelNode(YNode) Online Documentation}*/
abstract convertToLabelNode(dummyNode: YNode | null): void;
/**
Creates a bend {@link YNode} for the given {@link Edge} in the given {@link ILayer} and
assigns it to the given {@link SwimlaneDescriptor} instance.
@param layer the given {@link ILayer} instance
@param edge the given {@link Edge}
@param [laneDescriptor=null] a given {@link SwimlaneDescriptor} instance
@returns the new bend {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createBendNode(ILayer%2CEdge%2CSwimlaneDescriptor) Online Documentation}*/
abstract createBendNode(layer: ILayer | null, edge: Edge | null, laneDescriptor?: SwimlaneDescriptor | null): YNode | null;
/**
Creates a {@link INodeData} object for a bend {@link YNode} and associates it with the
{@link YNode}.
The bend is assigned to the given {@link SwimlaneDescriptor} instance.
@param node the given {@link YNode}
@param edge the given {@link Edge}
@param [laneDescriptor=null] the given {@link SwimlaneDescriptor} instance
@returns a {@link INodeData} object for the given bend {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createBendNodeData(YNode%2CEdge%2CSwimlaneDescriptor) Online Documentation}*/
abstract createBendNodeData(node: YNode | null, edge: Edge | null, laneDescriptor?: SwimlaneDescriptor | null): INodeData | null;
/**
Creates a dummy edge that models the bus segment of a {@link HierarchicLayout#BUS_DESCRIPTOR_DP_KEY bus substructure}.
Such an edge is created between the bus root node and the first {@link IItemFactory#createBusDummyNode bus dummy node} and between bus dummy nodes on
consecutive layers.
@param sourceBusNode the source bus dummy node or the root node of the bus substructure
@param targetBusNode the target bus dummy node or the root node of the bus substructure
@param edgeLayoutDescriptor the descriptor which is assigned to the new dummy edge
@param groupingId the edge grouping id assigned to the new dummy edge
@param groupingAtSource <code>true</code> if the dummy edge should be grouped at the source side, <code>false</code> if it should be grouped at the target side
@returns the newly created bus dummy edge connecting the two given nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createBusDummyEdge(YNode%2CYNode%2CHierarchicLayoutEdgeLayoutDescriptor%2CObject%2Cboolean) Online Documentation}*/
abstract createBusDummyEdge(sourceBusNode: YNode | null, targetBusNode: YNode | null, edgeLayoutDescriptor: HierarchicLayoutEdgeLayoutDescriptor | null, groupingId: any | null, groupingAtSource: boolean): Edge | null;
/**
Creates an {@link IEdgeData} instance for a {@link IItemFactory#createBusDummyEdge bus dummy edge}.
@param busDummyEdge the bus dummy edge
@param edgeLayoutDescriptor the descriptor which is assigned to the given bus dummy edge
@param groupingId the grouping id assigned to the given bus dummy edge
@param groupingAtSource <code>true</code> if the dummy edge is grouped at the source side, <code>false</code> if it is grouped at the target side
@returns the newly created {@link IEdgeData} for the given bus dummy edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createBusDummyEdgeData(Edge%2CHierarchicLayoutEdgeLayoutDescriptor%2CObject%2Cboolean) Online Documentation}*/
abstract createBusDummyEdgeData(busDummyEdge: Edge | null, edgeLayoutDescriptor: HierarchicLayoutEdgeLayoutDescriptor | null, groupingId: any | null, groupingAtSource: boolean): IEdgeData | null;
/**
Creates a dummy node that is used during the sequencing and drawing phase for the {@link HierarchicLayout#BUS_DESCRIPTOR_DP_KEY bus-style routing} edges at a certain
root node.
@param layer the layer to which the created dummy node is added
@param groupNode the group node to which the created dummy node is added
@param laneDescriptor the lane to which the dummy should belong to
@returns the newly created bus dummy node
@see HierarchicLayout#BUS_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createBusDummyNode(ILayer%2CYNode%2CSwimlaneDescriptor) Online Documentation}*/
abstract createBusDummyNode(layer: ILayer | null, groupNode: YNode | null, laneDescriptor: SwimlaneDescriptor | null): YNode | null;
/**
Creates a {@link INodeData} instance for a {@link IItemFactory#createBusDummyNode bus dummy node}.
@param busDummyNode the bus dummy node for which the data is created
@param groupNode the group node to which the bus dummy node belongs
@param laneDescriptor the lane to which the bus dummy node belongs
@returns the newly created {@link INodeData} instance for the given bus dummy node
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createBusDummyNodeData(YNode%2CYNode%2CSwimlaneDescriptor) Online Documentation}*/
abstract createBusDummyNodeData(busDummyNode: YNode | null, groupNode: YNode | null, laneDescriptor: SwimlaneDescriptor | null): INodeData | null;
/**
Creates a dummy {@link YNode} that mimics a connector to a group {@link YNode}.
@param groupNode the given group {@link YNode}
@param groupId the given group ID
@param layer the given {@link ILayer} instance
@param edge the given {@link Edge}
@returns the new {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createConnectorProxyForGroup(YNode%2CObject%2CILayer%2CEdge) Online Documentation}*/
abstract createConnectorProxyForGroup(groupNode: YNode | null, groupId: any | null, layer: ILayer | null, edge: Edge | null): YNode | null;
/**
Creates a dummy node that mimics a connector to a group node from a descendant.
@param groupNode the group node to which the edge is connected
@param groupId the id of the group node
@param layer the layer to which the dummy node will be assigned
@param e the edge that connects to the dummy node
@returns the dummy node
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createContentConnectorProxyForGroup(YNode%2CObject%2CILayer%2CEdge) Online Documentation}*/
abstract createContentConnectorProxyForGroup(groupNode: YNode | null, groupId: any | null, layer: ILayer | null, e: Edge | null): YNode | null;
/**
Creates a spacer {@link YNode} for the drawing phase using the given bounds in the given
{@link ILayer}.
@param layer the given {@link ILayer} instance
@param size the given bounds
@param edges an array of {@link Edge}s
@returns the new spacer {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createDistanceNode(ILayer%2CRectangle2D%2CArray%3CEdge%3E) Online Documentation}*/
abstract createDistanceNode(layer: ILayer | null, size: Rectangle2D | null, edges: Edge[] | null): YNode | null;
/**
Creates a dummy {@link Edge} using the given data.
@param newSource the source node of the {@link Edge}
@param newTarget the target node of the {@link Edge}
@param oldEdge the old {@link Edge} from which to retrieve the data
@param sourceEnd <code>true</code> if the source node of the edge is referred, <code>false</code> otherwise
@param targetEnd <code>true</code> if the target node of the edge is referred, <code>false</code> otherwise
@returns a new dummy {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createDummyEdge(YNode%2CYNode%2CEdge%2Cboolean%2Cboolean) Online Documentation}*/
abstract createDummyEdge(newSource: YNode | null, newTarget: YNode | null, oldEdge: Edge | null, sourceEnd: boolean, targetEnd: boolean): Edge | null;
/**
Creates an edge group {@link YNode} in the given {@link ILayer}.
@param layer the given {@link ILayer} instance
@param groupId the ID of the edge group
@returns the new group {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createEdgeGroupNode(ILayer%2CObject) Online Documentation}*/
abstract createEdgeGroupNode(layer: ILayer | null, groupId: any | null): YNode | null;
/**
Creates a {@link INodeData} object for an edge group {@link YNode} and associates it with the
{@link YNode}.
@param node the given {@link YNode}
@param groupId the given group ID
@param source <code>true</code> if the edge group is referred to the source node, <code>false</code> otherwise
@returns a {@link INodeData} object for the given edge group {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createEdgeGroupNodeData(YNode%2CObject%2Cboolean) Online Documentation}*/
abstract createEdgeGroupNodeData(node: YNode | null, groupId: any | null, source: boolean): INodeData | null;
/**
Creates an {@link Edge} that connects two group border {@link YNode}s.
@param source the source {@link YNode}
@param target the target {@link YNode}
@returns the new {@link Edge} connecting the two group {@link YNode}s
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createGroupBorderEdge(YNode%2CYNode) Online Documentation}*/
abstract createGroupBorderEdge(source: YNode | null, target: YNode | null): Edge | null;
/**
Creates a group boundary {@link YNode} for a group node in the given {@link ILayer}.
@param groupNode the given group {@link YNode}
@param layer the given {@link ILayer}
@param type one of {@link NodeDataType#GROUP_BEGIN} or {@link NodeDataType#GROUP_END}
@returns the new group boundary {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createGroupBorderNode(YNode%2CILayer%2CNodeDataType) Online Documentation}*/
abstract createGroupBorderNode(groupNode: YNode | null, layer: ILayer | null, type: NodeDataType | NodeDataTypeStringValues): YNode | null;
/**
Create a dummy {@link YNode} for a group {@link ILayer layer}.
@returns a new dummy {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createGroupLayerDummyNode() Online Documentation}*/
abstract createGroupLayerDummyNode(): YNode | null;
/**
Creates a group node connector {@link Edge} between two group {@link YNode}s.
@param groupNode1 the first group node
@param groupNode2 the first group node
@param representative the given representative {@link Edge}
@returns a new connector {@link Edge} between two group {@link YNode}s
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createGroupNodeConnectorEdge(YNode%2CYNode%2CEdge) Online Documentation}*/
abstract createGroupNodeConnectorEdge(groupNode1: YNode | null, groupNode2: YNode | null, representative: Edge | null): Edge | null;
/**
Creates an {@link IEdgeData} object for a normal {@link Edge} and associates it with the edge.
@param edge the given {@link Edge}
@returns an {@link IEdgeData} object for the given {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createNormalEdgeData(Edge) Online Documentation}*/
abstract createNormalEdgeData(edge: Edge | null): IEdgeData | null;
/**
Creates a {@link INodeData} object for a normal {@link YNode} and associates it with the
{@link YNode}.
@param node the given {@link YNode}
@returns a {@link INodeData} object for the given {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createNormalNodeData(YNode) Online Documentation}*/
abstract createNormalNodeData(node: YNode | null): INodeData | null;
/**
Creates a proxy {@link YNode} for an {@link Edge} during the drawing phase, changing the {@link Edge}
to end/start at the proxy and assigns it to a {@link SwimlaneDescriptor} instance.
@param edge the given {@link Edge}
@param source <code>true</code> if the node is the source node of the edge, <code>false</code> otherwise
@param [laneDescriptor=null] a given {@link SwimlaneDescriptor} instance
@returns the new proxy {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createProxyNode(Edge%2Cboolean%2CSwimlaneDescriptor) Online Documentation}*/
abstract createProxyNode(edge: Edge | null, source: boolean, laneDescriptor?: SwimlaneDescriptor | null): YNode | null;
/**
Creates a {@link INodeData} object for a proxy {@link YNode} and associates it with the proxy node.
@param node the given {@link YNode}
@param proxy the given proxy node
@param edge the given {@link Edge}
@returns a {@link INodeData} object for the given proxy node
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createProxyNodeData(YNode%2CYNode%2CEdge) Online Documentation}*/
abstract createProxyNodeData(node: YNode | null, proxy: YNode | null, edge: Edge | null): INodeData | null;
/**
Creates a dummy edge that belongs to a recursive {@link Edge}.
@param newSource the source node of the {@link Edge}
@param newTarget the target node of the {@link Edge}
@param oldEdge the old {@link Edge} from which to retrieve the data
@param sourceEnd <code>true</code> if the source node of the edge is referred, <code>false</code> otherwise
@param targetEnd <code>true</code> if the target node of the edge is referred, <code>false</code> otherwise
@returns a new dummy {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createRecursiveDummyEdge(YNode%2CYNode%2CEdge%2Cboolean%2Cboolean) Online Documentation}*/
abstract createRecursiveDummyEdge(newSource: YNode | null, newTarget: YNode | null, oldEdge: Edge | null, sourceEnd: boolean, targetEnd: boolean): Edge | null;
/**
Creates a dummy {@link YNode} that represents a bend node of a recursive {@link Edge}.
@param layer the given {@link ILayer} to which the dummy {@link YNode} is assigned
@param edge the given {@link Edge} to which the dummy {@link YNode} belongs
@param laneDescriptor the given {@link SwimlaneDescriptor} instance
@returns the new {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createRecursiveEdgeDummy(ILayer%2CEdge%2CSwimlaneDescriptor) Online Documentation}*/
abstract createRecursiveEdgeDummy(layer: ILayer | null, edge: Edge | null, laneDescriptor: SwimlaneDescriptor | null): YNode | null;
/**
Creates a {@link INodeData} for a dummy {@link YNode} of a recursive {@link Edge}.
@param node the given {@link YNode}
@param edge the given {@link Edge} to which the dummy {@link YNode} belongs
@param laneDescriptor the given {@link SwimlaneDescriptor} instance
@returns the new {@link INodeData} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createRecursiveEdgeNodeData(YNode%2CEdge%2CSwimlaneDescriptor) Online Documentation}*/
abstract createRecursiveEdgeNodeData(node: YNode | null, edge: Edge | null, laneDescriptor: SwimlaneDescriptor | null): INodeData | null;
/**
Creates an {@link IEdgeData} object for a same-layer recursive {@link Edge} and associates it with
the edge.
@param edge the given {@link Edge}
@param originalEdgeData the {@link IEdgeData} object of the original edge
@param isUpper <code>true</code> is the recursive edge is an upper same layer edge, <code>false</code> otherwise
@returns an {@link IEdgeData} object for the recursive same-layer {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createRecursiveSameLayerEdgeData(Edge%2CIEdgeData%2Cboolean) Online Documentation}*/
abstract createRecursiveSameLayerEdgeData(edge: Edge | null, originalEdgeData: IEdgeData | null, isUpper: boolean): IEdgeData | null;
/**
Creates a redirected {@link Edge} to replace an {@link Edge} connected to a group
{@link YNode}.
@param newSource the source node of the {@link Edge}
@param newTarget the target node of the {@link Edge}
@param groupEdge the given group {@link Edge}
@returns a new redirected {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createRedirectedGroupEdge(YNode%2CYNode%2CEdge) Online Documentation}*/
abstract createRedirectedGroupEdge(newSource: YNode | null, newTarget: YNode | null, groupEdge: Edge | null): Edge | null;
/**
Creates a reversed dummy {@link Edge} using the given data.
@param newSource the source node of the {@link Edge}
@param newTarget the target node of the {@link Edge}
@param oldEdge the old {@link Edge} from which to retrieve the data
@param sourceEnd <code>true</code> if the source node of the edge is referred, <code>false</code> otherwise
@param targetEnd <code>true</code> if the target node of the edge is referred, <code>false</code> otherwise
@returns a new reversed dummy {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createReverseDummyEdge(YNode%2CYNode%2CEdge%2Cboolean%2Cboolean) Online Documentation}*/
abstract createReverseDummyEdge(newSource: YNode | null, newTarget: YNode | null, oldEdge: Edge | null, sourceEnd: boolean, targetEnd: boolean): Edge | null;
/**
Creates an {@link IEdgeData} object for a same-layer {@link Edge} and associates it with the edge.
@param edge the given {@link Edge}
@returns an {@link IEdgeData} object for the same-layer {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createSameLayerEdgeData(Edge) Online Documentation}*/
abstract createSameLayerEdgeData(edge: Edge | null): IEdgeData | null;
/**
Creates a same-layer {@link Edge} proxy connecting a newly created proxy {@link YNode} and a given
proxy {@link YNode} in the given {@link ILayer}.
@param layer the given {@link ILayer}
@param edge the given {@link Edge}
@param toProxy the given proxy {@link YNode}
@returns a new redefined same-layer {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createSameLayerProxy(ILayer%2CEdge%2CYNode) Online Documentation}*/
abstract createSameLayerProxy(layer: ILayer | null, edge: Edge | null, toProxy: YNode | null): Edge | null;
/**
Creates a proxy {@link YNode} for a same-layer {@link Edge} during the drawing phase that ends at the
side of a node.
@param inLayer the given {@link ILayer} instance
@param forNode the given {@link YNode}
@param edge the given {@link Edge}
@param [sld=null] a given {@link SwimlaneDescriptor} instance
@returns the new proxy {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createSameLayerSideProxy(ILayer%2CYNode%2CEdge%2CSwimlaneDescriptor) Online Documentation}*/
abstract createSameLayerSideProxy(inLayer: ILayer | null, forNode: YNode | null, edge: Edge | null, sld?: SwimlaneDescriptor | null): YNode | null;
/**
Creates a same-layer switch proxy {@link YNode} (switching between two same-layer
{@link Edge}s on two different sides of the layer).
@param layer the given {@link ILayer}
@param edge the given {@link Edge}
@returns the new same-layer switch proxy {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createSameLayerSwitchProxy(ILayer%2CEdge) Online Documentation}*/
abstract createSameLayerSwitchProxy(layer: ILayer | null, edge: Edge | null): YNode | null;
/**
Creates an {@link IEdgeData} object for a self-loop {@link Edge} and associates it with the edge.
@param edge the given {@link Edge}
@returns an {@link IEdgeData} object for a self-loop {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23createSelfLoopEdgeData(Edge) Online Documentation}*/
abstract createSelfLoopEdgeData(edge: Edge | null): IEdgeData | null;
/**
Destroys a spacer {@link YNode} for the drawing phase created using
{@link IItemFactory#createDistanceNode}.
@param distanceNode the {@link YNode} to be destroyed
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23destroyDistanceNode(YNode) Online Documentation}*/
abstract destroyDistanceNode(distanceNode: YNode | null): void;
/**
Removes the given layer for proxies at groups to handle direct group content connector edges between a node in the
first/last layer of the group node and the groups border.
@param layer the layer to destroy
@param useInEdges whether the incoming edges should be used as the resulting edges
@see HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23destroyGroupConnectorLayer(ILayer%2Cboolean) Online Documentation}*/
abstract destroyGroupConnectorLayer(layer: ILayer | null, useInEdges: boolean): void;
/**
Destroys a previously created label layer
@param layer the layer to destroy
@param useInEdges whether the incoming edges should be used as the resulting edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23destroyLabelNodeLayer(ILayer%2Cboolean) Online Documentation}*/
abstract destroyLabelNodeLayer(layer: ILayer | null, useInEdges: boolean): void;
/**
Destroys a proxy {@link YNode} created using {@link IItemFactory#createProxyNode} for the drawing
phase.
@param proxyNode the proxy {@link YNode} to be destroyed
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23destroyProxyNode(YNode) Online Documentation}*/
abstract destroyProxyNode(proxyNode: YNode | null): Edge | null;
/**
Destroys a same-layer {@link Edge} proxy created using {@link IItemFactory#createSameLayerProxy}.
@param edge the {@link Edge} whose proxy will be destroyed
@returns the redefined {@link Edge} after the removal of the proxy {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23destroySameLayerProxy(Edge) Online Documentation}*/
abstract destroySameLayerProxy(edge: Edge | null): Edge | null;
/**
Destroys a proxy {@link YNode} created using {@link IItemFactory#createProxyNode} for the drawing
phase.
@param proxyNode the proxy {@link YNode} to be destroyed
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23destroySameLayerSideProxy(YNode) Online Documentation}*/
abstract destroySameLayerSideProxy(proxyNode: YNode | null): void;
/**
Destroys a same-layer switch proxy {@link YNode} created using
{@link IItemFactory#createSameLayerSwitchProxy}.
@param node the {@link YNode} to be destroyed
@returns the redefined {@link Edge} after the removal of the same-layer switch proxy {@link YNode}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23destroySameLayerSwitchProxy(YNode) Online Documentation}*/
abstract destroySameLayerSwitchProxy(node: YNode | null): Edge | null;
/**
Inserts a same-layer {@link Edge} or an {@link Edge} that may span multiple layers into the data
structure and returns the resulting list of
{@link Edge}s that has been created if this edge spans multiple layers.
@param edge the given {@link Edge}
@returns the resulting list of {@link Edge}s that has been created if this edge spans multiple layers
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23insertEdge(Edge) Online Documentation}*/
abstract insertEdge(edge: Edge | null): EdgeList | null;
/**
Inserts a layer for proxies at groups to handle direct group content connector edges between a node in the first/last
layer of the group node and the groups border.
@param top <code>true</code> if this layer is placed above a group nodes, <code>false</code> if placed below
@param index the index where the layer is inserted
@returns the new layer
@see HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23insertGroupConnectorLayer(boolean%2Cnumber) Online Documentation}*/
abstract insertGroupConnectorLayer(top: boolean, index: number): ILayer | null;
/**
Inserts a {@link ILayer layer} for group {@link YNode}s.
@param source <code>true</code> if the group refers to the source node of an edge, <code>false</code> otherwise
@param index the index at which the specified {@link ILayer layer} is to be inserted
@returns a {@link ILayer layer} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23insertGroupNodeLayer(boolean%2Cnumber) Online Documentation}*/
abstract insertGroupNodeLayer(source: boolean, index: number): ILayer | null;
/**
Inserts a {@link ILayer} for labels.
@param source <code>true</code> if the source node of the edge, <code>false</code> otherwise
@param index the index at which the specified {@link ILayer} is to be inserted
@returns a {@link ILayer layer} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23insertLabelNodeLayer(boolean%2Cnumber) Online Documentation}*/
abstract insertLabelNodeLayer(source: boolean, index: number): ILayer | null;
/**
Registers an {@link Edge} as a same-layer {@link Edge} appropriately into all data structures.
@param edge the given {@link Edge} to be registered
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23registerSameLayerEdge(Edge) Online Documentation}*/
abstract registerSameLayerEdge(edge: Edge | null): void;
/**
Converts a same-layer {@link Edge} to an {@link Edge} of type
{@link EdgeDataType#DIRECT_SAME_LAYER_EDGE}.
@param sameLayerEdge the given same-layer {@link Edge}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23setDirectlyConnectSameLayerEdge(Edge) Online Documentation}*/
abstract setDirectlyConnectSameLayerEdge(sameLayerEdge: Edge | null): void;
/**
Specifies the critical edge priority for an {@link Edge} and defines it in an {@link IEdgeData}
object.
@param edge the given edge
@param priority the new priority for the given edge
@returns an {@link IEdgeData} object with an adjusted critical edge priority
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23setTemporaryCriticalEdgePriority(Edge%2Cnumber) Online Documentation}*/
abstract setTemporaryCriticalEdgePriority(edge: Edge | null, priority: number): IEdgeData | null;
/**
Specifies the crossing cost for an {@link Edge} and defines it in an {@link IEdgeData} object.
@param edge the given edge
@param crossingCost the new crossing cost for the given edge
@returns an {@link IEdgeData} object with an adjusted edge crossing cost
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23setTemporaryCrossingCost(Edge%2Cnumber) Online Documentation}*/
abstract setTemporaryCrossingCost(edge: Edge | null, crossingCost: number): IEdgeData | null;
/**
Specifies an edge group constraint for an {@link Edge} and assigns it to an {@link IEdgeData}
object.
@param edge the given {@link Edge}
@param sgId the ID of the edge group at the source node
@param tgId the ID of the edge group at the target node
@returns an {@link IEdgeData} object
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23setTemporaryEdgeGroups(Edge%2CObject%2CObject) Online Documentation}*/
abstract setTemporaryEdgeGroups(edge: Edge | null, sgId: any | null, tgId: any | null): IEdgeData | null;
/**
Specifies the thickness for an {@link Edge} and defines it in an {@link IEdgeData} object.
@param edge the given {@link Edge}
@param thickness the new edge thickness
@returns an {@link IEdgeData} object with an adjusted edge thickness
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23setTemporaryEdgeThickness(Edge%2Cnumber) Online Documentation}*/
abstract setTemporaryEdgeThickness(edge: Edge | null, thickness: number): IEdgeData | null;
/**
Specifies a port constraint for an {@link Edge} at the given side and assigns it to an {@link IEdgeData}
object.
@param edge the given {@link Edge}
@param source <code>true</code> of the port constraint is defined on the source node of the {@link Edge}, <code>false</code> otherwise
@param pc the given {@link PortConstraint} instance
@returns an {@link IEdgeData} object
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23setTemporaryPortConstraint(Edge%2Cboolean%2CPortConstraint) Online Documentation}*/
abstract setTemporaryPortConstraint(edge: Edge | null, source: boolean, pc: PortConstraint | null): IEdgeData | null;
/**
Reverts a label node to the normal {@link YNode}.
@param labelNode the given label node to be converted
@see {@link https://docs.yworks.com/yfileshtml/#/api/IItemFactory%23unconvertToLabelNode(YNode) Online Documentation}*/
abstract unconvertToLabelNode(labelNode: YNode | null): void;
static $class: Class;
private $_IItemFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IItemFactory;
}
export interface ILayer {
}
/**
This interface describes a single layer in a hierarchic layout.
A layer consists of nodes along with the associated same-layer edges. Each layer has a {@link ILayer#type type} and an index representing
its order in the {@link ILayers list of layers}.
@see ILayers
@see HierarchicLayoutCore
@see ILayerer
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer Online Documentation}*/
export abstract class ILayer {
/**
Adds a newly created {@link YNode} to this {@link ILayer} instance.
@param node the {@link YNode} to be added
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23add(YNode) Online Documentation}*/
abstract add(node: YNode | null): void;
/**
Adds a same-layer {@link Edge} to this {@link ILayer} instance.
@param edge the same-layer {@link Edge} to be added
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23addSameLayerEdge(Edge) Online Documentation}*/
abstract addSameLayerEdge(edge: Edge | null): void;
/**
Removes a {@link YNode} from this {@link ILayer} instance.
@param node the {@link YNode} to be removed
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23remove(YNode) Online Documentation}*/
abstract remove(node: YNode | null): void;
/**
Removes the current layer from the {@link ILayers list of layers}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23remove() Online Documentation}*/
abstract remove(): void;
/**
Adjusts the order of the {@link YNode}s of this {@link ILayer} instance according to the given
order.
@param list the given list to be used
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23setNodeOrder(YList) Online Documentation}*/
abstract setNodeOrder(list: YList | null): void;
/**
Gets a list of all same-layer {@link Edge}s associated with this {@link ILayer} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23sameLayerEdges Online Documentation}*/
abstract readonly sameLayerEdges: YList | null;
/**
Gets a list of the {@link YNode}s associated with this {@link ILayer} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23list Online Documentation}*/
abstract readonly list: YNodeList | null;
/**
Gets the type of this {@link ILayer} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23type Online Documentation}*/
abstract readonly type: LayerType;
/**
Gets the index of this {@link ILayer} instance in the {@link ILayers list of all layers}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23index Online Documentation}*/
abstract readonly index: number;
/**
Gets or sets the {@link RowDescriptor} associated with this {@link ILayer} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayer%23row Online Documentation}*/
abstract row: RowDescriptor | null;
static $class: Class;
private $_ILayer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayer;
}
export interface ILayeredComponentsMerger {
}
/**
This interface merges two {@link ILayers} instances whose nodes reside in the same graph.
Implementations of this interface may provide different strategies for merging.
{@link DefaultLayeredComponentsMerger Default implementations} of this interface are used internally by {@link HierarchicLayoutCore} and {@link MultiComponentLayerer}
when e.g. the graph is grouped or it consists of more than one connected component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayeredComponentsMerger Online Documentation}*/
export abstract class ILayeredComponentsMerger {
/**
Merges two {@link ILayers} instances.
All nodes that belong to the source and target {@link ILayers} are part of the graph at the moment of
invocation.
The state of the source {@link ILayers} instance is discarded after this call and does not need to be updated
to reflect the changes. The target {@link ILayers} instance must be updated accordingly.
@param graph the graph containing the nodes of source and target {@link ILayers} instances
@param ldp the {@link ILayoutDataProvider} used for querying the layer indices
@param srcLayers the source {@link ILayers} instance that will be merged into the target {@link ILayers} instance
@param targetLayers the {@link ILayers} instance that will be modified to contain the resulting layering
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayeredComponentsMerger%23merge(LayoutGraph%2CILayoutDataProvider%2CILayers%2CILayers) Online Documentation}*/
abstract merge(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, srcLayers: ILayers | null, targetLayers: ILayers | null): void;
static $class: Class;
private $_ILayeredComponentsMerger: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayeredComponentsMerger;
}
export interface ILayerer {
}
/**
This interface is used by classes capable of generating a layer assignment.
Layer assignment takes place during the first part of the hierarchic layout algorithms. During this phase, each node of
the graph is assigned to a {@link ILayer layer}.
@see HierarchicLayoutCore#layerer
@see HierarchicLayout#fixedElementsLayerer
@see HierarchicLayout#fromScratchLayerer
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerer Online Documentation}*/
export abstract class ILayerer {
/**
Assigns all nodes of the graph to layers and adds them to the {@link ILayers} instance.
In order to create new layers, factory method {@link ILayers#insert} has to be used.
Information about the nodes is provided by a {@link ILayoutDataProvider}. However, positional information (see
{@link INodeData#position} and {@link INodeData#layer}) is not available during this phase.
@param graph the input graph
@param layers the {@link ILayers} instance that will be filled with the results of the calculation
@param ldp the {@link ILayoutDataProvider} used for querying information about the nodes and edges
@see ILayers#insert
@see ILayer#add
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayerer%23assignLayers(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
abstract assignLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
static $class: Class;
private $_ILayerer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayerer;
}
export interface ILayers {
}
/**
This interface manages multiple {@link ILayer} instances.
{@link HierarchicLayoutCore} creates instances of this class and passes them to the instances that handle the
various sub-tasks of the layout process during automatic layout.
Method {@link ILayers#createInstance} can be used as a factory method to obtain temporary {@link ILayers}
instances.
@see ILayer
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayers Online Documentation}*/
export abstract class ILayers {
/**
Creates a new and empty {@link ILayers} instance that can be used on the same graph instance for temporary
results.
@returns a new and empty {@link ILayers} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayers%23createInstance() Online Documentation}*/
abstract createInstance(): ILayers | null;
/**
Returns the {@link ILayer layer} at the specified position in this {@link ILayers list of layers}.
@param i the zero-based index of the {@link ILayer layer} to return
@returns the {@link ILayer layer} at the specified position
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayers%23getLayer(number) Online Documentation}*/
abstract getLayer(i: number): ILayer | null;
/**
Creates, inserts and returns a {@link ILayer layer} of a given type associated with this {@link ILayers list of layers}.
@param type one of the predefined layer types as defined in {@link ILayer} interface
@param position the index at which the specified {@link ILayer layer} is to be inserted
@returns a new {@link ILayer} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayers%23insert(LayerType%2Cnumber) Online Documentation}*/
abstract insert(type: LayerType | LayerTypeStringValues, position: number): ILayer | null;
/**
Removes the {@link ILayer layer} at the specified position from this {@link ILayers list of layers}.
@param index the zero-based index of the {@link ILayer layer} to remove
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayers%23remove(number) Online Documentation}*/
abstract remove(index: number): void;
/**
Returns the number of {@link ILayer layers} associated with this {@link ILayers} instance.
@returns the number of layers of this {@link ILayers} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayers%23size() Online Documentation}*/
abstract size(): number;
static $class: Class;
private $_ILayers: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayers;
}
export interface ILayoutDataProvider {
}
/**
This interface retrieves {@link INodeData} and {@link IEdgeData} instances for elements in the
current layout graph.
Instances of this interface are provided by {@link HierarchicLayoutCore} for instances implementing sub-tasks
during the layout.
@see HierarchicLayoutCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutDataProvider Online Documentation}*/
export abstract class ILayoutDataProvider {
/**
Returns the {@link IEdgeData} instance associated with the given {@link Edge}.
@param edge the given edge
@returns the {@link IEdgeData} instance associated with the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutDataProvider%23getEdgeData(Edge) Online Documentation}*/
abstract getEdgeData(edge: Edge | null): IEdgeData | null;
/**
Returns the {@link INodeData} instance associated with the given {@link YNode}.
@param node the given node
@returns the {@link INodeData} instance associated with the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutDataProvider%23getNodeData(YNode) Online Documentation}*/
abstract getNodeData(node: YNode | null): INodeData | null;
static $class: Class;
private $_ILayoutDataProvider: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayoutDataProvider;
}
export interface INodeData {
}
/**
This interface provides layout information about the nodes of the graph during the layout.
{@link INodeData} is a multi-purpose node descriptor for each node of the graph during the hierarchic layout
used internally by the algorithm implementations.
Instances of this interface can be obtained during the layout from instances of the {@link ILayoutDataProvider}
interface.
{@link INodeData} typically carries the {@link INodeData#type type} of the node and depending on the type an optional {@link INodeData#associatedEdge associated edge}
and
{@link INodeData#associatedNode associated node}. Optionally, it may carry a geometric description of the borders of the node and descriptors for various
aspects of the layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData Online Documentation}*/
export abstract class INodeData {
/**
Creates, stores and returns a {@link BorderLine} for the given side or returns the current {@link BorderLine}
instance if it already exists.
Not every {@link INodeData#type type} of node supports this operation. {@link NodeDataType#NORMAL} supports
{@link BorderLine}s.
Side value must be inside <code>[0,3]</code> interval, where <code>0</code> corresponds to the top side, <code>1</code> to the right side, <code>2</code> to the bottom side
and <code>3</code> to the left side.
@param side an integer value from <code>[0,3]</code> interval
@param nodeLayout the {@link INodeLayout} of the current node
@returns a {@link BorderLine} instance
@throws {Exception} <b>(NotSupportedError)</b> if this {@link INodeData#type type} of node does not support node borders
@see INodeData#getNormalizedBorderLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23createBorderLine(number%2CINodeLayout) Online Documentation}*/
abstract createBorderLine(side: number, nodeLayout: INodeLayout | null): BorderLine | null;
/**
Returns a {@link BorderLine} for the given side which is treated as if the position of the node is at (0,0).
Clients may modify this instance but should always reset it to the original position, since this is a shared instance.
It may return <code>null</code> to indicate that the {@link BorderLine} can be determined through the {@link INodeLayout}
instance.
Side value must be inside <code>[0,3]</code> interval, where <code>0</code> corresponds to the top side, <code>1</code> to the right side, <code>2</code> to the bottom side
and <code>3</code> to the left side.
@param side an integer value from <code>[0,3]</code> interval
@returns a {@link BorderLine} instance or <code>null</code>
@see INodeData#createBorderLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23getNormalizedBorderLine(number) Online Documentation}*/
abstract getNormalizedBorderLine(side: number): BorderLine | null;
/**
Returns the number of same-layer edges associated with the {@link YNode} of this {@link INodeData}
instance.
@returns the number of same-layer edges associated with the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23sameLayerEdgeCount() Online Documentation}*/
abstract sameLayerEdgeCount(): number;
/**
Gets the group node this node belongs to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23groupNode Online Documentation}*/
abstract readonly groupNode: YNode | null;
/**
Gets the first same-layer edge {@link ListCell} of all same-layer edges or <code>null</code> if there exists no same-layer
edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23firstSameLayerEdgeCell Online Documentation}*/
abstract readonly firstSameLayerEdgeCell: ListCell | null;
/**
Gets the type constant for the {@link YNode} of this {@link INodeData} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23type Online Documentation}*/
abstract readonly type: NodeDataType;
/**
Gets the group ID of the {@link YNode} of this {@link INodeData} instance if it is a bus-group node.
@see NodeDataType#SOURCE_GROUP_NODE
@see NodeDataType#TARGET_GROUP_NODE
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23groupId Online Documentation}*/
abstract readonly groupId: any | null;
/**
Gets the associated {@link YNode} of this {@link INodeData} instance (in case of e.g.
back-loop proxy and side proxy).
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23associatedNode Online Documentation}*/
abstract readonly associatedNode: YNode | null;
/**
Gets the associated {@link Edge} of the {@link YNode} of this {@link INodeData} instance
(in case of e.g.
bend or same-layer center node).
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23associatedEdge Online Documentation}*/
abstract readonly associatedEdge: Edge | null;
/**
Gets the index of the {@link ILayer} instance to which the {@link YNode} of this {@link INodeData}
instance belongs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23layer Online Documentation}*/
abstract readonly layer: number;
/**
Gets the current position of the {@link YNode} of this {@link INodeData} instance within its layer.
<h2>Note</h2>
This may not always be up-to-date depending on which phase the layout algorithm is currently executed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23position Online Documentation}*/
abstract readonly position: number;
/**
Gets the parent group node of the {@link YNode} of this {@link INodeData} instance (if any).
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23parentGroupNode Online Documentation}*/
abstract readonly parentGroupNode: YNode | null;
/**
Gets the inner layers of the group {@link YNode} of this {@link INodeData} instance.
If this node is of type {@link NodeDataType#GROUP}, this method returns the {@link ILayers} object
that describes the layering in the subgraph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23groupLayers Online Documentation}*/
abstract readonly groupLayers: ILayers | null;
/**
Gets the hint of the incremental mode of the {@link YNode} of this {@link INodeData} instance if this
node is of type {@link NodeDataType#NORMAL}.
The corresponding hint has been set by a {@link IDataProvider} using key
{@link HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23incrementalHint Online Documentation}*/
abstract readonly incrementalHint: Object | null;
/**
Gets the {@link HierarchicLayoutNodeLayoutDescriptor} instance initially bound to the {@link YNode} of this {@link INodeData}
instance or <code>null</code> if no {@link HierarchicLayoutNodeLayoutDescriptor} has been registered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23nodeLayoutDescriptor Online Documentation}*/
abstract readonly nodeLayoutDescriptor: HierarchicLayoutNodeLayoutDescriptor | null;
/**
Gets the {@link SwimlaneDescriptor} instance initially bound to the {@link YNode} of this {@link INodeData}
instance or <code>null</code> if no {@link SwimlaneDescriptor} has been registered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeData%23swimLaneDescriptor Online Documentation}*/
abstract readonly swimLaneDescriptor: SwimlaneDescriptor | null;
static $class: Class;
private $_INodeData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeData;
}
export interface IHierarchicLayoutNodePlacer {
}
/**
This interface is responsible for the assignment of the x-(sequence) and preliminary y-coordinates of the nodes in a
hierarchic layout.
Implementations of this interface are used during the drawing phase of
{@link HierarchicLayoutCore}. They determine preliminary y-coordinates (which may be adjusted during the final
edge routing) and the resulting x-coordinates of all elements that are part of the graph during the node placement
phase.
The minimum distance between each pair of nodes of the graph is determined by a {@link IDrawingDistanceCalculator}
instance.
The default implementation used by the {@link HierarchicLayoutCore} is {@link SimplexNodePlacer}.
Custom implementations of {@link IHierarchicLayoutNodePlacer} interface can be used for hierarchic layout algorithms using
methods {@link HierarchicLayoutCore#nodePlacer} or {@link HierarchicLayout#nodePlacer}.
@see SimplexNodePlacer
@see IDrawingDistanceCalculator
@see HierarchicLayoutCore
@see HierarchicLayoutCore#nodePlacer
@see HierarchicLayout#nodePlacer
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHierarchicLayoutNodePlacer Online Documentation}*/
export abstract class IHierarchicLayoutNodePlacer {
/**
Assigns preliminary y-coordinates for each layer of a hierarchic layout.
The distance between two layers will be adjusted later by the edge routing algorithm. This method is responsible for
assigning the relative positions of the nodes within each layer.
@param graph the input graph
@param layoutDataProvider the {@link ILayoutDataProvider} containing information about the elements
@param layers the {@link ILayers} instance that will be calculated by this method
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHierarchicLayoutNodePlacer%23assignLayerCoordinates(LayoutGraph%2CILayoutDataProvider%2CILayers) Online Documentation}*/
abstract assignLayerCoordinates(graph: LayoutGraph | null, layoutDataProvider: ILayoutDataProvider | null, layers: ILayers | null): void;
/**
Determines the resulting x-coordinates of a hierarchic layout.
The minimum distance between each pair of nodes of the graph is determined by a {@link IDrawingDistanceCalculator}
instance.
@param graph the input graph
@param layoutDataProvider the {@link ILayoutDataProvider} containing information about the elements
@param layers the {@link ILayers} instance that will be calculated by this method
@param drawingDistanceCalculator the given {@link IDrawingDistanceCalculator} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/IHierarchicLayoutNodePlacer%23assignSequenceCoordinates(LayoutGraph%2CILayoutDataProvider%2CILayers%2CIDrawingDistanceCalculator) Online Documentation}*/
abstract assignSequenceCoordinates(graph: LayoutGraph | null, layoutDataProvider: ILayoutDataProvider | null, layers: ILayers | null, drawingDistanceCalculator: IDrawingDistanceCalculator | null): void;
static $class: Class;
private $_INodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IHierarchicLayoutNodePlacer;
}
export interface IPortAllocator {
}
/**
This interface is responsible for assigning port coordinates to the adjacent edges of each node of the graph.
{@link IPortAllocator} is used by {@link HierarchicLayoutCore} after the sequencing phase.
@see HierarchicLayoutCore
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortAllocator Online Documentation}*/
export abstract class IPortAllocator {
/**
Assigns to each edge of the graph a source port and target port coordinate pair.
This method is called by {@link HierarchicLayoutCore} after the sequencing phase.
@param graph the input graph
@param layers the given {@link ILayers} instance containing the elements in the layering
@param ldp the {@link ILayoutDataProvider} containing layering information about the nodes and edges of the graph
@param itemFactory the {@link IItemFactory} used temporarily for modifying the graph instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortAllocator%23assignPorts(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
abstract assignPorts(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
static $class: Class;
private $_IPortAllocator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortAllocator;
}
export interface IPortConstraintOptimizer {
}
/**
This interface serves as a callback for {@link HierarchicLayoutCore} after the layering and sequencing
phases.
Implementations of this interface may modify the {@link PortConstraint port constraint} information via
{@link IItemFactory#setTemporaryPortConstraint}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortConstraintOptimizer Online Documentation}*/
export abstract class IPortConstraintOptimizer {
/**
Assigns new temporary port constraints after the layering information has been determined.
@param graph the input graph
@param layers the given {@link ILayers} instance
@param ldp the {@link ILayoutDataProvider} containing information about the elements
@param itemFactory the {@link IItemFactory} to set the temporary port constraints with
@see IItemFactory#setTemporaryPortConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortConstraintOptimizer%23optimizeAfterLayering(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
abstract optimizeAfterLayering(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
Assigns new temporary port constraints after the sequence of the nodes has been determined.
@param graph the input graph
@param layers the given {@link ILayers} instance
@param ldp the {@link ILayoutDataProvider} containing information about the elements
@param itemFactory the {@link IItemFactory} to set the temporary port constraints with
@see IItemFactory#setTemporaryPortConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPortConstraintOptimizer%23optimizeAfterSequencing(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
abstract optimizeAfterSequencing(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
static $class: Class;
private $_IPortConstraintOptimizer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPortConstraintOptimizer;
}
export interface ISequencer {
}
/**
This interface is used by {@link HierarchicLayoutCore} to calculate the order of the nodes within each {@link ILayers layer}.
Implementations are used during the second phase of the hierarchic layout process and can be set using method {@link HierarchicLayout#fixedElementsSequencer} or {@link HierarchicLayout#fromScratchSequencer}).
@see HierarchicLayoutCore#sequencer
@see HierarchicLayout#fixedElementsSequencer
@see HierarchicLayout#fromScratchSequencer
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISequencer Online Documentation}*/
export abstract class ISequencer {
/**
Calculates the sequence of the nodes within each {@link ILayers layer}.
This method is called by {@link HierarchicLayoutCore} during the second phase and finally writes back the
calculated sequence using the {@link ILayer#setNodeOrder} method.
@param graph the input graph
@param layers the given {@link ILayers} instance containing the elements in the layering
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param itemFactory the {@link IItemFactory} used temporarily for modifying the graph instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISequencer%23sequenceNodeLayers(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
abstract sequenceNodeLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
static $class: Class;
private $_ISequencer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ISequencer;
}
export interface MultiComponentLayerer extends YObject, ILayerer {
/**
@inheritdoc*/
assignLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
} /**
This class is a wrapper {@link ILayerer} implementation that delegates the actual layering to a delegate
{@link ILayerer}.
If the graph consists of multiple components, they will be identified and sorted and each of the components will be
layered separately using the delegate {@link ILayerer} instance. After that they will all be merged using the
specified {@link ILayeredComponentsMerger} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiComponentLayerer Online Documentation}*/
export class MultiComponentLayerer {
/**
Creates a new instance of {@link MultiComponentLayerer} using the given {@link ILayeredComponentsMerger}
and {@link IComparer} instances.
If the given merger is <code>null</code>, the {@link DefaultLayeredComponentsMerger} will be used instead.
@param singleComponentLayerer
@param [merger=null]
@param [componentComparer=null]
@throws {Exception} <b>(ArgumentError)</b> if the given layerer is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiComponentLayerer%23MultiComponentLayerer(ILayerer%2CILayeredComponentsMerger%2CIComparer%3CObject%3E) Online Documentation}*/
constructor(singleComponentLayerer: ILayerer | null, merger?: ILayeredComponentsMerger | null, componentComparer?: IComparer<any> | null);
/**
Creates a new instance of {@link MultiComponentLayerer} using the given {@link ILayeredComponentsMerger}
and {@link IComparer} instances.
If the given merger is <code>null</code>, the {@link DefaultLayeredComponentsMerger} will be used instead.
@param options The parameters to pass.
@param options.singleComponentLayerer
@param [options.merger=null]
@param [options.componentComparer=null]
@throws {Exception} <b>(ArgumentError)</b> if the given layerer is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiComponentLayerer%23MultiComponentLayerer(ILayerer%2CILayeredComponentsMerger%2CIComparer%3CObject%3E) Online Documentation}*/
constructor(options: {
singleComponentLayerer: ILayerer | null;
merger?: ILayeredComponentsMerger | null;
componentComparer?: IComparer<any> | null;
});
/**
Sorts an array which consists of
{@link YNodeList}s each of which contains nodes that belong to the same component.
It is called during {@link MultiComponentLayerer#assignLayers}.
By default this method uses the {@link MultiComponentLayerer#componentComparer ComponentComparator} to sort the list and has no effect if the instance is <code>null</code>.
@param nodeLists the array of {@link YNodeList}s each of which containing nodes that belong to the same component
@param componentIndexProvider the {@link IDataProvider} that returns an object (the component index) for each node (this value can be either
integer or object)
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiComponentLayerer%23sort(Array%3CYNodeList%3E%2CIDataProvider) Online Documentation}*/
sort(nodeLists: YNodeList[] | null, componentIndexProvider: IDataProvider | null): void;
/**
Gets or sets the delegate {@link ILayerer} instance.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link ILayerer} is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiComponentLayerer%23singleComponentLayerer Online Documentation}*/
singleComponentLayerer: ILayerer | null;
/**
Gets or sets the {@link ILayeredComponentsMerger merger} instance.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link ILayeredComponentsMerger} is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiComponentLayerer%23merger Online Documentation}*/
merger: ILayeredComponentsMerger | null;
/**
Gets or sets the component {@link IComparer} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiComponentLayerer%23componentComparer Online Documentation}*/
componentComparer: IComparer<any> | null;
static $class: Class;
private $_MultiComponentLayerer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MultiComponentLayerer;
}
export interface HierarchicLayoutNodeLayoutDescriptor extends YObject {
}
/**
This class is used by {@link HierarchicLayoutCore} during the various phases to provide the drawing details of
the nodes of the graph.
A {@link HierarchicLayoutNodeLayoutDescriptor} instance can be specified individually for single nodes using a {@link IDataProvider}
that returns a {@link HierarchicLayoutNodeLayoutDescriptor} instance for each node of the graph, or <code>null</code> if no {@link HierarchicLayoutNodeLayoutDescriptor}
is bound to an edge. The {@link IDataProvider} is registered with the graph with key
{@link HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY}.
This class is designed as a class to allow future additions of new getter methods.
<h2>Note</h2>
Not all of these values will be used for all kinds of edges and any kind of algorithm.
@see HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY
@see HierarchicLayoutEdgeLayoutDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor Online Documentation}*/
export class HierarchicLayoutNodeLayoutDescriptor {
/**
Creates a new instance of a {@link HierarchicLayoutNodeLayoutDescriptor} with the default values.
@param [options] The parameters to pass.
@param [options.portBorderGapRatios] The port border gap ratio for the port distribution on all sides of the node. This option sets the {@link HierarchicLayoutNodeLayoutDescriptor#portBorderGapRatios} property on the created object.
@param [options.layerAlignment] The alignment of the {@link YNode node} of this {@link HierarchicLayoutNodeLayoutDescriptor} instance within its layer. This option sets the {@link HierarchicLayoutNodeLayoutDescriptor#layerAlignment} property on the created object.
@param [options.nodeLabelMode] The mode that determines how node labels are handled by the algorithm during the layout. This option sets the {@link HierarchicLayoutNodeLayoutDescriptor#nodeLabelMode} property on the created object.
@param [options.minimumLayerHeight] The minimum height of the layer to which this node is assigned. This option sets the {@link HierarchicLayoutNodeLayoutDescriptor#minimumLayerHeight} property on the created object.
@param [options.minimumDistance] The preferred minimum distance to obstacles. This option sets the {@link HierarchicLayoutNodeLayoutDescriptor#minimumDistance} property on the created object.
@param [options.gridReference] The node's reference point that will be placed on a grid coordinate. This option sets the {@link HierarchicLayoutNodeLayoutDescriptor#gridReference} property on the created object.
@param [options.portAssignment] The assignment strategy which distributes the ports along the node borders. This option sets the {@link HierarchicLayoutNodeLayoutDescriptor#portAssignment} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor Online Documentation}*/
constructor(options?: {
portBorderGapRatios?: number;
layerAlignment?: number;
nodeLabelMode?: NodeLabelMode | NodeLabelModeStringValues;
minimumLayerHeight?: number;
minimumDistance?: number;
gridReference?: YPoint | null;
portAssignment?: HierarchicLayoutPortAssignmentMode | HierarchicLayoutPortAssignmentModeStringValues;
});
/**
Returns the port border gap ratio for the port distribution at a given side of the node.
This ratio determines the gap between a corner of the node and the first assigned port.
The ratio should be greater than <code>0</code>.
A value of <code>0.0d</code> results in ports being placed directly on the corner of the node (if there is more than one port).
A value of <code>0.5d</code> results in ports being distributed along the side of the node so that the distance between the corner of
the node and the first port is half as wide as the distance between two adjacent ports.
A value of <code>Double.POSITIVE_INFINITY</code> results in all ports being centered at the side in one point.
@param side the zero-based clockwise side index for top-to-bottom layouts (i.e., top is 0)
@returns the border gap ratio of a given side
@see HierarchicLayoutNodeLayoutDescriptor#portBorderGapRatios
@see HierarchicLayoutNodeLayoutDescriptor#setPortBorderGapRatio
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor%23getPortBorderGapRatio(number) Online Documentation}*/
getPortBorderGapRatio(side: number): number;
/**
Specifies the port border gap ratio for the port distribution at a given side of the node.
This ratio determines the gap between a corner of the node and the first assigned port.
The ratio should be greater than <code>0</code>.
A value of <code>0.0d</code> results in ports being placed directly on the corner of the node (if there is more than one port).
A value of <code>0.5d</code> results in ports being distributed along the side of the node so that the distance between the corner of
the node and the first port is half as wide as the distance between two adjacent ports.
A value of <code>Double.POSITIVE_INFINITY</code> results in all ports being centered at the side in one point.
@param side the zero-based clockwise side index for top-to-bottom layouts (i.e., top is 0)
@param ratio the given ratio
@throws {Exception} <b>(ArgumentError)</b> if the given ratio is negative
@see HierarchicLayoutNodeLayoutDescriptor#portBorderGapRatios
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor%23setPortBorderGapRatio(number%2Cnumber) Online Documentation}*/
setPortBorderGapRatio(side: number, ratio: number): void;
/**
Sets the port border gap ratio for the port distribution on all sides of the node.
This ratio determines the gap between a corner of the node and the first assigned port.
The ratio should be greater than <code>0</code>.
A value of <code>0.0d</code> results in ports being placed directly on the corner of the node (if there is more than one port).
A value of <code>0.5d</code> results in ports being distributed along the side of the node so that the distance between the corner of
the node and the first port is half as wide as the distance between two adjacent ports.
A value of <code>Double.POSITIVE_INFINITY</code> results in all ports being centered at the side in one point.
@throws {Exception} <b>(ArgumentError)</b> if the given ratio is negative
@see HierarchicLayoutNodeLayoutDescriptor#setPortBorderGapRatio
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor%23portBorderGapRatios Online Documentation}*/
portBorderGapRatios: number;
/**
Gets or sets the alignment of the {@link YNode node} of this {@link HierarchicLayoutNodeLayoutDescriptor} instance within its layer.
<ul>
<li>A value of <code>0.0d</code> corresponds to top-alignment.
<li>A value of <code>0.5d</code> corresponds to center-alignment.
<li>A value of <code>1.0d</code> corresponds to bottom-alignment.
</ul>
This value should be within <code>[0,1]</code> interval.
@throws {Exception} <b>(ArgumentError)</b> if the alignment is not within [0.0d .. 1.0d]
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor%23layerAlignment Online Documentation}*/
layerAlignment: number;
/**
Gets or sets the mode that determines how node labels are handled by the algorithm during the layout.
<b>Warning:</b> To enable this feature, information about the layout of the node labels must be provided. If {@link HierarchicLayout}
is used for the layout, method {@link HierarchicLayout#considerNodeLabels} should be set to <code>true</code>. Otherwise, one has to assure that e.g., via an instance of
{@link LabelLayoutTranslator}, the algorithm receives the appropriate information.
@throws {Exception} <b>(ArgumentError)</b> if the constant is unknown.
@see HierarchicLayout#considerNodeLabels
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor%23nodeLabelMode Online Documentation}*/
nodeLabelMode: NodeLabelMode;
/**
Gets or sets the minimum height of the layer to which this node is assigned.
Values should be greater than <code>0</code>.
<b>Warning:</b> This feature will have an effect if different {@link HierarchicLayoutNodeLayoutDescriptor#layerAlignment layerAlignments} are present.
@throws {Exception} <b>(ArgumentError)</b> if the minimum height is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor%23minimumLayerHeight Online Documentation}*/
minimumLayerHeight: number;
/**
Gets or sets the preferred minimum distance to obstacles.
Currently, this distance only affects the following scenarios:
<ul>
<li>Distance between group nodes and other elements (horizontally and vertically).
<li>The distance of normal nodes and group node to borders of a partition grid cell or swimlane.
<li>The segment length of self-loops associated with this node.
</ul>
The minimum distance should be greater than <code>0</code>.
<b>Warning:</b> This setting does not affect the distance between normal nodes or between an edge and a normal node. These distances
can be globally specified using {@link HierarchicLayout#nodeToNodeDistance} and {@link HierarchicLayout#nodeToEdgeDistance}, respectively. If individual spacings around nodes are required,
{@link NodeHalo}s can be defined.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor%23minimumDistance Online Documentation}*/
minimumDistance: number;
/**
Gets or sets the node's reference point that will be placed on a grid coordinate.
The reference point is specified as offset to the center of the node. Hence, if for example the upper left corner of the
node should be placed on a grid coordinate, you have to use -{@link LayoutGraph#getWidth width}/2 as x-offset and -{@link LayoutGraph#getHeight height}/2 as y-offset.
<b>Warning:</b> The grid reference point will only be considered if there actually is a {@link HierarchicLayout#gridSpacing grid}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor%23gridReference Online Documentation}*/
gridReference: YPoint | null;
/**
Gets or sets the assignment strategy which distributes the ports along the node borders.
<b>Warning:</b> Port assignments {@link HierarchicLayoutPortAssignmentMode#DEFAULT}, {@link HierarchicLayoutPortAssignmentMode#ON_GRID} and {@link HierarchicLayoutPortAssignmentMode#ON_SUBGRID}
are currently only supported in {@link DefaultPortAllocator}. A custom implementation of {@link IPortAllocator}
will have to handle these assignments itself or will replace them with its own behavior.
<b>Warning:</b> If no grid is specified ({@link HierarchicLayout#gridSpacing grid spacing} is smaller or equal to 0), assignment {@link HierarchicLayoutPortAssignmentMode#DEFAULT}
is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutNodeLayoutDescriptor%23portAssignment Online Documentation}*/
portAssignment: HierarchicLayoutPortAssignmentMode;
static $class: Class;
private $_NodeLayoutDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HierarchicLayoutNodeLayoutDescriptor;
}
export interface PartitionGridLayoutStage extends LayoutStageBase {
}
/**
This class implements a {@link ILayoutStage layout stage} that handles a given {@link PartitionGrid partition grid} structure.
Information about the {@link PartitionGrid partition grid} structure are held by a {@link IDataProvider} registered with the graph
with key {@link PartitionGrid#PARTITION_GRID_DP_KEY}.
<h2>Note</h2>
This stage is automatically added by the {@link HierarchicLayout}, if required.
@see PartitionGrid
@see HierarchicLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridLayoutStage Online Documentation}*/
export class PartitionGridLayoutStage {
/**
Creates a new instance of {@link PartitionGridLayoutStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridLayoutStage Online Documentation}*/
constructor();
/**
Calculates the layout based on a {@link PartitionGrid} structure.
If no {@link PartitionGrid} structure exists, the core layouter will be called immediately.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridLayoutStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
static $class: Class;
private $_PartitionGridLayoutStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionGridLayoutStage;
}
export interface PortCandidateOptimizer extends PortConstraintOptimizerBase {
/**
@inheritdoc*/
optimizeAfterLayering(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
@inheritdoc*/
optimizeAfterSequencing(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
@inheritdoc*/
optimizeAfterSequencingForSingleNode(node: YNode | null, inEdgeOrder: IComparer<any> | null, outEdgeOrder: IComparer<any> | null, graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
} /**
This class is an implementation of the {@link IPortConstraintOptimizer} interface that tries to assign the
edges of the graph to {@link PortConstraint ports} considering the specified {@link PortCandidate}s.
Instances of this class can be set using property {@link HierarchicLayoutCore#portConstraintOptimizer}.
This implementation will query for {@link IDataProvider}s registered with the graph with keys
{@link PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY}, {@link PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY}
and {@link PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY}. It will <b>try</b> to assign each edge one of the matching
{@link PortCandidate}s without introducing too many crossings and without violating the {@link PortCandidate#cost cost} or {@link PortCandidateSet#connectionCount capacity}
constraints.
@see PortCandidate#cost
@see PortCandidateSet#entries
@see PortCandidateSet#connectionCount
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateOptimizer Online Documentation}*/
export class PortCandidateOptimizer {
/**
Creates a new instance of {@link PortCandidateOptimizer} with the default settings.
@param [options] The parameters to pass.
@param [options.deterministic] Whether or not this implementation uses a deterministic algorithm to assign the {@link PortCandidate}s. This option sets the {@link PortCandidateOptimizer#deterministic} property on the created object.
@param [options.backLoopPenalty] The penalty cost associated with each back-loop. This option sets the {@link PortCandidateOptimizer#backLoopPenalty} property on the created object.
@param [options.crossingPenalty] The penalty cost associated with a crossing that would occur if a given combination of {@link PortCandidate}s would be chosen. This option sets the {@link PortCandidateOptimizer#crossingPenalty} property on the created object.
@param [options.overUsagePenalty] The penalty cost associated with each edge being assigned to a port which has already reached its {@link IPortCandidateSetEntry#connections capacity}. This option sets the {@link PortCandidateOptimizer#overUsagePenalty} property on the created object.
@param [options.mirrorMode] The mirror mask that defines which orientations should be mirrored. This option sets the {@link PortConstraintOptimizerBase#mirrorMode} property on the created object.
@param [options.layoutOrientation] The layout orientation. This option sets the {@link PortConstraintOptimizerBase#layoutOrientation} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateOptimizer Online Documentation}*/
constructor(options?: {
deterministic?: boolean;
backLoopPenalty?: number;
crossingPenalty?: number;
overUsagePenalty?: number;
mirrorMode?: MirrorModes | MirrorModesConvertible;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
});
/**
Returns the data provider that provides the port candidate sets for nodes.
@param graph The graph that stores the data provider
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateOptimizer%23getPortCandidateSetDataProvider(LayoutGraph) Online Documentation}*/
getPortCandidateSetDataProvider(graph: LayoutGraph | null): IDataProvider | null;
/**
Gets or sets whether or not this implementation uses a deterministic algorithm to assign the
{@link PortCandidate}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateOptimizer%23deterministic Online Documentation}*/
deterministic: boolean;
/**
Gets or sets the penalty cost associated with each back-loop.
Values should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the penalty cost is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateOptimizer%23backLoopPenalty Online Documentation}*/
backLoopPenalty: number;
/**
Gets or sets the penalty cost associated with a crossing that would occur if a given combination of
{@link PortCandidate}s would be chosen.
Values should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the crossing penalty is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateOptimizer%23crossingPenalty Online Documentation}*/
crossingPenalty: number;
/**
Gets or sets the penalty cost associated with each edge being assigned to a port which has already reached its
{@link IPortCandidateSetEntry#connections capacity}.
Values should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the penalty cost is negative
@see IPortCandidateSetEntry#connections
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortCandidateOptimizer%23overUsagePenalty Online Documentation}*/
overUsagePenalty: number;
static $class: Class;
private $_PortCandidateOptimizer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortCandidateOptimizer;
}
export interface PortConstraintOptimizerSameLayerData extends YObject {
}
/**
This static class provides information about the same-layer structures created by class
{@link PortConstraintOptimizerBase}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerSameLayerData Online Documentation}*/
export class PortConstraintOptimizerSameLayerData {
/**
Creates a new {@link PortConstraintOptimizerSameLayerData} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerSameLayerData Online Documentation}*/
constructor();
/**
Adds a given dummy {@link YNode} to this {@link PortConstraintOptimizerSameLayerData} instance and
associates it with the given original (same-layer) {@link Edge}.
@param sameLayerDummy a dummy node
@param originalEdge the original (same-layer) edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerSameLayerData%23addDummyNode(YNode%2CEdge) Online Documentation}*/
addDummyNode(sameLayerDummy: YNode | null, originalEdge: Edge | null): void;
/**
Returns the original (same-layer) edge associated with the given dummy {@link YNode}.
@param sameLayerDummy a given dummy node
@returns the original (same-layer) edge associated with the given dummy node
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerSameLayerData%23getOriginalEdge(YNode) Online Documentation}*/
getOriginalEdge(sameLayerDummy: YNode | null): Edge | null;
/**
Gets all dummy nodes of this {@link PortConstraintOptimizerSameLayerData} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerSameLayerData%23dummyNodes Online Documentation}*/
readonly dummyNodes: YNodeList | null;
static $class: Class;
private $_PortConstraintOptimizerSameLayerData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortConstraintOptimizerSameLayerData;
}
export interface PortConstraintOptimizerBase extends YObject, IPortConstraintOptimizer {
}
/**
This class is an abstract implementation of the {@link IPortConstraintOptimizer} interface that minimizes the
effort required to modify the port assignment after the sequencing phase of hierarchic layout algorithm.
The {@link PortConstraintOptimizerBase#optimizeAfterSequencing callback method invoked after sequencing} performs the following three steps:
<ol>
<li>Restores <b>temporarily</b> all same-layer edges which are not currently present in the layout graph at this time of the layout
algorithm.
<li>Invokes {@link PortConstraintOptimizerBase#optimizeAfterSequencingForSingleNode the hook} in which the custom port assignment should be done.
<li>Restores the original state of the layout graph by removing the temporary same-layer edges added in Step <code>1</code>.
</ol>
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase Online Documentation}*/
export abstract class PortConstraintOptimizerBase {
/**
Creates a new {@link PortConstraintOptimizerBase} instance with default settings.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase Online Documentation}*/
constructor();
/**
Inserts a same-layer edge structure for each same-layer edge of the original graph.
<h2>Note</h2>
In this phase of the layout, the graph does not contain same-layer edges. The structure for a same-layer edge <code>(s,t)</code>
consists of a temporary node <code>w</code> and edges <code>(s,w)</code> and <code>(t,w)</code>.
@param graph the input graph
@param layers the given {@link ILayers} instance holding the layering information
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param itemFactory the {@link IItemFactory factory} that sets the temporary port constraints
@see IItemFactory#setTemporaryPortConstraint
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase%23insertSameLayerStructures(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
insertSameLayerStructures(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): PortConstraintOptimizerSameLayerData | null;
/**
Assigns new temporary port constraints after the nodes have been assigned to layers.
More precisely, it is called after the layering information has been determined. In this phase, it is possible to create
back-loops by assigning incoming edges to the south (i.e. bottom) side or outgoing edges to the north (i.e. top) side,
respectively.
@param graph the input graph
@param layers the given {@link ILayers} instance holding the layering information
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param itemFactory the {@link IItemFactory factory} that sets the temporary port constraints
@see IItemFactory#setTemporaryPortConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase%23optimizeAfterLayering(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
abstract optimizeAfterLayering(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
Assigns new temporary port constraints after the order of the nodes in each layer has been determined.
More precisely, it is called after the sequence of the nodes has been determined. It {@link PortConstraintOptimizerBase#insertSameLayerStructures inserts the same-layer structures},
invokes {@link PortConstraintOptimizerBase#optimizeAfterSequencingForSingleNode the hook} in which the custom port assignment should be done, and finally {@link PortConstraintOptimizerBase#removeSameLayerStructures restores the original state} of the
layout graph by removing the temporary edges.
@param graph the input graph
@param layers the given {@link ILayers} instance holding the layering information
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param itemFactory the {@link IItemFactory factory} that sets the temporary port constraints
@see PortConstraintOptimizerBase#insertSameLayerStructures
@see PortConstraintOptimizerBase#optimizeAfterSequencing
@see PortConstraintOptimizerBase#removeSameLayerStructures
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase%23optimizeAfterSequencing(LayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
optimizeAfterSequencing(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
Assigns new temporary port constraints after the order of the nodes in each layer has been determined.
More precisely, it is called after the sequence of the nodes has been determined. It invokes {@link PortConstraintOptimizerBase#optimizeAfterSequencingForSingleNode}
for every node of the original layout graph and omits the nodes of the same-layer structures.
Incoming and outgoing edges are sorted using {@link IComparer} instances which define the preferred ordering of the incoming and outgoing
edges from left to right.
<h2>Note</h2>
In this phase, it is not allowed to create back-loops, i.e., incoming edges must not connect to the south (i.e., bottom)
side and outgoing edges must not connect to the north (i.e., top) side of a node.
@param inEdgeOrder a given {@link IComparer} instance for incoming edges
@param outEdgeOrder a given {@link IComparer} instance for outgoing edges
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param itemFactory the {@link IItemFactory factory} that sets the temporary port constraints
@param layers
@see PortConstraintOptimizerBase#optimizeAfterSequencing
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase%23optimizeAfterSequencingForAllNodes(IComparer%3CObject%3E%2CIComparer%3CObject%3E%2CLayoutGraph%2CILayers%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
optimizeAfterSequencingForAllNodes(inEdgeOrder: IComparer<any> | null, outEdgeOrder: IComparer<any> | null, graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
Assigns new temporary port constraints to a given node of the graph after the order of the nodes in each layer has been
determined.
More precisely, it is called after the sequence of the nodes has been determined.
Incoming and outgoing edges are sorted using {@link IComparer} instances which define the preferred ordering of the incoming and outgoing
edges from left to right.
<h2>Note</h2>
In this phase, it is not allowed to create back-loops, i.e., incoming edges must not connect to the south (i.e., bottom)
side and outgoing edges must not connect to the north (i.e., top) side of a node.
@param node the original node to set temporary port constraints
@param inEdgeOrder a given {@link IComparer} instance for incoming edges
@param outEdgeOrder a given {@link IComparer} instance for outgoing edges
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param itemFactory the {@link IItemFactory factory} that sets the temporary port constraints
@see PortConstraintOptimizerBase#optimizeAfterSequencing
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase%23optimizeAfterSequencingForSingleNode(YNode%2CIComparer%3CObject%3E%2CIComparer%3CObject%3E%2CLayoutGraph%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
abstract optimizeAfterSequencingForSingleNode(node: YNode | null, inEdgeOrder: IComparer<any> | null, outEdgeOrder: IComparer<any> | null, graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
Removes the same-layer edge structure created using
{@link PortConstraintOptimizerBase#insertSameLayerStructures}.
@param sameLayerData a given {@link PortConstraintOptimizerSameLayerData} instance holding the information about the same-layer
structures.
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param itemFactory the {@link IItemFactory factory} that sets the temporary port constraints
@see IItemFactory#setTemporaryPortConstraint
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase%23removeSameLayerStructures(PortConstraintOptimizerSameLayerData%2CLayoutGraph%2CILayoutDataProvider%2CIItemFactory) Online Documentation}*/
removeSameLayerStructures(sameLayerData: PortConstraintOptimizerSameLayerData | null, graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, itemFactory: IItemFactory | null): void;
/**
Gets or sets the mirror mask that defines which orientations should be mirrored.
Setting a layout orientation can be considered as rotating the graph by <code>90</code>, <code>180</code> or <code>270</code> degrees. Afterwards, the graph
can be mirrored at the x-axis (for horizontal layout orientations) or y-axis (for vertical layout orientations). Which
directions are mirrored can be defined by the given mask.
@see OrientationLayout
@see LayoutOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase%23mirrorMode Online Documentation}*/
mirrorMode: MirrorModes;
/**
Gets or sets the layout orientation.
This setting is necessary to correctly interpret the values provided by the {@link PortCandidate}s since the {@link OrientationLayout}
cannot automatically adjust these values.
@see MultiStageLayout#layoutOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortConstraintOptimizerBase%23layoutOrientation Online Documentation}*/
layoutOrientation: LayoutOrientation;
static $class: Class;
private $_PortConstraintOptimizerBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortConstraintOptimizerBase;
}
export interface HierarchicLayoutRoutingStyle extends YObject {
}
/**
This class is used by {@link HierarchicLayoutEdgeLayoutDescriptor} to specify the routing style for different types of edges.
@see HierarchicLayoutEdgeLayoutDescriptor#routingStyle
@see IEdgeData#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle Online Documentation}*/
export class HierarchicLayoutRoutingStyle {
/**
Creates a new {@link HierarchicLayoutRoutingStyle} instance with the given routing style for each edge and, optionally, also
specifying whether or not the resulting path should consist of cubic bezier control points.
@param routingStyle one of the predefined routing styles
@param [controlPointCreation=false] <code>true</code> if the resulting path should consist of cubic bezier control points, <code>false</code> otherwise
@throws {Exception} <b>(ArgumentError)</b> if an unknown routing style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23HierarchicLayoutRoutingStyle(HierarchicLayoutEdgeRoutingStyle%2Cboolean) Online Documentation}*/
constructor(routingStyle: HierarchicLayoutEdgeRoutingStyle | HierarchicLayoutEdgeRoutingStyleStringValues, controlPointCreation?: boolean);
/**
Creates a new instance of a {@link HierarchicLayoutRoutingStyle} with the default values.
@param [options] The parameters to pass.
@param [options.backLoopRoutingStyle] The routing style for back-loop edges. This option sets the {@link HierarchicLayoutRoutingStyle#backLoopRoutingStyle} property on the created object.
@param [options.edgeGroupRoutingStyle] The routing style for grouped edges at the common segments. This option sets the {@link HierarchicLayoutRoutingStyle#edgeGroupRoutingStyle} property on the created object.
@param [options.defaultEdgeRoutingStyle] The default routing style for edges that have no individual routing style. This option sets the {@link HierarchicLayoutRoutingStyle#defaultEdgeRoutingStyle} property on the created object.
@param [options.sameLayerEdgeRoutingStyle] The routing style for same-layer edges. This option sets the {@link HierarchicLayoutRoutingStyle#sameLayerEdgeRoutingStyle} property on the created object.
@param [options.selfLoopRoutingStyle] The routing style for self-loops. This option sets the {@link HierarchicLayoutRoutingStyle#selfLoopRoutingStyle} property on the created object.
@param [options.sourceCurveConnectionStyle] The style which determines how curved edge paths connect at the source side of the edge. This option sets the {@link HierarchicLayoutRoutingStyle#sourceCurveConnectionStyle} property on the created object.
@param [options.targetCurveConnectionStyle] The style which determines how curved edge paths connect at the target side of the edge. This option sets the {@link HierarchicLayoutRoutingStyle#targetCurveConnectionStyle} property on the created object.
@param [options.createControlPoints] Whether or not the points of the resulting edge path represent cubic bezier control points. This option sets the {@link HierarchicLayoutRoutingStyle#createControlPoints} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23HierarchicLayoutRoutingStyle() Online Documentation}*/
constructor(options?: {
backLoopRoutingStyle?: HierarchicLayoutEdgeRoutingStyle | HierarchicLayoutEdgeRoutingStyleStringValues;
edgeGroupRoutingStyle?: HierarchicLayoutEdgeRoutingStyle | HierarchicLayoutEdgeRoutingStyleStringValues;
defaultEdgeRoutingStyle?: HierarchicLayoutEdgeRoutingStyle | HierarchicLayoutEdgeRoutingStyleStringValues;
sameLayerEdgeRoutingStyle?: HierarchicLayoutEdgeRoutingStyle | HierarchicLayoutEdgeRoutingStyleStringValues;
selfLoopRoutingStyle?: HierarchicLayoutEdgeRoutingStyle | HierarchicLayoutEdgeRoutingStyleStringValues;
sourceCurveConnectionStyle?: CurveConnectionStyle | CurveConnectionStyleStringValues;
targetCurveConnectionStyle?: CurveConnectionStyle | CurveConnectionStyleStringValues;
createControlPoints?: boolean;
});
/**
Gets or sets the routing style for back-loop edges.
This style is used for routing U-turns of common edges (i.e., edges that are neither self-loops nor same-layer edges). A
U-turn is a non-monotonic part of the edge route that is required for reversed edges if option {@link HierarchicLayout#backLoopRouting} is enabled or in some
other scenarios with port constraints/candidates.
@throws {Exception} <b>(ArgumentError)</b> if an unknown routing style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23backLoopRoutingStyle Online Documentation}*/
backLoopRoutingStyle: HierarchicLayoutEdgeRoutingStyle;
/**
Gets or sets the routing style for grouped edges at the common segments.
More precisely, grouped edges are routed in a bus-style fashion (i.e., the paths of the edges will share a common edge
segment) and this option allows to specify the routing style at the bus.
@throws {Exception} <b>(ArgumentError)</b> if an unknown routing style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23edgeGroupRoutingStyle Online Documentation}*/
edgeGroupRoutingStyle: HierarchicLayoutEdgeRoutingStyle;
/**
Gets or sets the default routing style for edges that have no individual routing style.
@throws {Exception} <b>(ArgumentError)</b> if an unknown routing style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23defaultEdgeRoutingStyle Online Documentation}*/
defaultEdgeRoutingStyle: HierarchicLayoutEdgeRoutingStyle;
/**
Gets or sets the routing style for same-layer edges.
A same-layer edge is an edge whose source/target are assigned to the same layer.
@throws {Exception} <b>(ArgumentError)</b> if an unknown routing style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23sameLayerEdgeRoutingStyle Online Documentation}*/
sameLayerEdgeRoutingStyle: HierarchicLayoutEdgeRoutingStyle;
/**
Gets or sets the routing style for self-loops.
@throws {Exception} <b>(ArgumentError)</b> if an unknown routing style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23selfLoopRoutingStyle Online Documentation}*/
selfLoopRoutingStyle: HierarchicLayoutEdgeRoutingStyle;
/**
Gets or sets the style which determines how curved edge paths connect at the source side of the edge.
<h2>Note</h2>
When specifying style
{@link CurveConnectionStyle#ORGANIC}, the source port may end up not being on the side of the node pointing in the
main layout direction anymore. For example, the port may be moved to the left or right side of the node for the default
top-to-bottom layout orientation.
<b>Warning:</b> This setting only affects the source side of edges with a {@link HierarchicLayoutEdgeRoutingStyle#CURVED curved} edge routing style.
@throws {Exception} <b>(ArgumentError)</b> if an unknown connection style is given
@see HierarchicLayoutRoutingStyle#targetCurveConnectionStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23sourceCurveConnectionStyle Online Documentation}*/
sourceCurveConnectionStyle: CurveConnectionStyle;
/**
Gets or sets the style which determines how curved edge paths connect at the target side of the edge.
<h2>Note</h2>
When specifying style
{@link CurveConnectionStyle#ORGANIC}, the target port may end up not being on the side of the node pointing
against the main layout direction anymore. For example, the port may be moved to the left or right side of the node for
the default top-to-bottom layout orientation.
<b>Warning:</b> This setting only affects the target side of edges with a {@link HierarchicLayoutEdgeRoutingStyle#CURVED curved} edge routing style.
@throws {Exception} <b>(ArgumentError)</b> if an unknown connection style is given
@see HierarchicLayoutRoutingStyle#sourceCurveConnectionStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23targetCurveConnectionStyle Online Documentation}*/
targetCurveConnectionStyle: CurveConnectionStyle;
/**
Gets or sets whether or not the points of the resulting edge path represent cubic bezier control points.
<ul>
<li><b>Disabled (default):</b> Each point of the path represents a normal polyline bend point.
<li><b>Enabled:</b> Four consecutive points p1, p2, p3, p4 of the resulting edge path need to be interpreted such that p1 is the start
point of the cubic bezier curve, p4 is the end point of it and p2, p3 are the two intermediate control points. If p4 is
not the last point of the edge path it is at the same time the start point of the next curve. For example, points p1,
p2, p3, p4, p5, p6, p7 form a path that consists of two curves: the curve defined by p1, p2, p3, p4 and the curve
defined by p4, p5, p6, p7.
</ul>
<h2>Note</h2>
This feature is useful when the visualization of the edge draws bezier curves instead of polyline paths.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutRoutingStyle%23createControlPoints Online Documentation}*/
createControlPoints: boolean;
static $class: Class;
private $_RoutingStyle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HierarchicLayoutRoutingStyle;
}
export interface SelfLoopCalculator extends YObject {
}
/**
This class calculates bend points for self-loops routed in orthogonal or octilinear fashion.
It is used during the layout phase of {@link HierarchicLayoutCore}, but can be used as a stand-alone tool.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator Online Documentation}*/
export class SelfLoopCalculator {
/**
Creates a new instance of {@link SelfLoopCalculator} using the given parameters.
@param minFirstSegmentLength the minimum length of the first and last segment of an orthogonally routed self-loop
@param minEdgeDistance the minimum distance between a pair of self-loops
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator Online Documentation}*/
constructor(minFirstSegmentLength: number, minEdgeDistance: number);
/**
Convenience method that calculates a list of self-loops that belong to a given node.
The current port positions will be used for determining the start and end ports. The {@link IDataProvider}
instances hold information about the source/target ports of the edges and can be used for specifying the direction of
first and last segments.
@param graph the input graph
@param node the node whose self-loops will be routed
@param selfLoops a list of {@link Edge}s that will be routed
@param spc the {@link IDataProvider} that returns the {@link PortConstraint} object associated with the source
ports of the edges
@param tpc the {@link IDataProvider} that returns the {@link PortConstraint} object associated with the target
ports of the edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23calculateSelfLoops(LayoutGraph%2CYNode%2CYList%2CIDataProvider%2CIDataProvider) Online Documentation}*/
calculateSelfLoops(graph: LayoutGraph | null, node: YNode | null, selfLoops: YList | null, spc: IDataProvider | null, tpc: IDataProvider | null): void;
/**
Calculates all self-loops at the given node of the given input graph.
The current port positions will be used for determining the start and end ports. The {@link IDataProvider}
instances hold information about the source/target ports of the edges and can be used for specifying the direction of
first and last segments.
@param graph the input graph
@param node the node whose self-loops will be routed
@param spc the {@link IDataProvider} that returns the {@link PortConstraint} object associated with the source
ports of the edges
@param tpc the {@link IDataProvider} that returns the {@link PortConstraint} object associated with the target
ports of the edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23calculateSelfLoops(LayoutGraph%2CYNode%2CIDataProvider%2CIDataProvider) Online Documentation}*/
calculateSelfLoops(graph: LayoutGraph | null, node: YNode | null, spc: IDataProvider | null, tpc: IDataProvider | null): void;
/**
Returns the minimum length between two edges associated with the given edge.
The value is fetched from the corresponding {@link IDataProvider} registered with the graph with key
{@link SelfLoopCalculator#MINIMUM_EDGE_DISTANCE_DP_KEY}.
@param graph the input graph
@param edge the given edge
@returns the minimum length between two edges
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23getMinimumDistance(LayoutGraph%2CEdge) Online Documentation}*/
getMinimumDistance(graph: LayoutGraph | null, edge: Edge | null): number;
/**
Returns the minimum length of the first segment of the given edge.
The value is fetched from the corresponding {@link IDataProvider} registered with the graph with key
{@link SelfLoopCalculator#MINIMUM_FIRST_SEGMENT_LENGTH_DP_KEY}.
@param graph the input graph
@param edge the given edge
@returns the minimum length of the first segment of an edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23getMinimumFirstSegmentLength(LayoutGraph%2CEdge) Online Documentation}*/
getMinimumFirstSegmentLength(graph: LayoutGraph | null, edge: Edge | null): number;
/**
Returns the minimum length of the last segment of the given edge.
The value is fetched from the corresponding {@link IDataProvider} registered with the graph with key
{@link SelfLoopCalculator#MINIMUM_LAST_SEGMENT_LENGTH_DP_KEY}.
@param graph the input graph
@param edge the given edge
@returns the minimum length of the last segment of the given edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23getMinimumLastSegmentLength(LayoutGraph%2CEdge) Online Documentation}*/
getMinimumLastSegmentLength(graph: LayoutGraph | null, edge: Edge | null): number;
/**
Returns the minimum distance between two nodes associated with the given node.
The value is fetched from the corresponding {@link IDataProvider} registered with the graph with key
{@link SelfLoopCalculator#MINIMUM_NODE_DISTANCE_DP_KEY}.
@param graph the input graph
@param node the given node
@returns the minimum distance between two nodes
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23getMinimumNodeDistance(LayoutGraph%2CYNode) Online Documentation}*/
getMinimumNodeDistance(graph: LayoutGraph | null, node: YNode | null): number;
/**
Returns the minimum octilinear segment length of the given edge.
The value is fetched from the corresponding {@link IDataProvider} registered with the graph with key
{@link SelfLoopCalculator#MINIMUM_OCTILINEAR_SEGMENT_LENGTH_DP_KEY}.
@param graph the input graph
@param edge the given edge
@returns the minimum octilinear segment length of the given edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23getMinimumOctilinearSegmentLength(LayoutGraph%2CEdge) Online Documentation}*/
getMinimumOctilinearSegmentLength(graph: LayoutGraph | null, edge: Edge | null): number;
/**
Returns whether or not the given edge is octilinear.
The value is fetched from the corresponding {@link IDataProvider} registered with the graph with key
{@link SelfLoopCalculator#OCTILINEAR_EDGES_DP_KEY}.
@param graph the input graph
@param edge the given edge
@returns <code>true</code> if the edge is octilinear, <code>false</code> otherwise
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23isOctilinearEdge(LayoutGraph%2CEdge) Online Documentation}*/
isOctilinearEdge(graph: LayoutGraph | null, edge: Edge | null): boolean;
/**
Data provider key for defining for each node a minimum distance to the self-loop edges
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23MINIMUM_NODE_DISTANCE_DP_KEY Online Documentation}*/
static MINIMUM_NODE_DISTANCE_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for specifying whether or not an edge is octilinear
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23OCTILINEAR_EDGES_DP_KEY Online Documentation}*/
static OCTILINEAR_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Data provider key for specifying the minimum length of the octilinear segments of an octilinear edge
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23MINIMUM_OCTILINEAR_SEGMENT_LENGTH_DP_KEY Online Documentation}*/
static MINIMUM_OCTILINEAR_SEGMENT_LENGTH_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for specifying the minimum length of the first segment of each edge
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23MINIMUM_FIRST_SEGMENT_LENGTH_DP_KEY Online Documentation}*/
static MINIMUM_FIRST_SEGMENT_LENGTH_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for specifying the minimum length of the last segment of each edge
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23MINIMUM_LAST_SEGMENT_LENGTH_DP_KEY Online Documentation}*/
static MINIMUM_LAST_SEGMENT_LENGTH_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for specifying the minimum distance to the next edge or node side
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculator%23MINIMUM_EDGE_DISTANCE_DP_KEY Online Documentation}*/
static MINIMUM_EDGE_DISTANCE_DP_KEY: EdgeDpKey<number> | null;
static $class: Class;
private $_SelfLoopCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SelfLoopCalculator;
}
export interface SimplexNodePlacer extends YObject, IHierarchicLayoutNodePlacer {
/**
@inheritdoc*/
assignLayerCoordinates(graph: LayoutGraph | null, layoutDataProvider: ILayoutDataProvider | null, layers: ILayers | null): void;
/**
@inheritdoc*/
assignSequenceCoordinates(graph: LayoutGraph | null, layoutDataProvider: ILayoutDataProvider | null, layers: ILayers | null, drawingDistanceCalculator: IDrawingDistanceCalculator | null): void;
} /**
This class is a {@link IHierarchicLayoutNodePlacer} implementation based on {@link RankAssignmentAlgorithm rank assignment}.
The drawer works with integer values and rounds double values to integers using sophisticated quantizing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer Online Documentation}*/
export class SimplexNodePlacer {
/**
Creates a new instance of {@link SimplexNodePlacer} with the default settings.
@param [options] The parameters to pass.
@param [options.nodeCompaction] Whether or not nodes should be placed in a more compact style with respect to layout width. This option sets the {@link SimplexNodePlacer#nodeCompaction} property on the created object.
@param [options.minimumSublayerDistance] The minimum distance between the upper and lower sublayer. This option sets the {@link SimplexNodePlacer#minimumSublayerDistance} property on the created object.
@param [options.labelCompaction] Whether or not the {@link SimplexNodePlacer} places labels in a more compact style. This option sets the {@link SimplexNodePlacer#labelCompaction} property on the created object.
@param [options.bendReduction] Whether or not an optimization step should be applied that tries to further reduce the number of bends. This option sets the {@link SimplexNodePlacer#bendReduction} property on the created object.
@param [options.horizontalCompaction] Whether or not the {@link SimplexNodePlacer} tries to create a maximally compact horizontal layout at the cost of more bends. This option sets the {@link SimplexNodePlacer#horizontalCompaction} property on the created object.
@param [options.straightenEdges] Whether or not a postprocessing step should be applied to reduce the number of bends. This option sets the {@link SimplexNodePlacer#straightenEdges} property on the created object.
@param [options.groupCompactionStrategy] The strategy used for controlling the horizontal compactness of group nodes. This option sets the {@link SimplexNodePlacer#groupCompactionStrategy} property on the created object.
@param [options.exactPlacement] Whether or not the {@link SimplexNodePlacer} enforces the placement of nodes at their exact current positions, even if this violates minimum distance constraints. This option sets the {@link SimplexNodePlacer#exactPlacement} property on the created object.
@param [options.fromSketchLayerAssignment] Whether or not the {@link SimplexNodePlacer} should try to use the coordinates given from the current sketch for the determination of the layer coordinates. This option sets the {@link SimplexNodePlacer#fromSketchLayerAssignment} property on the created object.
@param [options.maximumDuration] The time limit (in milliseconds) set for the layout algorithm. This option sets the {@link SimplexNodePlacer#maximumDuration} property on the created object.
@param [options.swimLaneCrossingWeight] The relative weight of edges crossing a swimlane relative to edges that stay in their lane. This option sets the {@link SimplexNodePlacer#swimLaneCrossingWeight} property on the created object.
@param [options.barycenterMode] Whether or not a barycenter drawing mode should be used. This option sets the {@link SimplexNodePlacer#barycenterMode} property on the created object.
@param [options.breakLongSegments] Whether or not the {@link SimplexNodePlacer} should break <em>long</em> edge segments in favor of a potentially more compact layout. This option sets the {@link SimplexNodePlacer#breakLongSegments} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer Online Documentation}*/
constructor(options?: {
nodeCompaction?: boolean;
minimumSublayerDistance?: number;
labelCompaction?: boolean;
bendReduction?: boolean;
horizontalCompaction?: boolean;
straightenEdges?: boolean;
groupCompactionStrategy?: GroupCompactionPolicy | GroupCompactionPolicyStringValues;
exactPlacement?: boolean;
fromSketchLayerAssignment?: boolean;
maximumDuration?: number;
swimLaneCrossingWeight?: number;
barycenterMode?: boolean;
breakLongSegments?: boolean;
});
/**
Assigns the nodes of a given layer to the corresponding upper/lower sublayer if {@link SimplexNodePlacer#nodeCompaction node compaction} is enabled.
Information about sublayers is given by a {@link INodeMap} whose GetBool method has to return <code>true</code> for each
node that should be placed in the lower sublayer (values are set by this method).
@param layerNodes a {@link YNodeList list} of all @link y.layout.hierarchic.incremental.NodeData#TYPE_NORMAL normal nodes} of a layer
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param graph the given graph
@param lowerSublayer the {@link INodeMap} holding sublayer information
@see SimplexNodePlacer#nodeCompaction
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23assignNodesToSublayer(YNodeList%2CILayoutDataProvider%2CLayoutGraph%2CINodeMap) Online Documentation}*/
assignNodesToSublayer(layerNodes: YNodeList | null, ldp: ILayoutDataProvider | null, graph: LayoutGraph | null, lowerSublayer: INodeMap | null): void;
/**
Returns the alignment of the node with a specified layer.
This callback method used by {@link SimplexNodePlacer#assignLayerCoordinates}.
@param graph the input graph
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param node the given node
@param layerIndex the index of the layer
@param minLayerHeight the minimum height of the layer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23getLayerAlignment(LayoutGraph%2CILayoutDataProvider%2CYNode%2Cnumber%2Cnumber) Online Documentation}*/
getLayerAlignment(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, node: YNode | null, layerIndex: number, minLayerHeight: number): number;
/**
Specifies the minimum allowed distance between two nodes of the same given layer.
This callback method is used by {@link SimplexNodePlacer#assignSequenceCoordinates}.
@param graph the graph that contains the nodes
@param layer the layer that contains the nodes
@param predNode the node to the left of the other one, may be <code>null</code> to indicate a border line
@param succ the node to the right of the other one, may be <code>null</code> to indicate a border line
@returns the minimum distance allowed between those two nodes ( >=0 )
@see IDrawingDistanceCalculator
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23getMinDistance(LayoutGraph%2CILayer%2CYNode%2CYNode) Online Documentation}*/
getMinDistance(graph: LayoutGraph | null, layer: ILayer | null, predNode: YNode | null, succ: YNode | null): number;
/**
Returns the minimum height of a given {@link ILayer layer}.
This callback method is used by {@link SimplexNodePlacer#assignLayerCoordinates}.
@param graph the given graph
@param ldp the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param layer the given {@link ILayer} object
@returns the minimum height
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23getMinimumLayerHeight(LayoutGraph%2CILayoutDataProvider%2CILayer) Online Documentation}*/
getMinimumLayerHeight(graph: LayoutGraph | null, ldp: ILayoutDataProvider | null, layer: ILayer | null): number;
/**
Returns whether or not the given node should be treated as a node with fixed (given) coordinates.
This callback method is used by both {@link SimplexNodePlacer#assignLayerCoordinates} and
{@link SimplexNodePlacer#assignSequenceCoordinates}.
If <code>inLayer</code> argument is <code>true</code>, this method is called from within the {@link SimplexNodePlacer#assignLayerCoordinates}
method.
@param graph the given graph
@param provider the {@link ILayoutDataProvider} implementation which provides access to the {@link INodeData} and {@link IEdgeData}
instances
@param node the given node
@param inLayer <code>true</code> if the layer coordinate of the node is queried, <code>false</code> if the sequence coordinate is queried
@returns <code>true</code> if the node should be treated as fixed, <code>false</code> otherwise
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23isFixedNode(LayoutGraph%2CILayoutDataProvider%2CYNode%2Cboolean) Online Documentation}*/
isFixedNode(graph: LayoutGraph | null, provider: ILayoutDataProvider | null, node: YNode | null, inLayer: boolean): boolean;
/**
Gets or sets whether or not nodes should be placed in a more compact style with respect to layout width.
If enabled, the algorithm may place the nodes of a layer in a stacked style (horizontally interleaving), i.e., it splits
the layer into an upper and lower sublayer and places adjacent nodes into different sublayers if this reduces the width
of the layer.
<h2>Note</h2>
Enabling this option increases the required layout height.
<b>Warning:</b> This option does not work with customized {@link IDrawingDistanceCalculator}s, see {@link HierarchicLayoutCore#drawingDistanceCalculator}.
@see SimplexNodePlacer#minimumSublayerDistance
@see SimplexNodePlacer#assignNodesToSublayer
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23nodeCompaction Online Documentation}*/
nodeCompaction: boolean;
/**
Gets or sets the minimum distance between the upper and lower sublayer.
A too small distance value may prevent adjacent nodes from being placed in a stacked style (horizontally interleaving)
because the vertical distance between these nodes must be greater than or equal to the value specified by {@link HierarchicLayoutNodeLayoutDescriptor#minimumDistance}.
Values should be greater than <code>0</code>.
<b>Warning:</b> This option is only considered if the node compaction {@link SimplexNodePlacer#nodeCompaction is enabled}.
@throws {Exception} <b>(ArgumentError)</b> if minimum sublayer distance is negative
@see SimplexNodePlacer#nodeCompaction
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23minimumSublayerDistance Online Documentation}*/
minimumSublayerDistance: number;
/**
Gets or sets whether or not the {@link SimplexNodePlacer} places labels in a more compact style.
If this option is enabled the algorithm tries to place adjacent label elements in a stacked style, i.e., horizontally
interleaving.
<h2>Note</h2>
This option has no effect with customized {@link IDrawingDistanceCalculator}s, see {@link HierarchicLayoutCore#drawingDistanceCalculator}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23labelCompaction Online Documentation}*/
labelCompaction: boolean;
/**
Gets or sets whether or not an optimization step should be applied that tries to further reduce the number of bends.
<h2>Note</h2>
Using this option may increase the running time of the layout algorithm. If the running time exceeds the maximum
duration (see {@link SimplexNodePlacer#maximumDuration}) the number of bends is not reduced.
<h2>Note</h2>
Disabling option {@link SimplexNodePlacer#barycenterMode} may further reduce the number of bends.
@see SimplexNodePlacer#maximumDuration
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23bendReduction Online Documentation}*/
bendReduction: boolean;
/**
Gets or sets whether or not the {@link SimplexNodePlacer} tries to create a maximally compact horizontal
layout at the cost of more bends.
For best results, {@link SimplexNodePlacer#breakLongSegments} and {@link SimplexNodePlacer#labelCompaction} should also be enabled.
@see SimplexNodePlacer#labelCompaction
@see SimplexNodePlacer#breakLongSegments
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23horizontalCompaction Online Documentation}*/
horizontalCompaction: boolean;
/**
Gets or sets whether or not a postprocessing step should be applied to reduce the number of bends.
Furthermore, enabling this option may violate some minimum distances specified by the user and the edge distribution is
no longer uniform.
<h2>Note</h2>
This option is only considered if option {@link SimplexNodePlacer#barycenterMode} and {@link SimplexNodePlacer#nodeCompaction} are both disabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23straightenEdges Online Documentation}*/
straightenEdges: boolean;
/**
Gets or sets the strategy used for controlling the horizontal compactness of group nodes.
<h2>Note</h2>
To obtain compact groups you should additionally disable option {@link SimplexNodePlacer#bendReduction}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23groupCompactionStrategy Online Documentation}*/
groupCompactionStrategy: GroupCompactionPolicy;
/**
Gets or sets whether or not the {@link SimplexNodePlacer} enforces the placement of nodes at their exact
current positions, even if this violates minimum distance constraints.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23exactPlacement Online Documentation}*/
exactPlacement: boolean;
/**
Gets or sets whether or not the {@link SimplexNodePlacer} should try to use the coordinates given from the
current sketch for the determination of the layer coordinates.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23fromSketchLayerAssignment Online Documentation}*/
fromSketchLayerAssignment: boolean;
/**
Gets or sets the time limit (in milliseconds) set for the layout algorithm.
Values have to be greater than or equal to <code>0</code>.
<h2>Note</h2>
Restricting the maximum duration may result in a lower layout quality. Furthermore, the real runtime may exceed the
maximum duration since the layout algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the relative weight of edges crossing a swimlane relative to edges that stay in their lane.
Higher values lead to more compact drawings but may cause additional bends.
Values should lie within <code>[0,1]</code> interval.
@throws {Exception} <b>(ArgumentError)</b> if the relative weight does not lie within [0,1]
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23swimLaneCrossingWeight Online Documentation}*/
swimLaneCrossingWeight: number;
/**
Gets or sets whether or not a barycenter drawing mode should be used.
If this mode is enabled the resulting drawing is likely to have more bends, but may be more symmetric.
<h2>Note</h2>
Disabling option {@link SimplexNodePlacer#bendReduction} may produce more symmetric results.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23barycenterMode Online Documentation}*/
barycenterMode: boolean;
/**
Gets or sets whether or not the {@link SimplexNodePlacer} should break <em>long</em> edge segments in favor of a
potentially more compact layout.
A <em>long</em> edge is an edge that spans multiple layers. For each layer, a dummy node is necessary to model the edge path from
source to target.
<ul>
<li>Not breaking such edges means that there is a constraint that enforces the alignment of all these dummy nodes (same
x-coordinate). This favors straight edge paths.
<li>Breaking an edge means that the dummy nodes are not necessarily aligned, but additional bends can be generated if that
leads to a horizontally more compact layout result.
</ul>
<h2>Note</h2>
It is not guaranteed that enabling this setting leads to more compact results. The effect highly depends on the actual
input and other algorithm settings. Breaking up the segments might still not allow to make the layout horizontally more
compact.
<b>Warning:</b>
Enabling this setting will in general negatively affect the performance, which might be considerable at least for larger
graphs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimplexNodePlacer%23breakLongSegments Online Documentation}*/
breakLongSegments: boolean;
static $class: Class;
private $_SimplexNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SimplexNodePlacer;
}
export interface SwimlaneDescriptor extends YObject, IComparable {
}
/**
This class is used by {@link HierarchicLayoutCore} during the various phases and models a swimlane structure.
For layout orientation
{@link LayoutOrientation#TOP_TO_BOTTOM}, swimlanes partition the drawing area into vertical stripes. Each node is
placed within a user-specified lane. Swimlanes are, for example, often used in flow diagrams to separate logical units.
Each node should be associated with a SwimLaneDescriptor instance; nodes in the same lane may share one instance. The
results of the calculation of the swimlanes' geometry will be stored by the instances after the layout.
A {@link SwimlaneDescriptor} instance can be bound to the layout algorithm using a {@link IDataProvider}
registered with the graph using key {@link HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY} that returns a {@link SwimlaneDescriptor}
instance for each node of the graph.
<h2>Note</h2>
Class {@link PartitionGrid} offers a newer implementation which supports both horizontal and vertical swimlanes.
@see HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor Online Documentation}*/
export class SwimlaneDescriptor {
/**
Creates a new {@link SwimlaneDescriptor} instance with a given client {@link Object} and a given {@link IComparer}.
The given client {@link Object} should be an instance of {@link IComparable} if the given {@link IComparer} is <code>null</code>.
@param clientObject a client {@link Object} used for sorting the lanes
@param [comparer=null] a {@link IComparer} used for comparing the client objects
@throws {Exception} <b>(ArgumentError)</b> if the specified client {@link Object} is <code>null</code> or the specified {@link IComparer} is <code>null</code> but the specified client {@link Object} is not an instance of
{@link IComparable}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor Online Documentation}*/
constructor(clientObject: any | null, comparer?: IComparer<any> | null);
/**
Creates a new {@link SwimlaneDescriptor} instance with a given client {@link Object} and a given {@link IComparer}.
The given client {@link Object} should be an instance of {@link IComparable} if the given {@link IComparer} is <code>null</code>.
@param options The parameters to pass.
@param options.clientObject a client {@link Object} used for sorting the lanes
@param [options.comparer=null] a {@link IComparer} used for comparing the client objects
@param [options.indexFixed] Whether or not an index of a swimlane is fixed (based on the result of the comparison with the other swimlanes) or whether the index should be chosen automatically in order to minimize edge lengths. This option sets the {@link SwimlaneDescriptor#indexFixed} property on the created object.
@param [options.laneTightness] The tightness factor of the lane. This option sets the {@link SwimlaneDescriptor#laneTightness} property on the created object.
@param [options.minimumLaneWidth] The minimum width of the swimlane. This option sets the {@link SwimlaneDescriptor#minimumLaneWidth} property on the created object.
@param [options.leftLaneInset] The left insets of the swimlane in which no element will lie in the resulting layout. This option sets the {@link SwimlaneDescriptor#leftLaneInset} property on the created object.
@param [options.rightLaneInset] The right insets of the swimlane in which no element will lie in the resulting layout. This option sets the {@link SwimlaneDescriptor#rightLaneInset} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the specified client {@link Object} is <code>null</code> or the specified {@link IComparer} is <code>null</code> but the specified client {@link Object} is not an instance of
{@link IComparable}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor Online Documentation}*/
constructor(options: {
clientObject: any | null;
comparer?: IComparer<any> | null;
indexFixed?: boolean;
laneTightness?: number;
minimumLaneWidth?: number;
leftLaneInset?: number;
rightLaneInset?: number;
});
/**
Implements the {@link IComparable} interface using the {@link SwimlaneDescriptor#comparer} and {@link SwimlaneDescriptor#clientObject} fields.
@param o
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23compareTo(Object) Online Documentation}*/
compareTo(o: any | null): number;
/**
Gets or sets whether or not an index of a swimlane is fixed (based on the result of the comparison with the other
swimlanes) or whether the index should be chosen automatically in order to minimize edge lengths.
For all swimlanes where this property is set to
<code>true</code>, the relative ordering given by the client objects is preserved. The remaining swimlanes may be sorted again so
that the overall edge lengths are minimized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23indexFixed Online Documentation}*/
indexFixed: boolean;
/**
Gets or sets the client {@link Object object}.
The given client {@link Object} should be an instance of {@link IComparable}.
@throws {Exception} <b>(ArgumentError)</b> if there is currently no {@link IComparer comparator} and the specified object is not a {@link IComparable} instance
@see SwimlaneDescriptor#comparer
@see SwimlaneDescriptor#comparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23clientObject Online Documentation}*/
clientObject: any | null;
/**
Gets or sets the tightness factor of the lane.
This value should lie within <code>[0,1]</code> interval.
The greater the value, the more will the lane be forced to obtain its minimum possible width.
<ul>
<li>A value of <code>0.0d</code> will disable compression of the lane.
<li>A value of <code>1.0d</code> will try to force the lane to obtain its {@link SwimlaneDescriptor#minimumLaneWidth minimum width}.
</ul>
@throws {Exception} <b>(ArgumentError)</b> if the tightness value is outside <code>[0,1]</code> interval
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23laneTightness Online Documentation}*/
laneTightness: number;
/**
Gets or sets the {@link IComparer} instance used for sorting the lanes.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IComparer} is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23comparer Online Documentation}*/
comparer: IComparer<any> | null;
/**
Gets or sets the minimum width of the swimlane.
The minimum width should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified value negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23minimumLaneWidth Online Documentation}*/
minimumLaneWidth: number;
/**
Gets or sets the left insets of the swimlane in which no element will lie in the resulting layout.
The inset should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified value is less than <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23leftLaneInset Online Documentation}*/
leftLaneInset: number;
/**
Gets or sets the right insets of the swimlane in which no element will lie in the resulting layout.
The inset should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified value is less than <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23rightLaneInset Online Documentation}*/
rightLaneInset: number;
/**
Gets the computed position (smaller coordinate) of the swimlane after the layout has been calculated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23computedLanePosition Online Documentation}*/
readonly computedLanePosition: number;
/**
Gets the computed width of the swimlane after the layout has been calculated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23computedLaneWidth Online Documentation}*/
readonly computedLaneWidth: number;
/**
Gets the computed zero-based index of the swimlane after the layout has been calculated.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SwimlaneDescriptor%23computedLaneIndex Online Documentation}*/
readonly computedLaneIndex: number;
static $class: Class;
private $_SwimlaneDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SwimlaneDescriptor;
}
export interface TopLevelGroupToSwimlaneStage extends LayoutStageBase {
}
/**
This {@link ILayoutStage layout stage} can be used for automatically assigning {@link SwimlaneDescriptor} instances to nodes by
treating top-level group nodes as swimlanes.
This stage will modify the hierarchy by temporarily removing the top-level group nodes and assigning corresponding
descriptors to the remaining nodes. After running the core layout, the top-level group nodes will be arranged to
function as swimlanes. This class allows using the current coordinates of the group nodes to
{@link TopLevelGroupToSwimlaneStage#orderSwimlanesFromSketch arrange the swimlanes from sketch}. Also, the {@link TopLevelGroupToSwimlaneStage#spacing spacing} between swimlanes may be specified.
This stage can be {@link MultiStageLayout#appendStage appended} to the {@link HierarchicLayout}.
@see HierarchicLayout
@see SwimlaneDescriptor
@see GroupingKeys
@see {@link https://docs.yworks.com/yfileshtml/#/api/TopLevelGroupToSwimlaneStage Online Documentation}*/
export class TopLevelGroupToSwimlaneStage {
/**
Creates a new instance of {@link TopLevelGroupToSwimlaneStage} with the default settings.
@param [options] The parameters to pass.
@param [options.spacing] The spacing between the swimlanes. This option sets the {@link TopLevelGroupToSwimlaneStage#spacing} property on the created object.
@param [options.orderSwimlanesFromSketch] Whether or not the order of the swimlanes should be determined based on the current sketch. This option sets the {@link TopLevelGroupToSwimlaneStage#orderSwimlanesFromSketch} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TopLevelGroupToSwimlaneStage Online Documentation}*/
constructor(options?: {
spacing?: number;
orderSwimlanesFromSketch?: boolean;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Calculates the layout by automatically assigning {@link SwimlaneDescriptor} instances to nodes and treating
top-level group nodes as swimlanes.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TopLevelGroupToSwimlaneStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the spacing between the swimlanes.
The spacing should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/TopLevelGroupToSwimlaneStage%23spacing Online Documentation}*/
spacing: number;
/**
Gets or sets whether or not the order of the swimlanes should be determined based on the current sketch.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TopLevelGroupToSwimlaneStage%23orderSwimlanesFromSketch Online Documentation}*/
orderSwimlanesFromSketch: boolean;
static $class: Class;
private $_TopLevelGroupToSwimlaneStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TopLevelGroupToSwimlaneStage;
}
export interface TopologicalLayerer extends YObject, ILayerer {
}
/**
This class implements the layering phase of the {@link HierarchicLayout} (i.e., assignment of the nodes to
layers).
It uses topological sorting to assign layers to the nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TopologicalLayerer Online Documentation}*/
export class TopologicalLayerer {
/**
Creates a new instance of {@link TopologicalLayerer} with default settings.
@param [options] The parameters to pass.
@param [options.rankingPolicy] The (re-)ranking policy used by this {@link TopologicalLayerer} instance. This option sets the {@link TopologicalLayerer#rankingPolicy} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TopologicalLayerer Online Documentation}*/
constructor(options?: {
rankingPolicy?: RankingPolicy | RankingPolicyStringValues;
});
/**
Assigns all nodes of the graph to layers and adds them to the {@link ILayers} instance.
@param graph the input graph
@param layers the {@link ILayers} instance that will be filled with the results of the calculation
@param ldp the {@link ILayoutDataProvider} used for querying information about the nodes and edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/TopologicalLayerer%23assignLayers(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
assignLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
Gets or sets the (re-)ranking policy used by this {@link TopologicalLayerer} instance.
@throws {Exception} <b>(ArgumentError)</b> if an unknown ranking policy is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/TopologicalLayerer%23rankingPolicy Online Documentation}*/
rankingPolicy: RankingPolicy;
static $class: Class;
private $_TopologicalLayerer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TopologicalLayerer;
}
export interface TypeBasedDrawingDistanceCalculator extends YObject, IDrawingDistanceCalculator {
/**
@inheritdoc*/
dispose(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
@inheritdoc*/
getMinDistance(graph: LayoutGraph | null, layer: ILayer | null, ldp: ILayoutDataProvider | null, left: YNode | null, right: YNode | null): number;
/**
@inheritdoc*/
initialize(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
} /**
This class is a {@link IDrawingDistanceCalculator} implementation that returns the minimum distances for each
kind of node pair based on their type as returned by {@link INodeData#type}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeBasedDrawingDistanceCalculator Online Documentation}*/
export class TypeBasedDrawingDistanceCalculator {
/**
Creates a new instance of {@link TypeBasedDrawingDistanceCalculator} with default settings.
@param [options] The parameters to pass.
@param [options.compaction] Whether or not the compaction option is enabled. This option sets the {@link TypeBasedDrawingDistanceCalculator#compaction} property on the created object.
@param [options.nodeToNodeDistance] The minimum distance between two {@link YNode}s. This option sets the {@link TypeBasedDrawingDistanceCalculator#nodeToNodeDistance} property on the created object.
@param [options.nodeToEdgeDistance] The minimum distance between a {@link YNode} and an (non-adjacent) {@link Edge}. This option sets the {@link TypeBasedDrawingDistanceCalculator#nodeToEdgeDistance} property on the created object.
@param [options.edgeToEdgeDistance] The minimum distance between two edges. This option sets the {@link TypeBasedDrawingDistanceCalculator#edgeToEdgeDistance} property on the created object.
@param [options.minimumFirstSegmentLength] The minimum length of the first segment of the edge path (at the source node). This option sets the {@link TypeBasedDrawingDistanceCalculator#minimumFirstSegmentLength} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeBasedDrawingDistanceCalculator Online Documentation}*/
constructor(options?: {
compaction?: boolean;
nodeToNodeDistance?: number;
nodeToEdgeDistance?: number;
edgeToEdgeDistance?: number;
minimumFirstSegmentLength?: number;
});
/**
Gets or sets whether or not the compaction option is enabled.
<h2>Note</h2>
This option is automatically set by the layout algorithm if option {@link SimplexNodePlacer#nodeCompaction} or {@link SimplexNodePlacer#labelCompaction} is enabled.
@see SimplexNodePlacer#nodeCompaction
@see SimplexNodePlacer#labelCompaction
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeBasedDrawingDistanceCalculator%23compaction Online Documentation}*/
compaction: boolean;
/**
Gets or sets the minimum distance between two {@link YNode}s.
Minimum distance should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeBasedDrawingDistanceCalculator%23nodeToNodeDistance Online Documentation}*/
nodeToNodeDistance: number;
/**
Gets or sets the minimum distance between a {@link YNode} and an (non-adjacent) {@link Edge}.
Minimum distance should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeBasedDrawingDistanceCalculator%23nodeToEdgeDistance Online Documentation}*/
nodeToEdgeDistance: number;
/**
Gets or sets the minimum distance between two edges.
Minimum distance should be greater than <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeBasedDrawingDistanceCalculator%23edgeToEdgeDistance Online Documentation}*/
edgeToEdgeDistance: number;
/**
Gets or sets the minimum length of the first segment of the edge path (at the source node).
The minimum length should be greater than <code>0</code>.
If the value is <code>0.0d</code> the first segment does not need to be orthogonal.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length of the first segment is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeBasedDrawingDistanceCalculator%23minimumFirstSegmentLength Online Documentation}*/
minimumFirstSegmentLength: number;
static $class: Class;
private $_TypeBasedDrawingDistanceCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TypeBasedDrawingDistanceCalculator;
}
export interface WeightedLayerer extends YObject, ILayerer {
}
/**
This class implements the layering phase of the {@link HierarchicLayout} (i.e., the assignment of the nodes to
layers).
It first makes the directed graph acyclic. Then, it assigns the layers such that edge weights are respected. A weight
defines a minimum distance between the two end points of an edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer Online Documentation}*/
export class WeightedLayerer {
/**
Creates an instance of {@link WeightedLayerer} with the given key that provides access to the {@link IDataProvider}
that holds the edge weights and unlimited maximum duration for the {@link WeightedLayerer}.
@param [key=null] the key to access the {@link IDataProvider} that holds the edge weights
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer Online Documentation}*/
constructor(key?: any | null);
/**
Assigns all nodes of the graph to layers and adds them to the {@link ILayers} instance.
@param graph the input graph
@param layers the {@link ILayers} instance that will be filled with the results of the calculation
@param ldp the {@link ILayoutDataProvider} used for querying information about the nodes and edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23assignLayers(LayoutGraph%2CILayers%2CILayoutDataProvider) Online Documentation}*/
assignLayers(graph: LayoutGraph | null, layers: ILayers | null, ldp: ILayoutDataProvider | null): void;
/**
Assigns all nodes of the graph to layers.
@param graph the graph for which the layers are determined
@param layerID the {@link INodeMap} that will be filled by the layout algorithm and returns a zero-based layer index for each
node
@returns the number of layers
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23assignLayersFast(Graph%2CINodeMap) Online Documentation}*/
assignLayersFast(graph: Graph | null, layerID: INodeMap | null): number;
/**
Assigns all nodes of the graph to layers.
@param graph the graph for which the layers are determined
@param layerID the {@link INodeMap} that will be filled by the layout algorithm and returns the zero-based index of the layer
to which each node belongs
@returns the number of layers
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23assignLayersToMap(Graph%2CINodeMap) Online Documentation}*/
assignLayersToMap(graph: Graph | null, layerID: INodeMap | null): number;
/**
Assigns all nodes of the graph to layers.
@param graph the graph for which the layers are determined
@param layerID the {@link INodeMap} that will be filled by the layout algorithm and returns a zero-based layer index for each
node
@param reversedEdges an {@link EdgeList} containing the edges which had been reversed
@param weight the {@link IDataProvider} that returns an integer value (weight) for each edge
@returns the number of layers
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23assignNodeLayerWithDataProvider(LayoutGraph%2CINodeMap%2CEdgeList%2CIDataProvider) Online Documentation}*/
assignNodeLayerWithDataProvider(graph: LayoutGraph | null, layerID: INodeMap | null, reversedEdges: EdgeList | null, weight: IDataProvider | null): number;
/**
Returns the edge weight of the given edge using the weight based cycle removal heuristic.
By default, all edges are considered to have weight <code>1.0</code>.
It may be overridden if non-uniform weights are desired.
@param edge given edge
@returns the edge weight of the specified edge
@see WeightedLayerer#weightedCycleRemoval
@see WeightedLayerer#weightedCycleRemoval
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23getWeight(Edge) Online Documentation}*/
getWeight(edge: Edge | null): number;
/**
Removes cycles from the graph using a depth first search.
@param graph the graph
@param reversedEdges an {@link EdgeList} containing the edges which will be reversed
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23makeDFSAcyclic(Graph%2CEdgeList) Online Documentation}*/
makeDFSAcyclic(graph: Graph | null, reversedEdges: EdgeList | null): void;
/**
This is a {@link IDataProvider} that holds for each {@link Edge} an integer value representing its
weight.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23weight Online Documentation}*/
weight: IDataProvider | null;
/**
The key to access the {@link IDataProvider} that holds the edge weights.
More precisely, the corresponding {@link IDataProvider} maps each edge of the input graph to an integer value
representing the edge's weight.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23key Online Documentation}*/
key: any | null;
/**
Gets or sets whether or not the edges that need to be reversed are determined using an edge weight based heuristic or
using a depth first search based approach.
The weight based approach may result in fewer reversed edges (especially for cases with many multi-edges) whereas the
DFS based approach is likely to be faster.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23weightedCycleRemoval Online Documentation}*/
weightedCycleRemoval: boolean;
/**
Gets or sets the time limit in milliseconds set for the layout algorithm.
Values have to be greater than or equal to <code>0</code>.
<h2>Note</h2>
Restricting the maximum duration may result in a lower layout quality. Furthermore, the real runtime may exceed the
maximum duration since the layout algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightedLayerer%23maximumDuration Online Documentation}*/
maximumDuration: number;
static $class: Class;
private $_WeightedLayerer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WeightedLayerer;
}
export interface DefaultElementFactory extends YObject, IElementFactory {
/**
@inheritdoc*/
createConnectorEdge(context: LayoutContext | null, connector: YNode | null, opposite: YNode | null, origEdgeId: any | null, atTarget: boolean): Edge | null;
/**
@inheritdoc*/
createConnectorNode(context: LayoutContext | null, edgeIds: YList | null, representedNodeId: any | null): YNode | null;
/**
@inheritdoc*/
createProxyEdge(context: LayoutContext | null, proxyNode: YNode | null, opposite: YNode | null, replacingEdgeId: any | null, origNodeId: any | null): Edge | null;
/**
@inheritdoc*/
createProxyNode(context: LayoutContext | null, origNodeId: any | null): YNode | null;
/**
@inheritdoc*/
createProxyReferenceEdge(context: LayoutContext | null, proxyReference: YNode | null, opposite: YNode | null, referencingCopyId: any | null): Edge | null;
/**
@inheritdoc*/
createProxyReferenceNode(context: LayoutContext | null, referringProxyId: any | null): YNode | null;
} /**
This class is a default implementation of interface {@link IElementFactory}.
@see IElementFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultElementFactory Online Documentation}*/
export class DefaultElementFactory {
/**
Creates a new instance of {@link DefaultElementFactory}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultElementFactory Online Documentation}*/
constructor();
/**
Returns the default size for nodes created by this factory.
This method is called from {@link DefaultElementFactory#createConnectorNode},
{@link DefaultElementFactory#createProxyReferenceNode}, and/or
{@link DefaultElementFactory#createProxyNode}. It must return a non-<code>null</code> size with positive width and height.
Subclasses may implement this method to use custom size values.
@param context an object providing relevant layout information, for example, the current graph as well as various information about the
graph elements
@param id the element ID
@param type the type of the node to be created
@returns the default size of the new node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultElementFactory%23getDefaultNodeSize(LayoutContext%2CObject%2Cnumber) Online Documentation}*/
getDefaultNodeSize(context: LayoutContext | null, id: any | null, type: number): YDimension | null;
static $class: Class;
private $_DefaultElementFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultElementFactory;
}
/**
@see IEdgeInfo#type
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeType Online Documentation}*/
export enum EdgeType {
/**
An edge type that specifies a normal edge.
An edge is called <i>normal</i> if it does not belong to any other type.
@see IEdgeInfo#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeType%23NORMAL Online Documentation}*/
NORMAL,
/**
An edge type that specifies a connector edge.
An edge is called <i>connector</i> if it is connected to a connector node (a node of {@link INodeInfo#type type}
{@link NodeType#CONNECTOR}), i.e., it represents an edge of the input graph whose endpoints lie on different
pages.
@see IEdgeInfo#representedEdge
@see IEdgeInfo#type
@see NodeType#CONNECTOR
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeType%23CONNECTOR Online Documentation}*/
CONNECTOR,
/**
An edge type that specifies a proxy reference edge.
An edge is called <i>proxy reference</i> if it is connected to a proxy reference node (a node of {@link INodeInfo#type type}
{@link NodeType#PROXY_REFERENCE}), i.e., a node that refers to a proxy of an original node lying on another
page.
@see IEdgeInfo#type
@see NodeType#PROXY_REFERENCE
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeType%23PROXY_REFERENCE Online Documentation}*/
PROXY_REFERENCE,
/**
An edge type that specifies a proxy edge.
An edge is called <i>proxy</i> if it is connected to at least one proxy node and the other node isn't a connector or proxy reference
node. A proxy node has {@link INodeInfo#type type}
{@link NodeType#PROXY}) and represents a proxy of an original node lying on another page.
<h2>Note</h2>
Note that an edge between a proxy node and a connector/proxy reference node is always of type {@link EdgeType#CONNECTOR}
and {@link EdgeType#PROXY_REFERENCE} respectively.
@see IEdgeInfo#type
@see NodeType#PROXY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeType%23PROXY Online Documentation}*/
PROXY
}
/** Automatic conversion to {@link EdgeType}. */
type EdgeTypeStringValues = "normal" | "connector" | "proxy-reference" | "proxy";
/**
@see MultiPageLayout#groupingMode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingMode Online Documentation}*/
export enum GroupingMode {
/**
Node grouping specifier used when group nodes should be ignored.
@see MultiPageLayout#groupingMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingMode%23IGNORE Online Documentation}*/
IGNORE,
/**
Node grouping specifier used when only original nodes should be assigned to group nodes.
Hence, connector and proxy nodes (see {@link NodeType#CONNECTOR} and
{@link NodeType#PROXY}) are never assigned to group nodes even though the associated node lies within a group.
<h2>Note</h2>
A group node may be fragmented, i.e., its content may be placed on different pages.
@see MultiPageLayout#groupingMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingMode%23ORIGINAL_NODES_ONLY Online Documentation}*/
ORIGINAL_NODES_ONLY,
/**
Node grouping specifier used when special nodes like, e.g., connector and proxy nodes (see {@link NodeType#CONNECTOR}
and {@link NodeType#PROXY}) should be assigned to group nodes, too.
<h2>Note</h2>
A group node may be fragmented, i.e., its content may be placed on different pages.
@see MultiPageLayout#groupingMode
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupingMode%23ALL_NODES Online Documentation}*/
ALL_NODES
}
/** Automatic conversion to {@link GroupingMode}. */
type GroupingModeStringValues = "ignore" | "original-nodes-only" | "all-nodes";
/**
@see MultiPageLayout#edgeBundleModeMask
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleModes Online Documentation}*/
export enum EdgeBundleModes {
/**
Edge bundle mode constant that specifies that all multi-edges (edges with same endpoints) should be distinguished.
Hence, if such edges connect two nodes on different pages, a separate connector pair (see
{@link NodeType#CONNECTOR}) is used for each of them.
@see MultiPageLayout#edgeBundleModeMask
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleModes%23DISTINGUISH_MULTI_EDGES Online Documentation}*/
DISTINGUISH_MULTI_EDGES,
/**
Edge bundle mode constant that specifies that multi-edges (edges with same endpoints) should be distinguished if they
have different directions.
Hence, two edges with the same endpoints may share a connector pair (see
{@link NodeType#CONNECTOR}) if they point in the same direction (i.e., both have the same source and target
node).
@see MultiPageLayout#edgeBundleModeMask
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleModes%23DISTINGUISH_DIRECTIONS Online Documentation}*/
DISTINGUISH_DIRECTIONS,
/**
Edge bundle mode constant that specifies that multi-edges (edges with same endpoints) should be distinguished if they
are of different (user specified) type, see {@link MultiPageLayout#EDGE_TYPE_DP_KEY}.
Hence, two edges with the same endpoints and same type may share a connector pair (see
{@link NodeType#CONNECTOR}).
@see MultiPageLayout#edgeBundleModeMask
@see MultiPageLayout#EDGE_TYPE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundleModes%23DISTINGUISH_TYPES Online Documentation}*/
DISTINGUISH_TYPES
}
type EdgeBundleModesStringValues = "distinguish-multi-edges" | "distinguish-directions" | "distinguish-types";
/** Automatic conversion to {@link EdgeBundleModes}. */
type EdgeBundleModesConvertible = EdgeBundleModesStringValues | EdgeBundleModesStringValues[];
/**
@see INodeInfo#type
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeType Online Documentation}*/
export enum NodeType {
/**
A node type that specifies a normal node.
A node is called <i>normal</i> if it does not belong to any other type.
@see INodeInfo#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeType%23NORMAL Online Documentation}*/
NORMAL,
/**
A node type that specifies a group node.
A <i>group</i> node is a node that may contain other nodes.
@see INodeInfo#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeType%23GROUP Online Documentation}*/
GROUP,
/**
A node type that specifies a proxy node.
A <i>proxy</i> node is a copy of a {@link NodeType#NORMAL normal node}. It is created by the algorithm if a normal node and its neighbors or associated {@link NodeType#CONNECTOR connectors}
do not fit onto a single page. Hence, in such cases, the neighbors/connectors associated with a normal node may be
distributed among different proxy nodes associated with this node.
<h2>Note</h2>
A {@link NodeType#NORMAL normal node} may be associated with multiple {@link NodeType#PROXY proxy nodes} placed on different pages.
@see INodeInfo#type
@see NodeType#NORMAL
@see NodeType#PROXY_REFERENCE
@see MultiPageLayout#createProxyReferenceNodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeType%23PROXY Online Documentation}*/
PROXY,
/**
A node type that specifies a proxy reference node.
For each {@link NodeType#PROXY proxy node} there is exactly one matching proxy reference node (except if property {@link MultiPageLayout#createProxyReferenceNodes} is disabled). The reference
node refers to the proxy and is either connected to the {@link NodeType#NORMAL normal node} associated with this proxy or one of its other proxy
nodes (a normal node may be associated with multiple proxy nodes). Hence, for a node connected to a proxy reference node
there always exists a copy located on another page.
@see INodeInfo#type
@see NodeType#PROXY
@see MultiPageLayout#createProxyReferenceNodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeType%23PROXY_REFERENCE Online Documentation}*/
PROXY_REFERENCE,
/**
A node type that specifies a connector node.
Each edge of the input graph that connects two nodes <code>v</code> and <code>w</code> that are placed on different pages is split by a pair of
matching connector nodes (one for each of the two pages). One of the connector nodes is connected to <code>v</code> and the other to
<code>w</code>. Hence, each connector represents a jump mark to the matching connector and guarantees that no information is lost as
it would be the case if such edges are simply removed.
@see INodeInfo#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeType%23CONNECTOR Online Documentation}*/
CONNECTOR
}
/** Automatic conversion to {@link NodeType}. */
type NodeTypeStringValues = "normal" | "group" | "proxy" | "proxy-reference" | "connector";
export interface IEdgeInfo {
}
/**
A class that holds all information associated with an edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeInfo Online Documentation}*/
export abstract class IEdgeInfo {
/**
Gets the unique ID of the associated edge.
@see MultiPageLayoutResult#getEdgeInfo
@see IElementInfoManager#getEdgeInfo
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeInfo%23id Online Documentation}*/
abstract readonly id: any | null;
/**
Gets the type of the associated edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeInfo%23type Online Documentation}*/
abstract readonly type: EdgeType;
/**
Gets the original edge associated with this {@link IEdgeInfo}.
More precisely, for edges of {@link IEdgeInfo#type type} {@link EdgeType#CONNECTOR}, this method returns the edge of the input graph
that was split by the connector edge. Otherwise, it returns <code>null</code>.
@see EdgeType#CONNECTOR
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeInfo%23representedEdge Online Documentation}*/
abstract readonly representedEdge: Edge | null;
static $class: Class;
private $_IEdgeInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeInfo;
}
export interface IEdgeLabelInfo {
}
/**
A class that holds all information related to an edge label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelInfo Online Documentation}*/
export abstract class IEdgeLabelInfo {
/**
Gets the unique ID of the associated edge label.
@see MultiPageLayoutResult#getEdgeLabelInfo
@see IElementInfoManager#getEdgeLabelInfo
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEdgeLabelInfo%23id Online Documentation}*/
abstract readonly id: any | null;
static $class: Class;
private $_IEdgeLabelInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEdgeLabelInfo;
}
export interface IElementFactory {
}
/**
Factory used by class {@link MultiPageLayout} to create special nodes and edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementFactory Online Documentation}*/
export abstract class IElementFactory {
/**
Callback method for creating an edge of type {@link EdgeType#CONNECTOR}.
Such an edge connects the given {@link NodeType#CONNECTOR connector} and opposite node.
<h2>Note</h2>
All implementations have to use method {@link LayoutContext#createEdge} to create the edge.
@param context an object providing relevant layout information, for example, the current graph as well as various information about the
graph elements
@param connector the endpoint of the edge that represents the connector node
@param opposite the other endpoint of the edge
@param origEdgeId the ID of the edge that is split by the connector edge
@param atTarget <code>true</code> if the connector node is the target of the edge, <code>false</code> otherwise
@returns the created connector edge
@see EdgeType#CONNECTOR
@see NodeType#CONNECTOR
@see LayoutContext#createEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementFactory%23createConnectorEdge(LayoutContext%2CYNode%2CYNode%2CObject%2Cboolean) Online Documentation}*/
abstract createConnectorEdge(context: LayoutContext | null, connector: YNode | null, opposite: YNode | null, origEdgeId: any | null, atTarget: boolean): Edge | null;
/**
Callback method for creating a node of type {@link NodeType#CONNECTOR}.
<h2>Note</h2>
Multi-edges (edges connecting the same node pair) may be split by the same connector pair, see {@link MultiPageLayout#edgeBundleModeMask}. Hence, the list of <code>edgeIds</code>
may contain several entries.
<h2>Note</h2>
All implementations have to use method {@link LayoutContext#createNode} to create the node.
@param context an object providing relevant layout information, for example, the current graph as well as various information about the
graph elements
@param edgesIds a {@link YList list} that contains the IDs of edges that are split by this connector
@param representedNodeId the ID of the node that is represented by this connector
@returns the created connector node
@see NodeType#CONNECTOR
@see LayoutContext#createNode
@see INodeInfo#representedNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementFactory%23createConnectorNode(LayoutContext%2CYList%2CObject) Online Documentation}*/
abstract createConnectorNode(context: LayoutContext | null, edgesIds: YList | null, representedNodeId: any | null): YNode | null;
/**
Callback method for creating an edge of type {@link EdgeType#PROXY}.
Such an edge should connect the given {@link NodeType#PROXY proxy} and opposite node.
<h2>Note</h2>
All implementations have to use method {@link LayoutContext#createEdge} to create the edge.
@param context an object providing relevant layout information, for example, the current graph as well as various information about the
graph elements
@param proxyNode the endpoint of the edge that represents the proxy node
@param opposite the other endpoint of the new edge
@param replacingEdgeId the ID of the related edge that is connected to the original node
@param origNodeId the ID of the original node to which the proxy node refers
@returns the created proxy edge
@see EdgeType#PROXY
@see NodeType#PROXY
@see LayoutContext#createEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementFactory%23createProxyEdge(LayoutContext%2CYNode%2CYNode%2CObject%2CObject) Online Documentation}*/
abstract createProxyEdge(context: LayoutContext | null, proxyNode: YNode | null, opposite: YNode | null, replacingEdgeId: any | null, origNodeId: any | null): Edge | null;
/**
Callback method for creating a node of type {@link NodeType#PROXY}.
<h2>Note</h2>
All implementations have to use method {@link LayoutContext#createNode} to create the node.
@param context an object providing relevant layout information, for example, the current graph as well as various information about the
graph elements
@param origNodeId the ID of the node for which a proxy has to be created
@returns the created proxy node
@see LayoutContext#createNode
@see NodeType#PROXY_REFERENCE
@see NodeType#PROXY
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementFactory%23createProxyNode(LayoutContext%2CObject) Online Documentation}*/
abstract createProxyNode(context: LayoutContext | null, origNodeId: any | null): YNode | null;
/**
Callback method for creating an edge of type {@link EdgeType#PROXY_REFERENCE}.
Such an (undirected) edge should connect the given {@link NodeType#PROXY_REFERENCE proxy reference} and opposite node.
<h2>Note</h2>
All implementations have to use method {@link LayoutContext#createEdge} to create the edge.
@param context an object providing relevant layout information, for example, the current graph as well as various information about the
graph elements
@param proxyReference the endpoint of the edge that represents the proxy reference node
@param opposite the other endpoint of the new edge
@param referencingCopyId the ID of the proxy node that is referenced by the proxy reference node
@returns the created proxy reference edge
@see EdgeType#PROXY_REFERENCE
@see NodeType#PROXY_REFERENCE
@see LayoutContext#createEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementFactory%23createProxyReferenceEdge(LayoutContext%2CYNode%2CYNode%2CObject) Online Documentation}*/
abstract createProxyReferenceEdge(context: LayoutContext | null, proxyReference: YNode | null, opposite: YNode | null, referencingCopyId: any | null): Edge | null;
/**
Callback method for creating a node of type {@link NodeType#PROXY_REFERENCE}.
<h2>Note</h2>
All implementations have to use method {@link LayoutContext#createNode} to create the node.
@param context an object providing relevant layout information, for example, the current graph as well as various information about the
graph elements
@param referringProxyId the ID of the proxy to which the created node refers
@returns the new proxy reference node
@see NodeType#PROXY_REFERENCE
@see LayoutContext#createNode
@see NodeType#PROXY
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementFactory%23createProxyReferenceNode(LayoutContext%2CObject) Online Documentation}*/
abstract createProxyReferenceNode(context: LayoutContext | null, referringProxyId: any | null): YNode | null;
static $class: Class;
private $_IElementFactory: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IElementFactory;
}
export interface IElementInfoManager {
}
/**
Interface that offers access to the information objects of graph elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementInfoManager Online Documentation}*/
export abstract class IElementInfoManager {
/**
Returns the edge information object of the given edge.
@param edge the edge for which the information object should be returned
@returns the edge information object of the given edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementInfoManager%23getEdgeInfo(Edge) Online Documentation}*/
abstract getEdgeInfo(edge: Edge | null): IEdgeInfo | null;
/**
Returns the edge label information object of the given edge label layout.
@param edgeLabelLayout the edge label layout for which the information object should be returned
@returns the edge label information object of the given edge label layout
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementInfoManager%23getEdgeLabelInfo(IEdgeLabelLayout) Online Documentation}*/
abstract getEdgeLabelInfo(edgeLabelLayout: IEdgeLabelLayout | null): IEdgeLabelInfo | null;
/**
Returns the node information object of the given node.
@param node the node for which the information object should be returned
@returns the node information object of the given node
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementInfoManager%23getNodeInfo(YNode) Online Documentation}*/
abstract getNodeInfo(node: YNode | null): INodeInfo | null;
/**
Returns the node label layout information object of the given node label layout.
@param nodeLabelLayout the node label layout for which the information object should be returned
@returns the node label information object of the given node label layout
@see {@link https://docs.yworks.com/yfileshtml/#/api/IElementInfoManager%23getNodeLabelInfo(INodeLabelLayout) Online Documentation}*/
abstract getNodeLabelInfo(nodeLabelLayout: INodeLabelLayout | null): INodeLabelInfo | null;
static $class: Class;
private $_IElementInfoManager: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IElementInfoManager;
}
export interface ILayoutCallback {
}
/**
Callback that is invoked when a {@link MultiPageLayout} has calculated a new multi-page layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutCallback Online Documentation}*/
export abstract class ILayoutCallback {
/**
Creates an implementation of the interface {@link ILayoutCallback} by using the given function as
implementation for its {@link ILayoutCallback#layoutDone} method.
@param layoutDone A function for {@link ILayoutCallback}'s single abstract method {@link ILayoutCallback#layoutDone}.
@returns An instance of the {@link ILayoutCallback} interface based on the given function.*/
static create(layoutDone: (result: MultiPageLayoutResult | null) => void): ILayoutCallback;
/**
Invoked from {@link MultiPageLayout#applyLayout} when a new multi-page layout has been calculated.
@param result the result of the layout calculation
@see MultiPageLayout#applyLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ILayoutCallback%23layoutDone(MultiPageLayoutResult) Online Documentation}*/
abstract layoutDone(result: MultiPageLayoutResult | null): void;
static $class: Class;
private $_ILayoutCallback: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ILayoutCallback;
}
export interface INodeInfo {
}
/**
This interface offers access to information associated with a node.
The information can be either obtained from {@link MultiPageLayoutResult#getNodeInfo} or
{@link LayoutContext#getNodeInfo}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeInfo Online Documentation}*/
export abstract class INodeInfo {
/**
Gets the node of the input graph that is represented by the node associated with this {@link INodeInfo info object}.
Note that, for the special nodes inserted to guarantee that no information is lost, there are no corresponding nodes in
the input graph. Hence, special nodes are mapped to common nodes in the following way:
<ul>
<li>A {@link NodeType#CONNECTOR connector node} is mapped to the common node connected to the matching connector.
<li>A {@link NodeType#PROXY proxy node} is mapped to the associated common node (i.e., the proxy is a copy of that node).
<li>A {@link NodeType#PROXY_REFERENCE proxy reference node} is not mapped to any node, and, thus this method returns <code>null</code> for such nodes.
</ul>
<h2>Note</h2>
All nodes of the {@link MultiPageLayoutResult#getPage page graphs} are created by the layout algorithm, the input graph remains unchanged. Matching elements
are mapped to the same element ID, see {@link MultiPageLayout#NODE_ID_DP_KEY},
{@link MultiPageLayout#EDGE_ID_DP_KEY}, {@link MultiPageLayout#NODE_LABEL_ID_DP_KEY} and
{@link MultiPageLayout#EDGE_LABEL_ID_DP_KEY}. This method always returns a node of the input graph!
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeInfo%23representedNode Online Documentation}*/
abstract readonly representedNode: YNode | null;
/**
Gets the page number of the {@link MultiPageLayoutResult#getPage page graph} that contains the node associated with this {@link INodeInfo info object}.
<h2>Note</h2>
This information is first available after the layout run.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeInfo%23pageNo Online Documentation}*/
abstract readonly pageNo: number;
/**
Gets the type of the node associated with this {@link INodeInfo info object}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeInfo%23type Online Documentation}*/
abstract readonly type: NodeType;
/**
Gets the unique ID of the node associated with this {@link INodeInfo info object}.
@see MultiPageLayout#NODE_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeInfo%23id Online Documentation}*/
abstract readonly id: any | null;
/**
Gets the node that refers to the one associated with this {@link INodeInfo info object}.
More precisely, if the node associated with this info object
<ul>
<li>is a {@link NodeType#CONNECTOR connector node}, this method returns the matching connector node,
<li>is a {@link NodeType#PROXY proxy node}, it returns the corresponding {@link NodeType#PROXY_REFERENCE proxy reference node} (or the associated common node if property {@link MultiPageLayout#createProxyReferenceNodes} is
disabled),
<li>is a {@link NodeType#PROXY_REFERENCE proxy reference node}, it returns the corresponding {@link NodeType#PROXY proxy node},
<li>or otherwise, it returns <code>null</code>.
</ul>
@see NodeType#CONNECTOR
@see NodeType#PROXY
@see NodeType#PROXY_REFERENCE
@see MultiPageLayout#createProxyReferenceNodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeInfo%23referencingNode Online Documentation}*/
abstract readonly referencingNode: YNode | null;
static $class: Class;
private $_INodeInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeInfo;
}
export interface INodeLabelInfo {
}
/**
A class that holds all information related to a node label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelInfo Online Documentation}*/
export abstract class INodeLabelInfo {
/**
Gets the unique ID of the associated node label.
@see MultiPageLayoutResult#getNodeLabelInfo
@see IElementInfoManager#getNodeLabelInfo
@see {@link https://docs.yworks.com/yfileshtml/#/api/INodeLabelInfo%23id Online Documentation}*/
abstract readonly id: any | null;
static $class: Class;
private $_INodeLabelInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is INodeLabelInfo;
}
export interface LayoutContext extends YObject, IElementInfoManager {
/**
@inheritdoc*/
getEdgeInfo(edge: Edge | null): IEdgeInfo | null;
/**
@inheritdoc*/
getEdgeLabelInfo(edgeLabelLayout: IEdgeLabelLayout | null): IEdgeLabelInfo | null;
/**
@inheritdoc*/
getNodeInfo(node: YNode | null): INodeInfo | null;
/**
@inheritdoc*/
getNodeLabelInfo(nodeLabelLayout: INodeLabelLayout | null): INodeLabelInfo | null;
} /**
This class represents the layout context.
It provides several methods to access data as well as adding and removing elements.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext Online Documentation}*/
export class LayoutContext {
/**
Adds a label to a given edge.
@param label the label that should be added to the given edge
@param labeledElement the edge to which the label should be added
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23addEdgeLabel(IEdgeLabelLayout%2CEdge) Online Documentation}*/
addEdgeLabel(label: IEdgeLabelLayout | null, labeledElement: Edge | null): void;
/**
Adds a label to a given node.
@param label the label that should be added to the given node
@param labeledElement the node to which the label should be added
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23addNodeLabel(INodeLabelLayout%2CYNode) Online Documentation}*/
addNodeLabel(label: INodeLabelLayout | null, labeledElement: YNode | null): void;
/**
All methods of the {@link IElementFactory} have to use this method to create an edge.
@param source the source of the edge
@param target the target of the edge
@param sameDataElementId the ID of the edge whose data should be copied to the created edge (or <code>null</code> if no data should be copied)
@returns the created edge
@see IElementFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23createEdge(YNode%2CYNode%2CObject) Online Documentation}*/
createEdge(source: YNode | null, target: YNode | null, sameDataElementId: any | null): Edge | null;
/**
All methods of the {@link IElementFactory} have to use this method to create a node.
@param sameDataElementId the ID of the node whose data should be copied to the created node (or <code>null</code> if no data should be copied)
@returns the created node
@see IElementFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23createNode(Object) Online Documentation}*/
createNode(sameDataElementId: any | null): YNode | null;
/**
Returns the edge of the input graph with the given ID or <code>null</code> if there is no such edge.
@param id the ID of the edge that should be returned
@returns the edge of the input graph with the given ID or <code>null</code> if there is no such edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23getOriginalEdge(Object) Online Documentation}*/
getOriginalEdge(id: any | null): Edge | null;
/**
Returns the node of the input graph with the given ID or <code>null</code> if there is no such node.
@param id the ID of the node that should be returned
@returns the node of the input graph with the given ID or <code>null</code> if there is no such node
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23getOriginalNode(Object) Online Documentation}*/
getOriginalNode(id: any | null): YNode | null;
/**
Returns the edge of a page graph (a graph that represents the result of the multi-page layout) with the given ID or <code>null</code>
if there is no such edge.
@param id the ID of the edge that should be returned
@returns the edge of a page graph with the given ID or <code>null</code> if there is no such edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23getPageEdge(Object) Online Documentation}*/
getPageEdge(id: any | null): Edge | null;
/**
Returns the node of a page graph (a graph that represents the result of the multi-page layout) with the given ID or <code>null</code>
if there is no such node.
@param id the ID of the node that should be returned
@returns the node of a page graph with the given ID or <code>null</code> if there is no such node
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23getPageNode(Object) Online Documentation}*/
getPageNode(id: any | null): YNode | null;
/**
Removes a label from a given edge.
@param label the label that should be removed
@param labeledElement the edge owning the label
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23removeEdgeLabel(IEdgeLabelLayout%2CEdge) Online Documentation}*/
removeEdgeLabel(label: IEdgeLabelLayout | null, labeledElement: Edge | null): boolean;
/**
Removes a label from a given node.
@param label the label that should be removed
@param labeledElement the node owning the label
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23removeNodeLabel(INodeLabelLayout%2CYNode) Online Documentation}*/
removeNodeLabel(label: INodeLabelLayout | null, labeledElement: YNode | null): boolean;
/**
Gets the relevant graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23graph Online Documentation}*/
readonly graph: LayoutGraph | null;
/**
Gets the layouter object that belongs to this layout context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutContext%23layout Online Documentation}*/
readonly layout: MultiPageLayout | null;
static $class: Class;
private $_LayoutContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutContext;
}
export interface MultiPageLayout extends LayoutStageBase {
}
/**
This layout algorithm subdivides the input graph into several {@link LayoutGraph}s (called page graphs) such that
the layout (calculated by the specified {@link LayoutStageBase#coreLayout core layout algorithm}) of each graph fits the {@link MultiPageLayout#maximumPageSize specified maximum page size}.
The algorithm adds special nodes to guarantee that no information is lost when splitting edges that connect nodes placed
on different pages (for more details take a look at the concept described below).
<b>Layout Style</b>
This multi-page layout algorithm subdivides the input graph into several smaller ones and applies existing layout
algorithms to each of them. Hence, its layout style heavily depends on the selected {@link LayoutStageBase#coreLayout core layout algorithm} used for each
single page. Furthermore, the algorithm uses several refinement steps to produce more compact results.
<b>Features</b>
Similar to the layout style, the supported feature set mainly depends on the features supported by the specified
{@link LayoutStageBase#coreLayout core layout algorithm}. Note that due to the underlying approach of the {@link MultiPageLayout} it doesn't
support {@link PartitionGrid}s. Furthermore, while the {@link MultiPageLayout} is able to support groups
(see {@link MultiPageLayout#groupingMode}), it doesn't support edges incident to group nodes.
<b>Concept</b>
To guarantee that no information is lost, this layout algorithm replaces edges between nodes on different pages by
so-called {@link NodeType#CONNECTOR connector nodes}. Furthermore, it may replicate nodes (the clones are called {@link NodeType#PROXY proxy nodes}) and insert special
nodes to refer to such nodes (so-called {@link NodeType#PROXY_REFERENCE proxy reference nodes}).
Unlike other yFiles layout algorithms, {@link MultiPageLayout} does not modify its input graph but returns its
result as a {@link MultiPageLayoutResult}. To be able to profit as much as possible from existing layout
support, this layout algorithm implements the {@link ILayoutAlgorithm} interface, although method {@link ILayoutAlgorithm#applyLayout doLayout} does
not specify a return value. Therefore, client code has to {@link MultiPageLayout#layoutCallback register} a {@link ILayoutCallback} that is invoked
when the algorithm has calculated a new multi-page result.
<b>Warning:</b> Client code must register {@link IDataProvider}s for keys {@link MultiPageLayout#NODE_ID_DP_KEY},
{@link MultiPageLayout#EDGE_ID_DP_KEY}, {@link MultiPageLayout#NODE_LABEL_ID_DP_KEY}, and {@link MultiPageLayout#EDGE_LABEL_ID_DP_KEY}
before calling {@link MultiPageLayout#applyLayout} or {@link MultiPageLayout#calculateLayout}.
<h2>Note</h2>
This layout algorithm does not consider the initial drawing of the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout Online Documentation}*/
export class MultiPageLayout {
/**
Creates a new {@link MultiPageLayout} instance.
@param coreLayout the layout algorithm used for a single page
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout Online Documentation}*/
constructor(coreLayout: ILayoutAlgorithm | null);
/**
Creates a new {@link MultiPageLayout} instance.
@param options The parameters to pass.
@param options.coreLayout the layout algorithm used for a single page
@param [options.createProxyReferenceNodes] Whether or not the algorithm should create {@link NodeType#PROXY_REFERENCE proxy reference nodes}. This option sets the {@link MultiPageLayout#createProxyReferenceNodes} property on the created object.
@param [options.strictClusterSeparation] Whether or not the algorithm should separate nodes with different {@link MultiPageLayout#NODE_CLUSTER_ID_DP_KEY cluster IDs}. This option sets the {@link MultiPageLayout#strictClusterSeparation} property on the created object.
@param [options.labelingEnabled] Whether or not the given {@link MultiPageLayout#labeling labeling algorithm} places the labels of the input graph. This option sets the {@link MultiPageLayout#labelingEnabled} property on the created object.
@param [options.elementFactory] The {@link IElementFactory element factory} for creating special nodes and edges in a multi-page layout. This option sets the {@link MultiPageLayout#elementFactory} property on the created object.
@param [options.layoutCallback] The {@link ILayoutCallback callback} that is notified upon completion of multi-page layout calculation runs. This option sets the {@link MultiPageLayout#layoutCallback} property on the created object.
@param [options.additionalParentCount] The number of additional tree parent {@link NodeType#PROXY proxies} that the algorithm tries to place on a page. This option sets the {@link MultiPageLayout#additionalParentCount} property on the created object.
@param [options.edgeBundleModeMask] The bit mask for defining edge bundles. This option sets the {@link MultiPageLayout#edgeBundleModeMask} property on the created object.
@param [options.groupingMode] How to handle special nodes (like connector and proxy nodes) with respect to groups. This option sets the {@link MultiPageLayout#groupingMode} property on the created object.
@param [options.maximumDuration] The preferred time limit (in milliseconds) for the layout algorithm. This option sets the {@link MultiPageLayout#maximumDuration} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiPageLayout#labeling} property on the created object.
@param [options.multipleComponentsOnSinglePage] Whether or not different connected components may be placed on a single page. This option sets the {@link MultiPageLayout#multipleComponentsOnSinglePage} property on the created object.
@param [options.maximumPageSize] The maximum size of a single page. This option sets the {@link MultiPageLayout#maximumPageSize} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout Online Documentation}*/
constructor(options: {
coreLayout: ILayoutAlgorithm | null;
createProxyReferenceNodes?: boolean;
strictClusterSeparation?: boolean;
labelingEnabled?: boolean;
elementFactory?: IElementFactory | null;
layoutCallback?: ILayoutCallback | null;
additionalParentCount?: number;
edgeBundleModeMask?: EdgeBundleModes | EdgeBundleModesConvertible;
groupingMode?: GroupingMode | GroupingModeStringValues;
maximumDuration?: number;
labeling?: ILayoutStage | null;
multipleComponentsOnSinglePage?: boolean;
maximumPageSize?: YDimension | null;
});
/**
This method is called to further improve the layout results.
The marked nodes have to be placed without changing the coordinates of the fixed (non-marked) elements and without
exceeding the specified maximum page size. Furthermore, the basic layout properties should be maintained.
Subclasses may implement a custom layout strategy for this step.
@param graph the input graph
@param incrementalNodesDP a {@link IDataProvider} that returns a boolean value indicating whether or not a node is marked; the positions
of the non-marked nodes are not allowed to change
@param context the current layout context
@see LayoutContext
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23applyIncrementalLayout(LayoutGraph%2CIDataProvider%2CLayoutContext) Online Documentation}*/
applyIncrementalLayout(graph: LayoutGraph | null, incrementalNodesDP: IDataProvider | null, context: LayoutContext | null): void;
/**
Calculates a new multi-page layout for the specified graph.
This method calls {@link MultiPageLayout#calculateLayout} and notifies the registered {@link MultiPageLayout#layoutCallback layout callback} of the
calculated result.
<b>Warning:</b> Unlike other implementations of {@link ILayoutAlgorithm#applyLayout} method, the result of the layout calculation
will <b>not</b> be applied to the input graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Calculates a new multi-page layout for the specified graph.
Unlike method {@link MultiPageLayout#applyLayout}, this method ignores the registered {@link MultiPageLayout#layoutCallback layout callback} and
directly returns the calculated {@link MultiPageLayoutResult}.
<b>Warning:</b> Unlike other implementations of the {@link ILayoutAlgorithm#applyLayout} method, the result of the layout
calculation will <b>not</b> be applied to the input graph.
@param graph the input graph
@returns an instance of {@link MultiPageLayoutResult} that represents the result of the layout run
@throws {Exception} <b>(ArgumentError)</b> if for one of the keys {@link MultiPageLayout#NODE_ID_DP_KEY}, {@link MultiPageLayout#EDGE_ID_DP_KEY},
{@link MultiPageLayout#NODE_LABEL_ID_DP_KEY} and {@link MultiPageLayout#EDGE_LABEL_ID_DP_KEY} no {@link IDataProvider}
is registered with the given graph
@see MultiPageLayoutResult
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23calculateLayout(LayoutGraph) Online Documentation}*/
calculateLayout(graph: LayoutGraph | null): MultiPageLayoutResult | null;
/**
This method creates the {@link IElementFactory element factory} for multi-page layouts.
It is called once from {@link MultiPageLayout#elementFactory} if no factory has been explicitly set using {@link MultiPageLayout#elementFactory}. By default, this method returns an instance of
{@link DefaultElementFactory}.
Subclasses may create a custom implementation of the element factory.
@returns the created element factory
@see MultiPageLayout#elementFactory
@see MultiPageLayout#elementFactory
@see IElementFactory
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23createElementFactory() Online Documentation}*/
createElementFactory(): IElementFactory | null;
/**
This method is called during a postprocessing step that reduces the number of connectors.
A pair of connectors can be removed if both connector nodes are placed on the same page. The method removes both
connector nodes and restores the original edges by calling {@link MultiPageLayout#routeRestoredEdges}.
Subclasses may implement a custom remove/restore strategy.
<h2>Note</h2>
If this method returns <code>false</code> the graph was not changed, i.e., the connector pair still exists.
@param connector1 the first connector of the connector pair
@param connector2 the second connector of the connector pair
@param originalEdgeIds the IDs of the original edges that have to be restored
@param context the current layout context
@returns <code>true</code> if the removal of the connector pair was successful, <code>false</code> otherwise
@see MultiPageLayout#routeRestoredEdges
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23removeConnectorPair(YNode%2CYNode%2CYList%2CLayoutContext) Online Documentation}*/
removeConnectorPair(connector1: YNode | null, connector2: YNode | null, originalEdgeIds: YList | null, context: LayoutContext | null): boolean;
/**
This method is called whenever some single edges have to be routed without changing the position of nodes (e.g., by {@link MultiPageLayout#removeConnectorPair}
to route the restored edges).
It has to guarantee that the routes are fully contained within the given bounding rectangle.
Subclasses may implement a custom routing strategy for the restored edges.
@param graph the relevant graph
@param selectedEdgesDP a {@link IDataProvider} that returns a boolean value indicating whether or not an edge should be rerouted
@param boundingRectangle the edge routes should be fully contained within this rectangle
@see MultiPageLayout#removeConnectorPair
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23routeRestoredEdges(LayoutGraph%2CIDataProvider%2CYRectangle) Online Documentation}*/
routeRestoredEdges(graph: LayoutGraph | null, selectedEdgesDP: IDataProvider | null, boundingRectangle: YRectangle | null): void;
/**
Data provider key for assigning a cluster ID to common (non-group) nodes
<h2>Note</h2>
If there is no {@link IDataProvider} associated with this key, the algorithm assumes that there are no such
preferences.
<h2>Note</h2>
Property {@link MultiPageLayout#strictClusterSeparation} allows to specify whether or not a page may contain nodes with different cluster IDs.
@see MultiPageLayout#strictClusterSeparation
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23NODE_CLUSTER_ID_DP_KEY Online Documentation}*/
static NODE_CLUSTER_ID_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for mapping each node of the input graph to a unique ID
<b>Warning:</b> This {@link IDataProvider} has to be specified by the user!
<b>Warning:</b> There must not be two nodes associated with the same object.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23NODE_ID_DP_KEY Online Documentation}*/
static NODE_ID_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for mapping each edge of the input graph to a unique ID
<b>Warning:</b> This {@link IDataProvider} has to be specified by the user!
<b>Warning:</b> There must not be two edges associated with the same object.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23EDGE_ID_DP_KEY Online Documentation}*/
static EDGE_ID_DP_KEY: EdgeDpKey<any> | null;
/**
Data provider key for mapping each edge label of the input graph to a unique ID
<b>Warning:</b> This {@link IDataProvider} has to be specified by the user!
<b>Warning:</b> There must not be two edge labels associated with the same object.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23EDGE_LABEL_ID_DP_KEY Online Documentation}*/
static EDGE_LABEL_ID_DP_KEY: IEdgeLabelLayoutDpKey<any> | null;
/**
Data provider key for mapping each node label of the input graph to a unique ID
<b>Warning:</b> This {@link IDataProvider} has to be specified by the user!
<b>Warning:</b> There must not be two node labels associated with the same object.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23NODE_LABEL_ID_DP_KEY Online Documentation}*/
static NODE_LABEL_ID_DP_KEY: INodeLabelLayoutDpKey<any> | null;
/**
Data provider key for specifying the type of multi-edges
If {@link MultiPageLayout#edgeBundleModeMask} &amp; {@link EdgeBundleModes#DISTINGUISH_TYPES} == 1, multi-edges (edges with same endpoints) associated with
different types (objects) are distinguished, i.e., they are split by different connector pairs (see
{@link NodeType#CONNECTOR}).
@see EdgeBundleModes#DISTINGUISH_TYPES
@see MultiPageLayout#edgeBundleModeMask
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23EDGE_TYPE_DP_KEY Online Documentation}*/
static EDGE_TYPE_DP_KEY: EdgeDpKey<any> | null;
/**
Gets or sets whether or not the algorithm should create {@link NodeType#PROXY_REFERENCE proxy reference nodes}.
If this option is disabled, the algorithm doesn't create proxy reference nodes in which case the {@link INodeInfo#referencingNode referencing node} of a {@link NodeType#PROXY proxy node}
corresponds to its {@link INodeInfo#representedNode represented node}.
@see NodeType#PROXY_REFERENCE
@see NodeType#PROXY
@see INodeInfo#referencingNode
@see INodeInfo#representedNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23createProxyReferenceNodes Online Documentation}*/
createProxyReferenceNodes: boolean;
/**
Gets or sets whether or not the algorithm should separate nodes with different {@link MultiPageLayout#NODE_CLUSTER_ID_DP_KEY cluster IDs}.
More precisely, if this option is enabled, the algorithm doesn't place nodes with different cluster IDs onto the same
page. Otherwise, such a placement is possible.
<h2>Note</h2>
A node without cluster ID may be placed on any page, i.e., even though this option is enabled, a page may contain a node
with and one without a cluster ID.
<h2>Note</h2>
The cluster IDs are only considered for {@link NodeType#NORMAL normal nodes} and the associated {@link NodeType#PROXY proxies}. {@link NodeType#CONNECTOR Connector} and {@link NodeType#PROXY_REFERENCE proxy reference} nodes may
be placed on any page, otherwise information about the relationships would get lost.
@see MultiPageLayout#NODE_CLUSTER_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23strictClusterSeparation Online Documentation}*/
strictClusterSeparation: boolean;
/**
Gets or sets whether or not the given {@link MultiPageLayout#labeling labeling algorithm} places the labels of the input graph.
If this option is disabled, the labels are either ignored or placed by the integrated labeling approach of the
{@link LayoutStageBase#coreLayout core layout algorithm}.
@see MultiPageLayout#labeling
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23labelingEnabled Online Documentation}*/
labelingEnabled: boolean;
/**
Gets or sets the {@link IElementFactory element factory} for creating special nodes and edges in a multi-page layout.
If no element factory is set explicitly (see method {@link MultiPageLayout#elementFactory}), a new instance is created and set by method {@link MultiPageLayout#createElementFactory}
during the first layout run.
@see MultiPageLayout#createElementFactory
@see IElementFactory
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23elementFactory Online Documentation}*/
elementFactory: IElementFactory | null;
/**
Gets or sets the {@link ILayoutCallback callback} that is notified upon completion of multi-page layout calculation runs.
@see ILayoutCallback
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23layoutCallback Online Documentation}*/
layoutCallback: ILayoutCallback | null;
/**
Gets or sets the number of additional tree parent {@link NodeType#PROXY proxies} that the algorithm tries to place on a page.
If the input graph is a tree and the {@link LayoutStageBase#coreLayout core layout algorithm} is an instance of {@link TreeLayout} or
{@link ClassicTreeLayout}, the algorithm may place additional parent proxies of the subtree on a page. The number of
these additional parent proxies is limited by this value. Note that these additional nodes are {@link NodeType#PROXY proxies} and, thus, copies
of already existing nodes. The reason why it may be appropriate to add additional proxies is, that you can better
identify additional parents of the tree structure. Furthermore, if the additional count is larger than 0, the algorithm
doesn't insert any {@link NodeType#CONNECTOR connector nodes} but uses {@link NodeType#PROXY proxy nodes} instead.
<b>Warning:</b> This option is only considered if the input graph is a tree and the {@link LayoutStageBase#coreLayout core layout algorithm} is an instance of {@link TreeLayout}
or {@link ClassicTreeLayout}.
@throws {Exception} <b>(ArgumentError)</b> if the given additional parent count is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23additionalParentCount Online Documentation}*/
additionalParentCount: number;
/**
Gets or sets the bit mask for defining edge bundles.
All multi-edges (edges with same endpoints) belonging to the same edge bundle are split by the same connector pair (see
{@link NodeType#CONNECTOR}).
@see NodeType#CONNECTOR
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23edgeBundleModeMask Online Documentation}*/
edgeBundleModeMask: EdgeBundleModes;
/**
Gets or sets how to handle special nodes (like connector and proxy nodes) with respect to groups.
<b>Warning:</b> While the {@link MultiPageLayout} is able to handle groups (see
{@link MultiPageLayout#groupingMode}), it doesn't support edges incident to group nodes.
@throws {Exception} <b>(ArgumentError)</b> if the specified group mode is unknown
@see NodeType#CONNECTOR
@see NodeType#PROXY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23groupingMode Online Documentation}*/
groupingMode: GroupingMode;
/**
Gets or sets the preferred time limit (in milliseconds) for the layout algorithm.
The specified value has to be non-negative.
<h2>Note</h2>
Restricting the maximum duration may result in a worse layout quality. Furthermore, the actual runtime may exceed the
maximum duration since the layout algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the preferred time limit is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the {@link ILayoutStage} that places the labels of the input graph.
<h2>Note</h2>
This {@link ILayoutStage} needs to be {@link MultiPageLayout#labelingEnabled activated} in order to take effect.
@see MultiPageLayout#labelingEnabled
@see GenericLabeling
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23labeling Online Documentation}*/
labeling: ILayoutStage | null;
/**
Gets or sets whether or not different connected components may be placed on a single page.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23multipleComponentsOnSinglePage Online Documentation}*/
multipleComponentsOnSinglePage: boolean;
/**
Gets or sets the maximum size of a single page.
The layout algorithm subdivides the input graph such that each part is placed on a different page that fits the
specified maximum size.
Both the specified width and height have to be positive.
<h2>Note</h2>
A large page size may increase runtime significantly. To limit the runtime use method {@link MultiPageLayout#maximumDuration}.
@throws {Exception} <b>(ArgumentError)</b> if the specified width or height is not positive
@see MultiPageLayout#maximumDuration
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayout%23maximumPageSize Online Documentation}*/
maximumPageSize: YDimension | null;
static $class: Class;
private $_MultiPageLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MultiPageLayout;
}
export interface MultiPageLayoutResult extends YObject, IElementInfoManager {
/**
@inheritdoc*/
getEdgeInfo(edge: Edge | null): IEdgeInfo | null;
/**
@inheritdoc*/
getEdgeLabelInfo(edgeLabelLayout: IEdgeLabelLayout | null): IEdgeLabelInfo | null;
/**
@inheritdoc*/
getNodeInfo(node: YNode | null): INodeInfo | null;
/**
@inheritdoc*/
getNodeLabelInfo(nodeLabelLayout: INodeLabelLayout | null): INodeLabelInfo | null;
} /**
This class represents the result of a layout run of {@link MultiPageLayout}.
A layout run subdivides the input graph into several {@link LayoutGraph}s (called page graphs) such that the
layout of each graph fits the {@link MultiPageLayout#maximumPageSize specified maximum page size}. This class provides access to the different page graphs (see
method {@link MultiPageLayoutResult#getPage}).
An instance of this class can be either obtained by calling {@link MultiPageLayout#calculateLayout} or
{@link MultiPageLayout#applyLayout}. In the latter case, a {@link ILayoutCallback} has to be {@link MultiPageLayout#layoutCallback set}
before starting the layout run.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutResult Online Documentation}*/
export class MultiPageLayoutResult {
/**
Returns the page graph associated with the given page number.
A <i>page graph</i> is a subgraph of the input graph whose layout fits the {@link MultiPageLayout#maximumPageSize specified maximum page size}.
<h2>Note</h2>
A page graph contains special nodes (that are not contained in the input graph) to guarantee that no information is lost
when splitting edges that connect nodes placed on different pages.
<h2>Note</h2>
All nodes of the page graphs are created by the layout algorithm. The input graph remains unchanged. Matching elements
are mapped to the same element ID, see {@link MultiPageLayout#NODE_ID_DP_KEY},
{@link MultiPageLayout#EDGE_ID_DP_KEY}, {@link MultiPageLayout#NODE_LABEL_ID_DP_KEY} and
{@link MultiPageLayout#EDGE_LABEL_ID_DP_KEY}.
@param pageNo the given page number
@returns the page graph associated with the given page number
@throws {Exception} <b>(IndexOutOfRangeError)</b> if <code>pageNo >= pageCount</code> or <code>pageNo &lt; 0</code>
@see MultiPageLayoutResult#pageCount
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutResult%23getPage(number) Online Documentation}*/
getPage(pageNo: number): LayoutGraph | null;
/**
Returns the number of pages available in this {@link MultiPageLayoutResult}.
Each page contains a {@link MultiPageLayoutResult#getPage page graph} whose layout fits the {@link MultiPageLayout#maximumPageSize specified maximum page size}.
@returns the number of available pages
@see MultiPageLayoutResult#getPage
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutResult%23pageCount() Online Documentation}*/
pageCount(): number;
static $class: Class;
private $_MultiPageLayoutResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MultiPageLayoutResult;
}
export interface CircularLayout extends MultiStageLayout {
}
/**
This layout algorithm arranges graphs in a circular fashion.
<b>Layout Style</b>
The nodes are arranged in circles and stars which emphasize group and tree structures inside a graph. Circular layout
algorithms find applications in many areas such as social networking, network management, WWW visualization, eCommerce,
telecommunications.
<b>Concept</b>
The layout algorithm performs three steps when calculating a circular arrangement for a graph:
<ol>
<li>It searches for partitions in the input graph depending on connectivity and on the given
{@link CircularLayout#layoutStyle layout style}. Considering each partition as a node, the resulting graph has a tree-like structure.
<li>The partitions are laid out as circles using the selected {@link CircularLayout#partitionStyle partition style}.
<li>The algorithm delegates the layout calculation for the underlying tree (in which each node corresponds to a partition)
to a specialized layout algorithm accessible by {@link CircularLayout#singleCycleLayout} or {@link CircularLayout#balloonLayout}. The partitions are moved to their final location.
</ol>
<b>Features</b>
The layout algorithm places the nodes in circles that represent a partition. There are several ways to find partitions
in the input graph. Which one is applied is defined using {@link CircularLayout#layoutStyle}.
The nodes in a partition can either lie on or in the interior of a circle. The placement of the nodes affects the
compactness of the layout and can be specified using {@link CircularLayout#partitionStyle}.
Since edges are routed as straight lines, they may overlap with nodes or node labels. To resolve these overlaps, an edge
routing algorithm (e.g. {@link EdgeRouter} or {@link OrganicEdgeRouter}) can be appended.
Edges that belong to the same circle partition can be routed around the exterior of the circle. The {@link CircularLayout#edgeRoutingPolicy edge routing policy}
determines which edges are exterior and routing details of these edges can be configured via
{@link CircularLayout#exteriorEdgeLayoutDescriptor}. Exterior edge routes consist of smooth arcs. Edges that are routed externally can significantly improve the
readability of the circular layout by reducing the edge clutter in the interior of the circle. This holds true
especially for graph with a large number of edges. On the other hand, it significantly increases the amount of required
space and, thus, is not recommended when maximally compact layouts are desired.
This layout algorithm supports {@link CircularLayout#edgeBundling edge bundling}. In order to bundle the edges, the nodes of the graph are clustered in
groups. Edge bundling is supported only if {@link CircularLayout#partitionStyle partition layout style} is set to {@link PartitionStyle#CYCLE} and {@link CircularLayout#layoutStyle layout style}
is other than {@link CircularLayoutStyle#BCC_ISOLATED}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout Online Documentation}*/
export class CircularLayout {
/**
Creates a new {@link CircularLayout} instance with the default settings.
@param [options] The parameters to pass.
@param [options.considerNodeLabels] Whether or not the layout algorithm reserves space for node labels preventing possible overlaps. This option sets the {@link CircularLayout#considerNodeLabels} property on the created object.
@param [options.placeChildrenOnCommonRadius] Whether or not, in the underlying tree, the children of a tree node are placed on a common radius. This option sets the {@link CircularLayout#placeChildrenOnCommonRadius} property on the created object.
@param [options.fromSketchMode] Whether or not to take the coordinates of the input diagram into account when arranging the nodes of the partitions and the partitions themselves. This option sets the {@link CircularLayout#fromSketchMode} property on the created object.
@param [options.maximumDeviationAngle] The maximum deviation angle allowed for an edge. This option sets the {@link CircularLayout#maximumDeviationAngle} property on the created object.
@param [options.layoutStyle] The global layout style for this layout algorithm. This option sets the {@link CircularLayout#layoutStyle} property on the created object.
@param [options.partitionStyle] The style for the arrangement of each partition. This option sets the {@link CircularLayout#partitionStyle} property on the created object.
@param [options.edgeRoutingPolicy] The edge routing policy that determines whether edges are routed internally or externally with respect to a single partition circle. This option sets the {@link CircularLayout#edgeRoutingPolicy} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link CircularLayout#componentLayoutEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link CircularLayout#hideGroupsStageEnabled} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link CircularLayout#orientationLayoutEnabled} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link CircularLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout Online Documentation}*/
constructor(options?: {
considerNodeLabels?: boolean;
placeChildrenOnCommonRadius?: boolean;
fromSketchMode?: boolean;
maximumDeviationAngle?: number;
layoutStyle?: CircularLayoutStyle | CircularLayoutStyleStringValues;
partitionStyle?: PartitionStyle | PartitionStyleStringValues;
edgeRoutingPolicy?: CircularLayoutEdgeRoutingPolicy | CircularLayoutEdgeRoutingPolicyStringValues;
componentLayoutEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
orientationLayoutEnabled?: boolean;
selfLoopRouterEnabled?: boolean;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
labelingEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Calculates a circular layout for the given graph.
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
This method is not side effect free in the sense that the order of edges or nodes in the input graph may change during
the layout process.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Arranges the given graph in a circular fashion.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Data acceptor key for publishing the final circle information
For each node, the ID of the circle on which it was placed will be stored.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the circle information will be dropped.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23CIRCLE_ID_DP_KEY Online Documentation}*/
static CIRCLE_ID_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for defining custom node partitions
The {@link IDataProvider} needs to provide a mapping from {@link YNode}s to unique {@link Object} IDs for each
group of nodes. These groups will form the circles in the layout.
@see CircularLayoutStyle#CUSTOM_GROUPS
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23CUSTOM_GROUPS_DP_KEY Online Documentation}*/
static CUSTOM_GROUPS_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for marking edges that should be routed externally, around the circle instead of inside
The {@link IDataProvider} registered with the input with this key allows to individually select the edges that
are routed around the exterior of the circle when {@link CircularLayoutEdgeRoutingPolicy#MARKED_EXTERIOR} is
specified as {@link CircularLayout#edgeRoutingPolicy edge routing policy}.
<b>Warning:</b> This selection of exterior edges is only considered when using routing policy
{@link CircularLayoutEdgeRoutingPolicy#MARKED_EXTERIOR}.
<b>Warning:</b> If an edge that is {@link EdgeBundleDescriptor#bundled bundled} is selected for exterior routing, the bundling feature is ignored for that edge.
@see CircularLayout#edgeRoutingPolicy
@see CircularLayoutEdgeRoutingPolicy#MARKED_EXTERIOR
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23EXTERIOR_EDGES_DP_KEY Online Documentation}*/
static EXTERIOR_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets whether or not the layout algorithm reserves space for node labels preventing possible overlaps.
Nodes get temporarily enlarged such that they contain their labels.
This might result in layouts that need much space.
<h2>Note</h2>
This option only works correctly if the {@link MultiStageLayout#layoutOrientation layout orientation} is set to {@link LayoutOrientation#TOP_TO_BOTTOM}
(which is the default). Changing the layout orientation for an undirected layout algorithm like the {@link CircularLayout}
doesn't make sense.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets whether or not, in the underlying tree, the children of a tree node are placed on a common radius.
Disabling this feature may produce more compact layout results.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23placeChildrenOnCommonRadius Online Documentation}*/
placeChildrenOnCommonRadius: boolean;
/**
Gets or sets whether or not to take the coordinates of the input diagram into account when arranging the nodes of the
partitions and the partitions themselves.
If enabled:
<ul>
<li>For complex partitions (those consisting of more than one node), the layout algorithm tries to keep peripheral nodes and
maintain their circular order around the center of the disk/circle. Other partitions that connect to this node are moved
accordingly, if possible.
<li>For multiple partitions that connect to the same node, the layout algorithm tries to keep their circular order around
this node. This only works as expected for {@link CircularLayoutStyle#BCC_COMPACT} as layout style, since otherwise the
underlying tree structure is not well defined.
</ul>
<h2>Note</h2>
Switching between different layout styles or partition styles can lead to unexpected results.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23fromSketchMode Online Documentation}*/
fromSketchMode: boolean;
/**
Gets or sets the maximum deviation angle allowed for an edge.
The deviation angle for an edge is the difference between its optimal angle away from the parent cycle and its actual
angle.
<h2>Note</h2>
Using a small maximum deviation angle may significantly increase the length of the edges. The same holds if a small {@link BalloonLayout#preferredChildWedge preferred child wedge}
is set on the used {@link CircularLayout#balloonLayout balloon layout algorithm}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23maximumDeviationAngle Online Documentation}*/
maximumDeviationAngle: number;
/**
Gets or sets the global layout style for this layout algorithm.
@throws {Exception} <b>(ArgumentError)</b> if an unknown layout style is set
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23layoutStyle Online Documentation}*/
layoutStyle: CircularLayoutStyle;
/**
Gets the {@link SingleCycleLayout} instance used for laying out nodes on a single cycle.
Configuring this layout algorithm only affects layouts with only one partition.
@see CircularLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23singleCycleLayout Online Documentation}*/
readonly singleCycleLayout: SingleCycleLayout | null;
/**
Gets the {@link BalloonLayout} instance used for arranging multiple partitions.
Configuring this layout algorithm affects the arrangement of the partitions within the overall layout.
@see CircularLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23balloonLayout Online Documentation}*/
readonly balloonLayout: BalloonLayout | null;
/**
Gets or sets the style for the arrangement of each partition.
@throws {Exception} <b>(ArgumentError)</b> if an unknown style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23partitionStyle Online Documentation}*/
partitionStyle: PartitionStyle;
/**
Gets the descriptor that defines settings for the exterior edges.
Exterior edges are routed around the exterior of a circle partition. Only edges connecting nodes of the same partition
can be exterior. The {@link CircularLayout#edgeRoutingPolicy edge routing policy} determines which edges are exterior (if any).
<b>Warning:</b> The descriptor is completely ignored when using routing policy
{@link CircularLayoutEdgeRoutingPolicy#INTERIOR}, which is the default policy.
@see CircularLayout#edgeRoutingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23exteriorEdgeLayoutDescriptor Online Documentation}*/
readonly exteriorEdgeLayoutDescriptor: ExteriorEdgeLayoutDescriptor | null;
/**
Gets or sets the edge routing policy that determines whether edges are routed internally or externally with respect to a
single partition circle.
There are two types of supported edges, interior and exterior ones.
<ol>
<li>Interior edges are routed as simple straight lines that directly connect the source and target node. This is the default
style for the circular layout.
<li>Exterior edges get an arc-like curved path around the exterior of the circle. Only edges between nodes of the same
partition (i.e. on the same circle) can be exterior edges. These edges may be beneficial in cases with a large number of
edge crossings inside the circle: by moving some edges out, the number of crossings is reduced and the layout may become
more readable. The drawback is the larger space that is required.
</ol>
<h2>Note</h2>
Only exterior edges contain bends to model the arc-like edge path on the exterior of the circle.
@throws {Exception} <b>(ArgumentError)</b> if the given edge routing policy is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23edgeRoutingPolicy Online Documentation}*/
edgeRoutingPolicy: CircularLayoutEdgeRoutingPolicy;
/**
Sets whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated.
<b>Warning:</b> {@link CircularLayout} can only handle single components. Disabling {@link ComponentLayout} will lead to
errors during execution.
@see MultiStageLayout#componentLayoutEnabled
@see MultiStageLayout#componentLayout
@see ComponentLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23componentLayoutEnabled Online Documentation}*/
componentLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for hiding group nodes is activated.
<b>Warning:</b> {@link CircularLayout} cannot handle group nodes. Disabling {@link HideGroupsStage} will lead to errors
during execution.
@see MultiStageLayout#hideGroupsStageEnabled
@see MultiStageLayout#hideGroupsStage
@see HideGroupsStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23hideGroupsStageEnabled Online Documentation}*/
hideGroupsStageEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated.
<h2>Note</h2>
If the current layout orientation is
{@link LayoutOrientation#TOP_TO_BOTTOM}, the orientation of the layout will not be modified.
<b>Warning:</b> Based on the undirected layout style, disabling {@link OrientationLayout} has no significant effect.
@see MultiStageLayout#orientationLayoutEnabled
@see MultiStageLayout#orientationLayout
@see MultiStageLayout#layoutOrientation
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23orientationLayoutEnabled Online Documentation}*/
orientationLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing self-loops is activated.
<b>Warning:</b> {@link CircularLayout} cannot handle self-loops. Disabling {@link SelfLoopRouter} will lead to errors
during execution.
@see MultiStageLayout#selfLoopRouterEnabled
@see MultiStageLayout#selfLoopRouter
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23selfLoopRouterEnabled Online Documentation}*/
selfLoopRouterEnabled: boolean;
/**
Gets the {@link EdgeBundling} instance that defines the settings of the edge bundling feature.
The specified {@link EdgeBundling} defines global bundling properties. Settings for individual edges can be
defined by assigning an {@link EdgeBundleDescriptor} to an edge using a {@link IDataProvider} registered
with key {@link EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}.
<h2>Note</h2>
When using layout style
{@link CircularLayoutStyle#CUSTOM_GROUPS}, edge bundles between different groups might in some cases intersect with
other groups.
<b>Warning:</b> Edge bundling is supported only if {@link CircularLayout#partitionStyle partition layout style} is set to {@link PartitionStyle#CYCLE} and {@link CircularLayout#layoutStyle layout style}
is other than {@link CircularLayoutStyle#BCC_ISOLATED}.
<b>Warning:</b> Edges that are routed in an exterior fashion (see
{@link CircularLayout#edgeRoutingPolicy}) are never bundled, even when they are associated with a bundling descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayout%23edgeBundling Online Documentation}*/
readonly edgeBundling: EdgeBundling | null;
static $class: Class;
private $_CircularLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CircularLayout;
}
export interface ExteriorEdgeLayoutDescriptor extends YObject {
}
/**
This class provides routing settings for edges routed externally by the {@link CircularLayout}.
The offered settings affect only edges routed on the exterior of a circle partition. The {@link CircularLayout#edgeRoutingPolicy edge routing policy} determines
the edges that are exterior. An exterior edge consists of three different path components:
<ol>
<li>Two straight parts, one starting at the source node and one ending at the target node. The angle between the straight
parts of adjacent edges at the same node can be specified with the {@link ExteriorEdgeLayoutDescriptor#preferredAngle preferred angle} property.
<li>The main arc path that runs around the exterior of the circle from source to target. The radii of the arc paths depend
on the specified {@link ExteriorEdgeLayoutDescriptor#circleDistance} and the {@link ExteriorEdgeLayoutDescriptor#preferredCurveLength}. The {@link ExteriorEdgeLayoutDescriptor#edgeToEdgeDistance} determines the distance between arcs of different edges.
<li>Two curve parts that connect the straight part at the source side with the arc and the arc with the straight part at the
target side. Its length is specifiable via {@link ExteriorEdgeLayoutDescriptor#preferredCurveLength}.
</ol>
@see CircularLayout#edgeRoutingPolicy
@see CircularLayoutEdgeRoutingPolicy#AUTOMATIC
@see CircularLayoutEdgeRoutingPolicy#EXTERIOR
@see CircularLayoutEdgeRoutingPolicy#MARKED_EXTERIOR
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorEdgeLayoutDescriptor Online Documentation}*/
export class ExteriorEdgeLayoutDescriptor {
/**
Gets or sets the minimum distance that exterior edges keep to each other.
@throws {Exception} <b>(ArgumentError)</b> if the minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorEdgeLayoutDescriptor%23edgeToEdgeDistance Online Documentation}*/
edgeToEdgeDistance: number;
/**
Gets or sets the preferred angle (in degrees) between adjacent segments of exterior edges at the same node.
When the angle is zero, all exterior edges start/end with a segment of the same slope and, thus, overlap each other. The
segment in this case is the extension of the line connecting the partition circle center with the respective node
center.
An angle greater than zero means that the edge segments are distributed, get different slopes and are not overlapping
each other anymore. The preferred angle may at most be <code>45</code> degrees and must not be negative. Importantly, it can not
always be obeyed. When there are too many edges to distribute at a node or if the preferred angle would lead to overlaps
with other nodes, the actual angle can become smaller.
<b>Warning:</b> It is not guaranteed that this angle is obeyed, as a wedge of at most <code>90</code> degrees is available to distribute the edges
at a node (<code>45</code> degrees for edges running in clockwise direction and <code>45</code> degrees for edges running in counterclockwise
direction). To avoid overlaps with other elements, this maximum wedge angle is often even smaller than <code>90</code> degrees.
@throws {Exception} <b>(ArgumentError)</b> if the given angle is negative or larger than <code>45</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorEdgeLayoutDescriptor%23preferredAngle Online Documentation}*/
preferredAngle: number;
/**
Gets or sets the preferred length of the curve part that connects the first or last segment with the outer arc.
An exterior edge consists of a straight part starting at the source node (or ending at the target node) and a main arc
that runs around the exterior of the circle from source to target. A curve connects these two parts. Its length is
configured by this setting.
When the preferred length is zero, there is no curve but a <code>90</code> degree bend. Increasing the curve length means that the
radius on which the exterior edges are placed around the circle is increased too. This is necessary to insert the longer
curve while still satisfying the specified {@link ExteriorEdgeLayoutDescriptor#circleDistance}.
<h2>Note</h2>
The smoothness of the curve, i.e., the amount of bends inserted to model it, can be adjusted with the
{@link ExteriorEdgeLayoutDescriptor#smoothness smoothness property}.
@throws {Exception} <b>(ArgumentError)</b> if the given preferred length is negative
@see ExteriorEdgeLayoutDescriptor#smoothness
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorEdgeLayoutDescriptor%23preferredCurveLength Online Documentation}*/
preferredCurveLength: number;
/**
Gets or sets the minimum distance between the outer border of the circle partition and the arc path of exterior edges
that run around the circle.
This property determines the distance that innermost arcs of exterior edges must keep to the circle border. The circle
border in this case is the border enclosing all nodes of the circle.
@throws {Exception} <b>(ArgumentError)</b> if the given circle distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorEdgeLayoutDescriptor%23circleDistance Online Documentation}*/
circleDistance: number;
/**
Gets or sets the smoothness of the exterior edges which determines how many bend points are generated to approximate the
curvature of the edge paths.
The smoothness is defined to be a value from the interval
<code>[0,1]</code>, where a higher value means smoother curves, and, thus, more bends.
@throws {Exception} <b>(ArgumentError)</b> if the smoothness is not in the interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExteriorEdgeLayoutDescriptor%23smoothness Online Documentation}*/
smoothness: number;
static $class: Class;
private $_ExteriorEdgeLayoutDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ExteriorEdgeLayoutDescriptor;
}
/**
@see CircularLayout#layoutStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutStyle Online Documentation}*/
export enum CircularLayoutStyle {
/**
The specifier for a layout style where the nodes within a biconnected component of the input graph are treated as
separate partitions.
If a node belongs to more than one biconnected component, it will be assigned to the one that seems to be more
suitable.
@see CircularLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutStyle%23BCC_COMPACT Online Documentation}*/
BCC_COMPACT,
/**
The specifier for a layout style where the nodes within a biconnected component of the input graph are treated as
separate partitions.
If a node belongs to more than one biconnected component, it will form an isolated partition and will be laid out in
between of all its biconnected component cycles.
@see CircularLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutStyle%23BCC_ISOLATED Online Documentation}*/
BCC_ISOLATED,
/**
The specifier for a layout style where the user can specify groups that will form separate partitions.
To define the groups, a {@link IDataProvider} with key {@link CircularLayout#CUSTOM_GROUPS_DP_KEY}
must be registered.
@see CircularLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutStyle%23CUSTOM_GROUPS Online Documentation}*/
CUSTOM_GROUPS,
/**
The specifier for a layout style where all nodes of the input graph will be placed on or in the interior of a single
cycle.
@see CircularLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutStyle%23SINGLE_CYCLE Online Documentation}*/
SINGLE_CYCLE
}
/** Automatic conversion to {@link CircularLayoutStyle}. */
type CircularLayoutStyleStringValues = "bcc-compact" | "bcc-isolated" | "custom-groups" | "single-cycle";
/**
@see CircularLayout#partitionStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionStyle Online Documentation}*/
export enum PartitionStyle {
/**
Specifier for a partition layout style where all nodes of a partition are placed on the boundary of a circle.
<h2>Note</h2>
This style is very fast, but the results may need much space.
@see CircularLayout#partitionStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionStyle%23CYCLE Online Documentation}*/
CYCLE,
/**
Specifier for a partition style with a semi-compact layout for each partition.
This style places nodes in the interior and on the border of a circle and arranges them in an organic manner, leading to
a more compact layout. Only nodes with connections to other partitions are guaranteed to lie on the boundary of the
resulting disk.
<h2>Note</h2>
This style may come with a performance penalty.
@see CircularLayout#partitionStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionStyle%23DISK Online Documentation}*/
DISK,
/**
Specifier for a partition style with a compact layout for each partition.
This style creates an organic circular layout for each partition. Even nodes that connect to different partitions may be
placed in the interior of the resulting disk.
<h2>Note</h2>
This style may come with a performance penalty.
@see CircularLayout#partitionStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionStyle%23ORGANIC Online Documentation}*/
ORGANIC
}
/** Automatic conversion to {@link PartitionStyle}. */
type PartitionStyleStringValues = "cycle" | "disk" | "organic";
/**
@see CircularLayout#edgeRoutingPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutEdgeRoutingPolicy Online Documentation}*/
export enum CircularLayoutEdgeRoutingPolicy {
/**
An edge routing policy specifying that there all edges are routed inside the circle and on its border as simple straight
lines.
This is the default routing policy for the circular layout. No edges are routed on the exterior of the circle.
@see CircularLayout#edgeRoutingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutEdgeRoutingPolicy%23INTERIOR Online Documentation}*/
INTERIOR,
/**
An edge routing policy specifying that the algorithm automatically determines which edges to route internally and which
externally.
Interior edges are simple straight lines, whereas exterior edges get an arc-like, curved path.
The applied algorithm tries to optimize the readability of the final circular layout by extracting edges to the exterior
such that the number of edge crossings is reduced. Furthermore, crossings between exterior edges are avoided.
<h2>Note</h2>
Edges that connect neighboring nodes on the circle are always drawn as a single straight line and they never become a
curved exterior edge.
<b>Warning:</b> If an edge that is {@link EdgeBundleDescriptor#bundled bundled} is selected for exterior routing, the bundling feature is ignored for that edge.
@see CircularLayout#edgeRoutingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutEdgeRoutingPolicy%23AUTOMATIC Online Documentation}*/
AUTOMATIC,
/**
An edge routing policy specifying that all edges are routed around the exterior of the circle, except the ones that
connect two neighboring nodes.
Exterior edges get an arc-like, curved edge path.
@see CircularLayout#edgeRoutingPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutEdgeRoutingPolicy%23EXTERIOR Online Documentation}*/
EXTERIOR,
/**
An exterior edge routing policy specifying that exterior edges are explicitly marked by the user and the remaining edges
are routed on the interior.
Edges marked in the {@link IDataProvider} registered with the graph with key {@link CircularLayout#EXTERIOR_EDGES_DP_KEY}
are routed around the exterior of the circle. All other edges remain internal.
<b>Warning:</b> If an edge that is {@link EdgeBundleDescriptor#bundled bundled} is selected for exterior routing, the bundling feature is ignored for that edge.
@see CircularLayout#edgeRoutingPolicy
@see CircularLayout#EXTERIOR_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutEdgeRoutingPolicy%23MARKED_EXTERIOR Online Documentation}*/
MARKED_EXTERIOR
}
/** Automatic conversion to {@link CircularLayoutEdgeRoutingPolicy}. */
type CircularLayoutEdgeRoutingPolicyStringValues = "interior" | "automatic" | "exterior" | "marked-exterior";
export interface SingleCycleLayout extends MultiStageLayout {
}
/**
This layout algorithm places all nodes of a graph on a single cycle.
<b>Layout Style</b>
All nodes of a graph are placed on one common circle, generating <em>circular layouts</em>. The edges are drawn as simple, straight lines. The
resulting layouts look like a ring. Edges connecting nodes which are <em>not</em> next to each other on the ring are routed <em>inside</em> of the
circle.
Single-cycle circular layouts are suitable for the visualization of networks such as ring networks or smaller social
networks. They are also useful to layout cyclic subgraphs of a larger graph.
<b>Concept</b>
First, the nodes are sorted by applying either a custom or some default
{@link INodeSequencer}. Then, all nodes are placed on a circle with an appropriate radius according to the
calculated order.
<b>Features</b>
The order in which the nodes are placed on the circle is a crucial aspect of this algorithm. By specifying a {@link INodeSequencer}
instance via {@link SingleCycleLayout#nodeSequencer}, the order can be customized.
There are two options to determine the radius of the circle on which nodes are placed:
<ol>
<li>The radius can be determined automatically. A specified {@link SingleCycleLayout#minimumRadius minimum radius} will be obeyed when choosing the radius.
<li>The radius can be chosen manually. To do so, disable the {@link SingleCycleLayout#automaticRadius automatic radius selection} and specify the custom radius via {@link SingleCycleLayout#fixedRadius}.
</ol>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout Online Documentation}*/
export class SingleCycleLayout {
/**
Creates a new {@link SingleCycleLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.initialAngle] The angle that describes the location on the circle of the first node. This option sets the {@link SingleCycleLayout#initialAngle} property on the created object.
@param [options.fromSketchMode] Whether or not to consider the coordinates of the input diagram when arranging the graph. This option sets the {@link SingleCycleLayout#fromSketchMode} property on the created object.
@param [options.minimumNodeDistance] The minimum distance between nodes that are next to each other on the circle. This option sets the {@link SingleCycleLayout#minimumNodeDistance} property on the created object.
@param [options.fixedRadius] A fixed radius for the circle on which the nodes will be placed. This option sets the {@link SingleCycleLayout#fixedRadius} property on the created object.
@param [options.minimumRadius] The minimum radius for the circle on which the nodes are placed. This option sets the {@link SingleCycleLayout#minimumRadius} property on the created object.
@param [options.automaticRadius] Whether or not the radius of the circle should be determined automatically. This option sets the {@link SingleCycleLayout#automaticRadius} property on the created object.
@param [options.nodeSequencer] The custom {@link INodeSequencer} that will be used for determining the order of the nodes for the circle placement. This option sets the {@link SingleCycleLayout#nodeSequencer} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link SingleCycleLayout#orientationLayoutEnabled} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link SingleCycleLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link MultiStageLayout#componentLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout Online Documentation}*/
constructor(options?: {
initialAngle?: number;
fromSketchMode?: boolean;
minimumNodeDistance?: number;
fixedRadius?: number;
minimumRadius?: number;
automaticRadius?: boolean;
nodeSequencer?: INodeSequencer | null;
orientationLayoutEnabled?: boolean;
selfLoopRouterEnabled?: boolean;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
componentLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Arranges the nodes of the given graph such that they all lie on a common circle.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Gets or sets the angle that describes the location on the circle of the first node.
The angle is measured in radians and world coordinates, e.g., <code>0</code> means east, <code>Pi/2</code> means south.
<h2>Note</h2>
This setting is ignored if the {@link SingleCycleLayout} is used by {@link CircularLayout} to arrange
single-cycle partitions. The {@link CircularLayout} will automatically choose the initial angle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23initialAngle Online Documentation}*/
initialAngle: number;
/**
Gets or sets whether or not to consider the coordinates of the input diagram when arranging the graph.
If this feature is enabled, the original circular order of peripheral nodes around an estimated circle center will be
preserved.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23fromSketchMode Online Documentation}*/
fromSketchMode: boolean;
/**
Gets or sets the minimum distance between nodes that are next to each other on the circle.
<b>Warning:</b> The minimum node distance will be ignored if the {@link SingleCycleLayout#automaticRadius automatic radius selection} is <em>disabled</em>. In that case, the distance between
nodes will follow from the specified {@link SingleCycleLayout#fixedRadius fixed radius}.
@throws {Exception} <b>(ArgumentError)</b> if the given distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23minimumNodeDistance Online Documentation}*/
minimumNodeDistance: number;
/**
Gets or sets a fixed radius for the circle on which the nodes will be placed.
<h2>Note</h2>
If the radius is too small, the nodes on the circle may overlap with each other.
<b>Warning:</b> This fixed radius is only considered if the radius is not {@link SingleCycleLayout#automaticRadius automatically selected} by the layout algorithm.
@throws {Exception} <b>(ArgumentError)</b> if the given radius is negative
@see SingleCycleLayout#minimumRadius
@see SingleCycleLayout#automaticRadius
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23fixedRadius Online Documentation}*/
fixedRadius: number;
/**
Gets or sets the minimum radius for the circle on which the nodes are placed.
<b>Warning:</b> This setting is only considered if the {@link SingleCycleLayout#automaticRadius automatic radius selection} is enabled. Otherwise, the chosen {@link SingleCycleLayout#fixedRadius fixed radius} will
be used.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum radius is negative
@see SingleCycleLayout#fixedRadius
@see SingleCycleLayout#automaticRadius
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23minimumRadius Online Documentation}*/
minimumRadius: number;
/**
Gets or sets whether or not the radius of the circle should be determined automatically.
If enabled, the radius will be chosen such that the distance between nodes which are next to each other on the circle is
approximately the same as the {@link SingleCycleLayout#minimumNodeDistance minimum node distance}.
If this feature is disabled, the radius specified via {@link SingleCycleLayout#fixedRadius} will be applied.
@see SingleCycleLayout#minimumRadius
@see SingleCycleLayout#fixedRadius
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23automaticRadius Online Documentation}*/
automaticRadius: boolean;
/**
Gets or sets the custom {@link INodeSequencer} that will be used for determining the order of the nodes for
the circle placement.
<h2>Note</h2>
To restore the default sequencer, simply set the custom sequencer to <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23nodeSequencer Online Documentation}*/
nodeSequencer: INodeSequencer | null;
/**
Gets the radius that was applied to the last layout calculated by this algorithm.
This method is useful if one wants to know the actual radius of a layout calculated by this algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23lastAppliedRadius Online Documentation}*/
readonly lastAppliedRadius: number;
/**
Sets whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated.
<h2>Note</h2>
If the current layout orientation is
{@link LayoutOrientation#TOP_TO_BOTTOM}, the orientation of the layout will not be modified.
<b>Warning:</b> Based on the undirected layout style, disabling {@link OrientationLayout} has no significant effect.
@see MultiStageLayout#orientationLayoutEnabled
@see MultiStageLayout#orientationLayout
@see MultiStageLayout#layoutOrientation
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23orientationLayoutEnabled Online Documentation}*/
orientationLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing self-loops is activated.
<b>Warning:</b> {@link CircularLayout} cannot handle self-loops. Disabling {@link SelfLoopRouter} will lead to errors
during execution.
@see MultiStageLayout#selfLoopRouterEnabled
@see MultiStageLayout#selfLoopRouter
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleCycleLayout%23selfLoopRouterEnabled Online Documentation}*/
selfLoopRouterEnabled: boolean;
static $class: Class;
private $_SingleCycleLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SingleCycleLayout;
}
export interface ClassicOrganicLayout extends MultiStageLayout {
}
/**
A layout algorithm that arranges graphs in an organic fashion, using a force-directed drawing approach.
<b>Layout Style</b>
The organic layout style is characterized by a natural distribution of nodes. It is well suited to exhibit clusters and
symmetric properties of a graph. Nodes are placed in a space-saving manner, close to their adjacent nodes. Distances
between neighbors and edge lengths are highly uniform and edges are drawn as straight-line segments without bends.
Organic diagrams are well suited for visualizing relations in large networks, for example, in bioinformatics, enterprise
networking, social networks visualization, mesh visualization or system management.
<b>Concept</b>
This algorithm uses a force-directed approach to place the nodes of the input graph. The graph is modeled as a physical
system with forces acting on it. Nodes are considered as electrically charged particles with mutually repulsive forces.
Edges are modeled as springs that attract adjacent nodes. A good diagram is obtained from an equilibrium state of the
system, i.e., the nodes are rearranged based on the physical forces until the system reaches a (local) minimum of the
sum of the forces.
<b>Features</b>
The {@link ClassicOrganicLayout#activateTreeBeautifier tree beautifier} feature activates a special layout style for subgraphs with a tree structure. These subgraphs will
be arranged using an algorithm dedicated to tree graphs and incorporated into the main organic diagram.
Several parameters for the internal force-directed drawing algorithm can be modified. For example, the {@link ClassicOrganicLayout#repulsion repulsion}, {@link ClassicOrganicLayout#attraction attraction}
and {@link ClassicOrganicLayout#initialTemperature initial temperature} can be controlled. This allows experts to configure and fine-tune the algorithm with respect to
their needs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout Online Documentation}*/
export class ClassicOrganicLayout {
/**
Creates a new {@link ClassicOrganicLayout} with default settings.
@param [options] The parameters to pass.
@param [options.repulsion] The node repulsion value. This option sets the {@link ClassicOrganicLayout#repulsion} property on the created object.
@param [options.attraction] The edge attraction value. This option sets the {@link ClassicOrganicLayout#attraction} property on the created object.
@param [options.groupBoundsCalculator] The {@link ILayoutGroupBoundsCalculator} instance used for calculating the size of group nodes. This option sets the {@link ClassicOrganicLayout#groupBoundsCalculator} property on the created object.
@param [options.groupNodeCompactness] The compactness of group nodes. This option sets the {@link ClassicOrganicLayout#groupNodeCompactness} property on the created object.
@param [options.groupNodePolicy] The policy for the handling of group nodes. This option sets the {@link ClassicOrganicLayout#groupNodePolicy} property on the created object.
@param [options.initialTemperature] The initial temperature for the force-directed layout process. This option sets the {@link ClassicOrganicLayout#initialTemperature} property on the created object.
@param [options.finalTemperature] The absolute, final temperature value, which will cause the algorithm to stop if the average temperature falls under this value. This option sets the {@link ClassicOrganicLayout#finalTemperature} property on the created object.
@param [options.deterministic] Whether or not the deterministic mode of the layout algorithm is enabled. This option sets the {@link ClassicOrganicLayout#deterministic} property on the created object.
@param [options.activateTreeBeautifier] Whether or not to arrange subtrees of the input graph in a special, optimized tree style. This option sets the {@link ClassicOrganicLayout#activateTreeBeautifier} property on the created object.
@param [options.gravityFactor] The gravity factor whose magnitude determines the strength of the force towards the barycenter of the graph. This option sets the {@link ClassicOrganicLayout#gravityFactor} property on the created object.
@param [options.scope] The scope that determines which nodes are placed by this algorithm. This option sets the {@link ClassicOrganicLayout#scope} property on the created object.
@param [options.initialPlacement] The strategy for the initial placement of nodes. This option sets the {@link ClassicOrganicLayout#initialPlacement} property on the created object.
@param [options.maximumDuration] The maximum duration in milliseconds that this layout algorithm is allowed to run. This option sets the {@link ClassicOrganicLayout#maximumDuration} property on the created object.
@param [options.iterationFactor] The iteration factor which influences the maximum number of optimization iterations the algorithm may perform. This option sets the {@link ClassicOrganicLayout#iterationFactor} property on the created object.
@param [options.preferredEdgeLength] The default preferred edge length. This option sets the {@link ClassicOrganicLayout#preferredEdgeLength} property on the created object.
@param [options.considerNodeSizes] Whether or not to consider node sizes during layout calculation. This option sets the {@link ClassicOrganicLayout#considerNodeSizes} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link ClassicOrganicLayout#orientationLayoutEnabled} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link MultiStageLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link MultiStageLayout#componentLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout Online Documentation}*/
constructor(options?: {
repulsion?: number;
attraction?: number;
groupBoundsCalculator?: ILayoutGroupBoundsCalculator | null;
groupNodeCompactness?: number;
groupNodePolicy?: GroupsPolicy | GroupsPolicyStringValues;
initialTemperature?: number;
finalTemperature?: number;
deterministic?: boolean;
activateTreeBeautifier?: boolean;
gravityFactor?: number;
scope?: OrganicLayoutScope | OrganicLayoutScopeStringValues;
initialPlacement?: InitialPlacement | InitialPlacementStringValues;
maximumDuration?: number;
iterationFactor?: number;
preferredEdgeLength?: number;
considerNodeSizes?: boolean;
orientationLayoutEnabled?: boolean;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
selfLoopRouterEnabled?: boolean;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
componentLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Calculates an organic arrangement of the given graph.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Frees internal resources which were in use during a previous algorithm run.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23dispose() Online Documentation}*/
dispose(): void;
/**
Data provider key for marking the nodes that are part of the current scope
<b>Warning:</b> The specified subset is only considered if the {@link ClassicOrganicLayout#scope scope} of the algorithm is set to {@link OrganicLayoutScope#SUBSET} or
{@link OrganicLayoutScope#MAINLY_SUBSET}.
@see ClassicOrganicLayout#scope
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23AFFECTED_NODES_DP_KEY Online Documentation}*/
static AFFECTED_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for marking group nodes as fixed
The content of a fixed group node will not be changed by this algorithm but be treated as fixed.
The groups marked as fixed will be considered for the {@link ClassicOrganicLayout#groupNodePolicy group node policy} settings {@link GroupsPolicy#FIXED} and
{@link GroupsPolicy#LAYOUT}. When {@link GroupsPolicy#IGNORE} is chosen, the data provider registered with
this key will be ignored.
@see ClassicOrganicLayout#groupNodePolicy
@see GroupsPolicy#FIXED
@see GroupsPolicy#LAYOUT
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23FIXED_GROUP_NODES_DP_KEY Online Documentation}*/
static FIXED_GROUP_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for defining an individual preferred length for each edge
<h2>Note</h2>
The {@link ClassicOrganicLayout#preferredEdgeLength default preferred length} will be used for edges that have no individual length defined. For those that have, the
default preferred length will be ignored.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23PREFERRED_EDGE_LENGTH_DP_KEY Online Documentation}*/
static PREFERRED_EDGE_LENGTH_DP_KEY: EdgeDpKey<number> | null;
/**
Gets or sets the node repulsion value.
The repulsion influences the forces between nodes. A higher repulsion value can lead to more compact drawings, whereas a
lower repulsion value potentially enlarges drawings.
The repulsion is defined to range from <code>0</code> to <code>2</code>.
@throws {Exception} <b>(ArgumentError)</b> if the repulsion value is negative or greater than <code>2</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23repulsion Online Documentation}*/
repulsion: number;
/**
Gets or sets the edge attraction value.
The attraction influences the attractive force associated with edges. A higher attraction value can lead to more compact
drawings, whereas a lower value potentially enlarges drawings.
The attraction is defined to range from <code>0</code> to <code>2</code>.
@throws {Exception} <b>(ArgumentError)</b> if the attraction value is negative or greater than <code>2</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23attraction Online Documentation}*/
attraction: number;
/**
Gets or sets the {@link ILayoutGroupBoundsCalculator} instance used for calculating the size of group nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23groupBoundsCalculator Online Documentation}*/
groupBoundsCalculator: ILayoutGroupBoundsCalculator | null;
/**
Gets or sets the compactness of group nodes.
The compactness ranges from <code>0</code> to <code>1</code> where <code>0</code> results in group nodes not affecting the overall layout too much while <code>1</code>
forces nodes in the same group to be clustered tightly.
The compactness values needs to lie within <code>[0,1]</code>.
@throws {Exception} <b>(ArgumentError)</b> if compactness value does not lie within the interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23groupNodeCompactness Online Documentation}*/
groupNodeCompactness: number;
/**
Gets or sets the policy for the handling of group nodes.
The group policy defines whether this algorithm {@link GroupsPolicy#IGNORE ignores} groups, leaves them {@link GroupsPolicy#FIXED fixed} or actively {@link GroupsPolicy#LAYOUT arranges} them.
<b>Warning:</b> The group node policy will have an effect on the layout only if the input graph {@link LayoutGroupingSupport#isGrouped is grouped}.
@throws {Exception} <b>(ArgumentError)</b> if the given policy is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23groupNodePolicy Online Documentation}*/
groupNodePolicy: GroupsPolicy;
/**
Gets or sets the initial temperature for the force-directed layout process.
The algorithm starts with an initial temperature. During the iterations of the layout process, this temperature
decreases and if it reaches a certain limit, the process terminates. This limit can be controlled using {@link ClassicOrganicLayout#finalTemperature}.
The initial temperature will be multiplied by the {@link ClassicOrganicLayout#preferredEdgeLength preferred edge length} and then assigned as the actual initial heat of
nodes.
@see ClassicOrganicLayout#finalTemperature
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23initialTemperature Online Documentation}*/
initialTemperature: number;
/**
Gets or sets the absolute, final temperature value, which will cause the algorithm to stop if the average temperature
falls under this value.
The algorithm starts with an initial temperature. During the iterations of the layout process, this temperature
decreases.
@see ClassicOrganicLayout#initialTemperature
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23finalTemperature Online Documentation}*/
finalTemperature: number;
/**
Gets or sets whether or not the deterministic mode of the layout algorithm is enabled.
In deterministic mode, the layout algorithm will yield the same results if the exact same input and same settings are
given as input.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23deterministic Online Documentation}*/
deterministic: boolean;
/**
Gets or sets whether or not to arrange subtrees of the input graph in a special, optimized tree style.
Subtrees will be arranged in a {@link BalloonLayout balloon-like fashion}.
If this feature is disabled, subgraphs with a tree structure are arranged using the same style as for the rest of the
graph.
In case that the input graph is a {@link TreeAlgorithm#isRootedTree tree} and this feature is enabled, the whole graph will be drawn in a tree-like style.
<b>Warning:</b> This feature is only supported for graphs without group structure.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23activateTreeBeautifier Online Documentation}*/
activateTreeBeautifier: boolean;
/**
Gets or sets the gravity factor whose magnitude determines the strength of the force towards the barycenter of the
graph.
<ul>
<li>A high gravity factor value leads to layouts with nodes clustered around the barycenter of the graph.
<li>A low gravity factor value stretches the outskirts of the graph far away from the barycenter.
</ul>
<b>Warning:</b> Although the gravity factor is not restricted to a certain range, it is recommended to only use a range from slightly
negative values up to <code>2.0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23gravityFactor Online Documentation}*/
gravityFactor: number;
/**
Gets or sets the scope that determines which nodes are placed by this algorithm.
<b>Warning:</b> When using scopes that consider only a subset of the nodes, this subset needs to be marked using a {@link IDataProvider}
registered with key {@link ClassicOrganicLayout#AFFECTED_NODES_DP_KEY}. If this {@link IDataProvider}
is missing, all nodes will be arranged.
@throws {Exception} <b>(ArgumentError)</b> if the given scope is unknown
@see ClassicOrganicLayout#AFFECTED_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23scope Online Documentation}*/
scope: OrganicLayoutScope;
/**
Gets or sets the strategy for the initial placement of nodes.
The initial placement has an influence on the final result, because this force-directed algorithm starts with some
positions and then uses forces between nodes to finally aim for a stable result drawing.
@throws {Exception} <b>(ArgumentError)</b> if the given initial placement mode is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23initialPlacement Online Documentation}*/
initialPlacement: InitialPlacement;
/**
Gets or sets the maximum duration in milliseconds that this layout algorithm is allowed to run.
The duration needs to be non-negative.
<h2>Note</h2>
This is a soft limit. When it is reached, the layout algorithm may take some additional time to finish.
@throws {Exception} <b>(ArgumentError)</b> if the specified duration has a negative value
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the iteration factor which influences the maximum number of optimization iterations the algorithm may
perform.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23iterationFactor Online Documentation}*/
iterationFactor: number;
/**
Gets or sets the default preferred edge length.
The default preferred edge length will be used for edges that don't have an individual length preference defined via a {@link IDataProvider}
registered with key {@link ClassicOrganicLayout#PREFERRED_EDGE_LENGTH_DP_KEY}.
The preferred edge length needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified edge length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23preferredEdgeLength Online Documentation}*/
preferredEdgeLength: number;
/**
Gets or sets whether or not to consider node sizes during layout calculation.
If this feature is disabled, overlaps between nodes (e.g. due to large node sizes) may occur.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23considerNodeSizes Online Documentation}*/
considerNodeSizes: boolean;
/**
Sets whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated.
<h2>Note</h2>
If the current layout orientation is
{@link LayoutOrientation#TOP_TO_BOTTOM}, the orientation of the layout will not be modified.
<b>Warning:</b> Based on the undirected layout style, disabling {@link OrientationLayout} has no significant effect.
@see MultiStageLayout#orientationLayoutEnabled
@see MultiStageLayout#orientationLayout
@see MultiStageLayout#layoutOrientation
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicOrganicLayout%23orientationLayoutEnabled Online Documentation}*/
orientationLayoutEnabled: boolean;
static $class: Class;
private $_ClassicOrganicLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClassicOrganicLayout;
}
export interface InteractiveOrganicLayoutExecutionContext {
}
/**
A {@link InteractiveOrganicLayoutExecutionContext} provides control over the layout calculation in the case of
single-threaded algorithm execution.
@see InteractiveOrganicLayout#startLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayoutExecutionContext Online Documentation}*/
export abstract class InteractiveOrganicLayoutExecutionContext {
/**
Starts the layout calculation, continuing where it has stopped before and allowing a certain maximum duration in
milliseconds.
@param duration the duration in milliseconds the layout calculation is allowed to run
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayoutExecutionContext%23continueLayout(number) Online Documentation}*/
abstract continueLayout(duration: number): void;
/**
Stops the layout calculation.
<b>Warning:</b> After calling this method, the context instance cannot continue layout calculation again.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayoutExecutionContext%23stopLayout() Online Documentation}*/
abstract stopLayout(): void;
static $class: Class;
private $_InteractiveOrganicLayoutExecutionContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is InteractiveOrganicLayoutExecutionContext;
}
export interface InteractiveOrganicLayout extends YObject, ILayoutAlgorithm {
}
/**
This algorithm arranges graphs in an organic fashion and offers special functionality to interact with the graph while
the algorithm is running.
Besides the organic graph arrangement, this algorithm enables to immediately visualize changes made to a graph. Changes
can be committed and the layout will be locally updated. That way, live interactions with an adapting graph layout are
possible. Another advantage is that it is not necessary to compute a completely new layout if only small changes were
made.
<b>Layout Style</b>
This algorithm supports the organic layout style which is characterized by a natural distribution of nodes. It is well
suited to exhibit clusters and symmetric properties of a graph. Nodes are placed in a space-saving manner and distances
between neighbors are highly uniform. Edges maintain uniform lengths too and are routed with straight-line segments
without bends.
Organic diagrams are well suited for visualizing relations in large networks, for example, in bioinformatics, enterprise
networking, social networks visualization, mesh visualization or system management.
<b>Concept</b>
The internal basis for computing actual layouts is a force-directed approach placing the nodes of the input graph. The
graph is modeled as a physical system with appropriate forces acting on it. Nodes are considered as electrically charged
particles with mutually repulsive forces. Edges are modeled as forces that attract adjacent nodes. A good diagram is
obtained from an equilibrium state of the system, i.e., the nodes are rearranged based on the physical forces until the
system reaches a (local) minimum of the sum of the forces.
<b>Features</b>
Several update methods allow to indicate <em>lightweight</em> changes on the graph, for example, {@link InteractiveOrganicLayout#setCenter}
for node location updates. These updates will be scheduled and executed at a specific point within the life-cycle of the
layout algorithm. They are ideally suited for usage in interactive scenarios, where users, for example, change node
positions via mouse-dragging.
Method {@link InteractiveOrganicLayout#addStructureUpdate} allows to schedule a custom event handler instance
that is executed in a synchronized context and can safely update the structure of the graph.
<b>Usage Hints</b>
It is easiest to start the algorithm using {@link InteractiveOrganicLayout#startLayout}. Furthermore, it is
strongly recommended to start it by passing a copy of the original graph (i.e. a {@link CopiedLayoutGraph}
instance) as parameter to the mentioned method.
Importantly, <b>changes will not be automatically applied</b> to the input graph. Updates can be scheduled via various methods (e.g.
{@link InteractiveOrganicLayout#setCenter}). The whole layout will then internally be adjusted accordingly and
changes are stored as intermediate results. Such intermediate results can be {@link InteractiveOrganicLayout#commitPositions committed} in order to apply them to the
actual graph.
<h2>Note</h2>
For the mere use case of generating an organic layout of a graph without the need for special interaction, it is
recommended to use {@link OrganicLayout} instead of this class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout Online Documentation}*/
export class InteractiveOrganicLayout {
/**
Creates a new instance of the {@link InteractiveOrganicLayout} with default settings.
@param [options] The parameters to pass.
@param [options.compactnessFactor] The compactness factor for the layout algorithm. This option sets the {@link InteractiveOrganicLayout#compactnessFactor} property on the created object.
@param [options.preferredEdgeLength] The default preferred edge length. This option sets the {@link InteractiveOrganicLayout#preferredEdgeLength} property on the created object.
@param [options.preferredNodeDistance] The preferred distance between nodes. This option sets the {@link InteractiveOrganicLayout#preferredNodeDistance} property on the created object.
@param [options.maximumDuration] The maximum duration in milliseconds that this algorithm is allowed to run. This option sets the {@link InteractiveOrganicLayout#maximumDuration} property on the created object.
@param [options.qualityTimeRatio] The ratio of layout quality versus running time. This option sets the {@link InteractiveOrganicLayout#qualityTimeRatio} property on the created object.
@param [options.outputRestriction] An {@link OutputRestriction} which restricts the area for the layout result of this algorithm. This option sets the {@link InteractiveOrganicLayout#outputRestriction} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout Online Documentation}*/
constructor(options?: {
compactnessFactor?: number;
preferredEdgeLength?: number;
preferredNodeDistance?: number;
maximumDuration?: number;
qualityTimeRatio?: number;
outputRestriction?: OutputRestriction | null;
});
/**
Schedules an update for the structure of the graph, which will automatically be executed at a later point in the
life-cycle of this algorithm.
The given event handler will be queued and executed at a specific time. The event handler can make structural changes
(e.g. removal/creation of edges or nodes). They will be synchronized with the rest of the layout algorithm.
<h2>Note</h2>
This method should only be used for changes to the structure of the graph, because structural changes require the
internal data structures to be rebuilt. Therefore, <em>lightweight</em> changes like modifications of the locations of the nodes should be
made using the corresponding update methods, e.g., {@link InteractiveOrganicLayout#setCenter}.
<b>Warning:</b> Structural updates via this method only work if multi-threading is used.
@param handler The handler delegate that will be invoked using <code>null</code> as the sender and <code>evt</code> as the event arguments
@param evt The event argument that will be piped to the <code>handler</code> invocation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23addStructureUpdate(function(Object%2CEventArgs)%3Avoid%2CEventArgs) Online Documentation}*/
addStructureUpdate(handler: ((sender: any | null, evt: EventArgs | null) => void) | null, evt: EventArgs | null): void;
/**
Calculates an organic layout for the given input graph, however, the layout is <b>not automatically applied</b> to the graph.
Changes have to be {@link InteractiveOrganicLayout#commitPositions committed} to update the graph with the actual calculated positions.
<b>Warning:</b> It is <b>strongly recommended to use a </b> {@link CopiedLayoutGraph} to buffer the original graph.
<b>Warning:</b> This method will not return until {@link InteractiveOrganicLayout#stop} is called. The way to start a
multi-threaded layout calculation is to use {@link InteractiveOrganicLayout#startLayout}.
<h2>Note</h2>
If an instance of {@link CopiedLayoutGraph} is given as input, the setters and getters with a {@link YNode}
or {@link Edge} as parameter may be used with instances from both, the original graph <em>or</em> the copied graph.
@param graph the input graph
@see InteractiveOrganicLayout#startLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Writes calculated intermediate locations of nodes and edges to the actual graph.
Update methods like {@link InteractiveOrganicLayout#setCenter} schedule changes which cause that the whole layout
will internally be adjusted. All adjustments are stored as intermediate results. This method immediately transfers all
these intermediate results to the actual input graph.
<b>Warning:</b> This method only has an effect while the layout algorithm {@link InteractiveOrganicLayout#running is running}.
<b>Warning:</b> Synchronization issues may occur if the algorithm runs on the original graph instead of a copy.
@see InteractiveOrganicLayout#commitPositionsSmoothly
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23commitPositions() Online Documentation}*/
commitPositions(): void;
/**
Writes calculated intermediate locations of nodes and edges to the actual graph and returns the largest movement value.
Update methods like {@link InteractiveOrganicLayout#setCenter} schedule changes which cause that the whole layout
will internally be adjusted. All adjustments are stored as intermediate results. This method smoothly transfers all
these intermediate results to the actual input graph.
Positions are, however, <b>not transferred directly</b> (use {@link InteractiveOrganicLayout#commitPositions} if that is intended). Instead, the
nodes are moved towards the calculated position. The movement is restricted to the given maximum distance.
The movement will be calculated as <em>(movement) = (factor) * (distance between calculated and actual location)</em>
The returned largest movement can be used for estimating the difference between calculated layout and actual positions.
If the return value is <code>0</code>, the calculated layout was completely transferred.
<b>Warning:</b> This method only has an effect while the layout algorithms {@link InteractiveOrganicLayout#running is running}.
<b>Warning:</b> Synchronization issues may occur if the algorithm runs on the original graph instead of a copy.
@param maxMovement the maximum distance a node will be moved
@param factor a factor that determines the node movement
@returns the largest movement value or <code>0</code>, if the calculated layout has been transferred completely
@see InteractiveOrganicLayout#commitPositions
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23commitPositionsSmoothly(number%2Cnumber) Online Documentation}*/
commitPositionsSmoothly(maxMovement: number, factor: number): number;
/**
Disables all predefined {@link ILayoutStage}s so that upon {@link InteractiveOrganicLayout#applyLayout}
only the internal organic layout algorithm will be executed.
This method is called upon construction of this class so that by default additional
{@link ILayoutStage}s are deactivated. This method may be overridden if the additional stages should stay active
(e.g. override the method and return silently).
@see MultiStageLayout#disableAllStages
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23disableAllStages() Online Documentation}*/
disableAllStages(): void;
/**
Polls the current coordinates of the center of the given node.
The returned coordinates do not necessarily correspond to the actual location of the node in the input graph. They may
only be intermediate results stored in the algorithm. This will be the case if scheduled updates were not yet completely
{@link InteractiveOrganicLayout#commitPositions committed} to the actual graph.
<b>Warning:</b> Even if a certain location was {@link InteractiveOrganicLayout#setCenter set} earlier, it will not necessarily be the result of this method. Reasons can be that
changes need to be committed first, or the algorithm is {@link InteractiveOrganicLayout#sleeping sleeping}.
@param node the node for which the center should be polled
@returns the {@link YPoint} representing the center location of the given node, or <code>null</code> if nothing about the node is
known
@see InteractiveOrganicLayout#setCenter
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23getCenter(YNode) Online Documentation}*/
getCenter(node: YNode | null): YPoint | null;
/**
Polls the current x-coordinate of the center location of the given node.
The returned coordinate is not necessarily the actual x-coordinate of the node in the input graph but only an
intermediate result stored in the algorithm. This will be the case if scheduled updates were not yet completely {@link InteractiveOrganicLayout#commitPositions committed}
to the actual graph.
<b>Warning:</b> Even if a certain location was {@link InteractiveOrganicLayout#setCenterX set} earlier, it will not necessarily be the result of this method. Reasons can be that
changes need to be committed first, or the algorithm is {@link InteractiveOrganicLayout#sleeping sleeping}.
@param node the node for which the x-coordinate should be polled
@returns the x-coordinate of the center location of the given node
@see InteractiveOrganicLayout#setCenterX
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23getCenterX(YNode) Online Documentation}*/
getCenterX(node: YNode | null): number;
/**
Polls the current y-coordinate of the center location of the given node.
The returned coordinate is not necessarily the actual y-coordinate of the node in the input graph but only an
intermediate result stored in the algorithm. This will be the case if scheduled updates were not yet completely {@link InteractiveOrganicLayout#commitPositions committed}
to the actual graph.
<b>Warning:</b> Even if a certain location was {@link InteractiveOrganicLayout#setCenterY set} earlier, it will not necessarily be the result of this method. Reasons can be that
changes need to be committed first, or the algorithm is {@link InteractiveOrganicLayout#sleeping sleeping}.
@param node the node for which the y-coordinate should be polled
@returns the y-coordinate of the center location of the given node
@see InteractiveOrganicLayout#setCenterY
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23getCenterY(YNode) Online Documentation}*/
getCenterY(node: YNode | null): number;
/**
Polls the current stress value of a given node.
The stress value indicates how far a node will possibly move. The higher the stress of a node is, the farther it may
move.
The stress value is defined to be a value from the interval <code>[0,1]</code>.
<b>Warning:</b> If a certain stress value for a node was set earlier using {@link InteractiveOrganicLayout#setStress}, this
method will not necessarily return that same value for the same node. The reason is that the algorithm may be {@link InteractiveOrganicLayout#sleeping sleeping}
and did therefore not notice the change yet.
@param node the node for which the stress value should be polled
@returns the stress value of the given node
@see InteractiveOrganicLayout#setStress
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23getStress(YNode) Online Documentation}*/
getStress(node: YNode | null): number;
/**
Schedules an update for the center location of the given node.
This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
committed to the graph while the algorithm is running using methods {@link InteractiveOrganicLayout#commitPositions}
or {@link InteractiveOrganicLayout#commitPositionsSmoothly}.
@param node the node that should be updated
@param x the desired x-coordinate of the given node
@param y the desired y-coordinate of the given node
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23setCenter(YNode%2Cnumber%2Cnumber) Online Documentation}*/
setCenter(node: YNode | null, x: number, y: number): void;
/**
Schedules an update for the center location's x-coordinate of the given node.
This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
committed to the graph while the algorithm is running using methods {@link InteractiveOrganicLayout#commitPositions}
or {@link InteractiveOrganicLayout#commitPositionsSmoothly}.
@param node the node that should be updated
@param x the desired x-coordinate of the given node
@see InteractiveOrganicLayout#setCenterY
@see InteractiveOrganicLayout#setCenter
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23setCenterX(YNode%2Cnumber) Online Documentation}*/
setCenterX(node: YNode | null, x: number): void;
/**
Schedules an update for the center location's y-coordinate of the given node.
This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
committed to the graph while the algorithm is running using methods {@link InteractiveOrganicLayout#commitPositions}
or {@link InteractiveOrganicLayout#commitPositionsSmoothly}.
@param node the node that should be updated
@param y the desired y-coordinate of the given node
@see InteractiveOrganicLayout#setCenterX
@see InteractiveOrganicLayout#setCenter
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23setCenterY(YNode%2Cnumber) Online Documentation}*/
setCenterY(node: YNode | null, y: number): void;
/**
Schedules an update for the inertia of the given node.
The inertia is defined to be a value from the interval <code>[0,1]</code>.
<ul>
<li><code>1.0</code>: The node will not move.
<li><code>0.5</code>: The node will only move half as far as it would with an inertia of <code>0.0</code>.
<li><code>0.0</code>: The node will move as fast as possible.
</ul>
This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
committed to the graph while the algorithm is running using methods {@link InteractiveOrganicLayout#commitPositions}
or {@link InteractiveOrganicLayout#commitPositionsSmoothly}.
@param node the node whose inertia to set
@param inertia an inertia value between <code>0</code> and <code>1</code>
@throws {Exception} <b>(ArgumentError)</b> if the given inertia value is negative or greater than <code>1</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23setInertia(YNode%2Cnumber) Online Documentation}*/
setInertia(node: YNode | null, inertia: number): void;
/**
Schedules an update for the preferred length of the given edge.
This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
committed to the graph while the algorithm is running using methods {@link InteractiveOrganicLayout#commitPositions}
or {@link InteractiveOrganicLayout#commitPositionsSmoothly}.
@param edge the edge whose preferred length should be updated
@param newEdgeLength the new preferred edge length
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23setPreferredEdgeLength(Edge%2Cnumber) Online Documentation}*/
setPreferredEdgeLength(edge: Edge | null, newEdgeLength: number): void;
/**
Schedules an update for the radius of the given node.
This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
committed to the graph while the algorithm is running using methods {@link InteractiveOrganicLayout#commitPositions}
or {@link InteractiveOrganicLayout#commitPositionsSmoothly}.
@param node the node whose radius should be updated
@param radius the desired radius for the given node
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23setRadius(YNode%2Cnumber) Online Documentation}*/
setRadius(node: YNode | null, radius: number): void;
/**
Schedules an update for the stress value of the given node.
The stress value indicates how far a node will possibly move. The higher the stress of a node is, the farther it may
move.
This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
committed to the graph while the algorithm is running using methods {@link InteractiveOrganicLayout#commitPositions}
or {@link InteractiveOrganicLayout#commitPositionsSmoothly}.
The stress value is defined to be a value from the interval <code>[0,1]</code>.
@param node the node whose stress value should be updated
@param stress a stress value from the interval <code>[0,1]</code>
@throws {Exception} <b>(ArgumentError)</b> if the given stress value is negative or greater than <code>1</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23setStress(YNode%2Cnumber) Online Documentation}*/
setStress(node: YNode | null, stress: number): void;
/**
Creates a {@link InteractiveOrganicLayoutExecutionContext context object} that provides methods to continue and {@link InteractiveOrganicLayoutExecutionContext#stopLayout stop} the layout calculation for running this layout
algorithm in a single-threaded environment.
<em>Usage:</em> Call doLayout(long) on the created instance to run the actual layout calculation for some specified period of time,
whenever the layout should be recalculated. To actually transfer the changes, {@link InteractiveOrganicLayout#commitPositions}
should be called subsequently.
<h2>Note</h2>
Invoking this method will not compute any actual layout. A call to StartLayout needs to be made to do so.
@param graph the input graph
@returns a {@link InteractiveOrganicLayoutExecutionContext context instance} to control layout calculation
@see InteractiveOrganicLayoutExecutionContext
@see InteractiveOrganicLayout#applyLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23startLayout(LayoutGraph) Online Documentation}*/
startLayout(graph: LayoutGraph | null): InteractiveOrganicLayoutExecutionContext | null;
/**
Stops the layout algorithm.
In contrast to
{@link InteractiveOrganicLayout#stopAndWait}, the algorithm terminates immediately and will not wait until the
ongoing layout calculation is finished.
<b>Warning:</b> After calling this method, the algorithm cannot be awakened again via {@link InteractiveOrganicLayout#wakeUp}.
@see InteractiveOrganicLayout#stopped
@see InteractiveOrganicLayout#stopAndWait
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23stop() Online Documentation}*/
stop(): void;
/**
Stops a previously {@link InteractiveOrganicLayout#startLayout started} algorithm and then blocks until the current layout calculation is completed.
@see InteractiveOrganicLayout#stopped
@see InteractiveOrganicLayout#stop
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23stopAndWait() Online Documentation}*/
stopAndWait(): void;
/**
Synchronizes the structure of the graph copy with the original graph.
<b>Warning:</b> This method may be used only if the layout algorithm was previously started with a {@link CopiedLayoutGraph}
instance as parameter to methods {@link InteractiveOrganicLayout#startLayout}, {@link InteractiveOrganicLayout#applyLayout}
or StartLayoutSingleThreaded.
<h2>Note</h2>
A call to this method will have no effect if the algorithm is currently not {@link InteractiveOrganicLayout#running running}.
@throws {Exception} <b>(InvalidOperationError)</b> if the currently handled graph is not of type {@link CopiedLayoutGraph}
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23syncStructure() Online Documentation}*/
syncStructure(): void;
/**
Wakes up the algorithm with the effect that it will restart/continue the layout calculation.
This method is useful if the layouter is {@link InteractiveOrganicLayout#sleeping sleeping} but should be notified of changes (e.g. due to user interaction).
<h2>Note</h2>
If the algorithm already terminated (i.e. {@link InteractiveOrganicLayout#stopped stopped}), it can not be awakened again.
@see InteractiveOrganicLayout#sleeping
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23wakeUp() Online Documentation}*/
wakeUp(): void;
/**
Gets or sets the compactness factor for the layout algorithm.
Smaller values result in less compact drawings, greater values result in more compact drawings.
The compactness value needs to lie in <code>[0,1]</code>.
<h2>Note</h2>
If the graph consists of multiple components, using values less than <code>0.4</code> may lead to components drifting away from each
other.
@throws {Exception} <b>(ArgumentError)</b> if the specified value is outside the interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23compactnessFactor Online Documentation}*/
compactnessFactor: number;
/**
Gets the time when the last wake-up, that is, call to {@link InteractiveOrganicLayout#wakeUp}, occurred.
The time is defined in terms of the difference between the current time and midnight, January 1, 1970 UTC, measured in
milliseconds.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23lastWakeupTime Online Documentation}*/
readonly lastWakeupTime: number;
/**
Gets whether or not this layout algorithm has stopped.
If the algorithm has stopped, it terminated all its layout calculations. It is not {@link InteractiveOrganicLayout#running running} anymore and can not be
restarted/continued by calling {@link InteractiveOrganicLayout#wakeUp}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23stopped Online Documentation}*/
readonly stopped: boolean;
/**
Gets whether or not this layout algorithm is currently sleeping.
Sleeping indicates that the algorithm has not {@link InteractiveOrganicLayout#stopped stopped} yet but is waiting (i.e. doing nothing). It can be notified to
continue its work using {@link InteractiveOrganicLayout#wakeUp}.
@see InteractiveOrganicLayout#wakeUp
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23sleeping Online Documentation}*/
readonly sleeping: boolean;
/**
Gets whether or not this layout algorithm is currently running.
The algorithm is running if the layout process is still active, the algorithm has not {@link InteractiveOrganicLayout#stopped stopped} yet and is not {@link InteractiveOrganicLayout#sleeping sleeping}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23running Online Documentation}*/
readonly running: boolean;
/**
Gets or sets the default preferred edge length.
This length does not define the actual absolute length of edges, but the layout algorithm considers the specified
preference where possible.
The preferred edge length needs to be non-negative.
<b>Warning:</b> Changes to this property are not considered while the layout process is still active. The preferred edge length
specified here will only be considered when first starting the layout algorithm. Use {@link InteractiveOrganicLayout#setPreferredEdgeLength}
instead to schedule updates.
@throws {Exception} <b>(ArgumentError)</b> if the specified edge length is negative
@see InteractiveOrganicLayout#setPreferredEdgeLength
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23preferredEdgeLength Online Documentation}*/
preferredEdgeLength: number;
/**
Gets or sets the preferred distance between nodes.
The minimum node distance needs to be non-negative.
<b>Warning:</b>
It is not encouraged to change the node distance while the algorithm is running. Unlike other update methods, no update
scheduling is implemented for this setting, so that synchronization problems may occur.
@throws {Exception} <b>(ArgumentError)</b> if the specified minimum node distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23preferredNodeDistance Online Documentation}*/
preferredNodeDistance: number;
/**
Gets or sets the maximum duration in milliseconds that this algorithm is allowed to run.
The duration needs to be non-negative.
<h2>Note</h2>
If the maximum duration is exceeded, the algorithm will start to {@link InteractiveOrganicLayout#sleeping sleep}.
<h2>Note</h2>
A value of <code>0</code> will be interpreted as <code>0x7FFFFFFF</code> which roughly means that no time limit is specified and the algorithm may
run as long as necessary.
@throws {Exception} <b>(ArgumentError)</b> if the specified maximum duration has a negative value
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the ratio of layout quality versus running time.
The larger the ratio, the better the quality of the resulting layout but the longer it may take to perform the layout
calculation.
The value needs to lie within <code>[0,1]</code>.
@throws {Exception} <b>(ArgumentError)</b> if the specified ratio is outside the interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23qualityTimeRatio Online Documentation}*/
qualityTimeRatio: number;
/**
Gets or sets an {@link OutputRestriction} which restricts the area for the layout result of this algorithm.
@see OutputRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/InteractiveOrganicLayout%23outputRestriction Online Documentation}*/
outputRestriction: OutputRestriction | null;
static $class: Class;
private $_InteractiveOrganicLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is InteractiveOrganicLayout;
}
export interface OrganicLayout extends MultiStageLayout {
}
/**
This layout algorithm arranges graphs in an organic fashion.
<b>Layout Style</b>
The organic layout style is characterized by a natural distribution of nodes that exhibits clusters and symmetric
properties of the graph. Nodes are placed space-saving, close to their adjacent nodes. Edges maintain uniform lengths
and are routed with straight-line segments without bends.
Organic diagrams are commonly used for visualizing relations in large networks for example in bioinformatics, enterprise
networking, visualizing social networks, mesh visualization or system management.
<b>Concept</b>
{@link OrganicLayout} uses a force-directed approach to place the nodes of the input graph. According to this
approach, the graph is modeled as a physical system with appropriate forces acting on it. Nodes are considered as
electrically charged particles with mutually repulsive forces. Edges are modeled as springs that attract adjacent nodes.
A good diagram is obtained from an equilibrium state of the system, i.e., the nodes are rearranged based on the physical
forces until the system reaches a (local) minimum of the sum of the forces.
<b>Features</b>
The ratio between the layout quality and the running time is conveniently adjustable.
This layout algorithm is able to consider a {@link PartitionGrid} structure. However, for common nodes (i.e.
non-group nodes) it only considers single partition cells. Furthermore, the layout algorithm will throw a {@link Exception InvalidGraphStructureError}
if there is a partition grid and the descendants of a group node are assigned to different partition grid cells or if
there are group nodes that are associated with a group node mode other than {@link GroupNodeMode#NORMAL}.
Although overlaps between edges and nodes are avoided, this layout algorithm doesn't guarantee that they won't appear in
the resulting layout. To avoid overlaps, it is possible to route the edges afterwards using an edge routing algorithm,
e.g., {@link OrganicEdgeRouter}.
The algorithm is also able to detect certain types of substructures in a graph (i.e., {@link OrganicLayout#chainSubstructureStyle chains}, {@link OrganicLayout#starSubstructureStyle stars}, {@link OrganicLayout#cycleSubstructureStyle cycles} and
{@link OrganicLayout#parallelSubstructureStyle parallel structures}) and arrange these structures with special-purpose layout styles. Using substructure styles ensures
that the corrsponding structures are easily recognized in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout Online Documentation}*/
export class OrganicLayout {
/**
Creates a new {@link OrganicLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.create3DLayout] Whether or not the layout algorithm should create a 3D result. This option sets the {@link OrganicLayout#create3DLayout} property on the created object.
@param [options.groupNodeCompactness] The compactness of group nodes. This option sets the {@link OrganicLayout#groupNodeCompactness} property on the created object.
@param [options.automaticGroupNodeCompaction] Whether or not group nodes are compacted automatically. This option sets the {@link OrganicLayout#automaticGroupNodeCompaction} property on the created object.
@param [options.clusterNodes] Whether or not a clustering algorithm should be applied to the input graph. This option sets the {@link OrganicLayout#clusterNodes} property on the created object.
@param [options.clusteringQuality] The quality measure of the clustering algorithm. This option sets the {@link OrganicLayout#clusteringQuality} property on the created object.
@param [options.considerNodeLabels] Whether or not to reserve space for node labels during layout calculation. This option sets the {@link OrganicLayout#considerNodeLabels} property on the created object.
@param [options.groupBoundsCalculator] The {@link ILayoutGroupBoundsCalculator} instance used for calculating the size of group nodes. This option sets the {@link OrganicLayout#groupBoundsCalculator} property on the created object.
@param [options.smartComponentLayout] Whether or not this instance should configure the {@link ComponentLayout} to respect subsets of nodes. This option sets the {@link OrganicLayout#smartComponentLayout} property on the created object.
@param [options.nodeEdgeOverlapAvoided] Whether or not the layout algorithm tries to avoid node/edge overlaps. This option sets the {@link OrganicLayout#nodeEdgeOverlapAvoided} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link OrganicLayout#orientationLayoutEnabled} property on the created object.
@param [options.qualityTimeRatio] The ratio of layout quality versus running time. This option sets the {@link OrganicLayout#qualityTimeRatio} property on the created object.
@param [options.maximumDuration] The maximum duration in milliseconds that this layout algorithm is allowed to run. This option sets the {@link OrganicLayout#maximumDuration} property on the created object.
@param [options.scope] The scope that determines which nodes are placed by this algorithm. This option sets the {@link OrganicLayout#scope} property on the created object.
@param [options.chainSubstructureStyle] The style specifier for chain substructures. This option sets the {@link OrganicLayout#chainSubstructureStyle} property on the created object.
@param [options.cycleSubstructureStyle] The style specifier for cycle substructures. This option sets the {@link OrganicLayout#cycleSubstructureStyle} property on the created object.
@param [options.parallelSubstructureStyle] The style specifier for parallel substructures. This option sets the {@link OrganicLayout#parallelSubstructureStyle} property on the created object.
@param [options.starSubstructureStyle] The style specifier for star substructures. This option sets the {@link OrganicLayout#starSubstructureStyle} property on the created object.
@param [options.compactnessFactor] The compactness factor for the layout algorithm. This option sets the {@link OrganicLayout#compactnessFactor} property on the created object.
@param [options.preferredEdgeLength] The default preferred edge length. This option sets the {@link OrganicLayout#preferredEdgeLength} property on the created object.
@param [options.preferredMinimumNodeToEdgeDistance] The minimum preferred distance between nodes and edges when node-edge overlaps are not allowed. This option sets the {@link OrganicLayout#preferredMinimumNodeToEdgeDistance} property on the created object.
@param [options.considerNodeSizes] Whether or not to consider node sizes during layout calculation. This option sets the {@link OrganicLayout#considerNodeSizes} property on the created object.
@param [options.deterministic] Whether or not the deterministic mode of this algorithm is enabled. This option sets the {@link OrganicLayout#deterministic} property on the created object.
@param [options.minimumNodeDistance] The minimum node distance that this algorithm should enforce between all pairs of nodes. This option sets the {@link OrganicLayout#minimumNodeDistance} property on the created object.
@param [options.nodeOverlapsAllowed] Whether or not overlaps between nodes are allowed. This option sets the {@link OrganicLayout#nodeOverlapsAllowed} property on the created object.
@param [options.outputRestriction] The area restriction for the result of the layout algorithm. This option sets the {@link OrganicLayout#outputRestriction} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link MultiStageLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link MultiStageLayout#componentLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout Online Documentation}*/
constructor(options?: {
create3DLayout?: boolean;
groupNodeCompactness?: number;
automaticGroupNodeCompaction?: boolean;
clusterNodes?: boolean;
clusteringQuality?: number;
considerNodeLabels?: boolean;
groupBoundsCalculator?: ILayoutGroupBoundsCalculator | null;
smartComponentLayout?: boolean;
nodeEdgeOverlapAvoided?: boolean;
orientationLayoutEnabled?: boolean;
qualityTimeRatio?: number;
maximumDuration?: number;
scope?: OrganicLayoutScope | OrganicLayoutScopeStringValues;
chainSubstructureStyle?: ChainSubstructureStyle | ChainSubstructureStyleStringValues;
cycleSubstructureStyle?: CycleSubstructureStyle | CycleSubstructureStyleStringValues;
parallelSubstructureStyle?: ParallelSubstructureStyle | ParallelSubstructureStyleStringValues;
starSubstructureStyle?: StarSubstructureStyle | StarSubstructureStyleStringValues;
compactnessFactor?: number;
preferredEdgeLength?: number;
preferredMinimumNodeToEdgeDistance?: number;
considerNodeSizes?: boolean;
deterministic?: boolean;
minimumNodeDistance?: number;
nodeOverlapsAllowed?: boolean;
outputRestriction?: OutputRestriction | null;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
selfLoopRouterEnabled?: boolean;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
componentLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Calculates an organic arrangement of the graph.
In contrast to
{@link OrganicLayout#applyLayoutCore}, graph and layouter are prepared for an independent layout run.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Calculates an organic arrangement of the graph.
In contrast to {@link OrganicLayout#applyLayout}, graph and algorithm are not prepared specifically. This method
should only be called directly when using {@link OrganicLayout} as a {@link ILayoutStage}.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Configures the given {@link ComponentLayout} to take fixed nodes in components into account.
Components that contain fixed nodes will not be rearranged.
This method is called by {@link OrganicLayout#applyLayout} before the actual layout is calculated. It may be
overridden in order to manually configure the {@link ComponentLayout}.
<h2>Note</h2>
Everything that is added to the graph or the {@link ComponentLayout} needs to be removed in
{@link OrganicLayout#disposeComponentLayout}.
@param graph the input graph
@param layouter the {@link ComponentLayout} instance to reconfigure
@see OrganicLayout#smartComponentLayout
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23configureComponentLayout(LayoutGraph%2CComponentLayout) Online Documentation}*/
configureComponentLayout(graph: LayoutGraph | null, layouter: ComponentLayout | null): void;
/**
Disposes of the {@link ComponentLayout} instance.
This method is called by {@link OrganicLayout#applyLayout} after the actual layout is calculated. It may be
overridden in order to revert a custom configuration made in {@link OrganicLayout#configureComponentLayout}.
<h2>Note</h2>
Everything that was added to the graph or to the {@link ComponentLayout} in {@link OrganicLayout#configureComponentLayout}
needs to be removed here.
@param graph the input graph
@param layouter the {@link ComponentLayout} to reset
@see OrganicLayout#smartComponentLayout
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23disposeComponentLayout(LayoutGraph%2CComponentLayout) Online Documentation}*/
disposeComponentLayout(graph: LayoutGraph | null, layouter: ComponentLayout | null): void;
/**
Data provider key for specifying the directedness of edges
Generally, the organic layout algorithm doesn't consider the edge direction. Nevertheless, this {@link IDataProvider}
allows the user to specify hints on the directedness of edges. More precisely, a value of <code>1</code> indicates that the edge
should be considered to be directed from source to target, a value of <code>-1</code> that it is directed from target to source, and
a value of <code>0</code> means that it is undirected.
Currently, the specified values are only considered during the detection of special substructures, see {@link OrganicLayout#chainSubstructureStyle}, {@link OrganicLayout#cycleSubstructureStyle}, {@link OrganicLayout#parallelSubstructureStyle} and {@link OrganicLayout#starSubstructureStyle}.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the algorithm assumes that all edges are undirected.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23EDGE_DIRECTEDNESS_DP_KEY Online Documentation}*/
static EDGE_DIRECTEDNESS_DP_KEY: EdgeDpKey<number> | null;
/**
Data acceptor key for publishing the z-coordinate for all non-group nodes in the graph
For each non-group node, the center z-coordinate will be stored in this {@link IDataAcceptor} after the layout
run. This coordinate is only required if property {@link OrganicLayout#create3DLayout} is enabled. Otherwise, the algorithm produces a 2D result.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, the z-coordinate information of the nodes is
dropped.
@see OrganicLayout#create3DLayout
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23Z_COORDINATE_DP_KEY Online Documentation}*/
static Z_COORDINATE_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for specifying the inertia of nodes
The inertia can be specified for each non-group node if the {@link OrganicLayout#scope scope} is set to {@link OrganicLayoutScope#ALL}. It is defined to
be a value from the interval <code>[0,1]</code>.
<ul>
<li><code>1.0</code>: The node will not move.
<li><code>0.5</code>: The node will only move half as far as it would with an inertia of <code>0.0</code>.
<li><code>0.0</code>: The node will move as fast as possible.
</ul>
<b>Warning:</b> The inertia values are only considered for non-group nodes and if the {@link OrganicLayout#scope scope} is set to
{@link OrganicLayoutScope#ALL}. Furthermore, the specified inertia is not considered for nodes of sub-structures.
<h2>Note</h2>
If this {@link IDataProvider} is registered and there is at least one node with inertia value greater than
<code>0.0</code>, the algorithm considers the initial coordinates of the nodes (similar as for {@link OrganicLayoutScope#SUBSET}).
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key and the scope is {@link OrganicLayoutScope#ALL}, the
algorithm assumes that all nodes have inertia <code>0.0</code>.
@see OrganicLayout#NODE_STRESS_DP_KEY
@see OrganicLayout#scope
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23NODE_INERTIA_DP_KEY Online Documentation}*/
static NODE_INERTIA_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for specifying the stress of nodes
The stress value indicates how far a node will possibly move. The higher the stress of a node is, the farther it may
move.
The stress can be specified for each non-group node if the {@link OrganicLayout#scope scope} is set to {@link OrganicLayoutScope#ALL}. It is defined to be
a value from the interval <code>[0,1]</code>.
<b>Warning:</b> The stress values are only considered for non-group nodes and if the {@link OrganicLayout#scope scope} is set to
{@link OrganicLayoutScope#ALL}. Furthermore, the specified stress is not considered for nodes of sub-structures.
<h2>Note</h2>
If this {@link IDataProvider} is registered and there is at least one node with stress value less than <code>1.0</code>,
the algorithm considers the initial coordinates of the nodes (similar as for {@link OrganicLayoutScope#SUBSET}).
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key and the scope is {@link OrganicLayoutScope#ALL}, the
algorithm assumes that all nodes have stress <code>1.0</code>.
@see OrganicLayout#NODE_INERTIA_DP_KEY
@see OrganicLayout#scope
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23NODE_STRESS_DP_KEY Online Documentation}*/
static NODE_STRESS_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for marking the nodes that are part of the relevant subset
<b>Warning:</b> The specified subset is only considered if the scope is set to {@link OrganicLayoutScope#SUBSET} or
{@link OrganicLayoutScope#MAINLY_SUBSET}.
@see OrganicLayout#scope
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23AFFECTED_NODES_DP_KEY Online Documentation}*/
static AFFECTED_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for assigning individual modes for all group nodes
The modes specify how a group's content is handled and if it is resized during layout calculation.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23GROUP_NODE_MODE_DP_KEY Online Documentation}*/
static GROUP_NODE_MODE_DP_KEY: NodeDpKey<GroupNodeMode> | null;
/**
Data provider key for defining an individual preferred length for each edge
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23PREFERRED_EDGE_LENGTH_DP_KEY Online Documentation}*/
static PREFERRED_EDGE_LENGTH_DP_KEY: EdgeDpKey<number> | null;
/**
Gets or sets whether or not the layout algorithm should create a 3D result.
In order to retrieve the z-coordinates, a {@link IDataAcceptor} with key {@link OrganicLayout#Z_COORDINATE_DP_KEY}
must be registered with the input graph.
<b>Warning:</b> For graphs drawn on a plane, disabling this option produces better results.
@see OrganicLayout#Z_COORDINATE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23create3DLayout Online Documentation}*/
create3DLayout: boolean;
/**
Gets or sets the compactness of group nodes.
The compactness ranges from <code>0</code> to <code>1</code> where <code>0</code> results in group nodes not affecting the overall layout too much while <code>1</code>
forces nodes in the same group to be clustered tightly.
The values need to lie in <code>[0,1]</code>.
<b>Warning:</b> The specified value is only considered if automatic group node compaction is disabled and if there is no {@link PartitionGrid}
structure.
@throws {Exception} <b>(ArgumentError)</b> if specified compactness value is outside the interval <code>[0,1]</code>
@see OrganicLayout#automaticGroupNodeCompaction
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23groupNodeCompactness Online Documentation}*/
groupNodeCompactness: number;
/**
Gets or sets whether or not group nodes are compacted automatically.
When enabled, the compactness factor is determined automatically, i.e. it only depends on the general compactness
specified by option {@link OrganicLayout#compactnessFactor}.
<b>Warning:</b> If this option is enabled, the value specified by option {@link OrganicLayout#groupNodeCompactness} is ignored.
@see OrganicLayout#groupNodeCompactness
@see OrganicLayout#compactnessFactor
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23automaticGroupNodeCompaction Online Documentation}*/
automaticGroupNodeCompaction: boolean;
/**
Gets or sets whether or not a clustering algorithm should be applied to the input graph.
When this option is enabled, the following steps are performed during the layout:
<ol>
<li>A clustering algorithm is applied to the input graph.
<li>All nodes of the same cluster are put into a newly created group node.
<li>The layout is applied to the modified graph.
<li>Group nodes denoting clusters (inserted during step <code>2</code>) are removed.
</ol>
The user can also specify customized clusters by defining appropriate groups.
<h2>Note</h2>
The running time of the clustering algorithm does not depend on the specified maximum duration.
@see OrganicLayout#maximumDuration
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23clusterNodes Online Documentation}*/
clusterNodes: boolean;
/**
Gets or sets the quality measure of the clustering algorithm.
The higher the value, the higher the clustering quality.
The value needs to lie in <code>[0,1]</code>.
<b>Warning:</b> The clustering quality only has an effect if automatic clustering is enabled.
<h2>Note</h2>
The expected running time of the clustering algorithm increases with the quality measure. The running time does not
depend on the specified maximum duration.
@throws {Exception} <b>(ArgumentError)</b> if the specified quality measure is outside the interval <code>[0,1]</code>
@see OrganicLayout#clusterNodes
@see OrganicLayout#maximumDuration
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23clusteringQuality Online Documentation}*/
clusteringQuality: number;
/**
Gets or sets whether or not to reserve space for node labels during layout calculation.
<b>Warning:</b> Enabling this setting overrides the value of {@link OrganicLayout#considerNodeSizes}, i.e. node sizes are always considered.
<h2>Note</h2>
Note that this option only works correctly if the {@link MultiStageLayout#layoutOrientation layout orientation} is set to {@link LayoutOrientation#TOP_TO_BOTTOM}
(which is the default). Changing the layout orientation for an undirected layout algorithm like the {@link OrganicLayout}
doesn't make sense.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets the {@link ILayoutGroupBoundsCalculator} instance used for calculating the size of group nodes.
<h2>Note</h2>
The specified {@link ILayoutGroupBoundsCalculator} is not applied to empty group nodes (i.e. group nodes without
children). Such group nodes will not be resized by the algorithm even if that leads to a violation of the minimum group
node size specified by {@link GroupingKeys#MINIMUM_NODE_SIZE_DP_KEY}.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link ILayoutGroupBoundsCalculator} is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23groupBoundsCalculator Online Documentation}*/
groupBoundsCalculator: ILayoutGroupBoundsCalculator | null;
/**
Gets or sets whether or not this instance should configure the {@link ComponentLayout} to respect subsets of
nodes.
When only a subset of nodes is placed by the layout algorithm, the fixed nodes will keep their locations even if they
reside in different components.
<b>Warning:</b> This option is only considered if {@link OrganicLayoutScope#SUBSET} or {@link OrganicLayoutScope#MAINLY_SUBSET} is used.
@see MultiStageLayout#componentLayoutEnabled
@see MultiStageLayout#componentLayout
@see OrganicLayout#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23smartComponentLayout Online Documentation}*/
smartComponentLayout: boolean;
/**
Gets or sets whether or not the layout algorithm tries to avoid node/edge overlaps.
<h2>Note</h2>
The current implementation does not guarantee the non-existence of overlaps but tries to avoid them. Furthermore, the
post-processing step that removes node overlaps ignores node/edge overlaps, i.e., the number of node/edge overlaps may
increase if option {@link OrganicLayout#nodeOverlapsAllowed} is disabled.
@see OrganicLayout#nodeOverlapsAllowed
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23nodeEdgeOverlapAvoided Online Documentation}*/
nodeEdgeOverlapAvoided: boolean;
/**
Sets whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated.
<h2>Note</h2>
If the current layout orientation is
{@link LayoutOrientation#TOP_TO_BOTTOM}, the orientation of the layout will not be modified.
<b>Warning:</b> Based on the undirected layout style, disabling {@link OrientationLayout} has no significant effect.
@see MultiStageLayout#orientationLayoutEnabled
@see MultiStageLayout#orientationLayout
@see MultiStageLayout#layoutOrientation
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23orientationLayoutEnabled Online Documentation}*/
orientationLayoutEnabled: boolean;
/**
Gets or sets the ratio of layout quality versus running time.
The larger the ratio, the better the quality of the resulting layout but the longer it may take to perform the layout
calculation.
The value needs to lie within <code>[0,1]</code>.
@throws {Exception} <b>(ArgumentError)</b> if the specified ratio is outside the interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23qualityTimeRatio Online Documentation}*/
qualityTimeRatio: number;
/**
Gets or sets the maximum duration in milliseconds that this layout algorithm is allowed to run.
The duration needs to be non-negative.
<h2>Note</h2>
This is a soft limit. When it is reached, the layout algorithm may take some additional time to finish.
@throws {Exception} <b>(ArgumentError)</b> if the specified duration has a negative value
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the scope that determines which nodes are placed by this algorithm.
<b>Warning:</b> When using scopes that consider only a subset of the nodes, this subset needs to be marked using a {@link IDataProvider}
registered with key {@link OrganicLayout#AFFECTED_NODES_DP_KEY}. If this {@link IDataProvider} is
missing, all nodes will be arranged.
@throws {Exception} <b>(ArgumentError)</b> if the given scope is unknown
@see OrganicLayout#AFFECTED_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23scope Online Documentation}*/
scope: OrganicLayoutScope;
/**
Gets or sets the style specifier for chain substructures.
A chain is a simple edge path where the degree of the nodes is less than or equal to <code>2</code>.
<h2>Note</h2>
If a {@link IDataProvider} is registered with key
{@link OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY}, all edges of the chain must have the same direction.
<b>Warning:</b> If the algorithm only {@link OrganicLayout#scope considers a subset} of nodes, only these nodes may be contained in a substructure.
@throws {Exception} <b>(ArgumentError)</b> if the given style is unknown
@see OrganicLayout#scope
@see OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23chainSubstructureStyle Online Documentation}*/
chainSubstructureStyle: ChainSubstructureStyle;
/**
Gets or sets the style specifier for cycle substructures.
A cycle is a simple edge path where the first and last node are identical. The algorithm only considers cycles where the
number of edges connecting nodes of the cycle with the remaining nodes is less than or equal to <code>2</code>.
<h2>Note</h2>
If a {@link IDataProvider} is registered with key
{@link OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY}, all edges of the cycle must have the same direction.
<b>Warning:</b> If the algorithm only {@link OrganicLayout#scope considers a subset} of nodes, only these nodes may be contained in a substructure.
@throws {Exception} <b>(ArgumentError)</b> if the given style is unknown
@see OrganicLayout#scope
@see OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23cycleSubstructureStyle Online Documentation}*/
cycleSubstructureStyle: CycleSubstructureStyle;
/**
Gets or sets the style specifier for parallel substructures.
A parallel structure consists of a set of nodes (called the inner nodes) such that all nodes have degree two and are
connected to the same pair of neighbors (called the outer nodes).
<h2>Note</h2>
If a {@link IDataProvider} is registered with key
{@link OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY}, all edges of the parallel structure must have the same
direction.
<b>Warning:</b> If the algorithm only {@link OrganicLayout#scope considers a subset} of nodes, only these nodes may be contained in a substructure.
@throws {Exception} <b>(ArgumentError)</b> if the given style is unknown
@see OrganicLayout#scope
@see OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23parallelSubstructureStyle Online Documentation}*/
parallelSubstructureStyle: ParallelSubstructureStyle;
/**
Gets or sets the style specifier for star substructures.
A star consists of a set of degree one nodes that are all connected to the same node (called the root of the star).
<h2>Note</h2>
If a {@link IDataProvider} is registered with key
{@link OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY}, all edges of the star must have the same direction.
<b>Warning:</b> If the algorithm only {@link OrganicLayout#scope considers a subset} of nodes, only these nodes may be contained in a substructure.
<h2>Note</h2>
If a {@link IDataProvider} is registered with key {@link OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY} and
the style is set to
{@link StarSubstructureStyle#SEPARATED_RADIAL}, all edges of the star must have the same direction. Otherwise,
the edge directedness is ignored.
@throws {Exception} <b>(ArgumentError)</b> if the given style is unknown
@see OrganicLayout#scope
@see OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23starSubstructureStyle Online Documentation}*/
starSubstructureStyle: StarSubstructureStyle;
/**
Gets or sets the compactness factor for the layout algorithm.
Smaller values result in less compact drawings, greater values result in more compact drawings.
The compactness value needs to lie in <code>[0,1]</code>.
@throws {Exception} <b>(ArgumentError)</b> if the specified value is outside the interval <code>[0,1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23compactnessFactor Online Documentation}*/
compactnessFactor: number;
/**
Gets or sets the default preferred edge length.
If there is no specific preferred edge length assigned to an edge, this default preferred edge length is used.
The preferred edge length needs to be non-negative.
<b>Warning:</b> If the option {@link OrganicLayout#nodeOverlapsAllowed} is disabled, the preferred edge length may be violated because then the nodes are moved in a
post-processing step to resolve overlaps.
@throws {Exception} <b>(ArgumentError)</b> if the specified edge length is negative
@see OrganicLayout#PREFERRED_EDGE_LENGTH_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23preferredEdgeLength Online Documentation}*/
preferredEdgeLength: number;
/**
Gets or sets the minimum preferred distance between nodes and edges when node-edge overlaps are not allowed.
The minimum preferred distance needs to be non-negative.
<h2>Note</h2>
The algorithm doesn't guarantee that the specified minimum value is always maintained.
<b>Warning:</b> This value has an effect only if {@link OrganicLayout#nodeEdgeOverlapAvoided node-edge overlaps} are not allowed.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23preferredMinimumNodeToEdgeDistance Online Documentation}*/
preferredMinimumNodeToEdgeDistance: number;
/**
Gets or sets whether or not to consider node sizes during layout calculation.
If this option is enabled, the circumcircles of the nodes are used. If it is disabled, points will be used instead.
<h2>Note</h2>
This option doesn't affect the minimum node distance property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23considerNodeSizes Online Documentation}*/
considerNodeSizes: boolean;
/**
Gets or sets whether or not the deterministic mode of this algorithm is enabled.
In deterministic mode, the layout algorithm will yield the same results if the exact same input and same settings are
given as input.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23deterministic Online Documentation}*/
deterministic: boolean;
/**
Gets or sets the minimum node distance that this algorithm should enforce between all pairs of nodes.
The minimum node distance needs to be non-negative.
<b>Warning:</b> If {@link OrganicLayout#nodeOverlapsAllowed node overlaps} are allowed, the specified minimum node distance will be ignored.
@throws {Exception} <b>(ArgumentError)</b> if the specified minimum node distance is negative
@see OrganicLayout#nodeOverlapsAllowed
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23minimumNodeDistance Online Documentation}*/
minimumNodeDistance: number;
/**
Gets or sets whether or not overlaps between nodes are allowed.
<h2>Note</h2>
When disabled, a post-processing step is added that resolves node overlaps considering the specified
{@link OrganicLayout#minimumNodeDistance minimum node distance}. This procedure may affect other options like {@link OrganicLayout#outputRestriction output restrictions} and {@link OrganicLayout#preferredEdgeLength edge distances}.
<b>Warning:</b>
If only a subset of nodes is considered in the layout algorithm, there might be some overlapping nodes even if this
option is disabled.
@see OrganicLayout#minimumNodeDistance
@see OrganicLayout#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23nodeOverlapsAllowed Online Documentation}*/
nodeOverlapsAllowed: boolean;
/**
Gets or sets the area restriction for the result of the layout algorithm.
<b>Warning:</b> If option {@link OrganicLayout#nodeOverlapsAllowed} is disabled, the restriction area might be violated when node overlaps get resolved in a post-processing
step.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayout%23outputRestriction Online Documentation}*/
outputRestriction: OutputRestriction | null;
static $class: Class;
private $_OrganicLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrganicLayout;
}
export interface OrganicPartitionGridLayoutStage extends LayoutStageBase {
}
/**
This layout stage enables support for handling a graph with a {@link PartitionGrid} structure.
<b>Concept</b>
Before executing the
{@link LayoutStageBase#coreLayout core layout algorithm}, this stage prepares the grid structure of the graph accordingly, e.g., by hiding top-level grid
nodes. Then, the core layout algorithm is applied to the graph. Finally, the grid is restored, configured and positioned
such that it is consistent with the layout computed by the core algorithm.
Information about the {@link PartitionGrid} structure is retrieved from a {@link IDataProvider}
registered with the graph using key {@link PartitionGrid#PARTITION_GRID_DP_KEY}.
<b>Usage</b>
When using {@link OrganicLayout}, this stage will automatically be used, if required. It is not necessary to
append this stage manually to {@link OrganicLayout}.
Appending this stage can be useful to add support for {@link PartitionGrid partition grid} structures to other
{@link ILayoutAlgorithm}s.
The stage can be appended to all algorithms of type {@link MultiStageLayout} by using method
{@link MultiStageLayout#appendStage}.
<b>Warning:</b> Group nodes that span multiple grid cells are not supported by this stage.
@see PartitionGrid
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicPartitionGridLayoutStage Online Documentation}*/
export class OrganicPartitionGridLayoutStage {
/**
Creates a new instance of {@link OrganicPartitionGridLayoutStage} with the given layout algorithm as
{@link LayoutStageBase#coreLayout core algorithm}.
@param coreLayout the core layout algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicPartitionGridLayoutStage%23OrganicPartitionGridLayoutStage(ILayoutAlgorithm) Online Documentation}*/
constructor(coreLayout: ILayoutAlgorithm | null);
/**
Calculates the layout according to the {@link LayoutStageBase#coreLayout core layout algorithm}, while considering an existing {@link PartitionGrid partition grid} structure
defined for the graph.
If there is no {@link PartitionGrid} defined for the graph, only the core layout algorithm will be executed.
@param graph the input graph
@throws {Exception} <b>(InvalidGraphStructureError)</b> if the graph has group nodes whose content spans multiple grid cells or if the associated {@link OrganicLayout#GROUP_NODE_MODE_DP_KEY group node mode data} is equal
to {@link GroupNodeMode#FIX_CONTENTS} or {@link GroupNodeMode#FIX_BOUNDS}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicPartitionGridLayoutStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
static $class: Class;
private $_OrganicPartitionGridLayoutStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrganicPartitionGridLayoutStage;
}
export interface OrganicRemoveOverlapsStage extends LayoutStageBase {
}
/**
This layout stage removes node overlaps while changes to the original layout structure are avoided and a specified
minimum node distance is guaranteed.
<b>Layout Style</b>
Unlike the overlap removal approach implemented by
{@link RemoveOverlapsStage}, this stage tries to keep the original layout structure, i.e., it tries to keep the
geometric distances between the nodes.
The strategy for removing overlaps is similar to the layout strategy used by organic (force-directed) layout algorithms
(e.g.
{@link OrganicLayout}). Therefore, this stage is especially well suited to remove node overlaps in undirected
graph drawings with straight-line edges, as produced by organic layout approaches.
<b>Features</b>
It is possible to define a {@link OrganicRemoveOverlapsStage#minimumNodeDistance minimum distance} between nodes. This feature allows to not only remove overlaps but forces
nodes to keep the specified distance to other nodes.
Nodes can be marked as <em>fixed</em>. Such nodes will not be moved during the overlap removal process. A {@link IDataProvider}
with key {@link OrganicRemoveOverlapsStage#FIXED_NODE_DP_KEY} can be registered to mark the fixed nodes. Note
that inputs with fixed nodes may significantly increase the overall runtime. Hence class {@link ShuffleLayout}
may be the better choice for such inputs.
<h2>Note</h2>
Overlaps between nodes and edges are not removed by this stage.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicRemoveOverlapsStage Online Documentation}*/
export class OrganicRemoveOverlapsStage {
/**
Creates a new instance of {@link OrganicRemoveOverlapsStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicRemoveOverlapsStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new instance of {@link OrganicRemoveOverlapsStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.maximumDuration] The maximum duration in milliseconds that this layout stage is allowed to run. This option sets the {@link OrganicRemoveOverlapsStage#maximumDuration} property on the created object.
@param [options.minimumNodeDistance] The minimum distance to be kept between any two nodes of the graph. This option sets the {@link OrganicRemoveOverlapsStage#minimumNodeDistance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicRemoveOverlapsStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
maximumDuration?: number;
minimumNodeDistance?: number;
});
/**
Removes node overlaps and displaces nodes of the given graph such that they keep the specified {@link OrganicRemoveOverlapsStage#minimumNodeDistance minimum distance} to each
other.
If a {@link LayoutStageBase#coreLayout core layout algorithm} is specified, it will be executed first.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicRemoveOverlapsStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for marking nodes as fixed
A node marked as fixed will not be moved by this algorithm but stay at its current position.
<b>Warning:</b> If two nodes overlap and both nodes are fixed, then this algorithm will not remove the corresponding overlap.
<b>Warning:</b> Inputs with fixed nodes may significantly increase the overall runtime. Hence class {@link ShuffleLayout} may be
the better choice for such inputs.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicRemoveOverlapsStage%23FIXED_NODE_DP_KEY Online Documentation}*/
static FIXED_NODE_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the maximum duration in milliseconds that this layout stage is allowed to run.
This maximum time does not include the time required for the {@link LayoutStageBase#coreLayout core layout algorithm}.
The duration needs to be non-negative.
<h2>Note</h2>
Restricting the maximum duration may result in a worse layout quality, i.e., there could still be some node overlaps.
Furthermore, the actual runtime may exceed the maximum duration since the algorithm still has to find a valid result.
@throws {Exception} <b>(ArgumentError)</b> if the specified duration has a negative value
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicRemoveOverlapsStage%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the minimum distance to be kept between any two nodes of the graph.
Due to this distance not only nodes that overlap will be moved to resolve the overlap, but also nodes which are too
close to each other with respect to this distance will be displaced.
The distance needs to be a non-negative value.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicRemoveOverlapsStage%23minimumNodeDistance Online Documentation}*/
minimumNodeDistance: number;
static $class: Class;
private $_OrganicRemoveOverlapsStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrganicRemoveOverlapsStage;
}
export interface OutputRestriction extends YObject {
}
/**
{@link OutputRestriction}s restrict the output area and the shape of a layout in conjunction with organic layout
algorithms such as {@link OrganicLayout} or {@link InteractiveOrganicLayout}.
In consequence, all nodes need to be placed inside a specific restricted area.
@see OrganicLayout#outputRestriction
@see InteractiveOrganicLayout#outputRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputRestriction Online Documentation}*/
export abstract class OutputRestriction {
/**
Creates an {@link OutputRestriction} that confines the layout result to a rectangular area which roughly complies
with the given <em>aspect ratio</em>.
The aspect ratio is defined as the ratio between the width and the height of a layout.
<ul>
<li>aspect ratio 1: width and height of the layout should be equal
<li>aspect ratio between 0 and 1: the height of a layout should be greater than its width
<li>aspect ratio larger than 1: the width of a layout should be greater than its height
</ul>
The aspect ratio needs to be strictly positive.
@param ratio the preferred aspect ratio of the output
@returns an {@link OutputRestriction} imposing the given aspect ratio on the layout
@throws {Exception} <b>(ArgumentError)</b> if the given aspect ratio is <code>0</code> or negative
@see OrganicLayout#outputRestriction
@see InteractiveOrganicLayout#outputRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputRestriction%23createAspectRatioRestriction(number) Online Documentation}*/
static createAspectRatioRestriction(ratio: number): OutputRestriction | null;
/**
Creates an {@link OutputRestriction} that confines the layout result to a circle.
The actual <em>restriction circle</em> is defined by the given coordinates and radius.
The radius of the restriction circle needs to be greater than zero.
@param x the x-coordinate of the restriction circle's center
@param y the y-coordinate of the restriction circle's center
@param radius the radius of the restriction circle
@returns the {@link OutputRestriction} confining the layout to a circular area
@throws {Exception} <b>(ArgumentError)</b> if the given radius is less than or equal to zero
@see OrganicLayout#outputRestriction
@see InteractiveOrganicLayout#outputRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputRestriction%23createCircularCageRestriction(number%2Cnumber%2Cnumber) Online Documentation}*/
static createCircularCageRestriction(x: number, y: number, radius: number): OutputRestriction | null;
/**
Creates an {@link OutputRestriction} that confines the layout result to an elliptical area.
The <em>restriction ellipse</em> is defined by its bounding box specified by the given coordinates and size values.
The main axes of the ellipse are parallel to the coordinate axes.
The values for height and width of the ellipse's bounding box need to be greater than zero.
@param x the x-coordinate of the bounding box's upper left corner
@param y the y-coordinate of the bounding box's upper left corner
@param w the width of the bounding box
@param h the height of the bounding box
@returns the {@link OutputRestriction} confining the layout to an elliptical area
@throws {Exception} <b>(ArgumentError)</b> if the given width or height is <code>0</code> or negative
@see OrganicLayout#outputRestriction
@see InteractiveOrganicLayout#outputRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputRestriction%23createEllipticalCageRestriction(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static createEllipticalCageRestriction(x: number, y: number, w: number, h: number): OutputRestriction | null;
/**
Creates an {@link OutputRestriction} that confines the layout to a rectangular area.
This <em>restriction rectangle</em> is defined by the given coordinates and size values.
The values for height and width of the rectangle need to be greater than zero.
@param x the x-coordinate of the restriction rectangle's upper-left corner
@param y the y-coordinate of the restriction rectangle's upper-left corner
@param w width of the restriction rectangle
@param h height of the restriction rectangle
@returns the {@link OutputRestriction} confining the layout to a rectangular area
@throws {Exception} <b>(ArgumentError)</b> if the given width or height is less than or equal to zero
@see OrganicLayout#outputRestriction
@see InteractiveOrganicLayout#outputRestriction
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputRestriction%23createRectangularCageRestriction(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
static createRectangularCageRestriction(x: number, y: number, w: number, h: number): OutputRestriction | null;
/**
This {@link OutputRestriction} does not restrict the output by any means.
It is a no-op that can be used when the output area and shape of a layout should be unrestricted.
@see OrganicLayout#outputRestriction
@see InteractiveOrganicLayout#outputRestriction
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OutputRestriction%23NONE Online Documentation}*/
static NONE: OutputRestriction | null;
static $class: Class;
private $_OutputRestriction: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OutputRestriction;
}
export interface RecursiveShuffleLayout extends LayoutStageBase {
}
/**
This algorithm recursively removes node overlaps from a graph with a hierarchic group structure.
Basically, this algorithm combines a {@link RecursiveGroupLayout} with a specific customizable {@link RecursiveShuffleLayout#shuffleLayout shuffle algorithm}
as its
{@link LayoutStageBase#coreLayout core layout algorithm}. The shuffle algorithm will be recursively applied to all groups of the input graph to remove node
overlaps.
Since this algorithm is a {@link ILayoutStage}, a {@link LayoutStageBase#coreLayout core layout algorithm} can be specified. In consequence, this
algorithm will perform its work after the core algorithm was executed.
@see ShuffleLayout
@see RecursiveGroupLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveShuffleLayout Online Documentation}*/
export class RecursiveShuffleLayout {
/**
Creates a new {@link RecursiveShuffleLayout} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveShuffleLayout Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link RecursiveShuffleLayout} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.groupBoundsCalculator] The {@link ILayoutGroupBoundsCalculator} instance that computes the sizes of all group nodes. This option sets the {@link RecursiveShuffleLayout#groupBoundsCalculator} property on the created object.
@param [options.shuffleLayout] The {@link ILayoutAlgorithm} instance that is used for removing node overlaps. This option sets the {@link RecursiveShuffleLayout#shuffleLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveShuffleLayout Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
groupBoundsCalculator?: ILayoutGroupBoundsCalculator | null;
shuffleLayout?: ILayoutAlgorithm | null;
});
/**
Removes all node overlaps in the given graph, after executing the specified {@link LayoutStageBase#coreLayout core layout algorithm}.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveShuffleLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the {@link ILayoutGroupBoundsCalculator} instance that computes the sizes of all group nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveShuffleLayout%23groupBoundsCalculator Online Documentation}*/
groupBoundsCalculator: ILayoutGroupBoundsCalculator | null;
/**
Gets or sets the {@link ILayoutAlgorithm} instance that is used for removing node overlaps.
This instance will be applied recursively to all group nodes of the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveShuffleLayout%23shuffleLayout Online Documentation}*/
shuffleLayout: ILayoutAlgorithm | null;
static $class: Class;
private $_RecursiveShuffleLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RecursiveShuffleLayout;
}
export interface RemoveOverlapsStage extends YObject, ILayoutStage {
}
/**
This {@link ILayoutStage} removes node overlaps and guarantees a certain minimum distance between nodes.
<b>Layout Style</b>
This stage only removes overlaps between nodes of the input graph. To do so, overlapping nodes will be moved apart.
Results often look like a <em>stretched</em> version of the original graph. Before removing the node overlaps, an optional {@link RemoveOverlapsStage#coreLayout core layout algorithm}
is applied.
<b>Features</b>
It is possible to define a {@link RemoveOverlapsStage#minimumNodeDistance minimum distance} between nodes. This feature allows to not only remove overlaps but forces
nodes to keep a specified distance to other nodes.
<h2>Note</h2>
Overlaps between nodes and edges are not removed by this stage.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveOverlapsStage Online Documentation}*/
export class RemoveOverlapsStage {
/**
Creates a new {@link RemoveOverlapsStage} instance with a given {@link RemoveOverlapsStage#minimumNodeDistance minimum node distance}.
The minimum node distance needs to be non-negative.
@param minimumNodeDistance a non-negative minimum distance value
@throws {Exception} <b>(ArgumentError)</b> if the given distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveOverlapsStage%23RemoveOverlapsStage(number) Online Documentation}*/
constructor(minimumNodeDistance: number);
/**
Creates a new {@link RemoveOverlapsStage} instance with a given {@link RemoveOverlapsStage#minimumNodeDistance minimum node distance}.
The minimum node distance needs to be non-negative.
@param options The parameters to pass.
@param options.minimumNodeDistance a non-negative minimum distance value
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage and is applied before removing node overlaps. This option sets the {@link RemoveOverlapsStage#coreLayout} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the given distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveOverlapsStage%23RemoveOverlapsStage(number) Online Documentation}*/
constructor(options: {
minimumNodeDistance: number;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Removes node overlaps and displaces nodes of the given graph such that they keep the specified {@link RemoveOverlapsStage#minimumNodeDistance minimum distance} to each
other.
If a {@link RemoveOverlapsStage#coreLayout core layout algorithm} is specified, it will be executed first.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveOverlapsStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the minimum distance to be kept between any two nodes of the graph.
Due to this distance not only nodes that overlap will be moved to resolve the overlap, but also nodes which are too
close to each other with respect to this distance will be displaced.
The minimum node distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveOverlapsStage%23minimumNodeDistance Online Documentation}*/
minimumNodeDistance: number;
/**
Gets or sets the core layout algorithm that is wrapped by this stage and is applied before removing node overlaps.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RemoveOverlapsStage%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
static $class: Class;
private $_RemoveOverlapsStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RemoveOverlapsStage;
}
export interface SplitEdgeStage extends YObject, ILayoutStage {
}
/**
This layout stage replaces edges by a path of <em>split-edges</em> with a specified length before it calls a {@link SplitEdgeStage#coreLayout core layout algorithm}.
<b>Concept</b>
Each edge segment (or only segments that belong to a particular subset of edges) in the input graph will be split up
into several segments such that all segments have a maximum length defined by the specified
{@link SplitEdgeStage#splitSegmentLength split segment length}. If an edge segment is already shorter than the split segment length, then it will not be split
up.
At each split location, a dummy node (the so-called <em>split-node</em>) will be inserted. Split-nodes inserted for the same original edge
and the actual source and target node of the original edge will be connected by so called
<em>split-edges</em>. In consequence, the original source and target node are not connected by a single edge but by a path consisting of
split-edges. Furthermore, all edges are simple straight lines.
After this replacement step, the {@link SplitEdgeStage#coreLayout core layout algorithm} will be invoked.
When the core layout algorithm has finished, the split-nodes will be removed from the graph and the original edge will
be restored. The coordinates of the bends will be the center-coordinates of the corresponding split-node.
<b>Features</b>
The {@link IDataProvider} registered with key {@link SplitEdgeStage#splitNodesDpKey} can be used by the {@link SplitEdgeStage#coreLayout core layout algorithm} to determine whether
or not a certain node is a split-node that was created by this stage. This information may be useful when providing a
custom core layout algorithm which should handle split-nodes in a different way than normal nodes. If no {@link IDataProvider}
is registered with this key, this information is dropped.
By registering a {@link IDataProvider} with key {@link SplitEdgeStage#affectedEdgesDpKey}, the subset of edges that should be split by this layout
stage can be specified. If no {@link IDataProvider} with this key is registered, all edges of the input graph
will be split by the layout stage.
The amount of split-nodes replacing a single original edge can be controlled via the
{@link SplitEdgeStage#splitSegmentLength split segment length}. The smaller the value, the more split-nodes this stage will insert.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SplitEdgeStage Online Documentation}*/
export class SplitEdgeStage {
/**
Creates a new {@link SplitEdgeStage} with default settings.
@param [options] The parameters to pass.
@param [options.affectedEdgesDpKey] The {@link IDataProvider} key to determine which edges are to be split by this layout stage. This option sets the {@link SplitEdgeStage#affectedEdgesDpKey} property on the created object.
@param [options.splitNodesDpKey] The {@link IDataProvider} key for marking the nodes that were created by splitting edges, i.e., the split-nodes. This option sets the {@link SplitEdgeStage#splitNodesDpKey} property on the created object.
@param [options.splitSegmentLength] The maximum length for a <em>split-edge</em>. This option sets the {@link SplitEdgeStage#splitSegmentLength} property on the created object.
@param [options.splitNodeSize] The size of the split-nodes that are inserted when splitting edges. This option sets the {@link SplitEdgeStage#splitNodeSize} property on the created object.
@param [options.coreLayout] The core layout algorithm. This option sets the {@link SplitEdgeStage#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SplitEdgeStage Online Documentation}*/
constructor(options?: {
affectedEdgesDpKey?: any | null;
splitNodesDpKey?: any | null;
splitSegmentLength?: number;
splitNodeSize?: number;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Replaces the edges of the given graph by a path of split-edges, executes the {@link SplitEdgeStage#coreLayout core layout algorithm} and restores the
original edges, introducing bends at the locations of the split-nodes.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/SplitEdgeStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the {@link IDataProvider} key to determine which edges are to be split by this layout stage.
If no {@link IDataProvider} is registered with this key, all edges will be split.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SplitEdgeStage%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
/**
Gets or sets the {@link IDataProvider} key for marking the nodes that were created by splitting edges, i.e.,
the split-nodes.
The information provided by the {@link IDataProvider} registered with this key is useful for the {@link SplitEdgeStage#coreLayout core layout algorithm}
to know whether or not some node is a (dummy) split-node. After {@link SplitEdgeStage#applyLayout} finishes, the
split-nodes will have been removed and this information will not be available anymore.
<b>Warning:</b> If there is already a {@link IDataProvider} registered with this key, information about marked nodes will be
ignored by the stage during the layout process. However, this information will be restored after the layout has
finished. However, it is recommended to specify a custom key - only used for this purpose - if the core layout algorithm
should be able to clearly distinguish the newly created split-nodes.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SplitEdgeStage%23splitNodesDpKey Online Documentation}*/
splitNodesDpKey: any | null;
/**
Gets or sets the maximum length for a <em>split-edge</em>.
The length of the original edge path divided by the given split segment length yields the number of nodes on the new
path.
The split segment length must be a strictly positive value.
@throws {Exception} <b>(ArgumentError)</b> if the given split segment length is smaller than or equal to zero
@see {@link https://docs.yworks.com/yfileshtml/#/api/SplitEdgeStage%23splitSegmentLength Online Documentation}*/
splitSegmentLength: number;
/**
Gets or sets the size of the split-nodes that are inserted when splitting edges.
The given size will be used for defining the height and the width of the split-nodes.
<b>Warning:</b> Since some layout algorithms cannot handle zero-sized nodes, the size needs to be increased when using this stage with
such an algorithm as {@link SplitEdgeStage#coreLayout core layout algorithm}.
@throws {Exception} <b>(ArgumentError)</b> if the given size is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/SplitEdgeStage%23splitNodeSize Online Documentation}*/
splitNodeSize: number;
/**
Gets or sets the core layout algorithm.
This algorithm is wrapped by this stage. It is invoked in {@link ILayoutAlgorithm#applyLayout}. The {@link ILayoutStage}
may add pre- and post-processing steps before and after calling the core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SplitEdgeStage%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
static $class: Class;
private $_SplitEdgeStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SplitEdgeStage;
}
export interface OrthogonalLayoutEdgeLayoutDescriptor extends YObject {
}
/**
This class provides layout information for the edges of the graph.
{@link OrthogonalLayoutEdgeLayoutDescriptor} is used by {@link OrthogonalLayout} to determine the routing details
for the edges (e.g. the segment length).
An {@link OrthogonalLayoutEdgeLayoutDescriptor} can be specified individually for each edge by using a {@link IDataProvider}
which maps from an {@link Edge} to an {@link OrthogonalLayoutEdgeLayoutDescriptor} instance. The {@link IDataProvider}
is registered with the graph with key {@link OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
@see OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutEdgeLayoutDescriptor Online Documentation}*/
export class OrthogonalLayoutEdgeLayoutDescriptor {
/**
Creates a new instance of {@link OrthogonalLayoutEdgeLayoutDescriptor} with the default settings.
@param [options] The parameters to pass.
@param [options.minimumFirstSegmentLength] The minimum length of the first segment of the edge path (at the source node). This option sets the {@link OrthogonalLayoutEdgeLayoutDescriptor#minimumFirstSegmentLength} property on the created object.
@param [options.minimumLastSegmentLength] The minimum length of the last segment of the edge path (at the target node). This option sets the {@link OrthogonalLayoutEdgeLayoutDescriptor#minimumLastSegmentLength} property on the created object.
@param [options.minimumSegmentLength] The minimum segment length of an edge. This option sets the {@link OrthogonalLayoutEdgeLayoutDescriptor#minimumSegmentLength} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutEdgeLayoutDescriptor Online Documentation}*/
constructor(options?: {
minimumFirstSegmentLength?: number;
minimumLastSegmentLength?: number;
minimumSegmentLength?: number;
});
/**
Creates a copy of this {@link OrthogonalLayoutEdgeLayoutDescriptor} instance.
@returns a copy of this {@link OrthogonalLayoutEdgeLayoutDescriptor} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutEdgeLayoutDescriptor%23createCopy() Online Documentation}*/
createCopy(): OrthogonalLayoutEdgeLayoutDescriptor | null;
/**
Gets or sets the minimum length of the first segment of the edge path (at the source node).
The minimum length must be greater than or equal to <code>0</code>.
<h2>Note</h2>
The final segment length also depends on the used grid size (see
{@link OrthogonalLayout#gridSpacing}). If the grid is larger than the specified minimum length, then the grid size defines the actual minimum.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length of the first segment is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutEdgeLayoutDescriptor%23minimumFirstSegmentLength Online Documentation}*/
minimumFirstSegmentLength: number;
/**
Gets or sets the minimum length of the last segment of the edge path (at the target node).
The minimum length must be greater than or equal to <code>0</code>.
<h2>Note</h2>
The final segment length also depends on the used grid size (see
{@link OrthogonalLayout#gridSpacing}). If the grid is larger than the specified minimum length, then the grid size defines the actual minimum.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length of the last segment is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutEdgeLayoutDescriptor%23minimumLastSegmentLength Online Documentation}*/
minimumLastSegmentLength: number;
/**
Gets or sets the minimum segment length of an edge.
The minimum length must be greater than or equal to <code>0</code>.
<h2>Note</h2>
The final segment length also depends on the used grid size (see
{@link OrthogonalLayout#gridSpacing}). If the grid is larger than the specified minimum length, then the grid size defines the actual minimum.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutEdgeLayoutDescriptor%23minimumSegmentLength Online Documentation}*/
minimumSegmentLength: number;
static $class: Class;
private $_EdgeLayoutDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalLayoutEdgeLayoutDescriptor;
}
/**
@see OrthogonalLayout#treeStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutStyle Online Documentation}*/
export enum TreeLayoutStyle {
/**
A tree layout style that defines that subtrees should not be handled specifically.
Subtrees are <em>not detected</em> and will be arranged without specific emphasis of the tree structure.
@see OrthogonalLayout#treeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutStyle%23NONE Online Documentation}*/
NONE,
/**
A tree layout style that arranges subtrees in a layered tree fashion with grouped edge routes, generated by a {@link DefaultNodePlacer}
with routing style {@link TreeLayoutEdgeRoutingStyle#FORK} and alignment setting {@link RootAlignment#CENTER}.
Child nodes sharing the same parent node are placed next to each other above/below the root (vertical orientation) or
above each other left/right of the root (horizontal orientation).
In contrast to {@link TreeLayoutStyle#INTEGRATED} this style produces more compact layout results. However,
the overall layout may be less homogeneous because the subtrees are separately arranged by a dedicated tree layout
algorithm which does e.g., not consider the specified {@link OrthogonalLayout#gridSpacing grid}.
@see OrthogonalLayout#treeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutStyle%23DEFAULT Online Documentation}*/
DEFAULT,
/**
A tree layout style that arranges subtrees in a layered tree fashion with grouped edge routes and integrates the tree
elements into the orthogonal layout framework.
Child nodes sharing the same parent node are placed next to each other above/below the root (vertical orientation) or
above each other left/right of the root (horizontal orientation).
In contrast to {@link TreeLayoutStyle#DEFAULT}, the subtree elements are handled by the orthogonal layout
framework internally. This makes the final layout more homogeneous, but potentially less compact. Furthermore, the {@link OrthogonalLayout#gridSpacing grid}
will be correctly obeyed also for tree nodes.
@see OrthogonalLayout#treeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutStyle%23INTEGRATED Online Documentation}*/
INTEGRATED,
/**
A tree layout style that aims to create maximally compact layouts for the subtrees by using the {@link CompactNodePlacer}
for the tree layout algorithm.
The algorithm applied to the subtrees tries to combine different placement strategies such that the overall area
required by subtrees is small. This means that the style of different tree structures can vary within the same layout.
@see OrthogonalLayout#treeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutStyle%23COMPACT Online Documentation}*/
COMPACT,
/**
A tree layout style that yields tree layouts with an aspect ratio close to <code>1:1</code> that are generated by
{@link AspectRatioNodePlacer}.
To achieve the aspect ratio, tree child nodes are organized in as much rows/columns as needed. Edges are routed
orthogonally along the rows/columns. The parent node of the children is placed in a corner besides the child nodes
depending on the tree layout orientation, e.g., left above the child nodes for top-to-bottom orientation.
@see OrthogonalLayout#treeStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutStyle%23ASPECT_RATIO Online Documentation}*/
ASPECT_RATIO
}
/** Automatic conversion to {@link TreeLayoutStyle}. */
type TreeLayoutStyleStringValues = "none" | "default" | "integrated" | "compact" | "aspect-ratio";
/**
Orientation specifier that defines that the layout orientation of substructures is top to bottom.
@see OrthogonalLayout#treeOrientation
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureOrientation Online Documentation}*/
export enum SubstructureOrientation {
/**
Orientation specifier that defines that the layout orientation of substructures is top to bottom.
@see OrthogonalLayout#treeOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureOrientation%23TOP_TO_BOTTOM Online Documentation}*/
TOP_TO_BOTTOM,
/**
Orientation specifier that defines that the layout orientation of substructures is bottom to top.
@see OrthogonalLayout#treeOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureOrientation%23BOTTOM_TO_TOP Online Documentation}*/
BOTTOM_TO_TOP,
/**
Orientation specifier that defines that the layout orientation of substructures is left to right.
@see OrthogonalLayout#treeOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureOrientation%23LEFT_TO_RIGHT Online Documentation}*/
LEFT_TO_RIGHT,
/**
Orientation specifier that defines that the layout orientation of substructures is right to left.
@see OrthogonalLayout#treeOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureOrientation%23RIGHT_TO_LEFT Online Documentation}*/
RIGHT_TO_LEFT,
/**
Orientation specifier that defines that the layout orientation of substructures is chosen automatically.
The automatic orientation selection is made e.g., by searching a side at the root node were no edge connects yet and
inserting the substructure at that side using the respective orientation (for example, right side means left-to-right
layout). It is recommended to use this setting if layouts should be maximally compact, because specifying a fixed
orientation reduces the degree of freedom of the algorithm.
@see OrthogonalLayout#treeOrientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureOrientation%23AUTO_DETECT Online Documentation}*/
AUTO_DETECT
}
/** Automatic conversion to {@link SubstructureOrientation}. */
type SubstructureOrientationStringValues = "top-to-bottom" | "bottom-to-top" | "left-to-right" | "right-to-left" | "auto-detect";
/**
@see OrthogonalLayout#chainStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainLayoutStyle Online Documentation}*/
export enum ChainLayoutStyle {
/**
A chain layout style that defines that chains should not be handled specifically.
Chains in the input graph are <em>not detected</em> and will be arranged without specific emphasis of the chain structure.
@see OrthogonalLayout#chainStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainLayoutStyle%23NONE Online Documentation}*/
NONE,
/**
A chain layout style that defines that chains are arranged in a straight fashion.
@see OrthogonalLayout#chainStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainLayoutStyle%23STRAIGHT Online Documentation}*/
STRAIGHT,
/**
A chain layout style that defines that chains are line/column-wrapped such that nodes form the turns/corners of the
chain arrangement.
@see OrthogonalLayout#chainStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainLayoutStyle%23WRAPPED_WITH_NODES_AT_TURNS Online Documentation}*/
WRAPPED_WITH_NODES_AT_TURNS,
/**
A chain layout style that defines that chains are line/column wrapped such that bends of the chain edges form the
turns/corners of the chain arrangement.
@see OrthogonalLayout#chainStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainLayoutStyle%23WRAPPED_WITH_BENDS_AT_TURNS Online Documentation}*/
WRAPPED_WITH_BENDS_AT_TURNS
}
/** Automatic conversion to {@link ChainLayoutStyle}. */
type ChainLayoutStyleStringValues = "none" | "straight" | "wrapped-with-nodes-at-turns" | "wrapped-with-bends-at-turns";
/**
@see OrthogonalLayout#cycleStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleLayoutStyle Online Documentation}*/
export enum CycleLayoutStyle {
/**
A cycle layout style that defines that cycles should not be handled specifically.
Cycles in the input graph are <em>not detected</em> and will be arranged without specific emphasis of the cycle structure.
@see OrthogonalLayout#cycleStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleLayoutStyle%23NONE Online Documentation}*/
NONE,
/**
A cycle layout style that defines that cycle structures are arranged in a circular fashion such that nodes form the
corners of the circular layout.
@see OrthogonalLayout#cycleStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleLayoutStyle%23CIRCULAR_WITH_NODES_AT_CORNERS Online Documentation}*/
CIRCULAR_WITH_NODES_AT_CORNERS,
/**
A cycle layout style that defines that cycle structures are arranged in a circular fashion such that bends of cycle
edges form the corners of the circular layout.
@see OrthogonalLayout#cycleStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleLayoutStyle%23CIRCULAR_WITH_BENDS_AT_CORNERS Online Documentation}*/
CIRCULAR_WITH_BENDS_AT_CORNERS
}
/** Automatic conversion to {@link CycleLayoutStyle}. */
type CycleLayoutStyleStringValues = "none" | "circular-with-nodes-at-corners" | "circular-with-bends-at-corners";
/**
@see OrthogonalLayout#layoutStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutStyle Online Documentation}*/
export enum OrthogonalLayoutStyle {
/**
A layout style that produces orthogonal edges while maintaining the original node size.
@see OrthogonalLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutStyle%23NORMAL Online Documentation}*/
NORMAL,
/**
A layout style that produces orthogonal edges and nodes of uniform size.
Before calculating the layout, all nodes will be assigned equal sizes.
@see OrthogonalLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutStyle%23UNIFORM Online Documentation}*/
UNIFORM,
/**
A layout style that modifies the size of nodes to minimize the overall number of bends.
The layout algorithm will enlarge nodes based on the number and position of their neighbors in order to minimize the
number of required bends. The segments of all edges are either horizontal or vertical.
@see OrthogonalLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutStyle%23BOX Online Documentation}*/
BOX,
/**
A layout style that produces non-orthogonal edges and resizes the nodes to a small uniform size.
This layout style resembles {@link OrthogonalLayoutStyle#BOX}, but it resizes all nodes to equal size.
It introduces additional bends into some edges and routes the last edge segment of these edges non-orthogonally to their
adjacent nodes.
@see OrthogonalLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutStyle%23MIXED Online Documentation}*/
MIXED,
/**
A layout style similar to {@link OrthogonalLayoutStyle#MIXED} that maintains the original node size.
This style introduces additional bends into some edges and routes the last edge segment of these edges non-orthogonally
to their adjacent nodes.
@see OrthogonalLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutStyle%23FIXED_MIXED Online Documentation}*/
FIXED_MIXED,
/**
A layout style similar to {@link OrthogonalLayoutStyle#BOX} that maintains the original node size.
Nodes will be placed at the same positions as with
{@link OrthogonalLayoutStyle#FIXED_MIXED}, but edges will be routed strictly orthogonally.
@see OrthogonalLayout#layoutStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutStyle%23FIXED_BOX Online Documentation}*/
FIXED_BOX
}
/** Automatic conversion to {@link OrthogonalLayoutStyle}. */
type OrthogonalLayoutStyleStringValues = "normal" | "uniform" | "box" | "mixed" | "fixed-mixed" | "fixed-box";
export interface OrthogonalLayout extends MultiStageLayout {
}
/**
This layout algorithm arranges graphs in an orthogonal fashion.
<b>Layout Style</b>
This layout algorithm arranges the nodes of a given graph such that each edge is drawn as an alternating sequence of
horizontal and vertical segments.
It produces compact drawings with no overlapping nodes, few crossings and few bends and is well suited for small and
medium-sized sparse graphs.
Orthogonal drawings are common in engineering applications since they are able to provide clear representations of
complex networks that can also be optimal with respect to diverse objective functions such as bend minimization or area
minimization. Application domains of orthogonal drawings include software engineering, database schema representation,
system management, knowledge representation, VLSI circuits and floor planning applications.
This algorithm also optionally supports {@link OrthogonalLayout#DIRECTED_EDGE_DP_KEY directed edge drawings} (although not in combination with hierarchically nested
graphs or the non-default
{@link OrthogonalLayout#layoutStyle layout styles}). Application domains of directed orthogonal drawings include, for example, software engineering, database
schema and system management.
Also this algorithm supports hierarchically nested graphs (although not in combination with the directed edge drawings,
the {@link OrthogonalLayout#fromSketchMode from sketch} option, or the non-default {@link OrthogonalLayout#layoutStyle layout styles}) feature:
<b>Concept</b>
The orthogonal layout algorithm is based on the topology-shape-metrics approach and runs in three phases:
<ul>
<li><b>Planarization</b> – A planar embedding is computed.
<li><b>Orthogonalization</b> – The bends and the angles are computed.
<li><b>Compaction</b> – The coordinates for the nodes and edges are determined.
</ul>
<b>Features</b>
There exist several different layout styles that the orthogonal layout algorithm supports depending on whether or not
the edges should be routed completely orthogonal, or whether or not the original size of the nodes should be maintained.
Such layout styles are {@link OrthogonalLayoutStyle#NORMAL}, {@link OrthogonalLayoutStyle#UNIFORM},
{@link OrthogonalLayoutStyle#BOX}, {@link OrthogonalLayoutStyle#MIXED}, {@link OrthogonalLayoutStyle#FIXED_BOX}, {@link OrthogonalLayoutStyle#FIXED_MIXED}
and can be applied using method {@link OrthogonalLayout#layoutStyle}.
{@link OrthogonalLayout} is able to consider edge label data when arranging a graph. This means that the
layout algorithm will determine the positions of the nodes and edges such that the edge labels do not overlap with the
rest of the layout. Integrated edge labeling can be activated using method {@link OrthogonalLayout#integratedEdgeLabeling}.
{@link OrthogonalLayoutEdgeLayoutDescriptor} instances can be used for specifying individual information (e.g. distances) for
each edge in the graph. The descriptors are bound to the graph using {@link IDataProvider}s registered with
key {@link OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}. If there is no descriptor assigned to some edges,
a default descriptor will be used. To set default descriptors, use {@link OrthogonalLayout#edgeLayoutDescriptor}.
{@link OrthogonalLayout} tries to optimize diverse objective functions such as bend minimization, number of
edge crossings, edge length minimization or face maximization. These settings can be enabled using the corresponding
methods {@link OrthogonalLayout#optimizePerceivedBends}, {@link OrthogonalLayout#crossingReduction}, {@link OrthogonalLayout#edgeLengthReduction}, and
{@link OrthogonalLayout#faceMaximization}, respectively. The drawback when these settings are enabled is that the running time of the algorithm may be
drastically increased.
This layout algorithm is able to detect substructures in the input graph. If desired, it can handle and arrange them
explicitly, making the structure easy to recognize. Supported substructures are trees, chains and cycles. See the
corresponding style properties for details: {@link OrthogonalLayout#treeStyle}, {@link OrthogonalLayout#chainStyle} and {@link OrthogonalLayout#cycleStyle}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout Online Documentation}*/
export class OrthogonalLayout {
/**
Creates a new {@link OrthogonalLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.maximumDuration] The preferred time limit in milliseconds. This option sets the {@link OrthogonalLayout#maximumDuration} property on the created object.
@param [options.uniformPortAssignment] Whether or not the layout algorithm should try to obtain a uniform port assignment of the edges incident to the same node side. This option sets the {@link OrthogonalLayout#uniformPortAssignment} property on the created object.
@param [options.treeStyle] The tree layout style that defines the basic arrangement style for subtrees. This option sets the {@link OrthogonalLayout#treeStyle} property on the created object.
@param [options.treeSize] The minimum size (number of nodes) a subtree needs to have to be detected and explicitly handled as a tree substructure. This option sets the {@link OrthogonalLayout#treeSize} property on the created object.
@param [options.treeOrientation] The desired orientation for subtree layouts. This option sets the {@link OrthogonalLayout#treeOrientation} property on the created object.
@param [options.chainStyle] The chain layout style that defines how <em>chain</em> substructures are arranged. This option sets the {@link OrthogonalLayout#chainStyle} property on the created object.
@param [options.chainSize] The minimum size (number of nodes) a chain needs to have to be detected and handled as a chain substructure. This option sets the {@link OrthogonalLayout#chainSize} property on the created object.
@param [options.cycleStyle] The cycle layout style that defines how <em>cycle</em> substructures are arranged. This option sets the {@link OrthogonalLayout#cycleStyle} property on the created object.
@param [options.cycleSize] The minimum size (number of nodes) a cycle needs to have to be detected and explicitly handled as a cycle substructure. This option sets the {@link OrthogonalLayout#cycleSize} property on the created object.
@param [options.preferParallelRoutes] Whether or not parallel routes for parallel edges (multi-edges) are preferred over independent routes. This option sets the {@link OrthogonalLayout#preferParallelRoutes} property on the created object.
@param [options.edgeLayoutDescriptor] The {@link OrthogonalLayoutEdgeLayoutDescriptor} instance used for all those edges that do not have a specific layout descriptor assigned. This option sets the {@link OrthogonalLayout#edgeLayoutDescriptor} property on the created object.
@param [options.considerNodeLabels] Whether or not the layout algorithm considers node labels when calculating node positions to avoid overlaps. This option sets the {@link OrthogonalLayout#considerNodeLabels} property on the created object.
@param [options.integratedEdgeLabeling] Whether or not the layout algorithm preserves space and places edge labels. This option sets the {@link OrthogonalLayout#integratedEdgeLabeling} property on the created object.
@param [options.randomization] Whether or not a randomization strategy should be performed. This option sets the {@link OrthogonalLayout#randomization} property on the created object.
@param [options.alignDegreeOneNodes] Whether or not degree-one nodes that have the same neighbor should be aligned. This option sets the {@link OrthogonalLayout#alignDegreeOneNodes} property on the created object.
@param [options.faceMaximization] Whether or not one face of the embedding of the graph should be maximized. This option sets the {@link OrthogonalLayout#faceMaximization} property on the created object.
@param [options.crossingReduction] Whether or not the number of edge crossings should be reduced. This option sets the {@link OrthogonalLayout#crossingReduction} property on the created object.
@param [options.optimizePerceivedBends] Whether or not the number of perceived bends should be minimized. This option sets the {@link OrthogonalLayout#optimizePerceivedBends} property on the created object.
@param [options.gridSpacing] The equidistant spacing between the horizontal and vertical grid lines. This option sets the {@link OrthogonalLayout#gridSpacing} property on the created object.
@param [options.layoutStyle] The layout style for this layout algorithm. This option sets the {@link OrthogonalLayout#layoutStyle} property on the created object.
@param [options.edgeLengthReduction] Whether or not the overall edge length should be optimized. This option sets the {@link OrthogonalLayout#edgeLengthReduction} property on the created object.
@param [options.fromSketchMode] Whether or not the existing drawing should be used as a sketch of the resulting orthogonal layout. This option sets the {@link OrthogonalLayout#fromSketchMode} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link OrthogonalLayout#componentLayoutEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link OrthogonalLayout#hideGroupsStageEnabled} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link OrthogonalLayout#orientationLayoutEnabled} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link MultiStageLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout Online Documentation}*/
constructor(options?: {
maximumDuration?: number;
uniformPortAssignment?: boolean;
treeStyle?: TreeLayoutStyle | TreeLayoutStyleStringValues;
treeSize?: number;
treeOrientation?: SubstructureOrientation | SubstructureOrientationStringValues;
chainStyle?: ChainLayoutStyle | ChainLayoutStyleStringValues;
chainSize?: number;
cycleStyle?: CycleLayoutStyle | CycleLayoutStyleStringValues;
cycleSize?: number;
preferParallelRoutes?: boolean;
edgeLayoutDescriptor?: OrthogonalLayoutEdgeLayoutDescriptor | null;
considerNodeLabels?: boolean;
integratedEdgeLabeling?: boolean;
randomization?: boolean;
alignDegreeOneNodes?: boolean;
faceMaximization?: boolean;
crossingReduction?: boolean;
optimizePerceivedBends?: boolean;
gridSpacing?: number;
layoutStyle?: OrthogonalLayoutStyle | OrthogonalLayoutStyleStringValues;
edgeLengthReduction?: boolean;
fromSketchMode?: boolean;
componentLayoutEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
orientationLayoutEnabled?: boolean;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
selfLoopRouterEnabled?: boolean;
labelingEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Calculates an orthogonal layout for the given graph.
<h2>Note</h2>
The given graph will not be copied during the layout process and the result will be immediately applied to the input
graph.
<h2>Note</h2>
This method is not side effect free in the sense that the order of edges or nodes in the input graph may change during
the layout process.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Calculates an orthogonal layout for the given graph.
<h2>Note</h2>
The given graph will not be copied during the layout process and the result will be immediately applied to the input
graph.
<h2>Note</h2>
This method is not side effect free in the sense that the order of edges or nodes in the input graph may change during
the layout process.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Returns a new {@link OrthogonalLayoutEdgeLayoutDescriptor} instance that will be used during the various phases of the layout
algorithm to determine the drawing details of the edges of the graph.
This method may be overridden to create a new {@link OrthogonalLayoutEdgeLayoutDescriptor} instance with different
configuration settings.
<b>Warning:</b> This method must not return <code>null</code>.
@returns a new {@link OrthogonalLayoutEdgeLayoutDescriptor} instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23createEdgeLayoutDescriptor() Online Documentation}*/
createEdgeLayoutDescriptor(): OrthogonalLayoutEdgeLayoutDescriptor | null;
/**
Data provider key for providing layout information for each edge
If no edge layout descriptor is mapped for an edge, a default edge layout descriptor will be obtained using method
{@link OrthogonalLayout#createEdgeLayoutDescriptor}.
<b>Warning:</b> Minimum edge length constraints can only be considered for layout styles {@link OrthogonalLayoutStyle#NORMAL} and
{@link OrthogonalLayoutStyle#UNIFORM}.
@see OrthogonalLayout#layoutStyle
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23EDGE_LAYOUT_DESCRIPTOR_DP_KEY Online Documentation}*/
static EDGE_LAYOUT_DESCRIPTOR_DP_KEY: EdgeDpKey<OrthogonalLayoutEdgeLayoutDescriptor> | null;
/**
Data provider key for marking edges which should be routed such that they point to the main layout orientation
The main layout orientation can be set using method {@link MultiStageLayout#layoutOrientation}.
<b>Warning:</b> This implementation can not be used if there are group nodes in the graph. This directed edges will be ignored if {@link GroupingKeys#GROUP_DP_KEY group nodes}
exist in the graph component.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23DIRECTED_EDGE_DP_KEY Online Documentation}*/
static DIRECTED_EDGE_DP_KEY: EdgeDpKey<boolean> | null;
/**
Data provider key for specifying the directedness of edges for the detection of substructures
The edge directedness is considered for the detection of substructures in the input graph i.e., {@link OrthogonalLayout#treeStyle trees}, {@link OrthogonalLayout#chainStyle chains} and
{@link OrthogonalLayout#cycleStyle cycles}. A substructure is only identified as such if all edges are either undirected or consistently directed with
respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, all edges are treated as undirected.
<b>Warning:</b> This key should not be confused with {@link OrthogonalLayout#DIRECTED_EDGE_DP_KEY}. The latter one marks
edges that are <em>directed orthogonal</em> and shall point in the specified
{@link MultiStageLayout#layoutOrientation layout orientation}. On the other hand, this key is only responsible for the specification of the directedness with
respect to the substructure detection.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23EDGE_DIRECTEDNESS_DP_KEY Online Documentation}*/
static EDGE_DIRECTEDNESS_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for providing crossing costs for each edge
The layout algorithm considers the specified costs during the crossing minimization phase. Edges with lower costs are
more likely to have crossings. If no individual costs are defined, each crossing has a cost of <code>1</code>.
<h2>Note</h2>
The algorithm uses a heuristic approach for crossing minimization and, thus, doesn't guarantee an optimal solution.
<b>Warning:</b>
Avoid using very large cost values. Due to the fact that costs need to be multiplied and summed up, this might lead to
overflows and undesired results.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23EDGE_CROSSING_COST_DP_KEY Online Documentation}*/
static EDGE_CROSSING_COST_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for providing bend costs for each edge
The layout algorithm considers the specified costs during the bend minimization phase. Edges with lower costs are more
likely to have bends. If no individual costs are defined, each bend has a cost of <code>1</code>.
<h2>Note</h2>
The algorithm uses a heuristic approach for bend minimization and, thus, doesn't guarantee an optimal solution.
<b>Warning:</b>
Avoid using very large cost values. Due to the fact that costs need to be multiplied and summed up, this might lead to
overflows and undesired results.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23EDGE_BEND_COST_DP_KEY Online Documentation}*/
static EDGE_BEND_COST_DP_KEY: EdgeDpKey<number> | null;
/**
Gets or sets the preferred time limit in milliseconds.
Depending on the specified value and the size of the input graph, the layout algorithm may automatically disable
properties {@link OrthogonalLayout#randomization}, {@link OrthogonalLayout#edgeLengthReduction}, {@link OrthogonalLayout#optimizePerceivedBends} and {@link OrthogonalLayout#crossingReduction}. These properties improve the layout quality but also increase the runtime, especially for larger
graphs. If the time limit is set to <code>0x7FFFFFFF</code>, the layout algorithm runs unrestricted.
Values have to be greater or equal to <code>0</code>.
<h2>Note</h2>
Restricting the maximum duration may result in a lower layout quality. Furthermore, the real runtime may exceed the
maximum duration since the layout algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets whether or not the layout algorithm should try to obtain a uniform port assignment of the edges incident to
the same node side.
If this option is enabled, the algorithm may insert additional bends in order to obtain a more uniform port assignment.
<h2>Note</h2>
If option {@link OrthogonalLayout#fromSketchMode} is enabled, the algorithm doesn't insert additional bends which may result in non-uniform ports even though
this option is enabled.
<b>Warning:</b> This option can only be considered for layout styles {@link OrthogonalLayoutStyle#NORMAL} and
{@link OrthogonalLayoutStyle#UNIFORM}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23uniformPortAssignment Online Documentation}*/
uniformPortAssignment: boolean;
/**
Gets or sets the tree layout style that defines the basic arrangement style for subtrees.
<h2>Note</h2>
The identified subtrees do never contain parallel edges (multiple edges between the same endpoints).
<b>Warning:</b> The subtree layout style is ignored if the input graph contains
{@link GroupingKeys#GROUP_DP_KEY group nodes}. In this case, trees are not detected and not arranged in a special way.
<b>Warning:</b>
Tree edges that connect to the subtree root node are never grouped together with non-tree edges connecting to the root
node. Edge grouping constraints for these edges are ignored.
@throws {Exception} <b>(ArgumentError)</b> if an unknown tree style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23treeStyle Online Documentation}*/
treeStyle: TreeLayoutStyle;
/**
Gets or sets the minimum size (number of nodes) a subtree needs to have to be detected and explicitly handled as a tree
substructure.
<b>Warning:</b> This setting has no effect if style {@link TreeLayoutStyle#NONE} is specified.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum size is less than <code>3</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23treeSize Online Documentation}*/
treeSize: number;
/**
Gets or sets the desired orientation for subtree layouts.
This orientation affects only trees that are specifically handled as a tree substructure when the {@link OrthogonalLayout#treeStyle tree style} is not
{@link TreeLayoutStyle#NONE}. It is independent of the
{@link MultiStageLayout#layoutOrientation main layout orientation}. For example, the main orientation could be top-to-bottom while the subtree orientation is
left-to-right at the same time.
<b>Warning:</b> This property has no effect if {@link TreeLayoutStyle#NONE} is set as tree style.
@throws {Exception} <b>(ArgumentError)</b> if the given layout orientation is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23treeOrientation Online Documentation}*/
treeOrientation: SubstructureOrientation;
/**
Gets or sets the chain layout style that defines how <em>chain</em> substructures are arranged.
A chain is a simple edge path where the degree of the nodes is less than or equal to <code>2</code>.
Property {@link OrthogonalLayout#chainSize} defines the minimum length a chain needs to have to be handled explicitly.
<b>Warning:</b> The chain layout style is ignored if the input graph contains
{@link GroupingKeys#GROUP_DP_KEY group nodes}. In this case, chains are not detected and not arranged in a special way.
<h2>Note</h2>
If a chain is part of a handled subtree (tree style other than {@link TreeLayoutStyle#NONE} must be selected),
then this chain will be arranged according to the tree layout style and not handled as a chain substructure.
@throws {Exception} <b>(ArgumentError)</b> if an unknown chain style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23chainStyle Online Documentation}*/
chainStyle: ChainLayoutStyle;
/**
Gets or sets the minimum size (number of nodes) a chain needs to have to be detected and handled as a chain
substructure.
<b>Warning:</b> This setting has no effect if style {@link ChainLayoutStyle#NONE} is specified.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum size is less than <code>2</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23chainSize Online Documentation}*/
chainSize: number;
/**
Gets or sets the cycle layout style that defines how <em>cycle</em> substructures are arranged.
A <em>cycle</em> is a simple edge path where the first and last node are identical. The algorithm only considers cycles where at most
one edge connects a cycle node with the rest of the graph (i.e. isolated cycles that are connected with one edge).
<b>Warning:</b> The cycle layout style is ignored if the input graph contains
{@link GroupingKeys#GROUP_DP_KEY group nodes}. In this case, cycles are not detected and not arranged in a special way.
@throws {Exception} <b>(ArgumentError)</b> if an unknown cycle style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23cycleStyle Online Documentation}*/
cycleStyle: CycleLayoutStyle;
/**
Gets or sets the minimum size (number of nodes) a cycle needs to have to be detected and explicitly handled as a cycle
substructure.
<b>Warning:</b> This setting has no effect if style {@link CycleLayoutStyle#NONE} is specified.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum size is less than <code>4</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23cycleSize Online Documentation}*/
cycleSize: number;
/**
Gets or sets whether or not parallel routes for parallel edges (multi-edges) are preferred over independent routes.
When enabled, the algorithm routes multi-edges (edges that connect the same pair of nodes) in parallel, i.e., these
edges connect to the same side of the nodes and have the same or a similar path. Note that the path is not always
exactly the same since the algorithm has to omit label-edge intersections if {@link OrthogonalLayout#integratedEdgeLabeling integrated labeling} is enabled. If this
setting is disabled, the edges are simply independent and might as well connect to different node sides and get very
different paths.
A parallel routing is suitable for a lot of diagrams (especially larger ones) as it helps to better recognize multi-edge
structures. However, there are scenarios where the routes of such edges might as well be independent. For small examples
the structure can also be clear if the edges connect to other node sides, especially when they are in consecutive order
around the node.
<h2>Note</h2>
This property only affects edges that share the same source and target node.
<b>Warning:</b> This setting is ignored when {@link MultiStageLayout#parallelEdgeRouterEnabled} is enabled, that is, if parallel edges are not handled by this layout algorithm
internally but by a specified stage.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23preferParallelRoutes Online Documentation}*/
preferParallelRoutes: boolean;
/**
Gets or sets the {@link OrthogonalLayoutEdgeLayoutDescriptor} instance used for all those edges that do not have a specific
layout descriptor assigned.
By default, this method will return an {@link OrthogonalLayoutEdgeLayoutDescriptor} instance created with
{@link OrthogonalLayout#createEdgeLayoutDescriptor}.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link OrthogonalLayoutEdgeLayoutDescriptor} is <code>null</code>
@see OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23edgeLayoutDescriptor Online Documentation}*/
edgeLayoutDescriptor: OrthogonalLayoutEdgeLayoutDescriptor | null;
/**
Gets or sets whether or not the layout algorithm considers node labels when calculating node positions to avoid
overlaps.
This method is a convenience method that assures that the {@link MultiStageLayout#labeling labeling algorithm} is of type {@link LabelLayoutTranslator}
and {@link LabelLayoutTranslator#translateNodeLabels} is set to <code>true</code>.
<h2>Note</h2>
Setting this option may overwrite the currently set labeling algorithm. Hence, to combine this option with a generic
edge labeling algorithm, the generic labeling has to be applied in an additional step after calculating the layout.
@throws {Exception} <b>(InvalidOperationError)</b> if no properly configured {@link LabelLayoutTranslator} is registered even though this property was enabled
earlier (can happen when manually specifying the {@link MultiStageLayout#labeling labeling algorithm}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets whether or not the layout algorithm preserves space and places edge labels.
To define the desired placement for each label add a {@link PreferredPlacementDescriptor} on
{@link IEdgeLabelLayout}.
This method also assures that the {@link MultiStageLayout#labeling labeling algorithm} is of type {@link LabelLayoutTranslator} and {@link LabelLayoutTranslator#translateEdgeLabels} is set to
<code>true</code>.
<b>Warning:</b> When enabling this property, any previously specified {@link MultiStageLayout#labeling labeling algorithm} will be overwritten and when disabling it, any
currently specified labeling algorithm will be disabled via property
{@link MultiStageLayout#labelingEnabled}. Therefore, it is recommended to only use this convenience property instead of manually changing the mentioned other
properties.
<b>Warning:</b> Edge labels will only be considered if the layout style is set to {@link OrthogonalLayoutStyle#NORMAL}.
@throws {Exception} <b>(InvalidOperationError)</b> if no properly configured {@link LabelLayoutTranslator} is registered even though integrated labeling was enabled
earlier (can happen when manually specifying the {@link MultiStageLayout#labeling labeling algorithm}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23integratedEdgeLabeling Online Documentation}*/
integratedEdgeLabeling: boolean;
/**
Gets or sets whether or not a randomization strategy should be performed.
The randomization will increase the overall layout quality with high probability. The drawback is that the running time
of the layout algorithm may increase.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23randomization Online Documentation}*/
randomization: boolean;
/**
Gets or sets whether or not degree-one nodes that have the same neighbor should be aligned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23alignDegreeOneNodes Online Documentation}*/
alignDegreeOneNodes: boolean;
/**
Gets or sets whether or not one face of the embedding of the graph should be maximized.
An embedding of a graph is uniquely specified by the cyclic order of edges incident to the same node in a graph
drawing.
In any planar drawing of a planar graph, the edges divide the plane into different regions called faces. If one of these
faces (the outer face) gets maximized, all other faces will be more compact.
@see OrthogonalLayout#randomization
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23faceMaximization Online Documentation}*/
faceMaximization: boolean;
/**
Gets or sets whether or not the number of edge crossings should be reduced.
The layout algorithm will try to significantly reduce the number of crossings in a post-processing step.
<b>Warning:</b> The running time of the layout algorithm may increase.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23crossingReduction Online Documentation}*/
crossingReduction: boolean;
/**
Gets or sets whether or not the number of perceived bends should be minimized.
A perceived bend is not an actual bend but it is created when a helical arrangement of chains of nodes occurs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23optimizePerceivedBends Online Documentation}*/
optimizePerceivedBends: boolean;
/**
Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
Each node will be placed on a grid point. Edges will be routed such that all segments but the first and last one lie on
grid lines. (The first and last segments of an edge may or may not lie on grid lines.) Edges consisting of a single
segment always lie on grid lines.
The grid spacing has to be greater than <code>0</code>.
<b>Warning:</b> This option is only guaranteed if {@link OrthogonalLayout#layoutStyle layout style} is
{@link OrthogonalLayoutStyle#NORMAL}. For all other styles, it is used as a hint only.
@throws {Exception} <b>(ArgumentError)</b> if the grid spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23gridSpacing Online Documentation}*/
gridSpacing: number;
/**
Gets or sets the layout style for this layout algorithm.
@throws {Exception} <b>(ArgumentError)</b> if an unknown layout style is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23layoutStyle Online Documentation}*/
layoutStyle: OrthogonalLayoutStyle;
/**
Gets or sets whether or not the overall edge length should be optimized.
The algorithm will try to significantly reduce the length of some edges.
<h2>Note</h2>
The drawback is that the running time of the layout algorithm may increase.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23edgeLengthReduction Online Documentation}*/
edgeLengthReduction: boolean;
/**
Gets or sets whether or not the existing drawing should be used as a sketch of the resulting orthogonal layout.
The layout algorithm will try to orthogonalize the given sketch without making too many modifications with respect to
the original drawing.
<h2>Note</h2>
If this option is enabled, the existing drawing should be preserved as much as possible. Hence, optimization steps like
{@link OrthogonalLayout#crossingReduction}, {@link OrthogonalLayout#faceMaximization} and {@link OrthogonalLayout#optimizePerceivedBends} are not applied even if enabled.
<b>Warning:</b> This implementation can not use the sketch as input if there are group nodes in the graph. This property will be
ignored if {@link GroupingKeys#GROUP_DP_KEY group nodes} exist in the graph component
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23fromSketchMode Online Documentation}*/
fromSketchMode: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated.
<b>Warning:</b> {@link OrthogonalLayout} can only handle single components. Disabling {@link ComponentLayout} will lead
to errors during execution.
@see MultiStageLayout#componentLayoutEnabled
@see MultiStageLayout#componentLayout
@see ComponentLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23componentLayoutEnabled Online Documentation}*/
componentLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for hiding group nodes is activated.
<b>Warning:</b> {@link OrthogonalLayout} cannot handle group nodes. Disabling {@link HideGroupsStage} will lead to
errors during execution.
@see MultiStageLayout#hideGroupsStageEnabled
@see MultiStageLayout#hideGroupsStage
@see HideGroupsStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23hideGroupsStageEnabled Online Documentation}*/
hideGroupsStageEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated.
<h2>Note</h2>
If the current layout orientation is
{@link LayoutOrientation#TOP_TO_BOTTOM}, the orientation of the layout will not be modified.
<b>Warning:</b> Based on the undirected layout style, disabling {@link OrientationLayout} has no significant effect unless {@link OrthogonalLayout#DIRECTED_EDGE_DP_KEY directed orthogonal edges}
are enabled.
@see MultiStageLayout#orientationLayoutEnabled
@see MultiStageLayout#orientationLayout
@see MultiStageLayout#layoutOrientation
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayout%23orientationLayoutEnabled Online Documentation}*/
orientationLayoutEnabled: boolean;
static $class: Class;
private $_OrthogonalLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalLayout;
}
export interface CompactOrthogonalLayout extends PartitionLayout {
}
/**
This layout algorithm arranges a graph in a compact, orthogonal fashion.
<b>Layout Style</b>
This layout algorithm is a variant of {@link OrthogonalLayout} that tries to arrange nodes in a more compact
way. It allows to specify a desired aspect ratio for the bounds of the calculated layout. Nodes of a given graph are
arranged such that each edge is drawn as an alternating sequence of horizontal and vertical segments.
{@link CompactOrthogonalLayout} is well suited for small and medium-sized sparse graphs and produces compact
drawings with no overlapping nodes, few crossings and few bends. It is also especially suitable for printing the diagram
on a paper of predefined size. As a disadvantage, this algorithm will usually produce less elegant edge routes than the
ones of {@link OrthogonalLayout}.
Application domains of orthogonal drawings are software engineering, project management, function call graphs and
object-oriented class diagrams.
<b>Concept</b>
This algorithm is implemented as a {@link PartitionLayout} and applies the following steps:
<ul>
<li>Divides the graph into smaller partitions using the partition finder instance created by
{@link PartitionLayout#createEdgeBetweennessPartitionFinder}.
<li>Arranges the nodes of each graph partition using {@link OrthogonalLayout}.
<li>Arranges the partitions themselves in a compact manner using the partition placer instance created by
{@link PartitionLayout#createComponentPartitionPlacer}.
<li>Routes the edges between partitions (inter-edges) using the router instance created by CreateChannelInterEdgeRouter.
</ul>
Each of these steps can be individually configured and/or replaced. To do so, use methods {@link PartitionLayout#partitionFinder}, {@link LayoutStageBase#coreLayout}, {@link PartitionLayout#partitionPlacer} and {@link PartitionLayout#interEdgeRouter}, respectively.
<b>Features</b>
The aspect ratio of the resulting layout bounds, i.e., the ratio of width/height of the layout, can be configured using
{@link CompactOrthogonalLayout#aspectRatio}. {@link CompactOrthogonalLayout} tries to reach this preferred value. However, depending on the actual graph
structure, this may not always be possible.
Octilinear routing of edges can be supported by appending {@link PolylineLayoutStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactOrthogonalLayout Online Documentation}*/
export class CompactOrthogonalLayout {
/**
Creates a new {@link CompactOrthogonalLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.gridSpacing] The equidistant spacing between the horizontal and vertical grid lines. This option sets the {@link CompactOrthogonalLayout#gridSpacing} property on the created object.
@param [options.aspectRatio] The desired aspect ratio (that is the quotient width/height) of the resulting layout bounds. This option sets the {@link CompactOrthogonalLayout#aspectRatio} property on the created object.
@param [options.partitionFinder] The {@link IPartitionFinder} instance that is responsible for partitioning the input graph. This option sets the {@link PartitionLayout#partitionFinder} property on the created object.
@param [options.partitionPlacer] The {@link IPartitionPlacer} instance that is responsible for arranging the graph partitions. This option sets the {@link PartitionLayout#partitionPlacer} property on the created object.
@param [options.interEdgeRouter] The IInterEdgeRouter instance that is responsible for inter-edge routing. This option sets the {@link PartitionLayout#interEdgeRouter} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactOrthogonalLayout Online Documentation}*/
constructor(options?: {
gridSpacing?: number;
aspectRatio?: number;
partitionFinder?: IPartitionFinder | null;
partitionPlacer?: IPartitionPlacer | null;
interEdgeRouter?: IPartitionInterEdgeRouter | null;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
Each node will be placed on a grid point. Edges will be routed such that their segments lie on grid lines, if the
endpoints of the edges allow to place the ports accordingly.
The grid spacing has to be greater than <code>0</code>.
<b>Warning:</b>
Setting the grid spacing will only have an effect if the configurable stages have not been replaced by other custom
stage implementations.
<b>Warning:</b> When setting configurable stages (e.g. the
{@link PartitionLayout#partitionFinder partition placer}) after specifying the grid spacing, it is assumed that the stages are already properly configured. The
previously set grid spacing will not be passed to these stages.
@throws {Exception} <b>(ArgumentError)</b> if the grid spacing is negative or zero
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactOrthogonalLayout%23gridSpacing Online Documentation}*/
gridSpacing: number;
/**
Gets or sets the desired aspect ratio (that is the quotient width/height) of the resulting layout bounds.
The aspect ratio needs to be greater than <code>0</code>.
<b>Warning:</b> This setting will only have an effect if the {@link PartitionLayout#partitionPlacer partition placer} is created by
{@link PartitionLayout#createComponentPartitionPlacer}.
<b>Warning:</b> When setting configurable stages (e.g. the
{@link PartitionLayout#partitionFinder partition placer}) after specifying the aspect ratio, it is assumed that the stages are already properly configured. The
previously set aspect ratio will not be passed to these stages.
@throws {Exception} <b>(ArgumentError)</b> if the given aspect ratio is less than or equal to zero
@see PartitionLayout#partitionPlacer
@see PartitionLayout#createComponentPartitionPlacer
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactOrthogonalLayout%23aspectRatio Online Documentation}*/
aspectRatio: number;
static $class: Class;
private $_CompactOrthogonalLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CompactOrthogonalLayout;
}
export interface FixGroupLayoutStage extends LayoutStageBase {
}
/**
A {@link ILayoutStage layout stage} that can handle fixed nodes in hierarchically grouped graphs.
This stage guarantees that a {@link LayoutStageBase#coreLayout core layout algorithm} will <b>maintain
the relative positions</b> of nodes within the same group.
Also, this stage is responsible for routing edges that connect nodes belonging to hierarchically unrelated groups. More
precisely, the stage temporarily replaces fixed group nodes (see
{@link FixGroupLayoutStage#FIXED_GROUP_NODES_DP_KEY}) with normal nodes of the same size while the content of the
groups is removed from the graph during layout.
The nodes are used as placeholders and are responsible for keeping the relations to the remaining graph elements. Edges
that originate from the outside of the group node and terminate in its interior (inter-edges) are redirected such that
they connect to the node that replaces the group.
After the layout process, the group nodes are restored and the stage inserts the content at the new group position
(derived from the position of the associated node). The content itself (including the edges between content nodes)
remains unchanged and is only relocated. Furthermore, the inter-edges are connected to their original source/target and
the stage reroutes them using the specified inter-edge routing algorithm (see {@link FixGroupLayoutStage#interEdgeRouter}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutStage Online Documentation}*/
export class FixGroupLayoutStage {
/**
Creates a new instance of {@link FixGroupLayoutStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new instance of {@link FixGroupLayoutStage} with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.interEdgeRoutingStyle] The inter-edge routing style used by this layout stage. This option sets the {@link FixGroupLayoutStage#interEdgeRoutingStyle} property on the created object.
@param [options.interEdgeRouter] The edge routing algorithm that is applied to route the inter-edges. This option sets the {@link FixGroupLayoutStage#interEdgeRouter} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
interEdgeRoutingStyle?: InterEdgeRoutingStyle | InterEdgeRoutingStyleStringValues;
interEdgeRouter?: ILayoutAlgorithm | null;
});
/**
Calculates the layout for the given graph, that is, prepares the graph by hiding the content of fixed group nodes for
the {@link LayoutStageBase#coreLayout core layout algorithm}, executes it and, finally, restores the graph and routes inter-edges.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for selecting group nodes whose contents should keep their relative positions
A group node is considered <i>selected</i> in this context, if the corresponding {@link IDataProvider} returns <code>true</code> for the
group node or any of its ancestor (group) nodes.
<h2>Note</h2>
If there is no {@link IDataProvider} registered with this key, all group nodes are considered to be selected.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutStage%23FIXED_GROUP_NODES_DP_KEY Online Documentation}*/
static FIXED_GROUP_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the inter-edge routing style used by this layout stage.
@throws {Exception} <b>(ArgumentError)</b> if the given routing style is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutStage%23interEdgeRoutingStyle Online Documentation}*/
interEdgeRoutingStyle: InterEdgeRoutingStyle;
/**
Gets or sets the edge routing algorithm that is applied to route the inter-edges.
If no routing algorithm is set, a new instance of {@link CompositeLayoutStage} associated with a {@link HideGroupsStage}
and an {@link EdgeRouter} will be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutStage%23interEdgeRouter Online Documentation}*/
interEdgeRouter: ILayoutAlgorithm | null;
static $class: Class;
private $_FixGroupLayoutStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FixGroupLayoutStage;
}
export interface IPartitionFinder {
}
/**
A {@link IPartitionFinder} is responsible for partitioning the input graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartitionFinder Online Documentation}*/
export abstract class IPartitionFinder {
/**
Returns a partition ID for each node of an input graph.
Nodes that share a common partition ID are considered to belong to the same partition.
@param graph the input graph
@param partitionIDMap the map that is filled with the resulting partition IDs
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartitionFinder%23findPartitions(LayoutGraph%2CINodeMap) Online Documentation}*/
abstract findPartitions(graph: LayoutGraph | null, partitionIDMap: INodeMap | null): void;
static $class: Class;
private $_IPartitionFinder: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPartitionFinder;
}
export interface IPartitionPlacer {
}
/**
A {@link IPartitionPlacer} is responsible for arranging the graph partitions.
Arranging a graph partition usually involves applying a coordinate translation to all elements that belong to the
partition in order to generate a non-overlapping node arrangement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartitionPlacer Online Documentation}*/
export abstract class IPartitionPlacer {
/**
Arranges the graph while taking partition IDs and inter-edge information into account.
Edges which connect nodes that belong to two different partitions are called inter-edges
@param graph the input graph
@param partitionIDDP the partition ID for each node in the graph
@param interEdgeDP the map which marks inter-edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartitionPlacer%23placePartitions(LayoutGraph%2CIDataProvider%2CIDataProvider) Online Documentation}*/
abstract placePartitions(graph: LayoutGraph | null, partitionIDDP: IDataProvider | null, interEdgeDP: IDataProvider | null): void;
static $class: Class;
private $_IPartitionPlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPartitionPlacer;
}
export interface PartitionLayout extends LayoutStageBase {
}
/**
{@link PartitionLayout} provides a framework for layouts that are based on a divide and conquer approach.
<b>Concept</b>
The approach can be subdivided into four stages:
<ol>
<li>The input graph is partitioned into smaller units. Implementations of this phase can be set using method {@link PartitionLayout#partitionFinder}.
<li>Optionally, the layout algorithm arranges each partition. Implementations of this phase can be set using method {@link LayoutStageBase#coreLayout}.
<li>The graph partitions are arranged next to each other keeping their layout. Implementations of this phase can be set
using method {@link PartitionLayout#partitionPlacer}.
<li>Edges that connect nodes in different graph partitions will be routed. Implementations of this phase can be set using
method {@link PartitionLayout#interEdgeRouter}
</ol>
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayout Online Documentation}*/
export class PartitionLayout {
/**
Creates a new {@link PartitionLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.partitionFinder] The {@link IPartitionFinder} instance that is responsible for partitioning the input graph. This option sets the {@link PartitionLayout#partitionFinder} property on the created object.
@param [options.partitionPlacer] The {@link IPartitionPlacer} instance that is responsible for arranging the graph partitions. This option sets the {@link PartitionLayout#partitionPlacer} property on the created object.
@param [options.interEdgeRouter] The IInterEdgeRouter instance that is responsible for inter-edge routing. This option sets the {@link PartitionLayout#interEdgeRouter} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayout Online Documentation}*/
constructor(options?: {
partitionFinder?: IPartitionFinder | null;
partitionPlacer?: IPartitionPlacer | null;
interEdgeRouter?: IPartitionInterEdgeRouter | null;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Arranges and places independent partitions in the graph.
@param graph
@throws {Exception} <b>(InvalidOperationError)</b> if the {@link PartitionLayout#partitionFinder partition finder} is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Creates a {@link IPartitionPlacer} that uses {@link ComponentLayout} to place the partitions.
@param [componentLayout=null] An optional {@link ComponentLayout} to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayout%23createComponentPartitionPlacer(ComponentLayout) Online Documentation}*/
static createComponentPartitionPlacer(componentLayout?: ComponentLayout | null): IPartitionPlacer | null;
/**
Creates a {@link IPartitionFinder} that uses {@link GroupAlgorithm#edgeBetweennessClustering} as strategy for
finding graph partitions.
@param [minimumPartitionCount=0] The minimum number of partitions the result should have.
@param [maximumPartitionCount=0x7FFFFFFF] The maximum number of partitions the result should have.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayout%23createEdgeBetweennessPartitionFinder(number%2Cnumber) Online Documentation}*/
static createEdgeBetweennessPartitionFinder(minimumPartitionCount?: number, maximumPartitionCount?: number): IPartitionFinder | null;
/**
Creates a {@link IPartitionFinder} that uses {@link GroupAlgorithm#edgeBetweennessClustering} as strategy for
finding graph partitions.
@param options The parameters to pass.
@param [options.minimumPartitionCount=0] The minimum number of partitions the result should have.
@param [options.maximumPartitionCount=0x7FFFFFFF] The maximum number of partitions the result should have.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayout%23createEdgeBetweennessPartitionFinder(number%2Cnumber) Online Documentation}*/
static createEdgeBetweennessPartitionFinder(options: {
minimumPartitionCount?: number;
maximumPartitionCount?: number;
}): IPartitionFinder | null;
/**
Gets or sets the {@link IPartitionFinder} instance that is responsible for partitioning the input graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayout%23partitionFinder Online Documentation}*/
partitionFinder: IPartitionFinder | null;
/**
Gets or sets the {@link IPartitionPlacer} instance that is responsible for arranging the graph partitions.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayout%23partitionPlacer Online Documentation}*/
partitionPlacer: IPartitionPlacer | null;
/**
Gets or sets the IInterEdgeRouter instance that is responsible for inter-edge routing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayout%23interEdgeRouter Online Documentation}*/
interEdgeRouter: IPartitionInterEdgeRouter | null;
static $class: Class;
private $_PartitionLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionLayout;
}
export interface EdgeRouterBusDescriptor extends YObject {
}
/**
A {@link EdgeRouterBusDescriptor} defines settings for edges routed on a common bus by {@link EdgeRouter}.
Edges (except self-loops) mapped to the same {@link EdgeRouterBusDescriptor} instance form a bus. The mapping is
retrieved from a {@link IDataProvider} registered with the input graph with key
{@link EdgeRouter#BUS_DESCRIPTOR_DP_KEY}. A bus consists of a set of backbone segments shared by multiple edges to
which shorter segments that connect to the actual nodes are attached. This descriptor allows to specify settings about
the bus, for example, the location of the bus may be user-specified - see {@link EdgeRouterBusDescriptor#busPoints}.
With buses, the routing algorithm tries to produce routes where the edges share as much of their paths as possible. It
yields long line segments (so-called <b>backbone segments</b>) where ideally all but the first and last segments of all edge paths are drawn on
top of each other (forming the <b>bus</b>), with short connections branching off to the nodes (<b>bus connections</b>).
Edges that are fixed (i.e. are not marked for routing) may also belong to a bus. Then, the bus segments will be derived
using the existing path of the fixed edges. This way, buses can be incrementally updated, e.g., when a new edge should
be added to an existing bus structure. Carefully note that fixed edges must have a correct orthogonal path for this
feature to work properly.
<h2>Note</h2>
Self-loops (i.e., edges that start and end at the same node) are never included into a bus.
<h2>Note</h2>
The generated bus structure is always connected, even when there are nodes on the bus that are only connected to some of
the other bus nodes and it would in consequence not even be necessary to use some backbone segments. In this case, one
or more edges of the bus get additional segments (potentially containing zero-degree angles) that make the bus
connected. If such segments are undesired, they can be removed by applying {@link RemoveCollinearBendsStage}.
@see EdgeRouter#BUS_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterBusDescriptor Online Documentation}*/
export class EdgeRouterBusDescriptor {
/**
Creates a new instance of a {@link EdgeRouterBusDescriptor} with the default values.
@param [options] The parameters to pass.
@param [options.busPoints] The user-specified bus path that represents the backbone for edges associated with this bus. This option sets the {@link EdgeRouterBusDescriptor#busPoints} property on the created object.
@param [options.minimumBackboneSegmentLength] The preferred minimum length of backbone segments. This option sets the {@link EdgeRouterBusDescriptor#minimumBackboneSegmentLength} property on the created object.
@param [options.multipleBackboneSegments] Whether or not the algorithm may create buses that consist of multiple backbone segments. This option sets the {@link EdgeRouterBusDescriptor#multipleBackboneSegments} property on the created object.
@param [options.automaticEdgeGrouping] Whether or not bus edges ending at the same node are automatically grouped such that they share a common bus connection. This option sets the {@link EdgeRouterBusDescriptor#automaticEdgeGrouping} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterBusDescriptor Online Documentation}*/
constructor(options?: {
busPoints?: YPointPath | null;
minimumBackboneSegmentLength?: number;
multipleBackboneSegments?: boolean;
automaticEdgeGrouping?: boolean;
});
/**
Gets or sets the user-specified bus path that represents the backbone for edges associated with this bus.
If the user does not specify the bus path, the algorithm automatically detects a suitable bus path. Similar to an edge
path, a bus path is specified by an instance of {@link YPointPath}.
<h2>Note</h2>
The user-specified path has to be orthogonal.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterBusDescriptor%23busPoints Online Documentation}*/
busPoints: YPointPath | null;
/**
Gets the {@link YPointPath point path} that describes the bus computed by the {@link EdgeRouter}.
The computed path of a bus is always orthogonal and connected. The bus path represents the backbone of the edges
associated with the bus and the edges use parts of the bus path in their own path.
<h2>Note</h2>
If the bus path was user-specified, the returned point path is equal to that returned by method {@link EdgeRouterBusDescriptor#busPoints}.
@see EdgeRouterBusDescriptor#busPoints
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterBusDescriptor%23computedBusPoints Online Documentation}*/
readonly computedBusPoints: YPointPath | null;
/**
Gets or sets the preferred minimum length of backbone segments.
This number defines the minimum length of backbone segments which are computed during the selection of backbone
segments. Final backbone segments might actually be shorter because not the full segment is used by bus connections;
such dead-ends are not drawn.
The minimum length is defined to be a value greater than or equal to <code>1.0</code>.
<h2>Note</h2>
The minimum length should be at least as large as the typical distance between nodes to avoid small backbone segments.
It is reasonable to set this according to the dimension of the bounding box of the whole graph.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum length is smaller than <code>1.0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterBusDescriptor%23minimumBackboneSegmentLength Online Documentation}*/
minimumBackboneSegmentLength: number;
/**
Gets or sets whether or not the algorithm may create buses that consist of multiple backbone segments.
For edges routed on a bus, a preferred outcome is that the part of the edge that is not on the bus is short and has few
bends. Of course, for buses that may consist of multiple backbone segments (instead of just a single backbone) this is
usually easier to achieve. Hence, if this option is enabled, the algorithm automatically may decide to use multiple
backbone segments in order to produce more suitable results.
<b>Warning:</b> This option is only considered if the user does not specify a {@link EdgeRouterBusDescriptor#busPoints custom bus path}.
@see EdgeRouterBusDescriptor#busPoints
@see EdgeRouterBusDescriptor#minimumBackboneSegmentLength
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterBusDescriptor%23multipleBackboneSegments Online Documentation}*/
multipleBackboneSegments: boolean;
/**
Gets or sets whether or not bus edges ending at the same node are automatically grouped such that they share a common
bus connection.
This setting affects the bus connections of edges that belong to the same bus and end/start at the same node. Manually
defined edge groups are considered, too. See {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY} and
{@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}. There are generally three scenarios:
<ol>
<li>All edges share the same bus connections. Simply enable this property and define no other edge groupings.
<li>Each edge is routed individually between the bus and the end node. Disable this property and define no other edge
groupings.
<li>A mixture of the first two scenarios. Define the desired edge groupings and they will be considered. For edges that have
no grouping information, the value of this property is considered. This way multiple edges may share a bus connections
while others use their individual routes.
</ol>
<h2>Note</h2>
Manual edge grouping definitions are always stronger than this setting.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterBusDescriptor%23automaticEdgeGrouping Online Documentation}*/
automaticEdgeGrouping: boolean;
static $class: Class;
private $_EdgeRouterBusDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeRouterBusDescriptor;
}
export interface CellEntrance extends YObject {
}
/**
This class describes the location and the direction from which a given {@link PartitionCell partition cell} has been entered during a path
search.
The path that was followed in order to reach a given {@link PartitionCell partition cell} can be reconstructed by traversing all
{@link CellEntrance#previousEntrance previous entrances}. The {@link CellEntrance#enterInterval enter interval} and {@link CellEntrance#enterDirection enter direction} determine how the {@link PartitionCell partition cell} was entered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance Online Documentation}*/
export class CellEntrance {
/**
Creates a new {@link CellEntrance} instance that is a copy of a given other {@link CellEntrance}.
@param other the {@link CellEntrance} to be copied
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23CellEntrance(CellEntrance) Online Documentation}*/
constructor(other: CellEntrance);
/**
Creates a new {@link CellEntrance} instance for a given {@link PartitionCell}.
@param cell the {@link PartitionCell} that is entered
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23CellEntrance(PartitionCell) Online Documentation}*/
constructor(cell: PartitionCell);
/**
Returns a string representation of this {@link CellEntrance}.
@returns a string representation of this {@link CellEntrance}
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23toString() Online Documentation}*/
toString(): string;
/**
Gets the {@link PartitionCell partition cell} of this {@link CellEntrance} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23cell Online Documentation}*/
readonly cell: PartitionCell | null;
/**
Gets or sets the previous {@link CellEntrance} along the path that was traversed in order to reach the {@link CellEntrance#cell cell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23previousEntrance Online Documentation}*/
previousEntrance: CellEntrance | null;
/**
Gets or sets the {@link OrthogonalInterval} that was used for entering this {@link CellEntrance#cell cell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23enterInterval Online Documentation}*/
enterInterval: OrthogonalInterval | null;
/**
Gets or sets the direction from which this {@link CellEntrance#cell cell} was entered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23enterDirection Online Documentation}*/
enterDirection: Direction | null;
/**
Gets or sets the costs for the implicit path given by the {@link CellEntrance#previousEntrance previous entrances}.
The costs must be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the costs are negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23costs Online Documentation}*/
costs: number;
/**
Gets or sets the heuristic costs for the implicit path given by the {@link CellEntrance#previousEntrance previous entrances}.
Heuristic costs must be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the heuristic costs are negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23heuristicCosts Online Documentation}*/
heuristicCosts: number;
/**
Gets or sets the {@link EdgeCellInfo} describing how the previous {@link PartitionCell partition cell} was traversed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellEntrance%23previousEdgeCellInfo Online Documentation}*/
previousEdgeCellInfo: EdgeCellInfo | null;
static $class: Class;
private $_CellEntrance: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CellEntrance;
}
export interface CellSegmentInfo extends SegmentInfoBase {
}
/**
This class stores information about the part of an item segment that lies inside a specified {@link PartitionCell partition cell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellSegmentInfo Online Documentation}*/
export class CellSegmentInfo {
/**
Creates a new {@link CellSegmentInfo} instance.
@param item the item to which the segment of this info belongs
@param segmentIndex the index of the segment to which this info belongs
@param direction the direction to which the segment of this info points
@param locationRange the range within which the common location of this segment has to lie
@param minExtension the minimum interval in extension direction this segment part is known to intersect
@param maxExtension the maximum interval in extension direction this segment part will span
@param cell the partition cell within which this part of the edge segment lies
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellSegmentInfo Online Documentation}*/
constructor(item: any | null, segmentIndex: number, direction: Direction | null, locationRange: Interval | null, minExtension: Interval | null, maxExtension: Interval | null, cell: PartitionCell | null);
/**
Gets the direction to which the segment of this info points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellSegmentInfo%23direction Online Documentation}*/
readonly direction: Direction | null;
/**
Gets the {@link PartitionCell partition cell} within which this part of the edge segment lies.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CellSegmentInfo%23cell Online Documentation}*/
readonly cell: PartitionCell | null;
static $class: Class;
private $_CellSegmentInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CellSegmentInfo;
}
export interface Channel extends YObject {
}
/**
This class describes a channel, i.e., a set of vertical or horizontal edge segments whose locations depend on each
other.
Channels are used by the {@link ChannelBasedPathRouting} to route edges using
{@link SegmentInfo}s describing their path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel Online Documentation}*/
export class Channel {
/**
Creates a new {@link Channel} that contains the given {@link SegmentInfo segment info}.
@param segment the given segment info for which the {@link Channel} is created
@throws {Exception} <b>(ArgumentError)</b> if the given segment has a different orientation than the other segments already contained in the channel
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel Online Documentation}*/
constructor(segment: SegmentInfo | null);
/**
Adds a new {@link SegmentInfo segment info} to the {@link Channel channel}.
@param segment the segment info to be added to this channel
@throws {Exception} <b>(ArgumentError)</b> if the given segment has a different orientation than the other segments already contained in the channel
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23addSegment(SegmentInfo) Online Documentation}*/
addSegment(segment: SegmentInfo | null): void;
/**
Adds a {@link SegmentGroup segment group} to this {@link Channel channel}.
@param group the segment group to be added
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23addSegmentGroup(SegmentGroup) Online Documentation}*/
addSegmentGroup(group: SegmentGroup | null): void;
/**
Returns the current location that the given {@link SegmentInfo segment info} has been assigned within its {@link SegmentInfoBase#locationRange location range}.
@param segmentInfo the segment info for which the current location will be returned
@returns the current location of the segment info or {@link Number#NaN} if no location has been assigned before
@see Channel#setCurrentLocation
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23getCurrentLocation(SegmentInfo) Online Documentation}*/
getCurrentLocation(segmentInfo: SegmentInfo | null): number;
/**
Returns the {@link SegmentInfo segment info} with the given index in the list of segments of this {@link Channel channel}.
@param index the index of the segment in the list of segments
@returns the {@link SegmentInfo segment info} with the given index in the list of segments
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23getSegment(number) Online Documentation}*/
getSegment(index: number): SegmentInfo | null;
/**
Returns the {@link SegmentGroup segment group} with the given index in this {@link Channel channel}.
@param index the index of the segment group
@returns the {@link SegmentGroup segment group} with the given index in this channel
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23getSegmentGroup(number) Online Documentation}*/
getSegmentGroup(index: number): SegmentGroup | null;
/**
Returns the number of {@link SegmentInfo segment infos} in this channel.
@returns the number of segment infos in this channel
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23segmentCount() Online Documentation}*/
segmentCount(): number;
/**
Returns the number of {@link SegmentGroup segment groups} in this {@link Channel channel}.
@returns the number of segment groups in this channel
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23segmentGroupCount() Online Documentation}*/
segmentGroupCount(): number;
/**
Specifies the current location that the given {@link SegmentInfo segment info} has been assigned within its {@link SegmentInfoBase#locationRange location range}.
During the distribution of the segments in the channel, different locations can be assigned but only the last one is
finally assigned to the {@link SegmentInfo segment info}.
@param segmentInfo the segment info for which the current location will be set
@param location the location of the segment info within its location range
@see SegmentInfoBase#location
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23setCurrentLocation(SegmentInfo%2Cnumber) Online Documentation}*/
setCurrentLocation(segmentInfo: SegmentInfo | null, location: number): void;
/**
Returns a string representation of this {@link Channel} object.
@returns a string representation of this {@link Channel} object
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23toString() Online Documentation}*/
toString(): string;
/**
Gets an {@link Interval interval} spanning the union of the location ranges of the {@link SegmentInfo segment infos}.
@see SegmentInfoBase#locationRange
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23widthInterval Online Documentation}*/
readonly widthInterval: Interval | null;
/**
Gets an {@link Interval interval} spanning the union of the maximum extension of the {@link SegmentInfo segment infos}.
@see SegmentInfoBase#maxExtension
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23lengthInterval Online Documentation}*/
readonly lengthInterval: Interval | null;
/**
Gets whether or not the segments in this {@link Channel channel} are vertical.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23vertical Online Documentation}*/
readonly vertical: boolean;
/**
Gets the bounds of this {@link Channel channel}.
These bounds are determined by the channel's {@link Channel#widthInterval width} and {@link Channel#lengthInterval length}.
@see Channel#widthInterval
@see Channel#lengthInterval
@see {@link https://docs.yworks.com/yfileshtml/#/api/Channel%23bounds Online Documentation}*/
readonly bounds: YRectangle | null;
static $class: Class;
private $_Channel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Channel;
}
export interface ChannelBasedPathRouting extends YObject {
}
/**
This class routes edges using their pre-calculated {@link EdgeRouterPath} information.
The concept of this algorithm is to bundle edge {@link SegmentInfo segment}s into so-called {@link Channel}s and route each {@link Channel}
individually.
This class is initialized using {@link ChannelBasedPathRouting#initialize} which must have been called before the
edges are {@link ChannelBasedPathRouting#route routed}.
After a {@link ChannelBasedPathRouting} instance is no longer in use, the {@link Channel} information and
the reference to the {@link PathSearchConfiguration configuration} must be reset using method {@link ChannelBasedPathRouting#cleanUp}.
@see EdgeInfo#getSegmentInfo
@see LayoutGraph#setPath
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting Online Documentation}*/
export class ChannelBasedPathRouting {
/**
Creates a new instance of {@link ChannelBasedPathRouting}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting Online Documentation}*/
constructor();
/**
Adjusts the location assigned to a {@link SegmentInfo segment info} in the {@link Channel channel} considering the preferred alignment of the segment.
This method is called after the segments have been distributed in the channel but before the final locations of the
segments are assigned to the {@link SegmentInfo segment info}s.
@param channel the channel that contains the segment infos for which the alignment should be considered
@see Channel#getCurrentLocation
@see SegmentInfoBase#preferredAlignment
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting%23adjustSegmentLocations(Channel) Online Documentation}*/
adjustSegmentLocations(channel: Channel | null): void;
/**
Calculates the locations of the {@link SegmentInfo segment infos} in the given {@link Channel channel}.
This method is called after the order of the {@link SegmentInfo segment infos} within a channel has been {@link ChannelBasedPathRouting#optimizeSegmentOrder optimized}.
@param channel the channel that contains the segment infos for which the locations will be calculated
@see Channel#setCurrentLocation
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting%23calculateSegmentLocations(Channel) Online Documentation}*/
calculateSegmentLocations(channel: Channel | null): void;
/**
Resets all {@link Channel channel} information and the reference to the {@link PathSearchConfiguration configuration}.
Afterwards, {@link ChannelBasedPathRouting} is ready to calculate paths for a new layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting%23cleanUp() Online Documentation}*/
cleanUp(): void;
/**
Creates a {@link IComparer comparator} for sorting the {@link SegmentInfo}s in a {@link Channel} from left to right (for
vertical segments) or top to bottom (for horizontal segments), respectively.
This method is called in {@link ChannelBasedPathRouting#route} before edge segments are distributed within the
channels.
@param channels the channels whose segments will be sorted by the {@link IComparer comparator}
@param pathSearchResult the {@link PathSearchResult path search result} containing the {@link EdgeInfo}s with the
{@link SegmentInfo}s for which the comparator will be used
@param configuration the {@link PathSearchConfiguration configuration} used during this path search
@returns a {@link IComparer comparator} used for sorting the {@link SegmentInfo}s in a {@link Channel channel}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting%23createSegmentInfoComparer(YList%2CPathSearchResult%2CPathSearchConfiguration) Online Documentation}*/
createSegmentInfoComparer(channels: YList | null, pathSearchResult: PathSearchResult | null, configuration: PathSearchConfiguration | null): IComparer<any> | null;
/**
Initializes this {@link ChannelBasedPathRouting} instance for routing paths that have been found with the given
{@link PathSearchConfiguration configuration}.
This method must be called before the edges are {@link ChannelBasedPathRouting#route routed}.
@param configuration the {@link PathSearchConfiguration configuration} used for the following path routing
@see ChannelBasedPathRouting#route
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting%23initialize(PathSearchConfiguration) Online Documentation}*/
initialize(configuration: PathSearchConfiguration | null): void;
/**
Optimizes the order of the {@link SegmentInfo segments} in the given {@link Channel channel}.
After the segments in the channel have been sorted using the comparator created by method
{@link ChannelBasedPathRouting#createSegmentInfoComparer}, segments having a common {@link SegmentGroup} are
replaced by their common {@link SegmentGroup#commonSegmentInfo representative segment info}.
This method further improves the order of the resulting segment info list to make sure that the subsequent {@link ChannelBasedPathRouting#calculateSegmentLocations location calculation}
is able to respect the {@link SegmentInfoBase#locationRange location ranges} of the
{@link SegmentInfo}s and, if possible, the minimum edge to edge distances and possible grid constraints for the
segments.
@param channel the channel for which the order of the {@link SegmentInfo}s is optimized
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting%23optimizeSegmentOrder(Channel) Online Documentation}*/
optimizeSegmentOrder(channel: Channel | null): void;
/**
Routes the given edges.
Before calling this method, method {@link ChannelBasedPathRouting#initialize} must have been called.
As a first step, the {@link SegmentInfo}s of all edge paths are distributed to {@link Channel}s. After that,
the segments in each of these
{@link Channel}s are sorted and locations are assigned respecting the segments' location ranges by calling the
subsequent methods:
<ol>
<li>{@link ChannelBasedPathRouting#optimizeSegmentOrder}
<li>{@link ChannelBasedPathRouting#calculateSegmentLocations}
<li>{@link ChannelBasedPathRouting#adjustSegmentLocations}
</ol>
Finally, the control points for the paths are calculated.
@param edges the {@link EdgeList list of edges} to be routed
@param pathSearchResult the {@link PathSearchResult path search result} containing the {@link EdgeRouterPath} objects for the edges
@throws {Exception} <b>(InvalidOperationError)</b> if the locations are not within the segment's location range
@see PathSearchResult#getPath
@see SegmentInfoBase#locationRange
@see SegmentInfoBase#location
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting%23route(EdgeList%2CPathSearchResult) Online Documentation}*/
route(edges: EdgeList | null, pathSearchResult: PathSearchResult | null): void;
/**
Sorts the given list of {@link SegmentInfo}s using the given {@link IComparer comparator}.
This method is called in {@link ChannelBasedPathRouting#route} when the edge segments are distributed within the
channels.
@param segmentInfos the list of {@link SegmentInfo}s to be sorted
@param segmentInfoComparer the comparator used for sorting the list of {@link SegmentInfo}s
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting%23sortSegmentInfos(IList%3CObject%3E%2CIComparer%3CObject%3E) Online Documentation}*/
sortSegmentInfos(segmentInfos: IList<any> | null, segmentInfoComparer: IComparer<any> | null): void;
/**
Gets the {@link PathSearchConfiguration path search configuration} with which the {@link ChannelBasedPathRouting} has been {@link ChannelBasedPathRouting#initialize initialized}.
@see ChannelBasedPathRouting#initialize
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelBasedPathRouting%23configuration Online Documentation}*/
readonly configuration: PathSearchConfiguration | null;
static $class: Class;
private $_ChannelBasedPathRouting: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ChannelBasedPathRouting;
}
export interface DynamicObstacleDecomposition extends YObject, IObstaclePartition, IDynamicDecomposition {
}
/**
This class describes an {@link IObstaclePartition} that decomposes its area dynamically.
The partitioning strategy is based on binary space partitioning. It divides the partition space recursively in two {@link PartitionCell cells}
until each cell is completely covered by one or more {@link Obstacle}s or completely empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition Online Documentation}*/
export class DynamicObstacleDecomposition {
/**
Constructs a new instance of {@link DynamicObstacleDecomposition}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition Online Documentation}*/
constructor();
/**
Adds the given {@link IDecompositionListener dynamic decomposition listener} to receive {@link PartitionCell} subdivision and creation events
from this decomposition.
These events occur when the decomposition changes the partition by subdividing cells into sub-cells or when new cells
are created.
@param listener the dynamic decomposition listener to add
@see IDecompositionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23addDynamicDecompositionListener(IDecompositionListener) Online Documentation}*/
addDynamicDecompositionListener(listener: IDecompositionListener | null): void;
/**
Clears the partition data such that the {@link DynamicObstacleDecomposition} can be reused and {@link DynamicObstacleDecomposition#init initialized} with
new {@link Obstacle}s.
@see DynamicObstacleDecomposition#init
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23clear() Online Documentation}*/
clear(): void;
/**
Notifies all registered {@link IDecompositionListener dynamic decomposition listeners} that the given {@link PartitionCell partition cell} has been created.
This method is also called in {@link DynamicObstacleDecomposition#init}.
@param createdCell the newly created cell
@see IDecompositionListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23fireCreateCellEvent(PartitionCell) Online Documentation}*/
fireCreateCellEvent(createdCell: PartitionCell | null): void;
/**
Notifies all registered {@link IDecompositionListener dynamic decomposition listeners} that the given {@link PartitionCell partition cell} has been finalized.
@param finalizedCell the cell that has been finalized
@see IDecompositionListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23fireFinalizeCellEvent(PartitionCell) Online Documentation}*/
fireFinalizeCellEvent(finalizedCell: PartitionCell | null): void;
/**
Notifies all registered {@link IDecompositionListener dynamic decomposition listeners} of a subdivision of a given {@link PartitionCell partition cell}.
@param cell the cell that has been subdivided
@param subCells the new sub-cells resulting from the subdivision of the given cell
@see IDecompositionListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23fireSubdividedEvent(PartitionCell%2CIList%3CObject%3E) Online Documentation}*/
fireSubdividedEvent(cell: PartitionCell | null, subCells: IList<any> | null): void;
/**
Notifies all registered {@link IDecompositionListener dynamic decomposition listeners} that the given {@link PartitionCell partition cell} is unlocked again.
@param unlockedCell the cell that has been unlocked
@see IDecompositionListener
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23fireUnlockCellEvent(PartitionCell) Online Documentation}*/
fireUnlockCellEvent(unlockedCell: PartitionCell | null): void;
/**
Returns a list of all {@link PartitionCell}s that intersect or cover the given rectangle.
@param rect the rectangular area whose (partially) covered cells will be returned
@returns a list of {@link PartitionCell}s that (partially) cover the given rectangular area
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23getCells(YRectangle) Online Documentation}*/
getCells(rect: YRectangle | null): IList<any> | null;
/**
Returns all {@link PartitionCell partition cells} that are completely covered by the given {@link Obstacle}.
@param obstacle the obstacle for which the covered cells will be returned
@returns an unmodifiable list of {@link PartitionCell} instances that are completely covered by the given obstacle
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23getCellsForObstacle(Obstacle) Online Documentation}*/
getCellsForObstacle(obstacle: Obstacle | null): IList<any> | null;
/**
Calculates the cost of a cut with respect to the geometry of the sub-cells.
The cost can take values between <code>0</code> and <code>1</code>.
This method is called while a {@link PartitionCell} is divided into upper and lower or left and right child cells
depending on the cut costs (during {@link DynamicObstacleDecomposition#getCellsForObstacle}, {@link DynamicObstacleDecomposition#getCells}
and {@link DynamicObstacleDecomposition#getNeighbors} methods).
@param cut the coordinate of the cut
@param min the left side of the subdivided cell
@param max the right side of the subdivided cell
@param orthogonalMin the upper side of the subdivided cell
@param orthogonalMax the lower side of the subdivided cell
@returns the cost of a cut with respect to the geometry of the sub-cells
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23getGeometricCutCosts(number%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
getGeometricCutCosts(cut: number, min: number, max: number, orthogonalMin: number, orthogonalMax: number): number;
/**
Returns the neighbor {@link PartitionCell partition cells} of the given cell.
@param cell the cell whose neighbors will be returned
@returns the neighbor cells of the given cell
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23getNeighbors(PartitionCell) Online Documentation}*/
getNeighbors(cell: PartitionCell | null): IList<any> | null;
/**
Calculates the cost of a cut with respect to the subdivided obstacles.
The cost can take values between <code>0</code> and <code>1</code>.
This method is called while a {@link PartitionCell} is divided into upper and lower or left and right child cells
depending on the cut costs (during {@link DynamicObstacleDecomposition#getCellsForObstacle}, {@link DynamicObstacleDecomposition#getCells}
and {@link DynamicObstacleDecomposition#getNeighbors} methods).
@param numObstaclesInFirstHalf the number of obstacles that lie completely in the first half
@param numObstaclesInSecondHalf the number of obstacles that lie completely in the second half
@param numObstaclesOnCut the number of obstacles that lie on the cut
@returns the cost of a cut with respect to the subdivided obstacles
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23getObstacleCutCosts(number%2Cnumber%2Cnumber) Online Documentation}*/
getObstacleCutCosts(numObstaclesInFirstHalf: number, numObstaclesInSecondHalf: number, numObstaclesOnCut: number): number;
/**
Returns all {@link Obstacle}s that cover the given {@link PartitionCell partition cell}.
@param cell the partition cell for which the obstacles will be returned
@returns an unmodifiable list of {@link Obstacle} instances that cover the given cell
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23getObstacles(PartitionCell) Online Documentation}*/
getObstacles(cell: PartitionCell | null): IList<any> | null;
/**
Initializes this {@link DynamicObstacleDecomposition} instance with the given obstacles and partition bounds.
This method must be called before any other method is invoked.
@param obstacles a list of {@link Obstacle} objects
@param partitionBounds the bounds of the partition
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23init(IList%3CObject%3E%2CYRectangle) Online Documentation}*/
init(obstacles: IList<any> | null | IListConvertible<any>, partitionBounds: YRectangle | null): void;
/**
Removes the given {@link IDecompositionListener dynamic decomposition listener} such that it no longer receives {@link PartitionCell} subdivision
and creation events from this decomposition.
@param listener the dynamic decomposition listener to remove
@see IDecompositionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23removeDynamicDecompositionListener(IDecompositionListener) Online Documentation}*/
removeDynamicDecompositionListener(listener: IDecompositionListener | null): void;
/**
Gets or sets the costs incurred for every {@link Obstacle} that must be cut in a subdivision.
Values need to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the cost is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23cutObstacleCost Online Documentation}*/
cutObstacleCost: number;
/**
Gets or sets the costs incurred if the distribution after a subdivision of obstacles is unbalanced in sub-cells.
Values need to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the cost is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23unbalancedObstaclesCost Online Documentation}*/
unbalancedObstaclesCost: number;
/**
Gets or sets the costs incurred if the subdivision produces unbalanced rectangles.
Values need to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the cost is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23unbalancedRatioCost Online Documentation}*/
unbalancedRatioCost: number;
/**
Gets the bounds of the original rectangular area that is being partitioned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DynamicObstacleDecomposition%23bounds Online Documentation}*/
readonly bounds: YRectangle | null;
static $class: Class;
private $_DynamicObstacleDecomposition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DynamicObstacleDecomposition;
}
export interface EdgeCellInfo extends YObject {
}
/**
This class encapsulates the information that determines the route of a specific item within a specific {@link PartitionCell partition cell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo Online Documentation}*/
export class EdgeCellInfo {
/**
Creates a new {@link EdgeCellInfo} instance.
@param item the item whose routing in the cell is described
@param cell the cell in which the item is routed
@param enterInterval the interval that the item uses to enter the cell
@param exitInterval the interval that the item uses to exit the cell
@param enterDirection the direction in which the item enters the cell
@param exitDirection the direction in which the item exits the cell
@param enterSegmentNo the index of the item segment that enters the cell
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo Online Documentation}*/
constructor(item: any | null, cell: PartitionCell | null, enterInterval: OrthogonalInterval | null, exitInterval: OrthogonalInterval | null, enterDirection: Direction | null, exitDirection: Direction | null, enterSegmentNo: number);
/**
Returns a string representation of the shape while crossing the {@link PartitionCell partition cell}.
@returns a string representation of the shape while crossing the partition cell
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23toString() Online Documentation}*/
toString(): string;
/**
Gets the number of bends within this cell.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23bendCount Online Documentation}*/
readonly bendCount: number;
/**
Gets the routing type within this cell.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23type Online Documentation}*/
readonly type: RoutingType;
/**
Gets or sets the index of the segment that enters this cell.
Values should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given index is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23enterSegmentIndex Online Documentation}*/
enterSegmentIndex: number;
/**
Gets or sets the index of the segment that exits this cell.
Values should be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given index is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23exitSegmentIndex Online Documentation}*/
exitSegmentIndex: number;
/**
Gets or sets the {@link SegmentGroup segment group} containing the entering segment.
When specifying the segment group, the {@link EdgeCellInfo#enterInterval enter interval} is updated to use the segment group's common enter interval.
@see SegmentGroup#commonLocationRange
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23enterSegmentGroup Online Documentation}*/
enterSegmentGroup: SegmentGroup | null;
/**
Gets or sets the {@link SegmentGroup segment group} containing the exiting segment.
When specifying the segment group, the {@link EdgeCellInfo#exitInterval exit interval} is updated to use the segment group's common exit interval.
@see SegmentGroup#commonLocationRange
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23exitSegmentGroup Online Documentation}*/
exitSegmentGroup: SegmentGroup | null;
/**
Gets the item that is routed within the {@link EdgeCellInfo#cell partition cell}.
The item is either of type {@link PathRequest} or {@link Edge}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23item Online Documentation}*/
readonly item: any | null;
/**
Gets the {@link PartitionCell partition cell} within which the item is routed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23cell Online Documentation}*/
readonly cell: PartitionCell | null;
/**
Gets the {@link OrthogonalInterval interval} that is used to enter this cell.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23enterInterval Online Documentation}*/
readonly enterInterval: OrthogonalInterval | null;
/**
Gets the {@link OrthogonalInterval interval} that is used to exit this cell.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23exitInterval Online Documentation}*/
readonly exitInterval: OrthogonalInterval | null;
/**
Gets the {@link Direction direction} in which this cell is entered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23enterDirection Online Documentation}*/
readonly enterDirection: Direction | null;
/**
Gets the {@link Direction direction} in which this cell is exited.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23exitDirection Online Documentation}*/
readonly exitDirection: Direction | null;
/**
Calculates the list of {@link CellSegmentInfo}s for the segment parts that lie within the associated
{@link EdgeCellInfo#cell partition cell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeCellInfo%23cellSegmentInfos Online Documentation}*/
readonly cellSegmentInfos: YList | null;
static $class: Class;
private $_EdgeCellInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeCellInfo;
}
export interface EdgeInfo extends YObject {
}
/**
This class encapsulates the information required to route an edge with a path routing algorithm.
@see ChannelBasedPathRouting
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo Online Documentation}*/
export class EdgeInfo {
/**
Creates a new {@link EdgeInfo} instance.
@param edge the edge to create the info for
@param path the path that will be used for routing the edge
@param configuration the {@link PathSearchConfiguration configuration} of the path search algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23EdgeInfo(Edge%2CEdgeRouterPath%2CPathSearchConfiguration) Online Documentation}*/
constructor(edge: Edge | null, path: EdgeRouterPath | null, configuration: PathSearchConfiguration | null);
/**
Calculates an array of {@link LineSegment}s created by the {@link EdgeInfo#calculatePathPoints calculated path points}.
@returns an array of {@link LineSegment}s created by the calculated path points
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23calculateLineSegments() Online Documentation}*/
calculateLineSegments(): LineSegment[] | null;
/**
Calculates a list of {@link YPoint}s containing the source port, bend and target port locations.
@returns a list of {@link YPoint}s containing the source port, bend and target port locations
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23calculatePathPoints() Online Documentation}*/
calculatePathPoints(): YList | null;
/**
Returns the {@link EdgeCellInfo} of the given {@link CellEntrance} in the path.
@param entrance the entrance for which the {@link EdgeCellInfo} will be returned
@returns the {@link EdgeCellInfo} of the given {@link CellEntrance} in the path
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23getEdgeCellInfo(CellEntrance) Online Documentation}*/
getEdgeCellInfo(entrance: CellEntrance | null): EdgeCellInfo | null;
/**
Returns the {@link SegmentInfo} of the segment following the segment of the given {@link SegmentInfo}.
@param segment the segment info for which the next segment info will be returned
@returns the segment info of the segment following the segment of the given segment info
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23getNextSegment(SegmentInfo) Online Documentation}*/
getNextSegment(segment: SegmentInfo | null): SegmentInfo | null;
/**
Returns the {@link SegmentInfo} of the segment preceding the segment of the given {@link SegmentInfo}.
@param segment the segment info for which the previous segment info will be returned
@returns the segment info of the segment preceding the segment of the given segment info
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23getPreviousSegment(SegmentInfo) Online Documentation}*/
getPreviousSegment(segment: SegmentInfo | null): SegmentInfo | null;
/**
Returns the {@link SegmentInfo} of the edge segment with the given index.
@param segmentIndex the index of the edge segment for which the segment info will be returned
@returns the {@link SegmentInfo} of the edge segment with the given index
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23getSegmentInfo(number) Online Documentation}*/
getSegmentInfo(segmentIndex: number): SegmentInfo | null;
/**
Returns the number of segments of the edge.
@returns the number of segments of the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23segmentCount() Online Documentation}*/
segmentCount(): number;
/**
Returns a string representation of this {@link EdgeInfo} object.
@returns a string representation of this {@link EdgeInfo} object
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23toString() Online Documentation}*/
toString(): string;
/**
Gets whether the path of this {@link EdgeInfo#edge edge} is fixed or should be routed by the path search algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23fixed Online Documentation}*/
readonly fixed: boolean;
/**
Gets the edge whose route is described by this {@link EdgeInfo} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23edge Online Documentation}*/
readonly edge: Edge | null;
/**
Gets a list of {@link EdgeCellInfo}s describing how the edge traverses each {@link PartitionCell} on its
path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23edgeCellInfos Online Documentation}*/
readonly edgeCellInfos: IList<any> | null;
/**
Gets or sets the location of the strong source port that the edge uses.
Defining a new strong source port also updates the first {@link SegmentInfo} to use this fixed port location.
@throws {Exception} <b>(ArgumentError)</b> if the first segment is already fixed and the given port location contradicts the segment's fixed location
@see SegmentInfoBase#location
@see SegmentInfo#atStrongPortConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23strongSourcePort Online Documentation}*/
strongSourcePort: YPoint | null;
/**
Gets or sets the location of the strong target port that the edge uses.
Defining a new strong target port also updates the last {@link SegmentInfo} to use this fixed port location.
@throws {Exception} <b>(ArgumentError)</b> if the last segment is already fixed and the given port location contradicts the segment's fixed location
@see SegmentInfoBase#location
@see SegmentInfo#atStrongPortConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeInfo%23strongTargetPort Online Documentation}*/
strongTargetPort: YPoint | null;
static $class: Class;
private $_EdgeInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeInfo;
}
export interface EdgeRouter extends LayoutStageBase {
}
/**
This edge routing algorithm applies polyline routes to the edges of the graph.
<b>Layout Style</b>
Edges are by default routed in an orthogonal fashion, i.e., they only consist of horizontal and vertical segments. There
are two additional routing styles: the {@link EdgeRouterEdgeRoutingStyle#OCTILINEAR octilinear} style where additional sloped segments are inserted between horizontal
and vertical segments and the {@link EdgeRouterEdgeRoutingStyle#CURVED curved} style that replaces the segments with smooth curves.
During the routing process, the positions of the nodes are considered to be fixed and the routing algorithm will not
modify their locations or their sizes in any way.
The edge routing algorithm can be applied wherever it is needed to route the edges using orthogonal, octilinear or
curved segments without crossing any nodes, while keeping the positions of the nodes in the diagram fixed. Some
potential applications include electric circuit design, floor planning and navigation maps.
<b>Concept</b>
The edge routing algorithm basically performs three steps to achieve the main edge routing and an additional fourth step
for edges with the octilinear or curved routing style.
<ol>
<li>Creating a {@link IPartition} which divides the area of the graph area into several {@link PartitionCell}s.
<li>Finding the shortest/cheapest paths for all edges through the {@link IPartition} using {@link PathSearch}.
<li>Assigning coordinates to the segments of the edges based on the paths that were calculated before with
{@link ChannelBasedPathRouting}.
<li>Inserting non-orthogonal segments where horizontal and vertical segments meet for edges with routing style {@link EdgeRouterEdgeRoutingStyle#OCTILINEAR}
or inserting curved segments for edges with routing style {@link EdgeRouterEdgeRoutingStyle#CURVED}.
</ol>
The first two steps are customizable. {@link IGraphPartitionExtension}s are able to influence how the {@link IPartition}
is created. They add {@link PartitionCell}s and/or mark them for adding costs later in the process. The currently
used {@link EdgeRouter#registeredPartitionExtensions partition extensions} can be dropped or extended by custom implementations.
For example, the extension 'Node Partition' adds a {@link PartitionCell} to the {@link IPartition} for each
node and marks it as belonging to a node. During {@link PathSearch}, the extension 'Node Crossing' recognizes
these
{@link PartitionCell}s and adds costs that penalizes crossing a node. The edge will be routed around the nodes.
{@link PathSearchExtension}s influence the {@link PathSearch} by adding costs for traversing
{@link PartitionCell}s or narrowing their intervals to allow a less expensive traversal of a
{@link PartitionCell}. The currently used {@link EdgeRouter#registeredPathSearchExtensions partition extensions} can be dropped or extended by custom
implementations.
Using {@link EdgeRouterEdgeLayoutDescriptor}s, it is possible to add individual layout settings like routing styles to
edges. They are registered with the graph with key {@link EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}. If no
descriptor is provided for an edge, a {@link EdgeRouter#defaultEdgeLayoutDescriptor default edge layout descriptor} is used as fallback value.
<b>Features</b>
The routing algorithm supports two approaches to connect edges on a specific side or even on an exact location to a
node. {@link PortConstraint}s define a single constraint for the ports of an edge. To realize more complex port
restrictions, several {@link PortCandidate}s or {@link PortCandidateSet}s can be assigned to edges or nodes.
If an edge with registered {@link PortCandidate}s connects to nodes with {@link PortCandidateSet}s, the edge
router will try to match both collections in order to find an appropriate port. In case there is no matching port
candidate, a {@link PortCandidate} specified for the edge is preferred. Note that the routing algorithm doesn't
consider custom {@link IPortCandidateMatcher} implementations. Since their simultaneous existence at the same node
may be ambiguous, it is not recommended to use a combination of {@link PortConstraint}s and
{@link PortCandidate}s in the same diagram.
Strong {@link PortConstraint}s or fixed {@link PortCandidate}s defined for an edge end at a group node where
the strong/fixed port location is inside the group node are supported with the following characteristics. First of all,
if there are multiple {@link PortCandidate}s and one is not inside the group but, e.g., on its border, then such a
candidate is always preferred over an inner one. When an inner constraint is considered, then the algorithm actually
generates a proper route from the group's border to the port location. Obstacles on the way are considered like for any
other route. The group node is also entered at the side which is defined by the {@link PortConstraint} or
{@link PortCandidate}.
A {@link PartitionGrid} is respected in the way that the algorithm avoids that cell boundaries are left and
re-entered. This way, edge routes stay inside a cell if both source and target are in the same cell. For this feature to
work properly it is required that the values of the properties {@link ColumnDescriptor#originalPosition}, {@link RowDescriptor#originalPosition} {@link ColumnDescriptor#originalWidth} and {@link RowDescriptor#originalHeight} are correctly specified. This is usually
automatically the case when executing the {@link EdgeRouter} as a standalone algorithm via layout execution
convenience methods (e.g. the values are taken from the table visualization of the grid). However, if the router is
applied as part of a more complex layout pipeline it might be necessary to specify the values manually. For example, if
another algorithm previously computed the grid position values and stored them in the respective 'computed' properties
(e.g. {@link ColumnDescriptor#computedPosition}), and afterwards {@link EdgeRouter} should be applied, then the 'computed' values of the first algorithm
should be written to the 'original' values prior to the run of the {@link EdgeRouter}.
Edges can be grouped so that they share common segments at the beginning or end of their routes. Edge groups are
specified using {@link IDataProvider}s that provide the same ID object for all edges in the same group. Those
{@link IDataProvider}s are registered with the graph with key {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
for source groups or key {@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY} for target groups. Besides edge
grouping, the EdgeRouter also supports port grouping where all edges with the same port id at a node will share the same
port location but are still routed independently (i.e., do not share multiple segments as for edge groups). To specify
port groups use a {@link IDataProvider} and register it to the graph with key {@link PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY}
for source port groups or key {@link PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY} for target port groups. Note
that an edge can either be associated with a (source/target) group or a (source/target) port group id, but not both at
the same time.
Edges can be routed in a bus-style. Edges that should share common bus segments must be mapped to the same {@link EdgeRouterBusDescriptor}
(see also {@link EdgeRouter#BUS_DESCRIPTOR_DP_KEY}). The algorithm tries to route a large part of the edge using
the common bus segments. These segments are automatically selected depending on the involved edges/nodes, but they might
also be specified manually using
{@link EdgeRouterBusDescriptor#busPoints}. Edges that are fixed (i.e. are not marked for routing) may also belong to a bus. Then, the bus segments will be
derived using the existing path of the fixed edges. This way, buses can be incrementally updated, e.g., when a new edge
should be added to an existing bus structure. Bus routing can, e.g., be very useful in parts of a diagram where each
node is connected to each other node.
<h2>Note</h2>
All source/target grouped edges share the same source/target port. Hence, assigning them to different fixed port
locations (with {@link PortConstraint}s or {@link PortCandidate}s) doesn't make sense.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter Online Documentation}*/
export class EdgeRouter {
/**
Creates a new {@link EdgeRouter} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link EdgeRouter} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.maximumDuration] The time limit (in milliseconds) set for the edge routing algorithm. This option sets the {@link EdgeRouter#maximumDuration} property on the created object.
@param [options.integratedEdgeLabeling] Whether or not the layout algorithm will place edge labels. This option sets the {@link EdgeRouter#integratedEdgeLabeling} property on the created object.
@param [options.polylineRouting] Whether or not the routing algorithm will route the edges with non-orthogonal, octilinear segments. This option sets the {@link EdgeRouter#polylineRouting} property on the created object.
@param [options.preferredPolylineSegmentLength] The preferred length of (non-orthogonal) octilinear segments. This option sets the {@link EdgeRouter#preferredPolylineSegmentLength} property on the created object.
@param [options.maximumPolylineSegmentRatio] The maximum ratio between the horizontal/vertical part of a segment and the (non-orthogonal) octilinear part. This option sets the {@link EdgeRouter#maximumPolylineSegmentRatio} property on the created object.
@param [options.rerouting] Whether or not the routing algorithm uses an additional step to reroute the edges that are considered to have the worst paths. This option sets the {@link EdgeRouter#rerouting} property on the created object.
@param [options.scope] A (sub-)set of edges that shall be routed. This option sets the {@link EdgeRouter#scope} property on the created object.
@param [options.affectedNodesDpKey] The {@link IDataProvider} key to look up the selection state of the nodes. This option sets the {@link EdgeRouter#affectedNodesDpKey} property on the created object.
@param [options.affectedEdgesDpKey] The {@link IDataProvider} key to look up the selection state of the edges. This option sets the {@link EdgeRouter#affectedEdgesDpKey} property on the created object.
@param [options.edgeComparer] A custom {@link IComparer} to define the processing order of the edges. This option sets the {@link EdgeRouter#edgeComparer} property on the created object.
@param [options.considerNodeLabels] Whether or not the routing algorithm considers the labels of the nodes as obstacles when calculating the edge routes to avoid overlaps. This option sets the {@link EdgeRouter#considerNodeLabels} property on the created object.
@param [options.ignoreInnerNodeLabels] Whether or not this routing algorithm ignores node labels that are inside the bounds of their owner as obstacles for edge routes. This option sets the {@link EdgeRouter#ignoreInnerNodeLabels} property on the created object.
@param [options.considerEdgeLabels] Whether or not the routing algorithm considers as obstacles the edge labels that do not belong to the (sub-)set of edges to be routed when calculating the edge routes. This option sets the {@link EdgeRouter#considerEdgeLabels} property on the created object.
@param [options.grid] The {@link Grid} instance on which the routing algorithm places the orthogonal segments. This option sets the {@link EdgeRouter#grid} property on the created object.
@param [options.minimumNodeToEdgeDistance] The minimum distance between edges and node bounds. This option sets the {@link EdgeRouter#minimumNodeToEdgeDistance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
maximumDuration?: number;
integratedEdgeLabeling?: boolean;
polylineRouting?: boolean;
preferredPolylineSegmentLength?: number;
maximumPolylineSegmentRatio?: number;
rerouting?: boolean;
scope?: EdgeRouterScope | EdgeRouterScopeStringValues;
affectedNodesDpKey?: any | null;
affectedEdgesDpKey?: any | null;
edgeComparer?: IComparer<any> | null;
considerNodeLabels?: boolean;
ignoreInnerNodeLabels?: boolean;
considerEdgeLabels?: boolean;
grid?: Grid | null;
minimumNodeToEdgeDistance?: number;
});
/**
Performs the routing of the edges of the input graph.
This method also prepares the graph for the edge routing process.
<h2>Note</h2>
The given graph will not be copied during the edge routing process and the result will be immediately applied to the
input graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Checks the sizes of the nodes to be non-zero.
@param g The graph to check.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23checkNodeSize(LayoutGraph) Online Documentation}*/
checkNodeSize(g: LayoutGraph | null): void;
/**
Removes all registered {@link IGraphPartitionExtension}s from a given {@link GraphPartition} instance.
This method is called by {@link EdgeRouter#applyLayout} after the edge routes are calculated. It may be overridden
in order to provide a custom implementation for cleaning up a {@link GraphPartition} instance.
@param partition the given {@link GraphPartition} instance
@see EdgeRouter#configureGraphPartition
@see EdgeRouter#registeredPartitionExtensions
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23cleanUpGraphPartition(GraphPartition) Online Documentation}*/
cleanUpGraphPartition(partition: GraphPartition | null): void;
/**
Adds all registered {@link IGraphPartitionExtension}s instances to a given {@link GraphPartition} instance.
This method is called by {@link EdgeRouter#applyLayout} before the edge routes are calculated. It may be
overridden in order to adjust the configuration of the {@link GraphPartition} instance.
@param partition the given {@link GraphPartition} instance
@see EdgeRouter#registeredPartitionExtensions
@see EdgeRouter#cleanUpGraphPartition
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23configureGraphPartition(GraphPartition) Online Documentation}*/
configureGraphPartition(partition: GraphPartition | null): void;
/**
Adds all registered {@link PathSearchExtension}s to a given {@link PathSearch} instance.
This method is called by {@link EdgeRouter#applyLayout} before the edge routes are calculated. It may be
overridden in order to adjust the configuration of a {@link PathSearch} instance.
@param pathSearch a {@link PathSearch} instance
@see EdgeRouter#createPathSearch
@see EdgeRouter#registeredPathSearchExtensions
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23configurePathSearch(PathSearch) Online Documentation}*/
configurePathSearch(pathSearch: PathSearch | null): void;
/**
Creates a {@link PathSearchConfiguration} that is used during the path searching process.
This method is called by {@link EdgeRouter#applyLayout} before the edge routes are calculated. It may be
overridden in order to create a new {@link PathSearchConfiguration} object with custom configuration.
@param graph the input graph
@param grouping the grouping structure of the graph
@returns a {@link PathSearchConfiguration} instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23createConfiguration(LayoutGraph%2CLayoutGroupingSupport) Online Documentation}*/
createConfiguration(graph: LayoutGraph | null, grouping: LayoutGroupingSupport | null): PathSearchConfiguration | null;
/**
Creates a default {@link IComparer} instance to determine the order of the edges according to which they will be routed.
This method is called by {@link EdgeRouter#applyLayout} before the edge routes are calculated. It may be
overridden in order to create a new {@link IComparer} object with a custom configuration.
By default, this method returns an instance of the default implementation.
@param graph the input graph
@param configuration the given configuration for the path searching process
@returns a {@link IComparer} instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23createDefaultEdgeOrderComparer(LayoutGraph%2CPathSearchConfiguration) Online Documentation}*/
createDefaultEdgeOrderComparer(graph: LayoutGraph | null, configuration: PathSearchConfiguration | null): IComparer<any> | null;
/**
Creates a {@link GraphPartition} instance that divides the area of the graph into several rectangles.
This implementation creates a {@link GraphPartition} using the current {@link IObstaclePartition} instance.
This method is called by {@link EdgeRouter#applyLayout} before the edge routes are calculated. It may be
overridden in order to create a new {@link GraphPartition} object with a custom configuration.
@param decomposition the current {@link IObstaclePartition}
@returns a {@link GraphPartition} instance
@see EdgeRouter#configureGraphPartition
@see EdgeRouter#registeredPartitionExtensions
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23createGraphPartition(IObstaclePartition) Online Documentation}*/
createGraphPartition(decomposition: IObstaclePartition | null): GraphPartition | null;
/**
Creates a {@link DynamicObstacleDecomposition} that is used by the {@link GraphPartition} to divide the
graph area in rectangles.
This method is called by {@link EdgeRouter#applyLayout} before the edge routes are calculated. It may be
overridden in order to create a new {@link DynamicObstacleDecomposition} object with custom configuration.
@returns a {@link DynamicObstacleDecomposition} instance
@see EdgeRouter#createGraphPartition
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23createObstacleDecomposition() Online Documentation}*/
createObstacleDecomposition(): DynamicObstacleDecomposition | null;
/**
Creates a {@link ChannelBasedPathRouting} instance that routes the edges using pre-calculated {@link EdgeRouterPath}
objects.
This method is called by {@link EdgeRouter#applyLayout} before the edge routes are calculated. It may be
overridden in order to create a new {@link ChannelBasedPathRouting} object with custom configuration.
@returns a {@link ChannelBasedPathRouting} instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23createPathRouting() Online Documentation}*/
createPathRouting(): ChannelBasedPathRouting | null;
/**
Creates a {@link PathSearch} instance that finds the paths of the edges through the {@link GraphPartition}.
This method may be overridden in order to create a new {@link PathSearch} object with custom configuration.
@returns a {@link PathSearch} instance
@see EdgeRouter#configurePathSearch
@see EdgeRouter#registeredPathSearchExtensions
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23createPathSearch() Online Documentation}*/
createPathSearch(): PathSearch | null;
/**
Creates a {@link PathSearchContext} that provides context information for the path searching algorithm.
This method is called by {@link EdgeRouter#applyLayout} before the edge routes are calculated. It may be
overridden in order to create a new {@link PathSearchContext} object with custom configuration.
@param pathSearch a given {@link PathSearch} instance
@param configuration a given configuration for the path searching process
@returns a {@link PathSearchContext} instance
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23createPathSearchContext(PathSearch%2CPathSearchConfiguration) Online Documentation}*/
createPathSearchContext(pathSearch: PathSearch | null, configuration: PathSearchConfiguration | null): PathSearchContext | null;
/**
Returns the {@link EdgeRouterEdgeLayoutDescriptor} instance for a given edge that is provided by a {@link IDataProvider}
which is registered with the graph with key {@link EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
For all those edges that do not have a specific layout descriptor assigned, the default layout descriptor returned by {@link EdgeRouter#defaultEdgeLayoutDescriptor}
will be assigned.
This method may be overridden in order to create an {@link EdgeRouterEdgeLayoutDescriptor} with custom configuration.
@param edge the given edge
@returns the current {@link EdgeRouterEdgeLayoutDescriptor} instance for a given edge
@see EdgeRouter#defaultEdgeLayoutDescriptor
@see EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23getEdgeLayoutDescriptor(Edge) Online Documentation}*/
getEdgeLayoutDescriptor(edge: Edge | null): EdgeRouterEdgeLayoutDescriptor | null;
/**
Returns whether or not a given edge is selected.
If all the edges of the graph will be routed by {@link EdgeRouter}, i.e., the {@link EdgeRouter#scope scope} is set to
{@link EdgeRouterScope#ROUTE_ALL_EDGES}, this utility method returns <code>true</code> for all edges.
This method may be overridden in order to determine differently whether or not a given edge is considered to be
selected.
@param edge the given edge
@param graph the input graph
@returns <code>true</code> if the given edge is selected, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23isAffected(Edge%2CGraph) Online Documentation}*/
isAffected(edge: Edge | null, graph: Graph | null): boolean;
/**
Data provider key for specifying individual edge layout information
If this {@link IDataProvider} does not contain an {@link EdgeRouterEdgeLayoutDescriptor} for an edge, then the
layout algorithm will use the {@link EdgeRouter#defaultEdgeLayoutDescriptor default descriptor}.
@see EdgeRouter#defaultEdgeLayoutDescriptor
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23EDGE_LAYOUT_DESCRIPTOR_DP_KEY Online Documentation}*/
static EDGE_LAYOUT_DESCRIPTOR_DP_KEY: EdgeDpKey<EdgeRouterEdgeLayoutDescriptor> | null;
/**
Data provider key for specifying a bus descriptor for each edge
Edges can be assigned to a {@link EdgeRouterBusDescriptor} instance; all edges associated to the same descriptor
are routed in a bus-like style. A bus is a segment shared by multiple edges to which shorter segments that connect to
the actual nodes are attached. Observe that using such a bus representation with multiple edges drawn on top of each
other, information like the individual edge direction might be occluded.
The mapped {@link EdgeRouterBusDescriptor} allows to configure the bus formed by the associated edges.
<h2>Note</h2>
{@link EdgeRouterEdgeLayoutDescriptor#intermediateRoutingPoints Intermediate points} are ignored for edges that are part of a bus structure.
@see EdgeRouterBusDescriptor
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23BUS_DESCRIPTOR_DP_KEY Online Documentation}*/
static BUS_DESCRIPTOR_DP_KEY: EdgeDpKey<EdgeRouterBusDescriptor> | null;
/**
Data provider key for weighting the costs for crossing each label individually
If the factor for a label is <code>0</code> then it is allowed to cross it. Very important labels should get a high factor.
This factor is multiplied by the basic penalty arising when an edge must cross a {@link PenaltySettings#nodeLabelCrossingPenalty node label} or an {@link PenaltySettings#edgeLabelCrossingPenalty edge label} in order to
determine the final costs arising when this label is crossed.
@see EdgeRouter#defaultEdgeLayoutDescriptor
@see EdgeRouterEdgeLayoutDescriptor#penaltySettings
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23LABEL_CROSSING_PENALTY_FACTOR_DP_KEY Online Documentation}*/
static LABEL_CROSSING_PENALTY_FACTOR_DP_KEY: ILabelLayoutDpKey<number> | null;
/**
Gets or sets the time limit (in milliseconds) set for the edge routing algorithm.
The maximum duration has to be greater than or equal to <code>0</code>.
<h2>Note</h2>
Restricting the maximum duration may result in a lower layout quality. Furthermore, the real runtime may exceed the
maximum duration since the edge routing algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets the {@link EdgeRouterEdgeLayoutDescriptor} instance used for all those edges that do not have a specific edge layout
descriptor assigned.
@see EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23defaultEdgeLayoutDescriptor Online Documentation}*/
readonly defaultEdgeLayoutDescriptor: EdgeRouterEdgeLayoutDescriptor | null;
/**
Gets or sets whether or not the layout algorithm will place edge labels.
Enabling this feature, the routes of edges with labels can change significantly. The algorithm finds a position for
labels and routes the edge near the label trying to consider the
{@link PreferredPlacementDescriptor}. To do so, the route itself maybe needs to take a detour that might otherwise
not have been necessary. This especially holds true in case that there is very little space for the labels and/or the
labels are rather large.
<h2>Note</h2>
When this option is enabled, the overall runtime may increase significantly. Hence, if a short runtime is required or
the resulting routes have too many bend artifacts (mainly happens if there is little space), applying the {@link GenericLabeling generic labeling}
as post-processing step may be an alternative.
<h2>Note</h2>
When this option is enabled, property {@link EdgeRouter#considerEdgeLabels} is implicitly enabled, too.
<h2>Note</h2>
The integrated labeling cannot change the routes of edges with {@link EdgeRouterEdgeLayoutDescriptor#directGroupContentEdgeRouting direct group content routing} and, thus, labels of such
edges are more likely to overlap with other elements. The same is true for edges that are associated with a {@link EdgeRouterBusDescriptor bus}.
@see EdgeRouter#considerEdgeLabels
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23integratedEdgeLabeling Online Documentation}*/
integratedEdgeLabeling: boolean;
/**
Gets or sets whether or not the routing algorithm will route the edges with non-orthogonal, octilinear segments.
This property is deprecated! It is replaced by property {@link EdgeRouterEdgeLayoutDescriptor#routingStyle} on the edge layout descriptor. Use the routing styles {@link EdgeRouterEdgeRoutingStyle#ORTHOGONAL}
and {@link EdgeRouterEdgeRoutingStyle#OCTILINEAR} to switch between octilinear and orthogonal routing. With the descriptor,
the style can be specified for each edge individually, if desired.
This property only affects edges using the {@link EdgeRouter#defaultEdgeLayoutDescriptor default layout descriptor}. Changing this property value changes property {@link EdgeRouterEdgeLayoutDescriptor#routingStyle} of
the default edge layout descriptor instance.
@see EdgeRouter#preferredPolylineSegmentLength
@see EdgeRouter#maximumPolylineSegmentRatio
@deprecated Use the routing style property on the edge layout descriptor instead. See the documentation for details.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23polylineRouting Online Documentation}*/
polylineRouting: boolean;
/**
Gets or sets the preferred length of (non-orthogonal) octilinear segments.
This property is deprecated and replaced by
{@link EdgeRouterEdgeLayoutDescriptor#preferredOctilinearSegmentLength}. As the property is on the descriptor, it additionally allows to define the preferred length for each edge
individually.
This property only affects edges using the {@link EdgeRouter#defaultEdgeLayoutDescriptor default layout descriptor} and using routing style
{@link EdgeRouterEdgeRoutingStyle#OCTILINEAR}. Changing the value of this property changes property {@link EdgeRouterEdgeLayoutDescriptor#preferredOctilinearSegmentLength} of the default edge
layout descriptor instance.
@throws {Exception} <b>(ArgumentError)</b> if the preferred octilinear segment length is negative
@deprecated Use the respective preferred length property on the edge layout descriptor instead. See the documentation for details.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23preferredPolylineSegmentLength Online Documentation}*/
preferredPolylineSegmentLength: number;
/**
Gets or sets the maximum ratio between the horizontal/vertical part of a segment and the (non-orthogonal) octilinear
part.
This property is deprecated and replaced by
{@link EdgeRouterEdgeLayoutDescriptor#maximumOctilinearSegmentRatio}. As the property is on the descriptor, it additionally allows to define the maximum segment ratio each edge
individually.
This property only affects edges using the {@link EdgeRouter#defaultEdgeLayoutDescriptor default layout descriptor} and using routing style
{@link EdgeRouterEdgeRoutingStyle#OCTILINEAR}. Changing the value of this property changes property {@link EdgeRouterEdgeLayoutDescriptor#maximumOctilinearSegmentRatio} of the default edge
layout descriptor instance.
@throws {Exception} <b>(ArgumentError)</b> if the maximum segment length is negative or greater than <code>0.5</code>
@deprecated Use the respective maximum segment ratio property on the edge layout descriptor instead. See the documentation for
details.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23maximumPolylineSegmentRatio Online Documentation}*/
maximumPolylineSegmentRatio: number;
/**
Gets or sets whether or not the routing algorithm uses an additional step to reroute the edges that are considered to
have the worst paths.
<b>Warning:</b> The rerouting step will only be applied if the {@link EdgeRouter#maximumDuration maximum duration} has not been exceeded yet.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23rerouting Online Documentation}*/
rerouting: boolean;
/**
Gets or sets a (sub-)set of edges that shall be routed.
@throws {Exception} <b>(ArgumentError)</b> if the given scope is unknown
@see EdgeRouter#affectedEdgesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23scope Online Documentation}*/
scope: EdgeRouterScope;
/**
Gets or sets the {@link IDataProvider} key to look up the selection state of the nodes.
If the {@link EdgeRouter#scope scope} is set to
{@link EdgeRouterScope#ROUTE_EDGES_AT_AFFECTED_NODES}, only the edges that are incident to selected nodes will be routed,
while all other edges will be considered to have fixed routes.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IDataProvider} key is <code>null</code>
@see EdgeRouter#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23affectedNodesDpKey Online Documentation}*/
affectedNodesDpKey: any | null;
/**
Gets or sets the {@link IDataProvider} key to look up the selection state of the edges.
If the {@link EdgeRouter#scope scope} is set to
{@link EdgeRouterScope#ROUTE_AFFECTED_EDGES}, only the selected edges will be routed, while all other edges will be
considered to have fixed routes.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IDataProvider} key is <code>null</code>
@see EdgeRouter#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
/**
Gets or sets a custom {@link IComparer} to define the processing order of the edges.
<h2>Note</h2>
The processing order may affect the quality of the individual edge paths. When routing an edge, only the paths of the
already routed edges and of the fixed edges (that will not be routed at all) can be considered. Therefore, edges that
are processed first have to consider less other edge paths than the edges that are processed later, which might have to
use less optimal alternative paths.
<h2>Note</h2>
By default, this method returns a {@link IComparer} instance created with {@link EdgeRouter#createDefaultEdgeOrderComparer}.
<b>Warning:</b> When defining edge or port groups, the custom processing order defined by this property is not fully obeyed. Edges that
are grouped are routed together as a bundle. For example, assume that edge0 and edge2 are grouped and edge1 should be
routed after edge0 but before edge2: the grouped edges are routed as a bundle, either both before or both after edge1.
Note that the same is true for edges that share common bus segments, see
{@link EdgeRouter#BUS_DESCRIPTOR_DP_KEY}.
@see EdgeRouter#createDefaultEdgeOrderComparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23edgeComparer Online Documentation}*/
edgeComparer: IComparer<any> | null;
/**
Gets or sets whether or not the routing algorithm considers the labels of the nodes as obstacles when calculating the
edge routes to avoid overlaps.
<b>Warning:</b> This option takes effect only if {@link EdgeRouter#scope} is set to {@link EdgeRouterScope#ROUTE_EDGES_AT_AFFECTED_NODES} and a {@link IDataProvider}
instance with the key {@link EdgeRouter#affectedNodesDpKey} is registered with the graph.
@see PenaltySettings#nodeLabelCrossingPenalty
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets whether or not this routing algorithm ignores node labels that are inside the bounds of their owner as
obstacles for edge routes.
<b>Warning:</b> This setting can be applied only if node labels are {@link EdgeRouter#considerNodeLabels considered} and is especially useful in order to ignore inner node
labels of group nodes.
@see EdgeRouter#considerNodeLabels
@see PenaltySettings#nodeLabelCrossingPenalty
@see EdgeRouter#LABEL_CROSSING_PENALTY_FACTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23ignoreInnerNodeLabels Online Documentation}*/
ignoreInnerNodeLabels: boolean;
/**
Gets or sets whether or not the routing algorithm considers as obstacles the edge labels that do not belong to the
(sub-)set of edges to be routed when calculating the edge routes.
<h2>Note</h2>
If labels of routed edges should be placed, too, enable property {@link EdgeRouter#integratedEdgeLabeling}.
<b>Warning:</b> This option takes effect only if {@link EdgeRouter#scope} is set to {@link EdgeRouterScope#ROUTE_AFFECTED_EDGES} and a {@link IDataProvider}
instance with the key {@link EdgeRouter#affectedEdgesDpKey} is registered with the graph.
@see EdgeRouter#scope
@see EdgeRouter#affectedEdgesDpKey
@see PenaltySettings#edgeLabelCrossingPenalty
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23considerEdgeLabels Online Documentation}*/
considerEdgeLabels: boolean;
/**
Gets or sets the {@link Grid} instance on which the routing algorithm places the orthogonal segments.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23grid Online Documentation}*/
grid: Grid | null;
/**
Gets or sets the minimum distance between edges and node bounds.
The minimum distance should have a non-negative value.
@throws {Exception} <b>(ArgumentError)</b> if the minimum node-to-edge distance is negative
@see PenaltySettings#minimumNodeToEdgeDistancePenalty
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23minimumNodeToEdgeDistance Online Documentation}*/
minimumNodeToEdgeDistance: number;
/**
Gets a list of all registered {@link IGraphPartitionExtension}s.
{@link IGraphPartitionExtension}s can be added to a {@link GraphPartition} in order to create new
{@link Obstacle}s or can be removed from a {@link GraphPartition} instance.
By default, the following {@link IGraphPartitionExtension}s are registered with a given {@link GraphPartition}
instance:
<ul>
<li>Minimum Node To Edge Distance Partition
<li>Node Partition
<li>Partition Grid Partition
<li>Node Label Partition
<li>Edge Label Partition
<li>Fixed Edges Partition
<li>External Strong Port Restriction Partition
</ul>
@see EdgeRouter#createGraphPartition
@see EdgeRouter#configureGraphPartition
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23registeredPartitionExtensions Online Documentation}*/
readonly registeredPartitionExtensions: IList<any> | null;
/**
Gets a list of all registered {@link PathSearchExtension}s.
{@link PathSearchExtension}s can be added to a {@link PathSearch} instance in order to influence the path
searching process or can be removed from a {@link PathSearch} instance.
By default, the following {@link PathSearchExtension}s are registered with a {@link PathSearch} instance:
<ul>
<li>Fixed Grouped Edges
<li>Node Crossing
<li>Minimum Node To Edge Distance
<li>Group Node Crossing
<li>Minimum Group Node To Edge Distance
<li>Node Label Crossing
<li>Edge Label Crossing
<li>Bends In Node To Edge Distance
<li>Bend
<li>Monotonic Route
<li>Edge Length
<li>Partition Grid
<li>Port Restriction
<li>Edge Grouping
<li>Minimum Node Corner Distance
<li>Interval Based Crossing
<li>Minimum Edge To Edge Distance And Grid
<li>Minimum First Last Segment Length
<li>Intersecting Source And Target
</ul>
@see EdgeRouter#createPathSearch
@see EdgeRouter#configurePathSearch
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23registeredPathSearchExtensions Online Documentation}*/
readonly registeredPathSearchExtensions: IList<any> | null;
/**
Gets the {@link GraphPartition} instance used during the routing process.
<h2>Note</h2>
If the routing process is not running, <code>null</code> will be returned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouter%23partition Online Documentation}*/
readonly partition: GraphPartition | null;
static $class: Class;
private $_EdgeRouter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeRouter;
}
/**
Alignments for segments routed by the {@link EdgeRouter}.
@see SegmentInfoBase#preferredAlignment
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Alignment Online Documentation}*/
export enum Alignment {
/**
A constant specifying that the segment prefers to be placed close to the lower bound of its location range.
@see SegmentInfoBase#preferredAlignment
@see SegmentInfoBase#preferredAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/Alignment%23MIN Online Documentation}*/
MIN,
/**
A constant specifying that the segment prefers to be placed close to the upper bound of its location range.
@see SegmentInfoBase#preferredAlignment
@see SegmentInfoBase#preferredAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/Alignment%23MAX Online Documentation}*/
MAX,
/**
A constant specifying that the segment can be placed anywhere within its location range.
@see SegmentInfoBase#preferredAlignment
@see SegmentInfoBase#preferredAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/Alignment%23ANY Online Documentation}*/
ANY
}
/** Automatic conversion to {@link Alignment}. */
type AlignmentStringValues = "min" | "max" | "any";
/**
Routing types for segments routed by the {@link EdgeRouter}.
@see EdgeCellInfo#type
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RoutingType Online Documentation}*/
export enum RoutingType {
/**
A routing type representing a straight crossing of the
{@link PartitionCell partition cell}, i.e., a single orthogonal segment enters and exits the cell.
@see EdgeCellInfo#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/RoutingType%23STRAIGHT Online Documentation}*/
STRAIGHT,
/**
A routing type representing a crossing of the {@link PartitionCell partition cell} with three segments, i.e., the item enters and exits the
cell using the same direction but has an orthogonal middle segment between the entering and exiting segment.
@see EdgeCellInfo#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/RoutingType%23STRAIGHT_BENDING Online Documentation}*/
STRAIGHT_BENDING,
/**
A routing type representing a crossing of the {@link PartitionCell partition cell} with a single bend, i.e., the item enters the cell in one
direction, makes a bend and exits the cell in an orthogonal direction.
@see EdgeCellInfo#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/RoutingType%23BENDING Online Documentation}*/
BENDING,
/**
A routing type representing a crossing of the {@link PartitionCell partition cell} by making a U-turn, i.e., the item enters and exits the
cell on the same side using two turns and an orthogonal middle segment.
@see EdgeCellInfo#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/RoutingType%23U_TURN Online Documentation}*/
U_TURN,
/**
A routing type representing the source or target end of an item being in the cell, i.e., the enter and/or exit interval
is <code>null</code>.
@see EdgeCellInfo#type
@see {@link https://docs.yworks.com/yfileshtml/#/api/RoutingType%23END Online Documentation}*/
END
}
/** Automatic conversion to {@link RoutingType}. */
type RoutingTypeStringValues = "straight" | "straight-bending" | "bending" | "u-turn" | "end";
export interface GraphPartition extends YObject, IPartition, IDynamicDecomposition, IDecompositionListener {
}
/**
A {@link GraphPartition} partitions the bounding box of a graph into rectangles using the graph elements as
{@link Obstacle}s.
All {@link Obstacle}s used during the partitioning are {@link GraphPartition#createObstacle created} by {@link IGraphPartitionExtension}s.
{@link IGraphPartitionExtension}s have been {@link GraphPartition#addDynamicDecompositionListener registered} before using method {@link IGraphPartitionExtension#preparePartition}
which is called during the {@link GraphPartition#init initialization} of a {@link GraphPartition}.
The core partitioning is delegated to an {@link IObstaclePartition} that is {@link IObstaclePartition#init initialized} with these
{@link Obstacle}s.
If the {@link IObstaclePartition} is of type {@link IDynamicDecomposition}, {@link GraphPartition}
re-dispatches the decomposition notifications to its extensions.
@see IGraphPartitionExtension
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition Online Documentation}*/
export class GraphPartition {
/**
Creates a new {@link GraphPartition} instance using the given {@link GraphPartition#partition delegate inner obstacle partition}.
@param partition the delegate inner obstacle partition
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition Online Documentation}*/
constructor(partition: IObstaclePartition | null);
/**
Adds the given listener to the {@link GraphPartition}.
The listener will be notified upon dynamic decomposition events. If it is a {@link IGraphPartitionExtension},
method {@link IGraphPartitionExtension#preparePartition} is called during {@link GraphPartition#init initialization}.
@param listener the listener to be added
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23addDynamicDecompositionListener(IDecompositionListener) Online Documentation}*/
addDynamicDecompositionListener(listener: IDecompositionListener | null): void;
/**
Clears the {@link GraphPartition partition} data such that the {@link GraphPartition} can be reused and {@link GraphPartition#init initialized} with a new
configuration.
All {@link IGraphPartitionExtension}s are {@link IGraphPartitionExtension#cleanUp cleaned up}; partition listeners and obstacles are {@link GraphPartition#removeDynamicDecompositionListener removed}.
@see GraphPartition#init
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23clear() Online Documentation}*/
clear(): void;
/**
Creates and returns a new {@link Obstacle} using the given bounds and data.
All obstacles created via this method are delegated to the {@link GraphPartition#partition inner partition} upon {@link GraphPartition#init initialization}.
@param bounds the bounds of the obstacle
@param data the additional data that should be associated with the obstacle
@returns a new {@link Obstacle} using the given bounds and data
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23createObstacle(YRectangle%2CObject) Online Documentation}*/
createObstacle(bounds: YRectangle | null, data: any | null): Obstacle | null;
/**
Returns a list of all {@link PartitionCell}s that intersect or cover the given rectangle by delegating to the
corresponding method of the {@link GraphPartition#partition inner partition}.
@param rect the rectangular area whose (partially) covered cells will be returned
@returns a list containing (partially) covered {@link PartitionCell}s
@see IPartition#getCells
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23getCells(YRectangle) Online Documentation}*/
getCells(rect: YRectangle | null): IList<any> | null;
/**
Returns all {@link PartitionCell}s that are completely covered by the bounds of the given node.
@param node the node for which the covered cells will be returned
@returns an unmodifiable list of {@link PartitionCell} instances that are completely covered by the bounds of the given
node
@throws {Exception} <b>(ArgumentError)</b> if the node is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23getCellsForNode(YNode) Online Documentation}*/
getCellsForNode(node: YNode | null): IList<any> | null;
/**
Returns a list of all
{@link PartitionCell}s that are neighbors of the given cell, i.e., those cells that have a common border segment
with the given cell.
@param cell the cell whose neighbors will be returned
@returns a list of {@link PartitionCell}s that are neighbors of the given cell
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23getNeighbors(PartitionCell) Online Documentation}*/
getNeighbors(cell: PartitionCell | null): IList<any> | null;
/**
Returns a list of all {@link YNode}s whose bounds intersect or cover the bounds of the given cell.
@param cell the cell
@returns a list of {@link YNode}s that are intersected by the given cell
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23getNodes(PartitionCell) Online Documentation}*/
getNodes(cell: PartitionCell | null): IList<any> | null;
/**
Returns the {@link Obstacle} that has been created earlier for the given data object.
@param data the given data for which the obstacle will be returned
@returns the {@link Obstacle} that has been created earlier for the given data object
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23getObstacle(Object) Online Documentation}*/
getObstacle(data: any | null): Obstacle | null;
/**
Returns all {@link Obstacle}s covering the given {@link PartitionCell} by delegating to the corresponding
method of the {@link GraphPartition#partition inner partition}.
@param cell the cell whose obstacles will be returned
@returns a list of {@link Obstacle}s that cover the given cell
@see IObstaclePartition#getObstacles
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23getObstacles(PartitionCell) Online Documentation}*/
getObstacles(cell: PartitionCell | null): IList<any> | null;
/**
Initializes the {@link GraphPartition} with the given graph instance based on the given {@link PathSearchConfiguration configuration} used by path
search algorithms.
Before calling this method, any old partitioning information must be {@link GraphPartition#clear cleared}.
This initialization method calls {@link IGraphPartitionExtension#preparePartition} of all registered extensions.
Then, the {@link GraphPartition#partition inner delegate partition} is initialized passing all {@link GraphPartition#createObstacle added} obstacles as well as the given graph's bounds
extended by a small padding.
In the case where the {@link GraphPartition#partition delegate partition} is of type {@link IDynamicDecomposition}, this {@link GraphPartition}
will also be added as {@link IDecompositionListener} to it.
@param configuration the configuration used by path search algorithms
@see GraphPartition#clear
@see IGraphPartitionExtension#preparePartition
@see GraphPartition#addDynamicDecompositionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23init(PathSearchConfiguration) Online Documentation}*/
init(configuration: PathSearchConfiguration | null): void;
/**
Re-dispatches the event of the creation of a new {@link PartitionCell} to all registered
{@link IDecompositionListener}s.
This method is a callback after a {@link PartitionCell} has been created.
@param createdCell the newly created {@link PartitionCell}
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23onCellCreated(PartitionCell) Online Documentation}*/
onCellCreated(createdCell: PartitionCell | null): void;
/**
Re-dispatches the event of having completed the process of subdivisions of a given {@link PartitionCell} to all
registered {@link IDecompositionListener}s.
This method is a callback after a {@link PartitionCell} has been finalized and won't be further subdivided.
@param finalizedCell the cell that has been finalized
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23onCellFinalized(PartitionCell) Online Documentation}*/
onCellFinalized(finalizedCell: PartitionCell | null): void;
/**
Re-dispatches the event of the subdivision of a given {@link PartitionCell} into several sub-cells to all
registered {@link IDecompositionListener}s.
This method is a callback after a {@link PartitionCell} has been subdivided into several sub-cells.
@param cell the cell that has been subdivided
@param subCells the sub-cells resulting from the subdivision of the given {@link PartitionCell}
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23onCellSubdivided(PartitionCell%2CIList%3CObject%3E) Online Documentation}*/
onCellSubdivided(cell: PartitionCell | null, subCells: IList<any> | null | IListConvertible<any>): void;
/**
Re-dispatches the event of having unlocked a previously {@link GraphPartition#onCellFinalized finalized} {@link PartitionCell} to all registered
{@link IDecompositionListener}s.
This method is a callback after a previously finalized {@link PartitionCell} has been unlocked again.
@param unlockedCell the cell that has been unlocked
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23onCellUnlocked(PartitionCell) Online Documentation}*/
onCellUnlocked(unlockedCell: PartitionCell | null): void;
/**
Removes the given listener from the {@link GraphPartition}.
The listener will not be notified of dynamic decomposition events anymore.
@param listener the listener to be removed
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23removeDynamicDecompositionListener(IDecompositionListener) Online Documentation}*/
removeDynamicDecompositionListener(listener: IDecompositionListener | null): void;
/**
Gets the delegate inner obstacle partition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23partition Online Documentation}*/
readonly partition: IObstaclePartition | null;
/**
Gets the bounds of the original rectangular area that is being partitioned.
This method delegates to the corresponding method of the {@link GraphPartition#partition inner partition}.
@see IPartition#bounds
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartition%23bounds Online Documentation}*/
readonly bounds: YRectangle | null;
static $class: Class;
private $_GraphPartition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphPartition;
}
export interface GraphPartitionExtensionAdapter extends YObject, IGraphPartitionExtension {
/**
@inheritdoc*/
cleanUp(): void;
/**
@inheritdoc*/
onCellCreated(createdCell: PartitionCell | null): void;
/**
@inheritdoc*/
onCellFinalized(finalizedCell: PartitionCell | null): void;
/**
@inheritdoc*/
onCellSubdivided(cell: PartitionCell | null, subCells: IList<any> | null | IListConvertible<any>): void;
/**
@inheritdoc*/
onCellUnlocked(unlockedCell: PartitionCell | null): void;
} /**
{@link GraphPartitionExtensionAdapter} provides an empty implementation of
{@link IGraphPartitionExtension}.
The only real functionality is contained in method {@link IGraphPartitionExtension#preparePartition} which stores
the given {@link PathSearchConfiguration} and {@link GraphPartition} in properties.
@see GraphPartitionExtensionAdapter#configuration
@see GraphPartitionExtensionAdapter#graphPartition
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionExtensionAdapter Online Documentation}*/
export abstract class GraphPartitionExtensionAdapter {
/**
Creates a new instance of {@link GraphPartitionExtensionAdapter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionExtensionAdapter Online Documentation}*/
constructor();
/**
Stores the given {@link PathSearchConfiguration} and {@link GraphPartition} such that they are accessible
through {@link GraphPartitionExtensionAdapter#configuration} and {@link GraphPartitionExtensionAdapter#graphPartition}, respectively.
@param configuration the configuration used for the path search
@param graphPartition the graph partition that uses this extension
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionExtensionAdapter%23preparePartition(PathSearchConfiguration%2CGraphPartition) Online Documentation}*/
preparePartition(configuration: PathSearchConfiguration | null, graphPartition: GraphPartition | null): void;
/**
Gets the {@link GraphPartition} passed to {@link IGraphPartitionExtension#preparePartition}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionExtensionAdapter%23graphPartition Online Documentation}*/
readonly graphPartition: GraphPartition | null;
/**
Gets the configuration of the {@link PathSearch} passed to
{@link IGraphPartitionExtension#preparePartition}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphPartitionExtensionAdapter%23configuration Online Documentation}*/
readonly configuration: PathSearchConfiguration | null;
static $class: Class;
private $_GraphPartitionExtensionAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphPartitionExtensionAdapter;
}
export interface Grid extends YObject {
}
/**
This class provides support for routing edges on equidistant grid lines.
The grid is defined by its origin and the spacing between the equidistant grid lines.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Grid Online Documentation}*/
export class Grid {
/**
Creates a new {@link Grid} instance.
The spacing between the grid lines must be at least <code>1</code>. If the given spacing is smaller, value <code>1</code> is assumed.
@param originX the x-coordinate of the origin of the grid
@param originY the y-coordinate of the origin of the grid
@param spacing the spacing between the grid lines
@see {@link https://docs.yworks.com/yfileshtml/#/api/Grid Online Documentation}*/
constructor(originX: number, originY: number, spacing: number);
/**
Returns a string representation of the {@link Grid} object.
@returns a string representation of the {@link Grid} object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Grid%23toString() Online Documentation}*/
toString(): string;
/**
Gets the x-coordinate of the origin.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Grid%23originX Online Documentation}*/
readonly originX: number;
/**
Gets the y-coordinate of the origin.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Grid%23originY Online Documentation}*/
readonly originY: number;
/**
Gets the spacing between the horizontal and vertical grid lines.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Grid%23spacing Online Documentation}*/
readonly spacing: number;
static $class: Class;
private $_Grid: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Grid;
}
export interface IDynamicDecomposition {
}
/**
This interface should be implemented by classes that dispatch notifications about changes in {@link IPartition}s.
Such classes can be notified when {@link PartitionCell}s are created, divided, finalized and/or unlocked.
@see IDecompositionListener
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDynamicDecomposition Online Documentation}*/
export abstract class IDynamicDecomposition {
/**
Adds the given {@link IDecompositionListener} that will be notified of dynamic decomposition events to the
{@link IPartition}.
@param listener the listener to be added
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDynamicDecomposition%23addDynamicDecompositionListener(IDecompositionListener) Online Documentation}*/
abstract addDynamicDecompositionListener(listener: IDecompositionListener | null): void;
/**
Removes the given {@link IDecompositionListener} from the {@link IPartition}.
The {@link IDecompositionListener} will be no more notified of dynamic decomposition events.
@param listener the listener to be removed
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDynamicDecomposition%23removeDynamicDecompositionListener(IDecompositionListener) Online Documentation}*/
abstract removeDynamicDecompositionListener(listener: IDecompositionListener | null): void;
static $class: Class;
private $_IDynamicDecomposition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IDynamicDecomposition;
}
export interface IDecompositionListener {
}
/**
This interface is implemented by classes that require to be notified of changes in {@link IPartition}s.
Implementations are notified when {@link PartitionCell}s are created, divided, finalized and/or unlocked.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDecompositionListener Online Documentation}*/
export abstract class IDecompositionListener {
/**
Callback after the new given {@link PartitionCell} has been created.
@param createdCell the newly created {@link PartitionCell}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDecompositionListener%23onCellCreated(PartitionCell) Online Documentation}*/
abstract onCellCreated(createdCell: PartitionCell | null): void;
/**
Callback after the given {@link PartitionCell} has been finalized.
Before the cell may be further subdivided, it has to be {@link IDecompositionListener#onCellUnlocked unlocked} again.
@param finalizedCell the cell that has been finalized
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDecompositionListener%23onCellFinalized(PartitionCell) Online Documentation}*/
abstract onCellFinalized(finalizedCell: PartitionCell | null): void;
/**
Callback after the given {@link PartitionCell} has been subdivided into several sub-cells.
{@link IDecompositionListener} may not modify the list of sub-cells.
@param cell the cell that has been subdivided
@param subCells the sub-cells resulting from the subdivision of the given {@link PartitionCell}
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDecompositionListener%23onCellSubdivided(PartitionCell%2CIList%3CObject%3E) Online Documentation}*/
abstract onCellSubdivided(cell: PartitionCell | null, subCells: IList<any> | null | IListConvertible<any>): void;
/**
Callback after the given {@link PartitionCell} has been unlocked.
The cell had previously be {@link IDecompositionListener#onCellFinalized finalized} but has been unlocked again and may be {@link IDecompositionListener#onCellSubdivided subdivided} in the future.
@param unlockedCell the cell that has been unlocked
@see {@link https://docs.yworks.com/yfileshtml/#/api/IDecompositionListener%23onCellUnlocked(PartitionCell) Online Documentation}*/
abstract onCellUnlocked(unlockedCell: PartitionCell | null): void;
static $class: Class;
private $_IDecompositionListener: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IDecompositionListener;
}
export interface IEnterIntervalCalculator {
}
/**
This interface provides additional intervals in order to move from one {@link PartitionCell partition cell} into an adjacent one.
To find a {@link EdgeRouterPath} for an edge, the {@link PathSearch} algorithm moves from the current partition cell
to an adjacent partition cell. By default, the orthogonal interval shared by these two cells is used. Implementations of
this interface may be added to the {@link PathSearch} to calculate additional enter intervals that might
accumulate lower costs for entering the adjacent cell.
@see PartitionCell
@see PathSearch#addAdditionalEnterIntervalCalculator
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnterIntervalCalculator Online Documentation}*/
export abstract class IEnterIntervalCalculator {
/**
Appends additional enter intervals to the given list of enter intervals for moving from the {@link PartitionCell partition cell} of the
current {@link CellEntrance} to the neighboring cell.
@param currentEntrance the current {@link CellEntrance} that is exited
@param enteredCell the neighboring cell that will be entered
@param commonInterval the common interval of the left and entered partition cells
@param allEnterIntervals the list to which additional enter intervals have to be appended
@param context the context of the path search
@see {@link https://docs.yworks.com/yfileshtml/#/api/IEnterIntervalCalculator%23appendEnterIntervals(CellEntrance%2CPartitionCell%2COrthogonalInterval%2CIList%3CObject%3E%2CPathSearchContext) Online Documentation}*/
abstract appendEnterIntervals(currentEntrance: CellEntrance | null, enteredCell: PartitionCell | null, commonInterval: OrthogonalInterval | null, allEnterIntervals: IList<any> | null | IListConvertible<any>, context: PathSearchContext | null): void;
static $class: Class;
private $_IEnterIntervalCalculator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IEnterIntervalCalculator;
}
export interface IGraphPartitionExtension extends IDecompositionListener {
}
/**
This interface provides methods for extending the functionality of a {@link GraphPartition}.
A {@link IGraphPartitionExtension} can be used for {@link GraphPartition#createObstacle creating} new {@link Obstacle}s that will be given to the
initializer of the {@link IObstaclePartition} where they can react to the {@link IObstaclePartition}'s
decomposition events if it implements the {@link IDynamicDecomposition} interface.
A {@link IGraphPartitionExtension} has to be {@link GraphPartition#addDynamicDecompositionListener added} to the {@link GraphPartition} which will call method {@link IGraphPartitionExtension#preparePartition}
before the {@link IObstaclePartition} is initialized.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphPartitionExtension Online Documentation}*/
export abstract class IGraphPartitionExtension {
/**
Cleans up the extension from the last partitioning with the current {@link PathSearchConfiguration configuration} and {@link GraphPartition}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphPartitionExtension%23cleanUp() Online Documentation}*/
abstract cleanUp(): void;
/**
Adds a {@link IGraphPartitionExtension} to the {@link GraphPartition}.
It is called by the given {@link GraphPartition} upon initialization before its {@link IObstaclePartition}
is initialized. Implementations may use the graph upon which the given {@link GraphPartition} is based to
initialize themselves and/or to {@link GraphPartition#createObstacle add obstacles} to the list with which {@link IObstaclePartition} is initialized.
@param configuration the configuration used for the path search
@param graphPartition the graph partition that uses this extension
@see {@link https://docs.yworks.com/yfileshtml/#/api/IGraphPartitionExtension%23preparePartition(PathSearchConfiguration%2CGraphPartition) Online Documentation}*/
abstract preparePartition(configuration: PathSearchConfiguration | null, graphPartition: GraphPartition | null): void;
static $class: Class;
private $_IGraphPartitionExtension: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IGraphPartitionExtension;
}
export interface IObstaclePartition extends IPartition {
}
/**
This class implements a {@link IPartition} that takes a list of {@link Obstacle}s as input to subdivide an
area into {@link PartitionCell}s.
Implementations have to satisfy the following condition for each pair of {@link Obstacle} and
{@link PartitionCell}:
<ul>
<li>{@link Obstacle}s either completely cover {@link PartitionCell}s or do not intersect
{@link PartitionCell}s at all.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObstaclePartition Online Documentation}*/
export abstract class IObstaclePartition {
/**
Clears the partition data such that the {@link IObstaclePartition} can be reused and {@link IObstaclePartition#init initialized} with new
{@link Obstacle}s.
@see IObstaclePartition#init
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObstaclePartition%23clear() Online Documentation}*/
abstract clear(): void;
/**
Returns all {@link PartitionCell}s that are completely covered by the given {@link Obstacle}.
@param obstacle the obstacle for which the covered cells will be returned
@returns an unmodifiable list of {@link PartitionCell} instances that are completely covered by the given obstacle
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObstaclePartition%23getCellsForObstacle(Obstacle) Online Documentation}*/
abstract getCellsForObstacle(obstacle: Obstacle | null): IList<any> | null;
/**
Returns all {@link Obstacle}s that cover the given {@link PartitionCell}.
@param cell the partition cell for which the obstacles will be returned
@returns an unmodifiable list of {@link Obstacle} instances that cover the given cell
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObstaclePartition%23getObstacles(PartitionCell) Online Documentation}*/
abstract getObstacles(cell: PartitionCell | null): IList<any> | null;
/**
Initializes a new {@link IObstaclePartition} of the area with the given bounds using the given list of
{@link Obstacle}s.
Unused partitions must be cleared using method {@link IObstaclePartition#clear}.
@param obstacles a list of {@link Obstacle}s that have to be considered by the partition
@param bounds the bounds of the partition
@see IObstaclePartition#clear
@see {@link https://docs.yworks.com/yfileshtml/#/api/IObstaclePartition%23init(IList%3CObject%3E%2CYRectangle) Online Documentation}*/
abstract init(obstacles: IList<any> | null | IListConvertible<any>, bounds: YRectangle | null): void;
static $class: Class;
private $_IObstaclePartition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IObstaclePartition;
}
export interface IPartition {
}
/**
This class divides a rectangular area into one or more {@link PartitionCell}s.
{@link PartitionCell}s have to be disjoint. The union of all
{@link PartitionCell}s covers the original area.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartition Online Documentation}*/
export abstract class IPartition {
/**
Returns a list of all {@link PartitionCell}s that intersect or cover the given rectangle.
@param rect the rectangular area whose (partially) covered cells will be returned
@returns a list of {@link PartitionCell}s that (partially) cover the given rectangular area
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartition%23getCells(YRectangle) Online Documentation}*/
abstract getCells(rect: YRectangle | null): IList<any> | null;
/**
Returns a list of all
{@link PartitionCell}s that are neighbors of the given cell, i.e., those cells that have a common border segment
with the given cell.
@param cell the cell whose neighbors will be returned
@returns a list of {@link PartitionCell}s that are neighbors of the given cell
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartition%23getNeighbors(PartitionCell) Online Documentation}*/
abstract getNeighbors(cell: PartitionCell | null): IList<any> | null;
/**
Gets the bounds of the original rectangular area that is partitioned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IPartition%23bounds Online Documentation}*/
abstract readonly bounds: YRectangle | null;
static $class: Class;
private $_IPartition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IPartition;
}
export interface Interval extends YObject {
}
/**
This class represents a one-dimensional closed interval.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval Online Documentation}*/
export class Interval {
/**
Creates a new {@link Interval} between the given bounds.
The upper and lower bound of the two given values are determined automatically, i.e., the smaller one will be treated as
the lower bound and the larger one as the upper bound.
@param bound1 the first interval bound
@param bound2 the second interval bound
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval Online Documentation}*/
constructor(bound1: number, bound2: number);
/**
Returns the bridging {@link Interval interval} between the given {@link Interval intervals}.
The bridging interval is <code>null</code> if the given intervals intersect each other. Otherwise, the bridging interval is
<code>[minimum of upper bounds, maximum of lower bounds]</code>.
@param i1 the first interval
@param i2 the second interval
@returns the bridging interval between the given intervals or <code>null</code> if the intervals intersect
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23calculateBridge(Interval%2CInterval) Online Documentation}*/
static calculateBridge(i1: Interval | null, i2: Interval | null): Interval | null;
/**
Returns the intersection of the given {@link Interval intervals}.
The intersection is <code>null</code> if the given intervals do not intersect each other. Otherwise, the intersection is
<code>[maximum of lower bounds, minimum of upper bounds]</code>.
@param i1 the first interval
@param i2 the second interval
@returns the intersection of the given intervals or <code>null</code> if the intervals do not intersect
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23calculateIntersection(Interval%2CInterval) Online Documentation}*/
static calculateIntersection(i1: Interval | null, i2: Interval | null): Interval | null;
/**
Returns the spanning rectangle between the given {@link Interval intervals}.
The first interval spans the rectangle in horizontal dimension, the second interval in vertical dimension.
@param horizontal the interval that defines the location and length of the rectangle in horizontal dimension
@param vertical the interval that defines the location and length of the rectangle in vertical dimension
@returns the spanning rectangle between the given intervals
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23calculateSpanningRectangle(Interval%2CInterval) Online Documentation}*/
static calculateSpanningRectangle(horizontal: Interval | null, vertical: Interval | null): YRectangle | null;
/**
Returns the union of the given {@link Interval intervals} and the given value.
The union is between the minimum of the lower bound of the given interval and the given value and the maximum of the
upper bound of the given interval and the given value.
@param interval the interval to be extended
@param value the value to be in the union
@returns the union of the given interval and the given value
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23calculateUnion(Interval%2Cnumber) Online Documentation}*/
static calculateUnion(interval: Interval | null, value: number): Interval | null;
/**
Returns the union of the given {@link Interval intervals}.
The union is <code>[minimum of lower bounds, maximum of upper bounds]</code>.
@param i1 the first interval
@param i2 the second interval
@returns the union of the given intervals
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23calculateUnion(Interval%2CInterval) Online Documentation}*/
static calculateUnion(i1: Interval | null, i2: Interval | null): Interval | null;
/**
Checks whether or not the given value lies within the interval.
@param value the value to check
@returns <code>true</code> if the value lies within the interval, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23contains(number) Online Documentation}*/
contains(value: number): boolean;
/**
Checks whether or not the given {@link Interval interval} fully covers this {@link Interval interval} given a small epsilon value.
The given interval covers this interval, if <code>other lower bound - epsilon &lt;= this lower bound</code> and
<code>this upper bound &lt;= other upper bound + epsilon</code>.
@param other the other interval
@param [eps=1E-06] the epsilon value that determines the extended range of the other interval
@returns <code>true</code> if this interval is fully covered by the other interval, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23coveredBy(Interval%2Cnumber) Online Documentation}*/
coveredBy(other: Interval | null, eps?: number): boolean;
/**
Returns the distance between this {@link Interval interval} and the given {@link Interval interval}.
If the intervals {@link Interval#intersects intersect}, the distance is <code>0</code>. Otherwise, it is equal to
<code>dist(minimum of upper bounds, maximum of lower bounds)</code>.
@param other the other interval to which the distance is calculated
@returns the distance between this interval and the given interval
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23distanceTo(Interval) Online Documentation}*/
distanceTo(other: Interval | null): number;
/**
Returns the distance between the value and this {@link Interval interval}.
If this interval contains the specified value, the distance is <code>0</code>. Otherwise, it is the minimum of <code>dist(value, lower bound)</code>
and <code>dist(value, upper bound)</code>.
@param value the value to which the distance is calculated
@returns the distance between the value and the interval
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23distanceTo(number) Online Documentation}*/
distanceTo(value: number): number;
/**
Checks whether or not this {@link Interval interval} equals to the given {@link Interval interval}.
Two intervals are equal if they have the same lower and upper bounds.
@param other the other interval
@returns <code>true</code> if the two given intervals are equal, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Returns the {@link Interval interval} bound that is closest to the given value.
@param value the value for which the closest interval bound will be returned
@returns
<ul>
<li>the <code>value</code>, if the given value lies within the interval, or
<li>the lower bound, if the given value is smaller than or equal to the lower bound, or
<li>the upper bound, if the given value is greater than or equal to the upper bound
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23getClosest(number) Online Documentation}*/
getClosest(value: number): number;
/**
Returns the hash code for this {@link Interval interval}.
@returns the hash code for this interval
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Checks whether or not this {@link Interval interval} and the given {@link Interval interval} cover the same range.
The intervals are considered to cover the same range, if their minimum values as well as their maximum values differ
only in the given epsilon value.
@param other the other interval
@param [eps=1E-06] the maximum distance between the two minimum and the two maximum values
@returns <code>true</code> if the intervals cover the same range, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23hasSameRange(Interval%2Cnumber) Online Documentation}*/
hasSameRange(other: Interval | null, eps?: number): boolean;
/**
Checks whether or not this {@link Interval interval} intersects with the given
{@link Interval interval}, considering the given minimum intersection length.
@param other the other interval to check
@param [minIntersection=0] the minimum length of the intersection of the intervals
@returns <code>true</code> if the intervals intersect at least by the given minimum length, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23intersects(Interval%2Cnumber) Online Documentation}*/
intersects(other: Interval | null, minIntersection?: number): boolean;
/**
Checks whether or not the lower bound of this {@link Interval interval} is greater than the upper bound of the given interval.
@param other the interval to compare
@returns <code>true</code> if the current interval is on the right of the given one, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23isGreaterThan(Interval) Online Documentation}*/
isGreaterThan(other: Interval | null): boolean;
/**
Checks whether or not the upper bound of this {@link Interval interval} is smaller than the lower bound of the given interval.
@param other the interval to compare
@returns <code>true</code> if the current interval is on the left of the given one, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23isLessThan(Interval) Online Documentation}*/
isLessThan(other: Interval | null): boolean;
/**
Returns a string representation of this {@link Interval} object.
@returns a string representation of this {@link Interval} object
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23toString() Online Documentation}*/
toString(): string;
/**
Gets the lower bound of the interval.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23min Online Documentation}*/
readonly min: number;
/**
Gets the upper bound of the interval.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23max Online Documentation}*/
readonly max: number;
/**
Gets the midpoint of the interval.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23center Online Documentation}*/
readonly center: number;
/**
Gets the size of the interval, i.e., the difference between the {@link Interval#max upper bound} and the {@link Interval#min lower bound}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interval%23size Online Documentation}*/
readonly size: number;
static $class: Class;
private $_Interval: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Interval;
}
export interface Obstacle extends YObject {
}
/**
This class describes the obstacles that are used by implementations of {@link IObstaclePartition} to subdivide an
area into {@link PartitionCell}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Obstacle Online Documentation}*/
export class Obstacle {
/**
Creates a new {@link Obstacle} instance with the given bounds and additional data.
@param bounds the bounds of the obstacle
@param data the additional data associated to the obstacle
@see {@link https://docs.yworks.com/yfileshtml/#/api/Obstacle Online Documentation}*/
constructor(bounds: YRectangle | null, data: any | null);
/**
Gets the bounds of the obstacle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Obstacle%23bounds Online Documentation}*/
readonly bounds: YRectangle | null;
/**
Gets the additional data related to the obstacle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Obstacle%23data Online Documentation}*/
readonly data: any | null;
static $class: Class;
private $_Obstacle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Obstacle;
}
export interface OrthogonalInterval extends YObject {
}
/**
A horizontal or vertical {@link Interval} that additionally stores a y- (in the horizontal case) or x-coordinate
(in the vertical case).
During a {@link PathSearch}, orthogonal intervals are used for specifying the location where an edge enters or
exits a {@link PartitionCell partition cell}.
@see PathSearch
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval Online Documentation}*/
export class OrthogonalInterval {
/**
Creates a new {@link OrthogonalInterval} instance using the <code>from</code> and <code>to</code> value as interval bounds.
@param from the start of the interval bounds
@param to the end of the interval bounds
@param location the location in the two-dimensional coordinate system
@param isVertical <code>true</code> if the orthogonal interval is vertical, <code>false</code> if it is horizontal
@throws {Exception} <b>(ArgumentError)</b> if the given range is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23OrthogonalInterval(number%2Cnumber%2Cnumber%2Cboolean) Online Documentation}*/
constructor(from: number, to: number, location: number, isVertical: boolean);
/**
Creates a new {@link OrthogonalInterval} instance with the given interval bounds.
@param range the given range
@param location the location in the two-dimensional coordinate system
@param isVertical <code>true</code> if the orthogonal interval is vertical, <code>false</code> if it is horizontal
@throws {Exception} <b>(ArgumentError)</b> if the given range is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23OrthogonalInterval(Interval%2Cnumber%2Cboolean) Online Documentation}*/
constructor(range: Interval | null, location: number, isVertical: boolean);
/**
Calculates the intersection of the given {@link OrthogonalInterval orthogonal intervals}.
If the returned orthogonal interval is vertical, it will have the same horizontal location as <code>i2</code> and the shared vertical
range of both intervals. If it is horizontal, it will have the same vertical location as <code>i2</code> and the shared horizontal
range of both intervals. The orthogonal intervals must have same orientation.
@param i1 the first orthogonal interval
@param i2 the second orthogonal interval
@returns the shared orthogonal interval of both given orthogonal intervals
@throws {Exception} <b>(ArgumentError)</b> if the orthogonal intervals have different orientations
@see OrthogonalInterval#intersects
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23calculateIntersection(OrthogonalInterval%2COrthogonalInterval) Online Documentation}*/
static calculateIntersection(i1: OrthogonalInterval | null, i2: OrthogonalInterval | null): OrthogonalInterval | null;
/**
Calculates the union of the given {@link OrthogonalInterval orthogonal intervals}.
If the returned orthogonal interval is vertical, it will have the same horizontal location as <code>i2</code> and the shared vertical
range of both orthogonal intervals. If it is horizontal, it will have the same vertical location as <code>i2</code> and the shared
vertical range of both orthogonal intervals. The orthogonal intervals must have same orientation.
@param i1 the first orthogonal interval
@param i2 the second orthogonal interval
@returns the union of both given orthogonal intervals
@throws {Exception} <b>(ArgumentError)</b> if the orthogonal intervals have different orientations
@see OrthogonalInterval#intersects
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23calculateUnion(OrthogonalInterval%2COrthogonalInterval) Online Documentation}*/
static calculateUnion(i1: OrthogonalInterval | null, i2: OrthogonalInterval | null): OrthogonalInterval | null;
/**
Checks whether or not this {@link OrthogonalInterval orthogonal interval} is fully covered by the given {@link OrthogonalInterval orthogonal interval}.
The given orthogonal interval covers this orthogonal interval, if this interval's range is covered by the given
interval's range. The location of both orthogonal intervals is ignored. The orthogonal intervals must have the same
orientation.
@param other the other orthogonal interval
@returns <code>true</code> if this orthogonal interval is fully covered by the other orthogonal interval, <code>false</code> otherwise
@throws {Exception} <b>(ArgumentError)</b> if the orthogonal intervals have different orientations
@see PartitionCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23coveredBy(OrthogonalInterval) Online Documentation}*/
coveredBy(other: OrthogonalInterval | null): boolean;
/**
Determines whether or not this {@link OrthogonalInterval orthogonal interval} crosses the given one.
The orthogonal intervals must have different orientations.
@param other the other orthogonal interval
@returns <code>true</code> if this orthogonal interval crosses the given one, <code>false</code> otherwise
@throws {Exception} <b>(ArgumentError)</b> if the given orthogonal intervals have the same orientation
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23crosses(OrthogonalInterval) Online Documentation}*/
crosses(other: OrthogonalInterval | null): boolean;
/**
Returns the distance of the {@link OrthogonalInterval#range ranges} of this {@link OrthogonalInterval orthogonal interval} and the given {@link OrthogonalInterval orthogonal interval}.
The orthogonal intervals must have the same orientation.
@param other the other orthogonal interval
@returns <code>0</code> if the orthogonal intervals {@link OrthogonalInterval#intersects intersect} each other, otherwise the minimal distance between the orthogonal intervals is
returned
@throws {Exception} <b>(ArgumentError)</b> if the orthogonal intervals have different orientations
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23distanceTo(OrthogonalInterval) Online Documentation}*/
distanceTo(other: OrthogonalInterval | null): number;
/**
Checks whether or not this {@link OrthogonalInterval orthogonal interval} and the given {@link OrthogonalInterval orthogonal interval} cover the same range.
If both orthogonal intervals cover the same range, they are considered the same. The location of both orthogonal
intervals is ignored. The orthogonal intervals must have the same orientation.
@param other the other orthogonal interval
@returns <code>true</code> if the orthogonal intervals cover the same range, <code>false</code> otherwise
@throws {Exception} <b>(ArgumentError)</b> if the orthogonal intervals have different orientations
@see PartitionCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23hasSameRange(OrthogonalInterval) Online Documentation}*/
hasSameRange(other: OrthogonalInterval | null): boolean;
/**
Checks whether or not this {@link OrthogonalInterval orthogonal interval} intersects the given {@link OrthogonalInterval orthogonal interval} and the intersection has a given
minimum intersection size.
Orthogonal intervals intersect each other, if their ranges intersect each other by at least the given minimum
intersection size. The location of both orthogonal intervals is ignored. The orthogonal intervals must have the same
orientation.
@param other the other orthogonal interval
@param [minIntersection=0] the minimum size of the intersection
@returns <code>true</code> if this interval intersects the given other orthogonal interval, <code>false</code> otherwise
@throws {Exception} <b>(ArgumentError)</b> if the orthogonal intervals have different orientations
@see PartitionCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23intersects(OrthogonalInterval%2Cnumber) Online Documentation}*/
intersects(other: OrthogonalInterval | null, minIntersection?: number): boolean;
/**
Returns the Manhattan distance of this {@link OrthogonalInterval orthogonal interval} and the given {@link OrthogonalInterval orthogonal interval}.
If both intervals have the same orientation, the Manhattan distance is the sum of the distance between the intervals'
ranges and the distance between the intervals' locations.
If the intervals have different orientations, the Manhattan distance is the sum of the distances between one interval's
range and the other interval's location.
@param other the other orthogonal interval
@returns the Manhattan distance between this interval and the given one
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23manhattanDistanceTo(OrthogonalInterval) Online Documentation}*/
manhattanDistanceTo(other: OrthogonalInterval | null): number;
/**
Returns a string representation of this {@link OrthogonalInterval} object.
@returns a string representation of this {@link OrthogonalInterval} object
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23toString() Online Documentation}*/
toString(): string;
/**
Gets whether or not the {@link OrthogonalInterval orthogonal interval} is oriented vertically.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23vertical Online Documentation}*/
readonly vertical: boolean;
/**
Gets the lower bound of the {@link OrthogonalInterval orthogonal interval}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23min Online Documentation}*/
readonly min: number;
/**
Gets the upper bound of the {@link OrthogonalInterval orthogonal interval}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23max Online Documentation}*/
readonly max: number;
/**
Gets the length of the {@link OrthogonalInterval orthogonal interval}.
@see Interval#size
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23size Online Documentation}*/
readonly size: number;
/**
Gets the range of this {@link OrthogonalInterval orthogonal interval}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23range Online Documentation}*/
readonly range: Interval | null;
/**
Gets the vertical coordinate for horizontal intervals or the horizontal coordinate for vertical intervals,
respectively.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23location Online Documentation}*/
readonly location: number;
/**
Gets the center of this {@link OrthogonalInterval orthogonal interval} in the 2D-coordinate system.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalInterval%23center Online Documentation}*/
readonly center: YPoint | null;
static $class: Class;
private $_OrthogonalInterval: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalInterval;
}
export interface PartitionCellBorder extends YObject {
}
/**
This class is a type-safe enumeration used for defining the border of a {@link PartitionCell}.
@see PartitionCell#createBorderInterval
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder Online Documentation}*/
export class PartitionCellBorder {
/**
Returns the type of the border that is passed through when entering a {@link PartitionCell} in the given
direction.
@param direction the direction in which a {@link PartitionCell} has been entered
@returns the type of the border that is passed through when entering a {@link PartitionCell}
@see PartitionCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder%23valueOfEnterDirection(Direction) Online Documentation}*/
static valueOfEnterDirection(direction: Direction | null): PartitionCellBorder | null;
/**
Returns the type of the border that is passed through when exiting a {@link PartitionCell} in the given
direction.
@param direction the direction in which a {@link PartitionCell} has been exited
@returns the type of the border that is passed through when exiting a {@link PartitionCell}
@see PartitionCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder%23valueOfExitDirection(Direction) Online Documentation}*/
static valueOfExitDirection(direction: Direction | null): PartitionCellBorder | null;
/**
Returns the type of the border that is defined by the given {@link PortCandidate}.
The border type corresponds to the {@link PortCandidate#direction direction} of the given {@link PortCandidate}.
@param pc the port candidate for which the border type is returned
@returns the type of the border that is defined by the given port candidate
@see PortCandidate
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder%23valueOfPortCandidate(PortCandidate) Online Documentation}*/
static valueOfPortCandidate(pc: PortCandidate | null): PartitionCellBorder | null;
/**
Returns the type of the border that is defined by the given {@link PortConstraint}.
The border type corresponds to the {@link PortConstraint#side side} of the given {@link PortConstraint}.
@param pc the port constraint for which the border type is returned
@returns the type of the border that is defined by the given port constraint
@see PortConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder%23valueOfPortConstraint(PortConstraint) Online Documentation}*/
static valueOfPortConstraint(pc: PortConstraint | null): PartitionCellBorder | null;
/**
A type constant that defines the upper border of a {@link PartitionCell}.
@see PartitionCell#createBorderInterval
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder%23NORTH Online Documentation}*/
static NORTH: PartitionCellBorder | null;
/**
A type constant that defines the bottom border of a {@link PartitionCell}.
@see PartitionCell#createBorderInterval
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder%23SOUTH Online Documentation}*/
static SOUTH: PartitionCellBorder | null;
/**
A type constant that defines the right border of a {@link PartitionCell}.
@see PartitionCell#createBorderInterval
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder%23EAST Online Documentation}*/
static EAST: PartitionCellBorder | null;
/**
A type constant that defines the left border of a {@link PartitionCell}.
@see PartitionCell#createBorderInterval
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder%23WEST Online Documentation}*/
static WEST: PartitionCellBorder | null;
/**
Gets the type of the border that is on the opposite side of the border defined by this type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellBorder%23mirrorBorder Online Documentation}*/
readonly mirrorBorder: PartitionCellBorder | null;
static $class: Class;
private $_PartitionCellBorder: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionCellBorder;
}
export interface PartitionCell extends YObject {
}
/**
A {@link PartitionCell} represents a rectangular part of a {@link IPartition} as result of the decomposition
process.
Each instance of {@link PartitionCell} can hold arbitrary additional information (e.g. what element of the graph
is covered by a {@link PartitionCell}) which can be provided using method {@link PartitionCell#putData}.
Accessing and removing this information can be done using {@link PartitionCell#getData} and
{@link PartitionCell#removeData}, respectively.
@see PartitionCellKeys
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell Online Documentation}*/
export class PartitionCell {
/**
Creates a new {@link PartitionCell} instance of the given {@link IPartition} with the given bounds.
@param x the x-coordinate of the upper-left corner of the partition cell
@param y the y-coordinate of the upper-left corner of the partition cell
@param width the width of the partition cell
@param height the height of the partition cell
@param partition the partition to which the partition cell belongs
@see IPartition
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23PartitionCell(number%2Cnumber%2Cnumber%2Cnumber%2CIPartition) Online Documentation}*/
constructor(x: number, y: number, width: number, height: number, partition: IPartition | null);
/**
Creates a new {@link PartitionCell} of the given {@link IPartition} with the location and the size of the
given rectangle.
@param bounds the bounds describing the size and location of the partition cell
@param partition the partition to which the partition cell belongs
@see IPartition
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23PartitionCell(YRectangle%2CIPartition) Online Documentation}*/
constructor(bounds: YRectangle | null, partition: IPartition | null);
/**
Clears all additional data for this {@link PartitionCell}.
@see PartitionCell#getData
@see PartitionCell#putData
@see PartitionCell#removeData
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23clearData() Online Documentation}*/
clearData(): void;
/**
Returns an {@link OrthogonalInterval} that defines the location, the size and the orientation of the given
{@link PartitionCellBorder}.
The values defining the border are:
<ul>
<li>{@link PartitionCellBorder#WEST}
<li>{@link PartitionCellBorder#EAST}
<li>{@link PartitionCellBorder#NORTH}
<li>{@link PartitionCellBorder#SOUTH}
</ul>
@param border a border of this partition cell
@returns an {@link OrthogonalInterval} that defines the location, the size and the orientation of the given border
@see PartitionCellBorder
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23createBorderInterval(PartitionCellBorder) Online Documentation}*/
createBorderInterval(border: PartitionCellBorder | null): OrthogonalInterval | null;
/**
Returns the additional data associated with the given key for this {@link PartitionCell}.
@param key the key whose associated data will be returned
@returns the additional data associated with the given key or <code>null</code> if there is no data associated with the given key
@see PartitionCell#putData
@see PartitionCell#removeData
@see PartitionCell#clearData
@see PartitionCellKeys
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23getData(Object) Online Documentation}*/
getData(key: any | null): any | null;
/**
Stores the additional data associated with the given key for this {@link PartitionCell}.
@param key the key with which the additional data will be associated
@param data the additional data
@returns the previous data associated with the given key or <code>null</code> if there was no data associated with the given key
@see PartitionCell#getData
@see PartitionCell#removeData
@see PartitionCell#clearData
@see PartitionCellKeys
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23putData(Object%2CObject) Online Documentation}*/
putData(key: any | null, data: any | null): any | null;
/**
Removes the additional data associated with the given key for this {@link PartitionCell}.
@param key the key for which the associated data will be removed
@returns the additional data associated with the given key or <code>null</code> if there is no data associated with the given key
@see PartitionCell#getData
@see PartitionCell#putData
@see PartitionCell#clearData
@see PartitionCellKeys
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23removeData(Object) Online Documentation}*/
removeData(key: any | null): any | null;
/**
Gets the {@link IPartition partition} to which this {@link PartitionCell} belongs.
@see IPartition
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23partition Online Documentation}*/
readonly partition: IPartition | null;
/**
Gets a unique identifier of this {@link PartitionCell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23id Online Documentation}*/
readonly id: number;
/**
Gets the bounds of this {@link PartitionCell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23bounds Online Documentation}*/
readonly bounds: YRectangle | null;
/**
Gets the x-coordinate of the left border of this {@link PartitionCell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23minX Online Documentation}*/
readonly minX: number;
/**
Gets the y-coordinate of the upper border of this {@link PartitionCell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23minY Online Documentation}*/
readonly minY: number;
/**
Gets the x-coordinate of the right border of this {@link PartitionCell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23maxX Online Documentation}*/
readonly maxX: number;
/**
Gets the y-coordinate of the bottom border of this {@link PartitionCell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23maxY Online Documentation}*/
readonly maxY: number;
/**
Gets the width of this {@link PartitionCell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23width Online Documentation}*/
readonly width: number;
/**
Gets the height of this {@link PartitionCell}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCell%23height Online Documentation}*/
readonly height: number;
static $class: Class;
private $_PartitionCell: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionCell;
}
export interface PartitionCellKeys extends YObject {
}
/**
This class defines keys to obtain additional information about a {@link PartitionCell}.
In order to get the data of the {@link PartitionCell} that is associated with the given key, method {@link PartitionCell#getData}
should be used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys Online Documentation}*/
export class PartitionCellKeys {
/**
The key references a {@link IList} of nodes whose bounds are covering a certain {@link PartitionCell}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys%23NODES_KEY Online Documentation}*/
static NODES_KEY: any | null;
/**
The key references a {@link IList} of nodes being in node to edge distance to a certain {@link PartitionCell}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys%23NODES_IN_NODE_TO_EDGE_DISTANCE_KEY Online Documentation}*/
static NODES_IN_NODE_TO_EDGE_DISTANCE_KEY: any | null;
/**
The key references a {@link IList} of {@link INodeLabelLayout}s covering a certain {@link PartitionCell}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys%23NODE_LABEL_LAYOUTS_KEY Online Documentation}*/
static NODE_LABEL_LAYOUTS_KEY: any | null;
/**
The key references a {@link IList} of {@link number} values representing factors that are multiplied by an edge's {@link PenaltySettings#nodeLabelCrossingPenalty node label crossing costs} in case
the edge crosses the corresponding {@link INodeLabelLayout node label} registered with key
{@link PartitionCellKeys#NODE_LABEL_LAYOUTS_KEY}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys%23NODE_LABEL_CROSSING_COST_FACTORS_KEY Online Documentation}*/
static NODE_LABEL_CROSSING_COST_FACTORS_KEY: any | null;
/**
The key references a {@link IList} of {@link IEdgeLabelLayout}s covering a certain {@link PartitionCell}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys%23EDGE_LABEL_LAYOUTS_KEY Online Documentation}*/
static EDGE_LABEL_LAYOUTS_KEY: any | null;
/**
The key references a {@link IList} of {@link number} values representing factors that are multiplied by an edge's {@link PenaltySettings#edgeLabelCrossingPenalty edge label crossing costs} in case
the edge crosses the corresponding {@link IEdgeLabelLayout edge label} registered with key
{@link PartitionCellKeys#EDGE_LABEL_LAYOUTS_KEY}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys%23EDGE_LABEL_CROSSING_COST_FACTORS_KEY Online Documentation}*/
static EDGE_LABEL_CROSSING_COST_FACTORS_KEY: any | null;
/**
The key references the {@link PartitionCellId} of the {@link PartitionGrid} cell covering a certain
{@link PartitionCell}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys%23PARTITION_GRID_CELL_ID_KEY Online Documentation}*/
static PARTITION_GRID_CELL_ID_KEY: any | null;
/**
The key references the row index of the {@link PartitionGrid} cell covering a certain
{@link PartitionCell}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys%23PARTITION_GRID_ROW_INDEX_KEY Online Documentation}*/
static PARTITION_GRID_ROW_INDEX_KEY: any | null;
/**
The key references the column index of the {@link PartitionGrid} cell covering a certain
{@link PartitionCell}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionCellKeys%23PARTITION_GRID_COLUMN_INDEX_KEY Online Documentation}*/
static PARTITION_GRID_COLUMN_INDEX_KEY: any | null;
static $class: Class;
private $_PartitionCellKeys: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionCellKeys;
}
export interface EdgeRouterPath extends YObject {
}
/**
A {@link EdgeRouterPath} represents the result of a path search as a list of consecutive {@link CellEntrance}s.
A path starts with a {@link CellEntrance} for a {@link PartitionCell} of the source node and ends with one
for a {@link PartitionCell} of the target node.
@see PathSearch
@see PathSearchResult
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterPath Online Documentation}*/
export class EdgeRouterPath {
/**
Creates a new instance of {@link EdgeRouterPath} for the given edge, {@link CellEntrance cell entrances} and cost.
@param entrances the list of {@link CellEntrance} objects describing the edge path from source node to target node
@param cost the overall cost of this path
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterPath%23EdgeRouterPath(IList%3CObject%3E%2Cnumber) Online Documentation}*/
constructor(entrances: IList<any> | null | IListConvertible<any>, cost: number);
/**
Returns the {@link CellEntrance} at the given position of this path.
@param position the position of the {@link CellEntrance} to return
@returns the {@link CellEntrance} at the given position of this path
@see EdgeRouterPath#setEntrance
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterPath%23getEntrance(number) Online Documentation}*/
getEntrance(position: number): CellEntrance | null;
/**
Returns the number of {@link CellEntrance} objects in this path.
@returns the number of {@link CellEntrance} objects in this path
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterPath%23length() Online Documentation}*/
length(): number;
/**
Returns the position of the first occurrence of the given {@link CellEntrance} in this path, or <code>-1</code> if this path
does not contain this {@link CellEntrance}.
@param entrance the {@link CellEntrance} for which the position is retrieved
@returns the position of the first occurrence of the given {@link CellEntrance} in this path, or <code>-1</code> if this path does not
contain this {@link CellEntrance}
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterPath%23positionOf(CellEntrance) Online Documentation}*/
positionOf(entrance: CellEntrance | null): number;
/**
Replaces the {@link CellEntrance} at the given position of this path with the given {@link CellEntrance}.
@param position the position of the {@link CellEntrance} to replace
@param entrance the {@link CellEntrance} to be stored at the given position
@see EdgeRouterPath#getEntrance
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterPath%23setEntrance(number%2CCellEntrance) Online Documentation}*/
setEntrance(position: number, entrance: CellEntrance | null): void;
/**
Gets the overall cost of this path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeRouterPath%23cost Online Documentation}*/
readonly cost: number;
static $class: Class;
private $_Path: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeRouterPath;
}
export interface PathRequest extends YObject {
}
/**
A {@link PathRequest} describes a routing request from a {@link PathRequest#source source obstacle} to a {@link PathRequest#target target obstacle} which is handled and
considered during a {@link PathSearch}.
The path that is searched for an edge is potentially split into multiple requests. The path finding algorithm finds a
path for each request and, finally, the results are put together to a final, full path for the actual edge. Multiple
request are introduced, for example, to model intermediate routing points.
@see PathSearchExtension#initializeCurrentPathRequest
@see PathSearchExtension#cancelCurrentPathRequest
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathRequest Online Documentation}*/
export class PathRequest {
/**
Gets the source obstacle describing the area where this request starts.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathRequest%23source Online Documentation}*/
readonly source: Obstacle | null;
/**
Gets the target obstacle describing the area where this request ends.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathRequest%23target Online Documentation}*/
readonly target: Obstacle | null;
/**
Gets the {@link EdgeRouterEdgeLayoutDescriptor} that describes routing settings for this path request.
When implementing {@link PathSearchExtension}s and using descriptor settings, always this descriptor belonging to
the current request should be queried. It may be different from the one originally registered for the edge when there
are multiple {@link PathRequest}s for an edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathRequest%23descriptor Online Documentation}*/
readonly descriptor: EdgeRouterEdgeLayoutDescriptor | null;
/**
Gets the source
{@link PortCandidate}s associated with this path request and describing how the path must connect to the source
obstacle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathRequest%23sourcePortCandidates Online Documentation}*/
readonly sourcePortCandidates: ICollection<any> | null;
/**
Gets the target
{@link PortCandidate}s associated with this path request and describing how the path must connect to the target
obstacle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathRequest%23targetPortCandidates Online Documentation}*/
readonly targetPortCandidates: ICollection<any> | null;
/**
Gets the path that was found for this path request.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathRequest%23foundPath Online Documentation}*/
readonly foundPath: EdgeRouterPath | null;
static $class: Class;
private $_PathRequest: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PathRequest;
}
export interface PathSearch extends YObject {
}
/**
This is a pathfinding algorithm that calculates the shortest (i.e., the cheapest) paths for a set of edges through a
{@link GraphPartition}.
It is based on an <a href="http://en.wikipedia.org/wiki/A*_search_algorithm">A*-algorithm</a> and uses {@link PartitionCell}s as steps between source node and target node.
In each step, the algorithm takes a {@link CellEntrance}, that consists mainly of a {@link PartitionCell}
and from where it was entered, from a queue with all seen {@link CellEntrance}s, determines all possible neighbor
cells and their enter intervals and enqueues the resulting {@link CellEntrance}. To influence the order in which
the {@link CellEntrance}s will be processed, the path search assigns real costs (like Dijkstra) as well as
heuristic costs (A*-algorithm's heuristic) to the enqueued {@link CellEntrance}s. The real costs arise from
entering a neighbor cell, e.g., a bend has to be created, while the heuristic costs are an estimation of how expensive
it will be to reach the target node continuing the path with this neighboring cell. Therefore, the path search prefers
searching in the direction in which the target node lies. The {@link CellEntrance} with the lowest combined costs
is processed next until the target cell is reached.
{@link PathSearchExtension}s modify the path search as they are able to add start entrances and weight them with
costs. They also add real and heuristic costs to {@link CellEntrance}s that are created for the currently entered
{@link PartitionCell} and calculate shorter enter intervals that are less expensive to pass.
The algorithm gets a {@link PathSearchContext} which provides information about the graph and the currently routed
edge. It stores the results of the path search that can then be retrieved calling {@link PathSearchContext#pathSearchResult}.
@see PathSearch#addPathSearchExtension
@see PathSearch#addAdditionalEnterIntervalCalculator
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch Online Documentation}*/
export class PathSearch {
/**
Creates a new instance of {@link PathSearch}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch Online Documentation}*/
constructor();
/**
Adds a new interval calculator to the list of registered {@link IEnterIntervalCalculator}s.
{@link IEnterIntervalCalculator}s may add
{@link CellEntrance}s with narrowed intervals to the queue. The interval is more specific and will be judged with
lower costs.
@param enterIntervalCalculator the calculator to add
@returns <code>true</code> if the calculator was successfully added, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23addAdditionalEnterIntervalCalculator(IEnterIntervalCalculator) Online Documentation}*/
addAdditionalEnterIntervalCalculator(enterIntervalCalculator: IEnterIntervalCalculator | null): boolean;
/**
Adds the given extension to the list of {@link PathSearchExtension}s.
An extension can influence the pathfinding process by adding costs for entering certain {@link CellEntrance}s.
@param extension the extension to add to this path search
@returns <code>true</code> if the extension has been added, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23addPathSearchExtension(PathSearchExtension) Online Documentation}*/
addPathSearchExtension(extension: PathSearchExtension | null): boolean;
/**
Calculates the costs for moving from the current {@link CellEntrance} to the neighboring {@link PartitionCell}
using different enter intervals.
It is called by {@link PathSearch#handleNeighbor} to determine the costs for all
{@link CellEntrance}s that it will create and enqueue afterwards.
The costs for the given enter intervals are retrieved from all registered
{@link PathSearchExtension}s. The calculation stops when it reaches the given maximum cost value.
@param currentEntrance the current cell entrance
@param enteredCell the partition cell to enter
@param enterIntervals the different entering intervals of the entered cell
@param lastEdgeCellInfos the information about how the last cell was crossed
@param context the context information
@param costs the array in which the calculated costs for entering the neighbor cell via the according enter intervals shall be
written
@param maxAllowedCosts the maximum costs an enter interval may induce. If this cost is exceeded, no further additional costs for this interval
are calculated. Note that the entries in this array get modified during cost calculation
@see PathSearchExtension#calculateCosts
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23calculateCosts(CellEntrance%2CPartitionCell%2CArray%3COrthogonalInterval%3E%2CArray%3CEdgeCellInfo%3E%2CPathSearchContext%2CArray%3Cnumber%3E%2CArray%3Cnumber%3E) Online Documentation}*/
calculateCosts(currentEntrance: CellEntrance | null, enteredCell: PartitionCell | null, enterIntervals: OrthogonalInterval[] | null, lastEdgeCellInfos: EdgeCellInfo[] | null, context: PathSearchContext | null, costs: number[] | null, maxAllowedCosts: number[] | null): void;
/**
Returns the estimated costs for the rest of the path when using the given {@link CellEntrance} for the next step
in the path search.
It is called by {@link PathSearch#handleNeighbor} to determine the heuristic part of the costs with which the
entrance will be enqueued.
The heuristic costs for the given entrance are retrieved from all registered {@link PathSearchExtension}s.
@param entrance the current entrance
@param context the context information
@returns the heuristic costs for the rest of the path if the given entrance is used
@see PathSearchExtension#calculateHeuristicCosts
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23calculateHeuristicCosts(CellEntrance%2CPathSearchContext) Online Documentation}*/
calculateHeuristicCosts(entrance: CellEntrance | null, context: PathSearchContext | null): number;
/**
Resets all registered {@link PathSearchExtension}s and {@link DataProviders} added to this
{@link PathSearch}.
So, {@link PathSearch} is ready to calculate paths for a new layout.
@see PathSearchExtension#cleanUp
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23clear() Online Documentation}*/
clear(): void;
/**
Decreases the given penalty settings for the current edge.
If finding a path for the current edge takes too long according to the {@link EdgeRouter#maximumDuration maximum duration} of the {@link EdgeRouter edge routing algorithm},
the path search for the current edge is canceled and restarted using decreased penalties. The <code>decreaseFactor</code> indicates,
how much the penalties shall be reduced.
This method is called by
{@link PathSearch#findPathsForCurrentEdge}. If overriding this method, note that the penalty for creating bends
should not be reduced as this results in more possible turns of the edge path and, therefore, a longer runtime of the
path search. Furthermore, not all penalties should be decreased equally as these decreases would neutralize each other.
The <code>decreaseFactor</code> takes values from <code>[0,1]</code>, where <code>0</code> means no reduction while <code>1</code> means the strongest reduction.
@param penaltySettings the penalty settings whose penalties shall be reduced
@param decreaseFactor the factor with values between <code>0</code> and <code>1</code> that indicates how strong to reduce the penalties
@param context the context information of the current path search
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23decreasePenaltySettings(PenaltySettings%2Cnumber%2CPathSearchContext) Online Documentation}*/
decreasePenaltySettings(penaltySettings: PenaltySettings | null, decreaseFactor: number, context: PathSearchContext | null): void;
/**
Informs all registered path search extensions about completing a path by calling their {@link PathSearchExtension#finalizePath finalizePath(Path)} method.
That way, extensions can collect data about this path to use it later during path search.
This method is called by {@link PathSearch#findPathsForCurrentEdge} and may be overridden to use a custom
finalization step.
@param path the path to finalize
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23finalizePath(EdgeRouterPath) Online Documentation}*/
finalizePath(path: EdgeRouterPath | null): void;
/**
Finds paths for the edges in the given context and stores them in its {@link PathSearchContext#pathSearchResult PathSearchResult}.
This is the main method of {@link PathSearch}.
It initializes its extensions using {@link PathSearchExtension#initializeEdges} and delegates the path search for
each edge to {@link PathSearch#findPathsForCurrentEdge}.
The path calculations for all edges are finalized by calling the extensions' {@link PathSearchExtension#finalizeEdges}
method and, after that, the path search result is filled with the path for each edge.
At last, the extensions are asked to finalize the path search result using their {@link PathSearchExtension#finalizePathSearchResult}
callback.
@param context the context to use during the path search
@see PathSearchContext#edges
@see PathSearchContext#pathSearchResult
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23findPaths(PathSearchContext) Online Documentation}*/
findPaths(context: PathSearchContext | null): void;
/**
Finds the path for the current edge in the given context.
This method:
<ul>
<li>Calls {@link PathSearchExtension#initializeCurrentEdge} for all extensions.
<li>Collects and enqueues all start entrances.
<li>Iteratively processes the next cheapest cell entrance and
<ul>
<li>adds a path result after a {@link PathSearch#finalizePath} call if it is a valid target entrance and/or
<li>calls {@link PathSearch#handleNeighbor} for all neighbor cells of the entered cell.
</ul>
<li>Calls {@link PathSearchExtension#finalizeCurrentEdge} for all extensions.
</ul>
It is called by {@link PathSearch#findPaths} and may be overridden to skip certain edges or implement a custom
path search.
@param context the context information needed for finding a path
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23findPathsForCurrentEdge(PathSearchContext) Online Documentation}*/
findPathsForCurrentEdge(context: PathSearchContext | null): void;
/**
Returns the path for the given edge if it has already been finalized.
The path is finalized if the {@link PathSearch} chose it as the best result for the edge.
@param edge the edge for which the path is returned
@returns the finalized path for the given edge or <code>null</code> if no path has been found and finalized
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23getFinalizedPath(Edge) Online Documentation}*/
getFinalizedPath(edge: Edge | null): EdgeRouterPath | null;
/**
Adds
{@link CellEntrance}s for every interval through which the neighboring cell can be entered from the current
entrance to the queue.
The algorithm calls this method in every step for each neighbor of the current cell to collect all next possible
entrances for the current path. This path consists of several entrances where each knows the entrance through which it
has been entered.
After calculating all possible enter intervals to the given neighboring cell, each interval gets rated with costs. If
there is already an entrance for the neighboring cell whose interval is the same with one of these intervals, this
entrance will be used and re-enqueued, so that the path search can still reach it. The current entrance is set as its
predecessor within the current path and its enter interval and costs will be updated.
If there is an entrance for the neighboring cell whose interval is intersected by a current interval, new entrances will
be created with the new enter intervals and enqueued. The same happens if there is no entrance that has been matched
with one of the current intervals, yet. Costs will be added. If there are some entries afterwards that are intersected
by the current interval and have higher costs, they will be removed from the queue.
This method is called during cost calculation. It may be overridden to change the interval handling for the
{@link CellEntrance}s.
@param currentEntrance the current cell entrance
@param neighborCell the neighboring cell that is handled.
@param context the context information
@see PathSearch#calculateCosts
@see PathSearch#calculateHeuristicCosts
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23handleNeighbor(CellEntrance%2CPartitionCell%2CPathSearchContext) Online Documentation}*/
handleNeighbor(currentEntrance: CellEntrance | null, neighborCell: PartitionCell | null, context: PathSearchContext | null): void;
/**
Initializes the fields of this {@link PathSearch}.
This method also calls {@link PathSearchExtension#initialize} for all registered path search extensions.
@param configuration the configuration that the path search shall use
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23init(PathSearchConfiguration) Online Documentation}*/
init(configuration: PathSearchConfiguration | null): void;
/**
Removes the given interval calculator from the list of registered {@link IEnterIntervalCalculator}s.
@param enterIntervalCalculator the calculator to remove
@returns <code>true</code> if an interval calculator was removed as a result of this call, <code>false</code> if the given calculator was not part of the
list
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23removeAdditionalEnterIntervalCalculator(IEnterIntervalCalculator) Online Documentation}*/
removeAdditionalEnterIntervalCalculator(enterIntervalCalculator: IEnterIntervalCalculator | null): boolean;
/**
Removes the given extension from the list of {@link PathSearchExtension}s.
@param extension the extension to remove from the path search
@returns <code>true</code> if an extension was removed as a result of this call, <code>false</code> if the given extension was not contained in the list
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearch%23removePathSearchExtension(PathSearchExtension) Online Documentation}*/
removePathSearchExtension(extension: PathSearchExtension | null): boolean;
static $class: Class;
private $_PathSearch: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PathSearch;
}
export interface PathSearchConfiguration extends YObject {
}
/**
This class contains the basic configuration used by path search algorithms.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchConfiguration Online Documentation}*/
export class PathSearchConfiguration {
/**
Creates a new instance of {@link PathSearchConfiguration} used for a path search.
@param graph the graph containing the edges whose path will be searched
@param grouping the grouping information of the graph
@param edgeRouter the edge routing algorithm using this configuration
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchConfiguration Online Documentation}*/
constructor(graph: LayoutGraph | null, grouping: LayoutGroupingSupport | null, edgeRouter: EdgeRouter | null);
/**
Gets the graph containing the edges whose path is searched.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchConfiguration%23graph Online Documentation}*/
readonly graph: LayoutGraph | null;
/**
Gets the grouping information of the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchConfiguration%23grouping Online Documentation}*/
readonly grouping: LayoutGroupingSupport | null;
/**
Gets the edge routing algorithm using this configuration.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchConfiguration%23edgeRouter Online Documentation}*/
readonly edgeRouter: EdgeRouter | null;
/**
Gets the remaining time (in milliseconds) within which the algorithm should complete its calculation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchConfiguration%23remainingTime Online Documentation}*/
readonly remainingTime: number;
static $class: Class;
private $_PathSearchConfiguration: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PathSearchConfiguration;
}
export interface PathSearchContext extends YObject {
}
/**
This class provides context information that is useful for the {@link PathSearch} algorithm.
Most notably, the context provides access to the edge that is currently being routed (see {@link PathSearchContext#currentEdge}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext Online Documentation}*/
export class PathSearchContext {
/**
Creates a new instance of {@link PathSearchContext}.
@param pathSearch the path search that uses this context
@param configuration the configuration used for the path search
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext Online Documentation}*/
constructor(pathSearch: PathSearch | null, configuration: PathSearchConfiguration | null);
/**
Adds an additional source cell of the {@link PathSearchContext#currentRequest current path request} if it is not already contained in the list of source cells.
@param cell the new source cell to add
@see PathSearchContext#sourceCellCount
@see PathSearchContext#getSourceCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23addSourceCell(PartitionCell) Online Documentation}*/
addSourceCell(cell: PartitionCell | null): void;
/**
Adds an additional target cell of the {@link PathSearchContext#currentRequest current path request} if it is not already contained in the list of target cells.
@param cell the new target cell to add
@see PathSearchContext#targetCellCount
@see PathSearchContext#getTargetCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23addTargetCell(PartitionCell) Online Documentation}*/
addTargetCell(cell: PartitionCell | null): void;
/**
Returns the source cell with the given index in the list of all cells that are covered by the source of the
{@link PathSearchContext#currentRequest current path request}.
@param index the index of the source cell
@returns the source cell with the given index in the list of all source cells
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23getSourceCell(number) Online Documentation}*/
getSourceCell(index: number): PartitionCell | null;
/**
Returns the target cell with the given index in the list of all cells that are covered by the target of the
{@link PathSearchContext#currentRequest current path request}.
@param index the index of the target cell
@returns the target cell with the given index in the list of all cells that are covered by the target of the
{@link PathSearchContext#currentRequest current path request}
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23getTargetCell(number) Online Documentation}*/
getTargetCell(index: number): PartitionCell | null;
/**
Determines whether or not the given cell is a source cell of the {@link PathSearchContext#currentRequest current path request}.
@param cell the cell
@returns <code>true</code> if the given cell is in the list of source cells, <code>false</code> otherwise
@see PathSearchContext#sourceCellCount
@see PathSearchContext#getSourceCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23isSourceCell(PartitionCell) Online Documentation}*/
isSourceCell(cell: PartitionCell | null): boolean;
/**
Determines whether or not the given cell is a target cell of the {@link PathSearchContext#currentRequest current path request}.
@param cell the cell
@returns <code>true</code> if the given cell is in the list of target cells, <code>false</code> otherwise
@see PathSearchContext#targetCellCount
@see PathSearchContext#getTargetCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23isTargetCell(PartitionCell) Online Documentation}*/
isTargetCell(cell: PartitionCell | null): boolean;
/**
Specifies the edges that shall be routed.
@param edges the list of edges that shall be routed
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23setEdges(EdgeList) Online Documentation}*/
setEdges(edges: EdgeList | null): void;
/**
Returns the number of all cells that are covered by the source of the {@link PathSearchContext#currentRequest current path request}.
@returns the number of all cells that are covered by the source
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23sourceCellCount() Online Documentation}*/
sourceCellCount(): number;
/**
Returns the number of all cells that are covered by the target of the {@link PathSearchContext#currentRequest current path request}.
@returns the number of all cells that are covered by the target
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23targetCellCount() Online Documentation}*/
targetCellCount(): number;
/**
Gets the {@link PathSearch} that uses this context.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23pathSearch Online Documentation}*/
readonly pathSearch: PathSearch | null;
/**
Gets the configuration used for the {@link PathSearch}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23configuration Online Documentation}*/
readonly configuration: PathSearchConfiguration | null;
/**
Gets the {@link IEdgeCursor} to iterate over the edges that are routed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23edges Online Documentation}*/
readonly edges: IEdgeCursor | null;
/**
Gets or sets the edge that is currently routed.
This edge is one of those returned by {@link PathSearchContext#edges}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23currentEdge Online Documentation}*/
currentEdge: Edge | null;
/**
Gets or sets the {@link PathRequest} that is currently routed.
This request belongs to the {@link PathSearchContext#currentEdge current edge}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23currentRequest Online Documentation}*/
currentRequest: PathRequest | null;
/**
Gets the {@link EdgeRouterEdgeLayoutDescriptor} for the {@link PathSearchContext#currentEdge current edge} containing edge specific settings for the path search.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23currentEdgeLayoutDescriptor Online Documentation}*/
readonly currentEdgeLayoutDescriptor: EdgeRouterEdgeLayoutDescriptor | null;
/**
Gets an artificial {@link PartitionCell partition cell} with the size of the bounding box of all source cells of the {@link PathSearchContext#currentRequest current path request}.
@see PathSearchContext#getSourceCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23combinedSourceCell Online Documentation}*/
readonly combinedSourceCell: PartitionCell | null;
/**
Gets an artificial {@link PartitionCell partition cell} with the size of the bounding box of all target cells of the {@link PathSearchContext#currentRequest current path request}.
@see PathSearchContext#getTargetCell
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23combinedTargetCell Online Documentation}*/
readonly combinedTargetCell: PartitionCell | null;
/**
Gets the results of the {@link PathSearch}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchContext%23pathSearchResult Online Documentation}*/
readonly pathSearchResult: PathSearchResult | null;
static $class: Class;
private $_PathSearchContext: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PathSearchContext;
}
export interface PathSearchExtension extends YObject {
}
/**
Extensions are added to a {@link PathSearch} to influence the search process.
An extension contains several callback methods that are used by the {@link PathSearch} to, e.g., calculate the
costs for the next possible steps (i.e., entering the next partition cell) and to decide when the target is reached.
The {@link PathSearch} uses the callbacks in the following order:
<ol>
<li>{@link PathSearchExtension#initialize}
<li>{@link PathSearchExtension#initializeEdges}
<li>{@link PathSearchExtension#initializeCurrentEdge}
<li>{@link PathSearchExtension#initializeCurrentPathRequest}
<li>{@link PathSearchExtension#appendStartEntrances}
<li>{@link PathSearchExtension#calculateStartEntranceCost}
<li>{@link PathSearchExtension#calculateCosts}
<li>{@link PathSearchExtension#calculateHeuristicCosts}
<li>{@link PathSearchExtension#isValidTargetEntrance}
<li>{@link PathSearchExtension#finalizePath} or {@link PathSearchExtension#cancelCurrentPathRequest}
<li>{@link PathSearchExtension#finalizeCurrentEdge} or {@link PathSearchExtension#cancelCurrentEdge}
<li>{@link PathSearchExtension#finalizeEdges}
<li>{@link PathSearchExtension#finalizePathSearchResult}
<li>{@link PathSearchExtension#cleanUp}
</ol>
@see PathSearch#addPathSearchExtension
@see PathSearchContext#currentEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension Online Documentation}*/
export abstract class PathSearchExtension {
/**
Creates a new instance of {@link PathSearchExtension}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension Online Documentation}*/
constructor();
/**
Appends additional start {@link CellEntrance entrances} for the path search of the current edge to the given list of all previously generated
entrances.
This method is called before starting the actual path search to determine possible starting points. The current
implementation doesn't add any {@link CellEntrance}s. It may be overridden to add {@link CellEntrance}s with
specific enter intervals, e.g., to consider {@link PortConstraint}s.
@param allStartEntrances a list of all previously generated entrances
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23appendStartEntrances(IList%3CObject%3E) Online Documentation}*/
appendStartEntrances(allStartEntrances: IList<any> | null | IListConvertible<any>): void;
/**
Calculates the costs for entering the given {@link PartitionCell} via the given enter
{@link OrthogonalInterval}.
The given {@link EdgeCellInfo} describes how the partition cell that has been entered by the current entrance
would be traversed if this enter interval to the neighboring cell would be chosen.
The <code>maxAllowedCosts</code> describe the maximum costs that entering the neighboring cell via the given enter interval may
induce. If the maximum cost is exceeded, calculations that may further increase its cost may be skipped because this
interval won't be chosen anyway.
This implementation doesn't add any costs. It may be overridden to add costs, e.g., if entering through the given
interval would create a bend.
@param currentEntrance the entrance that was used for entering the current cell
@param enteredCell the neighboring cell that shall be entered
@param enterInterval the interval that shall be used for entering the neighboring cell
@param edgeCellInfo the information about how the current cell would be traversed if the neighboring cell would be entered by this enter
interval
@param maxAllowedCosts the maximum allowed costs for this enter interval
@returns the costs for entering the neighboring cell via the enter interval
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23calculateCosts(CellEntrance%2CPartitionCell%2COrthogonalInterval%2CEdgeCellInfo%2Cnumber) Online Documentation}*/
calculateCosts(currentEntrance: CellEntrance | null, enteredCell: PartitionCell | null, enterInterval: OrthogonalInterval | null, edgeCellInfo: EdgeCellInfo | null, maxAllowedCosts: number): number;
/**
Calculates the heuristic costs for the given {@link CellEntrance} that describe the minimal costs that will arise
to finish the path if the given cell entrance is used as next step.
After evaluating the costs for each enter interval to a neighboring cell, this method is called for each of the
resulting {@link CellEntrance}s.
This implementation doesn't add any costs. It may be overridden to add some heuristic costs, e.g., if the edge would
have to bend to reach the target node from the given {@link CellEntrance}.
@param entrance the entrance to calculate the heuristic costs for the rest of the path
@returns the minimal costs that will arise to finish the path if the given cell entrance is used as next step
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23calculateHeuristicCosts(CellEntrance) Online Documentation}*/
calculateHeuristicCosts(entrance: CellEntrance | null): number;
/**
Returns the cost for starting the path in the given entrance.
This method is called for each start entrance of the current edge.
The current implementation doesn't add any costs for the given {@link CellEntrance}. It may be overridden to make
start entrances with less specific intervals more expensive. For example, intervals that allow more than just the {@link PortConstraint}
location to connect to the source node become more expensive.
@param startEntrance the start entrance for which the cost is calculated
@returns the cost for starting the path in the given entrance
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23calculateStartEntranceCost(CellEntrance) Online Documentation}*/
calculateStartEntranceCost(startEntrance: CellEntrance | null): number;
/**
Handles the cancellation of the path search for the current edge.
This callback notifies the extension when the path search for the current edge is canceled. Then, the path search will
proceed with altering the penalties of the current edge and reinitializing it again.
This implementation does nothing. It may be overridden to reset some edge specific information, e.g., removing
previously added listeners.
@param context the context containing the current edge for which the path search has been canceled
@see PathSearchExtension#initializeCurrentEdge
@see PathSearchExtension#finalizeEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23cancelCurrentEdge(PathSearchContext) Online Documentation}*/
cancelCurrentEdge(context: PathSearchContext | null): void;
/**
Handles the cancellation of the path search for the current {@link PathRequest}.
This callback notifies the extension when the path search for the current request is canceled. In case that not all
requests of an edge are successful, then also the path search for that edge will be {@link PathSearchExtension#cancelCurrentEdge canceled}.
This implementation does nothing. It may be overridden to process or reset
{@link PathRequest}-specific data like e.g., previously registered listeners.
@param context the context containing the request for which the path search has been canceled
@see PathSearchExtension#initializeCurrentPathRequest
@see PathSearchContext#currentRequest
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23cancelCurrentPathRequest(PathSearchContext) Online Documentation}*/
cancelCurrentPathRequest(context: PathSearchContext | null): void;
/**
Cleans up the extension from the path searches with the current {@link PathSearchExtension#configuration configuration} and {@link PathSearchExtension#context context}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23cleanUp() Online Documentation}*/
cleanUp(): void;
/**
Handles the completion of the path search for the current edge.
This callback notifies the extension when enough paths are found for the current edge. After that, the path search
either proceeds with initializing the next current edge in the context's edge list or calls {@link PathSearchExtension#finalizeEdges}
if all edges in this list have been handled.
This implementation does nothing. It may be overridden to add/reset some edge specific information, e.g., removing
previously added listeners or adding new start entrances.
@param context the context containing the current edge for which the path search has been finished
@see PathSearchExtension#initializeCurrentEdge
@see PathSearchExtension#finalizeEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23finalizeCurrentEdge(PathSearchContext) Online Documentation}*/
finalizeCurrentEdge(context: PathSearchContext | null): void;
/**
Handles the completion of all marked edges.
This callback notifies the extension if enough paths have been found for all edges in the context's edge list.
@param context the context containing the list of edges for which paths have been found
@see PathSearchContext#edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23finalizeEdges(PathSearchContext) Online Documentation}*/
finalizeEdges(context: PathSearchContext | null): void;
/**
After finding a valid target entrance and creating a {@link EdgeRouterPath}, the extension is notified of the found path.
If the path search is configured to calculate several possible paths for an edge, the path search proceeds with choosing
another unhandled {@link CellEntrance}.
This implementation does nothing. It may be overridden to collect some information about the given path that may
influence the path search of later edges.
@param path the path found for the current edge in the context
@see PathSearchContext#currentEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23finalizePath(EdgeRouterPath) Online Documentation}*/
finalizePath(path: EdgeRouterPath | null): void;
/**
Completes the path search result.
This callback notifies the extensions of the paths chosen for the edges in the current context. After calling
{@link PathSearchExtension#finalizeEdges finalizeEdges}, the {@link PathSearch} decides, which of the found paths to use for each edge and adds them to the
{@link PathSearchResult}.
With this callback, the registered extensions are notified of this result before the path search either initializes the
next list of edges to route or ends the path search by calling
{@link PathSearchExtension#cleanUp}. This implementation does nothing.
@param pathSearchResult the path search result for the edge in the current context's edge list
@see PathSearchExtension#initializeEdges
@see PathSearchExtension#cleanUp
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23finalizePathSearchResult(PathSearchResult) Online Documentation}*/
finalizePathSearchResult(pathSearchResult: PathSearchResult | null): void;
/**
Initializes this extension for path searches using the given configuration.
This method is the first one to be called by the {@link PathSearch}. It may be overridden to initialize this
extension with necessary settings like {@link LayoutGroupingSupport}.
@param configuration the configuration to use for the following path searches
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23initialize(PathSearchConfiguration) Online Documentation}*/
initialize(configuration: PathSearchConfiguration | null): void;
/**
Initializes this extension with the current edge set in the given context.
This method is called for each edge of the {@link PathSearchExtension#context context}. Each call will be balanced by a call to either {@link PathSearchExtension#finalizeCurrentEdge}
or {@link PathSearchExtension#cancelCurrentEdge}.
It may be overridden to initialize/reset the settings for the current edge (e.g. minimum distances).
@param context the context containing the current edge
@see PathSearchContext#currentEdge
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23initializeCurrentEdge(PathSearchContext) Online Documentation}*/
initializeCurrentEdge(context: PathSearchContext | null): void;
/**
Initializes this extension with the current {@link PathRequest} set in the given context.
This method is called for each request of the context's {@link PathSearchContext#currentEdge current edge}. Each call will be balanced by a call to either {@link PathSearchExtension#finalizePath}
or {@link PathSearchExtension#cancelCurrentPathRequest}.
This method may be overridden to modify settings and data that should be individual for each {@link PathRequest}.
@param context the context containing the current path request
@see PathSearchContext#currentRequest
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23initializeCurrentPathRequest(PathSearchContext) Online Documentation}*/
initializeCurrentPathRequest(context: PathSearchContext | null): void;
/**
Initializes this extension with the context that contains the list of edges for which paths are calculated.
This method may be called several times during a path search. Each call will be balanced by calls to {@link PathSearchExtension#finalizeEdges}
and {@link PathSearchExtension#finalizePathSearchResult}.
It may be overridden to initialize/reset the settings for the routed edges (e.g. minimum distances).
@param context the context containing the list of edges for which paths shall be found
@see PathSearchContext#edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23initializeEdges(PathSearchContext) Online Documentation}*/
initializeEdges(context: PathSearchContext | null): void;
/**
Determines whether or not this extension considers the given {@link CellEntrance} as a valid target entrance,
i.e., the path may end with the given entrance.
Each time a {@link CellEntrance} is chosen as next step, all registered extensions are asked if this entrance is a
valid target entrance. Only if none of the extensions returns <code>false</code>, a {@link EdgeRouterPath} is created.
This implementation accepts all
{@link CellEntrance}s as valid targets. It may be overridden to only activate the target if the route fulfills a
certain condition.
@param entrance the entrance to decide if it is a valid target entrance
@returns <code>true</code> if the path may end with this entrance, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23isValidTargetEntrance(CellEntrance) Online Documentation}*/
isValidTargetEntrance(entrance: CellEntrance | null): boolean;
/**
Gets the configuration used for the path search.
This property is initialized in {@link PathSearchExtension#initialize}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23configuration Online Documentation}*/
readonly configuration: PathSearchConfiguration | null;
/**
Gets the current context of the path search.
This property is initialized in {@link PathSearchExtension#initializeEdges}.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchExtension%23context Online Documentation}*/
readonly context: PathSearchContext | null;
static $class: Class;
private $_PathSearchExtension: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PathSearchExtension;
}
export interface PathSearchResult extends YObject {
}
/**
This class manages the best {@link EdgeRouterPath}s found for each edge and provides according {@link EdgeInfo}s.
These are used by the {@link ChannelBasedPathRouting} algorithm to calculate the final segment locations for the
segments of an edge path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchResult Online Documentation}*/
export class PathSearchResult {
/**
Creates a new instance of {@link PathSearchResult}.
@param configuration the configuration used for the path search
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchResult Online Documentation}*/
constructor(configuration: PathSearchConfiguration | null);
/**
Returns an {@link EdgeInfo} for the given edge.
If the given edge shall be routed but no path has been set for it, yet, <code>null</code> is returned. If no {@link EdgeInfo}
has been created for the path by this context before, a new one is instantiated, otherwise the stored one is returned.
@param edge the edge for which the {@link EdgeInfo} will be returned
@returns an {@link EdgeInfo} describing the path of the edge
@see PathSearchResult#setPath
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchResult%23getEdgeInfo(Edge) Online Documentation}*/
getEdgeInfo(edge: Edge): EdgeInfo | null;
/**
Returns the previously registered path found for the edge.
@param edge the edge for which the path is provided
@returns the previously registered path found for the edge or <code>null</code> if no path has been registered, yet
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchResult%23getPath(Edge) Online Documentation}*/
getPath(edge: Edge | null): EdgeRouterPath | null;
/**
Specifies a found path for an edge.
@param edge the edge for which the path is set
@param path the found path
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathSearchResult%23setPath(Edge%2CEdgeRouterPath) Online Documentation}*/
setPath(edge: Edge | null, path: EdgeRouterPath | null): void;
static $class: Class;
private $_PathSearchResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PathSearchResult;
}
export interface PolylineLayoutStage extends LayoutStageBase {
}
/**
{@link PolylineLayoutStage} extends the orthogonal edge paths with polyline segments.
The current implementation only adds octilinear segments to all orthogonal edges. Non-orthogonal edges are completely
ignored.
An octilinear segment is added between two consecutive vertical and horizontal segments. The length of an octilinear
segment can be influenced by the {@link PolylineLayoutStage#maximumNonOrthogonalSegmentRatio maximum non-orthogonal segment ratio} and the {@link PolylineLayoutStage#preferredPolylineSegmentLength preferred polyline segment length}. The new
octilinear segments maintain a {@link PolylineLayoutStage#minimumNodeToEdgeDistance minimum distance} between the edges and all nodes in the graph if possible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage Online Documentation}*/
export class PolylineLayoutStage {
/**
Creates a new instance of {@link PolylineLayoutStage} using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] the core layout routine
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new instance of {@link PolylineLayoutStage} using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] the core layout routine
@param [options.affectedNodesDpKey] The {@link IDataProvider} key used for looking up the selected state of the nodes of the graph to be laid out. This option sets the {@link PolylineLayoutStage#affectedNodesDpKey} property on the created object.
@param [options.affectedEdgesDpKey] The {@link IDataProvider} key used for looking up the selected state of the edges of the graph to be laid out. This option sets the {@link PolylineLayoutStage#affectedEdgesDpKey} property on the created object.
@param [options.scope] The (sub-)set of edges to be routed in a polyline fashion. This option sets the {@link PolylineLayoutStage#scope} property on the created object.
@param [options.minimumNodeToEdgeDistance] The minimum distance between edges and node bounds. This option sets the {@link PolylineLayoutStage#minimumNodeToEdgeDistance} property on the created object.
@param [options.maximumNonOrthogonalSegmentRatio] The maximum segment length ratio at each end of an orthogonal segment that may get converted into a (non-orthogonal) polyline segment. This option sets the {@link PolylineLayoutStage#maximumNonOrthogonalSegmentRatio} property on the created object.
@param [options.preferredPolylineSegmentLength] The preferred segment length for (non-orthogonal) polyline segments. This option sets the {@link PolylineLayoutStage#preferredPolylineSegmentLength} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
affectedNodesDpKey?: any | null;
affectedEdgesDpKey?: any | null;
scope?: EdgeRouterScope | EdgeRouterScopeStringValues;
minimumNodeToEdgeDistance?: number;
maximumNonOrthogonalSegmentRatio?: number;
preferredPolylineSegmentLength?: number;
});
/**
Adds polyline segments to the edge routes produced by the {@link LayoutStageBase#coreLayout core layout algorithm}.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the {@link IDataProvider} key used for looking up the selected state of the nodes of the graph to
be laid out.
If the {@link PolylineLayoutStage#scope scope} is set to
{@link EdgeRouterScope#ROUTE_EDGES_AT_AFFECTED_NODES}, only edges of selected nodes are routed, while all other edges are
considered to have fixed routes.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see PolylineLayoutStage#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage%23affectedNodesDpKey Online Documentation}*/
affectedNodesDpKey: any | null;
/**
Gets or sets the {@link IDataProvider} key used for looking up the selected state of the edges of the graph to
be laid out.
If the {@link PolylineLayoutStage#scope scope} is set to
{@link EdgeRouterScope#ROUTE_AFFECTED_EDGES}, only the selected edges are routed, while all other edges are considered to
have fixed routes.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see PolylineLayoutStage#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
/**
Gets or sets the (sub-)set of edges to be routed in a polyline fashion.
@throws {Exception} <b>(ArgumentError)</b> if the given argument is none of the predefined scope values
@see PolylineLayoutStage#affectedEdgesDpKey
@see PolylineLayoutStage#affectedNodesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage%23scope Online Documentation}*/
scope: EdgeRouterScope;
/**
Gets or sets the minimum distance between edges and node bounds.
Values should be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage%23minimumNodeToEdgeDistance Online Documentation}*/
minimumNodeToEdgeDistance: number;
/**
Gets or sets the maximum segment length ratio at each end of an orthogonal segment that may get converted into a
(non-orthogonal) polyline segment.
The ratio must be between <code>0</code> and <code>0.5</code>.
@throws {Exception} <b>(ArgumentError)</b> if the ratio does not lie within <code>[0,0.5]</code> interval
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage%23maximumNonOrthogonalSegmentRatio Online Documentation}*/
maximumNonOrthogonalSegmentRatio: number;
/**
Gets or sets the preferred segment length for (non-orthogonal) polyline segments.
Values should be greater than or equal to <code>0</code>.
@throws {Exception} <b>(ArgumentError)</b> if the length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineLayoutStage%23preferredPolylineSegmentLength Online Documentation}*/
preferredPolylineSegmentLength: number;
static $class: Class;
private $_PolylineLayoutStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PolylineLayoutStage;
}
export interface SegmentGroup extends YObject {
}
/**
Represents a group of segments of different edges that shall be combined at their common source or target.
Grouped edges have the same source or target group ID assigned in the {@link IDataProvider} registered with
the graph with {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY} or {@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}
keys.
The {@link SegmentGroup} contains the {@link SegmentGroup#commonLocationRange common location range} for the edge group and provides a {@link SegmentGroup#commonSegmentInfo common segment info}
representing the grouped segments.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentGroup Online Documentation}*/
export class SegmentGroup {
/**
Creates a new instance of {@link SegmentGroup} that contains the given segments and a range to place them.
@param commonLocationRange the location range of the given {@link SegmentInfo}s
@param segmentInfos the list containing the grouped {@link SegmentInfo}s
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentGroup Online Documentation}*/
constructor(commonLocationRange: Interval | null, segmentInfos: IList<any> | null | IListConvertible<any>);
/**
Returns the hash code for this {@link SegmentGroup}.
@returns the hash code for this {@link SegmentGroup}
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentGroup%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Gets the common location range for the {@link SegmentGroup#segmentInfos segments} in this group.
For vertical grouped segments, this interval describes the horizontal range where this segment can be placed. Horizontal
grouped segments are placed in a vertical range.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentGroup%23commonLocationRange Online Documentation}*/
readonly commonLocationRange: Interval | null;
/**
Gets the list of {@link SegmentInfo}s contained in this group.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentGroup%23segmentInfos Online Documentation}*/
readonly segmentInfos: IList<any> | null;
/**
Gets the common {@link SegmentInfo} representing the grouped segments.
The returned segment info is used for placing the common segment. The coordinates are applied to all edges in the bundle
later.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentGroup%23commonSegmentInfo Online Documentation}*/
readonly commonSegmentInfo: SegmentInfo | null;
static $class: Class;
private $_SegmentGroup: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SegmentGroup;
}
export interface SegmentInfo extends SegmentInfoBase {
}
/**
A {@link SegmentInfo} holds information about the possible location of an orthogonal edge segment.
Most notably, it stores the {@link SegmentInfoBase#direction direction} of the segment and the {@link SegmentInfoBase#locationRange interval} that restricts the location of the segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfo Online Documentation}*/
export class SegmentInfo {
/**
Creates a new instance of {@link SegmentInfo} with the given information.
@param edge the edge to which the segment of this info belongs
@param segmentIndex the index of the segment to which this info belongs
@param direction the direction to which the segment of this info points
@param locationRange the range within which the common location of this segment has to lie
@param minExtension the minimum interval in extension direction this segment is known to intersect
@param maxExtension the maximum interval in extension direction this segment will span
@param cellSegmentInfos the list of {@link CellSegmentInfo} objects upon which this segment info is built
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfo%23SegmentInfo(Edge%2Cnumber%2CDirection%2CInterval%2CInterval%2CInterval%2CIList%3CObject%3E) Online Documentation}*/
constructor(edge: Edge | null, segmentIndex: number, direction: Direction | null, locationRange: Interval | null, minExtension: Interval | null, maxExtension: Interval | null, cellSegmentInfos: IList<any> | null | IListConvertible<any>);
/**
Creates a new instance of {@link SegmentInfo} using a {@link LineSegment} to describe the edge segment.
<h2>Note</h2>
This constructor is meant to be used for fixed orthogonal edge segments already knowing their extension and location.
@param edge the edge to which the segment of this info belongs
@param segmentIndex the index of the segment to which this info belongs
@param segment a line segment describing the edge segment
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfo%23SegmentInfo(Edge%2Cnumber%2CLineSegment) Online Documentation}*/
constructor(edge: Edge | null, segmentIndex: number, segment: LineSegment | null);
/**
Returns the number of {@link CellSegmentInfo} of this segment info.
@returns the number of {@link CellSegmentInfo} of this segment info
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfo%23cellSegmentInfoCount() Online Documentation}*/
cellSegmentInfoCount(): number;
/**
Returns the {@link CellSegmentInfo} at the given index of this segment info.
@param index the index of the {@link CellSegmentInfo} to return
@returns the {@link CellSegmentInfo} at the given index of this segment info
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfo%23getCellSegmentInfo(number) Online Documentation}*/
getCellSegmentInfo(index: number): CellSegmentInfo | null;
/**
Gets the edge to which the segment of this info belongs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfo%23edge Online Documentation}*/
readonly edge: Edge | null;
/**
Sets the {@link SegmentGroup segment group} for this segment info and its associated {@link CellSegmentInfo}s.
The {@link SegmentInfoBase#locationRange location ranges} are adjusted to use the {@link SegmentGroup#commonLocationRange}.
@see SegmentGroup
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfo%23segmentGroup Online Documentation}*/
segmentGroup: SegmentGroup | null;
/**
Gets or sets whether or not this edge segment has the restriction of a strong {@link PortConstraint}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfo%23atStrongPortConstraint Online Documentation}*/
atStrongPortConstraint: boolean;
static $class: Class;
private $_SegmentInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SegmentInfo;
}
export interface SegmentInfoBase extends YObject {
}
/**
This class stores location information for orthogonal item segments.
Most notably, {@link SegmentInfoBase} stores the {@link SegmentInfoBase#direction direction} to which the segment points and the {@link SegmentInfoBase#locationRange interval} describing
possible locations and extensions of a segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase Online Documentation}*/
export abstract class SegmentInfoBase {
/**
Creates a new instance of {@link SegmentInfoBase}.
@param item the item to which the segment of this info belongs
@param segmentIndex the index of the segment to which this info belongs
@param direction the direction to which the segment of this info points
@param locationRange the range within which the common location of this segment has to lie
@param minExtension the minimum interval in extension direction this segment is known to intersect
@param maxExtension the maximum interval in extension direction this segment will span
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23SegmentInfoBase(Object%2Cnumber%2CDirection%2CInterval%2CInterval%2CInterval) Online Documentation}*/
constructor(item: any | null, segmentIndex: number, direction: Direction | null, locationRange: Interval | null, minExtension: Interval | null, maxExtension: Interval | null);
/**
Creates a new instance of {@link SegmentInfoBase} using a line segment to describe the edge segment.
This constructor is meant to be used for fixed orthogonal edge segments.
@param item the item to which this segment info belongs
@param segmentIndex the index of the segment to which this info belongs
@param segment a line segment describing the item segment
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23SegmentInfoBase(Object%2Cnumber%2CLineSegment) Online Documentation}*/
constructor(item: any | null, segmentIndex: number, segment: LineSegment | null);
/**
Returns a string representation of this {@link SegmentInfoBase} object.
@returns a string representation of this {@link SegmentInfoBase} object
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23toString() Online Documentation}*/
toString(): string;
/**
Gets the minimum known extension of the segment, i.e., the minimum interval that this segment will cover in any case.
For horizontal intervals, this is the minimum horizontal extension; for vertical intervals, this is the minimum vertical
extension.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23minExtension Online Documentation}*/
readonly minExtension: Interval | null;
/**
Gets the maximum extension that this segment can span.
For horizontal intervals, this is the maximum horizontal extension; for vertical intervals, this is the maximum vertical
extension.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23maxExtension Online Documentation}*/
readonly maxExtension: Interval | null;
/**
Gets the range, i.e., the interval within which the segment should be placed.
For horizontal segments, this is the range containing the vertical location; for vertical segments, this is the range
containing the horizontal location.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23locationRange Online Documentation}*/
readonly locationRange: Interval | null;
/**
Gets or sets the preferred placement of this segment within its location range.
@throws {Exception} <b>(ArgumentError)</b> if an unknown alignment is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23preferredAlignment Online Documentation}*/
preferredAlignment: Alignment;
/**
Gets or sets the {@link SegmentInfoBase#fixed fixed} location within the {@link SegmentInfoBase#locationRange location range} for this segment info.
For horizontal segments, this is the vertical location; for vertical segments, this is the horizontal location.
@throws {Exception} <b>(InvalidOperationError)</b> if the segment info already has a fixed location
@throws {Exception} <b>(ArgumentError)</b> if the given location does not lie within the location range
@see SegmentInfoBase#fixed
@see SegmentInfoBase#locationRange
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23location Online Documentation}*/
location: number;
/**
Gets whether or not this segment info is fixed, i.e., a {@link SegmentInfoBase#location fixed location} has been determined within its {@link SegmentInfoBase#locationRange location range}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23fixed Online Documentation}*/
readonly fixed: boolean;
/**
Gets whether or not the associated segment is vertical.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23vertical Online Documentation}*/
readonly vertical: boolean;
/**
Gets the {@link Direction} to which the segment of this info points.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23direction Online Documentation}*/
readonly direction: Direction | null;
/**
Gets the index of the segment of this info.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23segmentIndex Online Documentation}*/
readonly segmentIndex: number;
/**
Gets or sets the segment group to which this segment info belongs.
When defining the segment group, the {@link SegmentInfoBase#locationRange location range} will be adjusted to use the common interval of the segment group.
@see SegmentGroup#commonLocationRange
@see {@link https://docs.yworks.com/yfileshtml/#/api/SegmentInfoBase%23segmentGroup Online Documentation}*/
segmentGroup: SegmentGroup | null;
static $class: Class;
private $_SegmentInfoBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SegmentInfoBase;
}
/**
@see RadialLayout#centerNodesPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CenterNodesPolicy Online Documentation}*/
export enum CenterNodesPolicy {
/**
Specifier for a center nodes policy which chooses all nodes with in-degree equal to <code>0</code> as center nodes.
If no node with this property is present, an arbitrary node will be selected.
@see RadialLayout#centerNodesPolicy
@see TreeAlgorithm#getRoot
@see {@link https://docs.yworks.com/yfileshtml/#/api/CenterNodesPolicy%23DIRECTED Online Documentation}*/
DIRECTED,
/**
Specifier for a center nodes policy which chooses the node with the highest centrality as center node.
@see RadialLayout#centerNodesPolicy
@see TreeAlgorithm#getCenterRoot
@see {@link https://docs.yworks.com/yfileshtml/#/api/CenterNodesPolicy%23CENTRALITY Online Documentation}*/
CENTRALITY,
/**
Specifier for a center nodes policy which chooses the node with the highest weighted centrality as center node.
@see RadialLayout#centerNodesPolicy
@see TreeAlgorithm#getWeightedCenterNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/CenterNodesPolicy%23WEIGHTED_CENTRALITY Online Documentation}*/
WEIGHTED_CENTRALITY,
/**
Specifier for a center nodes policy which uses center nodes specified by the user.
The center nodes have to be specified using a {@link IDataProvider} registered with the {@link RadialLayout#centerNodesDpKey according key}. The {@link IDataProvider}
returns <code>true</code> for nodes that should be placed in the center.
If there exists more than one center node, they will be placed on the innermost circle while the center position stays
empty.
@see RadialLayout#centerNodesPolicy
@see RadialLayout#centerNodesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/CenterNodesPolicy%23CUSTOM Online Documentation}*/
CUSTOM
}
/** Automatic conversion to {@link CenterNodesPolicy}. */
type CenterNodesPolicyStringValues = "directed" | "centrality" | "weighted-centrality" | "custom";
/**
@see RadialLayout#layeringStrategy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutLayeringStrategy Online Documentation}*/
export enum RadialLayoutLayeringStrategy {
/**
Specifier for a layering strategy that uses breadth first search (BFS) to determine a layering for the graph.
All edges will span at most one layer in the resulting drawing. Edges between nodes that belong to the same layer are
possible.
@see RadialLayout#layeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutLayeringStrategy%23BFS Online Documentation}*/
BFS,
/**
Specifier for a layering strategy that uses an optimal hierarchical layering strategy.
The layer distance of an edge is the absolute difference between the layer numbers of its source and target node. Layer
assignment will be done such that the overall sum of the layer distances of all edges in the layout is minimal.
@see RadialLayout#layeringStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutLayeringStrategy%23HIERARCHICAL Online Documentation}*/
HIERARCHICAL,
/**
Specifier for a layering strategy that uses a custom, user-defined layer/circle assignment.
The nodes' layer/circle assignment must be provided by a {@link IDataProvider} registered with the input graph
with the key
{@link RadialLayout#LAYER_ID_DP_KEY}. If no data is provided and this strategy is selected, then the algorithm
will throw an exception.
Using this strategy, the center nodes (nodes on the innermost circle) are not determined by the specified {@link RadialLayout#centerNodesPolicy center nodes policy}
anymore. Instead, the nodes with the smallest given ID (i.e. <code>0</code>) are part of the center nodes.
<b>Warning:</b> If applying this strategy, the {@link RadialLayout#centerNodesPolicy} is ignored.
@see RadialLayout#layeringStrategy
@see RadialLayout#LAYER_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutLayeringStrategy%23USER_DEFINED Online Documentation}*/
USER_DEFINED
}
/** Automatic conversion to {@link RadialLayoutLayeringStrategy}. */
type RadialLayoutLayeringStrategyStringValues = "bfs" | "hierarchical" | "user-defined";
/**
@see RadialLayout#edgeRoutingStrategy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutEdgeRoutingStrategy Online Documentation}*/
export enum RadialLayoutEdgeRoutingStrategy {
/**
Specifier for an edge routing strategy that will route edges with polyline segments from source to target with one bend
on each spanned circle.
@see RadialLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutEdgeRoutingStrategy%23POLYLINE Online Documentation}*/
POLYLINE,
/**
Specifier for an edge routing strategy that will route edges as arcs.
The source and target of an edge as well as one bend on each spanned circle are used as main control points. An easing
function is applied in order to calculate additional control points between the polar coordinates of the main control
points.
@see RadialLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutEdgeRoutingStrategy%23ARC Online Documentation}*/
ARC
}
/** Automatic conversion to {@link RadialLayoutEdgeRoutingStrategy}. */
type RadialLayoutEdgeRoutingStrategyStringValues = "polyline" | "arc";
export interface RadialLayoutNodeInfo extends YObject {
}
/**
Info object used for retrieving information about each node from the layout algorithm.
The information can be retrieved by registering with the graph a {@link IDataProvider} (which should also
implement the {@link IDataAcceptor} interface) using key {@link RadialLayout#NODE_INFO_DP_KEY}. The
layout algorithm will return a {@link RadialLayoutNodeInfo} object with appropriately filled fields for each {@link YNode}
when the layout is calculated.
<b>Warning:</b>
This info object only delivers additional data which were calculated during the layout run and has no influence on
layout calculation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutNodeInfo Online Documentation}*/
export class RadialLayoutNodeInfo {
/**
Creates a new {@link RadialLayoutNodeInfo} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutNodeInfo Online Documentation}*/
constructor();
/**
Gets the index of the circle the node is placed on.
The circles used by the {@link RadialLayout} are indexed from the innermost to the outermost circle. Index <code>0</code>
represents the center, i.e. the circle with radius <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutNodeInfo%23circleIndex Online Documentation}*/
readonly circleIndex: number;
/**
Gets the radius of the circle the node is placed on.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutNodeInfo%23radius Online Documentation}*/
readonly radius: number;
/**
Gets the offset from the center of the circle, the node is placed on, to the center of the node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutNodeInfo%23centerOffset Online Documentation}*/
readonly centerOffset: YPoint | null;
/**
Gets the start angle in degrees of the sector the node is placed in.
The angle refers to a counter-clockwise rotation starting at the 3&nbsp;o'clock position.
@see RadialLayoutNodeInfo#sectorSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutNodeInfo%23sectorStart Online Documentation}*/
readonly sectorStart: number;
/**
Gets the size in degrees of the sector the node is placed in.
@see RadialLayoutNodeInfo#sectorStart
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutNodeInfo%23sectorSize Online Documentation}*/
readonly sectorSize: number;
static $class: Class;
private $_RadialLayoutNodeInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RadialLayoutNodeInfo;
}
export interface RadialLayout extends MultiStageLayout {
}
/**
This layout algorithm arranges graphs in a radial fashion.
<b>Layout Style</b>
The nodes are distributed into circles (layers) around a common center based on predefined
{@link RadialLayout#layeringStrategy layering strategies}. Some potential applications include visualization of social networks, data clustering and
bioinformatics.
<b>Concept</b>
The layout algorithm creates a virtual tree structure for the input graph whose root node is placed in the center of the
circles. All children of a tree node are placed on a higher layer (i.e. on a circle of larger radius) than their parent.
A sector is calculated for each node such that the sector of each child node in the virtual tree is a sub-sector of its
parent's sector. The radii of the circles are calculated based on the sizes of the sectors required to accommodate each
whole subtree.
<b>Features</b>
There are different {@link RadialLayout#centerNodesPolicy center nodes policies} that can be applied in order to determine the nodes that will be placed in the
center (single center node) or on the innermost circle (more than one center node). The distribution of the nodes to the
circles is specified by a {@link RadialLayout#layeringStrategy layering strategy}.
The edges are routed based on the
{@link RadialLayout#edgeRoutingStrategy edge routing strategy}. Note that the edge routing cannot always prevent node-edge overlaps.
The minimum distance between nodes of the same circle is specified using
{@link RadialLayout#minimumNodeToNodeDistance}. If there are too many nodes assigned to a circle, the radius of the particular circle will be increased until all
nodes fit onto it.
The distance between consecutive layers is specified using {@link RadialLayout#layerSpacing}. The radii of the circles are a multiple of this spacing
value. If the layer spacing is <code>0</code>, the radii are chosen such that all nodes fit on their corresponding circles. A minimum
layer distance is specified by {@link RadialLayout#minimumLayerDistance}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout Online Documentation}*/
export class RadialLayout {
/**
Creates a new {@link RadialLayout} instance with the default settings.
@param [options] The parameters to pass.
@param [options.minimumNodeToNodeDistance] The minimum distance between two neighboring nodes of the same circle. This option sets the {@link RadialLayout#minimumNodeToNodeDistance} property on the created object.
@param [options.layerSpacing] The spacing between two consecutive circles (layers). This option sets the {@link RadialLayout#layerSpacing} property on the created object.
@param [options.minimumLayerDistance] The minimum distance between two circles. This option sets the {@link RadialLayout#minimumLayerDistance} property on the created object.
@param [options.maximumChildSectorAngle] The maximum sector angle around a node within which the node's children will be placed. This option sets the {@link RadialLayout#maximumChildSectorAngle} property on the created object.
@param [options.minimumBendAngle] The minimum bend angle (in degrees) between two adjacent edge segments. This option sets the {@link RadialLayout#minimumBendAngle} property on the created object.
@param [options.centerNodesPolicy] The policy that determines which nodes are selected as roots of the (virtual) tree for the layout process and are set in the center of the circles. This option sets the {@link RadialLayout#centerNodesPolicy} property on the created object.
@param [options.centerNodesDpKey] The {@link IDataProvider} key used for looking up the selected nodes that should be placed in the center. This option sets the {@link RadialLayout#centerNodesDpKey} property on the created object.
@param [options.layeringStrategy] The layering strategy that determines how nodes are distributed into different layers. This option sets the {@link RadialLayout#layeringStrategy} property on the created object.
@param [options.edgeRoutingStrategy] The edge routing strategy. This option sets the {@link RadialLayout#edgeRoutingStrategy} property on the created object.
@param [options.considerNodeLabels] Whether or not node labels are taken into account when calculating node positions. This option sets the {@link RadialLayout#considerNodeLabels} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link RadialLayout#orientationLayoutEnabled} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link MultiStageLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link MultiStageLayout#componentLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout Online Documentation}*/
constructor(options?: {
minimumNodeToNodeDistance?: number;
layerSpacing?: number;
minimumLayerDistance?: number;
maximumChildSectorAngle?: number;
minimumBendAngle?: number;
centerNodesPolicy?: CenterNodesPolicy | CenterNodesPolicyStringValues;
centerNodesDpKey?: any | null;
layeringStrategy?: RadialLayoutLayeringStrategy | RadialLayoutLayeringStrategyStringValues;
edgeRoutingStrategy?: RadialLayoutEdgeRoutingStrategy | RadialLayoutEdgeRoutingStrategyStringValues;
considerNodeLabels?: boolean;
orientationLayoutEnabled?: boolean;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
selfLoopRouterEnabled?: boolean;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
componentLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Calculates a radial arrangement for the given graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Data acceptor key for publishing the circle ID and sector data
For each node, the ID of the circle and data regarding the circular sector on which it was placed will be stored.
<h2>Note</h2>
If no {@link IDataProvider} is registered with this key, information about circle ID and sector data will be
dropped.
<b>Warning:</b> The layout algorithm will not take into account any settings that are initially specified by a {@link RadialLayoutNodeInfo}
provided by the {@link IDataProvider} registered with this key. It is only intended to publish information
after the layout has been calculated.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23NODE_INFO_DP_KEY Online Documentation}*/
static NODE_INFO_DP_KEY: NodeDpKey<RadialLayoutNodeInfo> | null;
/**
Data provider key for providing user-defined layer/circle IDs
The {@link IDataProvider} registered with this key provides for each node the layer/circle ID to which it
belongs. Nodes with the same ID are placed on the same circle while the circles are sorted according to their IDs such
that a smaller ID means that the circle has a smaller radius and is closer to the center of the radial layout.
When providing user-defined IDs, the center nodes (nodes on the innermost circle) are not determined by the specified {@link RadialLayout#centerNodesPolicy center nodes policy}
anymore. Instead, the nodes with the smallest given ID (i.e. <code>0</code>) define the set of center nodes.
<b>Warning:</b> This user-defined IDs are only considered if strategy {@link RadialLayoutLayeringStrategy#USER_DEFINED} is specified.
@see RadialLayoutLayeringStrategy#USER_DEFINED
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23LAYER_ID_DP_KEY Online Documentation}*/
static LAYER_ID_DP_KEY: NodeDpKey<number> | null;
/**
Gets or sets the minimum distance between two neighboring nodes of the same circle.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23minimumNodeToNodeDistance Online Documentation}*/
minimumNodeToNodeDistance: number;
/**
Gets or sets the spacing between two consecutive circles (layers).
The spacing is a non-negative value, where <code>0</code> means that no spacing is considered.
The radius of each circle will be a multiple of this value.
@throws {Exception} <b>(ArgumentError)</b> if the spacing value is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23layerSpacing Online Documentation}*/
layerSpacing: number;
/**
Gets or sets the minimum distance between two circles.
@throws {Exception} <b>(ArgumentError)</b> if the minimum layer distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23minimumLayerDistance Online Documentation}*/
minimumLayerDistance: number;
/**
Gets or sets the maximum sector angle around a node within which the node's children will be placed.
Based on the location of a given node, the sector of its children that lie on the next outer layer is described by this
angle. The vertex of the angle is located in the center of the node while its sides point away from the inner circles.
The child nodes are placed inside this angle. If there is not enough space, the radius of the child circle is
increased.
@throws {Exception} <b>(ArgumentError)</b> if the sector angle is negative or greater than <code>360</code> degrees
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23maximumChildSectorAngle Online Documentation}*/
maximumChildSectorAngle: number;
/**
Gets or sets the minimum bend angle (in degrees) between two adjacent edge segments.
The minimum bend angle is defined to be a value between <code>0</code> and <code>90</code> degrees.
<h2>Note</h2>
Increasing this value reduces the number of bends such that the edge path becomes less smooth.
<h2>Note</h2>
This setting has an influence only if {@link RadialLayoutEdgeRoutingStrategy#ARC} is selected as {@link RadialLayout#edgeRoutingStrategy routing strategy}.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum bend angle is negative or greater than <code>90</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23minimumBendAngle Online Documentation}*/
minimumBendAngle: number;
/**
Gets or sets the policy that determines which nodes are selected as roots of the (virtual) tree for the layout process
and are set in the center of the circles.
<b>Warning:</b> This policy is ignored if the layers (i.e. circles) are specified manually by using strategy
{@link RadialLayoutLayeringStrategy#USER_DEFINED}. Instead, the nodes with the smallest provided layer/circle ID determine the
center nodes.
@throws {Exception} <b>(ArgumentError)</b> if an invalid center node policy is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23centerNodesPolicy Online Documentation}*/
centerNodesPolicy: CenterNodesPolicy;
/**
Gets or sets the {@link IDataProvider} key used for looking up the selected nodes that should be placed in the
center.
<b>Warning:</b> This option will only take effect if {@link RadialLayout#centerNodesPolicy center node policy} is set to {@link CenterNodesPolicy#CUSTOM}.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23centerNodesDpKey Online Documentation}*/
centerNodesDpKey: any | null;
/**
Gets or sets the layering strategy that determines how nodes are distributed into different layers.
All nodes of a given layer will be placed on the same circle.
@throws {Exception} <b>(ArgumentError)</b> if an invalid layering strategy is set
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23layeringStrategy Online Documentation}*/
layeringStrategy: RadialLayoutLayeringStrategy;
/**
Gets or sets the edge routing strategy.
<h2>Note</h2>
The edge routing cannot always prevent node-edge overlaps.
@throws {Exception} <b>(ArgumentError)</b> if an invalid edge routing strategy is set
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23edgeRoutingStrategy Online Documentation}*/
edgeRoutingStrategy: RadialLayoutEdgeRoutingStrategy;
/**
Gets or sets whether or not node labels are taken into account when calculating node positions.
Thereby, possible node/node label or node label/node label overlaps are prevented.
<h2>Note</h2>
This option only works correctly if the {@link MultiStageLayout#layoutOrientation layout orientation} is set to {@link LayoutOrientation#TOP_TO_BOTTOM}
(which is the default). Changing the layout orientation for an undirected layout algorithm like the {@link RadialLayout}
doesn't make sense.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Sets whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated.
<h2>Note</h2>
If the current layout orientation is
{@link LayoutOrientation#TOP_TO_BOTTOM}, the orientation of the layout will not be modified.
<b>Warning:</b> Based on the undirected layout style, disabling {@link OrientationLayout} has no significant effect.
@see MultiStageLayout#orientationLayoutEnabled
@see MultiStageLayout#orientationLayout
@see MultiStageLayout#layoutOrientation
@see OrientationLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23orientationLayoutEnabled Online Documentation}*/
orientationLayoutEnabled: boolean;
/**
Gets the {@link EdgeBundling} instance that defines the settings of the edge bundling feature.
The specified {@link EdgeBundling} defines global bundling properties. Settings for individual edges can be
defined by assigning an {@link EdgeBundleDescriptor} to an edge using a {@link IDataProvider} registered
with key {@link EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}.
<b>Warning:</b> Edge bundling may be of low quality if the virtual tree is not drawn planar, i.e., there exist tree-edges that cross
each other. This is more likely to occur if {@link RadialLayoutLayeringStrategy#HIERARCHICAL hierarchic layering strategy} is selected.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayout%23edgeBundling Online Documentation}*/
readonly edgeBundling: EdgeBundling | null;
static $class: Class;
private $_RadialLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RadialLayout;
}
export interface PortPlacementStage extends LayoutStageBase {
}
/**
The {@link PortPlacementStage} assigns edges to the ports specified by {@link PortConstraint}s or
{@link PortCandidate}s after calling the {@link LayoutStageBase#coreLayout core layout algorithm}.
It can be used for adjusting ports for layout algorithms that cannot handle {@link PortConstraint}s.
If {@link PortCandidate}s are assigned for edges and nodes, this stage tries to match them. When there is no
match, the {@link PortCandidate} with the lowest costs specified for the edge is chosen. While
{@link PortConstraint}s are also matched to
{@link PortCandidate}s at nodes and edges, it is not recommended to mix them because they belong to different
concepts.
This stage also considers port groups (see {@link PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY} and
{@link PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY}). It assigns the same port to edges that are port-grouped
at a common node. Importantly, port grouping is prioritized over other constraints, for example, if two port-grouped
edges do not have at least one common {@link PortCandidate} or the same
{@link PortConstraint}, then the port of one of the edges will be chosen for both, violating the constraint of the
other edge.
Furthermore, option {@link PortPlacementStage#pathCorrection} allows to specify whether the edge path should be corrected after correcting a port.
It is recommended that the {@link PortPlacementStage} is prepended to the {@link LayoutStageBase#coreLayout core layout algorithm} in order to be
able to process all the edges and especially, self-loops and parallel edges. Note that edges that might be hidden by
other {@link ILayoutStage}s cannot be handled by {@link PortPlacementStage}.
<h2>Note</h2>
This stage can not properly deal with the case that the current ports are outside of the bounds of the adjacent nodes.
The current ports are expected to be on the border or inside the node (e.g. center).
@see PortConstraint
@see PortCandidate
@see PortCandidateSet
@see PortPlacementStage#pathCorrection
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortPlacementStage Online Documentation}*/
export class PortPlacementStage {
/**
Creates a new {@link PortPlacementStage} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param [coreLayout=null] The core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortPlacementStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link PortPlacementStage} instance with an optional {@link LayoutStageBase#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] The core layout algorithm.
@param [options.pathCorrection] Whether or not this {@link ILayoutStage} corrects the edge paths after moving the port to the location specified by a {@link PortConstraint} or {@link PortCandidate}s. This option sets the {@link PortPlacementStage#pathCorrection} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortPlacementStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
pathCorrection?: boolean;
});
/**
Corrects the ports of the edges considering {@link PortConstraint}s and {@link PortCandidate}s.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortPlacementStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets whether or not this {@link ILayoutStage} corrects the edge paths after moving the port to the
location specified by a {@link PortConstraint} or {@link PortCandidate}s.
If path correction is enabled, the edge will be rerouted within a specific area around its source/target node. Outside
this area, it will keep the route that was calculated by the
{@link LayoutStageBase#coreLayout core layout algorithm}. If this feature is disabled, only the ports are moved which may be more suitable for initially
straight edges.
<h2>Note</h2>
The path correction uses a simple orthogonal routing strategy that is fast but may produce overlapping edge segments as
well as edge-node intersections.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortPlacementStage%23pathCorrection Online Documentation}*/
pathCorrection: boolean;
static $class: Class;
private $_PortPlacementStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PortPlacementStage;
}
export interface BusRepresentations extends YObject {
}
/**
This class provides helper methods to convert between buses given by complete subgraphs and buses represented by
specific <em>hub</em> nodes.
In a representation by means of hubs, each connected subgraph induced by hubs establishes a bus. All regular nodes
(non-hubs) which are connected to hubs of the same component are end-nodes of the same bus. In other words, nodes which
are reachable on paths consisting of only hubs belong to the same bus. Of course, a node can be an end-node of more than
one bus.
Since a bus models a group of nodes in which every node is connected to every other node, it can be represented as a
complete subgraph of these nodes. To distinguish buses in this representation, each edge must be associated with a
unique ID which identifies the bus to which it is associated. This is the representation expected by
{@link BusRouter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRepresentations Online Documentation}*/
export class BusRepresentations {
/**
Changes the representation of buses from hubs to complete subgraphs by creating new edges between regular nodes and
removing the hubs.
This method respects multiple connections of nodes and creates appropriate
{@link BusRouterBusDescriptor}s consisting of the bus ID as well as source and target group IDs.
The path of each new edge follows the route defined by the component of the hubs. If the subgraph of hubs is not a tree,
these paths are ambiguous. In this case, an arbitrary tree is computed from the hub's subgraph which defines all the
paths.
@param graph the graph of the bus
@param hubEdgesLists an array of {@link EdgeList}s each containing the edges of a bus
@param hubMarker a {@link IDataProvider} that marks hub nodes
@param fixedMarker a {@link IDataProvider} that marks fixed edges
@param descriptorAcceptor a {@link IDataAcceptor} for storing the created bus descriptors
@returns a list of the created edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRepresentations%23replaceHubsBySubgraph(LayoutGraph%2CArray%3CEdgeList%3E%2CIDataProvider%2CIDataProvider%2CIDataAcceptor) Online Documentation}*/
static replaceHubsBySubgraph(graph: LayoutGraph | null, hubEdgesLists: EdgeList[] | null, hubMarker: IDataProvider | null, fixedMarker: IDataProvider | null, descriptorAcceptor: IDataAcceptor | null): EdgeList | null;
/**
Changes the representation of buses from hubs to complete subgraphs.
This method delegates to {@link BusRepresentations#replaceHubsBySubgraph}.
@param graph the graph of the bus
@param hubMarker a {@link IDataProvider} that marks hub nodes
@param descriptorAcceptor a {@link IDataAcceptor} for storing the created bus descriptors
@returns a list of the created edges
@see BusRepresentations#replaceHubsBySubgraph
@see BusRepresentations#toEdgeLists
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRepresentations%23replaceHubsBySubgraph(LayoutGraph%2CIDataProvider%2CIDataAcceptor) Online Documentation}*/
static replaceHubsBySubgraph(graph: LayoutGraph | null, hubMarker: IDataProvider | null, descriptorAcceptor: IDataAcceptor | null): EdgeList | null;
/**
Changes the representation of buses from complete subgraphs to hubs by replacing intersection points by hubs.
The edges of the given graph must form an orthogonal, cycle-free bus, otherwise an {@link Exception InvalidOperationError} is thrown.
@param graph the graph where each bus is represented by a complete subgraph of the corresponding nodes
@param edgeCursor an {@link IEdgeCursor} of the regular edges
@param descriptorProvider a {@link IDataProvider} that provides a {@link BusRouterBusDescriptor} for each edge
@param busIDAcceptor an optional {@link IDataAcceptor} that stores the bus ID for each new edge
@throws {Exception} <b>(InvalidOperationError)</b> if the path of an edge is not orthogonal or if some paths form a cycle
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRepresentations%23replaceSubgraphByHubs(LayoutGraph%2CIEdgeCursor%2CIDataProvider%2CIDataAcceptor) Online Documentation}*/
static replaceSubgraphByHubs(graph: LayoutGraph | null, edgeCursor: IEdgeCursor | null, descriptorProvider: IDataProvider | null, busIDAcceptor: IDataAcceptor | null): void;
/**
Calculates for every bus represented by hubs a list of all of its edges.
<h2>Note</h2>
This method cannot return isolated hubs.
<h2>Note</h2>
Each edge between two regular nodes establishes a separate bus.
@param graph the graph where buses are represented by means of hubs
@param hubMarker a {@link IDataProvider} which marks hub nodes
@returns an array of {@link EdgeList}s where each list contains all edges of a bus
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRepresentations%23toEdgeLists(Graph%2CIDataProvider) Online Documentation}*/
static toEdgeLists(graph: Graph | null, hubMarker: IDataProvider | null): EdgeList[] | null;
/**
Data acceptor key to transfer a user-defined object from each original edge to the respective newly created edge
Since there is no one-to-one mapping between original and new edges, the data provider of this key is responsible for
the edges whose source endpoint is a regular node.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRepresentations%23SOURCE_ID_DP_KEY Online Documentation}*/
static SOURCE_ID_DP_KEY: EdgeDpKey<any> | null;
/**
Data acceptor key to transfer a user-defined object from each original edge to the respective newly created edge
Since there is no one-to-one mapping between original and new edges, the data provider of this key is responsible for
the edges whose target endpoint is a regular node.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRepresentations%23TARGET_ID_DP_KEY Online Documentation}*/
static TARGET_ID_DP_KEY: EdgeDpKey<any> | null;
static $class: Class;
private $_BusRepresentations: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BusRepresentations;
}
export interface BusRouter extends LayoutStageBase {
}
/**
An edge routing algorithm which routes edges of a graph in an orthogonal bus-style.
Carefully observe that the resulting representation, with many edge segments drawn on top of each other, leaves little
room for a sensible interpretation of edge direction.
<b>Layout Style</b>
Edge segments are bundled to buses. A bus is a segment shared by multiple edges to which shorter segments that connect
to actual nodes are attached. Buses and all other segments are routed orthogonally.
A bus can, for example, be created in parts of a diagram where each node is connected to each other node. There are no
cycles induced by any two edge paths of the same bus, that is, the combination of all edge routes looks like an
orthogonal tree.
The algorithm tries to produce routes where the edges share as much of their paths as possible. It yields long line
segments (so-called <b>backbone segments</b>) where ideally all but the first and last segments of all edge paths are drawn on top of each other
(forming a
<b>bus</b>), with short connections branching off to the nodes
(<b>bus connections</b>). These short connections bundle the respective first or last segments of a node's incident edges.
This algorithm will not modify positions or sizes of nodes in any way, but will route the edges of the graph.
<b>Concept</b>
The algorithm uses a two-phase process:
<ol>
<li><b>Backbone Selection</b>: a set of suitable initial backbone segments is determined.
<li><b>Routing and Recombination</b>: each initial backbone segment is connected to all other backbone segments and to each node by using orthogonal edge
paths. Then, the resulting structure is reduced to the most optimal structure where backbone segments are long and
connections to the nodes are short.
</ol>
<b>Features</b>
To determine which edges belong to a common bus, a mapping that assigns a bus ID to each edge must be specified using
{@link BusRouterBusDescriptor}s. A {@link IDataProvider} holding {@link BusRouterBusDescriptor} instances is
expected to be registered with the graph using the
{@link BusRouter#EDGE_DESCRIPTOR_DP_KEY descriptor key}. In the absence of an individual bus ID for an edge, a bus consisting only of the single edge is
created.
This algorithm supports {@link PortConstraint}s as well as
{@link PortCandidate}s to control where edges should connect to nodes.
Note that if edges of the same bus connect to a common node but have inconsistent or contradicting port
constraints/candidates, then any of these constraints/candidates can determine the actual location of the common port.
The same applies for edges that, in addition, belong to the same edge group.
Also, the cardinality defined with a {@link PortCandidateSet} object is interpreted in terms of different bus IDs
(group IDs) instead of number of edges.
This algorithm supports incremental edge routing, that is, extending or updating an already existing bus-style
representation. This is useful to rearrange existing edges or to include additional edges in an existing bus.
Incremental routing is supported by denoting so-called fixed edges using the
{@link BusRouterBusDescriptor#fixed corresponding BusDescriptor property}. The paths of edges which are not marked as fixed are calculated by the algorithm.
The structure induced by the fixed edges must be orthogonal and cycle-free.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter Online Documentation}*/
export class BusRouter {
/**
Creates a new instance of {@link BusRouter} with default settings.
@param [options] The parameters to pass.
@param [options.gridSpacing] The equidistant spacing between the horizontal and vertical grid lines. This option sets the {@link BusRouter#gridSpacing} property on the created object.
@param [options.gridRouting] Whether or not to route edge segments on grid lines only. This option sets the {@link BusRouter#gridRouting} property on the created object.
@param [options.minimumDistanceToNode] The minimum distance between edge segments and nodes. This option sets the {@link BusRouter#minimumDistanceToNode} property on the created object.
@param [options.minimumDistanceToEdge] The minimum distance between any two edge segments. This option sets the {@link BusRouter#minimumDistanceToEdge} property on the created object.
@param [options.crossingCost] The cost for each edge crossing. This option sets the {@link BusRouter#crossingCost} property on the created object.
@param [options.rerouting] Whether or not to perform an additional step to reroute the edges such that the number of edge crossings is reduced. This option sets the {@link BusRouter#rerouting} property on the created object.
@param [options.preferredBackboneSegmentCount] The maximum number of selected backbone segments with the same orientation. This option sets the {@link BusRouter#preferredBackboneSegmentCount} property on the created object.
@param [options.minimumBackboneSegmentLength] The preferred minimum length of a backbone segment. This option sets the {@link BusRouter#minimumBackboneSegmentLength} property on the created object.
@param [options.minimumBusConnectionsCount] The minimum number of bus connections a backbone segment must have. This option sets the {@link BusRouter#minimumBusConnectionsCount} property on the created object.
@param [options.removeCollinearBends] Whether or not collinear bends are removed from the layout. This option sets the {@link BusRouter#removeCollinearBends} property on the created object.
@param [options.affectedEdgesDpKey] The key to register a {@link IDataProvider} for marking edges as <em>selected</em>. This option sets the {@link BusRouter#affectedEdgesDpKey} property on the created object.
@param [options.scope] The scope for this routing algorithm that determines which edges are routed. This option sets the {@link BusRouter#scope} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter Online Documentation}*/
constructor(options?: {
gridSpacing?: number;
gridRouting?: boolean;
minimumDistanceToNode?: number;
minimumDistanceToEdge?: number;
crossingCost?: number;
rerouting?: boolean;
preferredBackboneSegmentCount?: number;
minimumBackboneSegmentLength?: number;
minimumBusConnectionsCount?: number;
removeCollinearBends?: boolean;
affectedEdgesDpKey?: any | null;
scope?: EdgeRouterScope | EdgeRouterScopeStringValues;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Calculates bus-like routes for the edges in the given input graph.
<h2>Note</h2>
The given graph will not be copied during the edge routing process and the result will be immediately applied to the
input graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for specifying a bus descriptor object for each edge
The {@link BusRouterBusDescriptor} for an edge provides the edge's bus ID, its optional group IDs and whether or not the
edge is fixed.
<h2>Note</h2>
The bus ID of an edge is needed to determine which edges belong to a common bus. In the absence of an individual bus ID
for an edge, a bus consisting only of the single edge is created.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23EDGE_DESCRIPTOR_DP_KEY Online Documentation}*/
static EDGE_DESCRIPTOR_DP_KEY: EdgeDpKey<BusRouterBusDescriptor> | null;
/**
Data provider key for specifying the edge subset to be routed
This key is used if no custom key for specifying the subset is defined using {@link BusRouter#affectedEdgesDpKey}.
<b>Warning:</b> The specified subset is only considered if the scope is set to {@link EdgeRouterScope#ROUTE_AFFECTED_EDGES}.
@see BusRouter#scope
@see BusRouter#affectedEdgesDpKey
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23DEFAULT_AFFECTED_EDGES_DP_KEY Online Documentation}*/
static DEFAULT_AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
Positive values greater than <code>2</code> are allowed. Positive values less than <code>2</code> are ignored, while negative values are mapped to
their absolute value.
<b>Warning:</b> This option will take effect only if {@link BusRouter#gridRouting grid routing} is enabled.
@see BusRouter#gridRouting
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23gridSpacing Online Documentation}*/
gridSpacing: number;
/**
Gets or sets whether or not to route edge segments on grid lines only.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23gridRouting Online Documentation}*/
gridRouting: boolean;
/**
Gets or sets the minimum distance between edge segments and nodes.
Positive values greater than <code>2</code> are allowed. Positive values less than <code>2</code> are ignored, while negative values are mapped to
their absolutes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23minimumDistanceToNode Online Documentation}*/
minimumDistanceToNode: number;
/**
Gets or sets the minimum distance between any two edge segments.
The edge routing algorithm adheres to this value if possible, but reduces the distance value selectively, i.e., only for
a currently processed edge, when there is not enough space to find a path with the proper value.
Positive values greater than <code>4</code> are allowed. Positive values less than <code>4</code> are ignored, while negative values are mapped to
their absolute values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23minimumDistanceToEdge Online Documentation}*/
minimumDistanceToEdge: number;
/**
Gets or sets the cost for each edge crossing.
A cost value of <code>n</code> means that it is more profitable for a path to change its direction <code>n</code> times rather than crossing the
path of an edge. If the cost value is set to <code>0.0</code>, no global crossing optimization is performed.
The cost is defined to be a non-negative value.
<h2>Note</h2>
This setting is taken into account only in the routing and recombination phase and does not affect the selection of the
initial backbone segments.
@throws {Exception} <b>(ArgumentError)</b> if the given cost value is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23crossingCost Online Documentation}*/
crossingCost: number;
/**
Gets or sets whether or not to perform an additional step to reroute the edges such that the number of edge crossings is
reduced.
This features does not guarantee that the number of crossings will be the minimal.
<h2>Note</h2>
The rerouting step will only take effect if a value greater than <code>0.0</code> is assigned to the {@link BusRouter#crossingCost crossing cost}.
<h2>Note</h2>
The rerouting is taken into account only in the routing and recombination phase and does not affect the selection of the
initial backbone segments.
@see BusRouter#crossingCost
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23rerouting Online Documentation}*/
rerouting: boolean;
/**
Gets or sets the maximum number of selected backbone segments with the same orientation.
This setting defines the number of backbone segments of the same orientation which are computed by the backbone
selection phase. The final number of backbone segments may be different due to changes in the routing and recombination
phase.
The number must be a value greater than or equal to <code>1</code>.
<h2>Note</h2>
The more initial backbone segments, the longer the running time. It is not recommended to specify large values for this
setting.
@throws {Exception} <b>(ArgumentError)</b> if the given preferred number is smaller than <code>1</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23preferredBackboneSegmentCount Online Documentation}*/
preferredBackboneSegmentCount: number;
/**
Gets or sets the preferred minimum length of a backbone segment.
This number defines the minimum length of backbone segments which are computed by the backbone selection phase. Some of
the final backbone segments may be shorter due to changes in the routing and recombination phase.
The minimum length is defined to be a value greater than or equal to <code>1.0</code>.
<h2>Note</h2>
The minimum length should be at least as large as the typical distance between nodes to avoid small backbone segments.
It is reasonable to set this according to the dimension of the graph's {@link LayoutGraph#getBoundingBox bounding box}.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum length is smaller than <code>1.0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23minimumBackboneSegmentLength Online Documentation}*/
minimumBackboneSegmentLength: number;
/**
Gets or sets the minimum number of bus connections a backbone segment must have.
If a backbone segment has less connections, it is removed and the affected nodes connect to another backbone segment.
The minimum connection count must be a value greater than or equal to <code>1</code>.
<h2>Note</h2>
This setting is taken into account only in the routing and recombination phase and does not affect the selection of the
initial backbone segments.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum count is smaller than <code>1</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23minimumBusConnectionsCount Online Documentation}*/
minimumBusConnectionsCount: number;
/**
Gets or sets whether or not collinear bends are removed from the layout.
A collinear bend is a bend that lies on a common line with its predecessor bend and successor bend.
If an edge has a collinear bend, there is another edge which has a real bend at this point, i.e., the bend location is
an intersection of the bus. Therefore, it may be advantageous for some applications to keep such bends.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23removeCollinearBends Online Documentation}*/
removeCollinearBends: boolean;
/**
Gets or sets the key to register a {@link IDataProvider} for marking edges as <em>selected</em>.
If the {@link BusRouter#scope scope} is set to {@link EdgeRouterScope#ROUTE_AFFECTED_EDGES}, only the edges for which the registered {@link IDataProvider}
returns <code>true</code> will be routed. All other edges will be considered to have fixed routes.
@throws {Exception} <b>(ArgumentError)</b> if the specified key is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
/**
Gets or sets the scope for this routing algorithm that determines which edges are routed.
@throws {Exception} <b>(ArgumentError)</b> if an unknown scope is given
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouter%23scope Online Documentation}*/
scope: EdgeRouterScope;
static $class: Class;
private $_BusRouter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BusRouter;
}
export interface ChannelEdgeRouter extends LayoutStageBase {
}
/**
This edge routing algorithm generates orthogonal routes for the edges of the graph.
Compared to the routing algorithm implemented by class
{@link EdgeRouter}, this implementation is usually faster but supports less constraints and, by default, may
produce node-edge overlaps.
<b>Layout Style</b>
Edges are routed in an orthogonal fashion, i.e., edge paths consist only of vertical and horizontal segments.
During the routing process, the positions of the nodes are considered to be fixed and the routing algorithm will not
modify their locations or their sizes in any way.
The edge routing algorithm can be applied wherever it is required to route the edges as orthogonal segments, while
keeping the positions of the nodes in the diagram fixed. Some potential applications include electric circuit design,
floor planning, UML class diagrams/inheritance diagrams and navigation maps.
<b>Concept</b>
This edge routing algorithm combines two strategic steps of edge routing and executes them one after the other. The
first strategy is called path finder strategy and will route the edges, potentially with edge overlaps. The second
strategy will then split overlapping edge segments inside their channels and distribute them according to the specific
distribution strategy.
The default path finding strategy is {@link OrthogonalPatternEdgeRouter}. The default edge distribution strategy
is {@link OrthogonalSegmentDistributionStage}.
<b>Features</b>
This class itself has no special routing options except from specifying the {@link ChannelEdgeRouter#pathFinderStrategy path finding} and {@link ChannelEdgeRouter#edgeDistributionStrategy edge distribution} strategy.
Most of the features like minimum element distances, grid spacing and path restrictions have to be set on the used
instances for the two routing steps.
For incremental edge routing, {@link IDataProvider} key {@link ChannelEdgeRouter#AFFECTED_EDGES_DP_KEY}
has to be registered with the graph to mark all edges that should be routed by the algorithm. The routes of the other
edges remain unchanged. If there is no such key registered, the algorithm routes all edges.
<h2>Note</h2>
When exchanging only one of the strategies, make sure, that their algorithms will use the same (sub)set of graph
objects. The strategies bound to this routing algorithm, by default, will use key {@link ChannelEdgeRouter#AFFECTED_EDGES_DP_KEY}
to determine the affected edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouter Online Documentation}*/
export class ChannelEdgeRouter {
/**
Creates a new {@link ChannelEdgeRouter} instance with default settings.
@param [options] The parameters to pass.
@param [options.pathFinderStrategy] The strategy for the path finding step. This option sets the {@link ChannelEdgeRouter#pathFinderStrategy} property on the created object.
@param [options.edgeDistributionStrategy] The strategy for the edge distribution step. This option sets the {@link ChannelEdgeRouter#edgeDistributionStrategy} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouter Online Documentation}*/
constructor(options?: {
pathFinderStrategy?: ILayoutAlgorithm | null;
edgeDistributionStrategy?: ILayoutAlgorithm | null;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Performs the orthogonal routing of the edges of the input graph.
<h2>Note</h2>
The given graph will not be copied during the edge routing process and the result will be immediately applied to the
input graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouter%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Checks the sizes of the nodes to be non-zero.
@param g The graph to check.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouter%23checkNodeSize(LayoutGraph) Online Documentation}*/
checkNodeSize(g: LayoutGraph | null): void;
/**
Data provider key for marking the edges that should be routed
If this key is not registered with the graph, the algorithm will route all edges.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouter%23AFFECTED_EDGES_DP_KEY Online Documentation}*/
static AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets the strategy for the path finding step.
This step routes the edges and may produce edge overlaps which are later resolved in the {@link ChannelEdgeRouter#edgeDistributionStrategy edge distribution} step.
<h2>Note</h2>
When exchanging only one of the strategies, make sure, that their algorithms will use the same (sub)set of graph
objects. The strategies bound to this routing algorithm, by default, will use key {@link ChannelEdgeRouter#AFFECTED_EDGES_DP_KEY}
to determine the affected edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouter%23pathFinderStrategy Online Documentation}*/
pathFinderStrategy: ILayoutAlgorithm | null;
/**
Gets or sets the strategy for the edge distribution step.
This step distributes overlapping edge segments in their channel. The channel in which the segments can be distributed
is defined by the surrounding graph elements.
<h2>Note</h2>
When exchanging only one of the strategies, make sure, that their algorithms will use the same (sub)set of graph
objects. The strategies bound to this routing algorithm, by default, will use key {@link ChannelEdgeRouter#AFFECTED_EDGES_DP_KEY}
to determine the affected edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouter%23edgeDistributionStrategy Online Documentation}*/
edgeDistributionStrategy: ILayoutAlgorithm | null;
static $class: Class;
private $_ChannelEdgeRouter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ChannelEdgeRouter;
}
export interface CurveEdgeLayoutDescriptor extends YObject {
}
/**
This class is used by {@link CurveRoutingStage} to provide routing details for the edges of the graph.
A {@link CurveEdgeLayoutDescriptor} instance can be specified individually for single edges using a {@link IDataProvider}
that returns an {@link CurveEdgeLayoutDescriptor} instance for each edge of the graph, or <code>null</code> if no {@link CurveEdgeLayoutDescriptor}
is bound to an edge. The {@link IDataProvider} is registered with the graph using key
{@link CurveRoutingStage#CURVE_EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
This class is designed such that it allows future additions of new getter methods.
@see CurveRoutingStage
@see CurveRoutingStage#CURVE_EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveEdgeLayoutDescriptor Online Documentation}*/
export class CurveEdgeLayoutDescriptor {
/**
Creates a copy of this {@link CurveEdgeLayoutDescriptor} instance.
@returns a copy of this {@link CurveEdgeLayoutDescriptor} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveEdgeLayoutDescriptor%23createCopy() Online Documentation}*/
createCopy(): CurveEdgeLayoutDescriptor | null;
/**
Gets or sets the minimum length of the first segment of the edge path (at the source node).
The minimum length must be greater than or equal to <code>0</code>.
<b>Warning:</b> The first segment length is ignored when using source connection style {@link CurveConnectionStyle#ORGANIC}.
<h2>Note</h2>
The first segment length starts, if present, at the {@link NodeHalo halo} of the source node.
<h2>Note</h2>
The resulting edge routes have nicer curves when the minimum length is smaller than the actual segment. Therefore, it is
recommended to configure the {@link LayoutStageBase#coreLayout core layout} of {@link CurveRoutingStage} to have a larger minimum first segment
length.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length of the first segment is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveEdgeLayoutDescriptor%23minimumFirstSegmentLength Online Documentation}*/
minimumFirstSegmentLength: number;
/**
Gets or sets the minimum length of the last segment of the edge path (at the target node).
The minimum length must be greater than or equal to <code>0</code>.
If the minimum length is zero, the curved route may end with a non-orthogonal last segment such that the port location
provided by the {@link LayoutStageBase#coreLayout core layout} may also be changed.
<b>Warning:</b> The last segment length is ignored when using target connection style {@link CurveConnectionStyle#ORGANIC}.
<h2>Note</h2>
The last segment length ends, if present, at the {@link NodeHalo halo} of the target node.
<h2>Note</h2>
The resulting edge routes have nicer curves when the minimum length is smaller than the actual segment. Therefore, it is
recommended to configure the {@link LayoutStageBase#coreLayout core layout} of {@link CurveRoutingStage} to have a larger minimum last segment
length.
@throws {Exception} <b>(ArgumentError)</b> if the minimum length of the last segment is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveEdgeLayoutDescriptor%23minimumLastSegmentLength Online Documentation}*/
minimumLastSegmentLength: number;
/**
Gets or sets the minimum distance to segments belonging to other edges.
The minimum distance must be greater than or equal to <code>0</code>.
<h2>Note</h2>
The minimum edge distance influences how the curves are created. If this distance is already violated in the result of
the {@link LayoutStageBase#coreLayout core layout algorithm}, it has no effect on the curved routes, i.e. the curved route may also violate it.
@throws {Exception} <b>(ArgumentError)</b> if the minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveEdgeLayoutDescriptor%23minimumEdgeToEdgeDistance Online Documentation}*/
minimumEdgeToEdgeDistance: number;
/**
Gets or sets the minimum distance to nodes.
The minimum distance must be greater than or equal to <code>0</code>.
<h2>Note</h2>
The minimum node distance influences how the curves are created. If this distance is already violated in the result of
the {@link LayoutStageBase#coreLayout core layout algorithm}, it has no effect on the curved routes, i.e. the curved route may also violate it.
@throws {Exception} <b>(ArgumentError)</b> if the minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveEdgeLayoutDescriptor%23minimumNodeToEdgeDistance Online Documentation}*/
minimumNodeToEdgeDistance: number;
/**
Gets or sets whether or not the points of the resulting edge path represent cubic bezier control points.
<ul>
<li><b>Disabled (default):</b> Each point of the path represents a normal polyline bend point.
<li><b>Enabled:</b> Four consecutive points p1, p2, p3, p4 of the resulting edge path need to be interpreted such that p1 is the start
point of the cubic bezier curve, p4 is the end point of it and p2, p3 are the two intermediate control points. If p4 is
not the last point of the edge path it is at the same time the start point of the next curve. For example, points p1,
p2, p3, p4, p5, p6, p7 form a path that consists of two curves: the curve defined by p1, p2, p3, p4 and the curve
defined by p4, p5, p6, p7.
</ul>
<h2>Note</h2>
This feature is useful when the visualization of the edge draws bezier curves instead of polyline paths.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveEdgeLayoutDescriptor%23createControlPoints Online Documentation}*/
createControlPoints: boolean;
/**
Gets or sets the style which determines how curved edge paths connect at the source side of the edge.
@throws {Exception} <b>(ArgumentError)</b> if an unknown connection style is given
@see CurveEdgeLayoutDescriptor#targetCurveConnectionStyle
@see CurveConnectionStyle#KEEP_PORT
@see CurveConnectionStyle#ORGANIC
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveEdgeLayoutDescriptor%23sourceCurveConnectionStyle Online Documentation}*/
sourceCurveConnectionStyle: CurveConnectionStyle;
/**
Gets or sets the style which determines how curved edge paths connect at the target side of the edge.
@throws {Exception} <b>(ArgumentError)</b> if an unknown connection style is given
@see CurveEdgeLayoutDescriptor#sourceCurveConnectionStyle
@see CurveConnectionStyle#KEEP_PORT
@see CurveConnectionStyle#ORGANIC
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveEdgeLayoutDescriptor%23targetCurveConnectionStyle Online Documentation}*/
targetCurveConnectionStyle: CurveConnectionStyle;
static $class: Class;
private $_CurveEdgeLayoutDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CurveEdgeLayoutDescriptor;
}
export interface CurveRoutingStage extends LayoutStageBase {
}
/**
{@link CurveRoutingStage} replaces polyline edge paths with curved segments using cubic bezier splines.
This layout stage provides a generic way to add curved edge paths for a layout with polyline routes. It first calls the
{@link LayoutStageBase#coreLayout core layout algorithm} and converts the edge routes as a post-processing step. Note that {@link HierarchicLayout}
and {@link EdgeRouter} offer an integrated curve routing, making the usage of this layout stage unnecessary (see
routing styles {@link HierarchicLayoutEdgeRoutingStyle#CURVED} and {@link EdgeRouterEdgeRoutingStyle#CURVED}).
Using the {@link CurveRoutingStage#scope scope}, it is possible to select for which edges the path is converted to a curved one. This allows to only
change a subset of edges. Furthermore, the {@link CurveEdgeLayoutDescriptor} class allows to individually
configure settings for edges.
This stage is able to consider node and edge labels, see the respective properties {@link CurveRoutingStage#considerNodeLabels} and {@link CurveRoutingStage#considerEdgeLabels}. If edge labels are considered,
the {@link PreferredPlacementDescriptor} is respected too, such that the preferred placement remains correct if it
is already correct after the core layout execution.
This stage also supports edge grouping such that edges that are correctly grouped by the core layout algorithm remain
grouped in the curved layout result. However, it will not create new groupings. The same applies for other features
like, e.g., {@link PortConstraint}s or {@link PortCandidate}s. They are only respected if they are already
respected by the {@link LayoutStageBase#coreLayout core layout algorithm}.
<b>Warning:</b>
Curved segments can only be added to polyline edge routes. If the core layout algorithm creates straight-line edges,
they will stay this way. Thus, this stage requires that the edges have proper polyline/orthogonal routes after the core
layout was executed. It does not find new routes on its own.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage Online Documentation}*/
export class CurveRoutingStage {
/**
Creates a new {@link CurveRoutingStage} instance using the given {@link LayoutStageBase#coreLayout core layout algorithm}.
@param core the core layout algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage Online Documentation}*/
constructor(core: ILayoutAlgorithm | null);
/**
Adds curved segments to the edge routes produced by the {@link LayoutStageBase#coreLayout core layout algorithm}.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for specifying individual edge layout information
If the {@link IDataProvider} registered with this key does not contain a {@link CurveEdgeLayoutDescriptor}
for an edge, then the {@link CurveRoutingStage#defaultEdgeLayoutDescriptor default descriptor} is used.
@see CurveEdgeLayoutDescriptor
@see CurveRoutingStage#defaultEdgeLayoutDescriptor
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage%23CURVE_EDGE_LAYOUT_DESCRIPTOR_DP_KEY Online Documentation}*/
static CURVE_EDGE_LAYOUT_DESCRIPTOR_DP_KEY: EdgeDpKey<CurveEdgeLayoutDescriptor> | null;
/**
Gets or sets the (sub-)set of edges whose routes shall be converted to curved ones.
@throws {Exception} <b>(ArgumentError)</b> if the given scope is unknown
@see CurveRoutingStage#affectedEdgesDpKey
@see CurveRoutingStage#affectedNodesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage%23scope Online Documentation}*/
scope: EdgeRouterScope;
/**
Gets or sets the {@link IDataProvider} key to look up which edges are routed.
If the {@link CurveRoutingStage#scope scope} is set to
{@link EdgeRouterScope#ROUTE_AFFECTED_EDGES}, only the selected edges will be routed, while all other edges will be
considered to have fixed routes.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IDataProvider} key is <code>null</code>
@see CurveRoutingStage#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
/**
Gets or sets the {@link IDataProvider} key to look up the nodes whose incident edges are routed.
If the {@link CurveRoutingStage#scope} scope} is set to
{@link EdgeRouterScope#ROUTE_EDGES_AT_AFFECTED_NODES}, only the edges that are incident to selected nodes will be routed,
while all other edges will be considered to have fixed routes.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IDataProvider} key is <code>null</code>
@see CurveRoutingStage#scope
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage%23affectedNodesDpKey Online Documentation}*/
affectedNodesDpKey: any | null;
/**
Gets or sets the default {@link CurveEdgeLayoutDescriptor} which provides layout information that is considered
when adding curves to the edge path.
This {@link CurveEdgeLayoutDescriptor} is used for all edges that have no individual layout information assigned.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link CurveEdgeLayoutDescriptor} is <code>null</code>
@see CurveRoutingStage#CURVE_EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage%23defaultEdgeLayoutDescriptor Online Documentation}*/
defaultEdgeLayoutDescriptor: CurveEdgeLayoutDescriptor | null;
/**
Gets or sets whether or not edge labels are considered while determining curved edge routes.
The edge labels are taken into account when checking for intersections and the routes are prepared to get an associated
segment that fits the preferred placement of the label.
<h2>Note</h2>
In case the labels are not placed yet, there can be unexpected results. Hence, make sure to have the {@link LayoutStageBase#coreLayout core layout algorithm}
place the labels.
@see PreferredPlacementDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage%23considerEdgeLabels Online Documentation}*/
considerEdgeLabels: boolean;
/**
Gets or sets whether or not node labels are considered while determining spline routes.
The node labels will be taken into account when checking for intersections and the routes will be prepared to get an
associated segment that fits the the preferred placement of the label.
<h2>Note</h2>
In case the labels are not placed yet, there can be unexpected results. Hence, make sure to have the {@link LayoutStageBase#coreLayout core layout algorithm}
place the labels.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStage%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
static $class: Class;
private $_CurveRoutingStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CurveRoutingStage;
}
export interface EdgeBundlingStage extends LayoutStageBase {
}
/**
A {@link ILayoutStage} that bundles the edges of general undirected graphs to avoid visual cluttering.
Bundling together multiple edges means that their common parts are to some degree merged into a bundled part, which is a
useful technique to increase the readability of straight-line graph drawings with a high number of edges that connect a
comparably small number of nodes. This stage provides means to generate bundled edge paths for any given layout, i.e.,
the layout generated by a specified
{@link LayoutStageBase#coreLayout core layout algorithm}. To determine whether two edges can be bundled, the algorithm computes a compatibility measure
that is composed of various criteria (e.g. edges that are closer together or have similar length are more compatible).
Note that this stage ignores the edge bends and handles them all as straight-lines. Furthermore, it doesn't prevent
overlaps between nodes and edges.
<b>Concept</b>
The implemented algorithm is based on the description in: "Force-directed Edge Bundling for Graph Visualization" by
Danny Holten and Jarke J. van Wijk, in Computer Graphics Forum 28(3): 893-990, 2009.
The main idea of the algorithm is that the edges are subdivided into segments and between each pair of consecutive
subdivision points spring and electrostatic forces are exerted. At each iteration step, each subdivision point is moved
according to the combined force exerted on it. The algorithm runs in six cycles and starts with an initial number of
subdivision points for each edge and an initial step size that determines the distance to which each subdivision point
can be moved in the direction of the combined force (exerted on it) at each iteration step of the algorithm. After each
cycle run, the value of the step size is halved.
<b>Features</b>
Global settings of the bundling can be configured using an {@link EdgeBundling} instance. In particular, it is
possible to determine which of the edges are compatible and thus, can be bundled together by specifying the strength of
the edge bundles using the {@link EdgeBundling#bundlingStrength} property. For the particular algorithm, the default value for this property is <code>0.4</code>. The {@link EdgeBundling#bundlingQuality bundling quality}
is related to the distance to which a subdivision point can be moved at each iteration step of the algorithm. The
default value is <code>1</code>.
The individual style of each edge can be configured using an {@link EdgeBundleDescriptor}. A {@link IDataProvider}
can be registered with the input graph with key {@link EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY} to assign
descriptors to edges. This allows, for example, to define which edges should actually be bundled.
To visualize better the result of the edge bundling algorithm, it is recommended that the edges are rendered using an
opacity factor. In this manner, edges that are strongly bundled together will be emphasized more, in comparison to
others that are not bundled.
<h2>Note</h2>
This algorithm doesn't consider the current bends of the edges, i.e., it always handles them as straight-line.
Furthermore, this stage doesn't prevent overlaps between nodes and edges.
<b>Warning:</b> Using high values of for the {@link EdgeBundling#bundlingStrength bundling strength} can significantly affect the performance of the algorithm, especially
for large graphs. A value close to <code>0.4</code> is recommended. On the other hand, using zero values for the {@link EdgeBundling#bundlingStrength bundling strength} or
the {@link EdgeBundling#bundlingQuality bundling quality} leads to straight-line edges.
<b>Warning:</b> The bundle descriptor property {@link EdgeBundleDescriptor#considerDirection} is ignored by this stage.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundlingStage Online Documentation}*/
export class EdgeBundlingStage {
/**
Creates a new {@link EdgeBundlingStage} instance with the given core layout algorithm and default settings.
@param core the core layout algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundlingStage%23EdgeBundlingStage(ILayoutAlgorithm) Online Documentation}*/
constructor(core: ILayoutAlgorithm | null);
/**
Creates a new {@link EdgeBundlingStage} instance with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundlingStage%23EdgeBundlingStage() Online Documentation}*/
constructor();
/**
Applies the edge bundling algorithm after invoking the {@link LayoutStageBase#coreLayout core layout algorithm}.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundlingStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Gets or sets the maximum duration in milliseconds that this bundling algorithm is allowed to run.
The duration needs to be non-negative.
<h2>Note</h2>
Restricting the maximum duration may result in a lower bundling quality. Furthermore, the real runtime may exceed the
maximum duration since the algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the specified duration has a negative value
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundlingStage%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets the {@link EdgeBundling} instance that defines the settings of the edge bundling feature.
The specified {@link EdgeBundling} defines global bundling properties. Settings for individual edges can be
defined by assigning an {@link EdgeBundleDescriptor} to an edge using a {@link IDataProvider} registered
with key {@link EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}.
By default, all edges are bundled (property {@link EdgeBundleDescriptor#bundled} of the default bundle descriptor is enabled). The default for {@link EdgeBundling#bundlingStrength} is <code>0.4</code> and
the default for property {@link EdgeBundling#bundlingQuality} is <code>1.0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundlingStage%23edgeBundling Online Documentation}*/
readonly edgeBundling: EdgeBundling | null;
static $class: Class;
private $_EdgeBundlingStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeBundlingStage;
}
export interface OrganicEdgeRouter extends YObject, ILayoutStage {
}
/**
This edge routing algorithm applies organic routes to the edges of the graph.
<b>Layout Style</b>
Edges are routed organically, i.e. in smooth curves around the nodes observing a {@link OrganicEdgeRouter#minimumDistance minimum distance} to the nodes.
During the routing process, the positions of the nodes are considered to be fixed and the router will not modify their
locations or their sizes in any way.
The edge routing algorithm can be applied whenever edge paths should avoid crossing any nodes in organic or cyclic
layout styles.
<b>Concept</b>
The edge routing algorithm uses a force-directed approach to calculate the edge paths. Nodes are considered to be
repulsive forces while edges will try to become as short as possible.
Each edge is routed separately and is influenced by the nodes in a certain area around it. The algorithm will add bends
to the edge path that are placed by balancing the forces.
The quality of the result highly depends on how much space there is between the nodes. More precisely, the distance
between each pair of nodes should be at least twice the specified {@link OrganicEdgeRouter#minimumDistance minimum distance}. If it is not necessary that the
nodes keep their locations, this can be ensured using a combination of {@link RemoveOverlapsStage} and
{@link OrganicEdgeRouter#createNodeEnlargementStage node enlargement stage}.
<b>Features</b>
{@link OrganicEdgeRouter#minimumDistance} will make edges keep a custom distance to the nodes. However, if there is not enough space between the nodes, this
distance may be undershot (i.e. edges will be closer to nodes).
{@link OrganicEdgeRouter} is able to
{@link OrganicEdgeRouter#keepExistingBends reuse existing bends}. Edges will contain those bends along with other bends added by the layout algorithm.
This edge routing algorithm is realized as a {@link ILayoutStage} which can be applied to a graph directly or
using a {@link OrganicEdgeRouter#coreLayout core layout algorithm}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter Online Documentation}*/
export class OrganicEdgeRouter {
/**
Creates a new {@link OrganicEdgeRouter} with the given {@link OrganicEdgeRouter#coreLayout core layout algorithm}.
@param [coreLayout=null] the core layout algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23OrganicEdgeRouter(ILayoutAlgorithm) Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link OrganicEdgeRouter} with the given {@link OrganicEdgeRouter#coreLayout core layout algorithm}.
@param options The parameters to pass.
@param [options.coreLayout=null] the core layout algorithm
@param [options.edgeNodeOverlapAllowed] Whether or not edges are allowed to cross nodes. This option sets the {@link OrganicEdgeRouter#edgeNodeOverlapAllowed} property on the created object.
@param [options.minimumDistance] The minimum distance the algorithm should guarantee between nodes and non-incident edges. This option sets the {@link OrganicEdgeRouter#minimumDistance} property on the created object.
@param [options.keepExistingBends] Whether or not the initial bend coordinates are kept when determining the edge path. This option sets the {@link OrganicEdgeRouter#keepExistingBends} property on the created object.
@param [options.routeAllEdges] Whether a rerouting step is performed on all edges or just on a subset where distances are violated. This option sets the {@link OrganicEdgeRouter#routeAllEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23OrganicEdgeRouter(ILayoutAlgorithm) Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
edgeNodeOverlapAllowed?: boolean;
minimumDistance?: number;
keepExistingBends?: boolean;
routeAllEdges?: boolean;
});
/**
Performs the organic routing of the edges of the input graph.
<h2>Note</h2>
The given graph will not be copied during the edge routing process and the result will be immediately applied to the
input graph.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Returns a {@link ILayoutStage} which temporarily increases the sizes of the nodes to avoid overlaps.
The edges will keep a greater distance to the nodes. Therefore, they won't cross them.
<b>Warning:</b> The stage should be used in conjunction with {@link RemoveOverlapsStage}, see the provided example.
@returns the {@link ILayoutStage} that resizes the nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23createNodeEnlargementStage() Online Documentation}*/
createNodeEnlargementStage(): ILayoutStage | null;
/**
Data provider key for selecting edges that should be routed
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23AFFECTED_EDGES_DP_KEY Online Documentation}*/
static AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets whether or not edges are allowed to cross nodes.
Allowing edges to overlap with nodes will produce smoother edges, because the edge paths can be closer to the nodes.
@see OrganicEdgeRouter#createNodeEnlargementStage
@see OrganicEdgeRouter#minimumDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23edgeNodeOverlapAllowed Online Documentation}*/
edgeNodeOverlapAllowed: boolean;
/**
Gets or sets the core layout algorithm which arranges the graph before edge routing.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
/**
Gets or sets the minimum distance the algorithm should guarantee between nodes and non-incident edges.
The distance also influences how many bends are added to the path (a higher distance leads to less bends).
The minimum distance is defined to be a non-negative value.
<b>Warning:</b> If there is not enough space, the minimum distance may not be respected.
@throws {Exception} <b>(ArgumentError)</b> if the given minimum distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23minimumDistance Online Documentation}*/
minimumDistance: number;
/**
Gets or sets whether or not the initial bend coordinates are kept when determining the edge path.
The bends are considered as fixed nodes and stay part of the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23keepExistingBends Online Documentation}*/
keepExistingBends: boolean;
/**
Gets or sets whether a rerouting step is performed on all edges or just on a subset where distances are violated.
If only a subset of edges is rerouted, only those edges which cross nodes or come too close to a node are included.
During rerouting, more bends are added to the edges that will be influenced by the repulsive forces.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouter%23routeAllEdges Online Documentation}*/
routeAllEdges: boolean;
static $class: Class;
private $_OrganicEdgeRouter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrganicEdgeRouter;
}
export interface OrthogonalPatternEdgeRouter extends LayoutStageBase {
}
/**
{@link OrthogonalPatternEdgeRouter} routes edges orthogonally such that the resulting layout of the edges consists
only of vertical and horizontal segments.
Note that the location and size of the nodes in a diagram remains unchanged.
The edge router will not try to find a perfect route from source to edge (unlike to what {@link EdgeRouter} does)
but chooses the best path out of a set of fixed paths. The best path out of these possible paths is determined by its
cost. The costs may be influenced by setting several cost factors. The distance that an edge will have from its source
and target node is determined by {@link OrthogonalPatternEdgeRouter#minimumDistance}.
The edges whose paths have to be routed can be defined registering a {@link IDataProvider} with key
{@link OrthogonalPatternEdgeRouter#DEFAULT_AFFECTED_EDGES_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter Online Documentation}*/
export class OrthogonalPatternEdgeRouter {
/**
Creates a new instance of {@link OrthogonalPatternEdgeRouter} with default settings.
@param [options] The parameters to pass.
@param [options.monotonicPathRestriction] The monotonic path restriction that should be applied. This option sets the {@link OrthogonalPatternEdgeRouter#monotonicPathRestriction} property on the created object.
@param [options.affectedEdgesDpKey] The key to register a {@link IDataProvider} which determines the edges that shall be routed by this algorithm. This option sets the {@link OrthogonalPatternEdgeRouter#affectedEdgesDpKey} property on the created object.
@param [options.minimumDistance] The minimum distance that an edge will maintain from its source and target node. This option sets the {@link OrthogonalPatternEdgeRouter#minimumDistance} property on the created object.
@param [options.gridSpacing] The spacing of the grid on which edges are routed. This option sets the {@link OrthogonalPatternEdgeRouter#gridSpacing} property on the created object.
@param [options.gridOrigin] The origin of the grid. This option sets the {@link OrthogonalPatternEdgeRouter#gridOrigin} property on the created object.
@param [options.gridRouting] Whether or not to route the edges on a grid. This option sets the {@link OrthogonalPatternEdgeRouter#gridRouting} property on the created object.
@param [options.edgeCrossingCost] The costs for a crossing between two edges. This option sets the {@link OrthogonalPatternEdgeRouter#edgeCrossingCost} property on the created object.
@param [options.nodeCrossingCost] The costs for edges that cross nodes. This option sets the {@link OrthogonalPatternEdgeRouter#nodeCrossingCost} property on the created object.
@param [options.bendCost] The costs for creating a bend on the edge's path. This option sets the {@link OrthogonalPatternEdgeRouter#bendCost} property on the created object.
@param [options.edgeOverlapCost] The costs for overlapping edge paths. This option sets the {@link OrthogonalPatternEdgeRouter#edgeOverlapCost} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter Online Documentation}*/
constructor(options?: {
monotonicPathRestriction?: MonotonicPathRestriction | MonotonicPathRestrictionStringValues;
affectedEdgesDpKey?: any | null;
minimumDistance?: number;
gridSpacing?: number;
gridOrigin?: YPoint | null;
gridRouting?: boolean;
edgeCrossingCost?: number;
nodeCrossingCost?: number;
bendCost?: number;
edgeOverlapCost?: number;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Routes the edges of the given graph orthogonally.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Calculates the costs for all bends of the given path.
This method is called by {@link OrthogonalPatternEdgeRouter#calculateCost}. The default implementation multiplies
the number of bends with the according {@link OrthogonalPatternEdgeRouter#bendCost costs}. It may be overridden to change the calculation of these costs.
<h2>Note</h2>
Only bends are taken into account. The source and target ports, that are also part of the given path, are excluded.
@param edge the edge for which the costs are calculated
@param path the path of the given edge
@param spc the source {@link PortCandidate} for this edge
@param tpc the target {@link PortCandidate} for this edge
@returns the costs for the bends of this path
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23calculateBendCost(Edge%2CYList%2CPortCandidate%2CPortCandidate) Online Documentation}*/
calculateBendCost(edge: Edge | null, path: YList | null, spc: PortCandidate | null, tpc: PortCandidate | null): number;
/**
Calculates the edge cost of a possible edge path, in order to determine which path is the best, i.e., the cheapest.
This method is called by {@link OrthogonalPatternEdgeRouter#applyLayout} in order to decide which path is the
best. The default implementation considers the edge length, the number of bends, edge crossings,
{@link PortCandidate}s and monotonic path restrictions. It may be overridden to apply a different set of costs or
a different weighting.
@param edge the edge whose costs to calculate
@param path the edge's path
@param spc the source {@link PortCandidate} for this edge
@param tpc the target {@link PortCandidate} for this edge
@returns the sum of all costs for this edge's path
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23calculateCost(Edge%2CYList%2CPortCandidate%2CPortCandidate) Online Documentation}*/
calculateCost(edge: Edge | null, path: YList | null, spc: PortCandidate | null, tpc: PortCandidate | null): number;
/**
Calculates the overall crossing costs of the given path, including edge crossings, edge overlaps and node crossings.
This method is called by
{@link OrthogonalPatternEdgeRouter#calculateCost}. The default implementation will determine overlaps and
crossings between edge segments and nodes and sum up the costs. It may be overridden to use a different combination of
costs.
@param edge the edge for which the costs are calculated
@param path the path of the given edge
@param spc the source {@link PortCandidate} for this edge
@param tpc the target {@link PortCandidate} for this edge
@returns the overall crossing costs of the given path, including edge crossings, edge overlaps and node crossings
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23calculateCrossingCosts(Edge%2CYList%2CPortCandidate%2CPortCandidate) Online Documentation}*/
calculateCrossingCosts(edge: Edge | null, path: YList | null, spc: PortCandidate | null, tpc: PortCandidate | null): number;
/**
Calculates the costs for the length of the given path.
This method is called by {@link OrthogonalPatternEdgeRouter#calculateCost}. The default implementation returns
costs between <code>0</code> for short paths and <code>1</code> for long paths. Hence, it has relatively little impact on the overall costs. The
method may be overridden to introduce a different weighting of the edge length.
@param edge the edge for which the costs are calculated
@param path the path of the given edge
@param spc the source {@link PortCandidate} for this edge
@param tpc the target {@link PortCandidate} for this edge
@returns the costs for the length of the given path
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23calculateEdgeLength(Edge%2CYList%2CPortCandidate%2CPortCandidate) Online Documentation}*/
calculateEdgeLength(edge: Edge | null, path: YList | null, spc: PortCandidate | null, tpc: PortCandidate | null): number;
/**
Calculates the costs for the chosen {@link PortCandidate}s.
This method is called by {@link OrthogonalPatternEdgeRouter#calculateCost}. The default implementation will return
the {@link PortCandidate#cost candidates' costs}. It may be overridden to change the calculation of these costs.
@param edge the edge for which the costs are calculated
@param path the path of the given edge
@param spc the source {@link PortCandidate} for this edge
@param tpc the target {@link PortCandidate} for this edge
@returns the costs for the {@link PortCandidate}s
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23calculatePortCandidateCost(Edge%2CYList%2CPortCandidate%2CPortCandidate) Online Documentation}*/
calculatePortCandidateCost(edge: Edge | null, path: YList | null, spc: PortCandidate | null, tpc: PortCandidate | null): number;
/**
Calculates the extra penalty that is added to the path's cost if the edge is a self-loop and source and target ports
have the same direction.
This method is called by
{@link OrthogonalPatternEdgeRouter#calculateCost}. The default implementation adds costs for another bend if
source and target ports share the same node side. In this manner, paths with different source and target directions are
cheaper and thus preferred. The method may be overridden to change the calculation of this penalty.
@param edge the edge for which the costs are calculated
@param path the path of the given edge
@param spc the source {@link PortCandidate} for this edge
@param tpc the target {@link PortCandidate} for this edge
@returns an extra penalty for specific self-loop paths
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23calculateSelfLoopSelfSidePenaltyCost(Edge%2CYList%2CPortCandidate%2CPortCandidate) Online Documentation}*/
calculateSelfLoopSelfSidePenaltyCost(edge: Edge | null, path: YList | null, spc: PortCandidate | null, tpc: PortCandidate | null): number;
/**
Checks the sizes of the nodes to be non-zero.
@param g The graph to check.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23checkNodeSize(LayoutGraph) Online Documentation}*/
checkNodeSize(g: LayoutGraph | null): void;
/**
Data provider key for determining which edges are routed
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23DEFAULT_AFFECTED_EDGES_DP_KEY Online Documentation}*/
static DEFAULT_AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets the monotonic path restriction that should be applied.
@throws {Exception} <b>(ArgumentError)</b> if the specified path restriction is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23monotonicPathRestriction Online Documentation}*/
monotonicPathRestriction: MonotonicPathRestriction;
/**
Gets or sets the key to register a {@link IDataProvider} which determines the edges that shall be routed by
this algorithm.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IDataProvider} key is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
/**
Gets or sets the minimum distance that an edge will maintain from its source and target node.
The distance needs to be non-negative.
<h2>Note</h2>
When setting the minimum distance to zero, edge segments may exactly overlap the borders of the source or target node.
For example, a vertical segment can run on the left or right node border.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23minimumDistance Online Documentation}*/
minimumDistance: number;
/**
Gets or sets the spacing of the grid on which edges are routed.
The spacing between two grid lines must be at least <code>2</code>.
<b>Warning:</b> The spacing will only be considered if {@link OrthogonalPatternEdgeRouter#gridRouting grid routing is enabled}.
@throws {Exception} <b>(ArgumentError)</b> if the specified grid spacing is less than <code>2</code>
@see OrthogonalPatternEdgeRouter#gridRouting
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23gridSpacing Online Documentation}*/
gridSpacing: number;
/**
Gets or sets the origin of the grid.
<h2>Note</h2>
Grid routing has lower priority than the
{@link OrthogonalPatternEdgeRouter#minimumDistance minimum distance}. Thus, it is possible that edges are not routed on the grid if there is not enough space between two
borders.
<b>Warning:</b> The spacing will only be considered if {@link OrthogonalPatternEdgeRouter#gridRouting grid routing is enabled}.
@throws {Exception} <b>(ArgumentError)</b> if the specified point is <code>null</code>
@see OrthogonalPatternEdgeRouter#gridRouting
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23gridOrigin Online Documentation}*/
gridOrigin: YPoint | null;
/**
Gets or sets whether or not to route the edges on a grid.
The grid can be defined using methods {@link OrthogonalPatternEdgeRouter#gridOrigin} and {@link OrthogonalPatternEdgeRouter#gridSpacing}.
<h2>Note</h2>
The edges' source and target ports are not reassigned if they have fixed coordinates. Therefore, the first and last
segments of an edge might not be routed on the grid. Use weak
{@link PortConstraint}s or assign coordinates that already lie on the grid for these ports if grid routing is
desired.
<h2>Note</h2>
Grid routing has lower priority than the
{@link OrthogonalPatternEdgeRouter#minimumDistance minimum distance}. Thus, it is possible that edges are not routed on the grid if there is not enough space between two
borders.
@see OrthogonalPatternEdgeRouter#gridOrigin
@see OrthogonalPatternEdgeRouter#gridSpacing
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23gridRouting Online Documentation}*/
gridRouting: boolean;
/**
Gets or sets the costs for a crossing between two edges.
These costs are used for finding the best path out of the predefined paths from which the router can choose.
The costs need to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified costs are negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23edgeCrossingCost Online Documentation}*/
edgeCrossingCost: number;
/**
Gets or sets the costs for edges that cross nodes.
These costs are used for finding the best path out of the predefined paths from which the router can choose.
The costs need to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified costs are negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23nodeCrossingCost Online Documentation}*/
nodeCrossingCost: number;
/**
Gets or sets the costs for creating a bend on the edge's path.
These costs are used for finding the best path out of the predefined paths from which the router can choose.
The costs need to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified costs are negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23bendCost Online Documentation}*/
bendCost: number;
/**
Gets or sets the costs for overlapping edge paths.
These costs are used for finding the best path out of the predefined paths from which the router can choose.
The costs need to be non-negative.
<h2>Note</h2>
By default this {@link OrthogonalPatternEdgeRouter} is used in conjunction with a
{@link OrthogonalSegmentDistributionStage}. Hence, edge overlaps are desired and thus should not cause any costs.
@throws {Exception} <b>(ArgumentError)</b> if the specified costs are negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouter%23edgeOverlapCost Online Documentation}*/
edgeOverlapCost: number;
static $class: Class;
private $_OrthogonalPatternEdgeRouter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalPatternEdgeRouter;
}
export interface OrthogonalSegmentDistributionStage extends LayoutStageBase {
}
/**
{@link OrthogonalSegmentDistributionStage} distributes overlapping edge segments of orthogonally routed edges.
The edge segments are assigned to the so-called <i>channels</i> within which they are distributed. Those channels are defined by the
surrounding graph elements.
The algorithm will not distribute the segments, whose endpoints connect to a node using a port that has a strong {@link PortConstraint}
or a fixed {@link PortCandidate}.
<b>Features</b>
It is possible to define a {@link OrthogonalSegmentDistributionStage#preferredDistance preferred distance} between distributed edge segments. This distance will only be applied if
there is enough space in the according channel. In the case where there is not enough space, the preferred distance may
be {@link OrthogonalSegmentDistributionStage#flexiblePreferredDistance reduced} for the edges. If the layout algorithm is not allowed to reduce the preferred distance, these edges won't be
distributed.
Edges can be distributed on {@link OrthogonalSegmentDistributionStage#gridRouting grid coordinates}. The grid can only be considered if there is enough space in the according
channel for at least as many grid lines as the number of segments. In the case where there is not enough space, the {@link OrthogonalSegmentDistributionStage#gridSpacing grid spacing}
may be {@link OrthogonalSegmentDistributionStage#flexibleGridSpacing reduced} in this channel. If the layout algorithm is not allowed to reduce the grid spacing, these edges won't be
distributed.
Method {@link OrthogonalSegmentDistributionStage#lockFirstAndLastSegment} can be used to guarantee that the first and last segment of an edge won't be distributed. So, ports can easily
be maintained without setting explicit strong {@link PortConstraint}s or fixed {@link PortCandidate}s.
<h2>Note</h2>
{@link OrthogonalSegmentDistributionStage} is not capable of moving nodes. Hence, edges are only distributed if
they keep the aforementioned settings or the layout algorithm is allowed to adjust those settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage Online Documentation}*/
export class OrthogonalSegmentDistributionStage {
/**
Creates a new instance of {@link OrthogonalSegmentDistributionStage} with default settings.
@param [options] The parameters to pass.
@param [options.affectedEdgesDpKey] The key to register a {@link IDataProvider} which determines the edges that shall be routed by this algorithm. This option sets the {@link OrthogonalSegmentDistributionStage#affectedEdgesDpKey} property on the created object.
@param [options.flexiblePreferredDistance] Whether or not the preferred distance between edges can be reduced if there is not enough space. This option sets the {@link OrthogonalSegmentDistributionStage#flexiblePreferredDistance} property on the created object.
@param [options.flexibleGridSpacing] Whether or not the grid spacing can be reduced if there is not enough space. This option sets the {@link OrthogonalSegmentDistributionStage#flexibleGridSpacing} property on the created object.
@param [options.gridRouting] Whether or not the orthogonal segments are distributed on grid coordinates. This option sets the {@link OrthogonalSegmentDistributionStage#gridRouting} property on the created object.
@param [options.gridSpacing] The spacing between two grid lines. This option sets the {@link OrthogonalSegmentDistributionStage#gridSpacing} property on the created object.
@param [options.lockFirstAndLastSegment] Whether or not the first and last segment of an edge will be distributed. This option sets the {@link OrthogonalSegmentDistributionStage#lockFirstAndLastSegment} property on the created object.
@param [options.preferredDistance] The preferred distance between each two segments. This option sets the {@link OrthogonalSegmentDistributionStage#preferredDistance} property on the created object.
@param [options.coreLayout] The core layout algorithm that is wrapped by this stage. This option sets the {@link LayoutStageBase#coreLayout} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage Online Documentation}*/
constructor(options?: {
affectedEdgesDpKey?: any | null;
flexiblePreferredDistance?: boolean;
flexibleGridSpacing?: boolean;
gridRouting?: boolean;
gridSpacing?: number;
lockFirstAndLastSegment?: boolean;
preferredDistance?: number;
coreLayout?: ILayoutAlgorithm | null;
});
/**
Distributes overlapping orthogonal segments after the core layout has been calculated.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Specifies the coordinates of the origin of the grid.
The grid coordinates will be multiples of the {@link OrthogonalSegmentDistributionStage#gridSpacing grid spacing} added to this origin.
<b>Warning:</b> This option will only have an effect if grid distribution is enabled.
@param offsetX the x-coordinate of the grid origin
@param offsetY the y-coordinate of the grid origin
@see OrthogonalSegmentDistributionStage#gridRouting
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23setGridOrigin(number%2Cnumber) Online Documentation}*/
setGridOrigin(offsetX: number, offsetY: number): void;
/**
Data provider key for determining which edges are distributed
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23DEFAULT_AFFECTED_EDGES_DP_KEY Online Documentation}*/
static DEFAULT_AFFECTED_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets the key to register a {@link IDataProvider} which determines the edges that shall be routed by
this algorithm.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link IDataProvider} key is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23affectedEdgesDpKey Online Documentation}*/
affectedEdgesDpKey: any | null;
/**
Gets or sets whether or not the preferred distance between edges can be reduced if there is not enough space.
The space between the nodes restricts the distribution of the orthogonal segments while considering the
{@link OrthogonalSegmentDistributionStage#preferredDistance preferred distance}. In case this option is enabled, the layout algorithm may reduce the preferred distance, otherwise
the algorithm will only distribute segments if there is enough space.
<h2>Note</h2>
Preferred distances will not be maintained for segments connecting to nodes (first and last segment of an edge). The
segments are distributed equidistant all along the node side, and thus they may lead to greater distances between
segments.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23flexiblePreferredDistance Online Documentation}*/
flexiblePreferredDistance: boolean;
/**
Gets or sets whether or not the grid spacing can be reduced if there is not enough space.
The space between the nodes limits the number of grid lines on which edges are placed. In case this option is enabled,
the layout algorithm will divide the given grid spacing by <code>2</code> as long as there are not enough grid lines on which the
segments can be distributed, otherwise the grid spacing remains unchanged.
<h2>Note</h2>
The grid spacing will only be adjusted for segments that cannot maintain the originally set grid width.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23flexibleGridSpacing Online Documentation}*/
flexibleGridSpacing: boolean;
/**
Gets or sets whether or not the orthogonal segments are distributed on grid coordinates.
@see OrthogonalSegmentDistributionStage#setGridOrigin
@see OrthogonalSegmentDistributionStage#gridSpacing
@see OrthogonalSegmentDistributionStage#flexibleGridSpacing
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23gridRouting Online Documentation}*/
gridRouting: boolean;
/**
Gets or sets the spacing between two grid lines.
The spacing should be at least <code>0</code>.
<b>Warning:</b> This option will only have an effect if grid distribution is enabled.
@throws {Exception} <b>(ArgumentError)</b> if the specified spacing is negative
@see OrthogonalSegmentDistributionStage#gridRouting
@see OrthogonalSegmentDistributionStage#gridRouting
@see OrthogonalSegmentDistributionStage#flexibleGridSpacing
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23gridSpacing Online Documentation}*/
gridSpacing: number;
/**
Gets the x-coordinate of the origin of the grid.
The grid coordinates will be multiples of the {@link OrthogonalSegmentDistributionStage#gridSpacing grid spacing} added to this origin.
<b>Warning:</b> This option will only have an effect if grid distribution is enabled.
@see OrthogonalSegmentDistributionStage#setGridOrigin
@see OrthogonalSegmentDistributionStage#gridRouting
@see OrthogonalSegmentDistributionStage#gridOriginY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23gridOriginX Online Documentation}*/
readonly gridOriginX: number;
/**
Gets the y-coordinate of the origin of the grid.
The grid coordinates will be multiples of the {@link OrthogonalSegmentDistributionStage#gridSpacing grid spacing} added to this origin.
<b>Warning:</b> This option will only have an effect if grid distribution is enabled.
@see OrthogonalSegmentDistributionStage#setGridOrigin
@see OrthogonalSegmentDistributionStage#gridRouting
@see OrthogonalSegmentDistributionStage#gridOriginX
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23gridOriginY Online Documentation}*/
readonly gridOriginY: number;
/**
Gets or sets whether or not the first and last segment of an edge will be distributed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23lockFirstAndLastSegment Online Documentation}*/
lockFirstAndLastSegment: boolean;
/**
Gets or sets the preferred distance between each two segments.
This value also applies to the distance between a segment and the border of the containing channel.
The preferred distance should be at least <code>0</code>.
<h2>Note</h2>
In the case where the preferred distance is
{@link OrthogonalSegmentDistributionStage#flexiblePreferredDistance adjustable}, the preferred distance might be reduced to the maximum possible distance closest to the given value. This
happens if the channel is not large enough to fit in all segments with the desired preferred distance.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see OrthogonalSegmentDistributionStage#flexiblePreferredDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalSegmentDistributionStage%23preferredDistance Online Documentation}*/
preferredDistance: number;
static $class: Class;
private $_OrthogonalSegmentDistributionStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalSegmentDistributionStage;
}
export interface SnapOuterPortsToNodeBorderStage extends LayoutStageBase {
}
/**
{@link SnapOuterPortsToNodeBorderStage} moves edge ports that are outside the corresponding area of the nodes to
their border.
Thus, it guarantees that ports are always in the interior of the nodes or on their borders.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapOuterPortsToNodeBorderStage Online Documentation}*/
export class SnapOuterPortsToNodeBorderStage {
/**
Creates a new instance of {@link SnapOuterPortsToNodeBorderStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapOuterPortsToNodeBorderStage Online Documentation}*/
constructor();
/**
Snaps all edge ports that lie outside the bounds of the nodes to their border.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/SnapOuterPortsToNodeBorderStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
static $class: Class;
private $_SnapOuterPortsToNodeBorderStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SnapOuterPortsToNodeBorderStage;
}
export interface DefaultOutEdgeComparer extends YObject, IComparer<any> {
}
/**
{@link DefaultOutEdgeComparer} is the default implementation for the out-edge comparator used in
{@link SeriesParallelLayout}.
It keeps the order of the edges in the graph for edges without {@link PortConstraint}s and considers the
{@link PortConstraint}s such that the order of the target nodes of the outgoing edges is similar to the port
location at the source node. Note that this won't work for all cases because the target nodes may not be completely
independent.
@see SeriesParallelLayout#defaultOutEdgeComparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultOutEdgeComparer Online Documentation}*/
export class DefaultOutEdgeComparer {
/**
Creates a {@link DefaultOutEdgeComparer} instance with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultOutEdgeComparer Online Documentation}*/
constructor();
/**
Compares the two given edges considering their {@link PortConstraint}s.
The order of edges is kept unless there are
{@link PortConstraint}s assigned. Then the edges will be sorted to avoid edge crossings.
@param o1 the first edge
@param o2 the second edge
@returns the order of the given two edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultOutEdgeComparer%23compare(Object%2CObject) Online Documentation}*/
compare(o1: any, o2: any): number;
static $class: Class;
private $_DefaultOutEdgeComparer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultOutEdgeComparer;
}
export interface DefaultSeriesParallelLayoutPortAssignment extends YObject, ISeriesParallelLayoutPortAssignment {
}
/**
{@link DefaultSeriesParallelLayoutPortAssignment} implements the default port assignment strategy.
Ports are either placed at the {@link SeriesParallelLayoutPortAssignmentMode#CENTER center} or get {@link SeriesParallelLayoutPortAssignmentMode#DISTRIBUTED distributed} at the sides of the node.
If {@link PortConstraint}s are specified, the strong {@link PortConstraint}s are handled separately and keep
their initial location relative to the node while weak
{@link PortConstraint}s just influence on which side of the node the port is placed.
If there are edge groups specified, the edges which belong to the same group get the same port locations.
@see PortConstraint
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSeriesParallelLayoutPortAssignment Online Documentation}*/
export class DefaultSeriesParallelLayoutPortAssignment {
/**
Creates a new {@link DefaultSeriesParallelLayoutPortAssignment} instance using the given {@link DefaultSeriesParallelLayoutPortAssignment#mode mode} and a given value for the {@link DefaultSeriesParallelLayoutPortAssignment#borderGapToPortGapRatio ratio}
of the gap between the border and the ports to the gap between the ports themselves.
If no port assignment mode is given, the port assignment mode that resets all ports to the center of their nodes will be
used instead {@link SeriesParallelLayoutPortAssignmentMode#CENTER}.
@param [mode=SeriesParallelLayoutPortAssignmentMode.CENTER] one of the predefined port assignment modes
@param [ratio=0.5] the ratio of the distance between outer ports and the border to the distance between any two ports
@throws {Exception} <b>(ArgumentError)</b> if the given mode is unknown or the given ratio is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSeriesParallelLayoutPortAssignment Online Documentation}*/
constructor(mode?: SeriesParallelLayoutPortAssignmentMode | SeriesParallelLayoutPortAssignmentModeStringValues, ratio?: number);
/**
Creates a new {@link DefaultSeriesParallelLayoutPortAssignment} instance using the given {@link DefaultSeriesParallelLayoutPortAssignment#mode mode} and a given value for the {@link DefaultSeriesParallelLayoutPortAssignment#borderGapToPortGapRatio ratio}
of the gap between the border and the ports to the gap between the ports themselves.
If no port assignment mode is given, the port assignment mode that resets all ports to the center of their nodes will be
used instead {@link SeriesParallelLayoutPortAssignmentMode#CENTER}.
@param options The parameters to pass.
@param [options.mode=SeriesParallelLayoutPortAssignmentMode.CENTER] one of the predefined port assignment modes
@param [options.ratio=0.5] the ratio of the distance between outer ports and the border to the distance between any two ports
@param [options.borderGapToPortGapRatio] The ratio of the <em>border gap</em> (distance between the outer ports and the node border) to the <em>port gap</em> (distance between adjacent ports). This option sets the {@link DefaultSeriesParallelLayoutPortAssignment#borderGapToPortGapRatio} property on the created object.
@param [options.forkStyle] The fork style which influences the port assignment. This option sets the {@link DefaultSeriesParallelLayoutPortAssignment#forkStyle} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the given mode is unknown or the given ratio is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSeriesParallelLayoutPortAssignment Online Documentation}*/
constructor(options: {
mode?: SeriesParallelLayoutPortAssignmentMode | SeriesParallelLayoutPortAssignmentModeStringValues;
ratio?: number;
borderGapToPortGapRatio?: number;
forkStyle?: ForkStyle | ForkStyleStringValues;
});
/**
Places the ports of edges connecting to the given node according to the {@link DefaultSeriesParallelLayoutPortAssignment#mode port assignment mode}.
@param graph the input graph
@param node the node whose adjacent edges' ports should be placed
@see DefaultSeriesParallelLayoutPortAssignment#mode
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSeriesParallelLayoutPortAssignment%23assignPorts(LayoutGraph%2CYNode) Online Documentation}*/
assignPorts(graph: LayoutGraph | null, node: YNode | null): void;
/**
Determines the distance between the outer ports and the border of the node.
This implementation calculates the gap according to the number of edges at one side of the node. It also includes the
{@link DefaultSeriesParallelLayoutPortAssignment#borderGapToPortGapRatio ratio between border gap and port gap}.
This method is called by {@link DefaultSeriesParallelLayoutPortAssignment#assignPorts} to calculate the location of the first
port. It may be overridden to implement a different distribution of ports on the node's side.
@param sideLength the width/height of the side
@param edgeCount the number of edges that connect to this side
@returns the absolute gap to be used on both sides of the ports
@see DefaultSeriesParallelLayoutPortAssignment#assignPorts
@see DefaultSeriesParallelLayoutPortAssignment#borderGapToPortGapRatio
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSeriesParallelLayoutPortAssignment%23getPortBorderGap(number%2Cnumber) Online Documentation}*/
getPortBorderGap(sideLength: number, edgeCount: number): number;
/**
Determines the distance between two adjacent ports.
This implementation calculates the gap according to the number of edges at one side of the node. The edges are
distributed keeping the given gap to the border of the node.
This method is called by {@link DefaultSeriesParallelLayoutPortAssignment#assignPorts} to calculate the distance between two
ports. It may be overridden to implement a different distribution of ports on the node's side.
@param sideLength the width/height of the side
@param edgeCount the number of edges/ports that connect to this side
@param portBorderGap the port border gap (i.e. calculated by {@link DefaultSeriesParallelLayoutPortAssignment#getPortBorderGap})
@returns the absolute distance to be used between two adjacent ports
@see DefaultSeriesParallelLayoutPortAssignment#assignPorts
@see DefaultSeriesParallelLayoutPortAssignment#getPortBorderGap
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSeriesParallelLayoutPortAssignment%23getPortDistanceDelta(number%2Cnumber%2Cnumber) Online Documentation}*/
getPortDistanceDelta(sideLength: number, edgeCount: number, portBorderGap: number): number;
/**
Gets or sets the mode that describes how ports are placed.
@throws {Exception} <b>(ArgumentError)</b> if an unknown mode is specified
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSeriesParallelLayoutPortAssignment%23mode Online Documentation}*/
mode: SeriesParallelLayoutPortAssignmentMode;
/**
Gets or sets the ratio of the <em>border gap</em> (distance between the outer ports and the node border) to the <em>port gap</em> (distance between adjacent
ports).
The ratio must have a positive value. If the ratio is
<code>0</code>, the ports will be distributed along the whole side of the node without a gap between the last port and the corner.
<b>Warning:</b> This setting only affects the {@link SeriesParallelLayoutPortAssignmentMode#DISTRIBUTED distributed} port assignment.
@throws {Exception} <b>(ArgumentError)</b> if the specified ratio is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSeriesParallelLayoutPortAssignment%23borderGapToPortGapRatio Online Documentation}*/
borderGapToPortGapRatio: number;
/**
Gets or sets the fork style which influences the port assignment.
If the fork style is set to
{@link ForkStyle#OUTSIDE_NODE}, edges without port constraints will be assigned in flow direction.
If the fork style is set to
{@link ForkStyle#AT_NODE}, the ports of edges in flow direction also get distributed to the sides unless
they are without bends. Due to large minimum first/last segment lengths edges may overlap.
@throws {Exception} <b>(ArgumentError)</b> if the specified fork style is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultSeriesParallelLayoutPortAssignment%23forkStyle Online Documentation}*/
forkStyle: ForkStyle;
static $class: Class;
private $_DefaultPortAssignment: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultSeriesParallelLayoutPortAssignment;
}
export interface SeriesParallelLayoutEdgeLayoutDescriptor extends YObject {
}
/**
An {@link SeriesParallelLayoutEdgeLayoutDescriptor} provides routing details for an edge that are considered by
{@link SeriesParallelLayout}.
Such an {@link SeriesParallelLayoutEdgeLayoutDescriptor} can either be specified for a single edge or shared by several edges.
It is assigned by registering a {@link IDataProvider} with key
{@link SeriesParallelLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}. If no {@link IDataProvider} is
registered with this key, {@link SeriesParallelLayout#defaultEdgeLayoutDescriptor a default descriptor} is used.
@see SeriesParallelLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see SeriesParallelLayout#defaultEdgeLayoutDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutEdgeLayoutDescriptor Online Documentation}*/
export class SeriesParallelLayoutEdgeLayoutDescriptor {
/**
Creates a new instance of {@link SeriesParallelLayoutEdgeLayoutDescriptor} with default settings.
@param [options] The parameters to pass.
@param [options.minimumLength] The minimum length for the associated edge. This option sets the {@link SeriesParallelLayoutEdgeLayoutDescriptor#minimumLength} property on the created object.
@param [options.minimumFirstSegmentLength] The minimum length for the first segment of the associated edge. This option sets the {@link SeriesParallelLayoutEdgeLayoutDescriptor#minimumFirstSegmentLength} property on the created object.
@param [options.minimumLastSegmentLength] The minimum length for the last segment of the associated edge. This option sets the {@link SeriesParallelLayoutEdgeLayoutDescriptor#minimumLastSegmentLength} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutEdgeLayoutDescriptor Online Documentation}*/
constructor(options?: {
minimumLength?: number;
minimumFirstSegmentLength?: number;
minimumLastSegmentLength?: number;
});
/**
Creates a copy of this {@link SeriesParallelLayoutEdgeLayoutDescriptor} instance.
@returns a copy of this {@link SeriesParallelLayoutEdgeLayoutDescriptor} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutEdgeLayoutDescriptor%23createCopy() Online Documentation}*/
createCopy(): SeriesParallelLayoutEdgeLayoutDescriptor | null;
/**
Gets or sets the minimum length for the associated edge.
The minimum length needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> for negative edge lengths
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutEdgeLayoutDescriptor%23minimumLength Online Documentation}*/
minimumLength: number;
/**
Gets or sets the minimum length for the first segment of the associated edge.
The minimum first segment length needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> for negative minimum segment lengths
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutEdgeLayoutDescriptor%23minimumFirstSegmentLength Online Documentation}*/
minimumFirstSegmentLength: number;
/**
Gets or sets the minimum length for the last segment of the associated edge.
The minimum last segment length needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> for negative minimum segment lengths
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutEdgeLayoutDescriptor%23minimumLastSegmentLength Online Documentation}*/
minimumLastSegmentLength: number;
static $class: Class;
private $_EdgeLayoutDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SeriesParallelLayoutEdgeLayoutDescriptor;
}
/**
@see DefaultSeriesParallelLayoutPortAssignment
@see DefaultSeriesParallelLayoutPortAssignment#mode
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutPortAssignmentMode Online Documentation}*/
export enum SeriesParallelLayoutPortAssignmentMode {
/**
A port assignment specifier which defines that all ports are reset to the center of their nodes.
<h2>Note</h2>
Ports that are restricted by strong
{@link PortConstraint}s are excluded. They will keep their current locations on the node.
@see PortConstraint
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutPortAssignmentMode%23CENTER Online Documentation}*/
CENTER,
/**
A port assignment specifier which defines that the ports of incoming and outgoing edges are distributed evenly at the
side of their nodes.
The source ports of edges without {@link PortConstraint}s will be distributed at the sides of their source and
target nodes. Edges with weak {@link PortConstraint}s are distributed at the side of the nodes which is specified
in the {@link PortConstraint}.
Grouped edges will get the same port location.
<h2>Note</h2>
Ports that are restricted by strong
{@link PortConstraint}s are excluded. They will keep their current locations on the node.
@see PortConstraint
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutPortAssignmentMode%23DISTRIBUTED Online Documentation}*/
DISTRIBUTED
}
/** Automatic conversion to {@link SeriesParallelLayoutPortAssignmentMode}. */
type SeriesParallelLayoutPortAssignmentModeStringValues = "center" | "distributed";
/**
Fork style specifier that defines that the edges will fork on a node.
@see DefaultSeriesParallelLayoutPortAssignment#forkStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ForkStyle Online Documentation}*/
export enum ForkStyle {
/**
Fork style specifier that defines that the edges will fork outside a node.
Edges leave/enter the nodes south/north and bend between their source and target.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ForkStyle%23OUTSIDE_NODE Online Documentation}*/
OUTSIDE_NODE,
/**
Fork style specifier that defines that the edges will fork on a node.
Edges leave/enter the nodes south/north if they are straight (no bends) or east and west of the node.
<h2>Note</h2>
If this fork style is set and the {@link SeriesParallelLayout#routingStyle routing style} is set to
{@link SeriesParallelLayoutRoutingStyle#POLYLINE}, the integrated edge labeling may place labels far away from the associated
edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ForkStyle%23AT_NODE Online Documentation}*/
AT_NODE
}
/** Automatic conversion to {@link ForkStyle}. */
type ForkStyleStringValues = "outside-node" | "at-node";
/**
Style constant describing a polyline edge style.
@see SeriesParallelLayout#routingStyle
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutRoutingStyle Online Documentation}*/
export enum SeriesParallelLayoutRoutingStyle {
/**
Style constant describing an orthogonal edge style.
If this style is used, edges will contain only horizontal and vertical segments.
@see SeriesParallelLayout#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutRoutingStyle%23ORTHOGONAL Online Documentation}*/
ORTHOGONAL,
/**
Style constant describing an octilinear edge style.
If this style is used, edges will contain horizontal, vertical and <code>45</code>-degree sloped segments.
@see SeriesParallelLayout#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutRoutingStyle%23OCTILINEAR Online Documentation}*/
OCTILINEAR,
/**
Style constant describing a polyline edge style.
If this style is used, edges will contain vertical and other sloped segments.
<h2>Note</h2>
If this routing style is set and the {@link DefaultSeriesParallelLayoutPortAssignment#forkStyle fork style} is set to
{@link ForkStyle#AT_NODE}, the integrated edge labeling may place labels far away from the associated
edges.
@see SeriesParallelLayout#routingStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutRoutingStyle%23POLYLINE Online Documentation}*/
POLYLINE
}
/** Automatic conversion to {@link SeriesParallelLayoutRoutingStyle}. */
type SeriesParallelLayoutRoutingStyleStringValues = "orthogonal" | "octilinear" | "polyline";
export interface ISeriesParallelLayoutPortAssignment {
}
/**
A {@link ISeriesParallelLayoutPortAssignment} is used by {@link SeriesParallelLayout} for determining the
locations of the edges' ports at their incident nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISeriesParallelLayoutPortAssignment Online Documentation}*/
export abstract class ISeriesParallelLayoutPortAssignment {
/**
Assigns both the incoming edges' target ports as well as all source ports for the outgoing edges.
Outgoing edges are sorted according to the specified out-edge comparator from left to right and incoming edges are in
the order of the subgraphs they are coming from.
This method is called by {@link SeriesParallelLayout} before the actual layout of the graph takes place.
@param graph the graph instance the node is part of
@param node the node whose incident edges' ports are assigned
@see SeriesParallelLayout#OUT_EDGE_COMPARER_DP_KEY
@see SeriesParallelLayout#defaultOutEdgeComparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/ISeriesParallelLayoutPortAssignment%23assignPorts(LayoutGraph%2CYNode) Online Documentation}*/
abstract assignPorts(graph: LayoutGraph | null, node: YNode | null): void;
static $class: Class;
private $_IPortAssignment: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ISeriesParallelLayoutPortAssignment;
}
export interface SeriesParallelLayout extends MultiStageLayout {
}
/**
This layout algorithm arranges series-parallel graphs.
<b>Layout Style</b>
Series-parallel graphs are directed graphs with a single source (node without incoming edges) and a single sink (node
without outgoing edges). The layout algorithm highlights the main layout direction (from source to sink). It also
emphasizes the paths through the graph because edges are routed with few bends.
{@link SeriesParallelLayout} is suitable for the visualization of circuits, call trees or flowcharts.
<b>Concept</b>
Series-parallel graphs are directed graphs with a single source (node without incoming edges) and a single sink (node
without outgoing edges) that are built using only the following two rules:
<ol>
<li><b>Series composition:</b> The source of a subgraph is merged with the sink of a second subgraph.
<li><b>Parallel composition:</b> The sources and sinks of two subgraphs are merged.
</ol>
From the recursive structure of series-parallel graphs, the layout algorithm retrieves a decomposition tree where each
node represents one of the decomposition types. Then, this tree is traversed recursively from bottom to top, aligning
subgraphs above (series) or next to (parallel) each other until the whole graph is arranged. The edges are routed when
both end nodes are placed. Different {@link SeriesParallelLayout#routingStyle routing styles} can be used.
To avoid moving all nodes several times and to be aware of the area that the subtrees occupy, the layout algorithm keeps
track of the shape of the subtrees. These shapes are moved and merged during the layout calculation. The layout
algorithm also stores the connections between nodes and nodes that haven't already been placed in these shapes.
<b>Features</b>
{@link SeriesParallelLayout} can take strong
{@link PortConstraint}s into account. It will connect the edges to the specified locations, the directions,
however, will be ignored.
Grouping of nodes can also be handled by this layout algorithm. It is important that a group node contains a whole
series-parallel subgraph. Otherwise, the group nodes may overlap with each other or with other nodes. Edges which are
connected to non-empty group nodes are not allowed. Furthermore, the user may specify minimum size constraints for each
group node using {@link IDataProvider} key {@link GroupingKeys#MINIMUM_NODE_SIZE_DP_KEY}.
The layout algorithm can be configured to reserve space for node labels and to place the edge labels along the edge such
that the labels won't overlap with other graph elements. Edge labels are placed according to the information stored in a
{@link PreferredPlacementDescriptor} instance. However, labels that should be centered between source and target
are placed close to the target node, unless this edge connects to the local source and sink of a subgraph.
Parallel subgraphs can be aligned in different ways. Depending on the node sizes, a different alignment can increase the
compactness of the layout.
The way in which edges are distributed around their incident nodes is computed by an instance of
{@link ISeriesParallelLayoutPortAssignment}. The {@link DefaultSeriesParallelLayoutPortAssignment default assignment} is able to consider
{@link PortConstraint}s and edge groups.
The {@link SeriesParallelLayout#fromSketchMode From Sketch mode} allows to take the initial locations of the nodes into account. However, the layout algorithm won't
insert crossings because it maintains the order of children of each node.
{@link SeriesParallelLayout} supports custom sorting of the outgoing edges of a node. A {@link IComparer} can be assigned
individually for the nodes using a {@link IDataProvider} registered with key
{@link SeriesParallelLayout#OUT_EDGE_COMPARER_DP_KEY}. For all nodes for which the {@link IDataProvider}
returns <code>null</code>, the layout algorithm falls back to the {@link SeriesParallelLayout#defaultOutEdgeComparer default comparator}.
By default, this layout algorithm can only handle graphs with a series-parallel structure. To apply it to a general
graph, {@link SeriesParallelLayout#generalGraphHandling general graph handling} needs to be activated. Then, the layout algorithm will temporarily add and/or remove some
edges from the input graph until a series-parallel graph is obtained. The edges that were removed will be routed
separately afterwards.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout Online Documentation}*/
export class SeriesParallelLayout {
/**
Creates a new {@link SeriesParallelLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.considerNodeLabels] Whether or not the layout algorithm reserves space for node labels to avoid overlaps. This option sets the {@link SeriesParallelLayout#considerNodeLabels} property on the created object.
@param [options.integratedEdgeLabeling] Whether or not the layout algorithm will place edge labels and reserve space for them. This option sets the {@link SeriesParallelLayout#integratedEdgeLabeling} property on the created object.
@param [options.verticalAlignment] The vertical alignment of parallel subgraphs. This option sets the {@link SeriesParallelLayout#verticalAlignment} property on the created object.
@param [options.generalGraphHandling] Whether or not the layout algorithm can handle general graphs. This option sets the {@link SeriesParallelLayout#generalGraphHandling} property on the created object.
@param [options.nonSeriesParallelEdgeRouter] The edge routing algorithm used for the edges of a general graph that are not part of the series-parallel subgraph whose layout is calculated. This option sets the {@link SeriesParallelLayout#nonSeriesParallelEdgeRouter} property on the created object.
@param [options.nonSeriesParallelEdgesDpKey] The key to register a {@link IDataProvider} that is used for marking non-series-parallel edges. This option sets the {@link SeriesParallelLayout#nonSeriesParallelEdgesDpKey} property on the created object.
@param [options.nonSeriesParallelEdgeLabelingAlgorithm] The labeling algorithm that is applied to all edge labels that belong to non-series-parallel edges. This option sets the {@link SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm} property on the created object.
@param [options.nonSeriesParallelEdgeLabelSelectionKey] The key to register a {@link IDataProvider} that is used by the {@link SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm non-series-parallel edge labeling algorithm} to determine which edge labels it should place. This option sets the {@link SeriesParallelLayout#nonSeriesParallelEdgeLabelSelectionKey} property on the created object.
@param [options.defaultPortAssignment] The default {@link ISeriesParallelLayoutPortAssignment} used for those nodes that do not have their own specific instance. This option sets the {@link SeriesParallelLayout#defaultPortAssignment} property on the created object.
@param [options.defaultOutEdgeComparer] The default {@link IComparer} used for sorting the outgoing edges incident to nodes that do not have a specific {@link IComparer}. This option sets the {@link SeriesParallelLayout#defaultOutEdgeComparer} property on the created object.
@param [options.routingStyle] The currently used routing style for edges. This option sets the {@link SeriesParallelLayout#routingStyle} property on the created object.
@param [options.minimumPolylineSegmentLength] The minimum vertical distance of the edge segments that are not orthogonal. This option sets the {@link SeriesParallelLayout#minimumPolylineSegmentLength} property on the created object.
@param [options.minimumSlope] The minimum slope which a non-orthogonal edge segment should have. This option sets the {@link SeriesParallelLayout#minimumSlope} property on the created object.
@param [options.preferredOctilinearSegmentLength] The preferred length for non-orthogonal segments in octilinear edge routes. This option sets the {@link SeriesParallelLayout#preferredOctilinearSegmentLength} property on the created object.
@param [options.defaultEdgeLayoutDescriptor] The {@link SeriesParallelLayoutEdgeLayoutDescriptor} instance used for all those edges that do not have a specific layout descriptor assigned. This option sets the {@link SeriesParallelLayout#defaultEdgeLayoutDescriptor} property on the created object.
@param [options.minimumNodeToNodeDistance] The minimum distance between nodes. This option sets the {@link SeriesParallelLayout#minimumNodeToNodeDistance} property on the created object.
@param [options.minimumNodeToEdgeDistance] The minimum distance between nodes and edges. This option sets the {@link SeriesParallelLayout#minimumNodeToEdgeDistance} property on the created object.
@param [options.minimumEdgeToEdgeDistance] The minimum distance between edges. This option sets the {@link SeriesParallelLayout#minimumEdgeToEdgeDistance} property on the created object.
@param [options.fromSketchMode] Whether or not to take the coordinates of the input diagram into account when arranging the nodes. This option sets the {@link SeriesParallelLayout#fromSketchMode} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link SeriesParallelLayout#componentLayoutEnabled} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link SeriesParallelLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link MultiStageLayout#orientationLayoutEnabled} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link MultiStageLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout Online Documentation}*/
constructor(options?: {
considerNodeLabels?: boolean;
integratedEdgeLabeling?: boolean;
verticalAlignment?: number;
generalGraphHandling?: boolean;
nonSeriesParallelEdgeRouter?: ILayoutAlgorithm | null;
nonSeriesParallelEdgesDpKey?: any | null;
nonSeriesParallelEdgeLabelingAlgorithm?: ILayoutAlgorithm | null;
nonSeriesParallelEdgeLabelSelectionKey?: any | null;
defaultPortAssignment?: ISeriesParallelLayoutPortAssignment | null;
defaultOutEdgeComparer?: IComparer<any> | null;
routingStyle?: SeriesParallelLayoutRoutingStyle | SeriesParallelLayoutRoutingStyleStringValues;
minimumPolylineSegmentLength?: number;
minimumSlope?: number;
preferredOctilinearSegmentLength?: number;
defaultEdgeLayoutDescriptor?: SeriesParallelLayoutEdgeLayoutDescriptor | null;
minimumNodeToNodeDistance?: number;
minimumNodeToEdgeDistance?: number;
minimumEdgeToEdgeDistance?: number;
fromSketchMode?: boolean;
componentLayoutEnabled?: boolean;
selfLoopRouterEnabled?: boolean;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
orientationLayoutEnabled?: boolean;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Calculates a series-parallel layout for the given graph.
@param graph the input graph
@throws {Exception} <b>(InvalidGraphStructureError)</b> if the graph is not series-parallel
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Determines whether or not the given graph has a series-parallel structure.
The current implementation detects the series-parallel graph structure in linear time.
@param graph the input graph
@returns <code>true</code> if the given graph is series-parallel, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23isSeriesParallelGraph(Graph) Online Documentation}*/
static isSeriesParallelGraph(graph: Graph | null): boolean;
/**
Data provider key for storing individual settings for edges
If no edge layout descriptor is mapped for an edge, a default edge layout descriptor will be obtained by method {@link SeriesParallelLayout#defaultEdgeLayoutDescriptor}.
@see SeriesParallelLayout#defaultEdgeLayoutDescriptor
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23EDGE_LAYOUT_DESCRIPTOR_DP_KEY Online Documentation}*/
static EDGE_LAYOUT_DESCRIPTOR_DP_KEY: EdgeDpKey<SeriesParallelLayoutEdgeLayoutDescriptor> | null;
/**
Data provider key for assigning different orderings for outgoing edges of the nodes
If the {@link IComparer} associated with a node is <code>null</code>, the outgoing edges maintain their initial order.
<b>Warning:</b> The order cannot be maintained in all cases because the target nodes may not be completely independent.
<h2>Note</h2>
For general graphs, the {@link IComparer}s are only used for the series-parallel part of the graph. Non-series-parallel edges are not
included and will be routed by the {@link SeriesParallelLayout#nonSeriesParallelEdgeRouter edge router for non-series-parallel edges}.
@see SeriesParallelLayout#defaultOutEdgeComparer
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23OUT_EDGE_COMPARER_DP_KEY Online Documentation}*/
static OUT_EDGE_COMPARER_DP_KEY: NodeDpKey<IComparer<any>> | null;
/**
Data provider key for providing an individual port distribution at nodes
If there is no {@link ISeriesParallelLayoutPortAssignment} specified for a node, the layout algorithm uses the
{@link SeriesParallelLayout#defaultPortAssignment default assignment}.
@see SeriesParallelLayout#defaultPortAssignment
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23PORT_ASSIGNMENT_DP_KEY Online Documentation}*/
static PORT_ASSIGNMENT_DP_KEY: NodeDpKey<ISeriesParallelLayoutPortAssignment> | null;
/**
Data provider key for marking edge labels of non-series-parallel edges
<b>Warning:</b> This key only serves as default value for property {@link SeriesParallelLayout#nonSeriesParallelEdgeLabelSelectionKey}. During the layout, a {@link IDataProvider} marking the
non-series-parallel edges will automatically be registered with the graph. Thus, data provided by the user registered
with this specific key is ignored.
@see SeriesParallelLayout#nonSeriesParallelEdgeLabelSelectionKey
@see SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23NON_SERIES_PARALLEL_EDGE_LABELS_DP_KEY Online Documentation}*/
static NON_SERIES_PARALLEL_EDGE_LABELS_DP_KEY: IEdgeLabelLayoutDpKey<boolean> | null;
/**
Gets or sets whether or not the layout algorithm reserves space for node labels to avoid overlaps.
<b>Warning:</b> When enabling this property, any previously specified {@link MultiStageLayout#labeling labeling algorithm} will be overwritten and when disabling it, any
currently specified labeling algorithm will be disabled via property
{@link MultiStageLayout#labelingEnabled}. Therefore, it is recommended to only use this convenience property instead of manually changing the mentioned other
properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets whether or not the layout algorithm will place edge labels and reserve space for them.
<b>Warning:</b> When enabling this property, any previously specified {@link MultiStageLayout#labeling labeling algorithm} will be overwritten and when disabling it, any
currently specified labeling algorithm will be disabled via property
{@link MultiStageLayout#labelingEnabled}. Therefore, it is recommended to only use this convenience property instead of manually changing the mentioned other
properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23integratedEdgeLabeling Online Documentation}*/
integratedEdgeLabeling: boolean;
/**
Gets or sets the vertical alignment of parallel subgraphs.
The alignment is defined by a ratio:
<ul>
<li>Ratio <code>0</code> means that nodes are top-aligned
<li>Ratio <code>0.5</code> means that nodes are center-aligned
<li>Ratio <code>1</code> means that nodes are bottom-aligned
</ul>
<b>Warning:</b> Values outside interval <code>[0,1]</code> will be matched with the nearest interval end.
<h2>Note</h2>
In grouped graphs, the vertical alignment only applies to parallel subgraphs that belong to the same group.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23verticalAlignment Online Documentation}*/
verticalAlignment: number;
/**
Gets or sets whether or not the layout algorithm can handle general graphs.
General graphs are required to be transformed before being laid out (by adding or removing some edges) such that they
satisfy the criteria of a series-parallel graph. After the layout, the graph will be restored and the initial edges
which weren't included in the series-parallel graph are routed by a separate {@link SeriesParallelLayout#nonSeriesParallelEdgeRouter edge routing algorithm}. Furthermore, the
labels of these edges are placed by a customizable {@link SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm edge labeling algorithm}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23generalGraphHandling Online Documentation}*/
generalGraphHandling: boolean;
/**
Gets or sets the edge routing algorithm used for the edges of a general graph that are not part of the series-parallel
subgraph whose layout is calculated.
<h2>Note</h2>
It is required that a suitable {@link SeriesParallelLayout#nonSeriesParallelEdgesDpKey selection key} is set.
@throws {Exception} <b>(ArgumentError)</b> if <code>null</code> is specified
@see SeriesParallelLayout#generalGraphHandling
@see SeriesParallelLayout#nonSeriesParallelEdgesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23nonSeriesParallelEdgeRouter Online Documentation}*/
nonSeriesParallelEdgeRouter: ILayoutAlgorithm | null;
/**
Gets or sets the key to register a {@link IDataProvider} that is used for marking non-series-parallel edges.
This key is used for determining the edges that are not part of the series-parallel structure in a general graph, such
that the specified non-series-parallel edge router only routes marked edges.
<h2>Note</h2>
The layout algorithm automatically marks these edges and also registers the {@link IDataProvider} using the
specified key.
@throws {Exception} <b>(ArgumentError)</b> if the given key is set to <code>null</code>
@see SeriesParallelLayout#generalGraphHandling
@see SeriesParallelLayout#nonSeriesParallelEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23nonSeriesParallelEdgesDpKey Online Documentation}*/
nonSeriesParallelEdgesDpKey: any | null;
/**
Gets or sets the labeling algorithm that is applied to all edge labels that belong to non-series-parallel edges.
<b>Warning:</b> When specifying a custom labeling algorithm, make sure that it uses the according {@link SeriesParallelLayout#nonSeriesParallelEdgeLabelSelectionKey edge label selection key} to determine
which edge labels it should place. Otherwise, the labeling algorithm might place all labels, including those that belong
to actual series-parallel edges.
@see SeriesParallelLayout#generalGraphHandling
@see SeriesParallelLayout#nonSeriesParallelEdgeLabelSelectionKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23nonSeriesParallelEdgeLabelingAlgorithm Online Documentation}*/
nonSeriesParallelEdgeLabelingAlgorithm: ILayoutAlgorithm | null;
/**
Gets or sets the key to register a {@link IDataProvider} that is used by the {@link SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm non-series-parallel edge labeling algorithm}
to determine which edge labels it should place.
During the layout, a {@link IDataProvider} with this key will be registered with the graph. It will mark all
{@link IEdgeLabelLayout}s that belong to non-series-parallel edges. A specified custom non-series-parallel edge
labeling algorithm needs to obey this selection. If using {@link GenericLabeling} as labeling algorithm, set
this key as value of property {@link LabelingBase#affectedLabelsDpKey}.
The labeling algorithm set as default is already configured such that it uses the correct selection key.
<b>Warning:</b> When specifying a custom {@link SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm non-series-parallel edge labeling algorithm} but no edge label selection key, then the labeling
algorithm might place all labels, including those that belong to actual series-parallel edges.
@see SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23nonSeriesParallelEdgeLabelSelectionKey Online Documentation}*/
nonSeriesParallelEdgeLabelSelectionKey: any | null;
/**
Gets or sets the default {@link ISeriesParallelLayoutPortAssignment} used for those nodes that do not have their own specific
instance.
A {@link ISeriesParallelLayoutPortAssignment} instance is defined using a {@link IDataProvider} registered with the
graph with key {@link SeriesParallelLayout#PORT_ASSIGNMENT_DP_KEY}.
@throws {Exception} <b>(ArgumentError)</b> if <code>null</code> is specified
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23defaultPortAssignment Online Documentation}*/
defaultPortAssignment: ISeriesParallelLayoutPortAssignment | null;
/**
Gets or sets the default {@link IComparer} used for sorting the outgoing edges incident to nodes that do not have a specific {@link IComparer}.
Such a comparator is defined using a {@link IDataProvider} registered with the graph with key
{@link SeriesParallelLayout#OUT_EDGE_COMPARER_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23defaultOutEdgeComparer Online Documentation}*/
defaultOutEdgeComparer: IComparer<any> | null;
/**
Gets or sets the currently used routing style for edges.
<b>Warning:</b> The routing style has no effect on non-series-parallel edges, when {@link SeriesParallelLayout#generalGraphHandling} is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23routingStyle Online Documentation}*/
routingStyle: SeriesParallelLayoutRoutingStyle;
/**
Gets or sets the minimum vertical distance of the edge segments that are not orthogonal.
The distance needs to be non-negative.
<b>Warning:</b> This distance will only be considered when the {@link SeriesParallelLayout#routingStyle routing style} is {@link SeriesParallelLayoutRoutingStyle#POLYLINE}.
@throws {Exception} <b>(ArgumentError)</b> if the specified length is smaller than <code>0</code>
@see SeriesParallelLayout#minimumSlope
@see SeriesParallelLayout#routingStyle
@see SeriesParallelLayoutRoutingStyle#POLYLINE
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23minimumPolylineSegmentLength Online Documentation}*/
minimumPolylineSegmentLength: number;
/**
Gets or sets the minimum slope which a non-orthogonal edge segment should have.
A higher minimum slope prevents those segments in very wide graphs from becoming nearly horizontal. The slope needs to
have a non-negative value.
<b>Warning:</b> This distance will only be considered when the {@link SeriesParallelLayout#routingStyle routing style} is {@link SeriesParallelLayoutRoutingStyle#POLYLINE}.
@throws {Exception} <b>(ArgumentError)</b> if the specified slope is smaller than <code>0</code>
@see SeriesParallelLayout#minimumPolylineSegmentLength
@see SeriesParallelLayout#routingStyle
@see SeriesParallelLayoutRoutingStyle#POLYLINE
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23minimumSlope Online Documentation}*/
minimumSlope: number;
/**
Gets or sets the preferred length for non-orthogonal segments in octilinear edge routes.
If there is not enough space, those segments may be shorter than the specified length. The length needs to be
non-negative.
<b>Warning:</b> This segment length will only be considered when the {@link SeriesParallelLayout#routingStyle routing style} is {@link SeriesParallelLayoutRoutingStyle#OCTILINEAR}.
@throws {Exception} <b>(ArgumentError)</b> if the specified length is smaller than <code>0</code>
@see SeriesParallelLayout#routingStyle
@see SeriesParallelLayoutRoutingStyle#OCTILINEAR
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23preferredOctilinearSegmentLength Online Documentation}*/
preferredOctilinearSegmentLength: number;
/**
Gets or sets the {@link SeriesParallelLayoutEdgeLayoutDescriptor} instance used for all those edges that do not have a
specific layout descriptor assigned.
@throws {Exception} <b>(ArgumentError)</b> if the specified {@link SeriesParallelLayoutEdgeLayoutDescriptor} is <code>null</code>
@see SeriesParallelLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23defaultEdgeLayoutDescriptor Online Documentation}*/
defaultEdgeLayoutDescriptor: SeriesParallelLayoutEdgeLayoutDescriptor | null;
/**
Gets or sets the minimum distance between nodes.
This distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is smaller than <code>0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23minimumNodeToNodeDistance Online Documentation}*/
minimumNodeToNodeDistance: number;
/**
Gets or sets the minimum distance between nodes and edges.
This distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is smaller than <code>0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23minimumNodeToEdgeDistance Online Documentation}*/
minimumNodeToEdgeDistance: number;
/**
Gets or sets the minimum distance between edges.
This distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is smaller than <code>0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23minimumEdgeToEdgeDistance Online Documentation}*/
minimumEdgeToEdgeDistance: number;
/**
Gets or sets whether or not to take the coordinates of the input diagram into account when arranging the nodes.
The order of edges incident to the same source node will remain the same as in the original layout.
<b>Warning:</b> Previously specified {@link SeriesParallelLayout#OUT_EDGE_COMPARER_DP_KEY comparators} will be ignored if From Sketch mode is enabled.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23fromSketchMode Online Documentation}*/
fromSketchMode: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated.
<b>Warning:</b> {@link SeriesParallelLayout} can only handle single components. Disabling {@link ComponentLayout}
will lead to errors during execution.
@see MultiStageLayout#componentLayoutEnabled
@see MultiStageLayout#componentLayout
@see ComponentLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23componentLayoutEnabled Online Documentation}*/
componentLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing self-loops is activated.
<b>Warning:</b> {@link SeriesParallelLayout} cannot handle self-loops. Disabling {@link SelfLoopRouter} will lead
to errors during execution.
@see MultiStageLayout#selfLoopRouterEnabled
@see MultiStageLayout#selfLoopRouter
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayout%23selfLoopRouterEnabled Online Documentation}*/
selfLoopRouterEnabled: boolean;
static $class: Class;
private $_SeriesParallelLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SeriesParallelLayout;
}
export interface AspectRatioNodePlacer extends NodePlacerBase, IFromSketchNodePlacer {
}
/**
This {@link ITreeLayoutNodePlacer} places the subtrees such that the overall aspect ratio of the arranged subtree will be
close to a preferred aspect ratio.
<b>Layout Style</b>
The aspect ratio describes the relation between width and height of the subtree. To achieve the desired aspect ratio,
this {@link ITreeLayoutNodePlacer} divides the child
{@link SubtreeShape}s into as much rows as needed. The edges are either routed along the rows or along the columns.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer Online Documentation}*/
export class AspectRatioNodePlacer {
/**
Creates a new instance of {@link AspectRatioNodePlacer} with default settings.
@param [options] The parameters to pass.
@param [options.horizontal] Whether child nodes are distributed in horizontal rows or vertical columns. This option sets the {@link AspectRatioNodePlacer#horizontal} property on the created object.
@param [options.verticalDistance] The vertical distance between the {@link SubtreeShape}s. This option sets the {@link AspectRatioNodePlacer#verticalDistance} property on the created object.
@param [options.horizontalDistance] The horizontal distance between the {@link SubtreeShape}s. This option sets the {@link AspectRatioNodePlacer#horizontalDistance} property on the created object.
@param [options.aspectRatio] The preferred aspect ratio (width/height) for the subtrees. This option sets the {@link AspectRatioNodePlacer#aspectRatio} property on the created object.
@param [options.fillStyle] How rows/columns are aligned with each other. This option sets the {@link AspectRatioNodePlacer#fillStyle} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer%23AspectRatioNodePlacer() Online Documentation}*/
constructor(options?: {
horizontal?: boolean;
verticalDistance?: number;
horizontalDistance?: number;
aspectRatio?: number;
fillStyle?: FillStyle | FillStyleStringValues;
});
/**
Creates an {@link IComparer edge comparator} which takes the initial coordinates of the nodes into account.
This {@link IComparer} defines the order of the outgoing edges of a node before the actual placement of the node's subtree is calculated.
It also considers whether or not the rows are placed {@link AspectRatioNodePlacer#horizontal horiontally}.
@returns the {@link IComparer} for the edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer%23createFromSketchComparer() Online Documentation}*/
createFromSketchComparer(): IComparer<any> | null;
/**
Determines a connector direction for the given child node depending on whether or not the child nodes are arranged
{@link AspectRatioNodePlacer#horizontal horizontally}.
If the child nodes are distributed horizontally, the edge should connect {@link ParentConnectorDirection#NORTH north}, otherwise it should connect {@link ParentConnectorDirection#WEST west}.
@param child the child node
@returns {@link ParentConnectorDirection#NORTH} if nodes are placed in rows, {@link ParentConnectorDirection#WEST}
otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Arranges the {@link SubtreeShape SubtreeShapes} of children of the local root node in rows or columns that fit the specified {@link AspectRatioNodePlacer#aspectRatio aspect ratio}.
@param localRoot the local root node
@param parentConnectorDirection the direction of the connector
@returns the {@link SubtreeShape} containing the local root and all its subtrees
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): SubtreeShape | null;
/**
Gets or sets whether child nodes are distributed in horizontal rows or vertical columns.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer%23horizontal Online Documentation}*/
horizontal: boolean;
/**
Gets or sets the vertical distance between the {@link SubtreeShape}s.
The nodes will be placed below each other, keeping this distance.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer%23verticalDistance Online Documentation}*/
verticalDistance: number;
/**
Gets or sets the horizontal distance between the {@link SubtreeShape}s.
The nodes will be placed next to each other, keeping this distance.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer%23horizontalDistance Online Documentation}*/
horizontalDistance: number;
/**
Gets or sets the preferred aspect ratio (width/height) for the subtrees.
This ratio determines among how many rows/columns the nodes will be distributed.
The aspect ratio needs to be positive.
@throws {Exception} <b>(ArgumentError)</b> if the specified aspect ratio is <code>0</code> or negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer%23aspectRatio Online Documentation}*/
aspectRatio: number;
/**
Gets or sets how rows/columns are aligned with each other.
@throws {Exception} <b>(ArgumentError)</b> if an unknown fill style is specified
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioNodePlacer%23fillStyle Online Documentation}*/
fillStyle: FillStyle;
static $class: Class;
private $_AspectRatioNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AspectRatioNodePlacer;
}
export interface AspectRatioTreeLayout extends MultiStageLayout {
}
/**
The {@link AspectRatioTreeLayout} arranges tree graphs in a compact fashion.
<b>Layout Style</b>
This layout algorithm tries to generate compact tree layouts with a certain preferred aspect ratio.
<b>Concept</b>
The layout algorithm starts from the root and recursively assigns coordinates to all tree nodes. In this manner, leaf
nodes will be placed first, while each parent node is placed centered above its child nodes.
<b>Features</b>
The aspect ratio of each subtree can be specified individually. A {@link IDataProvider} registered with {@link AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY}
returns the aspect ratio for each local root node.
A custom node can be defined as root of the tree using a {@link IDataProvider} registered with the graph with
key {@link AspectRatioTreeLayout#SELECTED_ROOT_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout Online Documentation}*/
export class AspectRatioTreeLayout {
/**
Creates a new {@link AspectRatioTreeLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.horizontalDistance] The horizontal distance between adjacent nodes. This option sets the {@link AspectRatioTreeLayout#horizontalDistance} property on the created object.
@param [options.verticalDistance] The vertical distance between adjacent nodes. This option sets the {@link AspectRatioTreeLayout#verticalDistance} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link AspectRatioTreeLayout#componentLayoutEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link AspectRatioTreeLayout#hideGroupsStageEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link AspectRatioTreeLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link AspectRatioTreeLayout#selfLoopRouterEnabled} property on the created object.
@param [options.comparer] The {@link IComparer} that will be used for sorting the {@link YNode#sortOutEdges outgoing edges} of each local root in the tree before they are being arranged. This option sets the {@link AspectRatioTreeLayout#comparer} property on the created object.
@param [options.aspectRatio] The default aspect ratio for this {@link AspectRatioTreeLayout}. This option sets the {@link AspectRatioTreeLayout#aspectRatio} property on the created object.
@param [options.rootPlacement] The desired placement of the tree's root node. This option sets the {@link AspectRatioTreeLayout#rootPlacement} property on the created object.
@param [options.subtreeArrangement] How the children of a local root are arranged and how the edges between them are routed. This option sets the {@link AspectRatioTreeLayout#subtreeArrangement} property on the created object.
@param [options.bendDistance] The preferred distance between any two bends of an edge. This option sets the {@link AspectRatioTreeLayout#bendDistance} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link MultiStageLayout#orientationLayoutEnabled} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout Online Documentation}*/
constructor(options?: {
horizontalDistance?: number;
verticalDistance?: number;
componentLayoutEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
selfLoopRouterEnabled?: boolean;
comparer?: IComparer<any> | null;
aspectRatio?: number;
rootPlacement?: RootPlacement | RootPlacementStringValues;
subtreeArrangement?: SubtreeArrangement | SubtreeArrangementStringValues;
bendDistance?: number;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
orientationLayoutEnabled?: boolean;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
labelingEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Arranges the given graph considering the specified {@link AspectRatioTreeLayout#aspectRatio aspect ratio}.
@param graph the input graph
@throws {Exception} <b>(InvalidGraphStructureError)</b> if the input graph is not a tree
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Adds bends to the given {@link IEdgeLayout}.
This method is called by {@link AspectRatioTreeLayout#applyLayoutCore} for each edge, after the nodes are placed, to
route the path of the edges. It may be overridden to apply a custom routing style.
@param edgeLayout the layout of the edge that is routed
@param localRoot the local root of the subtree
@param child the child connected to the local root with the given edge
@param rootPlacement the placement specifier of the local root
@param routingPolicy the direction specifier for the routing
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23createBends(IEdgeLayout%2CYNode%2CYNode%2CRootPlacement%2CSubtreeArrangement) Online Documentation}*/
createBends(edgeLayout: IEdgeLayout | null, localRoot: YNode | null, child: YNode | null, rootPlacement: RootPlacement | RootPlacementStringValues, routingPolicy: SubtreeArrangement | SubtreeArrangementStringValues): void;
/**
Retrieves the aspect ratio for the subtree that is rooted at the given node.
If there is an individual aspect ratio for the subtree provided by a {@link IDataProvider} registered with key
{@link AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY}, that ratio will be returned. Otherwise, the {@link AspectRatioTreeLayout#aspectRatio default aspect ratio}
will be returned.
This method is called by {@link AspectRatioTreeLayout#applyLayoutCore} before a subtree is arranged. It may be
overridden to use another approach to define the aspect ratio for subtrees.
@param localRoot the root node of the subtree
@returns the aspect ratio of the subtree rooted at the local root node
@see AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY
@see AspectRatioTreeLayout#aspectRatio
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23getAspectRatio(YNode) Online Documentation}*/
getAspectRatio(localRoot: YNode | null): number;
/**
Retrieves the root placement for the subtree that is rooted at the given node.
Either an individual root placement for the subtree (defined via a {@link IDataProvider} registered with
{@link AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY}), or if there is none, the {@link AspectRatioTreeLayout#rootPlacement default root placement} is returned.
This method is called by {@link AspectRatioTreeLayout#applyLayoutCore} after a subtree is arranged to align the
local root node. It may be overridden to use another approach to define the root placement for a subtree.
@param localRoot the root of the subtree
@returns the root placement of the subtree
@see AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
@see AspectRatioTreeLayout#rootPlacement
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23getRootPlacement(Object) Online Documentation}*/
getRootPlacement(localRoot: any | null): RootPlacement;
/**
Retrieves the routing for the subtree that is rooted at the given node.
Either an individual routing for the subtree (defined via a {@link IDataProvider} registered with
{@link AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY}), or if there is none, the {@link AspectRatioTreeLayout#subtreeArrangement default routing} is returned.
This method is called by {@link AspectRatioTreeLayout#applyLayoutCore} to determine the direction of the subtrees.
It may be overridden to use another approach to define the routing for a subtree.
@param localRoot the root of the subtree
@returns the routing policy for the subtree
@see AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY
@see AspectRatioTreeLayout#subtreeArrangement
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23getSubtreeArrangement(Object) Online Documentation}*/
getSubtreeArrangement(localRoot: any | null): SubtreeArrangement;
/**
Retrieves all children of the given local root.
This method is called by {@link AspectRatioTreeLayout#applyLayoutCore} to arrange the subtrees below the local
root.
@param localRoot the root of a subtree
@returns all successors of the given node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23getSuccessors(YNode) Online Documentation}*/
getSuccessors(localRoot: YNode | null): INodeCursor | null;
/**
Data provider key for specifying a target aspect ratio for each subtree
The aspect ratio needs to be greater than <code>0</code>.
<ul>
<li>aspect ratio <code>1</code>: width and height of the layout should be the same
<li>aspect ratio between <code>0</code> and <code>1</code>: the height of the layout should be greater than its width
<li>aspect ratio greater than <code>1</code>: the width of the layout should be greater than its height
</ul>
If no specific ratio is defined for a subtree, the layout algorithm falls back to the default {@link AspectRatioTreeLayout#aspectRatio aspect ratio}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23SUBTREE_ASPECT_RATIO_DP_KEY Online Documentation}*/
static SUBTREE_ASPECT_RATIO_DP_KEY: NodeDpKey<number> | null;
/**
Data provider key for specifying the placement of each subtree root
If no root placement is specified for one of the subtree roots, the default {@link AspectRatioTreeLayout#rootPlacement root placement} will be used.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23ROOT_PLACEMENT_DP_KEY Online Documentation}*/
static ROOT_PLACEMENT_DP_KEY: NodeDpKey<RootPlacement> | null;
/**
Data provider key for specifying the routing direction for each subtree root
The children in a subtree are arranged either horizontally or vertically. The edges are routed to the top of the child
nodes or at the side, respectively. Directions depend on the {@link MultiStageLayout#layoutOrientation layout orientation} and refer to {@link LayoutOrientation#TOP_TO_BOTTOM}
for this description.
If no specific routing policy is specified for a subtree root, the layout algorithm uses the default {@link AspectRatioTreeLayout#subtreeArrangement routing direction}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23SUBTREE_ROUTING_POLICY_DP_KEY Online Documentation}*/
static SUBTREE_ROUTING_POLICY_DP_KEY: NodeDpKey<SubtreeArrangement> | null;
/**
Data provider key for marking the node that will be used as root node of the tree
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23SELECTED_ROOT_DP_KEY Online Documentation}*/
static SELECTED_ROOT_DP_KEY: NodeDpKey<boolean> | null;
/**
The input graph this algorithm is handling.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23graph Online Documentation}*/
graph: LayoutGraph | null;
/**
Gets or sets the horizontal distance between adjacent nodes.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23horizontalDistance Online Documentation}*/
horizontalDistance: number;
/**
Gets or sets the vertical distance between adjacent nodes.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23verticalDistance Online Documentation}*/
verticalDistance: number;
/**
Sets whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated.
<b>Warning:</b> {@link AspectRatioTreeLayout} can only handle single components. Disabling {@link ComponentLayout} will lead
to errors during execution.
@see MultiStageLayout#componentLayoutEnabled
@see MultiStageLayout#componentLayout
@see ComponentLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23componentLayoutEnabled Online Documentation}*/
componentLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for hiding group nodes is activated.
<b>Warning:</b> {@link AspectRatioTreeLayout} cannot handle group nodes. Disabling {@link HideGroupsStage} will lead to
errors during execution.
@see MultiStageLayout#hideGroupsStageEnabled
@see MultiStageLayout#hideGroupsStage
@see HideGroupsStage
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23hideGroupsStageEnabled Online Documentation}*/
hideGroupsStageEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing parallel edges is activated.
<b>Warning:</b> {@link AspectRatioTreeLayout} cannot handle parallel edges. Disabling {@link ParallelEdgeRouter} will lead to
errors during execution.
@see MultiStageLayout#parallelEdgeRouterEnabled
@see MultiStageLayout#parallelEdgeRouter
@see ParallelEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23parallelEdgeRouterEnabled Online Documentation}*/
parallelEdgeRouterEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing self-loops is activated.
<b>Warning:</b> {@link AspectRatioTreeLayout} cannot handle self-loops. Disabling {@link SelfLoopRouter} will lead to errors
during execution.
@see MultiStageLayout#selfLoopRouterEnabled
@see MultiStageLayout#selfLoopRouter
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23selfLoopRouterEnabled Online Documentation}*/
selfLoopRouterEnabled: boolean;
/**
Gets or sets the {@link IComparer} that will be used for sorting the {@link YNode#sortOutEdges outgoing edges} of each local root in the tree before they are being
arranged.
<h2>Note</h2>
The algorithm will only use the order induced by the
{@link IComparer}, if the nodes (more precisely, their whole subtrees) have equal sizes. The reason is that the algorithm tries to find a
good arrangement which optimizes the target aspect ratio. If the node ordering is already specified, this optimization
would be impossible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23comparer Online Documentation}*/
comparer: IComparer<any> | null;
/**
Gets or sets the default aspect ratio for this {@link AspectRatioTreeLayout}.
This aspect ratio is used for all subtrees for which there is no specific aspect ratio defined in a {@link IDataProvider}
registered with {@link AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY}.
The aspect ratio needs to have a positive value.
@throws {Exception} <b>(ArgumentError)</b> if the specified aspect ratio is <code>0</code> or negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23aspectRatio Online Documentation}*/
aspectRatio: number;
/**
Gets or sets the desired placement of the tree's root node.
@throws {Exception} <b>(ArgumentError)</b> if an unknown placement specifiers is set
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23rootPlacement Online Documentation}*/
rootPlacement: RootPlacement;
/**
Gets or sets how the children of a local root are arranged and how the edges between them are routed.
@throws {Exception} <b>(ArgumentError)</b> if the specified routing policy is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23subtreeArrangement Online Documentation}*/
subtreeArrangement: SubtreeArrangement;
/**
Gets or sets the preferred distance between any two bends of an edge.
Additionally, the preferred bend distance governs the distance between the first and last edges and the corresponding
ports.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayout%23bendDistance Online Documentation}*/
bendDistance: number;
static $class: Class;
private $_AspectRatioTreeLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AspectRatioTreeLayout;
}
export interface AssistantNodePlacer extends RotatableNodePlacerBase {
}
/**
The {@link AssistantNodePlacer} places nodes that are marked as assistants left and right of their parents and all
other nodes below the assistant nodes.
<b>Layout Style</b>
<b>Concept</b>
This {@link ITreeLayoutNodePlacer} delegates the placement of all assistant nodes to an instance of
{@link LeftRightNodePlacer}. The non-assistant nodes are handled by another {@link AssistantNodePlacer#childNodePlacer node placer}. To be able to do this, {@link AssistantNodePlacer}
uses a {@link IProcessor} which prepares the graph for node placing.
A {@link IDataProvider} that is registered using {@link AssistantNodePlacer#ASSISTANT_NODE_DP_KEY} returns
<code>true</code> for assistant nodes and <code>false</code> for all other nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer Online Documentation}*/
export class AssistantNodePlacer {
/**
Creates a new {@link AssistantNodePlacer} instance that uses the transformation defined by the given matrix.
@param modificationMatrix the transformation matrix
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23AssistantNodePlacer(RotatableNodePlacerMatrix) Online Documentation}*/
constructor(modificationMatrix: RotatableNodePlacerMatrix | null);
/**
Creates a new {@link AssistantNodePlacer} instance that uses the transformation defined by the given matrix.
@param options The parameters to pass.
@param options.modificationMatrix the transformation matrix
@param [options.spacing] The spacing between subtrees for the delegate {@link ITreeLayoutNodePlacer}. This option sets the {@link AssistantNodePlacer#spacing} property on the created object.
@param [options.childNodePlacer] The {@link ITreeLayoutNodePlacer} instance that places the non-assistant children. This option sets the {@link AssistantNodePlacer#childNodePlacer} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23AssistantNodePlacer(RotatableNodePlacerMatrix) Online Documentation}*/
constructor(options: {
modificationMatrix: RotatableNodePlacerMatrix | null;
spacing?: number;
childNodePlacer?: ITreeLayoutNodePlacer | null;
});
/**
Creates a new {@link AssistantNodePlacer} instance with default settings.
@param [options] The parameters to pass.
@param [options.spacing] The spacing between subtrees for the delegate {@link ITreeLayoutNodePlacer}. This option sets the {@link AssistantNodePlacer#spacing} property on the created object.
@param [options.childNodePlacer] The {@link ITreeLayoutNodePlacer} instance that places the non-assistant children. This option sets the {@link AssistantNodePlacer#childNodePlacer} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23AssistantNodePlacer() Online Documentation}*/
constructor(options?: {
spacing?: number;
childNodePlacer?: ITreeLayoutNodePlacer | null;
});
/**
Delegates the creation of {@link RotatableNodePlacerBase#createFromSketchComparer from-sketch comparators} to the two
{@link ITreeLayoutNodePlacer}s handling assistant and non-assistant nodes.
The from-sketch {@link IComparer}s are {@link AssistantNodePlacer#createCompoundComparer combined} into a compound comparator. The one for the edges connecting to assistant nodes is
retrieved from a {@link LeftRightNodePlacer} while the other one is provided by the {@link AssistantNodePlacer#childNodePlacer child placer}.
@returns the {@link IComparer} to compare edges, considering the initial locations of their target nodes
@see AssistantNodePlacer#childNodePlacer
@see AssistantNodePlacer#createCompoundComparer
@see IFromSketchNodePlacer#createFromSketchComparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23createComparer() Online Documentation}*/
createComparer(): IComparer<any> | null;
/**
Creates a {@link IComparer} that compares outgoing edges connecting to assistant nodes and edges connecting to non-assistant nodes
separately.
@param assistantComparer the {@link IComparer} for edges connecting to <em>assistant</em> nodes
@param childComparer the {@link IComparer} for edges connecting to <em>non-assistant</em> nodes
@returns a {@link IComparer} that compares edges to assistant nodes separately from edges to non-assistant nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23createCompoundComparer(IComparer%3CObject%3E%2CIComparer%3CObject%3E) Online Documentation}*/
static createCompoundComparer(assistantComparer: IComparer<any> | null, childComparer: IComparer<any> | null): IComparer<any> | null;
/**
Creates a {@link IProcessor} that prepares the graph for placing assistant node.
The pre-processing step adds a dummy node as a child to the current root and changes all edges of non-assistant children
such that they start at this dummy node. That way, {@link AssistantNodePlacer} is able to delegate the assistant
nodes to a {@link ITreeLayoutNodePlacer} different from the one used for placing non-assistant nodes. The {@link AssistantNodePlacer#childNodePlacer child node placer} is
assigned to the dummy node. So the non-assistant children are placed by this delegate placer.
All changes will be cleaned up during the post-processing step.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the root node handled by this {@link ITreeLayoutNodePlacer}
@returns the {@link IProcessor} instance that prepares the graph for {@link AssistantNodePlacer}
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Determines a connector direction for the given child node.
<b>Warning:</b> This method should not be called! Finding the directions for the connectors is delegated to the {@link ITreeLayoutNodePlacer}
for the assistant nodes by {@link AssistantNodePlacer#determineChildConnectors}.
@param child
@throws {Exception} <b>(InvalidOperationError)</b> if this method is called
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Delegates the calculation of the child connectors to the {@link LeftRightNodePlacer} which arranges the assistant
nodes.
@param localRoot the local root node
@param connectorMap the map that must be used for storing the direction specifiers of the child nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Delegates the arrangement of {@link SubtreeShape}s to an instance of {@link LeftRightNodePlacer}.
The {@link LeftRightNodePlacer} places all assistant nodes left and right of the local root node. The {@link SubtreeShape}
associated with the dummy node which contains
{@link SubtreeShape}s of the non-assistant nodes is placed below the assistant nodes, center-aligned with respect to
the local root node.
@param nodeShapeProvider the {@link IDataProvider} for obtaining an initial shape of the root node
@param subtreeShapeProvider the {@link IDataProvider} for accessing the pre-calculated shapes of the subtrees
@param graph the input graph
@param localRoot the root of the subtree that should be arranged by this method
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged {@link SubtreeShape} containing the local root and all assistant nodes and non-assistant nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23placeSubtree(IDataProvider%2CIDataProvider%2CLayoutGraph%2CYNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtree(nodeShapeProvider: IDataProvider | null, subtreeShapeProvider: IDataProvider | null, graph: LayoutGraph | null, localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection): SubtreeShape | null;
/**
Arranges the local root relative to the subtree shape of its children.
<b>Warning:</b> This method should not be called! Placing subtrees is delegated to the {@link ITreeLayoutNodePlacer} for the assistant nodes
and the {@link AssistantNodePlacer#childNodePlacer placer for non-assistant nodes} by method {@link AssistantNodePlacer#placeSubtree}.
@param localRoot
@param parentConnectorDirection
@throws {Exception} <b>(InvalidOperationError)</b> if this method is called
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
/**
Data provider key for marking which nodes are placed as assistants
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23ASSISTANT_NODE_DP_KEY Online Documentation}*/
static ASSISTANT_NODE_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the spacing between subtrees for the delegate {@link ITreeLayoutNodePlacer}.
The spacing needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23spacing Online Documentation}*/
spacing: number;
/**
Gets or sets the {@link ITreeLayoutNodePlacer} instance that places the non-assistant children.
These non-assistant child nodes will be placed below the assistant child nodes and are arranged by the given
{@link ITreeLayoutNodePlacer}. Non-assistant child nodes are the ones that are not marked in the {@link IDataProvider}
registered with {@link AssistantNodePlacer#ASSISTANT_NODE_DP_KEY}.
@see AssistantNodePlacer#ASSISTANT_NODE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/AssistantNodePlacer%23childNodePlacer Online Documentation}*/
childNodePlacer: ITreeLayoutNodePlacer | null;
static $class: Class;
private $_AssistantNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AssistantNodePlacer;
}
export interface BusNodePlacer extends RotatableNodePlacerBase {
}
/**
The {@link BusNodePlacer} creates a bus to which all child nodes and the local root node are connected.
<b>Layout Style</b>
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusNodePlacer Online Documentation}*/
export class BusNodePlacer {
/**
Creates a new {@link BusNodePlacer} instance that uses the transformation defined by the given matrix.
@param modificationMatrix the transformation matrix
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusNodePlacer%23BusNodePlacer(RotatableNodePlacerMatrix) Online Documentation}*/
constructor(modificationMatrix: RotatableNodePlacerMatrix | null);
/**
Creates a new {@link BusNodePlacer} instance that uses the transformation defined by the given matrix.
@param options The parameters to pass.
@param options.modificationMatrix the transformation matrix
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusNodePlacer%23BusNodePlacer(RotatableNodePlacerMatrix) Online Documentation}*/
constructor(options: {
modificationMatrix: RotatableNodePlacerMatrix | null;
spacing?: number;
});
/**
Creates a new instance of {@link BusNodePlacer} with default settings.
@param [options] The parameters to pass.
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusNodePlacer%23BusNodePlacer() Online Documentation}*/
constructor(options?: {
spacing?: number;
});
/**
Creates a {@link IComparer} for outgoing edges that takes the rotation into account.
This implementation keeps the current order of edges.
@returns <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusNodePlacer%23createComparer() Online Documentation}*/
createComparer(): IComparer<any> | null;
/**
Determines a connector direction for the given child node.
<b>Warning:</b> This method should not be called! Finding the directions for the connectors is done by
{@link BusNodePlacer#determineChildConnectors}.
@param child
@throws {Exception} <b>(InvalidOperationError)</b> if this method is called
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Determines connector directions depending on whether or not the child nodes are placed above or below the bus.
When a child node is placed above the bus, the edge should connect {@link ParentConnectorDirection#NORTH north}, otherwise it should connect {@link ParentConnectorDirection#SOUTH south}.
@param localRoot the local root
@param connectorMap the map that must be used for storing the direction specifiers of the child nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusNodePlacer%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Places all children of the local root below and above a bus segment that is shared by the edges.
The bus is located below the local root and the children are placed below this bus or next to the local root.
@param localRoot the local root node
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged subtree shape that contains the local root node and all its children
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
static $class: Class;
private $_BusNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BusNodePlacer;
}
export interface ClassicTreeLayout extends MultiStageLayout {
}
/**
This layout algorithm arranges graphs with a tree structure.
<b>Layout Style</b>
{@link ClassicTreeLayout} is designed to arrange directed and undirected trees that have a unique root node. All
children are placed below their parent in relation to the main layout direction. The edges of the graph are routed as
straight-line segments or in an orthogonal bus-like fashion.
Tree layout algorithms are commonly used for visualizing relational data and for producing diagrams of high quality that
are able to reveal possible hierarchic properties of the graph. More precisely, they find applications in dataflow
analysis, software engineering, bioinformatics and business administration.
<b>Concept</b>
The layout algorithm starts from the root and recursively assigns coordinates to all tree nodes. In this manner, leaf
nodes will be placed first, while each parent node is placed centered above its children.
<b>Features</b>
The layout algorithm supports custom sorting of sibling nodes by using a specific {@link NodeOrderComparer}. Each
tree node may have an individual {@link NodeOrderComparer} that is stored in a {@link IDataProvider}
registered with the graph with {@link NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY}. If this {@link NodeOrderComparer}
uses the initial coordinates of the nodes, it allows the incremental insertion of child nodes while keeping the relative
locations of the other nodes.
A custom node can be defined as root of the tree using a {@link IDataProvider} registered with the graph with
key {@link ClassicTreeLayout#SELECTED_ROOT_DP_KEY}.
{@link ClassicTreeLayout} can be configured to {@link ClassicTreeLayout#considerNodeLabels reserve space for node labels}. It can also {@link ClassicTreeLayout#integratedEdgeLabeling place edge labels along edges}
such that the labels won't overlap with other graph elements. Edge labels are placed according to the information stored
in a {@link PreferredPlacementDescriptor} instance. However, the placement along the edge will only affect the
order of multiple labels at the same edge. The algorithm will always place the labels close to the target node.
Grouping of nodes can also be handled by this layout algorithm. It is important that a group node contains a whole
subtree. Otherwise, the group nodes may overlap with each other or with other nodes. Furthermore, the user may specify
minimum size constraints for each group node using {@link IDataProvider} key
{@link GroupingKeys#MINIMUM_NODE_SIZE_DP_KEY}.
This layout algorithm can only handle graphs with a tree structure. To apply it to a general graph, a {@link TreeReductionStage}
can be appended. This stage will temporarily remove some edges of the input graph until a tree is obtained. After the
layout calculation, the stage will reinsert the edges that were removed and route them separately.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout Online Documentation}*/
export class ClassicTreeLayout {
/**
Creates a new {@link ClassicTreeLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.minimumBusSegmentDistance] The minimum (vertical) distance of the horizontal edge segments of the bus between the root node and the child nodes. This option sets the {@link ClassicTreeLayout#minimumBusSegmentDistance} property on the created object.
@param [options.enforceGlobalLayering] Whether or not a global layering is enforced, that is, each node spans exactly one layer. This option sets the {@link ClassicTreeLayout#enforceGlobalLayering} property on the created object.
@param [options.leafPlacement] The layout policy for leaf nodes. This option sets the {@link ClassicTreeLayout#leafPlacement} property on the created object.
@param [options.groupingSupported] Whether or not group nodes are handled by the layout algorithm. This option sets the {@link ClassicTreeLayout#groupingSupported} property on the created object.
@param [options.comparer] The {@link IComparer} instance that is used for sorting the outgoing edges of each node of the tree. This option sets the {@link ClassicTreeLayout#comparer} property on the created object.
@param [options.portStyle] The port assignment policy that will be applied. This option sets the {@link ClassicTreeLayout#portStyle} property on the created object.
@param [options.edgeRoutingStyle] The edge routing style used by this layout algorithm. This option sets the {@link ClassicTreeLayout#edgeRoutingStyle} property on the created object.
@param [options.minimumNodeDistance] The minimum horizontal distance between adjacent nodes within the same layer. This option sets the {@link ClassicTreeLayout#minimumNodeDistance} property on the created object.
@param [options.minimumLayerDistance] The minimum distance between two adjacent layers. This option sets the {@link ClassicTreeLayout#minimumLayerDistance} property on the created object.
@param [options.considerNodeLabels] Whether or not the layout algorithm reserves space for node labels. This option sets the {@link ClassicTreeLayout#considerNodeLabels} property on the created object.
@param [options.integratedEdgeLabeling] Whether or not the layout algorithm reserves space for edge labels and places them. This option sets the {@link ClassicTreeLayout#integratedEdgeLabeling} property on the created object.
@param [options.busAlignment] The vertical bus alignment for orthogonally routed edge buses. This option sets the {@link ClassicTreeLayout#busAlignment} property on the created object.
@param [options.verticalAlignment] The vertical alignment of the nodes within their corresponding layers. This option sets the {@link ClassicTreeLayout#verticalAlignment} property on the created object.
@param [options.modificationMatrix] The modification matrix used for rotating / mirroring the layout. This option sets the {@link ClassicTreeLayout#modificationMatrix} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link ClassicTreeLayout#componentLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link ClassicTreeLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link ClassicTreeLayout#selfLoopRouterEnabled} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link MultiStageLayout#orientationLayoutEnabled} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout Online Documentation}*/
constructor(options?: {
minimumBusSegmentDistance?: number;
enforceGlobalLayering?: boolean;
leafPlacement?: LeafPlacement | LeafPlacementStringValues;
groupingSupported?: boolean;
comparer?: IComparer<any> | null;
portStyle?: PortStyle | PortStyleStringValues;
edgeRoutingStyle?: ClassicTreeLayoutEdgeRoutingStyle | ClassicTreeLayoutEdgeRoutingStyleStringValues;
minimumNodeDistance?: number;
minimumLayerDistance?: number;
considerNodeLabels?: boolean;
integratedEdgeLabeling?: boolean;
busAlignment?: number;
verticalAlignment?: number;
modificationMatrix?: RotatableNodePlacerMatrix | null;
componentLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
selfLoopRouterEnabled?: boolean;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
orientationLayoutEnabled?: boolean;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Arranges the given graph as a tree.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Data provider key for marking the node that will be used as root node of the tree
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23SELECTED_ROOT_DP_KEY Online Documentation}*/
static SELECTED_ROOT_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the minimum (vertical) distance of the horizontal edge segments of the bus between the root node and the
child nodes.
If this distance is set to <code>0</code>, all segments are routed in a classical bus-like style.
<b>Warning:</b> The specified distance is only considered if the {@link ClassicTreeLayout#edgeRoutingStyle layout style} is set to {@link ClassicTreeLayoutEdgeRoutingStyle#ORTHOGONAL} and
option {@link ClassicTreeLayout#enforceGlobalLayering} is disabled.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is smaller than <code>0</code>
@see ClassicTreeLayout#edgeRoutingStyle
@see ClassicTreeLayout#portStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23minimumBusSegmentDistance Online Documentation}*/
minimumBusSegmentDistance: number;
/**
Gets or sets whether or not a global layering is enforced, that is, each node spans exactly one layer.
If it is enabled, the algorithm ensures that nodes never span more than one layer. Otherwise, nodes with larger size
might span two or more layers such that a more compact layout is produced.
This setting can be advantageous, if the hierarchical structure of the tree should be emphasized.
<b>Warning:</b>
Disabling this feature to generate more compact layouts may force the algorithm to ignore the specified relative
vertical alignment for nodes.
@see ClassicTreeLayout#verticalAlignment
@see ClassicTreeLayout#verticalAlignment
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23enforceGlobalLayering Online Documentation}*/
enforceGlobalLayering: boolean;
/**
Gets or sets the layout policy for leaf nodes.
@throws {Exception} <b>(ArgumentError)</b> if the specified policy is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23leafPlacement Online Documentation}*/
leafPlacement: LeafPlacement;
/**
Gets or sets whether or not group nodes are handled by the layout algorithm.
<h2>Note</h2>
If this option is disabled, group nodes will be temporarily hidden and reinserted to enclose their content. This may
lead to node overlaps.
<b>Warning:</b>
Grouping only works correctly, if each group contains a complete subtree. This means that for each group there is a node
such that the group contains exactly this node and all its descendants.
@see MultiStageLayout#hideGroupsStageEnabled
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23groupingSupported Online Documentation}*/
groupingSupported: boolean;
/**
Gets or sets the {@link IComparer} instance that is used for sorting the outgoing edges of each node of the tree.
This comparator defines the relative order of the child nodes in the layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23comparer Online Documentation}*/
comparer: IComparer<any> | null;
/**
Gets or sets the port assignment policy that will be applied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23portStyle Online Documentation}*/
portStyle: PortStyle;
/**
Gets or sets the edge routing style used by this layout algorithm.
If {@link ClassicTreeLayoutEdgeRoutingStyle#PLAIN} is set, the edges will be routed as straight-line segments. If {@link ClassicTreeLayoutEdgeRoutingStyle#ORTHOGONAL}
is set, all edges will be routed orthogonally in a bus-like fashion.
@throws {Exception} <b>(ArgumentError)</b> if an invalid layout style is provided
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23edgeRoutingStyle Online Documentation}*/
edgeRoutingStyle: ClassicTreeLayoutEdgeRoutingStyle;
/**
Gets or sets the minimum horizontal distance between adjacent nodes within the same layer.
@throws {Exception} <b>(ArgumentError)</b> if the given distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23minimumNodeDistance Online Documentation}*/
minimumNodeDistance: number;
/**
Gets or sets the minimum distance between two adjacent layers.
@throws {Exception} <b>(ArgumentError)</b> if the given distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23minimumLayerDistance Online Documentation}*/
minimumLayerDistance: number;
/**
Gets or sets whether or not the layout algorithm reserves space for node labels.
<b>Warning:</b> When enabling this property, any previously specified {@link MultiStageLayout#labeling labeling algorithm} will be overwritten and when disabling it, any
currently specified labeling algorithm will be disabled via property
{@link MultiStageLayout#labelingEnabled}. Therefore, it is recommended to only use this convenience property instead of manually changing the mentioned other
properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets whether or not the layout algorithm reserves space for edge labels and places them.
To define the desired placement for each label add a {@link PreferredPlacementDescriptor} on
{@link IEdgeLabelLayout}.
This method is a convenience method that assures that the {@link MultiStageLayout#labeling label layouter} is of type {@link LabelLayoutTranslator}
and {@link LabelLayoutTranslator#translateEdgeLabels} is set to <code>true</code>.
<b>Warning:</b> When enabling this property, any previously specified {@link MultiStageLayout#labeling labeling algorithm} will be overwritten and when disabling it, any
currently specified labeling algorithm will be disabled via property
{@link MultiStageLayout#labelingEnabled}. Therefore, it is recommended to only use this convenience property instead of manually changing the mentioned other
properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23integratedEdgeLabeling Online Documentation}*/
integratedEdgeLabeling: boolean;
/**
Gets or sets the vertical bus alignment for orthogonally routed edge buses.
The bus alignment determines the relative position of an edge bus between two subsequent layers. The value of this
property must lie within the interval <code>[0,1]</code> where:
<ul>
<li>A value of <code>0</code> places the bus at the top, directly below the parent node.
<li>A value of <code>0.5</code> places the bus in the middle between parent and child nodes.
<li>A value of <code>1</code> places the bus at the bottom, directly above the child nodes.
</ul>
<b>Warning:</b> Bus alignment is taken into account only for:
<ul>
<li>{@link ClassicTreeLayoutEdgeRoutingStyle#ORTHOGONAL} layout style with {@link ClassicTreeLayout#enforceGlobalLayering global layering} enforced.
<li>{@link ClassicTreeLayoutEdgeRoutingStyle#ORTHOGONAL} layout style with child placement policy
{@link LeafPlacement#ALL_LEAVES_ON_SAME_LAYER}.
</ul>
@throws {Exception} <b>(ArgumentError)</b> if <code>busAlignment</code> is not in <code>[0,1]</code>
@see ClassicTreeLayout#edgeRoutingStyle
@see ClassicTreeLayout#enforceGlobalLayering
@see ClassicTreeLayout#leafPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23busAlignment Online Documentation}*/
busAlignment: number;
/**
Gets or sets the vertical alignment of the nodes within their corresponding layers.
The value for the vertical alignment is considered as relative to the height of the corresponding layer, which is
determined by the maximum height of the nodes that belong to the particular layer.
The value of this property must lie within the interval <code>[0,1]</code>, where:
<ul>
<li>A value of <code>0</code> corresponds to top-alignment.
<li>A value of <code>0.5</code> corresponds to center-alignment.
<li>A value of <code>1</code> corresponds to bottom-alignment.
</ul>
<b>Warning:</b> If {@link ClassicTreeLayout#enforceGlobalLayering global layering} is not enforced, the value of this property is usually ignored.
@throws {Exception} <b>(ArgumentError)</b> if vertical alignment value does not lie within <code>[0,1]</code>
@see ClassicTreeLayout#enforceGlobalLayering
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23verticalAlignment Online Documentation}*/
verticalAlignment: number;
/**
Gets or sets the modification matrix used for rotating / mirroring the layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23modificationMatrix Online Documentation}*/
modificationMatrix: RotatableNodePlacerMatrix | null;
/**
Sets whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated.
<b>Warning:</b> {@link ClassicTreeLayout} can only handle single components. Disabling {@link ComponentLayout} will lead to
errors during execution.
@see MultiStageLayout#componentLayoutEnabled
@see MultiStageLayout#componentLayout
@see ComponentLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23componentLayoutEnabled Online Documentation}*/
componentLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing parallel edges is activated.
<b>Warning:</b> {@link ClassicTreeLayout} cannot handle parallel edges. Disabling {@link ParallelEdgeRouter} will lead to
errors during execution.
@see MultiStageLayout#parallelEdgeRouterEnabled
@see MultiStageLayout#parallelEdgeRouter
@see ParallelEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23parallelEdgeRouterEnabled Online Documentation}*/
parallelEdgeRouterEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing self-loops is activated.
<b>Warning:</b> {@link ClassicTreeLayout} cannot handle self-loops. Disabling {@link SelfLoopRouter} will lead to errors
during execution.
@see MultiStageLayout#selfLoopRouterEnabled
@see MultiStageLayout#selfLoopRouter
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassicTreeLayout%23selfLoopRouterEnabled Online Documentation}*/
selfLoopRouterEnabled: boolean;
static $class: Class;
private $_ClassicTreeLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClassicTreeLayout;
}
export interface CompactNodePlacer extends NodePlacerBase {
/**
@inheritdoc*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
@inheritdoc*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
} /**
The {@link CompactNodePlacer} produces a compact placement of the children.
<b>Layout Style</b>
This {@link ITreeLayoutNodePlacer} uses a dynamic optimization approach that chooses a placement strategy of the children of
the associated local root such that the overall result is compact with respect to a specified {@link CompactNodePlacer#preferredAspectRatio aspect ratio}. The set of
applied strategies is predefined and cannot be customized. A node can either be a normal tree node or an {@link AssistantNodePlacer#ASSISTANT_NODE_DP_KEY assistant node},
in which case the {@link AssistantNodePlacer} is chosen automatically (i.e., you shouldn't explicitly map nodes to
instances of {@link AssistantNodePlacer} but mark them by means of key
{@link AssistantNodePlacer#ASSISTANT_NODE_DP_KEY}).
<h2>Note</h2>
This placer always produces orthogonal edge routes and doesn't support {@link PortConstraint port constraints}, {@link PortCandidate port candidates} and
{@link TreeLayout#CRITICAL_EDGE_DP_KEY critical edges}.
<h2>Note</h2>
For larger graphs, the required runtime may be much higher than that of other {@link ITreeLayoutNodePlacer} implementations.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer Online Documentation}*/
export class CompactNodePlacer {
/**
Creates a new {@link CompactNodePlacer} instance with default settings.
@param [options] The parameters to pass.
@param [options.horizontalDistance] The horizontal distance between nodes or subtrees. This option sets the {@link CompactNodePlacer#horizontalDistance} property on the created object.
@param [options.verticalDistance] The vertical distance between nodes or subtrees. This option sets the {@link CompactNodePlacer#verticalDistance} property on the created object.
@param [options.preferredAspectRatio] The preferred aspect ratio (width/height) for the subtrees. This option sets the {@link CompactNodePlacer#preferredAspectRatio} property on the created object.
@param [options.minimumFirstSegmentLength] The minimum length for the first segment of an edge. This option sets the {@link CompactNodePlacer#minimumFirstSegmentLength} property on the created object.
@param [options.minimumLastSegmentLength] The minimum length for the last segment of an edge. This option sets the {@link CompactNodePlacer#minimumLastSegmentLength} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer Online Documentation}*/
constructor(options?: {
horizontalDistance?: number;
verticalDistance?: number;
preferredAspectRatio?: number;
minimumFirstSegmentLength?: number;
minimumLastSegmentLength?: number;
});
/**
Creates a {@link IProcessor}.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the root node handled by this {@link ITreeLayoutNodePlacer}
@returns a {@link IProcessor} instance or <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Arranges the local root relative to the subtree shape of its children.
@param localRoot
@param parentConnectorDirection
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): SubtreeShape | null;
/**
Data acceptor key for storing/restoring the placement strategy of the children of nodes
Even for small changes/updates in the graph structure, the results produced by the {@link CompactNodePlacer} may
differ significantly. If a {@link IDataAcceptor} is registered with this key, the layout algorithm stores the
applied placement strategy for the nodes' children and keeps it in subsequent layout runs.
<h2>Note</h2>
Adding new nodes may still change the result because some strategies can only be applied to specific subtrees, e.g., the
{@link LeftRightNodePlacer}-like strategy is only applied to nodes where all children are leaf nodes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer%23STRATEGY_MEMENTO_DP_KEY Online Documentation}*/
static STRATEGY_MEMENTO_DP_KEY: NodeDpKey<any> | null;
/**
Gets or sets the horizontal distance between nodes or subtrees.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer%23horizontalDistance Online Documentation}*/
horizontalDistance: number;
/**
Gets or sets the vertical distance between nodes or subtrees.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer%23verticalDistance Online Documentation}*/
verticalDistance: number;
/**
Gets or sets the preferred aspect ratio (width/height) for the subtrees.
The aspect ratio needs to be non-negative.
<h2>Note</h2>
The aspect ratio is only considered for nodes associated with a {@link CompactNodePlacer} and without a parent
associated with such a placer.
<h2>Note</h2>
If the aspect ratio is set to <code>0</code>, the algorithm automatically chooses one that leads to a small overall area.
@throws {Exception} <b>(ArgumentError)</b> if the specified aspect ratio is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer%23preferredAspectRatio Online Documentation}*/
preferredAspectRatio: number;
/**
Gets or sets the minimum length for the first segment of an edge.
During layout calculation, the edges of the tree are directed. Hence, the first segment is always the segment that is
attached to the source, which is the local root node.
The length needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the minimum segment length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer%23minimumFirstSegmentLength Online Documentation}*/
minimumFirstSegmentLength: number;
/**
Gets or sets the minimum length for the last segment of an edge.
During layout calculation, the edges of the tree are directed. Hence, the last segment is always the segment that is
attached to the target, which is a child node.
The length needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the minimum segment length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompactNodePlacer%23minimumLastSegmentLength Online Documentation}*/
minimumLastSegmentLength: number;
static $class: Class;
private $_CompactNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CompactNodePlacer;
}
export interface DefaultNodePlacer extends NodePlacerBase, IFromSketchNodePlacer, ICloneable {
}
/**
{@link DefaultNodePlacer} is the default implementation of interface {@link ITreeLayoutNodePlacer}.
<b>Layout Style</b>
With its default settings, this placer generates tree layouts with orthogonal edge routes and child nodes that are
placed next to each other and below their parent node.
The layout style can be customized in several ways. Subtrees sharing the same local root node are either placed above or
below each other.
<b>Features</b>
{@link SubtreeShape}s can be arranged in each main direction specified in
{@link DefaultNodePlacer#childPlacement}. That way, the subtrees of the same graph can have different directions.
This node placer aligns the local root node according to a chosen {@link DefaultNodePlacer#rootAlignment alignment}.
There are various {@link DefaultNodePlacer#routingStyle routing styles} that result in different edge routes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer Online Documentation}*/
export class DefaultNodePlacer {
/**
Creates a new {@link DefaultNodePlacer} instance with customized {@link DefaultNodePlacer#childPlacement child placement}, {@link DefaultNodePlacer#rootAlignment root alignment}, {@link DefaultNodePlacer#verticalDistance vertical} and
{@link DefaultNodePlacer#horizontalDistance horizontal distance}.
@param childPlacement the child placement specifier describing the style of the arrangement
@param rootAlignment the alignment specifier describing how the root node is aligned with its child nodes
@param verticalDistance the vertical distance between the {@link SubtreeShape}s
@param horizontalDistance the horizontal distance between the {@link SubtreeShape}s
@throws {Exception} <b>(ArgumentError)</b> if the placement specifier or the root alignment specifier is unknown or if the horizontal/vertical distance is
negative
@see DefaultNodePlacer#childPlacement
@see DefaultNodePlacer#rootAlignment
@see DefaultNodePlacer#verticalDistance
@see DefaultNodePlacer#horizontalDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23DefaultNodePlacer(ChildPlacement%2CRootAlignment%2Cnumber%2Cnumber) Online Documentation}*/
constructor(childPlacement: ChildPlacement | ChildPlacementStringValues, rootAlignment: RootAlignment | RootAlignmentStringValues, verticalDistance: number, horizontalDistance: number);
/**
Creates a new {@link DefaultNodePlacer} instance with customized {@link DefaultNodePlacer#childPlacement child placement}, {@link DefaultNodePlacer#verticalDistance vertical} and {@link DefaultNodePlacer#horizontalDistance horizontal distance}.
@param childPlacement the child placement specifier describing the style of the arrangement
@param verticalDistance the vertical distance between the {@link SubtreeShape}s
@param horizontalDistance the horizontal distance between the {@link SubtreeShape}s
@throws {Exception} <b>(ArgumentError)</b> if the placement specifier is unknown or if the horizontal/vertical distance is negative
@see DefaultNodePlacer#childPlacement
@see DefaultNodePlacer#verticalDistance
@see DefaultNodePlacer#horizontalDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23DefaultNodePlacer(ChildPlacement%2Cnumber%2Cnumber) Online Documentation}*/
constructor(childPlacement: ChildPlacement | ChildPlacementStringValues, verticalDistance: number, horizontalDistance: number);
/**
Creates a new {@link DefaultNodePlacer} instance with custom settings.
@param childPlacement the child placement specifier describing the style of the arrangement
@param rootAlignment the alignment specifier describing how the root node is aligned with its child nodes
@param routingStyle the routing style specifier
@param [verticalDistance=40.0] the vertical distance between the {@link SubtreeShape}s
@param [horizontalDistance=40.0] the horizontal distance between the {@link SubtreeShape}s
@param [minFirstSegmentLength=0.0] the minimum length of the first edge segment (connected to the root node)
@param [minLastSegmentLength=0.0] the minimum length of the last edge segment (connected to the child node)
@param [minSlope=0.0] the minimum slope between the root node and the {@link SubtreeShape}s
@param [minSlopeHeight=0.0] the minimum vertical height of sloped edge segments
@throws {Exception} <b>(ArgumentError)</b> if the placement specifier or the root alignment or the routing style is unknown or if the horizontal/vertical distance
or the minimum first/last segment length or the minimum slope or the minimum vertical height is negative
@see DefaultNodePlacer#childPlacement
@see DefaultNodePlacer#rootAlignment
@see DefaultNodePlacer#routingStyle
@see DefaultNodePlacer#verticalDistance
@see DefaultNodePlacer#horizontalDistance
@see DefaultNodePlacer#minimumFirstSegmentLength
@see DefaultNodePlacer#minimumLastSegmentLength
@see DefaultNodePlacer#minimumSlope
@see DefaultNodePlacer#minimumSlopeHeight
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23DefaultNodePlacer(ChildPlacement%2CRootAlignment%2CTreeLayoutEdgeRoutingStyle%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(childPlacement: ChildPlacement | ChildPlacementStringValues, rootAlignment: RootAlignment | RootAlignmentStringValues, routingStyle: TreeLayoutEdgeRoutingStyle | TreeLayoutEdgeRoutingStyleStringValues, verticalDistance?: number, horizontalDistance?: number, minFirstSegmentLength?: number, minLastSegmentLength?: number, minSlope?: number, minSlopeHeight?: number);
/**
Creates a new {@link DefaultNodePlacer} instance with custom settings.
@param options The parameters to pass.
@param options.childPlacement the child placement specifier describing the style of the arrangement
@param options.rootAlignment the alignment specifier describing how the root node is aligned with its child nodes
@param options.routingStyle the routing style specifier
@param [options.verticalDistance=40.0] the vertical distance between the {@link SubtreeShape}s
@param [options.horizontalDistance=40.0] the horizontal distance between the {@link SubtreeShape}s
@param [options.minFirstSegmentLength=0.0] the minimum length of the first edge segment (connected to the root node)
@param [options.minLastSegmentLength=0.0] the minimum length of the last edge segment (connected to the child node)
@param [options.minSlope=0.0] the minimum slope between the root node and the {@link SubtreeShape}s
@param [options.minSlopeHeight=0.0] the minimum vertical height of sloped edge segments
@throws {Exception} <b>(ArgumentError)</b> if the placement specifier or the root alignment or the routing style is unknown or if the horizontal/vertical distance
or the minimum first/last segment length or the minimum slope or the minimum vertical height is negative
@see DefaultNodePlacer#childPlacement
@see DefaultNodePlacer#rootAlignment
@see DefaultNodePlacer#routingStyle
@see DefaultNodePlacer#verticalDistance
@see DefaultNodePlacer#horizontalDistance
@see DefaultNodePlacer#minimumFirstSegmentLength
@see DefaultNodePlacer#minimumLastSegmentLength
@see DefaultNodePlacer#minimumSlope
@see DefaultNodePlacer#minimumSlopeHeight
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23DefaultNodePlacer(ChildPlacement%2CRootAlignment%2CTreeLayoutEdgeRoutingStyle%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
constructor(options: {
childPlacement: ChildPlacement | ChildPlacementStringValues;
rootAlignment: RootAlignment | RootAlignmentStringValues;
routingStyle: TreeLayoutEdgeRoutingStyle | TreeLayoutEdgeRoutingStyleStringValues;
verticalDistance?: number;
horizontalDistance?: number;
minFirstSegmentLength?: number;
minLastSegmentLength?: number;
minSlope?: number;
minSlopeHeight?: number;
});
/**
Creates a new {@link DefaultNodePlacer} instance with default settings.
@param [options] The parameters to pass.
@param [options.minimumChannelSegmentDistance] The minimum (non-negative) distance between edge segments that are routed orthogonally in the channel between the root node and the child nodes (see {@link TreeLayoutEdgeRoutingStyle#FORK}). This option sets the {@link DefaultNodePlacer#minimumChannelSegmentDistance} property on the created object.
@param [options.childPlacement] The direction in which the child nodes are placed with respect to their parent node. This option sets the {@link DefaultNodePlacer#childPlacement} property on the created object.
@param [options.rootAlignment] How the root node is aligned with its children and their {@link SubtreeShape}s. This option sets the {@link DefaultNodePlacer#rootAlignment} property on the created object.
@param [options.verticalAlignment] The relative vertical alignment of nodes with the same parent. This option sets the {@link DefaultNodePlacer#verticalAlignment} property on the created object.
@param [options.routingStyle] The style in which edge paths are routed. This option sets the {@link DefaultNodePlacer#routingStyle} property on the created object.
@param [options.verticalDistance] The vertical distance between nodes or {@link SubtreeShape}s. This option sets the {@link DefaultNodePlacer#verticalDistance} property on the created object.
@param [options.minimumFirstSegmentLength] The minimum length for the first segment of an edge. This option sets the {@link DefaultNodePlacer#minimumFirstSegmentLength} property on the created object.
@param [options.minimumLastSegmentLength] The minimum length for the last segment of an edge. This option sets the {@link DefaultNodePlacer#minimumLastSegmentLength} property on the created object.
@param [options.minimumSlope] The minimum slope of the edge segments. This option sets the {@link DefaultNodePlacer#minimumSlope} property on the created object.
@param [options.minimumSlopeHeight] The minimum height for sloped edge segments. This option sets the {@link DefaultNodePlacer#minimumSlopeHeight} property on the created object.
@param [options.horizontalDistance] The horizontal distance between nodes or {@link SubtreeShape}s. This option sets the {@link DefaultNodePlacer#horizontalDistance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23DefaultNodePlacer() Online Documentation}*/
constructor(options?: {
minimumChannelSegmentDistance?: number;
childPlacement?: ChildPlacement | ChildPlacementStringValues;
rootAlignment?: RootAlignment | RootAlignmentStringValues;
verticalAlignment?: number;
routingStyle?: TreeLayoutEdgeRoutingStyle | TreeLayoutEdgeRoutingStyleStringValues;
verticalDistance?: number;
minimumFirstSegmentLength?: number;
minimumLastSegmentLength?: number;
minimumSlope?: number;
minimumSlopeHeight?: number;
horizontalDistance?: number;
});
/**
Calculates the bends for the connector to the parent node.
This implementation adds bends to the connector for edges that route the edges around the child node in case there are
{@link PortConstraint}s that conflict with the direction of the {@link DefaultNodePlacer#childPlacement child placement}.
This method is called in {@link DefaultNodePlacer#placeSubtreeOfNode} to route the edges after the
{@link SubtreeShape}s are arranged. It may be overridden to customize the routing style.
@param graph the input graph
@param localRoot the local root node whose connector is calculated and stored in the given subtree shape
@param rootLayout the {@link INodeLayout} of the root node
@param subtreeShape the shape of the whole subtree of the local root
@param parentEdge the incoming edge of the local root node that will be the connector for the given subtree shape
@param parentEdgeLayout the current {@link IEdgeLayout} of the connector edge
@param direction the direction specifier as defined in the {@link ITreeLayoutNodePlacer} interface which should be used for the connector
@see NodePlacerBase#determineChildConnectors
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23calculateParentConnector(LayoutGraph%2CYNode%2CINodeLayout%2CSubtreeShape%2CEdge%2CIEdgeLayout%2Cnumber) Online Documentation}*/
calculateParentConnector(graph: LayoutGraph | null, localRoot: YNode | null, rootLayout: INodeLayout | null, subtreeShape: SubtreeShape | null, parentEdge: Edge | null, parentEdgeLayout: IEdgeLayout | null, direction: number): void;
/**
Calculates the routing of the source side of the edge to the given child node.
This method delegates to {@link DefaultNodePlacer#calculateSourceEdgeLayout} if the {@link DefaultNodePlacer#routingStyle routing style} is not
{@link TreeLayoutEdgeRoutingStyle#FORK}. It may be overridden to implement a custom routing style.
@param rootLayout the {@link INodeLayout} of the local root node
@param childShape the {@link SubtreeShape} instance of the corresponding child's subtree
@param edge the edge
@param rootForkCoordinate the y-coordinate of the first bend of the edge if the placement is horizontal, the x-coordinate otherwise
@param childForkCoordinate the y-coordinate of the second (= last) bend of the edge if the placement is horizontal, the x-coordinate otherwise
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23calculateSlopedSourceEdgeLayout(INodeLayout%2CSubtreeShape%2CEdge%2Cnumber%2Cnumber) Online Documentation}*/
calculateSlopedSourceEdgeLayout(rootLayout: INodeLayout | null, childShape: SubtreeShape | null, edge: Edge | null, childForkCoordinate: number, rootForkCoordinate: number): void;
/**
Calculates the routing of the source side of the edge to the given child node.
This implementation adds bends to the {@link IEdgeLayout} of the given edge according to the specified {@link DefaultNodePlacer#routingStyle routing style}
and {@link DefaultNodePlacer#childPlacement child placement}. It also updates the target points in the {@link SubtreeShape}.
This method is called by {@link DefaultNodePlacer#calculateSlopedSourceEdgeLayout} to route the edges. It may be
overridden to implement a custom edge routing style.
@param rootLayout the {@link INodeLayout} of the local root node
@param childShape the {@link SubtreeShape} of the child's subtree that is connected through the given edge
@param edge the edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23calculateSourceEdgeLayout(INodeLayout%2CSubtreeShape%2CEdge) Online Documentation}*/
calculateSourceEdgeLayout(rootLayout: INodeLayout | null, childShape: SubtreeShape | null, edge: Edge | null): void;
/**
Calculates the routing of the target side of the edge to the given child node.
This implementation just resets the current {@link IEdgeLayout}.
This method is called by {@link DefaultNodePlacer#placeSubtreeOfNode} after the {@link SubtreeShape shapes} of the local root's children
are placed. It may be overridden to add some {@link SubtreeShape#addTargetPoint target points} to the edge.
@param rootLayout the {@link INodeLayout} of the local root node
@param childShape the {@link SubtreeShape} of the child's subtree that is connected through the given edge
@param edge the edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23calculateTargetEdgeLayout(INodeLayout%2CSubtreeShape%2CEdge) Online Documentation}*/
calculateTargetEdgeLayout(rootLayout: INodeLayout | null, childShape: SubtreeShape | null, edge: Edge | null): void;
/**
Creates a {@link IComparer} for outgoing edges which takes the initial coordinates of the edges' targets into account.
Child nodes are sorted in a way that the layout will resemble the current arrangement of the nodes. This is especially
useful for interactive or incremental layout calculation.
@returns a {@link IComparer} that considers the initial coordinates of the nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23createComparer() Online Documentation}*/
createComparer(): IComparer<any> | null;
/**
Delegates to {@link DefaultNodePlacer#createComparer}.
@returns the From Sketch {@link IComparer}
@see DefaultNodePlacer#createComparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23createFromSketchComparer() Online Documentation}*/
createFromSketchComparer(): IComparer<any> | null;
/**
Determines a connector direction for the given child node depending on the selected {@link DefaultNodePlacer#childPlacement child placement}.
If the child nodes are placed {@link ChildPlacement#HORIZONTAL_DOWNWARD below} the root node, the edge should connect {@link ParentConnectorDirection#NORTH north}. If they are placed {@link ChildPlacement#HORIZONTAL_UPWARD above} the root node
it should connect {@link ParentConnectorDirection#SOUTH south}. The vertical placements are handled analogous.
@param child the child node
@returns the connector direction depending on the {@link DefaultNodePlacer#childPlacement child placement}
@see DefaultNodePlacer#childPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Arranges the root node and its children.
@param localRoot the local root node
@param parentConnectorDirection the direction of the connector
@returns the merged {@link SubtreeShape} containing the local root and all its subtrees
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): SubtreeShape | null;
/**
Gets or sets the minimum (non-negative) distance between edge segments that are routed orthogonally in the channel
between the root node and the child nodes (see {@link TreeLayoutEdgeRoutingStyle#FORK}).
If this distance is set to <code>0</code>, all segments are routed in a bus-like style.
<h2>Note</h2>
The specified distance is only considered if {@link DefaultNodePlacer#minimumSlope} is equal to <code>0</code>, {@link DefaultNodePlacer#minimumSlopeHeight} is equal to <code>0</code>, {@link DefaultNodePlacer#routingStyle} is {@link TreeLayoutEdgeRoutingStyle#FORK} and {@link DefaultNodePlacer#rootAlignment} is
not equal to {@link RootAlignment#LEADING_ON_BUS} or {@link RootAlignment#TRAILING_ON_BUS}.
<h2>Note</h2>
Values greater than <code>0</code> are especially useful if the {@link ITreeLayoutPortAssignment} distributes the ports at the nodes'
border.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is smaller than <code>0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23minimumChannelSegmentDistance Online Documentation}*/
minimumChannelSegmentDistance: number;
/**
Gets or sets the direction in which the child nodes are placed with respect to their parent node.
Subtrees can be placed below, above, left or right of their roots.
@throws {Exception} <b>(ArgumentError)</b> if an unknown placement specifier is set
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23childPlacement Online Documentation}*/
childPlacement: ChildPlacement;
/**
Gets or sets how the root node is aligned with its children and their {@link SubtreeShape}s.
@throws {Exception} <b>(ArgumentError)</b> if an unknown root alignment specifier is set
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23rootAlignment Online Documentation}*/
rootAlignment: RootAlignment;
/**
Gets or sets the relative vertical alignment of nodes with the same parent.
All values between <code>0</code> and <code>1</code> result in a valid alignment.
<ul>
<li><code>0</code> - nodes are top-aligned
<li><code>0.5</code> - nodes are center-aligned
<li><code>1</code> - nodes are bottom-aligned
</ul>
Values outside the interval <code>[0,1]</code> will result in a more compact node placement with unaligned nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23verticalAlignment Online Documentation}*/
verticalAlignment: number;
/**
Gets or sets the style in which edge paths are routed.
@throws {Exception} <b>(ArgumentError)</b> if the specified routing style is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23routingStyle Online Documentation}*/
routingStyle: TreeLayoutEdgeRoutingStyle;
/**
Gets or sets the vertical distance between nodes or {@link SubtreeShape}s.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23verticalDistance Online Documentation}*/
verticalDistance: number;
/**
Gets or sets the minimum length for the first segment of an edge.
During layout calculation, the edges of the tree are directed. Hence, the first segment is always the segment that is
attached to the source, which is the local root node.
The length needs to be non-negative.
<h2>Note</h2>
The specified length is only considered if the {@link DefaultNodePlacer#routingStyle routing style} is set to {@link TreeLayoutEdgeRoutingStyle#FORK}.
@throws {Exception} <b>(ArgumentError)</b> if the minimum segment length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23minimumFirstSegmentLength Online Documentation}*/
minimumFirstSegmentLength: number;
/**
Gets or sets the minimum length for the last segment of an edge.
During layout calculation, the edges of the tree are directed. Hence, the last segment is always the segment that is
attached to the target, which is a child node.
The length needs to be non-negative.
<h2>Note</h2>
The specified length is only considered if the {@link DefaultNodePlacer#routingStyle routing style} is set to {@link TreeLayoutEdgeRoutingStyle#FORK}.
@throws {Exception} <b>(ArgumentError)</b> if the minimum segment length is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23minimumLastSegmentLength Online Documentation}*/
minimumLastSegmentLength: number;
/**
Gets or sets the minimum slope of the edge segments.
These sloped segments are between the vertical first and last segment of the edge.
The segments may become steeper but they will have at least the specified slope. Negative minimum slopes can lead to
edge routes looking like spider legs.
<h2>Note</h2>
The actual slopes also depend on the {@link DefaultNodePlacer#minimumSlopeHeight vertical distance} slope segments can have.
@see DefaultNodePlacer#minimumSlopeHeight
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23minimumSlope Online Documentation}*/
minimumSlope: number;
/**
Gets or sets the minimum height for sloped edge segments.
These sloped segments are between the vertical first and last segment of the edge.
The segments may become steeper to reach the specified height.
<h2>Note</h2>
The actual slopes also depend on the {@link DefaultNodePlacer#minimumSlope} minimum slope} segments can have.
@see DefaultNodePlacer#minimumSlope
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23minimumSlopeHeight Online Documentation}*/
minimumSlopeHeight: number;
/**
Gets or sets the horizontal distance between nodes or {@link SubtreeShape}s.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultNodePlacer%23horizontalDistance Online Documentation}*/
horizontalDistance: number;
static $class: Class;
private $_DefaultNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultNodePlacer;
}
export interface DefaultTreeLayoutPortAssignment extends YObject, ITreeLayoutPortAssignment {
}
/**
The {@link DefaultTreeLayoutPortAssignment} provides some simple {@link DefaultTreeLayoutPortAssignment#mode port assignment styles}.
Ports are either {@link TreeLayoutPortAssignmentMode#NONE centered} on the node, distributed at one side of the node or placed considering
{@link PortConstraint}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment Online Documentation}*/
export class DefaultTreeLayoutPortAssignment {
/**
Creates a new {@link DefaultTreeLayoutPortAssignment} instance using the given port assignment mode and ratio between the
border and the ports and the gap between the ports themselves.
If no port assignment mode is given, the port assignment mode that resets all ports to the center of their nodes will be
used instead {@link TreeLayoutPortAssignmentMode#NONE}.
@param [mode=TreeLayoutPortAssignmentMode.NONE] one of the predefined port assignment specifiers
@param [ratio=0.5] the ratio of the distance between outer ports and the border to the distance between any two ports
@throws {Exception} <b>(ArgumentError)</b> if an unknown mode is specified or if the specified ratio is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment Online Documentation}*/
constructor(mode?: TreeLayoutPortAssignmentMode | TreeLayoutPortAssignmentModeStringValues, ratio?: number);
/**
Creates a new {@link DefaultTreeLayoutPortAssignment} instance using the given port assignment mode and ratio between the
border and the ports and the gap between the ports themselves.
If no port assignment mode is given, the port assignment mode that resets all ports to the center of their nodes will be
used instead {@link TreeLayoutPortAssignmentMode#NONE}.
@param options The parameters to pass.
@param [options.mode=TreeLayoutPortAssignmentMode.NONE] one of the predefined port assignment specifiers
@param [options.ratio=0.5] the ratio of the distance between outer ports and the border to the distance between any two ports
@param [options.borderGapToPortGapRatio] The ratio of the <em>border gap</em> (distance between the outer ports to the node border) to the <em>port gap</em> (distance between adjacent ports). This option sets the {@link DefaultTreeLayoutPortAssignment#borderGapToPortGapRatio} property on the created object.
@param [options.reversedPortOrder] Whether or not the port assignment should reverse the order of the ports. This option sets the {@link DefaultTreeLayoutPortAssignment#reversedPortOrder} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if an unknown mode is specified or if the specified ratio is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment Online Documentation}*/
constructor(options: {
mode?: TreeLayoutPortAssignmentMode | TreeLayoutPortAssignmentModeStringValues;
ratio?: number;
borderGapToPortGapRatio?: number;
reversedPortOrder?: boolean;
});
/**
Assigns relative coordinates to the source port of the outgoing edge to a child of the given node.
This method is called by {@link DefaultTreeLayoutPortAssignment#assignPorts} for each outgoing edge of the given node. It may
be overridden to assign a custom location to the source port of outgoing edges.
<h2>Note</h2>
This implementation considers {@link PortConstraint}s despite the {@link DefaultTreeLayoutPortAssignment#mode port assignment mode}.
@param graph the input graph
@param node the node
@param edge the edge to the child node
@param index the zero-based index of the child edge
@see IEdgeLayout#sourcePoint
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23assignChildEdgeSourcePort(LayoutGraph%2CYNode%2CEdge%2Cnumber) Online Documentation}*/
assignChildEdgeSourcePort(graph: LayoutGraph | null, node: YNode | null, edge: Edge | null, index: number): void;
/**
Assigns relative coordinates to the target port of the incoming edge from the parent of the given node.
This method is called by {@link DefaultTreeLayoutPortAssignment#assignPorts} if the given node has a parent node. It may be
overridden to assign a custom location to the target port of the incoming edge.
<h2>Note</h2>
This implementation considers {@link PortConstraint}s despite the {@link DefaultTreeLayoutPortAssignment#mode port assignment mode}.
@param graph the input graph
@param node the node
@param edge the edge from the parent node
@see IEdgeLayout#targetPoint
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23assignParentEdgeTargetPort(LayoutGraph%2CYNode%2CEdge) Online Documentation}*/
assignParentEdgeTargetPort(graph: LayoutGraph | null, node: YNode | null, edge: Edge | null): void;
/**
Places the ports of edges connecting to the given node according to the {@link DefaultTreeLayoutPortAssignment#mode port assignment mode}.
@param graph the input graph
@param node the node whose adjacent edges' ports should be placed
@see DefaultTreeLayoutPortAssignment#mode
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23assignPorts(LayoutGraph%2CYNode) Online Documentation}*/
assignPorts(graph: LayoutGraph | null, node: YNode | null): void;
/**
Determines the distance between the outer ports and the border of the node.
This implementation calculates the gap according to the number of edges at one side of the node. It also includes the
{@link DefaultTreeLayoutPortAssignment#borderGapToPortGapRatio ratio between border gap and port gap}.
This method is called by {@link DefaultTreeLayoutPortAssignment#assignChildEdgeSourcePort} to calculate the location of the
first port. It may be overridden to implement a different distribution of ports on the node's side.
@param sideLength the width/height of the side
@param edgeCount the number of edges that connect to this side
@returns the absolute gap to be used on both sides of the ports
@see DefaultTreeLayoutPortAssignment#assignChildEdgeSourcePort
@see DefaultTreeLayoutPortAssignment#borderGapToPortGapRatio
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23getPortBorderGap(number%2Cnumber) Online Documentation}*/
getPortBorderGap(sideLength: number, edgeCount: number): number;
/**
Determines the distance between two adjacent ports.
This implementation calculates the gap according to the number of edges at one side of the node. The edges are
distributed keeping the given gap to the border of the node.
This method is called by {@link DefaultTreeLayoutPortAssignment#assignChildEdgeSourcePort} to calculate the distance between
two ports. It may be overridden to implement a different distribution of ports on the node's side.
@param sideLength the width/height of the side
@param edgeCount the number of edges/ports that connect to this side
@param portBorderGap the port border gap (i.e. calculated by {@link DefaultTreeLayoutPortAssignment#getPortBorderGap})
@returns the absolute distance to be used between two adjacent ports
@see DefaultTreeLayoutPortAssignment#assignChildEdgeSourcePort
@see DefaultTreeLayoutPortAssignment#getPortBorderGap
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23getPortDistanceDelta(number%2Cnumber%2Cnumber) Online Documentation}*/
getPortDistanceDelta(sideLength: number, edgeCount: number, portBorderGap: number): number;
/**
Retrieves the source {@link PortConstraint} for the given edge.
This implementation accesses the {@link IDataProvider} registered with
{@link PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}.
This method is called by {@link DefaultTreeLayoutPortAssignment#assignChildEdgeSourcePort} in
{@link TreeLayoutPortAssignmentMode#PORT_CONSTRAINT}.
@param graph the input graph
@param edge the edge
@param index the index of the child that is the target of the given edge
@returns the source {@link PortConstraint} or <code>null</code> if no source {@link PortConstraint} is specified
@see PortConstraint
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see DefaultTreeLayoutPortAssignment#assignChildEdgeSourcePort
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23getSourcePortConstraint(LayoutGraph%2CEdge%2Cnumber) Online Documentation}*/
getSourcePortConstraint(graph: LayoutGraph | null, edge: Edge | null, index: number): PortConstraint | null;
/**
Retrieves the target {@link PortConstraint} for the given edge.
This implementation accesses the {@link IDataProvider} registered with
{@link PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}.
This method is called by {@link DefaultTreeLayoutPortAssignment#assignParentEdgeTargetPort} in
{@link TreeLayoutPortAssignmentMode#PORT_CONSTRAINT}.
@param graph the input graph
@param edge the edge
@returns the target {@link PortConstraint} or <code>null</code> if no target {@link PortConstraint} is specified
@see PortConstraint
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see DefaultTreeLayoutPortAssignment#assignParentEdgeTargetPort
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23getTargetPortConstraint(LayoutGraph%2CEdge) Online Documentation}*/
getTargetPortConstraint(graph: LayoutGraph | null, edge: Edge | null): PortConstraint | null;
/**
Gets or sets the mode that describes how ports are placed.
@throws {Exception} <b>(ArgumentError)</b> if an unknown mode is specified
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23mode Online Documentation}*/
mode: TreeLayoutPortAssignmentMode;
/**
Gets or sets the ratio of the <em>border gap</em> (distance between the outer ports to the node border) to the <em>port gap</em> (distance between adjacent
ports).
The ratio must have a positive value. When the ratio is <code>0</code> the ports will be distributed along the whole side of the node
without a gap between the last port and the corner.
<b>Warning:</b> This setting only affects distributed port assignments (e.g. {@link TreeLayoutPortAssignmentMode#DISTRIBUTED_NORTH}).
@throws {Exception} <b>(ArgumentError)</b> if the specified ratio is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23borderGapToPortGapRatio Online Documentation}*/
borderGapToPortGapRatio: number;
/**
Gets or sets whether or not the port assignment should reverse the order of the ports.
The normal order is from left to right and from top to bottom. It will be reversed if this option is enabled.
<b>Warning:</b> This setting only affects distributed port assignment (e.g. {@link TreeLayoutPortAssignmentMode#DISTRIBUTED_NORTH}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/DefaultTreeLayoutPortAssignment%23reversedPortOrder Online Documentation}*/
reversedPortOrder: boolean;
static $class: Class;
private $_DefaultPortAssignment: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DefaultTreeLayoutPortAssignment;
}
export interface DelegatingNodePlacer extends RotatableNodePlacerBase {
}
/**
{@link DelegatingNodePlacer} delegates the arrangement of the children to two different {@link ITreeLayoutNodePlacer}s.
<b>Layout Style</b>
This {@link ITreeLayoutNodePlacer} can, for example, be used for creating mind maps or a playing schedule of a sports
competition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer Online Documentation}*/
export class DelegatingNodePlacer {
/**
Creates a new {@link DefaultNodePlacer} instance with default settings using the given modification matrix and the
given node placers as delegates.
@param modificationMatrix the transformation matrix
@param placerUpperLeft the first delegate {@link ITreeLayoutNodePlacer} which places the left subtrees
@param placerLowerRight the second delegate {@link ITreeLayoutNodePlacer} which places the right subtrees
@see DelegatingNodePlacer#PRIMARY_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer Online Documentation}*/
constructor(modificationMatrix: RotatableNodePlacerMatrix | null, placerUpperLeft: ITreeLayoutNodePlacer | null, placerLowerRight: ITreeLayoutNodePlacer | null);
/**
Creates a new {@link DefaultNodePlacer} instance with default settings using the given modification matrix and the
given node placers as delegates.
@param options The parameters to pass.
@param options.modificationMatrix the transformation matrix
@param options.placerUpperLeft the first delegate {@link ITreeLayoutNodePlacer} which places the left subtrees
@param options.placerLowerRight the second delegate {@link ITreeLayoutNodePlacer} which places the right subtrees
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see DelegatingNodePlacer#PRIMARY_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer Online Documentation}*/
constructor(options: {
modificationMatrix: RotatableNodePlacerMatrix | null;
placerUpperLeft: ITreeLayoutNodePlacer | null;
placerLowerRight: ITreeLayoutNodePlacer | null;
spacing?: number;
});
/**
Creates a {@link IProcessor} that partitions child nodes of the current root node into two subsets.
In a pre-processing step the child nodes are divided into an upper-left and a lower-right subset. The first set is
arranged using {@link DelegatingNodePlacer#primaryPlacer the first delegate}, the second set is arranged using {@link DelegatingNodePlacer#secondaryPlacer the second delegate}. To be able to delegate to
different {@link ITreeLayoutNodePlacer}s this {@link IProcessor} adds dummy nodes to which it assigns the according
{@link ITreeLayoutNodePlacer}s.
The changes to the graph are restored in the post-processing step.
@param layouter
@param graph
@param currentRoot
@see DelegatingNodePlacer#secondaryPlacer
@see DelegatingNodePlacer#primaryPlacer
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Determines a connector direction for the given child node.
<b>Warning:</b> This method should not be called! Since the direction of the connectors is not used by
{@link DelegatingNodePlacer#determineChildConnectors}, it does not call this method.
@param child
@throws {Exception} <b>(InvalidOperationError)</b> if this method is called
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Does nothing.
Since the {@link DelegatingNodePlacer} always places one part of the tree on one side and the remaining tree on the
opposite side, it doesn't need to calculate the directions of the connectors.
@param localRoot the local root node
@param connectorMap the map that must be used for storing the direction specifiers of the child nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Places the {@link SubtreeShape}s of the child nodes using the two delegate node placers retrieved with {@link DelegatingNodePlacer#primaryPlacer} and {@link DelegatingNodePlacer#secondaryPlacer}.
@param nodeShapeProvider
@param subtreeShapeProvider
@param graph
@param localRoot
@param parentConnectorDirection
@returns the merged {@link SubtreeShape} for the subtree rooted at the local root node
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23placeSubtree(IDataProvider%2CIDataProvider%2CLayoutGraph%2CYNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtree(nodeShapeProvider: IDataProvider | null, subtreeShapeProvider: IDataProvider | null, graph: LayoutGraph | null, localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection): SubtreeShape | null;
/**
Arranges the local root relative to the subtree shape of its children.
<b>Warning:</b> This method should not be called! Placing subtrees is delegated to the {@link DelegatingNodePlacer#primaryPlacer upper-left} and the {@link DelegatingNodePlacer#secondaryPlacer lower-right node placer}
{@link DelegatingNodePlacer#placeSubtree}.
@param localRoot
@param parentConnectorDirection
@throws {Exception} <b>(InvalidOperationError)</b> if this method is called
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
/**
Returns all upper-left children of the local root node.
Which children belong to this list is determined using the {@link IDataProvider} registered with
{@link DelegatingNodePlacer#PRIMARY_NODES_DP_KEY}. If no such {@link IDataProvider} is registered, the
first half of the children is returned.
These children are placed using the {@link ITreeLayoutNodePlacer} returned by {@link DelegatingNodePlacer#primaryPlacer}.
@param localRoot the local root node
@param graph the input graph
@returns the list of upper-left child {@link YNode}s
@see DelegatingNodePlacer#PRIMARY_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23primaryChildren(YNode%2CLayoutGraph) Online Documentation}*/
primaryChildren(localRoot: YNode | null, graph: LayoutGraph | null): YNodeList | null;
/**
Returns all lower-right children of the local root node.
Which children belong to this list is determined using the {@link IDataProvider} registered with
{@link DelegatingNodePlacer#PRIMARY_NODES_DP_KEY}. If no such {@link IDataProvider} is registered, the
second half of the children is returned.
These children are placed using the {@link ITreeLayoutNodePlacer} returned by {@link DelegatingNodePlacer#secondaryPlacer}.
@param localRoot the local root node
@param graph the input graph
@returns the list of lower-right child {@link YNode}s
@see DelegatingNodePlacer#PRIMARY_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23secondaryChildren(YNode%2CLayoutGraph) Online Documentation}*/
secondaryChildren(localRoot: YNode | null, graph: LayoutGraph | null): YNodeList | null;
/**
Data provider key for dividing the children into two sets
If there is no {@link IDataProvider} registered with this key, half of the nodes are placed left of the root
and the remaining nodes are placed right of the root.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23PRIMARY_NODES_DP_KEY Online Documentation}*/
static PRIMARY_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets the {@link ITreeLayoutNodePlacer} instance that will arrange the lower-right part of the graph.
@see DelegatingNodePlacer#primaryPlacer
@see DelegatingNodePlacer#secondaryChildren
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23secondaryPlacer Online Documentation}*/
readonly secondaryPlacer: ITreeLayoutNodePlacer | null;
/**
Gets the {@link ITreeLayoutNodePlacer} instance that will arrange the upper-left part of the graph.
@see DelegatingNodePlacer#secondaryPlacer
@see DelegatingNodePlacer#primaryChildren
@see {@link https://docs.yworks.com/yfileshtml/#/api/DelegatingNodePlacer%23primaryPlacer Online Documentation}*/
readonly primaryPlacer: ITreeLayoutNodePlacer | null;
static $class: Class;
private $_DelegatingNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DelegatingNodePlacer;
}
export interface DendrogramNodePlacer extends YObject, IFromSketchNodePlacer {
}
/**
The {@link DendrogramNodePlacer} arranges subtrees as dendrograms.
<b>Layout Style</b>
This {@link ITreeLayoutNodePlacer} arranges nodes from top to bottom. It arranges subtrees in such a way that all subtrees of
a single local root are aligned with their bottom border.
Note: This implementation does not support {@link PortConstraint}s.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNodePlacer Online Documentation}*/
export class DendrogramNodePlacer {
/**
Creates a new {@link DendrogramNodePlacer} instance with default settings.
@param [options] The parameters to pass.
@param [options.minimumRootDistance] The minimum distance between the {@link SubtreeShape} of a root node and the bus connecting to all of its children. This option sets the {@link DendrogramNodePlacer#minimumRootDistance} property on the created object.
@param [options.minimumSubtreeDistance] The minimum horizontal distance between adjacent {@link SubtreeShape SubtreeShapes}. This option sets the {@link DendrogramNodePlacer#minimumSubtreeDistance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNodePlacer Online Documentation}*/
constructor(options?: {
minimumRootDistance?: number;
minimumSubtreeDistance?: number;
});
/**
Creates a {@link IComparer} that sorts outgoing edges according to the x-coordinate of their target nodes.
This {@link IComparer} can be used for achieving a From Sketch layout.
@returns the {@link IComparer}
@see DendrogramNodePlacer#createFromSketchComparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNodePlacer%23createComparer() Online Documentation}*/
createComparer(): IComparer<any> | null;
/**
Creates a {@link IComparer} for the edges which takes the initial x-coordinates of the nodes into account.
This implementation delegates to {@link DendrogramNodePlacer#createComparer}.
@returns the {@link IComparer} for the edges
@see DendrogramNodePlacer#createComparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNodePlacer%23createFromSketchComparer() Online Documentation}*/
createFromSketchComparer(): IComparer<any> | null;
/**
Creates an optional {@link IProcessor} for pre- and post-processing.
This implementation returns <code>null</code>. Hence, no pre- or post-processing is done.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the root node handled by this {@link ITreeLayoutNodePlacer}
@returns <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Provides {@link ParentConnectorDirection#NORTH} for all children.
Edges will always connect to the top of the child nodes.
@param localRoot the local root node
@param connectorMap the map that is used for storing the direction specifiers of the child nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNodePlacer%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Arranges the {@link SubtreeShape SubtreeShapes} of the local root and its children as a dendrogram.
@param nodeShapeProvider the {@link IDataProvider} for obtaining an initial shape of the root node
@param subtreeShapeProvider the {@link IDataProvider} for accessing the pre-calculated shapes of the subtrees
@param graph the input graph
@param localRoot the root of the subtree that should be arranged by this method
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the combined shape of the whole subtree rooted at the given local root node
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNodePlacer%23placeSubtree(IDataProvider%2CIDataProvider%2CLayoutGraph%2CYNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtree(nodeShapeProvider: IDataProvider | null, subtreeShapeProvider: IDataProvider | null, graph: LayoutGraph | null, localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection): SubtreeShape | null;
/**
Gets or sets the minimum distance between the {@link SubtreeShape} of a root node and the bus connecting to all of
its children.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNodePlacer%23minimumRootDistance Online Documentation}*/
minimumRootDistance: number;
/**
Gets or sets the minimum horizontal distance between adjacent {@link SubtreeShape SubtreeShapes}.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNodePlacer%23minimumSubtreeDistance Online Documentation}*/
minimumSubtreeDistance: number;
static $class: Class;
private $_DendrogramNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DendrogramNodePlacer;
}
export interface DoubleLineNodePlacer extends RotatableNodePlacerBase {
}
/**
The {@link DoubleLineNodePlacer} arranges the children of a local root alternating in two lines.
<b>Layout Style</b>
This {@link ITreeLayoutNodePlacer} arranges nodes from top to bottom. It places the subtrees in two horizontal lines below the
root. They are distributed in an alternating fashion, i.e., adjacent subtrees are in different lines.
This node placer is especially useful for leaves with a large width.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleLineNodePlacer Online Documentation}*/
export class DoubleLineNodePlacer {
/**
Creates a new {@link DoubleLineNodePlacer} instance whose direction is specified by the given modification matrix.
@param modificationMatrix the transformation matrix for this {@link RotatableNodePlacerBase}
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleLineNodePlacer%23DoubleLineNodePlacer(RotatableNodePlacerMatrix) Online Documentation}*/
constructor(modificationMatrix: RotatableNodePlacerMatrix | null);
/**
Creates a new {@link DoubleLineNodePlacer} instance whose direction is specified by the given modification matrix.
@param options The parameters to pass.
@param options.modificationMatrix the transformation matrix for this {@link RotatableNodePlacerBase}
@param [options.rootAlignment] The {@link RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes. This option sets the {@link DoubleLineNodePlacer#rootAlignment} property on the created object.
@param [options.doubleLineSpacingRatio] The ratio of the {@link RotatableNodePlacerBase#spacing spacing} between the root and the children in the upper line and the spacing between the two lines of subtrees. This option sets the {@link DoubleLineNodePlacer#doubleLineSpacingRatio} property on the created object.
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleLineNodePlacer%23DoubleLineNodePlacer(RotatableNodePlacerMatrix) Online Documentation}*/
constructor(options: {
modificationMatrix: RotatableNodePlacerMatrix | null;
rootAlignment?: RootNodeAlignment | null;
doubleLineSpacingRatio?: number;
spacing?: number;
});
/**
Creates a new {@link DoubleLineNodePlacer} instance with the default settings.
@param [options] The parameters to pass.
@param [options.rootAlignment] The {@link RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes. This option sets the {@link DoubleLineNodePlacer#rootAlignment} property on the created object.
@param [options.doubleLineSpacingRatio] The ratio of the {@link RotatableNodePlacerBase#spacing spacing} between the root and the children in the upper line and the spacing between the two lines of subtrees. This option sets the {@link DoubleLineNodePlacer#doubleLineSpacingRatio} property on the created object.
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleLineNodePlacer%23DoubleLineNodePlacer() Online Documentation}*/
constructor(options?: {
rootAlignment?: RootNodeAlignment | null;
doubleLineSpacingRatio?: number;
spacing?: number;
});
/**
Returns {@link ParentConnectorDirection#NORTH} for every given child node.
Edges always connect to the top of the child nodes.
@param child
@returns {@link ParentConnectorDirection#NORTH}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleLineNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Places the children of the given local root alternating in two horizontal lines.
@param localRoot the local root node
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged subtree shape that contains the local root node and all its children
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleLineNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
/**
Gets or sets the {@link RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleLineNodePlacer%23rootAlignment Online Documentation}*/
rootAlignment: RootNodeAlignment | null;
/**
Gets or sets the ratio of the {@link RotatableNodePlacerBase#spacing spacing} between the root and the children in the upper line and the spacing between the
two lines of subtrees.
<h2>Note</h2>
Negative ratios will lead to node overlaps.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DoubleLineNodePlacer%23doubleLineSpacingRatio Online Documentation}*/
doubleLineSpacingRatio: number;
static $class: Class;
private $_DoubleLineNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DoubleLineNodePlacer;
}
export interface FreeNodePlacer extends RotatableNodePlacerBase {
}
/**
A {@link FreeNodePlacer} is a simple <em>pseudo</em>-placer.
The children of local root nodes are not moved. The {@link ITreeLayoutNodePlacer} just merges their rotated shapes and routes
the edges.
This {@link ITreeLayoutNodePlacer} may be useful if a local root node does not have any children or you want to place them
manually.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePlacer Online Documentation}*/
export class FreeNodePlacer {
/**
Creates a new instance of {@link FreeNodePlacer} with default settings.
@param [options] The parameters to pass.
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePlacer Online Documentation}*/
constructor(options?: {
spacing?: number;
});
/**
Creates a {@link IProcessor} that will change all descending {@link ITreeLayoutNodePlacer}s to
{@link FreeNodePlacer}.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the current root node handled by this {@link ITreeLayoutNodePlacer}
@returns the {@link IProcessor} suitable for {@link FreeNodePlacer}
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Determines a connector direction for the given child node.
This implementation returns {@link ParentConnectorDirection#NONE} for every node.
@param child the child node
@returns {@link ParentConnectorDirection#NONE}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Merges the rotated shapes of the given local root and its children and routes the edges between them.
@param localRoot the local root node
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged subtree shape that contains the local root node and all its children
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/FreeNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
static $class: Class;
private $_FreeNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FreeNodePlacer;
}
export interface GridNodePlacer extends RotatableNodePlacerBase {
}
/**
The {@link GridNodePlacer} arranges the shapes of the children of a local root in a grid.
<b>Layout Style</b>
The grid consists of a number of rows. The subtrees are assigned to the rows using a {@link IDataProvider}
that is registered with the graph with {@link GridNodePlacer#ROW_INDEX_DP_KEY}. If there is no row mapping provided,
all subtrees are assigned to a single row. Furthermore, if property {@link GridNodePlacer#automaticRowAssignment} is enabled, the row assignment is made
automatically - the produced grid will be square-like with respect to the child count, ignoring actual subtree sizes.
For example, if there are nine children, a grid with three rows is generated.
The edges from the children to the local root are routed in a bus-like way. The placement of the bus can be configured
using the property {@link GridNodePlacer#busPlacement}. The alignment of the root node can be configured using {@link GridNodePlacer#rootAlignment}. To align the root node with the bus, use
{@link GridNodePlacer#BUS_ALIGNED}. A bus-aligned root in conjunction with {@link BusPlacement#CENTER} produces
a symmetric grid with the bus and root aligned in the middle.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer Online Documentation}*/
export class GridNodePlacer {
/**
Creates a new {@link GridNodePlacer} instance with the given {@link RotatableNodePlacerBase#modificationMatrix modification matrix} and {@link GridNodePlacer#rootAlignment root alignment}.
@param [modificationMatrix=null] the transformation matrix for this {@link RotatableNodePlacerBase}
@param [rootAlignment=null] the alignment of the local root with its subtrees
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23GridNodePlacer(RotatableNodePlacerMatrix%2CRootNodeAlignment) Online Documentation}*/
constructor(modificationMatrix?: RotatableNodePlacerMatrix | null, rootAlignment?: RootNodeAlignment | null);
/**
Creates a new {@link GridNodePlacer} instance with the given {@link RotatableNodePlacerBase#modificationMatrix modification matrix} and {@link GridNodePlacer#rootAlignment root alignment}.
@param options The parameters to pass.
@param [options.modificationMatrix=null] the transformation matrix for this {@link RotatableNodePlacerBase}
@param [options.rootAlignment=null] the alignment of the local root with its subtrees
@param [options.busPlacement] The placement of the bus with respect to the child node grid. This option sets the {@link GridNodePlacer#busPlacement} property on the created object.
@param [options.automaticRowAssignment] Whether or not the nodes are automatically assigned to the rows of the grid. This option sets the {@link GridNodePlacer#automaticRowAssignment} property on the created object.
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23GridNodePlacer(RotatableNodePlacerMatrix%2CRootNodeAlignment) Online Documentation}*/
constructor(options: {
modificationMatrix?: RotatableNodePlacerMatrix | null;
rootAlignment?: RootNodeAlignment | null;
busPlacement?: BusPlacement | BusPlacementStringValues;
automaticRowAssignment?: boolean;
spacing?: number;
});
/**
Creates a {@link IComparer} that sorts the outgoing edges according to the rows and x-coordinates of their target nodes.
Edges that connect to nodes in different rows are ordered by the row index. If two edges connect to the same row, they
are sorted by the x-coordinates of their target nodes.
<b>Warning:</b> If the rows are {@link GridNodePlacer#automaticRowAssignment automatically assigned}, the created from-sketch {@link IComparer} does not have an effect.
@returns a {@link IComparer} that orders edges lexicographically by row and x-coordinates of their target nodes
@see GridNodePlacer#ROW_INDEX_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23createFromSketchComparer() Online Documentation}*/
createFromSketchComparer(): IComparer<any> | null;
/**
Returns {@link ParentConnectorDirection#NORTH} for every given child node.
Edges always connect to the top of the child nodes.
@param child
@returns {@link ParentConnectorDirection#NORTH}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Places the children of a local root node in a grid.
@param localRoot the local root node
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged subtree shape that contains the local root node and all its children
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
/**
Data provider key for assigning nodes to the rows in the grid
If no {@link IDataProvider} is registered with this key, all nodes will be placed in the same row.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23ROW_INDEX_DP_KEY Online Documentation}*/
static ROW_INDEX_DP_KEY: NodeDpKey<number> | null;
/**
Gets or sets the placement of the bus with respect to the child node grid.
@throws {Exception} <b>(ArgumentError)</b> if the given bus placement is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23busPlacement Online Documentation}*/
busPlacement: BusPlacement;
/**
Gets or sets whether or not the nodes are automatically assigned to the rows of the grid.
If enabled, the nodes are assigned to rows automatically. User-specified row indices are ignored. The grid will be
square-like with respect to the child count, ignoring actual subtree sizes. For example, if there are nine children, a
grid with three rows - each row containing three nodes - is generated.
<b>Warning:</b> If this feature is enabled, the {@link GridNodePlacer#createFromSketchComparer from-sketch order} is ignored.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23automaticRowAssignment Online Documentation}*/
automaticRowAssignment: boolean;
/**
Gets or sets the {@link RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23rootAlignment Online Documentation}*/
rootAlignment: RootNodeAlignment | null;
/**
Horizontal {@link RootNodeAlignment} with the bus of the grid so that the position of the root depends on the
selected {@link GridNodePlacer#busPlacement bus placement}.
The root node is simply center-aligned with the bus of the subtree grid created by
{@link GridNodePlacer}. The given shapes of the children and the spacing are not considered in any way.
<b>Warning:</b> This alignment is only intended for usage with {@link GridNodePlacer}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/GridNodePlacer%23BUS_ALIGNED Online Documentation}*/
static BUS_ALIGNED: RootNodeAlignment | null;
static $class: Class;
private $_GridNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GridNodePlacer;
}
export interface GroupedNodePlacer extends YObject, ITreeLayoutNodePlacer {
}
/**
This {@link ITreeLayoutNodePlacer} places the children of a local root in groups.
The groups are determined by the grouping of the edges that connect the children to its root. The placement of the
groups is specified by a {@link ITreeLayoutNodePlacer} which will consider grouped children as one child. The children within
a group are placed by another {@link ITreeLayoutNodePlacer}.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupedNodePlacer Online Documentation}*/
export class GroupedNodePlacer {
/**
Creates a new {@link GroupedNodePlacer} instance.
@param groupPlacer the {@link ITreeLayoutNodePlacer} which places the child groups
@param childPlacer the {@link ITreeLayoutNodePlacer} which places the children within their group
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupedNodePlacer Online Documentation}*/
constructor(groupPlacer: ITreeLayoutNodePlacer | null, childPlacer: ITreeLayoutNodePlacer | null);
/**
Creates a {@link IProcessor} that prepares the graph for edge grouping.
In a pre-processing step, the {@link IProcessor} inserts a dummy node for each edge group and connects it to the
local root and the edges of the according group. The child node placer is assigned to those dummy nodes and will arrange
them during the layout calculation.
The changes to the graph are restored in a post-processing step.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the root node handled by this {@link ITreeLayoutNodePlacer}
@returns the {@link IProcessor} responsible for preparing the graph
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupedNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Delegates to the {@link ITreeLayoutNodePlacer} responsible for arranging the nodes in one group.
@param localRoot the local root node
@param connectorMap the map used for storing the direction specifiers of the child nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupedNodePlacer%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Places the groups of {@link SubtreeShape}s according to their group IDs.
@param nodeShapeProvider the {@link IDataProvider} for obtaining an initial shape of the root node
@param subtreeShapeProvider the {@link IDataProvider} for accessing the pre-calculated shapes of the subtrees
@param graph the input graph
@param localRoot the root of the subtree that should be arranged by this method
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged {@link SubtreeShape}s of the local root and its children
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/GroupedNodePlacer%23placeSubtree(IDataProvider%2CIDataProvider%2CLayoutGraph%2CYNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtree(nodeShapeProvider: IDataProvider | null, subtreeShapeProvider: IDataProvider | null, graph: LayoutGraph | null, localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection): SubtreeShape | null;
static $class: Class;
private $_GroupedNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GroupedNodePlacer;
}
export interface IFromSketchNodePlacer extends ITreeLayoutNodePlacer {
}
/**
A {@link IFromSketchNodePlacer} arranges the subtrees while taking the initial locations of the nodes into account.
It provides a {@link IComparer} which sorts the outgoing edges of a node according to the initial coordinates of their target nodes.
Keeping the original order of nodes, this {@link IComparer} allows to incrementally add new nodes to the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFromSketchNodePlacer Online Documentation}*/
export abstract class IFromSketchNodePlacer {
/**
Creates an {@link IComparer edge comparator} which takes the initial coordinates of the nodes into account.
This {@link IComparer} defines the order of the outgoing edges of a node before the actual placement of the node's subtree is
calculated.
@returns the {@link IComparer} for the edges
@see {@link https://docs.yworks.com/yfileshtml/#/api/IFromSketchNodePlacer%23createFromSketchComparer() Online Documentation}*/
abstract createFromSketchComparer(): IComparer<any> | null;
static $class: Class;
private $_IFromSketchNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IFromSketchNodePlacer;
}
export interface ITreeLayoutNodePlacer {
}
/**
A {@link ITreeLayoutNodePlacer} is responsible for the arrangement of a local root node and all of its subtrees.
Subtrees are represented as {@link SubtreeShape SubtreeShape}s with a connecting edge to the root of the subtree.
Implementations of this interface have to:
<ul>
<li>arrange the {@link SubtreeShape SubtreeShapes} and the root node (only the relative position matters, shapes can be placed at any absolute
location, {@link TreeLayout} will perform the final alignment).
<li>perform the edge routing from the root node to all child nodes with respect to the connector provided by the
{@link SubtreeShape SubtreeShapes}
<li>calculate the union of the {@link SubtreeShape SubtreeShapes} and the shape of the root node as well as any added edges
<li>initialize the connector of the root node with respect to the preferred connection direction
<li>return the {@link SubtreeShape SubtreeShape} for the subtree including the local root
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITreeLayoutNodePlacer Online Documentation}*/
export abstract class ITreeLayoutNodePlacer {
/**
Creates an optional {@link IProcessor} for pre- and post-processing.
This method is called by {@link TreeLayout} before arranging the {@link SubtreeShape SubtreeShapes}. If no pre- or post-processing is
needed, this method may return <code>null</code>.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the root node handled by this {@link ITreeLayoutNodePlacer}
@returns a {@link IProcessor} instance or <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITreeLayoutNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
abstract createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Provides the direction of the connector to the {@link SubtreeShape SubtreeShape} for each child node.
This method is called by {@link TreeLayout} before the {@link SubtreeShape SubtreeShapes} for the children of the local root node are
calculated. The direction specifiers are stored in the given map for all child nodes of the local root.
@param localRoot the local root node
@param connectorMap the map that must be used for storing the direction specifiers of the child nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITreeLayoutNodePlacer%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
abstract determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Arranges the {@link SubtreeShape SubtreeShapes} of the local root and its children and routes the edges that connect them.
This method is the main method of the tree layout algorithm. It is called by {@link TreeLayout} in each recursion
step to place the local root in relation to its children. Their shapes will be merged into one
{@link SubtreeShape SubtreeShape}, which is returned by this method.
The {@link SubtreeShape SubtreeShape} for the local root can be obtained from the <code>nodeShapeProvider</code>. <code>subtreeShapeProvider</code> contains the {@link SubtreeShape SubtreeShapes}
assigned to the child nodes.
Due to the order in which the tree nodes are processed, it is guaranteed that at the time of the invocation of this
method the {@link SubtreeShape SubtreeShape} of every descendant node has already been calculated.
@param nodeShapeProvider the {@link IDataProvider} for obtaining an initial shape of the root node
@param subtreeShapeProvider the {@link IDataProvider} for accessing the pre-calculated shapes of the subtrees
@param graph the input graph
@param localRoot the root of the subtree that should be arranged by this method
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the combined shape of the whole subtree
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITreeLayoutNodePlacer%23placeSubtree(IDataProvider%2CIDataProvider%2CLayoutGraph%2CYNode%2CParentConnectorDirection) Online Documentation}*/
abstract placeSubtree(nodeShapeProvider: IDataProvider | null, subtreeShapeProvider: IDataProvider | null, graph: LayoutGraph | null, localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection): SubtreeShape | null;
static $class: Class;
private $_INodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ITreeLayoutNodePlacer;
}
export interface ITreeLayoutPortAssignment {
}
/**
A {@link ITreeLayoutPortAssignment} places the ports of edges connecting to the same node.
Classes implementing this interface are responsible for the assignment of the edges' ports. {@link ITreeLayoutNodePlacer}
instances need to obey the currently set ports.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITreeLayoutPortAssignment Online Documentation}*/
export abstract class ITreeLayoutPortAssignment {
/**
Places the ports of edges connecting to the given node.
This method is called by {@link TreeLayout} before the actual layout calculation takes place. It assigns both the
single incoming edge's target port as well as all source ports of all outgoing child edges.
<h2>Note</h2>
At the time this method gets invoked, {@link TreeLayout} may have reversed some edges in order to normalize the tree
structure.
@param graph the graph
@param node the node whose adjacent edges' ports should be set
@see {@link https://docs.yworks.com/yfileshtml/#/api/ITreeLayoutPortAssignment%23assignPorts(LayoutGraph%2CYNode) Online Documentation}*/
abstract assignPorts(graph: LayoutGraph | null, node: YNode | null): void;
static $class: Class;
private $_IPortAssignment: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ITreeLayoutPortAssignment;
}
export interface IProcessor {
}
/**
A {@link IProcessor} provides the possibility for each {@link ITreeLayoutNodePlacer} to prepare (and clean up) the graph
for the children of the local root.
It is created by {@link ITreeLayoutNodePlacer#createProcessor} and called by the {@link TreeLayout} before and after the
actual arrangement of the nodes.
A {@link IProcessor} may change values within the
{@link IDataMap}s for its children. It can also temporarily change the graph by hiding nodes or adding dummy
nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IProcessor Online Documentation}*/
export abstract class IProcessor {
/**
This method is called by the {@link TreeLayout} after the layout has finished.
It can be used for cleanup tasks.
Changes on the graph structure that have been made in {@link IProcessor#preProcess} can be undone here.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IProcessor%23postProcess() Online Documentation}*/
abstract postProcess(): void;
/**
Performs preparations for the actual tree layout in a pre-processing step.
This method is called by {@link TreeLayout} before the {@link ITreeLayoutNodePlacer}s are called recursively. A {@link IProcessor}
may change some settings within this method. This can be used for specifying values for the children of the local root,
manipulate the graph structure or do anything else that needs to be done.
<b>Warning:</b> Only changes regarding the descendants of the current root node are allowed.
@param nodePlacerDataProvider the map that stores a {@link ITreeLayoutNodePlacer} instance for each node
@param portAssignmentDataProvider The map that stores a {@link ITreeLayoutPortAssignment} instance for each node
@param childComparatorProvider the map that stores a {@link IComparer} instance for each node
@see {@link https://docs.yworks.com/yfileshtml/#/api/IProcessor%23preProcess(IDataMap%2CIDataMap%2CIDataMap) Online Documentation}*/
abstract preProcess(nodePlacerDataProvider: IDataMap | null, portAssignmentDataProvider: IDataMap | null, childComparatorProvider: IDataMap | null): void;
static $class: Class;
private $_IProcessor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IProcessor;
}
export interface LayeredNodePlacer extends RotatableNodePlacerBase {
}
/**
A {@link LayeredNodePlacer} arranges the nodes of a subtree respecting layers.
<b>Layout Style</b>
Nodes that have the same depth in the tree will be placed in the same horizontal layer. They will be vertically
{@link LayeredNodePlacer#verticalAlignment aligned}.
<b>Warning:</b> Changing the {@link RotatableNodePlacerBase#modificationMatrix orientation} within the {@link LayeredNodePlacer} is strongly discouraged because it causes problems
with the layering.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer Online Documentation}*/
export class LayeredNodePlacer {
/**
Creates a new {@link LayeredNodePlacer} instance whose direction is specified by the given modification matrix.
The given ID identifies different {@link LayeredNodePlacer} instances that share the same layer heights.
@param modificationMatrix the transformation matrix for this {@link RotatableNodePlacerBase}
@param id the {@link LayeredNodePlacer} ID
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23LayeredNodePlacer(RotatableNodePlacerMatrix%2CObject) Online Documentation}*/
constructor(modificationMatrix: RotatableNodePlacerMatrix | null, id: any | null);
/**
Creates a new {@link LayeredNodePlacer} instance whose direction is specified by the given modification matrix.
The given ID identifies different {@link LayeredNodePlacer} instances that share the same layer heights.
@param options The parameters to pass.
@param options.modificationMatrix the transformation matrix for this {@link RotatableNodePlacerBase}
@param options.id the {@link LayeredNodePlacer} ID
@param [options.dendrogramStyle] Whether or not the layering is done dendrogram-like. This option sets the {@link LayeredNodePlacer#dendrogramStyle} property on the created object.
@param [options.rootAlignment] The {@link RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes. This option sets the {@link LayeredNodePlacer#rootAlignment} property on the created object.
@param [options.verticalAlignment] The relative vertical alignment of nodes within their respective layers. This option sets the {@link LayeredNodePlacer#verticalAlignment} property on the created object.
@param [options.layerSpacing] The distance between two adjacent layers. This option sets the {@link LayeredNodePlacer#layerSpacing} property on the created object.
@param [options.routingStyle] The routing style for the edges in the subtree. This option sets the {@link LayeredNodePlacer#routingStyle} property on the created object.
@param [options.busAlignment] The vertical bus alignment for orthogonally routed edges. This option sets the {@link LayeredNodePlacer#busAlignment} property on the created object.
@param [options.polylineLabeling] Whether or not straight edges are extended to avoid label overlaps. This option sets the {@link LayeredNodePlacer#polylineLabeling} property on the created object.
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23LayeredNodePlacer(RotatableNodePlacerMatrix%2CObject) Online Documentation}*/
constructor(options: {
modificationMatrix: RotatableNodePlacerMatrix | null;
id: any | null;
dendrogramStyle?: boolean;
rootAlignment?: RootNodeAlignment | null;
verticalAlignment?: number;
layerSpacing?: number;
routingStyle?: LayeredRoutingStyle | LayeredRoutingStyleStringValues;
busAlignment?: number;
polylineLabeling?: boolean;
spacing?: number;
});
/**
Creates a new {@link LayeredNodePlacer} instance with the default settings.
@param [options] The parameters to pass.
@param [options.dendrogramStyle] Whether or not the layering is done dendrogram-like. This option sets the {@link LayeredNodePlacer#dendrogramStyle} property on the created object.
@param [options.rootAlignment] The {@link RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes. This option sets the {@link LayeredNodePlacer#rootAlignment} property on the created object.
@param [options.verticalAlignment] The relative vertical alignment of nodes within their respective layers. This option sets the {@link LayeredNodePlacer#verticalAlignment} property on the created object.
@param [options.layerSpacing] The distance between two adjacent layers. This option sets the {@link LayeredNodePlacer#layerSpacing} property on the created object.
@param [options.routingStyle] The routing style for the edges in the subtree. This option sets the {@link LayeredNodePlacer#routingStyle} property on the created object.
@param [options.busAlignment] The vertical bus alignment for orthogonally routed edges. This option sets the {@link LayeredNodePlacer#busAlignment} property on the created object.
@param [options.polylineLabeling] Whether or not straight edges are extended to avoid label overlaps. This option sets the {@link LayeredNodePlacer#polylineLabeling} property on the created object.
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23LayeredNodePlacer() Online Documentation}*/
constructor(options?: {
dendrogramStyle?: boolean;
rootAlignment?: RootNodeAlignment | null;
verticalAlignment?: number;
layerSpacing?: number;
routingStyle?: LayeredRoutingStyle | LayeredRoutingStyleStringValues;
busAlignment?: number;
polylineLabeling?: boolean;
spacing?: number;
});
/**
Creates a {@link IProcessor} that distributes the nodes over the layers.
The {@link IProcessor} is chosen depending on the {@link LayeredNodePlacer#dendrogramStyle layout style}.
@param graph the input graph
@param currentRoot the current root node for this node placer
@param layouter
@returns the {@link IProcessor} or <code>null</code> if the layer information is already assigned to the current root
@see LayeredNodePlacer#dendrogramStyle
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Returns {@link ParentConnectorDirection#NORTH} for every given child node.
Edges always connect to the top of the child nodes.
@param child
@returns {@link ParentConnectorDirection#NORTH}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Places the children of local root in layers.
@param localRoot the local root node
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged subtree shape that contains the local root node and all its children
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
/**
Gets or sets whether or not the layering is done dendrogram-like.
When enabled, leaves are placed in the same layer and remaining tree nodes are assigned to a layer as close to their
children as possible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23dendrogramStyle Online Documentation}*/
dendrogramStyle: boolean;
/**
Gets or sets the {@link RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23rootAlignment Online Documentation}*/
rootAlignment: RootNodeAlignment | null;
/**
Gets or sets the relative vertical alignment of nodes within their respective layers.
A value of <code>0</code> means that nodes are top-aligned; a value of <code>0.5</code> means that nodes are center-aligned; a value of <code>1</code> means
that nodes are bottom-aligned;
Values for a valid vertical alignment are in the range <code>[0..1]</code>.
@throws {Exception} <b>(ArgumentError)</b> if the specified alignment is outside <code>[0..1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23verticalAlignment Online Documentation}*/
verticalAlignment: number;
/**
Gets the ID of this {@link LayeredNodePlacer}.
The ID is used for identifying
{@link LayeredNodePlacer}s that share information about the layer height. Using different IDs allows aligned layouts
within different subtrees.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23id Online Documentation}*/
readonly id: any | null;
/**
Gets or sets the distance between two adjacent layers.
The spacing needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23layerSpacing Online Documentation}*/
layerSpacing: number;
/**
Gets or sets the routing style for the edges in the subtree.
@throws {Exception} <b>(ArgumentError)</b> if the specified routing style specifier is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23routingStyle Online Documentation}*/
routingStyle: LayeredRoutingStyle;
/**
Gets or sets the vertical bus alignment for orthogonally routed edges.
The bus alignment determines the relative position of an edge bus in between two subsequent layers of nodes.
Values for a valid vertical alignment are in the range <code>[0..1]</code>, such that:
<ul>
<li>A value of <code>0.0</code> places the bus at the top, right below the parent node.
<li>A value of <code>0.5</code> places the bus in the middle between parent and child nodes.
<li>A value of <code>1.0</code> places the bus at the bottom, right above the child nodes.
</ul>
<b>Warning:</b> Bus alignment only applies if the {@link LayeredNodePlacer#routingStyle routing style} is set to {@link LayeredRoutingStyle#ORTHOGONAL}.
@throws {Exception} <b>(ArgumentError)</b> if the specified alignment is outside <code>[0..1]</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23busAlignment Online Documentation}*/
busAlignment: number;
/**
Gets or sets whether or not straight edges are extended to avoid label overlaps.
If enabled, polyline connectors between the parent and its children are added to the according subtree shape. Node or
edge labels will not be crossed.
<b>Warning:</b> Polyline labeling only has an effect if the {@link LayeredNodePlacer#routingStyle routing style} is {@link LayeredRoutingStyle#STRAIGHT}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayeredNodePlacer%23polylineLabeling Online Documentation}*/
polylineLabeling: boolean;
static $class: Class;
private $_LayeredNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayeredNodePlacer;
}
export interface LeafNodePlacer extends YObject, ITreeLayoutNodePlacer {
}
/**
A {@link LeafNodePlacer} creates a {@link SubtreeShape SubtreeShape} for a leaf node and routes its incoming edge at the target side.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafNodePlacer Online Documentation}*/
export class LeafNodePlacer {
/**
Creates a new {@link LeafNodePlacer} instance with default settings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafNodePlacer Online Documentation}*/
constructor();
/**
Creates an optional {@link IProcessor} for pre- and post-processing.
This implementation returns <code>null</code>. Hence, no pre- or post-processing is done.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the root node handled by this {@link ITreeLayoutNodePlacer}
@returns <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Does nothing because leaf nodes don't have children so there are no child connectors.
@param localRoot the local root node
@param connectorMap the map that must be used for storing the direction specifiers of the child nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafNodePlacer%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Creates a {@link SubtreeShape} for the local root node (a leaf node in this case) and adds target points to the
incoming edge.
The incoming edge is routed according to possibly assigned {@link PortConstraint}s.
@param nodeShapeProvider the {@link IDataProvider} for obtaining an initial shape of the root node
@param subtreeShapeProvider the {@link IDataProvider} for accessing the pre-calculated shapes of the subtrees
@param graph the input graph
@param localRoot the root of the subtree that should be arranged by this method
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the {@link SubtreeShape} of the leaf node including the connector
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeafNodePlacer%23placeSubtree(IDataProvider%2CIDataProvider%2CLayoutGraph%2CYNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtree(nodeShapeProvider: IDataProvider | null, subtreeShapeProvider: IDataProvider | null, graph: LayoutGraph | null, localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection): SubtreeShape | null;
static $class: Class;
private $_LeafNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LeafNodePlacer;
}
export interface LeftRightNodePlacer extends RotatableNodePlacerBase {
}
/**
The {@link LeftRightNodePlacer} arranges the children in a subtree on the left and the right of a vertical bus.
<b>Layout Style</b> The default settings generate a layout with a single vertical bus, where children are placed left and right of it. By
adjusting the
{@link LeftRightNodePlacer#branchCount branch count}, it is possible to generate multiple vertical buses (i.e. branches) that are placed next to each other,
below the root node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer Online Documentation}*/
export class LeftRightNodePlacer {
/**
Creates a new {@link LeftRightNodePlacer} instance using the given modification matrix.
@param modificationMatrix the transformation matrix for this {@link RotatableNodePlacerBase}
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23LeftRightNodePlacer(RotatableNodePlacerMatrix) Online Documentation}*/
constructor(modificationMatrix: RotatableNodePlacerMatrix | null);
/**
Creates a new {@link LeftRightNodePlacer} instance using the given modification matrix.
@param options The parameters to pass.
@param options.modificationMatrix the transformation matrix for this {@link RotatableNodePlacerBase}
@param [options.horizontalDistance] The horizontal distance between nodes or subtrees. This option sets the {@link LeftRightNodePlacer#horizontalDistance} property on the created object.
@param [options.verticalDistance] The vertical distance between nodes or subtrees. This option sets the {@link LeftRightNodePlacer#verticalDistance} property on the created object.
@param [options.spacing] A uniform horizontal and vertical spacing between subtrees. This option sets the {@link LeftRightNodePlacer#spacing} property on the created object.
@param [options.branchCount] The number of branches (buses) of the subtree layout. This option sets the {@link LeftRightNodePlacer#branchCount} property on the created object.
@param [options.placeLastOnBottom] Whether or not the last child is placed centered, aligned with the bus and below the other child nodes. This option sets the {@link LeftRightNodePlacer#placeLastOnBottom} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23LeftRightNodePlacer(RotatableNodePlacerMatrix) Online Documentation}*/
constructor(options: {
modificationMatrix: RotatableNodePlacerMatrix | null;
horizontalDistance?: number;
verticalDistance?: number;
spacing?: number;
branchCount?: number;
placeLastOnBottom?: boolean;
});
/**
Creates a new {@link LeftRightNodePlacer} instance with default settings.
@param [options] The parameters to pass.
@param [options.horizontalDistance] The horizontal distance between nodes or subtrees. This option sets the {@link LeftRightNodePlacer#horizontalDistance} property on the created object.
@param [options.verticalDistance] The vertical distance between nodes or subtrees. This option sets the {@link LeftRightNodePlacer#verticalDistance} property on the created object.
@param [options.spacing] A uniform horizontal and vertical spacing between subtrees. This option sets the {@link LeftRightNodePlacer#spacing} property on the created object.
@param [options.branchCount] The number of branches (buses) of the subtree layout. This option sets the {@link LeftRightNodePlacer#branchCount} property on the created object.
@param [options.placeLastOnBottom] Whether or not the last child is placed centered, aligned with the bus and below the other child nodes. This option sets the {@link LeftRightNodePlacer#placeLastOnBottom} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23LeftRightNodePlacer() Online Documentation}*/
constructor(options?: {
horizontalDistance?: number;
verticalDistance?: number;
spacing?: number;
branchCount?: number;
placeLastOnBottom?: boolean;
});
/**
Returns a {@link IComparer} for outgoing edges that considers the locations of the target nodes.
The {@link IComparer} applies the {@link RotatableNodePlacerBase#modificationMatrix modification matrix} of this {@link LeftRightNodePlacer} and then compares the y-coordinates of the
target nodes.
@returns the {@link IComparer} that compares outgoing edges according to the locations of their target nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23createComparer() Online Documentation}*/
createComparer(): IComparer<any> | null;
/**
Creates a special {@link IDataProvider} that determines whether a given node is placed left or right of the
parent node.
This {@link IDataProvider} can be registered with the graph with key
{@link LeftRightNodePlacer#LEFT_RIGHT_DP_KEY}. It calculates the side with respect to the initial coordinates of the
nodes in the graph, considering the orientation of the subtree.
If a node is assigned to the left side, this {@link IDataProvider} will return <code>true</code>. For the right side, it
will return <code>false</code>.
<b>Warning:</b> Since the side of a node depends on the rotation of the associated {@link ITreeLayoutNodePlacer}, this method expects a {@link IDataProvider}
containing the {@link ITreeLayoutNodePlacer} instances that are used for the layout calculation.
@param nodePlacerMap a {@link IDataProvider} containing the corresponding {@link ITreeLayoutNodePlacer}s for the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23createLeftRightDataProvider(IDataProvider) Online Documentation}*/
static createLeftRightDataProvider(nodePlacerMap: IDataProvider | null): IDataProvider | null;
/**
Creates a processor that distributes the child elements into multiple branches, if necessary.
The {@link LeftRightNodePlacer#branchCount branch count} defines whether or not the subtree layout will consist of multiple branches. If so, this method creates
a processor to assign the children to the branches.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the root node handled by this {@link ITreeLayoutNodePlacer}
@returns the processor distributing children to multiple branches or <code>null</code> if only a single-branch layout is created
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Determines a connector direction for the given child node.
Since the connector directions are already determined by
{@link LeftRightNodePlacer#determineChildConnectors}, this method doesn't need to be called.
@param child
@throws {Exception} <b>(InvalidOperationError)</b> if this method is called
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Determines the direction of the child connectors according to which side of the bus the child is assigned.
Children on the left will get a connector to the {@link ParentConnectorDirection#EAST}. Children on the right will
get one to the {@link ParentConnectorDirection#WEST}. A connector to the {@link ParentConnectorDirection#NORTH}
is assigned for the bottom child.
@param localRoot the local root node
@param connectorMap the map that must be used for storing the direction specifiers of the child nodes
@see LeftRightNodePlacer#placeLastOnBottom
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Places the subtree shapes of the children left and right of a common vertical bus.
@param localRoot the local root node
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged subtree shape that contains the local root node and all its children
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
/**
Data provider key for assigning nodes to the left or the right side of the bus
If no {@link IDataProvider} is specified using this key, the nodes will be placed alternating from left to
right.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23LEFT_RIGHT_DP_KEY Online Documentation}*/
static LEFT_RIGHT_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the horizontal distance between nodes or subtrees.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23horizontalDistance Online Documentation}*/
horizontalDistance: number;
/**
Gets or sets the vertical distance between nodes or subtrees.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23verticalDistance Online Documentation}*/
verticalDistance: number;
/**
Sets a uniform horizontal and vertical spacing between subtrees.
This method sets the given distance value for the {@link LeftRightNodePlacer#horizontalDistance horizontal spacing} and for the {@link LeftRightNodePlacer#verticalDistance vertical spacing}.
@throws {Exception} <b>(ArgumentError)</b> if the specified spacing value is negative
@see LeftRightNodePlacer#horizontalDistance
@see LeftRightNodePlacer#verticalDistance
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23spacing Online Documentation}*/
spacing: number;
/**
Gets or sets the number of branches (buses) of the subtree layout.
A branch consists of a vertical bus where the child nodes are placed left and right of it. If the subtree handled by
this {@link ITreeLayoutNodePlacer} consists of multiple branches, they are placed next to each other with the root node
centered above them. Child nodes are equally distributed over the different available branches. The {@link TreeLayout#OUT_EDGE_COMPARER_DP_KEY out-edge order} is
considered such that child subtrees are first assigned to the leftmost branch (e.g. if each branch contains four
children, the first four children are in the first, leftmost branch, the second four children children are in the second
branch and so on).
<h2>Note</h2>
The assignment of child nodes to the branch is solely based on the out-edge order of the edges from the root of the
child nodes. Therefore, if nodes have a user-specified left/right side, the different branches may contain a different
number of left and right nodes. By providing a customized
{@link TreeLayout#OUT_EDGE_COMPARER_DP_KEY out-edge order}, this can be adjusted to fit more specific requirements.
@throws {Exception} <b>(ArgumentError)</b> if the given branch count is smaller than <code>1</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23branchCount Online Documentation}*/
branchCount: number;
/**
Gets or sets whether or not the last child is placed centered, aligned with the bus and below the other child nodes.
When enabled, the last child will always be centered independent of the number of siblings.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LeftRightNodePlacer%23placeLastOnBottom Online Documentation}*/
placeLastOnBottom: boolean;
static $class: Class;
private $_LeftRightNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LeftRightNodePlacer;
}
export interface MultiParentDescriptor extends YObject {
}
/**
The {@link MultiParentDescriptor} provides style information for multi-parent structures.
This class is used by {@link TreeLayout} to determine the desired layout of nodes that constitute a multi-parent
structure. All nodes of such a structure are placed side by side and the incident edges are routed over common points
for incoming edges and for outgoing edges.
@see TreeLayout#MULTI_PARENT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentDescriptor Online Documentation}*/
export class MultiParentDescriptor {
/**
Creates a new {@link MultiParentDescriptor} instance with default settings.
@param [options] The parameters to pass.
@param [options.minimumNodeDistance] The minimum distance between two nodes belonging to the same multi-parent structure. This option sets the {@link MultiParentDescriptor#minimumNodeDistance} property on the created object.
@param [options.minimumBusDistance] The minimum distance between the nodes of a multi-parent structure and the bus connecting these nodes. This option sets the {@link MultiParentDescriptor#minimumBusDistance} property on the created object.
@param [options.verticalAlignment] The relative vertical alignment of nodes belonging to the same multi-parent structure. This option sets the {@link MultiParentDescriptor#verticalAlignment} property on the created object.
@param [options.edgeStyle] The routing style for edges that connect to nodes which constitute a multi-parent structure. This option sets the {@link MultiParentDescriptor#edgeStyle} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentDescriptor Online Documentation}*/
constructor(options?: {
minimumNodeDistance?: number;
minimumBusDistance?: number;
verticalAlignment?: number;
edgeStyle?: MultiParentRoutingStyle | MultiParentRoutingStyleStringValues;
});
/**
Gets or sets the minimum distance between two nodes belonging to the same multi-parent structure.
The distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given node distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentDescriptor%23minimumNodeDistance Online Documentation}*/
minimumNodeDistance: number;
/**
Gets or sets the minimum distance between the nodes of a multi-parent structure and the bus connecting these nodes.
In case the edge style is not orthogonal, the bus only consists of a common point.
The minimum distance needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given bus distance is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentDescriptor%23minimumBusDistance Online Documentation}*/
minimumBusDistance: number;
/**
Gets or sets the relative vertical alignment of nodes belonging to the same multi-parent structure.
A value of <code>0</code> means that nodes are top-aligned; a value of <code>1</code> means that nodes are bottom-aligned; a value of <code>0.5</code> means
that nodes are center-aligned.
Values outside the interval <code>[0,1]</code> will result in a compact node placement with unaligned nodes, especially when labels
are considered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentDescriptor%23verticalAlignment Online Documentation}*/
verticalAlignment: number;
/**
Gets or sets the routing style for edges that connect to nodes which constitute a multi-parent structure.
<h2>Note</h2>
The routing style is applied only to the part of the edge route that is not shared by multiple edges.
@throws {Exception} <b>(ArgumentError)</b> if the given routing style is unknown
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiParentDescriptor%23edgeStyle Online Documentation}*/
edgeStyle: MultiParentRoutingStyle;
static $class: Class;
private $_MultiParentDescriptor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MultiParentDescriptor;
}
export interface NodeOrderComparer extends YObject, IComparer<any> {
}
/**
The {@link NodeOrderComparer} compares edges according to a specified order.
It obtains the order from a {@link IDataProvider} registered with the graph with key
{@link NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY}.
Various tree layout algorithms can use this {@link IComparer} to influence the resulting layout. To achieve this, use this comparator and
assign it to the comparator property of the layout algorithm classes.
@see ClassicTreeLayout#comparer
@see TreeLayout#defaultOutEdgeComparer
@see TreeLayout#OUT_EDGE_COMPARER_DP_KEY
@see BalloonLayout#comparer
@see AspectRatioTreeLayout#comparer
@see NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderComparer Online Documentation}*/
export class NodeOrderComparer {
/**
Creates a new {@link NodeOrderComparer} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderComparer Online Documentation}*/
constructor();
/**
Compares two edges according to their index in the order specified using a {@link IDataProvider}.
Target nodes for which the {@link IDataProvider} registered with {@link NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY}
returns <code>null</code> or an {@link Object} that does not implement {@link IComparable} are treated as being smaller than other target
nodes.
@param edge1 the first edge
@param edge2 the second edge
@see NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderComparer%23compare(Object%2CObject) Online Documentation}*/
compare(edge1: any, edge2: any): number;
/**
Data provider key for registering an order for each node in the graph
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeOrderComparer%23NODE_ORDER_COMPARABLE_DP_KEY Online Documentation}*/
static NODE_ORDER_COMPARABLE_DP_KEY: NodeDpKey<IComparable> | null;
static $class: Class;
private $_NodeOrderComparer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeOrderComparer;
}
export interface NodePlacerBase extends YObject, ITreeLayoutNodePlacer, ICloneable {
}
/**
Abstract base class for implementations of the {@link ITreeLayoutNodePlacer} interface.
This class provides convenience methods for sub-tasks that are often used during the arrangement of a local root and its
children. It also offers prepared implementations for the methods of the {@link ITreeLayoutNodePlacer} interface.
Subclasses just need to override the following two abstract methods: {@link NodePlacerBase#placeSubtreeOfNode} and
{@link NodePlacerBase#determineChildConnectors}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase Online Documentation}*/
export abstract class NodePlacerBase {
/**
Creates a new instance of {@link NodePlacerBase}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase Online Documentation}*/
constructor();
/**
Returns a clone of this {@link ITreeLayoutNodePlacer} instance.
@returns an exact copy of this {@link ITreeLayoutNodePlacer} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase%23clone() Online Documentation}*/
clone(): this;
/**
Creates an optional {@link IProcessor} for pre- and post-processing.
This implementation returns <code>null</code>. Hence, no pre- or post-processing will be done.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the root node handled by this {@link ITreeLayoutNodePlacer}
@returns <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Determines a connector direction for the given child node.
This method is used in {@link NodePlacerBase#determineChildConnectors} and must be implemented by subclasses.
@param child the child node
@returns a direction specifier as defined in the {@link ITreeLayoutNodePlacer} interface
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase%23determineChildConnector(YNode) Online Documentation}*/
abstract determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Provides the direction of the connector to the {@link SubtreeShape SubtreeShape} for each child node.
This implementation uses {@link NodePlacerBase#determineChildConnector} to retrieve the direction for the
connectors.
@param localRoot the local root node
@param connectorMap the map that must be used for storing the direction specifiers of the child nodes
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Returns the {@link SubtreeShape} for the given node.
This is a convenience method that retrieves a {@link SubtreeShape} for a single node from the corresponding
{@link IDataProvider}.
This method is used in {@link NodePlacerBase#placeSubtreeOfNode} for retrieving the {@link SubtreeShape}s for
the local root nodes. It may be overridden to return a custom implementation of {@link SubtreeShape}.
<h2>Note</h2>
This method can first be called during the execution of {@link NodePlacerBase#placeSubtreeOfNode}. Otherwise, the {@link IDataProvider}
for querying results will not have been initialized yet.
@param node the node
@returns the {@link SubtreeShape} for the given node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase%23getNodeShape(YNode) Online Documentation}*/
getNodeShape(node: YNode | null): SubtreeShape | null;
/**
Returns the {@link SubtreeShape} for the subtree rooted at the given node.
This is a convenience method that retrieves a {@link SubtreeShape} of a complete subtree from the corresponding
{@link IDataProvider}.
This method is used in {@link NodePlacerBase#placeSubtreeOfNode} for retrieving the {@link SubtreeShape}s for
the subtrees that are arranged. It may be overridden to return a custom implementation of {@link SubtreeShape}.
<h2>Note</h2>
This method can first be called during the execution of {@link NodePlacerBase#placeSubtreeOfNode}. Before, the {@link IDataProvider}
for querying results will not have been initialized yet.
@param node the local root node
@returns the {@link SubtreeShape} of the subtree rooted at the given local root node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase%23getSubtreeShape(YNode) Online Documentation}*/
getSubtreeShape(node: YNode | null): SubtreeShape | null;
/**
Initializes the local data structures and then delegates the work to the abstract variant of this method
({@link NodePlacerBase#placeSubtreeOfNode})
@param nodeShapeProvider the {@link IDataProvider} for obtaining an initial shape of the root node
@param subtreeShapeProvider the {@link IDataProvider} for accessing the pre-calculated shapes of the subtrees
@param graph the input graph
@param localRoot the root of the subtree that should be arranged by this method
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged {@link SubtreeShape} for the subtree rooted at the local root node
@see NodePlacerBase#placeSubtreeOfNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase%23placeSubtree(IDataProvider%2CIDataProvider%2CLayoutGraph%2CYNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtree(nodeShapeProvider: IDataProvider | null, subtreeShapeProvider: IDataProvider | null, graph: LayoutGraph | null, localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection): SubtreeShape | null;
/**
Arranges the local root relative to the {@link SubtreeShape} of its children.
This is the main method that must be implemented by subclasses. It is called by
{@link NodePlacerBase#placeSubtree}.
@param localRoot the local root node
@param parentConnectorDirection the direction of the connector
@returns a {@link SubtreeShape} instance that describes the shape of the whole subtree rooted at the given local root
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
abstract placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): SubtreeShape | null;
/**
The graph instance this class is working on.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodePlacerBase%23graph Online Documentation}*/
graph: LayoutGraph | null;
static $class: Class;
private $_NodePlacerBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodePlacerBase;
}
export interface RootNodeAlignment extends YObject {
}
/**
A {@link RootNodeAlignment} describes how a local root node is arranged in relation to its children.
The actual placement depends on the orientation of the subtree rooted at the current local root node. All descriptions
of the provided alignments refer to the top-to-bottom orientation described by
{@link RotatableNodePlacerMatrix#DEFAULT}. In this case, the subtrees are placed below the local root and the local
root is aligned horizontally above them.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment Online Documentation}*/
export abstract class RootNodeAlignment {
/**
Calculates the common bounds of the given subtree shapes.
This method can be used when placing the local root above its subtrees in method
{@link RootNodeAlignment#placeParentHorizontal}.
@param shapes a list of rotated subtree shapes
@returns the rectangle describing the bounds containing all given shapes
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23getBounds(IList%3CObject%3E) Online Documentation}*/
static getBounds(shapes: IList<any> | null): Rectangle2D | null;
/**
Moves the shape of the local root to a suitable position relative to the children's shapes.
The movement should only be done on the x-axis (horizontally).
This method is called by {@link RotatableNodePlacerBase}s when they {@link RotatableNodePlacerBase#placeSubtreeOfNode place the subtrees}.
@param rootShape the shape of the local root node that will be moved
@param shapes a list of rotated subtree shapes, representing the shapes of the child nodes
@param shapeBounds accumulated bounds of the shapes of all child nodes
@param spacing the spacing value that should be used
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23placeParentHorizontal(RotatedSubtreeShape%2CIList%3CObject%3E%2CRectangle2D%2Cnumber) Online Documentation}*/
abstract placeParentHorizontal(rootShape: RotatedSubtreeShape | null, shapes: IList<any> | null, shapeBounds: Rectangle2D | null, spacing: number): void;
/**
Horizontal {@link RootNodeAlignment} at the center of the subtrees.
The root node is placed centered above its whole subgraph. To determine the alignment, only {@link INodeLayout}s
are considered. Node labels are not included although they are also contained in {@link SubtreeShape}s.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23CENTER Online Documentation}*/
static CENTER: RootNodeAlignment | null;
/**
Horizontal {@link RootNodeAlignment} at the median child node.
The root node is placed above the median of its children. To determine the alignment, only {@link INodeLayout}s
are considered. Node labels are not included although they are also contained in {@link SubtreeShape}s.
<h2>Note</h2>
This alignment is only suited for {@link ITreeLayoutNodePlacer}s that place all children in a single row.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23MEDIAN Online Documentation}*/
static MEDIAN: RootNodeAlignment | null;
/**
Horizontal {@link RootNodeAlignment} left of the children.
The local root node leaves a distance specified by the given spacing to the leftmost child. To determine the alignment,
only {@link INodeLayout}s are considered. Node labels are not included although they are also contained in
{@link SubtreeShape}s.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23LEADING Online Documentation}*/
static LEADING: RootNodeAlignment | null;
/**
Horizontal {@link RootNodeAlignment} right of the children.
The root node leaves a distance specified by the given spacing to the rightmost child. To determine the alignment, only
{@link INodeLayout}s are considered. Node labels are not included although they are also contained in
{@link SubtreeShape}s.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23TRAILING Online Documentation}*/
static TRAILING: RootNodeAlignment | null;
/**
Horizontal {@link RootNodeAlignment} at the left side.
The root node is placed left aligned with its leftmost child. To determine the alignment, only
{@link INodeLayout}s are considered. Node labels are not included although they are also contained in
{@link SubtreeShape}s.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23LEFT Online Documentation}*/
static LEFT: RootNodeAlignment | null;
/**
Horizontal {@link RootNodeAlignment} at the right side.
The root node is placed right aligned with its rightmost child. To determine the alignment, only
{@link INodeLayout}s are considered. Node labels are not included although they are also contained in
{@link SubtreeShape}s.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23RIGHT Online Documentation}*/
static RIGHT: RootNodeAlignment | null;
/**
Horizontal alignment at the center of the children.
The root node is placed centered over its direct children. To determine the alignment, only {@link INodeLayout}s
are considered. Node labels are not included although they are also contained in {@link SubtreeShape}s.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23CENTER_OVER_CHILDREN Online Documentation}*/
static CENTER_OVER_CHILDREN: RootNodeAlignment | null;
/**
An array containing all supported {@link RootNodeAlignment}s.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RootNodeAlignment%23ALL Online Documentation}*/
static ALL: RootNodeAlignment[] | null;
static $class: Class;
private $_RootNodeAlignment: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RootNodeAlignment;
}
export interface RotatableNodePlacerMatrix extends YObject {
}
/**
A matrix describes the rotation of the subtree handled by {@link RotatableNodePlacerBase}.
Subtrees may be rotated or mirrored. Also, matrices offering a combination of rotation and mirroring are provided.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix Online Documentation}*/
export class RotatableNodePlacerMatrix {
/**
Compares the values of the current {@link RotatableNodePlacerMatrix} with the values of the given
{@link RotatableNodePlacerMatrix}.
@param other the matrix
@returns <code>true</code> if this matrix and the given other matrix contain the same values, <code>false</code> otherwise
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23equalValues(RotatableNodePlacerMatrix) Online Documentation}*/
equalValues(other: RotatableNodePlacerMatrix | null): boolean;
/**
Multiplies this {@link RotatableNodePlacerMatrix} with the given other {@link RotatableNodePlacerMatrix}.
@param other the other matrix
@returns the product of both matrices
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23multiply(RotatableNodePlacerMatrix) Online Documentation}*/
multiply(other: RotatableNodePlacerMatrix | null): RotatableNodePlacerMatrix | null;
/**
Returns a {@link string} representation of this {@link RotatableNodePlacerMatrix}.
@returns the {@link string} representing this {@link RotatableNodePlacerMatrix}
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23toString() Online Documentation}*/
toString(): string;
/**
The identity matrix which does not apply any rotation or mirroring.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23DEFAULT Online Documentation}*/
static DEFAULT: RotatableNodePlacerMatrix | null;
/**
A matrix which applies a counter-clockwise rotation by 90 degrees.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23ROT90 Online Documentation}*/
static ROT90: RotatableNodePlacerMatrix | null;
/**
A matrix which applies a rotation by 180 degrees.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23ROT180 Online Documentation}*/
static ROT180: RotatableNodePlacerMatrix | null;
/**
A matrix which applies a counter-clockwise rotation by 270 degrees.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23ROT270 Online Documentation}*/
static ROT270: RotatableNodePlacerMatrix | null;
/**
A matrix which applies a horizontal mirroring.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23MIR_HOR Online Documentation}*/
static MIR_HOR: RotatableNodePlacerMatrix | null;
/**
A matrix which applies a vertical mirroring.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23MIR_VERT Online Documentation}*/
static MIR_VERT: RotatableNodePlacerMatrix | null;
/**
A matrix which applies a horizontal mirroring and a counter-clockwise rotation by 90 degrees.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23MIR_HOR_ROT90 Online Documentation}*/
static MIR_HOR_ROT90: RotatableNodePlacerMatrix | null;
/**
A matrix which applies a vertical mirroring and a counter-clockwise rotation by 90 degrees.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerMatrix%23MIR_VERT_ROT90 Online Documentation}*/
static MIR_VERT_ROT90: RotatableNodePlacerMatrix | null;
static $class: Class;
private $_RotatableNodePlacerMatrix: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RotatableNodePlacerMatrix;
}
export interface RotatableNodePlacerBase extends YObject, IFromSketchNodePlacer {
}
/**
Abstract base class for {@link ITreeLayoutNodePlacer}s that support rotations of the subtrees.
The direction specified in this {@link RotatableNodePlacerBase} (defined by a
{@link RotatableNodePlacerBase#modificationMatrix transformation matrix}) rotates the subtree that is placed. This is the final direction of this subtree. Rotations are
not accumulated.
{@link ITreeLayoutNodePlacer}s of this type will only implement the default direction (i.e. top-to-bottom). The other
directions are calculated using the specified modification {@link RotatableNodePlacerMatrix}. The direction used by
the implementing
{@link ITreeLayoutNodePlacer}s is called the model direction while the actual direction in the graph is the real direction.
Note that using rotatable {@link ITreeLayoutNodePlacer}s has some pitfalls. Calculations must be aware of the rotations.
Especially operations on {@link BorderLine}s should not be called directly (e.g.
{@link BorderLine#mergeWithMin}, {@link BorderLine#mergeWithMax}). The corresponding methods of {@link RotatableNodePlacerBase}
already support the different directions.
Subclasses just need to override the following two abstract methods: {@link RotatableNodePlacerBase#placeSubtreeOfNode}
and {@link RotatableNodePlacerBase#determineChildConnectors}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase Online Documentation}*/
export abstract class RotatableNodePlacerBase {
/**
Creates a new instance of {@link RotatableNodePlacerBase} whose direction is specified by the given modification
matrix.
@param modificationMatrix the transformation matrix for the {@link ITreeLayoutNodePlacer}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase Online Documentation}*/
constructor(modificationMatrix: RotatableNodePlacerMatrix | null);
/**
Creates a new instance of {@link RotatableNodePlacerBase} whose direction is specified by the given modification
matrix.
@param options The parameters to pass.
@param options.modificationMatrix the transformation matrix for the {@link ITreeLayoutNodePlacer}
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase Online Documentation}
@protected*/
constructor(options: {
modificationMatrix: RotatableNodePlacerMatrix | null;
spacing?: number;
});
/**
Creates a {@link IComparer} for outgoing edges that takes the rotation into account.
@returns the {@link IComparer} that considers the rotation
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23createComparer() Online Documentation}*/
createComparer(): IComparer<any> | null;
/**
Delegates to {@link RotatableNodePlacerBase#createComparer}.
@returns the From Sketch {@link IComparer}
@see RotatableNodePlacerBase#createComparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23createFromSketchComparer() Online Documentation}*/
createFromSketchComparer(): IComparer<any> | null;
/**
Creates an optional {@link IProcessor} for pre- and post-processing.
This implementation returns <code>null</code>. Hence, no pre- or post-processing is done.
@param layouter the current {@link TreeLayout} instance
@param graph the input graph
@param currentRoot the root node handled by this {@link ITreeLayoutNodePlacer}
@returns <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23createProcessor(TreeLayout%2CLayoutGraph%2CYNode) Online Documentation}*/
createProcessor(layouter: TreeLayout | null, graph: LayoutGraph | null, currentRoot: YNode | null): IProcessor | null;
/**
Determines a connector direction for the given child node.
This method is called by {@link RotatableNodePlacerBase#determineChildConnectors} and must be implemented by
subclasses. Implementations can assume the default direction (top-to-bottom) for their arrangement.
@param child the child node
@returns a direction specifier as defined in the {@link ITreeLayoutNodePlacer} interface
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23determineChildConnector(YNode) Online Documentation}*/
abstract determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Provides the direction of the connector to the {@link SubtreeShape SubtreeShape} for each child node.
This implementation uses {@link RotatableNodePlacerBase#determineChildConnector} and the {@link RotatableNodePlacerBase#modificationMatrix modification matrix} to
retrieve the direction for the connectors.
@param localRoot the local root
@param connectorMap the map that must be used for storing the direction specifiers of the child nodes
@see RotatableNodePlacerBase#determineChildConnector
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23determineChildConnectors(YNode%2CIDataMap) Online Documentation}*/
determineChildConnectors(localRoot: YNode | null, connectorMap: IDataMap | null): void;
/**
Returns the rotated subtree shape for the given node.
This is a convenience method that retrieves a rotated subtree shape for a single node from the corresponding
{@link IDataProvider}. This shape may be modified and returned by
{@link RotatableNodePlacerBase#placeSubtreeOfNode}.
<b>Warning:</b> This method can first be called during the execution of {@link RotatableNodePlacerBase#placeSubtreeOfNode}.
Otherwise, the {@link IDataProvider} for querying results will not have been initialized yet.
@param node the node
@returns the subtree shape for the given node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23getNodeShape(YNode) Online Documentation}*/
getNodeShape(node: YNode | null): RotatedSubtreeShape | null;
/**
Returns the target {@link PortConstraint} for the local root's incoming edge.
This method translates the {@link PortConstraint} according to the rotation described by the {@link RotatableNodePlacerBase#modificationMatrix modification matrix}.
It should be used instead of {@link PortConstraint#getTPC}.
This method may be used in {@link RotatableNodePlacerBase#placeSubtreeOfNode} to route the edges to the according
port. It may be overridden to provide constant {@link PortConstraint}s for each node.
@param localRoot the local root
@returns the {@link PortConstraint} for the incoming edge of the local root or {@link PortSide#ANY} if no {@link PortConstraint}
is assigned for that edge
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23getPortConstraint(YNode) Online Documentation}*/
getPortConstraint(localRoot: YNode | null): PortConstraint | null;
/**
Returns the absolute source point of the given edge depending on the rotation of this {@link ITreeLayoutNodePlacer}.
This method translates the source point according to the rotation described by the {@link RotatableNodePlacerBase#modificationMatrix modification matrix}. It should be
used instead of {@link LayoutGraph#getSourcePointAbs}.
It can be used in an implementation of {@link RotatableNodePlacerBase#placeSubtreeOfNode}.
@param edge the edge whose absolute source point should be calculated
@returns the translated absolute source point for the given edge and current modification matrix
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23getSourcePointAbs(Edge) Online Documentation}*/
getSourcePointAbs(edge: Edge | null): YPoint | null;
/**
Returns the rotated subtree shape for the subtree rooted at the given node.
This is a convenience method that retrieves a subtree shape of a complete subtree from the corresponding
{@link IDataProvider}.
<b>Warning:</b> This method can first be called during the execution of {@link RotatableNodePlacerBase#placeSubtreeOfNode}. Before,
the {@link IDataProvider} for querying results will not have been initialized yet.
@param node the local root node
@returns the rotated subtree shape of the subtree rooted at the given local root node
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23getSubtreeShape(YNode) Online Documentation}*/
getSubtreeShape(node: YNode | null): RotatedSubtreeShape | null;
/**
Initializes the local data structures and then delegates the work to the abstract variant of this method
({@link RotatableNodePlacerBase#placeSubtreeOfNode}).
@param nodeShapeProvider the {@link IDataProvider} for obtaining an initial shape of the root node
@param subtreeShapeProvider the {@link IDataProvider} for accessing the pre-calculated shapes of the subtrees
@param graph the input graph
@param localRoot the root of the subtree that should be arranged by this method
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged {@link SubtreeShape} for the subtree rooted at the local root node
@see RotatableNodePlacerBase#placeSubtreeOfNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23placeSubtree(IDataProvider%2CIDataProvider%2CLayoutGraph%2CYNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtree(nodeShapeProvider: IDataProvider | null, subtreeShapeProvider: IDataProvider | null, graph: LayoutGraph | null, localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection): SubtreeShape | null;
/**
Arranges the local root relative to the rotated subtree shapes of its children.
This is the main method that must be implemented by subclasses. For calculations and locations subclasses should use the
corresponding methods of {@link RotatableNodePlacerBase} which already consider the {@link RotatableNodePlacerBase#modificationMatrix modification matrix}.
@param localRoot the local root node
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged subtree shape that contains the local root node and all its children
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
abstract placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
/**
Translates the given real direction of the resulting layout to the model direction of this
{@link RotatableNodePlacerBase}.
This method is used in {@link RotatableNodePlacerBase#placeSubtreeOfNode} to be able to retrieve the model direction
for the connector
@param realDirection the real direction
@returns the model direction
@throws {Exception} <b>(ArgumentError)</b> if translation for the given real direction is not implemented
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23translateDirectionToModel(ParentConnectorDirection) Online Documentation}*/
translateDirectionToModel(realDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): number;
/**
Translates the given model direction to the real direction in the resulting layout.
This method is used in {@link RotatableNodePlacerBase#determineChildConnectors} to be able to return the real
connector direction.
@param modelDirection the model direction
@returns the real direction
@throws {Exception} <b>(ArgumentError)</b> if translation for the given model direction is not implemented
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23translateDirectionToReal(ParentConnectorDirection) Online Documentation}*/
translateDirectionToReal(modelDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): number;
/**
Translates a point from the real layout direction to the model direction using the given modification matrix.
@param modificationMatrix the modification matrix to use for the translation
@param realWorldPoint the point with coordinates from the real world
@returns the translated point with coordinates from the model
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23translatePoint(RotatableNodePlacerMatrix%2CYPoint) Online Documentation}*/
static translatePoint(modificationMatrix: RotatableNodePlacerMatrix | null, realWorldPoint: YPoint | null): YPoint | null;
/**
The graph instance this {@link ITreeLayoutNodePlacer} is working on.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23graph Online Documentation}*/
graph: LayoutGraph | null;
/**
The provider of {@link SubtreeShape}s for whole subtrees.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23subtreeShapeProvider Online Documentation}*/
subtreeShapeProvider: IDataProvider | null;
/**
The provider of {@link SubtreeShape}s for single nodes.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23nodeShapeProvider Online Documentation}*/
nodeShapeProvider: IDataProvider | null;
/**
The list containing the created children.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23createdChildren Online Documentation}*/
createdChildren: IList<any> | null;
/**
Gets the modification matrix that defines the direction of the subtree.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23modificationMatrix Online Documentation}*/
readonly modificationMatrix: RotatableNodePlacerMatrix | null;
/**
Gets or sets the spacing between subtrees.
The spacing needs to be non-negative.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatableNodePlacerBase%23spacing Online Documentation}*/
spacing: number;
static $class: Class;
private $_RotatableNodePlacerBase: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RotatableNodePlacerBase;
}
export interface RotatedSubtreeShape extends YObject {
}
/**
Decorates a {@link SubtreeShape} and provides rotated access on it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatedSubtreeShape Online Documentation}*/
export class RotatedSubtreeShape {
/**
For testing purposes only
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/RotatedSubtreeShape%23RotatedSubtreeShape() Online Documentation}*/
constructor();
static $class: Class;
private $_RotatedSubtreeShape: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RotatedSubtreeShape;
}
export interface SimpleNodePlacer extends RotatableNodePlacerBase {
}
/**
The {@link SimpleNodePlacer} arranges all children of a local root in a single row.
<b>Layout Style</b>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNodePlacer Online Documentation}*/
export class SimpleNodePlacer {
/**
Creates a new {@link SimpleNodePlacer} instance that uses the given {@link RotatableNodePlacerBase#modificationMatrix modification matrix} and {@link SimpleNodePlacer#rootAlignment root alignment}.
@param [modificationMatrix=null] the transformation matrix for this {@link RotatableNodePlacerBase}
@param [rootAlignment=null] the alignment for the local root in relation to its subtrees
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNodePlacer%23SimpleNodePlacer(RotatableNodePlacerMatrix%2CRootNodeAlignment) Online Documentation}*/
constructor(modificationMatrix?: RotatableNodePlacerMatrix | null, rootAlignment?: RootNodeAlignment | null);
/**
Creates a new {@link SimpleNodePlacer} instance that uses the given {@link RotatableNodePlacerBase#modificationMatrix modification matrix} and {@link SimpleNodePlacer#rootAlignment root alignment}.
@param options The parameters to pass.
@param [options.modificationMatrix=null] the transformation matrix for this {@link RotatableNodePlacerBase}
@param [options.rootAlignment=null] the alignment for the local root in relation to its subtrees
@param [options.minimumChannelSegmentDistance] The minimum (non-negative) distance between edge segments that are routed orthogonally in the channel between the root node and the child nodes. This option sets the {@link SimpleNodePlacer#minimumChannelSegmentDistance} property on the created object.
@param [options.verticalAlignment] The relative vertical alignment of nodes with the same parent. This option sets the {@link SimpleNodePlacer#verticalAlignment} property on the created object.
@param [options.createBus] Whether or not a bus is created in the space between the parent and its children. This option sets the {@link SimpleNodePlacer#createBus} property on the created object.
@param [options.spacing] The spacing between subtrees. This option sets the {@link RotatableNodePlacerBase#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNodePlacer%23SimpleNodePlacer(RotatableNodePlacerMatrix%2CRootNodeAlignment) Online Documentation}*/
constructor(options: {
modificationMatrix?: RotatableNodePlacerMatrix | null;
rootAlignment?: RootNodeAlignment | null;
minimumChannelSegmentDistance?: number;
verticalAlignment?: number;
createBus?: boolean;
spacing?: number;
});
/**
Provides {@link ParentConnectorDirection#NORTH} as connector direction for the child nodes.
@param child the child node
@returns {@link ParentConnectorDirection#NORTH}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNodePlacer%23determineChildConnector(YNode) Online Documentation}*/
determineChildConnector(child: YNode | null): ParentConnectorDirection;
/**
Places the children of the local root node in a single row.
@param localRoot the local root node
@param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
@returns the merged subtree shape containing the local root node and all its children
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNodePlacer%23placeSubtreeOfNode(YNode%2CParentConnectorDirection) Online Documentation}*/
placeSubtreeOfNode(localRoot: YNode | null, parentConnectorDirection: ParentConnectorDirection | ParentConnectorDirectionStringValues): RotatedSubtreeShape | null;
/**
Gets or sets the minimum (non-negative) distance between edge segments that are routed orthogonally in the channel
between the root node and the child nodes.
If this distance is set to <code>0</code>, all segments are routed in a bus-like style.
<h2>Note</h2>
The specified distance is only considered if {@link SimpleNodePlacer#createBus} is enabled.
<h2>Note</h2>
Values greater than <code>0</code> are especially useful if the {@link ITreeLayoutPortAssignment} distributes the ports at the nodes'
border.
@throws {Exception} <b>(ArgumentError)</b> if the specified distance is smaller than <code>0</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNodePlacer%23minimumChannelSegmentDistance Online Documentation}*/
minimumChannelSegmentDistance: number;
/**
Gets or sets the {@link RootNodeAlignment RootAlignment} that defines how the root node is aligned with its children and their shapes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNodePlacer%23rootAlignment Online Documentation}*/
rootAlignment: RootNodeAlignment | null;
/**
Gets or sets the relative vertical alignment of nodes with the same parent.
All values between <code>0</code> and <code>1</code> result in a valid alignment.
<ul>
<li><code>0</code> - nodes are top-aligned
<li><code>0.5</code> - nodes are center-aligned
<li><code>1</code> - nodes are bottom-aligned
</ul>
Values outside the interval <code>[0,1]</code> may result in a more compact node placement with unaligned nodes, especially when
labels are considered.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNodePlacer%23verticalAlignment Online Documentation}*/
verticalAlignment: number;
/**
Gets or sets whether or not a bus is created in the space between the parent and its children.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SimpleNodePlacer%23createBus Online Documentation}*/
createBus: boolean;
static $class: Class;
private $_SimpleNodePlacer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SimpleNodePlacer;
}
export interface TreeComponentLayout extends YObject, ILayoutStage {
}
/**
The {@link TreeComponentLayout} arranges tree-like subgraph structures in a mixed layout style.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout Online Documentation}*/
export class TreeComponentLayout {
/**
Creates a new {@link TreeComponentLayout} instance using the specified layouter for arranging the subtrees.
@param coreLayout the layout algorithm used for arranging the tree components
@throws {Exception} <b>(ArgumentError)</b> if the specified layout algorithm is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout Online Documentation}*/
constructor(coreLayout: ILayoutAlgorithm | null);
/**
Creates a new {@link TreeComponentLayout} instance using the specified layouter for arranging the subtrees.
@param options The parameters to pass.
@param options.coreLayout the layout algorithm used for arranging the tree components
@param [options.treeComponentCoreLayout] The layout algorithm that is applied to tree components. This option sets the {@link TreeComponentLayout#treeComponentCoreLayout} property on the created object.
@param [options.optimizeOrientation] Whether or not tree component arrangement is improved by changing the orientation of components. This option sets the {@link TreeComponentLayout#optimizeOrientation} property on the created object.
@param [options.considerUndirectedTrees] Whether or not <em>undirected</em> tree sub-graphs should be detected and handled. This option sets the {@link TreeComponentLayout#considerUndirectedTrees} property on the created object.
@throws {Exception} <b>(ArgumentError)</b> if the specified layout algorithm is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout Online Documentation}*/
constructor(options: {
coreLayout: ILayoutAlgorithm | null;
treeComponentCoreLayout?: ILayoutAlgorithm | null;
optimizeOrientation?: boolean;
considerUndirectedTrees?: boolean;
});
/**
Finds the tree components inside the given graph and applies a {@link TreeComponentLayout#treeComponentCoreLayout tree layout} to them.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Delegates the layout calculation to the {@link TreeComponentLayout#coreLayout core layout algorithm} and handles the dummy nodes.
The dummy nodes are marked using the given {@link IDataProvider}.
This implementation just registers the dummy node {@link IDataProvider} with the graph with key
{@link TreeComponentLayout#DUMMY_NODE_DP_KEY}, executes the core layout algorithm and finally removes the data
provider again.
This method is called by {@link TreeComponentLayout#applyLayout} to invoke the core layout algorithm. It may be
overridden to customize the handling of the dummy nodes.
@param graph the input graph
@param dummyDp the {@link IDataProvider} which marks the dummy nodes
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout%23applyLayoutUsingDummies(LayoutGraph%2CIDataProvider) Online Documentation}*/
applyLayoutUsingDummies(graph: LayoutGraph | null, dummyDp: IDataProvider | null): void;
/**
Data provider key for identifying dummy nodes during the layout calculation
This {@link IDataProvider} is registered and also removed in
{@link TreeComponentLayout#applyLayoutUsingDummies}.
@see TreeComponentLayout#applyLayoutUsingDummies
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout%23DUMMY_NODE_DP_KEY Online Documentation}*/
static DUMMY_NODE_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the core layout algorithm.
This algorithm is wrapped by this stage. It is invoked in {@link ILayoutAlgorithm#applyLayout}. The {@link ILayoutStage}
may add pre- and post-processing steps before and after calling the core layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout%23coreLayout Online Documentation}*/
coreLayout: ILayoutAlgorithm | null;
/**
Gets or sets the layout algorithm that is applied to tree components.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout%23treeComponentCoreLayout Online Documentation}*/
treeComponentCoreLayout: ILayoutAlgorithm | null;
/**
Gets or sets whether or not tree component arrangement is improved by changing the orientation of components.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout%23optimizeOrientation Online Documentation}*/
optimizeOrientation: boolean;
/**
Gets or sets whether or not <em>undirected</em> tree sub-graphs should be detected and handled.
When enabled, the direction of the edges that define a subtree is not relevant. Therefore, also undirected trees are
detected and handled as a tree component. Otherwise, only trees that are consistently directed from the root are handled
({@link TreeAlgorithm#isRootedTree directed rooted trees}).
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeComponentLayout%23considerUndirectedTrees Online Documentation}*/
considerUndirectedTrees: boolean;
static $class: Class;
private $_TreeComponentLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeComponentLayout;
}
export interface SubtreeShape extends YObject {
}
/**
A {@link SubtreeShape} represents the position and the borders of a subtree.
{@link ITreeLayoutNodePlacer}s will move {@link SubtreeShape}s and use the borders to calculate distances between the
subtrees. When two subtrees are placed in relation to their common root, their
{@link SubtreeShape}s are merged to represent the new subtree.
The borders on each side of the {@link SubtreeShape} are modeled with
{@link BorderLine}s. The resulting area includes the borders of each node and edge in the corresponding
subtree. It can also contain the bounds of labels belonging to those nodes and edges.
The {@link SubtreeShape} also describes a connector defining the location to which the edge from its parent is
attached. It is possible to add some bends (target points) at the end of this edge and include them in the
{@link SubtreeShape}. During layout calculation, the last target point is the connection point for the edge.
@see SubtreeShape#getBorderLine
@see SubtreeShape#connectorX
@see SubtreeShape#connectorY
@see SubtreeShape#addBoundsToShape
@see SubtreeShape#addTargetPoint
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape Online Documentation}*/
export class SubtreeShape {
/**
Extends this {@link SubtreeShape} instance by the given rectangle.
The border lines are updated with this segment.
@param x the x-coordinate of the upper-left corner of the rectangle
@param y the y-coordinate of the upper-left corner of the rectangle
@param width the width of the rectangle
@param height the height of the rectangle
@see SubtreeShape#getBorderLine
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23addBoundsToShape(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
addBoundsToShape(x: number, y: number, width: number, height: number): void;
/**
Adds all edge segments of the given edge to this {@link SubtreeShape} instance.
The border lines are updated.
@param graph the input graph
@param edge the given edge
@see SubtreeShape#getBorderLine
@see SubtreeShape#addLineSegment
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23addEdgeSegments(LayoutGraph%2CEdge) Online Documentation}*/
addEdgeSegments(graph: LayoutGraph | null, edge: Edge | null): void;
/**
Extends this {@link SubtreeShape} instance by the given segment.
The border lines are updated with this segment.
@param x1 the x-coordinate of the first point of the segment
@param y1 the y-coordinate of the first point of the segment
@param x2 the x-coordinate of the second point of the segment
@param y2 the y-coordinate of the second point of the segment
@see SubtreeShape#getBorderLine
@see SubtreeShape#addEdgeSegments
@see SubtreeShape#updateConnectorShape
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23addLineSegment(number%2Cnumber%2Cnumber%2Cnumber) Online Documentation}*/
addLineSegment(x1: number, y1: number, x2: number, y2: number): void;
/**
Adds another point to the connector.
The points will appear as bend of the incoming edge. Points are added in reverse direction of the edge. Hence, the last
target point is further away from the target node than the points before.
@param x the current x-coordinate of the point
@param y the current y-coordinate of the point
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23addTargetPoint(number%2Cnumber) Online Documentation}*/
addTargetPoint(x: number, y: number): void;
/**
Appends all target points that have previously been added to this shape to the given {@link IEdgeLayout}
instance.
This method is used by {@link ITreeLayoutNodePlacer} instances for the final edge paths.
@param edgeLayout the {@link IEdgeLayout} that will be modified
@see SubtreeShape#addTargetPoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23appendTargetPoints(IEdgeLayout) Online Documentation}*/
appendTargetPoints(edgeLayout: IEdgeLayout | null): void;
/**
Copies the state of this {@link SubtreeShape} instance to the given shape applying a modification matrix.
@param toShape the given shape that gets the assigned values
@param modificationMatrix the matrix that may add geometrical modifications
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23assignValuesTo(SubtreeShape%2CRotatableNodePlacerMatrix) Online Documentation}*/
assignValuesTo(toShape: SubtreeShape | null, modificationMatrix: RotatableNodePlacerMatrix | null): void;
/**
Creates a clone of this {@link SubtreeShape} instance, modifying it using the provided modification matrix.
@param matrix the matrix that defines the modification for the new instance
@returns the clone of this {@link SubtreeShape} instance
@see RotatableNodePlacerMatrix#DEFAULT
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23createCopy(RotatableNodePlacerMatrix) Online Documentation}*/
createCopy(matrix: RotatableNodePlacerMatrix | null): SubtreeShape | null;
/**
Returns the {@link BorderLine} instance that describes the {@link SubtreeShape} at the given side.
@param index one of the predefined borderline directions
@returns the {@link BorderLine} instance for the given side
@throws {Exception} <b>(ArgumentError)</b> if the given index is invalid
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23getBorderLine(number) Online Documentation}*/
getBorderLine(index: number): BorderLine | null;
/**
Merges the given {@link SubtreeShape} instance with this {@link SubtreeShape} instance.
The border lines are updated.
@param other the {@link SubtreeShape} to be merged with this {@link SubtreeShape}
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23mergeWith(SubtreeShape) Online Documentation}*/
mergeWith(other: SubtreeShape | null): void;
/**
Moves this {@link SubtreeShape} instance and its connector by the given offsets.
Border lines are updated.
<h2>Note</h2>
This is a cheap operation and does not depend on the size or complexity of the shape or connector.
@param dx the delta x-offset by which this shape will be moved
@param dy the delta y-offset by which this shape will be moved
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23move(number%2Cnumber) Online Documentation}*/
move(dx: number, dy: number): void;
/**
Returns a string representation of the {@link SubtreeShape} instance.
@returns a string representation of the {@link SubtreeShape} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23toString() Online Documentation}*/
toString(): string;
/**
Adds the line segments of the connector to the bounds of this {@link SubtreeShape} instance.
The line segments result from the target points.
@see SubtreeShape#addTargetPoint
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23updateConnectorShape() Online Documentation}*/
updateConnectorShape(): void;
/**
Gets the bounds of the {@link INodeLayout} of the root node of this {@link SubtreeShape} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23coreBounds Online Documentation}*/
readonly coreBounds: Rectangle2D | null;
/**
Gets the current bounds of this {@link SubtreeShape} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23bounds Online Documentation}*/
readonly bounds: Rectangle2D | null;
/**
Gets the current x-coordinate of the connection point to which the layout algorithm connects the ingoing edge.
The connection point can change when target points are added to the {@link SubtreeShape} instance.
@see SubtreeShape#addTargetPoint
@see SubtreeShape#appendTargetPoints
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23connectorX Online Documentation}*/
readonly connectorX: number;
/**
Gets the current y-coordinate of the connection point to which the layout algorithm connects the ingoing edge.
The connection point can change when target points are added to the {@link SubtreeShape} instance.
@see SubtreeShape#addTargetPoint
@see SubtreeShape#appendTargetPoints
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23connectorY Online Documentation}*/
readonly connectorY: number;
/**
Gets the direction of the last connector segment.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23connectorDirection Online Documentation}*/
readonly connectorDirection: ConnectorDirection;
/**
Gets the current minimum x-coordinate of this {@link SubtreeShape} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23minX Online Documentation}*/
readonly minX: number;
/**
Gets the current minimum y-coordinate of this {@link SubtreeShape} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23minY Online Documentation}*/
readonly minY: number;
/**
Gets the current maximum x-coordinate of this {@link SubtreeShape} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23maxX Online Documentation}*/
readonly maxX: number;
/**
Gets the current maximum y-coordinate of this {@link SubtreeShape} instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23maxY Online Documentation}*/
readonly maxY: number;
/**
Gets the current x-coordinate of the origin of this {@link SubtreeShape} instance.
The origin is defined as the upper left corner of the {@link INodeLayout} of the local root of this shape.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23originX Online Documentation}*/
readonly originX: number;
/**
Gets the current y-coordinate of the origin of this {@link SubtreeShape} instance.
The origin is defined as the upper left corner of the {@link INodeLayout} of the local root of this shape.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubtreeShape%23originY Online Documentation}*/
readonly originY: number;
static $class: Class;
private $_SubtreeShape: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SubtreeShape;
}
export interface TreeLayout extends MultiStageLayout {
}
/**
This layout algorithm arranges graphs with a tree structure.
<b>Layout Style</b>
{@link TreeLayout} provides multiple different arrangements of trees and subtrees. It is easy to customize the
{@link TreeLayout#OUT_EDGE_COMPARER_DP_KEY order of edges}, the {@link TreeLayout#PORT_ASSIGNMENT_DP_KEY port assignment} and the {@link TreeLayout#NODE_PLACER_DP_KEY arrangement of the nodes} for each subtree.
Tree layout algorithms are commonly applied to visualize relational data and produce diagrams of high quality that are
able to reveal possible hierarchic properties of the graph. More precisely, they find applications in dataflow analysis,
software engineering, bioinformatics and business administration.
<b>Concept</b>
The layout algorithm starts from the leaves and continues with their parents, then with the parents of the parents and
so on. When a node is processed, the algorithm will use the corresponding {@link ITreeLayoutNodePlacer} instance to move its
children (along with their subtrees) to a suitable position and to route the outgoing edges of this node. Then, the next
local root node will be processed.
To avoid moving all nodes several times and to know the area that the subtrees occupy, the layout algorithm uses
{@link SubtreeShape}s. These {@link SubtreeShape}s are moved and merged during layout calculation.
<b>Features</b>
Each subtree can have a different style of node placement. {@link ITreeLayoutNodePlacer}s are responsible for arranging
subtrees and their common root node. They can be specified separately for each local root with a {@link IDataProvider}
registered with the graph using key {@link TreeLayout#NODE_PLACER_DP_KEY}.
A custom node can be defined as root of the tree using a {@link IDataProvider} registered with the graph with
key {@link TreeLayout#SELECTED_ROOT_DP_KEY}.
The layout algorithm can be configured to reserve space for node labels and place the edge labels along edges such that
the labels won't overlap with other graph elements. Edge labels are placed according to the information stored in a {@link PreferredPlacementDescriptor}
instance. However, the placement along the edge will only affect the order of multiple labels at the same edge. The
algorithm will always place the labels close to the target node.
Grouping of nodes can also be handled by this layout algorithm. It is important that a group node contains a whole
subtree. Otherwise, the group nodes may overlap with each other or with other nodes. Furthermore, the user may specify
minimum size constraints for each group node using {@link IDataProvider} key
{@link GroupingKeys#MINIMUM_NODE_SIZE_DP_KEY}.
{@link TreeLayout} supports custom sorting of the outgoing edges of a node. For example, a {@link ITreeLayoutNodePlacer}
instance that implements {@link IFromSketchNodePlacer} provides a comparator that keeps the current order of
siblings, allowing to extend the graph incrementally.
This layout algorithm can only handle graphs with a tree structure. To apply it to a general graph, a {@link TreeReductionStage}
can be appended. This stage will temporarily remove some edges of the input graph until a tree is obtained. After the
layout calculation, the stage will reinsert the edges that were removed and route them separately.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout Online Documentation}*/
export class TreeLayout {
/**
Creates a new {@link TreeLayout} instance with default settings.
@param [options] The parameters to pass.
@param [options.groupingSupported] Whether or not group nodes are handled by the layout algorithm. This option sets the {@link TreeLayout#groupingSupported} property on the created object.
@param [options.componentLayoutEnabled] Whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated. This option sets the {@link TreeLayout#componentLayoutEnabled} property on the created object.
@param [options.parallelEdgeRouterEnabled] Whether or not the {@link ILayoutStage} used for routing parallel edges is activated. This option sets the {@link TreeLayout#parallelEdgeRouterEnabled} property on the created object.
@param [options.selfLoopRouterEnabled] Whether or not the {@link ILayoutStage} used for routing self-loops is activated. This option sets the {@link TreeLayout#selfLoopRouterEnabled} property on the created object.
@param [options.defaultNodePlacer] The default {@link ITreeLayoutNodePlacer} instance that arranges all subtrees that do not have a specific {@link ITreeLayoutNodePlacer} assigned using a {@link IDataProvider}. This option sets the {@link TreeLayout#defaultNodePlacer} property on the created object.
@param [options.defaultLeafPlacer] The default {@link ITreeLayoutNodePlacer} instance that places the leaf nodes of the tree. This option sets the {@link TreeLayout#defaultLeafPlacer} property on the created object.
@param [options.defaultPortAssignment] The default {@link ITreeLayoutPortAssignment} instance for all subtrees that do not have a specific {@link ITreeLayoutPortAssignment} assigned using a {@link IDataProvider}. This option sets the {@link TreeLayout#defaultPortAssignment} property on the created object.
@param [options.defaultOutEdgeComparer] The default {@link IComparer} instance that sorts the outgoing edges in all subtrees that do not have a specific {@link IComparer} assigned using a {@link IDataProvider}. This option sets the {@link TreeLayout#defaultOutEdgeComparer} property on the created object.
@param [options.considerNodeLabels] Whether or not the layout algorithm reserves space for node labels. This option sets the {@link TreeLayout#considerNodeLabels} property on the created object.
@param [options.integratedEdgeLabeling] Whether or not edge labels are placed by the layout algorithm. This option sets the {@link TreeLayout#integratedEdgeLabeling} property on the created object.
@param [options.multiParentAllowed] Whether or not multi-parent structures are allowed for this tree layout. This option sets the {@link TreeLayout#multiParentAllowed} property on the created object.
@param [options.labeling] The {@link ILayoutStage} that places the labels of the input graph. This option sets the {@link MultiStageLayout#labeling} property on the created object.
@param [options.selfLoopRouter] The {@link ILayoutStage} that routes self-loops. This option sets the {@link MultiStageLayout#selfLoopRouter} property on the created object.
@param [options.parallelEdgeRouter] The {@link ILayoutStage} that routes parallel edges. This option sets the {@link MultiStageLayout#parallelEdgeRouter} property on the created object.
@param [options.componentLayout] The {@link ILayoutStage} that arranges the connected components of an input graph. This option sets the {@link MultiStageLayout#componentLayout} property on the created object.
@param [options.subgraphLayout] The {@link ILayoutStage} that constrains the layout process to a subgraph of the input graph. This option sets the {@link MultiStageLayout#subgraphLayout} property on the created object.
@param [options.hideGroupsStage] The {@link ILayoutStage} that hides the group nodes of the input graph. This option sets the {@link MultiStageLayout#hideGroupsStage} property on the created object.
@param [options.orientationLayout] The {@link ILayoutStage} that modifies the orientation of a computed layout. This option sets the {@link MultiStageLayout#orientationLayout} property on the created object.
@param [options.orientationLayoutEnabled] Whether or not the {@link ILayoutStage} that modifies the orientation of the layout is activated. This option sets the {@link MultiStageLayout#orientationLayoutEnabled} property on the created object.
@param [options.layoutOrientation] The main orientation of the layout. This option sets the {@link MultiStageLayout#layoutOrientation} property on the created object.
@param [options.labelingEnabled] Whether or not the {@link ILayoutStage} used for placing the labels of the input graph is activated. This option sets the {@link MultiStageLayout#labelingEnabled} property on the created object.
@param [options.hideGroupsStageEnabled] Whether or not the {@link ILayoutStage} used for hiding group nodes is activated. This option sets the {@link MultiStageLayout#hideGroupsStageEnabled} property on the created object.
@param [options.subgraphLayoutEnabled] Whether or not the {@link ILayoutStage} used for constraining the layout process to a subgraph of the input graph is activated. This option sets the {@link MultiStageLayout#subgraphLayoutEnabled} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout Online Documentation}*/
constructor(options?: {
groupingSupported?: boolean;
componentLayoutEnabled?: boolean;
parallelEdgeRouterEnabled?: boolean;
selfLoopRouterEnabled?: boolean;
defaultNodePlacer?: ITreeLayoutNodePlacer | null;
defaultLeafPlacer?: ITreeLayoutNodePlacer | null;
defaultPortAssignment?: ITreeLayoutPortAssignment | null;
defaultOutEdgeComparer?: IComparer<any> | null;
considerNodeLabels?: boolean;
integratedEdgeLabeling?: boolean;
multiParentAllowed?: boolean;
labeling?: ILayoutStage | null;
selfLoopRouter?: ILayoutStage | null;
parallelEdgeRouter?: ILayoutStage | null;
componentLayout?: ILayoutStage | null;
subgraphLayout?: ILayoutStage | null;
hideGroupsStage?: ILayoutStage | null;
orientationLayout?: ILayoutStage | null;
orientationLayoutEnabled?: boolean;
layoutOrientation?: LayoutOrientation | LayoutOrientationStringValues;
labelingEnabled?: boolean;
hideGroupsStageEnabled?: boolean;
subgraphLayoutEnabled?: boolean;
});
/**
Arranges the given graph as a tree.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23applyLayoutCore(LayoutGraph) Online Documentation}*/
applyLayoutCore(graph: LayoutGraph | null): void;
/**
Creates a {@link SubtreeShape} for the given node that consists only of the bounds of this particular node.
It may also include node labels or {@link NodeHalo}s.
This method may be overridden to return a custom {@link SubtreeShape} for the given node.
@param node the given node
@returns a {@link SubtreeShape} instance
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23createNodeShape(YNode) Online Documentation}*/
createNodeShape(node: YNode | null): SubtreeShape | null;
/**
Returns a list of edges that need to be reversed in order to obtain a valid rooted and directed tree from the input
graph.
The root node of the tree is either a node marked by a {@link IDataProvider} registered with the graph with
key {@link TreeLayout#SELECTED_ROOT_DP_KEY} or is defined according to {@link TreeAlgorithm#getRoot}.
This method is called initially to calculate a rooted tree from the input graph. It may be overridden to apply a custom
algorithm that determines which edges need to be reversed.
@returns a list of edges that need to be reversed
@see TreeLayout#reverseEdges
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23directTree() Online Documentation}*/
directTree(): EdgeList | null;
/**
Returns the {@link ITreeLayoutNodePlacer} instance that is used for the placement of the local root node and the
{@link SubtreeShape}s.
The method may be overridden to return custom {@link ITreeLayoutNodePlacer} instances. It is possible to return a shared
instance for multiple different nodes because the instances are not used after subsequent calls to this method.
The current implementation returns the {@link ITreeLayoutNodePlacer} defined by the {@link IDataProvider} registered
with key {@link TreeLayout#NODE_PLACER_DP_KEY}. It falls back to the default {@link ITreeLayoutNodePlacer} if there is no
specific {@link ITreeLayoutNodePlacer} for the given node.
@param localRoot the root of the local subtree
@returns the {@link ITreeLayoutNodePlacer} instance that places the subtree below the given local root node
@see TreeLayout#NODE_PLACER_DP_KEY
@see TreeLayout#defaultNodePlacer
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23getNodePlacer(YNode) Online Documentation}*/
getNodePlacer(localRoot: YNode | null): ITreeLayoutNodePlacer | null;
/**
Returns the {@link IComparer} instance that will sort the outgoing edges connecting to the given node.
The {@link IComparer} can be <code>null</code> in case the initial edge order shall be used.
The method may be overridden to return custom {@link IComparer} instances. It is possible to return a shared instance for multiple
different nodes because the instances are not used after subsequent calls to this method.
The current implementation returns the {@link IComparer} defined in the {@link IDataProvider} registered with key
{@link TreeLayout#OUT_EDGE_COMPARER_DP_KEY}. It falls back to the default {@link IComparer} if there is no specific {@link IComparer} for the given
node.
@param localRoot the root of the local subtree
@returns the {@link IComparer} or <code>null</code> that sorts the outgoing edges of the given node
@see TreeLayout#OUT_EDGE_COMPARER_DP_KEY
@see TreeLayout#defaultOutEdgeComparer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23getOutEdgeComparer(YNode) Online Documentation}*/
getOutEdgeComparer(localRoot: YNode | null): IComparer<any> | null;
/**
Returns the {@link ITreeLayoutPortAssignment} instance that places the ports of the connecting edges of the given node.
The method may be overridden to return customized {@link ITreeLayoutPortAssignment} instances. It is possible to return a
shared instance for multiple different nodes because the instances are not used after subsequent calls to this method.
The current implementation returns the {@link ITreeLayoutPortAssignment} defined by the {@link IDataProvider}
registered with key {@link TreeLayout#OUT_EDGE_COMPARER_DP_KEY}. It falls back to the default {@link ITreeLayoutPortAssignment}
if there is no specific {@link ITreeLayoutPortAssignment} for the given node.
@param localRoot the root of the local subtree
@returns the {@link ITreeLayoutPortAssignment} instance that assigns the ports of the edges at the given node
@see TreeLayout#PORT_ASSIGNMENT_DP_KEY
@see TreeLayout#defaultPortAssignment
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23getPortAssignment(YNode) Online Documentation}*/
getPortAssignment(localRoot: YNode | null): ITreeLayoutPortAssignment | null;
/**
Returns an array of the nodes that will be laid out.
The given node will be considered as the root of the tree. The order of the elements ensures that no parent is processed
before one of its successors.
This method may be overridden to change the order in which the nodes (and their subtrees) are handled. However, it is
important to keep the parents after the successors. Only siblings can change places.
@param root the node to be considered as root of the tree
@returns an array of nodes that will be laid out
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23getRootsArray(YNode) Online Documentation}*/
getRootsArray(root: YNode | null): YNode[] | null;
/**
Provides access to the {@link SubtreeShape} for the given node.
The {@link SubtreeShape} contains information about the current extent and location of the subtree rooted at the
node. It should only be modified during the layout of the parent node. Also, it won't be initialized before the layout
of the corresponding subtree is calculated.
This method updates the {@link SubtreeShape} in {@link TreeLayout#layoutRoot}.
@param localRoot the root of the subtree
@returns the {@link SubtreeShape} instance if it has already been calculated, <code>null</code> otherwise
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23getSubtreeShape(YNode) Online Documentation}*/
getSubtreeShape(localRoot: YNode | null): SubtreeShape | null;
/**
Calculates the layout for the given root node and its subtrees.
This method is invoked for each node in the tree exactly once. Thus, children are always handled before their parents.
In this manner, the subtrees already have a layout and can be arranged with their parent.
To retrieve the shapes of the subtrees of all children of the local root, this method uses
{@link TreeLayout#getSubtreeShape}.
The method may be overridden to add configuration code.
@param localRoot the root of the subtree that whose layout is calculated
@returns the combined {@link SubtreeShape} of the local root node and all of its children and connecting edges
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23layoutRoot(YNode) Online Documentation}*/
layoutRoot(localRoot: YNode | null): SubtreeShape | null;
/**
Reverses the direction of given edges.
This method will be called initially after the {@link TreeLayout#directTree} method and finally after the layout has
been calculated. It may be overridden to additionally reverse data that depends on the direction of the edges.
@param reversedEdges the edges that will be reversed
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23reverseEdges(EdgeList) Online Documentation}*/
reverseEdges(reversedEdges: EdgeList | null): void;
/**
Data provider key for specifying a child node placer for each node
@see TreeLayout#defaultNodePlacer
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23NODE_PLACER_DP_KEY Online Documentation}*/
static NODE_PLACER_DP_KEY: NodeDpKey<ITreeLayoutNodePlacer> | null;
/**
Data provider key for assigning ports to nodes
@see TreeLayout#defaultPortAssignment
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23PORT_ASSIGNMENT_DP_KEY Online Documentation}*/
static PORT_ASSIGNMENT_DP_KEY: NodeDpKey<ITreeLayoutPortAssignment> | null;
/**
Data provider key for specifying the comparator for the outgoing edges
@see TreeLayout#defaultOutEdgeComparer
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23OUT_EDGE_COMPARER_DP_KEY Online Documentation}*/
static OUT_EDGE_COMPARER_DP_KEY: NodeDpKey<IComparer<any>> | null;
/**
Data provider key for defining the priority of critical edges
The layout algorithm tries to align each node pair that is connected by a critical edge (integer value >
<code>0</code>). This feature can, for example, be used for highlighting different edge paths that are important for a user.
Conflicts between different critical edges are always resolved in favor of the higher priority.
This feature is only supported by the following {@link ITreeLayoutNodePlacer}s:
<ul>
<li>{@link DefaultNodePlacer}
<li>{@link SimpleNodePlacer}
<li>{@link LayeredNodePlacer}
<li>{@link DoubleLineNodePlacer}
<li>{@link DendrogramNodePlacer}
</ul>
<h2>Note</h2>
The critical edge may not be straight if subtrees are rotated or port constraints are assigned.
<h2>Note</h2>
Critical edges will always align the centers of source and target node, thus replacing the current root alignment of the
{@link ITreeLayoutNodePlacer}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23CRITICAL_EDGE_DP_KEY Online Documentation}*/
static CRITICAL_EDGE_DP_KEY: EdgeDpKey<number> | null;
/**
Data provider key for retrieving descriptors for nodes in multi-parent structures
The returned descriptor affects the layout of the multi-parent structure.
<b>Warning:</b> This key is only considered if multi-parent structures are allowed, see
{@link TreeLayout#multiParentAllowed}. Furthermore, all nodes that belong to the same multi-parent structure should return the same descriptor.
@see TreeLayout#multiParentAllowed
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23MULTI_PARENT_DESCRIPTOR_DP_KEY Online Documentation}*/
static MULTI_PARENT_DESCRIPTOR_DP_KEY: NodeDpKey<MultiParentDescriptor> | null;
/**
Data provider key for marking the node that will be used as root node of the tree
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23SELECTED_ROOT_DP_KEY Online Documentation}*/
static SELECTED_ROOT_DP_KEY: NodeDpKey<boolean> | null;
/**
The input graph for which a layout is calculated.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23graph Online Documentation}*/
graph: LayoutGraph | null;
/**
Gets or sets whether or not group nodes are handled by the layout algorithm.
<h2>Note</h2>
If this option is disabled, group nodes will be temporarily hidden and reinserted to enclose their content. This may
lead to node overlaps.
<b>Warning:</b>
Grouping only works correctly, if each group contains a complete subtree. This means that for each group there is a node
such that the group contains exactly this node and all its descendants.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23groupingSupported Online Documentation}*/
groupingSupported: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for arranging the components of the graph is activated.
<b>Warning:</b> {@link TreeLayout} can only handle single components. Disabling {@link ComponentLayout} will lead to errors
during execution.
@see MultiStageLayout#componentLayoutEnabled
@see MultiStageLayout#componentLayout
@see ComponentLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23componentLayoutEnabled Online Documentation}*/
componentLayoutEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing parallel edges is activated.
<b>Warning:</b> {@link TreeLayout} cannot handle parallel edges. Disabling {@link ParallelEdgeRouter} will lead to errors
during execution.
@see MultiStageLayout#parallelEdgeRouterEnabled
@see MultiStageLayout#parallelEdgeRouter
@see ParallelEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23parallelEdgeRouterEnabled Online Documentation}*/
parallelEdgeRouterEnabled: boolean;
/**
Sets whether or not the {@link ILayoutStage} used for routing self-loops is activated.
<b>Warning:</b> {@link TreeLayout} cannot handle self-loops. Disabling {@link SelfLoopRouter} will lead to errors during
execution.
@see MultiStageLayout#selfLoopRouterEnabled
@see MultiStageLayout#selfLoopRouter
@see SelfLoopRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23selfLoopRouterEnabled Online Documentation}*/
selfLoopRouterEnabled: boolean;
/**
Gets a {@link IDataAcceptor} that can overwrite the source {@link PortConstraint port contraint} temporarily used during the layout.
This method sets temporary source {@link PortConstraint}s, e.g., after edge reversal. It may be overridden to
handle those {@link PortConstraint}s separately.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23sourcePortConstraintDataAcceptor Online Documentation}*/
readonly sourcePortConstraintDataAcceptor: IDataAcceptor | null;
/**
Gets a {@link IDataAcceptor} that can overwrite the target {@link PortConstraint port contraint} temporarily used during the layout.
This method sets temporary target {@link PortConstraint}s, e.g., after edge reversal. It may be overridden to
handle those {@link PortConstraint}s separately.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23targetPortConstraintDataAcceptor Online Documentation}*/
readonly targetPortConstraintDataAcceptor: IDataAcceptor | null;
/**
Gets a {@link IDataAcceptor} that can be used for temporarily overwriting the source group information used
during the layout.
This method may be overridden to handle those edge groups separately.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23sourceGroupDataAcceptor Online Documentation}*/
readonly sourceGroupDataAcceptor: IDataAcceptor | null;
/**
Gets a {@link IDataAcceptor} that can be used for temporarily overwriting the target group information used
during the layout.
This method may be overridden to handle those edge groups separately.
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23targetGroupDataAcceptor Online Documentation}*/
readonly targetGroupDataAcceptor: IDataAcceptor | null;
/**
Gets or sets the default {@link ITreeLayoutNodePlacer} instance that arranges all subtrees that do not have a specific {@link ITreeLayoutNodePlacer}
assigned using a {@link IDataProvider}.
@throws {Exception} <b>(ArgumentError)</b> if the default {@link ITreeLayoutNodePlacer} is set to <code>null</code>
@see TreeLayout#NODE_PLACER_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23defaultNodePlacer Online Documentation}*/
defaultNodePlacer: ITreeLayoutNodePlacer | null;
/**
Gets or sets the default {@link ITreeLayoutNodePlacer} instance that places the leaf nodes of the tree.
@throws {Exception} <b>(ArgumentError)</b> if the default {@link ITreeLayoutNodePlacer} is set to <code>null</code>
@see TreeLayout#defaultNodePlacer
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23defaultLeafPlacer Online Documentation}*/
defaultLeafPlacer: ITreeLayoutNodePlacer | null;
/**
Gets or sets the default {@link ITreeLayoutPortAssignment} instance for all subtrees that do not have a specific {@link ITreeLayoutPortAssignment}
assigned using a {@link IDataProvider}.
@throws {Exception} <b>(ArgumentError)</b> if the default {@link ITreeLayoutPortAssignment} is set to <code>null</code>
@see TreeLayout#PORT_ASSIGNMENT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23defaultPortAssignment Online Documentation}*/
defaultPortAssignment: ITreeLayoutPortAssignment | null;
/**
Gets or sets the default {@link IComparer} instance that sorts the outgoing edges in all subtrees that do not have a specific {@link IComparer} assigned
using a {@link IDataProvider}.
<h2>Note</h2>
When a {@link IComparer} for outgoing edges is <code>null</code>, the order of the edges will be maintained.
@see TreeLayout#OUT_EDGE_COMPARER_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23defaultOutEdgeComparer Online Documentation}*/
defaultOutEdgeComparer: IComparer<any> | null;
/**
Gets or sets whether or not the layout algorithm reserves space for node labels.
If this feature is enabled, the {@link SubtreeShape} of the nodes will also hold the shapes of its labels.
<b>Warning:</b> When enabling this property, any previously specified {@link MultiStageLayout#labeling labeling algorithm} will be overwritten and when disabling it, any
currently specified labeling algorithm will be disabled via property
{@link MultiStageLayout#labelingEnabled}. Therefore, it is recommended to only use this convenience property instead of manually changing the mentioned other
properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets whether or not edge labels are placed by the layout algorithm.
If this is enabled, the {@link SubtreeShape}s of the subtrees are extended by the edge labels.
The layout algorithm uses the information of {@link PreferredPlacementDescriptor} for an edge label to determine
the corresponding placement. However, edge labels are always placed at the target side of the edge. The placement along
the edge only affects the order of the edge labels at the same edge.
<b>Warning:</b> When enabling this property, any previously specified {@link MultiStageLayout#labeling labeling algorithm} will be overwritten and when disabling it, any
currently specified labeling algorithm will be disabled via property
{@link MultiStageLayout#labelingEnabled}. Therefore, it is recommended to only use this convenience property instead of manually changing the mentioned other
properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23integratedEdgeLabeling Online Documentation}*/
integratedEdgeLabeling: boolean;
/**
Gets or sets whether or not multi-parent structures are allowed for this tree layout.
If multi-parent structures are allowed, the directed input graph may contain structures of multiple nodes that share the
same predecessors as well as the same successors. All nodes belonging to such a structure are placed side by side and
the incident edges are routed in a bus-style manner.
<b>Warning:</b>
If the input graph contains groups, all nodes belonging to the same multi-parent structure must be associated with the
same group node, i.e., it is not possible to assign these nodes to different group nodes.
<h2>Note</h2>
As for the different ways in which the {@link ITreeLayoutNodePlacer}s route their edges, multi-parent structures only work for
some {@link ITreeLayoutNodePlacer} implementations:
<ul>
<li>{@link DefaultNodePlacer} delivers the best results for multi-parent structures. However, routing style {@link TreeLayoutEdgeRoutingStyle#FORK_AT_ROOT}
as well as root alignments {@link RootAlignment#LEADING_ON_BUS} and {@link RootAlignment#TRAILING_ON_BUS} are
not supported.
<li>{@link DendrogramNodePlacer} supports multi-parent structures.
<li>{@link BusNodePlacer} supports multi-parent structures.
<li>{@link LeftRightNodePlacer} supports multi-parent structures.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayout%23multiParentAllowed Online Documentation}*/
multiParentAllowed: boolean;
static $class: Class;
private $_TreeLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeLayout;
}
export interface NodeWeightComparer extends YObject, IComparer<any> {
/**
@inheritdoc*/
compare(o1: any, o2: any): number;
} /**
A {@link IComparer} that compares two nodes with respect to their weight value defined via the {@link IDataProvider} registered
with {@link TreeMapLayout#NODE_WEIGHT_DP_KEY}.
Nodes are sorted in descending order, i.e., nodes with a higher weight (and consequently larger area in the tree map)
are placed first. The sum of the child weights is used as the weight value for nodes that have children.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeWeightComparer Online Documentation}*/
export class NodeWeightComparer {
static $class: Class;
private $_NodeWeightComparer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeWeightComparer;
}
export interface TreeMapLayout extends YObject, ILayoutAlgorithm {
}
/**
This layout algorithm produces tree map layouts.
<b>Layout Style</b>
Tree maps are suitable to visualize hierarchical data structures. They use nested rectangles to do so, where the size of
the rectangles encodes its weight. More generally, the area of a rectangle is proportional to a specific dimension of
data associated to it.
Tree maps can, for example, be used to show a file structure on a hard disk, where the dimension of data is the size of
the files/folders. There are many other use cases where the tree map can show the distribution of the dimension of data
among the entities in a hierarchy.
<b>Concept</b>
{@link TreeMapLayout} offers implementations of the well-known treemapping-algorithms {@link TilingPolicy#SLICE_AND_DICE Slice-and-Dice} and {@link TilingPolicy#SQUARIFIED Squarified}.
These <em>tiling</em> policies divide a rectangle with a given size into multiple sub-rectangles. Speaking in terms of graphs this
means that nodes are placed in a nested fashion such that child nodes are placed inside the parent node.
Importantly, <b>node sizes are changed</b> by this layout algorithm to show the proportional weight of a node in the map. The weight is specified by
registering a {@link IDataProvider} with key {@link TreeMapLayout#NODE_WEIGHT_DP_KEY} with the input
graph.
<b>Features</b>
The input can be specified in two ways:
<ol>
<li>As a <em>hierarchic grouping structure</em> that defines the nesting. Note that for group nodes, the specified insets are considered (see
{@link GroupingKeys#GROUP_NODE_INSETS_DP_KEY}). If the graph is a tree graph, then the grouping structure is
ignored - see second input variant. To continue using the hierarchy information of the grouping structure on a tree
graph, one can temporarily hide the tree edges before running the layout.
<li>As a <em>directed tree graph</em>. The parent-child relation is directly taken from the graph's tree structure. If the graph also contains group
nodes, the hierarchy relation induced by them is ignored. If necessary, an undirected tree can be (temporarily) directed
prior to the layout run by using
{@link TreeAlgorithm#directTree}. The tree needs to be directed from the root (top-level hierarchy) to the children
(lower-level hierarchies). Note that a forest graph is no valid input.
</ol>
The offered tiling policies produce drawings with a quite different style. {@link TilingPolicy#SLICE_AND_DICE}
offers results that have a stable, clear ordering but the rectangles have a high aspect ratio. On the other hand, {@link TilingPolicy#SQUARIFIED}
tries to create rectangles close to a given aspect ratio - in consequence, the ordering of neighbor nodes might not be
clearly predictable.
<h2>Note</h2>
Due to insets specified at parent nodes of different hierarchies and the inter-node padding defined by the {@link TreeMapLayout#spacing spacing},
nodes that have the same actual weight can get slightly different sizes - especially if they are on different hierarchy
levels. This is necessary to satisfy the specified distances and insets. If exact same sizes throughout hierarchies is a
requirement, set the {@link TreeMapLayout#spacing spacing} to <code>0</code> and do not specify insets for the group nodes that have children (if input is
specified as grouping structure).
<h2>Note</h2>
Edges are generally not routed by this layout algorithm. If there are any, their path will be reset such that they
become straight lines. If defining the input as a tree graph, edges can be removed after the layout run to get a clean
tree map drawing without any edges across the map. If defining the input via grouping, no edges are necessary
whatsoever.
<b>Warning:</b> This layout algorithm does not consider {@link NodeHalo}s assigned to nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout Online Documentation}*/
export class TreeMapLayout {
/**
Initializes a new instance of the {@link TreeMapLayout} class with default settings.
@param [options] The parameters to pass.
@param [options.tilingPolicy] The tiling policy that defines how the input is divided into sub-rectangles. This option sets the {@link TreeMapLayout#tilingPolicy} property on the created object.
@param [options.nodeComparer] The {@link IComparer} that defines the order in which child nodes are placed inside their parent node. This option sets the {@link TreeMapLayout#nodeComparer} property on the created object.
@param [options.minimumNodeSize] The minimum size (height and width) a node in the tree map must have. This option sets the {@link TreeMapLayout#minimumNodeSize} property on the created object.
@param [options.preferredSize] The desired dimension of the root rectangle into which all nodes are placed. This option sets the {@link TreeMapLayout#preferredSize} property on the created object.
@param [options.aspectRatio] The target aspect ratio of the {@link TilingPolicy#SQUARIFIED squarified tiling policy}. This option sets the {@link TreeMapLayout#aspectRatio} property on the created object.
@param [options.spacing] The spacing between nodes of the same hierarchy. This option sets the {@link TreeMapLayout#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout Online Documentation}*/
constructor(options?: {
tilingPolicy?: TilingPolicy | TilingPolicyStringValues;
nodeComparer?: IComparer<any> | null;
minimumNodeSize?: YDimension | null;
preferredSize?: YDimension | null;
aspectRatio?: number;
spacing?: number;
});
/**
Arranges the given input graph as a tree map.
To create meaningful tree maps, the graph should either contain hierarchical grouping information or be a tree graph
that defines the hierarchy.
<h2>Note</h2>
The given graph will not be copied during the layout process and the layout will be immediately applied to the given
graph. Node sizes will be changed to create the tree map layout.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Data provider key for specifying the weight of the nodes
The weight of a node defines the area assigned to the node inside the available parent area with respect to the other
child nodes. Child nodes with the same weight get the same area, they do not necessarily get equal width and height
values.
All leaf nodes (nodes without further children, including empty group nodes) must have a weight greater than
<code>0</code>. Non-leaf nodes do not need to have a weight. They will basically get the sum of the weight of their children. If they
have a weight on their own, some free space is generated inside the representative rectangle (i.e. the parent node is
larger than the combined children).
<h2>Note</h2>
Due to insets specified at parent nodes of different hierarchies and the inter-node padding defined by
{@link TreeMapLayout#spacing}, nodes that have the same actual weight can get slightly different sizes.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout%23NODE_WEIGHT_DP_KEY Online Documentation}*/
static NODE_WEIGHT_DP_KEY: NodeDpKey<number> | null;
/**
Gets or sets the tiling policy that defines how the input is divided into sub-rectangles.
@throws {Exception} <b>(ArgumentError)</b> if an unknown policy is specified
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout%23tilingPolicy Online Documentation}*/
tilingPolicy: TilingPolicy;
/**
Gets or sets the {@link IComparer} that defines the order in which child nodes are placed inside their parent node.
@throws {Exception} <b>(ArgumentError)</b> if the given {@link IComparer} is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout%23nodeComparer Online Documentation}*/
nodeComparer: IComparer<any> | null;
/**
Gets or sets the minimum size (height and width) a node in the tree map must have.
If the {@link TreeMapLayout#NODE_WEIGHT_DP_KEY weight data} induces that a node would get a width/height smaller than specified by the minimum size to fit into
the
{@link TreeMapLayout#preferredSize preferred size}, then the output size will be increased until all nodes fulfill the minimum size requirement. This
scenario will mostly occur if the weights of nodes have a large range or if the specified preferred output size is
small.
@throws {Exception} <b>(ArgumentError)</b> if the width or height of the given size is smaller than <code>5</code> or if the given minimum size is <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout%23minimumNodeSize Online Documentation}*/
minimumNodeSize: YDimension | null;
/**
Gets or sets the desired dimension of the root rectangle into which all nodes are placed.
If it is possible to arrange all nodes such that they fit into the preferred size, then the final result will exactly
have this size. However, satisfying the specified {@link TreeMapLayout#spacing spacing} and {@link TreeMapLayout#minimumNodeSize minimum node size} has higher priority. This means that the
final output might become larger if necessary. On the other hand, it will never be smaller, even if it would be possible
to generate a smaller result.
@throws {Exception} <b>(ArgumentError)</b> if the given size is <code>null</code> or if its width/height value is less than or equal to <code>zero</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout%23preferredSize Online Documentation}*/
preferredSize: YDimension | null;
/**
Gets or sets the target aspect ratio of the {@link TilingPolicy#SQUARIFIED squarified tiling policy}.
The aspect ratio must be greater than or equal to <code>1</code>. A ratio of <code>1</code> means that the node rectangles should ideally be
squares. The orientation of the rectangles does not matter. For example, a ratio of <code>2</code> means that rectangles with an
aspect ratio of <code>1:2</code> or also <code>2:1</code> are desired.
This setting affects the ratio of a single node rectangle. To create a tree map with a certain overall aspect ratio, the
{@link TreeMapLayout#preferredSize preferred size} can be specified accordingly.
<b>Warning:</b> This setting only has an effect if {@link TilingPolicy#SQUARIFIED} is specified as {@link TreeMapLayout#tilingPolicy tiling policy}.
@throws {Exception} <b>(ArgumentError)</b> if the given aspect ratio is smaller than <code>1</code>
@see TilingPolicy#SQUARIFIED
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout%23aspectRatio Online Documentation}*/
aspectRatio: number;
/**
Gets or sets the spacing between nodes of the same hierarchy.
This spacing enables to insert a padding between all nodes/rectangles on the same hierarchy. It is not inserted between
a group node (border) and its children - use group node insets for that use case.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayout%23spacing Online Documentation}*/
spacing: number;
static $class: Class;
private $_TreeMapLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeMapLayout;
}
export interface TreeReductionStage extends LayoutStageBase {
}
/**
The {@link TreeReductionStage} temporarily reduces general graphs to trees.
<b>Layout Style</b>
This stage prepares a non-tree graph such that it can be processed by a tree layout algorithm.
<b>Concept</b>
This {@link ILayoutStage} works in three steps:
<ol>
<li>Remove some edges to get a tree or forest from the graph
<li>Invoke the {@link LayoutStageBase#coreLayout core layout algorithm} which supports tree graph handling
<li>Reinsert all previously removed edges
<li>Optionally route these edges using the non-tree edge router and place their labels using the non-tree edge labeling
algorithm.
</ol>
<b>Features</b>
The routing of the temporarily hidden non-tree edges can be customized by specifying an {@link TreeReductionStage#nonTreeEdgeRouter edge routing algorithm} for those
edges. Similarly, the placement of edge labels of non-tree edges can be delegated to a custom
{@link TreeReductionStage#nonTreeEdgeLabelingAlgorithm edge labeling algorithm}. As default, both custom algorithms are not specified and, thus, non-tree edges are not routed
and edge labels not placed.
This stage is also able to handle multi-parent structures, i.e., structures of multiple nodes that share the same
predecessors as well as the same successors. More precisely, if the specified {@link LayoutStageBase#coreLayout core layout algorithm} supports
multi-parent structures (see {@link TreeLayout#multiParentAllowed}) and option {@link TreeReductionStage#multiParentAllowed} is enabled, this stage does not hide such structures, i.e., the multi-parent
structures are passed to the {@link LayoutStageBase#coreLayout core layout algorithm}.
Non-tree edges can also be bundled together such that their common parts are to some degree merged into a bundled part.
The edge bundling can be specified by means of method {@link TreeReductionStage#edgeBundling}.
<h2>Note</h2>
If there are edges between group nodes, the {@link ClassicTreeLayout} may throw a
{@link Exception InvalidGraphStructureError}-Exception. Such exceptions can be prevented by temporarily removing those edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage Online Documentation}*/
export class TreeReductionStage {
/**
Creates a new {@link TreeReductionStage} instance with the given core layout algorithm and default settings.
@param [coreLayout=null] the core layout algorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage Online Documentation}*/
constructor(coreLayout?: ILayoutAlgorithm | null);
/**
Creates a new {@link TreeReductionStage} instance with the given core layout algorithm and default settings.
@param options The parameters to pass.
@param [options.coreLayout=null] the core layout algorithm
@param [options.multiParentAllowed] Whether or not multi-parent structures (structures of multiple nodes that share the same predecessors as well as the same successors) are allowed. This option sets the {@link TreeReductionStage#multiParentAllowed} property on the created object.
@param [options.nonTreeEdgeRouter] The edge routing algorithm that is applied to all non-tree edges. This option sets the {@link TreeReductionStage#nonTreeEdgeRouter} property on the created object.
@param [options.nonTreeEdgeSelectionKey] The key to register a {@link IDataProvider} that will be used by the {@link TreeReductionStage#nonTreeEdgeRouter non-tree edge routing algorithm} to determine the edges that need to be routed. This option sets the {@link TreeReductionStage#nonTreeEdgeSelectionKey} property on the created object.
@param [options.nonTreeEdgeLabelingAlgorithm] The labeling algorithm that is applied to all edge labels that belong to non-tree edges. This option sets the {@link TreeReductionStage#nonTreeEdgeLabelingAlgorithm} property on the created object.
@param [options.nonTreeEdgeLabelSelectionKey] The key to register a {@link IDataProvider} that is used by the {@link TreeReductionStage#nonTreeEdgeLabelingAlgorithm non-tree edge labeling algorithm} to determine which edge labels it should place. This option sets the {@link TreeReductionStage#nonTreeEdgeLabelSelectionKey} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage Online Documentation}*/
constructor(options: {
coreLayout?: ILayoutAlgorithm | null;
multiParentAllowed?: boolean;
nonTreeEdgeRouter?: ILayoutAlgorithm | null;
nonTreeEdgeSelectionKey?: any | null;
nonTreeEdgeLabelingAlgorithm?: ILayoutAlgorithm | null;
nonTreeEdgeLabelSelectionKey?: any | null;
});
/**
Determines a spanning tree of the graph and passes it to the {@link LayoutStageBase#coreLayout core layout algorithm}.
This {@link ILayoutStage} reduces the graph to a tree. Then, it applies the {@link LayoutStageBase#coreLayout core layout algorithm} to that tree.
After it gets the result, it reinserts the non-tree edges and routes them.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Creates a routing algorithm that routes edges as a single straight segment.
The created instance can be used for {@link TreeReductionStage#nonTreeEdgeRouter routing non-tree edges}.
@returns an edge routing algorithm that produces straight-line edges
@see TreeReductionStage#nonTreeEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23createStraightLineRouter() Online Documentation}*/
createStraightLineRouter(): ILayoutAlgorithm | null;
/**
Routes all edges that do not belong to the chosen spanning tree.
This method is called by {@link TreeReductionStage#applyLayout} after the tree was arranged by the
{@link LayoutStageBase#coreLayout core layout algorithm}. It may be overridden to apply custom edge routes.
<b>Warning:</b> This method will do nothing if no {@link TreeReductionStage#nonTreeEdgeRouter non-tree edge routing algorithm} was specified (i.e. if it is <code>null</code>).
@param graph the graph containing tree and non-tree edges
@param nonTreeEdgeMap the {@link IEdgeMap} that marks all non-tree edges in the graph
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23routeNonTreeEdges(LayoutGraph%2CIEdgeMap) Online Documentation}*/
routeNonTreeEdges(graph: LayoutGraph | null, nonTreeEdgeMap: IEdgeMap | null): void;
/**
Data provider key for explicitly marking (some) edges that should not be considered for the tree
@see TreeReductionStage#nonTreeEdgeRouter
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23NON_TREE_EDGES_DP_KEY Online Documentation}*/
static NON_TREE_EDGES_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets whether or not multi-parent structures (structures of multiple nodes that share the same predecessors as
well as the same successors) are allowed.
More precisely, if this option is enabled and the specified {@link LayoutStageBase#coreLayout core layout algorithm} can handle multi-parent structures
(see {@link TreeLayout#multiParentAllowed}), this stage does not hide such structures, but the multi-parent structures are passed on to the {@link LayoutStageBase#coreLayout} core layout
algorithm}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23multiParentAllowed Online Documentation}*/
multiParentAllowed: boolean;
/**
Gets or sets the edge routing algorithm that is applied to all non-tree edges.
<h2>Note</h2>
It is required that a suitable {@link TreeReductionStage#nonTreeEdgeSelectionKey edge selection key} is specified and the router's scope is reduced to the selected edges.
<h2>Note</h2>
The specified edge router is not applied to {@link TreeReductionStage#edgeBundling bundled} non-tree edges.
@see TreeReductionStage#nonTreeEdgeSelectionKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23nonTreeEdgeRouter Online Documentation}*/
nonTreeEdgeRouter: ILayoutAlgorithm | null;
/**
Gets or sets the key to register a {@link IDataProvider} that will be used by the {@link TreeReductionStage#nonTreeEdgeRouter non-tree edge routing algorithm}
to determine the edges that need to be routed.
<h2>Note</h2>
If there is already a {@link IDataAcceptor} registered with this key, the stage uses it to mark the non-tree
edges. This way it can be determined which edges the layout treated as tree and which as non-tree edges.
@see TreeReductionStage#nonTreeEdgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23nonTreeEdgeSelectionKey Online Documentation}*/
nonTreeEdgeSelectionKey: any | null;
/**
Gets or sets the labeling algorithm that is applied to all edge labels that belong to non-tree edges.
It is required that a suitable {@link TreeReductionStage#nonTreeEdgeLabelSelectionKey edge label selection key} is set. Otherwise, the edge labeling algorithm might also place
labels of tree edges.
@see TreeReductionStage#nonTreeEdgeLabelSelectionKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23nonTreeEdgeLabelingAlgorithm Online Documentation}*/
nonTreeEdgeLabelingAlgorithm: ILayoutAlgorithm | null;
/**
Gets or sets the key to register a {@link IDataProvider} that is used by the {@link TreeReductionStage#nonTreeEdgeLabelingAlgorithm non-tree edge labeling algorithm}
to determine which edge labels it should place.
<h2>Note</h2>
During the layout, a {@link IDataProvider} with this key will be registered with the graph. It will mark all
{@link IEdgeLabelLayout}s that belong to non-tree edges. The specified non-tree edge labeling algorithm needs to
obey this selection. If using {@link GenericLabeling} as labeling algorithm, set this key as value of property
{@link LabelingBase#affectedLabelsDpKey}.
<b>Warning:</b> When specifying a {@link TreeReductionStage#nonTreeEdgeLabelingAlgorithm non-tree edge labeling algorithm} but no edge label selection key, then the labeling algorithm might
place all labels, including those that belong to actual tree edges.
@see TreeReductionStage#nonTreeEdgeLabelingAlgorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23nonTreeEdgeLabelSelectionKey Online Documentation}*/
nonTreeEdgeLabelSelectionKey: any | null;
/**
Gets the {@link EdgeBundling} instance that defines the settings of the edge bundling feature.
The specified {@link EdgeBundling} defines global bundling properties. Settings for individual edges can be
defined by assigning an {@link EdgeBundleDescriptor} to an edge using a {@link IDataProvider} registered
with key {@link EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}. To enable bundling for all non-tree edges, set a {@link EdgeBundling#defaultBundleDescriptor default bundle descriptor}
which has {@link EdgeBundleDescriptor#bundled bundling} enabled.
If the graph contains self-loops that belong to the set of non-tree edges, the stage will invoke the router defined for
the non-tree edges in order to route them. In the case where this is not desired, the user has to hide them from the
stage and route them afterwards as desired.
<h2>Note</h2>
The specified {@link TreeReductionStage#nonTreeEdgeRouter non-tree layouter} is not applied to bundled edges.
<b>Warning:</b> Edge bundling has an effect only for non-tree edges for which {@link EdgeBundleDescriptor#bundled bundling} is enabled and if the graph does not contain
multi-parent structures.
<b>Warning:</b> Edge bundling ignores any {@link PortConstraint}s and
{@link PortCandidate}s assigned to nodes/edges and always resets the port to the center of the nodes. Also, when
edge bundling is enabled, node/edge label overlaps are allowed.
<b>Warning:</b> In the case where the {@link LayoutStageBase#coreLayout core layouter} uses the {@link ParallelEdgeRouter} to route possible parallel edges, all
edges that will be bundled and approximated by a cubic bezier curve using {@link CurveFittingLayoutStage} should
not be in the scope of {@link ParallelEdgeRouter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStage%23edgeBundling Online Documentation}*/
readonly edgeBundling: EdgeBundling | null;
static $class: Class;
private $_TreeReductionStage: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeReductionStage;
}
export interface BetweennessCentralityResult extends YObject {
}
/**
Represents the node and edge centrality values as computed by {@link BetweennessCentrality}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentralityResult Online Documentation}*/
export class BetweennessCentralityResult {
/**
Gets a mapping from each node to its absolute centrality value.
The absolute centrality value can change drastically depending on graph structure and edge weights. The {@link BetweennessCentralityResult#normalizedNodeCentrality} values don't
usually change as much, while still allowing to determine relative importance between nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentralityResult%23nodeCentrality Online Documentation}*/
nodeCentrality: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its normalized centrality value.
The normalized value is obtained by dividing {@link BetweennessCentralityResult#nodeCentrality} by {@link BetweennessCentralityResult#maximumNodeCentrality}. Thus the normalized values are all between <code>0</code> and <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentralityResult%23normalizedNodeCentrality Online Documentation}*/
normalizedNodeCentrality: ResultItemMapping<INode, number>;
/**
Gets the minimum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentralityResult%23minimumNodeCentrality Online Documentation}*/
minimumNodeCentrality: number;
/**
Gets the maximum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentralityResult%23maximumNodeCentrality Online Documentation}*/
maximumNodeCentrality: number;
/**
Gets a mapping from each edge to its absolute centrality value.
The absolute centrality value can change drastically depending on graph structure and edge weights. The {@link BetweennessCentralityResult#normalizedEdgeCentrality} values don't
usually change as much, while still allowing to determine relative importance between nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentralityResult%23edgeCentrality Online Documentation}*/
edgeCentrality: ResultItemMapping<IEdge, number>;
/**
Gets a mapping from each edge to its normalized centrality value.
The normalized value is obtained by dividing {@link BetweennessCentralityResult#nodeCentrality} by {@link BetweennessCentralityResult#maximumNodeCentrality}. Thus the normalized values are all between <code>0</code> and <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentralityResult%23normalizedEdgeCentrality Online Documentation}*/
normalizedEdgeCentrality: ResultItemMapping<IEdge, number>;
/**
Gets the minimum absolute edge centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentralityResult%23minimumEdgeCentrality Online Documentation}*/
minimumEdgeCentrality: number;
/**
Gets the maximum absolute edge centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentralityResult%23maximumEdgeCentrality Online Documentation}*/
maximumEdgeCentrality: number;
static $class: Class;
private $_BetweennessCentralityResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BetweennessCentralityResult;
}
export interface BetweennessCentrality extends YObject {
}
/**
* Computes the betweenness centrality for each node and edge of a given graph.
* <i>Betweenness centrality</i> is a measure for how often a node/edge lies on a shortest path between each pair of nodes in the graph. Removing a
* central node/edge will cause many shortest paths to change.
*
* The centrality can be computed for {@link BetweennessCentrality#directed directed} and undirected graphs.
*
* {@link BetweennessCentrality#weights} can be assigned to each edge. If no weights are provided, then all edges have uniform weight.
*
* <b>Other Centrality Measures</b>
*
* yFiles for HTML supports a number of other centrality measures:
*
* <ul>
* <li><b>{@link GraphCentrality}, {@link ClosenessCentrality}</b> – emphasize nodes that have short paths to other nodes</li>
* <li><b>{@link DegreeCentrality}</b> – emphasizes nodes with many edges</li>
* <li><b>{@link WeightCentrality}</b> – emphasizes nodes with highly-weighted edges</li>
* <li><b>{@link EigenvectorCentrality}</b> – computes the influence a node has on a network. The centrality value is higher if more nodes are connected to that node</li>
* <li><b>{@link PageRank}</b> – computes page rank values for all nodes based on their attached edges</li>
* </ul>
* <pre>
* const result = new BetweennessCentrality({
*   directed: true,
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* }).run(graph)
*
* // add edge labels for centrality values
* result.normalizedEdgeCentrality.forEach(({ key, value }) => {
*   const edge = key
*   const centrality = value
*   graph.addLabel(edge, String(centrality))
* })
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(key, String(value))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see CentralityAlgorithm#nodeEdgeBetweenness
* @see CentralityAlgorithm#nodeBetweenness
* @see CentralityAlgorithm#edgeBetweenness
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentrality Online Documentation}
*/
export class BetweennessCentrality {
/**
Creates a new {@link BetweennessCentrality} instance.
@param [options] The parameters to pass.
@param [options.weights] A mapping for edge weights. This option sets the {@link BetweennessCentrality#weights} property on the created object.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link BetweennessCentrality#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link BetweennessCentrality#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link BetweennessCentrality#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentrality Online Documentation}*/
constructor(options?: {
weights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Computes the betweenness centrality for each node and edge of a given graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link BetweennessCentralityResult} from which the calculated centrality values can be obtained.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentrality%23run(IGraph) Online Documentation}*/
run(graph: IGraph): BetweennessCentralityResult;
/**
Gets or sets a mapping for edge weights.
Betweenness centrality computes shortest paths throughout the graph. Edge weights influence the computed length of those
paths and thus the centrality measure. If no weights are provided, all edges have the same uniform weight of <code>1</code> and the
number of edges is effectively the shortest path length.
Edge weights for betweenness centrality must be finite and positive. Negative weights, weights too close to 0, and
missing weights are all treated as a weight of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentrality%23weights Online Documentation}*/
weights: ItemMapping<IEdge, number>;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentrality%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link BetweennessCentrality#run}
* method.
*
* <br><b>Calculating the betweenness centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new BetweennessCentrality({
*   directed: true,
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength(),
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add edge labels for centrality values
* result.normalizedEdgeCentrality.forEach(entry =>
*   graph.addLabel(entry.key, `${entry.value}`)
* )
* </pre>
*
* <br><b>Considering only selected nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes)
* </pre>
*
* <br><b>Ignoring all group nodes</b>
* <pre>
* const subset = new ItemCollection()
* subset.excludes = n => graph.isGroupNode(n)
* </pre>
*
* <br><b>Considering selected nodes but ignoring group nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes, n =>
*   graph.isGroupNode(n)
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentrality%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link BetweennessCentrality#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link BetweennessCentrality#run}
* method.
*
* <br><b>Calculating the betweenness centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new BetweennessCentrality({
*   directed: true,
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength(),
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add edge labels for centrality values
* result.normalizedEdgeCentrality.forEach(entry =>
*   graph.addLabel(entry.key, `${entry.value}`)
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BetweennessCentrality%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_BetweennessCentrality: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BetweennessCentrality;
}
export interface BfsResult extends YObject {
}
/**
Represents the results of a breadth-first search as computed by {@link Bfs}.
Nodes are grouped into {@link BfsResult#layers} based on distance from the original {@link Bfs#coreNodes}.
Contains the layers of nodes constructed by a directed/undirected breadth-first search where the {@link Bfs#layerCount number of layers} is
restricted.
The first of these layers contains all {@link Bfs#coreNodes} from which a breadth-first search to the other nodes starts..
In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsResult Online Documentation}*/
export class BfsResult {
/**
Gets a mapping from each node to its layer index.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsResult%23nodeLayerIds Online Documentation}*/
nodeLayerIds: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its layer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsResult%23nodeLayers Online Documentation}*/
nodeLayers: ResultItemMapping<INode, BfsLayer>;
/**
Gets a collection of all layers.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsResult%23layers Online Documentation}*/
layers: ResultItemCollection<BfsLayer>;
/**
Gets a collection of nodes that are either not reachable from the {@link Bfs#coreNodes} or which are in layers beyond the {@link Bfs#layerCount}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsResult%23unreachableNodes Online Documentation}*/
unreachableNodes: ResultItemCollection<INode>;
static $class: Class;
private $_BfsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BfsResult;
}
export interface BfsLayer extends YObject {
}
/**
Represents a layer of nodes as a result of a {@link Bfs breadth-first search}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsLayer Online Documentation}*/
export class BfsLayer {
/**
Gets the layer's index within the result.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsLayer%23index Online Documentation}*/
index: number;
/**
Gets a collection of all nodes belonging to this layer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BfsLayer%23nodes Online Documentation}*/
nodes: ResultItemCollection<INode>;
static $class: Class;
private $_BfsLayer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BfsLayer;
}
export interface Bfs extends YObject {
}
/**
* Computes the layers of nodes constructed by a directed or undirected breadth-first search.
* The first of these layers contains all {@link Bfs#coreNodes} from which a breadth-first search to the other nodes starts.
*
* Each layer <i>i</i> contains nodes that have a path of length <i>i</i> to at least one of the {@link Bfs#coreNodes}. Nodes that are unreachable because there
* is no path to them, or the path's length exceeds {@link Bfs#layerCount} have a layer index of <code>-1</code> instead.
*
* The search can be directed or undirected. In a directed search the edges can be traversed either forwards (from source
* to target) or backwards.
*
* <pre>
* // configure the algorithm
* const algorithm = new Bfs({
*   // specify the nodes of the 0-th layer to start from
*   coreNodes: centerNode
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add labels to indicate the layer
* result.nodeLayerIds.forEach(({ key: node, value: id }) => graph.addLabel(node, String(id)))
* </pre>
*
* @see BfsAlgorithm#getLayers
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Bfs Online Documentation}
*/
export class Bfs {
/**
Creates a new {@link Bfs} instance.
@param [options] The parameters to pass.
@param [options.coreNodes] A collection of nodes to start from. This option sets the {@link Bfs#coreNodes} property on the created object.
@param [options.traversalDirection] A value indicating whether to follow {@link TraversalDirection#PREDECESSOR incoming}, {@link TraversalDirection#SUCCESSOR outgoing}, or {@link TraversalDirection#BOTH all} edges. This option sets the {@link Bfs#traversalDirection} property on the created object.
@param [options.layerCount] The number of layers to return. This option sets the {@link Bfs#layerCount} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link Bfs#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link Bfs#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Bfs Online Documentation}*/
constructor(options?: {
coreNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
traversalDirection?: TraversalDirection | TraversalDirectionStringValues;
layerCount?: number;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Returns the layers of nodes constructed by a breadth-first search.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link BfsResult} containing the calculated node layers.
@throws {Exception} <b>(InvalidOperationError)</b> if no {@link Bfs#coreNodes} are provided.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see BfsAlgorithm#getLayers
@see {@link https://docs.yworks.com/yfileshtml/#/api/Bfs%23run(IGraph) Online Documentation}*/
run(graph: IGraph): BfsResult;
/**
Gets or sets a collection of nodes to start from.
These nodes form the 0-th layer.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Bfs%23coreNodes Online Documentation}*/
coreNodes: ItemCollection<INode>;
/**
Gets or sets a value indicating whether to follow {@link TraversalDirection#PREDECESSOR incoming}, {@link TraversalDirection#SUCCESSOR outgoing}, or {@link TraversalDirection#BOTH all} edges.
Default is {@link TraversalDirection#BOTH}.
@default TraversalDirection.BOTH
@see {@link https://docs.yworks.com/yfileshtml/#/api/Bfs%23traversalDirection Online Documentation}*/
traversalDirection: TraversalDirection;
/**
Gets or sets the number of layers to return.
This setting can improve performance if not all layers are needed. Setting <code>0</code> as value (the default) will return all
layers.
@default 0
@see {@link https://docs.yworks.com/yfileshtml/#/api/Bfs%23layerCount Online Documentation}*/
layerCount: number;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link Bfs#run} method.
*
* <b>Warning:</b> At least one of the nodes in {@link Bfs#coreNodes} must be included in {@link Bfs#subgraphNodes} if {@link Bfs#subgraphNodes} are set.
*
* <br><b>Performing the breadth-first search on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new Bfs({
*   // note that at least one core node must be included in the SubgraphNodes
*   coreNodes: centerNode,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add labels to indicate the layer
* result.nodeLayerIds.forEach(({ key, value }) => {
*   graph.addLabel(key, String(value))
* })
* </pre>
*
* <br><b>Considering only selected nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes)
* </pre>
*
* <br><b>Ignoring all group nodes</b>
* <pre>
* const subset = new ItemCollection()
* subset.excludes = n => graph.isGroupNode(n)
* </pre>
*
* <br><b>Considering selected nodes but ignoring group nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes, n =>
*   graph.isGroupNode(n)
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Bfs%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link Bfs#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link Bfs#run} method.
*
* <br><b>Performing the breadth-first search on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new Bfs({
*   // note that at least one core node must be included in the SubgraphNodes
*   coreNodes: centerNode,
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add labels to indicate the layer
* result.nodeLayerIds.forEach(({ key, value }) => {
*   graph.addLabel(key, String(value))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Bfs%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_Bfs: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Bfs;
}
export interface BiconnectedComponentClusteringResult extends YObject {
}
/**
Represents clusters in the graph as computed by {@link BiconnectedComponentClustering}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClusteringResult Online Documentation}*/
export class BiconnectedComponentClusteringResult {
/**
Gets a mapping from each node to its cluster ID.
If no {@link BiconnectedComponentClustering#subgraphNodes subgraph} is specified, the ID of a cluster is a non-negative {@link number} value which is common for all nodes of the same
cluster.
If a {@link BiconnectedComponentClustering#subgraphNodes subgraph} is specified, all nodes that are <b>not</b> part of the subgraph have the ID <code>-1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClusteringResult%23nodeClusterIds Online Documentation}*/
nodeClusterIds: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its cluster.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClusteringResult%23nodeClusters Online Documentation}*/
nodeClusters: ResultItemMapping<INode, Cluster>;
/**
Gets a collection of all clusters in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClusteringResult%23clusters Online Documentation}*/
clusters: ResultItemCollection<Cluster>;
/**
Gets a value which estimates the quality of the partition of the nodes into the {@link BiconnectedComponentClusteringResult#clusters}.
The modularity value estimates the quality of the partition of the nodes into the given communities and is a value
between
<code>[-1/2,1]</code>. High modularity values denote that nodes with dense connections are placed in the same community, while nodes
with sparse connections are assigned to different communities.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClusteringResult%23modularity Online Documentation}*/
modularity: number;
static $class: Class;
private $_BiconnectedComponentClusteringResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BiconnectedComponentClusteringResult;
}
export interface BiconnectedComponentClustering extends YObject {
}
/**
* Partitions the graph into clusters by analyzing its <i>biconnected components</i>.
* A biconnected component is a subgraph which is connected and non-separable, i.e. after removing one single node the
* component is still connected.
*
* Nodes will be clustered such that the nodes within each cluster are biconnected. Nodes that belong to multiple
* biconnected components (articulation nodes) will be assigned to exactly one of these clusters.
*
* <b>Other Clustering Algorithms</b>
*
* yFiles for HTML supports a number of other clustering algorithms:
*
* <ul>
* <li><b>{@link KMeansClustering}</b> – partitions the graph into clusters based on the distance between nodes and the cluster midpoints.</li>
* <li><b>{@link HierarchicalClustering}</b> – partitions the graph into clusters by merging smaller clusters based on their distance.</li>
* <li><b>{@link EdgeBetweennessClustering}</b> – partitions the graph into clusters based on edge-betweenness centrality.</li>
* <li><b>{@link LouvainModularityClustering}</b> – partitions the graph into clusters by applying the <i>Louvain modularity</i> method.</li>
* <li><b>{@link LabelPropagationClustering}</b> – partitions the graph into clusters by applying a label propagation algorithm.</li>
* </ul>
* <h2>Note</h2>
*
* Isolated nodes, that is, nodes that have no edges or only self-loops, are assigned to their own cluster.
*
* <br><b>Calculating the biconnected component clusters of a graph</b>
* <pre>
* // prepare the biconnected components clustering algorithm
* const algorithm = new BiconnectedComponentClustering()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see GroupAlgorithm#biconnectedComponentGrouping
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClustering Online Documentation}
*/
export class BiconnectedComponentClustering {
/**
Creates a new {@link BiconnectedComponentClustering} instance.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link BiconnectedComponentClustering#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link BiconnectedComponentClustering#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClustering Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Partitions the graph into clusters based on its biconnected components.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link BiconnectedComponentClusteringResult} of clusters in the graph.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClustering%23run(IGraph) Online Documentation}*/
run(graph: IGraph): BiconnectedComponentClusteringResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link BiconnectedComponentClustering#run}
* method.
*
* <br><b>Calculating the biconnected component clusters on a subset of the graph</b>
* <pre>
* // prepare the biconnected components clustering algorithm
* const algorithm = new BiconnectedComponentClustering({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClustering%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link BiconnectedComponentClustering#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link BiconnectedComponentClustering#run}
* method.
*
* <br><b>Calculating the biconnected component clusters on a subset of the graph</b>
* <pre>
* // prepare the biconnected components clustering algorithm
* const algorithm = new BiconnectedComponentClustering({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentClustering%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_BiconnectedComponentClustering: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BiconnectedComponentClustering;
}
export interface BiconnectedComponentsResult extends YObject {
}
/**
Represents the biconnected components of a graph, as computed by {@link BiconnectedComponents}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentsResult Online Documentation}*/
export class BiconnectedComponentsResult {
/**
Gets a collection of all biconnected components which contain the given <code>node</code>.
@param node
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentsResult%23getNodeComponents(INode) Online Documentation}*/
getNodeComponents(node: INode): ResultItemCollection<BiconnectedComponent>;
/**
Whether the given node is an articulation point.
@param node The node to test.
@returns <code>true</code> if the node is an articulation point, <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentsResult%23isArticulationNode(INode) Online Documentation}*/
isArticulationNode(node: INode): boolean;
/**
Gets a map which returns for each edge the ID of the component the edge belongs to.
The ID of a component is a non-negative {@link number} value which is common for all edges of the same component.
For self loops that cannot be associated unambiguously to a component, or for edges that do not belong to the
{@link BiconnectedComponents#subgraphEdges}, we return -1.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentsResult%23edgeComponentIds Online Documentation}*/
edgeComponentIds: ResultItemMapping<IEdge, number>;
/**
Gets a map which returns for each edge the {@link BiconnectedComponent component} the edge belongs to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentsResult%23edgeComponents Online Documentation}*/
edgeComponents: ResultItemMapping<IEdge, BiconnectedComponent>;
/**
Gets a collection of all biconnected components in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentsResult%23components Online Documentation}*/
components: ResultItemCollection<BiconnectedComponent>;
/**
Gets a collection of all articulation points in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponentsResult%23articulationNodes Online Documentation}*/
articulationNodes: ResultItemCollection<INode>;
static $class: Class;
private $_BiconnectedComponentsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BiconnectedComponentsResult;
}
export interface BiconnectedComponent extends YObject {
}
/**
A single biconnected component, composed of the {@link BiconnectedComponent#nodes} and {@link BiconnectedComponent#edges} which belong to this component and its {@link BiconnectedComponent#articulationNodes articulation points}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponent Online Documentation}*/
export class BiconnectedComponent {
/**
Gets a collection of all edges which belong to this component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponent%23edges Online Documentation}*/
edges: ResultItemCollection<IEdge>;
/**
Gets a collection of all nodes which belong to this component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponent%23nodes Online Documentation}*/
nodes: ResultItemCollection<INode>;
/**
Gets a collection of all articulation points which belong to this component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponent%23articulationNodes Online Documentation}*/
articulationNodes: ResultItemCollection<INode>;
static $class: Class;
private $_BiconnectedComponent: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BiconnectedComponent;
}
export interface BiconnectedComponents extends YObject {
}
/**
* Determines the biconnected components and articulation points of a given undirected graph.
* A biconnected component is a subgraph which is connected and non-separable, i.e. after removing one single node the
* component is still connected. Another way of looking at it is that every pair of nodes in a biconnected component has at
* least two different paths between them.
*
* In a connected (sub)graph the biconnected components are attached to each other at shared nodes, the so-called
* <i>articulation points</i>. Articulation points represent nodes that, when removed, cause the whole connected component to break up into multiple
* connected components.
*
* <b>Other Graph Connectivity Algorithms</b>
*
* yFiles for HTML supports a number of other analysis algorithms that partition the graph into components, based on
* various criteria.
*
* <ul>
* <li><b>{@link ConnectedComponents}</b> – Determines components defined by the existence of an undirected path between nodes</li>
* <li><b>{@link StronglyConnectedComponents}</b> – Determines components defined by the existence of a directed path between nodes</li>
* <li><b>{@link Bipartition}</b> – Divides a graph into two partitions where all edges have their source and target in different partitions</li>
* <li><b>{@link IndependentSets}</b> – Divides a graph into partitions where no nodes are connected within a partition</li>
* <li><b>{@link KCoreComponents}</b> – Calculates the k-cores of an undirected input graph. The k-core of an undirected input graph consists of the subgraph
* components where each node has at least degree <code>k</code>.</li>
* </ul>
* <br><b>Highlighting the edges of different biconnected components in a graph</b>
* <pre>
* // prepare the biconnected components algorithm
* const algorithm = new BiconnectedComponents()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the edges of the biconnected components with different styles
* for (const edge of graph.edges) {
*   const componentId = result.edgeComponentIds.get(edge)
*   graph.setStyle(edge, componentStyles.get(componentId))
* }
* </pre>
*
* @see GraphConnectivity#biconnectedComponents
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponents Online Documentation}
*/
export class BiconnectedComponents {
/**
Creates a new {@link BiconnectedComponents} instance.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link BiconnectedComponents#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link BiconnectedComponents#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponents Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Determines the biconnected components and articulation points of a given undirected graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link BiconnectedComponentsResult} containing the biconnected components and articulation points of <code>graph</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponents%23run(IGraph) Online Documentation}*/
run(graph: IGraph): BiconnectedComponentsResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link BiconnectedComponents#run}
* method.
*
* <br><b>Calculating biconnected components on a subset of the graph</b>
* <pre>
* // configure the biconnected components algorithm
* const algorithm = new BiconnectedComponents({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the edges of the biconnected components with different styles
* for (const edge of graph.edges) {
*   const componentId = result.edgeComponentIds.get(edge)
*   graph.setStyle(edge, componentStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponents%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link BiconnectedComponents#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link BiconnectedComponents#run}
* method.
*
* <br><b>Calculating biconnected components on a subset of the graph</b>
* <pre>
* // configure the biconnected components algorithm
* const algorithm = new BiconnectedComponents({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the edges of the biconnected components with different styles
* for (const edge of graph.edges) {
*   const componentId = result.edgeComponentIds.get(edge)
*   graph.setStyle(edge, componentStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BiconnectedComponents%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_BiconnectedComponents: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BiconnectedComponents;
}
export interface BipartitionResult extends YObject {
}
/**
Represents a bipartition of a graph.
If a bipartition of the graph exists, {@link BipartitionResult#isBipartite} is <code>true</code> and {@link BipartitionResult#firstPartition} and {@link BipartitionResult#secondPartition} contain the nodes that make up the partitions. The {@link BipartitionResult#inSamePartition}
method can be used to determine whether two nodes are in the same partition.
If no bipartition of the graph exists, {@link BipartitionResult#isBipartite} is <code>false</code> and any collections and mappings of the result are empty.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionResult Online Documentation}*/
export class BipartitionResult {
/**
Returns a value indicating whether the specified nodes are in the same partition.
@param node1 The first node to test.
@param node2 The second node to test.
@returns <code>true</code> if both nodes are in the same partition, <code>false</code> if they are in different partitions. If the graph is {@link BipartitionResult#isBipartite not bipartite},
<code>true</code> is returned.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionResult%23inSamePartition(INode%2CINode) Online Documentation}*/
inSamePartition(node1: INode, node2: INode): boolean;
/**
Gets a value indicating whether this result represents a bipartition of the graph.
If the original graph is not bipartite, this is set to <code>false</code> and all collections and mappings of this result are empty.
When using {@link Bipartition}, {@link BipartitionResult#isBipartite} should always be checked first before working with the result.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionResult%23isBipartite Online Documentation}*/
isBipartite: boolean;
/**
Gets the first partition of a bipartite graph.
If the graph is {@link BipartitionResult#isBipartite not bipartite} this partition is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionResult%23firstPartition Online Documentation}*/
firstPartition: BipartitionPartition;
/**
Gets the second partition of a bipartite graph.
If the graph is {@link BipartitionResult#isBipartite not bipartite} this partition is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionResult%23secondPartition Online Documentation}*/
secondPartition: BipartitionPartition;
/**
Gets a mapping from each node to its partition.
This is either the {@link BipartitionResult#firstPartition} or the {@link BipartitionResult#secondPartition}.
If the graph is {@link BipartitionResult#isBipartite not bipartite} this mapping is empty and returns <code>null</code> for all nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionResult%23nodePartitions Online Documentation}*/
nodePartitions: ResultItemMapping<INode, BipartitionPartition>;
static $class: Class;
private $_BipartitionResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BipartitionResult;
}
export interface BipartitionPartition extends YObject {
}
/**
Represents a partition in a bipartite graph.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionPartition Online Documentation}*/
export class BipartitionPartition {
/**
Gets a collection of nodes belonging to this partition.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BipartitionPartition%23nodes Online Documentation}*/
nodes: ResultItemCollection<INode>;
static $class: Class;
private $_BipartitionPartition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BipartitionPartition;
}
export interface Bipartition extends YObject {
}
/**
* Calculates a bipartition of the given graph, if one exists.
* A graph is bipartite if its nodes can be divided into two partitions where no edges are between nodes of the same
* partition. Instead, every edge connects a node from the first with a node from the second partition. <b>Other Graph Connectivity Algorithms</b>
*
* yFiles for HTML supports a number of other analysis algorithms that partition the graph into components, based on
* various criteria.
*
* <ul>
* <li><b>{@link ConnectedComponents}</b> – Determines components defined by the existence of an undirected path between nodes</li>
* <li><b>{@link StronglyConnectedComponents}</b> – Determines components defined by the existence of a directed path between nodes</li>
* <li><b>{@link BiconnectedComponents}</b> – Determines components defined by the existence of at least two separate undirected paths between all nodes</li>
* <li><b>{@link IndependentSets}</b> – Divides a graph into partitions where no nodes are connected within a partition</li>
* <li><b>{@link KCoreComponents}</b> – Calculates the k-cores of an undirected input graph. The k-core of an undirected input graph consists of the subgraph
* components where each node has at least degree <code>k</code>.</li>
* </ul>
* <br><b>Calculating the two partitions of a bipartite graph</b>
* <pre>
* // prepare the bipartition algorithm
* const algorithm = new Bipartition()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the partitions with different styles
* if (result.isBipartite) {
*   for (const node of result.firstPartition.nodes) {
*     graph.setStyle(node, firstPartitionStyle)
*   }
*   for (const node of result.secondPartition.nodes) {
*     graph.setStyle(node, secondPartitionStyle)
*   }
* }
* </pre>
*
* @see BipartitionAlgorithm#getBipartition
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Bipartition Online Documentation}
*/
export class Bipartition {
/**
Creates a new {@link Bipartition} instance.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link Bipartition#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link Bipartition#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Bipartition Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Calculates a bipartition of the given graph, if one exists.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link BipartitionResult} containing both partitions of <code>graph</code>. If the graph is not bipartite, the resulting
partitions are empty and {@link BipartitionResult#isBipartite} is <code>false</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Bipartition%23run(IGraph) Online Documentation}*/
run(graph: IGraph): BipartitionResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link Bipartition#run} method.
*
* <br><b>Calculating a bipartition on a subset of the graph</b>
* <pre>
* // configure the bipartition algorithm
* const algorithm = new Bipartition({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the partitions with different styles
* if (result.isBipartite) {
*   for (const node of result.firstPartition.nodes) {
*     graph.setStyle(node, firstPartitionStyle)
*   }
*   for (const node of result.secondPartition.nodes) {
*     graph.setStyle(node, secondPartitionStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Bipartition%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link Bipartition#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link Bipartition#run} method.
*
* <br><b>Calculating a bipartition on a subset of the graph</b>
* <pre>
* // configure the bipartition algorithm
* const algorithm = new Bipartition({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the partitions with different styles
* if (result.isBipartite) {
*   for (const node of result.firstPartition.nodes) {
*     graph.setStyle(node, firstPartitionStyle)
*   }
*   for (const node of result.secondPartition.nodes) {
*     graph.setStyle(node, secondPartitionStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Bipartition%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_Bipartition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Bipartition;
}
export interface ChainSubstructuresResult extends YObject {
}
/**
Represents the chains found by {@link ChainSubstructures}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructuresResult Online Documentation}*/
export class ChainSubstructuresResult {
/**
Gets the chains found by {@link ChainSubstructures}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructuresResult%23chains Online Documentation}*/
chains: ResultItemCollection<SubstructureItems>;
static $class: Class;
private $_ChainSubstructuresResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ChainSubstructuresResult;
}
export interface ChainSubstructures extends YObject {
}
/**
* Detects {@link SubstructureItems}s that represent isolated chains in the specified graph.
* A chain only consists of elements with the same {@link ChainSubstructures#edgeDirectedness} and {@link ChainSubstructures#nodeTypes}. More precisely, the {@link ChainSubstructures#edgeDirectedness} is considered as follows: A substructure is
* only identified as such if all edges are either undirected or consistently directed with respect to the specified
* directedness.
*
* <ul>
* <li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.</li>
* <li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.</li>
* <li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.</li>
* </ul>
* <h2>Note</h2>
*
* The smallest {@link ChainSubstructures#minimumSize} value that could be considered is <code>3</code>.
*
* <h2>Note</h2>
*
* A chain or star structure is also considered to be a tree.
*
* <h2>Note</h2>
*
* If no {@link ChainSubstructures#edgeDirectedness} is specified, all edges are treated as undirected. Furthermore, if no {@link ChainSubstructures#nodeTypes} are specified, all node are considered to
* be of the same type.
*
* <br><b>Finding the chains in a graph</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new ChainSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the chains
* for (const chain of result.chains) {
*   for (const node of chain.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of chain.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the chains in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new ChainSubstructures({
*   // only nodes with the same tags can be a chain
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the chains
* for (const chain of result.chains) {
*   for (const node of chain.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of chain.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructures Online Documentation}
*/
export class ChainSubstructures {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.minimumSize] The minimum size of a chain. This option sets the {@link ChainSubstructures#minimumSize} property on the created object.
@param [options.nodeTypes] A mapping which maps the type of each node. This option sets the {@link ChainSubstructures#nodeTypes} property on the created object.
@param [options.edgeDirectedness] A mapping that stores the directedness of the edges. This option sets the {@link ChainSubstructures#edgeDirectedness} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link ChainSubstructures#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link ChainSubstructures#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructures Online Documentation}*/
constructor(options?: {
minimumSize?: number;
nodeTypes?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Returns a list of {@link SubstructureItems}s that represent isolated chains in the specified graph.
* A chain only consists of elements with the same {@link ChainSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link ChainSubstructures#nodeTypes} is specified, all
* node are considered to be of the same type.
*
* The {@link ChainSubstructures#edgeDirectedness} is considered as follows: A substructure is only identified as such if all edges are either undirected or
* consistently directed with respect to the specified directedness.
*
* <ul>
* <li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.</li>
* <li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.</li>
* <li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.</li>
* </ul>
* <h2>Note</h2>
*
* The smallest {@link ChainSubstructures#minimumSize} value that could be considered is <code>3</code>.
*
* <h2>Note</h2>
*
* A chain or star structure is also considered to be a tree.
*
* <h2>Note</h2>
*
* If no {@link ChainSubstructures#edgeDirectedness} is specified, all edges are treated as undirected. Furthermore, if no {@link ChainSubstructures#nodeTypes} are specified, all node are considered to
* be of the same type.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Finding the chains in a graph</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new ChainSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the chains
* for (const chain of result.chains) {
*   for (const node of chain.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of chain.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the chains in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new ChainSubstructures({
*   // only nodes with the same tags can be a chain
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the chains
* for (const chain of result.chains) {
*   for (const node of chain.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of chain.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @param graph The graph to find the substructures in.
* @returns A list of {@link SubstructureItems}s that represent the chains.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructures%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): ChainSubstructuresResult;
/**
Gets or sets the minimum size of a chain.
Chains with less nodes are ignored.
Default is <code>3</code>.
<h2>Note</h2>
The smallest {@link ChainSubstructures#minimumSize} value that could be considered is <code>3</code>.
@default 3
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructures%23minimumSize Online Documentation}*/
minimumSize: number;
/**
Gets or sets a mapping which maps the type of each node.
An arbitrary object. Nodes returning equal objects are considered to be of the same type.
If none is provided all nodes are considered as equal.
<h2>Note</h2>
A chain only consists of elements with the same {@link ChainSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link ChainSubstructures#nodeTypes} is specified, all
node are considered to be of the same type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructures%23nodeTypes Online Documentation}*/
nodeTypes: ItemMapping<INode, any>;
/**
Gets or sets a mapping that stores the directedness of the edges.
If a value is provided here, the algorithm only considers a substructure if all edges are either undirected or
consistently directed with respect to the specified directedness.
<ul>
<li>the edge's directedness is greater than <code>0.0</code> and the edge is incoming (i.e. n is the edge's target),
<li>the edge's directedness is smaller than <code>0.0</code> and the edge is outgoing (i.e. n is the edge's source),
<li>or the edge's directedness is <code>0.0</code> (i.e. the edge is considered to be <b>undirected</b>).
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructures%23edgeDirectedness Online Documentation}*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link ChainSubstructures#run}
* method.
*
* <br><b>Finding the chains of a subset of the graph</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new ChainSubstructures({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the chains
* for (const chain of result.chains) {
*   for (const node of chain.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of chain.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructures%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link ChainSubstructures#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link ChainSubstructures#run}
* method.
*
* <br><b>Finding the chains of a subset of the graph</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new ChainSubstructures({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the chains
* for (const chain of result.chains) {
*   for (const node of chain.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of chain.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ChainSubstructures%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_ChainSubstructures: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ChainSubstructures;
}
export interface ChainsResult extends YObject {
}
/**
* Represents the chains found in the graph as computed by {@link ChainsResult#chains}.
* The chains are available in {@link Paths}.
*
* <br><b>Highlighting the edges along the chains</b>
* <pre>
* const result = new Chains({ directed: true }).run(graph)
* result.chains.forEach(path => {
*   path.edges.forEach(edge => {
*     graph.setStyle(edge, highlightEdgeStyle)
*   })
* })
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ChainsResult Online Documentation}
*/
export class ChainsResult {
/**
All chains in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChainsResult%23chains Online Documentation}*/
chains: ResultItemCollection<Path>;
static $class: Class;
private $_ChainsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ChainsResult;
}
export interface Chains extends YObject {
}
/**
* Finds all chains in a graph.
* A chain is a path of maximum length in which each internal node has degree <code>2</code>.
*
* The internal nodes on directed chains all have in-degree <code>1</code> and out-degree <code>1</code>.
*
* <b>Other Path-Related Algorithms</b>
*
* yFiles for HTML supports a number of other algorithms related to paths in a graph:
*
* <ul>
* <li><b>{@link Paths}</b> – finds all paths between a set of source and a set of target nodes</li>
* <li><b>{@link Cycle}</b> – finds a cycle if one exists</li>
* <li><b>{@link LongestPath}</b> – finds the longest path in the graph</li>
* </ul>
* <br><b>Highlighting the edges along the chains</b>
* <pre>
* const result = new Chains({ directed: true }).run(graph)
* result.chains.forEach(path => {
*   path.edges.forEach(edge => {
*     graph.setStyle(edge, highlightEdgeStyle)
*   })
* })
* </pre>
*
* <br><b>Highlighting all chains in a graph</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new Chains()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.chains.flatMap(c => c.edges)) {
*   graph.setStyle(edge, highlightChainsStyle)
* }
* </pre>
*
* @see PathAlgorithm#findAllChains
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Chains Online Documentation}
*/
export class Chains {
/**
Creates a new {@link Chains} instance.
@param [options] The parameters to pass.
@param [options.directed] A value indicating whether the graph is considered directed or not. This option sets the {@link Chains#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link Chains#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link Chains#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Chains Online Documentation}*/
constructor(options?: {
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Finds all chains in a graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link ChainsResult} which contains all {@link Path chains}.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see PathAlgorithm#findAllChains
@see {@link https://docs.yworks.com/yfileshtml/#/api/Chains%23run(IGraph) Online Documentation}*/
run(graph: IGraph): ChainsResult;
/**
Gets or sets a value indicating whether the graph is considered directed or not.
<code>true</code> if the chains should be considered directed, <code>false</code> otherwise.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/Chains%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link Chains#run} method.
*
* <br><b>Highlighting all chains on a subset of the graph</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new Chains({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.chains.flatMap(c => c.edges)) {
*   graph.setStyle(edge, highlightChainsStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Chains%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link Chains#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link Chains#run} method.
*
* <br><b>Highlighting all chains  on a subset of the graph</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new Chains({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.chains.flatMap(c => c.edges)) {
*   graph.setStyle(edge, highlightChainsStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Chains%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_Chains: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Chains;
}
export interface CliqueSubstructuresResult extends YObject {
}
/**
Represents the cliques found by {@link CliqueSubstructures}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CliqueSubstructuresResult Online Documentation}*/
export class CliqueSubstructuresResult {
/**
Gets the cliques found by {@link CliqueSubstructures}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CliqueSubstructuresResult%23cliques Online Documentation}*/
cliques: ResultItemCollection<SubstructureItems>;
static $class: Class;
private $_CliqueSubstructuresResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CliqueSubstructuresResult;
}
export interface CliqueSubstructures extends YObject {
}
/**
* Detects {@link SubstructureItems}s that represent the (undirected) cliques in the specified graph.
* A clique is a subset of nodes of the same type (NodeType) that are all adjacent (i.e., connected) to each other.
*
* Cliques with less than {@link CliqueSubstructures#minimumSize} nodes are ignored.
*
* Note that finding a maximum clique is NP-hard. Hence, we only use a simple heuristic approach that doesn't guarantee to
* find all/the largest clique. It only identifies cliques that have at least one node with at most one non-clique
* neighbor. Furthermore, a node can only be contained in a single clique, i.e., the returned cliques are non-overlapping.
*
* <h2>Note</h2>
*
* A clique only consists of elements with the same {@link CliqueSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link CliqueSubstructures#nodeTypes} is specified, all
* node are considered to be of the same type.
*
* <br><b>Finding the cliques in a graph</b>
* <pre>
* // prepare the clique detection algorithm
* const algorithm = new CliqueSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cliques
* for (const clique of result.cliques) {
*   for (const node of clique.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of clique.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the cliques in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the clique detection algorithm
* const algorithm = new CliqueSubstructures({
*   // only nodes with the same tags can be a clique
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cliques
* for (const clique of result.cliques) {
*   for (const node of clique.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of clique.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CliqueSubstructures Online Documentation}
*/
export class CliqueSubstructures {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.minimumSize] The minimum size of a clique. This option sets the {@link CliqueSubstructures#minimumSize} property on the created object.
@param [options.nodeTypes] A mapping which maps the type of each node. This option sets the {@link CliqueSubstructures#nodeTypes} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link CliqueSubstructures#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link CliqueSubstructures#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CliqueSubstructures Online Documentation}*/
constructor(options?: {
minimumSize?: number;
nodeTypes?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Returns a list of {@link SubstructureItems}s that represent the (undirected) cliques in the specified graph.
* A clique is a subset of nodes of the same type (NodeType) that are all adjacent (i.e., connected) to each other.
*
* Cliques with less than {@link CliqueSubstructures#minimumSize} nodes are ignored.
*
* Note that finding a maximum clique is NP-hard. Hence, we only use a simple heuristic approach that doesn't guarantee to
* find all/the largest clique. It only identifies cliques that have at least one node with at most one non-clique
* neighbor. Furthermore, a node can only be contained in a single clique, i.e., the returned cliques are non-overlapping.
*
* <h2>Note</h2>
*
* A clique only consists of elements with the same {@link CliqueSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link CliqueSubstructures#nodeTypes} is specified, all
* node are considered to be of the same type.
*
* <h2>Note</h2>
*
* The smallest {@link CliqueSubstructures#minimumSize} value that could be considered is <code>3</code>.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Finding the cliques in a graph</b>
* <pre>
* // prepare the clique detection algorithm
* const algorithm = new CliqueSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cliques
* for (const clique of result.cliques) {
*   for (const node of clique.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of clique.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the cliques in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the clique detection algorithm
* const algorithm = new CliqueSubstructures({
*   // only nodes with the same tags can be a clique
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cliques
* for (const clique of result.cliques) {
*   for (const node of clique.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of clique.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @param graph The graph to find the substructures in.
* @returns A list of {@link SubstructureItems}s that represent the cliques.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CliqueSubstructures%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): CliqueSubstructuresResult;
/**
Gets or sets the minimum size of a clique.
Cliques with less nodes are ignored.
Default is <code>3</code>.
<h2>Note</h2>
The smallest {@link CliqueSubstructures#minimumSize} value that could be considered is <code>3</code>.
@default 3
@see {@link https://docs.yworks.com/yfileshtml/#/api/CliqueSubstructures%23minimumSize Online Documentation}*/
minimumSize: number;
/**
Gets or sets a mapping which maps the type of each node.
An arbitrary object. Nodes returning equal objects are considered to be of the same type.
If none is provided all nodes are considered as equal.
<h2>Note</h2>
A clique only consists of elements with the same {@link CliqueSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link CliqueSubstructures#nodeTypes} is specified, all
node are considered to be of the same type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CliqueSubstructures%23nodeTypes Online Documentation}*/
nodeTypes: ItemMapping<INode, any>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link CliqueSubstructures#run}
* method.
*
* <br><b>Finding the cliques of a subset of the graph</b>
* <pre>
* // prepare the clique detection algorithm
* const algorithm = new CliqueSubstructures({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cliques
* for (const clique of result.cliques) {
*   for (const node of clique.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of clique.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CliqueSubstructures%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link CliqueSubstructures#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link CliqueSubstructures#run}
* method.
*
* <br><b>Finding the cliques of a subset of the graph</b>
* <pre>
* // prepare the clique detection algorithm
* const algorithm = new CliqueSubstructures({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cliques
* for (const clique of result.cliques) {
*   for (const node of clique.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of clique.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CliqueSubstructures%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_CliqueSubstructures: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CliqueSubstructures;
}
export interface SubstructureItems extends YObject {
}
/**
Represents a substructure of a graph, like a tree, chain, star, or clique.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureItems Online Documentation}*/
export class SubstructureItems {
/**
Gets the nodes that belong to this substructure.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureItems%23nodes Online Documentation}*/
nodes: ResultItemCollection<INode>;
/**
Gets the edges that belong to this substructure.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureItems%23edges Online Documentation}*/
edges: ResultItemCollection<IEdge>;
/**
Gets the edges connecting a node in the substructure with one outside of it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SubstructureItems%23interEdges Online Documentation}*/
interEdges: ResultItemCollection<IEdge>;
static $class: Class;
private $_SubstructureItems: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SubstructureItems;
}
export interface ClosenessCentralityResult extends YObject {
}
/**
Represents the node centrality values as computed by {@link ClosenessCentrality}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentralityResult Online Documentation}*/
export class ClosenessCentralityResult {
/**
Gets a mapping from each node to its absolute centrality value.
The absolute centrality value can change drastically depending on graph structure and edge weights. The {@link ClosenessCentralityResult#normalizedNodeCentrality} values don't
usually change as much, while still allowing to determine relative importance between nodes. If a {@link ClosenessCentrality#subgraphNodes subgraph} is specified
for the algorithm the value is <code>-1</code> for all nodes that are not part of the subgraph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentralityResult%23nodeCentrality Online Documentation}*/
nodeCentrality: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its normalized centrality value.
The normalized value is obtained by dividing {@link ClosenessCentralityResult#nodeCentrality} by {@link ClosenessCentralityResult#maximumNodeCentrality}. Thus the normalized values are all between <code>0</code> and <code>1</code>.
If the graph contains only one single node its normalized value will be positive infinity.
If a {@link ClosenessCentrality#subgraphNodes subgraph} is specified for the algorithm, though, the value is <code>-1</code> for all nodes that are not part of the subgraph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentralityResult%23normalizedNodeCentrality Online Documentation}*/
normalizedNodeCentrality: ResultItemMapping<INode, number>;
/**
Gets the minimum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentralityResult%23minimumNodeCentrality Online Documentation}*/
minimumNodeCentrality: number;
/**
Gets the maximum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentralityResult%23maximumNodeCentrality Online Documentation}*/
maximumNodeCentrality: number;
static $class: Class;
private $_ClosenessCentralityResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClosenessCentralityResult;
}
export interface ClosenessCentrality extends YObject {
}
/**
* Computes the closeness centrality for the nodes of a graph.
* <i>Closeness centrality</i> is defined as the reciprocal of the sum of shortest path distances of a node to all other nodes in the graph.
* Therefore, a node with high closeness centrality has short distances to all other nodes of a graph.
*
* <b>Other Centrality Measures</b>
*
* yFiles for HTML supports a number of other centrality measures:
*
* <ul>
* <li><b>{@link GraphCentrality}</b> – emphasizes nodes that have short paths to other nodes in a slightly different way</li>
* <li><b>{@link DegreeCentrality}</b> – emphasizes nodes with many edges</li>
* <li><b>{@link WeightCentrality}</b> – emphasizes nodes with highly-weighted edges</li>
* <li><b>{@link BetweennessCentrality}</b> – emphasizes nodes that are part of many short paths</li>
* <li><b>{@link EigenvectorCentrality}</b> – computes the influence a node has on a network. The centrality value is higher if more nodes are connected to that node</li>
* <li><b>{@link PageRank}</b> – computes page rank values for all nodes based on their attached edges</li>
* </ul>
* <h2>Note</h2>
*
* For graphs with multiple components, the centrality values will be calculated per component.
*
* <h2>Note</h2>
*
* For graphs which contain only a single node, the centrality value is {@link Number#POSITIVE_INFINITY}.
*
* <pre>
* const result = new ClosenessCentrality({
*   directed: true,
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* }).run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(key, String(value))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see CentralityAlgorithm#closenessCentrality
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentrality Online Documentation}
*/
export class ClosenessCentrality {
/**
Creates a new {@link ClosenessCentrality} instance.
@param [options] The parameters to pass.
@param [options.weights] A mapping for edge weights. This option sets the {@link ClosenessCentrality#weights} property on the created object.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link ClosenessCentrality#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link ClosenessCentrality#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link ClosenessCentrality#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentrality Online Documentation}*/
constructor(options?: {
weights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Computes the closeness centrality for the nodes of a graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link ClosenessCentralityResult} from which the calculated centrality values can be obtained.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentrality%23run(IGraph) Online Documentation}*/
run(graph: IGraph): ClosenessCentralityResult;
/**
Gets or sets a mapping for edge weights.
Closeness centrality computes shortest paths throughout the graph. Edge weights influence the computed length of those
paths and thus the centrality measure. If no weights are provided, all edges have the same uniform weight of <code>1</code> and the
number of edges is effectively the shortest path length.
Edge weights for closeness centrality must be positive.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentrality%23weights Online Documentation}*/
weights: ItemMapping<IEdge, number>;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentrality%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link ClosenessCentrality#run}
* method.
*
* <br><b>Calculating the closeness centrality on a subset of the graph</b>
* <pre>
* const result = new ClosenessCentrality({
*   directed: true,
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength(),
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* }).run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(entry => {
*   const node = entry.key
*   const centrality = entry.value
*   graph.addLabel(node, `${centrality}`)
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentrality%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link ClosenessCentrality#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link ClosenessCentrality#run}
* method.
*
* <br><b>Calculating the closeness centrality on a subset of the graph</b>
* <pre>
* const result = new ClosenessCentrality({
*   directed: true,
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength(),
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* }).run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(entry => {
*   const node = entry.key
*   const centrality = entry.value
*   graph.addLabel(node, `${centrality}`)
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClosenessCentrality%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_ClosenessCentrality: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClosenessCentrality;
}
export interface ClusteringCoefficientResult extends YObject {
}
/**
Represents the clustering coefficients in the graph as computed by {@link ClusteringCoefficient}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClusteringCoefficientResult Online Documentation}*/
export class ClusteringCoefficientResult {
/**
Gets a map which returns the clustering coefficient for each node.
The clustering coefficient measures the degree to which the nodes of a network tend to cluster together, see
https://en.wikipedia.org/wiki/Clustering_coefficient. More precisely, for a node n, the local clustering coefficient is
the actual number of edges between the neighbors of n divided by the maximum possible number of such edges. Hence, it is
always a {@link number} value between <code>0.0</code> and <code>1.0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClusteringCoefficientResult%23clusteringCoefficients Online Documentation}*/
clusteringCoefficients: ResultItemMapping<INode, number>;
/**
Gets the average clustering coefficient.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClusteringCoefficientResult%23averageCoefficient Online Documentation}*/
averageCoefficient: number;
static $class: Class;
private $_ClusteringCoefficientResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClusteringCoefficientResult;
}
export interface ClusteringCoefficient extends YObject {
}
/**
* Calculates the local clustering coefficient for each node and returns the average clustering coefficient.
* The clustering coefficient measures the degree to which the nodes of a network tend to cluster together, see
* https://en.wikipedia.org/wiki/Clustering_coefficient. More precisely, for a node n, the local clustering coefficient is
* the actual number of edges between the neighbors of n divided by the maximum possible number of such edges. Hence, it is
* always a {@link number} value between <code>0.0</code> and <code>1.0</code>.<b>Definitions</b>
*
* <ul>
* <li>The <b>local clustering coefficient</b> of a node depends on the number of edges between its neighbors. A value of <code>1</code> indicates that the node's neighbors
* form a clique (i.e., there is an edge between each pair of neighbors).</li>
* <li><b>Average clustering coefficient</b> is the average of all the local clustering coefficient values of a graph.</li>
* <li><b>Average weighted clustering coefficient</b> is the average of all the local clustering coefficient values of a graph where each local value of a node is weighted
* by the maximum possible number of edges between its neighbors.</li>
* </ul>
* <h2>Note</h2>
*
* The algorithm ignores self-loops and parallel edges. Group nodes are handled like common nodes.
*
* <br><b>Calculating the clustering coefficient.</b>
* <pre>
* // prepare the algorithm
* const algorithm = new ClusteringCoefficient({ directed: true })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add the result as label
* for (const node of graph.nodes) {
*   graph.addLabel(node, String(result.clusteringCoefficients.get(node)))
* }
* </pre>
*
* @see GroupAlgorithm#getClusteringCoefficient
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClusteringCoefficient Online Documentation}
*/
export class ClusteringCoefficient {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link ClusteringCoefficient#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link ClusteringCoefficient#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link ClusteringCoefficient#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClusteringCoefficient Online Documentation}*/
constructor(options?: {
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Calculates the local clustering coefficient for each node and returns the average clustering coefficient.
* The clustering coefficient measures the degree to which the nodes of a network tend to cluster together, see
* https://en.wikipedia.org/wiki/Clustering_coefficient. More precisely, for a node n, the local clustering coefficient is
* the actual number of edges between the neighbors of n divided by the maximum possible number of such edges. Hence, it is
* always a {@link number} value between <code>0.0</code> and <code>1.0</code>.
*
* <h2>Note</h2>
*
* The algorithm ignores self-loops and parallel edges. Group nodes are handled like common nodes.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Calculating the clustering coefficient.</b>
* <pre>
* // prepare the algorithm
* const algorithm = new ClusteringCoefficient({ directed: true })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add the result as label
* for (const node of graph.nodes) {
*   graph.addLabel(node, String(result.clusteringCoefficients.get(node)))
* }
* </pre>
*
* @param graph the input graph
* @returns The coefficients for each node and the average coefficient.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClusteringCoefficient%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): ClusteringCoefficientResult;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClusteringCoefficient%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link ClusteringCoefficient#run}
* method.
*
* <br><b>Calculating the clustering coefficient on a subset of the graph</b>
* <pre>
* // prepare the algorithm
* const algorithm = new ClusteringCoefficient({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add the result as label
* result.clusteringCoefficients.forEach(({ key: node, value: coefficient }) => {
*   graph.addLabel(node, String(coefficient))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClusteringCoefficient%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link ClusteringCoefficient#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link ClusteringCoefficient#run}
* method.
*
* <br><b>Calculating the clustering coefficient on a subset of the graph</b>
* <pre>
* // prepare the algorithm
* const algorithm = new ClusteringCoefficient({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add the result as label
* for (const node of graph.nodes) {
*   graph.addLabel(node, String(result.clusteringCoefficients.get(node)))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClusteringCoefficient%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_ClusteringCoefficient: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClusteringCoefficient;
}
export interface ConnectedComponentsResult extends YObject {
}
/**
Represents the connected components of a graph.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectedComponentsResult Online Documentation}*/
export class ConnectedComponentsResult {
/**
Gets a map which returns for each node the ID of the component the node belongs to.
If the node is not part of an optional {@link ConnectedComponents#subgraphNodes}, the return value is -1. Otherwise, the ID of a component is a non-negative {@link number}
value which is common for all nodes of the same component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectedComponentsResult%23nodeComponentIds Online Documentation}*/
nodeComponentIds: ResultItemMapping<INode, number>;
/**
Gets a map which returns for each node the {@link Component component} the node belongs to.
If the node does not belong to {@link ConnectedComponents#subgraphNodes}, the return value is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectedComponentsResult%23nodeComponents Online Documentation}*/
nodeComponents: ResultItemMapping<INode, Component>;
/**
Gets a collection of all connected components in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectedComponentsResult%23components Online Documentation}*/
components: ResultItemCollection<Component>;
static $class: Class;
private $_ConnectedComponentsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ConnectedComponentsResult;
}
export interface ConnectedComponents extends YObject {
}
/**
* Determines the connected components of a given graph.
* A graph <code>G</code> is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
*
* The connected components of a graph are the maximal connected subgraphs of which the graph consists.
*
* <b>Other Graph Connectivity Algorithms</b>
*
* yFiles for HTML supports a number of other analysis algorithms that partition the graph into components, based on
* various criteria.
*
* <ul>
* <li><b>{@link StronglyConnectedComponents}</b> – Determines components defined by the existence of a directed path between nodes</li>
* <li><b>{@link BiconnectedComponents}</b> – Determines components defined by the existence of at least two separate undirected paths between all nodes</li>
* <li><b>{@link Bipartition}</b> – Divides a graph into two partitions where all edges have their source and target in different partitions</li>
* <li><b>{@link IndependentSets}</b> – Divides a graph into partitions where no nodes are connected within a partition</li>
* <li><b>{@link KCoreComponents}</b> – Calculates the k-cores of an undirected input graph. The k-core of an undirected input graph consists of the subgraph
* components where each node has at least degree <code>k</code>.</li>
* </ul>
* <br><b>Calculating connected components on a graph</b>
* <pre>
* // prepare the connected components algorithm
* const algorithm = new ConnectedComponents()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the connected components with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeComponentIds.get(node)
*   graph.setStyle(node, componentStyles.get(componentId))
* }
* </pre>
*
* @see GraphConnectivity#connectedComponents
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectedComponents Online Documentation}
*/
export class ConnectedComponents {
/**
Creates a new {@link ConnectedComponents} instance.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link ConnectedComponents#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link ConnectedComponents#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectedComponents Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Determines the connected components of a given graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link ConnectedComponentsResult} containing the connected components of <code>graph</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectedComponents%23run(IGraph) Online Documentation}*/
run(graph: IGraph): ConnectedComponentsResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link ConnectedComponents#run}
* method.
*
* <br><b>Calculating connected components on a subset of the graph</b>
* <pre>
* // configure the connected components algorithm
* const algorithm = new ConnectedComponents({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the connected components with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeComponentIds.get(node)
*   graph.setStyle(node, componentStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectedComponents%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link ConnectedComponents#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link ConnectedComponents#run}
* method.
*
* <br><b>Calculating connected components on a subset of the graph</b>
* <pre>
* // configure the connected components algorithm
* const algorithm = new ConnectedComponents({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the connected components with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeComponentIds.get(node)
*   graph.setStyle(node, componentStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ConnectedComponents%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_ConnectedComponents: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ConnectedComponents;
}
export interface Component extends YObject {
}
/**
A single connected or strongly connected component, composed of the {@link Component#nodes} and {@link Component#inducedEdges} which belong to this component.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Component Online Documentation}*/
export class Component {
/**
Gets a collection of nodes belonging to this component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Component%23nodes Online Documentation}*/
nodes: ResultItemCollection<INode>;
/**
Gets a collection of edges belonging to this component which are induced by the component's {@link Component#nodes}.
These are all edges whose source and target node belong to this component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Component%23inducedEdges Online Documentation}*/
inducedEdges: ResultItemCollection<IEdge>;
static $class: Class;
private $_Component: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Component;
}
export interface CycleResult extends YObject {
}
/**
Represents a cycle's nodes and edges as found by {@link Cycle}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleResult Online Documentation}*/
export class CycleResult {
/**
Gets a collection of nodes in the cycle.
Nodes are in order they appear in the cycle.
If the graph has no cycles, this collection is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleResult%23nodes Online Documentation}*/
nodes: ResultItemCollection<INode>;
/**
Gets the edges of the cycle.
The edges are in the order they appear in the cycle.
If the graph has no cycles, this collection is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleResult%23edges Online Documentation}*/
edges: ResultItemCollection<IEdge>;
static $class: Class;
private $_CycleResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CycleResult;
}
export interface Cycle extends YObject {
}
/**
* Finds a directed or undirected cycle of edges in a graph.
* The graph can be {@link Cycle#directed directed} or undirected.
*
* <b>Other Path-Related Algorithms</b>
*
* yFiles for HTML supports a number of other algorithms related to paths in a graph:
*
* <ul>
* <li><b>{@link Paths}</b> – finds all paths between a set of source and a set of target nodes</li>
* <li><b>{@link Chains}</b> – finds all chains, that is, sequences of nodes that are each connected with just an edge without branches</li>
* <li><b>{@link LongestPath}</b> – finds the longest path in the graph</li>
* </ul>
* <br><b>Highlighting the edge of a cycle in a graph</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new Cycle({
*   // We are only interested in finding directed cycles
*   directed: true
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightCycleStyle)
* }
* </pre>
*
* @see CycleAlgorithm#findCycle
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Cycle Online Documentation}
*/
export class Cycle {
/**
Creates a new {@link Cycle} instance.
@param [options] The parameters to pass.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link Cycle#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link Cycle#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link Cycle#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cycle Online Documentation}*/
constructor(options?: {
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Finds a cycle in the given graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link CycleResult} with a cycle (if one exists in the graph).
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cycle%23run(IGraph) Online Documentation}*/
run(graph: IGraph): CycleResult;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cycle%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link Cycle#run} method.
*
* <br><b>Finding a cycle on a subset of the graph</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new Cycle({
*   // We are only interested in finding directed cycles
*   directed: true,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightCycleStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Cycle%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link Cycle#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link Cycle#run} method.
*
* <br><b>Finding a cycle on a subset of the graph</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new Cycle({
*   // We are only interested in finding directed cycles
*   directed: true,
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightCycleStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Cycle%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_Cycle: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Cycle;
}
export interface CycleEdgesResult extends YObject {
}
/**
Represents edges that are part of at least one directed or undirected simple cycle as computed by
{@link CycleEdges}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleEdgesResult Online Documentation}*/
export class CycleEdgesResult {
/**
Gets a collection of all edges that are part of a cycle.
If the graph has no cycles, this collection is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleEdgesResult%23edges Online Documentation}*/
edges: ResultItemCollection<IEdge>;
static $class: Class;
private $_CycleEdgesResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CycleEdgesResult;
}
export interface CycleEdges extends YObject {
}
/**
* Finds all edges that are part of at least one directed or undirected simple cycle in the graph.
* The graph can be {@link CycleEdges#directed directed} or undirected.
*
* <h2>Note</h2>
*
* Self-loops are always considered to be cycle edges.
*
* <br><b>Highlighting all cycles in a graph</b>
* <pre>
* // prepare the cycle edge detection algorithm
* const algorithm = new CycleEdges({
*   // We are only interested in finding directed cycles
*   directed: true
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightCycleEdgesStyle)
* }
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CycleEdges Online Documentation}
*/
export class CycleEdges {
/**
Creates a new {@link CycleEdges} instance.
@param [options] The parameters to pass.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link CycleEdges#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link CycleEdges#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link CycleEdges#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleEdges Online Documentation}*/
constructor(options?: {
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Finds all edges that are part of at least one directed or undirected simple cycle.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link CycleEdgesResult} with all edges that are part of at least one cycle in the graph.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleEdges%23run(IGraph) Online Documentation}*/
run(graph: IGraph): CycleEdgesResult;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleEdges%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link CycleEdges#run} method.
*
* <br><b>Finding all cycles on a subset of the graph</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new Cycle({
*   // We are only interested in finding directed cycles
*   directed: true,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightCycleStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CycleEdges%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link CycleEdges#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link CycleEdges#run} method.
*
* <br><b>Finding all cycles on a subset of the graph</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new Cycle({
*   // We are only interested in finding directed cycles
*   directed: true,
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightCycleStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CycleEdges%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_CycleEdges: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CycleEdges;
}
export interface CycleSubstructuresResult extends YObject {
}
/**
Represents the cycles found by {@link CycleSubstructures}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructuresResult Online Documentation}*/
export class CycleSubstructuresResult {
/**
Gets the cycles found by {@link CycleSubstructures}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructuresResult%23cycles Online Documentation}*/
cycles: ResultItemCollection<SubstructureItems>;
static $class: Class;
private $_CycleSubstructuresResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CycleSubstructuresResult;
}
export interface CycleSubstructures extends YObject {
}
/**
* Detects {@link SubstructureItems}s that represent the isolated cycles in the specified graph.
* A <em>cycle</em> is a simple edge path in which the first and last nodes are identical. The algorithm only considers cycles in which
* at most one edge connects a cycle node with the rest of the graph (i.e. isolated cycles that are connected with one
* edge). Furthermore, a cycle only consists of elements with the same {@link CycleSubstructures#edgeDirectedness} and {@link CycleSubstructures#nodeTypes}.
*
* The {@link CycleSubstructures#edgeDirectedness} is considered as follows: A substructure is only identified as such if all edges are either undirected or
* consistently directed with respect to the specified directedness.
*
* <ul>
* <li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.</li>
* <li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.</li>
* <li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.</li>
* </ul>
* <h2>Note</h2>
*
* The smallest {@link CycleSubstructures#minimumSize} value that could be considered is <code>3</code>.
*
* <h2>Note</h2>
*
* A chain or star structure is also considered to be a tree.
*
* <h2>Note</h2>
*
* If no {@link CycleSubstructures#edgeDirectedness} is specified, all edges are treated as undirected. Furthermore, if no {@link CycleSubstructures#nodeTypes} are specified, all node are considered to
* be of the same type.
*
* <br><b>Finding the cycles in a graph</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new CycleSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycles
* for (const cycle of result.cycles) {
*   for (const node of cycle.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of cycle.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the cycles in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new CycleSubstructures({
*   // only nodes with the same tags can be a cycle
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycles
* for (const cycle of result.cycles) {
*   for (const node of cycle.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of cycle.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructures Online Documentation}
*/
export class CycleSubstructures {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.minimumSize] The minimum size of a cycle. This option sets the {@link CycleSubstructures#minimumSize} property on the created object.
@param [options.nodeTypes] A mapping which maps the type of each node. This option sets the {@link CycleSubstructures#nodeTypes} property on the created object.
@param [options.edgeDirectedness] A mapping that stores the directedness of the edges. This option sets the {@link CycleSubstructures#edgeDirectedness} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link CycleSubstructures#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link CycleSubstructures#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructures Online Documentation}*/
constructor(options?: {
minimumSize?: number;
nodeTypes?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Returns a list of {@link SubstructureItems}s that represent isolated cycles in the specified graph.
* A cycle only consists of elements with the same {@link CycleSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link CycleSubstructures#nodeTypes} is specified, all
* node are considered to be of the same type.
*
* The {@link CycleSubstructures#edgeDirectedness} is considered as follows: A substructure is only identified as such if all edges are either undirected or
* consistently directed with respect to the specified directedness.
*
* <ul>
* <li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.</li>
* <li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.</li>
* <li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.</li>
* </ul>
* <h2>Note</h2>
*
* The smallest {@link CycleSubstructures#minimumSize} value that could be considered is <code>3</code>.
*
* <h2>Note</h2>
*
* A chain or star structure is also considered to be a tree.
*
* <h2>Note</h2>
*
* If no {@link CycleSubstructures#edgeDirectedness} is specified, all edges are treated as undirected. Furthermore, if no {@link CycleSubstructures#nodeTypes} are specified, all node are considered to
* be of the same type.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Finding the cycles in a graph</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new CycleSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycles
* for (const cycle of result.cycles) {
*   for (const node of cycle.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of cycle.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the cycles in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new CycleSubstructures({
*   // only nodes with the same tags can be a cycle
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycles
* for (const cycle of result.cycles) {
*   for (const node of cycle.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of cycle.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @param graph The graph to find the substructures in.
* @returns A list of {@link SubstructureItems}s that represent the cycles.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructures%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): CycleSubstructuresResult;
/**
Gets or sets the minimum size of a cycle.
Cycles with less nodes are ignored.
Default is <code>3</code>.
<h2>Note</h2>
The smallest {@link CycleSubstructures#minimumSize} value that could be considered is <code>3</code>.
@default 3
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructures%23minimumSize Online Documentation}*/
minimumSize: number;
/**
Gets or sets a mapping which maps the type of each node.
An arbitrary object. Nodes returning equal objects are considered to be of the same type.
If none is provided all nodes are considered as equal.
<h2>Note</h2>
A cycle only consists of elements with the same {@link CycleSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link CycleSubstructures#nodeTypes} is specified, all
node are considered to be of the same type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructures%23nodeTypes Online Documentation}*/
nodeTypes: ItemMapping<INode, any>;
/**
Gets or sets a mapping that stores the directedness of the edges.
If a value is provided here, the algorithm only considers a substructure if all edges are either undirected or
consistently directed with respect to the specified directedness.
<ul>
<li>the edge's directedness is greater than <code>0.0</code> and the edge is incoming (i.e. n is the edge's target),
<li>the edge's directedness is smaller than <code>0.0</code> and the edge is outgoing (i.e. n is the edge's source),
<li>or the edge's directedness is <code>0.0</code> (i.e. the edge is considered to be <b>undirected</b>).
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructures%23edgeDirectedness Online Documentation}*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link CycleSubstructures#run}
* method.
*
* <br><b>Finding the cycles of a subset of the graph</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new CycleSubstructures({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycles
* for (const cycle of result.cycles) {
*   for (const node of cycle.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of cycle.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructures%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link CycleSubstructures#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link CycleSubstructures#run}
* method.
*
* <br><b>Finding the cycles of a subset of the graph</b>
* <pre>
* // prepare the cycle detection algorithm
* const algorithm = new CycleSubstructures({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycles
* for (const cycle of result.cycles) {
*   for (const node of cycle.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of cycle.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CycleSubstructures%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_CycleSubstructures: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CycleSubstructures;
}
export interface DegreeCentralityResult extends YObject {
}
/**
Represents the node centrality values as computed by {@link DegreeCentrality}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentralityResult Online Documentation}*/
export class DegreeCentralityResult {
/**
Gets a mapping from each node to its absolute centrality value.
The absolute centrality value can change drastically depending on graph structure and edge weights. The {@link DegreeCentralityResult#normalizedNodeCentrality} values don't
usually change as much, while still allowing to determine relative importance between nodes. If a {@link DegreeCentrality#subgraphNodes subgraph} is specified
for the algorithm, the value is <code>-1</code> for all nodes that are not part of the subgraph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentralityResult%23nodeCentrality Online Documentation}*/
nodeCentrality: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its normalized centrality value.
The normalized value is obtained by dividing {@link DegreeCentralityResult#nodeCentrality} by {@link DegreeCentralityResult#maximumNodeCentrality}. Thus the normalized values are all between <code>0</code> and <code>1</code>. If a {@link DegreeCentrality#subgraphNodes subgraph} is
specified for the algorithm, the value is <code>-1</code> for all nodes that are not part of the subgraph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentralityResult%23normalizedNodeCentrality Online Documentation}*/
normalizedNodeCentrality: ResultItemMapping<INode, number>;
/**
Gets the minimum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentralityResult%23minimumNodeCentrality Online Documentation}*/
minimumNodeCentrality: number;
/**
Gets the maximum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentralityResult%23maximumNodeCentrality Online Documentation}*/
maximumNodeCentrality: number;
static $class: Class;
private $_DegreeCentralityResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DegreeCentralityResult;
}
export interface DegreeCentrality extends YObject {
}
/**
* Computes the degree centrality for the nodes of a given graph.
* <i>Degree centrality</i> is the number of the incoming, outgoing or overall edges incident to a node (measures incoming, outgoing and overall
* degree).
*
* If both {@link DegreeCentrality#considerIncomingEdges} and {@link DegreeCentrality#considerOutgoingEdges} are <code>false</code> an {@link Exception InvalidOperationError} is thrown.
*
* <b>Other Centrality Measures</b>
*
* yFiles for HTML supports a number of other centrality measures:
*
* <ul>
* <li><b>{@link GraphCentrality}, {@link ClosenessCentrality}</b> – emphasize nodes that have short paths to other nodes</li>
* <li><b>{@link WeightCentrality}</b> – emphasizes nodes with highly-weighted edges</li>
* <li><b>{@link BetweennessCentrality}</b> – emphasizes nodes and edges that are part of many short paths</li>
* <li><b>{@link EigenvectorCentrality}</b> – computes the influence a node has on a network. The centrality value is higher if more nodes are connected to that node</li>
* <li><b>{@link PageRank}</b> – computes page rank values for all nodes based on their attached edges</li>
* </ul>
* <pre>
* const result = new DegreeCentrality().run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(entry => {
*   const node = entry.key
*   const centrality = entry.value
*   graph.addLabel(node, `${centrality}`)
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see CentralityAlgorithm#degreeCentrality
* @see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentrality Online Documentation}
*/
export class DegreeCentrality {
/**
Creates a new {@link DegreeCentrality} instance.
@param [options] The parameters to pass.
@param [options.considerIncomingEdges] A value indicating whether incoming edges contribute to the centrality values of their target nodes. This option sets the {@link DegreeCentrality#considerIncomingEdges} property on the created object.
@param [options.considerOutgoingEdges] A value indicating whether outgoing edges contribute to the centrality values of their source nodes. This option sets the {@link DegreeCentrality#considerOutgoingEdges} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link DegreeCentrality#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link DegreeCentrality#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentrality Online Documentation}*/
constructor(options?: {
considerIncomingEdges?: boolean;
considerOutgoingEdges?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Computes the degree centrality for the nodes of a given graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link DegreeCentralityResult} from which the calculated centrality values can be obtained.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentrality%23run(IGraph) Online Documentation}*/
run(graph: IGraph): DegreeCentralityResult;
/**
Gets or sets a value indicating whether incoming edges contribute to the centrality values of their target nodes.
If both {@link DegreeCentrality#considerIncomingEdges} and {@link DegreeCentrality#considerOutgoingEdges} are <code>false</code> an {@link Exception InvalidOperationError} is thrown.
Default is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentrality%23considerIncomingEdges Online Documentation}*/
considerIncomingEdges: boolean;
/**
Gets or sets a value indicating whether outgoing edges contribute to the centrality values of their source nodes.
If both {@link DegreeCentrality#considerIncomingEdges} and {@link DegreeCentrality#considerOutgoingEdges} are <code>false</code> an {@link Exception InvalidOperationError} is thrown.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentrality%23considerOutgoingEdges Online Documentation}*/
considerOutgoingEdges: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link DegreeCentrality#run} method.
*
* <br><b>Calculating the degree centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new DegreeCentrality({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(entry => {
*   const node = entry.key
*   const centrality = entry.value
*   graph.addLabel(node, `${centrality}`)
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentrality%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link DegreeCentrality#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link DegreeCentrality#run} method.
*
* <br><b>Calculating the degree centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new DegreeCentrality({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(entry => {
*   const node = entry.key
*   const centrality = entry.value
*   graph.addLabel(node, `${centrality}`)
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/DegreeCentrality%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_DegreeCentrality: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DegreeCentrality;
}
export interface EdgeBetweennessClusteringResult extends YObject {
}
/**
Represents clusters in the graph as computed by {@link EdgeBetweennessClustering}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClusteringResult Online Documentation}*/
export class EdgeBetweennessClusteringResult {
/**
Gets a mapping from each node to its cluster ID.
If no {@link EdgeBetweennessClustering#subgraphNodes subgraph} is specified, the ID of a cluster is a non-negative {@link number} value which is common for all nodes of the same
cluster.
If a {@link EdgeBetweennessClustering#subgraphNodes subgraph} is specified, all nodes that are <b>not</b> part of the subgraph have the ID <code>-1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClusteringResult%23nodeClusterIds Online Documentation}*/
nodeClusterIds: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its cluster.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClusteringResult%23nodeClusters Online Documentation}*/
nodeClusters: ResultItemMapping<INode, Cluster>;
/**
Gets a collection of all clusters in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClusteringResult%23clusters Online Documentation}*/
clusters: ResultItemCollection<Cluster>;
/**
Gets a value which estimates the quality of the partition of the nodes into the {@link EdgeBetweennessClusteringResult#clusters}.
The modularity value estimates the quality of the partition of the nodes into the given communities and is a value
between
<code>[-1/2,1]</code>. High modularity values denote that nodes with dense connections are placed in the same community, while nodes
with sparse connections are assigned to different communities.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClusteringResult%23modularity Online Documentation}*/
modularity: number;
static $class: Class;
private $_EdgeBetweennessClusteringResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeBetweennessClusteringResult;
}
export interface EdgeBetweennessClustering extends YObject {
}
/**
* Partitions the graph into clusters using <i>edge betweenness centrality</i>, as proposed by Girvan and Newman.
* <b>Betweenness centrality</b> is a measure for how often a node lies on a shortest path between each pair of nodes in the graph.
*
* In each iteration the edge with the highest betweenness centrality is removed from the graph. The method stops if there
* are no more edges to remove or if the requested maximum number of clusters is found. The clustering with the best
* quality reached during the process is returned.
*
* The algorithm includes several heuristic speed-up techniques available through the
* {@link EdgeBetweennessClustering#qualityTimeRatio quality/time ratio}. For the highest quality setting, it is used almost unmodified. The fast betweenness approximation of
* Brandes and Pich (<i>Centrality Estimation in Large Networks</i>) is employed for values around <code>0.5</code>. Typically, this results in a tiny decrease in quality but a large
* speed-up and is the recommended setting. To achieve the lowest running time, a local betweenness calculation is used
* (Gregory: <i>Local Betweenness for Finding Communities in Networks</i>).
*
* <b>Other Clustering Algorithms</b>
*
* yFiles for HTML supports a number of other clustering algorithms:
*
* <ul>
* <li><b>{@link KMeansClustering}</b> – partitions the graph into clusters based on the distance between nodes and the cluster midpoints.</li>
* <li><b>{@link HierarchicalClustering}</b> – partitions the graph into clusters by merging smaller clusters based on their distance.</li>
* <li><b>{@link BiconnectedComponentClustering}</b> – partitions the graph into clusters based on its biconnected components.</li>
* <li><b>{@link LouvainModularityClustering}</b> – partitions the graph into clusters by applying the <i>Louvain modularity</i> method.</li>
* <li><b>{@link LabelPropagationClustering}</b> – partitions the graph into clusters by applying a label propagation algorithm.</li>
* </ul>
* <br><b>Calculating the edge betweenness clusters of a graph</b>
* <pre>
* // prepare the edge betweenness clustering algorithm
* const algorithm = new EdgeBetweennessClustering()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see GroupAlgorithm#edgeBetweennessClustering
* @see GroupAlgorithm#edgeBetweennessClustering
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering Online Documentation}
*/
export class EdgeBetweennessClustering {
/**
Creates a new {@link EdgeBetweennessClustering} instance.
@param [options] The parameters to pass.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link EdgeBetweennessClustering#directed} property on the created object.
@param [options.minimumClusterCount] The minimum number of clusters that will be returned. This option sets the {@link EdgeBetweennessClustering#minimumClusterCount} property on the created object.
@param [options.maximumClusterCount] The maximum number of clusters that will be returned. This option sets the {@link EdgeBetweennessClustering#maximumClusterCount} property on the created object.
@param [options.weights] A mapping for edge weights. This option sets the {@link EdgeBetweennessClustering#weights} property on the created object.
@param [options.qualityTimeRatio] A desired quality value for the result. This option sets the {@link EdgeBetweennessClustering#qualityTimeRatio} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link EdgeBetweennessClustering#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link EdgeBetweennessClustering#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering Online Documentation}*/
constructor(options?: {
directed?: boolean;
minimumClusterCount?: number;
maximumClusterCount?: number;
weights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
qualityTimeRatio?: number;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Partitions the graph into groups using <i>edge betweenness centrality</i>.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link EdgeBetweennessClusteringResult} containing the computed clusters.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering%23run(IGraph) Online Documentation}*/
run(graph: IGraph): EdgeBetweennessClusteringResult;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
<b>Warning:</b> Directedness is ignored if {@link EdgeBetweennessClustering#qualityTimeRatio} is less than <code>1</code> and the graph is considered undirected.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering%23directed Online Documentation}*/
directed: boolean;
/**
Gets or sets the minimum number of clusters that will be returned.
A negative value (or <code>0</code>) will not constrain the minimum number of clusters.
Default is <code>0</code>.
@default 0
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering%23minimumClusterCount Online Documentation}*/
minimumClusterCount: number;
/**
Gets or sets the maximum number of clusters that will be returned.
A negative value will not limit the number of clusters.
Despite the value of this setting, the number of clusters is always at least the number of connected components in the
graph.
Default is <code>-1</code> meaning that there is no limit for the number of clusters.
@default -1
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering%23maximumClusterCount Online Documentation}*/
maximumClusterCount: number;
/**
Gets or sets a mapping for edge weights.
Edge weights influence the importance of certain edges. If no weights are provided at all, all edges have the same
uniform weight of <code>1</code>.
Edge weights for edge betweenness clustering must be finite and positive.
<b>Warning:</b> Edge weights are ignored if {@link EdgeBetweennessClustering#qualityTimeRatio} is less than <code>1</code>, as the heuristics assume uniform weights.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering%23weights Online Documentation}*/
weights: ItemMapping<IEdge, number>;
/**
Gets or sets a desired quality value for the result.
This must be a value between <code>0.0</code> (low quality, faster) and <code>1.0</code> (high quality, slower). For large graphs the recommended
value is <code>0.5</code>.
Default is <code>1</code>.
<b>Warning:</b> For a {@link EdgeBetweennessClustering#qualityTimeRatio} less than <code>1</code> the properties {@link EdgeBetweennessClustering#directed} and {@link EdgeBetweennessClustering#weights} are ignored.
@default 1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering%23qualityTimeRatio Online Documentation}*/
qualityTimeRatio: number;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link EdgeBetweennessClustering#run}
* method.
*
* <br><b>Calculating the edge betweenness clusters on a subset of the graph</b>
* <pre>
* // prepare the edge betweenness clustering algorithm
* const algorithm = new EdgeBetweennessClustering({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link EdgeBetweennessClustering#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link EdgeBetweennessClustering#run}
* method.
*
* <br><b>Calculating the edge betweenness clusters on a subset of the graph</b>
* <pre>
* // prepare the edge betweenness clustering algorithm
* const algorithm = new EdgeBetweennessClustering({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBetweennessClustering%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_EdgeBetweennessClustering: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeBetweennessClustering;
}
export interface Cluster extends YObject {
}
/**
Represents a cluster, that is, a group of nodes that by some measure belong closely together.
@see BiconnectedComponentClustering
@see EdgeBetweennessClustering
@see HierarchicalClustering
@see KMeansClustering
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cluster Online Documentation}*/
export class Cluster {
/**
Gets a collection of all nodes in the cluster.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Cluster%23nodes Online Documentation}*/
nodes: ResultItemCollection<INode>;
static $class: Class;
private $_Cluster: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Cluster;
}
export interface EigenvectorCentralityResult extends YObject {
}
/**
Represents the node centrality values as computed by {@link EigenvectorCentrality}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/EigenvectorCentralityResult Online Documentation}*/
export class EigenvectorCentralityResult {
/**
Gets a mapping from each node to its centrality value.
A value between <code>0.0</code> and <code>1.0</code> that stores centrality found for this node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EigenvectorCentralityResult%23nodeCentrality Online Documentation}*/
nodeCentrality: ResultItemMapping<INode, number>;
/**
Gets whether a valid result could be obtained.
For some input graphs the power iteration method that is used to calculate the dominant eigenvector doesn't converge
and, thus, this method doesn't find a valid solution. In this case this property is <code>false</code>. In such cases, we recommend
to switch to the {@link PageRank} algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EigenvectorCentralityResult%23isValid Online Documentation}*/
isValid: boolean;
static $class: Class;
private $_EigenvectorCentralityResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EigenvectorCentralityResult;
}
export interface EigenvectorCentrality extends YObject {
}
/**
* Computes an eigenvector centrality for each node of a given undirected, unweighted graph.
* Eigenvector centrality is a measure of the influence a node has on a network: The more nodes point to a node the higher
* is that node's centrality.
*
* The centrality values are scaled so that the largest centrality value is <code>1.0</code>.
*
* <b>Other Centrality Measures</b>
*
* yFiles for HTML supports a number of other centrality measures:
*
* <ul>
* <li><b>{@link GraphCentrality}, {@link ClosenessCentrality}</b> – emphasize nodes that have short paths to other nodes</li>
* <li><b>{@link DegreeCentrality}</b> – emphasizes nodes with many edges</li>
* <li><b>{@link WeightCentrality}</b> – emphasizes nodes with highly-weighted edges</li>
* <li><b>{@link BetweennessCentrality}</b> – emphasizes nodes and edges that are part of many short paths</li>
* <li><b>{@link PageRank}</b> – computes page rank values for all nodes based on their attached edges</li>
* </ul>
* <h2>Note</h2>
*
* The {@link PageRank} algorithm is also based on eigenvector centrality but allows for more configuration
* options.
*
* <h2>Note</h2>
*
* For some input graphs the power iteration method that is used to calculate the dominant eigenvector doesn't converge
* and, thus, this method doesn't find a valid solution. This is indicated by a <code>false</code> {@link EigenvectorCentralityResult#isValid} of the result. In such cases, we
* recommend to switch to the {@link PageRank} algorithm.
*
* <pre>
* const result = new EigenvectorCentrality().run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.nodeCentrality.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(node, String(centrality))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EigenvectorCentrality Online Documentation}
*/
export class EigenvectorCentrality {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.precision] The precision used during the calculation of the power iteration method, i.e., the maximum possible difference to consider two values as equal. This option sets the {@link EigenvectorCentrality#precision} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link EigenvectorCentrality#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link EigenvectorCentrality#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EigenvectorCentrality Online Documentation}*/
constructor(options?: {
precision?: number;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Computes an eigenvector centrality for each node of a given undirected, unweighted graph.
* Eigenvector centrality is a measure of the influence a node has on a network: The more nodes point to a node the higher
* is that node's centrality.
*
* The centrality values are scaled so that the largest centrality value is <code>1.0</code>.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <pre>
* const result = new EigenvectorCentrality().run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.nodeCentrality.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(node, String(centrality))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @param graph The input graph to run the algorithm on.
* @returns A {@link EigenvectorCentralityResult} from which the calculated centrality values can be obtained.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EigenvectorCentrality%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): EigenvectorCentralityResult;
/**
Gets or sets the precision used during the calculation of the power iteration method, i.e., the maximum possible
difference to consider two values as equal.
The default is <code>0.001</code>.
@default 0.001
@see {@link https://docs.yworks.com/yfileshtml/#/api/EigenvectorCentrality%23precision Online Documentation}*/
precision: number;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link EigenvectorCentrality#run}
* method.
*
* <br><b>Calculating the eigenvector centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new EigenvectorCentrality({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add labels for centrality values
* result.nodeCentrality.forEach(({ key, value }) => {
*   graph.addLabel(key, String(value))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EigenvectorCentrality%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link EigenvectorCentrality#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link EigenvectorCentrality#run}
* method.
*
* <br><b>Calculating the eigenvector centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new EigenvectorCentrality({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add labels for centrality values
* result.nodeCentrality.forEach(({ key, value }) => {
*   graph.addLabel(key, String(value))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EigenvectorCentrality%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_EigenvectorCentrality: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EigenvectorCentrality;
}
export interface FeedbackEdgeSetResult extends YObject {
}
/**
Represents edges that can be removed or reversed in a graph to make it acyclic as computed by {@link FeedbackEdgeSetResult#feedbackEdgeSet}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/FeedbackEdgeSetResult Online Documentation}*/
export class FeedbackEdgeSetResult {
/**
Gets a collection of edges whose removal or reversal would make the graph acyclic.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FeedbackEdgeSetResult%23feedbackEdgeSet Online Documentation}*/
feedbackEdgeSet: ResultItemCollection<IEdge>;
static $class: Class;
private $_FeedbackEdgeSetResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FeedbackEdgeSetResult;
}
export interface FeedbackEdgeSet extends YObject {
}
/**
* Finds edges of a given graph whose removal or reversal would make the graph acyclic (also called <i>Feedback Arc Set</i>).
* This minimization is performed heuristically, since it is a well known hard problem to come up with an optimal
* solution.
*
* If {@link FeedbackEdgeSet#costs} are provided the algorithm tries to minimize the cost associated with the marked edges. Otherwise, a faster
* algorithm based on a depth-first search is used. This approach also provides more stable results when edges are added or
* removed over time.
*
* <b>Other Tree-Related Algorithms</b>
*
* yFiles for HTML supports a number of other algorithms and helpers related to trees:
*
* <ul>
* <li><b>{@link SpanningTree}</b> – calculates a (minimum) spanning tree for a graph</li>
* <li><b>{@link TreeAnalyzer}</b> – provides utility methods for analyzing directed trees</li>
* </ul>
* <br><b>Highlighting a feedback edge set of a graph</b>
* <pre>
* // prepare the feedback set detection algorithm
* const algorithm = new FeedbackEdgeSet()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.feedbackEdgeSet) {
*   graph.setStyle(edge, highlightFeedbackEdgeSetStyle)
* }
* </pre>
*
* @see CycleAlgorithm#findCycleEdges
* @see CycleAlgorithm#findCycleEdgesDFS
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FeedbackEdgeSet Online Documentation}
*/
export class FeedbackEdgeSet {
/**
Creates a new {@link FeedbackEdgeSet} instance.
@param [options] The parameters to pass.
@param [options.costs] A mapping for edge costs. This option sets the {@link FeedbackEdgeSet#costs} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link FeedbackEdgeSet#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link FeedbackEdgeSet#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FeedbackEdgeSet Online Documentation}*/
constructor(options?: {
costs?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Finds the edges of a given graph whose removal or reversal would make the graph acyclic.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link FeedbackEdgeSetResult} containing the edges whose removal or reversal would make the graph acyclic.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FeedbackEdgeSet%23run(IGraph) Online Documentation}*/
run(graph: IGraph): FeedbackEdgeSetResult;
/**
Gets or sets a mapping for edge costs.
When specifying costs the algorithm will try minimizing the total cost of edges that have to be removed or reversed to
make the graph acyclic.
Costs must not be negative.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FeedbackEdgeSet%23costs Online Documentation}*/
costs: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link FeedbackEdgeSet#run} method.
*
* <br><b>Highlighting a feedback edge set on a subset of the graph</b>
* <pre>
* // prepare the feedback set detection algorithm
* const algorithm = new FeedbackEdgeSet({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.feedbackEdgeSet) {
*   graph.setStyle(edge, highlightFeedbackEdgeSetStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FeedbackEdgeSet%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link FeedbackEdgeSet#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link FeedbackEdgeSet#run} method.
*
* <br><b>Highlighting a feedback edge set on a subset of the graph</b>
* <pre>
* // prepare the feedback set detection algorithm
* const algorithm = new FeedbackEdgeSet({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.feedbackEdgeSet) {
*   graph.setStyle(edge, highlightFeedbackEdgeSetStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FeedbackEdgeSet%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_FeedbackEdgeSet: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FeedbackEdgeSet;
}
export interface GraphCentralityResult extends YObject {
}
/**
Represents the node centrality values as computed by {@link GraphCentrality}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentralityResult Online Documentation}*/
export class GraphCentralityResult {
/**
Gets a mapping from each node to its centrality value.
The absolute centrality value can change drastically depending on graph structure and edge weights. The {@link GraphCentralityResult#normalizedNodeCentrality} values don't
usually change as much, while still allowing to determine relative importance between nodes. If a {@link GraphCentrality#subgraphNodes subgraph} is specified
for the algorithm, the value is <code>-1</code> for all nodes that are not part of the subgraph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentralityResult%23nodeCentrality Online Documentation}*/
nodeCentrality: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its normalized centrality value.
The normalized value is obtained by dividing {@link GraphCentralityResult#nodeCentrality} by {@link GraphCentralityResult#maximumNodeCentrality}. Thus the normalized values are all between <code>0</code> and <code>1</code>. If a {@link GraphCentrality#subgraphNodes subgraph} is
specified for the algorithm, the value is <code>-1</code> for all nodes that are not part of the subgraph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentralityResult%23normalizedNodeCentrality Online Documentation}*/
normalizedNodeCentrality: ResultItemMapping<INode, number>;
/**
Gets the minimum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentralityResult%23minimumNodeCentrality Online Documentation}*/
minimumNodeCentrality: number;
/**
Gets the maximum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentralityResult%23maximumNodeCentrality Online Documentation}*/
maximumNodeCentrality: number;
static $class: Class;
private $_GraphCentralityResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphCentralityResult;
}
export interface GraphCentrality extends YObject {
}
/**
* Computes the graph centrality for the nodes of a graph.
* <i>Graph centrality</i> is defined as the reciprocal of the maximum of all shortest path distances from a node to all other nodes in the graph.
* Nodes with high graph centrality have short distances to all other nodes in the graph. <b>Other Centrality Measures</b>
*
* yFiles for HTML supports a number of other centrality measures:
*
* <ul>
* <li><b>{@link ClosenessCentrality}</b> – emphasize nodes that have short paths to other nodes in a slightly different way</li>
* <li><b>{@link DegreeCentrality}</b> – emphasizes nodes with many edges</li>
* <li><b>{@link WeightCentrality}</b> – emphasizes nodes with highly-weighted edges</li>
* <li><b>{@link BetweennessCentrality}</b> – emphasizes nodes and edges that are part of many short paths</li>
* <li><b>{@link EigenvectorCentrality}</b> – computes the influence a node has on a network. The centrality value is higher if more nodes are connected to that node</li>
* <li><b>{@link PageRank}</b> – computes page rank values for all nodes based on their attached edges</li>
* </ul>
* <h2>Note</h2>
*
* For graphs with multiple components, the centrality values will be calculated per component.
*
* <h2>Note</h2>
*
* When a graph (or component) consists of a single node, this node is assumed to have a centrality of <code>1</code>.
*
* <pre>
* const result = new GraphCentrality({
*   directed: true,
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* }).run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(key, String(value))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see CentralityAlgorithm#graphCentrality
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentrality Online Documentation}
*/
export class GraphCentrality {
/**
Creates a new {@link GraphCentrality} instance.
@param [options] The parameters to pass.
@param [options.weights] A mapping for edge weights. This option sets the {@link GraphCentrality#weights} property on the created object.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link GraphCentrality#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link GraphCentrality#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link GraphCentrality#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentrality Online Documentation}*/
constructor(options?: {
weights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Computes the graph centrality for the nodes of a graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link GraphCentralityResult} from which the calculated centrality values can be obtained.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentrality%23run(IGraph) Online Documentation}*/
run(graph: IGraph): GraphCentralityResult;
/**
Gets or sets a mapping for edge weights.
Graph centrality computes shortest paths throughout the graph. Edge weights influence the computed length of those paths
and thus the centrality measure. If no weights are provided, all edges have the same uniform weight of <code>1</code> and the number
of edges is effectively the shortest path length.
Edge weights for graph centrality must be positive.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentrality%23weights Online Documentation}*/
weights: ItemMapping<IEdge, number>;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentrality%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link GraphCentrality#run} method.
*
* <br><b>Calculating the graph centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new GraphCentrality({
*   directed: true,
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength(),
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(entry => {
*   const node = entry.key
*   const centrality = entry.value
*   graph.addLabel(node, `${centrality}`)
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentrality%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link GraphCentrality#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link GraphCentrality#run} method.
*
* <br><b>Calculating the graph centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new GraphCentrality({
*   directed: true,
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength(),
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(entry => {
*   const node = entry.key
*   const centrality = entry.value
*   graph.addLabel(node, `${centrality}`)
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphCentrality%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_GraphCentrality: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphCentrality;
}
export interface GraphStructureAnalyzer extends YObject {
}
/**
* This class provides methods that check structural properties of a given graph.
* <b>Definitions</b>
*
* <ul>
* <li><b>Cycle</b> – An edge path with nodes <code>n0, n1, n2, ... , nk</code> forms a cycle if <code>n0 = nk</code> and consists of at least one edge.</li>
* <li><b>Acyclic graph</b> – A graph that contains no directed cycle.</li>
* <li><b>Cyclic graph</b> – A graph that contains a directed cycle.</li>
* <li><b>Connected graph</b> – A graph in which there exists an undirected path of edges between every pair of nodes.</li>
* <li><b>Strongly connected graph</b> – A graph in which there exists a directed path between each pair of nodes.</li>
* <li><b>Biconnected graph</b> – A graph that has no cut vertex or articulation point (i.e., a node whose removal disconnects the graph).</li>
* <li><b>Bipartite graph</b> – A graph whose nodes can be partitioned into two sets such that each edge connects two nodes of different sets.</li>
* <li><b>Tree</b> – A acyclic graph, in which any pair of nodes is connected through a path. If one node of a tree is distinguished from the
* other nodes, then the tree is called <b>rooted tree</b>.</li>
* <li><b>N-ary tree</b> – A directed rooted tree where each node has a maximum of <code>n</code> children.</li>
* <li><b>Forest</b> – A graph whose connected components are trees.</li>
* <li><b>Simple graph</b> – An undirected graph that contains no self-loops or parallel edges.</li>
* <li><b>Planar graph</b> – A graph that can be drawn on the plane without edge crossings.</li>
* <li><b>Multiple Edges</b> – Multiple edges are edges that are incident to the same two nodes.</li>
* </ul>
* An instance of {@link GraphStructureAnalyzer} is valid for an {@link IGraph} instance.
*
* <pre>
* const analyzer = new GraphStructureAnalyzer(graph)
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer Online Documentation}
*/
export class GraphStructureAnalyzer {
/**
Creates a new instance for the given graph.
@param graph The graph to provide analysis methods for.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer Online Documentation}*/
constructor(graph: IGraph);
/**
Computes the average degree of the graph.
The average degree measures the number of edges in comparison to the number of nodes and is defined as:
<ul>
<li><code>|E| / |V|</code> for directed graphs
<li><code>2 * |E| / |V|</code> for undirected graphs
</ul>
<h2>Note</h2>
Parallel edges and self-loops are both included in the calculation.
@param [directed=true] Whether or not to respect the direction of the edges.
@returns The average degree of the given graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23getAverageDegree(boolean) Online Documentation}*/
getAverageDegree(directed?: boolean): number;
/**
Computes the average weighted degree of the graph.
The average weighted degree measures the sum of the edge weights in comparison to the number of nodes and is defined
as:
<ul>
<li><code>edge_weight_sum / |V|</code> for directed graphs
<li><code>2 * edge_weight_sum / |V|</code> for undirected graphs
</ul>
<h2>Note</h2>
Parallel edges and self-loops are both included in the calculation.
@param edgeWeights The mapping that provides the weights of the edges.
@param [directed=true] Whether or not to respect the direction of the edges.
@returns The average weighted degree of the given graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23getAverageWeightedDegree(IMapper%3CIEdge%2Cnumber%3E%2Cboolean) Online Documentation}*/
getAverageWeightedDegree(edgeWeights: IMapper<IEdge, number> | IMapperConvertible<IEdge, number>, directed?: boolean): number;
/**
Computes the density of the graph.
The density is the ratio of edges of the graph to the maximum possible number of edges and equals to:
<ul>
<li><code>|E| / (|N| * (|N| - 1))</code> for directed simple graphs
<li><code>2 * |E| / (|N| * (|N| - 1))</code> for undirected simple graphs
</ul>
<h2>Note</h2>
This algorithm ignores self-loops and parallel edges.
@param [directed=true] Whether or not to respect the direction of the edges.
@returns The density of the given graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23getDensity(boolean) Online Documentation}*/
getDensity(directed?: boolean): number;
/**
Computes the diameter of the graph.
The diameter is the maximum eccentricity of any node in the graph and equals to the length of the longest of the
shortest path between any two nodes.
<h2>Note</h2>
Parallel edges are also included in the calculation.
<h2>Note</h2>
For the calculation, an algorithm that solved the all-pairs shortest path problem is used. Depending on the structure of
the given graph and the values of the given edge costs, the algorithm internally delegates its work to the algorithm
with the theoretically best running time. Please note that theory does not necessarily reflect practice.
@param edgeCosts The mapping that provides the costs of the edges.
@param [directed=true] Whether or not to respect the direction of the edges.
@returns The diameter of the given graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23getDiameter(IMapper%3CIEdge%2Cnumber%3E%2Cboolean) Online Documentation}*/
getDiameter(edgeCosts: IMapper<IEdge, number> | IMapperConvertible<IEdge, number>, directed?: boolean): number;
/**
Determines the multiple edges between the source and target node of the given edge of the directed or undirected graph.
Multiple edges connect the same two port owners. The given edge ist not part of the result.
@param edge The edge with the same source and target as the edges to determine.
@param [directed=true] Whether or not to respect the direction of the edges.
@returns The multiple edges.
@throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23getMultipleEdges(IEdge%2Cboolean) Online Documentation}*/
getMultipleEdges(edge: IEdge, directed?: boolean): IEnumerable<IEdge>;
/**
Determines the multiple edges adjacent to the given node of the directed or undirected graph.
Multiple edges connect the same two port owners.
@param node The node that is adjacent to the multiple edge to determine.
@param [directed=true] Whether or not to respect the direction of the edges.
@returns A collection of multiple edges.
@throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23getMultipleEdges(INode%2Cboolean) Online Documentation}*/
getMultipleEdges(node: INode, directed?: boolean): IEnumerable<IEnumerable<IEdge>>;
/**
Determines the multiple edges of the directed or undirected graph.
Multiple edges connect the same two port owners.
@param [directed=true] Whether or not to respect the direction of the edges.
@returns A collection of multiple edges.
@throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23getMultipleEdges(boolean) Online Documentation}*/
getMultipleEdges(directed?: boolean): IEnumerable<IEnumerable<IEdge>>;
/**
Gets the nodes of the graph in topological order.
A topological ordering of the nodes of a directed {@link GraphStructureAnalyzer#isAcyclic acyclic} graph is a linear ordering of the nodes such that for each
directed edge (<i>u</i>, <i>v</i>), the node <i>u</i> lies before <i>v</i> in the ordering.
@returns The nodes of the graph in topological order.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23getTopologicalOrder() Online Documentation}*/
getTopologicalOrder(): IEnumerable<INode>;
/**
* Determines whether or not the directed or undirected graph contains multiple edges.
* Multiple edges connect the same two port owners.
*
* <pre>
* const isMultipleEdgeFree = new GraphStructureAnalyzer(graph).isSimple()
* </pre>
*
* @param [directed=true]
* @returns <code>true</code> if the graph contains multiple edges, <code>false</code> otherwise
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23hasMultipleEdges(boolean) Online Documentation}
*/
hasMultipleEdges(directed?: boolean): boolean;
/**
* Determines whether or not the graph has self-loops.
* Self-loops connect a port owner with itself.
*
* <pre>
* const hasSelfLoops = new GraphStructureAnalyzer(graph).hasSelfLoops()
* </pre>
*
* @returns <code>true</code> if the graph contains self-loops, <code>false</code> otherwise
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23hasSelfLoops() Online Documentation}
*/
hasSelfLoops(): boolean;
/**
* Determines whether the graph is acyclic.
* A directed (undirected) graph is called <i>acyclic</i> if it contains no directed (undirected) cycle.
*
* <pre>
* const isAcyclic = new GraphStructureAnalyzer(graph).isAcyclic(true)
* </pre>
*
* @param [directed=true] Whether or not to respect the direction of the edges.
* @returns <code>true</code> if the graph is acyclic, <code>false</code>, otherwise
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see GraphStructureAnalyzer#isCyclic
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isAcyclic(boolean) Online Documentation}
*/
isAcyclic(directed?: boolean): boolean;
/**
* Determines whether or not the undirected graph is biconnected.
* A graph is called <i>biconnected</i> if it has no cut vertex or articulation point, i.e., no node whose removal disconnects the graph.
*
* <pre>
* const isBiconnected = new GraphStructureAnalyzer(graph).isBiconnected()
* </pre>
*
* @returns <code>true</code> if the graph is biconnected, <code>false</code> otherwise
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isBiconnected() Online Documentation}
*/
isBiconnected(): boolean;
/**
* Determines whether or not the undirected graph is bipartite.
* A graph is called <i>bipartite</i> if its nodes can be partitioned into two sets such that each edge connects two nodes of different
* sets.
*
* <pre>
* const isBipartite = new GraphStructureAnalyzer(graph).isBipartite()
* </pre>
*
* @returns <code>true</code> if the graph is bipartite, <code>false</code> otherwise
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isBipartite() Online Documentation}
*/
isBipartite(): boolean;
/**
* Determines whether or not the graph is connected.
* A graph is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
*
* <pre>
* const isConnected = new GraphStructureAnalyzer(graph).isConnected()
* </pre>
*
* @returns <code>true</code> if the graph is connected, <code>false</code> otherwise
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isConnected() Online Documentation}
*/
isConnected(): boolean;
/**
* Determines whether the graph is cyclic.
* A directed (undirected) graph is called <i>cyclic</i> if it contains a directed (undirected) cycle.
*
* <pre>
* const isCyclic = new GraphStructureAnalyzer(graph).isCyclic(true)
* </pre>
*
* @param [directed=true] Whether or not to respect the direction of the edges.
* @returns <code>true</code> if the graph is cyclic, <code>false</code>, otherwise
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see GraphStructureAnalyzer#isAcyclic
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isCyclic(boolean) Online Documentation}
*/
isCyclic(directed?: boolean): boolean;
/**
* Determines whether the graph is a directed (undirected) forest.
* A graph is a <i>forest</i> if its connected components are trees.
*
* <pre>
* const isForest = new GraphStructureAnalyzer(graph).isForest(true)
* </pre>
*
* @param [directed=true] Whether or not to respect the direction of the edges.
* @returns <code>true</code> if the graph is a forest, <code>false</code> otherwise
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isForest(boolean) Online Documentation}
*/
isForest(directed?: boolean): boolean;
/**
* Determines whether or not the graph is a directed rooted tree where each node has a maximum of <code>n</code> children.
* <pre>
* const isNaryTree = new GraphStructureAnalyzer(graph).isNaryTree(3)
* </pre>
*
* @param n
* @returns <code>true</code> if the graph is a directed rooted tree where each node has at most <code>n</code> children, <code>false</code> otherwise
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isNaryTree(number) Online Documentation}
*/
isNaryTree(n: number): boolean;
/**
* Determines whether or not the graph is planar.
* A graph is called <i>planar</i> if it can be drawn on the plane without edge crossings. Note that this is a property of the
* <i>graph</i>, not necessarily of the drawing, so the graph can still have crossing edges, despite being a planar graph.
*
* <pre>
* const isPlanar = new GraphStructureAnalyzer(graph).isPlanar()
* </pre>
*
* @returns <code>true</code> if the graph is planar, <code>false</code>, otherwise.
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isPlanar() Online Documentation}
*/
isPlanar(): boolean;
/**
* Determines whether or not the given undirected graph is simple.
* An undirected graph is called <i>simple</i> if there are no two edges that connect the same nodes as well as no self-loops.
*
* Since an {@link IGraph} can have edges ending at other edges, a graph containing those constructs will also not be
* simple.
*
* <pre>
* const isSimple = new GraphStructureAnalyzer(graph).isSimple()
* </pre>
*
* @returns <code>true</code> if the graph is simple, <code>false</code> otherwise
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isSimple() Online Documentation}
*/
isSimple(): boolean;
/**
* Determines whether or not the directed graph is strongly connected.
* A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
*
* <pre>
* const isStronglyConnected = new GraphStructureAnalyzer(graph).isStronglyConnected()
* </pre>
*
* @returns <code>true</code> if the graph is strongly connected, <code>false</code>, otherwise
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isStronglyConnected() Online Documentation}
*/
isStronglyConnected(): boolean;
/**
* Determines whether or not the graph is a directed (undirected) tree.
* <pre>
* const isTree = new GraphStructureAnalyzer(graph).isTree(false)
* </pre>
*
* @param [directed=true] Whether or not to respect the direction of the edges.
* @returns <code>true</code> if the graph is a tree, <code>false</code> otherwise
* @throws {Exception} <b>(InvalidOperationError)</b> If the graph has edge to edge connections.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GraphStructureAnalyzer%23isTree(boolean) Online Documentation}
*/
isTree(directed?: boolean): boolean;
static $class: Class;
private $_GraphStructureAnalyzer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GraphStructureAnalyzer;
}
export interface HierarchicalClusteringResult extends YObject {
}
/**
Represents clusters in the graph and the dendrogram as computed by {@link HierarchicalClustering}.
The {@link HierarchicalClusteringResult#dendrogramRoot dendrogram} remains valid for the chosen {@link HierarchicalClustering#metric} and {@link HierarchicalClustering#linkage} as long as the graph did not change in between. This can be used to
efficiently cluster the same graph again with a different number of clusters or a different cut-off value for
dissimilarity using either {@link HierarchicalClusteringResult#changeClusterCount} or
{@link HierarchicalClusteringResult#changeCutoff}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClusteringResult Online Documentation}*/
export class HierarchicalClusteringResult {
/**
Creates a new {@link HierarchicalClusteringResult} with the given numbers of clusters.
@param clusterCount The cluster count. Must be positive; if it's greater than the number of nodes in the graph, the number of clusters in
the result is equal to the number of nodes.
@returns A new {@link HierarchicalClusteringResult} with the same {@link HierarchicalClusteringResult#dendrogramRoot} and {@link HierarchicalClusteringResult#clusters} calculated based on that dendrogram and the
<code>clusterCount</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClusteringResult%23changeClusterCount(number) Online Documentation}*/
changeClusterCount(clusterCount: number): HierarchicalClusteringResult;
/**
Creates a new {@link HierarchicalClusteringResult} with clusters based on the given cutoff value.
@param cutoff The cutoff value for the clusters' dissimilarity.
@returns A new {@link HierarchicalClusteringResult} with the same {@link HierarchicalClusteringResult#dendrogramRoot} and {@link HierarchicalClusteringResult#clusters} calculated based on that dendrogram and <code>cutoff</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClusteringResult%23changeCutoff(number) Online Documentation}*/
changeCutoff(cutoff: number): HierarchicalClusteringResult;
/**
Gets the representative of <code>node</code> in the dendrogram.
The representative of <code>node</code> is one of the dendrogram's leaves which represents the cluster that only contains the <code>node</code>.
If a {@link HierarchicalClustering#subgraphNodes subgraph} is specified this method returns <code>null</code> if <code>node</code> is not part of the subgraph.
@param node
@see HierarchicalClusteringResult#dendrogramRoot
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClusteringResult%23getDendrogramNode(INode) Online Documentation}*/
getDendrogramNode(node: INode): DendrogramNode | null;
/**
Gets a mapping from each node to its cluster ID.
If no {@link HierarchicalClustering#subgraphNodes subgraph} is specified, the ID of a cluster is a non-negative {@link number} value which is common for all nodes of the same
cluster. For results on a subgraph, all nodes that are not part of the subgraph have the ID <code>-1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClusteringResult%23nodeClusterIds Online Documentation}*/
nodeClusterIds: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its cluster.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClusteringResult%23nodeClusters Online Documentation}*/
nodeClusters: ResultItemMapping<INode, Cluster>;
/**
Gets a collection of all clusters in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClusteringResult%23clusters Online Documentation}*/
clusters: ResultItemCollection<Cluster>;
/**
Gets the root of the dendrogram which is used internally to generate the {@link HierarchicalClusteringResult#clusters}.
Initially each node comprises its own cluster. At each step of the clustering algorithm two clusters are merged together
until all nodes belong to the same cluster. The tree which is generated this way is stored as dendrogram. The
dendrogram's leaves thus represent clusters that each contain a single node. The dendrogram's root represents a cluster
that contains every node of the graph.
The actual clusters can be calculated for a cluster count using {@link HierarchicalClusteringResult#changeClusterCount}
or for a cut-off value using {@link HierarchicalClusteringResult#changeCutoff}.
The dendrogram's root can be <code>null</code> if the original graph was empty.
@see HierarchicalClusteringResult#getDendrogramNode
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClusteringResult%23dendrogramRoot Online Documentation}*/
dendrogramRoot: DendrogramNode | null;
static $class: Class;
private $_HierarchicalClusteringResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HierarchicalClusteringResult;
}
/**
Provides constants to change how the distance between clusters is determined.
The linkage criterion has a direct effect on the dissimilarity value, which represents how close two clusters are for
the clustering.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinkageMethod Online Documentation}*/
export enum LinkageMethod {
/**
Perform single-linkage (or minimum-linkage) clustering.
The distance between two clusters is determined by the <i>minimum</i> distance. Clusters thus are as close as their <i>closest</i> pair of nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinkageMethod%23SINGLE Online Documentation}*/
SINGLE,
/**
Perform complete-linkage (or maximum-linkage) clustering.
The distance between two clusters is determined by the <i>maximum</i> distance. Clusters thus are as close as the pair of nodes that
are <i>furthest away</i> from each other.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinkageMethod%23COMPLETE Online Documentation}*/
COMPLETE,
/**
Perform mean-linkage (or average-linkage) clustering.
The distance between two clusters is the <i>average</i> distance between each pair of nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LinkageMethod%23AVERAGE Online Documentation}*/
AVERAGE
}
/** Automatic conversion to {@link LinkageMethod}. */
type LinkageMethodStringValues = "single" | "complete" | "average";
export interface DendrogramNode extends YObject {
}
/**
Represents a node of the dendrogram which is a binary tree of clusters after {@link HierarchicalClustering}.
A dendrogram node is a representation of a cluster.
The dendrogram is created using the agglomerative strategy (i.e., a bottom-up approach) based on the following steps:
<ol>
<li>Initially (i.e., at level
<code>0</code>), each node comprises its own cluster. At this point, each node of the original graph (the one that will be clustered)
is mapped to one of the dendrogram's leaf nodes.
<li>At each step, pairs of clusters are merged while moving up the hierarchy. In this case, a new dendrogram node is created
and the corresponding dissimilarity value between the two merged clusters can be obtained using
{@link DendrogramNode#dissimilarityValue}. The dissimilarity values are monotonously increasing starting from zero which corresponds to the leaf nodes of the
dendrogram.
<li>The algorithm continues until all nodes belong to the same cluster. At this point, the root of the dendrogram is reached
and can be obtained using {@link HierarchicalClusteringResult#dendrogramRoot}.
</ol>
The dendrogram is a binary tree that is directed from the root to the leaves. Moving from a node to its children
effectively represents a split of the node's cluster into two smaller clusters. Moving from a child to its parent
represents merging the child's cluster with another cluster to form a larger one.
For each node of the dendrogram, it is possible to retrieve the set of nodes of the original graph that belong to this
dendrogram node using {@link DendrogramNode#clusterNodes}. Only the {@link DendrogramNode#isLeaf leaf nodes} of the dendrogram actually represent a node of the original graph. This
means that for all other inner nodes of the dendrogram, the original {@link DendrogramNode#node} is <code>null</code>.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNode Online Documentation}*/
export class DendrogramNode {
/**
Gets the parent of the current node.
This represents the next larger cluster containing the same {@link DendrogramNode#clusterNodes nodes}.
The {@link HierarchicalClusteringResult#dendrogramRoot dendrogram root node} has no parent.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNode%23parent Online Documentation}*/
parent: DendrogramNode | null;
/**
Gets the children of this node.
This represents smaller clusters that are combined to form the cluster this node represents.
{@link DendrogramNode#isLeaf Leaf nodes} have no children.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNode%23children Online Documentation}*/
children: IListEnumerable<DendrogramNode>;
/**
Gets this node's level index in the dendrogram.
The dendrogram's {@link HierarchicalClusteringResult#dendrogramRoot root node} has a level of <code>0</code>. The level index increases downwards from the root to the leaves.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNode%23level Online Documentation}*/
level: number;
/**
Gets the dissimilarity value associated with this node.
The dissimilarity value is a measure of how "close" the {@link DendrogramNode#children child clusters} are to each other, based on the chosen {@link HierarchicalClustering#metric} and {@link HierarchicalClustering#linkage}.
{@link DendrogramNode#isLeaf Leaf nodes}, that is, clusters containing only a single node, have a {@link DendrogramNode#dissimilarityValue} of <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNode%23dissimilarityValue Online Documentation}*/
dissimilarityValue: number;
/**
Gets a value indicating whether this node is a leaf node in the dendrogram.
Leaf nodes have no {@link DendrogramNode#children} and represent a single {@link DendrogramNode#node} in the graph as the smallest clusters possible.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNode%23isLeaf Online Documentation}*/
readonly isLeaf: boolean;
/**
Gets the {@link INode} in the graph which is associated with this dendrogram node.
Only the {@link DendrogramNode#isLeaf leaf nodes} in a dendrogram are associated with a node. Other nodes represent larger clusters of multiple nodes
which can be obtained via their {@link DendrogramNode#clusterNodes}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNode%23node Online Documentation}*/
node: INode | null;
/**
Gets the {@link INode}s in the graph which belong to the cluster which is represented by this
{@link DendrogramNode}.
All nodes associated with the leaf nodes which are descendants of this node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DendrogramNode%23clusterNodes Online Documentation}*/
clusterNodes: IListEnumerable<INode>;
static $class: Class;
private $_DendrogramNode: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is DendrogramNode;
}
export interface HierarchicalClustering extends YObject {
}
/**
* Partitions the graph into clusters using hierarchical clustering.
* <b>Hierarchical clustering</b> creates a hierarchy of clusters in a bottom-to-top approach based on some distance metric and linkage.
*
* The clustering is performed using the agglomerative strategy i.e., a bottom-up approach according to which initially
* each node comprises its own cluster. At each step pairs of clusters are merged while moving up the hierarchy. The
* dissimilarity between clusters is determined based on the given linkage and the given node distances metric. The
* algorithm continues until all nodes belong to the same cluster.
*
* Predefined metrics are available as constants on {@link HierarchicalClustering}:
*
* <ul>
* <li>{@link HierarchicalClustering#EUCLIDEAN Euclidean distance}</li>
* <li>{@link HierarchicalClustering#EUCLIDEAN_SQUARED Squared Euclidean distance}</li>
* <li>{@link HierarchicalClustering#MANHATTAN Manhattan distance}</li>
* </ul>
* Internally the {@link HierarchicalClusteringResult} stores a {@link HierarchicalClusteringResult#dendrogramRoot dendrogram} which represents the result of the
* clustering algorithm as a binary tree structure. Based on the dendrogram the graph's nodes are partitioned into clusters
* according to either {@link HierarchicalClustering#clusterCount} or {@link HierarchicalClustering#cutoff}. Since both methods to compute the clusters only require the dendrogram, the {@link HierarchicalClusteringResult}
* offers methods to create a new clustering based on either criterion as long as either the graph or the subgraph
* specification did not change in between (which would require re-computing the dendrogram).
*
* <b>Other Clustering Algorithms</b>
*
* yFiles for HTML supports a number of other clustering algorithms:
*
* <ul>
* <li><b>{@link KMeansClustering}</b> – partitions the graph into clusters based on the distance between nodes and the cluster midpoints.</li>
* <li><b>{@link EdgeBetweennessClustering}</b> – partitions the graph into clusters based on edge-betweenness centrality.</li>
* <li><b>{@link BiconnectedComponentClustering}</b> – partitions the graph into clusters based on its biconnected components.</li>
* <li><b>{@link LouvainModularityClustering}</b> – partitions the graph into clusters by applying the <i>Louvain modularity</i> method.</li>
* <li><b>{@link LabelPropagationClustering}</b> – partitions the graph into clusters by applying a label propagation algorithm.</li>
* </ul>
* <br><b>Calculating hierarchic clusters of a graph</b>
* <pre>
* // prepare the hierarchical clustering algorithm
* const algorithm = new HierarchicalClustering()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering Online Documentation}
*/
export class HierarchicalClustering {
/**
Creates a new {@link HierarchicalClustering} instance.
@param [options] The parameters to pass.
@param [options.metric] A function which returns the distance between any two nodes. This option sets the {@link HierarchicalClustering#metric} property on the created object.
@param [options.linkage] The method how the distance between clusters is determined. This option sets the {@link HierarchicalClustering#linkage} property on the created object.
@param [options.clusterCount] The number of clusters. This option sets the {@link HierarchicalClustering#clusterCount} property on the created object.
@param [options.cutoff] The cut-off value. This option sets the {@link HierarchicalClustering#cutoff} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link HierarchicalClustering#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link HierarchicalClustering#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering Online Documentation}*/
constructor(options?: {
metric?: (arg1: INode, arg2: INode) => number;
linkage?: LinkageMethod | LinkageMethodStringValues;
clusterCount?: number;
cutoff?: number;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Partitions the graph into clusters using hierarchical clustering.
The returned {@link HierarchicalClusteringResult} can be used to efficiently create different clusterings of the
same graph based on the {@link HierarchicalClustering#metric} and {@link HierarchicalClustering#linkage} as long as the graph hasn't changed (and would thus invalidate the computed {@link HierarchicalClusteringResult#dendrogramRoot dendrogram}.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link HierarchicalClusteringResult} containing the computed clusters and {@link HierarchicalClusteringResult#dendrogramRoot dendrogram}.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23run(IGraph) Online Documentation}*/
run(graph: IGraph): HierarchicalClusteringResult;
/**
Gets or sets a function which returns the distance between any two nodes.
The result of the function must not be negative.
Predefined common metrics are available as constants on {@link HierarchicalClustering}:
<ul>
<li>{@link HierarchicalClustering#EUCLIDEAN Euclidean distance} (the default)
<li>{@link HierarchicalClustering#EUCLIDEAN_SQUARED Squared Euclidean distance}
<li>{@link HierarchicalClustering#MANHATTAN Manhattan distance}
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23metric Online Documentation}*/
metric: (arg1: INode | null, arg2: INode | null) => number;
/**
Gets or sets the method how the distance between clusters is determined.
The default is {@link LinkageMethod#SINGLE}.
@default LinkageMethod.SINGLE
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23linkage Online Documentation}*/
linkage: LinkageMethod;
/**
Gets or sets the number of clusters.
If set to a negative number or <code>0</code> the {@link HierarchicalClustering#cutoff} value is used to calculate the clusters. If both values are negative a number of <code>1</code>
is used (one cluster containing all nodes in the graph).
A new result for a different number of clusters can be obtained efficiently with {@link HierarchicalClusteringResult#changeClusterCount}
on a valid {@link HierarchicalClusteringResult}, as long as the graph, {@link HierarchicalClustering#linkage}, and {@link HierarchicalClustering#metric} are still the same.
@default -1
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23clusterCount Online Documentation}*/
clusterCount: number;
/**
Gets or sets the cut-off value.
The clusters are calculated based on the given cut-off value that is used for cutting the {@link HierarchicalClusteringResult#dendrogramRoot hierarchical tree} at a point
such that the dissimilarity values of the nodes that remain at the dendrogram are less than this value.
This value will only be used if {@link HierarchicalClustering#clusterCount} is <code>0</code> or negative.
This value must be non-negative, otherwise the result will be a single cluster which contains all nodes of the graph.
A new result for a different cut-off value can be obtained efficiently with {@link HierarchicalClusteringResult#changeCutoff}
on a valid {@link HierarchicalClusteringResult}, as long as the graph, {@link HierarchicalClustering#linkage}, and {@link HierarchicalClustering#metric} are still the same.
@default -1.0
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23cutoff Online Documentation}*/
cutoff: number;
/**
A predefined {@link HierarchicalClustering#metric} for Euclidean distance.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23EUCLIDEAN Online Documentation}*/
static EUCLIDEAN: (arg1: INode | null, arg2: INode | null) => number;
/**
A predefined {@link HierarchicalClustering#metric} for squared Euclidean distance.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23EUCLIDEAN_SQUARED Online Documentation}*/
static EUCLIDEAN_SQUARED: (arg1: INode | null, arg2: INode | null) => number;
/**
A predefined {@link HierarchicalClustering#metric} for Manhattan distance.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23MANHATTAN Online Documentation}*/
static MANHATTAN: (arg1: INode | null, arg2: INode | null) => number;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link HierarchicalClustering#run}
* method.
*
* <br><b>Calculating hierarchic on a subset of the graph</b>
* <pre>
* // prepare the hierarchical clustering algorithm
* const algorithm = new HierarchicalClustering({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link HierarchicalClustering#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link HierarchicalClustering#run}
* method.
*
* <br><b>Calculating hierarchic clusters on a subset of the graph</b>
* <pre>
* // prepare the hierarchical clustering algorithm
* const algorithm = new HierarchicalClustering({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicalClustering%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_HierarchicalClustering: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HierarchicalClustering;
}
export interface IndependentSetsResult extends YObject {
}
/**
Represents a partitioning of a graph into independent sets of nodes.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSetsResult Online Documentation}*/
export class IndependentSetsResult {
/**
Gets a mapping for each node to its set ID.
If the node is not part of an optional subgraph, this mapping returns <code>-1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSetsResult%23nodeSetIds Online Documentation}*/
nodeSetIds: ResultItemMapping<INode, number>;
/**
Gets a mapping for each node to its {@link IndependentSet}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSetsResult%23nodeSets Online Documentation}*/
nodeSets: ResultItemMapping<INode, IndependentSet>;
/**
Gets a collection of all independent sets in the result.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSetsResult%23independentSets Online Documentation}*/
independentSets: ResultItemCollection<IndependentSet>;
static $class: Class;
private $_IndependentSetsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IndependentSetsResult;
}
export interface IndependentSet extends YObject {
}
/**
Represents a single independent set of nodes in a graph.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSet Online Documentation}*/
export class IndependentSet {
/**
Gets a collection of nodes in this set.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSet%23nodes Online Documentation}*/
nodes: ResultItemCollection<INode>;
static $class: Class;
private $_IndependentSet: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IndependentSet;
}
export interface IndependentSets extends YObject {
}
/**
* Partitions the set of nodes of the given graph into independent sets.
* An <b>independent set</b> is a set of nodes in a graph, in which no two nodes are adjacent.
*
* yFiles for HTML supports a number of other analysis algorithms that partition the graph into components, based on
* various criteria.
*
* <ul>
* <li><b>{@link ConnectedComponents}</b> – Determines components defined by the existence of an undirected path between nodes</li>
* <li><b>{@link StronglyConnectedComponents}</b> – Determines components defined by the existence of a directed path between nodes</li>
* <li><b>{@link BiconnectedComponents}</b> – Determines components defined by the existence of at least two separate undirected paths between all nodes</li>
* <li><b>{@link Bipartition}</b> – Divides a graph into two partitions where all edges have their source and target in different partitions</li>
* <li><b>{@link KCoreComponents}</b> – Calculates the k-cores of an undirected input graph. The k-core of an undirected input graph consists of the subgraph
* components where each node has at least degree <code>k</code>.</li>
* </ul>
* <br><b>Calculating independent sets on a graph</b>
* <pre>
* // prepare the independent sets algorithm
* const algorithm = new IndependentSets()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the independent sets with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeSetIds.get(node)
*   graph.setStyle(node, nodeSetStyles.get(componentId))
* }
* </pre>
*
* @see IndependentSetAlgorithm#getIndependentSets
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSets Online Documentation}
*/
export class IndependentSets {
/**
Creates a new {@link IndependentSets} instance.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link IndependentSets#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link IndependentSets#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSets Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Partitions the set of nodes of the given graph into independent sets.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link IndependentSetsResult} containing the independent sets of nodes in <code>graph</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSets%23run(IGraph) Online Documentation}*/
run(graph: IGraph): IndependentSetsResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link IndependentSets#run} method.
*
* <br><b>Calculating independent sets on a subset of the graph</b>
* <pre>
* // configure the independent sets algorithm
* const algorithm = new IndependentSets({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the independent sets with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeSetIds.get(node)
*   graph.setStyle(node, nodeSetStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSets%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link IndependentSets#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link IndependentSets#run} method.
*
* <br><b>Calculating independent sets on a subset of the graph</b>
* <pre>
* // configure the independent sets algorithm
* const algorithm = new IndependentSets({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the independent sets with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeSetIds.get(node)
*   graph.setStyle(node, nodeSetStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IndependentSets%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_IndependentSets: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IndependentSets;
}
export interface KCoreComponentsResult extends YObject {
}
/**
Represents the k-Cores as computed by {@link KCoreComponents}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/KCoreComponentsResult Online Documentation}*/
export class KCoreComponentsResult {
/**
Gets the k-Core with the given degree <code>k</code>.
@param k The degree of the k-core.
@returns A collection of nodes which constitute the k-core with degree <code>k</code>. Might be empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KCoreComponentsResult%23getKCore(number) Online Documentation}*/
getKCore(k: number): ResultItemCollection<INode>;
/**
Gets the largest <code>k</code> for which the k-core is not empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KCoreComponentsResult%23maximumK Online Documentation}*/
maximumK: number;
/**
Gets a map which returns for each node the the largest <code>k</code> for which a node is contained in the k-core.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KCoreComponentsResult%23kCores Online Documentation}*/
kCores: ResultItemMapping<INode, number>;
static $class: Class;
private $_KCoreComponentsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is KCoreComponentsResult;
}
export interface KCoreComponents extends YObject {
}
/**
* Calculates the k-cores of an undirected input graph.
* The k-core of an undirected input graph consists of the subgraph components where each node has at least degree <code>k</code>. The
* algorithm has runtime complexity <code>O((n+m)log(n))</code> and requires linear space.<b>Other Graph Connectivity Algorithms</b>
*
* yFiles for HTML supports a number of other analysis algorithms that partition the graph into components, based on
* various criteria.
*
* <ul>
* <li><b>{@link ConnectedComponents}</b> – Determines components defined by the existence of an undirected path between nodes</li>
* <li><b>{@link StronglyConnectedComponents}</b> – Determines components defined by the existence of a directed path between nodes</li>
* <li><b>{@link BiconnectedComponents}</b> – Determines components defined by the existence of at least two separate undirected paths between all nodes</li>
* <li><b>{@link Bipartition}</b> – Divides a graph into two partitions where all edges have their source and target in different partitions</li>
* <li><b>{@link IndependentSets}</b> – Divides a graph into partitions where no nodes are connected within a partition</li>
* </ul>
* <h2>Note</h2>
*
* Self-loops are ignored.
*
* <br><b>Calculating the k-cores of a graph</b>
* <pre>
* // prepare the k-core algorithm
* const algorithm = new KCoreComponents()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of a given k-cores with different styles
* for (const node of result.getKCore(3)) {
*   graph.setStyle(node, highlightStyle)
* }
* </pre>
*
* @see GraphConnectivity#kCore
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/KCoreComponents Online Documentation}
*/
export class KCoreComponents {
/**
Creates a new {@link KCoreComponents} instance.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link KCoreComponents#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link KCoreComponents#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KCoreComponents Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Determines the k-Cores of a given graph.
* <h2>Note</h2>
*
* Self-loops are ignored.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Calculating the k-cores of a graph</b>
* <pre>
* // prepare the k-core algorithm
* const algorithm = new KCoreComponents()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of a given k-cores with different styles
* for (const node of result.getKCore(3)) {
*   graph.setStyle(node, highlightStyle)
* }
* </pre>
*
* @param graph The input graph to run the algorithm on.
* @returns A {@link KCoreComponentsResult} from which the k-cores of the <code>graph</code> can be obtained.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/KCoreComponents%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): KCoreComponentsResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link KCoreComponents#run} method.
*
* <br><b>Calculating the k-Cores of a subset of the graph</b>
* <pre>
* // prepare the k-core algorithm
* const algorithm = new KCoreComponents({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of a given k-cores with different styles
* for (const node of result.getKCore(3)) {
*   graph.setStyle(node, highlightStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/KCoreComponents%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link KCoreComponents#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link KCoreComponents#run} method.
*
* <br><b>Calculating the k-Cores a subset of the graph</b>
* <pre>
* // prepare the k-core algorithm
* const algorithm = new KCoreComponents({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of a given k-cores with different styles
* for (const node of result.getKCore(3)) {
*   graph.setStyle(node, highlightStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/KCoreComponents%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_KCoreComponents: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is KCoreComponents;
}
export interface KMeansClusteringResult extends YObject {
}
/**
Represents clusters in the graph as computed by {@link KMeansClustering}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClusteringResult Online Documentation}*/
export class KMeansClusteringResult {
/**
Gets the centroid of the <code>cluster</code>.
@param cluster The cluster to get the calculated centroid for.
@returns The calculated centroid of the given cluster.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClusteringResult%23getCentroid(Cluster) Online Documentation}*/
getCentroid(cluster: Cluster): Point;
/**
Gets a mapping from each node to its cluster ID.
If no {@link KMeansClustering#subgraphNodes subgraph} is specified, the ID of a cluster is a non-negative {@link number} value which is common for all nodes of the same
cluster.
If a {@link KMeansClustering#subgraphNodes subgraph} is specified, all nodes that are <b>not</b> part of the subgraph have the ID <code>-1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClusteringResult%23nodeClusterIds Online Documentation}*/
nodeClusterIds: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its cluster.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClusteringResult%23nodeClusters Online Documentation}*/
nodeClusters: ResultItemMapping<INode, Cluster>;
/**
Gets a collection of all clusters in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClusteringResult%23clusters Online Documentation}*/
clusters: ResultItemCollection<Cluster>;
/**
Gets a collection of the new centroids.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClusteringResult%23centroids Online Documentation}*/
centroids: ResultItemCollection<Point>;
/**
Gets a value which estimates the quality of the partition of the nodes into the {@link KMeansClusteringResult#clusters}.
The modularity value estimates the quality of the partition of the nodes into the given communities and is a value
between
<code>[-1/2,1]</code>. High modularity values denote that nodes with dense connections are placed in the same community, while nodes
with sparse connections are assigned to different communities.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClusteringResult%23modularity Online Documentation}*/
modularity: number;
static $class: Class;
private $_KMeansClusteringResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is KMeansClusteringResult;
}
export interface KMeansClustering extends YObject {
}
/**
* Partitions the graph into clusters using <i>k</i>-means clustering.
* The nodes of the graph will be partitioned into {@link KMeansClustering#k} clusters, based on their positions such that their distance to the
* cluster's mean (centroid) is minimized.
*
* <b>Other Clustering Algorithms</b>
*
* yFiles for HTML supports a number of other clustering algorithms:
*
* <ul>
* <li><b>{@link HierarchicalClustering}</b> – partitions the graph into clusters by merging smaller clusters based on their distance.</li>
* <li><b>{@link EdgeBetweennessClustering}</b> – partitions the graph into clusters based on edge-betweenness centrality.</li>
* <li><b>{@link BiconnectedComponentClustering}</b> – partitions the graph into clusters based on its biconnected components.</li>
* <li><b>{@link LouvainModularityClustering}</b> – partitions the graph into clusters by applying the <i>Louvain modularity</i> method.</li>
* <li><b>{@link LabelPropagationClustering}</b> – partitions the graph into clusters by applying a label propagation algorithm.</li>
* </ul>
* <b>Warning:</b> If the number of given centroids is smaller than <code>k</code> or if no centroids are given, random initial centroids will be
* assigned for all clusters.
*
* <br><b>Calculating the k-means clusters of a graph</b>
* <pre>
* // prepare the k-means clustering algorithm
* const algorithm = new KMeansClustering()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see GroupAlgorithm#kMeansClustering
* @see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClustering Online Documentation}
*/
export class KMeansClustering {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.metric] A metric to determine the distance between nodes and between nodes and {@link KMeansClustering#centroids}. This option sets the {@link KMeansClustering#metric} property on the created object.
@param [options.k] The number of clusters. This option sets the {@link KMeansClustering#k} property on the created object.
@param [options.maximumIterations] The maximum number of iterations performed by the algorithm for convergence. This option sets the {@link KMeansClustering#maximumIterations} property on the created object.
@param [options.centroids] The initial centroids. This option sets the {@link KMeansClustering#centroids} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link KMeansClustering#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link KMeansClustering#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClustering Online Documentation}*/
constructor(options?: {
metric?: DistanceMetric | DistanceMetricStringValues;
k?: number;
maximumIterations?: number;
centroids?: IEnumerable<Point> | null | IEnumerableConvertible<Point>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Partitions the graph into clusters using <i>k</i>-means clustering.
<b>Warning:</b> If the number of given centroids is smaller than <code>k</code> or if no centroids are given, random initial centroids will be
assigned for all clusters.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns The resulting (non-empty) clusters
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClustering%23run(IGraph) Online Documentation}*/
run(graph: IGraph): KMeansClusteringResult;
/**
Gets or sets a metric to determine the distance between nodes and between nodes and {@link KMeansClustering#centroids}.
Default is {@link DistanceMetric#EUCLIDEAN}.
@default DistanceMetric.EUCLIDEAN
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClustering%23metric Online Documentation}*/
metric: DistanceMetric;
/**
Gets or sets the number of clusters.
Default is <code>1</code>.
<b>Warning:</b> If the number of given centroids is smaller than {@link KMeansClustering#k} or if no centroids are given, random initial centroids will be
assigned for all clusters.
@default 1
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClustering%23k Online Documentation}*/
k: number;
/**
Gets or sets the maximum number of iterations performed by the algorithm for convergence.
Default is <code>100</code>.
@default 100
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClustering%23maximumIterations Online Documentation}*/
maximumIterations: number;
/**
Gets or sets the initial centroids.
<b>Warning:</b> If the number of initial centroids is smaller than {@link KMeansClustering#k} or if no centroids are given, random initial centroids will be
assigned for all clusters.
@see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClustering%23centroids Online Documentation}*/
centroids: IEnumerable<Point> | null;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link KMeansClustering#run} method.
*
* <br><b>Calculating the k-means clusters on a subset of the graph</b>
* <pre>
* // prepare the k-means clustering algorithm
* const algorithm = new KMeansClustering({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClustering%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link KMeansClustering#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link KMeansClustering#run} method.
*
* <br><b>Calculating the k-means clusters on a subset of the graph</b>
* <pre>
* // prepare the k-means clustering algorithm
* const algorithm = new KMeansClustering({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/KMeansClustering%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_KMeansClustering: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is KMeansClustering;
}
export interface LabelPropagationClusteringResult extends YObject {
}
/**
Represents clusters in the graph as computed by {@link LabelPropagationClustering}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClusteringResult Online Documentation}*/
export class LabelPropagationClusteringResult {
/**
Gets a mapping from each node to its label.
The value corresponds to the calculated (final) label. For nodes which are not associated with a cluster the index is
<code>-1</code>. Also, if a {@link LabelPropagationClustering#subgraphNodes subgraph} is specified, all nodes that are <b>not</b> part of the subgraph have the label <code>-1</code>.
<b>Warning:</b>
The labels which are considered in this algorithms are internally assigned integer values. They have nothing to do with
the marking objects which are commonly referred to as "label" in yFiles for HTML.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClusteringResult%23finalLabels Online Documentation}*/
finalLabels: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its cluster ID.
The value corresponds to the index of the associated cluster in {@link LabelPropagationClusteringResult#clusters}. For nodes which are not associated with a cluster the
index is <code>-1</code>. Also, if a {@link LabelPropagationClustering#subgraphNodes subgraph} is specified, all nodes that are <b>not</b> part of the subgraph have the ID <code>-1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClusteringResult%23nodeClusterIds Online Documentation}*/
nodeClusterIds: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its cluster.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClusteringResult%23nodeClusters Online Documentation}*/
nodeClusters: ResultItemMapping<INode, Cluster>;
/**
Gets a collection of all clusters in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClusteringResult%23clusters Online Documentation}*/
clusters: ResultItemCollection<Cluster>;
/**
Gets a value which estimates the quality of the partition of the nodes into the {@link LabelPropagationClusteringResult#clusters}.
The modularity value estimates the quality of the partition of the nodes into the given communities and is a value
between
<code>[-1/2,1]</code>. High modularity values denote that nodes with dense connections are placed in the same community, while nodes
with sparse connections are assigned to different communities.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClusteringResult%23modularity Online Documentation}*/
modularity: number;
static $class: Class;
private $_LabelPropagationClusteringResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelPropagationClusteringResult;
}
export interface LabelPropagationClustering extends YObject {
}
/**
* Detects the communities in the specified input graph by applying a label propagation algorithm.
* This algorithm iteratively assigns a label to each node. The label of a node is set to the most frequent label among its
* neighbors. If there are multiple candidates the algorithm randomly chooses one of them. After applying the algorithm,
* all nodes with the same label belong to the same community.
*
* The implementation is based on the description in: "Near linear time algorithm to detect community structures in
* large-scale networks" by U. N. Raghavan, R. Albert, and S. Kumara in Phys. Rev. E 76, 036106, 2007
*
* For a given node and neighbor, the weight of the neighbors label is <code>nodeWeight(neighbor) * edgeWeight((node, neighbor))</code>.
* In addition, the overall weight of a specific label is the sum of such weights for all neighbors with matching label.
* The weights are specified in {@link LabelPropagationClustering#nodeWeights} and {@link LabelPropagationClustering#edgeWeights}, respectively. If no values are provided uniform weights of <code>1.0</code> are applied.
*
* If no {@link LabelPropagationClustering#initialLabels} are provided each node starts with a unique label. Otherwise, at the start of the algorithm, each node is
* associated with the label specified with {@link LabelPropagationClustering#initialLabels} Negative values are ignored, i.e., at the start there may be nodes without
* label. This implies that there may be results, where some of the nodes are not associated with labels, too. Such nodes
* are not associated with any cluster.
*
* If {@link LabelPropagationClustering#edgeDirectedness} is specified, the algorithm only considers a subset of the neighbors when determining the label of a node. More
* precisely, it only considers the neighbors connected by edges to a node n for which
*
* <ul>
* <li>the edge's directedness is greater than <code>0.0</code> and the edge is incoming (i.e. n is the edge's target),</li>
* <li>the edge's directedness is smaller than <code>0.0</code> and the edge is outgoing (i.e. n is the edge's source),</li>
* <li>or the edge's directedness is <code>0.0</code> (i.e. the edge is considered to be <b>undirected</b>).</li>
* </ul>
* <b>Other Clustering Algorithms</b>
*
* yFiles for HTML supports a number of other clustering algorithms:
*
* <ul>
* <li><b>{@link EdgeBetweennessClustering}</b> – partitions the graph into clusters based on edge-betweenness centrality.</li>
* <li><b>{@link KMeansClustering}</b> – partitions the graph into clusters based on the distance between nodes and the cluster midpoints.</li>
* <li><b>{@link HierarchicalClustering}</b> – partitions the graph into clusters by merging smaller clusters based on their distance.</li>
* <li><b>{@link BiconnectedComponentClustering}</b> – partitions the graph into clusters based on its biconnected components.</li>
* <li><b>{@link LouvainModularityClustering}</b> – partitions the graph into clusters by applying the <i>Louvain modularity</i> method.</li>
* </ul>
* <h2>Note</h2>
*
* A node with a self-loop is considered to be a neighbor of itself. Hence, the node's label is also included when
* calculating the most frequent label among the neighbors. If you want to prevent this, you can explicitly {@link LabelPropagationClustering#subgraphEdges exclude}
* self-loop edges. Alternatively, you can specify suitable edge weights. Setting the weight of self-loops to <code>0.0</code> results
* in ignoring them.
*
* <h2>Note</h2>
*
* If none of the neighbors of a node is associated with a label with positive weight, the node keeps its current label.
*
* <h2>Note</h2>
*
* If no {@link LabelPropagationClustering#edgeDirectedness} is specified, the algorithm assumes that all edges are undirected (i.e. have directedness <code>0.0</code>).
*
* <b>Warning:</b>
* The labels which are considered in this algorithms are internally assigned integer values. They have nothing to do with
* the marking objects which are commonly referred to as "label" in yFiles for HTML.
*
* <br><b>Calculating clusters of a graph using the label propagation method</b>
* <pre>
* // prepare the label propagation algorithm
* const algorithm = new LabelPropagationClustering({
*   edgeWeights: edge => edge.style.stroke.thickness,
*   nodeWeights: node => node.layout.width
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* <br><b>Excluding self-loop edges</b>
* <pre>
* const algorithm = new LabelPropagationClustering({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge => edge.sourceNode === edge.targetNode
*   }
* })
* </pre>
*
* @see GroupAlgorithm#labelPropagation
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClustering Online Documentation}
*/
export class LabelPropagationClustering {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.initialLabels] A mapping that stores the initial integer labels of each node. This option sets the {@link LabelPropagationClustering#initialLabels} property on the created object.
@param [options.nodeWeights] A mapping for node weights. This option sets the {@link LabelPropagationClustering#nodeWeights} property on the created object.
@param [options.edgeWeights] A mapping for edge weights. This option sets the {@link LabelPropagationClustering#edgeWeights} property on the created object.
@param [options.edgeDirectedness] A mapping that stores the directedness of the edges. This option sets the {@link LabelPropagationClustering#edgeDirectedness} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link LabelPropagationClustering#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link LabelPropagationClustering#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClustering Online Documentation}*/
constructor(options?: {
initialLabels?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
nodeWeights?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
edgeWeights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Detects the communities in the specified input graph by applying a label propagation algorithm.
* This algorithm iteratively assigns a label to each node. The label of a node is set to the most frequent label among its
* neighbors. If there are multiple candidates the algorithm randomly chooses one of them. After applying the algorithm,
* all nodes with the same label belong to the same community.
*
* For a given node and neighbor, the weight of the neighbors label is <code>nodeWeight(neighbor) * edgeWeight((node, neighbor))</code>.
* In addition, the overall weight of a specific label is the sum of such weights for all neighbors with matching label.
* The weights are specified in {@link LabelPropagationClustering#nodeWeights} and {@link LabelPropagationClustering#edgeWeights}, respecitvely. If no values are provided uniform weights of <code>1.0</code> are applied.
*
* If no {@link LabelPropagationClustering#initialLabels} are provided each node starts with a unique label. Otherwise, at the start of the algorithm, each node is
* associated with the label specified with {@link LabelPropagationClustering#initialLabels} Negative values are ignored, i.e., at the start there may be nodes without
* label. This implies that there may be results, where some of the nodes are not associated with labels, too. Such nodes
* are not associated with any cluster.
*
* If {@link LabelPropagationClustering#edgeDirectedness} is specified, the algorithm only considers a subset of the neighbors when determining the label of a node. More
* precisely, it only considers the neighbors connected by edges to a node n for which
*
* <ul>
* <li>the edge's directedness is greater than <code>0.0</code> and the edge is incoming (i.e. n is the edge's target),</li>
* <li>the edge's directedness is smaller than <code>0.0</code> and the edge is outgoing (i.e. n is the edge's source),</li>
* <li>or the edge's directedness is <code>0.0</code> (i.e. the edge is considered to be <b>undirected</b>).</li>
* </ul>
* <h2>Note</h2>
*
* A node with a self-loop is considered to be a neighbor of itself. Hence, the node's label is also included when
* calculating the most frequent label among the neighbors. If you want to prevent this, you can explicitly {@link LabelPropagationClustering#subgraphEdges exclude}
* self-loop edges. Alternatively, you can specify suitable edge weights. Setting the weight of self-loops to <code>0.0</code> results
* in ignoring them.
*
* <h2>Note</h2>
*
* If none of the neighbors of a node is associated with a label with positive weight, the node keeps its current label.
*
* <h2>Note</h2>
*
* If no {@link LabelPropagationClustering#edgeDirectedness} is specified, the algorithm assumes that all edges are undirected (i.e. have directedness <code>0.0</code>).
*
* <b>Warning:</b>
* The labels which are considered in this algorithms are internally assigned integer values. They have nothing to do with
* the marking objects which are commonly referred to as "label" in yFiles for HTML.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Calculating clusters of a graph using the label propagation method</b>
* <pre>
* // prepare the label propagation algorithm
* const algorithm = new LabelPropagationClustering({
*   edgeWeights: edge => edge.style.stroke.thickness,
*   nodeWeights: node => node.layout.width
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* <br><b>Excluding self-loop edges</b>
* <pre>
* const algorithm = new LabelPropagationClustering({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge => edge.sourceNode === edge.targetNode
*   }
* })
* </pre>
*
* @param graph The undirected input graph.
* @returns The resulting clusters.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClustering%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): LabelPropagationClusteringResult;
/**
Gets or sets a mapping that stores the initial integer labels of each node.
At the start of the algorithm, each node is associated with the label specified here. Negative values are ignored, i.e.,
at the start there may be nodes without label. This implies that there may be results, where some of the nodes are not
associated with labels, too. Such nodes are not associated with any cluster.
If nothing is set each node starts with a unique label.
<b>Warning:</b>
The labels which are considered in this algorithms are internally assigned integer values. They have nothing to do with
the marking objects which are commonly referred to as "label" in yFiles for HTML.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClustering%23initialLabels Online Documentation}*/
initialLabels: ItemMapping<INode, number>;
/**
Gets or sets a mapping for node weights.
For a given node and neighbor, the weight of the neighbors label is <code>nodeWeight(neighbor) * edgeWeight((node, neighbor))</code>.
In addition, the overall weight of a specific label is the sum of such weights for all neighbors with matching label.
The weights are specified in {@link LabelPropagationClustering#nodeWeights} and {@link LabelPropagationClustering#edgeWeights}, respectively. If no values are provided uniform weights of <code>1.0</code> are applied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClustering%23nodeWeights Online Documentation}*/
nodeWeights: ItemMapping<INode, number>;
/**
Gets or sets a mapping for edge weights.
For a given node and neighbor, the weight of the neighbors label is <code>nodeWeight(neighbor) * edgeWeight((node, neighbor))</code>.
In addition, the overall weight of a specific label is the sum of such weights for all neighbors with matching label.
The weights are specified in {@link LabelPropagationClustering#nodeWeights} and {@link LabelPropagationClustering#edgeWeights}, respectively. If no values are provided uniform weights of <code>1.0</code> are applied.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClustering%23edgeWeights Online Documentation}*/
edgeWeights: ItemMapping<IEdge, number>;
/**
Gets or sets a mapping that stores the directedness of the edges.
If a value is provided here, the algorithm only considers a subset of the neighbors when determining the label of a
node. More precisely, it only considers the neighbors connected by edges to a node n for which
<ul>
<li>the edge's directedness is greater than <code>0.0</code> and the edge is incoming (i.e. n is the edge's target),
<li>the edge's directedness is smaller than <code>0.0</code> and the edge is outgoing (i.e. n is the edge's source),
<li>or the edge's directedness is <code>0.0</code> (i.e. the edge is considered to be <b>undirected</b>).
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClustering%23edgeDirectedness Online Documentation}*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link LabelPropagationClustering#run}
* method.
*
* <br><b>Calculating the clusters based on label propagation on a subset of the graph</b>
* <pre>
* // prepare the label propagation algorithm
* const algorithm = new LabelPropagationClustering({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClustering%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link LabelPropagationClustering#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link LabelPropagationClustering#run}
* method.
*
* <br><b>Calculating the clusters based on label propagation on a subset of the graph</b>
* <pre>
* // prepare the label propagation algorithm
* const algorithm = new LabelPropagationClustering({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelPropagationClustering%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_LabelPropagationClustering: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelPropagationClustering;
}
export interface LongestPathResult extends YObject {
}
/**
Represents the longest directed path in an acyclic graph as computed by {@link LongestPath}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPathResult Online Documentation}*/
export class LongestPathResult {
/**
Gets the path.
If there is no longest path in the graph (because there are no edges), this is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPathResult%23path Online Documentation}*/
path: Path | null;
/**
Gets the path's nodes.
If there is no longest path in the graph (because there are no edges), this collection is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPathResult%23nodes Online Documentation}*/
readonly nodes: ResultItemCollection<INode>;
/**
Gets the path's edges.
If there is no longest path in the graph (because there are no edges), this collection is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPathResult%23edges Online Documentation}*/
readonly edges: ResultItemCollection<IEdge>;
/**
Gets the path's length.
If there have been {@link LongestPath#costs edge costs} provided, this is the sum of costs along the path. If there were no costs, this is the
length of the path, as all edges have a uniform cost of <code>1</code>.
If there is no longest path in the graph (because there are no edges), this is <code>0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPathResult%23distance Online Documentation}*/
readonly distance: number;
static $class: Class;
private $_LongestPathResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LongestPathResult;
}
export interface LongestPath extends YObject {
}
/**
* Finds the longest directed path in an acyclic graph.
* <b>Other Path-Related Algorithms</b>
*
* yFiles for HTML supports a number of other algorithms related to paths in a graph:
*
* <ul>
* <li><b>{@link Paths}</b> – finds all paths between a set of source and a set of target nodes</li>
* <li><b>{@link Chains}</b> – finds all chains, that is, sequences of nodes that are each connected with just an edge without branches</li>
* <li><b>{@link Cycle}</b> – finds a cycle if one exists</li>
* <li><b>{@link LongestPath}</b> – finds the longest path in the graph</li>
* </ul>
* <b>Shortest Path Algorithms</b>
*
* yFiles for HTML also supports a number of other algorithms that compute shortest paths in a graph:
*
* <ul>
* <li><b>{@link ShortestPath}</b> – finds the shortest path between two nodes</li>
* <li><b>{@link SingleSourceShortestPaths}</b> – finds shortest paths from a single source node to several other nodes</li>
* <li><b>{@link AllPairsShortestPaths}</b> – finds all shortest paths between pairs of several source and sink nodes</li>
* </ul>
* <br><b>Highlighting a longest path of a graph</b>
* <pre>
* // prepare the longest path detection algorithm
* const algorithm = new LongestPath()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightLongestPathStyle)
* }
* </pre>
*
* @see PathAlgorithm#findLongestPath
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPath Online Documentation}
*/
export class LongestPath {
/**
Creates a new {@link LongestPath} instance.
@param [options] The parameters to pass.
@param [options.costs] A mapping for edge costs. This option sets the {@link LongestPath#costs} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link LongestPath#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link LongestPath#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPath Online Documentation}*/
constructor(options?: {
costs?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Finds the longest directed path in an acyclic graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link LongestPathResult} containing the longest directed path in the graph.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPath%23run(IGraph) Online Documentation}*/
run(graph: IGraph): LongestPathResult;
/**
Gets or sets a mapping for edge costs.
If no costs are given, all edges have a uniform cost of
<code>1</code>. Path length is then solely determined by the number of edges.
Costs must not be negative.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPath%23costs Online Documentation}*/
costs: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link LongestPath#run} method.
*
* <br><b>Highlighting a longest path on a subset of the graph</b>
* <pre>
* // prepare the longest path detection algorithm
* const algorithm = new LongestPath({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightLongestPathStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPath%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link LongestPath#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link LongestPath#run} method.
*
* <br><b>Highlighting a longest path on a subset of the graph</b>
* <pre>
* // prepare the longest path detection algorithm
* const algorithm = new LongestPath({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the cycle
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightLongestPathStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LongestPath%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_LongestPath: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LongestPath;
}
export interface LouvainModularityClusteringResult extends YObject {
}
/**
Represents clusters in the graph as computed by {@link LouvainModularityClustering}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClusteringResult Online Documentation}*/
export class LouvainModularityClusteringResult {
/**
Gets a mapping from each node to its cluster ID.
The value corresponds to the index of the associated cluster in {@link LouvainModularityClusteringResult#clusters}. For nodes which are not associated with a cluster the
index is <code>-1</code>. Also, if a {@link LouvainModularityClustering#subgraphNodes subgraph} is specified, all nodes that are <b>not</b> part of the subgraph have the ID <code>-1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClusteringResult%23nodeClusterIds Online Documentation}*/
nodeClusterIds: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its cluster.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClusteringResult%23nodeClusters Online Documentation}*/
nodeClusters: ResultItemMapping<INode, Cluster>;
/**
Gets a collection of all clusters in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClusteringResult%23clusters Online Documentation}*/
clusters: ResultItemCollection<Cluster>;
/**
Gets a value which estimates the quality of the partition of the nodes into the {@link LouvainModularityClusteringResult#clusters}.
The modularity value estimates the quality of the partition of the nodes into the given communities and is a value
between
<code>[-1/2,1]</code>. High modularity values denote that nodes with dense connections are placed in the same community, while nodes
with sparse connections are assigned to different communities.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClusteringResult%23modularity Online Documentation}*/
modularity: number;
static $class: Class;
private $_LouvainModularityClusteringResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LouvainModularityClusteringResult;
}
export interface LouvainModularityClustering extends YObject {
}
/**
* Detects the communities in the specified input graph by applying the <i>Louvain modularity</i> method.
* <b>Louvain Modularity</b> algorithm iteratively tries to construct communities by moving nodes from their current community to another until the
* modularity is locally optimized.
*
* The implementation is based on the description in: "Fast unfolding of communities in large networks" by V.D. Blondel,
* J.L. Guillaume, R. Lambiotte and E. Lefebvre, in Journal of Statistical Mechanics: Theory and Experiment 2008 (10),
* P10008 (12pp).
*
* The algorithm starts by assigning each node to its own community. Then, iteratively tries to construct communities by
* moving nodes from their current community to another until the modularity is locally optimized. At the next step, the
* small communities found are merged to a single node and the algorithm starts from the beginning until the modularity of
* the graph cannot be further improved.
*
* If no {@link LouvainModularityClustering#weights} are given, the algorithm will assume that all edges have edge weights equal to <code>1</code>.
*
* <b>Other Clustering Algorithms</b>
*
* yFiles for HTML supports a number of other clustering algorithms:
*
* <ul>
* <li><b>{@link EdgeBetweennessClustering}</b> – partitions the graph into clusters based on edge-betweenness centrality.</li>
* <li><b>{@link KMeansClustering}</b> – partitions the graph into clusters based on the distance between nodes and the cluster midpoints.</li>
* <li><b>{@link HierarchicalClustering}</b> – partitions the graph into clusters by merging smaller clusters based on their distance.</li>
* <li><b>{@link BiconnectedComponentClustering}</b> – partitions the graph into clusters based on its biconnected components.</li>
* <li><b>{@link LabelPropagationClustering}</b> – partitions the graph into clusters by applying a label propagation algorithm.</li>
* </ul>
* <br><b>Calculating clusters of a graph using the Louvain method</b>
* <pre>
* // prepare the louvain modularity algorithm
* const algorithm = new LouvainModularityClustering()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see GroupAlgorithm#louvainModularity
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClustering Online Documentation}
*/
export class LouvainModularityClustering {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.weights] A mapping for edge weights. This option sets the {@link LouvainModularityClustering#weights} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link LouvainModularityClustering#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link LouvainModularityClustering#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClustering Online Documentation}*/
constructor(options?: {
weights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Detects the communities in the specified input graph by applying the <i>Louvain modularity</i> method.
* The algorithm starts by assigning each node to its own community. Then, iteratively tries to construct communities by
* moving nodes from their current community to another until the modularity is locally optimized. At the next step, the
* small communities found are merged to a single node and the algorithm starts from the beginning until the modularity of
* the graph cannot be further improved.
*
* The community index of a node corresponds to the index of the associated community. If there are nodes that are not
* associated with a community, their index is <code>-1</code>.
*
* If no {@link LouvainModularityClustering#weights} are given, the algorithm will assume that all edges have edge weights equal to <code>1</code>.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Calculating clusters of a graph using the Louvain method</b>
* <pre>
* // prepare the louvain modularity algorithm
* const algorithm = new LouvainModularityClustering()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @param graph The graph to partition.
* @returns The calculated clusters.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClustering%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): LouvainModularityClusteringResult;
/**
Gets or sets a mapping for edge weights.
Edge weights influence the importance of certain edges. If no weights are provided at all, all edges have the same
uniform weight of <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClustering%23weights Online Documentation}*/
weights: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link LouvainModularityClustering#run}
* method.
*
* <br><b>Calculating the Louvain modularity on a subset of the graph</b>
* <pre>
* // prepare the louvain modularity algorithm
* const algorithm = new LouvainModularityClustering({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClustering%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link LouvainModularityClustering#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link LouvainModularityClustering#run}
* method.
*
* <br><b>Calculating the Louvain modularity on a subset of the graph</b>
* <pre>
* // prepare the louvain modularity algorithm
* const algorithm = new LouvainModularityClustering({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the clusters with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeClusterIds.get(node)
*   graph.setStyle(node, clusterStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LouvainModularityClustering%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_LouvainModularityClustering: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LouvainModularityClustering;
}
export interface MaximumFlowResult extends YObject {
}
/**
Represents the maximum flow through edges of a graph as computed by {@link MaximumFlowResult#maximumFlow}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlowResult Online Documentation}*/
export class MaximumFlowResult {
/**
Gets the overall maximum flow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlowResult%23maximumFlow Online Documentation}*/
maximumFlow: number;
/**
Gets a mapping from each edge to its flow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlowResult%23flow Online Documentation}*/
flow: ResultItemMapping<IEdge, number>;
/**
Gets a collection of the nodes which are at the source side of the {@link MaximumFlowResult#minimumCut minimum cut}.
The minimum cut is a set of edges separating the graph into two partitions: The {@link MaximumFlowResult#sourcePartition} which contains the {@link MaximumFlow#sources}, and the {@link MaximumFlowResult#sinkPartition}
containing the {@link MaximumFlow#sinks}. The total flow over those edges represents the {@link MaximumFlowResult#maximumFlow} and removing those edges will completely separate the {@link MaximumFlow#sources}
from the {@link MaximumFlow#sinks}, making the maximum possible flow 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlowResult%23sourcePartition Online Documentation}*/
sourcePartition: ResultItemCollection<INode>;
/**
Gets a collection of the nodes which are at the sink side of the {@link MaximumFlowResult#minimumCut minimum cut}.
The minimum cut is a set of edges separating the graph into two partitions: The {@link MaximumFlowResult#sourcePartition} which contains the {@link MaximumFlow#sources}, and the {@link MaximumFlowResult#sinkPartition}
containing the {@link MaximumFlow#sinks}. The total flow over those edges represents the {@link MaximumFlowResult#maximumFlow} and removing those edges will completely separate the {@link MaximumFlow#sources}
from the {@link MaximumFlow#sinks}, making the maximum possible flow 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlowResult%23sinkPartition Online Documentation}*/
sinkPartition: ResultItemCollection<INode>;
/**
Gets a collection of edges in the minimum cut.
The minimum cut is a set of edges separating the graph into two partitions: The {@link MaximumFlowResult#sourcePartition} which contains the {@link MaximumFlow#sources}, and the {@link MaximumFlowResult#sinkPartition}
containing the {@link MaximumFlow#sinks}. The total flow over those edges represents the {@link MaximumFlowResult#maximumFlow} and removing those edges will completely separate the {@link MaximumFlow#sources}
from the {@link MaximumFlow#sinks}, making the maximum possible flow 0.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlowResult%23minimumCut Online Documentation}*/
minimumCut: ResultItemCollection<IEdge>;
static $class: Class;
private $_MaximumFlowResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MaximumFlowResult;
}
export interface MaximumFlow extends YObject {
}
/**
* Solves a maximum flow problem.
* The <em>maximum flow problem</em> is an optimization problem that involves finding the maximum flow from source to sink nodes through a network of
* edges, each with a specified maximum capacity.
*
* The algorithm is an implementation of the <em>preflow-push algorithm</em> (also known as <em>push-relabel algorithm</em>) and is based on
*
* <ul>
* <li>Mehlhorn, Naeher: <i>LEDA: a platform for combinatorial and geometric computing</i>, Cambridge University Press, 2000, pp. 443â€“488.</li>
* </ul>
* <b>Other Flow Algorithms</b>
*
* yFiles for HTML supports another algorithm related to network flow:
*
* <ul>
* <li><b>{@link MinimumCostFlow}</b> – solves a minimum-cost flow problem where a given flow should be routed as cheaply as possible through a graph</li>
* </ul>
* <br><b>Calculating a maximum flow of the graph</b>
* <pre>
* // prepare the maximum flow algorithm
* const algorithm = new MaximumFlow({
*   sources,
*   sinks,
*   capacities
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the partitions and the cut edges
* for (const node of result.sourcePartition) {
*   graph.setStyle(node, sourcePartitionStyle)
* }
* for (const node of result.sinkPartition) {
*   graph.setStyle(node, sinkPartitionStyle)
* }
* for (const edge of result.minimumCut) {
*   graph.setStyle(edge, cutEdgeStyle)
* }
* // add labels indicating the actual flow
* for (const edge of graph.edges) {
*   graph.addLabel(edge, String(result.flow.get(edge)))
* }
* </pre>
*
* @see NetworkFlowAlgorithm#calcMaxFlowMinCut
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlow Online Documentation}
*/
export class MaximumFlow {
/**
Creates a new {@link MaximumFlow} instance.
@param [options] The parameters to pass.
@param [options.capacities] A mapping for capacities of edges. This option sets the {@link MaximumFlow#capacities} property on the created object.
@param [options.sources] A collection of source nodes. This option sets the {@link MaximumFlow#sources} property on the created object.
@param [options.sinks] A collection of sink nodes. This option sets the {@link MaximumFlow#sinks} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link MaximumFlow#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link MaximumFlow#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlow Online Documentation}*/
constructor(options?: {
capacities?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
sources?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
sinks?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Solves a maximum flow problem.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link MaximumFlowResult} containing the computed flows as well as a cut between sources and sinks.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlow%23run(IGraph) Online Documentation}*/
run(graph: IGraph): MaximumFlowResult;
/**
Gets or sets a mapping for capacities of edges.
The capacity of an edge is the maximum possible flow over that edge. <code>0x7FFFFFFF</code> as an edge's capacity signifies an
infinite capacity for that edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlow%23capacities Online Documentation}*/
capacities: ItemMapping<IEdge, number>;
/**
Gets or sets a collection of source nodes.
The source nodes must not be contained in the {@link MaximumFlow#sinks sink nodes} at the same time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlow%23sources Online Documentation}*/
sources: ItemCollection<INode>;
/**
Gets or sets a collection of sink nodes.
The sink nodes must not be contained in the {@link MaximumFlow#sources source node} at the same time.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlow%23sinks Online Documentation}*/
sinks: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link MaximumFlow#run} method.
*
* <br><b>Calculating a maximum flow of a subset of the graph</b>
* <pre>
* // prepare the maximum flow algorithm
* const algorithm = new MaximumFlow({
*   sources,
*   sinks,
*   capacities,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the partitions and the cut edges
* for (const node of result.sourcePartition) {
*   graph.setStyle(node, sourcePartitionStyle)
* }
* for (const node of result.sinkPartition) {
*   graph.setStyle(node, sinkPartitionStyle)
* }
* for (const edge of result.minimumCut) {
*   graph.setStyle(edge, cutEdgeStyle)
* }
* // add labels indicating the actual flow
* for (const edge of graph.edges) {
*   graph.addLabel(edge, String(result.flow.get(edge)))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlow%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link MaximumFlow#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link MaximumFlow#run} method.
*
* <br><b>Calculating a maximum flow of a subset of the graph</b>
* <pre>
* // prepare the maximum flow algorithm
* const algorithm = new MaximumFlow({
*   sources,
*   sinks,
*   capacities,
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the partitions and the cut edges
* for (const node of result.sourcePartition) {
*   graph.setStyle(node, sourcePartitionStyle)
* }
* for (const node of result.sinkPartition) {
*   graph.setStyle(node, sinkPartitionStyle)
* }
* for (const edge of result.minimumCut) {
*   graph.setStyle(edge, cutEdgeStyle)
* }
* // add labels indicating the actual flow
* for (const edge of graph.edges) {
*   graph.addLabel(edge, String(result.flow.get(edge)))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MaximumFlow%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_MaximumFlow: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MaximumFlow;
}
export interface MinimumCostFlowResult extends YObject {
}
/**
Represents a minimum-cost flow through a graph as calculated by {@link MinimumCostFlow}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlowResult Online Documentation}*/
export class MinimumCostFlowResult {
/**
Gets the overall cost.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlowResult%23totalCost Online Documentation}*/
totalCost: number;
/**
Gets a mapping from each edge to the flow over that edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlowResult%23flow Online Documentation}*/
flow: ResultItemMapping<IEdge, number>;
static $class: Class;
private $_MinimumCostFlowResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MinimumCostFlowResult;
}
export interface MinimumCostFlow extends YObject {
}
/**
* Solves a minimum-cost flow problem.
* The <em>minimum-cost flow problem</em> is an optimization problem to find the cheapest possible way of sending a certain amount of flow through a network
* with capacities and costs.
*
* There are three main ways of using this class, depending on how the problem to solve is stated. All variants require the
* {@link MinimumCostFlow#maximumCapacities} and {@link MinimumCostFlow#costs}.
*
* <ol>
* <li>There can be {@link MinimumCostFlow#minimumCapacities} for edges as well as the {@link MinimumCostFlow#maximumCapacities}. In this variant {@link MinimumCostFlow#supply} is considered for defining supply/demand for nodes in the
* network, while {@link MinimumCostFlow#source} and {@link MinimumCostFlow#sink} are ignored.</li>
* <li>A {@link MinimumCostFlow#source} and {@link MinimumCostFlow#sink} node can be provided, in which case {@link MinimumCostFlow#supply} is ignored.</li>
* <li>As a third option, multiple nodes can have specific values for supply and demand configured via {@link MinimumCostFlow#supply}.</li>
* </ol>
* <b>Other Flow Algorithms</b>
*
* yFiles for HTML supports another algorithm related to network flow:
*
* <ul>
* <li><b>{@link MaximumFlow}</b> – solves a maximum flow problem where the largest possible flow through a network should be determined</li>
* </ul>
* <br><b>Calculating a minimum cost flow of the graph</b>
* <pre>
* // prepare the minimum cost flow algorithm
* const algorithm = new MinimumCostFlow({
*   maximumCapacities,
*   supply,
*   costs
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add labels indicating the actual flow
* for (const edge of graph.edges) {
*   graph.addLabel(edge, String(result.flow.get(edge)))
* }
* </pre>
*
* @see NetworkFlowAlgorithm#minCostFlow
* @see NetworkFlowAlgorithm#minCostFlow
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow Online Documentation}
*/
export class MinimumCostFlow {
/**
Creates a new {@link MinimumCostFlow} instance.
@param [options] The parameters to pass.
@param [options.minimumCapacities] A mapping for the minimum capacities of edges. This option sets the {@link MinimumCostFlow#minimumCapacities} property on the created object.
@param [options.maximumCapacities] A mapping for maximum capacities of edges. This option sets the {@link MinimumCostFlow#maximumCapacities} property on the created object.
@param [options.costs] A mapping for edge costs. This option sets the {@link MinimumCostFlow#costs} property on the created object.
@param [options.supply] A mapping which provides the supply (or demand, if negative) for each node. This option sets the {@link MinimumCostFlow#supply} property on the created object.
@param [options.source] A single source node. This option sets the {@link MinimumCostFlow#source} property on the created object.
@param [options.sink] A single sink node. This option sets the {@link MinimumCostFlow#sink} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link MinimumCostFlow#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link MinimumCostFlow#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow Online Documentation}*/
constructor(options?: {
minimumCapacities?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
maximumCapacities?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
costs?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
supply?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
source?: SingleItem<INode> | SingleItemConvertible<INode>;
sink?: SingleItem<INode> | SingleItemConvertible<INode>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Solves the minimum-cost flow problem.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link MinimumCostFlowResult} containing the flow for each edge and the total cost.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow%23run(IGraph) Online Documentation}*/
run(graph: IGraph): MinimumCostFlowResult;
/**
Gets or sets a mapping for the minimum capacities of edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow%23minimumCapacities Online Documentation}*/
minimumCapacities: ItemMapping<IEdge, number>;
/**
Gets or sets a mapping for maximum capacities of edges.
The maximum capacity of an edge is the maximum possible flow over that edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow%23maximumCapacities Online Documentation}*/
maximumCapacities: ItemMapping<IEdge, number>;
/**
Gets or sets a mapping for edge costs.
Edge costs influence how expensive it is to route part of the overall flow over that edge. The goal of this algorithm is
to get as much flow through the overall graph while minimizing the total costs.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow%23costs Online Documentation}*/
costs: ItemMapping<IEdge, number>;
/**
Gets or sets a mapping which provides the supply (or demand, if negative) for each node.
Nodes with a positive supply effectively act as additional sources, while nodes with a negative supply (demand)
effectively act as additional sinks. However, both can appear anywhere in the network.
This value is ignored if both a {@link MinimumCostFlow#source} and {@link MinimumCostFlow#sink} node but no {@link MinimumCostFlow#minimumCapacities} are provided.
If no {@link MinimumCostFlow#source} or {@link MinimumCostFlow#sink} or neither of them are provided it is mandatory to provide a valid supply map.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow%23supply Online Documentation}*/
supply: ItemMapping<INode, number>;
/**
Gets or sets a single source node.
If none is provided the overall flow is calculated. In this case it is mandatory to provide a valid {@link MinimumCostFlow#supply} mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow%23source Online Documentation}*/
source: SingleItem<INode>;
/**
Gets or sets a single sink node.
If none is provided the overall flow is calculated. In this case it is mandatory to provide a valid {@link MinimumCostFlow#supply} mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow%23sink Online Documentation}*/
sink: SingleItem<INode>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link MinimumCostFlow#run} method.
*
* <br><b>Calculating a minimum cost flow of a subset of the graph</b>
* <pre>
* // prepare the minimum cost flow algorithm
* const algorithm = new MinimumCostFlow({
*   maximumCapacities,
*   supply,
*   costs,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add labels indicating the actual flow
* for (const edge of graph.edges) {
*   graph.addLabel(edge, String(result.flow.get(edge)))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link MinimumCostFlow#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link MinimumCostFlow#run} method.
*
* <br><b>Calculating a minimum cost flow of a subset of the graph</b>
* <pre>
* // prepare the minimum cost flow algorithm
* const algorithm = new MinimumCostFlow({
*   maximumCapacities,
*   supply,
*   costs,
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add labels indicating the actual flow
* for (const edge of graph.edges) {
*   graph.addLabel(edge, String(result.flow.get(edge)))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MinimumCostFlow%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_MinimumCostFlow: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MinimumCostFlow;
}
export interface NeighborhoodResult extends YObject {
}
/**
The direct or indirect neighbors of the a given set of {@link Neighborhood#startNodes}.
Represents result of a {@link Neighborhood} {@link Neighborhood#run}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/NeighborhoodResult Online Documentation}*/
export class NeighborhoodResult {
/**
The direct or indirect neighbors of the {@link Neighborhood#startNodes}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NeighborhoodResult%23neighbors Online Documentation}*/
neighbors: ResultItemCollection<INode>;
static $class: Class;
private $_NeighborhoodResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NeighborhoodResult;
}
export interface Neighborhood extends YObject {
}
/**
* Finds the direct or indirect neighbors of a given set of nodes.
* <ul>
* <li>A <em>direct</em> neighbor of a node is directly connected by an edge to that node.</li>
* <li>An <em>indirect</em> neighbor of a node is directly connected to another direct or indirect neighbor of a node.</li>
* </ul>
* The order of the returned nodes is determined by a breadth-first search. No start node will be part of the resulting
* set.
*
* <pre>
* // set up the algorithm to find both predecessors and successors
* const algorithm = new Neighborhood({
*   maximumDistance: 2,
*   traversalDirection: TraversalDirection.BOTH
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the neighbors
* for (const node of result.neighbors) {
*   graph.setStyle(node, highlightNodeStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Neighborhood Online Documentation}
*/
export class Neighborhood {
/**
Creates a new {@link Neighborhood} instance.
@param [options] The parameters to pass.
@param [options.startNodes] A collection of nodes from which to start the search. This option sets the {@link Neighborhood#startNodes} property on the created object.
@param [options.maximumDistance] The maximum distance from the {@link Neighborhood#startNodes} to consider. This option sets the {@link Neighborhood#maximumDistance} property on the created object.
@param [options.traversalDirection] A value indicating whether to follow {@link TraversalDirection#PREDECESSOR incoming}, {@link TraversalDirection#SUCCESSOR outgoing}, or {@link TraversalDirection#BOTH all} edges. This option sets the {@link Neighborhood#traversalDirection} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link Neighborhood#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link Neighborhood#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Neighborhood Online Documentation}*/
constructor(options?: {
startNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
maximumDistance?: number;
traversalDirection?: TraversalDirection | TraversalDirectionStringValues;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Finds neighbors of a set of nodes.
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <pre>
* // set up the algorithm to find both predecessors and successors
* const algorithm = new Neighborhood({
*   maximumDistance: 2,
*   traversalDirection: TraversalDirection.BOTH
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the neighbors
* for (const node of result.neighbors) {
*   graph.setStyle(node, highlightNodeStyle)
* }
* </pre>
*
* @param graph The input graph to run the algorithm on.
* @returns A {@link NeighborhoodResult} containing neighbors of the {@link Neighborhood#startNodes} in the given {@link Neighborhood#traversalDirection direction}.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Neighborhood%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): NeighborhoodResult;
/**
Gets or sets a collection of nodes from which to start the search.
If this collection is empty, the result will also be empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Neighborhood%23startNodes Online Documentation}*/
startNodes: ItemCollection<INode>;
/**
Gets or sets the maximum distance from the {@link Neighborhood#startNodes} to consider.
This is the maximum number of edges that will be followed b the search. A value of <code>1</code> will only include <em>direct</em> neighbors of {@link Neighborhood#startNodes},
while higher values will include <em>indirect</em> neighbors as well. A value of <code>0</code> will not limit the maximum search distance.
Default is <code>0</code>.
@default 0
@see {@link https://docs.yworks.com/yfileshtml/#/api/Neighborhood%23maximumDistance Online Documentation}*/
maximumDistance: number;
/**
Gets or sets a value indicating whether to follow {@link TraversalDirection#PREDECESSOR incoming}, {@link TraversalDirection#SUCCESSOR outgoing}, or {@link TraversalDirection#BOTH all} edges.
Default is {@link TraversalDirection#BOTH}.
@default TraversalDirection.BOTH
@see {@link https://docs.yworks.com/yfileshtml/#/api/Neighborhood%23traversalDirection Online Documentation}*/
traversalDirection: TraversalDirection;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link Neighborhood#run} method.
*
* <br><b>Finding the neighbors on a subset of the graph</b>
* <pre>
* // search for the direct successors
* const algorithm = new Neighborhood({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   },
*   traversalDirection: TraversalDirection.SUCCESSOR,
*   maximumDistance: 1
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the neighbors
* for (const node of result.neighbors) {
*   graph.setStyle(node, highlightNodeStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Neighborhood%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link Neighborhood#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link Neighborhood#run} method.
*
* <br><b>Finding the neighbors on a subset of the graph</b>
* <pre>
* // search for the predecessors up to depth 3
* const algorithm = new Neighborhood({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   },
*   maximumDistance: 3,
*   traversalDirection: TraversalDirection.PREDECESSOR
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the neighbors
* for (const node of result.neighbors) {
*   graph.setStyle(node, highlightNodeStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Neighborhood%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_Neighborhood: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Neighborhood;
}
export interface NodeAggregationResult extends YObject {
}
/**
The tree of {@link NodeAggregate} which is created by {@link NodeAggregation}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationResult Online Documentation}*/
export class NodeAggregationResult {
/**
Gets a map which maps an {@link NodeAggregate} to its corresponding node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationResult%23aggregateMap Online Documentation}*/
aggregateMap: ResultItemMapping<INode, NodeAggregate>;
/**
The root of the tree of {@link NodeAggregate} which is created by {@link NodeAggregation}.
The tree can be traversed using the {@link NodeAggregate#children} and {@link NodeAggregate#parent} relationships.
Only {@link NodeAggregate}s which are associated with a node have a non-<code>null</code> {@link NodeAggregate#node} property. These
{@link NodeAggregate}s can also be found in the {@link NodeAggregationResult#aggregateMap}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregationResult%23root Online Documentation}*/
root: NodeAggregate;
static $class: Class;
private $_NodeAggregationResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeAggregationResult;
}
export interface NodeAggregate extends YObject {
}
/**
Represents an aggregate or cluster found by {@link NodeAggregation}.
The clusters form a tree. The tree has exactly one aggregate which has no {@link NodeAggregate#parent}. This is the {@link NodeAggregationResult#root} of the tree.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregate Online Documentation}*/
export class NodeAggregate {
/**
The parent of this {@link NodeAggregate}.
This is <code>null</code> for the {@link NodeAggregationResult#root}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregate%23parent Online Documentation}*/
parent: NodeAggregate | null;
/**
Gets the number of nodes that are descendants of this aggregate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregate%23descendantCount Online Documentation}*/
descendantCount: number;
/**
Gets the sum of the weights of nodes that are descendants of this aggregate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregate%23descendantWeightSum Online Documentation}*/
descendantWeightSum: number;
/**
Gets the maximum weight of nodes that are descendants of this aggregate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregate%23maximumDescendantWeight Online Documentation}*/
maximumDescendantWeight: number;
/**
The node which is represented by this aggregate.
Not all aggregates represent a node, so this property may be <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregate%23node Online Documentation}*/
node: INode | null;
/**
The child aggregates of this aggregate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregate%23children Online Documentation}*/
children: IListEnumerable<NodeAggregate>;
/**
Returns all nodes which are associated with this {@link NodeAggregate} or its descendants.
@returns The associated nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregate%23nodes Online Documentation}*/
readonly nodes: IEnumerable<INode>;
static $class: Class;
private $_NodeAggregate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeAggregate;
}
export interface NodeAggregation extends YObject {
}
/**
* Aggregates the nodes of a given graph and creates a hierarchical clustering structure subject to user-specified
* constraints.
* The result is subject to user-specified constraints like the {@link NodeAggregation#nodeTypeHandling type of nodes} as well as the preferred {@link NodeAggregation#minimumClusterSize minimum} and {@link NodeAggregation#maximumClusterSize maximum}
* size of a cluster. The result of the aggregation can be used to (interactively) visualize parts of large graphs.
*
* Note that the resulting clustering structure corresponds to a directed rooted tree which is encoded by means of a set of
* {@link NodeAggregate} instances. More precisely, each node of the original graph is {@link NodeAggregationResult#aggregateMap mapped} to a unique {@link NodeAggregate}
* instance. Each {@link NodeAggregate} has a reference to its {@link NodeAggregate#parent} which induces a directed tree structure.There is
* always exactly one {@link NodeAggregate} without a parent that represents the {@link NodeAggregationResult#root} of the tree.
*
* {@link NodeAggregation#aggregation} specifies whether the algorithm should consider {@link NodeAggregationPolicy#STRUCTURAL structural properties} (i.e., considering the connectivity) for the
* aggregation or {@link NodeAggregationPolicy#GEOMETRIC geometric properties} (i.e., the distance between nodes).
*
* <pre>
* // prepare the node aggregation algorithm
* const algorithm = new NodeAggregation({
*   // group according to the node types which are specified in the node's tag
*   nodeTypeHandling: NodeAggregationNodeTypeHandlingPolicy.SEPARATE_AT_ROOT,
*   nodeTypes: node => node.tag,
*   // determine substructures according to the graph structure, not the geometry
*   aggregation: NodeAggregationPolicy.STRUCTURAL
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the aggregates with different styles
* // group by the top level aggregates
* let index = 0
* for (const aggregate of result.root.children) {
*   for (const node of aggregate.nodes) {
*     graph.setStyle(node, clusterStyles.get(index))
*   }
*   index++
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation Online Documentation}
*/
export class NodeAggregation {
/**
Creates a new instance with default settings.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link NodeAggregation#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link NodeAggregation#subgraphEdges} property on the created object.
@param [options.topLevelNodes] The top-level nodes of the aggregation info. This option sets the {@link NodeAggregation#topLevelNodes} property on the created object.
@param [options.nodeTypes] A mapping which maps nodes to an object which specifies their type. This option sets the {@link NodeAggregation#nodeTypes} property on the created object.
@param [options.nodeWeights] A mapping for specifying the (non-negative) weights of the nodes. This option sets the {@link NodeAggregation#nodeWeights} property on the created object.
@param [options.edgeWeights] A mapping for specifying the (non-negative) weights of the edges. This option sets the {@link NodeAggregation#edgeWeights} property on the created object.
@param [options.edgeDirectedness] A mapping for specifying the directedness of edges. This option sets the {@link NodeAggregation#edgeDirectedness} property on the created object.
@param [options.aggregation] The policy applied for determining the clusters. This option sets the {@link NodeAggregation#aggregation} property on the created object.
@param [options.multiThreadingAllowed] Whether or not the layout algorithm may use multi-threading to reduce the running time. This option sets the {@link NodeAggregation#multiThreadingAllowed} property on the created object.
@param [options.nodeTypeHandling] How node types are handled for aggregation. This option sets the {@link NodeAggregation#nodeTypeHandling} property on the created object.
@param [options.minimumClusterSize] The preferred minimum number of elements contained in a cluster. This option sets the {@link NodeAggregation#minimumClusterSize} property on the created object.
@param [options.maximumClusterSize] The preferred maximum number of elements contained in a cluster. This option sets the {@link NodeAggregation#maximumClusterSize} property on the created object.
@param [options.maximumDuration] The maximum duration that this algorithm is allowed to run. This option sets the {@link NodeAggregation#maximumDuration} property on the created object.
@param [options.nodesOnlyOnLeaves] Whether or not nodes are only mapped to leaves of the directed rooted {@link NodeAggregation#aggregation} tree that represents the hierarchical clustering structure. This option sets the {@link NodeAggregation#nodesOnlyOnLeaves} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
topLevelNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
nodeTypes?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
nodeWeights?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
edgeWeights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
aggregation?: NodeAggregationPolicy | NodeAggregationPolicyStringValues;
multiThreadingAllowed?: boolean;
nodeTypeHandling?: NodeAggregationNodeTypeHandlingPolicy | NodeAggregationNodeTypeHandlingPolicyStringValues;
minimumClusterSize?: number;
maximumClusterSize?: number;
maximumDuration?: TimeSpan | TimeSpanConvertible;
nodesOnlyOnLeaves?: boolean;
});
/**
* Aggregates the nodes of the given <code>graph</code> and creates a hierarchical clustering structure.
* The aggregation result corresponds to a directed {@link NodeAggregationResult#root rooted} tree which is encoded by means of a set of {@link NodeAggregate}
* instances. More precisely, each node of the original graph is {@link NodeAggregationResult#aggregateMap mapped} to a unique {@link NodeAggregate} instance.
* Each {@link NodeAggregate} has references to its {@link NodeAggregate#parent} and {@link NodeAggregate#children} which induces a directed tree structure. There is always
* exactly one {@link NodeAggregate} without a parent that represents the {@link NodeAggregationResult#root} of the tree.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <pre>
* // prepare the node aggregation algorithm
* const algorithm = new NodeAggregation({
*   // group according to the node types which are specified in the node's tag
*   nodeTypeHandling: NodeAggregationNodeTypeHandlingPolicy.SEPARATE_AT_ROOT,
*   nodeTypes: node => node.tag,
*   // determine substructures according to the graph structure, not the geometry
*   aggregation: NodeAggregationPolicy.STRUCTURAL
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the aggregates with different styles
* // group by the top level aggregates
* let index = 0
* for (const aggregate of result.root.children) {
*   for (const node of aggregate.nodes) {
*     graph.setStyle(node, clusterStyles.get(index))
*   }
*   index++
* }
* </pre>
*
* @param graph The input graph to run the algorithm on.
* @returns The hierarchical clustering of the <code>graph</code>.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): NodeAggregationResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link NodeAggregation#run} method.
*
* <br><b>Aggregating the clusters on a subset of the graph</b>
* <pre>
* // prepare the node aggregation algorithm
* const algorithm = new NodeAggregation({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the aggregates with different styles
* // group by the top level aggregates
* let index = 0
* for (const aggregate of result.root.children) {
*   for (const node of aggregate.nodes) {
*     graph.setStyle(node, clusterStyles.get(index))
*   }
*   index++
* }
* </pre>
*
* <br><b>Considering only selected nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes)
* </pre>
*
* <br><b>Ignoring all group nodes</b>
* <pre>
* const subset = new ItemCollection()
* subset.excludes = n => graph.isGroupNode(n)
* </pre>
*
* <br><b>Considering selected nodes but ignoring group nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes, n =>
*   graph.isGroupNode(n)
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link NodeAggregation#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link NodeAggregation#run} method.
*
* <br><b>Aggregating the clusters on a subset of the graph</b>
* <pre>
* // prepare the node aggregation algorithm
* const algorithm = new NodeAggregation({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the aggregates with different styles
* // group by the top level aggregates
* let index = 0
* for (const aggregate of result.root.children) {
*   for (const node of aggregate.nodes) {
*     graph.setStyle(node, clusterStyles.get(index))
*   }
*   index++
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
/**
Gets or sets the top-level nodes of the aggregation info.
Top-level nodes are directly contained in the root cluster of the aggregation result or in direct children of the root
if there are top-level nodes with different types(nodes of different type are not allowed to be directly contained in
the same cluster). The only exception to this rules is if there are {@link IGraph#isGroupNode group nodes} because a group is a kind of
user-specified cluster and, thus, a top-level node contained in a group is always placed in the cluster associated with
that group.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23topLevelNodes Online Documentation}*/
topLevelNodes: ItemCollection<INode>;
/**
Gets or sets a mapping which maps nodes to an object which specifies their type.
The type objects can be arbitrary objects. Nodes mapped to equal objects are considered to be of the same type. After
aggregating, the resulting clusters do never contain nodes with different type.
Node types are ignored if nothing is set here. Also, if the {@link NodeAggregation#nodeTypeHandling} is set to
{@link NodeAggregationNodeTypeHandlingPolicy#IGNORE}, the node types are not considered, either.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23nodeTypes Online Documentation}*/
nodeTypes: ItemMapping<INode, any>;
/**
Gets or sets a mapping for specifying the (non-negative) weights of the nodes.
Depending on the exact setting, the algorithm applies multiple different aggregation approaches and chooses the best
one. If nodes have weights, it prefers aggregation results where nodes with higher weight are closer to the root of the
aggregation hierarchy.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23nodeWeights Online Documentation}*/
nodeWeights: ItemMapping<INode, number>;
/**
Gets or sets a mapping for specifying the (non-negative) weights of the edges.
If the {@link NodeAggregation#aggregation} is set to {@link NodeAggregationPolicy#STRUCTURAL} nodes connected by edges with high weights are more
likely to be clustered together.
<h2>Note</h2>
Edge weights are only considered if the {@link NodeAggregationPolicy aggregation mode} is set to {@link NodeAggregationPolicy#STRUCTURAL}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23edgeWeights Online Documentation}*/
edgeWeights: ItemMapping<IEdge, number>;
/**
Gets or sets a mapping for specifying the directedness of edges.
Generally, for this node aggregation algorithm, the effect of specifying an edge directedness is quite low. It is mainly
used for the detection of substructures, e.g. {@link TreeSubstructures} or
{@link CycleSubstructures}. A substructure is only identified as such if all edges are either undirected or
consistently directed with respect to the specified directedness.
<ul>
<li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.
<li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.
<li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.
</ul>
<h2>Note</h2>
If nothing is set, the algorithm assumes that all edges have directedness <code>0.0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23edgeDirectedness Online Documentation}*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
Gets or sets the policy applied for determining the clusters.
Default is {@link NodeAggregationPolicy#STRUCTURAL}.
@default 'Structural'
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23aggregation Online Documentation}*/
aggregation: NodeAggregationPolicy;
/**
Gets or sets whether or not the layout algorithm may use multi-threading to reduce the running time.
Default is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23multiThreadingAllowed Online Documentation}*/
multiThreadingAllowed: boolean;
/**
Gets or sets how node types are handled for aggregation.
The types are provided by {@link NodeAggregation#nodeTypes}. Nodes with equal objects mapped to them are handled as equal types.
Default is {@link NodeAggregationNodeTypeHandlingPolicy#IGNORE}.
@default 'Ignore'
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23nodeTypeHandling Online Documentation}*/
nodeTypeHandling: NodeAggregationNodeTypeHandlingPolicy;
/**
Gets or sets the preferred minimum number of elements contained in a cluster.
Default ist <code>5</code>
<h2>Note</h2>
The algorithm doesn't guarantee that the found solution observes this value.
@throws {Exception} <b>(ArgumentError)</b> if a value <code>&lt; 1</code> is set.
@default 5
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23minimumClusterSize Online Documentation}*/
minimumClusterSize: number;
/**
Gets or sets the preferred maximum number of elements contained in a cluster.
Default ist <code>10</code>
<h2>Note</h2>
The algorithm doesn't guarantee that the found solution observes this value.
@throws {Exception} <b>(ArgumentError)</b> if a value <code>&lt; 2</code> is set.
@default 10
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23maximumClusterSize Online Documentation}*/
maximumClusterSize: number;
/**
Gets or sets the maximum duration that this algorithm is allowed to run.
Setting {@link TimeSpan#ZERO} lets the algorithm run unrestricted.
Default is {@link TimeSpan#ZERO}; the algorithm runs unrestricted.
<h2>Note</h2>
Restricting the maximum duration may result in a lower layout quality. Furthermore, the real runtime may exceed the
maximum duration since the aggregation algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the specified duration is less than {@link TimeSpan#ZERO}.
@default 'Zero'
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23maximumDuration Online Documentation}*/
maximumDuration: TimeSpan;
/**
Gets or sets whether or not nodes are only mapped to leaves of the directed rooted {@link NodeAggregation#aggregation} tree that represents the
hierarchical clustering structure.
If set to <code>false</code>, nodes can also represent inner nodes and, thus, the associated {@link NodeAggregate} can
contain {@link NodeAggregate#children other node elements}. This may allow a more efficient representation of the resulting tree structure. For
example, for a star-like structure in the input graph, the root of the star can directly represent the {@link NodeAggregate#parent parent aggregate}
of the star. Otherwise, an additional {@link NodeAggregate virtual tree node} is inserted.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/NodeAggregation%23nodesOnlyOnLeaves Online Documentation}*/
nodesOnlyOnLeaves: boolean;
static $class: Class;
private $_NodeAggregation: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is NodeAggregation;
}
export interface PageRankResult extends YObject {
}
/**
Represents the page rank values as computed by {@link PageRank}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRankResult Online Documentation}*/
export class PageRankResult {
/**
Gets a mapping from each node to its page rank value.
A value between <code>0.0</code> and <code>1.0</code> that stores the page rank found for this node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRankResult%23pageRank Online Documentation}*/
pageRank: ResultItemMapping<INode, number>;
/**
Gets the sum of all page ranks.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRankResult%23rankSum Online Documentation}*/
rankSum: number;
static $class: Class;
private $_PageRankResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PageRankResult;
}
export interface PageRank extends YObject {
}
/**
* Computes page rank values for all nodes based on their attached edges.
* The original algorithm was proposed by Sergey Brin and Lawrence Page as PageRank algorithm and refined later in several
* papers:
* <br />
* S. Brin and L. Page. The anatomy of a large-scale hypertextual Web search
* engine.
* <br />
* Computer Networks and ISDN Systems, 30(1-7):107-117
* <br />
* 1998.
*
* The algorithm starts by initializing the rank value for each node. After that it uses multiple iterations to propagate
* the rank of a node to its successors. The base factor for each successor is <code>1</code>. These base factors are multiplied by {@link PageRank#edgeWeights edge weights}
* and {@link PageRank#nodeWeights node weights} if specified.
*
* The final node factors are scaled afterwards to sum up to <code>1</code> so the overall rank stays the same after each iteration. The
* old page rank of the node multiplied with these scaled factors are then distributed to the successors.
*
* When a node can't distribute its rank, e.g. because it has no successors, its rank is distributed between all nodes
* scaled by their weight (see {@link PageRank#nodeWeights}). Furthermore a <code>damping factor</code> is used so that a share of each nodes' rank isn't propagated
* to the successors but also distributed between all nodes.
*
* After each iteration, the relative change of each node's page rank ((newRank - oldRank)/oldRank) is calculated. If all
* page rank changes are below the specified <code>precision</code> or if the maximal iterations are reached, the algorithm stops.
*
* The {@link PageRank#edgeDirectedness} defines the direction of the edges. This in consequence defines the propagation direction of ranks. Edges with a
* positive directedness value propagate the rank from the source to the target node - this is also the default if the
* given data provider is <code>null</code>. Edges which have a directedness of <code>0</code> are considered to be undirected so that propagation
* happens from source to target and from target to source. Edges with a negative directedness are considered to have the
* reverse direction such that they propagate ranks only from target to source.
*
* <b>Other Centrality Measures</b>
*
* yFiles for HTML supports a number of other centrality measures:
*
* <ul>
* <li><b>{@link GraphCentrality}, {@link ClosenessCentrality}</b> – emphasize nodes that have short paths to other nodes</li>
* <li><b>{@link DegreeCentrality}</b> – emphasizes nodes with many edges</li>
* <li><b>{@link WeightCentrality}</b> – emphasizes nodes with highly-weighted edges</li>
* <li><b>{@link BetweennessCentrality}</b> – emphasizes nodes and edges that are part of many short paths</li>
* <li><b>{@link EigenvectorCentrality}</b> – computes the influence a node has on a network. The centrality value is higher if more nodes are connected to that
* node.</li>
* </ul>
* <h2>Note</h2>
*
* If no {@link PageRank#initialPageRank} is set all nodes have an initial rank of <code>1.0</code>.
*
* <h2>Note</h2>
*
* If no {@link PageRank#nodeWeights} are provided, the algorithm assumes that all nodes have weight <code>1.0</code>.
*
* <h2>Note</h2>
*
* If no {@link PageRank#edgeWeights} are provided, the algorithm assumes that all edges have weight <code>1.0</code>.
*
* <h2>Note</h2>
*
* The {@link PageRank#dampingFactor damping factor} has to lie between <code>0</code> and <code>1</code> where <code>0</code> means all the rank of node is distributed between all nodes based
* on their node weight and <code>1</code> means all the rank of a node is only distributed between its successors.
*
* <pre>
* const result = new PageRank({
*   nodeWeights: node => node.layout.width,
*   edgeWeights: edge =>
*     edge.style instanceof PolylineEdgeStyle ? edge.style.stroke.thickness : 1
* }).run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.pageRank.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(node, String(centrality))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank Online Documentation}
*/
export class PageRank {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.precision] The non-negative precision of the calculated page ranks. This option sets the {@link PageRank#precision} property on the created object.
@param [options.dampingFactor] A value between <code>0</code> and <code>1</code> denoting the factor of a node's rank that shall be distributed to its successors. This option sets the {@link PageRank#dampingFactor} property on the created object.
@param [options.initialPageRank] A mapping that specifies the initial page rank for each node. This option sets the {@link PageRank#initialPageRank} property on the created object.
@param [options.nodeWeights] A mapping for node weights. This option sets the {@link PageRank#nodeWeights} property on the created object.
@param [options.edgeWeights] A mapping for edge weights. This option sets the {@link PageRank#edgeWeights} property on the created object.
@param [options.edgeDirectedness] A mapping that stores the directedness of the edges. This option sets the {@link PageRank#edgeDirectedness} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link PageRank#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link PageRank#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank Online Documentation}*/
constructor(options?: {
precision?: number;
dampingFactor?: number;
initialPageRank?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
nodeWeights?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
edgeWeights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Computes page rank values for all nodes based on their attached edges.
* The original algorithm was proposed by Sergey Brin and Lawrence Page as PageRank algorithm and refined later in several
* papers:
* <br />
* S. Brin and L. Page. The anatomy of a large-scale hypertextual Web search
* engine.
* <br />
* Computer Networks and ISDN Systems, 30(1-7):107-117
* <br />
* 1998.
*
* The algorithm starts by initializing the rank value for each node. After that it uses multiple iterations to propagate
* the rank of a node to its successors. The base factor for each successor is <code>1</code>. These base factors are multiplied by {@link PageRank#edgeWeights edge weights}
* and {@link PageRank#nodeWeights node weights} if specified.
*
* The final node factors are scaled afterwards to sum up to <code>1</code> so the overall rank stays the same after each iteration. The
* old page rank of the node multiplied with these scaled factors are then distributed to the successors.
*
* When a node can't distribute its rank, e.g. because it has no successors, its rank is distributed between all nodes
* scaled by their weight (see {@link PageRank#nodeWeights}). Furthermore a <code>damping factor</code> is used so that a share of each nodes' rank isn't propagated
* to the successors but also distributed between all nodes.
*
* After each iteration, the relative change of each node's page rank ((newRank - oldRank)/oldRank) is calculated. If all
* page rank changes are below the specified <code>precision</code> or if the maximal iterations are reached, the algorithm stops.
*
* The {@link PageRank#edgeDirectedness} defines the direction of the edges. This in consequence defines the propagation direction of ranks. Edges with a
* positive directedness value propagate the rank from the source to the target node - this is also the default if the
* given data provider is <code>null</code>. Edges which have a directedness of <code>0</code> are considered to be undirected so that propagation
* happens from source to target and from target to source. Edges with a negative directedness are considered to have the
* reverse direction such that they propagate ranks only from target to source.
*
* <b>Other Centrality Measures</b>
*
* yFiles for HTML supports a number of other centrality measures:
*
* <ul>
* <li><b>{@link GraphCentrality}, {@link ClosenessCentrality}</b> – emphasize nodes that have short paths to other nodes</li>
* <li><b>{@link DegreeCentrality}</b> – emphasizes nodes with many edges</li>
* <li><b>{@link WeightCentrality}</b> – emphasizes nodes with highly-weighted edges</li>
* <li><b>{@link BetweennessCentrality}</b> – emphasizes nodes and edges that are part of many short paths</li>
* </ul>
* <h2>Note</h2>
*
* If no {@link PageRank#initialPageRank} is set all nodes have an initial rank of <code>1.0</code>.
*
* <h2>Note</h2>
*
* If no {@link PageRank#nodeWeights} are provided, the algorithm assumes that all nodes have weight <code>1.0</code>.
*
* <h2>Note</h2>
*
* If no {@link PageRank#edgeWeights} are provided, the algorithm assumes that all edges have weight <code>1.0</code>.
*
* <h2>Note</h2>
*
* The {@link PageRank#dampingFactor damping factor} has to lie between <code>0</code> and <code>1</code> where <code>0</code> means all the rank of node is distributed between all nodes based
* on their node weight and <code>1</code> means all the rank of a node is only distributed between its successors.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <pre>
* const result = new PageRank({
*   nodeWeights: node => node.layout.width,
*   edgeWeights: edge =>
*     edge.style instanceof PolylineEdgeStyle ? edge.style.stroke.thickness : 1
* }).run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.pageRank.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(node, String(centrality))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @param graph The input graph to run the algorithm on.
* @returns A {@link PageRankResult} from which the calculated centrality values can be obtained. The values are between <code>0.0</code>
* and <code>1.0</code>.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): PageRankResult;
/**
Gets or sets the non-negative precision of the calculated page ranks.
The default is <code>0.001</code>.
@throws {Exception} <b>(ArgumentError)</b> if the value is negative.
@default 0.001
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank%23precision Online Documentation}*/
precision: number;
/**
Gets or sets a value between <code>0</code> and <code>1</code> denoting the factor of a node's rank that shall be distributed to its successors.
The damping factor has to lie between <code>0</code> and <code>1</code> where <code>0</code> means all the rank of node is distributed between all nodes based
on their node weight and <code>1</code> means all the rank of a node is only distributed between its successors. The default is
<code>0.85</code>.
@throws {Exception} <b>(ArgumentError)</b> if the value is not between <code>0</code> and <code>1</code>.
@default 0.85
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank%23dampingFactor Online Documentation}*/
dampingFactor: number;
/**
Gets or sets a mapping that specifies the initial page rank for each node.
If none is provided all nodes have an initial rank of <code>1.0</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank%23initialPageRank Online Documentation}*/
initialPageRank: ItemMapping<INode, number>;
/**
Gets or sets a mapping for node weights.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank%23nodeWeights Online Documentation}*/
nodeWeights: ItemMapping<INode, number>;
/**
Gets or sets a mapping for edge weights.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank%23edgeWeights Online Documentation}*/
edgeWeights: ItemMapping<IEdge, number>;
/**
Gets or sets a mapping that stores the directedness of the edges.
This in consequence defines the propagation direction of ranks. Edges with a positive directedness value propagate the
rank from the source to the target node - this is also the default if nothing is set here. Edges which have a
directedness of <code>0</code> are considered to be undirected so that propagation happens from source to target and from target to
source. Edges with a negative directedness are considered to have the reverse direction such that they propagate ranks
only from target to source.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank%23edgeDirectedness Online Documentation}*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link PageRank#run} method.
*
* <br><b>Calculating the page rank on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new PageRank({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add labels for centrality values
* result.pageRank.forEach(({ key, value }) => {
*   graph.addLabel(key, String(value))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link PageRank#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link PageRank#run} method.
*
* <br><b>Calculating the page rank on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new PageRank({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add labels for centrality values
* result.pageRank.forEach(({ key, value }) => {
*   graph.addLabel(key, String(value))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PageRank%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_PageRank: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PageRank;
}
export interface PathsResult extends YObject {
}
/**
Represents all paths between two sets of nodes as computed by {@link PathsResult#paths}.
When querying this result for a path or enumerating it, the paths are found on the fly. Enumerating all paths can be
slow if the graph is large and {@link Paths#startNodes} or {@link Paths#endNodes} contain many nodes. Generally this should be used to find paths satisfying certain
criteria and only enumerating until a suitable path is found to avoid walking the whole graph.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathsResult Online Documentation}*/
export class PathsResult {
/**
Gets a collection of all paths between the given <code>start</code> and <code>end</code> node.
This collection's {@link IEnumerable#getEnumerator enumerator} calculates paths only when needed. This also means that accessing the {@link ResultItemCollection#size} or via an index will
cause all paths to be calculated immediately which can be slow.
@param start The start node to get the paths for.
@param end The end node to get the paths for.
@returns All paths between <code>start</code> and <code>end</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathsResult%23getAllPathsBetween(INode%2CINode) Online Documentation}*/
getAllPathsBetween(start: INode, end: INode): ResultItemCollection<Path>;
/**
Returns a path between the given <code>start</code> and <code>end</code> node if one exists.
@param start The start node to get the path for.
@param end The end node to get the path for.
@returns A path between <code>start</code> and <code>end</code> or <code>null</code> if no path exists between the two nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathsResult%23getPathBetween(INode%2CINode) Online Documentation}*/
getPathBetween(start: INode, end: INode): Path | null;
/**
Determines whether a path between <code>start</code> and <code>end</code> exists.
@param start The start node.
@param end The end node.
@returns <code>true</code> if a path exists between <code>start</code> and <code>end</code>, <code>false</code> if not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathsResult%23hasPathBetween(INode%2CINode) Online Documentation}*/
hasPathBetween(start: INode, end: INode): boolean;
/**
Gets a collection of all paths between the {@link Paths#startNodes} and {@link Paths#endNodes}.
This collection's {@link IEnumerable#getEnumerator enumerator} calculates paths only when needed. This also means that accessing the {@link ResultItemCollection#size} or via an index will
cause all paths to be calculated immediately which can be slow.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PathsResult%23paths Online Documentation}*/
paths: ResultItemCollection<Path>;
static $class: Class;
private $_PathsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PathsResult;
}
export interface Paths extends YObject {
}
/**
* Finds all simple directed or undirected paths between one or more {@link Paths#startNodes} and {@link Paths#endNodes}.
* <b>Other Path-Related Algorithms</b>
*
* yFiles for HTML supports a number of other algorithms related to paths in a graph:
*
* <ul>
* <li><b>{@link Chains}</b> – finds all chains, that is, sequences of nodes that are each connected with just an edge without branches</li>
* <li><b>{@link Cycle}</b> – finds a cycle if one exists</li>
* <li><b>{@link LongestPath}</b> – finds the longest path in the graph</li>
* </ul>
* <br><b>Calculating all simple paths between start and end nodes</b>
* <pre>
* // prepare the paths algorithm
* const algorithm = new Paths({ startNodes: startNode, endNodes: endNode })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Highlight paths between start and end
* for (const path of result.paths) {
*   for (const pathEdge of path.edges) {
*     graph.setStyle(pathEdge, pathEdgeStyle)
*   }
* }
* </pre>
*
* @see PathAlgorithm#findAllPathsCursor
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Paths Online Documentation}
*/
export class Paths {
/**
Creates a new {@link Paths} instance.
@param [options] The parameters to pass.
@param [options.startNodes] A collection of start nodes. This option sets the {@link Paths#startNodes} property on the created object.
@param [options.endNodes] A collection of end nodes. This option sets the {@link Paths#endNodes} property on the created object.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link Paths#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link Paths#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link Paths#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Paths Online Documentation}*/
constructor(options?: {
startNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
endNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Finds all paths between {@link Paths#startNodes} and {@link Paths#endNodes}.
Unlike most algorithms, the result of this one computes the next path only when needed, so despite the complexity it is
not expensive up-front; only when enumerating every single path.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link PathsResult} containing all paths between {@link Paths#startNodes} and {@link Paths#endNodes}.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Paths%23run(IGraph) Online Documentation}*/
run(graph: IGraph): PathsResult;
/**
Gets or sets a collection of start nodes.
The algorithm finds all paths between all {@link Paths#startNodes} and all {@link Paths#endNodes}. If either {@link Paths#startNodes} or {@link Paths#endNodes} is empty, the result will contain no paths.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Paths%23startNodes Online Documentation}*/
startNodes: ItemCollection<INode>;
/**
Gets or sets a collection of end nodes.
The algorithm finds all paths between all {@link Paths#startNodes} and all {@link Paths#endNodes}. If either {@link Paths#startNodes} or {@link Paths#endNodes} is empty, the result will contain no paths.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Paths%23endNodes Online Documentation}*/
endNodes: ItemCollection<INode>;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/Paths%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link Paths#run} method.
*
* <br><b>Calculating all paths in a subset of the graph</b>
* <pre>
* // prepare the paths algorithm
* const algorithm = new Paths({
*   startNodes: startNode,
*   endNodes: endNode,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Highlight paths between start and end
* for (const path of result.paths) {
*   for (const pathEdge of path.edges) {
*     graph.setStyle(pathEdge, pathEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Paths%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link Paths#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link Paths#run} method.
*
* <br><b>Calculating all paths in a subset of the graph</b>
* <pre>
* // prepare the paths algorithm
* const algorithm = new Paths({
*   startNodes: startNode,
*   endNodes: endNode,
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Highlight paths between start and end
* for (const path of result.paths) {
*   for (const pathEdge of path.edges) {
*     graph.setStyle(pathEdge, pathEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Paths%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_Paths: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Paths;
}
export interface ReachabilityResult extends YObject {
}
/**
Represents the nodes that are reachable from a given set of other nodes as computed by {@link Reachability}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReachabilityResult Online Documentation}*/
export class ReachabilityResult {
/**
Determines whether a given node is reachable.
@param node
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReachabilityResult%23isReachable(INode) Online Documentation}*/
isReachable(node: INode): boolean;
/**
Gets a collection of nodes that are reachable from the {@link Reachability#startNodes}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReachabilityResult%23reachableNodes Online Documentation}*/
reachableNodes: ResultItemCollection<INode>;
static $class: Class;
private $_ReachabilityResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReachabilityResult;
}
export interface Reachability extends YObject {
}
/**
Determines the nodes that are reachable from one or more {@link Reachability#startNodes}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Reachability Online Documentation}*/
export class Reachability {
/**
Creates a new {@link Reachability} instance.
@param [options] The parameters to pass.
@param [options.startNodes] A collection of nodes from which to determine reachability. This option sets the {@link Reachability#startNodes} property on the created object.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link Reachability#directed} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Reachability Online Documentation}*/
constructor(options?: {
startNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
directed?: boolean;
});
/**
Determines the set of nodes that are reachable from the {@link Reachability#startNodes}.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link ReachabilityResult} containing the nodes reachable from {@link Reachability#startNodes}.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Reachability%23run(IGraph) Online Documentation}*/
run(graph: IGraph): ReachabilityResult;
/**
Gets or sets a collection of nodes from which to determine reachability.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Reachability%23startNodes Online Documentation}*/
startNodes: ItemCollection<INode>;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/Reachability%23directed Online Documentation}*/
directed: boolean;
static $class: Class;
private $_Reachability: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Reachability;
}
export interface ResultItemCollection<T> extends YObject, IListEnumerable<T> {
/**
@inheritdoc*/
get(i: number): T;
/**
@inheritdoc*/
getEnumerator(): IEnumerator<T>;
} /**
Represents an ordered or unordered collection of
{@link IModelItem}s that is part of a result of running a graph analysis algorithm.
Type parameter <code>T</code>: The type of items in this collection.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemCollection Online Documentation}*/
export class ResultItemCollection<T> {
/**
Determines whether a given <code>item</code> is contained in this collection.
Depending on the particular analysis algorithm result, this method may have a complexity of O(1) or O(n). As a general
rule, algorithms that return a result where containment is a useful check, will be O(1), e.g. {@link ReachabilityResult#reachableNodes}.
@param item The item to look for.
@returns <code>true</code> if <code>item</code> is contained in this collection, <code>false</code> if not.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemCollection%23contains(T) Online Documentation}*/
contains(item: T): boolean;
/**
Copies the contents of this collection to another collection.
@param collection The collection to add the items from this collection to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemCollection%23copyTo(ICollection%3CT%3E) Online Documentation}*/
copyTo(collection: ICollection<T> | ICollectionConvertible<T>): void;
static $class: Class;
private $_ResultItemCollection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is ResultItemCollection<T>;
}
export interface ResultItemMapping<TKey, TValue> extends YObject, IEnumerable<MapEntry<TKey, TValue>>, IMapper<TKey, TValue> {
/**
@inheritdoc*/
getEnumerator(): IEnumerator<MapEntry<TKey, TValue>>;
} /**
Represents a mapping from keys to values that is part of a result of running a graph analysis algorithm.
Type parameter <code>TKey</code>: The type of keys in this mapping, typically graph items.
Type parameter <code>TValue</code>: The type of the values.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemMapping Online Documentation}*/
export class ResultItemMapping<TKey, TValue> {
/**
Copies the entries of this mapping into the given <code>dictionary</code>.
@param dictionary The {@link IMap} to copy the entries of this mapping to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemMapping%23copyTo(IMap%3CTKey%2CTValue%3E) Online Documentation}*/
copyTo(dictionary: IMap<TKey, TValue>): void;
/**
Copies the entries of this mapping into the given <code>mapper</code>.
@param mapper The {@link IMapper} to copy the entries of this mapping to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemMapping%23copyTo(IMapper%3CTKey%2CTValue%3E) Online Documentation}*/
copyTo(mapper: IMapper<TKey, TValue> | IMapperConvertible<TKey, TValue>): void;
/**
Gets the value for a given key in the mapping.
This implementation is read-only.
@param key The key of the mapping.
@returns The value.
@throws {Exception} <b>(InvalidOperationError)</b> when trying to set a value.
@see ResultItemMapping#set
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemMapping%23get(TKey) Online Documentation}*/
get(key: TKey | null): TValue | null;
/**
Sets the value for a given key in the mapping.
This implementation is read-only.
@param key The key of the mapping.
@param value The value.
@throws {Exception} <b>(InvalidOperationError)</b> when trying to set a value.
@see ResultItemMapping#get
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemMapping%23set(TKey%2CTValue) Online Documentation}*/
set(key: TKey | null, value: TValue | null): void;
/**
Gets the number of elements in this mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemMapping%23size Online Documentation}*/
readonly size: number;
/**
Gets all keys in this mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemMapping%23keys Online Documentation}*/
readonly keys: IEnumerable<TKey>;
/**
Gets all values in this mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ResultItemMapping%23values Online Documentation}*/
readonly values: IEnumerable<TValue>;
static $class: Class;
private $_ResultItemMapping: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TKey, TValue>(o: any): o is ResultItemMapping<TKey, TValue>;
}
export interface ShortestPathResult extends YObject {
}
/**
Represents the shortest path between two nodes in a graph as computed by {@link ShortestPath}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathResult Online Documentation}*/
export class ShortestPathResult {
/**
Gets the shortest path found between {@link ShortestPath#source} and {@link ShortestPath#sink} if one exists.
If there is no path between the two nodes, this is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathResult%23path Online Documentation}*/
path: Path | null;
/**
Gets a collection of nodes along the path, starting with the {@link ShortestPath#source} node and ending with the {@link ShortestPath#sink} node.
If there is no path between the two nodes, this is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathResult%23nodes Online Documentation}*/
readonly nodes: ResultItemCollection<INode>;
/**
Gets a collection of edges along the path.
If there is no path between the two nodes (or {@link ShortestPath#source} and {@link ShortestPath#sink} are identical), this is empty.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathResult%23edges Online Documentation}*/
readonly edges: ResultItemCollection<IEdge>;
/**
* Gets the distance of the path.
* This is the sum of edge costs along the path. If there are no edge costs, the distance is the number of edges in the
* path.
*
* If there is no path between the two nodes, this is equal to {@link Number#POSITIVE_INFINITY}.
*
* <pre>
* algorithm.source = startNode
* algorithm.sink = endNode
* const result = algorithm.run(graph)
* console.log(result.distance) // the distance between startNode and endNode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathResult%23distance Online Documentation}
*/
readonly distance: number;
/**
* Gets a mapping from each node to the last incoming edge of the shortest path to this node.
* This is a common representation of a <em>shortest path tree</em> which exploits the fact that any shortest path in a graph has sub-paths that
* themselves are also shortest paths. A shortest path to a given node can be reconstructed backwards by querying this
* mapping for that node, following the returned edge to its source node, and continuing to query the mapping until the
* source node of the shortest-path search is reached.
*
* This mapping returns <code>null</code> for nodes that are not part of the found shortest path.
*
* Starting the target or sink node one can walk along the edges to the start node.
*
* <pre>
* // configure the shortest path algorithm
* const algorithm = new ShortestPath({
*   // single source - single sink
*   source: startNode,
*   sink: endNode,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
*
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the edge path
* const predResultsMap = result.predecessors
* let walker = endNode
* let edge = predResultsMap.get(walker)
* while (edge !== null) {
*   graph.setStyle(edge, highlightPathStyle)
*   walker = edge.opposite(walker)
*   edge = predResultsMap.get(walker)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPathResult%23predecessors Online Documentation}
*/
predecessors: ResultItemMapping<INode, IEdge>;
static $class: Class;
private $_ShortestPathResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ShortestPathResult;
}
export interface ShortestPath extends YObject {
}
/**
* Finds the shortest path between two nodes (also known as the <em>single-source single-sink shortest path</em> problem).
* The shortest path will be determined either by <em>Dijkstra's algorithm</em>, or an implementation of A*, depending on whether a {@link ShortestPath#heuristic} exists for
* estimating the remaining distance between a node and the {@link ShortestPath#sink} node.
*
* <b>Other Shortest Path Algorithms</b>
*
* yFiles for HTML supports a number of other algorithms that compute shortest paths in a graph:
*
* <ul>
* <li><b>{@link SingleSourceShortestPaths}</b> – finds shortest paths from a single source node to several other nodes</li>
* <li><b>{@link AllPairsShortestPaths}</b> – finds all shortest paths between pairs of several source and sink nodes</li>
* </ul>
* <b>Other Path-Related Algorithms</b>
*
* yFiles for HTML also supports a number of other algorithms related to paths in a graph:
*
* <ul>
* <li><b>{@link Paths}</b> – finds all paths between a set of source and a set of target nodes</li>
* <li><b>{@link Chains}</b> – finds all chains, that is, sequences of nodes that are each connected with just an edge without branches</li>
* <li><b>{@link Cycle}</b> – finds a cycle if one exists</li>
* <li><b>{@link LongestPath}</b> – finds the longest path in the graph</li>
* </ul>
* <br><b>Highlighting the shortest path edges between a start and an end node</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new ShortestPath({
*   // single source - single sink
*   source: startNode,
*   sink: endNode,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // highlight the edge path
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightPathStyle)
* }
* </pre>
*
* @see ShortestPathAlgorithm#singleSourceSingleSink
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath Online Documentation}
*/
export class ShortestPath {
/**
Creates a new {@link ShortestPath} instance.
@param [options] The parameters to pass.
@param [options.source] The source (start) node of the path. This option sets the {@link ShortestPath#source} property on the created object.
@param [options.sink] The sink (end, target) node of the path. This option sets the {@link ShortestPath#sink} property on the created object.
@param [options.costs] A mapping for the cost for traversing an edge. This option sets the {@link ShortestPath#costs} property on the created object.
@param [options.heuristic] A function that estimates the distance between two nodes. This option sets the {@link ShortestPath#heuristic} property on the created object.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link ShortestPath#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link ShortestPath#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link ShortestPath#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath Online Documentation}*/
constructor(options?: {
source?: SingleItem<INode> | SingleItemConvertible<INode>;
sink?: SingleItem<INode> | SingleItemConvertible<INode>;
costs?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
heuristic?: ((arg1: INode, arg2: INode) => number) | null;
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Finds the shortest path between {@link ShortestPath#source} and {@link ShortestPath#sink}.
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Highlighting the shortest path edges between a start and an end node</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new ShortestPath({
*   // single source - single sink
*   source: startNode,
*   sink: endNode,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // highlight the edge path
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightPathStyle)
* }
* </pre>
*
* @param graph The input graph to run the algorithm on.
* @returns A {@link ShortestPathResult} containing the shortest path between {@link ShortestPath#source} and {@link ShortestPath#sink} if one exists.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): ShortestPathResult;
/**
* Gets or sets the source (start) node of the path.
* <pre>
* // the item is the explicitly set item (the start node)
* algorithm = new ShortestPath({
*   source: startNode
* })
*
* // the item is the first item in the source (the first selected node)
* algorithm = new ShortestPath({
*   source: graphComponent.selection.selectedNodes
* })
*
* // the item is the first in the graph for which the predicate returns true
* // (the first node in the graph with "source" as tag)
* algorithm = new ShortestPath({
*   source: node => node.tag === 'source'
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath%23source Online Documentation}
*/
source: SingleItem<INode>;
/**
* Gets or sets the sink (end, target) node of the path.
* <pre>
* // the item is the explicitly set item (the end node)
* algorithm = new ShortestPath({
*   sink: endNode
* })
*
* // the item is the first item in the source (the first selected node)
* algorithm = new ShortestPath({
*   sink: graphComponent.selection.selectedNodes
* })
*
* // the item is the first in the graph for which the predicate returns true
* // (the first node in the graph with "sink" as tag)
* algorithm = new ShortestPath({
*   sink: node => node.tag === 'sink'
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath%23sink Online Documentation}
*/
sink: SingleItem<INode>;
/**
* Gets or sets a mapping for the cost for traversing an edge.
* For the shortest path algorithm this is a measure of the edge's length, so more expensive (higher cost) edges are
* considered longer and avoided if there's a shorter (cheaper) path elsewhere in the graph.
*
* The costs must not be negative.
*
* When no costs are provided, uniform costs of <code>1</code> are assumed for all edges. This makes the shortest path the path with the
* fewest edges.
*
* <br><b>Mapping a value to some edges</b>
* <pre>
* // setting the cost for traversing e1 and e2 to 1.0
* algorithm.costs.mapper.set(e1, 1.0)
* algorithm.costs.mapper.set(e2, 1.0)
* // all other edges are not set and return 0
* </pre>
*
* <br><b>Using a delegate to determine the cost</b>
* <pre>
* // returns the length of the actual edge path as cost
* algorithm.costs = edge =>
*   edge.style.renderer
*     .getPathGeometry(edge, edge.style)
*     .getPath()
*     .getLength()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath%23costs Online Documentation}
*/
costs: ItemMapping<IEdge, number>;
/**
Gets or sets a function that estimates the distance between two nodes.
This can be used to speed up the shortest path search by internally switching to an A* implementation which can skip
large parts of the graph that would lead further away from the target node.
A heuristic must be <em>admissible</em>, that is, it must never
<em>over</em>estimate the remaining distance to the target. Otherwise the found path may not be the shortest one.
The heuristic ties closely into the
{@link ShortestPath#costs}, as both should use the same, or similar measures. For example, if the costs are defined by the actual visual length of
an edge, the heuristic can be Euclidean distance between the two nodes because that's a useful estimate of the remaining
distance when the costs are defined that way.
Don't use a heuristic if there is no useful and quick way of <em>estimating</em> the remaining distance.
The second node passed to the function is always the {@link ShortestPath#sink} node as the remaining distance is only ever estimated towards the
sink node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath%23heuristic Online Documentation}*/
heuristic: ((arg1: INode | null, arg2: INode | null) => number) | null;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link ShortestPath#run} method.
*
* <br><b>Calculating shortest paths on a subset of the graph</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new ShortestPath({
*   // single source - single sink
*   source: startNode,
*   sink: endNode,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // highlight the edge path
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightPathStyle)
* }
* </pre>
*
* <br><b>Considering only selected nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes)
* </pre>
*
* <br><b>Ignoring all group nodes</b>
* <pre>
* const subset = new ItemCollection()
* subset.excludes = n => graph.isGroupNode(n)
* </pre>
*
* <br><b>Considering selected nodes but ignoring group nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes, n =>
*   graph.isGroupNode(n)
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link ShortestPath#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link ShortestPath#run} method.
*
* <br><b>Calculating shortest paths on a subset of the graph</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new ShortestPath({
*   // single source - single sink
*   source: startNode,
*   sink: endNode,
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // highlight the edge path
* for (const edge of result.edges) {
*   graph.setStyle(edge, highlightPathStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ShortestPath%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_ShortestPath: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ShortestPath;
}
export interface SingleSourceShortestPathsResult extends YObject {
}
/**
Represents all shortest paths between a single source node and multiple sink nodes nodes in a graph as computed by
{@link SingleSourceShortestPaths}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPathsResult Online Documentation}*/
export class SingleSourceShortestPathsResult {
/**
Gets the shortest path from the configured {@link SingleSourceShortestPaths#source source node} to the given <code>sink</code> node if one exists.
If <code>sink</code> was not part of the {@link SingleSourceShortestPaths#sinks} collection or no path between {@link SingleSourceShortestPaths#source} and <code>sink</code> exists, the return value is <code>null</code>.
@param sink The sink (target, end) node of the path to get.
@returns The shortest path between the source and the sink node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPathsResult%23getPathTo(INode) Online Documentation}*/
getPathTo(sink: INode): Path | null;
/**
* Gets a mapping from each node to the last incoming edge of the shortest path to this node.
* This is a common representation of a <em>shortest path tree</em> which exploits the fact that any shortest path in a graph has sub-paths that
* themselves are also shortest paths. A shortest path to a given node can be reconstructed backwards by querying this
* mapping for that node, following the returned edge to its source node, and continuing to query the mapping until the
* source node of the shortest-path search is reached.
*
* This mapping returns <code>null</code> for nodes that have no path from {@link SingleSourceShortestPaths#source}.
*
* Starting with a target (sink) node one can walk along the edges to the start node.
*
* <pre>
* // configure the shortest path algorithm
* const algorithm = new SingleSourceShortestPaths({
*   // single source
*   source: startNode,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
*
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the edge path
* const predResultsMap = result.predecessors
* let walker = endNode
* let edge = predResultsMap.get(walker)
* while (edge !== null) {
*   graph.setStyle(edge, highlightPathStyle)
*   walker = edge.opposite(walker)
*   edge = predResultsMap.get(walker)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPathsResult%23predecessors Online Documentation}
*/
predecessors: ResultItemMapping<INode, IEdge>;
/**
* Gets a mapping from each node to its shortest distance from the {@link SingleSourceShortestPaths#source source node}.
* Since every shortest path consists of other shortest paths, this mapping not only contains the shortest distances to the
* {@link SingleSourceShortestPaths#sinks sink nodes} but also all nodes on the paths to the sinks.
*
* The distance to a node is {@link Number#POSITIVE_INFINITY} if there is no path from {@link SingleSourceShortestPaths#source} to that node.
*
* <pre>
* algorithm.source = startNode
* algorithm.sinks = targetNodes
* const result = algorithm.run(graph)
* for (const node of targetNodes) {
*   graph.setLabelText(node.labels.get(0), `Distance: ${result.distances.get(node)}`)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPathsResult%23distances Online Documentation}
*/
distances: ResultItemMapping<INode, number>;
/**
Gets a collection of shortest paths between the {@link SingleSourceShortestPaths#source source node} and {@link SingleSourceShortestPaths#sinks sink nodes}.
This collection contains every existing shortest path between {@link SingleSourceShortestPaths#source} and the sink nodes. This means that there may well be
fewer paths than sinks if some of them are not reachable from the source node.
While the underlying algorithm always computes shortest paths to all nodes in the graph, this collection only contains
paths to the initially configured sink nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPathsResult%23paths Online Documentation}*/
paths: ResultItemCollection<Path>;
static $class: Class;
private $_SingleSourceShortestPathsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SingleSourceShortestPathsResult;
}
export interface SingleSourceShortestPaths extends YObject {
}
/**
* Finds the shortest path between one node and multiple other nodes in the graph (also known as the <em>single-source shortest path</em> problem).
* The shortest paths will be determined either by the <em>Bellman-Ford</em> algorithm or
* <em>Dijkstra's algorithm</em>, depending on whether or not edge costs are provided and the exact values of those edge costs.
*
* <b>Other Shortest Path Algorithms</b>
*
* yFiles for HTML supports a number of other algorithms that compute shortest paths in a graph:
*
* <ul>
* <li><b>{@link ShortestPath}</b> – finds the shortest path between two nodes</li>
* <li><b>{@link AllPairsShortestPaths}</b> – finds all shortest paths between pairs of several source and sink nodes</li>
* </ul>
* <b>Other Path-Related Algorithms</b>
*
* yFiles for HTML also supports a number of other algorithms related to paths in a graph:
*
* <ul>
* <li><b>{@link Paths}</b> – finds all paths between a set of source and a set of target nodes</li>
* <li><b>{@link Chains}</b> – finds all chains, that is, sequences of nodes that are each connected with just an edge without branches</li>
* <li><b>{@link Cycle}</b> – finds a cycle if one exists</li>
* <li><b>{@link LongestPath}</b> – finds the longest path in the graph</li>
* </ul>
* <br><b>Highlighting the shortes path edges between a start node and multiple end nodes</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new SingleSourceShortestPaths({
*   // single source
*   source: startNode,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
* // run the algorithm:
* // calculate paths from startNode to all nodes in the graph
* const result = algorithm.run(graph)
* // for each end node
* for (const targetNode of targetNodes) {
*   // set its distance to the startNode as label text
*   graph.setLabelText(targetNode.labels.get(0), `${result.getPathTo(targetNode).distance}`)
*   // and mark the edge path from start to the end node
*   for (const edge of result.getPathTo(targetNode).edges) {
*     graph.setStyle(edge, highlightPathStyle)
*   }
* }
* </pre>
*
* @see ShortestPathAlgorithm#singleSource
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPaths Online Documentation}
*/
export class SingleSourceShortestPaths {
/**
Creates a new {@link SingleSourceShortestPaths} instance.
@param [options] The parameters to pass.
@param [options.source] The source (start) node. This option sets the {@link SingleSourceShortestPaths#source} property on the created object.
@param [options.sinks] A collection of sink (end, target) nodes. This option sets the {@link SingleSourceShortestPaths#sinks} property on the created object.
@param [options.costs] A mapping for the cost for traversing an edge. This option sets the {@link SingleSourceShortestPaths#costs} property on the created object.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link SingleSourceShortestPaths#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link SingleSourceShortestPaths#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link SingleSourceShortestPaths#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPaths Online Documentation}*/
constructor(options?: {
source?: SingleItem<INode> | SingleItemConvertible<INode>;
sinks?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
costs?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Finds the shortest paths between {@link SingleSourceShortestPaths#source} and {@link SingleSourceShortestPaths#sinks}.
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Highlighting the shortes path edges between a start node and multiple end nodes</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new SingleSourceShortestPaths({
*   // single source
*   source: startNode,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
* // run the algorithm:
* // calculate paths from startNode to all nodes in the graph
* const result = algorithm.run(graph)
* // for each end node
* for (const targetNode of targetNodes) {
*   // set its distance to the startNode as label text
*   graph.setLabelText(targetNode.labels.get(0), `${result.getPathTo(targetNode).distance}`)
*   // and mark the edge path from start to the end node
*   for (const edge of result.getPathTo(targetNode).edges) {
*     graph.setStyle(edge, highlightPathStyle)
*   }
* }
* </pre>
*
* @param graph The input graph to run the algorithm on.
* @returns A {@link SingleSourceShortestPathsResult} containing the shortest paths between {@link SingleSourceShortestPaths#source} and {@link SingleSourceShortestPaths#sinks}.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPaths%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): SingleSourceShortestPathsResult;
/**
* Gets or sets the source (start) node.
* <pre>
* // the item is the explicitly set item (the start node)
* algorithm = new ShortestPath({
*   source: startNode
* })
*
* // the item is the first item in the source (the first selected node)
* algorithm = new ShortestPath({
*   source: graphComponent.selection.selectedNodes
* })
*
* // the item is the first in the graph for which the predicate returns true
* // (the first node in the graph with "source" as tag)
* algorithm = new SingleSourceShortestPaths({
*   source: node => node.tag === 'source'
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPaths%23source Online Documentation}
*/
source: SingleItem<INode>;
/**
* Gets or sets a collection of sink (end, target) nodes.
* The algorithm always finds paths from {@link SingleSourceShortestPaths#source} to all nodes. However, this collection affects which individual {@link SingleSourceShortestPathsResult#paths} are returned in
* the result.
*
* For a result that's typical of single-source shortest path algorithms the complete graph would be included here. If this
* collection is empty, no paths will be returned in the result.
*
* <pre>
* // the sink nodes are provided as collection (the selected nodes)
* algorithm = new SingleSourceShortestPaths({
*   sinks: graphComponent.selection.selectedNodes
* })
*
* // the sink nodes are the nodes in the graph for which the delegate returns true
* // (nodes in the graph with "sink" as tag)
* algorithm = new SingleSourceShortestPaths({
*   sinks: node => node.tag === 'sink'
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPaths%23sinks Online Documentation}
*/
sinks: ItemCollection<INode>;
/**
* Gets or sets a mapping for the cost for traversing an edge.
* For the shortest path algorithm this is a measure of the edge's length, so more expensive (higher cost) edges are
* considered longer and avoided if there's a shorter (cheaper) path elsewhere in the graph.
*
* When no costs are provided, uniform costs of <code>1</code> are assumed for all edges. This makes a shortest path the path with the
* fewest edges.
*
* Negative costs are supported only for {@link SingleSourceShortestPaths#directed} graphs and only if there is no cycle with negative costs. Negative costs are not
* supported for undirected graphs. {@link SingleSourceShortestPaths#run} will throw an {@link Exception InvalidOperationError} in
* these cases.
*
* <br><b>Mapping a value to some edges</b>
* <pre>
* // setting the cost for traversing e1 and e2 to 1.0
* algorithm.costs.mapper.set(e1, 1.0)
* algorithm.costs.mapper.set(e2, 1.0)
* // all other edges are not set and return 0
* </pre>
*
* <br><b>Using a delegate to determine the cost</b>
* <pre>
* // returns the length of the actual edge path as cost
* algorithm.costs = edge =>
*   edge.style.renderer
*     .getPathGeometry(edge, edge.style)
*     .getPath()
*     .getLength()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPaths%23costs Online Documentation}
*/
costs: ItemMapping<IEdge, number>;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPaths%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link SingleSourceShortestPaths#run}
* method.
*
* <br><b>Calculating shortest paths on a subset of the graph</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new SingleSourceShortestPaths({
*   // single source - single sink
*   source: startNode,
*   sinks: targetNodes,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // highlight the edge path
* for (const path of result.paths) {
*   for (const edge of path.edges) {
*     graph.setStyle(edge, highlightPathStyle)
*   }
* }
* </pre>
*
* <br><b>Considering only selected nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes)
* </pre>
*
* <br><b>Ignoring all group nodes</b>
* <pre>
* const subset = new ItemCollection()
* subset.excludes = n => graph.isGroupNode(n)
* </pre>
*
* <br><b>Considering selected nodes but ignoring group nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes, n =>
*   graph.isGroupNode(n)
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPaths%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link SingleSourceShortestPaths#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link SingleSourceShortestPaths#run}
* method.
*
* <br><b>Calculating shortest paths on a subset of the graph</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new SingleSourceShortestPaths({
*   // single source - single sink
*   source: startNode,
*   sinks: targetNodes,
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // highlight the edge path
* for (const path of result.paths) {
*   for (const edge of path.edges) {
*     graph.setStyle(edge, highlightPathStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleSourceShortestPaths%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_SingleSourceShortestPaths: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SingleSourceShortestPaths;
}
export interface AllPairsShortestPathsResult extends YObject {
}
/**
Represents all shortest paths between pairs of source and sink nodes as computed by
{@link AllPairsShortestPaths}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPathsResult Online Documentation}*/
export class AllPairsShortestPathsResult {
/**
Gets the length of the shortest path between <code>source</code> and <code>sink</code>.
If <code>source</code> was not in the collection of {@link AllPairsShortestPaths#sources source nodes} the result is {@link Number#POSITIVE_INFINITY}. This is also the result if
there is no path between <code>source</code> and <code>sink</code>.
@param source The path's source node.
@param sink The path's sink node.
@returns The distance between a given <code>source</code> node and a given <code>sink</code> node or {@link Number#POSITIVE_INFINITY} if there is no path between
them or <code>source</code> was not part of the {@link AllPairsShortestPaths#sources}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPathsResult%23getDistanceBetween(INode%2CINode) Online Documentation}*/
getDistanceBetween(source: INode, sink: INode): number;
/**
Gets the shortest path between a given <code>source</code> and <code>sink</code> node if one exists.
@param source The path's source node.
@param sink The path's sink node.
@returns The shortest path between a given <code>source</code> and <code>sink</code> node or <code>null</code> if there is no path between them or either <code>source</code> or <code>sink</code>
was not part of the {@link AllPairsShortestPaths#sources} or {@link AllPairsShortestPaths#sinks}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPathsResult%23getPathBetween(INode%2CINode) Online Documentation}*/
getPathBetween(source: INode, sink: INode): Path | null;
/**
* Gets a mapping from each node to the last incoming edge of the shortest path to <code>source</code>.
* This is a common representation of a <em>shortest path tree</em> which exploits the fact that any shortest path in a graph has sub-paths that
* themselves are also shortest paths. A shortest path to a given node can be reconstructed backwards by querying this
* mapping for that node, following the returned edge to its source node, and continuing to query the mapping until the
* source node of the shortest-path search is reached.
*
* This tree is rooted in the source node and since this result represents many paths between pairs of source and target
* nodes, this mapping can be retrieved for each source node.
*
* Starting with a target (sink) node one can walk along the edges to the source (start) node.
*
* <pre>
* // configure the shortest path algorithm
* const algorithm = new AllPairsShortestPaths({
*   // multiple source and sink nodes
*   sources: sourceNodes,
*   sinks: targetNodes,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
*
* // run the algorithm
* const result = algorithm.run(graph)
*
* for (const sourceNode of sourceNodes) {
*   // highlight the edge path
*   const predResultsMap = result.getPredecessorsForSource(sourceNode)
*   let walker = endNode
*   let edge = predResultsMap.get(walker)
*   while (edge !== null) {
*     graph.setStyle(edge, highlightPathStyle)
*     walker = edge.opposite(walker)
*     edge = predResultsMap.get(walker)
*   }
* }
* </pre>
*
* @param source The source node to get the predecessor mapping for.
* @returns A mapping from which shortest paths between <code>source</code> and other nodes can be reconstructed o <code>null</code> if <code>source</code> was not part of
* {@link AllPairsShortestPaths#sources}.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPathsResult%23getPredecessorsForSource(INode) Online Documentation}
*/
getPredecessorsForSource(source: INode): ResultItemMapping<INode, IEdge> | null;
/**
Gets a collection of shortest paths between the {@link AllPairsShortestPaths#sources source nodes} and {@link AllPairsShortestPaths#sinks sink nodes}.
This collection contains every existing shortest path between pairs of nodes from {@link AllPairsShortestPaths#sources} and {@link AllPairsShortestPaths#sinks}. Thus for <i>n</i> sources and <i>m</i> sinks
there may be up to <i>n</i> ⋅ <i>m</i> distinct paths. However, there may be fewer paths than that if some sink nodes are not reachable
from some source nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPathsResult%23paths Online Documentation}*/
paths: ResultItemCollection<Path>;
static $class: Class;
private $_AllPairsShortestPathsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AllPairsShortestPathsResult;
}
export interface AllPairsShortestPaths extends YObject {
}
/**
* Finds shortest paths between pairs of multiple {@link AllPairsShortestPaths#sources} and {@link AllPairsShortestPaths#sinks}.
* The shortest paths will be determined by invoking a {@link SingleSourceShortestPaths single-source shortest path search} for each node in {@link AllPairsShortestPaths#sources}. For <i>n</i> sources
* and <i>m</i> sinks the result thus may contain up to <i>n</i> ⋅ <i>m</i> distinct paths.
*
* <b>Other Shortest Path Algorithms</b>
*
* yFiles for HTML supports a number of other algorithms that compute shortest paths in a graph:
*
* <ul>
* <li><b>{@link ShortestPath}</b> – finds the shortest path between two nodes</li>
* <li><b>{@link SingleSourceShortestPaths}</b> – finds shortest paths from a single source node to several other nodes</li>
* </ul>
* <b>Other Path-Related Algorithms</b>
*
* yFiles for HTML also supports a number of other algorithms related to paths in a graph:
*
* <ul>
* <li><b>{@link Paths}</b> – finds all paths between a set of source and a set of target nodes</li>
* <li><b>{@link Chains}</b> – finds all chains, that is, sequences of nodes that are each connected with just an edge without branches</li>
* <li><b>{@link Cycle}</b> – finds a cycle if one exists</li>
* <li><b>{@link LongestPath}</b> – finds the longest path in the graph</li>
* </ul>
* <br><b>Highlighting the shortest path edges between multiple start and end nodes</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new AllPairsShortestPaths({
*   // multiple source and sink nodes
*   sources: sourceNodes,
*   sinks: targetNodes,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
* // run the algorithm:
* // calculate paths from startNode to all nodes in the graph
* const result = algorithm.run(graph)
* for (const sourceNode of sourceNodes) {
*   for (const targetNode of targetNodes) {
*     // and mark the edge path from start to the end node
*     for (const edge of result.getPathBetween(sourceNode, targetNode).edges) {
*       graph.setStyle(edge, highlightPathStyle)
*     }
*   }
* }
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPaths Online Documentation}
*/
export class AllPairsShortestPaths {
/**
Creates a new {@link AllPairsShortestPaths} instance.
@param [options] The parameters to pass.
@param [options.sources] A collection of source (start) nodes. This option sets the {@link AllPairsShortestPaths#sources} property on the created object.
@param [options.sinks] A collection of sink (end, target) nodes. This option sets the {@link AllPairsShortestPaths#sinks} property on the created object.
@param [options.costs] A mapping for the cost for traversing an edge. This option sets the {@link AllPairsShortestPaths#costs} property on the created object.
@param [options.directed] A value indicating whether edge direction should be considered. This option sets the {@link AllPairsShortestPaths#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link AllPairsShortestPaths#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link AllPairsShortestPaths#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPaths Online Documentation}*/
constructor(options?: {
sources?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
sinks?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
costs?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Finds all shortest paths between pairs of nodes from {@link AllPairsShortestPaths#sources} and {@link AllPairsShortestPaths#sinks}.
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Highlighting the shortes path edges between multiple start and end nodes</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new AllPairsShortestPaths({
*   // multiple source and sink nodes
*   sources: sourceNodes,
*   sinks: targetNodes,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
* // run the algorithm:
* // calculate paths from startNode to all nodes in the graph
* const result = algorithm.run(graph)
* for (const sourceNode of sourceNodes) {
*   for (const targetNode of targetNodes) {
*     // and mark the edge path from start to the end node
*     for (const edge of result.getPathBetween(sourceNode, targetNode).edges) {
*       graph.setStyle(edge, highlightPathStyle)
*     }
*   }
* }
* </pre>
*
* @param graph
* @returns {@link AllPairsShortestPathsResult} containing all shortest paths between nodes from {@link AllPairsShortestPaths#sources} and {@link AllPairsShortestPaths#sinks}.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPaths%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): AllPairsShortestPathsResult;
/**
* Gets or sets a collection of source (start) nodes.
* The algorithm finds the shortest paths from all source nodes to all sink nodes. If either {@link AllPairsShortestPaths#sources} or {@link AllPairsShortestPaths#sinks} is empty, the result will
* contain no paths.
*
* <pre>
* // the source nodes are provided as collection (the selected nodes)
* algorithm = new AllPairsShortestPaths({
*   sources: graphComponent.selection.selectedNodes
* })
*
* // the source nodes are the nodes in the graph for which the delegate returns true
* // (nodes in the graph with "source" as tag)
* algorithm = new AllPairsShortestPaths({
*   sources: node => node.tag === 'source'
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPaths%23sources Online Documentation}
*/
sources: ItemCollection<INode>;
/**
* Gets or sets a collection of sink (end, target) nodes.
* The algorithm finds the shortest paths from all source nodes to all sink nodes. If either {@link AllPairsShortestPaths#sources} or {@link AllPairsShortestPaths#sinks} is empty, the result will
* contain no paths.
*
* <pre>
* // the sink nodes are provided as collection (the selected nodes)
* algorithm = new AllPairsShortestPaths({
*   sinks: graphComponent.selection.selectedNodes
* })
*
* // the sink nodes are the nodes in the graph for which the delegate returns true
* // (nodes in the graph with "sink" as tag)
* algorithm = new AllPairsShortestPaths({
*   sinks: node => node.tag === 'sink'
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPaths%23sinks Online Documentation}
*/
sinks: ItemCollection<INode>;
/**
* Gets or sets a mapping for the cost for traversing an edge.
* For the shortest path algorithm this is a measure of the edge's length, so more expensive (higher cost) edges are
* considered longer and avoided if there's a shorter (cheaper) path elsewhere in the graph.
*
* When no costs are provided, uniform costs of <code>1</code> are assumed for all edges. This makes a shortest path the path with the
* fewest edges.
*
* Negative costs are supported only for {@link AllPairsShortestPaths#directed} graphs and only if there is no cycle with negative costs. Negative costs are not
* supported for undirected graphs. {@link AllPairsShortestPaths#run} will throw an {@link Exception InvalidOperationError} in these
* cases.
*
* <br><b>Mapping a value to some edges</b>
* <pre>
* // setting the cost for traversing e1 and e2 to 1.0
* algorithm.costs.mapper.set(e1, 1.0)
* algorithm.costs.mapper.set(e2, 1.0)
* // all other edges are not set and return 0
* </pre>
*
* <br><b>Using a delegate to determine the cost</b>
* <pre>
* // returns the length of the actual edge path as cost
* algorithm.costs = edge =>
*   edge.style.renderer
*     .getPathGeometry(edge, edge.style)
*     .getPath()
*     .getLength()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPaths%23costs Online Documentation}
*/
costs: ItemMapping<IEdge, number>;
/**
Gets or sets a value indicating whether edge direction should be considered.
Default is <code>true</code>.
@default true
@see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPaths%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link AllPairsShortestPaths#run}
* method.
*
* <br><b>Calculating shortest paths on a subset of the graph</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new AllPairsShortestPaths({
*   // single source - single sink
*   sources: sourceNodes,
*   sinks: targetNodes,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // highlight the edge path
* for (const path of result.paths) {
*   for (const edge of path.edges) {
*     graph.setStyle(edge, highlightPathStyle)
*   }
* }
* </pre>
*
* <br><b>Considering only selected nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes)
* </pre>
*
* <br><b>Ignoring all group nodes</b>
* <pre>
* const subset = new ItemCollection()
* subset.excludes = n => graph.isGroupNode(n)
* </pre>
*
* <br><b>Considering selected nodes but ignoring group nodes</b>
* <pre>
* const subset = new ItemCollection(graphComponent.selection.selectedNodes, n =>
*   graph.isGroupNode(n)
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPaths%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link AllPairsShortestPaths#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link AllPairsShortestPaths#run}
* method.
*
* <br><b>Calculating shortest paths on a subset of the graph</b>
* <pre>
* // configure the shortest path algorithm
* const algorithm = new AllPairsShortestPaths({
*   // single source - single sink
*   sources: sourceNodes,
*   sinks: targetNodes,
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // highlight the edge path
* for (const path of result.paths) {
*   for (const edge of path.edges) {
*     graph.setStyle(edge, highlightPathStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/AllPairsShortestPaths%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_AllPairsShortestPaths: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AllPairsShortestPaths;
}
export interface Path extends YObject {
}
/**
Represents a path between two nodes.
The path is defined by an ordered list of {@link Path#edges}. The {@link Path#nodes} along the path are also provided, but may be ambiguous if there are
multiple edges between certain nodes.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/Path Online Documentation}*/
export class Path {
/**
* Gets an ordered collection of edges defining this path.
* <pre>
* // configure the shortest path algorithm
* const algorithm = new SingleSourceShortestPaths({
*   // single source
*   source: startNode,
*   // add edge cost mapping which returns the actual length of the edge
*   costs: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* })
* // run the algorithm:
* // calculate paths from startNode to all nodes in the graph
* const result = algorithm.run(graph)
* // for each end node
* for (const targetNode of targetNodes) {
*   // set its distance to the startNode as label text
*   graph.setLabelText(targetNode.labels.get(0), `${result.getPathTo(targetNode).distance}`)
*   // and mark the edge path from start to the end node
*   for (const edge of result.getPathTo(targetNode).edges) {
*     graph.setStyle(edge, highlightPathStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Path%23edges Online Documentation}
*/
edges: ResultItemCollection<IEdge>;
/**
Gets a collection of nodes along this path.
This starts with {@link Path#start} and ends with {@link Path#end} by following the edges of {@link Path#edges}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Path%23nodes Online Documentation}*/
nodes: ResultItemCollection<INode>;
/**
* Gets the distance or length of the path.
* Some algorithms support edge weights (or costs), which makes the path's distance the sum of all edge weights along the
* path. If no explicit weights are provided, usually a uniform weight of <code>1</code> is assumed, which makes the path's distance
* merely the number of edges that comprise it.
*
* <br><b>Single-source single-sink</b>
* <pre>
* algorithm.source = startNode
* algorithm.sink = endNode
* const result = algorithm.run(graph)
* console.log(result.distance) // the distance between startNode and endNode
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Path%23distance Online Documentation}
*/
distance: number;
/**
Gets the start node of the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Path%23start Online Documentation}*/
start: INode;
/**
Gets the end node of the path.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Path%23end Online Documentation}*/
end: INode;
static $class: Class;
private $_Path: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Path;
}
export interface SingleItem<TItem> extends YObject {
}
/**
* Represents a single item of type <code>TItem</code>.
* Depending on which option is most convenient there are several ways to provide that item:
*
* <ul>
* <li><b>{@link SingleItem#item}</b> – to directly provide a single item that's already known</li>
* <li><b>{@link SingleItem#source}</b> – to provide an {@link IEnumerable} from which the first item is used. This is evaluated when running the algorithm and not before.</li>
* <li><b>{@link SingleItem#delegate}</b> – to provide a predicate that checks whether an item should be used. The first matching one is used.</li>
* </ul>
* Only a single property should ever be set. When setting multiple properties to a non-<code>null</code> value, some may be ignored and
* take precedence over others.
*
* Functions, lists and constant values are automatically converted to {@link SingleItem}s. For example:
*
* <pre>
* // Item
* new ShortestPath({ source: node1 })
*
* // Predicate
* new shortestPath({ source: node => isSource(node) })
*
* // Source
* new ShortestPath({ source: graphComponent.selection.selectedNodes })
* </pre>
*
* Furthermore, the conversion supports a parameter object to initialize the properties. For example:
*
* <pre>
* // Create a new SingleItem from a JavaScript Map and initialize its 'item' with the given element.
* // This also works for the different properties as well.
* const singleItem = SingleItem.from({
*   item: graph.nodes.first()
* })
*
* // Or, instead of providing the specific property, the 'includes' key can be used
* const singleItem = SingleItem.from({
*   includes: graph.nodes.first()
* })
* </pre>
*
* Type parameter <code>TItem</code>: The type of the single item.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SingleItem Online Documentation}
*/
export class SingleItem<TItem> {
/**
Constructor overload that directly sets the {@link SingleItem#delegate} that describes which item is contained in this instance.
@param delegate A delegate that implements the mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleItem%23SingleItem(function(TItem)%3Aboolean) Online Documentation}*/
constructor(delegate: (obj: TItem) => boolean);
/**
Constructor overload that directly sets the single {@link SingleItem#item}.
@param item The single item contained in this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleItem%23SingleItem(TItem) Online Documentation}*/
constructor(item: TItem);
/**
Constructor overload that directly sets the {@link SingleItem#source} that lists the item contained in this instance.
@param source An enumerable that lists the single item in this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleItem%23SingleItem(IEnumerable%3CTItem%3E) Online Documentation}*/
constructor(source: IEnumerable<TItem> | IEnumerableConvertible<TItem>);
/**
Default constructor that does not explicitly initialize any of the {@link SingleItem#delegate}, {@link SingleItem#item} or {@link SingleItem#source} properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleItem%23SingleItem() Online Documentation}*/
constructor();
/**
Creates a SingleItem instance from the given object by performing automatic type conversion.
Type parameter <code>TItem</code>: The type of the single item.
@param singleItemLike The object to convert to a SingleItem.
@returns Either <code>singleItemLike</code> if it is already a SingleItem or a new instance, initialized to /// the values found in
<code>singleItemLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleItem%23from(SingleItem%3CTItem%3E) Online Documentation}*/
static from<TItem>(singleItemLike: SingleItem<TItem> | SingleItemConvertible<TItem>): SingleItem<TItem>;
/**
Gets or sets an item.
This is the exact item that will be used for the layout or graph analysis algorithm.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleItem%23item Online Documentation}*/
item: TItem | null;
/**
Gets or sets a predicate that defines the item to use.
The first item matching the predicate will be used for the layout or graph analysis algorithm.
This predicate will be evaluated anew for each successive invocation of the layout or graph analysis algorithm, making
it possible to return a different item each time.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleItem%23delegate Online Documentation}*/
delegate: ((obj: TItem) => boolean) | null;
/**
An {@link IEnumerable} of items, the first of which is used.
The first item from the {@link IEnumerable} will be used for the layout or graph analysis algorithm.
This {@link IEnumerable} will be evaluated anew for each successive invocation of the layout or graph analysis algorithm, making it
possible to return a different item each time.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SingleItem%23source Online Documentation}*/
source: IEnumerable<TItem> | null;
static $class: Class;
private $_SingleItem: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TItem>(o: any): o is SingleItem<TItem>;
}
/** Automatic conversion to {@link SingleItem}. */
type SingleItemConvertible<TItem> = IEnumerable<TItem> | ((item: TItem) => boolean) | TItem | {
includes: any;
} | {
item: TItem;
} | {
delegate: (item: TItem) => boolean;
} | {
source: Object;
};
export interface SpanningTreeResult extends YObject {
}
/**
Represents the edges of a minimum spanning tree as calculated by {@link SpanningTree}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTreeResult Online Documentation}*/
export class SpanningTreeResult {
/**
Gets a collection of edges that form the spanning tree or forest.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTreeResult%23edges Online Documentation}*/
edges: ResultItemCollection<IEdge>;
/**
Gets the total cost for the calculated minimum spanning tree or forest.
If there were no edge costs configured, each edge is assumed to have a uniform cost of
<code>1</code>, which then makes this equal to the number of edges.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTreeResult%23totalCost Online Documentation}*/
totalCost: number;
static $class: Class;
private $_SpanningTreeResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SpanningTreeResult;
}
export interface SpanningTree extends YObject {
}
/**
* Calculates a minimum spanning tree or forest for a graph.
* <b>Definitions</b>
*
* <ul>
* <li>A <b>spanning tree</b> of an undirected connected graph is a subset of its edges that induce a tree that connects all nodes of the graph.</li>
* <li>A <b>minimum spanning tree</b> of a weighted connected graph is a spanning tree whose edges have minimum overall cost among all spanning trees of
* that graph.</li>
* </ul>
* If the graph is not connected, the result is a (minimum) spanning <i>forest</i> instead, whose components are spanning trees.
*
* <b>Other Tree-Related Algorithms</b>
*
* yFiles for HTML supports a number of other algorithms and helpers related to trees:
*
* <ul>
* <li><b>{@link FeedbackEdgeSet}</b> – finds edges that can be removed or reversed to make a graph into a tree</li>
* <li><b>{@link TreeAnalyzer}</b> – provides utility methods for analyzing directed trees</li>
* </ul>
* <br><b>Calculating a spanning tree of the graph</b>
* <pre>
* // prepare the spanning tree detection algorithm
* const algorithm = new SpanningTree()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Remove the edges in the reduction
* for (const edge of result.edges) {
*   graph.setStyle(edge, spanningTreeEdgeStyle)
* }
* </pre>
*
* @see SpanningTreeAlgorithm#uniform
* @see SpanningTreeAlgorithm#minimum
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTree Online Documentation}
*/
export class SpanningTree {
/**
Creates a new {@link SpanningTree} instance.
@param [options] The parameters to pass.
@param [options.costs] A mapping for edge costs. This option sets the {@link SpanningTree#costs} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link SpanningTree#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link SpanningTree#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTree Online Documentation}*/
constructor(options?: {
costs?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Calculates a minimum spanning tree or forest for the given graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link SpanningTreeResult} containing the edges that make up the spanning tree (or forest).
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTree%23run(IGraph) Online Documentation}*/
run(graph: IGraph): SpanningTreeResult;
/**
Gets or sets a mapping for edge costs.
When specifying costs the algorithm will minimize the total cost of edges that form the spanning tree.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTree%23costs Online Documentation}*/
costs: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link SpanningTree#run} method.
*
* <br><b>Calculating a spanning tree of a subset of the graph</b>
* <pre>
* // prepare the spanning tree detection algorithm
* const algorithm = new SpanningTree({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Remove the edges in the reduction
* for (const edge of result.edges) {
*   graph.setStyle(edge, spanningTreeEdgeStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTree%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link SpanningTree#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link SpanningTree#run} method.
*
* <br><b>Calculating a spanning tree of a subset of the graph</b>
* <pre>
* // prepare the spanning tree detection algorithm
* const algorithm = new SpanningTree({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Remove the edges in the reduction
* for (const edge of result.edges) {
*   graph.setStyle(edge, spanningTreeEdgeStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SpanningTree%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_SpanningTree: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SpanningTree;
}
export interface StarSubstructuresResult extends YObject {
}
/**
Represents the stars found by {@link StarSubstructures}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructuresResult Online Documentation}*/
export class StarSubstructuresResult {
/**
Gets the stars found by {@link StarSubstructures}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructuresResult%23stars Online Documentation}*/
stars: ResultItemCollection<SubstructureItems>;
static $class: Class;
private $_StarSubstructuresResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StarSubstructuresResult;
}
export interface StarSubstructures extends YObject {
}
/**
* Detects {@link SubstructureItems}s that represent isolated stars in the specified graph.
* A <em>star</em> consists of a <code>root</code> that is connected to multiple nodes with degree one.
*
* Since a star only consists of elements with the same {@link StarSubstructures#edgeDirectedness} and
* {@link StarSubstructures#nodeTypes}, a root may be associated with different stars. In this case, the algorithm only returns the largest star.
*
* The {@link StarSubstructures#edgeDirectedness} is considered as follows: A substructure is only identified as such if all edges are either undirected or
* consistently directed with respect to the specified directedness.
*
* <ul>
* <li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.</li>
* <li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.</li>
* <li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.</li>
* </ul>
* <h2>Note</h2>
*
* The smallest {@link StarSubstructures#minimumSize} value that could be considered is <code>2</code>.
*
* <h2>Note</h2>
*
* A chain or star structure is also considered to be a tree.
*
* <h2>Note</h2>
*
* If no {@link StarSubstructures#edgeDirectedness} is specified, all edges are treated as undirected. Furthermore, if no {@link StarSubstructures#nodeTypes} are specified, all node are considered to
* be of the same type.
*
* <br><b>Finding the stars in a graph</b>
* <pre>
* // prepare the star detection algorithm
* const algorithm = new StarSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the stars
* for (const star of result.stars) {
*   for (const node of star.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of star.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the stars in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the star detection algorithm
* const algorithm = new StarSubstructures({
*   // only nodes with the same tags can be a star
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the stars
* for (const star of result.stars) {
*   for (const node of star.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of star.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructures Online Documentation}
*/
export class StarSubstructures {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.minimumSize] The minimum size of a star. This option sets the {@link StarSubstructures#minimumSize} property on the created object.
@param [options.nodeTypes] A mapping which maps the type of each node. This option sets the {@link StarSubstructures#nodeTypes} property on the created object.
@param [options.edgeDirectedness] A mapping that stores the directedness of the edges. This option sets the {@link StarSubstructures#edgeDirectedness} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link StarSubstructures#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link StarSubstructures#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructures Online Documentation}*/
constructor(options?: {
minimumSize?: number;
nodeTypes?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Returns a list of {@link SubstructureItems}s that represent isolated stars in the specified graph.
* A <em>star</em> consists of a <code>root</code> that is connected to multiple nodes with degree one.
*
* Since a star only consists of elements with the same {@link StarSubstructures#edgeDirectedness} and
* {@link StarSubstructures#nodeTypes}, a root may be associated with different stars. In this case, the algorithm only returns the largest star.
*
* A star only consists of elements with the same {@link StarSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link StarSubstructures#nodeTypes} is specified, all
* node are considered to be of the same type.
*
* The {@link StarSubstructures#edgeDirectedness} is considered as follows: A substructure is only identified as such if all edges are either undirected or
* consistently directed with respect to the specified directedness.
*
* <ul>
* <li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.</li>
* <li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.</li>
* <li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.</li>
* </ul>
* <h2>Note</h2>
*
* The smallest {@link StarSubstructures#minimumSize} value that could be considered is <code>2</code>.
*
* <h2>Note</h2>
*
* A chain or star structure is also considered to be a tree.
*
* <h2>Note</h2>
*
* If no {@link StarSubstructures#edgeDirectedness} is specified, all edges are treated as undirected. Furthermore, if no {@link StarSubstructures#nodeTypes} are specified, all node are considered to
* be of the same type.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Finding the stars in a graph</b>
* <pre>
* // prepare the star detection algorithm
* const algorithm = new StarSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the stars
* for (const star of result.stars) {
*   for (const node of star.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of star.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the stars in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the star detection algorithm
* const algorithm = new StarSubstructures({
*   // only nodes with the same tags can be a star
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the stars
* for (const star of result.stars) {
*   for (const node of star.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of star.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @param graph The graph to find the substructures in.
* @returns A list of {@link SubstructureItems}s that represent the stars.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructures%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): StarSubstructuresResult;
/**
Gets or sets the minimum size of a star.
Stars with less nodes are ignored.
Default is <code>2</code>.
<h2>Note</h2>
The smallest {@link StarSubstructures#minimumSize} value that could be considered is <code>2</code>.
@default 2
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructures%23minimumSize Online Documentation}*/
minimumSize: number;
/**
Gets or sets a mapping which maps the type of each node.
An arbitrary object. Nodes returning equal objects are considered to be of the same type.
If none is provided all nodes are considered as equal.
<h2>Note</h2>
A star only consists of elements with the same {@link StarSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link StarSubstructures#nodeTypes} is specified, all
node are considered to be of the same type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructures%23nodeTypes Online Documentation}*/
nodeTypes: ItemMapping<INode, any>;
/**
Gets or sets a mapping that stores the directedness of the edges.
If a value is provided here, the algorithm only considers a substructure if all edges are either undirected or
consistently directed with respect to the specified directedness.
<ul>
<li>the edge's directedness is greater than <code>0.0</code> and the edge is incoming (i.e. n is the edge's target),
<li>the edge's directedness is smaller than <code>0.0</code> and the edge is outgoing (i.e. n is the edge's source),
<li>or the edge's directedness is <code>0.0</code> (i.e. the edge is considered to be <b>undirected</b>).
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructures%23edgeDirectedness Online Documentation}*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link StarSubstructures#run} method.
*
* <br><b>Finding the stars of a subset of the graph</b>
* <pre>
* // prepare the star detection algorithm
* const algorithm = new StarSubstructures({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the stars
* for (const star of result.stars) {
*   for (const node of star.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of star.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructures%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link StarSubstructures#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link StarSubstructures#run} method.
*
* <br><b>Finding the stars of a subset of the graph</b>
* <pre>
* // prepare the star detection algorithm
* const algorithm = new StarSubstructures({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the stars
* for (const star of result.stars) {
*   for (const node of star.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of star.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/StarSubstructures%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_StarSubstructures: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StarSubstructures;
}
export interface StronglyConnectedComponentsResult extends YObject {
}
/**
Represents the strongly connected components of a graph.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponentsResult Online Documentation}*/
export class StronglyConnectedComponentsResult {
/**
Gets a map which returns for each node the ID of the component the node belongs to.
If the node is not part of an optional {@link ConnectedComponents#subgraphNodes}, the return value is -1. Otherwise, the ID of a component is a non-negative {@link number}
value which is common for all nodes of the same component.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponentsResult%23nodeComponentIds Online Documentation}*/
nodeComponentIds: ResultItemMapping<INode, number>;
/**
Gets a map which returns for each node the {@link Component component} the node belongs to.
If the node does not belong to {@link ConnectedComponents#subgraphNodes}, the return value is <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponentsResult%23nodeComponents Online Documentation}*/
nodeComponents: ResultItemMapping<INode, Component>;
/**
Gets a collection of the components in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponentsResult%23components Online Documentation}*/
components: ResultItemCollection<Component>;
/**
Gets a collections of the edges which connect nodes in different components.
The edges between nodes in the same component can be obtained from that {@link Component}'s {@link Component#inducedEdges} collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponentsResult%23interComponentEdges Online Documentation}*/
interComponentEdges: ResultItemCollection<IEdge>;
static $class: Class;
private $_StronglyConnectedComponentsResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StronglyConnectedComponentsResult;
}
export interface StronglyConnectedComponents extends YObject {
}
/**
* Determines the strongly connected components of a given graph.
* A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
*
* The strongly connected components of a graph are the strongly connected subgraphs of which it consists.
*
* <b>Other Graph Connectivity Algorithms</b>
*
* yFiles for HTML supports a number of other analysis algorithms that partition the graph into components, based on
* various criteria.
*
* <ul>
* <li><b>{@link ConnectedComponents}</b> – Determines components defined by the existence of an undirected path between nodes</li>
* <li><b>{@link BiconnectedComponents}</b> – Determines components defined by the existence of at least two separate undirected paths between all nodes</li>
* <li><b>{@link Bipartition}</b> – Divides a graph into two partitions where all edges have their source and target in different partitions</li>
* <li><b>{@link IndependentSets}</b> – Divides a graph into partitions where no nodes are connected within a partition</li>
* <li><b>{@link KCoreComponents}</b> – Calculates the k-cores of an undirected input graph. The k-core of an undirected input graph consists of the subgraph
* components where each node has at least degree <code>k</code>.</li>
* </ul>
* <br><b>Calculating strongly connected components on a graph</b>
* <pre>
* // prepare the strongly connected components algorithm
* const algorithm = new StronglyConnectedComponents()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the connected components with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeComponentIds.get(node)
*   graph.setStyle(node, componentStyles.get(componentId))
* }
* </pre>
*
* @see GraphConnectivity#stronglyConnectedComponents
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponents Online Documentation}
*/
export class StronglyConnectedComponents {
/**
Creates a new {@link StronglyConnectedComponents} instance.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link StronglyConnectedComponents#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link StronglyConnectedComponents#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponents Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Determines the strongly connected components of a given graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link StronglyConnectedComponentsResult} containing the strongly connected components of <code>graph</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponents%23run(IGraph) Online Documentation}*/
run(graph: IGraph): StronglyConnectedComponentsResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link StronglyConnectedComponents#run}
* method.
*
* <br><b>Calculating strongly connected components on a subset of the graph</b>
* <pre>
* // configure the strongly connected components algorithm
* const algorithm = new StronglyConnectedComponents({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the connected components with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeComponentIds.get(node)
*   graph.setStyle(node, componentStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponents%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link StronglyConnectedComponents#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link StronglyConnectedComponents#run}
* method.
*
* <br><b>Calculating strongly connected components on a subset of the graph</b>
* <pre>
* // configure the strongly connected components algorithm
* const algorithm = new StronglyConnectedComponents({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the nodes of the connected components with different styles
* for (const node of graph.nodes) {
*   const componentId = result.nodeComponentIds.get(node)
*   graph.setStyle(node, componentStyles.get(componentId))
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/StronglyConnectedComponents%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_StronglyConnectedComponents: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StronglyConnectedComponents;
}
export interface TransitiveClosureResult extends YObject {
}
/**
Represents the edges that have to be added to obtain the transitive closure of a graph as computed by
{@link TransitiveClosure}.
Creating edges between the {@link TransitiveEdge#source} node and the {@link TransitiveEdge#target} node will make the graph transitive.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveClosureResult Online Documentation}*/
export class TransitiveClosureResult {
/**
Gets a collection of edge placeholders.
Inserting edges based on these placeholders into the graph will make it transitive.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveClosureResult%23edgesToAdd Online Documentation}*/
edgesToAdd: ResultItemCollection<TransitiveEdge>;
static $class: Class;
private $_TransitiveClosureResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TransitiveClosureResult;
}
export interface TransitiveEdge extends YObject {
}
/**
Represents a placeholder for an edge that does not yet exist in a graph.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdge Online Documentation}*/
export class TransitiveEdge {
/**
The source node of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdge%23source Online Documentation}*/
source: INode;
/**
The target node of the edge.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdge%23target Online Documentation}*/
target: INode;
static $class: Class;
private $_TransitiveEdge: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TransitiveEdge;
}
export interface TransitiveClosure extends YObject {
}
/**
* Calculates the transitive closure for a directed acyclic graph.
* Let <i>G</i> = (<i>V</i>, <i>E</i>) be a directed acyclic graph. The <em>transitive closure</em> of <i>G</i> is a graph which contains an edge (<i>v</i>, <i>w</i>) (with <i>v</i> ≠ <i>w</i>) only if there exists
* a path from <i>v</i> to <i>w</i> in <i>G</i>. This implementation produces the transitive closure and not the <em>reflexive</em>, transitive closure of the
* specified graph, since no representations for self-loops are added. <b>Other Transitivity Algorithms</b>
*
* yFiles for HTML supports other algorithms related to transitivity:
*
* <ul>
* <li><b>{@link TransitiveEdges}</b> – calculates the edges which connect the visible nodes in a graph if these are indirectly connected via hidden nodes.</li>
* <li><b>{@link TransitiveReduction}</b> – calculates the transitive reduction of a graph, i.e. the edges that are unnecessary to only ensure that the same
* reachability relation is represented</li>
* </ul>
* <br><b>Calculating the transitive closure of a graph</b>
* <pre>
* // prepare the transitive closure algorithm
* const algorithm = new TransitiveClosure()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Add the edges in the closure with a different style
* for (const edge of result.edgesToAdd) {
*   graph.createEdge(edge.source, edge.target, transitiveEdgeStyle)
* }
* </pre>
*
* @see TransitivityAlgorithm#transitiveClosure
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveClosure Online Documentation}
*/
export class TransitiveClosure {
/**
Creates a new {@link TransitiveClosure} instance.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link TransitiveClosure#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link TransitiveClosure#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveClosure Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Calculates the transitive closure for a directed acyclic graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link TransitiveClosureResult} containing placeholders for edges that can be inserted to obtain the
transitive closure of <code>graph</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveClosure%23run(IGraph) Online Documentation}*/
run(graph: IGraph): TransitiveClosureResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link TransitiveClosure#run} method.
*
* <br><b>Calculating the transitive closure of a subset of the graph</b>
* <pre>
* // prepare the transitive closure algorithm
* const algorithm = new TransitiveClosure({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Add the edges in the closure with a different style
* for (const edge of result.edgesToAdd) {
*   graph.createEdge(edge.source, edge.target, transitiveEdgeStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveClosure%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link TransitiveClosure#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link TransitiveClosure#run} method.
*
* <br><b>Calculating the transitive closure of a subset of the graph</b>
* <pre>
* // prepare the transitive closure algorithm
* const algorithm = new TransitiveClosure({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Add the edges in the closure with a different style
* for (const edge of result.edgesToAdd) {
*   graph.createEdge(edge.source, edge.target, transitiveEdgeStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveClosure%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_TransitiveClosure: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TransitiveClosure;
}
export interface TransitiveEdgesResult extends YObject {
}
/**
Represents the edges that have to be added to obtain the transitive edges of a graph as computed by
{@link TransitiveEdges}.
Creating edges between the {@link TransitiveEdge#source} node and the {@link TransitiveEdge#target} node will keep the {@link TransitiveEdges#visibleNodes} connected after the non-visible nodes have been removed.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdgesResult Online Documentation}*/
export class TransitiveEdgesResult {
/**
Gets a collection of edge placeholders.
Inserting edges based on these placeholders into the graph will keep the {@link TransitiveEdges#visibleNodes} connected after the non-visible nodes have
been removed.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdgesResult%23edgesToAdd Online Documentation}*/
edgesToAdd: ResultItemCollection<TransitiveEdge>;
static $class: Class;
private $_TransitiveEdgesResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TransitiveEdgesResult;
}
export interface TransitiveEdges extends YObject {
}
/**
* Calculates the transitive edges that connect the {@link TransitiveEdges#visibleNodes visible nodes} in the specified graph.
* The algorithm suggests a <code>transitive edge</code> between two visible nodes if:
*
* <ul>
* <li>there is a (directed/undirected) path between the nodes using only invisible nodes as intermediate nodes</li>
* <li>and there is not already an edge between the two nodes</li>
* </ul>
* yFiles for HTML supports other algorithms related to transitivity:
*
* <ul>
* <li><b>{@link TransitiveClosure}</b> – calculates the transitive closure of a graph, i.e. the edges that would have to be added that the set of edges defines
* the reachability relation in the graph</li>
* <li><b>{@link TransitiveReduction}</b> – calculates the transitive reduction of a graph, i.e. the edges that are unnecessary to only ensure that the same
* reachability relation is represented</li>
* </ul>
* Other than {@link TransitiveClosure}, this algorithm only suggests edges for nodes which are indirectly
* connected via nodes which are hidden, i.e. not {@link TransitiveEdges#visibleNodes visible}.
*
* This can be useful to simplify graphs where unimportant nodes are removed but the (indirect) connections between the
* important nodes should be kept:
*
* <br><b>Calculating the transitive edges between the selected nodes.</b>
* <pre>
* // calculate the transitive edges between the selected nodes
* const algorithm = new TransitiveEdges({
*   visibleNodes: graphComponent.selection.selectedNodes
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add the edges
* for (const edge of result.edgesToAdd) {
*   graph.createEdge(edge.source, edge.target, edgeStyle)
* }
* </pre>
*
* @see TransitivityAlgorithm#transitiveEdges
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdges Online Documentation}
*/
export class TransitiveEdges {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.visibleNodes] A collection of nodes to start from. This option sets the {@link TransitiveEdges#visibleNodes} property on the created object.
@param [options.directed] A boolean that specifies whether the graph is directed. This option sets the {@link TransitiveEdges#directed} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link TransitiveEdges#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link TransitiveEdges#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdges Online Documentation}*/
constructor(options?: {
visibleNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
directed?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Calculates the transitive edges that connect the {@link TransitiveEdges#visibleNodes visible nodes} in the specified graph.
* The algorithm suggests a <code>transitive edge</code> between two visible nodes if:
*
* <ul>
* <li>there is a (directed/undirected) path between the nodes using only invisible nodes as intermediate nodes</li>
* <li>and there is not already an edge between the two nodes</li>
* </ul>
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Calculating the transitive edges between the selected nodes.</b>
* <pre>
* // calculate the transitive edges between the selected nodes
* const algorithm = new TransitiveEdges({
*   visibleNodes: graphComponent.selection.selectedNodes
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add the edges
* for (const edge of result.edgesToAdd) {
*   graph.createEdge(edge.source, edge.target, edgeStyle)
* }
* </pre>
*
* @param graph the original graph that contains both the visible and invisible nodes
* @returns A {@link TransitiveEdgesResult} containing placeholders for edges that can be inserted to obtain the transitive
* closure of <code>graph</code>.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdges%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): TransitiveEdgesResult;
/**
Gets or sets a collection of nodes to start from.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdges%23visibleNodes Online Documentation}*/
visibleNodes: ItemCollection<INode>;
/**
Gets or sets a boolean that specifies whether the graph is directed.
Default is <code>false</code>.
@default false
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdges%23directed Online Documentation}*/
directed: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link TransitiveEdges#run} method.
*
* <br><b>Calculating the transitive edges of a subset of the graph</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new TransitiveEdges({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add the edges
* for (const edge of result.edgesToAdd) {
*   graph.createEdge(edge.source, edge.target, edgeStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdges%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link TransitiveEdges#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link TransitiveEdges#run} method.
*
* <br><b>Calculating the transitive edges of a subset of the graph</b>
* <pre>
* // prepare the chain detection algorithm
* const algorithm = new TransitiveEdges({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // add the edges
* for (const edge of result.edgesToAdd) {
*   graph.createEdge(edge.source, edge.target, edgeStyle)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveEdges%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_TransitiveEdges: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TransitiveEdges;
}
export interface TransitiveReductionResult extends YObject {
}
/**
Represents the transitive edges of a graph as computed by {@link TransitiveReduction}.
Removing those edges will turn the graph into its transitive reduction.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveReductionResult Online Documentation}*/
export class TransitiveReductionResult {
/**
Gets a collection of the transitive edges in the graph.
Removing those edges will turn the graph into its transitive reduction.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveReductionResult%23edgesToRemove Online Documentation}*/
edgesToRemove: ResultItemCollection<IEdge>;
static $class: Class;
private $_TransitiveReductionResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TransitiveReductionResult;
}
export interface TransitiveReduction extends YObject {
}
/**
* Calculates the transitive reduction for a directed acyclic graph.
* Given <i>G</i> = (<i>V</i>, <i>E</i>) be a directed acyclic graph. The <i>transitive reduction</i> of <i>G</i> is a graph which contains an edge (<i>v</i>, <i>w</i>) only if no path exists from <i>v</i>
* to <i>w</i> in <i>G</i> of length 2 or more.
*
* <b>Other Transitivity Algorithms</b>
*
* yFiles for HTML supports other algorithms related to transitivity:
*
* <ul>
* <li><b>{@link TransitiveClosure}</b> – calculates the transitive closure of a graph, i.e. the edges that would have to be added that the set of edges defines
* the reachability relation in the graph</li>
* <li><b>{@link TransitiveEdges}</b> – calculates the edges which connect the visible nodes in a graph if these are indirectly connected via hidden nodes.</li>
* </ul>
* <b>Warning:</b> This implementation is costly in terms of memory, since a (<i>n</i> × <i>n</i>) matrix is allocated to store reach data.
*
* <br><b>Calculating the transitive reduction of a graph</b>
* <pre>
* // prepare the transitive reduction algorithm
* const algorithm = new TransitiveReduction()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Remove the edges in the reduction
* for (const edge of result.edgesToRemove) {
*   graph.remove(edge)
* }
* </pre>
*
* @see TransitivityAlgorithm#transitiveReduction
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveReduction Online Documentation}
*/
export class TransitiveReduction {
/**
Creates a new {@link TransitiveReduction} instance.
@param [options] The parameters to pass.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link TransitiveReduction#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link TransitiveReduction#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveReduction Online Documentation}*/
constructor(options?: {
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Calculates the transitive reduction for a directed acyclic graph.
<b>Warning:</b> This implementation is costly in terms of memory, since a (<i>n</i> × <i>n</i>) matrix is allocated to store reach data.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link TransitiveReductionResult} containing the transitive edges of the given graph. Removing those edges
results in the transitive reduction of <code>graph</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveReduction%23run(IGraph) Online Documentation}*/
run(graph: IGraph): TransitiveReductionResult;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link TransitiveReduction#run}
* method.
*
* <br><b>Calculating the transitive reduction of a subset of the graph</b>
* <pre>
* // prepare the transitive reduction algorithm
* const algorithm = new TransitiveReduction({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Remove the edges in the reduction
* for (const edge of result.edgesToRemove) {
*   graph.remove(edge)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveReduction%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link TransitiveReduction#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link TransitiveReduction#run}
* method.
*
* <br><b>Calculating the transitive reduction of a subset of the graph</b>
* <pre>
* // prepare the transitive reduction algorithm
* const algorithm = new TransitiveReduction({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // Remove the edges in the reduction
* for (const edge of result.edgesToRemove) {
*   graph.remove(edge)
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TransitiveReduction%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_TransitiveReduction: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TransitiveReduction;
}
export interface TreeAnalyzer extends YObject {
}
/**
Provides methods for querying and analyzing directed trees.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer Online Documentation}*/
export class TreeAnalyzer {
/**
Initializes a new instance of the {@link TreeAnalyzer} class with the given graph.
@param graph The graph to use for subsequent queries.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer Online Documentation}*/
constructor(graph: IGraph);
/**
Gets the child nodes of the given node in the tree.
@param node The parent node to get children from.
@returns An {@link IEnumerable} of <code>node</code>'s children.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer%23getChildren(INode) Online Documentation}*/
getChildren(node: INode): IListEnumerable<INode>;
/**
Gets the depth of the node, that is, the length of the path between the {@link TreeAnalyzer#getRoot root} and <code>node</code>.
Nodes with the same depth are commonly drawn in layers in typical tree visualizations.
@param node The node to determine the depth for.
@returns The distance between the root node and <code>node</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If the graph is not a tree.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer%23getDepth(INode) Online Documentation}*/
getDepth(node: INode): number;
/**
Gets all leaf nodes of the tree.
Leaf nodes are nodes without outgoing edges.
@returns An {@link IEnumerable} of leaf nodes in the tree.
@throws {Exception} <b>(InvalidOperationError)</b> If the graph is not a tree.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer%23getLeafNodes() Online Documentation}*/
getLeafNodes(): IListEnumerable<INode>;
/**
Determines the nearest common ancestor for the given nodes.
Note that each node is considered to be its own ancestor so the returned node may be one of <code>nodes</code>. Another way to look
at this is to picture the result to be the root of the smallest sub-tree that contains all <code>nodes</code>.
@param nodes The nodes for which to find the nearest common ancestor.
@returns The nearest ancestor of all given nodes or <code>null</code> if <code>nodes</code> is empty.
@throws {Exception} <b>(InvalidOperationError)</b> If the graph is not a tree.
@throws {Exception} <b>(ArgumentError)</b> If <code>nodes</code> contain <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer%23getNearestCommonAncestor(INode) Online Documentation}*/
getNearestCommonAncestor(...nodes: INode[]): INode | null;
/**
Determines the nearest common ancestor for the given nodes.
Note that each node is considered to be its own ancestor so the returned node may be one of <code>nodes</code>. Another way to look
at this is to picture the result to be the root of the smallest sub-tree that contains all <code>nodes</code>.
@param nodes The nodes for which to find the nearest common ancestor.
@returns The nearest ancestor of all given nodes or <code>null</code> if <code>nodes</code> is empty.
@throws {Exception} <b>(InvalidOperationError)</b> If the nodes are not in a subtree of a common ancestor.
@throws {Exception} <b>(ArgumentError)</b> If <code>nodes</code> contain <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer%23getNearestCommonAncestor(IEnumerable%3CINode%3E) Online Documentation}*/
getNearestCommonAncestor(nodes: IEnumerable<INode> | IEnumerableConvertible<INode>): INode | null;
/**
Gets the parent of the given node in the tree.
@param node The child node to determine the parent of.
@returns The parent node of <code>node</code> or <code>null</code> if <code>node</code> is the root of the tree.
@throws {Exception} <b>(InvalidOperationError)</b> If the graph is not a tree.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer%23getParent(INode) Online Documentation}*/
getParent(node: INode): INode | null;
/**
Determines the root node of the tree.
The root node is the single node that has no incoming edges.
@returns The root node for the tree or <code>null</code> if the graph is empty.
@throws {Exception} <b>(InvalidOperationError)</b> If the graph is not a tree.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer%23getRoot() Online Documentation}*/
getRoot(): INode | null;
/**
Gets all nodes of the sub-tree that has <code>subtreeRoot</code> as root.
@param subtreeRoot The sub-tree root node.
@returns An {@link IEnumerable} of nodes in the sub-tree of <code>subtreeRoot</code>.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeAnalyzer%23getSubtree(INode) Online Documentation}*/
getSubtree(subtreeRoot: INode): IListEnumerable<INode>;
static $class: Class;
private $_TreeAnalyzer: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeAnalyzer;
}
export interface TreeSubstructuresResult extends YObject {
}
/**
Represents the trees found by {@link TreeSubstructures}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructuresResult Online Documentation}*/
export class TreeSubstructuresResult {
/**
Gets the trees found by {@link TreeSubstructures}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructuresResult%23trees Online Documentation}*/
trees: ResultItemCollection<SubstructureItems>;
static $class: Class;
private $_TreeSubstructuresResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeSubstructuresResult;
}
export interface TreeSubstructures extends YObject {
}
/**
* Detects {@link SubstructureItems}s that represent isolated trees in the specified graph.
* The <code>root</code> of a tree is the only node which may have non-tree edges. Furthermore, a tree only consists of elements with
* the same {@link TreeSubstructures#nodeTypes}.
*
* The {@link TreeSubstructures#edgeDirectedness} is considered as follows: A substructure is only identified as such if all edges are either undirected or
* consistently directed with respect to the specified directedness.
*
* <ul>
* <li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.</li>
* <li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.</li>
* <li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.</li>
* </ul>
* <h2>Note</h2>
*
* The smallest {@link TreeSubstructures#minimumSize} value that could be considered is <code>3</code>.
*
* <h2>Note</h2>
*
* A chain or star structure is also considered to be a tree.
*
* <h2>Note</h2>
*
* If no {@link TreeSubstructures#edgeDirectedness} is specified, all edges are treated as undirected. Furthermore, if no {@link TreeSubstructures#nodeTypes} are specified, all node are considered to
* be of the same type.
*
* <br><b>Finding the trees in a graph</b>
* <pre>
* // prepare the tree detection algorithm
* const algorithm = new TreeSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the trees
* for (const tree of result.trees) {
*   for (const node of tree.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of tree.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the trees in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the tree detection algorithm
* const algorithm = new TreeSubstructures({
*   // only nodes with the same tags can be a tree
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the trees
* for (const tree of result.trees) {
*   for (const node of tree.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of tree.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructures Online Documentation}
*/
export class TreeSubstructures {
/**
Creates a new instance of this class.
@param [options] The parameters to pass.
@param [options.minimumSize] The minimum size of a tree. This option sets the {@link TreeSubstructures#minimumSize} property on the created object.
@param [options.nodeTypes] A mapping which maps the type of each node. This option sets the {@link TreeSubstructures#nodeTypes} property on the created object.
@param [options.edgeDirectedness] A mapping that stores the directedness of the edges. This option sets the {@link TreeSubstructures#edgeDirectedness} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link TreeSubstructures#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link TreeSubstructures#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructures Online Documentation}*/
constructor(options?: {
minimumSize?: number;
nodeTypes?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Returns a list of {@link SubstructureItems}s that represent isolated trees in the specified graph.
* The <code>root</code> of a tree is the only node which may have non-tree edges. Furthermore, a tree only consists of elements with
* the same {@link TreeSubstructures#nodeTypes}.
*
* The {@link TreeSubstructures#edgeDirectedness} is considered as follows: A substructure is only identified as such if all edges are either undirected or
* consistently directed with respect to the specified directedness.
*
* <ul>
* <li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.</li>
* <li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.</li>
* <li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.</li>
* </ul>
* <h2>Note</h2>
*
* The smallest {@link TreeSubstructures#minimumSize} value that could be considered is <code>3</code>.
*
* <h2>Note</h2>
*
* A chain or star structure is also considered to be a tree.
*
* <h2>Note</h2>
*
* If no {@link TreeSubstructures#edgeDirectedness} is specified, all edges are treated as undirected. Furthermore, if no {@link TreeSubstructures#nodeTypes} are specified, all node are considered to
* be of the same type.
*
* <b>Warning:</b>
* The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
* adding or removing nodes or edges.
*
* <br><b>Finding the trees in a graph</b>
* <pre>
* // prepare the tree detection algorithm
* const algorithm = new TreeSubstructures()
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the trees
* for (const tree of result.trees) {
*   for (const node of tree.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of tree.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* <br><b>Finding only the trees in a graph whose nodes are of the same type</b>
* <pre>
* // prepare the tree detection algorithm
* const algorithm = new TreeSubstructures({
*   // only nodes with the same tags can be a tree
*   nodeTypes: node => node.tag
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the trees
* for (const tree of result.trees) {
*   for (const node of tree.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of tree.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @param graph The graph to find the substructures in.
* @returns A list of {@link SubstructureItems}s that represent the trees.
* @throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructures%23run(IGraph) Online Documentation}
*/
run(graph: IGraph): TreeSubstructuresResult;
/**
Gets or sets the minimum size of a tree.
Trees with less nodes are ignored.
Default is <code>3</code>.
<h2>Note</h2>
The smallest {@link TreeSubstructures#minimumSize} value that could be considered is <code>3</code>.
@default 3
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructures%23minimumSize Online Documentation}*/
minimumSize: number;
/**
Gets or sets a mapping which maps the type of each node.
An arbitrary object. Nodes returning equal objects are considered to be of the same type.
If none is provided all nodes are considered as equal.
<h2>Note</h2>
A tree only consists of elements with the same {@link TreeSubstructures#nodeTypes}, i.e., nodes associated with equal objects. If no {@link TreeSubstructures#nodeTypes} is specified, all
node are considered to be of the same type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructures%23nodeTypes Online Documentation}*/
nodeTypes: ItemMapping<INode, any>;
/**
Gets or sets a mapping that stores the directedness of the edges.
If a value is provided here, the algorithm only considers a substructure if all edges are either undirected or
consistently directed with respect to the specified directedness.
<ul>
<li>the edge's directedness is greater than <code>0.0</code> and the edge is incoming (i.e. n is the edge's target),
<li>the edge's directedness is smaller than <code>0.0</code> and the edge is outgoing (i.e. n is the edge's source),
<li>or the edge's directedness is <code>0.0</code> (i.e. the edge is considered to be <b>undirected</b>).
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructures%23edgeDirectedness Online Documentation}*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link TreeSubstructures#run} method.
*
* <br><b>Finding the trees of a subset of the graph</b>
* <pre>
* // prepare the tree detection algorithm
* const algorithm = new TreeSubstructures({
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the trees
* for (const tree of result.trees) {
*   for (const node of tree.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of tree.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructures%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link TreeSubstructures#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link TreeSubstructures#run} method.
*
* <br><b>Finding the trees of a subset of the graph</b>
* <pre>
* // prepare the tree detection algorithm
* const algorithm = new TreeSubstructures({
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
*
* // highlight the trees
* for (const tree of result.trees) {
*   for (const node of tree.nodes) {
*     graph.setStyle(node, highlightNodeStyle)
*   }
*   for (const edge of tree.edges) {
*     graph.setStyle(edge, highlightEdgeStyle)
*   }
* }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeSubstructures%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_TreeSubstructures: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeSubstructures;
}
export interface WeightCentralityResult extends YObject {
}
/**
Represents the node centrality values as computed by {@link WeightCentrality}.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentralityResult Online Documentation}*/
export class WeightCentralityResult {
/**
Gets a mapping from each node to its absolute centrality value.
The absolute centrality value can change drastically depending on graph structure and edge weights. The {@link WeightCentralityResult#normalizedNodeCentrality} values don't
usually change as much, while still allowing to determine relative importance between nodes.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentralityResult%23nodeCentrality Online Documentation}*/
nodeCentrality: ResultItemMapping<INode, number>;
/**
Gets a mapping from each node to its normalized centrality value.
The normalized value is obtained by dividing {@link WeightCentralityResult#nodeCentrality} by {@link WeightCentralityResult#maximumNodeCentrality}. Thus the normalized values are all between <code>0</code> and <code>1</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentralityResult%23normalizedNodeCentrality Online Documentation}*/
normalizedNodeCentrality: ResultItemMapping<INode, number>;
/**
Gets the minimum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentralityResult%23minimumNodeCentrality Online Documentation}*/
minimumNodeCentrality: number;
/**
Gets the maximum absolute node centrality value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentralityResult%23maximumNodeCentrality Online Documentation}*/
maximumNodeCentrality: number;
static $class: Class;
private $_WeightCentralityResult: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WeightCentralityResult;
}
export interface WeightCentrality extends YObject {
}
/**
* Computes the weight centrality for the nodes of a graph.
* <i>Weight centrality</i> measures the weight associated with incoming, outgoing, or all edges of a node.
*
* When no weights are associated with edges, weight centrality becomes the same as {@link DegreeCentrality}.
*
* <b>Other Centrality Measures</b>
*
* yFiles for HTML supports a number of other centrality measures:
*
* <ul>
* <li><b>{@link GraphCentrality}, {@link ClosenessCentrality}</b> – emphasize nodes that have short paths to other nodes</li>
* <li><b>{@link DegreeCentrality}</b> – emphasizes nodes with many edges</li>
* <li><b>{@link BetweennessCentrality}</b> – emphasizes nodes and edges that are part of many short paths</li>
* <li><b>{@link EigenvectorCentrality}</b> – computes the influence a node has on a network. The centrality value is higher if more nodes are connected to that node</li>
* <li><b>{@link PageRank}</b> – computes page rank values for all nodes based on their attached edges</li>
* </ul>
* <pre>
* const result = new WeightCentrality({
*   considerIncomingEdges: false,
*   considerOutgoingEdges: true,
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength()
* }).run(graph)
*
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(key, String(value))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see CentralityAlgorithm#weightCentrality
* @see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentrality Online Documentation}
*/
export class WeightCentrality {
/**
Creates a new {@link WeightCentrality} instance.
@param [options] The parameters to pass.
@param [options.weights] A mapping for edge weights. This option sets the {@link WeightCentrality#weights} property on the created object.
@param [options.considerIncomingEdges] A value indicating whether incoming edges contribute to the centrality values of their target nodes. This option sets the {@link WeightCentrality#considerIncomingEdges} property on the created object.
@param [options.considerOutgoingEdges] A value indicating whether outgoing edges contribute to the centrality values of their source nodes. This option sets the {@link WeightCentrality#considerOutgoingEdges} property on the created object.
@param [options.subgraphNodes] The collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on. This option sets the {@link WeightCentrality#subgraphNodes} property on the created object.
@param [options.subgraphEdges] The collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on. This option sets the {@link WeightCentrality#subgraphEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentrality Online Documentation}*/
constructor(options?: {
weights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
considerIncomingEdges?: boolean;
considerOutgoingEdges?: boolean;
subgraphNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
subgraphEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Computes the weight centrality for the nodes of a graph.
<b>Warning:</b>
The result obtained from this algorithm is a snapshot which is no longer valid once the graph has changed, e.g. by
adding or removing nodes or edges.
@param graph The input graph to run the algorithm on.
@returns A {@link WeightCentralityResult} from which the calculated centrality values can be obtained.
@throws {Exception} <b>(InvalidOperationError)</b> If the algorithm can't create a valid result due to an invalid graph structure or wrongly configured properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentrality%23run(IGraph) Online Documentation}*/
run(graph: IGraph): WeightCentralityResult;
/**
Gets or sets a mapping for edge weights.
Weight centrality sums the weights of adjacent edges on each node. Edge weights thus influence the centrality measure.
If no weights are provided, all edges have the same uniform weight of <code>1</code> which reduces the measure to the number of
adjacent edges. This is then the same as {@link DegreeCentrality}.
Edge weights for weight centrality must be positive.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentrality%23weights Online Documentation}*/
weights: ItemMapping<IEdge, number>;
/**
Gets or sets a value indicating whether incoming edges contribute to the centrality values of their target nodes.
If both {@link WeightCentrality#considerIncomingEdges} and {@link WeightCentrality#considerOutgoingEdges} are <code>false</code> all nodes will have the centrality <code>0</code>.
Default is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentrality%23considerIncomingEdges Online Documentation}*/
considerIncomingEdges: boolean;
/**
Gets or sets a value indicating whether outgoing edges contribute to the centrality values of their source nodes.
If both {@link WeightCentrality#considerIncomingEdges} and {@link WeightCentrality#considerOutgoingEdges} are <code>false</code> all nodes will have the centrality <code>0</code>.
Default is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentrality%23considerOutgoingEdges Online Documentation}*/
considerOutgoingEdges: boolean;
/**
* Gets or sets the collection of {@link INode nodes} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all nodes of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all nodes in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The nodes provided here must be part of the graph which is passed to the {@link WeightCentrality#run} method.
*
* <br><b>Calculating the weight centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new WeightCentrality({
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength(),
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(key, String(value))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentrality%23subgraphNodes Online Documentation}
*/
subgraphNodes: ItemCollection<INode>;
/**
* Gets or sets the collection of {@link IEdge edges} which define a subset of the graph for the algorithms to work on.
* If nothing is set, all edges of the graph will be processed.
*
* If only the {@link ItemCollection#excludes} are set all edges in the graph except those provided in the {@link ItemCollection#excludes} are processed.
*
* Note that edges which start or end at nodes which are not in the {@link WeightCentrality#subgraphNodes} are automatically not considered by the algorithm.
*
* {@link ItemCollection} instances may be shared among algorithm instances and will be (re-)evaluated upon (re-)execution of the algorithm.
*
* <b>Warning:</b> The edges provided here must be part of the graph which is passed to the {@link WeightCentrality#run} method.
*
* <br><b>Calculating the weight centrality on a subset of the graph</b>
* <pre>
* // configure the algorithm
* const algorithm = new WeightCentrality({
*   // Use the geometric edge length as weight
*   weights: edge =>
*     edge.style.renderer
*       .getPathGeometry(edge, edge.style)
*       .getPath()
*       .getLength(),
*   // Ignore edges without target arrow heads
*   subgraphEdges: {
*     excludes: edge =>
*       edge.style instanceof PolylineEdgeStyle && edge.style.targetArrow === IArrow.NONE
*   }
* })
* // run the algorithm
* const result = algorithm.run(graph)
* // add node labels for centrality values
* // and adjust node size according to centrality
* result.normalizedNodeCentrality.forEach(({ key, value }) => {
*   const node = key
*   const centrality = value
*   graph.addLabel(key, String(value))
*   graph.setNodeLayout(node, new Rect(node.layout.center, new Size(centrality, centrality)))
* })
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/WeightCentrality%23subgraphEdges Online Documentation}
*/
subgraphEdges: ItemCollection<IEdge>;
static $class: Class;
private $_WeightCentrality: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is WeightCentrality;
}
export interface AspectRatioTreeLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link AspectRatioTreeLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayoutData Online Documentation}*/
export class AspectRatioTreeLayoutData {
/**
Creates a new instance of {@link AspectRatioTreeLayoutData} which helps configuring
{@link AspectRatioTreeLayout}.
@param [options] The parameters to pass.
@param [options.subtreeAspectRatios] The mapping from nodes to the aspect ratio of their sub tree. This option sets the {@link AspectRatioTreeLayoutData#subtreeAspectRatios} property on the created object.
@param [options.rootPlacements] The mapping from nodes with sub trees to their placements. This option sets the {@link AspectRatioTreeLayoutData#rootPlacements} property on the created object.
@param [options.subtreeRoutingPolicies] The mapping from nodes to the routing policies of their sub trees. This option sets the {@link AspectRatioTreeLayoutData#subtreeRoutingPolicies} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayoutData Online Documentation}*/
constructor(options?: {
subtreeAspectRatios?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
rootPlacements?: ItemMapping<INode, RootPlacement> | ItemMappingConvertible<INode, RootPlacement>;
subtreeRoutingPolicies?: ItemMapping<INode, SubtreeArrangement> | ItemMappingConvertible<INode, SubtreeArrangement>;
});
/**
Gets or sets the mapping from nodes to the aspect ratio of their sub tree.
The aspect ratio needs to be greater than <code>0</code>.
<ul>
<li>aspect ratio <code>1</code>: width and height of the layout should be the same
<li>aspect ratio between <code>0</code> and <code>1</code>: the height of the layout should be greater than its width
<li>aspect ratio greater than <code>1</code>: the width of the layout should be greater than its height
</ul>
If no specific ratio is defined for a subtree, the layout algorithm falls back to the default {@link AspectRatioTreeLayout#aspectRatio aspect ratio}.
@see AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayoutData%23subtreeAspectRatios Online Documentation}*/
subtreeAspectRatios: ItemMapping<INode, number>;
/**
Gets or sets the mapping from nodes with sub trees to their placements.
If no root placement is specified for one of the subtree roots, the default {@link AspectRatioTreeLayout#rootPlacement root placement} will be used.
@see AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
@see AspectRatioTreeLayout#getRootPlacement
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayoutData%23rootPlacements Online Documentation}*/
rootPlacements: ItemMapping<INode, RootPlacement>;
/**
Gets or sets the mapping from nodes to the routing policies of their sub trees.
The children in a subtree are arranged either horizontally or vertically. The edges are routed to the top of the child
nodes or at the side, respectively. Directions depend on the {@link MultiStageLayout#layoutOrientation layout orientation} and refer to {@link LayoutOrientation#TOP_TO_BOTTOM}
for this description.
If no specific routing policy is specified for a subtree root, the layout algorithm uses the default {@link AspectRatioTreeLayout#subtreeArrangement routing direction}.
@see AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY
@see AspectRatioTreeLayout#getSubtreeArrangement
@see {@link https://docs.yworks.com/yfileshtml/#/api/AspectRatioTreeLayoutData%23subtreeRoutingPolicies Online Documentation}*/
subtreeRoutingPolicies: ItemMapping<INode, SubtreeArrangement>;
static $class: Class;
private $_AspectRatioTreeLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AspectRatioTreeLayoutData;
}
export interface BalloonLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link BalloonLayout}.
* The following example shows how to create a new instance of {@link BalloonLayoutData} and use it with an
* {@link BalloonLayout}:
*
* <br><b>Creating an instance of BalloonLayoutData</b>
* <pre>
* const layoutData = new BalloonLayoutData()
* // Use the node whose label says "Root" as root node for the tree
* layoutData.treeRoot.delegate = node => node.labels.get(0).text === 'Root'
* // Place children of nodes with many children interleaved
* layoutData.interleavedNodes.delegate = node => graphComponent.graph.outDegree(node) > 5
*
* graphComponent.graph.applyLayout(new BalloonLayout(), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link BalloonLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of BalloonLayoutData</b>
* <pre>
* const layoutData = new BalloonLayoutData({
*   // Use the node whose label says "Root" as root node for the tree
*   treeRoot: node => node.labels.get(0).text === 'Root',
*   // Place children of nodes with many children interleaved
*   interleavedNodes: node => graphComponent.graph.outDegree(node) > 5
* })
*
* graphComponent.graph.applyLayout(new BalloonLayout(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutData Online Documentation}
*/
export class BalloonLayoutData {
/**
Creates a new instance of {@link BalloonLayoutData} which helps configuring {@link BalloonLayout}.
@param [options] The parameters to pass.
@param [options.interleavedNodes] The collection of nodes whose children should be arranged in an interleaved fashion. This option sets the {@link BalloonLayoutData#interleavedNodes} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link BalloonLayoutData#nodeHalos} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link BalloonLayoutData#abortHandler} property on the created object.
@param [options.treeRoot] The mapping for marking the node that will be used as root node of the tree. This option sets the {@link BalloonLayoutData#treeRoot} property on the created object.
@param [options.edgeLabelPreferredPlacement] The mapping that provides a {@link PreferredPlacementDescriptor} instance for edge {@link ILabel}s. This option sets the {@link BalloonLayoutData#edgeLabelPreferredPlacement} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutData Online Documentation}*/
constructor(options?: {
interleavedNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
abortHandler?: AbortHandler | null;
treeRoot?: SingleItem<INode> | SingleItemConvertible<INode>;
edgeLabelPreferredPlacement?: ItemMapping<ILabel, PreferredPlacementDescriptor> | ItemMappingConvertible<ILabel, PreferredPlacementDescriptor>;
});
/**
* Gets or sets the collection of nodes whose children should be arranged in an interleaved fashion.
* This can be used to place children of nodes with many children in a more compact fashion.
*
* <b>Warning:</b> The custom node selection induced by this collection is only considered if the layout's {@link BalloonLayout#interleavedMode} is set to
* {@link InterleavedMode#MARKED_NODES}.
*
* If you already have a collection or {@link IEnumerable} containing the nodes whose children should be drawn interleaved, the easiest way is
* to set the {@link ItemCollection#source}:
*
* <br><b>Configuring the nodes for interleaved child placement via a collection or IEnumerable</b>
* <pre>
* // Place children of selected nodes interleaved
* layoutData.interleavedNodes = graphComponent.selection.selectedNodes
* </pre>
*
* If only a few nodes should be configured that way that are not yet in a collection, it's usually easier to use {@link ItemCollection#items}
* directly:
*
* <br><b>Configuring interleaved child placement for only certain selected nodes</b>
* <pre>
* layoutData.interleavedNodes.items.add(node1)
* layoutData.interleavedNodes.items.add(node2)
* </pre>
*
* If the criteria for which nodes should have their children placed interleaved are readily apparent from each node, then
* the best option is usually to use the {@link ItemCollection#delegate} property:
*
* <br><b>Using a delegate to configure interleaved child placement for each node individually</b>
* <pre>
* // Place children of nodes with many children interleaved
* layoutData.interleavedNodes = node => graph.outDegree(node) > 5
* </pre>
*
* @see BalloonLayout#INTERLEAVED_NODES_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutData%23interleavedNodes Online Documentation}
*/
interleavedNodes: ItemCollection<INode>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
* Gets or sets the mapping for marking the node that will be used as root node of the tree.
* The custom root node selection induced by this property is only considered if the {@link BalloonLayout#rootNodePolicy root node policy} is set to
* {@link RootNodePolicy#SELECTED_ROOT}.
*
* The simplest way to set a custom root node for the layout, if you already have an {@link INode} for that, would be
* to use the {@link SingleItem#item} to specify it as the root:
*
* <br><b>Specifying a custom root node directly</b>
* <pre>
* layoutData.treeRoot.item = root
* </pre>
*
* In cases where it's easier to look at an {@link INode} to determine whether it should be the root node or not, it's
* often most convenient to use the {@link SingleItem#delegate} property:
*
* <br><b>Specifying a custom root node via a delegate</b>
* <pre>
* // This is equivalent to the declaration above
* layoutData.treeRoot = node => node === root
* // Or use a different way of choosing the root node, e.g. by taking the node whose label
* // says 'Root'
* layoutData.treeRoot = node => node.labels.get(0).text === 'Root'
* </pre>
*
* Sometimes there might also be a collection that usually holds just the single node that should be the root, in which
* case {@link SingleItem#source} can be useful:
*
* <br><b>Specifying a custom root node from an IEnumerable</b>
* <pre>
* // Use the first selected node as the root
* // This will be evaluated anew each time the layout is applied
* layoutData.treeRoot.source = graphComponent.selection.selectedNodes
* </pre>
*
* @see BalloonLayout#SELECTED_ROOT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutData%23treeRoot Online Documentation}
*/
treeRoot: SingleItem<INode>;
/**
* Gets or sets the mapping that provides a {@link PreferredPlacementDescriptor} instance for edge
* {@link ILabel}s.
* Depending on how much customization is needed, some ways of setting {@link PreferredPlacementDescriptor}s are more
* convenient than others. For example, to set the same descriptor for all labels, you can just use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same PreferredPlacementDescriptor for all labels</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.constant = new PreferredPlacementDescriptor({
*   // Place labels along the edge
*   angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*   angle: 0,
*   // ... on either side
*   sideOfEdge: LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE,
*   // ... with a bit of distance to the edge
*   distanceToEdge: 5
* })
* </pre>
*
* If some labels should use custom placement or this has to be configured ahead of time, you can use the {@link ItemMapping#mapper} instead:
*
* <br><b>Customizing the placement of certain labels individually via a mapper</b>
* <pre>
* // Place label1 orthogonal to the edge anywhere on it
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label1,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.ANYWHERE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: Math.PI / 2
*   })
* )
* // Place label2 near the edge's source on either side of it, and make it parallel to the edge
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label2,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.AT_SOURCE,
*     sideOfEdge: LabelPlacements.RIGHT_OF_EDGE | LabelPlacements.LEFT_OF_EDGE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: 0
*   })
* )
* </pre>
*
* When the preferred placement can be inferred from the label itself, a delegate is usually the easiest choice:
*
* <br><b>Configuring preferred placement for all labels with a delegate</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.delegate = label => {
*   const customData = label.tag
*   const preferredPlacementDescriptor = new PreferredPlacementDescriptor({
*     angle: 0,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     // If the tag says to place the label in the center, put it in the center parallel to the edge's path
*     // All other labels can be placed anywhere, but on the side of the edge.
*     placeAlongEdge: customData.placeInCenter
*       ? LabelPlacements.AT_CENTER
*       : LabelPlacements.ANYWHERE,
*     sideOfEdge: customData.placeInCenter
*       ? LabelPlacements.ON_EDGE
*       : LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE
*   })
* }
* </pre>
*
* @see PreferredPlacementDescriptor
* @see LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BalloonLayoutData%23edgeLabelPreferredPlacement Online Documentation}
*/
edgeLabelPreferredPlacement: ItemMapping<ILabel, PreferredPlacementDescriptor>;
static $class: Class;
private $_BalloonLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BalloonLayoutData;
}
export interface BusRouterData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link BusRouter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData Online Documentation}*/
export class BusRouterData {
/**
Creates a new instance of {@link BusRouterData} which helps configuring {@link BusRouter}.
@param [options] The parameters to pass.
@param [options.affectedEdges] The collection of edges affected by this router. This option sets the {@link BusRouterData#affectedEdges} property on the created object.
@param [options.edgeDescriptors] The mapping of edges to their {@link BusRouterBusDescriptor} This option sets the {@link BusRouterData#edgeDescriptors} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link BusRouterData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link BusRouterData#targetPortConstraints} property on the created object.
@param [options.sourcePortCandidates] A mapping from edges to a collection of their source port {@link PortCandidate candidates}. This option sets the {@link BusRouterData#sourcePortCandidates} property on the created object.
@param [options.targetPortCandidates] A mapping from edges to a collection of their target port {@link PortCandidate candidates}. This option sets the {@link BusRouterData#targetPortCandidates} property on the created object.
@param [options.nodePortCandidateSets] A mapping from nodes to their {@link PortCandidateSet}. This option sets the {@link BusRouterData#nodePortCandidateSets} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link BusRouterData#abortHandler} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData Online Documentation}*/
constructor(options?: {
affectedEdges?: DpKeyItemCollection<IEdge> | DpKeyItemCollectionConvertible<IEdge>;
edgeDescriptors?: ItemMapping<IEdge, BusRouterBusDescriptor> | ItemMappingConvertible<IEdge, BusRouterBusDescriptor>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
sourcePortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
targetPortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
nodePortCandidateSets?: ItemMapping<INode, PortCandidateSet> | ItemMappingConvertible<INode, PortCandidateSet>;
abortHandler?: AbortHandler | null;
});
/**
Gets or sets the collection of edges affected by this router.
This collection is only used by the router if the {@link BusRouter#scope} is set to {@link EdgeRouterScope#ROUTE_AFFECTED_EDGES}. The {@link DpKeyItemCollection#dpKey} only has to
be specified if the {@link BusRouter#affectedEdgesDpKey} is not {@link BusRouter#DEFAULT_AFFECTED_EDGES_DP_KEY}
@see BusRouter#affectedEdgesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData%23affectedEdges Online Documentation}*/
affectedEdges: DpKeyItemCollection<IEdge>;
/**
Gets or sets the mapping of edges to their {@link BusRouterBusDescriptor}
The {@link BusRouterBusDescriptor} for an edge provides the edge's bus ID, its optional group IDs and whether or not the
edge is fixed.
<h2>Note</h2>
The bus ID of an edge is needed to determine which edges belong to a common bus. In the absence of an individual bus ID
for an edge, a bus consisting only of the single edge is created.
@see BusRouter#EDGE_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData%23edgeDescriptors Online Documentation}*/
edgeDescriptors: ItemMapping<IEdge, BusRouterBusDescriptor>;
/**
Gets or sets a mapping from edges to their source {@link PortConstraint}.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData%23sourcePortConstraints Online Documentation}*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to their target {@link PortConstraint}.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData%23targetPortConstraints Online Documentation}*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to a collection of their source port {@link PortCandidate candidates}.
@see PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData%23sourcePortCandidates Online Documentation}*/
sourcePortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from edges to a collection of their target port {@link PortCandidate candidates}.
@see PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData%23targetPortCandidates Online Documentation}*/
targetPortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from nodes to their {@link PortCandidateSet}.
@see PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData%23nodePortCandidateSets Online Documentation}*/
nodePortCandidateSets: ItemMapping<INode, PortCandidateSet>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/BusRouterData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
static $class: Class;
private $_BusRouterData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is BusRouterData;
}
export interface ChannelEdgeRouterData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link ChannelEdgeRouter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouterData Online Documentation}*/
export class ChannelEdgeRouterData {
/**
Creates a new instance of {@link ChannelEdgeRouterData} which helps configuring {@link ChannelEdgeRouter}.
@param [options] The parameters to pass.
@param [options.affectedEdges] The collection of edges affected by this router. This option sets the {@link ChannelEdgeRouterData#affectedEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouterData Online Documentation}*/
constructor(options?: {
affectedEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
Gets or sets the collection of edges affected by this router.
@see ChannelEdgeRouter#AFFECTED_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouterData%23affectedEdges Online Documentation}*/
affectedEdges: ItemCollection<IEdge>;
/**
Gets or sets the {@link ChannelEdgeRouterData#abortHandler} used during the layout.
@see AbortHandler#ABORT_HANDLER_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouterData%23abortHandler Online Documentation}*/
abortHandler: AbortHandler | null;
/**
Gets or sets a mapping from edges to their source {@link PortConstraint}.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouterData%23sourcePortConstraints Online Documentation}*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to their target {@link PortConstraint}.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouterData%23targetPortConstraints Online Documentation}*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to a collection of their source port {@link PortCandidate candidates}.
@see PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouterData%23sourcePortCandidates Online Documentation}*/
sourcePortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from edges to a collection of their target port {@link PortCandidate candidates}.
@see PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ChannelEdgeRouterData%23targetPortCandidates Online Documentation}*/
targetPortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
static $class: Class;
private $_ChannelEdgeRouterData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ChannelEdgeRouterData;
}
export interface CircularLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link CircularLayout}.
* The following example shows how to create a new instance of {@link CircularLayoutData} and use it with an
* {@link CircularLayout}:
*
* <br><b>Creating an instance of CircularLayoutData</b>
* <pre>
* const layoutData = new CircularLayoutData()
* // Put nodes of each color into their own circle
* layoutData.customGroups.delegate = node => node.style.fill
* // Retain a bit more space around nodes
* layoutData.nodeHalos.constant = NodeHalo.create(15)
*
* const layout = new CircularLayout()
* layout.layoutStyle = CircularLayoutStyle.CUSTOM_GROUPS
* graphComponent.graph.applyLayout(layout, layoutData)
* </pre>
*
* In many cases the complete initialization of {@link CircularLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of CircularLayoutData</b>
* <pre>
* const layoutData = new CircularLayoutData({
*   // Put nodes of each color into their own circle
*   customGroups: node => node.style.fill,
*   // Retain a bit more space around nodes
*   nodeHalos: NodeHalo.create(15)
* })
*
* const layout = new CircularLayout()
* layout.layoutStyle = CircularLayoutStyle.CUSTOM_GROUPS
* graphComponent.graph.applyLayout(layout, layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutData Online Documentation}
*/
export class CircularLayoutData {
/**
Creates a new instance of {@link CircularLayoutData} which helps configuring {@link CircularLayout}.
@param [options] The parameters to pass.
@param [options.circleIds] The mapper from nodes to their circle id. This option sets the {@link CircularLayoutData#circleIds} property on the created object.
@param [options.customGroups] The mapping from nodes to their custom group object. This option sets the {@link CircularLayoutData#customGroups} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link CircularLayoutData#nodeHalos} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link CircularLayoutData#abortHandler} property on the created object.
@param [options.edgeBundleDescriptors] The mapping of edges to their {@link EdgeBundleDescriptor}. This option sets the {@link CircularLayoutData#edgeBundleDescriptors} property on the created object.
@param [options.exteriorEdges] The collection of edges that are routed around the exterior of circle formed by each partition. This option sets the {@link CircularLayoutData#exteriorEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutData Online Documentation}*/
constructor(options?: {
circleIds?: IMapper<INode, number> | IMapperConvertible<INode, number>;
customGroups?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
abortHandler?: AbortHandler | null;
edgeBundleDescriptors?: ItemMapping<IEdge, EdgeBundleDescriptor> | ItemMappingConvertible<IEdge, EdgeBundleDescriptor>;
exteriorEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Gets or sets the mapper from nodes to their circle id.
* If this property is set, the mapper is filled by the layout with the id of the circle to which a node belongs.
*
* <br><b>Retrieving circle assignment information after a layout</b>
* <pre>
* const layoutData = new CircularLayoutData()
* layoutData.circleIds = new Mapper()
* graph.applyLayout(new CircularLayout(), layoutData)
*
* for (const node of graph.nodes) {
*   console.log(`Node ${node} has been placed in circle ${layoutData.circleIds[node]}`)
* }
* </pre>
*
* @see CircularLayout#CIRCLE_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutData%23circleIds Online Documentation}
*/
circleIds: IMapper<INode, number>;
/**
* Gets or sets the mapping from nodes to their custom group object.
* These groups will form the circles in the layout.
*
* Nodes that are mapped to <code>null</code> (or not explicitly mapped when using the {@link ItemMapping#mapper}) will not be part of any circle.
*
* <h2>Note</h2>
*
* The custom groups are only considered if {@link CircularLayout#layoutStyle} is set to {@link CircularLayoutStyle#CUSTOM_GROUPS}
*
* Often you might already have some way of grouping nodes into specific circles, in which case the {@link ItemMapping#delegate} property is usually
* the easiest, which allows you to derive the group identifier directly from each node:
*
* <br><b>Assigning group identifiers for all nodes based on each node</b>
* <pre>
* // Use the node's tag for circle assignment.
* // Nodes with the same tag are placed on the same circle.
* layoutData.customGroups = node => node.tag
* </pre>
*
* Another option would be to explicitly assign nodes that should appear on a given circle their respective group
* identifier via the {@link ItemMapping#mapper} property:
*
* <br><b>Assigning group identifiers for some nodes via a mapper</b>
* <pre>
* for (const node of circle1) {
*   // Use the collection as custom group ID, since it's the same for all nodes of it
*   layoutData.customGroups.mapper.set(node, circle1)
* }
* for (const node of circle2) {
*   // Use the collection as custom group ID, since it's the same for all nodes in it
*   layoutData.customGroups.mapper.set(node, circle2)
* }
* </pre>
*
* Finally, a rather pointless option would be to assign the same custom group identifier to all nodes, effectively forcing
* all nodes to be on the same circle:
*
* <br><b>Setting a constant group identifier for all nodes</b>
* <pre>
* layoutData.customGroups = {}
* // has the same effect as
* circularLayout.layoutStyle = CircularLayoutStyle.SINGLE_CYCLE
* </pre>
*
* As can be seen, there's an easier option for that.
*
* @see CircularLayout#CUSTOM_GROUPS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutData%23customGroups Online Documentation}
*/
customGroups: ItemMapping<INode, any>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
Gets or sets the mapping of edges to their {@link EdgeBundleDescriptor}.
Bundling together multiple edges means that their common parts are to some degree merged into a bundled part. At the
source and target point, the edges are again clearly split.
If an edge is mapped to <code>null</code>, the {@link EdgeBundling#defaultBundleDescriptor default descriptor} is used.
If an edge is marked for exterior routing (see {@link CircularLayoutData#exteriorEdges}), it is not bundled anymore.
@see EdgeBundleDescriptor
@see EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY
@see CircularLayout#edgeBundling
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutData%23edgeBundleDescriptors Online Documentation}*/
edgeBundleDescriptors: ItemMapping<IEdge, EdgeBundleDescriptor>;
/**
Gets or sets the collection of edges that are routed around the exterior of circle formed by each partition.
Ths property allows to individually select the exterior edges when {@link CircularLayoutEdgeRoutingPolicy#MARKED_EXTERIOR}
is specified as {@link CircularLayout#edgeRoutingPolicy edge routing policy}.
<b>Warning:</b> This selection of edges is only considered when using policy
{@link CircularLayoutEdgeRoutingPolicy#MARKED_EXTERIOR}.
<b>Warning:</b> Edges that are {@link EdgeBundleDescriptor#bundled bundled} but also marked for exterior routing here are not bundled anymore. Do not mark them for exterior
routing if bundling is desired.
@see CircularLayoutEdgeRoutingPolicy#MARKED_EXTERIOR
@see CircularLayout#EXTERIOR_EDGES_DP_KEY
@see CircularLayout#exteriorEdgeLayoutDescriptor
@see {@link https://docs.yworks.com/yfileshtml/#/api/CircularLayoutData%23exteriorEdges Online Documentation}*/
exteriorEdges: ItemCollection<IEdge>;
static $class: Class;
private $_CircularLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CircularLayoutData;
}
export interface ClearAreaLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link ClearAreaLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData Online Documentation}*/
export class ClearAreaLayoutData {
/**
Creates a new instance of {@link ClearAreaLayoutData}.
@param [options] The parameters to pass.
@param [options.expandedNode] The node that was expanded and, thus, defines the area that must be cleared. This option sets the {@link ClearAreaLayoutData#expandedNode} property on the created object.
@param [options.expandedNodeOriginalBounds] The original, non-expanded bounds of the {@link ClearAreaLayoutData#expandedNode}. This option sets the {@link ClearAreaLayoutData#expandedNodeOriginalBounds} property on the created object.
@param [options.expandedNodeOriginalEdgePaths] The mapping from edges adjacent to the {@link ClearAreaLayoutData#expandedNode} to their original edges paths, that is, their paths before the node was expanded. This option sets the {@link ClearAreaLayoutData#expandedNodeOriginalEdgePaths} property on the created object.
@param [options.areaNodes] The collection of nodes that define the area which must be cleared. This option sets the {@link ClearAreaLayoutData#areaNodes} property on the created object.
@param [options.areaGroupNode] The group node inside which the cleared area should be located. This option sets the {@link ClearAreaLayoutData#areaGroupNode} property on the created object.
@param [options.componentIds] The mapping from nodes to an object defining their component id. This option sets the {@link ClearAreaLayoutData#componentIds} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link ClearAreaLayoutData#nodeHalos} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link ClearAreaLayoutData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link ClearAreaLayoutData#targetPortConstraints} property on the created object.
@param [options.sourcePortCandidates] A mapping from edges to a collection of their source {@link PortCandidate port candidates}. This option sets the {@link ClearAreaLayoutData#sourcePortCandidates} property on the created object.
@param [options.targetPortCandidates] A mapping from edges to a collection of their target port {@link PortCandidate candidates}. This option sets the {@link ClearAreaLayoutData#targetPortCandidates} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link ClearAreaLayoutData#abortHandler} property on the created object.
@param [options.partitionGridData] The {@link PartitionGrid partition grid} layout data. This option sets the {@link ClearAreaLayoutData#partitionGridData} property on the created object.
@param [options.sourceGroupIds] A mapping from edges to an object representing their source edge group. This option sets the {@link ClearAreaLayoutData#sourceGroupIds} property on the created object.
@param [options.sourcePortGroupIds] A mapping from edges to an object representing their source port group. This option sets the {@link ClearAreaLayoutData#sourcePortGroupIds} property on the created object.
@param [options.targetGroupIds] A mapping from edges to an object representing their target edge group. This option sets the {@link ClearAreaLayoutData#targetGroupIds} property on the created object.
@param [options.targetPortGroupIds] A mapping from edges to an object representing their target port group. This option sets the {@link ClearAreaLayoutData#targetPortGroupIds} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData Online Documentation}*/
constructor(options?: {
expandedNode?: SingleItem<INode> | SingleItemConvertible<INode>;
expandedNodeOriginalBounds?: IRectangle | null | IRectangleConvertible;
expandedNodeOriginalEdgePaths?: ItemMapping<IEdge, IEnumerable<IPoint>> | ItemMappingConvertible<IEdge, IEnumerable<IPoint>>;
areaNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
areaGroupNode?: SingleItem<INode> | SingleItemConvertible<INode>;
componentIds?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
sourcePortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
targetPortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
abortHandler?: AbortHandler | null;
partitionGridData?: PartitionGridData;
sourceGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
sourcePortGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetPortGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
});
/**
Gets or sets the node that was expanded and, thus, defines the area that must be cleared.
When an expanded node is provided, the area that must be cleared is directly derived from the size of the expanded node.
Defining an expanded node is intended for the case that a node's size increased such that it might now overlap with
other graph elements. This includes the use case of expanding a folder node.
It is strongly recommended to additionally provide the original (smaller) bounds of the expanded node via property {@link ClearAreaLayoutData#expandedNodeOriginalBounds}. If
the node has incident edges, it is also recommended to provide the original paths of these edges via
{@link ClearAreaLayoutData#expandedNodeOriginalEdgePaths}. The original paths are the edge paths before the size of the node was changed (for example, before expanding a folder
node). This includes paths of edges that were only incident to the non-expanded node and are now incident to descendants
of it. Using this knowledge the algorithm clears the area occupied by the expanded node but also considers how the
layout looked before the change. This way the mental map may be preserved to a higher degree.
<b>Warning:</b> If an expanded node is specified, the {@link ClearAreaLayoutData#areaNodes} are ignored.
@see ClearAreaLayoutData#expandedNodeOriginalBounds
@see ClearAreaLayoutData#expandedNodeOriginalEdgePaths
@see ClearAreaLayout#EXPANDED_NODE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23expandedNode Online Documentation}*/
expandedNode: SingleItem<INode>;
/**
Gets or sets the original, non-expanded bounds of the {@link ClearAreaLayoutData#expandedNode}.
When an expanded node is provided, the area that must be cleared is directly derived from the bounds of the expanded
node. The original, non-expanded (smaller) bounds of the expanded node are helpful to preserve the mental map of the
overall drawing to a higher degree than without the additional knowledge about the previous layout. Note, however, that
only strategies {@link ClearAreaStrategy#PRESERVE_SHAPES}, {@link ClearAreaStrategy#PRESERVE_SHAPES_UNIFORM}
and {@link ClearAreaStrategy#GLOBAL} will significantly profit from it.
@see ClearAreaLayoutData#expandedNode
@see ClearAreaLayoutData#expandedNodeOriginalEdgePaths
@see ClearAreaLayout#EXPANDED_NODE_ORIGINAL_BOUNDS_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23expandedNodeOriginalBounds Online Documentation}*/
expandedNodeOriginalBounds: IRectangle | null;
/**
Gets or sets the mapping from edges adjacent to the {@link ClearAreaLayoutData#expandedNode} to their original edges paths, that is, their paths before the node
was expanded.
When an expanded node is provided, the area that must be cleared is directly derived from the bounds of the expanded
node. The original paths together with the original, non-expanded (smaller) bounds ({@link ClearAreaLayoutData#expandedNodeOriginalBounds}) of the expanded node are helpful
to preserve the mental map of the overall drawing by providing knowledge about the previous layout state. Note, however,
that only strategies {@link ClearAreaStrategy#PRESERVE_SHAPES}, {@link ClearAreaStrategy#PRESERVE_SHAPES_UNIFORM}
and {@link ClearAreaStrategy#GLOBAL} will significantly profit from it.
Carefully observe that the original paths must correspond to the ones that were valid when the expanded node still had
its original, non-expanded bounds. Thus, they should only be provided when the original bounds are provided too. Also
note that it is recommended that this mapping includes edges that are not adjacent to the expanded node anymore but were
adjacent when it was in non-expanded state.
<b>Warning:</b> An edge path consists of the location of the source port, followed by the locations of the bends, followed by the
location of the target port as returned by {@link IEdge#getPathPoints}.
@see ClearAreaLayoutData#expandedNode
@see ClearAreaLayoutData#expandedNodeOriginalBounds
@see ClearAreaLayout#EXPANDED_NODE_ORIGINAL_EDGE_PATH_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23expandedNodeOriginalEdgePaths Online Documentation}*/
expandedNodeOriginalEdgePaths: ItemMapping<IEdge, IEnumerable<IPoint>>;
/**
Gets or sets the collection of nodes that define the area which must be cleared.
These nodes define a sort of component from which the area that must be cleared is derived by calling method
{@link ClearAreaLayout#createAreaOutline}. This means that the area will exactly be defined by the current
locations of these nodes (including the edges among them) such that after the algorithm execution there will not be any
other elements intersecting with them.
Carefully observe that if group nodes are marked, all their descendants are treated as area nodes too. For example, if a
folder node was expanded (and became a group node) then it suffices to only mark the group to get the effect that the
area covered by it and all its descendants is cleared.
The {@link ClearAreaLayoutData#areaGroupNode area group node} can not be defined manually, but the nearest common ancestor of all area nodes will become the area
group node. For group nodes other than the area group either all their descendants including the group itself must be
area nodes or all their descendants including the group must not be area nodes. If the input is not complete in this
matter, the required nodes are internally automatically treated as area nodes too.
<b>Warning:</b> If area nodes are defined, then the values of properties {@link ClearAreaLayout#area} and {@link ClearAreaLayout#areaOutline} are ignored.
<b>Warning:</b> The specified area nodes are ignored when defining an {@link ClearAreaLayoutData#expandedNode}.
@see ClearAreaLayout#AREA_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23areaNodes Online Documentation}*/
areaNodes: ItemCollection<INode>;
/**
Gets or sets the group node inside which the cleared area should be located.
The {@link ClearAreaLayout#area area} that is cleared of elements can be associated to one specific group node. This means that this group can grow
because the free space must be created inside it. In consequence, all ancestor nodes of the group may grow, too. Other
groups are treated as fixed and must be moved in order to avoid an overlap with the specified area.
If no group is specified, the free space will be created in the top-level hierarchy.
<b>Warning:</b> The area group is ignored if the area is defined by means of
{@link ClearAreaLayoutData#areaNodes}. In that case the nearest common ancestor of the area nodes is automatically selected as the group node that contains
the free space.
@see ClearAreaLayout#AREA_GROUP_NODE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23areaGroupNode Online Documentation}*/
areaGroupNode: SingleItem<INode>;
/**
Gets or sets the mapping from nodes to an object defining their component id.
Nodes of the same component should preferably not be separated. While the algorithm may move a whole component, it
should tries to not move only a subset of its elements. This means that the algorithm tries to move all elements of a
component by the same offset (if at all). In order to achieve good results with this feature, the different components
should not overlap in the initial drawing.
<b>Warning:</b> The specified custom components are only considered when property {@link ClearAreaLayout#componentAssignmentStrategy} is set to
{@link ComponentAssignmentStrategy#CUSTOMIZED}. Furthermore, the components feature is only supported by {@link ClearAreaLayout#clearAreaStrategy clearing strategies}
{@link ClearAreaStrategy#PRESERVE_SHAPES}, {@link ClearAreaStrategy#PRESERVE_SHAPES_UNIFORM} and
{@link ClearAreaStrategy#GLOBAL}.
<b>Warning:</b> If the input graph contains group nodes or a {@link PartitionGrid} structure, maintaining the specified
components is not always possible. Furthermore, a custom component can never pass the border of a group node or
partition cell, i.e., all nodes of a custom component have to be associated with the same group or grid cell.
@see ClearAreaLayout#COMPONENT_ID_DP_KEY
@see ComponentAssignmentStrategy#CUSTOMIZED
@see ClearAreaLayout#componentAssignmentStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23componentIds Online Documentation}*/
componentIds: ItemMapping<INode, any>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets a mapping from edges to their source {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its source node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* A more general concept which allows finer control over where ports are placed, are {@link ClearAreaLayoutData#sourcePortCandidates port candidates}.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All source ports should be on the bottom side
* layoutData.sourcePortConstraints.constant = PortConstraint.create(PortSide.SOUTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should leave the source node on the left side, while keeping its exact port position (strong port constraint)
* layoutData.sourcePortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.WEST, true))
* // edge2 should leave the source node downwards, but the port may be placed anywhere along that node border
* layoutData.sourcePortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.SOUTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Source ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.sourcePortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see ClearAreaLayoutData#sourcePortCandidates
* @see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23sourcePortConstraints Online Documentation}
*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets a mapping from edges to their target {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its target node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* A more general concept which allows finer control over where ports are placed, are {@link ClearAreaLayoutData#targetPortCandidates port candidates}.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All target ports should be on the top side
* layoutData.targetPortConstraints.constant = PortConstraint.create(PortSide.NORTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should enter the target node on the right side, while keeping its exact port position (strong port
* // constraint)
* layoutData.targetPortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.EAST, true))
* // edge2 should enter the target node from the top, but the port may be placed anywhere along that node border
* layoutData.targetPortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.NORTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Target ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.targetPortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see ClearAreaLayoutData#targetPortCandidates
* @see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23targetPortConstraints Online Documentation}
*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets a mapping from edges to a collection of their source {@link PortCandidate port candidates}.
* Port constraints allow to define where an edge can connect to its source node and allow fine control over port
* placement.
*
* If all that is needed is to fix the source port in its location or on a node side, {@link ClearAreaLayoutData#sourcePortConstraints port constraints} are easier to work
* with, since they are a slightly simpler concept.
*
* Source port candidates are effectively a collection of possible port placements with different costs and the layout
* algorithm is free to choose the candidate that fits best into the overall layout, while also preferring candidates with
* a lower cost. To set the same candidate list for all edges, it's easiest to use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same set of source port candidates for all edges</b>
* <pre>
* layoutData.sourcePortCandidates.constant = new List([
*   // Prefer the center position and route the edge downwards if possible (no cost for this candidate)
*   PortCandidate.createCandidate(0, 0, PortDirections.SOUTH),
*   // Alternatively, use any position at the bottom border of the node, but with a higher cost
*   PortCandidate.createCandidate(PortDirections.SOUTH, 1),
*   // If that fails, use either the left or the right side
*   PortCandidate.createCandidate(PortDirections.EAST | PortDirections.WEST, 2)
* ])
* </pre>
*
* If certain edges need specific port candidates, it's usually convenient to use the {@link ItemMapping#mapper} property:
*
* <br><b>Setting different source port candidates for certain edges</b>
* <pre>
* // edge1 should leave the source node preferably on the node side in flow direction,
* // but can also use a fixed port location on the left of the node.
* layoutData.sourcePortCandidates.mapper.set(
*   edge1,
*   new List([
*     PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*     PortCandidate.createCandidate(-0.5, 0.25, PortDirections.WEST, 1)
*   ])
* )
*
* // edge2 should leave the source node anywhere on the upper side of the node
* layoutData.sourcePortCandidates.mapper.set(
*   edge2,
*   new List([PortCandidate.createCandidate(PortDirections.NORTH)])
* )
* </pre>
*
* For cases when the desired configuration of port candidates can be readily created from the edge itself, the {@link ItemMapping#delegate} is often
* the most convenient option:
*
* <br><b>Setting different source port candidates for each edge with a delegate</b>
* <pre>
* // Assuming a flowchart, the "yes" edge should leave the decision in flow direction,
* // and the "no" edge may leave either left or right of the flow direction.
* // This also ensures that the layout still looks natural if the layout uses different orientations.
* layoutData.sourcePortCandidates.delegate = edge => {
*   // Only handle decision nodes here
*   if (edge.sourceNode.tag instanceof FlowChartDecision) {
*     switch (edge.labels.get(0).text) {
*       case 'Yes':
*         return new List([PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW)])
*       case 'No':
*         return new List([
*           PortCandidate.createCandidate(
*             PortDirections.LEFT_IN_FLOW | PortDirections.RIGHT_IN_FLOW
*           )
*         ])
*     }
*   }
*   return null
* }
* </pre>
*
* @see ClearAreaLayoutData#sourcePortConstraints
* @see PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23sourcePortCandidates Online Documentation}
*/
sourcePortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
* Gets or sets a mapping from edges to a collection of their target port {@link PortCandidate candidates}.
* Port constraints allow to define where an edge can connect to its target node and allow fine control over port
* placement.
*
* If all that is needed is to fix the target port in its location or on a node side, {@link ClearAreaLayoutData#targetPortConstraints port constraints} are easier to work
* with, since they are a slightly simpler concept.
*
* Target port candidates are effectively a collection of possible port placements with different costs and the layout
* algorithm is free to choose the candidate that fits best into the overall layout, while also preferring candidates with
* a lower cost. To set the same candidate list for all edges, it's easiest to use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same set of target port candidates for all edges</b>
* <pre>
* layoutData.targetPortCandidates.constant = new List([
*   // Prefer the center position and enter the node from the top if possible (no cost for this candidate)
*   PortCandidate.createCandidate(0, 0, PortDirections.NORTH),
*   // Alternatively, use any position at the top border of the node, but with a higher cost
*   PortCandidate.createCandidate(PortDirections.NORTH, 1),
*   // If that fails, use either the left or the right side
*   PortCandidate.createCandidate(PortDirections.EAST | PortDirections.WEST, 2)
* ])
* </pre>
*
* If certain edges need specific port candidates, it's usually convenient to use the {@link ItemMapping#mapper} property:
*
* <br><b>Setting different target port candidates for certain edges</b>
* <pre>
* // edge1 should enter the target node preferably on the node side in flow direction,
* // but can also use a fixed port location on the left of the node.
* layoutData.targetPortCandidates.mapper.set(
*   edge1,
*   new List([
*     PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*     PortCandidate.createCandidate(-0.5, -0.25, PortDirections.WEST, 1)
*   ])
* )
*
* // edge2 should enter the target node anywhere on the bottom side of the node
* layoutData.targetPortCandidates.mapper.set(
*   edge2,
*   new List([PortCandidate.createCandidate(PortDirections.SOUTH)])
* )
* </pre>
*
* For cases when the desired configuration of port candidates can be readily created from the edge itself, the {@link ItemMapping#delegate} is often
* the most convenient option:
*
* <br><b>Setting different target port candidates for each edge with a delegate</b>
* <pre>
* // Assuming a flowchart, edges connecting to a terminal on the target side should enter that node either
* // in flow direction (preferred) or from one of the sides.
* // This also ensures that the layout still looks natural if the layout uses different orientations.
* layoutData.targetPortCandidates.delegate = edge => {
*   // Only handle terminal nodes here
*   if (edge.targetNode.tag instanceof FlowChartTerminal) {
*     return new List([
*       PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*       PortCandidate.createCandidate(PortDirections.LEFT_IN_FLOW | PortDirections.RIGHT_IN_FLOW, 1)
*     ])
*   }
*   return null
* }
* </pre>
*
* @see ClearAreaLayoutData#targetPortConstraints
* @see PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23targetPortCandidates Online Documentation}
*/
targetPortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
Gets or sets the {@link PartitionGrid partition grid} layout data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23partitionGridData Online Documentation}*/
partitionGridData: PartitionGridData;
/**
Gets or sets a mapping from edges to an object representing their source edge group.
Edges sharing a source group identifier share a common bus near the source or at a common source node.
The {@link ClearAreaLayout} considers edge grouping information only during the rerouting of edges using the
specified {@link ClearAreaLayout#edgeRouter}. The applied routing algorithm must support edge grouping for this property to have an effect. Furthermore,
if edges are not altered by {@link ClearAreaLayout} and not grouped in the input layout, then they will not be
changed only to satisfy the edge grouping.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23sourceGroupIds Online Documentation}*/
sourceGroupIds: ItemMapping<IEdge, any>;
/**
Gets or sets a mapping from edges to an object representing their source port group.
All edges with the same port id at a node will share the same port location. However, they are routed independently and
in contrast to {@link ClearAreaLayoutData#sourceGroupIds source groups} deviate from a common path as soon as possible instead of sharing as much of a common path
as possible.
The {@link ClearAreaLayout} considers port grouping information only during the rerouting of edges using the
specified {@link ClearAreaLayout#edgeRouter}. The applied routing algorithm must support port grouping for this property to have an effect. Furthermore,
if edges are not altered by {@link ClearAreaLayout} and not port-grouped in the input layout, then they will not
be changed only to satisfy the port grouping.
@see ClearAreaLayoutData#sourceGroupIds
@see PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23sourcePortGroupIds Online Documentation}*/
sourcePortGroupIds: ItemMapping<IEdge, any>;
/**
Gets or sets a mapping from edges to an object representing their target edge group.
Edges sharing a target group identifier share a common bus near the target or at a common target node.
The {@link ClearAreaLayout} considers edge grouping information only during the rerouting of edges using the
specified {@link ClearAreaLayout#edgeRouter}. The applied routing algorithm must support edge grouping for this property to have an effect. Furthermore,
if edges are not altered by {@link ClearAreaLayout} and not grouped in the input layout, then they will not be
changed only to satisfy the edge grouping.
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23targetGroupIds Online Documentation}*/
targetGroupIds: ItemMapping<IEdge, any>;
/**
Gets or sets a mapping from edges to an object representing their target port group.
All edges with the same port id at a node share the same port location. However, they are routed independently and in
contrast to {@link ClearAreaLayoutData#targetGroupIds target groups} deviate from a common path as soon as possible instead of sharing as much of a common path as
possible.
The {@link ClearAreaLayout} considers port grouping information only during the rerouting of edges using the
specified {@link ClearAreaLayout#edgeRouter}. The applied routing algorithm must support port grouping for this property to have an effect. Furthermore,
if edges are not altered by {@link ClearAreaLayout} and not port-grouped in the input layout, then they will not
be changed only to satisfy the port grouping.
@see ClearAreaLayoutData#targetGroupIds
@see PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayoutData%23targetPortGroupIds Online Documentation}*/
targetPortGroupIds: ItemMapping<IEdge, any>;
static $class: Class;
private $_ClearAreaLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClearAreaLayoutData;
}
export interface ComponentLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link ComponentLayout}.
* The following example shows how to create a new instance of {@link ComponentLayoutData} and use it with an
* {@link ComponentLayout}:
*
* <br><b>Creating an instance of ComponentLayoutData</b>
* <pre>
* const layoutData = new ComponentLayoutData()
* // Only process components that contain a selected node
* layoutData.affectedComponents.source = graphComponent.selection.selectedNodes
* // Reserve additional space around nodes
* layoutData.nodeHalos.constant = NodeHalo.create(15)
*
* graphComponent.graph.applyLayout(new ComponentLayout(), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link ComponentLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of ComponentLayoutData</b>
* <pre>
* const layoutData = new ComponentLayoutData({
*   // Only process components that contain a selected node
*   affectedComponents: graphComponent.selection.selectedNodes,
*   // Reserve additional space around nodes
*   nodeHalos: NodeHalo.create(15)
* })
*
* graphComponent.graph.applyLayout(new ComponentLayout(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayoutData Online Documentation}
*/
export class ComponentLayoutData {
/**
Creates a new instance of {@link ComponentLayoutData} which helps configuring {@link ComponentLayout}.
@param [options] The parameters to pass.
@param [options.affectedComponents] A collection of nodes that determine the components that shall be laid out. This option sets the {@link ComponentLayoutData#affectedComponents} property on the created object.
@param [options.componentIds] The mapping from nodes to their component comparable that is used to sort the components. This option sets the {@link ComponentLayoutData#componentIds} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link ComponentLayoutData#nodeHalos} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link ComponentLayoutData#abortHandler} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayoutData Online Documentation}*/
constructor(options?: {
affectedComponents?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
componentIds?: ItemMapping<INode, IComparable> | ItemMappingConvertible<INode, IComparable>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
abortHandler?: AbortHandler | null;
});
/**
* Gets or sets a collection of nodes that determine the components that shall be laid out.
* Note that components will be laid out if and only if at least one of the nodes is in this collection.
*
* Defining the subset of components that should be laid out can be done in various ways, mostly depending on which option
* is more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of nodes to define the components to layout</b>
* <pre>
* layoutData.affectedComponents = graphComponent.selection.selectedNodes
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual nodes to define components to layout</b>
* <pre>
* layoutData.affectedComponents.items.add(node1)
* layoutData.affectedComponents.items.add(node2)
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every node
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether a node belongs to a component to layout</b>
* <pre>
* // We assume here that all nodes have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.affectedComponents = node => node.tag
* </pre>
*
* @see ComponentLayout#AFFECTED_COMPONENTS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayoutData%23affectedComponents Online Documentation}
*/
affectedComponents: ItemCollection<INode>;
/**
* Gets or sets the mapping from nodes to their component comparable that is used to sort the components.
* <b>Warning:</b> Most layout algorithms cannot handle disconnected components (only the {@link HierarchicLayout}, {@link PartitionLayout}
* and {@link EdgeRouter} can handle such components). Hence, you should not assign the same component ID to nodes of
* different {@link GraphConnectivity#connectedComponents connected components} for algorithms that cannot handle the disconnected components.
*
* Since this mapping has to be complete most of the time, the easiest way is usually to use a delegate:
*
* <br><b>Setting a component ID for every node</b>
* <pre>
* // Split components according to the node's color
* layoutData.componentIds = node => node.style.fill.toString()
* </pre>
*
* @see ComponentLayout#COMPONENT_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayoutData%23componentIds Online Documentation}
*/
componentIds: ItemMapping<INode, IComparable>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ComponentLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
static $class: Class;
private $_ComponentLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ComponentLayoutData;
}
export interface CompositeLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Combines several {@link LayoutData} instances.
* This is especially useful if several layouts are run at once, for example a general layout and a subsequent edge
* routing.
*
* The {@link LayoutData} are applied in the order as they appear in the {@link CompositeLayoutData#items} collection. This may influence which
* property "wins" when there are multiple {@link LayoutData} that define the same mapping or collection, e.g.
* {@link AbortHandler}, or affected nodes. Generally those should only ever be set on one of the child {@link LayoutData}
* instances.
*
* The following example shows how {@link CompositeLayoutData} can be used to combine multiple {@link LayoutData}
* instances:
*
* <pre>
* // Create layout data to be combined first
*
* // Wrap them into CompositeLayoutData
* const layoutData = new CompositeLayoutData(fixNodeLayoutData, organicLayoutData, edgeRouterData)
*
* // Run the combined layout
* const layout = new EdgeRouter(new FixNodeLayoutStage(new OrganicLayout()))
* graphComponent.graph.applyLayout(layout, layoutData)
* </pre>
*
* {@link CompositeLayoutData} can also be created via an object initializer, where all nested {@link LayoutData}
* instances can be created inline as well:
*
* <pre>
* const layoutData = new CompositeLayoutData({
*   items: [fixNodeLayoutData, organicLayoutData, edgeRouterData]
* })
*
* // Run the combined layout
* const layout = new EdgeRouter(new FixNodeLayoutStage(new OrganicLayout()))
* graphComponent.graph.applyLayout(layout, layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutData Online Documentation}
*/
export class CompositeLayoutData {
/**
Creates a new instance which combines the given <code>items</code>.
@param items The {@link LayoutData} to combine.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutData Online Documentation}*/
constructor(...items: LayoutData[]);
/**
Gets or sets a collection of composed layout data objects.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CompositeLayoutData%23items Online Documentation}*/
items: ICollection<LayoutData>;
static $class: Class;
private $_CompositeLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CompositeLayoutData;
}
export interface ContextItemMapping<TItem, TValue, TContext> extends ItemMapping<TItem, TValue> {
}
/**
* Provides different ways to define a mapping from items to values that takes a context object into consideration, too.
* Only one of the provided ways to define a mapping can be used at the same time, therefore only one of the properties {@link ContextItemMapping#contextDelegate}, {@link ItemMapping#delegate},
* {@link ItemMapping#mapper}, and {@link ItemMapping#constant} should be set on each instance.
*
* This class supports {@link LayoutData} implementations and does not usually have to be used explicitly.
*
* Functions, {@link IMapper}s and constant values are automatically converted to {@link ContextItemMapping}s. For example:
*
* <pre>
* // Constant
* new PartitionGridData({ cellIds: partitionGrid.createCellId(0, 0) })
*
* // Context Delegate
* new PartitionGridData({ cellIds: (node, grid) => grid.createCellId(0, 0) })
* // or
* const partitionGrid = new PartitionGrid(4, 4)
* new PartitionGridData({ cellIds: node => partitionGrid.createCellId(0, 0) })
*
* // Mapper
* new PartitionGridData({ cellIds: new Mapper() })
* // or a JavaScript Map
* new PartitionGridData({ cellIds: new Map() })
* </pre>
*
* Furthermore, the conversion supports a parameter object to initialize the properties. For example:
*
* <pre>
* // Create a new ContextItemMapping from a JavaScript Map and initialize its 'mapper' with the new Map.
* // This also works for the different properties as well.
* const mapping = ContextItemMapping.from({
*   mapper: new Map()
* })
*
* // Or, instead of providing the specific property, the 'includes' key can be used
* const mapping = ContextItemMapping.from({
*   includes: new Map()
* })
* </pre>
*
* Type parameter <code>TItem</code>: The type of the items that are mapped.
* Type parameter <code>TValue</code>: The type of the values the items are mapped to.
* Type parameter <code>TContext</code>: The type of the context passed to the {@link ContextItemMapping#contextDelegate}
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ContextItemMapping Online Documentation}
*/
export class ContextItemMapping<TItem, TValue, TContext> {
/**
Constructor overload that directly sets the {@link ItemMapping#delegate}.
@param delegate A delegate that implements the mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextItemMapping%23ContextItemMapping(function(TItem)%3ATValue) Online Documentation}*/
constructor(delegate: (key: TItem) => TValue);
/**
Constructor overload that directly sets a {@link ItemMapping#constant} mapping.
@param constant A constant value all items are mapped to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextItemMapping%23ContextItemMapping(TValue) Online Documentation}*/
constructor(constant: TValue);
/**
Constructor overload that directly sets the {@link ItemMapping#mapper}.
@param mapper An {@link IMapper} that implements the mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextItemMapping%23ContextItemMapping(IMapper%3CTItem%2CTValue%3E) Online Documentation}*/
constructor(mapper: IMapper<TItem, TValue> | IMapperConvertible<TItem, TValue>);
/**
Default constructor that does not explicitly initialize any of the {@link ContextItemMapping#contextDelegate}, {@link ItemMapping#mapper}, {@link ItemMapping#delegate} or {@link ItemMapping#constant} properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextItemMapping%23ContextItemMapping() Online Documentation}*/
constructor();
/**
Creates a ContextItemMapping instance from the given object by performing automatic type conversion.
Type parameter <code>TItem</code>: The type of the items that are mapped.
Type parameter <code>TValue</code>: The type of the values the items are mapped to.
Type parameter <code>TContext</code>: The type of the context passed to the {@link ContextItemMapping#contextDelegate}
@param itemMappingLike The object to convert to a ContextItemMapping.
@returns Either <code>itemMappingLike</code> if it is already a ContextItemMapping or a new instance, initialized to /// the values found in
<code>itemMappingLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextItemMapping%23from(ContextItemMapping%3CTItem%2CTValue%2CTContext%3E) Online Documentation}*/
static from<TItem, TValue, TContext>(itemMappingLike: ContextItemMapping<TItem, TValue, TContext> | ContextItemMappingConvertible<TItem, TValue, TContext>): ContextItemMapping<TItem, TValue, TContext>;
/**
Provides a mapper mapping the items to values.
@param adapter The graph adapter the mapper shall be provided for.
@param layout The layout using the mapper.
@param context The context object passed to the {@link ContextItemMapping#contextDelegate}.
@returns A mapper mapping the items to values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextItemMapping%23provideMapperForContext(LayoutGraphAdapter%2CILayoutAlgorithm%2CTContext) Online Documentation}*/
provideMapperForContext(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, context: TContext): IMapper<TItem, TValue> | null;
/**
Gets or sets a delegate from the items and a context object to the values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ContextItemMapping%23contextDelegate Online Documentation}*/
contextDelegate: ((arg1: TItem, arg2: TContext) => TValue) | null;
static $class: Class;
private $_ContextItemMapping: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TItem, TValue, TContext>(o: any): o is ContextItemMapping<TItem, TValue, TContext>;
}
/** Automatic conversion to {@link ContextItemMapping}. */
type ContextItemMappingConvertible<TItem, TValue, TContext> = IMapper<TItem, TValue> | Map<TItem, TValue> | ((item: TItem, ctx: TContext) => TValue | null) | ((item: TItem) => TValue | null) | TValue | {
includes: any;
} | {
constant: TValue;
} | {
delegate: (item: TItem) => TValue | null;
} | {
contextDelegate: (item: TItem, ctx: TContext) => TValue | null;
} | {
mapper: Map<TItem, TValue>;
};
export interface CurveRoutingStageData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies the layout data for the {@link CurveRoutingStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData Online Documentation}*/
export class CurveRoutingStageData {
constructor();
/**
* Gets or sets the collection of affected edges whose paths will be replaced by curved paths.
* This collection is only used by the stage if the {@link CurveRoutingStage#scope} is set to {@link EdgeRouterScope#ROUTE_AFFECTED_EDGES}.
*
* Defining the subset of edges that should be laid out can be done in various ways, mostly depending on which option is
* more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of affected edges</b>
* <pre>
* layoutData.affectedEdges.source = graphComponent.selection.selectedEdges
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual affected edges</b>
* <pre>
* layoutData.affectedEdges.items.add(edge1)
* layoutData.affectedEdges.items.add(edge2)
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every edge
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether an edge is affected or not</b>
* <pre>
* // We assume here that all edges have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.affectedEdges.delegate = edge => edge.tag.includeInLayout
* </pre>
*
* @see CurveRoutingStage#affectedEdgesDpKey
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData%23affectedEdges Online Documentation}
*/
affectedEdges: DpKeyItemCollection<IEdge>;
/**
* Gets or sets the collection of affected nodes, meaning that the paths of all edges adjacent to an affected node will be
* replaced by curved paths.
* This collection is only used by the stage if the {@link CurveRoutingStage#scope} is set to {@link EdgeRouterScope#ROUTE_EDGES_AT_AFFECTED_NODES}.
*
* Defining the subset of nodes that should be laid out can be done in various ways, mostly depending on which option is
* more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of affected nodes</b>
* <pre>
* layoutData.affectedNodes.source = graphComponent.selection.selectedNodes
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual affected nodes</b>
* <pre>
* layoutData.affectedNodes.items.add(node1)
* layoutData.affectedNodes.items.add(node2)
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every node
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether a node is affected or not</b>
* <pre>
* // We assume here that all nodes have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.affectedNodes.delegate = node => node.tag.includeInLayout
* </pre>
*
* @see CurveRoutingStage#affectedNodesDpKey
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData%23affectedNodes Online Documentation}
*/
affectedNodes: DpKeyItemCollection<INode>;
/**
Gets or sets the mapping of edges to their {@link CurveEdgeLayoutDescriptor}
A descriptor provides additional routing information and allows to, e.g., specify individual minimum distance values
that must be considered when routing an edge.
If an edge is mapped to <code>null</code>, the {@link CurveRoutingStage} uses the the {@link CurveRoutingStage#defaultEdgeLayoutDescriptor default descriptor}.
@see CurveRoutingStage#CURVE_EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData%23edgeLayoutDescriptors Online Documentation}*/
edgeLayoutDescriptors: ItemMapping<IEdge, CurveEdgeLayoutDescriptor>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
Gets or sets the {@link PartitionGrid partition grid} layout data.
The {@link CurveRoutingStage} respects a partition grid such that curved edges are only allowed to intersect a
column/row boundary if the edge path created by the core layout algorithm already intersects it.
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData%23partitionGridData Online Documentation}*/
partitionGridData: PartitionGridData;
/**
Gets or sets the mapping from nodes to their {@link NodeHalo}.
{@link NodeHalo}s allow to reserve space around nodes. Carefully note that the {@link CurveRoutingStage}
only properly considers halos if the core layout already correctly considered them or if they are already correctly
considered in case that no core layout is defined.
@see NodeHalo#NODE_HALO_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData%23nodeHalos Online Documentation}*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the mapping that provides a {@link PreferredPlacementDescriptor} instance for edge
* {@link ILabel}s.
* Carefully note that the {@link CurveRoutingStage} only properly considers preferred edge label placement if the
* core layout already correctly considered them or if they are already correct in case that no core layout is defined.
*
* Depending on how much customization is needed, some ways of setting {@link PreferredPlacementDescriptor}s are more
* convenient than others. For example, to set the same descriptor for all labels, you can just use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same PreferredPlacementDescriptor for all labels</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.constant = new PreferredPlacementDescriptor({
*   // Place labels along the edge
*   angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*   angle: 0,
*   // ... on either side
*   sideOfEdge: LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE,
*   // ... with a bit of distance to the edge
*   distanceToEdge: 5
* })
* </pre>
*
* If some labels should use custom placement or this has to be configured ahead of time, you can use the {@link ItemMapping#mapper} instead:
*
* <br><b>Customizing the placement of certain labels individually via a mapper</b>
* <pre>
* // Place label1 orthogonal to the edge anywhere on it
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label1,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.ANYWHERE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: Math.PI / 2
*   })
* )
* // Place label2 near the edge's source on either side of it, and make it parallel to the edge
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label2,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.AT_SOURCE,
*     sideOfEdge: LabelPlacements.RIGHT_OF_EDGE | LabelPlacements.LEFT_OF_EDGE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: 0
*   })
* )
* </pre>
*
* When the preferred placement can be inferred from the label itself, a delegate is usually the easiest choice:
*
* <br><b>Configuring preferred placement for all labels with a delegate</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.delegate = label => {
*   const customData = label.tag
*   const preferredPlacementDescriptor = new PreferredPlacementDescriptor({
*     angle: 0,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     // If the tag says to place the label in the center, put it in the center parallel to the edge's path
*     // All other labels can be placed anywhere, but on the side of the edge.
*     placeAlongEdge: customData.placeInCenter
*       ? LabelPlacements.AT_CENTER
*       : LabelPlacements.ANYWHERE,
*     sideOfEdge: customData.placeInCenter
*       ? LabelPlacements.ON_EDGE
*       : LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE
*   })
* }
* </pre>
*
* @see PreferredPlacementDescriptor
* @see LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData%23edgeLabelPreferredPlacement Online Documentation}
*/
edgeLabelPreferredPlacement: ItemMapping<ILabel, PreferredPlacementDescriptor>;
/**
* Gets or sets a mapping from edges to an object representing their source edge group.
* Edges sharing a source group identifier will share a common bus near the source or at a common source node. Importantly,
* the {@link CurveRoutingStage} supports edge grouping such that edges that are correctly grouped by the core layout
* algorithm remain grouped in the curved layout result. However, it will not create new groupings.
*
* One simple way to use source groups is to use the edge's source node as group ID which effectively groups all edges with
* the same source together:
*
* <br><b>Grouping all edges on the source side</b>
* <pre>
* layoutData.sourceGroupIds = edge => edge.sourceNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.sourceGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.sourceGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see CurveRoutingStageData#targetGroupIds
* @see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData%23sourceGroupIds Online Documentation}
*/
sourceGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to an object representing their target edge group.
* Edges sharing a target group identifier will share a common bus near the target or at a common target node. Importantly,
* the {@link CurveRoutingStage} supports edge grouping such that edges that are correctly grouped by the core layout
* algorithm remain grouped in the curved layout result. However, it will not create new groupings.
*
* One simple way to use source groups is to use the edge's target node as group ID which effectively groups all edges with
* the same target together:
*
* <br><b>Grouping all edges on the target side</b>
* <pre>
* layoutData.targetGroupIds = edge => edge.targetNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.targetGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.targetGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see CurveRoutingStageData#sourceGroupIds
* @see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/CurveRoutingStageData%23targetGroupIds Online Documentation}
*/
targetGroupIds: ItemMapping<IEdge, any>;
static $class: Class;
private $_CurveRoutingStageData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is CurveRoutingStageData;
}
export interface MapperDataProviderAdapter<TKey, TValue> extends YObject, IDataProvider {
/**
@inheritdoc*/
get(dataHolder: any | null): any | null;
/**
@inheritdoc*/
getBoolean(dataHolder: any | null): boolean;
/**
@inheritdoc*/
getInt(dataHolder: any | null): number;
/**
@inheritdoc*/
getNumber(dataHolder: any | null): number;
} /**
Wraps an {@link IMapper} into an {@link IDataProvider}.
This class is used by {@link LayoutExecutor}, {@link LayoutGraphAdapter}, and {@link YGraphAdapter} to
convert an {@link IGraph} into a corresponding
{@link Graph}. Unless that conversion needs to be customized, or aforementioned classes cannot be used, this
class usually doesn't need to be used explicitly.
Type parameter <code>TKey</code>: The type of the keys.
Type parameter <code>TValue</code>: The type of the values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperDataProviderAdapter Online Documentation}*/
export class MapperDataProviderAdapter<TKey, TValue> {
/**
Creates a new instance that adapts the provided mapper instance.
@param keyType The type of the keys.
@param mapper The mapper to adapt.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MapperDataProviderAdapter Online Documentation}*/
constructor(keyType: Class, mapper: IMapper<TKey, TValue> | IMapperConvertible<TKey, TValue>);
static $class: Class;
private $_DataProviderAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TKey, TValue>(o: any): o is MapperDataProviderAdapter<TKey, TValue>;
}
export interface DataMapAdapter<K, V> extends MapperDataProviderAdapter<K, V>, IDataMap {
/**
@inheritdoc*/
set(dataHolder: any | null, value: any | null): void;
/**
@inheritdoc*/
setBoolean(dataHolder: any | null, value: boolean): void;
/**
@inheritdoc*/
setInt(dataHolder: any | null, value: number): void;
/**
@inheritdoc*/
setNumber(dataHolder: any | null, value: number): void;
} /**
An adapter implementation that wraps an {@link IMapper} to an {@link IDataMap}.
Type parameter <code>K</code>: The type of the key for the mapper.
Type parameter <code>V</code>: The type of the value for the mapper.
@see MapperDataProviderAdapter
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataMapAdapter Online Documentation}*/
export class DataMapAdapter<K, V> {
/**
Initializes a new instance of the {@link DataMapAdapter} class.
@param keyType The type of the key for the mapper.
@param valueType The type of the value for the mapper.
@param mapper The mapper to adapt.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DataMapAdapter Online Documentation}*/
constructor(keyType: Class, valueType: Class, mapper: IMapper<K, V> | IMapperConvertible<K, V>);
static $class: Class;
private $_DataMapAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<K, V>(o: any): o is DataMapAdapter<K, V>;
}
export interface DpKeyItemCollection<T> extends ItemCollection<T> {
}
/**
* An item collection that allows to specify the data provider key the collection is registered with.
* This class supports {@link LayoutData} implementations and does not usually have to be used explicitly.
*
* Functions, {@link IMapper}s, lists and single items are automatically converted to {@link DpKeyItemCollection}s. For example:
*
* <pre>
* // Delegate
* new StraightLineEdgeRouterData({ affectedEdges: edge => isAffected(edge) })
*
* // Item
* new StraightLineEdgeRouterData({ affectedEdges: edge1 })
*
* // Items
* new StraightLineEdgeRouterData({ affectedEdges: [edge1, edge2] })
*
* // Mapper
* new StraightLineEdgeRouterData({ affectedEdges: new Mapper() })
* // or a JavaScript Map
* new StraightLineEdgeRouterData({ affectedEdges: new Map() })
*
* // Source
* treeLayoutData.assistantNodes = graphComponent.selection.selectedNodes
* </pre>
*
* Furthermore, the conversion supports a parameter object to initialize the properties. For example:
*
* <pre>
* // This provides a convenient way of specifying the affectedNodes of StraightLineEdgeRouterData:
* const routerData = new StraightLineEdgeRouterData({
*   // only consider elliptical nodes in the graph
*   affectedNodes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
* })
*
* // This provides a convenient way of specifying the excluded elements of the DpKeyItemCollection:
* const routerData = new StraightLineEdgeRouterData({
*   affectedNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
*
* // Create a new DpKeyItemCollection from a JavaScript Map and initialize its 'mapper' with the new Map.
* // This also works for the other properties as well.
* const collection = DpKeyItemCollection.from({
*   mapper: new Map()
* })
*
* // Or, instead of providing the specific property, the 'includes' key can be used
* const collection = DpKeyItemCollection.from({
*   includes: new Map()
* })
* </pre>
*
* Type parameter <code>T</code>: The type of the items in the collection.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyItemCollection Online Documentation}
*/
export class DpKeyItemCollection<T> {
constructor();
/**
Creates a DpKeyItemCollection instance from the given object by performing automatic type conversion.
Type parameter <code>T</code>: The type of the items in the collection.
@param itemCollectionLike The object to convert to a DpKeyItemCollection.
@returns Either <code>itemCollectionLike</code> if it is already a DpKeyItemCollection or a new instance, initialized to /// the values found
in <code>itemCollectionLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyItemCollection%23from(DpKeyItemCollection%3CT%3E) Online Documentation}*/
static from<T>(itemCollectionLike: DpKeyItemCollection<T> | DpKeyItemCollectionConvertible<T>): DpKeyItemCollection<T>;
/**
Gets or sets the data provider key that is used for the item collection.
If no key is set, the default data provider key is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/DpKeyItemCollection%23dpKey Online Documentation}*/
dpKey: DpKeyBase<boolean> | null;
static $class: Class;
private $_DpKeyItemCollection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<T>(o: any): o is DpKeyItemCollection<T>;
}
/** Automatic conversion to {@link DpKeyItemCollection}. */
type DpKeyItemCollectionConvertible<T> = T[] | ICollection<T> | IEnumerable<T> | IMapper<T, boolean> | Map<T, boolean> | ((item: T) => boolean) | T | {
includes?: any;
excludes?: any;
} | {
delegate: (item: T) => boolean;
} | {
item: T;
} | {
items: Object;
} | {
mapper: Map<T, boolean>;
} | {
source: Object;
} | {
dpKey: Object;
};
export interface EdgeBundlingStageData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies the layout data for the {@link EdgeBundlingStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundlingStageData Online Documentation}*/
export class EdgeBundlingStageData {
constructor();
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundlingStageData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
Gets or sets the mapping of edges to their {@link EdgeBundleDescriptor}.
Bundling together multiple edges means that their common parts are to some degree merged into a bundled part. At the
source and target point, the edges are again clearly split.
If an edge is mapped to <code>null</code>, the {@link EdgeBundling#defaultBundleDescriptor default descriptor} is used.
@see EdgeBundleDescriptor
@see EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY
@see EdgeBundling
@see {@link https://docs.yworks.com/yfileshtml/#/api/EdgeBundlingStageData%23edgeBundleDescriptors Online Documentation}*/
edgeBundleDescriptors: ItemMapping<IEdge, EdgeBundleDescriptor>;
static $class: Class;
private $_EdgeBundlingStageData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EdgeBundlingStageData;
}
export interface ExtendedEdgeLabelCandidate extends EdgeLabelCandidate {
}
/**
An {@link EdgeLabelCandidate} that provides a {@link ILabelCandidateDescriptor candidate descriptor}.
This class is used by {@link LayoutExecutor} and {@link LayoutGraphAdapter} to convert an {@link IGraph}
into a corresponding
{@link Graph}. Unless that conversion needs to be customized, or aforementioned classes cannot be used, this
class usually doesn't need to be used explicitly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedEdgeLabelCandidate Online Documentation}*/
export class ExtendedEdgeLabelCandidate {
/**
Returns a new instance of {@link ExtendedEdgeLabelCandidate}.
@param pos the location of the upper left corner of the candidate.
@param size the size of the candidate.
@param param the parameters of the labeling model associated with this candidate.
@param owner the label associated to the candidate.
@param internal flag whether the candidate is allowed to overlap the layout of the edge.
@param descriptor the label candidate descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedEdgeLabelCandidate%23ExtendedEdgeLabelCandidate(YPoint%2CYDimension%2CObject%2CIEdgeLabelLayout%2Cboolean%2CILabelCandidateDescriptor) Online Documentation}*/
constructor(pos: YPoint, size: YDimension, param: any, owner: IEdgeLabelLayout, internal: boolean, descriptor: ILabelCandidateDescriptor | null);
/**
Returns a new instance of {@link ExtendedEdgeLabelCandidate}.
@param pos the location of the upper left corner of the candidate.
@param size the size of the candidate.
@param param the parameters of the labeling model associated with this candidate.
@param owner the label associated to the candidate.
@param descriptor the label candidate descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedEdgeLabelCandidate%23ExtendedEdgeLabelCandidate(YPoint%2CYDimension%2CObject%2CIEdgeLabelLayout%2CILabelCandidateDescriptor) Online Documentation}*/
constructor(pos: YPoint, size: YDimension, param: any, owner: IEdgeLabelLayout, descriptor: ILabelCandidateDescriptor | null);
/**
Returns a new instance of {@link ExtendedEdgeLabelCandidate}.
@param orientedBox the label size and orientation
@param param the parameters of the labeling model associated with this candidate.
@param owner the label associated to the candidate.
@param internal flag whether the candidate is allowed to overlap the layout of the edge.
@param descriptor the label candidate descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedEdgeLabelCandidate%23ExtendedEdgeLabelCandidate(YOrientedRectangle%2CObject%2CIEdgeLabelLayout%2Cboolean%2CILabelCandidateDescriptor) Online Documentation}*/
constructor(orientedBox: YOrientedRectangle, param: any, owner: IEdgeLabelLayout, internal: boolean, descriptor: ILabelCandidateDescriptor | null);
/**
Gets the label candidate descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedEdgeLabelCandidate%23descriptor Online Documentation}*/
descriptor: ILabelCandidateDescriptor | null;
static $class: Class;
private $_ExtendedEdgeLabelCandidate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ExtendedEdgeLabelCandidate;
}
export interface ExtendedLabelCandidateProfitModel extends YObject, IProfitModel {
}
/**
An {@link IProfitModel} that returns the profit of a label candidate for instances of {@link ExtendedNodeLabelCandidate}
and {@link ExtendedEdgeLabelCandidate}.
This class is used by {@link LayoutExecutor} and {@link LayoutGraphAdapter} to convert an {@link IGraph}
into a corresponding
{@link Graph}. Unless that conversion needs to be customized, or aforementioned classes cannot be used, this
class usually doesn't need to be used explicitly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedLabelCandidateProfitModel Online Documentation}*/
export class ExtendedLabelCandidateProfitModel {
/**
Returns the profit for placing a label-candidate.
This implementation returns the profit for instances of {@link ExtendedNodeLabelCandidate} or 0.0 for other
{@link LabelCandidate}s.
@param candidate a label candidate
@returns a value between 0 and 1.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedLabelCandidateProfitModel%23getProfit(LabelCandidate) Online Documentation}*/
getProfit(candidate: LabelCandidate | null): number;
static $class: Class;
private $_ExtendedLabelCandidateProfitModel: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ExtendedLabelCandidateProfitModel;
}
export interface ExtendedNodeLabelCandidate extends NodeLabelCandidate {
}
/**
A {@link NodeLabelCandidate} that provides a {@link ILabelCandidateDescriptor candidate descriptor}.
This class is used by {@link LayoutExecutor} and {@link LayoutGraphAdapter} to convert an {@link IGraph}
into a corresponding
{@link Graph}. Unless that conversion needs to be customized, or aforementioned classes cannot be used, this
class usually doesn't need to be used explicitly.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedNodeLabelCandidate Online Documentation}*/
export class ExtendedNodeLabelCandidate {
/**
Returns a new instance of {@link ExtendedNodeLabelCandidate}.
@param pos the location of the upper left corner of the candidate.
@param size the size of the candidate.
@param param the parameters of the labeling model associated with this candidate.
@param owner the label associated to the candidate.
@param internal flag whether the candidate is inside the node, or outside.
@param descriptor the label candidate descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedNodeLabelCandidate%23ExtendedNodeLabelCandidate(YPoint%2CYDimension%2CObject%2CINodeLabelLayout%2Cboolean%2CILabelCandidateDescriptor) Online Documentation}*/
constructor(pos: YPoint, size: YDimension, param: any, owner: INodeLabelLayout, internal: boolean, descriptor: ILabelCandidateDescriptor | null);
/**
Returns a new instance of {@link ExtendedNodeLabelCandidate}.
@param pos the location of the upper left corner of the candidate.
@param size the size of the candidate.
@param param the parameters of the labeling model associated with this candidate.
@param owner the label associated to the candidate.
@param descriptor the label candidate descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedNodeLabelCandidate%23ExtendedNodeLabelCandidate(YPoint%2CYDimension%2CObject%2CINodeLabelLayout%2CILabelCandidateDescriptor) Online Documentation}*/
constructor(pos: YPoint, size: YDimension, param: any, owner: INodeLabelLayout, descriptor: ILabelCandidateDescriptor | null);
/**
Returns a new instance of {@link ExtendedNodeLabelCandidate}.
@param orientedBox the label size and orientation
@param param the parameters of the labeling model associated with this candidate.
@param owner the label associated to the candidate.
@param internal flag whether the candidate is inside the node, or outside.
@param descriptor the label candidate descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedNodeLabelCandidate%23ExtendedNodeLabelCandidate(YOrientedRectangle%2CObject%2CINodeLabelLayout%2Cboolean%2CILabelCandidateDescriptor) Online Documentation}*/
constructor(orientedBox: YOrientedRectangle, param: any, owner: INodeLabelLayout, internal: boolean, descriptor: ILabelCandidateDescriptor | null);
/**
Gets the label candidate descriptor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ExtendedNodeLabelCandidate%23descriptor Online Documentation}*/
descriptor: ILabelCandidateDescriptor | null;
static $class: Class;
private $_ExtendedNodeLabelCandidate: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ExtendedNodeLabelCandidate;
}
export interface FamilyTreeLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link FamilyTreeLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayoutData Online Documentation}*/
export class FamilyTreeLayoutData {
/**
Creates a new instance of {@link FamilyTreeLayoutData} which helps configuring
{@link FamilyTreeLayout}.
@param [options] The parameters to pass.
@param [options.familyTypes] The mapping from nodes to their family type. This option sets the {@link FamilyTreeLayoutData#familyTypes} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayoutData Online Documentation}*/
constructor(options?: {
familyTypes?: ItemMapping<INode, FamilyType> | ItemMappingConvertible<INode, FamilyType>;
});
/**
Gets or sets the mapping from nodes to their family type.
The family type can be {@link FamilyType#FEMALE} for nodes that represent women, {@link FamilyType#MALE}
for nodes that represent men, {@link FamilyType#FAMILY} for nodes that connect partners with each other and
their children or <code>null</code> for other nodes
@see FamilyTreeLayout#FAMILY_TYPE_DP_KEY
@see FamilyType
@see {@link https://docs.yworks.com/yfileshtml/#/api/FamilyTreeLayoutData%23familyTypes Online Documentation}*/
familyTypes: ItemMapping<INode, FamilyType>;
static $class: Class;
private $_FamilyTreeLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FamilyTreeLayoutData;
}
export interface FillAreaLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link FillAreaLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayoutData Online Documentation}*/
export class FillAreaLayoutData {
/**
Creates a new instance of {@link FillAreaLayoutData}.
@param [options] The parameters to pass.
@param [options.fixedNodes] The collection of nodes that are fixed and must not be moved. This option sets the {@link FillAreaLayoutData#fixedNodes} property on the created object.
@param [options.componentIds] The mapping from nodes to an object defining their component id. This option sets the {@link FillAreaLayoutData#componentIds} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link FillAreaLayoutData#nodeHalos} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link FillAreaLayoutData#abortHandler} property on the created object.
@param [options.partitionGridData] The {@link PartitionGrid partition grid} layout data. This option sets the {@link FillAreaLayoutData#partitionGridData} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayoutData Online Documentation}*/
constructor(options?: {
fixedNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
componentIds?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
abortHandler?: AbortHandler | null;
partitionGridData?: PartitionGridData;
});
/**
Gets or sets the collection of nodes that are fixed and must not be moved.
<h2>Note</h2>
The algorithm tries to preserve the mental map and, thus, non-fixed nodes cannot simply pass/jump over the fixed nodes
(i.e. fixed nodes restrict the possible movement of non-fixed nodes).
@see FillAreaLayout#FIXED_NODE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayoutData%23fixedNodes Online Documentation}*/
fixedNodes: ItemCollection<INode>;
/**
Gets or sets the mapping from nodes to an object defining their component id.
Nodes of the same component should preferably not be separated. While the algorithm may move a whole component, it
should tries to not move only a subset of its elements. This means that the algorithm tries to move all elements of a
component by the same offset (if at all). In order to achieve good results with this feature, the different components
should not overlap in the initial drawing.
<b>Warning:</b> The specified custom components are only considered when property {@link FillAreaLayout#componentAssignmentStrategy} is set to
{@link ComponentAssignmentStrategy#CUSTOMIZED}.
<b>Warning:</b> If the input graph contains group nodes or a {@link PartitionGrid} structure, maintaining the specified
components is not always possible. Furthermore, a custom component can never pass the border of a group node or
partition cell, i.e., all nodes of a custom component have to be associated with the same group or grid cell.
@see FillAreaLayout#COMPONENT_ID_DP_KEY
@see ComponentAssignmentStrategy#CUSTOMIZED
@see FillAreaLayout#componentAssignmentStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayoutData%23componentIds Online Documentation}*/
componentIds: ItemMapping<INode, any>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
Gets or sets the {@link PartitionGrid partition grid} layout data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayoutData%23partitionGridData Online Documentation}*/
partitionGridData: PartitionGridData;
static $class: Class;
private $_FillAreaLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FillAreaLayoutData;
}
export interface FixGroupLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link FixGroupLayoutStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutData Online Documentation}*/
export class FixGroupLayoutData {
/**
Creates a new instance of {@link FixGroupLayoutData} which helps configuring {@link FixGroupLayoutStage}.
@param [options] The parameters to pass.
@param [options.fixedGroupNodes] The collection of group nodes whose contents should keep their relative positions. This option sets the {@link FixGroupLayoutData#fixedGroupNodes} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link FixGroupLayoutData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link FixGroupLayoutData#targetPortConstraints} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutData Online Documentation}*/
constructor(options?: {
fixedGroupNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
});
/**
Gets or sets the collection of group nodes whose contents should keep their relative positions.
@see FixGroupLayoutStage#FIXED_GROUP_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutData%23fixedGroupNodes Online Documentation}*/
fixedGroupNodes: ItemCollection<INode>;
/**
Gets or sets a mapping from edges to their source {@link PortConstraint}.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutData%23sourcePortConstraints Online Documentation}*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to their target {@link PortConstraint}.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixGroupLayoutData%23targetPortConstraints Online Documentation}*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
static $class: Class;
private $_FixGroupLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FixGroupLayoutData;
}
export interface FixNodeLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link FixNodeLayoutStage}.
* The following example shows how to create a new instance of {@link FixNodeLayoutData} and use it with an
* {@link FixNodeLayoutStage}:
*
* <br><b>Creating an instance of FixNodeLayoutData</b>
* <pre>
* // Create layout data for both the FixNodeLayoutStage and the layout that's actually being run
*
* // first create and configure the actual layout data object
* const organicLayoutData = new OrganicLayoutData()
* // Configure the organic layout data object ...
*
* const fixNodeLayoutData = new FixNodeLayoutData()
* fixNodeLayoutData.fixedNodes.source = graphComponent.selection.selectedNodes
*
* // Wrap both into a single layout data
* const layoutData = fixNodeLayoutData.combineWith(organicLayoutData)
*
* // Run the layout
* graphComponent.graph.applyLayout(new FixNodeLayoutStage(new OrganicLayout()), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link FixNodeLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of FixNodeLayoutData</b>
* <pre>
* // Create layout data for both the FixNodeLayoutStage and the layout that's actually being run
* const fixNodeLayoutData = new FixNodeLayoutData({
*   fixedNodes: graphComponent.selection.selectedNodes
* })
* const organicLayoutData = new OrganicLayoutData({
*   // Configure the other layout data object ...
* })
*
* // Wrap both into a CompositeLayoutData
* const layoutData = new CompositeLayoutData(fixNodeLayoutData, organicLayoutData)
*
* // Run the layout
* graphComponent.graph.applyLayout(new FixNodeLayoutStage(new OrganicLayout()), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutData Online Documentation}
*/
export class FixNodeLayoutData {
/**
Creates a new instance of {@link FixNodeLayoutData} which helps configuring {@link FixNodeLayoutStage}.
@param [options] The parameters to pass.
@param [options.fixedNodes] The collection of nodes which should remain fixed. This option sets the {@link FixNodeLayoutData#fixedNodes} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutData Online Documentation}*/
constructor(options?: {
fixedNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
});
/**
* Gets or sets the collection of nodes which should remain fixed.
* If only a single node is fixed, then it will retain its exact coordinates, while the graph around it will be changed
* accordingly by the {@link LayoutStageBase#coreLayout layout algorithm}. If multiple nodes are fixed, then a certain <em>fix point</em> between them will retain its location,
* even if the individual nodes are moved by the layout algorithm.
*
* If only a single node should be fixed, then the easiest option is {@link ItemCollection#item}:
*
* <br><b>Setting a single node to be fixed</b>
* <pre>
* layoutData.fixedNodes.item = node
* </pre>
*
* In case multiple nodes should be fixed and they're already in a collection or {@link IEnumerable}, the {@link ItemCollection#source} option is usually the most
* convenient:
*
* <br><b>Using a collection of fixed nodes</b>
* <pre>
* layoutData.fixedNodes.source = graphComponent.selection.selectedNodes
* </pre>
*
* If they're not already in a collection or need to be collected one at a time, there's also the {@link ItemCollection#items} property, which holds a
* collection that can be filled:
*
* <br><b>Adding fixed nodes one by one</b>
* <pre>
* layoutData.fixedNodes.items.add(node1)
* layoutData.fixedNodes.items.add(node2)
* </pre>
*
* And finally, if it's easy to determine from each node whether it should be fixed or not, then the {@link ItemCollection#delegate} option tends to be
* the easiest to use:
*
* <br><b>Using a delegate to decide for each node whether it should be fixed</b>
* <pre>
* layoutData.fixedNodes.delegate = node => node.tag.isFixed
* </pre>
*
* @see FixNodeLayoutStage#FIXED_NODE_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/FixNodeLayoutData%23fixedNodes Online Documentation}
*/
fixedNodes: ItemCollection<INode>;
static $class: Class;
private $_FixNodeLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FixNodeLayoutData;
}
export interface FixPortLocationStageData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom layout data for the {@link FixPortLocationStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPortLocationStageData Online Documentation}*/
export class FixPortLocationStageData {
/**
Creates a new instance of {@link FixPortLocationStageData} which helps configuring
{@link FixPortLocationStage}.
@param [options] The parameters to pass.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link FixPortLocationStageData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link FixPortLocationStageData#targetPortConstraints} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPortLocationStageData Online Documentation}*/
constructor(options?: {
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
});
/**
Gets or sets a mapping from edges to their source {@link PortConstraint}.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPortLocationStageData%23sourcePortConstraints Online Documentation}*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to their target {@link PortConstraint}.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FixPortLocationStageData%23targetPortConstraints Online Documentation}*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
static $class: Class;
private $_FixPortLocationStageData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FixPortLocationStageData;
}
export interface GenericPartitionGridStageData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies the layout data for the {@link GenericPartitionGridStage}.
* The following example shows how to create a new instance of {@link GenericPartitionGridStageData} and use it with
* a {@link GenericPartitionGridStage}:
*
* <br><b>Creating an instance of GenericPartitionGridStageData</b>
* <pre>
* const layoutData = new GenericPartitionGridStageData()
* // Define partition grid from custom data
* layoutData.partitionGridData.rowIndices.delegate = node => node.tag.rowIndex
* layoutData.partitionGridData.columnIndices.delegate = node => node.tag.columnIndex
* // Define split edges from custom data
* layoutData.splitEdges.delegate = edge => edge.tag.isSplitEdge
*
* // Combine the layout data of the stage and the core layout
* const compositeLayoutData = new CompositeLayoutData(coreLayoutData, layoutData)
*
* graph.applyLayout(new GivenCoordinatesStage(coreLayout), compositeLayoutData)
* </pre>
*
* In many cases the complete initialization of {@link GenericPartitionGridStageData} can also be done in a single
* object initializer:
*
* <br><b>Using an object initializer for easy creation of GenericPartitionGridStageData</b>
* <pre>
* const layoutData = new GenericPartitionGridStageData({
*   // Define partition grid from custom data
*   partitionGridData: {
*     rowIndices: node => node.tag.rowIndex,
*     columnIndices: node => node.tag.columnIndex
*   },
*   // Define split edges from custom data
*   splitEdges: edge => edge.tag.isSplitEdge
* })
*
* const compositeLayoutData = new CompositeLayoutData(coreLayoutData, layoutData)
* graph.applyLayout(new GenericPartitionGridStage(coreLayout), compositeLayoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStageData Online Documentation}
*/
export class GenericPartitionGridStageData {
/**
Creates a new instance of {@link GenericPartitionGridStageData} which helps configuring
{@link GenericPartitionGridStage}.
@param [options] The parameters to pass.
@param [options.partitionGridData] The {@link GenericPartitionGridStageData#partitionGridData} that allows to configure the {@link PartitionGrid} for the layout stage. This option sets the {@link GenericPartitionGridStageData#partitionGridData} property on the created object.
@param [options.splitEdges] The collection of edges that should not be considered when calculating the sub-components. This option sets the {@link GenericPartitionGridStageData#splitEdges} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStageData Online Documentation}*/
constructor(options?: {
partitionGridData?: PartitionGridData;
splitEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
});
/**
* Gets or sets the {@link GenericPartitionGridStageData#partitionGridData} that allows to configure the {@link PartitionGrid} for the layout stage.
* The following sample shows how to assign nodes to partition grid cells simply via cell indices:
*
* <br><b>Assigning nodes to partition grid cells via indices</b>
* <pre>
* // Create four nodes and place them in grid cells in the following way
* // +--+--+--+
* // | 1| 2| 3|
* // +--+--+--+
* //    | 4|
* //    +--+
*
* const gridData = layoutData.partitionGridData
*
* const node1 = graph.createNode()
* const node2 = graph.createNode()
* const node3 = graph.createNode()
* const node4 = graph.createNode()
*
* // Assign the nodes to their rows and columns.
* // Note that don't have to create or use PartitionGrid directly in this case.
* // Setting the indices is enough.
* gridData.rowIndices.mapper.set(node1, 0)
* gridData.rowIndices.mapper.set(node2, 0)
* gridData.rowIndices.mapper.set(node3, 0)
* gridData.rowIndices.mapper.set(node4, 1)
* gridData.columnIndices.mapper.set(node1, 0)
* gridData.columnIndices.mapper.set(node2, 1)
* gridData.columnIndices.mapper.set(node3, 2)
* gridData.columnIndices.mapper.set(node4, 1)
*
* const compositeLayoutData = new CompositeLayoutData(coreLayoutData, layoutData)
*
* graph.applyLayout(new GenericPartitionGridStage(coreLayout), compositeLayoutData)
* </pre>
*
* When used this way there is no need to create a {@link PartitionGrid} instance or work with it directly. For more
* flexibility, e.g. to use cells that span multiple columns or rows, the {@link PartitionGrid} can be used as well:
*
* <br><b>Assigning nodes to partition grid cells via factory methods on the grid</b>
* <pre>
* // Create three nodes and place them in grid cells in the following way
* // +--+--+
* // | 1| 2|
* // +--+--+
* // |  3  |
* // +--+--+
* const gridData = layoutData.partitionGridData
* const node1 = graph.createNode(new Rect(0, 0, 50, 50))
* const node2 = graph.createNode(new Rect(0, 0, 50, 50))
* const node3 = graph.createNode(new Rect(0, 0, 125, 50))
* // Create a new PartitionGrid with two rows and two columns
* const grid = new PartitionGrid(2, 2)
* gridData.grid = grid
* // Assign the nodes to their cells
* gridData.cellIds.mapper.set(node1, grid.createCellId(0, 0))
* gridData.cellIds.mapper.set(node2, grid.createCellId(0, 1))
* gridData.cellIds.mapper.set(node3, grid.createRowSpanId(1))
*
* graph.applyLayout(
*   new GenericPartitionGridStage(coreLayout),
*   new CompositeLayoutData(coreLayoutData, layoutData)
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStageData%23partitionGridData Online Documentation}
*/
partitionGridData: PartitionGridData;
/**
* Gets or sets the collection of edges that should not be considered when calculating the sub-components.
* This property is only considered if property {@link GenericPartitionGridStage#splitMultiCellComponents} is disabled. The split-edges are temporarily removed during the
* calculation of the sub-components and, finally, routed with the specified
* {@link GenericPartitionGridStage#interEdgeRouter}. This allows to control how the graph is split into smaller graph components. Recall that with this setting a component
* may contain nodes of different partition cells.
*
* If only a few edges should be removed during the calculation of the sub-components, the easiest way is often to add them
* one by one via the {@link ItemCollection#items} collection:
*
* <pre>
* layoutData.splitEdges.items.add(edge1)
* layoutData.splitEdges.items.add(edge2)
* </pre>
*
* If you already have a collection or {@link IEnumerable} for those edges, the {@link ItemCollection#source} property is usually easier:
*
* <pre>
* // Consider selected edges
* layoutData.splitEdges.source = graphComponent.selection.selectedEdges
* </pre>
*
* If it can be inferred directly from the edge itself whether it should be routed that way, a delegate is often most
* convenient:
*
* <pre>
* layoutData.splitEdges.delegate = edge => edge.tag.isSplitEdge
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GenericPartitionGridStageData%23splitEdges Online Documentation}
*/
splitEdges: ItemCollection<IEdge>;
static $class: Class;
private $_GenericPartitionGridStageData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GenericPartitionGridStageData;
}
export interface GivenCoordinatesStageData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom layout data for the {@link GivenCoordinatesStage}.
* The following example shows how to create a new instance of {@link GivenCoordinatesStageData} and use it with a
* {@link GivenCoordinatesStage}:
*
* <br><b>Creating an instance of GivenCoordinatesStageData</b>
* <pre>
* const layoutData = new GivenCoordinatesStageData()
* // Resize all nodes to same size
* layoutData.nodeSizes.constant = new Size(30, 30)
* // Reset all edge paths
* layoutData.edgePaths.constant = IEnumerable.EMPTY
*
* // Combine the layout data of the stage and the core layout
* const compositeLayoutData = new CompositeLayoutData(coreLayoutData, layoutData)
*
* graphComponent.graph.applyLayout(new GivenCoordinatesStage(coreLayout), compositeLayoutData)
* </pre>
*
* In many cases the complete initialization of {@link GivenCoordinatesStageData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of GivenCoordinatesStageData</b>
* <pre>
* const layoutData = new GivenCoordinatesStageData({
*   // Resize all nodes to same size
*   nodeSizes: new Size(30, 30),
*   // Reset all edge paths
*   edgePaths: IListEnumerable.EMPTY
* })
*
* // Combine the layout data of the stage and the core layout
* const compositeLayoutData = new CompositeLayoutData(coreLayoutData, layoutData)
*
* graphComponent.graph.applyLayout(new GivenCoordinatesStage(coreLayout), compositeLayoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStageData Online Documentation}
*/
export class GivenCoordinatesStageData {
/**
Creates a new instance of {@link GivenCoordinatesStageData} which helps configuring
{@link GivenCoordinatesStage}.
@param [options] The parameters to pass.
@param [options.nodeLocations] The mapping from nodes to their initial location. This option sets the {@link GivenCoordinatesStageData#nodeLocations} property on the created object.
@param [options.nodeSizes] The mapping from nodes to their sizes. This option sets the {@link GivenCoordinatesStageData#nodeSizes} property on the created object.
@param [options.edgePaths] A mapping from edges to their initial edge paths. This option sets the {@link GivenCoordinatesStageData#edgePaths} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStageData Online Documentation}*/
constructor(options?: {
nodeLocations?: ItemMapping<INode, IPoint> | ItemMappingConvertible<INode, IPoint>;
nodeSizes?: ItemMapping<INode, ISize> | ItemMappingConvertible<INode, ISize>;
edgePaths?: ItemMapping<IEdge, IEnumerable<IPoint>> | ItemMappingConvertible<IEdge, IEnumerable<IPoint>>;
});
/**
* Gets or sets the mapping from nodes to their initial location.
* Before calling the core layout algorithm, the {@link GivenCoordinatesStage} places the top-left corner of each
* node on the specified point location. Nodes that do not have a mapping are not changed by the stage.
*
* The easiest option is to place all nodes on the same location, by setting a constant location:
*
* <br><b>Using the same location for all nodes</b>
* <pre>
* // place all nodes to the center of the GraphComponent
* layoutData.nodeLocations.constant = graphComponent.viewport.center
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set the locations for certain nodes</b>
* <pre>
* // node1 get the initial location (10, 20)
* layoutData.nodeLocations.mapper.set(node1, new Point(10, 20))
* // and node2 (50, 70)
* layoutData.nodeLocations.mapper.set(node2, new Point(50, 70))
* // all other nodes don't change their initial locations
* </pre>
*
* In cases where the location for each node can be determined by looking at the node itself it's often easier to just set
* a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine the locations for all nodes</b>
* <pre>
* // Align all nodes horizontally to x = 0
* layoutData.nodeLocations.delegate = node => new Point(0, node.layout.y)
* </pre>
*
* @see GivenCoordinatesStage#NODE_LOCATION_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStageData%23nodeLocations Online Documentation}
*/
nodeLocations: ItemMapping<INode, IPoint>;
/**
* Gets or sets the mapping from nodes to their sizes.
* Before calling the core layout algorithm, the {@link GivenCoordinatesStage} sets the size of each node to the
* specified dimension. The size of nodes without associated value is not changed by the stage.
*
* The easiest option is to resize all nodes to the same dimension, by setting a constant size:
*
* <br><b>Using the same dimension for all nodes</b>
* <pre>
* // resize all nodes to (30, 30)
* layoutData.nodeSizes.constant = new Size(30, 30)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set the dimension for certain nodes</b>
* <pre>
* // node1 get the size (10, 20)
* layoutData.nodeSizes.mapper.set(node1, new Size(10, 20))
* // and node2 (50, 70)
* layoutData.nodeSizes.mapper.set(node2, new Size(50, 70))
* // all other nodes don't change their dimensions
* </pre>
*
* In cases where the dimension for each node can be determined by looking at the node itself it's often easier to just set
* a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine the dimension for all nodes</b>
* <pre>
* // all nodes grow by 50%
* layoutData.nodeSizes.delegate = node => node.layout.toSize() * 1.5
* </pre>
*
* @see GivenCoordinatesStage#NODE_SIZE_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStageData%23nodeSizes Online Documentation}
*/
nodeSizes: ItemMapping<INode, ISize>;
/**
* Gets or sets a mapping from edges to their initial edge paths.
* Before calling the core layout algorithm, the {@link GivenCoordinatesStage} sets the path of each edge to the
* mapped point path. The path must contain the source port location, followed by the bends (if any), followed by the
* target port location.
*
* If an empty path is specified, the current path will be reset (all bends are cleared, the current source/target port
* locations are kept). The path of edges that have no associated path are not changed by the stage.
*
* The easiest option is to reset all paths, by setting a constant empty path:
*
* <br><b>Using the same path for all edges</b>
* <pre>
* // reset all edge paths
* layoutData.edgePaths.constant = IEnumerable.EMPTY
* </pre>
*
* Handling only certain edges differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set the path for certain edges</b>
* <pre>
* // reset the edge paths of edge1
* layoutData.edgePaths.mapper.set(edge1, IEnumerable.EMPTY)
* // and edge2
* layoutData.edgePaths.mapper.set(edge2, IEnumerable.EMPTY)
* // all other edges don't change their paths
* </pre>
*
* In cases where the path for each edge can be determined by looking at the edge itself it's often easier to just set a
* delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine the path for all edge</b>
* <pre>
* // make all edge orthogonal
* layoutData.edgePaths.delegate = edge => {
*   const sourceLocation = edge.sourcePort.location
*   const targetLocation = edge.targetPort.location
*   return List.fromArray([
*     sourceLocation,
*     new Point(sourceLocation.x, targetLocation.y),
*     targetLocation
*   ])
* }
* </pre>
*
* @see GivenCoordinatesStage#EDGE_PATH_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/GivenCoordinatesStageData%23edgePaths Online Documentation}
*/
edgePaths: ItemMapping<IEdge, IEnumerable<IPoint>>;
static $class: Class;
private $_GivenCoordinatesStageData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is GivenCoordinatesStageData;
}
export interface HierarchicLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link HierarchicLayout}.
* The following example shows how to create a new instance of {@link HierarchicLayoutData} and use it with an
* {@link HierarchicLayout}:
*
* <br><b>Creating an instance of HierarchicLayoutData</b>
* <pre>
* const layoutData = new HierarchicLayoutData()
*
* // group all edges on their source side
* layoutData.sourceGroupIds.delegate = edge => edge.sourceNode
* // Use the stroke width of the style as edge thickness
* layoutData.edgeThickness.delegate = edge => edge.style.stroke.thickness
*
* graph.applyLayout(new HierarchicLayout(), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link HierarchicLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of HierarchicLayoutData</b>
* <pre>
* const layoutData = new HierarchicLayoutData({
*   // group all edges on their source side
*   sourceGroupIds: edge => edge.sourceNode,
*   // Use the stroke width of the style as edge thickness
*   edgeThickness: edge => edge.style.stroke.thickness
* })
*
* graph.applyLayout(new HierarchicLayout(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData Online Documentation}
*/
export class HierarchicLayoutData {
/**
Creates a new instance of {@link HierarchicLayoutData} which helps configuring
{@link HierarchicLayout}.
@param [options] The parameters to pass.
@param [options.alternativeEdgePaths] A mapping from edges to alternative paths for edges connecting to groups, group content or folder nodes. This option sets the {@link HierarchicLayoutData#alternativeEdgePaths} property on the created object.
@param [options.alternativeGroupBounds] A mapping from group/folder nodes to alternative bounds for these nodes. This option sets the {@link HierarchicLayoutData#alternativeGroupBounds} property on the created object.
@param [options.criticalEdgePriorities] A mapping from edges to their priority to be a 'critical' edge. This option sets the {@link HierarchicLayoutData#criticalEdgePriorities} property on the created object.
@param [options.reduceCriticalEdgeCrossings] Whether or not {@link HierarchicLayoutData#criticalEdgePriorities critical edges} automatically get {@link HierarchicLayoutData#edgeCrossingCosts crossing costs} assigned based on their critical edge priorities. This option sets the {@link HierarchicLayoutData#reduceCriticalEdgeCrossings} property on the created object.
@param [options.folderNodes] The collection of folder nodes used for recursive edge styles in incremental mode. This option sets the {@link HierarchicLayoutData#folderNodes} property on the created object.
@param [options.edgeLayoutDescriptors] The mapping of edges to their {@link HierarchicLayoutEdgeLayoutDescriptor}. This option sets the {@link HierarchicLayoutData#edgeLayoutDescriptors} property on the created object.
@param [options.nodeLayoutDescriptors] The mapping of nodes to their {@link HierarchicLayoutNodeLayoutDescriptor} This option sets the {@link HierarchicLayoutData#nodeLayoutDescriptors} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link HierarchicLayoutData#nodeHalos} property on the created object.
@param [options.subComponents] The mapping for sub-components of the input graph to a custom layout algorithm responsible for the arrangement of the sub-component. This option sets the {@link HierarchicLayoutData#subComponents} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link HierarchicLayoutData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link HierarchicLayoutData#targetPortConstraints} property on the created object.
@param [options.sourcePortCandidates] A mapping from edges to a collection of their source {@link PortCandidate port candidates}. This option sets the {@link HierarchicLayoutData#sourcePortCandidates} property on the created object.
@param [options.targetPortCandidates] A mapping from edges to a collection of their target port {@link PortCandidate candidates}. This option sets the {@link HierarchicLayoutData#targetPortCandidates} property on the created object.
@param [options.nodePortCandidateSets] A mapping from nodes to their {@link PortCandidateSet}. This option sets the {@link HierarchicLayoutData#nodePortCandidateSets} property on the created object.
@param [options.sourceGroupIds] A mapping from edges to an object representing their source edge group. This option sets the {@link HierarchicLayoutData#sourceGroupIds} property on the created object.
@param [options.sourcePortGroupIds] A mapping from edges to an object representing their source port group. This option sets the {@link HierarchicLayoutData#sourcePortGroupIds} property on the created object.
@param [options.targetGroupIds] A mapping from edges to an object representing their target edge group. This option sets the {@link HierarchicLayoutData#targetGroupIds} property on the created object.
@param [options.targetPortGroupIds] A mapping from edges to an object representing their target port group. This option sets the {@link HierarchicLayoutData#targetPortGroupIds} property on the created object.
@param [options.buses] The mapping from edges to their {@link HierarchicLayoutBusDescriptor bus descriptor} which defines the bus structure they belong to. This option sets the {@link HierarchicLayoutData#buses} property on the created object.
@param [options.busRootOffsets] The mapping from nodes to their layer index relative to the root node of the {@link HierarchicLayoutData#buses bus} they belong to. This option sets the {@link HierarchicLayoutData#busRootOffsets} property on the created object.
@param [options.nodesBeforeBus] The collection of {@link HierarchicLayoutData#buses bus} nodes that should be placed before the common bus segment. This option sets the {@link HierarchicLayoutData#nodesBeforeBus} property on the created object.
@param [options.layerConstraints] The layout data to specify layer constraints. This option sets the {@link HierarchicLayoutData#layerConstraints} property on the created object.
@param [options.sequenceConstraints] The layout data to specify sequence constraints. This option sets the {@link HierarchicLayoutData#sequenceConstraints} property on the created object.
@param [options.layerConstraintFactory] The factory to specify layer constraints. This option sets the {@link HierarchicLayoutData#layerConstraintFactory} property on the created object.
@param [options.sequenceConstraintFactory] The factory to specify sequence constraints. This option sets the {@link HierarchicLayoutData#sequenceConstraintFactory} property on the created object.
@param [options.incrementalHints] A mapping from nodes and edges to their incremental hints. This option sets the {@link HierarchicLayoutData#incrementalHints} property on the created object.
@param [options.layerIndices] A mapper from nodes to the index of their layer. This option sets the {@link HierarchicLayoutData#layerIndices} property on the created object.
@param [options.sequenceIndices] A mapper from nodes to the sequence index in their layer. This option sets the {@link HierarchicLayoutData#sequenceIndices} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link HierarchicLayoutData#abortHandler} property on the created object.
@param [options.partitionGridData] The {@link HierarchicLayoutData#partitionGridData partition grid} layout data. This option sets the {@link HierarchicLayoutData#partitionGridData} property on the created object.
@param [options.selfLoopCalculatorData] The layout data for the {@link SelfLoopCalculator}. This option sets the {@link HierarchicLayoutData#selfLoopCalculatorData} property on the created object.
@param [options.constraintIncrementalLayererAdditionalEdgeWeights] The mapping from edges to an additional weight used by the {@link ConstraintIncrementalLayerer}. This option sets the {@link HierarchicLayoutData#constraintIncrementalLayererAdditionalEdgeWeights} property on the created object.
@param [options.bfsLayererCoreNodes] The collection of core nodes used by the {@link BFSLayerer}. This option sets the {@link HierarchicLayoutData#bfsLayererCoreNodes} property on the created object.
@param [options.givenLayersLayererIds] The mapping from nodes to their layer index when using the {@link GivenLayersLayerer}. This option sets the {@link HierarchicLayoutData#givenLayersLayererIds} property on the created object.
@param [options.edgeDirectedness] The mapping from edges to their directedness. This option sets the {@link HierarchicLayoutData#edgeDirectedness} property on the created object.
@param [options.edgeThickness] The mapping from edges to their thickness. This option sets the {@link HierarchicLayoutData#edgeThickness} property on the created object.
@param [options.edgeCrossingCosts] The mapping from edges to their crossing cost. This option sets the {@link HierarchicLayoutData#edgeCrossingCosts} property on the created object.
@param [options.groupBorderCrossingCosts] The mapping from group nodes to the costs for a crossing with the group node border. This option sets the {@link HierarchicLayoutData#groupBorderCrossingCosts} property on the created object.
@param [options.edgeLabelPreferredPlacement] The mapping that provides a {@link PreferredPlacementDescriptor} instance for edge {@link ILabel}s. This option sets the {@link HierarchicLayoutData#edgeLabelPreferredPlacement} property on the created object.
@param [options.uniformPortAssignmentGroups] The collection of group nodes for which the ports of the adjacent edges are uniformly distributed. This option sets the {@link HierarchicLayoutData#uniformPortAssignmentGroups} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData Online Documentation}*/
constructor(options?: {
alternativeEdgePaths?: ItemMapping<IEdge, IEnumerable<IPoint>> | ItemMappingConvertible<IEdge, IEnumerable<IPoint>>;
alternativeGroupBounds?: ItemMapping<INode, IRectangle> | ItemMappingConvertible<INode, IRectangle>;
criticalEdgePriorities?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
reduceCriticalEdgeCrossings?: boolean;
folderNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
edgeLayoutDescriptors?: ItemMapping<IEdge, HierarchicLayoutEdgeLayoutDescriptor> | ItemMappingConvertible<IEdge, HierarchicLayoutEdgeLayoutDescriptor>;
nodeLayoutDescriptors?: ItemMapping<INode, HierarchicLayoutNodeLayoutDescriptor> | ItemMappingConvertible<INode, HierarchicLayoutNodeLayoutDescriptor>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
subComponents?: ItemCollectionMapping<INode, ILayoutAlgorithm>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
sourcePortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
targetPortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
nodePortCandidateSets?: ItemMapping<INode, PortCandidateSet> | ItemMappingConvertible<INode, PortCandidateSet>;
sourceGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
sourcePortGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetPortGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
buses?: ItemCollectionMapping<IEdge, HierarchicLayoutBusDescriptor>;
busRootOffsets?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
nodesBeforeBus?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
layerConstraints?: LayerConstraintData;
sequenceConstraints?: SequenceConstraintData;
layerConstraintFactory?: ILayerConstraintFactory;
sequenceConstraintFactory?: ISequenceConstraintFactory;
incrementalHints?: IncrementalHintItemMapping | IncrementalHintItemMappingConvertible;
layerIndices?: IMapper<INode, number> | null | IMapperConvertible<INode, number>;
sequenceIndices?: IMapper<INode, number> | null | IMapperConvertible<INode, number>;
abortHandler?: AbortHandler | null;
partitionGridData?: PartitionGridData;
selfLoopCalculatorData?: SelfLoopCalculatorData;
constraintIncrementalLayererAdditionalEdgeWeights?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
bfsLayererCoreNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
givenLayersLayererIds?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
edgeThickness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
edgeCrossingCosts?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
groupBorderCrossingCosts?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
edgeLabelPreferredPlacement?: ItemMapping<ILabel, PreferredPlacementDescriptor> | ItemMappingConvertible<ILabel, PreferredPlacementDescriptor>;
uniformPortAssignmentGroups?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
});
/**
Gets or sets a mapping from edges to alternative paths for edges connecting to groups, group content or folder nodes.
When running in {@link LayoutMode#INCREMENTAL incremental}
{@link HierarchicLayout#layoutMode layout mode}, the alternative edge paths are considered during the routing of fixed (i.e., non-incremental) edges.
The alternative paths should be used in conjunction with {@link HierarchicLayout#ALTERNATIVE_GROUP_BOUNDS_DP_KEY alternative group bounds} to achieve more stable layout results
when collapsing/expanding a group node as follows:
<ol>
<li>Collapsing: edges adjacent to the group itself and edges where one of the endpoints (source/target) lies inside the
group should get the path <em>before</em> collapsing the group as alternative path. If both endpoints are either inside or outside the
group, no alternative path is required.
<li>Expanding: edges adjacent to the expanded folder node (which is now a group) should get the path <em>before</em> expanding as
alternative path.
</ol>
<b>Warning:</b> An edge path consists of the location of the source port, followed by the locations of the bends, followed by the
location of the target port as returned by {@link IEdge#getPathPoints}.
@see HierarchicLayout#ALTERNATIVE_EDGE_PATH_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23alternativeEdgePaths Online Documentation}*/
alternativeEdgePaths: ItemMapping<IEdge, IEnumerable<IPoint>>;
/**
Gets or sets a mapping from group/folder nodes to alternative bounds for these nodes.
When running in {@link LayoutMode#INCREMENTAL incremental}
{@link HierarchicLayout#layoutMode layout mode}, the alternative bounds of the collapsed/expanded group will be used during the layering and sequencing
phase of the algorithm.
<h2>Note</h2>
To further improve the results you should also specify {@link HierarchicLayoutData#alternativeEdgePaths alternative edge paths} for edges adjacent to such groups. In
addition, {@link HierarchicLayout#recursiveGroupLayering recursive group layering} should be enabled and {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} should be set to
{@link RecursiveEdgeStyle#DIRECTED}. Edges that end at a group itself or a folder node should not have {@link PortCandidate port candidates}
or {@link PortConstraint port constraints} at this end (because this may destroy the recursive directed edge style required to obtain stable
results).
<b>Warning:</b> The algorithm assumes that there is at most one node with associated alternative bounds.
@see HierarchicLayout#ALTERNATIVE_GROUP_BOUNDS_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23alternativeGroupBounds Online Documentation}*/
alternativeGroupBounds: ItemMapping<INode, IRectangle>;
/**
* Gets or sets a mapping from edges to their priority to be a 'critical' edge.
* The layout tries to vertically align each node pair that is connected by a critical edge. Critical edges are marked by
* having a priority greater than <code>0</code>; edges with a zero or negative priority are not considered critical.
*
* Critical edges highlight different edge paths that are relevant for a user. The layout algorithm tries to vertically
* align each node pair that is connected by a critical edge. Conflicts between different critical edges are always
* resolved in favor of the higher priority.
*
* <h2>Note</h2>
*
* Critical edges are always connected to the center port. Hence, the edge distribution is no longer uniform.
*
* <h2>Note</h2>
*
* Critical edges by default also get crossing costs assigned based on their priorities (see property
* {@link HierarchicLayoutData#reduceCriticalEdgeCrossings}). Thus, they are favored during the edge crossing minimization phase.
*
* Critical edges are often determined by some external data source and are often part of a larger path. The following
* example shows how two different such paths can be made 'critical' with different priorities:
*
* <br><b>Configuring critical edges via a mapper</b>
* <pre>
* // Set two different critical edge paths with different priorities
* for (const edge of path1) {
*   layoutData.criticalEdgePriorities.mapper.set(edge, 5)
* }
* // Lower priority for the second path, which then may deviate from a straight line
* for (const edge of path2) {
*   layoutData.criticalEdgePriorities.mapper.set(edge, 2)
* }
* </pre>
*
* Alternatively, a delegate may be used as well, if determining edge criticality is easier when looking at the edge
* itself:
*
* <br><b>Configuring critical edges via a delegate</b>
* <pre>
* // Assume that the edge's tag holds an instance that has an 'IsCritical' property
* layoutData.criticalEdgePriorities = edge => (edge.tag.isCritical ? 1 : 0)
* </pre>
*
* @see HierarchicLayout#CRITICAL_EDGE_PRIORITY_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23criticalEdgePriorities Online Documentation}
*/
criticalEdgePriorities: ItemMapping<IEdge, number>;
/**
Gets or sets whether or not {@link HierarchicLayoutData#criticalEdgePriorities critical edges} automatically get {@link HierarchicLayoutData#edgeCrossingCosts crossing costs} assigned based on their critical edge
priorities.
If enabled, an edge gets the critical edge priority plus one as its crossing cost - unless, a higher crossing cost was
manually specified. Edges with a priority that is negative or zero get <code>1</code> as the crossing cost (which is the default for
it). If disabled, the critical edge priorities do not automatically affect the crossing minimization. The user can still
provide higher crossing costs manually, of course.
@see HierarchicLayoutData#edgeCrossingCosts
@see HierarchicLayoutData#criticalEdgePriorities
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23reduceCriticalEdgeCrossings Online Documentation}*/
reduceCriticalEdgeCrossings: boolean;
/**
* Gets or sets the collection of folder nodes used for recursive edge styles in incremental mode.
* When using recursive edge styles in incremental mode, edges will also start at the bottom and end at the top of marked
* folder nodes. This will keep the edge routes more stable since the connection sides won't change.
*
* Using the {@link HierarchicLayoutData#folderNodes} property is probably easiest with a delegate that checks for each node whether it's a folder node or not:
*
* <br><b>Dynamically determine whether a node is a folder node</b>
* <pre>
* layoutData.folderNodes = node => graph.foldingView.isInFoldingState(node)
* </pre>
*
* However, the other options are also still there, such as using an {@link IEnumerable} via {@link ItemCollection#source}:
*
* <br><b>Using a static collection of folder nodes</b>
* <pre>
* layoutData.folderNodes = graph.nodes.filter(node => graph.foldingView.isInFoldingState(node))
* </pre>
*
* or explicitly using the {@link ItemCollection#items} collection to add or remove items, perhaps based on the user collapsing/expanding group nodes:
*
* <br><b>Adding and removing nodes from the FolderNodes backing collection</b>
* <pre>
* // When collapsing a node, mark it as a folder node for layout
* graphEditorInputMode.navigationInputMode.addGroupCollapsedListener((sender, args) =>
*   layoutData.folderNodes.items.add(args.item)
* )
* // When expanding the node again, we unmark it again
* graphEditorInputMode.navigationInputMode.addGroupExpandedListener((sender, args) =>
*   layoutData.folderNodes.items.remove(args.item)
* )
* </pre>
*
* @see HierarchicLayout#FOLDER_NODES_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23folderNodes Online Documentation}
*/
folderNodes: ItemCollection<INode>;
/**
* Gets or sets the mapping of edges to their {@link HierarchicLayoutEdgeLayoutDescriptor}.
* If an edge is mapped to <code>null</code>, the {@link HierarchicLayout#edgeLayoutDescriptor default descriptor} is used.
*
* This property allows to change how edges are routed for each edge individually. To just change a few edges that deviate
* from the default style, using the {@link ItemMapping#mapper} is probably the best option:
*
* <br><b>Using a mapper to set different EdgeLayoutDescriptors for different edges</b>
* <pre>
* // Ensure larger lengths for the first and last segments of edge1
* layoutData.edgeLayoutDescriptors.mapper.set(
*   edge1,
*   new HierarchicLayoutEdgeLayoutDescriptor({
*     minimumFirstSegmentLength: 30,
*     minimumLastSegmentLength: 50
*   })
* )
* // Change the routing style to octilinear for edge2
* layoutData.edgeLayoutDescriptors.mapper.set(
*   edge2,
*   new HierarchicLayoutEdgeLayoutDescriptor({
*     routingStyle: new HierarchicLayoutRoutingStyle(HierarchicLayoutEdgeRoutingStyle.OCTILINEAR)
*   })
* )
* // All other edges not set in the mapper use the default EdgeLayoutDescriptor
* // set on HierarchicLayout.
* </pre>
*
* If an {@link HierarchicLayoutEdgeLayoutDescriptor} is easier to create from every edge itself, using a delegate is often
* easier:
*
* <br><b>Using a delegate to set a different EdgeLayoutDescriptor for each edge</b>
* <pre>
* // Use a property from a custom business object in the edge's tag as the minimum length
* layoutData.edgeLayoutDescriptors = edge => {
*   const customData = edge.tag
*   return new HierarchicLayoutEdgeLayoutDescriptor({
*     minimumLength: customData.minimumLength,
*     routingStyle: new HierarchicLayoutRoutingStyle(
*       customData.shouldUseOctilinearRouting
*         ? HierarchicLayoutEdgeRoutingStyle.OCTILINEAR
*         : HierarchicLayoutEdgeRoutingStyle.polyline
*     )
*   })
* }
* </pre>
*
* @see HierarchicLayout#edgeLayoutDescriptor
* @see HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23edgeLayoutDescriptors Online Documentation}
*/
edgeLayoutDescriptors: ItemMapping<IEdge, HierarchicLayoutEdgeLayoutDescriptor>;
/**
* Gets or sets the mapping of nodes to their {@link HierarchicLayoutNodeLayoutDescriptor}
* If a node is mapped to <code>null</code>, the {@link HierarchicLayout#nodeLayoutDescriptor default descriptor} is used.
*
* This property allows to change how nodes are laid out within their layer for each node individually. To just change a
* few nodes that deviate from the default handling, using the {@link ItemMapping#mapper} is probably the best option:
*
* <br><b>Using a mapper to set different NodeLayoutDescriptors for different nodes</b>
* <pre>
* // Don't consider node1's labels at all
* layoutData.nodeLayoutDescriptors.mapper.set(
*   node1,
*   new HierarchicLayoutNodeLayoutDescriptor({ nodeLabelMode: NodeLabelMode.NEVER })
* )
* // Align node2 at the bottom of its layer
* layoutData.nodeLayoutDescriptors.mapper.set(
*   node2,
*   new HierarchicLayoutNodeLayoutDescriptor({ layerAlignment: 1 })
* )
* // All other nodes not set in the mapper use the default HierarchicLayoutNodeLayoutDescriptor
* // set on HierarchicLayout.
* </pre>
*
* If a {@link HierarchicLayoutNodeLayoutDescriptor} is easier to create from every node itself, using a delegate is often
* easier:
*
* <br><b>Using a delegate to set a different NodeLayoutDescriptor for each node</b>
* <pre>
* // Use a property from a custom business object in the node's tag as the minimum length
* layoutData.nodeLayoutDescriptors = node => {
*   const customData = node.tag
*   const descriptor = new HierarchicLayoutNodeLayoutDescriptor({
*     minimumDistance: 5,
*     nodeLabelMode: NodeLabelMode.CONSIDER_FOR_DRAWING
*   })
*   switch (customData.alignment) {
*     case Alignment.TOP:
*       descriptor.layerAlignment = 0
*       break
*     case Alignment.CENTER:
*       descriptor.layerAlignment = 0.5
*       break
*     case Alignment.BOTTOM:
*       descriptor.layerAlignment = 1
*       break
*   }
*   return descriptor
* }
* </pre>
*
* @see HierarchicLayout#nodeLayoutDescriptor
* @see HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23nodeLayoutDescriptors Online Documentation}
*/
nodeLayoutDescriptors: ItemMapping<INode, HierarchicLayoutNodeLayoutDescriptor>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the mapping for sub-components of the input graph to a custom layout algorithm responsible for the
* arrangement of the sub-component.
* A sub-component is arranged by another layout algorithm and its layout is finally embedded into the global hierarchical
* layout. This feature allows, for example, to highlight special sub-structures by arranging them in a different fashion.
* Note that sub-components may not be nested, that is, a sub-component is not allowed to contain nodes that belong to yet
* another sub-component. Nesting of sub-components results in an exception.
*
* To conveniently define a sub-component as well as the layout algorithm responsible for it use the {@link ItemCollectionMapping#add} method of this
* property. It takes the layout algorithm as parameter and allows to define the nodes associated to it using the returned
* {@link ItemCollection}.
*
* A sub-component can be connected to nodes of another sub-component as well as to nodes that are not part of any
* sub-component. Edges that model these connections are so-called <em>inter-edges</em>. For inter-edges there apply some smaller restrictions
* with respect to the supported features (see the documentation of {@link HierarchicLayout#SUB_COMPONENT_ID_DP_KEY}
* for details).
*
* Sub-components can be defined on grouped graphs with the following restrictions:
*
* <ul>
* <li>All sub-component nodes must be on the <em>same hierarchy level</em> if the group node itself is not part of the sub-component.</li>
* <li>If a group is assigned to a sub-component, <em>all</em> its descendants (including other group nodes) must be in that component,
* too.</li>
* </ul>
* When running in
* {@link LayoutMode#INCREMENTAL incremental mode}, it is still up to the sub-layout algorithm to decide whether to treat a sub-component element as
* incremental or fixed. If the given layout of the sub-component should be considered, then the from-sketch mode of the
* used layout algorithm needs to be enabled (if such a mode is supported).
*
* <b>Warning:</b> When using a {@link PartitionGrid}, all nodes of a sub-component need to be assigned to the same {@link PartitionCellId}
* or, alternatively, not be assigned to a cell at all. Otherwise, an exception will the thrown during layout. Furthermore,
* it is not possible that the layout of a sub-component uses another partition grid instance at the same time - in other
* words, nested partition grids are not supported.
*
* <h2>Note</h2>
*
* For the layout of a sub-component, the specified sub-component layout algorithm is fully responsible. Thus, the
* supported features and layout style are defined solely by that algorithm.
*
* Sub-components can be configured by first {@link ItemCollectionMapping#add adding} a new layout algorithm and saving the resulting
* {@link ItemCollection}. That one can then be used as usual to define the subset of nodes which the layout should process:
*
* <pre>
* // Retrieve an ItemCollection<INode> which defines all nodes
* // of a sub-component. This sub-component will be arranged by an OrganicLayout
* const organicSubset = layoutData.subComponents.add(new OrganicLayout())
* // You can use this ItemCollection<INode> as usual, using either one of
* // the Source, Items, Mapper, or Delegate properties.
* // Here, we simply use the selected nodes:
* organicSubset.source = graphComponent.selection.selectedNodes
* </pre>
*
* Since adding a new layout algorithm for a sub-component returns the
* {@link ItemCollection}, configuring sub-components can also be conveniently chained:
*
* <pre>
* layoutData.subComponents.add(new OrganicLayout()).source = organicNodes
* layoutData.subComponents.add(new TreeLayout()).source = treeNodes
* layoutData.subComponents.add(new OrthogonalLayout()).source = orthogonalNodes
* </pre>
*
* @see HierarchicLayout#SUB_COMPONENT_ID_DP_KEY
* @see HierarchicLayout#SUB_COMPONENT_LAYOUT_ALGORITHM_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23subComponents Online Documentation}
*/
subComponents: ItemCollectionMapping<INode, ILayoutAlgorithm>;
/**
* Gets or sets a mapping from edges to their source {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its source node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* A more general concept which allows finer control over where ports are placed, are {@link HierarchicLayoutData#sourcePortCandidates port candidates}.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All source ports should be on the bottom side
* layoutData.sourcePortConstraints.constant = PortConstraint.create(PortSide.SOUTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should leave the source node on the left side, while keeping its exact port position (strong port constraint)
* layoutData.sourcePortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.WEST, true))
* // edge2 should leave the source node downwards, but the port may be placed anywhere along that node border
* layoutData.sourcePortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.SOUTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Source ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.sourcePortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see HierarchicLayoutData#sourcePortCandidates
* @see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23sourcePortConstraints Online Documentation}
*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets a mapping from edges to their target {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its target node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* A more general concept which allows finer control over where ports are placed, are {@link HierarchicLayoutData#targetPortCandidates port candidates}.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All target ports should be on the top side
* layoutData.targetPortConstraints.constant = PortConstraint.create(PortSide.NORTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should enter the target node on the right side, while keeping its exact port position (strong port
* // constraint)
* layoutData.targetPortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.EAST, true))
* // edge2 should enter the target node from the top, but the port may be placed anywhere along that node border
* layoutData.targetPortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.NORTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Target ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.targetPortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see HierarchicLayoutData#targetPortCandidates
* @see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23targetPortConstraints Online Documentation}
*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets a mapping from edges to a collection of their source {@link PortCandidate port candidates}.
* Port constraints allow to define where an edge can connect to its source node and allow fine control over port
* placement.
*
* If all that is needed is to fix the source port in its location or on a node side, {@link HierarchicLayoutData#sourcePortConstraints port constraints} are easier to work
* with, since they are a slightly simpler concept.
*
* Source port candidates are effectively a collection of possible port placements with different costs and the layout
* algorithm is free to choose the candidate that fits best into the overall layout, while also preferring candidates with
* a lower cost. To set the same candidate list for all edges, it's easiest to use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same set of source port candidates for all edges</b>
* <pre>
* layoutData.sourcePortCandidates.constant = new List([
*   // Prefer the center position and route the edge downwards if possible (no cost for this candidate)
*   PortCandidate.createCandidate(0, 0, PortDirections.SOUTH),
*   // Alternatively, use any position at the bottom border of the node, but with a higher cost
*   PortCandidate.createCandidate(PortDirections.SOUTH, 1),
*   // If that fails, use either the left or the right side
*   PortCandidate.createCandidate(PortDirections.EAST | PortDirections.WEST, 2)
* ])
* </pre>
*
* If certain edges need specific port candidates, it's usually convenient to use the {@link ItemMapping#mapper} property:
*
* <br><b>Setting different source port candidates for certain edges</b>
* <pre>
* // edge1 should leave the source node preferably on the node side in flow direction,
* // but can also use a fixed port location on the left of the node.
* layoutData.sourcePortCandidates.mapper.set(
*   edge1,
*   new List([
*     PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*     PortCandidate.createCandidate(-0.5, 0.25, PortDirections.WEST, 1)
*   ])
* )
*
* // edge2 should leave the source node anywhere on the upper side of the node
* layoutData.sourcePortCandidates.mapper.set(
*   edge2,
*   new List([PortCandidate.createCandidate(PortDirections.NORTH)])
* )
* </pre>
*
* For cases when the desired configuration of port candidates can be readily created from the edge itself, the {@link ItemMapping#delegate} is often
* the most convenient option:
*
* <br><b>Setting different source port candidates for each edge with a delegate</b>
* <pre>
* // Assuming a flowchart, the "yes" edge should leave the decision in flow direction,
* // and the "no" edge may leave either left or right of the flow direction.
* // This also ensures that the layout still looks natural if the layout uses different orientations.
* layoutData.sourcePortCandidates.delegate = edge => {
*   // Only handle decision nodes here
*   if (edge.sourceNode.tag instanceof FlowChartDecision) {
*     switch (edge.labels.get(0).text) {
*       case 'Yes':
*         return new List([PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW)])
*       case 'No':
*         return new List([
*           PortCandidate.createCandidate(
*             PortDirections.LEFT_IN_FLOW | PortDirections.RIGHT_IN_FLOW
*           )
*         ])
*     }
*   }
*   return null
* }
* </pre>
*
* @see HierarchicLayoutData#sourcePortConstraints
* @see PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23sourcePortCandidates Online Documentation}
*/
sourcePortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
* Gets or sets a mapping from edges to a collection of their target port {@link PortCandidate candidates}.
* Port constraints allow to define where an edge can connect to its target node and allow fine control over port
* placement.
*
* If all that is needed is to fix the target port in its location or on a node side, {@link HierarchicLayoutData#targetPortConstraints port constraints} are easier to work
* with, since they are a slightly simpler concept.
*
* Target port candidates are effectively a collection of possible port placements with different costs and the layout
* algorithm is free to choose the candidate that fits best into the overall layout, while also preferring candidates with
* a lower cost. To set the same candidate list for all edges, it's easiest to use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same set of target port candidates for all edges</b>
* <pre>
* layoutData.targetPortCandidates.constant = new List([
*   // Prefer the center position and enter the node from the top if possible (no cost for this candidate)
*   PortCandidate.createCandidate(0, 0, PortDirections.NORTH),
*   // Alternatively, use any position at the top border of the node, but with a higher cost
*   PortCandidate.createCandidate(PortDirections.NORTH, 1),
*   // If that fails, use either the left or the right side
*   PortCandidate.createCandidate(PortDirections.EAST | PortDirections.WEST, 2)
* ])
* </pre>
*
* If certain edges need specific port candidates, it's usually convenient to use the {@link ItemMapping#mapper} property:
*
* <br><b>Setting different target port candidates for certain edges</b>
* <pre>
* // edge1 should enter the target node preferably on the node side in flow direction,
* // but can also use a fixed port location on the left of the node.
* layoutData.targetPortCandidates.mapper.set(
*   edge1,
*   new List([
*     PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*     PortCandidate.createCandidate(-0.5, -0.25, PortDirections.WEST, 1)
*   ])
* )
*
* // edge2 should enter the target node anywhere on the bottom side of the node
* layoutData.targetPortCandidates.mapper.set(
*   edge2,
*   new List([PortCandidate.createCandidate(PortDirections.SOUTH)])
* )
* </pre>
*
* For cases when the desired configuration of port candidates can be readily created from the edge itself, the {@link ItemMapping#delegate} is often
* the most convenient option:
*
* <br><b>Setting different target port candidates for each edge with a delegate</b>
* <pre>
* // Assuming a flowchart, edges connecting to a terminal on the target side should enter that node either
* // in flow direction (preferred) or from one of the sides.
* // This also ensures that the layout still looks natural if the layout uses different orientations.
* layoutData.targetPortCandidates.delegate = edge => {
*   // Only handle terminal nodes here
*   if (edge.targetNode.tag instanceof FlowChartTerminal) {
*     return new List([
*       PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*       PortCandidate.createCandidate(PortDirections.LEFT_IN_FLOW | PortDirections.RIGHT_IN_FLOW, 1)
*     ])
*   }
*   return null
* }
* </pre>
*
* @see HierarchicLayoutData#targetPortConstraints
* @see PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23targetPortCandidates Online Documentation}
*/
targetPortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from nodes to their {@link PortCandidateSet}.
@see PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23nodePortCandidateSets Online Documentation}*/
nodePortCandidateSets: ItemMapping<INode, PortCandidateSet>;
/**
* Gets or sets a mapping from edges to an object representing their source edge group.
* Edges sharing a source group identifier will share a common bus near the source or at a common source node if possible.
*
* The {@link HierarchicLayout} is able to group incoming and outgoing edges. Therefore, the source group
* identifier of the outgoing edges of a node has to match with the target group identifier of the incoming edges.
*
* One simple way to use source groups is to use the edge's source node as group ID which effectively groups all edges with
* the same source together:
*
* <br><b>Grouping all edges on the source side</b>
* <pre>
* layoutData.sourceGroupIds = edge => edge.sourceNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.sourceGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.sourceGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see HierarchicLayoutData#sourcePortGroupIds
* @see HierarchicLayoutData#targetGroupIds
* @see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23sourceGroupIds Online Documentation}
*/
sourceGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to an object representing their source port group.
* All edges with the same port id at a node will share the same port location. However, they will be routed independently
* and in contrast to {@link HierarchicLayoutData#sourceGroupIds source groups} will deviate from a common path as soon as possible instead of sharing as much of a
* common path as possible.
*
* The simplest way to use source port groups is to use the same object as group ID for all edges. Since grouping is done
* per node, this has the effect of grouping all edges with the same source node together:
*
* <br><b>Grouping all edges on the source side</b>
* <pre>
* layoutData.sourcePortGroupIds = {}
* </pre>
*
* The same effect can be achieved with a delegate as well, by returning the source node as group ID for each edge.
* However, a more useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.sourcePortGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgePortGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.sourcePortGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see HierarchicLayoutData#targetPortGroupIds
* @see HierarchicLayoutData#sourceGroupIds
* @see PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23sourcePortGroupIds Online Documentation}
*/
sourcePortGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to an object representing their target edge group.
* Edges sharing a target group identifier will share a common bus near the target or at a common target node if possible.
*
* The {@link HierarchicLayout} is able to group incoming and outgoing edges. Therefore, the target group
* identifier of the incoming edges of a node has to match with the source group identifier of the outgoing edges.
*
* One simple way to use source groups is to use the edge's target node as group ID which effectively groups all edges with
* the same target together:
*
* <br><b>Grouping all edges on the target side</b>
* <pre>
* layoutData.targetGroupIds = edge => edge.targetNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.targetGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.targetGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see HierarchicLayoutData#sourceGroupIds
* @see HierarchicLayoutData#targetPortGroupIds
* @see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23targetGroupIds Online Documentation}
*/
targetGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to an object representing their target port group.
* All edges with the same port id at a node will share the same port location. However, they will be routed independently
* and in contrast to {@link HierarchicLayoutData#sourceGroupIds source groups} will deviate from a common path as soon as possible instead of sharing as much of a
* common path as possible.
*
* The simplest way to use target port groups is to use the same object as group ID for all edges. Since grouping is done
* per node, this has the effect of grouping all edges with the same target node together:
*
* <br><b>Grouping all edges on the target side</b>
* <pre>
* layoutData.targetPortGroupIds = {}
* </pre>
*
* The same effect can be achieved with a delegate as well, by returning the target node as group ID for each edge.
* However, a more useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.targetPortGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgePortGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges of it
*     layoutData.targetPortGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see HierarchicLayoutData#sourcePortGroupIds
* @see HierarchicLayoutData#targetGroupIds
* @see PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23targetPortGroupIds Online Documentation}
*/
targetPortGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets the mapping from edges to their {@link HierarchicLayoutBusDescriptor bus descriptor} which defines the bus structure they belong to.
* All edges with the same {@link HierarchicLayoutBusDescriptor} adjacent to a specific node (the root) and having the same
* direction are arranged in a compact, bus-like way. The {@link HierarchicLayoutData#edgeDirectedness} is also considered here. The direction defines the placement of
* the bus structure in the following way:
*
* <ul>
* <li>Bus nodes connected by out-edges are placed in the layers starting with the layer that comes directly after the layer of
* the root (e.g. below the root for top-to-bottom layout orientation).</li>
* <li>Bus nodes connected by in-edges are placed in the layers ending with the layer that comes directly before the layer of
* the root (e.g. above the root for top-to-bottom layout orientation).</li>
* </ul>
* The bus substructures are arranged using a style that yields more compact layout results. Edges to the child nodes, the
* so-called bus nodes, are routed using a shared segment that connects to the common root node. The bus nodes are arranged
* using several layers above or below the root node such that the whole substructure has a compact area. By default, each
* bus layer contains equally many bus nodes, bus nodes of adjacent layers are center-aligned and the shared bus segment is
* routed in the middle of the nodes. This produces compact, symmetric arrangements of the bus structures.
*
* The arrangement can be customized for each bus individually, by using the settings offered by
* {@link HierarchicLayoutBusDescriptor}. It allows to configure the number of bus nodes before and after the common bus
* segment.
*
* There apply some restrictions for bus structures:
*
* <ul>
* <li>Only nodes that are in the same {@link PartitionCellId partition grid cell} or same {@link SwimlaneDescriptor swimlane} as the root node are included in the bus
* structure.</li>
* <li>The bus nodes must all belong to the same group node. Also, a group node can not be part of a bus structure; neither as
* bus element nor as root.</li>
* <li>Bus structures can not be nested. That means that if a node is a bus node it can not be a root node of another bus at
* the same time.</li>
* <li>For bus edges, the integrated edge labeling feature does not place port labels as close to the actual port as it does
* for other edges (see {@link LabelPlacements#AT_SOURCE_PORT} and {@link LabelPlacements#AT_TARGET_PORT}).</li>
* <li>Edge grouping constraints at the side of the bus root node are ignored for bus edges; the grouping induced by the bus
* structure is considered to be stronger.</li>
* <li>If in {@link LayoutMode#INCREMENTAL incremental layout mode}, the bus edges are always treated in an incremental way such that the bus-style routing
* remains consistent (e.g. if a fixed bus edge contains bends that suggest another route, they are ignored). The reason is
* that the bus-style layout of the structures has higher priority. Furthermore, the location of the bus segment relative
* to the elements is not derived from the sketch but depends on the settings {@link HierarchicLayoutBusDescriptor#maximumNodesBeforeBus} and {@link HierarchicLayoutBusDescriptor#maximumNodesAfterBus} as well as the before/after constraints
* defined by
* {@link HierarchicLayoutData#nodesBeforeBus}. This means that it can happen that a fixed bus node which was placed before the bus in the input drawing may be placed
* after the bus if the bus was enlarged due to the insertion of incremental bus nodes.</li>
* <li>It is not recommended to define buses inside a group node which has an
* {@link IIncrementalHintsFactory#createIncrementalGroupHint incremental group hint}. Fixed bus nodes inside such a group are not necessarily kept fix but are treated like an
* incrementally inserted node.</li>
* </ul>
* <h2>Note</h2>
*
* When two parallel edges have the same {@link HierarchicLayoutBusDescriptor}, only one edge will be treated as a bus edge.
*
* <b>Warning:</b>
* If layering constraints are specified, they are not considered for bus nodes because the bus structure is layered
* independently.
*
* <b>Warning:</b>
* If bus nodes have edges to other bus nodes (from the same or from another bus), then these edges are not considered in
* the same way during the layering as other edges; in consequence, it can happen that both bus nodes end up in the same
* final layer and thus the edge becomes a same-layer edge. The same can happen if a group has an edge to a bus node where
* the bus node is outside said group but the root of the bus is a child of it.
*
* Buses can be configured by first {@link ItemCollectionMapping#add adding} a new {@link HierarchicLayoutBusDescriptor} and saving the resulting
* {@link ItemCollection}. That one can then be used to define the structure of the bus:
*
* <pre>
* // Retrieve an ItemCollection<IEdge> which defines all edges
* // of a bus. This bus will be arranged according to the HierarchicLayoutBusDescriptor
* const bus = layoutData.buses.add(new HierarchicLayoutBusDescriptor())
* // You can use this ItemCollection<IEdge> as usual, using either one of
* // the Source, Items, Mapper, or Delegate properties.
* // Here, we simply use the selected edges:
* bus.source = graphComponent.selection.selectedEdges
* </pre>
*
* Since adding a new {@link HierarchicLayoutBusDescriptor} for a bus returns the
* {@link ItemCollection}, configuring buses can also be conveniently chained:
*
* <pre>
* //  bus1Edges, bus2Edges, bus3Edges are of type IEnumerable.<IEdge>
* layoutData.buses.add(new HierarchicLayoutBusDescriptor()).source = bus1Edges
* layoutData.buses.add(new HierarchicLayoutBusDescriptor()).source = bus2Edges
* layoutData.buses.add(new HierarchicLayoutBusDescriptor()).source = bus3Edges
* </pre>
*
* @see HierarchicLayoutBusDescriptor
* @see HierarchicLayout#BUS_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23buses Online Documentation}
*/
buses: ItemCollectionMapping<IEdge, HierarchicLayoutBusDescriptor>;
/**
* Gets or sets the mapping from nodes to their layer index relative to the root node of the {@link HierarchicLayoutData#buses bus} they belong to.
* An offset value <code>i</code> specified for a bus node is interpreted relative to the root node as follows, assuming that the root
* node is in layer <code>x</code>.
*
* <ul>
* <li>If the bus is placed in the layers after the root, the bus node will be placed in layer <code>x + i</code></li>
* <li>If the bus is placed in the layers before the root, the bus node will be placed in layer <code>x - i</code></li>
* </ul>
* <h2>Note</h2>
*
* This setting is only considered for nodes that belong to bus structure defined by property {@link HierarchicLayoutData#buses}. If defining {@link HierarchicLayoutData#givenLayersLayererIds given layer indices}
* for all nodes, the index for nodes of a bus are also interpreted in a relative way as described here.
*
* <b>Warning:</b> The specified {@link HierarchicLayoutBusDescriptor#maximumNodesBeforeBus number of nodes before the bus} and {@link HierarchicLayoutBusDescriptor#maximumNodesAfterBus number of nodes after the bus} might be violated if the layer indices
* provided here are such that e.g., a layer contains too many nodes to fulfill the node count constraints.
*
* When the layer offsets can be readily inferred from the nodes, then the easiest option is usually to use the {@link ItemMapping#delegate}:
*
* <pre>
* layoutData.busRootOffsets = node => node.tag.offset
* </pre>
*
* In case only some nodes need to be mapped to their layer respective offset, it's sometimes easier to use the {@link ItemMapping#mapper}:
*
* <pre>
* layoutData.busRootOffsets.mapper.set(node1, 0)
* layoutData.busRootOffsets.mapper.set(node2, 1)
* </pre>
*
* @see HierarchicLayoutData#buses
* @see HierarchicLayoutBusDescriptor#ROOT_OFFSET_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23busRootOffsets Online Documentation}
*/
busRootOffsets: ItemMapping<INode, number>;
/**
* Gets or sets the collection of {@link HierarchicLayoutData#buses bus} nodes that should be placed before the common bus segment.
* If this collection is not specified, the nodes will be placed in an alternating way.
*
* <b>Warning:</b> The side assignment induced by this setting is ignored if there are any {@link HierarchicLayoutData#sequenceConstraints} between the bus nodes. Furthermore, it can
* happen that the specified maximum number of nodes {@link HierarchicLayoutBusDescriptor#maximumNodesBeforeBus before the bus} and {@link HierarchicLayoutBusDescriptor#maximumNodesAfterBus after the bus} is exceeded due to the side placement
* constraints.
*
* When the placement can be readily inferred from the nodes, then the easiest option is usually to use the {@link ItemCollection#delegate}:
*
* <pre>
* layoutData.nodesBeforeBus = node => graphComponent.selection.selectedNodes.isSelected(node)
* </pre>
*
* However, the other options are also still there, such as using an {@link IEnumerable} via {@link ItemCollection#source}:
*
* <pre>
* layoutData.folderNodes.source = graphComponent.selection.selectedNodes
* </pre>
*
* or explicitly using the {@link ItemCollection#items} collection to add or remove items:
*
* <pre>
* layoutData.nodesBeforeBus.items.add(node1)
* layoutData.nodesBeforeBus.items.add(node2)
* </pre>
*
* @see HierarchicLayoutData#buses
* @see HierarchicLayoutBusDescriptor#PLACE_BEFORE_BUS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23nodesBeforeBus Online Documentation}
*/
nodesBeforeBus: ItemCollection<INode>;
/**
* Gets or sets the layout data to specify layer constraints.
* Layer constraints affect the assignment of nodes to {@link ILayer layers}.
*
* If {@link LayerConstraintData#nodeComparables} are specified, they are used to sort all nodes accordingly and create {@link LayerConstraintData#placeBelow}
* constraints that work just like manually added constraints.
*
* If a fixed layer assignment is already available, the {@link HierarchicLayoutData#givenLayersLayererIds} should be used instead.
*
* One way to set layer constraints is by mapping nodes to an {@link IComparable} (e.g. a number) which define the
* order of layers in which those nodes are placed. This can be done with the {@link LayerConstraintData#nodeComparables} property. One way to map each node to such
* an {@link IComparable} is to use the {@link ItemMapping#delegate}:
*
* <br><b>Using a delegate to map nodes to objects that can be ordered to define relative layer placements</b>
* <pre>
* // Map each node to an IComparable, in this case taken from a custom object stored in the Tag
* layoutData.layerConstraints.nodeComparables = node => {
*   const data = node.tag
*   if (data !== null) {
*     return data.layerOrder
*   }
*   // Don't constrain layer placement if the tag doesn't contain our custom object.
*   return null
* }
* </pre>
*
* If only some nodes should be constrained, the {@link ItemMapping#mapper} can be used instead:
*
* <br><b>Using a mapper to map certain nodes to objects that can be ordered to define relative layer placements</b>
* <pre>
* // Define layer constraints for a few nodes via a mapper. Nodes that are not
* // explicitly mapped don't have constraints on their layer placement.
* const mapper = layoutData.layerConstraints.nodeComparables.mapper
* // Place both node1 and node2 in the same layer by assigning the same value
* mapper.set(node1, 5)
* mapper.set(node2, 5)
* // Place node3 somewhere above node1 and node2 by assigning a smaller value
* mapper.set(node3, 2)
* // Place node4 somewhere below node1 and node2 by assigning a larger value
* mapper.set(node4, 8)
* // The exact values don't matter much they just define an ordering. However, all
* // values must have the same type you cannot mix int and string or double, for example.
* </pre>
*
* If those mappings have been prepared beforehand, e.g. in a {@link HashMap} or {@link IMapper}, that property on the {@link ItemMapping} can also be set:
*
* <br><b>Setting a prepared mapper or dictionary for relative layer assignment</b>
* <pre>
* layoutData.layerConstraints.nodeComparables = layerOrderMapper
* // Or create a mapper from a map, instead:
* layoutData.layerConstraints.nodeComparables = new Mapper(layerOrderMap)
* </pre>
*
* {@link IComparable} is only the most natural option if such an object can be readily produced from a node, or there
* already exists such a mapping from nodes to something that can be compared. In many cases it can be easier to construct
* constraints manually with the respective methods on {@link LayerConstraintData}:
*
* <br><b>Defining custom layer constraints</b>
* <pre>
* // Place all nodes in sameLayerNodes into the same layer
* const firstSameLayerNode = sameLayerNodes.first()
* for (const node of sameLayerNodes) {
*   layoutData.layerConstraints.placeInSameLayer(firstSameLayerNode, node)
* }
* // Ensure that node2 is placed two layers below node1
* layoutData.layerConstraints.placeBelow(node1, node2, 2)
* // Also place another node in the top layer
* layoutData.layerConstraints.placeAtTop(nodeAtTop)
* </pre>
*
* @see HierarchicLayout#createLayerConstraintFactory
* @see ILayerConstraintFactory
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23layerConstraints Online Documentation}
*/
layerConstraints: LayerConstraintData;
/**
* Gets or sets the layout data to specify sequence constraints.
* One way to set sequence constraints is by mapping nodes or edges to an {@link IComparable} (e.g. a number) which
* define the order in which those nodes are placed within the same layer. This can be done with the {@link SequenceConstraintData#itemComparables} property. One way to
* map each item to such an {@link IComparable} is to use the {@link ItemMapping#delegate}:
*
* <br><b>Using a delegate to map items to objects that can be ordered to define relative placements within a layer</b>
* <pre>
* // Map each item to an IComparable, in this case taken from a custom object stored in the Tag
* layoutData.sequenceConstraints.itemComparables = item => {
*   const data = item.tag
*   if (data !== null) {
*     return data.sequenceOrder
*   }
*   // Don't constrain sequencing if the tag doesn't contain our custom object.
*   return null
* }
* </pre>
*
* If only some nodes should be constrained, the {@link ItemMapping#mapper} can be used instead:
*
* <br><b>Using a mapper to map certain items to objects that can be ordered to define relative placements within a layer</b>
* <pre>
* // Define sequence constraints for a few nodes via a mapper. Nodes that are not
* // explicitly mapped don't have constraints on their placement within the layer.
* const mapper = layoutData.sequenceConstraints.itemComparables.mapper
* // Place node1 before node2
* mapper.set(node1, 1)
* mapper.set(node2, 3)
* // Place node3 between both of them
* mapper.set(node3, 2)
* // Place node4 after the others by assigning a larger value
* mapper.set(node4, 8)
* // The exact values don't matter much they just define an ordering. However, all
* // values must have the same type you cannot mix int and string or double, for example.
* </pre>
*
* If those mappings have been prepared beforehand, e.g. in a {@link HashMap} or {@link IMapper}, that property on the {@link ItemMapping} can also be set:
*
* <br><b>Setting a prepared mapper or dictionary for relative placement within a layer</b>
* <pre>
* layoutData.sequenceConstraints.itemComparables = sequenceMapper
* // Or create a mapper from a dictionary, instead:
* layoutData.sequenceConstraints.itemComparables = new Mapper(sequenceDictionary)
* </pre>
*
* {@link IComparable} is only the most natural option if such an object can be readily produced from a node, or there
* already exists such a mapping from nodes to something that can be compared. In many cases it can be easier to construct
* constraints manually with the respective methods on {@link SequenceConstraintData}:
*
* <br><b>Defining custom sequence constraints</b>
* <pre>
* // Ensure that node2 is placed after node1
* layoutData.sequenceConstraints.placeAfter(node1, node2)
* // Also place another node at the very start
* layoutData.sequenceConstraints.placeAtHead(firstNodeInLayer)
* </pre>
*
* @see HierarchicLayout#createSequenceConstraintFactory
* @see ISequenceConstraintFactory
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23sequenceConstraints Online Documentation}
*/
sequenceConstraints: SequenceConstraintData;
/**
Gets or sets the factory to specify layer constraints.
Note that this factory doesn't have to be {@link ILayerConstraintFactory#dispose disposed} after the layout calculation.
@see HierarchicLayout#createLayerConstraintFactory
@see ILayerConstraintFactory
@deprecated Use LayerConstraints instead
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23layerConstraintFactory Online Documentation}*/
layerConstraintFactory: ILayerConstraintFactory;
/**
Gets or sets the factory to specify sequence constraints.
Note that this factory doesn't have to be {@link ISequenceConstraintFactory#dispose disposed} after the layout calculation.
@see HierarchicLayout#createSequenceConstraintFactory
@see ISequenceConstraintFactory
@deprecated Use SequenceConstraints instead
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23sequenceConstraintFactory Online Documentation}*/
sequenceConstraintFactory: ISequenceConstraintFactory;
/**
* Gets or sets a mapping from nodes and edges to their incremental hints.
* Incremental hint specifies if and where nodes and edges are incrementally inserted.
*
* Elements which should be placed incrementally can be provided as collections:
*
* <br><b>Creating incremental hints for nodes and edges</b>
* <pre>
* // create a HierarchicLayout which rearranges
* // only the incremental graph elements
* const hl = new HierarchicLayout({ layoutMode: LayoutMode.INCREMENTAL })
*
* // provide additional data to configure the HierarchicLayout
* const hlData = new HierarchicLayoutData()
* // specify the nodes to rearrange
* hlData.incrementalHints.incrementalLayeringNodes.source = incrementalNodes
* // specify the edges to rearrange
* hlData.incrementalHints.incrementalSequencingItems.source = incrementalEdges
*
* graph.applyLayout(hl, hlData)
* </pre>
*
* More specific hints can be created using an {@link IIncrementalHintsFactory}:
*
* <br><b>Creating incremental hints for nodes and edges using a factory</b>
* <pre>
* // create a HierarchicLayout which rearranges
* // only the incremental graph elements
* const hl = new HierarchicLayout({ layoutMode: LayoutMode.INCREMENTAL })
*
* // provide additional data to configure the HierarchicLayout
* const hlData = new HierarchicLayoutData()
*
* hlData.incrementalHints.contextDelegate = (item, factory) => {
*   if (item instanceof INode && incrementalNodes.includes(item)) {
*     // create a hint for incremental nodes
*     if (graph.isGroupNode(item)) {
*       // special hint for groups
*       return factory.createIncrementalGroupHint(item)
*     }
*     if (fixedNodes.includes(item)) {
*       // exact layer for the fixedNodes
*       return factory.createUseExactLayerCoordinatesHint(item)
*     }
*     // simple layer hint for all other nodes
*     return factory.createLayerIncrementallyHint(item)
*   }
*   if (item instanceof IEdge && incrementalEdges.includes(item)) {
*     // sequence hint for incremental edges
*     return factory.createSequenceIncrementallyHint(item)
*   }
*   // all other items don't get hints
*   return null
* }
*
* // provide hints
*
* graph.applyLayout(hl, hlData)
* </pre>
*
* @see HierarchicLayout#INCREMENTAL_HINTS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23incrementalHints Online Documentation}
*/
incrementalHints: IncrementalHintItemMapping;
/**
* Gets or sets a mapper from nodes to the index of their layer.
* If this property is set, it can be used after layout to retrieve the layer assignment for each node. If <em>only</em> the layer
* assignment is needed and not the whole layout, {@link HierarchicLayout} can be run with {@link HierarchicLayout#stopAfterLayering} set to
* <code>true</code>. This will skip the remaining stages of the layout and therefore reduce the runtime.
*
* <br><b>Retrieving layering information after a layout</b>
* <pre>
* const layoutData = new HierarchicLayoutData()
* layoutData.layerIndices = new Mapper()
* graph.applyLayout(new HierarchicLayout(), layoutData)
*
* for (const node of graph.nodes) {
*   console.log(`Node ${node} has been placed in layer ${layoutData.layerIndices.get(node)}`)
* }
* </pre>
*
* @see HierarchicLayout#LAYER_INDEX_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23layerIndices Online Documentation}
*/
layerIndices: IMapper<INode, number> | null;
/**
* Gets or sets a mapper from nodes to the sequence index in their layer.
* If this property is set, it can be used after layout to retrieve the sequencing for each node. If <em>only</em> the sequencing (and
* layering) information is needed and not the whole layout, {@link HierarchicLayout} can be run with {@link HierarchicLayout#stopAfterSequencing} set to
* <code>true</code>. This will skip the remaining stages of the layout and therefore reduce the runtime.
*
* <br><b>Retrieving layering and sequencing information after a layout</b>
* <pre>
* const layoutData = new HierarchicLayoutData()
* layoutData.layerIndices = new Mapper()
* layoutData.sequenceIndices = new Mapper()
*
* graph.applyLayout(new HierarchicLayout(), layoutData)
*
* for (const node of graph.nodes) {
*   console.log(
*     `Node ${node} has been placed in layer ${layoutData.layerIndices.get(
*       node
*     )} at position ${layoutData.sequenceIndices.get(node)}`
*   )
* }
* </pre>
*
* @see HierarchicLayout#SEQUENCE_INDEX_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23sequenceIndices Online Documentation}
*/
sequenceIndices: IMapper<INode, number> | null;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
* Gets or sets the {@link HierarchicLayoutData#partitionGridData partition grid} layout data.
* The following sample shows how to assign nodes to partition grid cells simply via cell indices:
*
* <br><b>Assigning nodes to partition grid cells via indices</b>
* <pre>
* // Create four nodes and place them in grid cells in the following way
* // +--+--+--+
* // | 1| 2| 3|
* // +--+--+--+
* //    | 4|
* //    +--+
*
* const layoutData = new HierarchicLayoutData()
* const gridData = layoutData.partitionGridData
*
* const node1 = graph.createNode()
* const node2 = graph.createNode()
* const node3 = graph.createNode()
* const node4 = graph.createNode()
*
* // Assign the nodes to their rows and columns.
* // Note that don't have to create or use PartitionGrid directly in this case.
* // Setting the indices is enough.
* gridData.rowIndices.mapper.set(node1, 0)
* gridData.rowIndices.mapper.set(node2, 0)
* gridData.rowIndices.mapper.set(node3, 0)
* gridData.rowIndices.mapper.set(node4, 1)
* gridData.columnIndices.mapper.set(node1, 0)
* gridData.columnIndices.mapper.set(node2, 1)
* gridData.columnIndices.mapper.set(node3, 2)
* gridData.columnIndices.mapper.set(node4, 1)
*
* graph.applyLayout(new HierarchicLayout(), layoutData)
* </pre>
*
* When used this way there is no need to create a {@link PartitionGrid} instance or work with it directly. For more
* flexibility, e.g. to use cells that span multiple columns or rows, the {@link PartitionGrid} can be used as well:
*
* <br><b>Assigning nodes to partition grid cells via factory methods on the grid</b>
* <pre>
* // Create three nodes and place them in grid cells in the following way
* // +--+--+
* // | 1| 2|
* // +--+--+
* // |  3  |
* // +--+--+
*
* const node1 = graph.createNode(new Rect(0, 0, 50, 50))
* const node2 = graph.createNode(new Rect(0, 0, 50, 50))
* const node3 = graph.createNode(new Rect(0, 0, 125, 50))
*
* // Create a new PartitionGrid with two rows and two columns
* const grid = new PartitionGrid(2, 2)
*
* // Assign the nodes to their cells
* const cellIdMapper = new Mapper()
* cellIdMapper.set(node1, grid.createCellId(0, 0))
* cellIdMapper.set(node2, grid.createCellId(0, 1))
* cellIdMapper.set(node3, grid.createRowSpanId(1))
*
* const layoutData = new HierarchicLayoutData({
*   partitionGridData: {
*     grid: grid,
*     cellIds: cellIdMapper
*   }
* })
*
* graph.applyLayout(new HierarchicLayout(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23partitionGridData Online Documentation}
*/
partitionGridData: PartitionGridData;
/**
Gets or sets the layout data for the {@link SelfLoopCalculator}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23selfLoopCalculatorData Online Documentation}*/
selfLoopCalculatorData: SelfLoopCalculatorData;
/**
Gets or sets the mapping from edges to an additional weight used by the {@link ConstraintIncrementalLayerer}.
The layerer tries to keep edges with higher weights short.
@see ConstraintIncrementalLayerer#ADDITIONAL_EDGE_WEIGHT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23constraintIncrementalLayererAdditionalEdgeWeights Online Documentation}*/
constraintIncrementalLayererAdditionalEdgeWeights: ItemMapping<IEdge, number>;
/**
Gets or sets the collection of core nodes used by the {@link BFSLayerer}.
The core nodes will be placed in the first layer. If there are no core nodes, then nodes with in-degree <code>0</code> are considered
to be the core nodes.
The {@link BFSLayerer} is used when {@link HierarchicLayout#fromScratchLayeringStrategy} is set to {@link HierarchicLayoutLayeringStrategy#BFS}.
@see BFSLayerer#CORE_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23bfsLayererCoreNodes Online Documentation}*/
bfsLayererCoreNodes: ItemCollection<INode>;
/**
Gets or set the mapping from nodes to their layer index when using the {@link GivenLayersLayerer}.
Nodes with the same layer index are in the same layer while the layers are sorted according to their layer indices such
that the smallest layer index represents the top layer.
The {@link GivenLayersLayerer} is used when {@link HierarchicLayout#fromScratchLayeringStrategy} is set to {@link HierarchicLayoutLayeringStrategy#USER_DEFINED}.
@see GivenLayersLayerer#LAYER_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23givenLayersLayererIds Online Documentation}*/
givenLayersLayererIds: ItemMapping<INode, number>;
/**
* Gets or sets the mapping from edges to their directedness.
* Generally, the hierarchic layout algorithm assigns nodes to layers such that most of the edges point in the
* {@link MultiStageLayout#layoutOrientation main layout direction}. The directedness of an edge specifies whether it should comply with this strategy. More
* precisely, a value of <code>1</code> means that the edge should fully comply, a value of <code>-1</code> that it should comply inversely (the edge
* should point against the main layout direction), and a value of <code>0</code> means that the direction doesn't matter at all and the
* endpoints of the edges may be placed at the same layer. If there are conflicting preferences, edges with higher absolute
* values are more likely to point in the desired direction.
*
* <b>Warning:</b> Considering the edge directedness requires a special layering algorithm. Hence, {@link HierarchicLayout}
* ignores the specified {@link HierarchicLayout#fromScratchLayerer layerer} and automatically switches to the {@link ConstraintIncrementalLayerer} in such
* cases.
*
* The easiest option is to define all edges with the same directedness:
*
* <br><b>Treating all edges as directed</b>
* <pre>
* layoutData.edgeDirectedness.constant = 1
* </pre>
*
* Handling only certain edges differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set directedness for certain edges</b>
* <pre>
* // edge1 should be considered directed
* layoutData.edgeDirectedness.mapper.set(edge1, 1)
* // edge2 should be considered directed against the flow
* layoutData.edgeDirectedness.mapper.set(edge2, -1)
* // All other edges not set in the mapper are treated as undirected
* </pre>
*
* In cases where the directedness for each edge can be determined by looking at the edge itself it's often easier to just
* set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine edge directedness</b>
* <pre>
* // Treat edges as directed or undirected based on their style's arrowhead
* layoutData.edgeDirectedness.delegate = edge => {
*   const style = edge.style
*   // edges with with either no arrows on both ends or an arrow on both ends should be considered undirected
*   if (
*     (style.sourceArrow === null && style.targetArrow === null) ||
*     (style.sourceArrow !== null && style.targetArrow !== null)
*   ) {
*     return 0
*   }
*   // edges with only a target arrow are directed from source to target
*   if (style.targetArrow !== null) {
*     return 1
*   }
*   // edges with only a source arrow are directed from target to source
*   if (style.sourceArrow !== null) {
*     return -1
*   }
*   return 0
* }
* </pre>
*
* @see HierarchicLayout#EDGE_DIRECTEDNESS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23edgeDirectedness Online Documentation}
*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
* Gets or sets the mapping from edges to their thickness.
* The specified non-negative thickness is considered when calculating minimum distances so that there are no overlaps
* between edges and other graph elements. By default, each edge has thickness <code>0</code>.
*
* <h2>Note</h2>
*
* The port assignments {@link HierarchicLayoutPortAssignmentMode#ON_GRID on grid} and {@link HierarchicLayoutPortAssignmentMode#ON_SUBGRID on sub-grid} ignore the thickness of edges.
*
* The easiest option is to define the same thickness for all edges:
*
* <br><b>Specifying a uniform thickness for all edges</b>
* <pre>
* layoutData.edgeThickness = 3
* </pre>
*
* Handling only certain edges differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set a custom thickness for certain edges</b>
* <pre>
* layoutData.edgeThickness.mapper.set(edge1, 2)
* layoutData.edgeThickness.mapper.set(edge2, 5)
* // All other edges not set in the mapper have no defined thickness
* </pre>
*
* In cases where the thickness for each edge can be determined by looking at the edge itself it's often easier to just set
* a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine edge thickness</b>
* <pre>
* // Use the style's pen thickness as edge thickness
* layoutData.edgeThickness = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.thickness
*   }
*   return 0
* }
* </pre>
*
* @see HierarchicLayout#EDGE_THICKNESS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23edgeThickness Online Documentation}
*/
edgeThickness: ItemMapping<IEdge, number>;
/**
* Gets or sets the mapping from edges to their crossing cost.
* The crossing costs of the edges are considered during the crossing minimization phase. The total crossing costs are
* minimized using a heuristic where the cost of a crossing between two edges is defined as the product of the edges'
* crossing costs. A crossing with an edge that has a cost of zero is cost-free and such a crossing will therefore not be
* avoided. The default crossing cost of an edge is
* <code>1</code>, which is used for edges which do not have an individual crossing cost.
*
* <b>Warning:</b>
* The implemented crossing minimization is a heuristic. Therefore, there is no guarantee that crossing a certain edge is
* actually avoided, even though it has high crossing costs.
*
* <b>Warning:</b>
* Avoid using very large cost values. Due to the fact that costs need to be multiplied and summed up, this might lead to
* overflows and undesired results.
*
* When there are only a few edges to customize crossing costs for, the easiest way is usually to use the {@link ItemMapping#mapper}:
*
* <br><b>Defining edge crossing costs via a mapper</b>
* <pre>
* const layoutData = new HierarchicLayoutData()
* // Try harder to prevent crossings with edge1
* layoutData.edgeCrossingCosts.mapper.set(edge1, 1.5)
* // Crossings with edge2 are not as bad
* layoutData.edgeCrossingCosts.mapper.set(edge2, 0.2)
* </pre>
*
* If the crossing cost can readily be computed from the edge itself, the {@link ItemMapping#delegate} is often the more convenient option:
*
* <br><b>Defining edge crossing costs via a delegate</b>
* <pre>
* // The more labels an edge has, the more important it is to
* // prevent crossing with it
* layoutData.edgeCrossingCosts = edge => 1 + edge.labels.size
* </pre>
*
* @see HierarchicLayout#EDGE_CROSSING_COST_DP_KEY
* @see HierarchicLayoutData#reduceCriticalEdgeCrossings
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23edgeCrossingCosts Online Documentation}
*/
edgeCrossingCosts: ItemMapping<IEdge, number>;
/**
* Gets or sets the mapping from group nodes to the costs for a crossing with the group node border.
* Like the {@link HierarchicLayoutData#edgeCrossingCosts}, these costs are considered during the crossing minimization phase. The total costs are minimized using a
* heuristic where the cost of a crossing between a group node border and an edge is defined as the product of the
* respective group border crossing cost and the edge crossing cost. If no individual crossing costs are defined, the cost
* for crossing a group node border is
* <code>5</code>. This means that by default, a crossing between an edge and a group node border is more expensive than a crossing
* between two edges. Also note that a crossing with a group node border that has a cost of zero is cost-free and such a
* crossing will therefore not be avoided.
*
* <h2>Note</h2>
*
* In case of a vertical layout orientation (default), these costs only apply to vertical group borders. If the layout
* orientation is horizontal (e.g. left-to-right), the costs only apply to horizontal group borders.
*
* <b>Warning:</b>
* Avoid using very large cost values. Due to the fact that costs need to be multiplied and summed up, this might lead to
* overflows and undesired results.
*
* When there are only a few group nodes to customize crossing costs for, the easiest way is usually to use the {@link ItemMapping#mapper}:
*
* <br><b>Defining group nodes crossing costs via a mapper</b>
* <pre>
* // Try harder to prevent crossings with group node1
* layoutData.groupBorderCrossingCosts.mapper.set(node1, 7)
* // Crossings with group node2 are not as bad
* layoutData.groupBorderCrossingCosts.mapper.set(node2, 3)
* </pre>
*
* If the crossing cost can readily be computed from the group node itself, the {@link ItemMapping#delegate} is often the more convenient option:
*
* <br><b>Defining group nodes crossing costs via a delegate</b>
* <pre>
* // The more labels a group node has, the more important it is to
* // prevent crossing with it
* layoutData.groupBorderCrossingCosts = node => 5 + node.labels.size
* </pre>
*
* @see HierarchicLayout#EDGE_CROSSING_COST_DP_KEY
* @see HierarchicLayout#GROUP_BORDER_CROSSING_COST_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23groupBorderCrossingCosts Online Documentation}
*/
groupBorderCrossingCosts: ItemMapping<INode, number>;
/**
* Gets or sets the mapping that provides a {@link PreferredPlacementDescriptor} instance for edge
* {@link ILabel}s.
* Depending on how much customization is needed, some ways of setting {@link PreferredPlacementDescriptor}s are more
* convenient than others. For example, to set the same descriptor for all labels, you can just use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same PreferredPlacementDescriptor for all labels</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.constant = new PreferredPlacementDescriptor({
*   // Place labels along the edge
*   angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*   angle: 0,
*   // ... on either side
*   sideOfEdge: LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE,
*   // ... with a bit of distance to the edge
*   distanceToEdge: 5
* })
* </pre>
*
* If some labels should use custom placement or this has to be configured ahead of time, you can use the {@link ItemMapping#mapper} instead:
*
* <br><b>Customizing the placement of certain labels individually via a mapper</b>
* <pre>
* // Place label1 orthogonal to the edge anywhere on it
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label1,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.ANYWHERE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: Math.PI / 2
*   })
* )
* // Place label2 near the edge's source on either side of it, and make it parallel to the edge
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label2,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.AT_SOURCE,
*     sideOfEdge: LabelPlacements.RIGHT_OF_EDGE | LabelPlacements.LEFT_OF_EDGE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: 0
*   })
* )
* </pre>
*
* When the preferred placement can be inferred from the label itself, a delegate is usually the easiest choice:
*
* <br><b>Configuring preferred placement for all labels with a delegate</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.delegate = label => {
*   const customData = label.tag
*   const preferredPlacementDescriptor = new PreferredPlacementDescriptor({
*     angle: 0,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     // If the tag says to place the label in the center, put it in the center parallel to the edge's path
*     // All other labels can be placed anywhere, but on the side of the edge.
*     placeAlongEdge: customData.placeInCenter
*       ? LabelPlacements.AT_CENTER
*       : LabelPlacements.ANYWHERE,
*     sideOfEdge: customData.placeInCenter
*       ? LabelPlacements.ON_EDGE
*       : LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE
*   })
* }
* </pre>
*
* @see PreferredPlacementDescriptor
* @see LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23edgeLabelPreferredPlacement Online Documentation}
*/
edgeLabelPreferredPlacement: ItemMapping<ILabel, PreferredPlacementDescriptor>;
/**
* Gets or sets the collection of group nodes for which the ports of the adjacent edges are uniformly distributed.
* By default, there is no guarantee that the ports at group nodes are uniformly distributed. This setting only affects the
* two sides of a group node that are in flow and against the flow of the layout orientation. For example, for orientation
* {@link LayoutOrientation#TOP_TO_BOTTOM} only the edges incident to the top and bottom group node side are
* considered but not the edges at the left and right side.
*
* The uniform group port assignment considers the {@link HierarchicLayout#edgeToEdgeDistance edge-to-edge distance} such that two ports keep the specified distance to
* each other. Group nodes may in consequence be enlarged to satisfy that constraints, potentially generating less compact
* layouts. The distance from the group node border to the first/last port on each group side may be influenced by using
* {@link GroupingKeys#GROUP_NODE_INSETS_DP_KEY group node insets}.
*
* Importantly, it is not guaranteed that the group node ports of a specific side can be uniformly distributed. In the
* following cases ports of a specific side are not uniformly distributed:
*
* <ol>
* <li>There are edges that cross the group node border because they connect to a child node of the group.</li>
* <li>There is a self-loop at the group node which has its source and target port at the same group node side.</li>
* </ol>
* <b>Warning:</b> Enabling uniform group ports for at least one group node may significantly increase the runtime of the algorithm. Also,
* the uniform distribution may be skipped in case that the {@link HierarchicLayout#maximumDuration maximum duration} is restricted and there is no time left.
*
* <h2>Note</h2>
*
* Forcing group nodes to have uniform ports can have several side-effects and also affect other parts of the layout. In
* general, results can become less compact and there might be more bends, because bends may be introduced where they would
* not be necessary when ports are not uniformly distributed.
*
* <br><b>Dynamically determine whether a node is a group node</b>
* <pre>
* layoutData.uniformPortAssignmentGroups.delegate = node => graph.isGroupNode(node)
* </pre>
*
* <br><b>Using a static collection of group nodes</b>
* <pre>
* layoutData.uniformPortAssignmentGroups.source = graph.nodes.filter(node =>
*   graph.isGroupNode(node)
* )
* </pre>
*
* @see HierarchicLayout#UNIFORM_PORT_ASSIGNMENT_GROUPS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/HierarchicLayoutData%23uniformPortAssignmentGroups Online Documentation}
*/
uniformPortAssignmentGroups: ItemCollection<INode>;
static $class: Class;
private $_HierarchicLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is HierarchicLayoutData;
}
export interface IncrementalHintItemMapping extends ContextItemMapping<IModelItem, any, IIncrementalHintsFactory> {
}
/**
* Provides different ways to define a mapping from model items to incremental hints for use with
* {@link HierarchicLayoutData}.
* Only one of the provided ways to define a mapping can be used at the same time, therefore only one of the properties of
* this class should be set on an instance.
*
* Functions, {@link IMapper}s and constant values are automatically converted to
* {@link IncrementalHintItemMapping}s. For example:
*
* <pre>
* // Constant
* new HierarchicLayoutData({
*   incrementalHints: hintsFactory.createSequenceIncrementallyHint(node)
* })
*
* // Context Delegate
* new HierarchicLayoutData({
*   incrementalHints: (item, factory) => incremental ? factory.createLayerIncrementallyHint(item) : null
* })
* // or
* const hintsFactory = hierarchicLayout.createIncrementalHintsFactory()
* new HierarchicLayoutData({
*   incrementalHints: item => incremental ? hintsFactory.createLayerIncrementallyHint(item) : null
* })
*
* // Mapper
* new HierarchicLayoutData({ incrementalHints: new Mapper() })
* // or a JavaScript Map
* new HierarchicLayoutData({ incrementalHints = new Map() })
* </pre>
*
* Note that the properties {@link IncrementalHintItemMapping#incrementalLayeringNodes} and {@link IncrementalHintItemMapping#incrementalSequencingItems} cannot be defined directly using this converter.
*
* Furthermore, the conversion supports a parameter object to initialize the properties. For example:
*
* <pre>
* // Create a new IncrementalHintItemMapping from a JavaScript Map and initialize its 'mapper' with the new Map.
* // This also works for the different properties as well.
* const mapping = IncrementalHintItemMapping.from({
*   mapper: new Map()
* })
*
* // Or, instead of providing the specific property, the 'includes' key can be used
* const mapping = IncrementalHintItemMapping.from({
*   includes: new Map()
* })
* </pre>
*
* Elements which should be placed incrementally can be provided as collections:
*
* <br><b>Creating incremental hints for nodes and edges</b>
* <pre>
* // create a HierarchicLayout which rearranges
* // only the incremental graph elements
* const hl = new HierarchicLayout({ layoutMode: LayoutMode.INCREMENTAL })
*
* // provide additional data to configure the HierarchicLayout
* const hlData = new HierarchicLayoutData()
* // specify the nodes to rearrange
* hlData.incrementalHints.incrementalLayeringNodes.source = incrementalNodes
* // specify the edges to rearrange
* hlData.incrementalHints.incrementalSequencingItems.source = incrementalEdges
*
* graph.applyLayout(hl, hlData)
* </pre>
*
* More specific hints can be created using an {@link IIncrementalHintsFactory}:
*
* <br><b>Creating incremental hints for nodes and edges using a factory</b>
* <pre>
* // create a HierarchicLayout which rearranges
* // only the incremental graph elements
* const hl = new HierarchicLayout({ layoutMode: LayoutMode.INCREMENTAL })
*
* // provide additional data to configure the HierarchicLayout
* const hlData = new HierarchicLayoutData()
*
* hlData.incrementalHints.contextDelegate = (item, factory) => {
*   if (item instanceof INode && incrementalNodes.includes(item)) {
*     // create a hint for incremental nodes
*     if (graph.isGroupNode(item)) {
*       // special hint for groups
*       return factory.createIncrementalGroupHint(item)
*     }
*     if (fixedNodes.includes(item)) {
*       // exact layer for the fixedNodes
*       return factory.createUseExactLayerCoordinatesHint(item)
*     }
*     // simple layer hint for all other nodes
*     return factory.createLayerIncrementallyHint(item)
*   }
*   if (item instanceof IEdge && incrementalEdges.includes(item)) {
*     // sequence hint for incremental edges
*     return factory.createSequenceIncrementallyHint(item)
*   }
*   // all other items don't get hints
*   return null
* }
*
* // provide hints
*
* graph.applyLayout(hl, hlData)
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IncrementalHintItemMapping Online Documentation}
*/
export class IncrementalHintItemMapping {
/**
Creates a new instance of {@link IncrementalHintItemMapping}.
@param [options] The parameters to pass.
@param [options.incrementalLayeringNodes] An item collection of nodes that are mapped to {@link IIncrementalHintsFactory#createLayerIncrementallyHint incremental layering hints}. This option sets the {@link IncrementalHintItemMapping#incrementalLayeringNodes} property on the created object.
@param [options.incrementalSequencingItems] An item collection of nodes or edges that are mapped to {@link IIncrementalHintsFactory#createSequenceIncrementallyHint incremental sequencing hints}. This option sets the {@link IncrementalHintItemMapping#incrementalSequencingItems} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IncrementalHintItemMapping Online Documentation}*/
constructor(options?: {
incrementalLayeringNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
incrementalSequencingItems?: ItemCollection<IModelItem> | ItemCollectionConvertible<IModelItem>;
});
/**
Creates a {@link IncrementalHintItemMapping} instance from the given object by performing automatic type
conversion.
@param itemMappingLike The object to convert to a {@link IncrementalHintItemMapping}.
@returns Either <code>itemMappingLike</code> if it is already a {@link IncrementalHintItemMapping} or a new instance, initialized to
/// the values found in <code>itemMappingLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IncrementalHintItemMapping%23from(IncrementalHintItemMapping) Online Documentation}*/
static from(itemMappingLike: IncrementalHintItemMapping | IncrementalHintItemMappingConvertible): IncrementalHintItemMapping;
/**
* Gets or sets an item collection of nodes that are mapped to {@link IIncrementalHintsFactory#createLayerIncrementallyHint incremental layering hints}.
* <pre>
* // create a HierarchicLayout which rearranges
* // only the incremental graph elements
* const hl = new HierarchicLayout({ layoutMode: LayoutMode.INCREMENTAL })
*
* // provide additional data to configure the HierarchicLayout
* const hlData = new HierarchicLayoutData()
* // specify the nodes to rearrange
* hlData.incrementalHints.incrementalLayeringNodes.source = incrementalNodes
* // specify the edges to rearrange
* hlData.incrementalHints.incrementalSequencingItems.source = incrementalEdges
*
* graph.applyLayout(hl, hlData)
* </pre>
*
* <pre>
* // create a HierarchicLayout which rearranges
* // only the incremental graph elements
* const hl = new HierarchicLayout({ layoutMode: LayoutMode.INCREMENTAL })
*
* // provide additional data to configure the HierarchicLayout
* const hlData = new HierarchicLayoutData()
* // specify the nodes to rearrange
* hlData.incrementalHints.incrementalLayeringNodes.delegate = node => isIncremental(node)
* // specify the nodes and edges to rearrange
* hlData.incrementalHints.incrementalSequencingItems.delegate = item => isIncremental(item)
*
* graph.applyLayout(hl, hlData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IncrementalHintItemMapping%23incrementalLayeringNodes Online Documentation}
*/
incrementalLayeringNodes: ItemCollection<INode>;
/**
* Gets or sets an item collection of nodes or edges that are mapped to {@link IIncrementalHintsFactory#createSequenceIncrementallyHint incremental sequencing hints}.
* <pre>
* // create a HierarchicLayout which rearranges
* // only the incremental graph elements
* const hl = new HierarchicLayout({ layoutMode: LayoutMode.INCREMENTAL })
*
* // provide additional data to configure the HierarchicLayout
* const hlData = new HierarchicLayoutData()
* // specify the nodes to rearrange
* hlData.incrementalHints.incrementalLayeringNodes.source = incrementalNodes
* // specify the edges to rearrange
* hlData.incrementalHints.incrementalSequencingItems.source = incrementalEdges
*
* graph.applyLayout(hl, hlData)
* </pre>
*
* <pre>
* // create a HierarchicLayout which rearranges
* // only the incremental graph elements
* const hl = new HierarchicLayout({ layoutMode: LayoutMode.INCREMENTAL })
*
* // provide additional data to configure the HierarchicLayout
* const hlData = new HierarchicLayoutData()
* // specify the nodes to rearrange
* hlData.incrementalHints.incrementalLayeringNodes.delegate = node => isIncremental(node)
* // specify the nodes and edges to rearrange
* hlData.incrementalHints.incrementalSequencingItems.delegate = item => isIncremental(item)
*
* graph.applyLayout(hl, hlData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/IncrementalHintItemMapping%23incrementalSequencingItems Online Documentation}
*/
incrementalSequencingItems: ItemCollection<IModelItem>;
static $class: Class;
private $_IncrementalHintItemMapping: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IncrementalHintItemMapping;
}
/** Automatic conversion to {@link IncrementalHintItemMapping}. */
type IncrementalHintItemMappingConvertible = any | ((arg1: IModelItem, arg2: IIncrementalHintsFactory) => any) | ((arg: IModelItem) => any) | IMapper<IModelItem, any> | Map<Object, any> | {
includes: any;
} | {
constant: any;
} | {
delegate: (item: Object) => any;
} | {
contextDelegate: (item: Object, ctx: Object) => any;
} | {
mapper: Map<Object, any>;
} | {
incrementalLayeringNodes: Object;
} | {
incrementalSequencingItems: Object;
};
export interface ItemCollection<TItem> extends YObject {
}
/**
* Provides different ways to define the items in a collection for use with {@link LayoutData}.
* Only one of the provided ways to define a collection can be used at the same time, therefore only one of the properties
* {@link ItemCollection#item}, {@link ItemCollection#items}, {@link ItemCollection#mapper}, and {@link ItemCollection#delegate} should be set on each instance.
*
* Items in the {@link ItemCollection#excludes} property are explicitly excluded from this collection, even if they are included in one of the {@link ItemCollection#item}, {@link ItemCollection#items}, {@link ItemCollection#mapper}, or {@link ItemCollection#delegate}
* properties.
*
* This class supports {@link LayoutData} implementations and does not usually have to be used explicitly.
*
* Functions, {@link IMapper}s, lists and single items are automatically converted to {@link ItemCollection}s. For example:
*
* <pre>
* // Delegate
* new TreeLayoutData({ assistantNodes: node => isAssistant(node) })
*
* // Item
* new TreeLayoutData({ assistantNodes: node1 })
*
* // Items
* new TreeLayoutData({ assistantNodes: [node1, node2] })
*
* // Mapper
* new TreeLayoutData({ assistantNodes: new Mapper() })
* // or a JavaScript Map
* new TreeLayoutData({ assistantNodes: new Map() })
*
* // Source
* new TreeLayoutData({ assistantNodes: graphComponent.selection.selectedNodes })
* </pre>
*
* Furthermore, the conversion supports a parameter object to initialize the properties. For example:
*
* <pre>
* // This provides a convenient way of specifying the subgraph for the algorithm:
* const algorithm = new ShortestPath({
*   // single source - single sink
*   source: startNode,
*   sink: endNode,
*   // only consider elliptical nodes in the graph
*   subgraphNodes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
* })
*
* // This provides a convenient way of specifying the excluded elements of the ItemCollection:
* const algorithm = new ShortestPath({
*   // single source - single sink
*   source: startNode,
*   sink: endNode,
*   subgraphNodes: {
*     // only consider elliptical nodes in the graph
*     includes: node =>
*       node.style instanceof ShapeNodeStyle && node.style.shape === ShapeNodeShape.ELLIPSE,
*     // but ignore the first node, regardless of its shape
*     excludes: graph.nodes.first()
*   }
* })
*
* // Create a new ItemCollection from a JavaScript Map and initialize its 'mapper' with the new Map.
* // This also works for the other properties as well.
* const collection = ItemCollection.from({
*   mapper: new Map()
* })
*
* // Or, instead of providing the specific property, the 'includes' key can be used
* const collection = ItemCollection.from({
*   includes: new Map()
* })
* </pre>
*
* Type parameter <code>TItem</code>: The type of the items in the collection.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection Online Documentation}
*/
export class ItemCollection<TItem> {
/**
Constructor overload that directly sets the {@link ItemCollection#delegate} that describes which items are contained in this instance.
@param delegate A delegate that implements the mapping.
@param [excludes=null] A collection of items which are explicitly excluded from this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23ItemCollection(function(TItem)%3Aboolean%2CItemCollection%3CTItem%3E) Online Documentation}*/
constructor(delegate: (obj: TItem) => boolean, excludes?: ItemCollection<TItem> | null | ItemCollectionConvertible<TItem>);
/**
Constructor overload that directly sets the single {@link ItemCollection#item}.
@param item The single item contained in this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23ItemCollection(TItem) Online Documentation}*/
constructor(item: TItem);
/**
Constructor overload that directly sets the {@link ItemCollection#mapper} that describes which items are contained in this instance.
@param mapper An {@link IMapper}that implements the mapping.
@param [excludes=null] A collection of items which are explicitly excluded from this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23ItemCollection(IMapper%3CTItem%2Cboolean%3E%2CItemCollection%3CTItem%3E) Online Documentation}*/
constructor(mapper: IMapper<TItem, boolean> | IMapperConvertible<TItem, boolean>, excludes?: ItemCollection<TItem> | null | ItemCollectionConvertible<TItem>);
/**
Constructor overload that directly sets the {@link ItemCollection#source} that lists which items are contained in this instance.
@param source An enumerable that lists the elements in this instance.
@param [excludes=null] A collection of items which are explicitly excluded from this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23ItemCollection(IEnumerable%3CTItem%3E%2CItemCollection%3CTItem%3E) Online Documentation}*/
constructor(source: IEnumerable<TItem> | IEnumerableConvertible<TItem>, excludes?: ItemCollection<TItem> | null | ItemCollectionConvertible<TItem>);
/**
Constructor overload that directly sets the {@link ItemCollection#items} that hold the items are contained in this instance.
@param items An enumerable that holds the elements in this instance.
@param [excludes=null] A collection of items which are explicitly excluded from this collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23ItemCollection(ICollection%3CTItem%3E%2CItemCollection%3CTItem%3E) Online Documentation}*/
constructor(items: ICollection<TItem> | ICollectionConvertible<TItem>, excludes?: ItemCollection<TItem> | null | ItemCollectionConvertible<TItem>);
/**
Default constructor that does not explicitly initialize any of the {@link ItemCollection#mapper}, {@link ItemCollection#delegate}, {@link ItemCollection#item}, {@link ItemCollection#items} or {@link ItemCollection#source} properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23ItemCollection() Online Documentation}*/
constructor();
/**
Creates a ItemCollection instance from the given object by performing automatic type conversion.
Type parameter <code>TItem</code>: The type of the items in the collection.
@param itemCollectionLike The object to convert to a ItemCollection.
@returns Either <code>itemCollectionLike</code> if it is already a ItemCollection or a new instance, initialized to /// the values found in
<code>itemCollectionLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23from(ItemCollection%3CTItem%3E) Online Documentation}*/
static from<TItem>(itemCollectionLike: ItemCollection<TItem> | ItemCollectionConvertible<TItem>): ItemCollection<TItem>;
/**
Provides a mapper describing which elements are contained in this {@link ItemCollection}.
@param adapter The graph adapter the mapper shall be provided for.
@param layout The layout using the item collection.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23provideMapper(LayoutGraphAdapter%2CILayoutAlgorithm) Online Documentation}*/
provideMapper(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm): IMapper<TItem, boolean> | null;
/**
Gets or sets a mapping from the items to a boolean value.
This property does not get initialized by default and needs to be set to a new instance by the user, if required.
The mapped boolean value specifies whether an item is contained in the collection.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23mapper Online Documentation}*/
mapper: IMapper<TItem, boolean> | null;
/**
Gets or sets a delegate from the items to a boolean value.
The returned boolean value describes whether an item is contained in the collection.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23delegate Online Documentation}*/
delegate: ((obj: TItem) => boolean) | null;
/**
Gets or sets a collection of items.
This property is initialized by default and the returned collection can be used directly to add items.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23items Online Documentation}*/
items: ICollection<TItem>;
/**
Get or sets an enumerable of items.
This collection contains exactly the items in the given enumerable.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23source Online Documentation}*/
source: IEnumerable<TItem> | null;
/**
Gets or sets the single item this item collection contains.
This facilitates declaring collections which contain only one single item.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23item Online Documentation}*/
item: TItem | null;
/**
A collection of items which are explicitly excluded from this collection.
Items in the {@link ItemCollection#excludes} property are explicitly excluded from this collection, even if they are included in one of the {@link ItemCollection#item}, {@link ItemCollection#items}, {@link ItemCollection#mapper}, or {@link ItemCollection#delegate}
properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollection%23excludes Online Documentation}*/
excludes: ItemCollection<TItem>;
static $class: Class;
private $_ItemCollection: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TItem>(o: any): o is ItemCollection<TItem>;
}
/** Automatic conversion to {@link ItemCollection}. */
type ItemCollectionConvertible<TItem> = TItem[] | ICollection<TItem> | IEnumerable<TItem> | IMapper<TItem, boolean> | Map<TItem, boolean> | ((item: TItem) => boolean) | TItem | {
includes?: any;
excludes?: any;
} | {
delegate: (item: TItem) => boolean;
} | {
item: TItem;
} | {
items: Object;
} | {
mapper: Map<TItem, boolean>;
} | {
source: Object;
};
export interface ItemCollectionMapping<TItem, TValue> extends YObject {
}
/**
Provides a way to define a mapping from a collection of items to a common value for each item for usage in conjunction
with class {@link LayoutData}.
This class supports {@link LayoutData} implementations and does not usually have to be instantiated by client code
explicitly. Instances of this class are properties of concrete {@link LayoutData} implementations.
Type parameter <code>TItem</code>: The type of the items that are mapped.
Type parameter <code>TValue</code>: The type of the values the items are mapped to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollectionMapping Online Documentation}*/
export class ItemCollectionMapping<TItem, TValue> {
/**
Creates a new instance of ItemCollectionMapping.
@param keyType The type of the items that are mapped.
@param valueType The type of the values the items are mapped to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollectionMapping Online Documentation}*/
constructor(keyType: Class, valueType: Class);
/**
Adds the specified value which will be associated to all items in the returned item collection.
Use the returned {@link ItemCollection} to conveniently specify the items which should be mapped to the the specified collection value.
@param collectionValue The value the items of the returned collection are mapped to.
@returns The items that are mapped to the given value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollectionMapping%23add(TValue) Online Documentation}*/
add(collectionValue: TValue | null): ItemCollection<TItem>;
/**
Clears all mappings that were defined using {@link ItemCollectionMapping#add}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemCollectionMapping%23clear() Online Documentation}*/
clear(): void;
static $class: Class;
private $_ItemCollectionMapping: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TItem, TValue>(o: any): o is ItemCollectionMapping<TItem, TValue>;
}
export interface ItemMapping<TItem, TValue> extends YObject {
}
/**
* Provides different ways to define a mapping from items to values for use with {@link LayoutData}.
* Only one of the provided ways to define a mapping can be used at the same time, therefore only one of the properties {@link ItemMapping#mapper}, {@link ItemMapping#delegate}
* and {@link ItemMapping#constant} should be set on each instance.
*
* This class supports {@link LayoutData} implementations and does not usually have to be instantiated by client code
* explicitly. Instances of this class are properties of concrete {@link LayoutData} implementations.
*
* Functions, {@link IMapper}s and constant values are automatically converted to {@link ItemMapping}s. For example:
*
* <pre>
* // Constant
* new HierarchicLayoutData({
*   sourcePortCandidates: [
*     PortCandidate.createCandidate('east', 0),
*     PortCandidate.createCandidate('west', 0)
*   ]
* })
*
* // Delegate
* new HierarchicLayoutData({
*   sourcePortCandidates: () =>
*     east ? [PortCandidate.createCandidate('east', 0)] : [PortCandidate.createCandidate('west', 0)]
* })
*
* // Mapper
* new HierarchicLayoutData({ sourceGroupIds: new Mapper() })
* // or a JavaScript Map
* new HierarchicLayoutData({ sourceGroupIds: new Map() })
* </pre>
*
* Furthermore, the conversion supports a parameter object to initialize the properties. For example:
*
* <pre>
* // Create a new ItemMapping from a JavaScript Map and initialize its 'mapper' with the new Map.
* // This also works for the different properties as well.
* const mapping = ItemMapping.from({
*   mapper: new Map()
* })
* // Or, instead of providing the specific property, the 'includes' key can be used
* const mapping = ItemMapping.from({
*   includes: new Map()
* })
* </pre>
*
* Type parameter <code>TItem</code>: The type of the items that are mapped.
* Type parameter <code>TValue</code>: The type of the values the items are mapped to.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping Online Documentation}
*/
export class ItemMapping<TItem, TValue> {
/**
Constructor overload that directly sets the {@link ItemMapping#delegate}.
@param delegate A delegate that implements the mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping%23ItemMapping(function(TItem)%3ATValue) Online Documentation}*/
constructor(delegate: (key: TItem) => TValue);
/**
Constructor overload that directly sets a {@link ItemMapping#constant} mapping.
@param constant A constant value all items are mapped to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping%23ItemMapping(TValue) Online Documentation}*/
constructor(constant: TValue);
/**
Constructor overload that directly sets the {@link ItemMapping#mapper}.
@param mapper An {@link IMapper} that implements the mapping.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping%23ItemMapping(IMapper%3CTItem%2CTValue%3E) Online Documentation}*/
constructor(mapper: IMapper<TItem, TValue> | IMapperConvertible<TItem, TValue>);
/**
Default constructor that does not explicitly initialize any of the {@link ItemMapping#mapper}, {@link ItemMapping#delegate} or {@link ItemMapping#constant} properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping%23ItemMapping() Online Documentation}*/
constructor();
/**
Creates a ItemMapping instance from the given object by performing automatic type conversion.
Type parameter <code>TItem</code>: The type of the items that are mapped.
Type parameter <code>TValue</code>: The type of the values the items are mapped to.
@param itemMappingLike The object to convert to a ItemMapping.
@returns Either <code>itemMappingLike</code> if it is already a ItemMapping or a new instance, initialized to /// the values found in
<code>itemMappingLike</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping%23from(ItemMapping%3CTItem%2CTValue%3E) Online Documentation}*/
static from<TItem, TValue>(itemMappingLike: ItemMapping<TItem, TValue> | ItemMappingConvertible<TItem, TValue>): ItemMapping<TItem, TValue>;
/**
Provides a mapper mapping the items to values.
This implementation returns the {@link ItemMapping#mapper} if one has been set, otherwise it will create a mapper from the {@link ItemMapping#delegate} if one has been
specified or uses a constant mapper if a {@link ItemMapping#constant} has been specified. If none have been specified, <code>null</code> is returned.
@param adapter The graph adapter the mapper shall be provided for.
@param layout The layout using the mapper.
@returns A mapper that maps items to values or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping%23provideMapper(LayoutGraphAdapter%2CILayoutAlgorithm) Online Documentation}*/
provideMapper(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm): IMapper<TItem, TValue> | null;
/**
Gets or sets a delegate that implements the mapping.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping%23delegate Online Documentation}*/
delegate: ((key: TItem) => TValue) | null;
/**
Gets or sets an {@link IMapper} that realizes the mapping.
This property is initialized by default and the returned {@link IMapper} can be used directly. Setting this property will override any
previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping%23mapper Online Documentation}*/
mapper: IMapper<TItem, TValue>;
/**
Gets or sets a constant value all items are mapped to.
Setting this property will override any previously set properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ItemMapping%23constant Online Documentation}*/
constant: TValue | null;
static $class: Class;
private $_ItemMapping: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance<TItem, TValue>(o: any): o is ItemMapping<TItem, TValue>;
}
/** Automatic conversion to {@link ItemMapping}. */
type ItemMappingConvertible<TItem, TValue> = IMapper<TItem, TValue> | Map<TItem, TValue> | ((item: TItem) => TValue | null) | TValue | {
includes: any;
} | {
constant: TValue;
} | {
delegate: (item: TItem) => TValue | null;
} | {
mapper: Map<TItem, TValue>;
};
export interface LabelingData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link GenericLabeling}.
* The following example shows how to create a new instance of {@link LabelingData} and use it with a
* {@link GenericLabeling}:
*
* <br><b>Creating an instance of LabelingData</b>
* <pre>
* const layoutData = new LabelingData()
* layoutData.affectedLabels.source = graphComponent.selection.selectedLabels
* layoutData.edgeLabelPreferredPlacement.delegate = label =>
*   new PreferredPlacementDescriptor({
*     // Place short labels horizontally, and others along the edge
*     angleReference:
*       label.text.length < 5
*         ? LabelAngleReferences.ABSOLUTE
*         : LabelAngleReferences.RELATIVE_TO_EDGE_FLOW
*   })
*
* graphComponent.graph.applyLayout(new GenericLabeling(), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link LabelingData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of LabelingData</b>
* <pre>
* const layoutData = new LabelingData({
*   affectedLabels: graphComponent.selection.selectedLabels,
*   edgeLabelPreferredPlacement: label =>
*     new PreferredPlacementDescriptor({
*       // Place short labels horizontally, and others along the edge
*       angleReference:
*         label.text.length < 5
*           ? LabelAngleReferences.ABSOLUTE
*           : LabelAngleReferences.RELATIVE_TO_EDGE_FLOW
*     })
* })
*
* graphComponent.graph.applyLayout(new GenericLabeling(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData Online Documentation}
*/
export class LabelingData {
/**
Creates a new instance of {@link LabelingData} which helps configuring {@link GenericLabeling}.
@param [options] The parameters to pass.
@param [options.affectedLabels] The collection of labels that shall be placed by the algorithm. This option sets the {@link LabelingData#affectedLabels} property on the created object.
@param [options.nodeLabelModels] The mapping from node labels to an {@link INodeLabelLayoutModel} used by the labeling algorithm instead of the original label model. This option sets the {@link LabelingData#nodeLabelModels} property on the created object.
@param [options.edgeLabelModels] The mapping from edge labels to an {@link IEdgeLabelLayoutModel} used by the labeling algorithm instead of the original label model. This option sets the {@link LabelingData#edgeLabelModels} property on the created object.
@param [options.sourceGroupIds] A mapping from edges to an object representing their source edge group. This option sets the {@link LabelingData#sourceGroupIds} property on the created object.
@param [options.targetGroupIds] A mapping from edges to an object representing their target edge group. This option sets the {@link LabelingData#targetGroupIds} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link LabelingData#nodeHalos} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link LabelingData#abortHandler} property on the created object.
@param [options.edgeLabelPreferredPlacement] The mapping that provides a {@link PreferredPlacementDescriptor} instance for edge {@link ILabel}s. This option sets the {@link LabelingData#edgeLabelPreferredPlacement} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData Online Documentation}*/
constructor(options?: {
affectedLabels?: DpKeyItemCollection<ILabel> | DpKeyItemCollectionConvertible<ILabel>;
nodeLabelModels?: ItemMapping<ILabel, INodeLabelLayoutModel> | ItemMappingConvertible<ILabel, INodeLabelLayoutModel>;
edgeLabelModels?: ItemMapping<ILabel, IEdgeLabelLayoutModel> | ItemMappingConvertible<ILabel, IEdgeLabelLayoutModel>;
sourceGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
abortHandler?: AbortHandler | null;
edgeLabelPreferredPlacement?: ItemMapping<ILabel, PreferredPlacementDescriptor> | ItemMappingConvertible<ILabel, PreferredPlacementDescriptor>;
});
/**
* Gets or sets the collection of labels that shall be placed by the algorithm.
* Note that the {@link DpKeyItemCollection#dpKey} has to be set to the key assigned to {@link LabelingBase#affectedLabelsDpKey}.
*
* Defining the subset of labels that should be laid out can be done in various ways, mostly depending on which option is
* more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of affected nodes</b>
* <pre>
* layoutData.affectedLabels.source = graphComponent.selection.selectedLabels
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual nodes as affected nodes</b>
* <pre>
* layoutData.affectedLabels.items.add(label1)
* layoutData.affectedLabels.items.add(label2)
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every node
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether a node is affected or not</b>
* <pre>
* // We assume here that all labels have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.affectedLabels.delegate = label => label.tag.includeInLayout
* </pre>
*
* @see LabelingBase#affectedLabelsDpKey
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData%23affectedLabels Online Documentation}
*/
affectedLabels: DpKeyItemCollection<ILabel>;
/**
* Gets or sets the mapping from node labels to an {@link INodeLabelLayoutModel} used by the labeling algorithm
* instead of the original label model.
* If a label has no mapping, the original model is used as normal.
*
* A model parameter that fits the calculated position of the replacement model best is set for the original label model.
*
* This feature is especially useful to restrict a free model. For example, to place a label upon its associated node, a
* centered {@link DiscreteNodeLabelLayoutModel} can be used as replacement model.
*
* Specifying a custom label model for node labels can be done in different ways, depending on which option is the most
* convenient for a given use case. If only a few labels must be customized in this way and most should be left alone,
* using the {@link ItemMapping#mapper} is usually the easiest option:
*
* <br><b>Using a mapper to specify a different label model for certain labels</b>
* <pre>
* // Place label1 either above or below the node
* layoutData.nodeLabelModels.mapper.set(
*   label1,
*   new DiscreteNodeLabelLayoutModel(
*     DiscreteNodeLabelPositions.NORTH | DiscreteNodeLabelPositions.SOUTH
*   )
* )
* // Place label2 in the center of the node
* layoutData.nodeLabelModels.mapper.set(
*   label2,
*   new DiscreteNodeLabelLayoutModel(DiscreteNodeLabelPositions.CENTER)
* )
* // All other labels not explicitly configured follow their respective ILabelModel and its valid parameters
* </pre>
*
* When the label model to use is easy to determine from each label itself, using a delegate is often easier:
*
* <br><b>Using a delegate to specify a label model for each label</b>
* <pre>
* layoutData.nodeLabelModels.delegate = label => {
*   const customData = label.tag
*   // Place labels with a special property in their tag in the center
*   // of their node and place all others as usual.
*   return customData.placeInCenter
*     ? new DiscreteNodeLabelLayoutModel(DiscreteNodeLabelPositions.CENTER)
*     : null
* }
* </pre>
*
* Finally, to override the default behavior in the same way for every single label, a constant could also be used:
*
* <br><b>Specifying the same label model for all node labels</b>
* <pre>
* layoutData.nodeLabelModels.constant = new DiscreteNodeLabelLayoutModel(
*   DiscreteNodeLabelPositions.BOTTOM |
*     DiscreteNodeLabelPositions.BOTTOM_LEFT |
*     DiscreteNodeLabelPositions.BOTTOM_RIGHT
* )
* </pre>
*
* @see LabelingBase#LABEL_MODEL_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData%23nodeLabelModels Online Documentation}
*/
nodeLabelModels: ItemMapping<ILabel, INodeLabelLayoutModel>;
/**
* Gets or sets the mapping from edge labels to an {@link IEdgeLabelLayoutModel} used by the labeling algorithm
* instead of the original label model.
* If a label has no mapping, the original model is used as normal.
*
* A model parameter that fits the calculated position of the replacement model best is set for the original label model.
*
* This feature is especially useful to restrict a free model. For example, to place a label upon its associated edge, a
* centered {@link SliderEdgeLabelLayoutModel} can be used as replacement model.
*
* Specifying a custom label model for edge labels can be done in different ways, depending on which option is the most
* convenient for a given use case. If only a few labels must be customized in this way and most should be left alone,
* using the {@link ItemMapping#mapper} is usually the easiest option:
*
* <br><b>Using a mapper to specify a different label model for certain labels</b>
* <pre>
* // Place label1 at the source side on the edge
* layoutData.edgeLabelModels.mapper.set(
*   label1,
*   new DiscreteEdgeLabelLayoutModel(DiscreteEdgeLabelPositions.SOURCE_CENTER)
* )
* // Place label2 somewhere along the side of the edge
* layoutData.edgeLabelModels.mapper.set(label2, new SliderEdgeLabelLayoutModel(SliderMode.SIDE))
* // All other labels not explicitly configured follow their respective ILabelModel and its valid parameters
* </pre>
*
* When the label model to use is easy to determine from each label itself, using a delegate is often easier:
*
* <br><b>Using a delegate to specify a label model for each label</b>
* <pre>
* layoutData.edgeLabelModels.delegate = label => {
*   const customData = label.tag
*   // Place labels with a special property in their tag in the center
*   // of their edge and place all others as usual.
*   return customData.placeInCenter
*     ? new DiscreteEdgeLabelLayoutModel(DiscreteEdgeLabelPositions.center)
*     : null
* }
* </pre>
*
* Finally, to override the default behavior in the same way for every single label, a constant could also be used:
*
* <br><b>Specifying the same label model for all edge labels</b>
* <pre>
* layoutData.edgeLabelModels.constant = new SliderEdgeLabelLayoutModel(SliderMode.CENTER)
* </pre>
*
* @see LabelingBase#LABEL_MODEL_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData%23edgeLabelModels Online Documentation}
*/
edgeLabelModels: ItemMapping<ILabel, IEdgeLabelLayoutModel>;
/**
Gets or sets a mapping from edges to an object representing their source edge group.
Source and target edge groups are only used for labeling in conjunction with the {@link LabelingBase#edgeGroupOverlapAllowed} setting where a label may overlap an
edge from the same group. Edges don't have to share parts of the same path for this purpose (as would be the result from
other layout algorithms that use edge grouping). Specifying source and target groups will not change edge paths, nor
will the labels be placed any different. It is mainly a tool to allow selective label/edge overlaps in certain cases.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData%23sourceGroupIds Online Documentation}*/
sourceGroupIds: ItemMapping<IEdge, any>;
/**
Gets or sets a mapping from edges to an object representing their target edge group.
Source and target edge groups are only used for labeling in conjunction with the {@link LabelingBase#edgeGroupOverlapAllowed} setting where a label may overlap an
edge from the same group. Edges don't have to share parts of the same path for this purpose (as would be the result from
other layout algorithms that use edge grouping). Specifying source and target groups will not change edge paths, nor
will the labels be placed any different. It is mainly a tool to allow selective label/edge overlaps in certain cases.
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData%23targetGroupIds Online Documentation}*/
targetGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
* Gets or sets the mapping that provides a {@link PreferredPlacementDescriptor} instance for edge
* {@link ILabel}s.
* Depending on how much customization is needed, some ways of setting {@link PreferredPlacementDescriptor}s are more
* convenient than others. For example, to set the same descriptor for all labels, you can just use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same PreferredPlacementDescriptor for all labels</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.constant = new PreferredPlacementDescriptor({
*   // Place labels along the edge
*   angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*   angle: 0,
*   // ... on either side
*   sideOfEdge: LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE,
*   // ... with a bit of distance to the edge
*   distanceToEdge: 5
* })
* </pre>
*
* If some labels should use custom placement or this has to be configured ahead of time, you can use the {@link ItemMapping#mapper} instead:
*
* <br><b>Customizing the placement of certain labels individually via a mapper</b>
* <pre>
* // Place label1 orthogonal to the edge anywhere on it
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label1,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.ANYWHERE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: Math.PI / 2
*   })
* )
* // Place label2 near the edge's source on either side of it, and make it parallel to the edge
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label2,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.AT_SOURCE,
*     sideOfEdge: LabelPlacements.RIGHT_OF_EDGE | LabelPlacements.LEFT_OF_EDGE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: 0
*   })
* )
* </pre>
*
* When the preferred placement can be inferred from the label itself, a delegate is usually the easiest choice:
*
* <br><b>Configuring preferred placement for all labels with a delegate</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.delegate = label => {
*   const customData = label.tag
*   const preferredPlacementDescriptor = new PreferredPlacementDescriptor({
*     angle: 0,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     // If the tag says to place the label in the center, put it in the center parallel to the edge's path
*     // All other labels can be placed anywhere, but on the side of the edge.
*     placeAlongEdge: customData.placeInCenter
*       ? LabelPlacements.AT_CENTER
*       : LabelPlacements.ANYWHERE,
*     sideOfEdge: customData.placeInCenter
*       ? LabelPlacements.ON_EDGE
*       : LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE
*   })
* }
* </pre>
*
* @see PreferredPlacementDescriptor
* @see LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LabelingData%23edgeLabelPreferredPlacement Online Documentation}
*/
edgeLabelPreferredPlacement: ItemMapping<ILabel, PreferredPlacementDescriptor>;
static $class: Class;
private $_LabelingData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LabelingData;
}
export interface LayerConstraintData extends LayoutData {
/**
@inheritdoc*/
apply(layoutGraphAdapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies layer constraints for {@link INode nodes} for a {@link HierarchicLayout}.
* Layer constraints affect the assignment of nodes to {@link ILayer layers}.
*
* If {@link LayerConstraintData#nodeComparables} are specified, they are used to sort all nodes accordingly and create {@link LayerConstraintData#placeBelow}
* constraints that work just like manually added constraints.
*
* If a fixed layer assignment is already available, the {@link HierarchicLayoutData#givenLayersLayererIds} should be used instead.
*
* One way to set layer constraints is by mapping nodes to an {@link IComparable} (e.g. a number) which define the
* order of layers in which those nodes are placed. This can be done with the {@link LayerConstraintData#nodeComparables} property. One way to map each node to such
* an {@link IComparable} is to use the {@link ItemMapping#delegate}:
*
* <br><b>Using a delegate to map nodes to objects that can be ordered to define relative layer placements</b>
* <pre>
* // Map each node to an IComparable, in this case taken from a custom object stored in the Tag
* layoutData.layerConstraints.nodeComparables = node => {
*   const data = node.tag
*   if (data !== null) {
*     return data.layerOrder
*   }
*   // Don't constrain layer placement if the tag doesn't contain our custom object.
*   return null
* }
* </pre>
*
* If only some nodes should be constrained, the {@link ItemMapping#mapper} can be used instead:
*
* <br><b>Using a mapper to map certain nodes to objects that can be ordered to define relative layer placements</b>
* <pre>
* // Define layer constraints for a few nodes via a mapper. Nodes that are not
* // explicitly mapped don't have constraints on their layer placement.
* const mapper = layoutData.layerConstraints.nodeComparables.mapper
* // Place both node1 and node2 in the same layer by assigning the same value
* mapper.set(node1, 5)
* mapper.set(node2, 5)
* // Place node3 somewhere above node1 and node2 by assigning a smaller value
* mapper.set(node3, 2)
* // Place node4 somewhere below node1 and node2 by assigning a larger value
* mapper.set(node4, 8)
* // The exact values don't matter much they just define an ordering. However, all
* // values must have the same type you cannot mix int and string or double, for example.
* </pre>
*
* If those mappings have been prepared beforehand, e.g. in a {@link HashMap} or {@link IMapper}, that property on the {@link ItemMapping} can also be set:
*
* <br><b>Setting a prepared mapper or dictionary for relative layer assignment</b>
* <pre>
* layoutData.layerConstraints.nodeComparables = layerOrderMapper
* // Or create a mapper from a map, instead:
* layoutData.layerConstraints.nodeComparables = new Mapper(layerOrderMap)
* </pre>
*
* {@link IComparable} is only the most natural option if such an object can be readily produced from a node, or there
* already exists such a mapping from nodes to something that can be compared. In many cases it can be easier to construct
* constraints manually with the respective methods on {@link LayerConstraintData}:
*
* <br><b>Defining custom layer constraints</b>
* <pre>
* // Place all nodes in sameLayerNodes into the same layer
* const firstSameLayerNode = sameLayerNodes.first()
* for (const node of sameLayerNodes) {
*   layoutData.layerConstraints.placeInSameLayer(firstSameLayerNode, node)
* }
* // Ensure that node2 is placed two layers below node1
* layoutData.layerConstraints.placeBelow(node1, node2, 2)
* // Also place another node in the top layer
* layoutData.layerConstraints.placeAtTop(nodeAtTop)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData Online Documentation}
*/
export class LayerConstraintData {
/**
Creates a new instance of {@link LayerConstraintData} which helps configuring
{@link ILayerConstraint}s for {@link HierarchicLayout}.
@param [options] The parameters to pass.
@param [options.nodeComparables] The mapping from nodes to an {@link IComparable} defining their layer sort order. This option sets the {@link LayerConstraintData#nodeComparables} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData Online Documentation}*/
constructor(options?: {
nodeComparables?: ItemMapping<INode, IComparable> | ItemMappingConvertible<INode, IComparable>;
});
/**
* Clears all previously defined constraints.
* All constraints that were previously defined via {@link LayerConstraintData#placeAbove},
* {@link LayerConstraintData#placeBelow}, {@link LayerConstraintData#placeInSameLayer}, {@link LayerConstraintData#placeAtTop}
* or {@link LayerConstraintData#placeAtBottom} were cleared so that new constraints can be declared.
*
* This doesn't affect indirect constraints specified by the {@link LayerConstraintData#nodeComparables}.
*
* <br><b>Clearing all previously set layer constraints</b>
* <pre>
* layoutData.layerConstraints.clearConstraints()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData%23clearConstraints() Online Documentation}
*/
clearConstraints(): void;
/**
* Adds a constraint that forces the <code>aboveNode</code> to lie at least <code>minDistance</code> layers above the <code>referenceNode</code> with a given <code>weight</code>
* penalty for larger layer differences.
* The minimum distance includes potentially empty layers that are removed by the {@link ILayerer} instance. In
* that case, the actual layer difference may be smaller than the minimum distance.
*
* <br><b>Placing a node above a reference node</b>
* <pre>
* layoutData.layerConstraints.placeAbove(referenceNode, node)
* </pre>
*
* @param referenceNode the reference node
* @param aboveNode the node that should lie above
* @param [minDistance=1] the minimum layer distance between the node and its reference node
* @param [weight=0] the weight penalty for larger layer differences
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData%23placeAbove(INode%2CINode%2Cnumber%2Cnumber) Online Documentation}
*/
placeAbove(referenceNode: INode, aboveNode: INode, minDistance?: number, weight?: number): void;
/**
* Adds a constraint that forces the <code>aboveNode</code> to lie at least <code>minDistance</code> layers above the <code>referenceNode</code> with a given <code>weight</code>
* penalty for larger layer differences.
* The minimum distance includes potentially empty layers that are removed by the {@link ILayerer} instance. In
* that case, the actual layer difference may be smaller than the minimum distance.
*
* <br><b>Placing a node above a reference node</b>
* <pre>
* layoutData.layerConstraints.placeAbove(referenceNode, node)
* </pre>
*
* @param options The parameters to pass.
* @param options.referenceNode the reference node
* @param options.aboveNode the node that should lie above
* @param [options.minDistance=1] the minimum layer distance between the node and its reference node
* @param [options.weight=0] the weight penalty for larger layer differences
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData%23placeAbove(INode%2CINode%2Cnumber%2Cnumber) Online Documentation}
*/
placeAbove(options: {
referenceNode: INode;
aboveNode: INode;
minDistance?: number;
weight?: number;
}): void;
/**
* Adds a constraint that places the <code>node</code> on the bottom layer.
* <br><b>Placing a node in the last layer</b>
* <pre>
* layoutData.layerConstraints.placeAtBottom(node)
* </pre>
*
* @param node the node that should lie on the bottom layer
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData%23placeAtBottom(INode) Online Documentation}
*/
placeAtBottom(node: INode): void;
/**
* Adds a constraint that places the <code>node</code> on the topmost layer.
* <br><b>Placing a node in the first layer</b>
* <pre>
* layoutData.layerConstraints.placeAtTop(node)
* </pre>
*
* @param node the node that should lie on the top layer
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData%23placeAtTop(INode) Online Documentation}
*/
placeAtTop(node: INode): void;
/**
* Adds a constraint that forces the <code>belowNode</code> to lie at least <code>minDistance</code> layers below the <code>referenceNode</code> with a given <code>weight</code>
* penalty for larger layer differences.
* The minimum distance includes potentially empty layers that are removed by the {@link ILayerer} instance. In
* that case, the actual layer difference may be smaller than the minimum distance.
*
* <br><b>Placing a node below a reference node</b>
* <pre>
* layoutData.layerConstraints.placeBelow(referenceNode, node)
* </pre>
*
* @param referenceNode the reference node
* @param belowNode the node that should lie below
* @param [minDistance=1] the minimum layer distance between the node and its reference node
* @param [weight=0] the weight penalty for larger layer differences
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData%23placeBelow(INode%2CINode%2Cnumber%2Cnumber) Online Documentation}
*/
placeBelow(referenceNode: INode, belowNode: INode, minDistance?: number, weight?: number): void;
/**
* Adds a constraint that forces the <code>belowNode</code> to lie at least <code>minDistance</code> layers below the <code>referenceNode</code> with a given <code>weight</code>
* penalty for larger layer differences.
* The minimum distance includes potentially empty layers that are removed by the {@link ILayerer} instance. In
* that case, the actual layer difference may be smaller than the minimum distance.
*
* <br><b>Placing a node below a reference node</b>
* <pre>
* layoutData.layerConstraints.placeBelow(referenceNode, node)
* </pre>
*
* @param options The parameters to pass.
* @param options.referenceNode the reference node
* @param options.belowNode the node that should lie below
* @param [options.minDistance=1] the minimum layer distance between the node and its reference node
* @param [options.weight=0] the weight penalty for larger layer differences
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData%23placeBelow(INode%2CINode%2Cnumber%2Cnumber) Online Documentation}
*/
placeBelow(options: {
referenceNode: INode;
belowNode: INode;
minDistance?: number;
weight?: number;
}): void;
/**
* Adds a constraint that forces the the <code>sameLayerNode</code> to lie in the same layer as the <code>referenceNode</code>.
* <br><b>Placing a node in the same layer as a reference node</b>
* <pre>
* layoutData.layerConstraints.placeInSameLayer(referenceNode, node)
* </pre>
*
* @param referenceNode the reference node
* @param sameLayerNode the node that should lie in the same layer
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData%23placeInSameLayer(INode%2CINode) Online Documentation}
*/
placeInSameLayer(referenceNode: INode, sameLayerNode: INode): void;
/**
* Gets or set the mapping from nodes to an {@link IComparable} defining their layer sort order.
* Nodes with a lower sort order are constraint to be placed above nodes with a higher sort order. For nodes mapped to
* <code>null</code>, no constraints are added.
*
* <br><b>Using the node's labels to indicate their order</b>
* <pre>
* layoutData.layerConstraints.nodeComparables = node =>
*   node.labels.size > 0 ? node.labels.get(0).text : null
* </pre>
*
* @see GivenLayersLayerer#LAYER_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/LayerConstraintData%23nodeComparables Online Documentation}
*/
nodeComparables: ItemMapping<INode, IComparable>;
static $class: Class;
private $_LayerConstraintData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayerConstraintData;
}
export interface LayoutData extends YObject {
}
/**
Base for classes that specify custom data for a layout calculation done with the {@link LayoutExecutor}.
Implementations of this class provide item-specific configuration options for a specific layout. Single properties that
affect the complete layout are available on the specific layout class itself.
The {@link LayoutExecutor} {@link LayoutData#apply applies} these configurations when {@link LayoutExecutor#start started}.
@see LayoutExecutor#layoutData
@see GraphComponent#morphLayout
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutData Online Documentation}*/
export abstract class LayoutData {
/**
Applies this layout data to the specified <code>layoutGraphAdapter</code>.
This method should be called before a layout is executed. It will configure the adapter and graph for to the layout.
@param layoutGraphAdapter The adapter to add the layout data to.
@param layout The layout that shall use this layout data.
@param layoutGraph The layout graph used for the layout calculation.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutData%23apply(LayoutGraphAdapter%2CILayoutAlgorithm%2CCopiedLayoutGraph) Online Documentation}*/
abstract apply(layoutGraphAdapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
/**
Combines this instance with the given layout data.
@param data The {@link LayoutData} to combine this instance with.
@returns The combined layout data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutData%23combineWith(LayoutData) Online Documentation}*/
combineWith(data: LayoutData): LayoutData;
static $class: Class;
private $_LayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutData;
}
export interface LayoutExecutor extends YObject {
}
/**
Executes a {@link ILayoutAlgorithm} and optionally animates the transition to the calculated layout.
This class is the preferred way to execute a layout. Its animation can be customized in various ways.
If no fine-grained control of the animation is required, the methods {@link IGraph#applyLayout} and {@link GraphComponent#morphLayout}
can be used instead.
@see LayoutGraphAdapter
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor Online Documentation}*/
export class LayoutExecutor {
/**
Initializes a new instance of the {@link LayoutExecutor} class.
@param graphComponent The control which will be animated and provides the {@link IGraph} instance.
@param graph The graph to layout.
@param layout The {@link ILayoutAlgorithm} to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23LayoutExecutor(GraphComponent%2CIGraph%2CILayoutAlgorithm) Online Documentation}*/
constructor(graphComponent: GraphComponent, graph: IGraph, layout: ILayoutAlgorithm);
/**
Initializes a new instance of the {@link LayoutExecutor} class.
@param options The parameters to pass.
@param options.graphComponent The control which will be animated and provides the {@link IGraph} instance.
@param options.graph The graph to layout.
@param options.layout The {@link ILayoutAlgorithm} to use.
@param [options.configureTableLayout] A value indicating whether to automatically perform calls to {@link TableLayoutConfigurator#prepare} and {@link TableLayoutConfigurator#restore} in order to layout table nodes. This option sets the {@link LayoutExecutor#configureTableLayout} property on the created object.
@param [options.duration] The duration of the animation. This option sets the {@link LayoutExecutor#duration} property on the created object.
@param [options.automaticEdgeGrouping] A value indicating whether edge groups are automatically created for edges that are connected to the same port. This option sets the {@link LayoutExecutor#automaticEdgeGrouping} property on the created object.
@param [options.fixPorts] A value that controls whether strong port constraints are automatically created. This option sets the {@link LayoutExecutor#fixPorts} property on the created object.
@param [options.animateViewport] A value indicating whether the viewport should be animated to the new bounds of the graph. This option sets the {@link LayoutExecutor#animateViewport} property on the created object.
@param [options.considerViewportLimiter] A value indicating whether to respect the {@link CanvasComponent#viewportLimiter} of the {@link GraphComponent} of this instance. This option sets the {@link LayoutExecutor#considerViewportLimiter} property on the created object.
@param [options.easedAnimation] A value indicating whether to use eased animation. This option sets the {@link LayoutExecutor#easedAnimation} property on the created object.
@param [options.targetBoundsInsets] The insets (in world coordinates) that will be added to the content rectangle when calculating the target viewport. This option sets the {@link LayoutExecutor#targetBoundsInsets} property on the created object.
@param [options.updateContentRect] A value indicating whether the {@link CanvasComponent#contentRect} property of the {@link LayoutExecutor#graphComponent} should be updated upon completion. This option sets the {@link LayoutExecutor#updateContentRect} property on the created object.
@param [options.allowUserInteraction] A value indicating whether user interaction should be allowed during the animation. This option sets the {@link LayoutExecutor#allowUserInteraction} property on the created object.
@param [options.layoutData] The layout data that is applied when {@link LayoutExecutor#start starting} the executor. This option sets the {@link LayoutExecutor#layoutData} property on the created object.
@param [options.portLabelPolicies] How {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm. This option sets the {@link LayoutExecutor#portLabelPolicies} property on the created object.
@param [options.hideEdgesAtEdges] A value that controls whether edges at other edges will be hidden from the layout graph or included. This option sets the {@link LayoutExecutor#hideEdgesAtEdges} property on the created object.
@param [options.portAdjustmentPolicy] The policy that specifies how port locations should be adjusted after a layout has been calculated. This option sets the {@link LayoutExecutor#portAdjustmentPolicy} property on the created object.
@param [options.selectionModel] The {@link ISelectionModel} to use for the automatically registered {@link IDataProvider} instances for {@link LayoutKeys#AFFECTED_NODES_DP_KEY} and {@link LayoutKeys#AFFECTED_EDGES_DP_KEY}. This option sets the {@link LayoutExecutor#selectionModel} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23LayoutExecutor(GraphComponent%2CIGraph%2CILayoutAlgorithm) Online Documentation}*/
constructor(options: {
graphComponent: GraphComponent;
graph: IGraph;
layout: ILayoutAlgorithm;
configureTableLayout?: boolean;
duration?: TimeSpan | TimeSpanConvertible;
automaticEdgeGrouping?: boolean;
fixPorts?: boolean;
animateViewport?: boolean;
considerViewportLimiter?: boolean;
easedAnimation?: boolean;
targetBoundsInsets?: Insets | InsetsConvertible;
updateContentRect?: boolean;
allowUserInteraction?: boolean;
layoutData?: LayoutData | null;
portLabelPolicies?: ItemMapping<ILabel, PortLabelPolicy> | ItemMappingConvertible<ILabel, PortLabelPolicy>;
hideEdgesAtEdges?: boolean;
portAdjustmentPolicy?: PortAdjustmentPolicy | PortAdjustmentPolicyStringValues;
selectionModel?: ISelectionModel<IModelItem> | null;
});
/**
Initializes a new instance of the {@link LayoutExecutor} class.
Use the {@link LayoutExecutor#start} method to actually execute the task. This will disable animation and content
rect updates.
@param graphComponent The control which will be animated and provides the {@link IGraph} instance.
@param layout The {@link ILayoutAlgorithm} to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23LayoutExecutor(GraphComponent%2CILayoutAlgorithm) Online Documentation}*/
constructor(graphComponent: GraphComponent, layout: ILayoutAlgorithm);
/**
Initializes a new instance of the {@link LayoutExecutor} class.
Use the {@link LayoutExecutor#start} method to actually execute the task. This will disable animation and content
rect updates.
@param options The parameters to pass.
@param options.graphComponent The control which will be animated and provides the {@link IGraph} instance.
@param options.layout The {@link ILayoutAlgorithm} to use.
@param [options.configureTableLayout] A value indicating whether to automatically perform calls to {@link TableLayoutConfigurator#prepare} and {@link TableLayoutConfigurator#restore} in order to layout table nodes. This option sets the {@link LayoutExecutor#configureTableLayout} property on the created object.
@param [options.duration] The duration of the animation. This option sets the {@link LayoutExecutor#duration} property on the created object.
@param [options.automaticEdgeGrouping] A value indicating whether edge groups are automatically created for edges that are connected to the same port. This option sets the {@link LayoutExecutor#automaticEdgeGrouping} property on the created object.
@param [options.fixPorts] A value that controls whether strong port constraints are automatically created. This option sets the {@link LayoutExecutor#fixPorts} property on the created object.
@param [options.animateViewport] A value indicating whether the viewport should be animated to the new bounds of the graph. This option sets the {@link LayoutExecutor#animateViewport} property on the created object.
@param [options.considerViewportLimiter] A value indicating whether to respect the {@link CanvasComponent#viewportLimiter} of the {@link GraphComponent} of this instance. This option sets the {@link LayoutExecutor#considerViewportLimiter} property on the created object.
@param [options.easedAnimation] A value indicating whether to use eased animation. This option sets the {@link LayoutExecutor#easedAnimation} property on the created object.
@param [options.targetBoundsInsets] The insets (in world coordinates) that will be added to the content rectangle when calculating the target viewport. This option sets the {@link LayoutExecutor#targetBoundsInsets} property on the created object.
@param [options.updateContentRect] A value indicating whether the {@link CanvasComponent#contentRect} property of the {@link LayoutExecutor#graphComponent} should be updated upon completion. This option sets the {@link LayoutExecutor#updateContentRect} property on the created object.
@param [options.allowUserInteraction] A value indicating whether user interaction should be allowed during the animation. This option sets the {@link LayoutExecutor#allowUserInteraction} property on the created object.
@param [options.layoutData] The layout data that is applied when {@link LayoutExecutor#start starting} the executor. This option sets the {@link LayoutExecutor#layoutData} property on the created object.
@param [options.portLabelPolicies] How {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm. This option sets the {@link LayoutExecutor#portLabelPolicies} property on the created object.
@param [options.hideEdgesAtEdges] A value that controls whether edges at other edges will be hidden from the layout graph or included. This option sets the {@link LayoutExecutor#hideEdgesAtEdges} property on the created object.
@param [options.portAdjustmentPolicy] The policy that specifies how port locations should be adjusted after a layout has been calculated. This option sets the {@link LayoutExecutor#portAdjustmentPolicy} property on the created object.
@param [options.selectionModel] The {@link ISelectionModel} to use for the automatically registered {@link IDataProvider} instances for {@link LayoutKeys#AFFECTED_NODES_DP_KEY} and {@link LayoutKeys#AFFECTED_EDGES_DP_KEY}. This option sets the {@link LayoutExecutor#selectionModel} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23LayoutExecutor(GraphComponent%2CILayoutAlgorithm) Online Documentation}*/
constructor(options: {
graphComponent: GraphComponent;
layout: ILayoutAlgorithm;
configureTableLayout?: boolean;
duration?: TimeSpan | TimeSpanConvertible;
automaticEdgeGrouping?: boolean;
fixPorts?: boolean;
animateViewport?: boolean;
considerViewportLimiter?: boolean;
easedAnimation?: boolean;
targetBoundsInsets?: Insets | InsetsConvertible;
updateContentRect?: boolean;
allowUserInteraction?: boolean;
layoutData?: LayoutData | null;
portLabelPolicies?: ItemMapping<ILabel, PortLabelPolicy> | ItemMappingConvertible<ILabel, PortLabelPolicy>;
hideEdgesAtEdges?: boolean;
portAdjustmentPolicy?: PortAdjustmentPolicy | PortAdjustmentPolicyStringValues;
selectionModel?: ISelectionModel<IModelItem> | null;
});
/**
Creates an instance of {@link AbortHandler}.
This method is called upon first access to the {@link LayoutExecutor#abortHandler} property.
@returns A new {@link AbortHandler} instance to use during layout calculation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23createAbortHandler() Online Documentation}*/
createAbortHandler(): AbortHandler;
/**
Factory method that creates the {@link IAnimation} that will be used by this instance after the layout has been
calculated.
@returns The animation to use after the layout.
@see LayoutExecutor#duration
@see LayoutExecutor#animateViewport
@see LayoutExecutor#createMorphAnimation
@see LayoutExecutor#createViewportAnimation
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23createAnimation() Online Documentation}*/
createAnimation(): IAnimation;
/**
Creates the {@link LayoutGraphAdapter} which is used when a layout is {@link LayoutExecutor#start executed}.
@returns A preconfigured {@link LayoutGraphAdapter}.
@see LayoutGraphAdapter
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23createLayoutGraphAdapter() Online Documentation}*/
createLayoutGraphAdapter(): LayoutGraphAdapter;
/**
Factory method that creates the animation for the {@link IGraph}.
@returns The animation instance.
@see LayoutExecutor#createAnimation
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23createMorphAnimation() Online Documentation}*/
createMorphAnimation(): IAnimation;
/**
Creates an animation that morphs the layout of all {@link ITable}s in the graph.
@see TableAnimation
@see LayoutExecutor#configureTableLayout
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23createTableAnimations() Online Documentation}*/
createTableAnimations(): IAnimation;
/**
Create a new instance of {@link TableLayoutConfigurator} that is used if {@link LayoutExecutor#configureTableLayout} is enabled.
This method is called upon first access to the {@link LayoutExecutor#tableLayoutConfigurator} property.
@returns A new instance of the {@link TableLayoutConfigurator} class.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23createTableLayoutConfigurator() Online Documentation}*/
createTableLayoutConfigurator(): TableLayoutConfigurator;
/**
Factory method that creates the animation for the {@link CanvasComponent#viewport}.
The created animation will morph the current viewport into the one where the whole graph fits. The result after the
animation is thus the same as calling {@link GraphComponent#fitGraphBounds}.
@param targetBounds The target bounds of the animation.
@returns The animation instance.
@see LayoutExecutor#createAnimation
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23createViewportAnimation(Rect) Online Documentation}*/
createViewportAnimation(targetBounds: Rect): IAnimation;
/**
Callback method that performs the actual layout.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23execute() Online Documentation}*/
execute(): void;
/**
Calculate the target bounds to be used for the {@link CanvasComponent#contentRect} as well as the {@link ViewportAnimation} after the layout has
finished.
By default, the value is computed with {@link LayoutGraphUtilities#getBoundingBox} with labels and {@link LayoutExecutor#targetBoundsInsets} taken into
account.
@returns The desired content rectangle and the bounds for a viewport animation.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23getTargetBounds() Online Documentation}*/
getTargetBounds(): Rect;
/**
Set up {@link LayoutExecutor#tableLayoutConfigurator} for a layout.
This implementation configures {@link TableLayoutConfigurator#horizontalLayout} according to the {@link MultiStageLayout#layoutOrientation} and calls {@link TableLayoutConfigurator#prepare}
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23prepareTableLayout() Online Documentation}*/
prepareTableLayout(): void;
/**
Writes the table layout information provided through {@link LayoutExecutor#tableLayoutConfigurator} back to all tables.
This method is only called when the layout is not animated.
@see LayoutExecutor#prepareTableLayout
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23restoreTableLayout() Online Documentation}*/
restoreTableLayout(): void;
/**
Actually starts the layout calculation and the optional animation asynchronously using a Promise.
This method will ultimately call the {@link LayoutExecutor#execute} method. If the {@link LayoutExecutor#duration} is zero no animation will be
performed.
If this instance is already running, this method returns immediately without doing anything and returns the previous
Promise.
The layout algorithm itself is executed using a {@link LayoutGraphAdapter} instance which is created and
configured in method {@link LayoutExecutor#createLayoutGraphAdapter}.
@returns A Promise that will be fulfilled once the layout and optional animation is done.
@see LayoutExecutor#execute
@see LayoutExecutor#createLayoutGraphAdapter
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23start() Online Documentation}*/
start(): Promise<any>;
/**
Stops a currently running layout calculation or animation.
If a layout calculation is still running, it will be requested to stopped via {@link AbortHandler#stop} and
the animation will not run. If the layout calculation was already completed, the animation will be aborted immediately
and the layout result will be shown immediately.
To just skip the animation but let the calculation finish normally, the {@link LayoutExecutor#duration} can be set to zero at any time before the
animation was started.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23stop() Online Documentation}*/
stop(): Promise<any>;
/**
Gets the {@link AbortHandler} that is used during the layout calculation, unless another {@link AbortHandler}
has been configured via {@link LayoutExecutor#layoutData} or by registering an {@link IMapper} in the graph's {@link IGraph#mapperRegistry}.
@see LayoutExecutor#createAbortHandler
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23abortHandler Online Documentation}*/
readonly abortHandler: AbortHandler;
/**
Gets the {@link LayoutExecutor#tableLayoutConfigurator} that is used if {@link LayoutExecutor#configureTableLayout} is enabled.
@see LayoutExecutor#createTableLayoutConfigurator
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23tableLayoutConfigurator Online Documentation}*/
readonly tableLayoutConfigurator: TableLayoutConfigurator;
/**
Gets or sets a value indicating whether to automatically perform calls to {@link TableLayoutConfigurator#prepare}
and {@link TableLayoutConfigurator#restore} in order to layout table nodes.
The default value is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23configureTableLayout Online Documentation}*/
configureTableLayout: boolean;
/**
Gets or sets the duration of the animation.
The default value is {@link TimeSpan#ZERO}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23duration Online Documentation}*/
duration: TimeSpan;
/**
Gets or sets a value indicating whether edge groups are automatically created for edges that are connected to the same
port.
If this property is enabled and no edge groups are explicitly created (no {@link IMapper} are registered for the keys {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
and {@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), for each port with multiple outgoing resp. incoming edges
these edges will be assigned to the same edge group. The necessary
{@link IDataProvider}s will be created automatically.
Without an {@link ILayoutAlgorithm} that supports edge groups, this property has no visual effect.
The default value is <code>true</code>.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see LayoutGraphAdapter#automaticEdgeGrouping
@see LayoutExecutor#fixPorts
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23automaticEdgeGrouping Online Documentation}*/
automaticEdgeGrouping: boolean;
/**
Gets or sets a value that controls whether strong port constraints are automatically created.
If this property is enabled and no port constraints have been added explicitly (no {@link IMapper} are registered for the keys {@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
and {@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), strong source and target port constraints are created
automatically. The necessary
{@link IDataProvider}s will be created automatically. The edge direction is inferred from the location of the
port relative to the node.
This ensures that view {@link IPort}s are not moved during the layout, if the {@link ILayoutAlgorithm}
supports port constraints.
The default value is <code>false</code>.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see LayoutGraphAdapter#fixPorts
@see LayoutExecutor#automaticEdgeGrouping
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23fixPorts Online Documentation}*/
fixPorts: boolean;
/**
Gets or sets a value indicating whether the viewport should be animated to the new bounds of the graph.
The result when this property is <code>true</code> after the animation is the same as calling
{@link GraphComponent#fitGraphBounds}. Setting this property to <code>true</code> and changing {@link LayoutExecutor#duration} to {@link TimeSpan#ZERO} will
disable the animation, but still change the viewport to the new graph bounds.
When the viewport should stay the same, the layout algorithms often have to be coerced to keep parts of the graph in the
same location. This can be done by wrapping the layout algorithm in an instance of {@link FixNodeLayoutStage}.
The default value is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23animateViewport Online Documentation}*/
animateViewport: boolean;
/**
Gets or sets a value indicating whether to respect the {@link CanvasComponent#viewportLimiter} of the {@link GraphComponent} of this instance.
The default value is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23considerViewportLimiter Online Documentation}*/
considerViewportLimiter: boolean;
/**
Gets or sets a value indicating whether to use eased animation.
The default value is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23easedAnimation Online Documentation}*/
easedAnimation: boolean;
/**
Gets or sets the insets (in world coordinates) that will be added to the content rectangle when calculating the target
viewport.
The default value is {@link Insets#EMPTY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23targetBoundsInsets Online Documentation}*/
targetBoundsInsets: Insets;
/**
Gets or sets a value indicating whether the {@link CanvasComponent#contentRect} property of the {@link LayoutExecutor#graphComponent} should be updated upon completion.
The default value is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23updateContentRect Online Documentation}*/
updateContentRect: boolean;
/**
Gets or sets a value indicating whether user interaction should be allowed during the animation.
If <code>false</code>, the {@link WaitInputMode} is queried from the {@link CanvasComponent} and {@link WaitInputMode#waiting} is enabled during the
animation.
The default value is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23allowUserInteraction Online Documentation}*/
allowUserInteraction: boolean;
/**
Gets a value indicating whether this instance is currently running.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23running Online Documentation}*/
readonly running: boolean;
/**
Gets or sets the layout data that is applied when {@link LayoutExecutor#start starting} the executor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23layoutData Online Documentation}*/
layoutData: LayoutData | null;
/**
Gets or sets how {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm.
This mapping is queried for each {@link ILabel}s at an {@link IPort} and should return a {@link PortLabelPolicy}
value indicating how the label should appear to the layout algorithms. The default is a constant {@link ItemMapping} returning {@link PortLabelPolicy#NODE_LABEL}
for all labels.
@see PortLabelPolicy
@see LayoutGraphAdapter#portLabelPolicies
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23portLabelPolicies Online Documentation}*/
portLabelPolicies: ItemMapping<ILabel, PortLabelPolicy>;
/**
Gets or sets a value that controls whether edges at other edges will be hidden from the layout graph or included.
If this property is set to <code>true</code>, edges at other edges are simply ignored and not part of the layout graph. The default
is <code>false</code> in which case dummy nodes are inserted into the layout graph for each source and target port of an {@link IEdge}
that is {@link IPort#owner owned} by an {@link IEdge}.
@see LayoutGraphAdapter#hideEdgesAtEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23hideEdgesAtEdges Online Documentation}*/
hideEdgesAtEdges: boolean;
/**
Gets or sets the policy that specifies how port locations should be adjusted after a layout has been calculated.
This can be useful if the port assignment calculated by the layout algorithm is insufficient.
Layout algorithms only consider rectangular nodes even though the actual shape of a node is, for example, circular.
Hence, the ports are usually placed at the border of the nodes' {@link INode#layout bounding boxes} (except for some layout algorithms that
produce straight-line edge routes and place the ports at the nodes' center).
Based on this setting the edges will be shortened or lengthened in a way that their {@link IEdge#sourcePort}s and {@link IEdge#targetPort}s will be placed on the node's
{@link IShapeGeometry#getOutline outline}.
The default value is {@link PortAdjustmentPolicy#NEVER}.
<h2>Note</h2>
The coordinates of a port will not be changed if the port is associated with a strong port constraint, e.g. by setting {@link LayoutExecutor#fixPorts}
to <code>true</code>.
@see PortCalculator
@see LayoutGraphAdapter#portAdjustmentPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23portAdjustmentPolicy Online Documentation}*/
portAdjustmentPolicy: PortAdjustmentPolicy;
/**
Gets or sets the {@link ISelectionModel} to use for the automatically registered {@link IDataProvider} instances for {@link LayoutKeys#AFFECTED_NODES_DP_KEY}
and {@link LayoutKeys#AFFECTED_EDGES_DP_KEY}.
If this value is <code>null</code> and {@link LayoutExecutor#graph} is the same instance as {@link LayoutExecutor#graphComponent}'s {@link IGraph} instance, the selection model from {@link LayoutExecutor#graphComponent} is used
instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23selectionModel Online Documentation}*/
selectionModel: ISelectionModel<IModelItem> | null;
/**
Gets the layout graph that is used by this instance to calculate the layout.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23layoutGraph Online Documentation}*/
readonly layoutGraph: CopiedLayoutGraph;
/**
Gets the control this instance has been created for.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23graphComponent Online Documentation}*/
readonly graphComponent: GraphComponent;
/**
Gets the {@link ILayoutAlgorithm} this instance is using.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23layout Online Documentation}*/
readonly layout: ILayoutAlgorithm;
/**
Gets the graph this instance is working on.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutExecutor%23graph Online Documentation}*/
readonly graph: IGraph;
static $class: Class;
private $_LayoutExecutor: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutExecutor;
}
/**
Specifies how {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm.
@see LayoutGraphAdapter#portLabelPolicies
@see LayoutExecutor#portLabelPolicies
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLabelPolicy Online Documentation}*/
export enum PortLabelPolicy {
/**
The port label is not visible for the layout algorithm.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLabelPolicy%23NONE Online Documentation}*/
NONE,
/**
The port label is converted to a node label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLabelPolicy%23NODE_LABEL Online Documentation}*/
NODE_LABEL,
/**
The port label is converted to an edge label.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLabelPolicy%23EDGE_LABEL Online Documentation}*/
EDGE_LABEL,
/**
The port label is converted to an edge label if the port has exactly one incident edge, and to a node label otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortLabelPolicy%23AUTO Online Documentation}*/
AUTO
}
/** Automatic conversion to {@link PortLabelPolicy}. */
type PortLabelPolicyStringValues = "none" | "node-label" | "edge-label" | "auto";
export interface LayoutGraphAdapter extends YObject {
}
/**
Adapter class that can be used to obtain an {@link LayoutGraph} from an {@link IGraph}.
Typically, this class is not used directly but through the {@link LayoutExecutor} or ApplyLayout and
{@link GraphComponent#morphLayout}
This class will use the provided {@link LayoutGraphAdapter#selectionModel} to register corresponding {@link IDataProvider}s for the data provider keys
{@link LayoutKeys#AFFECTED_NODES_DP_KEY} and {@link LayoutKeys#AFFECTED_EDGES_DP_KEY} to the layout graph.
Moreover, it will automatically convert {@link IMapper} instances registered with the {@link IMapperRegistry} of the {@link LayoutGraphAdapter#adaptedGraph} into
corresponding {@link IDataProvider} instances of the layout graph.
In addition, this class supports {@link IGraph}s with edge to edge connections, i.e., edges that are adjacent to
other edges. Since such edges are not allowed in an {@link LayoutGraph}, the property {@link LayoutGraphAdapter#hideEdgesAtEdges} specifies whether these
edges shouldn't be converted into the layout graph at all or converted into a corresponding structure with dummy nodes.
The chapter {@link https://docs.yworks.com/yfileshtml/#/dguide/layout-applying_a_layout Applying an Automatic Layout} describes how automatic layout and graph analysis algorithms from the yFiles
Layout and yFiles Basic library components can be accessed from yFiles for HTML.
@see CopiedLayoutGraph
@see ILayoutAlgorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter Online Documentation}*/
export class LayoutGraphAdapter {
/**
Creates the adapter using a given {@link IGraph} and {@link ISelectionModel}.
@param [adaptedGraph=null] The graph to build an adapter for.
@param [selectionModel=null] The optional selection model to use by the {@link LayoutKeys#AFFECTED_NODES_DP_KEY} and {@link LayoutKeys#AFFECTED_EDGES_DP_KEY}
{@link IDataProvider} instances that are automatically bound to this adapter.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter Online Documentation}*/
constructor(adaptedGraph?: IGraph | null, selectionModel?: ISelectionModel<IModelItem> | null);
/**
Creates the adapter using a given {@link IGraph} and {@link ISelectionModel}.
@param options The parameters to pass.
@param [options.adaptedGraph=null] The graph to build an adapter for.
@param [options.selectionModel=null] The optional selection model to use by the {@link LayoutKeys#AFFECTED_NODES_DP_KEY} and {@link LayoutKeys#AFFECTED_EDGES_DP_KEY}
{@link IDataProvider} instances that are automatically bound to this adapter.
@param [options.portLabelPolicies] How {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm. This option sets the {@link LayoutGraphAdapter#portLabelPolicies} property on the created object.
@param [options.edgePortNodeSize] The size of the nodes that are inserted for the ports that are created for {@link IEdge}s that are connected at other {@link IEdge}s. This option sets the {@link LayoutGraphAdapter#edgePortNodeSize} property on the created object.
@param [options.hideEdgesAtEdges] A value that controls whether edges at other edges will be hidden from the layout graph or included. This option sets the {@link LayoutGraphAdapter#hideEdgesAtEdges} property on the created object.
@param [options.automaticEdgeGrouping] A value that controls whether edge groups are automatically created for edges that are connected to the same port. This option sets the {@link LayoutGraphAdapter#automaticEdgeGrouping} property on the created object.
@param [options.fixPorts] A value that controls whether strong port constraints are automatically created. This option sets the {@link LayoutGraphAdapter#fixPorts} property on the created object.
@param [options.portAdjustmentPolicy] The policy that specifies how port locations should be adjusted after a layout has been calculated. This option sets the {@link LayoutGraphAdapter#portAdjustmentPolicy} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter Online Documentation}*/
constructor(options: {
adaptedGraph?: IGraph | null;
selectionModel?: ISelectionModel<IModelItem> | null;
portLabelPolicies?: ItemMapping<ILabel, PortLabelPolicy> | ItemMappingConvertible<ILabel, PortLabelPolicy>;
edgePortNodeSize?: Size | SizeConvertible;
hideEdgesAtEdges?: boolean;
automaticEdgeGrouping?: boolean;
fixPorts?: boolean;
portAdjustmentPolicy?: PortAdjustmentPolicy | PortAdjustmentPolicyStringValues;
});
/**
Registers a data provider with the graph using the given look-up key and wrapping the given mapper.
If a mapper is already registered under the given key, it is overwritten.
Type parameter <code>K</code>: The type of the key to use.
Type parameter <code>V</code>: The type of the value to use.
@param valueType The type of the value to use.
@param keyType The type of the key to use.
@param dataKey The look-up key to register the data provider with.
@param mapper The mapper wrapped as data provider.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23addDataProvider(Class%2CClass%2CObject%2CIMapper%3CK%2CV%3E) Online Documentation}*/
addDataProvider<K, V>(keyType: Class, valueType: Class, dataKey: any, mapper: IMapper<K, V> | IMapperConvertible<K, V>): IDataProvider;
/**
Applies a {@link ILayoutAlgorithm layout algorithm} to an {@link IGraph}, optionally applying some {@link LayoutData}.
@param layout The layout algorithm to apply.
@param [layoutData=null] The layout data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23applyLayout(ILayoutAlgorithm%2CLayoutData) Online Documentation}*/
applyLayout(layout: ILayoutAlgorithm, layoutData?: LayoutData | null): void;
/**
Creates a {@link CopiedLayoutGraph} for this instance.
@returns An instance whose {@link CopiedLayoutGraph#commitLayoutToOriginalGraph} will update this instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23createCopiedLayoutGraph() Online Documentation}*/
createCopiedLayoutGraph(): CopiedLayoutGraph;
/**
Creates the {@link EdgeLabelCandidate} for a given label and {@link ILabelModelParameter} combination.
@param originalLabel The original label.
@param dummyLabel The dummy label that describes the current laid out label.
@param layoutParameter The parameter to create a candidate for.
@param labelLayout The {@link IEdgeLabelLayout} of the label to pass to the candidate.
@returns A candidate to yield for the labeling algorithms or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23createEdgeLabelCandidate(ILabel%2CILabel%2CILabelModelParameter%2CIEdgeLabelLayout) Online Documentation}*/
createEdgeLabelCandidate(originalLabel: ILabel, dummyLabel: ILabel, layoutParameter: ILabelModelParameter, labelLayout: IEdgeLabelLayout): EdgeLabelCandidate | null;
/**
Factory method that creates an {@link IEdgeLayout} implementation that wraps the given {@link IEdge}
@param edge The edge to wrap.
@returns A wrapper instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23createEdgeLayout(IEdge) Online Documentation}*/
createEdgeLayout(edge: IEdge): IEdgeLayout;
/**
Callback factory method that creates the {@link INodeLayout} for the given {@link IPort} that is used as a
dummy to represent the port at the {@link IEdge} that owns <code>port</code>.
@param port The port to create the layout for.
@returns A suitable implementation of {@link INodeLayout} that uses the {@link IPort#location port's location} as the center of the node.
@see LayoutGraphAdapter#edgePortNodeSize
@see LayoutGraphAdapter#hideEdgesAtEdges
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23createEdgePortLayout(IPort) Online Documentation}*/
createEdgePortLayout(port: IPort): INodeLayout;
/**
Creates a {@link NodeLabelCandidate} for a given label and {@link ILabelModelParameter} combination.
@param originalLabel The original label.
@param dummyLabel The dummy label that describes the current laid out label.
@param layoutParameter The parameter to create a candidate for.
@param ownerLayout The {@link INodeLabelLayout} of the label to pass to the candidate.
@returns A candidate to yield for the labeling algorithms or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23createNodeLabelCandidate(ILabel%2CILabel%2CILabelModelParameter%2CINodeLabelLayout) Online Documentation}*/
createNodeLabelCandidate(originalLabel: ILabel, dummyLabel: ILabel, layoutParameter: ILabelModelParameter, ownerLayout: INodeLabelLayout): NodeLabelCandidate | null;
/**
Factory method that creates a {@link INodeLayout} implementation that wraps the given {@link INode}
@param node The node to wrap.
@returns A wrapper instance.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23createNodeLayout(INode) Online Documentation}*/
createNodeLayout(node: INode): INodeLayout;
/**
Returns an {@link IEnumerable} that provides access to all edges residing in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23edgeObjects() Online Documentation}*/
edgeObjects(): IEnumerable<any>;
/**
Returns the data provider that is registered with the graph using the given look-up key.
The look-up domain of a returned data provider normally consists of either the nodes of the graph, or its edges, or
both.
@param dataKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23getDataProvider(Object) Online Documentation}*/
getDataProvider(dataKey: any): IDataProvider | null;
/**
Returns the layout information for all edge labels belonging to the given edge.
This information contains the location and size of the labels.
@param edge the edge that owns the labels
@returns the array of all labels that belong to the given edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23getEdgeLabelLayout(Object) Online Documentation}*/
getEdgeLabelLayout(edge: any): IEdgeLabelLayout[];
/**
Returns the layout information associated with the given edge.
This information contains the coordinates of the source and target points as well as the coordinates of all control
points.
@param edge the edge
@returns the layout information for the edge
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23getEdgeLayout(Object) Online Documentation}*/
getEdgeLayout(edge: any): IEdgeLayout;
/**
Callback that retrieves the label candidate descriptor for the given label and parameter.
@param originalLabel The original label to which a parameter will be applied.
@param dummyLabel The dummy label to which the parameter will be applied.
@param layoutParameter The parameter to apply.
@returns A descriptor or <code>null</code>.
@see LayoutGraphAdapter#createNodeLabelCandidate
@see LayoutGraphAdapter#createEdgeLabelCandidate
@see LayoutGraphAdapter#getLabelCandidateDescriptorProvider
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23getLabelCandidateDescriptor(ILabel%2CILabel%2CILabelModelParameter) Online Documentation}*/
getLabelCandidateDescriptor(originalLabel: ILabel, dummyLabel: ILabel, layoutParameter: ILabelModelParameter): ILabelCandidateDescriptor | null;
/**
Callback method that gets the label candidate descriptor provider for the given {@link IGraph}.
This implementation uses the {@link IGraph}'s {@link ILookup} to find an implementation of the
{@link ILabelCandidateDescriptorProvider}.
@param graph The graph.
@returns The result of a {@link ILookup#lookup} query on <code>graph</code> for the {@link ILabelCandidateDescriptorProvider} type.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23getLabelCandidateDescriptorProvider(IGraph) Online Documentation}*/
getLabelCandidateDescriptorProvider(graph: IGraph): ILabelCandidateDescriptorProvider | null;
/**
Returns the layout information for all node labels belonging to the given node.
This information contains the location and size of the labels.
@param node the node that owns the labels
@returns the array of all labels that belong to the given node
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23getNodeLabelLayout(Object) Online Documentation}*/
getNodeLabelLayout(node: any): INodeLabelLayout[];
/**
Returns the layout information associated with the given node.
This information contains the location and size of the node.
@param node the node
@returns the layout information for the node
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23getNodeLayout(Object) Online Documentation}*/
getNodeLayout(node: any): INodeLayout;
/**
Returns the source node associated with the given edge.
@param edgeObject
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23getSource(Object) Online Documentation}*/
getSource(edgeObject: Object): Object;
/**
Returns the target node associated with the given edge.
@param edgeObject
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23getTarget(Object) Online Documentation}*/
getTarget(edgeObject: Object): Object;
/**
Returns an {@link IEnumerable} that provides access to all nodes residing in the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23nodeObjects() Online Documentation}*/
nodeObjects(): IEnumerable<any>;
/**
Callback method that assigns a new location that matches <code>newLocation</code> to <code>port</code>.
This method is called only for ports which are owned by edges. These are represented by dummy nodes during the layout,
and <code>newLocation</code> is the position of the dummy node after the layout.
@param port The edge port that should be placed.
@param newLocation The new location of the port as calculated by the layout algorithm.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23setEdgePortLocation(IPort%2CPoint) Online Documentation}*/
setEdgePortLocation(port: IPort, newLocation: Point): void;
/**
Callback method that assigns a new relative location to <code>port</code>.
The default implementation calls {@link IGraph#setRelativePortLocation} with
<code>newRelativeLocation</code>. Depending on the port location model, the actual resulting port location may be different.
@param port The port that should be placed.
@param newRelativeLocation The new coordinate offsets relative to the center of the node's {@link INode#layout}'s center.
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23setRelativePortLocation(IPort%2CPoint) Online Documentation}*/
setRelativePortLocation(port: IPort, newRelativeLocation: Point): void;
/**
An {@link IMapperRegistry} tag for an {@link IMapper} that provides for the edge {@link ILabel}s in the {@link LayoutGraphAdapter#adaptedGraph} a {@link PreferredPlacementDescriptor}
instance.
If there is a {@link IMapper} in the {@link IGraph#mapperRegistry} it will be queried for each {@link IEdge edge} {@link ILabel label} to set the {@link IEdgeLabelLayout#preferredPlacementDescriptor} property of the adapted implementation.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY Online Documentation}*/
static EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY: IEdgeLabelLayoutDpKey<PreferredPlacementDescriptor>;
/**
A data provider key that can be used to look up the original {@link INode} of a {@link YNode}.
This data provider is created automatically for a {@link CopiedLayoutGraph} that is created as a copy of an {@link IGraph}
with the help of a {@link LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link LayoutExecutor}
or methods {@link GraphComponent#morphLayout} and ApplyLayout.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23ORIGINAL_NODE_DP_KEY Online Documentation}*/
static ORIGINAL_NODE_DP_KEY: NodeDpKey<INode>;
/**
A data provider key that can be used to look up the original {@link IEdge} of an {@link Edge}.
This data provider is created automatically for a {@link CopiedLayoutGraph} that is created as a copy of an {@link IGraph}
with the help of a {@link LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link LayoutExecutor}
or methods {@link GraphComponent#morphLayout} and ApplyLayout.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23ORIGINAL_EDGE_DP_KEY Online Documentation}*/
static ORIGINAL_EDGE_DP_KEY: EdgeDpKey<IEdge>;
/**
A data provider key that can be used to look up the original {@link ILabel} of an {@link INodeLabelLayout}.
This data provider is created automatically for a {@link CopiedLayoutGraph} that is created as a copy of an {@link IGraph}
with the help of a {@link LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link LayoutExecutor}
or methods {@link GraphComponent#morphLayout} and ApplyLayout.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23ORIGINAL_NODE_LABEL_DP_KEY Online Documentation}*/
static ORIGINAL_NODE_LABEL_DP_KEY: INodeLabelLayoutDpKey<ILabel>;
/**
A data provider key that can be used to look up the original {@link ILabel} of an {@link IEdgeLabelLayout}.
This data provider is created automatically for a {@link CopiedLayoutGraph} that is created as a copy of an {@link IGraph}
with the help of a {@link LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link LayoutExecutor}
or methods {@link GraphComponent#morphLayout} and ApplyLayout.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23ORIGINAL_EDGE_LABEL_DP_KEY Online Documentation}*/
static ORIGINAL_EDGE_LABEL_DP_KEY: IEdgeLabelLayoutDpKey<ILabel>;
/**
A data provider key that can be used to look up the {@link IPort source port} of the original {@link IEdge} of an
{@link Edge}.
This data provider is created automatically for a {@link CopiedLayoutGraph} that is created as a copy of an {@link IGraph}
with the help of a {@link LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link LayoutExecutor}
or methods {@link GraphComponent#morphLayout} and ApplyLayout.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23ORIGINAL_SOURCE_PORT_DP_KEY Online Documentation}*/
static ORIGINAL_SOURCE_PORT_DP_KEY: EdgeDpKey<IPort>;
/**
A data provider key that can be used to look up the {@link IPort target port} of the original {@link IEdge} of an
{@link Edge}.
This data provider is created automatically for a {@link CopiedLayoutGraph} that is created as a copy of an {@link IGraph}
with the help of a {@link LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link LayoutExecutor}
or methods {@link GraphComponent#morphLayout} and ApplyLayout.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23ORIGINAL_TARGET_PORT_DP_KEY Online Documentation}*/
static ORIGINAL_TARGET_PORT_DP_KEY: EdgeDpKey<IPort>;
/**
A data provider key that can be used to look up the original <b>unconnected</b>{@link IPort ports} of the original {@link IPortOwner} of an {@link YNode}
or, in the case of edge-to-edge connections, an {@link Edge}.
You can access <b>all</b> (connected and unconnected) ports through the {@link IPortOwner#ports} property of the original {@link IGraph} elements,
which can be retrieved through the {@link LayoutGraphAdapter#ORIGINAL_NODE_DP_KEY} and {@link LayoutGraphAdapter#ORIGINAL_EDGE_DP_KEY}
data providers. The information about whether a port is connected or not is not readily accessible from there, though.
You can use this data provider to specifically find these port, e.g. to create additional
{@link PortCandidate}s or just reserve space for them.
This data provider is created automatically for a {@link CopiedLayoutGraph} that is created as a copy of an {@link IGraph}
with the help of a {@link LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link LayoutExecutor}
or methods {@link GraphComponent#morphLayout} and ApplyLayout.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23UNCONNECTED_PORTS_DP_KEY Online Documentation}*/
static UNCONNECTED_PORTS_DP_KEY: GraphObjectDpKey<IEnumerable<IPort>>;
/**
This key is used to provide information about dummy nodes that have been inserted into the graph to model edges at
edges.
This key can be used to query an {@link IDataProvider} that yields <code>true</code> for {@link IDataProvider#getBoolean}
for nodes that are actually representatives of {@link IPort}s at {@link IEdge}s. Also the {@link IDataProvider#get}
method will yield the corresponding original {@link IEdge}.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23PORT_DUMMY_NODE_DP_KEY Online Documentation}*/
static PORT_DUMMY_NODE_DP_KEY: NodeDpKey<IEdge>;
/**
Sets or yields the original {@link IGraph} instance that is adapted by this instance.
Before performing a layout, this property must be set <b>or</b> the adapter must have been created with
{@link LayoutGraphAdapter}. Note that this property cannot be assigned again if its value is already non <code>null</code>.
@throws {Exception} <b>(ArgumentError)</b> if the value of this property is already non <code>null</code>
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23adaptedGraph Online Documentation}*/
adaptedGraph: IGraph | null;
/**
Gets or sets the {@link ISelectionModel} to use for the automatically registered {@link IDataProvider} instances for the keys {@link LayoutKeys#AFFECTED_NODES_DP_KEY}
and {@link LayoutKeys#AFFECTED_EDGES_DP_KEY}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23selectionModel Online Documentation}*/
selectionModel: ISelectionModel<IModelItem> | null;
/**
Gets or sets how {@link ILabel}s at {@link IPort}s should be treated by the layout algorithm.
This mapping is queried for each {@link ILabel}s at an {@link IPort} and should return a {@link PortLabelPolicy}
value indicating how the label should appear to the layout algorithms. The default is a constant {@link ItemMapping} returning {@link PortLabelPolicy#NODE_LABEL}
for all labels.
@see PortLabelPolicy
@see LayoutExecutor#portLabelPolicies
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23portLabelPolicies Online Documentation}*/
portLabelPolicies: ItemMapping<ILabel, PortLabelPolicy>;
/**
Gets or sets the size of the nodes that are inserted for the ports that are created for {@link IEdge}s that are
connected at other {@link IEdge}s.
If this instance is configured to create dummy nodes for ports at edges, this will be the initial size of the dummy
nodes with the center of the node always being the location of the port. The default is <code>3x3</code>.
@see LayoutGraphAdapter#hideEdgesAtEdges
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23edgePortNodeSize Online Documentation}*/
edgePortNodeSize: Size;
/**
Gets or sets a value that controls whether edges at other edges will be hidden from the layout graph or included.
If this property is set to <code>true</code>, edges at other edges are simply ignored and not part of the layout graph. The default
is <code>false</code> in which case dummy nodes are inserted into the layout graph for each source and target port of an {@link IEdge}
that is {@link IPort#owner owned} by an {@link IEdge}.
@see LayoutGraphAdapter#edgePortNodeSize
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23hideEdgesAtEdges Online Documentation}*/
hideEdgesAtEdges: boolean;
/**
Gets or sets a value that controls whether edge groups are automatically created for edges that are connected to the
same port.
If this property is enabled and no edge groups are explicitly created ({@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
and {@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), for all ports with multiple outgoing resp. incoming edges
these edges will be assigned to the same edge group. The necessary
{@link IDataProvider}s will be created automatically.
This features requires edge group support from the {@link ILayoutAlgorithm}.
Default value is <code>true</code>
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see LayoutExecutor#automaticEdgeGrouping
@see LayoutGraphAdapter#fixPorts
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23automaticEdgeGrouping Online Documentation}*/
automaticEdgeGrouping: boolean;
/**
Gets or sets a value that controls whether strong port constraints are automatically created.
If this property is enabled and no port constraints have been added explicitly ({@link PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
and {@link PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), strong source and target port constraints are created
automatically. The necessary
{@link IDataProvider}s will be created automatically. The edge direction is inferred from the location of the
port relative to the node.
This ensures that view {@link IPort}s are not moved during the layout, if the {@link ILayoutAlgorithm}
supports port constraints.
Default value is <code>false</code>
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see LayoutExecutor#fixPorts
@see LayoutGraphAdapter#automaticEdgeGrouping
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23fixPorts Online Documentation}*/
fixPorts: boolean;
/**
Gets or sets the policy that specifies how port locations should be adjusted after a layout has been calculated.
This can be useful if the port assignment calculated by the layout algorithm is insufficient.
Layout algorithms only consider rectangular nodes even though the actual shape of a node is, for example, circular.
Hence, the ports are usually placed at the border of the nodes' {@link INode#layout bounding boxes} (except for some layout algorithms that
produce straight-line edge routes and place the ports at the nodes' center).
Based on this setting the edges will be shortened or lengthened in a way that their {@link IEdge#sourcePort}s and {@link IEdge#targetPort}s will be placed on the node's
{@link IShapeGeometry#getOutline outline}.
The default value is {@link PortAdjustmentPolicy#NEVER}.
<h2>Note</h2>
The coordinates of a port will not be changed if the port is associated with a strong port constraint, e.g. by setting {@link LayoutGraphAdapter#fixPorts}
to <code>true</code>.
@see PortCalculator
@see LayoutExecutor#portAdjustmentPolicy
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23portAdjustmentPolicy Online Documentation}*/
portAdjustmentPolicy: PortAdjustmentPolicy;
/**
Gets an array of all data provider look-up keys that are registered with the graph.
@see {@link https://docs.yworks.com/yfileshtml/#/api/LayoutGraphAdapter%23dataProviderKeys Online Documentation}*/
readonly dataProviderKeys: any[];
static $class: Class;
private $_LayoutGraphAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is LayoutGraphAdapter;
}
export interface MultiPageLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link MultiPageLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutData Online Documentation}*/
export class MultiPageLayoutData {
/**
Creates a new instance of {@link MultiPageLayoutData} which helps configuring
{@link MultiPageLayout}.
@param [options] The parameters to pass.
@param [options.nodeIds] The mapping from nodes to their unique IDs. This option sets the {@link MultiPageLayoutData#nodeIds} property on the created object.
@param [options.edgeIds] The mapping from edges to their unique IDs. This option sets the {@link MultiPageLayoutData#edgeIds} property on the created object.
@param [options.nodeLabelIds] The mapping from node labels to their unique IDs. This option sets the {@link MultiPageLayoutData#nodeLabelIds} property on the created object.
@param [options.edgeLabelIds] The mapping from edge labels to their unique IDs. This option sets the {@link MultiPageLayoutData#edgeLabelIds} property on the created object.
@param [options.nodeClusterIds] The mapping from nodes to their cluster id. This option sets the {@link MultiPageLayoutData#nodeClusterIds} property on the created object.
@param [options.edgeTypes] The mapping from edges to their type. This option sets the {@link MultiPageLayoutData#edgeTypes} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link MultiPageLayoutData#abortHandler} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutData Online Documentation}*/
constructor(options?: {
nodeIds?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
edgeIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
nodeLabelIds?: ItemMapping<ILabel, any> | ItemMappingConvertible<ILabel, any>;
edgeLabelIds?: ItemMapping<ILabel, any> | ItemMappingConvertible<ILabel, any>;
nodeClusterIds?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
edgeTypes?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
abortHandler?: AbortHandler | null;
});
/**
Gets or sets the mapping from nodes to their unique IDs.
@see MultiPageLayout#NODE_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutData%23nodeIds Online Documentation}*/
nodeIds: ItemMapping<INode, any>;
/**
Gets or sets the mapping from edges to their unique IDs.
@see MultiPageLayout#EDGE_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutData%23edgeIds Online Documentation}*/
edgeIds: ItemMapping<IEdge, any>;
/**
Gets or sets the mapping from node labels to their unique IDs.
@see MultiPageLayout#NODE_LABEL_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutData%23nodeLabelIds Online Documentation}*/
nodeLabelIds: ItemMapping<ILabel, any>;
/**
Gets or sets the mapping from edge labels to their unique IDs.
@see MultiPageLayout#EDGE_LABEL_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutData%23edgeLabelIds Online Documentation}*/
edgeLabelIds: ItemMapping<ILabel, any>;
/**
Gets or sets the mapping from nodes to their cluster id.
Nodes with the same cluster id should preferably be placed on the same page.
@see MultiPageLayout#NODE_CLUSTER_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutData%23nodeClusterIds Online Documentation}*/
nodeClusterIds: ItemMapping<INode, any>;
/**
Gets or sets the mapping from edges to their type.
Multi-edges (edges with same endpoints) of different kind are distinguished if {@link MultiPageLayout#edgeBundleModeMask} &amp; {@link EdgeBundleModes#DISTINGUISH_TYPES}
== 1.
@see MultiPageLayout#EDGE_TYPE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutData%23edgeTypes Online Documentation}*/
edgeTypes: ItemMapping<IEdge, any>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MultiPageLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
static $class: Class;
private $_MultiPageLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is MultiPageLayoutData;
}
export interface OrganicEdgeRouterData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link OrganicEdgeRouter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouterData Online Documentation}*/
export class OrganicEdgeRouterData {
/**
Creates a new instance of {@link OrganicEdgeRouterData} which helps configuring {@link OrganicEdgeRouter}.
@param [options] The parameters to pass.
@param [options.affectedEdges] The collection of edges routed by this router. This option sets the {@link OrganicEdgeRouterData#affectedEdges} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link OrganicEdgeRouterData#abortHandler} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouterData Online Documentation}*/
constructor(options?: {
affectedEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
abortHandler?: AbortHandler | null;
});
/**
* Gets or sets the collection of edges routed by this router.
* Defining the subset of edges that should be laid out can be done in various ways, mostly depending on which option is
* more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of affected edges</b>
* <pre>
* layoutData.affectedEdges.source = graphComponent.selection.selectedEdges
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual affected edges</b>
* <pre>
* layoutData.affectedEdges.items.add(edge1)
* layoutData.affectedEdges.items.add(edge2)
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every edge
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether an edge is affected or not</b>
* <pre>
* // We assume here that all edges have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.affectedEdges.delegate = edge => edge.tag.includeInLayout
* </pre>
*
* @see OrganicEdgeRouter#AFFECTED_EDGES_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouterData%23affectedEdges Online Documentation}
*/
affectedEdges: ItemCollection<IEdge>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicEdgeRouterData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
static $class: Class;
private $_OrganicEdgeRouterData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrganicEdgeRouterData;
}
export interface OrganicLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link OrganicLayout}.
* The following example shows how to create a new instance of {@link OrganicLayoutData} and use it with an
* {@link OrganicLayout}:
*
* <br><b>Creating an instance of OrganicLayoutData</b>
* <pre>
* const layoutData = new OrganicLayoutData()
* layoutData.preferredEdgeLengths.constant = 45
* layoutData.affectedNodes.source = graphComponent.selection.selectedNodes
*
* graphComponent.graph.applyLayout(new OrganicLayout(), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link OrganicLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of OrganicLayoutData</b>
* <pre>
* const layoutData = new OrganicLayoutData({
*   preferredEdgeLengths: 45,
*   affectedNodes: graphComponent.selection.selectedNodes
* })
*
* graphComponent.graph.applyLayout(new OrganicLayout(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData Online Documentation}
*/
export class OrganicLayoutData {
/**
Creates a new instance of {@link OrganicLayoutData} which helps configuring {@link OrganicLayout}.
@param [options] The parameters to pass.
@param [options.affectedNodes] The subset of nodes that is moved by the layout algorithm. This option sets the {@link OrganicLayoutData#affectedNodes} property on the created object.
@param [options.preferredEdgeLengths] A mapping from edges to their preferred lengths. This option sets the {@link OrganicLayoutData#preferredEdgeLengths} property on the created object.
@param [options.minimumNodeDistances] A mapping from nodes to their minimum distances to other nodes around them. This option sets the {@link OrganicLayoutData#minimumNodeDistances} property on the created object.
@param [options.groupNodeModes] The mapping from group nodes to a mode constant describing how to handle the group node. This option sets the {@link OrganicLayoutData#groupNodeModes} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link OrganicLayoutData#nodeHalos} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link OrganicLayoutData#abortHandler} property on the created object.
@param [options.partitionGridData] The {@link OrganicLayoutData#partitionGridData partition grid} layout data. This option sets the {@link OrganicLayoutData#partitionGridData} property on the created object.
@param [options.edgeDirectedness] The mapping from edges to their directedness. This option sets the {@link OrganicLayoutData#edgeDirectedness} property on the created object.
@param [options.sourceGroupIds] A mapping from edges to an object representing their source edge group. This option sets the {@link OrganicLayoutData#sourceGroupIds} property on the created object.
@param [options.targetGroupIds] A mapping from edges to an object representing their target edge group. This option sets the {@link OrganicLayoutData#targetGroupIds} property on the created object.
@param [options.nodeInertia] The mapping from non-group nodes to their inertia. This option sets the {@link OrganicLayoutData#nodeInertia} property on the created object.
@param [options.nodeStress] The mapping from non-group nodes to their stress value. This option sets the {@link OrganicLayoutData#nodeStress} property on the created object.
@param [options.zCoordinates] A mapper from non-group nodes to the computed center z-coordinate in case that a {@link OrganicLayout#create3DLayout 3D layout} was created. This option sets the {@link OrganicLayoutData#zCoordinates} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData Online Documentation}*/
constructor(options?: {
affectedNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
preferredEdgeLengths?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
minimumNodeDistances?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
groupNodeModes?: ItemMapping<INode, GroupNodeMode> | ItemMappingConvertible<INode, GroupNodeMode>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
abortHandler?: AbortHandler | null;
partitionGridData?: PartitionGridData;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
sourceGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
nodeInertia?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
nodeStress?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
zCoordinates?: IMapper<INode, number> | null | IMapperConvertible<INode, number>;
});
/**
* Gets or sets the subset of nodes that is moved by the layout algorithm.
* This property is only considered if the {@link OrganicLayout}'s {@link OrganicLayout#scope} is set to {@link OrganicLayoutScope#SUBSET} or
* {@link OrganicLayoutScope#MAINLY_SUBSET}.
*
* Defining the subset of nodes that should be laid out can be done in various ways, mostly depending on which option is
* more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of affected nodes</b>
* <pre>
* const layoutData = new OrganicLayoutData({
*   affectedNodes: graphComponent.selection.selectedNodes
* })
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual nodes as affected nodes</b>
* <pre>
* for (const /** @type {IEdge} *‍/ edge of graphComponent.selection.selectedEdges) {
*   layoutData.affectedNodes.items.add(edge.sourceNode)
*   layoutData.affectedNodes.items.add(edge.targetNode)
* }
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every node
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether a node is affected or not</b>
* <pre>
* // We assume here that all nodes have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.affectedNodes.delegate = node => node.tag.includeInLayout
* </pre>
*
* @see OrganicLayout#AFFECTED_NODES_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23affectedNodes Online Documentation}
*/
affectedNodes: ItemCollection<INode>;
/**
* Gets or sets a mapping from edges to their preferred lengths.
* If the preferred length for an edge is not explicitly mapped in the {@link ItemMapping#mapper}, or mapped to <code>0</code> or a negative value, the default
* value from {@link OrganicLayout#preferredEdgeLength} will be used instead.
*
* Specifying a preferred length for specific edges, overriding the default setting on {@link OrganicLayout} can be
* accomplished easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set different preferred lengths for some edges</b>
* <pre>
* layoutData.preferredEdgeLengths.mapper.set(edge1, 120)
* layoutData.preferredEdgeLengths.mapper.set(edge2, 75)
* // All other edges not set in the mapper implicitly get the default value
* // from OrganicLayout.preferredEdgeLength.
* </pre>
*
* In cases where the preferred length can be determined by looking at the edge itself it's often easier to just set a
* delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine preferred edge lengths</b>
* <pre>
* // Ensure that there is enough space for the edge's label.
* // We'll assume that every edge has a label here.
* layoutData.preferredEdgeLengths.delegate = edge => edge.labels.get(0).preferredSize.width * 1.2
* </pre>
*
* Finally, there's also the somewhat nonsensical option to use the same value for all edges, which, as the comment notes,
* can be achieved easier by just setting the respective property on {@link OrganicLayout}:
*
* <br><b>Using a constant preferred edge length</b>
* <pre>
* layoutData.preferredEdgeLengths.constant = 120
* // This is equivalent to the following property on OrganicLayout:
* organicLayout.preferredEdgeLength = 120
* </pre>
*
* @see OrganicLayout#PREFERRED_EDGE_LENGTH_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23preferredEdgeLengths Online Documentation}
*/
preferredEdgeLengths: ItemMapping<IEdge, number>;
/**
* Gets or sets a mapping from nodes to their minimum distances to other nodes around them.
* This setting only has an effect when {@link OrganicLayout#nodeOverlapsAllowed} is <code>false</code> and only applies to the post-processing step that moves overlapping
* nodes. So the distances from a node that never had overlaps to begin with, may not follow this setting.
*
* Minimum distance values must be greater than <code>0</code>.
*
* Specifying a minimum node distance for specific nodes, overriding the default setting on {@link OrganicLayout}
* can be accomplished easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set different minimum distances for some nodes</b>
* <pre>
* layoutData.minimumNodeDistances.mapper.set(node1, 25)
* layoutData.minimumNodeDistances.mapper.set(node2, 45)
* // All other edges not set in the mapper implicitly get the default value
* // from OrganicLayout.minimumNodeDistance.
* </pre>
*
* In cases where the minimum distance can be determined by looking at the node itself it's often easier to just set a
* delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine minimum node distances</b>
* <pre>
* // Scale the minimum distance around a node by the node size.
* // This ensures that larger nodes get more space.
* layoutData.minimumNodeDistances.delegate = node => Math.max(node.layout.width, node.layout.height)
* </pre>
*
* Finally, there's also the somewhat nonsensical option to use the same value for all edges, which, as the comment notes,
* can be achieved easier by just setting the respective property on {@link OrganicLayout}:
*
* <br><b>Using a constant minimum node distance</b>
* <pre>
* layoutData.minimumNodeDistances.constant = 25
* // This is equivalent to the following property on OrganicLayout:
* organicLayout.minimumNodeDistance = 25
* </pre>
*
* @see ShuffleLayout#MINIMUM_DISTANCE_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23minimumNodeDistances Online Documentation}
*/
minimumNodeDistances: ItemMapping<INode, number>;
/**
* Gets or sets the mapping from group nodes to a mode constant describing how to handle the group node.
* The modes specify how a group's content is handled and if it is resized during layout calculation. The algorithm expects
* for each group node to find one of the following constants:
*
* <ul>
* <li>{@link GroupNodeMode#NORMAL}</li>
* <li>{@link GroupNodeMode#FIX_BOUNDS}</li>
* <li>{@link GroupNodeMode#FIX_CONTENTS}</li>
* </ul>
* The easiest option is to treat all group nodes the same, by just setting a constant value:
*
* <br><b>Using a constant mode for handling group nodes</b>
* <pre>
* layoutData.groupNodeModes.constant = GroupNodeMode.FIX_CONTENTS
* </pre>
*
* Handling only certain group nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to treat certain group nodes differently</b>
* <pre>
* layoutData.groupNodeModes.mapper.set(groupNode1, GroupNodeMode.FIX_BOUNDS)
* layoutData.groupNodeModes.mapper.set(groupNode2, GroupNodeMode.FIX_CONTENTS)
* // All other edges not set in the mapper implicitly get the default value
* // GroupNodeMode.normal.
* </pre>
*
* In cases where the ideal {@link GroupNodeMode} can be determined by looking at the node itself it's often easier
* to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine how the layout should treat each group node</b>
* <pre>
* layoutData.groupNodeModes.delegate = node => {
*   if (graph.getChildren(node).size > 0) {
*     return GroupNodeMode.NORMAL
*   }
*   // Ensure that empty group nodes remain where they are
*   return GroupNodeMode.FIX_BOUNDS
* }
* </pre>
*
* @see OrganicLayout#GROUP_NODE_MODE_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23groupNodeModes Online Documentation}
*/
groupNodeModes: ItemMapping<INode, GroupNodeMode>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
* Gets or sets the {@link OrganicLayoutData#partitionGridData partition grid} layout data.
* Note that if you use {@link LayoutExecutor}'s {@link LayoutExecutor#configureTableLayout} feature, then this should not be used, as it overrides the {@link PartitionGrid}
* set by {@link TableLayoutConfigurator}.
*
* The following sample shows how to assign nodes to partition grid cells simply via cell indices:
*
* <br><b>Assigning nodes to partition grid cells via indices</b>
* <pre>
* // Create four nodes and place them in grid cells in the following way
* // +--+--+--+
* // | 1| 2| 3|
* // +--+--+--+
* //    | 4|
* //    +--+
*
* const layoutData = new OrganicLayoutData()
* const gridData = layoutData.partitionGridData
*
* const node1 = graph.createNode()
* const node2 = graph.createNode()
* const node3 = graph.createNode()
* const node4 = graph.createNode()
*
* // Assign the nodes to their rows and columns.
* // Note that don't have to create or use PartitionGrid directly in this case.
* // Setting the indices is enough.
* gridData.rowIndices.mapper.set(node1, 0)
* gridData.rowIndices.mapper.set(node2, 0)
* gridData.rowIndices.mapper.set(node3, 0)
* gridData.rowIndices.mapper.set(node4, 1)
* gridData.columnIndices.mapper.set(node1, 0)
* gridData.columnIndices.mapper.set(node2, 1)
* gridData.columnIndices.mapper.set(node3, 2)
* gridData.columnIndices.mapper.set(node4, 1)
*
* graph.applyLayout(new OrganicLayout(), layoutData)
* </pre>
*
* When used this way there is no need to create a {@link PartitionGrid} instance or work with it directly. For more
* flexibility, e.g. to use cells that span multiple columns or rows, the {@link PartitionGrid} can be used as well:
*
* <br><b>Assigning nodes to partition grid cells via factory methods on the grid</b>
* <pre>
* // Create three nodes and place them in grid cells in the following way
* // +--+--+
* // | 1| 2|
* // +--+--+
* // |  3  |
* // +--+--+
*
* const node1 = graph.createNode(new Rect(0, 0, 50, 50))
* const node2 = graph.createNode(new Rect(0, 0, 50, 50))
* const node3 = graph.createNode(new Rect(0, 0, 125, 50))
*
* // Create a new PartitionGrid with two rows and two columns
* const grid = new PartitionGrid(2, 2)
*
* // Assign the nodes to their cells
* const cellIdMapper = new Mapper()
* cellIdMapper.set(node1, grid.createCellId(0, 0))
* cellIdMapper.set(node2, grid.createCellId(0, 1))
* cellIdMapper.set(node3, grid.createRowSpanId(1))
*
* const layoutData = new OrganicLayoutData({
*   partitionGridData: {
*     grid: grid,
*     cellIds: cellIdMapper
*   }
* })
*
* graph.applyLayout(new OrganicLayout(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23partitionGridData Online Documentation}
*/
partitionGridData: PartitionGridData;
/**
* Gets or sets the mapping from edges to their directedness.
* Generally, the organic layout algorithm doesn't consider the edge direction. Nevertheless, this property allows the user
* to specify hints on the directedness of edges. More precisely, a value of <code>1</code> indicates that the edge should be considered
* to be directed from source to target, a value of <code>-1</code> that it is directed from target to source, and a value of <code>0</code> means
* that it is undirected.
*
* Currently, the specified values are only considered during the detection of special substructures, see {@link OrganicLayout#chainSubstructureStyle}, {@link OrganicLayout#cycleSubstructureStyle}, {@link OrganicLayout#parallelSubstructureStyle} and {@link OrganicLayout#starSubstructureStyle}.
*
* The easiest option is to define all edges with the same directedness:
*
* <br><b>Treating all edges as directed</b>
* <pre>
* layoutData.edgeDirectedness.constant = 1
* </pre>
*
* Handling only certain edges differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set directedness for certain edges</b>
* <pre>
* // edge1 should be considered directed
* layoutData.edgeDirectedness.mapper.set(edge1, 1)
* // edge2 should be considered directed against the flow
* layoutData.edgeDirectedness.mapper.set(edge2, -1)
* // All other edges not set in the mapper are treated as undirected
* </pre>
*
* In cases where the directedness for each edge can be determined by looking at the edge itself it's often easier to just
* set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine edge directedness</b>
* <pre>
* // Treat edges as directed or undirected based on their style's arrowhead
* layoutData.edgeDirectedness.delegate = edge => {
*   const style = edge.style
*   // edges with with either no arrows on both ends or an arrow on both ends should be considered undirected
*   if (
*     (style.sourceArrow === null && style.targetArrow === null) ||
*     (style.sourceArrow !== null && style.targetArrow !== null)
*   ) {
*     return 0
*   }
*   // edges with only a target arrow are directed from source to target
*   if (style.targetArrow !== null) {
*     return 1
*   }
*   // edges with only a source arrow are directed from target to source
*   if (style.sourceArrow !== null) {
*     return -1
*   }
*   return 0
* }
* </pre>
*
* @see OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23edgeDirectedness Online Documentation}
*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
* Gets or sets a mapping from edges to an object representing their source edge group.
* For normal structures, the organic layout does not support the classical edge grouping where edges at a common source
* node share a segment or a bus. The edge groups are only considered in the cases described below.
*
* <ul>
* <li>When the star substructure style {@link StarSubstructureStyle#SEPARATED_RADIAL} is applied, edges grouped at the
* root node are drawn in a grouped routing style.</li>
* <li>When one of the parallel substructure styles {@link ParallelSubstructureStyle#RECTANGULAR}, {@link ParallelSubstructureStyle#RADIAL}
* or {@link ParallelSubstructureStyle#STRAIGHT_LINE} is applied, then edges of the parallel structures may be
* grouped at their outer nodes.</li>
* </ul>
* One simple way to use source groups is to use the edge's source node as group ID which effectively groups all edges with
* the same source together:
*
* <br><b>Grouping all edges on the source side</b>
* <pre>
* layoutData.sourceGroupIds = edge => edge.sourceNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.sourceGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.sourceGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see OrganicLayoutData#targetGroupIds
* @see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23sourceGroupIds Online Documentation}
*/
sourceGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to an object representing their target edge group.
* For normal structures, the organic layout does not support the classical edge grouping where edges at a common target
* node share a segment or a bus. The edge groups are only considered in the cases described below.
*
* <ul>
* <li>When the star substructure style {@link StarSubstructureStyle#SEPARATED_RADIAL} is applied, edges grouped at the
* root node are drawn in a grouped routing style.</li>
* <li>When one of the parallel substructure styles {@link ParallelSubstructureStyle#RECTANGULAR}, {@link ParallelSubstructureStyle#RADIAL}
* or {@link ParallelSubstructureStyle#STRAIGHT_LINE} is applied, then edges of the parallel structures may be
* grouped at their outer nodes.</li>
* </ul>
* One simple way to use source groups is to use the edge's target node as group ID which effectively groups all edges with
* the same target together:
*
* <br><b>Grouping all edges on the target side</b>
* <pre>
* layoutData.targetGroupIds = edge => edge.targetNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.targetGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.targetGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see OrganicLayoutData#sourceGroupIds
* @see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23targetGroupIds Online Documentation}
*/
targetGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets the mapping from non-group nodes to their inertia.
* The inertia can be specified if the {@link OrganicLayout#scope scope} is set to {@link OrganicLayoutScope#ALL}. It is defined to be a value from the
* interval <code>[0,1]</code>. For nodes of sub-structures the inertia value is not considered. The default inertia value is <code>0.0</code> and
* applied if no inertia values are defined or if a node has no individual value.
*
* <ul>
* <li><b>1.0</b> – The node will not move.</li>
* <li><b>0.5</b> – The node will only move half as far as it would with an inertia of <code>0.0</code>.</li>
* <li><b>0.0</b> – The node will move as fast as possible.</li>
* </ul>
* <h2>Note</h2>
*
* If there is at least one node with inertia value greater than <code>0.0</code>, the algorithm considers the initial coordinates of
* the nodes (similar as for {@link OrganicLayoutScope#SUBSET}).
*
* The easiest option is to define the same inertia for all nodes:
*
* <br><b>Using the same inertia for all nodes</b>
* <pre>
* // Allow only a little movement for all nodes
* layoutData.nodeInertia.constant = 0.9
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set inertia for certain nodes</b>
* <pre>
* layoutData.nodeInertia.mapper.set(node1, 0.3)
* layoutData.nodeInertia.mapper.set(node2, 0.7)
* // All other edges not set in the mapper implicitly get
* // the default value 0.0
* </pre>
*
* In cases where the inertia for each node can be determined by looking at the node itself it's often easier to just set a
* delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine inertia</b>
* <pre>
* layoutData.nodeInertia.delegate = node =>
*   // Move only the selected nodes
*   graphComponent.selection.isSelected(node) ? 0.0 : 1.0
* </pre>
*
* @see OrganicLayout#NODE_INERTIA_DP_KEY
* @see OrganicLayoutData#nodeStress
* @see OrganicLayout#scope
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23nodeInertia Online Documentation}
*/
nodeInertia: ItemMapping<INode, number>;
/**
* Gets or sets the mapping from non-group nodes to their stress value.
* The stress value indicates how far a node will possibly move. The higher the stress of a node is, the farther it may
* move.
*
* The stress can be specified if the {@link OrganicLayout#scope scope} is set to {@link OrganicLayoutScope#ALL}. It is defined to be a value from the
* interval <code>[0,1]</code>. For nodes of sub-structures the stress value is not considered. The default stress value is <code>1.0</code> and
* applied if no stress values are defined or if a node has no individual value.
*
* <h2>Note</h2>
*
* If there is at least one node with stress value less than <code>1.0</code>, the algorithm considers the initial coordinates of the
* nodes (similar as for {@link OrganicLayoutScope#SUBSET}).
*
* The easiest option is to define the same stress for all nodes:
*
* <br><b>Using the same stress for all nodes</b>
* <pre>
* // All nodes should not move too far away
* layoutData.nodeStress.constant = 0.3
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set stress for certain nodes</b>
* <pre>
* layoutData.nodeStress.mapper.set(node1, 0.3)
* layoutData.nodeStress.mapper.set(node2, 0.7)
* // All other edges not set in the mapper implicitly get
* // the default value 1.0
* </pre>
*
* In cases where the inertia for each node can be determined by looking at the node itself it's often easier to just set a
* delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine stress</b>
* <pre>
* layoutData.nodeStress.delegate = node =>
*   // Move only the selected nodes
*   graphComponent.selection.isSelected(node) ? 1.0 : 0.0
* </pre>
*
* @see OrganicLayout#NODE_STRESS_DP_KEY
* @see OrganicLayoutData#nodeInertia
* @see OrganicLayout#scope
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23nodeStress Online Documentation}
*/
nodeStress: ItemMapping<INode, number>;
/**
* Gets or sets a mapper from non-group nodes to the computed center z-coordinate in case that a {@link OrganicLayout#create3DLayout 3D layout} was created.
* If this property is set, the mapper is filled by the layout with the computed z-coordinates of the non-group nodes. The
* node's world coordinates provides the matching x and y coordinates.
*
* This is only required if {@link OrganicLayout#create3DLayout} is enabled. Otherwise, the algorithm produces a 2D result.
*
* <br><b>Retrieving the z-coordinates of nodes after a layout</b>
* <pre>
* const layoutData = new OrganicLayoutData({
*   zCoordinates: new Mapper()
* })
*
* const layout = new OrganicLayout()
* // Enable creating a 3D result
* layout.create3DLayout = true
*
* graph.applyLayout(layout, layoutData)
*
* for (const /** @type {INode} *‍/ node in graph.nodes) {
*   // Print z-coordinate for all non-group nodes
*   if (!graph.isGroupNode(node)) {
*     console.log(`Node ${node} has z-coordinate ${layoutData.zCoordinates.get(node)}`)
*   }
* }
* </pre>
*
* @see OrganicLayout#Z_COORDINATE_DP_KEY
* @see OrganicLayout#create3DLayout
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrganicLayoutData%23zCoordinates Online Documentation}
*/
zCoordinates: IMapper<INode, number> | null;
static $class: Class;
private $_OrganicLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrganicLayoutData;
}
export interface OrthogonalLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link OrthogonalLayout}.
* The following example shows how to create a new instance of {@link OrthogonalLayoutData} and use it with an
* {@link OrthogonalLayout}:
*
* <br><b>Creating an instance of OrthogonalLayoutData</b>
* <pre>
* const layoutData = new OrthogonalLayoutData()
* layoutData.sourceGroupIds.delegate = edge => edge.sourceNode
* layoutData.directedEdges.source = graphComponent.selection.selectedEdges
* graphComponent.graph.applyLayout(new OrthogonalLayout(), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link OrthogonalLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of OrthogonalLayoutData</b>
* <pre>
* const layoutData = new OrthogonalLayoutData({
*   sourceGroupIds: edge => edge.sourceNode,
*   directedEdges: graphComponent.selection.selectedEdges
* })
*
* graphComponent.graph.applyLayout(new OrthogonalLayout(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData Online Documentation}
*/
export class OrthogonalLayoutData {
/**
Creates a new instance of {@link OrthogonalLayoutData} which helps configuring
{@link OrthogonalLayout}.
@param [options] The parameters to pass.
@param [options.edgeLayoutDescriptors] The mapping from edges to their {@link OrthogonalLayoutEdgeLayoutDescriptor}. This option sets the {@link OrthogonalLayoutData#edgeLayoutDescriptors} property on the created object.
@param [options.directedEdges] The collection of edges that should be routed in a way that point in the main layout direction. This option sets the {@link OrthogonalLayoutData#directedEdges} property on the created object.
@param [options.edgeDirectedness] The mapping from edges to their directedness, which is considered for the detection of substructures. This option sets the {@link OrthogonalLayoutData#edgeDirectedness} property on the created object.
@param [options.edgeCrossingCosts] The mapping from edges to their crossing cost. This option sets the {@link OrthogonalLayoutData#edgeCrossingCosts} property on the created object.
@param [options.edgeBendCosts] The mapping from edges to their bend cost. This option sets the {@link OrthogonalLayoutData#edgeBendCosts} property on the created object.
@param [options.sourceGroupIds] A mapping from edges to an object representing their source edge group. This option sets the {@link OrthogonalLayoutData#sourceGroupIds} property on the created object.
@param [options.targetGroupIds] A mapping from edges to an object representing their target edge group. This option sets the {@link OrthogonalLayoutData#targetGroupIds} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link OrthogonalLayoutData#nodeHalos} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link OrthogonalLayoutData#abortHandler} property on the created object.
@param [options.edgeLabelPreferredPlacement] The mapping that provides a {@link PreferredPlacementDescriptor} instance for edge {@link ILabel}s. This option sets the {@link OrthogonalLayoutData#edgeLabelPreferredPlacement} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData Online Documentation}*/
constructor(options?: {
edgeLayoutDescriptors?: ItemMapping<IEdge, OrthogonalLayoutEdgeLayoutDescriptor> | ItemMappingConvertible<IEdge, OrthogonalLayoutEdgeLayoutDescriptor>;
directedEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
edgeDirectedness?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
edgeCrossingCosts?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
edgeBendCosts?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
sourceGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
abortHandler?: AbortHandler | null;
edgeLabelPreferredPlacement?: ItemMapping<ILabel, PreferredPlacementDescriptor> | ItemMappingConvertible<ILabel, PreferredPlacementDescriptor>;
});
/**
Gets or sets the mapping from edges to their {@link OrthogonalLayoutEdgeLayoutDescriptor}.
If an edge is mapped to <code>null</code>, a default edge layout descriptor will be obtained using method
{@link OrthogonalLayout#createEdgeLayoutDescriptor}.
<b>Warning:</b> Minimum edge length constraints can only be considered for layout styles {@link OrthogonalLayoutStyle#NORMAL} and
{@link OrthogonalLayoutStyle#UNIFORM}.
@see OrthogonalLayoutEdgeLayoutDescriptor
@see OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23edgeLayoutDescriptors Online Documentation}*/
edgeLayoutDescriptors: ItemMapping<IEdge, OrthogonalLayoutEdgeLayoutDescriptor>;
/**
* Gets or sets the collection of edges that should be routed in a way that point in the main layout direction.
* The main layout orientation can be set using method {@link MultiStageLayout#layoutOrientation}.
*
* <b>Warning:</b> Directed edges will be ignored if group nodes exist in the graph.
*
* If only a few edges should be routed in layout direction, the easiest way is often to add them one by one via the {@link ItemCollection#items}
* collection:
*
* <pre>
* layoutData.directedEdges.items.add(edge1)
* layoutData.directedEdges.items.add(edge2)
* </pre>
*
* If you already have a collection or {@link IEnumerable} for those edges, the {@link ItemCollection#source} property is usually easier:
*
* <pre>
* // Consider outgoing edges at selected nodes
* layoutData.directedEdges.source = graphComponent.selection.selectedNodes.flatMap(node =>
*   graphComponent.graph.outEdgesAt(node)
* )
* </pre>
*
* If it can be inferred directly from the edge itself whether it should be routed that way, a delegate is often most
* convenient:
*
* <pre>
* layoutData.directedEdges.delegate = edge => edge.tag.routeInLayoutDirection
* </pre>
*
* @see OrthogonalLayout#DIRECTED_EDGE_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23directedEdges Online Documentation}
*/
directedEdges: ItemCollection<IEdge>;
/**
* Gets or sets the mapping from edges to their directedness, which is considered for the detection of substructures.
* The directedness is only considered for the detection of substructures in the input graph i.e., {@link OrthogonalLayout#treeStyle trees}, {@link OrthogonalLayout#chainStyle chains} and
* {@link OrthogonalLayout#cycleStyle cycles}. A substructure is only identified as such if all edges are either undirected or consistently directed with
* respect to the specified directedness.
*
* <ul>
* <li>A directedness value of <code>1</code> indicates that the edge is considered to be <b>directed from source to target</b>.</li>
* <li>A directedness value of <code>-1</code> indicates that the edge is considered to be <b>directed from target to source</b>.</li>
* <li>A directedness value of <code>0</code> indicates that the edge is considered to be <b>undirected</b>.</li>
* </ul>
* All edges are undirected per default.
*
* <b>Warning:</b> The edge directedness should not be confused with the definition of {@link OrthogonalLayoutData#directedEdges}. The latter one marks edges that are <em>directed orthogonal</em> and shall
* point in the specified
* {@link MultiStageLayout#layoutOrientation layout orientation}. On the other hand, the directedness solely affects the detection of substructures.
*
* The easiest option is to define all edges with the same directedness:
*
* <br><b>Treating all edges as directed</b>
* <pre>
* layoutData.edgeDirectedness.constant = 1
* </pre>
*
* Handling only certain edges differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set directedness for certain edges</b>
* <pre>
* // edge1 should be considered directed
* layoutData.edgeDirectedness.mapper.set(edge1, 1)
* // edge2 should be considered directed against the flow
* layoutData.edgeDirectedness.mapper.set(edge2, -1)
* // All other edges not set in the mapper are treated as undirected
* </pre>
*
* In cases where the directedness for each edge can be determined by looking at the edge itself it's often easier to just
* set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine edge directedness</b>
* <pre>
* // Treat edges as directed or undirected based on their style's arrowhead
* layoutData.edgeDirectedness.delegate = edge => {
*   const style = edge.style
*   // edges with with either no arrows on both ends or an arrow on both ends should be considered undirected
*   if (
*     (style.sourceArrow === null && style.targetArrow === null) ||
*     (style.sourceArrow !== null && style.targetArrow !== null)
*   ) {
*     return 0
*   }
*   // edges with only a target arrow are directed from source to target
*   if (style.targetArrow !== null) {
*     return 1
*   }
*   // edges with only a source arrow are directed from target to source
*   if (style.sourceArrow !== null) {
*     return -1
*   }
*   return 0
* }
* </pre>
*
* @see OrthogonalLayout#EDGE_DIRECTEDNESS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23edgeDirectedness Online Documentation}
*/
edgeDirectedness: ItemMapping<IEdge, number>;
/**
* Gets or sets the mapping from edges to their crossing cost.
* The crossing costs of the edges are considered during the crossing minimization phase. Edges with lower costs are more
* likely to have crossings. A crossing with an edge that has a cost of zero is cost-free and such a crossing will
* therefore not be avoided. The default crossing cost of an edge is
* <code>1</code>, which is used for edges which do not have an individual crossing cost.
*
* <b>Warning:</b>
* The implemented crossing minimization is a heuristic. Therefore, there is no guarantee that crossing a certain edge is
* actually avoided, even though it has high crossing costs.
*
* <b>Warning:</b>
* Avoid using very large cost values. Due to the fact that costs need to be multiplied and summed up, this might lead to
* overflows and undesired results.
*
* When there are only a few edges to customize crossing costs for, the easiest way is usually to use the {@link ItemMapping#mapper}:
*
* <br><b>Defining edge crossing costs via a mapper</b>
* <pre>
* // Try harder to prevent crossings with edge1
* layoutData.edgeCrossingCosts.mapper.set(edge1, 1.5)
* // Crossings with edge2 are not as bad
* layoutData.edgeCrossingCosts.mapper.set(edge2, 0.2)
* </pre>
*
* If the crossing cost can readily be computed from the edge itself, the {@link ItemMapping#delegate} is often the more convenient option:
*
* <br><b>Defining edge crossing costs via a delegate</b>
* <pre>
* // The more labels a edge has, the more important it is to
* // prevent crossing with it
* layoutData.edgeCrossingCosts.delegate = edge => 1 + edge.labels.size
* </pre>
*
* @see OrthogonalLayout#EDGE_CROSSING_COST_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23edgeCrossingCosts Online Documentation}
*/
edgeCrossingCosts: ItemMapping<IEdge, number>;
/**
* Gets or sets the mapping from edges to their bend cost.
* The bends costs of the edges are considered during the bend minimization phase. Edges with lower costs are more likely
* to have bends. The default bend cost of an edge is
* <code>1</code>, which is used for edges which do not have an individual bend cost.
*
* <b>Warning:</b> The implemented bend minimization is a heuristic and, thus, does not guarantee an optimal solution.
*
* <b>Warning:</b>
* Avoid using very large cost values. Due to the fact that costs need to be multiplied and summed up, this might lead to
* overflows and undesired results.
*
* When there are only a few edges to customize bend costs for, the easiest way is usually to use the {@link ItemMapping#mapper}:
*
* <br><b>Defining edge bend costs via a mapper</b>
* <pre>
* // Try harder to prevent bends for edge1
* layoutData.edgeBendCosts.mapper.set(edge1, 1.5)
* // Bends for edge2 are not as bad
* layoutData.edgeBendCosts.mapper.set(edge2, 0.2)
* </pre>
*
* If the bend cost can readily be computed from the edge itself, the {@link ItemMapping#delegate} is often the more convenient option:
*
* <br><b>Defining edge bend costs via a delegate</b>
* <pre>
* // The more labels a edge has, the more important it is to
* // prevent bends
* layoutData.edgeBendCosts.delegate = edge => 1 + edge.labels.size
* </pre>
*
* @see OrthogonalLayout#EDGE_BEND_COST_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23edgeBendCosts Online Documentation}
*/
edgeBendCosts: ItemMapping<IEdge, number>;
/**
* Gets or sets a mapping from edges to an object representing their source edge group.
* Edges can be grouped so that they share common segments at the beginning or end of their routes. Although a graph may
* contain source and target grouped edges, an edge can only be part of either a source or a target group.
*
* One simple way to use source groups is to use the edge's source node as group ID which effectively groups all edges with
* the same source together:
*
* <br><b>Grouping all edges on the source side</b>
* <pre>
* layoutData.sourceGroupIds = edge => edge.sourceNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.sourceGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.sourceGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see OrthogonalLayoutData#targetGroupIds
* @see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23sourceGroupIds Online Documentation}
*/
sourceGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to an object representing their target edge group.
* Edges can be grouped so that they share common segments at the beginning or end of their routes. Although a graph may
* contain source and target grouped edges, an edge can only be part of either a source or a target group.
*
* One simple way to use source groups is to use the edge's target node as group ID which effectively groups all edges with
* the same target together:
*
* <br><b>Grouping all edges on the target side</b>
* <pre>
* layoutData.targetGroupIds = edge => edge.targetNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.targetGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.targetGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see OrthogonalLayoutData#sourceGroupIds
* @see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23targetGroupIds Online Documentation}
*/
targetGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
* Gets or sets the mapping that provides a {@link PreferredPlacementDescriptor} instance for edge
* {@link ILabel}s.
* Depending on how much customization is needed, some ways of setting {@link PreferredPlacementDescriptor}s are more
* convenient than others. For example, to set the same descriptor for all labels, you can just use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same PreferredPlacementDescriptor for all labels</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.constant = new PreferredPlacementDescriptor({
*   // Place labels along the edge
*   angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*   angle: 0,
*   // ... on either side
*   sideOfEdge: LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE,
*   // ... with a bit of distance to the edge
*   distanceToEdge: 5
* })
* </pre>
*
* If some labels should use custom placement or this has to be configured ahead of time, you can use the {@link ItemMapping#mapper} instead:
*
* <br><b>Customizing the placement of certain labels individually via a mapper</b>
* <pre>
* // Place label1 orthogonal to the edge anywhere on it
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label1,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.ANYWHERE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: Math.PI / 2
*   })
* )
* // Place label2 near the edge's source on either side of it, and make it parallel to the edge
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label2,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.AT_SOURCE,
*     sideOfEdge: LabelPlacements.RIGHT_OF_EDGE | LabelPlacements.LEFT_OF_EDGE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: 0
*   })
* )
* </pre>
*
* When the preferred placement can be inferred from the label itself, a delegate is usually the easiest choice:
*
* <br><b>Configuring preferred placement for all labels with a delegate</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.delegate = label => {
*   const customData = label.tag
*   const preferredPlacementDescriptor = new PreferredPlacementDescriptor({
*     angle: 0,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     // If the tag says to place the label in the center, put it in the center parallel to the edge's path
*     // All other labels can be placed anywhere, but on the side of the edge.
*     placeAlongEdge: customData.placeInCenter
*       ? LabelPlacements.AT_CENTER
*       : LabelPlacements.ANYWHERE,
*     sideOfEdge: customData.placeInCenter
*       ? LabelPlacements.ON_EDGE
*       : LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE
*   })
* }
* </pre>
*
* @see PreferredPlacementDescriptor
* @see LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalLayoutData%23edgeLabelPreferredPlacement Online Documentation}
*/
edgeLabelPreferredPlacement: ItemMapping<ILabel, PreferredPlacementDescriptor>;
static $class: Class;
private $_OrthogonalLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalLayoutData;
}
export interface OrthogonalPatternEdgeRouterData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link OrthogonalPatternEdgeRouter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouterData Online Documentation}*/
export class OrthogonalPatternEdgeRouterData {
/**
Creates a new instance of {@link OrthogonalPatternEdgeRouterData} which helps configuring
{@link OrthogonalPatternEdgeRouter}.
@param [options] The parameters to pass.
@param [options.affectedEdges] The collection of edges affected by this router. This option sets the {@link OrthogonalPatternEdgeRouterData#affectedEdges} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link OrthogonalPatternEdgeRouterData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link OrthogonalPatternEdgeRouterData#targetPortConstraints} property on the created object.
@param [options.sourcePortCandidates] A mapping from edges to a collection of their source port {@link PortCandidate candidates}. This option sets the {@link OrthogonalPatternEdgeRouterData#sourcePortCandidates} property on the created object.
@param [options.targetPortCandidates] A mapping from edges to a collection of their target port {@link PortCandidate candidates}. This option sets the {@link OrthogonalPatternEdgeRouterData#targetPortCandidates} property on the created object.
@param [options.nodePortCandidateSets] A mapping from nodes to their {@link PortCandidateSet}. This option sets the {@link OrthogonalPatternEdgeRouterData#nodePortCandidateSets} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link OrthogonalPatternEdgeRouterData#abortHandler} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouterData Online Documentation}*/
constructor(options?: {
affectedEdges?: DpKeyItemCollection<IEdge> | DpKeyItemCollectionConvertible<IEdge>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
sourcePortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
targetPortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
nodePortCandidateSets?: ItemMapping<INode, PortCandidateSet> | ItemMappingConvertible<INode, PortCandidateSet>;
abortHandler?: AbortHandler | null;
});
/**
Gets or sets the collection of edges affected by this router.
@see OrthogonalPatternEdgeRouter#affectedEdgesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouterData%23affectedEdges Online Documentation}*/
affectedEdges: DpKeyItemCollection<IEdge>;
/**
Gets or sets a mapping from edges to their source {@link PortConstraint}.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouterData%23sourcePortConstraints Online Documentation}*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to their target {@link PortConstraint}.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouterData%23targetPortConstraints Online Documentation}*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to a collection of their source port {@link PortCandidate candidates}.
@see PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouterData%23sourcePortCandidates Online Documentation}*/
sourcePortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from edges to a collection of their target port {@link PortCandidate candidates}.
@see PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouterData%23targetPortCandidates Online Documentation}*/
targetPortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from nodes to their {@link PortCandidateSet}.
@see PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouterData%23nodePortCandidateSets Online Documentation}*/
nodePortCandidateSets: ItemMapping<INode, PortCandidateSet>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/OrthogonalPatternEdgeRouterData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
static $class: Class;
private $_OrthogonalPatternEdgeRouterData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is OrthogonalPatternEdgeRouterData;
}
export interface ParallelEdgeRouterData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link ParallelEdgeRouter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouterData Online Documentation}*/
export class ParallelEdgeRouterData {
/**
Creates a new instance of {@link ParallelEdgeRouterData} which helps configuring
{@link ParallelEdgeRouter}.
@param [options] The parameters to pass.
@param [options.affectedEdges] The collection of edges affected by this router. This option sets the {@link ParallelEdgeRouterData#affectedEdges} property on the created object.
@param [options.leadingEdges] The collection of leading edges. This option sets the {@link ParallelEdgeRouterData#leadingEdges} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link ParallelEdgeRouterData#abortHandler} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouterData Online Documentation}*/
constructor(options?: {
affectedEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
leadingEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
abortHandler?: AbortHandler | null;
});
/**
Gets or sets the collection of edges affected by this router.
@see ParallelEdgeRouter#AFFECTED_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouterData%23affectedEdges Online Documentation}*/
affectedEdges: ItemCollection<IEdge>;
/**
Gets or sets the collection of leading edges.
A leading edge is the edge whose layout is used to obtain the layout of the corresponding parallel edges.
If no leading edge is specified for a bundle of parallel edges, it will be determined automatically. If there is more
than one leading edge, the first one is chosen.
@see ParallelEdgeRouter#LEADING_EDGE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouterData%23leadingEdges Online Documentation}*/
leadingEdges: ItemCollection<IEdge>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/ParallelEdgeRouterData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
static $class: Class;
private $_ParallelEdgeRouterData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ParallelEdgeRouterData;
}
export interface PartialLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link PartialLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData Online Documentation}*/
export class PartialLayoutData {
/**
Creates a new instance of {@link PartialLayoutData} which helps configuring {@link PartialLayout}.
@param [options] The parameters to pass.
@param [options.affectedNodes] The collection of nodes placed by the layout. This option sets the {@link PartialLayoutData#affectedNodes} property on the created object.
@param [options.affectedEdges] The collection of edges placed by the layout. This option sets the {@link PartialLayoutData#affectedEdges} property on the created object.
@param [options.directedEdges] The collection of edges that are considered as directed by the layout. This option sets the {@link PartialLayoutData#directedEdges} property on the created object.
@param [options.componentIds] The mapping from partial nodes to an object defining their component assignment. This option sets the {@link PartialLayoutData#componentIds} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link PartialLayoutData#nodeHalos} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link PartialLayoutData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link PartialLayoutData#targetPortConstraints} property on the created object.
@param [options.sourcePortCandidates] A mapping from edges to a collection of their source port {@link PortCandidate candidates}. This option sets the {@link PartialLayoutData#sourcePortCandidates} property on the created object.
@param [options.targetPortCandidates] A mapping from edges to a collection of their target port {@link PortCandidate candidates}. This option sets the {@link PartialLayoutData#targetPortCandidates} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link PartialLayoutData#abortHandler} property on the created object.
@param [options.partitionGridData] The {@link PartitionGrid partition grid} layout data. This option sets the {@link PartialLayoutData#partitionGridData} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData Online Documentation}*/
constructor(options?: {
affectedNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
affectedEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
directedEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
componentIds?: ItemMapping<INode, any> | ItemMappingConvertible<INode, any>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
sourcePortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
targetPortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
abortHandler?: AbortHandler | null;
partitionGridData?: PartitionGridData;
});
/**
Gets or sets the collection of nodes placed by the layout.
@see PartialLayout#AFFECTED_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23affectedNodes Online Documentation}*/
affectedNodes: ItemCollection<INode>;
/**
Gets or sets the collection of edges placed by the layout.
@see PartialLayout#AFFECTED_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23affectedEdges Online Documentation}*/
affectedEdges: ItemCollection<IEdge>;
/**
Gets or sets the collection of edges that are considered as directed by the layout.
Only if the {@link PartialLayout#layoutOrientation} is not {@link PartialLayoutOrientation#NONE} the directedness of edges has an effect.
@see PartialLayout#DIRECTED_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23directedEdges Online Documentation}*/
directedEdges: ItemCollection<IEdge>;
/**
Gets or sets the mapping from partial nodes to an object defining their component assignment.
All partial nodes associated with the same object are assigned to the same subgraph component.
<h2>Note</h2>
The associated {@link IDataProvider} is only considered if {@link PartialLayout#componentAssignmentStrategy} is set to
{@link ComponentAssignmentStrategy#CUSTOMIZED}.
<b>Warning:</b> Nodes of a subgraph component cannot be assigned to different group nodes or different partition cells.
@see PartialLayout#COMPONENT_ID_DP_KEY
@see ComponentAssignmentStrategy#CUSTOMIZED
@see PartialLayout#componentAssignmentStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23componentIds Online Documentation}*/
componentIds: ItemMapping<INode, any>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
Gets or sets a mapping from edges to their source {@link PortConstraint}.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23sourcePortConstraints Online Documentation}*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to their target {@link PortConstraint}.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23targetPortConstraints Online Documentation}*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to a collection of their source port {@link PortCandidate candidates}.
@see PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23sourcePortCandidates Online Documentation}*/
sourcePortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from edges to a collection of their target port {@link PortCandidate candidates}.
@see PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23targetPortCandidates Online Documentation}*/
targetPortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
Gets or sets the {@link PartitionGrid partition grid} layout data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartialLayoutData%23partitionGridData Online Documentation}*/
partitionGridData: PartitionGridData;
static $class: Class;
private $_PartialLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartialLayoutData;
}
export interface PartitionGridData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies a {@link PartitionGrid} for a layout.
* The {@link PartitionCellId} mapping has to be specified by either setting the {@link PartitionGridData#cellIds} property, or either or both of the
* {@link PartitionGridData#rowIndices} and {@link PartitionGridData#columnIndices} properties. When {@link PartitionGridData#cellIds} is set, either the {@link PartitionGridData#grid} property has to be set as well or a {@link LayoutExecutor} with {@link LayoutExecutor#configureTableLayout automatic table configuration}
* has to be used. Otherwise when {@link PartitionGridData#rowIndices} and {@link PartitionGridData#columnIndices} are used, the size of the grid may be induced by the maximum returned row and
* column indices. The size is then used to extend the {@link PartitionGridData#grid} if necessary or create a new {@link PartitionGrid} if {@link PartitionGridData#grid} is not
* specified.
*
* Note that cell spanning is only supported using the {@link PartitionGridData#cellIds}.
*
* The following sample shows how to assign nodes to partition grid cells simply via cell indices:
*
* <br><b>Assigning nodes to partition grid cells via indices</b>
* <pre>
* // Create four nodes and place them in grid cells in the following way
* // +---+---+---+
* // | 1 | 2 | 3 |
* // +---+---+---+
* //     | 4 |
* //     +---+
*
* const gridData = new PartitionGridData()
*
* const node1 = graph.createNode()
* const node2 = graph.createNode()
* const node3 = graph.createNode()
* const node4 = graph.createNode()
*
* // Assign the nodes to their rows and columns.
* // Note that don't have to create or use PartitionGrid directly in this case.
* // Setting the indices is enough.
* gridData.rowIndices.mapper.set(node1, 0)
* gridData.rowIndices.mapper.set(node2, 0)
* gridData.rowIndices.mapper.set(node3, 0)
* gridData.rowIndices.mapper.set(node4, 1)
* gridData.columnIndices.mapper.set(node1, 0)
* gridData.columnIndices.mapper.set(node2, 1)
* gridData.columnIndices.mapper.set(node3, 2)
* gridData.columnIndices.mapper.set(node4, 1)
*
* // Run a layout with the PartitionGridData. Note that the layout-specific
* // LayoutData classes that support partition grids (e.g. HierarchicLayoutData)
* // also contain a PartitionGridData that can be used directly.
* graph.applyLayout(new HierarchicLayout(), gridData)
* </pre>
*
* When used this way there is no need to create a {@link PartitionGrid} instance or work with it directly. For more
* flexibility, e.g. to use cells that span multiple columns or rows, the {@link PartitionGrid} can be used as well:
*
* <br><b>Assigning nodes to partition grid cells via factory methods on the grid</b>
* <pre>
* // Create three nodes and place them in grid cells in the following way
* // +---+---+
* // | 1 | 2 |
* // +---+---+
* // |   3   |
* // +-------+
*
* const node1 = graph.createNode()
* const node2 = graph.createNode()
* const node3 = graph.createNode()
* const node4 = graph.createNode()
*
* // Assign the nodes to their rows and columns.
* // Note that don't have to create or use PartitionGrid directly in this case.
* // Setting the indices is enough.
* const rowIndicesMapper = new Mapper()
* rowIndicesMapper.set(node1, 0)
* rowIndicesMapper.set(node2, 0)
* rowIndicesMapper.set(node3, 0)
* rowIndicesMapper.set(node4, 1)
* const columnIndicesMapper = new Mapper()
* columnIndicesMapper.set(node1, 0)
* columnIndicesMapper.set(node2, 1)
* columnIndicesMapper.set(node3, 2)
* columnIndicesMapper.set(node4, 1)
*
* const gridData = new PartitionGridData({
*   rowIndices: rowIndicesMapper,
*   columnIndices: columnIndicesMapper
* })
*
* // Run a layout with the PartitionGridData. Note that the layout-specific
* // LayoutData classes that support partition grids (e.g. HierarchicLayoutData)
* // also contain a PartitionGridData that can be used directly.
* graph.applyLayout(new HierarchicLayout(), gridData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridData Online Documentation}
*/
export class PartitionGridData {
/**
Creates a new instance of {@link PartitionGridData} which helps configuring {@link PartitionGrid}.
@param [options] The parameters to pass.
@param [options.grid] The {@link PartitionGrid partition grid}. This option sets the {@link PartitionGridData#grid} property on the created object.
@param [options.cellIds] The mapping from nodes to {@link PartitionCellId} using the partition grid as context. This option sets the {@link PartitionGridData#cellIds} property on the created object.
@param [options.rowIndices] The mapping from a node to its row index. This option sets the {@link PartitionGridData#rowIndices} property on the created object.
@param [options.columnIndices] The mapping from a node to its column index. This option sets the {@link PartitionGridData#columnIndices} property on the created object.
@param [options.optimizeRowOrder] A value indicating whether or not the order of the rows should be chosen automatically to minimize edge lengths. This option sets the {@link PartitionGridData#optimizeRowOrder} property on the created object.
@param [options.optimizeColumnOrder] A value indicating whether or not the order of the columns should be chosen automatically to minimize edge lengths. This option sets the {@link PartitionGridData#optimizeColumnOrder} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridData Online Documentation}*/
constructor(options?: {
grid?: PartitionGrid;
cellIds?: ContextItemMapping<INode, PartitionCellId, PartitionGrid> | ContextItemMappingConvertible<INode, PartitionCellId, PartitionGrid>;
rowIndices?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
columnIndices?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
optimizeRowOrder?: boolean;
optimizeColumnOrder?: boolean;
});
/**
* Gets or sets the {@link PartitionGrid partition grid}.
* When the {@link LayoutExecutor#tableLayoutConfigurator} is {@link LayoutExecutor#configureTableLayout used}, this property should be ignored as it overrides the configurator's own grid which makes the {@link PartitionCellId cell ids}
* created by the configurator invalid.
*
* When creating a custom {@link PartitionGrid} this property can be set and subsequently used in {@link PartitionGridData#cellIds}:
*
* <pre>
* const gridData = new PartitionGridData({
*   grid: new PartitionGrid(2, 2)
* })
* </pre>
*
* The {@link PartitionGrid} from this property can also be modified and subsequently used. It will be created on
* first access to the property:
*
* <pre>
* const row1 = gridData.grid.addRow()
* const row2 = gridData.grid.addRow()
* const column1 = gridData.grid.addColumn()
* const column2 = gridData.grid.addColumn()
* </pre>
*
* @see PartitionGrid#PARTITION_GRID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridData%23grid Online Documentation}
*/
grid: PartitionGrid;
/**
* Gets or sets the mapping from nodes to {@link PartitionCellId} using the partition grid as context.
* A node is placed inside the columns/rows defined by the corresponding {@link PartitionCellId} identifier.
* Instances can be shared among multiple nodes, but don't have to be shared. Multi-cell identifiers (i.e., an identifier
* that represents multiple columns/rows) are not allowed to overlap each other.
*
* In case every node has a specific single cell, there's also the possibility to set {@link PartitionGridData#rowIndices row} and {@link PartitionGridData#columnIndices column indices} which will
* configure the {@link PartitionGridData#grid partition grid} based on the observed indices. This may be easier in some cases.
*
* There are different ways of defining the {@link PartitionCellId}s via this property, depending on which option is
* the most convenient to work with. If the desired {@link PartitionCellId}s can readily be inferred from the nodes
* themselves, then the most convenient option would be the {@link ContextItemMapping#contextDelegate} property:
*
* <pre>
* gridData.cellIds = (node, grid) => {
*   const customData = node.tag
*   return grid.createCellId(customData.row, customData.column)
* }
* </pre>
*
* This is mostly equivalent to the following usage of the {@link ItemMapping#delegate} property:
*
* <pre>
* const grid = gridData.grid
* gridData.cellIds = node => {
*   const customData = node.tag
*   return grid.createCellId(customData.row, customData.column)
* }
* </pre>
*
* The differences are that the {@link ContextItemMapping#contextDelegate} also works correctly when the {@link PartitionGrid} comes pre-configured from
* another source, such as {@link LayoutExecutor#tableLayoutConfigurator} and is not always the same instance as {@link PartitionGridData#grid}.
*
* When setting {@link PartitionCellId}s for only some nodes manually it's often easier to use the {@link ItemMapping#mapper} property:
*
* <pre>
* const grid = gridData.grid
* gridData.cellIds.mapper.set(node1, grid.createCellId(0, 0))
* gridData.cellIds.mapper.set(node2, grid.createCellId(2, 0))
* </pre>
*
* @see PartitionGridData#rowIndices
* @see PartitionGridData#columnIndices
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridData%23cellIds Online Documentation}
*/
cellIds: ContextItemMapping<INode, PartitionCellId, PartitionGrid>;
/**
* Gets or sets the mapping from a node to its row index.
* When using {@link PartitionGridData#rowIndices} or {@link PartitionGridData#columnIndices}, the {@link PartitionGridData#grid partition grid} is configured automatically with the required number of rows and columns.
*
* If nodes must be placed so that they span multiple cells in the partition grid, then {@link PartitionGridData#cellIds} must be used and the {@link PartitionGridData#grid partition grid}
* prepared accordingly.
*
* When the row and column indices can be readily inferred from the nodes, then the easiest option is usually to use the {@link ItemMapping#delegate}:
*
* <pre>
* gridData.rowIndices = node => node.tag.row
* gridData.columnIndices = node => node.tag.column
* </pre>
*
* In case only some nodes need to be mapped to their respective row/column indices, it's sometimes easier to use the {@link ItemMapping#mapper}:
*
* <pre>
* gridData.rowIndices.mapper.set(node1, 0)
* gridData.columnIndices.mapper.set(node1, 1)
* gridData.rowIndices.mapper.set(node2, 1)
* gridData.columnIndices.mapper.set(node2, 1)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridData%23rowIndices Online Documentation}
*/
rowIndices: ItemMapping<INode, number>;
/**
* Gets or sets the mapping from a node to its column index.
* When using {@link PartitionGridData#rowIndices} or {@link PartitionGridData#columnIndices}, the {@link PartitionGridData#grid partition grid} is configured automatically with the required number of rows and columns.
*
* If nodes must be placed so that they span multiple cells in the partition grid, then {@link PartitionGridData#cellIds} must be used and the {@link PartitionGridData#grid partition grid}
* prepared accordingly.
*
* When the row and column indices can be readily inferred from the nodes, then the easiest option is usually to use the {@link ItemMapping#delegate}:
*
* <pre>
* gridData.rowIndices = node => node.tag.row
* gridData.columnIndices = node => node.tag.column
* </pre>
*
* In case only some nodes need to be mapped to their respective row/column indices, it's sometimes easier to use the {@link ItemMapping#mapper}:
*
* <pre>
* gridData.rowIndices.mapper.set(node1, 0)
* gridData.columnIndices.mapper.set(node1, 1)
* gridData.rowIndices.mapper.set(node2, 1)
* gridData.columnIndices.mapper.set(node2, 1)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridData%23columnIndices Online Documentation}
*/
columnIndices: ItemMapping<INode, number>;
/**
Gets or sets a value indicating whether or not the order of the rows should be chosen automatically to minimize edge
lengths.
For all rows where {@link RowDescriptor#indexFixed} is set to
<code>true</code>, the relative ordering given by the indices is preserved. The remaining rows may be sorted again so that the
overall edge lengths are minimized.
If explicitly set, this value is propagated to the {@link PartitionGrid}'s {@link PartitionGrid#optimizeRowOrder} property.
<b>Warning:</b>
If at least one node is mapped to a multi-cell (i.e., a cell spanning multiple columns/rows), enabling this option has
no effect.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridData%23optimizeRowOrder Online Documentation}*/
optimizeRowOrder: boolean;
/**
Gets or sets a value indicating whether or not the order of the columns should be chosen automatically to minimize edge
lengths.
For all columns where {@link ColumnDescriptor#indexFixed} is set to
<code>true</code>, the relative ordering given by the indices is preserved. The remaining columns may be sorted again so that the
overall edge lengths are minimized.
If explicitly set, this value is propagated to the {@link PartitionGrid}'s {@link PartitionGrid#optimizeColumnOrder} property.
<b>Warning:</b>
If at least one node is mapped to a multi-cell (i.e., a cell spanning multiple columns/rows), enabling this option has
no effect.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionGridData%23optimizeColumnOrder Online Documentation}*/
optimizeColumnOrder: boolean;
static $class: Class;
private $_PartitionGridData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionGridData;
}
export interface PartitionLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link PartitionLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayoutData Online Documentation}*/
export class PartitionLayoutData {
/**
Creates a new instance of {@link PartitionLayoutData} which helps configuring {@link PartitionLayout}.
@param [options] The parameters to pass.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link PartitionLayoutData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link PartitionLayoutData#targetPortConstraints} property on the created object.
@param [options.sourcePortCandidates] A mapping from edges to a collection of their source port {@link PortCandidate candidates}. This option sets the {@link PartitionLayoutData#sourcePortCandidates} property on the created object.
@param [options.targetPortCandidates] A mapping from edges to a collection of their target port {@link PortCandidate candidates}. This option sets the {@link PartitionLayoutData#targetPortCandidates} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayoutData Online Documentation}*/
constructor(options?: {
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
sourcePortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
targetPortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
});
/**
Gets or sets a mapping from edges to their source {@link PortConstraint}.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayoutData%23sourcePortConstraints Online Documentation}*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to their target {@link PortConstraint}.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayoutData%23targetPortConstraints Online Documentation}*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to a collection of their source port {@link PortCandidate candidates}.
@see PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayoutData%23sourcePortCandidates Online Documentation}*/
sourcePortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from edges to a collection of their target port {@link PortCandidate candidates}.
@see PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PartitionLayoutData%23targetPortCandidates Online Documentation}*/
targetPortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
static $class: Class;
private $_PartitionLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PartitionLayoutData;
}
export interface PlaceNodesAtBarycenterStageData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link PlaceNodesAtBarycenterStage}.
* The following example shows how to create a new instance of {@link PlaceNodesAtBarycenterStageData} and use it
* with a {@link PlaceNodesAtBarycenterStage}:
*
* <br><b>Creating an instance of PlaceNodesAtBarycenterStageData</b>
* <pre>
* const layoutData = new PlaceNodesAtBarycenterStageData({
*   affectedNodes: graphComponent.selection.selectedNodes
* })
*
* // Combine the layout data of the stage and the core layout
* const compositeLayoutData = new CompositeLayoutData(coreLayoutData, layoutData)
*
* graphComponent.graph.applyLayout(
*   new PlaceNodesAtBarycenterStage(coreLayout),
*   compositeLayoutData
* )
* </pre>
*
* In many cases the complete initialization of {@link PlaceNodesAtBarycenterStageData} can also be done in a single
* object initializer:
*
* <br><b>Using an object initializer for easy creation of PlaceNodesAtBarycenterStageData</b>
* <pre>
* const layoutData = new PlaceNodesAtBarycenterStageData({
*   affectedNodes: graphComponent.selection.selectedNodes
* })
*
* graphComponent.graph.applyLayout(
*   new PlaceNodesAtBarycenterStage(coreLayout),
*   new CompositeLayoutData(coreLayoutData, layoutData)
* )
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStageData Online Documentation}
*/
export class PlaceNodesAtBarycenterStageData {
/**
Creates a new instance of {@link PlaceNodesAtBarycenterStageData} which helps configuring
{@link PlaceNodesAtBarycenterStage}.
@param [options] The parameters to pass.
@param [options.affectedNodes] The collection of nodes that should be affected by the {@link PlaceNodesAtBarycenterStage}. This option sets the {@link PlaceNodesAtBarycenterStageData#affectedNodes} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStageData Online Documentation}*/
constructor(options?: {
affectedNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
});
/**
* Gets or sets the collection of nodes that should be affected by the {@link PlaceNodesAtBarycenterStage}.
* These nodes are temporarily hidden, then the core layout algorithm (if any) is applied to the resulting graph, and,
* finally, the affected nodes are reinserted by placing them on the barycenter of their neighbors.
*
* Defining the subset of nodes that should be temporarily hidden can be done in various ways, mostly depending on which
* option is more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of affected nodes</b>
* <pre>
* layoutData.affectedNodes.source = graphComponent.selection.selectedNodes
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual nodes as affected nodes</b>
* <pre>
* for (const /** @type {IEdge} *‍/ edge of graphComponent.selection.selectedEdges) {
*   layoutData.affectedNodes.items.add(edge.sourceNode)
*   layoutData.affectedNodes.items.add(edge.targetNode)
* }
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every node
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether a node is affected or not</b>
* <pre>
* // Temporarily hide nodes with labels
* layoutData.affectedNodes.delegate = node => node.labels.size > 0
* </pre>
*
* @see PlaceNodesAtBarycenterStage#AFFECTED_NODES_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStageData%23affectedNodes Online Documentation}
*/
affectedNodes: ItemCollection<INode>;
/**
Gets or sets the mapping from the affected nodes to their sizes.
The dimension of the affected nodes will be changed to the specified sizes after applying the core layout algorithm.
@see PlaceNodesAtBarycenterStage#AFFECTED_NODES_SIZE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PlaceNodesAtBarycenterStageData%23affectedNodesSizes Online Documentation}*/
affectedNodesSizes: ItemMapping<INode, ISize>;
static $class: Class;
private $_PlaceNodesAtBarycenterStageData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PlaceNodesAtBarycenterStageData;
}
export interface PolylineEdgeRouterData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link EdgeRouter}.
* The following example shows how to create a new instance of {@link PolylineEdgeRouterData} and use it with an
* {@link EdgeRouter}:
*
* <br><b>Creating an instance of PolylineEdgeRouterData</b>
* <pre>
* const layoutData = new PolylineEdgeRouterData()
*
* // group edges by color on their source side
* layoutData.sourceGroupIds.delegate = edge => edge.style.stroke.fill
* // route only the selected edges
* layoutData.affectedEdges.source = graphComponent.selection.selectedEdges
*
* const edgeRouter = new EdgeRouter()
* edgeRouter.scope = EdgeRouterScope.ROUTE_AFFECTED_EDGES
*
* graphComponent.graph.applyLayout(edgeRouter, layoutData)
* </pre>
*
* In many cases the complete initialization of {@link PolylineEdgeRouterData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of PolylineEdgeRouterData</b>
* <pre>
* const layoutData = new PolylineEdgeRouterData({
*   // group edges by color on their source side
*   sourceGroupIds: edge => edge.style.stroke.fill,
*   // route only the selected edges
*   affectedEdges: graphComponent.selection.selectedEdges
* })
*
* const edgeRouter = new EdgeRouter()
* edgeRouter.scope = EdgeRouterScope.ROUTE_AFFECTED_EDGES
*
* graphComponent.graph.applyLayout(edgeRouter, layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData Online Documentation}
*/
export class PolylineEdgeRouterData {
/**
Creates a new instance of {@link PolylineEdgeRouterData} which helps configuring {@link EdgeRouter}.
@param [options] The parameters to pass.
@param [options.affectedEdges] The collection of affected edges. This option sets the {@link PolylineEdgeRouterData#affectedEdges} property on the created object.
@param [options.affectedNodes] The collection of affected nodes. This option sets the {@link PolylineEdgeRouterData#affectedNodes} property on the created object.
@param [options.ignoredLabels] The collection of labels of nodes or fixed edges that are ignored by the router. This option sets the {@link PolylineEdgeRouterData#ignoredLabels} property on the created object.
@param [options.edgeLayoutDescriptors] The mapping of edges to their {@link EdgeRouterEdgeLayoutDescriptor} This option sets the {@link PolylineEdgeRouterData#edgeLayoutDescriptors} property on the created object.
@param [options.labelCrossingPenaltyFactors] A mapping from labels to a crossing penalty factor. This option sets the {@link PolylineEdgeRouterData#labelCrossingPenaltyFactors} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link PolylineEdgeRouterData#nodeHalos} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link PolylineEdgeRouterData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link PolylineEdgeRouterData#targetPortConstraints} property on the created object.
@param [options.sourcePortCandidates] A mapping from edges to a collection of their source port {@link PortCandidate candidates}. This option sets the {@link PolylineEdgeRouterData#sourcePortCandidates} property on the created object.
@param [options.targetPortCandidates] A mapping from edges to a collection of their target port {@link PortCandidate candidates}. This option sets the {@link PolylineEdgeRouterData#targetPortCandidates} property on the created object.
@param [options.nodePortCandidateSets] A mapping from nodes to their {@link PortCandidateSet}. This option sets the {@link PolylineEdgeRouterData#nodePortCandidateSets} property on the created object.
@param [options.sourceGroupIds] A mapping from edges to an object representing their source edge group. This option sets the {@link PolylineEdgeRouterData#sourceGroupIds} property on the created object.
@param [options.sourcePortGroupIds] A mapping from edges to an object representing their source port group. This option sets the {@link PolylineEdgeRouterData#sourcePortGroupIds} property on the created object.
@param [options.targetGroupIds] A mapping from edges to an object representing their target edge group. This option sets the {@link PolylineEdgeRouterData#targetGroupIds} property on the created object.
@param [options.targetPortGroupIds] A mapping from edges to an object representing their target port group. This option sets the {@link PolylineEdgeRouterData#targetPortGroupIds} property on the created object.
@param [options.buses] The mapping from edges to their {@link EdgeRouterBusDescriptor bus descriptor}. This option sets the {@link PolylineEdgeRouterData#buses} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link PolylineEdgeRouterData#abortHandler} property on the created object.
@param [options.partitionGridData] The {@link PartitionGrid partition grid} layout data. This option sets the {@link PolylineEdgeRouterData#partitionGridData} property on the created object.
@param [options.edgeLabelPreferredPlacement] The mapping that provides a {@link PreferredPlacementDescriptor} instance for edge {@link ILabel}s. This option sets the {@link PolylineEdgeRouterData#edgeLabelPreferredPlacement} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData Online Documentation}*/
constructor(options?: {
affectedEdges?: DpKeyItemCollection<IEdge> | DpKeyItemCollectionConvertible<IEdge>;
affectedNodes?: DpKeyItemCollection<INode> | DpKeyItemCollectionConvertible<INode>;
ignoredLabels?: ItemCollection<ILabel> | ItemCollectionConvertible<ILabel>;
edgeLayoutDescriptors?: ItemMapping<IEdge, EdgeRouterEdgeLayoutDescriptor> | ItemMappingConvertible<IEdge, EdgeRouterEdgeLayoutDescriptor>;
labelCrossingPenaltyFactors?: ItemMapping<ILabel, number> | ItemMappingConvertible<ILabel, number>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
sourcePortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
targetPortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
nodePortCandidateSets?: ItemMapping<INode, PortCandidateSet> | ItemMappingConvertible<INode, PortCandidateSet>;
sourceGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
sourcePortGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetPortGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
buses?: ItemCollectionMapping<IEdge, EdgeRouterBusDescriptor>;
abortHandler?: AbortHandler | null;
partitionGridData?: PartitionGridData;
edgeLabelPreferredPlacement?: ItemMapping<ILabel, PreferredPlacementDescriptor> | ItemMappingConvertible<ILabel, PreferredPlacementDescriptor>;
});
/**
* Gets or sets the collection of affected edges.
* This collection is only used by the router if the {@link EdgeRouter#scope} is set to {@link EdgeRouterScope#ROUTE_AFFECTED_EDGES}.
*
* Defining the subset of edges that should be laid out can be done in various ways, mostly depending on which option is
* more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of affected edges</b>
* <pre>
* layoutData.affectedEdges.source = graphComponent.selection.selectedEdges
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual affected edges</b>
* <pre>
* layoutData.affectedEdges.items.add(edge1)
* layoutData.affectedEdges.items.add(edge2)
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every edge
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether an edge is affected or not</b>
* <pre>
* // We assume here that all edges have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.affectedEdges.delegate = edge => edge.tag.includeInLayout
* </pre>
*
* @see EdgeRouter#affectedEdgesDpKey
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23affectedEdges Online Documentation}
*/
affectedEdges: DpKeyItemCollection<IEdge>;
/**
* Gets or sets the collection of affected nodes.
* This collection is only used by the router if the {@link EdgeRouter#scope} is set to {@link EdgeRouterScope#ROUTE_EDGES_AT_AFFECTED_NODES}.
*
* Defining the subset of nodes that should be laid out can be done in various ways, mostly depending on which option is
* more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of affected nodes</b>
* <pre>
* layoutData.affectedNodes.source = graphComponent.selection.selectedNodes
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual affected nodes</b>
* <pre>
* layoutData.affectedNodes.items.add(node1)
* layoutData.affectedNodes.items.add(node2)
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every node
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether a node is affected or not</b>
* <pre>
* // We assume here that all nodes have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.affectedNodes.delegate = node => node.tag.includeInLayout
* </pre>
*
* @see EdgeRouter#affectedNodesDpKey
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23affectedNodes Online Documentation}
*/
affectedNodes: DpKeyItemCollection<INode>;
/**
* Gets or sets the collection of labels of nodes or fixed edges that are ignored by the router.
* Defining the subset of labels that should be ignored can be done in various ways, mostly depending on which option is
* more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of labels to ignore</b>
* <pre>
* // Ignore all labels of selected nodes
* layoutData.ignoredLabels.source = graphComponent.selection.selectedNodes.flatMap(
*   node => node.labels
* )
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual labels to ignore</b>
* <pre>
* layoutData.ignoredLabels.items.add(label1)
* layoutData.ignoredLabels.items.add(label2)
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every label
* whether it should be ignored or not:
*
* <br><b>Using a delegate to determine whether to ignore a label</b>
* <pre>
* // We assume here that all labels have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.ignoredLabels.delegate = label => !label.tag.includeInLayout
* </pre>
*
* @see LabelLayoutKeys#IGNORED_LABELS_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23ignoredLabels Online Documentation}
*/
ignoredLabels: ItemCollection<ILabel>;
/**
Gets or sets the mapping of edges to their {@link EdgeRouterEdgeLayoutDescriptor}
If an edge is mapped to <code>null</code>, the {@link EdgeRouter#defaultEdgeLayoutDescriptor default descriptor} is used.
@see EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23edgeLayoutDescriptors Online Documentation}*/
edgeLayoutDescriptors: ItemMapping<IEdge, EdgeRouterEdgeLayoutDescriptor>;
/**
* Gets or sets a mapping from labels to a crossing penalty factor.
* The crossing penalty factor of a label is multiplied with the basic penalty for an edge crossing the label so crossing a
* label with a higher cost factor is stronger avoided then crossing one with a lower factor.
*
* When there are only a few labels to customize penalties for, the easiest way is usually to use the {@link ItemMapping#mapper}:
*
* <br><b>Defining label crossing penalties via a mapper</b>
* <pre>
* // Try harder to prevent crossings with label1
* layoutData.labelCrossingPenaltyFactors.mapper.set(label1, 1.5)
* // Crossings with label2 are not as bad
* layoutData.labelCrossingPenaltyFactors.mapper.set(label2, 0.2)
* </pre>
*
* If the penalty can readily be computed from the label itself, the {@link ItemMapping#delegate} is often the more convenient option:
*
* <br><b>Defining label crossing penalties via a delegate</b>
* <pre>
* // The more upper-case letters a label has, the more important it must be
* // and crossings should be based on that metric
* layoutData.labelCrossingPenaltyFactors.delegate = label => label.text.match(/[A-Z]/g).length
* </pre>
*
* @see PenaltySettings#edgeLabelCrossingPenalty
* @see PenaltySettings#nodeLabelCrossingPenalty
* @see EdgeRouter#LABEL_CROSSING_PENALTY_FACTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23labelCrossingPenaltyFactors Online Documentation}
*/
labelCrossingPenaltyFactors: ItemMapping<ILabel, number>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets a mapping from edges to their source {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its source node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* A more general concept which allows finer control over where ports are placed, are {@link PolylineEdgeRouterData#sourcePortCandidates port candidates}.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All source ports should be on the bottom side
* layoutData.sourcePortConstraints.constant = PortConstraint.create(PortSide.SOUTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should leave the source node on the left side, while keeping its exact port position (strong port constraint)
* layoutData.sourcePortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.WEST, true))
* // edge2 should leave the source node downwards, but the port may be placed anywhere along that node border
* layoutData.sourcePortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.SOUTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Source ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.sourcePortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see PolylineEdgeRouterData#sourcePortCandidates
* @see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23sourcePortConstraints Online Documentation}
*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets a mapping from edges to their target {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its target node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* A more general concept which allows finer control over where ports are placed, are {@link PolylineEdgeRouterData#targetPortCandidates port candidates}.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All target ports should be on the top side
* layoutData.targetPortConstraints.constant = PortConstraint.create(PortSide.NORTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should enter the target node on the right side, while keeping its exact port position (strong port
* // constraint)
* layoutData.targetPortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.EAST, true))
* // edge2 should enter the target node from the top, but the port may be placed anywhere along that node border
* layoutData.targetPortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.NORTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Target ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.targetPortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see PolylineEdgeRouterData#targetPortCandidates
* @see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23targetPortConstraints Online Documentation}
*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets a mapping from edges to a collection of their source port {@link PortCandidate candidates}.
* Port constraints allow to define where an edge can connect to its source node and allow fine control over port
* placement.
*
* If all that is needed is to fix the source port in its location or on a node side, {@link PolylineEdgeRouterData#sourcePortConstraints port constraints} are easier to work
* with, since they are a slightly simpler concept.
*
* Source port candidates are effectively a collection of possible port placements with different costs and the layout
* algorithm is free to choose the candidate that fits best into the overall layout, while also preferring candidates with
* a lower cost. To set the same candidate list for all edges, it's easiest to use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same set of source port candidates for all edges</b>
* <pre>
* layoutData.sourcePortCandidates.constant = new List([
*   // Prefer the center position and route the edge downwards if possible (no cost for this candidate)
*   PortCandidate.createCandidate(0, 0, PortDirections.SOUTH),
*   // Alternatively, use any position at the bottom border of the node, but with a higher cost
*   PortCandidate.createCandidate(PortDirections.SOUTH, 1),
*   // If that fails, use either the left or the right side
*   PortCandidate.createCandidate(PortDirections.EAST | PortDirections.WEST, 2)
* ])
* </pre>
*
* If certain edges need specific port candidates, it's usually convenient to use the {@link ItemMapping#mapper} property:
*
* <br><b>Setting different source port candidates for certain edges</b>
* <pre>
* // edge1 should leave the source node preferably on the node side in flow direction,
* // but can also use a fixed port location on the left of the node.
* layoutData.sourcePortCandidates.mapper.set(
*   edge1,
*   new List([
*     PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*     PortCandidate.createCandidate(-0.5, 0.25, PortDirections.WEST, 1)
*   ])
* )
*
* // edge2 should leave the source node anywhere on the upper side of the node
* layoutData.sourcePortCandidates.mapper.set(
*   edge2,
*   new List([PortCandidate.createCandidate(PortDirections.NORTH)])
* )
* </pre>
*
* For cases when the desired configuration of port candidates can be readily created from the edge itself, the {@link ItemMapping#delegate} is often
* the most convenient option:
*
* <br><b>Setting different source port candidates for each edge with a delegate</b>
* <pre>
* // Assuming a flowchart, the "yes" edge should leave the decision in flow direction,
* // and the "no" edge may leave either left or right of the flow direction.
* // This also ensures that the layout still looks natural if the layout uses different orientations.
* layoutData.sourcePortCandidates.delegate = edge => {
*   // Only handle decision nodes here
*   if (edge.sourceNode.tag instanceof FlowChartDecision) {
*     switch (edge.labels.get(0).text) {
*       case 'Yes':
*         return new List([PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW)])
*       case 'No':
*         return new List([
*           PortCandidate.createCandidate(
*             PortDirections.LEFT_IN_FLOW | PortDirections.RIGHT_IN_FLOW
*           )
*         ])
*     }
*   }
*   return null
* }
* </pre>
*
* @see PolylineEdgeRouterData#sourcePortConstraints
* @see PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23sourcePortCandidates Online Documentation}
*/
sourcePortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
* Gets or sets a mapping from edges to a collection of their target port {@link PortCandidate candidates}.
* Port constraints allow to define where an edge can connect to its target node and allow fine control over port
* placement.
*
* If all that is needed is to fix the target port in its location or on a node side, {@link PolylineEdgeRouterData#targetPortConstraints port constraints} are easier to work
* with, since they are a slightly simpler concept.
*
* Target port candidates are effectively a collection of possible port placements with different costs and the layout
* algorithm is free to choose the candidate that fits best into the overall layout, while also preferring candidates with
* a lower cost. To set the same candidate list for all edges, it's easiest to use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same set of target port candidates for all edges</b>
* <pre>
* layoutData.targetPortCandidates.constant = new List([
*   // Prefer the center position and enter the node from the top if possible (no cost for this candidate)
*   PortCandidate.createCandidate(0, 0, PortDirections.NORTH),
*   // Alternatively, use any position at the top border of the node, but with a higher cost
*   PortCandidate.createCandidate(PortDirections.NORTH, 1),
*   // If that fails, use either the left or the right side
*   PortCandidate.createCandidate(PortDirections.EAST | PortDirections.WEST, 2)
* ])
* </pre>
*
* If certain edges need specific port candidates, it's usually convenient to use the {@link ItemMapping#mapper} property:
*
* <br><b>Setting different target port candidates for certain edges</b>
* <pre>
* // edge1 should enter the target node preferably on the node side in flow direction,
* // but can also use a fixed port location on the left of the node.
* layoutData.targetPortCandidates.mapper.set(
*   edge1,
*   new List([
*     PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*     PortCandidate.createCandidate(-0.5, -0.25, PortDirections.WEST, 1)
*   ])
* )
*
* // edge2 should enter the target node anywhere on the bottom side of the node
* layoutData.targetPortCandidates.mapper.set(
*   edge2,
*   new List([PortCandidate.createCandidate(PortDirections.SOUTH)])
* )
* </pre>
*
* For cases when the desired configuration of port candidates can be readily created from the edge itself, the {@link ItemMapping#delegate} is often
* the most convenient option:
*
* <br><b>Setting different target port candidates for each edge with a delegate</b>
* <pre>
* // Assuming a flowchart, edges connecting to a terminal on the target side should enter that node either
* // in flow direction (preferred) or from one of the sides.
* // This also ensures that the layout still looks natural if the layout uses different orientations.
* layoutData.targetPortCandidates.delegate = edge => {
*   // Only handle terminal nodes here
*   if (edge.targetNode.tag instanceof FlowChartTerminal) {
*     return new List([
*       PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*       PortCandidate.createCandidate(PortDirections.LEFT_IN_FLOW | PortDirections.RIGHT_IN_FLOW, 1)
*     ])
*   }
*   return null
* }
* </pre>
*
* @see PolylineEdgeRouterData#targetPortConstraints
* @see PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23targetPortCandidates Online Documentation}
*/
targetPortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from nodes to their {@link PortCandidateSet}.
@see PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23nodePortCandidateSets Online Documentation}*/
nodePortCandidateSets: ItemMapping<INode, PortCandidateSet>;
/**
* Gets or sets a mapping from edges to an object representing their source edge group.
* Edges sharing a source group identifier will share a common bus near the source or at a common source node if possible.
*
* One simple way to use source groups is to use the edge's source node as group ID which effectively groups all edges with
* the same source together:
*
* <br><b>Grouping all edges on the source side</b>
* <pre>
* layoutData.sourceGroupIds = edge => edge.sourceNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.sourceGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.sourceGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see PolylineEdgeRouterData#targetGroupIds
* @see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23sourceGroupIds Online Documentation}
*/
sourceGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to an object representing their source port group.
* All edges with the same port id at a node will share the same port location. However, they will be routed independently
* and in contrast to {@link PolylineEdgeRouterData#sourceGroupIds source groups} will deviate from a common path as soon as possible instead of sharing as much of a
* common path as possible.
*
* The simplest way to use source port groups is to use the same object as group ID for all edges. Since grouping is done
* per node, this has the effect of grouping all edges with the same source node together:
*
* <br><b>Grouping all edges on the source side</b>
* <pre>
* layoutData.sourcePortGroupIds = {}
* </pre>
*
* The same effect can be achieved with a delegate as well, by returning the source node as group ID for each edge.
* However, a more useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.sourcePortGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgePortGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.sourcePortGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see PolylineEdgeRouterData#targetPortGroupIds
* @see PolylineEdgeRouterData#sourceGroupIds
* @see PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23sourcePortGroupIds Online Documentation}
*/
sourcePortGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to an object representing their target edge group.
* Edges sharing a target group identifier will share a common bus near the target or at a common target node if possible.
*
* One simple way to use source groups is to use the edge's target node as group ID which effectively groups all edges with
* the same target together:
*
* <br><b>Grouping all edges on the target side</b>
* <pre>
* layoutData.targetGroupIds = edge => edge.targetNode
* </pre>
*
* Another useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.targetGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgeGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges in it
*     layoutData.targetGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see PolylineEdgeRouterData#sourceGroupIds
* @see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23targetGroupIds Online Documentation}
*/
targetGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to an object representing their target port group.
* All edges with the same port id at a node will share the same port location. However, they will be routed independently
* and in contrast to {@link PolylineEdgeRouterData#sourceGroupIds source groups} will deviate from a common path as soon as possible instead of sharing as much of a
* common path as possible.
*
* The simplest way to use target port groups is to use the same object as group ID for all edges. Since grouping is done
* per node, this has the effect of grouping all edges with the same target node together:
*
* <br><b>Grouping all edges on the target side</b>
* <pre>
* layoutData.targetPortGroupIds = {}
* </pre>
*
* The same effect can be achieved with a delegate as well, by returning the target node as group ID for each edge.
* However, a more useful way to use a delegate here would be grouping edges by some commonality, such as the same color:
*
* <br><b>Grouping edges by color</b>
* <pre>
* layoutData.targetPortGroupIds = edge => {
*   const style = edge.style
*   if (style !== null) {
*     return style.stroke.fill
*   }
*   return null
* }
* </pre>
*
* If only certain edges should be grouped it may sometimes be easier to use the {@link ItemMapping#mapper} to set the group IDs:
*
* <br><b>Grouping certain edges with a mapper</b>
* <pre>
* for (const group of edgePortGroups) {
*   for (const edge of group) {
*     // Use the collection as group ID, since it's common to all edges of it
*     layoutData.targetPortGroupIds.mapper.set(edge, group)
*   }
* }
* </pre>
*
* @see PolylineEdgeRouterData#sourcePortGroupIds
* @see PolylineEdgeRouterData#targetGroupIds
* @see PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23targetPortGroupIds Online Documentation}
*/
targetPortGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets the mapping from edges to their {@link EdgeRouterBusDescriptor bus descriptor}.
* All edges associated to the same bus descriptor are routed in a bus-like style, sharing a common backbone. A bus consist
* of segments that are shared by multiple edges to which shorter segments that connect to the actual nodes are attached
* to. Observe that using such a bus representation with multiple edges drawn on top of each other, information like the
* individual edge direction might be occluded. Bus routing can, for example, be very useful in parts of a diagram where
* each node is connected to each other node.
*
* The {@link EdgeRouterBusDescriptor} instance allows to configure the bus formed by the associated edges. To
* conveniently define buses use the {@link ItemCollectionMapping#add} method of this property. It takes the bus descriptor as parameter and allows to
* define the edges associated to the bus using the returned {@link ItemCollection}.
*
* <h2>Note</h2>
*
* {@link EdgeRouterEdgeLayoutDescriptor#intermediateRoutingPoints Intermediate points} are ignored for edges that are part of a bus.
*
* Assigning a list of edges to a bus
*
* <pre>
* // create a bus descriptor and, optionally, configure it
* const busDescriptor = new EdgeRouterBusDescriptor({
*   minimumBackboneSegmentLength: 200
* })
* // retrieve an ItemCollection which defines the edges that should belong to the bus
* const busEdgesCollection = polylineEdgeRouterData.buses.add(busDescriptor)
* // busEdgesList is an ICollection which contains all edges that should be part of the bus
* busEdgesCollection.items = busEdgesList
* </pre>
*
* @see EdgeRouter#BUS_DESCRIPTOR_DP_KEY
* @see EdgeRouterBusDescriptor
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23buses Online Documentation}
*/
buses: ItemCollectionMapping<IEdge, EdgeRouterBusDescriptor>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
Gets or sets the {@link PartitionGrid partition grid} layout data.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23partitionGridData Online Documentation}*/
partitionGridData: PartitionGridData;
/**
* Gets or sets the mapping that provides a {@link PreferredPlacementDescriptor} instance for edge
* {@link ILabel}s.
* Depending on how much customization is needed, some ways of setting {@link PreferredPlacementDescriptor}s are more
* convenient than others. For example, to set the same descriptor for all labels, you can just use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same PreferredPlacementDescriptor for all labels</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.constant = new PreferredPlacementDescriptor({
*   // Place labels along the edge
*   angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*   angle: 0,
*   // ... on either side
*   sideOfEdge: LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE,
*   // ... with a bit of distance to the edge
*   distanceToEdge: 5
* })
* </pre>
*
* If some labels should use custom placement or this has to be configured ahead of time, you can use the {@link ItemMapping#mapper} instead:
*
* <br><b>Customizing the placement of certain labels individually via a mapper</b>
* <pre>
* // Place label1 orthogonal to the edge anywhere on it
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label1,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.ANYWHERE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: Math.PI / 2
*   })
* )
* // Place label2 near the edge's source on either side of it, and make it parallel to the edge
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label2,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.AT_SOURCE,
*     sideOfEdge: LabelPlacements.RIGHT_OF_EDGE | LabelPlacements.LEFT_OF_EDGE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: 0
*   })
* )
* </pre>
*
* When the preferred placement can be inferred from the label itself, a delegate is usually the easiest choice:
*
* <br><b>Configuring preferred placement for all labels with a delegate</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.delegate = label => {
*   const customData = label.tag
*   const preferredPlacementDescriptor = new PreferredPlacementDescriptor({
*     angle: 0,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     // If the tag says to place the label in the center, put it in the center parallel to the edge's path
*     // All other labels can be placed anywhere, but on the side of the edge.
*     placeAlongEdge: customData.placeInCenter
*       ? LabelPlacements.AT_CENTER
*       : LabelPlacements.ANYWHERE,
*     sideOfEdge: customData.placeInCenter
*       ? LabelPlacements.ON_EDGE
*       : LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE
*   })
* }
* </pre>
*
* @see PreferredPlacementDescriptor
* @see LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/PolylineEdgeRouterData%23edgeLabelPreferredPlacement Online Documentation}
*/
edgeLabelPreferredPlacement: ItemMapping<ILabel, PreferredPlacementDescriptor>;
static $class: Class;
private $_PolylineEdgeRouterData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PolylineEdgeRouterData;
}
/**
Specifies how port locations should be adjusted after a layout has been calculated.
Layout algorithms only consider rectangular nodes even though the actual shape of a node is, for example, circular.
Hence, the ports are usually placed at the border of the nodes' {@link INode#layout bounding boxes} (except for some layout algorithms that
produce straight-line edge routes and place the ports at the nodes' center).
Based on this setting the edges will be shortened or lengthened in a way that their {@link IEdge#sourcePort}s and {@link IEdge#targetPort}s will be placed on the node's
{@link IShapeGeometry#getOutline outline}.
<h2>Note</h2>
The coordinates of a port will not be changed if the port is associated with a strong port constraint.
@see LayoutGraphAdapter#portAdjustmentPolicy
@see LayoutExecutor#portAdjustmentPolicy
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortAdjustmentPolicy Online Documentation}*/
export enum PortAdjustmentPolicy {
/**
Port locations should not be adjusted at all.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortAdjustmentPolicy%23NEVER Online Documentation}*/
NEVER,
/**
Edges at ports are lengthened if their port is {@link IShapeGeometry#isInside outside} of its owners {@link IShapeGeometry#getOutline outline}.
The port is moved along the lengthened edge until it hits the outline. If prolonging the edge wouldn't hit the outline,
the port location is kept.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortAdjustmentPolicy%23LENGTHEN Online Documentation}*/
LENGTHEN,
/**
Edges at ports are shortened if their port is {@link IShapeGeometry#isInside inside} of its owners {@link IShapeGeometry#getOutline outline}.
The port is moved along the edge until it hits the outline.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortAdjustmentPolicy%23SHORTEN Online Documentation}*/
SHORTEN,
/**
Edges at ports are lengthened or shortened if their port is not on its owners {@link IShapeGeometry#getOutline outline}.
The port is moved along the (lengthened) edge until it hits the outline. If starting outside and prolonging the edge
wouldn't hit the outline, the port location is kept.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PortAdjustmentPolicy%23ALWAYS Online Documentation}*/
ALWAYS
}
/** Automatic conversion to {@link PortAdjustmentPolicy}. */
type PortAdjustmentPolicyStringValues = "never" | "lengthen" | "shorten" | "always";
export interface RadialLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link RadialLayout}.
* The following example shows how to create a new instance of {@link RadialLayoutData} and use it with an
* {@link RadialLayout}:
*
* <br><b>Creating an instance of RadialLayoutData</b>
* <pre>
* const layoutData = new RadialLayoutData()
* // use the current selection as center nodes
* layoutData.centerNodes.source = graphComponent.selection.selectedNodes
* // group all edges on their source side
* layoutData.sourceGroups.delegate = edge => edge.sourceNode
* graph.applyLayout(new RadialLayout(), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link RadialLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of RadialLayoutData</b>
* <pre>
* const layoutData = new RadialLayoutData({
*   // use the current selection as center nodes
*   centerNodes: graphComponent.selection.selectedNodes,
*   // group all edges on their source side
*   sourceGroups: edge => edge.sourceNode
* })
*
* graph.applyLayout(new RadialLayout(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutData Online Documentation}
*/
export class RadialLayoutData {
/**
Creates a new instance of {@link RadialLayoutData} which helps configuring {@link RadialLayout}.
@param [options] The parameters to pass.
@param [options.centerNodes] The collection of nodes that shall be placed in the center. This option sets the {@link RadialLayoutData#centerNodes} property on the created object.
@param [options.nodeInfos] The mapping from nodes to a {@link RadialLayoutNodeInfo} describing on which circle and in which sector the node is placed. This option sets the {@link RadialLayoutData#nodeInfos} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link RadialLayoutData#nodeHalos} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link RadialLayoutData#abortHandler} property on the created object.
@param [options.edgeBundleDescriptors] The mapping of edges to their {@link EdgeBundleDescriptor}. This option sets the {@link RadialLayoutData#edgeBundleDescriptors} property on the created object.
@param [options.layerIds] The mapping from nodes to their layer/circle index when using the layering strategy {@link RadialLayoutLayeringStrategy#USER_DEFINED}. This option sets the {@link RadialLayoutData#layerIds} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutData Online Documentation}*/
constructor(options?: {
centerNodes?: DpKeyItemCollection<INode> | DpKeyItemCollectionConvertible<INode>;
nodeInfos?: IMapper<INode, RadialLayoutNodeInfo> | null | IMapperConvertible<INode, RadialLayoutNodeInfo>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
abortHandler?: AbortHandler | null;
edgeBundleDescriptors?: ItemMapping<IEdge, EdgeBundleDescriptor> | ItemMappingConvertible<IEdge, EdgeBundleDescriptor>;
layerIds?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
});
/**
* Gets or sets the collection of nodes that shall be placed in the center.
* This option will only take effect if {@link RadialLayout#centerNodesPolicy center node policy} is set to {@link CenterNodesPolicy#CUSTOM}.
*
* Defining the nodes that should be placed in the center can be done in various ways, mostly depending on which option is
* more convenient for a particular use case. You can use the {@link ItemCollection}'s {@link ItemCollection#source} property to use any .NET collection or {@link IEnumerable}:
*
* <br><b>Setting a collection of center nodes</b>
* <pre>
* layoutData.centerNodes = graphComponent.selection.selectedNodes
* </pre>
*
* Alternatively, {@link ItemCollection} also has an {@link ItemCollection#items} property, which is a collection that already exists, in case the items may have to be added
* one by one. This can be more convenient than defining an own list and setting it to {@link ItemCollection#source}:
*
* <br><b>Adding individual nodes as center nodes</b>
* <pre>
* layoutData.centerNodes.items.add(node1)
* layoutData.centerNodes.items.add(node2)
* </pre>
*
* A powerful option that doesn't use a collection is to use the {@link ItemCollection#delegate} to set a custom delegate that returns for every node
* whether it is contained in the set or not:
*
* <br><b>Using a delegate to determine whether a node should be placed in the center or not</b>
* <pre>
* // We assume here that all nodes have a CustomData instance as their Tag,
* // which then has a boolean property 'IncludeInLayout'.
* layoutData.centerNodes = node => node.tag.placeInCenter
* </pre>
*
* @see RadialLayout#centerNodesDpKey
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutData%23centerNodes Online Documentation}
*/
centerNodes: DpKeyItemCollection<INode>;
/**
* Gets or sets the mapping from nodes to a {@link RadialLayoutNodeInfo} describing on which circle and in which
* sector the node is placed.
* If the mapper contains a {@link RadialLayoutNodeInfo} for a node, the layout will update its property during the
* layout.
*
* The following example shows how to retrieve node placement information from the layout:
*
* <pre>
* const layoutData = new RadialLayoutData()
* layoutData.nodeInfos = new Mapper()
*
* graph.applyLayout(new RadialLayout(), layoutData)
*
* for (const node of graph.nodes) {
*   /** @type {RadialLayoutNodeInfo} *‍/
*   const info = layoutData.nodeInfos[node]
*   console.log(
*     `Node ${node} has been placed in circle ${info.circleIndex} with radius ${info.radius}.`
*   )
*   console.log(
*     `It has been placed in a segment from ${info.sectorStart} to ${
*       info.sectorStart + info.sectorSize
*     } degrees`
*   )
* }
* </pre>
*
* @see RadialLayout#NODE_INFO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutData%23nodeInfos Online Documentation}
*/
nodeInfos: IMapper<INode, RadialLayoutNodeInfo> | null;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
Gets or sets the mapping of edges to their {@link EdgeBundleDescriptor}.
Bundling together multiple edges means that their common parts are to some degree merged into a bundled part. At the
source and target point, the edges are again clearly split.
If an edge is mapped to <code>null</code>, the {@link EdgeBundling#defaultBundleDescriptor default descriptor} is used.
@see EdgeBundleDescriptor
@see RadialLayout#edgeBundling
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutData%23edgeBundleDescriptors Online Documentation}*/
edgeBundleDescriptors: ItemMapping<IEdge, EdgeBundleDescriptor>;
/**
Gets or sets the mapping from nodes to their layer/circle index when using the layering strategy
{@link RadialLayoutLayeringStrategy#USER_DEFINED}.
Nodes with the same ID are placed on the same circle while the circles are sorted according to their IDs such that a
smaller ID means that the circle has a smaller radius and is closer to the center of the radial layout.
When providing user-defined IDs, the center nodes (nodes on the innermost circle) are not determined by the specified {@link RadialLayout#centerNodesPolicy}
anymore. Instead, the nodes with the smallest given ID (i.e. <code>0</code>) define the set of center nodes.
<b>Warning:</b> The IDs provided here are only considered if {@link RadialLayoutLayeringStrategy#USER_DEFINED} is selected.
@see RadialLayoutLayeringStrategy#USER_DEFINED
@see RadialLayout#LAYER_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/RadialLayoutData%23layerIds Online Documentation}*/
layerIds: ItemMapping<INode, number>;
static $class: Class;
private $_RadialLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RadialLayoutData;
}
export interface RecursiveGroupLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link RecursiveGroupLayout}.
* The following example shows how to create a new instance of {@link RecursiveGroupLayoutData} and use it with an
* {@link RecursiveGroupLayout}:
*
* <br><b>Creating an instance of RecursiveGroupLayoutData</b>
* <pre>
* // Create layout data for both the RecursiveGroupLayout and the layout that's being run
* const recursiveGroupLayoutData = new RecursiveGroupLayoutData()
* // Use Orthogonal layout for group node contents
* recursiveGroupLayoutData.groupNodeLayouts.constant = new OrthogonalLayout()
*
* const organicLayoutData = new OrganicLayoutData()
* const orthogonalLayoutData = new OrthogonalLayoutData()
* // Configure the other layout data objects ...
*
* // Wrap both into a CompositeLayoutData
* const layoutData = new CompositeLayoutData(
*   recursiveGroupLayoutData,
*   organicLayoutData,
*   organicLayoutData
* )
*
* // Run the layout - The top-level layout will be done by an OrganicLayout
* graphComponent.graph.applyLayout(new RecursiveGroupLayout(new OrganicLayout()), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link RecursiveGroupLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of RecursiveGroupLayoutData</b>
* <pre>
* // Create layout data for both the RecursiveGroupLayout and the layout that's being run
* const recursiveGroupLayoutData = new RecursiveGroupLayoutData({
*   // Use Orthogonal layout for group node contents
*   groupNodeLayouts: new OrthogonalLayout()
* })
*
* const organicLayoutData = new OrganicLayoutData()
* // Configure the organic layout data ...
* const orthogonalLayoutData = new OrthogonalLayoutData()
* // Configure the orthogonal layout data ...
*
* // Wrap all three into a CompositeLayoutData
* const layoutData = new CompositeLayoutData(
*   recursiveGroupLayoutData,
*   organicLayoutData,
*   orthogonalLayoutData
* )
*
* // Run the layout - The top-level layout will be done by an OrganicLayout
* graphComponent.graph.applyLayout(new RecursiveGroupLayout(new OrganicLayout()), layoutData)
* </pre>
*
* Another fairly useful way of using {@link RecursiveGroupLayout} is to just do a top-level layout and leave the
* contents of the group nodes alone. This can be done by using {@link RecursiveGroupLayout#NULL_LAYOUT} as the
* layout algorithm for group nodes:
*
* <pre>
* const layoutData = new RecursiveGroupLayoutData()
* layoutData.groupNodeLayouts.constant = RecursiveGroupLayout.NULL_LAYOUT
*
* graphComponent.graph.applyLayout(new RecursiveGroupLayout(new OrthogonalLayout()), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData Online Documentation}
*/
export class RecursiveGroupLayoutData {
/**
Creates a new instance of {@link RecursiveGroupLayoutData} which helps configuring
{@link RecursiveGroupLayout}.
@param [options] The parameters to pass.
@param [options.groupNodeLayouts] The mapping from group nodes to the {@link ILayoutAlgorithm} used for their child nodes. This option sets the {@link RecursiveGroupLayoutData#groupNodeLayouts} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link RecursiveGroupLayoutData#nodeHalos} property on the created object.
@param [options.sourceSplitIds] A mapping from edges connecting to group nodes to source split ids. This option sets the {@link RecursiveGroupLayoutData#sourceSplitIds} property on the created object.
@param [options.targetSplitIds] A mapping from edges connecting to group nodes to target split ids. This option sets the {@link RecursiveGroupLayoutData#targetSplitIds} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link RecursiveGroupLayoutData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link RecursiveGroupLayoutData#targetPortConstraints} property on the created object.
@param [options.sourcePortCandidates] A mapping from edges to a collection of their source port {@link PortCandidate candidates}. This option sets the {@link RecursiveGroupLayoutData#sourcePortCandidates} property on the created object.
@param [options.targetPortCandidates] A mapping from edges to a collection of their target port {@link PortCandidate candidates}. This option sets the {@link RecursiveGroupLayoutData#targetPortCandidates} property on the created object.
@param [options.nodePortCandidateSets] A mapping from nodes to their {@link PortCandidateSet}. This option sets the {@link RecursiveGroupLayoutData#nodePortCandidateSets} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link RecursiveGroupLayoutData#abortHandler} property on the created object.
@param [options.partitionGridData] The {@link PartitionGridData partition grid} layout data. This option sets the {@link RecursiveGroupLayoutData#partitionGridData} property on the created object.
@param [options.groupNodePartitionGrids] A mapping from group nodes to the {@link PartitionGridData} used for their content. This option sets the {@link RecursiveGroupLayoutData#groupNodePartitionGrids} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData Online Documentation}*/
constructor(options?: {
groupNodeLayouts?: ItemMapping<INode, ILayoutAlgorithm> | ItemMappingConvertible<INode, ILayoutAlgorithm>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
sourceSplitIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetSplitIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
sourcePortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
targetPortCandidates?: ItemMapping<IEdge, ICollection<PortCandidate>> | ItemMappingConvertible<IEdge, ICollection<PortCandidate>>;
nodePortCandidateSets?: ItemMapping<INode, PortCandidateSet> | ItemMappingConvertible<INode, PortCandidateSet>;
abortHandler?: AbortHandler | null;
partitionGridData?: PartitionGridData;
groupNodePartitionGrids?: ItemMapping<INode, PartitionGridData> | ItemMappingConvertible<INode, PartitionGridData>;
});
/**
* Gets or sets the mapping from group nodes to the {@link ILayoutAlgorithm} used for their child nodes.
* The specified layout is applied to the content of the group node. To arrange the top level elements the {@link LayoutStageBase#coreLayout core layout algorithm}
* is used.
*
* The core layout is also applied to group nodes if there are no {@link RecursiveGroupLayoutData#groupNodeLayouts} defined at all. Importantly, this must be distinguished
* from the case that the mapping is defined but <code>null</code> is mapped with a group. Then, {@link RecursiveGroupLayout}
* handles this group node non-recursively. The group node and its content is arranged using the {@link ILayoutAlgorithm}
* instance specified for the nearest ancestor of the group node which is associated with a {@link ILayoutAlgorithm layout algorithm}.
*
* The layout algorithms for individual group nodes can be specified in various ways, depending on which option is the most
* convenient. The easiest way would be to set the same layout algorithm for all group nodes:
*
* <br><b>Using the same layout algorithm for all group nodes</b>
* <pre>
* layoutData.groupNodeLayouts.constant = new OrthogonalLayout()
* </pre>
*
* For scenarios where only some group nodes should have a specific layout algorithm the {@link ItemMapping#mapper} is usually the best option.
* However, due to {@link RecursiveGroupLayout}'s handling of individual group node layout's as described above, this
* can be a bit complicated to specify since the {@link IMapper} by default returns <code>null</code>.
*
* So when <em>only</em> the mapped group nodes should have a layout algorithm applied to their contents a new {@link Mapper} would have to be
* created:
*
* <pre>
* const mapper = new Mapper()
* mapper.defaultValue = RecursiveGroupLayout.NULL_LAYOUT
* mapper.set(groupNode1, new OrthogonalLayout())
* mapper.set(groupNode2, new HierarchicLayout())
* layoutData.groupNodeLayouts.mapper = mapper
* </pre>
*
* In cases where all group node's contents should be laid out, most of them with the {@link LayoutStageBase#coreLayout} and only some customized, the
* predefined {@link IMapper} can be used as is:
*
* <pre>
* layoutData.groupNodeLayouts.mapper.set(groupNode1, new OrthogonalLayout())
* layoutData.groupNodeLayouts.mapper.set(groupNode2, new HierarchicLayout())
* </pre>
*
* If the desired layout algorithm for a group node can be readily inferred from the node itself, the {@link ItemMapping#delegate} is usually the
* easiest to use:
*
* <br><b>Specifying custom layout algorithms for group node contents with a delegate</b>
* <pre>
* // Determine the layout for the group node from a property in the Tag object
* layoutData.groupNodeLayouts.delegate = groupNode => {
*   switch (groupNode.tag.layoutStyle) {
*     case 'hierarchic':
*       return new HierarchicLayout()
*     case 'organic':
*       return new OrganicLayout()
*     case 'orthogonal':
*       return new OrthogonalLayout()
*     case 'none':
*     default:
*       return RecursiveGroupLayout.NULL_LAYOUT
*   }
* }
* </pre>
*
* @see RecursiveGroupLayout#GROUP_NODE_LAYOUT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23groupNodeLayouts Online Documentation}
*/
groupNodeLayouts: ItemMapping<INode, ILayoutAlgorithm>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
Gets or sets a mapping from edges connecting to group nodes to source split ids.
Edges will be aligned with edges that connect to the same group node and have the same split id at their source
(preferably) or target.
<h2>Note</h2>
Marking edges with split ids will only provide good results if {@link HierarchicLayout} is used as core layout
algorithm. Also, the edges need to be {@link HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting routed directly from the content of a group to the group itself}.
<h2>Note</h2>
When a source split id is assigned to an edge that crosses a group node's border, it will be split automatically for
layout. In that case the {@link HierarchicLayout}'s {@link HierarchicLayout#edgeLayoutDescriptor default edge layout descriptor} must have {@link HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting direct edge routing from the content of a group to the group itself}
enabled for good results.
<b>Warning:</b>
In case a core layout algorithm is used that cannot handle groups, it will fail if it needs to layout a graph containing
an edge with a split id.
@see RecursiveGroupLayout#SOURCE_SPLIT_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23sourceSplitIds Online Documentation}*/
sourceSplitIds: ItemMapping<IEdge, any>;
/**
Gets or sets a mapping from edges connecting to group nodes to target split ids.
Edges will be aligned with edges that connect to the same group node and have the same split id at their source or
target (preferably).
<h2>Note</h2>
Marking edges with split ids will only provide good results if {@link HierarchicLayout} is used as core layout
algorithm. Also, the edges need to be {@link HierarchicLayoutEdgeLayoutDescriptor#directGroupContentEdgeRouting routed directly from the content of a group to the group itself}.
<b>Warning:</b>
In case a core layout algorithm is used that cannot handle groups, it will fail if it needs to layout a graph containing
an edge with a split id.
@see RecursiveGroupLayout#TARGET_SPLIT_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23targetSplitIds Online Documentation}*/
targetSplitIds: ItemMapping<IEdge, any>;
/**
* Gets or sets a mapping from edges to their source {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its source node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* A more general concept which allows finer control over where ports are placed, are {@link RecursiveGroupLayoutData#sourcePortCandidates port candidates}.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All source ports should be on the bottom side
* layoutData.sourcePortConstraints.constant = PortConstraint.create(PortSide.SOUTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should leave the source node on the left side, while keeping its exact port position (strong port constraint)
* layoutData.sourcePortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.WEST, true))
* // edge2 should leave the source node downwards, but the port may be placed anywhere along that node border
* layoutData.sourcePortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.SOUTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Source ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.sourcePortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see RecursiveGroupLayoutData#sourcePortCandidates
* @see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23sourcePortConstraints Online Documentation}
*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets a mapping from edges to their target {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its target node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* A more general concept which allows finer control over where ports are placed, are {@link RecursiveGroupLayoutData#targetPortCandidates port candidates}.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All target ports should be on the top side
* layoutData.targetPortConstraints.constant = PortConstraint.create(PortSide.NORTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should enter the target node on the right side, while keeping its exact port position (strong port
* // constraint)
* layoutData.targetPortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.EAST, true))
* // edge2 should enter the target node from the top, but the port may be placed anywhere along that node border
* layoutData.targetPortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.NORTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Target ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.targetPortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see RecursiveGroupLayoutData#targetPortCandidates
* @see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23targetPortConstraints Online Documentation}
*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets a mapping from edges to a collection of their source port {@link PortCandidate candidates}.
* Port constraints allow to define where an edge can connect to its source node and allow fine control over port
* placement.
*
* If all that is needed is to fix the source port in its location or on a node side, {@link RecursiveGroupLayoutData#sourcePortConstraints port constraints} are easier to work
* with, since they are a slightly simpler concept.
*
* Source port candidates are effectively a collection of possible port placements with different costs and the layout
* algorithm is free to choose the candidate that fits best into the overall layout, while also preferring candidates with
* a lower cost. To set the same candidate list for all edges, it's easiest to use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same set of source port candidates for all edges</b>
* <pre>
* layoutData.sourcePortCandidates.constant = new List([
*   // Prefer the center position and route the edge downwards if possible (no cost for this candidate)
*   PortCandidate.createCandidate(0, 0, PortDirections.SOUTH),
*   // Alternatively, use any position at the bottom border of the node, but with a higher cost
*   PortCandidate.createCandidate(PortDirections.SOUTH, 1),
*   // If that fails, use either the left or the right side
*   PortCandidate.createCandidate(PortDirections.EAST | PortDirections.WEST, 2)
* ])
* </pre>
*
* If certain edges need specific port candidates, it's usually convenient to use the {@link ItemMapping#mapper} property:
*
* <br><b>Setting different source port candidates for certain edges</b>
* <pre>
* // edge1 should leave the source node preferably on the node side in flow direction,
* // but can also use a fixed port location on the left of the node.
* layoutData.sourcePortCandidates.mapper.set(
*   edge1,
*   new List([
*     PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*     PortCandidate.createCandidate(-0.5, 0.25, PortDirections.WEST, 1)
*   ])
* )
*
* // edge2 should leave the source node anywhere on the upper side of the node
* layoutData.sourcePortCandidates.mapper.set(
*   edge2,
*   new List([PortCandidate.createCandidate(PortDirections.NORTH)])
* )
* </pre>
*
* For cases when the desired configuration of port candidates can be readily created from the edge itself, the {@link ItemMapping#delegate} is often
* the most convenient option:
*
* <br><b>Setting different source port candidates for each edge with a delegate</b>
* <pre>
* // Assuming a flowchart, the "yes" edge should leave the decision in flow direction,
* // and the "no" edge may leave either left or right of the flow direction.
* // This also ensures that the layout still looks natural if the layout uses different orientations.
* layoutData.sourcePortCandidates.delegate = edge => {
*   // Only handle decision nodes here
*   if (edge.sourceNode.tag instanceof FlowChartDecision) {
*     switch (edge.labels.get(0).text) {
*       case 'Yes':
*         return new List([PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW)])
*       case 'No':
*         return new List([
*           PortCandidate.createCandidate(
*             PortDirections.LEFT_IN_FLOW | PortDirections.RIGHT_IN_FLOW
*           )
*         ])
*     }
*   }
*   return null
* }
* </pre>
*
* @see RecursiveGroupLayoutData#sourcePortConstraints
* @see PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23sourcePortCandidates Online Documentation}
*/
sourcePortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
* Gets or sets a mapping from edges to a collection of their target port {@link PortCandidate candidates}.
* Port constraints allow to define where an edge can connect to its target node and allow fine control over port
* placement.
*
* If all that is needed is to fix the target port in its location or on a node side, {@link RecursiveGroupLayoutData#targetPortConstraints port constraints} are easier to work
* with, since they are a slightly simpler concept.
*
* Target port candidates are effectively a collection of possible port placements with different costs and the layout
* algorithm is free to choose the candidate that fits best into the overall layout, while also preferring candidates with
* a lower cost. To set the same candidate list for all edges, it's easiest to use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same set of target port candidates for all edges</b>
* <pre>
* layoutData.targetPortCandidates.constant = new List([
*   // Prefer the center position and enter the node from the top if possible (no cost for this candidate)
*   PortCandidate.createCandidate(0, 0, PortDirections.NORTH),
*   // Alternatively, use any position at the top border of the node, but with a higher cost
*   PortCandidate.createCandidate(PortDirections.NORTH, 1),
*   // If that fails, use either the left or the right side
*   PortCandidate.createCandidate(PortDirections.EAST | PortDirections.WEST, 2)
* ])
* </pre>
*
* If certain edges need specific port candidates, it's usually convenient to use the {@link ItemMapping#mapper} property:
*
* <br><b>Setting different target port candidates for certain edges</b>
* <pre>
* // edge1 should enter the target node preferably on the node side in flow direction,
* // but can also use a fixed port location on the left of the node.
* layoutData.targetPortCandidates.mapper.set(
*   edge1,
*   new List([
*     PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*     PortCandidate.createCandidate(-0.5, -0.25, PortDirections.WEST, 1)
*   ])
* )
*
* // edge2 should enter the target node anywhere on the bottom side of the node
* layoutData.targetPortCandidates.mapper.set(
*   edge2,
*   new List([PortCandidate.createCandidate(PortDirections.SOUTH)])
* )
* </pre>
*
* For cases when the desired configuration of port candidates can be readily created from the edge itself, the {@link ItemMapping#delegate} is often
* the most convenient option:
*
* <br><b>Setting different target port candidates for each edge with a delegate</b>
* <pre>
* // Assuming a flowchart, edges connecting to a terminal on the target side should enter that node either
* // in flow direction (preferred) or from one of the sides.
* // This also ensures that the layout still looks natural if the layout uses different orientations.
* layoutData.targetPortCandidates.delegate = edge => {
*   // Only handle terminal nodes here
*   if (edge.targetNode.tag instanceof FlowChartTerminal) {
*     return new List([
*       PortCandidate.createCandidate(PortDirections.WITH_THE_FLOW),
*       PortCandidate.createCandidate(PortDirections.LEFT_IN_FLOW | PortDirections.RIGHT_IN_FLOW, 1)
*     ])
*   }
*   return null
* }
* </pre>
*
* @see RecursiveGroupLayoutData#targetPortConstraints
* @see PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23targetPortCandidates Online Documentation}
*/
targetPortCandidates: ItemMapping<IEdge, ICollection<PortCandidate>>;
/**
Gets or sets a mapping from nodes to their {@link PortCandidateSet}.
@see PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23nodePortCandidateSets Online Documentation}*/
nodePortCandidateSets: ItemMapping<INode, PortCandidateSet>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
Gets or sets the {@link PartitionGridData partition grid} layout data.
This property defines the top-level global partition grid. To define a local partition grid for each recursively handled
group node use {@link RecursiveGroupLayoutData#groupNodePartitionGrids}. A top-level grid should only be registered if the layout algorithm responsible for the top-level
hierarchy i.e. the {@link LayoutStageBase#coreLayout core layout algorithm} supports a {@link PartitionGrid}.
Note that if you use {@link LayoutExecutor}'s {@link LayoutExecutor#configureTableLayout} feature this property should not be used as it overrides the {@link PartitionGrid}
set by {@link TableLayoutConfigurator}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23partitionGridData Online Documentation}*/
partitionGridData: PartitionGridData;
/**
Gets or set a mapping from group nodes to the {@link PartitionGridData} used for their content.
Each recursively handled group node can get its own local {@link PartitionGrid}. The grid is then valid for the
child nodes of the group and only visible for the {@link ILayoutAlgorithm layout algorithm} that is responsible for the group's content. Thus, to
properly consider the grid it is required that the responsible algorithm supports the partition grid structure.
In addition to the local partition grids it is also allowed to have a global grid which can be defined by using {@link RecursiveGroupLayoutData#partitionGridData}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/RecursiveGroupLayoutData%23groupNodePartitionGrids Online Documentation}*/
groupNodePartitionGrids: ItemMapping<INode, PartitionGridData>;
static $class: Class;
private $_RecursiveGroupLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is RecursiveGroupLayoutData;
}
export interface ReverseEdgesStageData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link ReverseEdgesStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStageData Online Documentation}*/
export class ReverseEdgesStageData {
/**
Creates a new instance of {@link ReverseEdgesStageData} which helps configuring {@link ReverseEdgesStage}.
@param [options] The parameters to pass.
@param [options.affectedEdges] The collection of affected edges. This option sets the {@link ReverseEdgesStageData#affectedEdges} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link ReverseEdgesStageData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link ReverseEdgesStageData#targetPortConstraints} property on the created object.
@param [options.sourceGroupIds] A mapping from edges to an object representing their source edge group. This option sets the {@link ReverseEdgesStageData#sourceGroupIds} property on the created object.
@param [options.targetGroupIds] A mapping from edges to an object representing their target edge group. This option sets the {@link ReverseEdgesStageData#targetGroupIds} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStageData Online Documentation}*/
constructor(options?: {
affectedEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
sourceGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
});
/**
Gets or sets the collection of affected edges.
@see ReverseEdgesStage#AFFECTED_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStageData%23affectedEdges Online Documentation}*/
affectedEdges: ItemCollection<IEdge>;
/**
Gets or sets a mapping from edges to their source {@link PortConstraint}.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStageData%23sourcePortConstraints Online Documentation}*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to their target {@link PortConstraint}.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStageData%23targetPortConstraints Online Documentation}*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to an object representing their source edge group.
Edges sharing a source group identifier will share a common bus near the source or at a common source node if possible.
The {@link HierarchicLayout} is able to group incoming and outgoing edges. Therefore, the source group
identifier of the outgoing edges of a node has to match with the target group identifier of the incoming edges.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStageData%23sourceGroupIds Online Documentation}*/
sourceGroupIds: ItemMapping<IEdge, any>;
/**
Gets or sets a mapping from edges to an object representing their target edge group.
Edges sharing a target group identifier will share a common bus near the target or at a common target node if possible.
The {@link HierarchicLayout} is able to group incoming and outgoing edges. Therefore, the target group
identifier of the incoming edges of a node has to match with the source group identifier of the outgoing edges.
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ReverseEdgesStageData%23targetGroupIds Online Documentation}*/
targetGroupIds: ItemMapping<IEdge, any>;
static $class: Class;
private $_ReverseEdgesStageData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ReverseEdgesStageData;
}
export interface SelfLoopCalculatorData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link SelfLoopCalculator}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculatorData Online Documentation}*/
export class SelfLoopCalculatorData {
/**
Creates a new instance of {@link SelfLoopCalculatorData} which helps configuring
{@link SelfLoopCalculator}.
@param [options] The parameters to pass.
@param [options.octilinearEdges] The collection of edges that are routed in an octilinear fashion. This option sets the {@link SelfLoopCalculatorData#octilinearEdges} property on the created object.
@param [options.minimumNodeDistances] The mapping from nodes to their minimum distance to self-loop edges. This option sets the {@link SelfLoopCalculatorData#minimumNodeDistances} property on the created object.
@param [options.minimumEdgeDistances] The mapping from edges to their minimum distance to the next edge and to node sides. This option sets the {@link SelfLoopCalculatorData#minimumEdgeDistances} property on the created object.
@param [options.minimumFirstSegmentLengths] The mapping from edges to the minimum length of their first segment. This option sets the {@link SelfLoopCalculatorData#minimumFirstSegmentLengths} property on the created object.
@param [options.minimumLastSegmentLengths] The mapping from edges to the minimum length of their last segment. This option sets the {@link SelfLoopCalculatorData#minimumLastSegmentLengths} property on the created object.
@param [options.minimumOctilinearSegmentLengths] The mapping from edges to the minimum length of their octilinear segments. This option sets the {@link SelfLoopCalculatorData#minimumOctilinearSegmentLengths} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculatorData Online Documentation}*/
constructor(options?: {
octilinearEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
minimumNodeDistances?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
minimumEdgeDistances?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
minimumFirstSegmentLengths?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
minimumLastSegmentLengths?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
minimumOctilinearSegmentLengths?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
});
/**
Gets or sets the collection of edges that are routed in an octilinear fashion.
@see SelfLoopCalculator#OCTILINEAR_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculatorData%23octilinearEdges Online Documentation}*/
octilinearEdges: ItemCollection<IEdge>;
/**
Gets or sets the mapping from nodes to their minimum distance to self-loop edges.
@see SelfLoopCalculator#MINIMUM_NODE_DISTANCE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculatorData%23minimumNodeDistances Online Documentation}*/
minimumNodeDistances: ItemMapping<INode, number>;
/**
Gets or sets the mapping from edges to their minimum distance to the next edge and to node sides.
@see SelfLoopCalculator#MINIMUM_EDGE_DISTANCE_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculatorData%23minimumEdgeDistances Online Documentation}*/
minimumEdgeDistances: ItemMapping<IEdge, number>;
/**
Gets or sets the mapping from edges to the minimum length of their first segment.
@see SelfLoopCalculator#MINIMUM_FIRST_SEGMENT_LENGTH_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculatorData%23minimumFirstSegmentLengths Online Documentation}*/
minimumFirstSegmentLengths: ItemMapping<IEdge, number>;
/**
Gets or sets the mapping from edges to the minimum length of their last segment.
@see SelfLoopCalculator#MINIMUM_LAST_SEGMENT_LENGTH_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculatorData%23minimumLastSegmentLengths Online Documentation}*/
minimumLastSegmentLengths: ItemMapping<IEdge, number>;
/**
Gets or sets the mapping from edges to the minimum length of their octilinear segments.
@see SelfLoopCalculator#MINIMUM_OCTILINEAR_SEGMENT_LENGTH_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SelfLoopCalculatorData%23minimumOctilinearSegmentLengths Online Documentation}*/
minimumOctilinearSegmentLengths: ItemMapping<IEdge, number>;
static $class: Class;
private $_SelfLoopCalculatorData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SelfLoopCalculatorData;
}
export interface SequenceConstraintData extends LayoutData {
/**
@inheritdoc*/
apply(layoutGraphAdapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies sequence constraints for {@link INode nodes} and {@link IEdge edges} for a {@link HierarchicLayout}.
* Sequence constraints affect the sequence of nodes within a {@link ILayer layer}.
*
* If {@link SequenceConstraintData#itemComparables} are specified, they are used to sort all nodes and edges accordingly and create {@link SequenceConstraintData#placeAfter}
* constraints that are handles just like manually added constraints.
*
* <h2>Note</h2>
*
* <ul>
* <li>Sequence constraints cannot be used together with {@link SwimlaneDescriptor swimlanes}.</li>
* <li>Sequence constraints that are specified for a group child node will be applied to the parent node instead.</li>
* </ul>
* One way to set sequence constraints is by mapping nodes or edges to an {@link IComparable} (e.g. a number) which
* define the order in which those nodes are placed within the same layer. This can be done with the {@link SequenceConstraintData#itemComparables} property. One way to
* map each item to such an {@link IComparable} is to use the {@link ItemMapping#delegate}:
*
* <br><b>Using a delegate to map items to objects that can be ordered to define relative placements within a layer</b>
* <pre>
* // Map each item to an IComparable, in this case taken from a custom object stored in the Tag
* layoutData.sequenceConstraints.itemComparables = item => {
*   const data = item.tag
*   if (data !== null) {
*     return data.sequenceOrder
*   }
*   // Don't constrain sequencing if the tag doesn't contain our custom object.
*   return null
* }
* </pre>
*
* If only some nodes should be constrained, the {@link ItemMapping#mapper} can be used instead:
*
* <br><b>Using a mapper to map certain items to objects that can be ordered to define relative placements within a layer</b>
* <pre>
* // Define sequence constraints for a few nodes via a mapper. Nodes that are not
* // explicitly mapped don't have constraints on their placement within the layer.
* const mapper = layoutData.sequenceConstraints.itemComparables.mapper
* // Place node1 before node2
* mapper.set(node1, 1)
* mapper.set(node2, 3)
* // Place node3 between both of them
* mapper.set(node3, 2)
* // Place node4 after the others by assigning a larger value
* mapper.set(node4, 8)
* // The exact values don't matter much they just define an ordering. However, all
* // values must have the same type you cannot mix int and string or double, for example.
* </pre>
*
* If those mappings have been prepared beforehand, e.g. in a {@link HashMap} or {@link IMapper}, that property on the {@link ItemMapping} can also be set:
*
* <br><b>Setting a prepared mapper or dictionary for relative placement within a layer</b>
* <pre>
* layoutData.sequenceConstraints.itemComparables = sequenceMapper
* // Or create a mapper from a dictionary, instead:
* layoutData.sequenceConstraints.itemComparables = new Mapper(sequenceDictionary)
* </pre>
*
* {@link IComparable} is only the most natural option if such an object can be readily produced from a node, or there
* already exists such a mapping from nodes to something that can be compared. In many cases it can be easier to construct
* constraints manually with the respective methods on {@link SequenceConstraintData}:
*
* <br><b>Defining custom sequence constraints</b>
* <pre>
* // Ensure that node2 is placed after node1
* layoutData.sequenceConstraints.placeAfter(node1, node2)
* // Also place another node at the very start
* layoutData.sequenceConstraints.placeAtHead(firstNodeInLayer)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SequenceConstraintData Online Documentation}
*/
export class SequenceConstraintData {
/**
Creates a new instance of {@link SequenceConstraintData} which helps configuring sequence constraints for
{@link HierarchicLayout}.
@param [options] The parameters to pass.
@param [options.itemComparables] The mapping from nodes or edges to an {@link IComparable} defining their sequence sort order. This option sets the {@link SequenceConstraintData#itemComparables} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SequenceConstraintData Online Documentation}*/
constructor(options?: {
itemComparables?: ItemMapping<IModelItem, IComparable> | ItemMappingConvertible<IModelItem, IComparable>;
});
/**
* Clears all previously defined constraints.
* All constraints that were previously defined via {@link SequenceConstraintData#placeBefore},
* {@link SequenceConstraintData#placeAfter}, {@link SequenceConstraintData#placeAtHead} or {@link SequenceConstraintData#placeAtTail}
* were cleared so that new constraints can be declared.
*
* This doesn't affect indirect constraints specified by the {@link SequenceConstraintData#itemComparables}.
*
* <br><b>Clearing all previously set sequence constraints</b>
* <pre>
* layoutData.sequenceConstraints.clearConstraints()
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SequenceConstraintData%23clearConstraints() Online Documentation}
*/
clearConstraints(): void;
/**
* Adds a constraint that forces the <code>afterItem</code> of type {@link INode} or {@link IEdge} to lie after the <code>item</code> of
* type {@link INode} or {@link IEdge}.
* <h2>Note</h2>
*
* A constraint is only considered if both elements share at least one layer. An edge is associated with each layer that it
* spans, i.e., each layer between it's source and target (excluding the source and target layer).
*
* <h2>Note</h2>
*
* A constraint is only considered if both elements share at least one layer. An edge constraint is associated with each
* layer that the edge spans including the edge's source and target (e.g., an after constraint between two edges also
* implies that the source/target of the <code>after</code> edge should be placed after that of the <code>reference</code> edge, if possible).
* Constraints of edges with {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} are ignored.
*
* <h2>Note</h2>
*
* If there are groups or swimlanes, there are implicit constraints on the edges, e.g., the edge path never leaves the
* bounding box of the group that is the common ancestor of the edge's endpoints. Furthermore, if both endpoints of an edge
* are associated with the same swimlane, the route does never leave this lane. Such constraints cannot be affected by
* sequence constraints.
*
* <br><b>Placing a node after a reference node</b>
* <pre>
* layoutData.sequenceConstraints.placeAfter(referenceNode, node)
* </pre>
*
* @param item the reference node or edge
* @param afterItem the node or edge that should be placed after the reference element
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SequenceConstraintData%23placeAfter(IModelItem%2CIModelItem) Online Documentation}
*/
placeAfter(item: IModelItem, afterItem: IModelItem): void;
/**
* Adds a constraint that places a {@link INode} or {@link IEdge} at the start of the sequence.
* <h2>Note</h2>
*
* For edges, this constraint applies to each layer that the edge spans including the edge's source and target. Edges with
* {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} are ignored.
*
* <h2>Note</h2>
*
* If there are groups or swimlanes, there are implicit constraints on the edges, e.g., the edge path never leaves the
* bounding box of the group that is the common ancestor of the edge's endpoints. Furthermore, if both endpoints of an edge
* are associated with the same swimlane, the route does never leave this lane. Such constraints cannot be affected by
* sequence constraints.
*
* <br><b>Placing a node at the start of the layer</b>
* <pre>
* layoutData.sequenceConstraints.placeAtHead(node)
* </pre>
*
* @param item the node or edge that should be placed at the start
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SequenceConstraintData%23placeAtHead(IModelItem) Online Documentation}
*/
placeAtHead(item: IModelItem): void;
/**
* Adds a constraint that places a {@link INode} or {@link IEdge} at the end of the sequence.
* <h2>Note</h2>
*
* For edges, this constraint applies to each layer that the edge spans including the edge's source and target. Edges with
* {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} are ignored.
*
* <h2>Note</h2>
*
* If there are groups or swimlanes, there are implicit constraints on the edges, e.g., the edge path never leaves the
* bounding box of the group that is the common ancestor of the edge's endpoints. Furthermore, if both endpoints of an edge
* are associated with the same swimlane, the route does never leave this lane. Such constraints cannot be affected by
* sequence constraints.
*
* <br><b>Placing a node at the end of a layer</b>
* <pre>
* layoutData.sequenceConstraints.placeAtTail(node)
* </pre>
*
* @param item the node or edge that should be placed at the end
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SequenceConstraintData%23placeAtTail(IModelItem) Online Documentation}
*/
placeAtTail(item: IModelItem): void;
/**
* Adds a constraint that forces the <code>beforeItem</code> of type {@link INode} or {@link IEdge} to lie before the <code>item</code> of
* type {@link INode} or {@link IEdge}.
* <h2>Note</h2>
*
* A constraint is only considered if both elements share at least one layer. An edge is associated with each layer that it
* spans, i.e., each layer between it's source and target (excluding the source and target layer).
*
* <h2>Note</h2>
*
* A constraint is only considered if both elements share at least one layer. An edge constraint is associated with each
* layer that the edge spans including the edge's source and target (e.g., a before constraint between two edges also
* implies that the source/target of the <code>before</code> edge should be placed before that of the <code>reference</code> edge, if possible).
* Constraints of edges with {@link HierarchicLayoutEdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style} are ignored.
*
* <h2>Note</h2>
*
* If there are groups or swimlanes, there are implicit constraints on the edges, e.g., the edge path never leaves the
* bounding box of the group that is the common ancestor of the edge's endpoints. Furthermore, if both endpoints of an edge
* are associated with the same swimlane, the route does never leave this lane. Such constraints cannot be affected by
* sequence constraints.
*
* <br><b>Placing a node before a reference node</b>
* <pre>
* layoutData.sequenceConstraints.placeBefore(referenceNode, node)
* </pre>
*
* @param item the reference node or edge
* @param beforeItem the node or edge that should be placed before the reference element
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SequenceConstraintData%23placeBefore(IModelItem%2CIModelItem) Online Documentation}
*/
placeBefore(item: IModelItem, beforeItem: IModelItem): void;
/**
* Gets or set the mapping from nodes or edges to an {@link IComparable} defining their sequence sort order.
* Items with a lower sort order are constraint to be placed before items with a higher sort order. For items mapped to
* <code>null</code>, no constraints are added.
*
* <br><b>Using the node's labels to indicate their order</b>
* <pre>
* layoutData.layerConstraints.nodeComparables = node =>
*   node.labels.size > 0 ? node.labels.get(0).text : null
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SequenceConstraintData%23itemComparables Online Documentation}
*/
itemComparables: ItemMapping<IModelItem, IComparable>;
static $class: Class;
private $_SequenceConstraintData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SequenceConstraintData;
}
export interface SeriesParallelLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link SeriesParallelLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutData Online Documentation}*/
export class SeriesParallelLayoutData {
/**
Creates a new instance of {@link SeriesParallelLayoutData} which helps configuring
{@link SeriesParallelLayout}.
@param [options] The parameters to pass.
@param [options.edgeLayoutDescriptors] The mapping from edges to their {@link SeriesParallelLayoutEdgeLayoutDescriptor}. This option sets the {@link SeriesParallelLayoutData#edgeLayoutDescriptors} property on the created object.
@param [options.portAssignments] The mapping from nodes to their {@link ISeriesParallelLayoutPortAssignment}. This option sets the {@link SeriesParallelLayoutData#portAssignments} property on the created object.
@param [options.outEdgeComparers] The mapping from nodes to comparison functions used to sort the nodes' outgoing edges. This option sets the {@link SeriesParallelLayoutData#outEdgeComparers} property on the created object.
@param [options.sourceGroupIds] A mapping from edges to an object representing their source edge group. This option sets the {@link SeriesParallelLayoutData#sourceGroupIds} property on the created object.
@param [options.targetGroupIds] A mapping from edges to an object representing their target edge group. This option sets the {@link SeriesParallelLayoutData#targetGroupIds} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link SeriesParallelLayoutData#abortHandler} property on the created object.
@param [options.edgeLabelPreferredPlacement] The mapping that provides a {@link PreferredPlacementDescriptor} instance for edge {@link ILabel}s. This option sets the {@link SeriesParallelLayoutData#edgeLabelPreferredPlacement} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutData Online Documentation}*/
constructor(options?: {
edgeLayoutDescriptors?: ItemMapping<IEdge, SeriesParallelLayoutEdgeLayoutDescriptor> | ItemMappingConvertible<IEdge, SeriesParallelLayoutEdgeLayoutDescriptor>;
portAssignments?: ItemMapping<INode, ISeriesParallelLayoutPortAssignment> | ItemMappingConvertible<INode, ISeriesParallelLayoutPortAssignment>;
outEdgeComparers?: ItemMapping<INode, (x: IEdge, y: IEdge) => number> | ItemMappingConvertible<INode, (x: IEdge, y: IEdge) => number>;
sourceGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
targetGroupIds?: ItemMapping<IEdge, any> | ItemMappingConvertible<IEdge, any>;
abortHandler?: AbortHandler | null;
edgeLabelPreferredPlacement?: ItemMapping<ILabel, PreferredPlacementDescriptor> | ItemMappingConvertible<ILabel, PreferredPlacementDescriptor>;
});
/**
Gets or sets the mapping from edges to their {@link SeriesParallelLayoutEdgeLayoutDescriptor}.
@see SeriesParallelLayoutEdgeLayoutDescriptor
@see SeriesParallelLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutData%23edgeLayoutDescriptors Online Documentation}*/
edgeLayoutDescriptors: ItemMapping<IEdge, SeriesParallelLayoutEdgeLayoutDescriptor>;
/**
Gets or sets the mapping from nodes to their {@link ISeriesParallelLayoutPortAssignment}.
If there is no {@link ISeriesParallelLayoutPortAssignment} specified for a node, the layout algorithm uses the
{@link SeriesParallelLayout#defaultPortAssignment default assignment}.
@see SeriesParallelLayout#PORT_ASSIGNMENT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutData%23portAssignments Online Documentation}*/
portAssignments: ItemMapping<INode, ISeriesParallelLayoutPortAssignment>;
/**
Gets or sets the mapping from nodes to comparison functions used to sort the nodes' outgoing edges.
If the comparison associated with a node is <code>null</code>, the outgoing edges maintain their initial order.
<b>Warning:</b> The order cannot be maintained in all cases because the target nodes may not be completely independent.
<h2>Note</h2>
For general graphs, the comparisons are only used for the series-parallel part of the graph. Non-series-parallel edges
are not included and will be routed by the {@link SeriesParallelLayout#nonSeriesParallelEdgeRouter edge router for non-series-parallel edges}.
<h2>Note</h2>
When {@link SeriesParallelLayoutData#outEdgeComparers} is used with type conversion, functions will be converted to the
{@link ItemMapping#delegate}. That means that a compare-function cannot be assigned directly.
@see SeriesParallelLayout#OUT_EDGE_COMPARER_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutData%23outEdgeComparers Online Documentation}*/
outEdgeComparers: ItemMapping<INode, Function>;
/**
Gets or sets a mapping from edges to an object representing their source edge group.
Edges sharing a source group identifier will share a common bus near the source or at a common source node if possible.
@see PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutData%23sourceGroupIds Online Documentation}*/
sourceGroupIds: ItemMapping<IEdge, any>;
/**
Gets or sets a mapping from edges to an object representing their target edge group.
Edges sharing a target group identifier will share a common bus near the target or at a common target node if possible.
@see PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutData%23targetGroupIds Online Documentation}*/
targetGroupIds: ItemMapping<IEdge, any>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
* Gets or sets the mapping that provides a {@link PreferredPlacementDescriptor} instance for edge
* {@link ILabel}s.
* Depending on how much customization is needed, some ways of setting {@link PreferredPlacementDescriptor}s are more
* convenient than others. For example, to set the same descriptor for all labels, you can just use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same PreferredPlacementDescriptor for all labels</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.constant = new PreferredPlacementDescriptor({
*   // Place labels along the edge
*   angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*   angle: 0,
*   // ... on either side
*   sideOfEdge: LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE,
*   // ... with a bit of distance to the edge
*   distanceToEdge: 5
* })
* </pre>
*
* If some labels should use custom placement or this has to be configured ahead of time, you can use the {@link ItemMapping#mapper} instead:
*
* <br><b>Customizing the placement of certain labels individually via a mapper</b>
* <pre>
* // Place label1 orthogonal to the edge anywhere on it
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label1,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.ANYWHERE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: Math.PI / 2
*   })
* )
* // Place label2 near the edge's source on either side of it, and make it parallel to the edge
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label2,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.AT_SOURCE,
*     sideOfEdge: LabelPlacements.RIGHT_OF_EDGE | LabelPlacements.LEFT_OF_EDGE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: 0
*   })
* )
* </pre>
*
* When the preferred placement can be inferred from the label itself, a delegate is usually the easiest choice:
*
* <br><b>Configuring preferred placement for all labels with a delegate</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.delegate = label => {
*   const customData = label.tag
*   const preferredPlacementDescriptor = new PreferredPlacementDescriptor({
*     angle: 0,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     // If the tag says to place the label in the center, put it in the center parallel to the edge's path
*     // All other labels can be placed anywhere, but on the side of the edge.
*     placeAlongEdge: customData.placeInCenter
*       ? LabelPlacements.AT_CENTER
*       : LabelPlacements.ANYWHERE,
*     sideOfEdge: customData.placeInCenter
*       ? LabelPlacements.ON_EDGE
*       : LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE
*   })
* }
* </pre>
*
* @see PreferredPlacementDescriptor
* @see LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/SeriesParallelLayoutData%23edgeLabelPreferredPlacement Online Documentation}
*/
edgeLabelPreferredPlacement: ItemMapping<ILabel, PreferredPlacementDescriptor>;
static $class: Class;
private $_SeriesParallelLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is SeriesParallelLayoutData;
}
export interface StraightLineEdgeRouterData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link StraightLineEdgeRouter}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouterData Online Documentation}*/
export class StraightLineEdgeRouterData {
/**
Creates a new instance of {@link StraightLineEdgeRouterData} which helps configuring
{@link StraightLineEdgeRouter}.
@param [options] The parameters to pass.
@param [options.affectedEdges] The collection of affected edges. This option sets the {@link StraightLineEdgeRouterData#affectedEdges} property on the created object.
@param [options.affectedNodes] The collection of affected nodes. This option sets the {@link StraightLineEdgeRouterData#affectedNodes} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link StraightLineEdgeRouterData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link StraightLineEdgeRouterData#targetPortConstraints} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouterData Online Documentation}*/
constructor(options?: {
affectedEdges?: DpKeyItemCollection<IEdge> | DpKeyItemCollectionConvertible<IEdge>;
affectedNodes?: DpKeyItemCollection<INode> | DpKeyItemCollectionConvertible<INode>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
});
/**
Gets or sets the collection of affected edges.
This collection is only used by the router if the {@link StraightLineEdgeRouter#scope} is set to {@link EdgeRouterScope#ROUTE_AFFECTED_EDGES}.
@see StraightLineEdgeRouter#affectedEdgesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouterData%23affectedEdges Online Documentation}*/
affectedEdges: DpKeyItemCollection<IEdge>;
/**
Gets or sets the collection of affected nodes.
This collection is only used by the router if the {@link StraightLineEdgeRouter#scope} is set to {@link EdgeRouterScope#ROUTE_EDGES_AT_AFFECTED_NODES}.
@see StraightLineEdgeRouter#affectedNodesDpKey
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouterData%23affectedNodes Online Documentation}*/
affectedNodes: DpKeyItemCollection<INode>;
/**
Gets or sets a mapping from edges to their source {@link PortConstraint}.
@see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouterData%23sourcePortConstraints Online Documentation}*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
Gets or sets a mapping from edges to their target {@link PortConstraint}.
@see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/StraightLineEdgeRouterData%23targetPortConstraints Online Documentation}*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
static $class: Class;
private $_StraightLineEdgeRouterData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is StraightLineEdgeRouterData;
}
export interface TableLayoutConfigurator extends YObject {
}
/**
Creates a {@link PartitionGrid} that matches a given table configuration of an {@link IGraph} and registers {@link IMapper}
for that grid at the graph.
Typically, this class is used by class {@link LayoutExecutor} to {@link TableLayoutConfigurator#prepare} a
partition grid before the layout is calculated and {@link TableLayoutConfigurator#restore} the graph afterwards.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator Online Documentation}*/
export class TableLayoutConfigurator {
/**
Creates a new instance.
@param [options] The parameters to pass.
@param [options.compaction] Whether size compaction for table nodes is enabled. This option sets the {@link TableLayoutConfigurator#compaction} property on the created object.
@param [options.fromSketch] Whether 'from sketch' mode is enabled. This option sets the {@link TableLayoutConfigurator#fromSketch} property on the created object.
@param [options.horizontalLayout] Whether the configurator will prepare its associated graph for a vertical (that is top-to-bottom or bottom-to-top) layout calculation or a horizontal (that is left-to-right or right-to-left) one. This option sets the {@link TableLayoutConfigurator#horizontalLayout} property on the created object.
@param [options.minimumTableDistance] The minimum distance between two different table nodes This option sets the {@link TableLayoutConfigurator#minimumTableDistance} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator Online Documentation}*/
constructor(options?: {
compaction?: boolean;
fromSketch?: boolean;
horizontalLayout?: boolean;
minimumTableDistance?: number;
});
/**
Destroy all information that results from a previous {@link TableLayoutConfigurator#prepare} and a subsequent
layout
@param graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23cleanUp(IGraph) Online Documentation}*/
cleanUp(graph: IGraph): void;
/**
Returns the {@link PartitionCellId} assigned to the <code>node</code>.
The id is assigned to the node in {@link TableLayoutConfigurator#prepare} and is only valid before {@link TableLayoutConfigurator#restore}
is called so an {@link Exception InvalidOperationError} is thrown when this method isn't called between those two.
@param node The node to return the assigned cell id for.
@returns The {@link PartitionCellId} assigned to the <code>node</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23getAssignedCellId(INode) Online Documentation}*/
getAssignedCellId(node: INode): PartitionCellId | null;
/**
Returns the {@link ITable} the <code>node</code> belongs to.
The node to table association is resolved in {@link TableLayoutConfigurator#prepare} and is only valid before {@link TableLayoutConfigurator#restore}
is called so an {@link Exception InvalidOperationError} is thrown when this method isn't called between those two.
The associated table is the {@link ITable} provided by the node itself or one of its {@link IGraph#getParent ancestors} via {@link ILookup#lookup lookup}. If no
associated table can be found, <code>null</code> is returned.
@param node The node to lookup the table for.
@returns The {@link ITable} the <code>node</code> is associated with or <code>null</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23getAssociatedTable(INode) Online Documentation}*/
getAssociatedTable(node: INode): ITable | null;
/**
Returns the {@link ColumnDescriptor} of the column in the {@link TableLayoutConfigurator#grid} to which the <code>leafColumn</code> is mapped.
The columns are mapped to the {@link TableLayoutConfigurator#grid} in {@link TableLayoutConfigurator#prepare} and are only valid before {@link TableLayoutConfigurator#restore}
is called so an {@link Exception InvalidOperationError} is thrown when this method isn't called between those two.
Note that only leaf columns (columns with no {@link IColumn#childColumns child columns}) are mapped to grid columns so <code>null</code> is returned for any other
column.
@param leafColumn The column to return the column descriptor for.
@returns The {@link ColumnDescriptor} the <code>leafColumn</code> is mapped to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23getColumnDescriptor(IColumn) Online Documentation}*/
getColumnDescriptor(leafColumn: IColumn): ColumnDescriptor | null;
/**
Return the sizes of all <b>leaf</b> rows.
This method does not modify <code>originalTable</code>. You also need to call {@link TableLayoutConfigurator#cleanUp}
explicitly if you don't need the {@link TableLayoutConfigurator} anymore.
@param originalTable The table
@param nodeLayout The final layout of the node that owns the table
@returns The sizes of all <b>leaf</b> rows in <code>originalTable</code>, in their natural order.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23getColumnLayout(ITable%2CRect) Online Documentation}*/
getColumnLayout(originalTable: ITable, nodeLayout: Rect): number[];
/**
Returns the {@link RowDescriptor} of the row in the {@link TableLayoutConfigurator#grid} to which the <code>leafRow</code> is mapped.
The rows are mapped to the {@link TableLayoutConfigurator#grid} in {@link TableLayoutConfigurator#prepare} and are only valid before {@link TableLayoutConfigurator#restore}
is called so an {@link Exception InvalidOperationError} is thrown when this method isn't called between those two.
Note that only leaf rows (rows with no {@link IRow#childRows child rows}) are mapped to grid rows so <code>null</code> is returned for any other row.
@param leafRow The row to return the row descriptor for.
@returns The {@link RowDescriptor} the <code>leafRow</code> is mapped to.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23getRowDescriptor(IRow) Online Documentation}*/
getRowDescriptor(leafRow: IRow): RowDescriptor | null;
/**
Return the sizes of all <b>leaf</b> rows.
This method does not modify <code>originalTable</code>. You also need to call {@link TableLayoutConfigurator#cleanUp}
explicitly if you don't need the {@link TableLayoutConfigurator} anymore.
@param originalTable The table
@param nodeLayout The final layout of the node that owns the table
@returns The sizes of all <b>leaf</b> rows in <code>originalTable</code>, in their natural order.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23getRowLayout(ITable%2CRect) Online Documentation}*/
getRowLayout(originalTable: ITable, nodeLayout: Rect): number[];
/**
Returns the {@link INode} which provides the <code>table</code> in its {@link ILookup#lookup lookup}.
The node to table association is resolved in {@link TableLayoutConfigurator#prepare} and is only valid before {@link TableLayoutConfigurator#restore}
is called so an {@link Exception InvalidOperationError} is thrown when this method isn't called between those two.
@param table The table for which the corresponding owner node is searched.
@returns The {@link INode} which provides the <code>table</code> in its {@link ILookup#lookup lookup}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23getTableNode(ITable) Online Documentation}*/
getTableNode(table: ITable): INode | null;
/**
Returns whether <code>node</code> is recognized as a table node by this instance.
@param node The node to check.
@returns <code>true</code> if this instance treats <code>node</code> as a table node.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23isTableNode(INode) Online Documentation}*/
isTableNode(node: INode): boolean;
/**
Setup partition grid information from a graph that contains table structures
@param graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23prepare(IGraph) Online Documentation}*/
prepare(graph: IGraph): void;
/**
Write back all information from the partition grid
@param graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23restore(IGraph) Online Documentation}*/
restore(graph: IGraph): void;
/**
Gets the {@link PartitionGrid} created in the {@link TableLayoutConfigurator#prepare} method.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23grid Online Documentation}*/
grid: PartitionGrid | null;
/**
Gets or sets the minimum distance between two different table nodes
The default value is <code>50</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23minimumTableDistance Online Documentation}*/
minimumTableDistance: number;
/**
Gets or sets whether size compaction for table nodes is enabled.
If size compaction is enabled, the size of a column or row (or a table node for that matter) may be reduced due to
layout calculation, but never below the column's or row's minimum size. If size compaction is disabled, columns and rows
(and therefore table nodes) may only grow due to layout calculation.
The default value for this property is <code>true</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23compaction Online Documentation}*/
compaction: boolean;
/**
Gets or sets whether the configurator will prepare its associated graph for a vertical (that is top-to-bottom or
bottom-to-top) layout calculation or a horizontal (that is left-to-right or right-to-left) one.
The default value for this property is <code>false</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23horizontalLayout Online Documentation}*/
horizontalLayout: boolean;
/**
Gets or sets whether 'from sketch' mode is enabled.
If enabled, the positions of the table group nodes are considered, i.e., for horizontal layout configuration these nodes
are sorted according to their y-coordinate and for vertical layout configuration according to their x-coordinate.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TableLayoutConfigurator%23fromSketch Online Documentation}*/
fromSketch: boolean;
static $class: Class;
private $_TableLayoutConfigurator: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TableLayoutConfigurator;
}
export interface TabularLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link TabularLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutData Online Documentation}*/
export class TabularLayoutData {
/**
Creates a new instance of {@link TabularLayoutData} which helps configuring {@link TabularLayout}.
@param [options] The parameters to pass.
@param [options.nodeLayoutDescriptors] The mapping of nodes to their {@link TabularLayoutNodeLayoutDescriptor} that defines custom node placement settings. This option sets the {@link TabularLayoutData#nodeLayoutDescriptors} property on the created object.
@param [options.partitionGridData] The partition grid layout data. This option sets the {@link TabularLayoutData#partitionGridData} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link TabularLayoutData#nodeHalos} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutData Online Documentation}*/
constructor(options?: {
nodeLayoutDescriptors?: ItemMapping<INode, TabularLayoutNodeLayoutDescriptor> | ItemMappingConvertible<INode, TabularLayoutNodeLayoutDescriptor>;
partitionGridData?: PartitionGridData;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
});
/**
Gets or sets the mapping of nodes to their {@link TabularLayoutNodeLayoutDescriptor} that defines custom node placement
settings.
If a node is mapped to <code>null</code>, then the {@link TabularLayout#defaultNodeLayoutDescriptor default descriptor} applies for that node.
@see TabularLayout#NODE_LAYOUT_DESCRIPTOR_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutData%23nodeLayoutDescriptors Online Documentation}*/
nodeLayoutDescriptors: ItemMapping<INode, TabularLayoutNodeLayoutDescriptor>;
/**
Gets or sets the partition grid layout data.
How the grid is used by {@link TabularLayout} depends on the use case and the chosen layout policy:
<ul>
<li>When using {@link TabularLayoutPolicy#AUTO_SIZE}, it is not necessary to define a partition grid at all. If a {@link PartitionGridData#grid Grid}
is specified, its existing rows and columns will be cleared and the grid is filled with the data resembling the computed
layout. That allows to query information about the rows and columns using classes {@link RowDescriptor} and {@link ColumnDescriptor}
(e.g. query computed position of row/column).
<li>For policy {@link TabularLayoutPolicy#FIXED_SIZE} it is necessary to define a partition grid instance. The size of
the given grid defines the size of the computed tabular layout. A node can be mapped to a specific grid cell to manually
define its location in the layout.
<li>Policy {@link TabularLayoutPolicy#FROM_SKETCH}: the same as for {@link TabularLayoutPolicy#AUTO_SIZE}
applies with respect to the necessity of the partition grid. That means that the grid is optional and only necessary if
the resulting dimension shall be queried after the layout.
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutData%23partitionGridData Online Documentation}*/
partitionGridData: PartitionGridData;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TabularLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
static $class: Class;
private $_TabularLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TabularLayoutData;
}
export interface TemporaryGroupNodeInsertionData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link TemporaryGroupNodeInsertionStage}.
* The {@link TemporaryGroupNodeInsertionStage} can be used for ungrouped graphs if nodes should be treated by layout
* algorithms as if they were actually grouped.
*
* Using a {@link RecursiveGroupLayout} together with a {@link TemporaryGroupNodeInsertionStage}
*
* <pre>
* // define component1 to use an OrganicLayout
* // with all nodes in nodeList belonging to component1
* const groupNodeInsertionData = new TemporaryGroupNodeInsertionData()
* const component1 = groupNodeInsertionData.components.add(new OrganicLayout())
* component1.items = nodeList
* // wrap the core layout in a RecursiveGroupLayout and then both in a TemporaryGroupNodeInsertionStage
* const layout = new TemporaryGroupNodeInsertionStage(new RecursiveGroupLayout(hierarchicLayout))
* // combine the HierarchicLayoutData and the TemporaryGroupNodeInsertionData
* const layoutData = new CompositeLayoutData(hierarchicLayoutData, groupNodeInsertionData)
* // run the layout
* await graphComponent.morphLayout({ layout, layoutData, morphDuration: '1s' })
* // do something after the Promise resolved
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TemporaryGroupNodeInsertionData Online Documentation}
*/
export class TemporaryGroupNodeInsertionData {
/**
Creates a new instance of {@link TemporaryGroupNodeInsertionData} which helps configuring
{@link TemporaryGroupNodeInsertionStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemporaryGroupNodeInsertionData Online Documentation}*/
constructor();
/**
Gets or sets the mapping from nodes to a component and, optionally, the layout algorithm that should be applied for that
component if the stage is used with {@link RecursiveGroupLayout} as core layout.
Nodes with the same component Id constitute a component and are assigned to the same, temporary, group node. It is
visible during the execution of the {@link LayoutStageBase#coreLayout core layout algorithm} and again removed afterwards.
To conveniently define a component as well as the layout algorithm responsible for it use the {@link ItemCollectionMapping#add} method of this
property. It takes the layout algorithm as parameter and allows to define the nodes associated to it using the returned
{@link ItemCollection}. Use <code>null</code> as parameter to solely define a component which is not associated to any layout algorithm (e.g. if not using
{@link RecursiveGroupLayout}).
<b>Warning:</b> It is not allowed to define nested components. If nesting is required, define a real grouping structure - see
{@link GroupingKeys}.
@see TemporaryGroupNodeInsertionStage#COMPONENT_ID_DP_KEY
@see TemporaryGroupNodeInsertionStage#COMPONENT_LAYOUT_ALGORITHM_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TemporaryGroupNodeInsertionData%23components Online Documentation}*/
components: ItemCollectionMapping<INode, ILayoutAlgorithm>;
static $class: Class;
private $_TemporaryGroupNodeInsertionData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TemporaryGroupNodeInsertionData;
}
export interface TreeLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
* Specifies custom data for the {@link TreeLayout}.
* The following example shows how to create a new instance of {@link TreeLayoutData} and use it with an
* {@link TreeLayout}:
*
* <br><b>Creating an instance of TreeLayoutData</b>
* <pre>
* const layoutData = new TreeLayoutData()
* layoutData.nodeHalos.constant = NodeHalo.create(15)
* layoutData.treeRoot.item = rootNode
*
* graph.applyLayout(new TreeLayout(), layoutData)
* </pre>
*
* In many cases the complete initialization of {@link TreeLayoutData} can also be done in a single object
* initializer:
*
* <br><b>Using an object initializer for easy creation of TreeLayoutData</b>
* <pre>
* const layoutData = new TreeLayoutData({
*   nodeHalos: NodeHalo.create(15),
*   treeRoot: node => node === rootNode
* })
*
* graph.applyLayout(new TreeLayout(), layoutData)
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData Online Documentation}
*/
export class TreeLayoutData {
/**
Creates a new instance of {@link TreeLayoutData} which helps configuring {@link TreeLayout}.
@param [options] The parameters to pass.
@param [options.nodePlacers] The mapping from nodes to their {@link ITreeLayoutNodePlacer}. This option sets the {@link TreeLayoutData#nodePlacers} property on the created object.
@param [options.portAssignments] The mapping from nodes to their {@link ITreeLayoutPortAssignment}. This option sets the {@link TreeLayoutData#portAssignments} property on the created object.
@param [options.outEdgeComparers] The mapping from nodes to comparison functions used to sort the nodes' outgoing edges. This option sets the {@link TreeLayoutData#outEdgeComparers} property on the created object.
@param [options.nodeHalos] The mapping from nodes to their {@link NodeHalo}. This option sets the {@link TreeLayoutData#nodeHalos} property on the created object.
@param [options.sourcePortConstraints] A mapping from edges to their source {@link PortConstraint}. This option sets the {@link TreeLayoutData#sourcePortConstraints} property on the created object.
@param [options.targetPortConstraints] A mapping from edges to their target {@link PortConstraint}. This option sets the {@link TreeLayoutData#targetPortConstraints} property on the created object.
@param [options.assistantNodes] The collection of nodes the {@link AssistantNodePlacer} considers as assistants. This option sets the {@link TreeLayoutData#assistantNodes} property on the created object.
@param [options.delegatingNodePlacerPrimaryNodes] The collection of nodes the {@link DelegatingNodePlacer} places with its {@link DelegatingNodePlacer#primaryPlacer}. This option sets the {@link TreeLayoutData#delegatingNodePlacerPrimaryNodes} property on the created object.
@param [options.leftRightNodePlacerLeftNodes] The set of nodes that are placed on the left side of the bus. This option sets the {@link TreeLayoutData#leftRightNodePlacerLeftNodes} property on the created object.
@param [options.gridNodePlacerRowIndices] The mapping from nodes to the index of the row the {@link GridNodePlacer} shall place the node in. This option sets the {@link TreeLayoutData#gridNodePlacerRowIndices} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link TreeLayoutData#abortHandler} property on the created object.
@param [options.treeRoot] The mapping for marking the node that will be used as root node of the tree. This option sets the {@link TreeLayoutData#treeRoot} property on the created object.
@param [options.edgeLabelPreferredPlacement] The mapping that provides a {@link PreferredPlacementDescriptor} instance for edge {@link ILabel}s. This option sets the {@link TreeLayoutData#edgeLabelPreferredPlacement} property on the created object.
@param [options.compactNodePlacerStrategyMementos] A mapper from nodes to a strategy memento object for the {@link CompactNodePlacer}. This option sets the {@link TreeLayoutData#compactNodePlacerStrategyMementos} property on the created object.
@param [options.criticalEdgePriorities] A mapping from edges to their priority to be a 'critical' edge. This option sets the {@link TreeLayoutData#criticalEdgePriorities} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData Online Documentation}*/
constructor(options?: {
nodePlacers?: ItemMapping<INode, ITreeLayoutNodePlacer> | ItemMappingConvertible<INode, ITreeLayoutNodePlacer>;
portAssignments?: ItemMapping<INode, ITreeLayoutPortAssignment> | ItemMappingConvertible<INode, ITreeLayoutPortAssignment>;
outEdgeComparers?: ItemMapping<INode, (x: IEdge, y: IEdge) => number> | ItemMappingConvertible<INode, (x: IEdge, y: IEdge) => number>;
nodeHalos?: ItemMapping<INode, NodeHalo> | ItemMappingConvertible<INode, NodeHalo>;
sourcePortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
targetPortConstraints?: ItemMapping<IEdge, PortConstraint> | ItemMappingConvertible<IEdge, PortConstraint>;
assistantNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
delegatingNodePlacerPrimaryNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
leftRightNodePlacerLeftNodes?: ItemCollection<INode> | ItemCollectionConvertible<INode>;
gridNodePlacerRowIndices?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
abortHandler?: AbortHandler | null;
treeRoot?: SingleItem<INode> | SingleItemConvertible<INode>;
edgeLabelPreferredPlacement?: ItemMapping<ILabel, PreferredPlacementDescriptor> | ItemMappingConvertible<ILabel, PreferredPlacementDescriptor>;
compactNodePlacerStrategyMementos?: IMapper<INode, any> | null | IMapperConvertible<INode, any>;
criticalEdgePriorities?: ItemMapping<IEdge, number> | ItemMappingConvertible<IEdge, number>;
});
/**
* Gets or sets the mapping from nodes to their {@link ITreeLayoutNodePlacer}.
* If a node is mapped to <code>null</code>, then the {@link TreeLayout#defaultNodePlacer default node placer} applies for that node.
*
* Depending on the customization needed, {@link ItemMapping} has several options that work well in different situations. To change the
* default only for some nodes that are known beforehand, it's often easiest to just use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to change node placers for certain subtrees</b>
* <pre>
* // Arrange node1's children below the node and to the right
* layoutData.nodePlacers.mapper.set(
*   node1,
*   new DefaultNodePlacer(ChildPlacement.VERTICAL_TO_RIGHT, RootAlignment.LEADING, 10, 10)
* )
* // Arrange node2's children as compactly as possible via CompactNodePlacer
* layoutData.nodePlacers.mapper.set(node2, new CompactNodePlacer())
* </pre>
*
* In case the desired configuration can readily be inferred from each node, the most convenient option usually is to use a
* delegate:
*
* <br><b>Using a delegate to determine a node placer for each subtree</b>
* <pre>
* layoutData.nodePlacers.delegate = node => {
*   // If the node has too many children, switch to CompactNodePlacer
*   const successors = graph.successors(node).toList()
*   if (successors.size > 10) {
*     return new CompactNodePlacer()
*   }
*   // Otherwise if there are assistant children, we use AssistantNodePlacer
*   if (successors.some(n => n.tag.isAssistant)) {
*     return new AssistantNodePlacer()
*   }
*   // In all other cases use the default configured on TreeLayout
*   return null
* }
* </pre>
*
* @see TreeLayout#defaultNodePlacer
* @see TreeLayout#NODE_PLACER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23nodePlacers Online Documentation}
*/
nodePlacers: ItemMapping<INode, ITreeLayoutNodePlacer>;
/**
* Gets or sets the mapping from nodes to their {@link ITreeLayoutPortAssignment}.
* Port assignments define where ports of incident edges are placed.
*
* Depending on the customization needed, {@link ItemMapping} has several options that work well in different situations. To change the
* default only for some nodes that are known beforehand, it's often easiest to just use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to change port assignments for certain nodes</b>
* <pre>
* // Place ports at node1 at the center of the node
* layoutData.portAssignments.mapper.set(
*   node1,
*   new DefaultTreeLayoutPortAssignment(TreeLayoutPortAssignmentMode.NONE)
* )
* // Distribute ports on node2 on the bottom border of the node
* layoutData.portAssignments.mapper.set(
*   node2,
*   new DefaultTreeLayoutPortAssignment(TreeLayoutPortAssignmentMode.DISTRIBUTED_SOUTH)
* )
* </pre>
*
* In case the desired configuration can readily be inferred from each node, the most convenient option usually is to use a
* delegate:
*
* <br><b>Using a delegate to determine the port assignment for each node</b>
* <pre>
* layoutData.portAssignments.delegate = node => {
*   const customData = node.tag
*   if (customData.portPlacement === 'left') {
*     return new DefaultTreeLayoutPortAssignment(TreeLayoutPortAssignmentMode.DISTRIBUTED_WEST)
*   }
*   if (customData.portPlacement === 'right') {
*     return new DefaultTreeLayoutPortAssignment(TreeLayoutPortAssignmentMode.DISTRIBUTED_EAST)
*   }
*   // Fall back to TreeLayout.defaultPortAssignment for other cases
*   return null
* }
* </pre>
*
* @see TreeLayout#defaultPortAssignment
* @see TreeLayout#PORT_ASSIGNMENT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23portAssignments Online Documentation}
*/
portAssignments: ItemMapping<INode, ITreeLayoutPortAssignment>;
/**
Gets or sets the mapping from nodes to comparison functions used to sort the nodes' outgoing edges.
<h2>Note</h2>
When {@link TreeLayoutData#outEdgeComparers} is used with type conversion, functions will be converted to the
{@link ItemMapping#delegate}. That means that a compare-function cannot be assigned directly.
@see TreeLayout#OUT_EDGE_COMPARER_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23outEdgeComparers Online Documentation}*/
outEdgeComparers: ItemMapping<INode, Function>;
/**
* Gets or sets the mapping from nodes to their {@link NodeHalo}.
* {@link NodeHalo}s allow to reserve space around nodes.
*
* The easiest option is to reserve the same space around all nodes, by setting a constant {@link NodeHalo}:
*
* <br><b>Using constant space around all nodes</b>
* <pre>
* layoutData.nodeHalos.constant = NodeHalo.create(20)
* </pre>
*
* Handling only certain nodes differently can be done easily by using the {@link ItemMapping#mapper} property:
*
* <br><b>Using a mapper to set halos for certain nodes</b>
* <pre>
* // node1 only reserves space above and below
* layoutData.nodeHalos.mapper.set(node1, NodeHalo.create(20, 0, 0, 10))
* // node2 has space all around
* layoutData.nodeHalos.mapper.set(node2, NodeHalo.create(25))
* // all other nodes don't get extra space
* </pre>
*
* In cases where the {@link NodeHalo} for each node can be determined by looking at the node itself it's often
* easier to just set a delegate instead of preparing a mapper:
*
* <br><b>Using a delegate to determine halos for all nodes</b>
* <pre>
* // Retrieve the space around the node from its Tag property
* layoutData.nodeHalos.delegate = node => NodeHalo.create(parseFloat(node.tag))
* </pre>
*
* @see NodeHalo#NODE_HALO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23nodeHalos Online Documentation}
*/
nodeHalos: ItemMapping<INode, NodeHalo>;
/**
* Gets or sets a mapping from edges to their source {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its source node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All source ports should be on the bottom side
* layoutData.sourcePortConstraints.constant = PortConstraint.create(PortSide.SOUTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should leave the source node on the left side, while keeping its exact port position (strong port constraint)
* layoutData.sourcePortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.WEST, true))
* // edge2 should leave the source node downwards, but the port may be placed anywhere along that node border
* layoutData.sourcePortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.SOUTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Source ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.sourcePortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23sourcePortConstraints Online Documentation}
*/
sourcePortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets a mapping from edges to their target {@link PortConstraint}.
* Port constraints allow to define where an edge attaches to its target node and can either restrict that to one of the
* node's sides, or to a fixed port position.
*
* If all edges should exit their source node on the same side, you can simply set a constant constraint for all edges:
*
* <br><b>Setting the same port constraint for all edges</b>
* <pre>
* // All target ports should be on the top side
* layoutData.targetPortConstraints.constant = PortConstraint.create(PortSide.NORTH)
* </pre>
*
* To change the constraints for individual edges, it's usually easiest to use the {@link ItemMapping#mapper}:
*
* <br><b>Using a mapper to set different port constraints for certain edges</b>
* <pre>
* // edge1 should enter the target node on the right side, while keeping its exact port position (strong port
* // constraint)
* layoutData.targetPortConstraints.mapper.set(edge1, PortConstraint.create(PortSide.EAST, true))
* // edge2 should enter the target node from the top, but the port may be placed anywhere along that node border
* layoutData.targetPortConstraints.mapper.set(edge2, PortConstraint.create(PortSide.NORTH))
* </pre>
*
* If a {@link PortConstraint} can readily be created from an edge, using a delegate is often easier:
*
* <br><b>Using a delegate to set a different port constraint for each edge</b>
* <pre>
* // Target ports will be placed on the right side and use a property in the edge's tag
* // to determine whether ports will stay fixed at their position.
* layoutData.targetPortConstraints.delegate = edge =>
*   PortConstraint.create(PortSide.EAST, edge.tag.fixPorts)
* </pre>
*
* @see PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23targetPortConstraints Online Documentation}
*/
targetPortConstraints: ItemMapping<IEdge, PortConstraint>;
/**
* Gets or sets the collection of nodes the {@link AssistantNodePlacer} considers as assistants.
* This collection is only considered if {@link TreeLayoutData#nodePlacers} returns an instance of {@link AssistantNodePlacer} or {@link CompactNodePlacer}
* for any node.
*
* Specifying which nodes to lay out as assistant nodes can be done in various ways, depending on which option is more
* convenient at the moment. For example, if you already have a collection or {@link IEnumerable} somewhere that contains all nodes that
* should be treated as assistant nodes, then the easiest option usually is to use the {@link ItemCollection#source} property:
*
* <br><b>Using a collection for assistant nodes</b>
* <pre>
* layoutData.assistantNodes.source = graphComponent.selection.selectedNodes
* </pre>
*
* In some cases only some nodes may need to be directly set as assistant nodes, but are not already in a collection, in
* which case the {@link ItemCollection#items} property can be used directly to add them:
*
* <br><b>Adding individual nodes as assistant nodes</b>
* <pre>
* const layoutData = new TreeLayoutData()
* layoutData.assistantNodes.items.add(assistantNode1)
* layoutData.assistantNodes.items.add(assistantNode2)
* </pre>
*
* In cases where it's directly apparent from a node whether to lay it out as an assistant node, the best option usually is
* to use a delegate:
*
* <br><b>Using a delegate to dynamically determine whether a node is an assistant node</b>
* <pre>
* layoutData.assistantNodes.delegate = node => node.tag.isAssistant
* </pre>
*
* @see AssistantNodePlacer#ASSISTANT_NODE_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23assistantNodes Online Documentation}
*/
assistantNodes: ItemCollection<INode>;
/**
Gets or sets the collection of nodes the {@link DelegatingNodePlacer} places with its {@link DelegatingNodePlacer#primaryPlacer}.
This collection is only considered if {@link TreeLayoutData#nodePlacers} returns any {@link DelegatingNodePlacer} .
@see DelegatingNodePlacer#PRIMARY_NODES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23delegatingNodePlacerPrimaryNodes Online Documentation}*/
delegatingNodePlacerPrimaryNodes: ItemCollection<INode>;
/**
* Gets or sets the set of nodes that are placed on the left side of the bus.
* Nodes that appear in this collection are placed left of the bus, those that do not will be placed on the right.
*
* <h2>Note</h2>
*
* This collection is only considered if {@link TreeLayoutData#nodePlacers} returns a {@link LeftRightNodePlacer}.
*
* <h2>Note</h2>
*
* If this property is not used, then nodes are placed alternating left and right.
*
* When customizing left/right placement for nodes it's common to have some sort of telling from the node where it should
* be placed. The simplest way to use this would be the {@link ItemMapping#delegate}:
*
* <br><b>Assigning left/right placement of nodes via a delegate</b>
* <pre>
* layoutData.leftRightPlacersLeftNodes.delegate = node => node.tag.placement === 'left'
* </pre>
*
* In other cases, the {@link ItemMapping#mapper} option allows more flexibility over its use:
*
* <br><b>Assigning left/right placement of nodes via a mapper</b>
* <pre>
* // Place up to three nodes to the left, and the others to the right
* for (const node of graph.nodes) {
*   let i = 0
*   for (const n of graph.successors(node).takeWhile(() => i++ < 3)) {
*     layoutData.leftRightPlacersLeftNodes.mapper.set(n, true)
*   }
*   // Placement to the right is the default, so nothing else is necessary here
* }
* </pre>
*
* @see LeftRightNodePlacer#LEFT_RIGHT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23leftRightNodePlacerLeftNodes Online Documentation}
*/
leftRightNodePlacerLeftNodes: ItemCollection<INode>;
/**
Gets or sets the mapping from nodes to the index of the row the {@link GridNodePlacer} shall place the node in.
This collection is only considered if {@link TreeLayoutData#nodePlacers} returns any {@link GridNodePlacer} .
@see GridNodePlacer#ROW_INDEX_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23gridNodePlacerRowIndices Online Documentation}*/
gridNodePlacerRowIndices: ItemMapping<INode, number>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
/**
* Gets or sets the mapping for marking the node that will be used as root node of the tree.
* The simplest way to set a custom root node for the layout, if you already have an {@link INode} for that, would be
* to use the {@link SingleItem#item} to specify it as the root:
*
* <br><b>Specifying a custom root node directly</b>
* <pre>
* layoutData.treeRoot.item = root
* </pre>
*
* In cases where it's easier to look at an {@link INode} to determine whether it should be the root node or not, it's
* often most convenient to use the {@link SingleItem#delegate} property:
*
* <br><b>Specifying a custom root node via a delegate</b>
* <pre>
* // This is equivalent to the declaration above
* layoutData.treeRoot = node => node === root
* // Or use a different way of choosing the root node, e.g. by taking the node whose label
* // says 'Root'
* layoutData.treeRoot = node => node.labels.get(0).text === 'Root'
* </pre>
*
* Sometimes there might also be a collection that usually holds just the single node that should be the root, in which
* case {@link SingleItem#source} can be useful:
*
* <br><b>Specifying a custom root node from an IEnumerable</b>
* <pre>
* // Use the first selected node as the root
* // This will be evaluated anew each time the layout is applied
* layoutData.treeRoot.source = graphComponent.selection.selectedNodes
* </pre>
*
* @see TreeLayout#SELECTED_ROOT_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23treeRoot Online Documentation}
*/
treeRoot: SingleItem<INode>;
/**
* Gets or sets the mapping that provides a {@link PreferredPlacementDescriptor} instance for edge
* {@link ILabel}s.
* Depending on how much customization is needed, some ways of setting {@link PreferredPlacementDescriptor}s are more
* convenient than others. For example, to set the same descriptor for all labels, you can just use the {@link ItemMapping#constant} property:
*
* <br><b>Setting the same PreferredPlacementDescriptor for all labels</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.constant = new PreferredPlacementDescriptor({
*   // Place labels along the edge
*   angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*   angle: 0,
*   // ... on either side
*   sideOfEdge: LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE,
*   // ... with a bit of distance to the edge
*   distanceToEdge: 5
* })
* </pre>
*
* If some labels should use custom placement or this has to be configured ahead of time, you can use the {@link ItemMapping#mapper} instead:
*
* <br><b>Customizing the placement of certain labels individually via a mapper</b>
* <pre>
* // Place label1 orthogonal to the edge anywhere on it
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label1,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.ANYWHERE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: Math.PI / 2
*   })
* )
* // Place label2 near the edge's source on either side of it, and make it parallel to the edge
* layoutData.edgeLabelPreferredPlacement.mapper.set(
*   label2,
*   new PreferredPlacementDescriptor({
*     placeAlongEdge: LabelPlacements.AT_SOURCE,
*     sideOfEdge: LabelPlacements.RIGHT_OF_EDGE | LabelPlacements.LEFT_OF_EDGE,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     angle: 0
*   })
* )
* </pre>
*
* When the preferred placement can be inferred from the label itself, a delegate is usually the easiest choice:
*
* <br><b>Configuring preferred placement for all labels with a delegate</b>
* <pre>
* layoutData.edgeLabelPreferredPlacement.delegate = label => {
*   const customData = label.tag
*   const preferredPlacementDescriptor = new PreferredPlacementDescriptor({
*     angle: 0,
*     angleReference: LabelAngleReferences.RELATIVE_TO_EDGE_FLOW,
*     // If the tag says to place the label in the center, put it in the center parallel to the edge's path
*     // All other labels can be placed anywhere, but on the side of the edge.
*     placeAlongEdge: customData.placeInCenter
*       ? LabelPlacements.AT_CENTER
*       : LabelPlacements.ANYWHERE,
*     sideOfEdge: customData.placeInCenter
*       ? LabelPlacements.ON_EDGE
*       : LabelPlacements.LEFT_OF_EDGE | LabelPlacements.RIGHT_OF_EDGE
*   })
* }
* </pre>
*
* @see PreferredPlacementDescriptor
* @see LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23edgeLabelPreferredPlacement Online Documentation}
*/
edgeLabelPreferredPlacement: ItemMapping<ILabel, PreferredPlacementDescriptor>;
/**
* Gets or sets a mapper from nodes to a strategy memento object for the {@link CompactNodePlacer}.
* This property has an effect only if {@link CompactNodePlacer} is selected as {@link TreeLayoutData#nodePlacers node placer} for at least one subtree of
* the input graph. Then, given that this property is set, the node placer stores the applied placement strategy for the
* nodes' children and keeps it in subsequent layout runs. This is useful to maintain similar layout styles over subsequent
* runs. Otherwise, even for small changes in the graph structure, the results produced by {@link CompactNodePlacer}
* can drastically change.
*
* The actual values stored in this mapper are opaque objects only intended for internal use by
* {@link CompactNodePlacer}.
*
* The following example shows how to set this property so that {@link CompactNodePlacer} remembers previous
* arrangements of subtrees:
*
* <pre>
* layoutData.compactNodePlacerStrategyMementos = new Mapper()
* </pre>
*
* @see CompactNodePlacer#STRATEGY_MEMENTO_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23compactNodePlacerStrategyMementos Online Documentation}
*/
compactNodePlacerStrategyMementos: IMapper<INode, any> | null;
/**
* Gets or sets a mapping from edges to their priority to be a 'critical' edge.
* The layout tries to align each node pair that is connected by a critical edge (integer value > 0).
*
* Critical edges highlight different edge paths that are relevant for a user. The layout tries to align each node pair
* that is connected by a critical edge. Conflicts between different critical edges are always resolved in favor of the
* higher priority.
*
* <h2>Note</h2>
*
* Critical edges will always align the centers of source and target node, thus replacing the current root alignment of the
* {@link ITreeLayoutNodePlacer}.
*
* <h2>Note</h2>
*
* The critical edge may not be straight if subtrees are rotated or port constraints are assigned.
*
* <h2>Note</h2>
*
* This feature is only supported by the following {@link ITreeLayoutNodePlacer}s:
*
* <ul>
* <li>{@link DefaultNodePlacer}</li>
* <li>{@link SimpleNodePlacer}</li>
* <li>{@link LayeredNodePlacer}</li>
* <li>{@link DoubleLineNodePlacer}</li>
* <li>{@link DendrogramNodePlacer}</li>
* </ul>
* Critical edges are often determined by some external data source and are often part of a larger path. The following
* example shows how two different such paths can be made 'critical' with different priorities:
*
* <br><b>Configuring critical edges via a mapper</b>
* <pre>
* // Set two different critical edge paths with different priorities
* for (const edge of path1) {
*   layoutData.criticalEdgePriorities.mapper.set(edge, 5)
* }
* // Lower priority for the second path, which then may deviate from a straight line
* for (const edge of path2) {
*   layoutData.criticalEdgePriorities.mapper.set(edge, 2)
* }
* </pre>
*
* Alternatively, a delegate may be used as well, if determining edge criticality is easier when looking at the edge
* itself:
*
* <br><b>Configuring critical edges via a delegate</b>
* <pre>
* // Assume that the edge's tag holds an instance that has an 'IsCritical' property
* layoutData.criticalEdgePriorities = edge => (edge.tag.isCritical ? 1 : 0)
* </pre>
*
* @see TreeLayout#CRITICAL_EDGE_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeLayoutData%23criticalEdgePriorities Online Documentation}
*/
criticalEdgePriorities: ItemMapping<IEdge, number>;
static $class: Class;
private $_TreeLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeLayoutData;
}
export interface TreeMapLayoutData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link TreeMapLayout}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayoutData Online Documentation}*/
export class TreeMapLayoutData {
/**
Creates a new instance of {@link TreeMapLayoutData} which helps configuring {@link TreeMapLayout}.
@param [options] The parameters to pass.
@param [options.nodeWeights] The mapping from nodes to the weight value used for the tree map layout. This option sets the {@link TreeMapLayoutData#nodeWeights} property on the created object.
@param [options.abortHandler] The {@link AbortHandler} used during the layout. This option sets the {@link TreeMapLayoutData#abortHandler} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayoutData Online Documentation}*/
constructor(options?: {
nodeWeights?: ItemMapping<INode, number> | ItemMappingConvertible<INode, number>;
abortHandler?: AbortHandler | null;
});
/**
Gets or sets the mapping from nodes to the weight value used for the tree map layout.
The weight of a node defines the area assigned to the node inside the available parent area with respect to the other
child nodes, i.e., the area of the node rectangle in the resulting layout is proportional to a the weight. Child nodes
at the same hierarchy level with equal weight get the same area, they do not necessarily get equal width and height
values.
All leaf nodes (nodes without further children) need to have a weight greater than
<code>0</code>. Non-leaf nodes do not need to have a weight. They will get the sum of the weight of their children. If they have a
weight on their own, some free space is generated inside the representative rectangle (i.e. the parent node is larger
than the combined children).
<h2>Note</h2>
Due to insets specified at parent nodes of different hierarchies and the inter-node padding defined by
{@link TreeMapLayout#spacing}, nodes that have the same actual weight can get slightly different sizes.
@see TreeMapLayout#NODE_WEIGHT_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayoutData%23nodeWeights Online Documentation}*/
nodeWeights: ItemMapping<INode, number>;
/**
* Gets or sets the {@link AbortHandler} used during the layout.
* An {@link AbortHandler} can be used to gracefully stop or cancel a running layout and offers options for
* automatically doing so after a predetermined time.
*
* An {@link AbortHandler} configured or set here overrides the one on {@link LayoutExecutor}.
*
* The most common use case would be to just configure the {@link AbortHandler} here, e.g. to set timeouts for a
* graceful stop or canceling the running layout:
*
* <pre>
* layoutData.abortHandler.stopDuration = TimeSpan.fromSeconds(10)
* layoutData.abortHandler.cancelDuration = TimeSpan.fromSeconds(30)
* </pre>
*
* If there's already an {@link AbortHandler} instance that's pre-configured or will be used in a different place
* to, e.g., cancel the layout when the user presses a button, you can also set one explicitly:
*
* <pre>
* layoutData.abortHandler = abortHandler
* </pre>
*
* @see LayoutExecutor#abortHandler
* @see AbortHandler#ABORT_HANDLER_DP_KEY
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TreeMapLayoutData%23abortHandler Online Documentation}
*/
abortHandler: AbortHandler | null;
static $class: Class;
private $_TreeMapLayoutData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeMapLayoutData;
}
export interface TreeReductionStageData extends LayoutData {
/**
@inheritdoc*/
apply(adapter: LayoutGraphAdapter, layout: ILayoutAlgorithm, layoutGraph: CopiedLayoutGraph): void;
} /**
Specifies custom data for the {@link TreeReductionStage}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStageData Online Documentation}*/
export class TreeReductionStageData {
/**
Creates a new instance of {@link TreeReductionStageData} which helps configuring {@link TreeReductionStage}.
@param [options] The parameters to pass.
@param [options.nonTreeEdges] The collection of edges explicitly marked as not belonging to a tree. This option sets the {@link TreeReductionStageData#nonTreeEdges} property on the created object.
@param [options.edgeBundleDescriptors] The mapping of edges to their {@link EdgeBundleDescriptor}. This option sets the {@link TreeReductionStageData#edgeBundleDescriptors} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStageData Online Documentation}*/
constructor(options?: {
nonTreeEdges?: ItemCollection<IEdge> | ItemCollectionConvertible<IEdge>;
edgeBundleDescriptors?: ItemMapping<IEdge, EdgeBundleDescriptor> | ItemMappingConvertible<IEdge, EdgeBundleDescriptor>;
});
/**
Gets or sets the collection of edges explicitly marked as not belonging to a tree.
@see TreeReductionStage#NON_TREE_EDGES_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStageData%23nonTreeEdges Online Documentation}*/
nonTreeEdges: ItemCollection<IEdge>;
/**
Gets or sets the mapping of edges to their {@link EdgeBundleDescriptor}.
Bundling together multiple edges means that their common parts are to some degree merged into a bundled part. At the
source and target point, the edges are again clearly split.
If an edge is mapped to <code>null</code>, the {@link EdgeBundling#defaultBundleDescriptor default descriptor} is used.
@see EdgeBundleDescriptor
@see TreeReductionStage#edgeBundling
@see {@link https://docs.yworks.com/yfileshtml/#/api/TreeReductionStageData%23edgeBundleDescriptors Online Documentation}*/
edgeBundleDescriptors: ItemMapping<IEdge, EdgeBundleDescriptor>;
static $class: Class;
private $_TreeReductionStageData: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is TreeReductionStageData;
}
export interface YGraphAdapter extends YObject {
}
/**
Converts an {@link IGraph} instance into a {@link Graph} instance.
This class creates a structural copy of a given {@link IGraph} and provides methods to convert items from one graph
instance to their corresponding item in the other graph instance.
The main use case of this class is executing one of the analysis algorithms on an {@link IGraph}, for example
{@link ShortestPathAlgorithm}, {@link CycleAlgorithm}, and {@link CentralityAlgorithm}. However, for most use cases
the algorithms that work directly on the {@link IGraph} instance are preferred.
For executing a layout algorithm, {@link LayoutExecutor} would be preferred or one of the ApplyLayout and {@link GraphComponent#morphLayout}
methods. {@link LayoutGraphAdapter} is the exact equivalent of this type for layout algorithms if low-level
control over the conversion to {@link LayoutGraph} is needed.
Edge to edge connections are not supported. If the original {@link IGraph} contains such connections, an {@link Exception InvalidGraphStructureError}
is thrown.
@see LayoutGraphAdapter
@see LayoutExecutor
@see BipartitionAlgorithm
@see CentralityAlgorithm
@see CycleAlgorithm
@see GraphChecker
@see GraphConnectivity
@see GroupAlgorithm
@see IndependentSetAlgorithm
@see NetworkFlowAlgorithm
@see NodeOrderAlgorithm
@see PathAlgorithm
@see RankAssignmentAlgorithm
@see ShortestPathAlgorithm
@see SpanningTreeAlgorithm
@see TransitivityAlgorithm
@see TreeAlgorithm
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter Online Documentation}*/
export class YGraphAdapter {
/**
Initializes a new instance of the {@link YGraphAdapter} class.
This will create a copy of the <code>originalGraph</code> that has the same structure and makes it available via the {@link YGraphAdapter#yGraph} property.
@param [subgraphNodes=null] The (sub) set of nodes to use to build the graph. If omitted, all {@link IGraph#nodes nodes} will be used. Providing nodes which are not part
of the <code>originalGraph</code> will result in an exception.
@param [subgraphEdges=null] The (sub) set of edges to use to build the graph. If omitted, all {@link IGraph#edges edges} will be used. Providing edges which are not part
of the <code>originalGraph</code> will result in an exception.
@param originalGraph The original graph whose structure will be copied.
@throws {Exception} <b>(InvalidGraphStructureError)</b> If <code>originalGraph</code> contains edge to edge connections.
@throws {Exception} <b>(ArgumentError)</b> If <code>subgraphNodes</code> or <code>subgraphEdges</code> contain elements which are not contained in the <code>originalGraph</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter Online Documentation}*/
constructor(originalGraph: IGraph, subgraphNodes?: IEnumerable<INode> | null | IEnumerableConvertible<INode>, subgraphEdges?: IEnumerable<IEdge> | null | IEnumerableConvertible<IEdge>);
/**
Initializes a new instance of the {@link YGraphAdapter} class.
This will create a copy of the <code>originalGraph</code> that has the same structure and makes it available via the {@link YGraphAdapter#yGraph} property.
@param options The parameters to pass.
@param [options.subgraphNodes=null] The (sub) set of nodes to use to build the graph. If omitted, all {@link IGraph#nodes nodes} will be used. Providing nodes which are not part
of the <code>originalGraph</code> will result in an exception.
@param [options.subgraphEdges=null] The (sub) set of edges to use to build the graph. If omitted, all {@link IGraph#edges edges} will be used. Providing edges which are not part
of the <code>originalGraph</code> will result in an exception.
@param options.originalGraph The original graph whose structure will be copied.
@throws {Exception} <b>(InvalidGraphStructureError)</b> If <code>originalGraph</code> contains edge to edge connections.
@throws {Exception} <b>(ArgumentError)</b> If <code>subgraphNodes</code> or <code>subgraphEdges</code> contain elements which are not contained in the <code>originalGraph</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter Online Documentation}*/
constructor(options: {
originalGraph: IGraph;
subgraphNodes?: IEnumerable<INode> | null | IEnumerableConvertible<INode>;
subgraphEdges?: IEnumerable<IEdge> | null | IEnumerableConvertible<IEdge>;
});
/**
Creates an {@link IDataMap} that delegates to the provided {@link IMapper} when queried for items in the {@link YGraphAdapter#yGraph}.
This method is useful where algorithms require {@link IDataMap}s as input and that input can be obtained from
{@link IMapper} instances that provide and accept values for the {@link YGraphAdapter#getOriginalNode original nodes} and {@link YGraphAdapter#getOriginalEdge edges}.
Type parameter <code>K</code>: The type of the keys in the mapper.
Type parameter <code>V</code>: The type of the values in the mapper.
@param valueType The type of the values in the mapper.
@param keyType The type of the keys in the mapper.
@param mapper The mapper to direct requests to.
@returns An {@link IDataMap} that will delegate the functionality to the <code>mapper</code>.
@see YGraphAdapter#createDataProvider
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createDataMap(Class%2CClass%2CIMapper%3CK%2CV%3E) Online Documentation}*/
createDataMap<K, V>(keyType: Class, valueType: Class, mapper: IMapper<K, V> | IMapperConvertible<K, V>): IDataMap;
/**
Creates an {@link IDataProvider} that delegates to the provided delegate when queried for items in the {@link YGraphAdapter#yGraph}.
This method is useful where algorithms require {@link IDataProvider}s as input and that input can be through a
dynamic callback that provides the values for the {@link YGraphAdapter#getOriginalNode original nodes} and {@link YGraphAdapter#getOriginalEdge edges}.
Type parameter <code>K</code>: The type of the keys in the mapper.
Type parameter <code>V</code>: The type of the values in the mapper.
@param valueType The type of the values in the mapper.
@param keyType The type of the keys in the mapper.
@param getter The mapper delegate to direct requests to.
@returns A DataProvider that will delegate requests to the <code>getter</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createDataProvider(Class%2CClass%2Cfunction(K)%3AV) Online Documentation}*/
createDataProvider<K, V>(keyType: Class, valueType: Class, getter: (key: K | null) => V): IDataProvider;
/**
Creates an {@link IDataProvider} that delegates to the provided {@link IMapper} when queried for items in the {@link YGraphAdapter#yGraph}.
This method is useful where algorithms require {@link IDataProvider}s as input and that input can be obtained
from {@link IMapper} instances that provide the values for the {@link YGraphAdapter#getOriginalNode original nodes} and {@link YGraphAdapter#getOriginalEdge edges}.
Type parameter <code>K</code>: The type of the keys in the mapper.
Type parameter <code>V</code>: The type of the values in the mapper.
@param valueType The type of the values in the mapper.
@param keyType The type of the keys in the mapper.
@param mapper The mapper to direct requests to.
@returns A DataProvider that will delegate requests to the <code>mapper</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createDataProvider(Class%2CClass%2CIMapper%3CK%2CV%3E) Online Documentation}*/
createDataProvider<K, V>(keyType: Class, valueType: Class, mapper: IMapper<K, V> | IMapperConvertible<K, V>): IDataProvider;
/**
Creates an {@link IEnumerable enumerable of edges} that maps the edges from the {@link EdgeList} to their {@link YGraphAdapter#getOriginalEdge original} ones.
This method is useful where algorithms yield {@link EdgeList} instances as results and the result needs to be
interpreted in terms of the {@link YGraphAdapter#originalGraph}.
@param edges The edge list.
@returns An enumerable that contains the {@link YGraphAdapter#getOriginalEdge original} edges for the ones in the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createEdgeEnumerable(EdgeList) Online Documentation}*/
createEdgeEnumerable(edges: EdgeList): IListEnumerable<IEdge>;
/**
Creates an {@link EdgeList} that maps the edges from the {@link IEnumerable} to their {@link YGraphAdapter#getCopiedEdge copied} ones.
This method is useful where algorithms require {@link EdgeList} instances as input and those edges are
available as {@link IEnumerable}s in the {@link YGraphAdapter#originalGraph}.
@param edges The edges from the {@link YGraphAdapter#originalGraph}.
@returns An {@link EdgeList} that contains the {@link YGraphAdapter#getCopiedEdge copied} edges for the ones in the enumerable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createEdgeList(IEnumerable%3CIEdge%3E) Online Documentation}*/
createEdgeList(edges: IEnumerable<IEdge> | IEnumerableConvertible<IEdge>): EdgeList;
/**
Creates an {@link IEdgeMap} that delegates to the provided delegate when queried for {@link Edge}s
in the {@link YGraphAdapter#yGraph}.
This method is useful where algorithms require {@link IEdgeMap}s as input and this can be mapped to a
corresponding dynamic callback that provides the values for the {@link YGraphAdapter#getOriginalEdge original edges}.
Type parameter <code>V</code>: The type of the values in the mapper.
@param getter The mapper delegate to direct read requests to.
@returns An {@link INodeMap} that will delegate reads to the <code>getter</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createEdgeMap(function(IEdge)%3AV) Online Documentation}*/
createEdgeMap<V>(getter: (key: IEdge | null) => V): IEdgeMap;
/**
Creates an {@link IEdgeMap} that delegates to the provided {@link IMapper} when queried for {@link Edge}s in the {@link YGraphAdapter#yGraph}.
This method is useful where algorithms require {@link IEdgeMap}s as input and this can be mapped to
corresponding {@link IMapper} instances that provide and accept the values for the {@link YGraphAdapter#getOriginalEdge original edges}.
Type parameter <code>V</code>: The type of the values in the mapper.
@param mapper The mapper to direct requests to.
@returns An {@link IEdgeMap} that will delegate reads and writes to the <code>mapper</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createEdgeMap(IMapper%3CIEdge%2CV%3E) Online Documentation}*/
createEdgeMap<V>(mapper: IMapper<IEdge, V> | IMapperConvertible<IEdge, V>): IEdgeMap;
/**
Creates an {@link IMapper} that delegates to the provided {@link IEdgeMap} when queried for items in the
{@link IGraph}.
This method is useful where algorithms yield {@link IDataProvider}s and their values should be provided using
{@link IMapper} instances that use the {@link YGraphAdapter#getOriginalEdge original edges}.
<b>Warning:</b> Currently, the names of the methods {@link YGraphAdapter#createNodeMapper} and {@link YGraphAdapter#createEdgeMapper}
are interchanged due to an error. To create a mapper for nodes call <code>createEdgeMap</code>, and to create a mapper for edges call
<code>createNodeMap</code>. This will be corrected in the next major release.
Type parameter <code>T</code>: The type of the values in the mapper.
@param edgeMap The map to direct requests to.
@returns A mapper that will delegate requests to the <code>edgeMap</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createEdgeMapper(IEdgeMap) Online Documentation}*/
createEdgeMapper<T>(edgeMap: IEdgeMap): IMapper<IEdge, T>;
/**
Creates an {@link IMapper} that delegates to the provided {@link IDataProvider} when queried for items in the
{@link IGraph}.
This method is useful where algorithms yield {@link IDataProvider}s and their values should be provided using
{@link IMapper} instances that use the {@link YGraphAdapter#getOriginalEdge original edges} and the {@link YGraphAdapter#getOriginalNode original nodes}.
Type parameter <code>T</code>: The type of the values in the mapper.
@param dataProvider The provider to direct requests to.
@returns A mapper that will delegate requests to the <code>dataProvider</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createMapper(IDataProvider) Online Documentation}*/
createMapper<T>(dataProvider: IDataProvider): IMapper<IModelItem, T>;
/**
Creates an {@link IEnumerable enumerable of nodes} that maps the nodes from the {@link YNodeList} to their {@link YGraphAdapter#getOriginalNode original} ones.
This method is useful where algorithms yield {@link YNodeList} instances as results and the result needs to be
interpreted in terms of the {@link YGraphAdapter#originalGraph}.
@param nodes The node list.
@returns An enumerable that contains the {@link YGraphAdapter#getOriginalNode original} nodes for the ones in the list.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createNodeEnumerable(YNodeList) Online Documentation}*/
createNodeEnumerable(nodes: YNodeList): IListEnumerable<INode>;
/**
Creates a {@link YNodeList} that maps the nodes from the {@link IEnumerable} to their {@link YGraphAdapter#getCopiedNode copied} ones.
This method is useful where algorithms require {@link YNodeList} instances as input and those nodes are
available as {@link IEnumerable}s in the {@link YGraphAdapter#originalGraph}.
@param nodes The nodes from the {@link YGraphAdapter#originalGraph}.
@returns A {@link YNodeList} that contains the {@link YGraphAdapter#getCopiedNode copied} nodes for the ones in the enumerable.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createNodeList(IEnumerable%3CINode%3E) Online Documentation}*/
createNodeList(nodes: IEnumerable<INode> | IEnumerableConvertible<INode>): YNodeList;
/**
Creates an {@link INodeMap} that delegates to the provided delegate when queried for {@link YNode}s
in the {@link YGraphAdapter#yGraph}.
This method is useful where algorithms require {@link INodeMap}s as input and this can be mapped to a
corresponding dynamic callback that provides the values for the {@link YGraphAdapter#getOriginalNode original nodes}.
Type parameter <code>V</code>: The type of the values in the mapper.
@param getter The mapper delegate to direct read requests to.
@returns An {@link INodeMap} that will delegate reads to the <code>getter</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createNodeMap(function(INode)%3AV) Online Documentation}*/
createNodeMap<V>(getter: (key: INode | null) => V): INodeMap;
/**
Creates an {@link INodeMap} that delegates to the provided {@link IMapper} when queried for {@link YNode}s in the {@link YGraphAdapter#yGraph}.
This method is useful where algorithms require {@link INodeMap}s as input and this can be mapped to
corresponding {@link IMapper} instances that provide and accept the values for the {@link YGraphAdapter#getOriginalNode original nodes}.
Type parameter <code>V</code>: The type of the values in the mapper.
@param mapper The mapper to direct requests to.
@returns An {@link INodeMap} that will delegate reads and writes to the <code>mapper</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createNodeMap(IMapper%3CINode%2CV%3E) Online Documentation}*/
createNodeMap<V>(mapper: IMapper<INode, V> | IMapperConvertible<INode, V>): INodeMap;
/**
Creates an {@link IMapper} that delegates to the provided {@link INodeMap} when queried for items in the
{@link IGraph}.
This method is useful where algorithms yield {@link INodeMap}s and their values should be provided using {@link IMapper}
instances that use the {@link YGraphAdapter#getOriginalNode original nodes}.
<b>Warning:</b> Currently, the names of the methods {@link YGraphAdapter#createNodeMapper} and {@link YGraphAdapter#createEdgeMapper}
are interchanged due to an error. To create a mapper for nodes call <code>createEdgeMap</code>, and to create a mapper for edges call
<code>createNodeMap</code>. This will be corrected in the next major release.
Type parameter <code>T</code>: The type of the values in the mapper.
@param nodeMap The map to direct requests to.
@returns A mapper that will delegate requests to the <code>nodeMap</code>.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23createNodeMapper(INodeMap) Online Documentation}*/
createNodeMapper<T>(nodeMap: INodeMap): IMapper<INode, T>;
/**
Yields the edge that has been created as a copy in {@link YGraphAdapter#yGraph} to represent the given edge.
@param edge The edge for which the representation is returned.
@returns The edge that represents <code>edge</code> in {@link YGraphAdapter#yGraph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23getCopiedEdge(IEdge) Online Documentation}*/
getCopiedEdge(edge: IEdge): Edge | null;
/**
Yields the node that has been created as a copy in {@link YGraphAdapter#yGraph} to represent the given node.
@param node The node for which the representation is returned.
@returns The node that represents <code>node</code> in {@link YGraphAdapter#yGraph}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23getCopiedNode(INode) Online Documentation}*/
getCopiedNode(node: INode): YNode | null;
/**
Yields the original edge from the {@link YGraphAdapter#originalGraph original IGraph} that the given edge has been created for.
@param edge The edge for which the original instance will be returned.
@returns The original instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23getOriginalEdge(Edge) Online Documentation}*/
getOriginalEdge(edge: Edge): IEdge | null;
/**
Yields the original node from the {@link YGraphAdapter#originalGraph original IGraph} that the given node has been created for.
@param node The node for which the original instance will be returned.
@returns The original instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23getOriginalNode(YNode) Online Documentation}*/
getOriginalNode(node: YNode): INode | null;
/**
Gets the graph instance that is created during the constructor call.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23yGraph Online Documentation}*/
readonly yGraph: Graph;
/**
Gets the original graph instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YGraphAdapter%23originalGraph Online Documentation}*/
readonly originalGraph: IGraph;
static $class: Class;
private $_YGraphAdapter: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YGraphAdapter;
}
interface ClassConstructor<T> {
new (...base: any[]): T & YObject;
$class: Class;
}
/**
* Helper function to support extending classes and implementing (multiple) interfaces.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
* @example
*  class [name]
*  extends BaseClass([class-type], [interface type #1], ... , [interface type #n])
*  implements [interface type #1], ..., [interface type #n] {
*    ...
*  }
* @param base - zero or one class types followed by an arbitrary number of interface types.
*/
export function BaseClass(...base: any[]): ClassConstructor<YObject>;
/**
* Generic helper function to support extending classes and implementing (multiple) interfaces.
*
* Can be used to implement an arbitrary number of types by utilizing {@link https://www.typescriptlang.org/docs/handbook/advanced-types.html#intersection-types|intersection types}.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
*/
export function BaseClass<T>(...base: any[]): ClassConstructor<T>;
/**
* Generic helper function to support extending classes and implementing (multiple) interfaces.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
*/
export function BaseClass<T1, T2>(...base: any[]): ClassConstructor<T1 & T2>;
/**
* Generic helper function to support extending classes and implementing (multiple) interfaces.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
*/
export function BaseClass<T1, T2, T3>(...base: any[]): ClassConstructor<T1 & T2 & T3>;
/**
* Generic helper function to support extending classes and implementing (multiple) interfaces.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
*/
export function BaseClass<T1, T2, T3, T4>(...base: any[]): ClassConstructor<T1 & T2 & T3 & T4>;
/**
* Generic helper function to support extending classes and implementing (multiple) interfaces.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
*/
export function BaseClass<T1, T2, T3, T4, T5>(...base: any[]): ClassConstructor<T1 & T2 & T3 & T4 & T5>;
/**
* Generic helper function to support extending classes and implementing (multiple) interfaces.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
*/
export function BaseClass<T1, T2, T3, T4, T5, T6>(...base: any[]): ClassConstructor<T1 & T2 & T3 & T4 & T5 & T6>;
/**
* Generic helper function to support extending classes and implementing (multiple) interfaces.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
*/
export function BaseClass<T1, T2, T3, T4, T5, T6, T7>(...base: any[]): ClassConstructor<T1 & T2 & T3 & T4 & T5 & T6 & T7>;
/**
* Generic helper function to support extending classes and implementing (multiple) interfaces.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
*/
export function BaseClass<T1, T2, T3, T4, T5, T6, T7, T8>(...base: any[]): ClassConstructor<T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8>;
/**
* Generic helper function to support extending classes and implementing (multiple) interfaces.
*
* For details on how to use this function, see
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_basic_interfaces|the developer's guide section about interface implementation}.
*/
export function BaseClass<T1, T2, T3, T4, T5, T6, T7, T8, T9>(...base: any[]): ClassConstructor<T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9>;
type OmitThis<T> = T extends (this: infer U) => infer R ? () => R : T extends (this: infer U, a: infer A) => infer R ? (a: A) => R : T extends (this: infer U, a1: infer A1, a2: infer A2) => infer R ? (a1: A1, a2: A2) => R : T extends (this: infer U, a1: infer A1, a2: infer A2, a3: infer A3) => infer R ? (a1: A1, a2: A2, a3: A3) => R : T extends (this: infer U, ...args: any[]) => infer R ? (...args: any[]) => R : T;
type Combined<F1 extends ((...args: any[]) => any) | null, F2 extends ((...args: any[]) => any) | null> = F1 extends null ? F2 extends null ? null : F2 : F2 extends null ? F1 : F1 extends () => any ? F2 : F1 extends (a11: infer A11) => any ? F2 extends () => infer R2 ? (a: A11) => R2 : F2 extends (a21: infer A21) => infer R2 ? (a: A11 & A21) => R2 : F2 extends (a21: infer A21, a22: infer A22) => infer R2 ? (a1: A11 & A21, a2: A22) => R2 : F2 extends (a21: infer A21, a22: infer A22, a23: infer A23) => infer R2 ? (a1: A11 & A21, a2: A22, a3: A23) => R2 : (...a: any[]) => any : F1 extends (a11: infer A11, a12: infer A12) => any ? F2 extends () => infer R2 ? (a1: A11, a2: A12) => R2 : F2 extends (a21: infer A21) => infer R2 ? (a: A11 & A21, a2: A12) => R2 : F2 extends (a21: infer A21, a22: infer A22) => infer R2 ? (a1: A11 & A21, a2: A12 & A22) => R2 : F2 extends (a21: infer A21, a22: infer A22, a23: infer A23) => infer R2 ? (a1: A11 & A21, a2: A12 & A22, a3: A23) => R2 : (...a: any[]) => any : F1 extends (a11: infer A11, a12: infer A12, a13: infer A13) => any ? F2 extends () => infer R2 ? (a1: A11, a2: A12, a3: A13) => R2 : F2 extends (a21: infer A21) => infer R2 ? (a: A11 & A21, a2: A12, a3: A13) => R2 : F2 extends (a21: infer A21, a22: infer A22) => infer R2 ? (a1: A11 & A21, a2: A12 & A22, a3: A13) => R2 : F2 extends (a21: infer A21, a22: infer A22, a23: infer A23) => infer R2 ? (a1: A11 & A21, a2: A12 & A22, a3: A13 & A23) => R2 : (...a: any[]) => any : (...a: any[]) => any;
type Removed<F1 extends ((...args: any[]) => any) | null, F2 extends ((...args: any[]) => any) | null> = F1 extends null ? null : F1;
/**
* Wrapper around JavaScript function objects. Similar to {@link Function#bind}.
*
* A common usage of delegate functions is as event listener. Since <code>delegate</code> adds an <code>equals</code>
* functionality to the wrapped function that will be used during event registration and de-registration, the same
* expression can be used for adding and removing the event listener, without the need for storing the event listener
* instance (see example). For more details, see the
* {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_events Events} section in the Developer's Guide.
*
* Multiple delegates can be {@link delegate#combine combined} into a single delegate instance and
* single delegates can be {@link delegate#remove removed} from a combined delegate.
*
* <br><b>Using a delegate as event listener</b>
* <pre>
* onQueryItemToolTip() {
*   // Do something
* }
*
* toggleListener(value) {
*   if (value) {
*     this.viewerInputMode.addQueryItemToolTipListener(delegate(this.onQueryItemToolTip, this))
*   } else {
*     // Since the delegate adds equals functionality, we can remove the listener here without
*     // introducing e.g. another field to store the listener instance.
*     this.viewerInputMode.removeQueryItemToolTipListener(delegate(this.onQueryItemToolTip, this))
*   }
* }
* </pre>
*/
export function delegate<T, F extends (this: T, ...args: any[]) => any>(fun: F, thisArg: T): OmitThis<F>;
export namespace delegate {
/**
* Combines two function delegates into a single one. When the resulting function is called,
* both original functions are called after another and the return value of the second function
* is returned.
* @param fun1 One of the functions to combine.
* @param fun2 One of the functions to combine.
* @returns A function that delegates to both original functions or <code>null</code> if both were <code>null</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_events Events}
*/
function combine<F1 extends ((...args: any[]) => any) | null, F2 extends ((...args: any[]) => any) | null>(fun1: F1, fun2: F2): Combined<F1, F2>;
/**
* Removes the first occurrence of <code>fun2</code> from the {@link delegate#getInvocationList invocation list} of <code>fun1</code>.
* @param fun1 The combined function delegate from which the second one should be removed. The instance itself will not be modified.
* @param fun2 The delegate function that should be removed from <code>fun1</code>.
* @returns A new delegate that consists of <code>fun1</code> with the first occurrence of <code>fun2</code> removed
* or <code>null</code> if <code>fun2</code> contained all delegate functions in <code>fun1</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_events Events}
*/
function remove<F1 extends ((...args: any[]) => any) | null, F2 extends ((...args: any[]) => any) | null>(fun1: F1 | null, fun2: F2 | null): Removed<F1, F2> | null;
/**
* Removes all occurrences of <code>fun2</code> from the {@link delegate#getInvocationList invocation list} of <code>fun1</code>.
* @param fun1 The combined function delegate from which the second one should be removed. The instance itself will not be modified.
* @param fun2 The delegate function that should be removed from <code>fun1</code>, no matter how often it occurs in <code>fun1</code>.
* @returns A new delegate that consists of <code>fun1</code> with the all occurrences of <code>fun2</code> removed
* or <code>null</code> if <code>fun2</code> contained all delegate functions in <code>fun1</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_events Events}
*/
function removeAll<F1 extends ((...args: any[]) => any) | null, F2 extends ((...args: any[]) => any) | null>(fun1: F1 | null, fun2: F2 | null): Removed<F1, F2> | null;
/**
* Return an array of all delegates that were {@link delegate#combine combined} into <code>fun</code>.
* @param fun The delegate to inspect.
* @returns An array of all delegates that were combined into <code>del</code>.
* @see {@link https://docs.yworks.com/yfileshtml/#/dguide/framework_events Events}
*/
function getInvocationList<F1 extends (...args: any[]) => any>(fun: F1 | null): F1[];
}
/**
* A map containing common string resources like command names, keyboard shortcuts, undo unit names and UI labels.
* By default, all resources are stored in yfiles.resources["invariant"].
*/
export const resources: {
[locale: string]: {
[key: string]: string;
};
} & {
invariant: {
/**
* The default value is 'Copy'.
*/
Copy: string;
/**
* The default value is 'Action+C;Ctrl+Ins'.
*/
CopyKey: string;
/**
* The default value is 'Cut'.
*/
Cut: string;
/**
* The default value is 'Action+X;Shift+Delete'.
*/
CutKey: string;
/**
* The default value is 'Zoom Out'.
*/
DecreaseZoom: string;
/**
* The default value is 'Delete'.
*/
Delete: string;
/**
* The default value is 'Delete;Backspace'.
*/
DeleteKey: string;
/**
* The default value is 'Extend Selection Down'.
*/
ExtendSelectionDown: string;
/**
* The default value is 'Shift+Down'.
*/
ExtendSelectionDownKey: string;
/**
* The default value is 'Extend Selection Left'.
*/
ExtendSelectionLeft: string;
/**
* The default value is 'Shift+Left'.
*/
ExtendSelectionLeftKey: string;
/**
* The default value is 'Extend Selection Right'.
*/
ExtendSelectionRight: string;
/**
* The default value is 'Shift+Right'.
*/
ExtendSelectionRightKey: string;
/**
* The default value is 'Extend Selection Up'.
*/
ExtendSelectionUp: string;
/**
* The default value is 'Shift+Up'.
*/
ExtendSelectionUpKey: string;
/**
* The default value is 'Fit Content'.
*/
FitContent: string;
/**
* The default value is 'Zoom In'.
*/
IncreaseZoom: string;
/**
* The default value is 'Move Down'.
*/
MoveDown: string;
/**
* The default value is 'Down'.
*/
MoveDownKey: string;
/**
* The default value is 'Move Focus Backward'.
*/
MoveFocusBackward: string;
/**
* The default value is 'Action+Left'.
*/
MoveFocusBackwardKey: string;
/**
* The default value is 'Move Focus Down'.
*/
MoveFocusDown: string;
/**
* The default value is 'Action+Down'.
*/
MoveFocusDownKey: string;
/**
* The default value is 'Move Focus Forward'.
*/
MoveFocusForward: string;
/**
* The default value is 'Action+Right'.
*/
MoveFocusForwardKey: string;
/**
* The default value is 'Move Focus Up'.
*/
MoveFocusUp: string;
/**
* The default value is 'Action+Up'.
*/
MoveFocusUpKey: string;
/**
* The default value is 'Move Left'.
*/
MoveLeft: string;
/**
* The default value is 'Left'.
*/
MoveLeftKey: string;
/**
* The default value is 'Move Right'.
*/
MoveRight: string;
/**
* The default value is 'Right'.
*/
MoveRightKey: string;
/**
* The default value is 'Move Page Down'.
*/
MoveToPageDown: string;
/**
* The default value is 'PageDown'.
*/
MoveToPageDownKey: string;
/**
* The default value is 'Move Page Up'.
*/
MoveToPageUp: string;
/**
* The default value is 'PageUp'.
*/
MoveToPageUpKey: string;
/**
* The default value is 'Move Up'.
*/
MoveUp: string;
/**
* The default value is 'Up'.
*/
MoveUpKey: string;
/**
* The default value is 'New'.
*/
New: string;
/**
* The default value is 'Action+N'.
*/
NewKey: string;
/**
* The default value is 'Open'.
*/
Open: string;
/**
* The default value is 'Paste'.
*/
Paste: string;
/**
* The default value is 'Action+V;Shift+Ins'.
*/
PasteKey: string;
/**
* The default value is 'Print'.
*/
Print: string;
/**
* The default value is 'Action+P'.
*/
PrintKey: string;
/**
* The default value is 'PrintPreview'.
*/
PrintPreview: string;
/**
* The default value is 'Ctrl+F2'.
*/
PrintPreviewKey: string;
/**
* The default value is 'Redo'.
*/
Redo: string;
/**
* The default value is 'Ctrl+Y;Command+Shift+Z'.
*/
RedoKey: string;
/**
* The default value is 'Save'.
*/
Save: string;
/**
* The default value is 'Save As...'.
*/
SaveAs: string;
/**
* The default value is 'Scroll Page Down'.
*/
ScrollPageDown: string;
/**
* The default value is 'PageDown'.
*/
ScrollPageDownKey: string;
/**
* The default value is 'Scroll Page Left'.
*/
ScrollPageLeft: string;
/**
* The default value is 'Scroll Page Right'.
*/
ScrollPageRight: string;
/**
* The default value is 'Scroll Page Up'.
*/
ScrollPageUp: string;
/**
* The default value is 'PageUp'.
*/
ScrollPageUpKey: string;
/**
* The default value is 'Select All'.
*/
SelectAll: string;
/**
* The default value is 'Action+A'.
*/
SelectAllKey: string;
/**
* The default value is 'Unavailable'.
*/
Unavailable: string;
/**
* The default value is 'Undo'.
*/
Undo: string;
/**
* The default value is 'Action+Z'.
*/
UndoKey: string;
/**
* The default value is 'Update Content Rectangle'.
*/
UpdateContentRect: string;
/**
* The default value is 'Zoom'.
*/
Zoom: string;
/**
* The default value is 'Add Bend'.
*/
"AddBend.RedoName": string;
/**
* The default value is 'Remove Bend'.
*/
"AddBend.UndoName": string;
/**
* The default value is 'Add Label'.
*/
AddLabel: string;
/**
* The default value is 'Add Label'.
*/
"AddLabel.RedoName": string;
/**
* The default value is 'Remove Label'.
*/
"AddLabel.UndoName": string;
/**
* The default value is 'Shift+F2'.
*/
AddLabelKey: string;
/**
* The default value is 'Add Port'.
*/
"AddPort.RedoName": string;
/**
* The default value is 'Remove Port'.
*/
"AddPort.UndoName": string;
/**
* The default value is 'Adjust Group Node Size'.
*/
AdjustGroupNodeSize: string;
/**
* The default value is 'Adjust Group Node Size'.
*/
"AdjustGroupNodeSize.RedoName": string;
/**
* The default value is 'Adjust Group Node Size'.
*/
"AdjustGroupNodeSize.UndoName": string;
/**
* The default value is 'Ctrl+Shift+G'.
*/
AdjustGroupNodeSizeKey: string;
/**
* The default value is 'Adjust Node Size'.
*/
"AdjustToSizeConstraints.RedoName": string;
/**
* The default value is 'Adjust Node Size'.
*/
"AdjustToSizeConstraints.UndoName": string;
/**
* The default value is 'Create Edge'.
*/
BeginEdgeCreation: string;
/**
* The default value is 'Collapse Group'.
*/
CollapseGroup: string;
/**
* The default value is 'Ctrl+Subtract'.
*/
CollapseGroupKey: string;
/**
* The default value is 'Collapse Node'.
*/
"CollapseNode.RedoName": string;
/**
* The default value is 'Expand Node'.
*/
"CollapseNode.UndoName": string;
/**
* The default value is 'Add Edge'.
*/
"CreateEdge.RedoName": string;
/**
* The default value is 'Remove Edge'.
*/
"CreateEdge.UndoName": string;
/**
* The default value is 'Add Node'.
*/
"CreateNode.RedoName": string;
/**
* The default value is 'Remove Node'.
*/
"CreateNode.UndoName": string;
/**
* The default value is 'CreateStripe'.
*/
"CreateStripe.RedoName": string;
/**
* The default value is 'Add Stripe'.
*/
"CreateStripe.UndoName": string;
/**
* The default value is 'Redo Cut'.
*/
"Cut.RedoName": string;
/**
* The default value is 'Undo Cut'.
*/
"Cut.UndoName": string;
/**
* The default value is 'Redo Delete'.
*/
"DeleteSelection.RedoName": string;
/**
* The default value is 'Undo Delete'.
*/
"DeleteSelection.UndoName": string;
/**
* The default value is 'Deselect All'.
*/
DeselectAll: string;
/**
* The default value is 'Action+Shift+A'.
*/
DeselectAllKey: string;
/**
* The default value is 'Deselect Item'.
*/
DeselectItem: string;
/**
* The default value is 'Duplicate'.
*/
Duplicate: string;
/**
* The default value is 'Action+D'.
*/
DuplicateKey: string;
/**
* The default value is 'Redo Duplicate'.
*/
"Duplicate.RedoName": string;
/**
* The default value is 'Undo Duplicate'.
*/
"Duplicate.UndoName": string;
/**
* The default value is 'Edit Label'.
*/
EditLabel: string;
/**
* The default value is 'F2'.
*/
EditLabelKey: string;
/**
* The default value is 'Enter Group'.
*/
EnterGroup: string;
/**
* The default value is 'Action+Enter'.
*/
EnterGroupKey: string;
/**
* The default value is 'Exit Group'.
*/
ExitGroup: string;
/**
* The default value is 'Action+Backspace'.
*/
ExitGroupKey: string;
/**
* The default value is 'Expand Group'.
*/
ExpandGroup: string;
/**
* The default value is 'Action+Add'.
*/
ExpandGroupKey: string;
/**
* The default value is 'Expand Node'.
*/
"ExpandNode.RedoName": string;
/**
* The default value is 'Collapse Node'.
*/
"ExpandNode.UndoName": string;
/**
* The default value is 'Fit Graph Bounds'.
*/
FitGraphBounds: string;
/**
* The default value is 'Group Selection'.
*/
GroupSelection: string;
/**
* The default value is 'Action+G'.
*/
GroupSelectionKey: string;
/**
* The default value is 'Move Items'.
*/
"Move.RedoName": string;
/**
* The default value is 'Move Items'.
*/
"Move.UndoName": string;
/**
* The default value is 'Move Handle'.
*/
"MoveHandle.RedoName": string;
/**
* The default value is 'Move Handle'.
*/
"MoveHandle.UndoName": string;
/**
* The default value is 'Add Node'.
*/
"NodeDropInputMode.NodeDropped.RedoName": string;
/**
* The default value is 'Remove Node'.
*/
"NodeDropInputMode.NodeDropped.UndoName": string;
/**
* The default value is 'Error Loading File'.
*/
"OpenFileFailed.Caption": string;
/**
* The default value is 'There was a problem during the loading of the file.'.
*/
"OpenFileFailed.Message": string;
/**
* The default value is 'Redo Paste'.
*/
"Paste.RedoName": string;
/**
* The default value is 'Undo Paste'.
*/
"Paste.UndoName": string;
/**
* The default value is 'Remove Bend'.
*/
"RemoveBend.RedoName": string;
/**
* The default value is 'Add Bend'.
*/
"RemoveBend.UndoName": string;
/**
* The default value is 'Remove Edge'.
*/
"RemoveEdge.RedoName": string;
/**
* The default value is 'Add Edge'.
*/
"RemoveEdge.UndoName": string;
/**
* The default value is 'Remove Label'.
*/
"RemoveLabel.RedoName": string;
/**
* The default value is 'Add Label'.
*/
"RemoveLabel.UndoName": string;
/**
* The default value is 'Remove Node'.
*/
"RemoveNode.RedoName": string;
/**
* The default value is 'Add Node'.
*/
"RemoveNode.UndoName": string;
/**
* The default value is 'Remove Port'.
*/
"RemovePort.RedoName": string;
/**
* The default value is 'Add Port'.
*/
"RemovePort.UndoName": string;
/**
* The default value is 'Add Stripe'.
*/
"RemoveStripe.RedoName": string;
/**
* The default value is 'Remove Stripe'.
*/
"RemoveStripe.UndoName": string;
/**
* The default value is 'Reparent Stripe'.
*/
"ReparentStripe.RedoName": string;
/**
* The default value is 'Reparent Stripe'.
*/
"ReparentStripe.UndoName": string;
/**
* The default value is 'Resize Node'.
*/
"ResizeNode.RedoName": string;
/**
* The default value is 'Resize Node'.
*/
"ResizeNode.UndoName": string;
/**
* The default value is 'Reverse Edge'.
*/
ReverseEdge: string;
/**
* The default value is 'Action+R'.
*/
ReverseEdgeKey: string;
/**
* The default value is 'Reverse Edge'.
*/
"ReverseEdge.RedoName": string;
/**
* The default value is 'Reverse Edge'.
*/
"ReverseEdge.UndoName": string;
/**
* The default value is 'Error Saving File'.
*/
"SaveFileFailed.Caption": string;
/**
* The default value is 'There was a problem during the saving of the file.'.
*/
"SaveFileFailed.Message": string;
/**
* The default value is 'Select Item'.
*/
SelectItem: string;
/**
* The default value is 'Set Current Item'.
*/
SetCurrentItem: string;
/**
* The default value is 'Change Edge Style'.
*/
"SetEdgeStyle.RedoName": string;
/**
* The default value is 'Change Edge Style'.
*/
"SetEdgeStyle.UndoName": string;
/**
* The default value is 'Change Label Position'.
*/
"SetLabelModelParameter.RedoName": string;
/**
* The default value is 'Change Label Position'.
*/
"SetLabelModelParameter.UndoName": string;
/**
* The default value is 'Change Label Style'.
*/
"SetLabelStyle.RedoName": string;
/**
* The default value is 'Change Label Style'.
*/
"SetLabelStyle.UndoName": string;
/**
* The default value is 'Change Label Text'.
*/
"SetLabelText.RedoName": string;
/**
* The default value is 'Change Label Text'.
*/
"SetLabelText.UndoName": string;
/**
* The default value is 'Convert Back to Group Node'.
*/
"SetLeaf.RedoName": string;
/**
* The default value is 'Convert to Normal Node'.
*/
"SetLeaf.UndoName": string;
/**
* The default value is 'Change Port Location'.
*/
"SetLocationModelParameter.RedoName": string;
/**
* The default value is 'Change Port Location'.
*/
"SetLocationModelParameter.UndoName": string;
/**
* The default value is 'Change Node Style'.
*/
"SetNodeStyle.RedoName": string;
/**
* The default value is 'Change Node Style'.
*/
"SetNodeStyle.UndoName": string;
/**
* The default value is 'Convert Back to Normal Node'.
*/
"SetNonLeaf.RedoName": string;
/**
* The default value is 'Convert to Group Node'.
*/
"SetNonLeaf.UndoName": string;
/**
* The default value is 'Change Parent'.
*/
"SetParent.RedoName": string;
/**
* The default value is 'Change Parent'.
*/
"SetParent.UndoName": string;
/**
* The default value is 'Reconnect Edge'.
*/
"SetPorts.RedoName": string;
/**
* The default value is 'Reconnect Edge'.
*/
"SetPorts.UndoName": string;
/**
* The default value is 'Change Port Style'.
*/
"SetPortStyle.RedoName": string;
/**
* The default value is 'Change Port Style'.
*/
"SetPortStyle.UndoName": string;
/**
* The default value is 'Change Preferred Label Size'.
*/
"SetPreferredLabelSize.RedoName": string;
/**
* The default value is 'Change Preferred Label Size'.
*/
"SetPreferredLabelSize.UndoName": string;
/**
* The default value is 'Change Stripe Insets'.
*/
"SetStripeInsets.RedoName": string;
/**
* The default value is 'Change Stripe Insets'.
*/
"SetStripeInsets.UndoName": string;
/**
* The default value is 'Change Minimum Stripe Size'.
*/
"SetStripeMinimumSize.RedoName": string;
/**
* The default value is 'Change Minimum Stripe Size'.
*/
"SetStripeMinimumSize.UndoName": string;
/**
* The default value is 'Change Stripe Size'.
*/
"SetStripeSize.RedoName": string;
/**
* The default value is 'Change Stripe Size'.
*/
"SetStripeSize.UndoName": string;
/**
* The default value is 'Change Stripe Style'.
*/
"SetStripeStyle.RedoName": string;
/**
* The default value is 'Change Stripe Style'.
*/
"SetStripeStyle.UndoName": string;
/**
* The default value is 'Toggle Group State'.
*/
ToggleGroupState: string;
/**
* The default value is 'Action+Multiply'.
*/
ToggleGroupStateKey: string;
/**
* The default value is 'Toggle Selection'.
*/
ToggleItemSelection: string;
/**
* The default value is 'Ctrl+Space'.
*/
ToggleItemSelectionKey: string;
/**
* The default value is 'Ungroup Selection'.
*/
UngroupSelection: string;
/**
* The default value is 'Action+U'.
*/
UngroupSelectionKey: string;
/**
* The default value is 'Zoom To Current Item'.
*/
ZoomToCurrentItem: string;
};
};
export interface PropertyInfo extends YObject {
}
/**
Contains information about a property.
@final
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo Online Documentation}*/
export class PropertyInfo {
/**
Gets the attributes associated with this property.
@param [inherit=true] if set to <code>true</code> inherited attributes will also be returned.
@returns The attributes or an empty array.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo%23getAttributes(boolean) Online Documentation}*/
getAttributes(inherit?: boolean): Attribute[];
/**
Gets the value of the property instance from the given object by performing automatic type conversion.
@param obj The object which has the property, <code>null</code> for static properties.
@returns The value of the property instance from the given object by performing automatic type conversion.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo%23getValue(Object) Online Documentation}*/
getValue(obj: any): any | null;
/**
Sets the value of the property on the given object.
@param obj The object whose property should be written, <code>null</code> for static properties.
@param value The value to set
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo%23setValue(Object%2CObject) Online Documentation}*/
setValue(obj: any | null, value: any | null): void;
/**
Gets a value indicating whether the property can be read.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo%23canRead Online Documentation}*/
readonly canRead: boolean;
/**
Gets a value indicating whether the property can be written.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo%23canWrite Online Documentation}*/
readonly canWrite: boolean;
/**
Gets a value indicating whether the property is a static property, rather than declared on an instance.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo%23isStatic Online Documentation}*/
readonly isStatic: boolean;
/**
Gets the type of the property.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo%23propertyType Online Documentation}*/
readonly propertyType: Class;
/**
Gets the declaring type, i.e., the type where the member was declared.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo%23declaringType Online Documentation}*/
readonly declaringType: Class;
/**
Gets the name of the member.
@see {@link https://docs.yworks.com/yfileshtml/#/api/PropertyInfo%23name Online Documentation}*/
readonly name: string;
static $class: Class;
private $_PropertyInfo: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is PropertyInfo;
}
export interface YObject {
}
/**
The root of the class hierarchy of the yFiles for HTML class framework.
Two of these methods are actually defined on the native JavaScript <code>object</code> prototype:
<ul>
<li>{@link YObject#hashCode}
<li>{@link YObject#equals}
</ul>
The other methods are part of the {@link YObject} type, only.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject Online Documentation}*/
export class YObject {
/**
Tests whether this object and the other object are equal.
@param other The tested object.
@returns <code>true</code>, if the objects are equal; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject%23equals(Object) Online Documentation}*/
equals(other: any | null): boolean;
/**
Tests if two objects are equal.
@param a One of the objects. Can be null or of any type.
@param b One of the objects. Can be null or of any type.
@returns <code>true</code>, if the objects are equal; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject%23equals(Object%2CObject) Online Documentation}*/
static equals(a: any | null, b: any | null): boolean;
/**
Returns the type of this object.
@returns The type of this object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject%23getClass() Online Documentation}*/
getClass(): Class;
/**
Returns an object with a get and/or a set method that is optionally bound to this instance.
The requested <code>name</code> must be a property (with getter and/or setter).
@param name The name of the property.
@param bound If <code>true</code>, then the get/set methods of the returned object will be bound to this instance.
@returns An object with a get and/or set method that reads/writes to the specified field. <code>null</code> if no property exists by that
name.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject%23getOwnProperty(string%2Cboolean) Online Documentation}*/
getOwnProperty(name: string, bound: boolean): Object | null;
/**
Returns the most recently defined property descriptor in the prototype chain of the given object.
@param o The object that the property belongs to
@param name The name of the property.
@returns The most recently defined property descriptor in the prototype chain of the given object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject%23getPropertyDescriptor(Object%2Cstring) Online Documentation}*/
static getPropertyDescriptor(o: any, name: string): Object | null;
/**
Returns the hash code of this object.
If two objects are {@link YObject#equals equal}, then they must have the same hash code.
@returns The hash code of this object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject%23hashCode() Online Documentation}*/
hashCode(): number;
/**
Shallow clones an object.
It will also clone any value types.
@returns A cloned instance of the same type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject%23memberwiseClone() Online Documentation}*/
memberwiseClone(): this;
/**
Tests if two objects are identical.
@param a One of the objects.
@param b One of the objects.
@returns <code>true</code>, if the objects are the identical; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject%23referenceEquals(Object%2CObject) Online Documentation}*/
static referenceEquals(a: any | null, b: any | null): boolean;
/**
Displays the textual representation of the object.
@returns The textual representation of this object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YObject%23toString() Online Documentation}*/
toString(): string;
static $class: Class;
private $_Object: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YObject;
}
export interface Abstract extends YObject {
}
/**
This type represents an abstract value, that is a function that is not implemented.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Abstract Online Documentation}*/
export class Abstract {
/**
Tests whether the given object or property descriptor is a representation of an abstract value.
@param o The object to test.
@returns <code>true</code>, if the object is abstract; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Abstract%23isAbstract(Object) Online Documentation}*/
static isAbstract(o: any): boolean;
static $class: Class;
private $_Abstract: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Abstract;
}
export interface License extends YObject {
}
/**
* This object is a container for the license information.
* It can be used as follows when ES modules are used:
*
* <pre>
* import {License} from "yfiles"
*
* License.value = { <license information> }
* </pre>
*
* In an environment where ES2015 is not used, there is the old option:
*
* <pre>
* yfiles.license = { <license information> };
* </pre>
*
* and the new one using this container type:
*
* <pre>
* yfiles.lang.License.value = { <license information> };
* </pre>
*
* @final
* @see {@link https://docs.yworks.com/yfileshtml/#/api/License Online Documentation}
*/
export class License {
/**
Sets the license information.
Use this property to store the license object with the library before using the functionality of the library.
@see {@link https://docs.yworks.com/yfileshtml/#/api/License%23value Online Documentation}*/
static value: Object | null;
static $class: Class;
private $_License: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is License;
}
/**
* An attribute that describes the type of a property, field, or the return type of a member function.
* This attribute can be used for the automatic XAML-like serialization by GraphML. It is required for fields and
* properties that should use attribute-based serialization to inform the reflection engine about the type of the
* field/property in the meta-data of the class.
*
* <pre>
* const MyClass = yfiles.lang.Class('MyClass', {
*     constructor: function() { // the constructor function
*       this.$y = 42;
*     },
*
*     myProperty: {
*       $meta:[yfiles.lang.TypeAttribute(yfiles.lang.Number.$class)],
*       get: function() {
*         return this.$y;
*       },
*     },
* });
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/TypeAttribute Online Documentation}
*/
export function TypeAttribute(type: Class): TypeAttribute;
interface TypeAttribute extends Attribute {
/**
Initializes a new instance of the {@link TypeAttribute} class.
@param type The type to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeAttribute Online Documentation}*/
init(type: Class): TypeAttribute;
/**
Initializes a new instance of the {@link TypeAttribute} class.
@param options The parameters to pass.
@param options.type The type to use.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeAttribute Online Documentation}*/
init(options: {
type?: Class;
}): TypeAttribute;
/**
Gets the type that this attribute references.
@see {@link https://docs.yworks.com/yfileshtml/#/api/TypeAttribute%23type Online Documentation}*/
readonly type: Class;
}
export module TypeAttribute {
const $class: Class;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
function isInstance(o: any): o is TypeAttribute;
}
export interface Class extends YObject {
}
/**
* Instances of the class <code>Class</code> represent classes and interfaces in a running yFiles application.
* This class gives reflective access to a type's {@link Class#getAttributes attributes} and {@link Class#getProperties properties}. It also provides some static utility methods
* like {@link Class#ensure} and {@link Class#fixType}.
*
* At runtime, a <code>Class</code> object can be obtained through the static <code>$class</code> property on yFiles types or the <code>getClass()</code> method
* on instances. The class framework automatically creates a <code>Class</code> object for all JavaScript classes that inherit from a
* yFiles type. It also adds the <code>$class</code> and <code>getClass</code> members.
*
* Note that the class framework creates the <code>Class</code> object for classes that inherit from yFiles classes only when the first
* instance of this subclass is created. If the <code>$class</code> property is required before an instance of the subclass was created,
* you need to call {@link Class#fixType} first.
*
* This class may also be used as a function to create a yFiles class from a class definition object.
*
* There are only very few cases where you will actually need to use this function to create a class. The yFiles class
* framework should be compatible with all major popular techniques of creating classes and instances. You may use
* prototypal inheritance, ES2015 classes, TypeScript classes and other techniques that use the same mechanics for
* inheritance.
*
* The class definition object may contain any of the following special identifiers:
*
* <ul>
* <li><code>$extends</code>: An object reference that points to the parent object.</li>
* <li><code>$with</code>: An array of interfaces this class implements.</li>
* <li><code>$abstract</code>: A boolean value that indicates whether or not this class is abstract.</li>
* <li><code>$static</code>: An object that contains methods, fields and properties that should be statically available and will be
* registered on the constructor function instead of its prototype.</li>
* <li><code>$meta</code>: An array of {@link Attribute}s or a function that returns an array of
* {@link Attribute}s that describe the class.</li>
* </ul>
* Additionally, it may contain methods, properties and fields that should be added to the constructor functions prototype.
* If the value of an field is an object with a <code>$meta</code> property, then it will be used as a property descriptor for the field
* (meaning that <code>writable</code>, <code>enumerable</code> and <code>configurable</code>, <code>value</code> and <code>get</code>, <code>set</code> can be used as in a property descriptor for <code>Object.defineProperty</code>
* and the value of the <code>$meta</code> property is stored for usage in the reflection API. If the object has a <code>get</code> and/or <code>set</code>
* property, then it will also be used as a property descriptor.
*
* <pre>
* var MyList = yfiles.lang.Class('MyList', {
*     $extends: my.AbstractList,
*     $with: [my.Iterable, my.Collection],
*
*     constructor: function() { // the constructor function
*     },
*
*     add: function(obj) { ... },
*     size: { // transformed into a property
*      'get': function() { return ...; }
*     },
*
*     $static: {
*      create: function() { ... }
*     }
* });
* // usage
* var list = new MyList();
* list.add("hello");
* list.size === 1; // true
* </pre>
*
* The constructor property can be either a function (in case of a single constructor) or an object that is transformed
* into named constructors.
*
* <pre>
* var Rect = yfiles.lang.Class('Rect', {
*  constructor: {
*      'default': function(x, y, width, height) { ... },
*      'FromPoints': function(topLeft, bottomRight) { ... },
*      'FromPoint': function(topLeft, width, height) { ... }
*  }
* });
* // usage:
* var myrect = new Rect(10, 10, 50, 50);
* myrect instanceof Rect; // true
* myrect = new Rect.FromPoints({x: 10, y: 10}, {x: 60, y: 60});
* myrect instanceof Rect; // true
* myrect = new Rect.FromPoint({x: 10, y: 10}, 50, 50);
* myrect instanceof Rect; // true
* </pre>
*
* Named constructors can call other constructors and can be written like any other constructor, i.e. <code>this</code> will be a new
* instance of the class that is being constructed.
*
* To access super methods, you can use the static <code>$super</code> property that is added to all classes.
*
* Example (inside a method or property of the previously defined Rect class):
*
* <pre>
* Rect.$super.toString.call(this);
* </pre>
*
* If you do not specify the parent class of a new class, then {@link YObject} will be used as its base type. The parent
* class of {@link YObject} is the standard JavaScript <code>Object.prototype</code>.
*
* Every class has a couple of static methods and properties:
*
* <ul>
* <li>{@link Class#isInstance}: tests whether an object is an instance of this class.</li>
* <li><code>$class</code>: a reference to the {@link Class} instance that provides reflective information about this type.</li>
* <li><code>$super</code>: the parent class.</li>
* </ul>
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Class Online Documentation}
*/
export class Class {
/**
Makes sure the yFiles for HTML modules that contain the given types are not stripped away by build optimizers like
tree-shaking tools.
Sometimes you want to load yFiles types without using them explicitly in your code, for example the styles used in a
GraphML files or implicit dependencies between different styles of the automatic layout. If you just import the
corresponding module without using any of its types, a build optimizer might remove this seemingly unused import. To
avoid this, you can pass the type to this method.
Note that yFiles shows a clear error message if a type is missing due to an implicit dependency. You don't have to call
this method for types that you're using explicitly.
This is an empty function without any code.
@param types A list of types.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23ensure(Function) Online Documentation}*/
static ensure(...types: Function[]): void;
/**
Makes sure the prototype of this type is properly set up and $class and getClass() is working as expected.
This method needs to be called once for custom types before they can be used in reflection use-cases.
@param type The type
@param [name=null] The optional (short) name to use for class. This will be returned by getClass()/$class.name. This will be guessed from
the function name if unspecified.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23fixType(Function%2Cstring) Online Documentation}*/
static fixType(type: Function, name?: string | null): void;
/**
Returns the class object for the given name, or <code>null</code> if no such class exists.
@param name The name of the requested class.
@returns The class object or <code>null</code>, if no such class exists.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23forName(string) Online Documentation}*/
static forName(name: string): Class | null;
/**
Returns all attributes that belong to this type.
@param [inherited=true] If <code>true</code>, then the attributes of the parent types are also added to the list. Default value is <code>false</code>.
@returns An array of attributes that belong to this type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23getAttributes(boolean) Online Documentation}*/
getAttributes(inherited?: boolean): Attribute[];
/**
Returns an array of all properties of the type.
@param [inherited=true]
@param [staticProperties=false]
@returns An array of properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23getProperties(boolean%2Cboolean) Online Documentation}*/
getProperties(inherited?: boolean, staticProperties?: boolean): PropertyInfo[];
/**
Returns an array of all properties of the type.
@param options The parameters to pass.
@param [options.inherited=true]
@param [options.staticProperties=false]
@returns An array of properties.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23getProperties(boolean%2Cboolean) Online Documentation}*/
getProperties(options: {
inherited?: boolean;
staticProperties?: boolean;
}): PropertyInfo[];
/**
Tests whether an object is an instance of this type.
This is the only way to test if an object implements a {@link Interface}.
@param o The object that may be an instance of this type.
@returns <code>true</code>, if the object is an instance of this type; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23isInstance(Object) Online Documentation}*/
isInstance(o: any | null): boolean;
/**
Creates a new instance of this type using its default constructor.
@returns A new instance of this type using its default constructor.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23newInstance() Online Documentation}*/
newInstance(): Object;
/**
<code>true</code>, if this type represents an enum; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23isEnum Online Documentation}*/
readonly isEnum: boolean;
/**
<code>true</code>, if this type represents a primitive type; <code>false</code> otherwise.
The primitive types are: number, string and boolean.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23isPrimitive Online Documentation}*/
readonly isPrimitive: boolean;
/**
<code>true</code>, if this type represents a value type; <code>false</code> otherwise.
A value type is either a primitive or an instance of a struct.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23isValueType Online Documentation}*/
readonly isValueType: boolean;
/**
<code>true</code>, if this type represents a regular class; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23isClass Online Documentation}*/
readonly isClass: boolean;
/**
The short name of this class.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Class%23name Online Documentation}*/
readonly name: string;
static $class: Class;
private $_Class: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Class;
}
export interface YNumber extends YObject, IComparable {
}
/**
Wrapper class for the JavaScript <code>number</code> primitive type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNumber Online Documentation}*/
export class YNumber {
/**
Compares this number with another one.
@param obj The other number to compare with.
@returns <code>0</code> if both numbers are equal, <code>-1</code> if this number is smaller than the other one, <code>1</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YNumber%23compareTo(Object) Online Documentation}*/
compareTo(obj: any | null): number;
static $class: Class;
private $_Number: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YNumber;
}
export interface YString extends YObject, IComparable {
}
/**
Wrapper class for the JavaScript <code>string</code> primitive type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YString Online Documentation}*/
export class YString {
/**
Compares this string with another one.
@param obj The other string to compare with.
@returns <code>0</code> if both strings are equal, <code>-1</code> if this string comes before the other one in lexicographic sort order, <code>1</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YString%23compareTo(Object) Online Documentation}*/
compareTo(obj: any | null): number;
static $class: Class;
private $_String: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YString;
}
export interface YBoolean extends YObject, IComparable {
}
/**
Wrapper class for the JavaScript <code>boolean</code> primitive type.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YBoolean Online Documentation}*/
export class YBoolean {
/**
Compares this boolean with another one.
@param obj The other boolean to compare with.
@returns <code>0</code> if both booleans are equal, <code>-1</code> if this boolean is <code>false</code> and the other is <code>true</code>, <code>1</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/YBoolean%23compareTo(Object) Online Documentation}*/
compareTo(obj: any | null): number;
static $class: Class;
private $_Boolean: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is YBoolean;
}
export interface Enum extends YObject {
}
/**
* The base type of an enumeration.
* All enums extend {@link Enum} and should be used instead of "magic values".
*
* The members of the enum type definition will be static and an enum can not be constructed. This type is not type safe.
*
* <pre>
* var RequestState = yfiles.lang.Enum('RequestState', {
*     SEND: 1,
*     PROCESSED: 2,
*     SHIPPED: 3,
*     FAILED: 4,
*    });
*    if(state === RequestState.FAILED) { ... }
* </pre>
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Enum Online Documentation}
*/
export class Enum {
/**
Returns the name of an enum value.
@param type The type of the enum.
@param value The value of the field.
@returns The name of the field.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Enum%23getName(Class%2CEnum) Online Documentation}*/
static getName(type: Class, value: Enum): string;
/**
Returns a list of all field names that represent an enum value.
@param type The type of the enum.
@returns A list of all field names that represent an enum value.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Enum%23getValueNames(Class) Online Documentation}*/
static getValueNames(type: Class): string[];
/**
Returns a list of all numeric values.
@param type The type of the enum.
@returns A list of all numeric values.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Enum%23getValues(Class) Online Documentation}*/
static getValues(type: Class): number[];
/**
Returns the value of the field with the given ID.
@param type The type of the enum.
@param id The name of the field.
@param ignoreCase If <code>true</code>, then the case of the field will be ignored and the search is case-insensitive.
@returns The value of the field with the given ID.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Enum%23parse(Class%2Cstring%2Cboolean) Online Documentation}*/
static parse(type: Class, id: string, ignoreCase: boolean): Enum;
static $class: Class;
private $_Enum: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Enum;
}
export interface Trait extends YObject {
}
/**
A trait is an interface that may or may not have default implementations of its methods, properties and fields.
<b>Note: This is a deprecated alias for {@link Interface}</b>.
Traits have very specific rules in case two traits with the same member are implemented by a subtype:
<ul>
<li>If both method definitions are abstract, then the method is abstract.
<li>If one method definition is abstract and the other concrete, then the concrete method is used.
<li>If both method definitions are concrete and the traits are implemented at the same level, then the method is abstract.
<li>If both method definitions are concrete and one of the traits is a sub-type of the other, then its method definition
will be used.
</ul>
In all of those cases, the rule is that a subtype overrides the parent types' definitions. Example: Given the traits A,
B, C and D where A defines a method "a" to be abstract and B and C extending trait A with concrete implementations of
"a", when D is a trait that implements both B and C, its version of "a" will be abstract, unless it defines its own
implementation of "a", in which case that definition is used.
All trait types have a <code>$class</code> property, which contains their {@link Class type}, and a static {@link Trait#isTrait} method that
can be used to test whether an object implements the trait.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Trait Online Documentation}*/
export class Trait {
/**
Tests whether the given object is a trait.
This method is equivalent to using <code>obj.$class.isTrait</code> but safe in case the object does not have a <code>$class</code> property.
@param o The trait candidate.
@returns <code>true</code>, if the object is a trait; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Trait%23isTrait(Object) Online Documentation}*/
static isTrait(o: any | null): boolean;
static $class: Class;
private $_Trait: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Trait;
}
export interface Interface extends YObject {
}
/**
An interface that may or may not have default implementations of its methods, properties and fields.
Interfaces have very specific rules in case two interfaces with the same member are implemented by a subtype:
<ul>
<li>If both method definitions are abstract, then the method is abstract.
<li>If one method definition is abstract and the other concrete, then the concrete method is used.
<li>If both method definitions are concrete and the interfaces are implemented at the same level, then the method is
abstract.
<li>If both method definitions are concrete and one of the interfaces is a sub-type of the other, then its method definition
will be used.
</ul>
In all of those cases, the rule is that a subtype overrides the parent types' definitions. Example: Given the interfaces
A, B, C and D where A defines a method "a" to be abstract and B and C extending interface A with concrete
implementations of "a", when D is a interface that implements both B and C, its version of "a" will be abstract, unless
it defines its own implementation of "a", in which case that definition is used.
All interface types have a <code>$class</code> property, which contains their {@link Class type}, and a static {@link Interface#isInterface}
method that can be used to test whether an object implements the interface.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interface Online Documentation}*/
export class Interface {
/**
Tests whether the given object is an interface.
@param o The interface candidate.
@returns <code>true</code>, if the object is an interface; <code>false</code> otherwise.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Interface%23isInterface(Object) Online Documentation}*/
static isInterface(o: any | null): boolean;
static $class: Class;
private $_Interface: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Interface;
}
export interface Attribute extends YObject {
}
/**
* Specialized type that can be used to describe meta information about other types, methods, fields, properties and
* constructors.
* All attributes extend {@link Attribute} or a subtype of an attribute. They can also implement {@link Interface Interfaces}.
*
* The actual creation of an attribute is delayed to the point in time when it is needed by the reflection API. Therefore,
* the attribute constructors are wrapped and replaced by a function that creates an instance of the attribute.
*
* <pre>
* const MyAttribute = Attribute('MyAttribute', {
*   constructor: function(message) {
*     this.message = message
*   }
* })
*
* class MyClass {
*   get myField() { ... }
*
*   // provide a static $meta property that returns an object with meta properties for each field
*   static get $meta() {
*     return {
*       myField: [MyAttribute('hello').init({ recipient: 'field' })]
*       // '$self' is for the class itself
*       $self: [MyAttribute('hello').init({ recipient: 'class' })]
*     }
*   }
* }
* Class.fixType(MyClass) // required for classes that don't inherit from a yFiles library class
*
* const fieldAttribute = MyClass.$class.getProperties()[0].getAttributes()[0]
* console.log(fieldAttribute.message + ' ' + fieldAttribute.recipient) // 'hello field'
*
* const classAttribute = MyClass.$class.getAttributes()[0]
* console.log(classAttribute.message + ' ' + classAttribute.recipient) // 'hello class'
* </pre>
*
* Even though MyAttribute is invoked at the definition of MyClass, no attribute is created until the reflection API is
* used (in the above example: using <code>getCustomAttribute</code>).
*
* The result of the constructor function of an attribute is a function, that, when invoked, returns an instance of the
* attribute. It also provides an init method that adds the properties of the given object to the attribute.
*
* <b>Note:</b> Attribute constructors are always called <b>without</b> the <code>new</code> keyword.
*
* @see {@link https://docs.yworks.com/yfileshtml/#/api/Attribute Online Documentation}
*/
export class Attribute {
static $class: Class;
private $_Attribute: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Attribute;
}
export interface IComparable {
}
/**
Provides a method by which implementors can be compared to each other.
@see {@link https://docs.yworks.com/yfileshtml/#/api/IComparable Online Documentation}*/
export abstract class IComparable {
/**
Compares this object to the given object of the same type.
@param obj The object to compare this to.
@returns
<ul>
<li>-1: this is less than <code>obj</code>
<li>0: this is equal to <code>obj</code>
<li>1: this is greater than <code>obj</code>
</ul>
@see {@link https://docs.yworks.com/yfileshtml/#/api/IComparable%23compareTo(Object) Online Documentation}*/
abstract compareTo(obj: any | null): number;
/**
Creates an implementation of the interface {@link IComparable} by using the given function as implementation for its
{@link IComparable#compareTo} method.
@param compareTo A function for {@link IComparable}'s single abstract method {@link IComparable#compareTo}.
@returns An instance of the {@link IComparable} interface based on the given function.*/
static create(compareTo: (obj: any | null) => number): IComparable;
static $class: Class;
private $_IComparable: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is IComparable;
}
export interface ClassDefinition extends YObject {
}
/**
A lazy class type which delays the creation of the class until it is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassDefinition Online Documentation}*/
export class ClassDefinition {
/**
A lazy class type which delays the creation of the class until it is used.
@param callback Returns the class definition as required by {@link Class}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClassDefinition Online Documentation}*/
constructor(callback: () => any | null);
static $class: Class;
private $_ClassDefinition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClassDefinition;
}
export interface InterfaceDefinition extends YObject {
}
/**
A lazy interface type which delays the creation of the interface until it is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterfaceDefinition Online Documentation}*/
export class InterfaceDefinition {
/**
A lazy interface type which delays the creation of the interface until it is used.
@param callback Returns the interface definition as required by {@link Interface}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/InterfaceDefinition Online Documentation}*/
constructor(callback: () => any | null);
static $class: Class;
private $_InterfaceDefinition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is InterfaceDefinition;
}
export interface EnumDefinition extends YObject {
}
/**
A lazy enum type which delays the creation of the enum until it is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EnumDefinition Online Documentation}*/
export class EnumDefinition {
/**
A lazy enum type which delays the creation of the enum until it is used.
@param callback Returns the enum definition as required by {@link Enum}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/EnumDefinition Online Documentation}*/
constructor(callback: () => any | null);
static $class: Class;
private $_EnumDefinition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is EnumDefinition;
}
export interface AttributeDefinition extends YObject {
}
/**
A lazy attribute definition which delays the creation of the attribute until it is used.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AttributeDefinition Online Documentation}*/
export class AttributeDefinition {
/**
A lazy attribute definition which delays the creation of the attribute until it is used.
@param callback Returns the attribute definition as required by {@link Attribute}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/AttributeDefinition Online Documentation}*/
constructor(callback: () => any | null);
static $class: Class;
private $_AttributeDefinition: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is AttributeDefinition;
}
/**
* A utility type that can be used to give the static <code>$meta</code> property of a TypeScript class a type other than <code>any</code>.
* The type automatically collects all properties of the class and expects an object that maps property names to
* {@link Attribute}s or arrays of them.
*
* Attributes for the class itself are annotated using the special <code>$self</code> key.
*
* Example:
*
* <pre>
* class MyClass extends MarkupExtension {
*   get myProperty(): any { ... }
*   set myProperty(v: any) { ... }
*
*   static get $meta(): MarkupDescriptor<MyClass> {
*     return {
*       myProperty: [GraphMLAttribute().init({})]
*     }
*   }
* }
* </pre>
*
* Type parameter <code>T</code>: The type of surrounding class.
* @see {@link https://docs.yworks.com/yfileshtml/#/api/MarkupDescriptor Online Documentation}
*/
export type MarkupDescriptor<T> = Partial<Record<Exclude<keyof T, "prototype" | "$meta" | "toString"> | "$self", Attribute | Array<Attribute>>>;
export interface Exception extends Error {
}
/**
Extension of the native JavaScript <code>Error</code> type that includes a stack trace.
yFiles includes the following specialized exception types indicated by the {@link Exception#name}:
<ul>
<li><b>ArgumentError</b> – Indicates that the value of a parameter of the current method is illegal.
<li><b>ArgumentNullError</b> – Indicates that the value of a parameter of the current method is <code>null</code> but a non-<code>null</code> value was expected.
<li><b>ArgumentOutOfRangeError</b> – Indicates that the value of a parameter of the current method is not in the legal range of values.
<li><b>IndexOutOfRangeError</b> – Indicates that an array index is not in the legal range.
<li><b>NullReferenceError</b> – Indicates that a value is <code>null</code> when it shouldn't.
<li><b>InvalidOperationError</b> – Indicates that the current operation is not valid in the instances's current state.
<li><b>NotImplementedError</b> – Indicates that the called method is not implemented.
<li><b>NotSupportedError</b> – Indicates that the operation is not supported.
<li><b>ArithmeticError</b> – Indicates an error in an arithmetic operation.
<li><b>IOError</b> – Indicates an error during an IO operation.
<li><b>KeyNotFoundError</b> – Indicates that a mapping does not contain the given key.
<li><b>AlgorithmAbortedError</b> – Indicates that an immediate termination request was detected by an algorithm.
<li><b>InvalidGraphStructureError</b> – Indicates that a graph cannot be used with a certain algorithm. Some graph algorithms require a specific graph
structure, for example trees or planar graph. These algorithms throw this error if a non-matching graph is used.
</ul>
This type also provides the static {@link Exception#catchErrors} property, which when enabled, causes most entry points (e.g. event listeners) to use
try/catch blocks to catch possible errors that could be thrown during event handling. The errors are handled by the {@link Exception#handleError}
function, which calls the {@link Exception#handler} function, if it has been set. Note that for some browsers/plugins, try/catch blocks prevent
error debugging functionality. Hence, {@link Exception#catchErrors} should be turned off while debugging.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Exception Online Documentation}*/
export class Exception {
/**
Creates a new exception with the given {@link Exception#message} and an optional error {@link Exception#name}.
@param [message=''] The exception message.
@param [name=''] The exception name. This name can be used in try...catch statements to distinguish between different types of errors.
@param [innerException=null] The inner exception, if another exception was wrapped.
@see Exception#message
@see {@link https://docs.yworks.com/yfileshtml/#/api/Exception Online Documentation}*/
constructor(message?: string, name?: string, innerException?: Exception | null);
/**
Creates a new exception with the given {@link Exception#message} and an optional error {@link Exception#name}.
@param options The parameters to pass.
@param [options.message=''] The exception message.
@param [options.name=''] The exception name. This name can be used in try...catch statements to distinguish between different types of errors.
@param [options.innerException=null] The inner exception, if another exception was wrapped.
@see Exception#message
@see {@link https://docs.yworks.com/yfileshtml/#/api/Exception Online Documentation}*/
constructor(options: {
message?: string;
name?: string;
innerException?: Exception | null;
});
/**
This function is called in catch blocks in the library if {@link Exception#catchErrors} is enabled.
If a {@link Exception#handler} is registered, the error is passed to the error handler, otherwise if {@link Exception#catchErrors} is set to <code>true</code> the error is logged to the
console using <code>console.warn</code> if available.
@param error The error object that has been caught
@see {@link https://docs.yworks.com/yfileshtml/#/api/Exception%23handleError(Object) Online Documentation}*/
static handleError(error: any): void;
/**
Gets the exception message.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Exception%23message Online Documentation}*/
message: string;
/**
Gets the exception name.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Exception%23name Online Documentation}*/
name: string;
/**
The inner exception, if another exception was wrapped.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Exception%23innerException Online Documentation}*/
innerException: Exception | null;
/**
Whether to use try/catch blocks when invoking event handlers.
If this property is enabled, try/catch blocks are used when invoking event handlers. As most browsers provide a stack
trace with Error objects, this property can be used to provide more detailed error information. If this property is set,
but no {@link Exception#handler} is registered, a warning will be logged to the console instead.
@see {@link https://docs.yworks.com/yfileshtml/#/api/Exception%23catchErrors Online Documentation}*/
static catchErrors: boolean;
/**
If {@link Exception#catchErrors} is enabled, any caught errors are passed to this function.
Setting a handler via this property automatically turns on the {@link Exception#catchErrors} flag!
@see {@link https://docs.yworks.com/yfileshtml/#/api/Exception%23handler Online Documentation}*/
static handler: ((obj: any | null) => void) | null;
static $class: Class;
private $_Exception: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is Exception;
}
export interface ClearAreaLayout extends YObject, ILayoutAlgorithm {
}
/**
This layout algorithm clears a specified area by moving intersecting elements, while trying to minimize the changes in
the given layout.
<b>Layout Style</b>
The algorithm can be applied if it is necessary to make space within a specific region/area in an existing graph layout.
It tries to keep the given layout style as much as possible. Ideally, only local changes in and around the marked area
are made, such that the mental map for the user is preserved.
An example application is the use case that new nodes or a whole graph component were added to the graph and should be
placed in a specific region of an existing graph layout. The region can then first be cleared using this algorithm such
that afterwards the nodes or the component could be inserted without overlaps with other elements. This specific case
can conveniently be implemented by defining a set of so-called <i>area nodes</i>, via {@link ClearAreaLayout#AREA_NODES_DP_KEY}.
Another, more general use case would be nodes that grow in size, including the case that a folder node is expanded and
converted to a larger group node. This case is easily realized by specifying the expanded node via
{@link ClearAreaLayout#EXPANDED_NODE_DP_KEY}.
<b>Concept</b>
The area, which can be defined as a {@link ClearAreaLayout#area rectangle} or an {@link ClearAreaLayout#areaOutline outline}, defines a region in the given input graph which must be
cleared. Clearing the area means no graph elements are allowed to intersect with it. This includes nodes and, depending
on settings {@link ClearAreaLayout#considerEdges}, {@link ClearAreaLayout#considerNodeLabels} and {@link ClearAreaLayout#considerEdgeLabels} also edges, node labels and edge labels.
In order to clear the area, the graph elements must be moved. Moving an element often induces the necessity to move
further elements along with them. Thus, depending on where the area is located in the input drawing, many elements may
be moved to make the necessary space. How elements are selected and how they are moved highly depends on the chosen
{@link ClearAreaLayout#clearAreaStrategy clearing strategy}.
<b>Features</b>
For grouped graphs, the group nodes are also correctly handled by this algorithm. However, it is important to note that
there is always one so-called <i>area group node</i>. It is the group node inside which the free space should be created and which may in
consequence need to grow, see {@link ClearAreaLayout#AREA_GROUP_NODE_DP_KEY} for more details. When defining the
free space via <i>area nodes</i> (see above), this group is automatically determined, otherwise it must explicitly be specified. If not
specified, the free space is created in the top-level hierarchy.
The algorithm is able to consider a specified {@link PartitionGrid} as long as there are no group nodes that span
multiple grid cells. Note that the cleared area is always allocated to a single cell of the grid, i.e., it is not
possible that the area spans multiple cells. For this feature to work properly it is required that the values of the
properties {@link ColumnDescriptor#originalPosition}, {@link RowDescriptor#originalPosition} {@link ColumnDescriptor#originalWidth} and {@link RowDescriptor#originalHeight} are correctly specified. This is usually automatically the case when executing the {@link ClearAreaLayout}
as a standalone algorithm via layout execution convenience methods (e.g. the values are taken from the table
visualization of the grid). However, if the {@link ClearAreaLayout} is applied as part of a more complex layout
pipeline it may be necessary to specify the values manually. For example, if another algorithm previously computed the
grid position values and stored them in the respective 'computed' properties (e.g. {@link ColumnDescriptor#computedPosition}), and afterwards {@link ClearAreaLayout}
should be applied, then the 'computed' values of the first algorithm should be written to the 'original' values prior to
the run of the {@link ClearAreaLayout}.
<b>Warning:</b>
The input graph should already have a rather good layout for this algorithm to produce sophisticated results. The reason
is that it analyzes the existing layout to decide how to change it. Overlapping nodes, edges intersecting nodes and
other suboptimal arrangements can cause that this algorithm generates unexpected results. Furthermore, it is not meant
to repair things like that in the input layout.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout Online Documentation}*/
export class ClearAreaLayout {
/**
Creates a new instance of {@link ClearAreaLayout} with default settings.
@param [options] The parameters to pass.
@param [options.clearAreaStrategy] The strategy applied for clearing the desired area. This option sets the {@link ClearAreaLayout#clearAreaStrategy} property on the created object.
@param [options.gridSpacing] The current grid spacing. This option sets the {@link ClearAreaLayout#gridSpacing} property on the created object.
@param [options.edgeRouter] The custom edge router instance that is applied to reroute edges. This option sets the {@link ClearAreaLayout#edgeRouter} property on the created object.
@param [options.edgeRoutingStrategy] The routing strategy that is preserved while clearing the area and applied when rerouting edges. This option sets the {@link ClearAreaLayout#edgeRoutingStrategy} property on the created object.
@param [options.componentAssignmentStrategy] The strategy that assigns nodes to components whose elements should preferably not be separated. This option sets the {@link ClearAreaLayout#componentAssignmentStrategy} property on the created object.
@param [options.maximumDuration] The time limit in milliseconds for the layout algorithm. This option sets the {@link ClearAreaLayout#maximumDuration} property on the created object.
@param [options.fixPorts] Whether the ports of the input graph must be fixed or if they can be changed. This option sets the {@link ClearAreaLayout#fixPorts} property on the created object.
@param [options.considerEdges] Whether or not edges are considered when clearing the {@link ClearAreaLayout#area area} such that no segments intersecting the area are allowed. This option sets the {@link ClearAreaLayout#considerEdges} property on the created object.
@param [options.area] The rectangular area that must be cleared. This option sets the {@link ClearAreaLayout#area} property on the created object.
@param [options.areaOutline] The outline describing the shape of the area that must be cleared. This option sets the {@link ClearAreaLayout#areaOutline} property on the created object.
@param [options.layoutOrientation] The layout orientation that is considered during the clearing of the area. This option sets the {@link ClearAreaLayout#layoutOrientation} property on the created object.
@param [options.considerEdgeLabels] Whether or not the layout algorithm considers edges labels, moving them outside the area, if necessary. This option sets the {@link ClearAreaLayout#considerEdgeLabels} property on the created object.
@param [options.considerNodeLabels] Whether or not the layout algorithm considers node labels, moving them outside the area, if necessary. This option sets the {@link ClearAreaLayout#considerNodeLabels} property on the created object.
@param [options.spacing] The spacing that is considered between elements when they are moved to clear the area. This option sets the {@link ClearAreaLayout#spacing} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout Online Documentation}*/
constructor(options?: {
clearAreaStrategy?: ClearAreaStrategy | ClearAreaStrategyStringValues;
gridSpacing?: number;
edgeRouter?: ILayoutAlgorithm | null;
edgeRoutingStrategy?: PartialLayoutEdgeRoutingStrategy | PartialLayoutEdgeRoutingStrategyStringValues;
componentAssignmentStrategy?: ComponentAssignmentStrategy | ComponentAssignmentStrategyStringValues;
maximumDuration?: number;
fixPorts?: boolean;
considerEdges?: boolean;
area?: YRectangle | null;
areaOutline?: BorderLine[] | null;
layoutOrientation?: PartialLayoutOrientation | PartialLayoutOrientationStringValues;
considerEdgeLabels?: boolean;
considerNodeLabels?: boolean;
spacing?: number;
});
/**
Clears the specified rectangular {@link ClearAreaLayout#area}, the area defined via {@link ClearAreaLayout#AREA_NODES_DP_KEY} or {@link ClearAreaLayout#EXPANDED_NODE_DP_KEY}
by moving all other graph elements currently intersecting with it.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Creates and sets the value of the property {@link ClearAreaLayout#areaOutline} of the <code>layout</code> determined from the <code>items</code>.
This method saves the user from having to create the {@link BorderLine}s of the {@link ClearAreaLayout#areaOutline} property himself.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@param items The {@link IModelItem}s from which the {@link BorderLine}s are determined.
@param spacing The spacing that is considered between the area and the elements.
@param [closed=true] Whether the area is closed or can be traversed by other elements (without overlaps).
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23configureAreaOutline(IEnumerable%3CIModelItem%3E%2Cnumber%2Cboolean) Online Documentation}*/
configureAreaOutline(items: IEnumerable<IModelItem> | IEnumerableConvertible<IModelItem>, spacing: number, closed?: boolean): void;
/**
This method is called each time when edges are rerouted with the given edge router.
This method may be overridden to customize the settings of the applied routing algorithm without manually specifying a
custom router instance. This can be beneficial because the basic configuration of the router has already been made upon
calling this method (e.g. setting up the scope). This method can then be implemented to fine-tune further settings,
like, for example, minimum distance values.
The type of the given instance depends on the edge routing strategy.
<ul>
<li>{@link EdgeRouter} for strategies {@link PartialLayoutEdgeRoutingStrategy#ORTHOGONAL} and
{@link PartialLayoutEdgeRoutingStrategy#OCTILINEAR}.
<li>{@link OrganicEdgeRouter} for strategy {@link PartialLayoutEdgeRoutingStrategy#ORGANIC}.
<li>{@link StraightLineEdgeRouter} for strategy {@link PartialLayoutEdgeRoutingStrategy#STRAIGHTLINE}.
<li>If strategy {@link PartialLayoutEdgeRoutingStrategy#AUTOMATIC} is selected, one of the mentioned types is chosen, depending on
the style of the given edge routes.
<li>If a custom {@link ClearAreaLayout#edgeRouter} is set, its instance corresponds to that of the parameter.
</ul>
@param edgeRouter the instance used for routing the edges
@see ClearAreaLayout#edgeRouter
@see ClearAreaLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23configureEdgeRouter(ILayoutAlgorithm) Online Documentation}*/
configureEdgeRouter(edgeRouter: ILayoutAlgorithm | null): void;
/**
Creates the outline that describes the shape of the given nodes and edges.
This method is called when the area to be cleared is derived from a specified set of area nodes, see
{@link ClearAreaLayout#AREA_NODES_DP_KEY}. The outline is defined by four {@link BorderLine}s, which
are then used as the {@link ClearAreaLayout#areaOutline area outline} for the layout run.
The default implementation of this method includes the bounds of the given nodes, the bounding boxes of the segments of
the given edges, as well as the bounds of the corresponding node and edge labels (depending on the properties {@link ClearAreaLayout#considerNodeLabels} and
{@link ClearAreaLayout#considerEdgeLabels}. This method may be overridden if another way of deriving the area shape from the defined nodes and edges is desired
(e.g. if only nodes shall be considered, ignoring edges).
The returned array must contain exactly four {@link BorderLine} instances where the indices are interpreted as
follows:
<ul>
<li>index <code>0</code>: the top border line
<li>index <code>1</code>: the right border line
<li>index <code>2</code>: the bottom border line
<li>index <code>3</code>: the left border line
</ul>
<h2>Note</h2>
The given list of edges does not include edges that connect an area node with a non-area node. Such edges will always be
rerouted and they do not affect the shape of the area.
<h2>Note</h2>
The list of nodes does not only contain the nodes explicitly marked as area nodes but also descendants of them in case a
group node is marked.
<b>Warning:</b> This method is only called if a custom set of {@link ClearAreaLayout#AREA_NODES_DP_KEY area nodes} is specified.
@param graph the input graph
@param areaNodes the nodes that are part of the area that should be cleared
@param areaEdges the edges that are adjacent to the given area nodes
@returns an array of four {@link BorderLine}s describing the outline of the area to be cleared
@protected
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23createAreaOutline(LayoutGraph%2CYNodeList%2CEdgeList) Online Documentation}*/
createAreaOutline(graph: LayoutGraph | null, areaNodes: YNodeList | null, areaEdges: EdgeList | null): BorderLine[] | null;
/**
Moves the {@link ClearAreaLayout#areaOutline} by the given <code>delta</code>.
This method prevents the user from having to modify the {@link BorderLine}s of the {@link ClearAreaLayout#areaOutline} property himself.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@param delta The delta to move the {@link ClearAreaLayout#areaOutline}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23moveAreaOutline(IPoint) Online Documentation}*/
moveAreaOutline(delta: IPoint): void;
/**
Data provider key for marking the node that was expanded and, thus, defines the area that must be cleared
When an expanded node is provided, the area that must be cleared is directly derived from the size of the expanded node.
Defining an expanded node is intended for the case that a node's size increased such that it might now overlap with
other graph elements. This includes the use case of expanding a folder node.
It is strongly recommended to additionally provide the original (smaller) bounds of the expanded node via
{@link ClearAreaLayout#EXPANDED_NODE_ORIGINAL_BOUNDS_DP_KEY}. If the node has incident edges, it is also
recommended to provide the original paths of these edges via key
{@link ClearAreaLayout#EXPANDED_NODE_ORIGINAL_EDGE_PATH_DP_KEY}, i.e. the edge paths before the size of the node
was changed (for example, before expanding a folder node). This includes paths of edges that were only incident to the
non-expanded node and are now incident to descendants of it. Using this knowledge the algorithm clears the area occupied
by the expanded node but also considers how the layout looked before the change. This way the mental map may be
preserved to a higher degree.
<h2>Note</h2>
If selecting an expanded node, the {@link ClearAreaLayout#AREA_NODES_DP_KEY area nodes} and the values of properties {@link ClearAreaLayout#area} and {@link ClearAreaLayout#areaOutline} are ignored.
<b>Warning:</b> Only a single node may be marked as expanded.
@see ClearAreaLayout#EXPANDED_NODE_ORIGINAL_BOUNDS_DP_KEY
@see ClearAreaLayout#EXPANDED_NODE_ORIGINAL_EDGE_PATH_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23EXPANDED_NODE_DP_KEY Online Documentation}*/
static EXPANDED_NODE_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for specifying the original, non-expanded bounds of the expanded node
When an expanded node is provided, the area that must be cleared is directly derived from the bounds of the expanded
node. The original, non-expanded (smaller) bounds of the expanded node are helpful to preserve the mental map of the
overall drawing to a higher degree than without the additional knowledge about the previous layout. Note, however, that
only strategies {@link ClearAreaStrategy#PRESERVE_SHAPES}, {@link ClearAreaStrategy#PRESERVE_SHAPES_UNIFORM}
and {@link ClearAreaStrategy#GLOBAL} will significantly profit from it.
@see ClearAreaLayout#EXPANDED_NODE_DP_KEY
@see ClearAreaLayout#EXPANDED_NODE_ORIGINAL_EDGE_PATH_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23EXPANDED_NODE_ORIGINAL_BOUNDS_DP_KEY Online Documentation}*/
static EXPANDED_NODE_ORIGINAL_BOUNDS_DP_KEY: GraphDpKey<YRectangle> | null;
/**
Data provider key for associating original paths for edges connecting to the expanded node, including edges that did so
only when the node was not yet expanded
When an expanded node is provided, the area that must be cleared is directly derived from the bounds of the expanded
node. The original paths together with the original, non-expanded (smaller) bounds (see
{@link ClearAreaLayout#EXPANDED_NODE_ORIGINAL_BOUNDS_DP_KEY}) of the expanded node are helpful to preserve the
mental map of the overall drawing by providing knowledge about the previous layout state. Note, however, that only
strategies {@link ClearAreaStrategy#PRESERVE_SHAPES}, {@link ClearAreaStrategy#PRESERVE_SHAPES_UNIFORM}
and {@link ClearAreaStrategy#GLOBAL} will significantly profit from it.
Carefully observe that the original paths must correspond to the ones that were valid when the expanded node still had
its original, non-expanded bounds. Thus, they should only be provided when the original bounds are provided too.
@see ClearAreaLayout#EXPANDED_NODE_DP_KEY
@see ClearAreaLayout#EXPANDED_NODE_ORIGINAL_BOUNDS_DP_KEY
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23EXPANDED_NODE_ORIGINAL_EDGE_PATH_DP_KEY Online Documentation}*/
static EXPANDED_NODE_ORIGINAL_EDGE_PATH_DP_KEY: EdgeDpKey<YPointPath> | null;
/**
Data provider key for defining custom components whose elements should preferably not be separated
While the algorithm may move a whole component, it should preferably not move only a subset of its elements. This means
that the algorithm tries to move all elements of a component by the same offset (if at all). In order to achieve good
results with this feature, the different components should not overlap in the initial drawing.
<b>Warning:</b> The specified custom components are only considered when property {@link ClearAreaLayout#componentAssignmentStrategy} is set to
{@link ComponentAssignmentStrategy#CUSTOMIZED}. Furthermore, the components feature is only supported by {@link ClearAreaLayout#clearAreaStrategy clearing strategies}
{@link ClearAreaStrategy#PRESERVE_SHAPES}, {@link ClearAreaStrategy#PRESERVE_SHAPES_UNIFORM} and
{@link ClearAreaStrategy#GLOBAL}.
<b>Warning:</b> If the input graph contains group nodes or a {@link PartitionGrid} structure, maintaining the specified
components is not always possible. Furthermore, a custom component can never pass the border of a group node or
partition cell, i.e., all nodes of a custom component have to be associated with the same group or grid cell.
@see ClearAreaLayout#componentAssignmentStrategy
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23COMPONENT_ID_DP_KEY Online Documentation}*/
static COMPONENT_ID_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for specifying a set of nodes that define the area which should be cleared
These nodes define a sort of component from which the area that must be cleared is derived by calling method
{@link ClearAreaLayout#createAreaOutline}. This means that the area will exactly be defined by the current
locations of these nodes (including the edges among them) such that after the algorithm execution there will not be any
other elements intersecting with them.
Carefully observe that if group nodes are marked, all their descendants are treated as area nodes too.
The {@link ClearAreaLayout#AREA_GROUP_NODE_DP_KEY area group node} can not be defined manually, but the nearest common ancestor of all area nodes will become the area
group node. For group nodes other than the area group either all their descendants including the group itself must be
area nodes or all their descendants including the group must not be area nodes. If the input is not complete in this
matter, the required nodes are internally automatically treated as area nodes too.
<b>Warning:</b> If area nodes are defined, then the values of properties {@link ClearAreaLayout#area} and {@link ClearAreaLayout#areaOutline} are ignored.
<b>Warning:</b> The area nodes are ignored if an expanded node is provided via {@link ClearAreaLayout#EXPANDED_NODE_DP_KEY}.
@see ClearAreaLayout#createAreaOutline
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23AREA_NODES_DP_KEY Online Documentation}*/
static AREA_NODES_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for specifying the group node inside which the cleared area should be located
The {@link ClearAreaLayout#area area} that is cleared of elements can be associated to one specific group node. This means that this group can grow
because the free space must be created inside it. In consequence, all ancestor nodes of the group may grow, too. Other
groups are treated as fixed and must be moved in order to avoid an overlap with the specified area.
If no group is specified, the free space will be created in the top-level hierarchy.
<b>Warning:</b> The area group is ignored if the area is defined by means of an area nodes component (see
{@link ClearAreaLayout#AREA_NODES_DP_KEY}) or by means of a single expanded node (see
{@link ClearAreaLayout#EXPANDED_NODE_DP_KEY}. In these cases, the nearest common ancestor of the area nodes or
the parent of the expanded node is automatically selected as the group node that contains the free space.
<b>Warning:</b> Only a single group must be defined as the area group.
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23AREA_GROUP_NODE_DP_KEY Online Documentation}*/
static AREA_GROUP_NODE_DP_KEY: NodeDpKey<boolean> | null;
/**
Data provider key for obtaining the edges that should be routed by the custom edge router
<b>Warning:</b> The associated {@link IDataProvider} must not be set by the user! The key is used by this algorithm to
temporarily add a {@link IDataProvider} that marks edges that should be routed by the specified edge router,
see {@link ClearAreaLayout#edgeRouter}.
@see ClearAreaLayout#edgeRouter
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23ROUTE_EDGE_DP_KEY Online Documentation}*/
static ROUTE_EDGE_DP_KEY: EdgeDpKey<boolean> | null;
/**
Gets or sets the strategy applied for clearing the desired area.
This property controls the main approach followed to clear the {@link ClearAreaLayout#area area} of graph elements.
@throws {Exception} <b>(ArgumentError)</b> if an unknown strategy is provided
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23clearAreaStrategy Online Documentation}*/
clearAreaStrategy: ClearAreaStrategy;
/**
Gets or sets the current grid spacing.
Elements are moved by multiples of this value, thus, keeping their offset to the grid. That way, components or parts of
them that were placed on a grid before, will stay on their original grid.
The grid spacing needs to be a non-negative value. If it is set to <code>0</code>, no grid is considered.
<b>Warning:</b> For edges that need to be rerouted, it is not guaranteed that the bends are placed on grid points.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23gridSpacing Online Documentation}*/
gridSpacing: number;
/**
Gets or sets the custom edge router instance that is applied to reroute edges.
Several types of edges can be routed by this algorithm:
<ol>
<li>Edges where one or both endpoints where displaced in order to clear the area. It depends on the {@link ClearAreaLayout#clearAreaStrategy strategy} if such edges
are actually handled by the routing algorithm or if they are implicitly moved. For example, strategy {@link ClearAreaStrategy#PRESERVE_SHAPES}
tries to keep the shapes and can avoid rerouting in many cases.
<li>So-called inter-edges resulting from the {@link ClearAreaLayout#AREA_NODES_DP_KEY area nodes} component. If such a component exists, the inter-edges are the edges
connecting area nodes with non-area nodes. They are always handled by this routing algorithm, independent of the
{@link ClearAreaLayout#clearAreaStrategy strategy}.
<li>Edges connecting to a specifically marked {@link ClearAreaLayout#EXPANDED_NODE_DP_KEY expanded node} and to the descendants of the expanded node in case it is a
group node.
</ol>
Changing the value of property {@link ClearAreaLayout#edgeRoutingStrategy} automatically sets a new edge router instance according to the specified routing
strategy, thus, overwriting any previously set instances. The following routing algorithms are set:
<ul>
<li>{@link EdgeRouter} for strategies {@link PartialLayoutEdgeRoutingStrategy#ORTHOGONAL} and
{@link PartialLayoutEdgeRoutingStrategy#OCTILINEAR}.
<li>{@link OrganicEdgeRouter} for strategy {@link PartialLayoutEdgeRoutingStrategy#ORGANIC}.
<li>{@link StraightLineEdgeRouter} for strategy {@link PartialLayoutEdgeRoutingStrategy#STRAIGHTLINE}.
</ul>
<b>Warning:</b> Edges that have to be routed with the specified router will be marked using {@link IDataProvider}-key
{@link ClearAreaLayout#ROUTE_EDGE_DP_KEY}. Hence, the specified router has to be configured such that it observes
this key and only routes marked edges!
@see ClearAreaLayout#ROUTE_EDGE_DP_KEY
@see ClearAreaLayout#edgeRoutingStrategy
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23edgeRouter Online Documentation}*/
edgeRouter: ILayoutAlgorithm | null;
/**
Gets or sets the routing strategy that is preserved while clearing the area and applied when rerouting edges.
The algorithm assumes that the existing edges have paths according to the specified strategy and takes this into account
when moving nodes and edges in order to clear the area. If the {@link PartialLayoutEdgeRoutingStrategy#AUTOMATIC automatic strategy} is set, then no assumptions are made
but the given paths are analyzed to derive the best fitting routing strategy. Note that in this case, the algorithm may
use different routing styles for the edges.
This routing strategy is also relevant when edges need to be rerouted, as it determines the routing algorithm used for
rerouting, see {@link ClearAreaLayout#edgeRouter} for more details.
<b>Warning:</b> It is strongly discouraged to specify a routing strategy different from the actual routing style of the input drawing.
<h2>Note</h2>
Changing the value of this property overwrites a previously set customized edge router, see {@link ClearAreaLayout#edgeRouter}.
@throws {Exception} <b>(ArgumentError)</b> if the given strategy does not match one of the predefined routing strategies
@see ClearAreaLayout#edgeRouter
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23edgeRoutingStrategy Online Documentation}*/
edgeRoutingStrategy: PartialLayoutEdgeRoutingStrategy;
/**
Gets or sets the strategy that assigns nodes to components whose elements should preferably not be separated.
While the algorithm may move a whole component, it tries to not move only a subset of its elements, thus, all elements
of a component are not moved at all or moved by the same offset.
<ul>
<li>{@link ComponentAssignmentStrategy#CUSTOMIZED}: the components can be user-defined. If the user does not specify
any, there are no components that should explicitly be kept together, which is equal to the behavior of
{@link ComponentAssignmentStrategy#SINGLE}.
<li>{@link ComponentAssignmentStrategy#CONNECTED} components are defined by the connected components of the graph.
<li>{@link ComponentAssignmentStrategy#CLUSTERING}: components are defined by edge betweenness clustering.
<li>{@link ComponentAssignmentStrategy#SINGLE}: each node is a separate component which basically means that there
are no components that should explicitly be kept together.
</ul>
<b>Warning:</b> This feature is only supported by {@link ClearAreaLayout#clearAreaStrategy clearing strategies} {@link ClearAreaStrategy#PRESERVE_SHAPES}, {@link ClearAreaStrategy#PRESERVE_SHAPES_UNIFORM}
and {@link ClearAreaStrategy#GLOBAL}.
<h2>Note</h2>
Borders of groups and partition cells always split up the components, i.e., two nodes assigned to different groups or
partition cells are always assigned to different subgraph components.
@throws {Exception} <b>(ArgumentError)</b> if the specified strategy does not match one of the predefined strategies
@see ClearAreaLayout#COMPONENT_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23componentAssignmentStrategy Online Documentation}*/
componentAssignmentStrategy: ComponentAssignmentStrategy;
/**
Gets or sets the time limit in milliseconds for the layout algorithm.
Values have to be greater than or equal to <code>0</code>.
<h2>Note</h2>
Restricting the maximum duration may result in a lower layout quality. Furthermore, the real runtime may exceed the
maximum duration since the layout algorithm still has to find a valid solution. For strategies {@link ClearAreaStrategy#LOCAL}
and {@link ClearAreaStrategy#LOCAL_UNIFORM} a restricted time can highly affect the quality of the edge routes
and, on the other hand, significantly improve the performance.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets whether the ports of the input graph must be fixed or if they can be changed.
This property is relevant for edges that are rerouted in order to clear the area, i.e., edges where one or both
endpoints where displaced or edges that intersect the area in the input graph (depending on property {@link ClearAreaLayout#considerEdges}) and that are
consequently handled by the specified {@link ClearAreaLayout#edgeRouter edge router}. Inter-edges resulting from the {@link ClearAreaLayout#AREA_NODES_DP_KEY area nodes} component are not affected
by this property (edges connecting area nodes with non-area nodes). For such edges, the behavior is the same as if this
property would be disabled, see the following description. The same applies for edges incident to the {@link ClearAreaLayout#EXPANDED_NODE_DP_KEY expanded node} or
the descendants of the expanded node (if one endpoint is not the expanded node or a descendant).
If the ports are fixed, then the port side and location are not changed, except for group nodes that grow due to the {@link ClearAreaLayout#area area}
inside them. Edges at these groups keep the port side, but the absolute location is not fixed.
If disabled, the algorithm considers specified {@link PortConstraint} and
{@link PortCandidate}s when rerouting edges. If there are no such constraints for an edge, then the ports of that
edge can be freely placed. This can often produce better edge routes in the resulting drawing, however, the overall
shape of the edge may then not be preserved.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23fixPorts Online Documentation}*/
fixPorts: boolean;
/**
Gets or sets whether or not edges are considered when clearing the {@link ClearAreaLayout#area area} such that no segments intersecting the area are
allowed.
If enabled, edges that must be rerouted are not allowed to intersect the {@link ClearAreaLayout#area area} that is cleared. Furthermore, edges whose
source and target node are not moved but where segments intersect the area in the input graph are rerouted in order to
remove that intersection.
If disabled, edges are allowed to intersect the area that is cleared. This has the effect that the area is cleared of
nodes only.
<b>Warning:</b> With edge routing strategies {@link PartialLayoutEdgeRoutingStrategy#STRAIGHTLINE} and
{@link PartialLayoutEdgeRoutingStrategy#ORGANIC}, intersections with the area are not resolved. Use strategies {@link PartialLayoutEdgeRoutingStrategy#ORTHOGONAL}
or {@link PartialLayoutEdgeRoutingStrategy#OCTILINEAR} if preventing them is an important requirement.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23considerEdges Online Documentation}*/
considerEdges: boolean;
/**
Gets or sets the rectangular area that must be cleared.
All elements (nodes, edges and labels) that currently intersect the area will be moved such that the area is cleared
after the layout run.
<b>Warning:</b> The specified area is ignored in case that the area is derived from a custom set of {@link ClearAreaLayout#AREA_NODES_DP_KEY area nodes} or from the size of an
{@link ClearAreaLayout#EXPANDED_NODE_DP_KEY expanded node}.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23area Online Documentation}*/
area: YRectangle | null;
/**
Gets or sets the outline describing the shape of the area that must be cleared.
This property should be used if the area is not only a simple rectangular shape but more complex. An outline consists of
four {@link BorderLine} instances, where the array is interpreted as follows:
<ul>
<li>index <code>0</code>: the top border line
<li>index <code>1</code>: the right border line
<li>index <code>2</code>: the bottom border line
<li>index <code>3</code>: the left border line
</ul>
<h2>Note</h2>
This property is synchronized with property {@link ClearAreaLayout#area} such that when one is changed, internally both are changed. The {@link ClearAreaLayout#area} property
is a more simple view of this property, as it only provides the rectangular bounding box.
<b>Warning:</b> The specified outline is ignored in case that the area is derived from a custom set of {@link ClearAreaLayout#AREA_NODES_DP_KEY area nodes} or from the size of
an {@link ClearAreaLayout#EXPANDED_NODE_DP_KEY expanded node}.
@throws {Exception} <b>(ArgumentError)</b> if the given array does not contain exactly four non-null {@link BorderLine}s
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23areaOutline Online Documentation}*/
areaOutline: BorderLine[] | null;
/**
Gets or sets the layout orientation that is considered during the clearing of the area.
The orientation affects the direction that the algorithm prefers when moving elements. For the vertical orientations {@link PartialLayoutOrientation#TOP_TO_BOTTOM}
and {@link PartialLayoutOrientation#BOTTOM_TO_TOP}, moving elements horizontally (i.e. to the left and to the right) is
preferred. For the horizontal orientations {@link PartialLayoutOrientation#LEFT_TO_RIGHT} and
{@link PartialLayoutOrientation#RIGHT_TO_LEFT}, the vertical moving direction is preferred. This is mainly useful for
layouts that have a clear directions and nodes are divided into layers with respect to this directions, like e.g.,
hierarchical layouts.
If this behavior is undesired, the orientation can be ignored by specifying {@link PartialLayoutOrientation#NONE}. No
specific moving direction will be preferred in that case. The orientation can also be automatically detected based on
the flow direction of the edges when choosing {@link PartialLayoutOrientation#AUTO_DETECT}.
@throws {Exception} <b>(ArgumentError)</b> if the specified orientation does not match one of the predefined orientations
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23layoutOrientation Online Documentation}*/
layoutOrientation: PartialLayoutOrientation;
/**
Gets or sets whether or not the layout algorithm considers edges labels, moving them outside the area, if necessary.
When enabled, the edge labels intersecting the {@link ClearAreaLayout#area} are explicitly rearranged even if the corresponding edges would not be
changed.
If disabled, edge labels may remain intersecting with the
{@link ClearAreaLayout#area area}. In addition, after moving elements to clear the area, there may be new edge label overlaps.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23considerEdgeLabels Online Documentation}*/
considerEdgeLabels: boolean;
/**
Gets or sets whether or not the layout algorithm considers node labels, moving them outside the area, if necessary.
Internally, nodes are treated like they would have a size including their labels. Therefore, layout results can get
significantly less compact if this feature is enabled.
If this feature is disabled, node labels may remain intersecting with the
{@link ClearAreaLayout#area area}. In addition, after moving elements to clear the area, there may be new node label overlaps.
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets the spacing that is considered between elements when they are moved to clear the area.
This spacing only affects the moving of elements out of the
{@link ClearAreaLayout#area area}. Elements keep the specified distance to other elements. Carefully observe that if the distance between two
elements is already smaller, then they may not be moved apart.
Strategies {@link ClearAreaStrategy#PRESERVE_SHAPES}, {@link ClearAreaStrategy#PRESERVE_SHAPES_UNIFORM}
and {@link ClearAreaStrategy#GLOBAL} consider the spacing for all graph elements, including nodes, edges, node
labels and edge labels when they are encountered during the movement process. On the other hand, strategies {@link ClearAreaStrategy#LOCAL}
and {@link ClearAreaStrategy#LOCAL_UNIFORM} consider the spacing only with respect to nodes.
<h2>Note</h2>
For edges that are rerouted with the {@link ClearAreaLayout#edgeRouter edge router}, this spacing is not considered. Instead, the respective
distance/spacing values of the router instance apply. To configure custom distance/spacing values for the routing, one
may override method {@link ClearAreaLayout#configureEdgeRouter} or specify a manually configured router
instance.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/ClearAreaLayout%23spacing Online Documentation}*/
spacing: number;
static $class: Class;
private $_ClearAreaLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is ClearAreaLayout;
}
export interface FillAreaLayout extends YObject, ILayoutAlgorithm {
}
/**
This layout algorithm tries to fill a specified area with graph elements by moving nearby elements towards it, with the
goal to make the existing layout more compact and not changing it too much.
<b>Layout Style</b>
The algorithm can make an existing layout more compact in the region of the defined
{@link FillAreaLayout#area area}. It moves nearby graph elements, while keeping the given layout style as much as possible. Ideally, only local
changes around the marked area are made such that the mental map for the user is preserved. The best applications are
those where it shall be avoided to calculate a completely new layout after local changes have been made to an already
existing and good layout. Also not that the results for this application are much better when the edge paths are already
good orthogonal, polyline or octilinear paths. For straight-line edge routes, the shape and overall mental map of the
layout can not be preserved that well and the changes may be more global.
An example application is the use case that nodes were removed from the graph. The region where the nodes have been
removed can then be defined as the {@link FillAreaLayout#area area} so that the algorithm can try to fill/use this free space and make the layout
more compact. This way it can be avoided to compute a completely new layout for such cases. Another use case would be
when a group node is collapsed and converted to a smaller folder node. In that case the folder node should be marked as
{@link FillAreaLayout#FIXED_NODE_DP_KEY fixed}. Note that it isn't guaranteed that the area is filled with elements after calling the algorithm.
<b>Concept</b>
The area (see {@link FillAreaLayout#area}) defines a region in the given graph which should be filled with elements, with the goal to make the
overall layout more compact. To do so, graph elements must be moved. This includes nodes, edges and their labels.
Whether node labels and edge labels should be considered can be controlled via settings {@link FillAreaLayout#considerNodeLabels} and {@link FillAreaLayout#considerEdgeLabels}.
Keep in mind that the goal is to make the layout more compact. Therefore, if the area is located such that it brings no
advantage to move elements towards it, the algorithm may also do nothing - based on a heuristic decision. This means
that it does not fill the specified area in any case.
<b>Features</b>
The algorithm is able to consider a specified {@link PartitionGrid} as long as there are no group nodes that span
multiple grid cells. For this feature to work properly it is required that the values of the properties {@link ColumnDescriptor#originalPosition}, {@link RowDescriptor#originalPosition} {@link ColumnDescriptor#originalWidth} and {@link RowDescriptor#originalHeight} are
correctly specified. This is usually automatically the case when executing the {@link FillAreaLayout} as a
standalone algorithm via layout execution convenience methods (e.g. the values are taken from the table visualization of
the grid). However, if the {@link FillAreaLayout} is applied as part of a more complex layout pipeline it may be
necessary to specify the values manually. For example, if another algorithm previously computed the grid position values
and stored them in the respective 'computed' properties (e.g. {@link ColumnDescriptor#computedPosition}), and afterwards {@link FillAreaLayout} should be
applied, then the 'computed' values of the first algorithm should be written to the 'original' values prior to the run
of the {@link FillAreaLayout}.
<b>Warning:</b>
The input graph should already have a rather good layout for this algorithm to produce sophisticated results. The reason
is that it analyzes the existing layout to decide how to change it. Overlapping nodes, edges intersecting nodes and
other suboptimal arrangements can cause that this algorithm generates unexpected results. Furthermore, it is not meant
to repair things like that in the input layout.
<h2>Note</h2>
This algorithm is not meant to be a general layout compaction algorithm. It focuses on the defined area but may also do
nothing if, for example, moving elements would heavily affect the mental map of the existing layout or not make the
layout more compact.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout Online Documentation}*/
export class FillAreaLayout {
/**
Creates a new instance of {@link FillAreaLayout} with default settings.
@param [options] The parameters to pass.
@param [options.gridSpacing] The current grid spacing. This option sets the {@link FillAreaLayout#gridSpacing} property on the created object.
@param [options.componentAssignmentStrategy] The strategy that assigns nodes to components whose elements should preferably not be separated. This option sets the {@link FillAreaLayout#componentAssignmentStrategy} property on the created object.
@param [options.maximumDuration] The time limit in milliseconds for the layout algorithm. This option sets the {@link FillAreaLayout#maximumDuration} property on the created object.
@param [options.area] The rectangular area that should be filled. This option sets the {@link FillAreaLayout#area} property on the created object.
@param [options.spacing] The spacing that is considered between elements when they are moved. This option sets the {@link FillAreaLayout#spacing} property on the created object.
@param [options.considerNodeLabels] Whether or not the layout algorithm considers node labels. This option sets the {@link FillAreaLayout#considerNodeLabels} property on the created object.
@param [options.considerEdgeLabels] Whether or not the layout algorithm considers edges labels. This option sets the {@link FillAreaLayout#considerEdgeLabels} property on the created object.
@param [options.layoutOrientation] The layout orientation that is considered during the compaction process. This option sets the {@link FillAreaLayout#layoutOrientation} property on the created object.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout Online Documentation}*/
constructor(options?: {
gridSpacing?: number;
componentAssignmentStrategy?: ComponentAssignmentStrategy | ComponentAssignmentStrategyStringValues;
maximumDuration?: number;
area?: YRectangle | null;
spacing?: number;
considerNodeLabels?: boolean;
considerEdgeLabels?: boolean;
layoutOrientation?: PartialLayoutOrientation | PartialLayoutOrientationStringValues;
});
/**
Tries to fill the specified area in the given graph with elements, such that the resulting layout is more compact.
@param graph the input graph
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23applyLayout(LayoutGraph) Online Documentation}*/
applyLayout(graph: LayoutGraph): void;
/**
Creates and sets the value of the property {@link FillAreaLayout#area} of the <code>layout</code> determined from the <code>items</code>.
<b>Warning:</b> This method is not available unless the module <b>view-layout-bridge</b> is loaded.
@param items The {@link IModelItem}s from which the rectangular area is determined.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23configureAreaOutline(IEnumerable%3CIModelItem%3E) Online Documentation}*/
configureAreaOutline(items: IEnumerable<IModelItem> | IEnumerableConvertible<IModelItem>): void;
/**
Data provider key for defining custom components whose elements should preferably not be separated
While the algorithm may move a whole component, it should preferably not move only a subset of its elements. This means
that the algorithm tries to move all elements of a component by the same offset (if at all). In order to achieve good
results with this feature, the different components should not overlap in the initial drawing.
<b>Warning:</b> The specified custom components are only considered when property {@link FillAreaLayout#componentAssignmentStrategy} is set to
{@link ComponentAssignmentStrategy#CUSTOMIZED}.
<b>Warning:</b> If the input graph contains group nodes or a {@link PartitionGrid} structure, maintaining the specified
components is not always possible. Furthermore, a custom component can never pass the border of a group node or
partition cell, i.e., all nodes of a custom component have to be associated with the same group or grid cell.
@see FillAreaLayout#componentAssignmentStrategy
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23COMPONENT_ID_DP_KEY Online Documentation}*/
static COMPONENT_ID_DP_KEY: NodeDpKey<any> | null;
/**
Data provider key for marking nodes as fixed
A node marked as fixed will not be moved by this algorithm but stay at its current position.
<h2>Note</h2>
The algorithm tries to preserve the mental map and, thus, non-fixed nodes cannot simply pass/jump over the fixed nodes
(i.e., fixed nodes restrict the possible movement of non-fixed nodes).
@const
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23FIXED_NODE_DP_KEY Online Documentation}*/
static FIXED_NODE_DP_KEY: NodeDpKey<boolean> | null;
/**
Gets or sets the current grid spacing.
Elements are moved by multiples of this value, thus, keeping their offset to the grid. That way, components or parts of
them that were placed on a grid before, will stay on their original grid.
The grid spacing needs to be a non-negative value. If it is set to <code>0</code>, no grid is considered.
<b>Warning:</b> For edges that need to be rerouted, it is not guaranteed that the bends are placed on grid points.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23gridSpacing Online Documentation}*/
gridSpacing: number;
/**
Gets or sets the strategy that assigns nodes to components whose elements should preferably not be separated.
While the algorithm may move a whole component, it tries to not move only a subset of its elements, thus, all elements
of a component are not moved at all or moved by the same offset.
<ul>
<li>{@link ComponentAssignmentStrategy#CUSTOMIZED}: the components can be user-defined. If the user does not specify
any, there are no components that should explicitly be kept together, which is equal to the behavior of
{@link ComponentAssignmentStrategy#SINGLE}.
<li>{@link ComponentAssignmentStrategy#CONNECTED} components are defined by the connected components of the graph.
<li>{@link ComponentAssignmentStrategy#CLUSTERING}: components are defined by edge betweenness clustering.
<li>{@link ComponentAssignmentStrategy#SINGLE}: each node is a separate component which basically means that there
are no components that should explicitly be kept together.
</ul>
<h2>Note</h2>
Borders of groups and partition cells always split up the components, i.e., two nodes assigned to different groups or
partition cells are always assigned to different subgraph components.
@throws {Exception} <b>(ArgumentError)</b> if the specified strategy does not match one of the predefined strategies
@see FillAreaLayout#COMPONENT_ID_DP_KEY
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23componentAssignmentStrategy Online Documentation}*/
componentAssignmentStrategy: ComponentAssignmentStrategy;
/**
Gets or sets the time limit in milliseconds for the layout algorithm.
Values have to be greater than or equal to <code>0</code>.
<h2>Note</h2>
Restricting the maximum duration may result in a lower layout quality. Furthermore, the real runtime may exceed the
maximum duration since the layout algorithm still has to find a valid solution.
@throws {Exception} <b>(ArgumentError)</b> if the maximum duration is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23maximumDuration Online Documentation}*/
maximumDuration: number;
/**
Gets or sets the rectangular area that should be filled.
The specified area may already contain some elements. In order to fill the area, the algorithm may move elements while
it still tries to preserve the mental map of the initial layout. Note that after applying this algorithm it is not
guaranteed that there are any elements in the specified area.
<h2>Note</h2>
If the area is set to <code>null</code> (default) or if it is specified and contains all elements of the graph, the algorithm
terminates without changing anything.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23area Online Documentation}*/
area: YRectangle | null;
/**
Gets or sets the spacing that is considered between elements when they are moved.
This spacing only affects the moving of elements towards the desired
{@link FillAreaLayout#area area}. Elements keep the specified distance to other elements and among each other. Carefully observe that if the
distance between two elements is already smaller, then they may not be moved apart.
The spacing is considered for all graph elements, including nodes, edges, node labels and edge labels when they are
encountered during the movement process.
@throws {Exception} <b>(ArgumentError)</b> if the given spacing is negative
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23spacing Online Documentation}*/
spacing: number;
/**
Gets or sets whether or not the layout algorithm considers node labels.
If enabled, node labels are considered when moving elements such that overlaps with them are not allowed.
If disabled, elements that are moved may overlap with node labels, even if they did not overlap in the input.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23considerNodeLabels Online Documentation}*/
considerNodeLabels: boolean;
/**
Gets or sets whether or not the layout algorithm considers edges labels.
If enabled, edge labels are considered when moving elements such that overlaps with them are not allowed. Furthermore,
they are correctly moved along with an edge if an edge is moved.
If disabled, elements that are moved may overlap with edge labels, even if they did not overlap in the input.
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23considerEdgeLabels Online Documentation}*/
considerEdgeLabels: boolean;
/**
Gets or sets the layout orientation that is considered during the compaction process.
The orientation affects the direction that the algorithm prefers when moving elements. For the vertical orientations {@link PartialLayoutOrientation#TOP_TO_BOTTOM}
and {@link PartialLayoutOrientation#BOTTOM_TO_TOP}, moving elements horizontally (i.e. to the left and to the right) is
preferred. For the horizontal orientations {@link PartialLayoutOrientation#LEFT_TO_RIGHT} and
{@link PartialLayoutOrientation#RIGHT_TO_LEFT}, the vertical moving direction is preferred. This is mainly useful for
layouts that have a clear directions and nodes are divided into layers with respect to this directions, like e.g.,
hierarchical layouts.
If this behavior is undesired, the orientation can be ignored by specifying {@link PartialLayoutOrientation#NONE}. No
specific moving direction will be preferred in that case. The orientation can also be automatically detected based on
the flow direction of the edges when choosing {@link PartialLayoutOrientation#AUTO_DETECT}.
@throws {Exception} <b>(ArgumentError)</b> if the specified orientation does not match one of the predefined orientations
@see {@link https://docs.yworks.com/yfileshtml/#/api/FillAreaLayout%23layoutOrientation Online Documentation}*/
layoutOrientation: PartialLayoutOrientation;
static $class: Class;
private $_FillAreaLayout: 0;
/**
Whether the given object is an instance of this type.
@param o The object to test.*/
static isInstance(o: any): o is FillAreaLayout;
}
